/**
 * @license AngularJS v1.3.20
 * (c) 2010-2014 Google, Inc. http://angularjs.org
 * License: MIT
 */
(function (window, document, undefined) {
  'use strict';
  /**
 * @description
 *
 * This object provides a utility for producing rich Error messages within
 * Angular. It can be called as follows:
 *
 * var exampleMinErr = minErr('example');
 * throw exampleMinErr('one', 'This {0} is {1}', foo, bar);
 *
 * The above creates an instance of minErr in the example namespace. The
 * resulting error will have a namespaced error code of example.one.  The
 * resulting error will replace {0} with the value of foo, and {1} with the
 * value of bar. The object is not restricted in the number of arguments it can
 * take.
 *
 * If fewer arguments are specified than necessary for interpolation, the extra
 * interpolation markers will be preserved in the final string.
 *
 * Since data will be parsed statically during a build step, some restrictions
 * are applied with respect to how minErr instances are created and called.
 * Instances should have names of the form namespaceMinErr for a minErr created
 * using minErr('namespace') . Error codes, namespaces and template strings
 * should all be static strings, not variables or general expressions.
 *
 * @param {string} module The namespace to use for the new minErr instance.
 * @param {function} ErrorConstructor Custom error constructor to be instantiated when returning
 *   error from returned function, for cases when a particular type of error is useful.
 * @returns {function(code:string, template:string, ...templateArgs): Error} minErr instance
 */
  function minErr(module, ErrorConstructor) {
    ErrorConstructor = ErrorConstructor || Error;
    return function () {
      var code = arguments[0], prefix = '[' + (module ? module + ':' : '') + code + '] ', template = arguments[1], templateArgs = arguments, message, i;
      message = prefix + template.replace(/\{\d+\}/g, function (match) {
        var index = +match.slice(1, -1), arg;
        if (index + 2 < templateArgs.length) {
          return toDebugString(templateArgs[index + 2]);
        }
        return match;
      });
      message = message + '\nhttp://errors.angularjs.org/1.3.20/' + (module ? module + '/' : '') + code;
      for (i = 2; i < arguments.length; i++) {
        message = message + (i == 2 ? '?' : '&') + 'p' + (i - 2) + '=' + encodeURIComponent(toDebugString(arguments[i]));
      }
      return new ErrorConstructor(message);
    };
  }
  /* We need to tell jshint what variables are being exported */
  /* global angular: true,
  msie: true,
  jqLite: true,
  jQuery: true,
  slice: true,
  splice: true,
  push: true,
  toString: true,
  ngMinErr: true,
  angularModule: true,
  uid: true,
  REGEX_STRING_REGEXP: true,
  VALIDITY_STATE_PROPERTY: true,

  lowercase: true,
  uppercase: true,
  manualLowercase: true,
  manualUppercase: true,
  nodeName_: true,
  isArrayLike: true,
  forEach: true,
  sortedKeys: true,
  forEachSorted: true,
  reverseParams: true,
  nextUid: true,
  setHashKey: true,
  extend: true,
  int: true,
  inherit: true,
  noop: true,
  identity: true,
  valueFn: true,
  isUndefined: true,
  isDefined: true,
  isObject: true,
  isString: true,
  isNumber: true,
  isDate: true,
  isArray: true,
  isFunction: true,
  isRegExp: true,
  isWindow: true,
  isScope: true,
  isFile: true,
  isFormData: true,
  isBlob: true,
  isBoolean: true,
  isPromiseLike: true,
  trim: true,
  escapeForRegexp: true,
  isElement: true,
  makeMap: true,
  includes: true,
  arrayRemove: true,
  copy: true,
  shallowCopy: true,
  equals: true,
  csp: true,
  concat: true,
  sliceArgs: true,
  bind: true,
  toJsonReplacer: true,
  toJson: true,
  fromJson: true,
  startingTag: true,
  tryDecodeURIComponent: true,
  parseKeyValue: true,
  toKeyValue: true,
  encodeUriSegment: true,
  encodeUriQuery: true,
  angularInit: true,
  bootstrap: true,
  getTestability: true,
  snake_case: true,
  bindJQuery: true,
  assertArg: true,
  assertArgFn: true,
  assertNotHasOwnProperty: true,
  getter: true,
  getBlockNodes: true,
  hasOwnProperty: true,
  createMap: true,

  NODE_TYPE_ELEMENT: true,
  NODE_TYPE_ATTRIBUTE: true,
  NODE_TYPE_TEXT: true,
  NODE_TYPE_COMMENT: true,
  NODE_TYPE_DOCUMENT: true,
  NODE_TYPE_DOCUMENT_FRAGMENT: true,
*/
  ////////////////////////////////////
  /**
 * @ngdoc module
 * @name ng
 * @module ng
 * @description
 *
 * # ng (core module)
 * The ng module is loaded by default when an AngularJS application is started. The module itself
 * contains the essential components for an AngularJS application to function. The table below
 * lists a high level breakdown of each of the services/factories, filters, directives and testing
 * components available within this core module.
 *
 * <div doc-module-components="ng"></div>
 */
  var REGEX_STRING_REGEXP = /^\/(.+)\/([a-z]*)$/;
  // The name of a form control's ValidityState property.
  // This is used so that it's possible for internal tests to create mock ValidityStates.
  var VALIDITY_STATE_PROPERTY = 'validity';
  /**
 * @ngdoc function
 * @name angular.lowercase
 * @module ng
 * @kind function
 *
 * @description Converts the specified string to lowercase.
 * @param {string} string String to be converted to lowercase.
 * @returns {string} Lowercased string.
 */
  var lowercase = function (string) {
    return isString(string) ? string.toLowerCase() : string;
  };
  var hasOwnProperty = Object.prototype.hasOwnProperty;
  /**
 * @ngdoc function
 * @name angular.uppercase
 * @module ng
 * @kind function
 *
 * @description Converts the specified string to uppercase.
 * @param {string} string String to be converted to uppercase.
 * @returns {string} Uppercased string.
 */
  var uppercase = function (string) {
    return isString(string) ? string.toUpperCase() : string;
  };
  var manualLowercase = function (s) {
    /* jshint bitwise: false */
    return isString(s) ? s.replace(/[A-Z]/g, function (ch) {
      return String.fromCharCode(ch.charCodeAt(0) | 32);
    }) : s;
  };
  var manualUppercase = function (s) {
    /* jshint bitwise: false */
    return isString(s) ? s.replace(/[a-z]/g, function (ch) {
      return String.fromCharCode(ch.charCodeAt(0) & ~32);
    }) : s;
  };
  // String#toLowerCase and String#toUpperCase don't produce correct results in browsers with Turkish
  // locale, for this reason we need to detect this case and redefine lowercase/uppercase methods
  // with correct but slower alternatives.
  if ('i' !== 'I'.toLowerCase()) {
    lowercase = manualLowercase;
    uppercase = manualUppercase;
  }
  var msie,
    // holds major version number for IE, or NaN if UA is not IE.
    jqLite,
    // delay binding since jQuery could be loaded after us.
    jQuery,
    // delay binding
    slice = [].slice, splice = [].splice, push = [].push, toString = Object.prototype.toString, ngMinErr = minErr('ng'),
    /** @name angular */
    angular = window.angular || (window.angular = {}), angularModule, uid = 0;
  /**
 * documentMode is an IE-only property
 * http://msdn.microsoft.com/en-us/library/ie/cc196988(v=vs.85).aspx
 */
  msie = document.documentMode;
  /**
 * @private
 * @param {*} obj
 * @return {boolean} Returns true if `obj` is an array or array-like object (NodeList, Arguments,
 *                   String ...)
 */
  function isArrayLike(obj) {
    if (obj == null || isWindow(obj)) {
      return false;
    }
    // Support: iOS 8.2 (not reproducible in simulator)
    // "length" in obj used to prevent JIT error (gh-11508)
    var length = 'length' in Object(obj) && obj.length;
    if (obj.nodeType === NODE_TYPE_ELEMENT && length) {
      return true;
    }
    return isString(obj) || isArray(obj) || length === 0 || typeof length === 'number' && length > 0 && length - 1 in obj;
  }
  /**
 * @ngdoc function
 * @name angular.forEach
 * @module ng
 * @kind function
 *
 * @description
 * Invokes the `iterator` function once for each item in `obj` collection, which can be either an
 * object or an array. The `iterator` function is invoked with `iterator(value, key, obj)`, where `value`
 * is the value of an object property or an array element, `key` is the object property key or
 * array element index and obj is the `obj` itself. Specifying a `context` for the function is optional.
 *
 * It is worth noting that `.forEach` does not iterate over inherited properties because it filters
 * using the `hasOwnProperty` method.
 *
 * Unlike ES262's
 * [Array.prototype.forEach](http://www.ecma-international.org/ecma-262/5.1/#sec-15.4.4.18),
 * Providing 'undefined' or 'null' values for `obj` will not throw a TypeError, but rather just
 * return the value provided.
 *
   ```js
     var values = {name: 'misko', gender: 'male'};
     var log = [];
     angular.forEach(values, function(value, key) {
       this.push(key + ': ' + value);
     }, log);
     expect(log).toEqual(['name: misko', 'gender: male']);
   ```
 *
 * @param {Object|Array} obj Object to iterate over.
 * @param {Function} iterator Iterator function.
 * @param {Object=} context Object to become context (`this`) for the iterator function.
 * @returns {Object|Array} Reference to `obj`.
 */
  function forEach(obj, iterator, context) {
    var key, length;
    if (obj) {
      if (isFunction(obj)) {
        for (key in obj) {
          // Need to check if hasOwnProperty exists,
          // as on IE8 the result of querySelectorAll is an object without a hasOwnProperty function
          if (key != 'prototype' && key != 'length' && key != 'name' && (!obj.hasOwnProperty || obj.hasOwnProperty(key))) {
            iterator.call(context, obj[key], key, obj);
          }
        }
      } else if (isArray(obj) || isArrayLike(obj)) {
        var isPrimitive = typeof obj !== 'object';
        for (key = 0, length = obj.length; key < length; key++) {
          if (isPrimitive || key in obj) {
            iterator.call(context, obj[key], key, obj);
          }
        }
      } else if (obj.forEach && obj.forEach !== forEach) {
        obj.forEach(iterator, context, obj);
      } else {
        for (key in obj) {
          if (obj.hasOwnProperty(key)) {
            iterator.call(context, obj[key], key, obj);
          }
        }
      }
    }
    return obj;
  }
  function sortedKeys(obj) {
    return Object.keys(obj).sort();
  }
  function forEachSorted(obj, iterator, context) {
    var keys = sortedKeys(obj);
    for (var i = 0; i < keys.length; i++) {
      iterator.call(context, obj[keys[i]], keys[i]);
    }
    return keys;
  }
  /**
 * when using forEach the params are value, key, but it is often useful to have key, value.
 * @param {function(string, *)} iteratorFn
 * @returns {function(*, string)}
 */
  function reverseParams(iteratorFn) {
    return function (value, key) {
      iteratorFn(key, value);
    };
  }
  /**
 * A consistent way of creating unique IDs in angular.
 *
 * Using simple numbers allows us to generate 28.6 million unique ids per second for 10 years before
 * we hit number precision issues in JavaScript.
 *
 * Math.pow(2,53) / 60 / 60 / 24 / 365 / 10 = 28.6M
 *
 * @returns {number} an unique alpha-numeric string
 */
  function nextUid() {
    return ++uid;
  }
  /**
 * Set or clear the hashkey for an object.
 * @param obj object
 * @param h the hashkey (!truthy to delete the hashkey)
 */
  function setHashKey(obj, h) {
    if (h) {
      obj.$$hashKey = h;
    } else {
      delete obj.$$hashKey;
    }
  }
  /**
 * @ngdoc function
 * @name angular.extend
 * @module ng
 * @kind function
 *
 * @description
 * Extends the destination object `dst` by copying own enumerable properties from the `src` object(s)
 * to `dst`. You can specify multiple `src` objects. If you want to preserve original objects, you can do so
 * by passing an empty object as the target: `var object = angular.extend({}, object1, object2)`.
 * Note: Keep in mind that `angular.extend` does not support recursive merge (deep copy).
 *
 * @param {Object} dst Destination object.
 * @param {...Object} src Source object(s).
 * @returns {Object} Reference to `dst`.
 */
  function extend(dst) {
    var h = dst.$$hashKey;
    for (var i = 1, ii = arguments.length; i < ii; i++) {
      var obj = arguments[i];
      if (obj) {
        var keys = Object.keys(obj);
        for (var j = 0, jj = keys.length; j < jj; j++) {
          var key = keys[j];
          dst[key] = obj[key];
        }
      }
    }
    setHashKey(dst, h);
    return dst;
  }
  function int(str) {
    return parseInt(str, 10);
  }
  function inherit(parent, extra) {
    return extend(Object.create(parent), extra);
  }
  /**
 * @ngdoc function
 * @name angular.noop
 * @module ng
 * @kind function
 *
 * @description
 * A function that performs no operations. This function can be useful when writing code in the
 * functional style.
   ```js
     function foo(callback) {
       var result = calculateResult();
       (callback || angular.noop)(result);
     }
   ```
 */
  function noop() {
  }
  noop.$inject = [];
  /**
 * @ngdoc function
 * @name angular.identity
 * @module ng
 * @kind function
 *
 * @description
 * A function that returns its first argument. This function is useful when writing code in the
 * functional style.
 *
   ```js
     function transformer(transformationFn, value) {
       return (transformationFn || angular.identity)(value);
     };
   ```
  * @param {*} value to be returned.
  * @returns {*} the value passed in.
 */
  function identity($) {
    return $;
  }
  identity.$inject = [];
  function valueFn(value) {
    return function () {
      return value;
    };
  }
  /**
 * @ngdoc function
 * @name angular.isUndefined
 * @module ng
 * @kind function
 *
 * @description
 * Determines if a reference is undefined.
 *
 * @param {*} value Reference to check.
 * @returns {boolean} True if `value` is undefined.
 */
  function isUndefined(value) {
    return typeof value === 'undefined';
  }
  /**
 * @ngdoc function
 * @name angular.isDefined
 * @module ng
 * @kind function
 *
 * @description
 * Determines if a reference is defined.
 *
 * @param {*} value Reference to check.
 * @returns {boolean} True if `value` is defined.
 */
  function isDefined(value) {
    return typeof value !== 'undefined';
  }
  /**
 * @ngdoc function
 * @name angular.isObject
 * @module ng
 * @kind function
 *
 * @description
 * Determines if a reference is an `Object`. Unlike `typeof` in JavaScript, `null`s are not
 * considered to be objects. Note that JavaScript arrays are objects.
 *
 * @param {*} value Reference to check.
 * @returns {boolean} True if `value` is an `Object` but not `null`.
 */
  function isObject(value) {
    // http://jsperf.com/isobject4
    return value !== null && typeof value === 'object';
  }
  /**
 * @ngdoc function
 * @name angular.isString
 * @module ng
 * @kind function
 *
 * @description
 * Determines if a reference is a `String`.
 *
 * @param {*} value Reference to check.
 * @returns {boolean} True if `value` is a `String`.
 */
  function isString(value) {
    return typeof value === 'string';
  }
  /**
 * @ngdoc function
 * @name angular.isNumber
 * @module ng
 * @kind function
 *
 * @description
 * Determines if a reference is a `Number`.
 *
 * This includes the "special" numbers `NaN`, `+Infinity` and `-Infinity`.
 *
 * If you wish to exclude these then you can use the native
 * [`isFinite'](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/isFinite)
 * method.
 *
 * @param {*} value Reference to check.
 * @returns {boolean} True if `value` is a `Number`.
 */
  function isNumber(value) {
    return typeof value === 'number';
  }
  /**
 * @ngdoc function
 * @name angular.isDate
 * @module ng
 * @kind function
 *
 * @description
 * Determines if a value is a date.
 *
 * @param {*} value Reference to check.
 * @returns {boolean} True if `value` is a `Date`.
 */
  function isDate(value) {
    return toString.call(value) === '[object Date]';
  }
  /**
 * @ngdoc function
 * @name angular.isArray
 * @module ng
 * @kind function
 *
 * @description
 * Determines if a reference is an `Array`.
 *
 * @param {*} value Reference to check.
 * @returns {boolean} True if `value` is an `Array`.
 */
  var isArray = Array.isArray;
  /**
 * @ngdoc function
 * @name angular.isFunction
 * @module ng
 * @kind function
 *
 * @description
 * Determines if a reference is a `Function`.
 *
 * @param {*} value Reference to check.
 * @returns {boolean} True if `value` is a `Function`.
 */
  function isFunction(value) {
    return typeof value === 'function';
  }
  /**
 * Determines if a value is a regular expression object.
 *
 * @private
 * @param {*} value Reference to check.
 * @returns {boolean} True if `value` is a `RegExp`.
 */
  function isRegExp(value) {
    return toString.call(value) === '[object RegExp]';
  }
  /**
 * Checks if `obj` is a window object.
 *
 * @private
 * @param {*} obj Object to check
 * @returns {boolean} True if `obj` is a window obj.
 */
  function isWindow(obj) {
    return obj && obj.window === obj;
  }
  function isScope(obj) {
    return obj && obj.$evalAsync && obj.$watch;
  }
  function isFile(obj) {
    return toString.call(obj) === '[object File]';
  }
  function isFormData(obj) {
    return toString.call(obj) === '[object FormData]';
  }
  function isBlob(obj) {
    return toString.call(obj) === '[object Blob]';
  }
  function isBoolean(value) {
    return typeof value === 'boolean';
  }
  function isPromiseLike(obj) {
    return obj && isFunction(obj.then);
  }
  var trim = function (value) {
    return isString(value) ? value.trim() : value;
  };
  // Copied from:
  // http://docs.closure-library.googlecode.com/git/local_closure_goog_string_string.js.source.html#line1021
  // Prereq: s is a string.
  var escapeForRegexp = function (s) {
    return s.replace(/([-()\[\]{}+?*.$\^|,:#<!\\])/g, '\\$1').replace(/\x08/g, '\\x08');
  };
  /**
 * @ngdoc function
 * @name angular.isElement
 * @module ng
 * @kind function
 *
 * @description
 * Determines if a reference is a DOM element (or wrapped jQuery element).
 *
 * @param {*} value Reference to check.
 * @returns {boolean} True if `value` is a DOM element (or wrapped jQuery element).
 */
  function isElement(node) {
    return !!(node && (node.nodeName || node.prop && node.attr && node.find));  // we have an on and find method part of jQuery API
  }
  /**
 * @param str 'key1,key2,...'
 * @returns {object} in the form of {key1:true, key2:true, ...}
 */
  function makeMap(str) {
    var obj = {}, items = str.split(','), i;
    for (i = 0; i < items.length; i++)
      obj[items[i]] = true;
    return obj;
  }
  function nodeName_(element) {
    return lowercase(element.nodeName || element[0] && element[0].nodeName);
  }
  function includes(array, obj) {
    return Array.prototype.indexOf.call(array, obj) != -1;
  }
  function arrayRemove(array, value) {
    var index = array.indexOf(value);
    if (index >= 0)
      array.splice(index, 1);
    return value;
  }
  /**
 * @ngdoc function
 * @name angular.copy
 * @module ng
 * @kind function
 *
 * @description
 * Creates a deep copy of `source`, which should be an object or an array.
 *
 * * If no destination is supplied, a copy of the object or array is created.
 * * If a destination is provided, all of its elements (for arrays) or properties (for objects)
 *   are deleted and then all elements/properties from the source are copied to it.
 * * If `source` is not an object or array (inc. `null` and `undefined`), `source` is returned.
 * * If `source` is identical to 'destination' an exception will be thrown.
 *
 * @param {*} source The source that will be used to make a copy.
 *                   Can be any type, including primitives, `null`, and `undefined`.
 * @param {(Object|Array)=} destination Destination into which the source is copied. If
 *     provided, must be of the same type as `source`.
 * @returns {*} The copy or updated `destination`, if `destination` was specified.
 *
 * @example
 <example module="copyExample">
 <file name="index.html">
 <div ng-controller="ExampleController">
 <form novalidate class="simple-form">
 Name: <input type="text" ng-model="user.name" /><br />
 E-mail: <input type="email" ng-model="user.email" /><br />
 Gender: <input type="radio" ng-model="user.gender" value="male" />male
 <input type="radio" ng-model="user.gender" value="female" />female<br />
 <button ng-click="reset()">RESET</button>
 <button ng-click="update(user)">SAVE</button>
 </form>
 <pre>form = {{user | json}}</pre>
 <pre>master = {{master | json}}</pre>
 </div>

 <script>
  angular.module('copyExample', [])
    .controller('ExampleController', ['$scope', function($scope) {
      $scope.master= {};

      $scope.update = function(user) {
        // Example with 1 argument
        $scope.master= angular.copy(user);
      };

      $scope.reset = function() {
        // Example with 2 arguments
        angular.copy($scope.master, $scope.user);
      };

      $scope.reset();
    }]);
 </script>
 </file>
 </example>
 */
  function copy(source, destination, stackSource, stackDest) {
    if (isWindow(source) || isScope(source)) {
      throw ngMinErr('cpws', 'Can\'t copy! Making copies of Window or Scope instances is not supported.');
    }
    if (!destination) {
      destination = source;
      if (source) {
        if (isArray(source)) {
          destination = copy(source, [], stackSource, stackDest);
        } else if (isDate(source)) {
          destination = new Date(source.getTime());
        } else if (isRegExp(source)) {
          destination = new RegExp(source.source, source.toString().match(/[^\/]*$/)[0]);
          destination.lastIndex = source.lastIndex;
        } else if (isObject(source)) {
          var emptyObject = Object.create(Object.getPrototypeOf(source));
          destination = copy(source, emptyObject, stackSource, stackDest);
        }
      }
    } else {
      if (source === destination)
        throw ngMinErr('cpi', 'Can\'t copy! Source and destination are identical.');
      stackSource = stackSource || [];
      stackDest = stackDest || [];
      if (isObject(source)) {
        var index = stackSource.indexOf(source);
        if (index !== -1)
          return stackDest[index];
        stackSource.push(source);
        stackDest.push(destination);
      }
      var result;
      if (isArray(source)) {
        destination.length = 0;
        for (var i = 0; i < source.length; i++) {
          result = copy(source[i], null, stackSource, stackDest);
          if (isObject(source[i])) {
            stackSource.push(source[i]);
            stackDest.push(result);
          }
          destination.push(result);
        }
      } else {
        var h = destination.$$hashKey;
        if (isArray(destination)) {
          destination.length = 0;
        } else {
          forEach(destination, function (value, key) {
            delete destination[key];
          });
        }
        for (var key in source) {
          if (source.hasOwnProperty(key)) {
            result = copy(source[key], null, stackSource, stackDest);
            if (isObject(source[key])) {
              stackSource.push(source[key]);
              stackDest.push(result);
            }
            destination[key] = result;
          }
        }
        setHashKey(destination, h);
      }
    }
    return destination;
  }
  /**
 * Creates a shallow copy of an object, an array or a primitive.
 *
 * Assumes that there are no proto properties for objects.
 */
  function shallowCopy(src, dst) {
    if (isArray(src)) {
      dst = dst || [];
      for (var i = 0, ii = src.length; i < ii; i++) {
        dst[i] = src[i];
      }
    } else if (isObject(src)) {
      dst = dst || {};
      for (var key in src) {
        if (!(key.charAt(0) === '$' && key.charAt(1) === '$')) {
          dst[key] = src[key];
        }
      }
    }
    return dst || src;
  }
  /**
 * @ngdoc function
 * @name angular.equals
 * @module ng
 * @kind function
 *
 * @description
 * Determines if two objects or two values are equivalent. Supports value types, regular
 * expressions, arrays and objects.
 *
 * Two objects or values are considered equivalent if at least one of the following is true:
 *
 * * Both objects or values pass `===` comparison.
 * * Both objects or values are of the same type and all of their properties are equal by
 *   comparing them with `angular.equals`.
 * * Both values are NaN. (In JavaScript, NaN == NaN => false. But we consider two NaN as equal)
 * * Both values represent the same regular expression (In JavaScript,
 *   /abc/ == /abc/ => false. But we consider two regular expressions as equal when their textual
 *   representation matches).
 *
 * During a property comparison, properties of `function` type and properties with names
 * that begin with `$` are ignored.
 *
 * Scope and DOMWindow objects are being compared only by identify (`===`).
 *
 * @param {*} o1 Object or value to compare.
 * @param {*} o2 Object or value to compare.
 * @returns {boolean} True if arguments are equal.
 */
  function equals(o1, o2) {
    if (o1 === o2)
      return true;
    if (o1 === null || o2 === null)
      return false;
    if (o1 !== o1 && o2 !== o2)
      return true;
    // NaN === NaN
    var t1 = typeof o1, t2 = typeof o2, length, key, keySet;
    if (t1 == t2) {
      if (t1 == 'object') {
        if (isArray(o1)) {
          if (!isArray(o2))
            return false;
          if ((length = o1.length) == o2.length) {
            for (key = 0; key < length; key++) {
              if (!equals(o1[key], o2[key]))
                return false;
            }
            return true;
          }
        } else if (isDate(o1)) {
          if (!isDate(o2))
            return false;
          return equals(o1.getTime(), o2.getTime());
        } else if (isRegExp(o1)) {
          return isRegExp(o2) ? o1.toString() == o2.toString() : false;
        } else {
          if (isScope(o1) || isScope(o2) || isWindow(o1) || isWindow(o2) || isArray(o2) || isDate(o2) || isRegExp(o2))
            return false;
          keySet = {};
          for (key in o1) {
            if (key.charAt(0) === '$' || isFunction(o1[key]))
              continue;
            if (!equals(o1[key], o2[key]))
              return false;
            keySet[key] = true;
          }
          for (key in o2) {
            if (!keySet.hasOwnProperty(key) && key.charAt(0) !== '$' && o2[key] !== undefined && !isFunction(o2[key]))
              return false;
          }
          return true;
        }
      }
    }
    return false;
  }
  var csp = function () {
    if (isDefined(csp.isActive_))
      return csp.isActive_;
    var active = !!(document.querySelector('[ng-csp]') || document.querySelector('[data-ng-csp]'));
    if (!active) {
      try {
        /* jshint -W031, -W054 */
        new Function('');  /* jshint +W031, +W054 */
      } catch (e) {
        active = true;
      }
    }
    return csp.isActive_ = active;
  };
  function concat(array1, array2, index) {
    return array1.concat(slice.call(array2, index));
  }
  function sliceArgs(args, startIndex) {
    return slice.call(args, startIndex || 0);
  }
  /* jshint -W101 */
  /**
 * @ngdoc function
 * @name angular.bind
 * @module ng
 * @kind function
 *
 * @description
 * Returns a function which calls function `fn` bound to `self` (`self` becomes the `this` for
 * `fn`). You can supply optional `args` that are prebound to the function. This feature is also
 * known as [partial application](http://en.wikipedia.org/wiki/Partial_application), as
 * distinguished from [function currying](http://en.wikipedia.org/wiki/Currying#Contrast_with_partial_function_application).
 *
 * @param {Object} self Context which `fn` should be evaluated in.
 * @param {function()} fn Function to be bound.
 * @param {...*} args Optional arguments to be prebound to the `fn` function call.
 * @returns {function()} Function that wraps the `fn` with all the specified bindings.
 */
  /* jshint +W101 */
  function bind(self, fn) {
    var curryArgs = arguments.length > 2 ? sliceArgs(arguments, 2) : [];
    if (isFunction(fn) && !(fn instanceof RegExp)) {
      return curryArgs.length ? function () {
        return arguments.length ? fn.apply(self, concat(curryArgs, arguments, 0)) : fn.apply(self, curryArgs);
      } : function () {
        return arguments.length ? fn.apply(self, arguments) : fn.call(self);
      };
    } else {
      // in IE, native methods are not functions so they cannot be bound (note: they don't need to be)
      return fn;
    }
  }
  function toJsonReplacer(key, value) {
    var val = value;
    if (typeof key === 'string' && key.charAt(0) === '$' && key.charAt(1) === '$') {
      val = undefined;
    } else if (isWindow(value)) {
      val = '$WINDOW';
    } else if (value && document === value) {
      val = '$DOCUMENT';
    } else if (isScope(value)) {
      val = '$SCOPE';
    }
    return val;
  }
  /**
 * @ngdoc function
 * @name angular.toJson
 * @module ng
 * @kind function
 *
 * @description
 * Serializes input into a JSON-formatted string. Properties with leading $$ characters will be
 * stripped since angular uses this notation internally.
 *
 * @param {Object|Array|Date|string|number} obj Input to be serialized into JSON.
 * @param {boolean|number} [pretty=2] If set to true, the JSON output will contain newlines and whitespace.
 *    If set to an integer, the JSON output will contain that many spaces per indentation.
 * @returns {string|undefined} JSON-ified string representing `obj`.
 */
  function toJson(obj, pretty) {
    if (typeof obj === 'undefined')
      return undefined;
    if (!isNumber(pretty)) {
      pretty = pretty ? 2 : null;
    }
    return JSON.stringify(obj, toJsonReplacer, pretty);
  }
  /**
 * @ngdoc function
 * @name angular.fromJson
 * @module ng
 * @kind function
 *
 * @description
 * Deserializes a JSON string.
 *
 * @param {string} json JSON string to deserialize.
 * @returns {Object|Array|string|number} Deserialized JSON string.
 */
  function fromJson(json) {
    return isString(json) ? JSON.parse(json) : json;
  }
  /**
 * @returns {string} Returns the string representation of the element.
 */
  function startingTag(element) {
    element = jqLite(element).clone();
    try {
      // turns out IE does not let you set .html() on elements which
      // are not allowed to have children. So we just ignore it.
      element.empty();
    } catch (e) {
    }
    var elemHtml = jqLite('<div>').append(element).html();
    try {
      return element[0].nodeType === NODE_TYPE_TEXT ? lowercase(elemHtml) : elemHtml.match(/^(<[^>]+>)/)[1].replace(/^<([\w\-]+)/, function (match, nodeName) {
        return '<' + lowercase(nodeName);
      });
    } catch (e) {
      return lowercase(elemHtml);
    }
  }
  /////////////////////////////////////////////////
  /**
 * Tries to decode the URI component without throwing an exception.
 *
 * @private
 * @param str value potential URI component to check.
 * @returns {boolean} True if `value` can be decoded
 * with the decodeURIComponent function.
 */
  function tryDecodeURIComponent(value) {
    try {
      return decodeURIComponent(value);
    } catch (e) {
    }
  }
  /**
 * Parses an escaped url query string into key-value pairs.
 * @returns {Object.<string,boolean|Array>}
 */
  function parseKeyValue(keyValue) {
    var obj = {}, key_value, key;
    forEach((keyValue || '').split('&'), function (keyValue) {
      if (keyValue) {
        key_value = keyValue.replace(/\+/g, '%20').split('=');
        key = tryDecodeURIComponent(key_value[0]);
        if (isDefined(key)) {
          var val = isDefined(key_value[1]) ? tryDecodeURIComponent(key_value[1]) : true;
          if (!hasOwnProperty.call(obj, key)) {
            obj[key] = val;
          } else if (isArray(obj[key])) {
            obj[key].push(val);
          } else {
            obj[key] = [
              obj[key],
              val
            ];
          }
        }
      }
    });
    return obj;
  }
  function toKeyValue(obj) {
    var parts = [];
    forEach(obj, function (value, key) {
      if (isArray(value)) {
        forEach(value, function (arrayValue) {
          parts.push(encodeUriQuery(key, true) + (arrayValue === true ? '' : '=' + encodeUriQuery(arrayValue, true)));
        });
      } else {
        parts.push(encodeUriQuery(key, true) + (value === true ? '' : '=' + encodeUriQuery(value, true)));
      }
    });
    return parts.length ? parts.join('&') : '';
  }
  /**
 * We need our custom method because encodeURIComponent is too aggressive and doesn't follow
 * http://www.ietf.org/rfc/rfc3986.txt with regards to the character set (pchar) allowed in path
 * segments:
 *    segment       = *pchar
 *    pchar         = unreserved / pct-encoded / sub-delims / ":" / "@"
 *    pct-encoded   = "%" HEXDIG HEXDIG
 *    unreserved    = ALPHA / DIGIT / "-" / "." / "_" / "~"
 *    sub-delims    = "!" / "$" / "&" / "'" / "(" / ")"
 *                     / "*" / "+" / "," / ";" / "="
 */
  function encodeUriSegment(val) {
    return encodeUriQuery(val, true).replace(/%26/gi, '&').replace(/%3D/gi, '=').replace(/%2B/gi, '+');
  }
  /**
 * This method is intended for encoding *key* or *value* parts of query component. We need a custom
 * method because encodeURIComponent is too aggressive and encodes stuff that doesn't have to be
 * encoded per http://tools.ietf.org/html/rfc3986:
 *    query       = *( pchar / "/" / "?" )
 *    pchar         = unreserved / pct-encoded / sub-delims / ":" / "@"
 *    unreserved    = ALPHA / DIGIT / "-" / "." / "_" / "~"
 *    pct-encoded   = "%" HEXDIG HEXDIG
 *    sub-delims    = "!" / "$" / "&" / "'" / "(" / ")"
 *                     / "*" / "+" / "," / ";" / "="
 */
  function encodeUriQuery(val, pctEncodeSpaces) {
    return encodeURIComponent(val).replace(/%40/gi, '@').replace(/%3A/gi, ':').replace(/%24/g, '$').replace(/%2C/gi, ',').replace(/%3B/gi, ';').replace(/%20/g, pctEncodeSpaces ? '%20' : '+');
  }
  var ngAttrPrefixes = [
      'ng-',
      'data-ng-',
      'ng:',
      'x-ng-'
    ];
  function getNgAttribute(element, ngAttr) {
    var attr, i, ii = ngAttrPrefixes.length;
    element = jqLite(element);
    for (i = 0; i < ii; ++i) {
      attr = ngAttrPrefixes[i] + ngAttr;
      if (isString(attr = element.attr(attr))) {
        return attr;
      }
    }
    return null;
  }
  /**
 * @ngdoc directive
 * @name ngApp
 * @module ng
 *
 * @element ANY
 * @param {angular.Module} ngApp an optional application
 *   {@link angular.module module} name to load.
 * @param {boolean=} ngStrictDi if this attribute is present on the app element, the injector will be
 *   created in "strict-di" mode. This means that the application will fail to invoke functions which
 *   do not use explicit function annotation (and are thus unsuitable for minification), as described
 *   in {@link guide/di the Dependency Injection guide}, and useful debugging info will assist in
 *   tracking down the root of these bugs.
 *
 * @description
 *
 * Use this directive to **auto-bootstrap** an AngularJS application. The `ngApp` directive
 * designates the **root element** of the application and is typically placed near the root element
 * of the page - e.g. on the `<body>` or `<html>` tags.
 *
 * Only one AngularJS application can be auto-bootstrapped per HTML document. The first `ngApp`
 * found in the document will be used to define the root element to auto-bootstrap as an
 * application. To run multiple applications in an HTML document you must manually bootstrap them using
 * {@link angular.bootstrap} instead. AngularJS applications cannot be nested within each other.
 *
 * You can specify an **AngularJS module** to be used as the root module for the application.  This
 * module will be loaded into the {@link auto.$injector} when the application is bootstrapped. It
 * should contain the application code needed or have dependencies on other modules that will
 * contain the code. See {@link angular.module} for more information.
 *
 * In the example below if the `ngApp` directive were not placed on the `html` element then the
 * document would not be compiled, the `AppController` would not be instantiated and the `{{ a+b }}`
 * would not be resolved to `3`.
 *
 * `ngApp` is the easiest, and most common way to bootstrap an application.
 *
 <example module="ngAppDemo">
   <file name="index.html">
   <div ng-controller="ngAppDemoController">
     I can add: {{a}} + {{b}} =  {{ a+b }}
   </div>
   </file>
   <file name="script.js">
   angular.module('ngAppDemo', []).controller('ngAppDemoController', function($scope) {
     $scope.a = 1;
     $scope.b = 2;
   });
   </file>
 </example>
 *
 * Using `ngStrictDi`, you would see something like this:
 *
 <example ng-app-included="true">
   <file name="index.html">
   <div ng-app="ngAppStrictDemo" ng-strict-di>
       <div ng-controller="GoodController1">
           I can add: {{a}} + {{b}} =  {{ a+b }}

           <p>This renders because the controller does not fail to
              instantiate, by using explicit annotation style (see
              script.js for details)
           </p>
       </div>

       <div ng-controller="GoodController2">
           Name: <input ng-model="name"><br />
           Hello, {{name}}!

           <p>This renders because the controller does not fail to
              instantiate, by using explicit annotation style
              (see script.js for details)
           </p>
       </div>

       <div ng-controller="BadController">
           I can add: {{a}} + {{b}} =  {{ a+b }}

           <p>The controller could not be instantiated, due to relying
              on automatic function annotations (which are disabled in
              strict mode). As such, the content of this section is not
              interpolated, and there should be an error in your web console.
           </p>
       </div>
   </div>
   </file>
   <file name="script.js">
   angular.module('ngAppStrictDemo', [])
     // BadController will fail to instantiate, due to relying on automatic function annotation,
     // rather than an explicit annotation
     .controller('BadController', function($scope) {
       $scope.a = 1;
       $scope.b = 2;
     })
     // Unlike BadController, GoodController1 and GoodController2 will not fail to be instantiated,
     // due to using explicit annotations using the array style and $inject property, respectively.
     .controller('GoodController1', ['$scope', function($scope) {
       $scope.a = 1;
       $scope.b = 2;
     }])
     .controller('GoodController2', GoodController2);
     function GoodController2($scope) {
       $scope.name = "World";
     }
     GoodController2.$inject = ['$scope'];
   </file>
   <file name="style.css">
   div[ng-controller] {
       margin-bottom: 1em;
       -webkit-border-radius: 4px;
       border-radius: 4px;
       border: 1px solid;
       padding: .5em;
   }
   div[ng-controller^=Good] {
       border-color: #d6e9c6;
       background-color: #dff0d8;
       color: #3c763d;
   }
   div[ng-controller^=Bad] {
       border-color: #ebccd1;
       background-color: #f2dede;
       color: #a94442;
       margin-bottom: 0;
   }
   </file>
 </example>
 */
  function angularInit(element, bootstrap) {
    var appElement, module, config = {};
    // The element `element` has priority over any other element
    forEach(ngAttrPrefixes, function (prefix) {
      var name = prefix + 'app';
      if (!appElement && element.hasAttribute && element.hasAttribute(name)) {
        appElement = element;
        module = element.getAttribute(name);
      }
    });
    forEach(ngAttrPrefixes, function (prefix) {
      var name = prefix + 'app';
      var candidate;
      if (!appElement && (candidate = element.querySelector('[' + name.replace(':', '\\:') + ']'))) {
        appElement = candidate;
        module = candidate.getAttribute(name);
      }
    });
    if (appElement) {
      config.strictDi = getNgAttribute(appElement, 'strict-di') !== null;
      bootstrap(appElement, module ? [module] : [], config);
    }
  }
  /**
 * @ngdoc function
 * @name angular.bootstrap
 * @module ng
 * @description
 * Use this function to manually start up angular application.
 *
 * See: {@link guide/bootstrap Bootstrap}
 *
 * Note that Protractor based end-to-end tests cannot use this function to bootstrap manually.
 * They must use {@link ng.directive:ngApp ngApp}.
 *
 * Angular will detect if it has been loaded into the browser more than once and only allow the
 * first loaded script to be bootstrapped and will report a warning to the browser console for
 * each of the subsequent scripts. This prevents strange results in applications, where otherwise
 * multiple instances of Angular try to work on the DOM.
 *
 * ```html
 * <!doctype html>
 * <html>
 * <body>
 * <div ng-controller="WelcomeController">
 *   {{greeting}}
 * </div>
 *
 * <script src="angular.js"></script>
 * <script>
 *   var app = angular.module('demo', [])
 *   .controller('WelcomeController', function($scope) {
 *       $scope.greeting = 'Welcome!';
 *   });
 *   angular.bootstrap(document, ['demo']);
 * </script>
 * </body>
 * </html>
 * ```
 *
 * @param {DOMElement} element DOM element which is the root of angular application.
 * @param {Array<String|Function|Array>=} modules an array of modules to load into the application.
 *     Each item in the array should be the name of a predefined module or a (DI annotated)
 *     function that will be invoked by the injector as a `config` block.
 *     See: {@link angular.module modules}
 * @param {Object=} config an object for defining configuration options for the application. The
 *     following keys are supported:
 *
 * * `strictDi` - disable automatic function annotation for the application. This is meant to
 *   assist in finding bugs which break minified code. Defaults to `false`.
 *
 * @returns {auto.$injector} Returns the newly created injector for this app.
 */
  function bootstrap(element, modules, config) {
    if (!isObject(config))
      config = {};
    var defaultConfig = { strictDi: false };
    config = extend(defaultConfig, config);
    var doBootstrap = function () {
      element = jqLite(element);
      if (element.injector()) {
        var tag = element[0] === document ? 'document' : startingTag(element);
        //Encode angle brackets to prevent input from being sanitized to empty string #8683
        throw ngMinErr('btstrpd', 'App Already Bootstrapped with this Element \'{0}\'', tag.replace(/</, '&lt;').replace(/>/, '&gt;'));
      }
      modules = modules || [];
      modules.unshift([
        '$provide',
        function ($provide) {
          $provide.value('$rootElement', element);
        }
      ]);
      if (config.debugInfoEnabled) {
        // Pushing so that this overrides `debugInfoEnabled` setting defined in user's `modules`.
        modules.push([
          '$compileProvider',
          function ($compileProvider) {
            $compileProvider.debugInfoEnabled(true);
          }
        ]);
      }
      modules.unshift('ng');
      var injector = createInjector(modules, config.strictDi);
      injector.invoke([
        '$rootScope',
        '$rootElement',
        '$compile',
        '$injector',
        function bootstrapApply(scope, element, compile, injector) {
          scope.$apply(function () {
            element.data('$injector', injector);
            compile(element)(scope);
          });
        }
      ]);
      return injector;
    };
    var NG_ENABLE_DEBUG_INFO = /^NG_ENABLE_DEBUG_INFO!/;
    var NG_DEFER_BOOTSTRAP = /^NG_DEFER_BOOTSTRAP!/;
    if (window && NG_ENABLE_DEBUG_INFO.test(window.name)) {
      config.debugInfoEnabled = true;
      window.name = window.name.replace(NG_ENABLE_DEBUG_INFO, '');
    }
    if (window && !NG_DEFER_BOOTSTRAP.test(window.name)) {
      return doBootstrap();
    }
    window.name = window.name.replace(NG_DEFER_BOOTSTRAP, '');
    angular.resumeBootstrap = function (extraModules) {
      forEach(extraModules, function (module) {
        modules.push(module);
      });
      return doBootstrap();
    };
    if (isFunction(angular.resumeDeferredBootstrap)) {
      angular.resumeDeferredBootstrap();
    }
  }
  /**
 * @ngdoc function
 * @name angular.reloadWithDebugInfo
 * @module ng
 * @description
 * Use this function to reload the current application with debug information turned on.
 * This takes precedence over a call to `$compileProvider.debugInfoEnabled(false)`.
 *
 * See {@link ng.$compileProvider#debugInfoEnabled} for more.
 */
  function reloadWithDebugInfo() {
    window.name = 'NG_ENABLE_DEBUG_INFO!' + window.name;
    window.location.reload();
  }
  /**
 * @name angular.getTestability
 * @module ng
 * @description
 * Get the testability service for the instance of Angular on the given
 * element.
 * @param {DOMElement} element DOM element which is the root of angular application.
 */
  function getTestability(rootElement) {
    var injector = angular.element(rootElement).injector();
    if (!injector) {
      throw ngMinErr('test', 'no injector found for element argument to getTestability');
    }
    return injector.get('$$testability');
  }
  var SNAKE_CASE_REGEXP = /[A-Z]/g;
  function snake_case(name, separator) {
    separator = separator || '_';
    return name.replace(SNAKE_CASE_REGEXP, function (letter, pos) {
      return (pos ? separator : '') + letter.toLowerCase();
    });
  }
  var bindJQueryFired = false;
  var skipDestroyOnNextJQueryCleanData;
  function bindJQuery() {
    var originalCleanData;
    if (bindJQueryFired) {
      return;
    }
    // bind to jQuery if present;
    jQuery = window.jQuery;
    // Use jQuery if it exists with proper functionality, otherwise default to us.
    // Angular 1.2+ requires jQuery 1.7+ for on()/off() support.
    // Angular 1.3+ technically requires at least jQuery 2.1+ but it may work with older
    // versions. It will not work for sure with jQuery <1.7, though.
    if (jQuery && jQuery.fn.on) {
      jqLite = jQuery;
      extend(jQuery.fn, {
        scope: JQLitePrototype.scope,
        isolateScope: JQLitePrototype.isolateScope,
        controller: JQLitePrototype.controller,
        injector: JQLitePrototype.injector,
        inheritedData: JQLitePrototype.inheritedData
      });
      // All nodes removed from the DOM via various jQuery APIs like .remove()
      // are passed through jQuery.cleanData. Monkey-patch this method to fire
      // the $destroy event on all removed nodes.
      originalCleanData = jQuery.cleanData;
      jQuery.cleanData = function (elems) {
        var events;
        if (!skipDestroyOnNextJQueryCleanData) {
          for (var i = 0, elem; (elem = elems[i]) != null; i++) {
            events = jQuery._data(elem, 'events');
            if (events && events.$destroy) {
              jQuery(elem).triggerHandler('$destroy');
            }
          }
        } else {
          skipDestroyOnNextJQueryCleanData = false;
        }
        originalCleanData(elems);
      };
    } else {
      jqLite = JQLite;
    }
    angular.element = jqLite;
    // Prevent double-proxying.
    bindJQueryFired = true;
  }
  /**
 * throw error if the argument is falsy.
 */
  function assertArg(arg, name, reason) {
    if (!arg) {
      throw ngMinErr('areq', 'Argument \'{0}\' is {1}', name || '?', reason || 'required');
    }
    return arg;
  }
  function assertArgFn(arg, name, acceptArrayAnnotation) {
    if (acceptArrayAnnotation && isArray(arg)) {
      arg = arg[arg.length - 1];
    }
    assertArg(isFunction(arg), name, 'not a function, got ' + (arg && typeof arg === 'object' ? arg.constructor.name || 'Object' : typeof arg));
    return arg;
  }
  /**
 * throw error if the name given is hasOwnProperty
 * @param  {String} name    the name to test
 * @param  {String} context the context in which the name is used, such as module or directive
 */
  function assertNotHasOwnProperty(name, context) {
    if (name === 'hasOwnProperty') {
      throw ngMinErr('badname', 'hasOwnProperty is not a valid {0} name', context);
    }
  }
  /**
 * Return the value accessible from the object by path. Any undefined traversals are ignored
 * @param {Object} obj starting object
 * @param {String} path path to traverse
 * @param {boolean} [bindFnToScope=true]
 * @returns {Object} value as accessible by path
 */
  //TODO(misko): this function needs to be removed
  function getter(obj, path, bindFnToScope) {
    if (!path)
      return obj;
    var keys = path.split('.');
    var key;
    var lastInstance = obj;
    var len = keys.length;
    for (var i = 0; i < len; i++) {
      key = keys[i];
      if (obj) {
        obj = (lastInstance = obj)[key];
      }
    }
    if (!bindFnToScope && isFunction(obj)) {
      return bind(lastInstance, obj);
    }
    return obj;
  }
  /**
 * Return the DOM siblings between the first and last node in the given array.
 * @param {Array} array like object
 * @returns {jqLite} jqLite collection containing the nodes
 */
  function getBlockNodes(nodes) {
    // TODO(perf): just check if all items in `nodes` are siblings and if they are return the original
    //             collection, otherwise update the original collection.
    var node = nodes[0];
    var endNode = nodes[nodes.length - 1];
    var blockNodes = [node];
    do {
      node = node.nextSibling;
      if (!node)
        break;
      blockNodes.push(node);
    } while (node !== endNode);
    return jqLite(blockNodes);
  }
  /**
 * Creates a new object without a prototype. This object is useful for lookup without having to
 * guard against prototypically inherited properties via hasOwnProperty.
 *
 * Related micro-benchmarks:
 * - http://jsperf.com/object-create2
 * - http://jsperf.com/proto-map-lookup/2
 * - http://jsperf.com/for-in-vs-object-keys2
 *
 * @returns {Object}
 */
  function createMap() {
    return Object.create(null);
  }
  var NODE_TYPE_ELEMENT = 1;
  var NODE_TYPE_ATTRIBUTE = 2;
  var NODE_TYPE_TEXT = 3;
  var NODE_TYPE_COMMENT = 8;
  var NODE_TYPE_DOCUMENT = 9;
  var NODE_TYPE_DOCUMENT_FRAGMENT = 11;
  /**
 * @ngdoc type
 * @name angular.Module
 * @module ng
 * @description
 *
 * Interface for configuring angular {@link angular.module modules}.
 */
  function setupModuleLoader(window) {
    var $injectorMinErr = minErr('$injector');
    var ngMinErr = minErr('ng');
    function ensure(obj, name, factory) {
      return obj[name] || (obj[name] = factory());
    }
    var angular = ensure(window, 'angular', Object);
    // We need to expose `angular.$$minErr` to modules such as `ngResource` that reference it during bootstrap
    angular.$$minErr = angular.$$minErr || minErr;
    return ensure(angular, 'module', function () {
      /** @type {Object.<string, angular.Module>} */
      var modules = {};
      /**
     * @ngdoc function
     * @name angular.module
     * @module ng
     * @description
     *
     * The `angular.module` is a global place for creating, registering and retrieving Angular
     * modules.
     * All modules (angular core or 3rd party) that should be available to an application must be
     * registered using this mechanism.
     *
     * When passed two or more arguments, a new module is created.  If passed only one argument, an
     * existing module (the name passed as the first argument to `module`) is retrieved.
     *
     *
     * # Module
     *
     * A module is a collection of services, directives, controllers, filters, and configuration information.
     * `angular.module` is used to configure the {@link auto.$injector $injector}.
     *
     * ```js
     * // Create a new module
     * var myModule = angular.module('myModule', []);
     *
     * // register a new service
     * myModule.value('appName', 'MyCoolApp');
     *
     * // configure existing services inside initialization blocks.
     * myModule.config(['$locationProvider', function($locationProvider) {
     *   // Configure existing providers
     *   $locationProvider.hashPrefix('!');
     * }]);
     * ```
     *
     * Then you can create an injector and load your modules like this:
     *
     * ```js
     * var injector = angular.injector(['ng', 'myModule'])
     * ```
     *
     * However it's more likely that you'll just use
     * {@link ng.directive:ngApp ngApp} or
     * {@link angular.bootstrap} to simplify this process for you.
     *
     * @param {!string} name The name of the module to create or retrieve.
     * @param {!Array.<string>=} requires If specified then new module is being created. If
     *        unspecified then the module is being retrieved for further configuration.
     * @param {Function=} configFn Optional configuration function for the module. Same as
     *        {@link angular.Module#config Module#config()}.
     * @returns {module} new module with the {@link angular.Module} api.
     */
      return function module(name, requires, configFn) {
        var assertNotHasOwnProperty = function (name, context) {
          if (name === 'hasOwnProperty') {
            throw ngMinErr('badname', 'hasOwnProperty is not a valid {0} name', context);
          }
        };
        assertNotHasOwnProperty(name, 'module');
        if (requires && modules.hasOwnProperty(name)) {
          modules[name] = null;
        }
        return ensure(modules, name, function () {
          if (!requires) {
            throw $injectorMinErr('nomod', 'Module \'{0}\' is not available! You either misspelled ' + 'the module name or forgot to load it. If registering a module ensure that you ' + 'specify the dependencies as the second argument.', name);
          }
          /** @type {!Array.<Array.<*>>} */
          var invokeQueue = [];
          /** @type {!Array.<Function>} */
          var configBlocks = [];
          /** @type {!Array.<Function>} */
          var runBlocks = [];
          var config = invokeLater('$injector', 'invoke', 'push', configBlocks);
          /** @type {angular.Module} */
          var moduleInstance = {
              _invokeQueue: invokeQueue,
              _configBlocks: configBlocks,
              _runBlocks: runBlocks,
              requires: requires,
              name: name,
              provider: invokeLater('$provide', 'provider'),
              factory: invokeLater('$provide', 'factory'),
              service: invokeLater('$provide', 'service'),
              value: invokeLater('$provide', 'value'),
              constant: invokeLater('$provide', 'constant', 'unshift'),
              animation: invokeLater('$animateProvider', 'register'),
              filter: invokeLater('$filterProvider', 'register'),
              controller: invokeLater('$controllerProvider', 'register'),
              directive: invokeLater('$compileProvider', 'directive'),
              config: config,
              run: function (block) {
                runBlocks.push(block);
                return this;
              }
            };
          if (configFn) {
            config(configFn);
          }
          return moduleInstance;
          /**
         * @param {string} provider
         * @param {string} method
         * @param {String=} insertMethod
         * @returns {angular.Module}
         */
          function invokeLater(provider, method, insertMethod, queue) {
            if (!queue)
              queue = invokeQueue;
            return function () {
              queue[insertMethod || 'push']([
                provider,
                method,
                arguments
              ]);
              return moduleInstance;
            };
          }
        });
      };
    });
  }
  /* global: toDebugString: true */
  function serializeObject(obj) {
    var seen = [];
    return JSON.stringify(obj, function (key, val) {
      val = toJsonReplacer(key, val);
      if (isObject(val)) {
        if (seen.indexOf(val) >= 0)
          return '<<already seen>>';
        seen.push(val);
      }
      return val;
    });
  }
  function toDebugString(obj) {
    if (typeof obj === 'function') {
      return obj.toString().replace(/ \{[\s\S]*$/, '');
    } else if (typeof obj === 'undefined') {
      return 'undefined';
    } else if (typeof obj !== 'string') {
      return serializeObject(obj);
    }
    return obj;
  }
  /* global angularModule: true,
  version: true,

  $LocaleProvider,
  $CompileProvider,

  htmlAnchorDirective,
  inputDirective,
  inputDirective,
  formDirective,
  scriptDirective,
  selectDirective,
  styleDirective,
  optionDirective,
  ngBindDirective,
  ngBindHtmlDirective,
  ngBindTemplateDirective,
  ngClassDirective,
  ngClassEvenDirective,
  ngClassOddDirective,
  ngCspDirective,
  ngCloakDirective,
  ngControllerDirective,
  ngFormDirective,
  ngHideDirective,
  ngIfDirective,
  ngIncludeDirective,
  ngIncludeFillContentDirective,
  ngInitDirective,
  ngNonBindableDirective,
  ngPluralizeDirective,
  ngRepeatDirective,
  ngShowDirective,
  ngStyleDirective,
  ngSwitchDirective,
  ngSwitchWhenDirective,
  ngSwitchDefaultDirective,
  ngOptionsDirective,
  ngTranscludeDirective,
  ngModelDirective,
  ngListDirective,
  ngChangeDirective,
  patternDirective,
  patternDirective,
  requiredDirective,
  requiredDirective,
  minlengthDirective,
  minlengthDirective,
  maxlengthDirective,
  maxlengthDirective,
  ngValueDirective,
  ngModelOptionsDirective,
  ngAttributeAliasDirectives,
  ngEventDirectives,

  $AnchorScrollProvider,
  $AnimateProvider,
  $BrowserProvider,
  $CacheFactoryProvider,
  $ControllerProvider,
  $DocumentProvider,
  $ExceptionHandlerProvider,
  $FilterProvider,
  $InterpolateProvider,
  $IntervalProvider,
  $HttpProvider,
  $HttpBackendProvider,
  $LocationProvider,
  $LogProvider,
  $ParseProvider,
  $RootScopeProvider,
  $QProvider,
  $$QProvider,
  $$SanitizeUriProvider,
  $SceProvider,
  $SceDelegateProvider,
  $SnifferProvider,
  $TemplateCacheProvider,
  $TemplateRequestProvider,
  $$TestabilityProvider,
  $TimeoutProvider,
  $$RAFProvider,
  $$AsyncCallbackProvider,
  $WindowProvider,
  $$jqLiteProvider
*/
  /**
 * @ngdoc object
 * @name angular.version
 * @module ng
 * @description
 * An object that contains information about the current AngularJS version. This object has the
 * following properties:
 *
 * - `full` – `{string}` – Full version string, such as "0.9.18".
 * - `major` – `{number}` – Major version number, such as "0".
 * - `minor` – `{number}` – Minor version number, such as "9".
 * - `dot` – `{number}` – Dot version number, such as "18".
 * - `codeName` – `{string}` – Code name of the release, such as "jiggling-armfat".
 */
  var version = {
      full: '1.3.20',
      major: 1,
      minor: 3,
      dot: 20,
      codeName: 'shallow-translucence'
    };
  function publishExternalAPI(angular) {
    extend(angular, {
      'bootstrap': bootstrap,
      'copy': copy,
      'extend': extend,
      'equals': equals,
      'element': jqLite,
      'forEach': forEach,
      'injector': createInjector,
      'noop': noop,
      'bind': bind,
      'toJson': toJson,
      'fromJson': fromJson,
      'identity': identity,
      'isUndefined': isUndefined,
      'isDefined': isDefined,
      'isString': isString,
      'isFunction': isFunction,
      'isObject': isObject,
      'isNumber': isNumber,
      'isElement': isElement,
      'isArray': isArray,
      'version': version,
      'isDate': isDate,
      'lowercase': lowercase,
      'uppercase': uppercase,
      'callbacks': { counter: 0 },
      'getTestability': getTestability,
      '$$minErr': minErr,
      '$$csp': csp,
      'reloadWithDebugInfo': reloadWithDebugInfo
    });
    angularModule = setupModuleLoader(window);
    try {
      angularModule('ngLocale');
    } catch (e) {
      angularModule('ngLocale', []).provider('$locale', $LocaleProvider);
    }
    angularModule('ng', ['ngLocale'], [
      '$provide',
      function ngModule($provide) {
        // $$sanitizeUriProvider needs to be before $compileProvider as it is used by it.
        $provide.provider({ $$sanitizeUri: $$SanitizeUriProvider });
        $provide.provider('$compile', $CompileProvider).directive({
          a: htmlAnchorDirective,
          input: inputDirective,
          textarea: inputDirective,
          form: formDirective,
          script: scriptDirective,
          select: selectDirective,
          style: styleDirective,
          option: optionDirective,
          ngBind: ngBindDirective,
          ngBindHtml: ngBindHtmlDirective,
          ngBindTemplate: ngBindTemplateDirective,
          ngClass: ngClassDirective,
          ngClassEven: ngClassEvenDirective,
          ngClassOdd: ngClassOddDirective,
          ngCloak: ngCloakDirective,
          ngController: ngControllerDirective,
          ngForm: ngFormDirective,
          ngHide: ngHideDirective,
          ngIf: ngIfDirective,
          ngInclude: ngIncludeDirective,
          ngInit: ngInitDirective,
          ngNonBindable: ngNonBindableDirective,
          ngPluralize: ngPluralizeDirective,
          ngRepeat: ngRepeatDirective,
          ngShow: ngShowDirective,
          ngStyle: ngStyleDirective,
          ngSwitch: ngSwitchDirective,
          ngSwitchWhen: ngSwitchWhenDirective,
          ngSwitchDefault: ngSwitchDefaultDirective,
          ngOptions: ngOptionsDirective,
          ngTransclude: ngTranscludeDirective,
          ngModel: ngModelDirective,
          ngList: ngListDirective,
          ngChange: ngChangeDirective,
          pattern: patternDirective,
          ngPattern: patternDirective,
          required: requiredDirective,
          ngRequired: requiredDirective,
          minlength: minlengthDirective,
          ngMinlength: minlengthDirective,
          maxlength: maxlengthDirective,
          ngMaxlength: maxlengthDirective,
          ngValue: ngValueDirective,
          ngModelOptions: ngModelOptionsDirective
        }).directive({ ngInclude: ngIncludeFillContentDirective }).directive(ngAttributeAliasDirectives).directive(ngEventDirectives);
        $provide.provider({
          $anchorScroll: $AnchorScrollProvider,
          $animate: $AnimateProvider,
          $browser: $BrowserProvider,
          $cacheFactory: $CacheFactoryProvider,
          $controller: $ControllerProvider,
          $document: $DocumentProvider,
          $exceptionHandler: $ExceptionHandlerProvider,
          $filter: $FilterProvider,
          $interpolate: $InterpolateProvider,
          $interval: $IntervalProvider,
          $http: $HttpProvider,
          $httpBackend: $HttpBackendProvider,
          $location: $LocationProvider,
          $log: $LogProvider,
          $parse: $ParseProvider,
          $rootScope: $RootScopeProvider,
          $q: $QProvider,
          $$q: $$QProvider,
          $sce: $SceProvider,
          $sceDelegate: $SceDelegateProvider,
          $sniffer: $SnifferProvider,
          $templateCache: $TemplateCacheProvider,
          $templateRequest: $TemplateRequestProvider,
          $$testability: $$TestabilityProvider,
          $timeout: $TimeoutProvider,
          $window: $WindowProvider,
          $$rAF: $$RAFProvider,
          $$asyncCallback: $$AsyncCallbackProvider,
          $$jqLite: $$jqLiteProvider
        });
      }
    ]);
  }
  /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *     Any commits to this file should be reviewed with security in mind.  *
 *   Changes to this file can potentially create security vulnerabilities. *
 *          An approval from 2 Core members with history of modifying      *
 *                         this file is required.                          *
 *                                                                         *
 *  Does the change somehow allow for arbitrary javascript to be executed? *
 *    Or allows for someone to change the prototype of built-in objects?   *
 *     Or gives undesired access to variables likes document or window?    *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
  /* global JQLitePrototype: true,
  addEventListenerFn: true,
  removeEventListenerFn: true,
  BOOLEAN_ATTR: true,
  ALIASED_ATTR: true,
*/
  //////////////////////////////////
  //JQLite
  //////////////////////////////////
  /**
 * @ngdoc function
 * @name angular.element
 * @module ng
 * @kind function
 *
 * @description
 * Wraps a raw DOM element or HTML string as a [jQuery](http://jquery.com) element.
 *
 * If jQuery is available, `angular.element` is an alias for the
 * [jQuery](http://api.jquery.com/jQuery/) function. If jQuery is not available, `angular.element`
 * delegates to Angular's built-in subset of jQuery, called "jQuery lite" or "jqLite."
 *
 * <div class="alert alert-success">jqLite is a tiny, API-compatible subset of jQuery that allows
 * Angular to manipulate the DOM in a cross-browser compatible way. **jqLite** implements only the most
 * commonly needed functionality with the goal of having a very small footprint.</div>
 *
 * To use `jQuery`, simply ensure it is loaded before the `angular.js` file.
 *
 * <div class="alert">**Note:** all element references in Angular are always wrapped with jQuery or
 * jqLite; they are never raw DOM references.</div>
 *
 * ## Angular's jqLite
 * jqLite provides only the following jQuery methods:
 *
 * - [`addClass()`](http://api.jquery.com/addClass/)
 * - [`after()`](http://api.jquery.com/after/)
 * - [`append()`](http://api.jquery.com/append/)
 * - [`attr()`](http://api.jquery.com/attr/) - Does not support functions as parameters
 * - [`bind()`](http://api.jquery.com/bind/) - Does not support namespaces, selectors or eventData
 * - [`children()`](http://api.jquery.com/children/) - Does not support selectors
 * - [`clone()`](http://api.jquery.com/clone/)
 * - [`contents()`](http://api.jquery.com/contents/)
 * - [`css()`](http://api.jquery.com/css/) - Only retrieves inline-styles, does not call `getComputedStyle()`. As a setter, does not convert numbers to strings or append 'px'.
 * - [`data()`](http://api.jquery.com/data/)
 * - [`detach()`](http://api.jquery.com/detach/)
 * - [`empty()`](http://api.jquery.com/empty/)
 * - [`eq()`](http://api.jquery.com/eq/)
 * - [`find()`](http://api.jquery.com/find/) - Limited to lookups by tag name
 * - [`hasClass()`](http://api.jquery.com/hasClass/)
 * - [`html()`](http://api.jquery.com/html/)
 * - [`next()`](http://api.jquery.com/next/) - Does not support selectors
 * - [`on()`](http://api.jquery.com/on/) - Does not support namespaces, selectors or eventData
 * - [`off()`](http://api.jquery.com/off/) - Does not support namespaces or selectors
 * - [`one()`](http://api.jquery.com/one/) - Does not support namespaces or selectors
 * - [`parent()`](http://api.jquery.com/parent/) - Does not support selectors
 * - [`prepend()`](http://api.jquery.com/prepend/)
 * - [`prop()`](http://api.jquery.com/prop/)
 * - [`ready()`](http://api.jquery.com/ready/)
 * - [`remove()`](http://api.jquery.com/remove/)
 * - [`removeAttr()`](http://api.jquery.com/removeAttr/)
 * - [`removeClass()`](http://api.jquery.com/removeClass/)
 * - [`removeData()`](http://api.jquery.com/removeData/)
 * - [`replaceWith()`](http://api.jquery.com/replaceWith/)
 * - [`text()`](http://api.jquery.com/text/)
 * - [`toggleClass()`](http://api.jquery.com/toggleClass/)
 * - [`triggerHandler()`](http://api.jquery.com/triggerHandler/) - Passes a dummy event object to handlers.
 * - [`unbind()`](http://api.jquery.com/unbind/) - Does not support namespaces
 * - [`val()`](http://api.jquery.com/val/)
 * - [`wrap()`](http://api.jquery.com/wrap/)
 *
 * ## jQuery/jqLite Extras
 * Angular also provides the following additional methods and events to both jQuery and jqLite:
 *
 * ### Events
 * - `$destroy` - AngularJS intercepts all jqLite/jQuery's DOM destruction apis and fires this event
 *    on all DOM nodes being removed.  This can be used to clean up any 3rd party bindings to the DOM
 *    element before it is removed.
 *
 * ### Methods
 * - `controller(name)` - retrieves the controller of the current element or its parent. By default
 *   retrieves controller associated with the `ngController` directive. If `name` is provided as
 *   camelCase directive name, then the controller for this directive will be retrieved (e.g.
 *   `'ngModel'`).
 * - `injector()` - retrieves the injector of the current element or its parent.
 * - `scope()` - retrieves the {@link ng.$rootScope.Scope scope} of the current
 *   element or its parent. Requires {@link guide/production#disabling-debug-data Debug Data} to
 *   be enabled.
 * - `isolateScope()` - retrieves an isolate {@link ng.$rootScope.Scope scope} if one is attached directly to the
 *   current element. This getter should be used only on elements that contain a directive which starts a new isolate
 *   scope. Calling `scope()` on this element always returns the original non-isolate scope.
 *   Requires {@link guide/production#disabling-debug-data Debug Data} to be enabled.
 * - `inheritedData()` - same as `data()`, but walks up the DOM until a value is found or the top
 *   parent element is reached.
 *
 * @param {string|DOMElement} element HTML string or DOMElement to be wrapped into jQuery.
 * @returns {Object} jQuery object.
 */
  JQLite.expando = 'ng339';
  var jqCache = JQLite.cache = {}, jqId = 1, addEventListenerFn = function (element, type, fn) {
      element.addEventListener(type, fn, false);
    }, removeEventListenerFn = function (element, type, fn) {
      element.removeEventListener(type, fn, false);
    };
  /*
 * !!! This is an undocumented "private" function !!!
 */
  JQLite._data = function (node) {
    //jQuery always returns an object on cache miss
    return this.cache[node[this.expando]] || {};
  };
  function jqNextId() {
    return ++jqId;
  }
  var SPECIAL_CHARS_REGEXP = /([\:\-\_]+(.))/g;
  var MOZ_HACK_REGEXP = /^moz([A-Z])/;
  var MOUSE_EVENT_MAP = {
      mouseleave: 'mouseout',
      mouseenter: 'mouseover'
    };
  var jqLiteMinErr = minErr('jqLite');
  /**
 * Converts snake_case to camelCase.
 * Also there is special case for Moz prefix starting with upper case letter.
 * @param name Name to normalize
 */
  function camelCase(name) {
    return name.replace(SPECIAL_CHARS_REGEXP, function (_, separator, letter, offset) {
      return offset ? letter.toUpperCase() : letter;
    }).replace(MOZ_HACK_REGEXP, 'Moz$1');
  }
  var SINGLE_TAG_REGEXP = /^<(\w+)\s*\/?>(?:<\/\1>|)$/;
  var HTML_REGEXP = /<|&#?\w+;/;
  var TAG_NAME_REGEXP = /<([\w:]+)/;
  var XHTML_TAG_REGEXP = /<(?!area|br|col|embed|hr|img|input|link|meta|param)(([\w:]+)[^>]*)\/>/gi;
  var wrapMap = {
      'option': [
        1,
        '<select multiple="multiple">',
        '</select>'
      ],
      'thead': [
        1,
        '<table>',
        '</table>'
      ],
      'col': [
        2,
        '<table><colgroup>',
        '</colgroup></table>'
      ],
      'tr': [
        2,
        '<table><tbody>',
        '</tbody></table>'
      ],
      'td': [
        3,
        '<table><tbody><tr>',
        '</tr></tbody></table>'
      ],
      '_default': [
        0,
        '',
        ''
      ]
    };
  wrapMap.optgroup = wrapMap.option;
  wrapMap.tbody = wrapMap.tfoot = wrapMap.colgroup = wrapMap.caption = wrapMap.thead;
  wrapMap.th = wrapMap.td;
  function jqLiteIsTextNode(html) {
    return !HTML_REGEXP.test(html);
  }
  function jqLiteAcceptsData(node) {
    // The window object can accept data but has no nodeType
    // Otherwise we are only interested in elements (1) and documents (9)
    var nodeType = node.nodeType;
    return nodeType === NODE_TYPE_ELEMENT || !nodeType || nodeType === NODE_TYPE_DOCUMENT;
  }
  function jqLiteBuildFragment(html, context) {
    var tmp, tag, wrap, fragment = context.createDocumentFragment(), nodes = [], i;
    if (jqLiteIsTextNode(html)) {
      // Convert non-html into a text node
      nodes.push(context.createTextNode(html));
    } else {
      // Convert html into DOM nodes
      tmp = tmp || fragment.appendChild(context.createElement('div'));
      tag = (TAG_NAME_REGEXP.exec(html) || [
        '',
        ''
      ])[1].toLowerCase();
      wrap = wrapMap[tag] || wrapMap._default;
      tmp.innerHTML = wrap[1] + html.replace(XHTML_TAG_REGEXP, '<$1></$2>') + wrap[2];
      // Descend through wrappers to the right content
      i = wrap[0];
      while (i--) {
        tmp = tmp.lastChild;
      }
      nodes = concat(nodes, tmp.childNodes);
      tmp = fragment.firstChild;
      tmp.textContent = '';
    }
    // Remove wrapper from fragment
    fragment.textContent = '';
    fragment.innerHTML = '';
    // Clear inner HTML
    forEach(nodes, function (node) {
      fragment.appendChild(node);
    });
    return fragment;
  }
  function jqLiteParseHTML(html, context) {
    context = context || document;
    var parsed;
    if (parsed = SINGLE_TAG_REGEXP.exec(html)) {
      return [context.createElement(parsed[1])];
    }
    if (parsed = jqLiteBuildFragment(html, context)) {
      return parsed.childNodes;
    }
    return [];
  }
  /////////////////////////////////////////////
  function JQLite(element) {
    if (element instanceof JQLite) {
      return element;
    }
    var argIsString;
    if (isString(element)) {
      element = trim(element);
      argIsString = true;
    }
    if (!(this instanceof JQLite)) {
      if (argIsString && element.charAt(0) != '<') {
        throw jqLiteMinErr('nosel', 'Looking up elements via selectors is not supported by jqLite! See: http://docs.angularjs.org/api/angular.element');
      }
      return new JQLite(element);
    }
    if (argIsString) {
      jqLiteAddNodes(this, jqLiteParseHTML(element));
    } else {
      jqLiteAddNodes(this, element);
    }
  }
  function jqLiteClone(element) {
    return element.cloneNode(true);
  }
  function jqLiteDealoc(element, onlyDescendants) {
    if (!onlyDescendants)
      jqLiteRemoveData(element);
    if (element.querySelectorAll) {
      var descendants = element.querySelectorAll('*');
      for (var i = 0, l = descendants.length; i < l; i++) {
        jqLiteRemoveData(descendants[i]);
      }
    }
  }
  function jqLiteOff(element, type, fn, unsupported) {
    if (isDefined(unsupported))
      throw jqLiteMinErr('offargs', 'jqLite#off() does not support the `selector` argument');
    var expandoStore = jqLiteExpandoStore(element);
    var events = expandoStore && expandoStore.events;
    var handle = expandoStore && expandoStore.handle;
    if (!handle)
      return;
    //no listeners registered
    if (!type) {
      for (type in events) {
        if (type !== '$destroy') {
          removeEventListenerFn(element, type, handle);
        }
        delete events[type];
      }
    } else {
      forEach(type.split(' '), function (type) {
        if (isDefined(fn)) {
          var listenerFns = events[type];
          arrayRemove(listenerFns || [], fn);
          if (listenerFns && listenerFns.length > 0) {
            return;
          }
        }
        removeEventListenerFn(element, type, handle);
        delete events[type];
      });
    }
  }
  function jqLiteRemoveData(element, name) {
    var expandoId = element.ng339;
    var expandoStore = expandoId && jqCache[expandoId];
    if (expandoStore) {
      if (name) {
        delete expandoStore.data[name];
        return;
      }
      if (expandoStore.handle) {
        if (expandoStore.events.$destroy) {
          expandoStore.handle({}, '$destroy');
        }
        jqLiteOff(element);
      }
      delete jqCache[expandoId];
      element.ng339 = undefined;  // don't delete DOM expandos. IE and Chrome don't like it
    }
  }
  function jqLiteExpandoStore(element, createIfNecessary) {
    var expandoId = element.ng339, expandoStore = expandoId && jqCache[expandoId];
    if (createIfNecessary && !expandoStore) {
      element.ng339 = expandoId = jqNextId();
      expandoStore = jqCache[expandoId] = {
        events: {},
        data: {},
        handle: undefined
      };
    }
    return expandoStore;
  }
  function jqLiteData(element, key, value) {
    if (jqLiteAcceptsData(element)) {
      var isSimpleSetter = isDefined(value);
      var isSimpleGetter = !isSimpleSetter && key && !isObject(key);
      var massGetter = !key;
      var expandoStore = jqLiteExpandoStore(element, !isSimpleGetter);
      var data = expandoStore && expandoStore.data;
      if (isSimpleSetter) {
        // data('key', value)
        data[key] = value;
      } else {
        if (massGetter) {
          // data()
          return data;
        } else {
          if (isSimpleGetter) {
            // data('key')
            // don't force creation of expandoStore if it doesn't exist yet
            return data && data[key];
          } else {
            // mass-setter: data({key1: val1, key2: val2})
            extend(data, key);
          }
        }
      }
    }
  }
  function jqLiteHasClass(element, selector) {
    if (!element.getAttribute)
      return false;
    return (' ' + (element.getAttribute('class') || '') + ' ').replace(/[\n\t]/g, ' ').indexOf(' ' + selector + ' ') > -1;
  }
  function jqLiteRemoveClass(element, cssClasses) {
    if (cssClasses && element.setAttribute) {
      forEach(cssClasses.split(' '), function (cssClass) {
        element.setAttribute('class', trim((' ' + (element.getAttribute('class') || '') + ' ').replace(/[\n\t]/g, ' ').replace(' ' + trim(cssClass) + ' ', ' ')));
      });
    }
  }
  function jqLiteAddClass(element, cssClasses) {
    if (cssClasses && element.setAttribute) {
      var existingClasses = (' ' + (element.getAttribute('class') || '') + ' ').replace(/[\n\t]/g, ' ');
      forEach(cssClasses.split(' '), function (cssClass) {
        cssClass = trim(cssClass);
        if (existingClasses.indexOf(' ' + cssClass + ' ') === -1) {
          existingClasses += cssClass + ' ';
        }
      });
      element.setAttribute('class', trim(existingClasses));
    }
  }
  function jqLiteAddNodes(root, elements) {
    // THIS CODE IS VERY HOT. Don't make changes without benchmarking.
    if (elements) {
      // if a Node (the most common case)
      if (elements.nodeType) {
        root[root.length++] = elements;
      } else {
        var length = elements.length;
        // if an Array or NodeList and not a Window
        if (typeof length === 'number' && elements.window !== elements) {
          if (length) {
            for (var i = 0; i < length; i++) {
              root[root.length++] = elements[i];
            }
          }
        } else {
          root[root.length++] = elements;
        }
      }
    }
  }
  function jqLiteController(element, name) {
    return jqLiteInheritedData(element, '$' + (name || 'ngController') + 'Controller');
  }
  function jqLiteInheritedData(element, name, value) {
    // if element is the document object work with the html element instead
    // this makes $(document).scope() possible
    if (element.nodeType == NODE_TYPE_DOCUMENT) {
      element = element.documentElement;
    }
    var names = isArray(name) ? name : [name];
    while (element) {
      for (var i = 0, ii = names.length; i < ii; i++) {
        if ((value = jqLite.data(element, names[i])) !== undefined)
          return value;
      }
      // If dealing with a document fragment node with a host element, and no parent, use the host
      // element as the parent. This enables directives within a Shadow DOM or polyfilled Shadow DOM
      // to lookup parent controllers.
      element = element.parentNode || element.nodeType === NODE_TYPE_DOCUMENT_FRAGMENT && element.host;
    }
  }
  function jqLiteEmpty(element) {
    jqLiteDealoc(element, true);
    while (element.firstChild) {
      element.removeChild(element.firstChild);
    }
  }
  function jqLiteRemove(element, keepData) {
    if (!keepData)
      jqLiteDealoc(element);
    var parent = element.parentNode;
    if (parent)
      parent.removeChild(element);
  }
  function jqLiteDocumentLoaded(action, win) {
    win = win || window;
    if (win.document.readyState === 'complete') {
      // Force the action to be run async for consistent behaviour
      // from the action's point of view
      // i.e. it will definitely not be in a $apply
      win.setTimeout(action);
    } else {
      // No need to unbind this handler as load is only ever called once
      jqLite(win).on('load', action);
    }
  }
  //////////////////////////////////////////
  // Functions which are declared directly.
  //////////////////////////////////////////
  var JQLitePrototype = JQLite.prototype = {
      ready: function (fn) {
        var fired = false;
        function trigger() {
          if (fired)
            return;
          fired = true;
          fn();
        }
        // check if document is already loaded
        if (document.readyState === 'complete') {
          setTimeout(trigger);
        } else {
          this.on('DOMContentLoaded', trigger);
          // works for modern browsers and IE9
          // we can not use jqLite since we are not done loading and jQuery could be loaded later.
          // jshint -W064
          JQLite(window).on('load', trigger);  // fallback to window.onload for others
                                               // jshint +W064
        }
      },
      toString: function () {
        var value = [];
        forEach(this, function (e) {
          value.push('' + e);
        });
        return '[' + value.join(', ') + ']';
      },
      eq: function (index) {
        return index >= 0 ? jqLite(this[index]) : jqLite(this[this.length + index]);
      },
      length: 0,
      push: push,
      sort: [].sort,
      splice: [].splice
    };
  //////////////////////////////////////////
  // Functions iterating getter/setters.
  // these functions return self on setter and
  // value on get.
  //////////////////////////////////////////
  var BOOLEAN_ATTR = {};
  forEach('multiple,selected,checked,disabled,readOnly,required,open'.split(','), function (value) {
    BOOLEAN_ATTR[lowercase(value)] = value;
  });
  var BOOLEAN_ELEMENTS = {};
  forEach('input,select,option,textarea,button,form,details'.split(','), function (value) {
    BOOLEAN_ELEMENTS[value] = true;
  });
  var ALIASED_ATTR = {
      'ngMinlength': 'minlength',
      'ngMaxlength': 'maxlength',
      'ngMin': 'min',
      'ngMax': 'max',
      'ngPattern': 'pattern'
    };
  function getBooleanAttrName(element, name) {
    // check dom last since we will most likely fail on name
    var booleanAttr = BOOLEAN_ATTR[name.toLowerCase()];
    // booleanAttr is here twice to minimize DOM access
    return booleanAttr && BOOLEAN_ELEMENTS[nodeName_(element)] && booleanAttr;
  }
  function getAliasedAttrName(element, name) {
    var nodeName = element.nodeName;
    return (nodeName === 'INPUT' || nodeName === 'TEXTAREA') && ALIASED_ATTR[name];
  }
  forEach({
    data: jqLiteData,
    removeData: jqLiteRemoveData
  }, function (fn, name) {
    JQLite[name] = fn;
  });
  forEach({
    data: jqLiteData,
    inheritedData: jqLiteInheritedData,
    scope: function (element) {
      // Can't use jqLiteData here directly so we stay compatible with jQuery!
      return jqLite.data(element, '$scope') || jqLiteInheritedData(element.parentNode || element, [
        '$isolateScope',
        '$scope'
      ]);
    },
    isolateScope: function (element) {
      // Can't use jqLiteData here directly so we stay compatible with jQuery!
      return jqLite.data(element, '$isolateScope') || jqLite.data(element, '$isolateScopeNoTemplate');
    },
    controller: jqLiteController,
    injector: function (element) {
      return jqLiteInheritedData(element, '$injector');
    },
    removeAttr: function (element, name) {
      element.removeAttribute(name);
    },
    hasClass: jqLiteHasClass,
    css: function (element, name, value) {
      name = camelCase(name);
      if (isDefined(value)) {
        element.style[name] = value;
      } else {
        return element.style[name];
      }
    },
    attr: function (element, name, value) {
      var nodeType = element.nodeType;
      if (nodeType === NODE_TYPE_TEXT || nodeType === NODE_TYPE_ATTRIBUTE || nodeType === NODE_TYPE_COMMENT) {
        return;
      }
      var lowercasedName = lowercase(name);
      if (BOOLEAN_ATTR[lowercasedName]) {
        if (isDefined(value)) {
          if (!!value) {
            element[name] = true;
            element.setAttribute(name, lowercasedName);
          } else {
            element[name] = false;
            element.removeAttribute(lowercasedName);
          }
        } else {
          return element[name] || (element.attributes.getNamedItem(name) || noop).specified ? lowercasedName : undefined;
        }
      } else if (isDefined(value)) {
        element.setAttribute(name, value);
      } else if (element.getAttribute) {
        // the extra argument "2" is to get the right thing for a.href in IE, see jQuery code
        // some elements (e.g. Document) don't have get attribute, so return undefined
        var ret = element.getAttribute(name, 2);
        // normalize non-existing attributes to undefined (as jQuery)
        return ret === null ? undefined : ret;
      }
    },
    prop: function (element, name, value) {
      if (isDefined(value)) {
        element[name] = value;
      } else {
        return element[name];
      }
    },
    text: function () {
      getText.$dv = '';
      return getText;
      function getText(element, value) {
        if (isUndefined(value)) {
          var nodeType = element.nodeType;
          return nodeType === NODE_TYPE_ELEMENT || nodeType === NODE_TYPE_TEXT ? element.textContent : '';
        }
        element.textContent = value;
      }
    }(),
    val: function (element, value) {
      if (isUndefined(value)) {
        if (element.multiple && nodeName_(element) === 'select') {
          var result = [];
          forEach(element.options, function (option) {
            if (option.selected) {
              result.push(option.value || option.text);
            }
          });
          return result.length === 0 ? null : result;
        }
        return element.value;
      }
      element.value = value;
    },
    html: function (element, value) {
      if (isUndefined(value)) {
        return element.innerHTML;
      }
      jqLiteDealoc(element, true);
      element.innerHTML = value;
    },
    empty: jqLiteEmpty
  }, function (fn, name) {
    /**
   * Properties: writes return selection, reads return first value
   */
    JQLite.prototype[name] = function (arg1, arg2) {
      var i, key;
      var nodeCount = this.length;
      // jqLiteHasClass has only two arguments, but is a getter-only fn, so we need to special-case it
      // in a way that survives minification.
      // jqLiteEmpty takes no arguments but is a setter.
      if (fn !== jqLiteEmpty && (fn.length == 2 && (fn !== jqLiteHasClass && fn !== jqLiteController) ? arg1 : arg2) === undefined) {
        if (isObject(arg1)) {
          // we are a write, but the object properties are the key/values
          for (i = 0; i < nodeCount; i++) {
            if (fn === jqLiteData) {
              // data() takes the whole object in jQuery
              fn(this[i], arg1);
            } else {
              for (key in arg1) {
                fn(this[i], key, arg1[key]);
              }
            }
          }
          // return self for chaining
          return this;
        } else {
          // we are a read, so read the first child.
          // TODO: do we still need this?
          var value = fn.$dv;
          // Only if we have $dv do we iterate over all, otherwise it is just the first element.
          var jj = value === undefined ? Math.min(nodeCount, 1) : nodeCount;
          for (var j = 0; j < jj; j++) {
            var nodeValue = fn(this[j], arg1, arg2);
            value = value ? value + nodeValue : nodeValue;
          }
          return value;
        }
      } else {
        // we are a write, so apply to all children
        for (i = 0; i < nodeCount; i++) {
          fn(this[i], arg1, arg2);
        }
        // return self for chaining
        return this;
      }
    };
  });
  function createEventHandler(element, events) {
    var eventHandler = function (event, type) {
      // jQuery specific api
      event.isDefaultPrevented = function () {
        return event.defaultPrevented;
      };
      var eventFns = events[type || event.type];
      var eventFnsLength = eventFns ? eventFns.length : 0;
      if (!eventFnsLength)
        return;
      if (isUndefined(event.immediatePropagationStopped)) {
        var originalStopImmediatePropagation = event.stopImmediatePropagation;
        event.stopImmediatePropagation = function () {
          event.immediatePropagationStopped = true;
          if (event.stopPropagation) {
            event.stopPropagation();
          }
          if (originalStopImmediatePropagation) {
            originalStopImmediatePropagation.call(event);
          }
        };
      }
      event.isImmediatePropagationStopped = function () {
        return event.immediatePropagationStopped === true;
      };
      // Copy event handlers in case event handlers array is modified during execution.
      if (eventFnsLength > 1) {
        eventFns = shallowCopy(eventFns);
      }
      for (var i = 0; i < eventFnsLength; i++) {
        if (!event.isImmediatePropagationStopped()) {
          eventFns[i].call(element, event);
        }
      }
    };
    // TODO: this is a hack for angularMocks/clearDataCache that makes it possible to deregister all
    //       events on `element`
    eventHandler.elem = element;
    return eventHandler;
  }
  //////////////////////////////////////////
  // Functions iterating traversal.
  // These functions chain results into a single
  // selector.
  //////////////////////////////////////////
  forEach({
    removeData: jqLiteRemoveData,
    on: function jqLiteOn(element, type, fn, unsupported) {
      if (isDefined(unsupported))
        throw jqLiteMinErr('onargs', 'jqLite#on() does not support the `selector` or `eventData` parameters');
      // Do not add event handlers to non-elements because they will not be cleaned up.
      if (!jqLiteAcceptsData(element)) {
        return;
      }
      var expandoStore = jqLiteExpandoStore(element, true);
      var events = expandoStore.events;
      var handle = expandoStore.handle;
      if (!handle) {
        handle = expandoStore.handle = createEventHandler(element, events);
      }
      // http://jsperf.com/string-indexof-vs-split
      var types = type.indexOf(' ') >= 0 ? type.split(' ') : [type];
      var i = types.length;
      while (i--) {
        type = types[i];
        var eventFns = events[type];
        if (!eventFns) {
          events[type] = [];
          if (type === 'mouseenter' || type === 'mouseleave') {
            // Refer to jQuery's implementation of mouseenter & mouseleave
            // Read about mouseenter and mouseleave:
            // http://www.quirksmode.org/js/events_mouse.html#link8
            jqLiteOn(element, MOUSE_EVENT_MAP[type], function (event) {
              var target = this, related = event.relatedTarget;
              // For mousenter/leave call the handler if related is outside the target.
              // NB: No relatedTarget if the mouse left/entered the browser window
              if (!related || related !== target && !target.contains(related)) {
                handle(event, type);
              }
            });
          } else {
            if (type !== '$destroy') {
              addEventListenerFn(element, type, handle);
            }
          }
          eventFns = events[type];
        }
        eventFns.push(fn);
      }
    },
    off: jqLiteOff,
    one: function (element, type, fn) {
      element = jqLite(element);
      //add the listener twice so that when it is called
      //you can remove the original function and still be
      //able to call element.off(ev, fn) normally
      element.on(type, function onFn() {
        element.off(type, fn);
        element.off(type, onFn);
      });
      element.on(type, fn);
    },
    replaceWith: function (element, replaceNode) {
      var index, parent = element.parentNode;
      jqLiteDealoc(element);
      forEach(new JQLite(replaceNode), function (node) {
        if (index) {
          parent.insertBefore(node, index.nextSibling);
        } else {
          parent.replaceChild(node, element);
        }
        index = node;
      });
    },
    children: function (element) {
      var children = [];
      forEach(element.childNodes, function (element) {
        if (element.nodeType === NODE_TYPE_ELEMENT)
          children.push(element);
      });
      return children;
    },
    contents: function (element) {
      return element.contentDocument || element.childNodes || [];
    },
    append: function (element, node) {
      var nodeType = element.nodeType;
      if (nodeType !== NODE_TYPE_ELEMENT && nodeType !== NODE_TYPE_DOCUMENT_FRAGMENT)
        return;
      node = new JQLite(node);
      for (var i = 0, ii = node.length; i < ii; i++) {
        var child = node[i];
        element.appendChild(child);
      }
    },
    prepend: function (element, node) {
      if (element.nodeType === NODE_TYPE_ELEMENT) {
        var index = element.firstChild;
        forEach(new JQLite(node), function (child) {
          element.insertBefore(child, index);
        });
      }
    },
    wrap: function (element, wrapNode) {
      wrapNode = jqLite(wrapNode).eq(0).clone()[0];
      var parent = element.parentNode;
      if (parent) {
        parent.replaceChild(wrapNode, element);
      }
      wrapNode.appendChild(element);
    },
    remove: jqLiteRemove,
    detach: function (element) {
      jqLiteRemove(element, true);
    },
    after: function (element, newElement) {
      var index = element, parent = element.parentNode;
      newElement = new JQLite(newElement);
      for (var i = 0, ii = newElement.length; i < ii; i++) {
        var node = newElement[i];
        parent.insertBefore(node, index.nextSibling);
        index = node;
      }
    },
    addClass: jqLiteAddClass,
    removeClass: jqLiteRemoveClass,
    toggleClass: function (element, selector, condition) {
      if (selector) {
        forEach(selector.split(' '), function (className) {
          var classCondition = condition;
          if (isUndefined(classCondition)) {
            classCondition = !jqLiteHasClass(element, className);
          }
          (classCondition ? jqLiteAddClass : jqLiteRemoveClass)(element, className);
        });
      }
    },
    parent: function (element) {
      var parent = element.parentNode;
      return parent && parent.nodeType !== NODE_TYPE_DOCUMENT_FRAGMENT ? parent : null;
    },
    next: function (element) {
      return element.nextElementSibling;
    },
    find: function (element, selector) {
      if (element.getElementsByTagName) {
        return element.getElementsByTagName(selector);
      } else {
        return [];
      }
    },
    clone: jqLiteClone,
    triggerHandler: function (element, event, extraParameters) {
      var dummyEvent, eventFnsCopy, handlerArgs;
      var eventName = event.type || event;
      var expandoStore = jqLiteExpandoStore(element);
      var events = expandoStore && expandoStore.events;
      var eventFns = events && events[eventName];
      if (eventFns) {
        // Create a dummy event to pass to the handlers
        dummyEvent = {
          preventDefault: function () {
            this.defaultPrevented = true;
          },
          isDefaultPrevented: function () {
            return this.defaultPrevented === true;
          },
          stopImmediatePropagation: function () {
            this.immediatePropagationStopped = true;
          },
          isImmediatePropagationStopped: function () {
            return this.immediatePropagationStopped === true;
          },
          stopPropagation: noop,
          type: eventName,
          target: element
        };
        // If a custom event was provided then extend our dummy event with it
        if (event.type) {
          dummyEvent = extend(dummyEvent, event);
        }
        // Copy event handlers in case event handlers array is modified during execution.
        eventFnsCopy = shallowCopy(eventFns);
        handlerArgs = extraParameters ? [dummyEvent].concat(extraParameters) : [dummyEvent];
        forEach(eventFnsCopy, function (fn) {
          if (!dummyEvent.isImmediatePropagationStopped()) {
            fn.apply(element, handlerArgs);
          }
        });
      }
    }
  }, function (fn, name) {
    /**
   * chaining functions
   */
    JQLite.prototype[name] = function (arg1, arg2, arg3) {
      var value;
      for (var i = 0, ii = this.length; i < ii; i++) {
        if (isUndefined(value)) {
          value = fn(this[i], arg1, arg2, arg3);
          if (isDefined(value)) {
            // any function which returns a value needs to be wrapped
            value = jqLite(value);
          }
        } else {
          jqLiteAddNodes(value, fn(this[i], arg1, arg2, arg3));
        }
      }
      return isDefined(value) ? value : this;
    };
    // bind legacy bind/unbind to on/off
    JQLite.prototype.bind = JQLite.prototype.on;
    JQLite.prototype.unbind = JQLite.prototype.off;
  });
  // Provider for private $$jqLite service
  function $$jqLiteProvider() {
    this.$get = function $$jqLite() {
      return extend(JQLite, {
        hasClass: function (node, classes) {
          if (node.attr)
            node = node[0];
          return jqLiteHasClass(node, classes);
        },
        addClass: function (node, classes) {
          if (node.attr)
            node = node[0];
          return jqLiteAddClass(node, classes);
        },
        removeClass: function (node, classes) {
          if (node.attr)
            node = node[0];
          return jqLiteRemoveClass(node, classes);
        }
      });
    };
  }
  /**
 * Computes a hash of an 'obj'.
 * Hash of a:
 *  string is string
 *  number is number as string
 *  object is either result of calling $$hashKey function on the object or uniquely generated id,
 *         that is also assigned to the $$hashKey property of the object.
 *
 * @param obj
 * @returns {string} hash string such that the same input will have the same hash string.
 *         The resulting string key is in 'type:hashKey' format.
 */
  function hashKey(obj, nextUidFn) {
    var key = obj && obj.$$hashKey;
    if (key) {
      if (typeof key === 'function') {
        key = obj.$$hashKey();
      }
      return key;
    }
    var objType = typeof obj;
    if (objType == 'function' || objType == 'object' && obj !== null) {
      key = obj.$$hashKey = objType + ':' + (nextUidFn || nextUid)();
    } else {
      key = objType + ':' + obj;
    }
    return key;
  }
  /**
 * HashMap which can use objects as keys
 */
  function HashMap(array, isolatedUid) {
    if (isolatedUid) {
      var uid = 0;
      this.nextUid = function () {
        return ++uid;
      };
    }
    forEach(array, this.put, this);
  }
  HashMap.prototype = {
    put: function (key, value) {
      this[hashKey(key, this.nextUid)] = value;
    },
    get: function (key) {
      return this[hashKey(key, this.nextUid)];
    },
    remove: function (key) {
      var value = this[key = hashKey(key, this.nextUid)];
      delete this[key];
      return value;
    }
  };
  /**
 * @ngdoc function
 * @module ng
 * @name angular.injector
 * @kind function
 *
 * @description
 * Creates an injector object that can be used for retrieving services as well as for
 * dependency injection (see {@link guide/di dependency injection}).
 *
 * @param {Array.<string|Function>} modules A list of module functions or their aliases. See
 *     {@link angular.module}. The `ng` module must be explicitly added.
 * @param {boolean=} [strictDi=false] Whether the injector should be in strict mode, which
 *     disallows argument name annotation inference.
 * @returns {injector} Injector object. See {@link auto.$injector $injector}.
 *
 * @example
 * Typical usage
 * ```js
 *   // create an injector
 *   var $injector = angular.injector(['ng']);
 *
 *   // use the injector to kick off your application
 *   // use the type inference to auto inject arguments, or use implicit injection
 *   $injector.invoke(function($rootScope, $compile, $document) {
 *     $compile($document)($rootScope);
 *     $rootScope.$digest();
 *   });
 * ```
 *
 * Sometimes you want to get access to the injector of a currently running Angular app
 * from outside Angular. Perhaps, you want to inject and compile some markup after the
 * application has been bootstrapped. You can do this using the extra `injector()` added
 * to JQuery/jqLite elements. See {@link angular.element}.
 *
 * *This is fairly rare but could be the case if a third party library is injecting the
 * markup.*
 *
 * In the following example a new block of HTML containing a `ng-controller`
 * directive is added to the end of the document body by JQuery. We then compile and link
 * it into the current AngularJS scope.
 *
 * ```js
 * var $div = $('<div ng-controller="MyCtrl">{{content.label}}</div>');
 * $(document.body).append($div);
 *
 * angular.element(document).injector().invoke(function($compile) {
 *   var scope = angular.element($div).scope();
 *   $compile($div)(scope);
 * });
 * ```
 */
  /**
 * @ngdoc module
 * @name auto
 * @description
 *
 * Implicit module which gets automatically added to each {@link auto.$injector $injector}.
 */
  var FN_ARGS = /^function\s*[^\(]*\(\s*([^\)]*)\)/m;
  var FN_ARG_SPLIT = /,/;
  var FN_ARG = /^\s*(_?)(\S+?)\1\s*$/;
  var STRIP_COMMENTS = /((\/\/.*$)|(\/\*[\s\S]*?\*\/))/gm;
  var $injectorMinErr = minErr('$injector');
  function anonFn(fn) {
    // For anonymous functions, showing at the very least the function signature can help in
    // debugging.
    var fnText = fn.toString().replace(STRIP_COMMENTS, ''), args = fnText.match(FN_ARGS);
    if (args) {
      return 'function(' + (args[1] || '').replace(/[\s\r\n]+/, ' ') + ')';
    }
    return 'fn';
  }
  function annotate(fn, strictDi, name) {
    var $inject, fnText, argDecl, last;
    if (typeof fn === 'function') {
      if (!($inject = fn.$inject)) {
        $inject = [];
        if (fn.length) {
          if (strictDi) {
            if (!isString(name) || !name) {
              name = fn.name || anonFn(fn);
            }
            throw $injectorMinErr('strictdi', '{0} is not using explicit annotation and cannot be invoked in strict mode', name);
          }
          fnText = fn.toString().replace(STRIP_COMMENTS, '');
          argDecl = fnText.match(FN_ARGS);
          forEach(argDecl[1].split(FN_ARG_SPLIT), function (arg) {
            arg.replace(FN_ARG, function (all, underscore, name) {
              $inject.push(name);
            });
          });
        }
        fn.$inject = $inject;
      }
    } else if (isArray(fn)) {
      last = fn.length - 1;
      assertArgFn(fn[last], 'fn');
      $inject = fn.slice(0, last);
    } else {
      assertArgFn(fn, 'fn', true);
    }
    return $inject;
  }
  ///////////////////////////////////////
  /**
 * @ngdoc service
 * @name $injector
 *
 * @description
 *
 * `$injector` is used to retrieve object instances as defined by
 * {@link auto.$provide provider}, instantiate types, invoke methods,
 * and load modules.
 *
 * The following always holds true:
 *
 * ```js
 *   var $injector = angular.injector();
 *   expect($injector.get('$injector')).toBe($injector);
 *   expect($injector.invoke(function($injector) {
 *     return $injector;
 *   })).toBe($injector);
 * ```
 *
 * # Injection Function Annotation
 *
 * JavaScript does not have annotations, and annotations are needed for dependency injection. The
 * following are all valid ways of annotating function with injection arguments and are equivalent.
 *
 * ```js
 *   // inferred (only works if code not minified/obfuscated)
 *   $injector.invoke(function(serviceA){});
 *
 *   // annotated
 *   function explicit(serviceA) {};
 *   explicit.$inject = ['serviceA'];
 *   $injector.invoke(explicit);
 *
 *   // inline
 *   $injector.invoke(['serviceA', function(serviceA){}]);
 * ```
 *
 * ## Inference
 *
 * In JavaScript calling `toString()` on a function returns the function definition. The definition
 * can then be parsed and the function arguments can be extracted. This method of discovering
 * annotations is disallowed when the injector is in strict mode.
 * *NOTE:* This does not work with minification, and obfuscation tools since these tools change the
 * argument names.
 *
 * ## `$inject` Annotation
 * By adding an `$inject` property onto a function the injection parameters can be specified.
 *
 * ## Inline
 * As an array of injection names, where the last item in the array is the function to call.
 */
  /**
 * @ngdoc method
 * @name $injector#get
 *
 * @description
 * Return an instance of the service.
 *
 * @param {string} name The name of the instance to retrieve.
 * @param {string=} caller An optional string to provide the origin of the function call for error messages.
 * @return {*} The instance.
 */
  /**
 * @ngdoc method
 * @name $injector#invoke
 *
 * @description
 * Invoke the method and supply the method arguments from the `$injector`.
 *
 * @param {Function|Array.<string|Function>} fn The injectable function to invoke. Function parameters are
 *   injected according to the {@link guide/di $inject Annotation} rules.
 * @param {Object=} self The `this` for the invoked method.
 * @param {Object=} locals Optional object. If preset then any argument names are read from this
 *                         object first, before the `$injector` is consulted.
 * @returns {*} the value returned by the invoked `fn` function.
 */
  /**
 * @ngdoc method
 * @name $injector#has
 *
 * @description
 * Allows the user to query if the particular service exists.
 *
 * @param {string} name Name of the service to query.
 * @returns {boolean} `true` if injector has given service.
 */
  /**
 * @ngdoc method
 * @name $injector#instantiate
 * @description
 * Create a new instance of JS type. The method takes a constructor function, invokes the new
 * operator, and supplies all of the arguments to the constructor function as specified by the
 * constructor annotation.
 *
 * @param {Function} Type Annotated constructor function.
 * @param {Object=} locals Optional object. If preset then any argument names are read from this
 * object first, before the `$injector` is consulted.
 * @returns {Object} new instance of `Type`.
 */
  /**
 * @ngdoc method
 * @name $injector#annotate
 *
 * @description
 * Returns an array of service names which the function is requesting for injection. This API is
 * used by the injector to determine which services need to be injected into the function when the
 * function is invoked. There are three ways in which the function can be annotated with the needed
 * dependencies.
 *
 * # Argument names
 *
 * The simplest form is to extract the dependencies from the arguments of the function. This is done
 * by converting the function into a string using `toString()` method and extracting the argument
 * names.
 * ```js
 *   // Given
 *   function MyController($scope, $route) {
 *     // ...
 *   }
 *
 *   // Then
 *   expect(injector.annotate(MyController)).toEqual(['$scope', '$route']);
 * ```
 *
 * You can disallow this method by using strict injection mode.
 *
 * This method does not work with code minification / obfuscation. For this reason the following
 * annotation strategies are supported.
 *
 * # The `$inject` property
 *
 * If a function has an `$inject` property and its value is an array of strings, then the strings
 * represent names of services to be injected into the function.
 * ```js
 *   // Given
 *   var MyController = function(obfuscatedScope, obfuscatedRoute) {
 *     // ...
 *   }
 *   // Define function dependencies
 *   MyController['$inject'] = ['$scope', '$route'];
 *
 *   // Then
 *   expect(injector.annotate(MyController)).toEqual(['$scope', '$route']);
 * ```
 *
 * # The array notation
 *
 * It is often desirable to inline Injected functions and that's when setting the `$inject` property
 * is very inconvenient. In these situations using the array notation to specify the dependencies in
 * a way that survives minification is a better choice:
 *
 * ```js
 *   // We wish to write this (not minification / obfuscation safe)
 *   injector.invoke(function($compile, $rootScope) {
 *     // ...
 *   });
 *
 *   // We are forced to write break inlining
 *   var tmpFn = function(obfuscatedCompile, obfuscatedRootScope) {
 *     // ...
 *   };
 *   tmpFn.$inject = ['$compile', '$rootScope'];
 *   injector.invoke(tmpFn);
 *
 *   // To better support inline function the inline annotation is supported
 *   injector.invoke(['$compile', '$rootScope', function(obfCompile, obfRootScope) {
 *     // ...
 *   }]);
 *
 *   // Therefore
 *   expect(injector.annotate(
 *      ['$compile', '$rootScope', function(obfus_$compile, obfus_$rootScope) {}])
 *    ).toEqual(['$compile', '$rootScope']);
 * ```
 *
 * @param {Function|Array.<string|Function>} fn Function for which dependent service names need to
 * be retrieved as described above.
 *
 * @param {boolean=} [strictDi=false] Disallow argument name annotation inference.
 *
 * @returns {Array.<string>} The names of the services which the function requires.
 */
  /**
 * @ngdoc service
 * @name $provide
 *
 * @description
 *
 * The {@link auto.$provide $provide} service has a number of methods for registering components
 * with the {@link auto.$injector $injector}. Many of these functions are also exposed on
 * {@link angular.Module}.
 *
 * An Angular **service** is a singleton object created by a **service factory**.  These **service
 * factories** are functions which, in turn, are created by a **service provider**.
 * The **service providers** are constructor functions. When instantiated they must contain a
 * property called `$get`, which holds the **service factory** function.
 *
 * When you request a service, the {@link auto.$injector $injector} is responsible for finding the
 * correct **service provider**, instantiating it and then calling its `$get` **service factory**
 * function to get the instance of the **service**.
 *
 * Often services have no configuration options and there is no need to add methods to the service
 * provider.  The provider will be no more than a constructor function with a `$get` property. For
 * these cases the {@link auto.$provide $provide} service has additional helper methods to register
 * services without specifying a provider.
 *
 * * {@link auto.$provide#provider provider(provider)} - registers a **service provider** with the
 *     {@link auto.$injector $injector}
 * * {@link auto.$provide#constant constant(obj)} - registers a value/object that can be accessed by
 *     providers and services.
 * * {@link auto.$provide#value value(obj)} - registers a value/object that can only be accessed by
 *     services, not providers.
 * * {@link auto.$provide#factory factory(fn)} - registers a service **factory function**, `fn`,
 *     that will be wrapped in a **service provider** object, whose `$get` property will contain the
 *     given factory function.
 * * {@link auto.$provide#service service(class)} - registers a **constructor function**, `class`
 *     that will be wrapped in a **service provider** object, whose `$get` property will instantiate
 *      a new object using the given constructor function.
 *
 * See the individual methods for more information and examples.
 */
  /**
 * @ngdoc method
 * @name $provide#provider
 * @description
 *
 * Register a **provider function** with the {@link auto.$injector $injector}. Provider functions
 * are constructor functions, whose instances are responsible for "providing" a factory for a
 * service.
 *
 * Service provider names start with the name of the service they provide followed by `Provider`.
 * For example, the {@link ng.$log $log} service has a provider called
 * {@link ng.$logProvider $logProvider}.
 *
 * Service provider objects can have additional methods which allow configuration of the provider
 * and its service. Importantly, you can configure what kind of service is created by the `$get`
 * method, or how that service will act. For example, the {@link ng.$logProvider $logProvider} has a
 * method {@link ng.$logProvider#debugEnabled debugEnabled}
 * which lets you specify whether the {@link ng.$log $log} service will log debug messages to the
 * console or not.
 *
 * @param {string} name The name of the instance. NOTE: the provider will be available under `name +
                        'Provider'` key.
 * @param {(Object|function())} provider If the provider is:
 *
 *   - `Object`: then it should have a `$get` method. The `$get` method will be invoked using
 *     {@link auto.$injector#invoke $injector.invoke()} when an instance needs to be created.
 *   - `Constructor`: a new instance of the provider will be created using
 *     {@link auto.$injector#instantiate $injector.instantiate()}, then treated as `object`.
 *
 * @returns {Object} registered provider instance

 * @example
 *
 * The following example shows how to create a simple event tracking service and register it using
 * {@link auto.$provide#provider $provide.provider()}.
 *
 * ```js
 *  // Define the eventTracker provider
 *  function EventTrackerProvider() {
 *    var trackingUrl = '/track';
 *
 *    // A provider method for configuring where the tracked events should been saved
 *    this.setTrackingUrl = function(url) {
 *      trackingUrl = url;
 *    };
 *
 *    // The service factory function
 *    this.$get = ['$http', function($http) {
 *      var trackedEvents = {};
 *      return {
 *        // Call this to track an event
 *        event: function(event) {
 *          var count = trackedEvents[event] || 0;
 *          count += 1;
 *          trackedEvents[event] = count;
 *          return count;
 *        },
 *        // Call this to save the tracked events to the trackingUrl
 *        save: function() {
 *          $http.post(trackingUrl, trackedEvents);
 *        }
 *      };
 *    }];
 *  }
 *
 *  describe('eventTracker', function() {
 *    var postSpy;
 *
 *    beforeEach(module(function($provide) {
 *      // Register the eventTracker provider
 *      $provide.provider('eventTracker', EventTrackerProvider);
 *    }));
 *
 *    beforeEach(module(function(eventTrackerProvider) {
 *      // Configure eventTracker provider
 *      eventTrackerProvider.setTrackingUrl('/custom-track');
 *    }));
 *
 *    it('tracks events', inject(function(eventTracker) {
 *      expect(eventTracker.event('login')).toEqual(1);
 *      expect(eventTracker.event('login')).toEqual(2);
 *    }));
 *
 *    it('saves to the tracking url', inject(function(eventTracker, $http) {
 *      postSpy = spyOn($http, 'post');
 *      eventTracker.event('login');
 *      eventTracker.save();
 *      expect(postSpy).toHaveBeenCalled();
 *      expect(postSpy.mostRecentCall.args[0]).not.toEqual('/track');
 *      expect(postSpy.mostRecentCall.args[0]).toEqual('/custom-track');
 *      expect(postSpy.mostRecentCall.args[1]).toEqual({ 'login': 1 });
 *    }));
 *  });
 * ```
 */
  /**
 * @ngdoc method
 * @name $provide#factory
 * @description
 *
 * Register a **service factory**, which will be called to return the service instance.
 * This is short for registering a service where its provider consists of only a `$get` property,
 * which is the given service factory function.
 * You should use {@link auto.$provide#factory $provide.factory(getFn)} if you do not need to
 * configure your service in a provider.
 *
 * @param {string} name The name of the instance.
 * @param {Function|Array.<string|Function>} $getFn The injectable $getFn for the instance creation.
 *                      Internally this is a short hand for `$provide.provider(name, {$get: $getFn})`.
 * @returns {Object} registered provider instance
 *
 * @example
 * Here is an example of registering a service
 * ```js
 *   $provide.factory('ping', ['$http', function($http) {
 *     return function ping() {
 *       return $http.send('/ping');
 *     };
 *   }]);
 * ```
 * You would then inject and use this service like this:
 * ```js
 *   someModule.controller('Ctrl', ['ping', function(ping) {
 *     ping();
 *   }]);
 * ```
 */
  /**
 * @ngdoc method
 * @name $provide#service
 * @description
 *
 * Register a **service constructor**, which will be invoked with `new` to create the service
 * instance.
 * This is short for registering a service where its provider's `$get` property is the service
 * constructor function that will be used to instantiate the service instance.
 *
 * You should use {@link auto.$provide#service $provide.service(class)} if you define your service
 * as a type/class.
 *
 * @param {string} name The name of the instance.
 * @param {Function|Array.<string|Function>} constructor An injectable class (constructor function)
 *     that will be instantiated.
 * @returns {Object} registered provider instance
 *
 * @example
 * Here is an example of registering a service using
 * {@link auto.$provide#service $provide.service(class)}.
 * ```js
 *   var Ping = function($http) {
 *     this.$http = $http;
 *   };
 *
 *   Ping.$inject = ['$http'];
 *
 *   Ping.prototype.send = function() {
 *     return this.$http.get('/ping');
 *   };
 *   $provide.service('ping', Ping);
 * ```
 * You would then inject and use this service like this:
 * ```js
 *   someModule.controller('Ctrl', ['ping', function(ping) {
 *     ping.send();
 *   }]);
 * ```
 */
  /**
 * @ngdoc method
 * @name $provide#value
 * @description
 *
 * Register a **value service** with the {@link auto.$injector $injector}, such as a string, a
 * number, an array, an object or a function.  This is short for registering a service where its
 * provider's `$get` property is a factory function that takes no arguments and returns the **value
 * service**.
 *
 * Value services are similar to constant services, except that they cannot be injected into a
 * module configuration function (see {@link angular.Module#config}) but they can be overridden by
 * an Angular
 * {@link auto.$provide#decorator decorator}.
 *
 * @param {string} name The name of the instance.
 * @param {*} value The value.
 * @returns {Object} registered provider instance
 *
 * @example
 * Here are some examples of creating value services.
 * ```js
 *   $provide.value('ADMIN_USER', 'admin');
 *
 *   $provide.value('RoleLookup', { admin: 0, writer: 1, reader: 2 });
 *
 *   $provide.value('halfOf', function(value) {
 *     return value / 2;
 *   });
 * ```
 */
  /**
 * @ngdoc method
 * @name $provide#constant
 * @description
 *
 * Register a **constant service**, such as a string, a number, an array, an object or a function,
 * with the {@link auto.$injector $injector}. Unlike {@link auto.$provide#value value} it can be
 * injected into a module configuration function (see {@link angular.Module#config}) and it cannot
 * be overridden by an Angular {@link auto.$provide#decorator decorator}.
 *
 * @param {string} name The name of the constant.
 * @param {*} value The constant value.
 * @returns {Object} registered instance
 *
 * @example
 * Here a some examples of creating constants:
 * ```js
 *   $provide.constant('SHARD_HEIGHT', 306);
 *
 *   $provide.constant('MY_COLOURS', ['red', 'blue', 'grey']);
 *
 *   $provide.constant('double', function(value) {
 *     return value * 2;
 *   });
 * ```
 */
  /**
 * @ngdoc method
 * @name $provide#decorator
 * @description
 *
 * Register a **service decorator** with the {@link auto.$injector $injector}. A service decorator
 * intercepts the creation of a service, allowing it to override or modify the behaviour of the
 * service. The object returned by the decorator may be the original service, or a new service
 * object which replaces or wraps and delegates to the original service.
 *
 * @param {string} name The name of the service to decorate.
 * @param {Function|Array.<string|Function>} decorator This function will be invoked when the service needs to be
 *    instantiated and should return the decorated service instance. The function is called using
 *    the {@link auto.$injector#invoke injector.invoke} method and is therefore fully injectable.
 *    Local injection arguments:
 *
 *    * `$delegate` - The original service instance, which can be monkey patched, configured,
 *      decorated or delegated to.
 *
 * @example
 * Here we decorate the {@link ng.$log $log} service to convert warnings to errors by intercepting
 * calls to {@link ng.$log#error $log.warn()}.
 * ```js
 *   $provide.decorator('$log', ['$delegate', function($delegate) {
 *     $delegate.warn = $delegate.error;
 *     return $delegate;
 *   }]);
 * ```
 */
  function createInjector(modulesToLoad, strictDi) {
    strictDi = strictDi === true;
    var INSTANTIATING = {}, providerSuffix = 'Provider', path = [], loadedModules = new HashMap([], true), providerCache = {
        $provide: {
          provider: supportObject(provider),
          factory: supportObject(factory),
          service: supportObject(service),
          value: supportObject(value),
          constant: supportObject(constant),
          decorator: decorator
        }
      }, providerInjector = providerCache.$injector = createInternalInjector(providerCache, function (serviceName, caller) {
        if (angular.isString(caller)) {
          path.push(caller);
        }
        throw $injectorMinErr('unpr', 'Unknown provider: {0}', path.join(' <- '));
      }), instanceCache = {}, instanceInjector = instanceCache.$injector = createInternalInjector(instanceCache, function (serviceName, caller) {
        var provider = providerInjector.get(serviceName + providerSuffix, caller);
        return instanceInjector.invoke(provider.$get, provider, undefined, serviceName);
      });
    forEach(loadModules(modulesToLoad), function (fn) {
      instanceInjector.invoke(fn || noop);
    });
    return instanceInjector;
    ////////////////////////////////////
    // $provider
    ////////////////////////////////////
    function supportObject(delegate) {
      return function (key, value) {
        if (isObject(key)) {
          forEach(key, reverseParams(delegate));
        } else {
          return delegate(key, value);
        }
      };
    }
    function provider(name, provider_) {
      assertNotHasOwnProperty(name, 'service');
      if (isFunction(provider_) || isArray(provider_)) {
        provider_ = providerInjector.instantiate(provider_);
      }
      if (!provider_.$get) {
        throw $injectorMinErr('pget', 'Provider \'{0}\' must define $get factory method.', name);
      }
      return providerCache[name + providerSuffix] = provider_;
    }
    function enforceReturnValue(name, factory) {
      return function enforcedReturnValue() {
        var result = instanceInjector.invoke(factory, this);
        if (isUndefined(result)) {
          throw $injectorMinErr('undef', 'Provider \'{0}\' must return a value from $get factory method.', name);
        }
        return result;
      };
    }
    function factory(name, factoryFn, enforce) {
      return provider(name, { $get: enforce !== false ? enforceReturnValue(name, factoryFn) : factoryFn });
    }
    function service(name, constructor) {
      return factory(name, [
        '$injector',
        function ($injector) {
          return $injector.instantiate(constructor);
        }
      ]);
    }
    function value(name, val) {
      return factory(name, valueFn(val), false);
    }
    function constant(name, value) {
      assertNotHasOwnProperty(name, 'constant');
      providerCache[name] = value;
      instanceCache[name] = value;
    }
    function decorator(serviceName, decorFn) {
      var origProvider = providerInjector.get(serviceName + providerSuffix), orig$get = origProvider.$get;
      origProvider.$get = function () {
        var origInstance = instanceInjector.invoke(orig$get, origProvider);
        return instanceInjector.invoke(decorFn, null, { $delegate: origInstance });
      };
    }
    ////////////////////////////////////
    // Module Loading
    ////////////////////////////////////
    function loadModules(modulesToLoad) {
      var runBlocks = [], moduleFn;
      forEach(modulesToLoad, function (module) {
        if (loadedModules.get(module))
          return;
        loadedModules.put(module, true);
        function runInvokeQueue(queue) {
          var i, ii;
          for (i = 0, ii = queue.length; i < ii; i++) {
            var invokeArgs = queue[i], provider = providerInjector.get(invokeArgs[0]);
            provider[invokeArgs[1]].apply(provider, invokeArgs[2]);
          }
        }
        try {
          if (isString(module)) {
            moduleFn = angularModule(module);
            runBlocks = runBlocks.concat(loadModules(moduleFn.requires)).concat(moduleFn._runBlocks);
            runInvokeQueue(moduleFn._invokeQueue);
            runInvokeQueue(moduleFn._configBlocks);
          } else if (isFunction(module)) {
            runBlocks.push(providerInjector.invoke(module));
          } else if (isArray(module)) {
            runBlocks.push(providerInjector.invoke(module));
          } else {
            assertArgFn(module, 'module');
          }
        } catch (e) {
          if (isArray(module)) {
            module = module[module.length - 1];
          }
          if (e.message && e.stack && e.stack.indexOf(e.message) == -1) {
            // Safari & FF's stack traces don't contain error.message content
            // unlike those of Chrome and IE
            // So if stack doesn't contain message, we create a new string that contains both.
            // Since error.stack is read-only in Safari, I'm overriding e and not e.stack here.
            /* jshint -W022 */
            e = e.message + '\n' + e.stack;
          }
          throw $injectorMinErr('modulerr', 'Failed to instantiate module {0} due to:\n{1}', module, e.stack || e.message || e);
        }
      });
      return runBlocks;
    }
    ////////////////////////////////////
    // internal Injector
    ////////////////////////////////////
    function createInternalInjector(cache, factory) {
      function getService(serviceName, caller) {
        if (cache.hasOwnProperty(serviceName)) {
          if (cache[serviceName] === INSTANTIATING) {
            throw $injectorMinErr('cdep', 'Circular dependency found: {0}', serviceName + ' <- ' + path.join(' <- '));
          }
          return cache[serviceName];
        } else {
          try {
            path.unshift(serviceName);
            cache[serviceName] = INSTANTIATING;
            return cache[serviceName] = factory(serviceName, caller);
          } catch (err) {
            if (cache[serviceName] === INSTANTIATING) {
              delete cache[serviceName];
            }
            throw err;
          } finally {
            path.shift();
          }
        }
      }
      function invoke(fn, self, locals, serviceName) {
        if (typeof locals === 'string') {
          serviceName = locals;
          locals = null;
        }
        var args = [], $inject = createInjector.$$annotate(fn, strictDi, serviceName), length, i, key;
        for (i = 0, length = $inject.length; i < length; i++) {
          key = $inject[i];
          if (typeof key !== 'string') {
            throw $injectorMinErr('itkn', 'Incorrect injection token! Expected service name as string, got {0}', key);
          }
          args.push(locals && locals.hasOwnProperty(key) ? locals[key] : getService(key, serviceName));
        }
        if (isArray(fn)) {
          fn = fn[length];
        }
        // http://jsperf.com/angularjs-invoke-apply-vs-switch
        // #5388
        return fn.apply(self, args);
      }
      function instantiate(Type, locals, serviceName) {
        // Check if Type is annotated and use just the given function at n-1 as parameter
        // e.g. someModule.factory('greeter', ['$window', function(renamed$window) {}]);
        // Object creation: http://jsperf.com/create-constructor/2
        var instance = Object.create((isArray(Type) ? Type[Type.length - 1] : Type).prototype || null);
        var returnedValue = invoke(Type, instance, locals, serviceName);
        return isObject(returnedValue) || isFunction(returnedValue) ? returnedValue : instance;
      }
      return {
        invoke: invoke,
        instantiate: instantiate,
        get: getService,
        annotate: createInjector.$$annotate,
        has: function (name) {
          return providerCache.hasOwnProperty(name + providerSuffix) || cache.hasOwnProperty(name);
        }
      };
    }
  }
  createInjector.$$annotate = annotate;
  /**
 * @ngdoc provider
 * @name $anchorScrollProvider
 *
 * @description
 * Use `$anchorScrollProvider` to disable automatic scrolling whenever
 * {@link ng.$location#hash $location.hash()} changes.
 */
  function $AnchorScrollProvider() {
    var autoScrollingEnabled = true;
    /**
   * @ngdoc method
   * @name $anchorScrollProvider#disableAutoScrolling
   *
   * @description
   * By default, {@link ng.$anchorScroll $anchorScroll()} will automatically detect changes to
   * {@link ng.$location#hash $location.hash()} and scroll to the element matching the new hash.<br />
   * Use this method to disable automatic scrolling.
   *
   * If automatic scrolling is disabled, one must explicitly call
   * {@link ng.$anchorScroll $anchorScroll()} in order to scroll to the element related to the
   * current hash.
   */
    this.disableAutoScrolling = function () {
      autoScrollingEnabled = false;
    };
    /**
   * @ngdoc service
   * @name $anchorScroll
   * @kind function
   * @requires $window
   * @requires $location
   * @requires $rootScope
   *
   * @description
   * When called, it checks the current value of {@link ng.$location#hash $location.hash()} and
   * scrolls to the related element, according to the rules specified in the
   * [Html5 spec](http://dev.w3.org/html5/spec/Overview.html#the-indicated-part-of-the-document).
   *
   * It also watches the {@link ng.$location#hash $location.hash()} and automatically scrolls to
   * match any anchor whenever it changes. This can be disabled by calling
   * {@link ng.$anchorScrollProvider#disableAutoScrolling $anchorScrollProvider.disableAutoScrolling()}.
   *
   * Additionally, you can use its {@link ng.$anchorScroll#yOffset yOffset} property to specify a
   * vertical scroll-offset (either fixed or dynamic).
   *
   * @property {(number|function|jqLite)} yOffset
   * If set, specifies a vertical scroll-offset. This is often useful when there are fixed
   * positioned elements at the top of the page, such as navbars, headers etc.
   *
   * `yOffset` can be specified in various ways:
   * - **number**: A fixed number of pixels to be used as offset.<br /><br />
   * - **function**: A getter function called everytime `$anchorScroll()` is executed. Must return
   *   a number representing the offset (in pixels).<br /><br />
   * - **jqLite**: A jqLite/jQuery element to be used for specifying the offset. The distance from
   *   the top of the page to the element's bottom will be used as offset.<br />
   *   **Note**: The element will be taken into account only as long as its `position` is set to
   *   `fixed`. This option is useful, when dealing with responsive navbars/headers that adjust
   *   their height and/or positioning according to the viewport's size.
   *
   * <br />
   * <div class="alert alert-warning">
   * In order for `yOffset` to work properly, scrolling should take place on the document's root and
   * not some child element.
   * </div>
   *
   * @example
     <example module="anchorScrollExample">
       <file name="index.html">
         <div id="scrollArea" ng-controller="ScrollController">
           <a ng-click="gotoBottom()">Go to bottom</a>
           <a id="bottom"></a> You're at the bottom!
         </div>
       </file>
       <file name="script.js">
         angular.module('anchorScrollExample', [])
           .controller('ScrollController', ['$scope', '$location', '$anchorScroll',
             function ($scope, $location, $anchorScroll) {
               $scope.gotoBottom = function() {
                 // set the location.hash to the id of
                 // the element you wish to scroll to.
                 $location.hash('bottom');

                 // call $anchorScroll()
                 $anchorScroll();
               };
             }]);
       </file>
       <file name="style.css">
         #scrollArea {
           height: 280px;
           overflow: auto;
         }

         #bottom {
           display: block;
           margin-top: 2000px;
         }
       </file>
     </example>
   *
   * <hr />
   * The example below illustrates the use of a vertical scroll-offset (specified as a fixed value).
   * See {@link ng.$anchorScroll#yOffset $anchorScroll.yOffset} for more details.
   *
   * @example
     <example module="anchorScrollOffsetExample">
       <file name="index.html">
         <div class="fixed-header" ng-controller="headerCtrl">
           <a href="" ng-click="gotoAnchor(x)" ng-repeat="x in [1,2,3,4,5]">
             Go to anchor {{x}}
           </a>
         </div>
         <div id="anchor{{x}}" class="anchor" ng-repeat="x in [1,2,3,4,5]">
           Anchor {{x}} of 5
         </div>
       </file>
       <file name="script.js">
         angular.module('anchorScrollOffsetExample', [])
           .run(['$anchorScroll', function($anchorScroll) {
             $anchorScroll.yOffset = 50;   // always scroll by 50 extra pixels
           }])
           .controller('headerCtrl', ['$anchorScroll', '$location', '$scope',
             function ($anchorScroll, $location, $scope) {
               $scope.gotoAnchor = function(x) {
                 var newHash = 'anchor' + x;
                 if ($location.hash() !== newHash) {
                   // set the $location.hash to `newHash` and
                   // $anchorScroll will automatically scroll to it
                   $location.hash('anchor' + x);
                 } else {
                   // call $anchorScroll() explicitly,
                   // since $location.hash hasn't changed
                   $anchorScroll();
                 }
               };
             }
           ]);
       </file>
       <file name="style.css">
         body {
           padding-top: 50px;
         }

         .anchor {
           border: 2px dashed DarkOrchid;
           padding: 10px 10px 200px 10px;
         }

         .fixed-header {
           background-color: rgba(0, 0, 0, 0.2);
           height: 50px;
           position: fixed;
           top: 0; left: 0; right: 0;
         }

         .fixed-header > a {
           display: inline-block;
           margin: 5px 15px;
         }
       </file>
     </example>
   */
    this.$get = [
      '$window',
      '$location',
      '$rootScope',
      function ($window, $location, $rootScope) {
        var document = $window.document;
        // Helper function to get first anchor from a NodeList
        // (using `Array#some()` instead of `angular#forEach()` since it's more performant
        //  and working in all supported browsers.)
        function getFirstAnchor(list) {
          var result = null;
          Array.prototype.some.call(list, function (element) {
            if (nodeName_(element) === 'a') {
              result = element;
              return true;
            }
          });
          return result;
        }
        function getYOffset() {
          var offset = scroll.yOffset;
          if (isFunction(offset)) {
            offset = offset();
          } else if (isElement(offset)) {
            var elem = offset[0];
            var style = $window.getComputedStyle(elem);
            if (style.position !== 'fixed') {
              offset = 0;
            } else {
              offset = elem.getBoundingClientRect().bottom;
            }
          } else if (!isNumber(offset)) {
            offset = 0;
          }
          return offset;
        }
        function scrollTo(elem) {
          if (elem) {
            elem.scrollIntoView();
            var offset = getYOffset();
            if (offset) {
              // `offset` is the number of pixels we should scroll UP in order to align `elem` properly.
              // This is true ONLY if the call to `elem.scrollIntoView()` initially aligns `elem` at the
              // top of the viewport.
              //
              // IF the number of pixels from the top of `elem` to the end of the page's content is less
              // than the height of the viewport, then `elem.scrollIntoView()` will align the `elem` some
              // way down the page.
              //
              // This is often the case for elements near the bottom of the page.
              //
              // In such cases we do not need to scroll the whole `offset` up, just the difference between
              // the top of the element and the offset, which is enough to align the top of `elem` at the
              // desired position.
              var elemTop = elem.getBoundingClientRect().top;
              $window.scrollBy(0, elemTop - offset);
            }
          } else {
            $window.scrollTo(0, 0);
          }
        }
        function scroll() {
          var hash = $location.hash(), elm;
          // empty hash, scroll to the top of the page
          if (!hash)
            scrollTo(null);  // element with given id
          else if (elm = document.getElementById(hash))
            scrollTo(elm);  // first anchor with given name :-D
          else if (elm = getFirstAnchor(document.getElementsByName(hash)))
            scrollTo(elm);  // no element and hash == 'top', scroll to the top of the page
          else if (hash === 'top')
            scrollTo(null);
        }
        // does not scroll when user clicks on anchor link that is currently on
        // (no url change, no $location.hash() change), browser native does scroll
        if (autoScrollingEnabled) {
          $rootScope.$watch(function autoScrollWatch() {
            return $location.hash();
          }, function autoScrollWatchAction(newVal, oldVal) {
            // skip the initial scroll if $location.hash is empty
            if (newVal === oldVal && newVal === '')
              return;
            jqLiteDocumentLoaded(function () {
              $rootScope.$evalAsync(scroll);
            });
          });
        }
        return scroll;
      }
    ];
  }
  var $animateMinErr = minErr('$animate');
  /**
 * @ngdoc provider
 * @name $animateProvider
 *
 * @description
 * Default implementation of $animate that doesn't perform any animations, instead just
 * synchronously performs DOM
 * updates and calls done() callbacks.
 *
 * In order to enable animations the ngAnimate module has to be loaded.
 *
 * To see the functional implementation check out src/ngAnimate/animate.js
 */
  var $AnimateProvider = [
      '$provide',
      function ($provide) {
        this.$$selectors = {};
        /**
   * @ngdoc method
   * @name $animateProvider#register
   *
   * @description
   * Registers a new injectable animation factory function. The factory function produces the
   * animation object which contains callback functions for each event that is expected to be
   * animated.
   *
   *   * `eventFn`: `function(Element, doneFunction)` The element to animate, the `doneFunction`
   *   must be called once the element animation is complete. If a function is returned then the
   *   animation service will use this function to cancel the animation whenever a cancel event is
   *   triggered.
   *
   *
   * ```js
   *   return {
     *     eventFn : function(element, done) {
     *       //code to run the animation
     *       //once complete, then run done()
     *       return function cancellationFunction() {
     *         //code to cancel the animation
     *       }
     *     }
     *   }
   * ```
   *
   * @param {string} name The name of the animation.
   * @param {Function} factory The factory function that will be executed to return the animation
   *                           object.
   */
        this.register = function (name, factory) {
          var key = name + '-animation';
          if (name && name.charAt(0) != '.')
            throw $animateMinErr('notcsel', 'Expecting class selector starting with \'.\' got \'{0}\'.', name);
          this.$$selectors[name.substr(1)] = key;
          $provide.factory(key, factory);
        };
        /**
   * @ngdoc method
   * @name $animateProvider#classNameFilter
   *
   * @description
   * Sets and/or returns the CSS class regular expression that is checked when performing
   * an animation. Upon bootstrap the classNameFilter value is not set at all and will
   * therefore enable $animate to attempt to perform an animation on any element.
   * When setting the classNameFilter value, animations will only be performed on elements
   * that successfully match the filter expression. This in turn can boost performance
   * for low-powered devices as well as applications containing a lot of structural operations.
   * @param {RegExp=} expression The className expression which will be checked against all animations
   * @return {RegExp} The current CSS className expression value. If null then there is no expression value
   */
        this.classNameFilter = function (expression) {
          if (arguments.length === 1) {
            this.$$classNameFilter = expression instanceof RegExp ? expression : null;
          }
          return this.$$classNameFilter;
        };
        this.$get = [
          '$$q',
          '$$asyncCallback',
          '$rootScope',
          function ($$q, $$asyncCallback, $rootScope) {
            var currentDefer;
            function runAnimationPostDigest(fn) {
              var cancelFn, defer = $$q.defer();
              defer.promise.$$cancelFn = function ngAnimateMaybeCancel() {
                cancelFn && cancelFn();
              };
              $rootScope.$$postDigest(function ngAnimatePostDigest() {
                cancelFn = fn(function ngAnimateNotifyComplete() {
                  defer.resolve();
                });
              });
              return defer.promise;
            }
            function resolveElementClasses(element, classes) {
              var toAdd = [], toRemove = [];
              var hasClasses = createMap();
              forEach((element.attr('class') || '').split(/\s+/), function (className) {
                hasClasses[className] = true;
              });
              forEach(classes, function (status, className) {
                var hasClass = hasClasses[className];
                // If the most recent class manipulation (via $animate) was to remove the class, and the
                // element currently has the class, the class is scheduled for removal. Otherwise, if
                // the most recent class manipulation (via $animate) was to add the class, and the
                // element does not currently have the class, the class is scheduled to be added.
                if (status === false && hasClass) {
                  toRemove.push(className);
                } else if (status === true && !hasClass) {
                  toAdd.push(className);
                }
              });
              return toAdd.length + toRemove.length > 0 && [
                toAdd.length ? toAdd : null,
                toRemove.length ? toRemove : null
              ];
            }
            function cachedClassManipulation(cache, classes, op) {
              for (var i = 0, ii = classes.length; i < ii; ++i) {
                var className = classes[i];
                cache[className] = op;
              }
            }
            function asyncPromise() {
              // only serve one instance of a promise in order to save CPU cycles
              if (!currentDefer) {
                currentDefer = $$q.defer();
                $$asyncCallback(function () {
                  currentDefer.resolve();
                  currentDefer = null;
                });
              }
              return currentDefer.promise;
            }
            function applyStyles(element, options) {
              if (angular.isObject(options)) {
                var styles = extend(options.from || {}, options.to || {});
                element.css(styles);
              }
            }
            /**
     *
     * @ngdoc service
     * @name $animate
     * @description The $animate service provides rudimentary DOM manipulation functions to
     * insert, remove and move elements within the DOM, as well as adding and removing classes.
     * This service is the core service used by the ngAnimate $animator service which provides
     * high-level animation hooks for CSS and JavaScript.
     *
     * $animate is available in the AngularJS core, however, the ngAnimate module must be included
     * to enable full out animation support. Otherwise, $animate will only perform simple DOM
     * manipulation operations.
     *
     * To learn more about enabling animation support, click here to visit the {@link ngAnimate
     * ngAnimate module page} as well as the {@link ngAnimate.$animate ngAnimate $animate service
     * page}.
     */
            return {
              animate: function (element, from, to) {
                applyStyles(element, {
                  from: from,
                  to: to
                });
                return asyncPromise();
              },
              enter: function (element, parent, after, options) {
                applyStyles(element, options);
                after ? after.after(element) : parent.prepend(element);
                return asyncPromise();
              },
              leave: function (element, options) {
                applyStyles(element, options);
                element.remove();
                return asyncPromise();
              },
              move: function (element, parent, after, options) {
                // Do not remove element before insert. Removing will cause data associated with the
                // element to be dropped. Insert will implicitly do the remove.
                return this.enter(element, parent, after, options);
              },
              addClass: function (element, className, options) {
                return this.setClass(element, className, [], options);
              },
              $$addClassImmediately: function (element, className, options) {
                element = jqLite(element);
                className = !isString(className) ? isArray(className) ? className.join(' ') : '' : className;
                forEach(element, function (element) {
                  jqLiteAddClass(element, className);
                });
                applyStyles(element, options);
                return asyncPromise();
              },
              removeClass: function (element, className, options) {
                return this.setClass(element, [], className, options);
              },
              $$removeClassImmediately: function (element, className, options) {
                element = jqLite(element);
                className = !isString(className) ? isArray(className) ? className.join(' ') : '' : className;
                forEach(element, function (element) {
                  jqLiteRemoveClass(element, className);
                });
                applyStyles(element, options);
                return asyncPromise();
              },
              setClass: function (element, add, remove, options) {
                var self = this;
                var STORAGE_KEY = '$$animateClasses';
                var createdCache = false;
                element = jqLite(element);
                var cache = element.data(STORAGE_KEY);
                if (!cache) {
                  cache = {
                    classes: {},
                    options: options
                  };
                  createdCache = true;
                } else if (options && cache.options) {
                  cache.options = angular.extend(cache.options || {}, options);
                }
                var classes = cache.classes;
                add = isArray(add) ? add : add.split(' ');
                remove = isArray(remove) ? remove : remove.split(' ');
                cachedClassManipulation(classes, add, true);
                cachedClassManipulation(classes, remove, false);
                if (createdCache) {
                  cache.promise = runAnimationPostDigest(function (done) {
                    var cache = element.data(STORAGE_KEY);
                    element.removeData(STORAGE_KEY);
                    // in the event that the element is removed before postDigest
                    // is run then the cache will be undefined and there will be
                    // no need anymore to add or remove and of the element classes
                    if (cache) {
                      var classes = resolveElementClasses(element, cache.classes);
                      if (classes) {
                        self.$$setClassImmediately(element, classes[0], classes[1], cache.options);
                      }
                    }
                    done();
                  });
                  element.data(STORAGE_KEY, cache);
                }
                return cache.promise;
              },
              $$setClassImmediately: function (element, add, remove, options) {
                add && this.$$addClassImmediately(element, add);
                remove && this.$$removeClassImmediately(element, remove);
                applyStyles(element, options);
                return asyncPromise();
              },
              enabled: noop,
              cancel: noop
            };
          }
        ];
      }
    ];
  function $$AsyncCallbackProvider() {
    this.$get = [
      '$$rAF',
      '$timeout',
      function ($$rAF, $timeout) {
        return $$rAF.supported ? function (fn) {
          return $$rAF(fn);
        } : function (fn) {
          return $timeout(fn, 0, false);
        };
      }
    ];
  }
  /* global stripHash: true */
  /**
 * ! This is a private undocumented service !
 *
 * @name $browser
 * @requires $log
 * @description
 * This object has two goals:
 *
 * - hide all the global state in the browser caused by the window object
 * - abstract away all the browser specific features and inconsistencies
 *
 * For tests we provide {@link ngMock.$browser mock implementation} of the `$browser`
 * service, which can be used for convenient testing of the application without the interaction with
 * the real browser apis.
 */
  /**
 * @param {object} window The global window object.
 * @param {object} document jQuery wrapped document.
 * @param {object} $log window.console or an object with the same interface.
 * @param {object} $sniffer $sniffer service
 */
  function Browser(window, document, $log, $sniffer) {
    var self = this, rawDocument = document[0], location = window.location, history = window.history, setTimeout = window.setTimeout, clearTimeout = window.clearTimeout, pendingDeferIds = {};
    self.isMock = false;
    var outstandingRequestCount = 0;
    var outstandingRequestCallbacks = [];
    // TODO(vojta): remove this temporary api
    self.$$completeOutstandingRequest = completeOutstandingRequest;
    self.$$incOutstandingRequestCount = function () {
      outstandingRequestCount++;
    };
    /**
   * Executes the `fn` function(supports currying) and decrements the `outstandingRequestCallbacks`
   * counter. If the counter reaches 0, all the `outstandingRequestCallbacks` are executed.
   */
    function completeOutstandingRequest(fn) {
      try {
        fn.apply(null, sliceArgs(arguments, 1));
      } finally {
        outstandingRequestCount--;
        if (outstandingRequestCount === 0) {
          while (outstandingRequestCallbacks.length) {
            try {
              outstandingRequestCallbacks.pop()();
            } catch (e) {
              $log.error(e);
            }
          }
        }
      }
    }
    function getHash(url) {
      var index = url.indexOf('#');
      return index === -1 ? '' : url.substr(index);
    }
    /**
   * @private
   * Note: this method is used only by scenario runner
   * TODO(vojta): prefix this method with $$ ?
   * @param {function()} callback Function that will be called when no outstanding request
   */
    self.notifyWhenNoOutstandingRequests = function (callback) {
      // force browser to execute all pollFns - this is needed so that cookies and other pollers fire
      // at some deterministic time in respect to the test runner's actions. Leaving things up to the
      // regular poller would result in flaky tests.
      forEach(pollFns, function (pollFn) {
        pollFn();
      });
      if (outstandingRequestCount === 0) {
        callback();
      } else {
        outstandingRequestCallbacks.push(callback);
      }
    };
    //////////////////////////////////////////////////////////////
    // Poll Watcher API
    //////////////////////////////////////////////////////////////
    var pollFns = [], pollTimeout;
    /**
   * @name $browser#addPollFn
   *
   * @param {function()} fn Poll function to add
   *
   * @description
   * Adds a function to the list of functions that poller periodically executes,
   * and starts polling if not started yet.
   *
   * @returns {function()} the added function
   */
    self.addPollFn = function (fn) {
      if (isUndefined(pollTimeout))
        startPoller(100, setTimeout);
      pollFns.push(fn);
      return fn;
    };
    /**
   * @param {number} interval How often should browser call poll functions (ms)
   * @param {function()} setTimeout Reference to a real or fake `setTimeout` function.
   *
   * @description
   * Configures the poller to run in the specified intervals, using the specified
   * setTimeout fn and kicks it off.
   */
    function startPoller(interval, setTimeout) {
      (function check() {
        forEach(pollFns, function (pollFn) {
          pollFn();
        });
        pollTimeout = setTimeout(check, interval);
      }());
    }
    //////////////////////////////////////////////////////////////
    // URL API
    //////////////////////////////////////////////////////////////
    var cachedState, lastHistoryState, lastBrowserUrl = location.href, baseElement = document.find('base'), reloadLocation = null;
    cacheState();
    lastHistoryState = cachedState;
    /**
   * @name $browser#url
   *
   * @description
   * GETTER:
   * Without any argument, this method just returns current value of location.href.
   *
   * SETTER:
   * With at least one argument, this method sets url to new value.
   * If html5 history api supported, pushState/replaceState is used, otherwise
   * location.href/location.replace is used.
   * Returns its own instance to allow chaining
   *
   * NOTE: this api is intended for use only by the $location service. Please use the
   * {@link ng.$location $location service} to change url.
   *
   * @param {string} url New url (when used as setter)
   * @param {boolean=} replace Should new url replace current history record?
   * @param {object=} state object to use with pushState/replaceState
   */
    self.url = function (url, replace, state) {
      // In modern browsers `history.state` is `null` by default; treating it separately
      // from `undefined` would cause `$browser.url('/foo')` to change `history.state`
      // to undefined via `pushState`. Instead, let's change `undefined` to `null` here.
      if (isUndefined(state)) {
        state = null;
      }
      // Android Browser BFCache causes location, history reference to become stale.
      if (location !== window.location)
        location = window.location;
      if (history !== window.history)
        history = window.history;
      // setter
      if (url) {
        var sameState = lastHistoryState === state;
        // Don't change anything if previous and current URLs and states match. This also prevents
        // IE<10 from getting into redirect loop when in LocationHashbangInHtml5Url mode.
        // See https://github.com/angular/angular.js/commit/ffb2701
        if (lastBrowserUrl === url && (!$sniffer.history || sameState)) {
          return self;
        }
        var sameBase = lastBrowserUrl && stripHash(lastBrowserUrl) === stripHash(url);
        lastBrowserUrl = url;
        lastHistoryState = state;
        // Don't use history API if only the hash changed
        // due to a bug in IE10/IE11 which leads
        // to not firing a `hashchange` nor `popstate` event
        // in some cases (see #9143).
        if ($sniffer.history && (!sameBase || !sameState)) {
          history[replace ? 'replaceState' : 'pushState'](state, '', url);
          cacheState();
          // Do the assignment again so that those two variables are referentially identical.
          lastHistoryState = cachedState;
        } else {
          if (!sameBase || reloadLocation) {
            reloadLocation = url;
          }
          if (replace) {
            location.replace(url);
          } else if (!sameBase) {
            location.href = url;
          } else {
            location.hash = getHash(url);
          }
        }
        return self;  // getter
      } else {
        // - reloadLocation is needed as browsers don't allow to read out
        //   the new location.href if a reload happened.
        // - the replacement is a workaround for https://bugzilla.mozilla.org/show_bug.cgi?id=407172
        return reloadLocation || location.href.replace(/%27/g, '\'');
      }
    };
    /**
   * @name $browser#state
   *
   * @description
   * This method is a getter.
   *
   * Return history.state or null if history.state is undefined.
   *
   * @returns {object} state
   */
    self.state = function () {
      return cachedState;
    };
    var urlChangeListeners = [], urlChangeInit = false;
    function cacheStateAndFireUrlChange() {
      cacheState();
      fireUrlChange();
    }
    function getCurrentState() {
      try {
        return history.state;
      } catch (e) {
      }
    }
    // This variable should be used *only* inside the cacheState function.
    var lastCachedState = null;
    function cacheState() {
      // This should be the only place in $browser where `history.state` is read.
      cachedState = getCurrentState();
      cachedState = isUndefined(cachedState) ? null : cachedState;
      // Prevent callbacks fo fire twice if both hashchange & popstate were fired.
      if (equals(cachedState, lastCachedState)) {
        cachedState = lastCachedState;
      }
      lastCachedState = cachedState;
    }
    function fireUrlChange() {
      if (lastBrowserUrl === self.url() && lastHistoryState === cachedState) {
        return;
      }
      lastBrowserUrl = self.url();
      lastHistoryState = cachedState;
      forEach(urlChangeListeners, function (listener) {
        listener(self.url(), cachedState);
      });
    }
    /**
   * @name $browser#onUrlChange
   *
   * @description
   * Register callback function that will be called, when url changes.
   *
   * It's only called when the url is changed from outside of angular:
   * - user types different url into address bar
   * - user clicks on history (forward/back) button
   * - user clicks on a link
   *
   * It's not called when url is changed by $browser.url() method
   *
   * The listener gets called with new url as parameter.
   *
   * NOTE: this api is intended for use only by the $location service. Please use the
   * {@link ng.$location $location service} to monitor url changes in angular apps.
   *
   * @param {function(string)} listener Listener function to be called when url changes.
   * @return {function(string)} Returns the registered listener fn - handy if the fn is anonymous.
   */
    self.onUrlChange = function (callback) {
      // TODO(vojta): refactor to use node's syntax for events
      if (!urlChangeInit) {
        // We listen on both (hashchange/popstate) when available, as some browsers (e.g. Opera)
        // don't fire popstate when user change the address bar and don't fire hashchange when url
        // changed by push/replaceState
        // html5 history api - popstate event
        if ($sniffer.history)
          jqLite(window).on('popstate', cacheStateAndFireUrlChange);
        // hashchange event
        jqLite(window).on('hashchange', cacheStateAndFireUrlChange);
        urlChangeInit = true;
      }
      urlChangeListeners.push(callback);
      return callback;
    };
    /**
   * Checks whether the url has changed outside of Angular.
   * Needs to be exported to be able to check for changes that have been done in sync,
   * as hashchange/popstate events fire in async.
   */
    self.$$checkUrlChange = fireUrlChange;
    //////////////////////////////////////////////////////////////
    // Misc API
    //////////////////////////////////////////////////////////////
    /**
   * @name $browser#baseHref
   *
   * @description
   * Returns current <base href>
   * (always relative - without domain)
   *
   * @returns {string} The current base href
   */
    self.baseHref = function () {
      var href = baseElement.attr('href');
      return href ? href.replace(/^(https?\:)?\/\/[^\/]*/, '') : '';
    };
    //////////////////////////////////////////////////////////////
    // Cookies API
    //////////////////////////////////////////////////////////////
    var lastCookies = {};
    var lastCookieString = '';
    var cookiePath = self.baseHref();
    function safeDecodeURIComponent(str) {
      try {
        return decodeURIComponent(str);
      } catch (e) {
        return str;
      }
    }
    /**
   * @name $browser#cookies
   *
   * @param {string=} name Cookie name
   * @param {string=} value Cookie value
   *
   * @description
   * The cookies method provides a 'private' low level access to browser cookies.
   * It is not meant to be used directly, use the $cookie service instead.
   *
   * The return values vary depending on the arguments that the method was called with as follows:
   *
   * - cookies() -> hash of all cookies, this is NOT a copy of the internal state, so do not modify
   *   it
   * - cookies(name, value) -> set name to value, if value is undefined delete the cookie
   * - cookies(name) -> the same as (name, undefined) == DELETES (no one calls it right now that
   *   way)
   *
   * @returns {Object} Hash of all cookies (if called without any parameter)
   */
    self.cookies = function (name, value) {
      var cookieLength, cookieArray, cookie, i, index;
      if (name) {
        if (value === undefined) {
          rawDocument.cookie = encodeURIComponent(name) + '=;path=' + cookiePath + ';expires=Thu, 01 Jan 1970 00:00:00 GMT';
        } else {
          if (isString(value)) {
            cookieLength = (rawDocument.cookie = encodeURIComponent(name) + '=' + encodeURIComponent(value) + ';path=' + cookiePath).length + 1;
            // per http://www.ietf.org/rfc/rfc2109.txt browser must allow at minimum:
            // - 300 cookies
            // - 20 cookies per unique domain
            // - 4096 bytes per cookie
            if (cookieLength > 4096) {
              $log.warn('Cookie \'' + name + '\' possibly not set or overflowed because it was too large (' + cookieLength + ' > 4096 bytes)!');
            }
          }
        }
      } else {
        if (rawDocument.cookie !== lastCookieString) {
          lastCookieString = rawDocument.cookie;
          cookieArray = lastCookieString.split('; ');
          lastCookies = {};
          for (i = 0; i < cookieArray.length; i++) {
            cookie = cookieArray[i];
            index = cookie.indexOf('=');
            if (index > 0) {
              //ignore nameless cookies
              name = safeDecodeURIComponent(cookie.substring(0, index));
              // the first value that is seen for a cookie is the most
              // specific one.  values for the same cookie name that
              // follow are for less specific paths.
              if (lastCookies[name] === undefined) {
                lastCookies[name] = safeDecodeURIComponent(cookie.substring(index + 1));
              }
            }
          }
        }
        return lastCookies;
      }
    };
    /**
   * @name $browser#defer
   * @param {function()} fn A function, who's execution should be deferred.
   * @param {number=} [delay=0] of milliseconds to defer the function execution.
   * @returns {*} DeferId that can be used to cancel the task via `$browser.defer.cancel()`.
   *
   * @description
   * Executes a fn asynchronously via `setTimeout(fn, delay)`.
   *
   * Unlike when calling `setTimeout` directly, in test this function is mocked and instead of using
   * `setTimeout` in tests, the fns are queued in an array, which can be programmatically flushed
   * via `$browser.defer.flush()`.
   *
   */
    self.defer = function (fn, delay) {
      var timeoutId;
      outstandingRequestCount++;
      timeoutId = setTimeout(function () {
        delete pendingDeferIds[timeoutId];
        completeOutstandingRequest(fn);
      }, delay || 0);
      pendingDeferIds[timeoutId] = true;
      return timeoutId;
    };
    /**
   * @name $browser#defer.cancel
   *
   * @description
   * Cancels a deferred task identified with `deferId`.
   *
   * @param {*} deferId Token returned by the `$browser.defer` function.
   * @returns {boolean} Returns `true` if the task hasn't executed yet and was successfully
   *                    canceled.
   */
    self.defer.cancel = function (deferId) {
      if (pendingDeferIds[deferId]) {
        delete pendingDeferIds[deferId];
        clearTimeout(deferId);
        completeOutstandingRequest(noop);
        return true;
      }
      return false;
    };
  }
  function $BrowserProvider() {
    this.$get = [
      '$window',
      '$log',
      '$sniffer',
      '$document',
      function ($window, $log, $sniffer, $document) {
        return new Browser($window, $document, $log, $sniffer);
      }
    ];
  }
  /**
 * @ngdoc service
 * @name $cacheFactory
 *
 * @description
 * Factory that constructs {@link $cacheFactory.Cache Cache} objects and gives access to
 * them.
 *
 * ```js
 *
 *  var cache = $cacheFactory('cacheId');
 *  expect($cacheFactory.get('cacheId')).toBe(cache);
 *  expect($cacheFactory.get('noSuchCacheId')).not.toBeDefined();
 *
 *  cache.put("key", "value");
 *  cache.put("another key", "another value");
 *
 *  // We've specified no options on creation
 *  expect(cache.info()).toEqual({id: 'cacheId', size: 2});
 *
 * ```
 *
 *
 * @param {string} cacheId Name or id of the newly created cache.
 * @param {object=} options Options object that specifies the cache behavior. Properties:
 *
 *   - `{number=}` `capacity` — turns the cache into LRU cache.
 *
 * @returns {object} Newly created cache object with the following set of methods:
 *
 * - `{object}` `info()` — Returns id, size, and options of cache.
 * - `{{*}}` `put({string} key, {*} value)` — Puts a new key-value pair into the cache and returns
 *   it.
 * - `{{*}}` `get({string} key)` — Returns cached value for `key` or undefined for cache miss.
 * - `{void}` `remove({string} key)` — Removes a key-value pair from the cache.
 * - `{void}` `removeAll()` — Removes all cached values.
 * - `{void}` `destroy()` — Removes references to this cache from $cacheFactory.
 *
 * @example
   <example module="cacheExampleApp">
     <file name="index.html">
       <div ng-controller="CacheController">
         <input ng-model="newCacheKey" placeholder="Key">
         <input ng-model="newCacheValue" placeholder="Value">
         <button ng-click="put(newCacheKey, newCacheValue)">Cache</button>

         <p ng-if="keys.length">Cached Values</p>
         <div ng-repeat="key in keys">
           <span ng-bind="key"></span>
           <span>: </span>
           <b ng-bind="cache.get(key)"></b>
         </div>

         <p>Cache Info</p>
         <div ng-repeat="(key, value) in cache.info()">
           <span ng-bind="key"></span>
           <span>: </span>
           <b ng-bind="value"></b>
         </div>
       </div>
     </file>
     <file name="script.js">
       angular.module('cacheExampleApp', []).
         controller('CacheController', ['$scope', '$cacheFactory', function($scope, $cacheFactory) {
           $scope.keys = [];
           $scope.cache = $cacheFactory('cacheId');
           $scope.put = function(key, value) {
             if ($scope.cache.get(key) === undefined) {
               $scope.keys.push(key);
             }
             $scope.cache.put(key, value === undefined ? null : value);
           };
         }]);
     </file>
     <file name="style.css">
       p {
         margin: 10px 0 3px;
       }
     </file>
   </example>
 */
  function $CacheFactoryProvider() {
    this.$get = function () {
      var caches = {};
      function cacheFactory(cacheId, options) {
        if (cacheId in caches) {
          throw minErr('$cacheFactory')('iid', 'CacheId \'{0}\' is already taken!', cacheId);
        }
        var size = 0, stats = extend({}, options, { id: cacheId }), data = {}, capacity = options && options.capacity || Number.MAX_VALUE, lruHash = {}, freshEnd = null, staleEnd = null;
        /**
       * @ngdoc type
       * @name $cacheFactory.Cache
       *
       * @description
       * A cache object used to store and retrieve data, primarily used by
       * {@link $http $http} and the {@link ng.directive:script script} directive to cache
       * templates and other data.
       *
       * ```js
       *  angular.module('superCache')
       *    .factory('superCache', ['$cacheFactory', function($cacheFactory) {
       *      return $cacheFactory('super-cache');
       *    }]);
       * ```
       *
       * Example test:
       *
       * ```js
       *  it('should behave like a cache', inject(function(superCache) {
       *    superCache.put('key', 'value');
       *    superCache.put('another key', 'another value');
       *
       *    expect(superCache.info()).toEqual({
       *      id: 'super-cache',
       *      size: 2
       *    });
       *
       *    superCache.remove('another key');
       *    expect(superCache.get('another key')).toBeUndefined();
       *
       *    superCache.removeAll();
       *    expect(superCache.info()).toEqual({
       *      id: 'super-cache',
       *      size: 0
       *    });
       *  }));
       * ```
       */
        return caches[cacheId] = {
          put: function (key, value) {
            if (capacity < Number.MAX_VALUE) {
              var lruEntry = lruHash[key] || (lruHash[key] = { key: key });
              refresh(lruEntry);
            }
            if (isUndefined(value))
              return;
            if (!(key in data))
              size++;
            data[key] = value;
            if (size > capacity) {
              this.remove(staleEnd.key);
            }
            return value;
          },
          get: function (key) {
            if (capacity < Number.MAX_VALUE) {
              var lruEntry = lruHash[key];
              if (!lruEntry)
                return;
              refresh(lruEntry);
            }
            return data[key];
          },
          remove: function (key) {
            if (capacity < Number.MAX_VALUE) {
              var lruEntry = lruHash[key];
              if (!lruEntry)
                return;
              if (lruEntry == freshEnd)
                freshEnd = lruEntry.p;
              if (lruEntry == staleEnd)
                staleEnd = lruEntry.n;
              link(lruEntry.n, lruEntry.p);
              delete lruHash[key];
            }
            delete data[key];
            size--;
          },
          removeAll: function () {
            data = {};
            size = 0;
            lruHash = {};
            freshEnd = staleEnd = null;
          },
          destroy: function () {
            data = null;
            stats = null;
            lruHash = null;
            delete caches[cacheId];
          },
          info: function () {
            return extend({}, stats, { size: size });
          }
        };
        /**
       * makes the `entry` the freshEnd of the LRU linked list
       */
        function refresh(entry) {
          if (entry != freshEnd) {
            if (!staleEnd) {
              staleEnd = entry;
            } else if (staleEnd == entry) {
              staleEnd = entry.n;
            }
            link(entry.n, entry.p);
            link(entry, freshEnd);
            freshEnd = entry;
            freshEnd.n = null;
          }
        }
        /**
       * bidirectionally links two entries of the LRU linked list
       */
        function link(nextEntry, prevEntry) {
          if (nextEntry != prevEntry) {
            if (nextEntry)
              nextEntry.p = prevEntry;
            //p stands for previous, 'prev' didn't minify
            if (prevEntry)
              prevEntry.n = nextEntry;  //n stands for next, 'next' didn't minify
          }
        }
      }
      /**
   * @ngdoc method
   * @name $cacheFactory#info
   *
   * @description
   * Get information about all the caches that have been created
   *
   * @returns {Object} - key-value map of `cacheId` to the result of calling `cache#info`
   */
      cacheFactory.info = function () {
        var info = {};
        forEach(caches, function (cache, cacheId) {
          info[cacheId] = cache.info();
        });
        return info;
      };
      /**
   * @ngdoc method
   * @name $cacheFactory#get
   *
   * @description
   * Get access to a cache object by the `cacheId` used when it was created.
   *
   * @param {string} cacheId Name or id of a cache to access.
   * @returns {object} Cache object identified by the cacheId or undefined if no such cache.
   */
      cacheFactory.get = function (cacheId) {
        return caches[cacheId];
      };
      return cacheFactory;
    };
  }
  /**
 * @ngdoc service
 * @name $templateCache
 *
 * @description
 * The first time a template is used, it is loaded in the template cache for quick retrieval. You
 * can load templates directly into the cache in a `script` tag, or by consuming the
 * `$templateCache` service directly.
 *
 * Adding via the `script` tag:
 *
 * ```html
 *   <script type="text/ng-template" id="templateId.html">
 *     <p>This is the content of the template</p>
 *   </script>
 * ```
 *
 * **Note:** the `script` tag containing the template does not need to be included in the `head` of
 * the document, but it must be a descendent of the {@link ng.$rootElement $rootElement} (IE,
 * element with ng-app attribute), otherwise the template will be ignored.
 *
 * Adding via the `$templateCache` service:
 *
 * ```js
 * var myApp = angular.module('myApp', []);
 * myApp.run(function($templateCache) {
 *   $templateCache.put('templateId.html', 'This is the content of the template');
 * });
 * ```
 *
 * To retrieve the template later, simply use it in your HTML:
 * ```html
 * <div ng-include=" 'templateId.html' "></div>
 * ```
 *
 * or get it via Javascript:
 * ```js
 * $templateCache.get('templateId.html')
 * ```
 *
 * See {@link ng.$cacheFactory $cacheFactory}.
 *
 */
  function $TemplateCacheProvider() {
    this.$get = [
      '$cacheFactory',
      function ($cacheFactory) {
        return $cacheFactory('templates');
      }
    ];
  }
  /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *     Any commits to this file should be reviewed with security in mind.  *
 *   Changes to this file can potentially create security vulnerabilities. *
 *          An approval from 2 Core members with history of modifying      *
 *                         this file is required.                          *
 *                                                                         *
 *  Does the change somehow allow for arbitrary javascript to be executed? *
 *    Or allows for someone to change the prototype of built-in objects?   *
 *     Or gives undesired access to variables likes document or window?    *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
  /* ! VARIABLE/FUNCTION NAMING CONVENTIONS THAT APPLY TO THIS FILE!
 *
 * DOM-related variables:
 *
 * - "node" - DOM Node
 * - "element" - DOM Element or Node
 * - "$node" or "$element" - jqLite-wrapped node or element
 *
 *
 * Compiler related stuff:
 *
 * - "linkFn" - linking fn of a single directive
 * - "nodeLinkFn" - function that aggregates all linking fns for a particular node
 * - "childLinkFn" -  function that aggregates all linking fns for child nodes of a particular node
 * - "compositeLinkFn" - function that aggregates all linking fns for a compilation root (nodeList)
 */
  /**
 * @ngdoc service
 * @name $compile
 * @kind function
 *
 * @description
 * Compiles an HTML string or DOM into a template and produces a template function, which
 * can then be used to link {@link ng.$rootScope.Scope `scope`} and the template together.
 *
 * The compilation is a process of walking the DOM tree and matching DOM elements to
 * {@link ng.$compileProvider#directive directives}.
 *
 * <div class="alert alert-warning">
 * **Note:** This document is an in-depth reference of all directive options.
 * For a gentle introduction to directives with examples of common use cases,
 * see the {@link guide/directive directive guide}.
 * </div>
 *
 * ## Comprehensive Directive API
 *
 * There are many different options for a directive.
 *
 * The difference resides in the return value of the factory function.
 * You can either return a "Directive Definition Object" (see below) that defines the directive properties,
 * or just the `postLink` function (all other properties will have the default values).
 *
 * <div class="alert alert-success">
 * **Best Practice:** It's recommended to use the "directive definition object" form.
 * </div>
 *
 * Here's an example directive declared with a Directive Definition Object:
 *
 * ```js
 *   var myModule = angular.module(...);
 *
 *   myModule.directive('directiveName', function factory(injectables) {
 *     var directiveDefinitionObject = {
 *       priority: 0,
 *       template: '<div></div>', // or // function(tElement, tAttrs) { ... },
 *       // or
 *       // templateUrl: 'directive.html', // or // function(tElement, tAttrs) { ... },
 *       transclude: false,
 *       restrict: 'A',
 *       templateNamespace: 'html',
 *       scope: false,
 *       controller: function($scope, $element, $attrs, $transclude, otherInjectables) { ... },
 *       controllerAs: 'stringIdentifier',
 *       bindToController: false,
 *       require: 'siblingDirectiveName', // or // ['^parentDirectiveName', '?optionalDirectiveName', '?^optionalParent'],
 *       compile: function compile(tElement, tAttrs, transclude) {
 *         return {
 *           pre: function preLink(scope, iElement, iAttrs, controller) { ... },
 *           post: function postLink(scope, iElement, iAttrs, controller) { ... }
 *         }
 *         // or
 *         // return function postLink( ... ) { ... }
 *       },
 *       // or
 *       // link: {
 *       //  pre: function preLink(scope, iElement, iAttrs, controller) { ... },
 *       //  post: function postLink(scope, iElement, iAttrs, controller) { ... }
 *       // }
 *       // or
 *       // link: function postLink( ... ) { ... }
 *     };
 *     return directiveDefinitionObject;
 *   });
 * ```
 *
 * <div class="alert alert-warning">
 * **Note:** Any unspecified options will use the default value. You can see the default values below.
 * </div>
 *
 * Therefore the above can be simplified as:
 *
 * ```js
 *   var myModule = angular.module(...);
 *
 *   myModule.directive('directiveName', function factory(injectables) {
 *     var directiveDefinitionObject = {
 *       link: function postLink(scope, iElement, iAttrs) { ... }
 *     };
 *     return directiveDefinitionObject;
 *     // or
 *     // return function postLink(scope, iElement, iAttrs) { ... }
 *   });
 * ```
 *
 *
 *
 * ### Directive Definition Object
 *
 * The directive definition object provides instructions to the {@link ng.$compile
 * compiler}. The attributes are:
 *
 * #### `multiElement`
 * When this property is set to true, the HTML compiler will collect DOM nodes between
 * nodes with the attributes `directive-name-start` and `directive-name-end`, and group them
 * together as the directive elements. It is recommended that this feature be used on directives
 * which are not strictly behavioural (such as {@link ngClick}), and which
 * do not manipulate or replace child nodes (such as {@link ngInclude}).
 *
 * #### `priority`
 * When there are multiple directives defined on a single DOM element, sometimes it
 * is necessary to specify the order in which the directives are applied. The `priority` is used
 * to sort the directives before their `compile` functions get called. Priority is defined as a
 * number. Directives with greater numerical `priority` are compiled first. Pre-link functions
 * are also run in priority order, but post-link functions are run in reverse order. The order
 * of directives with the same priority is undefined. The default priority is `0`.
 *
 * #### `terminal`
 * If set to true then the current `priority` will be the last set of directives
 * which will execute (any directives at the current priority will still execute
 * as the order of execution on same `priority` is undefined). Note that expressions
 * and other directives used in the directive's template will also be excluded from execution.
 *
 * #### `scope`
 * **If set to `true`,** then a new scope will be created for this directive. If multiple directives on the
 * same element request a new scope, only one new scope is created. The new scope rule does not
 * apply for the root of the template since the root of the template always gets a new scope.
 *
 * **If set to `{}` (object hash),** then a new "isolate" scope is created. The 'isolate' scope differs from
 * normal scope in that it does not prototypically inherit from the parent scope. This is useful
 * when creating reusable components, which should not accidentally read or modify data in the
 * parent scope.
 *
 * The 'isolate' scope takes an object hash which defines a set of local scope properties
 * derived from the parent scope. These local properties are useful for aliasing values for
 * templates. Locals definition is a hash of local scope property to its source:
 *
 * * `@` or `@attr` - bind a local scope property to the value of DOM attribute. The result is
 *   always a string since DOM attributes are strings. If no `attr` name is specified  then the
 *   attribute name is assumed to be the same as the local name.
 *   Given `<widget my-attr="hello {{name}}">` and widget definition
 *   of `scope: { localName:'@myAttr' }`, then widget scope property `localName` will reflect
 *   the interpolated value of `hello {{name}}`. As the `name` attribute changes so will the
 *   `localName` property on the widget scope. The `name` is read from the parent scope (not
 *   component scope).
 *
 * * `=` or `=attr` - set up bi-directional binding between a local scope property and the
 *   parent scope property of name defined via the value of the `attr` attribute. If no `attr`
 *   name is specified then the attribute name is assumed to be the same as the local name.
 *   Given `<widget my-attr="parentModel">` and widget definition of
 *   `scope: { localModel:'=myAttr' }`, then widget scope property `localModel` will reflect the
 *   value of `parentModel` on the parent scope. Any changes to `parentModel` will be reflected
 *   in `localModel` and any changes in `localModel` will reflect in `parentModel`. If the parent
 *   scope property doesn't exist, it will throw a NON_ASSIGNABLE_MODEL_EXPRESSION exception. You
 *   can avoid this behavior using `=?` or `=?attr` in order to flag the property as optional. If
 *   you want to shallow watch for changes (i.e. $watchCollection instead of $watch) you can use
 *   `=*` or `=*attr` (`=*?` or `=*?attr` if the property is optional).
 *
 * * `&` or `&attr` - provides a way to execute an expression in the context of the parent scope.
 *   If no `attr` name is specified then the attribute name is assumed to be the same as the
 *   local name. Given `<widget my-attr="count = count + value">` and widget definition of
 *   `scope: { localFn:'&myAttr' }`, then isolate scope property `localFn` will point to
 *   a function wrapper for the `count = count + value` expression. Often it's desirable to
 *   pass data from the isolated scope via an expression to the parent scope, this can be
 *   done by passing a map of local variable names and values into the expression wrapper fn.
 *   For example, if the expression is `increment(amount)` then we can specify the amount value
 *   by calling the `localFn` as `localFn({amount: 22})`.
 *
 *
 * #### `bindToController`
 * When an isolate scope is used for a component (see above), and `controllerAs` is used, `bindToController: true` will
 * allow a component to have its properties bound to the controller, rather than to scope. When the controller
 * is instantiated, the initial values of the isolate scope bindings are already available.
 *
 * #### `controller`
 * Controller constructor function. The controller is instantiated before the
 * pre-linking phase and it is shared with other directives (see
 * `require` attribute). This allows the directives to communicate with each other and augment
 * each other's behavior. The controller is injectable (and supports bracket notation) with the following locals:
 *
 * * `$scope` - Current scope associated with the element
 * * `$element` - Current element
 * * `$attrs` - Current attributes object for the element
 * * `$transclude` - A transclude linking function pre-bound to the correct transclusion scope:
 *   `function([scope], cloneLinkingFn, futureParentElement)`.
 *    * `scope`: optional argument to override the scope.
 *    * `cloneLinkingFn`: optional argument to create clones of the original transcluded content.
 *    * `futureParentElement`:
 *        * defines the parent to which the `cloneLinkingFn` will add the cloned elements.
 *        * default: `$element.parent()` resp. `$element` for `transclude:'element'` resp. `transclude:true`.
 *        * only needed for transcludes that are allowed to contain non html elements (e.g. SVG elements)
 *          and when the `cloneLinkinFn` is passed,
 *          as those elements need to created and cloned in a special way when they are defined outside their
 *          usual containers (e.g. like `<svg>`).
 *        * See also the `directive.templateNamespace` property.
 *
 *
 * #### `require`
 * Require another directive and inject its controller as the fourth argument to the linking function. The
 * `require` takes a string name (or array of strings) of the directive(s) to pass in. If an array is used, the
 * injected argument will be an array in corresponding order. If no such directive can be
 * found, or if the directive does not have a controller, then an error is raised (unless no link function
 * is specified, in which case error checking is skipped). The name can be prefixed with:
 *
 * * (no prefix) - Locate the required controller on the current element. Throw an error if not found.
 * * `?` - Attempt to locate the required controller or pass `null` to the `link` fn if not found.
 * * `^` - Locate the required controller by searching the element and its parents. Throw an error if not found.
 * * `^^` - Locate the required controller by searching the element's parents. Throw an error if not found.
 * * `?^` - Attempt to locate the required controller by searching the element and its parents or pass
 *   `null` to the `link` fn if not found.
 * * `?^^` - Attempt to locate the required controller by searching the element's parents, or pass
 *   `null` to the `link` fn if not found.
 *
 *
 * #### `controllerAs`
 * Controller alias at the directive scope. An alias for the controller so it
 * can be referenced at the directive template. The directive needs to define a scope for this
 * configuration to be used. Useful in the case when directive is used as component.
 *
 *
 * #### `restrict`
 * String of subset of `EACM` which restricts the directive to a specific directive
 * declaration style. If omitted, the defaults (elements and attributes) are used.
 *
 * * `E` - Element name (default): `<my-directive></my-directive>`
 * * `A` - Attribute (default): `<div my-directive="exp"></div>`
 * * `C` - Class: `<div class="my-directive: exp;"></div>`
 * * `M` - Comment: `<!-- directive: my-directive exp -->`
 *
 *
 * #### `templateNamespace`
 * String representing the document type used by the markup in the template.
 * AngularJS needs this information as those elements need to be created and cloned
 * in a special way when they are defined outside their usual containers like `<svg>` and `<math>`.
 *
 * * `html` - All root nodes in the template are HTML. Root nodes may also be
 *   top-level elements such as `<svg>` or `<math>`.
 * * `svg` - The root nodes in the template are SVG elements (excluding `<math>`).
 * * `math` - The root nodes in the template are MathML elements (excluding `<svg>`).
 *
 * If no `templateNamespace` is specified, then the namespace is considered to be `html`.
 *
 * #### `template`
 * HTML markup that may:
 * * Replace the contents of the directive's element (default).
 * * Replace the directive's element itself (if `replace` is true - DEPRECATED).
 * * Wrap the contents of the directive's element (if `transclude` is true).
 *
 * Value may be:
 *
 * * A string. For example `<div red-on-hover>{{delete_str}}</div>`.
 * * A function which takes two arguments `tElement` and `tAttrs` (described in the `compile`
 *   function api below) and returns a string value.
 *
 *
 * #### `templateUrl`
 * This is similar to `template` but the template is loaded from the specified URL, asynchronously.
 *
 * Because template loading is asynchronous the compiler will suspend compilation of directives on that element
 * for later when the template has been resolved.  In the meantime it will continue to compile and link
 * sibling and parent elements as though this element had not contained any directives.
 *
 * The compiler does not suspend the entire compilation to wait for templates to be loaded because this
 * would result in the whole app "stalling" until all templates are loaded asynchronously - even in the
 * case when only one deeply nested directive has `templateUrl`.
 *
 * Template loading is asynchronous even if the template has been preloaded into the {@link $templateCache}
 *
 * You can specify `templateUrl` as a string representing the URL or as a function which takes two
 * arguments `tElement` and `tAttrs` (described in the `compile` function api below) and returns
 * a string value representing the url.  In either case, the template URL is passed through {@link
 * $sce#getTrustedResourceUrl $sce.getTrustedResourceUrl}.
 *
 *
 * #### `replace` ([*DEPRECATED*!], will be removed in next major release - i.e. v2.0)
 * specify what the template should replace. Defaults to `false`.
 *
 * * `true` - the template will replace the directive's element.
 * * `false` - the template will replace the contents of the directive's element.
 *
 * The replacement process migrates all of the attributes / classes from the old element to the new
 * one. See the {@link guide/directive#template-expanding-directive
 * Directives Guide} for an example.
 *
 * There are very few scenarios where element replacement is required for the application function,
 * the main one being reusable custom components that are used within SVG contexts
 * (because SVG doesn't work with custom elements in the DOM tree).
 *
 * #### `transclude`
 * Extract the contents of the element where the directive appears and make it available to the directive.
 * The contents are compiled and provided to the directive as a **transclusion function**. See the
 * {@link $compile#transclusion Transclusion} section below.
 *
 * There are two kinds of transclusion depending upon whether you want to transclude just the contents of the
 * directive's element or the entire element:
 *
 * * `true` - transclude the content (i.e. the child nodes) of the directive's element.
 * * `'element'` - transclude the whole of the directive's element including any directives on this
 *   element that defined at a lower priority than this directive. When used, the `template`
 *   property is ignored.
 *
 *
 * #### `compile`
 *
 * ```js
 *   function compile(tElement, tAttrs, transclude) { ... }
 * ```
 *
 * The compile function deals with transforming the template DOM. Since most directives do not do
 * template transformation, it is not used often. The compile function takes the following arguments:
 *
 *   * `tElement` - template element - The element where the directive has been declared. It is
 *     safe to do template transformation on the element and child elements only.
 *
 *   * `tAttrs` - template attributes - Normalized list of attributes declared on this element shared
 *     between all directive compile functions.
 *
 *   * `transclude` -  [*DEPRECATED*!] A transclude linking function: `function(scope, cloneLinkingFn)`
 *
 * <div class="alert alert-warning">
 * **Note:** The template instance and the link instance may be different objects if the template has
 * been cloned. For this reason it is **not** safe to do anything other than DOM transformations that
 * apply to all cloned DOM nodes within the compile function. Specifically, DOM listener registration
 * should be done in a linking function rather than in a compile function.
 * </div>

 * <div class="alert alert-warning">
 * **Note:** The compile function cannot handle directives that recursively use themselves in their
 * own templates or compile functions. Compiling these directives results in an infinite loop and a
 * stack overflow errors.
 *
 * This can be avoided by manually using $compile in the postLink function to imperatively compile
 * a directive's template instead of relying on automatic template compilation via `template` or
 * `templateUrl` declaration or manual compilation inside the compile function.
 * </div>
 *
 * <div class="alert alert-error">
 * **Note:** The `transclude` function that is passed to the compile function is deprecated, as it
 *   e.g. does not know about the right outer scope. Please use the transclude function that is passed
 *   to the link function instead.
 * </div>

 * A compile function can have a return value which can be either a function or an object.
 *
 * * returning a (post-link) function - is equivalent to registering the linking function via the
 *   `link` property of the config object when the compile function is empty.
 *
 * * returning an object with function(s) registered via `pre` and `post` properties - allows you to
 *   control when a linking function should be called during the linking phase. See info about
 *   pre-linking and post-linking functions below.
 *
 *
 * #### `link`
 * This property is used only if the `compile` property is not defined.
 *
 * ```js
 *   function link(scope, iElement, iAttrs, controller, transcludeFn) { ... }
 * ```
 *
 * The link function is responsible for registering DOM listeners as well as updating the DOM. It is
 * executed after the template has been cloned. This is where most of the directive logic will be
 * put.
 *
 *   * `scope` - {@link ng.$rootScope.Scope Scope} - The scope to be used by the
 *     directive for registering {@link ng.$rootScope.Scope#$watch watches}.
 *
 *   * `iElement` - instance element - The element where the directive is to be used. It is safe to
 *     manipulate the children of the element only in `postLink` function since the children have
 *     already been linked.
 *
 *   * `iAttrs` - instance attributes - Normalized list of attributes declared on this element shared
 *     between all directive linking functions.
 *
 *   * `controller` - the directive's required controller instance(s) - Instances are shared
 *     among all directives, which allows the directives to use the controllers as a communication
 *     channel. The exact value depends on the directive's `require` property:
 *       * `string`: the controller instance
 *       * `array`: array of controller instances
 *       * no controller(s) required: `undefined`
 *
 *     If a required controller cannot be found, and it is optional, the instance is `null`,
 *     otherwise the {@link error:$compile:ctreq Missing Required Controller} error is thrown.
 *
 *   * `transcludeFn` - A transclude linking function pre-bound to the correct transclusion scope.
 *     This is the same as the `$transclude`
 *     parameter of directive controllers, see there for details.
 *     `function([scope], cloneLinkingFn, futureParentElement)`.
 *
 * #### Pre-linking function
 *
 * Executed before the child elements are linked. Not safe to do DOM transformation since the
 * compiler linking function will fail to locate the correct elements for linking.
 *
 * #### Post-linking function
 *
 * Executed after the child elements are linked.
 *
 * Note that child elements that contain `templateUrl` directives will not have been compiled
 * and linked since they are waiting for their template to load asynchronously and their own
 * compilation and linking has been suspended until that occurs.
 *
 * It is safe to do DOM transformation in the post-linking function on elements that are not waiting
 * for their async templates to be resolved.
 *
 *
 * ### Transclusion
 *
 * Transclusion is the process of extracting a collection of DOM elements from one part of the DOM and
 * copying them to another part of the DOM, while maintaining their connection to the original AngularJS
 * scope from where they were taken.
 *
 * Transclusion is used (often with {@link ngTransclude}) to insert the
 * original contents of a directive's element into a specified place in the template of the directive.
 * The benefit of transclusion, over simply moving the DOM elements manually, is that the transcluded
 * content has access to the properties on the scope from which it was taken, even if the directive
 * has isolated scope.
 * See the {@link guide/directive#creating-a-directive-that-wraps-other-elements Directives Guide}.
 *
 * This makes it possible for the widget to have private state for its template, while the transcluded
 * content has access to its originating scope.
 *
 * <div class="alert alert-warning">
 * **Note:** When testing an element transclude directive you must not place the directive at the root of the
 * DOM fragment that is being compiled. See {@link guide/unit-testing#testing-transclusion-directives
 * Testing Transclusion Directives}.
 * </div>
 *
 * #### Transclusion Functions
 *
 * When a directive requests transclusion, the compiler extracts its contents and provides a **transclusion
 * function** to the directive's `link` function and `controller`. This transclusion function is a special
 * **linking function** that will return the compiled contents linked to a new transclusion scope.
 *
 * <div class="alert alert-info">
 * If you are just using {@link ngTransclude} then you don't need to worry about this function, since
 * ngTransclude will deal with it for us.
 * </div>
 *
 * If you want to manually control the insertion and removal of the transcluded content in your directive
 * then you must use this transclude function. When you call a transclude function it returns a a jqLite/JQuery
 * object that contains the compiled DOM, which is linked to the correct transclusion scope.
 *
 * When you call a transclusion function you can pass in a **clone attach function**. This function accepts
 * two parameters, `function(clone, scope) { ... }`, where the `clone` is a fresh compiled copy of your transcluded
 * content and the `scope` is the newly created transclusion scope, to which the clone is bound.
 *
 * <div class="alert alert-info">
 * **Best Practice**: Always provide a `cloneFn` (clone attach function) when you call a translude function
 * since you then get a fresh clone of the original DOM and also have access to the new transclusion scope.
 * </div>
 *
 * It is normal practice to attach your transcluded content (`clone`) to the DOM inside your **clone
 * attach function**:
 *
 * ```js
 * var transcludedContent, transclusionScope;
 *
 * $transclude(function(clone, scope) {
 *   element.append(clone);
 *   transcludedContent = clone;
 *   transclusionScope = scope;
 * });
 * ```
 *
 * Later, if you want to remove the transcluded content from your DOM then you should also destroy the
 * associated transclusion scope:
 *
 * ```js
 * transcludedContent.remove();
 * transclusionScope.$destroy();
 * ```
 *
 * <div class="alert alert-info">
 * **Best Practice**: if you intend to add and remove transcluded content manually in your directive
 * (by calling the transclude function to get the DOM and and calling `element.remove()` to remove it),
 * then you are also responsible for calling `$destroy` on the transclusion scope.
 * </div>
 *
 * The built-in DOM manipulation directives, such as {@link ngIf}, {@link ngSwitch} and {@link ngRepeat}
 * automatically destroy their transluded clones as necessary so you do not need to worry about this if
 * you are simply using {@link ngTransclude} to inject the transclusion into your directive.
 *
 *
 * #### Transclusion Scopes
 *
 * When you call a transclude function it returns a DOM fragment that is pre-bound to a **transclusion
 * scope**. This scope is special, in that it is a child of the directive's scope (and so gets destroyed
 * when the directive's scope gets destroyed) but it inherits the properties of the scope from which it
 * was taken.
 *
 * For example consider a directive that uses transclusion and isolated scope. The DOM hierarchy might look
 * like this:
 *
 * ```html
 * <div ng-app>
 *   <div isolate>
 *     <div transclusion>
 *     </div>
 *   </div>
 * </div>
 * ```
 *
 * The `$parent` scope hierarchy will look like this:
 *
 * ```
 * - $rootScope
 *   - isolate
 *     - transclusion
 * ```
 *
 * but the scopes will inherit prototypically from different scopes to their `$parent`.
 *
 * ```
 * - $rootScope
 *   - transclusion
 * - isolate
 * ```
 *
 *
 * ### Attributes
 *
 * The {@link ng.$compile.directive.Attributes Attributes} object - passed as a parameter in the
 * `link()` or `compile()` functions. It has a variety of uses.
 *
 * accessing *Normalized attribute names:*
 * Directives like 'ngBind' can be expressed in many ways: 'ng:bind', `data-ng-bind`, or 'x-ng-bind'.
 * the attributes object allows for normalized access to
 *   the attributes.
 *
 * * *Directive inter-communication:* All directives share the same instance of the attributes
 *   object which allows the directives to use the attributes object as inter directive
 *   communication.
 *
 * * *Supports interpolation:* Interpolation attributes are assigned to the attribute object
 *   allowing other directives to read the interpolated value.
 *
 * * *Observing interpolated attributes:* Use `$observe` to observe the value changes of attributes
 *   that contain interpolation (e.g. `src="{{bar}}"`). Not only is this very efficient but it's also
 *   the only way to easily get the actual value because during the linking phase the interpolation
 *   hasn't been evaluated yet and so the value is at this time set to `undefined`.
 *
 * ```js
 * function linkingFn(scope, elm, attrs, ctrl) {
 *   // get the attribute value
 *   console.log(attrs.ngModel);
 *
 *   // change the attribute
 *   attrs.$set('ngModel', 'new value');
 *
 *   // observe changes to interpolated attribute
 *   attrs.$observe('ngModel', function(value) {
 *     console.log('ngModel has changed value to ' + value);
 *   });
 * }
 * ```
 *
 * ## Example
 *
 * <div class="alert alert-warning">
 * **Note**: Typically directives are registered with `module.directive`. The example below is
 * to illustrate how `$compile` works.
 * </div>
 *
 <example module="compileExample">
   <file name="index.html">
    <script>
      angular.module('compileExample', [], function($compileProvider) {
        // configure new 'compile' directive by passing a directive
        // factory function. The factory function injects the '$compile'
        $compileProvider.directive('compile', function($compile) {
          // directive factory creates a link function
          return function(scope, element, attrs) {
            scope.$watch(
              function(scope) {
                 // watch the 'compile' expression for changes
                return scope.$eval(attrs.compile);
              },
              function(value) {
                // when the 'compile' expression changes
                // assign it into the current DOM
                element.html(value);

                // compile the new DOM and link it to the current
                // scope.
                // NOTE: we only compile .childNodes so that
                // we don't get into infinite loop compiling ourselves
                $compile(element.contents())(scope);
              }
            );
          };
        });
      })
      .controller('GreeterController', ['$scope', function($scope) {
        $scope.name = 'Angular';
        $scope.html = 'Hello {{name}}';
      }]);
    </script>
    <div ng-controller="GreeterController">
      <input ng-model="name"> <br>
      <textarea ng-model="html"></textarea> <br>
      <div compile="html"></div>
    </div>
   </file>
   <file name="protractor.js" type="protractor">
     it('should auto compile', function() {
       var textarea = $('textarea');
       var output = $('div[compile]');
       // The initial state reads 'Hello Angular'.
       expect(output.getText()).toBe('Hello Angular');
       textarea.clear();
       textarea.sendKeys('{{name}}!');
       expect(output.getText()).toBe('Angular!');
     });
   </file>
 </example>

 *
 *
 * @param {string|DOMElement} element Element or HTML string to compile into a template function.
 * @param {function(angular.Scope, cloneAttachFn=)} transclude function available to directives - DEPRECATED.
 *
 * <div class="alert alert-error">
 * **Note:** Passing a `transclude` function to the $compile function is deprecated, as it
 *   e.g. will not use the right outer scope. Please pass the transclude function as a
 *   `parentBoundTranscludeFn` to the link function instead.
 * </div>
 *
 * @param {number} maxPriority only apply directives lower than given priority (Only effects the
 *                 root element(s), not their children)
 * @returns {function(scope, cloneAttachFn=, options=)} a link function which is used to bind template
 * (a DOM element/tree) to a scope. Where:
 *
 *  * `scope` - A {@link ng.$rootScope.Scope Scope} to bind to.
 *  * `cloneAttachFn` - If `cloneAttachFn` is provided, then the link function will clone the
 *  `template` and call the `cloneAttachFn` function allowing the caller to attach the
 *  cloned elements to the DOM document at the appropriate place. The `cloneAttachFn` is
 *  called as: <br> `cloneAttachFn(clonedElement, scope)` where:
 *
 *      * `clonedElement` - is a clone of the original `element` passed into the compiler.
 *      * `scope` - is the current scope with which the linking function is working with.
 *
 *  * `options` - An optional object hash with linking options. If `options` is provided, then the following
 *  keys may be used to control linking behavior:
 *
 *      * `parentBoundTranscludeFn` - the transclude function made available to
 *        directives; if given, it will be passed through to the link functions of
 *        directives found in `element` during compilation.
 *      * `transcludeControllers` - an object hash with keys that map controller names
 *        to controller instances; if given, it will make the controllers
 *        available to directives.
 *      * `futureParentElement` - defines the parent to which the `cloneAttachFn` will add
 *        the cloned elements; only needed for transcludes that are allowed to contain non html
 *        elements (e.g. SVG elements). See also the directive.controller property.
 *
 * Calling the linking function returns the element of the template. It is either the original
 * element passed in, or the clone of the element if the `cloneAttachFn` is provided.
 *
 * After linking the view is not updated until after a call to $digest which typically is done by
 * Angular automatically.
 *
 * If you need access to the bound view, there are two ways to do it:
 *
 * - If you are not asking the linking function to clone the template, create the DOM element(s)
 *   before you send them to the compiler and keep this reference around.
 *   ```js
 *     var element = $compile('<p>{{total}}</p>')(scope);
 *   ```
 *
 * - if on the other hand, you need the element to be cloned, the view reference from the original
 *   example would not point to the clone, but rather to the original template that was cloned. In
 *   this case, you can access the clone via the cloneAttachFn:
 *   ```js
 *     var templateElement = angular.element('<p>{{total}}</p>'),
 *         scope = ....;
 *
 *     var clonedElement = $compile(templateElement)(scope, function(clonedElement, scope) {
 *       //attach the clone to DOM document at the right place
 *     });
 *
 *     //now we have reference to the cloned DOM via `clonedElement`
 *   ```
 *
 *
 * For information on how the compiler works, see the
 * {@link guide/compiler Angular HTML Compiler} section of the Developer Guide.
 */
  var $compileMinErr = minErr('$compile');
  /**
 * @ngdoc provider
 * @name $compileProvider
 *
 * @description
 */
  $CompileProvider.$inject = [
    '$provide',
    '$$sanitizeUriProvider'
  ];
  function $CompileProvider($provide, $$sanitizeUriProvider) {
    var hasDirectives = {}, Suffix = 'Directive', COMMENT_DIRECTIVE_REGEXP = /^\s*directive\:\s*([\w\-]+)\s+(.*)$/, CLASS_DIRECTIVE_REGEXP = /(([\w\-]+)(?:\:([^;]+))?;?)/, ALL_OR_NOTHING_ATTRS = makeMap('ngSrc,ngSrcset,src,srcset'), REQUIRE_PREFIX_REGEXP = /^(?:(\^\^?)?(\?)?(\^\^?)?)?/;
    // Ref: http://developers.whatwg.org/webappapis.html#event-handler-idl-attributes
    // The assumption is that future DOM event attribute names will begin with
    // 'on' and be composed of only English letters.
    var EVENT_HANDLER_ATTR_REGEXP = /^(on[a-z]+|formaction)$/;
    function parseIsolateBindings(scope, directiveName) {
      var LOCAL_REGEXP = /^\s*([@&]|=(\*?))(\??)\s*(\w*)\s*$/;
      var bindings = {};
      forEach(scope, function (definition, scopeName) {
        var match = definition.match(LOCAL_REGEXP);
        if (!match) {
          throw $compileMinErr('iscp', 'Invalid isolate scope definition for directive \'{0}\'.' + ' Definition: {... {1}: \'{2}\' ...}', directiveName, scopeName, definition);
        }
        bindings[scopeName] = {
          mode: match[1][0],
          collection: match[2] === '*',
          optional: match[3] === '?',
          attrName: match[4] || scopeName
        };
      });
      return bindings;
    }
    /**
   * @ngdoc method
   * @name $compileProvider#directive
   * @kind function
   *
   * @description
   * Register a new directive with the compiler.
   *
   * @param {string|Object} name Name of the directive in camel-case (i.e. <code>ngBind</code> which
   *    will match as <code>ng-bind</code>), or an object map of directives where the keys are the
   *    names and the values are the factories.
   * @param {Function|Array} directiveFactory An injectable directive factory function. See
   *    {@link guide/directive} for more info.
   * @returns {ng.$compileProvider} Self for chaining.
   */
    this.directive = function registerDirective(name, directiveFactory) {
      assertNotHasOwnProperty(name, 'directive');
      if (isString(name)) {
        assertArg(directiveFactory, 'directiveFactory');
        if (!hasDirectives.hasOwnProperty(name)) {
          hasDirectives[name] = [];
          $provide.factory(name + Suffix, [
            '$injector',
            '$exceptionHandler',
            function ($injector, $exceptionHandler) {
              var directives = [];
              forEach(hasDirectives[name], function (directiveFactory, index) {
                try {
                  var directive = $injector.invoke(directiveFactory);
                  if (isFunction(directive)) {
                    directive = { compile: valueFn(directive) };
                  } else if (!directive.compile && directive.link) {
                    directive.compile = valueFn(directive.link);
                  }
                  directive.priority = directive.priority || 0;
                  directive.index = index;
                  directive.name = directive.name || name;
                  directive.require = directive.require || directive.controller && directive.name;
                  directive.restrict = directive.restrict || 'EA';
                  if (isObject(directive.scope)) {
                    directive.$$isolateBindings = parseIsolateBindings(directive.scope, directive.name);
                  }
                  directives.push(directive);
                } catch (e) {
                  $exceptionHandler(e);
                }
              });
              return directives;
            }
          ]);
        }
        hasDirectives[name].push(directiveFactory);
      } else {
        forEach(name, reverseParams(registerDirective));
      }
      return this;
    };
    /**
   * @ngdoc method
   * @name $compileProvider#aHrefSanitizationWhitelist
   * @kind function
   *
   * @description
   * Retrieves or overrides the default regular expression that is used for whitelisting of safe
   * urls during a[href] sanitization.
   *
   * The sanitization is a security measure aimed at preventing XSS attacks via html links.
   *
   * Any url about to be assigned to a[href] via data-binding is first normalized and turned into
   * an absolute url. Afterwards, the url is matched against the `aHrefSanitizationWhitelist`
   * regular expression. If a match is found, the original url is written into the dom. Otherwise,
   * the absolute url is prefixed with `'unsafe:'` string and only then is it written into the DOM.
   *
   * @param {RegExp=} regexp New regexp to whitelist urls with.
   * @returns {RegExp|ng.$compileProvider} Current RegExp if called without value or self for
   *    chaining otherwise.
   */
    this.aHrefSanitizationWhitelist = function (regexp) {
      if (isDefined(regexp)) {
        $$sanitizeUriProvider.aHrefSanitizationWhitelist(regexp);
        return this;
      } else {
        return $$sanitizeUriProvider.aHrefSanitizationWhitelist();
      }
    };
    /**
   * @ngdoc method
   * @name $compileProvider#imgSrcSanitizationWhitelist
   * @kind function
   *
   * @description
   * Retrieves or overrides the default regular expression that is used for whitelisting of safe
   * urls during img[src] sanitization.
   *
   * The sanitization is a security measure aimed at prevent XSS attacks via html links.
   *
   * Any url about to be assigned to img[src] via data-binding is first normalized and turned into
   * an absolute url. Afterwards, the url is matched against the `imgSrcSanitizationWhitelist`
   * regular expression. If a match is found, the original url is written into the dom. Otherwise,
   * the absolute url is prefixed with `'unsafe:'` string and only then is it written into the DOM.
   *
   * @param {RegExp=} regexp New regexp to whitelist urls with.
   * @returns {RegExp|ng.$compileProvider} Current RegExp if called without value or self for
   *    chaining otherwise.
   */
    this.imgSrcSanitizationWhitelist = function (regexp) {
      if (isDefined(regexp)) {
        $$sanitizeUriProvider.imgSrcSanitizationWhitelist(regexp);
        return this;
      } else {
        return $$sanitizeUriProvider.imgSrcSanitizationWhitelist();
      }
    };
    /**
   * @ngdoc method
   * @name  $compileProvider#debugInfoEnabled
   *
   * @param {boolean=} enabled update the debugInfoEnabled state if provided, otherwise just return the
   * current debugInfoEnabled state
   * @returns {*} current value if used as getter or itself (chaining) if used as setter
   *
   * @kind function
   *
   * @description
   * Call this method to enable/disable various debug runtime information in the compiler such as adding
   * binding information and a reference to the current scope on to DOM elements.
   * If enabled, the compiler will add the following to DOM elements that have been bound to the scope
   * * `ng-binding` CSS class
   * * `$binding` data property containing an array of the binding expressions
   *
   * You may want to disable this in production for a significant performance boost. See
   * {@link guide/production#disabling-debug-data Disabling Debug Data} for more.
   *
   * The default value is true.
   */
    var debugInfoEnabled = true;
    this.debugInfoEnabled = function (enabled) {
      if (isDefined(enabled)) {
        debugInfoEnabled = enabled;
        return this;
      }
      return debugInfoEnabled;
    };
    this.$get = [
      '$injector',
      '$interpolate',
      '$exceptionHandler',
      '$templateRequest',
      '$parse',
      '$controller',
      '$rootScope',
      '$document',
      '$sce',
      '$animate',
      '$$sanitizeUri',
      function ($injector, $interpolate, $exceptionHandler, $templateRequest, $parse, $controller, $rootScope, $document, $sce, $animate, $$sanitizeUri) {
        var Attributes = function (element, attributesToCopy) {
          if (attributesToCopy) {
            var keys = Object.keys(attributesToCopy);
            var i, l, key;
            for (i = 0, l = keys.length; i < l; i++) {
              key = keys[i];
              this[key] = attributesToCopy[key];
            }
          } else {
            this.$attr = {};
          }
          this.$$element = element;
        };
        Attributes.prototype = {
          $normalize: directiveNormalize,
          $addClass: function (classVal) {
            if (classVal && classVal.length > 0) {
              $animate.addClass(this.$$element, classVal);
            }
          },
          $removeClass: function (classVal) {
            if (classVal && classVal.length > 0) {
              $animate.removeClass(this.$$element, classVal);
            }
          },
          $updateClass: function (newClasses, oldClasses) {
            var toAdd = tokenDifference(newClasses, oldClasses);
            if (toAdd && toAdd.length) {
              $animate.addClass(this.$$element, toAdd);
            }
            var toRemove = tokenDifference(oldClasses, newClasses);
            if (toRemove && toRemove.length) {
              $animate.removeClass(this.$$element, toRemove);
            }
          },
          $set: function (key, value, writeAttr, attrName) {
            // TODO: decide whether or not to throw an error if "class"
            //is set through this function since it may cause $updateClass to
            //become unstable.
            var node = this.$$element[0], booleanKey = getBooleanAttrName(node, key), aliasedKey = getAliasedAttrName(node, key), observer = key, nodeName;
            if (booleanKey) {
              this.$$element.prop(key, value);
              attrName = booleanKey;
            } else if (aliasedKey) {
              this[aliasedKey] = value;
              observer = aliasedKey;
            }
            this[key] = value;
            // translate normalized key to actual key
            if (attrName) {
              this.$attr[key] = attrName;
            } else {
              attrName = this.$attr[key];
              if (!attrName) {
                this.$attr[key] = attrName = snake_case(key, '-');
              }
            }
            nodeName = nodeName_(this.$$element);
            if (nodeName === 'a' && key === 'href' || nodeName === 'img' && key === 'src') {
              // sanitize a[href] and img[src] values
              this[key] = value = $$sanitizeUri(value, key === 'src');
            } else if (nodeName === 'img' && key === 'srcset') {
              // sanitize img[srcset] values
              var result = '';
              // first check if there are spaces because it's not the same pattern
              var trimmedSrcset = trim(value);
              //                (   999x   ,|   999w   ,|   ,|,   )
              var srcPattern = /(\s+\d+x\s*,|\s+\d+w\s*,|\s+,|,\s+)/;
              var pattern = /\s/.test(trimmedSrcset) ? srcPattern : /(,)/;
              // split srcset into tuple of uri and descriptor except for the last item
              var rawUris = trimmedSrcset.split(pattern);
              // for each tuples
              var nbrUrisWith2parts = Math.floor(rawUris.length / 2);
              for (var i = 0; i < nbrUrisWith2parts; i++) {
                var innerIdx = i * 2;
                // sanitize the uri
                result += $$sanitizeUri(trim(rawUris[innerIdx]), true);
                // add the descriptor
                result += ' ' + trim(rawUris[innerIdx + 1]);
              }
              // split the last item into uri and descriptor
              var lastTuple = trim(rawUris[i * 2]).split(/\s/);
              // sanitize the last uri
              result += $$sanitizeUri(trim(lastTuple[0]), true);
              // and add the last descriptor if any
              if (lastTuple.length === 2) {
                result += ' ' + trim(lastTuple[1]);
              }
              this[key] = value = result;
            }
            if (writeAttr !== false) {
              if (value === null || value === undefined) {
                this.$$element.removeAttr(attrName);
              } else {
                this.$$element.attr(attrName, value);
              }
            }
            // fire observers
            var $$observers = this.$$observers;
            $$observers && forEach($$observers[observer], function (fn) {
              try {
                fn(value);
              } catch (e) {
                $exceptionHandler(e);
              }
            });
          },
          $observe: function (key, fn) {
            var attrs = this, $$observers = attrs.$$observers || (attrs.$$observers = createMap()), listeners = $$observers[key] || ($$observers[key] = []);
            listeners.push(fn);
            $rootScope.$evalAsync(function () {
              if (!listeners.$$inter && attrs.hasOwnProperty(key)) {
                // no one registered attribute interpolation function, so lets call it manually
                fn(attrs[key]);
              }
            });
            return function () {
              arrayRemove(listeners, fn);
            };
          }
        };
        function safeAddClass($element, className) {
          try {
            $element.addClass(className);
          } catch (e) {
          }
        }
        var startSymbol = $interpolate.startSymbol(), endSymbol = $interpolate.endSymbol(), denormalizeTemplate = startSymbol == '{{' || endSymbol == '}}' ? identity : function denormalizeTemplate(template) {
            return template.replace(/\{\{/g, startSymbol).replace(/}}/g, endSymbol);
          }, NG_ATTR_BINDING = /^ngAttr[A-Z]/;
        compile.$$addBindingInfo = debugInfoEnabled ? function $$addBindingInfo($element, binding) {
          var bindings = $element.data('$binding') || [];
          if (isArray(binding)) {
            bindings = bindings.concat(binding);
          } else {
            bindings.push(binding);
          }
          $element.data('$binding', bindings);
        } : noop;
        compile.$$addBindingClass = debugInfoEnabled ? function $$addBindingClass($element) {
          safeAddClass($element, 'ng-binding');
        } : noop;
        compile.$$addScopeInfo = debugInfoEnabled ? function $$addScopeInfo($element, scope, isolated, noTemplate) {
          var dataName = isolated ? noTemplate ? '$isolateScopeNoTemplate' : '$isolateScope' : '$scope';
          $element.data(dataName, scope);
        } : noop;
        compile.$$addScopeClass = debugInfoEnabled ? function $$addScopeClass($element, isolated) {
          safeAddClass($element, isolated ? 'ng-isolate-scope' : 'ng-scope');
        } : noop;
        return compile;
        //================================
        function compile($compileNodes, transcludeFn, maxPriority, ignoreDirective, previousCompileContext) {
          if (!($compileNodes instanceof jqLite)) {
            // jquery always rewraps, whereas we need to preserve the original selector so that we can
            // modify it.
            $compileNodes = jqLite($compileNodes);
          }
          // We can not compile top level text elements since text nodes can be merged and we will
          // not be able to attach scope data to them, so we will wrap them in <span>
          forEach($compileNodes, function (node, index) {
            if (node.nodeType == NODE_TYPE_TEXT && node.nodeValue.match(/\S+/)) {
              $compileNodes[index] = jqLite(node).wrap('<span></span>').parent()[0];
            }
          });
          var compositeLinkFn = compileNodes($compileNodes, transcludeFn, $compileNodes, maxPriority, ignoreDirective, previousCompileContext);
          compile.$$addScopeClass($compileNodes);
          var namespace = null;
          return function publicLinkFn(scope, cloneConnectFn, options) {
            assertArg(scope, 'scope');
            options = options || {};
            var parentBoundTranscludeFn = options.parentBoundTranscludeFn, transcludeControllers = options.transcludeControllers, futureParentElement = options.futureParentElement;
            // When `parentBoundTranscludeFn` is passed, it is a
            // `controllersBoundTransclude` function (it was previously passed
            // as `transclude` to directive.link) so we must unwrap it to get
            // its `boundTranscludeFn`
            if (parentBoundTranscludeFn && parentBoundTranscludeFn.$$boundTransclude) {
              parentBoundTranscludeFn = parentBoundTranscludeFn.$$boundTransclude;
            }
            if (!namespace) {
              namespace = detectNamespaceForChildElements(futureParentElement);
            }
            var $linkNode;
            if (namespace !== 'html') {
              // When using a directive with replace:true and templateUrl the $compileNodes
              // (or a child element inside of them)
              // might change, so we need to recreate the namespace adapted compileNodes
              // for call to the link function.
              // Note: This will already clone the nodes...
              $linkNode = jqLite(wrapTemplate(namespace, jqLite('<div>').append($compileNodes).html()));
            } else if (cloneConnectFn) {
              // important!!: we must call our jqLite.clone() since the jQuery one is trying to be smart
              // and sometimes changes the structure of the DOM.
              $linkNode = JQLitePrototype.clone.call($compileNodes);
            } else {
              $linkNode = $compileNodes;
            }
            if (transcludeControllers) {
              for (var controllerName in transcludeControllers) {
                $linkNode.data('$' + controllerName + 'Controller', transcludeControllers[controllerName].instance);
              }
            }
            compile.$$addScopeInfo($linkNode, scope);
            if (cloneConnectFn)
              cloneConnectFn($linkNode, scope);
            if (compositeLinkFn)
              compositeLinkFn(scope, $linkNode, $linkNode, parentBoundTranscludeFn);
            return $linkNode;
          };
        }
        function detectNamespaceForChildElements(parentElement) {
          // TODO: Make this detect MathML as well...
          var node = parentElement && parentElement[0];
          if (!node) {
            return 'html';
          } else {
            return nodeName_(node) !== 'foreignobject' && node.toString().match(/SVG/) ? 'svg' : 'html';
          }
        }
        /**
     * Compile function matches each node in nodeList against the directives. Once all directives
     * for a particular node are collected their compile functions are executed. The compile
     * functions return values - the linking functions - are combined into a composite linking
     * function, which is the a linking function for the node.
     *
     * @param {NodeList} nodeList an array of nodes or NodeList to compile
     * @param {function(angular.Scope, cloneAttachFn=)} transcludeFn A linking function, where the
     *        scope argument is auto-generated to the new child of the transcluded parent scope.
     * @param {DOMElement=} $rootElement If the nodeList is the root of the compilation tree then
     *        the rootElement must be set the jqLite collection of the compile root. This is
     *        needed so that the jqLite collection items can be replaced with widgets.
     * @param {number=} maxPriority Max directive priority.
     * @returns {Function} A composite linking function of all of the matched directives or null.
     */
        function compileNodes(nodeList, transcludeFn, $rootElement, maxPriority, ignoreDirective, previousCompileContext) {
          var linkFns = [], attrs, directives, nodeLinkFn, childNodes, childLinkFn, linkFnFound, nodeLinkFnFound;
          for (var i = 0; i < nodeList.length; i++) {
            attrs = new Attributes();
            // we must always refer to nodeList[i] since the nodes can be replaced underneath us.
            directives = collectDirectives(nodeList[i], [], attrs, i === 0 ? maxPriority : undefined, ignoreDirective);
            nodeLinkFn = directives.length ? applyDirectivesToNode(directives, nodeList[i], attrs, transcludeFn, $rootElement, null, [], [], previousCompileContext) : null;
            if (nodeLinkFn && nodeLinkFn.scope) {
              compile.$$addScopeClass(attrs.$$element);
            }
            childLinkFn = nodeLinkFn && nodeLinkFn.terminal || !(childNodes = nodeList[i].childNodes) || !childNodes.length ? null : compileNodes(childNodes, nodeLinkFn ? (nodeLinkFn.transcludeOnThisElement || !nodeLinkFn.templateOnThisElement) && nodeLinkFn.transclude : transcludeFn);
            if (nodeLinkFn || childLinkFn) {
              linkFns.push(i, nodeLinkFn, childLinkFn);
              linkFnFound = true;
              nodeLinkFnFound = nodeLinkFnFound || nodeLinkFn;
            }
            //use the previous context only for the first element in the virtual group
            previousCompileContext = null;
          }
          // return a linking function if we have found anything, null otherwise
          return linkFnFound ? compositeLinkFn : null;
          function compositeLinkFn(scope, nodeList, $rootElement, parentBoundTranscludeFn) {
            var nodeLinkFn, childLinkFn, node, childScope, i, ii, idx, childBoundTranscludeFn;
            var stableNodeList;
            if (nodeLinkFnFound) {
              // copy nodeList so that if a nodeLinkFn removes or adds an element at this DOM level our
              // offsets don't get screwed up
              var nodeListLength = nodeList.length;
              stableNodeList = new Array(nodeListLength);
              // create a sparse array by only copying the elements which have a linkFn
              for (i = 0; i < linkFns.length; i += 3) {
                idx = linkFns[i];
                stableNodeList[idx] = nodeList[idx];
              }
            } else {
              stableNodeList = nodeList;
            }
            for (i = 0, ii = linkFns.length; i < ii;) {
              node = stableNodeList[linkFns[i++]];
              nodeLinkFn = linkFns[i++];
              childLinkFn = linkFns[i++];
              if (nodeLinkFn) {
                if (nodeLinkFn.scope) {
                  childScope = scope.$new();
                  compile.$$addScopeInfo(jqLite(node), childScope);
                } else {
                  childScope = scope;
                }
                if (nodeLinkFn.transcludeOnThisElement) {
                  childBoundTranscludeFn = createBoundTranscludeFn(scope, nodeLinkFn.transclude, parentBoundTranscludeFn, nodeLinkFn.elementTranscludeOnThisElement);
                } else if (!nodeLinkFn.templateOnThisElement && parentBoundTranscludeFn) {
                  childBoundTranscludeFn = parentBoundTranscludeFn;
                } else if (!parentBoundTranscludeFn && transcludeFn) {
                  childBoundTranscludeFn = createBoundTranscludeFn(scope, transcludeFn);
                } else {
                  childBoundTranscludeFn = null;
                }
                nodeLinkFn(childLinkFn, childScope, node, $rootElement, childBoundTranscludeFn);
              } else if (childLinkFn) {
                childLinkFn(scope, node.childNodes, undefined, parentBoundTranscludeFn);
              }
            }
          }
        }
        function createBoundTranscludeFn(scope, transcludeFn, previousBoundTranscludeFn, elementTransclusion) {
          var boundTranscludeFn = function (transcludedScope, cloneFn, controllers, futureParentElement, containingScope) {
            if (!transcludedScope) {
              transcludedScope = scope.$new(false, containingScope);
              transcludedScope.$$transcluded = true;
            }
            return transcludeFn(transcludedScope, cloneFn, {
              parentBoundTranscludeFn: previousBoundTranscludeFn,
              transcludeControllers: controllers,
              futureParentElement: futureParentElement
            });
          };
          return boundTranscludeFn;
        }
        /**
     * Looks for directives on the given node and adds them to the directive collection which is
     * sorted.
     *
     * @param node Node to search.
     * @param directives An array to which the directives are added to. This array is sorted before
     *        the function returns.
     * @param attrs The shared attrs object which is used to populate the normalized attributes.
     * @param {number=} maxPriority Max directive priority.
     */
        function collectDirectives(node, directives, attrs, maxPriority, ignoreDirective) {
          var nodeType = node.nodeType, attrsMap = attrs.$attr, match, className;
          switch (nodeType) {
          case NODE_TYPE_ELEMENT:
            /* Element */
            // use the node name: <directive>
            addDirective(directives, directiveNormalize(nodeName_(node)), 'E', maxPriority, ignoreDirective);
            // iterate over the attributes
            for (var attr, name, nName, ngAttrName, value, isNgAttr, nAttrs = node.attributes, j = 0, jj = nAttrs && nAttrs.length; j < jj; j++) {
              var attrStartName = false;
              var attrEndName = false;
              attr = nAttrs[j];
              name = attr.name;
              value = trim(attr.value);
              // support ngAttr attribute binding
              ngAttrName = directiveNormalize(name);
              if (isNgAttr = NG_ATTR_BINDING.test(ngAttrName)) {
                name = name.replace(PREFIX_REGEXP, '').substr(8).replace(/_(.)/g, function (match, letter) {
                  return letter.toUpperCase();
                });
              }
              var directiveNName = ngAttrName.replace(/(Start|End)$/, '');
              if (directiveIsMultiElement(directiveNName)) {
                if (ngAttrName === directiveNName + 'Start') {
                  attrStartName = name;
                  attrEndName = name.substr(0, name.length - 5) + 'end';
                  name = name.substr(0, name.length - 6);
                }
              }
              nName = directiveNormalize(name.toLowerCase());
              attrsMap[nName] = name;
              if (isNgAttr || !attrs.hasOwnProperty(nName)) {
                attrs[nName] = value;
                if (getBooleanAttrName(node, nName)) {
                  attrs[nName] = true;  // presence means true
                }
              }
              addAttrInterpolateDirective(node, directives, value, nName, isNgAttr);
              addDirective(directives, nName, 'A', maxPriority, ignoreDirective, attrStartName, attrEndName);
            }
            // use class as directive
            className = node.className;
            if (isObject(className)) {
              // Maybe SVGAnimatedString
              className = className.animVal;
            }
            if (isString(className) && className !== '') {
              while (match = CLASS_DIRECTIVE_REGEXP.exec(className)) {
                nName = directiveNormalize(match[2]);
                if (addDirective(directives, nName, 'C', maxPriority, ignoreDirective)) {
                  attrs[nName] = trim(match[3]);
                }
                className = className.substr(match.index + match[0].length);
              }
            }
            break;
          case NODE_TYPE_TEXT:
            /* Text Node */
            addTextInterpolateDirective(directives, node.nodeValue);
            break;
          case NODE_TYPE_COMMENT:
            /* Comment */
            try {
              match = COMMENT_DIRECTIVE_REGEXP.exec(node.nodeValue);
              if (match) {
                nName = directiveNormalize(match[1]);
                if (addDirective(directives, nName, 'M', maxPriority, ignoreDirective)) {
                  attrs[nName] = trim(match[2]);
                }
              }
            } catch (e) {
            }
            break;
          }
          directives.sort(byPriority);
          return directives;
        }
        /**
     * Given a node with an directive-start it collects all of the siblings until it finds
     * directive-end.
     * @param node
     * @param attrStart
     * @param attrEnd
     * @returns {*}
     */
        function groupScan(node, attrStart, attrEnd) {
          var nodes = [];
          var depth = 0;
          if (attrStart && node.hasAttribute && node.hasAttribute(attrStart)) {
            do {
              if (!node) {
                throw $compileMinErr('uterdir', 'Unterminated attribute, found \'{0}\' but no matching \'{1}\' found.', attrStart, attrEnd);
              }
              if (node.nodeType == NODE_TYPE_ELEMENT) {
                if (node.hasAttribute(attrStart))
                  depth++;
                if (node.hasAttribute(attrEnd))
                  depth--;
              }
              nodes.push(node);
              node = node.nextSibling;
            } while (depth > 0);
          } else {
            nodes.push(node);
          }
          return jqLite(nodes);
        }
        /**
     * Wrapper for linking function which converts normal linking function into a grouped
     * linking function.
     * @param linkFn
     * @param attrStart
     * @param attrEnd
     * @returns {Function}
     */
        function groupElementsLinkFnWrapper(linkFn, attrStart, attrEnd) {
          return function (scope, element, attrs, controllers, transcludeFn) {
            element = groupScan(element[0], attrStart, attrEnd);
            return linkFn(scope, element, attrs, controllers, transcludeFn);
          };
        }
        /**
     * Once the directives have been collected, their compile functions are executed. This method
     * is responsible for inlining directive templates as well as terminating the application
     * of the directives if the terminal directive has been reached.
     *
     * @param {Array} directives Array of collected directives to execute their compile function.
     *        this needs to be pre-sorted by priority order.
     * @param {Node} compileNode The raw DOM node to apply the compile functions to
     * @param {Object} templateAttrs The shared attribute function
     * @param {function(angular.Scope, cloneAttachFn=)} transcludeFn A linking function, where the
     *                                                  scope argument is auto-generated to the new
     *                                                  child of the transcluded parent scope.
     * @param {JQLite} jqCollection If we are working on the root of the compile tree then this
     *                              argument has the root jqLite array so that we can replace nodes
     *                              on it.
     * @param {Object=} originalReplaceDirective An optional directive that will be ignored when
     *                                           compiling the transclusion.
     * @param {Array.<Function>} preLinkFns
     * @param {Array.<Function>} postLinkFns
     * @param {Object} previousCompileContext Context used for previous compilation of the current
     *                                        node
     * @returns {Function} linkFn
     */
        function applyDirectivesToNode(directives, compileNode, templateAttrs, transcludeFn, jqCollection, originalReplaceDirective, preLinkFns, postLinkFns, previousCompileContext) {
          previousCompileContext = previousCompileContext || {};
          var terminalPriority = -Number.MAX_VALUE, newScopeDirective, controllerDirectives = previousCompileContext.controllerDirectives, controllers, newIsolateScopeDirective = previousCompileContext.newIsolateScopeDirective, templateDirective = previousCompileContext.templateDirective, nonTlbTranscludeDirective = previousCompileContext.nonTlbTranscludeDirective, hasTranscludeDirective = false, hasTemplate = false, hasElementTranscludeDirective = previousCompileContext.hasElementTranscludeDirective, $compileNode = templateAttrs.$$element = jqLite(compileNode), directive, directiveName, $template, replaceDirective = originalReplaceDirective, childTranscludeFn = transcludeFn, linkFn, directiveValue;
          // executes all directives on the current element
          for (var i = 0, ii = directives.length; i < ii; i++) {
            directive = directives[i];
            var attrStart = directive.$$start;
            var attrEnd = directive.$$end;
            // collect multiblock sections
            if (attrStart) {
              $compileNode = groupScan(compileNode, attrStart, attrEnd);
            }
            $template = undefined;
            if (terminalPriority > directive.priority) {
              break;  // prevent further processing of directives
            }
            if (directiveValue = directive.scope) {
              // skip the check for directives with async templates, we'll check the derived sync
              // directive when the template arrives
              if (!directive.templateUrl) {
                if (isObject(directiveValue)) {
                  // This directive is trying to add an isolated scope.
                  // Check that there is no scope of any kind already
                  assertNoDuplicate('new/isolated scope', newIsolateScopeDirective || newScopeDirective, directive, $compileNode);
                  newIsolateScopeDirective = directive;
                } else {
                  // This directive is trying to add a child scope.
                  // Check that there is no isolated scope already
                  assertNoDuplicate('new/isolated scope', newIsolateScopeDirective, directive, $compileNode);
                }
              }
              newScopeDirective = newScopeDirective || directive;
            }
            directiveName = directive.name;
            if (!directive.templateUrl && directive.controller) {
              directiveValue = directive.controller;
              controllerDirectives = controllerDirectives || {};
              assertNoDuplicate('\'' + directiveName + '\' controller', controllerDirectives[directiveName], directive, $compileNode);
              controllerDirectives[directiveName] = directive;
            }
            if (directiveValue = directive.transclude) {
              hasTranscludeDirective = true;
              // Special case ngIf and ngRepeat so that we don't complain about duplicate transclusion.
              // This option should only be used by directives that know how to safely handle element transclusion,
              // where the transcluded nodes are added or replaced after linking.
              if (!directive.$$tlb) {
                assertNoDuplicate('transclusion', nonTlbTranscludeDirective, directive, $compileNode);
                nonTlbTranscludeDirective = directive;
              }
              if (directiveValue == 'element') {
                hasElementTranscludeDirective = true;
                terminalPriority = directive.priority;
                $template = $compileNode;
                $compileNode = templateAttrs.$$element = jqLite(document.createComment(' ' + directiveName + ': ' + templateAttrs[directiveName] + ' '));
                compileNode = $compileNode[0];
                replaceWith(jqCollection, sliceArgs($template), compileNode);
                childTranscludeFn = compile($template, transcludeFn, terminalPriority, replaceDirective && replaceDirective.name, { nonTlbTranscludeDirective: nonTlbTranscludeDirective });
              } else {
                $template = jqLite(jqLiteClone(compileNode)).contents();
                $compileNode.empty();
                // clear contents
                childTranscludeFn = compile($template, transcludeFn);
              }
            }
            if (directive.template) {
              hasTemplate = true;
              assertNoDuplicate('template', templateDirective, directive, $compileNode);
              templateDirective = directive;
              directiveValue = isFunction(directive.template) ? directive.template($compileNode, templateAttrs) : directive.template;
              directiveValue = denormalizeTemplate(directiveValue);
              if (directive.replace) {
                replaceDirective = directive;
                if (jqLiteIsTextNode(directiveValue)) {
                  $template = [];
                } else {
                  $template = removeComments(wrapTemplate(directive.templateNamespace, trim(directiveValue)));
                }
                compileNode = $template[0];
                if ($template.length != 1 || compileNode.nodeType !== NODE_TYPE_ELEMENT) {
                  throw $compileMinErr('tplrt', 'Template for directive \'{0}\' must have exactly one root element. {1}', directiveName, '');
                }
                replaceWith(jqCollection, $compileNode, compileNode);
                var newTemplateAttrs = { $attr: {} };
                // combine directives from the original node and from the template:
                // - take the array of directives for this element
                // - split it into two parts, those that already applied (processed) and those that weren't (unprocessed)
                // - collect directives from the template and sort them by priority
                // - combine directives as: processed + template + unprocessed
                var templateDirectives = collectDirectives(compileNode, [], newTemplateAttrs);
                var unprocessedDirectives = directives.splice(i + 1, directives.length - (i + 1));
                if (newIsolateScopeDirective) {
                  markDirectivesAsIsolate(templateDirectives);
                }
                directives = directives.concat(templateDirectives).concat(unprocessedDirectives);
                mergeTemplateAttributes(templateAttrs, newTemplateAttrs);
                ii = directives.length;
              } else {
                $compileNode.html(directiveValue);
              }
            }
            if (directive.templateUrl) {
              hasTemplate = true;
              assertNoDuplicate('template', templateDirective, directive, $compileNode);
              templateDirective = directive;
              if (directive.replace) {
                replaceDirective = directive;
              }
              nodeLinkFn = compileTemplateUrl(directives.splice(i, directives.length - i), $compileNode, templateAttrs, jqCollection, hasTranscludeDirective && childTranscludeFn, preLinkFns, postLinkFns, {
                controllerDirectives: controllerDirectives,
                newIsolateScopeDirective: newIsolateScopeDirective,
                templateDirective: templateDirective,
                nonTlbTranscludeDirective: nonTlbTranscludeDirective
              });
              ii = directives.length;
            } else if (directive.compile) {
              try {
                linkFn = directive.compile($compileNode, templateAttrs, childTranscludeFn);
                if (isFunction(linkFn)) {
                  addLinkFns(null, linkFn, attrStart, attrEnd);
                } else if (linkFn) {
                  addLinkFns(linkFn.pre, linkFn.post, attrStart, attrEnd);
                }
              } catch (e) {
                $exceptionHandler(e, startingTag($compileNode));
              }
            }
            if (directive.terminal) {
              nodeLinkFn.terminal = true;
              terminalPriority = Math.max(terminalPriority, directive.priority);
            }
          }
          nodeLinkFn.scope = newScopeDirective && newScopeDirective.scope === true;
          nodeLinkFn.transcludeOnThisElement = hasTranscludeDirective;
          nodeLinkFn.elementTranscludeOnThisElement = hasElementTranscludeDirective;
          nodeLinkFn.templateOnThisElement = hasTemplate;
          nodeLinkFn.transclude = childTranscludeFn;
          previousCompileContext.hasElementTranscludeDirective = hasElementTranscludeDirective;
          // might be normal or delayed nodeLinkFn depending on if templateUrl is present
          return nodeLinkFn;
          ////////////////////
          function addLinkFns(pre, post, attrStart, attrEnd) {
            if (pre) {
              if (attrStart)
                pre = groupElementsLinkFnWrapper(pre, attrStart, attrEnd);
              pre.require = directive.require;
              pre.directiveName = directiveName;
              if (newIsolateScopeDirective === directive || directive.$$isolateScope) {
                pre = cloneAndAnnotateFn(pre, { isolateScope: true });
              }
              preLinkFns.push(pre);
            }
            if (post) {
              if (attrStart)
                post = groupElementsLinkFnWrapper(post, attrStart, attrEnd);
              post.require = directive.require;
              post.directiveName = directiveName;
              if (newIsolateScopeDirective === directive || directive.$$isolateScope) {
                post = cloneAndAnnotateFn(post, { isolateScope: true });
              }
              postLinkFns.push(post);
            }
          }
          function getControllers(directiveName, require, $element, elementControllers) {
            var value, retrievalMethod = 'data', optional = false;
            var $searchElement = $element;
            var match;
            if (isString(require)) {
              match = require.match(REQUIRE_PREFIX_REGEXP);
              require = require.substring(match[0].length);
              if (match[3]) {
                if (match[1])
                  match[3] = null;
                else
                  match[1] = match[3];
              }
              if (match[1] === '^') {
                retrievalMethod = 'inheritedData';
              } else if (match[1] === '^^') {
                retrievalMethod = 'inheritedData';
                $searchElement = $element.parent();
              }
              if (match[2] === '?') {
                optional = true;
              }
              value = null;
              if (elementControllers && retrievalMethod === 'data') {
                if (value = elementControllers[require]) {
                  value = value.instance;
                }
              }
              value = value || $searchElement[retrievalMethod]('$' + require + 'Controller');
              if (!value && !optional) {
                throw $compileMinErr('ctreq', 'Controller \'{0}\', required by directive \'{1}\', can\'t be found!', require, directiveName);
              }
              return value || null;
            } else if (isArray(require)) {
              value = [];
              forEach(require, function (require) {
                value.push(getControllers(directiveName, require, $element, elementControllers));
              });
            }
            return value;
          }
          function nodeLinkFn(childLinkFn, scope, linkNode, $rootElement, boundTranscludeFn) {
            var i, ii, linkFn, controller, isolateScope, elementControllers, transcludeFn, $element, attrs;
            if (compileNode === linkNode) {
              attrs = templateAttrs;
              $element = templateAttrs.$$element;
            } else {
              $element = jqLite(linkNode);
              attrs = new Attributes($element, templateAttrs);
            }
            if (newIsolateScopeDirective) {
              isolateScope = scope.$new(true);
            }
            if (boundTranscludeFn) {
              // track `boundTranscludeFn` so it can be unwrapped if `transcludeFn`
              // is later passed as `parentBoundTranscludeFn` to `publicLinkFn`
              transcludeFn = controllersBoundTransclude;
              transcludeFn.$$boundTransclude = boundTranscludeFn;
            }
            if (controllerDirectives) {
              // TODO: merge `controllers` and `elementControllers` into single object.
              controllers = {};
              elementControllers = {};
              forEach(controllerDirectives, function (directive) {
                var locals = {
                    $scope: directive === newIsolateScopeDirective || directive.$$isolateScope ? isolateScope : scope,
                    $element: $element,
                    $attrs: attrs,
                    $transclude: transcludeFn
                  }, controllerInstance;
                controller = directive.controller;
                if (controller == '@') {
                  controller = attrs[directive.name];
                }
                controllerInstance = $controller(controller, locals, true, directive.controllerAs);
                // For directives with element transclusion the element is a comment,
                // but jQuery .data doesn't support attaching data to comment nodes as it's hard to
                // clean up (http://bugs.jquery.com/ticket/8335).
                // Instead, we save the controllers for the element in a local hash and attach to .data
                // later, once we have the actual element.
                elementControllers[directive.name] = controllerInstance;
                if (!hasElementTranscludeDirective) {
                  $element.data('$' + directive.name + 'Controller', controllerInstance.instance);
                }
                controllers[directive.name] = controllerInstance;
              });
            }
            if (newIsolateScopeDirective) {
              compile.$$addScopeInfo($element, isolateScope, true, !(templateDirective && (templateDirective === newIsolateScopeDirective || templateDirective === newIsolateScopeDirective.$$originalDirective)));
              compile.$$addScopeClass($element, true);
              var isolateScopeController = controllers && controllers[newIsolateScopeDirective.name];
              var isolateBindingContext = isolateScope;
              if (isolateScopeController && isolateScopeController.identifier && newIsolateScopeDirective.bindToController === true) {
                isolateBindingContext = isolateScopeController.instance;
              }
              forEach(isolateScope.$$isolateBindings = newIsolateScopeDirective.$$isolateBindings, function (definition, scopeName) {
                var attrName = definition.attrName, optional = definition.optional, mode = definition.mode,
                  // @, =, or &
                  lastValue, parentGet, parentSet, compare;
                switch (mode) {
                case '@':
                  attrs.$observe(attrName, function (value) {
                    isolateBindingContext[scopeName] = value;
                  });
                  attrs.$$observers[attrName].$$scope = scope;
                  if (attrs[attrName]) {
                    // If the attribute has been provided then we trigger an interpolation to ensure
                    // the value is there for use in the link fn
                    isolateBindingContext[scopeName] = $interpolate(attrs[attrName])(scope);
                  }
                  break;
                case '=':
                  if (optional && !attrs[attrName]) {
                    return;
                  }
                  parentGet = $parse(attrs[attrName]);
                  if (parentGet.literal) {
                    compare = equals;
                  } else {
                    compare = function (a, b) {
                      return a === b || a !== a && b !== b;
                    };
                  }
                  parentSet = parentGet.assign || function () {
                    // reset the change, or we will throw this exception on every $digest
                    lastValue = isolateBindingContext[scopeName] = parentGet(scope);
                    throw $compileMinErr('nonassign', 'Expression \'{0}\' used with directive \'{1}\' is non-assignable!', attrs[attrName], newIsolateScopeDirective.name);
                  };
                  lastValue = isolateBindingContext[scopeName] = parentGet(scope);
                  var parentValueWatch = function parentValueWatch(parentValue) {
                    if (!compare(parentValue, isolateBindingContext[scopeName])) {
                      // we are out of sync and need to copy
                      if (!compare(parentValue, lastValue)) {
                        // parent changed and it has precedence
                        isolateBindingContext[scopeName] = parentValue;
                      } else {
                        // if the parent can be assigned then do so
                        parentSet(scope, parentValue = isolateBindingContext[scopeName]);
                      }
                    }
                    return lastValue = parentValue;
                  };
                  parentValueWatch.$stateful = true;
                  var unwatch;
                  if (definition.collection) {
                    unwatch = scope.$watchCollection(attrs[attrName], parentValueWatch);
                  } else {
                    unwatch = scope.$watch($parse(attrs[attrName], parentValueWatch), null, parentGet.literal);
                  }
                  isolateScope.$on('$destroy', unwatch);
                  break;
                case '&':
                  parentGet = $parse(attrs[attrName]);
                  isolateBindingContext[scopeName] = function (locals) {
                    return parentGet(scope, locals);
                  };
                  break;
                }
              });
            }
            if (controllers) {
              forEach(controllers, function (controller) {
                controller();
              });
              controllers = null;
            }
            // PRELINKING
            for (i = 0, ii = preLinkFns.length; i < ii; i++) {
              linkFn = preLinkFns[i];
              invokeLinkFn(linkFn, linkFn.isolateScope ? isolateScope : scope, $element, attrs, linkFn.require && getControllers(linkFn.directiveName, linkFn.require, $element, elementControllers), transcludeFn);
            }
            // RECURSION
            // We only pass the isolate scope, if the isolate directive has a template,
            // otherwise the child elements do not belong to the isolate directive.
            var scopeToChild = scope;
            if (newIsolateScopeDirective && (newIsolateScopeDirective.template || newIsolateScopeDirective.templateUrl === null)) {
              scopeToChild = isolateScope;
            }
            childLinkFn && childLinkFn(scopeToChild, linkNode.childNodes, undefined, boundTranscludeFn);
            // POSTLINKING
            for (i = postLinkFns.length - 1; i >= 0; i--) {
              linkFn = postLinkFns[i];
              invokeLinkFn(linkFn, linkFn.isolateScope ? isolateScope : scope, $element, attrs, linkFn.require && getControllers(linkFn.directiveName, linkFn.require, $element, elementControllers), transcludeFn);
            }
            // This is the function that is injected as `$transclude`.
            // Note: all arguments are optional!
            function controllersBoundTransclude(scope, cloneAttachFn, futureParentElement) {
              var transcludeControllers;
              // No scope passed in:
              if (!isScope(scope)) {
                futureParentElement = cloneAttachFn;
                cloneAttachFn = scope;
                scope = undefined;
              }
              if (hasElementTranscludeDirective) {
                transcludeControllers = elementControllers;
              }
              if (!futureParentElement) {
                futureParentElement = hasElementTranscludeDirective ? $element.parent() : $element;
              }
              return boundTranscludeFn(scope, cloneAttachFn, transcludeControllers, futureParentElement, scopeToChild);
            }
          }
        }
        function markDirectivesAsIsolate(directives) {
          // mark all directives as needing isolate scope.
          for (var j = 0, jj = directives.length; j < jj; j++) {
            directives[j] = inherit(directives[j], { $$isolateScope: true });
          }
        }
        /**
     * looks up the directive and decorates it with exception handling and proper parameters. We
     * call this the boundDirective.
     *
     * @param {string} name name of the directive to look up.
     * @param {string} location The directive must be found in specific format.
     *   String containing any of theses characters:
     *
     *   * `E`: element name
     *   * `A': attribute
     *   * `C`: class
     *   * `M`: comment
     * @returns {boolean} true if directive was added.
     */
        function addDirective(tDirectives, name, location, maxPriority, ignoreDirective, startAttrName, endAttrName) {
          if (name === ignoreDirective)
            return null;
          var match = null;
          if (hasDirectives.hasOwnProperty(name)) {
            for (var directive, directives = $injector.get(name + Suffix), i = 0, ii = directives.length; i < ii; i++) {
              try {
                directive = directives[i];
                if ((maxPriority === undefined || maxPriority > directive.priority) && directive.restrict.indexOf(location) != -1) {
                  if (startAttrName) {
                    directive = inherit(directive, {
                      $$start: startAttrName,
                      $$end: endAttrName
                    });
                  }
                  tDirectives.push(directive);
                  match = directive;
                }
              } catch (e) {
                $exceptionHandler(e);
              }
            }
          }
          return match;
        }
        /**
     * looks up the directive and returns true if it is a multi-element directive,
     * and therefore requires DOM nodes between -start and -end markers to be grouped
     * together.
     *
     * @param {string} name name of the directive to look up.
     * @returns true if directive was registered as multi-element.
     */
        function directiveIsMultiElement(name) {
          if (hasDirectives.hasOwnProperty(name)) {
            for (var directive, directives = $injector.get(name + Suffix), i = 0, ii = directives.length; i < ii; i++) {
              directive = directives[i];
              if (directive.multiElement) {
                return true;
              }
            }
          }
          return false;
        }
        /**
     * When the element is replaced with HTML template then the new attributes
     * on the template need to be merged with the existing attributes in the DOM.
     * The desired effect is to have both of the attributes present.
     *
     * @param {object} dst destination attributes (original DOM)
     * @param {object} src source attributes (from the directive template)
     */
        function mergeTemplateAttributes(dst, src) {
          var srcAttr = src.$attr, dstAttr = dst.$attr, $element = dst.$$element;
          // reapply the old attributes to the new element
          forEach(dst, function (value, key) {
            if (key.charAt(0) != '$') {
              if (src[key] && src[key] !== value) {
                value += (key === 'style' ? ';' : ' ') + src[key];
              }
              dst.$set(key, value, true, srcAttr[key]);
            }
          });
          // copy the new attributes on the old attrs object
          forEach(src, function (value, key) {
            if (key == 'class') {
              safeAddClass($element, value);
              dst['class'] = (dst['class'] ? dst['class'] + ' ' : '') + value;
            } else if (key == 'style') {
              $element.attr('style', $element.attr('style') + ';' + value);
              dst['style'] = (dst['style'] ? dst['style'] + ';' : '') + value;  // `dst` will never contain hasOwnProperty as DOM parser won't let it.
                                                                                // You will get an "InvalidCharacterError: DOM Exception 5" error if you
                                                                                // have an attribute like "has-own-property" or "data-has-own-property", etc.
            } else if (key.charAt(0) != '$' && !dst.hasOwnProperty(key)) {
              dst[key] = value;
              dstAttr[key] = srcAttr[key];
            }
          });
        }
        function compileTemplateUrl(directives, $compileNode, tAttrs, $rootElement, childTranscludeFn, preLinkFns, postLinkFns, previousCompileContext) {
          var linkQueue = [], afterTemplateNodeLinkFn, afterTemplateChildLinkFn, beforeTemplateCompileNode = $compileNode[0], origAsyncDirective = directives.shift(), derivedSyncDirective = inherit(origAsyncDirective, {
              templateUrl: null,
              transclude: null,
              replace: null,
              $$originalDirective: origAsyncDirective
            }), templateUrl = isFunction(origAsyncDirective.templateUrl) ? origAsyncDirective.templateUrl($compileNode, tAttrs) : origAsyncDirective.templateUrl, templateNamespace = origAsyncDirective.templateNamespace;
          $compileNode.empty();
          $templateRequest(templateUrl).then(function (content) {
            var compileNode, tempTemplateAttrs, $template, childBoundTranscludeFn;
            content = denormalizeTemplate(content);
            if (origAsyncDirective.replace) {
              if (jqLiteIsTextNode(content)) {
                $template = [];
              } else {
                $template = removeComments(wrapTemplate(templateNamespace, trim(content)));
              }
              compileNode = $template[0];
              if ($template.length != 1 || compileNode.nodeType !== NODE_TYPE_ELEMENT) {
                throw $compileMinErr('tplrt', 'Template for directive \'{0}\' must have exactly one root element. {1}', origAsyncDirective.name, templateUrl);
              }
              tempTemplateAttrs = { $attr: {} };
              replaceWith($rootElement, $compileNode, compileNode);
              var templateDirectives = collectDirectives(compileNode, [], tempTemplateAttrs);
              if (isObject(origAsyncDirective.scope)) {
                markDirectivesAsIsolate(templateDirectives);
              }
              directives = templateDirectives.concat(directives);
              mergeTemplateAttributes(tAttrs, tempTemplateAttrs);
            } else {
              compileNode = beforeTemplateCompileNode;
              $compileNode.html(content);
            }
            directives.unshift(derivedSyncDirective);
            afterTemplateNodeLinkFn = applyDirectivesToNode(directives, compileNode, tAttrs, childTranscludeFn, $compileNode, origAsyncDirective, preLinkFns, postLinkFns, previousCompileContext);
            forEach($rootElement, function (node, i) {
              if (node == compileNode) {
                $rootElement[i] = $compileNode[0];
              }
            });
            afterTemplateChildLinkFn = compileNodes($compileNode[0].childNodes, childTranscludeFn);
            while (linkQueue.length) {
              var scope = linkQueue.shift(), beforeTemplateLinkNode = linkQueue.shift(), linkRootElement = linkQueue.shift(), boundTranscludeFn = linkQueue.shift(), linkNode = $compileNode[0];
              if (scope.$$destroyed)
                continue;
              if (beforeTemplateLinkNode !== beforeTemplateCompileNode) {
                var oldClasses = beforeTemplateLinkNode.className;
                if (!(previousCompileContext.hasElementTranscludeDirective && origAsyncDirective.replace)) {
                  // it was cloned therefore we have to clone as well.
                  linkNode = jqLiteClone(compileNode);
                }
                replaceWith(linkRootElement, jqLite(beforeTemplateLinkNode), linkNode);
                // Copy in CSS classes from original node
                safeAddClass(jqLite(linkNode), oldClasses);
              }
              if (afterTemplateNodeLinkFn.transcludeOnThisElement) {
                childBoundTranscludeFn = createBoundTranscludeFn(scope, afterTemplateNodeLinkFn.transclude, boundTranscludeFn);
              } else {
                childBoundTranscludeFn = boundTranscludeFn;
              }
              afterTemplateNodeLinkFn(afterTemplateChildLinkFn, scope, linkNode, $rootElement, childBoundTranscludeFn);
            }
            linkQueue = null;
          });
          return function delayedNodeLinkFn(ignoreChildLinkFn, scope, node, rootElement, boundTranscludeFn) {
            var childBoundTranscludeFn = boundTranscludeFn;
            if (scope.$$destroyed)
              return;
            if (linkQueue) {
              linkQueue.push(scope, node, rootElement, childBoundTranscludeFn);
            } else {
              if (afterTemplateNodeLinkFn.transcludeOnThisElement) {
                childBoundTranscludeFn = createBoundTranscludeFn(scope, afterTemplateNodeLinkFn.transclude, boundTranscludeFn);
              }
              afterTemplateNodeLinkFn(afterTemplateChildLinkFn, scope, node, rootElement, childBoundTranscludeFn);
            }
          };
        }
        /**
     * Sorting function for bound directives.
     */
        function byPriority(a, b) {
          var diff = b.priority - a.priority;
          if (diff !== 0)
            return diff;
          if (a.name !== b.name)
            return a.name < b.name ? -1 : 1;
          return a.index - b.index;
        }
        function assertNoDuplicate(what, previousDirective, directive, element) {
          if (previousDirective) {
            throw $compileMinErr('multidir', 'Multiple directives [{0}, {1}] asking for {2} on: {3}', previousDirective.name, directive.name, what, startingTag(element));
          }
        }
        function addTextInterpolateDirective(directives, text) {
          var interpolateFn = $interpolate(text, true);
          if (interpolateFn) {
            directives.push({
              priority: 0,
              compile: function textInterpolateCompileFn(templateNode) {
                var templateNodeParent = templateNode.parent(), hasCompileParent = !!templateNodeParent.length;
                // When transcluding a template that has bindings in the root
                // we don't have a parent and thus need to add the class during linking fn.
                if (hasCompileParent)
                  compile.$$addBindingClass(templateNodeParent);
                return function textInterpolateLinkFn(scope, node) {
                  var parent = node.parent();
                  if (!hasCompileParent)
                    compile.$$addBindingClass(parent);
                  compile.$$addBindingInfo(parent, interpolateFn.expressions);
                  scope.$watch(interpolateFn, function interpolateFnWatchAction(value) {
                    node[0].nodeValue = value;
                  });
                };
              }
            });
          }
        }
        function wrapTemplate(type, template) {
          type = lowercase(type || 'html');
          switch (type) {
          case 'svg':
          case 'math':
            var wrapper = document.createElement('div');
            wrapper.innerHTML = '<' + type + '>' + template + '</' + type + '>';
            return wrapper.childNodes[0].childNodes;
          default:
            return template;
          }
        }
        function getTrustedContext(node, attrNormalizedName) {
          if (attrNormalizedName == 'srcdoc') {
            return $sce.HTML;
          }
          var tag = nodeName_(node);
          // maction[xlink:href] can source SVG.  It's not limited to <maction>.
          if (attrNormalizedName == 'xlinkHref' || tag == 'form' && attrNormalizedName == 'action' || tag != 'img' && (attrNormalizedName == 'src' || attrNormalizedName == 'ngSrc')) {
            return $sce.RESOURCE_URL;
          }
        }
        function addAttrInterpolateDirective(node, directives, value, name, allOrNothing) {
          var trustedContext = getTrustedContext(node, name);
          allOrNothing = ALL_OR_NOTHING_ATTRS[name] || allOrNothing;
          var interpolateFn = $interpolate(value, true, trustedContext, allOrNothing);
          // no interpolation found -> ignore
          if (!interpolateFn)
            return;
          if (name === 'multiple' && nodeName_(node) === 'select') {
            throw $compileMinErr('selmulti', 'Binding to the \'multiple\' attribute is not supported. Element: {0}', startingTag(node));
          }
          directives.push({
            priority: 100,
            compile: function () {
              return {
                pre: function attrInterpolatePreLinkFn(scope, element, attr) {
                  var $$observers = attr.$$observers || (attr.$$observers = {});
                  if (EVENT_HANDLER_ATTR_REGEXP.test(name)) {
                    throw $compileMinErr('nodomevents', 'Interpolations for HTML DOM event attributes are disallowed.  Please use the ' + 'ng- versions (such as ng-click instead of onclick) instead.');
                  }
                  // If the attribute has changed since last $interpolate()ed
                  var newValue = attr[name];
                  if (newValue !== value) {
                    // we need to interpolate again since the attribute value has been updated
                    // (e.g. by another directive's compile function)
                    // ensure unset/empty values make interpolateFn falsy
                    interpolateFn = newValue && $interpolate(newValue, true, trustedContext, allOrNothing);
                    value = newValue;
                  }
                  // if attribute was updated so that there is no interpolation going on we don't want to
                  // register any observers
                  if (!interpolateFn)
                    return;
                  // initialize attr object so that it's ready in case we need the value for isolate
                  // scope initialization, otherwise the value would not be available from isolate
                  // directive's linking fn during linking phase
                  attr[name] = interpolateFn(scope);
                  ($$observers[name] || ($$observers[name] = [])).$$inter = true;
                  (attr.$$observers && attr.$$observers[name].$$scope || scope).$watch(interpolateFn, function interpolateFnWatchAction(newValue, oldValue) {
                    //special case for class attribute addition + removal
                    //so that class changes can tap into the animation
                    //hooks provided by the $animate service. Be sure to
                    //skip animations when the first digest occurs (when
                    //both the new and the old values are the same) since
                    //the CSS classes are the non-interpolated values
                    if (name === 'class' && newValue != oldValue) {
                      attr.$updateClass(newValue, oldValue);
                    } else {
                      attr.$set(name, newValue);
                    }
                  });
                }
              };
            }
          });
        }
        /**
     * This is a special jqLite.replaceWith, which can replace items which
     * have no parents, provided that the containing jqLite collection is provided.
     *
     * @param {JqLite=} $rootElement The root of the compile tree. Used so that we can replace nodes
     *                               in the root of the tree.
     * @param {JqLite} elementsToRemove The jqLite element which we are going to replace. We keep
     *                                  the shell, but replace its DOM node reference.
     * @param {Node} newNode The new DOM node.
     */
        function replaceWith($rootElement, elementsToRemove, newNode) {
          var firstElementToRemove = elementsToRemove[0], removeCount = elementsToRemove.length, parent = firstElementToRemove.parentNode, i, ii;
          if ($rootElement) {
            for (i = 0, ii = $rootElement.length; i < ii; i++) {
              if ($rootElement[i] == firstElementToRemove) {
                $rootElement[i++] = newNode;
                for (var j = i, j2 = j + removeCount - 1, jj = $rootElement.length; j < jj; j++, j2++) {
                  if (j2 < jj) {
                    $rootElement[j] = $rootElement[j2];
                  } else {
                    delete $rootElement[j];
                  }
                }
                $rootElement.length -= removeCount - 1;
                // If the replaced element is also the jQuery .context then replace it
                // .context is a deprecated jQuery api, so we should set it only when jQuery set it
                // http://api.jquery.com/context/
                if ($rootElement.context === firstElementToRemove) {
                  $rootElement.context = newNode;
                }
                break;
              }
            }
          }
          if (parent) {
            parent.replaceChild(newNode, firstElementToRemove);
          }
          // TODO(perf): what's this document fragment for? is it needed? can we at least reuse it?
          var fragment = document.createDocumentFragment();
          fragment.appendChild(firstElementToRemove);
          // Copy over user data (that includes Angular's $scope etc.). Don't copy private
          // data here because there's no public interface in jQuery to do that and copying over
          // event listeners (which is the main use of private data) wouldn't work anyway.
          jqLite(newNode).data(jqLite(firstElementToRemove).data());
          // Remove data of the replaced element. We cannot just call .remove()
          // on the element it since that would deallocate scope that is needed
          // for the new node. Instead, remove the data "manually".
          if (!jQuery) {
            delete jqLite.cache[firstElementToRemove[jqLite.expando]];
          } else {
            // jQuery 2.x doesn't expose the data storage. Use jQuery.cleanData to clean up after
            // the replaced element. The cleanData version monkey-patched by Angular would cause
            // the scope to be trashed and we do need the very same scope to work with the new
            // element. However, we cannot just cache the non-patched version and use it here as
            // that would break if another library patches the method after Angular does (one
            // example is jQuery UI). Instead, set a flag indicating scope destroying should be
            // skipped this one time.
            skipDestroyOnNextJQueryCleanData = true;
            jQuery.cleanData([firstElementToRemove]);
          }
          for (var k = 1, kk = elementsToRemove.length; k < kk; k++) {
            var element = elementsToRemove[k];
            jqLite(element).remove();
            // must do this way to clean up expando
            fragment.appendChild(element);
            delete elementsToRemove[k];
          }
          elementsToRemove[0] = newNode;
          elementsToRemove.length = 1;
        }
        function cloneAndAnnotateFn(fn, annotation) {
          return extend(function () {
            return fn.apply(null, arguments);
          }, fn, annotation);
        }
        function invokeLinkFn(linkFn, scope, $element, attrs, controllers, transcludeFn) {
          try {
            linkFn(scope, $element, attrs, controllers, transcludeFn);
          } catch (e) {
            $exceptionHandler(e, startingTag($element));
          }
        }
      }
    ];
  }
  var PREFIX_REGEXP = /^((?:x|data)[\:\-_])/i;
  /**
 * Converts all accepted directives format into proper directive name.
 * @param name Name to normalize
 */
  function directiveNormalize(name) {
    return camelCase(name.replace(PREFIX_REGEXP, ''));
  }
  /**
 * @ngdoc type
 * @name $compile.directive.Attributes
 *
 * @description
 * A shared object between directive compile / linking functions which contains normalized DOM
 * element attributes. The values reflect current binding state `{{ }}`. The normalization is
 * needed since all of these are treated as equivalent in Angular:
 *
 * ```
 *    <span ng:bind="a" ng-bind="a" data-ng-bind="a" x-ng-bind="a">
 * ```
 */
  /**
 * @ngdoc property
 * @name $compile.directive.Attributes#$attr
 *
 * @description
 * A map of DOM element attribute names to the normalized name. This is
 * needed to do reverse lookup from normalized name back to actual name.
 */
  /**
 * @ngdoc method
 * @name $compile.directive.Attributes#$set
 * @kind function
 *
 * @description
 * Set DOM element attribute value.
 *
 *
 * @param {string} name Normalized element attribute name of the property to modify. The name is
 *          reverse-translated using the {@link ng.$compile.directive.Attributes#$attr $attr}
 *          property to the original name.
 * @param {string} value Value to set the attribute to. The value can be an interpolated string.
 */
  /**
 * Closure compiler type information
 */
  function nodesetLinkingFn(scope, nodeList, rootElement, boundTranscludeFn) {
  }
  function directiveLinkingFn(nodesetLinkingFn, scope, node, rootElement, boundTranscludeFn) {
  }
  function tokenDifference(str1, str2) {
    var values = '', tokens1 = str1.split(/\s+/), tokens2 = str2.split(/\s+/);
    outer:
      for (var i = 0; i < tokens1.length; i++) {
        var token = tokens1[i];
        for (var j = 0; j < tokens2.length; j++) {
          if (token == tokens2[j])
            continue outer;
        }
        values += (values.length > 0 ? ' ' : '') + token;
      }
    return values;
  }
  function removeComments(jqNodes) {
    jqNodes = jqLite(jqNodes);
    var i = jqNodes.length;
    if (i <= 1) {
      return jqNodes;
    }
    while (i--) {
      var node = jqNodes[i];
      if (node.nodeType === NODE_TYPE_COMMENT) {
        splice.call(jqNodes, i, 1);
      }
    }
    return jqNodes;
  }
  var $controllerMinErr = minErr('$controller');
  /**
 * @ngdoc provider
 * @name $controllerProvider
 * @description
 * The {@link ng.$controller $controller service} is used by Angular to create new
 * controllers.
 *
 * This provider allows controller registration via the
 * {@link ng.$controllerProvider#register register} method.
 */
  function $ControllerProvider() {
    var controllers = {}, globals = false, CNTRL_REG = /^(\S+)(\s+as\s+(\w+))?$/;
    /**
   * @ngdoc method
   * @name $controllerProvider#register
   * @param {string|Object} name Controller name, or an object map of controllers where the keys are
   *    the names and the values are the constructors.
   * @param {Function|Array} constructor Controller constructor fn (optionally decorated with DI
   *    annotations in the array notation).
   */
    this.register = function (name, constructor) {
      assertNotHasOwnProperty(name, 'controller');
      if (isObject(name)) {
        extend(controllers, name);
      } else {
        controllers[name] = constructor;
      }
    };
    /**
   * @ngdoc method
   * @name $controllerProvider#allowGlobals
   * @description If called, allows `$controller` to find controller constructors on `window`
   */
    this.allowGlobals = function () {
      globals = true;
    };
    this.$get = [
      '$injector',
      '$window',
      function ($injector, $window) {
        /**
     * @ngdoc service
     * @name $controller
     * @requires $injector
     *
     * @param {Function|string} constructor If called with a function then it's considered to be the
     *    controller constructor function. Otherwise it's considered to be a string which is used
     *    to retrieve the controller constructor using the following steps:
     *
     *    * check if a controller with given name is registered via `$controllerProvider`
     *    * check if evaluating the string on the current scope returns a constructor
     *    * if $controllerProvider#allowGlobals, check `window[constructor]` on the global
     *      `window` object (not recommended)
     *
     *    The string can use the `controller as property` syntax, where the controller instance is published
     *    as the specified property on the `scope`; the `scope` must be injected into `locals` param for this
     *    to work correctly.
     *
     * @param {Object} locals Injection locals for Controller.
     * @return {Object} Instance of given controller.
     *
     * @description
     * `$controller` service is responsible for instantiating controllers.
     *
     * It's just a simple call to {@link auto.$injector $injector}, but extracted into
     * a service, so that one can override this service with [BC version](https://gist.github.com/1649788).
     */
        return function (expression, locals, later, ident) {
          // PRIVATE API:
          //   param `later` --- indicates that the controller's constructor is invoked at a later time.
          //                     If true, $controller will allocate the object with the correct
          //                     prototype chain, but will not invoke the controller until a returned
          //                     callback is invoked.
          //   param `ident` --- An optional label which overrides the label parsed from the controller
          //                     expression, if any.
          var instance, match, constructor, identifier;
          later = later === true;
          if (ident && isString(ident)) {
            identifier = ident;
          }
          if (isString(expression)) {
            match = expression.match(CNTRL_REG);
            if (!match) {
              throw $controllerMinErr('ctrlfmt', 'Badly formed controller string \'{0}\'. ' + 'Must match `__name__ as __id__` or `__name__`.', expression);
            }
            constructor = match[1], identifier = identifier || match[3];
            expression = controllers.hasOwnProperty(constructor) ? controllers[constructor] : getter(locals.$scope, constructor, true) || (globals ? getter($window, constructor, true) : undefined);
            assertArgFn(expression, constructor, true);
          }
          if (later) {
            // Instantiate controller later:
            // This machinery is used to create an instance of the object before calling the
            // controller's constructor itself.
            //
            // This allows properties to be added to the controller before the constructor is
            // invoked. Primarily, this is used for isolate scope bindings in $compile.
            //
            // This feature is not intended for use by applications, and is thus not documented
            // publicly.
            // Object creation: http://jsperf.com/create-constructor/2
            var controllerPrototype = (isArray(expression) ? expression[expression.length - 1] : expression).prototype;
            instance = Object.create(controllerPrototype || null);
            if (identifier) {
              addIdentifier(locals, identifier, instance, constructor || expression.name);
            }
            return extend(function () {
              $injector.invoke(expression, instance, locals, constructor);
              return instance;
            }, {
              instance: instance,
              identifier: identifier
            });
          }
          instance = $injector.instantiate(expression, locals, constructor);
          if (identifier) {
            addIdentifier(locals, identifier, instance, constructor || expression.name);
          }
          return instance;
        };
        function addIdentifier(locals, identifier, instance, name) {
          if (!(locals && isObject(locals.$scope))) {
            throw minErr('$controller')('noscp', 'Cannot export controller \'{0}\' as \'{1}\'! No $scope object provided via `locals`.', name, identifier);
          }
          locals.$scope[identifier] = instance;
        }
      }
    ];
  }
  /**
 * @ngdoc service
 * @name $document
 * @requires $window
 *
 * @description
 * A {@link angular.element jQuery or jqLite} wrapper for the browser's `window.document` object.
 *
 * @example
   <example module="documentExample">
     <file name="index.html">
       <div ng-controller="ExampleController">
         <p>$document title: <b ng-bind="title"></b></p>
         <p>window.document title: <b ng-bind="windowTitle"></b></p>
       </div>
     </file>
     <file name="script.js">
       angular.module('documentExample', [])
         .controller('ExampleController', ['$scope', '$document', function($scope, $document) {
           $scope.title = $document[0].title;
           $scope.windowTitle = angular.element(window.document)[0].title;
         }]);
     </file>
   </example>
 */
  function $DocumentProvider() {
    this.$get = [
      '$window',
      function (window) {
        return jqLite(window.document);
      }
    ];
  }
  /**
 * @ngdoc service
 * @name $exceptionHandler
 * @requires ng.$log
 *
 * @description
 * Any uncaught exception in angular expressions is delegated to this service.
 * The default implementation simply delegates to `$log.error` which logs it into
 * the browser console.
 *
 * In unit tests, if `angular-mocks.js` is loaded, this service is overridden by
 * {@link ngMock.$exceptionHandler mock $exceptionHandler} which aids in testing.
 *
 * ## Example:
 *
 * ```js
 *   angular.module('exceptionOverride', []).factory('$exceptionHandler', function() {
 *     return function(exception, cause) {
 *       exception.message += ' (caused by "' + cause + '")';
 *       throw exception;
 *     };
 *   });
 * ```
 *
 * This example will override the normal action of `$exceptionHandler`, to make angular
 * exceptions fail hard when they happen, instead of just logging to the console.
 *
 * <hr />
 * Note, that code executed in event-listeners (even those registered using jqLite's `on`/`bind`
 * methods) does not delegate exceptions to the {@link ng.$exceptionHandler $exceptionHandler}
 * (unless executed during a digest).
 *
 * If you wish, you can manually delegate exceptions, e.g.
 * `try { ... } catch(e) { $exceptionHandler(e); }`
 *
 * @param {Error} exception Exception associated with the error.
 * @param {string=} cause optional information about the context in which
 *       the error was thrown.
 *
 */
  function $ExceptionHandlerProvider() {
    this.$get = [
      '$log',
      function ($log) {
        return function (exception, cause) {
          $log.error.apply($log, arguments);
        };
      }
    ];
  }
  var APPLICATION_JSON = 'application/json';
  var CONTENT_TYPE_APPLICATION_JSON = { 'Content-Type': APPLICATION_JSON + ';charset=utf-8' };
  var JSON_START = /^\[|^\{(?!\{)/;
  var JSON_ENDS = {
      '[': /]$/,
      '{': /}$/
    };
  var JSON_PROTECTION_PREFIX = /^\)\]\}',?\n/;
  function defaultHttpResponseTransform(data, headers) {
    if (isString(data)) {
      // Strip json vulnerability protection prefix and trim whitespace
      var tempData = data.replace(JSON_PROTECTION_PREFIX, '').trim();
      if (tempData) {
        var contentType = headers('Content-Type');
        if (contentType && contentType.indexOf(APPLICATION_JSON) === 0 || isJsonLike(tempData)) {
          data = fromJson(tempData);
        }
      }
    }
    return data;
  }
  function isJsonLike(str) {
    var jsonStart = str.match(JSON_START);
    return jsonStart && JSON_ENDS[jsonStart[0]].test(str);
  }
  /**
 * Parse headers into key value object
 *
 * @param {string} headers Raw headers as a string
 * @returns {Object} Parsed headers as key value object
 */
  function parseHeaders(headers) {
    var parsed = createMap(), key, val, i;
    if (!headers)
      return parsed;
    forEach(headers.split('\n'), function (line) {
      i = line.indexOf(':');
      key = lowercase(trim(line.substr(0, i)));
      val = trim(line.substr(i + 1));
      if (key) {
        parsed[key] = parsed[key] ? parsed[key] + ', ' + val : val;
      }
    });
    return parsed;
  }
  /**
 * Returns a function that provides access to parsed headers.
 *
 * Headers are lazy parsed when first requested.
 * @see parseHeaders
 *
 * @param {(string|Object)} headers Headers to provide access to.
 * @returns {function(string=)} Returns a getter function which if called with:
 *
 *   - if called with single an argument returns a single header value or null
 *   - if called with no arguments returns an object containing all headers.
 */
  function headersGetter(headers) {
    var headersObj = isObject(headers) ? headers : undefined;
    return function (name) {
      if (!headersObj)
        headersObj = parseHeaders(headers);
      if (name) {
        var value = headersObj[lowercase(name)];
        if (value === void 0) {
          value = null;
        }
        return value;
      }
      return headersObj;
    };
  }
  /**
 * Chain all given functions
 *
 * This function is used for both request and response transforming
 *
 * @param {*} data Data to transform.
 * @param {function(string=)} headers HTTP headers getter fn.
 * @param {number} status HTTP status code of the response.
 * @param {(Function|Array.<Function>)} fns Function or an array of functions.
 * @returns {*} Transformed data.
 */
  function transformData(data, headers, status, fns) {
    if (isFunction(fns))
      return fns(data, headers, status);
    forEach(fns, function (fn) {
      data = fn(data, headers, status);
    });
    return data;
  }
  function isSuccess(status) {
    return 200 <= status && status < 300;
  }
  /**
 * @ngdoc provider
 * @name $httpProvider
 * @description
 * Use `$httpProvider` to change the default behavior of the {@link ng.$http $http} service.
 * */
  function $HttpProvider() {
    /**
   * @ngdoc property
   * @name $httpProvider#defaults
   * @description
   *
   * Object containing default values for all {@link ng.$http $http} requests.
   *
   * - **`defaults.cache`** - {Object} - an object built with {@link ng.$cacheFactory `$cacheFactory`}
   * that will provide the cache for all requests who set their `cache` property to `true`.
   * If you set the `default.cache = false` then only requests that specify their own custom
   * cache object will be cached. See {@link $http#caching $http Caching} for more information.
   *
   * - **`defaults.xsrfCookieName`** - {string} - Name of cookie containing the XSRF token.
   * Defaults value is `'XSRF-TOKEN'`.
   *
   * - **`defaults.xsrfHeaderName`** - {string} - Name of HTTP header to populate with the
   * XSRF token. Defaults value is `'X-XSRF-TOKEN'`.
   *
   * - **`defaults.headers`** - {Object} - Default headers for all $http requests.
   * Refer to {@link ng.$http#setting-http-headers $http} for documentation on
   * setting default headers.
   *     - **`defaults.headers.common`**
   *     - **`defaults.headers.post`**
   *     - **`defaults.headers.put`**
   *     - **`defaults.headers.patch`**
   *
   **/
    var defaults = this.defaults = {
        transformResponse: [defaultHttpResponseTransform],
        transformRequest: [function (d) {
            return isObject(d) && !isFile(d) && !isBlob(d) && !isFormData(d) ? toJson(d) : d;
          }],
        headers: {
          common: { 'Accept': 'application/json, text/plain, */*' },
          post: shallowCopy(CONTENT_TYPE_APPLICATION_JSON),
          put: shallowCopy(CONTENT_TYPE_APPLICATION_JSON),
          patch: shallowCopy(CONTENT_TYPE_APPLICATION_JSON)
        },
        xsrfCookieName: 'XSRF-TOKEN',
        xsrfHeaderName: 'X-XSRF-TOKEN'
      };
    var useApplyAsync = false;
    /**
   * @ngdoc method
   * @name $httpProvider#useApplyAsync
   * @description
   *
   * Configure $http service to combine processing of multiple http responses received at around
   * the same time via {@link ng.$rootScope.Scope#$applyAsync $rootScope.$applyAsync}. This can result in
   * significant performance improvement for bigger applications that make many HTTP requests
   * concurrently (common during application bootstrap).
   *
   * Defaults to false. If no value is specifed, returns the current configured value.
   *
   * @param {boolean=} value If true, when requests are loaded, they will schedule a deferred
   *    "apply" on the next tick, giving time for subsequent requests in a roughly ~10ms window
   *    to load and share the same digest cycle.
   *
   * @returns {boolean|Object} If a value is specified, returns the $httpProvider for chaining.
   *    otherwise, returns the current configured value.
   **/
    this.useApplyAsync = function (value) {
      if (isDefined(value)) {
        useApplyAsync = !!value;
        return this;
      }
      return useApplyAsync;
    };
    /**
   * @ngdoc property
   * @name $httpProvider#interceptors
   * @description
   *
   * Array containing service factories for all synchronous or asynchronous {@link ng.$http $http}
   * pre-processing of request or postprocessing of responses.
   *
   * These service factories are ordered by request, i.e. they are applied in the same order as the
   * array, on request, but reverse order, on response.
   *
   * {@link ng.$http#interceptors Interceptors detailed info}
   **/
    var interceptorFactories = this.interceptors = [];
    this.$get = [
      '$httpBackend',
      '$browser',
      '$cacheFactory',
      '$rootScope',
      '$q',
      '$injector',
      function ($httpBackend, $browser, $cacheFactory, $rootScope, $q, $injector) {
        var defaultCache = $cacheFactory('$http');
        /**
     * Interceptors stored in reverse order. Inner interceptors before outer interceptors.
     * The reversal is needed so that we can build up the interception chain around the
     * server request.
     */
        var reversedInterceptors = [];
        forEach(interceptorFactories, function (interceptorFactory) {
          reversedInterceptors.unshift(isString(interceptorFactory) ? $injector.get(interceptorFactory) : $injector.invoke(interceptorFactory));
        });
        /**
     * @ngdoc service
     * @kind function
     * @name $http
     * @requires ng.$httpBackend
     * @requires $cacheFactory
     * @requires $rootScope
     * @requires $q
     * @requires $injector
     *
     * @description
     * The `$http` service is a core Angular service that facilitates communication with the remote
     * HTTP servers via the browser's [XMLHttpRequest](https://developer.mozilla.org/en/xmlhttprequest)
     * object or via [JSONP](http://en.wikipedia.org/wiki/JSONP).
     *
     * For unit testing applications that use `$http` service, see
     * {@link ngMock.$httpBackend $httpBackend mock}.
     *
     * For a higher level of abstraction, please check out the {@link ngResource.$resource
     * $resource} service.
     *
     * The $http API is based on the {@link ng.$q deferred/promise APIs} exposed by
     * the $q service. While for simple usage patterns this doesn't matter much, for advanced usage
     * it is important to familiarize yourself with these APIs and the guarantees they provide.
     *
     *
     * ## General usage
     * The `$http` service is a function which takes a single argument — a configuration object —
     * that is used to generate an HTTP request and returns  a {@link ng.$q promise}
     * with two $http specific methods: `success` and `error`.
     *
     * ```js
     *   // Simple GET request example :
     *   $http.get('/someUrl').
     *     success(function(data, status, headers, config) {
     *       // this callback will be called asynchronously
     *       // when the response is available
     *     }).
     *     error(function(data, status, headers, config) {
     *       // called asynchronously if an error occurs
     *       // or server returns response with an error status.
     *     });
     * ```
     *
     * ```js
     *   // Simple POST request example (passing data) :
     *   $http.post('/someUrl', {msg:'hello word!'}).
     *     success(function(data, status, headers, config) {
     *       // this callback will be called asynchronously
     *       // when the response is available
     *     }).
     *     error(function(data, status, headers, config) {
     *       // called asynchronously if an error occurs
     *       // or server returns response with an error status.
     *     });
     * ```
     *
     *
     * Since the returned value of calling the $http function is a `promise`, you can also use
     * the `then` method to register callbacks, and these callbacks will receive a single argument –
     * an object representing the response. See the API signature and type info below for more
     * details.
     *
     * A response status code between 200 and 299 is considered a success status and
     * will result in the success callback being called. Note that if the response is a redirect,
     * XMLHttpRequest will transparently follow it, meaning that the error callback will not be
     * called for such responses.
     *
     * ## Writing Unit Tests that use $http
     * When unit testing (using {@link ngMock ngMock}), it is necessary to call
     * {@link ngMock.$httpBackend#flush $httpBackend.flush()} to flush each pending
     * request using trained responses.
     *
     * ```
     * $httpBackend.expectGET(...);
     * $http.get(...);
     * $httpBackend.flush();
     * ```
     *
     * ## Shortcut methods
     *
     * Shortcut methods are also available. All shortcut methods require passing in the URL, and
     * request data must be passed in for POST/PUT requests.
     *
     * ```js
     *   $http.get('/someUrl').success(successCallback);
     *   $http.post('/someUrl', data).success(successCallback);
     * ```
     *
     * Complete list of shortcut methods:
     *
     * - {@link ng.$http#get $http.get}
     * - {@link ng.$http#head $http.head}
     * - {@link ng.$http#post $http.post}
     * - {@link ng.$http#put $http.put}
     * - {@link ng.$http#delete $http.delete}
     * - {@link ng.$http#jsonp $http.jsonp}
     * - {@link ng.$http#patch $http.patch}
     *
     *
     * ## Setting HTTP Headers
     *
     * The $http service will automatically add certain HTTP headers to all requests. These defaults
     * can be fully configured by accessing the `$httpProvider.defaults.headers` configuration
     * object, which currently contains this default configuration:
     *
     * - `$httpProvider.defaults.headers.common` (headers that are common for all requests):
     *   - `Accept: application/json, text/plain, * / *`
     * - `$httpProvider.defaults.headers.post`: (header defaults for POST requests)
     *   - `Content-Type: application/json`
     * - `$httpProvider.defaults.headers.put` (header defaults for PUT requests)
     *   - `Content-Type: application/json`
     *
     * To add or overwrite these defaults, simply add or remove a property from these configuration
     * objects. To add headers for an HTTP method other than POST or PUT, simply add a new object
     * with the lowercased HTTP method name as the key, e.g.
     * `$httpProvider.defaults.headers.get = { 'My-Header' : 'value' }.
     *
     * The defaults can also be set at runtime via the `$http.defaults` object in the same
     * fashion. For example:
     *
     * ```
     * module.run(function($http) {
     *   $http.defaults.headers.common.Authorization = 'Basic YmVlcDpib29w'
     * });
     * ```
     *
     * In addition, you can supply a `headers` property in the config object passed when
     * calling `$http(config)`, which overrides the defaults without changing them globally.
     *
     * To explicitly remove a header automatically added via $httpProvider.defaults.headers on a per request basis,
     * Use the `headers` property, setting the desired header to `undefined`. For example:
     *
     * ```js
     * var req = {
     *  method: 'POST',
     *  url: 'http://example.com',
     *  headers: {
     *    'Content-Type': undefined
     *  },
     *  data: { test: 'test' }
     * }
     *
     * $http(req).success(function(){...}).error(function(){...});
     * ```
     *
     * ## Transforming Requests and Responses
     *
     * Both requests and responses can be transformed using transformation functions: `transformRequest`
     * and `transformResponse`. These properties can be a single function that returns
     * the transformed value (`function(data, headersGetter, status)`) or an array of such transformation functions,
     * which allows you to `push` or `unshift` a new transformation function into the transformation chain.
     *
     * ### Default Transformations
     *
     * The `$httpProvider` provider and `$http` service expose `defaults.transformRequest` and
     * `defaults.transformResponse` properties. If a request does not provide its own transformations
     * then these will be applied.
     *
     * You can augment or replace the default transformations by modifying these properties by adding to or
     * replacing the array.
     *
     * Angular provides the following default transformations:
     *
     * Request transformations (`$httpProvider.defaults.transformRequest` and `$http.defaults.transformRequest`):
     *
     * - If the `data` property of the request configuration object contains an object, serialize it
     *   into JSON format.
     *
     * Response transformations (`$httpProvider.defaults.transformResponse` and `$http.defaults.transformResponse`):
     *
     *  - If XSRF prefix is detected, strip it (see Security Considerations section below).
     *  - If JSON response is detected, deserialize it using a JSON parser.
     *
     *
     * ### Overriding the Default Transformations Per Request
     *
     * If you wish override the request/response transformations only for a single request then provide
     * `transformRequest` and/or `transformResponse` properties on the configuration object passed
     * into `$http`.
     *
     * Note that if you provide these properties on the config object the default transformations will be
     * overwritten. If you wish to augment the default transformations then you must include them in your
     * local transformation array.
     *
     * The following code demonstrates adding a new response transformation to be run after the default response
     * transformations have been run.
     *
     * ```js
     * function appendTransform(defaults, transform) {
     *
     *   // We can't guarantee that the default transformation is an array
     *   defaults = angular.isArray(defaults) ? defaults : [defaults];
     *
     *   // Append the new transformation to the defaults
     *   return defaults.concat(transform);
     * }
     *
     * $http({
     *   url: '...',
     *   method: 'GET',
     *   transformResponse: appendTransform($http.defaults.transformResponse, function(value) {
     *     return doTransform(value);
     *   })
     * });
     * ```
     *
     *
     * ## Caching
     *
     * To enable caching, set the request configuration `cache` property to `true` (to use default
     * cache) or to a custom cache object (built with {@link ng.$cacheFactory `$cacheFactory`}).
     * When the cache is enabled, `$http` stores the response from the server in the specified
     * cache. The next time the same request is made, the response is served from the cache without
     * sending a request to the server.
     *
     * Note that even if the response is served from cache, delivery of the data is asynchronous in
     * the same way that real requests are.
     *
     * If there are multiple GET requests for the same URL that should be cached using the same
     * cache, but the cache is not populated yet, only one request to the server will be made and
     * the remaining requests will be fulfilled using the response from the first request.
     *
     * You can change the default cache to a new object (built with
     * {@link ng.$cacheFactory `$cacheFactory`}) by updating the
     * {@link ng.$http#defaults `$http.defaults.cache`} property. All requests who set
     * their `cache` property to `true` will now use this cache object.
     *
     * If you set the default cache to `false` then only requests that specify their own custom
     * cache object will be cached.
     *
     * ## Interceptors
     *
     * Before you start creating interceptors, be sure to understand the
     * {@link ng.$q $q and deferred/promise APIs}.
     *
     * For purposes of global error handling, authentication, or any kind of synchronous or
     * asynchronous pre-processing of request or postprocessing of responses, it is desirable to be
     * able to intercept requests before they are handed to the server and
     * responses before they are handed over to the application code that
     * initiated these requests. The interceptors leverage the {@link ng.$q
     * promise APIs} to fulfill this need for both synchronous and asynchronous pre-processing.
     *
     * The interceptors are service factories that are registered with the `$httpProvider` by
     * adding them to the `$httpProvider.interceptors` array. The factory is called and
     * injected with dependencies (if specified) and returns the interceptor.
     *
     * There are two kinds of interceptors (and two kinds of rejection interceptors):
     *
     *   * `request`: interceptors get called with a http `config` object. The function is free to
     *     modify the `config` object or create a new one. The function needs to return the `config`
     *     object directly, or a promise containing the `config` or a new `config` object.
     *   * `requestError`: interceptor gets called when a previous interceptor threw an error or
     *     resolved with a rejection.
     *   * `response`: interceptors get called with http `response` object. The function is free to
     *     modify the `response` object or create a new one. The function needs to return the `response`
     *     object directly, or as a promise containing the `response` or a new `response` object.
     *   * `responseError`: interceptor gets called when a previous interceptor threw an error or
     *     resolved with a rejection.
     *
     *
     * ```js
     *   // register the interceptor as a service
     *   $provide.factory('myHttpInterceptor', function($q, dependency1, dependency2) {
     *     return {
     *       // optional method
     *       'request': function(config) {
     *         // do something on success
     *         return config;
     *       },
     *
     *       // optional method
     *      'requestError': function(rejection) {
     *         // do something on error
     *         if (canRecover(rejection)) {
     *           return responseOrNewPromise
     *         }
     *         return $q.reject(rejection);
     *       },
     *
     *
     *
     *       // optional method
     *       'response': function(response) {
     *         // do something on success
     *         return response;
     *       },
     *
     *       // optional method
     *      'responseError': function(rejection) {
     *         // do something on error
     *         if (canRecover(rejection)) {
     *           return responseOrNewPromise
     *         }
     *         return $q.reject(rejection);
     *       }
     *     };
     *   });
     *
     *   $httpProvider.interceptors.push('myHttpInterceptor');
     *
     *
     *   // alternatively, register the interceptor via an anonymous factory
     *   $httpProvider.interceptors.push(function($q, dependency1, dependency2) {
     *     return {
     *      'request': function(config) {
     *          // same as above
     *       },
     *
     *       'response': function(response) {
     *          // same as above
     *       }
     *     };
     *   });
     * ```
     *
     * ## Security Considerations
     *
     * When designing web applications, consider security threats from:
     *
     * - [JSON vulnerability](http://haacked.com/archive/2008/11/20/anatomy-of-a-subtle-json-vulnerability.aspx)
     * - [XSRF](http://en.wikipedia.org/wiki/Cross-site_request_forgery)
     *
     * Both server and the client must cooperate in order to eliminate these threats. Angular comes
     * pre-configured with strategies that address these issues, but for this to work backend server
     * cooperation is required.
     *
     * ### JSON Vulnerability Protection
     *
     * A [JSON vulnerability](http://haacked.com/archive/2008/11/20/anatomy-of-a-subtle-json-vulnerability.aspx)
     * allows third party website to turn your JSON resource URL into
     * [JSONP](http://en.wikipedia.org/wiki/JSONP) request under some conditions. To
     * counter this your server can prefix all JSON requests with following string `")]}',\n"`.
     * Angular will automatically strip the prefix before processing it as JSON.
     *
     * For example if your server needs to return:
     * ```js
     * ['one','two']
     * ```
     *
     * which is vulnerable to attack, your server can return:
     * ```js
     * )]}',
     * ['one','two']
     * ```
     *
     * Angular will strip the prefix, before processing the JSON.
     *
     *
     * ### Cross Site Request Forgery (XSRF) Protection
     *
     * [XSRF](http://en.wikipedia.org/wiki/Cross-site_request_forgery) is a technique by which
     * an unauthorized site can gain your user's private data. Angular provides a mechanism
     * to counter XSRF. When performing XHR requests, the $http service reads a token from a cookie
     * (by default, `XSRF-TOKEN`) and sets it as an HTTP header (`X-XSRF-TOKEN`). Since only
     * JavaScript that runs on your domain could read the cookie, your server can be assured that
     * the XHR came from JavaScript running on your domain. The header will not be set for
     * cross-domain requests.
     *
     * To take advantage of this, your server needs to set a token in a JavaScript readable session
     * cookie called `XSRF-TOKEN` on the first HTTP GET request. On subsequent XHR requests the
     * server can verify that the cookie matches `X-XSRF-TOKEN` HTTP header, and therefore be sure
     * that only JavaScript running on your domain could have sent the request. The token must be
     * unique for each user and must be verifiable by the server (to prevent the JavaScript from
     * making up its own tokens). We recommend that the token is a digest of your site's
     * authentication cookie with a [salt](https://en.wikipedia.org/wiki/Salt_(cryptography&#41;)
     * for added security.
     *
     * The name of the headers can be specified using the xsrfHeaderName and xsrfCookieName
     * properties of either $httpProvider.defaults at config-time, $http.defaults at run-time,
     * or the per-request config object.
     *
     *
     * @param {object} config Object describing the request to be made and how it should be
     *    processed. The object has following properties:
     *
     *    - **method** – `{string}` – HTTP method (e.g. 'GET', 'POST', etc)
     *    - **url** – `{string}` – Absolute or relative URL of the resource that is being requested.
     *    - **params** – `{Object.<string|Object>}` – Map of strings or objects which will be turned
     *      to `?key1=value1&key2=value2` after the url. If the value is not a string, it will be
     *      JSONified.
     *    - **data** – `{string|Object}` – Data to be sent as the request message data.
     *    - **headers** – `{Object}` – Map of strings or functions which return strings representing
     *      HTTP headers to send to the server. If the return value of a function is null, the
     *      header will not be sent.
     *    - **xsrfHeaderName** – `{string}` – Name of HTTP header to populate with the XSRF token.
     *    - **xsrfCookieName** – `{string}` – Name of cookie containing the XSRF token.
     *    - **transformRequest** –
     *      `{function(data, headersGetter)|Array.<function(data, headersGetter)>}` –
     *      transform function or an array of such functions. The transform function takes the http
     *      request body and headers and returns its transformed (typically serialized) version.
     *      See {@link ng.$http#overriding-the-default-transformations-per-request
     *      Overriding the Default Transformations}
     *    - **transformResponse** –
     *      `{function(data, headersGetter, status)|Array.<function(data, headersGetter, status)>}` –
     *      transform function or an array of such functions. The transform function takes the http
     *      response body, headers and status and returns its transformed (typically deserialized) version.
     *      See {@link ng.$http#overriding-the-default-transformations-per-request
     *      Overriding the Default Transformations}
     *    - **cache** – `{boolean|Cache}` – If true, a default $http cache will be used to cache the
     *      GET request, otherwise if a cache instance built with
     *      {@link ng.$cacheFactory $cacheFactory}, this cache will be used for
     *      caching.
     *    - **timeout** – `{number|Promise}` – timeout in milliseconds, or {@link ng.$q promise}
     *      that should abort the request when resolved.
     *    - **withCredentials** - `{boolean}` - whether to set the `withCredentials` flag on the
     *      XHR object. See [requests with credentials](https://developer.mozilla.org/docs/Web/HTTP/Access_control_CORS#Requests_with_credentials)
     *      for more information.
     *    - **responseType** - `{string}` - see
     *      [requestType](https://developer.mozilla.org/en-US/docs/DOM/XMLHttpRequest#responseType).
     *
     * @returns {HttpPromise} Returns a {@link ng.$q promise} object with the
     *   standard `then` method and two http specific methods: `success` and `error`. The `then`
     *   method takes two arguments a success and an error callback which will be called with a
     *   response object. The `success` and `error` methods take a single argument - a function that
     *   will be called when the request succeeds or fails respectively. The arguments passed into
     *   these functions are destructured representation of the response object passed into the
     *   `then` method. The response object has these properties:
     *
     *   - **data** – `{string|Object}` – The response body transformed with the transform
     *     functions.
     *   - **status** – `{number}` – HTTP status code of the response.
     *   - **headers** – `{function([headerName])}` – Header getter function.
     *   - **config** – `{Object}` – The configuration object that was used to generate the request.
     *   - **statusText** – `{string}` – HTTP status text of the response.
     *
     * @property {Array.<Object>} pendingRequests Array of config objects for currently pending
     *   requests. This is primarily meant to be used for debugging purposes.
     *
     *
     * @example
<example module="httpExample">
<file name="index.html">
  <div ng-controller="FetchController">
    <select ng-model="method">
      <option>GET</option>
      <option>JSONP</option>
    </select>
    <input type="text" ng-model="url" size="80"/>
    <button id="fetchbtn" ng-click="fetch()">fetch</button><br>
    <button id="samplegetbtn" ng-click="updateModel('GET', 'http-hello.html')">Sample GET</button>
    <button id="samplejsonpbtn"
      ng-click="updateModel('JSONP',
                    'https://angularjs.org/greet.php?callback=JSON_CALLBACK&name=Super%20Hero')">
      Sample JSONP
    </button>
    <button id="invalidjsonpbtn"
      ng-click="updateModel('JSONP', 'https://angularjs.org/doesntexist&callback=JSON_CALLBACK')">
        Invalid JSONP
      </button>
    <pre>http status code: {{status}}</pre>
    <pre>http response data: {{data}}</pre>
  </div>
</file>
<file name="script.js">
  angular.module('httpExample', [])
    .controller('FetchController', ['$scope', '$http', '$templateCache',
      function($scope, $http, $templateCache) {
        $scope.method = 'GET';
        $scope.url = 'http-hello.html';

        $scope.fetch = function() {
          $scope.code = null;
          $scope.response = null;

          $http({method: $scope.method, url: $scope.url, cache: $templateCache}).
            success(function(data, status) {
              $scope.status = status;
              $scope.data = data;
            }).
            error(function(data, status) {
              $scope.data = data || "Request failed";
              $scope.status = status;
          });
        };

        $scope.updateModel = function(method, url) {
          $scope.method = method;
          $scope.url = url;
        };
      }]);
</file>
<file name="http-hello.html">
  Hello, $http!
</file>
<file name="protractor.js" type="protractor">
  var status = element(by.binding('status'));
  var data = element(by.binding('data'));
  var fetchBtn = element(by.id('fetchbtn'));
  var sampleGetBtn = element(by.id('samplegetbtn'));
  var sampleJsonpBtn = element(by.id('samplejsonpbtn'));
  var invalidJsonpBtn = element(by.id('invalidjsonpbtn'));

  it('should make an xhr GET request', function() {
    sampleGetBtn.click();
    fetchBtn.click();
    expect(status.getText()).toMatch('200');
    expect(data.getText()).toMatch(/Hello, \$http!/);
  });

// Commented out due to flakes. See https://github.com/angular/angular.js/issues/9185
// it('should make a JSONP request to angularjs.org', function() {
//   sampleJsonpBtn.click();
//   fetchBtn.click();
//   expect(status.getText()).toMatch('200');
//   expect(data.getText()).toMatch(/Super Hero!/);
// });

  it('should make JSONP request to invalid URL and invoke the error handler',
      function() {
    invalidJsonpBtn.click();
    fetchBtn.click();
    expect(status.getText()).toMatch('0');
    expect(data.getText()).toMatch('Request failed');
  });
</file>
</example>
     */
        function $http(requestConfig) {
          if (!angular.isObject(requestConfig)) {
            throw minErr('$http')('badreq', 'Http request configuration must be an object.  Received: {0}', requestConfig);
          }
          var config = extend({
              method: 'get',
              transformRequest: defaults.transformRequest,
              transformResponse: defaults.transformResponse
            }, requestConfig);
          config.headers = mergeHeaders(requestConfig);
          config.method = uppercase(config.method);
          var serverRequest = function (config) {
            var headers = config.headers;
            var reqData = transformData(config.data, headersGetter(headers), undefined, config.transformRequest);
            // strip content-type if data is undefined
            if (isUndefined(reqData)) {
              forEach(headers, function (value, header) {
                if (lowercase(header) === 'content-type') {
                  delete headers[header];
                }
              });
            }
            if (isUndefined(config.withCredentials) && !isUndefined(defaults.withCredentials)) {
              config.withCredentials = defaults.withCredentials;
            }
            // send request
            return sendReq(config, reqData).then(transformResponse, transformResponse);
          };
          var chain = [
              serverRequest,
              undefined
            ];
          var promise = $q.when(config);
          // apply interceptors
          forEach(reversedInterceptors, function (interceptor) {
            if (interceptor.request || interceptor.requestError) {
              chain.unshift(interceptor.request, interceptor.requestError);
            }
            if (interceptor.response || interceptor.responseError) {
              chain.push(interceptor.response, interceptor.responseError);
            }
          });
          while (chain.length) {
            var thenFn = chain.shift();
            var rejectFn = chain.shift();
            promise = promise.then(thenFn, rejectFn);
          }
          promise.success = function (fn) {
            assertArgFn(fn, 'fn');
            promise.then(function (response) {
              fn(response.data, response.status, response.headers, config);
            });
            return promise;
          };
          promise.error = function (fn) {
            assertArgFn(fn, 'fn');
            promise.then(null, function (response) {
              fn(response.data, response.status, response.headers, config);
            });
            return promise;
          };
          return promise;
          function transformResponse(response) {
            // make a copy since the response must be cacheable
            var resp = extend({}, response);
            if (!response.data) {
              resp.data = response.data;
            } else {
              resp.data = transformData(response.data, response.headers, response.status, config.transformResponse);
            }
            return isSuccess(response.status) ? resp : $q.reject(resp);
          }
          function executeHeaderFns(headers) {
            var headerContent, processedHeaders = {};
            forEach(headers, function (headerFn, header) {
              if (isFunction(headerFn)) {
                headerContent = headerFn();
                if (headerContent != null) {
                  processedHeaders[header] = headerContent;
                }
              } else {
                processedHeaders[header] = headerFn;
              }
            });
            return processedHeaders;
          }
          function mergeHeaders(config) {
            var defHeaders = defaults.headers, reqHeaders = extend({}, config.headers), defHeaderName, lowercaseDefHeaderName, reqHeaderName;
            defHeaders = extend({}, defHeaders.common, defHeaders[lowercase(config.method)]);
            // using for-in instead of forEach to avoid unecessary iteration after header has been found
            defaultHeadersIteration:
              for (defHeaderName in defHeaders) {
                lowercaseDefHeaderName = lowercase(defHeaderName);
                for (reqHeaderName in reqHeaders) {
                  if (lowercase(reqHeaderName) === lowercaseDefHeaderName) {
                    continue defaultHeadersIteration;
                  }
                }
                reqHeaders[defHeaderName] = defHeaders[defHeaderName];
              }
            // execute if header value is a function for merged headers
            return executeHeaderFns(reqHeaders);
          }
        }
        $http.pendingRequests = [];
        /**
     * @ngdoc method
     * @name $http#get
     *
     * @description
     * Shortcut method to perform `GET` request.
     *
     * @param {string} url Relative or absolute URL specifying the destination of the request
     * @param {Object=} config Optional configuration object
     * @returns {HttpPromise} Future object
     */
        /**
     * @ngdoc method
     * @name $http#delete
     *
     * @description
     * Shortcut method to perform `DELETE` request.
     *
     * @param {string} url Relative or absolute URL specifying the destination of the request
     * @param {Object=} config Optional configuration object
     * @returns {HttpPromise} Future object
     */
        /**
     * @ngdoc method
     * @name $http#head
     *
     * @description
     * Shortcut method to perform `HEAD` request.
     *
     * @param {string} url Relative or absolute URL specifying the destination of the request
     * @param {Object=} config Optional configuration object
     * @returns {HttpPromise} Future object
     */
        /**
     * @ngdoc method
     * @name $http#jsonp
     *
     * @description
     * Shortcut method to perform `JSONP` request.
     *
     * @param {string} url Relative or absolute URL specifying the destination of the request.
     *                     The name of the callback should be the string `JSON_CALLBACK`.
     * @param {Object=} config Optional configuration object
     * @returns {HttpPromise} Future object
     */
        createShortMethods('get', 'delete', 'head', 'jsonp');
        /**
     * @ngdoc method
     * @name $http#post
     *
     * @description
     * Shortcut method to perform `POST` request.
     *
     * @param {string} url Relative or absolute URL specifying the destination of the request
     * @param {*} data Request content
     * @param {Object=} config Optional configuration object
     * @returns {HttpPromise} Future object
     */
        /**
     * @ngdoc method
     * @name $http#put
     *
     * @description
     * Shortcut method to perform `PUT` request.
     *
     * @param {string} url Relative or absolute URL specifying the destination of the request
     * @param {*} data Request content
     * @param {Object=} config Optional configuration object
     * @returns {HttpPromise} Future object
     */
        /**
      * @ngdoc method
      * @name $http#patch
      *
      * @description
      * Shortcut method to perform `PATCH` request.
      *
      * @param {string} url Relative or absolute URL specifying the destination of the request
      * @param {*} data Request content
      * @param {Object=} config Optional configuration object
      * @returns {HttpPromise} Future object
      */
        createShortMethodsWithData('post', 'put', 'patch');
        /**
         * @ngdoc property
         * @name $http#defaults
         *
         * @description
         * Runtime equivalent of the `$httpProvider.defaults` property. Allows configuration of
         * default headers, withCredentials as well as request and response transformations.
         *
         * See "Setting HTTP Headers" and "Transforming Requests and Responses" sections above.
         */
        $http.defaults = defaults;
        return $http;
        function createShortMethods(names) {
          forEach(arguments, function (name) {
            $http[name] = function (url, config) {
              return $http(extend(config || {}, {
                method: name,
                url: url
              }));
            };
          });
        }
        function createShortMethodsWithData(name) {
          forEach(arguments, function (name) {
            $http[name] = function (url, data, config) {
              return $http(extend(config || {}, {
                method: name,
                url: url,
                data: data
              }));
            };
          });
        }
        /**
     * Makes the request.
     *
     * !!! ACCESSES CLOSURE VARS:
     * $httpBackend, defaults, $log, $rootScope, defaultCache, $http.pendingRequests
     */
        function sendReq(config, reqData) {
          var deferred = $q.defer(), promise = deferred.promise, cache, cachedResp, reqHeaders = config.headers, url = buildUrl(config.url, config.params);
          $http.pendingRequests.push(config);
          promise.then(removePendingReq, removePendingReq);
          if ((config.cache || defaults.cache) && config.cache !== false && (config.method === 'GET' || config.method === 'JSONP')) {
            cache = isObject(config.cache) ? config.cache : isObject(defaults.cache) ? defaults.cache : defaultCache;
          }
          if (cache) {
            cachedResp = cache.get(url);
            if (isDefined(cachedResp)) {
              if (isPromiseLike(cachedResp)) {
                // cached request has already been sent, but there is no response yet
                cachedResp.then(resolvePromiseWithResult, resolvePromiseWithResult);
              } else {
                // serving from cache
                if (isArray(cachedResp)) {
                  resolvePromise(cachedResp[1], cachedResp[0], shallowCopy(cachedResp[2]), cachedResp[3]);
                } else {
                  resolvePromise(cachedResp, 200, {}, 'OK');
                }
              }
            } else {
              // put the promise for the non-transformed response into cache as a placeholder
              cache.put(url, promise);
            }
          }
          // if we won't have the response in cache, set the xsrf headers and
          // send the request to the backend
          if (isUndefined(cachedResp)) {
            var xsrfValue = urlIsSameOrigin(config.url) ? $browser.cookies()[config.xsrfCookieName || defaults.xsrfCookieName] : undefined;
            if (xsrfValue) {
              reqHeaders[config.xsrfHeaderName || defaults.xsrfHeaderName] = xsrfValue;
            }
            $httpBackend(config.method, url, reqData, done, reqHeaders, config.timeout, config.withCredentials, config.responseType);
          }
          return promise;
          /**
       * Callback registered to $httpBackend():
       *  - caches the response if desired
       *  - resolves the raw $http promise
       *  - calls $apply
       */
          function done(status, response, headersString, statusText) {
            if (cache) {
              if (isSuccess(status)) {
                cache.put(url, [
                  status,
                  response,
                  parseHeaders(headersString),
                  statusText
                ]);
              } else {
                // remove promise from the cache
                cache.remove(url);
              }
            }
            function resolveHttpPromise() {
              resolvePromise(response, status, headersString, statusText);
            }
            if (useApplyAsync) {
              $rootScope.$applyAsync(resolveHttpPromise);
            } else {
              resolveHttpPromise();
              if (!$rootScope.$$phase)
                $rootScope.$apply();
            }
          }
          /**
       * Resolves the raw $http promise.
       */
          function resolvePromise(response, status, headers, statusText) {
            //status: HTTP response status code, 0, -1 (aborted by timeout / promise)
            status = status >= -1 ? status : 0;
            (isSuccess(status) ? deferred.resolve : deferred.reject)({
              data: response,
              status: status,
              headers: headersGetter(headers),
              config: config,
              statusText: statusText
            });
          }
          function resolvePromiseWithResult(result) {
            resolvePromise(result.data, result.status, shallowCopy(result.headers()), result.statusText);
          }
          function removePendingReq() {
            var idx = $http.pendingRequests.indexOf(config);
            if (idx !== -1)
              $http.pendingRequests.splice(idx, 1);
          }
        }
        function buildUrl(url, params) {
          if (!params)
            return url;
          var parts = [];
          forEachSorted(params, function (value, key) {
            if (value === null || isUndefined(value))
              return;
            if (!isArray(value))
              value = [value];
            forEach(value, function (v) {
              if (isObject(v)) {
                if (isDate(v)) {
                  v = v.toISOString();
                } else {
                  v = toJson(v);
                }
              }
              parts.push(encodeUriQuery(key) + '=' + encodeUriQuery(v));
            });
          });
          if (parts.length > 0) {
            url += (url.indexOf('?') == -1 ? '?' : '&') + parts.join('&');
          }
          return url;
        }
      }
    ];
  }
  function createXhr() {
    return new window.XMLHttpRequest();
  }
  /**
 * @ngdoc service
 * @name $httpBackend
 * @requires $window
 * @requires $document
 *
 * @description
 * HTTP backend used by the {@link ng.$http service} that delegates to
 * XMLHttpRequest object or JSONP and deals with browser incompatibilities.
 *
 * You should never need to use this service directly, instead use the higher-level abstractions:
 * {@link ng.$http $http} or {@link ngResource.$resource $resource}.
 *
 * During testing this implementation is swapped with {@link ngMock.$httpBackend mock
 * $httpBackend} which can be trained with responses.
 */
  function $HttpBackendProvider() {
    this.$get = [
      '$browser',
      '$window',
      '$document',
      function ($browser, $window, $document) {
        return createHttpBackend($browser, createXhr, $browser.defer, $window.angular.callbacks, $document[0]);
      }
    ];
  }
  function createHttpBackend($browser, createXhr, $browserDefer, callbacks, rawDocument) {
    // TODO(vojta): fix the signature
    return function (method, url, post, callback, headers, timeout, withCredentials, responseType) {
      $browser.$$incOutstandingRequestCount();
      url = url || $browser.url();
      if (lowercase(method) == 'jsonp') {
        var callbackId = '_' + (callbacks.counter++).toString(36);
        callbacks[callbackId] = function (data) {
          callbacks[callbackId].data = data;
          callbacks[callbackId].called = true;
        };
        var jsonpDone = jsonpReq(url.replace('JSON_CALLBACK', 'angular.callbacks.' + callbackId), callbackId, function (status, text) {
            completeRequest(callback, status, callbacks[callbackId].data, '', text);
            callbacks[callbackId] = noop;
          });
      } else {
        var xhr = createXhr();
        xhr.open(method, url, true);
        forEach(headers, function (value, key) {
          if (isDefined(value)) {
            xhr.setRequestHeader(key, value);
          }
        });
        xhr.onload = function requestLoaded() {
          var statusText = xhr.statusText || '';
          // responseText is the old-school way of retrieving response (supported by IE9)
          // response/responseType properties were introduced in XHR Level2 spec (supported by IE10)
          var response = 'response' in xhr ? xhr.response : xhr.responseText;
          // normalize IE9 bug (http://bugs.jquery.com/ticket/1450)
          var status = xhr.status === 1223 ? 204 : xhr.status;
          // fix status code when it is 0 (0 status is undocumented).
          // Occurs when accessing file resources or on Android 4.1 stock browser
          // while retrieving files from application cache.
          if (status === 0) {
            status = response ? 200 : urlResolve(url).protocol == 'file' ? 404 : 0;
          }
          completeRequest(callback, status, response, xhr.getAllResponseHeaders(), statusText);
        };
        var requestError = function () {
          // The response is always empty
          // See https://xhr.spec.whatwg.org/#request-error-steps and https://fetch.spec.whatwg.org/#concept-network-error
          completeRequest(callback, -1, null, null, '');
        };
        xhr.onerror = requestError;
        xhr.onabort = requestError;
        if (withCredentials) {
          xhr.withCredentials = true;
        }
        if (responseType) {
          try {
            xhr.responseType = responseType;
          } catch (e) {
            // WebKit added support for the json responseType value on 09/03/2013
            // https://bugs.webkit.org/show_bug.cgi?id=73648. Versions of Safari prior to 7 are
            // known to throw when setting the value "json" as the response type. Other older
            // browsers implementing the responseType
            //
            // The json response type can be ignored if not supported, because JSON payloads are
            // parsed on the client-side regardless.
            if (responseType !== 'json') {
              throw e;
            }
          }
        }
        xhr.send(post || null);
      }
      if (timeout > 0) {
        var timeoutId = $browserDefer(timeoutRequest, timeout);
      } else if (isPromiseLike(timeout)) {
        timeout.then(timeoutRequest);
      }
      function timeoutRequest() {
        jsonpDone && jsonpDone();
        xhr && xhr.abort();
      }
      function completeRequest(callback, status, response, headersString, statusText) {
        // cancel timeout and subsequent timeout promise resolution
        if (timeoutId !== undefined) {
          $browserDefer.cancel(timeoutId);
        }
        jsonpDone = xhr = null;
        callback(status, response, headersString, statusText);
        $browser.$$completeOutstandingRequest(noop);
      }
    };
    function jsonpReq(url, callbackId, done) {
      // we can't use jQuery/jqLite here because jQuery does crazy stuff with script elements, e.g.:
      // - fetches local scripts via XHR and evals them
      // - adds and immediately removes script elements from the document
      var script = rawDocument.createElement('script'), callback = null;
      script.type = 'text/javascript';
      script.src = url;
      script.async = true;
      callback = function (event) {
        removeEventListenerFn(script, 'load', callback);
        removeEventListenerFn(script, 'error', callback);
        rawDocument.body.removeChild(script);
        script = null;
        var status = -1;
        var text = 'unknown';
        if (event) {
          if (event.type === 'load' && !callbacks[callbackId].called) {
            event = { type: 'error' };
          }
          text = event.type;
          status = event.type === 'error' ? 404 : 200;
        }
        if (done) {
          done(status, text);
        }
      };
      addEventListenerFn(script, 'load', callback);
      addEventListenerFn(script, 'error', callback);
      rawDocument.body.appendChild(script);
      return callback;
    }
  }
  var $interpolateMinErr = minErr('$interpolate');
  /**
 * @ngdoc provider
 * @name $interpolateProvider
 *
 * @description
 *
 * Used for configuring the interpolation markup. Defaults to `{{` and `}}`.
 *
 * @example
<example module="customInterpolationApp">
<file name="index.html">
<script>
  var customInterpolationApp = angular.module('customInterpolationApp', []);

  customInterpolationApp.config(function($interpolateProvider) {
    $interpolateProvider.startSymbol('//');
    $interpolateProvider.endSymbol('//');
  });


  customInterpolationApp.controller('DemoController', function() {
      this.label = "This binding is brought you by // interpolation symbols.";
  });
</script>
<div ng-app="App" ng-controller="DemoController as demo">
    //demo.label//
</div>
</file>
<file name="protractor.js" type="protractor">
  it('should interpolate binding with custom symbols', function() {
    expect(element(by.binding('demo.label')).getText()).toBe('This binding is brought you by // interpolation symbols.');
  });
</file>
</example>
 */
  function $InterpolateProvider() {
    var startSymbol = '{{';
    var endSymbol = '}}';
    /**
   * @ngdoc method
   * @name $interpolateProvider#startSymbol
   * @description
   * Symbol to denote start of expression in the interpolated string. Defaults to `{{`.
   *
   * @param {string=} value new value to set the starting symbol to.
   * @returns {string|self} Returns the symbol when used as getter and self if used as setter.
   */
    this.startSymbol = function (value) {
      if (value) {
        startSymbol = value;
        return this;
      } else {
        return startSymbol;
      }
    };
    /**
   * @ngdoc method
   * @name $interpolateProvider#endSymbol
   * @description
   * Symbol to denote the end of expression in the interpolated string. Defaults to `}}`.
   *
   * @param {string=} value new value to set the ending symbol to.
   * @returns {string|self} Returns the symbol when used as getter and self if used as setter.
   */
    this.endSymbol = function (value) {
      if (value) {
        endSymbol = value;
        return this;
      } else {
        return endSymbol;
      }
    };
    this.$get = [
      '$parse',
      '$exceptionHandler',
      '$sce',
      function ($parse, $exceptionHandler, $sce) {
        var startSymbolLength = startSymbol.length, endSymbolLength = endSymbol.length, escapedStartRegexp = new RegExp(startSymbol.replace(/./g, escape), 'g'), escapedEndRegexp = new RegExp(endSymbol.replace(/./g, escape), 'g');
        function escape(ch) {
          return '\\\\\\' + ch;
        }
        /**
     * @ngdoc service
     * @name $interpolate
     * @kind function
     *
     * @requires $parse
     * @requires $sce
     *
     * @description
     *
     * Compiles a string with markup into an interpolation function. This service is used by the
     * HTML {@link ng.$compile $compile} service for data binding. See
     * {@link ng.$interpolateProvider $interpolateProvider} for configuring the
     * interpolation markup.
     *
     *
     * ```js
     *   var $interpolate = ...; // injected
     *   var exp = $interpolate('Hello {{name | uppercase}}!');
     *   expect(exp({name:'Angular'}).toEqual('Hello ANGULAR!');
     * ```
     *
     * `$interpolate` takes an optional fourth argument, `allOrNothing`. If `allOrNothing` is
     * `true`, the interpolation function will return `undefined` unless all embedded expressions
     * evaluate to a value other than `undefined`.
     *
     * ```js
     *   var $interpolate = ...; // injected
     *   var context = {greeting: 'Hello', name: undefined };
     *
     *   // default "forgiving" mode
     *   var exp = $interpolate('{{greeting}} {{name}}!');
     *   expect(exp(context)).toEqual('Hello !');
     *
     *   // "allOrNothing" mode
     *   exp = $interpolate('{{greeting}} {{name}}!', false, null, true);
     *   expect(exp(context)).toBeUndefined();
     *   context.name = 'Angular';
     *   expect(exp(context)).toEqual('Hello Angular!');
     * ```
     *
     * `allOrNothing` is useful for interpolating URLs. `ngSrc` and `ngSrcset` use this behavior.
     *
     * ####Escaped Interpolation
     * $interpolate provides a mechanism for escaping interpolation markers. Start and end markers
     * can be escaped by preceding each of their characters with a REVERSE SOLIDUS U+005C (backslash).
     * It will be rendered as a regular start/end marker, and will not be interpreted as an expression
     * or binding.
     *
     * This enables web-servers to prevent script injection attacks and defacing attacks, to some
     * degree, while also enabling code examples to work without relying on the
     * {@link ng.directive:ngNonBindable ngNonBindable} directive.
     *
     * **For security purposes, it is strongly encouraged that web servers escape user-supplied data,
     * replacing angle brackets (&lt;, &gt;) with &amp;lt; and &amp;gt; respectively, and replacing all
     * interpolation start/end markers with their escaped counterparts.**
     *
     * Escaped interpolation markers are only replaced with the actual interpolation markers in rendered
     * output when the $interpolate service processes the text. So, for HTML elements interpolated
     * by {@link ng.$compile $compile}, or otherwise interpolated with the `mustHaveExpression` parameter
     * set to `true`, the interpolated text must contain an unescaped interpolation expression. As such,
     * this is typically useful only when user-data is used in rendering a template from the server, or
     * when otherwise untrusted data is used by a directive.
     *
     * <example>
     *  <file name="index.html">
     *    <div ng-init="username='A user'">
     *      <p ng-init="apptitle='Escaping demo'">{{apptitle}}: \{\{ username = "defaced value"; \}\}
     *        </p>
     *      <p><strong>{{username}}</strong> attempts to inject code which will deface the
     *        application, but fails to accomplish their task, because the server has correctly
     *        escaped the interpolation start/end markers with REVERSE SOLIDUS U+005C (backslash)
     *        characters.</p>
     *      <p>Instead, the result of the attempted script injection is visible, and can be removed
     *        from the database by an administrator.</p>
     *    </div>
     *  </file>
     * </example>
     *
     * @param {string} text The text with markup to interpolate.
     * @param {boolean=} mustHaveExpression if set to true then the interpolation string must have
     *    embedded expression in order to return an interpolation function. Strings with no
     *    embedded expression will return null for the interpolation function.
     * @param {string=} trustedContext when provided, the returned function passes the interpolated
     *    result through {@link ng.$sce#getTrusted $sce.getTrusted(interpolatedResult,
     *    trustedContext)} before returning it.  Refer to the {@link ng.$sce $sce} service that
     *    provides Strict Contextual Escaping for details.
     * @param {boolean=} allOrNothing if `true`, then the returned function returns undefined
     *    unless all embedded expressions evaluate to a value other than `undefined`.
     * @returns {function(context)} an interpolation function which is used to compute the
     *    interpolated string. The function has these parameters:
     *
     * - `context`: evaluation context for all expressions embedded in the interpolated text
     */
        function $interpolate(text, mustHaveExpression, trustedContext, allOrNothing) {
          allOrNothing = !!allOrNothing;
          var startIndex, endIndex, index = 0, expressions = [], parseFns = [], textLength = text.length, exp, concat = [], expressionPositions = [];
          while (index < textLength) {
            if ((startIndex = text.indexOf(startSymbol, index)) != -1 && (endIndex = text.indexOf(endSymbol, startIndex + startSymbolLength)) != -1) {
              if (index !== startIndex) {
                concat.push(unescapeText(text.substring(index, startIndex)));
              }
              exp = text.substring(startIndex + startSymbolLength, endIndex);
              expressions.push(exp);
              parseFns.push($parse(exp, parseStringifyInterceptor));
              index = endIndex + endSymbolLength;
              expressionPositions.push(concat.length);
              concat.push('');
            } else {
              // we did not find an interpolation, so we have to add the remainder to the separators array
              if (index !== textLength) {
                concat.push(unescapeText(text.substring(index)));
              }
              break;
            }
          }
          // Concatenating expressions makes it hard to reason about whether some combination of
          // concatenated values are unsafe to use and could easily lead to XSS.  By requiring that a
          // single expression be used for iframe[src], object[src], etc., we ensure that the value
          // that's used is assigned or constructed by some JS code somewhere that is more testable or
          // make it obvious that you bound the value to some user controlled value.  This helps reduce
          // the load when auditing for XSS issues.
          if (trustedContext && concat.length > 1) {
            throw $interpolateMinErr('noconcat', 'Error while interpolating: {0}\nStrict Contextual Escaping disallows ' + 'interpolations that concatenate multiple expressions when a trusted value is ' + 'required.  See http://docs.angularjs.org/api/ng.$sce', text);
          }
          if (!mustHaveExpression || expressions.length) {
            var compute = function (values) {
              for (var i = 0, ii = expressions.length; i < ii; i++) {
                if (allOrNothing && isUndefined(values[i]))
                  return;
                concat[expressionPositions[i]] = values[i];
              }
              return concat.join('');
            };
            var getValue = function (value) {
              return trustedContext ? $sce.getTrusted(trustedContext, value) : $sce.valueOf(value);
            };
            var stringify = function (value) {
              if (value == null) {
                // null || undefined
                return '';
              }
              switch (typeof value) {
              case 'string':
                break;
              case 'number':
                value = '' + value;
                break;
              default:
                value = toJson(value);
              }
              return value;
            };
            return extend(function interpolationFn(context) {
              var i = 0;
              var ii = expressions.length;
              var values = new Array(ii);
              try {
                for (; i < ii; i++) {
                  values[i] = parseFns[i](context);
                }
                return compute(values);
              } catch (err) {
                var newErr = $interpolateMinErr('interr', 'Can\'t interpolate: {0}\n{1}', text, err.toString());
                $exceptionHandler(newErr);
              }
            }, {
              exp: text,
              expressions: expressions,
              $$watchDelegate: function (scope, listener, objectEquality) {
                var lastValue;
                return scope.$watchGroup(parseFns, function interpolateFnWatcher(values, oldValues) {
                  var currValue = compute(values);
                  if (isFunction(listener)) {
                    listener.call(this, currValue, values !== oldValues ? lastValue : currValue, scope);
                  }
                  lastValue = currValue;
                }, objectEquality);
              }
            });
          }
          function unescapeText(text) {
            return text.replace(escapedStartRegexp, startSymbol).replace(escapedEndRegexp, endSymbol);
          }
          function parseStringifyInterceptor(value) {
            try {
              value = getValue(value);
              return allOrNothing && !isDefined(value) ? value : stringify(value);
            } catch (err) {
              var newErr = $interpolateMinErr('interr', 'Can\'t interpolate: {0}\n{1}', text, err.toString());
              $exceptionHandler(newErr);
            }
          }
        }
        /**
     * @ngdoc method
     * @name $interpolate#startSymbol
     * @description
     * Symbol to denote the start of expression in the interpolated string. Defaults to `{{`.
     *
     * Use {@link ng.$interpolateProvider#startSymbol `$interpolateProvider.startSymbol`} to change
     * the symbol.
     *
     * @returns {string} start symbol.
     */
        $interpolate.startSymbol = function () {
          return startSymbol;
        };
        /**
     * @ngdoc method
     * @name $interpolate#endSymbol
     * @description
     * Symbol to denote the end of expression in the interpolated string. Defaults to `}}`.
     *
     * Use {@link ng.$interpolateProvider#endSymbol `$interpolateProvider.endSymbol`} to change
     * the symbol.
     *
     * @returns {string} end symbol.
     */
        $interpolate.endSymbol = function () {
          return endSymbol;
        };
        return $interpolate;
      }
    ];
  }
  function $IntervalProvider() {
    this.$get = [
      '$rootScope',
      '$window',
      '$q',
      '$$q',
      function ($rootScope, $window, $q, $$q) {
        var intervals = {};
        /**
      * @ngdoc service
      * @name $interval
      *
      * @description
      * Angular's wrapper for `window.setInterval`. The `fn` function is executed every `delay`
      * milliseconds.
      *
      * The return value of registering an interval function is a promise. This promise will be
      * notified upon each tick of the interval, and will be resolved after `count` iterations, or
      * run indefinitely if `count` is not defined. The value of the notification will be the
      * number of iterations that have run.
      * To cancel an interval, call `$interval.cancel(promise)`.
      *
      * In tests you can use {@link ngMock.$interval#flush `$interval.flush(millis)`} to
      * move forward by `millis` milliseconds and trigger any functions scheduled to run in that
      * time.
      *
      * <div class="alert alert-warning">
      * **Note**: Intervals created by this service must be explicitly destroyed when you are finished
      * with them.  In particular they are not automatically destroyed when a controller's scope or a
      * directive's element are destroyed.
      * You should take this into consideration and make sure to always cancel the interval at the
      * appropriate moment.  See the example below for more details on how and when to do this.
      * </div>
      *
      * @param {function()} fn A function that should be called repeatedly.
      * @param {number} delay Number of milliseconds between each function call.
      * @param {number=} [count=0] Number of times to repeat. If not set, or 0, will repeat
      *   indefinitely.
      * @param {boolean=} [invokeApply=true] If set to `false` skips model dirty checking, otherwise
      *   will invoke `fn` within the {@link ng.$rootScope.Scope#$apply $apply} block.
      * @returns {promise} A promise which will be notified on each iteration.
      *
      * @example
      * <example module="intervalExample">
      * <file name="index.html">
      *   <script>
      *     angular.module('intervalExample', [])
      *       .controller('ExampleController', ['$scope', '$interval',
      *         function($scope, $interval) {
      *           $scope.format = 'M/d/yy h:mm:ss a';
      *           $scope.blood_1 = 100;
      *           $scope.blood_2 = 120;
      *
      *           var stop;
      *           $scope.fight = function() {
      *             // Don't start a new fight if we are already fighting
      *             if ( angular.isDefined(stop) ) return;
      *
      *             stop = $interval(function() {
      *               if ($scope.blood_1 > 0 && $scope.blood_2 > 0) {
      *                 $scope.blood_1 = $scope.blood_1 - 3;
      *                 $scope.blood_2 = $scope.blood_2 - 4;
      *               } else {
      *                 $scope.stopFight();
      *               }
      *             }, 100);
      *           };
      *
      *           $scope.stopFight = function() {
      *             if (angular.isDefined(stop)) {
      *               $interval.cancel(stop);
      *               stop = undefined;
      *             }
      *           };
      *
      *           $scope.resetFight = function() {
      *             $scope.blood_1 = 100;
      *             $scope.blood_2 = 120;
      *           };
      *
      *           $scope.$on('$destroy', function() {
      *             // Make sure that the interval is destroyed too
      *             $scope.stopFight();
      *           });
      *         }])
      *       // Register the 'myCurrentTime' directive factory method.
      *       // We inject $interval and dateFilter service since the factory method is DI.
      *       .directive('myCurrentTime', ['$interval', 'dateFilter',
      *         function($interval, dateFilter) {
      *           // return the directive link function. (compile function not needed)
      *           return function(scope, element, attrs) {
      *             var format,  // date format
      *                 stopTime; // so that we can cancel the time updates
      *
      *             // used to update the UI
      *             function updateTime() {
      *               element.text(dateFilter(new Date(), format));
      *             }
      *
      *             // watch the expression, and update the UI on change.
      *             scope.$watch(attrs.myCurrentTime, function(value) {
      *               format = value;
      *               updateTime();
      *             });
      *
      *             stopTime = $interval(updateTime, 1000);
      *
      *             // listen on DOM destroy (removal) event, and cancel the next UI update
      *             // to prevent updating time after the DOM element was removed.
      *             element.on('$destroy', function() {
      *               $interval.cancel(stopTime);
      *             });
      *           }
      *         }]);
      *   </script>
      *
      *   <div>
      *     <div ng-controller="ExampleController">
      *       Date format: <input ng-model="format"> <hr/>
      *       Current time is: <span my-current-time="format"></span>
      *       <hr/>
      *       Blood 1 : <font color='red'>{{blood_1}}</font>
      *       Blood 2 : <font color='red'>{{blood_2}}</font>
      *       <button type="button" data-ng-click="fight()">Fight</button>
      *       <button type="button" data-ng-click="stopFight()">StopFight</button>
      *       <button type="button" data-ng-click="resetFight()">resetFight</button>
      *     </div>
      *   </div>
      *
      * </file>
      * </example>
      */
        function interval(fn, delay, count, invokeApply) {
          var setInterval = $window.setInterval, clearInterval = $window.clearInterval, iteration = 0, skipApply = isDefined(invokeApply) && !invokeApply, deferred = (skipApply ? $$q : $q).defer(), promise = deferred.promise;
          count = isDefined(count) ? count : 0;
          promise.then(null, null, fn);
          promise.$$intervalId = setInterval(function tick() {
            deferred.notify(iteration++);
            if (count > 0 && iteration >= count) {
              deferred.resolve(iteration);
              clearInterval(promise.$$intervalId);
              delete intervals[promise.$$intervalId];
            }
            if (!skipApply)
              $rootScope.$apply();
          }, delay);
          intervals[promise.$$intervalId] = deferred;
          return promise;
        }
        /**
      * @ngdoc method
      * @name $interval#cancel
      *
      * @description
      * Cancels a task associated with the `promise`.
      *
      * @param {promise} promise returned by the `$interval` function.
      * @returns {boolean} Returns `true` if the task was successfully canceled.
      */
        interval.cancel = function (promise) {
          if (promise && promise.$$intervalId in intervals) {
            intervals[promise.$$intervalId].reject('canceled');
            $window.clearInterval(promise.$$intervalId);
            delete intervals[promise.$$intervalId];
            return true;
          }
          return false;
        };
        return interval;
      }
    ];
  }
  /**
 * @ngdoc service
 * @name $locale
 *
 * @description
 * $locale service provides localization rules for various Angular components. As of right now the
 * only public api is:
 *
 * * `id` – `{string}` – locale id formatted as `languageId-countryId` (e.g. `en-us`)
 */
  function $LocaleProvider() {
    this.$get = function () {
      return {
        id: 'en-us',
        NUMBER_FORMATS: {
          DECIMAL_SEP: '.',
          GROUP_SEP: ',',
          PATTERNS: [
            {
              minInt: 1,
              minFrac: 0,
              maxFrac: 3,
              posPre: '',
              posSuf: '',
              negPre: '-',
              negSuf: '',
              gSize: 3,
              lgSize: 3
            },
            {
              minInt: 1,
              minFrac: 2,
              maxFrac: 2,
              posPre: '\xa4',
              posSuf: '',
              negPre: '(\xa4',
              negSuf: ')',
              gSize: 3,
              lgSize: 3
            }
          ],
          CURRENCY_SYM: '$'
        },
        DATETIME_FORMATS: {
          MONTH: 'January,February,March,April,May,June,July,August,September,October,November,December'.split(','),
          SHORTMONTH: 'Jan,Feb,Mar,Apr,May,Jun,Jul,Aug,Sep,Oct,Nov,Dec'.split(','),
          DAY: 'Sunday,Monday,Tuesday,Wednesday,Thursday,Friday,Saturday'.split(','),
          SHORTDAY: 'Sun,Mon,Tue,Wed,Thu,Fri,Sat'.split(','),
          AMPMS: [
            'AM',
            'PM'
          ],
          medium: 'MMM d, y h:mm:ss a',
          'short': 'M/d/yy h:mm a',
          fullDate: 'EEEE, MMMM d, y',
          longDate: 'MMMM d, y',
          mediumDate: 'MMM d, y',
          shortDate: 'M/d/yy',
          mediumTime: 'h:mm:ss a',
          shortTime: 'h:mm a',
          ERANAMES: [
            'Before Christ',
            'Anno Domini'
          ],
          ERAS: [
            'BC',
            'AD'
          ]
        },
        pluralCat: function (num) {
          if (num === 1) {
            return 'one';
          }
          return 'other';
        }
      };
    };
  }
  var PATH_MATCH = /^([^\?#]*)(\?([^#]*))?(#(.*))?$/, DEFAULT_PORTS = {
      'http': 80,
      'https': 443,
      'ftp': 21
    };
  var $locationMinErr = minErr('$location');
  /**
 * Encode path using encodeUriSegment, ignoring forward slashes
 *
 * @param {string} path Path to encode
 * @returns {string}
 */
  function encodePath(path) {
    var segments = path.split('/'), i = segments.length;
    while (i--) {
      segments[i] = encodeUriSegment(segments[i]);
    }
    return segments.join('/');
  }
  function parseAbsoluteUrl(absoluteUrl, locationObj) {
    var parsedUrl = urlResolve(absoluteUrl);
    locationObj.$$protocol = parsedUrl.protocol;
    locationObj.$$host = parsedUrl.hostname;
    locationObj.$$port = int(parsedUrl.port) || DEFAULT_PORTS[parsedUrl.protocol] || null;
  }
  function parseAppUrl(relativeUrl, locationObj) {
    var prefixed = relativeUrl.charAt(0) !== '/';
    if (prefixed) {
      relativeUrl = '/' + relativeUrl;
    }
    var match = urlResolve(relativeUrl);
    locationObj.$$path = decodeURIComponent(prefixed && match.pathname.charAt(0) === '/' ? match.pathname.substring(1) : match.pathname);
    locationObj.$$search = parseKeyValue(match.search);
    locationObj.$$hash = decodeURIComponent(match.hash);
    // make sure path starts with '/';
    if (locationObj.$$path && locationObj.$$path.charAt(0) != '/') {
      locationObj.$$path = '/' + locationObj.$$path;
    }
  }
  /**
 *
 * @param {string} begin
 * @param {string} whole
 * @returns {string} returns text from whole after begin or undefined if it does not begin with
 *                   expected string.
 */
  function beginsWith(begin, whole) {
    if (whole.indexOf(begin) === 0) {
      return whole.substr(begin.length);
    }
  }
  function stripHash(url) {
    var index = url.indexOf('#');
    return index == -1 ? url : url.substr(0, index);
  }
  function trimEmptyHash(url) {
    return url.replace(/(#.+)|#$/, '$1');
  }
  function stripFile(url) {
    return url.substr(0, stripHash(url).lastIndexOf('/') + 1);
  }
  /* return the server only (scheme://host:port) */
  function serverBase(url) {
    return url.substring(0, url.indexOf('/', url.indexOf('//') + 2));
  }
  /**
 * LocationHtml5Url represents an url
 * This object is exposed as $location service when HTML5 mode is enabled and supported
 *
 * @constructor
 * @param {string} appBase application base URL
 * @param {string} appBaseNoFile application base URL stripped of any filename
 * @param {string} basePrefix url path prefix
 */
  function LocationHtml5Url(appBase, appBaseNoFile, basePrefix) {
    this.$$html5 = true;
    basePrefix = basePrefix || '';
    parseAbsoluteUrl(appBase, this);
    /**
   * Parse given html5 (regular) url string into properties
   * @param {string} url HTML5 url
   * @private
   */
    this.$$parse = function (url) {
      var pathUrl = beginsWith(appBaseNoFile, url);
      if (!isString(pathUrl)) {
        throw $locationMinErr('ipthprfx', 'Invalid url "{0}", missing path prefix "{1}".', url, appBaseNoFile);
      }
      parseAppUrl(pathUrl, this);
      if (!this.$$path) {
        this.$$path = '/';
      }
      this.$$compose();
    };
    /**
   * Compose url and update `absUrl` property
   * @private
   */
    this.$$compose = function () {
      var search = toKeyValue(this.$$search), hash = this.$$hash ? '#' + encodeUriSegment(this.$$hash) : '';
      this.$$url = encodePath(this.$$path) + (search ? '?' + search : '') + hash;
      this.$$absUrl = appBaseNoFile + this.$$url.substr(1);  // first char is always '/'
    };
    this.$$parseLinkUrl = function (url, relHref) {
      if (relHref && relHref[0] === '#') {
        // special case for links to hash fragments:
        // keep the old url and only replace the hash fragment
        this.hash(relHref.slice(1));
        return true;
      }
      var appUrl, prevAppUrl;
      var rewrittenUrl;
      if ((appUrl = beginsWith(appBase, url)) !== undefined) {
        prevAppUrl = appUrl;
        if ((appUrl = beginsWith(basePrefix, appUrl)) !== undefined) {
          rewrittenUrl = appBaseNoFile + (beginsWith('/', appUrl) || appUrl);
        } else {
          rewrittenUrl = appBase + prevAppUrl;
        }
      } else if ((appUrl = beginsWith(appBaseNoFile, url)) !== undefined) {
        rewrittenUrl = appBaseNoFile + appUrl;
      } else if (appBaseNoFile == url + '/') {
        rewrittenUrl = appBaseNoFile;
      }
      if (rewrittenUrl) {
        this.$$parse(rewrittenUrl);
      }
      return !!rewrittenUrl;
    };
  }
  /**
 * LocationHashbangUrl represents url
 * This object is exposed as $location service when developer doesn't opt into html5 mode.
 * It also serves as the base class for html5 mode fallback on legacy browsers.
 *
 * @constructor
 * @param {string} appBase application base URL
 * @param {string} appBaseNoFile application base URL stripped of any filename
 * @param {string} hashPrefix hashbang prefix
 */
  function LocationHashbangUrl(appBase, appBaseNoFile, hashPrefix) {
    parseAbsoluteUrl(appBase, this);
    /**
   * Parse given hashbang url into properties
   * @param {string} url Hashbang url
   * @private
   */
    this.$$parse = function (url) {
      var withoutBaseUrl = beginsWith(appBase, url) || beginsWith(appBaseNoFile, url);
      var withoutHashUrl;
      if (!isUndefined(withoutBaseUrl) && withoutBaseUrl.charAt(0) === '#') {
        // The rest of the url starts with a hash so we have
        // got either a hashbang path or a plain hash fragment
        withoutHashUrl = beginsWith(hashPrefix, withoutBaseUrl);
        if (isUndefined(withoutHashUrl)) {
          // There was no hashbang prefix so we just have a hash fragment
          withoutHashUrl = withoutBaseUrl;
        }
      } else {
        // There was no hashbang path nor hash fragment:
        // If we are in HTML5 mode we use what is left as the path;
        // Otherwise we ignore what is left
        if (this.$$html5) {
          withoutHashUrl = withoutBaseUrl;
        } else {
          withoutHashUrl = '';
          if (isUndefined(withoutBaseUrl)) {
            appBase = url;
            this.replace();
          }
        }
      }
      parseAppUrl(withoutHashUrl, this);
      this.$$path = removeWindowsDriveName(this.$$path, withoutHashUrl, appBase);
      this.$$compose();
      /*
     * In Windows, on an anchor node on documents loaded from
     * the filesystem, the browser will return a pathname
     * prefixed with the drive name ('/C:/path') when a
     * pathname without a drive is set:
     *  * a.setAttribute('href', '/foo')
     *   * a.pathname === '/C:/foo' //true
     *
     * Inside of Angular, we're always using pathnames that
     * do not include drive names for routing.
     */
      function removeWindowsDriveName(path, url, base) {
        /*
      Matches paths for file protocol on windows,
      such as /C:/foo/bar, and captures only /foo/bar.
      */
        var windowsFilePathExp = /^\/[A-Z]:(\/.*)/;
        var firstPathSegmentMatch;
        //Get the relative path from the input URL.
        if (url.indexOf(base) === 0) {
          url = url.replace(base, '');
        }
        // The input URL intentionally contains a first path segment that ends with a colon.
        if (windowsFilePathExp.exec(url)) {
          return path;
        }
        firstPathSegmentMatch = windowsFilePathExp.exec(path);
        return firstPathSegmentMatch ? firstPathSegmentMatch[1] : path;
      }
    };
    /**
   * Compose hashbang url and update `absUrl` property
   * @private
   */
    this.$$compose = function () {
      var search = toKeyValue(this.$$search), hash = this.$$hash ? '#' + encodeUriSegment(this.$$hash) : '';
      this.$$url = encodePath(this.$$path) + (search ? '?' + search : '') + hash;
      this.$$absUrl = appBase + (this.$$url ? hashPrefix + this.$$url : '');
    };
    this.$$parseLinkUrl = function (url, relHref) {
      if (stripHash(appBase) == stripHash(url)) {
        this.$$parse(url);
        return true;
      }
      return false;
    };
  }
  /**
 * LocationHashbangUrl represents url
 * This object is exposed as $location service when html5 history api is enabled but the browser
 * does not support it.
 *
 * @constructor
 * @param {string} appBase application base URL
 * @param {string} appBaseNoFile application base URL stripped of any filename
 * @param {string} hashPrefix hashbang prefix
 */
  function LocationHashbangInHtml5Url(appBase, appBaseNoFile, hashPrefix) {
    this.$$html5 = true;
    LocationHashbangUrl.apply(this, arguments);
    this.$$parseLinkUrl = function (url, relHref) {
      if (relHref && relHref[0] === '#') {
        // special case for links to hash fragments:
        // keep the old url and only replace the hash fragment
        this.hash(relHref.slice(1));
        return true;
      }
      var rewrittenUrl;
      var appUrl;
      if (appBase == stripHash(url)) {
        rewrittenUrl = url;
      } else if (appUrl = beginsWith(appBaseNoFile, url)) {
        rewrittenUrl = appBase + hashPrefix + appUrl;
      } else if (appBaseNoFile === url + '/') {
        rewrittenUrl = appBaseNoFile;
      }
      if (rewrittenUrl) {
        this.$$parse(rewrittenUrl);
      }
      return !!rewrittenUrl;
    };
    this.$$compose = function () {
      var search = toKeyValue(this.$$search), hash = this.$$hash ? '#' + encodeUriSegment(this.$$hash) : '';
      this.$$url = encodePath(this.$$path) + (search ? '?' + search : '') + hash;
      // include hashPrefix in $$absUrl when $$url is empty so IE9 does not reload page because of removal of '#'
      this.$$absUrl = appBase + hashPrefix + this.$$url;
    };
  }
  var locationPrototype = {
      $$html5: false,
      $$replace: false,
      absUrl: locationGetter('$$absUrl'),
      url: function (url) {
        if (isUndefined(url))
          return this.$$url;
        var match = PATH_MATCH.exec(url);
        if (match[1] || url === '')
          this.path(decodeURIComponent(match[1]));
        if (match[2] || match[1] || url === '')
          this.search(match[3] || '');
        this.hash(match[5] || '');
        return this;
      },
      protocol: locationGetter('$$protocol'),
      host: locationGetter('$$host'),
      port: locationGetter('$$port'),
      path: locationGetterSetter('$$path', function (path) {
        path = path !== null ? path.toString() : '';
        return path.charAt(0) == '/' ? path : '/' + path;
      }),
      search: function (search, paramValue) {
        switch (arguments.length) {
        case 0:
          return this.$$search;
        case 1:
          if (isString(search) || isNumber(search)) {
            search = search.toString();
            this.$$search = parseKeyValue(search);
          } else if (isObject(search)) {
            search = copy(search, {});
            // remove object undefined or null properties
            forEach(search, function (value, key) {
              if (value == null)
                delete search[key];
            });
            this.$$search = search;
          } else {
            throw $locationMinErr('isrcharg', 'The first argument of the `$location#search()` call must be a string or an object.');
          }
          break;
        default:
          if (isUndefined(paramValue) || paramValue === null) {
            delete this.$$search[search];
          } else {
            this.$$search[search] = paramValue;
          }
        }
        this.$$compose();
        return this;
      },
      hash: locationGetterSetter('$$hash', function (hash) {
        return hash !== null ? hash.toString() : '';
      }),
      replace: function () {
        this.$$replace = true;
        return this;
      }
    };
  forEach([
    LocationHashbangInHtml5Url,
    LocationHashbangUrl,
    LocationHtml5Url
  ], function (Location) {
    Location.prototype = Object.create(locationPrototype);
    /**
   * @ngdoc method
   * @name $location#state
   *
   * @description
   * This method is getter / setter.
   *
   * Return the history state object when called without any parameter.
   *
   * Change the history state object when called with one parameter and return `$location`.
   * The state object is later passed to `pushState` or `replaceState`.
   *
   * NOTE: This method is supported only in HTML5 mode and only in browsers supporting
   * the HTML5 History API (i.e. methods `pushState` and `replaceState`). If you need to support
   * older browsers (like IE9 or Android < 4.0), don't use this method.
   *
   * @param {object=} state State object for pushState or replaceState
   * @return {object} state
   */
    Location.prototype.state = function (state) {
      if (!arguments.length)
        return this.$$state;
      if (Location !== LocationHtml5Url || !this.$$html5) {
        throw $locationMinErr('nostate', 'History API state support is available only ' + 'in HTML5 mode and only in browsers supporting HTML5 History API');
      }
      // The user might modify `stateObject` after invoking `$location.state(stateObject)`
      // but we're changing the $$state reference to $browser.state() during the $digest
      // so the modification window is narrow.
      this.$$state = isUndefined(state) ? null : state;
      return this;
    };
  });
  function locationGetter(property) {
    return function () {
      return this[property];
    };
  }
  function locationGetterSetter(property, preprocess) {
    return function (value) {
      if (isUndefined(value))
        return this[property];
      this[property] = preprocess(value);
      this.$$compose();
      return this;
    };
  }
  /**
 * @ngdoc service
 * @name $location
 *
 * @requires $rootElement
 *
 * @description
 * The $location service parses the URL in the browser address bar (based on the
 * [window.location](https://developer.mozilla.org/en/window.location)) and makes the URL
 * available to your application. Changes to the URL in the address bar are reflected into
 * $location service and changes to $location are reflected into the browser address bar.
 *
 * **The $location service:**
 *
 * - Exposes the current URL in the browser address bar, so you can
 *   - Watch and observe the URL.
 *   - Change the URL.
 * - Synchronizes the URL with the browser when the user
 *   - Changes the address bar.
 *   - Clicks the back or forward button (or clicks a History link).
 *   - Clicks on a link.
 * - Represents the URL object as a set of methods (protocol, host, port, path, search, hash).
 *
 * For more information see {@link guide/$location Developer Guide: Using $location}
 */
  /**
 * @ngdoc provider
 * @name $locationProvider
 * @description
 * Use the `$locationProvider` to configure how the application deep linking paths are stored.
 */
  function $LocationProvider() {
    var hashPrefix = '', html5Mode = {
        enabled: false,
        requireBase: true,
        rewriteLinks: true
      };
    /**
   * @ngdoc method
   * @name $locationProvider#hashPrefix
   * @description
   * @param {string=} prefix Prefix for hash part (containing path and search)
   * @returns {*} current value if used as getter or itself (chaining) if used as setter
   */
    this.hashPrefix = function (prefix) {
      if (isDefined(prefix)) {
        hashPrefix = prefix;
        return this;
      } else {
        return hashPrefix;
      }
    };
    /**
   * @ngdoc method
   * @name $locationProvider#html5Mode
   * @description
   * @param {(boolean|Object)=} mode If boolean, sets `html5Mode.enabled` to value.
   *   If object, sets `enabled`, `requireBase` and `rewriteLinks` to respective values. Supported
   *   properties:
   *   - **enabled** – `{boolean}` – (default: false) If true, will rely on `history.pushState` to
   *     change urls where supported. Will fall back to hash-prefixed paths in browsers that do not
   *     support `pushState`.
   *   - **requireBase** - `{boolean}` - (default: `true`) When html5Mode is enabled, specifies
   *     whether or not a <base> tag is required to be present. If `enabled` and `requireBase` are
   *     true, and a base tag is not present, an error will be thrown when `$location` is injected.
   *     See the {@link guide/$location $location guide for more information}
   *   - **rewriteLinks** - `{boolean}` - (default: `true`) When html5Mode is enabled,
   *     enables/disables url rewriting for relative links.
   *
   * @returns {Object} html5Mode object if used as getter or itself (chaining) if used as setter
   */
    this.html5Mode = function (mode) {
      if (isBoolean(mode)) {
        html5Mode.enabled = mode;
        return this;
      } else if (isObject(mode)) {
        if (isBoolean(mode.enabled)) {
          html5Mode.enabled = mode.enabled;
        }
        if (isBoolean(mode.requireBase)) {
          html5Mode.requireBase = mode.requireBase;
        }
        if (isBoolean(mode.rewriteLinks)) {
          html5Mode.rewriteLinks = mode.rewriteLinks;
        }
        return this;
      } else {
        return html5Mode;
      }
    };
    /**
   * @ngdoc event
   * @name $location#$locationChangeStart
   * @eventType broadcast on root scope
   * @description
   * Broadcasted before a URL will change.
   *
   * This change can be prevented by calling
   * `preventDefault` method of the event. See {@link ng.$rootScope.Scope#$on} for more
   * details about event object. Upon successful change
   * {@link ng.$location#$locationChangeSuccess $locationChangeSuccess} is fired.
   *
   * The `newState` and `oldState` parameters may be defined only in HTML5 mode and when
   * the browser supports the HTML5 History API.
   *
   * @param {Object} angularEvent Synthetic event object.
   * @param {string} newUrl New URL
   * @param {string=} oldUrl URL that was before it was changed.
   * @param {string=} newState New history state object
   * @param {string=} oldState History state object that was before it was changed.
   */
    /**
   * @ngdoc event
   * @name $location#$locationChangeSuccess
   * @eventType broadcast on root scope
   * @description
   * Broadcasted after a URL was changed.
   *
   * The `newState` and `oldState` parameters may be defined only in HTML5 mode and when
   * the browser supports the HTML5 History API.
   *
   * @param {Object} angularEvent Synthetic event object.
   * @param {string} newUrl New URL
   * @param {string=} oldUrl URL that was before it was changed.
   * @param {string=} newState New history state object
   * @param {string=} oldState History state object that was before it was changed.
   */
    this.$get = [
      '$rootScope',
      '$browser',
      '$sniffer',
      '$rootElement',
      '$window',
      function ($rootScope, $browser, $sniffer, $rootElement, $window) {
        var $location, LocationMode, baseHref = $browser.baseHref(),
          // if base[href] is undefined, it defaults to ''
          initialUrl = $browser.url(), appBase;
        if (html5Mode.enabled) {
          if (!baseHref && html5Mode.requireBase) {
            throw $locationMinErr('nobase', '$location in HTML5 mode requires a <base> tag to be present!');
          }
          appBase = serverBase(initialUrl) + (baseHref || '/');
          LocationMode = $sniffer.history ? LocationHtml5Url : LocationHashbangInHtml5Url;
        } else {
          appBase = stripHash(initialUrl);
          LocationMode = LocationHashbangUrl;
        }
        var appBaseNoFile = stripFile(appBase);
        $location = new LocationMode(appBase, appBaseNoFile, '#' + hashPrefix);
        $location.$$parseLinkUrl(initialUrl, initialUrl);
        $location.$$state = $browser.state();
        var IGNORE_URI_REGEXP = /^\s*(javascript|mailto):/i;
        function setBrowserUrlWithFallback(url, replace, state) {
          var oldUrl = $location.url();
          var oldState = $location.$$state;
          try {
            $browser.url(url, replace, state);
            // Make sure $location.state() returns referentially identical (not just deeply equal)
            // state object; this makes possible quick checking if the state changed in the digest
            // loop. Checking deep equality would be too expensive.
            $location.$$state = $browser.state();
          } catch (e) {
            // Restore old values if pushState fails
            $location.url(oldUrl);
            $location.$$state = oldState;
            throw e;
          }
        }
        $rootElement.on('click', function (event) {
          // TODO(vojta): rewrite link when opening in new tab/window (in legacy browser)
          // currently we open nice url link and redirect then
          if (!html5Mode.rewriteLinks || event.ctrlKey || event.metaKey || event.shiftKey || event.which == 2 || event.button == 2)
            return;
          var elm = jqLite(event.target);
          // traverse the DOM up to find first A tag
          while (nodeName_(elm[0]) !== 'a') {
            // ignore rewriting if no A tag (reached root element, or no parent - removed from document)
            if (elm[0] === $rootElement[0] || !(elm = elm.parent())[0])
              return;
          }
          var absHref = elm.prop('href');
          // get the actual href attribute - see
          // http://msdn.microsoft.com/en-us/library/ie/dd347148(v=vs.85).aspx
          var relHref = elm.attr('href') || elm.attr('xlink:href');
          if (isObject(absHref) && absHref.toString() === '[object SVGAnimatedString]') {
            // SVGAnimatedString.animVal should be identical to SVGAnimatedString.baseVal, unless during
            // an animation.
            absHref = urlResolve(absHref.animVal).href;
          }
          // Ignore when url is started with javascript: or mailto:
          if (IGNORE_URI_REGEXP.test(absHref))
            return;
          if (absHref && !elm.attr('target') && !event.isDefaultPrevented()) {
            if ($location.$$parseLinkUrl(absHref, relHref)) {
              // We do a preventDefault for all urls that are part of the angular application,
              // in html5mode and also without, so that we are able to abort navigation without
              // getting double entries in the location history.
              event.preventDefault();
              // update location manually
              if ($location.absUrl() != $browser.url()) {
                $rootScope.$apply();
                // hack to work around FF6 bug 684208 when scenario runner clicks on links
                $window.angular['ff-684208-preventDefault'] = true;
              }
            }
          }
        });
        // rewrite hashbang url <> html5 url
        if (trimEmptyHash($location.absUrl()) != trimEmptyHash(initialUrl)) {
          $browser.url($location.absUrl(), true);
        }
        var initializing = true;
        // update $location when $browser url changes
        $browser.onUrlChange(function (newUrl, newState) {
          if (isUndefined(beginsWith(appBaseNoFile, newUrl))) {
            // If we are navigating outside of the app then force a reload
            $window.location.href = newUrl;
            return;
          }
          $rootScope.$evalAsync(function () {
            var oldUrl = $location.absUrl();
            var oldState = $location.$$state;
            var defaultPrevented;
            $location.$$parse(newUrl);
            $location.$$state = newState;
            defaultPrevented = $rootScope.$broadcast('$locationChangeStart', newUrl, oldUrl, newState, oldState).defaultPrevented;
            // if the location was changed by a `$locationChangeStart` handler then stop
            // processing this location change
            if ($location.absUrl() !== newUrl)
              return;
            if (defaultPrevented) {
              $location.$$parse(oldUrl);
              $location.$$state = oldState;
              setBrowserUrlWithFallback(oldUrl, false, oldState);
            } else {
              initializing = false;
              afterLocationChange(oldUrl, oldState);
            }
          });
          if (!$rootScope.$$phase)
            $rootScope.$digest();
        });
        // update browser
        $rootScope.$watch(function $locationWatch() {
          var oldUrl = trimEmptyHash($browser.url());
          var newUrl = trimEmptyHash($location.absUrl());
          var oldState = $browser.state();
          var currentReplace = $location.$$replace;
          var urlOrStateChanged = oldUrl !== newUrl || $location.$$html5 && $sniffer.history && oldState !== $location.$$state;
          if (initializing || urlOrStateChanged) {
            initializing = false;
            $rootScope.$evalAsync(function () {
              var newUrl = $location.absUrl();
              var defaultPrevented = $rootScope.$broadcast('$locationChangeStart', newUrl, oldUrl, $location.$$state, oldState).defaultPrevented;
              // if the location was changed by a `$locationChangeStart` handler then stop
              // processing this location change
              if ($location.absUrl() !== newUrl)
                return;
              if (defaultPrevented) {
                $location.$$parse(oldUrl);
                $location.$$state = oldState;
              } else {
                if (urlOrStateChanged) {
                  setBrowserUrlWithFallback(newUrl, currentReplace, oldState === $location.$$state ? null : $location.$$state);
                }
                afterLocationChange(oldUrl, oldState);
              }
            });
          }
          $location.$$replace = false;  // we don't need to return anything because $evalAsync will make the digest loop dirty when
                                        // there is a change
        });
        return $location;
        function afterLocationChange(oldUrl, oldState) {
          $rootScope.$broadcast('$locationChangeSuccess', $location.absUrl(), oldUrl, $location.$$state, oldState);
        }
      }
    ];
  }
  /**
 * @ngdoc service
 * @name $log
 * @requires $window
 *
 * @description
 * Simple service for logging. Default implementation safely writes the message
 * into the browser's console (if present).
 *
 * The main purpose of this service is to simplify debugging and troubleshooting.
 *
 * The default is to log `debug` messages. You can use
 * {@link ng.$logProvider ng.$logProvider#debugEnabled} to change this.
 *
 * @example
   <example module="logExample">
     <file name="script.js">
       angular.module('logExample', [])
         .controller('LogController', ['$scope', '$log', function($scope, $log) {
           $scope.$log = $log;
           $scope.message = 'Hello World!';
         }]);
     </file>
     <file name="index.html">
       <div ng-controller="LogController">
         <p>Reload this page with open console, enter text and hit the log button...</p>
         Message:
         <input type="text" ng-model="message"/>
         <button ng-click="$log.log(message)">log</button>
         <button ng-click="$log.warn(message)">warn</button>
         <button ng-click="$log.info(message)">info</button>
         <button ng-click="$log.error(message)">error</button>
         <button ng-click="$log.debug(message)">debug</button>
       </div>
     </file>
   </example>
 */
  /**
 * @ngdoc provider
 * @name $logProvider
 * @description
 * Use the `$logProvider` to configure how the application logs messages
 */
  function $LogProvider() {
    var debug = true, self = this;
    /**
   * @ngdoc method
   * @name $logProvider#debugEnabled
   * @description
   * @param {boolean=} flag enable or disable debug level messages
   * @returns {*} current value if used as getter or itself (chaining) if used as setter
   */
    this.debugEnabled = function (flag) {
      if (isDefined(flag)) {
        debug = flag;
        return this;
      } else {
        return debug;
      }
    };
    this.$get = [
      '$window',
      function ($window) {
        return {
          log: consoleLog('log'),
          info: consoleLog('info'),
          warn: consoleLog('warn'),
          error: consoleLog('error'),
          debug: function () {
            var fn = consoleLog('debug');
            return function () {
              if (debug) {
                fn.apply(self, arguments);
              }
            };
          }()
        };
        function formatError(arg) {
          if (arg instanceof Error) {
            if (arg.stack) {
              arg = arg.message && arg.stack.indexOf(arg.message) === -1 ? 'Error: ' + arg.message + '\n' + arg.stack : arg.stack;
            } else if (arg.sourceURL) {
              arg = arg.message + '\n' + arg.sourceURL + ':' + arg.line;
            }
          }
          return arg;
        }
        function consoleLog(type) {
          var console = $window.console || {}, logFn = console[type] || console.log || noop, hasApply = false;
          // Note: reading logFn.apply throws an error in IE11 in IE8 document mode.
          // The reason behind this is that console.log has type "object" in IE8...
          try {
            hasApply = !!logFn.apply;
          } catch (e) {
          }
          if (hasApply) {
            return function () {
              var args = [];
              forEach(arguments, function (arg) {
                args.push(formatError(arg));
              });
              return logFn.apply(console, args);
            };
          }
          // we are IE which either doesn't have window.console => this is noop and we do nothing,
          // or we are IE where console.log doesn't have apply so we log at least first 2 args
          return function (arg1, arg2) {
            logFn(arg1, arg2 == null ? '' : arg2);
          };
        }
      }
    ];
  }
  /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *     Any commits to this file should be reviewed with security in mind.  *
 *   Changes to this file can potentially create security vulnerabilities. *
 *          An approval from 2 Core members with history of modifying      *
 *                         this file is required.                          *
 *                                                                         *
 *  Does the change somehow allow for arbitrary javascript to be executed? *
 *    Or allows for someone to change the prototype of built-in objects?   *
 *     Or gives undesired access to variables likes document or window?    *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
  var $parseMinErr = minErr('$parse');
  // Sandboxing Angular Expressions
  // ------------------------------
  // Angular expressions are generally considered safe because these expressions only have direct
  // access to `$scope` and locals. However, one can obtain the ability to execute arbitrary JS code by
  // obtaining a reference to native JS functions such as the Function constructor.
  //
  // As an example, consider the following Angular expression:
  //
  //   {}.toString.constructor('alert("evil JS code")')
  //
  // This sandboxing technique is not perfect and doesn't aim to be. The goal is to prevent exploits
  // against the expression language, but not to prevent exploits that were enabled by exposing
  // sensitive JavaScript or browser APIs on Scope. Exposing such objects on a Scope is never a good
  // practice and therefore we are not even trying to protect against interaction with an object
  // explicitly exposed in this way.
  //
  // In general, it is not possible to access a Window object from an angular expression unless a
  // window or some DOM object that has a reference to window is published onto a Scope.
  // Similarly we prevent invocations of function known to be dangerous, as well as assignments to
  // native objects.
  //
  // See https://docs.angularjs.org/guide/security
  function ensureSafeMemberName(name, fullExpression) {
    if (name === '__defineGetter__' || name === '__defineSetter__' || name === '__lookupGetter__' || name === '__lookupSetter__' || name === '__proto__') {
      throw $parseMinErr('isecfld', 'Attempting to access a disallowed field in Angular expressions! ' + 'Expression: {0}', fullExpression);
    }
    return name;
  }
  function getStringValue(name, fullExpression) {
    // From the JavaScript docs:
    // Property names must be strings. This means that non-string objects cannot be used
    // as keys in an object. Any non-string object, including a number, is typecasted
    // into a string via the toString method.
    //
    // So, to ensure that we are checking the same `name` that JavaScript would use,
    // we cast it to a string, if possible.
    // Doing `name + ''` can cause a repl error if the result to `toString` is not a string,
    // this is, this will handle objects that misbehave.
    name = name + '';
    if (!isString(name)) {
      throw $parseMinErr('iseccst', 'Cannot convert object to primitive value! ' + 'Expression: {0}', fullExpression);
    }
    return name;
  }
  function ensureSafeObject(obj, fullExpression) {
    // nifty check if obj is Function that is fast and works across iframes and other contexts
    if (obj) {
      if (obj.constructor === obj) {
        throw $parseMinErr('isecfn', 'Referencing Function in Angular expressions is disallowed! Expression: {0}', fullExpression);
      } else if (obj.window === obj) {
        throw $parseMinErr('isecwindow', 'Referencing the Window in Angular expressions is disallowed! Expression: {0}', fullExpression);
      } else if (obj.children && (obj.nodeName || obj.prop && obj.attr && obj.find)) {
        throw $parseMinErr('isecdom', 'Referencing DOM nodes in Angular expressions is disallowed! Expression: {0}', fullExpression);
      } else if (obj === Object) {
        throw $parseMinErr('isecobj', 'Referencing Object in Angular expressions is disallowed! Expression: {0}', fullExpression);
      }
    }
    return obj;
  }
  var CALL = Function.prototype.call;
  var APPLY = Function.prototype.apply;
  var BIND = Function.prototype.bind;
  function ensureSafeFunction(obj, fullExpression) {
    if (obj) {
      if (obj.constructor === obj) {
        throw $parseMinErr('isecfn', 'Referencing Function in Angular expressions is disallowed! Expression: {0}', fullExpression);
      } else if (obj === CALL || obj === APPLY || obj === BIND) {
        throw $parseMinErr('isecff', 'Referencing call, apply or bind in Angular expressions is disallowed! Expression: {0}', fullExpression);
      }
    }
  }
  //Keyword constants
  var CONSTANTS = createMap();
  forEach({
    'null': function () {
      return null;
    },
    'true': function () {
      return true;
    },
    'false': function () {
      return false;
    },
    'undefined': function () {
    }
  }, function (constantGetter, name) {
    constantGetter.constant = constantGetter.literal = constantGetter.sharedGetter = true;
    CONSTANTS[name] = constantGetter;
  });
  //Not quite a constant, but can be lex/parsed the same
  CONSTANTS['this'] = function (self) {
    return self;
  };
  CONSTANTS['this'].sharedGetter = true;
  //Operators - will be wrapped by binaryFn/unaryFn/assignment/filter
  var OPERATORS = extend(createMap(), {
      '+': function (self, locals, a, b) {
        a = a(self, locals);
        b = b(self, locals);
        if (isDefined(a)) {
          if (isDefined(b)) {
            return a + b;
          }
          return a;
        }
        return isDefined(b) ? b : undefined;
      },
      '-': function (self, locals, a, b) {
        a = a(self, locals);
        b = b(self, locals);
        return (isDefined(a) ? a : 0) - (isDefined(b) ? b : 0);
      },
      '*': function (self, locals, a, b) {
        return a(self, locals) * b(self, locals);
      },
      '/': function (self, locals, a, b) {
        return a(self, locals) / b(self, locals);
      },
      '%': function (self, locals, a, b) {
        return a(self, locals) % b(self, locals);
      },
      '===': function (self, locals, a, b) {
        return a(self, locals) === b(self, locals);
      },
      '!==': function (self, locals, a, b) {
        return a(self, locals) !== b(self, locals);
      },
      '==': function (self, locals, a, b) {
        return a(self, locals) == b(self, locals);
      },
      '!=': function (self, locals, a, b) {
        return a(self, locals) != b(self, locals);
      },
      '<': function (self, locals, a, b) {
        return a(self, locals) < b(self, locals);
      },
      '>': function (self, locals, a, b) {
        return a(self, locals) > b(self, locals);
      },
      '<=': function (self, locals, a, b) {
        return a(self, locals) <= b(self, locals);
      },
      '>=': function (self, locals, a, b) {
        return a(self, locals) >= b(self, locals);
      },
      '&&': function (self, locals, a, b) {
        return a(self, locals) && b(self, locals);
      },
      '||': function (self, locals, a, b) {
        return a(self, locals) || b(self, locals);
      },
      '!': function (self, locals, a) {
        return !a(self, locals);
      },
      '=': true,
      '|': true
    });
  var ESCAPE = {
      'n': '\n',
      'f': '\f',
      'r': '\r',
      't': '\t',
      'v': '\x0B',
      '\'': '\'',
      '"': '"'
    };
  /////////////////////////////////////////
  /**
 * @constructor
 */
  var Lexer = function (options) {
    this.options = options;
  };
  Lexer.prototype = {
    constructor: Lexer,
    lex: function (text) {
      this.text = text;
      this.index = 0;
      this.tokens = [];
      while (this.index < this.text.length) {
        var ch = this.text.charAt(this.index);
        if (ch === '"' || ch === '\'') {
          this.readString(ch);
        } else if (this.isNumber(ch) || ch === '.' && this.isNumber(this.peek())) {
          this.readNumber();
        } else if (this.isIdent(ch)) {
          this.readIdent();
        } else if (this.is(ch, '(){}[].,;:?')) {
          this.tokens.push({
            index: this.index,
            text: ch
          });
          this.index++;
        } else if (this.isWhitespace(ch)) {
          this.index++;
        } else {
          var ch2 = ch + this.peek();
          var ch3 = ch2 + this.peek(2);
          var op1 = OPERATORS[ch];
          var op2 = OPERATORS[ch2];
          var op3 = OPERATORS[ch3];
          if (op1 || op2 || op3) {
            var token = op3 ? ch3 : op2 ? ch2 : ch;
            this.tokens.push({
              index: this.index,
              text: token,
              operator: true
            });
            this.index += token.length;
          } else {
            this.throwError('Unexpected next character ', this.index, this.index + 1);
          }
        }
      }
      return this.tokens;
    },
    is: function (ch, chars) {
      return chars.indexOf(ch) !== -1;
    },
    peek: function (i) {
      var num = i || 1;
      return this.index + num < this.text.length ? this.text.charAt(this.index + num) : false;
    },
    isNumber: function (ch) {
      return '0' <= ch && ch <= '9' && typeof ch === 'string';
    },
    isWhitespace: function (ch) {
      // IE treats non-breaking space as \u00A0
      return ch === ' ' || ch === '\r' || ch === '\t' || ch === '\n' || ch === '\x0B' || ch === '\xa0';
    },
    isIdent: function (ch) {
      return 'a' <= ch && ch <= 'z' || 'A' <= ch && ch <= 'Z' || '_' === ch || ch === '$';
    },
    isExpOperator: function (ch) {
      return ch === '-' || ch === '+' || this.isNumber(ch);
    },
    throwError: function (error, start, end) {
      end = end || this.index;
      var colStr = isDefined(start) ? 's ' + start + '-' + this.index + ' [' + this.text.substring(start, end) + ']' : ' ' + end;
      throw $parseMinErr('lexerr', 'Lexer Error: {0} at column{1} in expression [{2}].', error, colStr, this.text);
    },
    readNumber: function () {
      var number = '';
      var start = this.index;
      while (this.index < this.text.length) {
        var ch = lowercase(this.text.charAt(this.index));
        if (ch == '.' || this.isNumber(ch)) {
          number += ch;
        } else {
          var peekCh = this.peek();
          if (ch == 'e' && this.isExpOperator(peekCh)) {
            number += ch;
          } else if (this.isExpOperator(ch) && peekCh && this.isNumber(peekCh) && number.charAt(number.length - 1) == 'e') {
            number += ch;
          } else if (this.isExpOperator(ch) && (!peekCh || !this.isNumber(peekCh)) && number.charAt(number.length - 1) == 'e') {
            this.throwError('Invalid exponent');
          } else {
            break;
          }
        }
        this.index++;
      }
      this.tokens.push({
        index: start,
        text: number,
        constant: true,
        value: Number(number)
      });
    },
    readIdent: function () {
      var start = this.index;
      while (this.index < this.text.length) {
        var ch = this.text.charAt(this.index);
        if (!(this.isIdent(ch) || this.isNumber(ch))) {
          break;
        }
        this.index++;
      }
      this.tokens.push({
        index: start,
        text: this.text.slice(start, this.index),
        identifier: true
      });
    },
    readString: function (quote) {
      var start = this.index;
      this.index++;
      var string = '';
      var rawString = quote;
      var escape = false;
      while (this.index < this.text.length) {
        var ch = this.text.charAt(this.index);
        rawString += ch;
        if (escape) {
          if (ch === 'u') {
            var hex = this.text.substring(this.index + 1, this.index + 5);
            if (!hex.match(/[\da-f]{4}/i))
              this.throwError('Invalid unicode escape [\\u' + hex + ']');
            this.index += 4;
            string += String.fromCharCode(parseInt(hex, 16));
          } else {
            var rep = ESCAPE[ch];
            string = string + (rep || ch);
          }
          escape = false;
        } else if (ch === '\\') {
          escape = true;
        } else if (ch === quote) {
          this.index++;
          this.tokens.push({
            index: start,
            text: rawString,
            constant: true,
            value: string
          });
          return;
        } else {
          string += ch;
        }
        this.index++;
      }
      this.throwError('Unterminated quote', start);
    }
  };
  function isConstant(exp) {
    return exp.constant;
  }
  /**
 * @constructor
 */
  var Parser = function (lexer, $filter, options) {
    this.lexer = lexer;
    this.$filter = $filter;
    this.options = options;
  };
  Parser.ZERO = extend(function () {
    return 0;
  }, {
    sharedGetter: true,
    constant: true
  });
  Parser.prototype = {
    constructor: Parser,
    parse: function (text) {
      this.text = text;
      this.tokens = this.lexer.lex(text);
      var value = this.statements();
      if (this.tokens.length !== 0) {
        this.throwError('is an unexpected token', this.tokens[0]);
      }
      value.literal = !!value.literal;
      value.constant = !!value.constant;
      return value;
    },
    primary: function () {
      var primary;
      if (this.expect('(')) {
        primary = this.filterChain();
        this.consume(')');
      } else if (this.expect('[')) {
        primary = this.arrayDeclaration();
      } else if (this.expect('{')) {
        primary = this.object();
      } else if (this.peek().identifier && this.peek().text in CONSTANTS) {
        primary = CONSTANTS[this.consume().text];
      } else if (this.peek().identifier) {
        primary = this.identifier();
      } else if (this.peek().constant) {
        primary = this.constant();
      } else {
        this.throwError('not a primary expression', this.peek());
      }
      var next, context;
      while (next = this.expect('(', '[', '.')) {
        if (next.text === '(') {
          primary = this.functionCall(primary, context);
          context = null;
        } else if (next.text === '[') {
          context = primary;
          primary = this.objectIndex(primary);
        } else if (next.text === '.') {
          context = primary;
          primary = this.fieldAccess(primary);
        } else {
          this.throwError('IMPOSSIBLE');
        }
      }
      return primary;
    },
    throwError: function (msg, token) {
      throw $parseMinErr('syntax', 'Syntax Error: Token \'{0}\' {1} at column {2} of the expression [{3}] starting at [{4}].', token.text, msg, token.index + 1, this.text, this.text.substring(token.index));
    },
    peekToken: function () {
      if (this.tokens.length === 0)
        throw $parseMinErr('ueoe', 'Unexpected end of expression: {0}', this.text);
      return this.tokens[0];
    },
    peek: function (e1, e2, e3, e4) {
      return this.peekAhead(0, e1, e2, e3, e4);
    },
    peekAhead: function (i, e1, e2, e3, e4) {
      if (this.tokens.length > i) {
        var token = this.tokens[i];
        var t = token.text;
        if (t === e1 || t === e2 || t === e3 || t === e4 || !e1 && !e2 && !e3 && !e4) {
          return token;
        }
      }
      return false;
    },
    expect: function (e1, e2, e3, e4) {
      var token = this.peek(e1, e2, e3, e4);
      if (token) {
        this.tokens.shift();
        return token;
      }
      return false;
    },
    consume: function (e1) {
      if (this.tokens.length === 0) {
        throw $parseMinErr('ueoe', 'Unexpected end of expression: {0}', this.text);
      }
      var token = this.expect(e1);
      if (!token) {
        this.throwError('is unexpected, expecting [' + e1 + ']', this.peek());
      }
      return token;
    },
    unaryFn: function (op, right) {
      var fn = OPERATORS[op];
      return extend(function $parseUnaryFn(self, locals) {
        return fn(self, locals, right);
      }, {
        constant: right.constant,
        inputs: [right]
      });
    },
    binaryFn: function (left, op, right, isBranching) {
      var fn = OPERATORS[op];
      return extend(function $parseBinaryFn(self, locals) {
        return fn(self, locals, left, right);
      }, {
        constant: left.constant && right.constant,
        inputs: !isBranching && [
          left,
          right
        ]
      });
    },
    identifier: function () {
      var id = this.consume().text;
      //Continue reading each `.identifier` unless it is a method invocation
      while (this.peek('.') && this.peekAhead(1).identifier && !this.peekAhead(2, '(')) {
        id += this.consume().text + this.consume().text;
      }
      return getterFn(id, this.options, this.text);
    },
    constant: function () {
      var value = this.consume().value;
      return extend(function $parseConstant() {
        return value;
      }, {
        constant: true,
        literal: true
      });
    },
    statements: function () {
      var statements = [];
      while (true) {
        if (this.tokens.length > 0 && !this.peek('}', ')', ';', ']'))
          statements.push(this.filterChain());
        if (!this.expect(';')) {
          // optimize for the common case where there is only one statement.
          // TODO(size): maybe we should not support multiple statements?
          return statements.length === 1 ? statements[0] : function $parseStatements(self, locals) {
            var value;
            for (var i = 0, ii = statements.length; i < ii; i++) {
              value = statements[i](self, locals);
            }
            return value;
          };
        }
      }
    },
    filterChain: function () {
      var left = this.expression();
      var token;
      while (token = this.expect('|')) {
        left = this.filter(left);
      }
      return left;
    },
    filter: function (inputFn) {
      var fn = this.$filter(this.consume().text);
      var argsFn;
      var args;
      if (this.peek(':')) {
        argsFn = [];
        args = [];
        // we can safely reuse the array
        while (this.expect(':')) {
          argsFn.push(this.expression());
        }
      }
      var inputs = [inputFn].concat(argsFn || []);
      return extend(function $parseFilter(self, locals) {
        var input = inputFn(self, locals);
        if (args) {
          args[0] = input;
          var i = argsFn.length;
          while (i--) {
            args[i + 1] = argsFn[i](self, locals);
          }
          return fn.apply(undefined, args);
        }
        return fn(input);
      }, {
        constant: !fn.$stateful && inputs.every(isConstant),
        inputs: !fn.$stateful && inputs
      });
    },
    expression: function () {
      return this.assignment();
    },
    assignment: function () {
      var left = this.ternary();
      var right;
      var token;
      if (token = this.expect('=')) {
        if (!left.assign) {
          this.throwError('implies assignment but [' + this.text.substring(0, token.index) + '] can not be assigned to', token);
        }
        right = this.ternary();
        return extend(function $parseAssignment(scope, locals) {
          return left.assign(scope, right(scope, locals), locals);
        }, {
          inputs: [
            left,
            right
          ]
        });
      }
      return left;
    },
    ternary: function () {
      var left = this.logicalOR();
      var middle;
      var token;
      if (token = this.expect('?')) {
        middle = this.assignment();
        if (this.consume(':')) {
          var right = this.assignment();
          return extend(function $parseTernary(self, locals) {
            return left(self, locals) ? middle(self, locals) : right(self, locals);
          }, { constant: left.constant && middle.constant && right.constant });
        }
      }
      return left;
    },
    logicalOR: function () {
      var left = this.logicalAND();
      var token;
      while (token = this.expect('||')) {
        left = this.binaryFn(left, token.text, this.logicalAND(), true);
      }
      return left;
    },
    logicalAND: function () {
      var left = this.equality();
      var token;
      while (token = this.expect('&&')) {
        left = this.binaryFn(left, token.text, this.equality(), true);
      }
      return left;
    },
    equality: function () {
      var left = this.relational();
      var token;
      while (token = this.expect('==', '!=', '===', '!==')) {
        left = this.binaryFn(left, token.text, this.relational());
      }
      return left;
    },
    relational: function () {
      var left = this.additive();
      var token;
      while (token = this.expect('<', '>', '<=', '>=')) {
        left = this.binaryFn(left, token.text, this.additive());
      }
      return left;
    },
    additive: function () {
      var left = this.multiplicative();
      var token;
      while (token = this.expect('+', '-')) {
        left = this.binaryFn(left, token.text, this.multiplicative());
      }
      return left;
    },
    multiplicative: function () {
      var left = this.unary();
      var token;
      while (token = this.expect('*', '/', '%')) {
        left = this.binaryFn(left, token.text, this.unary());
      }
      return left;
    },
    unary: function () {
      var token;
      if (this.expect('+')) {
        return this.primary();
      } else if (token = this.expect('-')) {
        return this.binaryFn(Parser.ZERO, token.text, this.unary());
      } else if (token = this.expect('!')) {
        return this.unaryFn(token.text, this.unary());
      } else {
        return this.primary();
      }
    },
    fieldAccess: function (object) {
      var getter = this.identifier();
      return extend(function $parseFieldAccess(scope, locals, self) {
        var o = self || object(scope, locals);
        return o == null ? undefined : getter(o);
      }, {
        assign: function (scope, value, locals) {
          var o = object(scope, locals);
          if (!o)
            object.assign(scope, o = {}, locals);
          return getter.assign(o, value);
        }
      });
    },
    objectIndex: function (obj) {
      var expression = this.text;
      var indexFn = this.expression();
      this.consume(']');
      return extend(function $parseObjectIndex(self, locals) {
        var o = obj(self, locals), i = getStringValue(indexFn(self, locals), expression), v;
        ensureSafeMemberName(i, expression);
        if (!o)
          return undefined;
        v = ensureSafeObject(o[i], expression);
        return v;
      }, {
        assign: function (self, value, locals) {
          var key = ensureSafeMemberName(getStringValue(indexFn(self, locals), expression), expression);
          // prevent overwriting of Function.constructor which would break ensureSafeObject check
          var o = ensureSafeObject(obj(self, locals), expression);
          if (!o)
            obj.assign(self, o = {}, locals);
          return o[key] = value;
        }
      });
    },
    functionCall: function (fnGetter, contextGetter) {
      var argsFn = [];
      if (this.peekToken().text !== ')') {
        do {
          argsFn.push(this.expression());
        } while (this.expect(','));
      }
      this.consume(')');
      var expressionText = this.text;
      // we can safely reuse the array across invocations
      var args = argsFn.length ? [] : null;
      return function $parseFunctionCall(scope, locals) {
        var context = contextGetter ? contextGetter(scope, locals) : isDefined(contextGetter) ? undefined : scope;
        var fn = fnGetter(scope, locals, context) || noop;
        if (args) {
          var i = argsFn.length;
          while (i--) {
            args[i] = ensureSafeObject(argsFn[i](scope, locals), expressionText);
          }
        }
        ensureSafeObject(context, expressionText);
        ensureSafeFunction(fn, expressionText);
        // IE doesn't have apply for some native functions
        var v = fn.apply ? fn.apply(context, args) : fn(args[0], args[1], args[2], args[3], args[4]);
        if (args) {
          // Free-up the memory (arguments of the last function call).
          args.length = 0;
        }
        return ensureSafeObject(v, expressionText);
      };
    },
    arrayDeclaration: function () {
      var elementFns = [];
      if (this.peekToken().text !== ']') {
        do {
          if (this.peek(']')) {
            // Support trailing commas per ES5.1.
            break;
          }
          elementFns.push(this.expression());
        } while (this.expect(','));
      }
      this.consume(']');
      return extend(function $parseArrayLiteral(self, locals) {
        var array = [];
        for (var i = 0, ii = elementFns.length; i < ii; i++) {
          array.push(elementFns[i](self, locals));
        }
        return array;
      }, {
        literal: true,
        constant: elementFns.every(isConstant),
        inputs: elementFns
      });
    },
    object: function () {
      var keys = [], valueFns = [];
      if (this.peekToken().text !== '}') {
        do {
          if (this.peek('}')) {
            // Support trailing commas per ES5.1.
            break;
          }
          var token = this.consume();
          if (token.constant) {
            keys.push(token.value);
          } else if (token.identifier) {
            keys.push(token.text);
          } else {
            this.throwError('invalid key', token);
          }
          this.consume(':');
          valueFns.push(this.expression());
        } while (this.expect(','));
      }
      this.consume('}');
      return extend(function $parseObjectLiteral(self, locals) {
        var object = {};
        for (var i = 0, ii = valueFns.length; i < ii; i++) {
          object[keys[i]] = valueFns[i](self, locals);
        }
        return object;
      }, {
        literal: true,
        constant: valueFns.every(isConstant),
        inputs: valueFns
      });
    }
  };
  //////////////////////////////////////////////////
  // Parser helper functions
  //////////////////////////////////////////////////
  function setter(obj, locals, path, setValue, fullExp) {
    ensureSafeObject(obj, fullExp);
    ensureSafeObject(locals, fullExp);
    var element = path.split('.'), key;
    for (var i = 0; element.length > 1; i++) {
      key = ensureSafeMemberName(element.shift(), fullExp);
      var propertyObj = i === 0 && locals && locals[key] || obj[key];
      if (!propertyObj) {
        propertyObj = {};
        obj[key] = propertyObj;
      }
      obj = ensureSafeObject(propertyObj, fullExp);
    }
    key = ensureSafeMemberName(element.shift(), fullExp);
    ensureSafeObject(obj[key], fullExp);
    obj[key] = setValue;
    return setValue;
  }
  var getterFnCacheDefault = createMap();
  var getterFnCacheExpensive = createMap();
  function isPossiblyDangerousMemberName(name) {
    return name == 'constructor';
  }
  /**
 * Implementation of the "Black Hole" variant from:
 * - http://jsperf.com/angularjs-parse-getter/4
 * - http://jsperf.com/path-evaluation-simplified/7
 */
  function cspSafeGetterFn(key0, key1, key2, key3, key4, fullExp, expensiveChecks) {
    ensureSafeMemberName(key0, fullExp);
    ensureSafeMemberName(key1, fullExp);
    ensureSafeMemberName(key2, fullExp);
    ensureSafeMemberName(key3, fullExp);
    ensureSafeMemberName(key4, fullExp);
    var eso = function (o) {
      return ensureSafeObject(o, fullExp);
    };
    var eso0 = expensiveChecks || isPossiblyDangerousMemberName(key0) ? eso : identity;
    var eso1 = expensiveChecks || isPossiblyDangerousMemberName(key1) ? eso : identity;
    var eso2 = expensiveChecks || isPossiblyDangerousMemberName(key2) ? eso : identity;
    var eso3 = expensiveChecks || isPossiblyDangerousMemberName(key3) ? eso : identity;
    var eso4 = expensiveChecks || isPossiblyDangerousMemberName(key4) ? eso : identity;
    return function cspSafeGetter(scope, locals) {
      var pathVal = locals && locals.hasOwnProperty(key0) ? locals : scope;
      if (pathVal == null)
        return pathVal;
      pathVal = eso0(pathVal[key0]);
      if (!key1)
        return pathVal;
      if (pathVal == null)
        return undefined;
      pathVal = eso1(pathVal[key1]);
      if (!key2)
        return pathVal;
      if (pathVal == null)
        return undefined;
      pathVal = eso2(pathVal[key2]);
      if (!key3)
        return pathVal;
      if (pathVal == null)
        return undefined;
      pathVal = eso3(pathVal[key3]);
      if (!key4)
        return pathVal;
      if (pathVal == null)
        return undefined;
      pathVal = eso4(pathVal[key4]);
      return pathVal;
    };
  }
  function getterFnWithEnsureSafeObject(fn, fullExpression) {
    return function (s, l) {
      return fn(s, l, ensureSafeObject, fullExpression);
    };
  }
  function getterFn(path, options, fullExp) {
    var expensiveChecks = options.expensiveChecks;
    var getterFnCache = expensiveChecks ? getterFnCacheExpensive : getterFnCacheDefault;
    var fn = getterFnCache[path];
    if (fn)
      return fn;
    var pathKeys = path.split('.'), pathKeysLength = pathKeys.length;
    // http://jsperf.com/angularjs-parse-getter/6
    if (options.csp) {
      if (pathKeysLength < 6) {
        fn = cspSafeGetterFn(pathKeys[0], pathKeys[1], pathKeys[2], pathKeys[3], pathKeys[4], fullExp, expensiveChecks);
      } else {
        fn = function cspSafeGetter(scope, locals) {
          var i = 0, val;
          do {
            val = cspSafeGetterFn(pathKeys[i++], pathKeys[i++], pathKeys[i++], pathKeys[i++], pathKeys[i++], fullExp, expensiveChecks)(scope, locals);
            locals = undefined;
            // clear after first iteration
            scope = val;
          } while (i < pathKeysLength);
          return val;
        };
      }
    } else {
      var code = '';
      if (expensiveChecks) {
        code += 's = eso(s, fe);\nl = eso(l, fe);\n';
      }
      var needsEnsureSafeObject = expensiveChecks;
      forEach(pathKeys, function (key, index) {
        ensureSafeMemberName(key, fullExp);
        var lookupJs = (index ? 's' : '((l&&l.hasOwnProperty("' + key + '"))?l:s)') + '.' + key;
        if (expensiveChecks || isPossiblyDangerousMemberName(key)) {
          lookupJs = 'eso(' + lookupJs + ', fe)';
          needsEnsureSafeObject = true;
        }
        code += 'if(s == null) return undefined;\n' + 's=' + lookupJs + ';\n';
      });
      code += 'return s;';
      /* jshint -W054 */
      var evaledFnGetter = new Function('s', 'l', 'eso', 'fe', code);
      // s=scope, l=locals, eso=ensureSafeObject
      /* jshint +W054 */
      evaledFnGetter.toString = valueFn(code);
      if (needsEnsureSafeObject) {
        evaledFnGetter = getterFnWithEnsureSafeObject(evaledFnGetter, fullExp);
      }
      fn = evaledFnGetter;
    }
    fn.sharedGetter = true;
    fn.assign = function (self, value, locals) {
      return setter(self, locals, path, value, path);
    };
    getterFnCache[path] = fn;
    return fn;
  }
  var objectValueOf = Object.prototype.valueOf;
  function getValueOf(value) {
    return isFunction(value.valueOf) ? value.valueOf() : objectValueOf.call(value);
  }
  ///////////////////////////////////
  /**
 * @ngdoc service
 * @name $parse
 * @kind function
 *
 * @description
 *
 * Converts Angular {@link guide/expression expression} into a function.
 *
 * ```js
 *   var getter = $parse('user.name');
 *   var setter = getter.assign;
 *   var context = {user:{name:'angular'}};
 *   var locals = {user:{name:'local'}};
 *
 *   expect(getter(context)).toEqual('angular');
 *   setter(context, 'newValue');
 *   expect(context.user.name).toEqual('newValue');
 *   expect(getter(context, locals)).toEqual('local');
 * ```
 *
 *
 * @param {string} expression String expression to compile.
 * @returns {function(context, locals)} a function which represents the compiled expression:
 *
 *    * `context` – `{object}` – an object against which any expressions embedded in the strings
 *      are evaluated against (typically a scope object).
 *    * `locals` – `{object=}` – local variables context object, useful for overriding values in
 *      `context`.
 *
 *    The returned function also has the following properties:
 *      * `literal` – `{boolean}` – whether the expression's top-level node is a JavaScript
 *        literal.
 *      * `constant` – `{boolean}` – whether the expression is made entirely of JavaScript
 *        constant literals.
 *      * `assign` – `{?function(context, value)}` – if the expression is assignable, this will be
 *        set to a function to change its value on the given context.
 *
 */
  /**
 * @ngdoc provider
 * @name $parseProvider
 *
 * @description
 * `$parseProvider` can be used for configuring the default behavior of the {@link ng.$parse $parse}
 *  service.
 */
  function $ParseProvider() {
    var cacheDefault = createMap();
    var cacheExpensive = createMap();
    this.$get = [
      '$filter',
      '$sniffer',
      function ($filter, $sniffer) {
        var $parseOptions = {
            csp: $sniffer.csp,
            expensiveChecks: false
          }, $parseOptionsExpensive = {
            csp: $sniffer.csp,
            expensiveChecks: true
          };
        function wrapSharedExpression(exp) {
          var wrapped = exp;
          if (exp.sharedGetter) {
            wrapped = function $parseWrapper(self, locals) {
              return exp(self, locals);
            };
            wrapped.literal = exp.literal;
            wrapped.constant = exp.constant;
            wrapped.assign = exp.assign;
          }
          return wrapped;
        }
        return function $parse(exp, interceptorFn, expensiveChecks) {
          var parsedExpression, oneTime, cacheKey;
          switch (typeof exp) {
          case 'string':
            cacheKey = exp = exp.trim();
            var cache = expensiveChecks ? cacheExpensive : cacheDefault;
            parsedExpression = cache[cacheKey];
            if (!parsedExpression) {
              if (exp.charAt(0) === ':' && exp.charAt(1) === ':') {
                oneTime = true;
                exp = exp.substring(2);
              }
              var parseOptions = expensiveChecks ? $parseOptionsExpensive : $parseOptions;
              var lexer = new Lexer(parseOptions);
              var parser = new Parser(lexer, $filter, parseOptions);
              parsedExpression = parser.parse(exp);
              if (parsedExpression.constant) {
                parsedExpression.$$watchDelegate = constantWatchDelegate;
              } else if (oneTime) {
                //oneTime is not part of the exp passed to the Parser so we may have to
                //wrap the parsedExpression before adding a $$watchDelegate
                parsedExpression = wrapSharedExpression(parsedExpression);
                parsedExpression.$$watchDelegate = parsedExpression.literal ? oneTimeLiteralWatchDelegate : oneTimeWatchDelegate;
              } else if (parsedExpression.inputs) {
                parsedExpression.$$watchDelegate = inputsWatchDelegate;
              }
              cache[cacheKey] = parsedExpression;
            }
            return addInterceptor(parsedExpression, interceptorFn);
          case 'function':
            return addInterceptor(exp, interceptorFn);
          default:
            return addInterceptor(noop, interceptorFn);
          }
        };
        function collectExpressionInputs(inputs, list) {
          for (var i = 0, ii = inputs.length; i < ii; i++) {
            var input = inputs[i];
            if (!input.constant) {
              if (input.inputs) {
                collectExpressionInputs(input.inputs, list);
              } else if (list.indexOf(input) === -1) {
                // TODO(perf) can we do better?
                list.push(input);
              }
            }
          }
          return list;
        }
        function expressionInputDirtyCheck(newValue, oldValueOfValue) {
          if (newValue == null || oldValueOfValue == null) {
            // null/undefined
            return newValue === oldValueOfValue;
          }
          if (typeof newValue === 'object') {
            // attempt to convert the value to a primitive type
            // TODO(docs): add a note to docs that by implementing valueOf even objects and arrays can
            //             be cheaply dirty-checked
            newValue = getValueOf(newValue);
            if (typeof newValue === 'object') {
              // objects/arrays are not supported - deep-watching them would be too expensive
              return false;
            }  // fall-through to the primitive equality check
          }
          //Primitive or NaN
          return newValue === oldValueOfValue || newValue !== newValue && oldValueOfValue !== oldValueOfValue;
        }
        function inputsWatchDelegate(scope, listener, objectEquality, parsedExpression) {
          var inputExpressions = parsedExpression.$$inputs || (parsedExpression.$$inputs = collectExpressionInputs(parsedExpression.inputs, []));
          var lastResult;
          if (inputExpressions.length === 1) {
            var oldInputValue = expressionInputDirtyCheck;
            // init to something unique so that equals check fails
            inputExpressions = inputExpressions[0];
            return scope.$watch(function expressionInputWatch(scope) {
              var newInputValue = inputExpressions(scope);
              if (!expressionInputDirtyCheck(newInputValue, oldInputValue)) {
                lastResult = parsedExpression(scope);
                oldInputValue = newInputValue && getValueOf(newInputValue);
              }
              return lastResult;
            }, listener, objectEquality);
          }
          var oldInputValueOfValues = [];
          for (var i = 0, ii = inputExpressions.length; i < ii; i++) {
            oldInputValueOfValues[i] = expressionInputDirtyCheck;  // init to something unique so that equals check fails
          }
          return scope.$watch(function expressionInputsWatch(scope) {
            var changed = false;
            for (var i = 0, ii = inputExpressions.length; i < ii; i++) {
              var newInputValue = inputExpressions[i](scope);
              if (changed || (changed = !expressionInputDirtyCheck(newInputValue, oldInputValueOfValues[i]))) {
                oldInputValueOfValues[i] = newInputValue && getValueOf(newInputValue);
              }
            }
            if (changed) {
              lastResult = parsedExpression(scope);
            }
            return lastResult;
          }, listener, objectEquality);
        }
        function oneTimeWatchDelegate(scope, listener, objectEquality, parsedExpression) {
          var unwatch, lastValue;
          return unwatch = scope.$watch(function oneTimeWatch(scope) {
            return parsedExpression(scope);
          }, function oneTimeListener(value, old, scope) {
            lastValue = value;
            if (isFunction(listener)) {
              listener.apply(this, arguments);
            }
            if (isDefined(value)) {
              scope.$$postDigest(function () {
                if (isDefined(lastValue)) {
                  unwatch();
                }
              });
            }
          }, objectEquality);
        }
        function oneTimeLiteralWatchDelegate(scope, listener, objectEquality, parsedExpression) {
          var unwatch, lastValue;
          return unwatch = scope.$watch(function oneTimeWatch(scope) {
            return parsedExpression(scope);
          }, function oneTimeListener(value, old, scope) {
            lastValue = value;
            if (isFunction(listener)) {
              listener.call(this, value, old, scope);
            }
            if (isAllDefined(value)) {
              scope.$$postDigest(function () {
                if (isAllDefined(lastValue))
                  unwatch();
              });
            }
          }, objectEquality);
          function isAllDefined(value) {
            var allDefined = true;
            forEach(value, function (val) {
              if (!isDefined(val))
                allDefined = false;
            });
            return allDefined;
          }
        }
        function constantWatchDelegate(scope, listener, objectEquality, parsedExpression) {
          var unwatch;
          return unwatch = scope.$watch(function constantWatch(scope) {
            return parsedExpression(scope);
          }, function constantListener(value, old, scope) {
            if (isFunction(listener)) {
              listener.apply(this, arguments);
            }
            unwatch();
          }, objectEquality);
        }
        function addInterceptor(parsedExpression, interceptorFn) {
          if (!interceptorFn)
            return parsedExpression;
          var watchDelegate = parsedExpression.$$watchDelegate;
          var regularWatch = watchDelegate !== oneTimeLiteralWatchDelegate && watchDelegate !== oneTimeWatchDelegate;
          var fn = regularWatch ? function regularInterceptedExpression(scope, locals) {
              var value = parsedExpression(scope, locals);
              return interceptorFn(value, scope, locals);
            } : function oneTimeInterceptedExpression(scope, locals) {
              var value = parsedExpression(scope, locals);
              var result = interceptorFn(value, scope, locals);
              // we only return the interceptor's result if the
              // initial value is defined (for bind-once)
              return isDefined(value) ? result : value;
            };
          // Propagate $$watchDelegates other then inputsWatchDelegate
          if (parsedExpression.$$watchDelegate && parsedExpression.$$watchDelegate !== inputsWatchDelegate) {
            fn.$$watchDelegate = parsedExpression.$$watchDelegate;
          } else if (!interceptorFn.$stateful) {
            // If there is an interceptor, but no watchDelegate then treat the interceptor like
            // we treat filters - it is assumed to be a pure function unless flagged with $stateful
            fn.$$watchDelegate = inputsWatchDelegate;
            fn.inputs = [parsedExpression];
          }
          return fn;
        }
      }
    ];
  }
  /**
 * @ngdoc service
 * @name $q
 * @requires $rootScope
 *
 * @description
 * A service that helps you run functions asynchronously, and use their return values (or exceptions)
 * when they are done processing.
 *
 * This is an implementation of promises/deferred objects inspired by
 * [Kris Kowal's Q](https://github.com/kriskowal/q).
 *
 * $q can be used in two fashions --- one which is more similar to Kris Kowal's Q or jQuery's Deferred
 * implementations, and the other which resembles ES6 promises to some degree.
 *
 * # $q constructor
 *
 * The streamlined ES6 style promise is essentially just using $q as a constructor which takes a `resolver`
 * function as the first argument. This is similar to the native Promise implementation from ES6 Harmony,
 * see [MDN](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Promise).
 *
 * While the constructor-style use is supported, not all of the supporting methods from ES6 Harmony promises are
 * available yet.
 *
 * It can be used like so:
 *
 * ```js
 *   // for the purpose of this example let's assume that variables `$q` and `okToGreet`
 *   // are available in the current lexical scope (they could have been injected or passed in).
 *
 *   function asyncGreet(name) {
 *     // perform some asynchronous operation, resolve or reject the promise when appropriate.
 *     return $q(function(resolve, reject) {
 *       setTimeout(function() {
 *         if (okToGreet(name)) {
 *           resolve('Hello, ' + name + '!');
 *         } else {
 *           reject('Greeting ' + name + ' is not allowed.');
 *         }
 *       }, 1000);
 *     });
 *   }
 *
 *   var promise = asyncGreet('Robin Hood');
 *   promise.then(function(greeting) {
 *     alert('Success: ' + greeting);
 *   }, function(reason) {
 *     alert('Failed: ' + reason);
 *   });
 * ```
 *
 * Note: progress/notify callbacks are not currently supported via the ES6-style interface.
 *
 * However, the more traditional CommonJS-style usage is still available, and documented below.
 *
 * [The CommonJS Promise proposal](http://wiki.commonjs.org/wiki/Promises) describes a promise as an
 * interface for interacting with an object that represents the result of an action that is
 * performed asynchronously, and may or may not be finished at any given point in time.
 *
 * From the perspective of dealing with error handling, deferred and promise APIs are to
 * asynchronous programming what `try`, `catch` and `throw` keywords are to synchronous programming.
 *
 * ```js
 *   // for the purpose of this example let's assume that variables `$q` and `okToGreet`
 *   // are available in the current lexical scope (they could have been injected or passed in).
 *
 *   function asyncGreet(name) {
 *     var deferred = $q.defer();
 *
 *     setTimeout(function() {
 *       deferred.notify('About to greet ' + name + '.');
 *
 *       if (okToGreet(name)) {
 *         deferred.resolve('Hello, ' + name + '!');
 *       } else {
 *         deferred.reject('Greeting ' + name + ' is not allowed.');
 *       }
 *     }, 1000);
 *
 *     return deferred.promise;
 *   }
 *
 *   var promise = asyncGreet('Robin Hood');
 *   promise.then(function(greeting) {
 *     alert('Success: ' + greeting);
 *   }, function(reason) {
 *     alert('Failed: ' + reason);
 *   }, function(update) {
 *     alert('Got notification: ' + update);
 *   });
 * ```
 *
 * At first it might not be obvious why this extra complexity is worth the trouble. The payoff
 * comes in the way of guarantees that promise and deferred APIs make, see
 * https://github.com/kriskowal/uncommonjs/blob/master/promises/specification.md.
 *
 * Additionally the promise api allows for composition that is very hard to do with the
 * traditional callback ([CPS](http://en.wikipedia.org/wiki/Continuation-passing_style)) approach.
 * For more on this please see the [Q documentation](https://github.com/kriskowal/q) especially the
 * section on serial or parallel joining of promises.
 *
 * # The Deferred API
 *
 * A new instance of deferred is constructed by calling `$q.defer()`.
 *
 * The purpose of the deferred object is to expose the associated Promise instance as well as APIs
 * that can be used for signaling the successful or unsuccessful completion, as well as the status
 * of the task.
 *
 * **Methods**
 *
 * - `resolve(value)` – resolves the derived promise with the `value`. If the value is a rejection
 *   constructed via `$q.reject`, the promise will be rejected instead.
 * - `reject(reason)` – rejects the derived promise with the `reason`. This is equivalent to
 *   resolving it with a rejection constructed via `$q.reject`.
 * - `notify(value)` - provides updates on the status of the promise's execution. This may be called
 *   multiple times before the promise is either resolved or rejected.
 *
 * **Properties**
 *
 * - promise – `{Promise}` – promise object associated with this deferred.
 *
 *
 * # The Promise API
 *
 * A new promise instance is created when a deferred instance is created and can be retrieved by
 * calling `deferred.promise`.
 *
 * The purpose of the promise object is to allow for interested parties to get access to the result
 * of the deferred task when it completes.
 *
 * **Methods**
 *
 * - `then(successCallback, errorCallback, notifyCallback)` – regardless of when the promise was or
 *   will be resolved or rejected, `then` calls one of the success or error callbacks asynchronously
 *   as soon as the result is available. The callbacks are called with a single argument: the result
 *   or rejection reason. Additionally, the notify callback may be called zero or more times to
 *   provide a progress indication, before the promise is resolved or rejected.
 *
 *   This method *returns a new promise* which is resolved or rejected via the return value of the
 *   `successCallback`, `errorCallback`. It also notifies via the return value of the
 *   `notifyCallback` method. The promise cannot be resolved or rejected from the notifyCallback
 *   method.
 *
 * - `catch(errorCallback)` – shorthand for `promise.then(null, errorCallback)`
 *
 * - `finally(callback, notifyCallback)` – allows you to observe either the fulfillment or rejection of a promise,
 *   but to do so without modifying the final value. This is useful to release resources or do some
 *   clean-up that needs to be done whether the promise was rejected or resolved. See the [full
 *   specification](https://github.com/kriskowal/q/wiki/API-Reference#promisefinallycallback) for
 *   more information.
 *
 * # Chaining promises
 *
 * Because calling the `then` method of a promise returns a new derived promise, it is easily
 * possible to create a chain of promises:
 *
 * ```js
 *   promiseB = promiseA.then(function(result) {
 *     return result + 1;
 *   });
 *
 *   // promiseB will be resolved immediately after promiseA is resolved and its value
 *   // will be the result of promiseA incremented by 1
 * ```
 *
 * It is possible to create chains of any length and since a promise can be resolved with another
 * promise (which will defer its resolution further), it is possible to pause/defer resolution of
 * the promises at any point in the chain. This makes it possible to implement powerful APIs like
 * $http's response interceptors.
 *
 *
 * # Differences between Kris Kowal's Q and $q
 *
 *  There are two main differences:
 *
 * - $q is integrated with the {@link ng.$rootScope.Scope} Scope model observation
 *   mechanism in angular, which means faster propagation of resolution or rejection into your
 *   models and avoiding unnecessary browser repaints, which would result in flickering UI.
 * - Q has many more features than $q, but that comes at a cost of bytes. $q is tiny, but contains
 *   all the important functionality needed for common async tasks.
 *
 *  # Testing
 *
 *  ```js
 *    it('should simulate promise', inject(function($q, $rootScope) {
 *      var deferred = $q.defer();
 *      var promise = deferred.promise;
 *      var resolvedValue;
 *
 *      promise.then(function(value) { resolvedValue = value; });
 *      expect(resolvedValue).toBeUndefined();
 *
 *      // Simulate resolving of promise
 *      deferred.resolve(123);
 *      // Note that the 'then' function does not get called synchronously.
 *      // This is because we want the promise API to always be async, whether or not
 *      // it got called synchronously or asynchronously.
 *      expect(resolvedValue).toBeUndefined();
 *
 *      // Propagate promise resolution to 'then' functions using $apply().
 *      $rootScope.$apply();
 *      expect(resolvedValue).toEqual(123);
 *    }));
 *  ```
 *
 * @param {function(function, function)} resolver Function which is responsible for resolving or
 *   rejecting the newly created promise. The first parameter is a function which resolves the
 *   promise, the second parameter is a function which rejects the promise.
 *
 * @returns {Promise} The newly created promise.
 */
  function $QProvider() {
    this.$get = [
      '$rootScope',
      '$exceptionHandler',
      function ($rootScope, $exceptionHandler) {
        return qFactory(function (callback) {
          $rootScope.$evalAsync(callback);
        }, $exceptionHandler);
      }
    ];
  }
  function $$QProvider() {
    this.$get = [
      '$browser',
      '$exceptionHandler',
      function ($browser, $exceptionHandler) {
        return qFactory(function (callback) {
          $browser.defer(callback);
        }, $exceptionHandler);
      }
    ];
  }
  /**
 * Constructs a promise manager.
 *
 * @param {function(function)} nextTick Function for executing functions in the next turn.
 * @param {function(...*)} exceptionHandler Function into which unexpected exceptions are passed for
 *     debugging purposes.
 * @returns {object} Promise manager.
 */
  function qFactory(nextTick, exceptionHandler) {
    var $qMinErr = minErr('$q', TypeError);
    function callOnce(self, resolveFn, rejectFn) {
      var called = false;
      function wrap(fn) {
        return function (value) {
          if (called)
            return;
          called = true;
          fn.call(self, value);
        };
      }
      return [
        wrap(resolveFn),
        wrap(rejectFn)
      ];
    }
    /**
   * @ngdoc method
   * @name ng.$q#defer
   * @kind function
   *
   * @description
   * Creates a `Deferred` object which represents a task which will finish in the future.
   *
   * @returns {Deferred} Returns a new instance of deferred.
   */
    var defer = function () {
      return new Deferred();
    };
    function Promise() {
      this.$$state = { status: 0 };
    }
    Promise.prototype = {
      then: function (onFulfilled, onRejected, progressBack) {
        var result = new Deferred();
        this.$$state.pending = this.$$state.pending || [];
        this.$$state.pending.push([
          result,
          onFulfilled,
          onRejected,
          progressBack
        ]);
        if (this.$$state.status > 0)
          scheduleProcessQueue(this.$$state);
        return result.promise;
      },
      'catch': function (callback) {
        return this.then(null, callback);
      },
      'finally': function (callback, progressBack) {
        return this.then(function (value) {
          return handleCallback(value, true, callback);
        }, function (error) {
          return handleCallback(error, false, callback);
        }, progressBack);
      }
    };
    //Faster, more basic than angular.bind http://jsperf.com/angular-bind-vs-custom-vs-native
    function simpleBind(context, fn) {
      return function (value) {
        fn.call(context, value);
      };
    }
    function processQueue(state) {
      var fn, promise, pending;
      pending = state.pending;
      state.processScheduled = false;
      state.pending = undefined;
      for (var i = 0, ii = pending.length; i < ii; ++i) {
        promise = pending[i][0];
        fn = pending[i][state.status];
        try {
          if (isFunction(fn)) {
            promise.resolve(fn(state.value));
          } else if (state.status === 1) {
            promise.resolve(state.value);
          } else {
            promise.reject(state.value);
          }
        } catch (e) {
          promise.reject(e);
          exceptionHandler(e);
        }
      }
    }
    function scheduleProcessQueue(state) {
      if (state.processScheduled || !state.pending)
        return;
      state.processScheduled = true;
      nextTick(function () {
        processQueue(state);
      });
    }
    function Deferred() {
      this.promise = new Promise();
      //Necessary to support unbound execution :/
      this.resolve = simpleBind(this, this.resolve);
      this.reject = simpleBind(this, this.reject);
      this.notify = simpleBind(this, this.notify);
    }
    Deferred.prototype = {
      resolve: function (val) {
        if (this.promise.$$state.status)
          return;
        if (val === this.promise) {
          this.$$reject($qMinErr('qcycle', 'Expected promise to be resolved with value other than itself \'{0}\'', val));
        } else {
          this.$$resolve(val);
        }
      },
      $$resolve: function (val) {
        var then, fns;
        fns = callOnce(this, this.$$resolve, this.$$reject);
        try {
          if (isObject(val) || isFunction(val))
            then = val && val.then;
          if (isFunction(then)) {
            this.promise.$$state.status = -1;
            then.call(val, fns[0], fns[1], this.notify);
          } else {
            this.promise.$$state.value = val;
            this.promise.$$state.status = 1;
            scheduleProcessQueue(this.promise.$$state);
          }
        } catch (e) {
          fns[1](e);
          exceptionHandler(e);
        }
      },
      reject: function (reason) {
        if (this.promise.$$state.status)
          return;
        this.$$reject(reason);
      },
      $$reject: function (reason) {
        this.promise.$$state.value = reason;
        this.promise.$$state.status = 2;
        scheduleProcessQueue(this.promise.$$state);
      },
      notify: function (progress) {
        var callbacks = this.promise.$$state.pending;
        if (this.promise.$$state.status <= 0 && callbacks && callbacks.length) {
          nextTick(function () {
            var callback, result;
            for (var i = 0, ii = callbacks.length; i < ii; i++) {
              result = callbacks[i][0];
              callback = callbacks[i][3];
              try {
                result.notify(isFunction(callback) ? callback(progress) : progress);
              } catch (e) {
                exceptionHandler(e);
              }
            }
          });
        }
      }
    };
    /**
   * @ngdoc method
   * @name $q#reject
   * @kind function
   *
   * @description
   * Creates a promise that is resolved as rejected with the specified `reason`. This api should be
   * used to forward rejection in a chain of promises. If you are dealing with the last promise in
   * a promise chain, you don't need to worry about it.
   *
   * When comparing deferreds/promises to the familiar behavior of try/catch/throw, think of
   * `reject` as the `throw` keyword in JavaScript. This also means that if you "catch" an error via
   * a promise error callback and you want to forward the error to the promise derived from the
   * current promise, you have to "rethrow" the error by returning a rejection constructed via
   * `reject`.
   *
   * ```js
   *   promiseB = promiseA.then(function(result) {
   *     // success: do something and resolve promiseB
   *     //          with the old or a new result
   *     return result;
   *   }, function(reason) {
   *     // error: handle the error if possible and
   *     //        resolve promiseB with newPromiseOrValue,
   *     //        otherwise forward the rejection to promiseB
   *     if (canHandle(reason)) {
   *      // handle the error and recover
   *      return newPromiseOrValue;
   *     }
   *     return $q.reject(reason);
   *   });
   * ```
   *
   * @param {*} reason Constant, message, exception or an object representing the rejection reason.
   * @returns {Promise} Returns a promise that was already resolved as rejected with the `reason`.
   */
    var reject = function (reason) {
      var result = new Deferred();
      result.reject(reason);
      return result.promise;
    };
    var makePromise = function makePromise(value, resolved) {
      var result = new Deferred();
      if (resolved) {
        result.resolve(value);
      } else {
        result.reject(value);
      }
      return result.promise;
    };
    var handleCallback = function handleCallback(value, isResolved, callback) {
      var callbackOutput = null;
      try {
        if (isFunction(callback))
          callbackOutput = callback();
      } catch (e) {
        return makePromise(e, false);
      }
      if (isPromiseLike(callbackOutput)) {
        return callbackOutput.then(function () {
          return makePromise(value, isResolved);
        }, function (error) {
          return makePromise(error, false);
        });
      } else {
        return makePromise(value, isResolved);
      }
    };
    /**
   * @ngdoc method
   * @name $q#when
   * @kind function
   *
   * @description
   * Wraps an object that might be a value or a (3rd party) then-able promise into a $q promise.
   * This is useful when you are dealing with an object that might or might not be a promise, or if
   * the promise comes from a source that can't be trusted.
   *
   * @param {*} value Value or a promise
   * @returns {Promise} Returns a promise of the passed value or promise
   */
    var when = function (value, callback, errback, progressBack) {
      var result = new Deferred();
      result.resolve(value);
      return result.promise.then(callback, errback, progressBack);
    };
    /**
   * @ngdoc method
   * @name $q#all
   * @kind function
   *
   * @description
   * Combines multiple promises into a single promise that is resolved when all of the input
   * promises are resolved.
   *
   * @param {Array.<Promise>|Object.<Promise>} promises An array or hash of promises.
   * @returns {Promise} Returns a single promise that will be resolved with an array/hash of values,
   *   each value corresponding to the promise at the same index/key in the `promises` array/hash.
   *   If any of the promises is resolved with a rejection, this resulting promise will be rejected
   *   with the same rejection value.
   */
    function all(promises) {
      var deferred = new Deferred(), counter = 0, results = isArray(promises) ? [] : {};
      forEach(promises, function (promise, key) {
        counter++;
        when(promise).then(function (value) {
          if (results.hasOwnProperty(key))
            return;
          results[key] = value;
          if (!--counter)
            deferred.resolve(results);
        }, function (reason) {
          if (results.hasOwnProperty(key))
            return;
          deferred.reject(reason);
        });
      });
      if (counter === 0) {
        deferred.resolve(results);
      }
      return deferred.promise;
    }
    var $Q = function Q(resolver) {
      if (!isFunction(resolver)) {
        throw $qMinErr('norslvr', 'Expected resolverFn, got \'{0}\'', resolver);
      }
      if (!(this instanceof Q)) {
        // More useful when $Q is the Promise itself.
        return new Q(resolver);
      }
      var deferred = new Deferred();
      function resolveFn(value) {
        deferred.resolve(value);
      }
      function rejectFn(reason) {
        deferred.reject(reason);
      }
      resolver(resolveFn, rejectFn);
      return deferred.promise;
    };
    $Q.defer = defer;
    $Q.reject = reject;
    $Q.when = when;
    $Q.all = all;
    return $Q;
  }
  function $$RAFProvider() {
    //rAF
    this.$get = [
      '$window',
      '$timeout',
      function ($window, $timeout) {
        var requestAnimationFrame = $window.requestAnimationFrame || $window.webkitRequestAnimationFrame;
        var cancelAnimationFrame = $window.cancelAnimationFrame || $window.webkitCancelAnimationFrame || $window.webkitCancelRequestAnimationFrame;
        var rafSupported = !!requestAnimationFrame;
        var rafFn = rafSupported ? function (fn) {
            var id = requestAnimationFrame(fn);
            return function () {
              cancelAnimationFrame(id);
            };
          } : function (fn) {
            var timer = $timeout(fn, 16.66, false);
            // 1000 / 60 = 16.666
            return function () {
              $timeout.cancel(timer);
            };
          };
        queueFn.supported = rafSupported;
        var cancelLastRAF;
        var taskCount = 0;
        var taskQueue = [];
        return queueFn;
        function flush() {
          for (var i = 0; i < taskQueue.length; i++) {
            var task = taskQueue[i];
            if (task) {
              taskQueue[i] = null;
              task();
            }
          }
          taskCount = taskQueue.length = 0;
        }
        function queueFn(asyncFn) {
          var index = taskQueue.length;
          taskCount++;
          taskQueue.push(asyncFn);
          if (index === 0) {
            cancelLastRAF = rafFn(flush);
          }
          return function cancelQueueFn() {
            if (index >= 0) {
              taskQueue[index] = null;
              index = null;
              if (--taskCount === 0 && cancelLastRAF) {
                cancelLastRAF();
                cancelLastRAF = null;
                taskQueue.length = 0;
              }
            }
          };
        }
      }
    ];
  }
  /**
 * DESIGN NOTES
 *
 * The design decisions behind the scope are heavily favored for speed and memory consumption.
 *
 * The typical use of scope is to watch the expressions, which most of the time return the same
 * value as last time so we optimize the operation.
 *
 * Closures construction is expensive in terms of speed as well as memory:
 *   - No closures, instead use prototypical inheritance for API
 *   - Internal state needs to be stored on scope directly, which means that private state is
 *     exposed as $$____ properties
 *
 * Loop operations are optimized by using while(count--) { ... }
 *   - this means that in order to keep the same order of execution as addition we have to add
 *     items to the array at the beginning (unshift) instead of at the end (push)
 *
 * Child scopes are created and removed often
 *   - Using an array would be slow since inserts in middle are expensive so we use linked list
 *
 * There are few watches then a lot of observers. This is why you don't want the observer to be
 * implemented in the same way as watch. Watch requires return of initialization function which
 * are expensive to construct.
 */
  /**
 * @ngdoc provider
 * @name $rootScopeProvider
 * @description
 *
 * Provider for the $rootScope service.
 */
  /**
 * @ngdoc method
 * @name $rootScopeProvider#digestTtl
 * @description
 *
 * Sets the number of `$digest` iterations the scope should attempt to execute before giving up and
 * assuming that the model is unstable.
 *
 * The current default is 10 iterations.
 *
 * In complex applications it's possible that the dependencies between `$watch`s will result in
 * several digest iterations. However if an application needs more than the default 10 digest
 * iterations for its model to stabilize then you should investigate what is causing the model to
 * continuously change during the digest.
 *
 * Increasing the TTL could have performance implications, so you should not change it without
 * proper justification.
 *
 * @param {number} limit The number of digest iterations.
 */
  /**
 * @ngdoc service
 * @name $rootScope
 * @description
 *
 * Every application has a single root {@link ng.$rootScope.Scope scope}.
 * All other scopes are descendant scopes of the root scope. Scopes provide separation
 * between the model and the view, via a mechanism for watching the model for changes.
 * They also provide an event emission/broadcast and subscription facility. See the
 * {@link guide/scope developer guide on scopes}.
 */
  function $RootScopeProvider() {
    var TTL = 10;
    var $rootScopeMinErr = minErr('$rootScope');
    var lastDirtyWatch = null;
    var applyAsyncId = null;
    this.digestTtl = function (value) {
      if (arguments.length) {
        TTL = value;
      }
      return TTL;
    };
    function createChildScopeClass(parent) {
      function ChildScope() {
        this.$$watchers = this.$$nextSibling = this.$$childHead = this.$$childTail = null;
        this.$$listeners = {};
        this.$$listenerCount = {};
        this.$id = nextUid();
        this.$$ChildScope = null;
      }
      ChildScope.prototype = parent;
      return ChildScope;
    }
    this.$get = [
      '$injector',
      '$exceptionHandler',
      '$parse',
      '$browser',
      function ($injector, $exceptionHandler, $parse, $browser) {
        function destroyChildScope($event) {
          $event.currentScope.$$destroyed = true;
        }
        /**
     * @ngdoc type
     * @name $rootScope.Scope
     *
     * @description
     * A root scope can be retrieved using the {@link ng.$rootScope $rootScope} key from the
     * {@link auto.$injector $injector}. Child scopes are created using the
     * {@link ng.$rootScope.Scope#$new $new()} method. (Most scopes are created automatically when
     * compiled HTML template is executed.) See also the {@link guide/scope Scopes guide} for
     * an in-depth introduction and usage examples.
     *
     *
     * # Inheritance
     * A scope can inherit from a parent scope, as in this example:
     * ```js
         var parent = $rootScope;
         var child = parent.$new();

         parent.salutation = "Hello";
         expect(child.salutation).toEqual('Hello');

         child.salutation = "Welcome";
         expect(child.salutation).toEqual('Welcome');
         expect(parent.salutation).toEqual('Hello');
     * ```
     *
     * When interacting with `Scope` in tests, additional helper methods are available on the
     * instances of `Scope` type. See {@link ngMock.$rootScope.Scope ngMock Scope} for additional
     * details.
     *
     *
     * @param {Object.<string, function()>=} providers Map of service factory which need to be
     *                                       provided for the current scope. Defaults to {@link ng}.
     * @param {Object.<string, *>=} instanceCache Provides pre-instantiated services which should
     *                              append/override services provided by `providers`. This is handy
     *                              when unit-testing and having the need to override a default
     *                              service.
     * @returns {Object} Newly created scope.
     *
     */
        function Scope() {
          this.$id = nextUid();
          this.$$phase = this.$parent = this.$$watchers = this.$$nextSibling = this.$$prevSibling = this.$$childHead = this.$$childTail = null;
          this.$root = this;
          this.$$destroyed = false;
          this.$$listeners = {};
          this.$$listenerCount = {};
          this.$$isolateBindings = null;
        }
        /**
     * @ngdoc property
     * @name $rootScope.Scope#$id
     *
     * @description
     * Unique scope ID (monotonically increasing) useful for debugging.
     */
        /**
      * @ngdoc property
      * @name $rootScope.Scope#$parent
      *
      * @description
      * Reference to the parent scope.
      */
        /**
       * @ngdoc property
       * @name $rootScope.Scope#$root
       *
       * @description
       * Reference to the root scope.
       */
        Scope.prototype = {
          constructor: Scope,
          $new: function (isolate, parent) {
            var child;
            parent = parent || this;
            if (isolate) {
              child = new Scope();
              child.$root = this.$root;
            } else {
              // Only create a child scope class if somebody asks for one,
              // but cache it to allow the VM to optimize lookups.
              if (!this.$$ChildScope) {
                this.$$ChildScope = createChildScopeClass(this);
              }
              child = new this.$$ChildScope();
            }
            child.$parent = parent;
            child.$$prevSibling = parent.$$childTail;
            if (parent.$$childHead) {
              parent.$$childTail.$$nextSibling = child;
              parent.$$childTail = child;
            } else {
              parent.$$childHead = parent.$$childTail = child;
            }
            // When the new scope is not isolated or we inherit from `this`, and
            // the parent scope is destroyed, the property `$$destroyed` is inherited
            // prototypically. In all other cases, this property needs to be set
            // when the parent scope is destroyed.
            // The listener needs to be added after the parent is set
            if (isolate || parent != this)
              child.$on('$destroy', destroyChildScope);
            return child;
          },
          $watch: function (watchExp, listener, objectEquality) {
            var get = $parse(watchExp);
            if (get.$$watchDelegate) {
              return get.$$watchDelegate(this, listener, objectEquality, get);
            }
            var scope = this, array = scope.$$watchers, watcher = {
                fn: listener,
                last: initWatchVal,
                get: get,
                exp: watchExp,
                eq: !!objectEquality
              };
            lastDirtyWatch = null;
            if (!isFunction(listener)) {
              watcher.fn = noop;
            }
            if (!array) {
              array = scope.$$watchers = [];
            }
            // we use unshift since we use a while loop in $digest for speed.
            // the while loop reads in reverse order.
            array.unshift(watcher);
            return function deregisterWatch() {
              arrayRemove(array, watcher);
              lastDirtyWatch = null;
            };
          },
          $watchGroup: function (watchExpressions, listener) {
            var oldValues = new Array(watchExpressions.length);
            var newValues = new Array(watchExpressions.length);
            var deregisterFns = [];
            var self = this;
            var changeReactionScheduled = false;
            var firstRun = true;
            if (!watchExpressions.length) {
              // No expressions means we call the listener ASAP
              var shouldCall = true;
              self.$evalAsync(function () {
                if (shouldCall)
                  listener(newValues, newValues, self);
              });
              return function deregisterWatchGroup() {
                shouldCall = false;
              };
            }
            if (watchExpressions.length === 1) {
              // Special case size of one
              return this.$watch(watchExpressions[0], function watchGroupAction(value, oldValue, scope) {
                newValues[0] = value;
                oldValues[0] = oldValue;
                listener(newValues, value === oldValue ? newValues : oldValues, scope);
              });
            }
            forEach(watchExpressions, function (expr, i) {
              var unwatchFn = self.$watch(expr, function watchGroupSubAction(value, oldValue) {
                  newValues[i] = value;
                  oldValues[i] = oldValue;
                  if (!changeReactionScheduled) {
                    changeReactionScheduled = true;
                    self.$evalAsync(watchGroupAction);
                  }
                });
              deregisterFns.push(unwatchFn);
            });
            function watchGroupAction() {
              changeReactionScheduled = false;
              if (firstRun) {
                firstRun = false;
                listener(newValues, newValues, self);
              } else {
                listener(newValues, oldValues, self);
              }
            }
            return function deregisterWatchGroup() {
              while (deregisterFns.length) {
                deregisterFns.shift()();
              }
            };
          },
          $watchCollection: function (obj, listener) {
            $watchCollectionInterceptor.$stateful = true;
            var self = this;
            // the current value, updated on each dirty-check run
            var newValue;
            // a shallow copy of the newValue from the last dirty-check run,
            // updated to match newValue during dirty-check run
            var oldValue;
            // a shallow copy of the newValue from when the last change happened
            var veryOldValue;
            // only track veryOldValue if the listener is asking for it
            var trackVeryOldValue = listener.length > 1;
            var changeDetected = 0;
            var changeDetector = $parse(obj, $watchCollectionInterceptor);
            var internalArray = [];
            var internalObject = {};
            var initRun = true;
            var oldLength = 0;
            function $watchCollectionInterceptor(_value) {
              newValue = _value;
              var newLength, key, bothNaN, newItem, oldItem;
              // If the new value is undefined, then return undefined as the watch may be a one-time watch
              if (isUndefined(newValue))
                return;
              if (!isObject(newValue)) {
                // if primitive
                if (oldValue !== newValue) {
                  oldValue = newValue;
                  changeDetected++;
                }
              } else if (isArrayLike(newValue)) {
                if (oldValue !== internalArray) {
                  // we are transitioning from something which was not an array into array.
                  oldValue = internalArray;
                  oldLength = oldValue.length = 0;
                  changeDetected++;
                }
                newLength = newValue.length;
                if (oldLength !== newLength) {
                  // if lengths do not match we need to trigger change notification
                  changeDetected++;
                  oldValue.length = oldLength = newLength;
                }
                // copy the items to oldValue and look for changes.
                for (var i = 0; i < newLength; i++) {
                  oldItem = oldValue[i];
                  newItem = newValue[i];
                  bothNaN = oldItem !== oldItem && newItem !== newItem;
                  if (!bothNaN && oldItem !== newItem) {
                    changeDetected++;
                    oldValue[i] = newItem;
                  }
                }
              } else {
                if (oldValue !== internalObject) {
                  // we are transitioning from something which was not an object into object.
                  oldValue = internalObject = {};
                  oldLength = 0;
                  changeDetected++;
                }
                // copy the items to oldValue and look for changes.
                newLength = 0;
                for (key in newValue) {
                  if (newValue.hasOwnProperty(key)) {
                    newLength++;
                    newItem = newValue[key];
                    oldItem = oldValue[key];
                    if (key in oldValue) {
                      bothNaN = oldItem !== oldItem && newItem !== newItem;
                      if (!bothNaN && oldItem !== newItem) {
                        changeDetected++;
                        oldValue[key] = newItem;
                      }
                    } else {
                      oldLength++;
                      oldValue[key] = newItem;
                      changeDetected++;
                    }
                  }
                }
                if (oldLength > newLength) {
                  // we used to have more keys, need to find them and destroy them.
                  changeDetected++;
                  for (key in oldValue) {
                    if (!newValue.hasOwnProperty(key)) {
                      oldLength--;
                      delete oldValue[key];
                    }
                  }
                }
              }
              return changeDetected;
            }
            function $watchCollectionAction() {
              if (initRun) {
                initRun = false;
                listener(newValue, newValue, self);
              } else {
                listener(newValue, veryOldValue, self);
              }
              // make a copy for the next time a collection is changed
              if (trackVeryOldValue) {
                if (!isObject(newValue)) {
                  //primitive
                  veryOldValue = newValue;
                } else if (isArrayLike(newValue)) {
                  veryOldValue = new Array(newValue.length);
                  for (var i = 0; i < newValue.length; i++) {
                    veryOldValue[i] = newValue[i];
                  }
                } else {
                  // if object
                  veryOldValue = {};
                  for (var key in newValue) {
                    if (hasOwnProperty.call(newValue, key)) {
                      veryOldValue[key] = newValue[key];
                    }
                  }
                }
              }
            }
            return this.$watch(changeDetector, $watchCollectionAction);
          },
          $digest: function () {
            var watch, value, last, watchers, length, dirty, ttl = TTL, next, current, target = this, watchLog = [], logIdx, logMsg, asyncTask;
            beginPhase('$digest');
            // Check for changes to browser url that happened in sync before the call to $digest
            $browser.$$checkUrlChange();
            if (this === $rootScope && applyAsyncId !== null) {
              // If this is the root scope, and $applyAsync has scheduled a deferred $apply(), then
              // cancel the scheduled $apply and flush the queue of expressions to be evaluated.
              $browser.defer.cancel(applyAsyncId);
              flushApplyAsync();
            }
            lastDirtyWatch = null;
            do {
              // "while dirty" loop
              dirty = false;
              current = target;
              while (asyncQueue.length) {
                try {
                  asyncTask = asyncQueue.shift();
                  asyncTask.scope.$eval(asyncTask.expression, asyncTask.locals);
                } catch (e) {
                  $exceptionHandler(e);
                }
                lastDirtyWatch = null;
              }
              traverseScopesLoop:
                do {
                  // "traverse the scopes" loop
                  if (watchers = current.$$watchers) {
                    // process our watches
                    length = watchers.length;
                    while (length--) {
                      try {
                        watch = watchers[length];
                        // Most common watches are on primitives, in which case we can short
                        // circuit it with === operator, only when === fails do we use .equals
                        if (watch) {
                          if ((value = watch.get(current)) !== (last = watch.last) && !(watch.eq ? equals(value, last) : typeof value === 'number' && typeof last === 'number' && isNaN(value) && isNaN(last))) {
                            dirty = true;
                            lastDirtyWatch = watch;
                            watch.last = watch.eq ? copy(value, null) : value;
                            watch.fn(value, last === initWatchVal ? value : last, current);
                            if (ttl < 5) {
                              logIdx = 4 - ttl;
                              if (!watchLog[logIdx])
                                watchLog[logIdx] = [];
                              watchLog[logIdx].push({
                                msg: isFunction(watch.exp) ? 'fn: ' + (watch.exp.name || watch.exp.toString()) : watch.exp,
                                newVal: value,
                                oldVal: last
                              });
                            }
                          } else if (watch === lastDirtyWatch) {
                            // If the most recently dirty watcher is now clean, short circuit since the remaining watchers
                            // have already been tested.
                            dirty = false;
                            break traverseScopesLoop;
                          }
                        }
                      } catch (e) {
                        $exceptionHandler(e);
                      }
                    }
                  }
                  // Insanity Warning: scope depth-first traversal
                  // yes, this code is a bit crazy, but it works and we have tests to prove it!
                  // this piece should be kept in sync with the traversal in $broadcast
                  if (!(next = current.$$childHead || current !== target && current.$$nextSibling)) {
                    while (current !== target && !(next = current.$$nextSibling)) {
                      current = current.$parent;
                    }
                  }
                } while (current = next);
              // `break traverseScopesLoop;` takes us to here
              if ((dirty || asyncQueue.length) && !ttl--) {
                clearPhase();
                throw $rootScopeMinErr('infdig', '{0} $digest() iterations reached. Aborting!\n' + 'Watchers fired in the last 5 iterations: {1}', TTL, watchLog);
              }
            } while (dirty || asyncQueue.length);
            clearPhase();
            while (postDigestQueue.length) {
              try {
                postDigestQueue.shift()();
              } catch (e) {
                $exceptionHandler(e);
              }
            }
          },
          $destroy: function () {
            // we can't destroy the root scope or a scope that has been already destroyed
            if (this.$$destroyed)
              return;
            var parent = this.$parent;
            this.$broadcast('$destroy');
            this.$$destroyed = true;
            if (this === $rootScope)
              return;
            for (var eventName in this.$$listenerCount) {
              decrementListenerCount(this, this.$$listenerCount[eventName], eventName);
            }
            // sever all the references to parent scopes (after this cleanup, the current scope should
            // not be retained by any of our references and should be eligible for garbage collection)
            if (parent.$$childHead == this)
              parent.$$childHead = this.$$nextSibling;
            if (parent.$$childTail == this)
              parent.$$childTail = this.$$prevSibling;
            if (this.$$prevSibling)
              this.$$prevSibling.$$nextSibling = this.$$nextSibling;
            if (this.$$nextSibling)
              this.$$nextSibling.$$prevSibling = this.$$prevSibling;
            // Disable listeners, watchers and apply/digest methods
            this.$destroy = this.$digest = this.$apply = this.$evalAsync = this.$applyAsync = noop;
            this.$on = this.$watch = this.$watchGroup = function () {
              return noop;
            };
            this.$$listeners = {};
            // All of the code below is bogus code that works around V8's memory leak via optimized code
            // and inline caches.
            //
            // see:
            // - https://code.google.com/p/v8/issues/detail?id=2073#c26
            // - https://github.com/angular/angular.js/issues/6794#issuecomment-38648909
            // - https://github.com/angular/angular.js/issues/1313#issuecomment-10378451
            this.$parent = this.$$nextSibling = this.$$prevSibling = this.$$childHead = this.$$childTail = this.$root = this.$$watchers = null;
          },
          $eval: function (expr, locals) {
            return $parse(expr)(this, locals);
          },
          $evalAsync: function (expr, locals) {
            // if we are outside of an $digest loop and this is the first time we are scheduling async
            // task also schedule async auto-flush
            if (!$rootScope.$$phase && !asyncQueue.length) {
              $browser.defer(function () {
                if (asyncQueue.length) {
                  $rootScope.$digest();
                }
              });
            }
            asyncQueue.push({
              scope: this,
              expression: expr,
              locals: locals
            });
          },
          $$postDigest: function (fn) {
            postDigestQueue.push(fn);
          },
          $apply: function (expr) {
            try {
              beginPhase('$apply');
              return this.$eval(expr);
            } catch (e) {
              $exceptionHandler(e);
            } finally {
              clearPhase();
              try {
                $rootScope.$digest();
              } catch (e) {
                $exceptionHandler(e);
                throw e;
              }
            }
          },
          $applyAsync: function (expr) {
            var scope = this;
            expr && applyAsyncQueue.push($applyAsyncExpression);
            scheduleApplyAsync();
            function $applyAsyncExpression() {
              scope.$eval(expr);
            }
          },
          $on: function (name, listener) {
            var namedListeners = this.$$listeners[name];
            if (!namedListeners) {
              this.$$listeners[name] = namedListeners = [];
            }
            namedListeners.push(listener);
            var current = this;
            do {
              if (!current.$$listenerCount[name]) {
                current.$$listenerCount[name] = 0;
              }
              current.$$listenerCount[name]++;
            } while (current = current.$parent);
            var self = this;
            return function () {
              var indexOfListener = namedListeners.indexOf(listener);
              if (indexOfListener !== -1) {
                namedListeners[indexOfListener] = null;
                decrementListenerCount(self, 1, name);
              }
            };
          },
          $emit: function (name, args) {
            var empty = [], namedListeners, scope = this, stopPropagation = false, event = {
                name: name,
                targetScope: scope,
                stopPropagation: function () {
                  stopPropagation = true;
                },
                preventDefault: function () {
                  event.defaultPrevented = true;
                },
                defaultPrevented: false
              }, listenerArgs = concat([event], arguments, 1), i, length;
            do {
              namedListeners = scope.$$listeners[name] || empty;
              event.currentScope = scope;
              for (i = 0, length = namedListeners.length; i < length; i++) {
                // if listeners were deregistered, defragment the array
                if (!namedListeners[i]) {
                  namedListeners.splice(i, 1);
                  i--;
                  length--;
                  continue;
                }
                try {
                  //allow all listeners attached to the current scope to run
                  namedListeners[i].apply(null, listenerArgs);
                } catch (e) {
                  $exceptionHandler(e);
                }
              }
              //if any listener on the current scope stops propagation, prevent bubbling
              if (stopPropagation) {
                event.currentScope = null;
                return event;
              }
              //traverse upwards
              scope = scope.$parent;
            } while (scope);
            event.currentScope = null;
            return event;
          },
          $broadcast: function (name, args) {
            var target = this, current = target, next = target, event = {
                name: name,
                targetScope: target,
                preventDefault: function () {
                  event.defaultPrevented = true;
                },
                defaultPrevented: false
              };
            if (!target.$$listenerCount[name])
              return event;
            var listenerArgs = concat([event], arguments, 1), listeners, i, length;
            //down while you can, then up and next sibling or up and next sibling until back at root
            while (current = next) {
              event.currentScope = current;
              listeners = current.$$listeners[name] || [];
              for (i = 0, length = listeners.length; i < length; i++) {
                // if listeners were deregistered, defragment the array
                if (!listeners[i]) {
                  listeners.splice(i, 1);
                  i--;
                  length--;
                  continue;
                }
                try {
                  listeners[i].apply(null, listenerArgs);
                } catch (e) {
                  $exceptionHandler(e);
                }
              }
              // Insanity Warning: scope depth-first traversal
              // yes, this code is a bit crazy, but it works and we have tests to prove it!
              // this piece should be kept in sync with the traversal in $digest
              // (though it differs due to having the extra check for $$listenerCount)
              if (!(next = current.$$listenerCount[name] && current.$$childHead || current !== target && current.$$nextSibling)) {
                while (current !== target && !(next = current.$$nextSibling)) {
                  current = current.$parent;
                }
              }
            }
            event.currentScope = null;
            return event;
          }
        };
        var $rootScope = new Scope();
        //The internal queues. Expose them on the $rootScope for debugging/testing purposes.
        var asyncQueue = $rootScope.$$asyncQueue = [];
        var postDigestQueue = $rootScope.$$postDigestQueue = [];
        var applyAsyncQueue = $rootScope.$$applyAsyncQueue = [];
        return $rootScope;
        function beginPhase(phase) {
          if ($rootScope.$$phase) {
            throw $rootScopeMinErr('inprog', '{0} already in progress', $rootScope.$$phase);
          }
          $rootScope.$$phase = phase;
        }
        function clearPhase() {
          $rootScope.$$phase = null;
        }
        function decrementListenerCount(current, count, name) {
          do {
            current.$$listenerCount[name] -= count;
            if (current.$$listenerCount[name] === 0) {
              delete current.$$listenerCount[name];
            }
          } while (current = current.$parent);
        }
        /**
     * function used as an initial value for watchers.
     * because it's unique we can easily tell it apart from other values
     */
        function initWatchVal() {
        }
        function flushApplyAsync() {
          while (applyAsyncQueue.length) {
            try {
              applyAsyncQueue.shift()();
            } catch (e) {
              $exceptionHandler(e);
            }
          }
          applyAsyncId = null;
        }
        function scheduleApplyAsync() {
          if (applyAsyncId === null) {
            applyAsyncId = $browser.defer(function () {
              $rootScope.$apply(flushApplyAsync);
            });
          }
        }
      }
    ];
  }
  /**
 * @description
 * Private service to sanitize uris for links and images. Used by $compile and $sanitize.
 */
  function $$SanitizeUriProvider() {
    var aHrefSanitizationWhitelist = /^\s*(https?|ftp|mailto|tel|file):/, imgSrcSanitizationWhitelist = /^\s*((https?|ftp|file|blob):|data:image\/)/;
    /**
   * @description
   * Retrieves or overrides the default regular expression that is used for whitelisting of safe
   * urls during a[href] sanitization.
   *
   * The sanitization is a security measure aimed at prevent XSS attacks via html links.
   *
   * Any url about to be assigned to a[href] via data-binding is first normalized and turned into
   * an absolute url. Afterwards, the url is matched against the `aHrefSanitizationWhitelist`
   * regular expression. If a match is found, the original url is written into the dom. Otherwise,
   * the absolute url is prefixed with `'unsafe:'` string and only then is it written into the DOM.
   *
   * @param {RegExp=} regexp New regexp to whitelist urls with.
   * @returns {RegExp|ng.$compileProvider} Current RegExp if called without value or self for
   *    chaining otherwise.
   */
    this.aHrefSanitizationWhitelist = function (regexp) {
      if (isDefined(regexp)) {
        aHrefSanitizationWhitelist = regexp;
        return this;
      }
      return aHrefSanitizationWhitelist;
    };
    /**
   * @description
   * Retrieves or overrides the default regular expression that is used for whitelisting of safe
   * urls during img[src] sanitization.
   *
   * The sanitization is a security measure aimed at prevent XSS attacks via html links.
   *
   * Any url about to be assigned to img[src] via data-binding is first normalized and turned into
   * an absolute url. Afterwards, the url is matched against the `imgSrcSanitizationWhitelist`
   * regular expression. If a match is found, the original url is written into the dom. Otherwise,
   * the absolute url is prefixed with `'unsafe:'` string and only then is it written into the DOM.
   *
   * @param {RegExp=} regexp New regexp to whitelist urls with.
   * @returns {RegExp|ng.$compileProvider} Current RegExp if called without value or self for
   *    chaining otherwise.
   */
    this.imgSrcSanitizationWhitelist = function (regexp) {
      if (isDefined(regexp)) {
        imgSrcSanitizationWhitelist = regexp;
        return this;
      }
      return imgSrcSanitizationWhitelist;
    };
    this.$get = function () {
      return function sanitizeUri(uri, isImage) {
        var regex = isImage ? imgSrcSanitizationWhitelist : aHrefSanitizationWhitelist;
        var normalizedVal;
        normalizedVal = urlResolve(uri).href;
        if (normalizedVal !== '' && !normalizedVal.match(regex)) {
          return 'unsafe:' + normalizedVal;
        }
        return uri;
      };
    };
  }
  /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *     Any commits to this file should be reviewed with security in mind.  *
 *   Changes to this file can potentially create security vulnerabilities. *
 *          An approval from 2 Core members with history of modifying      *
 *                         this file is required.                          *
 *                                                                         *
 *  Does the change somehow allow for arbitrary javascript to be executed? *
 *    Or allows for someone to change the prototype of built-in objects?   *
 *     Or gives undesired access to variables likes document or window?    *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
  var $sceMinErr = minErr('$sce');
  var SCE_CONTEXTS = {
      HTML: 'html',
      CSS: 'css',
      URL: 'url',
      RESOURCE_URL: 'resourceUrl',
      JS: 'js'
    };
  // Helper functions follow.
  function adjustMatcher(matcher) {
    if (matcher === 'self') {
      return matcher;
    } else if (isString(matcher)) {
      // Strings match exactly except for 2 wildcards - '*' and '**'.
      // '*' matches any character except those from the set ':/.?&'.
      // '**' matches any character (like .* in a RegExp).
      // More than 2 *'s raises an error as it's ill defined.
      if (matcher.indexOf('***') > -1) {
        throw $sceMinErr('iwcard', 'Illegal sequence *** in string matcher.  String: {0}', matcher);
      }
      matcher = escapeForRegexp(matcher).replace('\\*\\*', '.*').replace('\\*', '[^:/.?&;]*');
      return new RegExp('^' + matcher + '$');
    } else if (isRegExp(matcher)) {
      // The only other type of matcher allowed is a Regexp.
      // Match entire URL / disallow partial matches.
      // Flags are reset (i.e. no global, ignoreCase or multiline)
      return new RegExp('^' + matcher.source + '$');
    } else {
      throw $sceMinErr('imatcher', 'Matchers may only be "self", string patterns or RegExp objects');
    }
  }
  function adjustMatchers(matchers) {
    var adjustedMatchers = [];
    if (isDefined(matchers)) {
      forEach(matchers, function (matcher) {
        adjustedMatchers.push(adjustMatcher(matcher));
      });
    }
    return adjustedMatchers;
  }
  /**
 * @ngdoc service
 * @name $sceDelegate
 * @kind function
 *
 * @description
 *
 * `$sceDelegate` is a service that is used by the `$sce` service to provide {@link ng.$sce Strict
 * Contextual Escaping (SCE)} services to AngularJS.
 *
 * Typically, you would configure or override the {@link ng.$sceDelegate $sceDelegate} instead of
 * the `$sce` service to customize the way Strict Contextual Escaping works in AngularJS.  This is
 * because, while the `$sce` provides numerous shorthand methods, etc., you really only need to
 * override 3 core functions (`trustAs`, `getTrusted` and `valueOf`) to replace the way things
 * work because `$sce` delegates to `$sceDelegate` for these operations.
 *
 * Refer {@link ng.$sceDelegateProvider $sceDelegateProvider} to configure this service.
 *
 * The default instance of `$sceDelegate` should work out of the box with little pain.  While you
 * can override it completely to change the behavior of `$sce`, the common case would
 * involve configuring the {@link ng.$sceDelegateProvider $sceDelegateProvider} instead by setting
 * your own whitelists and blacklists for trusting URLs used for loading AngularJS resources such as
 * templates.  Refer {@link ng.$sceDelegateProvider#resourceUrlWhitelist
 * $sceDelegateProvider.resourceUrlWhitelist} and {@link
 * ng.$sceDelegateProvider#resourceUrlBlacklist $sceDelegateProvider.resourceUrlBlacklist}
 */
  /**
 * @ngdoc provider
 * @name $sceDelegateProvider
 * @description
 *
 * The `$sceDelegateProvider` provider allows developers to configure the {@link ng.$sceDelegate
 * $sceDelegate} service.  This allows one to get/set the whitelists and blacklists used to ensure
 * that the URLs used for sourcing Angular templates are safe.  Refer {@link
 * ng.$sceDelegateProvider#resourceUrlWhitelist $sceDelegateProvider.resourceUrlWhitelist} and
 * {@link ng.$sceDelegateProvider#resourceUrlBlacklist $sceDelegateProvider.resourceUrlBlacklist}
 *
 * For the general details about this service in Angular, read the main page for {@link ng.$sce
 * Strict Contextual Escaping (SCE)}.
 *
 * **Example**:  Consider the following case. <a name="example"></a>
 *
 * - your app is hosted at url `http://myapp.example.com/`
 * - but some of your templates are hosted on other domains you control such as
 *   `http://srv01.assets.example.com/`,  `http://srv02.assets.example.com/`, etc.
 * - and you have an open redirect at `http://myapp.example.com/clickThru?...`.
 *
 * Here is what a secure configuration for this scenario might look like:
 *
 * ```
 *  angular.module('myApp', []).config(function($sceDelegateProvider) {
 *    $sceDelegateProvider.resourceUrlWhitelist([
 *      // Allow same origin resource loads.
 *      'self',
 *      // Allow loading from our assets domain.  Notice the difference between * and **.
 *      'http://srv*.assets.example.com/**'
 *    ]);
 *
 *    // The blacklist overrides the whitelist so the open redirect here is blocked.
 *    $sceDelegateProvider.resourceUrlBlacklist([
 *      'http://myapp.example.com/clickThru**'
 *    ]);
 *  });
 * ```
 */
  function $SceDelegateProvider() {
    this.SCE_CONTEXTS = SCE_CONTEXTS;
    // Resource URLs can also be trusted by policy.
    var resourceUrlWhitelist = ['self'], resourceUrlBlacklist = [];
    /**
   * @ngdoc method
   * @name $sceDelegateProvider#resourceUrlWhitelist
   * @kind function
   *
   * @param {Array=} whitelist When provided, replaces the resourceUrlWhitelist with the value
   *     provided.  This must be an array or null.  A snapshot of this array is used so further
   *     changes to the array are ignored.
   *
   *     Follow {@link ng.$sce#resourceUrlPatternItem this link} for a description of the items
   *     allowed in this array.
   *
   *     Note: **an empty whitelist array will block all URLs**!
   *
   * @return {Array} the currently set whitelist array.
   *
   * The **default value** when no whitelist has been explicitly set is `['self']` allowing only
   * same origin resource requests.
   *
   * @description
   * Sets/Gets the whitelist of trusted resource URLs.
   */
    this.resourceUrlWhitelist = function (value) {
      if (arguments.length) {
        resourceUrlWhitelist = adjustMatchers(value);
      }
      return resourceUrlWhitelist;
    };
    /**
   * @ngdoc method
   * @name $sceDelegateProvider#resourceUrlBlacklist
   * @kind function
   *
   * @param {Array=} blacklist When provided, replaces the resourceUrlBlacklist with the value
   *     provided.  This must be an array or null.  A snapshot of this array is used so further
   *     changes to the array are ignored.
   *
   *     Follow {@link ng.$sce#resourceUrlPatternItem this link} for a description of the items
   *     allowed in this array.
   *
   *     The typical usage for the blacklist is to **block
   *     [open redirects](http://cwe.mitre.org/data/definitions/601.html)** served by your domain as
   *     these would otherwise be trusted but actually return content from the redirected domain.
   *
   *     Finally, **the blacklist overrides the whitelist** and has the final say.
   *
   * @return {Array} the currently set blacklist array.
   *
   * The **default value** when no whitelist has been explicitly set is the empty array (i.e. there
   * is no blacklist.)
   *
   * @description
   * Sets/Gets the blacklist of trusted resource URLs.
   */
    this.resourceUrlBlacklist = function (value) {
      if (arguments.length) {
        resourceUrlBlacklist = adjustMatchers(value);
      }
      return resourceUrlBlacklist;
    };
    this.$get = [
      '$injector',
      function ($injector) {
        var htmlSanitizer = function htmlSanitizer(html) {
          throw $sceMinErr('unsafe', 'Attempting to use an unsafe value in a safe context.');
        };
        if ($injector.has('$sanitize')) {
          htmlSanitizer = $injector.get('$sanitize');
        }
        function matchUrl(matcher, parsedUrl) {
          if (matcher === 'self') {
            return urlIsSameOrigin(parsedUrl);
          } else {
            // definitely a regex.  See adjustMatchers()
            return !!matcher.exec(parsedUrl.href);
          }
        }
        function isResourceUrlAllowedByPolicy(url) {
          var parsedUrl = urlResolve(url.toString());
          var i, n, allowed = false;
          // Ensure that at least one item from the whitelist allows this url.
          for (i = 0, n = resourceUrlWhitelist.length; i < n; i++) {
            if (matchUrl(resourceUrlWhitelist[i], parsedUrl)) {
              allowed = true;
              break;
            }
          }
          if (allowed) {
            // Ensure that no item from the blacklist blocked this url.
            for (i = 0, n = resourceUrlBlacklist.length; i < n; i++) {
              if (matchUrl(resourceUrlBlacklist[i], parsedUrl)) {
                allowed = false;
                break;
              }
            }
          }
          return allowed;
        }
        function generateHolderType(Base) {
          var holderType = function TrustedValueHolderType(trustedValue) {
            this.$$unwrapTrustedValue = function () {
              return trustedValue;
            };
          };
          if (Base) {
            holderType.prototype = new Base();
          }
          holderType.prototype.valueOf = function sceValueOf() {
            return this.$$unwrapTrustedValue();
          };
          holderType.prototype.toString = function sceToString() {
            return this.$$unwrapTrustedValue().toString();
          };
          return holderType;
        }
        var trustedValueHolderBase = generateHolderType(), byType = {};
        byType[SCE_CONTEXTS.HTML] = generateHolderType(trustedValueHolderBase);
        byType[SCE_CONTEXTS.CSS] = generateHolderType(trustedValueHolderBase);
        byType[SCE_CONTEXTS.URL] = generateHolderType(trustedValueHolderBase);
        byType[SCE_CONTEXTS.JS] = generateHolderType(trustedValueHolderBase);
        byType[SCE_CONTEXTS.RESOURCE_URL] = generateHolderType(byType[SCE_CONTEXTS.URL]);
        /**
     * @ngdoc method
     * @name $sceDelegate#trustAs
     *
     * @description
     * Returns an object that is trusted by angular for use in specified strict
     * contextual escaping contexts (such as ng-bind-html, ng-include, any src
     * attribute interpolation, any dom event binding attribute interpolation
     * such as for onclick,  etc.) that uses the provided value.
     * See {@link ng.$sce $sce} for enabling strict contextual escaping.
     *
     * @param {string} type The kind of context in which this value is safe for use.  e.g. url,
     *   resourceUrl, html, js and css.
     * @param {*} value The value that that should be considered trusted/safe.
     * @returns {*} A value that can be used to stand in for the provided `value` in places
     * where Angular expects a $sce.trustAs() return value.
     */
        function trustAs(type, trustedValue) {
          var Constructor = byType.hasOwnProperty(type) ? byType[type] : null;
          if (!Constructor) {
            throw $sceMinErr('icontext', 'Attempted to trust a value in invalid context. Context: {0}; Value: {1}', type, trustedValue);
          }
          if (trustedValue === null || trustedValue === undefined || trustedValue === '') {
            return trustedValue;
          }
          // All the current contexts in SCE_CONTEXTS happen to be strings.  In order to avoid trusting
          // mutable objects, we ensure here that the value passed in is actually a string.
          if (typeof trustedValue !== 'string') {
            throw $sceMinErr('itype', 'Attempted to trust a non-string value in a content requiring a string: Context: {0}', type);
          }
          return new Constructor(trustedValue);
        }
        /**
     * @ngdoc method
     * @name $sceDelegate#valueOf
     *
     * @description
     * If the passed parameter had been returned by a prior call to {@link ng.$sceDelegate#trustAs
     * `$sceDelegate.trustAs`}, returns the value that had been passed to {@link
     * ng.$sceDelegate#trustAs `$sceDelegate.trustAs`}.
     *
     * If the passed parameter is not a value that had been returned by {@link
     * ng.$sceDelegate#trustAs `$sceDelegate.trustAs`}, returns it as-is.
     *
     * @param {*} value The result of a prior {@link ng.$sceDelegate#trustAs `$sceDelegate.trustAs`}
     *      call or anything else.
     * @returns {*} The `value` that was originally provided to {@link ng.$sceDelegate#trustAs
     *     `$sceDelegate.trustAs`} if `value` is the result of such a call.  Otherwise, returns
     *     `value` unchanged.
     */
        function valueOf(maybeTrusted) {
          if (maybeTrusted instanceof trustedValueHolderBase) {
            return maybeTrusted.$$unwrapTrustedValue();
          } else {
            return maybeTrusted;
          }
        }
        /**
     * @ngdoc method
     * @name $sceDelegate#getTrusted
     *
     * @description
     * Takes the result of a {@link ng.$sceDelegate#trustAs `$sceDelegate.trustAs`} call and
     * returns the originally supplied value if the queried context type is a supertype of the
     * created type.  If this condition isn't satisfied, throws an exception.
     *
     * @param {string} type The kind of context in which this value is to be used.
     * @param {*} maybeTrusted The result of a prior {@link ng.$sceDelegate#trustAs
     *     `$sceDelegate.trustAs`} call.
     * @returns {*} The value the was originally provided to {@link ng.$sceDelegate#trustAs
     *     `$sceDelegate.trustAs`} if valid in this context.  Otherwise, throws an exception.
     */
        function getTrusted(type, maybeTrusted) {
          if (maybeTrusted === null || maybeTrusted === undefined || maybeTrusted === '') {
            return maybeTrusted;
          }
          var constructor = byType.hasOwnProperty(type) ? byType[type] : null;
          if (constructor && maybeTrusted instanceof constructor) {
            return maybeTrusted.$$unwrapTrustedValue();
          }
          // If we get here, then we may only take one of two actions.
          // 1. sanitize the value for the requested type, or
          // 2. throw an exception.
          if (type === SCE_CONTEXTS.RESOURCE_URL) {
            if (isResourceUrlAllowedByPolicy(maybeTrusted)) {
              return maybeTrusted;
            } else {
              throw $sceMinErr('insecurl', 'Blocked loading resource from url not allowed by $sceDelegate policy.  URL: {0}', maybeTrusted.toString());
            }
          } else if (type === SCE_CONTEXTS.HTML) {
            return htmlSanitizer(maybeTrusted);
          }
          throw $sceMinErr('unsafe', 'Attempting to use an unsafe value in a safe context.');
        }
        return {
          trustAs: trustAs,
          getTrusted: getTrusted,
          valueOf: valueOf
        };
      }
    ];
  }
  /**
 * @ngdoc provider
 * @name $sceProvider
 * @description
 *
 * The $sceProvider provider allows developers to configure the {@link ng.$sce $sce} service.
 * -   enable/disable Strict Contextual Escaping (SCE) in a module
 * -   override the default implementation with a custom delegate
 *
 * Read more about {@link ng.$sce Strict Contextual Escaping (SCE)}.
 */
  /* jshint maxlen: false*/
  /**
 * @ngdoc service
 * @name $sce
 * @kind function
 *
 * @description
 *
 * `$sce` is a service that provides Strict Contextual Escaping services to AngularJS.
 *
 * # Strict Contextual Escaping
 *
 * Strict Contextual Escaping (SCE) is a mode in which AngularJS requires bindings in certain
 * contexts to result in a value that is marked as safe to use for that context.  One example of
 * such a context is binding arbitrary html controlled by the user via `ng-bind-html`.  We refer
 * to these contexts as privileged or SCE contexts.
 *
 * As of version 1.2, Angular ships with SCE enabled by default.
 *
 * Note:  When enabled (the default), IE<11 in quirks mode is not supported.  In this mode, IE<11 allow
 * one to execute arbitrary javascript by the use of the expression() syntax.  Refer
 * <http://blogs.msdn.com/b/ie/archive/2008/10/16/ending-expressions.aspx> to learn more about them.
 * You can ensure your document is in standards mode and not quirks mode by adding `<!doctype html>`
 * to the top of your HTML document.
 *
 * SCE assists in writing code in way that (a) is secure by default and (b) makes auditing for
 * security vulnerabilities such as XSS, clickjacking, etc. a lot easier.
 *
 * Here's an example of a binding in a privileged context:
 *
 * ```
 * <input ng-model="userHtml">
 * <div ng-bind-html="userHtml"></div>
 * ```
 *
 * Notice that `ng-bind-html` is bound to `userHtml` controlled by the user.  With SCE
 * disabled, this application allows the user to render arbitrary HTML into the DIV.
 * In a more realistic example, one may be rendering user comments, blog articles, etc. via
 * bindings.  (HTML is just one example of a context where rendering user controlled input creates
 * security vulnerabilities.)
 *
 * For the case of HTML, you might use a library, either on the client side, or on the server side,
 * to sanitize unsafe HTML before binding to the value and rendering it in the document.
 *
 * How would you ensure that every place that used these types of bindings was bound to a value that
 * was sanitized by your library (or returned as safe for rendering by your server?)  How can you
 * ensure that you didn't accidentally delete the line that sanitized the value, or renamed some
 * properties/fields and forgot to update the binding to the sanitized value?
 *
 * To be secure by default, you want to ensure that any such bindings are disallowed unless you can
 * determine that something explicitly says it's safe to use a value for binding in that
 * context.  You can then audit your code (a simple grep would do) to ensure that this is only done
 * for those values that you can easily tell are safe - because they were received from your server,
 * sanitized by your library, etc.  You can organize your codebase to help with this - perhaps
 * allowing only the files in a specific directory to do this.  Ensuring that the internal API
 * exposed by that code doesn't markup arbitrary values as safe then becomes a more manageable task.
 *
 * In the case of AngularJS' SCE service, one uses {@link ng.$sce#trustAs $sce.trustAs}
 * (and shorthand methods such as {@link ng.$sce#trustAsHtml $sce.trustAsHtml}, etc.) to
 * obtain values that will be accepted by SCE / privileged contexts.
 *
 *
 * ## How does it work?
 *
 * In privileged contexts, directives and code will bind to the result of {@link ng.$sce#getTrusted
 * $sce.getTrusted(context, value)} rather than to the value directly.  Directives use {@link
 * ng.$sce#parseAs $sce.parseAs} rather than `$parse` to watch attribute bindings, which performs the
 * {@link ng.$sce#getTrusted $sce.getTrusted} behind the scenes on non-constant literals.
 *
 * As an example, {@link ng.directive:ngBindHtml ngBindHtml} uses {@link
 * ng.$sce#parseAsHtml $sce.parseAsHtml(binding expression)}.  Here's the actual code (slightly
 * simplified):
 *
 * ```
 * var ngBindHtmlDirective = ['$sce', function($sce) {
 *   return function(scope, element, attr) {
 *     scope.$watch($sce.parseAsHtml(attr.ngBindHtml), function(value) {
 *       element.html(value || '');
 *     });
 *   };
 * }];
 * ```
 *
 * ## Impact on loading templates
 *
 * This applies both to the {@link ng.directive:ngInclude `ng-include`} directive as well as
 * `templateUrl`'s specified by {@link guide/directive directives}.
 *
 * By default, Angular only loads templates from the same domain and protocol as the application
 * document.  This is done by calling {@link ng.$sce#getTrustedResourceUrl
 * $sce.getTrustedResourceUrl} on the template URL.  To load templates from other domains and/or
 * protocols, you may either either {@link ng.$sceDelegateProvider#resourceUrlWhitelist whitelist
 * them} or {@link ng.$sce#trustAsResourceUrl wrap it} into a trusted value.
 *
 * *Please note*:
 * The browser's
 * [Same Origin Policy](https://code.google.com/p/browsersec/wiki/Part2#Same-origin_policy_for_XMLHttpRequest)
 * and [Cross-Origin Resource Sharing (CORS)](http://www.w3.org/TR/cors/)
 * policy apply in addition to this and may further restrict whether the template is successfully
 * loaded.  This means that without the right CORS policy, loading templates from a different domain
 * won't work on all browsers.  Also, loading templates from `file://` URL does not work on some
 * browsers.
 *
 * ## This feels like too much overhead
 *
 * It's important to remember that SCE only applies to interpolation expressions.
 *
 * If your expressions are constant literals, they're automatically trusted and you don't need to
 * call `$sce.trustAs` on them (remember to include the `ngSanitize` module) (e.g.
 * `<div ng-bind-html="'<b>implicitly trusted</b>'"></div>`) just works.
 *
 * Additionally, `a[href]` and `img[src]` automatically sanitize their URLs and do not pass them
 * through {@link ng.$sce#getTrusted $sce.getTrusted}.  SCE doesn't play a role here.
 *
 * The included {@link ng.$sceDelegate $sceDelegate} comes with sane defaults to allow you to load
 * templates in `ng-include` from your application's domain without having to even know about SCE.
 * It blocks loading templates from other domains or loading templates over http from an https
 * served document.  You can change these by setting your own custom {@link
 * ng.$sceDelegateProvider#resourceUrlWhitelist whitelists} and {@link
 * ng.$sceDelegateProvider#resourceUrlBlacklist blacklists} for matching such URLs.
 *
 * This significantly reduces the overhead.  It is far easier to pay the small overhead and have an
 * application that's secure and can be audited to verify that with much more ease than bolting
 * security onto an application later.
 *
 * <a name="contexts"></a>
 * ## What trusted context types are supported?
 *
 * | Context             | Notes          |
 * |---------------------|----------------|
 * | `$sce.HTML`         | For HTML that's safe to source into the application.  The {@link ng.directive:ngBindHtml ngBindHtml} directive uses this context for bindings. If an unsafe value is encountered and the {@link ngSanitize $sanitize} module is present this will sanitize the value instead of throwing an error. |
 * | `$sce.CSS`          | For CSS that's safe to source into the application.  Currently unused.  Feel free to use it in your own directives. |
 * | `$sce.URL`          | For URLs that are safe to follow as links.  Currently unused (`<a href=` and `<img src=` sanitize their urls and don't constitute an SCE context. |
 * | `$sce.RESOURCE_URL` | For URLs that are not only safe to follow as links, but whose contents are also safe to include in your application.  Examples include `ng-include`, `src` / `ngSrc` bindings for tags other than `IMG` (e.g. `IFRAME`, `OBJECT`, etc.)  <br><br>Note that `$sce.RESOURCE_URL` makes a stronger statement about the URL than `$sce.URL` does and therefore contexts requiring values trusted for `$sce.RESOURCE_URL` can be used anywhere that values trusted for `$sce.URL` are required. |
 * | `$sce.JS`           | For JavaScript that is safe to execute in your application's context.  Currently unused.  Feel free to use it in your own directives. |
 *
 * ## Format of items in {@link ng.$sceDelegateProvider#resourceUrlWhitelist resourceUrlWhitelist}/{@link ng.$sceDelegateProvider#resourceUrlBlacklist Blacklist} <a name="resourceUrlPatternItem"></a>
 *
 *  Each element in these arrays must be one of the following:
 *
 *  - **'self'**
 *    - The special **string**, `'self'`, can be used to match against all URLs of the **same
 *      domain** as the application document using the **same protocol**.
 *  - **String** (except the special value `'self'`)
 *    - The string is matched against the full *normalized / absolute URL* of the resource
 *      being tested (substring matches are not good enough.)
 *    - There are exactly **two wildcard sequences** - `*` and `**`.  All other characters
 *      match themselves.
 *    - `*`: matches zero or more occurrences of any character other than one of the following 6
 *      characters: '`:`', '`/`', '`.`', '`?`', '`&`' and ';'.  It's a useful wildcard for use
 *      in a whitelist.
 *    - `**`: matches zero or more occurrences of *any* character.  As such, it's not
 *      appropriate for use in a scheme, domain, etc. as it would match too much.  (e.g.
 *      http://**.example.com/ would match http://evil.com/?ignore=.example.com/ and that might
 *      not have been the intention.)  Its usage at the very end of the path is ok.  (e.g.
 *      http://foo.example.com/templates/**).
 *  - **RegExp** (*see caveat below*)
 *    - *Caveat*:  While regular expressions are powerful and offer great flexibility,  their syntax
 *      (and all the inevitable escaping) makes them *harder to maintain*.  It's easy to
 *      accidentally introduce a bug when one updates a complex expression (imho, all regexes should
 *      have good test coverage).  For instance, the use of `.` in the regex is correct only in a
 *      small number of cases.  A `.` character in the regex used when matching the scheme or a
 *      subdomain could be matched against a `:` or literal `.` that was likely not intended.   It
 *      is highly recommended to use the string patterns and only fall back to regular expressions
 *      as a last resort.
 *    - The regular expression must be an instance of RegExp (i.e. not a string.)  It is
 *      matched against the **entire** *normalized / absolute URL* of the resource being tested
 *      (even when the RegExp did not have the `^` and `$` codes.)  In addition, any flags
 *      present on the RegExp (such as multiline, global, ignoreCase) are ignored.
 *    - If you are generating your JavaScript from some other templating engine (not
 *      recommended, e.g. in issue [#4006](https://github.com/angular/angular.js/issues/4006)),
 *      remember to escape your regular expression (and be aware that you might need more than
 *      one level of escaping depending on your templating engine and the way you interpolated
 *      the value.)  Do make use of your platform's escaping mechanism as it might be good
 *      enough before coding your own.  E.g. Ruby has
 *      [Regexp.escape(str)](http://www.ruby-doc.org/core-2.0.0/Regexp.html#method-c-escape)
 *      and Python has [re.escape](http://docs.python.org/library/re.html#re.escape).
 *      Javascript lacks a similar built in function for escaping.  Take a look at Google
 *      Closure library's [goog.string.regExpEscape(s)](
 *      http://docs.closure-library.googlecode.com/git/closure_goog_string_string.js.source.html#line962).
 *
 * Refer {@link ng.$sceDelegateProvider $sceDelegateProvider} for an example.
 *
 * ## Show me an example using SCE.
 *
 * <example module="mySceApp" deps="angular-sanitize.js">
 * <file name="index.html">
 *   <div ng-controller="AppController as myCtrl">
 *     <i ng-bind-html="myCtrl.explicitlyTrustedHtml" id="explicitlyTrustedHtml"></i><br><br>
 *     <b>User comments</b><br>
 *     By default, HTML that isn't explicitly trusted (e.g. Alice's comment) is sanitized when
 *     $sanitize is available.  If $sanitize isn't available, this results in an error instead of an
 *     exploit.
 *     <div class="well">
 *       <div ng-repeat="userComment in myCtrl.userComments">
 *         <b>{{userComment.name}}</b>:
 *         <span ng-bind-html="userComment.htmlComment" class="htmlComment"></span>
 *         <br>
 *       </div>
 *     </div>
 *   </div>
 * </file>
 *
 * <file name="script.js">
 *   angular.module('mySceApp', ['ngSanitize'])
 *     .controller('AppController', ['$http', '$templateCache', '$sce',
 *       function($http, $templateCache, $sce) {
 *         var self = this;
 *         $http.get("test_data.json", {cache: $templateCache}).success(function(userComments) {
 *           self.userComments = userComments;
 *         });
 *         self.explicitlyTrustedHtml = $sce.trustAsHtml(
 *             '<span onmouseover="this.textContent=&quot;Explicitly trusted HTML bypasses ' +
 *             'sanitization.&quot;">Hover over this text.</span>');
 *       }]);
 * </file>
 *
 * <file name="test_data.json">
 * [
 *   { "name": "Alice",
 *     "htmlComment":
 *         "<span onmouseover='this.textContent=\"PWN3D!\"'>Is <i>anyone</i> reading this?</span>"
 *   },
 *   { "name": "Bob",
 *     "htmlComment": "<i>Yes!</i>  Am I the only other one?"
 *   }
 * ]
 * </file>
 *
 * <file name="protractor.js" type="protractor">
 *   describe('SCE doc demo', function() {
 *     it('should sanitize untrusted values', function() {
 *       expect(element.all(by.css('.htmlComment')).first().getInnerHtml())
 *           .toBe('<span>Is <i>anyone</i> reading this?</span>');
 *     });
 *
 *     it('should NOT sanitize explicitly trusted values', function() {
 *       expect(element(by.id('explicitlyTrustedHtml')).getInnerHtml()).toBe(
 *           '<span onmouseover="this.textContent=&quot;Explicitly trusted HTML bypasses ' +
 *           'sanitization.&quot;">Hover over this text.</span>');
 *     });
 *   });
 * </file>
 * </example>
 *
 *
 *
 * ## Can I disable SCE completely?
 *
 * Yes, you can.  However, this is strongly discouraged.  SCE gives you a lot of security benefits
 * for little coding overhead.  It will be much harder to take an SCE disabled application and
 * either secure it on your own or enable SCE at a later stage.  It might make sense to disable SCE
 * for cases where you have a lot of existing code that was written before SCE was introduced and
 * you're migrating them a module at a time.
 *
 * That said, here's how you can completely disable SCE:
 *
 * ```
 * angular.module('myAppWithSceDisabledmyApp', []).config(function($sceProvider) {
 *   // Completely disable SCE.  For demonstration purposes only!
 *   // Do not use in new projects.
 *   $sceProvider.enabled(false);
 * });
 * ```
 *
 */
  /* jshint maxlen: 100 */
  function $SceProvider() {
    var enabled = true;
    /**
   * @ngdoc method
   * @name $sceProvider#enabled
   * @kind function
   *
   * @param {boolean=} value If provided, then enables/disables SCE.
   * @return {boolean} true if SCE is enabled, false otherwise.
   *
   * @description
   * Enables/disables SCE and returns the current value.
   */
    this.enabled = function (value) {
      if (arguments.length) {
        enabled = !!value;
      }
      return enabled;
    };
    /* Design notes on the default implementation for SCE.
   *
   * The API contract for the SCE delegate
   * -------------------------------------
   * The SCE delegate object must provide the following 3 methods:
   *
   * - trustAs(contextEnum, value)
   *     This method is used to tell the SCE service that the provided value is OK to use in the
   *     contexts specified by contextEnum.  It must return an object that will be accepted by
   *     getTrusted() for a compatible contextEnum and return this value.
   *
   * - valueOf(value)
   *     For values that were not produced by trustAs(), return them as is.  For values that were
   *     produced by trustAs(), return the corresponding input value to trustAs.  Basically, if
   *     trustAs is wrapping the given values into some type, this operation unwraps it when given
   *     such a value.
   *
   * - getTrusted(contextEnum, value)
   *     This function should return the a value that is safe to use in the context specified by
   *     contextEnum or throw and exception otherwise.
   *
   * NOTE: This contract deliberately does NOT state that values returned by trustAs() must be
   * opaque or wrapped in some holder object.  That happens to be an implementation detail.  For
   * instance, an implementation could maintain a registry of all trusted objects by context.  In
   * such a case, trustAs() would return the same object that was passed in.  getTrusted() would
   * return the same object passed in if it was found in the registry under a compatible context or
   * throw an exception otherwise.  An implementation might only wrap values some of the time based
   * on some criteria.  getTrusted() might return a value and not throw an exception for special
   * constants or objects even if not wrapped.  All such implementations fulfill this contract.
   *
   *
   * A note on the inheritance model for SCE contexts
   * ------------------------------------------------
   * I've used inheritance and made RESOURCE_URL wrapped types a subtype of URL wrapped types.  This
   * is purely an implementation details.
   *
   * The contract is simply this:
   *
   *     getTrusted($sce.RESOURCE_URL, value) succeeding implies that getTrusted($sce.URL, value)
   *     will also succeed.
   *
   * Inheritance happens to capture this in a natural way.  In some future, we
   * may not use inheritance anymore.  That is OK because no code outside of
   * sce.js and sceSpecs.js would need to be aware of this detail.
   */
    this.$get = [
      '$parse',
      '$sceDelegate',
      function ($parse, $sceDelegate) {
        // Prereq: Ensure that we're not running in IE<11 quirks mode.  In that mode, IE < 11 allow
        // the "expression(javascript expression)" syntax which is insecure.
        if (enabled && msie < 8) {
          throw $sceMinErr('iequirks', 'Strict Contextual Escaping does not support Internet Explorer version < 11 in quirks ' + 'mode.  You can fix this by adding the text <!doctype html> to the top of your HTML ' + 'document.  See http://docs.angularjs.org/api/ng.$sce for more information.');
        }
        var sce = shallowCopy(SCE_CONTEXTS);
        /**
     * @ngdoc method
     * @name $sce#isEnabled
     * @kind function
     *
     * @return {Boolean} true if SCE is enabled, false otherwise.  If you want to set the value, you
     * have to do it at module config time on {@link ng.$sceProvider $sceProvider}.
     *
     * @description
     * Returns a boolean indicating if SCE is enabled.
     */
        sce.isEnabled = function () {
          return enabled;
        };
        sce.trustAs = $sceDelegate.trustAs;
        sce.getTrusted = $sceDelegate.getTrusted;
        sce.valueOf = $sceDelegate.valueOf;
        if (!enabled) {
          sce.trustAs = sce.getTrusted = function (type, value) {
            return value;
          };
          sce.valueOf = identity;
        }
        /**
     * @ngdoc method
     * @name $sce#parseAs
     *
     * @description
     * Converts Angular {@link guide/expression expression} into a function.  This is like {@link
     * ng.$parse $parse} and is identical when the expression is a literal constant.  Otherwise, it
     * wraps the expression in a call to {@link ng.$sce#getTrusted $sce.getTrusted(*type*,
     * *result*)}
     *
     * @param {string} type The kind of SCE context in which this result will be used.
     * @param {string} expression String expression to compile.
     * @returns {function(context, locals)} a function which represents the compiled expression:
     *
     *    * `context` – `{object}` – an object against which any expressions embedded in the strings
     *      are evaluated against (typically a scope object).
     *    * `locals` – `{object=}` – local variables context object, useful for overriding values in
     *      `context`.
     */
        sce.parseAs = function sceParseAs(type, expr) {
          var parsed = $parse(expr);
          if (parsed.literal && parsed.constant) {
            return parsed;
          } else {
            return $parse(expr, function (value) {
              return sce.getTrusted(type, value);
            });
          }
        };
        /**
     * @ngdoc method
     * @name $sce#trustAs
     *
     * @description
     * Delegates to {@link ng.$sceDelegate#trustAs `$sceDelegate.trustAs`}.  As such,
     * returns an object that is trusted by angular for use in specified strict contextual
     * escaping contexts (such as ng-bind-html, ng-include, any src attribute
     * interpolation, any dom event binding attribute interpolation such as for onclick,  etc.)
     * that uses the provided value.  See * {@link ng.$sce $sce} for enabling strict contextual
     * escaping.
     *
     * @param {string} type The kind of context in which this value is safe for use.  e.g. url,
     *   resource_url, html, js and css.
     * @param {*} value The value that that should be considered trusted/safe.
     * @returns {*} A value that can be used to stand in for the provided `value` in places
     * where Angular expects a $sce.trustAs() return value.
     */
        /**
     * @ngdoc method
     * @name $sce#trustAsHtml
     *
     * @description
     * Shorthand method.  `$sce.trustAsHtml(value)` →
     *     {@link ng.$sceDelegate#trustAs `$sceDelegate.trustAs($sce.HTML, value)`}
     *
     * @param {*} value The value to trustAs.
     * @returns {*} An object that can be passed to {@link ng.$sce#getTrustedHtml
     *     $sce.getTrustedHtml(value)} to obtain the original value.  (privileged directives
     *     only accept expressions that are either literal constants or are the
     *     return value of {@link ng.$sce#trustAs $sce.trustAs}.)
     */
        /**
     * @ngdoc method
     * @name $sce#trustAsUrl
     *
     * @description
     * Shorthand method.  `$sce.trustAsUrl(value)` →
     *     {@link ng.$sceDelegate#trustAs `$sceDelegate.trustAs($sce.URL, value)`}
     *
     * @param {*} value The value to trustAs.
     * @returns {*} An object that can be passed to {@link ng.$sce#getTrustedUrl
     *     $sce.getTrustedUrl(value)} to obtain the original value.  (privileged directives
     *     only accept expressions that are either literal constants or are the
     *     return value of {@link ng.$sce#trustAs $sce.trustAs}.)
     */
        /**
     * @ngdoc method
     * @name $sce#trustAsResourceUrl
     *
     * @description
     * Shorthand method.  `$sce.trustAsResourceUrl(value)` →
     *     {@link ng.$sceDelegate#trustAs `$sceDelegate.trustAs($sce.RESOURCE_URL, value)`}
     *
     * @param {*} value The value to trustAs.
     * @returns {*} An object that can be passed to {@link ng.$sce#getTrustedResourceUrl
     *     $sce.getTrustedResourceUrl(value)} to obtain the original value.  (privileged directives
     *     only accept expressions that are either literal constants or are the return
     *     value of {@link ng.$sce#trustAs $sce.trustAs}.)
     */
        /**
     * @ngdoc method
     * @name $sce#trustAsJs
     *
     * @description
     * Shorthand method.  `$sce.trustAsJs(value)` →
     *     {@link ng.$sceDelegate#trustAs `$sceDelegate.trustAs($sce.JS, value)`}
     *
     * @param {*} value The value to trustAs.
     * @returns {*} An object that can be passed to {@link ng.$sce#getTrustedJs
     *     $sce.getTrustedJs(value)} to obtain the original value.  (privileged directives
     *     only accept expressions that are either literal constants or are the
     *     return value of {@link ng.$sce#trustAs $sce.trustAs}.)
     */
        /**
     * @ngdoc method
     * @name $sce#getTrusted
     *
     * @description
     * Delegates to {@link ng.$sceDelegate#getTrusted `$sceDelegate.getTrusted`}.  As such,
     * takes the result of a {@link ng.$sce#trustAs `$sce.trustAs`}() call and returns the
     * originally supplied value if the queried context type is a supertype of the created type.
     * If this condition isn't satisfied, throws an exception.
     *
     * @param {string} type The kind of context in which this value is to be used.
     * @param {*} maybeTrusted The result of a prior {@link ng.$sce#trustAs `$sce.trustAs`}
     *                         call.
     * @returns {*} The value the was originally provided to
     *              {@link ng.$sce#trustAs `$sce.trustAs`} if valid in this context.
     *              Otherwise, throws an exception.
     */
        /**
     * @ngdoc method
     * @name $sce#getTrustedHtml
     *
     * @description
     * Shorthand method.  `$sce.getTrustedHtml(value)` →
     *     {@link ng.$sceDelegate#getTrusted `$sceDelegate.getTrusted($sce.HTML, value)`}
     *
     * @param {*} value The value to pass to `$sce.getTrusted`.
     * @returns {*} The return value of `$sce.getTrusted($sce.HTML, value)`
     */
        /**
     * @ngdoc method
     * @name $sce#getTrustedCss
     *
     * @description
     * Shorthand method.  `$sce.getTrustedCss(value)` →
     *     {@link ng.$sceDelegate#getTrusted `$sceDelegate.getTrusted($sce.CSS, value)`}
     *
     * @param {*} value The value to pass to `$sce.getTrusted`.
     * @returns {*} The return value of `$sce.getTrusted($sce.CSS, value)`
     */
        /**
     * @ngdoc method
     * @name $sce#getTrustedUrl
     *
     * @description
     * Shorthand method.  `$sce.getTrustedUrl(value)` →
     *     {@link ng.$sceDelegate#getTrusted `$sceDelegate.getTrusted($sce.URL, value)`}
     *
     * @param {*} value The value to pass to `$sce.getTrusted`.
     * @returns {*} The return value of `$sce.getTrusted($sce.URL, value)`
     */
        /**
     * @ngdoc method
     * @name $sce#getTrustedResourceUrl
     *
     * @description
     * Shorthand method.  `$sce.getTrustedResourceUrl(value)` →
     *     {@link ng.$sceDelegate#getTrusted `$sceDelegate.getTrusted($sce.RESOURCE_URL, value)`}
     *
     * @param {*} value The value to pass to `$sceDelegate.getTrusted`.
     * @returns {*} The return value of `$sce.getTrusted($sce.RESOURCE_URL, value)`
     */
        /**
     * @ngdoc method
     * @name $sce#getTrustedJs
     *
     * @description
     * Shorthand method.  `$sce.getTrustedJs(value)` →
     *     {@link ng.$sceDelegate#getTrusted `$sceDelegate.getTrusted($sce.JS, value)`}
     *
     * @param {*} value The value to pass to `$sce.getTrusted`.
     * @returns {*} The return value of `$sce.getTrusted($sce.JS, value)`
     */
        /**
     * @ngdoc method
     * @name $sce#parseAsHtml
     *
     * @description
     * Shorthand method.  `$sce.parseAsHtml(expression string)` →
     *     {@link ng.$sce#parseAs `$sce.parseAs($sce.HTML, value)`}
     *
     * @param {string} expression String expression to compile.
     * @returns {function(context, locals)} a function which represents the compiled expression:
     *
     *    * `context` – `{object}` – an object against which any expressions embedded in the strings
     *      are evaluated against (typically a scope object).
     *    * `locals` – `{object=}` – local variables context object, useful for overriding values in
     *      `context`.
     */
        /**
     * @ngdoc method
     * @name $sce#parseAsCss
     *
     * @description
     * Shorthand method.  `$sce.parseAsCss(value)` →
     *     {@link ng.$sce#parseAs `$sce.parseAs($sce.CSS, value)`}
     *
     * @param {string} expression String expression to compile.
     * @returns {function(context, locals)} a function which represents the compiled expression:
     *
     *    * `context` – `{object}` – an object against which any expressions embedded in the strings
     *      are evaluated against (typically a scope object).
     *    * `locals` – `{object=}` – local variables context object, useful for overriding values in
     *      `context`.
     */
        /**
     * @ngdoc method
     * @name $sce#parseAsUrl
     *
     * @description
     * Shorthand method.  `$sce.parseAsUrl(value)` →
     *     {@link ng.$sce#parseAs `$sce.parseAs($sce.URL, value)`}
     *
     * @param {string} expression String expression to compile.
     * @returns {function(context, locals)} a function which represents the compiled expression:
     *
     *    * `context` – `{object}` – an object against which any expressions embedded in the strings
     *      are evaluated against (typically a scope object).
     *    * `locals` – `{object=}` – local variables context object, useful for overriding values in
     *      `context`.
     */
        /**
     * @ngdoc method
     * @name $sce#parseAsResourceUrl
     *
     * @description
     * Shorthand method.  `$sce.parseAsResourceUrl(value)` →
     *     {@link ng.$sce#parseAs `$sce.parseAs($sce.RESOURCE_URL, value)`}
     *
     * @param {string} expression String expression to compile.
     * @returns {function(context, locals)} a function which represents the compiled expression:
     *
     *    * `context` – `{object}` – an object against which any expressions embedded in the strings
     *      are evaluated against (typically a scope object).
     *    * `locals` – `{object=}` – local variables context object, useful for overriding values in
     *      `context`.
     */
        /**
     * @ngdoc method
     * @name $sce#parseAsJs
     *
     * @description
     * Shorthand method.  `$sce.parseAsJs(value)` →
     *     {@link ng.$sce#parseAs `$sce.parseAs($sce.JS, value)`}
     *
     * @param {string} expression String expression to compile.
     * @returns {function(context, locals)} a function which represents the compiled expression:
     *
     *    * `context` – `{object}` – an object against which any expressions embedded in the strings
     *      are evaluated against (typically a scope object).
     *    * `locals` – `{object=}` – local variables context object, useful for overriding values in
     *      `context`.
     */
        // Shorthand delegations.
        var parse = sce.parseAs, getTrusted = sce.getTrusted, trustAs = sce.trustAs;
        forEach(SCE_CONTEXTS, function (enumValue, name) {
          var lName = lowercase(name);
          sce[camelCase('parse_as_' + lName)] = function (expr) {
            return parse(enumValue, expr);
          };
          sce[camelCase('get_trusted_' + lName)] = function (value) {
            return getTrusted(enumValue, value);
          };
          sce[camelCase('trust_as_' + lName)] = function (value) {
            return trustAs(enumValue, value);
          };
        });
        return sce;
      }
    ];
  }
  /**
 * !!! This is an undocumented "private" service !!!
 *
 * @name $sniffer
 * @requires $window
 * @requires $document
 *
 * @property {boolean} history Does the browser support html5 history api ?
 * @property {boolean} transitions Does the browser support CSS transition events ?
 * @property {boolean} animations Does the browser support CSS animation events ?
 *
 * @description
 * This is very simple implementation of testing browser's features.
 */
  function $SnifferProvider() {
    this.$get = [
      '$window',
      '$document',
      function ($window, $document) {
        var eventSupport = {}, android = int((/android (\d+)/.exec(lowercase(($window.navigator || {}).userAgent)) || [])[1]), boxee = /Boxee/i.test(($window.navigator || {}).userAgent), document = $document[0] || {}, vendorPrefix, vendorRegex = /^(Moz|webkit|ms)(?=[A-Z])/, bodyStyle = document.body && document.body.style, transitions = false, animations = false, match;
        if (bodyStyle) {
          for (var prop in bodyStyle) {
            if (match = vendorRegex.exec(prop)) {
              vendorPrefix = match[0];
              vendorPrefix = vendorPrefix.substr(0, 1).toUpperCase() + vendorPrefix.substr(1);
              break;
            }
          }
          if (!vendorPrefix) {
            vendorPrefix = 'WebkitOpacity' in bodyStyle && 'webkit';
          }
          transitions = !!('transition' in bodyStyle || vendorPrefix + 'Transition' in bodyStyle);
          animations = !!('animation' in bodyStyle || vendorPrefix + 'Animation' in bodyStyle);
          if (android && (!transitions || !animations)) {
            transitions = isString(document.body.style.webkitTransition);
            animations = isString(document.body.style.webkitAnimation);
          }
        }
        return {
          history: !!($window.history && $window.history.pushState && !(android < 4) && !boxee),
          hasEvent: function (event) {
            // IE9 implements 'input' event it's so fubared that we rather pretend that it doesn't have
            // it. In particular the event is not fired when backspace or delete key are pressed or
            // when cut operation is performed.
            // IE10+ implements 'input' event but it erroneously fires under various situations,
            // e.g. when placeholder changes, or a form is focused.
            if (event === 'input' && msie <= 11)
              return false;
            if (isUndefined(eventSupport[event])) {
              var divElm = document.createElement('div');
              eventSupport[event] = 'on' + event in divElm;
            }
            return eventSupport[event];
          },
          csp: csp(),
          vendorPrefix: vendorPrefix,
          transitions: transitions,
          animations: animations,
          android: android
        };
      }
    ];
  }
  var $compileMinErr = minErr('$compile');
  /**
 * @ngdoc service
 * @name $templateRequest
 *
 * @description
 * The `$templateRequest` service runs security checks then downloads the provided template using
 * `$http` and, upon success, stores the contents inside of `$templateCache`. If the HTTP request
 * fails or the response data of the HTTP request is empty, a `$compile` error will be thrown (the
 * exception can be thwarted by setting the 2nd parameter of the function to true). Note that the
 * contents of `$templateCache` are trusted, so the call to `$sce.getTrustedUrl(tpl)` is omitted
 * when `tpl` is of type string and `$templateCache` has the matching entry.
 *
 * @param {string|TrustedResourceUrl} tpl The HTTP request template URL
 * @param {boolean=} ignoreRequestError Whether or not to ignore the exception when the request fails or the template is empty
 *
 * @return {Promise} the HTTP Promise for the given.
 *
 * @property {number} totalPendingRequests total amount of pending template requests being downloaded.
 */
  function $TemplateRequestProvider() {
    this.$get = [
      '$templateCache',
      '$http',
      '$q',
      '$sce',
      function ($templateCache, $http, $q, $sce) {
        function handleRequestFn(tpl, ignoreRequestError) {
          handleRequestFn.totalPendingRequests++;
          // We consider the template cache holds only trusted templates, so
          // there's no need to go through whitelisting again for keys that already
          // are included in there. This also makes Angular accept any script
          // directive, no matter its name. However, we still need to unwrap trusted
          // types.
          if (!isString(tpl) || !$templateCache.get(tpl)) {
            tpl = $sce.getTrustedResourceUrl(tpl);
          }
          var transformResponse = $http.defaults && $http.defaults.transformResponse;
          if (isArray(transformResponse)) {
            transformResponse = transformResponse.filter(function (transformer) {
              return transformer !== defaultHttpResponseTransform;
            });
          } else if (transformResponse === defaultHttpResponseTransform) {
            transformResponse = null;
          }
          var httpOptions = {
              cache: $templateCache,
              transformResponse: transformResponse
            };
          return $http.get(tpl, httpOptions)['finally'](function () {
            handleRequestFn.totalPendingRequests--;
          }).then(function (response) {
            return response.data;
          }, handleError);
          function handleError(resp) {
            if (!ignoreRequestError) {
              throw $compileMinErr('tpload', 'Failed to load template: {0}', tpl);
            }
            return $q.reject(resp);
          }
        }
        handleRequestFn.totalPendingRequests = 0;
        return handleRequestFn;
      }
    ];
  }
  function $$TestabilityProvider() {
    this.$get = [
      '$rootScope',
      '$browser',
      '$location',
      function ($rootScope, $browser, $location) {
        /**
     * @name $testability
     *
     * @description
     * The private $$testability service provides a collection of methods for use when debugging
     * or by automated test and debugging tools.
     */
        var testability = {};
        /**
     * @name $$testability#findBindings
     *
     * @description
     * Returns an array of elements that are bound (via ng-bind or {{}})
     * to expressions matching the input.
     *
     * @param {Element} element The element root to search from.
     * @param {string} expression The binding expression to match.
     * @param {boolean} opt_exactMatch If true, only returns exact matches
     *     for the expression. Filters and whitespace are ignored.
     */
        testability.findBindings = function (element, expression, opt_exactMatch) {
          var bindings = element.getElementsByClassName('ng-binding');
          var matches = [];
          forEach(bindings, function (binding) {
            var dataBinding = angular.element(binding).data('$binding');
            if (dataBinding) {
              forEach(dataBinding, function (bindingName) {
                if (opt_exactMatch) {
                  var matcher = new RegExp('(^|\\s)' + escapeForRegexp(expression) + '(\\s|\\||$)');
                  if (matcher.test(bindingName)) {
                    matches.push(binding);
                  }
                } else {
                  if (bindingName.indexOf(expression) != -1) {
                    matches.push(binding);
                  }
                }
              });
            }
          });
          return matches;
        };
        /**
     * @name $$testability#findModels
     *
     * @description
     * Returns an array of elements that are two-way found via ng-model to
     * expressions matching the input.
     *
     * @param {Element} element The element root to search from.
     * @param {string} expression The model expression to match.
     * @param {boolean} opt_exactMatch If true, only returns exact matches
     *     for the expression.
     */
        testability.findModels = function (element, expression, opt_exactMatch) {
          var prefixes = [
              'ng-',
              'data-ng-',
              'ng\\:'
            ];
          for (var p = 0; p < prefixes.length; ++p) {
            var attributeEquals = opt_exactMatch ? '=' : '*=';
            var selector = '[' + prefixes[p] + 'model' + attributeEquals + '"' + expression + '"]';
            var elements = element.querySelectorAll(selector);
            if (elements.length) {
              return elements;
            }
          }
        };
        /**
     * @name $$testability#getLocation
     *
     * @description
     * Shortcut for getting the location in a browser agnostic way. Returns
     *     the path, search, and hash. (e.g. /path?a=b#hash)
     */
        testability.getLocation = function () {
          return $location.url();
        };
        /**
     * @name $$testability#setLocation
     *
     * @description
     * Shortcut for navigating to a location without doing a full page reload.
     *
     * @param {string} url The location url (path, search and hash,
     *     e.g. /path?a=b#hash) to go to.
     */
        testability.setLocation = function (url) {
          if (url !== $location.url()) {
            $location.url(url);
            $rootScope.$digest();
          }
        };
        /**
     * @name $$testability#whenStable
     *
     * @description
     * Calls the callback when $timeout and $http requests are completed.
     *
     * @param {function} callback
     */
        testability.whenStable = function (callback) {
          $browser.notifyWhenNoOutstandingRequests(callback);
        };
        return testability;
      }
    ];
  }
  function $TimeoutProvider() {
    this.$get = [
      '$rootScope',
      '$browser',
      '$q',
      '$$q',
      '$exceptionHandler',
      function ($rootScope, $browser, $q, $$q, $exceptionHandler) {
        var deferreds = {};
        /**
      * @ngdoc service
      * @name $timeout
      *
      * @description
      * Angular's wrapper for `window.setTimeout`. The `fn` function is wrapped into a try/catch
      * block and delegates any exceptions to
      * {@link ng.$exceptionHandler $exceptionHandler} service.
      *
      * The return value of registering a timeout function is a promise, which will be resolved when
      * the timeout is reached and the timeout function is executed.
      *
      * To cancel a timeout request, call `$timeout.cancel(promise)`.
      *
      * In tests you can use {@link ngMock.$timeout `$timeout.flush()`} to
      * synchronously flush the queue of deferred functions.
      *
      * @param {function()} fn A function, whose execution should be delayed.
      * @param {number=} [delay=0] Delay in milliseconds.
      * @param {boolean=} [invokeApply=true] If set to `false` skips model dirty checking, otherwise
      *   will invoke `fn` within the {@link ng.$rootScope.Scope#$apply $apply} block.
      * @returns {Promise} Promise that will be resolved when the timeout is reached. The value this
      *   promise will be resolved with is the return value of the `fn` function.
      *
      */
        function timeout(fn, delay, invokeApply) {
          var skipApply = isDefined(invokeApply) && !invokeApply, deferred = (skipApply ? $$q : $q).defer(), promise = deferred.promise, timeoutId;
          timeoutId = $browser.defer(function () {
            try {
              deferred.resolve(fn());
            } catch (e) {
              deferred.reject(e);
              $exceptionHandler(e);
            } finally {
              delete deferreds[promise.$$timeoutId];
            }
            if (!skipApply)
              $rootScope.$apply();
          }, delay);
          promise.$$timeoutId = timeoutId;
          deferreds[timeoutId] = deferred;
          return promise;
        }
        /**
      * @ngdoc method
      * @name $timeout#cancel
      *
      * @description
      * Cancels a task associated with the `promise`. As a result of this, the promise will be
      * resolved with a rejection.
      *
      * @param {Promise=} promise Promise returned by the `$timeout` function.
      * @returns {boolean} Returns `true` if the task hasn't executed yet and was successfully
      *   canceled.
      */
        timeout.cancel = function (promise) {
          if (promise && promise.$$timeoutId in deferreds) {
            deferreds[promise.$$timeoutId].reject('canceled');
            delete deferreds[promise.$$timeoutId];
            return $browser.defer.cancel(promise.$$timeoutId);
          }
          return false;
        };
        return timeout;
      }
    ];
  }
  // NOTE:  The usage of window and document instead of $window and $document here is
  // deliberate.  This service depends on the specific behavior of anchor nodes created by the
  // browser (resolving and parsing URLs) that is unlikely to be provided by mock objects and
  // cause us to break tests.  In addition, when the browser resolves a URL for XHR, it
  // doesn't know about mocked locations and resolves URLs to the real document - which is
  // exactly the behavior needed here.  There is little value is mocking these out for this
  // service.
  var urlParsingNode = document.createElement('a');
  var originUrl = urlResolve(window.location.href);
  /**
 *
 * Implementation Notes for non-IE browsers
 * ----------------------------------------
 * Assigning a URL to the href property of an anchor DOM node, even one attached to the DOM,
 * results both in the normalizing and parsing of the URL.  Normalizing means that a relative
 * URL will be resolved into an absolute URL in the context of the application document.
 * Parsing means that the anchor node's host, hostname, protocol, port, pathname and related
 * properties are all populated to reflect the normalized URL.  This approach has wide
 * compatibility - Safari 1+, Mozilla 1+, Opera 7+,e etc.  See
 * http://www.aptana.com/reference/html/api/HTMLAnchorElement.html
 *
 * Implementation Notes for IE
 * ---------------------------
 * IE <= 10 normalizes the URL when assigned to the anchor node similar to the other
 * browsers.  However, the parsed components will not be set if the URL assigned did not specify
 * them.  (e.g. if you assign a.href = "foo", then a.protocol, a.host, etc. will be empty.)  We
 * work around that by performing the parsing in a 2nd step by taking a previously normalized
 * URL (e.g. by assigning to a.href) and assigning it a.href again.  This correctly populates the
 * properties such as protocol, hostname, port, etc.
 *
 * References:
 *   http://developer.mozilla.org/en-US/docs/Web/API/HTMLAnchorElement
 *   http://www.aptana.com/reference/html/api/HTMLAnchorElement.html
 *   http://url.spec.whatwg.org/#urlutils
 *   https://github.com/angular/angular.js/pull/2902
 *   http://james.padolsey.com/javascript/parsing-urls-with-the-dom/
 *
 * @kind function
 * @param {string} url The URL to be parsed.
 * @description Normalizes and parses a URL.
 * @returns {object} Returns the normalized URL as a dictionary.
 *
 *   | member name   | Description    |
 *   |---------------|----------------|
 *   | href          | A normalized version of the provided URL if it was not an absolute URL |
 *   | protocol      | The protocol including the trailing colon                              |
 *   | host          | The host and port (if the port is non-default) of the normalizedUrl    |
 *   | search        | The search params, minus the question mark                             |
 *   | hash          | The hash string, minus the hash symbol
 *   | hostname      | The hostname
 *   | port          | The port, without ":"
 *   | pathname      | The pathname, beginning with "/"
 *
 */
  function urlResolve(url) {
    var href = url;
    if (msie) {
      // Normalize before parse.  Refer Implementation Notes on why this is
      // done in two steps on IE.
      urlParsingNode.setAttribute('href', href);
      href = urlParsingNode.href;
    }
    urlParsingNode.setAttribute('href', href);
    // urlParsingNode provides the UrlUtils interface - http://url.spec.whatwg.org/#urlutils
    return {
      href: urlParsingNode.href,
      protocol: urlParsingNode.protocol ? urlParsingNode.protocol.replace(/:$/, '') : '',
      host: urlParsingNode.host,
      search: urlParsingNode.search ? urlParsingNode.search.replace(/^\?/, '') : '',
      hash: urlParsingNode.hash ? urlParsingNode.hash.replace(/^#/, '') : '',
      hostname: urlParsingNode.hostname,
      port: urlParsingNode.port,
      pathname: urlParsingNode.pathname.charAt(0) === '/' ? urlParsingNode.pathname : '/' + urlParsingNode.pathname
    };
  }
  /**
 * Parse a request URL and determine whether this is a same-origin request as the application document.
 *
 * @param {string|object} requestUrl The url of the request as a string that will be resolved
 * or a parsed URL object.
 * @returns {boolean} Whether the request is for the same origin as the application document.
 */
  function urlIsSameOrigin(requestUrl) {
    var parsed = isString(requestUrl) ? urlResolve(requestUrl) : requestUrl;
    return parsed.protocol === originUrl.protocol && parsed.host === originUrl.host;
  }
  /**
 * @ngdoc service
 * @name $window
 *
 * @description
 * A reference to the browser's `window` object. While `window`
 * is globally available in JavaScript, it causes testability problems, because
 * it is a global variable. In angular we always refer to it through the
 * `$window` service, so it may be overridden, removed or mocked for testing.
 *
 * Expressions, like the one defined for the `ngClick` directive in the example
 * below, are evaluated with respect to the current scope.  Therefore, there is
 * no risk of inadvertently coding in a dependency on a global value in such an
 * expression.
 *
 * @example
   <example module="windowExample">
     <file name="index.html">
       <script>
         angular.module('windowExample', [])
           .controller('ExampleController', ['$scope', '$window', function($scope, $window) {
             $scope.greeting = 'Hello, World!';
             $scope.doGreeting = function(greeting) {
               $window.alert(greeting);
             };
           }]);
       </script>
       <div ng-controller="ExampleController">
         <input type="text" ng-model="greeting" />
         <button ng-click="doGreeting(greeting)">ALERT</button>
       </div>
     </file>
     <file name="protractor.js" type="protractor">
      it('should display the greeting in the input box', function() {
       element(by.model('greeting')).sendKeys('Hello, E2E Tests');
       // If we click the button it will block the test runner
       // element(':button').click();
      });
     </file>
   </example>
 */
  function $WindowProvider() {
    this.$get = valueFn(window);
  }
  /* global currencyFilter: true,
 dateFilter: true,
 filterFilter: true,
 jsonFilter: true,
 limitToFilter: true,
 lowercaseFilter: true,
 numberFilter: true,
 orderByFilter: true,
 uppercaseFilter: true,
 */
  /**
 * @ngdoc provider
 * @name $filterProvider
 * @description
 *
 * Filters are just functions which transform input to an output. However filters need to be
 * Dependency Injected. To achieve this a filter definition consists of a factory function which is
 * annotated with dependencies and is responsible for creating a filter function.
 *
 * <div class="alert alert-warning">
 * **Note:** Filter names must be valid angular {@link expression} identifiers, such as `uppercase` or `orderBy`.
 * Names with special characters, such as hyphens and dots, are not allowed. If you wish to namespace
 * your filters, then you can use capitalization (`myappSubsectionFilterx`) or underscores
 * (`myapp_subsection_filterx`).
 * </div>
 *
 * ```js
 *   // Filter registration
 *   function MyModule($provide, $filterProvider) {
 *     // create a service to demonstrate injection (not always needed)
 *     $provide.value('greet', function(name){
 *       return 'Hello ' + name + '!';
 *     });
 *
 *     // register a filter factory which uses the
 *     // greet service to demonstrate DI.
 *     $filterProvider.register('greet', function(greet){
 *       // return the filter function which uses the greet service
 *       // to generate salutation
 *       return function(text) {
 *         // filters need to be forgiving so check input validity
 *         return text && greet(text) || text;
 *       };
 *     });
 *   }
 * ```
 *
 * The filter function is registered with the `$injector` under the filter name suffix with
 * `Filter`.
 *
 * ```js
 *   it('should be the same instance', inject(
 *     function($filterProvider) {
 *       $filterProvider.register('reverse', function(){
 *         return ...;
 *       });
 *     },
 *     function($filter, reverseFilter) {
 *       expect($filter('reverse')).toBe(reverseFilter);
 *     });
 * ```
 *
 *
 * For more information about how angular filters work, and how to create your own filters, see
 * {@link guide/filter Filters} in the Angular Developer Guide.
 */
  /**
 * @ngdoc service
 * @name $filter
 * @kind function
 * @description
 * Filters are used for formatting data displayed to the user.
 *
 * The general syntax in templates is as follows:
 *
 *         {{ expression [| filter_name[:parameter_value] ... ] }}
 *
 * @param {String} name Name of the filter function to retrieve
 * @return {Function} the filter function
 * @example
   <example name="$filter" module="filterExample">
     <file name="index.html">
       <div ng-controller="MainCtrl">
        <h3>{{ originalText }}</h3>
        <h3>{{ filteredText }}</h3>
       </div>
     </file>

     <file name="script.js">
      angular.module('filterExample', [])
      .controller('MainCtrl', function($scope, $filter) {
        $scope.originalText = 'hello';
        $scope.filteredText = $filter('uppercase')($scope.originalText);
      });
     </file>
   </example>
  */
  $FilterProvider.$inject = ['$provide'];
  function $FilterProvider($provide) {
    var suffix = 'Filter';
    /**
   * @ngdoc method
   * @name $filterProvider#register
   * @param {string|Object} name Name of the filter function, or an object map of filters where
   *    the keys are the filter names and the values are the filter factories.
   *
   *    <div class="alert alert-warning">
   *    **Note:** Filter names must be valid angular {@link expression} identifiers, such as `uppercase` or `orderBy`.
   *    Names with special characters, such as hyphens and dots, are not allowed. If you wish to namespace
   *    your filters, then you can use capitalization (`myappSubsectionFilterx`) or underscores
   *    (`myapp_subsection_filterx`).
   *    </div>
   * @returns {Object} Registered filter instance, or if a map of filters was provided then a map
   *    of the registered filter instances.
   */
    function register(name, factory) {
      if (isObject(name)) {
        var filters = {};
        forEach(name, function (filter, key) {
          filters[key] = register(key, filter);
        });
        return filters;
      } else {
        return $provide.factory(name + suffix, factory);
      }
    }
    this.register = register;
    this.$get = [
      '$injector',
      function ($injector) {
        return function (name) {
          return $injector.get(name + suffix);
        };
      }
    ];
    ////////////////////////////////////////
    /* global
    currencyFilter: false,
    dateFilter: false,
    filterFilter: false,
    jsonFilter: false,
    limitToFilter: false,
    lowercaseFilter: false,
    numberFilter: false,
    orderByFilter: false,
    uppercaseFilter: false,
  */
    register('currency', currencyFilter);
    register('date', dateFilter);
    register('filter', filterFilter);
    register('json', jsonFilter);
    register('limitTo', limitToFilter);
    register('lowercase', lowercaseFilter);
    register('number', numberFilter);
    register('orderBy', orderByFilter);
    register('uppercase', uppercaseFilter);
  }
  /**
 * @ngdoc filter
 * @name filter
 * @kind function
 *
 * @description
 * Selects a subset of items from `array` and returns it as a new array.
 *
 * @param {Array} array The source array.
 * @param {string|Object|function()} expression The predicate to be used for selecting items from
 *   `array`.
 *
 *   Can be one of:
 *
 *   - `string`: The string is used for matching against the contents of the `array`. All strings or
 *     objects with string properties in `array` that match this string will be returned. This also
 *     applies to nested object properties.
 *     The predicate can be negated by prefixing the string with `!`.
 *
 *   - `Object`: A pattern object can be used to filter specific properties on objects contained
 *     by `array`. For example `{name:"M", phone:"1"}` predicate will return an array of items
 *     which have property `name` containing "M" and property `phone` containing "1". A special
 *     property name `$` can be used (as in `{$:"text"}`) to accept a match against any
 *     property of the object or its nested object properties. That's equivalent to the simple
 *     substring match with a `string` as described above. The predicate can be negated by prefixing
 *     the string with `!`.
 *     For example `{name: "!M"}` predicate will return an array of items which have property `name`
 *     not containing "M".
 *
 *     Note that a named property will match properties on the same level only, while the special
 *     `$` property will match properties on the same level or deeper. E.g. an array item like
 *     `{name: {first: 'John', last: 'Doe'}}` will **not** be matched by `{name: 'John'}`, but
 *     **will** be matched by `{$: 'John'}`.
 *
 *   - `function(value, index)`: A predicate function can be used to write arbitrary filters. The
 *     function is called for each element of `array`. The final result is an array of those
 *     elements that the predicate returned true for.
 *
 * @param {function(actual, expected)|true|undefined} comparator Comparator which is used in
 *     determining if the expected value (from the filter expression) and actual value (from
 *     the object in the array) should be considered a match.
 *
 *   Can be one of:
 *
 *   - `function(actual, expected)`:
 *     The function will be given the object value and the predicate value to compare and
 *     should return true if both values should be considered equal.
 *
 *   - `true`: A shorthand for `function(actual, expected) { return angular.equals(actual, expected)}`.
 *     This is essentially strict comparison of expected and actual.
 *
 *   - `false|undefined`: A short hand for a function which will look for a substring match in case
 *     insensitive way.
 *
 * @example
   <example>
     <file name="index.html">
       <div ng-init="friends = [{name:'John', phone:'555-1276'},
                                {name:'Mary', phone:'800-BIG-MARY'},
                                {name:'Mike', phone:'555-4321'},
                                {name:'Adam', phone:'555-5678'},
                                {name:'Julie', phone:'555-8765'},
                                {name:'Juliette', phone:'555-5678'}]"></div>

       Search: <input ng-model="searchText">
       <table id="searchTextResults">
         <tr><th>Name</th><th>Phone</th></tr>
         <tr ng-repeat="friend in friends | filter:searchText">
           <td>{{friend.name}}</td>
           <td>{{friend.phone}}</td>
         </tr>
       </table>
       <hr>
       Any: <input ng-model="search.$"> <br>
       Name only <input ng-model="search.name"><br>
       Phone only <input ng-model="search.phone"><br>
       Equality <input type="checkbox" ng-model="strict"><br>
       <table id="searchObjResults">
         <tr><th>Name</th><th>Phone</th></tr>
         <tr ng-repeat="friendObj in friends | filter:search:strict">
           <td>{{friendObj.name}}</td>
           <td>{{friendObj.phone}}</td>
         </tr>
       </table>
     </file>
     <file name="protractor.js" type="protractor">
       var expectFriendNames = function(expectedNames, key) {
         element.all(by.repeater(key + ' in friends').column(key + '.name')).then(function(arr) {
           arr.forEach(function(wd, i) {
             expect(wd.getText()).toMatch(expectedNames[i]);
           });
         });
       };

       it('should search across all fields when filtering with a string', function() {
         var searchText = element(by.model('searchText'));
         searchText.clear();
         searchText.sendKeys('m');
         expectFriendNames(['Mary', 'Mike', 'Adam'], 'friend');

         searchText.clear();
         searchText.sendKeys('76');
         expectFriendNames(['John', 'Julie'], 'friend');
       });

       it('should search in specific fields when filtering with a predicate object', function() {
         var searchAny = element(by.model('search.$'));
         searchAny.clear();
         searchAny.sendKeys('i');
         expectFriendNames(['Mary', 'Mike', 'Julie', 'Juliette'], 'friendObj');
       });
       it('should use a equal comparison when comparator is true', function() {
         var searchName = element(by.model('search.name'));
         var strict = element(by.model('strict'));
         searchName.clear();
         searchName.sendKeys('Julie');
         strict.click();
         expectFriendNames(['Julie'], 'friendObj');
       });
     </file>
   </example>
 */
  function filterFilter() {
    return function (array, expression, comparator) {
      if (!isArray(array))
        return array;
      var expressionType = expression !== null ? typeof expression : 'null';
      var predicateFn;
      var matchAgainstAnyProp;
      switch (expressionType) {
      case 'function':
        predicateFn = expression;
        break;
      case 'boolean':
      case 'null':
      case 'number':
      case 'string':
        matchAgainstAnyProp = true;
      //jshint -W086
      case 'object':
        //jshint +W086
        predicateFn = createPredicateFn(expression, comparator, matchAgainstAnyProp);
        break;
      default:
        return array;
      }
      return array.filter(predicateFn);
    };
  }
  // Helper functions for `filterFilter`
  function createPredicateFn(expression, comparator, matchAgainstAnyProp) {
    var shouldMatchPrimitives = isObject(expression) && '$' in expression;
    var predicateFn;
    if (comparator === true) {
      comparator = equals;
    } else if (!isFunction(comparator)) {
      comparator = function (actual, expected) {
        if (isUndefined(actual)) {
          // No substring matching against `undefined`
          return false;
        }
        if (actual === null || expected === null) {
          // No substring matching against `null`; only match against `null`
          return actual === expected;
        }
        if (isObject(actual) || isObject(expected)) {
          // Prevent an object to be considered equal to a string like `'[object'`
          return false;
        }
        actual = lowercase('' + actual);
        expected = lowercase('' + expected);
        return actual.indexOf(expected) !== -1;
      };
    }
    predicateFn = function (item) {
      if (shouldMatchPrimitives && !isObject(item)) {
        return deepCompare(item, expression.$, comparator, false);
      }
      return deepCompare(item, expression, comparator, matchAgainstAnyProp);
    };
    return predicateFn;
  }
  function deepCompare(actual, expected, comparator, matchAgainstAnyProp, dontMatchWholeObject) {
    var actualType = actual !== null ? typeof actual : 'null';
    var expectedType = expected !== null ? typeof expected : 'null';
    if (expectedType === 'string' && expected.charAt(0) === '!') {
      return !deepCompare(actual, expected.substring(1), comparator, matchAgainstAnyProp);
    } else if (isArray(actual)) {
      // In case `actual` is an array, consider it a match
      // if ANY of it's items matches `expected`
      return actual.some(function (item) {
        return deepCompare(item, expected, comparator, matchAgainstAnyProp);
      });
    }
    switch (actualType) {
    case 'object':
      var key;
      if (matchAgainstAnyProp) {
        for (key in actual) {
          if (key.charAt(0) !== '$' && deepCompare(actual[key], expected, comparator, true)) {
            return true;
          }
        }
        return dontMatchWholeObject ? false : deepCompare(actual, expected, comparator, false);
      } else if (expectedType === 'object') {
        for (key in expected) {
          var expectedVal = expected[key];
          if (isFunction(expectedVal) || isUndefined(expectedVal)) {
            continue;
          }
          var matchAnyProperty = key === '$';
          var actualVal = matchAnyProperty ? actual : actual[key];
          if (!deepCompare(actualVal, expectedVal, comparator, matchAnyProperty, matchAnyProperty)) {
            return false;
          }
        }
        return true;
      } else {
        return comparator(actual, expected);
      }
      break;
    case 'function':
      return false;
    default:
      return comparator(actual, expected);
    }
  }
  /**
 * @ngdoc filter
 * @name currency
 * @kind function
 *
 * @description
 * Formats a number as a currency (ie $1,234.56). When no currency symbol is provided, default
 * symbol for current locale is used.
 *
 * @param {number} amount Input to filter.
 * @param {string=} symbol Currency symbol or identifier to be displayed.
 * @param {number=} fractionSize Number of decimal places to round the amount to, defaults to default max fraction size for current locale
 * @returns {string} Formatted number.
 *
 *
 * @example
   <example module="currencyExample">
     <file name="index.html">
       <script>
         angular.module('currencyExample', [])
           .controller('ExampleController', ['$scope', function($scope) {
             $scope.amount = 1234.56;
           }]);
       </script>
       <div ng-controller="ExampleController">
         <input type="number" ng-model="amount"> <br>
         default currency symbol ($): <span id="currency-default">{{amount | currency}}</span><br>
         custom currency identifier (USD$): <span id="currency-custom">{{amount | currency:"USD$"}}</span>
         no fractions (0): <span id="currency-no-fractions">{{amount | currency:"USD$":0}}</span>
       </div>
     </file>
     <file name="protractor.js" type="protractor">
       it('should init with 1234.56', function() {
         expect(element(by.id('currency-default')).getText()).toBe('$1,234.56');
         expect(element(by.id('currency-custom')).getText()).toBe('USD$1,234.56');
         expect(element(by.id('currency-no-fractions')).getText()).toBe('USD$1,235');
       });
       it('should update', function() {
         if (browser.params.browser == 'safari') {
           // Safari does not understand the minus key. See
           // https://github.com/angular/protractor/issues/481
           return;
         }
         element(by.model('amount')).clear();
         element(by.model('amount')).sendKeys('-1234');
         expect(element(by.id('currency-default')).getText()).toBe('($1,234.00)');
         expect(element(by.id('currency-custom')).getText()).toBe('(USD$1,234.00)');
         expect(element(by.id('currency-no-fractions')).getText()).toBe('(USD$1,234)');
       });
     </file>
   </example>
 */
  currencyFilter.$inject = ['$locale'];
  function currencyFilter($locale) {
    var formats = $locale.NUMBER_FORMATS;
    return function (amount, currencySymbol, fractionSize) {
      if (isUndefined(currencySymbol)) {
        currencySymbol = formats.CURRENCY_SYM;
      }
      if (isUndefined(fractionSize)) {
        fractionSize = formats.PATTERNS[1].maxFrac;
      }
      // if null or undefined pass it through
      return amount == null ? amount : formatNumber(amount, formats.PATTERNS[1], formats.GROUP_SEP, formats.DECIMAL_SEP, fractionSize).replace(/\u00A4/g, currencySymbol);
    };
  }
  /**
 * @ngdoc filter
 * @name number
 * @kind function
 *
 * @description
 * Formats a number as text.
 *
 * If the input is null or undefined, it will just be returned.
 * If the input is infinite (Infinity/-Infinity) the Infinity symbol '∞' is returned.
 * If the input is not a number an empty string is returned.
 *
 * @param {number|string} number Number to format.
 * @param {(number|string)=} fractionSize Number of decimal places to round the number to.
 * If this is not provided then the fraction size is computed from the current locale's number
 * formatting pattern. In the case of the default locale, it will be 3.
 * @returns {string} Number rounded to decimalPlaces and places a “,” after each third digit.
 *
 * @example
   <example module="numberFilterExample">
     <file name="index.html">
       <script>
         angular.module('numberFilterExample', [])
           .controller('ExampleController', ['$scope', function($scope) {
             $scope.val = 1234.56789;
           }]);
       </script>
       <div ng-controller="ExampleController">
         Enter number: <input ng-model='val'><br>
         Default formatting: <span id='number-default'>{{val | number}}</span><br>
         No fractions: <span>{{val | number:0}}</span><br>
         Negative number: <span>{{-val | number:4}}</span>
       </div>
     </file>
     <file name="protractor.js" type="protractor">
       it('should format numbers', function() {
         expect(element(by.id('number-default')).getText()).toBe('1,234.568');
         expect(element(by.binding('val | number:0')).getText()).toBe('1,235');
         expect(element(by.binding('-val | number:4')).getText()).toBe('-1,234.5679');
       });

       it('should update', function() {
         element(by.model('val')).clear();
         element(by.model('val')).sendKeys('3374.333');
         expect(element(by.id('number-default')).getText()).toBe('3,374.333');
         expect(element(by.binding('val | number:0')).getText()).toBe('3,374');
         expect(element(by.binding('-val | number:4')).getText()).toBe('-3,374.3330');
      });
     </file>
   </example>
 */
  numberFilter.$inject = ['$locale'];
  function numberFilter($locale) {
    var formats = $locale.NUMBER_FORMATS;
    return function (number, fractionSize) {
      // if null or undefined pass it through
      return number == null ? number : formatNumber(number, formats.PATTERNS[0], formats.GROUP_SEP, formats.DECIMAL_SEP, fractionSize);
    };
  }
  var DECIMAL_SEP = '.';
  function formatNumber(number, pattern, groupSep, decimalSep, fractionSize) {
    if (!isFinite(number) || isObject(number))
      return '';
    var isNegative = number < 0;
    number = Math.abs(number);
    var numStr = number + '', formatedText = '', parts = [];
    var hasExponent = false;
    if (numStr.indexOf('e') !== -1) {
      var match = numStr.match(/([\d\.]+)e(-?)(\d+)/);
      if (match && match[2] == '-' && match[3] > fractionSize + 1) {
        number = 0;
      } else {
        formatedText = numStr;
        hasExponent = true;
      }
    }
    if (!hasExponent) {
      var fractionLen = (numStr.split(DECIMAL_SEP)[1] || '').length;
      // determine fractionSize if it is not specified
      if (isUndefined(fractionSize)) {
        fractionSize = Math.min(Math.max(pattern.minFrac, fractionLen), pattern.maxFrac);
      }
      // safely round numbers in JS without hitting imprecisions of floating-point arithmetics
      // inspired by:
      // https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Math/round
      number = +(Math.round(+(number.toString() + 'e' + fractionSize)).toString() + 'e' + -fractionSize);
      var fraction = ('' + number).split(DECIMAL_SEP);
      var whole = fraction[0];
      fraction = fraction[1] || '';
      var i, pos = 0, lgroup = pattern.lgSize, group = pattern.gSize;
      if (whole.length >= lgroup + group) {
        pos = whole.length - lgroup;
        for (i = 0; i < pos; i++) {
          if ((pos - i) % group === 0 && i !== 0) {
            formatedText += groupSep;
          }
          formatedText += whole.charAt(i);
        }
      }
      for (i = pos; i < whole.length; i++) {
        if ((whole.length - i) % lgroup === 0 && i !== 0) {
          formatedText += groupSep;
        }
        formatedText += whole.charAt(i);
      }
      // format fraction part.
      while (fraction.length < fractionSize) {
        fraction += '0';
      }
      if (fractionSize && fractionSize !== '0')
        formatedText += decimalSep + fraction.substr(0, fractionSize);
    } else {
      if (fractionSize > 0 && number < 1) {
        formatedText = number.toFixed(fractionSize);
        number = parseFloat(formatedText);
      }
    }
    if (number === 0) {
      isNegative = false;
    }
    parts.push(isNegative ? pattern.negPre : pattern.posPre, formatedText, isNegative ? pattern.negSuf : pattern.posSuf);
    return parts.join('');
  }
  function padNumber(num, digits, trim) {
    var neg = '';
    if (num < 0) {
      neg = '-';
      num = -num;
    }
    num = '' + num;
    while (num.length < digits)
      num = '0' + num;
    if (trim)
      num = num.substr(num.length - digits);
    return neg + num;
  }
  function dateGetter(name, size, offset, trim) {
    offset = offset || 0;
    return function (date) {
      var value = date['get' + name]();
      if (offset > 0 || value > -offset)
        value += offset;
      if (value === 0 && offset == -12)
        value = 12;
      return padNumber(value, size, trim);
    };
  }
  function dateStrGetter(name, shortForm) {
    return function (date, formats) {
      var value = date['get' + name]();
      var get = uppercase(shortForm ? 'SHORT' + name : name);
      return formats[get][value];
    };
  }
  function timeZoneGetter(date) {
    var zone = -1 * date.getTimezoneOffset();
    var paddedZone = zone >= 0 ? '+' : '';
    paddedZone += padNumber(Math[zone > 0 ? 'floor' : 'ceil'](zone / 60), 2) + padNumber(Math.abs(zone % 60), 2);
    return paddedZone;
  }
  function getFirstThursdayOfYear(year) {
    // 0 = index of January
    var dayOfWeekOnFirst = new Date(year, 0, 1).getDay();
    // 4 = index of Thursday (+1 to account for 1st = 5)
    // 11 = index of *next* Thursday (+1 account for 1st = 12)
    return new Date(year, 0, (dayOfWeekOnFirst <= 4 ? 5 : 12) - dayOfWeekOnFirst);
  }
  function getThursdayThisWeek(datetime) {
    return new Date(datetime.getFullYear(), datetime.getMonth(), datetime.getDate() + (4 - datetime.getDay()));
  }
  function weekGetter(size) {
    return function (date) {
      var firstThurs = getFirstThursdayOfYear(date.getFullYear()), thisThurs = getThursdayThisWeek(date);
      var diff = +thisThurs - +firstThurs, result = 1 + Math.round(diff / 604800000);
      // 6.048e8 ms per week
      return padNumber(result, size);
    };
  }
  function ampmGetter(date, formats) {
    return date.getHours() < 12 ? formats.AMPMS[0] : formats.AMPMS[1];
  }
  function eraGetter(date, formats) {
    return date.getFullYear() <= 0 ? formats.ERAS[0] : formats.ERAS[1];
  }
  function longEraGetter(date, formats) {
    return date.getFullYear() <= 0 ? formats.ERANAMES[0] : formats.ERANAMES[1];
  }
  var DATE_FORMATS = {
      yyyy: dateGetter('FullYear', 4),
      yy: dateGetter('FullYear', 2, 0, true),
      y: dateGetter('FullYear', 1),
      MMMM: dateStrGetter('Month'),
      MMM: dateStrGetter('Month', true),
      MM: dateGetter('Month', 2, 1),
      M: dateGetter('Month', 1, 1),
      dd: dateGetter('Date', 2),
      d: dateGetter('Date', 1),
      HH: dateGetter('Hours', 2),
      H: dateGetter('Hours', 1),
      hh: dateGetter('Hours', 2, -12),
      h: dateGetter('Hours', 1, -12),
      mm: dateGetter('Minutes', 2),
      m: dateGetter('Minutes', 1),
      ss: dateGetter('Seconds', 2),
      s: dateGetter('Seconds', 1),
      sss: dateGetter('Milliseconds', 3),
      EEEE: dateStrGetter('Day'),
      EEE: dateStrGetter('Day', true),
      a: ampmGetter,
      Z: timeZoneGetter,
      ww: weekGetter(2),
      w: weekGetter(1),
      G: eraGetter,
      GG: eraGetter,
      GGG: eraGetter,
      GGGG: longEraGetter
    };
  var DATE_FORMATS_SPLIT = /((?:[^yMdHhmsaZEwG']+)|(?:'(?:[^']|'')*')|(?:E+|y+|M+|d+|H+|h+|m+|s+|a|Z|G+|w+))(.*)/, NUMBER_STRING = /^\-?\d+$/;
  /**
 * @ngdoc filter
 * @name date
 * @kind function
 *
 * @description
 *   Formats `date` to a string based on the requested `format`.
 *
 *   `format` string can be composed of the following elements:
 *
 *   * `'yyyy'`: 4 digit representation of year (e.g. AD 1 => 0001, AD 2010 => 2010)
 *   * `'yy'`: 2 digit representation of year, padded (00-99). (e.g. AD 2001 => 01, AD 2010 => 10)
 *   * `'y'`: 1 digit representation of year, e.g. (AD 1 => 1, AD 199 => 199)
 *   * `'MMMM'`: Month in year (January-December)
 *   * `'MMM'`: Month in year (Jan-Dec)
 *   * `'MM'`: Month in year, padded (01-12)
 *   * `'M'`: Month in year (1-12)
 *   * `'dd'`: Day in month, padded (01-31)
 *   * `'d'`: Day in month (1-31)
 *   * `'EEEE'`: Day in Week,(Sunday-Saturday)
 *   * `'EEE'`: Day in Week, (Sun-Sat)
 *   * `'HH'`: Hour in day, padded (00-23)
 *   * `'H'`: Hour in day (0-23)
 *   * `'hh'`: Hour in AM/PM, padded (01-12)
 *   * `'h'`: Hour in AM/PM, (1-12)
 *   * `'mm'`: Minute in hour, padded (00-59)
 *   * `'m'`: Minute in hour (0-59)
 *   * `'ss'`: Second in minute, padded (00-59)
 *   * `'s'`: Second in minute (0-59)
 *   * `'sss'`: Millisecond in second, padded (000-999)
 *   * `'a'`: AM/PM marker
 *   * `'Z'`: 4 digit (+sign) representation of the timezone offset (-1200-+1200)
 *   * `'ww'`: Week of year, padded (00-53). Week 01 is the week with the first Thursday of the year
 *   * `'w'`: Week of year (0-53). Week 1 is the week with the first Thursday of the year
 *   * `'G'`, `'GG'`, `'GGG'`: The abbreviated form of the era string (e.g. 'AD')
 *   * `'GGGG'`: The long form of the era string (e.g. 'Anno Domini')
 *
 *   `format` string can also be one of the following predefined
 *   {@link guide/i18n localizable formats}:
 *
 *   * `'medium'`: equivalent to `'MMM d, y h:mm:ss a'` for en_US locale
 *     (e.g. Sep 3, 2010 12:05:08 PM)
 *   * `'short'`: equivalent to `'M/d/yy h:mm a'` for en_US  locale (e.g. 9/3/10 12:05 PM)
 *   * `'fullDate'`: equivalent to `'EEEE, MMMM d, y'` for en_US  locale
 *     (e.g. Friday, September 3, 2010)
 *   * `'longDate'`: equivalent to `'MMMM d, y'` for en_US  locale (e.g. September 3, 2010)
 *   * `'mediumDate'`: equivalent to `'MMM d, y'` for en_US  locale (e.g. Sep 3, 2010)
 *   * `'shortDate'`: equivalent to `'M/d/yy'` for en_US locale (e.g. 9/3/10)
 *   * `'mediumTime'`: equivalent to `'h:mm:ss a'` for en_US locale (e.g. 12:05:08 PM)
 *   * `'shortTime'`: equivalent to `'h:mm a'` for en_US locale (e.g. 12:05 PM)
 *
 *   `format` string can contain literal values. These need to be escaped by surrounding with single quotes (e.g.
 *   `"h 'in the morning'"`). In order to output a single quote, escape it - i.e., two single quotes in a sequence
 *   (e.g. `"h 'o''clock'"`).
 *
 * @param {(Date|number|string)} date Date to format either as Date object, milliseconds (string or
 *    number) or various ISO 8601 datetime string formats (e.g. yyyy-MM-ddTHH:mm:ss.sssZ and its
 *    shorter versions like yyyy-MM-ddTHH:mmZ, yyyy-MM-dd or yyyyMMddTHHmmssZ). If no timezone is
 *    specified in the string input, the time is considered to be in the local timezone.
 * @param {string=} format Formatting rules (see Description). If not specified,
 *    `mediumDate` is used.
 * @param {string=} timezone Timezone to be used for formatting. Right now, only `'UTC'` is supported.
 *    If not specified, the timezone of the browser will be used.
 * @returns {string} Formatted string or the input if input is not recognized as date/millis.
 *
 * @example
   <example>
     <file name="index.html">
       <span ng-non-bindable>{{1288323623006 | date:'medium'}}</span>:
           <span>{{1288323623006 | date:'medium'}}</span><br>
       <span ng-non-bindable>{{1288323623006 | date:'yyyy-MM-dd HH:mm:ss Z'}}</span>:
          <span>{{1288323623006 | date:'yyyy-MM-dd HH:mm:ss Z'}}</span><br>
       <span ng-non-bindable>{{1288323623006 | date:'MM/dd/yyyy @ h:mma'}}</span>:
          <span>{{'1288323623006' | date:'MM/dd/yyyy @ h:mma'}}</span><br>
       <span ng-non-bindable>{{1288323623006 | date:"MM/dd/yyyy 'at' h:mma"}}</span>:
          <span>{{'1288323623006' | date:"MM/dd/yyyy 'at' h:mma"}}</span><br>
     </file>
     <file name="protractor.js" type="protractor">
       it('should format date', function() {
         expect(element(by.binding("1288323623006 | date:'medium'")).getText()).
            toMatch(/Oct 2\d, 2010 \d{1,2}:\d{2}:\d{2} (AM|PM)/);
         expect(element(by.binding("1288323623006 | date:'yyyy-MM-dd HH:mm:ss Z'")).getText()).
            toMatch(/2010\-10\-2\d \d{2}:\d{2}:\d{2} (\-|\+)?\d{4}/);
         expect(element(by.binding("'1288323623006' | date:'MM/dd/yyyy @ h:mma'")).getText()).
            toMatch(/10\/2\d\/2010 @ \d{1,2}:\d{2}(AM|PM)/);
         expect(element(by.binding("'1288323623006' | date:\"MM/dd/yyyy 'at' h:mma\"")).getText()).
            toMatch(/10\/2\d\/2010 at \d{1,2}:\d{2}(AM|PM)/);
       });
     </file>
   </example>
 */
  dateFilter.$inject = ['$locale'];
  function dateFilter($locale) {
    var R_ISO8601_STR = /^(\d{4})-?(\d\d)-?(\d\d)(?:T(\d\d)(?::?(\d\d)(?::?(\d\d)(?:\.(\d+))?)?)?(Z|([+-])(\d\d):?(\d\d))?)?$/;
    // 1        2       3         4          5          6          7          8  9     10      11
    function jsonStringToDate(string) {
      var match;
      if (match = string.match(R_ISO8601_STR)) {
        var date = new Date(0), tzHour = 0, tzMin = 0, dateSetter = match[8] ? date.setUTCFullYear : date.setFullYear, timeSetter = match[8] ? date.setUTCHours : date.setHours;
        if (match[9]) {
          tzHour = int(match[9] + match[10]);
          tzMin = int(match[9] + match[11]);
        }
        dateSetter.call(date, int(match[1]), int(match[2]) - 1, int(match[3]));
        var h = int(match[4] || 0) - tzHour;
        var m = int(match[5] || 0) - tzMin;
        var s = int(match[6] || 0);
        var ms = Math.round(parseFloat('0.' + (match[7] || 0)) * 1000);
        timeSetter.call(date, h, m, s, ms);
        return date;
      }
      return string;
    }
    return function (date, format, timezone) {
      var text = '', parts = [], fn, match;
      format = format || 'mediumDate';
      format = $locale.DATETIME_FORMATS[format] || format;
      if (isString(date)) {
        date = NUMBER_STRING.test(date) ? int(date) : jsonStringToDate(date);
      }
      if (isNumber(date)) {
        date = new Date(date);
      }
      if (!isDate(date)) {
        return date;
      }
      while (format) {
        match = DATE_FORMATS_SPLIT.exec(format);
        if (match) {
          parts = concat(parts, match, 1);
          format = parts.pop();
        } else {
          parts.push(format);
          format = null;
        }
      }
      if (timezone && timezone === 'UTC') {
        date = new Date(date.getTime());
        date.setMinutes(date.getMinutes() + date.getTimezoneOffset());
      }
      forEach(parts, function (value) {
        fn = DATE_FORMATS[value];
        text += fn ? fn(date, $locale.DATETIME_FORMATS) : value.replace(/(^'|'$)/g, '').replace(/''/g, '\'');
      });
      return text;
    };
  }
  /**
 * @ngdoc filter
 * @name json
 * @kind function
 *
 * @description
 *   Allows you to convert a JavaScript object into JSON string.
 *
 *   This filter is mostly useful for debugging. When using the double curly {{value}} notation
 *   the binding is automatically converted to JSON.
 *
 * @param {*} object Any JavaScript object (including arrays and primitive types) to filter.
 * @param {number=} spacing The number of spaces to use per indentation, defaults to 2.
 * @returns {string} JSON string.
 *
 *
 * @example
   <example>
     <file name="index.html">
       <pre id="default-spacing">{{ {'name':'value'} | json }}</pre>
       <pre id="custom-spacing">{{ {'name':'value'} | json:4 }}</pre>
     </file>
     <file name="protractor.js" type="protractor">
       it('should jsonify filtered objects', function() {
         expect(element(by.id('default-spacing')).getText()).toMatch(/\{\n  "name": ?"value"\n}/);
         expect(element(by.id('custom-spacing')).getText()).toMatch(/\{\n    "name": ?"value"\n}/);
       });
     </file>
   </example>
 *
 */
  function jsonFilter() {
    return function (object, spacing) {
      if (isUndefined(spacing)) {
        spacing = 2;
      }
      return toJson(object, spacing);
    };
  }
  /**
 * @ngdoc filter
 * @name lowercase
 * @kind function
 * @description
 * Converts string to lowercase.
 * @see angular.lowercase
 */
  var lowercaseFilter = valueFn(lowercase);
  /**
 * @ngdoc filter
 * @name uppercase
 * @kind function
 * @description
 * Converts string to uppercase.
 * @see angular.uppercase
 */
  var uppercaseFilter = valueFn(uppercase);
  /**
 * @ngdoc filter
 * @name limitTo
 * @kind function
 *
 * @description
 * Creates a new array or string containing only a specified number of elements. The elements
 * are taken from either the beginning or the end of the source array, string or number, as specified by
 * the value and sign (positive or negative) of `limit`. If a number is used as input, it is
 * converted to a string.
 *
 * @param {Array|string|number} input Source array, string or number to be limited.
 * @param {string|number} limit The length of the returned array or string. If the `limit` number
 *     is positive, `limit` number of items from the beginning of the source array/string are copied.
 *     If the number is negative, `limit` number  of items from the end of the source array/string
 *     are copied. The `limit` will be trimmed if it exceeds `array.length`
 * @returns {Array|string} A new sub-array or substring of length `limit` or less if input array
 *     had less than `limit` elements.
 *
 * @example
   <example module="limitToExample">
     <file name="index.html">
       <script>
         angular.module('limitToExample', [])
           .controller('ExampleController', ['$scope', function($scope) {
             $scope.numbers = [1,2,3,4,5,6,7,8,9];
             $scope.letters = "abcdefghi";
             $scope.longNumber = 2345432342;
             $scope.numLimit = 3;
             $scope.letterLimit = 3;
             $scope.longNumberLimit = 3;
           }]);
       </script>
       <div ng-controller="ExampleController">
         Limit {{numbers}} to: <input type="number" step="1" ng-model="numLimit">
         <p>Output numbers: {{ numbers | limitTo:numLimit }}</p>
         Limit {{letters}} to: <input type="number" step="1" ng-model="letterLimit">
         <p>Output letters: {{ letters | limitTo:letterLimit }}</p>
         Limit {{longNumber}} to: <input type="number" step="1" ng-model="longNumberLimit">
         <p>Output long number: {{ longNumber | limitTo:longNumberLimit }}</p>
       </div>
     </file>
     <file name="protractor.js" type="protractor">
       var numLimitInput = element(by.model('numLimit'));
       var letterLimitInput = element(by.model('letterLimit'));
       var longNumberLimitInput = element(by.model('longNumberLimit'));
       var limitedNumbers = element(by.binding('numbers | limitTo:numLimit'));
       var limitedLetters = element(by.binding('letters | limitTo:letterLimit'));
       var limitedLongNumber = element(by.binding('longNumber | limitTo:longNumberLimit'));

       it('should limit the number array to first three items', function() {
         expect(numLimitInput.getAttribute('value')).toBe('3');
         expect(letterLimitInput.getAttribute('value')).toBe('3');
         expect(longNumberLimitInput.getAttribute('value')).toBe('3');
         expect(limitedNumbers.getText()).toEqual('Output numbers: [1,2,3]');
         expect(limitedLetters.getText()).toEqual('Output letters: abc');
         expect(limitedLongNumber.getText()).toEqual('Output long number: 234');
       });

       // There is a bug in safari and protractor that doesn't like the minus key
       // it('should update the output when -3 is entered', function() {
       //   numLimitInput.clear();
       //   numLimitInput.sendKeys('-3');
       //   letterLimitInput.clear();
       //   letterLimitInput.sendKeys('-3');
       //   longNumberLimitInput.clear();
       //   longNumberLimitInput.sendKeys('-3');
       //   expect(limitedNumbers.getText()).toEqual('Output numbers: [7,8,9]');
       //   expect(limitedLetters.getText()).toEqual('Output letters: ghi');
       //   expect(limitedLongNumber.getText()).toEqual('Output long number: 342');
       // });

       it('should not exceed the maximum size of input array', function() {
         numLimitInput.clear();
         numLimitInput.sendKeys('100');
         letterLimitInput.clear();
         letterLimitInput.sendKeys('100');
         longNumberLimitInput.clear();
         longNumberLimitInput.sendKeys('100');
         expect(limitedNumbers.getText()).toEqual('Output numbers: [1,2,3,4,5,6,7,8,9]');
         expect(limitedLetters.getText()).toEqual('Output letters: abcdefghi');
         expect(limitedLongNumber.getText()).toEqual('Output long number: 2345432342');
       });
     </file>
   </example>
*/
  function limitToFilter() {
    return function (input, limit) {
      if (isNumber(input))
        input = input.toString();
      if (!isArray(input) && !isString(input))
        return input;
      if (Math.abs(Number(limit)) === Infinity) {
        limit = Number(limit);
      } else {
        limit = int(limit);
      }
      //NaN check on limit
      if (limit) {
        return limit > 0 ? input.slice(0, limit) : input.slice(limit);
      } else {
        return isString(input) ? '' : [];
      }
    };
  }
  /**
 * @ngdoc filter
 * @name orderBy
 * @kind function
 *
 * @description
 * Orders a specified `array` by the `expression` predicate. It is ordered alphabetically
 * for strings and numerically for numbers. Note: if you notice numbers are not being sorted
 * correctly, make sure they are actually being saved as numbers and not strings.
 *
 * @param {Array} array The array to sort.
 * @param {function(*)|string|Array.<(function(*)|string)>=} expression A predicate to be
 *    used by the comparator to determine the order of elements.
 *
 *    Can be one of:
 *
 *    - `function`: Getter function. The result of this function will be sorted using the
 *      `<`, `===`, `>` operator.
 *    - `string`: An Angular expression. The result of this expression is used to compare elements
 *      (for example `name` to sort by a property called `name` or `name.substr(0, 3)` to sort by
 *      3 first characters of a property called `name`). The result of a constant expression
 *      is interpreted as a property name to be used in comparisons (for example `"special name"`
 *      to sort object by the value of their `special name` property). An expression can be
 *      optionally prefixed with `+` or `-` to control ascending or descending sort order
 *      (for example, `+name` or `-name`). If no property is provided, (e.g. `'+'`) then the array
 *      element itself is used to compare where sorting.
 *    - `Array`: An array of function or string predicates. The first predicate in the array
 *      is used for sorting, but when two items are equivalent, the next predicate is used.
 *
 *    If the predicate is missing or empty then it defaults to `'+'`.
 *
 * @param {boolean=} reverse Reverse the order of the array.
 * @returns {Array} Sorted copy of the source array.
 *
 *
 * @example
 * The example below demonstrates a simple ngRepeat, where the data is sorted
 * by age in descending order (predicate is set to `'-age'`).
 * `reverse` is not set, which means it defaults to `false`.
   <example module="orderByExample">
     <file name="index.html">
       <script>
         angular.module('orderByExample', [])
           .controller('ExampleController', ['$scope', function($scope) {
             $scope.friends =
                 [{name:'John', phone:'555-1212', age:10},
                  {name:'Mary', phone:'555-9876', age:19},
                  {name:'Mike', phone:'555-4321', age:21},
                  {name:'Adam', phone:'555-5678', age:35},
                  {name:'Julie', phone:'555-8765', age:29}];
           }]);
       </script>
       <div ng-controller="ExampleController">
         <table class="friend">
           <tr>
             <th>Name</th>
             <th>Phone Number</th>
             <th>Age</th>
           </tr>
           <tr ng-repeat="friend in friends | orderBy:'-age'">
             <td>{{friend.name}}</td>
             <td>{{friend.phone}}</td>
             <td>{{friend.age}}</td>
           </tr>
         </table>
       </div>
     </file>
   </example>
 *
 * The predicate and reverse parameters can be controlled dynamically through scope properties,
 * as shown in the next example.
 * @example
   <example module="orderByExample">
     <file name="index.html">
       <script>
         angular.module('orderByExample', [])
           .controller('ExampleController', ['$scope', function($scope) {
             $scope.friends =
                 [{name:'John', phone:'555-1212', age:10},
                  {name:'Mary', phone:'555-9876', age:19},
                  {name:'Mike', phone:'555-4321', age:21},
                  {name:'Adam', phone:'555-5678', age:35},
                  {name:'Julie', phone:'555-8765', age:29}];
             $scope.predicate = '-age';
           }]);
       </script>
       <div ng-controller="ExampleController">
         <pre>Sorting predicate = {{predicate}}; reverse = {{reverse}}</pre>
         <hr/>
         [ <a href="" ng-click="predicate=''">unsorted</a> ]
         <table class="friend">
           <tr>
             <th><a href="" ng-click="predicate = 'name'; reverse=false">Name</a>
                 (<a href="" ng-click="predicate = '-name'; reverse=false">^</a>)</th>
             <th><a href="" ng-click="predicate = 'phone'; reverse=!reverse">Phone Number</a></th>
             <th><a href="" ng-click="predicate = 'age'; reverse=!reverse">Age</a></th>
           </tr>
           <tr ng-repeat="friend in friends | orderBy:predicate:reverse">
             <td>{{friend.name}}</td>
             <td>{{friend.phone}}</td>
             <td>{{friend.age}}</td>
           </tr>
         </table>
       </div>
     </file>
   </example>
 *
 * It's also possible to call the orderBy filter manually, by injecting `$filter`, retrieving the
 * filter routine with `$filter('orderBy')`, and calling the returned filter routine with the
 * desired parameters.
 *
 * Example:
 *
 * @example
  <example module="orderByExample">
    <file name="index.html">
      <div ng-controller="ExampleController">
        <table class="friend">
          <tr>
            <th><a href="" ng-click="reverse=false;order('name', false)">Name</a>
              (<a href="" ng-click="order('-name',false)">^</a>)</th>
            <th><a href="" ng-click="reverse=!reverse;order('phone', reverse)">Phone Number</a></th>
            <th><a href="" ng-click="reverse=!reverse;order('age',reverse)">Age</a></th>
          </tr>
          <tr ng-repeat="friend in friends">
            <td>{{friend.name}}</td>
            <td>{{friend.phone}}</td>
            <td>{{friend.age}}</td>
          </tr>
        </table>
      </div>
    </file>

    <file name="script.js">
      angular.module('orderByExample', [])
        .controller('ExampleController', ['$scope', '$filter', function($scope, $filter) {
          var orderBy = $filter('orderBy');
          $scope.friends = [
            { name: 'John',    phone: '555-1212',    age: 10 },
            { name: 'Mary',    phone: '555-9876',    age: 19 },
            { name: 'Mike',    phone: '555-4321',    age: 21 },
            { name: 'Adam',    phone: '555-5678',    age: 35 },
            { name: 'Julie',   phone: '555-8765',    age: 29 }
          ];
          $scope.order = function(predicate, reverse) {
            $scope.friends = orderBy($scope.friends, predicate, reverse);
          };
          $scope.order('-age',false);
        }]);
    </file>
</example>
 */
  orderByFilter.$inject = ['$parse'];
  function orderByFilter($parse) {
    return function (array, sortPredicate, reverseOrder) {
      if (!isArrayLike(array))
        return array;
      sortPredicate = isArray(sortPredicate) ? sortPredicate : [sortPredicate];
      if (sortPredicate.length === 0) {
        sortPredicate = ['+'];
      }
      sortPredicate = sortPredicate.map(function (predicate) {
        var descending = false, get = predicate || identity;
        if (isString(predicate)) {
          if (predicate.charAt(0) == '+' || predicate.charAt(0) == '-') {
            descending = predicate.charAt(0) == '-';
            predicate = predicate.substring(1);
          }
          if (predicate === '') {
            // Effectively no predicate was passed so we compare identity
            return reverseComparator(compare, descending);
          }
          get = $parse(predicate);
          if (get.constant) {
            var key = get();
            return reverseComparator(function (a, b) {
              return compare(a[key], b[key]);
            }, descending);
          }
        }
        return reverseComparator(function (a, b) {
          return compare(get(a), get(b));
        }, descending);
      });
      return slice.call(array).sort(reverseComparator(comparator, reverseOrder));
      function comparator(o1, o2) {
        for (var i = 0; i < sortPredicate.length; i++) {
          var comp = sortPredicate[i](o1, o2);
          if (comp !== 0)
            return comp;
        }
        return 0;
      }
      function reverseComparator(comp, descending) {
        return descending ? function (a, b) {
          return comp(b, a);
        } : comp;
      }
      function isPrimitive(value) {
        switch (typeof value) {
        case 'number':
        /* falls through */
        case 'boolean':
        /* falls through */
        case 'string':
          return true;
        default:
          return false;
        }
      }
      function objectToString(value) {
        if (value === null)
          return 'null';
        if (typeof value.valueOf === 'function') {
          value = value.valueOf();
          if (isPrimitive(value))
            return value;
        }
        if (typeof value.toString === 'function') {
          value = value.toString();
          if (isPrimitive(value))
            return value;
        }
        return '';
      }
      function compare(v1, v2) {
        var t1 = typeof v1;
        var t2 = typeof v2;
        if (t1 === t2 && t1 === 'object') {
          v1 = objectToString(v1);
          v2 = objectToString(v2);
        }
        if (t1 === t2) {
          if (t1 === 'string') {
            v1 = v1.toLowerCase();
            v2 = v2.toLowerCase();
          }
          if (v1 === v2)
            return 0;
          return v1 < v2 ? -1 : 1;
        } else {
          return t1 < t2 ? -1 : 1;
        }
      }
    };
  }
  function ngDirective(directive) {
    if (isFunction(directive)) {
      directive = { link: directive };
    }
    directive.restrict = directive.restrict || 'AC';
    return valueFn(directive);
  }
  /**
 * @ngdoc directive
 * @name a
 * @restrict E
 *
 * @description
 * Modifies the default behavior of the html A tag so that the default action is prevented when
 * the href attribute is empty.
 *
 * This change permits the easy creation of action links with the `ngClick` directive
 * without changing the location or causing page reloads, e.g.:
 * `<a href="" ng-click="list.addItem()">Add Item</a>`
 */
  var htmlAnchorDirective = valueFn({
      restrict: 'E',
      compile: function (element, attr) {
        if (!attr.href && !attr.xlinkHref && !attr.name) {
          return function (scope, element) {
            // If the linked element is not an anchor tag anymore, do nothing
            if (element[0].nodeName.toLowerCase() !== 'a')
              return;
            // SVGAElement does not use the href attribute, but rather the 'xlinkHref' attribute.
            var href = toString.call(element.prop('href')) === '[object SVGAnimatedString]' ? 'xlink:href' : 'href';
            element.on('click', function (event) {
              // if we have no href url, then don't navigate anywhere.
              if (!element.attr(href)) {
                event.preventDefault();
              }
            });
          };
        }
      }
    });
  /**
 * @ngdoc directive
 * @name ngHref
 * @restrict A
 * @priority 99
 *
 * @description
 * Using Angular markup like `{{hash}}` in an href attribute will
 * make the link go to the wrong URL if the user clicks it before
 * Angular has a chance to replace the `{{hash}}` markup with its
 * value. Until Angular replaces the markup the link will be broken
 * and will most likely return a 404 error. The `ngHref` directive
 * solves this problem.
 *
 * The wrong way to write it:
 * ```html
 * <a href="http://www.gravatar.com/avatar/{{hash}}">link1</a>
 * ```
 *
 * The correct way to write it:
 * ```html
 * <a ng-href="http://www.gravatar.com/avatar/{{hash}}">link1</a>
 * ```
 *
 * @element A
 * @param {template} ngHref any string which can contain `{{}}` markup.
 *
 * @example
 * This example shows various combinations of `href`, `ng-href` and `ng-click` attributes
 * in links and their different behaviors:
    <example>
      <file name="index.html">
        <input ng-model="value" /><br />
        <a id="link-1" href ng-click="value = 1">link 1</a> (link, don't reload)<br />
        <a id="link-2" href="" ng-click="value = 2">link 2</a> (link, don't reload)<br />
        <a id="link-3" ng-href="/{{'123'}}">link 3</a> (link, reload!)<br />
        <a id="link-4" href="" name="xx" ng-click="value = 4">anchor</a> (link, don't reload)<br />
        <a id="link-5" name="xxx" ng-click="value = 5">anchor</a> (no link)<br />
        <a id="link-6" ng-href="{{value}}">link</a> (link, change location)
      </file>
      <file name="protractor.js" type="protractor">
        it('should execute ng-click but not reload when href without value', function() {
          element(by.id('link-1')).click();
          expect(element(by.model('value')).getAttribute('value')).toEqual('1');
          expect(element(by.id('link-1')).getAttribute('href')).toBe('');
        });

        it('should execute ng-click but not reload when href empty string', function() {
          element(by.id('link-2')).click();
          expect(element(by.model('value')).getAttribute('value')).toEqual('2');
          expect(element(by.id('link-2')).getAttribute('href')).toBe('');
        });

        it('should execute ng-click and change url when ng-href specified', function() {
          expect(element(by.id('link-3')).getAttribute('href')).toMatch(/\/123$/);

          element(by.id('link-3')).click();

          // At this point, we navigate away from an Angular page, so we need
          // to use browser.driver to get the base webdriver.

          browser.wait(function() {
            return browser.driver.getCurrentUrl().then(function(url) {
              return url.match(/\/123$/);
            });
          }, 5000, 'page should navigate to /123');
        });

        xit('should execute ng-click but not reload when href empty string and name specified', function() {
          element(by.id('link-4')).click();
          expect(element(by.model('value')).getAttribute('value')).toEqual('4');
          expect(element(by.id('link-4')).getAttribute('href')).toBe('');
        });

        it('should execute ng-click but not reload when no href but name specified', function() {
          element(by.id('link-5')).click();
          expect(element(by.model('value')).getAttribute('value')).toEqual('5');
          expect(element(by.id('link-5')).getAttribute('href')).toBe(null);
        });

        it('should only change url when only ng-href', function() {
          element(by.model('value')).clear();
          element(by.model('value')).sendKeys('6');
          expect(element(by.id('link-6')).getAttribute('href')).toMatch(/\/6$/);

          element(by.id('link-6')).click();

          // At this point, we navigate away from an Angular page, so we need
          // to use browser.driver to get the base webdriver.
          browser.wait(function() {
            return browser.driver.getCurrentUrl().then(function(url) {
              return url.match(/\/6$/);
            });
          }, 5000, 'page should navigate to /6');
        });
      </file>
    </example>
 */
  /**
 * @ngdoc directive
 * @name ngSrc
 * @restrict A
 * @priority 99
 *
 * @description
 * Using Angular markup like `{{hash}}` in a `src` attribute doesn't
 * work right: The browser will fetch from the URL with the literal
 * text `{{hash}}` until Angular replaces the expression inside
 * `{{hash}}`. The `ngSrc` directive solves this problem.
 *
 * The buggy way to write it:
 * ```html
 * <img src="http://www.gravatar.com/avatar/{{hash}}"/>
 * ```
 *
 * The correct way to write it:
 * ```html
 * <img ng-src="http://www.gravatar.com/avatar/{{hash}}"/>
 * ```
 *
 * @element IMG
 * @param {template} ngSrc any string which can contain `{{}}` markup.
 */
  /**
 * @ngdoc directive
 * @name ngSrcset
 * @restrict A
 * @priority 99
 *
 * @description
 * Using Angular markup like `{{hash}}` in a `srcset` attribute doesn't
 * work right: The browser will fetch from the URL with the literal
 * text `{{hash}}` until Angular replaces the expression inside
 * `{{hash}}`. The `ngSrcset` directive solves this problem.
 *
 * The buggy way to write it:
 * ```html
 * <img srcset="http://www.gravatar.com/avatar/{{hash}} 2x"/>
 * ```
 *
 * The correct way to write it:
 * ```html
 * <img ng-srcset="http://www.gravatar.com/avatar/{{hash}} 2x"/>
 * ```
 *
 * @element IMG
 * @param {template} ngSrcset any string which can contain `{{}}` markup.
 */
  /**
 * @ngdoc directive
 * @name ngDisabled
 * @restrict A
 * @priority 100
 *
 * @description
 *
 * This directive sets the `disabled` attribute on the element if the
 * {@link guide/expression expression} inside `ngDisabled` evaluates to truthy.
 *
 * A special directive is necessary because we cannot use interpolation inside the `disabled`
 * attribute.  The following example would make the button enabled on Chrome/Firefox
 * but not on older IEs:
 *
 * ```html
 * <!-- See below for an example of ng-disabled being used correctly -->
 * <div ng-init="isDisabled = false">
 *  <button disabled="{{isDisabled}}">Disabled</button>
 * </div>
 * ```
 *
 * This is because the HTML specification does not require browsers to preserve the values of
 * boolean attributes such as `disabled` (Their presence means true and their absence means false.)
 * If we put an Angular interpolation expression into such an attribute then the
 * binding information would be lost when the browser removes the attribute.
 *
 * @example
    <example>
      <file name="index.html">
        Click me to toggle: <input type="checkbox" ng-model="checked"><br/>
        <button ng-model="button" ng-disabled="checked">Button</button>
      </file>
      <file name="protractor.js" type="protractor">
        it('should toggle button', function() {
          expect(element(by.css('button')).getAttribute('disabled')).toBeFalsy();
          element(by.model('checked')).click();
          expect(element(by.css('button')).getAttribute('disabled')).toBeTruthy();
        });
      </file>
    </example>
 *
 * @element INPUT
 * @param {expression} ngDisabled If the {@link guide/expression expression} is truthy,
 *     then the `disabled` attribute will be set on the element
 */
  /**
 * @ngdoc directive
 * @name ngChecked
 * @restrict A
 * @priority 100
 *
 * @description
 * The HTML specification does not require browsers to preserve the values of boolean attributes
 * such as checked. (Their presence means true and their absence means false.)
 * If we put an Angular interpolation expression into such an attribute then the
 * binding information would be lost when the browser removes the attribute.
 * The `ngChecked` directive solves this problem for the `checked` attribute.
 * This complementary directive is not removed by the browser and so provides
 * a permanent reliable place to store the binding information.
 * @example
    <example>
      <file name="index.html">
        Check me to check both: <input type="checkbox" ng-model="master"><br/>
        <input id="checkSlave" type="checkbox" ng-checked="master">
      </file>
      <file name="protractor.js" type="protractor">
        it('should check both checkBoxes', function() {
          expect(element(by.id('checkSlave')).getAttribute('checked')).toBeFalsy();
          element(by.model('master')).click();
          expect(element(by.id('checkSlave')).getAttribute('checked')).toBeTruthy();
        });
      </file>
    </example>
 *
 * @element INPUT
 * @param {expression} ngChecked If the {@link guide/expression expression} is truthy,
 *     then special attribute "checked" will be set on the element
 */
  /**
 * @ngdoc directive
 * @name ngReadonly
 * @restrict A
 * @priority 100
 *
 * @description
 * The HTML specification does not require browsers to preserve the values of boolean attributes
 * such as readonly. (Their presence means true and their absence means false.)
 * If we put an Angular interpolation expression into such an attribute then the
 * binding information would be lost when the browser removes the attribute.
 * The `ngReadonly` directive solves this problem for the `readonly` attribute.
 * This complementary directive is not removed by the browser and so provides
 * a permanent reliable place to store the binding information.
 * @example
    <example>
      <file name="index.html">
        Check me to make text readonly: <input type="checkbox" ng-model="checked"><br/>
        <input type="text" ng-readonly="checked" value="I'm Angular"/>
      </file>
      <file name="protractor.js" type="protractor">
        it('should toggle readonly attr', function() {
          expect(element(by.css('[type="text"]')).getAttribute('readonly')).toBeFalsy();
          element(by.model('checked')).click();
          expect(element(by.css('[type="text"]')).getAttribute('readonly')).toBeTruthy();
        });
      </file>
    </example>
 *
 * @element INPUT
 * @param {expression} ngReadonly If the {@link guide/expression expression} is truthy,
 *     then special attribute "readonly" will be set on the element
 */
  /**
 * @ngdoc directive
 * @name ngSelected
 * @restrict A
 * @priority 100
 *
 * @description
 * The HTML specification does not require browsers to preserve the values of boolean attributes
 * such as selected. (Their presence means true and their absence means false.)
 * If we put an Angular interpolation expression into such an attribute then the
 * binding information would be lost when the browser removes the attribute.
 * The `ngSelected` directive solves this problem for the `selected` attribute.
 * This complementary directive is not removed by the browser and so provides
 * a permanent reliable place to store the binding information.
 *
 * @example
    <example>
      <file name="index.html">
        Check me to select: <input type="checkbox" ng-model="selected"><br/>
        <select>
          <option>Hello!</option>
          <option id="greet" ng-selected="selected">Greetings!</option>
        </select>
      </file>
      <file name="protractor.js" type="protractor">
        it('should select Greetings!', function() {
          expect(element(by.id('greet')).getAttribute('selected')).toBeFalsy();
          element(by.model('selected')).click();
          expect(element(by.id('greet')).getAttribute('selected')).toBeTruthy();
        });
      </file>
    </example>
 *
 * @element OPTION
 * @param {expression} ngSelected If the {@link guide/expression expression} is truthy,
 *     then special attribute "selected" will be set on the element
 */
  /**
 * @ngdoc directive
 * @name ngOpen
 * @restrict A
 * @priority 100
 *
 * @description
 * The HTML specification does not require browsers to preserve the values of boolean attributes
 * such as open. (Their presence means true and their absence means false.)
 * If we put an Angular interpolation expression into such an attribute then the
 * binding information would be lost when the browser removes the attribute.
 * The `ngOpen` directive solves this problem for the `open` attribute.
 * This complementary directive is not removed by the browser and so provides
 * a permanent reliable place to store the binding information.
 * @example
     <example>
       <file name="index.html">
         Check me check multiple: <input type="checkbox" ng-model="open"><br/>
         <details id="details" ng-open="open">
            <summary>Show/Hide me</summary>
         </details>
       </file>
       <file name="protractor.js" type="protractor">
         it('should toggle open', function() {
           expect(element(by.id('details')).getAttribute('open')).toBeFalsy();
           element(by.model('open')).click();
           expect(element(by.id('details')).getAttribute('open')).toBeTruthy();
         });
       </file>
     </example>
 *
 * @element DETAILS
 * @param {expression} ngOpen If the {@link guide/expression expression} is truthy,
 *     then special attribute "open" will be set on the element
 */
  var ngAttributeAliasDirectives = {};
  // boolean attrs are evaluated
  forEach(BOOLEAN_ATTR, function (propName, attrName) {
    // binding to multiple is not supported
    if (propName == 'multiple')
      return;
    var normalized = directiveNormalize('ng-' + attrName);
    ngAttributeAliasDirectives[normalized] = function () {
      return {
        restrict: 'A',
        priority: 100,
        link: function (scope, element, attr) {
          scope.$watch(attr[normalized], function ngBooleanAttrWatchAction(value) {
            attr.$set(attrName, !!value);
          });
        }
      };
    };
  });
  // aliased input attrs are evaluated
  forEach(ALIASED_ATTR, function (htmlAttr, ngAttr) {
    ngAttributeAliasDirectives[ngAttr] = function () {
      return {
        priority: 100,
        link: function (scope, element, attr) {
          //special case ngPattern when a literal regular expression value
          //is used as the expression (this way we don't have to watch anything).
          if (ngAttr === 'ngPattern' && attr.ngPattern.charAt(0) == '/') {
            var match = attr.ngPattern.match(REGEX_STRING_REGEXP);
            if (match) {
              attr.$set('ngPattern', new RegExp(match[1], match[2]));
              return;
            }
          }
          scope.$watch(attr[ngAttr], function ngAttrAliasWatchAction(value) {
            attr.$set(ngAttr, value);
          });
        }
      };
    };
  });
  // ng-src, ng-srcset, ng-href are interpolated
  forEach([
    'src',
    'srcset',
    'href'
  ], function (attrName) {
    var normalized = directiveNormalize('ng-' + attrName);
    ngAttributeAliasDirectives[normalized] = function () {
      return {
        priority: 99,
        link: function (scope, element, attr) {
          var propName = attrName, name = attrName;
          if (attrName === 'href' && toString.call(element.prop('href')) === '[object SVGAnimatedString]') {
            name = 'xlinkHref';
            attr.$attr[name] = 'xlink:href';
            propName = null;
          }
          attr.$observe(normalized, function (value) {
            if (!value) {
              if (attrName === 'href') {
                attr.$set(name, null);
              }
              return;
            }
            attr.$set(name, value);
            // on IE, if "ng:src" directive declaration is used and "src" attribute doesn't exist
            // then calling element.setAttribute('src', 'foo') doesn't do anything, so we need
            // to set the property as well to achieve the desired effect.
            // we use attr[attrName] value since $set can sanitize the url.
            if (msie && propName)
              element.prop(propName, attr[name]);
          });
        }
      };
    };
  });
  /* global -nullFormCtrl, -SUBMITTED_CLASS, addSetValidityMethod: true
 */
  var nullFormCtrl = {
      $addControl: noop,
      $$renameControl: nullFormRenameControl,
      $removeControl: noop,
      $setValidity: noop,
      $setDirty: noop,
      $setPristine: noop,
      $setSubmitted: noop
    }, SUBMITTED_CLASS = 'ng-submitted';
  function nullFormRenameControl(control, name) {
    control.$name = name;
  }
  /**
 * @ngdoc type
 * @name form.FormController
 *
 * @property {boolean} $pristine True if user has not interacted with the form yet.
 * @property {boolean} $dirty True if user has already interacted with the form.
 * @property {boolean} $valid True if all of the containing forms and controls are valid.
 * @property {boolean} $invalid True if at least one containing control or form is invalid.
 * @property {boolean} $submitted True if user has submitted the form even if its invalid.
 *
 * @property {Object} $error Is an object hash, containing references to controls or
 *  forms with failing validators, where:
 *
 *  - keys are validation tokens (error names),
 *  - values are arrays of controls or forms that have a failing validator for given error name.
 *
 *  Built-in validation tokens:
 *
 *  - `email`
 *  - `max`
 *  - `maxlength`
 *  - `min`
 *  - `minlength`
 *  - `number`
 *  - `pattern`
 *  - `required`
 *  - `url`
 *  - `date`
 *  - `datetimelocal`
 *  - `time`
 *  - `week`
 *  - `month`
 *
 * @description
 * `FormController` keeps track of all its controls and nested forms as well as the state of them,
 * such as being valid/invalid or dirty/pristine.
 *
 * Each {@link ng.directive:form form} directive creates an instance
 * of `FormController`.
 *
 */
  //asks for $scope to fool the BC controller module
  FormController.$inject = [
    '$element',
    '$attrs',
    '$scope',
    '$animate',
    '$interpolate'
  ];
  function FormController(element, attrs, $scope, $animate, $interpolate) {
    var form = this, controls = [];
    var parentForm = form.$$parentForm = element.parent().controller('form') || nullFormCtrl;
    // init state
    form.$error = {};
    form.$$success = {};
    form.$pending = undefined;
    form.$name = $interpolate(attrs.name || attrs.ngForm || '')($scope);
    form.$dirty = false;
    form.$pristine = true;
    form.$valid = true;
    form.$invalid = false;
    form.$submitted = false;
    parentForm.$addControl(form);
    /**
   * @ngdoc method
   * @name form.FormController#$rollbackViewValue
   *
   * @description
   * Rollback all form controls pending updates to the `$modelValue`.
   *
   * Updates may be pending by a debounced event or because the input is waiting for a some future
   * event defined in `ng-model-options`. This method is typically needed by the reset button of
   * a form that uses `ng-model-options` to pend updates.
   */
    form.$rollbackViewValue = function () {
      forEach(controls, function (control) {
        control.$rollbackViewValue();
      });
    };
    /**
   * @ngdoc method
   * @name form.FormController#$commitViewValue
   *
   * @description
   * Commit all form controls pending updates to the `$modelValue`.
   *
   * Updates may be pending by a debounced event or because the input is waiting for a some future
   * event defined in `ng-model-options`. This method is rarely needed as `NgModelController`
   * usually handles calling this in response to input events.
   */
    form.$commitViewValue = function () {
      forEach(controls, function (control) {
        control.$commitViewValue();
      });
    };
    /**
   * @ngdoc method
   * @name form.FormController#$addControl
   *
   * @description
   * Register a control with the form.
   *
   * Input elements using ngModelController do this automatically when they are linked.
   */
    form.$addControl = function (control) {
      // Breaking change - before, inputs whose name was "hasOwnProperty" were quietly ignored
      // and not added to the scope.  Now we throw an error.
      assertNotHasOwnProperty(control.$name, 'input');
      controls.push(control);
      if (control.$name) {
        form[control.$name] = control;
      }
    };
    // Private API: rename a form control
    form.$$renameControl = function (control, newName) {
      var oldName = control.$name;
      if (form[oldName] === control) {
        delete form[oldName];
      }
      form[newName] = control;
      control.$name = newName;
    };
    /**
   * @ngdoc method
   * @name form.FormController#$removeControl
   *
   * @description
   * Deregister a control from the form.
   *
   * Input elements using ngModelController do this automatically when they are destroyed.
   */
    form.$removeControl = function (control) {
      if (control.$name && form[control.$name] === control) {
        delete form[control.$name];
      }
      forEach(form.$pending, function (value, name) {
        form.$setValidity(name, null, control);
      });
      forEach(form.$error, function (value, name) {
        form.$setValidity(name, null, control);
      });
      forEach(form.$$success, function (value, name) {
        form.$setValidity(name, null, control);
      });
      arrayRemove(controls, control);
    };
    /**
   * @ngdoc method
   * @name form.FormController#$setValidity
   *
   * @description
   * Sets the validity of a form control.
   *
   * This method will also propagate to parent forms.
   */
    addSetValidityMethod({
      ctrl: this,
      $element: element,
      set: function (object, property, controller) {
        var list = object[property];
        if (!list) {
          object[property] = [controller];
        } else {
          var index = list.indexOf(controller);
          if (index === -1) {
            list.push(controller);
          }
        }
      },
      unset: function (object, property, controller) {
        var list = object[property];
        if (!list) {
          return;
        }
        arrayRemove(list, controller);
        if (list.length === 0) {
          delete object[property];
        }
      },
      parentForm: parentForm,
      $animate: $animate
    });
    /**
   * @ngdoc method
   * @name form.FormController#$setDirty
   *
   * @description
   * Sets the form to a dirty state.
   *
   * This method can be called to add the 'ng-dirty' class and set the form to a dirty
   * state (ng-dirty class). This method will also propagate to parent forms.
   */
    form.$setDirty = function () {
      $animate.removeClass(element, PRISTINE_CLASS);
      $animate.addClass(element, DIRTY_CLASS);
      form.$dirty = true;
      form.$pristine = false;
      parentForm.$setDirty();
    };
    /**
   * @ngdoc method
   * @name form.FormController#$setPristine
   *
   * @description
   * Sets the form to its pristine state.
   *
   * This method can be called to remove the 'ng-dirty' class and set the form to its pristine
   * state (ng-pristine class). This method will also propagate to all the controls contained
   * in this form.
   *
   * Setting a form back to a pristine state is often useful when we want to 'reuse' a form after
   * saving or resetting it.
   */
    form.$setPristine = function () {
      $animate.setClass(element, PRISTINE_CLASS, DIRTY_CLASS + ' ' + SUBMITTED_CLASS);
      form.$dirty = false;
      form.$pristine = true;
      form.$submitted = false;
      forEach(controls, function (control) {
        control.$setPristine();
      });
    };
    /**
   * @ngdoc method
   * @name form.FormController#$setUntouched
   *
   * @description
   * Sets the form to its untouched state.
   *
   * This method can be called to remove the 'ng-touched' class and set the form controls to their
   * untouched state (ng-untouched class).
   *
   * Setting a form controls back to their untouched state is often useful when setting the form
   * back to its pristine state.
   */
    form.$setUntouched = function () {
      forEach(controls, function (control) {
        control.$setUntouched();
      });
    };
    /**
   * @ngdoc method
   * @name form.FormController#$setSubmitted
   *
   * @description
   * Sets the form to its submitted state.
   */
    form.$setSubmitted = function () {
      $animate.addClass(element, SUBMITTED_CLASS);
      form.$submitted = true;
      parentForm.$setSubmitted();
    };
  }
  /**
 * @ngdoc directive
 * @name ngForm
 * @restrict EAC
 *
 * @description
 * Nestable alias of {@link ng.directive:form `form`} directive. HTML
 * does not allow nesting of form elements. It is useful to nest forms, for example if the validity of a
 * sub-group of controls needs to be determined.
 *
 * Note: the purpose of `ngForm` is to group controls,
 * but not to be a replacement for the `<form>` tag with all of its capabilities
 * (e.g. posting to the server, ...).
 *
 * @param {string=} ngForm|name Name of the form. If specified, the form controller will be published into
 *                       related scope, under this name.
 *
 */
  /**
 * @ngdoc directive
 * @name form
 * @restrict E
 *
 * @description
 * Directive that instantiates
 * {@link form.FormController FormController}.
 *
 * If the `name` attribute is specified, the form controller is published onto the current scope under
 * this name.
 *
 * # Alias: {@link ng.directive:ngForm `ngForm`}
 *
 * In Angular, forms can be nested. This means that the outer form is valid when all of the child
 * forms are valid as well. However, browsers do not allow nesting of `<form>` elements, so
 * Angular provides the {@link ng.directive:ngForm `ngForm`} directive which behaves identically to
 * `<form>` but can be nested.  This allows you to have nested forms, which is very useful when
 * using Angular validation directives in forms that are dynamically generated using the
 * {@link ng.directive:ngRepeat `ngRepeat`} directive. Since you cannot dynamically generate the `name`
 * attribute of input elements using interpolation, you have to wrap each set of repeated inputs in an
 * `ngForm` directive and nest these in an outer `form` element.
 *
 *
 * # CSS classes
 *  - `ng-valid` is set if the form is valid.
 *  - `ng-invalid` is set if the form is invalid.
 *  - `ng-pristine` is set if the form is pristine.
 *  - `ng-dirty` is set if the form is dirty.
 *  - `ng-submitted` is set if the form was submitted.
 *
 * Keep in mind that ngAnimate can detect each of these classes when added and removed.
 *
 *
 * # Submitting a form and preventing the default action
 *
 * Since the role of forms in client-side Angular applications is different than in classical
 * roundtrip apps, it is desirable for the browser not to translate the form submission into a full
 * page reload that sends the data to the server. Instead some javascript logic should be triggered
 * to handle the form submission in an application-specific way.
 *
 * For this reason, Angular prevents the default action (form submission to the server) unless the
 * `<form>` element has an `action` attribute specified.
 *
 * You can use one of the following two ways to specify what javascript method should be called when
 * a form is submitted:
 *
 * - {@link ng.directive:ngSubmit ngSubmit} directive on the form element
 * - {@link ng.directive:ngClick ngClick} directive on the first
  *  button or input field of type submit (input[type=submit])
 *
 * To prevent double execution of the handler, use only one of the {@link ng.directive:ngSubmit ngSubmit}
 * or {@link ng.directive:ngClick ngClick} directives.
 * This is because of the following form submission rules in the HTML specification:
 *
 * - If a form has only one input field then hitting enter in this field triggers form submit
 * (`ngSubmit`)
 * - if a form has 2+ input fields and no buttons or input[type=submit] then hitting enter
 * doesn't trigger submit
 * - if a form has one or more input fields and one or more buttons or input[type=submit] then
 * hitting enter in any of the input fields will trigger the click handler on the *first* button or
 * input[type=submit] (`ngClick`) *and* a submit handler on the enclosing form (`ngSubmit`)
 *
 * Any pending `ngModelOptions` changes will take place immediately when an enclosing form is
 * submitted. Note that `ngClick` events will occur before the model is updated. Use `ngSubmit`
 * to have access to the updated model.
 *
 * ## Animation Hooks
 *
 * Animations in ngForm are triggered when any of the associated CSS classes are added and removed.
 * These classes are: `.ng-pristine`, `.ng-dirty`, `.ng-invalid` and `.ng-valid` as well as any
 * other validations that are performed within the form. Animations in ngForm are similar to how
 * they work in ngClass and animations can be hooked into using CSS transitions, keyframes as well
 * as JS animations.
 *
 * The following example shows a simple way to utilize CSS transitions to style a form element
 * that has been rendered as invalid after it has been validated:
 *
 * <pre>
 * //be sure to include ngAnimate as a module to hook into more
 * //advanced animations
 * .my-form {
 *   transition:0.5s linear all;
 *   background: white;
 * }
 * .my-form.ng-invalid {
 *   background: red;
 *   color:white;
 * }
 * </pre>
 *
 * @example
    <example deps="angular-animate.js" animations="true" fixBase="true" module="formExample">
      <file name="index.html">
       <script>
         angular.module('formExample', [])
           .controller('FormController', ['$scope', function($scope) {
             $scope.userType = 'guest';
           }]);
       </script>
       <style>
        .my-form {
          -webkit-transition:all linear 0.5s;
          transition:all linear 0.5s;
          background: transparent;
        }
        .my-form.ng-invalid {
          background: red;
        }
       </style>
       <form name="myForm" ng-controller="FormController" class="my-form">
         userType: <input name="input" ng-model="userType" required>
         <span class="error" ng-show="myForm.input.$error.required">Required!</span><br>
         <code>userType = {{userType}}</code><br>
         <code>myForm.input.$valid = {{myForm.input.$valid}}</code><br>
         <code>myForm.input.$error = {{myForm.input.$error}}</code><br>
         <code>myForm.$valid = {{myForm.$valid}}</code><br>
         <code>myForm.$error.required = {{!!myForm.$error.required}}</code><br>
        </form>
      </file>
      <file name="protractor.js" type="protractor">
        it('should initialize to model', function() {
          var userType = element(by.binding('userType'));
          var valid = element(by.binding('myForm.input.$valid'));

          expect(userType.getText()).toContain('guest');
          expect(valid.getText()).toContain('true');
        });

        it('should be invalid if empty', function() {
          var userType = element(by.binding('userType'));
          var valid = element(by.binding('myForm.input.$valid'));
          var userInput = element(by.model('userType'));

          userInput.clear();
          userInput.sendKeys('');

          expect(userType.getText()).toEqual('userType =');
          expect(valid.getText()).toContain('false');
        });
      </file>
    </example>
 *
 * @param {string=} name Name of the form. If specified, the form controller will be published into
 *                       related scope, under this name.
 */
  var formDirectiveFactory = function (isNgForm) {
    return [
      '$timeout',
      function ($timeout) {
        var formDirective = {
            name: 'form',
            restrict: isNgForm ? 'EAC' : 'E',
            controller: FormController,
            compile: function ngFormCompile(formElement, attr) {
              // Setup initial state of the control
              formElement.addClass(PRISTINE_CLASS).addClass(VALID_CLASS);
              var nameAttr = attr.name ? 'name' : isNgForm && attr.ngForm ? 'ngForm' : false;
              return {
                pre: function ngFormPreLink(scope, formElement, attr, controller) {
                  // if `action` attr is not present on the form, prevent the default action (submission)
                  if (!('action' in attr)) {
                    // we can't use jq events because if a form is destroyed during submission the default
                    // action is not prevented. see #1238
                    //
                    // IE 9 is not affected because it doesn't fire a submit event and try to do a full
                    // page reload if the form was destroyed by submission of the form via a click handler
                    // on a button in the form. Looks like an IE9 specific bug.
                    var handleFormSubmission = function (event) {
                      scope.$apply(function () {
                        controller.$commitViewValue();
                        controller.$setSubmitted();
                      });
                      event.preventDefault();
                    };
                    addEventListenerFn(formElement[0], 'submit', handleFormSubmission);
                    // unregister the preventDefault listener so that we don't not leak memory but in a
                    // way that will achieve the prevention of the default action.
                    formElement.on('$destroy', function () {
                      $timeout(function () {
                        removeEventListenerFn(formElement[0], 'submit', handleFormSubmission);
                      }, 0, false);
                    });
                  }
                  var parentFormCtrl = controller.$$parentForm;
                  if (nameAttr) {
                    setter(scope, null, controller.$name, controller, controller.$name);
                    attr.$observe(nameAttr, function (newValue) {
                      if (controller.$name === newValue)
                        return;
                      setter(scope, null, controller.$name, undefined, controller.$name);
                      parentFormCtrl.$$renameControl(controller, newValue);
                      setter(scope, null, controller.$name, controller, controller.$name);
                    });
                  }
                  formElement.on('$destroy', function () {
                    parentFormCtrl.$removeControl(controller);
                    if (nameAttr) {
                      setter(scope, null, attr[nameAttr], undefined, controller.$name);
                    }
                    extend(controller, nullFormCtrl);  //stop propagating child destruction handlers upwards
                  });
                }
              };
            }
          };
        return formDirective;
      }
    ];
  };
  var formDirective = formDirectiveFactory();
  var ngFormDirective = formDirectiveFactory(true);
  /* global VALID_CLASS: false,
  INVALID_CLASS: false,
  PRISTINE_CLASS: false,
  DIRTY_CLASS: false,
  UNTOUCHED_CLASS: false,
  TOUCHED_CLASS: false,
  ngModelMinErr: false,
*/
  // Regex code is obtained from SO: https://stackoverflow.com/questions/3143070/javascript-regex-iso-datetime#answer-3143231
  var ISO_DATE_REGEXP = /\d{4}-[01]\d-[0-3]\dT[0-2]\d:[0-5]\d:[0-5]\d\.\d+([+-][0-2]\d:[0-5]\d|Z)/;
  var URL_REGEXP = /^(ftp|http|https):\/\/(\w+:{0,1}\w*@)?(\S+)(:[0-9]+)?(\/|\/([\w#!:.?+=&%@!\-\/]))?$/;
  var EMAIL_REGEXP = /^[a-z0-9!#$%&'*+\/=?^_`{|}~.-]+@[a-z0-9]([a-z0-9-]*[a-z0-9])?(\.[a-z0-9]([a-z0-9-]*[a-z0-9])?)*$/i;
  var NUMBER_REGEXP = /^\s*(\-|\+)?(\d+|(\d*(\.\d*)))\s*$/;
  var DATE_REGEXP = /^(\d{4})-(\d{2})-(\d{2})$/;
  var DATETIMELOCAL_REGEXP = /^(\d{4})-(\d\d)-(\d\d)T(\d\d):(\d\d)(?::(\d\d)(\.\d{1,3})?)?$/;
  var WEEK_REGEXP = /^(\d{4})-W(\d\d)$/;
  var MONTH_REGEXP = /^(\d{4})-(\d\d)$/;
  var TIME_REGEXP = /^(\d\d):(\d\d)(?::(\d\d)(\.\d{1,3})?)?$/;
  var inputType = {
      'text': textInputType,
      'date': createDateInputType('date', DATE_REGEXP, createDateParser(DATE_REGEXP, [
        'yyyy',
        'MM',
        'dd'
      ]), 'yyyy-MM-dd'),
      'datetime-local': createDateInputType('datetimelocal', DATETIMELOCAL_REGEXP, createDateParser(DATETIMELOCAL_REGEXP, [
        'yyyy',
        'MM',
        'dd',
        'HH',
        'mm',
        'ss',
        'sss'
      ]), 'yyyy-MM-ddTHH:mm:ss.sss'),
      'time': createDateInputType('time', TIME_REGEXP, createDateParser(TIME_REGEXP, [
        'HH',
        'mm',
        'ss',
        'sss'
      ]), 'HH:mm:ss.sss'),
      'week': createDateInputType('week', WEEK_REGEXP, weekParser, 'yyyy-Www'),
      'month': createDateInputType('month', MONTH_REGEXP, createDateParser(MONTH_REGEXP, [
        'yyyy',
        'MM'
      ]), 'yyyy-MM'),
      'number': numberInputType,
      'url': urlInputType,
      'email': emailInputType,
      'radio': radioInputType,
      'checkbox': checkboxInputType,
      'hidden': noop,
      'button': noop,
      'submit': noop,
      'reset': noop,
      'file': noop
    };
  function stringBasedInputType(ctrl) {
    ctrl.$formatters.push(function (value) {
      return ctrl.$isEmpty(value) ? value : value.toString();
    });
  }
  function textInputType(scope, element, attr, ctrl, $sniffer, $browser) {
    baseInputType(scope, element, attr, ctrl, $sniffer, $browser);
    stringBasedInputType(ctrl);
  }
  function baseInputType(scope, element, attr, ctrl, $sniffer, $browser) {
    var type = lowercase(element[0].type);
    // In composition mode, users are still inputing intermediate text buffer,
    // hold the listener until composition is done.
    // More about composition events: https://developer.mozilla.org/en-US/docs/Web/API/CompositionEvent
    if (!$sniffer.android) {
      var composing = false;
      element.on('compositionstart', function (data) {
        composing = true;
      });
      element.on('compositionend', function () {
        composing = false;
        listener();
      });
    }
    var listener = function (ev) {
      if (timeout) {
        $browser.defer.cancel(timeout);
        timeout = null;
      }
      if (composing)
        return;
      var value = element.val(), event = ev && ev.type;
      // By default we will trim the value
      // If the attribute ng-trim exists we will avoid trimming
      // If input type is 'password', the value is never trimmed
      if (type !== 'password' && (!attr.ngTrim || attr.ngTrim !== 'false')) {
        value = trim(value);
      }
      // If a control is suffering from bad input (due to native validators), browsers discard its
      // value, so it may be necessary to revalidate (by calling $setViewValue again) even if the
      // control's value is the same empty value twice in a row.
      if (ctrl.$viewValue !== value || value === '' && ctrl.$$hasNativeValidators) {
        ctrl.$setViewValue(value, event);
      }
    };
    // if the browser does support "input" event, we are fine - except on IE9 which doesn't fire the
    // input event on backspace, delete or cut
    if ($sniffer.hasEvent('input')) {
      element.on('input', listener);
    } else {
      var timeout;
      var deferListener = function (ev, input, origValue) {
        if (!timeout) {
          timeout = $browser.defer(function () {
            timeout = null;
            if (!input || input.value !== origValue) {
              listener(ev);
            }
          });
        }
      };
      element.on('keydown', function (event) {
        var key = event.keyCode;
        // ignore
        //    command            modifiers                   arrows
        if (key === 91 || 15 < key && key < 19 || 37 <= key && key <= 40)
          return;
        deferListener(event, this, this.value);
      });
      // if user modifies input value using context menu in IE, we need "paste" and "cut" events to catch it
      if ($sniffer.hasEvent('paste')) {
        element.on('paste cut', deferListener);
      }
    }
    // if user paste into input using mouse on older browser
    // or form autocomplete on newer browser, we need "change" event to catch it
    element.on('change', listener);
    ctrl.$render = function () {
      element.val(ctrl.$isEmpty(ctrl.$viewValue) ? '' : ctrl.$viewValue);
    };
  }
  function weekParser(isoWeek, existingDate) {
    if (isDate(isoWeek)) {
      return isoWeek;
    }
    if (isString(isoWeek)) {
      WEEK_REGEXP.lastIndex = 0;
      var parts = WEEK_REGEXP.exec(isoWeek);
      if (parts) {
        var year = +parts[1], week = +parts[2], hours = 0, minutes = 0, seconds = 0, milliseconds = 0, firstThurs = getFirstThursdayOfYear(year), addDays = (week - 1) * 7;
        if (existingDate) {
          hours = existingDate.getHours();
          minutes = existingDate.getMinutes();
          seconds = existingDate.getSeconds();
          milliseconds = existingDate.getMilliseconds();
        }
        return new Date(year, 0, firstThurs.getDate() + addDays, hours, minutes, seconds, milliseconds);
      }
    }
    return NaN;
  }
  function createDateParser(regexp, mapping) {
    return function (iso, date) {
      var parts, map;
      if (isDate(iso)) {
        return iso;
      }
      if (isString(iso)) {
        // When a date is JSON'ified to wraps itself inside of an extra
        // set of double quotes. This makes the date parsing code unable
        // to match the date string and parse it as a date.
        if (iso.charAt(0) == '"' && iso.charAt(iso.length - 1) == '"') {
          iso = iso.substring(1, iso.length - 1);
        }
        if (ISO_DATE_REGEXP.test(iso)) {
          return new Date(iso);
        }
        regexp.lastIndex = 0;
        parts = regexp.exec(iso);
        if (parts) {
          parts.shift();
          if (date) {
            map = {
              yyyy: date.getFullYear(),
              MM: date.getMonth() + 1,
              dd: date.getDate(),
              HH: date.getHours(),
              mm: date.getMinutes(),
              ss: date.getSeconds(),
              sss: date.getMilliseconds() / 1000
            };
          } else {
            map = {
              yyyy: 1970,
              MM: 1,
              dd: 1,
              HH: 0,
              mm: 0,
              ss: 0,
              sss: 0
            };
          }
          forEach(parts, function (part, index) {
            if (index < mapping.length) {
              map[mapping[index]] = +part;
            }
          });
          return new Date(map.yyyy, map.MM - 1, map.dd, map.HH, map.mm, map.ss || 0, map.sss * 1000 || 0);
        }
      }
      return NaN;
    };
  }
  function createDateInputType(type, regexp, parseDate, format) {
    return function dynamicDateInputType(scope, element, attr, ctrl, $sniffer, $browser, $filter) {
      badInputChecker(scope, element, attr, ctrl);
      baseInputType(scope, element, attr, ctrl, $sniffer, $browser);
      var timezone = ctrl && ctrl.$options && ctrl.$options.timezone;
      var previousDate;
      ctrl.$$parserName = type;
      ctrl.$parsers.push(function (value) {
        if (ctrl.$isEmpty(value))
          return null;
        if (regexp.test(value)) {
          // Note: We cannot read ctrl.$modelValue, as there might be a different
          // parser/formatter in the processing chain so that the model
          // contains some different data format!
          var parsedDate = parseDate(value, previousDate);
          if (timezone === 'UTC') {
            parsedDate.setMinutes(parsedDate.getMinutes() - parsedDate.getTimezoneOffset());
          }
          return parsedDate;
        }
        return undefined;
      });
      ctrl.$formatters.push(function (value) {
        if (value && !isDate(value)) {
          throw ngModelMinErr('datefmt', 'Expected `{0}` to be a date', value);
        }
        if (isValidDate(value)) {
          previousDate = value;
          if (previousDate && timezone === 'UTC') {
            var timezoneOffset = 60000 * previousDate.getTimezoneOffset();
            previousDate = new Date(previousDate.getTime() + timezoneOffset);
          }
          return $filter('date')(value, format, timezone);
        } else {
          previousDate = null;
          return '';
        }
      });
      if (isDefined(attr.min) || attr.ngMin) {
        var minVal;
        ctrl.$validators.min = function (value) {
          return !isValidDate(value) || isUndefined(minVal) || parseDate(value) >= minVal;
        };
        attr.$observe('min', function (val) {
          minVal = parseObservedDateValue(val);
          ctrl.$validate();
        });
      }
      if (isDefined(attr.max) || attr.ngMax) {
        var maxVal;
        ctrl.$validators.max = function (value) {
          return !isValidDate(value) || isUndefined(maxVal) || parseDate(value) <= maxVal;
        };
        attr.$observe('max', function (val) {
          maxVal = parseObservedDateValue(val);
          ctrl.$validate();
        });
      }
      function isValidDate(value) {
        // Invalid Date: getTime() returns NaN
        return value && !(value.getTime && value.getTime() !== value.getTime());
      }
      function parseObservedDateValue(val) {
        return isDefined(val) ? isDate(val) ? val : parseDate(val) : undefined;
      }
    };
  }
  function badInputChecker(scope, element, attr, ctrl) {
    var node = element[0];
    var nativeValidation = ctrl.$$hasNativeValidators = isObject(node.validity);
    if (nativeValidation) {
      ctrl.$parsers.push(function (value) {
        var validity = element.prop(VALIDITY_STATE_PROPERTY) || {};
        // Detect bug in FF35 for input[email] (https://bugzilla.mozilla.org/show_bug.cgi?id=1064430):
        // - also sets validity.badInput (should only be validity.typeMismatch).
        // - see http://www.whatwg.org/specs/web-apps/current-work/multipage/forms.html#e-mail-state-(type=email)
        // - can ignore this case as we can still read out the erroneous email...
        return validity.badInput && !validity.typeMismatch ? undefined : value;
      });
    }
  }
  function numberInputType(scope, element, attr, ctrl, $sniffer, $browser) {
    badInputChecker(scope, element, attr, ctrl);
    baseInputType(scope, element, attr, ctrl, $sniffer, $browser);
    ctrl.$$parserName = 'number';
    ctrl.$parsers.push(function (value) {
      if (ctrl.$isEmpty(value))
        return null;
      if (NUMBER_REGEXP.test(value))
        return parseFloat(value);
      return undefined;
    });
    ctrl.$formatters.push(function (value) {
      if (!ctrl.$isEmpty(value)) {
        if (!isNumber(value)) {
          throw ngModelMinErr('numfmt', 'Expected `{0}` to be a number', value);
        }
        value = value.toString();
      }
      return value;
    });
    if (isDefined(attr.min) || attr.ngMin) {
      var minVal;
      ctrl.$validators.min = function (value) {
        return ctrl.$isEmpty(value) || isUndefined(minVal) || value >= minVal;
      };
      attr.$observe('min', function (val) {
        if (isDefined(val) && !isNumber(val)) {
          val = parseFloat(val, 10);
        }
        minVal = isNumber(val) && !isNaN(val) ? val : undefined;
        // TODO(matsko): implement validateLater to reduce number of validations
        ctrl.$validate();
      });
    }
    if (isDefined(attr.max) || attr.ngMax) {
      var maxVal;
      ctrl.$validators.max = function (value) {
        return ctrl.$isEmpty(value) || isUndefined(maxVal) || value <= maxVal;
      };
      attr.$observe('max', function (val) {
        if (isDefined(val) && !isNumber(val)) {
          val = parseFloat(val, 10);
        }
        maxVal = isNumber(val) && !isNaN(val) ? val : undefined;
        // TODO(matsko): implement validateLater to reduce number of validations
        ctrl.$validate();
      });
    }
  }
  function urlInputType(scope, element, attr, ctrl, $sniffer, $browser) {
    // Note: no badInputChecker here by purpose as `url` is only a validation
    // in browsers, i.e. we can always read out input.value even if it is not valid!
    baseInputType(scope, element, attr, ctrl, $sniffer, $browser);
    stringBasedInputType(ctrl);
    ctrl.$$parserName = 'url';
    ctrl.$validators.url = function (modelValue, viewValue) {
      var value = modelValue || viewValue;
      return ctrl.$isEmpty(value) || URL_REGEXP.test(value);
    };
  }
  function emailInputType(scope, element, attr, ctrl, $sniffer, $browser) {
    // Note: no badInputChecker here by purpose as `url` is only a validation
    // in browsers, i.e. we can always read out input.value even if it is not valid!
    baseInputType(scope, element, attr, ctrl, $sniffer, $browser);
    stringBasedInputType(ctrl);
    ctrl.$$parserName = 'email';
    ctrl.$validators.email = function (modelValue, viewValue) {
      var value = modelValue || viewValue;
      return ctrl.$isEmpty(value) || EMAIL_REGEXP.test(value);
    };
  }
  function radioInputType(scope, element, attr, ctrl) {
    // make the name unique, if not defined
    if (isUndefined(attr.name)) {
      element.attr('name', nextUid());
    }
    var listener = function (ev) {
      if (element[0].checked) {
        ctrl.$setViewValue(attr.value, ev && ev.type);
      }
    };
    element.on('click', listener);
    ctrl.$render = function () {
      var value = attr.value;
      element[0].checked = value == ctrl.$viewValue;
    };
    attr.$observe('value', ctrl.$render);
  }
  function parseConstantExpr($parse, context, name, expression, fallback) {
    var parseFn;
    if (isDefined(expression)) {
      parseFn = $parse(expression);
      if (!parseFn.constant) {
        throw ngModelMinErr('constexpr', 'Expected constant expression for `{0}`, but saw ' + '`{1}`.', name, expression);
      }
      return parseFn(context);
    }
    return fallback;
  }
  function checkboxInputType(scope, element, attr, ctrl, $sniffer, $browser, $filter, $parse) {
    var trueValue = parseConstantExpr($parse, scope, 'ngTrueValue', attr.ngTrueValue, true);
    var falseValue = parseConstantExpr($parse, scope, 'ngFalseValue', attr.ngFalseValue, false);
    var listener = function (ev) {
      ctrl.$setViewValue(element[0].checked, ev && ev.type);
    };
    element.on('click', listener);
    ctrl.$render = function () {
      element[0].checked = ctrl.$viewValue;
    };
    // Override the standard `$isEmpty` because the $viewValue of an empty checkbox is always set to `false`
    // This is because of the parser below, which compares the `$modelValue` with `trueValue` to convert
    // it to a boolean.
    ctrl.$isEmpty = function (value) {
      return value === false;
    };
    ctrl.$formatters.push(function (value) {
      return equals(value, trueValue);
    });
    ctrl.$parsers.push(function (value) {
      return value ? trueValue : falseValue;
    });
  }
  /**
 * @ngdoc directive
 * @name textarea
 * @restrict E
 *
 * @description
 * HTML textarea element control with angular data-binding. The data-binding and validation
 * properties of this element are exactly the same as those of the
 * {@link ng.directive:input input element}.
 *
 * @param {string} ngModel Assignable angular expression to data-bind to.
 * @param {string=} name Property name of the form under which the control is published.
 * @param {string=} required Sets `required` validation error key if the value is not entered.
 * @param {string=} ngRequired Adds `required` attribute and `required` validation constraint to
 *    the element when the ngRequired expression evaluates to true. Use `ngRequired` instead of
 *    `required` when you want to data-bind to the `required` attribute.
 * @param {number=} ngMinlength Sets `minlength` validation error key if the value is shorter than
 *    minlength.
 * @param {number=} ngMaxlength Sets `maxlength` validation error key if the value is longer than
 *    maxlength. Setting the attribute to a negative or non-numeric value, allows view values of any
 *    length.
 * @param {string=} ngPattern Sets `pattern` validation error key if the value does not match the
 *    RegExp pattern expression. Expected value is `/regexp/` for inline patterns or `regexp` for
 *    patterns defined as scope expressions.
 * @param {string=} ngChange Angular expression to be executed when input changes due to user
 *    interaction with the input element.
 * @param {boolean=} [ngTrim=true] If set to false Angular will not automatically trim the input.
 */
  /**
 * @ngdoc directive
 * @name input
 * @restrict E
 *
 * @description
 * HTML input element control. When used together with {@link ngModel `ngModel`}, it provides data-binding,
 * input state control, and validation.
 * Input control follows HTML5 input types and polyfills the HTML5 validation behavior for older browsers.
 *
 * <div class="alert alert-warning">
 * **Note:** Not every feature offered is available for all input types.
 * Specifically, data binding and event handling via `ng-model` is unsupported for `input[file]`.
 * </div>
 *
 * @param {string} ngModel Assignable angular expression to data-bind to.
 * @param {string=} name Property name of the form under which the control is published.
 * @param {string=} required Sets `required` validation error key if the value is not entered.
 * @param {boolean=} ngRequired Sets `required` attribute if set to true
 * @param {number=} ngMinlength Sets `minlength` validation error key if the value is shorter than
 *    minlength.
 * @param {number=} ngMaxlength Sets `maxlength` validation error key if the value is longer than
 *    maxlength. Setting the attribute to a negative or non-numeric value, allows view values of any
 *    length.
 * @param {string=} ngPattern Sets `pattern` validation error key if the value does not match the
 *    RegExp pattern expression. Expected value is `/regexp/` for inline patterns or `regexp` for
 *    patterns defined as scope expressions.
 * @param {string=} ngChange Angular expression to be executed when input changes due to user
 *    interaction with the input element.
 * @param {boolean=} [ngTrim=true] If set to false Angular will not automatically trim the input.
 *    This parameter is ignored for input[type=password] controls, which will never trim the
 *    input.
 *
 * @example
    <example name="input-directive" module="inputExample">
      <file name="index.html">
       <script>
          angular.module('inputExample', [])
            .controller('ExampleController', ['$scope', function($scope) {
              $scope.user = {name: 'guest', last: 'visitor'};
            }]);
       </script>
       <div ng-controller="ExampleController">
         <form name="myForm">
           User name: <input type="text" name="userName" ng-model="user.name" required>
           <span class="error" ng-show="myForm.userName.$error.required">
             Required!</span><br>
           Last name: <input type="text" name="lastName" ng-model="user.last"
             ng-minlength="3" ng-maxlength="10">
           <span class="error" ng-show="myForm.lastName.$error.minlength">
             Too short!</span>
           <span class="error" ng-show="myForm.lastName.$error.maxlength">
             Too long!</span><br>
         </form>
         <hr>
         <tt>user = {{user}}</tt><br/>
         <tt>myForm.userName.$valid = {{myForm.userName.$valid}}</tt><br>
         <tt>myForm.userName.$error = {{myForm.userName.$error}}</tt><br>
         <tt>myForm.lastName.$valid = {{myForm.lastName.$valid}}</tt><br>
         <tt>myForm.lastName.$error = {{myForm.lastName.$error}}</tt><br>
         <tt>myForm.$valid = {{myForm.$valid}}</tt><br>
         <tt>myForm.$error.required = {{!!myForm.$error.required}}</tt><br>
         <tt>myForm.$error.minlength = {{!!myForm.$error.minlength}}</tt><br>
         <tt>myForm.$error.maxlength = {{!!myForm.$error.maxlength}}</tt><br>
       </div>
      </file>
      <file name="protractor.js" type="protractor">
        var user = element(by.exactBinding('user'));
        var userNameValid = element(by.binding('myForm.userName.$valid'));
        var lastNameValid = element(by.binding('myForm.lastName.$valid'));
        var lastNameError = element(by.binding('myForm.lastName.$error'));
        var formValid = element(by.binding('myForm.$valid'));
        var userNameInput = element(by.model('user.name'));
        var userLastInput = element(by.model('user.last'));

        it('should initialize to model', function() {
          expect(user.getText()).toContain('{"name":"guest","last":"visitor"}');
          expect(userNameValid.getText()).toContain('true');
          expect(formValid.getText()).toContain('true');
        });

        it('should be invalid if empty when required', function() {
          userNameInput.clear();
          userNameInput.sendKeys('');

          expect(user.getText()).toContain('{"last":"visitor"}');
          expect(userNameValid.getText()).toContain('false');
          expect(formValid.getText()).toContain('false');
        });

        it('should be valid if empty when min length is set', function() {
          userLastInput.clear();
          userLastInput.sendKeys('');

          expect(user.getText()).toContain('{"name":"guest","last":""}');
          expect(lastNameValid.getText()).toContain('true');
          expect(formValid.getText()).toContain('true');
        });

        it('should be invalid if less than required min length', function() {
          userLastInput.clear();
          userLastInput.sendKeys('xx');

          expect(user.getText()).toContain('{"name":"guest"}');
          expect(lastNameValid.getText()).toContain('false');
          expect(lastNameError.getText()).toContain('minlength');
          expect(formValid.getText()).toContain('false');
        });

        it('should be invalid if longer than max length', function() {
          userLastInput.clear();
          userLastInput.sendKeys('some ridiculously long name');

          expect(user.getText()).toContain('{"name":"guest"}');
          expect(lastNameValid.getText()).toContain('false');
          expect(lastNameError.getText()).toContain('maxlength');
          expect(formValid.getText()).toContain('false');
        });
      </file>
    </example>
 */
  var inputDirective = [
      '$browser',
      '$sniffer',
      '$filter',
      '$parse',
      function ($browser, $sniffer, $filter, $parse) {
        return {
          restrict: 'E',
          require: ['?ngModel'],
          link: {
            pre: function (scope, element, attr, ctrls) {
              if (ctrls[0]) {
                (inputType[lowercase(attr.type)] || inputType.text)(scope, element, attr, ctrls[0], $sniffer, $browser, $filter, $parse);
              }
            }
          }
        };
      }
    ];
  var CONSTANT_VALUE_REGEXP = /^(true|false|\d+)$/;
  /**
 * @ngdoc directive
 * @name ngValue
 *
 * @description
 * Binds the given expression to the value of `<option>` or {@link input[radio] `input[radio]`},
 * so that when the element is selected, the {@link ngModel `ngModel`} of that element is set to
 * the bound value.
 *
 * `ngValue` is useful when dynamically generating lists of radio buttons using
 * {@link ngRepeat `ngRepeat`}, as shown below.
 *
 * Likewise, `ngValue` can be used to generate `<option>` elements for
 * the {@link select `select`} element. In that case however, only strings are supported
 * for the `value `attribute, so the resulting `ngModel` will always be a string.
 * Support for `select` models with non-string values is available via `ngOptions`.
 *
 * @element input
 * @param {string=} ngValue angular expression, whose value will be bound to the `value` attribute
 *   of the `input` element
 *
 * @example
    <example name="ngValue-directive" module="valueExample">
      <file name="index.html">
       <script>
          angular.module('valueExample', [])
            .controller('ExampleController', ['$scope', function($scope) {
              $scope.names = ['pizza', 'unicorns', 'robots'];
              $scope.my = { favorite: 'unicorns' };
            }]);
       </script>
        <form ng-controller="ExampleController">
          <h2>Which is your favorite?</h2>
            <label ng-repeat="name in names" for="{{name}}">
              {{name}}
              <input type="radio"
                     ng-model="my.favorite"
                     ng-value="name"
                     id="{{name}}"
                     name="favorite">
            </label>
          <div>You chose {{my.favorite}}</div>
        </form>
      </file>
      <file name="protractor.js" type="protractor">
        var favorite = element(by.binding('my.favorite'));

        it('should initialize to model', function() {
          expect(favorite.getText()).toContain('unicorns');
        });
        it('should bind the values to the inputs', function() {
          element.all(by.model('my.favorite')).get(0).click();
          expect(favorite.getText()).toContain('pizza');
        });
      </file>
    </example>
 */
  var ngValueDirective = function () {
    return {
      restrict: 'A',
      priority: 100,
      compile: function (tpl, tplAttr) {
        if (CONSTANT_VALUE_REGEXP.test(tplAttr.ngValue)) {
          return function ngValueConstantLink(scope, elm, attr) {
            attr.$set('value', scope.$eval(attr.ngValue));
          };
        } else {
          return function ngValueLink(scope, elm, attr) {
            scope.$watch(attr.ngValue, function valueWatchAction(value) {
              attr.$set('value', value);
            });
          };
        }
      }
    };
  };
  /**
 * @ngdoc directive
 * @name ngBind
 * @restrict AC
 *
 * @description
 * The `ngBind` attribute tells Angular to replace the text content of the specified HTML element
 * with the value of a given expression, and to update the text content when the value of that
 * expression changes.
 *
 * Typically, you don't use `ngBind` directly, but instead you use the double curly markup like
 * `{{ expression }}` which is similar but less verbose.
 *
 * It is preferable to use `ngBind` instead of `{{ expression }}` if a template is momentarily
 * displayed by the browser in its raw state before Angular compiles it. Since `ngBind` is an
 * element attribute, it makes the bindings invisible to the user while the page is loading.
 *
 * An alternative solution to this problem would be using the
 * {@link ng.directive:ngCloak ngCloak} directive.
 *
 *
 * @element ANY
 * @param {expression} ngBind {@link guide/expression Expression} to evaluate.
 *
 * @example
 * Enter a name in the Live Preview text box; the greeting below the text box changes instantly.
   <example module="bindExample">
     <file name="index.html">
       <script>
         angular.module('bindExample', [])
           .controller('ExampleController', ['$scope', function($scope) {
             $scope.name = 'Whirled';
           }]);
       </script>
       <div ng-controller="ExampleController">
         Enter name: <input type="text" ng-model="name"><br>
         Hello <span ng-bind="name"></span>!
       </div>
     </file>
     <file name="protractor.js" type="protractor">
       it('should check ng-bind', function() {
         var nameInput = element(by.model('name'));

         expect(element(by.binding('name')).getText()).toBe('Whirled');
         nameInput.clear();
         nameInput.sendKeys('world');
         expect(element(by.binding('name')).getText()).toBe('world');
       });
     </file>
   </example>
 */
  var ngBindDirective = [
      '$compile',
      function ($compile) {
        return {
          restrict: 'AC',
          compile: function ngBindCompile(templateElement) {
            $compile.$$addBindingClass(templateElement);
            return function ngBindLink(scope, element, attr) {
              $compile.$$addBindingInfo(element, attr.ngBind);
              element = element[0];
              scope.$watch(attr.ngBind, function ngBindWatchAction(value) {
                element.textContent = value === undefined ? '' : value;
              });
            };
          }
        };
      }
    ];
  /**
 * @ngdoc directive
 * @name ngBindTemplate
 *
 * @description
 * The `ngBindTemplate` directive specifies that the element
 * text content should be replaced with the interpolation of the template
 * in the `ngBindTemplate` attribute.
 * Unlike `ngBind`, the `ngBindTemplate` can contain multiple `{{` `}}`
 * expressions. This directive is needed since some HTML elements
 * (such as TITLE and OPTION) cannot contain SPAN elements.
 *
 * @element ANY
 * @param {string} ngBindTemplate template of form
 *   <tt>{{</tt> <tt>expression</tt> <tt>}}</tt> to eval.
 *
 * @example
 * Try it here: enter text in text box and watch the greeting change.
   <example module="bindExample">
     <file name="index.html">
       <script>
         angular.module('bindExample', [])
           .controller('ExampleController', ['$scope', function($scope) {
             $scope.salutation = 'Hello';
             $scope.name = 'World';
           }]);
       </script>
       <div ng-controller="ExampleController">
        Salutation: <input type="text" ng-model="salutation"><br>
        Name: <input type="text" ng-model="name"><br>
        <pre ng-bind-template="{{salutation}} {{name}}!"></pre>
       </div>
     </file>
     <file name="protractor.js" type="protractor">
       it('should check ng-bind', function() {
         var salutationElem = element(by.binding('salutation'));
         var salutationInput = element(by.model('salutation'));
         var nameInput = element(by.model('name'));

         expect(salutationElem.getText()).toBe('Hello World!');

         salutationInput.clear();
         salutationInput.sendKeys('Greetings');
         nameInput.clear();
         nameInput.sendKeys('user');

         expect(salutationElem.getText()).toBe('Greetings user!');
       });
     </file>
   </example>
 */
  var ngBindTemplateDirective = [
      '$interpolate',
      '$compile',
      function ($interpolate, $compile) {
        return {
          compile: function ngBindTemplateCompile(templateElement) {
            $compile.$$addBindingClass(templateElement);
            return function ngBindTemplateLink(scope, element, attr) {
              var interpolateFn = $interpolate(element.attr(attr.$attr.ngBindTemplate));
              $compile.$$addBindingInfo(element, interpolateFn.expressions);
              element = element[0];
              attr.$observe('ngBindTemplate', function (value) {
                element.textContent = value === undefined ? '' : value;
              });
            };
          }
        };
      }
    ];
  /**
 * @ngdoc directive
 * @name ngBindHtml
 *
 * @description
 * Evaluates the expression and inserts the resulting HTML into the element in a secure way. By default,
 * the resulting HTML content will be sanitized using the {@link ngSanitize.$sanitize $sanitize} service.
 * To utilize this functionality, ensure that `$sanitize` is available, for example, by including {@link
 * ngSanitize} in your module's dependencies (not in core Angular). In order to use {@link ngSanitize}
 * in your module's dependencies, you need to include "angular-sanitize.js" in your application.
 *
 * You may also bypass sanitization for values you know are safe. To do so, bind to
 * an explicitly trusted value via {@link ng.$sce#trustAsHtml $sce.trustAsHtml}.  See the example
 * under {@link ng.$sce#show-me-an-example-using-sce- Strict Contextual Escaping (SCE)}.
 *
 * Note: If a `$sanitize` service is unavailable and the bound value isn't explicitly trusted, you
 * will have an exception (instead of an exploit.)
 *
 * @element ANY
 * @param {expression} ngBindHtml {@link guide/expression Expression} to evaluate.
 *
 * @example

   <example module="bindHtmlExample" deps="angular-sanitize.js">
     <file name="index.html">
       <div ng-controller="ExampleController">
        <p ng-bind-html="myHTML"></p>
       </div>
     </file>

     <file name="script.js">
       angular.module('bindHtmlExample', ['ngSanitize'])
         .controller('ExampleController', ['$scope', function($scope) {
           $scope.myHTML =
              'I am an <code>HTML</code>string with ' +
              '<a href="#">links!</a> and other <em>stuff</em>';
         }]);
     </file>

     <file name="protractor.js" type="protractor">
       it('should check ng-bind-html', function() {
         expect(element(by.binding('myHTML')).getText()).toBe(
             'I am an HTMLstring with links! and other stuff');
       });
     </file>
   </example>
 */
  var ngBindHtmlDirective = [
      '$sce',
      '$parse',
      '$compile',
      function ($sce, $parse, $compile) {
        return {
          restrict: 'A',
          compile: function ngBindHtmlCompile(tElement, tAttrs) {
            var ngBindHtmlGetter = $parse(tAttrs.ngBindHtml);
            var ngBindHtmlWatch = $parse(tAttrs.ngBindHtml, function getStringValue(value) {
                return (value || '').toString();
              });
            $compile.$$addBindingClass(tElement);
            return function ngBindHtmlLink(scope, element, attr) {
              $compile.$$addBindingInfo(element, attr.ngBindHtml);
              scope.$watch(ngBindHtmlWatch, function ngBindHtmlWatchAction() {
                // we re-evaluate the expr because we want a TrustedValueHolderType
                // for $sce, not a string
                element.html($sce.getTrustedHtml(ngBindHtmlGetter(scope)) || '');
              });
            };
          }
        };
      }
    ];
  /**
 * @ngdoc directive
 * @name ngChange
 *
 * @description
 * Evaluate the given expression when the user changes the input.
 * The expression is evaluated immediately, unlike the JavaScript onchange event
 * which only triggers at the end of a change (usually, when the user leaves the
 * form element or presses the return key).
 *
 * The `ngChange` expression is only evaluated when a change in the input value causes
 * a new value to be committed to the model.
 *
 * It will not be evaluated:
 * * if the value returned from the `$parsers` transformation pipeline has not changed
 * * if the input has continued to be invalid since the model will stay `null`
 * * if the model is changed programmatically and not by a change to the input value
 *
 *
 * Note, this directive requires `ngModel` to be present.
 *
 * @element input
 * @param {expression} ngChange {@link guide/expression Expression} to evaluate upon change
 * in input value.
 *
 * @example
 * <example name="ngChange-directive" module="changeExample">
 *   <file name="index.html">
 *     <script>
 *       angular.module('changeExample', [])
 *         .controller('ExampleController', ['$scope', function($scope) {
 *           $scope.counter = 0;
 *           $scope.change = function() {
 *             $scope.counter++;
 *           };
 *         }]);
 *     </script>
 *     <div ng-controller="ExampleController">
 *       <input type="checkbox" ng-model="confirmed" ng-change="change()" id="ng-change-example1" />
 *       <input type="checkbox" ng-model="confirmed" id="ng-change-example2" />
 *       <label for="ng-change-example2">Confirmed</label><br />
 *       <tt>debug = {{confirmed}}</tt><br/>
 *       <tt>counter = {{counter}}</tt><br/>
 *     </div>
 *   </file>
 *   <file name="protractor.js" type="protractor">
 *     var counter = element(by.binding('counter'));
 *     var debug = element(by.binding('confirmed'));
 *
 *     it('should evaluate the expression if changing from view', function() {
 *       expect(counter.getText()).toContain('0');
 *
 *       element(by.id('ng-change-example1')).click();
 *
 *       expect(counter.getText()).toContain('1');
 *       expect(debug.getText()).toContain('true');
 *     });
 *
 *     it('should not evaluate the expression if changing from model', function() {
 *       element(by.id('ng-change-example2')).click();

 *       expect(counter.getText()).toContain('0');
 *       expect(debug.getText()).toContain('true');
 *     });
 *   </file>
 * </example>
 */
  var ngChangeDirective = valueFn({
      restrict: 'A',
      require: 'ngModel',
      link: function (scope, element, attr, ctrl) {
        ctrl.$viewChangeListeners.push(function () {
          scope.$eval(attr.ngChange);
        });
      }
    });
  function classDirective(name, selector) {
    name = 'ngClass' + name;
    return [
      '$animate',
      function ($animate) {
        return {
          restrict: 'AC',
          link: function (scope, element, attr) {
            var oldVal;
            scope.$watch(attr[name], ngClassWatchAction, true);
            attr.$observe('class', function (value) {
              ngClassWatchAction(scope.$eval(attr[name]));
            });
            if (name !== 'ngClass') {
              scope.$watch('$index', function ($index, old$index) {
                // jshint bitwise: false
                var mod = $index & 1;
                if (mod !== (old$index & 1)) {
                  var classes = arrayClasses(scope.$eval(attr[name]));
                  mod === selector ? addClasses(classes) : removeClasses(classes);
                }
              });
            }
            function addClasses(classes) {
              var newClasses = digestClassCounts(classes, 1);
              attr.$addClass(newClasses);
            }
            function removeClasses(classes) {
              var newClasses = digestClassCounts(classes, -1);
              attr.$removeClass(newClasses);
            }
            function digestClassCounts(classes, count) {
              var classCounts = element.data('$classCounts') || {};
              var classesToUpdate = [];
              forEach(classes, function (className) {
                if (count > 0 || classCounts[className]) {
                  classCounts[className] = (classCounts[className] || 0) + count;
                  if (classCounts[className] === +(count > 0)) {
                    classesToUpdate.push(className);
                  }
                }
              });
              element.data('$classCounts', classCounts);
              return classesToUpdate.join(' ');
            }
            function updateClasses(oldClasses, newClasses) {
              var toAdd = arrayDifference(newClasses, oldClasses);
              var toRemove = arrayDifference(oldClasses, newClasses);
              toAdd = digestClassCounts(toAdd, 1);
              toRemove = digestClassCounts(toRemove, -1);
              if (toAdd && toAdd.length) {
                $animate.addClass(element, toAdd);
              }
              if (toRemove && toRemove.length) {
                $animate.removeClass(element, toRemove);
              }
            }
            function ngClassWatchAction(newVal) {
              if (selector === true || scope.$index % 2 === selector) {
                var newClasses = arrayClasses(newVal || []);
                if (!oldVal) {
                  addClasses(newClasses);
                } else if (!equals(newVal, oldVal)) {
                  var oldClasses = arrayClasses(oldVal);
                  updateClasses(oldClasses, newClasses);
                }
              }
              oldVal = shallowCopy(newVal);
            }
          }
        };
        function arrayDifference(tokens1, tokens2) {
          var values = [];
          outer:
            for (var i = 0; i < tokens1.length; i++) {
              var token = tokens1[i];
              for (var j = 0; j < tokens2.length; j++) {
                if (token == tokens2[j])
                  continue outer;
              }
              values.push(token);
            }
          return values;
        }
        function arrayClasses(classVal) {
          if (isArray(classVal)) {
            return classVal;
          } else if (isString(classVal)) {
            return classVal.split(' ');
          } else if (isObject(classVal)) {
            var classes = [];
            forEach(classVal, function (v, k) {
              if (v) {
                classes = classes.concat(k.split(' '));
              }
            });
            return classes;
          }
          return classVal;
        }
      }
    ];
  }
  /**
 * @ngdoc directive
 * @name ngClass
 * @restrict AC
 *
 * @description
 * The `ngClass` directive allows you to dynamically set CSS classes on an HTML element by databinding
 * an expression that represents all classes to be added.
 *
 * The directive operates in three different ways, depending on which of three types the expression
 * evaluates to:
 *
 * 1. If the expression evaluates to a string, the string should be one or more space-delimited class
 * names.
 *
 * 2. If the expression evaluates to an array, each element of the array should be a string that is
 * one or more space-delimited class names.
 *
 * 3. If the expression evaluates to an object, then for each key-value pair of the
 * object with a truthy value the corresponding key is used as a class name.
 *
 * The directive won't add duplicate classes if a particular class was already set.
 *
 * When the expression changes, the previously added classes are removed and only then the
 * new classes are added.
 *
 * @animations
 * **add** - happens just before the class is applied to the elements
 *
 * **remove** - happens just before the class is removed from the element
 *
 * @element ANY
 * @param {expression} ngClass {@link guide/expression Expression} to eval. The result
 *   of the evaluation can be a string representing space delimited class
 *   names, an array, or a map of class names to boolean values. In the case of a map, the
 *   names of the properties whose values are truthy will be added as css classes to the
 *   element.
 *
 * @example Example that demonstrates basic bindings via ngClass directive.
   <example>
     <file name="index.html">
       <p ng-class="{strike: deleted, bold: important, red: error}">Map Syntax Example</p>
       <input type="checkbox" ng-model="deleted"> deleted (apply "strike" class)<br>
       <input type="checkbox" ng-model="important"> important (apply "bold" class)<br>
       <input type="checkbox" ng-model="error"> error (apply "red" class)
       <hr>
       <p ng-class="style">Using String Syntax</p>
       <input type="text" ng-model="style" placeholder="Type: bold strike red">
       <hr>
       <p ng-class="[style1, style2, style3]">Using Array Syntax</p>
       <input ng-model="style1" placeholder="Type: bold, strike or red"><br>
       <input ng-model="style2" placeholder="Type: bold, strike or red"><br>
       <input ng-model="style3" placeholder="Type: bold, strike or red"><br>
     </file>
     <file name="style.css">
       .strike {
         text-decoration: line-through;
       }
       .bold {
           font-weight: bold;
       }
       .red {
           color: red;
       }
     </file>
     <file name="protractor.js" type="protractor">
       var ps = element.all(by.css('p'));

       it('should let you toggle the class', function() {

         expect(ps.first().getAttribute('class')).not.toMatch(/bold/);
         expect(ps.first().getAttribute('class')).not.toMatch(/red/);

         element(by.model('important')).click();
         expect(ps.first().getAttribute('class')).toMatch(/bold/);

         element(by.model('error')).click();
         expect(ps.first().getAttribute('class')).toMatch(/red/);
       });

       it('should let you toggle string example', function() {
         expect(ps.get(1).getAttribute('class')).toBe('');
         element(by.model('style')).clear();
         element(by.model('style')).sendKeys('red');
         expect(ps.get(1).getAttribute('class')).toBe('red');
       });

       it('array example should have 3 classes', function() {
         expect(ps.last().getAttribute('class')).toBe('');
         element(by.model('style1')).sendKeys('bold');
         element(by.model('style2')).sendKeys('strike');
         element(by.model('style3')).sendKeys('red');
         expect(ps.last().getAttribute('class')).toBe('bold strike red');
       });
     </file>
   </example>

   ## Animations

   The example below demonstrates how to perform animations using ngClass.

   <example module="ngAnimate" deps="angular-animate.js" animations="true">
     <file name="index.html">
      <input id="setbtn" type="button" value="set" ng-click="myVar='my-class'">
      <input id="clearbtn" type="button" value="clear" ng-click="myVar=''">
      <br>
      <span class="base-class" ng-class="myVar">Sample Text</span>
     </file>
     <file name="style.css">
       .base-class {
         -webkit-transition:all cubic-bezier(0.250, 0.460, 0.450, 0.940) 0.5s;
         transition:all cubic-bezier(0.250, 0.460, 0.450, 0.940) 0.5s;
       }

       .base-class.my-class {
         color: red;
         font-size:3em;
       }
     </file>
     <file name="protractor.js" type="protractor">
       it('should check ng-class', function() {
         expect(element(by.css('.base-class')).getAttribute('class')).not.
           toMatch(/my-class/);

         element(by.id('setbtn')).click();

         expect(element(by.css('.base-class')).getAttribute('class')).
           toMatch(/my-class/);

         element(by.id('clearbtn')).click();

         expect(element(by.css('.base-class')).getAttribute('class')).not.
           toMatch(/my-class/);
       });
     </file>
   </example>


   ## ngClass and pre-existing CSS3 Transitions/Animations
   The ngClass directive still supports CSS3 Transitions/Animations even if they do not follow the ngAnimate CSS naming structure.
   Upon animation ngAnimate will apply supplementary CSS classes to track the start and end of an animation, but this will not hinder
   any pre-existing CSS transitions already on the element. To get an idea of what happens during a class-based animation, be sure
   to view the step by step details of {@link ng.$animate#addClass $animate.addClass} and
   {@link ng.$animate#removeClass $animate.removeClass}.
 */
  var ngClassDirective = classDirective('', true);
  /**
 * @ngdoc directive
 * @name ngClassOdd
 * @restrict AC
 *
 * @description
 * The `ngClassOdd` and `ngClassEven` directives work exactly as
 * {@link ng.directive:ngClass ngClass}, except they work in
 * conjunction with `ngRepeat` and take effect only on odd (even) rows.
 *
 * This directive can be applied only within the scope of an
 * {@link ng.directive:ngRepeat ngRepeat}.
 *
 * @element ANY
 * @param {expression} ngClassOdd {@link guide/expression Expression} to eval. The result
 *   of the evaluation can be a string representing space delimited class names or an array.
 *
 * @example
   <example>
     <file name="index.html">
        <ol ng-init="names=['John', 'Mary', 'Cate', 'Suz']">
          <li ng-repeat="name in names">
           <span ng-class-odd="'odd'" ng-class-even="'even'">
             {{name}}
           </span>
          </li>
        </ol>
     </file>
     <file name="style.css">
       .odd {
         color: red;
       }
       .even {
         color: blue;
       }
     </file>
     <file name="protractor.js" type="protractor">
       it('should check ng-class-odd and ng-class-even', function() {
         expect(element(by.repeater('name in names').row(0).column('name')).getAttribute('class')).
           toMatch(/odd/);
         expect(element(by.repeater('name in names').row(1).column('name')).getAttribute('class')).
           toMatch(/even/);
       });
     </file>
   </example>
 */
  var ngClassOddDirective = classDirective('Odd', 0);
  /**
 * @ngdoc directive
 * @name ngClassEven
 * @restrict AC
 *
 * @description
 * The `ngClassOdd` and `ngClassEven` directives work exactly as
 * {@link ng.directive:ngClass ngClass}, except they work in
 * conjunction with `ngRepeat` and take effect only on odd (even) rows.
 *
 * This directive can be applied only within the scope of an
 * {@link ng.directive:ngRepeat ngRepeat}.
 *
 * @element ANY
 * @param {expression} ngClassEven {@link guide/expression Expression} to eval. The
 *   result of the evaluation can be a string representing space delimited class names or an array.
 *
 * @example
   <example>
     <file name="index.html">
        <ol ng-init="names=['John', 'Mary', 'Cate', 'Suz']">
          <li ng-repeat="name in names">
           <span ng-class-odd="'odd'" ng-class-even="'even'">
             {{name}} &nbsp; &nbsp; &nbsp;
           </span>
          </li>
        </ol>
     </file>
     <file name="style.css">
       .odd {
         color: red;
       }
       .even {
         color: blue;
       }
     </file>
     <file name="protractor.js" type="protractor">
       it('should check ng-class-odd and ng-class-even', function() {
         expect(element(by.repeater('name in names').row(0).column('name')).getAttribute('class')).
           toMatch(/odd/);
         expect(element(by.repeater('name in names').row(1).column('name')).getAttribute('class')).
           toMatch(/even/);
       });
     </file>
   </example>
 */
  var ngClassEvenDirective = classDirective('Even', 1);
  /**
 * @ngdoc directive
 * @name ngCloak
 * @restrict AC
 *
 * @description
 * The `ngCloak` directive is used to prevent the Angular html template from being briefly
 * displayed by the browser in its raw (uncompiled) form while your application is loading. Use this
 * directive to avoid the undesirable flicker effect caused by the html template display.
 *
 * The directive can be applied to the `<body>` element, but the preferred usage is to apply
 * multiple `ngCloak` directives to small portions of the page to permit progressive rendering
 * of the browser view.
 *
 * `ngCloak` works in cooperation with the following css rule embedded within `angular.js` and
 * `angular.min.js`.
 * For CSP mode please add `angular-csp.css` to your html file (see {@link ng.directive:ngCsp ngCsp}).
 *
 * ```css
 * [ng\:cloak], [ng-cloak], [data-ng-cloak], [x-ng-cloak], .ng-cloak, .x-ng-cloak {
 *   display: none !important;
 * }
 * ```
 *
 * When this css rule is loaded by the browser, all html elements (including their children) that
 * are tagged with the `ngCloak` directive are hidden. When Angular encounters this directive
 * during the compilation of the template it deletes the `ngCloak` element attribute, making
 * the compiled element visible.
 *
 * For the best result, the `angular.js` script must be loaded in the head section of the html
 * document; alternatively, the css rule above must be included in the external stylesheet of the
 * application.
 *
 * @element ANY
 *
 * @example
   <example>
     <file name="index.html">
        <div id="template1" ng-cloak>{{ 'hello' }}</div>
        <div id="template2" class="ng-cloak">{{ 'world' }}</div>
     </file>
     <file name="protractor.js" type="protractor">
       it('should remove the template directive and css class', function() {
         expect($('#template1').getAttribute('ng-cloak')).
           toBeNull();
         expect($('#template2').getAttribute('ng-cloak')).
           toBeNull();
       });
     </file>
   </example>
 *
 */
  var ngCloakDirective = ngDirective({
      compile: function (element, attr) {
        attr.$set('ngCloak', undefined);
        element.removeClass('ng-cloak');
      }
    });
  /**
 * @ngdoc directive
 * @name ngController
 *
 * @description
 * The `ngController` directive attaches a controller class to the view. This is a key aspect of how angular
 * supports the principles behind the Model-View-Controller design pattern.
 *
 * MVC components in angular:
 *
 * * Model — Models are the properties of a scope; scopes are attached to the DOM where scope properties
 *   are accessed through bindings.
 * * View — The template (HTML with data bindings) that is rendered into the View.
 * * Controller — The `ngController` directive specifies a Controller class; the class contains business
 *   logic behind the application to decorate the scope with functions and values
 *
 * Note that you can also attach controllers to the DOM by declaring it in a route definition
 * via the {@link ngRoute.$route $route} service. A common mistake is to declare the controller
 * again using `ng-controller` in the template itself.  This will cause the controller to be attached
 * and executed twice.
 *
 * @element ANY
 * @scope
 * @priority 500
 * @param {expression} ngController Name of a constructor function registered with the current
 * {@link ng.$controllerProvider $controllerProvider} or an {@link guide/expression expression}
 * that on the current scope evaluates to a constructor function.
 *
 * The controller instance can be published into a scope property by specifying
 * `ng-controller="as propertyName"`.
 *
 * If the current `$controllerProvider` is configured to use globals (via
 * {@link ng.$controllerProvider#allowGlobals `$controllerProvider.allowGlobals()` }), this may
 * also be the name of a globally accessible constructor function (not recommended).
 *
 * @example
 * Here is a simple form for editing user contact information. Adding, removing, clearing, and
 * greeting are methods declared on the controller (see source tab). These methods can
 * easily be called from the angular markup. Any changes to the data are automatically reflected
 * in the View without the need for a manual update.
 *
 * Two different declaration styles are included below:
 *
 * * one binds methods and properties directly onto the controller using `this`:
 * `ng-controller="SettingsController1 as settings"`
 * * one injects `$scope` into the controller:
 * `ng-controller="SettingsController2"`
 *
 * The second option is more common in the Angular community, and is generally used in boilerplates
 * and in this guide. However, there are advantages to binding properties directly to the controller
 * and avoiding scope.
 *
 * * Using `controller as` makes it obvious which controller you are accessing in the template when
 * multiple controllers apply to an element.
 * * If you are writing your controllers as classes you have easier access to the properties and
 * methods, which will appear on the scope, from inside the controller code.
 * * Since there is always a `.` in the bindings, you don't have to worry about prototypal
 * inheritance masking primitives.
 *
 * This example demonstrates the `controller as` syntax.
 *
 * <example name="ngControllerAs" module="controllerAsExample">
 *   <file name="index.html">
 *    <div id="ctrl-as-exmpl" ng-controller="SettingsController1 as settings">
 *      Name: <input type="text" ng-model="settings.name"/>
 *      [ <a href="" ng-click="settings.greet()">greet</a> ]<br/>
 *      Contact:
 *      <ul>
 *        <li ng-repeat="contact in settings.contacts">
 *          <select ng-model="contact.type">
 *             <option>phone</option>
 *             <option>email</option>
 *          </select>
 *          <input type="text" ng-model="contact.value"/>
 *          [ <a href="" ng-click="settings.clearContact(contact)">clear</a>
 *          | <a href="" ng-click="settings.removeContact(contact)">X</a> ]
 *        </li>
 *        <li>[ <a href="" ng-click="settings.addContact()">add</a> ]</li>
 *     </ul>
 *    </div>
 *   </file>
 *   <file name="app.js">
 *    angular.module('controllerAsExample', [])
 *      .controller('SettingsController1', SettingsController1);
 *
 *    function SettingsController1() {
 *      this.name = "John Smith";
 *      this.contacts = [
 *        {type: 'phone', value: '408 555 1212'},
 *        {type: 'email', value: 'john.smith@example.org'} ];
 *    }
 *
 *    SettingsController1.prototype.greet = function() {
 *      alert(this.name);
 *    };
 *
 *    SettingsController1.prototype.addContact = function() {
 *      this.contacts.push({type: 'email', value: 'yourname@example.org'});
 *    };
 *
 *    SettingsController1.prototype.removeContact = function(contactToRemove) {
 *     var index = this.contacts.indexOf(contactToRemove);
 *      this.contacts.splice(index, 1);
 *    };
 *
 *    SettingsController1.prototype.clearContact = function(contact) {
 *      contact.type = 'phone';
 *      contact.value = '';
 *    };
 *   </file>
 *   <file name="protractor.js" type="protractor">
 *     it('should check controller as', function() {
 *       var container = element(by.id('ctrl-as-exmpl'));
 *         expect(container.element(by.model('settings.name'))
 *           .getAttribute('value')).toBe('John Smith');
 *
 *       var firstRepeat =
 *           container.element(by.repeater('contact in settings.contacts').row(0));
 *       var secondRepeat =
 *           container.element(by.repeater('contact in settings.contacts').row(1));
 *
 *       expect(firstRepeat.element(by.model('contact.value')).getAttribute('value'))
 *           .toBe('408 555 1212');
 *
 *       expect(secondRepeat.element(by.model('contact.value')).getAttribute('value'))
 *           .toBe('john.smith@example.org');
 *
 *       firstRepeat.element(by.linkText('clear')).click();
 *
 *       expect(firstRepeat.element(by.model('contact.value')).getAttribute('value'))
 *           .toBe('');
 *
 *       container.element(by.linkText('add')).click();
 *
 *       expect(container.element(by.repeater('contact in settings.contacts').row(2))
 *           .element(by.model('contact.value'))
 *           .getAttribute('value'))
 *           .toBe('yourname@example.org');
 *     });
 *   </file>
 * </example>
 *
 * This example demonstrates the "attach to `$scope`" style of controller.
 *
 * <example name="ngController" module="controllerExample">
 *  <file name="index.html">
 *   <div id="ctrl-exmpl" ng-controller="SettingsController2">
 *     Name: <input type="text" ng-model="name"/>
 *     [ <a href="" ng-click="greet()">greet</a> ]<br/>
 *     Contact:
 *     <ul>
 *       <li ng-repeat="contact in contacts">
 *         <select ng-model="contact.type">
 *            <option>phone</option>
 *            <option>email</option>
 *         </select>
 *         <input type="text" ng-model="contact.value"/>
 *         [ <a href="" ng-click="clearContact(contact)">clear</a>
 *         | <a href="" ng-click="removeContact(contact)">X</a> ]
 *       </li>
 *       <li>[ <a href="" ng-click="addContact()">add</a> ]</li>
 *    </ul>
 *   </div>
 *  </file>
 *  <file name="app.js">
 *   angular.module('controllerExample', [])
 *     .controller('SettingsController2', ['$scope', SettingsController2]);
 *
 *   function SettingsController2($scope) {
 *     $scope.name = "John Smith";
 *     $scope.contacts = [
 *       {type:'phone', value:'408 555 1212'},
 *       {type:'email', value:'john.smith@example.org'} ];
 *
 *     $scope.greet = function() {
 *       alert($scope.name);
 *     };
 *
 *     $scope.addContact = function() {
 *       $scope.contacts.push({type:'email', value:'yourname@example.org'});
 *     };
 *
 *     $scope.removeContact = function(contactToRemove) {
 *       var index = $scope.contacts.indexOf(contactToRemove);
 *       $scope.contacts.splice(index, 1);
 *     };
 *
 *     $scope.clearContact = function(contact) {
 *       contact.type = 'phone';
 *       contact.value = '';
 *     };
 *   }
 *  </file>
 *  <file name="protractor.js" type="protractor">
 *    it('should check controller', function() {
 *      var container = element(by.id('ctrl-exmpl'));
 *
 *      expect(container.element(by.model('name'))
 *          .getAttribute('value')).toBe('John Smith');
 *
 *      var firstRepeat =
 *          container.element(by.repeater('contact in contacts').row(0));
 *      var secondRepeat =
 *          container.element(by.repeater('contact in contacts').row(1));
 *
 *      expect(firstRepeat.element(by.model('contact.value')).getAttribute('value'))
 *          .toBe('408 555 1212');
 *      expect(secondRepeat.element(by.model('contact.value')).getAttribute('value'))
 *          .toBe('john.smith@example.org');
 *
 *      firstRepeat.element(by.linkText('clear')).click();
 *
 *      expect(firstRepeat.element(by.model('contact.value')).getAttribute('value'))
 *          .toBe('');
 *
 *      container.element(by.linkText('add')).click();
 *
 *      expect(container.element(by.repeater('contact in contacts').row(2))
 *          .element(by.model('contact.value'))
 *          .getAttribute('value'))
 *          .toBe('yourname@example.org');
 *    });
 *  </file>
 *</example>

 */
  var ngControllerDirective = [function () {
        return {
          restrict: 'A',
          scope: true,
          controller: '@',
          priority: 500
        };
      }];
  /**
 * @ngdoc directive
 * @name ngCsp
 *
 * @element html
 * @description
 * Enables [CSP (Content Security Policy)](https://developer.mozilla.org/en/Security/CSP) support.
 *
 * This is necessary when developing things like Google Chrome Extensions or Universal Windows Apps.
 *
 * CSP forbids apps to use `eval` or `Function(string)` generated functions (among other things).
 * For Angular to be CSP compatible there are only two things that we need to do differently:
 *
 * - don't use `Function` constructor to generate optimized value getters
 * - don't inject custom stylesheet into the document
 *
 * AngularJS uses `Function(string)` generated functions as a speed optimization. Applying the `ngCsp`
 * directive will cause Angular to use CSP compatibility mode. When this mode is on AngularJS will
 * evaluate all expressions up to 30% slower than in non-CSP mode, but no security violations will
 * be raised.
 *
 * CSP forbids JavaScript to inline stylesheet rules. In non CSP mode Angular automatically
 * includes some CSS rules (e.g. {@link ng.directive:ngCloak ngCloak}).
 * To make those directives work in CSP mode, include the `angular-csp.css` manually.
 *
 * Angular tries to autodetect if CSP is active and automatically turn on the CSP-safe mode. This
 * autodetection however triggers a CSP error to be logged in the console:
 *
 * ```
 * Refused to evaluate a string as JavaScript because 'unsafe-eval' is not an allowed source of
 * script in the following Content Security Policy directive: "default-src 'self'". Note that
 * 'script-src' was not explicitly set, so 'default-src' is used as a fallback.
 * ```
 *
 * This error is harmless but annoying. To prevent the error from showing up, put the `ngCsp`
 * directive on the root element of the application or on the `angular.js` script tag, whichever
 * appears first in the html document.
 *
 * *Note: This directive is only available in the `ng-csp` and `data-ng-csp` attribute form.*
 *
 * @example
 * This example shows how to apply the `ngCsp` directive to the `html` tag.
   ```html
     <!doctype html>
     <html ng-app ng-csp>
     ...
     ...
     </html>
   ```
  * @example
      // Note: the suffix `.csp` in the example name triggers
      // csp mode in our http server!
      <example name="example.csp" module="cspExample" ng-csp="true">
        <file name="index.html">
          <div ng-controller="MainController as ctrl">
            <div>
              <button ng-click="ctrl.inc()" id="inc">Increment</button>
              <span id="counter">
                {{ctrl.counter}}
              </span>
            </div>

            <div>
              <button ng-click="ctrl.evil()" id="evil">Evil</button>
              <span id="evilError">
                {{ctrl.evilError}}
              </span>
            </div>
          </div>
        </file>
        <file name="script.js">
           angular.module('cspExample', [])
             .controller('MainController', function() {
                this.counter = 0;
                this.inc = function() {
                  this.counter++;
                };
                this.evil = function() {
                  // jshint evil:true
                  try {
                    eval('1+2');
                  } catch (e) {
                    this.evilError = e.message;
                  }
                };
              });
        </file>
        <file name="protractor.js" type="protractor">
          var util, webdriver;

          var incBtn = element(by.id('inc'));
          var counter = element(by.id('counter'));
          var evilBtn = element(by.id('evil'));
          var evilError = element(by.id('evilError'));

          function getAndClearSevereErrors() {
            return browser.manage().logs().get('browser').then(function(browserLog) {
              return browserLog.filter(function(logEntry) {
                return logEntry.level.value > webdriver.logging.Level.WARNING.value;
              });
            });
          }

          function clearErrors() {
            getAndClearSevereErrors();
          }

          function expectNoErrors() {
            getAndClearSevereErrors().then(function(filteredLog) {
              expect(filteredLog.length).toEqual(0);
              if (filteredLog.length) {
                console.log('browser console errors: ' + util.inspect(filteredLog));
              }
            });
          }

          function expectError(regex) {
            getAndClearSevereErrors().then(function(filteredLog) {
              var found = false;
              filteredLog.forEach(function(log) {
                if (log.message.match(regex)) {
                  found = true;
                }
              });
              if (!found) {
                throw new Error('expected an error that matches ' + regex);
              }
            });
          }

          beforeEach(function() {
            util = require('util');
            webdriver = require('protractor/node_modules/selenium-webdriver');
          });

          // For now, we only test on Chrome,
          // as Safari does not load the page with Protractor's injected scripts,
          // and Firefox webdriver always disables content security policy (#6358)
          if (browser.params.browser !== 'chrome') {
            return;
          }

          it('should not report errors when the page is loaded', function() {
            // clear errors so we are not dependent on previous tests
            clearErrors();
            // Need to reload the page as the page is already loaded when
            // we come here
            browser.driver.getCurrentUrl().then(function(url) {
              browser.get(url);
            });
            expectNoErrors();
          });

          it('should evaluate expressions', function() {
            expect(counter.getText()).toEqual('0');
            incBtn.click();
            expect(counter.getText()).toEqual('1');
            expectNoErrors();
          });

          it('should throw and report an error when using "eval"', function() {
            evilBtn.click();
            expect(evilError.getText()).toMatch(/Content Security Policy/);
            expectError(/Content Security Policy/);
          });
        </file>
      </example>
  */
  // ngCsp is not implemented as a proper directive any more, because we need it be processed while we
  // bootstrap the system (before $parse is instantiated), for this reason we just have
  // the csp.isActive() fn that looks for ng-csp attribute anywhere in the current doc
  /**
 * @ngdoc directive
 * @name ngClick
 *
 * @description
 * The ngClick directive allows you to specify custom behavior when
 * an element is clicked.
 *
 * @element ANY
 * @priority 0
 * @param {expression} ngClick {@link guide/expression Expression} to evaluate upon
 * click. ({@link guide/expression#-event- Event object is available as `$event`})
 *
 * @example
   <example>
     <file name="index.html">
      <button ng-click="count = count + 1" ng-init="count=0">
        Increment
      </button>
      <span>
        count: {{count}}
      </span>
     </file>
     <file name="protractor.js" type="protractor">
       it('should check ng-click', function() {
         expect(element(by.binding('count')).getText()).toMatch('0');
         element(by.css('button')).click();
         expect(element(by.binding('count')).getText()).toMatch('1');
       });
     </file>
   </example>
 */
  /*
 * A collection of directives that allows creation of custom event handlers that are defined as
 * angular expressions and are compiled and executed within the current scope.
 */
  var ngEventDirectives = {};
  // For events that might fire synchronously during DOM manipulation
  // we need to execute their event handlers asynchronously using $evalAsync,
  // so that they are not executed in an inconsistent state.
  var forceAsyncEvents = {
      'blur': true,
      'focus': true
    };
  forEach('click dblclick mousedown mouseup mouseover mouseout mousemove mouseenter mouseleave keydown keyup keypress submit focus blur copy cut paste'.split(' '), function (eventName) {
    var directiveName = directiveNormalize('ng-' + eventName);
    ngEventDirectives[directiveName] = [
      '$parse',
      '$rootScope',
      function ($parse, $rootScope) {
        return {
          restrict: 'A',
          compile: function ($element, attr) {
            // We expose the powerful $event object on the scope that provides access to the Window,
            // etc. that isn't protected by the fast paths in $parse.  We explicitly request better
            // checks at the cost of speed since event handler expressions are not executed as
            // frequently as regular change detection.
            var fn = $parse(attr[directiveName], null, true);
            return function ngEventHandler(scope, element) {
              element.on(eventName, function (event) {
                var callback = function () {
                  fn(scope, { $event: event });
                };
                if (forceAsyncEvents[eventName] && $rootScope.$$phase) {
                  scope.$evalAsync(callback);
                } else {
                  scope.$apply(callback);
                }
              });
            };
          }
        };
      }
    ];
  });
  /**
 * @ngdoc directive
 * @name ngDblclick
 *
 * @description
 * The `ngDblclick` directive allows you to specify custom behavior on a dblclick event.
 *
 * @element ANY
 * @priority 0
 * @param {expression} ngDblclick {@link guide/expression Expression} to evaluate upon
 * a dblclick. (The Event object is available as `$event`)
 *
 * @example
   <example>
     <file name="index.html">
      <button ng-dblclick="count = count + 1" ng-init="count=0">
        Increment (on double click)
      </button>
      count: {{count}}
     </file>
   </example>
 */
  /**
 * @ngdoc directive
 * @name ngMousedown
 *
 * @description
 * The ngMousedown directive allows you to specify custom behavior on mousedown event.
 *
 * @element ANY
 * @priority 0
 * @param {expression} ngMousedown {@link guide/expression Expression} to evaluate upon
 * mousedown. ({@link guide/expression#-event- Event object is available as `$event`})
 *
 * @example
   <example>
     <file name="index.html">
      <button ng-mousedown="count = count + 1" ng-init="count=0">
        Increment (on mouse down)
      </button>
      count: {{count}}
     </file>
   </example>
 */
  /**
 * @ngdoc directive
 * @name ngMouseup
 *
 * @description
 * Specify custom behavior on mouseup event.
 *
 * @element ANY
 * @priority 0
 * @param {expression} ngMouseup {@link guide/expression Expression} to evaluate upon
 * mouseup. ({@link guide/expression#-event- Event object is available as `$event`})
 *
 * @example
   <example>
     <file name="index.html">
      <button ng-mouseup="count = count + 1" ng-init="count=0">
        Increment (on mouse up)
      </button>
      count: {{count}}
     </file>
   </example>
 */
  /**
 * @ngdoc directive
 * @name ngMouseover
 *
 * @description
 * Specify custom behavior on mouseover event.
 *
 * @element ANY
 * @priority 0
 * @param {expression} ngMouseover {@link guide/expression Expression} to evaluate upon
 * mouseover. ({@link guide/expression#-event- Event object is available as `$event`})
 *
 * @example
   <example>
     <file name="index.html">
      <button ng-mouseover="count = count + 1" ng-init="count=0">
        Increment (when mouse is over)
      </button>
      count: {{count}}
     </file>
   </example>
 */
  /**
 * @ngdoc directive
 * @name ngMouseenter
 *
 * @description
 * Specify custom behavior on mouseenter event.
 *
 * @element ANY
 * @priority 0
 * @param {expression} ngMouseenter {@link guide/expression Expression} to evaluate upon
 * mouseenter. ({@link guide/expression#-event- Event object is available as `$event`})
 *
 * @example
   <example>
     <file name="index.html">
      <button ng-mouseenter="count = count + 1" ng-init="count=0">
        Increment (when mouse enters)
      </button>
      count: {{count}}
     </file>
   </example>
 */
  /**
 * @ngdoc directive
 * @name ngMouseleave
 *
 * @description
 * Specify custom behavior on mouseleave event.
 *
 * @element ANY
 * @priority 0
 * @param {expression} ngMouseleave {@link guide/expression Expression} to evaluate upon
 * mouseleave. ({@link guide/expression#-event- Event object is available as `$event`})
 *
 * @example
   <example>
     <file name="index.html">
      <button ng-mouseleave="count = count + 1" ng-init="count=0">
        Increment (when mouse leaves)
      </button>
      count: {{count}}
     </file>
   </example>
 */
  /**
 * @ngdoc directive
 * @name ngMousemove
 *
 * @description
 * Specify custom behavior on mousemove event.
 *
 * @element ANY
 * @priority 0
 * @param {expression} ngMousemove {@link guide/expression Expression} to evaluate upon
 * mousemove. ({@link guide/expression#-event- Event object is available as `$event`})
 *
 * @example
   <example>
     <file name="index.html">
      <button ng-mousemove="count = count + 1" ng-init="count=0">
        Increment (when mouse moves)
      </button>
      count: {{count}}
     </file>
   </example>
 */
  /**
 * @ngdoc directive
 * @name ngKeydown
 *
 * @description
 * Specify custom behavior on keydown event.
 *
 * @element ANY
 * @priority 0
 * @param {expression} ngKeydown {@link guide/expression Expression} to evaluate upon
 * keydown. (Event object is available as `$event` and can be interrogated for keyCode, altKey, etc.)
 *
 * @example
   <example>
     <file name="index.html">
      <input ng-keydown="count = count + 1" ng-init="count=0">
      key down count: {{count}}
     </file>
   </example>
 */
  /**
 * @ngdoc directive
 * @name ngKeyup
 *
 * @description
 * Specify custom behavior on keyup event.
 *
 * @element ANY
 * @priority 0
 * @param {expression} ngKeyup {@link guide/expression Expression} to evaluate upon
 * keyup. (Event object is available as `$event` and can be interrogated for keyCode, altKey, etc.)
 *
 * @example
   <example>
     <file name="index.html">
       <p>Typing in the input box below updates the key count</p>
       <input ng-keyup="count = count + 1" ng-init="count=0"> key up count: {{count}}

       <p>Typing in the input box below updates the keycode</p>
       <input ng-keyup="event=$event">
       <p>event keyCode: {{ event.keyCode }}</p>
       <p>event altKey: {{ event.altKey }}</p>
     </file>
   </example>
 */
  /**
 * @ngdoc directive
 * @name ngKeypress
 *
 * @description
 * Specify custom behavior on keypress event.
 *
 * @element ANY
 * @param {expression} ngKeypress {@link guide/expression Expression} to evaluate upon
 * keypress. ({@link guide/expression#-event- Event object is available as `$event`}
 * and can be interrogated for keyCode, altKey, etc.)
 *
 * @example
   <example>
     <file name="index.html">
      <input ng-keypress="count = count + 1" ng-init="count=0">
      key press count: {{count}}
     </file>
   </example>
 */
  /**
 * @ngdoc directive
 * @name ngSubmit
 *
 * @description
 * Enables binding angular expressions to onsubmit events.
 *
 * Additionally it prevents the default action (which for form means sending the request to the
 * server and reloading the current page), but only if the form does not contain `action`,
 * `data-action`, or `x-action` attributes.
 *
 * <div class="alert alert-warning">
 * **Warning:** Be careful not to cause "double-submission" by using both the `ngClick` and
 * `ngSubmit` handlers together. See the
 * {@link form#submitting-a-form-and-preventing-the-default-action `form` directive documentation}
 * for a detailed discussion of when `ngSubmit` may be triggered.
 * </div>
 *
 * @element form
 * @priority 0
 * @param {expression} ngSubmit {@link guide/expression Expression} to eval.
 * ({@link guide/expression#-event- Event object is available as `$event`})
 *
 * @example
   <example module="submitExample">
     <file name="index.html">
      <script>
        angular.module('submitExample', [])
          .controller('ExampleController', ['$scope', function($scope) {
            $scope.list = [];
            $scope.text = 'hello';
            $scope.submit = function() {
              if ($scope.text) {
                $scope.list.push(this.text);
                $scope.text = '';
              }
            };
          }]);
      </script>
      <form ng-submit="submit()" ng-controller="ExampleController">
        Enter text and hit enter:
        <input type="text" ng-model="text" name="text" />
        <input type="submit" id="submit" value="Submit" />
        <pre>list={{list}}</pre>
      </form>
     </file>
     <file name="protractor.js" type="protractor">
       it('should check ng-submit', function() {
         expect(element(by.binding('list')).getText()).toBe('list=[]');
         element(by.css('#submit')).click();
         expect(element(by.binding('list')).getText()).toContain('hello');
         expect(element(by.model('text')).getAttribute('value')).toBe('');
       });
       it('should ignore empty strings', function() {
         expect(element(by.binding('list')).getText()).toBe('list=[]');
         element(by.css('#submit')).click();
         element(by.css('#submit')).click();
         expect(element(by.binding('list')).getText()).toContain('hello');
        });
     </file>
   </example>
 */
  /**
 * @ngdoc directive
 * @name ngFocus
 *
 * @description
 * Specify custom behavior on focus event.
 *
 * Note: As the `focus` event is executed synchronously when calling `input.focus()`
 * AngularJS executes the expression using `scope.$evalAsync` if the event is fired
 * during an `$apply` to ensure a consistent state.
 *
 * @element window, input, select, textarea, a
 * @priority 0
 * @param {expression} ngFocus {@link guide/expression Expression} to evaluate upon
 * focus. ({@link guide/expression#-event- Event object is available as `$event`})
 *
 * @example
 * See {@link ng.directive:ngClick ngClick}
 */
  /**
 * @ngdoc directive
 * @name ngBlur
 *
 * @description
 * Specify custom behavior on blur event.
 *
 * A [blur event](https://developer.mozilla.org/en-US/docs/Web/Events/blur) fires when
 * an element has lost focus.
 *
 * Note: As the `blur` event is executed synchronously also during DOM manipulations
 * (e.g. removing a focussed input),
 * AngularJS executes the expression using `scope.$evalAsync` if the event is fired
 * during an `$apply` to ensure a consistent state.
 *
 * @element window, input, select, textarea, a
 * @priority 0
 * @param {expression} ngBlur {@link guide/expression Expression} to evaluate upon
 * blur. ({@link guide/expression#-event- Event object is available as `$event`})
 *
 * @example
 * See {@link ng.directive:ngClick ngClick}
 */
  /**
 * @ngdoc directive
 * @name ngCopy
 *
 * @description
 * Specify custom behavior on copy event.
 *
 * @element window, input, select, textarea, a
 * @priority 0
 * @param {expression} ngCopy {@link guide/expression Expression} to evaluate upon
 * copy. ({@link guide/expression#-event- Event object is available as `$event`})
 *
 * @example
   <example>
     <file name="index.html">
      <input ng-copy="copied=true" ng-init="copied=false; value='copy me'" ng-model="value">
      copied: {{copied}}
     </file>
   </example>
 */
  /**
 * @ngdoc directive
 * @name ngCut
 *
 * @description
 * Specify custom behavior on cut event.
 *
 * @element window, input, select, textarea, a
 * @priority 0
 * @param {expression} ngCut {@link guide/expression Expression} to evaluate upon
 * cut. ({@link guide/expression#-event- Event object is available as `$event`})
 *
 * @example
   <example>
     <file name="index.html">
      <input ng-cut="cut=true" ng-init="cut=false; value='cut me'" ng-model="value">
      cut: {{cut}}
     </file>
   </example>
 */
  /**
 * @ngdoc directive
 * @name ngPaste
 *
 * @description
 * Specify custom behavior on paste event.
 *
 * @element window, input, select, textarea, a
 * @priority 0
 * @param {expression} ngPaste {@link guide/expression Expression} to evaluate upon
 * paste. ({@link guide/expression#-event- Event object is available as `$event`})
 *
 * @example
   <example>
     <file name="index.html">
      <input ng-paste="paste=true" ng-init="paste=false" placeholder='paste here'>
      pasted: {{paste}}
     </file>
   </example>
 */
  /**
 * @ngdoc directive
 * @name ngIf
 * @restrict A
 *
 * @description
 * The `ngIf` directive removes or recreates a portion of the DOM tree based on an
 * {expression}. If the expression assigned to `ngIf` evaluates to a false
 * value then the element is removed from the DOM, otherwise a clone of the
 * element is reinserted into the DOM.
 *
 * `ngIf` differs from `ngShow` and `ngHide` in that `ngIf` completely removes and recreates the
 * element in the DOM rather than changing its visibility via the `display` css property.  A common
 * case when this difference is significant is when using css selectors that rely on an element's
 * position within the DOM, such as the `:first-child` or `:last-child` pseudo-classes.
 *
 * Note that when an element is removed using `ngIf` its scope is destroyed and a new scope
 * is created when the element is restored.  The scope created within `ngIf` inherits from
 * its parent scope using
 * [prototypal inheritance](https://github.com/angular/angular.js/wiki/Understanding-Scopes#javascript-prototypal-inheritance).
 * An important implication of this is if `ngModel` is used within `ngIf` to bind to
 * a javascript primitive defined in the parent scope. In this case any modifications made to the
 * variable within the child scope will override (hide) the value in the parent scope.
 *
 * Also, `ngIf` recreates elements using their compiled state. An example of this behavior
 * is if an element's class attribute is directly modified after it's compiled, using something like
 * jQuery's `.addClass()` method, and the element is later removed. When `ngIf` recreates the element
 * the added class will be lost because the original compiled state is used to regenerate the element.
 *
 * Additionally, you can provide animations via the `ngAnimate` module to animate the `enter`
 * and `leave` effects.
 *
 * @animations
 * enter - happens just after the `ngIf` contents change and a new DOM element is created and injected into the `ngIf` container
 * leave - happens just before the `ngIf` contents are removed from the DOM
 *
 * @element ANY
 * @scope
 * @priority 600
 * @param {expression} ngIf If the {@link guide/expression expression} is falsy then
 *     the element is removed from the DOM tree. If it is truthy a copy of the compiled
 *     element is added to the DOM tree.
 *
 * @example
  <example module="ngAnimate" deps="angular-animate.js" animations="true">
    <file name="index.html">
      Click me: <input type="checkbox" ng-model="checked" ng-init="checked=true" /><br/>
      Show when checked:
      <span ng-if="checked" class="animate-if">
        This is removed when the checkbox is unchecked.
      </span>
    </file>
    <file name="animations.css">
      .animate-if {
        background:white;
        border:1px solid black;
        padding:10px;
      }

      .animate-if.ng-enter, .animate-if.ng-leave {
        -webkit-transition:all cubic-bezier(0.250, 0.460, 0.450, 0.940) 0.5s;
        transition:all cubic-bezier(0.250, 0.460, 0.450, 0.940) 0.5s;
      }

      .animate-if.ng-enter,
      .animate-if.ng-leave.ng-leave-active {
        opacity:0;
      }

      .animate-if.ng-leave,
      .animate-if.ng-enter.ng-enter-active {
        opacity:1;
      }
    </file>
  </example>
 */
  var ngIfDirective = [
      '$animate',
      function ($animate) {
        return {
          multiElement: true,
          transclude: 'element',
          priority: 600,
          terminal: true,
          restrict: 'A',
          $$tlb: true,
          link: function ($scope, $element, $attr, ctrl, $transclude) {
            var block, childScope, previousElements;
            $scope.$watch($attr.ngIf, function ngIfWatchAction(value) {
              if (value) {
                if (!childScope) {
                  $transclude(function (clone, newScope) {
                    childScope = newScope;
                    clone[clone.length++] = document.createComment(' end ngIf: ' + $attr.ngIf + ' ');
                    // Note: We only need the first/last node of the cloned nodes.
                    // However, we need to keep the reference to the jqlite wrapper as it might be changed later
                    // by a directive with templateUrl when its template arrives.
                    block = { clone: clone };
                    $animate.enter(clone, $element.parent(), $element);
                  });
                }
              } else {
                if (previousElements) {
                  previousElements.remove();
                  previousElements = null;
                }
                if (childScope) {
                  childScope.$destroy();
                  childScope = null;
                }
                if (block) {
                  previousElements = getBlockNodes(block.clone);
                  $animate.leave(previousElements).then(function () {
                    previousElements = null;
                  });
                  block = null;
                }
              }
            });
          }
        };
      }
    ];
  /**
 * @ngdoc directive
 * @name ngInclude
 * @restrict ECA
 *
 * @description
 * Fetches, compiles and includes an external HTML fragment.
 *
 * By default, the template URL is restricted to the same domain and protocol as the
 * application document. This is done by calling {@link $sce#getTrustedResourceUrl
 * $sce.getTrustedResourceUrl} on it. To load templates from other domains or protocols
 * you may either {@link ng.$sceDelegateProvider#resourceUrlWhitelist whitelist them} or
 * {@link $sce#trustAsResourceUrl wrap them} as trusted values. Refer to Angular's {@link
 * ng.$sce Strict Contextual Escaping}.
 *
 * In addition, the browser's
 * [Same Origin Policy](https://code.google.com/p/browsersec/wiki/Part2#Same-origin_policy_for_XMLHttpRequest)
 * and [Cross-Origin Resource Sharing (CORS)](http://www.w3.org/TR/cors/)
 * policy may further restrict whether the template is successfully loaded.
 * For example, `ngInclude` won't work for cross-domain requests on all browsers and for `file://`
 * access on some browsers.
 *
 * @animations
 * enter - animation is used to bring new content into the browser.
 * leave - animation is used to animate existing content away.
 *
 * The enter and leave animation occur concurrently.
 *
 * @scope
 * @priority 400
 *
 * @param {string} ngInclude|src angular expression evaluating to URL. If the source is a string constant,
 *                 make sure you wrap it in **single** quotes, e.g. `src="'myPartialTemplate.html'"`.
 * @param {string=} onload Expression to evaluate when a new partial is loaded.
 *
 * @param {string=} autoscroll Whether `ngInclude` should call {@link ng.$anchorScroll
 *                  $anchorScroll} to scroll the viewport after the content is loaded.
 *
 *                  - If the attribute is not set, disable scrolling.
 *                  - If the attribute is set without value, enable scrolling.
 *                  - Otherwise enable scrolling only if the expression evaluates to truthy value.
 *
 * @example
  <example module="includeExample" deps="angular-animate.js" animations="true">
    <file name="index.html">
     <div ng-controller="ExampleController">
       <select ng-model="template" ng-options="t.name for t in templates">
        <option value="">(blank)</option>
       </select>
       url of the template: <code>{{template.url}}</code>
       <hr/>
       <div class="slide-animate-container">
         <div class="slide-animate" ng-include="template.url"></div>
       </div>
     </div>
    </file>
    <file name="script.js">
      angular.module('includeExample', ['ngAnimate'])
        .controller('ExampleController', ['$scope', function($scope) {
          $scope.templates =
            [ { name: 'template1.html', url: 'template1.html'},
              { name: 'template2.html', url: 'template2.html'} ];
          $scope.template = $scope.templates[0];
        }]);
     </file>
    <file name="template1.html">
      Content of template1.html
    </file>
    <file name="template2.html">
      Content of template2.html
    </file>
    <file name="animations.css">
      .slide-animate-container {
        position:relative;
        background:white;
        border:1px solid black;
        height:40px;
        overflow:hidden;
      }

      .slide-animate {
        padding:10px;
      }

      .slide-animate.ng-enter, .slide-animate.ng-leave {
        -webkit-transition:all cubic-bezier(0.250, 0.460, 0.450, 0.940) 0.5s;
        transition:all cubic-bezier(0.250, 0.460, 0.450, 0.940) 0.5s;

        position:absolute;
        top:0;
        left:0;
        right:0;
        bottom:0;
        display:block;
        padding:10px;
      }

      .slide-animate.ng-enter {
        top:-50px;
      }
      .slide-animate.ng-enter.ng-enter-active {
        top:0;
      }

      .slide-animate.ng-leave {
        top:0;
      }
      .slide-animate.ng-leave.ng-leave-active {
        top:50px;
      }
    </file>
    <file name="protractor.js" type="protractor">
      var templateSelect = element(by.model('template'));
      var includeElem = element(by.css('[ng-include]'));

      it('should load template1.html', function() {
        expect(includeElem.getText()).toMatch(/Content of template1.html/);
      });

      it('should load template2.html', function() {
        if (browser.params.browser == 'firefox') {
          // Firefox can't handle using selects
          // See https://github.com/angular/protractor/issues/480
          return;
        }
        templateSelect.click();
        templateSelect.all(by.css('option')).get(2).click();
        expect(includeElem.getText()).toMatch(/Content of template2.html/);
      });

      it('should change to blank', function() {
        if (browser.params.browser == 'firefox') {
          // Firefox can't handle using selects
          return;
        }
        templateSelect.click();
        templateSelect.all(by.css('option')).get(0).click();
        expect(includeElem.isPresent()).toBe(false);
      });
    </file>
  </example>
 */
  /**
 * @ngdoc event
 * @name ngInclude#$includeContentRequested
 * @eventType emit on the scope ngInclude was declared in
 * @description
 * Emitted every time the ngInclude content is requested.
 *
 * @param {Object} angularEvent Synthetic event object.
 * @param {String} src URL of content to load.
 */
  /**
 * @ngdoc event
 * @name ngInclude#$includeContentLoaded
 * @eventType emit on the current ngInclude scope
 * @description
 * Emitted every time the ngInclude content is reloaded.
 *
 * @param {Object} angularEvent Synthetic event object.
 * @param {String} src URL of content to load.
 */
  /**
 * @ngdoc event
 * @name ngInclude#$includeContentError
 * @eventType emit on the scope ngInclude was declared in
 * @description
 * Emitted when a template HTTP request yields an erroneous response (status < 200 || status > 299)
 *
 * @param {Object} angularEvent Synthetic event object.
 * @param {String} src URL of content to load.
 */
  var ngIncludeDirective = [
      '$templateRequest',
      '$anchorScroll',
      '$animate',
      function ($templateRequest, $anchorScroll, $animate) {
        return {
          restrict: 'ECA',
          priority: 400,
          terminal: true,
          transclude: 'element',
          controller: angular.noop,
          compile: function (element, attr) {
            var srcExp = attr.ngInclude || attr.src, onloadExp = attr.onload || '', autoScrollExp = attr.autoscroll;
            return function (scope, $element, $attr, ctrl, $transclude) {
              var changeCounter = 0, currentScope, previousElement, currentElement;
              var cleanupLastIncludeContent = function () {
                if (previousElement) {
                  previousElement.remove();
                  previousElement = null;
                }
                if (currentScope) {
                  currentScope.$destroy();
                  currentScope = null;
                }
                if (currentElement) {
                  $animate.leave(currentElement).then(function () {
                    previousElement = null;
                  });
                  previousElement = currentElement;
                  currentElement = null;
                }
              };
              scope.$watch(srcExp, function ngIncludeWatchAction(src) {
                var afterAnimation = function () {
                  if (isDefined(autoScrollExp) && (!autoScrollExp || scope.$eval(autoScrollExp))) {
                    $anchorScroll();
                  }
                };
                var thisChangeId = ++changeCounter;
                if (src) {
                  //set the 2nd param to true to ignore the template request error so that the inner
                  //contents and scope can be cleaned up.
                  $templateRequest(src, true).then(function (response) {
                    if (thisChangeId !== changeCounter)
                      return;
                    var newScope = scope.$new();
                    ctrl.template = response;
                    // Note: This will also link all children of ng-include that were contained in the original
                    // html. If that content contains controllers, ... they could pollute/change the scope.
                    // However, using ng-include on an element with additional content does not make sense...
                    // Note: We can't remove them in the cloneAttchFn of $transclude as that
                    // function is called before linking the content, which would apply child
                    // directives to non existing elements.
                    var clone = $transclude(newScope, function (clone) {
                        cleanupLastIncludeContent();
                        $animate.enter(clone, null, $element).then(afterAnimation);
                      });
                    currentScope = newScope;
                    currentElement = clone;
                    currentScope.$emit('$includeContentLoaded', src);
                    scope.$eval(onloadExp);
                  }, function () {
                    if (thisChangeId === changeCounter) {
                      cleanupLastIncludeContent();
                      scope.$emit('$includeContentError', src);
                    }
                  });
                  scope.$emit('$includeContentRequested', src);
                } else {
                  cleanupLastIncludeContent();
                  ctrl.template = null;
                }
              });
            };
          }
        };
      }
    ];
  // This directive is called during the $transclude call of the first `ngInclude` directive.
  // It will replace and compile the content of the element with the loaded template.
  // We need this directive so that the element content is already filled when
  // the link function of another directive on the same element as ngInclude
  // is called.
  var ngIncludeFillContentDirective = [
      '$compile',
      function ($compile) {
        return {
          restrict: 'ECA',
          priority: -400,
          require: 'ngInclude',
          link: function (scope, $element, $attr, ctrl) {
            if (/SVG/.test($element[0].toString())) {
              // WebKit: https://bugs.webkit.org/show_bug.cgi?id=135698 --- SVG elements do not
              // support innerHTML, so detect this here and try to generate the contents
              // specially.
              $element.empty();
              $compile(jqLiteBuildFragment(ctrl.template, document).childNodes)(scope, function namespaceAdaptedClone(clone) {
                $element.append(clone);
              }, { futureParentElement: $element });
              return;
            }
            $element.html(ctrl.template);
            $compile($element.contents())(scope);
          }
        };
      }
    ];
  /**
 * @ngdoc directive
 * @name ngInit
 * @restrict AC
 *
 * @description
 * The `ngInit` directive allows you to evaluate an expression in the
 * current scope.
 *
 * <div class="alert alert-error">
 * The only appropriate use of `ngInit` is for aliasing special properties of
 * {@link ng.directive:ngRepeat `ngRepeat`}, as seen in the demo below. Besides this case, you
 * should use {@link guide/controller controllers} rather than `ngInit`
 * to initialize values on a scope.
 * </div>
 * <div class="alert alert-warning">
 * **Note**: If you have assignment in `ngInit` along with {@link ng.$filter `$filter`}, make
 * sure you have parenthesis for correct precedence:
 * <pre class="prettyprint">
 * `<div ng-init="test1 = (data | orderBy:'name')"></div>`
 * </pre>
 * </div>
 *
 * @priority 450
 *
 * @element ANY
 * @param {expression} ngInit {@link guide/expression Expression} to eval.
 *
 * @example
   <example module="initExample">
     <file name="index.html">
   <script>
     angular.module('initExample', [])
       .controller('ExampleController', ['$scope', function($scope) {
         $scope.list = [['a', 'b'], ['c', 'd']];
       }]);
   </script>
   <div ng-controller="ExampleController">
     <div ng-repeat="innerList in list" ng-init="outerIndex = $index">
       <div ng-repeat="value in innerList" ng-init="innerIndex = $index">
          <span class="example-init">list[ {{outerIndex}} ][ {{innerIndex}} ] = {{value}};</span>
       </div>
     </div>
   </div>
     </file>
     <file name="protractor.js" type="protractor">
       it('should alias index positions', function() {
         var elements = element.all(by.css('.example-init'));
         expect(elements.get(0).getText()).toBe('list[ 0 ][ 0 ] = a;');
         expect(elements.get(1).getText()).toBe('list[ 0 ][ 1 ] = b;');
         expect(elements.get(2).getText()).toBe('list[ 1 ][ 0 ] = c;');
         expect(elements.get(3).getText()).toBe('list[ 1 ][ 1 ] = d;');
       });
     </file>
   </example>
 */
  var ngInitDirective = ngDirective({
      priority: 450,
      compile: function () {
        return {
          pre: function (scope, element, attrs) {
            scope.$eval(attrs.ngInit);
          }
        };
      }
    });
  /**
 * @ngdoc directive
 * @name ngList
 *
 * @description
 * Text input that converts between a delimited string and an array of strings. The default
 * delimiter is a comma followed by a space - equivalent to `ng-list=", "`. You can specify a custom
 * delimiter as the value of the `ngList` attribute - for example, `ng-list=" | "`.
 *
 * The behaviour of the directive is affected by the use of the `ngTrim` attribute.
 * * If `ngTrim` is set to `"false"` then whitespace around both the separator and each
 *   list item is respected. This implies that the user of the directive is responsible for
 *   dealing with whitespace but also allows you to use whitespace as a delimiter, such as a
 *   tab or newline character.
 * * Otherwise whitespace around the delimiter is ignored when splitting (although it is respected
 *   when joining the list items back together) and whitespace around each list item is stripped
 *   before it is added to the model.
 *
 * ### Example with Validation
 *
 * <example name="ngList-directive" module="listExample">
 *   <file name="app.js">
 *      angular.module('listExample', [])
 *        .controller('ExampleController', ['$scope', function($scope) {
 *          $scope.names = ['morpheus', 'neo', 'trinity'];
 *        }]);
 *   </file>
 *   <file name="index.html">
 *    <form name="myForm" ng-controller="ExampleController">
 *      List: <input name="namesInput" ng-model="names" ng-list required>
 *      <span class="error" ng-show="myForm.namesInput.$error.required">
 *        Required!</span>
 *      <br>
 *      <tt>names = {{names}}</tt><br/>
 *      <tt>myForm.namesInput.$valid = {{myForm.namesInput.$valid}}</tt><br/>
 *      <tt>myForm.namesInput.$error = {{myForm.namesInput.$error}}</tt><br/>
 *      <tt>myForm.$valid = {{myForm.$valid}}</tt><br/>
 *      <tt>myForm.$error.required = {{!!myForm.$error.required}}</tt><br/>
 *     </form>
 *   </file>
 *   <file name="protractor.js" type="protractor">
 *     var listInput = element(by.model('names'));
 *     var names = element(by.exactBinding('names'));
 *     var valid = element(by.binding('myForm.namesInput.$valid'));
 *     var error = element(by.css('span.error'));
 *
 *     it('should initialize to model', function() {
 *       expect(names.getText()).toContain('["morpheus","neo","trinity"]');
 *       expect(valid.getText()).toContain('true');
 *       expect(error.getCssValue('display')).toBe('none');
 *     });
 *
 *     it('should be invalid if empty', function() {
 *       listInput.clear();
 *       listInput.sendKeys('');
 *
 *       expect(names.getText()).toContain('');
 *       expect(valid.getText()).toContain('false');
 *       expect(error.getCssValue('display')).not.toBe('none');
 *     });
 *   </file>
 * </example>
 *
 * ### Example - splitting on whitespace
 * <example name="ngList-directive-newlines">
 *   <file name="index.html">
 *    <textarea ng-model="list" ng-list="&#10;" ng-trim="false"></textarea>
 *    <pre>{{ list | json }}</pre>
 *   </file>
 *   <file name="protractor.js" type="protractor">
 *     it("should split the text by newlines", function() {
 *       var listInput = element(by.model('list'));
 *       var output = element(by.binding('list | json'));
 *       listInput.sendKeys('abc\ndef\nghi');
 *       expect(output.getText()).toContain('[\n  "abc",\n  "def",\n  "ghi"\n]');
 *     });
 *   </file>
 * </example>
 *
 * @element input
 * @param {string=} ngList optional delimiter that should be used to split the value.
 */
  var ngListDirective = function () {
    return {
      restrict: 'A',
      priority: 100,
      require: 'ngModel',
      link: function (scope, element, attr, ctrl) {
        // We want to control whitespace trimming so we use this convoluted approach
        // to access the ngList attribute, which doesn't pre-trim the attribute
        var ngList = element.attr(attr.$attr.ngList) || ', ';
        var trimValues = attr.ngTrim !== 'false';
        var separator = trimValues ? trim(ngList) : ngList;
        var parse = function (viewValue) {
          // If the viewValue is invalid (say required but empty) it will be `undefined`
          if (isUndefined(viewValue))
            return;
          var list = [];
          if (viewValue) {
            forEach(viewValue.split(separator), function (value) {
              if (value)
                list.push(trimValues ? trim(value) : value);
            });
          }
          return list;
        };
        ctrl.$parsers.push(parse);
        ctrl.$formatters.push(function (value) {
          if (isArray(value)) {
            return value.join(ngList);
          }
          return undefined;
        });
        // Override the standard $isEmpty because an empty array means the input is empty.
        ctrl.$isEmpty = function (value) {
          return !value || !value.length;
        };
      }
    };
  };
  /* global VALID_CLASS: true,
  INVALID_CLASS: true,
  PRISTINE_CLASS: true,
  DIRTY_CLASS: true,
  UNTOUCHED_CLASS: true,
  TOUCHED_CLASS: true,
*/
  var VALID_CLASS = 'ng-valid', INVALID_CLASS = 'ng-invalid', PRISTINE_CLASS = 'ng-pristine', DIRTY_CLASS = 'ng-dirty', UNTOUCHED_CLASS = 'ng-untouched', TOUCHED_CLASS = 'ng-touched', PENDING_CLASS = 'ng-pending';
  var ngModelMinErr = minErr('ngModel');
  /**
 * @ngdoc type
 * @name ngModel.NgModelController
 *
 * @property {string} $viewValue Actual string value in the view.
 * @property {*} $modelValue The value in the model that the control is bound to.
 * @property {Array.<Function>} $parsers Array of functions to execute, as a pipeline, whenever
       the control reads value from the DOM. The functions are called in array order, each passing
       its return value through to the next. The last return value is forwarded to the
       {@link ngModel.NgModelController#$validators `$validators`} collection.

Parsers are used to sanitize / convert the {@link ngModel.NgModelController#$viewValue
`$viewValue`}.

Returning `undefined` from a parser means a parse error occurred. In that case,
no {@link ngModel.NgModelController#$validators `$validators`} will run and the `ngModel`
will be set to `undefined` unless {@link ngModelOptions `ngModelOptions.allowInvalid`}
is set to `true`. The parse error is stored in `ngModel.$error.parse`.

 *
 * @property {Array.<Function>} $formatters Array of functions to execute, as a pipeline, whenever
       the model value changes. The functions are called in reverse array order, each passing the value through to the
       next. The last return value is used as the actual DOM value.
       Used to format / convert values for display in the control.
 * ```js
 * function formatter(value) {
 *   if (value) {
 *     return value.toUpperCase();
 *   }
 * }
 * ngModel.$formatters.push(formatter);
 * ```
 *
 * @property {Object.<string, function>} $validators A collection of validators that are applied
 *      whenever the model value changes. The key value within the object refers to the name of the
 *      validator while the function refers to the validation operation. The validation operation is
 *      provided with the model value as an argument and must return a true or false value depending
 *      on the response of that validation.
 *
 * ```js
 * ngModel.$validators.validCharacters = function(modelValue, viewValue) {
 *   var value = modelValue || viewValue;
 *   return /[0-9]+/.test(value) &&
 *          /[a-z]+/.test(value) &&
 *          /[A-Z]+/.test(value) &&
 *          /\W+/.test(value);
 * };
 * ```
 *
 * @property {Object.<string, function>} $asyncValidators A collection of validations that are expected to
 *      perform an asynchronous validation (e.g. a HTTP request). The validation function that is provided
 *      is expected to return a promise when it is run during the model validation process. Once the promise
 *      is delivered then the validation status will be set to true when fulfilled and false when rejected.
 *      When the asynchronous validators are triggered, each of the validators will run in parallel and the model
 *      value will only be updated once all validators have been fulfilled. As long as an asynchronous validator
 *      is unfulfilled, its key will be added to the controllers `$pending` property. Also, all asynchronous validators
 *      will only run once all synchronous validators have passed.
 *
 * Please note that if $http is used then it is important that the server returns a success HTTP response code
 * in order to fulfill the validation and a status level of `4xx` in order to reject the validation.
 *
 * ```js
 * ngModel.$asyncValidators.uniqueUsername = function(modelValue, viewValue) {
 *   var value = modelValue || viewValue;
 *
 *   // Lookup user by username
 *   return $http.get('/api/users/' + value).
 *      then(function resolved() {
 *        //username exists, this means validation fails
 *        return $q.reject('exists');
 *      }, function rejected() {
 *        //username does not exist, therefore this validation passes
 *        return true;
 *      });
 * };
 * ```
 *
 * @property {Array.<Function>} $viewChangeListeners Array of functions to execute whenever the
 *     view value has changed. It is called with no arguments, and its return value is ignored.
 *     This can be used in place of additional $watches against the model value.
 *
 * @property {Object} $error An object hash with all failing validator ids as keys.
 * @property {Object} $pending An object hash with all pending validator ids as keys.
 *
 * @property {boolean} $untouched True if control has not lost focus yet.
 * @property {boolean} $touched True if control has lost focus.
 * @property {boolean} $pristine True if user has not interacted with the control yet.
 * @property {boolean} $dirty True if user has already interacted with the control.
 * @property {boolean} $valid True if there is no error.
 * @property {boolean} $invalid True if at least one error on the control.
 * @property {string} $name The name attribute of the control.
 *
 * @description
 *
 * `NgModelController` provides API for the {@link ngModel `ngModel`} directive.
 * The controller contains services for data-binding, validation, CSS updates, and value formatting
 * and parsing. It purposefully does not contain any logic which deals with DOM rendering or
 * listening to DOM events.
 * Such DOM related logic should be provided by other directives which make use of
 * `NgModelController` for data-binding to control elements.
 * Angular provides this DOM logic for most {@link input `input`} elements.
 * At the end of this page you can find a {@link ngModel.NgModelController#custom-control-example
 * custom control example} that uses `ngModelController` to bind to `contenteditable` elements.
 *
 * @example
 * ### Custom Control Example
 * This example shows how to use `NgModelController` with a custom control to achieve
 * data-binding. Notice how different directives (`contenteditable`, `ng-model`, and `required`)
 * collaborate together to achieve the desired result.
 *
 * `contenteditable` is an HTML5 attribute, which tells the browser to let the element
 * contents be edited in place by the user.
 *
 * We are using the {@link ng.service:$sce $sce} service here and include the {@link ngSanitize $sanitize}
 * module to automatically remove "bad" content like inline event listener (e.g. `<span onclick="...">`).
 * However, as we are using `$sce` the model can still decide to provide unsafe content if it marks
 * that content using the `$sce` service.
 *
 * <example name="NgModelController" module="customControl" deps="angular-sanitize.js">
    <file name="style.css">
      [contenteditable] {
        border: 1px solid black;
        background-color: white;
        min-height: 20px;
      }

      .ng-invalid {
        border: 1px solid red;
      }

    </file>
    <file name="script.js">
      angular.module('customControl', ['ngSanitize']).
        directive('contenteditable', ['$sce', function($sce) {
          return {
            restrict: 'A', // only activate on element attribute
            require: '?ngModel', // get a hold of NgModelController
            link: function(scope, element, attrs, ngModel) {
              if (!ngModel) return; // do nothing if no ng-model

              // Specify how UI should be updated
              ngModel.$render = function() {
                element.html($sce.getTrustedHtml(ngModel.$viewValue || ''));
              };

              // Listen for change events to enable binding
              element.on('blur keyup change', function() {
                scope.$evalAsync(read);
              });
              read(); // initialize

              // Write data to the model
              function read() {
                var html = element.html();
                // When we clear the content editable the browser leaves a <br> behind
                // If strip-br attribute is provided then we strip this out
                if ( attrs.stripBr && html == '<br>' ) {
                  html = '';
                }
                ngModel.$setViewValue(html);
              }
            }
          };
        }]);
    </file>
    <file name="index.html">
      <form name="myForm">
       <div contenteditable
            name="myWidget" ng-model="userContent"
            strip-br="true"
            required>Change me!</div>
        <span ng-show="myForm.myWidget.$error.required">Required!</span>
       <hr>
       <textarea ng-model="userContent"></textarea>
      </form>
    </file>
    <file name="protractor.js" type="protractor">
    it('should data-bind and become invalid', function() {
      if (browser.params.browser == 'safari' || browser.params.browser == 'firefox') {
        // SafariDriver can't handle contenteditable
        // and Firefox driver can't clear contenteditables very well
        return;
      }
      var contentEditable = element(by.css('[contenteditable]'));
      var content = 'Change me!';

      expect(contentEditable.getText()).toEqual(content);

      contentEditable.clear();
      contentEditable.sendKeys(protractor.Key.BACK_SPACE);
      expect(contentEditable.getText()).toEqual('');
      expect(contentEditable.getAttribute('class')).toMatch(/ng-invalid-required/);
    });
    </file>
 * </example>
 *
 *
 */
  var NgModelController = [
      '$scope',
      '$exceptionHandler',
      '$attrs',
      '$element',
      '$parse',
      '$animate',
      '$timeout',
      '$rootScope',
      '$q',
      '$interpolate',
      function ($scope, $exceptionHandler, $attr, $element, $parse, $animate, $timeout, $rootScope, $q, $interpolate) {
        this.$viewValue = Number.NaN;
        this.$modelValue = Number.NaN;
        this.$$rawModelValue = undefined;
        // stores the parsed modelValue / model set from scope regardless of validity.
        this.$validators = {};
        this.$asyncValidators = {};
        this.$parsers = [];
        this.$formatters = [];
        this.$viewChangeListeners = [];
        this.$untouched = true;
        this.$touched = false;
        this.$pristine = true;
        this.$dirty = false;
        this.$valid = true;
        this.$invalid = false;
        this.$error = {};
        // keep invalid keys here
        this.$$success = {};
        // keep valid keys here
        this.$pending = undefined;
        // keep pending keys here
        this.$name = $interpolate($attr.name || '', false)($scope);
        var parsedNgModel = $parse($attr.ngModel), parsedNgModelAssign = parsedNgModel.assign, ngModelGet = parsedNgModel, ngModelSet = parsedNgModelAssign, pendingDebounce = null, parserValid, ctrl = this;
        this.$$setOptions = function (options) {
          ctrl.$options = options;
          if (options && options.getterSetter) {
            var invokeModelGetter = $parse($attr.ngModel + '()'), invokeModelSetter = $parse($attr.ngModel + '($$$p)');
            ngModelGet = function ($scope) {
              var modelValue = parsedNgModel($scope);
              if (isFunction(modelValue)) {
                modelValue = invokeModelGetter($scope);
              }
              return modelValue;
            };
            ngModelSet = function ($scope, newValue) {
              if (isFunction(parsedNgModel($scope))) {
                invokeModelSetter($scope, { $$$p: ctrl.$modelValue });
              } else {
                parsedNgModelAssign($scope, ctrl.$modelValue);
              }
            };
          } else if (!parsedNgModel.assign) {
            throw ngModelMinErr('nonassign', 'Expression \'{0}\' is non-assignable. Element: {1}', $attr.ngModel, startingTag($element));
          }
        };
        /**
   * @ngdoc method
   * @name ngModel.NgModelController#$render
   *
   * @description
   * Called when the view needs to be updated. It is expected that the user of the ng-model
   * directive will implement this method.
   *
   * The `$render()` method is invoked in the following situations:
   *
   * * `$rollbackViewValue()` is called.  If we are rolling back the view value to the last
   *   committed value then `$render()` is called to update the input control.
   * * The value referenced by `ng-model` is changed programmatically and both the `$modelValue` and
   *   the `$viewValue` are different to last time.
   *
   * Since `ng-model` does not do a deep watch, `$render()` is only invoked if the values of
   * `$modelValue` and `$viewValue` are actually different to their previous value. If `$modelValue`
   * or `$viewValue` are objects (rather than a string or number) then `$render()` will not be
   * invoked if you only change a property on the objects.
   */
        this.$render = noop;
        /**
   * @ngdoc method
   * @name ngModel.NgModelController#$isEmpty
   *
   * @description
   * This is called when we need to determine if the value of an input is empty.
   *
   * For instance, the required directive does this to work out if the input has data or not.
   *
   * The default `$isEmpty` function checks whether the value is `undefined`, `''`, `null` or `NaN`.
   *
   * You can override this for input directives whose concept of being empty is different to the
   * default. The `checkboxInputType` directive does this because in its case a value of `false`
   * implies empty.
   *
   * @param {*} value The value of the input to check for emptiness.
   * @returns {boolean} True if `value` is "empty".
   */
        this.$isEmpty = function (value) {
          return isUndefined(value) || value === '' || value === null || value !== value;
        };
        var parentForm = $element.inheritedData('$formController') || nullFormCtrl, currentValidationRunId = 0;
        /**
   * @ngdoc method
   * @name ngModel.NgModelController#$setValidity
   *
   * @description
   * Change the validity state, and notify the form.
   *
   * This method can be called within $parsers/$formatters or a custom validation implementation.
   * However, in most cases it should be sufficient to use the `ngModel.$validators` and
   * `ngModel.$asyncValidators` collections which will call `$setValidity` automatically.
   *
   * @param {string} validationErrorKey Name of the validator. The `validationErrorKey` will be assigned
   *        to either `$error[validationErrorKey]` or `$pending[validationErrorKey]`
   *        (for unfulfilled `$asyncValidators`), so that it is available for data-binding.
   *        The `validationErrorKey` should be in camelCase and will get converted into dash-case
   *        for class name. Example: `myError` will result in `ng-valid-my-error` and `ng-invalid-my-error`
   *        class and can be bound to as  `{{someForm.someControl.$error.myError}}` .
   * @param {boolean} isValid Whether the current state is valid (true), invalid (false), pending (undefined),
   *                          or skipped (null). Pending is used for unfulfilled `$asyncValidators`.
   *                          Skipped is used by Angular when validators do not run because of parse errors and
   *                          when `$asyncValidators` do not run because any of the `$validators` failed.
   */
        addSetValidityMethod({
          ctrl: this,
          $element: $element,
          set: function (object, property) {
            object[property] = true;
          },
          unset: function (object, property) {
            delete object[property];
          },
          parentForm: parentForm,
          $animate: $animate
        });
        /**
   * @ngdoc method
   * @name ngModel.NgModelController#$setPristine
   *
   * @description
   * Sets the control to its pristine state.
   *
   * This method can be called to remove the `ng-dirty` class and set the control to its pristine
   * state (`ng-pristine` class). A model is considered to be pristine when the control
   * has not been changed from when first compiled.
   */
        this.$setPristine = function () {
          ctrl.$dirty = false;
          ctrl.$pristine = true;
          $animate.removeClass($element, DIRTY_CLASS);
          $animate.addClass($element, PRISTINE_CLASS);
        };
        /**
   * @ngdoc method
   * @name ngModel.NgModelController#$setDirty
   *
   * @description
   * Sets the control to its dirty state.
   *
   * This method can be called to remove the `ng-pristine` class and set the control to its dirty
   * state (`ng-dirty` class). A model is considered to be dirty when the control has been changed
   * from when first compiled.
   */
        this.$setDirty = function () {
          ctrl.$dirty = true;
          ctrl.$pristine = false;
          $animate.removeClass($element, PRISTINE_CLASS);
          $animate.addClass($element, DIRTY_CLASS);
          parentForm.$setDirty();
        };
        /**
   * @ngdoc method
   * @name ngModel.NgModelController#$setUntouched
   *
   * @description
   * Sets the control to its untouched state.
   *
   * This method can be called to remove the `ng-touched` class and set the control to its
   * untouched state (`ng-untouched` class). Upon compilation, a model is set as untouched
   * by default, however this function can be used to restore that state if the model has
   * already been touched by the user.
   */
        this.$setUntouched = function () {
          ctrl.$touched = false;
          ctrl.$untouched = true;
          $animate.setClass($element, UNTOUCHED_CLASS, TOUCHED_CLASS);
        };
        /**
   * @ngdoc method
   * @name ngModel.NgModelController#$setTouched
   *
   * @description
   * Sets the control to its touched state.
   *
   * This method can be called to remove the `ng-untouched` class and set the control to its
   * touched state (`ng-touched` class). A model is considered to be touched when the user has
   * first focused the control element and then shifted focus away from the control (blur event).
   */
        this.$setTouched = function () {
          ctrl.$touched = true;
          ctrl.$untouched = false;
          $animate.setClass($element, TOUCHED_CLASS, UNTOUCHED_CLASS);
        };
        /**
   * @ngdoc method
   * @name ngModel.NgModelController#$rollbackViewValue
   *
   * @description
   * Cancel an update and reset the input element's value to prevent an update to the `$modelValue`,
   * which may be caused by a pending debounced event or because the input is waiting for a some
   * future event.
   *
   * If you have an input that uses `ng-model-options` to set up debounced events or events such
   * as blur you can have a situation where there is a period when the `$viewValue`
   * is out of synch with the ngModel's `$modelValue`.
   *
   * In this case, you can run into difficulties if you try to update the ngModel's `$modelValue`
   * programmatically before these debounced/future events have resolved/occurred, because Angular's
   * dirty checking mechanism is not able to tell whether the model has actually changed or not.
   *
   * The `$rollbackViewValue()` method should be called before programmatically changing the model of an
   * input which may have such events pending. This is important in order to make sure that the
   * input field will be updated with the new model value and any pending operations are cancelled.
   *
   * <example name="ng-model-cancel-update" module="cancel-update-example">
   *   <file name="app.js">
   *     angular.module('cancel-update-example', [])
   *
   *     .controller('CancelUpdateController', ['$scope', function($scope) {
   *       $scope.resetWithCancel = function(e) {
   *         if (e.keyCode == 27) {
   *           $scope.myForm.myInput1.$rollbackViewValue();
   *           $scope.myValue = '';
   *         }
   *       };
   *       $scope.resetWithoutCancel = function(e) {
   *         if (e.keyCode == 27) {
   *           $scope.myValue = '';
   *         }
   *       };
   *     }]);
   *   </file>
   *   <file name="index.html">
   *     <div ng-controller="CancelUpdateController">
   *       <p>Try typing something in each input.  See that the model only updates when you
   *          blur off the input.
   *        </p>
   *        <p>Now see what happens if you start typing then press the Escape key</p>
   *
   *       <form name="myForm" ng-model-options="{ updateOn: 'blur' }">
   *         <p>With $rollbackViewValue()</p>
   *         <input name="myInput1" ng-model="myValue" ng-keydown="resetWithCancel($event)"><br/>
   *         myValue: "{{ myValue }}"
   *
   *         <p>Without $rollbackViewValue()</p>
   *         <input name="myInput2" ng-model="myValue" ng-keydown="resetWithoutCancel($event)"><br/>
   *         myValue: "{{ myValue }}"
   *       </form>
   *     </div>
   *   </file>
   * </example>
   */
        this.$rollbackViewValue = function () {
          $timeout.cancel(pendingDebounce);
          ctrl.$viewValue = ctrl.$$lastCommittedViewValue;
          ctrl.$render();
        };
        /**
   * @ngdoc method
   * @name ngModel.NgModelController#$validate
   *
   * @description
   * Runs each of the registered validators (first synchronous validators and then
   * asynchronous validators).
   * If the validity changes to invalid, the model will be set to `undefined`,
   * unless {@link ngModelOptions `ngModelOptions.allowInvalid`} is `true`.
   * If the validity changes to valid, it will set the model to the last available valid
   * `$modelValue`, i.e. either the last parsed value or the last value set from the scope.
   */
        this.$validate = function () {
          // ignore $validate before model is initialized
          if (isNumber(ctrl.$modelValue) && isNaN(ctrl.$modelValue)) {
            return;
          }
          var viewValue = ctrl.$$lastCommittedViewValue;
          // Note: we use the $$rawModelValue as $modelValue might have been
          // set to undefined during a view -> model update that found validation
          // errors. We can't parse the view here, since that could change
          // the model although neither viewValue nor the model on the scope changed
          var modelValue = ctrl.$$rawModelValue;
          var prevValid = ctrl.$valid;
          var prevModelValue = ctrl.$modelValue;
          var allowInvalid = ctrl.$options && ctrl.$options.allowInvalid;
          ctrl.$$runValidators(modelValue, viewValue, function (allValid) {
            // If there was no change in validity, don't update the model
            // This prevents changing an invalid modelValue to undefined
            if (!allowInvalid && prevValid !== allValid) {
              // Note: Don't check ctrl.$valid here, as we could have
              // external validators (e.g. calculated on the server),
              // that just call $setValidity and need the model value
              // to calculate their validity.
              ctrl.$modelValue = allValid ? modelValue : undefined;
              if (ctrl.$modelValue !== prevModelValue) {
                ctrl.$$writeModelToScope();
              }
            }
          });
        };
        this.$$runValidators = function (modelValue, viewValue, doneCallback) {
          currentValidationRunId++;
          var localValidationRunId = currentValidationRunId;
          // check parser error
          if (!processParseErrors()) {
            validationDone(false);
            return;
          }
          if (!processSyncValidators()) {
            validationDone(false);
            return;
          }
          processAsyncValidators();
          function processParseErrors() {
            var errorKey = ctrl.$$parserName || 'parse';
            if (parserValid === undefined) {
              setValidity(errorKey, null);
            } else {
              if (!parserValid) {
                forEach(ctrl.$validators, function (v, name) {
                  setValidity(name, null);
                });
                forEach(ctrl.$asyncValidators, function (v, name) {
                  setValidity(name, null);
                });
              }
              // Set the parse error last, to prevent unsetting it, should a $validators key == parserName
              setValidity(errorKey, parserValid);
              return parserValid;
            }
            return true;
          }
          function processSyncValidators() {
            var syncValidatorsValid = true;
            forEach(ctrl.$validators, function (validator, name) {
              var result = validator(modelValue, viewValue);
              syncValidatorsValid = syncValidatorsValid && result;
              setValidity(name, result);
            });
            if (!syncValidatorsValid) {
              forEach(ctrl.$asyncValidators, function (v, name) {
                setValidity(name, null);
              });
              return false;
            }
            return true;
          }
          function processAsyncValidators() {
            var validatorPromises = [];
            var allValid = true;
            forEach(ctrl.$asyncValidators, function (validator, name) {
              var promise = validator(modelValue, viewValue);
              if (!isPromiseLike(promise)) {
                throw ngModelMinErr('$asyncValidators', 'Expected asynchronous validator to return a promise but got \'{0}\' instead.', promise);
              }
              setValidity(name, undefined);
              validatorPromises.push(promise.then(function () {
                setValidity(name, true);
              }, function (error) {
                allValid = false;
                setValidity(name, false);
              }));
            });
            if (!validatorPromises.length) {
              validationDone(true);
            } else {
              $q.all(validatorPromises).then(function () {
                validationDone(allValid);
              }, noop);
            }
          }
          function setValidity(name, isValid) {
            if (localValidationRunId === currentValidationRunId) {
              ctrl.$setValidity(name, isValid);
            }
          }
          function validationDone(allValid) {
            if (localValidationRunId === currentValidationRunId) {
              doneCallback(allValid);
            }
          }
        };
        /**
   * @ngdoc method
   * @name ngModel.NgModelController#$commitViewValue
   *
   * @description
   * Commit a pending update to the `$modelValue`.
   *
   * Updates may be pending by a debounced event or because the input is waiting for a some future
   * event defined in `ng-model-options`. this method is rarely needed as `NgModelController`
   * usually handles calling this in response to input events.
   */
        this.$commitViewValue = function () {
          var viewValue = ctrl.$viewValue;
          $timeout.cancel(pendingDebounce);
          // If the view value has not changed then we should just exit, except in the case where there is
          // a native validator on the element. In this case the validation state may have changed even though
          // the viewValue has stayed empty.
          if (ctrl.$$lastCommittedViewValue === viewValue && (viewValue !== '' || !ctrl.$$hasNativeValidators)) {
            return;
          }
          ctrl.$$lastCommittedViewValue = viewValue;
          // change to dirty
          if (ctrl.$pristine) {
            this.$setDirty();
          }
          this.$$parseAndValidate();
        };
        this.$$parseAndValidate = function () {
          var viewValue = ctrl.$$lastCommittedViewValue;
          var modelValue = viewValue;
          parserValid = isUndefined(modelValue) ? undefined : true;
          if (parserValid) {
            for (var i = 0; i < ctrl.$parsers.length; i++) {
              modelValue = ctrl.$parsers[i](modelValue);
              if (isUndefined(modelValue)) {
                parserValid = false;
                break;
              }
            }
          }
          if (isNumber(ctrl.$modelValue) && isNaN(ctrl.$modelValue)) {
            // ctrl.$modelValue has not been touched yet...
            ctrl.$modelValue = ngModelGet($scope);
          }
          var prevModelValue = ctrl.$modelValue;
          var allowInvalid = ctrl.$options && ctrl.$options.allowInvalid;
          ctrl.$$rawModelValue = modelValue;
          if (allowInvalid) {
            ctrl.$modelValue = modelValue;
            writeToModelIfNeeded();
          }
          // Pass the $$lastCommittedViewValue here, because the cached viewValue might be out of date.
          // This can happen if e.g. $setViewValue is called from inside a parser
          ctrl.$$runValidators(modelValue, ctrl.$$lastCommittedViewValue, function (allValid) {
            if (!allowInvalid) {
              // Note: Don't check ctrl.$valid here, as we could have
              // external validators (e.g. calculated on the server),
              // that just call $setValidity and need the model value
              // to calculate their validity.
              ctrl.$modelValue = allValid ? modelValue : undefined;
              writeToModelIfNeeded();
            }
          });
          function writeToModelIfNeeded() {
            if (ctrl.$modelValue !== prevModelValue) {
              ctrl.$$writeModelToScope();
            }
          }
        };
        this.$$writeModelToScope = function () {
          ngModelSet($scope, ctrl.$modelValue);
          forEach(ctrl.$viewChangeListeners, function (listener) {
            try {
              listener();
            } catch (e) {
              $exceptionHandler(e);
            }
          });
        };
        /**
   * @ngdoc method
   * @name ngModel.NgModelController#$setViewValue
   *
   * @description
   * Update the view value.
   *
   * This method should be called when an input directive want to change the view value; typically,
   * this is done from within a DOM event handler.
   *
   * For example {@link ng.directive:input input} calls it when the value of the input changes and
   * {@link ng.directive:select select} calls it when an option is selected.
   *
   * If the new `value` is an object (rather than a string or a number), we should make a copy of the
   * object before passing it to `$setViewValue`.  This is because `ngModel` does not perform a deep
   * watch of objects, it only looks for a change of identity. If you only change the property of
   * the object then ngModel will not realise that the object has changed and will not invoke the
   * `$parsers` and `$validators` pipelines.
   *
   * For this reason, you should not change properties of the copy once it has been passed to
   * `$setViewValue`. Otherwise you may cause the model value on the scope to change incorrectly.
   *
   * When this method is called, the new `value` will be staged for committing through the `$parsers`
   * and `$validators` pipelines. If there are no special {@link ngModelOptions} specified then the staged
   * value sent directly for processing, finally to be applied to `$modelValue` and then the
   * **expression** specified in the `ng-model` attribute.
   *
   * Lastly, all the registered change listeners, in the `$viewChangeListeners` list, are called.
   *
   * In case the {@link ng.directive:ngModelOptions ngModelOptions} directive is used with `updateOn`
   * and the `default` trigger is not listed, all those actions will remain pending until one of the
   * `updateOn` events is triggered on the DOM element.
   * All these actions will be debounced if the {@link ng.directive:ngModelOptions ngModelOptions}
   * directive is used with a custom debounce for this particular event.
   *
   * Note that calling this function does not trigger a `$digest`.
   *
   * @param {string} value Value from the view.
   * @param {string} trigger Event that triggered the update.
   */
        this.$setViewValue = function (value, trigger) {
          ctrl.$viewValue = value;
          if (!ctrl.$options || ctrl.$options.updateOnDefault) {
            ctrl.$$debounceViewValueCommit(trigger);
          }
        };
        this.$$debounceViewValueCommit = function (trigger) {
          var debounceDelay = 0, options = ctrl.$options, debounce;
          if (options && isDefined(options.debounce)) {
            debounce = options.debounce;
            if (isNumber(debounce)) {
              debounceDelay = debounce;
            } else if (isNumber(debounce[trigger])) {
              debounceDelay = debounce[trigger];
            } else if (isNumber(debounce['default'])) {
              debounceDelay = debounce['default'];
            }
          }
          $timeout.cancel(pendingDebounce);
          if (debounceDelay) {
            pendingDebounce = $timeout(function () {
              ctrl.$commitViewValue();
            }, debounceDelay);
          } else if ($rootScope.$$phase) {
            ctrl.$commitViewValue();
          } else {
            $scope.$apply(function () {
              ctrl.$commitViewValue();
            });
          }
        };
        // model -> value
        // Note: we cannot use a normal scope.$watch as we want to detect the following:
        // 1. scope value is 'a'
        // 2. user enters 'b'
        // 3. ng-change kicks in and reverts scope value to 'a'
        //    -> scope value did not change since the last digest as
        //       ng-change executes in apply phase
        // 4. view should be changed back to 'a'
        $scope.$watch(function ngModelWatch() {
          var modelValue = ngModelGet($scope);
          // if scope model value and ngModel value are out of sync
          // TODO(perf): why not move this to the action fn?
          if (modelValue !== ctrl.$modelValue && (ctrl.$modelValue === ctrl.$modelValue || modelValue === modelValue)) {
            ctrl.$modelValue = ctrl.$$rawModelValue = modelValue;
            parserValid = undefined;
            var formatters = ctrl.$formatters, idx = formatters.length;
            var viewValue = modelValue;
            while (idx--) {
              viewValue = formatters[idx](viewValue);
            }
            if (ctrl.$viewValue !== viewValue) {
              ctrl.$viewValue = ctrl.$$lastCommittedViewValue = viewValue;
              ctrl.$render();
              ctrl.$$runValidators(modelValue, viewValue, noop);
            }
          }
          return modelValue;
        });
      }
    ];
  /**
 * @ngdoc directive
 * @name ngModel
 *
 * @element input
 * @priority 1
 *
 * @description
 * The `ngModel` directive binds an `input`,`select`, `textarea` (or custom form control) to a
 * property on the scope using {@link ngModel.NgModelController NgModelController},
 * which is created and exposed by this directive.
 *
 * `ngModel` is responsible for:
 *
 * - Binding the view into the model, which other directives such as `input`, `textarea` or `select`
 *   require.
 * - Providing validation behavior (i.e. required, number, email, url).
 * - Keeping the state of the control (valid/invalid, dirty/pristine, touched/untouched, validation errors).
 * - Setting related css classes on the element (`ng-valid`, `ng-invalid`, `ng-dirty`, `ng-pristine`, `ng-touched`, `ng-untouched`) including animations.
 * - Registering the control with its parent {@link ng.directive:form form}.
 *
 * Note: `ngModel` will try to bind to the property given by evaluating the expression on the
 * current scope. If the property doesn't already exist on this scope, it will be created
 * implicitly and added to the scope.
 *
 * For best practices on using `ngModel`, see:
 *
 *  - [Understanding Scopes](https://github.com/angular/angular.js/wiki/Understanding-Scopes)
 *
 * For basic examples, how to use `ngModel`, see:
 *
 *  - {@link ng.directive:input input}
 *    - {@link input[text] text}
 *    - {@link input[checkbox] checkbox}
 *    - {@link input[radio] radio}
 *    - {@link input[number] number}
 *    - {@link input[email] email}
 *    - {@link input[url] url}
 *    - {@link input[date] date}
 *    - {@link input[datetime-local] datetime-local}
 *    - {@link input[time] time}
 *    - {@link input[month] month}
 *    - {@link input[week] week}
 *  - {@link ng.directive:select select}
 *  - {@link ng.directive:textarea textarea}
 *
 * # CSS classes
 * The following CSS classes are added and removed on the associated input/select/textarea element
 * depending on the validity of the model.
 *
 *  - `ng-valid`: the model is valid
 *  - `ng-invalid`: the model is invalid
 *  - `ng-valid-[key]`: for each valid key added by `$setValidity`
 *  - `ng-invalid-[key]`: for each invalid key added by `$setValidity`
 *  - `ng-pristine`: the control hasn't been interacted with yet
 *  - `ng-dirty`: the control has been interacted with
 *  - `ng-touched`: the control has been blurred
 *  - `ng-untouched`: the control hasn't been blurred
 *  - `ng-pending`: any `$asyncValidators` are unfulfilled
 *
 * Keep in mind that ngAnimate can detect each of these classes when added and removed.
 *
 * ## Animation Hooks
 *
 * Animations within models are triggered when any of the associated CSS classes are added and removed
 * on the input element which is attached to the model. These classes are: `.ng-pristine`, `.ng-dirty`,
 * `.ng-invalid` and `.ng-valid` as well as any other validations that are performed on the model itself.
 * The animations that are triggered within ngModel are similar to how they work in ngClass and
 * animations can be hooked into using CSS transitions, keyframes as well as JS animations.
 *
 * The following example shows a simple way to utilize CSS transitions to style an input element
 * that has been rendered as invalid after it has been validated:
 *
 * <pre>
 * //be sure to include ngAnimate as a module to hook into more
 * //advanced animations
 * .my-input {
 *   transition:0.5s linear all;
 *   background: white;
 * }
 * .my-input.ng-invalid {
 *   background: red;
 *   color:white;
 * }
 * </pre>
 *
 * @example
 * <example deps="angular-animate.js" animations="true" fixBase="true" module="inputExample">
     <file name="index.html">
       <script>
        angular.module('inputExample', [])
          .controller('ExampleController', ['$scope', function($scope) {
            $scope.val = '1';
          }]);
       </script>
       <style>
         .my-input {
           -webkit-transition:all linear 0.5s;
           transition:all linear 0.5s;
           background: transparent;
         }
         .my-input.ng-invalid {
           color:white;
           background: red;
         }
       </style>
       Update input to see transitions when valid/invalid.
       Integer is a valid value.
       <form name="testForm" ng-controller="ExampleController">
         <input ng-model="val" ng-pattern="/^\d+$/" name="anim" class="my-input" />
       </form>
     </file>
 * </example>
 *
 * ## Binding to a getter/setter
 *
 * Sometimes it's helpful to bind `ngModel` to a getter/setter function.  A getter/setter is a
 * function that returns a representation of the model when called with zero arguments, and sets
 * the internal state of a model when called with an argument. It's sometimes useful to use this
 * for models that have an internal representation that's different than what the model exposes
 * to the view.
 *
 * <div class="alert alert-success">
 * **Best Practice:** It's best to keep getters fast because Angular is likely to call them more
 * frequently than other parts of your code.
 * </div>
 *
 * You use this behavior by adding `ng-model-options="{ getterSetter: true }"` to an element that
 * has `ng-model` attached to it. You can also add `ng-model-options="{ getterSetter: true }"` to
 * a `<form>`, which will enable this behavior for all `<input>`s within it. See
 * {@link ng.directive:ngModelOptions `ngModelOptions`} for more.
 *
 * The following example shows how to use `ngModel` with a getter/setter:
 *
 * @example
 * <example name="ngModel-getter-setter" module="getterSetterExample">
     <file name="index.html">
       <div ng-controller="ExampleController">
         <form name="userForm">
           Name:
           <input type="text" name="userName"
                  ng-model="user.name"
                  ng-model-options="{ getterSetter: true }" />
         </form>
         <pre>user.name = <span ng-bind="user.name()"></span></pre>
       </div>
     </file>
     <file name="app.js">
       angular.module('getterSetterExample', [])
         .controller('ExampleController', ['$scope', function($scope) {
           var _name = 'Brian';
           $scope.user = {
             name: function(newName) {
              // Note that newName can be undefined for two reasons:
              // 1. Because it is called as a getter and thus called with no arguments
              // 2. Because the property should actually be set to undefined. This happens e.g. if the
              //    input is invalid
              return arguments.length ? (_name = newName) : _name;
             }
           };
         }]);
     </file>
 * </example>
 */
  var ngModelDirective = [
      '$rootScope',
      function ($rootScope) {
        return {
          restrict: 'A',
          require: [
            'ngModel',
            '^?form',
            '^?ngModelOptions'
          ],
          controller: NgModelController,
          priority: 1,
          compile: function ngModelCompile(element) {
            // Setup initial state of the control
            element.addClass(PRISTINE_CLASS).addClass(UNTOUCHED_CLASS).addClass(VALID_CLASS);
            return {
              pre: function ngModelPreLink(scope, element, attr, ctrls) {
                var modelCtrl = ctrls[0], formCtrl = ctrls[1] || nullFormCtrl;
                modelCtrl.$$setOptions(ctrls[2] && ctrls[2].$options);
                // notify others, especially parent forms
                formCtrl.$addControl(modelCtrl);
                attr.$observe('name', function (newValue) {
                  if (modelCtrl.$name !== newValue) {
                    formCtrl.$$renameControl(modelCtrl, newValue);
                  }
                });
                scope.$on('$destroy', function () {
                  formCtrl.$removeControl(modelCtrl);
                });
              },
              post: function ngModelPostLink(scope, element, attr, ctrls) {
                var modelCtrl = ctrls[0];
                if (modelCtrl.$options && modelCtrl.$options.updateOn) {
                  element.on(modelCtrl.$options.updateOn, function (ev) {
                    modelCtrl.$$debounceViewValueCommit(ev && ev.type);
                  });
                }
                element.on('blur', function (ev) {
                  if (modelCtrl.$touched)
                    return;
                  if ($rootScope.$$phase) {
                    scope.$evalAsync(modelCtrl.$setTouched);
                  } else {
                    scope.$apply(modelCtrl.$setTouched);
                  }
                });
              }
            };
          }
        };
      }
    ];
  var DEFAULT_REGEXP = /(\s+|^)default(\s+|$)/;
  /**
 * @ngdoc directive
 * @name ngModelOptions
 *
 * @description
 * Allows tuning how model updates are done. Using `ngModelOptions` you can specify a custom list of
 * events that will trigger a model update and/or a debouncing delay so that the actual update only
 * takes place when a timer expires; this timer will be reset after another change takes place.
 *
 * Given the nature of `ngModelOptions`, the value displayed inside input fields in the view might
 * be different than the value in the actual model. This means that if you update the model you
 * should also invoke {@link ngModel.NgModelController `$rollbackViewValue`} on the relevant input field in
 * order to make sure it is synchronized with the model and that any debounced action is canceled.
 *
 * The easiest way to reference the control's {@link ngModel.NgModelController `$rollbackViewValue`}
 * method is by making sure the input is placed inside a form that has a `name` attribute. This is
 * important because `form` controllers are published to the related scope under the name in their
 * `name` attribute.
 *
 * Any pending changes will take place immediately when an enclosing form is submitted via the
 * `submit` event. Note that `ngClick` events will occur before the model is updated. Use `ngSubmit`
 * to have access to the updated model.
 *
 * `ngModelOptions` has an effect on the element it's declared on and its descendants.
 *
 * @param {Object} ngModelOptions options to apply to the current model. Valid keys are:
 *   - `updateOn`: string specifying which event should the input be bound to. You can set several
 *     events using an space delimited list. There is a special event called `default` that
 *     matches the default events belonging of the control.
 *   - `debounce`: integer value which contains the debounce model update value in milliseconds. A
 *     value of 0 triggers an immediate update. If an object is supplied instead, you can specify a
 *     custom value for each event. For example:
 *     `ng-model-options="{ updateOn: 'default blur', debounce: {'default': 500, 'blur': 0} }"`
 *   - `allowInvalid`: boolean value which indicates that the model can be set with values that did
 *     not validate correctly instead of the default behavior of setting the model to undefined.
 *   - `getterSetter`: boolean value which determines whether or not to treat functions bound to
       `ngModel` as getters/setters.
 *   - `timezone`: Defines the timezone to be used to read/write the `Date` instance in the model for
 *     `<input type="date">`, `<input type="time">`, ... . Right now, the only supported value is `'UTC'`,
 *     otherwise the default timezone of the browser will be used.
 *
 * @example

  The following example shows how to override immediate updates. Changes on the inputs within the
  form will update the model only when the control loses focus (blur event). If `escape` key is
  pressed while the input field is focused, the value is reset to the value in the current model.

  <example name="ngModelOptions-directive-blur" module="optionsExample">
    <file name="index.html">
      <div ng-controller="ExampleController">
        <form name="userForm">
          Name:
          <input type="text" name="userName"
                 ng-model="user.name"
                 ng-model-options="{ updateOn: 'blur' }"
                 ng-keyup="cancel($event)" /><br />

          Other data:
          <input type="text" ng-model="user.data" /><br />
        </form>
        <pre>user.name = <span ng-bind="user.name"></span></pre>
      </div>
    </file>
    <file name="app.js">
      angular.module('optionsExample', [])
        .controller('ExampleController', ['$scope', function($scope) {
          $scope.user = { name: 'say', data: '' };

          $scope.cancel = function(e) {
            if (e.keyCode == 27) {
              $scope.userForm.userName.$rollbackViewValue();
            }
          };
        }]);
    </file>
    <file name="protractor.js" type="protractor">
      var model = element(by.binding('user.name'));
      var input = element(by.model('user.name'));
      var other = element(by.model('user.data'));

      it('should allow custom events', function() {
        input.sendKeys(' hello');
        input.click();
        expect(model.getText()).toEqual('say');
        other.click();
        expect(model.getText()).toEqual('say hello');
      });

      it('should $rollbackViewValue when model changes', function() {
        input.sendKeys(' hello');
        expect(input.getAttribute('value')).toEqual('say hello');
        input.sendKeys(protractor.Key.ESCAPE);
        expect(input.getAttribute('value')).toEqual('say');
        other.click();
        expect(model.getText()).toEqual('say');
      });
    </file>
  </example>

  This one shows how to debounce model changes. Model will be updated only 1 sec after last change.
  If the `Clear` button is pressed, any debounced action is canceled and the value becomes empty.

  <example name="ngModelOptions-directive-debounce" module="optionsExample">
    <file name="index.html">
      <div ng-controller="ExampleController">
        <form name="userForm">
          Name:
          <input type="text" name="userName"
                 ng-model="user.name"
                 ng-model-options="{ debounce: 1000 }" />
          <button ng-click="userForm.userName.$rollbackViewValue(); user.name=''">Clear</button><br />
        </form>
        <pre>user.name = <span ng-bind="user.name"></span></pre>
      </div>
    </file>
    <file name="app.js">
      angular.module('optionsExample', [])
        .controller('ExampleController', ['$scope', function($scope) {
          $scope.user = { name: 'say' };
        }]);
    </file>
  </example>

  This one shows how to bind to getter/setters:

  <example name="ngModelOptions-directive-getter-setter" module="getterSetterExample">
    <file name="index.html">
      <div ng-controller="ExampleController">
        <form name="userForm">
          Name:
          <input type="text" name="userName"
                 ng-model="user.name"
                 ng-model-options="{ getterSetter: true }" />
        </form>
        <pre>user.name = <span ng-bind="user.name()"></span></pre>
      </div>
    </file>
    <file name="app.js">
      angular.module('getterSetterExample', [])
        .controller('ExampleController', ['$scope', function($scope) {
          var _name = 'Brian';
          $scope.user = {
            name: function(newName) {
              // Note that newName can be undefined for two reasons:
              // 1. Because it is called as a getter and thus called with no arguments
              // 2. Because the property should actually be set to undefined. This happens e.g. if the
              //    input is invalid
              return arguments.length ? (_name = newName) : _name;
            }
          };
        }]);
    </file>
  </example>
 */
  var ngModelOptionsDirective = function () {
    return {
      restrict: 'A',
      controller: [
        '$scope',
        '$attrs',
        function ($scope, $attrs) {
          var that = this;
          this.$options = $scope.$eval($attrs.ngModelOptions);
          // Allow adding/overriding bound events
          if (this.$options.updateOn !== undefined) {
            this.$options.updateOnDefault = false;
            // extract "default" pseudo-event from list of events that can trigger a model update
            this.$options.updateOn = trim(this.$options.updateOn.replace(DEFAULT_REGEXP, function () {
              that.$options.updateOnDefault = true;
              return ' ';
            }));
          } else {
            this.$options.updateOnDefault = true;
          }
        }
      ]
    };
  };
  // helper methods
  function addSetValidityMethod(context) {
    var ctrl = context.ctrl, $element = context.$element, classCache = {}, set = context.set, unset = context.unset, parentForm = context.parentForm, $animate = context.$animate;
    classCache[INVALID_CLASS] = !(classCache[VALID_CLASS] = $element.hasClass(VALID_CLASS));
    ctrl.$setValidity = setValidity;
    function setValidity(validationErrorKey, state, controller) {
      if (state === undefined) {
        createAndSet('$pending', validationErrorKey, controller);
      } else {
        unsetAndCleanup('$pending', validationErrorKey, controller);
      }
      if (!isBoolean(state)) {
        unset(ctrl.$error, validationErrorKey, controller);
        unset(ctrl.$$success, validationErrorKey, controller);
      } else {
        if (state) {
          unset(ctrl.$error, validationErrorKey, controller);
          set(ctrl.$$success, validationErrorKey, controller);
        } else {
          set(ctrl.$error, validationErrorKey, controller);
          unset(ctrl.$$success, validationErrorKey, controller);
        }
      }
      if (ctrl.$pending) {
        cachedToggleClass(PENDING_CLASS, true);
        ctrl.$valid = ctrl.$invalid = undefined;
        toggleValidationCss('', null);
      } else {
        cachedToggleClass(PENDING_CLASS, false);
        ctrl.$valid = isObjectEmpty(ctrl.$error);
        ctrl.$invalid = !ctrl.$valid;
        toggleValidationCss('', ctrl.$valid);
      }
      // re-read the state as the set/unset methods could have
      // combined state in ctrl.$error[validationError] (used for forms),
      // where setting/unsetting only increments/decrements the value,
      // and does not replace it.
      var combinedState;
      if (ctrl.$pending && ctrl.$pending[validationErrorKey]) {
        combinedState = undefined;
      } else if (ctrl.$error[validationErrorKey]) {
        combinedState = false;
      } else if (ctrl.$$success[validationErrorKey]) {
        combinedState = true;
      } else {
        combinedState = null;
      }
      toggleValidationCss(validationErrorKey, combinedState);
      parentForm.$setValidity(validationErrorKey, combinedState, ctrl);
    }
    function createAndSet(name, value, controller) {
      if (!ctrl[name]) {
        ctrl[name] = {};
      }
      set(ctrl[name], value, controller);
    }
    function unsetAndCleanup(name, value, controller) {
      if (ctrl[name]) {
        unset(ctrl[name], value, controller);
      }
      if (isObjectEmpty(ctrl[name])) {
        ctrl[name] = undefined;
      }
    }
    function cachedToggleClass(className, switchValue) {
      if (switchValue && !classCache[className]) {
        $animate.addClass($element, className);
        classCache[className] = true;
      } else if (!switchValue && classCache[className]) {
        $animate.removeClass($element, className);
        classCache[className] = false;
      }
    }
    function toggleValidationCss(validationErrorKey, isValid) {
      validationErrorKey = validationErrorKey ? '-' + snake_case(validationErrorKey, '-') : '';
      cachedToggleClass(VALID_CLASS + validationErrorKey, isValid === true);
      cachedToggleClass(INVALID_CLASS + validationErrorKey, isValid === false);
    }
  }
  function isObjectEmpty(obj) {
    if (obj) {
      for (var prop in obj) {
        return false;
      }
    }
    return true;
  }
  /**
 * @ngdoc directive
 * @name ngNonBindable
 * @restrict AC
 * @priority 1000
 *
 * @description
 * The `ngNonBindable` directive tells Angular not to compile or bind the contents of the current
 * DOM element. This is useful if the element contains what appears to be Angular directives and
 * bindings but which should be ignored by Angular. This could be the case if you have a site that
 * displays snippets of code, for instance.
 *
 * @element ANY
 *
 * @example
 * In this example there are two locations where a simple interpolation binding (`{{}}`) is present,
 * but the one wrapped in `ngNonBindable` is left alone.
 *
 * @example
    <example>
      <file name="index.html">
        <div>Normal: {{1 + 2}}</div>
        <div ng-non-bindable>Ignored: {{1 + 2}}</div>
      </file>
      <file name="protractor.js" type="protractor">
       it('should check ng-non-bindable', function() {
         expect(element(by.binding('1 + 2')).getText()).toContain('3');
         expect(element.all(by.css('div')).last().getText()).toMatch(/1 \+ 2/);
       });
      </file>
    </example>
 */
  var ngNonBindableDirective = ngDirective({
      terminal: true,
      priority: 1000
    });
  /**
 * @ngdoc directive
 * @name ngPluralize
 * @restrict EA
 *
 * @description
 * `ngPluralize` is a directive that displays messages according to en-US localization rules.
 * These rules are bundled with angular.js, but can be overridden
 * (see {@link guide/i18n Angular i18n} dev guide). You configure ngPluralize directive
 * by specifying the mappings between
 * [plural categories](http://unicode.org/repos/cldr-tmp/trunk/diff/supplemental/language_plural_rules.html)
 * and the strings to be displayed.
 *
 * # Plural categories and explicit number rules
 * There are two
 * [plural categories](http://unicode.org/repos/cldr-tmp/trunk/diff/supplemental/language_plural_rules.html)
 * in Angular's default en-US locale: "one" and "other".
 *
 * While a plural category may match many numbers (for example, in en-US locale, "other" can match
 * any number that is not 1), an explicit number rule can only match one number. For example, the
 * explicit number rule for "3" matches the number 3. There are examples of plural categories
 * and explicit number rules throughout the rest of this documentation.
 *
 * # Configuring ngPluralize
 * You configure ngPluralize by providing 2 attributes: `count` and `when`.
 * You can also provide an optional attribute, `offset`.
 *
 * The value of the `count` attribute can be either a string or an {@link guide/expression
 * Angular expression}; these are evaluated on the current scope for its bound value.
 *
 * The `when` attribute specifies the mappings between plural categories and the actual
 * string to be displayed. The value of the attribute should be a JSON object.
 *
 * The following example shows how to configure ngPluralize:
 *
 * ```html
 * <ng-pluralize count="personCount"
                 when="{'0': 'Nobody is viewing.',
 *                      'one': '1 person is viewing.',
 *                      'other': '{} people are viewing.'}">
 * </ng-pluralize>
 *```
 *
 * In the example, `"0: Nobody is viewing."` is an explicit number rule. If you did not
 * specify this rule, 0 would be matched to the "other" category and "0 people are viewing"
 * would be shown instead of "Nobody is viewing". You can specify an explicit number rule for
 * other numbers, for example 12, so that instead of showing "12 people are viewing", you can
 * show "a dozen people are viewing".
 *
 * You can use a set of closed braces (`{}`) as a placeholder for the number that you want substituted
 * into pluralized strings. In the previous example, Angular will replace `{}` with
 * <span ng-non-bindable>`{{personCount}}`</span>. The closed braces `{}` is a placeholder
 * for <span ng-non-bindable>{{numberExpression}}</span>.
 *
 * # Configuring ngPluralize with offset
 * The `offset` attribute allows further customization of pluralized text, which can result in
 * a better user experience. For example, instead of the message "4 people are viewing this document",
 * you might display "John, Kate and 2 others are viewing this document".
 * The offset attribute allows you to offset a number by any desired value.
 * Let's take a look at an example:
 *
 * ```html
 * <ng-pluralize count="personCount" offset=2
 *               when="{'0': 'Nobody is viewing.',
 *                      '1': '{{person1}} is viewing.',
 *                      '2': '{{person1}} and {{person2}} are viewing.',
 *                      'one': '{{person1}}, {{person2}} and one other person are viewing.',
 *                      'other': '{{person1}}, {{person2}} and {} other people are viewing.'}">
 * </ng-pluralize>
 * ```
 *
 * Notice that we are still using two plural categories(one, other), but we added
 * three explicit number rules 0, 1 and 2.
 * When one person, perhaps John, views the document, "John is viewing" will be shown.
 * When three people view the document, no explicit number rule is found, so
 * an offset of 2 is taken off 3, and Angular uses 1 to decide the plural category.
 * In this case, plural category 'one' is matched and "John, Mary and one other person are viewing"
 * is shown.
 *
 * Note that when you specify offsets, you must provide explicit number rules for
 * numbers from 0 up to and including the offset. If you use an offset of 3, for example,
 * you must provide explicit number rules for 0, 1, 2 and 3. You must also provide plural strings for
 * plural categories "one" and "other".
 *
 * @param {string|expression} count The variable to be bound to.
 * @param {string} when The mapping between plural category to its corresponding strings.
 * @param {number=} offset Offset to deduct from the total number.
 *
 * @example
    <example module="pluralizeExample">
      <file name="index.html">
        <script>
          angular.module('pluralizeExample', [])
            .controller('ExampleController', ['$scope', function($scope) {
              $scope.person1 = 'Igor';
              $scope.person2 = 'Misko';
              $scope.personCount = 1;
            }]);
        </script>
        <div ng-controller="ExampleController">
          Person 1:<input type="text" ng-model="person1" value="Igor" /><br/>
          Person 2:<input type="text" ng-model="person2" value="Misko" /><br/>
          Number of People:<input type="text" ng-model="personCount" value="1" /><br/>

          <!--- Example with simple pluralization rules for en locale --->
          Without Offset:
          <ng-pluralize count="personCount"
                        when="{'0': 'Nobody is viewing.',
                               'one': '1 person is viewing.',
                               'other': '{} people are viewing.'}">
          </ng-pluralize><br>

          <!--- Example with offset --->
          With Offset(2):
          <ng-pluralize count="personCount" offset=2
                        when="{'0': 'Nobody is viewing.',
                               '1': '{{person1}} is viewing.',
                               '2': '{{person1}} and {{person2}} are viewing.',
                               'one': '{{person1}}, {{person2}} and one other person are viewing.',
                               'other': '{{person1}}, {{person2}} and {} other people are viewing.'}">
          </ng-pluralize>
        </div>
      </file>
      <file name="protractor.js" type="protractor">
        it('should show correct pluralized string', function() {
          var withoutOffset = element.all(by.css('ng-pluralize')).get(0);
          var withOffset = element.all(by.css('ng-pluralize')).get(1);
          var countInput = element(by.model('personCount'));

          expect(withoutOffset.getText()).toEqual('1 person is viewing.');
          expect(withOffset.getText()).toEqual('Igor is viewing.');

          countInput.clear();
          countInput.sendKeys('0');

          expect(withoutOffset.getText()).toEqual('Nobody is viewing.');
          expect(withOffset.getText()).toEqual('Nobody is viewing.');

          countInput.clear();
          countInput.sendKeys('2');

          expect(withoutOffset.getText()).toEqual('2 people are viewing.');
          expect(withOffset.getText()).toEqual('Igor and Misko are viewing.');

          countInput.clear();
          countInput.sendKeys('3');

          expect(withoutOffset.getText()).toEqual('3 people are viewing.');
          expect(withOffset.getText()).toEqual('Igor, Misko and one other person are viewing.');

          countInput.clear();
          countInput.sendKeys('4');

          expect(withoutOffset.getText()).toEqual('4 people are viewing.');
          expect(withOffset.getText()).toEqual('Igor, Misko and 2 other people are viewing.');
        });
        it('should show data-bound names', function() {
          var withOffset = element.all(by.css('ng-pluralize')).get(1);
          var personCount = element(by.model('personCount'));
          var person1 = element(by.model('person1'));
          var person2 = element(by.model('person2'));
          personCount.clear();
          personCount.sendKeys('4');
          person1.clear();
          person1.sendKeys('Di');
          person2.clear();
          person2.sendKeys('Vojta');
          expect(withOffset.getText()).toEqual('Di, Vojta and 2 other people are viewing.');
        });
      </file>
    </example>
 */
  var ngPluralizeDirective = [
      '$locale',
      '$interpolate',
      function ($locale, $interpolate) {
        var BRACE = /{}/g, IS_WHEN = /^when(Minus)?(.+)$/;
        return {
          restrict: 'EA',
          link: function (scope, element, attr) {
            var numberExp = attr.count, whenExp = attr.$attr.when && element.attr(attr.$attr.when),
              // we have {{}} in attrs
              offset = attr.offset || 0, whens = scope.$eval(whenExp) || {}, whensExpFns = {}, startSymbol = $interpolate.startSymbol(), endSymbol = $interpolate.endSymbol(), braceReplacement = startSymbol + numberExp + '-' + offset + endSymbol, watchRemover = angular.noop, lastCount;
            forEach(attr, function (expression, attributeName) {
              var tmpMatch = IS_WHEN.exec(attributeName);
              if (tmpMatch) {
                var whenKey = (tmpMatch[1] ? '-' : '') + lowercase(tmpMatch[2]);
                whens[whenKey] = element.attr(attr.$attr[attributeName]);
              }
            });
            forEach(whens, function (expression, key) {
              whensExpFns[key] = $interpolate(expression.replace(BRACE, braceReplacement));
            });
            scope.$watch(numberExp, function ngPluralizeWatchAction(newVal) {
              var count = parseFloat(newVal);
              var countIsNaN = isNaN(count);
              if (!countIsNaN && !(count in whens)) {
                // If an explicit number rule such as 1, 2, 3... is defined, just use it.
                // Otherwise, check it against pluralization rules in $locale service.
                count = $locale.pluralCat(count - offset);
              }
              // If both `count` and `lastCount` are NaN, we don't need to re-register a watch.
              // In JS `NaN !== NaN`, so we have to exlicitly check.
              if (count !== lastCount && !(countIsNaN && isNaN(lastCount))) {
                watchRemover();
                watchRemover = scope.$watch(whensExpFns[count], updateElementText);
                lastCount = count;
              }
            });
            function updateElementText(newText) {
              element.text(newText || '');
            }
          }
        };
      }
    ];
  /**
 * @ngdoc directive
 * @name ngRepeat
 *
 * @description
 * The `ngRepeat` directive instantiates a template once per item from a collection. Each template
 * instance gets its own scope, where the given loop variable is set to the current collection item,
 * and `$index` is set to the item index or key.
 *
 * Special properties are exposed on the local scope of each template instance, including:
 *
 * | Variable  | Type            | Details                                                                     |
 * |-----------|-----------------|-----------------------------------------------------------------------------|
 * | `$index`  | {@type number}  | iterator offset of the repeated element (0..length-1)                       |
 * | `$first`  | {@type boolean} | true if the repeated element is first in the iterator.                      |
 * | `$middle` | {@type boolean} | true if the repeated element is between the first and last in the iterator. |
 * | `$last`   | {@type boolean} | true if the repeated element is last in the iterator.                       |
 * | `$even`   | {@type boolean} | true if the iterator position `$index` is even (otherwise false).           |
 * | `$odd`    | {@type boolean} | true if the iterator position `$index` is odd (otherwise false).            |
 *
 * Creating aliases for these properties is possible with {@link ng.directive:ngInit `ngInit`}.
 * This may be useful when, for instance, nesting ngRepeats.
 *
 * # Iterating over object properties
 *
 * It is possible to get `ngRepeat` to iterate over the properties of an object using the following
 * syntax:
 *
 * ```js
 * <div ng-repeat="(key, value) in myObj"> ... </div>
 * ```
 *
 * You need to be aware that the JavaScript specification does not define what order
 * it will return the keys for an object. In order to have a guaranteed deterministic order
 * for the keys, Angular versions up to and including 1.3 **sort the keys alphabetically**.
 *
 * If this is not desired, the recommended workaround is to convert your object into an array
 * that is sorted into the order that you prefer before providing it to `ngRepeat`.  You could
 * do this with a filter such as [toArrayFilter](http://ngmodules.org/modules/angular-toArrayFilter)
 * or implement a `$watch` on the object yourself.
 *
 * In version 1.4 we will remove the sorting, since it seems that browsers generally follow the
 * strategy of providing keys in the order in which they were defined, although there are exceptions
 * when keys are deleted and reinstated.
 *
 *
 * # Tracking and Duplicates
 *
 * When the contents of the collection change, `ngRepeat` makes the corresponding changes to the DOM:
 *
 * * When an item is added, a new instance of the template is added to the DOM.
 * * When an item is removed, its template instance is removed from the DOM.
 * * When items are reordered, their respective templates are reordered in the DOM.
 *
 * By default, `ngRepeat` does not allow duplicate items in arrays. This is because when
 * there are duplicates, it is not possible to maintain a one-to-one mapping between collection
 * items and DOM elements.
 *
 * If you do need to repeat duplicate items, you can substitute the default tracking behavior
 * with your own using the `track by` expression.
 *
 * For example, you may track items by the index of each item in the collection, using the
 * special scope property `$index`:
 * ```html
 *    <div ng-repeat="n in [42, 42, 43, 43] track by $index">
 *      {{n}}
 *    </div>
 * ```
 *
 * You may use arbitrary expressions in `track by`, including references to custom functions
 * on the scope:
 * ```html
 *    <div ng-repeat="n in [42, 42, 43, 43] track by myTrackingFunction(n)">
 *      {{n}}
 *    </div>
 * ```
 *
 * If you are working with objects that have an identifier property, you can track
 * by the identifier instead of the whole object. Should you reload your data later, `ngRepeat`
 * will not have to rebuild the DOM elements for items it has already rendered, even if the
 * JavaScript objects in the collection have been substituted for new ones:
 * ```html
 *    <div ng-repeat="model in collection track by model.id">
 *      {{model.name}}
 *    </div>
 * ```
 *
 * When no `track by` expression is provided, it is equivalent to tracking by the built-in
 * `$id` function, which tracks items by their identity:
 * ```html
 *    <div ng-repeat="obj in collection track by $id(obj)">
 *      {{obj.prop}}
 *    </div>
 * ```
 *
 * # Special repeat start and end points
 * To repeat a series of elements instead of just one parent element, ngRepeat (as well as other ng directives) supports extending
 * the range of the repeater by defining explicit start and end points by using **ng-repeat-start** and **ng-repeat-end** respectively.
 * The **ng-repeat-start** directive works the same as **ng-repeat**, but will repeat all the HTML code (including the tag it's defined on)
 * up to and including the ending HTML tag where **ng-repeat-end** is placed.
 *
 * The example below makes use of this feature:
 * ```html
 *   <header ng-repeat-start="item in items">
 *     Header {{ item }}
 *   </header>
 *   <div class="body">
 *     Body {{ item }}
 *   </div>
 *   <footer ng-repeat-end>
 *     Footer {{ item }}
 *   </footer>
 * ```
 *
 * And with an input of {@type ['A','B']} for the items variable in the example above, the output will evaluate to:
 * ```html
 *   <header>
 *     Header A
 *   </header>
 *   <div class="body">
 *     Body A
 *   </div>
 *   <footer>
 *     Footer A
 *   </footer>
 *   <header>
 *     Header B
 *   </header>
 *   <div class="body">
 *     Body B
 *   </div>
 *   <footer>
 *     Footer B
 *   </footer>
 * ```
 *
 * The custom start and end points for ngRepeat also support all other HTML directive syntax flavors provided in AngularJS (such
 * as **data-ng-repeat-start**, **x-ng-repeat-start** and **ng:repeat-start**).
 *
 * @animations
 * **.enter** - when a new item is added to the list or when an item is revealed after a filter
 *
 * **.leave** - when an item is removed from the list or when an item is filtered out
 *
 * **.move** - when an adjacent item is filtered out causing a reorder or when the item contents are reordered
 *
 * @element ANY
 * @scope
 * @priority 1000
 * @param {repeat_expression} ngRepeat The expression indicating how to enumerate a collection. These
 *   formats are currently supported:
 *
 *   * `variable in expression` – where variable is the user defined loop variable and `expression`
 *     is a scope expression giving the collection to enumerate.
 *
 *     For example: `album in artist.albums`.
 *
 *   * `(key, value) in expression` – where `key` and `value` can be any user defined identifiers,
 *     and `expression` is the scope expression giving the collection to enumerate.
 *
 *     For example: `(name, age) in {'adam':10, 'amalie':12}`.
 *
 *   * `variable in expression track by tracking_expression` – You can also provide an optional tracking expression
 *     which can be used to associate the objects in the collection with the DOM elements. If no tracking expression
 *     is specified, ng-repeat associates elements by identity. It is an error to have
 *     more than one tracking expression value resolve to the same key. (This would mean that two distinct objects are
 *     mapped to the same DOM element, which is not possible.)  If filters are used in the expression, they should be
 *     applied before the tracking expression.
 *
 *     For example: `item in items` is equivalent to `item in items track by $id(item)`. This implies that the DOM elements
 *     will be associated by item identity in the array.
 *
 *     For example: `item in items track by $id(item)`. A built in `$id()` function can be used to assign a unique
 *     `$$hashKey` property to each item in the array. This property is then used as a key to associated DOM elements
 *     with the corresponding item in the array by identity. Moving the same object in array would move the DOM
 *     element in the same way in the DOM.
 *
 *     For example: `item in items track by item.id` is a typical pattern when the items come from the database. In this
 *     case the object identity does not matter. Two objects are considered equivalent as long as their `id`
 *     property is same.
 *
 *     For example: `item in items | filter:searchText track by item.id` is a pattern that might be used to apply a filter
 *     to items in conjunction with a tracking expression.
 *
 *   * `variable in expression as alias_expression` – You can also provide an optional alias expression which will then store the
 *     intermediate results of the repeater after the filters have been applied. Typically this is used to render a special message
 *     when a filter is active on the repeater, but the filtered result set is empty.
 *
 *     For example: `item in items | filter:x as results` will store the fragment of the repeated items as `results`, but only after
 *     the items have been processed through the filter.
 *
 * @example
 * This example initializes the scope to a list of names and
 * then uses `ngRepeat` to display every person:
  <example module="ngAnimate" deps="angular-animate.js" animations="true">
    <file name="index.html">
      <div ng-init="friends = [
        {name:'John', age:25, gender:'boy'},
        {name:'Jessie', age:30, gender:'girl'},
        {name:'Johanna', age:28, gender:'girl'},
        {name:'Joy', age:15, gender:'girl'},
        {name:'Mary', age:28, gender:'girl'},
        {name:'Peter', age:95, gender:'boy'},
        {name:'Sebastian', age:50, gender:'boy'},
        {name:'Erika', age:27, gender:'girl'},
        {name:'Patrick', age:40, gender:'boy'},
        {name:'Samantha', age:60, gender:'girl'}
      ]">
        I have {{friends.length}} friends. They are:
        <input type="search" ng-model="q" placeholder="filter friends..." />
        <ul class="example-animate-container">
          <li class="animate-repeat" ng-repeat="friend in friends | filter:q as results">
            [{{$index + 1}}] {{friend.name}} who is {{friend.age}} years old.
          </li>
          <li class="animate-repeat" ng-if="results.length == 0">
            <strong>No results found...</strong>
          </li>
        </ul>
      </div>
    </file>
    <file name="animations.css">
      .example-animate-container {
        background:white;
        border:1px solid black;
        list-style:none;
        margin:0;
        padding:0 10px;
      }

      .animate-repeat {
        line-height:40px;
        list-style:none;
        box-sizing:border-box;
      }

      .animate-repeat.ng-move,
      .animate-repeat.ng-enter,
      .animate-repeat.ng-leave {
        -webkit-transition:all linear 0.5s;
        transition:all linear 0.5s;
      }

      .animate-repeat.ng-leave.ng-leave-active,
      .animate-repeat.ng-move,
      .animate-repeat.ng-enter {
        opacity:0;
        max-height:0;
      }

      .animate-repeat.ng-leave,
      .animate-repeat.ng-move.ng-move-active,
      .animate-repeat.ng-enter.ng-enter-active {
        opacity:1;
        max-height:40px;
      }
    </file>
    <file name="protractor.js" type="protractor">
      var friends = element.all(by.repeater('friend in friends'));

      it('should render initial data set', function() {
        expect(friends.count()).toBe(10);
        expect(friends.get(0).getText()).toEqual('[1] John who is 25 years old.');
        expect(friends.get(1).getText()).toEqual('[2] Jessie who is 30 years old.');
        expect(friends.last().getText()).toEqual('[10] Samantha who is 60 years old.');
        expect(element(by.binding('friends.length')).getText())
            .toMatch("I have 10 friends. They are:");
      });

       it('should update repeater when filter predicate changes', function() {
         expect(friends.count()).toBe(10);

         element(by.model('q')).sendKeys('ma');

         expect(friends.count()).toBe(2);
         expect(friends.get(0).getText()).toEqual('[1] Mary who is 28 years old.');
         expect(friends.last().getText()).toEqual('[2] Samantha who is 60 years old.');
       });
      </file>
    </example>
 */
  var ngRepeatDirective = [
      '$parse',
      '$animate',
      function ($parse, $animate) {
        var NG_REMOVED = '$$NG_REMOVED';
        var ngRepeatMinErr = minErr('ngRepeat');
        var updateScope = function (scope, index, valueIdentifier, value, keyIdentifier, key, arrayLength) {
          // TODO(perf): generate setters to shave off ~40ms or 1-1.5%
          scope[valueIdentifier] = value;
          if (keyIdentifier)
            scope[keyIdentifier] = key;
          scope.$index = index;
          scope.$first = index === 0;
          scope.$last = index === arrayLength - 1;
          scope.$middle = !(scope.$first || scope.$last);
          // jshint bitwise: false
          scope.$odd = !(scope.$even = (index & 1) === 0);  // jshint bitwise: true
        };
        var getBlockStart = function (block) {
          return block.clone[0];
        };
        var getBlockEnd = function (block) {
          return block.clone[block.clone.length - 1];
        };
        return {
          restrict: 'A',
          multiElement: true,
          transclude: 'element',
          priority: 1000,
          terminal: true,
          $$tlb: true,
          compile: function ngRepeatCompile($element, $attr) {
            var expression = $attr.ngRepeat;
            var ngRepeatEndComment = document.createComment(' end ngRepeat: ' + expression + ' ');
            var match = expression.match(/^\s*([\s\S]+?)\s+in\s+([\s\S]+?)(?:\s+as\s+([\s\S]+?))?(?:\s+track\s+by\s+([\s\S]+?))?\s*$/);
            if (!match) {
              throw ngRepeatMinErr('iexp', 'Expected expression in form of \'_item_ in _collection_[ track by _id_]\' but got \'{0}\'.', expression);
            }
            var lhs = match[1];
            var rhs = match[2];
            var aliasAs = match[3];
            var trackByExp = match[4];
            match = lhs.match(/^(?:(\s*[\$\w]+)|\(\s*([\$\w]+)\s*,\s*([\$\w]+)\s*\))$/);
            if (!match) {
              throw ngRepeatMinErr('iidexp', '\'_item_\' in \'_item_ in _collection_\' should be an identifier or \'(_key_, _value_)\' expression, but got \'{0}\'.', lhs);
            }
            var valueIdentifier = match[3] || match[1];
            var keyIdentifier = match[2];
            if (aliasAs && (!/^[$a-zA-Z_][$a-zA-Z0-9_]*$/.test(aliasAs) || /^(null|undefined|this|\$index|\$first|\$middle|\$last|\$even|\$odd|\$parent|\$root|\$id)$/.test(aliasAs))) {
              throw ngRepeatMinErr('badident', 'alias \'{0}\' is invalid --- must be a valid JS identifier which is not a reserved name.', aliasAs);
            }
            var trackByExpGetter, trackByIdExpFn, trackByIdArrayFn, trackByIdObjFn;
            var hashFnLocals = { $id: hashKey };
            if (trackByExp) {
              trackByExpGetter = $parse(trackByExp);
            } else {
              trackByIdArrayFn = function (key, value) {
                return hashKey(value);
              };
              trackByIdObjFn = function (key) {
                return key;
              };
            }
            return function ngRepeatLink($scope, $element, $attr, ctrl, $transclude) {
              if (trackByExpGetter) {
                trackByIdExpFn = function (key, value, index) {
                  // assign key, value, and $index to the locals so that they can be used in hash functions
                  if (keyIdentifier)
                    hashFnLocals[keyIdentifier] = key;
                  hashFnLocals[valueIdentifier] = value;
                  hashFnLocals.$index = index;
                  return trackByExpGetter($scope, hashFnLocals);
                };
              }
              // Store a list of elements from previous run. This is a hash where key is the item from the
              // iterator, and the value is objects with following properties.
              //   - scope: bound scope
              //   - element: previous element.
              //   - index: position
              //
              // We are using no-proto object so that we don't need to guard against inherited props via
              // hasOwnProperty.
              var lastBlockMap = createMap();
              //watch props
              $scope.$watchCollection(rhs, function ngRepeatAction(collection) {
                var index, length, previousNode = $element[0],
                  // node that cloned nodes should be inserted after
                  // initialized to the comment node anchor
                  nextNode,
                  // Same as lastBlockMap but it has the current state. It will become the
                  // lastBlockMap on the next iteration.
                  nextBlockMap = createMap(), collectionLength, key, value,
                  // key/value of iteration
                  trackById, trackByIdFn, collectionKeys, block,
                  // last object information {scope, element, id}
                  nextBlockOrder, elementsToRemove;
                if (aliasAs) {
                  $scope[aliasAs] = collection;
                }
                if (isArrayLike(collection)) {
                  collectionKeys = collection;
                  trackByIdFn = trackByIdExpFn || trackByIdArrayFn;
                } else {
                  trackByIdFn = trackByIdExpFn || trackByIdObjFn;
                  // if object, extract keys, sort them and use to determine order of iteration over obj props
                  collectionKeys = [];
                  for (var itemKey in collection) {
                    if (collection.hasOwnProperty(itemKey) && itemKey.charAt(0) != '$') {
                      collectionKeys.push(itemKey);
                    }
                  }
                  collectionKeys.sort();
                }
                collectionLength = collectionKeys.length;
                nextBlockOrder = new Array(collectionLength);
                // locate existing items
                for (index = 0; index < collectionLength; index++) {
                  key = collection === collectionKeys ? index : collectionKeys[index];
                  value = collection[key];
                  trackById = trackByIdFn(key, value, index);
                  if (lastBlockMap[trackById]) {
                    // found previously seen block
                    block = lastBlockMap[trackById];
                    delete lastBlockMap[trackById];
                    nextBlockMap[trackById] = block;
                    nextBlockOrder[index] = block;
                  } else if (nextBlockMap[trackById]) {
                    // if collision detected. restore lastBlockMap and throw an error
                    forEach(nextBlockOrder, function (block) {
                      if (block && block.scope)
                        lastBlockMap[block.id] = block;
                    });
                    throw ngRepeatMinErr('dupes', 'Duplicates in a repeater are not allowed. Use \'track by\' expression to specify unique keys. Repeater: {0}, Duplicate key: {1}, Duplicate value: {2}', expression, trackById, value);
                  } else {
                    // new never before seen block
                    nextBlockOrder[index] = {
                      id: trackById,
                      scope: undefined,
                      clone: undefined
                    };
                    nextBlockMap[trackById] = true;
                  }
                }
                // remove leftover items
                for (var blockKey in lastBlockMap) {
                  block = lastBlockMap[blockKey];
                  elementsToRemove = getBlockNodes(block.clone);
                  $animate.leave(elementsToRemove);
                  if (elementsToRemove[0].parentNode) {
                    // if the element was not removed yet because of pending animation, mark it as deleted
                    // so that we can ignore it later
                    for (index = 0, length = elementsToRemove.length; index < length; index++) {
                      elementsToRemove[index][NG_REMOVED] = true;
                    }
                  }
                  block.scope.$destroy();
                }
                // we are not using forEach for perf reasons (trying to avoid #call)
                for (index = 0; index < collectionLength; index++) {
                  key = collection === collectionKeys ? index : collectionKeys[index];
                  value = collection[key];
                  block = nextBlockOrder[index];
                  if (block.scope) {
                    // if we have already seen this object, then we need to reuse the
                    // associated scope/element
                    nextNode = previousNode;
                    // skip nodes that are already pending removal via leave animation
                    do {
                      nextNode = nextNode.nextSibling;
                    } while (nextNode && nextNode[NG_REMOVED]);
                    if (getBlockStart(block) != nextNode) {
                      // existing item which got moved
                      $animate.move(getBlockNodes(block.clone), null, jqLite(previousNode));
                    }
                    previousNode = getBlockEnd(block);
                    updateScope(block.scope, index, valueIdentifier, value, keyIdentifier, key, collectionLength);
                  } else {
                    // new item which we don't know about
                    $transclude(function ngRepeatTransclude(clone, scope) {
                      block.scope = scope;
                      // http://jsperf.com/clone-vs-createcomment
                      var endNode = ngRepeatEndComment.cloneNode(false);
                      clone[clone.length++] = endNode;
                      // TODO(perf): support naked previousNode in `enter` to avoid creation of jqLite wrapper?
                      $animate.enter(clone, null, jqLite(previousNode));
                      previousNode = endNode;
                      // Note: We only need the first/last node of the cloned nodes.
                      // However, we need to keep the reference to the jqlite wrapper as it might be changed later
                      // by a directive with templateUrl when its template arrives.
                      block.clone = clone;
                      nextBlockMap[block.id] = block;
                      updateScope(block.scope, index, valueIdentifier, value, keyIdentifier, key, collectionLength);
                    });
                  }
                }
                lastBlockMap = nextBlockMap;
              });
            };
          }
        };
      }
    ];
  var NG_HIDE_CLASS = 'ng-hide';
  var NG_HIDE_IN_PROGRESS_CLASS = 'ng-hide-animate';
  /**
 * @ngdoc directive
 * @name ngShow
 *
 * @description
 * The `ngShow` directive shows or hides the given HTML element based on the expression
 * provided to the `ngShow` attribute. The element is shown or hidden by removing or adding
 * the `.ng-hide` CSS class onto the element. The `.ng-hide` CSS class is predefined
 * in AngularJS and sets the display style to none (using an !important flag).
 * For CSP mode please add `angular-csp.css` to your html file (see {@link ng.directive:ngCsp ngCsp}).
 *
 * ```html
 * <!-- when $scope.myValue is truthy (element is visible) -->
 * <div ng-show="myValue"></div>
 *
 * <!-- when $scope.myValue is falsy (element is hidden) -->
 * <div ng-show="myValue" class="ng-hide"></div>
 * ```
 *
 * When the `ngShow` expression evaluates to a falsy value then the `.ng-hide` CSS class is added to the class
 * attribute on the element causing it to become hidden. When truthy, the `.ng-hide` CSS class is removed
 * from the element causing the element not to appear hidden.
 *
 * ## Why is !important used?
 *
 * You may be wondering why !important is used for the `.ng-hide` CSS class. This is because the `.ng-hide` selector
 * can be easily overridden by heavier selectors. For example, something as simple
 * as changing the display style on a HTML list item would make hidden elements appear visible.
 * This also becomes a bigger issue when dealing with CSS frameworks.
 *
 * By using !important, the show and hide behavior will work as expected despite any clash between CSS selector
 * specificity (when !important isn't used with any conflicting styles). If a developer chooses to override the
 * styling to change how to hide an element then it is just a matter of using !important in their own CSS code.
 *
 * ### Overriding `.ng-hide`
 *
 * By default, the `.ng-hide` class will style the element with `display: none!important`. If you wish to change
 * the hide behavior with ngShow/ngHide then this can be achieved by restating the styles for the `.ng-hide`
 * class CSS. Note that the selector that needs to be used is actually `.ng-hide:not(.ng-hide-animate)` to cope
 * with extra animation classes that can be added.
 *
 * ```css
 * .ng-hide:not(.ng-hide-animate) {
 *   /&#42; this is just another form of hiding an element &#42;/
 *   display: block!important;
 *   position: absolute;
 *   top: -9999px;
 *   left: -9999px;
 * }
 * ```
 *
 * By default you don't need to override in CSS anything and the animations will work around the display style.
 *
 * ## A note about animations with `ngShow`
 *
 * Animations in ngShow/ngHide work with the show and hide events that are triggered when the directive expression
 * is true and false. This system works like the animation system present with ngClass except that
 * you must also include the !important flag to override the display property
 * so that you can perform an animation when the element is hidden during the time of the animation.
 *
 * ```css
 * //
 * //a working example can be found at the bottom of this page
 * //
 * .my-element.ng-hide-add, .my-element.ng-hide-remove {
 *   /&#42; this is required as of 1.3x to properly
 *      apply all styling in a show/hide animation &#42;/
 *   transition: 0s linear all;
 * }
 *
 * .my-element.ng-hide-add-active,
 * .my-element.ng-hide-remove-active {
 *   /&#42; the transition is defined in the active class &#42;/
 *   transition: 1s linear all;
 * }
 *
 * .my-element.ng-hide-add { ... }
 * .my-element.ng-hide-add.ng-hide-add-active { ... }
 * .my-element.ng-hide-remove { ... }
 * .my-element.ng-hide-remove.ng-hide-remove-active { ... }
 * ```
 *
 * Keep in mind that, as of AngularJS version 1.3.0-beta.11, there is no need to change the display
 * property to block during animation states--ngAnimate will handle the style toggling automatically for you.
 *
 * @animations
 * addClass: `.ng-hide` - happens after the `ngShow` expression evaluates to a truthy value and the just before contents are set to visible
 * removeClass: `.ng-hide` - happens after the `ngShow` expression evaluates to a non truthy value and just before the contents are set to hidden
 *
 * @element ANY
 * @param {expression} ngShow If the {@link guide/expression expression} is truthy
 *     then the element is shown or hidden respectively.
 *
 * @example
  <example module="ngAnimate" deps="angular-animate.js" animations="true">
    <file name="index.html">
      Click me: <input type="checkbox" ng-model="checked"><br/>
      <div>
        Show:
        <div class="check-element animate-show" ng-show="checked">
          <span class="glyphicon glyphicon-thumbs-up"></span> I show up when your checkbox is checked.
        </div>
      </div>
      <div>
        Hide:
        <div class="check-element animate-show" ng-hide="checked">
          <span class="glyphicon glyphicon-thumbs-down"></span> I hide when your checkbox is checked.
        </div>
      </div>
    </file>
    <file name="glyphicons.css">
      @import url(../../components/bootstrap-3.1.1/css/bootstrap.css);
    </file>
    <file name="animations.css">
      .animate-show {
        line-height: 20px;
        opacity: 1;
        padding: 10px;
        border: 1px solid black;
        background: white;
      }

      .animate-show.ng-hide-add.ng-hide-add-active,
      .animate-show.ng-hide-remove.ng-hide-remove-active {
        -webkit-transition: all linear 0.5s;
        transition: all linear 0.5s;
      }

      .animate-show.ng-hide {
        line-height: 0;
        opacity: 0;
        padding: 0 10px;
      }

      .check-element {
        padding: 10px;
        border: 1px solid black;
        background: white;
      }
    </file>
    <file name="protractor.js" type="protractor">
      var thumbsUp = element(by.css('span.glyphicon-thumbs-up'));
      var thumbsDown = element(by.css('span.glyphicon-thumbs-down'));

      it('should check ng-show / ng-hide', function() {
        expect(thumbsUp.isDisplayed()).toBeFalsy();
        expect(thumbsDown.isDisplayed()).toBeTruthy();

        element(by.model('checked')).click();

        expect(thumbsUp.isDisplayed()).toBeTruthy();
        expect(thumbsDown.isDisplayed()).toBeFalsy();
      });
    </file>
  </example>
 */
  var ngShowDirective = [
      '$animate',
      function ($animate) {
        return {
          restrict: 'A',
          multiElement: true,
          link: function (scope, element, attr) {
            scope.$watch(attr.ngShow, function ngShowWatchAction(value) {
              // we're adding a temporary, animation-specific class for ng-hide since this way
              // we can control when the element is actually displayed on screen without having
              // to have a global/greedy CSS selector that breaks when other animations are run.
              // Read: https://github.com/angular/angular.js/issues/9103#issuecomment-58335845
              $animate[value ? 'removeClass' : 'addClass'](element, NG_HIDE_CLASS, { tempClasses: NG_HIDE_IN_PROGRESS_CLASS });
            });
          }
        };
      }
    ];
  /**
 * @ngdoc directive
 * @name ngHide
 *
 * @description
 * The `ngHide` directive shows or hides the given HTML element based on the expression
 * provided to the `ngHide` attribute. The element is shown or hidden by removing or adding
 * the `ng-hide` CSS class onto the element. The `.ng-hide` CSS class is predefined
 * in AngularJS and sets the display style to none (using an !important flag).
 * For CSP mode please add `angular-csp.css` to your html file (see {@link ng.directive:ngCsp ngCsp}).
 *
 * ```html
 * <!-- when $scope.myValue is truthy (element is hidden) -->
 * <div ng-hide="myValue" class="ng-hide"></div>
 *
 * <!-- when $scope.myValue is falsy (element is visible) -->
 * <div ng-hide="myValue"></div>
 * ```
 *
 * When the `ngHide` expression evaluates to a truthy value then the `.ng-hide` CSS class is added to the class
 * attribute on the element causing it to become hidden. When falsy, the `.ng-hide` CSS class is removed
 * from the element causing the element not to appear hidden.
 *
 * ## Why is !important used?
 *
 * You may be wondering why !important is used for the `.ng-hide` CSS class. This is because the `.ng-hide` selector
 * can be easily overridden by heavier selectors. For example, something as simple
 * as changing the display style on a HTML list item would make hidden elements appear visible.
 * This also becomes a bigger issue when dealing with CSS frameworks.
 *
 * By using !important, the show and hide behavior will work as expected despite any clash between CSS selector
 * specificity (when !important isn't used with any conflicting styles). If a developer chooses to override the
 * styling to change how to hide an element then it is just a matter of using !important in their own CSS code.
 *
 * ### Overriding `.ng-hide`
 *
 * By default, the `.ng-hide` class will style the element with `display: none!important`. If you wish to change
 * the hide behavior with ngShow/ngHide then this can be achieved by restating the styles for the `.ng-hide`
 * class in CSS:
 *
 * ```css
 * .ng-hide {
 *   /&#42; this is just another form of hiding an element &#42;/
 *   display: block!important;
 *   position: absolute;
 *   top: -9999px;
 *   left: -9999px;
 * }
 * ```
 *
 * By default you don't need to override in CSS anything and the animations will work around the display style.
 *
 * ## A note about animations with `ngHide`
 *
 * Animations in ngShow/ngHide work with the show and hide events that are triggered when the directive expression
 * is true and false. This system works like the animation system present with ngClass, except that the `.ng-hide`
 * CSS class is added and removed for you instead of your own CSS class.
 *
 * ```css
 * //
 * //a working example can be found at the bottom of this page
 * //
 * .my-element.ng-hide-add, .my-element.ng-hide-remove {
 *   transition: 0.5s linear all;
 * }
 *
 * .my-element.ng-hide-add { ... }
 * .my-element.ng-hide-add.ng-hide-add-active { ... }
 * .my-element.ng-hide-remove { ... }
 * .my-element.ng-hide-remove.ng-hide-remove-active { ... }
 * ```
 *
 * Keep in mind that, as of AngularJS version 1.3.0-beta.11, there is no need to change the display
 * property to block during animation states--ngAnimate will handle the style toggling automatically for you.
 *
 * @animations
 * removeClass: `.ng-hide` - happens after the `ngHide` expression evaluates to a truthy value and just before the contents are set to hidden
 * addClass: `.ng-hide` - happens after the `ngHide` expression evaluates to a non truthy value and just before the contents are set to visible
 *
 * @element ANY
 * @param {expression} ngHide If the {@link guide/expression expression} is truthy then
 *     the element is shown or hidden respectively.
 *
 * @example
  <example module="ngAnimate" deps="angular-animate.js" animations="true">
    <file name="index.html">
      Click me: <input type="checkbox" ng-model="checked"><br/>
      <div>
        Show:
        <div class="check-element animate-hide" ng-show="checked">
          <span class="glyphicon glyphicon-thumbs-up"></span> I show up when your checkbox is checked.
        </div>
      </div>
      <div>
        Hide:
        <div class="check-element animate-hide" ng-hide="checked">
          <span class="glyphicon glyphicon-thumbs-down"></span> I hide when your checkbox is checked.
        </div>
      </div>
    </file>
    <file name="glyphicons.css">
      @import url(../../components/bootstrap-3.1.1/css/bootstrap.css);
    </file>
    <file name="animations.css">
      .animate-hide {
        -webkit-transition: all linear 0.5s;
        transition: all linear 0.5s;
        line-height: 20px;
        opacity: 1;
        padding: 10px;
        border: 1px solid black;
        background: white;
      }

      .animate-hide.ng-hide {
        line-height: 0;
        opacity: 0;
        padding: 0 10px;
      }

      .check-element {
        padding: 10px;
        border: 1px solid black;
        background: white;
      }
    </file>
    <file name="protractor.js" type="protractor">
      var thumbsUp = element(by.css('span.glyphicon-thumbs-up'));
      var thumbsDown = element(by.css('span.glyphicon-thumbs-down'));

      it('should check ng-show / ng-hide', function() {
        expect(thumbsUp.isDisplayed()).toBeFalsy();
        expect(thumbsDown.isDisplayed()).toBeTruthy();

        element(by.model('checked')).click();

        expect(thumbsUp.isDisplayed()).toBeTruthy();
        expect(thumbsDown.isDisplayed()).toBeFalsy();
      });
    </file>
  </example>
 */
  var ngHideDirective = [
      '$animate',
      function ($animate) {
        return {
          restrict: 'A',
          multiElement: true,
          link: function (scope, element, attr) {
            scope.$watch(attr.ngHide, function ngHideWatchAction(value) {
              // The comment inside of the ngShowDirective explains why we add and
              // remove a temporary class for the show/hide animation
              $animate[value ? 'addClass' : 'removeClass'](element, NG_HIDE_CLASS, { tempClasses: NG_HIDE_IN_PROGRESS_CLASS });
            });
          }
        };
      }
    ];
  /**
 * @ngdoc directive
 * @name ngStyle
 * @restrict AC
 *
 * @description
 * The `ngStyle` directive allows you to set CSS style on an HTML element conditionally.
 *
 * @element ANY
 * @param {expression} ngStyle
 *
 * {@link guide/expression Expression} which evals to an
 * object whose keys are CSS style names and values are corresponding values for those CSS
 * keys.
 *
 * Since some CSS style names are not valid keys for an object, they must be quoted.
 * See the 'background-color' style in the example below.
 *
 * @example
   <example>
     <file name="index.html">
        <input type="button" value="set color" ng-click="myStyle={color:'red'}">
        <input type="button" value="set background" ng-click="myStyle={'background-color':'blue'}">
        <input type="button" value="clear" ng-click="myStyle={}">
        <br/>
        <span ng-style="myStyle">Sample Text</span>
        <pre>myStyle={{myStyle}}</pre>
     </file>
     <file name="style.css">
       span {
         color: black;
       }
     </file>
     <file name="protractor.js" type="protractor">
       var colorSpan = element(by.css('span'));

       it('should check ng-style', function() {
         expect(colorSpan.getCssValue('color')).toBe('rgba(0, 0, 0, 1)');
         element(by.css('input[value=\'set color\']')).click();
         expect(colorSpan.getCssValue('color')).toBe('rgba(255, 0, 0, 1)');
         element(by.css('input[value=clear]')).click();
         expect(colorSpan.getCssValue('color')).toBe('rgba(0, 0, 0, 1)');
       });
     </file>
   </example>
 */
  var ngStyleDirective = ngDirective(function (scope, element, attr) {
      scope.$watch(attr.ngStyle, function ngStyleWatchAction(newStyles, oldStyles) {
        if (oldStyles && newStyles !== oldStyles) {
          forEach(oldStyles, function (val, style) {
            element.css(style, '');
          });
        }
        if (newStyles)
          element.css(newStyles);
      }, true);
    });
  /**
 * @ngdoc directive
 * @name ngSwitch
 * @restrict EA
 *
 * @description
 * The `ngSwitch` directive is used to conditionally swap DOM structure on your template based on a scope expression.
 * Elements within `ngSwitch` but without `ngSwitchWhen` or `ngSwitchDefault` directives will be preserved at the location
 * as specified in the template.
 *
 * The directive itself works similar to ngInclude, however, instead of downloading template code (or loading it
 * from the template cache), `ngSwitch` simply chooses one of the nested elements and makes it visible based on which element
 * matches the value obtained from the evaluated expression. In other words, you define a container element
 * (where you place the directive), place an expression on the **`on="..."` attribute**
 * (or the **`ng-switch="..."` attribute**), define any inner elements inside of the directive and place
 * a when attribute per element. The when attribute is used to inform ngSwitch which element to display when the on
 * expression is evaluated. If a matching expression is not found via a when attribute then an element with the default
 * attribute is displayed.
 *
 * <div class="alert alert-info">
 * Be aware that the attribute values to match against cannot be expressions. They are interpreted
 * as literal string values to match against.
 * For example, **`ng-switch-when="someVal"`** will match against the string `"someVal"` not against the
 * value of the expression `$scope.someVal`.
 * </div>

 * @animations
 * enter - happens after the ngSwitch contents change and the matched child element is placed inside the container
 * leave - happens just after the ngSwitch contents change and just before the former contents are removed from the DOM
 *
 * @usage
 *
 * ```
 * <ANY ng-switch="expression">
 *   <ANY ng-switch-when="matchValue1">...</ANY>
 *   <ANY ng-switch-when="matchValue2">...</ANY>
 *   <ANY ng-switch-default>...</ANY>
 * </ANY>
 * ```
 *
 *
 * @scope
 * @priority 1200
 * @param {*} ngSwitch|on expression to match against <code>ng-switch-when</code>.
 * On child elements add:
 *
 * * `ngSwitchWhen`: the case statement to match against. If match then this
 *   case will be displayed. If the same match appears multiple times, all the
 *   elements will be displayed.
 * * `ngSwitchDefault`: the default case when no other case match. If there
 *   are multiple default cases, all of them will be displayed when no other
 *   case match.
 *
 *
 * @example
  <example module="switchExample" deps="angular-animate.js" animations="true">
    <file name="index.html">
      <div ng-controller="ExampleController">
        <select ng-model="selection" ng-options="item for item in items">
        </select>
        <code>selection={{selection}}</code>
        <hr/>
        <div class="animate-switch-container"
          ng-switch on="selection">
            <div class="animate-switch" ng-switch-when="settings">Settings Div</div>
            <div class="animate-switch" ng-switch-when="home">Home Span</div>
            <div class="animate-switch" ng-switch-default>default</div>
        </div>
      </div>
    </file>
    <file name="script.js">
      angular.module('switchExample', ['ngAnimate'])
        .controller('ExampleController', ['$scope', function($scope) {
          $scope.items = ['settings', 'home', 'other'];
          $scope.selection = $scope.items[0];
        }]);
    </file>
    <file name="animations.css">
      .animate-switch-container {
        position:relative;
        background:white;
        border:1px solid black;
        height:40px;
        overflow:hidden;
      }

      .animate-switch {
        padding:10px;
      }

      .animate-switch.ng-animate {
        -webkit-transition:all cubic-bezier(0.250, 0.460, 0.450, 0.940) 0.5s;
        transition:all cubic-bezier(0.250, 0.460, 0.450, 0.940) 0.5s;

        position:absolute;
        top:0;
        left:0;
        right:0;
        bottom:0;
      }

      .animate-switch.ng-leave.ng-leave-active,
      .animate-switch.ng-enter {
        top:-50px;
      }
      .animate-switch.ng-leave,
      .animate-switch.ng-enter.ng-enter-active {
        top:0;
      }
    </file>
    <file name="protractor.js" type="protractor">
      var switchElem = element(by.css('[ng-switch]'));
      var select = element(by.model('selection'));

      it('should start in settings', function() {
        expect(switchElem.getText()).toMatch(/Settings Div/);
      });
      it('should change to home', function() {
        select.all(by.css('option')).get(1).click();
        expect(switchElem.getText()).toMatch(/Home Span/);
      });
      it('should select default', function() {
        select.all(by.css('option')).get(2).click();
        expect(switchElem.getText()).toMatch(/default/);
      });
    </file>
  </example>
 */
  var ngSwitchDirective = [
      '$animate',
      function ($animate) {
        return {
          restrict: 'EA',
          require: 'ngSwitch',
          controller: [
            '$scope',
            function ngSwitchController() {
              this.cases = {};
            }
          ],
          link: function (scope, element, attr, ngSwitchController) {
            var watchExpr = attr.ngSwitch || attr.on, selectedTranscludes = [], selectedElements = [], previousLeaveAnimations = [], selectedScopes = [];
            var spliceFactory = function (array, index) {
              return function () {
                array.splice(index, 1);
              };
            };
            scope.$watch(watchExpr, function ngSwitchWatchAction(value) {
              var i, ii;
              for (i = 0, ii = previousLeaveAnimations.length; i < ii; ++i) {
                $animate.cancel(previousLeaveAnimations[i]);
              }
              previousLeaveAnimations.length = 0;
              for (i = 0, ii = selectedScopes.length; i < ii; ++i) {
                var selected = getBlockNodes(selectedElements[i].clone);
                selectedScopes[i].$destroy();
                var promise = previousLeaveAnimations[i] = $animate.leave(selected);
                promise.then(spliceFactory(previousLeaveAnimations, i));
              }
              selectedElements.length = 0;
              selectedScopes.length = 0;
              if (selectedTranscludes = ngSwitchController.cases['!' + value] || ngSwitchController.cases['?']) {
                forEach(selectedTranscludes, function (selectedTransclude) {
                  selectedTransclude.transclude(function (caseElement, selectedScope) {
                    selectedScopes.push(selectedScope);
                    var anchor = selectedTransclude.element;
                    caseElement[caseElement.length++] = document.createComment(' end ngSwitchWhen: ');
                    var block = { clone: caseElement };
                    selectedElements.push(block);
                    $animate.enter(caseElement, anchor.parent(), anchor);
                  });
                });
              }
            });
          }
        };
      }
    ];
  var ngSwitchWhenDirective = ngDirective({
      transclude: 'element',
      priority: 1200,
      require: '^ngSwitch',
      multiElement: true,
      link: function (scope, element, attrs, ctrl, $transclude) {
        ctrl.cases['!' + attrs.ngSwitchWhen] = ctrl.cases['!' + attrs.ngSwitchWhen] || [];
        ctrl.cases['!' + attrs.ngSwitchWhen].push({
          transclude: $transclude,
          element: element
        });
      }
    });
  var ngSwitchDefaultDirective = ngDirective({
      transclude: 'element',
      priority: 1200,
      require: '^ngSwitch',
      multiElement: true,
      link: function (scope, element, attr, ctrl, $transclude) {
        ctrl.cases['?'] = ctrl.cases['?'] || [];
        ctrl.cases['?'].push({
          transclude: $transclude,
          element: element
        });
      }
    });
  /**
 * @ngdoc directive
 * @name ngTransclude
 * @restrict EAC
 *
 * @description
 * Directive that marks the insertion point for the transcluded DOM of the nearest parent directive that uses transclusion.
 *
 * Any existing content of the element that this directive is placed on will be removed before the transcluded content is inserted.
 *
 * @element ANY
 *
 * @example
   <example module="transcludeExample">
     <file name="index.html">
       <script>
         angular.module('transcludeExample', [])
          .directive('pane', function(){
             return {
               restrict: 'E',
               transclude: true,
               scope: { title:'@' },
               template: '<div style="border: 1px solid black;">' +
                           '<div style="background-color: gray">{{title}}</div>' +
                           '<ng-transclude></ng-transclude>' +
                         '</div>'
             };
         })
         .controller('ExampleController', ['$scope', function($scope) {
           $scope.title = 'Lorem Ipsum';
           $scope.text = 'Neque porro quisquam est qui dolorem ipsum quia dolor...';
         }]);
       </script>
       <div ng-controller="ExampleController">
         <input ng-model="title"> <br/>
         <textarea ng-model="text"></textarea> <br/>
         <pane title="{{title}}">{{text}}</pane>
       </div>
     </file>
     <file name="protractor.js" type="protractor">
        it('should have transcluded', function() {
          var titleElement = element(by.model('title'));
          titleElement.clear();
          titleElement.sendKeys('TITLE');
          var textElement = element(by.model('text'));
          textElement.clear();
          textElement.sendKeys('TEXT');
          expect(element(by.binding('title')).getText()).toEqual('TITLE');
          expect(element(by.binding('text')).getText()).toEqual('TEXT');
        });
     </file>
   </example>
 *
 */
  var ngTranscludeDirective = ngDirective({
      restrict: 'EAC',
      link: function ($scope, $element, $attrs, controller, $transclude) {
        if (!$transclude) {
          throw minErr('ngTransclude')('orphan', 'Illegal use of ngTransclude directive in the template! ' + 'No parent directive that requires a transclusion found. ' + 'Element: {0}', startingTag($element));
        }
        $transclude(function (clone) {
          $element.empty();
          $element.append(clone);
        });
      }
    });
  /**
 * @ngdoc directive
 * @name script
 * @restrict E
 *
 * @description
 * Load the content of a `<script>` element into {@link ng.$templateCache `$templateCache`}, so that the
 * template can be used by {@link ng.directive:ngInclude `ngInclude`},
 * {@link ngRoute.directive:ngView `ngView`}, or {@link guide/directive directives}. The type of the
 * `<script>` element must be specified as `text/ng-template`, and a cache name for the template must be
 * assigned through the element's `id`, which can then be used as a directive's `templateUrl`.
 *
 * @param {string} type Must be set to `'text/ng-template'`.
 * @param {string} id Cache name of the template.
 *
 * @example
  <example>
    <file name="index.html">
      <script type="text/ng-template" id="/tpl.html">
        Content of the template.
      </script>

      <a ng-click="currentTpl='/tpl.html'" id="tpl-link">Load inlined template</a>
      <div id="tpl-content" ng-include src="currentTpl"></div>
    </file>
    <file name="protractor.js" type="protractor">
      it('should load template defined inside script tag', function() {
        element(by.css('#tpl-link')).click();
        expect(element(by.css('#tpl-content')).getText()).toMatch(/Content of the template/);
      });
    </file>
  </example>
 */
  var scriptDirective = [
      '$templateCache',
      function ($templateCache) {
        return {
          restrict: 'E',
          terminal: true,
          compile: function (element, attr) {
            if (attr.type == 'text/ng-template') {
              var templateUrl = attr.id, text = element[0].text;
              $templateCache.put(templateUrl, text);
            }
          }
        };
      }
    ];
  var ngOptionsMinErr = minErr('ngOptions');
  /**
 * @ngdoc directive
 * @name select
 * @restrict E
 *
 * @description
 * HTML `SELECT` element with angular data-binding.
 *
 * # `ngOptions`
 *
 * The `ngOptions` attribute can be used to dynamically generate a list of `<option>`
 * elements for the `<select>` element using the array or object obtained by evaluating the
 * `ngOptions` comprehension expression.
 *
 * In many cases, `ngRepeat` can be used on `<option>` elements instead of `ngOptions` to achieve a
 * similar result. However, `ngOptions` provides some benefits such as reducing memory and
 * increasing speed by not creating a new scope for each repeated instance, as well as providing
 * more flexibility in how the `<select>`'s model is assigned via the `select` **`as`** part of the
 * comprehension expression. `ngOptions` should be used when the `<select>` model needs to be bound
 *  to a non-string value. This is because an option element can only be bound to string values at
 * present.
 *
 * When an item in the `<select>` menu is selected, the array element or object property
 * represented by the selected option will be bound to the model identified by the `ngModel`
 * directive.
 *
 * Optionally, a single hard-coded `<option>` element, with the value set to an empty string, can
 * be nested into the `<select>` element. This element will then represent the `null` or "not selected"
 * option. See example below for demonstration.
 *
 * <div class="alert alert-warning">
 * **Note:** `ngModel` compares by reference, not value. This is important when binding to an
 * array of objects. See an example [in this jsfiddle](http://jsfiddle.net/qWzTb/).
 * </div>
 *
 * ## `select` **`as`**
 *
 * Using `select` **`as`** will bind the result of the `select` expression to the model, but
 * the value of the `<select>` and `<option>` html elements will be either the index (for array data sources)
 * or property name (for object data sources) of the value within the collection. If a **`track by`** expression
 * is used, the result of that expression will be set as the value of the `option` and `select` elements.
 *
 *
 * ### `select` **`as`** and **`track by`**
 *
 * <div class="alert alert-warning">
 * Do not use `select` **`as`** and **`track by`** in the same expression. They are not designed to work together.
 * </div>
 *
 * Consider the following example:
 *
 * ```html
 * <select ng-options="item.subItem as item.label for item in values track by item.id" ng-model="selected">
 * ```
 *
 * ```js
 * $scope.values = [{
 *   id: 1,
 *   label: 'aLabel',
 *   subItem: { name: 'aSubItem' }
 * }, {
 *   id: 2,
 *   label: 'bLabel',
 *   subItem: { name: 'bSubItem' }
 * }];
 *
 * $scope.selected = { name: 'aSubItem' };
 * ```
 *
 * With the purpose of preserving the selection, the **`track by`** expression is always applied to the element
 * of the data source (to `item` in this example). To calculate whether an element is selected, we do the
 * following:
 *
 * 1. Apply **`track by`** to the elements in the array. In the example: `[1, 2]`
 * 2. Apply **`track by`** to the already selected value in `ngModel`.
 *    In the example: this is not possible as **`track by`** refers to `item.id`, but the selected
 *    value from `ngModel` is `{name: 'aSubItem'}`, so the **`track by`** expression is applied to
 *    a wrong object, the selected element can't be found, `<select>` is always reset to the "not
 *    selected" option.
 *
 *
 * @param {string} ngModel Assignable angular expression to data-bind to.
 * @param {string=} name Property name of the form under which the control is published.
 * @param {string=} required The control is considered valid only if value is entered.
 * @param {string=} ngRequired Adds `required` attribute and `required` validation constraint to
 *    the element when the ngRequired expression evaluates to true. Use `ngRequired` instead of
 *    `required` when you want to data-bind to the `required` attribute.
 * @param {comprehension_expression=} ngOptions in one of the following forms:
 *
 *   * for array data sources:
 *     * `label` **`for`** `value` **`in`** `array`
 *     * `select` **`as`** `label` **`for`** `value` **`in`** `array`
 *     * `label` **`group by`** `group` **`for`** `value` **`in`** `array`
 *     * `label` **`group by`** `group` **`for`** `value` **`in`** `array` **`track by`** `trackexpr`
 *     * `label` **`for`** `value` **`in`** `array` | orderBy:`orderexpr` **`track by`** `trackexpr`
 *        (for including a filter with `track by`)
 *   * for object data sources:
 *     * `label` **`for (`**`key` **`,`** `value`**`) in`** `object`
 *     * `select` **`as`** `label` **`for (`**`key` **`,`** `value`**`) in`** `object`
 *     * `label` **`group by`** `group` **`for (`**`key`**`,`** `value`**`) in`** `object`
 *     * `select` **`as`** `label` **`group by`** `group`
 *         **`for` `(`**`key`**`,`** `value`**`) in`** `object`
 *
 * Where:
 *
 *   * `array` / `object`: an expression which evaluates to an array / object to iterate over.
 *   * `value`: local variable which will refer to each item in the `array` or each property value
 *      of `object` during iteration.
 *   * `key`: local variable which will refer to a property name in `object` during iteration.
 *   * `label`: The result of this expression will be the label for `<option>` element. The
 *     `expression` will most likely refer to the `value` variable (e.g. `value.propertyName`).
 *   * `select`: The result of this expression will be bound to the model of the parent `<select>`
 *      element. If not specified, `select` expression will default to `value`.
 *   * `group`: The result of this expression will be used to group options using the `<optgroup>`
 *      DOM element.
 *   * `trackexpr`: Used when working with an array of objects. The result of this expression will be
 *      used to identify the objects in the array. The `trackexpr` will most likely refer to the
 *     `value` variable (e.g. `value.propertyName`). With this the selection is preserved
 *      even when the options are recreated (e.g. reloaded from the server).
 *
 * @example
    <example module="selectExample">
      <file name="index.html">
        <script>
        angular.module('selectExample', [])
          .controller('ExampleController', ['$scope', function($scope) {
            $scope.colors = [
              {name:'black', shade:'dark'},
              {name:'white', shade:'light'},
              {name:'red', shade:'dark'},
              {name:'blue', shade:'dark'},
              {name:'yellow', shade:'light'}
            ];
            $scope.myColor = $scope.colors[2]; // red
          }]);
        </script>
        <div ng-controller="ExampleController">
          <ul>
            <li ng-repeat="color in colors">
              Name: <input ng-model="color.name">
              [<a href ng-click="colors.splice($index, 1)">X</a>]
            </li>
            <li>
              [<a href ng-click="colors.push({})">add</a>]
            </li>
          </ul>
          <hr/>
          Color (null not allowed):
          <select ng-model="myColor" ng-options="color.name for color in colors"></select><br>

          Color (null allowed):
          <span  class="nullable">
            <select ng-model="myColor" ng-options="color.name for color in colors">
              <option value="">-- choose color --</option>
            </select>
          </span><br/>

          Color grouped by shade:
          <select ng-model="myColor" ng-options="color.name group by color.shade for color in colors">
          </select><br/>


          Select <a href ng-click="myColor = { name:'not in list', shade: 'other' }">bogus</a>.<br>
          <hr/>
          Currently selected: {{ {selected_color:myColor} }}
          <div style="border:solid 1px black; height:20px"
               ng-style="{'background-color':myColor.name}">
          </div>
        </div>
      </file>
      <file name="protractor.js" type="protractor">
         it('should check ng-options', function() {
           expect(element(by.binding('{selected_color:myColor}')).getText()).toMatch('red');
           element.all(by.model('myColor')).first().click();
           element.all(by.css('select[ng-model="myColor"] option')).first().click();
           expect(element(by.binding('{selected_color:myColor}')).getText()).toMatch('black');
           element(by.css('.nullable select[ng-model="myColor"]')).click();
           element.all(by.css('.nullable select[ng-model="myColor"] option')).first().click();
           expect(element(by.binding('{selected_color:myColor}')).getText()).toMatch('null');
         });
      </file>
    </example>
 */
  var ngOptionsDirective = valueFn({
      restrict: 'A',
      terminal: true
    });
  // jshint maxlen: false
  var selectDirective = [
      '$compile',
      '$parse',
      function ($compile, $parse) {
        //000011111111110000000000022222222220000000000000000000003333333333000000000000004444444444444440000000005555555555555550000000666666666666666000000000000000777777777700000000000000000008888888888
        var NG_OPTIONS_REGEXP = /^\s*([\s\S]+?)(?:\s+as\s+([\s\S]+?))?(?:\s+group\s+by\s+([\s\S]+?))?\s+for\s+(?:([\$\w][\$\w]*)|(?:\(\s*([\$\w][\$\w]*)\s*,\s*([\$\w][\$\w]*)\s*\)))\s+in\s+([\s\S]+?)(?:\s+track\s+by\s+([\s\S]+?))?$/, nullModelCtrl = { $setViewValue: noop };
        // jshint maxlen: 100
        return {
          restrict: 'E',
          require: [
            'select',
            '?ngModel'
          ],
          controller: [
            '$element',
            '$scope',
            '$attrs',
            function ($element, $scope, $attrs) {
              var self = this, optionsMap = {}, ngModelCtrl = nullModelCtrl, nullOption, unknownOption;
              self.databound = $attrs.ngModel;
              self.init = function (ngModelCtrl_, nullOption_, unknownOption_) {
                ngModelCtrl = ngModelCtrl_;
                nullOption = nullOption_;
                unknownOption = unknownOption_;
              };
              self.addOption = function (value, element) {
                assertNotHasOwnProperty(value, '"option value"');
                optionsMap[value] = true;
                if (ngModelCtrl.$viewValue == value) {
                  $element.val(value);
                  if (unknownOption.parent())
                    unknownOption.remove();
                }
                // Workaround for https://code.google.com/p/chromium/issues/detail?id=381459
                // Adding an <option selected="selected"> element to a <select required="required"> should
                // automatically select the new element
                if (element && element[0].hasAttribute('selected')) {
                  element[0].selected = true;
                }
              };
              self.removeOption = function (value) {
                if (this.hasOption(value)) {
                  delete optionsMap[value];
                  if (ngModelCtrl.$viewValue === value) {
                    this.renderUnknownOption(value);
                  }
                }
              };
              self.renderUnknownOption = function (val) {
                var unknownVal = '? ' + hashKey(val) + ' ?';
                unknownOption.val(unknownVal);
                $element.prepend(unknownOption);
                $element.val(unknownVal);
                unknownOption.prop('selected', true);  // needed for IE
              };
              self.hasOption = function (value) {
                return optionsMap.hasOwnProperty(value);
              };
              $scope.$on('$destroy', function () {
                // disable unknown option so that we don't do work when the whole select is being destroyed
                self.renderUnknownOption = noop;
              });
            }
          ],
          link: function (scope, element, attr, ctrls) {
            // if ngModel is not defined, we don't need to do anything
            if (!ctrls[1])
              return;
            var selectCtrl = ctrls[0], ngModelCtrl = ctrls[1], multiple = attr.multiple, optionsExp = attr.ngOptions, nullOption = false,
              // if false, user will not be able to select it (used by ngOptions)
              emptyOption, renderScheduled = false,
              // we can't just jqLite('<option>') since jqLite is not smart enough
              // to create it in <select> and IE barfs otherwise.
              optionTemplate = jqLite(document.createElement('option')), optGroupTemplate = jqLite(document.createElement('optgroup')), unknownOption = optionTemplate.clone();
            // find "null" option
            for (var i = 0, children = element.children(), ii = children.length; i < ii; i++) {
              if (children[i].value === '') {
                emptyOption = nullOption = children.eq(i);
                break;
              }
            }
            selectCtrl.init(ngModelCtrl, nullOption, unknownOption);
            // required validator
            if (multiple) {
              ngModelCtrl.$isEmpty = function (value) {
                return !value || value.length === 0;
              };
            }
            if (optionsExp)
              setupAsOptions(scope, element, ngModelCtrl);
            else if (multiple)
              setupAsMultiple(scope, element, ngModelCtrl);
            else
              setupAsSingle(scope, element, ngModelCtrl, selectCtrl);
            ////////////////////////////
            function setupAsSingle(scope, selectElement, ngModelCtrl, selectCtrl) {
              ngModelCtrl.$render = function () {
                var viewValue = ngModelCtrl.$viewValue;
                if (selectCtrl.hasOption(viewValue)) {
                  if (unknownOption.parent())
                    unknownOption.remove();
                  selectElement.val(viewValue);
                  if (viewValue === '')
                    emptyOption.prop('selected', true);  // to make IE9 happy
                } else {
                  if (viewValue == null && emptyOption) {
                    selectElement.val('');
                  } else {
                    selectCtrl.renderUnknownOption(viewValue);
                  }
                }
              };
              selectElement.on('change', function () {
                scope.$apply(function () {
                  if (unknownOption.parent())
                    unknownOption.remove();
                  ngModelCtrl.$setViewValue(selectElement.val());
                });
              });
            }
            function setupAsMultiple(scope, selectElement, ctrl) {
              var lastView;
              ctrl.$render = function () {
                var items = new HashMap(ctrl.$viewValue);
                forEach(selectElement.find('option'), function (option) {
                  option.selected = isDefined(items.get(option.value));
                });
              };
              // we have to do it on each watch since ngModel watches reference, but
              // we need to work of an array, so we need to see if anything was inserted/removed
              scope.$watch(function selectMultipleWatch() {
                if (!equals(lastView, ctrl.$viewValue)) {
                  lastView = shallowCopy(ctrl.$viewValue);
                  ctrl.$render();
                }
              });
              selectElement.on('change', function () {
                scope.$apply(function () {
                  var array = [];
                  forEach(selectElement.find('option'), function (option) {
                    if (option.selected) {
                      array.push(option.value);
                    }
                  });
                  ctrl.$setViewValue(array);
                });
              });
            }
            function setupAsOptions(scope, selectElement, ctrl) {
              var match;
              if (!(match = optionsExp.match(NG_OPTIONS_REGEXP))) {
                throw ngOptionsMinErr('iexp', 'Expected expression in form of ' + '\'_select_ (as _label_)? for (_key_,)?_value_ in _collection_\'' + ' but got \'{0}\'. Element: {1}', optionsExp, startingTag(selectElement));
              }
              var displayFn = $parse(match[2] || match[1]), valueName = match[4] || match[6], selectAs = / as /.test(match[0]) && match[1], selectAsFn = selectAs ? $parse(selectAs) : null, keyName = match[5], groupByFn = $parse(match[3] || ''), valueFn = $parse(match[2] ? match[1] : valueName), valuesFn = $parse(match[7]), track = match[8], trackFn = track ? $parse(match[8]) : null, trackKeysCache = {},
                // This is an array of array of existing option groups in DOM.
                // We try to reuse these if possible
                // - optionGroupsCache[0] is the options with no option group
                // - optionGroupsCache[?][0] is the parent: either the SELECT or OPTGROUP element
                optionGroupsCache = [[{
                      element: selectElement,
                      label: ''
                    }]],
                //re-usable object to represent option's locals
                locals = {};
              if (nullOption) {
                // compile the element since there might be bindings in it
                $compile(nullOption)(scope);
                // remove the class, which is added automatically because we recompile the element and it
                // becomes the compilation root
                nullOption.removeClass('ng-scope');
                // we need to remove it before calling selectElement.empty() because otherwise IE will
                // remove the label from the element. wtf?
                nullOption.remove();
              }
              // clear contents, we'll add what's needed based on the model
              selectElement.empty();
              selectElement.on('change', selectionChanged);
              ctrl.$render = render;
              scope.$watchCollection(valuesFn, scheduleRendering);
              scope.$watchCollection(getLabels, scheduleRendering);
              if (multiple) {
                scope.$watchCollection(function () {
                  return ctrl.$modelValue;
                }, scheduleRendering);
              }
              // ------------------------------------------------------------------ //
              function callExpression(exprFn, key, value) {
                locals[valueName] = value;
                if (keyName)
                  locals[keyName] = key;
                return exprFn(scope, locals);
              }
              function selectionChanged() {
                scope.$apply(function () {
                  var collection = valuesFn(scope) || [];
                  var viewValue;
                  if (multiple) {
                    viewValue = [];
                    forEach(selectElement.val(), function (selectedKey) {
                      selectedKey = trackFn ? trackKeysCache[selectedKey] : selectedKey;
                      viewValue.push(getViewValue(selectedKey, collection[selectedKey]));
                    });
                  } else {
                    var selectedKey = trackFn ? trackKeysCache[selectElement.val()] : selectElement.val();
                    viewValue = getViewValue(selectedKey, collection[selectedKey]);
                  }
                  ctrl.$setViewValue(viewValue);
                  render();
                });
              }
              function getViewValue(key, value) {
                if (key === '?') {
                  return undefined;
                } else if (key === '') {
                  return null;
                } else {
                  var viewValueFn = selectAsFn ? selectAsFn : valueFn;
                  return callExpression(viewValueFn, key, value);
                }
              }
              function getLabels() {
                var values = valuesFn(scope);
                var toDisplay;
                if (values && isArray(values)) {
                  toDisplay = new Array(values.length);
                  for (var i = 0, ii = values.length; i < ii; i++) {
                    toDisplay[i] = callExpression(displayFn, i, values[i]);
                  }
                  return toDisplay;
                } else if (values) {
                  // TODO: Add a test for this case
                  toDisplay = {};
                  for (var prop in values) {
                    if (values.hasOwnProperty(prop)) {
                      toDisplay[prop] = callExpression(displayFn, prop, values[prop]);
                    }
                  }
                }
                return toDisplay;
              }
              function createIsSelectedFn(viewValue) {
                var selectedSet;
                if (multiple) {
                  if (trackFn && isArray(viewValue)) {
                    selectedSet = new HashMap([]);
                    for (var trackIndex = 0; trackIndex < viewValue.length; trackIndex++) {
                      // tracking by key
                      selectedSet.put(callExpression(trackFn, null, viewValue[trackIndex]), true);
                    }
                  } else {
                    selectedSet = new HashMap(viewValue);
                  }
                } else if (trackFn) {
                  viewValue = callExpression(trackFn, null, viewValue);
                }
                return function isSelected(key, value) {
                  var compareValueFn;
                  if (trackFn) {
                    compareValueFn = trackFn;
                  } else if (selectAsFn) {
                    compareValueFn = selectAsFn;
                  } else {
                    compareValueFn = valueFn;
                  }
                  if (multiple) {
                    return isDefined(selectedSet.remove(callExpression(compareValueFn, key, value)));
                  } else {
                    return viewValue === callExpression(compareValueFn, key, value);
                  }
                };
              }
              function scheduleRendering() {
                if (!renderScheduled) {
                  scope.$$postDigest(render);
                  renderScheduled = true;
                }
              }
              /**
         * A new labelMap is created with each render.
         * This function is called for each existing option with added=false,
         * and each new option with added=true.
         * - Labels that are passed to this method twice,
         * (once with added=true and once with added=false) will end up with a value of 0, and
         * will cause no change to happen to the corresponding option.
         * - Labels that are passed to this method only once with added=false will end up with a
         * value of -1 and will eventually be passed to selectCtrl.removeOption()
         * - Labels that are passed to this method only once with added=true will end up with a
         * value of 1 and will eventually be passed to selectCtrl.addOption()
        */
              function updateLabelMap(labelMap, label, added) {
                labelMap[label] = labelMap[label] || 0;
                labelMap[label] += added ? 1 : -1;
              }
              function render() {
                renderScheduled = false;
                // Temporary location for the option groups before we render them
                var optionGroups = { '': [] }, optionGroupNames = [''], optionGroupName, optionGroup, option, existingParent, existingOptions, existingOption, viewValue = ctrl.$viewValue, values = valuesFn(scope) || [], keys = keyName ? sortedKeys(values) : values, key, value, groupLength, length, groupIndex, index, labelMap = {}, selected, isSelected = createIsSelectedFn(viewValue), anySelected = false, lastElement, element, label, optionId;
                trackKeysCache = {};
                // We now build up the list of options we need (we merge later)
                for (index = 0; length = keys.length, index < length; index++) {
                  key = index;
                  if (keyName) {
                    key = keys[index];
                    if (key.charAt(0) === '$')
                      continue;
                  }
                  value = values[key];
                  optionGroupName = callExpression(groupByFn, key, value) || '';
                  if (!(optionGroup = optionGroups[optionGroupName])) {
                    optionGroup = optionGroups[optionGroupName] = [];
                    optionGroupNames.push(optionGroupName);
                  }
                  selected = isSelected(key, value);
                  anySelected = anySelected || selected;
                  label = callExpression(displayFn, key, value);
                  // what will be seen by the user
                  // doing displayFn(scope, locals) || '' overwrites zero values
                  label = isDefined(label) ? label : '';
                  optionId = trackFn ? trackFn(scope, locals) : keyName ? keys[index] : index;
                  if (trackFn) {
                    trackKeysCache[optionId] = key;
                  }
                  optionGroup.push({
                    id: optionId,
                    label: label,
                    selected: selected
                  });
                }
                if (!multiple) {
                  if (nullOption || viewValue === null) {
                    // insert null option if we have a placeholder, or the model is null
                    optionGroups[''].unshift({
                      id: '',
                      label: '',
                      selected: !anySelected
                    });
                  } else if (!anySelected) {
                    // option could not be found, we have to insert the undefined item
                    optionGroups[''].unshift({
                      id: '?',
                      label: '',
                      selected: true
                    });
                  }
                }
                // Now we need to update the list of DOM nodes to match the optionGroups we computed above
                for (groupIndex = 0, groupLength = optionGroupNames.length; groupIndex < groupLength; groupIndex++) {
                  // current option group name or '' if no group
                  optionGroupName = optionGroupNames[groupIndex];
                  // list of options for that group. (first item has the parent)
                  optionGroup = optionGroups[optionGroupName];
                  if (optionGroupsCache.length <= groupIndex) {
                    // we need to grow the optionGroups
                    existingParent = {
                      element: optGroupTemplate.clone().attr('label', optionGroupName),
                      label: optionGroup.label
                    };
                    existingOptions = [existingParent];
                    optionGroupsCache.push(existingOptions);
                    selectElement.append(existingParent.element);
                  } else {
                    existingOptions = optionGroupsCache[groupIndex];
                    existingParent = existingOptions[0];
                    // either SELECT (no group) or OPTGROUP element
                    // update the OPTGROUP label if not the same.
                    if (existingParent.label != optionGroupName) {
                      existingParent.element.attr('label', existingParent.label = optionGroupName);
                    }
                  }
                  lastElement = null;
                  // start at the beginning
                  for (index = 0, length = optionGroup.length; index < length; index++) {
                    option = optionGroup[index];
                    if (existingOption = existingOptions[index + 1]) {
                      // reuse elements
                      lastElement = existingOption.element;
                      if (existingOption.label !== option.label) {
                        updateLabelMap(labelMap, existingOption.label, false);
                        updateLabelMap(labelMap, option.label, true);
                        lastElement.text(existingOption.label = option.label);
                        lastElement.prop('label', existingOption.label);
                      }
                      if (existingOption.id !== option.id) {
                        lastElement.val(existingOption.id = option.id);
                      }
                      // lastElement.prop('selected') provided by jQuery has side-effects
                      if (lastElement[0].selected !== option.selected) {
                        lastElement.prop('selected', existingOption.selected = option.selected);
                        if (msie) {
                          // See #7692
                          // The selected item wouldn't visually update on IE without this.
                          // Tested on Win7: IE9, IE10 and IE11. Future IEs should be tested as well
                          lastElement.prop('selected', existingOption.selected);
                        }
                      }
                    } else {
                      // grow elements
                      // if it's a null option
                      if (option.id === '' && nullOption) {
                        // put back the pre-compiled element
                        element = nullOption;
                      } else {
                        // jQuery(v1.4.2) Bug: We should be able to chain the method calls, but
                        // in this version of jQuery on some browser the .text() returns a string
                        // rather then the element.
                        (element = optionTemplate.clone()).val(option.id).prop('selected', option.selected).attr('selected', option.selected).prop('label', option.label).text(option.label);
                      }
                      existingOptions.push(existingOption = {
                        element: element,
                        label: option.label,
                        id: option.id,
                        selected: option.selected
                      });
                      updateLabelMap(labelMap, option.label, true);
                      if (lastElement) {
                        lastElement.after(element);
                      } else {
                        existingParent.element.append(element);
                      }
                      lastElement = element;
                    }
                  }
                  // remove any excessive OPTIONs in a group
                  index++;
                  // increment since the existingOptions[0] is parent element not OPTION
                  while (existingOptions.length > index) {
                    option = existingOptions.pop();
                    updateLabelMap(labelMap, option.label, false);
                    option.element.remove();
                  }
                }
                // remove any excessive OPTGROUPs from select
                while (optionGroupsCache.length > groupIndex) {
                  // remove all the labels in the option group
                  optionGroup = optionGroupsCache.pop();
                  for (index = 1; index < optionGroup.length; ++index) {
                    updateLabelMap(labelMap, optionGroup[index].label, false);
                  }
                  optionGroup[0].element.remove();
                }
                forEach(labelMap, function (count, label) {
                  if (count > 0) {
                    selectCtrl.addOption(label);
                  } else if (count < 0) {
                    selectCtrl.removeOption(label);
                  }
                });
              }
            }
          }
        };
      }
    ];
  var optionDirective = [
      '$interpolate',
      function ($interpolate) {
        var nullSelectCtrl = {
            addOption: noop,
            removeOption: noop
          };
        return {
          restrict: 'E',
          priority: 100,
          compile: function (element, attr) {
            if (isUndefined(attr.value)) {
              var interpolateFn = $interpolate(element.text(), true);
              if (!interpolateFn) {
                attr.$set('value', element.text());
              }
            }
            return function (scope, element, attr) {
              var selectCtrlName = '$selectController', parent = element.parent(), selectCtrl = parent.data(selectCtrlName) || parent.parent().data(selectCtrlName);
              // in case we are in optgroup
              if (!selectCtrl || !selectCtrl.databound) {
                selectCtrl = nullSelectCtrl;
              }
              if (interpolateFn) {
                scope.$watch(interpolateFn, function interpolateWatchAction(newVal, oldVal) {
                  attr.$set('value', newVal);
                  if (oldVal !== newVal) {
                    selectCtrl.removeOption(oldVal);
                  }
                  selectCtrl.addOption(newVal, element);
                });
              } else {
                selectCtrl.addOption(attr.value, element);
              }
              element.on('$destroy', function () {
                selectCtrl.removeOption(attr.value);
              });
            };
          }
        };
      }
    ];
  var styleDirective = valueFn({
      restrict: 'E',
      terminal: false
    });
  var requiredDirective = function () {
    return {
      restrict: 'A',
      require: '?ngModel',
      link: function (scope, elm, attr, ctrl) {
        if (!ctrl)
          return;
        attr.required = true;
        // force truthy in case we are on non input element
        ctrl.$validators.required = function (modelValue, viewValue) {
          return !attr.required || !ctrl.$isEmpty(viewValue);
        };
        attr.$observe('required', function () {
          ctrl.$validate();
        });
      }
    };
  };
  var patternDirective = function () {
    return {
      restrict: 'A',
      require: '?ngModel',
      link: function (scope, elm, attr, ctrl) {
        if (!ctrl)
          return;
        var regexp, patternExp = attr.ngPattern || attr.pattern;
        attr.$observe('pattern', function (regex) {
          if (isString(regex) && regex.length > 0) {
            regex = new RegExp('^' + regex + '$');
          }
          if (regex && !regex.test) {
            throw minErr('ngPattern')('noregexp', 'Expected {0} to be a RegExp but was {1}. Element: {2}', patternExp, regex, startingTag(elm));
          }
          regexp = regex || undefined;
          ctrl.$validate();
        });
        ctrl.$validators.pattern = function (modelValue, viewValue) {
          // HTML5 pattern constraint validates the input value, so we validate the viewValue
          return ctrl.$isEmpty(viewValue) || isUndefined(regexp) || regexp.test(viewValue);
        };
      }
    };
  };
  var maxlengthDirective = function () {
    return {
      restrict: 'A',
      require: '?ngModel',
      link: function (scope, elm, attr, ctrl) {
        if (!ctrl)
          return;
        var maxlength = -1;
        attr.$observe('maxlength', function (value) {
          var intVal = int(value);
          maxlength = isNaN(intVal) ? -1 : intVal;
          ctrl.$validate();
        });
        ctrl.$validators.maxlength = function (modelValue, viewValue) {
          return maxlength < 0 || ctrl.$isEmpty(viewValue) || viewValue.length <= maxlength;
        };
      }
    };
  };
  var minlengthDirective = function () {
    return {
      restrict: 'A',
      require: '?ngModel',
      link: function (scope, elm, attr, ctrl) {
        if (!ctrl)
          return;
        var minlength = 0;
        attr.$observe('minlength', function (value) {
          minlength = int(value) || 0;
          ctrl.$validate();
        });
        ctrl.$validators.minlength = function (modelValue, viewValue) {
          return ctrl.$isEmpty(viewValue) || viewValue.length >= minlength;
        };
      }
    };
  };
  if (window.angular.bootstrap) {
    //AngularJS is already loaded, so we can return here...
    console.log('WARNING: Tried to load angular more than once.');
    return;
  }
  //try to bind to jquery now so that one can write jqLite(document).ready()
  //but we will rebind on bootstrap again.
  bindJQuery();
  publishExternalAPI(angular);
  jqLite(document).ready(function () {
    angularInit(document, bootstrap);
  });
}(window, document));
!window.angular.$$csp() && window.angular.element(document.head).prepend('<style type="text/css">@charset "UTF-8";[ng\\:cloak],[ng-cloak],[data-ng-cloak],[x-ng-cloak],.ng-cloak,.x-ng-cloak,.ng-hide:not(.ng-hide-animate){display:none !important;}ng\\:form{display:block;}</style>');
/**
 * @license AngularJS v1.3.20
 * (c) 2010-2014 Google, Inc. http://angularjs.org
 * License: MIT
 */
(function (window, angular, undefined) {
  'use strict';
  /* jshint maxlen: false */
  /**
 * @ngdoc module
 * @name ngAnimate
 * @description
 *
 * The `ngAnimate` module provides support for JavaScript, CSS3 transition and CSS3 keyframe animation hooks within existing core and custom directives.
 *
 * <div doc-module-components="ngAnimate"></div>
 *
 * # Usage
 *
 * To see animations in action, all that is required is to define the appropriate CSS classes
 * or to register a JavaScript animation via the `myModule.animation()` function. The directives that support animation automatically are:
 * `ngRepeat`, `ngInclude`, `ngIf`, `ngSwitch`, `ngShow`, `ngHide`, `ngView` and `ngClass`. Custom directives can take advantage of animation
 * by using the `$animate` service.
 *
 * Below is a more detailed breakdown of the supported animation events provided by pre-existing ng directives:
 *
 * | Directive                                                                                                | Supported Animations                                                     |
 * |----------------------------------------------------------------------------------------------------------|--------------------------------------------------------------------------|
 * | {@link ng.directive:ngRepeat#animations ngRepeat}                                                        | enter, leave and move                                                    |
 * | {@link ngRoute.directive:ngView#animations ngView}                                                       | enter and leave                                                          |
 * | {@link ng.directive:ngInclude#animations ngInclude}                                                      | enter and leave                                                          |
 * | {@link ng.directive:ngSwitch#animations ngSwitch}                                                        | enter and leave                                                          |
 * | {@link ng.directive:ngIf#animations ngIf}                                                                | enter and leave                                                          |
 * | {@link ng.directive:ngClass#animations ngClass}                                                          | add and remove (the CSS class(es) present)                               |
 * | {@link ng.directive:ngShow#animations ngShow} & {@link ng.directive:ngHide#animations ngHide}            | add and remove (the ng-hide class value)                                 |
 * | {@link ng.directive:form#animation-hooks form} & {@link ng.directive:ngModel#animation-hooks ngModel}    | add and remove (dirty, pristine, valid, invalid & all other validations) |
 * | {@link module:ngMessages#animations ngMessages}                                                          | add and remove (ng-active & ng-inactive)                                 |
 * | {@link module:ngMessages#animations ngMessage}                                                           | enter and leave                                                          |
 *
 * You can find out more information about animations upon visiting each directive page.
 *
 * Below is an example of how to apply animations to a directive that supports animation hooks:
 *
 * ```html
 * <style type="text/css">
 * .slide.ng-enter, .slide.ng-leave {
 *   -webkit-transition:0.5s linear all;
 *   transition:0.5s linear all;
 * }
 *
 * .slide.ng-enter { }        /&#42; starting animations for enter &#42;/
 * .slide.ng-enter.ng-enter-active { } /&#42; terminal animations for enter &#42;/
 * .slide.ng-leave { }        /&#42; starting animations for leave &#42;/
 * .slide.ng-leave.ng-leave-active { } /&#42; terminal animations for leave &#42;/
 * </style>
 *
 * <!--
 * the animate service will automatically add .ng-enter and .ng-leave to the element
 * to trigger the CSS transition/animations
 * -->
 * <ANY class="slide" ng-include="..."></ANY>
 * ```
 *
 * Keep in mind that, by default, if an animation is running, any child elements cannot be animated
 * until the parent element's animation has completed. This blocking feature can be overridden by
 * placing the `ng-animate-children` attribute on a parent container tag.
 *
 * ```html
 * <div class="slide-animation" ng-if="on" ng-animate-children>
 *   <div class="fade-animation" ng-if="on">
 *     <div class="explode-animation" ng-if="on">
 *        ...
 *     </div>
 *   </div>
 * </div>
 * ```
 *
 * When the `on` expression value changes and an animation is triggered then each of the elements within
 * will all animate without the block being applied to child elements.
 *
 * ## Are animations run when the application starts?
 * No they are not. When an application is bootstrapped Angular will disable animations from running to avoid
 * a frenzy of animations from being triggered as soon as the browser has rendered the screen. For this to work,
 * Angular will wait for two digest cycles until enabling animations. From there on, any animation-triggering
 * layout changes in the application will trigger animations as normal.
 *
 * In addition, upon bootstrap, if the routing system or any directives or load remote data (via $http) then Angular
 * will automatically extend the wait time to enable animations once **all** of the outbound HTTP requests
 * are complete.
 *
 * ## CSS-defined Animations
 * The animate service will automatically apply two CSS classes to the animated element and these two CSS classes
 * are designed to contain the start and end CSS styling. Both CSS transitions and keyframe animations are supported
 * and can be used to play along with this naming structure.
 *
 * The following code below demonstrates how to perform animations using **CSS transitions** with Angular:
 *
 * ```html
 * <style type="text/css">
 * /&#42;
 *  The animate class is apart of the element and the ng-enter class
 *  is attached to the element once the enter animation event is triggered
 * &#42;/
 * .reveal-animation.ng-enter {
 *  -webkit-transition: 1s linear all; /&#42; Safari/Chrome &#42;/
 *  transition: 1s linear all; /&#42; All other modern browsers and IE10+ &#42;/
 *
 *  /&#42; The animation preparation code &#42;/
 *  opacity: 0;
 * }
 *
 * /&#42;
 *  Keep in mind that you want to combine both CSS
 *  classes together to avoid any CSS-specificity
 *  conflicts
 * &#42;/
 * .reveal-animation.ng-enter.ng-enter-active {
 *  /&#42; The animation code itself &#42;/
 *  opacity: 1;
 * }
 * </style>
 *
 * <div class="view-container">
 *   <div ng-view class="reveal-animation"></div>
 * </div>
 * ```
 *
 * The following code below demonstrates how to perform animations using **CSS animations** with Angular:
 *
 * ```html
 * <style type="text/css">
 * .reveal-animation.ng-enter {
 *   -webkit-animation: enter_sequence 1s linear; /&#42; Safari/Chrome &#42;/
 *   animation: enter_sequence 1s linear; /&#42; IE10+ and Future Browsers &#42;/
 * }
 * @-webkit-keyframes enter_sequence {
 *   from { opacity:0; }
 *   to { opacity:1; }
 * }
 * @keyframes enter_sequence {
 *   from { opacity:0; }
 *   to { opacity:1; }
 * }
 * </style>
 *
 * <div class="view-container">
 *   <div ng-view class="reveal-animation"></div>
 * </div>
 * ```
 *
 * Both CSS3 animations and transitions can be used together and the animate service will figure out the correct duration and delay timing.
 *
 * Upon DOM mutation, the event class is added first (something like `ng-enter`), then the browser prepares itself to add
 * the active class (in this case `ng-enter-active`) which then triggers the animation. The animation module will automatically
 * detect the CSS code to determine when the animation ends. Once the animation is over then both CSS classes will be
 * removed from the DOM. If a browser does not support CSS transitions or CSS animations then the animation will start and end
 * immediately resulting in a DOM element that is at its final state. This final state is when the DOM element
 * has no CSS transition/animation classes applied to it.
 *
 * ### Structural transition animations
 *
 * Structural transitions (such as enter, leave and move) will always apply a `0s none` transition
 * value to force the browser into rendering the styles defined in the setup (`.ng-enter`, `.ng-leave`
 * or `.ng-move`) class. This means that any active transition animations operating on the element
 * will be cut off to make way for the enter, leave or move animation.
 *
 * ### Class-based transition animations
 *
 * Class-based transitions refer to transition animations that are triggered when a CSS class is
 * added to or removed from the element (via `$animate.addClass`, `$animate.removeClass`,
 * `$animate.setClass`, or by directives such as `ngClass`, `ngModel` and `form`).
 * They are different when compared to structural animations since they **do not cancel existing
 * animations** nor do they **block successive transitions** from rendering on the same element.
 * This distinction allows for **multiple class-based transitions** to be performed on the same element.
 *
 * In addition to ngAnimate supporting the default (natural) functionality of class-based transition
 * animations, ngAnimate also decorates the element with starting and ending CSS classes to aid the
 * developer in further styling the element throughout the transition animation. Earlier versions
 * of ngAnimate may have caused natural CSS transitions to break and not render properly due to
 * $animate temporarily blocking transitions using `0s none` in order to allow the setup CSS class
 * (the `-add` or `-remove` class) to be applied without triggering an animation. However, as of
 * **version 1.3**, this workaround has been removed with ngAnimate and all non-ngAnimate CSS
 * class transitions are compatible with ngAnimate.
 *
 * There is, however, one special case when dealing with class-based transitions in ngAnimate.
 * When rendering class-based transitions that make use of the setup and active CSS classes
 * (e.g. `.fade-add` and `.fade-add-active` for when `.fade` is added) be sure to define
 * the transition value **on the active CSS class** and not the setup class.
 *
 * ```css
 * .fade-add {
 *   /&#42; remember to place a 0s transition here
 *      to ensure that the styles are applied instantly
 *      even if the element already has a transition style &#42;/
 *   transition:0s linear all;
 *
 *   /&#42; starting CSS styles &#42;/
 *   opacity:1;
 * }
 * .fade-add.fade-add-active {
 *   /&#42; this will be the length of the animation &#42;/
 *   transition:1s linear all;
 *   opacity:0;
 * }
 * ```
 *
 * The setup CSS class (in this case `.fade-add`) also has a transition style property, however, it
 * has a duration of zero. This may not be required, however, incase the browser is unable to render
 * the styling present in this CSS class instantly then it could be that the browser is attempting
 * to perform an unnecessary transition.
 *
 * This workaround, however, does not apply to  standard class-based transitions that are rendered
 * when a CSS class containing a transition is applied to an element:
 *
 * ```css
 * /&#42; this works as expected &#42;/
 * .fade {
 *   transition:1s linear all;
 *   opacity:0;
 * }
 * ```
 *
 * Please keep this in mind when coding the CSS markup that will be used within class-based transitions.
 * Also, try not to mix the two class-based animation flavors together since the CSS code may become
 * overly complex.
 *
 *
 * ### Preventing Collisions With Third Party Libraries
 *
 * Some third-party frameworks place animation duration defaults across many element or className
 * selectors in order to make their code small and reuseable. This can lead to issues with ngAnimate, which
 * is expecting actual animations on these elements and has to wait for their completion.
 *
 * You can prevent this unwanted behavior by using a prefix on all your animation classes:
 *
 * ```css
 * /&#42; prefixed with animate- &#42;/
 * .animate-fade-add.animate-fade-add-active {
 *   transition:1s linear all;
 *   opacity:0;
 * }
 * ```
 *
 * You then configure `$animate` to enforce this prefix:
 *
 * ```js
 * $animateProvider.classNameFilter(/animate-/);
 * ```
 * </div>
 *
 * ### CSS Staggering Animations
 * A Staggering animation is a collection of animations that are issued with a slight delay in between each successive operation resulting in a
 * curtain-like effect. The ngAnimate module (versions >=1.2) supports staggering animations and the stagger effect can be
 * performed by creating a **ng-EVENT-stagger** CSS class and attaching that class to the base CSS class used for
 * the animation. The style property expected within the stagger class can either be a **transition-delay** or an
 * **animation-delay** property (or both if your animation contains both transitions and keyframe animations).
 *
 * ```css
 * .my-animation.ng-enter {
 *   /&#42; standard transition code &#42;/
 *   -webkit-transition: 1s linear all;
 *   transition: 1s linear all;
 *   opacity:0;
 * }
 * .my-animation.ng-enter-stagger {
 *   /&#42; this will have a 100ms delay between each successive leave animation &#42;/
 *   -webkit-transition-delay: 0.1s;
 *   transition-delay: 0.1s;
 *
 *   /&#42; in case the stagger doesn't work then these two values
 *    must be set to 0 to avoid an accidental CSS inheritance &#42;/
 *   -webkit-transition-duration: 0s;
 *   transition-duration: 0s;
 * }
 * .my-animation.ng-enter.ng-enter-active {
 *   /&#42; standard transition styles &#42;/
 *   opacity:1;
 * }
 * ```
 *
 * Staggering animations work by default in ngRepeat (so long as the CSS class is defined). Outside of ngRepeat, to use staggering animations
 * on your own, they can be triggered by firing multiple calls to the same event on $animate. However, the restrictions surrounding this
 * are that each of the elements must have the same CSS className value as well as the same parent element. A stagger operation
 * will also be reset if more than 10ms has passed after the last animation has been fired.
 *
 * The following code will issue the **ng-leave-stagger** event on the element provided:
 *
 * ```js
 * var kids = parent.children();
 *
 * $animate.leave(kids[0]); //stagger index=0
 * $animate.leave(kids[1]); //stagger index=1
 * $animate.leave(kids[2]); //stagger index=2
 * $animate.leave(kids[3]); //stagger index=3
 * $animate.leave(kids[4]); //stagger index=4
 *
 * $timeout(function() {
 *   //stagger has reset itself
 *   $animate.leave(kids[5]); //stagger index=0
 *   $animate.leave(kids[6]); //stagger index=1
 * }, 100, false);
 * ```
 *
 * Stagger animations are currently only supported within CSS-defined animations.
 *
 * ## JavaScript-defined Animations
 * In the event that you do not want to use CSS3 transitions or CSS3 animations or if you wish to offer animations on browsers that do not
 * yet support CSS transitions/animations, then you can make use of JavaScript animations defined inside of your AngularJS module.
 *
 * ```js
 * //!annotate="YourApp" Your AngularJS Module|Replace this or ngModule with the module that you used to define your application.
 * var ngModule = angular.module('YourApp', ['ngAnimate']);
 * ngModule.animation('.my-crazy-animation', function() {
 *   return {
 *     enter: function(element, done) {
 *       //run the animation here and call done when the animation is complete
 *       return function(cancelled) {
 *         //this (optional) function will be called when the animation
 *         //completes or when the animation is cancelled (the cancelled
 *         //flag will be set to true if cancelled).
 *       };
 *     },
 *     leave: function(element, done) { },
 *     move: function(element, done) { },
 *
 *     //animation that can be triggered before the class is added
 *     beforeAddClass: function(element, className, done) { },
 *
 *     //animation that can be triggered after the class is added
 *     addClass: function(element, className, done) { },
 *
 *     //animation that can be triggered before the class is removed
 *     beforeRemoveClass: function(element, className, done) { },
 *
 *     //animation that can be triggered after the class is removed
 *     removeClass: function(element, className, done) { }
 *   };
 * });
 * ```
 *
 * JavaScript-defined animations are created with a CSS-like class selector and a collection of events which are set to run
 * a javascript callback function. When an animation is triggered, $animate will look for a matching animation which fits
 * the element's CSS class attribute value and then run the matching animation event function (if found).
 * In other words, if the CSS classes present on the animated element match any of the JavaScript animations then the callback function will
 * be executed. It should be also noted that only simple, single class selectors are allowed (compound class selectors are not supported).
 *
 * Within a JavaScript animation, an object containing various event callback animation functions is expected to be returned.
 * As explained above, these callbacks are triggered based on the animation event. Therefore if an enter animation is run,
 * and the JavaScript animation is found, then the enter callback will handle that animation (in addition to the CSS keyframe animation
 * or transition code that is defined via a stylesheet).
 *
 *
 * ### Applying Directive-specific Styles to an Animation
 * In some cases a directive or service may want to provide `$animate` with extra details that the animation will
 * include into its animation. Let's say for example we wanted to render an animation that animates an element
 * towards the mouse coordinates as to where the user clicked last. By collecting the X/Y coordinates of the click
 * (via the event parameter) we can set the `top` and `left` styles into an object and pass that into our function
 * call to `$animate.addClass`.
 *
 * ```js
 * canvas.on('click', function(e) {
 *   $animate.addClass(element, 'on', {
 *     to: {
 *       left : e.client.x + 'px',
 *       top : e.client.y + 'px'
 *     }
 *   }):
 * });
 * ```
 *
 * Now when the animation runs, and a transition or keyframe animation is picked up, then the animation itself will
 * also include and transition the styling of the `left` and `top` properties into its running animation. If we want
 * to provide some starting animation values then we can do so by placing the starting animations styles into an object
 * called `from` in the same object as the `to` animations.
 *
 * ```js
 * canvas.on('click', function(e) {
 *   $animate.addClass(element, 'on', {
 *     from: {
 *        position: 'absolute',
 *        left: '0px',
 *        top: '0px'
 *     },
 *     to: {
 *       left : e.client.x + 'px',
 *       top : e.client.y + 'px'
 *     }
 *   }):
 * });
 * ```
 *
 * Once the animation is complete or cancelled then the union of both the before and after styles are applied to the
 * element. If `ngAnimate` is not present then the styles will be applied immediately.
 *
 */
  angular.module('ngAnimate', ['ng']).directive('ngAnimateChildren', function () {
    var NG_ANIMATE_CHILDREN = '$$ngAnimateChildren';
    return function (scope, element, attrs) {
      var val = attrs.ngAnimateChildren;
      if (angular.isString(val) && val.length === 0) {
        //empty attribute
        element.data(NG_ANIMATE_CHILDREN, true);
      } else {
        scope.$watch(val, function (value) {
          element.data(NG_ANIMATE_CHILDREN, !!value);
        });
      }
    };
  }).factory('$$animateReflow', [
    '$$rAF',
    '$document',
    function ($$rAF, $document) {
      var bod = $document[0].body;
      return function (fn) {
        //the returned function acts as the cancellation function
        return $$rAF(function () {
          //the line below will force the browser to perform a repaint
          //so that all the animated elements within the animation frame
          //will be properly updated and drawn on screen. This is
          //required to perform multi-class CSS based animations with
          //Firefox. DO NOT REMOVE THIS LINE. DO NOT OPTIMIZE THIS LINE.
          //THE MINIFIER WILL REMOVE IT OTHERWISE WHICH WILL RESULT IN AN
          //UNPREDICTABLE BUG THAT IS VERY HARD TO TRACK DOWN AND WILL
          //TAKE YEARS AWAY FROM YOUR LIFE!
          fn(bod.offsetWidth);
        });
      };
    }
  ]).config([
    '$provide',
    '$animateProvider',
    function ($provide, $animateProvider) {
      var noop = angular.noop;
      var forEach = angular.forEach;
      var selectors = $animateProvider.$$selectors;
      var isArray = angular.isArray;
      var isString = angular.isString;
      var isObject = angular.isObject;
      var ELEMENT_NODE = 1;
      var NG_ANIMATE_STATE = '$$ngAnimateState';
      var NG_ANIMATE_CHILDREN = '$$ngAnimateChildren';
      var NG_ANIMATE_CLASS_NAME = 'ng-animate';
      var rootAnimateState = { running: true };
      function extractElementNode(element) {
        for (var i = 0; i < element.length; i++) {
          var elm = element[i];
          if (elm.nodeType == ELEMENT_NODE) {
            return elm;
          }
        }
      }
      function prepareElement(element) {
        return element && angular.element(element);
      }
      function stripCommentsFromElement(element) {
        return angular.element(extractElementNode(element));
      }
      function isMatchingElement(elm1, elm2) {
        return extractElementNode(elm1) == extractElementNode(elm2);
      }
      var $$jqLite;
      $provide.decorator('$animate', [
        '$delegate',
        '$$q',
        '$injector',
        '$sniffer',
        '$rootElement',
        '$$asyncCallback',
        '$rootScope',
        '$document',
        '$templateRequest',
        '$$jqLite',
        function ($delegate, $$q, $injector, $sniffer, $rootElement, $$asyncCallback, $rootScope, $document, $templateRequest, $$$jqLite) {
          $$jqLite = $$$jqLite;
          $rootElement.data(NG_ANIMATE_STATE, rootAnimateState);
          // Wait until all directive and route-related templates are downloaded and
          // compiled. The $templateRequest.totalPendingRequests variable keeps track of
          // all of the remote templates being currently downloaded. If there are no
          // templates currently downloading then the watcher will still fire anyway.
          var deregisterWatch = $rootScope.$watch(function () {
              return $templateRequest.totalPendingRequests;
            }, function (val, oldVal) {
              if (val !== 0)
                return;
              deregisterWatch();
              // Now that all templates have been downloaded, $animate will wait until
              // the post digest queue is empty before enabling animations. By having two
              // calls to $postDigest calls we can ensure that the flag is enabled at the
              // very end of the post digest queue. Since all of the animations in $animate
              // use $postDigest, it's important that the code below executes at the end.
              // This basically means that the page is fully downloaded and compiled before
              // any animations are triggered.
              $rootScope.$$postDigest(function () {
                $rootScope.$$postDigest(function () {
                  rootAnimateState.running = false;
                });
              });
            });
          var globalAnimationCounter = 0;
          var classNameFilter = $animateProvider.classNameFilter();
          var isAnimatableClassName = !classNameFilter ? function () {
              return true;
            } : function (className) {
              return classNameFilter.test(className);
            };
          function classBasedAnimationsBlocked(element, setter) {
            var data = element.data(NG_ANIMATE_STATE) || {};
            if (setter) {
              data.running = true;
              data.structural = true;
              element.data(NG_ANIMATE_STATE, data);
            }
            return data.disabled || data.running && data.structural;
          }
          function runAnimationPostDigest(fn) {
            var cancelFn, defer = $$q.defer();
            defer.promise.$$cancelFn = function () {
              cancelFn && cancelFn();
            };
            $rootScope.$$postDigest(function () {
              cancelFn = fn(function () {
                defer.resolve();
              });
            });
            return defer.promise;
          }
          function parseAnimateOptions(options) {
            // some plugin code may still be passing in the callback
            // function as the last param for the $animate methods so
            // it's best to only allow string or array values for now
            if (isObject(options)) {
              if (options.tempClasses && isString(options.tempClasses)) {
                options.tempClasses = options.tempClasses.split(/\s+/);
              }
              return options;
            }
          }
          function resolveElementClasses(element, cache, runningAnimations) {
            runningAnimations = runningAnimations || {};
            var lookup = {};
            forEach(runningAnimations, function (data, selector) {
              forEach(selector.split(' '), function (s) {
                lookup[s] = data;
              });
            });
            var hasClasses = Object.create(null);
            forEach((element.attr('class') || '').split(/\s+/), function (className) {
              hasClasses[className] = true;
            });
            var toAdd = [], toRemove = [];
            forEach(cache && cache.classes || [], function (status, className) {
              var hasClass = hasClasses[className];
              var matchingAnimation = lookup[className] || {};
              // When addClass and removeClass is called then $animate will check to
              // see if addClass and removeClass cancel each other out. When there are
              // more calls to removeClass than addClass then the count falls below 0
              // and then the removeClass animation will be allowed. Otherwise if the
              // count is above 0 then that means an addClass animation will commence.
              // Once an animation is allowed then the code will also check to see if
              // there exists any on-going animation that is already adding or remvoing
              // the matching CSS class.
              if (status === false) {
                //does it have the class or will it have the class
                if (hasClass || matchingAnimation.event == 'addClass') {
                  toRemove.push(className);
                }
              } else if (status === true) {
                //is the class missing or will it be removed?
                if (!hasClass || matchingAnimation.event == 'removeClass') {
                  toAdd.push(className);
                }
              }
            });
            return toAdd.length + toRemove.length > 0 && [
              toAdd.join(' '),
              toRemove.join(' ')
            ];
          }
          function lookup(name) {
            if (name) {
              var matches = [], flagMap = {}, classes = name.substr(1).split('.');
              //the empty string value is the default animation
              //operation which performs CSS transition and keyframe
              //animations sniffing. This is always included for each
              //element animation procedure if the browser supports
              //transitions and/or keyframe animations. The default
              //animation is added to the top of the list to prevent
              //any previous animations from affecting the element styling
              //prior to the element being animated.
              if ($sniffer.transitions || $sniffer.animations) {
                matches.push($injector.get(selectors['']));
              }
              for (var i = 0; i < classes.length; i++) {
                var klass = classes[i], selectorFactoryName = selectors[klass];
                if (selectorFactoryName && !flagMap[klass]) {
                  matches.push($injector.get(selectorFactoryName));
                  flagMap[klass] = true;
                }
              }
              return matches;
            }
          }
          function animationRunner(element, animationEvent, className, options) {
            //transcluded directives may sometimes fire an animation using only comment nodes
            //best to catch this early on to prevent any animation operations from occurring
            var node = element[0];
            if (!node) {
              return;
            }
            if (options) {
              options.to = options.to || {};
              options.from = options.from || {};
            }
            var classNameAdd;
            var classNameRemove;
            if (isArray(className)) {
              classNameAdd = className[0];
              classNameRemove = className[1];
              if (!classNameAdd) {
                className = classNameRemove;
                animationEvent = 'removeClass';
              } else if (!classNameRemove) {
                className = classNameAdd;
                animationEvent = 'addClass';
              } else {
                className = classNameAdd + ' ' + classNameRemove;
              }
            }
            var isSetClassOperation = animationEvent == 'setClass';
            var isClassBased = isSetClassOperation || animationEvent == 'addClass' || animationEvent == 'removeClass' || animationEvent == 'animate';
            var currentClassName = element.attr('class');
            var classes = currentClassName + ' ' + className;
            if (!isAnimatableClassName(classes)) {
              return;
            }
            var beforeComplete = noop, beforeCancel = [], before = [], afterComplete = noop, afterCancel = [], after = [];
            var animationLookup = (' ' + classes).replace(/\s+/g, '.');
            forEach(lookup(animationLookup), function (animationFactory) {
              var created = registerAnimation(animationFactory, animationEvent);
              if (!created && isSetClassOperation) {
                registerAnimation(animationFactory, 'addClass');
                registerAnimation(animationFactory, 'removeClass');
              }
            });
            function registerAnimation(animationFactory, event) {
              var afterFn = animationFactory[event];
              var beforeFn = animationFactory['before' + event.charAt(0).toUpperCase() + event.substr(1)];
              if (afterFn || beforeFn) {
                if (event == 'leave') {
                  beforeFn = afterFn;
                  //when set as null then animation knows to skip this phase
                  afterFn = null;
                }
                after.push({
                  event: event,
                  fn: afterFn
                });
                before.push({
                  event: event,
                  fn: beforeFn
                });
                return true;
              }
            }
            function run(fns, cancellations, allCompleteFn) {
              var animations = [];
              forEach(fns, function (animation) {
                animation.fn && animations.push(animation);
              });
              var count = 0;
              function afterAnimationComplete(index) {
                if (cancellations) {
                  (cancellations[index] || noop)();
                  if (++count < animations.length)
                    return;
                  cancellations = null;
                }
                allCompleteFn();
              }
              //The code below adds directly to the array in order to work with
              //both sync and async animations. Sync animations are when the done()
              //operation is called right away. DO NOT REFACTOR!
              forEach(animations, function (animation, index) {
                var progress = function () {
                  afterAnimationComplete(index);
                };
                switch (animation.event) {
                case 'setClass':
                  cancellations.push(animation.fn(element, classNameAdd, classNameRemove, progress, options));
                  break;
                case 'animate':
                  cancellations.push(animation.fn(element, className, options.from, options.to, progress));
                  break;
                case 'addClass':
                  cancellations.push(animation.fn(element, classNameAdd || className, progress, options));
                  break;
                case 'removeClass':
                  cancellations.push(animation.fn(element, classNameRemove || className, progress, options));
                  break;
                default:
                  cancellations.push(animation.fn(element, progress, options));
                  break;
                }
              });
              if (cancellations && cancellations.length === 0) {
                allCompleteFn();
              }
            }
            return {
              node: node,
              event: animationEvent,
              className: className,
              isClassBased: isClassBased,
              isSetClassOperation: isSetClassOperation,
              applyStyles: function () {
                if (options) {
                  element.css(angular.extend(options.from || {}, options.to || {}));
                }
              },
              before: function (allCompleteFn) {
                beforeComplete = allCompleteFn;
                run(before, beforeCancel, function () {
                  beforeComplete = noop;
                  allCompleteFn();
                });
              },
              after: function (allCompleteFn) {
                afterComplete = allCompleteFn;
                run(after, afterCancel, function () {
                  afterComplete = noop;
                  allCompleteFn();
                });
              },
              cancel: function () {
                if (beforeCancel) {
                  forEach(beforeCancel, function (cancelFn) {
                    (cancelFn || noop)(true);
                  });
                  beforeComplete(true);
                }
                if (afterCancel) {
                  forEach(afterCancel, function (cancelFn) {
                    (cancelFn || noop)(true);
                  });
                  afterComplete(true);
                }
              }
            };
          }
          /**
       * @ngdoc service
       * @name $animate
       * @kind object
       *
       * @description
       * The `$animate` service provides animation detection support while performing DOM operations (enter, leave and move) as well as during addClass and removeClass operations.
       * When any of these operations are run, the $animate service
       * will examine any JavaScript-defined animations (which are defined by using the $animateProvider provider object)
       * as well as any CSS-defined animations against the CSS classes present on the element once the DOM operation is run.
       *
       * The `$animate` service is used behind the scenes with pre-existing directives and animation with these directives
       * will work out of the box without any extra configuration.
       *
       * Requires the {@link ngAnimate `ngAnimate`} module to be installed.
       *
       * Please visit the {@link ngAnimate `ngAnimate`} module overview page learn more about how to use animations in your application.
       * ## Callback Promises
       * With AngularJS 1.3, each of the animation methods, on the `$animate` service, return a promise when called. The
       * promise itself is then resolved once the animation has completed itself, has been cancelled or has been
       * skipped due to animations being disabled. (Note that even if the animation is cancelled it will still
       * call the resolve function of the animation.)
       *
       * ```js
       * $animate.enter(element, container).then(function() {
       *   //...this is called once the animation is complete...
       * });
       * ```
       *
       * Also note that, due to the nature of the callback promise, if any Angular-specific code (like changing the scope,
       * location of the page, etc...) is executed within the callback promise then be sure to wrap the code using
       * `$scope.$apply(...)`;
       *
       * ```js
       * $animate.leave(element).then(function() {
       *   $scope.$apply(function() {
       *     $location.path('/new-page');
       *   });
       * });
       * ```
       *
       * An animation can also be cancelled by calling the `$animate.cancel(promise)` method with the provided
       * promise that was returned when the animation was started.
       *
       * ```js
       * var promise = $animate.addClass(element, 'super-long-animation');
       * promise.then(function() {
       *   //this will still be called even if cancelled
       * });
       *
       * element.on('click', function() {
       *   //tooo lazy to wait for the animation to end
       *   $animate.cancel(promise);
       * });
       * ```
       *
       * (Keep in mind that the promise cancellation is unique to `$animate` since promises in
       * general cannot be cancelled.)
       *
       */
          return {
            animate: function (element, from, to, className, options) {
              className = className || 'ng-inline-animate';
              options = parseAnimateOptions(options) || {};
              options.from = to ? from : null;
              options.to = to ? to : from;
              return runAnimationPostDigest(function (done) {
                return performAnimation('animate', className, stripCommentsFromElement(element), null, null, noop, options, done);
              });
            },
            enter: function (element, parentElement, afterElement, options) {
              options = parseAnimateOptions(options);
              element = angular.element(element);
              parentElement = prepareElement(parentElement);
              afterElement = prepareElement(afterElement);
              classBasedAnimationsBlocked(element, true);
              $delegate.enter(element, parentElement, afterElement);
              return runAnimationPostDigest(function (done) {
                return performAnimation('enter', 'ng-enter', stripCommentsFromElement(element), parentElement, afterElement, noop, options, done);
              });
            },
            leave: function (element, options) {
              options = parseAnimateOptions(options);
              element = angular.element(element);
              cancelChildAnimations(element);
              classBasedAnimationsBlocked(element, true);
              return runAnimationPostDigest(function (done) {
                return performAnimation('leave', 'ng-leave', stripCommentsFromElement(element), null, null, function () {
                  $delegate.leave(element);
                }, options, done);
              });
            },
            move: function (element, parentElement, afterElement, options) {
              options = parseAnimateOptions(options);
              element = angular.element(element);
              parentElement = prepareElement(parentElement);
              afterElement = prepareElement(afterElement);
              cancelChildAnimations(element);
              classBasedAnimationsBlocked(element, true);
              $delegate.move(element, parentElement, afterElement);
              return runAnimationPostDigest(function (done) {
                return performAnimation('move', 'ng-move', stripCommentsFromElement(element), parentElement, afterElement, noop, options, done);
              });
            },
            addClass: function (element, className, options) {
              return this.setClass(element, className, [], options);
            },
            removeClass: function (element, className, options) {
              return this.setClass(element, [], className, options);
            },
            setClass: function (element, add, remove, options) {
              options = parseAnimateOptions(options);
              var STORAGE_KEY = '$$animateClasses';
              element = angular.element(element);
              element = stripCommentsFromElement(element);
              if (classBasedAnimationsBlocked(element)) {
                return $delegate.$$setClassImmediately(element, add, remove, options);
              }
              // we're using a combined array for both the add and remove
              // operations since the ORDER OF addClass and removeClass matters
              var classes, cache = element.data(STORAGE_KEY);
              var hasCache = !!cache;
              if (!cache) {
                cache = {};
                cache.classes = {};
              }
              classes = cache.classes;
              add = isArray(add) ? add : add.split(' ');
              forEach(add, function (c) {
                if (c && c.length) {
                  classes[c] = true;
                }
              });
              remove = isArray(remove) ? remove : remove.split(' ');
              forEach(remove, function (c) {
                if (c && c.length) {
                  classes[c] = false;
                }
              });
              if (hasCache) {
                if (options && cache.options) {
                  cache.options = angular.extend(cache.options || {}, options);
                }
                //the digest cycle will combine all the animations into one function
                return cache.promise;
              } else {
                element.data(STORAGE_KEY, cache = {
                  classes: classes,
                  options: options
                });
              }
              return cache.promise = runAnimationPostDigest(function (done) {
                var cache, parentNode, parentElement, elementNode = extractElementNode(element);
                if (elementNode) {
                  cache = element.data(STORAGE_KEY);
                  element.removeData(STORAGE_KEY);
                  parentElement = element.parent();
                  parentNode = elementNode.parentNode;
                }
                // TODO(matsko): move this code into the animationsDisabled() function once #8092 is fixed
                if (!parentNode || parentNode['$$NG_REMOVED'] || elementNode['$$NG_REMOVED']) {
                  done();
                  return;
                }
                var state = element.data(NG_ANIMATE_STATE) || {};
                var classes = resolveElementClasses(element, cache, state.active);
                return !classes ? done() : performAnimation('setClass', classes, element, parentElement, null, function () {
                  if (classes[0])
                    $delegate.$$addClassImmediately(element, classes[0]);
                  if (classes[1])
                    $delegate.$$removeClassImmediately(element, classes[1]);
                }, cache.options, done);
              });
            },
            cancel: function (promise) {
              promise.$$cancelFn();
            },
            enabled: function (value, element) {
              switch (arguments.length) {
              case 2:
                if (value) {
                  cleanup(element);
                } else {
                  var data = element.data(NG_ANIMATE_STATE) || {};
                  data.disabled = true;
                  element.data(NG_ANIMATE_STATE, data);
                }
                break;
              case 1:
                rootAnimateState.disabled = !value;
                break;
              default:
                value = !rootAnimateState.disabled;
                break;
              }
              return !!value;
            }
          };
          /*
        all animations call this shared animation triggering function internally.
        The animationEvent variable refers to the JavaScript animation event that will be triggered
        and the className value is the name of the animation that will be applied within the
        CSS code. Element, `parentElement` and `afterElement` are provided DOM elements for the animation
        and the onComplete callback will be fired once the animation is fully complete.
      */
          function performAnimation(animationEvent, className, element, parentElement, afterElement, domOperation, options, doneCallback) {
            var noopCancel = noop;
            var runner = animationRunner(element, animationEvent, className, options);
            if (!runner) {
              fireDOMOperation();
              fireBeforeCallbackAsync();
              fireAfterCallbackAsync();
              closeAnimation();
              return noopCancel;
            }
            animationEvent = runner.event;
            className = runner.className;
            var elementEvents = angular.element._data(runner.node);
            elementEvents = elementEvents && elementEvents.events;
            if (!parentElement) {
              parentElement = afterElement ? afterElement.parent() : element.parent();
            }
            //skip the animation if animations are disabled, a parent is already being animated,
            //the element is not currently attached to the document body or then completely close
            //the animation if any matching animations are not found at all.
            //NOTE: IE8 + IE9 should close properly (run closeAnimation()) in case an animation was found.
            if (animationsDisabled(element, parentElement)) {
              fireDOMOperation();
              fireBeforeCallbackAsync();
              fireAfterCallbackAsync();
              closeAnimation();
              return noopCancel;
            }
            var ngAnimateState = element.data(NG_ANIMATE_STATE) || {};
            var runningAnimations = ngAnimateState.active || {};
            var totalActiveAnimations = ngAnimateState.totalActive || 0;
            var lastAnimation = ngAnimateState.last;
            var skipAnimation = false;
            if (totalActiveAnimations > 0) {
              var animationsToCancel = [];
              if (!runner.isClassBased) {
                if (animationEvent == 'leave' && runningAnimations['ng-leave']) {
                  skipAnimation = true;
                } else {
                  //cancel all animations when a structural animation takes place
                  for (var klass in runningAnimations) {
                    animationsToCancel.push(runningAnimations[klass]);
                  }
                  ngAnimateState = {};
                  cleanup(element, true);
                }
              } else if (lastAnimation.event == 'setClass') {
                animationsToCancel.push(lastAnimation);
                cleanup(element, className);
              } else if (runningAnimations[className]) {
                var current = runningAnimations[className];
                if (current.event == animationEvent) {
                  skipAnimation = true;
                } else {
                  animationsToCancel.push(current);
                  cleanup(element, className);
                }
              }
              if (animationsToCancel.length > 0) {
                forEach(animationsToCancel, function (operation) {
                  operation.cancel();
                });
              }
            }
            if (runner.isClassBased && !runner.isSetClassOperation && animationEvent != 'animate' && !skipAnimation) {
              skipAnimation = animationEvent == 'addClass' == element.hasClass(className);  //opposite of XOR
            }
            if (skipAnimation) {
              fireDOMOperation();
              fireBeforeCallbackAsync();
              fireAfterCallbackAsync();
              fireDoneCallbackAsync();
              return noopCancel;
            }
            runningAnimations = ngAnimateState.active || {};
            totalActiveAnimations = ngAnimateState.totalActive || 0;
            if (animationEvent == 'leave') {
              //there's no need to ever remove the listener since the element
              //will be removed (destroyed) after the leave animation ends or
              //is cancelled midway
              element.one('$destroy', function (e) {
                var element = angular.element(this);
                var state = element.data(NG_ANIMATE_STATE);
                if (state) {
                  var activeLeaveAnimation = state.active['ng-leave'];
                  if (activeLeaveAnimation) {
                    activeLeaveAnimation.cancel();
                    cleanup(element, 'ng-leave');
                  }
                }
              });
            }
            //the ng-animate class does nothing, but it's here to allow for
            //parent animations to find and cancel child animations when needed
            $$jqLite.addClass(element, NG_ANIMATE_CLASS_NAME);
            if (options && options.tempClasses) {
              forEach(options.tempClasses, function (className) {
                $$jqLite.addClass(element, className);
              });
            }
            var localAnimationCount = globalAnimationCounter++;
            totalActiveAnimations++;
            runningAnimations[className] = runner;
            element.data(NG_ANIMATE_STATE, {
              last: runner,
              active: runningAnimations,
              index: localAnimationCount,
              totalActive: totalActiveAnimations
            });
            //first we run the before animations and when all of those are complete
            //then we perform the DOM operation and run the next set of animations
            fireBeforeCallbackAsync();
            runner.before(function (cancelled) {
              var data = element.data(NG_ANIMATE_STATE);
              cancelled = cancelled || !data || !data.active[className] || runner.isClassBased && data.active[className].event != animationEvent;
              fireDOMOperation();
              if (cancelled === true) {
                closeAnimation();
              } else {
                fireAfterCallbackAsync();
                runner.after(closeAnimation);
              }
            });
            return runner.cancel;
            function fireDOMCallback(animationPhase) {
              var eventName = '$animate:' + animationPhase;
              if (elementEvents && elementEvents[eventName] && elementEvents[eventName].length > 0) {
                $$asyncCallback(function () {
                  element.triggerHandler(eventName, {
                    event: animationEvent,
                    className: className
                  });
                });
              }
            }
            function fireBeforeCallbackAsync() {
              fireDOMCallback('before');
            }
            function fireAfterCallbackAsync() {
              fireDOMCallback('after');
            }
            function fireDoneCallbackAsync() {
              fireDOMCallback('close');
              doneCallback();
            }
            //it is less complicated to use a flag than managing and canceling
            //timeouts containing multiple callbacks.
            function fireDOMOperation() {
              if (!fireDOMOperation.hasBeenRun) {
                fireDOMOperation.hasBeenRun = true;
                domOperation();
              }
            }
            function closeAnimation() {
              if (!closeAnimation.hasBeenRun) {
                if (runner) {
                  //the runner doesn't exist if it fails to instantiate
                  runner.applyStyles();
                }
                closeAnimation.hasBeenRun = true;
                if (options && options.tempClasses) {
                  forEach(options.tempClasses, function (className) {
                    $$jqLite.removeClass(element, className);
                  });
                }
                var data = element.data(NG_ANIMATE_STATE);
                if (data) {
                  /* only structural animations wait for reflow before removing an
                 animation, but class-based animations don't. An example of this
                 failing would be when a parent HTML tag has a ng-class attribute
                 causing ALL directives below to skip animations during the digest */
                  if (runner && runner.isClassBased) {
                    cleanup(element, className);
                  } else {
                    $$asyncCallback(function () {
                      var data = element.data(NG_ANIMATE_STATE) || {};
                      if (localAnimationCount == data.index) {
                        cleanup(element, className, animationEvent);
                      }
                    });
                    element.data(NG_ANIMATE_STATE, data);
                  }
                }
                fireDoneCallbackAsync();
              }
            }
          }
          function cancelChildAnimations(element) {
            var node = extractElementNode(element);
            if (node) {
              var nodes = angular.isFunction(node.getElementsByClassName) ? node.getElementsByClassName(NG_ANIMATE_CLASS_NAME) : node.querySelectorAll('.' + NG_ANIMATE_CLASS_NAME);
              forEach(nodes, function (element) {
                element = angular.element(element);
                var data = element.data(NG_ANIMATE_STATE);
                if (data && data.active) {
                  forEach(data.active, function (runner) {
                    runner.cancel();
                  });
                }
              });
            }
          }
          function cleanup(element, className) {
            if (isMatchingElement(element, $rootElement)) {
              if (!rootAnimateState.disabled) {
                rootAnimateState.running = false;
                rootAnimateState.structural = false;
              }
            } else if (className) {
              var data = element.data(NG_ANIMATE_STATE) || {};
              var removeAnimations = className === true;
              if (!removeAnimations && data.active && data.active[className]) {
                data.totalActive--;
                delete data.active[className];
              }
              if (removeAnimations || !data.totalActive) {
                $$jqLite.removeClass(element, NG_ANIMATE_CLASS_NAME);
                element.removeData(NG_ANIMATE_STATE);
              }
            }
          }
          function animationsDisabled(element, parentElement) {
            if (rootAnimateState.disabled) {
              return true;
            }
            if (isMatchingElement(element, $rootElement)) {
              return rootAnimateState.running;
            }
            var allowChildAnimations, parentRunningAnimation, hasParent;
            do {
              //the element did not reach the root element which means that it
              //is not apart of the DOM. Therefore there is no reason to do
              //any animations on it
              if (parentElement.length === 0)
                break;
              var isRoot = isMatchingElement(parentElement, $rootElement);
              var state = isRoot ? rootAnimateState : parentElement.data(NG_ANIMATE_STATE) || {};
              if (state.disabled) {
                return true;
              }
              //no matter what, for an animation to work it must reach the root element
              //this implies that the element is attached to the DOM when the animation is run
              if (isRoot) {
                hasParent = true;
              }
              //once a flag is found that is strictly false then everything before
              //it will be discarded and all child animations will be restricted
              if (allowChildAnimations !== false) {
                var animateChildrenFlag = parentElement.data(NG_ANIMATE_CHILDREN);
                if (angular.isDefined(animateChildrenFlag)) {
                  allowChildAnimations = animateChildrenFlag;
                }
              }
              parentRunningAnimation = parentRunningAnimation || state.running || state.last && !state.last.isClassBased;
            } while (parentElement = parentElement.parent());
            return !hasParent || !allowChildAnimations && parentRunningAnimation;
          }
        }
      ]);
      $animateProvider.register('', [
        '$window',
        '$sniffer',
        '$timeout',
        '$$animateReflow',
        function ($window, $sniffer, $timeout, $$animateReflow) {
          // Detect proper transitionend/animationend event names.
          var CSS_PREFIX = '', TRANSITION_PROP, TRANSITIONEND_EVENT, ANIMATION_PROP, ANIMATIONEND_EVENT;
          // If unprefixed events are not supported but webkit-prefixed are, use the latter.
          // Otherwise, just use W3C names, browsers not supporting them at all will just ignore them.
          // Note: Chrome implements `window.onwebkitanimationend` and doesn't implement `window.onanimationend`
          // but at the same time dispatches the `animationend` event and not `webkitAnimationEnd`.
          // Register both events in case `window.onanimationend` is not supported because of that,
          // do the same for `transitionend` as Safari is likely to exhibit similar behavior.
          // Also, the only modern browser that uses vendor prefixes for transitions/keyframes is webkit
          // therefore there is no reason to test anymore for other vendor prefixes: http://caniuse.com/#search=transition
          if (window.ontransitionend === undefined && window.onwebkittransitionend !== undefined) {
            CSS_PREFIX = '-webkit-';
            TRANSITION_PROP = 'WebkitTransition';
            TRANSITIONEND_EVENT = 'webkitTransitionEnd transitionend';
          } else {
            TRANSITION_PROP = 'transition';
            TRANSITIONEND_EVENT = 'transitionend';
          }
          if (window.onanimationend === undefined && window.onwebkitanimationend !== undefined) {
            CSS_PREFIX = '-webkit-';
            ANIMATION_PROP = 'WebkitAnimation';
            ANIMATIONEND_EVENT = 'webkitAnimationEnd animationend';
          } else {
            ANIMATION_PROP = 'animation';
            ANIMATIONEND_EVENT = 'animationend';
          }
          var DURATION_KEY = 'Duration';
          var PROPERTY_KEY = 'Property';
          var DELAY_KEY = 'Delay';
          var ANIMATION_ITERATION_COUNT_KEY = 'IterationCount';
          var ANIMATION_PLAYSTATE_KEY = 'PlayState';
          var NG_ANIMATE_PARENT_KEY = '$$ngAnimateKey';
          var NG_ANIMATE_CSS_DATA_KEY = '$$ngAnimateCSS3Data';
          var ELAPSED_TIME_MAX_DECIMAL_PLACES = 3;
          var CLOSING_TIME_BUFFER = 1.5;
          var ONE_SECOND = 1000;
          var lookupCache = {};
          var parentCounter = 0;
          var animationReflowQueue = [];
          var cancelAnimationReflow;
          function clearCacheAfterReflow() {
            if (!cancelAnimationReflow) {
              cancelAnimationReflow = $$animateReflow(function () {
                animationReflowQueue = [];
                cancelAnimationReflow = null;
                lookupCache = {};
              });
            }
          }
          function afterReflow(element, callback) {
            if (cancelAnimationReflow) {
              cancelAnimationReflow();
            }
            animationReflowQueue.push(callback);
            cancelAnimationReflow = $$animateReflow(function () {
              forEach(animationReflowQueue, function (fn) {
                fn();
              });
              animationReflowQueue = [];
              cancelAnimationReflow = null;
              lookupCache = {};
            });
          }
          var closingTimer = null;
          var closingTimestamp = 0;
          var animationElementQueue = [];
          function animationCloseHandler(element, totalTime) {
            var node = extractElementNode(element);
            element = angular.element(node);
            //this item will be garbage collected by the closing
            //animation timeout
            animationElementQueue.push(element);
            //but it may not need to cancel out the existing timeout
            //if the timestamp is less than the previous one
            var futureTimestamp = Date.now() + totalTime;
            if (futureTimestamp <= closingTimestamp) {
              return;
            }
            $timeout.cancel(closingTimer);
            closingTimestamp = futureTimestamp;
            closingTimer = $timeout(function () {
              closeAllAnimations(animationElementQueue);
              animationElementQueue = [];
            }, totalTime, false);
          }
          function closeAllAnimations(elements) {
            forEach(elements, function (element) {
              var elementData = element.data(NG_ANIMATE_CSS_DATA_KEY);
              if (elementData) {
                forEach(elementData.closeAnimationFns, function (fn) {
                  fn();
                });
              }
            });
          }
          function getElementAnimationDetails(element, cacheKey) {
            var data = cacheKey ? lookupCache[cacheKey] : null;
            if (!data) {
              var transitionDuration = 0;
              var transitionDelay = 0;
              var animationDuration = 0;
              var animationDelay = 0;
              //we want all the styles defined before and after
              forEach(element, function (element) {
                if (element.nodeType == ELEMENT_NODE) {
                  var elementStyles = $window.getComputedStyle(element) || {};
                  var transitionDurationStyle = elementStyles[TRANSITION_PROP + DURATION_KEY];
                  transitionDuration = Math.max(parseMaxTime(transitionDurationStyle), transitionDuration);
                  var transitionDelayStyle = elementStyles[TRANSITION_PROP + DELAY_KEY];
                  transitionDelay = Math.max(parseMaxTime(transitionDelayStyle), transitionDelay);
                  var animationDelayStyle = elementStyles[ANIMATION_PROP + DELAY_KEY];
                  animationDelay = Math.max(parseMaxTime(elementStyles[ANIMATION_PROP + DELAY_KEY]), animationDelay);
                  var aDuration = parseMaxTime(elementStyles[ANIMATION_PROP + DURATION_KEY]);
                  if (aDuration > 0) {
                    aDuration *= parseInt(elementStyles[ANIMATION_PROP + ANIMATION_ITERATION_COUNT_KEY], 10) || 1;
                  }
                  animationDuration = Math.max(aDuration, animationDuration);
                }
              });
              data = {
                total: 0,
                transitionDelay: transitionDelay,
                transitionDuration: transitionDuration,
                animationDelay: animationDelay,
                animationDuration: animationDuration
              };
              if (cacheKey) {
                lookupCache[cacheKey] = data;
              }
            }
            return data;
          }
          function parseMaxTime(str) {
            var maxValue = 0;
            var values = isString(str) ? str.split(/\s*,\s*/) : [];
            forEach(values, function (value) {
              maxValue = Math.max(parseFloat(value) || 0, maxValue);
            });
            return maxValue;
          }
          function getCacheKey(element) {
            var parentElement = element.parent();
            var parentID = parentElement.data(NG_ANIMATE_PARENT_KEY);
            if (!parentID) {
              parentElement.data(NG_ANIMATE_PARENT_KEY, ++parentCounter);
              parentID = parentCounter;
            }
            return parentID + '-' + extractElementNode(element).getAttribute('class');
          }
          function animateSetup(animationEvent, element, className, styles) {
            var structural = [
                'ng-enter',
                'ng-leave',
                'ng-move'
              ].indexOf(className) >= 0;
            var cacheKey = getCacheKey(element);
            var eventCacheKey = cacheKey + ' ' + className;
            var itemIndex = lookupCache[eventCacheKey] ? ++lookupCache[eventCacheKey].total : 0;
            var stagger = {};
            if (itemIndex > 0) {
              var staggerClassName = className + '-stagger';
              var staggerCacheKey = cacheKey + ' ' + staggerClassName;
              var applyClasses = !lookupCache[staggerCacheKey];
              applyClasses && $$jqLite.addClass(element, staggerClassName);
              stagger = getElementAnimationDetails(element, staggerCacheKey);
              applyClasses && $$jqLite.removeClass(element, staggerClassName);
            }
            $$jqLite.addClass(element, className);
            var formerData = element.data(NG_ANIMATE_CSS_DATA_KEY) || {};
            var timings = getElementAnimationDetails(element, eventCacheKey);
            var transitionDuration = timings.transitionDuration;
            var animationDuration = timings.animationDuration;
            if (structural && transitionDuration === 0 && animationDuration === 0) {
              $$jqLite.removeClass(element, className);
              return false;
            }
            var blockTransition = styles || structural && transitionDuration > 0;
            var blockAnimation = animationDuration > 0 && stagger.animationDelay > 0 && stagger.animationDuration === 0;
            var closeAnimationFns = formerData.closeAnimationFns || [];
            element.data(NG_ANIMATE_CSS_DATA_KEY, {
              stagger: stagger,
              cacheKey: eventCacheKey,
              running: formerData.running || 0,
              itemIndex: itemIndex,
              blockTransition: blockTransition,
              closeAnimationFns: closeAnimationFns
            });
            var node = extractElementNode(element);
            if (blockTransition) {
              blockTransitions(node, true);
              if (styles) {
                element.css(styles);
              }
            }
            if (blockAnimation) {
              blockAnimations(node, true);
            }
            return true;
          }
          function animateRun(animationEvent, element, className, activeAnimationComplete, styles) {
            var node = extractElementNode(element);
            var elementData = element.data(NG_ANIMATE_CSS_DATA_KEY);
            if (node.getAttribute('class').indexOf(className) == -1 || !elementData) {
              activeAnimationComplete();
              return;
            }
            var activeClassName = '';
            var pendingClassName = '';
            forEach(className.split(' '), function (klass, i) {
              var prefix = (i > 0 ? ' ' : '') + klass;
              activeClassName += prefix + '-active';
              pendingClassName += prefix + '-pending';
            });
            var style = '';
            var appliedStyles = [];
            var itemIndex = elementData.itemIndex;
            var stagger = elementData.stagger;
            var staggerTime = 0;
            if (itemIndex > 0) {
              var transitionStaggerDelay = 0;
              if (stagger.transitionDelay > 0 && stagger.transitionDuration === 0) {
                transitionStaggerDelay = stagger.transitionDelay * itemIndex;
              }
              var animationStaggerDelay = 0;
              if (stagger.animationDelay > 0 && stagger.animationDuration === 0) {
                animationStaggerDelay = stagger.animationDelay * itemIndex;
                appliedStyles.push(CSS_PREFIX + 'animation-play-state');
              }
              staggerTime = Math.round(Math.max(transitionStaggerDelay, animationStaggerDelay) * 100) / 100;
            }
            if (!staggerTime) {
              $$jqLite.addClass(element, activeClassName);
              if (elementData.blockTransition) {
                blockTransitions(node, false);
              }
            }
            var eventCacheKey = elementData.cacheKey + ' ' + activeClassName;
            var timings = getElementAnimationDetails(element, eventCacheKey);
            var maxDuration = Math.max(timings.transitionDuration, timings.animationDuration);
            if (maxDuration === 0) {
              $$jqLite.removeClass(element, activeClassName);
              animateClose(element, className);
              activeAnimationComplete();
              return;
            }
            if (!staggerTime && styles && Object.keys(styles).length > 0) {
              if (!timings.transitionDuration) {
                element.css('transition', timings.animationDuration + 's linear all');
                appliedStyles.push('transition');
              }
              element.css(styles);
            }
            var maxDelay = Math.max(timings.transitionDelay, timings.animationDelay);
            var maxDelayTime = maxDelay * ONE_SECOND;
            if (appliedStyles.length > 0) {
              //the element being animated may sometimes contain comment nodes in
              //the jqLite object, so we're safe to use a single variable to house
              //the styles since there is always only one element being animated
              var oldStyle = node.getAttribute('style') || '';
              if (oldStyle.charAt(oldStyle.length - 1) !== ';') {
                oldStyle += ';';
              }
              node.setAttribute('style', oldStyle + ' ' + style);
            }
            var startTime = Date.now();
            var css3AnimationEvents = ANIMATIONEND_EVENT + ' ' + TRANSITIONEND_EVENT;
            var animationTime = (maxDelay + maxDuration) * CLOSING_TIME_BUFFER;
            var totalTime = (staggerTime + animationTime) * ONE_SECOND;
            var staggerTimeout;
            if (staggerTime > 0) {
              $$jqLite.addClass(element, pendingClassName);
              staggerTimeout = $timeout(function () {
                staggerTimeout = null;
                if (timings.transitionDuration > 0) {
                  blockTransitions(node, false);
                }
                if (timings.animationDuration > 0) {
                  blockAnimations(node, false);
                }
                $$jqLite.addClass(element, activeClassName);
                $$jqLite.removeClass(element, pendingClassName);
                if (styles) {
                  if (timings.transitionDuration === 0) {
                    element.css('transition', timings.animationDuration + 's linear all');
                  }
                  element.css(styles);
                  appliedStyles.push('transition');
                }
              }, staggerTime * ONE_SECOND, false);
            }
            element.on(css3AnimationEvents, onAnimationProgress);
            elementData.closeAnimationFns.push(function () {
              onEnd();
              activeAnimationComplete();
            });
            elementData.running++;
            animationCloseHandler(element, totalTime);
            return onEnd;
            // This will automatically be called by $animate so
            // there is no need to attach this internally to the
            // timeout done method.
            function onEnd() {
              element.off(css3AnimationEvents, onAnimationProgress);
              $$jqLite.removeClass(element, activeClassName);
              $$jqLite.removeClass(element, pendingClassName);
              if (staggerTimeout) {
                $timeout.cancel(staggerTimeout);
              }
              animateClose(element, className);
              var node = extractElementNode(element);
              for (var i in appliedStyles) {
                node.style.removeProperty(appliedStyles[i]);
              }
            }
            function onAnimationProgress(event) {
              event.stopPropagation();
              var ev = event.originalEvent || event;
              var timeStamp = ev.$manualTimeStamp || ev.timeStamp || Date.now();
              /* Firefox (or possibly just Gecko) likes to not round values up
           * when a ms measurement is used for the animation */
              var elapsedTime = parseFloat(ev.elapsedTime.toFixed(ELAPSED_TIME_MAX_DECIMAL_PLACES));
              /* $manualTimeStamp is a mocked timeStamp value which is set
           * within browserTrigger(). This is only here so that tests can
           * mock animations properly. Real events fallback to event.timeStamp,
           * or, if they don't, then a timeStamp is automatically created for them.
           * We're checking to see if the timeStamp surpasses the expected delay,
           * but we're using elapsedTime instead of the timeStamp on the 2nd
           * pre-condition since animations sometimes close off early */
              if (Math.max(timeStamp - startTime, 0) >= maxDelayTime && elapsedTime >= maxDuration) {
                activeAnimationComplete();
              }
            }
          }
          function blockTransitions(node, bool) {
            node.style[TRANSITION_PROP + PROPERTY_KEY] = bool ? 'none' : '';
          }
          function blockAnimations(node, bool) {
            node.style[ANIMATION_PROP + ANIMATION_PLAYSTATE_KEY] = bool ? 'paused' : '';
          }
          function animateBefore(animationEvent, element, className, styles) {
            if (animateSetup(animationEvent, element, className, styles)) {
              return function (cancelled) {
                cancelled && animateClose(element, className);
              };
            }
          }
          function animateAfter(animationEvent, element, className, afterAnimationComplete, styles) {
            if (element.data(NG_ANIMATE_CSS_DATA_KEY)) {
              return animateRun(animationEvent, element, className, afterAnimationComplete, styles);
            } else {
              animateClose(element, className);
              afterAnimationComplete();
            }
          }
          function animate(animationEvent, element, className, animationComplete, options) {
            //If the animateSetup function doesn't bother returning a
            //cancellation function then it means that there is no animation
            //to perform at all
            var preReflowCancellation = animateBefore(animationEvent, element, className, options.from);
            if (!preReflowCancellation) {
              clearCacheAfterReflow();
              animationComplete();
              return;
            }
            //There are two cancellation functions: one is before the first
            //reflow animation and the second is during the active state
            //animation. The first function will take care of removing the
            //data from the element which will not make the 2nd animation
            //happen in the first place
            var cancel = preReflowCancellation;
            afterReflow(element, function () {
              //once the reflow is complete then we point cancel to
              //the new cancellation function which will remove all of the
              //animation properties from the active animation
              cancel = animateAfter(animationEvent, element, className, animationComplete, options.to);
            });
            return function (cancelled) {
              (cancel || noop)(cancelled);
            };
          }
          function animateClose(element, className) {
            $$jqLite.removeClass(element, className);
            var data = element.data(NG_ANIMATE_CSS_DATA_KEY);
            if (data) {
              if (data.running) {
                data.running--;
              }
              if (!data.running || data.running === 0) {
                element.removeData(NG_ANIMATE_CSS_DATA_KEY);
              }
            }
          }
          return {
            animate: function (element, className, from, to, animationCompleted, options) {
              options = options || {};
              options.from = from;
              options.to = to;
              return animate('animate', element, className, animationCompleted, options);
            },
            enter: function (element, animationCompleted, options) {
              options = options || {};
              return animate('enter', element, 'ng-enter', animationCompleted, options);
            },
            leave: function (element, animationCompleted, options) {
              options = options || {};
              return animate('leave', element, 'ng-leave', animationCompleted, options);
            },
            move: function (element, animationCompleted, options) {
              options = options || {};
              return animate('move', element, 'ng-move', animationCompleted, options);
            },
            beforeSetClass: function (element, add, remove, animationCompleted, options) {
              options = options || {};
              var className = suffixClasses(remove, '-remove') + ' ' + suffixClasses(add, '-add');
              var cancellationMethod = animateBefore('setClass', element, className, options.from);
              if (cancellationMethod) {
                afterReflow(element, animationCompleted);
                return cancellationMethod;
              }
              clearCacheAfterReflow();
              animationCompleted();
            },
            beforeAddClass: function (element, className, animationCompleted, options) {
              options = options || {};
              var cancellationMethod = animateBefore('addClass', element, suffixClasses(className, '-add'), options.from);
              if (cancellationMethod) {
                afterReflow(element, animationCompleted);
                return cancellationMethod;
              }
              clearCacheAfterReflow();
              animationCompleted();
            },
            beforeRemoveClass: function (element, className, animationCompleted, options) {
              options = options || {};
              var cancellationMethod = animateBefore('removeClass', element, suffixClasses(className, '-remove'), options.from);
              if (cancellationMethod) {
                afterReflow(element, animationCompleted);
                return cancellationMethod;
              }
              clearCacheAfterReflow();
              animationCompleted();
            },
            setClass: function (element, add, remove, animationCompleted, options) {
              options = options || {};
              remove = suffixClasses(remove, '-remove');
              add = suffixClasses(add, '-add');
              var className = remove + ' ' + add;
              return animateAfter('setClass', element, className, animationCompleted, options.to);
            },
            addClass: function (element, className, animationCompleted, options) {
              options = options || {};
              return animateAfter('addClass', element, suffixClasses(className, '-add'), animationCompleted, options.to);
            },
            removeClass: function (element, className, animationCompleted, options) {
              options = options || {};
              return animateAfter('removeClass', element, suffixClasses(className, '-remove'), animationCompleted, options.to);
            }
          };
          function suffixClasses(classes, suffix) {
            var className = '';
            classes = isArray(classes) ? classes : classes.split(/\s+/);
            forEach(classes, function (klass, i) {
              if (klass && klass.length > 0) {
                className += (i > 0 ? ' ' : '') + klass + suffix;
              }
            });
            return className;
          }
        }
      ]);
    }
  ]);
}(window, window.angular));
/**
 * @license AngularJS v1.3.20
 * (c) 2010-2014 Google, Inc. http://angularjs.org
 * License: MIT
 */
(function (window, angular, undefined) {
  'use strict';
  /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *     Any commits to this file should be reviewed with security in mind.  *
 *   Changes to this file can potentially create security vulnerabilities. *
 *          An approval from 2 Core members with history of modifying      *
 *                         this file is required.                          *
 *                                                                         *
 *  Does the change somehow allow for arbitrary javascript to be executed? *
 *    Or allows for someone to change the prototype of built-in objects?   *
 *     Or gives undesired access to variables likes document or window?    *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
  var $sanitizeMinErr = angular.$$minErr('$sanitize');
  /**
 * @ngdoc module
 * @name ngSanitize
 * @description
 *
 * # ngSanitize
 *
 * The `ngSanitize` module provides functionality to sanitize HTML.
 *
 *
 * <div doc-module-components="ngSanitize"></div>
 *
 * See {@link ngSanitize.$sanitize `$sanitize`} for usage.
 */
  /*
 * HTML Parser By Misko Hevery (misko@hevery.com)
 * based on:  HTML Parser By John Resig (ejohn.org)
 * Original code by Erik Arvidsson, Mozilla Public License
 * http://erik.eae.net/simplehtmlparser/simplehtmlparser.js
 *
 * // Use like so:
 * htmlParser(htmlString, {
 *     start: function(tag, attrs, unary) {},
 *     end: function(tag) {},
 *     chars: function(text) {},
 *     comment: function(text) {}
 * });
 *
 */
  /**
 * @ngdoc service
 * @name $sanitize
 * @kind function
 *
 * @description
 *   The input is sanitized by parsing the HTML into tokens. All safe tokens (from a whitelist) are
 *   then serialized back to properly escaped html string. This means that no unsafe input can make
 *   it into the returned string, however, since our parser is more strict than a typical browser
 *   parser, it's possible that some obscure input, which would be recognized as valid HTML by a
 *   browser, won't make it through the sanitizer. The input may also contain SVG markup.
 *   The whitelist is configured using the functions `aHrefSanitizationWhitelist` and
 *   `imgSrcSanitizationWhitelist` of {@link ng.$compileProvider `$compileProvider`}.
 *
 * @param {string} html HTML input.
 * @returns {string} Sanitized HTML.
 *
 * @example
   <example module="sanitizeExample" deps="angular-sanitize.js">
   <file name="index.html">
     <script>
         angular.module('sanitizeExample', ['ngSanitize'])
           .controller('ExampleController', ['$scope', '$sce', function($scope, $sce) {
             $scope.snippet =
               '<p style="color:blue">an html\n' +
               '<em onmouseover="this.textContent=\'PWN3D!\'">click here</em>\n' +
               'snippet</p>';
             $scope.deliberatelyTrustDangerousSnippet = function() {
               return $sce.trustAsHtml($scope.snippet);
             };
           }]);
     </script>
     <div ng-controller="ExampleController">
        Snippet: <textarea ng-model="snippet" cols="60" rows="3"></textarea>
       <table>
         <tr>
           <td>Directive</td>
           <td>How</td>
           <td>Source</td>
           <td>Rendered</td>
         </tr>
         <tr id="bind-html-with-sanitize">
           <td>ng-bind-html</td>
           <td>Automatically uses $sanitize</td>
           <td><pre>&lt;div ng-bind-html="snippet"&gt;<br/>&lt;/div&gt;</pre></td>
           <td><div ng-bind-html="snippet"></div></td>
         </tr>
         <tr id="bind-html-with-trust">
           <td>ng-bind-html</td>
           <td>Bypass $sanitize by explicitly trusting the dangerous value</td>
           <td>
           <pre>&lt;div ng-bind-html="deliberatelyTrustDangerousSnippet()"&gt;
&lt;/div&gt;</pre>
           </td>
           <td><div ng-bind-html="deliberatelyTrustDangerousSnippet()"></div></td>
         </tr>
         <tr id="bind-default">
           <td>ng-bind</td>
           <td>Automatically escapes</td>
           <td><pre>&lt;div ng-bind="snippet"&gt;<br/>&lt;/div&gt;</pre></td>
           <td><div ng-bind="snippet"></div></td>
         </tr>
       </table>
       </div>
   </file>
   <file name="protractor.js" type="protractor">
     it('should sanitize the html snippet by default', function() {
       expect(element(by.css('#bind-html-with-sanitize div')).getInnerHtml()).
         toBe('<p>an html\n<em>click here</em>\nsnippet</p>');
     });

     it('should inline raw snippet if bound to a trusted value', function() {
       expect(element(by.css('#bind-html-with-trust div')).getInnerHtml()).
         toBe("<p style=\"color:blue\">an html\n" +
              "<em onmouseover=\"this.textContent='PWN3D!'\">click here</em>\n" +
              "snippet</p>");
     });

     it('should escape snippet without any filter', function() {
       expect(element(by.css('#bind-default div')).getInnerHtml()).
         toBe("&lt;p style=\"color:blue\"&gt;an html\n" +
              "&lt;em onmouseover=\"this.textContent='PWN3D!'\"&gt;click here&lt;/em&gt;\n" +
              "snippet&lt;/p&gt;");
     });

     it('should update', function() {
       element(by.model('snippet')).clear();
       element(by.model('snippet')).sendKeys('new <b onclick="alert(1)">text</b>');
       expect(element(by.css('#bind-html-with-sanitize div')).getInnerHtml()).
         toBe('new <b>text</b>');
       expect(element(by.css('#bind-html-with-trust div')).getInnerHtml()).toBe(
         'new <b onclick="alert(1)">text</b>');
       expect(element(by.css('#bind-default div')).getInnerHtml()).toBe(
         "new &lt;b onclick=\"alert(1)\"&gt;text&lt;/b&gt;");
     });
   </file>
   </example>
 */
  function $SanitizeProvider() {
    this.$get = [
      '$$sanitizeUri',
      function ($$sanitizeUri) {
        return function (html) {
          var buf = [];
          htmlParser(html, htmlSanitizeWriter(buf, function (uri, isImage) {
            return !/^unsafe/.test($$sanitizeUri(uri, isImage));
          }));
          return buf.join('');
        };
      }
    ];
  }
  function sanitizeText(chars) {
    var buf = [];
    var writer = htmlSanitizeWriter(buf, angular.noop);
    writer.chars(chars);
    return buf.join('');
  }
  // Regular Expressions for parsing tags and attributes
  var START_TAG_REGEXP = /^<((?:[a-zA-Z])[\w:-]*)((?:\s+[\w:-]+(?:\s*=\s*(?:(?:"[^"]*")|(?:'[^']*')|[^>\s]+))?)*)\s*(\/?)\s*(>?)/, END_TAG_REGEXP = /^<\/\s*([\w:-]+)[^>]*>/, ATTR_REGEXP = /([\w:-]+)(?:\s*=\s*(?:(?:"((?:[^"])*)")|(?:'((?:[^'])*)')|([^>\s]+)))?/g, BEGIN_TAG_REGEXP = /^</, BEGING_END_TAGE_REGEXP = /^<\//, COMMENT_REGEXP = /<!--(.*?)-->/g, DOCTYPE_REGEXP = /<!DOCTYPE([^>]*?)>/i, CDATA_REGEXP = /<!\[CDATA\[(.*?)]]>/g, SURROGATE_PAIR_REGEXP = /[\uD800-\uDBFF][\uDC00-\uDFFF]/g,
    // Match everything outside of normal chars and " (quote character)
    NON_ALPHANUMERIC_REGEXP = /([^\#-~| |!])/g;
  // Good source of info about elements and attributes
  // http://dev.w3.org/html5/spec/Overview.html#semantics
  // http://simon.html5.org/html-elements
  // Safe Void Elements - HTML5
  // http://dev.w3.org/html5/spec/Overview.html#void-elements
  var voidElements = makeMap('area,br,col,hr,img,wbr');
  // Elements that you can, intentionally, leave open (and which close themselves)
  // http://dev.w3.org/html5/spec/Overview.html#optional-tags
  var optionalEndTagBlockElements = makeMap('colgroup,dd,dt,li,p,tbody,td,tfoot,th,thead,tr'), optionalEndTagInlineElements = makeMap('rp,rt'), optionalEndTagElements = angular.extend({}, optionalEndTagInlineElements, optionalEndTagBlockElements);
  // Safe Block Elements - HTML5
  var blockElements = angular.extend({}, optionalEndTagBlockElements, makeMap('address,article,' + 'aside,blockquote,caption,center,del,dir,div,dl,figure,figcaption,footer,h1,h2,h3,h4,h5,' + 'h6,header,hgroup,hr,ins,map,menu,nav,ol,pre,script,section,table,ul'));
  // Inline Elements - HTML5
  var inlineElements = angular.extend({}, optionalEndTagInlineElements, makeMap('a,abbr,acronym,b,' + 'bdi,bdo,big,br,cite,code,del,dfn,em,font,i,img,ins,kbd,label,map,mark,q,ruby,rp,rt,s,' + 'samp,small,span,strike,strong,sub,sup,time,tt,u,var'));
  // SVG Elements
  // https://wiki.whatwg.org/wiki/Sanitization_rules#svg_Elements
  var svgElements = makeMap('animate,animateColor,animateMotion,animateTransform,circle,defs,' + 'desc,ellipse,font-face,font-face-name,font-face-src,g,glyph,hkern,image,linearGradient,' + 'line,marker,metadata,missing-glyph,mpath,path,polygon,polyline,radialGradient,rect,set,' + 'stop,svg,switch,text,title,tspan,use');
  // Special Elements (can contain anything)
  var specialElements = makeMap('script,style');
  var validElements = angular.extend({}, voidElements, blockElements, inlineElements, optionalEndTagElements, svgElements);
  //Attributes that have href and hence need to be sanitized
  var uriAttrs = makeMap('background,cite,href,longdesc,src,usemap,xlink:href');
  var htmlAttrs = makeMap('abbr,align,alt,axis,bgcolor,border,cellpadding,cellspacing,class,clear,' + 'color,cols,colspan,compact,coords,dir,face,headers,height,hreflang,hspace,' + 'ismap,lang,language,nohref,nowrap,rel,rev,rows,rowspan,rules,' + 'scope,scrolling,shape,size,span,start,summary,target,title,type,' + 'valign,value,vspace,width');
  // SVG attributes (without "id" and "name" attributes)
  // https://wiki.whatwg.org/wiki/Sanitization_rules#svg_Attributes
  var svgAttrs = makeMap('accent-height,accumulate,additive,alphabetic,arabic-form,ascent,' + 'attributeName,attributeType,baseProfile,bbox,begin,by,calcMode,cap-height,class,color,' + 'color-rendering,content,cx,cy,d,dx,dy,descent,display,dur,end,fill,fill-rule,font-family,' + 'font-size,font-stretch,font-style,font-variant,font-weight,from,fx,fy,g1,g2,glyph-name,' + 'gradientUnits,hanging,height,horiz-adv-x,horiz-origin-x,ideographic,k,keyPoints,' + 'keySplines,keyTimes,lang,marker-end,marker-mid,marker-start,markerHeight,markerUnits,' + 'markerWidth,mathematical,max,min,offset,opacity,orient,origin,overline-position,' + 'overline-thickness,panose-1,path,pathLength,points,preserveAspectRatio,r,refX,refY,' + 'repeatCount,repeatDur,requiredExtensions,requiredFeatures,restart,rotate,rx,ry,slope,stemh,' + 'stemv,stop-color,stop-opacity,strikethrough-position,strikethrough-thickness,stroke,' + 'stroke-dasharray,stroke-dashoffset,stroke-linecap,stroke-linejoin,stroke-miterlimit,' + 'stroke-opacity,stroke-width,systemLanguage,target,text-anchor,to,transform,type,u1,u2,' + 'underline-position,underline-thickness,unicode,unicode-range,units-per-em,values,version,' + 'viewBox,visibility,width,widths,x,x-height,x1,x2,xlink:actuate,xlink:arcrole,xlink:role,' + 'xlink:show,xlink:title,xlink:type,xml:base,xml:lang,xml:space,xmlns,xmlns:xlink,y,y1,y2,' + 'zoomAndPan');
  var validAttrs = angular.extend({}, uriAttrs, svgAttrs, htmlAttrs);
  function makeMap(str) {
    var obj = {}, items = str.split(','), i;
    for (i = 0; i < items.length; i++)
      obj[items[i]] = true;
    return obj;
  }
  /**
 * @example
 * htmlParser(htmlString, {
 *     start: function(tag, attrs, unary) {},
 *     end: function(tag) {},
 *     chars: function(text) {},
 *     comment: function(text) {}
 * });
 *
 * @param {string} html string
 * @param {object} handler
 */
  function htmlParser(html, handler) {
    if (typeof html !== 'string') {
      if (html === null || typeof html === 'undefined') {
        html = '';
      } else {
        html = '' + html;
      }
    }
    var index, chars, match, stack = [], last = html, text;
    stack.last = function () {
      return stack[stack.length - 1];
    };
    while (html) {
      text = '';
      chars = true;
      // Make sure we're not in a script or style element
      if (!stack.last() || !specialElements[stack.last()]) {
        // Comment
        if (html.indexOf('<!--') === 0) {
          // comments containing -- are not allowed unless they terminate the comment
          index = html.indexOf('--', 4);
          if (index >= 0 && html.lastIndexOf('-->', index) === index) {
            if (handler.comment)
              handler.comment(html.substring(4, index));
            html = html.substring(index + 3);
            chars = false;
          }  // DOCTYPE
        } else if (DOCTYPE_REGEXP.test(html)) {
          match = html.match(DOCTYPE_REGEXP);
          if (match) {
            html = html.replace(match[0], '');
            chars = false;
          }  // end tag
        } else if (BEGING_END_TAGE_REGEXP.test(html)) {
          match = html.match(END_TAG_REGEXP);
          if (match) {
            html = html.substring(match[0].length);
            match[0].replace(END_TAG_REGEXP, parseEndTag);
            chars = false;
          }  // start tag
        } else if (BEGIN_TAG_REGEXP.test(html)) {
          match = html.match(START_TAG_REGEXP);
          if (match) {
            // We only have a valid start-tag if there is a '>'.
            if (match[4]) {
              html = html.substring(match[0].length);
              match[0].replace(START_TAG_REGEXP, parseStartTag);
            }
            chars = false;
          } else {
            // no ending tag found --- this piece should be encoded as an entity.
            text += '<';
            html = html.substring(1);
          }
        }
        if (chars) {
          index = html.indexOf('<');
          text += index < 0 ? html : html.substring(0, index);
          html = index < 0 ? '' : html.substring(index);
          if (handler.chars)
            handler.chars(decodeEntities(text));
        }
      } else {
        // IE versions 9 and 10 do not understand the regex '[^]', so using a workaround with [\W\w].
        html = html.replace(new RegExp('([\\W\\w]*)<\\s*\\/\\s*' + stack.last() + '[^>]*>', 'i'), function (all, text) {
          text = text.replace(COMMENT_REGEXP, '$1').replace(CDATA_REGEXP, '$1');
          if (handler.chars)
            handler.chars(decodeEntities(text));
          return '';
        });
        parseEndTag('', stack.last());
      }
      if (html == last) {
        throw $sanitizeMinErr('badparse', 'The sanitizer was unable to parse the following block ' + 'of html: {0}', html);
      }
      last = html;
    }
    // Clean up any remaining tags
    parseEndTag();
    function parseStartTag(tag, tagName, rest, unary) {
      tagName = angular.lowercase(tagName);
      if (blockElements[tagName]) {
        while (stack.last() && inlineElements[stack.last()]) {
          parseEndTag('', stack.last());
        }
      }
      if (optionalEndTagElements[tagName] && stack.last() == tagName) {
        parseEndTag('', tagName);
      }
      unary = voidElements[tagName] || !!unary;
      if (!unary)
        stack.push(tagName);
      var attrs = {};
      rest.replace(ATTR_REGEXP, function (match, name, doubleQuotedValue, singleQuotedValue, unquotedValue) {
        var value = doubleQuotedValue || singleQuotedValue || unquotedValue || '';
        attrs[name] = decodeEntities(value);
      });
      if (handler.start)
        handler.start(tagName, attrs, unary);
    }
    function parseEndTag(tag, tagName) {
      var pos = 0, i;
      tagName = angular.lowercase(tagName);
      if (tagName)
        // Find the closest opened tag of the same type
        for (pos = stack.length - 1; pos >= 0; pos--)
          if (stack[pos] == tagName)
            break;
      if (pos >= 0) {
        // Close all the open elements, up the stack
        for (i = stack.length - 1; i >= pos; i--)
          if (handler.end)
            handler.end(stack[i]);
        // Remove the open elements from the stack
        stack.length = pos;
      }
    }
  }
  var hiddenPre = document.createElement('pre');
  /**
 * decodes all entities into regular string
 * @param value
 * @returns {string} A string with decoded entities.
 */
  function decodeEntities(value) {
    if (!value) {
      return '';
    }
    hiddenPre.innerHTML = value.replace(/</g, '&lt;');
    // innerText depends on styling as it doesn't display hidden elements.
    // Therefore, it's better to use textContent not to cause unnecessary reflows.
    return hiddenPre.textContent;
  }
  /**
 * Escapes all potentially dangerous characters, so that the
 * resulting string can be safely inserted into attribute or
 * element text.
 * @param value
 * @returns {string} escaped text
 */
  function encodeEntities(value) {
    return value.replace(/&/g, '&amp;').replace(SURROGATE_PAIR_REGEXP, function (value) {
      var hi = value.charCodeAt(0);
      var low = value.charCodeAt(1);
      return '&#' + ((hi - 55296) * 1024 + (low - 56320) + 65536) + ';';
    }).replace(NON_ALPHANUMERIC_REGEXP, function (value) {
      return '&#' + value.charCodeAt(0) + ';';
    }).replace(/</g, '&lt;').replace(/>/g, '&gt;');
  }
  /**
 * create an HTML/XML writer which writes to buffer
 * @param {Array} buf use buf.jain('') to get out sanitized html string
 * @returns {object} in the form of {
 *     start: function(tag, attrs, unary) {},
 *     end: function(tag) {},
 *     chars: function(text) {},
 *     comment: function(text) {}
 * }
 */
  function htmlSanitizeWriter(buf, uriValidator) {
    var ignore = false;
    var out = angular.bind(buf, buf.push);
    return {
      start: function (tag, attrs, unary) {
        tag = angular.lowercase(tag);
        if (!ignore && specialElements[tag]) {
          ignore = tag;
        }
        if (!ignore && validElements[tag] === true) {
          out('<');
          out(tag);
          angular.forEach(attrs, function (value, key) {
            var lkey = angular.lowercase(key);
            var isImage = tag === 'img' && lkey === 'src' || lkey === 'background';
            if (validAttrs[lkey] === true && (uriAttrs[lkey] !== true || uriValidator(value, isImage))) {
              out(' ');
              out(key);
              out('="');
              out(encodeEntities(value));
              out('"');
            }
          });
          out(unary ? '/>' : '>');
        }
      },
      end: function (tag) {
        tag = angular.lowercase(tag);
        if (!ignore && validElements[tag] === true) {
          out('</');
          out(tag);
          out('>');
        }
        if (tag == ignore) {
          ignore = false;
        }
      },
      chars: function (chars) {
        if (!ignore) {
          out(encodeEntities(chars));
        }
      }
    };
  }
  // define ngSanitize module and register $sanitize service
  angular.module('ngSanitize', []).provider('$sanitize', $SanitizeProvider);
  /* global sanitizeText: false */
  /**
 * @ngdoc filter
 * @name linky
 * @kind function
 *
 * @description
 * Finds links in text input and turns them into html links. Supports http/https/ftp/mailto and
 * plain email address links.
 *
 * Requires the {@link ngSanitize `ngSanitize`} module to be installed.
 *
 * @param {string} text Input text.
 * @param {string} target Window (_blank|_self|_parent|_top) or named frame to open links in.
 * @returns {string} Html-linkified text.
 *
 * @usage
   <span ng-bind-html="linky_expression | linky"></span>
 *
 * @example
   <example module="linkyExample" deps="angular-sanitize.js">
     <file name="index.html">
       <script>
         angular.module('linkyExample', ['ngSanitize'])
           .controller('ExampleController', ['$scope', function($scope) {
             $scope.snippet =
               'Pretty text with some links:\n'+
               'http://angularjs.org/,\n'+
               'mailto:us@somewhere.org,\n'+
               'another@somewhere.org,\n'+
               'and one more: ftp://127.0.0.1/.';
             $scope.snippetWithTarget = 'http://angularjs.org/';
           }]);
       </script>
       <div ng-controller="ExampleController">
       Snippet: <textarea ng-model="snippet" cols="60" rows="3"></textarea>
       <table>
         <tr>
           <td>Filter</td>
           <td>Source</td>
           <td>Rendered</td>
         </tr>
         <tr id="linky-filter">
           <td>linky filter</td>
           <td>
             <pre>&lt;div ng-bind-html="snippet | linky"&gt;<br>&lt;/div&gt;</pre>
           </td>
           <td>
             <div ng-bind-html="snippet | linky"></div>
           </td>
         </tr>
         <tr id="linky-target">
          <td>linky target</td>
          <td>
            <pre>&lt;div ng-bind-html="snippetWithTarget | linky:'_blank'"&gt;<br>&lt;/div&gt;</pre>
          </td>
          <td>
            <div ng-bind-html="snippetWithTarget | linky:'_blank'"></div>
          </td>
         </tr>
         <tr id="escaped-html">
           <td>no filter</td>
           <td><pre>&lt;div ng-bind="snippet"&gt;<br>&lt;/div&gt;</pre></td>
           <td><div ng-bind="snippet"></div></td>
         </tr>
       </table>
     </file>
     <file name="protractor.js" type="protractor">
       it('should linkify the snippet with urls', function() {
         expect(element(by.id('linky-filter')).element(by.binding('snippet | linky')).getText()).
             toBe('Pretty text with some links: http://angularjs.org/, us@somewhere.org, ' +
                  'another@somewhere.org, and one more: ftp://127.0.0.1/.');
         expect(element.all(by.css('#linky-filter a')).count()).toEqual(4);
       });

       it('should not linkify snippet without the linky filter', function() {
         expect(element(by.id('escaped-html')).element(by.binding('snippet')).getText()).
             toBe('Pretty text with some links: http://angularjs.org/, mailto:us@somewhere.org, ' +
                  'another@somewhere.org, and one more: ftp://127.0.0.1/.');
         expect(element.all(by.css('#escaped-html a')).count()).toEqual(0);
       });

       it('should update', function() {
         element(by.model('snippet')).clear();
         element(by.model('snippet')).sendKeys('new http://link.');
         expect(element(by.id('linky-filter')).element(by.binding('snippet | linky')).getText()).
             toBe('new http://link.');
         expect(element.all(by.css('#linky-filter a')).count()).toEqual(1);
         expect(element(by.id('escaped-html')).element(by.binding('snippet')).getText())
             .toBe('new http://link.');
       });

       it('should work with the target property', function() {
        expect(element(by.id('linky-target')).
            element(by.binding("snippetWithTarget | linky:'_blank'")).getText()).
            toBe('http://angularjs.org/');
        expect(element(by.css('#linky-target a')).getAttribute('target')).toEqual('_blank');
       });
     </file>
   </example>
 */
  angular.module('ngSanitize').filter('linky', [
    '$sanitize',
    function ($sanitize) {
      var LINKY_URL_REGEXP = /((ftp|https?):\/\/|(www\.)|(mailto:)?[A-Za-z0-9._%+-]+@)\S*[^\s.;,(){}<>"”’]/i, MAILTO_REGEXP = /^mailto:/i;
      return function (text, target) {
        if (!text)
          return text;
        var match;
        var raw = text;
        var html = [];
        var url;
        var i;
        while (match = raw.match(LINKY_URL_REGEXP)) {
          // We can not end in these as they are sometimes found at the end of the sentence
          url = match[0];
          // if we did not match ftp/http/www/mailto then assume mailto
          if (!match[2] && !match[4]) {
            url = (match[3] ? 'http://' : 'mailto:') + url;
          }
          i = match.index;
          addText(raw.substr(0, i));
          addLink(url, match[0].replace(MAILTO_REGEXP, ''));
          raw = raw.substring(i + match[0].length);
        }
        addText(raw);
        return $sanitize(html.join(''));
        function addText(text) {
          if (!text) {
            return;
          }
          html.push(sanitizeText(text));
        }
        function addLink(url, text) {
          html.push('<a ');
          if (angular.isDefined(target)) {
            html.push('target="', target, '" ');
          }
          html.push('href="', url.replace(/"/g, '&quot;'), '">');
          addText(text);
          html.push('</a>');
        }
      };
    }
  ]);
}(window, window.angular));
/**
 * State-based routing for AngularJS
 * @version v0.2.18
 * @link http://angular-ui.github.com/
 * @license MIT License, http://www.opensource.org/licenses/MIT
 */
/* commonjs package manager support (eg componentjs) */
if (typeof module !== 'undefined' && typeof exports !== 'undefined' && module.exports === exports) {
  module.exports = 'ui.router';
}
(function (window, angular, undefined) {
  /*jshint globalstrict:true*/
  /*global angular:false*/
  'use strict';
  var isDefined = angular.isDefined, isFunction = angular.isFunction, isString = angular.isString, isObject = angular.isObject, isArray = angular.isArray, forEach = angular.forEach, extend = angular.extend, copy = angular.copy, toJson = angular.toJson;
  function inherit(parent, extra) {
    return extend(new (extend(function () {
    }, { prototype: parent }))(), extra);
  }
  function merge(dst) {
    forEach(arguments, function (obj) {
      if (obj !== dst) {
        forEach(obj, function (value, key) {
          if (!dst.hasOwnProperty(key))
            dst[key] = value;
        });
      }
    });
    return dst;
  }
  /**
 * Finds the common ancestor path between two states.
 *
 * @param {Object} first The first state.
 * @param {Object} second The second state.
 * @return {Array} Returns an array of state names in descending order, not including the root.
 */
  function ancestors(first, second) {
    var path = [];
    for (var n in first.path) {
      if (first.path[n] !== second.path[n])
        break;
      path.push(first.path[n]);
    }
    return path;
  }
  /**
 * IE8-safe wrapper for `Object.keys()`.
 *
 * @param {Object} object A JavaScript object.
 * @return {Array} Returns the keys of the object as an array.
 */
  function objectKeys(object) {
    if (Object.keys) {
      return Object.keys(object);
    }
    var result = [];
    forEach(object, function (val, key) {
      result.push(key);
    });
    return result;
  }
  /**
 * IE8-safe wrapper for `Array.prototype.indexOf()`.
 *
 * @param {Array} array A JavaScript array.
 * @param {*} value A value to search the array for.
 * @return {Number} Returns the array index value of `value`, or `-1` if not present.
 */
  function indexOf(array, value) {
    if (Array.prototype.indexOf) {
      return array.indexOf(value, Number(arguments[2]) || 0);
    }
    var len = array.length >>> 0, from = Number(arguments[2]) || 0;
    from = from < 0 ? Math.ceil(from) : Math.floor(from);
    if (from < 0)
      from += len;
    for (; from < len; from++) {
      if (from in array && array[from] === value)
        return from;
    }
    return -1;
  }
  /**
 * Merges a set of parameters with all parameters inherited between the common parents of the
 * current state and a given destination state.
 *
 * @param {Object} currentParams The value of the current state parameters ($stateParams).
 * @param {Object} newParams The set of parameters which will be composited with inherited params.
 * @param {Object} $current Internal definition of object representing the current state.
 * @param {Object} $to Internal definition of object representing state to transition to.
 */
  function inheritParams(currentParams, newParams, $current, $to) {
    var parents = ancestors($current, $to), parentParams, inherited = {}, inheritList = [];
    for (var i in parents) {
      if (!parents[i] || !parents[i].params)
        continue;
      parentParams = objectKeys(parents[i].params);
      if (!parentParams.length)
        continue;
      for (var j in parentParams) {
        if (indexOf(inheritList, parentParams[j]) >= 0)
          continue;
        inheritList.push(parentParams[j]);
        inherited[parentParams[j]] = currentParams[parentParams[j]];
      }
    }
    return extend({}, inherited, newParams);
  }
  /**
 * Performs a non-strict comparison of the subset of two objects, defined by a list of keys.
 *
 * @param {Object} a The first object.
 * @param {Object} b The second object.
 * @param {Array} keys The list of keys within each object to compare. If the list is empty or not specified,
 *                     it defaults to the list of keys in `a`.
 * @return {Boolean} Returns `true` if the keys match, otherwise `false`.
 */
  function equalForKeys(a, b, keys) {
    if (!keys) {
      keys = [];
      for (var n in a)
        keys.push(n);  // Used instead of Object.keys() for IE8 compatibility
    }
    for (var i = 0; i < keys.length; i++) {
      var k = keys[i];
      if (a[k] != b[k])
        return false;  // Not '===', values aren't necessarily normalized
    }
    return true;
  }
  /**
 * Returns the subset of an object, based on a list of keys.
 *
 * @param {Array} keys
 * @param {Object} values
 * @return {Boolean} Returns a subset of `values`.
 */
  function filterByKeys(keys, values) {
    var filtered = {};
    forEach(keys, function (name) {
      filtered[name] = values[name];
    });
    return filtered;
  }
  // like _.indexBy
  // when you know that your index values will be unique, or you want last-one-in to win
  function indexBy(array, propName) {
    var result = {};
    forEach(array, function (item) {
      result[item[propName]] = item;
    });
    return result;
  }
  // extracted from underscore.js
  // Return a copy of the object only containing the whitelisted properties.
  function pick(obj) {
    var copy = {};
    var keys = Array.prototype.concat.apply(Array.prototype, Array.prototype.slice.call(arguments, 1));
    forEach(keys, function (key) {
      if (key in obj)
        copy[key] = obj[key];
    });
    return copy;
  }
  // extracted from underscore.js
  // Return a copy of the object omitting the blacklisted properties.
  function omit(obj) {
    var copy = {};
    var keys = Array.prototype.concat.apply(Array.prototype, Array.prototype.slice.call(arguments, 1));
    for (var key in obj) {
      if (indexOf(keys, key) == -1)
        copy[key] = obj[key];
    }
    return copy;
  }
  function pluck(collection, key) {
    var result = isArray(collection) ? [] : {};
    forEach(collection, function (val, i) {
      result[i] = isFunction(key) ? key(val) : val[key];
    });
    return result;
  }
  function filter(collection, callback) {
    var array = isArray(collection);
    var result = array ? [] : {};
    forEach(collection, function (val, i) {
      if (callback(val, i)) {
        result[array ? result.length : i] = val;
      }
    });
    return result;
  }
  function map(collection, callback) {
    var result = isArray(collection) ? [] : {};
    forEach(collection, function (val, i) {
      result[i] = callback(val, i);
    });
    return result;
  }
  /**
 * @ngdoc overview
 * @name ui.router.util
 *
 * @description
 * # ui.router.util sub-module
 *
 * This module is a dependency of other sub-modules. Do not include this module as a dependency
 * in your angular app (use {@link ui.router} module instead).
 *
 */
  angular.module('ui.router.util', ['ng']);
  /**
 * @ngdoc overview
 * @name ui.router.router
 * 
 * @requires ui.router.util
 *
 * @description
 * # ui.router.router sub-module
 *
 * This module is a dependency of other sub-modules. Do not include this module as a dependency
 * in your angular app (use {@link ui.router} module instead).
 */
  angular.module('ui.router.router', ['ui.router.util']);
  /**
 * @ngdoc overview
 * @name ui.router.state
 * 
 * @requires ui.router.router
 * @requires ui.router.util
 *
 * @description
 * # ui.router.state sub-module
 *
 * This module is a dependency of the main ui.router module. Do not include this module as a dependency
 * in your angular app (use {@link ui.router} module instead).
 * 
 */
  angular.module('ui.router.state', [
    'ui.router.router',
    'ui.router.util'
  ]);
  /**
 * @ngdoc overview
 * @name ui.router
 *
 * @requires ui.router.state
 *
 * @description
 * # ui.router
 * 
 * ## The main module for ui.router 
 * There are several sub-modules included with the ui.router module, however only this module is needed
 * as a dependency within your angular app. The other modules are for organization purposes. 
 *
 * The modules are:
 * * ui.router - the main "umbrella" module
 * * ui.router.router - 
 * 
 * *You'll need to include **only** this module as the dependency within your angular app.*
 * 
 * <pre>
 * <!doctype html>
 * <html ng-app="myApp">
 * <head>
 *   <script src="js/angular.js"></script>
 *   <!-- Include the ui-router script -->
 *   <script src="js/angular-ui-router.min.js"></script>
 *   <script>
 *     // ...and add 'ui.router' as a dependency
 *     var myApp = angular.module('myApp', ['ui.router']);
 *   </script>
 * </head>
 * <body>
 * </body>
 * </html>
 * </pre>
 */
  angular.module('ui.router', ['ui.router.state']);
  angular.module('ui.router.compat', ['ui.router']);
  /**
 * @ngdoc object
 * @name ui.router.util.$resolve
 *
 * @requires $q
 * @requires $injector
 *
 * @description
 * Manages resolution of (acyclic) graphs of promises.
 */
  $Resolve.$inject = [
    '$q',
    '$injector'
  ];
  function $Resolve($q, $injector) {
    var VISIT_IN_PROGRESS = 1, VISIT_DONE = 2, NOTHING = {}, NO_DEPENDENCIES = [], NO_LOCALS = NOTHING, NO_PARENT = extend($q.when(NOTHING), {
        $$promises: NOTHING,
        $$values: NOTHING
      });
    /**
   * @ngdoc function
   * @name ui.router.util.$resolve#study
   * @methodOf ui.router.util.$resolve
   *
   * @description
   * Studies a set of invocables that are likely to be used multiple times.
   * <pre>
   * $resolve.study(invocables)(locals, parent, self)
   * </pre>
   * is equivalent to
   * <pre>
   * $resolve.resolve(invocables, locals, parent, self)
   * </pre>
   * but the former is more efficient (in fact `resolve` just calls `study` 
   * internally).
   *
   * @param {object} invocables Invocable objects
   * @return {function} a function to pass in locals, parent and self
   */
    this.study = function (invocables) {
      if (!isObject(invocables))
        throw new Error('\'invocables\' must be an object');
      var invocableKeys = objectKeys(invocables || {});
      // Perform a topological sort of invocables to build an ordered plan
      var plan = [], cycle = [], visited = {};
      function visit(value, key) {
        if (visited[key] === VISIT_DONE)
          return;
        cycle.push(key);
        if (visited[key] === VISIT_IN_PROGRESS) {
          cycle.splice(0, indexOf(cycle, key));
          throw new Error('Cyclic dependency: ' + cycle.join(' -> '));
        }
        visited[key] = VISIT_IN_PROGRESS;
        if (isString(value)) {
          plan.push(key, [function () {
              return $injector.get(value);
            }], NO_DEPENDENCIES);
        } else {
          var params = $injector.annotate(value);
          forEach(params, function (param) {
            if (param !== key && invocables.hasOwnProperty(param))
              visit(invocables[param], param);
          });
          plan.push(key, value, params);
        }
        cycle.pop();
        visited[key] = VISIT_DONE;
      }
      forEach(invocables, visit);
      invocables = cycle = visited = null;
      // plan is all that's required
      function isResolve(value) {
        return isObject(value) && value.then && value.$$promises;
      }
      return function (locals, parent, self) {
        if (isResolve(locals) && self === undefined) {
          self = parent;
          parent = locals;
          locals = null;
        }
        if (!locals)
          locals = NO_LOCALS;
        else if (!isObject(locals)) {
          throw new Error('\'locals\' must be an object');
        }
        if (!parent)
          parent = NO_PARENT;
        else if (!isResolve(parent)) {
          throw new Error('\'parent\' must be a promise returned by $resolve.resolve()');
        }
        // To complete the overall resolution, we have to wait for the parent
        // promise and for the promise for each invokable in our plan.
        var resolution = $q.defer(), result = resolution.promise, promises = result.$$promises = {}, values = extend({}, locals), wait = 1 + plan.length / 3, merged = false;
        function done() {
          // Merge parent values we haven't got yet and publish our own $$values
          if (!--wait) {
            if (!merged)
              merge(values, parent.$$values);
            result.$$values = values;
            result.$$promises = result.$$promises || true;
            // keep for isResolve()
            delete result.$$inheritedValues;
            resolution.resolve(values);
          }
        }
        function fail(reason) {
          result.$$failure = reason;
          resolution.reject(reason);
        }
        // Short-circuit if parent has already failed
        if (isDefined(parent.$$failure)) {
          fail(parent.$$failure);
          return result;
        }
        if (parent.$$inheritedValues) {
          merge(values, omit(parent.$$inheritedValues, invocableKeys));
        }
        // Merge parent values if the parent has already resolved, or merge
        // parent promises and wait if the parent resolve is still in progress.
        extend(promises, parent.$$promises);
        if (parent.$$values) {
          merged = merge(values, omit(parent.$$values, invocableKeys));
          result.$$inheritedValues = omit(parent.$$values, invocableKeys);
          done();
        } else {
          if (parent.$$inheritedValues) {
            result.$$inheritedValues = omit(parent.$$inheritedValues, invocableKeys);
          }
          parent.then(done, fail);
        }
        // Process each invocable in the plan, but ignore any where a local of the same name exists.
        for (var i = 0, ii = plan.length; i < ii; i += 3) {
          if (locals.hasOwnProperty(plan[i]))
            done();
          else
            invoke(plan[i], plan[i + 1], plan[i + 2]);
        }
        function invoke(key, invocable, params) {
          // Create a deferred for this invocation. Failures will propagate to the resolution as well.
          var invocation = $q.defer(), waitParams = 0;
          function onfailure(reason) {
            invocation.reject(reason);
            fail(reason);
          }
          // Wait for any parameter that we have a promise for (either from parent or from this
          // resolve; in that case study() will have made sure it's ordered before us in the plan).
          forEach(params, function (dep) {
            if (promises.hasOwnProperty(dep) && !locals.hasOwnProperty(dep)) {
              waitParams++;
              promises[dep].then(function (result) {
                values[dep] = result;
                if (!--waitParams)
                  proceed();
              }, onfailure);
            }
          });
          if (!waitParams)
            proceed();
          function proceed() {
            if (isDefined(result.$$failure))
              return;
            try {
              invocation.resolve($injector.invoke(invocable, self, values));
              invocation.promise.then(function (result) {
                values[key] = result;
                done();
              }, onfailure);
            } catch (e) {
              onfailure(e);
            }
          }
          // Publish promise synchronously; invocations further down in the plan may depend on it.
          promises[key] = invocation.promise;
        }
        return result;
      };
    };
    /**
   * @ngdoc function
   * @name ui.router.util.$resolve#resolve
   * @methodOf ui.router.util.$resolve
   *
   * @description
   * Resolves a set of invocables. An invocable is a function to be invoked via 
   * `$injector.invoke()`, and can have an arbitrary number of dependencies. 
   * An invocable can either return a value directly,
   * or a `$q` promise. If a promise is returned it will be resolved and the 
   * resulting value will be used instead. Dependencies of invocables are resolved 
   * (in this order of precedence)
   *
   * - from the specified `locals`
   * - from another invocable that is part of this `$resolve` call
   * - from an invocable that is inherited from a `parent` call to `$resolve` 
   *   (or recursively
   * - from any ancestor `$resolve` of that parent).
   *
   * The return value of `$resolve` is a promise for an object that contains 
   * (in this order of precedence)
   *
   * - any `locals` (if specified)
   * - the resolved return values of all injectables
   * - any values inherited from a `parent` call to `$resolve` (if specified)
   *
   * The promise will resolve after the `parent` promise (if any) and all promises 
   * returned by injectables have been resolved. If any invocable 
   * (or `$injector.invoke`) throws an exception, or if a promise returned by an 
   * invocable is rejected, the `$resolve` promise is immediately rejected with the 
   * same error. A rejection of a `parent` promise (if specified) will likewise be 
   * propagated immediately. Once the `$resolve` promise has been rejected, no 
   * further invocables will be called.
   * 
   * Cyclic dependencies between invocables are not permitted and will cause `$resolve`
   * to throw an error. As a special case, an injectable can depend on a parameter 
   * with the same name as the injectable, which will be fulfilled from the `parent` 
   * injectable of the same name. This allows inherited values to be decorated. 
   * Note that in this case any other injectable in the same `$resolve` with the same
   * dependency would see the decorated value, not the inherited value.
   *
   * Note that missing dependencies -- unlike cyclic dependencies -- will cause an 
   * (asynchronous) rejection of the `$resolve` promise rather than a (synchronous) 
   * exception.
   *
   * Invocables are invoked eagerly as soon as all dependencies are available. 
   * This is true even for dependencies inherited from a `parent` call to `$resolve`.
   *
   * As a special case, an invocable can be a string, in which case it is taken to 
   * be a service name to be passed to `$injector.get()`. This is supported primarily 
   * for backwards-compatibility with the `resolve` property of `$routeProvider` 
   * routes.
   *
   * @param {object} invocables functions to invoke or 
   * `$injector` services to fetch.
   * @param {object} locals  values to make available to the injectables
   * @param {object} parent  a promise returned by another call to `$resolve`.
   * @param {object} self  the `this` for the invoked methods
   * @return {object} Promise for an object that contains the resolved return value
   * of all invocables, as well as any inherited and local values.
   */
    this.resolve = function (invocables, locals, parent, self) {
      return this.study(invocables)(locals, parent, self);
    };
  }
  angular.module('ui.router.util').service('$resolve', $Resolve);
  /**
 * @ngdoc object
 * @name ui.router.util.$templateFactory
 *
 * @requires $http
 * @requires $templateCache
 * @requires $injector
 *
 * @description
 * Service. Manages loading of templates.
 */
  $TemplateFactory.$inject = [
    '$http',
    '$templateCache',
    '$injector'
  ];
  function $TemplateFactory($http, $templateCache, $injector) {
    /**
   * @ngdoc function
   * @name ui.router.util.$templateFactory#fromConfig
   * @methodOf ui.router.util.$templateFactory
   *
   * @description
   * Creates a template from a configuration object. 
   *
   * @param {object} config Configuration object for which to load a template. 
   * The following properties are search in the specified order, and the first one 
   * that is defined is used to create the template:
   *
   * @param {string|object} config.template html string template or function to 
   * load via {@link ui.router.util.$templateFactory#fromString fromString}.
   * @param {string|object} config.templateUrl url to load or a function returning 
   * the url to load via {@link ui.router.util.$templateFactory#fromUrl fromUrl}.
   * @param {Function} config.templateProvider function to invoke via 
   * {@link ui.router.util.$templateFactory#fromProvider fromProvider}.
   * @param {object} params  Parameters to pass to the template function.
   * @param {object} locals Locals to pass to `invoke` if the template is loaded 
   * via a `templateProvider`. Defaults to `{ params: params }`.
   *
   * @return {string|object}  The template html as a string, or a promise for 
   * that string,or `null` if no template is configured.
   */
    this.fromConfig = function (config, params, locals) {
      return isDefined(config.template) ? this.fromString(config.template, params) : isDefined(config.templateUrl) ? this.fromUrl(config.templateUrl, params) : isDefined(config.templateProvider) ? this.fromProvider(config.templateProvider, params, locals) : null;
    };
    /**
   * @ngdoc function
   * @name ui.router.util.$templateFactory#fromString
   * @methodOf ui.router.util.$templateFactory
   *
   * @description
   * Creates a template from a string or a function returning a string.
   *
   * @param {string|object} template html template as a string or function that 
   * returns an html template as a string.
   * @param {object} params Parameters to pass to the template function.
   *
   * @return {string|object} The template html as a string, or a promise for that 
   * string.
   */
    this.fromString = function (template, params) {
      return isFunction(template) ? template(params) : template;
    };
    /**
   * @ngdoc function
   * @name ui.router.util.$templateFactory#fromUrl
   * @methodOf ui.router.util.$templateFactory
   * 
   * @description
   * Loads a template from the a URL via `$http` and `$templateCache`.
   *
   * @param {string|Function} url url of the template to load, or a function 
   * that returns a url.
   * @param {Object} params Parameters to pass to the url function.
   * @return {string|Promise.<string>} The template html as a string, or a promise 
   * for that string.
   */
    this.fromUrl = function (url, params) {
      if (isFunction(url))
        url = url(params);
      if (url == null)
        return null;
      else
        return $http.get(url, {
          cache: $templateCache,
          headers: { Accept: 'text/html' }
        }).then(function (response) {
          return response.data;
        });
    };
    /**
   * @ngdoc function
   * @name ui.router.util.$templateFactory#fromProvider
   * @methodOf ui.router.util.$templateFactory
   *
   * @description
   * Creates a template by invoking an injectable provider function.
   *
   * @param {Function} provider Function to invoke via `$injector.invoke`
   * @param {Object} params Parameters for the template.
   * @param {Object} locals Locals to pass to `invoke`. Defaults to 
   * `{ params: params }`.
   * @return {string|Promise.<string>} The template html as a string, or a promise 
   * for that string.
   */
    this.fromProvider = function (provider, params, locals) {
      return $injector.invoke(provider, null, locals || { params: params });
    };
  }
  angular.module('ui.router.util').service('$templateFactory', $TemplateFactory);
  var $$UMFP;
  // reference to $UrlMatcherFactoryProvider
  /**
 * @ngdoc object
 * @name ui.router.util.type:UrlMatcher
 *
 * @description
 * Matches URLs against patterns and extracts named parameters from the path or the search
 * part of the URL. A URL pattern consists of a path pattern, optionally followed by '?' and a list
 * of search parameters. Multiple search parameter names are separated by '&'. Search parameters
 * do not influence whether or not a URL is matched, but their values are passed through into
 * the matched parameters returned by {@link ui.router.util.type:UrlMatcher#methods_exec exec}.
 *
 * Path parameter placeholders can be specified using simple colon/catch-all syntax or curly brace
 * syntax, which optionally allows a regular expression for the parameter to be specified:
 *
 * * `':'` name - colon placeholder
 * * `'*'` name - catch-all placeholder
 * * `'{' name '}'` - curly placeholder
 * * `'{' name ':' regexp|type '}'` - curly placeholder with regexp or type name. Should the
 *   regexp itself contain curly braces, they must be in matched pairs or escaped with a backslash.
 *
 * Parameter names may contain only word characters (latin letters, digits, and underscore) and
 * must be unique within the pattern (across both path and search parameters). For colon
 * placeholders or curly placeholders without an explicit regexp, a path parameter matches any
 * number of characters other than '/'. For catch-all placeholders the path parameter matches
 * any number of characters.
 *
 * Examples:
 *
 * * `'/hello/'` - Matches only if the path is exactly '/hello/'. There is no special treatment for
 *   trailing slashes, and patterns have to match the entire path, not just a prefix.
 * * `'/user/:id'` - Matches '/user/bob' or '/user/1234!!!' or even '/user/' but not '/user' or
 *   '/user/bob/details'. The second path segment will be captured as the parameter 'id'.
 * * `'/user/{id}'` - Same as the previous example, but using curly brace syntax.
 * * `'/user/{id:[^/]*}'` - Same as the previous example.
 * * `'/user/{id:[0-9a-fA-F]{1,8}}'` - Similar to the previous example, but only matches if the id
 *   parameter consists of 1 to 8 hex digits.
 * * `'/files/{path:.*}'` - Matches any URL starting with '/files/' and captures the rest of the
 *   path into the parameter 'path'.
 * * `'/files/*path'` - ditto.
 * * `'/calendar/{start:date}'` - Matches "/calendar/2014-11-12" (because the pattern defined
 *   in the built-in  `date` Type matches `2014-11-12`) and provides a Date object in $stateParams.start
 *
 * @param {string} pattern  The pattern to compile into a matcher.
 * @param {Object} config  A configuration object hash:
 * @param {Object=} parentMatcher Used to concatenate the pattern/config onto
 *   an existing UrlMatcher
 *
 * * `caseInsensitive` - `true` if URL matching should be case insensitive, otherwise `false`, the default value (for backward compatibility) is `false`.
 * * `strict` - `false` if matching against a URL with a trailing slash should be treated as equivalent to a URL without a trailing slash, the default value is `true`.
 *
 * @property {string} prefix  A static prefix of this pattern. The matcher guarantees that any
 *   URL matching this matcher (i.e. any string for which {@link ui.router.util.type:UrlMatcher#methods_exec exec()} returns
 *   non-null) will start with this prefix.
 *
 * @property {string} source  The pattern that was passed into the constructor
 *
 * @property {string} sourcePath  The path portion of the source property
 *
 * @property {string} sourceSearch  The search portion of the source property
 *
 * @property {string} regex  The constructed regex that will be used to match against the url when
 *   it is time to determine which url will match.
 *
 * @returns {Object}  New `UrlMatcher` object
 */
  function UrlMatcher(pattern, config, parentMatcher) {
    config = extend({ params: {} }, isObject(config) ? config : {});
    // Find all placeholders and create a compiled pattern, using either classic or curly syntax:
    //   '*' name
    //   ':' name
    //   '{' name '}'
    //   '{' name ':' regexp '}'
    // The regular expression is somewhat complicated due to the need to allow curly braces
    // inside the regular expression. The placeholder regexp breaks down as follows:
    //    ([:*])([\w\[\]]+)              - classic placeholder ($1 / $2) (search version has - for snake-case)
    //    \{([\w\[\]]+)(?:\:\s*( ... ))?\}  - curly brace placeholder ($3) with optional regexp/type ... ($4) (search version has - for snake-case
    //    (?: ... | ... | ... )+         - the regexp consists of any number of atoms, an atom being either
    //    [^{}\\]+                       - anything other than curly braces or backslash
    //    \\.                            - a backslash escape
    //    \{(?:[^{}\\]+|\\.)*\}          - a matched set of curly braces containing other atoms
    var placeholder = /([:*])([\w\[\]]+)|\{([\w\[\]]+)(?:\:\s*((?:[^{}\\]+|\\.|\{(?:[^{}\\]+|\\.)*\})+))?\}/g, searchPlaceholder = /([:]?)([\w\[\].-]+)|\{([\w\[\].-]+)(?:\:\s*((?:[^{}\\]+|\\.|\{(?:[^{}\\]+|\\.)*\})+))?\}/g, compiled = '^', last = 0, m, segments = this.segments = [], parentParams = parentMatcher ? parentMatcher.params : {}, params = this.params = parentMatcher ? parentMatcher.params.$$new() : new $$UMFP.ParamSet(), paramNames = [];
    function addParameter(id, type, config, location) {
      paramNames.push(id);
      if (parentParams[id])
        return parentParams[id];
      if (!/^\w+([-.]+\w+)*(?:\[\])?$/.test(id))
        throw new Error('Invalid parameter name \'' + id + '\' in pattern \'' + pattern + '\'');
      if (params[id])
        throw new Error('Duplicate parameter name \'' + id + '\' in pattern \'' + pattern + '\'');
      params[id] = new $$UMFP.Param(id, type, config, location);
      return params[id];
    }
    function quoteRegExp(string, pattern, squash, optional) {
      var surroundPattern = [
          '',
          ''
        ], result = string.replace(/[\\\[\]\^$*+?.()|{}]/g, '\\$&');
      if (!pattern)
        return result;
      switch (squash) {
      case false:
        surroundPattern = [
          '(',
          ')' + (optional ? '?' : '')
        ];
        break;
      case true:
        result = result.replace(/\/$/, '');
        surroundPattern = [
          '(?:/(',
          ')|/)?'
        ];
        break;
      default:
        surroundPattern = [
          '(' + squash + '|',
          ')?'
        ];
        break;
      }
      return result + surroundPattern[0] + pattern + surroundPattern[1];
    }
    this.source = pattern;
    // Split into static segments separated by path parameter placeholders.
    // The number of segments is always 1 more than the number of parameters.
    function matchDetails(m, isSearch) {
      var id, regexp, segment, type, cfg, arrayMode;
      id = m[2] || m[3];
      // IE[78] returns '' for unmatched groups instead of null
      cfg = config.params[id];
      segment = pattern.substring(last, m.index);
      regexp = isSearch ? m[4] : m[4] || (m[1] == '*' ? '.*' : null);
      if (regexp) {
        type = $$UMFP.type(regexp) || inherit($$UMFP.type('string'), { pattern: new RegExp(regexp, config.caseInsensitive ? 'i' : undefined) });
      }
      return {
        id: id,
        regexp: regexp,
        segment: segment,
        type: type,
        cfg: cfg
      };
    }
    var p, param, segment;
    while (m = placeholder.exec(pattern)) {
      p = matchDetails(m, false);
      if (p.segment.indexOf('?') >= 0)
        break;
      // we're into the search part
      param = addParameter(p.id, p.type, p.cfg, 'path');
      compiled += quoteRegExp(p.segment, param.type.pattern.source, param.squash, param.isOptional);
      segments.push(p.segment);
      last = placeholder.lastIndex;
    }
    segment = pattern.substring(last);
    // Find any search parameter names and remove them from the last segment
    var i = segment.indexOf('?');
    if (i >= 0) {
      var search = this.sourceSearch = segment.substring(i);
      segment = segment.substring(0, i);
      this.sourcePath = pattern.substring(0, last + i);
      if (search.length > 0) {
        last = 0;
        while (m = searchPlaceholder.exec(search)) {
          p = matchDetails(m, true);
          param = addParameter(p.id, p.type, p.cfg, 'search');
          last = placeholder.lastIndex;  // check if ?&
        }
      }
    } else {
      this.sourcePath = pattern;
      this.sourceSearch = '';
    }
    compiled += quoteRegExp(segment) + (config.strict === false ? '/?' : '') + '$';
    segments.push(segment);
    this.regexp = new RegExp(compiled, config.caseInsensitive ? 'i' : undefined);
    this.prefix = segments[0];
    this.$$paramNames = paramNames;
  }
  /**
 * @ngdoc function
 * @name ui.router.util.type:UrlMatcher#concat
 * @methodOf ui.router.util.type:UrlMatcher
 *
 * @description
 * Returns a new matcher for a pattern constructed by appending the path part and adding the
 * search parameters of the specified pattern to this pattern. The current pattern is not
 * modified. This can be understood as creating a pattern for URLs that are relative to (or
 * suffixes of) the current pattern.
 *
 * @example
 * The following two matchers are equivalent:
 * <pre>
 * new UrlMatcher('/user/{id}?q').concat('/details?date');
 * new UrlMatcher('/user/{id}/details?q&date');
 * </pre>
 *
 * @param {string} pattern  The pattern to append.
 * @param {Object} config  An object hash of the configuration for the matcher.
 * @returns {UrlMatcher}  A matcher for the concatenated pattern.
 */
  UrlMatcher.prototype.concat = function (pattern, config) {
    // Because order of search parameters is irrelevant, we can add our own search
    // parameters to the end of the new pattern. Parse the new pattern by itself
    // and then join the bits together, but it's much easier to do this on a string level.
    var defaultConfig = {
        caseInsensitive: $$UMFP.caseInsensitive(),
        strict: $$UMFP.strictMode(),
        squash: $$UMFP.defaultSquashPolicy()
      };
    return new UrlMatcher(this.sourcePath + pattern + this.sourceSearch, extend(defaultConfig, config), this);
  };
  UrlMatcher.prototype.toString = function () {
    return this.source;
  };
  /**
 * @ngdoc function
 * @name ui.router.util.type:UrlMatcher#exec
 * @methodOf ui.router.util.type:UrlMatcher
 *
 * @description
 * Tests the specified path against this matcher, and returns an object containing the captured
 * parameter values, or null if the path does not match. The returned object contains the values
 * of any search parameters that are mentioned in the pattern, but their value may be null if
 * they are not present in `searchParams`. This means that search parameters are always treated
 * as optional.
 *
 * @example
 * <pre>
 * new UrlMatcher('/user/{id}?q&r').exec('/user/bob', {
 *   x: '1', q: 'hello'
 * });
 * // returns { id: 'bob', q: 'hello', r: null }
 * </pre>
 *
 * @param {string} path  The URL path to match, e.g. `$location.path()`.
 * @param {Object} searchParams  URL search parameters, e.g. `$location.search()`.
 * @returns {Object}  The captured parameter values.
 */
  UrlMatcher.prototype.exec = function (path, searchParams) {
    var m = this.regexp.exec(path);
    if (!m)
      return null;
    searchParams = searchParams || {};
    var paramNames = this.parameters(), nTotal = paramNames.length, nPath = this.segments.length - 1, values = {}, i, j, cfg, paramName;
    if (nPath !== m.length - 1)
      throw new Error('Unbalanced capture group in route \'' + this.source + '\'');
    function decodePathArray(string) {
      function reverseString(str) {
        return str.split('').reverse().join('');
      }
      function unquoteDashes(str) {
        return str.replace(/\\-/g, '-');
      }
      var split = reverseString(string).split(/-(?!\\)/);
      var allReversed = map(split, reverseString);
      return map(allReversed, unquoteDashes).reverse();
    }
    var param, paramVal;
    for (i = 0; i < nPath; i++) {
      paramName = paramNames[i];
      param = this.params[paramName];
      paramVal = m[i + 1];
      // if the param value matches a pre-replace pair, replace the value before decoding.
      for (j = 0; j < param.replace.length; j++) {
        if (param.replace[j].from === paramVal)
          paramVal = param.replace[j].to;
      }
      if (paramVal && param.array === true)
        paramVal = decodePathArray(paramVal);
      if (isDefined(paramVal))
        paramVal = param.type.decode(paramVal);
      values[paramName] = param.value(paramVal);
    }
    for (; i < nTotal; i++) {
      paramName = paramNames[i];
      values[paramName] = this.params[paramName].value(searchParams[paramName]);
      param = this.params[paramName];
      paramVal = searchParams[paramName];
      for (j = 0; j < param.replace.length; j++) {
        if (param.replace[j].from === paramVal)
          paramVal = param.replace[j].to;
      }
      if (isDefined(paramVal))
        paramVal = param.type.decode(paramVal);
      values[paramName] = param.value(paramVal);
    }
    return values;
  };
  /**
 * @ngdoc function
 * @name ui.router.util.type:UrlMatcher#parameters
 * @methodOf ui.router.util.type:UrlMatcher
 *
 * @description
 * Returns the names of all path and search parameters of this pattern in an unspecified order.
 *
 * @returns {Array.<string>}  An array of parameter names. Must be treated as read-only. If the
 *    pattern has no parameters, an empty array is returned.
 */
  UrlMatcher.prototype.parameters = function (param) {
    if (!isDefined(param))
      return this.$$paramNames;
    return this.params[param] || null;
  };
  /**
 * @ngdoc function
 * @name ui.router.util.type:UrlMatcher#validates
 * @methodOf ui.router.util.type:UrlMatcher
 *
 * @description
 * Checks an object hash of parameters to validate their correctness according to the parameter
 * types of this `UrlMatcher`.
 *
 * @param {Object} params The object hash of parameters to validate.
 * @returns {boolean} Returns `true` if `params` validates, otherwise `false`.
 */
  UrlMatcher.prototype.validates = function (params) {
    return this.params.$$validates(params);
  };
  /**
 * @ngdoc function
 * @name ui.router.util.type:UrlMatcher#format
 * @methodOf ui.router.util.type:UrlMatcher
 *
 * @description
 * Creates a URL that matches this pattern by substituting the specified values
 * for the path and search parameters. Null values for path parameters are
 * treated as empty strings.
 *
 * @example
 * <pre>
 * new UrlMatcher('/user/{id}?q').format({ id:'bob', q:'yes' });
 * // returns '/user/bob?q=yes'
 * </pre>
 *
 * @param {Object} values  the values to substitute for the parameters in this pattern.
 * @returns {string}  the formatted URL (path and optionally search part).
 */
  UrlMatcher.prototype.format = function (values) {
    values = values || {};
    var segments = this.segments, params = this.parameters(), paramset = this.params;
    if (!this.validates(values))
      return null;
    var i, search = false, nPath = segments.length - 1, nTotal = params.length, result = segments[0];
    function encodeDashes(str) {
      // Replace dashes with encoded "\-"
      return encodeURIComponent(str).replace(/-/g, function (c) {
        return '%5C%' + c.charCodeAt(0).toString(16).toUpperCase();
      });
    }
    for (i = 0; i < nTotal; i++) {
      var isPathParam = i < nPath;
      var name = params[i], param = paramset[name], value = param.value(values[name]);
      var isDefaultValue = param.isOptional && param.type.equals(param.value(), value);
      var squash = isDefaultValue ? param.squash : false;
      var encoded = param.type.encode(value);
      if (isPathParam) {
        var nextSegment = segments[i + 1];
        var isFinalPathParam = i + 1 === nPath;
        if (squash === false) {
          if (encoded != null) {
            if (isArray(encoded)) {
              result += map(encoded, encodeDashes).join('-');
            } else {
              result += encodeURIComponent(encoded);
            }
          }
          result += nextSegment;
        } else if (squash === true) {
          var capture = result.match(/\/$/) ? /\/?(.*)/ : /(.*)/;
          result += nextSegment.match(capture)[1];
        } else if (isString(squash)) {
          result += squash + nextSegment;
        }
        if (isFinalPathParam && param.squash === true && result.slice(-1) === '/')
          result = result.slice(0, -1);
      } else {
        if (encoded == null || isDefaultValue && squash !== false)
          continue;
        if (!isArray(encoded))
          encoded = [encoded];
        if (encoded.length === 0)
          continue;
        encoded = map(encoded, encodeURIComponent).join('&' + name + '=');
        result += (search ? '&' : '?') + (name + '=' + encoded);
        search = true;
      }
    }
    return result;
  };
  /**
 * @ngdoc object
 * @name ui.router.util.type:Type
 *
 * @description
 * Implements an interface to define custom parameter types that can be decoded from and encoded to
 * string parameters matched in a URL. Used by {@link ui.router.util.type:UrlMatcher `UrlMatcher`}
 * objects when matching or formatting URLs, or comparing or validating parameter values.
 *
 * See {@link ui.router.util.$urlMatcherFactory#methods_type `$urlMatcherFactory#type()`} for more
 * information on registering custom types.
 *
 * @param {Object} config  A configuration object which contains the custom type definition.  The object's
 *        properties will override the default methods and/or pattern in `Type`'s public interface.
 * @example
 * <pre>
 * {
 *   decode: function(val) { return parseInt(val, 10); },
 *   encode: function(val) { return val && val.toString(); },
 *   equals: function(a, b) { return this.is(a) && a === b; },
 *   is: function(val) { return angular.isNumber(val) isFinite(val) && val % 1 === 0; },
 *   pattern: /\d+/
 * }
 * </pre>
 *
 * @property {RegExp} pattern The regular expression pattern used to match values of this type when
 *           coming from a substring of a URL.
 *
 * @returns {Object}  Returns a new `Type` object.
 */
  function Type(config) {
    extend(this, config);
  }
  /**
 * @ngdoc function
 * @name ui.router.util.type:Type#is
 * @methodOf ui.router.util.type:Type
 *
 * @description
 * Detects whether a value is of a particular type. Accepts a native (decoded) value
 * and determines whether it matches the current `Type` object.
 *
 * @param {*} val  The value to check.
 * @param {string} key  Optional. If the type check is happening in the context of a specific
 *        {@link ui.router.util.type:UrlMatcher `UrlMatcher`} object, this is the name of the
 *        parameter in which `val` is stored. Can be used for meta-programming of `Type` objects.
 * @returns {Boolean}  Returns `true` if the value matches the type, otherwise `false`.
 */
  Type.prototype.is = function (val, key) {
    return true;
  };
  /**
 * @ngdoc function
 * @name ui.router.util.type:Type#encode
 * @methodOf ui.router.util.type:Type
 *
 * @description
 * Encodes a custom/native type value to a string that can be embedded in a URL. Note that the
 * return value does *not* need to be URL-safe (i.e. passed through `encodeURIComponent()`), it
 * only needs to be a representation of `val` that has been coerced to a string.
 *
 * @param {*} val  The value to encode.
 * @param {string} key  The name of the parameter in which `val` is stored. Can be used for
 *        meta-programming of `Type` objects.
 * @returns {string}  Returns a string representation of `val` that can be encoded in a URL.
 */
  Type.prototype.encode = function (val, key) {
    return val;
  };
  /**
 * @ngdoc function
 * @name ui.router.util.type:Type#decode
 * @methodOf ui.router.util.type:Type
 *
 * @description
 * Converts a parameter value (from URL string or transition param) to a custom/native value.
 *
 * @param {string} val  The URL parameter value to decode.
 * @param {string} key  The name of the parameter in which `val` is stored. Can be used for
 *        meta-programming of `Type` objects.
 * @returns {*}  Returns a custom representation of the URL parameter value.
 */
  Type.prototype.decode = function (val, key) {
    return val;
  };
  /**
 * @ngdoc function
 * @name ui.router.util.type:Type#equals
 * @methodOf ui.router.util.type:Type
 *
 * @description
 * Determines whether two decoded values are equivalent.
 *
 * @param {*} a  A value to compare against.
 * @param {*} b  A value to compare against.
 * @returns {Boolean}  Returns `true` if the values are equivalent/equal, otherwise `false`.
 */
  Type.prototype.equals = function (a, b) {
    return a == b;
  };
  Type.prototype.$subPattern = function () {
    var sub = this.pattern.toString();
    return sub.substr(1, sub.length - 2);
  };
  Type.prototype.pattern = /.*/;
  Type.prototype.toString = function () {
    return '{Type:' + this.name + '}';
  };
  /** Given an encoded string, or a decoded object, returns a decoded object */
  Type.prototype.$normalize = function (val) {
    return this.is(val) ? val : this.decode(val);
  };
  /*
 * Wraps an existing custom Type as an array of Type, depending on 'mode'.
 * e.g.:
 * - urlmatcher pattern "/path?{queryParam[]:int}"
 * - url: "/path?queryParam=1&queryParam=2
 * - $stateParams.queryParam will be [1, 2]
 * if `mode` is "auto", then
 * - url: "/path?queryParam=1 will create $stateParams.queryParam: 1
 * - url: "/path?queryParam=1&queryParam=2 will create $stateParams.queryParam: [1, 2]
 */
  Type.prototype.$asArray = function (mode, isSearch) {
    if (!mode)
      return this;
    if (mode === 'auto' && !isSearch)
      throw new Error('\'auto\' array mode is for query parameters only');
    function ArrayType(type, mode) {
      function bindTo(type, callbackName) {
        return function () {
          return type[callbackName].apply(type, arguments);
        };
      }
      // Wrap non-array value as array
      function arrayWrap(val) {
        return isArray(val) ? val : isDefined(val) ? [val] : [];
      }
      // Unwrap array value for "auto" mode. Return undefined for empty array.
      function arrayUnwrap(val) {
        switch (val.length) {
        case 0:
          return undefined;
        case 1:
          return mode === 'auto' ? val[0] : val;
        default:
          return val;
        }
      }
      function falsey(val) {
        return !val;
      }
      // Wraps type (.is/.encode/.decode) functions to operate on each value of an array
      function arrayHandler(callback, allTruthyMode) {
        return function handleArray(val) {
          if (isArray(val) && val.length === 0)
            return val;
          val = arrayWrap(val);
          var result = map(val, callback);
          if (allTruthyMode === true)
            return filter(result, falsey).length === 0;
          return arrayUnwrap(result);
        };
      }
      // Wraps type (.equals) functions to operate on each value of an array
      function arrayEqualsHandler(callback) {
        return function handleArray(val1, val2) {
          var left = arrayWrap(val1), right = arrayWrap(val2);
          if (left.length !== right.length)
            return false;
          for (var i = 0; i < left.length; i++) {
            if (!callback(left[i], right[i]))
              return false;
          }
          return true;
        };
      }
      this.encode = arrayHandler(bindTo(type, 'encode'));
      this.decode = arrayHandler(bindTo(type, 'decode'));
      this.is = arrayHandler(bindTo(type, 'is'), true);
      this.equals = arrayEqualsHandler(bindTo(type, 'equals'));
      this.pattern = type.pattern;
      this.$normalize = arrayHandler(bindTo(type, '$normalize'));
      this.name = type.name;
      this.$arrayMode = mode;
    }
    return new ArrayType(this, mode);
  };
  /**
 * @ngdoc object
 * @name ui.router.util.$urlMatcherFactory
 *
 * @description
 * Factory for {@link ui.router.util.type:UrlMatcher `UrlMatcher`} instances. The factory
 * is also available to providers under the name `$urlMatcherFactoryProvider`.
 */
  function $UrlMatcherFactory() {
    $$UMFP = this;
    var isCaseInsensitive = false, isStrictMode = true, defaultSquashPolicy = false;
    // Use tildes to pre-encode slashes.
    // If the slashes are simply URLEncoded, the browser can choose to pre-decode them,
    // and bidirectional encoding/decoding fails.
    // Tilde was chosen because it's not a RFC 3986 section 2.2 Reserved Character
    function valToString(val) {
      return val != null ? val.toString().replace(/~/g, '~~').replace(/\//g, '~2F') : val;
    }
    function valFromString(val) {
      return val != null ? val.toString().replace(/~2F/g, '/').replace(/~~/g, '~') : val;
    }
    var $types = {}, enqueue = true, typeQueue = [], injector, defaultTypes = {
        'string': {
          encode: valToString,
          decode: valFromString,
          is: function (val) {
            return val == null || !isDefined(val) || typeof val === 'string';
          },
          pattern: /[^/]*/
        },
        'int': {
          encode: valToString,
          decode: function (val) {
            return parseInt(val, 10);
          },
          is: function (val) {
            return isDefined(val) && this.decode(val.toString()) === val;
          },
          pattern: /\d+/
        },
        'bool': {
          encode: function (val) {
            return val ? 1 : 0;
          },
          decode: function (val) {
            return parseInt(val, 10) !== 0;
          },
          is: function (val) {
            return val === true || val === false;
          },
          pattern: /0|1/
        },
        'date': {
          encode: function (val) {
            if (!this.is(val))
              return undefined;
            return [
              val.getFullYear(),
              ('0' + (val.getMonth() + 1)).slice(-2),
              ('0' + val.getDate()).slice(-2)
            ].join('-');
          },
          decode: function (val) {
            if (this.is(val))
              return val;
            var match = this.capture.exec(val);
            return match ? new Date(match[1], match[2] - 1, match[3]) : undefined;
          },
          is: function (val) {
            return val instanceof Date && !isNaN(val.valueOf());
          },
          equals: function (a, b) {
            return this.is(a) && this.is(b) && a.toISOString() === b.toISOString();
          },
          pattern: /[0-9]{4}-(?:0[1-9]|1[0-2])-(?:0[1-9]|[1-2][0-9]|3[0-1])/,
          capture: /([0-9]{4})-(0[1-9]|1[0-2])-(0[1-9]|[1-2][0-9]|3[0-1])/
        },
        'json': {
          encode: angular.toJson,
          decode: angular.fromJson,
          is: angular.isObject,
          equals: angular.equals,
          pattern: /[^/]*/
        },
        'any': {
          encode: angular.identity,
          decode: angular.identity,
          equals: angular.equals,
          pattern: /.*/
        }
      };
    function getDefaultConfig() {
      return {
        strict: isStrictMode,
        caseInsensitive: isCaseInsensitive
      };
    }
    function isInjectable(value) {
      return isFunction(value) || isArray(value) && isFunction(value[value.length - 1]);
    }
    /**
   * [Internal] Get the default value of a parameter, which may be an injectable function.
   */
    $UrlMatcherFactory.$$getDefaultValue = function (config) {
      if (!isInjectable(config.value))
        return config.value;
      if (!injector)
        throw new Error('Injectable functions cannot be called at configuration time');
      return injector.invoke(config.value);
    };
    /**
   * @ngdoc function
   * @name ui.router.util.$urlMatcherFactory#caseInsensitive
   * @methodOf ui.router.util.$urlMatcherFactory
   *
   * @description
   * Defines whether URL matching should be case sensitive (the default behavior), or not.
   *
   * @param {boolean} value `false` to match URL in a case sensitive manner; otherwise `true`;
   * @returns {boolean} the current value of caseInsensitive
   */
    this.caseInsensitive = function (value) {
      if (isDefined(value))
        isCaseInsensitive = value;
      return isCaseInsensitive;
    };
    /**
   * @ngdoc function
   * @name ui.router.util.$urlMatcherFactory#strictMode
   * @methodOf ui.router.util.$urlMatcherFactory
   *
   * @description
   * Defines whether URLs should match trailing slashes, or not (the default behavior).
   *
   * @param {boolean=} value `false` to match trailing slashes in URLs, otherwise `true`.
   * @returns {boolean} the current value of strictMode
   */
    this.strictMode = function (value) {
      if (isDefined(value))
        isStrictMode = value;
      return isStrictMode;
    };
    /**
   * @ngdoc function
   * @name ui.router.util.$urlMatcherFactory#defaultSquashPolicy
   * @methodOf ui.router.util.$urlMatcherFactory
   *
   * @description
   * Sets the default behavior when generating or matching URLs with default parameter values.
   *
   * @param {string} value A string that defines the default parameter URL squashing behavior.
   *    `nosquash`: When generating an href with a default parameter value, do not squash the parameter value from the URL
   *    `slash`: When generating an href with a default parameter value, squash (remove) the parameter value, and, if the
   *             parameter is surrounded by slashes, squash (remove) one slash from the URL
   *    any other string, e.g. "~": When generating an href with a default parameter value, squash (remove)
   *             the parameter value from the URL and replace it with this string.
   */
    this.defaultSquashPolicy = function (value) {
      if (!isDefined(value))
        return defaultSquashPolicy;
      if (value !== true && value !== false && !isString(value))
        throw new Error('Invalid squash policy: ' + value + '. Valid policies: false, true, arbitrary-string');
      defaultSquashPolicy = value;
      return value;
    };
    /**
   * @ngdoc function
   * @name ui.router.util.$urlMatcherFactory#compile
   * @methodOf ui.router.util.$urlMatcherFactory
   *
   * @description
   * Creates a {@link ui.router.util.type:UrlMatcher `UrlMatcher`} for the specified pattern.
   *
   * @param {string} pattern  The URL pattern.
   * @param {Object} config  The config object hash.
   * @returns {UrlMatcher}  The UrlMatcher.
   */
    this.compile = function (pattern, config) {
      return new UrlMatcher(pattern, extend(getDefaultConfig(), config));
    };
    /**
   * @ngdoc function
   * @name ui.router.util.$urlMatcherFactory#isMatcher
   * @methodOf ui.router.util.$urlMatcherFactory
   *
   * @description
   * Returns true if the specified object is a `UrlMatcher`, or false otherwise.
   *
   * @param {Object} object  The object to perform the type check against.
   * @returns {Boolean}  Returns `true` if the object matches the `UrlMatcher` interface, by
   *          implementing all the same methods.
   */
    this.isMatcher = function (o) {
      if (!isObject(o))
        return false;
      var result = true;
      forEach(UrlMatcher.prototype, function (val, name) {
        if (isFunction(val)) {
          result = result && (isDefined(o[name]) && isFunction(o[name]));
        }
      });
      return result;
    };
    /**
   * @ngdoc function
   * @name ui.router.util.$urlMatcherFactory#type
   * @methodOf ui.router.util.$urlMatcherFactory
   *
   * @description
   * Registers a custom {@link ui.router.util.type:Type `Type`} object that can be used to
   * generate URLs with typed parameters.
   *
   * @param {string} name  The type name.
   * @param {Object|Function} definition   The type definition. See
   *        {@link ui.router.util.type:Type `Type`} for information on the values accepted.
   * @param {Object|Function} definitionFn (optional) A function that is injected before the app
   *        runtime starts.  The result of this function is merged into the existing `definition`.
   *        See {@link ui.router.util.type:Type `Type`} for information on the values accepted.
   *
   * @returns {Object}  Returns `$urlMatcherFactoryProvider`.
   *
   * @example
   * This is a simple example of a custom type that encodes and decodes items from an
   * array, using the array index as the URL-encoded value:
   *
   * <pre>
   * var list = ['John', 'Paul', 'George', 'Ringo'];
   *
   * $urlMatcherFactoryProvider.type('listItem', {
   *   encode: function(item) {
   *     // Represent the list item in the URL using its corresponding index
   *     return list.indexOf(item);
   *   },
   *   decode: function(item) {
   *     // Look up the list item by index
   *     return list[parseInt(item, 10)];
   *   },
   *   is: function(item) {
   *     // Ensure the item is valid by checking to see that it appears
   *     // in the list
   *     return list.indexOf(item) > -1;
   *   }
   * });
   *
   * $stateProvider.state('list', {
   *   url: "/list/{item:listItem}",
   *   controller: function($scope, $stateParams) {
   *     console.log($stateParams.item);
   *   }
   * });
   *
   * // ...
   *
   * // Changes URL to '/list/3', logs "Ringo" to the console
   * $state.go('list', { item: "Ringo" });
   * </pre>
   *
   * This is a more complex example of a type that relies on dependency injection to
   * interact with services, and uses the parameter name from the URL to infer how to
   * handle encoding and decoding parameter values:
   *
   * <pre>
   * // Defines a custom type that gets a value from a service,
   * // where each service gets different types of values from
   * // a backend API:
   * $urlMatcherFactoryProvider.type('dbObject', {}, function(Users, Posts) {
   *
   *   // Matches up services to URL parameter names
   *   var services = {
   *     user: Users,
   *     post: Posts
   *   };
   *
   *   return {
   *     encode: function(object) {
   *       // Represent the object in the URL using its unique ID
   *       return object.id;
   *     },
   *     decode: function(value, key) {
   *       // Look up the object by ID, using the parameter
   *       // name (key) to call the correct service
   *       return services[key].findById(value);
   *     },
   *     is: function(object, key) {
   *       // Check that object is a valid dbObject
   *       return angular.isObject(object) && object.id && services[key];
   *     }
   *     equals: function(a, b) {
   *       // Check the equality of decoded objects by comparing
   *       // their unique IDs
   *       return a.id === b.id;
   *     }
   *   };
   * });
   *
   * // In a config() block, you can then attach URLs with
   * // type-annotated parameters:
   * $stateProvider.state('users', {
   *   url: "/users",
   *   // ...
   * }).state('users.item', {
   *   url: "/{user:dbObject}",
   *   controller: function($scope, $stateParams) {
   *     // $stateParams.user will now be an object returned from
   *     // the Users service
   *   },
   *   // ...
   * });
   * </pre>
   */
    this.type = function (name, definition, definitionFn) {
      if (!isDefined(definition))
        return $types[name];
      if ($types.hasOwnProperty(name))
        throw new Error('A type named \'' + name + '\' has already been defined.');
      $types[name] = new Type(extend({ name: name }, definition));
      if (definitionFn) {
        typeQueue.push({
          name: name,
          def: definitionFn
        });
        if (!enqueue)
          flushTypeQueue();
      }
      return this;
    };
    // `flushTypeQueue()` waits until `$urlMatcherFactory` is injected before invoking the queued `definitionFn`s
    function flushTypeQueue() {
      while (typeQueue.length) {
        var type = typeQueue.shift();
        if (type.pattern)
          throw new Error('You cannot override a type\'s .pattern at runtime.');
        angular.extend($types[type.name], injector.invoke(type.def));
      }
    }
    // Register default types. Store them in the prototype of $types.
    forEach(defaultTypes, function (type, name) {
      $types[name] = new Type(extend({ name: name }, type));
    });
    $types = inherit($types, {});
    /* No need to document $get, since it returns this */
    this.$get = [
      '$injector',
      function ($injector) {
        injector = $injector;
        enqueue = false;
        flushTypeQueue();
        forEach(defaultTypes, function (type, name) {
          if (!$types[name])
            $types[name] = new Type(type);
        });
        return this;
      }
    ];
    this.Param = function Param(id, type, config, location) {
      var self = this;
      config = unwrapShorthand(config);
      type = getType(config, type, location);
      var arrayMode = getArrayMode();
      type = arrayMode ? type.$asArray(arrayMode, location === 'search') : type;
      if (type.name === 'string' && !arrayMode && location === 'path' && config.value === undefined)
        config.value = '';
      // for 0.2.x; in 0.3.0+ do not automatically default to ""
      var isOptional = config.value !== undefined;
      var squash = getSquashPolicy(config, isOptional);
      var replace = getReplace(config, arrayMode, isOptional, squash);
      function unwrapShorthand(config) {
        var keys = isObject(config) ? objectKeys(config) : [];
        var isShorthand = indexOf(keys, 'value') === -1 && indexOf(keys, 'type') === -1 && indexOf(keys, 'squash') === -1 && indexOf(keys, 'array') === -1;
        if (isShorthand)
          config = { value: config };
        config.$$fn = isInjectable(config.value) ? config.value : function () {
          return config.value;
        };
        return config;
      }
      function getType(config, urlType, location) {
        if (config.type && urlType)
          throw new Error('Param \'' + id + '\' has two type configurations.');
        if (urlType)
          return urlType;
        if (!config.type)
          return location === 'config' ? $types.any : $types.string;
        if (angular.isString(config.type))
          return $types[config.type];
        if (config.type instanceof Type)
          return config.type;
        return new Type(config.type);
      }
      // array config: param name (param[]) overrides default settings.  explicit config overrides param name.
      function getArrayMode() {
        var arrayDefaults = { array: location === 'search' ? 'auto' : false };
        var arrayParamNomenclature = id.match(/\[\]$/) ? { array: true } : {};
        return extend(arrayDefaults, arrayParamNomenclature, config).array;
      }
      /**
     * returns false, true, or the squash value to indicate the "default parameter url squash policy".
     */
      function getSquashPolicy(config, isOptional) {
        var squash = config.squash;
        if (!isOptional || squash === false)
          return false;
        if (!isDefined(squash) || squash == null)
          return defaultSquashPolicy;
        if (squash === true || isString(squash))
          return squash;
        throw new Error('Invalid squash policy: \'' + squash + '\'. Valid policies: false, true, or arbitrary string');
      }
      function getReplace(config, arrayMode, isOptional, squash) {
        var replace, configuredKeys, defaultPolicy = [
            {
              from: '',
              to: isOptional || arrayMode ? undefined : ''
            },
            {
              from: null,
              to: isOptional || arrayMode ? undefined : ''
            }
          ];
        replace = isArray(config.replace) ? config.replace : [];
        if (isString(squash))
          replace.push({
            from: squash,
            to: undefined
          });
        configuredKeys = map(replace, function (item) {
          return item.from;
        });
        return filter(defaultPolicy, function (item) {
          return indexOf(configuredKeys, item.from) === -1;
        }).concat(replace);
      }
      /**
     * [Internal] Get the default value of a parameter, which may be an injectable function.
     */
      function $$getDefaultValue() {
        if (!injector)
          throw new Error('Injectable functions cannot be called at configuration time');
        var defaultValue = injector.invoke(config.$$fn);
        if (defaultValue !== null && defaultValue !== undefined && !self.type.is(defaultValue))
          throw new Error('Default value (' + defaultValue + ') for parameter \'' + self.id + '\' is not an instance of Type (' + self.type.name + ')');
        return defaultValue;
      }
      /**
     * [Internal] Gets the decoded representation of a value if the value is defined, otherwise, returns the
     * default value, which may be the result of an injectable function.
     */
      function $value(value) {
        function hasReplaceVal(val) {
          return function (obj) {
            return obj.from === val;
          };
        }
        function $replace(value) {
          var replacement = map(filter(self.replace, hasReplaceVal(value)), function (obj) {
              return obj.to;
            });
          return replacement.length ? replacement[0] : value;
        }
        value = $replace(value);
        return !isDefined(value) ? $$getDefaultValue() : self.type.$normalize(value);
      }
      function toString() {
        return '{Param:' + id + ' ' + type + ' squash: \'' + squash + '\' optional: ' + isOptional + '}';
      }
      extend(this, {
        id: id,
        type: type,
        location: location,
        array: arrayMode,
        squash: squash,
        replace: replace,
        isOptional: isOptional,
        value: $value,
        dynamic: undefined,
        config: config,
        toString: toString
      });
    };
    function ParamSet(params) {
      extend(this, params || {});
    }
    ParamSet.prototype = {
      $$new: function () {
        return inherit(this, extend(new ParamSet(), { $$parent: this }));
      },
      $$keys: function () {
        var keys = [], chain = [], parent = this, ignore = objectKeys(ParamSet.prototype);
        while (parent) {
          chain.push(parent);
          parent = parent.$$parent;
        }
        chain.reverse();
        forEach(chain, function (paramset) {
          forEach(objectKeys(paramset), function (key) {
            if (indexOf(keys, key) === -1 && indexOf(ignore, key) === -1)
              keys.push(key);
          });
        });
        return keys;
      },
      $$values: function (paramValues) {
        var values = {}, self = this;
        forEach(self.$$keys(), function (key) {
          values[key] = self[key].value(paramValues && paramValues[key]);
        });
        return values;
      },
      $$equals: function (paramValues1, paramValues2) {
        var equal = true, self = this;
        forEach(self.$$keys(), function (key) {
          var left = paramValues1 && paramValues1[key], right = paramValues2 && paramValues2[key];
          if (!self[key].type.equals(left, right))
            equal = false;
        });
        return equal;
      },
      $$validates: function $$validate(paramValues) {
        var keys = this.$$keys(), i, param, rawVal, normalized, encoded;
        for (i = 0; i < keys.length; i++) {
          param = this[keys[i]];
          rawVal = paramValues[keys[i]];
          if ((rawVal === undefined || rawVal === null) && param.isOptional)
            break;
          // There was no parameter value, but the param is optional
          normalized = param.type.$normalize(rawVal);
          if (!param.type.is(normalized))
            return false;
          // The value was not of the correct Type, and could not be decoded to the correct Type
          encoded = param.type.encode(normalized);
          if (angular.isString(encoded) && !param.type.pattern.exec(encoded))
            return false;  // The value was of the correct type, but when encoded, did not match the Type's regexp
        }
        return true;
      },
      $$parent: undefined
    };
    this.ParamSet = ParamSet;
  }
  // Register as a provider so it's available to other providers
  angular.module('ui.router.util').provider('$urlMatcherFactory', $UrlMatcherFactory);
  angular.module('ui.router.util').run([
    '$urlMatcherFactory',
    function ($urlMatcherFactory) {
    }
  ]);
  /**
 * @ngdoc object
 * @name ui.router.router.$urlRouterProvider
 *
 * @requires ui.router.util.$urlMatcherFactoryProvider
 * @requires $locationProvider
 *
 * @description
 * `$urlRouterProvider` has the responsibility of watching `$location`. 
 * When `$location` changes it runs through a list of rules one by one until a 
 * match is found. `$urlRouterProvider` is used behind the scenes anytime you specify 
 * a url in a state configuration. All urls are compiled into a UrlMatcher object.
 *
 * There are several methods on `$urlRouterProvider` that make it useful to use directly
 * in your module config.
 */
  $UrlRouterProvider.$inject = [
    '$locationProvider',
    '$urlMatcherFactoryProvider'
  ];
  function $UrlRouterProvider($locationProvider, $urlMatcherFactory) {
    var rules = [], otherwise = null, interceptDeferred = false, listener;
    // Returns a string that is a prefix of all strings matching the RegExp
    function regExpPrefix(re) {
      var prefix = /^\^((?:\\[^a-zA-Z0-9]|[^\\\[\]\^$*+?.()|{}]+)*)/.exec(re.source);
      return prefix != null ? prefix[1].replace(/\\(.)/g, '$1') : '';
    }
    // Interpolates matched values into a String.replace()-style pattern
    function interpolate(pattern, match) {
      return pattern.replace(/\$(\$|\d{1,2})/, function (m, what) {
        return match[what === '$' ? 0 : Number(what)];
      });
    }
    /**
   * @ngdoc function
   * @name ui.router.router.$urlRouterProvider#rule
   * @methodOf ui.router.router.$urlRouterProvider
   *
   * @description
   * Defines rules that are used by `$urlRouterProvider` to find matches for
   * specific URLs.
   *
   * @example
   * <pre>
   * var app = angular.module('app', ['ui.router.router']);
   *
   * app.config(function ($urlRouterProvider) {
   *   // Here's an example of how you might allow case insensitive urls
   *   $urlRouterProvider.rule(function ($injector, $location) {
   *     var path = $location.path(),
   *         normalized = path.toLowerCase();
   *
   *     if (path !== normalized) {
   *       return normalized;
   *     }
   *   });
   * });
   * </pre>
   *
   * @param {function} rule Handler function that takes `$injector` and `$location`
   * services as arguments. You can use them to return a valid path as a string.
   *
   * @return {object} `$urlRouterProvider` - `$urlRouterProvider` instance
   */
    this.rule = function (rule) {
      if (!isFunction(rule))
        throw new Error('\'rule\' must be a function');
      rules.push(rule);
      return this;
    };
    /**
   * @ngdoc object
   * @name ui.router.router.$urlRouterProvider#otherwise
   * @methodOf ui.router.router.$urlRouterProvider
   *
   * @description
   * Defines a path that is used when an invalid route is requested.
   *
   * @example
   * <pre>
   * var app = angular.module('app', ['ui.router.router']);
   *
   * app.config(function ($urlRouterProvider) {
   *   // if the path doesn't match any of the urls you configured
   *   // otherwise will take care of routing the user to the
   *   // specified url
   *   $urlRouterProvider.otherwise('/index');
   *
   *   // Example of using function rule as param
   *   $urlRouterProvider.otherwise(function ($injector, $location) {
   *     return '/a/valid/url';
   *   });
   * });
   * </pre>
   *
   * @param {string|function} rule The url path you want to redirect to or a function 
   * rule that returns the url path. The function version is passed two params: 
   * `$injector` and `$location` services, and must return a url string.
   *
   * @return {object} `$urlRouterProvider` - `$urlRouterProvider` instance
   */
    this.otherwise = function (rule) {
      if (isString(rule)) {
        var redirect = rule;
        rule = function () {
          return redirect;
        };
      } else if (!isFunction(rule))
        throw new Error('\'rule\' must be a function');
      otherwise = rule;
      return this;
    };
    function handleIfMatch($injector, handler, match) {
      if (!match)
        return false;
      var result = $injector.invoke(handler, handler, { $match: match });
      return isDefined(result) ? result : true;
    }
    /**
   * @ngdoc function
   * @name ui.router.router.$urlRouterProvider#when
   * @methodOf ui.router.router.$urlRouterProvider
   *
   * @description
   * Registers a handler for a given url matching. 
   * 
   * If the handler is a string, it is
   * treated as a redirect, and is interpolated according to the syntax of match
   * (i.e. like `String.replace()` for `RegExp`, or like a `UrlMatcher` pattern otherwise).
   *
   * If the handler is a function, it is injectable. It gets invoked if `$location`
   * matches. You have the option of inject the match object as `$match`.
   *
   * The handler can return
   *
   * - **falsy** to indicate that the rule didn't match after all, then `$urlRouter`
   *   will continue trying to find another one that matches.
   * - **string** which is treated as a redirect and passed to `$location.url()`
   * - **void** or any **truthy** value tells `$urlRouter` that the url was handled.
   *
   * @example
   * <pre>
   * var app = angular.module('app', ['ui.router.router']);
   *
   * app.config(function ($urlRouterProvider) {
   *   $urlRouterProvider.when($state.url, function ($match, $stateParams) {
   *     if ($state.$current.navigable !== state ||
   *         !equalForKeys($match, $stateParams) {
   *      $state.transitionTo(state, $match, false);
   *     }
   *   });
   * });
   * </pre>
   *
   * @param {string|object} what The incoming path that you want to redirect.
   * @param {string|function} handler The path you want to redirect your user to.
   */
    this.when = function (what, handler) {
      var redirect, handlerIsString = isString(handler);
      if (isString(what))
        what = $urlMatcherFactory.compile(what);
      if (!handlerIsString && !isFunction(handler) && !isArray(handler))
        throw new Error('invalid \'handler\' in when()');
      var strategies = {
          matcher: function (what, handler) {
            if (handlerIsString) {
              redirect = $urlMatcherFactory.compile(handler);
              handler = [
                '$match',
                function ($match) {
                  return redirect.format($match);
                }
              ];
            }
            return extend(function ($injector, $location) {
              return handleIfMatch($injector, handler, what.exec($location.path(), $location.search()));
            }, { prefix: isString(what.prefix) ? what.prefix : '' });
          },
          regex: function (what, handler) {
            if (what.global || what.sticky)
              throw new Error('when() RegExp must not be global or sticky');
            if (handlerIsString) {
              redirect = handler;
              handler = [
                '$match',
                function ($match) {
                  return interpolate(redirect, $match);
                }
              ];
            }
            return extend(function ($injector, $location) {
              return handleIfMatch($injector, handler, what.exec($location.path()));
            }, { prefix: regExpPrefix(what) });
          }
        };
      var check = {
          matcher: $urlMatcherFactory.isMatcher(what),
          regex: what instanceof RegExp
        };
      for (var n in check) {
        if (check[n])
          return this.rule(strategies[n](what, handler));
      }
      throw new Error('invalid \'what\' in when()');
    };
    /**
   * @ngdoc function
   * @name ui.router.router.$urlRouterProvider#deferIntercept
   * @methodOf ui.router.router.$urlRouterProvider
   *
   * @description
   * Disables (or enables) deferring location change interception.
   *
   * If you wish to customize the behavior of syncing the URL (for example, if you wish to
   * defer a transition but maintain the current URL), call this method at configuration time.
   * Then, at run time, call `$urlRouter.listen()` after you have configured your own
   * `$locationChangeSuccess` event handler.
   *
   * @example
   * <pre>
   * var app = angular.module('app', ['ui.router.router']);
   *
   * app.config(function ($urlRouterProvider) {
   *
   *   // Prevent $urlRouter from automatically intercepting URL changes;
   *   // this allows you to configure custom behavior in between
   *   // location changes and route synchronization:
   *   $urlRouterProvider.deferIntercept();
   *
   * }).run(function ($rootScope, $urlRouter, UserService) {
   *
   *   $rootScope.$on('$locationChangeSuccess', function(e) {
   *     // UserService is an example service for managing user state
   *     if (UserService.isLoggedIn()) return;
   *
   *     // Prevent $urlRouter's default handler from firing
   *     e.preventDefault();
   *
   *     UserService.handleLogin().then(function() {
   *       // Once the user has logged in, sync the current URL
   *       // to the router:
   *       $urlRouter.sync();
   *     });
   *   });
   *
   *   // Configures $urlRouter's listener *after* your custom listener
   *   $urlRouter.listen();
   * });
   * </pre>
   *
   * @param {boolean} defer Indicates whether to defer location change interception. Passing
            no parameter is equivalent to `true`.
   */
    this.deferIntercept = function (defer) {
      if (defer === undefined)
        defer = true;
      interceptDeferred = defer;
    };
    /**
   * @ngdoc object
   * @name ui.router.router.$urlRouter
   *
   * @requires $location
   * @requires $rootScope
   * @requires $injector
   * @requires $browser
   *
   * @description
   *
   */
    this.$get = $get;
    $get.$inject = [
      '$location',
      '$rootScope',
      '$injector',
      '$browser',
      '$sniffer'
    ];
    function $get($location, $rootScope, $injector, $browser, $sniffer) {
      var baseHref = $browser.baseHref(), location = $location.url(), lastPushedUrl;
      function appendBasePath(url, isHtml5, absolute) {
        if (baseHref === '/')
          return url;
        if (isHtml5)
          return baseHref.slice(0, -1) + url;
        if (absolute)
          return baseHref.slice(1) + url;
        return url;
      }
      // TODO: Optimize groups of rules with non-empty prefix into some sort of decision tree
      function update(evt) {
        if (evt && evt.defaultPrevented)
          return;
        var ignoreUpdate = lastPushedUrl && $location.url() === lastPushedUrl;
        lastPushedUrl = undefined;
        // TODO: Re-implement this in 1.0 for https://github.com/angular-ui/ui-router/issues/1573
        //if (ignoreUpdate) return true;
        function check(rule) {
          var handled = rule($injector, $location);
          if (!handled)
            return false;
          if (isString(handled))
            $location.replace().url(handled);
          return true;
        }
        var n = rules.length, i;
        for (i = 0; i < n; i++) {
          if (check(rules[i]))
            return;
        }
        // always check otherwise last to allow dynamic updates to the set of rules
        if (otherwise)
          check(otherwise);
      }
      function listen() {
        listener = listener || $rootScope.$on('$locationChangeSuccess', update);
        return listener;
      }
      if (!interceptDeferred)
        listen();
      return {
        sync: function () {
          update();
        },
        listen: function () {
          return listen();
        },
        update: function (read) {
          if (read) {
            location = $location.url();
            return;
          }
          if ($location.url() === location)
            return;
          $location.url(location);
          $location.replace();
        },
        push: function (urlMatcher, params, options) {
          var url = urlMatcher.format(params || {});
          // Handle the special hash param, if needed
          if (url !== null && params && params['#']) {
            url += '#' + params['#'];
          }
          $location.url(url);
          lastPushedUrl = options && options.$$avoidResync ? $location.url() : undefined;
          if (options && options.replace)
            $location.replace();
        },
        href: function (urlMatcher, params, options) {
          if (!urlMatcher.validates(params))
            return null;
          var isHtml5 = $locationProvider.html5Mode();
          if (angular.isObject(isHtml5)) {
            isHtml5 = isHtml5.enabled;
          }
          isHtml5 = isHtml5 && $sniffer.history;
          var url = urlMatcher.format(params);
          options = options || {};
          if (!isHtml5 && url !== null) {
            url = '#' + $locationProvider.hashPrefix() + url;
          }
          // Handle special hash param, if needed
          if (url !== null && params && params['#']) {
            url += '#' + params['#'];
          }
          url = appendBasePath(url, isHtml5, options.absolute);
          if (!options.absolute || !url) {
            return url;
          }
          var slash = !isHtml5 && url ? '/' : '', port = $location.port();
          port = port === 80 || port === 443 ? '' : ':' + port;
          return [
            $location.protocol(),
            '://',
            $location.host(),
            port,
            slash,
            url
          ].join('');
        }
      };
    }
  }
  angular.module('ui.router.router').provider('$urlRouter', $UrlRouterProvider);
  /**
 * @ngdoc object
 * @name ui.router.state.$stateProvider
 *
 * @requires ui.router.router.$urlRouterProvider
 * @requires ui.router.util.$urlMatcherFactoryProvider
 *
 * @description
 * The new `$stateProvider` works similar to Angular's v1 router, but it focuses purely
 * on state.
 *
 * A state corresponds to a "place" in the application in terms of the overall UI and
 * navigation. A state describes (via the controller / template / view properties) what
 * the UI looks like and does at that place.
 *
 * States often have things in common, and the primary way of factoring out these
 * commonalities in this model is via the state hierarchy, i.e. parent/child states aka
 * nested states.
 *
 * The `$stateProvider` provides interfaces to declare these states for your app.
 */
  $StateProvider.$inject = [
    '$urlRouterProvider',
    '$urlMatcherFactoryProvider'
  ];
  function $StateProvider($urlRouterProvider, $urlMatcherFactory) {
    var root, states = {}, $state, queue = {}, abstractKey = 'abstract';
    // Builds state properties from definition passed to registerState()
    var stateBuilder = {
        parent: function (state) {
          if (isDefined(state.parent) && state.parent)
            return findState(state.parent);
          // regex matches any valid composite state name
          // would match "contact.list" but not "contacts"
          var compositeName = /^(.+)\.[^.]+$/.exec(state.name);
          return compositeName ? findState(compositeName[1]) : root;
        },
        data: function (state) {
          if (state.parent && state.parent.data) {
            state.data = state.self.data = inherit(state.parent.data, state.data);
          }
          return state.data;
        },
        url: function (state) {
          var url = state.url, config = { params: state.params || {} };
          if (isString(url)) {
            if (url.charAt(0) == '^')
              return $urlMatcherFactory.compile(url.substring(1), config);
            return (state.parent.navigable || root).url.concat(url, config);
          }
          if (!url || $urlMatcherFactory.isMatcher(url))
            return url;
          throw new Error('Invalid url \'' + url + '\' in state \'' + state + '\'');
        },
        navigable: function (state) {
          return state.url ? state : state.parent ? state.parent.navigable : null;
        },
        ownParams: function (state) {
          var params = state.url && state.url.params || new $$UMFP.ParamSet();
          forEach(state.params || {}, function (config, id) {
            if (!params[id])
              params[id] = new $$UMFP.Param(id, null, config, 'config');
          });
          return params;
        },
        params: function (state) {
          var ownParams = pick(state.ownParams, state.ownParams.$$keys());
          return state.parent && state.parent.params ? extend(state.parent.params.$$new(), ownParams) : new $$UMFP.ParamSet();
        },
        views: function (state) {
          var views = {};
          forEach(isDefined(state.views) ? state.views : { '': state }, function (view, name) {
            if (name.indexOf('@') < 0)
              name += '@' + state.parent.name;
            views[name] = view;
          });
          return views;
        },
        path: function (state) {
          return state.parent ? state.parent.path.concat(state) : [];  // exclude root from path
        },
        includes: function (state) {
          var includes = state.parent ? extend({}, state.parent.includes) : {};
          includes[state.name] = true;
          return includes;
        },
        $delegates: {}
      };
    function isRelative(stateName) {
      return stateName.indexOf('.') === 0 || stateName.indexOf('^') === 0;
    }
    function findState(stateOrName, base) {
      if (!stateOrName)
        return undefined;
      var isStr = isString(stateOrName), name = isStr ? stateOrName : stateOrName.name, path = isRelative(name);
      if (path) {
        if (!base)
          throw new Error('No reference point given for path \'' + name + '\'');
        base = findState(base);
        var rel = name.split('.'), i = 0, pathLength = rel.length, current = base;
        for (; i < pathLength; i++) {
          if (rel[i] === '' && i === 0) {
            current = base;
            continue;
          }
          if (rel[i] === '^') {
            if (!current.parent)
              throw new Error('Path \'' + name + '\' not valid for state \'' + base.name + '\'');
            current = current.parent;
            continue;
          }
          break;
        }
        rel = rel.slice(i).join('.');
        name = current.name + (current.name && rel ? '.' : '') + rel;
      }
      var state = states[name];
      if (state && (isStr || !isStr && (state === stateOrName || state.self === stateOrName))) {
        return state;
      }
      return undefined;
    }
    function queueState(parentName, state) {
      if (!queue[parentName]) {
        queue[parentName] = [];
      }
      queue[parentName].push(state);
    }
    function flushQueuedChildren(parentName) {
      var queued = queue[parentName] || [];
      while (queued.length) {
        registerState(queued.shift());
      }
    }
    function registerState(state) {
      // Wrap a new object around the state so we can store our private details easily.
      state = inherit(state, {
        self: state,
        resolve: state.resolve || {},
        toString: function () {
          return this.name;
        }
      });
      var name = state.name;
      if (!isString(name) || name.indexOf('@') >= 0)
        throw new Error('State must have a valid name');
      if (states.hasOwnProperty(name))
        throw new Error('State \'' + name + '\' is already defined');
      // Get parent name
      var parentName = name.indexOf('.') !== -1 ? name.substring(0, name.lastIndexOf('.')) : isString(state.parent) ? state.parent : isObject(state.parent) && isString(state.parent.name) ? state.parent.name : '';
      // If parent is not registered yet, add state to queue and register later
      if (parentName && !states[parentName]) {
        return queueState(parentName, state.self);
      }
      for (var key in stateBuilder) {
        if (isFunction(stateBuilder[key]))
          state[key] = stateBuilder[key](state, stateBuilder.$delegates[key]);
      }
      states[name] = state;
      // Register the state in the global state list and with $urlRouter if necessary.
      if (!state[abstractKey] && state.url) {
        $urlRouterProvider.when(state.url, [
          '$match',
          '$stateParams',
          function ($match, $stateParams) {
            if ($state.$current.navigable != state || !equalForKeys($match, $stateParams)) {
              $state.transitionTo(state, $match, {
                inherit: true,
                location: false
              });
            }
          }
        ]);
      }
      // Register any queued children
      flushQueuedChildren(name);
      return state;
    }
    // Checks text to see if it looks like a glob.
    function isGlob(text) {
      return text.indexOf('*') > -1;
    }
    // Returns true if glob matches current $state name.
    function doesStateMatchGlob(glob) {
      var globSegments = glob.split('.'), segments = $state.$current.name.split('.');
      //match single stars
      for (var i = 0, l = globSegments.length; i < l; i++) {
        if (globSegments[i] === '*') {
          segments[i] = '*';
        }
      }
      //match greedy starts
      if (globSegments[0] === '**') {
        segments = segments.slice(indexOf(segments, globSegments[1]));
        segments.unshift('**');
      }
      //match greedy ends
      if (globSegments[globSegments.length - 1] === '**') {
        segments.splice(indexOf(segments, globSegments[globSegments.length - 2]) + 1, Number.MAX_VALUE);
        segments.push('**');
      }
      if (globSegments.length != segments.length) {
        return false;
      }
      return segments.join('') === globSegments.join('');
    }
    // Implicit root state that is always active
    root = registerState({
      name: '',
      url: '^',
      views: null,
      'abstract': true
    });
    root.navigable = null;
    /**
   * @ngdoc function
   * @name ui.router.state.$stateProvider#decorator
   * @methodOf ui.router.state.$stateProvider
   *
   * @description
   * Allows you to extend (carefully) or override (at your own peril) the 
   * `stateBuilder` object used internally by `$stateProvider`. This can be used 
   * to add custom functionality to ui-router, for example inferring templateUrl 
   * based on the state name.
   *
   * When passing only a name, it returns the current (original or decorated) builder
   * function that matches `name`.
   *
   * The builder functions that can be decorated are listed below. Though not all
   * necessarily have a good use case for decoration, that is up to you to decide.
   *
   * In addition, users can attach custom decorators, which will generate new 
   * properties within the state's internal definition. There is currently no clear 
   * use-case for this beyond accessing internal states (i.e. $state.$current), 
   * however, expect this to become increasingly relevant as we introduce additional 
   * meta-programming features.
   *
   * **Warning**: Decorators should not be interdependent because the order of 
   * execution of the builder functions in non-deterministic. Builder functions 
   * should only be dependent on the state definition object and super function.
   *
   *
   * Existing builder functions and current return values:
   *
   * - **parent** `{object}` - returns the parent state object.
   * - **data** `{object}` - returns state data, including any inherited data that is not
   *   overridden by own values (if any).
   * - **url** `{object}` - returns a {@link ui.router.util.type:UrlMatcher UrlMatcher}
   *   or `null`.
   * - **navigable** `{object}` - returns closest ancestor state that has a URL (aka is 
   *   navigable).
   * - **params** `{object}` - returns an array of state params that are ensured to 
   *   be a super-set of parent's params.
   * - **views** `{object}` - returns a views object where each key is an absolute view 
   *   name (i.e. "viewName@stateName") and each value is the config object 
   *   (template, controller) for the view. Even when you don't use the views object 
   *   explicitly on a state config, one is still created for you internally.
   *   So by decorating this builder function you have access to decorating template 
   *   and controller properties.
   * - **ownParams** `{object}` - returns an array of params that belong to the state, 
   *   not including any params defined by ancestor states.
   * - **path** `{string}` - returns the full path from the root down to this state. 
   *   Needed for state activation.
   * - **includes** `{object}` - returns an object that includes every state that 
   *   would pass a `$state.includes()` test.
   *
   * @example
   * <pre>
   * // Override the internal 'views' builder with a function that takes the state
   * // definition, and a reference to the internal function being overridden:
   * $stateProvider.decorator('views', function (state, parent) {
   *   var result = {},
   *       views = parent(state);
   *
   *   angular.forEach(views, function (config, name) {
   *     var autoName = (state.name + '.' + name).replace('.', '/');
   *     config.templateUrl = config.templateUrl || '/partials/' + autoName + '.html';
   *     result[name] = config;
   *   });
   *   return result;
   * });
   *
   * $stateProvider.state('home', {
   *   views: {
   *     'contact.list': { controller: 'ListController' },
   *     'contact.item': { controller: 'ItemController' }
   *   }
   * });
   *
   * // ...
   *
   * $state.go('home');
   * // Auto-populates list and item views with /partials/home/contact/list.html,
   * // and /partials/home/contact/item.html, respectively.
   * </pre>
   *
   * @param {string} name The name of the builder function to decorate. 
   * @param {object} func A function that is responsible for decorating the original 
   * builder function. The function receives two parameters:
   *
   *   - `{object}` - state - The state config object.
   *   - `{object}` - super - The original builder function.
   *
   * @return {object} $stateProvider - $stateProvider instance
   */
    this.decorator = decorator;
    function decorator(name, func) {
      /*jshint validthis: true */
      if (isString(name) && !isDefined(func)) {
        return stateBuilder[name];
      }
      if (!isFunction(func) || !isString(name)) {
        return this;
      }
      if (stateBuilder[name] && !stateBuilder.$delegates[name]) {
        stateBuilder.$delegates[name] = stateBuilder[name];
      }
      stateBuilder[name] = func;
      return this;
    }
    /**
   * @ngdoc function
   * @name ui.router.state.$stateProvider#state
   * @methodOf ui.router.state.$stateProvider
   *
   * @description
   * Registers a state configuration under a given state name. The stateConfig object
   * has the following acceptable properties.
   *
   * @param {string} name A unique state name, e.g. "home", "about", "contacts".
   * To create a parent/child state use a dot, e.g. "about.sales", "home.newest".
   * @param {object} stateConfig State configuration object.
   * @param {string|function=} stateConfig.template
   * <a id='template'></a>
   *   html template as a string or a function that returns
   *   an html template as a string which should be used by the uiView directives. This property 
   *   takes precedence over templateUrl.
   *   
   *   If `template` is a function, it will be called with the following parameters:
   *
   *   - {array.&lt;object&gt;} - state parameters extracted from the current $location.path() by
   *     applying the current state
   *
   * <pre>template:
   *   "<h1>inline template definition</h1>" +
   *   "<div ui-view></div>"</pre>
   * <pre>template: function(params) {
   *       return "<h1>generated template</h1>"; }</pre>
   * </div>
   *
   * @param {string|function=} stateConfig.templateUrl
   * <a id='templateUrl'></a>
   *
   *   path or function that returns a path to an html
   *   template that should be used by uiView.
   *   
   *   If `templateUrl` is a function, it will be called with the following parameters:
   *
   *   - {array.&lt;object&gt;} - state parameters extracted from the current $location.path() by 
   *     applying the current state
   *
   * <pre>templateUrl: "home.html"</pre>
   * <pre>templateUrl: function(params) {
   *     return myTemplates[params.pageId]; }</pre>
   *
   * @param {function=} stateConfig.templateProvider
   * <a id='templateProvider'></a>
   *    Provider function that returns HTML content string.
   * <pre> templateProvider:
   *       function(MyTemplateService, params) {
   *         return MyTemplateService.getTemplate(params.pageId);
   *       }</pre>
   *
   * @param {string|function=} stateConfig.controller
   * <a id='controller'></a>
   *
   *  Controller fn that should be associated with newly
   *   related scope or the name of a registered controller if passed as a string.
   *   Optionally, the ControllerAs may be declared here.
   * <pre>controller: "MyRegisteredController"</pre>
   * <pre>controller:
   *     "MyRegisteredController as fooCtrl"}</pre>
   * <pre>controller: function($scope, MyService) {
   *     $scope.data = MyService.getData(); }</pre>
   *
   * @param {function=} stateConfig.controllerProvider
   * <a id='controllerProvider'></a>
   *
   * Injectable provider function that returns the actual controller or string.
   * <pre>controllerProvider:
   *   function(MyResolveData) {
   *     if (MyResolveData.foo)
   *       return "FooCtrl"
   *     else if (MyResolveData.bar)
   *       return "BarCtrl";
   *     else return function($scope) {
   *       $scope.baz = "Qux";
   *     }
   *   }</pre>
   *
   * @param {string=} stateConfig.controllerAs
   * <a id='controllerAs'></a>
   * 
   * A controller alias name. If present the controller will be
   *   published to scope under the controllerAs name.
   * <pre>controllerAs: "myCtrl"</pre>
   *
   * @param {string|object=} stateConfig.parent
   * <a id='parent'></a>
   * Optionally specifies the parent state of this state.
   *
   * <pre>parent: 'parentState'</pre>
   * <pre>parent: parentState // JS variable</pre>
   *
   * @param {object=} stateConfig.resolve
   * <a id='resolve'></a>
   *
   * An optional map&lt;string, function&gt; of dependencies which
   *   should be injected into the controller. If any of these dependencies are promises, 
   *   the router will wait for them all to be resolved before the controller is instantiated.
   *   If all the promises are resolved successfully, the $stateChangeSuccess event is fired
   *   and the values of the resolved promises are injected into any controllers that reference them.
   *   If any  of the promises are rejected the $stateChangeError event is fired.
   *
   *   The map object is:
   *   
   *   - key - {string}: name of dependency to be injected into controller
   *   - factory - {string|function}: If string then it is alias for service. Otherwise if function, 
   *     it is injected and return value it treated as dependency. If result is a promise, it is 
   *     resolved before its value is injected into controller.
   *
   * <pre>resolve: {
   *     myResolve1:
   *       function($http, $stateParams) {
   *         return $http.get("/api/foos/"+stateParams.fooID);
   *       }
   *     }</pre>
   *
   * @param {string=} stateConfig.url
   * <a id='url'></a>
   *
   *   A url fragment with optional parameters. When a state is navigated or
   *   transitioned to, the `$stateParams` service will be populated with any 
   *   parameters that were passed.
   *
   *   (See {@link ui.router.util.type:UrlMatcher UrlMatcher} `UrlMatcher`} for
   *   more details on acceptable patterns )
   *
   * examples:
   * <pre>url: "/home"
   * url: "/users/:userid"
   * url: "/books/{bookid:[a-zA-Z_-]}"
   * url: "/books/{categoryid:int}"
   * url: "/books/{publishername:string}/{categoryid:int}"
   * url: "/messages?before&after"
   * url: "/messages?{before:date}&{after:date}"
   * url: "/messages/:mailboxid?{before:date}&{after:date}"
   * </pre>
   *
   * @param {object=} stateConfig.views
   * <a id='views'></a>
   * an optional map&lt;string, object&gt; which defined multiple views, or targets views
   * manually/explicitly.
   *
   * Examples:
   *
   * Targets three named `ui-view`s in the parent state's template
   * <pre>views: {
   *     header: {
   *       controller: "headerCtrl",
   *       templateUrl: "header.html"
   *     }, body: {
   *       controller: "bodyCtrl",
   *       templateUrl: "body.html"
   *     }, footer: {
   *       controller: "footCtrl",
   *       templateUrl: "footer.html"
   *     }
   *   }</pre>
   *
   * Targets named `ui-view="header"` from grandparent state 'top''s template, and named `ui-view="body" from parent state's template.
   * <pre>views: {
   *     'header@top': {
   *       controller: "msgHeaderCtrl",
   *       templateUrl: "msgHeader.html"
   *     }, 'body': {
   *       controller: "messagesCtrl",
   *       templateUrl: "messages.html"
   *     }
   *   }</pre>
   *
   * @param {boolean=} [stateConfig.abstract=false]
   * <a id='abstract'></a>
   * An abstract state will never be directly activated,
   *   but can provide inherited properties to its common children states.
   * <pre>abstract: true</pre>
   *
   * @param {function=} stateConfig.onEnter
   * <a id='onEnter'></a>
   *
   * Callback function for when a state is entered. Good way
   *   to trigger an action or dispatch an event, such as opening a dialog.
   * If minifying your scripts, make sure to explicitly annotate this function,
   * because it won't be automatically annotated by your build tools.
   *
   * <pre>onEnter: function(MyService, $stateParams) {
   *     MyService.foo($stateParams.myParam);
   * }</pre>
   *
   * @param {function=} stateConfig.onExit
   * <a id='onExit'></a>
   *
   * Callback function for when a state is exited. Good way to
   *   trigger an action or dispatch an event, such as opening a dialog.
   * If minifying your scripts, make sure to explicitly annotate this function,
   * because it won't be automatically annotated by your build tools.
   *
   * <pre>onExit: function(MyService, $stateParams) {
   *     MyService.cleanup($stateParams.myParam);
   * }</pre>
   *
   * @param {boolean=} [stateConfig.reloadOnSearch=true]
   * <a id='reloadOnSearch'></a>
   *
   * If `false`, will not retrigger the same state
   *   just because a search/query parameter has changed (via $location.search() or $location.hash()). 
   *   Useful for when you'd like to modify $location.search() without triggering a reload.
   * <pre>reloadOnSearch: false</pre>
   *
   * @param {object=} stateConfig.data
   * <a id='data'></a>
   *
   * Arbitrary data object, useful for custom configuration.  The parent state's `data` is
   *   prototypally inherited.  In other words, adding a data property to a state adds it to
   *   the entire subtree via prototypal inheritance.
   *
   * <pre>data: {
   *     requiredRole: 'foo'
   * } </pre>
   *
   * @param {object=} stateConfig.params
   * <a id='params'></a>
   *
   * A map which optionally configures parameters declared in the `url`, or
   *   defines additional non-url parameters.  For each parameter being
   *   configured, add a configuration object keyed to the name of the parameter.
   *
   *   Each parameter configuration object may contain the following properties:
   *
   *   - ** value ** - {object|function=}: specifies the default value for this
   *     parameter.  This implicitly sets this parameter as optional.
   *
   *     When UI-Router routes to a state and no value is
   *     specified for this parameter in the URL or transition, the
   *     default value will be used instead.  If `value` is a function,
   *     it will be injected and invoked, and the return value used.
   *
   *     *Note*: `undefined` is treated as "no default value" while `null`
   *     is treated as "the default value is `null`".
   *
   *     *Shorthand*: If you only need to configure the default value of the
   *     parameter, you may use a shorthand syntax.   In the **`params`**
   *     map, instead mapping the param name to a full parameter configuration
   *     object, simply set map it to the default parameter value, e.g.:
   *
   * <pre>// define a parameter's default value
   * params: {
   *     param1: { value: "defaultValue" }
   * }
   * // shorthand default values
   * params: {
   *     param1: "defaultValue",
   *     param2: "param2Default"
   * }</pre>
   *
   *   - ** array ** - {boolean=}: *(default: false)* If true, the param value will be
   *     treated as an array of values.  If you specified a Type, the value will be
   *     treated as an array of the specified Type.  Note: query parameter values
   *     default to a special `"auto"` mode.
   *
   *     For query parameters in `"auto"` mode, if multiple  values for a single parameter
   *     are present in the URL (e.g.: `/foo?bar=1&bar=2&bar=3`) then the values
   *     are mapped to an array (e.g.: `{ foo: [ '1', '2', '3' ] }`).  However, if
   *     only one value is present (e.g.: `/foo?bar=1`) then the value is treated as single
   *     value (e.g.: `{ foo: '1' }`).
   *
   * <pre>params: {
   *     param1: { array: true }
   * }</pre>
   *
   *   - ** squash ** - {bool|string=}: `squash` configures how a default parameter value is represented in the URL when
   *     the current parameter value is the same as the default value. If `squash` is not set, it uses the
   *     configured default squash policy.
   *     (See {@link ui.router.util.$urlMatcherFactory#methods_defaultSquashPolicy `defaultSquashPolicy()`})
   *
   *   There are three squash settings:
   *
   *     - false: The parameter's default value is not squashed.  It is encoded and included in the URL
   *     - true: The parameter's default value is omitted from the URL.  If the parameter is preceeded and followed
   *       by slashes in the state's `url` declaration, then one of those slashes are omitted.
   *       This can allow for cleaner looking URLs.
   *     - `"<arbitrary string>"`: The parameter's default value is replaced with an arbitrary placeholder of  your choice.
   *
   * <pre>params: {
   *     param1: {
   *       value: "defaultId",
   *       squash: true
   * } }
   * // squash "defaultValue" to "~"
   * params: {
   *     param1: {
   *       value: "defaultValue",
   *       squash: "~"
   * } }
   * </pre>
   *
   *
   * @example
   * <pre>
   * // Some state name examples
   *
   * // stateName can be a single top-level name (must be unique).
   * $stateProvider.state("home", {});
   *
   * // Or it can be a nested state name. This state is a child of the
   * // above "home" state.
   * $stateProvider.state("home.newest", {});
   *
   * // Nest states as deeply as needed.
   * $stateProvider.state("home.newest.abc.xyz.inception", {});
   *
   * // state() returns $stateProvider, so you can chain state declarations.
   * $stateProvider
   *   .state("home", {})
   *   .state("about", {})
   *   .state("contacts", {});
   * </pre>
   *
   */
    this.state = state;
    function state(name, definition) {
      /*jshint validthis: true */
      if (isObject(name))
        definition = name;
      else
        definition.name = name;
      registerState(definition);
      return this;
    }
    /**
   * @ngdoc object
   * @name ui.router.state.$state
   *
   * @requires $rootScope
   * @requires $q
   * @requires ui.router.state.$view
   * @requires $injector
   * @requires ui.router.util.$resolve
   * @requires ui.router.state.$stateParams
   * @requires ui.router.router.$urlRouter
   *
   * @property {object} params A param object, e.g. {sectionId: section.id)}, that 
   * you'd like to test against the current active state.
   * @property {object} current A reference to the state's config object. However 
   * you passed it in. Useful for accessing custom data.
   * @property {object} transition Currently pending transition. A promise that'll 
   * resolve or reject.
   *
   * @description
   * `$state` service is responsible for representing states as well as transitioning
   * between them. It also provides interfaces to ask for current state or even states
   * you're coming from.
   */
    this.$get = $get;
    $get.$inject = [
      '$rootScope',
      '$q',
      '$view',
      '$injector',
      '$resolve',
      '$stateParams',
      '$urlRouter',
      '$location',
      '$urlMatcherFactory'
    ];
    function $get($rootScope, $q, $view, $injector, $resolve, $stateParams, $urlRouter, $location, $urlMatcherFactory) {
      var TransitionSuperseded = $q.reject(new Error('transition superseded'));
      var TransitionPrevented = $q.reject(new Error('transition prevented'));
      var TransitionAborted = $q.reject(new Error('transition aborted'));
      var TransitionFailed = $q.reject(new Error('transition failed'));
      // Handles the case where a state which is the target of a transition is not found, and the user
      // can optionally retry or defer the transition
      function handleRedirect(redirect, state, params, options) {
        /**
       * @ngdoc event
       * @name ui.router.state.$state#$stateNotFound
       * @eventOf ui.router.state.$state
       * @eventType broadcast on root scope
       * @description
       * Fired when a requested state **cannot be found** using the provided state name during transition.
       * The event is broadcast allowing any handlers a single chance to deal with the error (usually by
       * lazy-loading the unfound state). A special `unfoundState` object is passed to the listener handler,
       * you can see its three properties in the example. You can use `event.preventDefault()` to abort the
       * transition and the promise returned from `go` will be rejected with a `'transition aborted'` value.
       *
       * @param {Object} event Event object.
       * @param {Object} unfoundState Unfound State information. Contains: `to, toParams, options` properties.
       * @param {State} fromState Current state object.
       * @param {Object} fromParams Current state params.
       *
       * @example
       *
       * <pre>
       * // somewhere, assume lazy.state has not been defined
       * $state.go("lazy.state", {a:1, b:2}, {inherit:false});
       *
       * // somewhere else
       * $scope.$on('$stateNotFound',
       * function(event, unfoundState, fromState, fromParams){
       *     console.log(unfoundState.to); // "lazy.state"
       *     console.log(unfoundState.toParams); // {a:1, b:2}
       *     console.log(unfoundState.options); // {inherit:false} + default options
       * })
       * </pre>
       */
        var evt = $rootScope.$broadcast('$stateNotFound', redirect, state, params);
        if (evt.defaultPrevented) {
          $urlRouter.update();
          return TransitionAborted;
        }
        if (!evt.retry) {
          return null;
        }
        // Allow the handler to return a promise to defer state lookup retry
        if (options.$retry) {
          $urlRouter.update();
          return TransitionFailed;
        }
        var retryTransition = $state.transition = $q.when(evt.retry);
        retryTransition.then(function () {
          if (retryTransition !== $state.transition)
            return TransitionSuperseded;
          redirect.options.$retry = true;
          return $state.transitionTo(redirect.to, redirect.toParams, redirect.options);
        }, function () {
          return TransitionAborted;
        });
        $urlRouter.update();
        return retryTransition;
      }
      root.locals = {
        resolve: null,
        globals: { $stateParams: {} }
      };
      $state = {
        params: {},
        current: root.self,
        $current: root,
        transition: null
      };
      /**
     * @ngdoc function
     * @name ui.router.state.$state#reload
     * @methodOf ui.router.state.$state
     *
     * @description
     * A method that force reloads the current state. All resolves are re-resolved,
     * controllers reinstantiated, and events re-fired.
     *
     * @example
     * <pre>
     * var app angular.module('app', ['ui.router']);
     *
     * app.controller('ctrl', function ($scope, $state) {
     *   $scope.reload = function(){
     *     $state.reload();
     *   }
     * });
     * </pre>
     *
     * `reload()` is just an alias for:
     * <pre>
     * $state.transitionTo($state.current, $stateParams, { 
     *   reload: true, inherit: false, notify: true
     * });
     * </pre>
     *
     * @param {string=|object=} state - A state name or a state object, which is the root of the resolves to be re-resolved.
     * @example
     * <pre>
     * //assuming app application consists of 3 states: 'contacts', 'contacts.detail', 'contacts.detail.item' 
     * //and current state is 'contacts.detail.item'
     * var app angular.module('app', ['ui.router']);
     *
     * app.controller('ctrl', function ($scope, $state) {
     *   $scope.reload = function(){
     *     //will reload 'contact.detail' and 'contact.detail.item' states
     *     $state.reload('contact.detail');
     *   }
     * });
     * </pre>
     *
     * `reload()` is just an alias for:
     * <pre>
     * $state.transitionTo($state.current, $stateParams, { 
     *   reload: true, inherit: false, notify: true
     * });
     * </pre>

     * @returns {promise} A promise representing the state of the new transition. See
     * {@link ui.router.state.$state#methods_go $state.go}.
     */
      $state.reload = function reload(state) {
        return $state.transitionTo($state.current, $stateParams, {
          reload: state || true,
          inherit: false,
          notify: true
        });
      };
      /**
     * @ngdoc function
     * @name ui.router.state.$state#go
     * @methodOf ui.router.state.$state
     *
     * @description
     * Convenience method for transitioning to a new state. `$state.go` calls 
     * `$state.transitionTo` internally but automatically sets options to 
     * `{ location: true, inherit: true, relative: $state.$current, notify: true }`. 
     * This allows you to easily use an absolute or relative to path and specify 
     * only the parameters you'd like to update (while letting unspecified parameters 
     * inherit from the currently active ancestor states).
     *
     * @example
     * <pre>
     * var app = angular.module('app', ['ui.router']);
     *
     * app.controller('ctrl', function ($scope, $state) {
     *   $scope.changeState = function () {
     *     $state.go('contact.detail');
     *   };
     * });
     * </pre>
     * <img src='../ngdoc_assets/StateGoExamples.png'/>
     *
     * @param {string} to Absolute state name or relative state path. Some examples:
     *
     * - `$state.go('contact.detail')` - will go to the `contact.detail` state
     * - `$state.go('^')` - will go to a parent state
     * - `$state.go('^.sibling')` - will go to a sibling state
     * - `$state.go('.child.grandchild')` - will go to grandchild state
     *
     * @param {object=} params A map of the parameters that will be sent to the state, 
     * will populate $stateParams. Any parameters that are not specified will be inherited from currently 
     * defined parameters. Only parameters specified in the state definition can be overridden, new 
     * parameters will be ignored. This allows, for example, going to a sibling state that shares parameters
     * specified in a parent state. Parameter inheritance only works between common ancestor states, I.e.
     * transitioning to a sibling will get you the parameters for all parents, transitioning to a child
     * will get you all current parameters, etc.
     * @param {object=} options Options object. The options are:
     *
     * - **`location`** - {boolean=true|string=} - If `true` will update the url in the location bar, if `false`
     *    will not. If string, must be `"replace"`, which will update url and also replace last history record.
     * - **`inherit`** - {boolean=true}, If `true` will inherit url parameters from current url.
     * - **`relative`** - {object=$state.$current}, When transitioning with relative path (e.g '^'), 
     *    defines which state to be relative from.
     * - **`notify`** - {boolean=true}, If `true` will broadcast $stateChangeStart and $stateChangeSuccess events.
     * - **`reload`** (v0.2.5) - {boolean=false|string|object}, If `true` will force transition even if no state or params
     *    have changed.  It will reload the resolves and views of the current state and parent states.
     *    If `reload` is a string (or state object), the state object is fetched (by name, or object reference); and \
     *    the transition reloads the resolves and views for that matched state, and all its children states.
     *
     * @returns {promise} A promise representing the state of the new transition.
     *
     * Possible success values:
     *
     * - $state.current
     *
     * <br/>Possible rejection values:
     *
     * - 'transition superseded' - when a newer transition has been started after this one
     * - 'transition prevented' - when `event.preventDefault()` has been called in a `$stateChangeStart` listener
     * - 'transition aborted' - when `event.preventDefault()` has been called in a `$stateNotFound` listener or
     *   when a `$stateNotFound` `event.retry` promise errors.
     * - 'transition failed' - when a state has been unsuccessfully found after 2 tries.
     * - *resolve error* - when an error has occurred with a `resolve`
     *
     */
      $state.go = function go(to, params, options) {
        return $state.transitionTo(to, params, extend({
          inherit: true,
          relative: $state.$current
        }, options));
      };
      /**
     * @ngdoc function
     * @name ui.router.state.$state#transitionTo
     * @methodOf ui.router.state.$state
     *
     * @description
     * Low-level method for transitioning to a new state. {@link ui.router.state.$state#methods_go $state.go}
     * uses `transitionTo` internally. `$state.go` is recommended in most situations.
     *
     * @example
     * <pre>
     * var app = angular.module('app', ['ui.router']);
     *
     * app.controller('ctrl', function ($scope, $state) {
     *   $scope.changeState = function () {
     *     $state.transitionTo('contact.detail');
     *   };
     * });
     * </pre>
     *
     * @param {string} to State name.
     * @param {object=} toParams A map of the parameters that will be sent to the state,
     * will populate $stateParams.
     * @param {object=} options Options object. The options are:
     *
     * - **`location`** - {boolean=true|string=} - If `true` will update the url in the location bar, if `false`
     *    will not. If string, must be `"replace"`, which will update url and also replace last history record.
     * - **`inherit`** - {boolean=false}, If `true` will inherit url parameters from current url.
     * - **`relative`** - {object=}, When transitioning with relative path (e.g '^'), 
     *    defines which state to be relative from.
     * - **`notify`** - {boolean=true}, If `true` will broadcast $stateChangeStart and $stateChangeSuccess events.
     * - **`reload`** (v0.2.5) - {boolean=false|string=|object=}, If `true` will force transition even if the state or params 
     *    have not changed, aka a reload of the same state. It differs from reloadOnSearch because you'd
     *    use this when you want to force a reload when *everything* is the same, including search params.
     *    if String, then will reload the state with the name given in reload, and any children.
     *    if Object, then a stateObj is expected, will reload the state found in stateObj, and any children.
     *
     * @returns {promise} A promise representing the state of the new transition. See
     * {@link ui.router.state.$state#methods_go $state.go}.
     */
      $state.transitionTo = function transitionTo(to, toParams, options) {
        toParams = toParams || {};
        options = extend({
          location: true,
          inherit: false,
          relative: null,
          notify: true,
          reload: false,
          $retry: false
        }, options || {});
        var from = $state.$current, fromParams = $state.params, fromPath = from.path;
        var evt, toState = findState(to, options.relative);
        // Store the hash param for later (since it will be stripped out by various methods)
        var hash = toParams['#'];
        if (!isDefined(toState)) {
          var redirect = {
              to: to,
              toParams: toParams,
              options: options
            };
          var redirectResult = handleRedirect(redirect, from.self, fromParams, options);
          if (redirectResult) {
            return redirectResult;
          }
          // Always retry once if the $stateNotFound was not prevented
          // (handles either redirect changed or state lazy-definition)
          to = redirect.to;
          toParams = redirect.toParams;
          options = redirect.options;
          toState = findState(to, options.relative);
          if (!isDefined(toState)) {
            if (!options.relative)
              throw new Error('No such state \'' + to + '\'');
            throw new Error('Could not resolve \'' + to + '\' from state \'' + options.relative + '\'');
          }
        }
        if (toState[abstractKey])
          throw new Error('Cannot transition to abstract state \'' + to + '\'');
        if (options.inherit)
          toParams = inheritParams($stateParams, toParams || {}, $state.$current, toState);
        if (!toState.params.$$validates(toParams))
          return TransitionFailed;
        toParams = toState.params.$$values(toParams);
        to = toState;
        var toPath = to.path;
        // Starting from the root of the path, keep all levels that haven't changed
        var keep = 0, state = toPath[keep], locals = root.locals, toLocals = [];
        if (!options.reload) {
          while (state && state === fromPath[keep] && state.ownParams.$$equals(toParams, fromParams)) {
            locals = toLocals[keep] = state.locals;
            keep++;
            state = toPath[keep];
          }
        } else if (isString(options.reload) || isObject(options.reload)) {
          if (isObject(options.reload) && !options.reload.name) {
            throw new Error('Invalid reload state object');
          }
          var reloadState = options.reload === true ? fromPath[0] : findState(options.reload);
          if (options.reload && !reloadState) {
            throw new Error('No such reload state \'' + (isString(options.reload) ? options.reload : options.reload.name) + '\'');
          }
          while (state && state === fromPath[keep] && state !== reloadState) {
            locals = toLocals[keep] = state.locals;
            keep++;
            state = toPath[keep];
          }
        }
        // If we're going to the same state and all locals are kept, we've got nothing to do.
        // But clear 'transition', as we still want to cancel any other pending transitions.
        // TODO: We may not want to bump 'transition' if we're called from a location change
        // that we've initiated ourselves, because we might accidentally abort a legitimate
        // transition initiated from code?
        if (shouldSkipReload(to, toParams, from, fromParams, locals, options)) {
          if (hash)
            toParams['#'] = hash;
          $state.params = toParams;
          copy($state.params, $stateParams);
          copy(filterByKeys(to.params.$$keys(), $stateParams), to.locals.globals.$stateParams);
          if (options.location && to.navigable && to.navigable.url) {
            $urlRouter.push(to.navigable.url, toParams, {
              $$avoidResync: true,
              replace: options.location === 'replace'
            });
            $urlRouter.update(true);
          }
          $state.transition = null;
          return $q.when($state.current);
        }
        // Filter parameters before we pass them to event handlers etc.
        toParams = filterByKeys(to.params.$$keys(), toParams || {});
        // Re-add the saved hash before we start returning things or broadcasting $stateChangeStart
        if (hash)
          toParams['#'] = hash;
        // Broadcast start event and cancel the transition if requested
        if (options.notify) {
          /**
         * @ngdoc event
         * @name ui.router.state.$state#$stateChangeStart
         * @eventOf ui.router.state.$state
         * @eventType broadcast on root scope
         * @description
         * Fired when the state transition **begins**. You can use `event.preventDefault()`
         * to prevent the transition from happening and then the transition promise will be
         * rejected with a `'transition prevented'` value.
         *
         * @param {Object} event Event object.
         * @param {State} toState The state being transitioned to.
         * @param {Object} toParams The params supplied to the `toState`.
         * @param {State} fromState The current state, pre-transition.
         * @param {Object} fromParams The params supplied to the `fromState`.
         *
         * @example
         *
         * <pre>
         * $rootScope.$on('$stateChangeStart',
         * function(event, toState, toParams, fromState, fromParams){
         *     event.preventDefault();
         *     // transitionTo() promise will be rejected with
         *     // a 'transition prevented' error
         * })
         * </pre>
         */
          if ($rootScope.$broadcast('$stateChangeStart', to.self, toParams, from.self, fromParams, options).defaultPrevented) {
            $rootScope.$broadcast('$stateChangeCancel', to.self, toParams, from.self, fromParams);
            //Don't update and resync url if there's been a new transition started. see issue #2238, #600
            if ($state.transition == null)
              $urlRouter.update();
            return TransitionPrevented;
          }
        }
        // Resolve locals for the remaining states, but don't update any global state just
        // yet -- if anything fails to resolve the current state needs to remain untouched.
        // We also set up an inheritance chain for the locals here. This allows the view directive
        // to quickly look up the correct definition for each view in the current state. Even
        // though we create the locals object itself outside resolveState(), it is initially
        // empty and gets filled asynchronously. We need to keep track of the promise for the
        // (fully resolved) current locals, and pass this down the chain.
        var resolved = $q.when(locals);
        for (var l = keep; l < toPath.length; l++, state = toPath[l]) {
          locals = toLocals[l] = inherit(locals);
          resolved = resolveState(state, toParams, state === to, resolved, locals, options);
        }
        // Once everything is resolved, we are ready to perform the actual transition
        // and return a promise for the new state. We also keep track of what the
        // current promise is, so that we can detect overlapping transitions and
        // keep only the outcome of the last transition.
        var transition = $state.transition = resolved.then(function () {
            var l, entering, exiting;
            if ($state.transition !== transition)
              return TransitionSuperseded;
            // Exit 'from' states not kept
            for (l = fromPath.length - 1; l >= keep; l--) {
              exiting = fromPath[l];
              if (exiting.self.onExit) {
                $injector.invoke(exiting.self.onExit, exiting.self, exiting.locals.globals);
              }
              exiting.locals = null;
            }
            // Enter 'to' states not kept
            for (l = keep; l < toPath.length; l++) {
              entering = toPath[l];
              entering.locals = toLocals[l];
              if (entering.self.onEnter) {
                $injector.invoke(entering.self.onEnter, entering.self, entering.locals.globals);
              }
            }
            // Run it again, to catch any transitions in callbacks
            if ($state.transition !== transition)
              return TransitionSuperseded;
            // Update globals in $state
            $state.$current = to;
            $state.current = to.self;
            $state.params = toParams;
            copy($state.params, $stateParams);
            $state.transition = null;
            if (options.location && to.navigable) {
              $urlRouter.push(to.navigable.url, to.navigable.locals.globals.$stateParams, {
                $$avoidResync: true,
                replace: options.location === 'replace'
              });
            }
            if (options.notify) {
              /**
         * @ngdoc event
         * @name ui.router.state.$state#$stateChangeSuccess
         * @eventOf ui.router.state.$state
         * @eventType broadcast on root scope
         * @description
         * Fired once the state transition is **complete**.
         *
         * @param {Object} event Event object.
         * @param {State} toState The state being transitioned to.
         * @param {Object} toParams The params supplied to the `toState`.
         * @param {State} fromState The current state, pre-transition.
         * @param {Object} fromParams The params supplied to the `fromState`.
         */
              $rootScope.$broadcast('$stateChangeSuccess', to.self, toParams, from.self, fromParams);
            }
            $urlRouter.update(true);
            return $state.current;
          }, function (error) {
            if ($state.transition !== transition)
              return TransitionSuperseded;
            $state.transition = null;
            /**
         * @ngdoc event
         * @name ui.router.state.$state#$stateChangeError
         * @eventOf ui.router.state.$state
         * @eventType broadcast on root scope
         * @description
         * Fired when an **error occurs** during transition. It's important to note that if you
         * have any errors in your resolve functions (javascript errors, non-existent services, etc)
         * they will not throw traditionally. You must listen for this $stateChangeError event to
         * catch **ALL** errors.
         *
         * @param {Object} event Event object.
         * @param {State} toState The state being transitioned to.
         * @param {Object} toParams The params supplied to the `toState`.
         * @param {State} fromState The current state, pre-transition.
         * @param {Object} fromParams The params supplied to the `fromState`.
         * @param {Error} error The resolve error object.
         */
            evt = $rootScope.$broadcast('$stateChangeError', to.self, toParams, from.self, fromParams, error);
            if (!evt.defaultPrevented) {
              $urlRouter.update();
            }
            return $q.reject(error);
          });
        return transition;
      };
      /**
     * @ngdoc function
     * @name ui.router.state.$state#is
     * @methodOf ui.router.state.$state
     *
     * @description
     * Similar to {@link ui.router.state.$state#methods_includes $state.includes},
     * but only checks for the full state name. If params is supplied then it will be
     * tested for strict equality against the current active params object, so all params
     * must match with none missing and no extras.
     *
     * @example
     * <pre>
     * $state.$current.name = 'contacts.details.item';
     *
     * // absolute name
     * $state.is('contact.details.item'); // returns true
     * $state.is(contactDetailItemStateObject); // returns true
     *
     * // relative name (. and ^), typically from a template
     * // E.g. from the 'contacts.details' template
     * <div ng-class="{highlighted: $state.is('.item')}">Item</div>
     * </pre>
     *
     * @param {string|object} stateOrName The state name (absolute or relative) or state object you'd like to check.
     * @param {object=} params A param object, e.g. `{sectionId: section.id}`, that you'd like
     * to test against the current active state.
     * @param {object=} options An options object.  The options are:
     *
     * - **`relative`** - {string|object} -  If `stateOrName` is a relative state name and `options.relative` is set, .is will
     * test relative to `options.relative` state (or name).
     *
     * @returns {boolean} Returns true if it is the state.
     */
      $state.is = function is(stateOrName, params, options) {
        options = extend({ relative: $state.$current }, options || {});
        var state = findState(stateOrName, options.relative);
        if (!isDefined(state)) {
          return undefined;
        }
        if ($state.$current !== state) {
          return false;
        }
        return params ? equalForKeys(state.params.$$values(params), $stateParams) : true;
      };
      /**
     * @ngdoc function
     * @name ui.router.state.$state#includes
     * @methodOf ui.router.state.$state
     *
     * @description
     * A method to determine if the current active state is equal to or is the child of the
     * state stateName. If any params are passed then they will be tested for a match as well.
     * Not all the parameters need to be passed, just the ones you'd like to test for equality.
     *
     * @example
     * Partial and relative names
     * <pre>
     * $state.$current.name = 'contacts.details.item';
     *
     * // Using partial names
     * $state.includes("contacts"); // returns true
     * $state.includes("contacts.details"); // returns true
     * $state.includes("contacts.details.item"); // returns true
     * $state.includes("contacts.list"); // returns false
     * $state.includes("about"); // returns false
     *
     * // Using relative names (. and ^), typically from a template
     * // E.g. from the 'contacts.details' template
     * <div ng-class="{highlighted: $state.includes('.item')}">Item</div>
     * </pre>
     *
     * Basic globbing patterns
     * <pre>
     * $state.$current.name = 'contacts.details.item.url';
     *
     * $state.includes("*.details.*.*"); // returns true
     * $state.includes("*.details.**"); // returns true
     * $state.includes("**.item.**"); // returns true
     * $state.includes("*.details.item.url"); // returns true
     * $state.includes("*.details.*.url"); // returns true
     * $state.includes("*.details.*"); // returns false
     * $state.includes("item.**"); // returns false
     * </pre>
     *
     * @param {string} stateOrName A partial name, relative name, or glob pattern
     * to be searched for within the current state name.
     * @param {object=} params A param object, e.g. `{sectionId: section.id}`,
     * that you'd like to test against the current active state.
     * @param {object=} options An options object.  The options are:
     *
     * - **`relative`** - {string|object=} -  If `stateOrName` is a relative state reference and `options.relative` is set,
     * .includes will test relative to `options.relative` state (or name).
     *
     * @returns {boolean} Returns true if it does include the state
     */
      $state.includes = function includes(stateOrName, params, options) {
        options = extend({ relative: $state.$current }, options || {});
        if (isString(stateOrName) && isGlob(stateOrName)) {
          if (!doesStateMatchGlob(stateOrName)) {
            return false;
          }
          stateOrName = $state.$current.name;
        }
        var state = findState(stateOrName, options.relative);
        if (!isDefined(state)) {
          return undefined;
        }
        if (!isDefined($state.$current.includes[state.name])) {
          return false;
        }
        return params ? equalForKeys(state.params.$$values(params), $stateParams, objectKeys(params)) : true;
      };
      /**
     * @ngdoc function
     * @name ui.router.state.$state#href
     * @methodOf ui.router.state.$state
     *
     * @description
     * A url generation method that returns the compiled url for the given state populated with the given params.
     *
     * @example
     * <pre>
     * expect($state.href("about.person", { person: "bob" })).toEqual("/about/bob");
     * </pre>
     *
     * @param {string|object} stateOrName The state name or state object you'd like to generate a url from.
     * @param {object=} params An object of parameter values to fill the state's required parameters.
     * @param {object=} options Options object. The options are:
     *
     * - **`lossy`** - {boolean=true} -  If true, and if there is no url associated with the state provided in the
     *    first parameter, then the constructed href url will be built from the first navigable ancestor (aka
     *    ancestor with a valid url).
     * - **`inherit`** - {boolean=true}, If `true` will inherit url parameters from current url.
     * - **`relative`** - {object=$state.$current}, When transitioning with relative path (e.g '^'), 
     *    defines which state to be relative from.
     * - **`absolute`** - {boolean=false},  If true will generate an absolute url, e.g. "http://www.example.com/fullurl".
     * 
     * @returns {string} compiled state url
     */
      $state.href = function href(stateOrName, params, options) {
        options = extend({
          lossy: true,
          inherit: true,
          absolute: false,
          relative: $state.$current
        }, options || {});
        var state = findState(stateOrName, options.relative);
        if (!isDefined(state))
          return null;
        if (options.inherit)
          params = inheritParams($stateParams, params || {}, $state.$current, state);
        var nav = state && options.lossy ? state.navigable : state;
        if (!nav || nav.url === undefined || nav.url === null) {
          return null;
        }
        return $urlRouter.href(nav.url, filterByKeys(state.params.$$keys().concat('#'), params || {}), { absolute: options.absolute });
      };
      /**
     * @ngdoc function
     * @name ui.router.state.$state#get
     * @methodOf ui.router.state.$state
     *
     * @description
     * Returns the state configuration object for any specific state or all states.
     *
     * @param {string|object=} stateOrName (absolute or relative) If provided, will only get the config for
     * the requested state. If not provided, returns an array of ALL state configs.
     * @param {string|object=} context When stateOrName is a relative state reference, the state will be retrieved relative to context.
     * @returns {Object|Array} State configuration object or array of all objects.
     */
      $state.get = function (stateOrName, context) {
        if (arguments.length === 0)
          return map(objectKeys(states), function (name) {
            return states[name].self;
          });
        var state = findState(stateOrName, context || $state.$current);
        return state && state.self ? state.self : null;
      };
      function resolveState(state, params, paramsAreFiltered, inherited, dst, options) {
        // Make a restricted $stateParams with only the parameters that apply to this state if
        // necessary. In addition to being available to the controller and onEnter/onExit callbacks,
        // we also need $stateParams to be available for any $injector calls we make during the
        // dependency resolution process.
        var $stateParams = paramsAreFiltered ? params : filterByKeys(state.params.$$keys(), params);
        var locals = { $stateParams: $stateParams };
        // Resolve 'global' dependencies for the state, i.e. those not specific to a view.
        // We're also including $stateParams in this; that way the parameters are restricted
        // to the set that should be visible to the state, and are independent of when we update
        // the global $state and $stateParams values.
        dst.resolve = $resolve.resolve(state.resolve, locals, dst.resolve, state);
        var promises = [dst.resolve.then(function (globals) {
              dst.globals = globals;
            })];
        if (inherited)
          promises.push(inherited);
        function resolveViews() {
          var viewsPromises = [];
          // Resolve template and dependencies for all views.
          forEach(state.views, function (view, name) {
            var injectables = view.resolve && view.resolve !== state.resolve ? view.resolve : {};
            injectables.$template = [function () {
                return $view.load(name, {
                  view: view,
                  locals: dst.globals,
                  params: $stateParams,
                  notify: options.notify
                }) || '';
              }];
            viewsPromises.push($resolve.resolve(injectables, dst.globals, dst.resolve, state).then(function (result) {
              // References to the controller (only instantiated at link time)
              if (isFunction(view.controllerProvider) || isArray(view.controllerProvider)) {
                var injectLocals = angular.extend({}, injectables, dst.globals);
                result.$$controller = $injector.invoke(view.controllerProvider, null, injectLocals);
              } else {
                result.$$controller = view.controller;
              }
              // Provide access to the state itself for internal use
              result.$$state = state;
              result.$$controllerAs = view.controllerAs;
              dst[name] = result;
            }));
          });
          return $q.all(viewsPromises).then(function () {
            return dst.globals;
          });
        }
        // Wait for all the promises and then return the activation object
        return $q.all(promises).then(resolveViews).then(function (values) {
          return dst;
        });
      }
      return $state;
    }
    function shouldSkipReload(to, toParams, from, fromParams, locals, options) {
      // Return true if there are no differences in non-search (path/object) params, false if there are differences
      function nonSearchParamsEqual(fromAndToState, fromParams, toParams) {
        // Identify whether all the parameters that differ between `fromParams` and `toParams` were search params.
        function notSearchParam(key) {
          return fromAndToState.params[key].location != 'search';
        }
        var nonQueryParamKeys = fromAndToState.params.$$keys().filter(notSearchParam);
        var nonQueryParams = pick.apply({}, [fromAndToState.params].concat(nonQueryParamKeys));
        var nonQueryParamSet = new $$UMFP.ParamSet(nonQueryParams);
        return nonQueryParamSet.$$equals(fromParams, toParams);
      }
      // If reload was not explicitly requested
      // and we're transitioning to the same state we're already in
      // and    the locals didn't change
      //     or they changed in a way that doesn't merit reloading
      //        (reloadOnParams:false, or reloadOnSearch.false and only search params changed)
      // Then return true.
      if (!options.reload && to === from && (locals === from.locals || to.self.reloadOnSearch === false && nonSearchParamsEqual(from, fromParams, toParams))) {
        return true;
      }
    }
  }
  angular.module('ui.router.state').factory('$stateParams', function () {
    return {};
  }).provider('$state', $StateProvider);
  $ViewProvider.$inject = [];
  function $ViewProvider() {
    this.$get = $get;
    /**
   * @ngdoc object
   * @name ui.router.state.$view
   *
   * @requires ui.router.util.$templateFactory
   * @requires $rootScope
   *
   * @description
   *
   */
    $get.$inject = [
      '$rootScope',
      '$templateFactory'
    ];
    function $get($rootScope, $templateFactory) {
      return {
        load: function load(name, options) {
          var result, defaults = {
              template: null,
              controller: null,
              view: null,
              locals: null,
              notify: true,
              async: true,
              params: {}
            };
          options = extend(defaults, options);
          if (options.view) {
            result = $templateFactory.fromConfig(options.view, options.params, options.locals);
          }
          return result;
        }
      };
    }
  }
  angular.module('ui.router.state').provider('$view', $ViewProvider);
  /**
 * @ngdoc object
 * @name ui.router.state.$uiViewScrollProvider
 *
 * @description
 * Provider that returns the {@link ui.router.state.$uiViewScroll} service function.
 */
  function $ViewScrollProvider() {
    var useAnchorScroll = false;
    /**
   * @ngdoc function
   * @name ui.router.state.$uiViewScrollProvider#useAnchorScroll
   * @methodOf ui.router.state.$uiViewScrollProvider
   *
   * @description
   * Reverts back to using the core [`$anchorScroll`](http://docs.angularjs.org/api/ng.$anchorScroll) service for
   * scrolling based on the url anchor.
   */
    this.useAnchorScroll = function () {
      useAnchorScroll = true;
    };
    /**
   * @ngdoc object
   * @name ui.router.state.$uiViewScroll
   *
   * @requires $anchorScroll
   * @requires $timeout
   *
   * @description
   * When called with a jqLite element, it scrolls the element into view (after a
   * `$timeout` so the DOM has time to refresh).
   *
   * If you prefer to rely on `$anchorScroll` to scroll the view to the anchor,
   * this can be enabled by calling {@link ui.router.state.$uiViewScrollProvider#methods_useAnchorScroll `$uiViewScrollProvider.useAnchorScroll()`}.
   */
    this.$get = [
      '$anchorScroll',
      '$timeout',
      function ($anchorScroll, $timeout) {
        if (useAnchorScroll) {
          return $anchorScroll;
        }
        return function ($element) {
          return $timeout(function () {
            $element[0].scrollIntoView();
          }, 0, false);
        };
      }
    ];
  }
  angular.module('ui.router.state').provider('$uiViewScroll', $ViewScrollProvider);
  var ngMajorVer = angular.version.major;
  var ngMinorVer = angular.version.minor;
  /**
 * @ngdoc directive
 * @name ui.router.state.directive:ui-view
 *
 * @requires ui.router.state.$state
 * @requires $compile
 * @requires $controller
 * @requires $injector
 * @requires ui.router.state.$uiViewScroll
 * @requires $document
 *
 * @restrict ECA
 *
 * @description
 * The ui-view directive tells $state where to place your templates.
 *
 * @param {string=} name A view name. The name should be unique amongst the other views in the
 * same state. You can have views of the same name that live in different states.
 *
 * @param {string=} autoscroll It allows you to set the scroll behavior of the browser window
 * when a view is populated. By default, $anchorScroll is overridden by ui-router's custom scroll
 * service, {@link ui.router.state.$uiViewScroll}. This custom service let's you
 * scroll ui-view elements into view when they are populated during a state activation.
 *
 * @param {string=} noanimation If truthy, the non-animated renderer will be selected (no animations
 * will be applied to the ui-view)
 *
 * *Note: To revert back to old [`$anchorScroll`](http://docs.angularjs.org/api/ng.$anchorScroll)
 * functionality, call `$uiViewScrollProvider.useAnchorScroll()`.*
 *
 * @param {string=} onload Expression to evaluate whenever the view updates.
 * 
 * @example
 * A view can be unnamed or named. 
 * <pre>
 * <!-- Unnamed -->
 * <div ui-view></div> 
 * 
 * <!-- Named -->
 * <div ui-view="viewName"></div>
 * </pre>
 *
 * You can only have one unnamed view within any template (or root html). If you are only using a 
 * single view and it is unnamed then you can populate it like so:
 * <pre>
 * <div ui-view></div> 
 * $stateProvider.state("home", {
 *   template: "<h1>HELLO!</h1>"
 * })
 * </pre>
 * 
 * The above is a convenient shortcut equivalent to specifying your view explicitly with the {@link ui.router.state.$stateProvider#views `views`}
 * config property, by name, in this case an empty name:
 * <pre>
 * $stateProvider.state("home", {
 *   views: {
 *     "": {
 *       template: "<h1>HELLO!</h1>"
 *     }
 *   }    
 * })
 * </pre>
 * 
 * But typically you'll only use the views property if you name your view or have more than one view 
 * in the same template. There's not really a compelling reason to name a view if its the only one, 
 * but you could if you wanted, like so:
 * <pre>
 * <div ui-view="main"></div>
 * </pre> 
 * <pre>
 * $stateProvider.state("home", {
 *   views: {
 *     "main": {
 *       template: "<h1>HELLO!</h1>"
 *     }
 *   }    
 * })
 * </pre>
 * 
 * Really though, you'll use views to set up multiple views:
 * <pre>
 * <div ui-view></div>
 * <div ui-view="chart"></div> 
 * <div ui-view="data"></div> 
 * </pre>
 * 
 * <pre>
 * $stateProvider.state("home", {
 *   views: {
 *     "": {
 *       template: "<h1>HELLO!</h1>"
 *     },
 *     "chart": {
 *       template: "<chart_thing/>"
 *     },
 *     "data": {
 *       template: "<data_thing/>"
 *     }
 *   }    
 * })
 * </pre>
 *
 * Examples for `autoscroll`:
 *
 * <pre>
 * <!-- If autoscroll present with no expression,
 *      then scroll ui-view into view -->
 * <ui-view autoscroll/>
 *
 * <!-- If autoscroll present with valid expression,
 *      then scroll ui-view into view if expression evaluates to true -->
 * <ui-view autoscroll='true'/>
 * <ui-view autoscroll='false'/>
 * <ui-view autoscroll='scopeVariable'/>
 * </pre>
 */
  $ViewDirective.$inject = [
    '$state',
    '$injector',
    '$uiViewScroll',
    '$interpolate'
  ];
  function $ViewDirective($state, $injector, $uiViewScroll, $interpolate) {
    function getService() {
      return $injector.has ? function (service) {
        return $injector.has(service) ? $injector.get(service) : null;
      } : function (service) {
        try {
          return $injector.get(service);
        } catch (e) {
          return null;
        }
      };
    }
    var service = getService(), $animator = service('$animator'), $animate = service('$animate');
    // Returns a set of DOM manipulation functions based on which Angular version
    // it should use
    function getRenderer(attrs, scope) {
      var statics = {
          enter: function (element, target, cb) {
            target.after(element);
            cb();
          },
          leave: function (element, cb) {
            element.remove();
            cb();
          }
        };
      if (!!attrs.noanimation)
        return statics;
      function animEnabled(element) {
        if (ngMajorVer === 1 && ngMinorVer >= 4)
          return !!$animate.enabled(element);
        if (ngMajorVer === 1 && ngMinorVer >= 2)
          return !!$animate.enabled();
        return !!$animator;
      }
      // ng 1.2+
      if ($animate) {
        return {
          enter: function (element, target, cb) {
            if (!animEnabled(element)) {
              statics.enter(element, target, cb);
            } else if (angular.version.minor > 2) {
              $animate.enter(element, null, target).then(cb);
            } else {
              $animate.enter(element, null, target, cb);
            }
          },
          leave: function (element, cb) {
            if (!animEnabled(element)) {
              statics.leave(element, cb);
            } else if (angular.version.minor > 2) {
              $animate.leave(element).then(cb);
            } else {
              $animate.leave(element, cb);
            }
          }
        };
      }
      // ng 1.1.5
      if ($animator) {
        var animate = $animator && $animator(scope, attrs);
        return {
          enter: function (element, target, cb) {
            animate.enter(element, null, target);
            cb();
          },
          leave: function (element, cb) {
            animate.leave(element);
            cb();
          }
        };
      }
      return statics;
    }
    var directive = {
        restrict: 'ECA',
        terminal: true,
        priority: 400,
        transclude: 'element',
        compile: function (tElement, tAttrs, $transclude) {
          return function (scope, $element, attrs) {
            var previousEl, currentEl, currentScope, latestLocals, onloadExp = attrs.onload || '', autoScrollExp = attrs.autoscroll, renderer = getRenderer(attrs, scope);
            scope.$on('$stateChangeSuccess', function () {
              updateView(false);
            });
            updateView(true);
            function cleanupLastView() {
              var _previousEl = previousEl;
              var _currentScope = currentScope;
              if (_currentScope) {
                _currentScope._willBeDestroyed = true;
              }
              function cleanOld() {
                if (_previousEl) {
                  _previousEl.remove();
                }
                if (_currentScope) {
                  _currentScope.$destroy();
                }
              }
              if (currentEl) {
                renderer.leave(currentEl, function () {
                  cleanOld();
                  previousEl = null;
                });
                previousEl = currentEl;
              } else {
                cleanOld();
                previousEl = null;
              }
              currentEl = null;
              currentScope = null;
            }
            function updateView(firstTime) {
              var newScope, name = getUiViewName(scope, attrs, $element, $interpolate), previousLocals = name && $state.$current && $state.$current.locals[name];
              if (!firstTime && previousLocals === latestLocals || scope._willBeDestroyed)
                return;
              // nothing to do
              newScope = scope.$new();
              latestLocals = $state.$current.locals[name];
              /**
           * @ngdoc event
           * @name ui.router.state.directive:ui-view#$viewContentLoading
           * @eventOf ui.router.state.directive:ui-view
           * @eventType emits on ui-view directive scope
           * @description
           *
           * Fired once the view **begins loading**, *before* the DOM is rendered.
           *
           * @param {Object} event Event object.
           * @param {string} viewName Name of the view.
           */
              newScope.$emit('$viewContentLoading', name);
              var clone = $transclude(newScope, function (clone) {
                  renderer.enter(clone, $element, function onUiViewEnter() {
                    if (currentScope) {
                      currentScope.$emit('$viewContentAnimationEnded');
                    }
                    if (angular.isDefined(autoScrollExp) && !autoScrollExp || scope.$eval(autoScrollExp)) {
                      $uiViewScroll(clone);
                    }
                  });
                  cleanupLastView();
                });
              currentEl = clone;
              currentScope = newScope;
              /**
           * @ngdoc event
           * @name ui.router.state.directive:ui-view#$viewContentLoaded
           * @eventOf ui.router.state.directive:ui-view
           * @eventType emits on ui-view directive scope
           * @description
           * Fired once the view is **loaded**, *after* the DOM is rendered.
           *
           * @param {Object} event Event object.
           * @param {string} viewName Name of the view.
           */
              currentScope.$emit('$viewContentLoaded', name);
              currentScope.$eval(onloadExp);
            }
          };
        }
      };
    return directive;
  }
  $ViewDirectiveFill.$inject = [
    '$compile',
    '$controller',
    '$state',
    '$interpolate'
  ];
  function $ViewDirectiveFill($compile, $controller, $state, $interpolate) {
    return {
      restrict: 'ECA',
      priority: -400,
      compile: function (tElement) {
        var initial = tElement.html();
        return function (scope, $element, attrs) {
          var current = $state.$current, name = getUiViewName(scope, attrs, $element, $interpolate), locals = current && current.locals[name];
          if (!locals) {
            return;
          }
          $element.data('$uiView', {
            name: name,
            state: locals.$$state
          });
          $element.html(locals.$template ? locals.$template : initial);
          var link = $compile($element.contents());
          if (locals.$$controller) {
            locals.$scope = scope;
            locals.$element = $element;
            var controller = $controller(locals.$$controller, locals);
            if (locals.$$controllerAs) {
              scope[locals.$$controllerAs] = controller;
            }
            $element.data('$ngControllerController', controller);
            $element.children().data('$ngControllerController', controller);
          }
          link(scope);
        };
      }
    };
  }
  /**
 * Shared ui-view code for both directives:
 * Given scope, element, and its attributes, return the view's name
 */
  function getUiViewName(scope, attrs, element, $interpolate) {
    var name = $interpolate(attrs.uiView || attrs.name || '')(scope);
    var inherited = element.inheritedData('$uiView');
    return name.indexOf('@') >= 0 ? name : name + '@' + (inherited ? inherited.state.name : '');
  }
  angular.module('ui.router.state').directive('uiView', $ViewDirective);
  angular.module('ui.router.state').directive('uiView', $ViewDirectiveFill);
  function parseStateRef(ref, current) {
    var preparsed = ref.match(/^\s*({[^}]*})\s*$/), parsed;
    if (preparsed)
      ref = current + '(' + preparsed[1] + ')';
    parsed = ref.replace(/\n/g, ' ').match(/^([^(]+?)\s*(\((.*)\))?$/);
    if (!parsed || parsed.length !== 4)
      throw new Error('Invalid state ref \'' + ref + '\'');
    return {
      state: parsed[1],
      paramExpr: parsed[3] || null
    };
  }
  function stateContext(el) {
    var stateData = el.parent().inheritedData('$uiView');
    if (stateData && stateData.state && stateData.state.name) {
      return stateData.state;
    }
  }
  function getTypeInfo(el) {
    // SVGAElement does not use the href attribute, but rather the 'xlinkHref' attribute.
    var isSvg = Object.prototype.toString.call(el.prop('href')) === '[object SVGAnimatedString]';
    var isForm = el[0].nodeName === 'FORM';
    return {
      attr: isForm ? 'action' : isSvg ? 'xlink:href' : 'href',
      isAnchor: el.prop('tagName').toUpperCase() === 'A',
      clickable: !isForm
    };
  }
  function clickHook(el, $state, $timeout, type, current) {
    return function (e) {
      var button = e.which || e.button, target = current();
      if (!(button > 1 || e.ctrlKey || e.metaKey || e.shiftKey || el.attr('target'))) {
        // HACK: This is to allow ng-clicks to be processed before the transition is initiated:
        var transition = $timeout(function () {
            $state.go(target.state, target.params, target.options);
          });
        e.preventDefault();
        // if the state has no URL, ignore one preventDefault from the <a> directive.
        var ignorePreventDefaultCount = type.isAnchor && !target.href ? 1 : 0;
        e.preventDefault = function () {
          if (ignorePreventDefaultCount-- <= 0)
            $timeout.cancel(transition);
        };
      }
    };
  }
  function defaultOpts(el, $state) {
    return {
      relative: stateContext(el) || $state.$current,
      inherit: true
    };
  }
  /**
 * @ngdoc directive
 * @name ui.router.state.directive:ui-sref
 *
 * @requires ui.router.state.$state
 * @requires $timeout
 *
 * @restrict A
 *
 * @description
 * A directive that binds a link (`<a>` tag) to a state. If the state has an associated
 * URL, the directive will automatically generate & update the `href` attribute via
 * the {@link ui.router.state.$state#methods_href $state.href()} method. Clicking
 * the link will trigger a state transition with optional parameters.
 *
 * Also middle-clicking, right-clicking, and ctrl-clicking on the link will be
 * handled natively by the browser.
 *
 * You can also use relative state paths within ui-sref, just like the relative
 * paths passed to `$state.go()`. You just need to be aware that the path is relative
 * to the state that the link lives in, in other words the state that loaded the
 * template containing the link.
 *
 * You can specify options to pass to {@link ui.router.state.$state#go $state.go()}
 * using the `ui-sref-opts` attribute. Options are restricted to `location`, `inherit`,
 * and `reload`.
 *
 * @example
 * Here's an example of how you'd use ui-sref and how it would compile. If you have the
 * following template:
 * <pre>
 * <a ui-sref="home">Home</a> | <a ui-sref="about">About</a> | <a ui-sref="{page: 2}">Next page</a>
 *
 * <ul>
 *     <li ng-repeat="contact in contacts">
 *         <a ui-sref="contacts.detail({ id: contact.id })">{{ contact.name }}</a>
 *     </li>
 * </ul>
 * </pre>
 *
 * Then the compiled html would be (assuming Html5Mode is off and current state is contacts):
 * <pre>
 * <a href="#/home" ui-sref="home">Home</a> | <a href="#/about" ui-sref="about">About</a> | <a href="#/contacts?page=2" ui-sref="{page: 2}">Next page</a>
 *
 * <ul>
 *     <li ng-repeat="contact in contacts">
 *         <a href="#/contacts/1" ui-sref="contacts.detail({ id: contact.id })">Joe</a>
 *     </li>
 *     <li ng-repeat="contact in contacts">
 *         <a href="#/contacts/2" ui-sref="contacts.detail({ id: contact.id })">Alice</a>
 *     </li>
 *     <li ng-repeat="contact in contacts">
 *         <a href="#/contacts/3" ui-sref="contacts.detail({ id: contact.id })">Bob</a>
 *     </li>
 * </ul>
 *
 * <a ui-sref="home" ui-sref-opts="{reload: true}">Home</a>
 * </pre>
 *
 * @param {string} ui-sref 'stateName' can be any valid absolute or relative state
 * @param {Object} ui-sref-opts options to pass to {@link ui.router.state.$state#go $state.go()}
 */
  $StateRefDirective.$inject = [
    '$state',
    '$timeout'
  ];
  function $StateRefDirective($state, $timeout) {
    return {
      restrict: 'A',
      require: [
        '?^uiSrefActive',
        '?^uiSrefActiveEq'
      ],
      link: function (scope, element, attrs, uiSrefActive) {
        var ref = parseStateRef(attrs.uiSref, $state.current.name);
        var def = {
            state: ref.state,
            href: null,
            params: null
          };
        var type = getTypeInfo(element);
        var active = uiSrefActive[1] || uiSrefActive[0];
        def.options = extend(defaultOpts(element, $state), attrs.uiSrefOpts ? scope.$eval(attrs.uiSrefOpts) : {});
        var update = function (val) {
          if (val)
            def.params = angular.copy(val);
          def.href = $state.href(ref.state, def.params, def.options);
          if (active)
            active.$$addStateInfo(ref.state, def.params);
          if (def.href !== null)
            attrs.$set(type.attr, def.href);
        };
        if (ref.paramExpr) {
          scope.$watch(ref.paramExpr, function (val) {
            if (val !== def.params)
              update(val);
          }, true);
          def.params = angular.copy(scope.$eval(ref.paramExpr));
        }
        update();
        if (!type.clickable)
          return;
        element.bind('click', clickHook(element, $state, $timeout, type, function () {
          return def;
        }));
      }
    };
  }
  /**
 * @ngdoc directive
 * @name ui.router.state.directive:ui-state
 *
 * @requires ui.router.state.uiSref
 *
 * @restrict A
 *
 * @description
 * Much like ui-sref, but will accept named $scope properties to evaluate for a state definition,
 * params and override options.
 *
 * @param {string} ui-state 'stateName' can be any valid absolute or relative state
 * @param {Object} ui-state-params params to pass to {@link ui.router.state.$state#href $state.href()}
 * @param {Object} ui-state-opts options to pass to {@link ui.router.state.$state#go $state.go()}
 */
  $StateRefDynamicDirective.$inject = [
    '$state',
    '$timeout'
  ];
  function $StateRefDynamicDirective($state, $timeout) {
    return {
      restrict: 'A',
      require: [
        '?^uiSrefActive',
        '?^uiSrefActiveEq'
      ],
      link: function (scope, element, attrs, uiSrefActive) {
        var type = getTypeInfo(element);
        var active = uiSrefActive[1] || uiSrefActive[0];
        var group = [
            attrs.uiState,
            attrs.uiStateParams || null,
            attrs.uiStateOpts || null
          ];
        var watch = '[' + group.map(function (val) {
            return val || 'null';
          }).join(', ') + ']';
        var def = {
            state: null,
            params: null,
            options: null,
            href: null
          };
        function runStateRefLink(group) {
          def.state = group[0];
          def.params = group[1];
          def.options = group[2];
          def.href = $state.href(def.state, def.params, def.options);
          if (active)
            active.$$addStateInfo(def.state, def.params);
          if (def.href)
            attrs.$set(type.attr, def.href);
        }
        scope.$watch(watch, runStateRefLink, true);
        runStateRefLink(scope.$eval(watch));
        if (!type.clickable)
          return;
        element.bind('click', clickHook(element, $state, $timeout, type, function () {
          return def;
        }));
      }
    };
  }
  /**
 * @ngdoc directive
 * @name ui.router.state.directive:ui-sref-active
 *
 * @requires ui.router.state.$state
 * @requires ui.router.state.$stateParams
 * @requires $interpolate
 *
 * @restrict A
 *
 * @description
 * A directive working alongside ui-sref to add classes to an element when the
 * related ui-sref directive's state is active, and removing them when it is inactive.
 * The primary use-case is to simplify the special appearance of navigation menus
 * relying on `ui-sref`, by having the "active" state's menu button appear different,
 * distinguishing it from the inactive menu items.
 *
 * ui-sref-active can live on the same element as ui-sref or on a parent element. The first
 * ui-sref-active found at the same level or above the ui-sref will be used.
 *
 * Will activate when the ui-sref's target state or any child state is active. If you
 * need to activate only when the ui-sref target state is active and *not* any of
 * it's children, then you will use
 * {@link ui.router.state.directive:ui-sref-active-eq ui-sref-active-eq}
 *
 * @example
 * Given the following template:
 * <pre>
 * <ul>
 *   <li ui-sref-active="active" class="item">
 *     <a href ui-sref="app.user({user: 'bilbobaggins'})">@bilbobaggins</a>
 *   </li>
 * </ul>
 * </pre>
 *
 *
 * When the app state is "app.user" (or any children states), and contains the state parameter "user" with value "bilbobaggins",
 * the resulting HTML will appear as (note the 'active' class):
 * <pre>
 * <ul>
 *   <li ui-sref-active="active" class="item active">
 *     <a ui-sref="app.user({user: 'bilbobaggins'})" href="/users/bilbobaggins">@bilbobaggins</a>
 *   </li>
 * </ul>
 * </pre>
 *
 * The class name is interpolated **once** during the directives link time (any further changes to the
 * interpolated value are ignored).
 *
 * Multiple classes may be specified in a space-separated format:
 * <pre>
 * <ul>
 *   <li ui-sref-active='class1 class2 class3'>
 *     <a ui-sref="app.user">link</a>
 *   </li>
 * </ul>
 * </pre>
 *
 * It is also possible to pass ui-sref-active an expression that evaluates
 * to an object hash, whose keys represent active class names and whose
 * values represent the respective state names/globs.
 * ui-sref-active will match if the current active state **includes** any of
 * the specified state names/globs, even the abstract ones.
 *
 * @Example
 * Given the following template, with "admin" being an abstract state:
 * <pre>
 * <div ui-sref-active="{'active': 'admin.*'}">
 *   <a ui-sref-active="active" ui-sref="admin.roles">Roles</a>
 * </div>
 * </pre>
 *
 * When the current state is "admin.roles" the "active" class will be applied
 * to both the <div> and <a> elements. It is important to note that the state
 * names/globs passed to ui-sref-active shadow the state provided by ui-sref.
 */
  /**
 * @ngdoc directive
 * @name ui.router.state.directive:ui-sref-active-eq
 *
 * @requires ui.router.state.$state
 * @requires ui.router.state.$stateParams
 * @requires $interpolate
 *
 * @restrict A
 *
 * @description
 * The same as {@link ui.router.state.directive:ui-sref-active ui-sref-active} but will only activate
 * when the exact target state used in the `ui-sref` is active; no child states.
 *
 */
  $StateRefActiveDirective.$inject = [
    '$state',
    '$stateParams',
    '$interpolate'
  ];
  function $StateRefActiveDirective($state, $stateParams, $interpolate) {
    return {
      restrict: 'A',
      controller: [
        '$scope',
        '$element',
        '$attrs',
        '$timeout',
        function ($scope, $element, $attrs, $timeout) {
          var states = [], activeClasses = {}, activeEqClass, uiSrefActive;
          // There probably isn't much point in $observing this
          // uiSrefActive and uiSrefActiveEq share the same directive object with some
          // slight difference in logic routing
          activeEqClass = $interpolate($attrs.uiSrefActiveEq || '', false)($scope);
          try {
            uiSrefActive = $scope.$eval($attrs.uiSrefActive);
          } catch (e) {
          }
          uiSrefActive = uiSrefActive || $interpolate($attrs.uiSrefActive || '', false)($scope);
          if (isObject(uiSrefActive)) {
            forEach(uiSrefActive, function (stateOrName, activeClass) {
              if (isString(stateOrName)) {
                var ref = parseStateRef(stateOrName, $state.current.name);
                addState(ref.state, $scope.$eval(ref.paramExpr), activeClass);
              }
            });
          }
          // Allow uiSref to communicate with uiSrefActive[Equals]
          this.$$addStateInfo = function (newState, newParams) {
            // we already got an explicit state provided by ui-sref-active, so we
            // shadow the one that comes from ui-sref
            if (isObject(uiSrefActive) && states.length > 0) {
              return;
            }
            addState(newState, newParams, uiSrefActive);
            update();
          };
          $scope.$on('$stateChangeSuccess', update);
          function addState(stateName, stateParams, activeClass) {
            var state = $state.get(stateName, stateContext($element));
            var stateHash = createStateHash(stateName, stateParams);
            states.push({
              state: state || { name: stateName },
              params: stateParams,
              hash: stateHash
            });
            activeClasses[stateHash] = activeClass;
          }
          /**
       * @param {string} state
       * @param {Object|string} [params]
       * @return {string}
       */
          function createStateHash(state, params) {
            if (!isString(state)) {
              throw new Error('state should be a string');
            }
            if (isObject(params)) {
              return state + toJson(params);
            }
            params = $scope.$eval(params);
            if (isObject(params)) {
              return state + toJson(params);
            }
            return state;
          }
          // Update route state
          function update() {
            for (var i = 0; i < states.length; i++) {
              if (anyMatch(states[i].state, states[i].params)) {
                addClass($element, activeClasses[states[i].hash]);
              } else {
                removeClass($element, activeClasses[states[i].hash]);
              }
              if (exactMatch(states[i].state, states[i].params)) {
                addClass($element, activeEqClass);
              } else {
                removeClass($element, activeEqClass);
              }
            }
          }
          function addClass(el, className) {
            $timeout(function () {
              el.addClass(className);
            });
          }
          function removeClass(el, className) {
            el.removeClass(className);
          }
          function anyMatch(state, params) {
            return $state.includes(state.name, params);
          }
          function exactMatch(state, params) {
            return $state.is(state.name, params);
          }
          update();
        }
      ]
    };
  }
  angular.module('ui.router.state').directive('uiSref', $StateRefDirective).directive('uiSrefActive', $StateRefActiveDirective).directive('uiSrefActiveEq', $StateRefActiveDirective).directive('uiState', $StateRefDynamicDirective);
  /**
 * @ngdoc filter
 * @name ui.router.state.filter:isState
 *
 * @requires ui.router.state.$state
 *
 * @description
 * Translates to {@link ui.router.state.$state#methods_is $state.is("stateName")}.
 */
  $IsStateFilter.$inject = ['$state'];
  function $IsStateFilter($state) {
    var isFilter = function (state, params) {
      return $state.is(state, params);
    };
    isFilter.$stateful = true;
    return isFilter;
  }
  /**
 * @ngdoc filter
 * @name ui.router.state.filter:includedByState
 *
 * @requires ui.router.state.$state
 *
 * @description
 * Translates to {@link ui.router.state.$state#methods_includes $state.includes('fullOrPartialStateName')}.
 */
  $IncludedByStateFilter.$inject = ['$state'];
  function $IncludedByStateFilter($state) {
    var includesFilter = function (state, params, options) {
      return $state.includes(state, params, options);
    };
    includesFilter.$stateful = true;
    return includesFilter;
  }
  angular.module('ui.router.state').filter('isState', $IsStateFilter).filter('includedByState', $IncludedByStateFilter);
}(window, window.angular));
/**
 * @license AngularJS v1.3.20
 * (c) 2010-2014 Google, Inc. http://angularjs.org
 * License: MIT
 */
(function (window, angular, undefined) {
  'use strict';
  var $resourceMinErr = angular.$$minErr('$resource');
  // Helper functions and regex to lookup a dotted path on an object
  // stopping at undefined/null.  The path must be composed of ASCII
  // identifiers (just like $parse)
  var MEMBER_NAME_REGEX = /^(\.[a-zA-Z_$][0-9a-zA-Z_$]*)+$/;
  function isValidDottedPath(path) {
    return path != null && path !== '' && path !== 'hasOwnProperty' && MEMBER_NAME_REGEX.test('.' + path);
  }
  function lookupDottedPath(obj, path) {
    if (!isValidDottedPath(path)) {
      throw $resourceMinErr('badmember', 'Dotted member path "@{0}" is invalid.', path);
    }
    var keys = path.split('.');
    for (var i = 0, ii = keys.length; i < ii && obj !== undefined; i++) {
      var key = keys[i];
      obj = obj !== null ? obj[key] : undefined;
    }
    return obj;
  }
  /**
 * Create a shallow copy of an object and clear other fields from the destination
 */
  function shallowClearAndCopy(src, dst) {
    dst = dst || {};
    angular.forEach(dst, function (value, key) {
      delete dst[key];
    });
    for (var key in src) {
      if (src.hasOwnProperty(key) && !(key.charAt(0) === '$' && key.charAt(1) === '$')) {
        dst[key] = src[key];
      }
    }
    return dst;
  }
  /**
 * @ngdoc module
 * @name ngResource
 * @description
 *
 * # ngResource
 *
 * The `ngResource` module provides interaction support with RESTful services
 * via the $resource service.
 *
 *
 * <div doc-module-components="ngResource"></div>
 *
 * See {@link ngResource.$resource `$resource`} for usage.
 */
  /**
 * @ngdoc service
 * @name $resource
 * @requires $http
 *
 * @description
 * A factory which creates a resource object that lets you interact with
 * [RESTful](http://en.wikipedia.org/wiki/Representational_State_Transfer) server-side data sources.
 *
 * The returned resource object has action methods which provide high-level behaviors without
 * the need to interact with the low level {@link ng.$http $http} service.
 *
 * Requires the {@link ngResource `ngResource`} module to be installed.
 *
 * By default, trailing slashes will be stripped from the calculated URLs,
 * which can pose problems with server backends that do not expect that
 * behavior.  This can be disabled by configuring the `$resourceProvider` like
 * this:
 *
 * ```js
     app.config(['$resourceProvider', function($resourceProvider) {
       // Don't strip trailing slashes from calculated URLs
       $resourceProvider.defaults.stripTrailingSlashes = false;
     }]);
 * ```
 *
 * @param {string} url A parametrized URL template with parameters prefixed by `:` as in
 *   `/user/:username`. If you are using a URL with a port number (e.g.
 *   `http://example.com:8080/api`), it will be respected.
 *
 *   If you are using a url with a suffix, just add the suffix, like this:
 *   `$resource('http://example.com/resource.json')` or `$resource('http://example.com/:id.json')`
 *   or even `$resource('http://example.com/resource/:resource_id.:format')`
 *   If the parameter before the suffix is empty, :resource_id in this case, then the `/.` will be
 *   collapsed down to a single `.`.  If you need this sequence to appear and not collapse then you
 *   can escape it with `/\.`.
 *
 * @param {Object=} paramDefaults Default values for `url` parameters. These can be overridden in
 *   `actions` methods. If any of the parameter value is a function, it will be executed every time
 *   when a param value needs to be obtained for a request (unless the param was overridden).
 *
 *   Each key value in the parameter object is first bound to url template if present and then any
 *   excess keys are appended to the url search query after the `?`.
 *
 *   Given a template `/path/:verb` and parameter `{verb:'greet', salutation:'Hello'}` results in
 *   URL `/path/greet?salutation=Hello`.
 *
 *   If the parameter value is prefixed with `@` then the value for that parameter will be extracted
 *   from the corresponding property on the `data` object (provided when calling an action method).  For
 *   example, if the `defaultParam` object is `{someParam: '@someProp'}` then the value of `someParam`
 *   will be `data.someProp`.
 *
 * @param {Object.<Object>=} actions Hash with declaration of custom actions that should extend
 *   the default set of resource actions. The declaration should be created in the format of {@link
 *   ng.$http#usage $http.config}:
 *
 *       {action1: {method:?, params:?, isArray:?, headers:?, ...},
 *        action2: {method:?, params:?, isArray:?, headers:?, ...},
 *        ...}
 *
 *   Where:
 *
 *   - **`action`** – {string} – The name of action. This name becomes the name of the method on
 *     your resource object.
 *   - **`method`** – {string} – Case insensitive HTTP method (e.g. `GET`, `POST`, `PUT`,
 *     `DELETE`, `JSONP`, etc).
 *   - **`params`** – {Object=} – Optional set of pre-bound parameters for this action. If any of
 *     the parameter value is a function, it will be executed every time when a param value needs to
 *     be obtained for a request (unless the param was overridden).
 *   - **`url`** – {string} – action specific `url` override. The url templating is supported just
 *     like for the resource-level urls.
 *   - **`isArray`** – {boolean=} – If true then the returned object for this action is an array,
 *     see `returns` section.
 *   - **`transformRequest`** –
 *     `{function(data, headersGetter)|Array.<function(data, headersGetter)>}` –
 *     transform function or an array of such functions. The transform function takes the http
 *     request body and headers and returns its transformed (typically serialized) version.
 *     By default, transformRequest will contain one function that checks if the request data is
 *     an object and serializes to using `angular.toJson`. To prevent this behavior, set
 *     `transformRequest` to an empty array: `transformRequest: []`
 *   - **`transformResponse`** –
 *     `{function(data, headersGetter)|Array.<function(data, headersGetter)>}` –
 *     transform function or an array of such functions. The transform function takes the http
 *     response body and headers and returns its transformed (typically deserialized) version.
 *     By default, transformResponse will contain one function that checks if the response looks like
 *     a JSON string and deserializes it using `angular.fromJson`. To prevent this behavior, set
 *     `transformResponse` to an empty array: `transformResponse: []`
 *   - **`cache`** – `{boolean|Cache}` – If true, a default $http cache will be used to cache the
 *     GET request, otherwise if a cache instance built with
 *     {@link ng.$cacheFactory $cacheFactory}, this cache will be used for
 *     caching.
 *   - **`timeout`** – `{number|Promise}` – timeout in milliseconds, or {@link ng.$q promise} that
 *     should abort the request when resolved.
 *   - **`withCredentials`** - `{boolean}` - whether to set the `withCredentials` flag on the
 *     XHR object. See
 *     [requests with credentials](https://developer.mozilla.org/en/http_access_control#section_5)
 *     for more information.
 *   - **`responseType`** - `{string}` - see
 *     [requestType](https://developer.mozilla.org/en-US/docs/DOM/XMLHttpRequest#responseType).
 *   - **`interceptor`** - `{Object=}` - The interceptor object has two optional methods -
 *     `response` and `responseError`. Both `response` and `responseError` interceptors get called
 *     with `http response` object. See {@link ng.$http $http interceptors}.
 *
 * @param {Object} options Hash with custom settings that should extend the
 *   default `$resourceProvider` behavior.  The only supported option is
 *
 *   Where:
 *
 *   - **`stripTrailingSlashes`** – {boolean} – If true then the trailing
 *   slashes from any calculated URL will be stripped. (Defaults to true.)
 *
 * @returns {Object} A resource "class" object with methods for the default set of resource actions
 *   optionally extended with custom `actions`. The default set contains these actions:
 *   ```js
 *   { 'get':    {method:'GET'},
 *     'save':   {method:'POST'},
 *     'query':  {method:'GET', isArray:true},
 *     'remove': {method:'DELETE'},
 *     'delete': {method:'DELETE'} };
 *   ```
 *
 *   Calling these methods invoke an {@link ng.$http} with the specified http method,
 *   destination and parameters. When the data is returned from the server then the object is an
 *   instance of the resource class. The actions `save`, `remove` and `delete` are available on it
 *   as  methods with the `$` prefix. This allows you to easily perform CRUD operations (create,
 *   read, update, delete) on server-side data like this:
 *   ```js
 *   var User = $resource('/user/:userId', {userId:'@id'});
 *   var user = User.get({userId:123}, function() {
 *     user.abc = true;
 *     user.$save();
 *   });
 *   ```
 *
 *   It is important to realize that invoking a $resource object method immediately returns an
 *   empty reference (object or array depending on `isArray`). Once the data is returned from the
 *   server the existing reference is populated with the actual data. This is a useful trick since
 *   usually the resource is assigned to a model which is then rendered by the view. Having an empty
 *   object results in no rendering, once the data arrives from the server then the object is
 *   populated with the data and the view automatically re-renders itself showing the new data. This
 *   means that in most cases one never has to write a callback function for the action methods.
 *
 *   The action methods on the class object or instance object can be invoked with the following
 *   parameters:
 *
 *   - HTTP GET "class" actions: `Resource.action([parameters], [success], [error])`
 *   - non-GET "class" actions: `Resource.action([parameters], postData, [success], [error])`
 *   - non-GET instance actions:  `instance.$action([parameters], [success], [error])`
 *
 *
 *   Success callback is called with (value, responseHeaders) arguments. Error callback is called
 *   with (httpResponse) argument.
 *
 *   Class actions return empty instance (with additional properties below).
 *   Instance actions return promise of the action.
 *
 *   The Resource instances and collection have these additional properties:
 *
 *   - `$promise`: the {@link ng.$q promise} of the original server interaction that created this
 *     instance or collection.
 *
 *     On success, the promise is resolved with the same resource instance or collection object,
 *     updated with data from server. This makes it easy to use in
 *     {@link ngRoute.$routeProvider resolve section of $routeProvider.when()} to defer view
 *     rendering until the resource(s) are loaded.
 *
 *     On failure, the promise is resolved with the {@link ng.$http http response} object, without
 *     the `resource` property.
 *
 *     If an interceptor object was provided, the promise will instead be resolved with the value
 *     returned by the interceptor.
 *
 *   - `$resolved`: `true` after first server interaction is completed (either with success or
 *      rejection), `false` before that. Knowing if the Resource has been resolved is useful in
 *      data-binding.
 *
 * @example
 *
 * # Credit card resource
 *
 * ```js
     // Define CreditCard class
     var CreditCard = $resource('/user/:userId/card/:cardId',
      {userId:123, cardId:'@id'}, {
       charge: {method:'POST', params:{charge:true}}
      });

     // We can retrieve a collection from the server
     var cards = CreditCard.query(function() {
       // GET: /user/123/card
       // server returns: [ {id:456, number:'1234', name:'Smith'} ];

       var card = cards[0];
       // each item is an instance of CreditCard
       expect(card instanceof CreditCard).toEqual(true);
       card.name = "J. Smith";
       // non GET methods are mapped onto the instances
       card.$save();
       // POST: /user/123/card/456 {id:456, number:'1234', name:'J. Smith'}
       // server returns: {id:456, number:'1234', name: 'J. Smith'};

       // our custom method is mapped as well.
       card.$charge({amount:9.99});
       // POST: /user/123/card/456?amount=9.99&charge=true {id:456, number:'1234', name:'J. Smith'}
     });

     // we can create an instance as well
     var newCard = new CreditCard({number:'0123'});
     newCard.name = "Mike Smith";
     newCard.$save();
     // POST: /user/123/card {number:'0123', name:'Mike Smith'}
     // server returns: {id:789, number:'0123', name: 'Mike Smith'};
     expect(newCard.id).toEqual(789);
 * ```
 *
 * The object returned from this function execution is a resource "class" which has "static" method
 * for each action in the definition.
 *
 * Calling these methods invoke `$http` on the `url` template with the given `method`, `params` and
 * `headers`.
 * When the data is returned from the server then the object is an instance of the resource type and
 * all of the non-GET methods are available with `$` prefix. This allows you to easily support CRUD
 * operations (create, read, update, delete) on server-side data.

   ```js
     var User = $resource('/user/:userId', {userId:'@id'});
     User.get({userId:123}, function(user) {
       user.abc = true;
       user.$save();
     });
   ```
 *
 * It's worth noting that the success callback for `get`, `query` and other methods gets passed
 * in the response that came from the server as well as $http header getter function, so one
 * could rewrite the above example and get access to http headers as:
 *
   ```js
     var User = $resource('/user/:userId', {userId:'@id'});
     User.get({userId:123}, function(u, getResponseHeaders){
       u.abc = true;
       u.$save(function(u, putResponseHeaders) {
         //u => saved user object
         //putResponseHeaders => $http header getter
       });
     });
   ```
 *
 * You can also access the raw `$http` promise via the `$promise` property on the object returned
 *
   ```
     var User = $resource('/user/:userId', {userId:'@id'});
     User.get({userId:123})
         .$promise.then(function(user) {
           $scope.user = user;
         });
   ```

 * # Creating a custom 'PUT' request
 * In this example we create a custom method on our resource to make a PUT request
 * ```js
 *    var app = angular.module('app', ['ngResource', 'ngRoute']);
 *
 *    // Some APIs expect a PUT request in the format URL/object/ID
 *    // Here we are creating an 'update' method
 *    app.factory('Notes', ['$resource', function($resource) {
 *    return $resource('/notes/:id', null,
 *        {
 *            'update': { method:'PUT' }
 *        });
 *    }]);
 *
 *    // In our controller we get the ID from the URL using ngRoute and $routeParams
 *    // We pass in $routeParams and our Notes factory along with $scope
 *    app.controller('NotesCtrl', ['$scope', '$routeParams', 'Notes',
                                      function($scope, $routeParams, Notes) {
 *    // First get a note object from the factory
 *    var note = Notes.get({ id:$routeParams.id });
 *    $id = note.id;
 *
 *    // Now call update passing in the ID first then the object you are updating
 *    Notes.update({ id:$id }, note);
 *
 *    // This will PUT /notes/ID with the note object in the request payload
 *    }]);
 * ```
 */
  angular.module('ngResource', ['ng']).provider('$resource', function () {
    var provider = this;
    this.defaults = {
      stripTrailingSlashes: true,
      actions: {
        'get': { method: 'GET' },
        'save': { method: 'POST' },
        'query': {
          method: 'GET',
          isArray: true
        },
        'remove': { method: 'DELETE' },
        'delete': { method: 'DELETE' }
      }
    };
    this.$get = [
      '$http',
      '$q',
      function ($http, $q) {
        var noop = angular.noop, forEach = angular.forEach, extend = angular.extend, copy = angular.copy, isFunction = angular.isFunction;
        /**
       * We need our custom method because encodeURIComponent is too aggressive and doesn't follow
       * http://www.ietf.org/rfc/rfc3986.txt with regards to the character set
       * (pchar) allowed in path segments:
       *    segment       = *pchar
       *    pchar         = unreserved / pct-encoded / sub-delims / ":" / "@"
       *    pct-encoded   = "%" HEXDIG HEXDIG
       *    unreserved    = ALPHA / DIGIT / "-" / "." / "_" / "~"
       *    sub-delims    = "!" / "$" / "&" / "'" / "(" / ")"
       *                     / "*" / "+" / "," / ";" / "="
       */
        function encodeUriSegment(val) {
          return encodeUriQuery(val, true).replace(/%26/gi, '&').replace(/%3D/gi, '=').replace(/%2B/gi, '+');
        }
        /**
       * This method is intended for encoding *key* or *value* parts of query component. We need a
       * custom method because encodeURIComponent is too aggressive and encodes stuff that doesn't
       * have to be encoded per http://tools.ietf.org/html/rfc3986:
       *    query       = *( pchar / "/" / "?" )
       *    pchar         = unreserved / pct-encoded / sub-delims / ":" / "@"
       *    unreserved    = ALPHA / DIGIT / "-" / "." / "_" / "~"
       *    pct-encoded   = "%" HEXDIG HEXDIG
       *    sub-delims    = "!" / "$" / "&" / "'" / "(" / ")"
       *                     / "*" / "+" / "," / ";" / "="
       */
        function encodeUriQuery(val, pctEncodeSpaces) {
          return encodeURIComponent(val).replace(/%40/gi, '@').replace(/%3A/gi, ':').replace(/%24/g, '$').replace(/%2C/gi, ',').replace(/%20/g, pctEncodeSpaces ? '%20' : '+');
        }
        function Route(template, defaults) {
          this.template = template;
          this.defaults = extend({}, provider.defaults, defaults);
          this.urlParams = {};
        }
        Route.prototype = {
          setUrlParams: function (config, params, actionUrl) {
            var self = this, url = actionUrl || self.template, val, encodedVal;
            var urlParams = self.urlParams = {};
            forEach(url.split(/\W/), function (param) {
              if (param === 'hasOwnProperty') {
                throw $resourceMinErr('badname', 'hasOwnProperty is not a valid parameter name.');
              }
              if (!new RegExp('^\\d+$').test(param) && param && new RegExp('(^|[^\\\\]):' + param + '(\\W|$)').test(url)) {
                urlParams[param] = true;
              }
            });
            url = url.replace(/\\:/g, ':');
            params = params || {};
            forEach(self.urlParams, function (_, urlParam) {
              val = params.hasOwnProperty(urlParam) ? params[urlParam] : self.defaults[urlParam];
              if (angular.isDefined(val) && val !== null) {
                encodedVal = encodeUriSegment(val);
                url = url.replace(new RegExp(':' + urlParam + '(\\W|$)', 'g'), function (match, p1) {
                  return encodedVal + p1;
                });
              } else {
                url = url.replace(new RegExp('(/?):' + urlParam + '(\\W|$)', 'g'), function (match, leadingSlashes, tail) {
                  if (tail.charAt(0) == '/') {
                    return tail;
                  } else {
                    return leadingSlashes + tail;
                  }
                });
              }
            });
            // strip trailing slashes and set the url (unless this behavior is specifically disabled)
            if (self.defaults.stripTrailingSlashes) {
              url = url.replace(/\/+$/, '') || '/';
            }
            // then replace collapse `/.` if found in the last URL path segment before the query
            // E.g. `http://url.com/id./format?q=x` becomes `http://url.com/id.format?q=x`
            url = url.replace(/\/\.(?=\w+($|\?))/, '.');
            // replace escaped `/\.` with `/.`
            config.url = url.replace(/\/\\\./, '/.');
            // set params - delegate param encoding to $http
            forEach(params, function (value, key) {
              if (!self.urlParams[key]) {
                config.params = config.params || {};
                config.params[key] = value;
              }
            });
          }
        };
        function resourceFactory(url, paramDefaults, actions, options) {
          var route = new Route(url, options);
          actions = extend({}, provider.defaults.actions, actions);
          function extractParams(data, actionParams) {
            var ids = {};
            actionParams = extend({}, paramDefaults, actionParams);
            forEach(actionParams, function (value, key) {
              if (isFunction(value)) {
                value = value();
              }
              ids[key] = value && value.charAt && value.charAt(0) == '@' ? lookupDottedPath(data, value.substr(1)) : value;
            });
            return ids;
          }
          function defaultResponseInterceptor(response) {
            return response.resource;
          }
          function Resource(value) {
            shallowClearAndCopy(value || {}, this);
          }
          Resource.prototype.toJSON = function () {
            var data = extend({}, this);
            delete data.$promise;
            delete data.$resolved;
            return data;
          };
          forEach(actions, function (action, name) {
            var hasBody = /^(POST|PUT|PATCH)$/i.test(action.method);
            Resource[name] = function (a1, a2, a3, a4) {
              var params = {}, data, success, error;
              /* jshint -W086 */
              /* (purposefully fall through case statements) */
              switch (arguments.length) {
              case 4:
                error = a4;
                success = a3;
              //fallthrough
              case 3:
              case 2:
                if (isFunction(a2)) {
                  if (isFunction(a1)) {
                    success = a1;
                    error = a2;
                    break;
                  }
                  success = a2;
                  error = a3;  //fallthrough
                } else {
                  params = a1;
                  data = a2;
                  success = a3;
                  break;
                }
              case 1:
                if (isFunction(a1))
                  success = a1;
                else if (hasBody)
                  data = a1;
                else
                  params = a1;
                break;
              case 0:
                break;
              default:
                throw $resourceMinErr('badargs', 'Expected up to 4 arguments [params, data, success, error], got {0} arguments', arguments.length);
              }
              /* jshint +W086 */
              /* (purposefully fall through case statements) */
              var isInstanceCall = this instanceof Resource;
              var value = isInstanceCall ? data : action.isArray ? [] : new Resource(data);
              var httpConfig = {};
              var responseInterceptor = action.interceptor && action.interceptor.response || defaultResponseInterceptor;
              var responseErrorInterceptor = action.interceptor && action.interceptor.responseError || undefined;
              forEach(action, function (value, key) {
                if (key != 'params' && key != 'isArray' && key != 'interceptor') {
                  httpConfig[key] = copy(value);
                }
              });
              if (hasBody)
                httpConfig.data = data;
              route.setUrlParams(httpConfig, extend({}, extractParams(data, action.params || {}), params), action.url);
              var promise = $http(httpConfig).then(function (response) {
                  var data = response.data, promise = value.$promise;
                  if (data) {
                    // Need to convert action.isArray to boolean in case it is undefined
                    // jshint -W018
                    if (angular.isArray(data) !== !!action.isArray) {
                      throw $resourceMinErr('badcfg', 'Error in resource configuration for action `{0}`. Expected response to ' + 'contain an {1} but got an {2}', name, action.isArray ? 'array' : 'object', angular.isArray(data) ? 'array' : 'object');
                    }
                    // jshint +W018
                    if (action.isArray) {
                      value.length = 0;
                      forEach(data, function (item) {
                        if (typeof item === 'object') {
                          value.push(new Resource(item));
                        } else {
                          // Valid JSON values may be string literals, and these should not be converted
                          // into objects. These items will not have access to the Resource prototype
                          // methods, but unfortunately there
                          value.push(item);
                        }
                      });
                    } else {
                      shallowClearAndCopy(data, value);
                      value.$promise = promise;
                    }
                  }
                  value.$resolved = true;
                  response.resource = value;
                  return response;
                }, function (response) {
                  value.$resolved = true;
                  (error || noop)(response);
                  return $q.reject(response);
                });
              promise = promise.then(function (response) {
                var value = responseInterceptor(response);
                (success || noop)(value, response.headers);
                return value;
              }, responseErrorInterceptor);
              if (!isInstanceCall) {
                // we are creating instance / collection
                // - set the initial promise
                // - return the instance / collection
                value.$promise = promise;
                value.$resolved = false;
                return value;
              }
              // instance call
              return promise;
            };
            Resource.prototype['$' + name] = function (params, success, error) {
              if (isFunction(params)) {
                error = success;
                success = params;
                params = {};
              }
              var result = Resource[name].call(this, params, this, success, error);
              return result.$promise || result;
            };
          });
          Resource.bind = function (additionalParamDefaults) {
            return resourceFactory(url, extend({}, paramDefaults, additionalParamDefaults), actions);
          };
          return Resource;
        }
        return resourceFactory;
      }
    ];
  });
}(window, window.angular));
/**
 * @license AngularJS v1.3.20
 * (c) 2010-2014 Google, Inc. http://angularjs.org
 * License: MIT
 */
(function (window, angular, undefined) {
  'use strict';
  /**
 * @ngdoc module
 * @name ngCookies
 * @description
 *
 * # ngCookies
 *
 * The `ngCookies` module provides a convenient wrapper for reading and writing browser cookies.
 *
 *
 * <div doc-module-components="ngCookies"></div>
 *
 * See {@link ngCookies.$cookies `$cookies`} and
 * {@link ngCookies.$cookieStore `$cookieStore`} for usage.
 */
  angular.module('ngCookies', ['ng']).factory('$cookies', [
    '$rootScope',
    '$browser',
    function ($rootScope, $browser) {
      var cookies = {}, lastCookies = {}, lastBrowserCookies, runEval = false, copy = angular.copy, isUndefined = angular.isUndefined;
      //creates a poller fn that copies all cookies from the $browser to service & inits the service
      $browser.addPollFn(function () {
        var currentCookies = $browser.cookies();
        if (lastBrowserCookies != currentCookies) {
          //relies on browser.cookies() impl
          lastBrowserCookies = currentCookies;
          copy(currentCookies, lastCookies);
          copy(currentCookies, cookies);
          if (runEval)
            $rootScope.$apply();
        }
      })();
      runEval = true;
      //at the end of each eval, push cookies
      //TODO: this should happen before the "delayed" watches fire, because if some cookies are not
      //      strings or browser refuses to store some cookies, we update the model in the push fn.
      $rootScope.$watch(push);
      return cookies;
      /**
       * Pushes all the cookies from the service to the browser and verifies if all cookies were
       * stored.
       */
      function push() {
        var name, value, browserCookies, updated;
        //delete any cookies deleted in $cookies
        for (name in lastCookies) {
          if (isUndefined(cookies[name])) {
            $browser.cookies(name, undefined);
            delete lastCookies[name];
          }
        }
        //update all cookies updated in $cookies
        for (name in cookies) {
          value = cookies[name];
          if (!angular.isString(value)) {
            value = '' + value;
            cookies[name] = value;
          }
          if (value !== lastCookies[name]) {
            $browser.cookies(name, value);
            lastCookies[name] = value;
            updated = true;
          }
        }
        //verify what was actually stored
        if (updated) {
          browserCookies = $browser.cookies();
          for (name in cookies) {
            if (cookies[name] !== browserCookies[name]) {
              //delete or reset all cookies that the browser dropped from $cookies
              if (isUndefined(browserCookies[name])) {
                delete cookies[name];
                delete lastCookies[name];
              } else {
                cookies[name] = lastCookies[name] = browserCookies[name];
              }
            }
          }
        }
      }
    }
  ]).factory('$cookieStore', [
    '$cookies',
    function ($cookies) {
      return {
        get: function (key) {
          var value = $cookies[key];
          return value ? angular.fromJson(value) : value;
        },
        put: function (key, value) {
          $cookies[key] = angular.toJson(value);
        },
        remove: function (key) {
          delete $cookies[key];
        }
      };
    }
  ]);
}(window, window.angular));
/**
 * @license AngularJS v1.3.20
 * (c) 2010-2014 Google, Inc. http://angularjs.org
 * License: MIT
 */
(function (window, angular, undefined) {
  'use strict';
  /**
 * @ngdoc module
 * @name ngAria
 * @description
 *
 * The `ngAria` module provides support for common
 * [<abbr title="Accessible Rich Internet Applications">ARIA</abbr>](http://www.w3.org/TR/wai-aria/)
 * attributes that convey state or semantic information about the application for users
 * of assistive technologies, such as screen readers.
 *
 * <div doc-module-components="ngAria"></div>
 *
 * ## Usage
 *
 * For ngAria to do its magic, simply include the module as a dependency. The directives supported
 * by ngAria are:
 * `ngModel`, `ngDisabled`, `ngShow`, `ngHide`, `ngClick`, `ngDblClick`, and `ngMessages`.
 *
 * Below is a more detailed breakdown of the attributes handled by ngAria:
 *
 * | Directive                                   | Supported Attributes                                                                   |
 * |---------------------------------------------|----------------------------------------------------------------------------------------|
 * | {@link ng.directive:ngDisabled ngDisabled}  | aria-disabled                                                                          |
 * | {@link ng.directive:ngShow ngShow}          | aria-hidden                                                                            |
 * | {@link ng.directive:ngHide ngHide}          | aria-hidden                                                                            |
 * | {@link ng.directive:ngDblclick ngDblclick}  | tabindex                                                                               |
 * | {@link module:ngMessages ngMessages}        | aria-live                                                                              |
 * | {@link ng.directive:ngModel ngModel}        | aria-checked, aria-valuemin, aria-valuemax, aria-valuenow, aria-invalid, aria-required, input roles |
 * | {@link ng.directive:ngClick ngClick}        | tabindex, keypress event, button role                                                               |
 *
 * Find out more information about each directive by reading the
 * {@link guide/accessibility ngAria Developer Guide}.
 *
 * ##Example
 * Using ngDisabled with ngAria:
 * ```html
 * <md-checkbox ng-disabled="disabled">
 * ```
 * Becomes:
 * ```html
 * <md-checkbox ng-disabled="disabled" aria-disabled="true">
 * ```
 *
 * ##Disabling Attributes
 * It's possible to disable individual attributes added by ngAria with the
 * {@link ngAria.$ariaProvider#config config} method. For more details, see the
 * {@link guide/accessibility Developer Guide}.
 */
  /* global -ngAriaModule */
  var ngAriaModule = angular.module('ngAria', ['ng']).provider('$aria', $AriaProvider);
  /**
 * @ngdoc provider
 * @name $ariaProvider
 *
 * @description
 *
 * Used for configuring the ARIA attributes injected and managed by ngAria.
 *
 * ```js
 * angular.module('myApp', ['ngAria'], function config($ariaProvider) {
 *   $ariaProvider.config({
 *     ariaValue: true,
 *     tabindex: false
 *   });
 * });
 *```
 *
 * ## Dependencies
 * Requires the {@link ngAria} module to be installed.
 *
 */
  function $AriaProvider() {
    var config = {
        ariaHidden: true,
        ariaChecked: true,
        ariaDisabled: true,
        ariaRequired: true,
        ariaInvalid: true,
        ariaMultiline: true,
        ariaValue: true,
        tabindex: true,
        bindKeypress: true
      };
    /**
   * @ngdoc method
   * @name $ariaProvider#config
   *
   * @param {object} config object to enable/disable specific ARIA attributes
   *
   *  - **ariaHidden** – `{boolean}` – Enables/disables aria-hidden tags
   *  - **ariaChecked** – `{boolean}` – Enables/disables aria-checked tags
   *  - **ariaDisabled** – `{boolean}` – Enables/disables aria-disabled tags
   *  - **ariaRequired** – `{boolean}` – Enables/disables aria-required tags
   *  - **ariaInvalid** – `{boolean}` – Enables/disables aria-invalid tags
   *  - **ariaMultiline** – `{boolean}` – Enables/disables aria-multiline tags
   *  - **ariaValue** – `{boolean}` – Enables/disables aria-valuemin, aria-valuemax and aria-valuenow tags
   *  - **tabindex** – `{boolean}` – Enables/disables tabindex tags
   *  - **bindKeypress** – `{boolean}` – Enables/disables keypress event binding on `&lt;div&gt;` and
   *    `&lt;li&gt;` elements with ng-click
   *
   * @description
   * Enables/disables various ARIA attributes
   */
    this.config = function (newConfig) {
      config = angular.extend(config, newConfig);
    };
    function watchExpr(attrName, ariaAttr, negate) {
      return function (scope, elem, attr) {
        var ariaCamelName = attr.$normalize(ariaAttr);
        if (config[ariaCamelName] && !attr[ariaCamelName]) {
          scope.$watch(attr[attrName], function (boolVal) {
            if (negate) {
              boolVal = !boolVal;
            }
            elem.attr(ariaAttr, boolVal);
          });
        }
      };
    }
    /**
   * @ngdoc service
   * @name $aria
   *
   * @description
   * @priority 200
   *
   * The $aria service contains helper methods for applying common
   * [ARIA](http://www.w3.org/TR/wai-aria/) attributes to HTML directives.
   *
   * ngAria injects common accessibility attributes that tell assistive technologies when HTML
   * elements are enabled, selected, hidden, and more. To see how this is performed with ngAria,
   * let's review a code snippet from ngAria itself:
   *
   *```js
   * ngAriaModule.directive('ngDisabled', ['$aria', function($aria) {
   *   return $aria.$$watchExpr('ngDisabled', 'aria-disabled');
   * }])
   *```
   * Shown above, the ngAria module creates a directive with the same signature as the
   * traditional `ng-disabled` directive. But this ngAria version is dedicated to
   * solely managing accessibility attributes. The internal `$aria` service is used to watch the
   * boolean attribute `ngDisabled`. If it has not been explicitly set by the developer,
   * `aria-disabled` is injected as an attribute with its value synchronized to the value in
   * `ngDisabled`.
   *
   * Because ngAria hooks into the `ng-disabled` directive, developers do not have to do
   * anything to enable this feature. The `aria-disabled` attribute is automatically managed
   * simply as a silent side-effect of using `ng-disabled` with the ngAria module.
   *
   * The full list of directives that interface with ngAria:
   * * **ngModel**
   * * **ngShow**
   * * **ngHide**
   * * **ngClick**
   * * **ngDblclick**
   * * **ngMessages**
   * * **ngDisabled**
   *
   * Read the {@link guide/accessibility ngAria Developer Guide} for a thorough explanation of each
   * directive.
   *
   *
   * ## Dependencies
   * Requires the {@link ngAria} module to be installed.
   */
    this.$get = function () {
      return {
        config: function (key) {
          return config[key];
        },
        $$watchExpr: watchExpr
      };
    };
  }
  ngAriaModule.directive('ngShow', [
    '$aria',
    function ($aria) {
      return $aria.$$watchExpr('ngShow', 'aria-hidden', true);
    }
  ]).directive('ngHide', [
    '$aria',
    function ($aria) {
      return $aria.$$watchExpr('ngHide', 'aria-hidden', false);
    }
  ]).directive('ngModel', [
    '$aria',
    function ($aria) {
      function shouldAttachAttr(attr, normalizedAttr, elem) {
        return $aria.config(normalizedAttr) && !elem.attr(attr);
      }
      function shouldAttachRole(role, elem) {
        return !elem.attr('role') && elem.attr('type') === role && elem[0].nodeName !== 'INPUT';
      }
      function getShape(attr, elem) {
        var type = attr.type, role = attr.role;
        return (type || role) === 'checkbox' || role === 'menuitemcheckbox' ? 'checkbox' : (type || role) === 'radio' || role === 'menuitemradio' ? 'radio' : type === 'range' || role === 'progressbar' || role === 'slider' ? 'range' : (type || role) === 'textbox' || elem[0].nodeName === 'TEXTAREA' ? 'multiline' : '';
      }
      return {
        restrict: 'A',
        require: '?ngModel',
        priority: 200,
        compile: function (elem, attr) {
          var shape = getShape(attr, elem);
          return {
            pre: function (scope, elem, attr, ngModel) {
              if (shape === 'checkbox' && attr.type !== 'checkbox') {
                //Use the input[checkbox] $isEmpty implementation for elements with checkbox roles
                ngModel.$isEmpty = function (value) {
                  return value === false;
                };
              }
            },
            post: function (scope, elem, attr, ngModel) {
              var needsTabIndex = shouldAttachAttr('tabindex', 'tabindex', elem);
              function ngAriaWatchModelValue() {
                return ngModel.$modelValue;
              }
              function getRadioReaction() {
                if (needsTabIndex) {
                  needsTabIndex = false;
                  return function ngAriaRadioReaction(newVal) {
                    var boolVal = attr.value == ngModel.$viewValue;
                    elem.attr('aria-checked', boolVal);
                    elem.attr('tabindex', 0 - !boolVal);
                  };
                } else {
                  return function ngAriaRadioReaction(newVal) {
                    elem.attr('aria-checked', attr.value == ngModel.$viewValue);
                  };
                }
              }
              function ngAriaCheckboxReaction() {
                elem.attr('aria-checked', !ngModel.$isEmpty(ngModel.$viewValue));
              }
              switch (shape) {
              case 'radio':
              case 'checkbox':
                if (shouldAttachRole(shape, elem)) {
                  elem.attr('role', shape);
                }
                if (shouldAttachAttr('aria-checked', 'ariaChecked', elem)) {
                  scope.$watch(ngAriaWatchModelValue, shape === 'radio' ? getRadioReaction() : ngAriaCheckboxReaction);
                }
                break;
              case 'range':
                if (shouldAttachRole(shape, elem)) {
                  elem.attr('role', 'slider');
                }
                if ($aria.config('ariaValue')) {
                  if (attr.min && !elem.attr('aria-valuemin')) {
                    elem.attr('aria-valuemin', attr.min);
                  }
                  if (attr.max && !elem.attr('aria-valuemax')) {
                    elem.attr('aria-valuemax', attr.max);
                  }
                  if (!elem.attr('aria-valuenow')) {
                    scope.$watch(ngAriaWatchModelValue, function ngAriaValueNowReaction(newVal) {
                      elem.attr('aria-valuenow', newVal);
                    });
                  }
                }
                break;
              case 'multiline':
                if (shouldAttachAttr('aria-multiline', 'ariaMultiline', elem)) {
                  elem.attr('aria-multiline', true);
                }
                break;
              }
              if (needsTabIndex) {
                elem.attr('tabindex', 0);
              }
              if (ngModel.$validators.required && shouldAttachAttr('aria-required', 'ariaRequired', elem)) {
                scope.$watch(function ngAriaRequiredWatch() {
                  return ngModel.$error.required;
                }, function ngAriaRequiredReaction(newVal) {
                  elem.attr('aria-required', !!newVal);
                });
              }
              if (shouldAttachAttr('aria-invalid', 'ariaInvalid', elem)) {
                scope.$watch(function ngAriaInvalidWatch() {
                  return ngModel.$invalid;
                }, function ngAriaInvalidReaction(newVal) {
                  elem.attr('aria-invalid', !!newVal);
                });
              }
            }
          };
        }
      };
    }
  ]).directive('ngDisabled', [
    '$aria',
    function ($aria) {
      return $aria.$$watchExpr('ngDisabled', 'aria-disabled');
    }
  ]).directive('ngMessages', function () {
    return {
      restrict: 'A',
      require: '?ngMessages',
      link: function (scope, elem, attr, ngMessages) {
        if (!elem.attr('aria-live')) {
          elem.attr('aria-live', 'assertive');
        }
      }
    };
  }).directive('ngClick', [
    '$aria',
    '$parse',
    function ($aria, $parse) {
      return {
        restrict: 'A',
        compile: function (elem, attr) {
          var fn = $parse(attr.ngClick, null, true);
          return function (scope, elem, attr) {
            var nodeBlackList = [
                'BUTTON',
                'A',
                'INPUT',
                'TEXTAREA'
              ];
            function isNodeOneOf(elem, nodeTypeArray) {
              if (nodeTypeArray.indexOf(elem[0].nodeName) !== -1) {
                return true;
              }
            }
            if (!elem.attr('role') && !isNodeOneOf(elem, nodeBlackList)) {
              elem.attr('role', 'button');
            }
            if ($aria.config('tabindex') && !elem.attr('tabindex')) {
              elem.attr('tabindex', 0);
            }
            if ($aria.config('bindKeypress') && !attr.ngKeypress && !isNodeOneOf(elem, nodeBlackList)) {
              elem.on('keypress', function (event) {
                if (event.keyCode === 32 || event.keyCode === 13) {
                  scope.$apply(callback);
                }
                function callback() {
                  fn(scope, { $event: event });
                }
              });
            }
          };
        }
      };
    }
  ]).directive('ngDblclick', [
    '$aria',
    function ($aria) {
      return function (scope, elem, attr) {
        if ($aria.config('tabindex') && !elem.attr('tabindex')) {
          elem.attr('tabindex', 0);
        }
      };
    }
  ]);
}(window, window.angular));
/**
 * @license AngularJS v1.3.20
 * (c) 2010-2014 Google, Inc. http://angularjs.org
 * License: MIT
 */
(function (window, angular, undefined) {
  'use strict';
  /**
 * @ngdoc module
 * @name ngMessages
 * @description
 *
 * The `ngMessages` module provides enhanced support for displaying messages within templates
 * (typically within forms or when rendering message objects that return key/value data).
 * Instead of relying on JavaScript code and/or complex ng-if statements within your form template to
 * show and hide error messages specific to the state of an input field, the `ngMessages` and
 * `ngMessage` directives are designed to handle the complexity, inheritance and priority
 * sequencing based on the order of how the messages are defined in the template.
 *
 * Currently, the ngMessages module only contains the code for the `ngMessages`
 * and `ngMessage` directives.
 *
 * # Usage
 * The `ngMessages` directive listens on a key/value collection which is set on the ngMessages attribute.
 * Since the {@link ngModel ngModel} directive exposes an `$error` object, this error object can be
 * used with `ngMessages` to display control error messages in an easier way than with just regular angular
 * template directives.
 *
 * ```html
 * <form name="myForm">
 *   <input type="text" ng-model="field" name="myField" required minlength="5" />
 *   <div ng-messages="myForm.myField.$error">
 *     <div ng-message="required">You did not enter a field</div>
 *     <div ng-message="minlength">The value entered is too short</div>
 *   </div>
 * </form>
 * ```
 *
 * Now whatever key/value entries are present within the provided object (in this case `$error`) then
 * the ngMessages directive will render the inner first ngMessage directive (depending if the key values
 * match the attribute value present on each ngMessage directive). In other words, if your errors
 * object contains the following data:
 *
 * ```javascript
 * <!-- keep in mind that ngModel automatically sets these error flags -->
 * myField.$error = { minlength : true, required : false };
 * ```
 *
 * Then the `required` message will be displayed first. When required is false then the `minlength` message
 * will be displayed right after (since these messages are ordered this way in the template HTML code).
 * The prioritization of each message is determined by what order they're present in the DOM.
 * Therefore, instead of having custom JavaScript code determine the priority of what errors are
 * present before others, the presentation of the errors are handled within the template.
 *
 * By default, ngMessages will only display one error at a time. However, if you wish to display all
 * messages then the `ng-messages-multiple` attribute flag can be used on the element containing the
 * ngMessages directive to make this happen.
 *
 * ```html
 * <div ng-messages="myForm.myField.$error" ng-messages-multiple>...</div>
 * ```
 *
 * ## Reusing and Overriding Messages
 * In addition to prioritization, ngMessages also allows for including messages from a remote or an inline
 * template. This allows for generic collection of messages to be reused across multiple parts of an
 * application.
 *
 * ```html
 * <script type="text/ng-template" id="error-messages">
 *   <div ng-message="required">This field is required</div>
 *   <div ng-message="minlength">This field is too short</div>
 * </script>
 * <div ng-messages="myForm.myField.$error" ng-messages-include="error-messages"></div>
 * ```
 *
 * However, including generic messages may not be useful enough to match all input fields, therefore,
 * `ngMessages` provides the ability to override messages defined in the remote template by redefining
 * then within the directive container.
 *
 * ```html
 * <!-- a generic template of error messages known as "my-custom-messages" -->
 * <script type="text/ng-template" id="my-custom-messages">
 *   <div ng-message="required">This field is required</div>
 *   <div ng-message="minlength">This field is too short</div>
 * </script>
 *
 * <form name="myForm">
 *   <input type="email"
 *          id="email"
 *          name="myEmail"
 *          ng-model="email"
 *          minlength="5"
 *          required />
 *
 *   <div ng-messages="myForm.myEmail.$error" ng-messages-include="my-custom-messages">
 *     <!-- this required message has overridden the template message -->
 *     <div ng-message="required">You did not enter your email address</div>
 *
 *     <!-- this is a brand new message and will appear last in the prioritization -->
 *     <div ng-message="email">Your email address is invalid</div>
 *   </div>
 * </form>
 * ```
 *
 * In the example HTML code above the message that is set on required will override the corresponding
 * required message defined within the remote template. Therefore, with particular input fields (such
 * email addresses, date fields, autocomplete inputs, etc...), specialized error messages can be applied
 * while more generic messages can be used to handle other, more general input errors.
 *
 * ## Animations
 * If the `ngAnimate` module is active within the application then both the `ngMessages` and
 * `ngMessage` directives will trigger animations whenever any messages are added and removed
 * from the DOM by the `ngMessages` directive.
 *
 * Whenever the `ngMessages` directive contains one or more visible messages then the `.ng-active` CSS
 * class will be added to the element. The `.ng-inactive` CSS class will be applied when there are no
 * animations present. Therefore, CSS transitions and keyframes as well as JavaScript animations can
 * hook into the animations whenever these classes are added/removed.
 *
 * Let's say that our HTML code for our messages container looks like so:
 *
 * ```html
 * <div ng-messages="myMessages" class="my-messages">
 *   <div ng-message="alert" class="some-message">...</div>
 *   <div ng-message="fail" class="some-message">...</div>
 * </div>
 * ```
 *
 * Then the CSS animation code for the message container looks like so:
 *
 * ```css
 * .my-messages {
 *   transition:1s linear all;
 * }
 * .my-messages.ng-active {
 *   // messages are visible
 * }
 * .my-messages.ng-inactive {
 *   // messages are hidden
 * }
 * ```
 *
 * Whenever an inner message is attached (becomes visible) or removed (becomes hidden) then the enter
 * and leave animation is triggered for each particular element bound to the `ngMessage` directive.
 *
 * Therefore, the CSS code for the inner messages looks like so:
 *
 * ```css
 * .some-message {
 *   transition:1s linear all;
 * }
 *
 * .some-message.ng-enter {}
 * .some-message.ng-enter.ng-enter-active {}
 *
 * .some-message.ng-leave {}
 * .some-message.ng-leave.ng-leave-active {}
 * ```
 *
 * {@link ngAnimate Click here} to learn how to use JavaScript animations or to learn more about ngAnimate.
 */
  angular.module('ngMessages', []).directive('ngMessages', [
    '$compile',
    '$animate',
    '$templateRequest',
    function ($compile, $animate, $templateRequest) {
      var ACTIVE_CLASS = 'ng-active';
      var INACTIVE_CLASS = 'ng-inactive';
      return {
        restrict: 'AE',
        controller: function () {
          this.$renderNgMessageClasses = angular.noop;
          var messages = [];
          this.registerMessage = function (index, message) {
            for (var i = 0; i < messages.length; i++) {
              if (messages[i].type == message.type) {
                if (index != i) {
                  var temp = messages[index];
                  messages[index] = messages[i];
                  if (index < messages.length) {
                    messages[i] = temp;
                  } else {
                    messages.splice(0, i);  //remove the old one (and shift left)
                  }
                }
                return;
              }
            }
            messages.splice(index, 0, message);  //add the new one (and shift right)
          };
          this.renderMessages = function (values, multiple) {
            values = values || {};
            var found;
            angular.forEach(messages, function (message) {
              if ((!found || multiple) && truthyVal(values[message.type])) {
                message.attach();
                found = true;
              } else {
                message.detach();
              }
            });
            this.renderElementClasses(found);
            function truthyVal(value) {
              return value !== null && value !== false && value;
            }
          };
        },
        require: 'ngMessages',
        link: function ($scope, element, $attrs, ctrl) {
          ctrl.renderElementClasses = function (bool) {
            bool ? $animate.setClass(element, ACTIVE_CLASS, INACTIVE_CLASS) : $animate.setClass(element, INACTIVE_CLASS, ACTIVE_CLASS);
          };
          //JavaScript treats empty strings as false, but ng-message-multiple by itself is an empty string
          var multiple = angular.isString($attrs.ngMessagesMultiple) || angular.isString($attrs.multiple);
          var cachedValues, watchAttr = $attrs.ngMessages || $attrs['for'];
          //for is a reserved keyword
          $scope.$watchCollection(watchAttr, function (values) {
            cachedValues = values;
            ctrl.renderMessages(values, multiple);
          });
          var tpl = $attrs.ngMessagesInclude || $attrs.include;
          if (tpl) {
            $templateRequest(tpl).then(function processTemplate(html) {
              var after, container = angular.element('<div/>').html(html);
              angular.forEach(container.children(), function (elm) {
                elm = angular.element(elm);
                after ? after.after(elm) : element.prepend(elm);
                //start of the container
                after = elm;
                $compile(elm)($scope);
              });
              ctrl.renderMessages(cachedValues, multiple);
            });
          }
        }
      };
    }
  ]).directive('ngMessage', [
    '$animate',
    function ($animate) {
      var COMMENT_NODE = 8;
      return {
        require: '^ngMessages',
        transclude: 'element',
        terminal: true,
        restrict: 'AE',
        link: function ($scope, $element, $attrs, ngMessages, $transclude) {
          var index, element;
          var commentNode = $element[0];
          var parentNode = commentNode.parentNode;
          for (var i = 0, j = 0; i < parentNode.childNodes.length; i++) {
            var node = parentNode.childNodes[i];
            if (node.nodeType == COMMENT_NODE && node.nodeValue.indexOf('ngMessage') >= 0) {
              if (node === commentNode) {
                index = j;
                break;
              }
              j++;
            }
          }
          ngMessages.registerMessage(index, {
            type: $attrs.ngMessage || $attrs.when,
            attach: function () {
              if (!element) {
                $transclude($scope, function (clone) {
                  $animate.enter(clone, null, $element);
                  element = clone;
                });
              }
            },
            detach: function (now) {
              if (element) {
                $animate.leave(element);
                element = null;
              }
            }
          });
        }
      };
    }
  ]);
}(window, window.angular));
/*!
 * Angular Material Design
 * https://github.com/angular/material
 * @license MIT
 * v1.0.0
 */
(function (window, angular, undefined) {
  'use strict';
  (function () {
    'use strict';
    angular.module('ngMaterial', [
      'ng',
      'ngAnimate',
      'ngAria',
      'material.core',
      'material.core.gestures',
      'material.core.layout',
      'material.core.theming.palette',
      'material.core.theming',
      'material.core.animate',
      'material.components.autocomplete',
      'material.components.backdrop',
      'material.components.bottomSheet',
      'material.components.button',
      'material.components.card',
      'material.components.checkbox',
      'material.components.chips',
      'material.components.content',
      'material.components.datepicker',
      'material.components.dialog',
      'material.components.divider',
      'material.components.fabActions',
      'material.components.fabShared',
      'material.components.fabSpeedDial',
      'material.components.fabToolbar',
      'material.components.fabTrigger',
      'material.components.gridList',
      'material.components.icon',
      'material.components.input',
      'material.components.list',
      'material.components.menu',
      'material.components.menuBar',
      'material.components.progressCircular',
      'material.components.radioButton',
      'material.components.progressLinear',
      'material.components.showHide',
      'material.components.select',
      'material.components.slider',
      'material.components.sidenav',
      'material.components.sticky',
      'material.components.subheader',
      'material.components.switch',
      'material.components.swipe',
      'material.components.tabs',
      'material.components.toast',
      'material.components.toolbar',
      'material.components.virtualRepeat',
      'material.components.tooltip',
      'material.components.whiteframe'
    ]);
  }());
  (function () {
    'use strict';
    /**
 * Initialization function that validates environment
 * requirements.
 */
    angular.module('material.core', [
      'ngAnimate',
      'material.core.animate',
      'material.core.layout',
      'material.core.gestures',
      'material.core.theming'
    ]).config(MdCoreConfigure).run(DetectNgTouch);
    /**
 * Detect if the ng-Touch module is also being used.
 * Warn if detected.
 */
    function DetectNgTouch($log, $injector) {
      if ($injector.has('$swipe')) {
        var msg = '' + 'You are using the ngTouch module. \n' + 'Angular Material already has mobile click, tap, and swipe support... \n' + 'ngTouch is not supported with Angular Material!';
        $log.warn(msg);
      }
    }
    DetectNgTouch.$inject = [
      '$log',
      '$injector'
    ];
    function MdCoreConfigure($provide, $mdThemingProvider) {
      $provide.decorator('$$rAF', [
        '$delegate',
        rAFDecorator
      ]);
      $mdThemingProvider.theme('default').primaryPalette('indigo').accentPalette('pink').warnPalette('deep-orange').backgroundPalette('grey');
    }
    MdCoreConfigure.$inject = [
      '$provide',
      '$mdThemingProvider'
    ];
    function rAFDecorator($delegate) {
      /**
   * Use this to throttle events that come in often.
   * The throttled function will always use the *last* invocation before the
   * coming frame.
   *
   * For example, window resize events that fire many times a second:
   * If we set to use an raf-throttled callback on window resize, then
   * our callback will only be fired once per frame, with the last resize
   * event that happened before that frame.
   *
   * @param {function} callback function to debounce
   */
      $delegate.throttle = function (cb) {
        var queuedArgs, alreadyQueued, queueCb, context;
        return function debounced() {
          queuedArgs = arguments;
          context = this;
          queueCb = cb;
          if (!alreadyQueued) {
            alreadyQueued = true;
            $delegate(function () {
              queueCb.apply(context, Array.prototype.slice.call(queuedArgs));
              alreadyQueued = false;
            });
          }
        };
      };
      return $delegate;
    }
  }());
  (function () {
    'use strict';
    angular.module('material.core').factory('$mdConstant', MdConstantFactory);
    /**
 * Factory function that creates the grab-bag $mdConstant service.
 * @ngInject
 */
    function MdConstantFactory($sniffer) {
      var webkit = /webkit/i.test($sniffer.vendorPrefix);
      function vendorProperty(name) {
        return webkit ? 'webkit' + name.charAt(0).toUpperCase() + name.substring(1) : name;
      }
      return {
        KEY_CODE: {
          COMMA: 188,
          ENTER: 13,
          ESCAPE: 27,
          SPACE: 32,
          PAGE_UP: 33,
          PAGE_DOWN: 34,
          END: 35,
          HOME: 36,
          LEFT_ARROW: 37,
          UP_ARROW: 38,
          RIGHT_ARROW: 39,
          DOWN_ARROW: 40,
          TAB: 9,
          BACKSPACE: 8,
          DELETE: 46
        },
        CSS: {
          TRANSITIONEND: 'transitionend' + (webkit ? ' webkitTransitionEnd' : ''),
          ANIMATIONEND: 'animationend' + (webkit ? ' webkitAnimationEnd' : ''),
          TRANSFORM: vendorProperty('transform'),
          TRANSFORM_ORIGIN: vendorProperty('transformOrigin'),
          TRANSITION: vendorProperty('transition'),
          TRANSITION_DURATION: vendorProperty('transitionDuration'),
          ANIMATION_PLAY_STATE: vendorProperty('animationPlayState'),
          ANIMATION_DURATION: vendorProperty('animationDuration'),
          ANIMATION_NAME: vendorProperty('animationName'),
          ANIMATION_TIMING: vendorProperty('animationTimingFunction'),
          ANIMATION_DIRECTION: vendorProperty('animationDirection')
        },
        MEDIA: {
          'xs': '(max-width: 599px)',
          'gt-xs': '(min-width: 600px)',
          'sm': '(min-width: 600px) and (max-width: 959px)',
          'gt-sm': '(min-width: 960px)',
          'md': '(min-width: 960px) and (max-width: 1279px)',
          'gt-md': '(min-width: 1280px)',
          'lg': '(min-width: 1280px) and (max-width: 1919px)',
          'gt-lg': '(min-width: 1920px)',
          'xl': '(min-width: 1920px)'
        },
        MEDIA_PRIORITY: [
          'xl',
          'gt-lg',
          'lg',
          'gt-md',
          'md',
          'gt-sm',
          'sm',
          'gt-xs',
          'xs'
        ]
      };
    }
    MdConstantFactory.$inject = ['$sniffer'];
  }());
  (function () {
    'use strict';
    angular.module('material.core').config([
      '$provide',
      function ($provide) {
        $provide.decorator('$mdUtil', [
          '$delegate',
          function ($delegate) {
            /**
            * Inject the iterator facade to easily support iteration and accessors
            * @see iterator below
            */
            $delegate.iterator = MdIterator;
            return $delegate;
          }
        ]);
      }
    ]);
    /**
   * iterator is a list facade to easily support iteration and accessors
   *
   * @param items Array list which this iterator will enumerate
   * @param reloop Boolean enables iterator to consider the list as an endless reloop
   */
    function MdIterator(items, reloop) {
      var trueFn = function () {
        return true;
      };
      if (items && !angular.isArray(items)) {
        items = Array.prototype.slice.call(items);
      }
      reloop = !!reloop;
      var _items = items || [];
      // Published API
      return {
        items: getItems,
        count: count,
        inRange: inRange,
        contains: contains,
        indexOf: indexOf,
        itemAt: itemAt,
        findBy: findBy,
        add: add,
        remove: remove,
        first: first,
        last: last,
        next: angular.bind(null, findSubsequentItem, false),
        previous: angular.bind(null, findSubsequentItem, true),
        hasPrevious: hasPrevious,
        hasNext: hasNext
      };
      /**
     * Publish copy of the enumerable set
     * @returns {Array|*}
     */
      function getItems() {
        return [].concat(_items);
      }
      /**
     * Determine length of the list
     * @returns {Array.length|*|number}
     */
      function count() {
        return _items.length;
      }
      /**
     * Is the index specified valid
     * @param index
     * @returns {Array.length|*|number|boolean}
     */
      function inRange(index) {
        return _items.length && index > -1 && index < _items.length;
      }
      /**
     * Can the iterator proceed to the next item in the list; relative to
     * the specified item.
     *
     * @param item
     * @returns {Array.length|*|number|boolean}
     */
      function hasNext(item) {
        return item ? inRange(indexOf(item) + 1) : false;
      }
      /**
     * Can the iterator proceed to the previous item in the list; relative to
     * the specified item.
     *
     * @param item
     * @returns {Array.length|*|number|boolean}
     */
      function hasPrevious(item) {
        return item ? inRange(indexOf(item) - 1) : false;
      }
      /**
     * Get item at specified index/position
     * @param index
     * @returns {*}
     */
      function itemAt(index) {
        return inRange(index) ? _items[index] : null;
      }
      /**
     * Find all elements matching the key/value pair
     * otherwise return null
     *
     * @param val
     * @param key
     *
     * @return array
     */
      function findBy(key, val) {
        return _items.filter(function (item) {
          return item[key] === val;
        });
      }
      /**
     * Add item to list
     * @param item
     * @param index
     * @returns {*}
     */
      function add(item, index) {
        if (!item)
          return -1;
        if (!angular.isNumber(index)) {
          index = _items.length;
        }
        _items.splice(index, 0, item);
        return indexOf(item);
      }
      /**
     * Remove item from list...
     * @param item
     */
      function remove(item) {
        if (contains(item)) {
          _items.splice(indexOf(item), 1);
        }
      }
      /**
     * Get the zero-based index of the target item
     * @param item
     * @returns {*}
     */
      function indexOf(item) {
        return _items.indexOf(item);
      }
      /**
     * Boolean existence check
     * @param item
     * @returns {boolean}
     */
      function contains(item) {
        return item && indexOf(item) > -1;
      }
      /**
     * Return first item in the list
     * @returns {*}
     */
      function first() {
        return _items.length ? _items[0] : null;
      }
      /**
     * Return last item in the list...
     * @returns {*}
     */
      function last() {
        return _items.length ? _items[_items.length - 1] : null;
      }
      /**
     * Find the next item. If reloop is true and at the end of the list, it will go back to the
     * first item. If given, the `validate` callback will be used to determine whether the next item
     * is valid. If not valid, it will try to find the next item again.
     *
     * @param {boolean} backwards Specifies the direction of searching (forwards/backwards)
     * @param {*} item The item whose subsequent item we are looking for
     * @param {Function=} validate The `validate` function
     * @param {integer=} limit The recursion limit
     *
     * @returns {*} The subsequent item or null
     */
      function findSubsequentItem(backwards, item, validate, limit) {
        validate = validate || trueFn;
        var curIndex = indexOf(item);
        while (true) {
          if (!inRange(curIndex))
            return null;
          var nextIndex = curIndex + (backwards ? -1 : 1);
          var foundItem = null;
          if (inRange(nextIndex)) {
            foundItem = _items[nextIndex];
          } else if (reloop) {
            foundItem = backwards ? last() : first();
            nextIndex = indexOf(foundItem);
          }
          if (foundItem === null || nextIndex === limit)
            return null;
          if (validate(foundItem))
            return foundItem;
          if (angular.isUndefined(limit))
            limit = nextIndex;
          curIndex = nextIndex;
        }
      }
    }
  }());
  (function () {
    'use strict';
    angular.module('material.core').factory('$mdMedia', mdMediaFactory);
    /**
 * @ngdoc service
 * @name $mdMedia
 * @module material.core
 *
 * @description
 * `$mdMedia` is used to evaluate whether a given media query is true or false given the
 * current device's screen / window size. The media query will be re-evaluated on resize, allowing
 * you to register a watch.
 *
 * `$mdMedia` also has pre-programmed support for media queries that match the layout breakpoints:
 *
 *  <table class="md-api-table">
 *    <thead>
 *    <tr>
 *      <th>Breakpoint</th>
 *      <th>mediaQuery</th>
 *    </tr>
 *    </thead>
 *    <tbody>
 *    <tr>
 *      <td>xs</td>
 *      <td>(max-width: 599px)</td>
 *    </tr>
 *    <tr>
 *      <td>gt-xs</td>
 *      <td>(min-width: 600px)</td>
 *    </tr>
 *    <tr>
 *      <td>sm</td>
 *      <td>(min-width: 600px) and (max-width: 959px)</td>
 *    </tr>
 *    <tr>
 *      <td>gt-sm</td>
 *      <td>(min-width: 960px)</td>
 *    </tr>
 *    <tr>
 *      <td>md</td>
 *      <td>(min-width: 960px) and (max-width: 1279px)</td>
 *    </tr>
 *    <tr>
 *      <td>gt-md</td>
 *      <td>(min-width: 1280px)</td>
 *    </tr>
 *    <tr>
 *      <td>lg</td>
 *      <td>(min-width: 1280px) and (max-width: 1919px)</td>
 *    </tr>
 *    <tr>
 *      <td>gt-lg</td>
 *      <td>(min-width: 1920px)</td>
 *    </tr>
 *    <tr>
 *      <td>xl</td>
 *      <td>(min-width: 1920px)</td>
 *    </tr>
 *    </tbody>
 *  </table>
 *
 *  See Material Design's <a href="https://www.google.com/design/spec/layout/adaptive-ui.html">Layout - Adaptive UI</a> for more details.
 *
 *  <a href="https://www.google.com/design/spec/layout/adaptive-ui.html">
 *  <img src="https://material-design.storage.googleapis.com/publish/material_v_4/material_ext_publish/0B8olV15J7abPSGFxemFiQVRtb1k/layout_adaptive_breakpoints_01.png" width="100%" height="100%"></img>
 *  </a>
 *
 * @returns {boolean} a boolean representing whether or not the given media query is true or false.
 *
 * @usage
 * <hljs lang="js">
 * app.controller('MyController', function($mdMedia, $scope) {
 *   $scope.$watch(function() { return $mdMedia('lg'); }, function(big) {
 *     $scope.bigScreen = big;
 *   });
 *
 *   $scope.screenIsSmall = $mdMedia('sm');
 *   $scope.customQuery = $mdMedia('(min-width: 1234px)');
 *   $scope.anotherCustom = $mdMedia('max-width: 300px');
 * });
 * </hljs>
 */
    function mdMediaFactory($mdConstant, $rootScope, $window) {
      var queries = {};
      var mqls = {};
      var results = {};
      var normalizeCache = {};
      $mdMedia.getResponsiveAttribute = getResponsiveAttribute;
      $mdMedia.getQuery = getQuery;
      $mdMedia.watchResponsiveAttributes = watchResponsiveAttributes;
      return $mdMedia;
      function $mdMedia(query) {
        var validated = queries[query];
        if (angular.isUndefined(validated)) {
          validated = queries[query] = validate(query);
        }
        var result = results[validated];
        if (angular.isUndefined(result)) {
          result = add(validated);
        }
        return result;
      }
      function validate(query) {
        return $mdConstant.MEDIA[query] || (query.charAt(0) !== '(' ? '(' + query + ')' : query);
      }
      function add(query) {
        var result = mqls[query];
        if (!result) {
          result = mqls[query] = $window.matchMedia(query);
        }
        result.addListener(onQueryChange);
        return results[result.media] = !!result.matches;
      }
      function onQueryChange(query) {
        $rootScope.$evalAsync(function () {
          results[query.media] = !!query.matches;
        });
      }
      function getQuery(name) {
        return mqls[name];
      }
      function getResponsiveAttribute(attrs, attrName) {
        for (var i = 0; i < $mdConstant.MEDIA_PRIORITY.length; i++) {
          var mediaName = $mdConstant.MEDIA_PRIORITY[i];
          if (!mqls[queries[mediaName]].matches) {
            continue;
          }
          var normalizedName = getNormalizedName(attrs, attrName + '-' + mediaName);
          if (attrs[normalizedName]) {
            return attrs[normalizedName];
          }
        }
        // fallback on unprefixed
        return attrs[getNormalizedName(attrs, attrName)];
      }
      function watchResponsiveAttributes(attrNames, attrs, watchFn) {
        var unwatchFns = [];
        attrNames.forEach(function (attrName) {
          var normalizedName = getNormalizedName(attrs, attrName);
          if (angular.isDefined(attrs[normalizedName])) {
            unwatchFns.push(attrs.$observe(normalizedName, angular.bind(void 0, watchFn, null)));
          }
          for (var mediaName in $mdConstant.MEDIA) {
            normalizedName = getNormalizedName(attrs, attrName + '-' + mediaName);
            if (angular.isDefined(attrs[normalizedName])) {
              unwatchFns.push(attrs.$observe(normalizedName, angular.bind(void 0, watchFn, mediaName)));
            }
          }
        });
        return function unwatch() {
          unwatchFns.forEach(function (fn) {
            fn();
          });
        };
      }
      // Improves performance dramatically
      function getNormalizedName(attrs, attrName) {
        return normalizeCache[attrName] || (normalizeCache[attrName] = attrs.$normalize(attrName));
      }
    }
    mdMediaFactory.$inject = [
      '$mdConstant',
      '$rootScope',
      '$window'
    ];
  }());
  (function () {
    'use strict';
    /*
 * This var has to be outside the angular factory, otherwise when
 * there are multiple material apps on the same page, each app
 * will create its own instance of this array and the app's IDs
 * will not be unique.
 */
    var nextUniqueId = 0;
    /**
 * @ngdoc module
 * @name material.core.util
 * @description
 * Util
 */
    angular.module('material.core').factory('$mdUtil', UtilFactory);
    function UtilFactory($document, $timeout, $compile, $rootScope, $$mdAnimate, $interpolate, $log, $rootElement, $window) {
      // Setup some core variables for the processTemplate method
      var startSymbol = $interpolate.startSymbol(), endSymbol = $interpolate.endSymbol(), usesStandardSymbols = startSymbol === '{{' && endSymbol === '}}';
      /**
   * Checks if the target element has the requested style by key
   * @param {DOMElement|JQLite} target Target element
   * @param {string} key Style key
   * @param {string=} expectedVal Optional expected value
   * @returns {boolean} Whether the target element has the style or not
   */
      var hasComputedStyle = function (target, key, expectedVal) {
        var hasValue = false;
        if (target && target.length) {
          var computedStyles = $window.getComputedStyle(target[0]);
          hasValue = angular.isDefined(computedStyles[key]) && (expectedVal ? computedStyles[key] == expectedVal : true);
        }
        return hasValue;
      };
      var $mdUtil = {
          dom: {},
          now: window.performance ? angular.bind(window.performance, window.performance.now) : Date.now || function () {
            return new Date().getTime();
          },
          clientRect: function (element, offsetParent, isOffsetRect) {
            var node = getNode(element);
            offsetParent = getNode(offsetParent || node.offsetParent || document.body);
            var nodeRect = node.getBoundingClientRect();
            // The user can ask for an offsetRect: a rect relative to the offsetParent,
            // or a clientRect: a rect relative to the page
            var offsetRect = isOffsetRect ? offsetParent.getBoundingClientRect() : {
                left: 0,
                top: 0,
                width: 0,
                height: 0
              };
            return {
              left: nodeRect.left - offsetRect.left,
              top: nodeRect.top - offsetRect.top,
              width: nodeRect.width,
              height: nodeRect.height
            };
          },
          offsetRect: function (element, offsetParent) {
            return $mdUtil.clientRect(element, offsetParent, true);
          },
          nodesToArray: function (nodes) {
            nodes = nodes || [];
            var results = [];
            for (var i = 0; i < nodes.length; ++i) {
              results.push(nodes.item(i));
            }
            return results;
          },
          scrollTop: function (element) {
            element = angular.element(element || $document[0].body);
            var body = element[0] == $document[0].body ? $document[0].body : undefined;
            var scrollTop = body ? body.scrollTop + body.parentElement.scrollTop : 0;
            // Calculate the positive scroll offset
            return scrollTop || Math.abs(element[0].getBoundingClientRect().top);
          },
          findFocusTarget: function (containerEl, attributeVal) {
            var AUTO_FOCUS = '[md-autofocus]';
            var elToFocus;
            elToFocus = scanForFocusable(containerEl, attributeVal || AUTO_FOCUS);
            if (!elToFocus && attributeVal != AUTO_FOCUS) {
              // Scan for deprecated attribute
              elToFocus = scanForFocusable(containerEl, '[md-auto-focus]');
              if (!elToFocus) {
                // Scan for fallback to 'universal' API
                elToFocus = scanForFocusable(containerEl, AUTO_FOCUS);
              }
            }
            return elToFocus;
            /**
       * Can target and nested children for specified Selector (attribute)
       * whose value may be an expression that evaluates to True/False.
       */
            function scanForFocusable(target, selector) {
              var elFound, items = target[0].querySelectorAll(selector);
              // Find the last child element with the focus attribute
              if (items && items.length) {
                var EXP_ATTR = /\s*\[?([\-a-z]*)\]?\s*/i;
                var matches = EXP_ATTR.exec(selector);
                var attribute = matches ? matches[1] : null;
                items.length && angular.forEach(items, function (it) {
                  it = angular.element(it);
                  // If the expression evaluates to FALSE, then it is not focusable target
                  var focusExpression = it[0].getAttribute(attribute);
                  var isFocusable = !focusExpression || !$mdUtil.validateScope(it) ? true : it.scope().$eval(focusExpression) !== false;
                  if (isFocusable)
                    elFound = it;
                });
              }
              return elFound;
            }
          },
          disableScrollAround: function (element, parent) {
            $mdUtil.disableScrollAround._count = $mdUtil.disableScrollAround._count || 0;
            ++$mdUtil.disableScrollAround._count;
            if ($mdUtil.disableScrollAround._enableScrolling)
              return $mdUtil.disableScrollAround._enableScrolling;
            element = angular.element(element);
            var body = $document[0].body, restoreBody = disableBodyScroll(), restoreElement = disableElementScroll(parent);
            return $mdUtil.disableScrollAround._enableScrolling = function () {
              if (!--$mdUtil.disableScrollAround._count) {
                restoreBody();
                restoreElement();
                delete $mdUtil.disableScrollAround._enableScrolling;
              }
            };
            // Creates a virtual scrolling mask to absorb touchmove, keyboard, scrollbar clicking, and wheel events
            function disableElementScroll(element) {
              element = angular.element(element || body)[0];
              var zIndex = 50;
              var scrollMask = angular.element('<div class="md-scroll-mask" style="z-index: ' + zIndex + '">' + '  <div class="md-scroll-mask-bar"></div>' + '</div>');
              element.appendChild(scrollMask[0]);
              scrollMask.on('wheel', preventDefault);
              scrollMask.on('touchmove', preventDefault);
              $document.on('keydown', disableKeyNav);
              return function restoreScroll() {
                scrollMask.off('wheel');
                scrollMask.off('touchmove');
                scrollMask[0].parentNode.removeChild(scrollMask[0]);
                $document.off('keydown', disableKeyNav);
                delete $mdUtil.disableScrollAround._enableScrolling;
              };
              // Prevent keypresses from elements inside the body
              // used to stop the keypresses that could cause the page to scroll
              // (arrow keys, spacebar, tab, etc).
              function disableKeyNav(e) {
              }
              function preventDefault(e) {
                e.preventDefault();
              }
            }
            // Converts the body to a position fixed block and translate it to the proper scroll
            // position
            function disableBodyScroll() {
              var htmlNode = body.parentNode;
              var restoreHtmlStyle = htmlNode.getAttribute('style') || '';
              var restoreBodyStyle = body.getAttribute('style') || '';
              var scrollOffset = $mdUtil.scrollTop(body);
              var clientWidth = body.clientWidth;
              if (body.scrollHeight > body.clientHeight + 1) {
                applyStyles(body, {
                  position: 'fixed',
                  width: '100%',
                  top: -scrollOffset + 'px'
                });
                applyStyles(htmlNode, { overflowY: 'scroll' });
              }
              if (body.clientWidth < clientWidth)
                applyStyles(body, { overflow: 'hidden' });
              return function restoreScroll() {
                body.setAttribute('style', restoreBodyStyle);
                htmlNode.setAttribute('style', restoreHtmlStyle);
                body.scrollTop = scrollOffset;
                htmlNode.scrollTop = scrollOffset;
              };
            }
            function applyStyles(el, styles) {
              for (var key in styles) {
                el.style[key] = styles[key];
              }
            }
          },
          enableScrolling: function () {
            var method = this.disableScrollAround._enableScrolling;
            method && method();
          },
          floatingScrollbars: function () {
            if (this.floatingScrollbars.cached === undefined) {
              var tempNode = angular.element('<div style="width: 100%; z-index: -1; position: absolute; height: 35px; overflow-y: scroll"><div style="height: 60px;"></div></div>');
              $document[0].body.appendChild(tempNode[0]);
              this.floatingScrollbars.cached = tempNode[0].offsetWidth == tempNode[0].childNodes[0].offsetWidth;
              tempNode.remove();
            }
            return this.floatingScrollbars.cached;
          },
          forceFocus: function (element) {
            var node = element[0] || element;
            document.addEventListener('click', function focusOnClick(ev) {
              if (ev.target === node && ev.$focus) {
                node.focus();
                ev.stopImmediatePropagation();
                ev.preventDefault();
                node.removeEventListener('click', focusOnClick);
              }
            }, true);
            var newEvent = document.createEvent('MouseEvents');
            newEvent.initMouseEvent('click', false, true, window, {}, 0, 0, 0, 0, false, false, false, false, 0, null);
            newEvent.$material = true;
            newEvent.$focus = true;
            node.dispatchEvent(newEvent);
          },
          createBackdrop: function (scope, addClass) {
            return $compile($mdUtil.supplant('<md-backdrop class="{0}">', [addClass]))(scope);
          },
          supplant: function (template, values, pattern) {
            pattern = pattern || /\{([^\{\}]*)\}/g;
            return template.replace(pattern, function (a, b) {
              var p = b.split('.'), r = values;
              try {
                for (var s in p) {
                  if (p.hasOwnProperty(s)) {
                    r = r[p[s]];
                  }
                }
              } catch (e) {
                r = a;
              }
              return typeof r === 'string' || typeof r === 'number' ? r : a;
            });
          },
          fakeNgModel: function () {
            return {
              $fake: true,
              $setTouched: angular.noop,
              $setViewValue: function (value) {
                this.$viewValue = value;
                this.$render(value);
                this.$viewChangeListeners.forEach(function (cb) {
                  cb();
                });
              },
              $isEmpty: function (value) {
                return ('' + value).length === 0;
              },
              $parsers: [],
              $formatters: [],
              $viewChangeListeners: [],
              $render: angular.noop
            };
          },
          debounce: function (func, wait, scope, invokeApply) {
            var timer;
            return function debounced() {
              var context = scope, args = Array.prototype.slice.call(arguments);
              $timeout.cancel(timer);
              timer = $timeout(function () {
                timer = undefined;
                func.apply(context, args);
              }, wait || 10, invokeApply);
            };
          },
          throttle: function throttle(func, delay) {
            var recent;
            return function throttled() {
              var context = this;
              var args = arguments;
              var now = $mdUtil.now();
              if (!recent || now - recent > delay) {
                func.apply(context, args);
                recent = now;
              }
            };
          },
          time: function time(cb) {
            var start = $mdUtil.now();
            cb();
            return $mdUtil.now() - start;
          },
          valueOnUse: function (scope, key, getter) {
            var value = null, args = Array.prototype.slice.call(arguments);
            var params = args.length > 3 ? args.slice(3) : [];
            Object.defineProperty(scope, key, {
              get: function () {
                if (value === null)
                  value = getter.apply(scope, params);
                return value;
              }
            });
          },
          nextUid: function () {
            return '' + nextUniqueId++;
          },
          validateScope: function (element) {
            var hasScope = element && angular.isDefined(element.scope());
            if (!hasScope) {
              $log.warn('element.scope() is not available when \'debug mode\' == false. @see https://docs.angularjs.org/guide/production!');
            }
            return hasScope;
          },
          disconnectScope: function disconnectScope(scope) {
            if (!scope)
              return;
            // we can't destroy the root scope or a scope that has been already destroyed
            if (scope.$root === scope)
              return;
            if (scope.$$destroyed)
              return;
            var parent = scope.$parent;
            scope.$$disconnected = true;
            // See Scope.$destroy
            if (parent.$$childHead === scope)
              parent.$$childHead = scope.$$nextSibling;
            if (parent.$$childTail === scope)
              parent.$$childTail = scope.$$prevSibling;
            if (scope.$$prevSibling)
              scope.$$prevSibling.$$nextSibling = scope.$$nextSibling;
            if (scope.$$nextSibling)
              scope.$$nextSibling.$$prevSibling = scope.$$prevSibling;
            scope.$$nextSibling = scope.$$prevSibling = null;
          },
          reconnectScope: function reconnectScope(scope) {
            if (!scope)
              return;
            // we can't disconnect the root node or scope already disconnected
            if (scope.$root === scope)
              return;
            if (!scope.$$disconnected)
              return;
            var child = scope;
            var parent = child.$parent;
            child.$$disconnected = false;
            // See Scope.$new for this logic...
            child.$$prevSibling = parent.$$childTail;
            if (parent.$$childHead) {
              parent.$$childTail.$$nextSibling = child;
              parent.$$childTail = child;
            } else {
              parent.$$childHead = parent.$$childTail = child;
            }
          },
          getClosest: function getClosest(el, tagName, onlyParent) {
            if (el instanceof angular.element)
              el = el[0];
            tagName = tagName.toUpperCase();
            if (onlyParent)
              el = el.parentNode;
            if (!el)
              return null;
            do {
              if (el.nodeName === tagName) {
                return el;
              }
            } while (el = el.parentNode);
            return null;
          },
          elementContains: function (node, child) {
            var hasContains = window.Node && window.Node.prototype && Node.prototype.contains;
            var findFn = hasContains ? angular.bind(node, node.contains) : angular.bind(node, function (arg) {
                // compares the positions of two nodes and returns a bitmask
                return node === child || !!(this.compareDocumentPosition(arg) & 16);
              });
            return findFn(child);
          },
          extractElementByName: function (element, nodeName, scanDeep, warnNotFound) {
            var found = scanTree(element);
            if (!found && !!warnNotFound) {
              $log.warn($mdUtil.supplant('Unable to find node \'{0}\' in element \'{1}\'.', [
                nodeName,
                element[0].outerHTML
              ]));
            }
            return angular.element(found || element);
            /**
       * Breadth-First tree scan for element with matching `nodeName`
       */
            function scanTree(element) {
              return scanLevel(element) || (!!scanDeep ? scanChildren(element) : null);
            }
            /**
       * Case-insensitive scan of current elements only (do not descend).
       */
            function scanLevel(element) {
              if (element) {
                for (var i = 0, len = element.length; i < len; i++) {
                  if (element[i].nodeName.toLowerCase() === nodeName) {
                    return element[i];
                  }
                }
              }
              return null;
            }
            /**
       * Scan children of specified node
       */
            function scanChildren(element) {
              var found;
              if (element) {
                for (var i = 0, len = element.length; i < len; i++) {
                  var target = element[i];
                  if (!found) {
                    for (var j = 0, numChild = target.childNodes.length; j < numChild; j++) {
                      found = found || scanTree([target.childNodes[j]]);
                    }
                  }
                }
              }
              return found;
            }
          },
          initOptionalProperties: function (scope, attr, defaults) {
            defaults = defaults || {};
            angular.forEach(scope.$$isolateBindings, function (binding, key) {
              if (binding.optional && angular.isUndefined(scope[key])) {
                var attrIsDefined = angular.isDefined(attr[binding.attrName]);
                scope[key] = angular.isDefined(defaults[key]) ? defaults[key] : attrIsDefined;
              }
            });
          },
          nextTick: function (callback, digest, scope) {
            //-- grab function reference for storing state details
            var nextTick = $mdUtil.nextTick;
            var timeout = nextTick.timeout;
            var queue = nextTick.queue || [];
            //-- add callback to the queue
            queue.push(callback);
            //-- set default value for digest
            if (digest == null)
              digest = true;
            //-- store updated digest/queue values
            nextTick.digest = nextTick.digest || digest;
            nextTick.queue = queue;
            //-- either return existing timeout or create a new one
            return timeout || (nextTick.timeout = $timeout(processQueue, 0, false));
            /**
       * Grab a copy of the current queue
       * Clear the queue for future use
       * Process the existing queue
       * Trigger digest if necessary
       */
            function processQueue() {
              var skip = scope && scope.$$destroyed;
              var queue = !skip ? nextTick.queue : [];
              var digest = !skip ? nextTick.digest : null;
              nextTick.queue = [];
              nextTick.timeout = null;
              nextTick.digest = false;
              queue.forEach(function (callback) {
                callback();
              });
              if (digest)
                $rootScope.$digest();
            }
          },
          processTemplate: function (template) {
            if (usesStandardSymbols) {
              return template;
            } else {
              if (!template || !angular.isString(template))
                return template;
              return template.replace(/\{\{/g, startSymbol).replace(/}}/g, endSymbol);
            }
          },
          getParentWithPointerEvents: function (element) {
            var parent = element.parent();
            // jqLite might return a non-null, but still empty, parent; so check for parent and length
            while (hasComputedStyle(parent, 'pointer-events', 'none')) {
              parent = parent.parent();
            }
            return parent;
          },
          getNearestContentElement: function (element) {
            var current = element.parent()[0];
            // Look for the nearest parent md-content, stopping at the rootElement.
            while (current && current !== $rootElement[0] && current !== document.body && current.nodeName.toUpperCase() !== 'MD-CONTENT') {
              current = current.parentNode;
            }
            return current;
          },
          hasComputedStyle: hasComputedStyle
        };
      // Instantiate other namespace utility methods
      $mdUtil.dom.animator = $$mdAnimate($mdUtil);
      return $mdUtil;
      function getNode(el) {
        return el[0] || el;
      }
    }
    UtilFactory.$inject = [
      '$document',
      '$timeout',
      '$compile',
      '$rootScope',
      '$$mdAnimate',
      '$interpolate',
      '$log',
      '$rootElement',
      '$window'
    ];
    /*
 * Since removing jQuery from the demos, some code that uses `element.focus()` is broken.
 * We need to add `element.focus()`, because it's testable unlike `element[0].focus`.
 */
    angular.element.prototype.focus = angular.element.prototype.focus || function () {
      if (this.length) {
        this[0].focus();
      }
      return this;
    };
    angular.element.prototype.blur = angular.element.prototype.blur || function () {
      if (this.length) {
        this[0].blur();
      }
      return this;
    };
  }());
  (function () {
    'use strict';
    angular.module('material.core').service('$mdAria', AriaService);
    /*
 * @ngInject
 */
    function AriaService($$rAF, $log, $window) {
      return {
        expect: expect,
        expectAsync: expectAsync,
        expectWithText: expectWithText
      };
      /**
   * Check if expected attribute has been specified on the target element or child
   * @param element
   * @param attrName
   * @param {optional} defaultValue What to set the attr to if no value is found
   */
      function expect(element, attrName, defaultValue) {
        var node = angular.element(element)[0] || element;
        // if node exists and neither it nor its children have the attribute
        if (node && ((!node.hasAttribute(attrName) || node.getAttribute(attrName).length === 0) && !childHasAttribute(node, attrName))) {
          defaultValue = angular.isString(defaultValue) ? defaultValue.trim() : '';
          if (defaultValue.length) {
            element.attr(attrName, defaultValue);
          } else {
            $log.warn('ARIA: Attribute "', attrName, '", required for accessibility, is missing on node:', node);
          }
        }
      }
      function expectAsync(element, attrName, defaultValueGetter) {
        // Problem: when retrieving the element's contents synchronously to find the label,
        // the text may not be defined yet in the case of a binding.
        // There is a higher chance that a binding will be defined if we wait one frame.
        $$rAF(function () {
          expect(element, attrName, defaultValueGetter());
        });
      }
      function expectWithText(element, attrName) {
        expectAsync(element, attrName, function () {
          return getText(element);
        });
      }
      function getText(element) {
        return element.text().trim();
      }
      function childHasAttribute(node, attrName) {
        var hasChildren = node.hasChildNodes(), hasAttr = false;
        function isHidden(el) {
          var style = el.currentStyle ? el.currentStyle : $window.getComputedStyle(el);
          return style.display === 'none';
        }
        if (hasChildren) {
          var children = node.childNodes;
          for (var i = 0; i < children.length; i++) {
            var child = children[i];
            if (child.nodeType === 1 && child.hasAttribute(attrName)) {
              if (!isHidden(child)) {
                hasAttr = true;
              }
            }
          }
        }
        return hasAttr;
      }
    }
    AriaService.$inject = [
      '$$rAF',
      '$log',
      '$window'
    ];
  }());
  (function () {
    'use strict';
    angular.module('material.core').service('$mdCompiler', mdCompilerService);
    function mdCompilerService($q, $http, $injector, $compile, $controller, $templateCache) {
      /* jshint validthis: true */
      /*
   * @ngdoc service
   * @name $mdCompiler
   * @module material.core
   * @description
   * The $mdCompiler service is an abstraction of angular's compiler, that allows the developer
   * to easily compile an element with a templateUrl, controller, and locals.
   *
   * @usage
   * <hljs lang="js">
   * $mdCompiler.compile({
   *   templateUrl: 'modal.html',
   *   controller: 'ModalCtrl',
   *   locals: {
   *     modal: myModalInstance;
   *   }
   * }).then(function(compileData) {
   *   compileData.element; // modal.html's template in an element
   *   compileData.link(myScope); //attach controller & scope to element
   * });
   * </hljs>
   */
      /*
    * @ngdoc method
    * @name $mdCompiler#compile
    * @description A helper to compile an HTML template/templateUrl with a given controller,
    * locals, and scope.
    * @param {object} options An options object, with the following properties:
    *
    *    - `controller` - `{(string=|function()=}` Controller fn that should be associated with
    *      newly created scope or the name of a registered controller if passed as a string.
    *    - `controllerAs` - `{string=}` A controller alias name. If present the controller will be
    *      published to scope under the `controllerAs` name.
    *    - `template` - `{string=}` An html template as a string.
    *    - `templateUrl` - `{string=}` A path to an html template.
    *    - `transformTemplate` - `{function(template)=}` A function which transforms the template after
    *      it is loaded. It will be given the template string as a parameter, and should
    *      return a a new string representing the transformed template.
    *    - `resolve` - `{Object.<string, function>=}` - An optional map of dependencies which should
    *      be injected into the controller. If any of these dependencies are promises, the compiler
    *      will wait for them all to be resolved, or if one is rejected before the controller is
    *      instantiated `compile()` will fail..
    *      * `key` - `{string}`: a name of a dependency to be injected into the controller.
    *      * `factory` - `{string|function}`: If `string` then it is an alias for a service.
    *        Otherwise if function, then it is injected and the return value is treated as the
    *        dependency. If the result is a promise, it is resolved before its value is
    *        injected into the controller.
    *
    * @returns {object=} promise A promise, which will be resolved with a `compileData` object.
    * `compileData` has the following properties:
    *
    *   - `element` - `{element}`: an uncompiled element matching the provided template.
    *   - `link` - `{function(scope)}`: A link function, which, when called, will compile
    *     the element and instantiate the provided controller (if given).
    *   - `locals` - `{object}`: The locals which will be passed into the controller once `link` is
    *     called. If `bindToController` is true, they will be coppied to the ctrl instead
    *   - `bindToController` - `bool`: bind the locals to the controller, instead of passing them in.
    */
      this.compile = function (options) {
        var templateUrl = options.templateUrl;
        var template = options.template || '';
        var controller = options.controller;
        var controllerAs = options.controllerAs;
        var resolve = angular.extend({}, options.resolve || {});
        var locals = angular.extend({}, options.locals || {});
        var transformTemplate = options.transformTemplate || angular.identity;
        var bindToController = options.bindToController;
        // Take resolve values and invoke them.
        // Resolves can either be a string (value: 'MyRegisteredAngularConst'),
        // or an invokable 'factory' of sorts: (value: function ValueGetter($dependency) {})
        angular.forEach(resolve, function (value, key) {
          if (angular.isString(value)) {
            resolve[key] = $injector.get(value);
          } else {
            resolve[key] = $injector.invoke(value);
          }
        });
        //Add the locals, which are just straight values to inject
        //eg locals: { three: 3 }, will inject three into the controller
        angular.extend(resolve, locals);
        if (templateUrl) {
          resolve.$template = $http.get(templateUrl, { cache: $templateCache }).then(function (response) {
            return response.data;
          });
        } else {
          resolve.$template = $q.when(template);
        }
        // Wait for all the resolves to finish if they are promises
        return $q.all(resolve).then(function (locals) {
          var compiledData;
          var template = transformTemplate(locals.$template, options);
          var element = options.element || angular.element('<div>').html(template.trim()).contents();
          var linkFn = $compile(element);
          // Return a linking function that can be used later when the element is ready
          return compiledData = {
            locals: locals,
            element: element,
            link: function link(scope) {
              locals.$scope = scope;
              //Instantiate controller if it exists, because we have scope
              if (controller) {
                var invokeCtrl = $controller(controller, locals, true);
                if (bindToController) {
                  angular.extend(invokeCtrl.instance, locals);
                }
                var ctrl = invokeCtrl();
                //See angular-route source for this logic
                element.data('$ngControllerController', ctrl);
                element.children().data('$ngControllerController', ctrl);
                if (controllerAs) {
                  scope[controllerAs] = ctrl;
                }
                // Publish reference to this controller
                compiledData.controller = ctrl;
              }
              return linkFn(scope);
            }
          };
        });
      };
    }
    mdCompilerService.$inject = [
      '$q',
      '$http',
      '$injector',
      '$compile',
      '$controller',
      '$templateCache'
    ];
  }());
  (function () {
    'use strict';
    var HANDLERS = {};
    /* The state of the current 'pointer'
 * The pointer represents the state of the current touch.
 * It contains normalized x and y coordinates from DOM events,
 * as well as other information abstracted from the DOM.
 */
    var pointer, lastPointer, forceSkipClickHijack = false;
    /**
 * The position of the most recent click if that click was on a label element.
 * @type {{x: number, y: number}?}
 */
    var lastLabelClickPos = null;
    // Used to attach event listeners once when multiple ng-apps are running.
    var isInitialized = false;
    angular.module('material.core.gestures', []).provider('$mdGesture', MdGestureProvider).factory('$$MdGestureHandler', MdGestureHandler).run(attachToDocument);
    /**
   * @ngdoc service
   * @name $mdGestureProvider
   * @module material.core.gestures
   *
   * @description
   * In some scenarios on Mobile devices (without jQuery), the click events should NOT be hijacked.
   * `$mdGestureProvider` is used to configure the Gesture module to ignore or skip click hijacking on mobile
   * devices.
   *
   * <hljs lang="js">
   *   app.config(function($mdGestureProvider) {
   *
   *     // For mobile devices without jQuery loaded, do not
   *     // intercept click events during the capture phase.
   *     $mdGestureProvider.skipClickHijack();
   *
   *   });
   * </hljs>
   *
   */
    function MdGestureProvider() {
    }
    MdGestureProvider.prototype = {
      skipClickHijack: function () {
        return forceSkipClickHijack = true;
      },
      $get: [
        '$$MdGestureHandler',
        '$$rAF',
        '$timeout',
        function ($$MdGestureHandler, $$rAF, $timeout) {
          return new MdGesture($$MdGestureHandler, $$rAF, $timeout);
        }
      ]
    };
    /**
 * MdGesture factory construction function
 * @ngInject
 */
    function MdGesture($$MdGestureHandler, $$rAF, $timeout) {
      var userAgent = navigator.userAgent || navigator.vendor || window.opera;
      var isIos = userAgent.match(/ipad|iphone|ipod/i);
      var isAndroid = userAgent.match(/android/i);
      var hasJQuery = typeof window.jQuery !== 'undefined' && angular.element === window.jQuery;
      var self = {
          handler: addHandler,
          register: register,
          isHijackingClicks: (isIos || isAndroid) && !hasJQuery && !forceSkipClickHijack
        };
      if (self.isHijackingClicks) {
        var maxClickDistance = 6;
        self.handler('click', {
          options: { maxDistance: maxClickDistance },
          onEnd: checkDistanceAndEmit('click')
        });
        self.handler('focus', {
          options: { maxDistance: maxClickDistance },
          onEnd: function (ev, pointer) {
            if (pointer.distance < this.state.options.maxDistance) {
              if (canFocus(ev.target)) {
                this.dispatchEvent(ev, 'focus', pointer);
                ev.target.focus();
              }
            }
            function canFocus(element) {
              var focusableElements = [
                  'INPUT',
                  'SELECT',
                  'BUTTON',
                  'TEXTAREA',
                  'VIDEO',
                  'AUDIO'
                ];
              return element.getAttribute('tabindex') != '-1' && !element.hasAttribute('DISABLED') && (element.hasAttribute('tabindex') || element.hasAttribute('href') || focusableElements.indexOf(element.nodeName) != -1);
            }
          }
        });
        self.handler('mouseup', {
          options: { maxDistance: maxClickDistance },
          onEnd: checkDistanceAndEmit('mouseup')
        });
        self.handler('mousedown', {
          onStart: function (ev) {
            this.dispatchEvent(ev, 'mousedown');
          }
        });
      }
      function checkDistanceAndEmit(eventName) {
        return function (ev, pointer) {
          if (pointer.distance < this.state.options.maxDistance) {
            this.dispatchEvent(ev, eventName, pointer);
          }
        };
      }
      /*
   * Register an element to listen for a handler.
   * This allows an element to override the default options for a handler.
   * Additionally, some handlers like drag and hold only dispatch events if
   * the domEvent happens inside an element that's registered to listen for these events.
   *
   * @see GestureHandler for how overriding of default options works.
   * @example $mdGesture.register(myElement, 'drag', { minDistance: 20, horziontal: false })
   */
      function register(element, handlerName, options) {
        var handler = HANDLERS[handlerName.replace(/^\$md./, '')];
        if (!handler) {
          throw new Error('Failed to register element with handler ' + handlerName + '. ' + 'Available handlers: ' + Object.keys(HANDLERS).join(', '));
        }
        return handler.registerElement(element, options);
      }
      /*
   * add a handler to $mdGesture. see below.
   */
      function addHandler(name, definition) {
        var handler = new $$MdGestureHandler(name);
        angular.extend(handler, definition);
        HANDLERS[name] = handler;
        return self;
      }
      /*
   * Register handlers. These listen to touch/start/move events, interpret them,
   * and dispatch gesture events depending on options & conditions. These are all
   * instances of GestureHandler.
   * @see GestureHandler 
   */
      return self.handler('press', {
        onStart: function (ev, pointer) {
          this.dispatchEvent(ev, '$md.pressdown');
        },
        onEnd: function (ev, pointer) {
          this.dispatchEvent(ev, '$md.pressup');
        }
      }).handler('hold', {
        options: {
          maxDistance: 6,
          delay: 500
        },
        onCancel: function () {
          $timeout.cancel(this.state.timeout);
        },
        onStart: function (ev, pointer) {
          // For hold, require a parent to be registered with $mdGesture.register()
          // Because we prevent scroll events, this is necessary.
          if (!this.state.registeredParent)
            return this.cancel();
          this.state.pos = {
            x: pointer.x,
            y: pointer.y
          };
          this.state.timeout = $timeout(angular.bind(this, function holdDelayFn() {
            this.dispatchEvent(ev, '$md.hold');
            this.cancel();  //we're done!
          }), this.state.options.delay, false);
        },
        onMove: function (ev, pointer) {
          // Don't scroll while waiting for hold.
          // If we don't preventDefault touchmove events here, Android will assume we don't
          // want to listen to anymore touch events. It will start scrolling and stop sending
          // touchmove events.
          ev.preventDefault();
          // If the user moves greater than <maxDistance> pixels, stop the hold timer
          // set in onStart
          var dx = this.state.pos.x - pointer.x;
          var dy = this.state.pos.y - pointer.y;
          if (Math.sqrt(dx * dx + dy * dy) > this.options.maxDistance) {
            this.cancel();
          }
        },
        onEnd: function () {
          this.onCancel();
        }
      }).handler('drag', {
        options: {
          minDistance: 6,
          horizontal: true,
          cancelMultiplier: 1.5
        },
        onStart: function (ev) {
          // For drag, require a parent to be registered with $mdGesture.register()
          if (!this.state.registeredParent)
            this.cancel();
        },
        onMove: function (ev, pointer) {
          var shouldStartDrag, shouldCancel;
          // Don't scroll while deciding if this touchmove qualifies as a drag event.
          // If we don't preventDefault touchmove events here, Android will assume we don't
          // want to listen to anymore touch events. It will start scrolling and stop sending
          // touchmove events.
          ev.preventDefault();
          if (!this.state.dragPointer) {
            if (this.state.options.horizontal) {
              shouldStartDrag = Math.abs(pointer.distanceX) > this.state.options.minDistance;
              shouldCancel = Math.abs(pointer.distanceY) > this.state.options.minDistance * this.state.options.cancelMultiplier;
            } else {
              shouldStartDrag = Math.abs(pointer.distanceY) > this.state.options.minDistance;
              shouldCancel = Math.abs(pointer.distanceX) > this.state.options.minDistance * this.state.options.cancelMultiplier;
            }
            if (shouldStartDrag) {
              // Create a new pointer representing this drag, starting at this point where the drag started.
              this.state.dragPointer = makeStartPointer(ev);
              updatePointerState(ev, this.state.dragPointer);
              this.dispatchEvent(ev, '$md.dragstart', this.state.dragPointer);
            } else if (shouldCancel) {
              this.cancel();
            }
          } else {
            this.dispatchDragMove(ev);
          }
        },
        dispatchDragMove: $$rAF.throttle(function (ev) {
          // Make sure the drag didn't stop while waiting for the next frame
          if (this.state.isRunning) {
            updatePointerState(ev, this.state.dragPointer);
            this.dispatchEvent(ev, '$md.drag', this.state.dragPointer);
          }
        }),
        onEnd: function (ev, pointer) {
          if (this.state.dragPointer) {
            updatePointerState(ev, this.state.dragPointer);
            this.dispatchEvent(ev, '$md.dragend', this.state.dragPointer);
          }
        }
      }).handler('swipe', {
        options: {
          minVelocity: 0.65,
          minDistance: 10
        },
        onEnd: function (ev, pointer) {
          var eventType;
          if (Math.abs(pointer.velocityX) > this.state.options.minVelocity && Math.abs(pointer.distanceX) > this.state.options.minDistance) {
            eventType = pointer.directionX == 'left' ? '$md.swipeleft' : '$md.swiperight';
            this.dispatchEvent(ev, eventType);
          } else if (Math.abs(pointer.velocityY) > this.state.options.minVelocity && Math.abs(pointer.distanceY) > this.state.options.minDistance) {
            eventType = pointer.directionY == 'up' ? '$md.swipeup' : '$md.swipedown';
            this.dispatchEvent(ev, eventType);
          }
        }
      });
    }
    MdGesture.$inject = [
      '$$MdGestureHandler',
      '$$rAF',
      '$timeout'
    ];
    /**
 * MdGestureHandler
 * A GestureHandler is an object which is able to dispatch custom dom events
 * based on native dom {touch,pointer,mouse}{start,move,end} events.
 *
 * A gesture will manage its lifecycle through the start,move,end, and cancel
 * functions, which are called by native dom events.
 *
 * A gesture has the concept of 'options' (eg a swipe's required velocity), which can be
 * overridden by elements registering through $mdGesture.register()
 */
    function GestureHandler(name) {
      this.name = name;
      this.state = {};
    }
    function MdGestureHandler() {
      var hasJQuery = typeof window.jQuery !== 'undefined' && angular.element === window.jQuery;
      GestureHandler.prototype = {
        options: {},
        dispatchEvent: hasJQuery ? jQueryDispatchEvent : nativeDispatchEvent,
        onStart: angular.noop,
        onMove: angular.noop,
        onEnd: angular.noop,
        onCancel: angular.noop,
        start: function (ev, pointer) {
          if (this.state.isRunning)
            return;
          var parentTarget = this.getNearestParent(ev.target);
          // Get the options from the nearest registered parent
          var parentTargetOptions = parentTarget && parentTarget.$mdGesture[this.name] || {};
          this.state = {
            isRunning: true,
            options: angular.extend({}, this.options, parentTargetOptions),
            registeredParent: parentTarget
          };
          this.onStart(ev, pointer);
        },
        move: function (ev, pointer) {
          if (!this.state.isRunning)
            return;
          this.onMove(ev, pointer);
        },
        end: function (ev, pointer) {
          if (!this.state.isRunning)
            return;
          this.onEnd(ev, pointer);
          this.state.isRunning = false;
        },
        cancel: function (ev, pointer) {
          this.onCancel(ev, pointer);
          this.state = {};
        },
        getNearestParent: function (node) {
          var current = node;
          while (current) {
            if ((current.$mdGesture || {})[this.name]) {
              return current;
            }
            current = current.parentNode;
          }
          return null;
        },
        registerElement: function (element, options) {
          var self = this;
          element[0].$mdGesture = element[0].$mdGesture || {};
          element[0].$mdGesture[this.name] = options || {};
          element.on('$destroy', onDestroy);
          return onDestroy;
          function onDestroy() {
            delete element[0].$mdGesture[self.name];
            element.off('$destroy', onDestroy);
          }
        }
      };
      return GestureHandler;
      /*
   * Dispatch an event with jQuery
   * TODO: Make sure this sends bubbling events
   *
   * @param srcEvent the original DOM touch event that started this.
   * @param eventType the name of the custom event to send (eg 'click' or '$md.drag')
   * @param eventPointer the pointer object that matches this event.
   */
      function jQueryDispatchEvent(srcEvent, eventType, eventPointer) {
        eventPointer = eventPointer || pointer;
        var eventObj = new angular.element.Event(eventType);
        eventObj.$material = true;
        eventObj.pointer = eventPointer;
        eventObj.srcEvent = srcEvent;
        angular.extend(eventObj, {
          clientX: eventPointer.x,
          clientY: eventPointer.y,
          screenX: eventPointer.x,
          screenY: eventPointer.y,
          pageX: eventPointer.x,
          pageY: eventPointer.y,
          ctrlKey: srcEvent.ctrlKey,
          altKey: srcEvent.altKey,
          shiftKey: srcEvent.shiftKey,
          metaKey: srcEvent.metaKey
        });
        angular.element(eventPointer.target).trigger(eventObj);
      }
      /*
   * NOTE: nativeDispatchEvent is very performance sensitive.
   * @param srcEvent the original DOM touch event that started this.
   * @param eventType the name of the custom event to send (eg 'click' or '$md.drag')
   * @param eventPointer the pointer object that matches this event.
   */
      function nativeDispatchEvent(srcEvent, eventType, eventPointer) {
        eventPointer = eventPointer || pointer;
        var eventObj;
        if (eventType === 'click' || eventType == 'mouseup' || eventType == 'mousedown') {
          eventObj = document.createEvent('MouseEvents');
          eventObj.initMouseEvent(eventType, true, true, window, srcEvent.detail, eventPointer.x, eventPointer.y, eventPointer.x, eventPointer.y, srcEvent.ctrlKey, srcEvent.altKey, srcEvent.shiftKey, srcEvent.metaKey, srcEvent.button, srcEvent.relatedTarget || null);
        } else {
          eventObj = document.createEvent('CustomEvent');
          eventObj.initCustomEvent(eventType, true, true, {});
        }
        eventObj.$material = true;
        eventObj.pointer = eventPointer;
        eventObj.srcEvent = srcEvent;
        eventPointer.target.dispatchEvent(eventObj);
      }
    }
    /**
 * Attach Gestures: hook document and check shouldHijack clicks
 * @ngInject
 */
    function attachToDocument($mdGesture, $$MdGestureHandler) {
      // Polyfill document.contains for IE11.
      // TODO: move to util
      document.contains || (document.contains = function (node) {
        return document.body.contains(node);
      });
      if (!isInitialized && $mdGesture.isHijackingClicks) {
        /*
     * If hijack clicks is true, we preventDefault any click that wasn't
     * sent by ngMaterial. This is because on older Android & iOS, a false, or 'ghost',
     * click event will be sent ~400ms after a touchend event happens.
     * The only way to know if this click is real is to prevent any normal
     * click events, and add a flag to events sent by material so we know not to prevent those.
     * 
     * Two exceptions to click events that should be prevented are:
     *  - click events sent by the keyboard (eg form submit)
     *  - events that originate from an Ionic app
     */
        document.addEventListener('click', clickHijacker, true);
        document.addEventListener('mouseup', mouseInputHijacker, true);
        document.addEventListener('mousedown', mouseInputHijacker, true);
        document.addEventListener('focus', mouseInputHijacker, true);
        isInitialized = true;
      }
      function mouseInputHijacker(ev) {
        var isKeyClick = !ev.clientX && !ev.clientY;
        if (!isKeyClick && !ev.$material && !ev.isIonicTap && !isInputEventFromLabelClick(ev)) {
          ev.preventDefault();
          ev.stopPropagation();
        }
      }
      function clickHijacker(ev) {
        var isKeyClick = ev.clientX === 0 && ev.clientY === 0;
        if (!isKeyClick && !ev.$material && !ev.isIonicTap && !isInputEventFromLabelClick(ev)) {
          ev.preventDefault();
          ev.stopPropagation();
          lastLabelClickPos = null;
        } else {
          lastLabelClickPos = null;
          if (ev.target.tagName.toLowerCase() == 'label') {
            lastLabelClickPos = {
              x: ev.x,
              y: ev.y
            };
          }
        }
      }
      // Listen to all events to cover all platforms.
      var START_EVENTS = 'mousedown touchstart pointerdown';
      var MOVE_EVENTS = 'mousemove touchmove pointermove';
      var END_EVENTS = 'mouseup mouseleave touchend touchcancel pointerup pointercancel';
      angular.element(document).on(START_EVENTS, gestureStart).on(MOVE_EVENTS, gestureMove).on(END_EVENTS, gestureEnd).on('$$mdGestureReset', function gestureClearCache() {
        lastPointer = pointer = null;
      });
      /*
   * When a DOM event happens, run all registered gesture handlers' lifecycle
   * methods which match the DOM event.
   * Eg when a 'touchstart' event happens, runHandlers('start') will call and
   * run `handler.cancel()` and `handler.start()` on all registered handlers.
   */
      function runHandlers(handlerEvent, event) {
        var handler;
        for (var name in HANDLERS) {
          handler = HANDLERS[name];
          if (handler instanceof $$MdGestureHandler) {
            if (handlerEvent === 'start') {
              // Run cancel to reset any handlers' state
              handler.cancel();
            }
            handler[handlerEvent](event, pointer);
          }
        }
      }
      /*
   * gestureStart vets if a start event is legitimate (and not part of a 'ghost click' from iOS/Android)
   * If it is legitimate, we initiate the pointer state and mark the current pointer's type
   * For example, for a touchstart event, mark the current pointer as a 'touch' pointer, so mouse events
   * won't effect it.
   */
      function gestureStart(ev) {
        // If we're already touched down, abort
        if (pointer)
          return;
        var now = +Date.now();
        // iOS & old android bug: after a touch event, a click event is sent 350 ms later.
        // If <400ms have passed, don't allow an event of a different type than the previous event
        if (lastPointer && !typesMatch(ev, lastPointer) && now - lastPointer.endTime < 1500) {
          return;
        }
        pointer = makeStartPointer(ev);
        runHandlers('start', ev);
      }
      /*
   * If a move event happens of the right type, update the pointer and run all the move handlers.
   * "of the right type": if a mousemove happens but our pointer started with a touch event, do nothing.
   */
      function gestureMove(ev) {
        if (!pointer || !typesMatch(ev, pointer))
          return;
        updatePointerState(ev, pointer);
        runHandlers('move', ev);
      }
      /*
   * If an end event happens of the right type, update the pointer, run endHandlers, and save the pointer as 'lastPointer'
   */
      function gestureEnd(ev) {
        if (!pointer || !typesMatch(ev, pointer))
          return;
        updatePointerState(ev, pointer);
        pointer.endTime = +Date.now();
        runHandlers('end', ev);
        lastPointer = pointer;
        pointer = null;
      }
    }
    attachToDocument.$inject = [
      '$mdGesture',
      '$$MdGestureHandler'
    ];
    // ********************
    // Module Functions
    // ********************
    /*
 * Initiate the pointer. x, y, and the pointer's type.
 */
    function makeStartPointer(ev) {
      var point = getEventPoint(ev);
      var startPointer = {
          startTime: +Date.now(),
          target: ev.target,
          type: ev.type.charAt(0)
        };
      startPointer.startX = startPointer.x = point.pageX;
      startPointer.startY = startPointer.y = point.pageY;
      return startPointer;
    }
    /*
 * return whether the pointer's type matches the event's type.
 * Eg if a touch event happens but the pointer has a mouse type, return false.
 */
    function typesMatch(ev, pointer) {
      return ev && pointer && ev.type.charAt(0) === pointer.type;
    }
    /**
 * Gets whether the given event is an input event that was caused by clicking on an
 * associated label element.
 *
 * This is necessary because the browser will, upon clicking on a label element, fire an
 * *extra* click event on its associated input (if any). mdGesture is able to flag the label
 * click as with `$material` correctly, but not the second input click.
 *
 * In order to determine whether an input event is from a label click, we compare the (x, y) for
 * the event to the (x, y) for the most recent label click (which is cleared whenever a non-label
 * click occurs). Unfortunately, there are no event properties that tie the input and the label
 * together (such as relatedTarget).
 *
 * @param {MouseEvent} event
 * @returns {boolean}
 */
    function isInputEventFromLabelClick(event) {
      return lastLabelClickPos && lastLabelClickPos.x == event.x && lastLabelClickPos.y == event.y;
    }
    /*
 * Update the given pointer based upon the given DOMEvent.
 * Distance, velocity, direction, duration, etc
 */
    function updatePointerState(ev, pointer) {
      var point = getEventPoint(ev);
      var x = pointer.x = point.pageX;
      var y = pointer.y = point.pageY;
      pointer.distanceX = x - pointer.startX;
      pointer.distanceY = y - pointer.startY;
      pointer.distance = Math.sqrt(pointer.distanceX * pointer.distanceX + pointer.distanceY * pointer.distanceY);
      pointer.directionX = pointer.distanceX > 0 ? 'right' : pointer.distanceX < 0 ? 'left' : '';
      pointer.directionY = pointer.distanceY > 0 ? 'down' : pointer.distanceY < 0 ? 'up' : '';
      pointer.duration = +Date.now() - pointer.startTime;
      pointer.velocityX = pointer.distanceX / pointer.duration;
      pointer.velocityY = pointer.distanceY / pointer.duration;
    }
    /*
 * Normalize the point where the DOM event happened whether it's touch or mouse.
 * @returns point event obj with pageX and pageY on it.
 */
    function getEventPoint(ev) {
      ev = ev.originalEvent || ev;
      // support jQuery events
      return ev.touches && ev.touches[0] || ev.changedTouches && ev.changedTouches[0] || ev;
    }
  }());
  (function () {
    'use strict';
    angular.module('material.core').provider('$$interimElement', InterimElementProvider);
    /*
 * @ngdoc service
 * @name $$interimElement
 * @module material.core
 *
 * @description
 *
 * Factory that contructs `$$interimElement.$service` services.
 * Used internally in material design for elements that appear on screen temporarily.
 * The service provides a promise-like API for interacting with the temporary
 * elements.
 *
 * ```js
 * app.service('$mdToast', function($$interimElement) {
 *   var $mdToast = $$interimElement(toastDefaultOptions);
 *   return $mdToast;
 * });
 * ```
 * @param {object=} defaultOptions Options used by default for the `show` method on the service.
 *
 * @returns {$$interimElement.$service}
 *
 */
    function InterimElementProvider() {
      createInterimElementProvider.$get = InterimElementFactory;
      InterimElementFactory.$inject = [
        '$document',
        '$q',
        '$$q',
        '$rootScope',
        '$timeout',
        '$rootElement',
        '$animate',
        '$mdUtil',
        '$mdCompiler',
        '$mdTheming',
        '$injector'
      ];
      return createInterimElementProvider;
      /**
   * Returns a new provider which allows configuration of a new interimElement
   * service. Allows configuration of default options & methods for options,
   * as well as configuration of 'preset' methods (eg dialog.basic(): basic is a preset method)
   */
      function createInterimElementProvider(interimFactoryName) {
        var EXPOSED_METHODS = [
            'onHide',
            'onShow',
            'onRemove'
          ];
        var customMethods = {};
        var providerConfig = { presets: {} };
        var provider = {
            setDefaults: setDefaults,
            addPreset: addPreset,
            addMethod: addMethod,
            $get: factory
          };
        /**
     * all interim elements will come with the 'build' preset
     */
        provider.addPreset('build', {
          methods: [
            'controller',
            'controllerAs',
            'resolve',
            'template',
            'templateUrl',
            'themable',
            'transformTemplate',
            'parent'
          ]
        });
        factory.$inject = [
          '$$interimElement',
          '$injector'
        ];
        return provider;
        /**
     * Save the configured defaults to be used when the factory is instantiated
     */
        function setDefaults(definition) {
          providerConfig.optionsFactory = definition.options;
          providerConfig.methods = (definition.methods || []).concat(EXPOSED_METHODS);
          return provider;
        }
        /**
     * Add a method to the factory that isn't specific to any interim element operations
     */
        function addMethod(name, fn) {
          customMethods[name] = fn;
          return provider;
        }
        /**
     * Save the configured preset to be used when the factory is instantiated
     */
        function addPreset(name, definition) {
          definition = definition || {};
          definition.methods = definition.methods || [];
          definition.options = definition.options || function () {
            return {};
          };
          if (/^cancel|hide|show$/.test(name)) {
            throw new Error('Preset \'' + name + '\' in ' + interimFactoryName + ' is reserved!');
          }
          if (definition.methods.indexOf('_options') > -1) {
            throw new Error('Method \'_options\' in ' + interimFactoryName + ' is reserved!');
          }
          providerConfig.presets[name] = {
            methods: definition.methods.concat(EXPOSED_METHODS),
            optionsFactory: definition.options,
            argOption: definition.argOption
          };
          return provider;
        }
        function addPresetMethod(presetName, methodName, method) {
          providerConfig.presets[presetName][methodName] = method;
        }
        /**
     * Create a factory that has the given methods & defaults implementing interimElement
     */
        /* @ngInject */
        function factory($$interimElement, $injector) {
          var defaultMethods;
          var defaultOptions;
          var interimElementService = $$interimElement();
          /*
       * publicService is what the developer will be using.
       * It has methods hide(), cancel(), show(), build(), and any other
       * presets which were set during the config phase.
       */
          var publicService = {
              hide: interimElementService.hide,
              cancel: interimElementService.cancel,
              show: showInterimElement,
              destroy: destroyInterimElement
            };
          defaultMethods = providerConfig.methods || [];
          // This must be invoked after the publicService is initialized
          defaultOptions = invokeFactory(providerConfig.optionsFactory, {});
          // Copy over the simple custom methods
          angular.forEach(customMethods, function (fn, name) {
            publicService[name] = fn;
          });
          angular.forEach(providerConfig.presets, function (definition, name) {
            var presetDefaults = invokeFactory(definition.optionsFactory, {});
            var presetMethods = (definition.methods || []).concat(defaultMethods);
            // Every interimElement built with a preset has a field called `$type`,
            // which matches the name of the preset.
            // Eg in preset 'confirm', options.$type === 'confirm'
            angular.extend(presetDefaults, { $type: name });
            // This creates a preset class which has setter methods for every
            // method given in the `.addPreset()` function, as well as every
            // method given in the `.setDefaults()` function.
            //
            // @example
            // .setDefaults({
            //   methods: ['hasBackdrop', 'clickOutsideToClose', 'escapeToClose', 'targetEvent'],
            //   options: dialogDefaultOptions
            // })
            // .addPreset('alert', {
            //   methods: ['title', 'ok'],
            //   options: alertDialogOptions
            // })
            //
            // Set values will be passed to the options when interimElement.show() is called.
            function Preset(opts) {
              this._options = angular.extend({}, presetDefaults, opts);
            }
            angular.forEach(presetMethods, function (name) {
              Preset.prototype[name] = function (value) {
                this._options[name] = value;
                return this;
              };
            });
            // Create shortcut method for one-linear methods
            if (definition.argOption) {
              var methodName = 'show' + name.charAt(0).toUpperCase() + name.slice(1);
              publicService[methodName] = function (arg) {
                var config = publicService[name](arg);
                return publicService.show(config);
              };
            }
            // eg $mdDialog.alert() will return a new alert preset
            publicService[name] = function (arg) {
              // If argOption is supplied, eg `argOption: 'content'`, then we assume
              // if the argument is not an options object then it is the `argOption` option.
              //
              // @example `$mdToast.simple('hello')` // sets options.content to hello
              //                                     // because argOption === 'content'
              if (arguments.length && definition.argOption && !angular.isObject(arg) && !angular.isArray(arg)) {
                return new Preset()[definition.argOption](arg);
              } else {
                return new Preset(arg);
              }
            };
          });
          return publicService;
          /**
       *
       */
          function showInterimElement(opts) {
            // opts is either a preset which stores its options on an _options field,
            // or just an object made up of options
            opts = opts || {};
            if (opts._options)
              opts = opts._options;
            return interimElementService.show(angular.extend({}, defaultOptions, opts));
          }
          /**
       *  Special method to hide and destroy an interimElement WITHOUT
       *  any 'leave` or hide animations ( an immediate force hide/remove )
       *
       *  NOTE: This calls the onRemove() subclass method for each component...
       *  which must have code to respond to `options.$destroy == true`
       */
          function destroyInterimElement(opts) {
            return interimElementService.destroy(opts);
          }
          /**
       * Helper to call $injector.invoke with a local of the factory name for
       * this provider.
       * If an $mdDialog is providing options for a dialog and tries to inject
       * $mdDialog, a circular dependency error will happen.
       * We get around that by manually injecting $mdDialog as a local.
       */
          function invokeFactory(factory, defaultVal) {
            var locals = {};
            locals[interimFactoryName] = publicService;
            return $injector.invoke(factory || function () {
              return defaultVal;
            }, {}, locals);
          }
        }
      }
      /* @ngInject */
      function InterimElementFactory($document, $q, $$q, $rootScope, $timeout, $rootElement, $animate, $mdUtil, $mdCompiler, $mdTheming, $injector) {
        return function createInterimElementService() {
          var SHOW_CANCELLED = false;
          /*
       * @ngdoc service
       * @name $$interimElement.$service
       *
       * @description
       * A service used to control inserting and removing an element into the DOM.
       *
       */
          var service, stack = [];
          // Publish instance $$interimElement service;
          // ... used as $mdDialog, $mdToast, $mdMenu, and $mdSelect
          return service = {
            show: show,
            hide: hide,
            cancel: cancel,
            destroy: destroy,
            $injector_: $injector
          };
          /*
       * @ngdoc method
       * @name $$interimElement.$service#show
       * @kind function
       *
       * @description
       * Adds the `$interimElement` to the DOM and returns a special promise that will be resolved or rejected
       * with hide or cancel, respectively. To external cancel/hide, developers should use the
       *
       * @param {*} options is hashMap of settings
       * @returns a Promise
       *
       */
          function show(options) {
            options = options || {};
            var interimElement = new InterimElement(options || {});
            var hideExisting = !options.skipHide && stack.length ? service.hide() : $q.when(true);
            // This hide()s only the current interim element before showing the next, new one
            // NOTE: this is not reversible (e.g. interim elements are not stackable)
            hideExisting.finally(function () {
              stack.push(interimElement);
              interimElement.show().catch(function (reason) {
                //$log.error("InterimElement.show() error: " + reason );
                return reason;
              });
            });
            // Return a promise that will be resolved when the interim
            // element is hidden or cancelled...
            return interimElement.deferred.promise;
          }
          /*
       * @ngdoc method
       * @name $$interimElement.$service#hide
       * @kind function
       *
       * @description
       * Removes the `$interimElement` from the DOM and resolves the promise returned from `show`
       *
       * @param {*} resolveParam Data to resolve the promise with
       * @returns a Promise that will be resolved after the element has been removed.
       *
       */
          function hide(reason, options) {
            if (!stack.length)
              return $q.when(reason);
            options = options || {};
            if (options.closeAll) {
              var promise = $q.all(stack.reverse().map(closeElement));
              stack = [];
              return promise;
            } else if (options.closeTo !== undefined) {
              return $q.all(stack.splice(options.closeTo).map(closeElement));
            } else {
              var interim = stack.pop();
              return closeElement(interim);
            }
            function closeElement(interim) {
              interim.remove(reason, false, options || {}).catch(function (reason) {
                //$log.error("InterimElement.hide() error: " + reason );
                return reason;
              });
              return interim.deferred.promise;
            }
          }
          /*
       * @ngdoc method
       * @name $$interimElement.$service#cancel
       * @kind function
       *
       * @description
       * Removes the `$interimElement` from the DOM and rejects the promise returned from `show`
       *
       * @param {*} reason Data to reject the promise with
       * @returns Promise that will be resolved after the element has been removed.
       *
       */
          function cancel(reason, options) {
            var interim = stack.shift();
            if (!interim)
              return $q.when(reason);
            interim.remove(reason, true, options || {}).catch(function (reason) {
              //$log.error("InterimElement.cancel() error: " + reason );
              return reason;
            });
            return interim.deferred.promise;
          }
          /*
       * Special method to quick-remove the interim element without animations
       * Note: interim elements are in "interim containers"
       */
          function destroy(target) {
            var interim = !target ? stack.shift() : null;
            var cntr = angular.element(target).length ? angular.element(target)[0].parentNode : null;
            if (cntr) {
              // Try to find the interim element in the stack which corresponds to the supplied DOM element.
              var filtered = stack.filter(function (entry) {
                  var currNode = entry.options.element[0];
                  return currNode === cntr;
                });
              // Note: this function might be called when the element already has been removed, in which
              //       case we won't find any matches. That's ok.
              if (filtered.length > 0) {
                interim = filtered[0];
                stack.splice(stack.indexOf(interim), 1);
              }
            }
            return interim ? interim.remove(SHOW_CANCELLED, false, { '$destroy': true }) : $q.when(SHOW_CANCELLED);
          }
          /*
       * Internal Interim Element Object
       * Used internally to manage the DOM element and related data
       */
          function InterimElement(options) {
            var self, element, showAction = $q.when(true);
            options = configureScopeAndTransitions(options);
            return self = {
              options: options,
              deferred: $q.defer(),
              show: createAndTransitionIn,
              remove: transitionOutAndRemove
            };
            /**
         * Compile, link, and show this interim element
         * Use optional autoHided and transition-in effects
         */
            function createAndTransitionIn() {
              return $q(function (resolve, reject) {
                compileElement(options).then(function (compiledData) {
                  element = linkElement(compiledData, options);
                  showAction = showElement(element, options, compiledData.controller).then(resolve, rejectAll);
                }, rejectAll);
                function rejectAll(fault) {
                  // Force the '$md<xxx>.show()' promise to reject
                  self.deferred.reject(fault);
                  // Continue rejection propagation
                  reject(fault);
                }
              });
            }
            /**
         * After the show process has finished/rejected:
         * - announce 'removing',
         * - perform the transition-out, and
         * - perform optional clean up scope.
         */
            function transitionOutAndRemove(response, isCancelled, opts) {
              // abort if the show() and compile failed
              if (!element)
                return $q.when(false);
              options = angular.extend(options || {}, opts || {});
              options.cancelAutoHide && options.cancelAutoHide();
              options.element.triggerHandler('$mdInterimElementRemove');
              if (options.$destroy === true) {
                return hideElement(options.element, options).then(function () {
                  isCancelled && rejectAll(response) || resolveAll(response);
                });
              } else {
                $q.when(showAction).finally(function () {
                  hideElement(options.element, options).then(function () {
                    isCancelled && rejectAll(response) || resolveAll(response);
                  }, rejectAll);
                });
                return self.deferred.promise;
              }
              /**
           * The `show()` returns a promise that will be resolved when the interim
           * element is hidden or cancelled...
           */
              function resolveAll(response) {
                self.deferred.resolve(response);
              }
              /**
           * Force the '$md<xxx>.show()' promise to reject
           */
              function rejectAll(fault) {
                self.deferred.reject(fault);
              }
            }
            /**
         * Prepare optional isolated scope and prepare $animate with default enter and leave
         * transitions for the new element instance.
         */
            function configureScopeAndTransitions(options) {
              options = options || {};
              if (options.template) {
                options.template = $mdUtil.processTemplate(options.template);
              }
              return angular.extend({
                preserveScope: false,
                cancelAutoHide: angular.noop,
                scope: options.scope || $rootScope.$new(options.isolateScope),
                onShow: function transitionIn(scope, element, options) {
                  return $animate.enter(element, options.parent);
                },
                onRemove: function transitionOut(scope, element) {
                  // Element could be undefined if a new element is shown before
                  // the old one finishes compiling.
                  return element && $animate.leave(element) || $q.when();
                }
              }, options);
            }
            /**
         * Compile an element with a templateUrl, controller, and locals
         */
            function compileElement(options) {
              var compiled = !options.skipCompile ? $mdCompiler.compile(options) : null;
              return compiled || $q(function (resolve) {
                resolve({
                  locals: {},
                  link: function () {
                    return options.element;
                  }
                });
              });
            }
            /**
         *  Link an element with compiled configuration
         */
            function linkElement(compileData, options) {
              angular.extend(compileData.locals, options);
              var element = compileData.link(options.scope);
              // Search for parent at insertion time, if not specified
              options.element = element;
              options.parent = findParent(element, options);
              if (options.themable)
                $mdTheming(element);
              return element;
            }
            /**
         * Search for parent at insertion time, if not specified
         */
            function findParent(element, options) {
              var parent = options.parent;
              // Search for parent at insertion time, if not specified
              if (angular.isFunction(parent)) {
                parent = parent(options.scope, element, options);
              } else if (angular.isString(parent)) {
                parent = angular.element($document[0].querySelector(parent));
              } else {
                parent = angular.element(parent);
              }
              // If parent querySelector/getter function fails, or it's just null,
              // find a default.
              if (!(parent || {}).length) {
                var el;
                if ($rootElement[0] && $rootElement[0].querySelector) {
                  el = $rootElement[0].querySelector(':not(svg) > body');
                }
                if (!el)
                  el = $rootElement[0];
                if (el.nodeName == '#comment') {
                  el = $document[0].body;
                }
                return angular.element(el);
              }
              return parent;
            }
            /**
         * If auto-hide is enabled, start timer and prepare cancel function
         */
            function startAutoHide() {
              var autoHideTimer, cancelAutoHide = angular.noop;
              if (options.hideDelay) {
                autoHideTimer = $timeout(service.hide, options.hideDelay);
                cancelAutoHide = function () {
                  $timeout.cancel(autoHideTimer);
                };
              }
              // Cache for subsequent use
              options.cancelAutoHide = function () {
                cancelAutoHide();
                options.cancelAutoHide = undefined;
              };
            }
            /**
         * Show the element ( with transitions), notify complete and start
         * optional auto-Hide
         */
            function showElement(element, options, controller) {
              // Trigger onShowing callback before the `show()` starts
              var notifyShowing = options.onShowing || angular.noop;
              // Trigger onComplete callback when the `show()` finishes
              var notifyComplete = options.onComplete || angular.noop;
              notifyShowing(options.scope, element, options, controller);
              return $q(function (resolve, reject) {
                try {
                  // Start transitionIn
                  $q.when(options.onShow(options.scope, element, options, controller)).then(function () {
                    notifyComplete(options.scope, element, options);
                    startAutoHide();
                    resolve(element);
                  }, reject);
                } catch (e) {
                  reject(e.message);
                }
              });
            }
            function hideElement(element, options) {
              var announceRemoving = options.onRemoving || angular.noop;
              return $$q(function (resolve, reject) {
                try {
                  // Start transitionIn
                  var action = $$q.when(options.onRemove(options.scope, element, options) || true);
                  // Trigger callback *before* the remove operation starts
                  announceRemoving(element, action);
                  if (options.$destroy == true) {
                    // For $destroy, onRemove should be synchronous
                    resolve(element);
                  } else {
                    // Wait until transition-out is done
                    action.then(function () {
                      if (!options.preserveScope && options.scope) {
                        options.scope.$destroy();
                      }
                      resolve(element);
                    }, reject);
                  }
                } catch (e) {
                  reject(e.message);
                }
              });
            }
          }
        };
      }
    }
  }());
  (function () {
    'use strict';
    (function () {
      'use strict';
      var $mdUtil, $interpolate, $log;
      var SUFFIXES = /(-gt)?-(sm|md|lg)/g;
      var WHITESPACE = /\s+/g;
      var FLEX_OPTIONS = [
          'grow',
          'initial',
          'auto',
          'none',
          'noshrink',
          'nogrow'
        ];
      var LAYOUT_OPTIONS = [
          'row',
          'column'
        ];
      var ALIGNMENT_MAIN_AXIS = [
          '',
          'start',
          'center',
          'end',
          'stretch',
          'space-around',
          'space-between'
        ];
      var ALIGNMENT_CROSS_AXIS = [
          '',
          'start',
          'center',
          'end',
          'stretch'
        ];
      var config = {
          enabled: true,
          breakpoints: []
        };
      registerLayoutAPI(angular.module('material.core.layout', ['ng']));
      /**
   *   registerLayoutAPI()
   *
   *   The original ngMaterial Layout solution used attribute selectors and CSS.
   *
   *  ```html
   *  <div layout="column"> My Content </div>
   *  ```
   *
   *  ```css
   *  [layout] {
   *    box-sizing: border-box;
   *    display:flex;
   *  }
   *  [layout=column] {
   *    flex-direction : column
   *  }
   *  ```
   *
   *  Use of attribute selectors creates significant performance impacts in some
   *  browsers... mainly IE.
   *
   *  This module registers directives that allow the same layout attributes to be
   *  interpreted and converted to class selectors. The directive will add equivalent classes to each element that
   *  contains a Layout directive.
   *
   * ```html
   *   <div layout="column" class="layout layout-column"> My Content </div>
   *```
   *
   *  ```css
   *  .layout {
   *    box-sizing: border-box;
   *    display:flex;
   *  }
   *  .layout-column {
   *    flex-direction : column
   *  }
   *  ```
   */
      function registerLayoutAPI(module) {
        var PREFIX_REGEXP = /^((?:x|data)[\:\-_])/i;
        var SPECIAL_CHARS_REGEXP = /([\:\-\_]+(.))/g;
        // NOTE: these are also defined in constants::MEDIA_PRIORITY and constants::MEDIA
        var BREAKPOINTS = [
            '',
            'xs',
            'gt-xs',
            'sm',
            'gt-sm',
            'md',
            'gt-md',
            'lg',
            'gt-lg',
            'xl'
          ];
        var API_WITH_VALUES = [
            'layout',
            'flex',
            'flex-order',
            'flex-offset',
            'layout-align'
          ];
        var API_NO_VALUES = [
            'show',
            'hide',
            'layout-padding',
            'layout-margin'
          ];
        // Build directive registration functions for the standard Layout API... for all breakpoints.
        angular.forEach(BREAKPOINTS, function (mqb) {
          // Attribute directives with expected, observable value(s)
          angular.forEach(API_WITH_VALUES, function (name) {
            var fullName = mqb ? name + '-' + mqb : name;
            module.directive(directiveNormalize(fullName), attributeWithObserve(fullName));
          });
          // Attribute directives with no expected value(s)
          angular.forEach(API_NO_VALUES, function (name) {
            var fullName = mqb ? name + '-' + mqb : name;
            module.directive(directiveNormalize(fullName), attributeWithoutValue(fullName));
          });
        });
        // Register other, special directive functions for the Layout features:
        module.directive('mdLayoutCss', disableLayoutDirective).directive('ngCloak', buildCloakInterceptor('ng-cloak')).directive('layoutWrap', attributeWithoutValue('layout-wrap')).directive('layoutNoWrap', attributeWithoutValue('layout-no-wrap')).directive('layoutFill', attributeWithoutValue('layout-fill')).directive('layoutLtMd', warnAttrNotSupported('layout-lt-md', true)).directive('layoutLtLg', warnAttrNotSupported('layout-lt-lg', true)).directive('flexLtMd', warnAttrNotSupported('flex-lt-md', true)).directive('flexLtLg', warnAttrNotSupported('flex-lt-lg', true)).directive('layoutAlignLtMd', warnAttrNotSupported('layout-align-lt-md')).directive('layoutAlignLtLg', warnAttrNotSupported('layout-align-lt-lg')).directive('flexOrderLtMd', warnAttrNotSupported('flex-order-lt-md')).directive('flexOrderLtLg', warnAttrNotSupported('flex-order-lt-lg')).directive('offsetLtMd', warnAttrNotSupported('flex-offset-lt-md')).directive('offsetLtLg', warnAttrNotSupported('flex-offset-lt-lg')).directive('hideLtMd', warnAttrNotSupported('hide-lt-md')).directive('hideLtLg', warnAttrNotSupported('hide-lt-lg')).directive('showLtMd', warnAttrNotSupported('show-lt-md')).directive('showLtLg', warnAttrNotSupported('show-lt-lg'));
        /**
     * Converts snake_case to camelCase.
     * Also there is special case for Moz prefix starting with upper case letter.
     * @param name Name to normalize
     */
        function directiveNormalize(name) {
          return name.replace(PREFIX_REGEXP, '').replace(SPECIAL_CHARS_REGEXP, function (_, separator, letter, offset) {
            return offset ? letter.toUpperCase() : letter;
          });
        }
      }
      /**
   * Special directive that will disable ALL Layout conversions of layout
   * attribute(s) to classname(s).
   *
   * <link rel="stylesheet" href="angular-material.min.css">
   * <link rel="stylesheet" href="angular-material.layout.css">
   *
   * <body md-layout-css>
   *  ...
   * </body>
   *
   * Note: Using md-layout-css directive requires the developer to load the Material
   * Layout Attribute stylesheet (which only uses attribute selectors):
   *
   *       `angular-material.layout.css`
   *
   * Another option is to use the LayoutProvider to configure and disable the attribute
   * conversions; this would obviate the use of the `md-layout-css` directive
   *
   */
      function disableLayoutDirective() {
        return {
          restrict: 'A',
          priority: '900',
          compile: function (element, attr) {
            config.enabled = false;
            return angular.noop;
          }
        };
      }
      /**
   * Tail-hook ngCloak to delay the uncloaking while Layout transformers
   * finish processing. Eliminates flicker with Material.Layoouts
   */
      function buildCloakInterceptor(className) {
        return [
          '$timeout',
          function ($timeout) {
            return {
              restrict: 'A',
              priority: -10,
              compile: function (element) {
                if (!config.enabled)
                  return angular.noop;
                // Re-add the cloak
                element.addClass(className);
                return function (scope, element) {
                  // Wait while layout injectors configure, then uncloak
                  // NOTE: $rAF does not delay enough... and this is a 1x-only event,
                  //       $timeout is acceptable.
                  $timeout(function () {
                    element.removeClass(className);
                  }, 10, false);
                };
              }
            };
          }
        ];
      }
      // *********************************************************************************
      //
      // These functions create registration functions for ngMaterial Layout attribute directives
      // This provides easy translation to switch ngMaterial attribute selectors to
      // CLASS selectors and directives; which has huge performance implications
      // for IE Browsers
      //
      // *********************************************************************************
      /**
   * Creates a directive registration function where a possible dynamic attribute
   * value will be observed/watched.
   * @param {string} className attribute name; eg `layout-gt-md` with value ="row"
   */
      function attributeWithObserve(className) {
        return [
          '$mdUtil',
          '$interpolate',
          '$log',
          function (_$mdUtil_, _$interpolate_, _$log_) {
            $mdUtil = _$mdUtil_;
            $interpolate = _$interpolate_;
            $log = _$log_;
            return {
              restrict: 'A',
              compile: function (element, attr) {
                var linkFn;
                if (config.enabled) {
                  // immediately replace static (non-interpolated) invalid values...
                  validateAttributeUsage(className, attr, element, $log);
                  validateAttributeValue(className, getNormalizedAttrValue(className, attr, ''), buildUpdateFn(element, className, attr));
                  linkFn = translateWithValueToCssClass;
                }
                // Use for postLink to account for transforms after ng-transclude.
                return linkFn || angular.noop;
              }
            };
          }
        ];
        /**
     * Add as transformed class selector(s), then
     * remove the deprecated attribute selector
     */
        function translateWithValueToCssClass(scope, element, attrs) {
          var updateFn = updateClassWithValue(element, className, attrs);
          var unwatch = attrs.$observe(attrs.$normalize(className), updateFn);
          updateFn(getNormalizedAttrValue(className, attrs, ''));
          scope.$on('$destroy', function () {
            unwatch();
          });
        }
      }
      /**
   * Creates a registration function for ngMaterial Layout attribute directive.
   * This is a `simple` transpose of attribute usage to class usage; where we ignore
   * any attribute value
   */
      function attributeWithoutValue(className) {
        return [
          '$mdUtil',
          '$interpolate',
          '$log',
          function (_$mdUtil_, _$interpolate_, _$log_) {
            $mdUtil = _$mdUtil_;
            $interpolate = _$interpolate_;
            $log = _$log_;
            return {
              restrict: 'A',
              compile: function (element, attr) {
                var linkFn;
                if (config.enabled) {
                  // immediately replace static (non-interpolated) invalid values...
                  validateAttributeValue(className, getNormalizedAttrValue(className, attr, ''), buildUpdateFn(element, className, attr));
                  translateToCssClass(null, element);
                  // Use for postLink to account for transforms after ng-transclude.
                  linkFn = translateToCssClass;
                }
                return linkFn || angular.noop;
              }
            };
          }
        ];
        /**
     * Add as transformed class selector, then
     * remove the deprecated attribute selector
     */
        function translateToCssClass(scope, element) {
          element.addClass(className);
        }
      }
      /**
   * After link-phase, do NOT remove deprecated layout attribute selector.
   * Instead watch the attribute so interpolated data-bindings to layout
   * selectors will continue to be supported.
   *
   * $observe() the className and update with new class (after removing the last one)
   *
   * e.g. `layout="{{layoutDemo.direction}}"` will update...
   *
   * NOTE: The value must match one of the specified styles in the CSS.
   * For example `flex-gt-md="{{size}}`  where `scope.size == 47` will NOT work since
   * only breakpoints for 0, 5, 10, 15... 100, 33, 34, 66, 67 are defined.
   *
   */
      function updateClassWithValue(element, className) {
        var lastClass;
        return function updateClassFn(newValue) {
          var value = validateAttributeValue(className, newValue || '');
          if (angular.isDefined(value)) {
            if (lastClass)
              element.removeClass(lastClass);
            lastClass = !value ? className : className + '-' + value.replace(WHITESPACE, '-');
            element.addClass(lastClass);
          }
        };
      }
      /**
   * Provide console warning that this layout attribute has been deprecated
   *
   */
      function warnAttrNotSupported(className) {
        var parts = className.split('-');
        return [
          '$log',
          function ($log) {
            $log.warn(className + 'has been deprecated. Please use a `' + parts[0] + '-gt-<xxx>` variant.');
            return angular.noop;
          }
        ];
      }
      /**
   * Centralize warnings for known flexbox issues (especially IE-related issues)
   */
      function validateAttributeUsage(className, attr, element, $log) {
        var message, usage, url;
        var nodeName = element[0].nodeName.toLowerCase();
        switch (className.replace(SUFFIXES, '')) {
        case 'flex':
          if (nodeName == 'md-button' || nodeName == 'fieldset') {
            // @see https://github.com/philipwalton/flexbugs#9-some-html-elements-cant-be-flex-containers
            // Use <div flex> wrapper inside (preferred) or outside
            usage = '<' + nodeName + ' ' + className + '></' + nodeName + '>';
            url = 'https://github.com/philipwalton/flexbugs#9-some-html-elements-cant-be-flex-containers';
            message = 'Markup \'{0}\' may not work as expected in IE Browsers. Consult \'{1}\' for details.';
            $log.warn($mdUtil.supplant(message, [
              usage,
              url
            ]));
          }
        }
      }
      /**
   * For the Layout attribute value, validate or replace with default
   * fallback value
   */
      function validateAttributeValue(className, value, updateFn) {
        var origValue = value;
        if (!needsInterpolation(value)) {
          switch (className.replace(SUFFIXES, '')) {
          case 'layout':
            if (!findIn(value, LAYOUT_OPTIONS)) {
              value = LAYOUT_OPTIONS[0];  // 'row';
            }
            break;
          case 'flex':
            if (!findIn(value, FLEX_OPTIONS)) {
              if (isNaN(value)) {
                value = '';
              }
            }
            break;
          case 'flex-offset':
          case 'flex-order':
            if (!value || isNaN(+value)) {
              value = '0';
            }
            break;
          case 'layout-align':
            var axis = extractAlignAxis(value);
            value = $mdUtil.supplant('{main}-{cross}', axis);
            break;
          case 'layout-padding':
          case 'layout-margin':
          case 'layout-fill':
          case 'layout-wrap':
          case 'layout-no-wrap':
            value = '';
            break;
          }
          if (value != origValue) {
            (updateFn || angular.noop)(value);
          }
        }
        return value;
      }
      /**
   * Replace current attribute value with fallback value
   */
      function buildUpdateFn(element, className, attrs) {
        return function updateAttrValue(fallback) {
          if (!needsInterpolation(fallback)) {
            // Do not modify the element's attribute value; so
            // uses '<ui-layout layout="/api/sidebar.html" />' will not
            // be affected. Just update the attrs value.
            attrs[attrs.$normalize(className)] = fallback;
          }
        };
      }
      /**
   * See if the original value has interpolation symbols:
   * e.g.  flex-gt-md="{{triggerPoint}}"
   */
      function needsInterpolation(value) {
        return (value || '').indexOf($interpolate.startSymbol()) > -1;
      }
      function getNormalizedAttrValue(className, attrs, defaultVal) {
        var normalizedAttr = attrs.$normalize(className);
        return attrs[normalizedAttr] ? attrs[normalizedAttr].replace(WHITESPACE, '-') : defaultVal || null;
      }
      function findIn(item, list, replaceWith) {
        item = replaceWith && item ? item.replace(WHITESPACE, replaceWith) : item;
        var found = false;
        if (item) {
          list.forEach(function (it) {
            it = replaceWith ? it.replace(WHITESPACE, replaceWith) : it;
            found = found || it === item;
          });
        }
        return found;
      }
      function extractAlignAxis(attrValue) {
        var axis = {
            main: 'start',
            cross: 'stretch'
          }, values;
        attrValue = attrValue || '';
        if (attrValue.indexOf('-') == 0 || attrValue.indexOf(' ') == 0) {
          // For missing main-axis values
          attrValue = 'none' + attrValue;
        }
        values = attrValue.toLowerCase().trim().replace(WHITESPACE, '-').split('-');
        if (values.length && values[0] === 'space') {
          // for main-axis values of "space-around" or "space-between"
          values = [
            values[0] + '-' + values[1],
            values[2]
          ];
        }
        if (values.length > 0)
          axis.main = values[0] || axis.main;
        if (values.length > 1)
          axis.cross = values[1] || axis.cross;
        if (ALIGNMENT_MAIN_AXIS.indexOf(axis.main) < 0)
          axis.main = 'start';
        if (ALIGNMENT_CROSS_AXIS.indexOf(axis.cross) < 0)
          axis.cross = 'stretch';
        return axis;
      }
    }());
  }());
  (function () {
    'use strict';
    /**
   * @ngdoc module
   * @name material.core.componentRegistry
   *
   * @description
   * A component instance registration service.
   * Note: currently this as a private service in the SideNav component.
   */
    angular.module('material.core').factory('$mdComponentRegistry', ComponentRegistry);
    /*
   * @private
   * @ngdoc factory
   * @name ComponentRegistry
   * @module material.core.componentRegistry
   *
   */
    function ComponentRegistry($log, $q) {
      var self;
      var instances = [];
      var pendings = {};
      return self = {
        notFoundError: function (handle) {
          $log.error('No instance found for handle', handle);
        },
        getInstances: function () {
          return instances;
        },
        get: function (handle) {
          if (!isValidID(handle))
            return null;
          var i, j, instance;
          for (i = 0, j = instances.length; i < j; i++) {
            instance = instances[i];
            if (instance.$$mdHandle === handle) {
              return instance;
            }
          }
          return null;
        },
        register: function (instance, handle) {
          if (!handle)
            return angular.noop;
          instance.$$mdHandle = handle;
          instances.push(instance);
          resolveWhen();
          return deregister;
          /**
         * Remove registration for an instance
         */
          function deregister() {
            var index = instances.indexOf(instance);
            if (index !== -1) {
              instances.splice(index, 1);
            }
          }
          /**
         * Resolve any pending promises for this instance
         */
          function resolveWhen() {
            var dfd = pendings[handle];
            if (dfd) {
              dfd.resolve(instance);
              delete pendings[handle];
            }
          }
        },
        when: function (handle) {
          if (isValidID(handle)) {
            var deferred = $q.defer();
            var instance = self.get(handle);
            if (instance) {
              deferred.resolve(instance);
            } else {
              pendings[handle] = deferred;
            }
            return deferred.promise;
          }
          return $q.reject('Invalid `md-component-id` value.');
        }
      };
      function isValidID(handle) {
        return handle && handle !== '';
      }
    }
    ComponentRegistry.$inject = [
      '$log',
      '$q'
    ];
  }());
  (function () {
    'use strict';
    (function () {
      'use strict';
      /**
   * @ngdoc service
   * @name $mdButtonInkRipple
   * @module material.core
   *
   * @description
   * Provides ripple effects for md-button.  See $mdInkRipple service for all possible configuration options.
   *
   * @param {object=} scope Scope within the current context
   * @param {object=} element The element the ripple effect should be applied to
   * @param {object=} options (Optional) Configuration options to override the default ripple configuration
   */
      angular.module('material.core').factory('$mdButtonInkRipple', MdButtonInkRipple);
      function MdButtonInkRipple($mdInkRipple) {
        return {
          attach: function attachRipple(scope, element, options) {
            options = angular.extend(optionsForElement(element), options);
            return $mdInkRipple.attach(scope, element, options);
          }
        };
        function optionsForElement(element) {
          if (element.hasClass('md-icon-button')) {
            return {
              isMenuItem: element.hasClass('md-menu-item'),
              fitRipple: true,
              center: true
            };
          } else {
            return {
              isMenuItem: element.hasClass('md-menu-item'),
              dimBackground: true
            };
          }
        }
        ;
      }
      MdButtonInkRipple.$inject = ['$mdInkRipple'];
      ;
    }());
  }());
  (function () {
    'use strict';
    (function () {
      'use strict';
      /**
   * @ngdoc service
   * @name $mdCheckboxInkRipple
   * @module material.core
   *
   * @description
   * Provides ripple effects for md-checkbox.  See $mdInkRipple service for all possible configuration options.
   *
   * @param {object=} scope Scope within the current context
   * @param {object=} element The element the ripple effect should be applied to
   * @param {object=} options (Optional) Configuration options to override the defaultripple configuration
   */
      angular.module('material.core').factory('$mdCheckboxInkRipple', MdCheckboxInkRipple);
      function MdCheckboxInkRipple($mdInkRipple) {
        return { attach: attach };
        function attach(scope, element, options) {
          return $mdInkRipple.attach(scope, element, angular.extend({
            center: true,
            dimBackground: false,
            fitRipple: true
          }, options));
        }
        ;
      }
      MdCheckboxInkRipple.$inject = ['$mdInkRipple'];
      ;
    }());
  }());
  (function () {
    'use strict';
    (function () {
      'use strict';
      /**
   * @ngdoc service
   * @name $mdListInkRipple
   * @module material.core
   *
   * @description
   * Provides ripple effects for md-list.  See $mdInkRipple service for all possible configuration options.
   *
   * @param {object=} scope Scope within the current context
   * @param {object=} element The element the ripple effect should be applied to
   * @param {object=} options (Optional) Configuration options to override the defaultripple configuration
   */
      angular.module('material.core').factory('$mdListInkRipple', MdListInkRipple);
      function MdListInkRipple($mdInkRipple) {
        return { attach: attach };
        function attach(scope, element, options) {
          return $mdInkRipple.attach(scope, element, angular.extend({
            center: false,
            dimBackground: true,
            outline: false,
            rippleSize: 'full'
          }, options));
        }
        ;
      }
      MdListInkRipple.$inject = ['$mdInkRipple'];
      ;
    }());
  }());
  (function () {
    'use strict';
    /**
 * @ngdoc module
 * @name material.core.ripple
 * @description
 * Ripple
 */
    angular.module('material.core').factory('$mdInkRipple', InkRippleService).directive('mdInkRipple', InkRippleDirective).directive('mdNoInk', attrNoDirective).directive('mdNoBar', attrNoDirective).directive('mdNoStretch', attrNoDirective);
    var DURATION = 450;
    /**
 * @ngdoc directive
 * @name mdInkRipple
 * @module material.core.ripple
 *
 * @description
 * The `md-ink-ripple` directive allows you to specify the ripple color or id a ripple is allowed.
 *
 * @param {string|boolean} md-ink-ripple A color string `#FF0000` or boolean (`false` or `0`) for preventing ripple
 *
 * @usage
 * ### String values
 * <hljs lang="html">
 *   <ANY md-ink-ripple="#FF0000">
 *     Ripples in red
 *   </ANY>
 *
 *   <ANY md-ink-ripple="false">
 *     Not rippling
 *   </ANY>
 * </hljs>
 *
 * ### Interpolated values
 * <hljs lang="html">
 *   <ANY md-ink-ripple="{{ randomColor() }}">
 *     Ripples with the return value of 'randomColor' function
 *   </ANY>
 *
 *   <ANY md-ink-ripple="{{ canRipple() }}">
 *     Ripples if 'canRipple' function return value is not 'false' or '0'
 *   </ANY>
 * </hljs>
 */
    function InkRippleDirective($mdButtonInkRipple, $mdCheckboxInkRipple) {
      return {
        controller: angular.noop,
        link: function (scope, element, attr) {
          attr.hasOwnProperty('mdInkRippleCheckbox') ? $mdCheckboxInkRipple.attach(scope, element) : $mdButtonInkRipple.attach(scope, element);
        }
      };
    }
    InkRippleDirective.$inject = [
      '$mdButtonInkRipple',
      '$mdCheckboxInkRipple'
    ];
    /**
 * @ngdoc service
 * @name $mdInkRipple
 * @module material.core.ripple
 *
 * @description
 * `$mdInkRipple` is a service for adding ripples to any element
 *
 * @usage
 * <hljs lang="js">
 * app.factory('$myElementInkRipple', function($mdInkRipple) {
 *   return {
 *     attach: function (scope, element, options) {
 *       return $mdInkRipple.attach(scope, element, angular.extend({
 *         center: false,
 *         dimBackground: true
 *       }, options));
 *     }
 *   };
 * });
 *
 * app.controller('myController', function ($scope, $element, $myElementInkRipple) {
 *   $scope.onClick = function (ev) {
 *     $myElementInkRipple.attach($scope, angular.element(ev.target), { center: true });
 *   }
 * });
 * </hljs>
 */
    /**
 * @ngdoc method
 * @name $mdInkRipple#attach
 *
 * @description
 * Attaching given scope, element and options to inkRipple controller
 *
 * @param {object=} scope Scope within the current context
 * @param {object=} element The element the ripple effect should be applied to
 * @param {object=} options (Optional) Configuration options to override the defaultRipple configuration
 * * `center` -  Whether the ripple should start from the center of the container element
 * * `dimBackground` - Whether the background should be dimmed with the ripple color
 * * `colorElement` - The element the ripple should take its color from, defined by css property `color`
 * * `fitRipple` - Whether the ripple should fill the element
 */
    function InkRippleService($injector) {
      return { attach: attach };
      function attach(scope, element, options) {
        if (element.controller('mdNoInk'))
          return angular.noop;
        return $injector.instantiate(InkRippleCtrl, {
          $scope: scope,
          $element: element,
          rippleOptions: options
        });
      }
    }
    InkRippleService.$inject = ['$injector'];
    /**
 * Controller used by the ripple service in order to apply ripples
 * @ngInject
 */
    function InkRippleCtrl($scope, $element, rippleOptions, $window, $timeout, $mdUtil) {
      this.$window = $window;
      this.$timeout = $timeout;
      this.$mdUtil = $mdUtil;
      this.$scope = $scope;
      this.$element = $element;
      this.options = rippleOptions;
      this.mousedown = false;
      this.ripples = [];
      this.timeout = null;
      // Stores a reference to the most-recent ripple timeout
      this.lastRipple = null;
      $mdUtil.valueOnUse(this, 'container', this.createContainer);
      this.$element.addClass('md-ink-ripple');
      // attach method for unit tests
      ($element.controller('mdInkRipple') || {}).createRipple = angular.bind(this, this.createRipple);
      ($element.controller('mdInkRipple') || {}).setColor = angular.bind(this, this.color);
      this.bindEvents();
    }
    InkRippleCtrl.$inject = [
      '$scope',
      '$element',
      'rippleOptions',
      '$window',
      '$timeout',
      '$mdUtil'
    ];
    /**
 * Either remove or unlock any remaining ripples when the user mouses off of the element (either by
 * mouseup or mouseleave event)
 */
    function autoCleanup(self, cleanupFn) {
      if (self.mousedown || self.lastRipple) {
        self.mousedown = false;
        self.$mdUtil.nextTick(angular.bind(self, cleanupFn), false);
      }
    }
    /**
 * Returns the color that the ripple should be (either based on CSS or hard-coded)
 * @returns {string}
 */
    InkRippleCtrl.prototype.color = function (value) {
      var self = this;
      // If assigning a color value, apply it to background and the ripple color
      if (angular.isDefined(value)) {
        self._color = self._parseColor(value);
      }
      // If color lookup, use assigned, defined, or inherited
      return self._color || self._parseColor(self.inkRipple()) || self._parseColor(getElementColor());
      /**
   * Finds the color element and returns its text color for use as default ripple color
   * @returns {string}
   */
      function getElementColor() {
        var items = self.options && self.options.colorElement ? self.options.colorElement : [];
        var elem = items.length ? items[0] : self.$element[0];
        return elem ? self.$window.getComputedStyle(elem).color : 'rgb(0,0,0)';
      }
    };
    /**
 * Updating the ripple colors based on the current inkRipple value
 * or the element's computed style color
 */
    InkRippleCtrl.prototype.calculateColor = function () {
      return this.color();
    };
    /**
 * Takes a string color and converts it to RGBA format
 * @param color {string}
 * @param [multiplier] {int}
 * @returns {string}
 */
    InkRippleCtrl.prototype._parseColor = function parseColor(color, multiplier) {
      multiplier = multiplier || 1;
      if (!color)
        return;
      if (color.indexOf('rgba') === 0)
        return color.replace(/\d?\.?\d*\s*\)\s*$/, (0.1 * multiplier).toString() + ')');
      if (color.indexOf('rgb') === 0)
        return rgbToRGBA(color);
      if (color.indexOf('#') === 0)
        return hexToRGBA(color);
      /**
   * Converts hex value to RGBA string
   * @param color {string}
   * @returns {string}
   */
      function hexToRGBA(color) {
        var hex = color[0] === '#' ? color.substr(1) : color, dig = hex.length / 3, red = hex.substr(0, dig), green = hex.substr(dig, dig), blue = hex.substr(dig * 2);
        if (dig === 1) {
          red += red;
          green += green;
          blue += blue;
        }
        return 'rgba(' + parseInt(red, 16) + ',' + parseInt(green, 16) + ',' + parseInt(blue, 16) + ',0.1)';
      }
      /**
   * Converts an RGB color to RGBA
   * @param color {string}
   * @returns {string}
   */
      function rgbToRGBA(color) {
        return color.replace(')', ', 0.1)').replace('(', 'a(');
      }
    };
    /**
 * Binds events to the root element for
 */
    InkRippleCtrl.prototype.bindEvents = function () {
      this.$element.on('mousedown', angular.bind(this, this.handleMousedown));
      this.$element.on('mouseup touchend', angular.bind(this, this.handleMouseup));
      this.$element.on('mouseleave', angular.bind(this, this.handleMouseup));
      this.$element.on('touchmove', angular.bind(this, this.handleTouchmove));
    };
    /**
 * Create a new ripple on every mousedown event from the root element
 * @param event {MouseEvent}
 */
    InkRippleCtrl.prototype.handleMousedown = function (event) {
      if (this.mousedown)
        return;
      // When jQuery is loaded, we have to get the original event
      if (event.hasOwnProperty('originalEvent'))
        event = event.originalEvent;
      this.mousedown = true;
      if (this.options.center) {
        this.createRipple(this.container.prop('clientWidth') / 2, this.container.prop('clientWidth') / 2);
      } else {
        // We need to calculate the relative coordinates if the target is a sublayer of the ripple element
        if (event.srcElement !== this.$element[0]) {
          var layerRect = this.$element[0].getBoundingClientRect();
          var layerX = event.clientX - layerRect.left;
          var layerY = event.clientY - layerRect.top;
          this.createRipple(layerX, layerY);
        } else {
          this.createRipple(event.offsetX, event.offsetY);
        }
      }
    };
    /**
 * Either remove or unlock any remaining ripples when the user mouses off of the element (either by
 * mouseup, touchend or mouseleave event)
 */
    InkRippleCtrl.prototype.handleMouseup = function () {
      autoCleanup(this, this.clearRipples);
    };
    /**
 * Either remove or unlock any remaining ripples when the user mouses off of the element (by
 * touchmove)
 */
    InkRippleCtrl.prototype.handleTouchmove = function () {
      autoCleanup(this, this.deleteRipples);
    };
    /**
 * Cycles through all ripples and attempts to remove them.
 */
    InkRippleCtrl.prototype.deleteRipples = function () {
      for (var i = 0; i < this.ripples.length; i++) {
        this.ripples[i].remove();
      }
    };
    /**
 * Cycles through all ripples and attempts to remove them with fade.
 * Depending on logic within `fadeInComplete`, some removals will be postponed.
 */
    InkRippleCtrl.prototype.clearRipples = function () {
      for (var i = 0; i < this.ripples.length; i++) {
        this.fadeInComplete(this.ripples[i]);
      }
    };
    /**
 * Creates the ripple container element
 * @returns {*}
 */
    InkRippleCtrl.prototype.createContainer = function () {
      var container = angular.element('<div class="md-ripple-container"></div>');
      this.$element.append(container);
      return container;
    };
    InkRippleCtrl.prototype.clearTimeout = function () {
      if (this.timeout) {
        this.$timeout.cancel(this.timeout);
        this.timeout = null;
      }
    };
    InkRippleCtrl.prototype.isRippleAllowed = function () {
      var element = this.$element[0];
      do {
        if (!element.tagName || element.tagName === 'BODY')
          break;
        if (element && angular.isFunction(element.hasAttribute)) {
          if (element.hasAttribute('disabled'))
            return false;
          if (this.inkRipple() === 'false' || this.inkRipple() === '0')
            return false;
        }
      } while (element = element.parentNode);
      return true;
    };
    /**
 * The attribute `md-ink-ripple` may be a static or interpolated
 * color value OR a boolean indicator (used to disable ripples)
 */
    InkRippleCtrl.prototype.inkRipple = function () {
      return this.$element.attr('md-ink-ripple');
    };
    /**
 * Creates a new ripple and adds it to the container.  Also tracks ripple in `this.ripples`.
 * @param left
 * @param top
 */
    InkRippleCtrl.prototype.createRipple = function (left, top) {
      if (!this.isRippleAllowed())
        return;
      var ctrl = this;
      var ripple = angular.element('<div class="md-ripple"></div>');
      var width = this.$element.prop('clientWidth');
      var height = this.$element.prop('clientHeight');
      var x = Math.max(Math.abs(width - left), left) * 2;
      var y = Math.max(Math.abs(height - top), top) * 2;
      var size = getSize(this.options.fitRipple, x, y);
      var color = this.calculateColor();
      ripple.css({
        left: left + 'px',
        top: top + 'px',
        background: 'black',
        width: size + 'px',
        height: size + 'px',
        backgroundColor: rgbaToRGB(color),
        borderColor: rgbaToRGB(color)
      });
      this.lastRipple = ripple;
      // we only want one timeout to be running at a time
      this.clearTimeout();
      this.timeout = this.$timeout(function () {
        ctrl.clearTimeout();
        if (!ctrl.mousedown)
          ctrl.fadeInComplete(ripple);
      }, DURATION * 0.35, false);
      if (this.options.dimBackground)
        this.container.css({ backgroundColor: color });
      this.container.append(ripple);
      this.ripples.push(ripple);
      ripple.addClass('md-ripple-placed');
      this.$mdUtil.nextTick(function () {
        ripple.addClass('md-ripple-scaled md-ripple-active');
        ctrl.$timeout(function () {
          ctrl.clearRipples();
        }, DURATION, false);
      }, false);
      function rgbaToRGB(color) {
        return color ? color.replace('rgba', 'rgb').replace(/,[^\),]+\)/, ')') : 'rgb(0,0,0)';
      }
      function getSize(fit, x, y) {
        return fit ? Math.max(x, y) : Math.sqrt(Math.pow(x, 2) + Math.pow(y, 2));
      }
    };
    /**
 * After fadeIn finishes, either kicks off the fade-out animation or queues the element for removal on mouseup
 * @param ripple
 */
    InkRippleCtrl.prototype.fadeInComplete = function (ripple) {
      if (this.lastRipple === ripple) {
        if (!this.timeout && !this.mousedown) {
          this.removeRipple(ripple);
        }
      } else {
        this.removeRipple(ripple);
      }
    };
    /**
 * Kicks off the animation for removing a ripple
 * @param ripple {Element}
 */
    InkRippleCtrl.prototype.removeRipple = function (ripple) {
      var ctrl = this;
      var index = this.ripples.indexOf(ripple);
      if (index < 0)
        return;
      this.ripples.splice(this.ripples.indexOf(ripple), 1);
      ripple.removeClass('md-ripple-active');
      if (this.ripples.length === 0)
        this.container.css({ backgroundColor: '' });
      // use a 2-second timeout in order to allow for the animation to finish
      // we don't actually care how long the animation takes
      this.$timeout(function () {
        ctrl.fadeOutComplete(ripple);
      }, DURATION, false);
    };
    /**
 * Removes the provided ripple from the DOM
 * @param ripple
 */
    InkRippleCtrl.prototype.fadeOutComplete = function (ripple) {
      ripple.remove();
      this.lastRipple = null;
    };
    /**
 * Used to create an empty directive.  This is used to track flag-directives whose children may have
 * functionality based on them.
 *
 * Example: `md-no-ink` will potentially be used by all child directives.
 */
    function attrNoDirective() {
      return { controller: angular.noop };
    }
  }());
  (function () {
    'use strict';
    (function () {
      'use strict';
      /**
   * @ngdoc service
   * @name $mdTabInkRipple
   * @module material.core
   *
   * @description
   * Provides ripple effects for md-tabs.  See $mdInkRipple service for all possible configuration options.
   *
   * @param {object=} scope Scope within the current context
   * @param {object=} element The element the ripple effect should be applied to
   * @param {object=} options (Optional) Configuration options to override the defaultripple configuration
   */
      angular.module('material.core').factory('$mdTabInkRipple', MdTabInkRipple);
      function MdTabInkRipple($mdInkRipple) {
        return { attach: attach };
        function attach(scope, element, options) {
          return $mdInkRipple.attach(scope, element, angular.extend({
            center: false,
            dimBackground: true,
            outline: false,
            rippleSize: 'full'
          }, options));
        }
        ;
      }
      MdTabInkRipple.$inject = ['$mdInkRipple'];
      ;
    }());
  }());
  (function () {
    'use strict';
    angular.module('material.core.theming.palette', []).constant('$mdColorPalette', {
      'red': {
        '50': '#ffebee',
        '100': '#ffcdd2',
        '200': '#ef9a9a',
        '300': '#e57373',
        '400': '#ef5350',
        '500': '#f44336',
        '600': '#e53935',
        '700': '#d32f2f',
        '800': '#c62828',
        '900': '#b71c1c',
        'A100': '#ff8a80',
        'A200': '#ff5252',
        'A400': '#ff1744',
        'A700': '#d50000',
        'contrastDefaultColor': 'light',
        'contrastDarkColors': '50 100 200 300 A100',
        'contrastStrongLightColors': '400 500 600 700 A200 A400 A700'
      },
      'pink': {
        '50': '#fce4ec',
        '100': '#f8bbd0',
        '200': '#f48fb1',
        '300': '#f06292',
        '400': '#ec407a',
        '500': '#e91e63',
        '600': '#d81b60',
        '700': '#c2185b',
        '800': '#ad1457',
        '900': '#880e4f',
        'A100': '#ff80ab',
        'A200': '#ff4081',
        'A400': '#f50057',
        'A700': '#c51162',
        'contrastDefaultColor': 'light',
        'contrastDarkColors': '50 100 200 A100',
        'contrastStrongLightColors': '500 600 A200 A400 A700'
      },
      'purple': {
        '50': '#f3e5f5',
        '100': '#e1bee7',
        '200': '#ce93d8',
        '300': '#ba68c8',
        '400': '#ab47bc',
        '500': '#9c27b0',
        '600': '#8e24aa',
        '700': '#7b1fa2',
        '800': '#6a1b9a',
        '900': '#4a148c',
        'A100': '#ea80fc',
        'A200': '#e040fb',
        'A400': '#d500f9',
        'A700': '#aa00ff',
        'contrastDefaultColor': 'light',
        'contrastDarkColors': '50 100 200 A100',
        'contrastStrongLightColors': '300 400 A200 A400 A700'
      },
      'deep-purple': {
        '50': '#ede7f6',
        '100': '#d1c4e9',
        '200': '#b39ddb',
        '300': '#9575cd',
        '400': '#7e57c2',
        '500': '#673ab7',
        '600': '#5e35b1',
        '700': '#512da8',
        '800': '#4527a0',
        '900': '#311b92',
        'A100': '#b388ff',
        'A200': '#7c4dff',
        'A400': '#651fff',
        'A700': '#6200ea',
        'contrastDefaultColor': 'light',
        'contrastDarkColors': '50 100 200 A100',
        'contrastStrongLightColors': '300 400 A200'
      },
      'indigo': {
        '50': '#e8eaf6',
        '100': '#c5cae9',
        '200': '#9fa8da',
        '300': '#7986cb',
        '400': '#5c6bc0',
        '500': '#3f51b5',
        '600': '#3949ab',
        '700': '#303f9f',
        '800': '#283593',
        '900': '#1a237e',
        'A100': '#8c9eff',
        'A200': '#536dfe',
        'A400': '#3d5afe',
        'A700': '#304ffe',
        'contrastDefaultColor': 'light',
        'contrastDarkColors': '50 100 200 A100',
        'contrastStrongLightColors': '300 400 A200 A400'
      },
      'blue': {
        '50': '#e3f2fd',
        '100': '#bbdefb',
        '200': '#90caf9',
        '300': '#64b5f6',
        '400': '#42a5f5',
        '500': '#2196f3',
        '600': '#1e88e5',
        '700': '#1976d2',
        '800': '#1565c0',
        '900': '#0d47a1',
        'A100': '#82b1ff',
        'A200': '#448aff',
        'A400': '#2979ff',
        'A700': '#2962ff',
        'contrastDefaultColor': 'light',
        'contrastDarkColors': '50 100 200 300 400 A100',
        'contrastStrongLightColors': '500 600 700 A200 A400 A700'
      },
      'light-blue': {
        '50': '#e1f5fe',
        '100': '#b3e5fc',
        '200': '#81d4fa',
        '300': '#4fc3f7',
        '400': '#29b6f6',
        '500': '#03a9f4',
        '600': '#039be5',
        '700': '#0288d1',
        '800': '#0277bd',
        '900': '#01579b',
        'A100': '#80d8ff',
        'A200': '#40c4ff',
        'A400': '#00b0ff',
        'A700': '#0091ea',
        'contrastDefaultColor': 'dark',
        'contrastLightColors': '600 700 800 900 A700',
        'contrastStrongLightColors': '600 700 800 A700'
      },
      'cyan': {
        '50': '#e0f7fa',
        '100': '#b2ebf2',
        '200': '#80deea',
        '300': '#4dd0e1',
        '400': '#26c6da',
        '500': '#00bcd4',
        '600': '#00acc1',
        '700': '#0097a7',
        '800': '#00838f',
        '900': '#006064',
        'A100': '#84ffff',
        'A200': '#18ffff',
        'A400': '#00e5ff',
        'A700': '#00b8d4',
        'contrastDefaultColor': 'dark',
        'contrastLightColors': '700 800 900',
        'contrastStrongLightColors': '700 800 900'
      },
      'teal': {
        '50': '#e0f2f1',
        '100': '#b2dfdb',
        '200': '#80cbc4',
        '300': '#4db6ac',
        '400': '#26a69a',
        '500': '#009688',
        '600': '#00897b',
        '700': '#00796b',
        '800': '#00695c',
        '900': '#004d40',
        'A100': '#a7ffeb',
        'A200': '#64ffda',
        'A400': '#1de9b6',
        'A700': '#00bfa5',
        'contrastDefaultColor': 'dark',
        'contrastLightColors': '500 600 700 800 900',
        'contrastStrongLightColors': '500 600 700'
      },
      'green': {
        '50': '#e8f5e9',
        '100': '#c8e6c9',
        '200': '#a5d6a7',
        '300': '#81c784',
        '400': '#66bb6a',
        '500': '#4caf50',
        '600': '#43a047',
        '700': '#388e3c',
        '800': '#2e7d32',
        '900': '#1b5e20',
        'A100': '#b9f6ca',
        'A200': '#69f0ae',
        'A400': '#00e676',
        'A700': '#00c853',
        'contrastDefaultColor': 'dark',
        'contrastLightColors': '600 700 800 900',
        'contrastStrongLightColors': '600 700'
      },
      'light-green': {
        '50': '#f1f8e9',
        '100': '#dcedc8',
        '200': '#c5e1a5',
        '300': '#aed581',
        '400': '#9ccc65',
        '500': '#8bc34a',
        '600': '#7cb342',
        '700': '#689f38',
        '800': '#558b2f',
        '900': '#33691e',
        'A100': '#ccff90',
        'A200': '#b2ff59',
        'A400': '#76ff03',
        'A700': '#64dd17',
        'contrastDefaultColor': 'dark',
        'contrastLightColors': '700 800 900',
        'contrastStrongLightColors': '700 800 900'
      },
      'lime': {
        '50': '#f9fbe7',
        '100': '#f0f4c3',
        '200': '#e6ee9c',
        '300': '#dce775',
        '400': '#d4e157',
        '500': '#cddc39',
        '600': '#c0ca33',
        '700': '#afb42b',
        '800': '#9e9d24',
        '900': '#827717',
        'A100': '#f4ff81',
        'A200': '#eeff41',
        'A400': '#c6ff00',
        'A700': '#aeea00',
        'contrastDefaultColor': 'dark',
        'contrastLightColors': '900',
        'contrastStrongLightColors': '900'
      },
      'yellow': {
        '50': '#fffde7',
        '100': '#fff9c4',
        '200': '#fff59d',
        '300': '#fff176',
        '400': '#ffee58',
        '500': '#ffeb3b',
        '600': '#fdd835',
        '700': '#fbc02d',
        '800': '#f9a825',
        '900': '#f57f17',
        'A100': '#ffff8d',
        'A200': '#ffff00',
        'A400': '#ffea00',
        'A700': '#ffd600',
        'contrastDefaultColor': 'dark'
      },
      'amber': {
        '50': '#fff8e1',
        '100': '#ffecb3',
        '200': '#ffe082',
        '300': '#ffd54f',
        '400': '#ffca28',
        '500': '#ffc107',
        '600': '#ffb300',
        '700': '#ffa000',
        '800': '#ff8f00',
        '900': '#ff6f00',
        'A100': '#ffe57f',
        'A200': '#ffd740',
        'A400': '#ffc400',
        'A700': '#ffab00',
        'contrastDefaultColor': 'dark'
      },
      'orange': {
        '50': '#fff3e0',
        '100': '#ffe0b2',
        '200': '#ffcc80',
        '300': '#ffb74d',
        '400': '#ffa726',
        '500': '#ff9800',
        '600': '#fb8c00',
        '700': '#f57c00',
        '800': '#ef6c00',
        '900': '#e65100',
        'A100': '#ffd180',
        'A200': '#ffab40',
        'A400': '#ff9100',
        'A700': '#ff6d00',
        'contrastDefaultColor': 'dark',
        'contrastLightColors': '800 900',
        'contrastStrongLightColors': '800 900'
      },
      'deep-orange': {
        '50': '#fbe9e7',
        '100': '#ffccbc',
        '200': '#ffab91',
        '300': '#ff8a65',
        '400': '#ff7043',
        '500': '#ff5722',
        '600': '#f4511e',
        '700': '#e64a19',
        '800': '#d84315',
        '900': '#bf360c',
        'A100': '#ff9e80',
        'A200': '#ff6e40',
        'A400': '#ff3d00',
        'A700': '#dd2c00',
        'contrastDefaultColor': 'light',
        'contrastDarkColors': '50 100 200 300 400 A100 A200',
        'contrastStrongLightColors': '500 600 700 800 900 A400 A700'
      },
      'brown': {
        '50': '#efebe9',
        '100': '#d7ccc8',
        '200': '#bcaaa4',
        '300': '#a1887f',
        '400': '#8d6e63',
        '500': '#795548',
        '600': '#6d4c41',
        '700': '#5d4037',
        '800': '#4e342e',
        '900': '#3e2723',
        'A100': '#d7ccc8',
        'A200': '#bcaaa4',
        'A400': '#8d6e63',
        'A700': '#5d4037',
        'contrastDefaultColor': 'light',
        'contrastDarkColors': '50 100 200',
        'contrastStrongLightColors': '300 400'
      },
      'grey': {
        '50': '#fafafa',
        '100': '#f5f5f5',
        '200': '#eeeeee',
        '300': '#e0e0e0',
        '400': '#bdbdbd',
        '500': '#9e9e9e',
        '600': '#757575',
        '700': '#616161',
        '800': '#424242',
        '900': '#212121',
        '1000': '#000000',
        'A100': '#ffffff',
        'A200': '#eeeeee',
        'A400': '#bdbdbd',
        'A700': '#616161',
        'contrastDefaultColor': 'dark',
        'contrastLightColors': '600 700 800 900'
      },
      'blue-grey': {
        '50': '#eceff1',
        '100': '#cfd8dc',
        '200': '#b0bec5',
        '300': '#90a4ae',
        '400': '#78909c',
        '500': '#607d8b',
        '600': '#546e7a',
        '700': '#455a64',
        '800': '#37474f',
        '900': '#263238',
        'A100': '#cfd8dc',
        'A200': '#b0bec5',
        'A400': '#78909c',
        'A700': '#455a64',
        'contrastDefaultColor': 'light',
        'contrastDarkColors': '50 100 200 300',
        'contrastStrongLightColors': '400 500'
      }
    });
  }());
  (function () {
    'use strict';
    angular.module('material.core.theming', ['material.core.theming.palette']).directive('mdTheme', ThemingDirective).directive('mdThemable', ThemableDirective).provider('$mdTheming', ThemingProvider).run(generateAllThemes);
    /**
 * @ngdoc service
 * @name $mdThemingProvider
 * @module material.core.theming
 *
 * @description Provider to configure the `$mdTheming` service.
 */
    /**
 * @ngdoc method
 * @name $mdThemingProvider#setDefaultTheme
 * @param {string} themeName Default theme name to be applied to elements. Default value is `default`.
 */
    /**
 * @ngdoc method
 * @name $mdThemingProvider#alwaysWatchTheme
 * @param {boolean} watch Whether or not to always watch themes for changes and re-apply
 * classes when they change. Default is `false`. Enabling can reduce performance.
 */
    /* Some Example Valid Theming Expressions
 * =======================================
 *
 * Intention group expansion: (valid for primary, accent, warn, background)
 *
 * {{primary-100}} - grab shade 100 from the primary palette
 * {{primary-100-0.7}} - grab shade 100, apply opacity of 0.7
 * {{primary-100-contrast}} - grab shade 100's contrast color
 * {{primary-hue-1}} - grab the shade assigned to hue-1 from the primary palette
 * {{primary-hue-1-0.7}} - apply 0.7 opacity to primary-hue-1
 * {{primary-color}} - Generates .md-hue-1, .md-hue-2, .md-hue-3 with configured shades set for each hue
 * {{primary-color-0.7}} - Apply 0.7 opacity to each of the above rules
 * {{primary-contrast}} - Generates .md-hue-1, .md-hue-2, .md-hue-3 with configured contrast (ie. text) color shades set for each hue
 * {{primary-contrast-0.7}} - Apply 0.7 opacity to each of the above rules
 *
 * Foreground expansion: Applies rgba to black/white foreground text
 *
 * {{foreground-1}} - used for primary text
 * {{foreground-2}} - used for secondary text/divider
 * {{foreground-3}} - used for disabled text
 * {{foreground-4}} - used for dividers
 *
 */
    // In memory generated CSS rules; registered by theme.name
    var GENERATED = {};
    // In memory storage of defined themes and color palettes (both loaded by CSS, and user specified)
    var PALETTES;
    var THEMES;
    var DARK_FOREGROUND = {
        name: 'dark',
        '1': 'rgba(0,0,0,0.87)',
        '2': 'rgba(0,0,0,0.54)',
        '3': 'rgba(0,0,0,0.26)',
        '4': 'rgba(0,0,0,0.12)'
      };
    var LIGHT_FOREGROUND = {
        name: 'light',
        '1': 'rgba(255,255,255,1.0)',
        '2': 'rgba(255,255,255,0.7)',
        '3': 'rgba(255,255,255,0.3)',
        '4': 'rgba(255,255,255,0.12)'
      };
    var DARK_SHADOW = '1px 1px 0px rgba(0,0,0,0.4), -1px -1px 0px rgba(0,0,0,0.4)';
    var LIGHT_SHADOW = '';
    var DARK_CONTRAST_COLOR = colorToRgbaArray('rgba(0,0,0,0.87)');
    var LIGHT_CONTRAST_COLOR = colorToRgbaArray('rgba(255,255,255,0.87)');
    var STRONG_LIGHT_CONTRAST_COLOR = colorToRgbaArray('rgb(255,255,255)');
    var THEME_COLOR_TYPES = [
        'primary',
        'accent',
        'warn',
        'background'
      ];
    var DEFAULT_COLOR_TYPE = 'primary';
    // A color in a theme will use these hues by default, if not specified by user.
    var LIGHT_DEFAULT_HUES = {
        'accent': {
          'default': 'A200',
          'hue-1': 'A100',
          'hue-2': 'A400',
          'hue-3': 'A700'
        },
        'background': {
          'default': 'A100',
          'hue-1': '300',
          'hue-2': '800',
          'hue-3': '900'
        }
      };
    var DARK_DEFAULT_HUES = {
        'background': {
          'default': '800',
          'hue-1': '600',
          'hue-2': '300',
          'hue-3': '900'
        }
      };
    THEME_COLOR_TYPES.forEach(function (colorType) {
      // Color types with unspecified default hues will use these default hue values
      var defaultDefaultHues = {
          'default': '500',
          'hue-1': '300',
          'hue-2': '800',
          'hue-3': 'A100'
        };
      if (!LIGHT_DEFAULT_HUES[colorType])
        LIGHT_DEFAULT_HUES[colorType] = defaultDefaultHues;
      if (!DARK_DEFAULT_HUES[colorType])
        DARK_DEFAULT_HUES[colorType] = defaultDefaultHues;
    });
    var VALID_HUE_VALUES = [
        '50',
        '100',
        '200',
        '300',
        '400',
        '500',
        '600',
        '700',
        '800',
        '900',
        'A100',
        'A200',
        'A400',
        'A700'
      ];
    // Whether or not themes are to be generated on-demand (vs. eagerly).
    var generateOnDemand = false;
    function ThemingProvider($mdColorPalette) {
      PALETTES = {};
      THEMES = {};
      var themingProvider;
      var defaultTheme = 'default';
      var alwaysWatchTheme = false;
      // Load JS Defined Palettes
      angular.extend(PALETTES, $mdColorPalette);
      // Default theme defined in core.js
      ThemingService.$inject = [
        '$rootScope',
        '$log'
      ];
      return themingProvider = {
        definePalette: definePalette,
        extendPalette: extendPalette,
        theme: registerTheme,
        setDefaultTheme: function (theme) {
          defaultTheme = theme;
        },
        alwaysWatchTheme: function (alwaysWatch) {
          alwaysWatchTheme = alwaysWatch;
        },
        generateThemesOnDemand: function (onDemand) {
          generateOnDemand = onDemand;
        },
        $get: ThemingService,
        _LIGHT_DEFAULT_HUES: LIGHT_DEFAULT_HUES,
        _DARK_DEFAULT_HUES: DARK_DEFAULT_HUES,
        _PALETTES: PALETTES,
        _THEMES: THEMES,
        _parseRules: parseRules,
        _rgba: rgba
      };
      // Example: $mdThemingProvider.definePalette('neonRed', { 50: '#f5fafa', ... });
      function definePalette(name, map) {
        map = map || {};
        PALETTES[name] = checkPaletteValid(name, map);
        return themingProvider;
      }
      // Returns an new object which is a copy of a given palette `name` with variables from
      // `map` overwritten
      // Example: var neonRedMap = $mdThemingProvider.extendPalette('red', { 50: '#f5fafafa' });
      function extendPalette(name, map) {
        return checkPaletteValid(name, angular.extend({}, PALETTES[name] || {}, map));
      }
      // Make sure that palette has all required hues
      function checkPaletteValid(name, map) {
        var missingColors = VALID_HUE_VALUES.filter(function (field) {
            return !map[field];
          });
        if (missingColors.length) {
          throw new Error('Missing colors %1 in palette %2!'.replace('%1', missingColors.join(', ')).replace('%2', name));
        }
        return map;
      }
      // Register a theme (which is a collection of color palettes to use with various states
      // ie. warn, accent, primary )
      // Optionally inherit from an existing theme
      // $mdThemingProvider.theme('custom-theme').primaryPalette('red');
      function registerTheme(name, inheritFrom) {
        if (THEMES[name])
          return THEMES[name];
        inheritFrom = inheritFrom || 'default';
        var parentTheme = typeof inheritFrom === 'string' ? THEMES[inheritFrom] : inheritFrom;
        var theme = new Theme(name);
        if (parentTheme) {
          angular.forEach(parentTheme.colors, function (color, colorType) {
            theme.colors[colorType] = {
              name: color.name,
              hues: angular.extend({}, color.hues)
            };
          });
        }
        THEMES[name] = theme;
        return theme;
      }
      function Theme(name) {
        var self = this;
        self.name = name;
        self.colors = {};
        self.dark = setDark;
        setDark(false);
        function setDark(isDark) {
          isDark = arguments.length === 0 ? true : !!isDark;
          // If no change, abort
          if (isDark === self.isDark)
            return;
          self.isDark = isDark;
          self.foregroundPalette = self.isDark ? LIGHT_FOREGROUND : DARK_FOREGROUND;
          self.foregroundShadow = self.isDark ? DARK_SHADOW : LIGHT_SHADOW;
          // Light and dark themes have different default hues.
          // Go through each existing color type for this theme, and for every
          // hue value that is still the default hue value from the previous light/dark setting,
          // set it to the default hue value from the new light/dark setting.
          var newDefaultHues = self.isDark ? DARK_DEFAULT_HUES : LIGHT_DEFAULT_HUES;
          var oldDefaultHues = self.isDark ? LIGHT_DEFAULT_HUES : DARK_DEFAULT_HUES;
          angular.forEach(newDefaultHues, function (newDefaults, colorType) {
            var color = self.colors[colorType];
            var oldDefaults = oldDefaultHues[colorType];
            if (color) {
              for (var hueName in color.hues) {
                if (color.hues[hueName] === oldDefaults[hueName]) {
                  color.hues[hueName] = newDefaults[hueName];
                }
              }
            }
          });
          return self;
        }
        THEME_COLOR_TYPES.forEach(function (colorType) {
          var defaultHues = (self.isDark ? DARK_DEFAULT_HUES : LIGHT_DEFAULT_HUES)[colorType];
          self[colorType + 'Palette'] = function setPaletteType(paletteName, hues) {
            var color = self.colors[colorType] = {
                name: paletteName,
                hues: angular.extend({}, defaultHues, hues)
              };
            Object.keys(color.hues).forEach(function (name) {
              if (!defaultHues[name]) {
                throw new Error('Invalid hue name \'%1\' in theme %2\'s %3 color %4. Available hue names: %4'.replace('%1', name).replace('%2', self.name).replace('%3', paletteName).replace('%4', Object.keys(defaultHues).join(', ')));
              }
            });
            Object.keys(color.hues).map(function (key) {
              return color.hues[key];
            }).forEach(function (hueValue) {
              if (VALID_HUE_VALUES.indexOf(hueValue) == -1) {
                throw new Error('Invalid hue value \'%1\' in theme %2\'s %3 color %4. Available hue values: %5'.replace('%1', hueValue).replace('%2', self.name).replace('%3', colorType).replace('%4', paletteName).replace('%5', VALID_HUE_VALUES.join(', ')));
              }
            });
            return self;
          };
          self[colorType + 'Color'] = function () {
            var args = Array.prototype.slice.call(arguments);
            console.warn('$mdThemingProviderTheme.' + colorType + 'Color() has been deprecated. ' + 'Use $mdThemingProviderTheme.' + colorType + 'Palette() instead.');
            return self[colorType + 'Palette'].apply(self, args);
          };
        });
      }
      /**
   * @ngdoc service
   * @name $mdTheming
   *
   * @description
   *
   * Service that makes an element apply theming related classes to itself.
   *
   * ```js
   * app.directive('myFancyDirective', function($mdTheming) {
   *   return {
   *     restrict: 'e',
   *     link: function(scope, el, attrs) {
   *       $mdTheming(el);
   *     }
   *   };
   * });
   * ```
   * @param {el=} element to apply theming to
   */
      /* @ngInject */
      function ThemingService($rootScope, $log) {
        applyTheme.inherit = function (el, parent) {
          var ctrl = parent.controller('mdTheme');
          var attrThemeValue = el.attr('md-theme-watch');
          if ((alwaysWatchTheme || angular.isDefined(attrThemeValue)) && attrThemeValue != 'false') {
            var deregisterWatch = $rootScope.$watch(function () {
                return ctrl && ctrl.$mdTheme || (defaultTheme == 'default' ? '' : defaultTheme);
              }, changeTheme);
            el.on('$destroy', deregisterWatch);
          } else {
            var theme = ctrl && ctrl.$mdTheme || (defaultTheme == 'default' ? '' : defaultTheme);
            changeTheme(theme);
          }
          function changeTheme(theme) {
            if (!theme)
              return;
            if (!registered(theme)) {
              $log.warn('Attempted to use unregistered theme \'' + theme + '\'. ' + 'Register it with $mdThemingProvider.theme().');
            }
            var oldTheme = el.data('$mdThemeName');
            if (oldTheme)
              el.removeClass('md-' + oldTheme + '-theme');
            el.addClass('md-' + theme + '-theme');
            el.data('$mdThemeName', theme);
            if (ctrl) {
              el.data('$mdThemeController', ctrl);
            }
          }
        };
        applyTheme.THEMES = angular.extend({}, THEMES);
        applyTheme.defaultTheme = function () {
          return defaultTheme;
        };
        applyTheme.registered = registered;
        applyTheme.generateTheme = generateTheme;
        return applyTheme;
        function registered(themeName) {
          if (themeName === undefined || themeName === '')
            return true;
          return applyTheme.THEMES[themeName] !== undefined;
        }
        function applyTheme(scope, el) {
          // Allow us to be invoked via a linking function signature.
          if (el === undefined) {
            el = scope;
            scope = undefined;
          }
          if (scope === undefined) {
            scope = $rootScope;
          }
          applyTheme.inherit(el, el);
        }
      }
    }
    ThemingProvider.$inject = ['$mdColorPalette'];
    function ThemingDirective($mdTheming, $interpolate, $log) {
      return {
        priority: 100,
        link: {
          pre: function (scope, el, attrs) {
            var ctrl = {
                $setTheme: function (theme) {
                  if (!$mdTheming.registered(theme)) {
                    $log.warn('attempted to use unregistered theme \'' + theme + '\'');
                  }
                  ctrl.$mdTheme = theme;
                }
              };
            el.data('$mdThemeController', ctrl);
            ctrl.$setTheme($interpolate(attrs.mdTheme)(scope));
            attrs.$observe('mdTheme', ctrl.$setTheme);
          }
        }
      };
    }
    ThemingDirective.$inject = [
      '$mdTheming',
      '$interpolate',
      '$log'
    ];
    function ThemableDirective($mdTheming) {
      return $mdTheming;
    }
    ThemableDirective.$inject = ['$mdTheming'];
    function parseRules(theme, colorType, rules) {
      checkValidPalette(theme, colorType);
      rules = rules.replace(/THEME_NAME/g, theme.name);
      var generatedRules = [];
      var color = theme.colors[colorType];
      var themeNameRegex = new RegExp('.md-' + theme.name + '-theme', 'g');
      // Matches '{{ primary-color }}', etc
      var hueRegex = new RegExp('(\'|")?{{\\s*(' + colorType + ')-(color|contrast)-?(\\d\\.?\\d*)?\\s*}}("|\')?', 'g');
      var simpleVariableRegex = /'?"?\{\{\s*([a-zA-Z]+)-(A?\d+|hue\-[0-3]|shadow)-?(\d\.?\d*)?(contrast)?\s*\}\}'?"?/g;
      var palette = PALETTES[color.name];
      // find and replace simple variables where we use a specific hue, not an entire palette
      // eg. "{{primary-100}}"
      //\(' + THEME_COLOR_TYPES.join('\|') + '\)'
      rules = rules.replace(simpleVariableRegex, function (match, colorType, hue, opacity, contrast) {
        if (colorType === 'foreground') {
          if (hue == 'shadow') {
            return theme.foregroundShadow;
          } else {
            return theme.foregroundPalette[hue] || theme.foregroundPalette['1'];
          }
        }
        if (hue.indexOf('hue') === 0) {
          hue = theme.colors[colorType].hues[hue];
        }
        return rgba((PALETTES[theme.colors[colorType].name][hue] || '')[contrast ? 'contrast' : 'value'], opacity);
      });
      // For each type, generate rules for each hue (ie. default, md-hue-1, md-hue-2, md-hue-3)
      angular.forEach(color.hues, function (hueValue, hueName) {
        var newRule = rules.replace(hueRegex, function (match, _, colorType, hueType, opacity) {
            return rgba(palette[hueValue][hueType === 'color' ? 'value' : 'contrast'], opacity);
          });
        if (hueName !== 'default') {
          newRule = newRule.replace(themeNameRegex, '.md-' + theme.name + '-theme.md-' + hueName);
        }
        // Don't apply a selector rule to the default theme, making it easier to override
        // styles of the base-component
        if (theme.name == 'default') {
          var themeRuleRegex = /((?:(?:(?: |>|\.|\w|-|:|\(|\)|\[|\]|"|'|=)+) )?)((?:(?:\w|\.|-)+)?)\.md-default-theme((?: |>|\.|\w|-|:|\(|\)|\[|\]|"|'|=)*)/g;
          newRule = newRule.replace(themeRuleRegex, function (match, prefix, target, suffix) {
            return match + ', ' + prefix + target + suffix;
          });
        }
        generatedRules.push(newRule);
      });
      return generatedRules;
    }
    var rulesByType = {};
    // Generate our themes at run time given the state of THEMES and PALETTES
    function generateAllThemes($injector) {
      var head = document.head;
      var firstChild = head ? head.firstElementChild : null;
      var themeCss = $injector.has('$MD_THEME_CSS') ? $injector.get('$MD_THEME_CSS') : '';
      if (!firstChild)
        return;
      if (themeCss.length === 0)
        return;
      // no rules, so no point in running this expensive task
      // Expose contrast colors for palettes to ensure that text is always readable
      angular.forEach(PALETTES, sanitizePalette);
      // MD_THEME_CSS is a string generated by the build process that includes all the themable
      // components as templates
      // Break the CSS into individual rules
      var rules = themeCss.split(/\}(?!(\}|'|"|;))/).filter(function (rule) {
          return rule && rule.length;
        }).map(function (rule) {
          return rule.trim() + '}';
        });
      var ruleMatchRegex = new RegExp('md-(' + THEME_COLOR_TYPES.join('|') + ')', 'g');
      THEME_COLOR_TYPES.forEach(function (type) {
        rulesByType[type] = '';
      });
      // Sort the rules based on type, allowing us to do color substitution on a per-type basis
      rules.forEach(function (rule) {
        var match = rule.match(ruleMatchRegex);
        // First: test that if the rule has '.md-accent', it goes into the accent set of rules
        for (var i = 0, type; type = THEME_COLOR_TYPES[i]; i++) {
          if (rule.indexOf('.md-' + type) > -1) {
            return rulesByType[type] += rule;
          }
        }
        // If no eg 'md-accent' class is found, try to just find 'accent' in the rule and guess from
        // there
        for (i = 0; type = THEME_COLOR_TYPES[i]; i++) {
          if (rule.indexOf(type) > -1) {
            return rulesByType[type] += rule;
          }
        }
        // Default to the primary array
        return rulesByType[DEFAULT_COLOR_TYPE] += rule;
      });
      // If themes are being generated on-demand, quit here. The user will later manually
      // call generateTheme to do this on a theme-by-theme basis.
      if (generateOnDemand)
        return;
      angular.forEach(THEMES, function (theme) {
        if (!GENERATED[theme.name]) {
          generateTheme(theme.name);
        }
      });
      // *************************
      // Internal functions
      // *************************
      // The user specifies a 'default' contrast color as either light or dark,
      // then explicitly lists which hues are the opposite contrast (eg. A100 has dark, A200 has light)
      function sanitizePalette(palette) {
        var defaultContrast = palette.contrastDefaultColor;
        var lightColors = palette.contrastLightColors || [];
        var strongLightColors = palette.contrastStrongLightColors || [];
        var darkColors = palette.contrastDarkColors || [];
        // These colors are provided as space-separated lists
        if (typeof lightColors === 'string')
          lightColors = lightColors.split(' ');
        if (typeof strongLightColors === 'string')
          strongLightColors = strongLightColors.split(' ');
        if (typeof darkColors === 'string')
          darkColors = darkColors.split(' ');
        // Cleanup after ourselves
        delete palette.contrastDefaultColor;
        delete palette.contrastLightColors;
        delete palette.contrastStrongLightColors;
        delete palette.contrastDarkColors;
        // Change { 'A100': '#fffeee' } to { 'A100': { value: '#fffeee', contrast:DARK_CONTRAST_COLOR }
        angular.forEach(palette, function (hueValue, hueName) {
          if (angular.isObject(hueValue))
            return;
          // Already converted
          // Map everything to rgb colors
          var rgbValue = colorToRgbaArray(hueValue);
          if (!rgbValue) {
            throw new Error('Color %1, in palette %2\'s hue %3, is invalid. Hex or rgb(a) color expected.'.replace('%1', hueValue).replace('%2', palette.name).replace('%3', hueName));
          }
          palette[hueName] = {
            value: rgbValue,
            contrast: getContrastColor()
          };
          function getContrastColor() {
            if (defaultContrast === 'light') {
              if (darkColors.indexOf(hueName) > -1) {
                return DARK_CONTRAST_COLOR;
              } else {
                return strongLightColors.indexOf(hueName) > -1 ? STRONG_LIGHT_CONTRAST_COLOR : LIGHT_CONTRAST_COLOR;
              }
            } else {
              if (lightColors.indexOf(hueName) > -1) {
                return strongLightColors.indexOf(hueName) > -1 ? STRONG_LIGHT_CONTRAST_COLOR : LIGHT_CONTRAST_COLOR;
              } else {
                return DARK_CONTRAST_COLOR;
              }
            }
          }
        });
      }
    }
    generateAllThemes.$inject = ['$injector'];
    function generateTheme(name) {
      var theme = THEMES[name];
      var head = document.head;
      var firstChild = head ? head.firstElementChild : null;
      if (!GENERATED[name]) {
        // For each theme, use the color palettes specified for
        // `primary`, `warn` and `accent` to generate CSS rules.
        THEME_COLOR_TYPES.forEach(function (colorType) {
          var styleStrings = parseRules(theme, colorType, rulesByType[colorType]);
          while (styleStrings.length) {
            var styleContent = styleStrings.shift();
            if (styleContent) {
              var style = document.createElement('style');
              style.setAttribute('md-theme-style', '');
              style.appendChild(document.createTextNode(styleContent));
              head.insertBefore(style, firstChild);
            }
          }
        });
        if (theme.colors.primary.name == theme.colors.accent.name) {
          console.warn('$mdThemingProvider: Using the same palette for primary and' + ' accent. This violates the material design spec.');
        }
        GENERATED[theme.name] = true;
      }
    }
    function checkValidPalette(theme, colorType) {
      // If theme attempts to use a palette that doesnt exist, throw error
      if (!PALETTES[(theme.colors[colorType] || {}).name]) {
        throw new Error('You supplied an invalid color palette for theme %1\'s %2 palette. Available palettes: %3'.replace('%1', theme.name).replace('%2', colorType).replace('%3', Object.keys(PALETTES).join(', ')));
      }
    }
    function colorToRgbaArray(clr) {
      if (angular.isArray(clr) && clr.length == 3)
        return clr;
      if (/^rgb/.test(clr)) {
        return clr.replace(/(^\s*rgba?\(|\)\s*$)/g, '').split(',').map(function (value, i) {
          return i == 3 ? parseFloat(value, 10) : parseInt(value, 10);
        });
      }
      if (clr.charAt(0) == '#')
        clr = clr.substring(1);
      if (!/^([a-fA-F0-9]{3}){1,2}$/g.test(clr))
        return;
      var dig = clr.length / 3;
      var red = clr.substr(0, dig);
      var grn = clr.substr(dig, dig);
      var blu = clr.substr(dig * 2);
      if (dig === 1) {
        red += red;
        grn += grn;
        blu += blu;
      }
      return [
        parseInt(red, 16),
        parseInt(grn, 16),
        parseInt(blu, 16)
      ];
    }
    function rgba(rgbArray, opacity) {
      if (!rgbArray)
        return 'rgb(\'0,0,0\')';
      if (rgbArray.length == 4) {
        rgbArray = angular.copy(rgbArray);
        opacity ? rgbArray.pop() : opacity = rgbArray.pop();
      }
      return opacity && (typeof opacity == 'number' || typeof opacity == 'string' && opacity.length) ? 'rgba(' + rgbArray.join(',') + ',' + opacity + ')' : 'rgb(' + rgbArray.join(',') + ')';
    }
  }());
  (function () {
    'use strict';
    // Polyfill angular < 1.4 (provide $animateCss)
    angular.module('material.core').factory('$$mdAnimate', [
      '$q',
      '$timeout',
      '$mdConstant',
      '$animateCss',
      function ($q, $timeout, $mdConstant, $animateCss) {
        // Since $$mdAnimate is injected into $mdUtil... use a wrapper function
        // to subsequently inject $mdUtil as an argument to the AnimateDomUtils
        return function ($mdUtil) {
          return AnimateDomUtils($mdUtil, $q, $timeout, $mdConstant, $animateCss);
        };
      }
    ]);
    /**
 * Factory function that requires special injections
 */
    function AnimateDomUtils($mdUtil, $q, $timeout, $mdConstant, $animateCss) {
      var self;
      return self = {
        translate3d: function (target, from, to, options) {
          return $animateCss(target, {
            from: from,
            to: to,
            addClass: options.transitionInClass
          }).start().then(function () {
            // Resolve with reverser function...
            return reverseTranslate;
          });
          /**
       * Specific reversal of the request translate animation above...
       */
          function reverseTranslate(newFrom) {
            return $animateCss(target, {
              to: newFrom || from,
              addClass: options.transitionOutClass,
              removeClass: options.transitionInClass
            }).start();
          }
        },
        waitTransitionEnd: function (element, opts) {
          var TIMEOUT = 3000;
          // fallback is 3 secs
          return $q(function (resolve, reject) {
            opts = opts || {};
            var timer = $timeout(finished, opts.timeout || TIMEOUT);
            element.on($mdConstant.CSS.TRANSITIONEND, finished);
            /**
           * Upon timeout or transitionEnd, reject or resolve (respectively) this promise.
           * NOTE: Make sure this transitionEnd didn't bubble up from a child
           */
            function finished(ev) {
              if (ev && ev.target !== element[0])
                return;
              if (ev)
                $timeout.cancel(timer);
              element.off($mdConstant.CSS.TRANSITIONEND, finished);
              // Never reject since ngAnimate may cause timeouts due missed transitionEnd events
              resolve();
            }
          });
        },
        calculateZoomToOrigin: function (element, originator) {
          var origin = originator.element;
          var bounds = originator.bounds;
          var zoomTemplate = 'translate3d( {centerX}px, {centerY}px, 0 ) scale( {scaleX}, {scaleY} )';
          var buildZoom = angular.bind(null, $mdUtil.supplant, zoomTemplate);
          var zoomStyle = buildZoom({
              centerX: 0,
              centerY: 0,
              scaleX: 0.5,
              scaleY: 0.5
            });
          if (origin || bounds) {
            var originBnds = origin ? self.clientRect(origin) || currentBounds() : self.copyRect(bounds);
            var dialogRect = self.copyRect(element[0].getBoundingClientRect());
            var dialogCenterPt = self.centerPointFor(dialogRect);
            var originCenterPt = self.centerPointFor(originBnds);
            // Build the transform to zoom from the dialog center to the origin center
            zoomStyle = buildZoom({
              centerX: originCenterPt.x - dialogCenterPt.x,
              centerY: originCenterPt.y - dialogCenterPt.y,
              scaleX: Math.round(100 * Math.min(0.5, originBnds.width / dialogRect.width)) / 100,
              scaleY: Math.round(100 * Math.min(0.5, originBnds.height / dialogRect.height)) / 100
            });
          }
          return zoomStyle;
          /**
       * This is a fallback if the origin information is no longer valid, then the
       * origin bounds simply becomes the current bounds for the dialogContainer's parent
       */
          function currentBounds() {
            var cntr = element ? element.parent() : null;
            var parent = cntr ? cntr.parent() : null;
            return parent ? self.clientRect(parent) : null;
          }
        },
        toCss: function (raw) {
          var css = {};
          var lookups = 'left top right bottom width height x y min-width min-height max-width max-height';
          angular.forEach(raw, function (value, key) {
            if (angular.isUndefined(value))
              return;
            if (lookups.indexOf(key) >= 0) {
              css[key] = value + 'px';
            } else {
              switch (key) {
              case 'transition':
                convertToVendor(key, $mdConstant.CSS.TRANSITION, value);
                break;
              case 'transform':
                convertToVendor(key, $mdConstant.CSS.TRANSFORM, value);
                break;
              case 'transformOrigin':
                convertToVendor(key, $mdConstant.CSS.TRANSFORM_ORIGIN, value);
                break;
              }
            }
          });
          return css;
          function convertToVendor(key, vendor, value) {
            angular.forEach(vendor.split(' '), function (key) {
              css[key] = value;
            });
          }
        },
        toTransformCss: function (transform, addTransition, transition) {
          var css = {};
          angular.forEach($mdConstant.CSS.TRANSFORM.split(' '), function (key) {
            css[key] = transform;
          });
          if (addTransition) {
            transition = transition || 'all 0.4s cubic-bezier(0.25, 0.8, 0.25, 1) !important';
            css['transition'] = transition;
          }
          return css;
        },
        copyRect: function (source, destination) {
          if (!source)
            return null;
          destination = destination || {};
          angular.forEach('left top right bottom width height'.split(' '), function (key) {
            destination[key] = Math.round(source[key]);
          });
          destination.width = destination.width || destination.right - destination.left;
          destination.height = destination.height || destination.bottom - destination.top;
          return destination;
        },
        clientRect: function (element) {
          var bounds = angular.element(element)[0].getBoundingClientRect();
          var isPositiveSizeClientRect = function (rect) {
            return rect && rect.width > 0 && rect.height > 0;
          };
          // If the event origin element has zero size, it has probably been hidden.
          return isPositiveSizeClientRect(bounds) ? self.copyRect(bounds) : null;
        },
        centerPointFor: function (targetRect) {
          return targetRect ? {
            x: Math.round(targetRect.left + targetRect.width / 2),
            y: Math.round(targetRect.top + targetRect.height / 2)
          } : {
            x: 0,
            y: 0
          };
        }
      };
    }
    ;
  }());
  (function () {
    'use strict';
    'use strict';
    if (angular.version.minor >= 4) {
      angular.module('material.core.animate', []);
    } else {
      (function () {
        var forEach = angular.forEach;
        var WEBKIT = angular.isDefined(document.documentElement.style.WebkitAppearance);
        var TRANSITION_PROP = WEBKIT ? 'WebkitTransition' : 'transition';
        var ANIMATION_PROP = WEBKIT ? 'WebkitAnimation' : 'animation';
        var PREFIX = WEBKIT ? '-webkit-' : '';
        var TRANSITION_EVENTS = (WEBKIT ? 'webkitTransitionEnd ' : '') + 'transitionend';
        var ANIMATION_EVENTS = (WEBKIT ? 'webkitAnimationEnd ' : '') + 'animationend';
        var $$ForceReflowFactory = [
            '$document',
            function ($document) {
              return function () {
                return $document[0].body.clientWidth + 1;
              };
            }
          ];
        var $$rAFMutexFactory = [
            '$$rAF',
            function ($$rAF) {
              return function () {
                var passed = false;
                $$rAF(function () {
                  passed = true;
                });
                return function (fn) {
                  passed ? fn() : $$rAF(fn);
                };
              };
            }
          ];
        var $$AnimateRunnerFactory = [
            '$q',
            '$$rAFMutex',
            function ($q, $$rAFMutex) {
              var INITIAL_STATE = 0;
              var DONE_PENDING_STATE = 1;
              var DONE_COMPLETE_STATE = 2;
              function AnimateRunner(host) {
                this.setHost(host);
                this._doneCallbacks = [];
                this._runInAnimationFrame = $$rAFMutex();
                this._state = 0;
              }
              AnimateRunner.prototype = {
                setHost: function (host) {
                  this.host = host || {};
                },
                done: function (fn) {
                  if (this._state === DONE_COMPLETE_STATE) {
                    fn();
                  } else {
                    this._doneCallbacks.push(fn);
                  }
                },
                progress: angular.noop,
                getPromise: function () {
                  if (!this.promise) {
                    var self = this;
                    this.promise = $q(function (resolve, reject) {
                      self.done(function (status) {
                        status === false ? reject() : resolve();
                      });
                    });
                  }
                  return this.promise;
                },
                then: function (resolveHandler, rejectHandler) {
                  return this.getPromise().then(resolveHandler, rejectHandler);
                },
                'catch': function (handler) {
                  return this.getPromise()['catch'](handler);
                },
                'finally': function (handler) {
                  return this.getPromise()['finally'](handler);
                },
                pause: function () {
                  if (this.host.pause) {
                    this.host.pause();
                  }
                },
                resume: function () {
                  if (this.host.resume) {
                    this.host.resume();
                  }
                },
                end: function () {
                  if (this.host.end) {
                    this.host.end();
                  }
                  this._resolve(true);
                },
                cancel: function () {
                  if (this.host.cancel) {
                    this.host.cancel();
                  }
                  this._resolve(false);
                },
                complete: function (response) {
                  var self = this;
                  if (self._state === INITIAL_STATE) {
                    self._state = DONE_PENDING_STATE;
                    self._runInAnimationFrame(function () {
                      self._resolve(response);
                    });
                  }
                },
                _resolve: function (response) {
                  if (this._state !== DONE_COMPLETE_STATE) {
                    forEach(this._doneCallbacks, function (fn) {
                      fn(response);
                    });
                    this._doneCallbacks.length = 0;
                    this._state = DONE_COMPLETE_STATE;
                  }
                }
              };
              return AnimateRunner;
            }
          ];
        angular.module('material.core.animate', []).factory('$$forceReflow', $$ForceReflowFactory).factory('$$AnimateRunner', $$AnimateRunnerFactory).factory('$$rAFMutex', $$rAFMutexFactory).factory('$animateCss', [
          '$window',
          '$$rAF',
          '$$AnimateRunner',
          '$$forceReflow',
          '$$jqLite',
          '$timeout',
          function ($window, $$rAF, $$AnimateRunner, $$forceReflow, $$jqLite, $timeout) {
            function init(element, options) {
              var temporaryStyles = [];
              var node = getDomNode(element);
              if (options.transitionStyle) {
                temporaryStyles.push([
                  PREFIX + 'transition',
                  options.transitionStyle
                ]);
              }
              if (options.keyframeStyle) {
                temporaryStyles.push([
                  PREFIX + 'animation',
                  options.keyframeStyle
                ]);
              }
              if (options.delay) {
                temporaryStyles.push([
                  PREFIX + 'transition-delay',
                  options.delay + 's'
                ]);
              }
              if (options.duration) {
                temporaryStyles.push([
                  PREFIX + 'transition-duration',
                  options.duration + 's'
                ]);
              }
              var hasCompleteStyles = options.keyframeStyle || options.to && (options.duration > 0 || options.transitionStyle);
              var hasCompleteClasses = !!options.addClass || !!options.removeClass;
              var hasCompleteAnimation = hasCompleteStyles || hasCompleteClasses;
              blockTransition(element, true);
              applyAnimationFromStyles(element, options);
              var animationClosed = false;
              var events, eventFn;
              return {
                close: $window.close,
                start: function () {
                  var runner = new $$AnimateRunner();
                  waitUntilQuiet(function () {
                    blockTransition(element, false);
                    if (!hasCompleteAnimation) {
                      return close();
                    }
                    forEach(temporaryStyles, function (entry) {
                      var key = entry[0];
                      var value = entry[1];
                      node.style[camelCase(key)] = value;
                    });
                    applyClasses(element, options);
                    var timings = computeTimings(element);
                    if (timings.duration === 0) {
                      return close();
                    }
                    var moreStyles = [];
                    if (options.easing) {
                      if (timings.transitionDuration) {
                        moreStyles.push([
                          PREFIX + 'transition-timing-function',
                          options.easing
                        ]);
                      }
                      if (timings.animationDuration) {
                        moreStyles.push([
                          PREFIX + 'animation-timing-function',
                          options.easing
                        ]);
                      }
                    }
                    if (options.delay && timings.animationDelay) {
                      moreStyles.push([
                        PREFIX + 'animation-delay',
                        options.delay + 's'
                      ]);
                    }
                    if (options.duration && timings.animationDuration) {
                      moreStyles.push([
                        PREFIX + 'animation-duration',
                        options.duration + 's'
                      ]);
                    }
                    forEach(moreStyles, function (entry) {
                      var key = entry[0];
                      var value = entry[1];
                      node.style[camelCase(key)] = value;
                      temporaryStyles.push(entry);
                    });
                    var maxDelay = timings.delay;
                    var maxDelayTime = maxDelay * 1000;
                    var maxDuration = timings.duration;
                    var maxDurationTime = maxDuration * 1000;
                    var startTime = Date.now();
                    events = [];
                    if (timings.transitionDuration) {
                      events.push(TRANSITION_EVENTS);
                    }
                    if (timings.animationDuration) {
                      events.push(ANIMATION_EVENTS);
                    }
                    events = events.join(' ');
                    eventFn = function (event) {
                      event.stopPropagation();
                      var ev = event.originalEvent || event;
                      var timeStamp = ev.timeStamp || Date.now();
                      var elapsedTime = parseFloat(ev.elapsedTime.toFixed(3));
                      if (Math.max(timeStamp - startTime, 0) >= maxDelayTime && elapsedTime >= maxDuration) {
                        close();
                      }
                    };
                    element.on(events, eventFn);
                    applyAnimationToStyles(element, options);
                    $timeout(close, maxDelayTime + maxDurationTime * 1.5, false);
                  });
                  return runner;
                  function close() {
                    if (animationClosed)
                      return;
                    animationClosed = true;
                    if (events && eventFn) {
                      element.off(events, eventFn);
                    }
                    applyClasses(element, options);
                    applyAnimationStyles(element, options);
                    forEach(temporaryStyles, function (entry) {
                      node.style[camelCase(entry[0])] = '';
                    });
                    runner.complete(true);
                    return runner;
                  }
                }
              };
            }
            function applyClasses(element, options) {
              if (options.addClass) {
                $$jqLite.addClass(element, options.addClass);
                options.addClass = null;
              }
              if (options.removeClass) {
                $$jqLite.removeClass(element, options.removeClass);
                options.removeClass = null;
              }
            }
            function computeTimings(element) {
              var node = getDomNode(element);
              var cs = $window.getComputedStyle(node);
              var tdr = parseMaxTime(cs[prop('transitionDuration')]);
              var adr = parseMaxTime(cs[prop('animationDuration')]);
              var tdy = parseMaxTime(cs[prop('transitionDelay')]);
              var ady = parseMaxTime(cs[prop('animationDelay')]);
              adr *= parseInt(cs[prop('animationIterationCount')], 10) || 1;
              var duration = Math.max(adr, tdr);
              var delay = Math.max(ady, tdy);
              return {
                duration: duration,
                delay: delay,
                animationDuration: adr,
                transitionDuration: tdr,
                animationDelay: ady,
                transitionDelay: tdy
              };
              function prop(key) {
                return WEBKIT ? 'Webkit' + key.charAt(0).toUpperCase() + key.substr(1) : key;
              }
            }
            function parseMaxTime(str) {
              var maxValue = 0;
              var values = (str || '').split(/\s*,\s*/);
              forEach(values, function (value) {
                // it's always safe to consider only second values and omit `ms` values since
                // getComputedStyle will always handle the conversion for us
                if (value.charAt(value.length - 1) == 's') {
                  value = value.substring(0, value.length - 1);
                }
                value = parseFloat(value) || 0;
                maxValue = maxValue ? Math.max(value, maxValue) : value;
              });
              return maxValue;
            }
            var cancelLastRAFRequest;
            var rafWaitQueue = [];
            function waitUntilQuiet(callback) {
              if (cancelLastRAFRequest) {
                cancelLastRAFRequest();  //cancels the request
              }
              rafWaitQueue.push(callback);
              cancelLastRAFRequest = $$rAF(function () {
                cancelLastRAFRequest = null;
                // DO NOT REMOVE THIS LINE OR REFACTOR OUT THE `pageWidth` variable.
                // PLEASE EXAMINE THE `$$forceReflow` service to understand why.
                var pageWidth = $$forceReflow();
                // we use a for loop to ensure that if the queue is changed
                // during this looping then it will consider new requests
                for (var i = 0; i < rafWaitQueue.length; i++) {
                  rafWaitQueue[i](pageWidth);
                }
                rafWaitQueue.length = 0;
              });
            }
            function applyAnimationStyles(element, options) {
              applyAnimationFromStyles(element, options);
              applyAnimationToStyles(element, options);
            }
            function applyAnimationFromStyles(element, options) {
              if (options.from) {
                element.css(options.from);
                options.from = null;
              }
            }
            function applyAnimationToStyles(element, options) {
              if (options.to) {
                element.css(options.to);
                options.to = null;
              }
            }
            function getDomNode(element) {
              for (var i = 0; i < element.length; i++) {
                if (element[i].nodeType === 1)
                  return element[i];
              }
            }
            function blockTransition(element, bool) {
              var node = getDomNode(element);
              var key = camelCase(PREFIX + 'transition-delay');
              node.style[key] = bool ? '-9999s' : '';
            }
            return init;
          }
        ]);
        /**
   * Older browsers [FF31] expect camelCase
   * property keys.
   * e.g.
   *  animation-duration --> animationDuration
   */
        function camelCase(str) {
          return str.replace(/-[a-z]/g, function (str) {
            return str.charAt(1).toUpperCase();
          });
        }
      }());
    }
  }());
  (function () {
    'use strict';
    /**
 * @ngdoc module
 * @name material.components.autocomplete
 */
    /*
 * @see js folder for autocomplete implementation
 */
    angular.module('material.components.autocomplete', [
      'material.core',
      'material.components.icon',
      'material.components.virtualRepeat'
    ]);
  }());
  (function () {
    'use strict';
    /*
 * @ngdoc module
 * @name material.components.backdrop
 * @description Backdrop
 */
    /**
 * @ngdoc directive
 * @name mdBackdrop
 * @module material.components.backdrop
 *
 * @restrict E
 *
 * @description
 * `<md-backdrop>` is a backdrop element used by other components, such as dialog and bottom sheet.
 * Apply class `opaque` to make the backdrop use the theme backdrop color.
 *
 */
    angular.module('material.components.backdrop', ['material.core']).directive('mdBackdrop', [
      '$mdTheming',
      '$animate',
      '$rootElement',
      '$window',
      '$log',
      '$$rAF',
      '$document',
      function BackdropDirective($mdTheming, $animate, $rootElement, $window, $log, $$rAF, $document) {
        var ERROR_CSS_POSITION = '<md-backdrop> may not work properly in a scrolled, static-positioned parent container.';
        return {
          restrict: 'E',
          link: postLink
        };
        function postLink(scope, element, attrs) {
          // If body scrolling has been disabled using mdUtil.disableBodyScroll(),
          // adjust the 'backdrop' height to account for the fixed 'body' top offset
          var body = $window.getComputedStyle($document[0].body);
          if (body.position == 'fixed') {
            var hViewport = parseInt(body.height, 10) + Math.abs(parseInt(body.top, 10));
            element.css({ height: hViewport + 'px' });
          }
          // backdrop may be outside the $rootElement, tell ngAnimate to animate regardless
          if ($animate.pin)
            $animate.pin(element, $rootElement);
          $$rAF(function () {
            // Often $animate.enter() is used to append the backDrop element
            // so let's wait until $animate is done...
            var parent = element.parent()[0];
            if (parent) {
              if (parent.nodeName == 'BODY') {
                element.css({ position: 'fixed' });
              }
              var styles = $window.getComputedStyle(parent);
              if (styles.position == 'static') {
                // backdrop uses position:absolute and will not work properly with parent position:static (default)
                $log.warn(ERROR_CSS_POSITION);
              }
            }
            $mdTheming.inherit(element, element.parent());
          });
        }
      }
    ]);
  }());
  (function () {
    'use strict';
    /**
 * @ngdoc module
 * @name material.components.bottomSheet
 * @description
 * BottomSheet
 */
    angular.module('material.components.bottomSheet', [
      'material.core',
      'material.components.backdrop'
    ]).directive('mdBottomSheet', MdBottomSheetDirective).provider('$mdBottomSheet', MdBottomSheetProvider);
    /* @ngInject */
    function MdBottomSheetDirective($mdBottomSheet) {
      return {
        restrict: 'E',
        link: function postLink(scope, element, attr) {
          // When navigation force destroys an interimElement, then
          // listen and $destroy() that interim instance...
          scope.$on('$destroy', function () {
            $mdBottomSheet.destroy();
          });
        }
      };
    }
    MdBottomSheetDirective.$inject = ['$mdBottomSheet'];
    /**
 * @ngdoc service
 * @name $mdBottomSheet
 * @module material.components.bottomSheet
 *
 * @description
 * `$mdBottomSheet` opens a bottom sheet over the app and provides a simple promise API.
 *
 * ## Restrictions
 *
 * - The bottom sheet's template must have an outer `<md-bottom-sheet>` element.
 * - Add the `md-grid` class to the bottom sheet for a grid layout.
 * - Add the `md-list` class to the bottom sheet for a list layout.
 *
 * @usage
 * <hljs lang="html">
 * <div ng-controller="MyController">
 *   <md-button ng-click="openBottomSheet()">
 *     Open a Bottom Sheet!
 *   </md-button>
 * </div>
 * </hljs>
 * <hljs lang="js">
 * var app = angular.module('app', ['ngMaterial']);
 * app.controller('MyController', function($scope, $mdBottomSheet) {
 *   $scope.openBottomSheet = function() {
 *     $mdBottomSheet.show({
 *       template: '<md-bottom-sheet>Hello!</md-bottom-sheet>'
 *     });
 *   };
 * });
 * </hljs>
 */
    /**
 * @ngdoc method
 * @name $mdBottomSheet#show
 *
 * @description
 * Show a bottom sheet with the specified options.
 *
 * @param {object} options An options object, with the following properties:
 *
 *   - `templateUrl` - `{string=}`: The url of an html template file that will
 *   be used as the content of the bottom sheet. Restrictions: the template must
 *   have an outer `md-bottom-sheet` element.
 *   - `template` - `{string=}`: Same as templateUrl, except this is an actual
 *   template string.
 *   - `scope` - `{object=}`: the scope to link the template / controller to. If none is specified, it will create a new child scope.
 *     This scope will be destroyed when the bottom sheet is removed unless `preserveScope` is set to true.
 *   - `preserveScope` - `{boolean=}`: whether to preserve the scope when the element is removed. Default is false
 *   - `controller` - `{string=}`: The controller to associate with this bottom sheet.
 *   - `locals` - `{string=}`: An object containing key/value pairs. The keys will
 *   be used as names of values to inject into the controller. For example,
 *   `locals: {three: 3}` would inject `three` into the controller with the value
 *   of 3.
 *   - `clickOutsideToClose` - `{boolean=}`: Whether the user can click outside the bottom sheet to
 *     close it. Default true.
 *   - `escapeToClose` - `{boolean=}`: Whether the user can press escape to close the bottom sheet.
 *     Default true.
 *   - `resolve` - `{object=}`: Similar to locals, except it takes promises as values
 *   and the bottom sheet will not open until the promises resolve.
 *   - `controllerAs` - `{string=}`: An alias to assign the controller to on the scope.
 *   - `parent` - `{element=}`: The element to append the bottom sheet to. The `parent` may be a `function`, `string`,
 *   `object`, or null. Defaults to appending to the body of the root element (or the root element) of the application.
 *   e.g. angular.element(document.getElementById('content')) or "#content"
 *   - `disableParentScroll` - `{boolean=}`: Whether to disable scrolling while the bottom sheet is open.
 *     Default true.
 *
 * @returns {promise} A promise that can be resolved with `$mdBottomSheet.hide()` or
 * rejected with `$mdBottomSheet.cancel()`.
 */
    /**
 * @ngdoc method
 * @name $mdBottomSheet#hide
 *
 * @description
 * Hide the existing bottom sheet and resolve the promise returned from
 * `$mdBottomSheet.show()`. This call will close the most recently opened/current bottomsheet (if any).
 *
 * @param {*=} response An argument for the resolved promise.
 *
 */
    /**
 * @ngdoc method
 * @name $mdBottomSheet#cancel
 *
 * @description
 * Hide the existing bottom sheet and reject the promise returned from
 * `$mdBottomSheet.show()`.
 *
 * @param {*=} response An argument for the rejected promise.
 *
 */
    function MdBottomSheetProvider($$interimElementProvider) {
      // how fast we need to flick down to close the sheet, pixels/ms
      var CLOSING_VELOCITY = 0.5;
      var PADDING = 80;
      // same as css
      bottomSheetDefaults.$inject = [
        '$animate',
        '$mdConstant',
        '$mdUtil',
        '$mdTheming',
        '$mdBottomSheet',
        '$rootElement',
        '$mdGesture'
      ];
      return $$interimElementProvider('$mdBottomSheet').setDefaults({
        methods: [
          'disableParentScroll',
          'escapeToClose',
          'clickOutsideToClose'
        ],
        options: bottomSheetDefaults
      });
      /* @ngInject */
      function bottomSheetDefaults($animate, $mdConstant, $mdUtil, $mdTheming, $mdBottomSheet, $rootElement, $mdGesture) {
        var backdrop;
        return {
          themable: true,
          onShow: onShow,
          onRemove: onRemove,
          escapeToClose: true,
          clickOutsideToClose: true,
          disableParentScroll: true
        };
        function onShow(scope, element, options, controller) {
          element = $mdUtil.extractElementByName(element, 'md-bottom-sheet');
          // Add a backdrop that will close on click
          backdrop = $mdUtil.createBackdrop(scope, 'md-bottom-sheet-backdrop md-opaque');
          if (options.clickOutsideToClose) {
            backdrop.on('click', function () {
              $mdUtil.nextTick($mdBottomSheet.cancel, true);
            });
          }
          $mdTheming.inherit(backdrop, options.parent);
          $animate.enter(backdrop, options.parent, null);
          var bottomSheet = new BottomSheet(element, options.parent);
          options.bottomSheet = bottomSheet;
          $mdTheming.inherit(bottomSheet.element, options.parent);
          if (options.disableParentScroll) {
            options.restoreScroll = $mdUtil.disableScrollAround(bottomSheet.element, options.parent);
          }
          return $animate.enter(bottomSheet.element, options.parent).then(function () {
            var focusable = $mdUtil.findFocusTarget(element) || angular.element(element[0].querySelector('button') || element[0].querySelector('a') || element[0].querySelector('[ng-click]'));
            focusable.focus();
            if (options.escapeToClose) {
              options.rootElementKeyupCallback = function (e) {
                if (e.keyCode === $mdConstant.KEY_CODE.ESCAPE) {
                  $mdUtil.nextTick($mdBottomSheet.cancel, true);
                }
              };
              $rootElement.on('keyup', options.rootElementKeyupCallback);
            }
          });
        }
        function onRemove(scope, element, options) {
          var bottomSheet = options.bottomSheet;
          $animate.leave(backdrop);
          return $animate.leave(bottomSheet.element).then(function () {
            if (options.disableParentScroll) {
              options.restoreScroll();
              delete options.restoreScroll;
            }
            bottomSheet.cleanup();
          });
        }
        /**
     * BottomSheet class to apply bottom-sheet behavior to an element
     */
        function BottomSheet(element, parent) {
          var deregister = $mdGesture.register(parent, 'drag', { horizontal: false });
          parent.on('$md.dragstart', onDragStart).on('$md.drag', onDrag).on('$md.dragend', onDragEnd);
          return {
            element: element,
            cleanup: function cleanup() {
              deregister();
              parent.off('$md.dragstart', onDragStart);
              parent.off('$md.drag', onDrag);
              parent.off('$md.dragend', onDragEnd);
            }
          };
          function onDragStart(ev) {
            // Disable transitions on transform so that it feels fast
            element.css($mdConstant.CSS.TRANSITION_DURATION, '0ms');
          }
          function onDrag(ev) {
            var transform = ev.pointer.distanceY;
            if (transform < 5) {
              // Slow down drag when trying to drag up, and stop after PADDING
              transform = Math.max(-PADDING, transform / 2);
            }
            element.css($mdConstant.CSS.TRANSFORM, 'translate3d(0,' + (PADDING + transform) + 'px,0)');
          }
          function onDragEnd(ev) {
            if (ev.pointer.distanceY > 0 && (ev.pointer.distanceY > 20 || Math.abs(ev.pointer.velocityY) > CLOSING_VELOCITY)) {
              var distanceRemaining = element.prop('offsetHeight') - ev.pointer.distanceY;
              var transitionDuration = Math.min(distanceRemaining / ev.pointer.velocityY * 0.75, 500);
              element.css($mdConstant.CSS.TRANSITION_DURATION, transitionDuration + 'ms');
              $mdUtil.nextTick($mdBottomSheet.cancel, true);
            } else {
              element.css($mdConstant.CSS.TRANSITION_DURATION, '');
              element.css($mdConstant.CSS.TRANSFORM, '');
            }
          }
        }
      }
    }
    MdBottomSheetProvider.$inject = ['$$interimElementProvider'];
  }());
  (function () {
    'use strict';
    /**
 * @ngdoc module
 * @name material.components.button
 * @description
 *
 * Button
 */
    angular.module('material.components.button', ['material.core']).directive('mdButton', MdButtonDirective);
    /**
 * @ngdoc directive
 * @name mdButton
 * @module material.components.button
 *
 * @restrict E
 *
 * @description
 * `<md-button>` is a button directive with optional ink ripples (default enabled).
 *
 * If you supply a `href` or `ng-href` attribute, it will become an `<a>` element. Otherwise, it will
 * become a `<button>` element. As per the [Material Design specifications](http://www.google.com/design/spec/style/color.html#color-ui-color-application)
 * the FAB button background is filled with the accent color [by default]. The primary color palette may be used with
 * the `md-primary` class.
 *
 * @param {boolean=} md-no-ink If present, disable ripple ink effects.
 * @param {expression=} ng-disabled En/Disable based on the expression
 * @param {string=} md-ripple-size Overrides the default ripple size logic. Options: `full`, `partial`, `auto`
 * @param {string=} aria-label Adds alternative text to button for accessibility, useful for icon buttons.
 * If no default text is found, a warning will be logged.
 *
 * @usage
 *
 * Regular buttons:
 *
 * <hljs lang="html">
 *  <md-button> Flat Button </md-button>
 *  <md-button href="http://google.com"> Flat link </md-button>
 *  <md-button class="md-raised"> Raised Button </md-button>
 *  <md-button ng-disabled="true"> Disabled Button </md-button>
 *  <md-button>
 *    <md-icon md-svg-src="your/icon.svg"></md-icon>
 *    Register Now
 *  </md-button>
 * </hljs>
 *
 * FAB buttons:
 *
 * <hljs lang="html">
 *  <md-button class="md-fab" aria-label="FAB">
 *    <md-icon md-svg-src="your/icon.svg"></md-icon>
 *  </md-button>
 *  <!-- mini-FAB -->
 *  <md-button class="md-fab md-mini" aria-label="Mini FAB">
 *    <md-icon md-svg-src="your/icon.svg"></md-icon>
 *  </md-button>
 *  <!-- Button with SVG Icon -->
 *  <md-button class="md-icon-button" aria-label="Custom Icon Button">
 *    <md-icon md-svg-icon="path/to/your.svg"></md-icon>
 *  </md-button>
 * </hljs>
 */
    function MdButtonDirective($mdButtonInkRipple, $mdTheming, $mdAria, $timeout) {
      return {
        restrict: 'EA',
        replace: true,
        transclude: true,
        template: getTemplate,
        link: postLink
      };
      function isAnchor(attr) {
        return angular.isDefined(attr.href) || angular.isDefined(attr.ngHref) || angular.isDefined(attr.ngLink) || angular.isDefined(attr.uiSref);
      }
      function getTemplate(element, attr) {
        if (isAnchor(attr)) {
          return '<a class="md-button" ng-transclude></a>';
        } else {
          //If buttons don't have type="button", they will submit forms automatically.
          var btnType = typeof attr.type === 'undefined' ? 'button' : attr.type;
          return '<button class="md-button" type="' + btnType + '" ng-transclude></button>';
        }
      }
      function postLink(scope, element, attr) {
        var node = element[0];
        $mdTheming(element);
        $mdButtonInkRipple.attach(scope, element);
        var elementHasText = node.textContent.trim();
        if (!elementHasText) {
          $mdAria.expect(element, 'aria-label');
        }
        // For anchor elements, we have to set tabindex manually when the
        // element is disabled
        if (isAnchor(attr) && angular.isDefined(attr.ngDisabled)) {
          scope.$watch(attr.ngDisabled, function (isDisabled) {
            element.attr('tabindex', isDisabled ? -1 : 0);
          });
        }
        // disabling click event when disabled is true
        element.on('click', function (e) {
          if (attr.disabled === true) {
            e.preventDefault();
            e.stopImmediatePropagation();
          }
        });
        // restrict focus styles to the keyboard
        scope.mouseActive = false;
        element.on('mousedown', function () {
          scope.mouseActive = true;
          $timeout(function () {
            scope.mouseActive = false;
          }, 100);
        }).on('focus', function () {
          if (scope.mouseActive === false) {
            element.addClass('md-focused');
          }
        }).on('blur', function (ev) {
          element.removeClass('md-focused');
        });
      }
    }
    MdButtonDirective.$inject = [
      '$mdButtonInkRipple',
      '$mdTheming',
      '$mdAria',
      '$timeout'
    ];
  }());
  (function () {
    'use strict';
    /**
 * @ngdoc module
 * @name material.components.card
 *
 * @description
 * Card components.
 */
    angular.module('material.components.card', ['material.core']).directive('mdCard', mdCardDirective);
    /**
 * @ngdoc directive
 * @name mdCard
 * @module material.components.card
 *
 * @restrict E
 *
 * @description
 * The `<md-card>` directive is a container element used within `<md-content>` containers.
 *
 * An image included as a direct descendant will fill the card's width, while the `<md-card-content>`
 * container will wrap text content and provide padding. An `<md-card-footer>` element can be
 * optionally included to put content flush against the bottom edge of the card.
 *
 * Action buttons can be included in an `<md-card-actions>` element, similar to `<md-dialog-actions>`.
 * You can then position buttons using layout attributes.
 *
 * Card is built with:
 * * `<md-card-header>` - Header for the card, holds avatar, text and squared image
 *  - `<md-card-avatar>` - Card avatar
 *    - `md-user-avatar` - Class for user image
 *    - `<md-icon>`
 *  - `<md-card-header-text>` - Contains elements for the card description
 *    - `md-title` - Class for the card title
 *    - `md-subhead` - Class for the card sub header
 * * `<img>` - Image for the card
 * * `<md-card-title>` - Card content title
 *  - `<md-card-title-text>`
 *    - `md-headline` - Class for the card content title
 *    - `md-subhead` - Class for the card content sub header
 *  - `<md-card-title-media>` - Squared image within the title
 *    - `md-media-sm` - Class for small image
 *    - `md-media-md` - Class for medium image
 *    - `md-media-lg` - Class for large image
 * * `<md-card-content>` - Card content
 *  - `md-media-xl` - Class for extra large image
 * * `<md-card-actions>` - Card actions
 *  - `<md-card-icon-actions>` - Icon actions
 *
 * Cards have constant width and variable heights; where the maximum height is limited to what can
 * fit within a single view on a platform, but it can temporarily expand as needed.
 *
 * @usage
 * ### Card with optional footer
 * <hljs lang="html">
 * <md-card>
 *  <img src="card-image.png" class="md-card-image" alt="image caption">
 *  <md-card-content>
 *    <h2>Card headline</h2>
 *    <p>Card content</p>
 *  </md-card-content>
 *  <md-card-footer>
 *    Card footer
 *  </md-card-footer>
 * </md-card>
 * </hljs>
 *
 * ### Card with actions
 * <hljs lang="html">
 * <md-card>
 *  <img src="card-image.png" class="md-card-image" alt="image caption">
 *  <md-card-content>
 *    <h2>Card headline</h2>
 *    <p>Card content</p>
 *  </md-card-content>
 *  <md-card-actions layout="row" layout-align="end center">
 *    <md-button>Action 1</md-button>
 *    <md-button>Action 2</md-button>
 *  </md-card-actions>
 * </md-card>
 * </hljs>
 *
 * ### Card with header, image, title actions and content
 * <hljs lang="html">
 * <md-card>
 *   <md-card-header>
 *     <md-card-avatar>
 *       <img class="md-user-avatar" src="avatar.png"/>
 *     </md-card-avatar>
 *     <md-card-header-text>
 *       <span class="md-title">Title</span>
 *       <span class="md-subhead">Sub header</span>
 *     </md-card-header-text>
 *   </md-card-header>
 *   <img ng-src="card-image.png" class="md-card-image" alt="image caption">
 *   <md-card-title>
 *     <md-card-title-text>
 *       <span class="md-headline">Card headline</span>
 *       <span class="md-subhead">Card subheader</span>
 *     </md-card-title-text>
 *   </md-card-title>
 *   <md-card-actions layout="row" layout-align="start center">
 *     <md-button>Action 1</md-button>
 *     <md-button>Action 2</md-button>
 *     <md-card-icon-actions>
 *       <md-button class="md-icon-button" aria-label="icon">
 *         <md-icon md-svg-icon="icon"></md-icon>
 *       </md-button>
 *     </md-card-icon-actions>
 *   </md-card-actions>
 *   <md-card-content>
 *     <p>
 *      Card content
 *     </p>
 *   </md-card-content>
 * </md-card>
 * </hljs>
 */
    function mdCardDirective($mdTheming) {
      return {
        restrict: 'E',
        link: function ($scope, $element) {
          $mdTheming($element);
        }
      };
    }
    mdCardDirective.$inject = ['$mdTheming'];
  }());
  (function () {
    'use strict';
    /**
 * @ngdoc module
 * @name material.components.checkbox
 * @description Checkbox module!
 */
    angular.module('material.components.checkbox', ['material.core']).directive('mdCheckbox', MdCheckboxDirective);
    /**
 * @ngdoc directive
 * @name mdCheckbox
 * @module material.components.checkbox
 * @restrict E
 *
 * @description
 * The checkbox directive is used like the normal [angular checkbox](https://docs.angularjs.org/api/ng/input/input%5Bcheckbox%5D).
 *
 * As per the [material design spec](http://www.google.com/design/spec/style/color.html#color-ui-color-application)
 * the checkbox is in the accent color by default. The primary color palette may be used with
 * the `md-primary` class.
 *
 * @param {string} ng-model Assignable angular expression to data-bind to.
 * @param {string=} name Property name of the form under which the control is published.
 * @param {expression=} ng-true-value The value to which the expression should be set when selected.
 * @param {expression=} ng-false-value The value to which the expression should be set when not selected.
 * @param {string=} ng-change Angular expression to be executed when input changes due to user interaction with the input element.
 * @param {boolean=} md-no-ink Use of attribute indicates use of ripple ink effects
 * @param {string=} aria-label Adds label to checkbox for accessibility.
 * Defaults to checkbox's text. If no default text is found, a warning will be logged.
 *
 * @usage
 * <hljs lang="html">
 * <md-checkbox ng-model="isChecked" aria-label="Finished?">
 *   Finished ?
 * </md-checkbox>
 *
 * <md-checkbox md-no-ink ng-model="hasInk" aria-label="No Ink Effects">
 *   No Ink Effects
 * </md-checkbox>
 *
 * <md-checkbox ng-disabled="true" ng-model="isDisabled" aria-label="Disabled">
 *   Disabled
 * </md-checkbox>
 *
 * </hljs>
 *
 */
    function MdCheckboxDirective(inputDirective, $mdAria, $mdConstant, $mdTheming, $mdUtil, $timeout) {
      inputDirective = inputDirective[0];
      var CHECKED_CSS = 'md-checked';
      return {
        restrict: 'E',
        transclude: true,
        require: '?ngModel',
        priority: 210,
        template: '<div class="md-container" md-ink-ripple md-ink-ripple-checkbox>' + '<div class="md-icon"></div>' + '</div>' + '<div ng-transclude class="md-label"></div>',
        compile: compile
      };
      // **********************************************************
      // Private Methods
      // **********************************************************
      function compile(tElement, tAttrs) {
        tAttrs.type = 'checkbox';
        tAttrs.tabindex = tAttrs.tabindex || '0';
        tElement.attr('role', tAttrs.type);
        // Attach a click handler in compile in order to immediately stop propagation
        // (especially for ng-click) when the checkbox is disabled.
        tElement.on('click', function (event) {
          if (this.hasAttribute('disabled')) {
            event.stopImmediatePropagation();
          }
        });
        return function postLink(scope, element, attr, ngModelCtrl) {
          ngModelCtrl = ngModelCtrl || $mdUtil.fakeNgModel();
          $mdTheming(element);
          if (attr.ngChecked) {
            scope.$watch(scope.$eval.bind(scope, attr.ngChecked), ngModelCtrl.$setViewValue.bind(ngModelCtrl));
          }
          $$watchExpr('ngDisabled', 'tabindex', {
            true: '-1',
            false: attr.tabindex
          });
          $mdAria.expectWithText(element, 'aria-label');
          // Reuse the original input[type=checkbox] directive from Angular core.
          // This is a bit hacky as we need our own event listener and own render
          // function.
          inputDirective.link.pre(scope, {
            on: angular.noop,
            0: {}
          }, attr, [ngModelCtrl]);
          scope.mouseActive = false;
          element.on('click', listener).on('keypress', keypressHandler).on('mousedown', function () {
            scope.mouseActive = true;
            $timeout(function () {
              scope.mouseActive = false;
            }, 100);
          }).on('focus', function () {
            if (scope.mouseActive === false) {
              element.addClass('md-focused');
            }
          }).on('blur', function () {
            element.removeClass('md-focused');
          });
          ngModelCtrl.$render = render;
          function $$watchExpr(expr, htmlAttr, valueOpts) {
            if (attr[expr]) {
              scope.$watch(attr[expr], function (val) {
                if (valueOpts[val]) {
                  element.attr(htmlAttr, valueOpts[val]);
                }
              });
            }
          }
          function keypressHandler(ev) {
            var keyCode = ev.which || ev.keyCode;
            if (keyCode === $mdConstant.KEY_CODE.SPACE || keyCode === $mdConstant.KEY_CODE.ENTER) {
              ev.preventDefault();
              if (!element.hasClass('md-focused')) {
                element.addClass('md-focused');
              }
              listener(ev);
            }
          }
          function listener(ev) {
            if (element[0].hasAttribute('disabled')) {
              return;
            }
            scope.$apply(function () {
              // Toggle the checkbox value...
              var viewValue = attr.ngChecked ? attr.checked : !ngModelCtrl.$viewValue;
              ngModelCtrl.$setViewValue(viewValue, ev && ev.type);
              ngModelCtrl.$render();
            });
          }
          function render() {
            if (ngModelCtrl.$viewValue) {
              element.addClass(CHECKED_CSS);
            } else {
              element.removeClass(CHECKED_CSS);
            }
          }
        };
      }
    }
    MdCheckboxDirective.$inject = [
      'inputDirective',
      '$mdAria',
      '$mdConstant',
      '$mdTheming',
      '$mdUtil',
      '$timeout'
    ];
  }());
  (function () {
    'use strict';
    /**
 * @ngdoc module
 * @name material.components.chips
 */
    /*
 * @see js folder for chips implementation
 */
    angular.module('material.components.chips', [
      'material.core',
      'material.components.autocomplete'
    ]);
  }());
  (function () {
    'use strict';
    /**
 * @ngdoc module
 * @name material.components.content
 *
 * @description
 * Scrollable content
 */
    angular.module('material.components.content', ['material.core']).directive('mdContent', mdContentDirective);
    /**
 * @ngdoc directive
 * @name mdContent
 * @module material.components.content
 *
 * @restrict E
 *
 * @description
 * The `<md-content>` directive is a container element useful for scrollable content
 *
 * @usage
 *
 * - Add the `[layout-padding]` attribute to make the content padded.
 *
 * <hljs lang="html">
 *  <md-content layout-padding>
 *      Lorem ipsum dolor sit amet, ne quod novum mei.
 *  </md-content>
 * </hljs>
 *
 */
    function mdContentDirective($mdTheming) {
      return {
        restrict: 'E',
        controller: [
          '$scope',
          '$element',
          ContentController
        ],
        link: function (scope, element, attr) {
          var node = element[0];
          $mdTheming(element);
          scope.$broadcast('$mdContentLoaded', element);
          iosScrollFix(element[0]);
        }
      };
      function ContentController($scope, $element) {
        this.$scope = $scope;
        this.$element = $element;
      }
    }
    mdContentDirective.$inject = ['$mdTheming'];
    function iosScrollFix(node) {
      // IOS FIX:
      // If we scroll where there is no more room for the webview to scroll,
      // by default the webview itself will scroll up and down, this looks really
      // bad.  So if we are scrolling to the very top or bottom, add/subtract one
      angular.element(node).on('$md.pressdown', function (ev) {
        // Only touch events
        if (ev.pointer.type !== 't')
          return;
        // Don't let a child content's touchstart ruin it for us.
        if (ev.$materialScrollFixed)
          return;
        ev.$materialScrollFixed = true;
        if (node.scrollTop === 0) {
          node.scrollTop = 1;
        } else if (node.scrollHeight === node.scrollTop + node.offsetHeight) {
          node.scrollTop -= 1;
        }
      });
    }
  }());
  (function () {
    'use strict';
    (function () {
      'use strict';
      /**
   * @ngdoc module
   * @name material.components.datepicker
   * @description Datepicker
   */
      angular.module('material.components.datepicker', [
        'material.core',
        'material.components.icon',
        'material.components.virtualRepeat'
      ]).directive('mdCalendar', calendarDirective);
      // POST RELEASE
      // TODO(jelbourn): Mac Cmd + left / right == Home / End
      // TODO(jelbourn): Clicking on the month label opens the month-picker.
      // TODO(jelbourn): Minimum and maximum date
      // TODO(jelbourn): Refactor month element creation to use cloneNode (performance).
      // TODO(jelbourn): Define virtual scrolling constants (compactness) users can override.
      // TODO(jelbourn): Animated month transition on ng-model change (virtual-repeat)
      // TODO(jelbourn): Scroll snapping (virtual repeat)
      // TODO(jelbourn): Remove superfluous row from short months (virtual-repeat)
      // TODO(jelbourn): Month headers stick to top when scrolling.
      // TODO(jelbourn): Previous month opacity is lowered when partially scrolled out of view.
      // TODO(jelbourn): Support md-calendar standalone on a page (as a tabstop w/ aria-live
      //     announcement and key handling).
      // Read-only calendar (not just date-picker).
      /**
   * Height of one calendar month tbody. This must be made known to the virtual-repeat and is
   * subsequently used for scrolling to specific months.
   */
      var TBODY_HEIGHT = 265;
      /**
   * Height of a calendar month with a single row. This is needed to calculate the offset for
   * rendering an extra month in virtual-repeat that only contains one row.
   */
      var TBODY_SINGLE_ROW_HEIGHT = 45;
      function calendarDirective() {
        return {
          template: '<table aria-hidden="true" class="md-calendar-day-header"><thead></thead></table>' + '<div class="md-calendar-scroll-mask">' + '<md-virtual-repeat-container class="md-calendar-scroll-container" ' + 'md-offset-size="' + (TBODY_SINGLE_ROW_HEIGHT - TBODY_HEIGHT) + '">' + '<table role="grid" tabindex="0" class="md-calendar" aria-readonly="true">' + '<tbody role="rowgroup" md-virtual-repeat="i in ctrl.items" md-calendar-month ' + 'md-month-offset="$index" class="md-calendar-month" ' + 'md-start-index="ctrl.getSelectedMonthIndex()" ' + 'md-item-size="' + TBODY_HEIGHT + '"></tbody>' + '</table>' + '</md-virtual-repeat-container>' + '</div>',
          scope: {
            minDate: '=mdMinDate',
            maxDate: '=mdMaxDate',
            dateFilter: '=mdDateFilter'
          },
          require: [
            'ngModel',
            'mdCalendar'
          ],
          controller: CalendarCtrl,
          controllerAs: 'ctrl',
          bindToController: true,
          link: function (scope, element, attrs, controllers) {
            var ngModelCtrl = controllers[0];
            var mdCalendarCtrl = controllers[1];
            mdCalendarCtrl.configureNgModel(ngModelCtrl);
          }
        };
      }
      /** Class applied to the selected date cell/. */
      var SELECTED_DATE_CLASS = 'md-calendar-selected-date';
      /** Class applied to the focused date cell/. */
      var FOCUSED_DATE_CLASS = 'md-focus';
      /** Next identifier for calendar instance. */
      var nextUniqueId = 0;
      /** The first renderable date in the virtual-scrolling calendar (for all instances). */
      var firstRenderableDate = null;
      /**
   * Controller for the mdCalendar component.
   * @ngInject @constructor
   */
      function CalendarCtrl($element, $attrs, $scope, $animate, $q, $mdConstant, $mdTheming, $$mdDateUtil, $mdDateLocale, $mdInkRipple, $mdUtil) {
        $mdTheming($element);
        /**
     * Dummy array-like object for virtual-repeat to iterate over. The length is the total
     * number of months that can be viewed. This is shorter than ideal because of (potential)
     * Firefox bug https://bugzilla.mozilla.org/show_bug.cgi?id=1181658.
     */
        this.items = { length: 2000 };
        if (this.maxDate && this.minDate) {
          // Limit the number of months if min and max dates are set.
          var numMonths = $$mdDateUtil.getMonthDistance(this.minDate, this.maxDate) + 1;
          numMonths = Math.max(numMonths, 1);
          // Add an additional month as the final dummy month for rendering purposes.
          numMonths += 1;
          this.items.length = numMonths;
        }
        /** @final {!angular.$animate} */
        this.$animate = $animate;
        /** @final {!angular.$q} */
        this.$q = $q;
        /** @final */
        this.$mdInkRipple = $mdInkRipple;
        /** @final */
        this.$mdUtil = $mdUtil;
        /** @final */
        this.keyCode = $mdConstant.KEY_CODE;
        /** @final */
        this.dateUtil = $$mdDateUtil;
        /** @final */
        this.dateLocale = $mdDateLocale;
        /** @final {!angular.JQLite} */
        this.$element = $element;
        /** @final {!angular.Scope} */
        this.$scope = $scope;
        /** @final {HTMLElement} */
        this.calendarElement = $element[0].querySelector('.md-calendar');
        /** @final {HTMLElement} */
        this.calendarScroller = $element[0].querySelector('.md-virtual-repeat-scroller');
        /** @final {Date} */
        this.today = this.dateUtil.createDateAtMidnight();
        /** @type {Date} */
        this.firstRenderableDate = this.dateUtil.incrementMonths(this.today, -this.items.length / 2);
        if (this.minDate && this.minDate > this.firstRenderableDate) {
          this.firstRenderableDate = this.minDate;
        } else if (this.maxDate) {
          // Calculate the difference between the start date and max date.
          // Subtract 1 because it's an inclusive difference and 1 for the final dummy month.
          //
          var monthDifference = this.items.length - 2;
          this.firstRenderableDate = this.dateUtil.incrementMonths(this.maxDate, -(this.items.length - 2));
        }
        /** @final {number} Unique ID for this calendar instance. */
        this.id = nextUniqueId++;
        /** @type {!angular.NgModelController} */
        this.ngModelCtrl = null;
        /**
     * The selected date. Keep track of this separately from the ng-model value so that we
     * can know, when the ng-model value changes, what the previous value was before its updated
     * in the component's UI.
     *
     * @type {Date}
     */
        this.selectedDate = null;
        /**
     * The date that is currently focused or showing in the calendar. This will initially be set
     * to the ng-model value if set, otherwise to today. It will be updated as the user navigates
     * to other months. The cell corresponding to the displayDate does not necesarily always have
     * focus in the document (such as for cases when the user is scrolling the calendar).
     * @type {Date}
     */
        this.displayDate = null;
        /**
     * The date that has or should have focus.
     * @type {Date}
     */
        this.focusDate = null;
        /** @type {boolean} */
        this.isInitialized = false;
        /** @type {boolean} */
        this.isMonthTransitionInProgress = false;
        // Unless the user specifies so, the calendar should not be a tab stop.
        // This is necessary because ngAria might add a tabindex to anything with an ng-model
        // (based on whether or not the user has turned that particular feature on/off).
        if (!$attrs['tabindex']) {
          $element.attr('tabindex', '-1');
        }
        var self = this;
        /**
     * Handles a click event on a date cell.
     * Created here so that every cell can use the same function instance.
     * @this {HTMLTableCellElement} The cell that was clicked.
     */
        this.cellClickHandler = function () {
          var cellElement = this;
          if (this.hasAttribute('data-timestamp')) {
            $scope.$apply(function () {
              var timestamp = Number(cellElement.getAttribute('data-timestamp'));
              self.setNgModelValue(self.dateUtil.createDateAtMidnight(timestamp));
            });
          }
        };
        this.attachCalendarEventListeners();
      }
      CalendarCtrl.$inject = [
        '$element',
        '$attrs',
        '$scope',
        '$animate',
        '$q',
        '$mdConstant',
        '$mdTheming',
        '$$mdDateUtil',
        '$mdDateLocale',
        '$mdInkRipple',
        '$mdUtil'
      ];
      /*** Initialization ***/
      /**
   * Sets up the controller's reference to ngModelController.
   * @param {!angular.NgModelController} ngModelCtrl
   */
      CalendarCtrl.prototype.configureNgModel = function (ngModelCtrl) {
        this.ngModelCtrl = ngModelCtrl;
        var self = this;
        ngModelCtrl.$render = function () {
          self.changeSelectedDate(self.ngModelCtrl.$viewValue);
        };
      };
      /**
   * Initialize the calendar by building the months that are initially visible.
   * Initialization should occur after the ngModel value is known.
   */
      CalendarCtrl.prototype.buildInitialCalendarDisplay = function () {
        this.buildWeekHeader();
        this.hideVerticalScrollbar();
        this.displayDate = this.selectedDate || this.today;
        this.isInitialized = true;
      };
      /**
   * Hides the vertical scrollbar on the calendar scroller by setting the width on the
   * calendar scroller and the `overflow: hidden` wrapper around the scroller, and then setting
   * a padding-right on the scroller equal to the width of the browser's scrollbar.
   *
   * This will cause a reflow.
   */
      CalendarCtrl.prototype.hideVerticalScrollbar = function () {
        var element = this.$element[0];
        var scrollMask = element.querySelector('.md-calendar-scroll-mask');
        var scroller = this.calendarScroller;
        var headerWidth = element.querySelector('.md-calendar-day-header').clientWidth;
        var scrollbarWidth = scroller.offsetWidth - scroller.clientWidth;
        scrollMask.style.width = headerWidth + 'px';
        scroller.style.width = headerWidth + scrollbarWidth + 'px';
        scroller.style.paddingRight = scrollbarWidth + 'px';
      };
      /** Attach event listeners for the calendar. */
      CalendarCtrl.prototype.attachCalendarEventListeners = function () {
        // Keyboard interaction.
        this.$element.on('keydown', angular.bind(this, this.handleKeyEvent));
      };
      /*** User input handling ***/
      /**
   * Handles a key event in the calendar with the appropriate action. The action will either
   * be to select the focused date or to navigate to focus a new date.
   * @param {KeyboardEvent} event
   */
      CalendarCtrl.prototype.handleKeyEvent = function (event) {
        var self = this;
        this.$scope.$apply(function () {
          // Capture escape and emit back up so that a wrapping component
          // (such as a date-picker) can decide to close.
          if (event.which == self.keyCode.ESCAPE || event.which == self.keyCode.TAB) {
            self.$scope.$emit('md-calendar-close');
            if (event.which == self.keyCode.TAB) {
              event.preventDefault();
            }
            return;
          }
          // Remaining key events fall into two categories: selection and navigation.
          // Start by checking if this is a selection event.
          if (event.which === self.keyCode.ENTER) {
            self.setNgModelValue(self.displayDate);
            event.preventDefault();
            return;
          }
          // Selection isn't occuring, so the key event is either navigation or nothing.
          var date = self.getFocusDateFromKeyEvent(event);
          if (date) {
            date = self.boundDateByMinAndMax(date);
            event.preventDefault();
            event.stopPropagation();
            // Since this is a keyboard interaction, actually give the newly focused date keyboard
            // focus after the been brought into view.
            self.changeDisplayDate(date).then(function () {
              self.focus(date);
            });
          }
        });
      };
      /**
   * Gets the date to focus as the result of a key event.
   * @param {KeyboardEvent} event
   * @returns {Date} Date to navigate to, or null if the key does not match a calendar shortcut.
   */
      CalendarCtrl.prototype.getFocusDateFromKeyEvent = function (event) {
        var dateUtil = this.dateUtil;
        var keyCode = this.keyCode;
        switch (event.which) {
        case keyCode.RIGHT_ARROW:
          return dateUtil.incrementDays(this.displayDate, 1);
        case keyCode.LEFT_ARROW:
          return dateUtil.incrementDays(this.displayDate, -1);
        case keyCode.DOWN_ARROW:
          return event.metaKey ? dateUtil.incrementMonths(this.displayDate, 1) : dateUtil.incrementDays(this.displayDate, 7);
        case keyCode.UP_ARROW:
          return event.metaKey ? dateUtil.incrementMonths(this.displayDate, -1) : dateUtil.incrementDays(this.displayDate, -7);
        case keyCode.PAGE_DOWN:
          return dateUtil.incrementMonths(this.displayDate, 1);
        case keyCode.PAGE_UP:
          return dateUtil.incrementMonths(this.displayDate, -1);
        case keyCode.HOME:
          return dateUtil.getFirstDateOfMonth(this.displayDate);
        case keyCode.END:
          return dateUtil.getLastDateOfMonth(this.displayDate);
        default:
          return null;
        }
      };
      /**
   * Gets the "index" of the currently selected date as it would be in the virtual-repeat.
   * @returns {number}
   */
      CalendarCtrl.prototype.getSelectedMonthIndex = function () {
        return this.dateUtil.getMonthDistance(this.firstRenderableDate, this.selectedDate || this.today);
      };
      /**
   * Scrolls to the month of the given date.
   * @param {Date} date
   */
      CalendarCtrl.prototype.scrollToMonth = function (date) {
        if (!this.dateUtil.isValidDate(date)) {
          return;
        }
        var monthDistance = this.dateUtil.getMonthDistance(this.firstRenderableDate, date);
        this.calendarScroller.scrollTop = monthDistance * TBODY_HEIGHT;
      };
      /**
   * Sets the ng-model value for the calendar and emits a change event.
   * @param {Date} date
   */
      CalendarCtrl.prototype.setNgModelValue = function (date) {
        this.$scope.$emit('md-calendar-change', date);
        this.ngModelCtrl.$setViewValue(date);
        this.ngModelCtrl.$render();
      };
      /**
   * Focus the cell corresponding to the given date.
   * @param {Date=} opt_date
   */
      CalendarCtrl.prototype.focus = function (opt_date) {
        var date = opt_date || this.selectedDate || this.today;
        var previousFocus = this.calendarElement.querySelector('.md-focus');
        if (previousFocus) {
          previousFocus.classList.remove(FOCUSED_DATE_CLASS);
        }
        var cellId = this.getDateId(date);
        var cell = document.getElementById(cellId);
        if (cell) {
          cell.classList.add(FOCUSED_DATE_CLASS);
          cell.focus();
        } else {
          this.focusDate = date;
        }
      };
      /**
   * If a date exceeds minDate or maxDate, returns date matching minDate or maxDate, respectively.
   * Otherwise, returns the date.
   * @param {Date} date
   * @return {Date}
   */
      CalendarCtrl.prototype.boundDateByMinAndMax = function (date) {
        var boundDate = date;
        if (this.minDate && date < this.minDate) {
          boundDate = new Date(this.minDate.getTime());
        }
        if (this.maxDate && date > this.maxDate) {
          boundDate = new Date(this.maxDate.getTime());
        }
        return boundDate;
      };
      /*** Updating the displayed / selected date ***/
      /**
   * Change the selected date in the calendar (ngModel value has already been changed).
   * @param {Date} date
   */
      CalendarCtrl.prototype.changeSelectedDate = function (date) {
        var self = this;
        var previousSelectedDate = this.selectedDate;
        this.selectedDate = date;
        this.changeDisplayDate(date).then(function () {
          // Remove the selected class from the previously selected date, if any.
          if (previousSelectedDate) {
            var prevDateCell = document.getElementById(self.getDateId(previousSelectedDate));
            if (prevDateCell) {
              prevDateCell.classList.remove(SELECTED_DATE_CLASS);
              prevDateCell.setAttribute('aria-selected', 'false');
            }
          }
          // Apply the select class to the new selected date if it is set.
          if (date) {
            var dateCell = document.getElementById(self.getDateId(date));
            if (dateCell) {
              dateCell.classList.add(SELECTED_DATE_CLASS);
              dateCell.setAttribute('aria-selected', 'true');
            }
          }
        });
      };
      /**
   * Change the date that is being shown in the calendar. If the given date is in a different
   * month, the displayed month will be transitioned.
   * @param {Date} date
   */
      CalendarCtrl.prototype.changeDisplayDate = function (date) {
        // Initialization is deferred until this function is called because we want to reflect
        // the starting value of ngModel.
        if (!this.isInitialized) {
          this.buildInitialCalendarDisplay();
          return this.$q.when();
        }
        // If trying to show an invalid date or a transition is in progress, do nothing.
        if (!this.dateUtil.isValidDate(date) || this.isMonthTransitionInProgress) {
          return this.$q.when();
        }
        this.isMonthTransitionInProgress = true;
        var animationPromise = this.animateDateChange(date);
        this.displayDate = date;
        var self = this;
        animationPromise.then(function () {
          self.isMonthTransitionInProgress = false;
        });
        return animationPromise;
      };
      /**
   * Animates the transition from the calendar's current month to the given month.
   * @param {Date} date
   * @returns {angular.$q.Promise} The animation promise.
   */
      CalendarCtrl.prototype.animateDateChange = function (date) {
        this.scrollToMonth(date);
        return this.$q.when();
      };
      /*** Constructing the calendar table ***/
      /**
   * Builds and appends a day-of-the-week header to the calendar.
   * This should only need to be called once during initialization.
   */
      CalendarCtrl.prototype.buildWeekHeader = function () {
        var firstDayOfWeek = this.dateLocale.firstDayOfWeek;
        var shortDays = this.dateLocale.shortDays;
        var row = document.createElement('tr');
        for (var i = 0; i < 7; i++) {
          var th = document.createElement('th');
          th.textContent = shortDays[(i + firstDayOfWeek) % 7];
          row.appendChild(th);
        }
        this.$element.find('thead').append(row);
      };
      /**
   * Gets an identifier for a date unique to the calendar instance for internal
   * purposes. Not to be displayed.
   * @param {Date} date
   * @returns {string}
   */
      CalendarCtrl.prototype.getDateId = function (date) {
        return [
          'md',
          this.id,
          date.getFullYear(),
          date.getMonth(),
          date.getDate()
        ].join('-');
      };
    }());
  }());
  (function () {
    'use strict';
    (function () {
      'use strict';
      angular.module('material.components.datepicker').directive('mdCalendarMonth', mdCalendarMonthDirective);
      /**
   * Private directive consumed by md-calendar. Having this directive lets the calender use
   * md-virtual-repeat and also cleanly separates the month DOM construction functions from
   * the rest of the calendar controller logic.
   */
      function mdCalendarMonthDirective() {
        return {
          require: [
            '^^mdCalendar',
            'mdCalendarMonth'
          ],
          scope: { offset: '=mdMonthOffset' },
          controller: CalendarMonthCtrl,
          controllerAs: 'mdMonthCtrl',
          bindToController: true,
          link: function (scope, element, attrs, controllers) {
            var calendarCtrl = controllers[0];
            var monthCtrl = controllers[1];
            monthCtrl.calendarCtrl = calendarCtrl;
            monthCtrl.generateContent();
            // The virtual-repeat re-uses the same DOM elements, so there are only a limited number
            // of repeated items that are linked, and then those elements have their bindings updataed.
            // Since the months are not generated by bindings, we simply regenerate the entire thing
            // when the binding (offset) changes.
            scope.$watch(function () {
              return monthCtrl.offset;
            }, function (offset, oldOffset) {
              if (offset != oldOffset) {
                monthCtrl.generateContent();
              }
            });
          }
        };
      }
      /** Class applied to the cell for today. */
      var TODAY_CLASS = 'md-calendar-date-today';
      /** Class applied to the selected date cell/. */
      var SELECTED_DATE_CLASS = 'md-calendar-selected-date';
      /** Class applied to the focused date cell/. */
      var FOCUSED_DATE_CLASS = 'md-focus';
      /**
   * Controller for a single calendar month.
   * @ngInject @constructor
   */
      function CalendarMonthCtrl($element, $$mdDateUtil, $mdDateLocale) {
        this.dateUtil = $$mdDateUtil;
        this.dateLocale = $mdDateLocale;
        this.$element = $element;
        this.calendarCtrl = null;
        /**
     * Number of months from the start of the month "items" that the currently rendered month
     * occurs. Set via angular data binding.
     * @type {number}
     */
        this.offset;
        /**
     * Date cell to focus after appending the month to the document.
     * @type {HTMLElement}
     */
        this.focusAfterAppend = null;
      }
      CalendarMonthCtrl.$inject = [
        '$element',
        '$$mdDateUtil',
        '$mdDateLocale'
      ];
      /** Generate and append the content for this month to the directive element. */
      CalendarMonthCtrl.prototype.generateContent = function () {
        var calendarCtrl = this.calendarCtrl;
        var date = this.dateUtil.incrementMonths(calendarCtrl.firstRenderableDate, this.offset);
        this.$element.empty();
        this.$element.append(this.buildCalendarForMonth(date));
        if (this.focusAfterAppend) {
          this.focusAfterAppend.classList.add(FOCUSED_DATE_CLASS);
          this.focusAfterAppend.focus();
          this.focusAfterAppend = null;
        }
      };
      /**
   * Creates a single cell to contain a date in the calendar with all appropriate
   * attributes and classes added. If a date is given, the cell content will be set
   * based on the date.
   * @param {Date=} opt_date
   * @returns {HTMLElement}
   */
      CalendarMonthCtrl.prototype.buildDateCell = function (opt_date) {
        var calendarCtrl = this.calendarCtrl;
        // TODO(jelbourn): cloneNode is likely a faster way of doing this.
        var cell = document.createElement('td');
        cell.tabIndex = -1;
        cell.classList.add('md-calendar-date');
        cell.setAttribute('role', 'gridcell');
        if (opt_date) {
          cell.setAttribute('tabindex', '-1');
          cell.setAttribute('aria-label', this.dateLocale.longDateFormatter(opt_date));
          cell.id = calendarCtrl.getDateId(opt_date);
          // Use `data-timestamp` attribute because IE10 does not support the `dataset` property.
          cell.setAttribute('data-timestamp', opt_date.getTime());
          // TODO(jelourn): Doing these comparisons for class addition during generation might be slow.
          // It may be better to finish the construction and then query the node and add the class.
          if (this.dateUtil.isSameDay(opt_date, calendarCtrl.today)) {
            cell.classList.add(TODAY_CLASS);
          }
          if (this.dateUtil.isValidDate(calendarCtrl.selectedDate) && this.dateUtil.isSameDay(opt_date, calendarCtrl.selectedDate)) {
            cell.classList.add(SELECTED_DATE_CLASS);
            cell.setAttribute('aria-selected', 'true');
          }
          var cellText = this.dateLocale.dates[opt_date.getDate()];
          if (this.isDateEnabled(opt_date)) {
            // Add a indicator for select, hover, and focus states.
            var selectionIndicator = document.createElement('span');
            cell.appendChild(selectionIndicator);
            selectionIndicator.classList.add('md-calendar-date-selection-indicator');
            selectionIndicator.textContent = cellText;
            cell.addEventListener('click', calendarCtrl.cellClickHandler);
            if (calendarCtrl.focusDate && this.dateUtil.isSameDay(opt_date, calendarCtrl.focusDate)) {
              this.focusAfterAppend = cell;
            }
          } else {
            cell.classList.add('md-calendar-date-disabled');
            cell.textContent = cellText;
          }
        }
        return cell;
      };
      /**
   * Check whether date is in range and enabled
   * @param {Date=} opt_date
   * @return {boolean} Whether the date is enabled.
   */
      CalendarMonthCtrl.prototype.isDateEnabled = function (opt_date) {
        return this.dateUtil.isDateWithinRange(opt_date, this.calendarCtrl.minDate, this.calendarCtrl.maxDate) && (!angular.isFunction(this.calendarCtrl.dateFilter) || this.calendarCtrl.dateFilter(opt_date));
      };
      /**
   * Builds a `tr` element for the calendar grid.
   * @param rowNumber The week number within the month.
   * @returns {HTMLElement}
   */
      CalendarMonthCtrl.prototype.buildDateRow = function (rowNumber) {
        var row = document.createElement('tr');
        row.setAttribute('role', 'row');
        // Because of an NVDA bug (with Firefox), the row needs an aria-label in order
        // to prevent the entire row being read aloud when the user moves between rows.
        // See http://community.nvda-project.org/ticket/4643.
        row.setAttribute('aria-label', this.dateLocale.weekNumberFormatter(rowNumber));
        return row;
      };
      /**
   * Builds the <tbody> content for the given date's month.
   * @param {Date=} opt_dateInMonth
   * @returns {DocumentFragment} A document fragment containing the <tr> elements.
   */
      CalendarMonthCtrl.prototype.buildCalendarForMonth = function (opt_dateInMonth) {
        var date = this.dateUtil.isValidDate(opt_dateInMonth) ? opt_dateInMonth : new Date();
        var firstDayOfMonth = this.dateUtil.getFirstDateOfMonth(date);
        var firstDayOfTheWeek = this.getLocaleDay_(firstDayOfMonth);
        var numberOfDaysInMonth = this.dateUtil.getNumberOfDaysInMonth(date);
        // Store rows for the month in a document fragment so that we can append them all at once.
        var monthBody = document.createDocumentFragment();
        var rowNumber = 1;
        var row = this.buildDateRow(rowNumber);
        monthBody.appendChild(row);
        // If this is the final month in the list of items, only the first week should render,
        // so we should return immediately after the first row is complete and has been
        // attached to the body.
        var isFinalMonth = this.offset === this.calendarCtrl.items.length - 1;
        // Add a label for the month. If the month starts on a Sun/Mon/Tues, the month label
        // goes on a row above the first of the month. Otherwise, the month label takes up the first
        // two cells of the first row.
        var blankCellOffset = 0;
        var monthLabelCell = document.createElement('td');
        monthLabelCell.classList.add('md-calendar-month-label');
        // If the entire month is after the max date, render the label as a disabled state.
        if (this.calendarCtrl.maxDate && firstDayOfMonth > this.calendarCtrl.maxDate) {
          monthLabelCell.classList.add('md-calendar-month-label-disabled');
        }
        monthLabelCell.textContent = this.dateLocale.monthHeaderFormatter(date);
        if (firstDayOfTheWeek <= 2) {
          monthLabelCell.setAttribute('colspan', '7');
          var monthLabelRow = this.buildDateRow();
          monthLabelRow.appendChild(monthLabelCell);
          monthBody.insertBefore(monthLabelRow, row);
          if (isFinalMonth) {
            return monthBody;
          }
        } else {
          blankCellOffset = 2;
          monthLabelCell.setAttribute('colspan', '2');
          row.appendChild(monthLabelCell);
        }
        // Add a blank cell for each day of the week that occurs before the first of the month.
        // For example, if the first day of the month is a Tuesday, add blank cells for Sun and Mon.
        // The blankCellOffset is needed in cases where the first N cells are used by the month label.
        for (var i = blankCellOffset; i < firstDayOfTheWeek; i++) {
          row.appendChild(this.buildDateCell());
        }
        // Add a cell for each day of the month, keeping track of the day of the week so that
        // we know when to start a new row.
        var dayOfWeek = firstDayOfTheWeek;
        var iterationDate = firstDayOfMonth;
        for (var d = 1; d <= numberOfDaysInMonth; d++) {
          // If we've reached the end of the week, start a new row.
          if (dayOfWeek === 7) {
            // We've finished the first row, so we're done if this is the final month.
            if (isFinalMonth) {
              return monthBody;
            }
            dayOfWeek = 0;
            rowNumber++;
            row = this.buildDateRow(rowNumber);
            monthBody.appendChild(row);
          }
          iterationDate.setDate(d);
          var cell = this.buildDateCell(iterationDate);
          row.appendChild(cell);
          dayOfWeek++;
        }
        // Ensure that the last row of the month has 7 cells.
        while (row.childNodes.length < 7) {
          row.appendChild(this.buildDateCell());
        }
        // Ensure that all months have 6 rows. This is necessary for now because the virtual-repeat
        // requires that all items have exactly the same height.
        while (monthBody.childNodes.length < 6) {
          var whitespaceRow = this.buildDateRow();
          for (var i = 0; i < 7; i++) {
            whitespaceRow.appendChild(this.buildDateCell());
          }
          monthBody.appendChild(whitespaceRow);
        }
        return monthBody;
      };
      /**
   * Gets the day-of-the-week index for a date for the current locale.
   * @private
   * @param {Date} date
   * @returns {number} The column index of the date in the calendar.
   */
      CalendarMonthCtrl.prototype.getLocaleDay_ = function (date) {
        return (date.getDay() + (7 - this.dateLocale.firstDayOfWeek)) % 7;
      };
    }());
  }());
  (function () {
    'use strict';
    (function () {
      'use strict';
      /**
   * @ngdoc service
   * @name $mdDateLocaleProvider
   * @module material.components.datepicker
   *
   * @description
   * The `$mdDateLocaleProvider` is the provider that creates the `$mdDateLocale` service.
   * This provider that allows the user to specify messages, formatters, and parsers for date
   * internationalization. The `$mdDateLocale` service itself is consumed by Angular Material
   * components that that deal with dates.
   *
   * @property {(Array<string>)=} months Array of month names (in order).
   * @property {(Array<string>)=} shortMonths Array of abbreviated month names.
   * @property {(Array<string>)=} days Array of the days of the week (in order).
   * @property {(Array<string>)=} shortDays Array of abbreviated dayes of the week.
   * @property {(Array<string>)=} dates Array of dates of the month. Only necessary for locales
   *     using a numeral system other than [1, 2, 3...].
   * @property {(Array<string>)=} firstDayOfWeek The first day of the week. Sunday = 0, Monday = 1,
   *    etc.
   * @property {(function(string): Date)=} parseDate Function to parse a date object from a string.
   * @property {(function(Date): string)=} formatDate Function to format a date object to a string.
   * @property {(function(Date): string)=} monthHeaderFormatter Function that returns the label for
   *     a month given a date.
   * @property {(function(number): string)=} weekNumberFormatter Function that returns a label for
   *     a week given the week number.
   * @property {(string)=} msgCalendar Translation of the label "Calendar" for the current locale.
   * @property {(string)=} msgOpenCalendar Translation of the button label "Open calendar" for the
   *     current locale.
   *
   * @usage
   * <hljs lang="js">
   *   myAppModule.config(function($mdDateLocaleProvider) {
   *
   *     // Example of a French localization.
   *     $mdDateLocaleProvider.months = ['janvier', 'février', 'mars', ...];
   *     $mdDateLocaleProvider.shortMonths = ['janv', 'févr', 'mars', ...];
   *     $mdDateLocaleProvider.days = ['dimanche', 'lundi', 'mardi', ...];
   *     $mdDateLocaleProvider.shortDays = ['Di', 'Lu', 'Ma', ...];
   *
   *     // Can change week display to start on Monday.
   *     $mdDateLocaleProvider.firstDayOfWeek = 1;
   *
   *     // Optional.
   *     $mdDateLocaleProvider.dates = [1, 2, 3, 4, 5, 6, ...];
   *
   *     // Example uses moment.js to parse and format dates.
   *     $mdDateLocaleProvider.parseDate = function(dateString) {
   *       var m = moment(dateString, 'L', true);
   *       return m.isValid() ? m.toDate() : new Date(NaN);
   *     };
   *
   *     $mdDateLocaleProvider.formatDate = function(date) {
   *       return moment(date).format('L');
   *     };
   *
   *     $mdDateLocaleProvider.monthHeaderFormatter = function(date) {
   *       return myShortMonths[date.getMonth()] + ' ' + date.getFullYear();
   *     };
   *
   *     // In addition to date display, date components also need localized messages
   *     // for aria-labels for screen-reader users.
   *
   *     $mdDateLocaleProvider.weekNumberFormatter = function(weekNumber) {
   *       return 'Semaine ' + weekNumber;
   *     };
   *
   *     $mdDateLocaleProvider.msgCalendar = 'Calendrier';
   *     $mdDateLocaleProvider.msgOpenCalendar = 'Ouvrir le calendrier';
   *
   * });
   * </hljs>
   *
   */
      angular.module('material.components.datepicker').config([
        '$provide',
        function ($provide) {
          // TODO(jelbourn): Assert provided values are correctly formatted. Need assertions.
          /** @constructor */
          function DateLocaleProvider() {
            /** Array of full month names. E.g., ['January', 'Febuary', ...] */
            this.months = null;
            /** Array of abbreviated month names. E.g., ['Jan', 'Feb', ...] */
            this.shortMonths = null;
            /** Array of full day of the week names. E.g., ['Monday', 'Tuesday', ...] */
            this.days = null;
            /** Array of abbreviated dat of the week names. E.g., ['M', 'T', ...] */
            this.shortDays = null;
            /** Array of dates of a month (1 - 31). Characters might be different in some locales. */
            this.dates = null;
            /** Index of the first day of the week. 0 = Sunday, 1 = Monday, etc. */
            this.firstDayOfWeek = 0;
            /**
       * Function that converts the date portion of a Date to a string.
       * @type {(function(Date): string)}
       */
            this.formatDate = null;
            /**
       * Function that converts a date string to a Date object (the date portion)
       * @type {function(string): Date}
       */
            this.parseDate = null;
            /**
       * Function that formats a Date into a month header string.
       * @type {function(Date): string}
       */
            this.monthHeaderFormatter = null;
            /**
       * Function that formats a week number into a label for the week.
       * @type {function(number): string}
       */
            this.weekNumberFormatter = null;
            /**
       * Function that formats a date into a long aria-label that is read
       * when the focused date changes.
       * @type {function(Date): string}
       */
            this.longDateFormatter = null;
            /**
       * ARIA label for the calendar "dialog" used in the datepicker.
       * @type {string}
       */
            this.msgCalendar = '';
            /**
       * ARIA label for the datepicker's "Open calendar" buttons.
       * @type {string}
       */
            this.msgOpenCalendar = '';
          }
          /**
     * Factory function that returns an instance of the dateLocale service.
     * @ngInject
     * @param $locale
     * @returns {DateLocale}
     */
          DateLocaleProvider.prototype.$get = function ($locale) {
            /**
       * Default date-to-string formatting function.
       * @param {!Date} date
       * @returns {string}
       */
            function defaultFormatDate(date) {
              if (!date) {
                return '';
              }
              // All of the dates created through ng-material *should* be set to midnight.
              // If we encounter a date where the localeTime shows at 11pm instead of midnight,
              // we have run into an issue with DST where we need to increment the hour by one:
              // var d = new Date(1992, 9, 8, 0, 0, 0);
              // d.toLocaleString(); // == "10/7/1992, 11:00:00 PM"
              var localeTime = date.toLocaleTimeString();
              var formatDate = date;
              if (date.getHours() == 0 && (localeTime.indexOf('11:') !== -1 || localeTime.indexOf('23:') !== -1)) {
                formatDate = new Date(date.getFullYear(), date.getMonth(), date.getDate(), 1, 0, 0);
              }
              return formatDate.toLocaleDateString();
            }
            /**
       * Default string-to-date parsing function.
       * @param {string} dateString
       * @returns {!Date}
       */
            function defaultParseDate(dateString) {
              return new Date(dateString);
            }
            /**
       * Default function to determine whether a string makes sense to be
       * parsed to a Date object.
       *
       * This is very permissive and is just a basic sanity check to ensure that
       * things like single integers aren't able to be parsed into dates.
       * @param {string} dateString
       * @returns {boolean}
       */
            function defaultIsDateComplete(dateString) {
              dateString = dateString.trim();
              // Looks for three chunks of content (either numbers or text) separated
              // by delimiters.
              var re = /^(([a-zA-Z]{3,}|[0-9]{1,4})([ \.,]+|[\/\-])){2}([a-zA-Z]{3,}|[0-9]{1,4})$/;
              return re.test(dateString);
            }
            /**
       * Default date-to-string formatter to get a month header.
       * @param {!Date} date
       * @returns {string}
       */
            function defaultMonthHeaderFormatter(date) {
              return service.shortMonths[date.getMonth()] + ' ' + date.getFullYear();
            }
            /**
       * Default week number formatter.
       * @param number
       * @returns {string}
       */
            function defaultWeekNumberFormatter(number) {
              return 'Week ' + number;
            }
            /**
       * Default formatter for date cell aria-labels.
       * @param {!Date} date
       * @returns {string}
       */
            function defaultLongDateFormatter(date) {
              // Example: 'Thursday June 18 2015'
              return [
                service.days[date.getDay()],
                service.months[date.getMonth()],
                service.dates[date.getDate()],
                date.getFullYear()
              ].join(' ');
            }
            // The default "short" day strings are the first character of each day,
            // e.g., "Monday" => "M".
            var defaultShortDays = $locale.DATETIME_FORMATS.DAY.map(function (day) {
                return day[0];
              });
            // The default dates are simply the numbers 1 through 31.
            var defaultDates = Array(32);
            for (var i = 1; i <= 31; i++) {
              defaultDates[i] = i;
            }
            // Default ARIA messages are in English (US).
            var defaultMsgCalendar = 'Calendar';
            var defaultMsgOpenCalendar = 'Open calendar';
            var service = {
                months: this.months || $locale.DATETIME_FORMATS.MONTH,
                shortMonths: this.shortMonths || $locale.DATETIME_FORMATS.SHORTMONTH,
                days: this.days || $locale.DATETIME_FORMATS.DAY,
                shortDays: this.shortDays || defaultShortDays,
                dates: this.dates || defaultDates,
                firstDayOfWeek: this.firstDayOfWeek || 0,
                formatDate: this.formatDate || defaultFormatDate,
                parseDate: this.parseDate || defaultParseDate,
                isDateComplete: this.isDateComplete || defaultIsDateComplete,
                monthHeaderFormatter: this.monthHeaderFormatter || defaultMonthHeaderFormatter,
                weekNumberFormatter: this.weekNumberFormatter || defaultWeekNumberFormatter,
                longDateFormatter: this.longDateFormatter || defaultLongDateFormatter,
                msgCalendar: this.msgCalendar || defaultMsgCalendar,
                msgOpenCalendar: this.msgOpenCalendar || defaultMsgOpenCalendar
              };
            return service;
          };
          DateLocaleProvider.prototype.$get.$inject = ['$locale'];
          $provide.provider('$mdDateLocale', new DateLocaleProvider());
        }
      ]);
    }());
  }());
  (function () {
    'use strict';
    (function () {
      'use strict';
      // POST RELEASE
      // TODO(jelbourn): Demo that uses moment.js
      // TODO(jelbourn): make sure this plays well with validation and ngMessages.
      // TODO(jelbourn): calendar pane doesn't open up outside of visible viewport.
      // TODO(jelbourn): forward more attributes to the internal input (required, autofocus, etc.)
      // TODO(jelbourn): something better for mobile (calendar panel takes up entire screen?)
      // TODO(jelbourn): input behavior (masking? auto-complete?)
      // TODO(jelbourn): UTC mode
      // TODO(jelbourn): RTL
      angular.module('material.components.datepicker').directive('mdDatepicker', datePickerDirective);
      /**
   * @ngdoc directive
   * @name mdDatepicker
   * @module material.components.datepicker
   *
   * @param {Date} ng-model The component's model. Expects a JavaScript Date object.
   * @param {expression=} ng-change Expression evaluated when the model value changes.
   * @param {Date=} md-min-date Expression representing a min date (inclusive).
   * @param {Date=} md-max-date Expression representing a max date (inclusive).
   * @param {(function(Date): boolean)=} md-date-filter Function expecting a date and returning a boolean whether it can be selected or not.
   * @param {String=} md-placeholder The date input placeholder value.
   * @param {boolean=} ng-disabled Whether the datepicker is disabled.
   * @param {boolean=} ng-required Whether a value is required for the datepicker.
   *
   * @description
   * `<md-datepicker>` is a component used to select a single date.
   * For information on how to configure internationalization for the date picker,
   * see `$mdDateLocaleProvider`.
   *
   * This component supports [ngMessages](https://docs.angularjs.org/api/ngMessages/directive/ngMessages).
   * Supported attributes are:
   * * `required`: whether a required date is not set.
   * * `mindate`: whether the selected date is before the minimum allowed date.
   * * `maxdate`: whether the selected date is after the maximum allowed date.
   *
   * @usage
   * <hljs lang="html">
   *   <md-datepicker ng-model="birthday"></md-datepicker>
   * </hljs>
   *
   */
      function datePickerDirective() {
        return {
          template: '<md-button class="md-datepicker-button md-icon-button" type="button" ' + 'tabindex="-1" aria-hidden="true" ' + 'ng-click="ctrl.openCalendarPane($event)">' + '<md-icon class="md-datepicker-calendar-icon" md-svg-icon="md-calendar"></md-icon>' + '</md-button>' + '<div class="md-datepicker-input-container" ' + 'ng-class="{\'md-datepicker-focused\': ctrl.isFocused}">' + '<input class="md-datepicker-input" aria-haspopup="true" ' + 'ng-focus="ctrl.setFocused(true)" ng-blur="ctrl.setFocused(false)">' + '<md-button type="button" md-no-ink ' + 'class="md-datepicker-triangle-button md-icon-button" ' + 'ng-click="ctrl.openCalendarPane($event)" ' + 'aria-label="{{::ctrl.dateLocale.msgOpenCalendar}}">' + '<div class="md-datepicker-expand-triangle"></div>' + '</md-button>' + '</div>' + '<div class="md-datepicker-calendar-pane md-whiteframe-z1">' + '<div class="md-datepicker-input-mask">' + '<div class="md-datepicker-input-mask-opaque"></div>' + '</div>' + '<div class="md-datepicker-calendar">' + '<md-calendar role="dialog" aria-label="{{::ctrl.dateLocale.msgCalendar}}" ' + 'md-min-date="ctrl.minDate" md-max-date="ctrl.maxDate"' + 'md-date-filter="ctrl.dateFilter"' + 'ng-model="ctrl.date" ng-if="ctrl.isCalendarOpen">' + '</md-calendar>' + '</div>' + '</div>',
          require: [
            'ngModel',
            'mdDatepicker',
            '?^mdInputContainer'
          ],
          scope: {
            minDate: '=mdMinDate',
            maxDate: '=mdMaxDate',
            placeholder: '@mdPlaceholder',
            dateFilter: '=mdDateFilter'
          },
          controller: DatePickerCtrl,
          controllerAs: 'ctrl',
          bindToController: true,
          link: function (scope, element, attr, controllers) {
            var ngModelCtrl = controllers[0];
            var mdDatePickerCtrl = controllers[1];
            var mdInputContainer = controllers[2];
            if (mdInputContainer) {
              throw Error('md-datepicker should not be placed inside md-input-container.');
            }
            mdDatePickerCtrl.configureNgModel(ngModelCtrl);
          }
        };
      }
      /** Additional offset for the input's `size` attribute, which is updated based on its content. */
      var EXTRA_INPUT_SIZE = 3;
      /** Class applied to the container if the date is invalid. */
      var INVALID_CLASS = 'md-datepicker-invalid';
      /** Default time in ms to debounce input event by. */
      var DEFAULT_DEBOUNCE_INTERVAL = 500;
      /**
   * Height of the calendar pane used to check if the pane is going outside the boundary of
   * the viewport. See calendar.scss for how $md-calendar-height is computed; an extra 20px is
   * also added to space the pane away from the exact edge of the screen.
   *
   *  This is computed statically now, but can be changed to be measured if the circumstances
   *  of calendar sizing are changed.
   */
      var CALENDAR_PANE_HEIGHT = 368;
      /**
   * Width of the calendar pane used to check if the pane is going outside the boundary of
   * the viewport. See calendar.scss for how $md-calendar-width is computed; an extra 20px is
   * also added to space the pane away from the exact edge of the screen.
   *
   *  This is computed statically now, but can be changed to be measured if the circumstances
   *  of calendar sizing are changed.
   */
      var CALENDAR_PANE_WIDTH = 360;
      /**
   * Controller for md-datepicker.
   *
   * @ngInject @constructor
   */
      function DatePickerCtrl($scope, $element, $attrs, $compile, $timeout, $window, $mdConstant, $mdTheming, $mdUtil, $mdDateLocale, $$mdDateUtil, $$rAF) {
        /** @final */
        this.$compile = $compile;
        /** @final */
        this.$timeout = $timeout;
        /** @final */
        this.$window = $window;
        /** @final */
        this.dateLocale = $mdDateLocale;
        /** @final */
        this.dateUtil = $$mdDateUtil;
        /** @final */
        this.$mdConstant = $mdConstant;
        /* @final */
        this.$mdUtil = $mdUtil;
        /** @final */
        this.$$rAF = $$rAF;
        /**
     * The root document element. This is used for attaching a top-level click handler to
     * close the calendar panel when a click outside said panel occurs. We use `documentElement`
     * instead of body because, when scrolling is disabled, some browsers consider the body element
     * to be completely off the screen and propagate events directly to the html element.
     * @type {!angular.JQLite}
     */
        this.documentElement = angular.element(document.documentElement);
        /** @type {!angular.NgModelController} */
        this.ngModelCtrl = null;
        /** @type {HTMLInputElement} */
        this.inputElement = $element[0].querySelector('input');
        /** @final {!angular.JQLite} */
        this.ngInputElement = angular.element(this.inputElement);
        /** @type {HTMLElement} */
        this.inputContainer = $element[0].querySelector('.md-datepicker-input-container');
        /** @type {HTMLElement} Floating calendar pane. */
        this.calendarPane = $element[0].querySelector('.md-datepicker-calendar-pane');
        /** @type {HTMLElement} Calendar icon button. */
        this.calendarButton = $element[0].querySelector('.md-datepicker-button');
        /**
     * Element covering everything but the input in the top of the floating calendar pane.
     * @type {HTMLElement}
     */
        this.inputMask = $element[0].querySelector('.md-datepicker-input-mask-opaque');
        /** @final {!angular.JQLite} */
        this.$element = $element;
        /** @final {!angular.Attributes} */
        this.$attrs = $attrs;
        /** @final {!angular.Scope} */
        this.$scope = $scope;
        /** @type {Date} */
        this.date = null;
        /** @type {boolean} */
        this.isFocused = false;
        /** @type {boolean} */
        this.isDisabled;
        this.setDisabled($element[0].disabled || angular.isString($attrs['disabled']));
        /** @type {boolean} Whether the date-picker's calendar pane is open. */
        this.isCalendarOpen = false;
        /**
     * Element from which the calendar pane was opened. Keep track of this so that we can return
     * focus to it when the pane is closed.
     * @type {HTMLElement}
     */
        this.calendarPaneOpenedFrom = null;
        this.calendarPane.id = 'md-date-pane' + $mdUtil.nextUid();
        $mdTheming($element);
        /** Pre-bound click handler is saved so that the event listener can be removed. */
        this.bodyClickHandler = angular.bind(this, this.handleBodyClick);
        /** Pre-bound resize handler so that the event listener can be removed. */
        this.windowResizeHandler = $mdUtil.debounce(angular.bind(this, this.closeCalendarPane), 100);
        // Unless the user specifies so, the datepicker should not be a tab stop.
        // This is necessary because ngAria might add a tabindex to anything with an ng-model
        // (based on whether or not the user has turned that particular feature on/off).
        if (!$attrs['tabindex']) {
          $element.attr('tabindex', '-1');
        }
        this.installPropertyInterceptors();
        this.attachChangeListeners();
        this.attachInteractionListeners();
        var self = this;
        $scope.$on('$destroy', function () {
          self.detachCalendarPane();
        });
      }
      DatePickerCtrl.$inject = [
        '$scope',
        '$element',
        '$attrs',
        '$compile',
        '$timeout',
        '$window',
        '$mdConstant',
        '$mdTheming',
        '$mdUtil',
        '$mdDateLocale',
        '$$mdDateUtil',
        '$$rAF'
      ];
      /**
   * Sets up the controller's reference to ngModelController.
   * @param {!angular.NgModelController} ngModelCtrl
   */
      DatePickerCtrl.prototype.configureNgModel = function (ngModelCtrl) {
        this.ngModelCtrl = ngModelCtrl;
        var self = this;
        ngModelCtrl.$render = function () {
          var value = self.ngModelCtrl.$viewValue;
          if (value && !(value instanceof Date)) {
            throw Error('The ng-model for md-datepicker must be a Date instance. ' + 'Currently the model is a: ' + typeof value);
          }
          self.date = value;
          self.inputElement.value = self.dateLocale.formatDate(value);
          self.resizeInputElement();
          self.updateErrorState();
        };
      };
      /**
   * Attach event listeners for both the text input and the md-calendar.
   * Events are used instead of ng-model so that updates don't infinitely update the other
   * on a change. This should also be more performant than using a $watch.
   */
      DatePickerCtrl.prototype.attachChangeListeners = function () {
        var self = this;
        self.$scope.$on('md-calendar-change', function (event, date) {
          self.ngModelCtrl.$setViewValue(date);
          self.date = date;
          self.inputElement.value = self.dateLocale.formatDate(date);
          self.closeCalendarPane();
          self.resizeInputElement();
          self.updateErrorState();
        });
        self.ngInputElement.on('input', angular.bind(self, self.resizeInputElement));
        // TODO(chenmike): Add ability for users to specify this interval.
        self.ngInputElement.on('input', self.$mdUtil.debounce(self.handleInputEvent, DEFAULT_DEBOUNCE_INTERVAL, self));
      };
      /** Attach event listeners for user interaction. */
      DatePickerCtrl.prototype.attachInteractionListeners = function () {
        var self = this;
        var $scope = this.$scope;
        var keyCodes = this.$mdConstant.KEY_CODE;
        // Add event listener through angular so that we can triggerHandler in unit tests.
        self.ngInputElement.on('keydown', function (event) {
          if (event.altKey && event.keyCode == keyCodes.DOWN_ARROW) {
            self.openCalendarPane(event);
            $scope.$digest();
          }
        });
        $scope.$on('md-calendar-close', function () {
          self.closeCalendarPane();
        });
      };
      /**
   * Capture properties set to the date-picker and imperitively handle internal changes.
   * This is done to avoid setting up additional $watches.
   */
      DatePickerCtrl.prototype.installPropertyInterceptors = function () {
        var self = this;
        if (this.$attrs['ngDisabled']) {
          // The expression is to be evaluated against the directive element's scope and not
          // the directive's isolate scope.
          var scope = this.$mdUtil.validateScope(this.$element) ? this.$element.scope() : null;
          if (scope) {
            scope.$watch(this.$attrs['ngDisabled'], function (isDisabled) {
              self.setDisabled(isDisabled);
            });
          }
        }
        Object.defineProperty(this, 'placeholder', {
          get: function () {
            return self.inputElement.placeholder;
          },
          set: function (value) {
            self.inputElement.placeholder = value || '';
          }
        });
      };
      /**
   * Sets whether the date-picker is disabled.
   * @param {boolean} isDisabled
   */
      DatePickerCtrl.prototype.setDisabled = function (isDisabled) {
        this.isDisabled = isDisabled;
        this.inputElement.disabled = isDisabled;
        this.calendarButton.disabled = isDisabled;
      };
      /**
   * Sets the custom ngModel.$error flags to be consumed by ngMessages. Flags are:
   *   - mindate: whether the selected date is before the minimum date.
   *   - maxdate: whether the selected flag is after the maximum date.
   *   - filtered: whether the selected date is allowed by the custom filtering function.
   *   - valid: whether the entered text input is a valid date
   *
   * The 'required' flag is handled automatically by ngModel.
   *
   * @param {Date=} opt_date Date to check. If not given, defaults to the datepicker's model value.
   */
      DatePickerCtrl.prototype.updateErrorState = function (opt_date) {
        var date = opt_date || this.date;
        // Clear any existing errors to get rid of anything that's no longer relevant.
        this.clearErrorState();
        if (this.dateUtil.isValidDate(date)) {
          if (this.dateUtil.isValidDate(this.minDate)) {
            this.ngModelCtrl.$setValidity('mindate', date >= this.minDate);
          }
          if (this.dateUtil.isValidDate(this.maxDate)) {
            this.ngModelCtrl.$setValidity('maxdate', date <= this.maxDate);
          }
          if (angular.isFunction(this.dateFilter)) {
            this.ngModelCtrl.$setValidity('filtered', this.dateFilter(date));
          }
        } else {
          // The date is seen as "not a valid date" if there is *something* set
          // (i.e.., not null or undefined), but that something isn't a valid date.
          this.ngModelCtrl.$setValidity('valid', date == null);
        }
        // TODO(jelbourn): Change this to classList.toggle when we stop using PhantomJS in unit tests
        // because it doesn't conform to the DOMTokenList spec.
        // See https://github.com/ariya/phantomjs/issues/12782.
        if (!this.ngModelCtrl.$valid) {
          this.inputContainer.classList.add(INVALID_CLASS);
        }
      };
      /** Clears any error flags set by `updateErrorState`. */
      DatePickerCtrl.prototype.clearErrorState = function () {
        this.inputContainer.classList.remove(INVALID_CLASS);
        [
          'mindate',
          'maxdate',
          'filtered',
          'valid'
        ].forEach(function (field) {
          this.ngModelCtrl.$setValidity(field, true);
        }, this);
      };
      /** Resizes the input element based on the size of its content. */
      DatePickerCtrl.prototype.resizeInputElement = function () {
        this.inputElement.size = this.inputElement.value.length + EXTRA_INPUT_SIZE;
      };
      /**
   * Sets the model value if the user input is a valid date.
   * Adds an invalid class to the input element if not.
   */
      DatePickerCtrl.prototype.handleInputEvent = function () {
        var inputString = this.inputElement.value;
        var parsedDate = inputString ? this.dateLocale.parseDate(inputString) : null;
        this.dateUtil.setDateTimeToMidnight(parsedDate);
        // An input string is valid if it is either empty (representing no date)
        // or if it parses to a valid date that the user is allowed to select.
        var isValidInput = inputString == '' || this.dateUtil.isValidDate(parsedDate) && this.dateLocale.isDateComplete(inputString) && this.isDateEnabled(parsedDate);
        // The datepicker's model is only updated when there is a valid input.
        if (isValidInput) {
          this.ngModelCtrl.$setViewValue(parsedDate);
          this.date = parsedDate;
        }
        this.updateErrorState(parsedDate);
      };
      /**
   * Check whether date is in range and enabled
   * @param {Date=} opt_date
   * @return {boolean} Whether the date is enabled.
   */
      DatePickerCtrl.prototype.isDateEnabled = function (opt_date) {
        return this.dateUtil.isDateWithinRange(opt_date, this.minDate, this.maxDate) && (!angular.isFunction(this.dateFilter) || this.dateFilter(opt_date));
      };
      /** Position and attach the floating calendar to the document. */
      DatePickerCtrl.prototype.attachCalendarPane = function () {
        var calendarPane = this.calendarPane;
        calendarPane.style.transform = '';
        this.$element.addClass('md-datepicker-open');
        var elementRect = this.inputContainer.getBoundingClientRect();
        var bodyRect = document.body.getBoundingClientRect();
        // Check to see if the calendar pane would go off the screen. If so, adjust position
        // accordingly to keep it within the viewport.
        var paneTop = elementRect.top - bodyRect.top;
        var paneLeft = elementRect.left - bodyRect.left;
        // If ng-material has disabled body scrolling (for example, if a dialog is open),
        // then it's possible that the already-scrolled body has a negative top/left. In this case,
        // we want to treat the "real" top as (0 - bodyRect.top). In a normal scrolling situation,
        // though, the top of the viewport should just be the body's scroll position.
        var viewportTop = bodyRect.top < 0 && document.body.scrollTop == 0 ? -bodyRect.top : document.body.scrollTop;
        var viewportLeft = bodyRect.left < 0 && document.body.scrollLeft == 0 ? -bodyRect.left : document.body.scrollLeft;
        var viewportBottom = viewportTop + this.$window.innerHeight;
        var viewportRight = viewportLeft + this.$window.innerWidth;
        // If the right edge of the pane would be off the screen and shifting it left by the
        // difference would not go past the left edge of the screen. If the calendar pane is too
        // big to fit on the screen at all, move it to the left of the screen and scale the entire
        // element down to fit.
        if (paneLeft + CALENDAR_PANE_WIDTH > viewportRight) {
          if (viewportRight - CALENDAR_PANE_WIDTH > 0) {
            paneLeft = viewportRight - CALENDAR_PANE_WIDTH;
          } else {
            paneLeft = viewportLeft;
            var scale = this.$window.innerWidth / CALENDAR_PANE_WIDTH;
            calendarPane.style.transform = 'scale(' + scale + ')';
          }
          calendarPane.classList.add('md-datepicker-pos-adjusted');
        }
        // If the bottom edge of the pane would be off the screen and shifting it up by the
        // difference would not go past the top edge of the screen.
        if (paneTop + CALENDAR_PANE_HEIGHT > viewportBottom && viewportBottom - CALENDAR_PANE_HEIGHT > viewportTop) {
          paneTop = viewportBottom - CALENDAR_PANE_HEIGHT;
          calendarPane.classList.add('md-datepicker-pos-adjusted');
        }
        calendarPane.style.left = paneLeft + 'px';
        calendarPane.style.top = paneTop + 'px';
        document.body.appendChild(calendarPane);
        // The top of the calendar pane is a transparent box that shows the text input underneath.
        // Since the pane is floating, though, the page underneath the pane *adjacent* to the input is
        // also shown unless we cover it up. The inputMask does this by filling up the remaining space
        // based on the width of the input.
        this.inputMask.style.left = elementRect.width + 'px';
        // Add CSS class after one frame to trigger open animation.
        this.$$rAF(function () {
          calendarPane.classList.add('md-pane-open');
        });
      };
      /** Detach the floating calendar pane from the document. */
      DatePickerCtrl.prototype.detachCalendarPane = function () {
        this.$element.removeClass('md-datepicker-open');
        this.calendarPane.classList.remove('md-pane-open');
        this.calendarPane.classList.remove('md-datepicker-pos-adjusted');
        if (this.calendarPane.parentNode) {
          // Use native DOM removal because we do not want any of the angular state of this element
          // to be disposed.
          this.calendarPane.parentNode.removeChild(this.calendarPane);
        }
      };
      /**
   * Open the floating calendar pane.
   * @param {Event} event
   */
      DatePickerCtrl.prototype.openCalendarPane = function (event) {
        if (!this.isCalendarOpen && !this.isDisabled) {
          this.isCalendarOpen = true;
          this.calendarPaneOpenedFrom = event.target;
          // Because the calendar pane is attached directly to the body, it is possible that the
          // rest of the component (input, etc) is in a different scrolling container, such as
          // an md-content. This means that, if the container is scrolled, the pane would remain
          // stationary. To remedy this, we disable scrolling while the calendar pane is open, which
          // also matches the native behavior for things like `<select>` on Mac and Windows.
          this.$mdUtil.disableScrollAround(this.calendarPane);
          this.attachCalendarPane();
          this.focusCalendar();
          // Attach click listener inside of a timeout because, if this open call was triggered by a
          // click, we don't want it to be immediately propogated up to the body and handled.
          var self = this;
          this.$mdUtil.nextTick(function () {
            // Use 'touchstart` in addition to click in order to work on iOS Safari, where click
            // events aren't propogated under most circumstances.
            // See http://www.quirksmode.org/blog/archives/2014/02/mouse_event_bub.html
            self.documentElement.on('click touchstart', self.bodyClickHandler);
          }, false);
          window.addEventListener('resize', this.windowResizeHandler);
        }
      };
      /** Close the floating calendar pane. */
      DatePickerCtrl.prototype.closeCalendarPane = function () {
        if (this.isCalendarOpen) {
          this.isCalendarOpen = false;
          this.detachCalendarPane();
          this.calendarPaneOpenedFrom.focus();
          this.calendarPaneOpenedFrom = null;
          this.$mdUtil.enableScrolling();
          this.documentElement.off('click touchstart', this.bodyClickHandler);
          window.removeEventListener('resize', this.windowResizeHandler);
        }
      };
      /** Gets the controller instance for the calendar in the floating pane. */
      DatePickerCtrl.prototype.getCalendarCtrl = function () {
        return angular.element(this.calendarPane.querySelector('md-calendar')).controller('mdCalendar');
      };
      /** Focus the calendar in the floating pane. */
      DatePickerCtrl.prototype.focusCalendar = function () {
        // Use a timeout in order to allow the calendar to be rendered, as it is gated behind an ng-if.
        var self = this;
        this.$mdUtil.nextTick(function () {
          self.getCalendarCtrl().focus();
        }, false);
      };
      /**
   * Sets whether the input is currently focused.
   * @param {boolean} isFocused
   */
      DatePickerCtrl.prototype.setFocused = function (isFocused) {
        this.isFocused = isFocused;
      };
      /**
   * Handles a click on the document body when the floating calendar pane is open.
   * Closes the floating calendar pane if the click is not inside of it.
   * @param {MouseEvent} event
   */
      DatePickerCtrl.prototype.handleBodyClick = function (event) {
        if (this.isCalendarOpen) {
          // TODO(jelbourn): way want to also include the md-datepicker itself in this check.
          var isInCalendar = this.$mdUtil.getClosest(event.target, 'md-calendar');
          if (!isInCalendar) {
            this.closeCalendarPane();
          }
          this.$scope.$digest();
        }
      };
    }());
  }());
  (function () {
    'use strict';
    (function () {
      'use strict';
      /**
   * Utility for performing date calculations to facilitate operation of the calendar and
   * datepicker.
   */
      angular.module('material.components.datepicker').factory('$$mdDateUtil', function () {
        return {
          getFirstDateOfMonth: getFirstDateOfMonth,
          getNumberOfDaysInMonth: getNumberOfDaysInMonth,
          getDateInNextMonth: getDateInNextMonth,
          getDateInPreviousMonth: getDateInPreviousMonth,
          isInNextMonth: isInNextMonth,
          isInPreviousMonth: isInPreviousMonth,
          getDateMidpoint: getDateMidpoint,
          isSameMonthAndYear: isSameMonthAndYear,
          getWeekOfMonth: getWeekOfMonth,
          incrementDays: incrementDays,
          incrementMonths: incrementMonths,
          getLastDateOfMonth: getLastDateOfMonth,
          isSameDay: isSameDay,
          getMonthDistance: getMonthDistance,
          isValidDate: isValidDate,
          setDateTimeToMidnight: setDateTimeToMidnight,
          createDateAtMidnight: createDateAtMidnight,
          isDateWithinRange: isDateWithinRange
        };
        /**
     * Gets the first day of the month for the given date's month.
     * @param {Date} date
     * @returns {Date}
     */
        function getFirstDateOfMonth(date) {
          return new Date(date.getFullYear(), date.getMonth(), 1);
        }
        /**
     * Gets the number of days in the month for the given date's month.
     * @param date
     * @returns {number}
     */
        function getNumberOfDaysInMonth(date) {
          return new Date(date.getFullYear(), date.getMonth() + 1, 0).getDate();
        }
        /**
     * Get an arbitrary date in the month after the given date's month.
     * @param date
     * @returns {Date}
     */
        function getDateInNextMonth(date) {
          return new Date(date.getFullYear(), date.getMonth() + 1, 1);
        }
        /**
     * Get an arbitrary date in the month before the given date's month.
     * @param date
     * @returns {Date}
     */
        function getDateInPreviousMonth(date) {
          return new Date(date.getFullYear(), date.getMonth() - 1, 1);
        }
        /**
     * Gets whether two dates have the same month and year.
     * @param {Date} d1
     * @param {Date} d2
     * @returns {boolean}
     */
        function isSameMonthAndYear(d1, d2) {
          return d1.getFullYear() === d2.getFullYear() && d1.getMonth() === d2.getMonth();
        }
        /**
     * Gets whether two dates are the same day (not not necesarily the same time).
     * @param {Date} d1
     * @param {Date} d2
     * @returns {boolean}
     */
        function isSameDay(d1, d2) {
          return d1.getDate() == d2.getDate() && isSameMonthAndYear(d1, d2);
        }
        /**
     * Gets whether a date is in the month immediately after some date.
     * @param {Date} startDate The date from which to compare.
     * @param {Date} endDate The date to check.
     * @returns {boolean}
     */
        function isInNextMonth(startDate, endDate) {
          var nextMonth = getDateInNextMonth(startDate);
          return isSameMonthAndYear(nextMonth, endDate);
        }
        /**
     * Gets whether a date is in the month immediately before some date.
     * @param {Date} startDate The date from which to compare.
     * @param {Date} endDate The date to check.
     * @returns {boolean}
     */
        function isInPreviousMonth(startDate, endDate) {
          var previousMonth = getDateInPreviousMonth(startDate);
          return isSameMonthAndYear(endDate, previousMonth);
        }
        /**
     * Gets the midpoint between two dates.
     * @param {Date} d1
     * @param {Date} d2
     * @returns {Date}
     */
        function getDateMidpoint(d1, d2) {
          return createDateAtMidnight((d1.getTime() + d2.getTime()) / 2);
        }
        /**
     * Gets the week of the month that a given date occurs in.
     * @param {Date} date
     * @returns {number} Index of the week of the month (zero-based).
     */
        function getWeekOfMonth(date) {
          var firstDayOfMonth = getFirstDateOfMonth(date);
          return Math.floor((firstDayOfMonth.getDay() + date.getDate() - 1) / 7);
        }
        /**
     * Gets a new date incremented by the given number of days. Number of days can be negative.
     * @param {Date} date
     * @param {number} numberOfDays
     * @returns {Date}
     */
        function incrementDays(date, numberOfDays) {
          return new Date(date.getFullYear(), date.getMonth(), date.getDate() + numberOfDays);
        }
        /**
     * Gets a new date incremented by the given number of months. Number of months can be negative.
     * If the date of the given month does not match the target month, the date will be set to the
     * last day of the month.
     * @param {Date} date
     * @param {number} numberOfMonths
     * @returns {Date}
     */
        function incrementMonths(date, numberOfMonths) {
          // If the same date in the target month does not actually exist, the Date object will
          // automatically advance *another* month by the number of missing days.
          // For example, if you try to go from Jan. 30 to Feb. 30, you'll end up on March 2.
          // So, we check if the month overflowed and go to the last day of the target month instead.
          var dateInTargetMonth = new Date(date.getFullYear(), date.getMonth() + numberOfMonths, 1);
          var numberOfDaysInMonth = getNumberOfDaysInMonth(dateInTargetMonth);
          if (numberOfDaysInMonth < date.getDate()) {
            dateInTargetMonth.setDate(numberOfDaysInMonth);
          } else {
            dateInTargetMonth.setDate(date.getDate());
          }
          return dateInTargetMonth;
        }
        /**
     * Get the integer distance between two months. This *only* considers the month and year
     * portion of the Date instances.
     *
     * @param {Date} start
     * @param {Date} end
     * @returns {number} Number of months between `start` and `end`. If `end` is before `start`
     *     chronologically, this number will be negative.
     */
        function getMonthDistance(start, end) {
          return 12 * (end.getFullYear() - start.getFullYear()) + (end.getMonth() - start.getMonth());
        }
        /**
     * Gets the last day of the month for the given date.
     * @param {Date} date
     * @returns {Date}
     */
        function getLastDateOfMonth(date) {
          return new Date(date.getFullYear(), date.getMonth(), getNumberOfDaysInMonth(date));
        }
        /**
     * Checks whether a date is valid.
     * @param {Date} date
     * @return {boolean} Whether the date is a valid Date.
     */
        function isValidDate(date) {
          return date != null && date.getTime && !isNaN(date.getTime());
        }
        /**
     * Sets a date's time to midnight.
     * @param {Date} date
     */
        function setDateTimeToMidnight(date) {
          if (isValidDate(date)) {
            date.setHours(0, 0, 0, 0);
          }
        }
        /**
     * Creates a date with the time set to midnight.
     * Drop-in replacement for two forms of the Date constructor:
     * 1. No argument for Date representing now.
     * 2. Single-argument value representing number of seconds since Unix Epoch.
     * @param {number=} opt_value
     * @return {Date} New date with time set to midnight.
     */
        function createDateAtMidnight(opt_value) {
          var date;
          if (angular.isUndefined(opt_value)) {
            date = new Date();
          } else {
            date = new Date(opt_value);
          }
          setDateTimeToMidnight(date);
          return date;
        }
        /**
      * Checks if a date is within a min and max range.
      * If minDate or maxDate are not dates, they are ignored.
      * @param {Date} date
      * @param {Date} minDate
      * @param {Date} maxDate
      */
        function isDateWithinRange(date, minDate, maxDate) {
          return (!angular.isDate(minDate) || minDate <= date) && (!angular.isDate(maxDate) || maxDate >= date);
        }
      });
    }());
  }());
  (function () {
    'use strict';
    /**
 * @ngdoc module
 * @name material.components.dialog
 */
    angular.module('material.components.dialog', [
      'material.core',
      'material.components.backdrop'
    ]).directive('mdDialog', MdDialogDirective).provider('$mdDialog', MdDialogProvider);
    /**
 * @ngdoc directive
 * @name mdDialog
 * @module material.components.dialog
 *
 * @restrict E
 *
 * @description
 * `<md-dialog>` - The dialog's template must be inside this element.
 *
 * Inside, use an `<md-dialog-content>` element for the dialog's content, and use
 * an `<md-dialog-actions>` element for the dialog's actions.
 *
 * * ## CSS
 * - `.md-dialog-content` - class that sets the padding on the content as the spec file
 *
 * @usage
 * ### Dialog template
 * <hljs lang="html">
 * <md-dialog aria-label="List dialog">
 *   <md-dialog-content>
 *     <md-list>
 *       <md-list-item ng-repeat="item in items">
 *         <p>Number {{item}}</p>
 *       </md-list-item>
 *     </md-list>
 *   </md-dialog-content>
 *   <md-dialog-actions>
 *     <md-button ng-click="closeDialog()" class="md-primary">Close Dialog</md-button>
 *   </md-dialog-actions>
 * </md-dialog>
 * </hljs>
 */
    function MdDialogDirective($$rAF, $mdTheming, $mdDialog) {
      return {
        restrict: 'E',
        link: function (scope, element, attr) {
          $mdTheming(element);
          $$rAF(function () {
            var images;
            var content = element[0].querySelector('md-dialog-content');
            if (content) {
              images = content.getElementsByTagName('img');
              addOverflowClass();
              //-- delayed image loading may impact scroll height, check after images are loaded
              angular.element(images).on('load', addOverflowClass);
            }
            scope.$on('$destroy', function () {
              $mdDialog.destroy(element);
            });
            /**
         *
         */
            function addOverflowClass() {
              element.toggleClass('md-content-overflow', content.scrollHeight > content.clientHeight);
            }
          });
        }
      };
    }
    MdDialogDirective.$inject = [
      '$$rAF',
      '$mdTheming',
      '$mdDialog'
    ];
    /**
 * @ngdoc service
 * @name $mdDialog
 * @module material.components.dialog
 *
 * @description
 * `$mdDialog` opens a dialog over the app to inform users about critical information or require
 *  them to make decisions. There are two approaches for setup: a simple promise API
 *  and regular object syntax.
 *
 * ## Restrictions
 *
 * - The dialog is always given an isolate scope.
 * - The dialog's template must have an outer `<md-dialog>` element.
 *   Inside, use an `<md-dialog-content>` element for the dialog's content, and use
 *   an `<md-dialog-actions>` element for the dialog's actions.
 * - Dialogs must cover the entire application to keep interactions inside of them.
 * Use the `parent` option to change where dialogs are appended.
 *
 * ## Sizing
 * - Complex dialogs can be sized with `flex="percentage"`, i.e. `flex="66"`.
 * - Default max-width is 80% of the `rootElement` or `parent`.
 *
 * ## CSS
 * - `.md-dialog-content` - class that sets the padding on the content as the spec file
 *
 * @usage
 * <hljs lang="html">
 * <div  ng-app="demoApp" ng-controller="EmployeeController">
 *   <div>
 *     <md-button ng-click="showAlert()" class="md-raised md-warn">
 *       Employee Alert!
 *       </md-button>
 *   </div>
 *   <div>
 *     <md-button ng-click="showDialog($event)" class="md-raised">
 *       Custom Dialog
 *       </md-button>
 *   </div>
 *   <div>
 *     <md-button ng-click="closeAlert()" ng-disabled="!hasAlert()" class="md-raised">
 *       Close Alert
 *     </md-button>
 *   </div>
 *   <div>
 *     <md-button ng-click="showGreeting($event)" class="md-raised md-primary" >
 *       Greet Employee
 *       </md-button>
 *   </div>
 * </div>
 * </hljs>
 *
 * ### JavaScript: object syntax
 * <hljs lang="js">
 * (function(angular, undefined){
 *   "use strict";
 *
 *   angular
 *    .module('demoApp', ['ngMaterial'])
 *    .controller('AppCtrl', AppController);
 *
 *   function AppController($scope, $mdDialog) {
 *     var alert;
 *     $scope.showAlert = showAlert;
 *     $scope.showDialog = showDialog;
 *     $scope.items = [1, 2, 3];
 *
 *     // Internal method
 *     function showAlert() {
 *       alert = $mdDialog.alert({
 *         title: 'Attention',
 *         textContent: 'This is an example of how easy dialogs can be!',
 *         ok: 'Close'
 *       });
 *
 *       $mdDialog
 *         .show( alert )
 *         .finally(function() {
 *           alert = undefined;
 *         });
 *     }
 *
 *     function showDialog($event) {
 *        var parentEl = angular.element(document.body);
 *        $mdDialog.show({
 *          parent: parentEl,
 *          targetEvent: $event,
 *          template:
 *            '<md-dialog aria-label="List dialog">' +
 *            '  <md-dialog-content>'+
 *            '    <md-list>'+
 *            '      <md-list-item ng-repeat="item in items">'+
 *            '       <p>Number {{item}}</p>' +
 *            '      </md-item>'+
 *            '    </md-list>'+
 *            '  </md-dialog-content>' +
 *            '  <md-dialog-actions>' +
 *            '    <md-button ng-click="closeDialog()" class="md-primary">' +
 *            '      Close Dialog' +
 *            '    </md-button>' +
 *            '  </md-dialog-actions>' +
 *            '</md-dialog>',
 *          locals: {
 *            items: $scope.items
 *          },
 *          controller: DialogController
 *       });
 *       function DialogController($scope, $mdDialog, items) {
 *         $scope.items = items;
 *         $scope.closeDialog = function() {
 *           $mdDialog.hide();
 *         }
 *       }
 *     }
 *   }
 * })(angular);
 * </hljs>
 *
 * ### JavaScript: promise API syntax, custom dialog template
 * <hljs lang="js">
 * (function(angular, undefined){
 *   "use strict";
 *
 *   angular
 *     .module('demoApp', ['ngMaterial'])
 *     .controller('EmployeeController', EmployeeEditor)
 *     .controller('GreetingController', GreetingController);
 *
 *   // Fictitious Employee Editor to show how to use simple and complex dialogs.
 *
 *   function EmployeeEditor($scope, $mdDialog) {
 *     var alert;
 *
 *     $scope.showAlert = showAlert;
 *     $scope.closeAlert = closeAlert;
 *     $scope.showGreeting = showCustomGreeting;
 *
 *     $scope.hasAlert = function() { return !!alert };
 *     $scope.userName = $scope.userName || 'Bobby';
 *
 *     // Dialog #1 - Show simple alert dialog and cache
 *     // reference to dialog instance
 *
 *     function showAlert() {
 *       alert = $mdDialog.alert()
 *         .title('Attention, ' + $scope.userName)
 *         .textContent('This is an example of how easy dialogs can be!')
 *         .ok('Close');
 *
 *       $mdDialog
 *           .show( alert )
 *           .finally(function() {
 *             alert = undefined;
 *           });
 *     }
 *
 *     // Close the specified dialog instance and resolve with 'finished' flag
 *     // Normally this is not needed, just use '$mdDialog.hide()' to close
 *     // the most recent dialog popup.
 *
 *     function closeAlert() {
 *       $mdDialog.hide( alert, "finished" );
 *       alert = undefined;
 *     }
 *
 *     // Dialog #2 - Demonstrate more complex dialogs construction and popup.
 *
 *     function showCustomGreeting($event) {
 *         $mdDialog.show({
 *           targetEvent: $event,
 *           template:
 *             '<md-dialog>' +
 *
 *             '  <md-dialog-content>Hello {{ employee }}!</md-dialog-content>' +
 *
 *             '  <md-dialog-actions>' +
 *             '    <md-button ng-click="closeDialog()" class="md-primary">' +
 *             '      Close Greeting' +
 *             '    </md-button>' +
 *             '  </md-dialog-actions>' +
 *             '</md-dialog>',
 *           controller: 'GreetingController',
 *           onComplete: afterShowAnimation,
 *           locals: { employee: $scope.userName }
 *         });
 *
 *         // When the 'enter' animation finishes...
 *
 *         function afterShowAnimation(scope, element, options) {
 *            // post-show code here: DOM element focus, etc.
 *         }
 *     }
 *
 *     // Dialog #3 - Demonstrate use of ControllerAs and passing $scope to dialog
 *     //             Here we used ng-controller="GreetingController as vm" and
 *     //             $scope.vm === <controller instance>
 *
 *     function showCustomGreeting() {
 *
 *        $mdDialog.show({
 *           clickOutsideToClose: true,
 *
 *           scope: $scope,        // use parent scope in template
 *           preserveScope: true,  // do not forget this if use parent scope

 *           // Since GreetingController is instantiated with ControllerAs syntax
 *           // AND we are passing the parent '$scope' to the dialog, we MUST
 *           // use 'vm.<xxx>' in the template markup
 *
 *           template: '<md-dialog>' +
 *                     '  <md-dialog-content>' +
 *                     '     Hi There {{vm.employee}}' +
 *                     '  </md-dialog-content>' +
 *                     '</md-dialog>',
 *
 *           controller: function DialogController($scope, $mdDialog) {
 *             $scope.closeDialog = function() {
 *               $mdDialog.hide();
 *             }
 *           }
 *        });
 *     }
 *
 *   }
 *
 *   // Greeting controller used with the more complex 'showCustomGreeting()' custom dialog
 *
 *   function GreetingController($scope, $mdDialog, employee) {
 *     // Assigned from construction <code>locals</code> options...
 *     $scope.employee = employee;
 *
 *     $scope.closeDialog = function() {
 *       // Easily hides most recent dialog shown...
 *       // no specific instance reference is needed.
 *       $mdDialog.hide();
 *     };
 *   }
 *
 * })(angular);
 * </hljs>
 */
    /**
 * @ngdoc method
 * @name $mdDialog#alert
 *
 * @description
 * Builds a preconfigured dialog with the specified message.
 *
 * @returns {obj} an `$mdDialogPreset` with the chainable configuration methods:
 *
 * - $mdDialogPreset#title(string) - Sets the alert title.
 * - $mdDialogPreset#textContent(string) - Sets the alert message.
 * - $mdDialogPreset#htmlContent(string) - Sets the alert message as HTML. Requires ngSanitize
 *     module to be loaded. HTML is not run through Angular's compiler.
 * - $mdDialogPreset#ok(string) - Sets the alert "Okay" button text.
 * - $mdDialogPreset#theme(string) - Sets the theme of the alert dialog.
 * - $mdDialogPreset#targetEvent(DOMClickEvent=) - A click's event object. When passed in as an option,
 *     the location of the click will be used as the starting point for the opening animation
 *     of the the dialog.
 *
 */
    /**
 * @ngdoc method
 * @name $mdDialog#confirm
 *
 * @description
 * Builds a preconfigured dialog with the specified message. You can call show and the promise returned
 * will be resolved only if the user clicks the confirm action on the dialog.
 *
 * @returns {obj} an `$mdDialogPreset` with the chainable configuration methods:
 *
 * Additionally, it supports the following methods:
 *
 * - $mdDialogPreset#title(string) - Sets the confirm title.
 * - $mdDialogPreset#textContent(string) - Sets the confirm message.
 * - $mdDialogPreset#htmlContent(string) - Sets the confirm message as HTML. Requires ngSanitize
 *     module to be loaded. HTML is not run through Angular's compiler.
 * - $mdDialogPreset#ok(string) - Sets the confirm "Okay" button text.
 * - $mdDialogPreset#cancel(string) - Sets the confirm "Cancel" button text.
 * - $mdDialogPreset#theme(string) - Sets the theme of the confirm dialog.
 * - $mdDialogPreset#targetEvent(DOMClickEvent=) - A click's event object. When passed in as an option,
 *     the location of the click will be used as the starting point for the opening animation
 *     of the the dialog.
 *
 */
    /**
 * @ngdoc method
 * @name $mdDialog#show
 *
 * @description
 * Show a dialog with the specified options.
 *
 * @param {object} optionsOrPreset Either provide an `$mdDialogPreset` returned from `alert()`, and
 * `confirm()`, or an options object with the following properties:
 *   - `templateUrl` - `{string=}`: The url of a template that will be used as the content
 *   of the dialog.
 *   - `template` - `{string=}`: HTML template to show in the dialog. This **must** be trusted HTML
 *      with respect to Angular's [$sce service](https://docs.angularjs.org/api/ng/service/$sce).
 *      This template should **never** be constructed with any kind of user input or user data.
 *   - `autoWrap` - `{boolean=}`: Whether or not to automatically wrap the template with a
 *     `<md-dialog>` tag if one is not provided. Defaults to true. Can be disabled if you provide a
 *     custom dialog directive.
 *   - `targetEvent` - `{DOMClickEvent=}`: A click's event object. When passed in as an option,
 *     the location of the click will be used as the starting point for the opening animation
 *     of the the dialog.
 *   - `openFrom` - `{string|Element|object}`: The query selector, DOM element or the Rect object
 *     that is used to determine the bounds (top, left, height, width) from which the Dialog will
 *     originate.
 *   - `closeTo` - `{string|Element|object}`: The query selector, DOM element or the Rect object
 *     that is used to determine the bounds (top, left, height, width) to which the Dialog will
 *     target.
 *   - `scope` - `{object=}`: the scope to link the template / controller to. If none is specified,
 *     it will create a new isolate scope.
 *     This scope will be destroyed when the dialog is removed unless `preserveScope` is set to true.
 *   - `preserveScope` - `{boolean=}`: whether to preserve the scope when the element is removed. Default is false
 *   - `disableParentScroll` - `{boolean=}`: Whether to disable scrolling while the dialog is open.
 *     Default true.
 *   - `hasBackdrop` - `{boolean=}`: Whether there should be an opaque backdrop behind the dialog.
 *     Default true.
 *   - `clickOutsideToClose` - `{boolean=}`: Whether the user can click outside the dialog to
 *     close it. Default false.
 *   - `escapeToClose` - `{boolean=}`: Whether the user can press escape to close the dialog.
 *     Default true.
 *   - `focusOnOpen` - `{boolean=}`: An option to override focus behavior on open. Only disable if
 *     focusing some other way, as focus management is required for dialogs to be accessible.
 *     Defaults to true.
 *   - `controller` - `{function|string=}`: The controller to associate with the dialog. The controller
 *     will be injected with the local `$mdDialog`, which passes along a scope for the dialog.
 *   - `locals` - `{object=}`: An object containing key/value pairs. The keys will be used as names
 *     of values to inject into the controller. For example, `locals: {three: 3}` would inject
 *     `three` into the controller, with the value 3. If `bindToController` is true, they will be
 *     copied to the controller instead.
 *   - `bindToController` - `bool`: bind the locals to the controller, instead of passing them in.
 *     These values will not be available until after initialization.
 *   - `resolve` - `{object=}`: Similar to locals, except it takes promises as values, and the
 *     dialog will not open until all of the promises resolve.
 *   - `controllerAs` - `{string=}`: An alias to assign the controller to on the scope.
 *   - `parent` - `{element=}`: The element to append the dialog to. Defaults to appending
 *     to the root element of the application.
 *   - `onShowing` `{function=} Callback function used to announce the show() action is
 *     starting.
 *   - `onComplete` `{function=}`: Callback function used to announce when the show() action is
 *     finished.
 *   - `onRemoving` `{function=}`: Callback function used to announce the close/hide() action is
 *     starting. This allows developers to run custom animations in parallel the close animations.
 *   - `fullscreen` `{boolean=}`: An option to apply `.md-dialog-fullscreen` class on open.
 * @returns {promise} A promise that can be resolved with `$mdDialog.hide()` or
 * rejected with `$mdDialog.cancel()`.
 */
    /**
 * @ngdoc method
 * @name $mdDialog#hide
 *
 * @description
 * Hide an existing dialog and resolve the promise returned from `$mdDialog.show()`.
 *
 * @param {*=} response An argument for the resolved promise.
 *
 * @returns {promise} A promise that is resolved when the dialog has been closed.
 */
    /**
 * @ngdoc method
 * @name $mdDialog#cancel
 *
 * @description
 * Hide an existing dialog and reject the promise returned from `$mdDialog.show()`.
 *
 * @param {*=} response An argument for the rejected promise.
 *
 * @returns {promise} A promise that is resolved when the dialog has been closed.
 */
    function MdDialogProvider($$interimElementProvider) {
      // Elements to capture and redirect focus when the user presses tab at the dialog boundary.
      var topFocusTrap, bottomFocusTrap;
      advancedDialogOptions.$inject = [
        '$mdDialog',
        '$mdTheming'
      ];
      dialogDefaultOptions.$inject = [
        '$mdDialog',
        '$mdAria',
        '$mdUtil',
        '$mdConstant',
        '$animate',
        '$document',
        '$window',
        '$rootElement',
        '$log',
        '$injector'
      ];
      return $$interimElementProvider('$mdDialog').setDefaults({
        methods: [
          'disableParentScroll',
          'hasBackdrop',
          'clickOutsideToClose',
          'escapeToClose',
          'targetEvent',
          'closeTo',
          'openFrom',
          'parent',
          'fullscreen'
        ],
        options: dialogDefaultOptions
      }).addPreset('alert', {
        methods: [
          'title',
          'htmlContent',
          'textContent',
          'content',
          'ariaLabel',
          'ok',
          'theme',
          'css'
        ],
        options: advancedDialogOptions
      }).addPreset('confirm', {
        methods: [
          'title',
          'htmlContent',
          'textContent',
          'content',
          'ariaLabel',
          'ok',
          'cancel',
          'theme',
          'css'
        ],
        options: advancedDialogOptions
      });
      /* @ngInject */
      function advancedDialogOptions($mdDialog, $mdTheming) {
        return {
          template: [
            '<md-dialog md-theme="{{ dialog.theme }}" aria-label="{{ dialog.ariaLabel }}" ng-class="dialog.css">',
            '  <md-dialog-content class="md-dialog-content" role="document" tabIndex="-1">',
            '    <h2 class="md-title">{{ dialog.title }}</h2>',
            '    <div ng-if="::dialog.mdHtmlContent" class="md-dialog-content-body" ',
            '        ng-bind-html="::dialog.mdHtmlContent"></div>',
            '    <div ng-if="::!dialog.mdHtmlContent" class="md-dialog-content-body">',
            '      <p>{{::dialog.mdTextContent}}</p>',
            '    </div>',
            '  </md-dialog-content>',
            '  <md-dialog-actions>',
            '    <md-button ng-if="dialog.$type == \'confirm\'"' + '               ng-click="dialog.abort()" class="md-primary">',
            '      {{ dialog.cancel }}',
            '    </md-button>',
            '    <md-button ng-click="dialog.hide()" class="md-primary" md-autofocus="dialog.$type!=\'confirm\'">',
            '      {{ dialog.ok }}',
            '    </md-button>',
            '  </md-dialog-actions>',
            '</md-dialog>'
          ].join('').replace(/\s\s+/g, ''),
          controller: function mdDialogCtrl() {
            this.hide = function () {
              $mdDialog.hide(true);
            };
            this.abort = function () {
              $mdDialog.cancel();
            };
          },
          controllerAs: 'dialog',
          bindToController: true,
          theme: $mdTheming.defaultTheme()
        };
      }
      /* @ngInject */
      function dialogDefaultOptions($mdDialog, $mdAria, $mdUtil, $mdConstant, $animate, $document, $window, $rootElement, $log, $injector) {
        return {
          hasBackdrop: true,
          isolateScope: true,
          onShow: onShow,
          onShowing: beforeShow,
          onRemove: onRemove,
          clickOutsideToClose: false,
          escapeToClose: true,
          targetEvent: null,
          closeTo: null,
          openFrom: null,
          focusOnOpen: true,
          disableParentScroll: true,
          autoWrap: true,
          fullscreen: false,
          transformTemplate: function (template, options) {
            return '<div class="md-dialog-container">' + validatedTemplate(template) + '</div>';
            /**
         * The specified template should contain a <md-dialog> wrapper element....
         */
            function validatedTemplate(template) {
              if (options.autoWrap && !/<\/md-dialog>/g.test(template)) {
                return '<md-dialog>' + (template || '') + '</md-dialog>';
              } else {
                return template || '';
              }
            }
          }
        };
        function beforeShow(scope, element, options, controller) {
          if (controller) {
            controller.mdHtmlContent = controller.htmlContent || options.htmlContent || '';
            controller.mdTextContent = controller.textContent || options.textContent || controller.content || options.content || '';
            if (controller.mdHtmlContent && !$injector.has('$sanitize')) {
              throw Error('The ngSanitize module must be loaded in order to use htmlContent.');
            }
            if (controller.mdHtmlContent && controller.mdTextContent) {
              throw Error('md-dialog cannot have both `htmlContent` and `textContent`');
            }
          }
        }
        /** Show method for dialogs */
        function onShow(scope, element, options, controller) {
          angular.element($document[0].body).addClass('md-dialog-is-showing');
          captureParentAndFromToElements(options);
          configureAria(element.find('md-dialog'), options);
          showBackdrop(scope, element, options);
          return dialogPopIn(element, options).then(function () {
            activateListeners(element, options);
            lockScreenReader(element, options);
            warnDeprecatedActions();
            focusOnOpen();
          });
          /**
       * Check to see if they used the deprecated .md-actions class and log a warning
       */
          function warnDeprecatedActions() {
            var badActions = element[0].querySelectorAll('.md-actions');
            if (badActions.length > 0) {
              $log.warn('Using a class of md-actions is deprected, please use <md-dialog-actions>.');
            }
          }
          /**
       * For alerts, focus on content... otherwise focus on
       * the close button (or equivalent)
       */
          function focusOnOpen() {
            if (options.focusOnOpen) {
              var target = $mdUtil.findFocusTarget(element) || findCloseButton();
              target.focus();
            }
            /**
         * If no element with class dialog-close, try to find the last
         * button child in md-actions and assume it is a close button.
         *
         * If we find no actions at all, log a warning to the console.
         */
            function findCloseButton() {
              var closeButton = element[0].querySelector('.dialog-close');
              if (!closeButton) {
                var actionButtons = element[0].querySelectorAll('.md-actions button, md-dialog-actions button');
                closeButton = actionButtons[actionButtons.length - 1];
              }
              return angular.element(closeButton);
            }
          }
        }
        /**
     * Remove function for all dialogs
     */
        function onRemove(scope, element, options) {
          options.deactivateListeners();
          options.unlockScreenReader();
          options.hideBackdrop(options.$destroy);
          // Remove the focus traps that we added earlier for keeping focus within the dialog.
          if (topFocusTrap && topFocusTrap.parentNode) {
            topFocusTrap.parentNode.removeChild(topFocusTrap);
          }
          if (bottomFocusTrap && bottomFocusTrap.parentNode) {
            bottomFocusTrap.parentNode.removeChild(bottomFocusTrap);
          }
          // For navigation $destroy events, do a quick, non-animated removal,
          // but for normal closes (from clicks, etc) animate the removal
          return !!options.$destroy ? detachAndClean() : animateRemoval().then(detachAndClean);
          /**
       * For normal closes, animate the removal.
       * For forced closes (like $destroy events), skip the animations
       */
          function animateRemoval() {
            return dialogPopOut(element, options);
          }
          /**
       * Detach the element
       */
          function detachAndClean() {
            angular.element($document[0].body).removeClass('md-dialog-is-showing');
            element.remove();
            if (!options.$destroy)
              options.origin.focus();
          }
        }
        /**
     * Capture originator/trigger/from/to element information (if available)
     * and the parent container for the dialog; defaults to the $rootElement
     * unless overridden in the options.parent
     */
        function captureParentAndFromToElements(options) {
          options.origin = angular.extend({
            element: null,
            bounds: null,
            focus: angular.noop
          }, options.origin || {});
          options.parent = getDomElement(options.parent, $rootElement);
          options.closeTo = getBoundingClientRect(getDomElement(options.closeTo));
          options.openFrom = getBoundingClientRect(getDomElement(options.openFrom));
          if (options.targetEvent) {
            options.origin = getBoundingClientRect(options.targetEvent.target, options.origin);
          }
          /**
           * Identify the bounding RECT for the target element
           *
           */
          function getBoundingClientRect(element, orig) {
            var source = angular.element(element || {});
            if (source && source.length) {
              // Compute and save the target element's bounding rect, so that if the
              // element is hidden when the dialog closes, we can shrink the dialog
              // back to the same position it expanded from.
              //
              // Checking if the source is a rect object or a DOM element
              var bounds = {
                  top: 0,
                  left: 0,
                  height: 0,
                  width: 0
                };
              var hasFn = angular.isFunction(source[0].getBoundingClientRect);
              return angular.extend(orig || {}, {
                element: hasFn ? source : undefined,
                bounds: hasFn ? source[0].getBoundingClientRect() : angular.extend({}, bounds, source[0]),
                focus: angular.bind(source, source.focus)
              });
            }
          }
          /**
           * If the specifier is a simple string selector, then query for
           * the DOM element.
           */
          function getDomElement(element, defaultElement) {
            if (angular.isString(element)) {
              var simpleSelector = element, container = $document[0].querySelectorAll(simpleSelector);
              element = container.length ? container[0] : null;
            }
            // If we have a reference to a raw dom element, always wrap it in jqLite
            return angular.element(element || defaultElement);
          }
        }
        /**
     * Listen for escape keys and outside clicks to auto close
     */
        function activateListeners(element, options) {
          var window = angular.element($window);
          var onWindowResize = $mdUtil.debounce(function () {
              stretchDialogContainerToViewport(element, options);
            }, 60);
          var removeListeners = [];
          var smartClose = function () {
            // Only 'confirm' dialogs have a cancel button... escape/clickOutside will
            // cancel or fallback to hide.
            var closeFn = options.$type == 'alert' ? $mdDialog.hide : $mdDialog.cancel;
            $mdUtil.nextTick(closeFn, true);
          };
          if (options.escapeToClose) {
            var target = options.parent;
            var keyHandlerFn = function (ev) {
              if (ev.keyCode === $mdConstant.KEY_CODE.ESCAPE) {
                ev.stopPropagation();
                ev.preventDefault();
                smartClose();
              }
            };
            // Add keydown listeners
            element.on('keydown', keyHandlerFn);
            target.on('keydown', keyHandlerFn);
            window.on('resize', onWindowResize);
            // Queue remove listeners function
            removeListeners.push(function () {
              element.off('keydown', keyHandlerFn);
              target.off('keydown', keyHandlerFn);
              window.off('resize', onWindowResize);
            });
          }
          if (options.clickOutsideToClose) {
            var target = element;
            var sourceElem;
            // Keep track of the element on which the mouse originally went down
            // so that we can only close the backdrop when the 'click' started on it.
            // A simple 'click' handler does not work,
            // it sets the target object as the element the mouse went down on.
            var mousedownHandler = function (ev) {
              sourceElem = ev.target;
            };
            // We check if our original element and the target is the backdrop
            // because if the original was the backdrop and the target was inside the dialog
            // we don't want to dialog to close.
            var mouseupHandler = function (ev) {
              if (sourceElem === target[0] && ev.target === target[0]) {
                ev.stopPropagation();
                ev.preventDefault();
                smartClose();
              }
            };
            // Add listeners
            target.on('mousedown', mousedownHandler);
            target.on('mouseup', mouseupHandler);
            // Queue remove listeners function
            removeListeners.push(function () {
              target.off('mousedown', mousedownHandler);
              target.off('mouseup', mouseupHandler);
            });
          }
          // Attach specific `remove` listener handler
          options.deactivateListeners = function () {
            removeListeners.forEach(function (removeFn) {
              removeFn();
            });
            options.deactivateListeners = null;
          };
        }
        /**
     * Show modal backdrop element...
     */
        function showBackdrop(scope, element, options) {
          if (options.disableParentScroll) {
            // !! DO this before creating the backdrop; since disableScrollAround()
            //    configures the scroll offset; which is used by mdBackDrop postLink()
            options.restoreScroll = $mdUtil.disableScrollAround(element, options.parent);
          }
          if (options.hasBackdrop) {
            options.backdrop = $mdUtil.createBackdrop(scope, 'md-dialog-backdrop md-opaque');
            $animate.enter(options.backdrop, options.parent);
          }
          /**
       * Hide modal backdrop element...
       */
          options.hideBackdrop = function hideBackdrop($destroy) {
            if (options.backdrop) {
              if (!!$destroy)
                options.backdrop.remove();
              else
                $animate.leave(options.backdrop);
            }
            if (options.disableParentScroll) {
              options.restoreScroll();
              delete options.restoreScroll;
            }
            options.hideBackdrop = null;
          };
        }
        /**
     * Inject ARIA-specific attributes appropriate for Dialogs
     */
        function configureAria(element, options) {
          var role = options.$type === 'alert' ? 'alertdialog' : 'dialog';
          var dialogContent = element.find('md-dialog-content');
          var dialogId = element.attr('id') || 'dialog_' + $mdUtil.nextUid();
          element.attr({
            'role': role,
            'tabIndex': '-1'
          });
          if (dialogContent.length === 0) {
            dialogContent = element;
          }
          dialogContent.attr('id', dialogId);
          element.attr('aria-describedby', dialogId);
          if (options.ariaLabel) {
            $mdAria.expect(element, 'aria-label', options.ariaLabel);
          } else {
            $mdAria.expectAsync(element, 'aria-label', function () {
              var words = dialogContent.text().split(/\s+/);
              if (words.length > 3)
                words = words.slice(0, 3).concat('...');
              return words.join(' ');
            });
          }
          // Set up elements before and after the dialog content to capture focus and
          // redirect back into the dialog.
          topFocusTrap = document.createElement('div');
          topFocusTrap.classList.add('md-dialog-focus-trap');
          topFocusTrap.tabIndex = 0;
          bottomFocusTrap = topFocusTrap.cloneNode(false);
          // When focus is about to move out of the dialog, we want to intercept it and redirect it
          // back to the dialog element.
          var focusHandler = angular.bind(element, element.focus);
          topFocusTrap.addEventListener('focus', focusHandler);
          bottomFocusTrap.addEventListener('focus', focusHandler);
          // The top focus trap inserted immeidately before the md-dialog element (as a sibling).
          // The bottom focus trap is inserted at the very end of the md-dialog element (as a child).
          element[0].parentNode.insertBefore(topFocusTrap, element[0]);
          element.append(bottomFocusTrap);
        }
        /**
     * Prevents screen reader interaction behind modal window
     * on swipe interfaces
     */
        function lockScreenReader(element, options) {
          var isHidden = true;
          // get raw DOM node
          walkDOM(element[0]);
          options.unlockScreenReader = function () {
            isHidden = false;
            walkDOM(element[0]);
            options.unlockScreenReader = null;
          };
          /**
       * Walk DOM to apply or remove aria-hidden on sibling nodes
       * and parent sibling nodes
       *
       */
          function walkDOM(element) {
            while (element.parentNode) {
              if (element === document.body) {
                return;
              }
              var children = element.parentNode.children;
              for (var i = 0; i < children.length; i++) {
                // skip over child if it is an ascendant of the dialog
                // or a script or style tag
                if (element !== children[i] && !isNodeOneOf(children[i], [
                    'SCRIPT',
                    'STYLE'
                  ])) {
                  children[i].setAttribute('aria-hidden', isHidden);
                }
              }
              walkDOM(element = element.parentNode);
            }
          }
        }
        /**
     * Ensure the dialog container fill-stretches to the viewport
     */
        function stretchDialogContainerToViewport(container, options) {
          var isFixed = $window.getComputedStyle($document[0].body).position == 'fixed';
          var backdrop = options.backdrop ? $window.getComputedStyle(options.backdrop[0]) : null;
          var height = backdrop ? Math.min($document[0].body.clientHeight, Math.ceil(Math.abs(parseInt(backdrop.height, 10)))) : 0;
          container.css({
            top: (isFixed ? $mdUtil.scrollTop(options.parent) : 0) + 'px',
            height: height ? height + 'px' : '100%'
          });
          return container;
        }
        /**
     *  Dialog open and pop-in animation
     */
        function dialogPopIn(container, options) {
          // Add the `md-dialog-container` to the DOM
          options.parent.append(container);
          stretchDialogContainerToViewport(container, options);
          var dialogEl = container.find('md-dialog');
          var animator = $mdUtil.dom.animator;
          var buildTranslateToOrigin = animator.calculateZoomToOrigin;
          var translateOptions = {
              transitionInClass: 'md-transition-in',
              transitionOutClass: 'md-transition-out'
            };
          var from = animator.toTransformCss(buildTranslateToOrigin(dialogEl, options.openFrom || options.origin));
          var to = animator.toTransformCss('');
          // defaults to center display (or parent or $rootElement)
          if (options.fullscreen) {
            dialogEl.addClass('md-dialog-fullscreen');
          }
          return animator.translate3d(dialogEl, from, to, translateOptions).then(function (animateReversal) {
            // Build a reversal translate function synched to this translation...
            options.reverseAnimate = function () {
              delete options.reverseAnimate;
              if (options.closeTo) {
                // Using the opposite classes to create a close animation to the closeTo element
                translateOptions = {
                  transitionInClass: 'md-transition-out',
                  transitionOutClass: 'md-transition-in'
                };
                from = to;
                to = animator.toTransformCss(buildTranslateToOrigin(dialogEl, options.closeTo));
                return animator.translate3d(dialogEl, from, to, translateOptions);
              }
              return animateReversal(animator.toTransformCss(buildTranslateToOrigin(dialogEl, options.origin)));
            };
            return true;
          });
        }
        /**
     * Dialog close and pop-out animation
     */
        function dialogPopOut(container, options) {
          return options.reverseAnimate();
        }
        /**
     * Utility function to filter out raw DOM nodes
     */
        function isNodeOneOf(elem, nodeTypeArray) {
          if (nodeTypeArray.indexOf(elem.nodeName) !== -1) {
            return true;
          }
        }
      }
    }
    MdDialogProvider.$inject = ['$$interimElementProvider'];
  }());
  (function () {
    'use strict';
    /**
 * @ngdoc module
 * @name material.components.divider
 * @description Divider module!
 */
    angular.module('material.components.divider', ['material.core']).directive('mdDivider', MdDividerDirective);
    /**
 * @ngdoc directive
 * @name mdDivider
 * @module material.components.divider
 * @restrict E
 *
 * @description
 * Dividers group and separate content within lists and page layouts using strong visual and spatial distinctions. This divider is a thin rule, lightweight enough to not distract the user from content.
 *
 * @param {boolean=} md-inset Add this attribute to activate the inset divider style.
 * @usage
 * <hljs lang="html">
 * <md-divider></md-divider>
 *
 * <md-divider md-inset></md-divider>
 * </hljs>
 *
 */
    function MdDividerDirective($mdTheming) {
      return {
        restrict: 'E',
        link: $mdTheming
      };
    }
    MdDividerDirective.$inject = ['$mdTheming'];
  }());
  (function () {
    'use strict';
    (function () {
      'use strict';
      /**
   * @ngdoc module
   * @name material.components.fabActions
   */
      angular.module('material.components.fabActions', ['material.core']).directive('mdFabActions', MdFabActionsDirective);
      /**
   * @ngdoc directive
   * @name mdFabActions
   * @module material.components.fabActions
   *
   * @restrict E
   *
   * @description
   * The `<md-fab-actions>` directive is used inside of a `<md-fab-speed-dial>` or
   * `<md-fab-toolbar>` directive to mark an element (or elements) as the actions and setup the
   * proper event listeners.
   *
   * @usage
   * See the `<md-fab-speed-dial>` or `<md-fab-toolbar>` directives for example usage.
   */
      function MdFabActionsDirective() {
        return {
          restrict: 'E',
          require: [
            '^?mdFabSpeedDial',
            '^?mdFabToolbar'
          ],
          compile: function (element, attributes) {
            var children = element.children();
            var hasNgRepeat = false;
            angular.forEach([
              '',
              'data-',
              'x-'
            ], function (prefix) {
              hasNgRepeat = hasNgRepeat || (children.attr(prefix + 'ng-repeat') ? true : false);
            });
            // Support both ng-repeat and static content
            if (hasNgRepeat) {
              children.addClass('md-fab-action-item');
            } else {
              // Wrap every child in a new div and add a class that we can scale/fling independently
              children.wrap('<div class="md-fab-action-item">');
            }
          }
        };
      }
    }());
  }());
  (function () {
    'use strict';
    (function () {
      'use strict';
      angular.module('material.components.fabShared', ['material.core']).controller('FabController', FabController);
      function FabController($scope, $element, $animate, $mdUtil, $mdConstant, $timeout) {
        var vm = this;
        // NOTE: We use async eval(s) below to avoid conflicts with any existing digest loops
        vm.open = function () {
          $scope.$evalAsync('vm.isOpen = true');
        };
        vm.close = function () {
          // Async eval to avoid conflicts with existing digest loops
          $scope.$evalAsync('vm.isOpen = false');
          // Focus the trigger when the element closes so users can still tab to the next item
          $element.find('md-fab-trigger')[0].focus();
        };
        // Toggle the open/close state when the trigger is clicked
        vm.toggle = function () {
          $scope.$evalAsync('vm.isOpen = !vm.isOpen');
        };
        setupDefaults();
        setupListeners();
        setupWatchers();
        var initialAnimationAttempts = 0;
        fireInitialAnimations();
        function setupDefaults() {
          // Set the default direction to 'down' if none is specified
          vm.direction = vm.direction || 'down';
          // Set the default to be closed
          vm.isOpen = vm.isOpen || false;
          // Start the keyboard interaction at the first action
          resetActionIndex();
          // Add an animations waiting class so we know not to run
          $element.addClass('md-animations-waiting');
        }
        function setupListeners() {
          var eventTypes = [
              'click',
              'focusin',
              'focusout'
            ];
          // Add our listeners
          angular.forEach(eventTypes, function (eventType) {
            $element.on(eventType, parseEvents);
          });
          // Remove our listeners when destroyed
          $scope.$on('$destroy', function () {
            angular.forEach(eventTypes, function (eventType) {
              $element.off(eventType, parseEvents);
            });
            // remove any attached keyboard handlers in case element is removed while
            // speed dial is open
            disableKeyboard();
          });
        }
        var closeTimeout;
        function parseEvents(event) {
          // If the event is a click, just handle it
          if (event.type == 'click') {
            handleItemClick(event);
          }
          // If we focusout, set a timeout to close the element
          if (event.type == 'focusout' && !closeTimeout) {
            closeTimeout = $timeout(function () {
              vm.close();
            }, 100, false);
          }
          // If we see a focusin and there is a timeout about to run, cancel it so we stay open
          if (event.type == 'focusin' && closeTimeout) {
            $timeout.cancel(closeTimeout);
            closeTimeout = null;
          }
        }
        function resetActionIndex() {
          vm.currentActionIndex = -1;
        }
        function setupWatchers() {
          // Watch for changes to the direction and update classes/attributes
          $scope.$watch('vm.direction', function (newDir, oldDir) {
            // Add the appropriate classes so we can target the direction in the CSS
            $animate.removeClass($element, 'md-' + oldDir);
            $animate.addClass($element, 'md-' + newDir);
            // Reset the action index since it may have changed
            resetActionIndex();
          });
          var trigger, actions;
          // Watch for changes to md-open
          $scope.$watch('vm.isOpen', function (isOpen) {
            // Reset the action index since it may have changed
            resetActionIndex();
            // We can't get the trigger/actions outside of the watch because the component hasn't been
            // linked yet, so we wait until the first watch fires to cache them.
            if (!trigger || !actions) {
              trigger = getTriggerElement();
              actions = getActionsElement();
            }
            if (isOpen) {
              enableKeyboard();
            } else {
              disableKeyboard();
            }
            var toAdd = isOpen ? 'md-is-open' : '';
            var toRemove = isOpen ? '' : 'md-is-open';
            // Set the proper ARIA attributes
            trigger.attr('aria-haspopup', true);
            trigger.attr('aria-expanded', isOpen);
            actions.attr('aria-hidden', !isOpen);
            // Animate the CSS classes
            $animate.setClass($element, toAdd, toRemove);
          });
        }
        function fireInitialAnimations() {
          // If the element is actually visible on the screen
          if ($element[0].scrollHeight > 0) {
            // Fire our animation
            $animate.addClass($element, 'md-animations-ready').then(function () {
              // Remove the waiting class
              $element.removeClass('md-animations-waiting');
            });
          }  // Otherwise, try for up to 1 second before giving up
          else if (initialAnimationAttempts < 10) {
            $timeout(fireInitialAnimations, 100);
            // Increment our counter
            initialAnimationAttempts = initialAnimationAttempts + 1;
          }
        }
        function enableKeyboard() {
          $element.on('keydown', keyPressed);
          // On the next tick, setup a check for outside clicks; we do this on the next tick to avoid
          // clicks/touches that result in the isOpen attribute changing (e.g. a bound radio button)
          $mdUtil.nextTick(function () {
            angular.element(document).on('click touchend', checkForOutsideClick);
          });  // TODO: On desktop, we should be able to reset the indexes so you cannot tab through, but
               // this breaks accessibility, especially on mobile, since you have no arrow keys to press
               //resetActionTabIndexes();
        }
        function disableKeyboard() {
          $element.off('keydown', keyPressed);
          angular.element(document).off('click touchend', checkForOutsideClick);
        }
        function checkForOutsideClick(event) {
          if (event.target) {
            var closestTrigger = $mdUtil.getClosest(event.target, 'md-fab-trigger');
            var closestActions = $mdUtil.getClosest(event.target, 'md-fab-actions');
            if (!closestTrigger && !closestActions) {
              vm.close();
            }
          }
        }
        function keyPressed(event) {
          switch (event.which) {
          case $mdConstant.KEY_CODE.ESCAPE:
            vm.close();
            event.preventDefault();
            return false;
          case $mdConstant.KEY_CODE.LEFT_ARROW:
            doKeyLeft(event);
            return false;
          case $mdConstant.KEY_CODE.UP_ARROW:
            doKeyUp(event);
            return false;
          case $mdConstant.KEY_CODE.RIGHT_ARROW:
            doKeyRight(event);
            return false;
          case $mdConstant.KEY_CODE.DOWN_ARROW:
            doKeyDown(event);
            return false;
          }
        }
        function doActionPrev(event) {
          focusAction(event, -1);
        }
        function doActionNext(event) {
          focusAction(event, 1);
        }
        function focusAction(event, direction) {
          var actions = resetActionTabIndexes();
          // Increment/decrement the counter with restrictions
          vm.currentActionIndex = vm.currentActionIndex + direction;
          vm.currentActionIndex = Math.min(actions.length - 1, vm.currentActionIndex);
          vm.currentActionIndex = Math.max(0, vm.currentActionIndex);
          // Focus the element
          var focusElement = angular.element(actions[vm.currentActionIndex]).children()[0];
          angular.element(focusElement).attr('tabindex', 0);
          focusElement.focus();
          // Make sure the event doesn't bubble and cause something else
          event.preventDefault();
          event.stopImmediatePropagation();
        }
        function resetActionTabIndexes() {
          // Grab all of the actions
          var actions = getActionsElement()[0].querySelectorAll('.md-fab-action-item');
          // Disable all other actions for tabbing
          angular.forEach(actions, function (action) {
            angular.element(angular.element(action).children()[0]).attr('tabindex', -1);
          });
          return actions;
        }
        function doKeyLeft(event) {
          if (vm.direction === 'left') {
            doActionNext(event);
          } else {
            doActionPrev(event);
          }
        }
        function doKeyUp(event) {
          if (vm.direction === 'down') {
            doActionPrev(event);
          } else {
            doActionNext(event);
          }
        }
        function doKeyRight(event) {
          if (vm.direction === 'left') {
            doActionPrev(event);
          } else {
            doActionNext(event);
          }
        }
        function doKeyDown(event) {
          if (vm.direction === 'up') {
            doActionPrev(event);
          } else {
            doActionNext(event);
          }
        }
        function isTrigger(element) {
          return $mdUtil.getClosest(element, 'md-fab-trigger');
        }
        function isAction(element) {
          return $mdUtil.getClosest(element, 'md-fab-actions');
        }
        function handleItemClick(event) {
          if (isTrigger(event.target)) {
            vm.toggle();
          }
          if (isAction(event.target)) {
            vm.close();
          }
        }
        function getTriggerElement() {
          return $element.find('md-fab-trigger');
        }
        function getActionsElement() {
          return $element.find('md-fab-actions');
        }
      }
      FabController.$inject = [
        '$scope',
        '$element',
        '$animate',
        '$mdUtil',
        '$mdConstant',
        '$timeout'
      ];
    }());
  }());
  (function () {
    'use strict';
    (function () {
      'use strict';
      /**
   * The duration of the CSS animation in milliseconds.
   *
   * @type {number}
   */
      var cssAnimationDuration = 300;
      /**
   * @ngdoc module
   * @name material.components.fabSpeedDial
   */
      angular.module('material.components.fabSpeedDial', [
        'material.core',
        'material.components.fabShared',
        'material.components.fabTrigger',
        'material.components.fabActions'
      ]).directive('mdFabSpeedDial', MdFabSpeedDialDirective).animation('.md-fling', MdFabSpeedDialFlingAnimation).animation('.md-scale', MdFabSpeedDialScaleAnimation).service('mdFabSpeedDialFlingAnimation', MdFabSpeedDialFlingAnimation).service('mdFabSpeedDialScaleAnimation', MdFabSpeedDialScaleAnimation);
      /**
   * @ngdoc directive
   * @name mdFabSpeedDial
   * @module material.components.fabSpeedDial
   *
   * @restrict E
   *
   * @description
   * The `<md-fab-speed-dial>` directive is used to present a series of popup elements (usually
   * `<md-button>`s) for quick access to common actions.
   *
   * There are currently two animations available by applying one of the following classes to
   * the component:
   *
   *  - `md-fling` - The speed dial items appear from underneath the trigger and move into their
   *    appropriate positions.
   *  - `md-scale` - The speed dial items appear in their proper places by scaling from 0% to 100%.
   *
   * You may also easily position the trigger by applying one one of the following classes to the
   * `<md-fab-speed-dial>` element:
   *  - `md-fab-top-left`
   *  - `md-fab-top-right`
   *  - `md-fab-bottom-left`
   *  - `md-fab-bottom-right`
   *
   * These CSS classes use `position: absolute`, so you need to ensure that the container element
   * also uses `position: absolute` or `position: relative` in order for them to work.
   *
   * Additionally, you may use the standard `ng-mouseenter` and `ng-mouseleave` directives to
   * open or close the speed dial. However, if you wish to allow users to hover over the empty
   * space where the actions will appear, you must also add the `md-hover-full` class to the speed
   * dial element. Without this, the hover effect will only occur on top of the trigger.
   *
   * @usage
   * <hljs lang="html">
   * <md-fab-speed-dial md-direction="up" class="md-fling">
   *   <md-fab-trigger>
   *     <md-button aria-label="Add..."><md-icon icon="/img/icons/plus.svg"></md-icon></md-button>
   *   </md-fab-trigger>
   *
   *   <md-fab-actions>
   *     <md-button aria-label="Add User">
   *       <md-icon icon="/img/icons/user.svg"></md-icon>
   *     </md-button>
   *
   *     <md-button aria-label="Add Group">
   *       <md-icon icon="/img/icons/group.svg"></md-icon>
   *     </md-button>
   *   </md-fab-actions>
   * </md-fab-speed-dial>
   * </hljs>
   *
   * @param {string} md-direction From which direction you would like the speed dial to appear
   * relative to the trigger element.
   * @param {expression=} md-open Programmatically control whether or not the speed-dial is visible.
   */
      function MdFabSpeedDialDirective() {
        return {
          restrict: 'E',
          scope: {
            direction: '@?mdDirection',
            isOpen: '=?mdOpen'
          },
          bindToController: true,
          controller: 'FabController',
          controllerAs: 'vm',
          link: FabSpeedDialLink
        };
        function FabSpeedDialLink(scope, element) {
          // Prepend an element to hold our CSS variables so we can use them in the animations below
          element.prepend('<div class="md-css-variables"></div>');
        }
      }
      function MdFabSpeedDialFlingAnimation($timeout) {
        function delayDone(done) {
          $timeout(done, cssAnimationDuration, false);
        }
        function runAnimation(element) {
          // Don't run if we are still waiting and we are not ready
          if (element.hasClass('md-animations-waiting') && !element.hasClass('md-animations-ready')) {
            return;
          }
          var el = element[0];
          var ctrl = element.controller('mdFabSpeedDial');
          var items = el.querySelectorAll('.md-fab-action-item');
          // Grab our trigger element
          var triggerElement = el.querySelector('md-fab-trigger');
          // Grab our element which stores CSS variables
          var variablesElement = el.querySelector('.md-css-variables');
          // Setup JS variables based on our CSS variables
          var startZIndex = parseInt(window.getComputedStyle(variablesElement).zIndex);
          // Always reset the items to their natural position/state
          angular.forEach(items, function (item, index) {
            var styles = item.style;
            styles.transform = styles.webkitTransform = '';
            styles.transitionDelay = '';
            styles.opacity = 1;
            // Make the items closest to the trigger have the highest z-index
            styles.zIndex = items.length - index + startZIndex;
          });
          // Set the trigger to be above all of the actions so they disappear behind it.
          triggerElement.style.zIndex = startZIndex + items.length + 1;
          // If the control is closed, hide the items behind the trigger
          if (!ctrl.isOpen) {
            angular.forEach(items, function (item, index) {
              var newPosition, axis;
              var styles = item.style;
              // Make sure to account for differences in the dimensions of the trigger verses the items
              // so that we can properly center everything; this helps hide the item's shadows behind
              // the trigger.
              var triggerItemHeightOffset = (triggerElement.clientHeight - item.clientHeight) / 2;
              var triggerItemWidthOffset = (triggerElement.clientWidth - item.clientWidth) / 2;
              switch (ctrl.direction) {
              case 'up':
                newPosition = item.scrollHeight * (index + 1) + triggerItemHeightOffset;
                axis = 'Y';
                break;
              case 'down':
                newPosition = -(item.scrollHeight * (index + 1) + triggerItemHeightOffset);
                axis = 'Y';
                break;
              case 'left':
                newPosition = item.scrollWidth * (index + 1) + triggerItemWidthOffset;
                axis = 'X';
                break;
              case 'right':
                newPosition = -(item.scrollWidth * (index + 1) + triggerItemWidthOffset);
                axis = 'X';
                break;
              }
              var newTranslate = 'translate' + axis + '(' + newPosition + 'px)';
              styles.transform = styles.webkitTransform = newTranslate;
            });
          }
        }
        return {
          addClass: function (element, className, done) {
            if (element.hasClass('md-fling')) {
              runAnimation(element);
              delayDone(done);
            } else {
              done();
            }
          },
          removeClass: function (element, className, done) {
            runAnimation(element);
            delayDone(done);
          }
        };
      }
      MdFabSpeedDialFlingAnimation.$inject = ['$timeout'];
      function MdFabSpeedDialScaleAnimation($timeout) {
        function delayDone(done) {
          $timeout(done, cssAnimationDuration, false);
        }
        var delay = 65;
        function runAnimation(element) {
          var el = element[0];
          var ctrl = element.controller('mdFabSpeedDial');
          var items = el.querySelectorAll('.md-fab-action-item');
          // Grab our element which stores CSS variables
          var variablesElement = el.querySelector('.md-css-variables');
          // Setup JS variables based on our CSS variables
          var startZIndex = parseInt(window.getComputedStyle(variablesElement).zIndex);
          // Always reset the items to their natural position/state
          angular.forEach(items, function (item, index) {
            var styles = item.style, offsetDelay = index * delay;
            styles.opacity = ctrl.isOpen ? 1 : 0;
            styles.transform = styles.webkitTransform = ctrl.isOpen ? 'scale(1)' : 'scale(0.1)';
            styles.transitionDelay = (ctrl.isOpen ? offsetDelay : items.length - offsetDelay) + 'ms';
            // Make the items closest to the trigger have the highest z-index
            styles.zIndex = items.length - index + startZIndex;
          });
        }
        return {
          addClass: function (element, className, done) {
            runAnimation(element);
            delayDone(done);
          },
          removeClass: function (element, className, done) {
            runAnimation(element);
            delayDone(done);
          }
        };
      }
      MdFabSpeedDialScaleAnimation.$inject = ['$timeout'];
    }());
  }());
  (function () {
    'use strict';
    (function () {
      'use strict';
      /**
   * @ngdoc module
   * @name material.components.fabToolbar
   */
      angular.module('material.components.fabToolbar', [
        'material.core',
        'material.components.fabShared',
        'material.components.fabTrigger',
        'material.components.fabActions'
      ]).directive('mdFabToolbar', MdFabToolbarDirective).animation('.md-fab-toolbar', MdFabToolbarAnimation).service('mdFabToolbarAnimation', MdFabToolbarAnimation);
      /**
   * @ngdoc directive
   * @name mdFabToolbar
   * @module material.components.fabToolbar
   *
   * @restrict E
   *
   * @description
   *
   * The `<md-fab-toolbar>` directive is used present a toolbar of elements (usually `<md-button>`s)
   * for quick access to common actions when a floating action button is activated (via click or
   * keyboard navigation).
   *
   * You may also easily position the trigger by applying one one of the following classes to the
   * `<md-fab-toolbar>` element:
   *  - `md-fab-top-left`
   *  - `md-fab-top-right`
   *  - `md-fab-bottom-left`
   *  - `md-fab-bottom-right`
   *
   * These CSS classes use `position: absolute`, so you need to ensure that the container element
   * also uses `position: absolute` or `position: relative` in order for them to work.
   *
   * @usage
   *
   * <hljs lang="html">
   * <md-fab-toolbar md-direction='left'>
   *   <md-fab-trigger>
   *     <md-button aria-label="Add..."><md-icon icon="/img/icons/plus.svg"></md-icon></md-button>
   *   </md-fab-trigger>
   *
   *   <md-fab-actions>
   *     <md-button aria-label="Add User">
   *       <md-icon icon="/img/icons/user.svg"></md-icon>
   *     </md-button>
   *
   *     <md-button aria-label="Add Group">
   *       <md-icon icon="/img/icons/group.svg"></md-icon>
   *     </md-button>
   *   </md-fab-actions>
   * </md-fab-toolbar>
   * </hljs>
   *
   * @param {string} md-direction From which direction you would like the toolbar items to appear
   * relative to the trigger element. Supports `left` and `right` directions.
   * @param {expression=} md-open Programmatically control whether or not the toolbar is visible.
   */
      function MdFabToolbarDirective() {
        return {
          restrict: 'E',
          transclude: true,
          template: '<div class="md-fab-toolbar-wrapper">' + '  <div class="md-fab-toolbar-content" ng-transclude></div>' + '</div>',
          scope: {
            direction: '@?mdDirection',
            isOpen: '=?mdOpen'
          },
          bindToController: true,
          controller: 'FabController',
          controllerAs: 'vm',
          link: link
        };
        function link(scope, element, attributes) {
          // Add the base class for animations
          element.addClass('md-fab-toolbar');
          // Prepend the background element to the trigger's button
          element.find('md-fab-trigger').find('button').prepend('<div class="md-fab-toolbar-background"></div>');
        }
      }
      function MdFabToolbarAnimation() {
        function runAnimation(element, className, done) {
          // If no className was specified, don't do anything
          if (!className) {
            return;
          }
          var el = element[0];
          var ctrl = element.controller('mdFabToolbar');
          // Grab the relevant child elements
          var backgroundElement = el.querySelector('.md-fab-toolbar-background');
          var triggerElement = el.querySelector('md-fab-trigger button');
          var toolbarElement = el.querySelector('md-toolbar');
          var iconElement = el.querySelector('md-fab-trigger button md-icon');
          var actions = element.find('md-fab-actions').children();
          // If we have both elements, use them to position the new background
          if (triggerElement && backgroundElement) {
            // Get our variables
            var color = window.getComputedStyle(triggerElement).getPropertyValue('background-color');
            var width = el.offsetWidth;
            var height = el.offsetHeight;
            // Make it twice as big as it should be since we scale from the center
            var scale = 2 * (width / triggerElement.offsetWidth);
            // Set some basic styles no matter what animation we're doing
            backgroundElement.style.backgroundColor = color;
            backgroundElement.style.borderRadius = width + 'px';
            // If we're open
            if (ctrl.isOpen) {
              // Turn on toolbar pointer events when closed
              toolbarElement.style.pointerEvents = 'initial';
              backgroundElement.style.width = triggerElement.offsetWidth + 'px';
              backgroundElement.style.height = triggerElement.offsetHeight + 'px';
              backgroundElement.style.transform = 'scale(' + scale + ')';
              // Set the next close animation to have the proper delays
              backgroundElement.style.transitionDelay = '0ms';
              iconElement && (iconElement.style.transitionDelay = '.3s');
              // Apply a transition delay to actions
              angular.forEach(actions, function (action, index) {
                action.style.transitionDelay = (actions.length - index) * 25 + 'ms';
              });
            } else {
              // Turn off toolbar pointer events when closed
              toolbarElement.style.pointerEvents = 'none';
              // Scale it back down to the trigger's size
              backgroundElement.style.transform = 'scale(1)';
              // Reset the position
              backgroundElement.style.top = '0';
              if (element.hasClass('md-right')) {
                backgroundElement.style.left = '0';
                backgroundElement.style.right = null;
              }
              if (element.hasClass('md-left')) {
                backgroundElement.style.right = '0';
                backgroundElement.style.left = null;
              }
              // Set the next open animation to have the proper delays
              backgroundElement.style.transitionDelay = '200ms';
              iconElement && (iconElement.style.transitionDelay = '0ms');
              // Apply a transition delay to actions
              angular.forEach(actions, function (action, index) {
                action.style.transitionDelay = 200 + index * 25 + 'ms';
              });
            }
          }
        }
        return {
          addClass: function (element, className, done) {
            runAnimation(element, className, done);
            done();
          },
          removeClass: function (element, className, done) {
            runAnimation(element, className, done);
            done();
          }
        };
      }
    }());
  }());
  (function () {
    'use strict';
    (function () {
      'use strict';
      /**
   * @ngdoc module
   * @name material.components.fabTrigger
   */
      angular.module('material.components.fabTrigger', ['material.core']).directive('mdFabTrigger', MdFabTriggerDirective);
      /**
   * @ngdoc directive
   * @name mdFabTrigger
   * @module material.components.fabSpeedDial
   *
   * @restrict E
   *
   * @description
   * The `<md-fab-trigger>` directive is used inside of a `<md-fab-speed-dial>` or
   * `<md-fab-toolbar>` directive to mark an element (or elements) as the trigger and setup the
   * proper event listeners.
   *
   * @usage
   * See the `<md-fab-speed-dial>` or `<md-fab-toolbar>` directives for example usage.
   */
      function MdFabTriggerDirective() {
        // TODO: Remove this completely?
        return {
          restrict: 'E',
          require: [
            '^?mdFabSpeedDial',
            '^?mdFabToolbar'
          ]
        };
      }
    }());
  }());
  (function () {
    'use strict';
    /**
 * @ngdoc module
 * @name material.components.gridList
 */
    angular.module('material.components.gridList', ['material.core']).directive('mdGridList', GridListDirective).directive('mdGridTile', GridTileDirective).directive('mdGridTileFooter', GridTileCaptionDirective).directive('mdGridTileHeader', GridTileCaptionDirective).factory('$mdGridLayout', GridLayoutFactory);
    /**
 * @ngdoc directive
 * @name mdGridList
 * @module material.components.gridList
 * @restrict E
 * @description
 * Grid lists are an alternative to standard list views. Grid lists are distinct
 * from grids used for layouts and other visual presentations.
 *
 * A grid list is best suited to presenting a homogenous data type, typically
 * images, and is optimized for visual comprehension and differentiating between
 * like data types.
 *
 * A grid list is a continuous element consisting of tessellated, regular
 * subdivisions called cells that contain tiles (`md-grid-tile`).
 *
 * <img src="//material-design.storage.googleapis.com/publish/v_2/material_ext_publish/0Bx4BSt6jniD7OVlEaXZ5YmU1Xzg/components_grids_usage2.png"
 *    style="width: 300px; height: auto; margin-right: 16px;" alt="Concept of grid explained visually">
 * <img src="//material-design.storage.googleapis.com/publish/v_2/material_ext_publish/0Bx4BSt6jniD7VGhsOE5idWlJWXM/components_grids_usage3.png"
 *    style="width: 300px; height: auto;" alt="Grid concepts legend">
 *
 * Cells are arrayed vertically and horizontally within the grid.
 *
 * Tiles hold content and can span one or more cells vertically or horizontally.
 *
 * ### Responsive Attributes
 *
 * The `md-grid-list` directive supports "responsive" attributes, which allow
 * different `md-cols`, `md-gutter` and `md-row-height` values depending on the
 * currently matching media query (as defined in `$mdConstant.MEDIA`).
 *
 * In order to set a responsive attribute, first define the fallback value with
 * the standard attribute name, then add additional attributes with the
 * following convention: `{base-attribute-name}-{media-query-name}="{value}"`
 * (ie. `md-cols-lg="8"`)
 *
 * @param {number} md-cols Number of columns in the grid.
 * @param {string} md-row-height One of
 * <ul>
 *   <li>CSS length - Fixed height rows (eg. `8px` or `1rem`)</li>
 *   <li>`{width}:{height}` - Ratio of width to height (eg.
 *   `md-row-height="16:9"`)</li>
 *   <li>`"fit"` - Height will be determined by subdividing the available
 *   height by the number of rows</li>
 * </ul>
 * @param {string=} md-gutter The amount of space between tiles in CSS units
 *     (default 1px)
 * @param {expression=} md-on-layout Expression to evaluate after layout. Event
 *     object is available as `$event`, and contains performance information.
 *
 * @usage
 * Basic:
 * <hljs lang="html">
 * <md-grid-list md-cols="5" md-gutter="1em" md-row-height="4:3">
 *   <md-grid-tile></md-grid-tile>
 * </md-grid-list>
 * </hljs>
 *
 * Fixed-height rows:
 * <hljs lang="html">
 * <md-grid-list md-cols="4" md-row-height="200px" ...>
 *   <md-grid-tile></md-grid-tile>
 * </md-grid-list>
 * </hljs>
 *
 * Fit rows:
 * <hljs lang="html">
 * <md-grid-list md-cols="4" md-row-height="fit" style="height: 400px;" ...>
 *   <md-grid-tile></md-grid-tile>
 * </md-grid-list>
 * </hljs>
 *
 * Using responsive attributes:
 * <hljs lang="html">
 * <md-grid-list
 *     md-cols-sm="2"
 *     md-cols-md="4"
 *     md-cols-lg="8"
 *     md-cols-gt-lg="12"
 *     ...>
 *   <md-grid-tile></md-grid-tile>
 * </md-grid-list>
 * </hljs>
 */
    function GridListDirective($interpolate, $mdConstant, $mdGridLayout, $mdMedia) {
      return {
        restrict: 'E',
        controller: GridListController,
        scope: { mdOnLayout: '&' },
        link: postLink
      };
      function postLink(scope, element, attrs, ctrl) {
        // Apply semantics
        element.attr('role', 'list');
        // Provide the controller with a way to trigger layouts.
        ctrl.layoutDelegate = layoutDelegate;
        var invalidateLayout = angular.bind(ctrl, ctrl.invalidateLayout), unwatchAttrs = watchMedia();
        scope.$on('$destroy', unwatchMedia);
        /**
     * Watches for changes in media, invalidating layout as necessary.
     */
        function watchMedia() {
          for (var mediaName in $mdConstant.MEDIA) {
            $mdMedia(mediaName);
            // initialize
            $mdMedia.getQuery($mdConstant.MEDIA[mediaName]).addListener(invalidateLayout);
          }
          return $mdMedia.watchResponsiveAttributes([
            'md-cols',
            'md-row-height',
            'md-gutter'
          ], attrs, layoutIfMediaMatch);
        }
        function unwatchMedia() {
          ctrl.layoutDelegate = angular.noop;
          unwatchAttrs();
          for (var mediaName in $mdConstant.MEDIA) {
            $mdMedia.getQuery($mdConstant.MEDIA[mediaName]).removeListener(invalidateLayout);
          }
        }
        /**
     * Performs grid layout if the provided mediaName matches the currently
     * active media type.
     */
        function layoutIfMediaMatch(mediaName) {
          if (mediaName == null) {
            // TODO(shyndman): It would be nice to only layout if we have
            // instances of attributes using this media type
            ctrl.invalidateLayout();
          } else if ($mdMedia(mediaName)) {
            ctrl.invalidateLayout();
          }
        }
        var lastLayoutProps;
        /**
     * Invokes the layout engine, and uses its results to lay out our
     * tile elements.
     *
     * @param {boolean} tilesInvalidated Whether tiles have been
     *    added/removed/moved since the last layout. This is to avoid situations
     *    where tiles are replaced with properties identical to their removed
     *    counterparts.
     */
        function layoutDelegate(tilesInvalidated) {
          var tiles = getTileElements();
          var props = {
              tileSpans: getTileSpans(tiles),
              colCount: getColumnCount(),
              rowMode: getRowMode(),
              rowHeight: getRowHeight(),
              gutter: getGutter()
            };
          if (!tilesInvalidated && angular.equals(props, lastLayoutProps)) {
            return;
          }
          var performance = $mdGridLayout(props.colCount, props.tileSpans, tiles).map(function (tilePositions, rowCount) {
              return {
                grid: {
                  element: element,
                  style: getGridStyle(props.colCount, rowCount, props.gutter, props.rowMode, props.rowHeight)
                },
                tiles: tilePositions.map(function (ps, i) {
                  return {
                    element: angular.element(tiles[i]),
                    style: getTileStyle(ps.position, ps.spans, props.colCount, rowCount, props.gutter, props.rowMode, props.rowHeight)
                  };
                })
              };
            }).reflow().performance();
          // Report layout
          scope.mdOnLayout({ $event: { performance: performance } });
          lastLayoutProps = props;
        }
        // Use $interpolate to do some simple string interpolation as a convenience.
        var startSymbol = $interpolate.startSymbol();
        var endSymbol = $interpolate.endSymbol();
        // Returns an expression wrapped in the interpolator's start and end symbols.
        function expr(exprStr) {
          return startSymbol + exprStr + endSymbol;
        }
        // The amount of space a single 1x1 tile would take up (either width or height), used as
        // a basis for other calculations. This consists of taking the base size percent (as would be
        // if evenly dividing the size between cells), and then subtracting the size of one gutter.
        // However, since there are no gutters on the edges, each tile only uses a fration
        // (gutterShare = numGutters / numCells) of the gutter size. (Imagine having one gutter per
        // tile, and then breaking up the extra gutter on the edge evenly among the cells).
        var UNIT = $interpolate(expr('share') + '% - (' + expr('gutter') + ' * ' + expr('gutterShare') + ')');
        // The horizontal or vertical position of a tile, e.g., the 'top' or 'left' property value.
        // The position comes the size of a 1x1 tile plus gutter for each previous tile in the
        // row/column (offset).
        var POSITION = $interpolate('calc((' + expr('unit') + ' + ' + expr('gutter') + ') * ' + expr('offset') + ')');
        // The actual size of a tile, e.g., width or height, taking rowSpan or colSpan into account.
        // This is computed by multiplying the base unit by the rowSpan/colSpan, and then adding back
        // in the space that the gutter would normally have used (which was already accounted for in
        // the base unit calculation).
        var DIMENSION = $interpolate('calc((' + expr('unit') + ') * ' + expr('span') + ' + (' + expr('span') + ' - 1) * ' + expr('gutter') + ')');
        /**
     * Gets the styles applied to a tile element described by the given parameters.
     * @param {{row: number, col: number}} position The row and column indices of the tile.
     * @param {{row: number, col: number}} spans The rowSpan and colSpan of the tile.
     * @param {number} colCount The number of columns.
     * @param {number} rowCount The number of rows.
     * @param {string} gutter The amount of space between tiles. This will be something like
     *     '5px' or '2em'.
     * @param {string} rowMode The row height mode. Can be one of:
     *     'fixed': all rows have a fixed size, given by rowHeight,
     *     'ratio': row height defined as a ratio to width, or
     *     'fit': fit to the grid-list element height, divinding evenly among rows.
     * @param {string|number} rowHeight The height of a row. This is only used for 'fixed' mode and
     *     for 'ratio' mode. For 'ratio' mode, this is the *ratio* of width-to-height (e.g., 0.75).
     * @returns {Object} Map of CSS properties to be applied to the style element. Will define
     *     values for top, left, width, height, marginTop, and paddingTop.
     */
        function getTileStyle(position, spans, colCount, rowCount, gutter, rowMode, rowHeight) {
          // TODO(shyndman): There are style caching opportunities here.
          // Percent of the available horizontal space that one column takes up.
          var hShare = 1 / colCount * 100;
          // Fraction of the gutter size that each column takes up.
          var hGutterShare = (colCount - 1) / colCount;
          // Base horizontal size of a column.
          var hUnit = UNIT({
              share: hShare,
              gutterShare: hGutterShare,
              gutter: gutter
            });
          // The width and horizontal position of each tile is always calculated the same way, but the
          // height and vertical position depends on the rowMode.
          var style = {
              left: POSITION({
                unit: hUnit,
                offset: position.col,
                gutter: gutter
              }),
              width: DIMENSION({
                unit: hUnit,
                span: spans.col,
                gutter: gutter
              }),
              paddingTop: '',
              marginTop: '',
              top: '',
              height: ''
            };
          switch (rowMode) {
          case 'fixed':
            // In fixed mode, simply use the given rowHeight.
            style.top = POSITION({
              unit: rowHeight,
              offset: position.row,
              gutter: gutter
            });
            style.height = DIMENSION({
              unit: rowHeight,
              span: spans.row,
              gutter: gutter
            });
            break;
          case 'ratio':
            // Percent of the available vertical space that one row takes up. Here, rowHeight holds
            // the ratio value. For example, if the width:height ratio is 4:3, rowHeight = 1.333.
            var vShare = hShare / rowHeight;
            // Base veritcal size of a row.
            var vUnit = UNIT({
                share: vShare,
                gutterShare: hGutterShare,
                gutter: gutter
              });
            // padidngTop and marginTop are used to maintain the given aspect ratio, as
            // a percentage-based value for these properties is applied to the *width* of the
            // containing block. See http://www.w3.org/TR/CSS2/box.html#margin-properties
            style.paddingTop = DIMENSION({
              unit: vUnit,
              span: spans.row,
              gutter: gutter
            });
            style.marginTop = POSITION({
              unit: vUnit,
              offset: position.row,
              gutter: gutter
            });
            break;
          case 'fit':
            // Fraction of the gutter size that each column takes up.
            var vGutterShare = (rowCount - 1) / rowCount;
            // Percent of the available vertical space that one row takes up.
            var vShare = 1 / rowCount * 100;
            // Base vertical size of a row.
            var vUnit = UNIT({
                share: vShare,
                gutterShare: vGutterShare,
                gutter: gutter
              });
            style.top = POSITION({
              unit: vUnit,
              offset: position.row,
              gutter: gutter
            });
            style.height = DIMENSION({
              unit: vUnit,
              span: spans.row,
              gutter: gutter
            });
            break;
          }
          return style;
        }
        function getGridStyle(colCount, rowCount, gutter, rowMode, rowHeight) {
          var style = {};
          switch (rowMode) {
          case 'fixed':
            style.height = DIMENSION({
              unit: rowHeight,
              span: rowCount,
              gutter: gutter
            });
            style.paddingBottom = '';
            break;
          case 'ratio':
            // rowHeight is width / height
            var hGutterShare = colCount === 1 ? 0 : (colCount - 1) / colCount, hShare = 1 / colCount * 100, vShare = hShare * (1 / rowHeight), vUnit = UNIT({
                share: vShare,
                gutterShare: hGutterShare,
                gutter: gutter
              });
            style.height = '';
            style.paddingBottom = DIMENSION({
              unit: vUnit,
              span: rowCount,
              gutter: gutter
            });
            break;
          case 'fit':
            // noop, as the height is user set
            break;
          }
          return style;
        }
        function getTileElements() {
          return [].filter.call(element.children(), function (ele) {
            return ele.tagName == 'MD-GRID-TILE' && !ele.$$mdDestroyed;
          });
        }
        /**
     * Gets an array of objects containing the rowspan and colspan for each tile.
     * @returns {Array<{row: number, col: number}>}
     */
        function getTileSpans(tileElements) {
          return [].map.call(tileElements, function (ele) {
            var ctrl = angular.element(ele).controller('mdGridTile');
            return {
              row: parseInt($mdMedia.getResponsiveAttribute(ctrl.$attrs, 'md-rowspan'), 10) || 1,
              col: parseInt($mdMedia.getResponsiveAttribute(ctrl.$attrs, 'md-colspan'), 10) || 1
            };
          });
        }
        function getColumnCount() {
          var colCount = parseInt($mdMedia.getResponsiveAttribute(attrs, 'md-cols'), 10);
          if (isNaN(colCount)) {
            throw 'md-grid-list: md-cols attribute was not found, or contained a non-numeric value';
          }
          return colCount;
        }
        function getGutter() {
          return applyDefaultUnit($mdMedia.getResponsiveAttribute(attrs, 'md-gutter') || 1);
        }
        function getRowHeight() {
          var rowHeight = $mdMedia.getResponsiveAttribute(attrs, 'md-row-height');
          switch (getRowMode()) {
          case 'fixed':
            return applyDefaultUnit(rowHeight);
          case 'ratio':
            var whRatio = rowHeight.split(':');
            return parseFloat(whRatio[0]) / parseFloat(whRatio[1]);
          case 'fit':
            return 0;  // N/A
          }
        }
        function getRowMode() {
          var rowHeight = $mdMedia.getResponsiveAttribute(attrs, 'md-row-height');
          if (rowHeight == 'fit') {
            return 'fit';
          } else if (rowHeight.indexOf(':') !== -1) {
            return 'ratio';
          } else {
            return 'fixed';
          }
        }
        function applyDefaultUnit(val) {
          return /\D$/.test(val) ? val : val + 'px';
        }
      }
    }
    GridListDirective.$inject = [
      '$interpolate',
      '$mdConstant',
      '$mdGridLayout',
      '$mdMedia'
    ];
    /* @ngInject */
    function GridListController($mdUtil) {
      this.layoutInvalidated = false;
      this.tilesInvalidated = false;
      this.$timeout_ = $mdUtil.nextTick;
      this.layoutDelegate = angular.noop;
    }
    GridListController.$inject = ['$mdUtil'];
    GridListController.prototype = {
      invalidateTiles: function () {
        this.tilesInvalidated = true;
        this.invalidateLayout();
      },
      invalidateLayout: function () {
        if (this.layoutInvalidated) {
          return;
        }
        this.layoutInvalidated = true;
        this.$timeout_(angular.bind(this, this.layout));
      },
      layout: function () {
        try {
          this.layoutDelegate(this.tilesInvalidated);
        } finally {
          this.layoutInvalidated = false;
          this.tilesInvalidated = false;
        }
      }
    };
    /* @ngInject */
    function GridLayoutFactory($mdUtil) {
      var defaultAnimator = GridTileAnimator;
      /**
   * Set the reflow animator callback
   */
      GridLayout.animateWith = function (customAnimator) {
        defaultAnimator = !angular.isFunction(customAnimator) ? GridTileAnimator : customAnimator;
      };
      return GridLayout;
      /**
   * Publish layout function
   */
      function GridLayout(colCount, tileSpans) {
        var self, layoutInfo, gridStyles, layoutTime, mapTime, reflowTime;
        layoutTime = $mdUtil.time(function () {
          layoutInfo = calculateGridFor(colCount, tileSpans);
        });
        return self = {
          layoutInfo: function () {
            return layoutInfo;
          },
          map: function (updateFn) {
            mapTime = $mdUtil.time(function () {
              var info = self.layoutInfo();
              gridStyles = updateFn(info.positioning, info.rowCount);
            });
            return self;
          },
          reflow: function (animatorFn) {
            reflowTime = $mdUtil.time(function () {
              var animator = animatorFn || defaultAnimator;
              animator(gridStyles.grid, gridStyles.tiles);
            });
            return self;
          },
          performance: function () {
            return {
              tileCount: tileSpans.length,
              layoutTime: layoutTime,
              mapTime: mapTime,
              reflowTime: reflowTime,
              totalTime: layoutTime + mapTime + reflowTime
            };
          }
        };
      }
      /**
   * Default Gridlist animator simple sets the css for each element;
   * NOTE: any transitions effects must be manually set in the CSS.
   * e.g.
   *
   *  md-grid-tile {
   *    transition: all 700ms ease-out 50ms;
   *  }
   *
   */
      function GridTileAnimator(grid, tiles) {
        grid.element.css(grid.style);
        tiles.forEach(function (t) {
          t.element.css(t.style);
        });
      }
      /**
   * Calculates the positions of tiles.
   *
   * The algorithm works as follows:
   *    An Array<Number> with length colCount (spaceTracker) keeps track of
   *    available tiling positions, where elements of value 0 represents an
   *    empty position. Space for a tile is reserved by finding a sequence of
   *    0s with length <= than the tile's colspan. When such a space has been
   *    found, the occupied tile positions are incremented by the tile's
   *    rowspan value, as these positions have become unavailable for that
   *    many rows.
   *
   *    If the end of a row has been reached without finding space for the
   *    tile, spaceTracker's elements are each decremented by 1 to a minimum
   *    of 0. Rows are searched in this fashion until space is found.
   */
      function calculateGridFor(colCount, tileSpans) {
        var curCol = 0, curRow = 0, spaceTracker = newSpaceTracker();
        return {
          positioning: tileSpans.map(function (spans, i) {
            return {
              spans: spans,
              position: reserveSpace(spans, i)
            };
          }),
          rowCount: curRow + Math.max.apply(Math, spaceTracker)
        };
        function reserveSpace(spans, i) {
          if (spans.col > colCount) {
            throw 'md-grid-list: Tile at position ' + i + ' has a colspan ' + '(' + spans.col + ') that exceeds the column count ' + '(' + colCount + ')';
          }
          var start = 0, end = 0;
          // TODO(shyndman): This loop isn't strictly necessary if you can
          // determine the minimum number of rows before a space opens up. To do
          // this, recognize that you've iterated across an entire row looking for
          // space, and if so fast-forward by the minimum rowSpan count. Repeat
          // until the required space opens up.
          while (end - start < spans.col) {
            if (curCol >= colCount) {
              nextRow();
              continue;
            }
            start = spaceTracker.indexOf(0, curCol);
            if (start === -1 || (end = findEnd(start + 1)) === -1) {
              start = end = 0;
              nextRow();
              continue;
            }
            curCol = end + 1;
          }
          adjustRow(start, spans.col, spans.row);
          curCol = start + spans.col;
          return {
            col: start,
            row: curRow
          };
        }
        function nextRow() {
          curCol = 0;
          curRow++;
          adjustRow(0, colCount, -1);  // Decrement row spans by one
        }
        function adjustRow(from, cols, by) {
          for (var i = from; i < from + cols; i++) {
            spaceTracker[i] = Math.max(spaceTracker[i] + by, 0);
          }
        }
        function findEnd(start) {
          var i;
          for (i = start; i < spaceTracker.length; i++) {
            if (spaceTracker[i] !== 0) {
              return i;
            }
          }
          if (i === spaceTracker.length) {
            return i;
          }
        }
        function newSpaceTracker() {
          var tracker = [];
          for (var i = 0; i < colCount; i++) {
            tracker.push(0);
          }
          return tracker;
        }
      }
    }
    GridLayoutFactory.$inject = ['$mdUtil'];
    /**
 * @ngdoc directive
 * @name mdGridTile
 * @module material.components.gridList
 * @restrict E
 * @description
 * Tiles contain the content of an `md-grid-list`. They span one or more grid
 * cells vertically or horizontally, and use `md-grid-tile-{footer,header}` to
 * display secondary content.
 *
 * ### Responsive Attributes
 *
 * The `md-grid-tile` directive supports "responsive" attributes, which allow
 * different `md-rowspan` and `md-colspan` values depending on the currently
 * matching media query (as defined in `$mdConstant.MEDIA`).
 *
 * In order to set a responsive attribute, first define the fallback value with
 * the standard attribute name, then add additional attributes with the
 * following convention: `{base-attribute-name}-{media-query-name}="{value}"`
 * (ie. `md-colspan-sm="4"`)
 *
 * @param {number=} md-colspan The number of columns to span (default 1). Cannot
 *    exceed the number of columns in the grid. Supports interpolation.
 * @param {number=} md-rowspan The number of rows to span (default 1). Supports
 *     interpolation.
 *
 * @usage
 * With header:
 * <hljs lang="html">
 * <md-grid-tile>
 *   <md-grid-tile-header>
 *     <h3>This is a header</h3>
 *   </md-grid-tile-header>
 * </md-grid-tile>
 * </hljs>
 *
 * With footer:
 * <hljs lang="html">
 * <md-grid-tile>
 *   <md-grid-tile-footer>
 *     <h3>This is a footer</h3>
 *   </md-grid-tile-footer>
 * </md-grid-tile>
 * </hljs>
 *
 * Spanning multiple rows/columns:
 * <hljs lang="html">
 * <md-grid-tile md-colspan="2" md-rowspan="3">
 * </md-grid-tile>
 * </hljs>
 *
 * Responsive attributes:
 * <hljs lang="html">
 * <md-grid-tile md-colspan="1" md-colspan-sm="3" md-colspan-md="5">
 * </md-grid-tile>
 * </hljs>
 */
    function GridTileDirective($mdMedia) {
      return {
        restrict: 'E',
        require: '^mdGridList',
        template: '<figure ng-transclude></figure>',
        transclude: true,
        scope: {},
        controller: [
          '$attrs',
          function ($attrs) {
            this.$attrs = $attrs;
          }
        ],
        link: postLink
      };
      function postLink(scope, element, attrs, gridCtrl) {
        // Apply semantics
        element.attr('role', 'listitem');
        // If our colspan or rowspan changes, trigger a layout
        var unwatchAttrs = $mdMedia.watchResponsiveAttributes([
            'md-colspan',
            'md-rowspan'
          ], attrs, angular.bind(gridCtrl, gridCtrl.invalidateLayout));
        // Tile registration/deregistration
        gridCtrl.invalidateTiles();
        scope.$on('$destroy', function () {
          // Mark the tile as destroyed so it is no longer considered in layout,
          // even if the DOM element sticks around (like during a leave animation)
          element[0].$$mdDestroyed = true;
          unwatchAttrs();
          gridCtrl.invalidateLayout();
        });
        if (angular.isDefined(scope.$parent.$index)) {
          scope.$watch(function () {
            return scope.$parent.$index;
          }, function indexChanged(newIdx, oldIdx) {
            if (newIdx === oldIdx) {
              return;
            }
            gridCtrl.invalidateTiles();
          });
        }
      }
    }
    GridTileDirective.$inject = ['$mdMedia'];
    function GridTileCaptionDirective() {
      return {
        template: '<figcaption ng-transclude></figcaption>',
        transclude: true
      };
    }
  }());
  (function () {
    'use strict';
    /**
 * @ngdoc module
 * @name material.components.icon
 * @description
 * Icon
 */
    angular.module('material.components.icon', ['material.core']);
  }());
  (function () {
    'use strict';
    /**
 * @ngdoc module
 * @name material.components.input
 */
    angular.module('material.components.input', ['material.core']).directive('mdInputContainer', mdInputContainerDirective).directive('label', labelDirective).directive('input', inputTextareaDirective).directive('textarea', inputTextareaDirective).directive('mdMaxlength', mdMaxlengthDirective).directive('placeholder', placeholderDirective).directive('ngMessages', ngMessagesDirective).directive('ngMessage', ngMessageDirective).directive('ngMessageExp', ngMessageDirective).animation('.md-input-invalid', mdInputInvalidMessagesAnimation).animation('.md-input-messages-animation', ngMessagesAnimation).animation('.md-input-message-animation', ngMessageAnimation);
    /**
 * @ngdoc directive
 * @name mdInputContainer
 * @module material.components.input
 *
 * @restrict E
 *
 * @description
 * `<md-input-container>` is the parent of any input or textarea element.
 *
 * Input and textarea elements will not behave properly unless the md-input-container
 * parent is provided.
 *
 * @param md-is-error {expression=} When the given expression evaluates to true, the input container
 *   will go into error state. Defaults to erroring if the input has been touched and is invalid.
 * @param md-no-float {boolean=} When present, placeholders will not be converted to floating
 *   labels.
 *
 * @usage
 * <hljs lang="html">
 *
 * <md-input-container>
 *   <label>Username</label>
 *   <input type="text" ng-model="user.name">
 * </md-input-container>
 *
 * <md-input-container>
 *   <label>Description</label>
 *   <textarea ng-model="user.description"></textarea>
 * </md-input-container>
 *
 * </hljs>
 */
    function mdInputContainerDirective($mdTheming, $parse) {
      ContainerCtrl.$inject = [
        '$scope',
        '$element',
        '$attrs',
        '$animate'
      ];
      return {
        restrict: 'E',
        link: postLink,
        controller: ContainerCtrl
      };
      function postLink(scope, element, attr) {
        $mdTheming(element);
        if (element.find('md-icon').length)
          element.addClass('md-has-icon');
      }
      function ContainerCtrl($scope, $element, $attrs, $animate) {
        var self = this;
        self.isErrorGetter = $attrs.mdIsError && $parse($attrs.mdIsError);
        self.delegateClick = function () {
          self.input.focus();
        };
        self.element = $element;
        self.setFocused = function (isFocused) {
          $element.toggleClass('md-input-focused', !!isFocused);
        };
        self.setHasValue = function (hasValue) {
          $element.toggleClass('md-input-has-value', !!hasValue);
        };
        self.setHasPlaceholder = function (hasPlaceholder) {
          $element.toggleClass('md-input-has-placeholder', !!hasPlaceholder);
        };
        self.setInvalid = function (isInvalid) {
          if (isInvalid) {
            $animate.addClass($element, 'md-input-invalid');
          } else {
            $animate.removeClass($element, 'md-input-invalid');
          }
        };
        $scope.$watch(function () {
          return self.label && self.input;
        }, function (hasLabelAndInput) {
          if (hasLabelAndInput && !self.label.attr('for')) {
            self.label.attr('for', self.input.attr('id'));
          }
        });
      }
    }
    mdInputContainerDirective.$inject = [
      '$mdTheming',
      '$parse'
    ];
    function labelDirective() {
      return {
        restrict: 'E',
        require: '^?mdInputContainer',
        link: function (scope, element, attr, containerCtrl) {
          if (!containerCtrl || attr.mdNoFloat || element.hasClass('md-container-ignore'))
            return;
          containerCtrl.label = element;
          scope.$on('$destroy', function () {
            containerCtrl.label = null;
          });
        }
      };
    }
    /**
 * @ngdoc directive
 * @name mdInput
 * @restrict E
 * @module material.components.input
 *
 * @description
 * You can use any `<input>` or `<textarea>` element as a child of an `<md-input-container>`. This
 * allows you to build complex forms for data entry.
 *
 * @param {number=} md-maxlength The maximum number of characters allowed in this input. If this is
 *   specified, a character counter will be shown underneath the input.<br/><br/>
 *   The purpose of **`md-maxlength`** is exactly to show the max length counter text. If you don't
 *   want the counter text and only need "plain" validation, you can use the "simple" `ng-maxlength`
 *   or maxlength attributes.
 * @param {string=} aria-label Aria-label is required when no label is present.  A warning message
 *   will be logged in the console if not present.
 * @param {string=} placeholder An alternative approach to using aria-label when the label is not
 *   PRESENT. The placeholder text is copied to the aria-label attribute.
 * @param md-no-autogrow {boolean=} When present, textareas will not grow automatically.
 * @param md-detect-hidden {boolean=} When present, textareas will be sized properly when they are
 *   revealed after being hidden. This is off by default for performance reasons because it
 *   guarantees a reflow every digest cycle.
 *
 * @usage
 * <hljs lang="html">
 * <md-input-container>
 *   <label>Color</label>
 *   <input type="text" ng-model="color" required md-maxlength="10">
 * </md-input-container>
 * </hljs>
 *
 * <h3>With Errors</h3>
 *
 * `md-input-container` also supports errors using the standard `ng-messages` directives and
 * animates the messages when they become visible using from the `ngEnter`/`ngLeave` events or
 * the `ngShow`/`ngHide` events.
 *
 * By default, the messages will be hidden until the input is in an error state. This is based off
 * of the `md-is-error` expression of the `md-input-container`. This gives the user a chance to
 * fill out the form before the errors become visible.
 *
 * <hljs lang="html">
 * <form name="colorForm">
 *   <md-input-container>
 *     <label>Favorite Color</label>
 *     <input name="favoriteColor" ng-model="favoriteColor" required>
 *     <div ng-messages="userForm.lastName.$error">
 *       <div ng-message="required">This is required!</div>
 *     </div>
 *   </md-input-container>
 * </form>
 * </hljs>
 *
 * We automatically disable this auto-hiding functionality if you provide any of the following
 * visibility directives on the `ng-messages` container:
 *
 *  - `ng-if`
 *  - `ng-show`/`ng-hide`
 *  - `ng-switch-when`/`ng-switch-default`
 *
 * You can also disable this functionality manually by adding the `md-auto-hide="false"` expression
 * to the `ng-messages` container. This may be helpful if you always want to see the error messages
 * or if you are building your own visibilty directive.
 *
 * _<b>Note:</b> The `md-auto-hide` attribute is a static string that is  only checked upon
 * initialization of the `ng-messages` directive to see if it equals the string `false`._
 *
 * <hljs lang="html">
 * <form name="userForm">
 *   <md-input-container>
 *     <label>Last Name</label>
 *     <input name="lastName" ng-model="lastName" required md-maxlength="10" minlength="4">
 *     <div ng-messages="userForm.lastName.$error" ng-show="userForm.lastName.$dirty">
 *       <div ng-message="required">This is required!</div>
 *       <div ng-message="md-maxlength">That's too long!</div>
 *       <div ng-message="minlength">That's too short!</div>
 *     </div>
 *   </md-input-container>
 *   <md-input-container>
 *     <label>Biography</label>
 *     <textarea name="bio" ng-model="biography" required md-maxlength="150"></textarea>
 *     <div ng-messages="userForm.bio.$error" ng-show="userForm.bio.$dirty">
 *       <div ng-message="required">This is required!</div>
 *       <div ng-message="md-maxlength">That's too long!</div>
 *     </div>
 *   </md-input-container>
 *   <md-input-container>
 *     <input aria-label='title' ng-model='title'>
 *   </md-input-container>
 *   <md-input-container>
 *     <input placeholder='title' ng-model='title'>
 *   </md-input-container>
 * </form>
 * </hljs>
 *
 * <h3>Notes</h3>
 *
 * - Requires [ngMessages](https://docs.angularjs.org/api/ngMessages).
 * - Behaves like the [AngularJS input directive](https://docs.angularjs.org/api/ng/directive/input).
 *
 * The `md-input` and `md-input-container` directives use very specific positioning to achieve the
 * error animation effects. Therefore, it is *not* advised to use the Layout system inside of the
 * `<md-input-container>` tags. Instead, use relative or absolute positioning.
 *
 */
    function inputTextareaDirective($mdUtil, $window, $mdAria) {
      return {
        restrict: 'E',
        require: [
          '^?mdInputContainer',
          '?ngModel'
        ],
        link: postLink
      };
      function postLink(scope, element, attr, ctrls) {
        var containerCtrl = ctrls[0];
        var hasNgModel = !!ctrls[1];
        var ngModelCtrl = ctrls[1] || $mdUtil.fakeNgModel();
        var isReadonly = angular.isDefined(attr.readonly);
        if (!containerCtrl)
          return;
        if (containerCtrl.input) {
          throw new Error('<md-input-container> can only have *one* <input>, <textarea> or <md-select> child element!');
        }
        containerCtrl.input = element;
        // Add an error spacer div after our input to provide space for the char counter and any ng-messages
        var errorsSpacer = angular.element('<div class="md-errors-spacer">');
        element.after(errorsSpacer);
        if (!containerCtrl.label) {
          $mdAria.expect(element, 'aria-label', element.attr('placeholder'));
        }
        element.addClass('md-input');
        if (!element.attr('id')) {
          element.attr('id', 'input_' + $mdUtil.nextUid());
        }
        if (element[0].tagName.toLowerCase() === 'textarea') {
          setupTextarea();
        }
        // If the input doesn't have an ngModel, it may have a static value. For that case,
        // we have to do one initial check to determine if the container should be in the
        // "has a value" state.
        if (!hasNgModel) {
          inputCheckValue();
        }
        var isErrorGetter = containerCtrl.isErrorGetter || function () {
            return ngModelCtrl.$invalid && (ngModelCtrl.$touched || ngModelCtrl.$$parentForm && ngModelCtrl.$$parentForm.$submitted);
          };
        scope.$watch(isErrorGetter, containerCtrl.setInvalid);
        ngModelCtrl.$parsers.push(ngModelPipelineCheckValue);
        ngModelCtrl.$formatters.push(ngModelPipelineCheckValue);
        element.on('input', inputCheckValue);
        if (!isReadonly) {
          element.on('focus', function (ev) {
            containerCtrl.setFocused(true);
          }).on('blur', function (ev) {
            containerCtrl.setFocused(false);
            inputCheckValue();
          });
        }
        //ngModelCtrl.$setTouched();
        //if( ngModelCtrl.$invalid ) containerCtrl.setInvalid();
        scope.$on('$destroy', function () {
          containerCtrl.setFocused(false);
          containerCtrl.setHasValue(false);
          containerCtrl.input = null;
        });
        /**
     *
     */
        function ngModelPipelineCheckValue(arg) {
          containerCtrl.setHasValue(!ngModelCtrl.$isEmpty(arg));
          return arg;
        }
        function inputCheckValue() {
          // An input's value counts if its length > 0,
          // or if the input's validity state says it has bad input (eg string in a number input)
          containerCtrl.setHasValue(element.val().length > 0 || (element[0].validity || {}).badInput);
        }
        function setupTextarea() {
          if (angular.isDefined(element.attr('md-no-autogrow'))) {
            return;
          }
          var node = element[0];
          var container = containerCtrl.element[0];
          var min_rows = NaN;
          var lineHeight = null;
          // can't check if height was or not explicity set,
          // so rows attribute will take precedence if present
          if (node.hasAttribute('rows')) {
            min_rows = parseInt(node.getAttribute('rows'));
          }
          var onChangeTextarea = $mdUtil.debounce(growTextarea, 1);
          function pipelineListener(value) {
            onChangeTextarea();
            return value;
          }
          if (ngModelCtrl) {
            ngModelCtrl.$formatters.push(pipelineListener);
            ngModelCtrl.$viewChangeListeners.push(pipelineListener);
          } else {
            onChangeTextarea();
          }
          element.on('keydown input', onChangeTextarea);
          if (isNaN(min_rows)) {
            element.attr('rows', '1');
            element.on('scroll', onScroll);
          }
          angular.element($window).on('resize', onChangeTextarea);
          scope.$on('$destroy', function () {
            angular.element($window).off('resize', onChangeTextarea);
          });
          function growTextarea() {
            // sets the md-input-container height to avoid jumping around
            container.style.height = container.offsetHeight + 'px';
            // temporarily disables element's flex so its height 'runs free'
            element.addClass('md-no-flex');
            if (isNaN(min_rows)) {
              node.style.height = 'auto';
              node.scrollTop = 0;
              var height = getHeight();
              if (height)
                node.style.height = height + 'px';
            } else {
              node.setAttribute('rows', 1);
              if (!lineHeight) {
                node.style.minHeight = '0';
                lineHeight = element.prop('clientHeight');
                node.style.minHeight = null;
              }
              var rows = Math.min(min_rows, Math.round(node.scrollHeight / lineHeight));
              node.setAttribute('rows', rows);
              node.style.height = lineHeight * rows + 'px';
            }
            // reset everything back to normal
            element.removeClass('md-no-flex');
            container.style.height = 'auto';
          }
          function getHeight() {
            var line = node.scrollHeight - node.offsetHeight;
            return node.offsetHeight + (line > 0 ? line : 0);
          }
          function onScroll(e) {
            node.scrollTop = 0;
            // for smooth new line adding
            var line = node.scrollHeight - node.offsetHeight;
            var height = node.offsetHeight + line;
            node.style.height = height + 'px';
          }
          // Attach a watcher to detect when the textarea gets shown.
          if (angular.isDefined(element.attr('md-detect-hidden'))) {
            var handleHiddenChange = function () {
                var wasHidden = false;
                return function () {
                  var isHidden = node.offsetHeight === 0;
                  if (isHidden === false && wasHidden === true) {
                    growTextarea();
                  }
                  wasHidden = isHidden;
                };
              }();
            // Check every digest cycle whether the visibility of the textarea has changed.
            // Queue up to run after the digest cycle is complete.
            scope.$watch(function () {
              $mdUtil.nextTick(handleHiddenChange, false);
              return true;
            });
          }
        }
      }
    }
    inputTextareaDirective.$inject = [
      '$mdUtil',
      '$window',
      '$mdAria'
    ];
    function mdMaxlengthDirective($animate, $mdUtil) {
      return {
        restrict: 'A',
        require: [
          'ngModel',
          '^mdInputContainer'
        ],
        link: postLink
      };
      function postLink(scope, element, attr, ctrls) {
        var maxlength;
        var ngModelCtrl = ctrls[0];
        var containerCtrl = ctrls[1];
        var charCountEl, errorsSpacer;
        // Wait until the next tick to ensure that the input has setup the errors spacer where we will
        // append our counter
        $mdUtil.nextTick(function () {
          errorsSpacer = angular.element(containerCtrl.element[0].querySelector('.md-errors-spacer'));
          charCountEl = angular.element('<div class="md-char-counter">');
          // Append our character counter inside the errors spacer
          errorsSpacer.append(charCountEl);
          // Stop model from trimming. This makes it so whitespace
          // over the maxlength still counts as invalid.
          attr.$set('ngTrim', 'false');
          ngModelCtrl.$formatters.push(renderCharCount);
          ngModelCtrl.$viewChangeListeners.push(renderCharCount);
          element.on('input keydown keyup', function () {
            renderCharCount();  //make sure it's called with no args
          });
          scope.$watch(attr.mdMaxlength, function (value) {
            maxlength = value;
            if (angular.isNumber(value) && value > 0) {
              if (!charCountEl.parent().length) {
                $animate.enter(charCountEl, errorsSpacer);
              }
              renderCharCount();
            } else {
              $animate.leave(charCountEl);
            }
          });
          ngModelCtrl.$validators['md-maxlength'] = function (modelValue, viewValue) {
            if (!angular.isNumber(maxlength) || maxlength < 0) {
              return true;
            }
            return (modelValue || element.val() || viewValue || '').length <= maxlength;
          };
        });
        function renderCharCount(value) {
          // If we have not been appended to the body yet; do not render
          if (!charCountEl.parent) {
            return value;
          }
          // Force the value into a string since it may be a number,
          // which does not have a length property.
          charCountEl.text(String(element.val() || value || '').length + '/' + maxlength);
          return value;
        }
      }
    }
    mdMaxlengthDirective.$inject = [
      '$animate',
      '$mdUtil'
    ];
    function placeholderDirective($log) {
      return {
        restrict: 'A',
        require: '^^?mdInputContainer',
        priority: 200,
        link: postLink
      };
      function postLink(scope, element, attr, inputContainer) {
        // If there is no input container, just return
        if (!inputContainer)
          return;
        var label = inputContainer.element.find('label');
        var hasNoFloat = angular.isDefined(inputContainer.element.attr('md-no-float'));
        // If we have a label, or they specify the md-no-float attribute, just return
        if (label && label.length || hasNoFloat) {
          // Add a placeholder class so we can target it in the CSS
          inputContainer.setHasPlaceholder(true);
          return;
        }
        // Otherwise, grab/remove the placeholder
        var placeholderText = attr.placeholder;
        element.removeAttr('placeholder');
        // And add the placeholder text as a separate label
        if (inputContainer.input && inputContainer.input[0].nodeName != 'MD-SELECT') {
          var placeholder = '<label ng-click="delegateClick()">' + placeholderText + '</label>';
          inputContainer.element.addClass('md-icon-float');
          inputContainer.element.prepend(placeholder);
        }
      }
    }
    placeholderDirective.$inject = ['$log'];
    var visibilityDirectives = [
        'ngIf',
        'ngShow',
        'ngHide',
        'ngSwitchWhen',
        'ngSwitchDefault'
      ];
    function ngMessagesDirective() {
      return {
        restrict: 'EA',
        link: postLink,
        require: '^^?mdInputContainer'
      };
      function postLink(scope, element, attrs, inputContainer) {
        // If we are not a child of an input container, don't do anything
        if (!inputContainer)
          return;
        // Add our animation class
        element.toggleClass('md-input-messages-animation', true);
        // Add our md-auto-hide class to automatically hide/show messages when container is invalid
        element.toggleClass('md-auto-hide', true);
        // If we see some known visibility directives, remove the md-auto-hide class
        if (attrs.mdAutoHide == 'false' || hasVisibiltyDirective(attrs)) {
          element.toggleClass('md-auto-hide', false);
        }
      }
      function hasVisibiltyDirective(attrs) {
        return visibilityDirectives.some(function (attr) {
          return attrs[attr];
        });
      }
    }
    function ngMessageDirective($mdUtil) {
      return {
        restrict: 'EA',
        compile: compile,
        priority: 100
      };
      function compile(element) {
        var inputContainer = $mdUtil.getClosest(element, 'md-input-container');
        // If we are not a child of an input container, don't do anything
        if (!inputContainer)
          return;
        // Add our animation class
        element.toggleClass('md-input-message-animation', true);
        return {};
      }
    }
    ngMessageDirective.$inject = ['$mdUtil'];
    function mdInputInvalidMessagesAnimation($q, $animateCss) {
      return {
        addClass: function (element, className, done) {
          var messages = getMessagesElement(element);
          if (className == 'md-input-invalid' && messages.hasClass('md-auto-hide')) {
            showInputMessages(element, $animateCss, $q).finally(done);
          }
        }  // NOTE: We do not need the removeClass method, because the message ng-leave animation will fire
      };
    }
    mdInputInvalidMessagesAnimation.$inject = [
      '$q',
      '$animateCss'
    ];
    function ngMessagesAnimation($q, $animateCss) {
      return {
        enter: function (element, done) {
          showInputMessages(element, $animateCss, $q).finally(done);
        },
        leave: function (element, done) {
          hideInputMessages(element, $animateCss, $q).finally(done);
        },
        addClass: function (element, className, done) {
          if (className == 'ng-hide') {
            hideInputMessages(element, $animateCss, $q).finally(done);
          } else {
            done();
          }
        },
        removeClass: function (element, className, done) {
          if (className == 'ng-hide') {
            showInputMessages(element, $animateCss, $q).finally(done);
          } else {
            done();
          }
        }
      };
    }
    ngMessagesAnimation.$inject = [
      '$q',
      '$animateCss'
    ];
    function ngMessageAnimation($animateCss) {
      return {
        enter: function (element, done) {
          var messages = getMessagesElement(element);
          // If we have the md-auto-hide class, the md-input-invalid animation will fire, so we can skip
          if (messages.hasClass('md-auto-hide')) {
            done();
            return;
          }
          return showMessage(element, $animateCss);
        },
        leave: function (element, done) {
          return hideMessage(element, $animateCss);
        }
      };
    }
    ngMessageAnimation.$inject = ['$animateCss'];
    function showInputMessages(element, $animateCss, $q) {
      var animators = [], animator;
      var messages = getMessagesElement(element);
      angular.forEach(messages.children(), function (child) {
        animator = showMessage(angular.element(child), $animateCss);
        animators.push(animator.start());
      });
      return $q.all(animators);
    }
    function hideInputMessages(element, $animateCss, $q) {
      var animators = [], animator;
      var messages = getMessagesElement(element);
      angular.forEach(messages.children(), function (child) {
        animator = hideMessage(angular.element(child), $animateCss);
        animators.push(animator.start());
      });
      return $q.all(animators);
    }
    function showMessage(element, $animateCss) {
      var height = element[0].offsetHeight;
      return $animateCss(element, {
        event: 'enter',
        structural: true,
        from: {
          'opacity': 0,
          'margin-top': -height + 'px'
        },
        to: {
          'opacity': 1,
          'margin-top': '0'
        },
        duration: 0.3
      });
    }
    function hideMessage(element, $animateCss) {
      var height = element[0].offsetHeight;
      var styles = window.getComputedStyle(element[0]);
      // If we are already hidden, just return an empty animation
      if (styles.opacity == 0) {
        return $animateCss(element, {});
      }
      // Otherwise, animate
      return $animateCss(element, {
        event: 'leave',
        structural: true,
        from: {
          'opacity': 1,
          'margin-top': 0
        },
        to: {
          'opacity': 0,
          'margin-top': -height + 'px'
        },
        duration: 0.3
      });
    }
    function getInputElement(element) {
      var inputContainer = element.controller('mdInputContainer');
      return inputContainer.element;
    }
    function getMessagesElement(element) {
      var input = getInputElement(element);
      var selector = 'ng-messages,data-ng-messages,x-ng-messages,' + '[ng-messages],[data-ng-messages],[x-ng-messages]';
      return angular.element(input[0].querySelector(selector));
    }
  }());
  (function () {
    'use strict';
    /**
 * @ngdoc module
 * @name material.components.list
 * @description
 * List module
 */
    angular.module('material.components.list', ['material.core']).controller('MdListController', MdListController).directive('mdList', mdListDirective).directive('mdListItem', mdListItemDirective);
    /**
 * @ngdoc directive
 * @name mdList
 * @module material.components.list
 *
 * @restrict E
 *
 * @description
 * The `<md-list>` directive is a list container for 1..n `<md-list-item>` tags.
 *
 * @usage
 * <hljs lang="html">
 * <md-list>
 *   <md-list-item class="md-2-line" ng-repeat="item in todos">
 *     <md-checkbox ng-model="item.done"></md-checkbox>
 *     <div class="md-list-item-text">
 *       <h3>{{item.title}}</h3>
 *       <p>{{item.description}}</p>
 *     </div>
 *   </md-list-item>
 * </md-list>
 * </hljs>
 */
    function mdListDirective($mdTheming) {
      return {
        restrict: 'E',
        compile: function (tEl) {
          tEl[0].setAttribute('role', 'list');
          return $mdTheming;
        }
      };
    }
    mdListDirective.$inject = ['$mdTheming'];
    /**
 * @ngdoc directive
 * @name mdListItem
 * @module material.components.list
 *
 * @restrict E
 *
 * @description
 * The `<md-list-item>` directive is a container intended for row items in a `<md-list>` container.
 * The `md-2-line` and `md-3-line` classes can be added to a `<md-list-item>` 
 * to increase the height with 22px and 40px respectively.
 *
 * ## CSS
 * `.md-avatar` - class for image avatars
 *
 * `.md-avatar-icon` - class for icon avatars
 *
 * `.md-offset` - on content without an avatar
 *
 * @usage
 * <hljs lang="html">
 *  <md-list>
 *    <md-list-item>
 *      <img class="md-avatar" ng-src="path/to/img"/>
 *      <span>Item content in list</span>
 *    </md-list-item>
 *    <md-list-item>
 *      <md-icon class="md-avatar-icon" md-svg-icon="communication:phone"></md-icon>
 *      <span>Item content in list</span>
 *    </md-list-item>
 *  </md-list>
 * </hljs>
 *
 * _**Note:** We automatically apply special styling when the inner contents are wrapped inside
 * of a `<md-button>` tag. This styling is automatically ignored for `class="md-secondary"` buttons
 * and you can include a class of `class="md-exclude"` if you need to use a non-secondary button
 * that is inside the list, but does not wrap the contents._
 */
    function mdListItemDirective($mdAria, $mdConstant, $mdUtil, $timeout) {
      var proxiedTypes = [
          'md-checkbox',
          'md-switch'
        ];
      return {
        restrict: 'E',
        controller: 'MdListController',
        compile: function (tEl, tAttrs) {
          // Check for proxy controls (no ng-click on parent, and a control inside)
          var secondaryItem = tEl[0].querySelector('.md-secondary');
          var hasProxiedElement;
          var proxyElement;
          tEl[0].setAttribute('role', 'listitem');
          if (tAttrs.ngClick || tAttrs.ngHref || tAttrs.href || tAttrs.uiSref || tAttrs.ngAttrUiSref) {
            wrapIn('button');
          } else {
            for (var i = 0, type; type = proxiedTypes[i]; ++i) {
              if (proxyElement = tEl[0].querySelector(type)) {
                hasProxiedElement = true;
                break;
              }
            }
            if (hasProxiedElement) {
              wrapIn('div');
            } else if (!tEl[0].querySelector('md-button:not(.md-secondary):not(.md-exclude)')) {
              tEl.addClass('md-no-proxy');
            }
          }
          wrapSecondary();
          setupToggleAria();
          function setupToggleAria() {
            var toggleTypes = [
                'md-switch',
                'md-checkbox'
              ];
            var toggle;
            for (var i = 0, toggleType; toggleType = toggleTypes[i]; ++i) {
              if (toggle = tEl.find(toggleType)[0]) {
                if (!toggle.hasAttribute('aria-label')) {
                  var p = tEl.find('p')[0];
                  if (!p)
                    return;
                  toggle.setAttribute('aria-label', 'Toggle ' + p.textContent);
                }
              }
            }
          }
          function wrapIn(type) {
            var container;
            if (type == 'div') {
              container = angular.element('<div class="md-no-style md-list-item-inner">');
              container.append(tEl.contents());
              tEl.addClass('md-proxy-focus');
            } else {
              container = angular.element('<md-button class="md-no-style"><div class="md-list-item-inner"></div></md-button>');
              copyAttributes(tEl[0], container[0]);
              container.children().eq(0).append(tEl.contents());
            }
            tEl[0].setAttribute('tabindex', '-1');
            tEl.append(container);
          }
          function wrapSecondary() {
            if (secondaryItem && !isButton(secondaryItem) && secondaryItem.hasAttribute('ng-click')) {
              $mdAria.expect(secondaryItem, 'aria-label');
              var buttonWrapper = angular.element('<md-button class="md-secondary-container md-icon-button">');
              copyAttributes(secondaryItem, buttonWrapper[0]);
              secondaryItem.setAttribute('tabindex', '-1');
              secondaryItem.classList.remove('md-secondary');
              buttonWrapper.append(secondaryItem);
              secondaryItem = buttonWrapper[0];
            }
            // Check for a secondary item and move it outside
            if (secondaryItem && (secondaryItem.hasAttribute('ng-click') || tAttrs.ngClick && isProxiedElement(secondaryItem))) {
              tEl.addClass('md-with-secondary');
              tEl.append(secondaryItem);
            }
          }
          function copyAttributes(item, wrapper) {
            var copiedAttrs = [
                'ng-if',
                'ng-click',
                'aria-label',
                'ng-disabled',
                'ui-sref',
                'href',
                'ng-href',
                'ng-attr-ui-sref'
              ];
            angular.forEach(copiedAttrs, function (attr) {
              if (item.hasAttribute(attr)) {
                wrapper.setAttribute(attr, item.getAttribute(attr));
                item.removeAttribute(attr);
              }
            });
          }
          function isProxiedElement(el) {
            return proxiedTypes.indexOf(el.nodeName.toLowerCase()) != -1;
          }
          function isButton(el) {
            var nodeName = el.nodeName.toUpperCase();
            return nodeName == 'MD-BUTTON' || nodeName == 'BUTTON';
          }
          return postLink;
          function postLink($scope, $element, $attr, ctrl) {
            var proxies = [], firstChild = $element[0].firstElementChild, hasClick = firstChild && hasClickEvent(firstChild);
            computeProxies();
            computeClickable();
            if ($element.hasClass('md-proxy-focus') && proxies.length) {
              angular.forEach(proxies, function (proxy) {
                proxy = angular.element(proxy);
                $scope.mouseActive = false;
                proxy.on('mousedown', function () {
                  $scope.mouseActive = true;
                  $timeout(function () {
                    $scope.mouseActive = false;
                  }, 100);
                }).on('focus', function () {
                  if ($scope.mouseActive === false) {
                    $element.addClass('md-focused');
                  }
                  proxy.on('blur', function proxyOnBlur() {
                    $element.removeClass('md-focused');
                    proxy.off('blur', proxyOnBlur);
                  });
                });
              });
            }
            function hasClickEvent(element) {
              var attr = element.attributes;
              for (var i = 0; i < attr.length; i++) {
                if ($attr.$normalize(attr[i].name) === 'ngClick')
                  return true;
              }
              return false;
            }
            function computeProxies() {
              var children = $element.children();
              if (children.length && !children[0].hasAttribute('ng-click')) {
                angular.forEach(proxiedTypes, function (type) {
                  angular.forEach(firstChild.querySelectorAll(type), function (child) {
                    proxies.push(child);
                  });
                });
              }
            }
            function computeClickable() {
              if (proxies.length == 1 || hasClick) {
                $element.addClass('md-clickable');
                if (!hasClick) {
                  ctrl.attachRipple($scope, angular.element($element[0].querySelector('.md-no-style')));
                }
              }
            }
            if (!hasClick && !proxies.length) {
              firstChild && firstChild.addEventListener('keypress', function (e) {
                if (e.target.nodeName != 'INPUT' && e.target.nodeName != 'TEXTAREA') {
                  var keyCode = e.which || e.keyCode;
                  if (keyCode == $mdConstant.KEY_CODE.SPACE) {
                    if (firstChild) {
                      firstChild.click();
                      e.preventDefault();
                      e.stopPropagation();
                    }
                  }
                }
              });
            }
            $element.off('click');
            $element.off('keypress');
            if (proxies.length == 1 && firstChild) {
              $element.children().eq(0).on('click', function (e) {
                var parentButton = $mdUtil.getClosest(e.target, 'BUTTON');
                if (!parentButton && firstChild.contains(e.target)) {
                  angular.forEach(proxies, function (proxy) {
                    if (e.target !== proxy && !proxy.contains(e.target)) {
                      angular.element(proxy).triggerHandler('click');
                    }
                  });
                }
              });
            }
          }
        }
      };
    }
    mdListItemDirective.$inject = [
      '$mdAria',
      '$mdConstant',
      '$mdUtil',
      '$timeout'
    ];
    /*
 * @private
 * @ngdoc controller
 * @name MdListController
 * @module material.components.list
 *
 */
    function MdListController($scope, $element, $mdListInkRipple) {
      var ctrl = this;
      ctrl.attachRipple = attachRipple;
      function attachRipple(scope, element) {
        var options = {};
        $mdListInkRipple.attach(scope, element, options);
      }
    }
    MdListController.$inject = [
      '$scope',
      '$element',
      '$mdListInkRipple'
    ];
  }());
  (function () {
    'use strict';
    /**
 * @ngdoc module
 * @name material.components.menu
 */
    angular.module('material.components.menu', [
      'material.core',
      'material.components.backdrop'
    ]);
  }());
  (function () {
    'use strict';
    /**
 * @ngdoc module
 * @name material.components.menu-bar
 */
    angular.module('material.components.menuBar', [
      'material.core',
      'material.components.menu'
    ]);
  }());
  (function () {
    'use strict';
    /**
 * @ngdoc module
 * @name material.components.progressCircular
 * @description Circular Progress module!
 */
    angular.module('material.components.progressCircular', ['material.core']).directive('mdProgressCircular', MdProgressCircularDirective);
    /**
 * @ngdoc directive
 * @name mdProgressCircular
 * @module material.components.progressCircular
 * @restrict E
 *
* @description
 * The circular progress directive is used to make loading content in your app as delightful and
 * painless as possible by minimizing the amount of visual change a user sees before they can view
 * and interact with content.
 *
 * For operations where the percentage of the operation completed can be determined, use a
 * determinate indicator. They give users a quick sense of how long an operation will take.
 *
 * For operations where the user is asked to wait a moment while something finishes up, and it’s
 * not necessary to expose what's happening behind the scenes and how long it will take, use an
 * indeterminate indicator.
 *
 * @param {string} md-mode Select from one of two modes: **'determinate'** and **'indeterminate'**.
 *
 * Note: if the `md-mode` value is set as undefined or specified as not 1 of the two (2) valid modes, then `.ng-hide`
 * will be auto-applied as a style to the component.
 *
 * Note: if not configured, the `md-mode="indeterminate"` will be auto injected as an attribute.
 * If `value=""` is also specified, however, then `md-mode="determinate"` would be auto-injected instead.
 * @param {number=} value In determinate mode, this number represents the percentage of the
 *     circular progress. Default: 0
 * @param {number=} md-diameter This specifies the diamter of the circular progress. The value
 * may be a percentage (eg '25%') or a pixel-size value (eg '48'). If this attribute is
 * not present then a default value of '48px' is assumed.
 *
 * @usage
 * <hljs lang="html">
 * <md-progress-circular md-mode="determinate" value="..."></md-progress-circular>
 *
 * <md-progress-circular md-mode="determinate" ng-value="..."></md-progress-circular>
 *
 * <md-progress-circular md-mode="determinate" value="..." md-diameter="100"></md-progress-circular>
 *
 * <md-progress-circular md-mode="indeterminate"></md-progress-circular>
 * </hljs>
 */
    function MdProgressCircularDirective($mdTheming, $mdUtil, $log) {
      var DEFAULT_PROGRESS_SIZE = 100;
      var DEFAULT_SCALING = 0.5;
      var MODE_DETERMINATE = 'determinate', MODE_INDETERMINATE = 'indeterminate';
      return {
        restrict: 'E',
        scope: true,
        template: '<div class="md-scale-wrapper">' + '<div class="md-spinner-wrapper">' + '<div class="md-inner">' + '<div class="md-gap"></div>' + '<div class="md-left">' + '<div class="md-half-circle"></div>' + '</div>' + '<div class="md-right">' + '<div class="md-half-circle"></div>' + '</div>' + '</div>' + '</div>' + '</div>',
        compile: compile
      };
      function compile(tElement) {
        // The javascript in this file is mainly responsible for setting the correct aria attributes.
        // The animation of the progress spinner is done entirely with just CSS.
        tElement.attr('aria-valuemin', 0);
        tElement.attr('aria-valuemax', 100);
        tElement.attr('role', 'progressbar');
        return postLink;
      }
      function postLink(scope, element, attr) {
        $mdTheming(element);
        var circle = element;
        var spinnerWrapper = angular.element(element.children()[0]);
        var lastMode, toVendorCSS = $mdUtil.dom.animator.toCss;
        element.attr('md-mode', mode());
        updateScale();
        validateMode();
        watchAttributes();
        /**
     * Watch the value and md-mode attributes
     */
        function watchAttributes() {
          attr.$observe('value', function (value) {
            var percentValue = clamp(value);
            element.attr('aria-valuenow', percentValue);
            if (mode() == MODE_DETERMINATE) {
              animateIndicator(percentValue);
            }
          });
          attr.$observe('mdMode', function (mode) {
            switch (mode) {
            case MODE_DETERMINATE:
            case MODE_INDETERMINATE:
              spinnerWrapper.removeClass('ng-hide');
              if (lastMode)
                spinnerWrapper.removeClass(lastMode);
              spinnerWrapper.addClass(lastMode = 'md-mode-' + mode);
              break;
            default:
              if (lastMode)
                spinnerWrapper.removeClass(lastMode);
              spinnerWrapper.addClass('ng-hide');
              lastMode = undefined;
              break;
            }
          });
        }
        /**
     * Update size/scaling of the progress indicator
     * Watch the "value" and "md-mode" attributes
     */
        function updateScale() {
          // set the outer container to the size the user specified
          circle.css({
            width: 100 * getDiameterRatio() + 'px',
            height: 100 * getDiameterRatio() + 'px'
          });
          // the internal element is still 100px, so we have to scale it down to match the size
          circle.children().eq(0).css(toVendorCSS({ transform: $mdUtil.supplant('translate(-50%, -50%) scale( {0} )', [getDiameterRatio()]) }));
        }
        /**
     * Auto-defaults the mode to either `determinate` or `indeterminate` mode; if not specified
     */
        function validateMode() {
          if (angular.isUndefined(attr.mdMode)) {
            var hasValue = angular.isDefined(attr.value);
            var mode = hasValue ? MODE_DETERMINATE : MODE_INDETERMINATE;
            var info = 'Auto-adding the missing md-mode=\'{0}\' to the ProgressCircular element';
            $log.debug($mdUtil.supplant(info, [mode]));
            element.attr('md-mode', mode);
            attr['mdMode'] = mode;
          }
        }
        var leftC, rightC, gap;
        /**
     * Manually animate the Determinate indicator based on the specified
     * percentage value (0-100).
     *
     * Note: this animation was previously done using SCSS.
     * - generated 54K of styles
     * - use attribute selectors which had poor performances in IE
     */
        function animateIndicator(value) {
          if (!mode())
            return;
          leftC = leftC || angular.element(element[0].querySelector('.md-left > .md-half-circle'));
          rightC = rightC || angular.element(element[0].querySelector('.md-right > .md-half-circle'));
          gap = gap || angular.element(element[0].querySelector('.md-gap'));
          var gapStyles = removeEmptyValues({
              borderBottomColor: value <= 50 ? 'transparent !important' : '',
              transition: value <= 50 ? '' : 'borderBottomColor 0.1s linear'
            }), leftStyles = removeEmptyValues({
              transition: value <= 50 ? 'transform 0.1s linear' : '',
              transform: $mdUtil.supplant('rotate({0}deg)', [value <= 50 ? 135 : (value - 50) / 50 * 180 + 135])
            }), rightStyles = removeEmptyValues({
              transition: value >= 50 ? 'transform 0.1s linear' : '',
              transform: $mdUtil.supplant('rotate({0}deg)', [value >= 50 ? 45 : value / 50 * 180 - 135])
            });
          leftC.css(toVendorCSS(leftStyles));
          rightC.css(toVendorCSS(rightStyles));
          gap.css(toVendorCSS(gapStyles));
        }
        /**
     * We will scale the progress circle based on the default diameter.
     *
     * Determine the diameter percentage (defaults to 100%)
     * May be express as float, percentage, or integer
     */
        function getDiameterRatio() {
          if (!attr.mdDiameter)
            return DEFAULT_SCALING;
          var match = /([0-9]*)%/.exec(attr.mdDiameter);
          var value = Math.max(0, match && match[1] / 100 || parseFloat(attr.mdDiameter));
          // should return ratio; DEFAULT_PROGRESS_SIZE === 100px is default size
          return value > 1 ? value / DEFAULT_PROGRESS_SIZE : value;
        }
        /**
     * Is the md-mode a valid option?
     */
        function mode() {
          var value = (attr.mdMode || '').trim();
          if (value) {
            switch (value) {
            case MODE_DETERMINATE:
            case MODE_INDETERMINATE:
              break;
            default:
              value = undefined;
              break;
            }
          }
          return value;
        }
      }
      /**
   * Clamps the value to be between 0 and 100.
   * @param {number} value The value to clamp.
   * @returns {number}
   */
      function clamp(value) {
        return Math.max(0, Math.min(value || 0, 100));
      }
      function removeEmptyValues(target) {
        for (var key in target) {
          if (target.hasOwnProperty(key)) {
            if (target[key] == '')
              delete target[key];
          }
        }
        return target;
      }
    }
    MdProgressCircularDirective.$inject = [
      '$mdTheming',
      '$mdUtil',
      '$log'
    ];
  }());
  (function () {
    'use strict';
    /**
 * @ngdoc module
 * @name material.components.radioButton
 * @description radioButton module!
 */
    angular.module('material.components.radioButton', ['material.core']).directive('mdRadioGroup', mdRadioGroupDirective).directive('mdRadioButton', mdRadioButtonDirective);
    /**
 * @ngdoc directive
 * @module material.components.radioButton
 * @name mdRadioGroup
 *
 * @restrict E
 *
 * @description
 * The `<md-radio-group>` directive identifies a grouping
 * container for the 1..n grouped radio buttons; specified using nested
 * `<md-radio-button>` tags.
 *
 * As per the [material design spec](http://www.google.com/design/spec/style/color.html#color-ui-color-application)
 * the radio button is in the accent color by default. The primary color palette may be used with
 * the `md-primary` class.
 *
 * Note: `<md-radio-group>` and `<md-radio-button>` handle tabindex differently
 * than the native `<input type='radio'>` controls. Whereas the native controls
 * force the user to tab through all the radio buttons, `<md-radio-group>`
 * is focusable, and by default the `<md-radio-button>`s are not.
 *
 * @param {string} ng-model Assignable angular expression to data-bind to.
 * @param {boolean=} md-no-ink Use of attribute indicates flag to disable ink ripple effects.
 *
 * @usage
 * <hljs lang="html">
 * <md-radio-group ng-model="selected">
 *
 *   <md-radio-button
 *        ng-repeat="d in colorOptions"
 *        ng-value="d.value" aria-label="{{ d.label }}">
 *
 *          {{ d.label }}
 *
 *   </md-radio-button>
 *
 * </md-radio-group>
 * </hljs>
 *
 */
    function mdRadioGroupDirective($mdUtil, $mdConstant, $mdTheming, $timeout) {
      RadioGroupController.prototype = createRadioGroupControllerProto();
      return {
        restrict: 'E',
        controller: [
          '$element',
          RadioGroupController
        ],
        require: [
          'mdRadioGroup',
          '?ngModel'
        ],
        link: { pre: linkRadioGroup }
      };
      function linkRadioGroup(scope, element, attr, ctrls) {
        $mdTheming(element);
        var rgCtrl = ctrls[0];
        var ngModelCtrl = ctrls[1] || $mdUtil.fakeNgModel();
        rgCtrl.init(ngModelCtrl);
        scope.mouseActive = false;
        element.attr({
          'role': 'radiogroup',
          'tabIndex': element.attr('tabindex') || '0'
        }).on('keydown', keydownListener).on('mousedown', function (event) {
          scope.mouseActive = true;
          $timeout(function () {
            scope.mouseActive = false;
          }, 100);
        }).on('focus', function () {
          if (scope.mouseActive === false) {
            rgCtrl.$element.addClass('md-focused');
          }
        }).on('blur', function () {
          rgCtrl.$element.removeClass('md-focused');
        });
        /**
     *
     */
        function setFocus() {
          if (!element.hasClass('md-focused')) {
            element.addClass('md-focused');
          }
        }
        /**
     *
     */
        function keydownListener(ev) {
          var keyCode = ev.which || ev.keyCode;
          // Only listen to events that we originated ourselves
          // so that we don't trigger on things like arrow keys in
          // inputs.
          if (keyCode != $mdConstant.KEY_CODE.ENTER && ev.currentTarget != ev.target) {
            return;
          }
          switch (keyCode) {
          case $mdConstant.KEY_CODE.LEFT_ARROW:
          case $mdConstant.KEY_CODE.UP_ARROW:
            ev.preventDefault();
            rgCtrl.selectPrevious();
            setFocus();
            break;
          case $mdConstant.KEY_CODE.RIGHT_ARROW:
          case $mdConstant.KEY_CODE.DOWN_ARROW:
            ev.preventDefault();
            rgCtrl.selectNext();
            setFocus();
            break;
          case $mdConstant.KEY_CODE.ENTER:
            var form = angular.element($mdUtil.getClosest(element[0], 'form'));
            if (form.length > 0) {
              form.triggerHandler('submit');
            }
            break;
          }
        }
      }
      function RadioGroupController($element) {
        this._radioButtonRenderFns = [];
        this.$element = $element;
      }
      function createRadioGroupControllerProto() {
        return {
          init: function (ngModelCtrl) {
            this._ngModelCtrl = ngModelCtrl;
            this._ngModelCtrl.$render = angular.bind(this, this.render);
          },
          add: function (rbRender) {
            this._radioButtonRenderFns.push(rbRender);
          },
          remove: function (rbRender) {
            var index = this._radioButtonRenderFns.indexOf(rbRender);
            if (index !== -1) {
              this._radioButtonRenderFns.splice(index, 1);
            }
          },
          render: function () {
            this._radioButtonRenderFns.forEach(function (rbRender) {
              rbRender();
            });
          },
          setViewValue: function (value, eventType) {
            this._ngModelCtrl.$setViewValue(value, eventType);
            // update the other radio buttons as well
            this.render();
          },
          getViewValue: function () {
            return this._ngModelCtrl.$viewValue;
          },
          selectNext: function () {
            return changeSelectedButton(this.$element, 1);
          },
          selectPrevious: function () {
            return changeSelectedButton(this.$element, -1);
          },
          setActiveDescendant: function (radioId) {
            this.$element.attr('aria-activedescendant', radioId);
          }
        };
      }
      /**
   * Change the radio group's selected button by a given increment.
   * If no button is selected, select the first button.
   */
      function changeSelectedButton(parent, increment) {
        // Coerce all child radio buttons into an array, then wrap then in an iterator
        var buttons = $mdUtil.iterator(parent[0].querySelectorAll('md-radio-button'), true);
        if (buttons.count()) {
          var validate = function (button) {
            // If disabled, then NOT valid
            return !angular.element(button).attr('disabled');
          };
          var selected = parent[0].querySelector('md-radio-button.md-checked');
          var target = buttons[increment < 0 ? 'previous' : 'next'](selected, validate) || buttons.first();
          // Activate radioButton's click listener (triggerHandler won't create a real click event)
          angular.element(target).triggerHandler('click');
        }
      }
    }
    mdRadioGroupDirective.$inject = [
      '$mdUtil',
      '$mdConstant',
      '$mdTheming',
      '$timeout'
    ];
    /**
 * @ngdoc directive
 * @module material.components.radioButton
 * @name mdRadioButton
 *
 * @restrict E
 *
 * @description
 * The `<md-radio-button>`directive is the child directive required to be used within `<md-radio-group>` elements.
 *
 * While similar to the `<input type="radio" ng-model="" value="">` directive,
 * the `<md-radio-button>` directive provides ink effects, ARIA support, and
 * supports use within named radio groups.
 *
 * @param {string} ngModel Assignable angular expression to data-bind to.
 * @param {string=} ngChange Angular expression to be executed when input changes due to user
 *    interaction with the input element.
 * @param {string} ngValue Angular expression which sets the value to which the expression should
 *    be set when selected.
 * @param {string} value The value to which the expression should be set when selected.
 * @param {string=} name Property name of the form under which the control is published.
 * @param {string=} aria-label Adds label to radio button for accessibility.
 * Defaults to radio button's text. If no text content is available, a warning will be logged.
 *
 * @usage
 * <hljs lang="html">
 *
 * <md-radio-button value="1" aria-label="Label 1">
 *   Label 1
 * </md-radio-button>
 *
 * <md-radio-button ng-model="color" ng-value="specialValue" aria-label="Green">
 *   Green
 * </md-radio-button>
 *
 * </hljs>
 *
 */
    function mdRadioButtonDirective($mdAria, $mdUtil, $mdTheming) {
      var CHECKED_CSS = 'md-checked';
      return {
        restrict: 'E',
        require: '^mdRadioGroup',
        transclude: true,
        template: '<div class="md-container" md-ink-ripple md-ink-ripple-checkbox>' + '<div class="md-off"></div>' + '<div class="md-on"></div>' + '</div>' + '<div ng-transclude class="md-label"></div>',
        link: link
      };
      function link(scope, element, attr, rgCtrl) {
        var lastChecked;
        $mdTheming(element);
        configureAria(element, scope);
        initialize();
        /**
     *
     */
        function initialize(controller) {
          if (!rgCtrl) {
            throw 'RadioGroupController not found.';
          }
          rgCtrl.add(render);
          attr.$observe('value', render);
          element.on('click', listener).on('$destroy', function () {
            rgCtrl.remove(render);
          });
        }
        /**
     *
     */
        function listener(ev) {
          if (element[0].hasAttribute('disabled'))
            return;
          scope.$apply(function () {
            rgCtrl.setViewValue(attr.value, ev && ev.type);
          });
        }
        /**
     *  Add or remove the `.md-checked` class from the RadioButton (and conditionally its parent).
     *  Update the `aria-activedescendant` attribute.
     */
        function render() {
          var checked = rgCtrl.getViewValue() == attr.value;
          if (checked === lastChecked) {
            return;
          }
          lastChecked = checked;
          element.attr('aria-checked', checked);
          if (checked) {
            markParentAsChecked(true);
            element.addClass(CHECKED_CSS);
            rgCtrl.setActiveDescendant(element.attr('id'));
          } else {
            markParentAsChecked(false);
            element.removeClass(CHECKED_CSS);
          }
          /**
       * If the radioButton is inside a div, then add class so highlighting will work...
       */
          function markParentAsChecked(addClass) {
            if (element.parent()[0].nodeName != 'MD-RADIO-GROUP') {
              element.parent()[!!addClass ? 'addClass' : 'removeClass'](CHECKED_CSS);
            }
          }
        }
        /**
     * Inject ARIA-specific attributes appropriate for each radio button
     */
        function configureAria(element, scope) {
          scope.ariaId = buildAriaID();
          element.attr({
            'id': scope.ariaId,
            'role': 'radio',
            'aria-checked': 'false'
          });
          $mdAria.expectWithText(element, 'aria-label');
          /**
       * Build a unique ID for each radio button that will be used with aria-activedescendant.
       * Preserve existing ID if already specified.
       * @returns {*|string}
       */
          function buildAriaID() {
            return attr.id || 'radio' + '_' + $mdUtil.nextUid();
          }
        }
      }
    }
    mdRadioButtonDirective.$inject = [
      '$mdAria',
      '$mdUtil',
      '$mdTheming'
    ];
  }());
  (function () {
    'use strict';
    /**
 * @ngdoc module
 * @name material.components.progressLinear
 * @description Linear Progress module!
 */
    angular.module('material.components.progressLinear', ['material.core']).directive('mdProgressLinear', MdProgressLinearDirective);
    /**
 * @ngdoc directive
 * @name mdProgressLinear
 * @module material.components.progressLinear
 * @restrict E
 *
 * @description
 * The linear progress directive is used to make loading content
 * in your app as delightful and painless as possible by minimizing
 * the amount of visual change a user sees before they can view
 * and interact with content.
 *
 * Each operation should only be represented by one activity indicator
 * For example: one refresh operation should not display both a
 * refresh bar and an activity circle.
 *
 * For operations where the percentage of the operation completed
 * can be determined, use a determinate indicator. They give users
 * a quick sense of how long an operation will take.
 *
 * For operations where the user is asked to wait a moment while
 * something finishes up, and it’s not necessary to expose what's
 * happening behind the scenes and how long it will take, use an
 * indeterminate indicator.
 *
 * @param {string} md-mode Select from one of four modes: determinate, indeterminate, buffer or query.
 *
 * Note: if the `md-mode` value is set as undefined or specified as 1 of the four (4) valid modes, then `.ng-hide`
 * will be auto-applied as a style to the component.
 *
 * Note: if not configured, the `md-mode="indeterminate"` will be auto injected as an attribute. If `value=""` is also specified, however,
 * then `md-mode="determinate"` would be auto-injected instead.
 * @param {number=} value In determinate and buffer modes, this number represents the percentage of the primary progress bar. Default: 0
 * @param {number=} md-buffer-value In the buffer mode, this number represents the percentage of the secondary progress bar. Default: 0
 *
 * @usage
 * <hljs lang="html">
 * <md-progress-linear md-mode="determinate" value="..."></md-progress-linear>
 *
 * <md-progress-linear md-mode="determinate" ng-value="..."></md-progress-linear>
 *
 * <md-progress-linear md-mode="indeterminate"></md-progress-linear>
 *
 * <md-progress-linear md-mode="buffer" value="..." md-buffer-value="..."></md-progress-linear>
 *
 * <md-progress-linear md-mode="query"></md-progress-linear>
 * </hljs>
 */
    function MdProgressLinearDirective($mdTheming, $mdUtil, $log) {
      var MODE_DETERMINATE = 'determinate', MODE_INDETERMINATE = 'indeterminate', MODE_BUFFER = 'buffer', MODE_QUERY = 'query';
      return {
        restrict: 'E',
        template: '<div class="md-container">' + '<div class="md-dashed"></div>' + '<div class="md-bar md-bar1"></div>' + '<div class="md-bar md-bar2"></div>' + '</div>',
        compile: compile
      };
      function compile(tElement, tAttrs, transclude) {
        tElement.attr('aria-valuemin', 0);
        tElement.attr('aria-valuemax', 100);
        tElement.attr('role', 'progressbar');
        return postLink;
      }
      function postLink(scope, element, attr) {
        $mdTheming(element);
        var lastMode, toVendorCSS = $mdUtil.dom.animator.toCss;
        var bar1 = angular.element(element[0].querySelector('.md-bar1')), bar2 = angular.element(element[0].querySelector('.md-bar2')), container = angular.element(element[0].querySelector('.md-container'));
        element.attr('md-mode', mode());
        validateMode();
        watchAttributes();
        /**
     * Watch the value, md-buffer-value, and md-mode attributes
     */
        function watchAttributes() {
          attr.$observe('value', function (value) {
            var percentValue = clamp(value);
            element.attr('aria-valuenow', percentValue);
            if (mode() != MODE_QUERY)
              animateIndicator(bar2, percentValue);
          });
          attr.$observe('mdBufferValue', function (value) {
            animateIndicator(bar1, clamp(value));
          });
          attr.$observe('mdMode', function (mode) {
            switch (mode) {
            case MODE_QUERY:
            case MODE_BUFFER:
            case MODE_DETERMINATE:
            case MODE_INDETERMINATE:
              container.removeClass('ng-hide' + ' ' + lastMode);
              container.addClass(lastMode = 'md-mode-' + mode);
              break;
            default:
              if (lastMode)
                container.removeClass(lastMode);
              container.addClass('ng-hide');
              lastMode = undefined;
              break;
            }
          });
        }
        /**
     * Auto-defaults the mode to either `determinate` or `indeterminate` mode; if not specified
     */
        function validateMode() {
          if (angular.isUndefined(attr.mdMode)) {
            var hasValue = angular.isDefined(attr.value);
            var mode = hasValue ? MODE_DETERMINATE : MODE_INDETERMINATE;
            var info = 'Auto-adding the missing md-mode=\'{0}\' to the ProgressLinear element';
            $log.debug($mdUtil.supplant(info, [mode]));
            element.attr('md-mode', mode);
            attr['mdMode'] = mode;
          }
        }
        /**
     * Is the md-mode a valid option?
     */
        function mode() {
          var value = (attr.mdMode || '').trim();
          if (value) {
            switch (value) {
            case MODE_DETERMINATE:
            case MODE_INDETERMINATE:
            case MODE_BUFFER:
            case MODE_QUERY:
              break;
            default:
              value = undefined;
              break;
            }
          }
          return value;
        }
        /**
     * Manually set CSS to animate the Determinate indicator based on the specified
     * percentage value (0-100).
     */
        function animateIndicator(target, value) {
          if (!mode())
            return;
          var to = $mdUtil.supplant('translateX({0}%) scale({1},1)', [
              (value - 100) / 2,
              value / 100
            ]);
          var styles = toVendorCSS({ transform: to });
          angular.element(target).css(styles);
        }
      }
      /**
   * Clamps the value to be between 0 and 100.
   * @param {number} value The value to clamp.
   * @returns {number}
   */
      function clamp(value) {
        return Math.max(0, Math.min(value || 0, 100));
      }
    }
    MdProgressLinearDirective.$inject = [
      '$mdTheming',
      '$mdUtil',
      '$log'
    ];
  }());
  (function () {
    'use strict';
    /**
 * @ngdoc module
 * @name material.components.showHide
 */
    // Add additional handlers to ng-show and ng-hide that notify directives
    // contained within that they should recompute their size.
    // These run in addition to Angular's built-in ng-hide and ng-show directives.
    angular.module('material.components.showHide', ['material.core']).directive('ngShow', createDirective('ngShow', true)).directive('ngHide', createDirective('ngHide', false));
    function createDirective(name, targetValue) {
      return [
        '$mdUtil',
        function ($mdUtil) {
          return {
            restrict: 'A',
            multiElement: true,
            link: function ($scope, $element, $attr) {
              var unregister = $scope.$on('$md-resize-enable', function () {
                  unregister();
                  $scope.$watch($attr[name], function (value) {
                    if (!!value === targetValue) {
                      $mdUtil.nextTick(function () {
                        $scope.$broadcast('$md-resize');
                      });
                      $mdUtil.dom.animator.waitTransitionEnd($element).then(function () {
                        $scope.$broadcast('$md-resize');
                      });
                    }
                  });
                });
            }
          };
        }
      ];
    }
  }());
  (function () {
    'use strict';
    /**
 * @ngdoc module
 * @name material.components.select
 */
    /***************************************************

 ### TODO - POST RC1 ###
 - [ ] Abstract placement logic in $mdSelect service to $mdMenu service

 ***************************************************/
    var SELECT_EDGE_MARGIN = 8;
    var selectNextId = 0;
    angular.module('material.components.select', [
      'material.core',
      'material.components.backdrop'
    ]).directive('mdSelect', SelectDirective).directive('mdSelectMenu', SelectMenuDirective).directive('mdOption', OptionDirective).directive('mdOptgroup', OptgroupDirective).provider('$mdSelect', SelectProvider);
    /**
 * @ngdoc directive
 * @name mdSelect
 * @restrict E
 * @module material.components.select
 *
 * @description Displays a select box, bound to an ng-model.
 *
 * @param {expression} ng-model The model!
 * @param {boolean=} multiple Whether it's multiple.
 * @param {expression=} md-on-close Expression to be evaluated when the select is closed.
 * @param {string=} placeholder Placeholder hint text.
 * @param {string=} aria-label Optional label for accessibility. Only necessary if no placeholder or
 * explicit label is present.
 * @param {string=} md-container-class Class list to get applied to the `.md-select-menu-container`
 * element (for custom styling).
 *
 * @usage
 * With a placeholder (label and aria-label are added dynamically)
 * <hljs lang="html">
 *   <md-input-container>
 *     <md-select
 *       ng-model="someModel"
 *       placeholder="Select a state">
 *       <md-option ng-value="opt" ng-repeat="opt in neighborhoods2">{{ opt }}</md-option>
 *     </md-select>
 *   </md-input-container>
 * </hljs>
 *
 * With an explicit label
 * <hljs lang="html">
 *   <md-input-container>
 *     <label>State</label>
 *     <md-select
 *       ng-model="someModel">
 *       <md-option ng-value="opt" ng-repeat="opt in neighborhoods2">{{ opt }}</md-option>
 *     </md-select>
 *   </md-input-container>
 * </hljs>
 *
 * ## Selects and object equality
 * When using a `md-select` to pick from a list of objects, it is important to realize how javascript handles
 * equality. Consider the following example:
 * <hljs lang="js">
 * angular.controller('MyCtrl', function($scope) {
 *   $scope.users = [
 *     { id: 1, name: 'Bob' },
 *     { id: 2, name: 'Alice' },
 *     { id: 3, name: 'Steve' }
 *   ];
 *   $scope.selectedUser = { id: 1, name: 'Bob' };
 * });
 * </hljs>
 * <hljs lang="html">
 * <div ng-controller="MyCtrl">
 *   <md-select ng-model="selectedUser">
 *     <md-option ng-value="user" ng-repeat="user in users">{{ user.name }}</md-option>
 *   </md-select>
 * </div>
 * </hljs>
 *
 * At first one might expect that the select should be populated with "Bob" as the selected user. However,
 * this is not true. To determine whether something is selected, 
 * `ngModelController` is looking at whether `$scope.selectedUser == (any user in $scope.users);`;
 * 
 * Javascript's `==` operator does not check for deep equality (ie. that all properties
 * on the object are the same), but instead whether the objects are *the same object in memory*.
 * In this case, we have two instances of identical objects, but they exist in memory as unique
 * entities. Because of this, the select will have no value populated for a selected user.
 *
 * To get around this, `ngModelController` provides a `track by` option that allows us to specify a different
 * expression which will be used for the equality operator. As such, we can update our `html` to
 * make use of this by specifying the `ng-model-options="{trackBy: '$value.id'}"` on the `md-select`
 * element. This converts our equality expression to be 
 * `$scope.selectedUser.id == (any id in $scope.users.map(function(u) { return u.id; }));`
 * which results in Bob being selected as desired.
 *
 * Working HTML:
 * <hljs lang="html">
 * <div ng-controller="MyCtrl">
 *   <md-select ng-model="selectedUser" ng-model-options="{trackBy: '$value.id'}">
 *     <md-option ng-value="user" ng-repeat="user in users">{{ user.name }}</md-option>
 *   </md-select>
 * </div>
 * </hljs>
 */
    function SelectDirective($mdSelect, $mdUtil, $mdTheming, $mdAria, $compile, $parse) {
      return {
        restrict: 'E',
        require: [
          '^?mdInputContainer',
          'mdSelect',
          'ngModel',
          '?^form'
        ],
        compile: compile,
        controller: function () {
        }  // empty placeholder controller to be initialized in link
      };
      function compile(element, attr) {
        // add the select value that will hold our placeholder or selected option value
        var valueEl = angular.element('<md-select-value><span></span></md-select-value>');
        valueEl.append('<span class="md-select-icon" aria-hidden="true"></span>');
        valueEl.addClass('md-select-value');
        if (!valueEl[0].hasAttribute('id')) {
          valueEl.attr('id', 'select_value_label_' + $mdUtil.nextUid());
        }
        // There's got to be an md-content inside. If there's not one, let's add it.
        if (!element.find('md-content').length) {
          element.append(angular.element('<md-content>').append(element.contents()));
        }
        // Add progress spinner for md-options-loading
        if (attr.mdOnOpen) {
          // Show progress indicator while loading async
          // Use ng-hide for `display:none` so the indicator does not interfere with the options list
          element.find('md-content').prepend(angular.element('<div>' + ' <md-progress-circular md-mode="{{progressMode}}" ng-hide="$$loadingAsyncDone"></md-progress-circular>' + '</div>'));
          // Hide list [of item options] while loading async
          element.find('md-option').attr('ng-show', '$$loadingAsyncDone');
        }
        if (attr.name) {
          var autofillClone = angular.element('<select class="md-visually-hidden">');
          autofillClone.attr({
            'name': '.' + attr.name,
            'ng-model': attr.ngModel,
            'aria-hidden': 'true',
            'tabindex': '-1'
          });
          var opts = element.find('md-option');
          angular.forEach(opts, function (el) {
            var newEl = angular.element('<option>' + el.innerHTML + '</option>');
            if (el.hasAttribute('ng-value'))
              newEl.attr('ng-value', el.getAttribute('ng-value'));
            else if (el.hasAttribute('value'))
              newEl.attr('value', el.getAttribute('value'));
            autofillClone.append(newEl);
          });
          element.parent().append(autofillClone);
        }
        // Use everything that's left inside element.contents() as the contents of the menu
        var multiple = angular.isDefined(attr.multiple) ? 'multiple' : '';
        var selectTemplate = '' + '<div class="md-select-menu-container" aria-hidden="true">' + '<md-select-menu {0}>{1}</md-select-menu>' + '</div>';
        selectTemplate = $mdUtil.supplant(selectTemplate, [
          multiple,
          element.html()
        ]);
        element.empty().append(valueEl);
        element.append(selectTemplate);
        attr.tabindex = attr.tabindex || '0';
        return function postLink(scope, element, attr, ctrls) {
          var untouched = true;
          var isDisabled, ariaLabelBase;
          var containerCtrl = ctrls[0];
          var mdSelectCtrl = ctrls[1];
          var ngModelCtrl = ctrls[2];
          var formCtrl = ctrls[3];
          // grab a reference to the select menu value label
          var valueEl = element.find('md-select-value');
          var isReadonly = angular.isDefined(attr.readonly);
          if (containerCtrl) {
            var isErrorGetter = containerCtrl.isErrorGetter || function () {
                return ngModelCtrl.$invalid && ngModelCtrl.$touched;
              };
            if (containerCtrl.input) {
              throw new Error('<md-input-container> can only have *one* child <input>, <textarea> or <select> element!');
            }
            containerCtrl.input = element;
            if (!containerCtrl.label) {
              $mdAria.expect(element, 'aria-label', element.attr('placeholder'));
            }
            scope.$watch(isErrorGetter, containerCtrl.setInvalid);
          }
          var selectContainer, selectScope, selectMenuCtrl;
          findSelectContainer();
          $mdTheming(element);
          if (attr.name && formCtrl) {
            var selectEl = element.parent()[0].querySelector('select[name=".' + attr.name + '"]');
            $mdUtil.nextTick(function () {
              var controller = angular.element(selectEl).controller('ngModel');
              if (controller) {
                formCtrl.$removeControl(controller);
              }
            });
          }
          if (formCtrl) {
            $mdUtil.nextTick(function () {
              formCtrl.$setPristine();
            });
          }
          var originalRender = ngModelCtrl.$render;
          ngModelCtrl.$render = function () {
            originalRender();
            syncLabelText();
            syncAriaLabel();
            inputCheckValue();
          };
          attr.$observe('placeholder', ngModelCtrl.$render);
          mdSelectCtrl.setLabelText = function (text) {
            mdSelectCtrl.setIsPlaceholder(!text);
            // Use placeholder attribute, otherwise fallback to the md-input-container label
            var tmpPlaceholder = attr.placeholder || (containerCtrl && containerCtrl.label ? containerCtrl.label.text() : '');
            text = text || tmpPlaceholder || '';
            var target = valueEl.children().eq(0);
            target.html(text);
          };
          mdSelectCtrl.setIsPlaceholder = function (isPlaceholder) {
            if (isPlaceholder) {
              valueEl.addClass('md-select-placeholder');
              if (containerCtrl && containerCtrl.label) {
                containerCtrl.label.addClass('md-placeholder');
              }
            } else {
              valueEl.removeClass('md-select-placeholder');
              if (containerCtrl && containerCtrl.label) {
                containerCtrl.label.removeClass('md-placeholder');
              }
            }
          };
          if (!isReadonly) {
            element.on('focus', function (ev) {
              // only set focus on if we don't currently have a selected value. This avoids the "bounce"
              // on the label transition because the focus will immediately switch to the open menu.
              if (containerCtrl && containerCtrl.element.hasClass('md-input-has-value')) {
                containerCtrl.setFocused(true);
              }
            });
            // Wait until postDigest so that we attach after ngModel's 
            // blur listener so we can set untouched.
            $mdUtil.nextTick(function () {
              element.on('blur', function () {
                if (untouched) {
                  untouched = false;
                  ngModelCtrl.$setUntouched();
                }
                if (selectScope.isOpen)
                  return;
                containerCtrl && containerCtrl.setFocused(false);
                inputCheckValue();
              });
            });
          }
          mdSelectCtrl.triggerClose = function () {
            $parse(attr.mdOnClose)(scope);
          };
          scope.$$postDigest(function () {
            initAriaLabel();
            syncLabelText();
            syncAriaLabel();
          });
          function initAriaLabel() {
            var labelText = element.attr('aria-label') || element.attr('placeholder');
            if (!labelText && containerCtrl && containerCtrl.label) {
              labelText = containerCtrl.label.text();
            }
            ariaLabelBase = labelText;
            $mdAria.expect(element, 'aria-label', labelText);
          }
          scope.$watch(selectMenuCtrl.selectedLabels, syncLabelText);
          function syncLabelText() {
            if (selectContainer) {
              selectMenuCtrl = selectMenuCtrl || selectContainer.find('md-select-menu').controller('mdSelectMenu');
              mdSelectCtrl.setLabelText(selectMenuCtrl.selectedLabels());
            }
          }
          function syncAriaLabel() {
            if (!ariaLabelBase)
              return;
            var ariaLabels = selectMenuCtrl.selectedLabels({ mode: 'aria' });
            element.attr('aria-label', ariaLabels.length ? ariaLabelBase + ': ' + ariaLabels : ariaLabelBase);
          }
          var deregisterWatcher;
          attr.$observe('ngMultiple', function (val) {
            if (deregisterWatcher)
              deregisterWatcher();
            var parser = $parse(val);
            deregisterWatcher = scope.$watch(function () {
              return parser(scope);
            }, function (multiple, prevVal) {
              if (multiple === undefined && prevVal === undefined)
                return;
              // assume compiler did a good job
              if (multiple) {
                element.attr('multiple', 'multiple');
              } else {
                element.removeAttr('multiple');
              }
              element.attr('aria-multiselectable', multiple ? 'true' : 'false');
              if (selectContainer) {
                selectMenuCtrl.setMultiple(multiple);
                originalRender = ngModelCtrl.$render;
                ngModelCtrl.$render = function () {
                  originalRender();
                  syncLabelText();
                  syncAriaLabel();
                  inputCheckValue();
                };
                ngModelCtrl.$render();
              }
            });
          });
          attr.$observe('disabled', function (disabled) {
            if (angular.isString(disabled)) {
              disabled = true;
            }
            // Prevent click event being registered twice
            if (isDisabled !== undefined && isDisabled === disabled) {
              return;
            }
            isDisabled = disabled;
            if (disabled) {
              element.attr({
                'tabindex': -1,
                'aria-disabled': 'true'
              });
              element.off('click', openSelect);
              element.off('keydown', handleKeypress);
            } else {
              element.attr({
                'tabindex': attr.tabindex,
                'aria-disabled': 'false'
              });
              element.on('click', openSelect);
              element.on('keydown', handleKeypress);
            }
          });
          if (!attr.disabled && !attr.ngDisabled) {
            element.attr({
              'tabindex': attr.tabindex,
              'aria-disabled': 'false'
            });
            element.on('click', openSelect);
            element.on('keydown', handleKeypress);
          }
          var ariaAttrs = {
              role: 'listbox',
              'aria-expanded': 'false',
              'aria-multiselectable': attr.multiple !== undefined && !attr.ngMultiple ? 'true' : 'false'
            };
          if (!element[0].hasAttribute('id')) {
            ariaAttrs.id = 'select_' + $mdUtil.nextUid();
          }
          var containerId = 'select_container_' + $mdUtil.nextUid();
          selectContainer.attr('id', containerId);
          ariaAttrs['aria-owns'] = containerId;
          element.attr(ariaAttrs);
          scope.$on('$destroy', function () {
            $mdSelect.destroy().finally(function () {
              if (containerCtrl) {
                containerCtrl.setFocused(false);
                containerCtrl.setHasValue(false);
                containerCtrl.input = null;
              }
              ngModelCtrl.$setTouched();
            });
          });
          function inputCheckValue() {
            // The select counts as having a value if one or more options are selected,
            // or if the input's validity state says it has bad input (eg string in a number input)
            containerCtrl && containerCtrl.setHasValue(selectMenuCtrl.selectedLabels().length > 0 || (element[0].validity || {}).badInput);
          }
          function findSelectContainer() {
            selectContainer = angular.element(element[0].querySelector('.md-select-menu-container'));
            selectScope = scope;
            if (element.attr('md-container-class')) {
              var value = selectContainer[0].getAttribute('class') + ' ' + element.attr('md-container-class');
              selectContainer[0].setAttribute('class', value);
            }
            selectMenuCtrl = selectContainer.find('md-select-menu').controller('mdSelectMenu');
            selectMenuCtrl.init(ngModelCtrl, attr.ngModel);
            element.on('$destroy', function () {
              selectContainer.remove();
            });
          }
          function handleKeypress(e) {
            var allowedCodes = [
                32,
                13,
                38,
                40
              ];
            if (allowedCodes.indexOf(e.keyCode) != -1) {
              // prevent page scrolling on interaction
              e.preventDefault();
              openSelect(e);
            } else {
              if (e.keyCode <= 90 && e.keyCode >= 31) {
                e.preventDefault();
                var node = selectMenuCtrl.optNodeForKeyboardSearch(e);
                if (!node)
                  return;
                var optionCtrl = angular.element(node).controller('mdOption');
                if (!selectMenuCtrl.isMultiple) {
                  selectMenuCtrl.deselect(Object.keys(selectMenuCtrl.selected)[0]);
                }
                selectMenuCtrl.select(optionCtrl.hashKey, optionCtrl.value);
                selectMenuCtrl.refreshViewValue();
              }
            }
          }
          function openSelect() {
            selectScope.isOpen = true;
            element.attr('aria-expanded', 'true');
            $mdSelect.show({
              scope: selectScope,
              preserveScope: true,
              skipCompile: true,
              element: selectContainer,
              target: element[0],
              selectCtrl: mdSelectCtrl,
              preserveElement: true,
              hasBackdrop: true,
              loadingAsync: attr.mdOnOpen ? scope.$eval(attr.mdOnOpen) || true : false
            }).finally(function () {
              selectScope.isOpen = false;
              element.focus();
              element.attr('aria-expanded', 'false');
              ngModelCtrl.$setTouched();
            });
          }
        };
      }
    }
    SelectDirective.$inject = [
      '$mdSelect',
      '$mdUtil',
      '$mdTheming',
      '$mdAria',
      '$compile',
      '$parse'
    ];
    function SelectMenuDirective($parse, $mdUtil, $mdTheming) {
      SelectMenuController.$inject = [
        '$scope',
        '$attrs',
        '$element'
      ];
      return {
        restrict: 'E',
        require: ['mdSelectMenu'],
        scope: true,
        controller: SelectMenuController,
        link: { pre: preLink }
      };
      // We use preLink instead of postLink to ensure that the select is initialized before
      // its child options run postLink.
      function preLink(scope, element, attr, ctrls) {
        var selectCtrl = ctrls[0];
        $mdTheming(element);
        element.on('click', clickListener);
        element.on('keypress', keyListener);
        function keyListener(e) {
          if (e.keyCode == 13 || e.keyCode == 32) {
            clickListener(e);
          }
        }
        function clickListener(ev) {
          var option = $mdUtil.getClosest(ev.target, 'md-option');
          var optionCtrl = option && angular.element(option).data('$mdOptionController');
          if (!option || !optionCtrl)
            return;
          if (option.hasAttribute('disabled')) {
            ev.stopImmediatePropagation();
            return false;
          }
          var optionHashKey = selectCtrl.hashGetter(optionCtrl.value);
          var isSelected = angular.isDefined(selectCtrl.selected[optionHashKey]);
          scope.$apply(function () {
            if (selectCtrl.isMultiple) {
              if (isSelected) {
                selectCtrl.deselect(optionHashKey);
              } else {
                selectCtrl.select(optionHashKey, optionCtrl.value);
              }
            } else {
              if (!isSelected) {
                selectCtrl.deselect(Object.keys(selectCtrl.selected)[0]);
                selectCtrl.select(optionHashKey, optionCtrl.value);
              }
            }
            selectCtrl.refreshViewValue();
          });
        }
      }
      function SelectMenuController($scope, $attrs, $element) {
        var self = this;
        self.isMultiple = angular.isDefined($attrs.multiple);
        // selected is an object with keys matching all of the selected options' hashed values
        self.selected = {};
        // options is an object with keys matching every option's hash value,
        // and values matching every option's controller.
        self.options = {};
        $scope.$watchCollection(function () {
          return self.options;
        }, function () {
          self.ngModel.$render();
        });
        var deregisterCollectionWatch;
        var defaultIsEmpty;
        self.setMultiple = function (isMultiple) {
          var ngModel = self.ngModel;
          defaultIsEmpty = defaultIsEmpty || ngModel.$isEmpty;
          self.isMultiple = isMultiple;
          if (deregisterCollectionWatch)
            deregisterCollectionWatch();
          if (self.isMultiple) {
            ngModel.$validators['md-multiple'] = validateArray;
            ngModel.$render = renderMultiple;
            // watchCollection on the model because by default ngModel only watches the model's
            // reference. This allowed the developer to also push and pop from their array.
            $scope.$watchCollection(self.modelBinding, function (value) {
              if (validateArray(value))
                renderMultiple(value);
              self.ngModel.$setPristine();
            });
            ngModel.$isEmpty = function (value) {
              return !value || value.length === 0;
            };
          } else {
            delete ngModel.$validators['md-multiple'];
            ngModel.$render = renderSingular;
          }
          function validateArray(modelValue, viewValue) {
            // If a value is truthy but not an array, reject it.
            // If value is undefined/falsy, accept that it's an empty array.
            return angular.isArray(modelValue || viewValue || []);
          }
        };
        var searchStr = '';
        var clearSearchTimeout, optNodes, optText;
        var CLEAR_SEARCH_AFTER = 300;
        self.optNodeForKeyboardSearch = function (e) {
          clearSearchTimeout && clearTimeout(clearSearchTimeout);
          clearSearchTimeout = setTimeout(function () {
            clearSearchTimeout = undefined;
            searchStr = '';
            optText = undefined;
            optNodes = undefined;
          }, CLEAR_SEARCH_AFTER);
          searchStr += String.fromCharCode(e.keyCode);
          var search = new RegExp('^' + searchStr, 'i');
          if (!optNodes) {
            optNodes = $element.find('md-option');
            optText = new Array(optNodes.length);
            angular.forEach(optNodes, function (el, i) {
              optText[i] = el.textContent.trim();
            });
          }
          for (var i = 0; i < optText.length; ++i) {
            if (search.test(optText[i])) {
              return optNodes[i];
            }
          }
        };
        self.init = function (ngModel, binding) {
          self.ngModel = ngModel;
          self.modelBinding = binding;
          // Allow users to provide `ng-model="foo" ng-model-options="{trackBy: 'foo.id'}"` so
          // that we can properly compare objects set on the model to the available options
          if (ngModel.$options && ngModel.$options.trackBy) {
            var trackByLocals = {};
            var trackByParsed = $parse(ngModel.$options.trackBy);
            self.hashGetter = function (value, valueScope) {
              trackByLocals.$value = value;
              return trackByParsed(valueScope || $scope, trackByLocals);
            };  // If the user doesn't provide a trackBy, we automatically generate an id for every
                // value passed in
          } else {
            self.hashGetter = function getHashValue(value) {
              if (angular.isObject(value)) {
                return 'object_' + (value.$$mdSelectId || (value.$$mdSelectId = ++selectNextId));
              }
              return value;
            };
          }
          self.setMultiple(self.isMultiple);
        };
        self.selectedLabels = function (opts) {
          opts = opts || {};
          var mode = opts.mode || 'html';
          var selectedOptionEls = $mdUtil.nodesToArray($element[0].querySelectorAll('md-option[selected]'));
          if (selectedOptionEls.length) {
            var mapFn;
            if (mode == 'html') {
              mapFn = function (el) {
                return el.innerHTML;
              };
            } else if (mode == 'aria') {
              mapFn = function (el) {
                return el.hasAttribute('aria-label') ? el.getAttribute('aria-label') : el.textContent;
              };
            }
            return selectedOptionEls.map(mapFn).join(', ');
          } else {
            return '';
          }
        };
        self.select = function (hashKey, hashedValue) {
          var option = self.options[hashKey];
          option && option.setSelected(true);
          self.selected[hashKey] = hashedValue;
        };
        self.deselect = function (hashKey) {
          var option = self.options[hashKey];
          option && option.setSelected(false);
          delete self.selected[hashKey];
        };
        self.addOption = function (hashKey, optionCtrl) {
          if (angular.isDefined(self.options[hashKey])) {
            throw new Error('Duplicate md-option values are not allowed in a select. ' + 'Duplicate value "' + optionCtrl.value + '" found.');
          }
          self.options[hashKey] = optionCtrl;
          // If this option's value was already in our ngModel, go ahead and select it.
          if (angular.isDefined(self.selected[hashKey])) {
            self.select(hashKey, optionCtrl.value);
            self.refreshViewValue();
          }
        };
        self.removeOption = function (hashKey) {
          delete self.options[hashKey];  // Don't deselect an option when it's removed - the user's ngModel should be allowed
                                         // to have values that do not match a currently available option.
        };
        self.refreshViewValue = function () {
          var values = [];
          var option;
          for (var hashKey in self.selected) {
            // If this hashKey has an associated option, push that option's value to the model.
            if (option = self.options[hashKey]) {
              values.push(option.value);
            } else {
              // Otherwise, the given hashKey has no associated option, and we got it
              // from an ngModel value at an earlier time. Push the unhashed value of
              // this hashKey to the model.
              // This allows the developer to put a value in the model that doesn't yet have
              // an associated option.
              values.push(self.selected[hashKey]);
            }
          }
          var usingTrackBy = self.ngModel.$options && self.ngModel.$options.trackBy;
          var newVal = self.isMultiple ? values : values[0];
          var prevVal = self.ngModel.$modelValue;
          if (usingTrackBy ? !angular.equals(prevVal, newVal) : prevVal != newVal) {
            self.ngModel.$setViewValue(newVal);
            self.ngModel.$render();
          }
        };
        function renderMultiple() {
          var newSelectedValues = self.ngModel.$modelValue || self.ngModel.$viewValue || [];
          if (!angular.isArray(newSelectedValues))
            return;
          var oldSelected = Object.keys(self.selected);
          var newSelectedHashes = newSelectedValues.map(self.hashGetter);
          var deselected = oldSelected.filter(function (hash) {
              return newSelectedHashes.indexOf(hash) === -1;
            });
          deselected.forEach(self.deselect);
          newSelectedHashes.forEach(function (hashKey, i) {
            self.select(hashKey, newSelectedValues[i]);
          });
        }
        function renderSingular() {
          var value = self.ngModel.$viewValue || self.ngModel.$modelValue;
          Object.keys(self.selected).forEach(self.deselect);
          self.select(self.hashGetter(value), value);
        }
      }
    }
    SelectMenuDirective.$inject = [
      '$parse',
      '$mdUtil',
      '$mdTheming'
    ];
    function OptionDirective($mdButtonInkRipple, $mdUtil) {
      OptionController.$inject = ['$element'];
      return {
        restrict: 'E',
        require: [
          'mdOption',
          '^^mdSelectMenu'
        ],
        controller: OptionController,
        compile: compile
      };
      function compile(element, attr) {
        // Manual transclusion to avoid the extra inner <span> that ng-transclude generates
        element.append(angular.element('<div class="md-text">').append(element.contents()));
        element.attr('tabindex', attr.tabindex || '0');
        return postLink;
      }
      function postLink(scope, element, attr, ctrls) {
        var optionCtrl = ctrls[0];
        var selectCtrl = ctrls[1];
        if (angular.isDefined(attr.ngValue)) {
          scope.$watch(attr.ngValue, setOptionValue);
        } else if (angular.isDefined(attr.value)) {
          setOptionValue(attr.value);
        } else {
          scope.$watch(function () {
            return element.text();
          }, setOptionValue);
        }
        attr.$observe('disabled', function (disabled) {
          if (disabled) {
            element.attr('tabindex', '-1');
          } else {
            element.attr('tabindex', '0');
          }
        });
        scope.$$postDigest(function () {
          attr.$observe('selected', function (selected) {
            if (!angular.isDefined(selected))
              return;
            if (typeof selected == 'string')
              selected = true;
            if (selected) {
              if (!selectCtrl.isMultiple) {
                selectCtrl.deselect(Object.keys(selectCtrl.selected)[0]);
              }
              selectCtrl.select(optionCtrl.hashKey, optionCtrl.value);
            } else {
              selectCtrl.deselect(optionCtrl.hashKey);
            }
            selectCtrl.refreshViewValue();
          });
        });
        $mdButtonInkRipple.attach(scope, element);
        configureAria();
        function setOptionValue(newValue, oldValue, prevAttempt) {
          if (!selectCtrl.hashGetter) {
            if (!prevAttempt) {
              scope.$$postDigest(function () {
                setOptionValue(newValue, oldValue, true);
              });
            }
            return;
          }
          var oldHashKey = selectCtrl.hashGetter(oldValue, scope);
          var newHashKey = selectCtrl.hashGetter(newValue, scope);
          optionCtrl.hashKey = newHashKey;
          optionCtrl.value = newValue;
          selectCtrl.removeOption(oldHashKey, optionCtrl);
          selectCtrl.addOption(newHashKey, optionCtrl);
        }
        scope.$on('$destroy', function () {
          selectCtrl.removeOption(optionCtrl.hashKey, optionCtrl);
        });
        function configureAria() {
          var ariaAttrs = {
              'role': 'option',
              'aria-selected': 'false'
            };
          if (!element[0].hasAttribute('id')) {
            ariaAttrs.id = 'select_option_' + $mdUtil.nextUid();
          }
          element.attr(ariaAttrs);
        }
      }
      function OptionController($element) {
        this.selected = false;
        this.setSelected = function (isSelected) {
          if (isSelected && !this.selected) {
            $element.attr({
              'selected': 'selected',
              'aria-selected': 'true'
            });
          } else if (!isSelected && this.selected) {
            $element.removeAttr('selected');
            $element.attr('aria-selected', 'false');
          }
          this.selected = isSelected;
        };
      }
    }
    OptionDirective.$inject = [
      '$mdButtonInkRipple',
      '$mdUtil'
    ];
    function OptgroupDirective() {
      return {
        restrict: 'E',
        compile: compile
      };
      function compile(el, attrs) {
        var labelElement = el.find('label');
        if (!labelElement.length) {
          labelElement = angular.element('<label>');
          el.prepend(labelElement);
        }
        labelElement.addClass('md-container-ignore');
        if (attrs.label)
          labelElement.text(attrs.label);
      }
    }
    function SelectProvider($$interimElementProvider) {
      selectDefaultOptions.$inject = [
        '$mdSelect',
        '$mdConstant',
        '$mdUtil',
        '$window',
        '$q',
        '$$rAF',
        '$animateCss',
        '$animate',
        '$document'
      ];
      return $$interimElementProvider('$mdSelect').setDefaults({
        methods: ['target'],
        options: selectDefaultOptions
      });
      /* @ngInject */
      function selectDefaultOptions($mdSelect, $mdConstant, $mdUtil, $window, $q, $$rAF, $animateCss, $animate, $document) {
        var ERRROR_TARGET_EXPECTED = '$mdSelect.show() expected a target element in options.target but got \'{0}\'!';
        var animator = $mdUtil.dom.animator;
        return {
          parent: 'body',
          themable: true,
          onShow: onShow,
          onRemove: onRemove,
          hasBackdrop: true,
          disableParentScroll: true
        };
        /**
     * Interim-element onRemove logic....
     */
        function onRemove(scope, element, opts) {
          opts = opts || {};
          opts.cleanupInteraction();
          opts.cleanupResizing();
          opts.hideBackdrop();
          // For navigation $destroy events, do a quick, non-animated removal,
          // but for normal closes (from clicks, etc) animate the removal
          return opts.$destroy === true ? cleanElement() : animateRemoval().then(cleanElement);
          /**
       * For normal closes (eg clicks), animate the removal.
       * For forced closes (like $destroy events from navigation),
       * skip the animations
       */
          function animateRemoval() {
            return $animateCss(element, { addClass: 'md-leave' }).start();
          }
          /**
       * Restore the element to a closed state
       */
          function cleanElement() {
            element.removeClass('md-active');
            element.attr('aria-hidden', 'true');
            element[0].style.display = 'none';
            announceClosed(opts);
            if (!opts.$destroy && opts.restoreFocus) {
              opts.target.focus();
            }
          }
        }
        /**
     * Interim-element onShow logic....
     */
        function onShow(scope, element, opts) {
          watchAsyncLoad();
          sanitizeAndConfigure(scope, opts);
          opts.hideBackdrop = showBackdrop(scope, element, opts);
          return showDropDown(scope, element, opts).then(function (response) {
            element.attr('aria-hidden', 'false');
            opts.alreadyOpen = true;
            opts.cleanupInteraction = activateInteraction();
            opts.cleanupResizing = activateResizing();
            return response;
          }, opts.hideBackdrop);
          // ************************************
          // Closure Functions
          // ************************************
          /**
       *  Attach the select DOM element(s) and animate to the correct positions
       *  and scalings...
       */
          function showDropDown(scope, element, opts) {
            opts.parent.append(element);
            return $q(function (resolve, reject) {
              try {
                $animateCss(element, {
                  removeClass: 'md-leave',
                  duration: 0
                }).start().then(positionAndFocusMenu).then(resolve);
              } catch (e) {
                reject(e);
              }
            });
          }
          /**
       * Initialize container and dropDown menu positions/scale, then animate
       * to show... and autoFocus.
       */
          function positionAndFocusMenu() {
            return $q(function (resolve) {
              if (opts.isRemoved)
                return $q.reject(false);
              var info = calculateMenuPositions(scope, element, opts);
              info.container.element.css(animator.toCss(info.container.styles));
              info.dropDown.element.css(animator.toCss(info.dropDown.styles));
              $$rAF(function () {
                element.addClass('md-active');
                info.dropDown.element.css(animator.toCss({ transform: '' }));
                autoFocus(opts.focusedNode);
                resolve();
              });
            });
          }
          /**
       * Show modal backdrop element...
       */
          function showBackdrop(scope, element, options) {
            // If we are not within a dialog...
            if (options.disableParentScroll && !$mdUtil.getClosest(options.target, 'MD-DIALOG')) {
              // !! DO this before creating the backdrop; since disableScrollAround()
              //    configures the scroll offset; which is used by mdBackDrop postLink()
              options.restoreScroll = $mdUtil.disableScrollAround(options.element, options.parent);
            } else {
              options.disableParentScroll = false;
            }
            if (options.hasBackdrop) {
              // Override duration to immediately show invisible backdrop
              options.backdrop = $mdUtil.createBackdrop(scope, 'md-select-backdrop md-click-catcher');
              $animate.enter(options.backdrop, $document[0].body, null, { duration: 0 });
            }
            /**
         * Hide modal backdrop element...
         */
            return function hideBackdrop() {
              if (options.backdrop)
                options.backdrop.remove();
              if (options.disableParentScroll)
                options.restoreScroll();
              delete options.restoreScroll;
            };
          }
          /**
       *
       */
          function autoFocus(focusedNode) {
            if (focusedNode && !focusedNode.hasAttribute('disabled')) {
              focusedNode.focus();
            }
          }
          /**
       * Check for valid opts and set some sane defaults
       */
          function sanitizeAndConfigure(scope, options) {
            var selectEl = element.find('md-select-menu');
            if (!options.target) {
              throw new Error($mdUtil.supplant(ERRROR_TARGET_EXPECTED, [options.target]));
            }
            angular.extend(options, {
              isRemoved: false,
              target: angular.element(options.target),
              parent: angular.element(options.parent),
              selectEl: selectEl,
              contentEl: element.find('md-content'),
              optionNodes: selectEl[0].getElementsByTagName('md-option')
            });
          }
          /**
       * Configure various resize listeners for screen changes
       */
          function activateResizing() {
            var debouncedOnResize = function (scope, target, options) {
                return function () {
                  if (options.isRemoved)
                    return;
                  var updates = calculateMenuPositions(scope, target, options);
                  var container = updates.container;
                  var dropDown = updates.dropDown;
                  container.element.css(animator.toCss(container.styles));
                  dropDown.element.css(animator.toCss(dropDown.styles));
                };
              }(scope, element, opts);
            var window = angular.element($window);
            window.on('resize', debouncedOnResize);
            window.on('orientationchange', debouncedOnResize);
            // Publish deactivation closure...
            return function deactivateResizing() {
              // Disable resizing handlers
              window.off('resize', debouncedOnResize);
              window.off('orientationchange', debouncedOnResize);
            };
          }
          /**
       *  If asynchronously loading, watch and update internal
       *  '$$loadingAsyncDone' flag
       */
          function watchAsyncLoad() {
            if (opts.loadingAsync && !opts.isRemoved) {
              scope.$$loadingAsyncDone = false;
              scope.progressMode = 'indeterminate';
              $q.when(opts.loadingAsync).then(function () {
                scope.$$loadingAsyncDone = true;
                scope.progressMode = '';
                delete opts.loadingAsync;
              }).then(function () {
                $$rAF(positionAndFocusMenu);
              });
            }
          }
          /**
       *
       */
          function activateInteraction() {
            if (opts.isRemoved)
              return;
            var dropDown = opts.selectEl;
            var selectCtrl = dropDown.controller('mdSelectMenu') || {};
            element.addClass('md-clickable');
            // Close on backdrop click
            opts.backdrop && opts.backdrop.on('click', onBackdropClick);
            // Escape to close
            // Cycling of options, and closing on enter
            dropDown.on('keydown', onMenuKeyDown);
            dropDown.on('click', checkCloseMenu);
            return function cleanupInteraction() {
              opts.backdrop && opts.backdrop.off('click', onBackdropClick);
              dropDown.off('keydown', onMenuKeyDown);
              dropDown.off('click', checkCloseMenu);
              element.removeClass('md-clickable');
              opts.isRemoved = true;
            };
            // ************************************
            // Closure Functions
            // ************************************
            function onBackdropClick(e) {
              e.preventDefault();
              e.stopPropagation();
              opts.restoreFocus = false;
              $mdUtil.nextTick($mdSelect.hide, true);
            }
            function onMenuKeyDown(ev) {
              var keyCodes = $mdConstant.KEY_CODE;
              ev.preventDefault();
              ev.stopPropagation();
              switch (ev.keyCode) {
              case keyCodes.UP_ARROW:
                return focusPrevOption();
              case keyCodes.DOWN_ARROW:
                return focusNextOption();
              case keyCodes.SPACE:
              case keyCodes.ENTER:
                var option = $mdUtil.getClosest(ev.target, 'md-option');
                if (option) {
                  dropDown.triggerHandler({
                    type: 'click',
                    target: option
                  });
                  ev.preventDefault();
                }
                checkCloseMenu(ev);
                break;
              case keyCodes.TAB:
              case keyCodes.ESCAPE:
                ev.stopPropagation();
                ev.preventDefault();
                opts.restoreFocus = true;
                $mdUtil.nextTick($mdSelect.hide, true);
                break;
              default:
                if (ev.keyCode >= 31 && ev.keyCode <= 90) {
                  var optNode = dropDown.controller('mdSelectMenu').optNodeForKeyboardSearch(ev);
                  opts.focusedNode = optNode || opts.focusedNode;
                  optNode && optNode.focus();
                }
              }
            }
            function focusOption(direction) {
              var optionsArray = $mdUtil.nodesToArray(opts.optionNodes);
              var index = optionsArray.indexOf(opts.focusedNode);
              var newOption;
              do {
                if (index === -1) {
                  // We lost the previously focused element, reset to first option
                  index = 0;
                } else if (direction === 'next' && index < optionsArray.length - 1) {
                  index++;
                } else if (direction === 'prev' && index > 0) {
                  index--;
                }
                newOption = optionsArray[index];
                if (newOption.hasAttribute('disabled'))
                  newOption = undefined;
              } while (!newOption && index < optionsArray.length - 1 && index > 0);
              newOption && newOption.focus();
              opts.focusedNode = newOption;
            }
            function focusNextOption() {
              focusOption('next');
            }
            function focusPrevOption() {
              focusOption('prev');
            }
            function checkCloseMenu(ev) {
              if (ev && ev.type == 'click' && ev.currentTarget != dropDown[0])
                return;
              if (mouseOnScrollbar())
                return;
              var option = $mdUtil.getClosest(ev.target, 'md-option');
              if (option && option.hasAttribute && !option.hasAttribute('disabled')) {
                ev.preventDefault();
                ev.stopPropagation();
                if (!selectCtrl.isMultiple) {
                  opts.restoreFocus = true;
                  $mdUtil.nextTick(function () {
                    $mdSelect.hide(selectCtrl.ngModel.$viewValue);
                  }, true);
                }
              }
              /**
           * check if the mouseup event was on a scrollbar
           */
              function mouseOnScrollbar() {
                var clickOnScrollbar = false;
                if (ev && ev.currentTarget.children.length > 0) {
                  var child = ev.currentTarget.children[0];
                  var hasScrollbar = child.scrollHeight > child.clientHeight;
                  if (hasScrollbar && child.children.length > 0) {
                    var relPosX = ev.pageX - ev.currentTarget.getBoundingClientRect().left;
                    if (relPosX > child.querySelector('md-option').offsetWidth)
                      clickOnScrollbar = true;
                  }
                }
                return clickOnScrollbar;
              }
            }
          }
        }
        /**
     * To notify listeners that the Select menu has closed,
     * trigger the [optional] user-defined expression
     */
        function announceClosed(opts) {
          var mdSelect = opts.selectCtrl;
          if (mdSelect) {
            var menuController = opts.selectEl.controller('mdSelectMenu');
            mdSelect.setLabelText(menuController.selectedLabels());
            mdSelect.triggerClose();
          }
        }
        /**
     * Calculate the
     */
        function calculateMenuPositions(scope, element, opts) {
          var containerNode = element[0], targetNode = opts.target[0].children[0],
            // target the label
            parentNode = $document[0].body, selectNode = opts.selectEl[0], contentNode = opts.contentEl[0], parentRect = parentNode.getBoundingClientRect(), targetRect = targetNode.getBoundingClientRect(), shouldOpenAroundTarget = false, bounds = {
              left: parentRect.left + SELECT_EDGE_MARGIN,
              top: SELECT_EDGE_MARGIN,
              bottom: parentRect.height - SELECT_EDGE_MARGIN,
              right: parentRect.width - SELECT_EDGE_MARGIN - ($mdUtil.floatingScrollbars() ? 16 : 0)
            }, spaceAvailable = {
              top: targetRect.top - bounds.top,
              left: targetRect.left - bounds.left,
              right: bounds.right - (targetRect.left + targetRect.width),
              bottom: bounds.bottom - (targetRect.top + targetRect.height)
            }, maxWidth = parentRect.width - SELECT_EDGE_MARGIN * 2, isScrollable = contentNode.scrollHeight > contentNode.offsetHeight, selectedNode = selectNode.querySelector('md-option[selected]'), optionNodes = selectNode.getElementsByTagName('md-option'), optgroupNodes = selectNode.getElementsByTagName('md-optgroup');
          var loading = isPromiseLike(opts.loadingAsync);
          var centeredNode;
          if (!loading) {
            // If a selected node, center around that
            if (selectedNode) {
              centeredNode = selectedNode;  // If there are option groups, center around the first option group
            } else if (optgroupNodes.length) {
              centeredNode = optgroupNodes[0];  // Otherwise - if we are not loading async - center around the first optionNode
            } else if (optionNodes.length) {
              centeredNode = optionNodes[0];  // In case there are no options, center on whatever's in there... (eg progress indicator)
            } else {
              centeredNode = contentNode.firstElementChild || contentNode;
            }
          } else {
            // If loading, center on progress indicator
            centeredNode = contentNode.firstElementChild || contentNode;
          }
          if (contentNode.offsetWidth > maxWidth) {
            contentNode.style['max-width'] = maxWidth + 'px';
          } else {
            contentNode.style.maxWidth = null;
          }
          if (shouldOpenAroundTarget) {
            contentNode.style['min-width'] = targetRect.width + 'px';
          }
          // Remove padding before we compute the position of the menu
          if (isScrollable) {
            selectNode.classList.add('md-overflow');
          }
          var focusedNode = centeredNode;
          if ((focusedNode.tagName || '').toUpperCase() === 'MD-OPTGROUP') {
            focusedNode = optionNodes[0] || contentNode.firstElementChild || contentNode;
            centeredNode = focusedNode;
          }
          // Cache for autoFocus()
          opts.focusedNode = focusedNode;
          // Get the selectMenuRect *after* max-width is possibly set above
          containerNode.style.display = 'block';
          var selectMenuRect = selectNode.getBoundingClientRect();
          var centeredRect = getOffsetRect(centeredNode);
          if (centeredNode) {
            var centeredStyle = $window.getComputedStyle(centeredNode);
            centeredRect.paddingLeft = parseInt(centeredStyle.paddingLeft, 10) || 0;
            centeredRect.paddingRight = parseInt(centeredStyle.paddingRight, 10) || 0;
          }
          if (isScrollable) {
            var scrollBuffer = contentNode.offsetHeight / 2;
            contentNode.scrollTop = centeredRect.top + centeredRect.height / 2 - scrollBuffer;
            if (spaceAvailable.top < scrollBuffer) {
              contentNode.scrollTop = Math.min(centeredRect.top, contentNode.scrollTop + scrollBuffer - spaceAvailable.top);
            } else if (spaceAvailable.bottom < scrollBuffer) {
              contentNode.scrollTop = Math.max(centeredRect.top + centeredRect.height - selectMenuRect.height, contentNode.scrollTop - scrollBuffer + spaceAvailable.bottom);
            }
          }
          var left, top, transformOrigin, minWidth;
          if (shouldOpenAroundTarget) {
            left = targetRect.left;
            top = targetRect.top + targetRect.height;
            transformOrigin = '50% 0';
            if (top + selectMenuRect.height > bounds.bottom) {
              top = targetRect.top - selectMenuRect.height;
              transformOrigin = '50% 100%';
            }
          } else {
            left = targetRect.left + centeredRect.left - centeredRect.paddingLeft + 2;
            top = Math.floor(targetRect.top + targetRect.height / 2 - centeredRect.height / 2 - centeredRect.top + contentNode.scrollTop) + 4;
            transformOrigin = centeredRect.left + targetRect.width / 2 + 'px ' + (centeredRect.top + centeredRect.height / 2 - contentNode.scrollTop) + 'px 0px';
            minWidth = Math.min(targetRect.width + centeredRect.paddingLeft + centeredRect.paddingRight, maxWidth);
          }
          // Keep left and top within the window
          var containerRect = containerNode.getBoundingClientRect();
          var scaleX = Math.round(100 * Math.min(targetRect.width / selectMenuRect.width, 1)) / 100;
          var scaleY = Math.round(100 * Math.min(targetRect.height / selectMenuRect.height, 1)) / 100;
          return {
            container: {
              element: angular.element(containerNode),
              styles: {
                left: Math.floor(clamp(bounds.left, left, bounds.right - containerRect.width)),
                top: Math.floor(clamp(bounds.top, top, bounds.bottom - containerRect.height)),
                'min-width': minWidth
              }
            },
            dropDown: {
              element: angular.element(selectNode),
              styles: {
                transformOrigin: transformOrigin,
                transform: !opts.alreadyOpen ? $mdUtil.supplant('scale({0},{1})', [
                  scaleX,
                  scaleY
                ]) : ''
              }
            }
          };
        }
      }
      function isPromiseLike(obj) {
        return obj && angular.isFunction(obj.then);
      }
      function clamp(min, n, max) {
        return Math.max(min, Math.min(n, max));
      }
      function getOffsetRect(node) {
        return node ? {
          left: node.offsetLeft,
          top: node.offsetTop,
          width: node.offsetWidth,
          height: node.offsetHeight
        } : {
          left: 0,
          top: 0,
          width: 0,
          height: 0
        };
      }
    }
    SelectProvider.$inject = ['$$interimElementProvider'];
  }());
  (function () {
    'use strict';
    /**
   * @ngdoc module
   * @name material.components.slider
   */
    angular.module('material.components.slider', ['material.core']).directive('mdSlider', SliderDirective);
    /**
 * @ngdoc directive
 * @name mdSlider
 * @module material.components.slider
 * @restrict E
 * @description
 * The `<md-slider>` component allows the user to choose from a range of
 * values.
 *
 * As per the [material design spec](http://www.google.com/design/spec/style/color.html#color-ui-color-application)
 * the slider is in the accent color by default. The primary color palette may be used with
 * the `md-primary` class.
 *
 * It has two modes: 'normal' mode, where the user slides between a wide range
 * of values, and 'discrete' mode, where the user slides between only a few
 * select values.
 *
 * To enable discrete mode, add the `md-discrete` attribute to a slider,
 * and use the `step` attribute to change the distance between
 * values the user is allowed to pick.
 *
 * @usage
 * <h4>Normal Mode</h4>
 * <hljs lang="html">
 * <md-slider ng-model="myValue" min="5" max="500">
 * </md-slider>
 * </hljs>
 * <h4>Discrete Mode</h4>
 * <hljs lang="html">
 * <md-slider md-discrete ng-model="myDiscreteValue" step="10" min="10" max="130">
 * </md-slider>
 * </hljs>
 *
 * @param {boolean=} md-discrete Whether to enable discrete mode.
 * @param {number=} step The distance between values the user is allowed to pick. Default 1.
 * @param {number=} min The minimum value the user is allowed to pick. Default 0.
 * @param {number=} max The maximum value the user is allowed to pick. Default 100.
 */
    function SliderDirective($$rAF, $window, $mdAria, $mdUtil, $mdConstant, $mdTheming, $mdGesture, $parse, $log) {
      return {
        scope: {},
        require: '?ngModel',
        template: '<div class="md-slider-wrapper">' + '<div class="md-track-container">' + '<div class="md-track"></div>' + '<div class="md-track md-track-fill"></div>' + '<div class="md-track-ticks"></div>' + '</div>' + '<div class="md-thumb-container">' + '<div class="md-thumb"></div>' + '<div class="md-focus-thumb"></div>' + '<div class="md-focus-ring"></div>' + '<div class="md-sign">' + '<span class="md-thumb-text"></span>' + '</div>' + '<div class="md-disabled-thumb"></div>' + '</div>' + '</div>',
        compile: compile
      };
      // **********************************************************
      // Private Methods
      // **********************************************************
      function compile(tElement, tAttrs) {
        tElement.attr({
          tabIndex: 0,
          role: 'slider'
        });
        $mdAria.expect(tElement, 'aria-label');
        return postLink;
      }
      function postLink(scope, element, attr, ngModelCtrl) {
        $mdTheming(element);
        ngModelCtrl = ngModelCtrl || {
          $setViewValue: function (val) {
            this.$viewValue = val;
            this.$viewChangeListeners.forEach(function (cb) {
              cb();
            });
          },
          $parsers: [],
          $formatters: [],
          $viewChangeListeners: []
        };
        var isDisabledGetter = angular.noop;
        if (attr.disabled != null) {
          isDisabledGetter = function () {
            return true;
          };
        } else if (attr.ngDisabled) {
          isDisabledGetter = angular.bind(null, $parse(attr.ngDisabled), scope.$parent);
        }
        var thumb = angular.element(element[0].querySelector('.md-thumb'));
        var thumbText = angular.element(element[0].querySelector('.md-thumb-text'));
        var thumbContainer = thumb.parent();
        var trackContainer = angular.element(element[0].querySelector('.md-track-container'));
        var activeTrack = angular.element(element[0].querySelector('.md-track-fill'));
        var tickContainer = angular.element(element[0].querySelector('.md-track-ticks'));
        var throttledRefreshDimensions = $mdUtil.throttle(refreshSliderDimensions, 5000);
        // Default values, overridable by attrs
        angular.isDefined(attr.min) ? attr.$observe('min', updateMin) : updateMin(0);
        angular.isDefined(attr.max) ? attr.$observe('max', updateMax) : updateMax(100);
        angular.isDefined(attr.step) ? attr.$observe('step', updateStep) : updateStep(1);
        // We have to manually stop the $watch on ngDisabled because it exists
        // on the parent scope, and won't be automatically destroyed when
        // the component is destroyed.
        var stopDisabledWatch = angular.noop;
        if (attr.ngDisabled) {
          stopDisabledWatch = scope.$parent.$watch(attr.ngDisabled, updateAriaDisabled);
        }
        $mdGesture.register(element, 'drag');
        element.on('keydown', keydownListener).on('$md.pressdown', onPressDown).on('$md.pressup', onPressUp).on('$md.dragstart', onDragStart).on('$md.drag', onDrag).on('$md.dragend', onDragEnd);
        // On resize, recalculate the slider's dimensions and re-render
        function updateAll() {
          refreshSliderDimensions();
          ngModelRender();
          redrawTicks();
        }
        setTimeout(updateAll, 0);
        var debouncedUpdateAll = $$rAF.throttle(updateAll);
        angular.element($window).on('resize', debouncedUpdateAll);
        scope.$on('$destroy', function () {
          angular.element($window).off('resize', debouncedUpdateAll);
          stopDisabledWatch();
        });
        ngModelCtrl.$render = ngModelRender;
        ngModelCtrl.$viewChangeListeners.push(ngModelRender);
        ngModelCtrl.$formatters.push(minMaxValidator);
        ngModelCtrl.$formatters.push(stepValidator);
        /**
     * Attributes
     */
        var min;
        var max;
        var step;
        function updateMin(value) {
          min = parseFloat(value);
          element.attr('aria-valuemin', value);
          updateAll();
        }
        function updateMax(value) {
          max = parseFloat(value);
          element.attr('aria-valuemax', value);
          updateAll();
        }
        function updateStep(value) {
          step = parseFloat(value);
          redrawTicks();
        }
        function updateAriaDisabled(isDisabled) {
          element.attr('aria-disabled', !!isDisabled);
        }
        // Draw the ticks with canvas.
        // The alternative to drawing ticks with canvas is to draw one element for each tick,
        // which could quickly become a performance bottleneck.
        var tickCanvas, tickCtx;
        function redrawTicks() {
          if (!angular.isDefined(attr.mdDiscrete))
            return;
          if (angular.isUndefined(step))
            return;
          if (step <= 0) {
            var msg = 'Slider step value must be greater than zero when in discrete mode';
            $log.error(msg);
            throw new Error(msg);
          }
          var numSteps = Math.floor((max - min) / step);
          if (!tickCanvas) {
            tickCanvas = angular.element('<canvas style="position:absolute;">');
            tickContainer.append(tickCanvas);
            var trackTicksStyle = $window.getComputedStyle(tickContainer[0]);
            tickCtx = tickCanvas[0].getContext('2d');
            tickCtx.fillStyle = trackTicksStyle.backgroundColor || 'black';
          }
          var dimensions = getSliderDimensions();
          tickCanvas[0].width = dimensions.width;
          tickCanvas[0].height = dimensions.height;
          var distance;
          for (var i = 0; i <= numSteps; i++) {
            distance = Math.floor(dimensions.width * (i / numSteps));
            tickCtx.fillRect(distance - 1, 0, 2, dimensions.height);
          }
        }
        /**
     * Refreshing Dimensions
     */
        var sliderDimensions = {};
        refreshSliderDimensions();
        function refreshSliderDimensions() {
          sliderDimensions = trackContainer[0].getBoundingClientRect();
        }
        function getSliderDimensions() {
          throttledRefreshDimensions();
          return sliderDimensions;
        }
        /**
     * left/right arrow listener
     */
        function keydownListener(ev) {
          if (element[0].hasAttribute('disabled')) {
            return;
          }
          var changeAmount;
          if (ev.keyCode === $mdConstant.KEY_CODE.LEFT_ARROW) {
            changeAmount = -step;
          } else if (ev.keyCode === $mdConstant.KEY_CODE.RIGHT_ARROW) {
            changeAmount = step;
          }
          if (changeAmount) {
            if (ev.metaKey || ev.ctrlKey || ev.altKey) {
              changeAmount *= 4;
            }
            ev.preventDefault();
            ev.stopPropagation();
            scope.$evalAsync(function () {
              setModelValue(ngModelCtrl.$viewValue + changeAmount);
            });
          }
        }
        /**
     * ngModel setters and validators
     */
        function setModelValue(value) {
          ngModelCtrl.$setViewValue(minMaxValidator(stepValidator(value)));
        }
        function ngModelRender() {
          if (isNaN(ngModelCtrl.$viewValue)) {
            ngModelCtrl.$viewValue = ngModelCtrl.$modelValue;
          }
          var percent = (ngModelCtrl.$viewValue - min) / (max - min);
          scope.modelValue = ngModelCtrl.$viewValue;
          element.attr('aria-valuenow', ngModelCtrl.$viewValue);
          setSliderPercent(percent);
          thumbText.text(ngModelCtrl.$viewValue);
        }
        function minMaxValidator(value) {
          if (angular.isNumber(value)) {
            return Math.max(min, Math.min(max, value));
          }
        }
        function stepValidator(value) {
          if (angular.isNumber(value)) {
            var formattedValue = Math.round((value - min) / step) * step + min;
            // Format to 3 digits after the decimal point - fixes #2015.
            return Math.round(formattedValue * 1000) / 1000;
          }
        }
        /**
     * @param percent 0-1
     */
        function setSliderPercent(percent) {
          var percentStr = percent * 100 + '%';
          activeTrack.css('width', percentStr);
          thumbContainer.css('left', percentStr);
          element.toggleClass('md-min', percent === 0);
          element.toggleClass('md-max', percent === 1);
        }
        /**
     * Slide listeners
     */
        var isDragging = false;
        var isDiscrete = angular.isDefined(attr.mdDiscrete);
        function onPressDown(ev) {
          if (isDisabledGetter())
            return;
          element.addClass('md-active');
          element[0].focus();
          refreshSliderDimensions();
          var exactVal = percentToValue(positionToPercent(ev.pointer.x));
          var closestVal = minMaxValidator(stepValidator(exactVal));
          scope.$apply(function () {
            setModelValue(closestVal);
            setSliderPercent(valueToPercent(closestVal));
          });
        }
        function onPressUp(ev) {
          if (isDisabledGetter())
            return;
          element.removeClass('md-dragging md-active');
          var exactVal = percentToValue(positionToPercent(ev.pointer.x));
          var closestVal = minMaxValidator(stepValidator(exactVal));
          scope.$apply(function () {
            setModelValue(closestVal);
            ngModelRender();
          });
        }
        function onDragStart(ev) {
          if (isDisabledGetter())
            return;
          isDragging = true;
          ev.stopPropagation();
          element.addClass('md-dragging');
          setSliderFromEvent(ev);
        }
        function onDrag(ev) {
          if (!isDragging)
            return;
          ev.stopPropagation();
          setSliderFromEvent(ev);
        }
        function onDragEnd(ev) {
          if (!isDragging)
            return;
          ev.stopPropagation();
          isDragging = false;
        }
        function setSliderFromEvent(ev) {
          // While panning discrete, update only the
          // visual positioning but not the model value.
          if (isDiscrete)
            adjustThumbPosition(ev.pointer.x);
          else
            doSlide(ev.pointer.x);
        }
        /**
     * Slide the UI by changing the model value
     * @param x
     */
        function doSlide(x) {
          scope.$evalAsync(function () {
            setModelValue(percentToValue(positionToPercent(x)));
          });
        }
        /**
     * Slide the UI without changing the model (while dragging/panning)
     * @param x
     */
        function adjustThumbPosition(x) {
          var exactVal = percentToValue(positionToPercent(x));
          var closestVal = minMaxValidator(stepValidator(exactVal));
          setSliderPercent(positionToPercent(x));
          thumbText.text(closestVal);
        }
        /**
     * Convert horizontal position on slider to percentage value of offset from beginning...
     * @param x
     * @returns {number}
     */
        function positionToPercent(x) {
          return Math.max(0, Math.min(1, (x - sliderDimensions.left) / sliderDimensions.width));
        }
        /**
     * Convert percentage offset on slide to equivalent model value
     * @param percent
     * @returns {*}
     */
        function percentToValue(percent) {
          return min + percent * (max - min);
        }
        function valueToPercent(val) {
          return (val - min) / (max - min);
        }
      }
    }
    SliderDirective.$inject = [
      '$$rAF',
      '$window',
      '$mdAria',
      '$mdUtil',
      '$mdConstant',
      '$mdTheming',
      '$mdGesture',
      '$parse',
      '$log'
    ];
  }());
  (function () {
    'use strict';
    /**
 * @ngdoc module
 * @name material.components.sidenav
 *
 * @description
 * A Sidenav QP component.
 */
    angular.module('material.components.sidenav', [
      'material.core',
      'material.components.backdrop'
    ]).factory('$mdSidenav', SidenavService).directive('mdSidenav', SidenavDirective).directive('mdSidenavFocus', SidenavFocusDirective).controller('$mdSidenavController', SidenavController);
    /**
 * @ngdoc service
 * @name $mdSidenav
 * @module material.components.sidenav
 *
 * @description
 * `$mdSidenav` makes it easy to interact with multiple sidenavs
 * in an app.
 *
 * @usage
 * <hljs lang="js">
 * // Async lookup for sidenav instance; will resolve when the instance is available
 * $mdSidenav(componentId).then(function(instance) {
 *   $log.debug( componentId + "is now ready" );
 * });
 * // Async toggle the given sidenav;
 * // when instance is known ready and lazy lookup is not needed.
 * $mdSidenav(componentId)
 *    .toggle()
 *    .then(function(){
 *      $log.debug('toggled');
 *    });
 * // Async open the given sidenav
 * $mdSidenav(componentId)
 *    .open()
 *    .then(function(){
 *      $log.debug('opened');
 *    });
 * // Async close the given sidenav
 * $mdSidenav(componentId)
 *    .close()
 *    .then(function(){
 *      $log.debug('closed');
 *    });
 * // Sync check to see if the specified sidenav is set to be open
 * $mdSidenav(componentId).isOpen();
 * // Sync check to whether given sidenav is locked open
 * // If this is true, the sidenav will be open regardless of close()
 * $mdSidenav(componentId).isLockedOpen();
 * </hljs>
 */
    function SidenavService($mdComponentRegistry, $q) {
      return function (handle) {
        // Lookup the controller instance for the specified sidNav instance
        var self;
        var errorMsg = 'SideNav \'' + handle + '\' is not available!';
        var instance = $mdComponentRegistry.get(handle);
        if (!instance) {
          $mdComponentRegistry.notFoundError(handle);
        }
        return self = {
          isOpen: function () {
            return instance && instance.isOpen();
          },
          isLockedOpen: function () {
            return instance && instance.isLockedOpen();
          },
          toggle: function () {
            return instance ? instance.toggle() : $q.reject(errorMsg);
          },
          open: function () {
            return instance ? instance.open() : $q.reject(errorMsg);
          },
          close: function () {
            return instance ? instance.close() : $q.reject(errorMsg);
          },
          then: function (callbackFn) {
            var promise = instance ? $q.when(instance) : waitForInstance();
            return promise.then(callbackFn || angular.noop);
          }
        };
        /**
     * Deferred lookup of component instance using $component registry
     */
        function waitForInstance() {
          return $mdComponentRegistry.when(handle).then(function (it) {
            instance = it;
            return it;
          });
        }
      };
    }
    SidenavService.$inject = [
      '$mdComponentRegistry',
      '$q'
    ];
    /**
 * @ngdoc directive
 * @name mdSidenavFocus
 * @module material.components.sidenav
 *
 * @restrict A
 *
 * @description
 * `mdSidenavFocus` provides a way to specify the focused element when a sidenav opens.
 * This is completely optional, as the sidenav itself is focused by default.
 *
 * @usage
 * <hljs lang="html">
 * <md-sidenav>
 *   <form>
 *     <md-input-container>
 *       <label for="testInput">Label</label>
 *       <input id="testInput" type="text" md-sidenav-focus>
 *     </md-input-container>
 *   </form>
 * </md-sidenav>
 * </hljs>
 **/
    function SidenavFocusDirective() {
      return {
        restrict: 'A',
        require: '^mdSidenav',
        link: function (scope, element, attr, sidenavCtrl) {
        }
      };
    }
    /**
 * @ngdoc directive
 * @name mdSidenav
 * @module material.components.sidenav
 * @restrict E
 *
 * @description
 *
 * A Sidenav component that can be opened and closed programatically.
 *
 * By default, upon opening it will slide out on top of the main content area.
 *
 * For keyboard and screen reader accessibility, focus is sent to the sidenav wrapper by default.
 * It can be overridden with the `md-autofocus` directive on the child element you want focused.
 *
 * @usage
 * <hljs lang="html">
 * <div layout="row" ng-controller="MyController">
 *   <md-sidenav md-component-id="left" class="md-sidenav-left">
 *     Left Nav!
 *   </md-sidenav>
 *
 *   <md-content>
 *     Center Content
 *     <md-button ng-click="openLeftMenu()">
 *       Open Left Menu
 *     </md-button>
 *   </md-content>
 *
 *   <md-sidenav md-component-id="right"
 *     md-is-locked-open="$mdMedia('min-width: 333px')"
 *     class="md-sidenav-right">
 *     <form>
 *       <md-input-container>
 *         <label for="testInput">Test input</label>
 *         <input id="testInput" type="text"
 *                ng-model="data" md-autofocus>
 *       </md-input-container>
 *     </form>
 *   </md-sidenav>
 * </div>
 * </hljs>
 *
 * <hljs lang="js">
 * var app = angular.module('myApp', ['ngMaterial']);
 * app.controller('MyController', function($scope, $mdSidenav) {
 *   $scope.openLeftMenu = function() {
 *     $mdSidenav('left').toggle();
 *   };
 * });
 * </hljs>
 *
 * @param {expression=} md-is-open A model bound to whether the sidenav is opened.
 * @param {string=} md-component-id componentId to use with $mdSidenav service.
 * @param {expression=} md-is-locked-open When this expression evalutes to true,
 * the sidenav 'locks open': it falls into the content's flow instead
 * of appearing over it. This overrides the `is-open` attribute.
 *
 * The $mdMedia() service is exposed to the is-locked-open attribute, which
 * can be given a media query or one of the `sm`, `gt-sm`, `md`, `gt-md`, `lg` or `gt-lg` presets.
 * Examples:
 *
 *   - `<md-sidenav md-is-locked-open="shouldLockOpen"></md-sidenav>`
 *   - `<md-sidenav md-is-locked-open="$mdMedia('min-width: 1000px')"></md-sidenav>`
 *   - `<md-sidenav md-is-locked-open="$mdMedia('sm')"></md-sidenav>` (locks open on small screens)
 */
    function SidenavDirective($mdMedia, $mdUtil, $mdConstant, $mdTheming, $animate, $compile, $parse, $log, $q, $document) {
      return {
        restrict: 'E',
        scope: { isOpen: '=?mdIsOpen' },
        controller: '$mdSidenavController',
        compile: function (element) {
          element.addClass('md-closed');
          element.attr('tabIndex', '-1');
          return postLink;
        }
      };
      /**
   * Directive Post Link function...
   */
      function postLink(scope, element, attr, sidenavCtrl) {
        var lastParentOverFlow;
        var triggeringElement = null;
        var promise = $q.when(true);
        var isLockedOpenParsed = $parse(attr.mdIsLockedOpen);
        var isLocked = function () {
          return isLockedOpenParsed(scope.$parent, {
            $media: function (arg) {
              $log.warn('$media is deprecated for is-locked-open. Use $mdMedia instead.');
              return $mdMedia(arg);
            },
            $mdMedia: $mdMedia
          });
        };
        var backdrop = $mdUtil.createBackdrop(scope, 'md-sidenav-backdrop md-opaque ng-enter');
        $mdTheming.inherit(backdrop, element);
        element.on('$destroy', function () {
          backdrop.remove();
          sidenavCtrl.destroy();
        });
        scope.$on('$destroy', function () {
          backdrop.remove();
        });
        scope.$watch(isLocked, updateIsLocked);
        scope.$watch('isOpen', updateIsOpen);
        // Publish special accessor for the Controller instance
        sidenavCtrl.$toggleOpen = toggleOpen;
        /**
     * Toggle the DOM classes to indicate `locked`
     * @param isLocked
     */
        function updateIsLocked(isLocked, oldValue) {
          scope.isLockedOpen = isLocked;
          if (isLocked === oldValue) {
            element.toggleClass('md-locked-open', !!isLocked);
          } else {
            $animate[isLocked ? 'addClass' : 'removeClass'](element, 'md-locked-open');
          }
          backdrop.toggleClass('md-locked-open', !!isLocked);
        }
        /**
     * Toggle the SideNav view and attach/detach listeners
     * @param isOpen
     */
        function updateIsOpen(isOpen) {
          // Support deprecated md-sidenav-focus attribute as fallback
          var focusEl = $mdUtil.findFocusTarget(element) || $mdUtil.findFocusTarget(element, '[md-sidenav-focus]') || element;
          var parent = element.parent();
          parent[isOpen ? 'on' : 'off']('keydown', onKeyDown);
          backdrop[isOpen ? 'on' : 'off']('click', close);
          if (isOpen) {
            // Capture upon opening..
            triggeringElement = $document[0].activeElement;
          }
          disableParentScroll(isOpen);
          return promise = $q.all([
            isOpen ? $animate.enter(backdrop, parent) : $animate.leave(backdrop),
            $animate[isOpen ? 'removeClass' : 'addClass'](element, 'md-closed')
          ]).then(function () {
            // Perform focus when animations are ALL done...
            if (scope.isOpen) {
              focusEl && focusEl.focus();
            }
          });
        }
        /**
     * Prevent parent scrolling (when the SideNav is open)
     */
        function disableParentScroll(disabled) {
          var parent = element.parent();
          if (disabled && !lastParentOverFlow) {
            lastParentOverFlow = parent.css('overflow');
            parent.css('overflow', 'hidden');
          } else if (angular.isDefined(lastParentOverFlow)) {
            parent.css('overflow', lastParentOverFlow);
            lastParentOverFlow = undefined;
          }
        }
        /**
     * Toggle the sideNav view and publish a promise to be resolved when
     * the view animation finishes.
     *
     * @param isOpen
     * @returns {*}
     */
        function toggleOpen(isOpen) {
          if (scope.isOpen == isOpen) {
            return $q.when(true);
          } else {
            return $q(function (resolve) {
              // Toggle value to force an async `updateIsOpen()` to run
              scope.isOpen = isOpen;
              $mdUtil.nextTick(function () {
                // When the current `updateIsOpen()` animation finishes
                promise.then(function (result) {
                  if (!scope.isOpen) {
                    // reset focus to originating element (if available) upon close
                    triggeringElement && triggeringElement.focus();
                    triggeringElement = null;
                  }
                  resolve(result);
                });
              });
            });
          }
        }
        /**
     * Auto-close sideNav when the `escape` key is pressed.
     * @param evt
     */
        function onKeyDown(ev) {
          var isEscape = ev.keyCode === $mdConstant.KEY_CODE.ESCAPE;
          return isEscape ? close(ev) : $q.when(true);
        }
        /**
     * With backdrop `clicks` or `escape` key-press, immediately
     * apply the CSS close transition... Then notify the controller
     * to close() and perform its own actions.
     */
        function close(ev) {
          ev.preventDefault();
          return sidenavCtrl.close();
        }
      }
    }
    SidenavDirective.$inject = [
      '$mdMedia',
      '$mdUtil',
      '$mdConstant',
      '$mdTheming',
      '$animate',
      '$compile',
      '$parse',
      '$log',
      '$q',
      '$document'
    ];
    /*
 * @private
 * @ngdoc controller
 * @name SidenavController
 * @module material.components.sidenav
 *
 */
    function SidenavController($scope, $element, $attrs, $mdComponentRegistry, $q) {
      var self = this;
      // Use Default internal method until overridden by directive postLink
      // Synchronous getters
      self.isOpen = function () {
        return !!$scope.isOpen;
      };
      self.isLockedOpen = function () {
        return !!$scope.isLockedOpen;
      };
      // Async actions
      self.open = function () {
        return self.$toggleOpen(true);
      };
      self.close = function () {
        return self.$toggleOpen(false);
      };
      self.toggle = function () {
        return self.$toggleOpen(!$scope.isOpen);
      };
      self.$toggleOpen = function (value) {
        return $q.when($scope.isOpen = value);
      };
      self.destroy = $mdComponentRegistry.register(self, $attrs.mdComponentId);
    }
    SidenavController.$inject = [
      '$scope',
      '$element',
      '$attrs',
      '$mdComponentRegistry',
      '$q'
    ];
  }());
  (function () {
    'use strict';
    /**
 * @ngdoc module
 * @name material.components.sticky
 * @description
 * Sticky effects for md
 *
 */
    angular.module('material.components.sticky', [
      'material.core',
      'material.components.content'
    ]).factory('$mdSticky', MdSticky);
    /**
 * @ngdoc service
 * @name $mdSticky
 * @module material.components.sticky
 *
 * @description
 * The `$mdSticky`service provides a mixin to make elements sticky.
 *
 * @returns A `$mdSticky` function that takes three arguments:
 *   - `scope`
 *   - `element`: The element that will be 'sticky'
 *   - `elementClone`: A clone of the element, that will be shown
 *     when the user starts scrolling past the original element.
 *     If not provided, it will use the result of `element.clone()`.
 */
    function MdSticky($document, $mdConstant, $$rAF, $mdUtil) {
      var browserStickySupport = checkStickySupport();
      /**
   * Registers an element as sticky, used internally by directives to register themselves
   */
      return function registerStickyElement(scope, element, stickyClone) {
        var contentCtrl = element.controller('mdContent');
        if (!contentCtrl)
          return;
        if (browserStickySupport) {
          element.css({
            position: browserStickySupport,
            top: 0,
            'z-index': 2
          });
        } else {
          var $$sticky = contentCtrl.$element.data('$$sticky');
          if (!$$sticky) {
            $$sticky = setupSticky(contentCtrl);
            contentCtrl.$element.data('$$sticky', $$sticky);
          }
          var deregister = $$sticky.add(element, stickyClone || element.clone());
          scope.$on('$destroy', deregister);
        }
      };
      function setupSticky(contentCtrl) {
        var contentEl = contentCtrl.$element;
        // Refresh elements is very expensive, so we use the debounced
        // version when possible.
        var debouncedRefreshElements = $$rAF.throttle(refreshElements);
        // setupAugmentedScrollEvents gives us `$scrollstart` and `$scroll`,
        // more reliable than `scroll` on android.
        setupAugmentedScrollEvents(contentEl);
        contentEl.on('$scrollstart', debouncedRefreshElements);
        contentEl.on('$scroll', onScroll);
        var self;
        return self = {
          prev: null,
          current: null,
          next: null,
          items: [],
          add: add,
          refreshElements: refreshElements
        };
        /***************
     * Public
     ***************/
        // Add an element and its sticky clone to this content's sticky collection
        function add(element, stickyClone) {
          stickyClone.addClass('md-sticky-clone');
          var item = {
              element: element,
              clone: stickyClone
            };
          self.items.push(item);
          $mdUtil.nextTick(function () {
            contentEl.prepend(item.clone);
          });
          debouncedRefreshElements();
          return function remove() {
            self.items.forEach(function (item, index) {
              if (item.element[0] === element[0]) {
                self.items.splice(index, 1);
                item.clone.remove();
              }
            });
            debouncedRefreshElements();
          };
        }
        function refreshElements() {
          // Sort our collection of elements by their current position in the DOM.
          // We need to do this because our elements' order of being added may not
          // be the same as their order of display.
          self.items.forEach(refreshPosition);
          self.items = self.items.sort(function (a, b) {
            return a.top < b.top ? -1 : 1;
          });
          // Find which item in the list should be active, 
          // based upon the content's current scroll position
          var item;
          var currentScrollTop = contentEl.prop('scrollTop');
          for (var i = self.items.length - 1; i >= 0; i--) {
            if (currentScrollTop > self.items[i].top) {
              item = self.items[i];
              break;
            }
          }
          setCurrentItem(item);
        }
        /***************
     * Private
     ***************/
        // Find the `top` of an item relative to the content element,
        // and also the height.
        function refreshPosition(item) {
          // Find the top of an item by adding to the offsetHeight until we reach the 
          // content element.
          var current = item.element[0];
          item.top = 0;
          item.left = 0;
          while (current && current !== contentEl[0]) {
            item.top += current.offsetTop;
            item.left += current.offsetLeft;
            current = current.offsetParent;
          }
          item.height = item.element.prop('offsetHeight');
          item.clone.css('margin-left', item.left + 'px');
          if ($mdUtil.floatingScrollbars()) {
            item.clone.css('margin-right', '0');
          }
        }
        // As we scroll, push in and select the correct sticky element.
        function onScroll() {
          var scrollTop = contentEl.prop('scrollTop');
          var isScrollingDown = scrollTop > (onScroll.prevScrollTop || 0);
          // Store the previous scroll so we know which direction we are scrolling
          onScroll.prevScrollTop = scrollTop;
          //
          // AT TOP (not scrolling)
          //
          if (scrollTop === 0) {
            // If we're at the top, just clear the current item and return
            setCurrentItem(null);
            return;
          }
          //
          // SCROLLING DOWN (going towards the next item)
          //
          if (isScrollingDown) {
            // If we've scrolled down past the next item's position, sticky it and return
            if (self.next && self.next.top <= scrollTop) {
              setCurrentItem(self.next);
              return;
            }
            // If the next item is close to the current one, push the current one up out of the way
            if (self.current && self.next && self.next.top - scrollTop <= self.next.height) {
              translate(self.current, scrollTop + (self.next.top - self.next.height - scrollTop));
              return;
            }
          }
          //
          // SCROLLING UP (not at the top & not scrolling down; must be scrolling up)
          //
          if (!isScrollingDown) {
            // If we've scrolled up past the previous item's position, sticky it and return
            if (self.current && self.prev && scrollTop < self.current.top) {
              setCurrentItem(self.prev);
              return;
            }
            // If the next item is close to the current one, pull the current one down into view
            if (self.next && self.current && scrollTop >= self.next.top - self.current.height) {
              translate(self.current, scrollTop + (self.next.top - scrollTop - self.current.height));
              return;
            }
          }
          //
          // Otherwise, just move the current item to the proper place (scrolling up or down)
          //
          if (self.current) {
            translate(self.current, scrollTop);
          }
        }
        function setCurrentItem(item) {
          if (self.current === item)
            return;
          // Deactivate currently active item
          if (self.current) {
            translate(self.current, null);
            setStickyState(self.current, null);
          }
          // Activate new item if given
          if (item) {
            setStickyState(item, 'active');
          }
          self.current = item;
          var index = self.items.indexOf(item);
          // If index === -1, index + 1 = 0. It works out.
          self.next = self.items[index + 1];
          self.prev = self.items[index - 1];
          setStickyState(self.next, 'next');
          setStickyState(self.prev, 'prev');
        }
        function setStickyState(item, state) {
          if (!item || item.state === state)
            return;
          if (item.state) {
            item.clone.attr('sticky-prev-state', item.state);
            item.element.attr('sticky-prev-state', item.state);
          }
          item.clone.attr('sticky-state', state);
          item.element.attr('sticky-state', state);
          item.state = state;
        }
        function translate(item, amount) {
          if (!item)
            return;
          if (amount === null || amount === undefined) {
            if (item.translateY) {
              item.translateY = null;
              item.clone.css($mdConstant.CSS.TRANSFORM, '');
            }
          } else {
            item.translateY = amount;
            item.clone.css($mdConstant.CSS.TRANSFORM, 'translate3d(' + item.left + 'px,' + amount + 'px,0)');
          }
        }
      }
      // Function to check for browser sticky support
      function checkStickySupport($el) {
        var stickyProp;
        var testEl = angular.element('<div>');
        $document[0].body.appendChild(testEl[0]);
        var stickyProps = [
            'sticky',
            '-webkit-sticky'
          ];
        for (var i = 0; i < stickyProps.length; ++i) {
          testEl.css({
            position: stickyProps[i],
            top: 0,
            'z-index': 2
          });
          if (testEl.css('position') == stickyProps[i]) {
            stickyProp = stickyProps[i];
            break;
          }
        }
        testEl.remove();
        return stickyProp;
      }
      // Android 4.4 don't accurately give scroll events.
      // To fix this problem, we setup a fake scroll event. We say:
      // > If a scroll or touchmove event has happened in the last DELAY milliseconds, 
      //   then send a `$scroll` event every animationFrame.
      // Additionally, we add $scrollstart and $scrollend events.
      function setupAugmentedScrollEvents(element) {
        var SCROLL_END_DELAY = 200;
        var isScrolling;
        var lastScrollTime;
        element.on('scroll touchmove', function () {
          if (!isScrolling) {
            isScrolling = true;
            $$rAF.throttle(loopScrollEvent);
            element.triggerHandler('$scrollstart');
          }
          element.triggerHandler('$scroll');
          lastScrollTime = +$mdUtil.now();
        });
        function loopScrollEvent() {
          if (+$mdUtil.now() - lastScrollTime > SCROLL_END_DELAY) {
            isScrolling = false;
            element.triggerHandler('$scrollend');
          } else {
            element.triggerHandler('$scroll');
            $$rAF.throttle(loopScrollEvent);
          }
        }
      }
    }
    MdSticky.$inject = [
      '$document',
      '$mdConstant',
      '$$rAF',
      '$mdUtil'
    ];
  }());
  (function () {
    'use strict';
    /**
 * @ngdoc module
 * @name material.components.subheader
 * @description
 * SubHeader module
 *
 *  Subheaders are special list tiles that delineate distinct sections of a
 *  list or grid list and are typically related to the current filtering or
 *  sorting criteria. Subheader tiles are either displayed inline with tiles or
 *  can be associated with content, for example, in an adjacent column.
 *
 *  Upon scrolling, subheaders remain pinned to the top of the screen and remain
 *  pinned until pushed on or off screen by the next subheader. @see [Material
 *  Design Specifications](https://www.google.com/design/spec/components/subheaders.html)
 *
 *  > To improve the visual grouping of content, use the system color for your subheaders.
 *
 */
    angular.module('material.components.subheader', [
      'material.core',
      'material.components.sticky'
    ]).directive('mdSubheader', MdSubheaderDirective);
    /**
 * @ngdoc directive
 * @name mdSubheader
 * @module material.components.subheader
 *
 * @restrict E
 *
 * @description
 * The `<md-subheader>` directive is a subheader for a section. By default it is sticky.
 * You can make it not sticky by applying the `md-no-sticky` class to the subheader.
 *
 *
 * @usage
 * <hljs lang="html">
 * <md-subheader>Online Friends</md-subheader>
 * </hljs>
 */
    function MdSubheaderDirective($mdSticky, $compile, $mdTheming, $mdUtil) {
      return {
        restrict: 'E',
        replace: true,
        transclude: true,
        template: '<div class="md-subheader">' + '  <div class="md-subheader-inner">' + '    <span class="md-subheader-content"></span>' + '  </div>' + '</div>',
        link: function postLink(scope, element, attr, controllers, transclude) {
          $mdTheming(element);
          var outerHTML = element[0].outerHTML;
          function getContent(el) {
            return angular.element(el[0].querySelector('.md-subheader-content'));
          }
          // Transclude the user-given contents of the subheader
          // the conventional way.
          transclude(scope, function (clone) {
            getContent(element).append(clone);
          });
          // Create another clone, that uses the outer and inner contents
          // of the element, that will be 'stickied' as the user scrolls.
          if (!element.hasClass('md-no-sticky')) {
            transclude(scope, function (clone) {
              // If the user adds an ng-if or ng-repeat directly to the md-subheader element, the
              // compiled clone below will only be a comment tag (since they replace their elements with
              // a comment) which cannot be properly passed to the $mdSticky; so we wrap it in our own
              // DIV to ensure we have something $mdSticky can use
              var wrapperHtml = '<div class="md-subheader-wrapper">' + outerHTML + '</div>';
              var stickyClone = $compile(wrapperHtml)(scope);
              // Append the sticky
              $mdSticky(scope, element, stickyClone);
              // Delay initialization until after any `ng-if`/`ng-repeat`/etc has finished before
              // attempting to create the clone
              $mdUtil.nextTick(function () {
                getContent(stickyClone).append(clone);
              });
            });
          }
        }
      };
    }
    MdSubheaderDirective.$inject = [
      '$mdSticky',
      '$compile',
      '$mdTheming',
      '$mdUtil'
    ];
  }());
  (function () {
    'use strict';
    /**
 * @private
 * @ngdoc module
 * @name material.components.switch
 */
    angular.module('material.components.switch', [
      'material.core',
      'material.components.checkbox'
    ]).directive('mdSwitch', MdSwitch);
    /**
 * @private
 * @ngdoc directive
 * @module material.components.switch
 * @name mdSwitch
 * @restrict E
 *
 * The switch directive is used very much like the normal [angular checkbox](https://docs.angularjs.org/api/ng/input/input%5Bcheckbox%5D).
 *
 * As per the [material design spec](http://www.google.com/design/spec/style/color.html#color-ui-color-application)
 * the switch is in the accent color by default. The primary color palette may be used with
 * the `md-primary` class.
 *
 * @param {string} ng-model Assignable angular expression to data-bind to.
 * @param {string=} name Property name of the form under which the control is published.
 * @param {expression=} ng-true-value The value to which the expression should be set when selected.
 * @param {expression=} ng-false-value The value to which the expression should be set when not selected.
 * @param {string=} ng-change Angular expression to be executed when input changes due to user interaction with the input element.
 * @param {expression=} ng-disabled En/Disable based on the expression.
 * @param {boolean=} md-no-ink Use of attribute indicates use of ripple ink effects.
 * @param {string=} aria-label Publish the button label used by screen-readers for accessibility. Defaults to the switch's text.
 *
 * @usage
 * <hljs lang="html">
 * <md-switch ng-model="isActive" aria-label="Finished?">
 *   Finished ?
 * </md-switch>
 *
 * <md-switch md-no-ink ng-model="hasInk" aria-label="No Ink Effects">
 *   No Ink Effects
 * </md-switch>
 *
 * <md-switch ng-disabled="true" ng-model="isDisabled" aria-label="Disabled">
 *   Disabled
 * </md-switch>
 *
 * </hljs>
 */
    function MdSwitch(mdCheckboxDirective, $mdUtil, $mdConstant, $parse, $$rAF, $mdGesture) {
      var checkboxDirective = mdCheckboxDirective[0];
      return {
        restrict: 'E',
        priority: 210,
        transclude: true,
        template: '<div class="md-container">' + '<div class="md-bar"></div>' + '<div class="md-thumb-container">' + '<div class="md-thumb" md-ink-ripple md-ink-ripple-checkbox></div>' + '</div>' + '</div>' + '<div ng-transclude class="md-label"></div>',
        require: '?ngModel',
        compile: mdSwitchCompile
      };
      function mdSwitchCompile(element, attr) {
        var checkboxLink = checkboxDirective.compile(element, attr);
        // No transition on initial load.
        element.addClass('md-dragging');
        return function (scope, element, attr, ngModel) {
          ngModel = ngModel || $mdUtil.fakeNgModel();
          var disabledGetter = null;
          if (attr.disabled != null) {
            disabledGetter = function () {
              return true;
            };
          } else if (attr.ngDisabled) {
            disabledGetter = $parse(attr.ngDisabled);
          }
          var thumbContainer = angular.element(element[0].querySelector('.md-thumb-container'));
          var switchContainer = angular.element(element[0].querySelector('.md-container'));
          // no transition on initial load
          $$rAF(function () {
            element.removeClass('md-dragging');
          });
          checkboxLink(scope, element, attr, ngModel);
          if (disabledGetter) {
            scope.$watch(disabledGetter, function (isDisabled) {
              element.attr('tabindex', isDisabled ? -1 : 0);
            });
          }
          // These events are triggered by setup drag
          $mdGesture.register(switchContainer, 'drag');
          switchContainer.on('$md.dragstart', onDragStart).on('$md.drag', onDrag).on('$md.dragend', onDragEnd);
          var drag;
          function onDragStart(ev) {
            // Don't go if the switch is disabled.
            if (disabledGetter && disabledGetter(scope))
              return;
            ev.stopPropagation();
            element.addClass('md-dragging');
            drag = { width: thumbContainer.prop('offsetWidth') };
            element.removeClass('transition');
          }
          function onDrag(ev) {
            if (!drag)
              return;
            ev.stopPropagation();
            ev.srcEvent && ev.srcEvent.preventDefault();
            var percent = ev.pointer.distanceX / drag.width;
            //if checked, start from right. else, start from left
            var translate = ngModel.$viewValue ? 1 + percent : percent;
            // Make sure the switch stays inside its bounds, 0-1%
            translate = Math.max(0, Math.min(1, translate));
            thumbContainer.css($mdConstant.CSS.TRANSFORM, 'translate3d(' + 100 * translate + '%,0,0)');
            drag.translate = translate;
          }
          function onDragEnd(ev) {
            if (!drag)
              return;
            ev.stopPropagation();
            element.removeClass('md-dragging');
            thumbContainer.css($mdConstant.CSS.TRANSFORM, '');
            // We changed if there is no distance (this is a click a click),
            // or if the drag distance is >50% of the total.
            var isChanged = ngModel.$viewValue ? drag.translate > 0.5 : drag.translate < 0.5;
            if (isChanged) {
              applyModelValue(!ngModel.$viewValue);
            }
            drag = null;
          }
          function applyModelValue(newValue) {
            scope.$apply(function () {
              ngModel.$setViewValue(newValue);
              ngModel.$render();
            });
          }
        };
      }
    }
    MdSwitch.$inject = [
      'mdCheckboxDirective',
      '$mdUtil',
      '$mdConstant',
      '$parse',
      '$$rAF',
      '$mdGesture'
    ];
  }());
  (function () {
    'use strict';
    /**
 * @ngdoc module
 * @name material.components.swipe
 * @description Swipe module!
 */
    /**
 * @ngdoc directive
 * @module material.components.swipe
 * @name mdSwipeLeft
 *
 * @restrict A
 *
 * @description
 * The md-swipe-left directive allows you to specify custom behavior when an element is swiped
 * left.
 *
 * @usage
 * <hljs lang="html">
 * <div md-swipe-left="onSwipeLeft()">Swipe me left!</div>
 * </hljs>
 */
    /**
 * @ngdoc directive
 * @module material.components.swipe
 * @name mdSwipeRight
 *
 * @restrict A
 *
 * @description
 * The md-swipe-right directive allows you to specify custom behavior when an element is swiped
 * right.
 *
 * @usage
 * <hljs lang="html">
 * <div md-swipe-right="onSwipeRight()">Swipe me right!</div>
 * </hljs>
 */
    /**
 * @ngdoc directive
 * @module material.components.swipe
 * @name mdSwipeUp
 *
 * @restrict A
 *
 * @description
 * The md-swipe-up directive allows you to specify custom behavior when an element is swiped
 * up.
 *
 * @usage
 * <hljs lang="html">
 * <div md-swipe-up="onSwipeUp()">Swipe me up!</div>
 * </hljs>
 */
    /**
 * @ngdoc directive
 * @module material.components.swipe
 * @name mdSwipeDown
 *
 * @restrict A
 *
 * @description
 * The md-swipe-down directive allows you to specify custom behavior when an element is swiped
 * down.
 *
 * @usage
 * <hljs lang="html">
 * <div md-swipe-down="onSwipDown()">Swipe me down!</div>
 * </hljs>
 */
    angular.module('material.components.swipe', ['material.core']).directive('mdSwipeLeft', getDirective('SwipeLeft')).directive('mdSwipeRight', getDirective('SwipeRight')).directive('mdSwipeUp', getDirective('SwipeUp')).directive('mdSwipeDown', getDirective('SwipeDown'));
    function getDirective(name) {
      var directiveName = 'md' + name;
      var eventName = '$md.' + name.toLowerCase();
      DirectiveFactory.$inject = ['$parse'];
      return DirectiveFactory;
      /* @ngInject */
      function DirectiveFactory($parse) {
        return {
          restrict: 'A',
          link: postLink
        };
        function postLink(scope, element, attr) {
          var fn = $parse(attr[directiveName]);
          element.on(eventName, function (ev) {
            scope.$apply(function () {
              fn(scope, { $event: ev });
            });
          });
        }
      }
    }
  }());
  (function () {
    'use strict';
    /**
 * @ngdoc module
 * @name material.components.tabs
 * @description
 *
 *  Tabs, created with the `<md-tabs>` directive provide *tabbed* navigation with different styles.
 *  The Tabs component consists of clickable tabs that are aligned horizontally side-by-side.
 *
 *  Features include support for:
 *
 *  - static or dynamic tabs,
 *  - responsive designs,
 *  - accessibility support (ARIA),
 *  - tab pagination,
 *  - external or internal tab content,
 *  - focus indicators and arrow-key navigations,
 *  - programmatic lookup and access to tab controllers, and
 *  - dynamic transitions through different tab contents.
 *
 */
    /*
 * @see js folder for tabs implementation
 */
    angular.module('material.components.tabs', [
      'material.core',
      'material.components.icon'
    ]);
  }());
  (function () {
    'use strict';
    /**
  * @ngdoc module
  * @name material.components.toast
  * @description
  * Toast
  */
    angular.module('material.components.toast', [
      'material.core',
      'material.components.button'
    ]).directive('mdToast', MdToastDirective).provider('$mdToast', MdToastProvider);
    /* @ngInject */
    function MdToastDirective($mdToast) {
      return {
        restrict: 'E',
        link: function postLink(scope, element, attr) {
          // When navigation force destroys an interimElement, then
          // listen and $destroy() that interim instance...
          scope.$on('$destroy', function () {
            $mdToast.destroy();
          });
        }
      };
    }
    MdToastDirective.$inject = ['$mdToast'];
    /**
  * @ngdoc service
  * @name $mdToast
  * @module material.components.toast
  *
  * @description
  * `$mdToast` is a service to build a toast notification on any position
  * on the screen with an optional duration, and provides a simple promise API.
  *
  *
  * ## Restrictions on custom toasts
  * - The toast's template must have an outer `<md-toast>` element.
  * - For a toast action, use element with class `md-action`.
  * - Add the class `md-capsule` for curved corners.
  *
  * ## Parent container notes
  *
  * The toast is positioned using absolute positioning relative to it's first non-static parent
  * container. Thus, if the requested parent container uses static positioning, we will temporarily
  * set it's positioning to `relative` while the toast is visible and reset it when the toast is
  * hidden.
  *
  * Because of this, it is usually best to ensure that the parent container has a fixed height and
  * prevents scrolling by setting the `overflow: hidden;` style. Since the position is based off of
  * the parent's height, the toast may be mispositioned if you allow the parent to scroll.
  *
  * You can, however, have a scrollable element inside of the container; just make sure the
  * container itself does not scroll.
  *
  * <hljs lang="html">
  * <div layout-fill id="toast-container">
  *   <md-content>
  *     I can have lots of content and scroll!
  *   </md-content>
  * </div>
  * </hljs>
  *
  * @usage
  * <hljs lang="html">
  * <div ng-controller="MyController">
  *   <md-button ng-click="openToast()">
  *     Open a Toast!
  *   </md-button>
  * </div>
  * </hljs>
  *
  * <hljs lang="js">
  * var app = angular.module('app', ['ngMaterial']);
  * app.controller('MyController', function($scope, $mdToast) {
  *   $scope.openToast = function($event) {
  *     $mdToast.show($mdToast.simple().textContent('Hello!'));
  *     // Could also do $mdToast.showSimple('Hello');
  *   };
  * });
  * </hljs>
  */
    /**
 * @ngdoc method
 * @name $mdToast#showSimple
 * 
 * @param {string} message The message to display inside the toast
 * @description
 * Convenience method which builds and shows a simple toast.
 *
 * @returns {promise} A promise that can be resolved with `$mdToast.hide()` or
 * rejected with `$mdToast.cancel()`.
 *
 */
    /**
  * @ngdoc method
  * @name $mdToast#simple
  *
  * @description
  * Builds a preconfigured toast.
  *
  * @returns {obj} a `$mdToastPreset` with the following chainable configuration methods.
  *
  * _**Note:** These configuration methods are provided in addition to the methods provided by
  *   the `build()` and `show()` methods below._
  *
  * - `.textContent(string)` - Sets the toast content to the specified string.
  *
  * - `.action(string)` - Adds an action button. If clicked, the promise (returned from `show()`)
  * will resolve with the value `'ok'`; otherwise, it is resolved with `true` after a `hideDelay`
  * timeout.
  *
  * - `.highlightAction(boolean)` - Whether or not the action button will have an additional
  * highlight class.
  *
  * - `.capsule(boolean)` - Whether or not to add the `md-capsule` class to the toast to provide
  * rounded corners.
  *
  * - `.theme(string)` - Sets the theme on the toast to the requested theme. Default is
  * `$mdThemingProvider`'s default.
  */
    /**
  * @ngdoc method
  * @name $mdToast#updateTextContent
  *
  * @description
  * Updates the content of an existing toast. Useful for updating things like counts, etc.
  *
  */
    /**
  * @ngdoc method
  * @name $mdToast#build
  *
  * @description
  * Creates a custom `$mdToastPreset` that you can configure.
  *
  * @returns {obj} a `$mdToastPreset` with the chainable configuration methods for shows' options (see below).
  */
    /**
  * @ngdoc method
  * @name $mdToast#show
  *
  * @description Shows the toast.
  *
  * @param {object} optionsOrPreset Either provide an `$mdToastPreset` returned from `simple()`
  * and `build()`, or an options object with the following properties:
  *
  *   - `templateUrl` - `{string=}`: The url of an html template file that will
  *     be used as the content of the toast. Restrictions: the template must
  *     have an outer `md-toast` element.
  *   - `template` - `{string=}`: Same as templateUrl, except this is an actual
  *     template string.
  *   - `autoWrap` - `{boolean=}`: Whether or not to automatically wrap the template content with a
  *     `<div class="md-toast-content">` if one is not provided. Defaults to true. Can be disabled if you provide a
  *     custom toast directive.
  *   - `scope` - `{object=}`: the scope to link the template / controller to. If none is specified, it will create a new child scope.
  *     This scope will be destroyed when the toast is removed unless `preserveScope` is set to true.
  *   - `preserveScope` - `{boolean=}`: whether to preserve the scope when the element is removed. Default is false
  *   - `hideDelay` - `{number=}`: How many milliseconds the toast should stay
  *     active before automatically closing.  Set to 0 or false to have the toast stay open until
  *     closed manually. Default: 3000.
  *   - `position` - `{string=}`: Where to place the toast. Available: any combination
  *     of 'bottom', 'left', 'top', 'right'. Default: 'bottom left'.
  *   - `controller` - `{string=}`: The controller to associate with this toast.
  *     The controller will be injected the local `$mdToast.hide( )`, which is a function
  *     used to hide the toast.
  *   - `locals` - `{string=}`: An object containing key/value pairs. The keys will
  *     be used as names of values to inject into the controller. For example,
  *     `locals: {three: 3}` would inject `three` into the controller with the value
  *     of 3.
  *   - `bindToController` - `bool`: bind the locals to the controller, instead of passing them in. These values will not be available until after initialization.
  *   - `resolve` - `{object=}`: Similar to locals, except it takes promises as values
  *     and the toast will not open until the promises resolve.
  *   - `controllerAs` - `{string=}`: An alias to assign the controller to on the scope.
  *   - `parent` - `{element=}`: The element to append the toast to. Defaults to appending
  *     to the root element of the application.
  *
  * @returns {promise} A promise that can be resolved with `$mdToast.hide()` or
  * rejected with `$mdToast.cancel()`. `$mdToast.hide()` will resolve either with a Boolean
  * value == 'true' or the value passed as an argument to `$mdToast.hide()`.
  * And `$mdToast.cancel()` will resolve the promise with a Boolean value == 'false'
  */
    /**
  * @ngdoc method
  * @name $mdToast#hide
  *
  * @description
  * Hide an existing toast and resolve the promise returned from `$mdToast.show()`.
  *
  * @param {*=} response An argument for the resolved promise.
  *
  * @returns {promise} a promise that is called when the existing element is removed from the DOM.
  * The promise is resolved with either a Boolean value == 'true' or the value passed as the
  * argument to `.hide()`.
  *
  */
    /**
  * @ngdoc method
  * @name $mdToast#cancel
  *
  * @description
  * `DEPRECATED` - The promise returned from opening a toast is used only to notify about the closing of the toast.
  * As such, there isn't any reason to also allow that promise to be rejected,
  * since it's not clear what the difference between resolve and reject would be.
  *
  * Hide the existing toast and reject the promise returned from
  * `$mdToast.show()`.
  *
  * @param {*=} response An argument for the rejected promise.
  *
  * @returns {promise} a promise that is called when the existing element is removed from the DOM
  * The promise is resolved with a Boolean value == 'false'.
  *
  */
    function MdToastProvider($$interimElementProvider) {
      // Differentiate promise resolves: hide timeout (value == true) and hide action clicks (value == ok).
      var ACTION_RESOLVE = 'ok';
      var activeToastContent;
      var $mdToast = $$interimElementProvider('$mdToast').setDefaults({
          methods: [
            'position',
            'hideDelay',
            'capsule',
            'parent'
          ],
          options: toastDefaultOptions
        }).addPreset('simple', {
          argOption: 'textContent',
          methods: [
            'textContent',
            'content',
            'action',
            'highlightAction',
            'theme',
            'parent'
          ],
          options: [
            '$mdToast',
            '$mdTheming',
            function ($mdToast, $mdTheming) {
              var opts = {
                  template: '<md-toast md-theme="{{ toast.theme }}" ng-class="{\'md-capsule\': toast.capsule}">' + '  <div class="md-toast-content">' + '    <span flex role="alert" aria-relevant="all" aria-atomic="true">' + '      {{ toast.content }}' + '    </span>' + '    <md-button class="md-action" ng-if="toast.action" ng-click="toast.resolve()" ng-class="{\'md-highlight\': toast.highlightAction}">' + '      {{ toast.action }}' + '    </md-button>' + '  </div>' + '</md-toast>',
                  controller: [
                    '$scope',
                    function mdToastCtrl($scope) {
                      var self = this;
                      $scope.$watch(function () {
                        return activeToastContent;
                      }, function () {
                        self.content = activeToastContent;
                      });
                      this.resolve = function () {
                        $mdToast.hide(ACTION_RESOLVE);
                      };
                    }
                  ],
                  theme: $mdTheming.defaultTheme(),
                  controllerAs: 'toast',
                  bindToController: true
                };
              return opts;
            }
          ]
        }).addMethod('updateTextContent', updateTextContent).addMethod('updateContent', updateTextContent);
      function updateTextContent(newContent) {
        activeToastContent = newContent;
      }
      toastDefaultOptions.$inject = [
        '$animate',
        '$mdToast',
        '$mdUtil',
        '$mdMedia'
      ];
      return $mdToast;
      /* @ngInject */
      function toastDefaultOptions($animate, $mdToast, $mdUtil, $mdMedia) {
        var SWIPE_EVENTS = '$md.swipeleft $md.swiperight $md.swipeup $md.swipedown';
        return {
          onShow: onShow,
          onRemove: onRemove,
          position: 'bottom left',
          themable: true,
          hideDelay: 3000,
          autoWrap: true,
          transformTemplate: function (template, options) {
            var shouldAddWrapper = options.autoWrap && template && !/md-toast-content/g.test(template);
            if (shouldAddWrapper) {
              // Root element of template will be <md-toast>. We need to wrap all of its content inside of
              // of <div class="md-toast-content">. All templates provided here should be static, developer-controlled
              // content (meaning we're not attempting to guard against XSS).
              var parsedTemplate = angular.element(template);
              var wrappedContent = '<div class="md-toast-content">' + parsedTemplate.html() + '</div>';
              parsedTemplate.empty().append(wrappedContent);
              // Underlying interimElement expects a template string.
              return parsedTemplate[0].outerHTML;
            }
            return shouldAddWrapper ? '<div class="md-toast-content">' + template + '</div>' : template || '';
          }
        };
        function onShow(scope, element, options) {
          activeToastContent = options.textContent || options.content;
          // support deprecated #content method
          var isSmScreen = !$mdMedia('gt-sm');
          element = $mdUtil.extractElementByName(element, 'md-toast', true);
          options.onSwipe = function (ev, gesture) {
            //Add the relevant swipe class to the element so it can animate correctly
            var swipe = ev.type.replace('$md.', '');
            var direction = swipe.replace('swipe', '');
            // If the swipe direction is down/up but the toast came from top/bottom don't fade away
            // Unless the screen is small, then the toast always on bottom
            if (direction === 'down' && options.position.indexOf('top') != -1 && !isSmScreen || direction === 'up' && (options.position.indexOf('bottom') != -1 || isSmScreen)) {
              return;
            }
            if ((direction === 'left' || direction === 'right') && isSmScreen) {
              return;
            }
            element.addClass('md-' + swipe);
            $mdUtil.nextTick($mdToast.cancel);
          };
          options.openClass = toastOpenClass(options.position);
          // 'top left' -> 'md-top md-left'
          options.parent.addClass(options.openClass);
          // static is the default position
          if ($mdUtil.hasComputedStyle(options.parent, 'position', 'static')) {
            options.parent.css('position', 'relative');
          }
          element.on(SWIPE_EVENTS, options.onSwipe);
          element.addClass(isSmScreen ? 'md-bottom' : options.position.split(' ').map(function (pos) {
            return 'md-' + pos;
          }).join(' '));
          if (options.parent)
            options.parent.addClass('md-toast-animating');
          return $animate.enter(element, options.parent).then(function () {
            if (options.parent)
              options.parent.removeClass('md-toast-animating');
          });
        }
        function onRemove(scope, element, options) {
          element.off(SWIPE_EVENTS, options.onSwipe);
          if (options.parent)
            options.parent.addClass('md-toast-animating');
          if (options.openClass)
            options.parent.removeClass(options.openClass);
          return (options.$destroy == true ? element.remove() : $animate.leave(element)).then(function () {
            if (options.parent)
              options.parent.removeClass('md-toast-animating');
            if ($mdUtil.hasComputedStyle(options.parent, 'position', 'static')) {
              options.parent.css('position', '');
            }
          });
        }
        function toastOpenClass(position) {
          if (!$mdMedia('gt-sm')) {
            return 'md-toast-open-bottom';
          }
          return 'md-toast-open-' + (position.indexOf('top') > -1 ? 'top' : 'bottom');
        }
      }
    }
    MdToastProvider.$inject = ['$$interimElementProvider'];
  }());
  (function () {
    'use strict';
    /**
 * @ngdoc module
 * @name material.components.toolbar
 */
    angular.module('material.components.toolbar', [
      'material.core',
      'material.components.content'
    ]).directive('mdToolbar', mdToolbarDirective);
    /**
 * @ngdoc directive
 * @name mdToolbar
 * @module material.components.toolbar
 * @restrict E
 * @description
 * `md-toolbar` is used to place a toolbar in your app.
 *
 * Toolbars are usually used above a content area to display the title of the
 * current page, and show relevant action buttons for that page.
 *
 * You can change the height of the toolbar by adding either the
 * `md-medium-tall` or `md-tall` class to the toolbar.
 *
 * @usage
 * <hljs lang="html">
 * <div layout="column" layout-fill>
 *   <md-toolbar>
 *
 *     <div class="md-toolbar-tools">
 *       <span>My App's Title</span>
 *
 *       <!-- fill up the space between left and right area -->
 *       <span flex></span>
 *
 *       <md-button>
 *         Right Bar Button
 *       </md-button>
 *     </div>
 *
 *   </md-toolbar>
 *   <md-content>
 *     Hello!
 *   </md-content>
 * </div>
 * </hljs>
 *
 * @param {boolean=} md-scroll-shrink Whether the header should shrink away as
 * the user scrolls down, and reveal itself as the user scrolls up.
 *
 * _**Note (1):** for scrollShrink to work, the toolbar must be a sibling of a
 * `md-content` element, placed before it. See the scroll shrink demo._
 *
 * _**Note (2):** The `md-scroll-shrink` attribute is only parsed on component
 * initialization, it does not watch for scope changes._
 *
 *
 * @param {number=} md-shrink-speed-factor How much to change the speed of the toolbar's
 * shrinking by. For example, if 0.25 is given then the toolbar will shrink
 * at one fourth the rate at which the user scrolls down. Default 0.5.
 */
    function mdToolbarDirective($$rAF, $mdConstant, $mdUtil, $mdTheming, $animate) {
      var translateY = angular.bind(null, $mdUtil.supplant, 'translate3d(0,{0}px,0)');
      return {
        template: '',
        restrict: 'E',
        link: function (scope, element, attr) {
          $mdTheming(element);
          if (angular.isDefined(attr.mdScrollShrink)) {
            setupScrollShrink();
          }
          function setupScrollShrink() {
            var toolbarHeight;
            var contentElement;
            var disableScrollShrink = angular.noop;
            // Current "y" position of scroll
            // Store the last scroll top position
            var y = 0;
            var prevScrollTop = 0;
            var shrinkSpeedFactor = attr.mdShrinkSpeedFactor || 0.5;
            var debouncedContentScroll = $$rAF.throttle(onContentScroll);
            var debouncedUpdateHeight = $mdUtil.debounce(updateToolbarHeight, 5 * 1000);
            // Wait for $mdContentLoaded event from mdContent directive.
            // If the mdContent element is a sibling of our toolbar, hook it up
            // to scroll events.
            scope.$on('$mdContentLoaded', onMdContentLoad);
            // If the toolbar is used inside an ng-if statement, we may miss the
            // $mdContentLoaded event, so we attempt to fake it if we have a
            // md-content close enough.
            attr.$observe('mdScrollShrink', onChangeScrollShrink);
            // If the toolbar has ngShow or ngHide we need to update height immediately as it changed
            // and not wait for $mdUtil.debounce to happen
            if (attr.ngShow) {
              scope.$watch(attr.ngShow, updateToolbarHeight);
            }
            if (attr.ngHide) {
              scope.$watch(attr.ngHide, updateToolbarHeight);
            }
            // If the scope is destroyed (which could happen with ng-if), make sure
            // to disable scroll shrinking again
            scope.$on('$destroy', disableScrollShrink);
            /**
         *
         */
            function onChangeScrollShrink(shrinkWithScroll) {
              var closestContent = element.parent().find('md-content');
              // If we have a content element, fake the call; this might still fail
              // if the content element isn't a sibling of the toolbar
              if (!contentElement && closestContent.length) {
                onMdContentLoad(null, closestContent);
              }
              // Evaluate the expression
              shrinkWithScroll = scope.$eval(shrinkWithScroll);
              // Disable only if the attribute's expression evaluates to false
              if (shrinkWithScroll === false) {
                disableScrollShrink();
              } else {
                disableScrollShrink = enableScrollShrink();
              }
            }
            /**
         *
         */
            function onMdContentLoad($event, newContentEl) {
              // Toolbar and content must be siblings
              if (newContentEl && element.parent()[0] === newContentEl.parent()[0]) {
                // unhook old content event listener if exists
                if (contentElement) {
                  contentElement.off('scroll', debouncedContentScroll);
                }
                contentElement = newContentEl;
                disableScrollShrink = enableScrollShrink();
              }
            }
            /**
         *
         */
            function onContentScroll(e) {
              var scrollTop = e ? e.target.scrollTop : prevScrollTop;
              debouncedUpdateHeight();
              y = Math.min(toolbarHeight / shrinkSpeedFactor, Math.max(0, y + scrollTop - prevScrollTop));
              element.css($mdConstant.CSS.TRANSFORM, translateY([-y * shrinkSpeedFactor]));
              contentElement.css($mdConstant.CSS.TRANSFORM, translateY([(toolbarHeight - y) * shrinkSpeedFactor]));
              prevScrollTop = scrollTop;
              $mdUtil.nextTick(function () {
                var hasWhiteFrame = element.hasClass('md-whiteframe-z1');
                if (hasWhiteFrame && !y) {
                  $animate.removeClass(element, 'md-whiteframe-z1');
                } else if (!hasWhiteFrame && y) {
                  $animate.addClass(element, 'md-whiteframe-z1');
                }
              });
            }
            /**
         *
         */
            function enableScrollShrink() {
              if (!contentElement)
                return angular.noop;
              // no md-content
              contentElement.on('scroll', debouncedContentScroll);
              contentElement.attr('scroll-shrink', 'true');
              $$rAF(updateToolbarHeight);
              return function disableScrollShrink() {
                contentElement.off('scroll', debouncedContentScroll);
                contentElement.attr('scroll-shrink', 'false');
                $$rAF(updateToolbarHeight);
              };
            }
            /**
         *
         */
            function updateToolbarHeight() {
              toolbarHeight = element.prop('offsetHeight');
              // Add a negative margin-top the size of the toolbar to the content el.
              // The content will start transformed down the toolbarHeight amount,
              // so everything looks normal.
              //
              // As the user scrolls down, the content will be transformed up slowly
              // to put the content underneath where the toolbar was.
              var margin = -toolbarHeight * shrinkSpeedFactor + 'px';
              contentElement.css({
                'margin-top': margin,
                'margin-bottom': margin
              });
              onContentScroll();
            }
          }
        }
      };
    }
    mdToolbarDirective.$inject = [
      '$$rAF',
      '$mdConstant',
      '$mdUtil',
      '$mdTheming',
      '$animate'
    ];
  }());
  (function () {
    'use strict';
    /**
 * @ngdoc module
 * @name material.components.virtualRepeat
 */
    angular.module('material.components.virtualRepeat', [
      'material.core',
      'material.components.showHide'
    ]).directive('mdVirtualRepeatContainer', VirtualRepeatContainerDirective).directive('mdVirtualRepeat', VirtualRepeatDirective);
    /**
 * @ngdoc directive
 * @name mdVirtualRepeatContainer
 * @module material.components.virtualRepeat
 * @restrict E
 * @description
 * `md-virtual-repeat-container` provides the scroll container for md-virtual-repeat.
 *
 * Virtual repeat is a limited substitute for ng-repeat that renders only
 * enough dom nodes to fill the container and recycling them as the user scrolls.
 *
 * @usage
 * <hljs lang="html">
 *
 * <md-virtual-repeat-container md-top-index="topIndex">
 *   <div md-virtual-repeat="i in items" md-item-size="20">Hello {{i}}!</div>
 * </md-virtual-repeat-container>
 * </hljs>
 *
 * @param {number=} md-top-index Binds the index of the item that is at the top of the scroll
 *     container to $scope. It can both read and set the scroll position.
 * @param {boolean=} md-orient-horizontal Whether the container should scroll horizontally
 *     (defaults to orientation and scrolling vertically).
 * @param {boolean=} md-auto-shrink When present, the container will shrink to fit
 *     the number of items when that number is less than its original size.
 * @param {number=} md-auto-shrink-min Minimum number of items that md-auto-shrink
 *     will shrink to (default: 0).
 */
    function VirtualRepeatContainerDirective() {
      return {
        controller: VirtualRepeatContainerController,
        template: virtualRepeatContainerTemplate,
        compile: function virtualRepeatContainerCompile($element, $attrs) {
          $element.addClass('md-virtual-repeat-container').addClass($attrs.hasOwnProperty('mdOrientHorizontal') ? 'md-orient-horizontal' : 'md-orient-vertical');
        }
      };
    }
    function virtualRepeatContainerTemplate($element) {
      return '<div class="md-virtual-repeat-scroller">' + '<div class="md-virtual-repeat-sizer"></div>' + '<div class="md-virtual-repeat-offsetter">' + $element[0].innerHTML + '</div></div>';
    }
    /**
 * Maximum size, in pixels, that can be explicitly set to an element. The actual value varies
 * between browsers, but IE11 has the very lowest size at a mere 1,533,917px. Ideally we could
 * *compute* this value, but Firefox always reports an element to have a size of zero if it
 * goes over the max, meaning that we'd have to binary search for the value.
 * @const {number}
 */
    var MAX_ELEMENT_SIZE = 1533917;
    /**
 * Number of additional elements to render above and below the visible area inside
 * of the virtual repeat container. A higher number results in less flicker when scrolling
 * very quickly in Safari, but comes with a higher rendering and dirty-checking cost.
 * @const {number}
 */
    var NUM_EXTRA = 3;
    /** @ngInject */
    function VirtualRepeatContainerController($$rAF, $mdUtil, $parse, $rootScope, $window, $scope, $element, $attrs) {
      this.$rootScope = $rootScope;
      this.$scope = $scope;
      this.$element = $element;
      this.$attrs = $attrs;
      /** @type {number} The width or height of the container */
      this.size = 0;
      /** @type {number} The scroll width or height of the scroller */
      this.scrollSize = 0;
      /** @type {number} The scrollLeft or scrollTop of the scroller */
      this.scrollOffset = 0;
      /** @type {boolean} Whether the scroller is oriented horizontally */
      this.horizontal = this.$attrs.hasOwnProperty('mdOrientHorizontal');
      /** @type {!VirtualRepeatController} The repeater inside of this container */
      this.repeater = null;
      /** @type {boolean} Whether auto-shrink is enabled */
      this.autoShrink = this.$attrs.hasOwnProperty('mdAutoShrink');
      /** @type {number} Minimum number of items to auto-shrink to */
      this.autoShrinkMin = parseInt(this.$attrs.mdAutoShrinkMin, 10) || 0;
      /** @type {?number} Original container size when shrank */
      this.originalSize = null;
      /** @type {number} Amount to offset the total scroll size by. */
      this.offsetSize = parseInt(this.$attrs.mdOffsetSize, 10) || 0;
      if (this.$attrs.mdTopIndex) {
        /** @type {function(angular.Scope): number} Binds to topIndex on Angular scope */
        this.bindTopIndex = $parse(this.$attrs.mdTopIndex);
        /** @type {number} The index of the item that is at the top of the scroll container */
        this.topIndex = this.bindTopIndex(this.$scope);
        if (!angular.isDefined(this.topIndex)) {
          this.topIndex = 0;
          this.bindTopIndex.assign(this.$scope, 0);
        }
        this.$scope.$watch(this.bindTopIndex, angular.bind(this, function (newIndex) {
          if (newIndex !== this.topIndex) {
            this.scrollToIndex(newIndex);
          }
        }));
      } else {
        this.topIndex = 0;
      }
      this.scroller = $element[0].getElementsByClassName('md-virtual-repeat-scroller')[0];
      this.sizer = this.scroller.getElementsByClassName('md-virtual-repeat-sizer')[0];
      this.offsetter = this.scroller.getElementsByClassName('md-virtual-repeat-offsetter')[0];
      // After the dom stablizes, measure the initial size of the container and
      // make a best effort at re-measuring as it changes.
      var boundUpdateSize = angular.bind(this, this.updateSize);
      $$rAF(angular.bind(this, function () {
        boundUpdateSize();
        var debouncedUpdateSize = $mdUtil.debounce(boundUpdateSize, 10, null, false);
        var jWindow = angular.element($window);
        // Make one more attempt to get the size if it is 0.
        // This is not by any means a perfect approach, but there's really no
        // silver bullet here.
        if (!this.size) {
          debouncedUpdateSize();
        }
        jWindow.on('resize', debouncedUpdateSize);
        $scope.$on('$destroy', function () {
          jWindow.off('resize', debouncedUpdateSize);
        });
        $scope.$emit('$md-resize-enable');
        $scope.$on('$md-resize', boundUpdateSize);
      }));
    }
    VirtualRepeatContainerController.$inject = [
      '$$rAF',
      '$mdUtil',
      '$parse',
      '$rootScope',
      '$window',
      '$scope',
      '$element',
      '$attrs'
    ];
    /** Called by the md-virtual-repeat inside of the container at startup. */
    VirtualRepeatContainerController.prototype.register = function (repeaterCtrl) {
      this.repeater = repeaterCtrl;
      angular.element(this.scroller).on('scroll wheel touchmove touchend', angular.bind(this, this.handleScroll_));
    };
    /** @return {boolean} Whether the container is configured for horizontal scrolling. */
    VirtualRepeatContainerController.prototype.isHorizontal = function () {
      return this.horizontal;
    };
    /** @return {number} The size (width or height) of the container. */
    VirtualRepeatContainerController.prototype.getSize = function () {
      return this.size;
    };
    /**
 * Resizes the container.
 * @private
 * @param {number} The new size to set.
 */
    VirtualRepeatContainerController.prototype.setSize_ = function (size) {
      this.size = size;
      this.$element[0].style[this.isHorizontal() ? 'width' : 'height'] = size + 'px';
    };
    /** Instructs the container to re-measure its size. */
    VirtualRepeatContainerController.prototype.updateSize = function () {
      if (this.originalSize)
        return;
      this.size = this.isHorizontal() ? this.$element[0].clientWidth : this.$element[0].clientHeight;
      this.repeater && this.repeater.containerUpdated();
    };
    /** @return {number} The container's scrollHeight or scrollWidth. */
    VirtualRepeatContainerController.prototype.getScrollSize = function () {
      return this.scrollSize;
    };
    /**
 * Sets the scroller element to the specified size.
 * @private
 * @param {number} size The new size.
 */
    VirtualRepeatContainerController.prototype.sizeScroller_ = function (size) {
      var dimension = this.isHorizontal() ? 'width' : 'height';
      var crossDimension = this.isHorizontal() ? 'height' : 'width';
      // Clear any existing dimensions.
      this.sizer.innerHTML = '';
      // If the size falls within the browser's maximum explicit size for a single element, we can
      // set the size and be done. Otherwise, we have to create children that add up the the desired
      // size.
      if (size < MAX_ELEMENT_SIZE) {
        this.sizer.style[dimension] = size + 'px';
      } else {
        this.sizer.style[dimension] = 'auto';
        this.sizer.style[crossDimension] = 'auto';
        // Divide the total size we have to render into N max-size pieces.
        var numChildren = Math.floor(size / MAX_ELEMENT_SIZE);
        // Element template to clone for each max-size piece.
        var sizerChild = document.createElement('div');
        sizerChild.style[dimension] = MAX_ELEMENT_SIZE + 'px';
        sizerChild.style[crossDimension] = '1px';
        for (var i = 0; i < numChildren; i++) {
          this.sizer.appendChild(sizerChild.cloneNode(false));
        }
        // Re-use the element template for the remainder.
        sizerChild.style[dimension] = size - numChildren * MAX_ELEMENT_SIZE + 'px';
        this.sizer.appendChild(sizerChild);
      }
    };
    /**
 * If auto-shrinking is enabled, shrinks or unshrinks as appropriate.
 * @private
 * @param {number} size The new size.
 */
    VirtualRepeatContainerController.prototype.autoShrink_ = function (size) {
      var shrinkSize = Math.max(size, this.autoShrinkMin * this.repeater.getItemSize());
      if (this.autoShrink && shrinkSize !== this.size) {
        var currentSize = this.originalSize || this.size;
        if (!currentSize || shrinkSize < currentSize) {
          if (!this.originalSize) {
            this.originalSize = this.size;
          }
          this.setSize_(shrinkSize);
        } else if (this.originalSize) {
          this.setSize_(this.originalSize);
          this.originalSize = null;
        }
        this.repeater.containerUpdated();
      }
    };
    /**
 * Sets the scrollHeight or scrollWidth. Called by the repeater based on
 * its item count and item size.
 * @param {number} itemsSize The total size of the items.
 */
    VirtualRepeatContainerController.prototype.setScrollSize = function (itemsSize) {
      var size = itemsSize + this.offsetSize;
      if (this.scrollSize === size)
        return;
      this.sizeScroller_(size);
      this.autoShrink_(size);
      this.scrollSize = size;
    };
    /** @return {number} The container's current scroll offset. */
    VirtualRepeatContainerController.prototype.getScrollOffset = function () {
      return this.scrollOffset;
    };
    /**
 * Scrolls to a given scrollTop position.
 * @param {number} position
 */
    VirtualRepeatContainerController.prototype.scrollTo = function (position) {
      this.scroller[this.isHorizontal() ? 'scrollLeft' : 'scrollTop'] = position;
      this.handleScroll_();
    };
    /**
 * Scrolls the item with the given index to the top of the scroll container.
 * @param {number} index
 */
    VirtualRepeatContainerController.prototype.scrollToIndex = function (index) {
      var itemSize = this.repeater.getItemSize();
      var itemsLength = this.repeater.itemsLength;
      if (index > itemsLength) {
        index = itemsLength - 1;
      }
      this.scrollTo(itemSize * index);
    };
    VirtualRepeatContainerController.prototype.resetScroll = function () {
      this.scrollTo(0);
    };
    VirtualRepeatContainerController.prototype.handleScroll_ = function () {
      var offset = this.isHorizontal() ? this.scroller.scrollLeft : this.scroller.scrollTop;
      if (offset === this.scrollOffset)
        return;
      var itemSize = this.repeater.getItemSize();
      if (!itemSize)
        return;
      var numItems = Math.max(0, Math.floor(offset / itemSize) - NUM_EXTRA);
      var transform = this.isHorizontal() ? 'translateX(' : 'translateY(';
      transform += numItems * itemSize + 'px)';
      this.scrollOffset = offset;
      this.offsetter.style.webkitTransform = transform;
      this.offsetter.style.transform = transform;
      if (this.bindTopIndex) {
        var topIndex = Math.floor(offset / itemSize);
        if (topIndex !== this.topIndex && topIndex < this.repeater.itemsLength) {
          this.topIndex = topIndex;
          this.bindTopIndex.assign(this.$scope, topIndex);
          if (!this.$rootScope.$$phase)
            this.$scope.$digest();
        }
      }
      this.repeater.containerUpdated();
    };
    /**
 * @ngdoc directive
 * @name mdVirtualRepeat
 * @module material.components.virtualRepeat
 * @restrict A
 * @priority 1000
 * @description
 * `md-virtual-repeat` specifies an element to repeat using virtual scrolling.
 *
 * Virtual repeat is a limited substitute for ng-repeat that renders only
 * enough dom nodes to fill the container and recycling them as the user scrolls.
 * Arrays, but not objects are supported for iteration.
 * Track by, as alias, and (key, value) syntax are not supported.
 *
 * @usage
 * <hljs lang="html">
 * <md-virtual-repeat-container>
 *   <div md-virtual-repeat="i in items">Hello {{i}}!</div>
 * </md-virtual-repeat-container>
 *
 * <md-virtual-repeat-container md-orient-horizontal>
 *   <div md-virtual-repeat="i in items" md-item-size="20">Hello {{i}}!</div>
 * </md-virtual-repeat-container>
 * </hljs>
 *
 * @param {number=} md-item-size The height or width of the repeated elements (which must be
 *   identical for each element). Optional. Will attempt to read the size from the dom if missing,
 *   but still assumes that all repeated nodes have same height or width.
 * @param {string=} md-extra-name Evaluates to an additional name to which the current iterated item
 *   can be assigned on the repeated scope (needed for use in `md-autocomplete`).
 * @param {boolean=} md-on-demand When present, treats the md-virtual-repeat argument as an object
 *   that can fetch rows rather than an array.
 *
 *   **NOTE:** This object must implement the following interface with two (2) methods:
 *
 *   - `getItemAtIndex: function(index) [object]` The item at that index or null if it is not yet
 *     loaded (it should start downloading the item in that case).
 *   - `getLength: function() [number]` The data length to which the repeater container
 *     should be sized. Ideally, when the count is known, this method should return it.
 *     Otherwise, return a higher number than the currently loaded items to produce an
 *     infinite-scroll behavior.
 */
    function VirtualRepeatDirective($parse) {
      return {
        controller: VirtualRepeatController,
        priority: 1000,
        require: [
          'mdVirtualRepeat',
          '^^mdVirtualRepeatContainer'
        ],
        restrict: 'A',
        terminal: true,
        transclude: 'element',
        compile: function VirtualRepeatCompile($element, $attrs) {
          var expression = $attrs.mdVirtualRepeat;
          var match = expression.match(/^\s*([\s\S]+?)\s+in\s+([\s\S]+?)\s*$/);
          var repeatName = match[1];
          var repeatListExpression = $parse(match[2]);
          var extraName = $attrs.mdExtraName && $parse($attrs.mdExtraName);
          return function VirtualRepeatLink($scope, $element, $attrs, ctrl, $transclude) {
            ctrl[0].link_(ctrl[1], $transclude, repeatName, repeatListExpression, extraName);
          };
        }
      };
    }
    VirtualRepeatDirective.$inject = ['$parse'];
    /** @ngInject */
    function VirtualRepeatController($scope, $element, $attrs, $browser, $document, $rootScope, $$rAF) {
      this.$scope = $scope;
      this.$element = $element;
      this.$attrs = $attrs;
      this.$browser = $browser;
      this.$document = $document;
      this.$rootScope = $rootScope;
      this.$$rAF = $$rAF;
      /** @type {boolean} Whether we are in on-demand mode. */
      this.onDemand = $attrs.hasOwnProperty('mdOnDemand');
      /** @type {!Function} Backup reference to $browser.$$checkUrlChange */
      this.browserCheckUrlChange = $browser.$$checkUrlChange;
      /** @type {number} Most recent starting repeat index (based on scroll offset) */
      this.newStartIndex = 0;
      /** @type {number} Most recent ending repeat index (based on scroll offset) */
      this.newEndIndex = 0;
      /** @type {number} Most recent end visible index (based on scroll offset) */
      this.newVisibleEnd = 0;
      /** @type {number} Previous starting repeat index (based on scroll offset) */
      this.startIndex = 0;
      /** @type {number} Previous ending repeat index (based on scroll offset) */
      this.endIndex = 0;
      // TODO: measure width/height of first element from dom if not provided.
      // getComputedStyle?
      /** @type {?number} Height/width of repeated elements. */
      this.itemSize = $scope.$eval($attrs.mdItemSize) || null;
      /** @type {boolean} Whether this is the first time that items are rendered. */
      this.isFirstRender = true;
      /**
   * @private {boolean} Whether the items in the list are already being updated. Used to prevent
   *     nested calls to virtualRepeatUpdate_.
   */
      this.isVirtualRepeatUpdating_ = false;
      /** @type {number} Most recently seen length of items. */
      this.itemsLength = 0;
      /**
   * @type {!Function} Unwatch callback for item size (when md-items-size is
   *     not specified), or angular.noop otherwise.
   */
      this.unwatchItemSize_ = angular.noop;
      /**
   * Presently rendered blocks by repeat index.
   * @type {Object<number, !VirtualRepeatController.Block}
   */
      this.blocks = {};
      /** @type {Array<!VirtualRepeatController.Block>} A pool of presently unused blocks. */
      this.pooledBlocks = [];
    }
    VirtualRepeatController.$inject = [
      '$scope',
      '$element',
      '$attrs',
      '$browser',
      '$document',
      '$rootScope',
      '$$rAF'
    ];
    /**
 * An object representing a repeated item.
 * @typedef {{element: !jqLite, new: boolean, scope: !angular.Scope}}
 */
    VirtualRepeatController.Block;
    /**
 * Called at startup by the md-virtual-repeat postLink function.
 * @param {!VirtualRepeatContainerController} container The container's controller.
 * @param {!Function} transclude The repeated element's bound transclude function.
 * @param {string} repeatName The left hand side of the repeat expression, indicating
 *     the name for each item in the array.
 * @param {!Function} repeatListExpression A compiled expression based on the right hand side
 *     of the repeat expression. Points to the array to repeat over.
 * @param {string|undefined} extraName The optional extra repeatName.
 */
    VirtualRepeatController.prototype.link_ = function (container, transclude, repeatName, repeatListExpression, extraName) {
      this.container = container;
      this.transclude = transclude;
      this.repeatName = repeatName;
      this.rawRepeatListExpression = repeatListExpression;
      this.extraName = extraName;
      this.sized = false;
      this.repeatListExpression = angular.bind(this, this.repeatListExpression_);
      this.container.register(this);
    };
    /** @private Attempts to set itemSize by measuring a repeated element in the dom */
    VirtualRepeatController.prototype.readItemSize_ = function () {
      if (this.itemSize) {
        // itemSize was successfully read in a different asynchronous call.
        return;
      }
      this.items = this.repeatListExpression(this.$scope);
      this.parentNode = this.$element[0].parentNode;
      var block = this.getBlock_(0);
      if (!block.element[0].parentNode) {
        this.parentNode.appendChild(block.element[0]);
      }
      this.itemSize = block.element[0][this.container.isHorizontal() ? 'offsetWidth' : 'offsetHeight'] || null;
      this.blocks[0] = block;
      this.poolBlock_(0);
      if (this.itemSize) {
        this.containerUpdated();
      }
    };
    /**
 * Returns the user-specified repeat list, transforming it into an array-like
 * object in the case of infinite scroll/dynamic load mode.
 * @param {!angular.Scope} The scope.
 * @return {!Array|!Object} An array or array-like object for iteration.
 */
    VirtualRepeatController.prototype.repeatListExpression_ = function (scope) {
      var repeatList = this.rawRepeatListExpression(scope);
      if (this.onDemand && repeatList) {
        var virtualList = new VirtualRepeatModelArrayLike(repeatList);
        virtualList.$$includeIndexes(this.newStartIndex, this.newVisibleEnd);
        return virtualList;
      } else {
        return repeatList;
      }
    };
    /**
 * Called by the container. Informs us that the containers scroll or size has
 * changed.
 */
    VirtualRepeatController.prototype.containerUpdated = function () {
      // If itemSize is unknown, attempt to measure it.
      if (!this.itemSize) {
        this.unwatchItemSize_ = this.$scope.$watchCollection(this.repeatListExpression, angular.bind(this, function (items) {
          if (items && items.length) {
            this.$$rAF(angular.bind(this, this.readItemSize_));
          }
        }));
        if (!this.$rootScope.$$phase)
          this.$scope.$digest();
        return;
      } else if (!this.sized) {
        this.items = this.repeatListExpression(this.$scope);
      }
      if (!this.sized) {
        this.unwatchItemSize_();
        this.sized = true;
        this.$scope.$watchCollection(this.repeatListExpression, angular.bind(this, function (items, oldItems) {
          if (!this.isVirtualRepeatUpdating_) {
            this.virtualRepeatUpdate_(items, oldItems);
          }
        }));
      }
      this.updateIndexes_();
      if (this.newStartIndex !== this.startIndex || this.newEndIndex !== this.endIndex || this.container.getScrollOffset() > this.container.getScrollSize()) {
        if (this.items instanceof VirtualRepeatModelArrayLike) {
          this.items.$$includeIndexes(this.newStartIndex, this.newEndIndex);
        }
        this.virtualRepeatUpdate_(this.items, this.items);
      }
    };
    /**
 * Called by the container. Returns the size of a single repeated item.
 * @return {?number} Size of a repeated item.
 */
    VirtualRepeatController.prototype.getItemSize = function () {
      return this.itemSize;
    };
    /**
 * Updates the order and visible offset of repeated blocks in response to scrolling
 * or items updates.
 * @private
 */
    VirtualRepeatController.prototype.virtualRepeatUpdate_ = function (items, oldItems) {
      this.isVirtualRepeatUpdating_ = true;
      var itemsLength = items && items.length || 0;
      var lengthChanged = false;
      // If the number of items shrank, scroll up to the top.
      if (this.items && itemsLength < this.items.length && this.container.getScrollOffset() !== 0) {
        this.items = items;
        this.container.resetScroll();
        return;
      }
      if (itemsLength !== this.itemsLength) {
        lengthChanged = true;
        this.itemsLength = itemsLength;
      }
      this.items = items;
      if (items !== oldItems || lengthChanged) {
        this.updateIndexes_();
      }
      this.parentNode = this.$element[0].parentNode;
      if (lengthChanged) {
        this.container.setScrollSize(itemsLength * this.itemSize);
      }
      if (this.isFirstRender) {
        this.isFirstRender = false;
        var startIndex = this.$attrs.mdStartIndex ? this.$scope.$eval(this.$attrs.mdStartIndex) : this.container.topIndex;
        this.container.scrollToIndex(startIndex);
      }
      // Detach and pool any blocks that are no longer in the viewport.
      Object.keys(this.blocks).forEach(function (blockIndex) {
        var index = parseInt(blockIndex, 10);
        if (index < this.newStartIndex || index >= this.newEndIndex) {
          this.poolBlock_(index);
        }
      }, this);
      // Add needed blocks.
      // For performance reasons, temporarily block browser url checks as we digest
      // the restored block scopes ($$checkUrlChange reads window.location to
      // check for changes and trigger route change, etc, which we don't need when
      // trying to scroll at 60fps).
      this.$browser.$$checkUrlChange = angular.noop;
      var i, block, newStartBlocks = [], newEndBlocks = [];
      // Collect blocks at the top.
      for (i = this.newStartIndex; i < this.newEndIndex && this.blocks[i] == null; i++) {
        block = this.getBlock_(i);
        this.updateBlock_(block, i);
        newStartBlocks.push(block);
      }
      // Update blocks that are already rendered.
      for (; this.blocks[i] != null; i++) {
        this.updateBlock_(this.blocks[i], i);
      }
      var maxIndex = i - 1;
      // Collect blocks at the end.
      for (; i < this.newEndIndex; i++) {
        block = this.getBlock_(i);
        this.updateBlock_(block, i);
        newEndBlocks.push(block);
      }
      // Attach collected blocks to the document.
      if (newStartBlocks.length) {
        this.parentNode.insertBefore(this.domFragmentFromBlocks_(newStartBlocks), this.$element[0].nextSibling);
      }
      if (newEndBlocks.length) {
        this.parentNode.insertBefore(this.domFragmentFromBlocks_(newEndBlocks), this.blocks[maxIndex] && this.blocks[maxIndex].element[0].nextSibling);
      }
      // Restore $$checkUrlChange.
      this.$browser.$$checkUrlChange = this.browserCheckUrlChange;
      this.startIndex = this.newStartIndex;
      this.endIndex = this.newEndIndex;
      this.isVirtualRepeatUpdating_ = false;
    };
    /**
 * @param {number} index Where the block is to be in the repeated list.
 * @return {!VirtualRepeatController.Block} A new or pooled block to place at the specified index.
 * @private
 */
    VirtualRepeatController.prototype.getBlock_ = function (index) {
      if (this.pooledBlocks.length) {
        return this.pooledBlocks.pop();
      }
      var block;
      this.transclude(angular.bind(this, function (clone, scope) {
        block = {
          element: clone,
          new: true,
          scope: scope
        };
        this.updateScope_(scope, index);
        this.parentNode.appendChild(clone[0]);
      }));
      return block;
    };
    /**
 * Updates and if not in a digest cycle, digests the specified block's scope to the data
 * at the specified index.
 * @param {!VirtualRepeatController.Block} block The block whose scope should be updated.
 * @param {number} index The index to set.
 * @private
 */
    VirtualRepeatController.prototype.updateBlock_ = function (block, index) {
      this.blocks[index] = block;
      if (!block.new && (block.scope.$index === index && block.scope[this.repeatName] === this.items[index])) {
        return;
      }
      block.new = false;
      // Update and digest the block's scope.
      this.updateScope_(block.scope, index);
      // Perform digest before reattaching the block.
      // Any resulting synchronous dom mutations should be much faster as a result.
      // This might break some directives, but I'm going to try it for now.
      if (!this.$rootScope.$$phase) {
        block.scope.$digest();
      }
    };
    /**
 * Updates scope to the data at the specified index.
 * @param {!angular.Scope} scope The scope which should be updated.
 * @param {number} index The index to set.
 * @private
 */
    VirtualRepeatController.prototype.updateScope_ = function (scope, index) {
      scope.$index = index;
      scope[this.repeatName] = this.items && this.items[index];
      if (this.extraName)
        scope[this.extraName(this.$scope)] = this.items[index];
    };
    /**
 * Pools the block at the specified index (Pulls its element out of the dom and stores it).
 * @param {number} index The index at which the block to pool is stored.
 * @private
 */
    VirtualRepeatController.prototype.poolBlock_ = function (index) {
      this.pooledBlocks.push(this.blocks[index]);
      this.parentNode.removeChild(this.blocks[index].element[0]);
      delete this.blocks[index];
    };
    /**
 * Produces a dom fragment containing the elements from the list of blocks.
 * @param {!Array<!VirtualRepeatController.Block>} blocks The blocks whose elements
 *     should be added to the document fragment.
 * @return {DocumentFragment}
 * @private
 */
    VirtualRepeatController.prototype.domFragmentFromBlocks_ = function (blocks) {
      var fragment = this.$document[0].createDocumentFragment();
      blocks.forEach(function (block) {
        fragment.appendChild(block.element[0]);
      });
      return fragment;
    };
    /**
 * Updates start and end indexes based on length of repeated items and container size.
 * @private
 */
    VirtualRepeatController.prototype.updateIndexes_ = function () {
      var itemsLength = this.items ? this.items.length : 0;
      var containerLength = Math.ceil(this.container.getSize() / this.itemSize);
      this.newStartIndex = Math.max(0, Math.min(itemsLength - containerLength, Math.floor(this.container.getScrollOffset() / this.itemSize)));
      this.newVisibleEnd = this.newStartIndex + containerLength + NUM_EXTRA;
      this.newEndIndex = Math.min(itemsLength, this.newVisibleEnd);
      this.newStartIndex = Math.max(0, this.newStartIndex - NUM_EXTRA);
    };
    /**
 * This VirtualRepeatModelArrayLike class enforces the interface requirements
 * for infinite scrolling within a mdVirtualRepeatContainer. An object with this
 * interface must implement the following interface with two (2) methods:
 *
 * getItemAtIndex: function(index) -> item at that index or null if it is not yet
 *     loaded (It should start downloading the item in that case).
 *
 * getLength: function() -> number The data legnth to which the repeater container
 *     should be sized. Ideally, when the count is known, this method should return it.
 *     Otherwise, return a higher number than the currently loaded items to produce an
 *     infinite-scroll behavior.
 *
 * @usage
 * <hljs lang="html">
 *  <md-virtual-repeat-container md-orient-horizontal>
 *    <div md-virtual-repeat="i in items" md-on-demand>
 *      Hello {{i}}!
 *    </div>
 *  </md-virtual-repeat-container>
 * </hljs>
 *
 */
    function VirtualRepeatModelArrayLike(model) {
      if (!angular.isFunction(model.getItemAtIndex) || !angular.isFunction(model.getLength)) {
        throw Error('When md-on-demand is enabled, the Object passed to md-virtual-repeat must implement ' + 'functions getItemAtIndex() and getLength() ');
      }
      this.model = model;
    }
    VirtualRepeatModelArrayLike.prototype.$$includeIndexes = function (start, end) {
      for (var i = start; i < end; i++) {
        if (!this.hasOwnProperty(i)) {
          this[i] = this.model.getItemAtIndex(i);
        }
      }
      this.length = this.model.getLength();
    };
    function abstractMethod() {
      throw Error('Non-overridden abstract method called.');
    }
  }());
  (function () {
    'use strict';
    /**
 * @ngdoc module
 * @name material.components.tooltip
 */
    angular.module('material.components.tooltip', ['material.core']).directive('mdTooltip', MdTooltipDirective);
    /**
 * @ngdoc directive
 * @name mdTooltip
 * @module material.components.tooltip
 * @description
 * Tooltips are used to describe elements that are interactive and primarily graphical (not textual).
 *
 * Place a `<md-tooltip>` as a child of the element it describes.
 *
 * A tooltip will activate when the user focuses, hovers over, or touches the parent.
 *
 * @usage
 * <hljs lang="html">
 * <md-button class="md-fab md-accent" aria-label="Play">
 *   <md-tooltip>
 *     Play Music
 *   </md-tooltip>
 *   <md-icon icon="img/icons/ic_play_arrow_24px.svg"></md-icon>
 * </md-button>
 * </hljs>
 *
 * @param {expression=} md-visible Boolean bound to whether the tooltip is currently visible.
 * @param {number=} md-delay How many milliseconds to wait to show the tooltip after the user focuses, hovers, or touches the parent. Defaults to 300ms.
 * @param {boolean=} md-autohide If present or provided with a boolean value, the tooltip will hide on mouse leave, regardless of focus
 * @param {string=} md-direction Which direction would you like the tooltip to go?  Supports left, right, top, and bottom.  Defaults to bottom.
 */
    function MdTooltipDirective($timeout, $window, $$rAF, $document, $mdUtil, $mdTheming, $rootElement, $animate, $q) {
      var TOOLTIP_SHOW_DELAY = 0;
      var TOOLTIP_WINDOW_EDGE_SPACE = 8;
      return {
        restrict: 'E',
        transclude: true,
        priority: 210,
        template: '<div class="md-content" ng-transclude></div>',
        scope: {
          delay: '=?mdDelay',
          visible: '=?mdVisible',
          autohide: '=?mdAutohide',
          direction: '@?mdDirection'
        },
        link: postLink
      };
      function postLink(scope, element, attr) {
        $mdTheming(element);
        var parent = $mdUtil.getParentWithPointerEvents(element), content = angular.element(element[0].getElementsByClassName('md-content')[0]), tooltipParent = angular.element(document.body), debouncedOnResize = $$rAF.throttle(function () {
            updatePosition();
          });
        if ($animate.pin)
          $animate.pin(element, parent);
        // Initialize element
        setDefaults();
        manipulateElement();
        bindEvents();
        // Default origin transform point is 'center top'
        // positionTooltip() is always relative to center top
        updateContentOrigin();
        configureWatchers();
        addAriaLabel();
        function setDefaults() {
          if (!angular.isDefined(attr.mdDelay))
            scope.delay = TOOLTIP_SHOW_DELAY;
        }
        function updateContentOrigin() {
          var origin = 'center top';
          switch (scope.direction) {
          case 'left':
            origin = 'right center';
            break;
          case 'right':
            origin = 'left center';
            break;
          case 'top':
            origin = 'center bottom';
            break;
          case 'bottom':
            origin = 'center top';
            break;
          }
          content.css('transform-origin', origin);
        }
        function configureWatchers() {
          scope.$on('$destroy', function () {
            scope.visible = false;
            element.remove();
            angular.element($window).off('resize', debouncedOnResize);
          });
          scope.$watch('visible', function (isVisible) {
            if (isVisible)
              showTooltip();
            else
              hideTooltip();
          });
          scope.$watch('direction', updatePosition);
        }
        function addAriaLabel() {
          if (!parent.attr('aria-label') && !parent.text().trim()) {
            parent.attr('aria-label', element.text().trim());
          }
        }
        function manipulateElement() {
          element.detach();
          element.attr('role', 'tooltip');
        }
        function bindEvents() {
          var mouseActive = false;
          var ngWindow = angular.element($window);
          // add an mutationObserver when there is support for it
          // and the need for it in the form of viable host(parent[0])
          if (parent[0] && 'MutationObserver' in $window) {
            // use an mutationObserver to tackle #2602
            var attributeObserver = new MutationObserver(function (mutations) {
                mutations.forEach(function (mutation) {
                  if (mutation.attributeName === 'disabled' && parent[0].disabled) {
                    setVisible(false);
                    scope.$digest();  // make sure the elements gets updated
                  }
                });
              });
            attributeObserver.observe(parent[0], { attributes: true });
          }
          // Store whether the element was focused when the window loses focus.
          var windowBlurHandler = function () {
            elementFocusedOnWindowBlur = document.activeElement === parent[0];
          };
          var elementFocusedOnWindowBlur = false;
          function windowScrollHandler() {
            setVisible(false);
          }
          ngWindow.on('blur', windowBlurHandler);
          ngWindow.on('resize', debouncedOnResize);
          document.addEventListener('scroll', windowScrollHandler, true);
          scope.$on('$destroy', function () {
            ngWindow.off('blur', windowBlurHandler);
            ngWindow.off('resize', debouncedOnResize);
            document.removeEventListener('scroll', windowScrollHandler, true);
            attributeObserver && attributeObserver.disconnect();
          });
          var enterHandler = function (e) {
            // Prevent the tooltip from showing when the window is receiving focus.
            if (e.type === 'focus' && elementFocusedOnWindowBlur) {
              elementFocusedOnWindowBlur = false;
              return;
            }
            parent.on('blur mouseleave touchend touchcancel', leaveHandler);
            setVisible(true);
          };
          var leaveHandler = function () {
            var autohide = scope.hasOwnProperty('autohide') ? scope.autohide : attr.hasOwnProperty('mdAutohide');
            if (autohide || mouseActive || $document[0].activeElement !== parent[0]) {
              parent.off('blur mouseleave touchend touchcancel', leaveHandler);
              parent.triggerHandler('blur');
              setVisible(false);
            }
            mouseActive = false;
          };
          // to avoid `synthetic clicks` we listen to mousedown instead of `click`
          parent.on('mousedown', function () {
            mouseActive = true;
          });
          parent.on('focus mouseenter touchstart', enterHandler);
        }
        function setVisible(value) {
          setVisible.value = !!value;
          if (!setVisible.queued) {
            if (value) {
              setVisible.queued = true;
              $timeout(function () {
                scope.visible = setVisible.value;
                setVisible.queued = false;
              }, scope.delay);
            } else {
              $mdUtil.nextTick(function () {
                scope.visible = false;
              });
            }
          }
        }
        function showTooltip() {
          // Insert the element before positioning it, so we can get the position
          // and check if we should display it
          tooltipParent.append(element);
          // Check if we should display it or not.
          // This handles hide-* and show-* along with any user defined css
          if ($mdUtil.hasComputedStyle(element, 'display', 'none')) {
            scope.visible = false;
            element.detach();
            return;
          }
          updatePosition();
          angular.forEach([
            element,
            content
          ], function (element) {
            $animate.addClass(element, 'md-show');
          });
        }
        function hideTooltip() {
          var promises = [];
          angular.forEach([
            element,
            content
          ], function (it) {
            if (it.parent() && it.hasClass('md-show')) {
              promises.push($animate.removeClass(it, 'md-show'));
            }
          });
          $q.all(promises).then(function () {
            if (!scope.visible)
              element.detach();
          });
        }
        function updatePosition() {
          if (!scope.visible)
            return;
          updateContentOrigin();
          positionTooltip();
        }
        function positionTooltip() {
          var tipRect = $mdUtil.offsetRect(element, tooltipParent);
          var parentRect = $mdUtil.offsetRect(parent, tooltipParent);
          var newPosition = getPosition(scope.direction);
          var offsetParent = element.prop('offsetParent');
          // If the user provided a direction, just nudge the tooltip onto the screen
          // Otherwise, recalculate based on 'top' since default is 'bottom'
          if (scope.direction) {
            newPosition = fitInParent(newPosition);
          } else if (offsetParent && newPosition.top > offsetParent.scrollHeight - tipRect.height - TOOLTIP_WINDOW_EDGE_SPACE) {
            newPosition = fitInParent(getPosition('top'));
          }
          element.css({
            left: newPosition.left + 'px',
            top: newPosition.top + 'px'
          });
          function fitInParent(pos) {
            var newPosition = {
                left: pos.left,
                top: pos.top
              };
            newPosition.left = Math.min(newPosition.left, tooltipParent.prop('scrollWidth') - tipRect.width - TOOLTIP_WINDOW_EDGE_SPACE);
            newPosition.left = Math.max(newPosition.left, TOOLTIP_WINDOW_EDGE_SPACE);
            newPosition.top = Math.min(newPosition.top, tooltipParent.prop('scrollHeight') - tipRect.height - TOOLTIP_WINDOW_EDGE_SPACE);
            newPosition.top = Math.max(newPosition.top, TOOLTIP_WINDOW_EDGE_SPACE);
            return newPosition;
          }
          function getPosition(dir) {
            return dir === 'left' ? {
              left: parentRect.left - tipRect.width - TOOLTIP_WINDOW_EDGE_SPACE,
              top: parentRect.top + parentRect.height / 2 - tipRect.height / 2
            } : dir === 'right' ? {
              left: parentRect.left + parentRect.width + TOOLTIP_WINDOW_EDGE_SPACE,
              top: parentRect.top + parentRect.height / 2 - tipRect.height / 2
            } : dir === 'top' ? {
              left: parentRect.left + parentRect.width / 2 - tipRect.width / 2,
              top: parentRect.top - tipRect.height - TOOLTIP_WINDOW_EDGE_SPACE
            } : {
              left: parentRect.left + parentRect.width / 2 - tipRect.width / 2,
              top: parentRect.top + parentRect.height + TOOLTIP_WINDOW_EDGE_SPACE
            };
          }
        }
      }
    }
    MdTooltipDirective.$inject = [
      '$timeout',
      '$window',
      '$$rAF',
      '$document',
      '$mdUtil',
      '$mdTheming',
      '$rootElement',
      '$animate',
      '$q'
    ];
  }());
  (function () {
    'use strict';
    /**
 * @ngdoc module
 * @name material.components.whiteframe
 */
    angular.module('material.components.whiteframe', []);
  }());
  (function () {
    'use strict';
    angular.module('material.components.autocomplete').controller('MdAutocompleteCtrl', MdAutocompleteCtrl);
    var ITEM_HEIGHT = 41, MAX_HEIGHT = 5.5 * ITEM_HEIGHT, MENU_PADDING = 8, INPUT_PADDING = 2;
    // Padding provided by `md-input-container`
    function MdAutocompleteCtrl($scope, $element, $mdUtil, $mdConstant, $mdTheming, $window, $animate, $rootElement, $attrs, $q) {
      //-- private variables
      var ctrl = this, itemParts = $scope.itemsExpr.split(/ in /i), itemExpr = itemParts[1], elements = null, cache = {}, noBlur = false, selectedItemWatchers = [], hasFocus = false, lastCount = 0;
      //-- public variables with handlers
      defineProperty('hidden', handleHiddenChange, true);
      //-- public variables
      ctrl.scope = $scope;
      ctrl.parent = $scope.$parent;
      ctrl.itemName = itemParts[0];
      ctrl.matches = [];
      ctrl.loading = false;
      ctrl.hidden = true;
      ctrl.index = null;
      ctrl.messages = [];
      ctrl.id = $mdUtil.nextUid();
      ctrl.isDisabled = null;
      ctrl.isRequired = null;
      ctrl.hasNotFound = false;
      //-- public methods
      ctrl.keydown = keydown;
      ctrl.blur = blur;
      ctrl.focus = focus;
      ctrl.clear = clearValue;
      ctrl.select = select;
      ctrl.listEnter = onListEnter;
      ctrl.listLeave = onListLeave;
      ctrl.mouseUp = onMouseup;
      ctrl.getCurrentDisplayValue = getCurrentDisplayValue;
      ctrl.registerSelectedItemWatcher = registerSelectedItemWatcher;
      ctrl.unregisterSelectedItemWatcher = unregisterSelectedItemWatcher;
      ctrl.notFoundVisible = notFoundVisible;
      ctrl.loadingIsVisible = loadingIsVisible;
      return init();
      //-- initialization methods
      /**
   * Initialize the controller, setup watchers, gather elements
   */
      function init() {
        $mdUtil.initOptionalProperties($scope, $attrs, {
          searchText: null,
          selectedItem: null
        });
        $mdTheming($element);
        configureWatchers();
        $mdUtil.nextTick(function () {
          gatherElements();
          moveDropdown();
          focusElement();
          $element.on('focus', focusElement);
        });
      }
      /**
   * Calculates the dropdown's position and applies the new styles to the menu element
   * @returns {*}
   */
      function positionDropdown() {
        if (!elements)
          return $mdUtil.nextTick(positionDropdown, false, $scope);
        var hrect = elements.wrap.getBoundingClientRect(), vrect = elements.snap.getBoundingClientRect(), root = elements.root.getBoundingClientRect(), top = vrect.bottom - root.top, bot = root.bottom - vrect.top, left = hrect.left - root.left, width = hrect.width, offset = getVerticalOffset(), styles;
        // Adjust the width to account for the padding provided by `md-input-container`
        if ($attrs.mdFloatingLabel) {
          left += INPUT_PADDING;
          width -= INPUT_PADDING * 2;
        }
        styles = {
          left: left + 'px',
          minWidth: width + 'px',
          maxWidth: Math.max(hrect.right - root.left, root.right - hrect.left) - MENU_PADDING + 'px'
        };
        if (top > bot && root.height - hrect.bottom - MENU_PADDING < MAX_HEIGHT) {
          styles.top = 'auto';
          styles.bottom = bot + 'px';
          styles.maxHeight = Math.min(MAX_HEIGHT, hrect.top - root.top - MENU_PADDING) + 'px';
        } else {
          styles.top = top - offset + 'px';
          styles.bottom = 'auto';
          styles.maxHeight = Math.min(MAX_HEIGHT, root.bottom + $mdUtil.scrollTop() - hrect.bottom - MENU_PADDING) + 'px';
        }
        elements.$.scrollContainer.css(styles);
        $mdUtil.nextTick(correctHorizontalAlignment, false);
        /**
     * Calculates the vertical offset for floating label examples to account for ngMessages
     * @returns {number}
     */
        function getVerticalOffset() {
          var offset = 0;
          var inputContainer = $element.find('md-input-container');
          if (inputContainer.length) {
            var input = inputContainer.find('input');
            offset = inputContainer.prop('offsetHeight');
            offset -= input.prop('offsetTop');
            offset -= input.prop('offsetHeight');
            // add in the height left up top for the floating label text
            offset += inputContainer.prop('offsetTop');
          }
          return offset;
        }
        /**
     * Makes sure that the menu doesn't go off of the screen on either side.
     */
        function correctHorizontalAlignment() {
          var dropdown = elements.scrollContainer.getBoundingClientRect(), styles = {};
          if (dropdown.right > root.right - MENU_PADDING) {
            styles.left = hrect.right - dropdown.width + 'px';
          }
          elements.$.scrollContainer.css(styles);
        }
      }
      /**
   * Moves the dropdown menu to the body tag in order to avoid z-index and overflow issues.
   */
      function moveDropdown() {
        if (!elements.$.root.length)
          return;
        $mdTheming(elements.$.scrollContainer);
        elements.$.scrollContainer.detach();
        elements.$.root.append(elements.$.scrollContainer);
        if ($animate.pin)
          $animate.pin(elements.$.scrollContainer, $rootElement);
      }
      /**
   * Sends focus to the input element.
   */
      function focusElement() {
        if ($scope.autofocus)
          elements.input.focus();
      }
      /**
   * Sets up any watchers used by autocomplete
   */
      function configureWatchers() {
        var wait = parseInt($scope.delay, 10) || 0;
        $attrs.$observe('disabled', function (value) {
          ctrl.isDisabled = !!value;
        });
        $attrs.$observe('required', function (value) {
          ctrl.isRequired = !!value;
        });
        $scope.$watch('searchText', wait ? $mdUtil.debounce(handleSearchText, wait) : handleSearchText);
        $scope.$watch('selectedItem', selectedItemChange);
        angular.element($window).on('resize', positionDropdown);
        $scope.$on('$destroy', cleanup);
      }
      /**
   * Removes any events or leftover elements created by this controller
   */
      function cleanup() {
        angular.element($window).off('resize', positionDropdown);
        if (elements) {
          var items = 'ul scroller scrollContainer input'.split(' ');
          angular.forEach(items, function (key) {
            elements.$[key].remove();
          });
        }
      }
      /**
   * Gathers all of the elements needed for this controller
   */
      function gatherElements() {
        elements = {
          main: $element[0],
          scrollContainer: $element[0].getElementsByClassName('md-virtual-repeat-container')[0],
          scroller: $element[0].getElementsByClassName('md-virtual-repeat-scroller')[0],
          ul: $element.find('ul')[0],
          input: $element.find('input')[0],
          wrap: $element.find('md-autocomplete-wrap')[0],
          root: document.body
        };
        elements.li = elements.ul.getElementsByTagName('li');
        elements.snap = getSnapTarget();
        elements.$ = getAngularElements(elements);
      }
      /**
   * Finds the element that the menu will base its position on
   * @returns {*}
   */
      function getSnapTarget() {
        for (var element = $element; element.length; element = element.parent()) {
          if (angular.isDefined(element.attr('md-autocomplete-snap')))
            return element[0];
        }
        return elements.wrap;
      }
      /**
   * Gathers angular-wrapped versions of each element
   * @param elements
   * @returns {{}}
   */
      function getAngularElements(elements) {
        var obj = {};
        for (var key in elements) {
          if (elements.hasOwnProperty(key))
            obj[key] = angular.element(elements[key]);
        }
        return obj;
      }
      //-- event/change handlers
      /**
   * Handles changes to the `hidden` property.
   * @param hidden
   * @param oldHidden
   */
      function handleHiddenChange(hidden, oldHidden) {
        if (!hidden && oldHidden) {
          positionDropdown();
          if (elements) {
            $mdUtil.nextTick(function () {
              $mdUtil.disableScrollAround(elements.ul);
            }, false, $scope);
          }
        } else if (hidden && !oldHidden) {
          $mdUtil.nextTick(function () {
            $mdUtil.enableScrolling();
          }, false, $scope);
        }
      }
      /**
   * When the user mouses over the dropdown menu, ignore blur events.
   */
      function onListEnter() {
        noBlur = true;
      }
      /**
   * When the user's mouse leaves the menu, blur events may hide the menu again.
   */
      function onListLeave() {
        if (!hasFocus)
          elements.input.focus();
        noBlur = false;
        ctrl.hidden = shouldHide();
      }
      /**
   * When the mouse button is released, send focus back to the input field.
   */
      function onMouseup() {
        elements.input.focus();
      }
      /**
   * Handles changes to the selected item.
   * @param selectedItem
   * @param previousSelectedItem
   */
      function selectedItemChange(selectedItem, previousSelectedItem) {
        if (selectedItem) {
          getDisplayValue(selectedItem).then(function (val) {
            $scope.searchText = val;
            handleSelectedItemChange(selectedItem, previousSelectedItem);
          });
        }
        if (selectedItem !== previousSelectedItem)
          announceItemChange();
      }
      /**
   * Use the user-defined expression to announce changes each time a new item is selected
   */
      function announceItemChange() {
        angular.isFunction($scope.itemChange) && $scope.itemChange(getItemAsNameVal($scope.selectedItem));
      }
      /**
   * Use the user-defined expression to announce changes each time the search text is changed
   */
      function announceTextChange() {
        angular.isFunction($scope.textChange) && $scope.textChange();
      }
      /**
   * Calls any external watchers listening for the selected item.  Used in conjunction with
   * `registerSelectedItemWatcher`.
   * @param selectedItem
   * @param previousSelectedItem
   */
      function handleSelectedItemChange(selectedItem, previousSelectedItem) {
        selectedItemWatchers.forEach(function (watcher) {
          watcher(selectedItem, previousSelectedItem);
        });
      }
      /**
   * Register a function to be called when the selected item changes.
   * @param cb
   */
      function registerSelectedItemWatcher(cb) {
        if (selectedItemWatchers.indexOf(cb) == -1) {
          selectedItemWatchers.push(cb);
        }
      }
      /**
   * Unregister a function previously registered for selected item changes.
   * @param cb
   */
      function unregisterSelectedItemWatcher(cb) {
        var i = selectedItemWatchers.indexOf(cb);
        if (i != -1) {
          selectedItemWatchers.splice(i, 1);
        }
      }
      /**
   * Handles changes to the searchText property.
   * @param searchText
   * @param previousSearchText
   */
      function handleSearchText(searchText, previousSearchText) {
        ctrl.index = getDefaultIndex();
        // do nothing on init
        if (searchText === previousSearchText)
          return;
        getDisplayValue($scope.selectedItem).then(function (val) {
          // clear selected item if search text no longer matches it
          if (searchText !== val) {
            $scope.selectedItem = null;
            // trigger change event if available
            if (searchText !== previousSearchText)
              announceTextChange();
            // cancel results if search text is not long enough
            if (!isMinLengthMet()) {
              ctrl.matches = [];
              setLoading(false);
              updateMessages();
            } else {
              handleQuery();
            }
          }
        });
      }
      /**
   * Handles input blur event, determines if the dropdown should hide.
   */
      function blur() {
        hasFocus = false;
        if (!noBlur) {
          ctrl.hidden = shouldHide();
        }
      }
      /**
   * Force blur on input element
   * @param forceBlur
   */
      function doBlur(forceBlur) {
        if (forceBlur) {
          noBlur = false;
          hasFocus = false;
        }
        elements.input.blur();
      }
      /**
   * Handles input focus event, determines if the dropdown should show.
   */
      function focus() {
        hasFocus = true;
        //-- if searchText is null, let's force it to be a string
        if (!angular.isString($scope.searchText))
          $scope.searchText = '';
        ctrl.hidden = shouldHide();
        if (!ctrl.hidden)
          handleQuery();
      }
      /**
   * Handles keyboard input.
   * @param event
   */
      function keydown(event) {
        switch (event.keyCode) {
        case $mdConstant.KEY_CODE.DOWN_ARROW:
          if (ctrl.loading)
            return;
          event.stopPropagation();
          event.preventDefault();
          ctrl.index = Math.min(ctrl.index + 1, ctrl.matches.length - 1);
          updateScroll();
          updateMessages();
          break;
        case $mdConstant.KEY_CODE.UP_ARROW:
          if (ctrl.loading)
            return;
          event.stopPropagation();
          event.preventDefault();
          ctrl.index = ctrl.index < 0 ? ctrl.matches.length - 1 : Math.max(0, ctrl.index - 1);
          updateScroll();
          updateMessages();
          break;
        case $mdConstant.KEY_CODE.TAB:
          // If we hit tab, assume that we've left the list so it will close
          onListLeave();
          if (ctrl.hidden || ctrl.loading || ctrl.index < 0 || ctrl.matches.length < 1)
            return;
          select(ctrl.index);
          break;
        case $mdConstant.KEY_CODE.ENTER:
          if (ctrl.hidden || ctrl.loading || ctrl.index < 0 || ctrl.matches.length < 1)
            return;
          if (hasSelection())
            return;
          event.stopPropagation();
          event.preventDefault();
          select(ctrl.index);
          break;
        case $mdConstant.KEY_CODE.ESCAPE:
          event.stopPropagation();
          event.preventDefault();
          clearValue();
          // Force the component to blur if they hit escape
          doBlur(true);
          break;
        default:
        }
      }
      //-- getters
      /**
   * Returns the minimum length needed to display the dropdown.
   * @returns {*}
   */
      function getMinLength() {
        return angular.isNumber($scope.minLength) ? $scope.minLength : 1;
      }
      /**
   * Returns the display value for an item.
   * @param item
   * @returns {*}
   */
      function getDisplayValue(item) {
        return $q.when(getItemText(item) || item);
        /**
     * Getter function to invoke user-defined expression (in the directive)
     * to convert your object to a single string.
     */
        function getItemText(item) {
          return item && $scope.itemText ? $scope.itemText(getItemAsNameVal(item)) : null;
        }
      }
      /**
   * Returns the locals object for compiling item templates.
   * @param item
   * @returns {{}}
   */
      function getItemAsNameVal(item) {
        if (!item)
          return undefined;
        var locals = {};
        if (ctrl.itemName)
          locals[ctrl.itemName] = item;
        return locals;
      }
      /**
   * Returns the default index based on whether or not autoselect is enabled.
   * @returns {number}
   */
      function getDefaultIndex() {
        return $scope.autoselect ? 0 : -1;
      }
      /**
   * Sets the loading parameter and updates the hidden state.
   * @param value {boolean} Whether or not the component is currently loading.
   */
      function setLoading(value) {
        if (ctrl.loading != value) {
          ctrl.loading = value;
        }
        // Always refresh the hidden variable as something else might have changed
        ctrl.hidden = shouldHide();
      }
      /**
   * Determines if the menu should be hidden.
   * @returns {boolean}
   */
      function shouldHide() {
        if (ctrl.loading && !hasMatches())
          return true;  // Hide while loading initial matches
        else if (hasSelection())
          return true;  // Hide if there is already a selection
        else if (!hasFocus)
          return true;  // Hide if the input does not have focus
        else
          return !shouldShow();  // Defer to standard show logic
      }
      /**
   * Determines if the menu should be shown.
   * @returns {boolean}
   */
      function shouldShow() {
        return isMinLengthMet() && hasMatches() || notFoundVisible();
      }
      /**
   * Returns true if the search text has matches.
   * @returns {boolean}
   */
      function hasMatches() {
        return ctrl.matches.length ? true : false;
      }
      /**
   * Returns true if the autocomplete has a valid selection.
   * @returns {boolean}
   */
      function hasSelection() {
        return ctrl.scope.selectedItem ? true : false;
      }
      /**
   * Returns true if the loading indicator is, or should be, visible.
   * @returns {boolean}
   */
      function loadingIsVisible() {
        return ctrl.loading && !hasSelection();
      }
      /**
   * Returns the display value of the current item.
   * @returns {*}
   */
      function getCurrentDisplayValue() {
        return getDisplayValue(ctrl.matches[ctrl.index]);
      }
      /**
   * Determines if the minimum length is met by the search text.
   * @returns {*}
   */
      function isMinLengthMet() {
        return ($scope.searchText || '').length >= getMinLength();
      }
      //-- actions
      /**
   * Defines a public property with a handler and a default value.
   * @param key
   * @param handler
   * @param value
   */
      function defineProperty(key, handler, value) {
        Object.defineProperty(ctrl, key, {
          get: function () {
            return value;
          },
          set: function (newValue) {
            var oldValue = value;
            value = newValue;
            handler(newValue, oldValue);
          }
        });
      }
      /**
   * Selects the item at the given index.
   * @param index
   */
      function select(index) {
        //-- force form to update state for validation
        $mdUtil.nextTick(function () {
          getDisplayValue(ctrl.matches[index]).then(function (val) {
            var ngModel = elements.$.input.controller('ngModel');
            ngModel.$setViewValue(val);
            ngModel.$render();
          }).finally(function () {
            $scope.selectedItem = ctrl.matches[index];
            setLoading(false);
          });
        }, false);
      }
      /**
   * Clears the searchText value and selected item.
   */
      function clearValue() {
        // Set the loading to true so we don't see flashes of content
        setLoading(true);
        // Reset our variables
        ctrl.index = 0;
        ctrl.matches = [];
        $scope.searchText = '';
        // Tell the select to fire and select nothing
        select(-1);
        // Per http://www.w3schools.com/jsref/event_oninput.asp
        var eventObj = document.createEvent('CustomEvent');
        eventObj.initCustomEvent('input', true, true, { value: $scope.searchText });
        elements.input.dispatchEvent(eventObj);
        elements.input.focus();
      }
      /**
   * Fetches the results for the provided search text.
   * @param searchText
   */
      function fetchResults(searchText) {
        var items = $scope.$parent.$eval(itemExpr), term = searchText.toLowerCase();
        if (angular.isArray(items)) {
          handleResults(items);
        } else if (items) {
          setLoading(true);
          $mdUtil.nextTick(function () {
            if (items.success)
              items.success(handleResults);
            if (items.then)
              items.then(handleResults);
            if (items.finally)
              items.finally(function () {
                setLoading(false);
              });
          }, true, $scope);
        }
        function handleResults(matches) {
          cache[term] = matches;
          if ((searchText || '') !== ($scope.searchText || ''))
            return;
          //-- just cache the results if old request
          ctrl.matches = matches;
          ctrl.hidden = shouldHide();
          if ($scope.selectOnMatch)
            selectItemOnMatch();
          updateMessages();
          positionDropdown();
        }
      }
      /**
   * Updates the ARIA messages
   */
      function updateMessages() {
        getCurrentDisplayValue().then(function (msg) {
          ctrl.messages = [
            getCountMessage(),
            msg
          ];
        });
      }
      /**
   * Returns the ARIA message for how many results match the current query.
   * @returns {*}
   */
      function getCountMessage() {
        if (lastCount === ctrl.matches.length)
          return '';
        lastCount = ctrl.matches.length;
        switch (ctrl.matches.length) {
        case 0:
          return 'There are no matches available.';
        case 1:
          return 'There is 1 match available.';
        default:
          return 'There are ' + ctrl.matches.length + ' matches available.';
        }
      }
      /**
   * Makes sure that the focused element is within view.
   */
      function updateScroll() {
        if (!elements.li[0])
          return;
        var height = elements.li[0].offsetHeight, top = height * ctrl.index, bot = top + height, hgt = elements.scroller.clientHeight, scrollTop = elements.scroller.scrollTop;
        if (top < scrollTop) {
          scrollTo(top);
        } else if (bot > scrollTop + hgt) {
          scrollTo(bot - hgt);
        }
      }
      function scrollTo(offset) {
        elements.$.scrollContainer.controller('mdVirtualRepeatContainer').scrollTo(offset);
      }
      function notFoundVisible() {
        var textLength = (ctrl.scope.searchText || '').length;
        return ctrl.hasNotFound && !hasMatches() && !ctrl.loading && textLength >= getMinLength() && hasFocus && !hasSelection();
      }
      /**
   * Starts the query to gather the results for the current searchText.  Attempts to return cached
   * results first, then forwards the process to `fetchResults` if necessary.
   */
      function handleQuery() {
        var searchText = $scope.searchText || '', term = searchText.toLowerCase();
        //-- if results are cached, pull in cached results
        if (!$scope.noCache && cache[term]) {
          ctrl.matches = cache[term];
          updateMessages();
        } else {
          fetchResults(searchText);
        }
        ctrl.hidden = shouldHide();
      }
      /**
   * If there is only one matching item and the search text matches its display value exactly,
   * automatically select that item.  Note: This function is only called if the user uses the
   * `md-select-on-match` flag.
   */
      function selectItemOnMatch() {
        var searchText = $scope.searchText, matches = ctrl.matches, item = matches[0];
        if (matches.length === 1)
          getDisplayValue(item).then(function (displayValue) {
            if (searchText == displayValue)
              select(0);
          });
      }
    }
    MdAutocompleteCtrl.$inject = [
      '$scope',
      '$element',
      '$mdUtil',
      '$mdConstant',
      '$mdTheming',
      '$window',
      '$animate',
      '$rootElement',
      '$attrs',
      '$q'
    ];
  }());
  (function () {
    'use strict';
    angular.module('material.components.autocomplete').directive('mdAutocomplete', MdAutocomplete);
    /**
 * @ngdoc directive
 * @name mdAutocomplete
 * @module material.components.autocomplete
 *
 * @description
 * `<md-autocomplete>` is a special input component with a drop-down of all possible matches to a
 *     custom query. This component allows you to provide real-time suggestions as the user types
 *     in the input area.
 *
 * To start, you will need to specify the required parameters and provide a template for your
 *     results. The content inside `md-autocomplete` will be treated as a template.
 *
 * In more complex cases, you may want to include other content such as a message to display when
 *     no matches were found.  You can do this by wrapping your template in `md-item-template` and
 *     adding a tag for `md-not-found`.  An example of this is shown below.
 *
 * ### Validation
 *
 * You can use `ng-messages` to include validation the same way that you would normally validate;
 *     however, if you want to replicate a standard input with a floating label, you will have to
 *     do the following:
 *
 * - Make sure that your template is wrapped in `md-item-template`
 * - Add your `ng-messages` code inside of `md-autocomplete`
 * - Add your validation properties to `md-autocomplete` (ie. `required`)
 * - Add a `name` to `md-autocomplete` (to be used on the generated `input`)
 *
 * There is an example below of how this should look.
 *
 *
 * @param {expression} md-items An expression in the format of `item in items` to iterate over
 *     matches for your search.
 * @param {expression=} md-selected-item-change An expression to be run each time a new item is
 *     selected
 * @param {expression=} md-search-text-change An expression to be run each time the search text
 *     updates
 * @param {expression=} md-search-text A model to bind the search query text to
 * @param {object=} md-selected-item A model to bind the selected item to
 * @param {expression=} md-item-text An expression that will convert your object to a single string.
 * @param {string=} placeholder Placeholder text that will be forwarded to the input.
 * @param {boolean=} md-no-cache Disables the internal caching that happens in autocomplete
 * @param {boolean=} ng-disabled Determines whether or not to disable the input field
 * @param {number=} md-min-length Specifies the minimum length of text before autocomplete will
 *     make suggestions
 * @param {number=} md-delay Specifies the amount of time (in milliseconds) to wait before looking
 *     for results
 * @param {boolean=} md-autofocus If true, will immediately focus the input element
 * @param {boolean=} md-autoselect If true, the first item will be selected by default
 * @param {string=} md-menu-class This will be applied to the dropdown menu for styling
 * @param {string=} md-floating-label This will add a floating label to autocomplete and wrap it in
 *     `md-input-container`
 * @param {string=} md-input-name The name attribute given to the input element to be used with
 *     FormController
 * @param {string=} md-input-id An ID to be added to the input element
 * @param {number=} md-input-minlength The minimum length for the input's value for validation
 * @param {number=} md-input-maxlength The maximum length for the input's value for validation
 * @param {boolean=} md-select-on-match When set, autocomplete will automatically select exact
 *     the item if the search text is an exact match
 *
 * @usage
 * ### Basic Example
 * <hljs lang="html">
 *   <md-autocomplete
 *       md-selected-item="selectedItem"
 *       md-search-text="searchText"
 *       md-items="item in getMatches(searchText)"
 *       md-item-text="item.display">
 *     <span md-highlight-text="searchText">{{item.display}}</span>
 *   </md-autocomplete>
 * </hljs>
 *
 * ### Example with "not found" message
 * <hljs lang="html">
 * <md-autocomplete
 *     md-selected-item="selectedItem"
 *     md-search-text="searchText"
 *     md-items="item in getMatches(searchText)"
 *     md-item-text="item.display">
 *   <md-item-template>
 *     <span md-highlight-text="searchText">{{item.display}}</span>
 *   </md-item-template>
 *   <md-not-found>
 *     No matches found.
 *   </md-not-found>
 * </md-autocomplete>
 * </hljs>
 *
 * In this example, our code utilizes `md-item-template` and `md-not-found` to specify the
 *     different parts that make up our component.
 *
 * ### Example with validation
 * <hljs lang="html">
 * <form name="autocompleteForm">
 *   <md-autocomplete
 *       required
 *       md-input-name="autocomplete"
 *       md-selected-item="selectedItem"
 *       md-search-text="searchText"
 *       md-items="item in getMatches(searchText)"
 *       md-item-text="item.display">
 *     <md-item-template>
 *       <span md-highlight-text="searchText">{{item.display}}</span>
 *     </md-item-template>
 *     <div ng-messages="autocompleteForm.autocomplete.$error">
 *       <div ng-message="required">This field is required</div>
 *     </div>
 *   </md-autocomplete>
 * </form>
 * </hljs>
 *
 * In this example, our code utilizes `md-item-template` and `md-not-found` to specify the
 *     different parts that make up our component.
 */
    function MdAutocomplete() {
      var hasNotFoundTemplate = false;
      return {
        controller: 'MdAutocompleteCtrl',
        controllerAs: '$mdAutocompleteCtrl',
        scope: {
          inputName: '@mdInputName',
          inputMinlength: '@mdInputMinlength',
          inputMaxlength: '@mdInputMaxlength',
          searchText: '=?mdSearchText',
          selectedItem: '=?mdSelectedItem',
          itemsExpr: '@mdItems',
          itemText: '&mdItemText',
          placeholder: '@placeholder',
          noCache: '=?mdNoCache',
          selectOnMatch: '=?mdSelectOnMatch',
          itemChange: '&?mdSelectedItemChange',
          textChange: '&?mdSearchTextChange',
          minLength: '=?mdMinLength',
          delay: '=?mdDelay',
          autofocus: '=?mdAutofocus',
          floatingLabel: '@?mdFloatingLabel',
          autoselect: '=?mdAutoselect',
          menuClass: '@?mdMenuClass',
          inputId: '@?mdInputId'
        },
        link: function (scope, element, attrs, controller) {
          controller.hasNotFound = hasNotFoundTemplate;
        },
        template: function (element, attr) {
          var noItemsTemplate = getNoItemsTemplate(), itemTemplate = getItemTemplate(), leftover = element.html(), tabindex = attr.tabindex;
          // Set our variable for the link function above which runs later
          hasNotFoundTemplate = noItemsTemplate ? true : false;
          if (!attr.hasOwnProperty('tabindex'))
            element.attr('tabindex', '-1');
          return '        <md-autocomplete-wrap            layout="row"            ng-class="{ \'md-whiteframe-z1\': !floatingLabel, \'md-menu-showing\': !$mdAutocompleteCtrl.hidden }"            role="listbox">          ' + getInputElement() + '          <md-progress-linear              class="' + (attr.mdFloatingLabel ? 'md-inline' : '') + '"              ng-if="$mdAutocompleteCtrl.loadingIsVisible()"              md-mode="indeterminate"></md-progress-linear>          <md-virtual-repeat-container              md-auto-shrink              md-auto-shrink-min="1"              ng-mouseenter="$mdAutocompleteCtrl.listEnter()"              ng-mouseleave="$mdAutocompleteCtrl.listLeave()"              ng-mouseup="$mdAutocompleteCtrl.mouseUp()"              ng-hide="$mdAutocompleteCtrl.hidden"              class="md-autocomplete-suggestions-container md-whiteframe-z1"              ng-class="{ \'md-not-found\': $mdAutocompleteCtrl.notFoundVisible() }"              role="presentation">            <ul class="md-autocomplete-suggestions"                ng-class="::menuClass"                id="ul-{{$mdAutocompleteCtrl.id}}">              <li md-virtual-repeat="item in $mdAutocompleteCtrl.matches"                  ng-class="{ selected: $index === $mdAutocompleteCtrl.index }"                  ng-click="$mdAutocompleteCtrl.select($index)"                  md-extra-name="$mdAutocompleteCtrl.itemName">                  ' + itemTemplate + '                  </li>' + noItemsTemplate + '            </ul>          </md-virtual-repeat-container>        </md-autocomplete-wrap>        <aria-status            class="md-visually-hidden"            role="status"            aria-live="assertive">          <p ng-repeat="message in $mdAutocompleteCtrl.messages track by $index" ng-if="message">{{message}}</p>        </aria-status>';
          function getItemTemplate() {
            var templateTag = element.find('md-item-template').detach(), html = templateTag.length ? templateTag.html() : element.html();
            if (!templateTag.length)
              element.empty();
            return '<md-autocomplete-parent-scope md-autocomplete-replace>' + html + '</md-autocomplete-parent-scope>';
          }
          function getNoItemsTemplate() {
            var templateTag = element.find('md-not-found').detach(), template = templateTag.length ? templateTag.html() : '';
            return template ? '<li ng-if="$mdAutocompleteCtrl.notFoundVisible()"                         md-autocomplete-parent-scope>' + template + '</li>' : '';
          }
          function getInputElement() {
            if (attr.mdFloatingLabel) {
              return '            <md-input-container flex ng-if="floatingLabel">              <label>{{floatingLabel}}</label>              <input type="search"                  ' + (tabindex != null ? 'tabindex="' + tabindex + '"' : '') + '                  id="{{ inputId || \'fl-input-\' + $mdAutocompleteCtrl.id }}"                  name="{{inputName}}"                  autocomplete="off"                  ng-required="$mdAutocompleteCtrl.isRequired"                  ng-minlength="inputMinlength"                  ng-maxlength="inputMaxlength"                  ng-disabled="$mdAutocompleteCtrl.isDisabled"                  ng-model="$mdAutocompleteCtrl.scope.searchText"                  ng-keydown="$mdAutocompleteCtrl.keydown($event)"                  ng-blur="$mdAutocompleteCtrl.blur()"                  ng-focus="$mdAutocompleteCtrl.focus()"                  aria-owns="ul-{{$mdAutocompleteCtrl.id}}"                  aria-label="{{floatingLabel}}"                  aria-autocomplete="list"                  aria-haspopup="true"                  aria-activedescendant=""                  aria-expanded="{{!$mdAutocompleteCtrl.hidden}}"/>              <div md-autocomplete-parent-scope md-autocomplete-replace>' + leftover + '</div>            </md-input-container>';
            } else {
              return '            <input flex type="search"                ' + (tabindex != null ? 'tabindex="' + tabindex + '"' : '') + '                id="{{ inputId || \'input-\' + $mdAutocompleteCtrl.id }}"                name="{{inputName}}"                ng-if="!floatingLabel"                autocomplete="off"                ng-required="$mdAutocompleteCtrl.isRequired"                ng-disabled="$mdAutocompleteCtrl.isDisabled"                ng-model="$mdAutocompleteCtrl.scope.searchText"                ng-keydown="$mdAutocompleteCtrl.keydown($event)"                ng-blur="$mdAutocompleteCtrl.blur()"                ng-focus="$mdAutocompleteCtrl.focus()"                placeholder="{{placeholder}}"                aria-owns="ul-{{$mdAutocompleteCtrl.id}}"                aria-label="{{placeholder}}"                aria-autocomplete="list"                aria-haspopup="true"                aria-activedescendant=""                aria-expanded="{{!$mdAutocompleteCtrl.hidden}}"/>            <button                type="button"                tabindex="-1"                ng-if="$mdAutocompleteCtrl.scope.searchText && !$mdAutocompleteCtrl.isDisabled"                ng-click="$mdAutocompleteCtrl.clear()">              <md-icon md-svg-icon="md-close"></md-icon>              <span class="md-visually-hidden">Clear</span>            </button>                ';
            }
          }
        }
      };
    }
  }());
  (function () {
    'use strict';
    angular.module('material.components.autocomplete').directive('mdAutocompleteParentScope', MdAutocompleteItemScopeDirective);
    function MdAutocompleteItemScopeDirective($compile, $mdUtil) {
      return {
        restrict: 'AE',
        compile: compile,
        terminal: true,
        transclude: 'element'
      };
      function compile(tElement, tAttr, transclude) {
        return function postLink(scope, element, attr) {
          var ctrl = scope.$mdAutocompleteCtrl;
          var newScope = ctrl.parent.$new();
          var itemName = ctrl.itemName;
          // Watch for changes to our scope's variables and copy them to the new scope
          watchVariable('$index', '$index');
          watchVariable('item', itemName);
          // Ensure that $digest calls on our scope trigger $digest on newScope.
          connectScopes();
          // Link the element against newScope.
          transclude(newScope, function (clone) {
            element.after(clone);
          });
          /**
       * Creates a watcher for variables that are copied from the parent scope
       * @param variable
       * @param alias
       */
          function watchVariable(variable, alias) {
            newScope[alias] = scope[variable];
            scope.$watch(variable, function (value) {
              $mdUtil.nextTick(function () {
                newScope[alias] = value;
              });
            });
          }
          /**
       * Creates watchers on scope and newScope that ensure that for any
       * $digest of scope, newScope is also $digested.
       */
          function connectScopes() {
            var scopeDigesting = false;
            var newScopeDigesting = false;
            scope.$watch(function () {
              if (newScopeDigesting || scopeDigesting) {
                return;
              }
              scopeDigesting = true;
              scope.$$postDigest(function () {
                if (!newScopeDigesting) {
                  newScope.$digest();
                }
                scopeDigesting = newScopeDigesting = false;
              });
            });
            newScope.$watch(function () {
              newScopeDigesting = true;
            });
          }
        };
      }
    }
    MdAutocompleteItemScopeDirective.$inject = [
      '$compile',
      '$mdUtil'
    ];
  }());
  (function () {
    'use strict';
    angular.module('material.components.autocomplete').controller('MdHighlightCtrl', MdHighlightCtrl);
    function MdHighlightCtrl($scope, $element, $attrs) {
      this.init = init;
      function init(termExpr, unsafeTextExpr) {
        var text = null, regex = null, flags = $attrs.mdHighlightFlags || '', watcher = $scope.$watch(function ($scope) {
            return {
              term: termExpr($scope),
              unsafeText: unsafeTextExpr($scope)
            };
          }, function (state, prevState) {
            if (text === null || state.unsafeText !== prevState.unsafeText) {
              text = angular.element('<div>').text(state.unsafeText).html();
            }
            if (regex === null || state.term !== prevState.term) {
              regex = getRegExp(state.term, flags);
            }
            $element.html(text.replace(regex, '<span class="highlight">$&</span>'));
          }, true);
        $element.on('$destroy', watcher);
      }
      function sanitize(term) {
        return term && term.replace(/[\\\^\$\*\+\?\.\(\)\|\{}\[\]]/g, '\\$&');
      }
      function getRegExp(text, flags) {
        var str = '';
        if (flags.indexOf('^') >= 1)
          str += '^';
        str += text;
        if (flags.indexOf('$') >= 1)
          str += '$';
        return new RegExp(sanitize(str), flags.replace(/[\$\^]/g, ''));
      }
    }
    MdHighlightCtrl.$inject = [
      '$scope',
      '$element',
      '$attrs'
    ];
  }());
  (function () {
    'use strict';
    angular.module('material.components.autocomplete').directive('mdHighlightText', MdHighlight);
    /**
 * @ngdoc directive
 * @name mdHighlightText
 * @module material.components.autocomplete
 *
 * @description
 * The `md-highlight-text` directive allows you to specify text that should be highlighted within
 *     an element.  Highlighted text will be wrapped in `<span class="highlight"></span>` which can
 *     be styled through CSS.  Please note that child elements may not be used with this directive.
 *
 * @param {string} md-highlight-text A model to be searched for
 * @param {string=} md-highlight-flags A list of flags (loosely based on JavaScript RexExp flags).
 * #### **Supported flags**:
 * - `g`: Find all matches within the provided text
 * - `i`: Ignore case when searching for matches
 * - `$`: Only match if the text ends with the search term
 * - `^`: Only match if the text begins with the search term
 *
 * @usage
 * <hljs lang="html">
 * <input placeholder="Enter a search term..." ng-model="searchTerm" type="text" />
 * <ul>
 *   <li ng-repeat="result in results" md-highlight-text="searchTerm">
 *     {{result.text}}
 *   </li>
 * </ul>
 * </hljs>
 */
    function MdHighlight($interpolate, $parse) {
      return {
        terminal: true,
        controller: 'MdHighlightCtrl',
        compile: function mdHighlightCompile(tElement, tAttr) {
          var termExpr = $parse(tAttr.mdHighlightText);
          var unsafeTextExpr = $interpolate(tElement.html());
          return function mdHighlightLink(scope, element, attr, ctrl) {
            ctrl.init(termExpr, unsafeTextExpr);
          };
        }
      };
    }
    MdHighlight.$inject = [
      '$interpolate',
      '$parse'
    ];
  }());
  (function () {
    'use strict';
    angular.module('material.components.chips').directive('mdChip', MdChip);
    /**
 * @ngdoc directive
 * @name mdChip
 * @module material.components.chips
 *
 * @description
 * `<md-chip>` is a component used within `<md-chips>` and is responsible for rendering individual
 * chips.
 *
 *
 * @usage
 * <hljs lang="html">
 *   <md-chip>{{$chip}}</md-chip>
 * </hljs>
 *
 */
    // This hint text is hidden within a chip but used by screen readers to
    // inform the user how they can interact with a chip.
    var DELETE_HINT_TEMPLATE = '    <span ng-if="!$mdChipsCtrl.readonly" class="md-visually-hidden">      {{$mdChipsCtrl.deleteHint}}    </span>';
    /**
 * MDChip Directive Definition
 *
 * @param $mdTheming
 * @param $mdInkRipple
 * @ngInject
 */
    function MdChip($mdTheming, $mdUtil) {
      var hintTemplate = $mdUtil.processTemplate(DELETE_HINT_TEMPLATE);
      return {
        restrict: 'E',
        require: '^?mdChips',
        compile: compile
      };
      function compile(element, attr) {
        // Append the delete template
        element.append($mdUtil.processTemplate(hintTemplate));
        return function postLink(scope, element, attr, ctrl) {
          element.addClass('md-chip');
          $mdTheming(element);
          if (ctrl)
            angular.element(element[0].querySelector('.md-chip-content')).on('blur', function () {
              ctrl.selectedChip = -1;
            });
        };
      }
    }
    MdChip.$inject = [
      '$mdTheming',
      '$mdUtil'
    ];
  }());
  (function () {
    'use strict';
    angular.module('material.components.chips').directive('mdChipRemove', MdChipRemove);
    /**
 * @ngdoc directive
 * @name mdChipRemove
 * @module material.components.chips
 *
 * @description
 * `<md-chip-remove>`
 * Designates an element to be used as the delete button for a chip. This
 * element is passed as a child of the `md-chips` element.
 *
 * @usage
 * <hljs lang="html">
 *   <md-chips><button md-chip-remove>DEL</button></md-chips>
 * </hljs>
 */
    /**
 * MdChipRemove Directive Definition.
 * 
 * @param $compile
 * @param $timeout
 * @returns {{restrict: string, require: string[], link: Function, scope: boolean}}
 * @constructor
 */
    function MdChipRemove($timeout) {
      return {
        restrict: 'A',
        require: '^mdChips',
        scope: false,
        link: postLink
      };
      function postLink(scope, element, attr, ctrl) {
        element.on('click', function (event) {
          scope.$apply(function () {
            ctrl.removeChip(scope.$$replacedScope.$index);
          });
        });
        // Child elements aren't available until after a $timeout tick as they are hidden by an
        // `ng-if`. see http://goo.gl/zIWfuw
        $timeout(function () {
          element.attr({
            tabindex: -1,
            'aria-hidden': true
          });
          element.find('button').attr('tabindex', '-1');
        });
      }
    }
    MdChipRemove.$inject = ['$timeout'];
  }());
  (function () {
    'use strict';
    angular.module('material.components.chips').directive('mdChipTransclude', MdChipTransclude);
    function MdChipTransclude($compile) {
      return {
        restrict: 'EA',
        terminal: true,
        link: link,
        scope: false
      };
      function link(scope, element, attr) {
        var ctrl = scope.$parent.$mdChipsCtrl, newScope = ctrl.parent.$new(false, ctrl.parent);
        newScope.$$replacedScope = scope;
        newScope.$chip = scope.$chip;
        newScope.$index = scope.$index;
        newScope.$mdChipsCtrl = ctrl;
        var newHtml = ctrl.$scope.$eval(attr.mdChipTransclude);
        element.html(newHtml);
        $compile(element.contents())(newScope);
      }
    }
    MdChipTransclude.$inject = ['$compile'];
  }());
  (function () {
    'use strict';
    angular.module('material.components.chips').controller('MdChipsCtrl', MdChipsCtrl);
    /**
 * Controller for the MdChips component. Responsible for adding to and
 * removing from the list of chips, marking chips as selected, and binding to
 * the models of various input components.
 *
 * @param $scope
 * @param $mdConstant
 * @param $log
 * @param $element
 * @constructor
 */
    function MdChipsCtrl($scope, $mdConstant, $log, $element, $timeout) {
      /** @type {$timeout} **/
      this.$timeout = $timeout;
      /** @type {Object} */
      this.$mdConstant = $mdConstant;
      /** @type {angular.$scope} */
      this.$scope = $scope;
      /** @type {angular.$scope} */
      this.parent = $scope.$parent;
      /** @type {$log} */
      this.$log = $log;
      /** @type {$element} */
      this.$element = $element;
      /** @type {angular.NgModelController} */
      this.ngModelCtrl = null;
      /** @type {angular.NgModelController} */
      this.userInputNgModelCtrl = null;
      /** @type {Element} */
      this.userInputElement = null;
      /** @type {Array.<Object>} */
      this.items = [];
      /** @type {number} */
      this.selectedChip = -1;
      /** @type {boolean} */
      this.hasAutocomplete = false;
      /**
   * Hidden hint text for how to delete a chip. Used to give context to screen readers.
   * @type {string}
   */
      this.deleteHint = 'Press delete to remove this chip.';
      /**
   * Hidden label for the delete button. Used to give context to screen readers.
   * @type {string}
   */
      this.deleteButtonLabel = 'Remove';
      /**
   * Model used by the input element.
   * @type {string}
   */
      this.chipBuffer = '';
      /**
   * Whether to use the onAppend expression to transform the chip buffer
   * before appending it to the list.
   * @type {boolean}
   *
   *
   * @deprecated Will remove in 1.0.
   */
      this.useOnAppend = false;
      /**
   * Whether to use the transformChip expression to transform the chip buffer
   * before appending it to the list.
   * @type {boolean}
   */
      this.useTransformChip = false;
      /**
   * Whether to use the onAdd expression to notify of chip additions.
   * @type {boolean}
   */
      this.useOnAdd = false;
      /**
   * Whether to use the onRemove expression to notify of chip removals.
   * @type {boolean}
   */
      this.useOnRemove = false;
      /**
   * Whether to use the onSelect expression to notify the component's user
   * after selecting a chip from the list.
   * @type {boolean}
   */
      this.useOnSelect = false;
    }
    MdChipsCtrl.$inject = [
      '$scope',
      '$mdConstant',
      '$log',
      '$element',
      '$timeout'
    ];
    /**
 * Handles the keydown event on the input element: by default <enter> appends
 * the buffer to the chip list, while backspace removes the last chip in the
 * list if the current buffer is empty.
 * @param event
 */
    MdChipsCtrl.prototype.inputKeydown = function (event) {
      var chipBuffer = this.getChipBuffer();
      // If we have an autocomplete, and it handled the event, we have nothing to do
      if (this.hasAutocomplete && event.isDefaultPrevented && event.isDefaultPrevented()) {
        return;
      }
      if (event.keyCode === this.$mdConstant.KEY_CODE.BACKSPACE) {
        if (chipBuffer)
          return;
        event.preventDefault();
        event.stopPropagation();
        if (this.items.length)
          this.selectAndFocusChipSafe(this.items.length - 1);
        return;
      }
      // By default <enter> appends the buffer to the chip list.
      if (!this.separatorKeys || this.separatorKeys.length < 1) {
        this.separatorKeys = [this.$mdConstant.KEY_CODE.ENTER];
      }
      // Support additional separator key codes in an array of `md-separator-keys`.
      if (this.separatorKeys.indexOf(event.keyCode) !== -1) {
        if (this.hasAutocomplete && this.requireMatch || !chipBuffer)
          return;
        event.preventDefault();
        this.appendChip(chipBuffer);
        this.resetChipBuffer();
      }
    };
    /**
 * Handles the keydown event on the chip elements: backspace removes the selected chip, arrow
 * keys switch which chips is active
 * @param event
 */
    MdChipsCtrl.prototype.chipKeydown = function (event) {
      if (this.getChipBuffer())
        return;
      switch (event.keyCode) {
      case this.$mdConstant.KEY_CODE.BACKSPACE:
      case this.$mdConstant.KEY_CODE.DELETE:
        if (this.selectedChip < 0)
          return;
        event.preventDefault();
        this.removeAndSelectAdjacentChip(this.selectedChip);
        break;
      case this.$mdConstant.KEY_CODE.LEFT_ARROW:
        event.preventDefault();
        if (this.selectedChip < 0)
          this.selectedChip = this.items.length;
        if (this.items.length)
          this.selectAndFocusChipSafe(this.selectedChip - 1);
        break;
      case this.$mdConstant.KEY_CODE.RIGHT_ARROW:
        event.preventDefault();
        this.selectAndFocusChipSafe(this.selectedChip + 1);
        break;
      case this.$mdConstant.KEY_CODE.ESCAPE:
      case this.$mdConstant.KEY_CODE.TAB:
        if (this.selectedChip < 0)
          return;
        event.preventDefault();
        this.onFocus();
        break;
      }
    };
    /**
 * Get the input's placeholder - uses `placeholder` when list is empty and `secondary-placeholder`
 * when the list is non-empty. If `secondary-placeholder` is not provided, `placeholder` is used
 * always.
 */
    MdChipsCtrl.prototype.getPlaceholder = function () {
      // Allow `secondary-placeholder` to be blank.
      var useSecondary = this.items.length && (this.secondaryPlaceholder == '' || this.secondaryPlaceholder);
      return useSecondary ? this.placeholder : this.secondaryPlaceholder;
    };
    /**
 * Removes chip at {@code index} and selects the adjacent chip.
 * @param index
 */
    MdChipsCtrl.prototype.removeAndSelectAdjacentChip = function (index) {
      var selIndex = this.getAdjacentChipIndex(index);
      this.removeChip(index);
      this.$timeout(angular.bind(this, function () {
        this.selectAndFocusChipSafe(selIndex);
      }));
    };
    /**
 * Sets the selected chip index to -1.
 */
    MdChipsCtrl.prototype.resetSelectedChip = function () {
      this.selectedChip = -1;
    };
    /**
 * Gets the index of an adjacent chip to select after deletion. Adjacency is
 * determined as the next chip in the list, unless the target chip is the
 * last in the list, then it is the chip immediately preceding the target. If
 * there is only one item in the list, -1 is returned (select none).
 * The number returned is the index to select AFTER the target has been
 * removed.
 * If the current chip is not selected, then -1 is returned to select none.
 */
    MdChipsCtrl.prototype.getAdjacentChipIndex = function (index) {
      var len = this.items.length - 1;
      return len == 0 ? -1 : index == len ? index - 1 : index;
    };
    /**
 * Append the contents of the buffer to the chip list. This method will first
 * call out to the md-transform-chip method, if provided.
 *
 * @param newChip
 */
    MdChipsCtrl.prototype.appendChip = function (newChip) {
      if (this.useTransformChip && this.transformChip) {
        var transformedChip = this.transformChip({ '$chip': newChip });
        // Check to make sure the chip is defined before assigning it, otherwise, we'll just assume
        // they want the string version.
        if (angular.isDefined(transformedChip)) {
          newChip = transformedChip;
        }
      }
      // If items contains an identical object to newChip, do not append
      if (angular.isObject(newChip)) {
        var identical = this.items.some(function (item) {
            return angular.equals(newChip, item);
          });
        if (identical)
          return;
      }
      // Check for a null (but not undefined), or existing chip and cancel appending
      if (newChip == null || this.items.indexOf(newChip) + 1)
        return;
      // Append the new chip onto our list
      var index = this.items.push(newChip);
      // If they provide the md-on-add attribute, notify them of the chip addition
      if (this.useOnAdd && this.onAdd) {
        this.onAdd({
          '$chip': newChip,
          '$index': index
        });
      }
    };
    /**
 * Sets whether to use the md-on-append expression. This expression is
 * bound to scope and controller in {@code MdChipsDirective} as
 * {@code onAppend}. Due to the nature of directive scope bindings, the
 * controller cannot know on its own/from the scope whether an expression was
 * actually provided.
 *
 * @deprecated
 *
 * TODO: Remove deprecated md-on-append functionality in 1.0
 */
    MdChipsCtrl.prototype.useOnAppendExpression = function () {
      this.$log.warn('md-on-append is deprecated; please use md-transform-chip or md-on-add instead');
      if (!this.useTransformChip || !this.transformChip) {
        this.useTransformChip = true;
        this.transformChip = this.onAppend;
      }
    };
    /**
 * Sets whether to use the md-transform-chip expression. This expression is
 * bound to scope and controller in {@code MdChipsDirective} as
 * {@code transformChip}. Due to the nature of directive scope bindings, the
 * controller cannot know on its own/from the scope whether an expression was
 * actually provided.
 */
    MdChipsCtrl.prototype.useTransformChipExpression = function () {
      this.useTransformChip = true;
    };
    /**
 * Sets whether to use the md-on-add expression. This expression is
 * bound to scope and controller in {@code MdChipsDirective} as
 * {@code onAdd}. Due to the nature of directive scope bindings, the
 * controller cannot know on its own/from the scope whether an expression was
 * actually provided.
 */
    MdChipsCtrl.prototype.useOnAddExpression = function () {
      this.useOnAdd = true;
    };
    /**
 * Sets whether to use the md-on-remove expression. This expression is
 * bound to scope and controller in {@code MdChipsDirective} as
 * {@code onRemove}. Due to the nature of directive scope bindings, the
 * controller cannot know on its own/from the scope whether an expression was
 * actually provided.
 */
    MdChipsCtrl.prototype.useOnRemoveExpression = function () {
      this.useOnRemove = true;
    };
    /*
 * Sets whether to use the md-on-select expression. This expression is
 * bound to scope and controller in {@code MdChipsDirective} as
 * {@code onSelect}. Due to the nature of directive scope bindings, the
 * controller cannot know on its own/from the scope whether an expression was
 * actually provided.
 */
    MdChipsCtrl.prototype.useOnSelectExpression = function () {
      this.useOnSelect = true;
    };
    /**
 * Gets the input buffer. The input buffer can be the model bound to the
 * default input item {@code this.chipBuffer}, the {@code selectedItem}
 * model of an {@code md-autocomplete}, or, through some magic, the model
 * bound to any inpput or text area element found within a
 * {@code md-input-container} element.
 * @return {Object|string}
 */
    MdChipsCtrl.prototype.getChipBuffer = function () {
      return !this.userInputElement ? this.chipBuffer : this.userInputNgModelCtrl ? this.userInputNgModelCtrl.$viewValue : this.userInputElement[0].value;
    };
    /**
 * Resets the input buffer for either the internal input or user provided input element.
 */
    MdChipsCtrl.prototype.resetChipBuffer = function () {
      if (this.userInputElement) {
        if (this.userInputNgModelCtrl) {
          this.userInputNgModelCtrl.$setViewValue('');
          this.userInputNgModelCtrl.$render();
        } else {
          this.userInputElement[0].value = '';
        }
      } else {
        this.chipBuffer = '';
      }
    };
    /**
 * Removes the chip at the given index.
 * @param index
 */
    MdChipsCtrl.prototype.removeChip = function (index) {
      var removed = this.items.splice(index, 1);
      if (removed && removed.length && this.useOnRemove && this.onRemove) {
        this.onRemove({
          '$chip': removed[0],
          '$index': index
        });
      }
    };
    MdChipsCtrl.prototype.removeChipAndFocusInput = function (index) {
      this.removeChip(index);
      this.onFocus();
    };
    /**
 * Selects the chip at `index`,
 * @param index
 */
    MdChipsCtrl.prototype.selectAndFocusChipSafe = function (index) {
      if (!this.items.length) {
        this.selectChip(-1);
        this.onFocus();
        return;
      }
      if (index === this.items.length)
        return this.onFocus();
      index = Math.max(index, 0);
      index = Math.min(index, this.items.length - 1);
      this.selectChip(index);
      this.focusChip(index);
    };
    /**
 * Marks the chip at the given index as selected.
 * @param index
 */
    MdChipsCtrl.prototype.selectChip = function (index) {
      if (index >= -1 && index <= this.items.length) {
        this.selectedChip = index;
        // Fire the onSelect if provided
        if (this.useOnSelect && this.onSelect) {
          this.onSelect({ '$chip': this.items[this.selectedChip] });
        }
      } else {
        this.$log.warn('Selected Chip index out of bounds; ignoring.');
      }
    };
    /**
 * Selects the chip at `index` and gives it focus.
 * @param index
 */
    MdChipsCtrl.prototype.selectAndFocusChip = function (index) {
      this.selectChip(index);
      if (index != -1) {
        this.focusChip(index);
      }
    };
    /**
 * Call `focus()` on the chip at `index`
 */
    MdChipsCtrl.prototype.focusChip = function (index) {
      this.$element[0].querySelector('md-chip[index="' + index + '"] .md-chip-content').focus();
    };
    /**
 * Configures the required interactions with the ngModel Controller.
 * Specifically, set {@code this.items} to the {@code NgModelCtrl#$viewVale}.
 * @param ngModelCtrl
 */
    MdChipsCtrl.prototype.configureNgModel = function (ngModelCtrl) {
      this.ngModelCtrl = ngModelCtrl;
      var self = this;
      ngModelCtrl.$render = function () {
        // model is updated. do something.
        self.items = self.ngModelCtrl.$viewValue;
      };
    };
    MdChipsCtrl.prototype.onFocus = function () {
      var input = this.$element[0].querySelector('input');
      input && input.focus();
      this.resetSelectedChip();
    };
    MdChipsCtrl.prototype.onInputFocus = function () {
      this.inputHasFocus = true;
      this.resetSelectedChip();
    };
    MdChipsCtrl.prototype.onInputBlur = function () {
      this.inputHasFocus = false;
    };
    /**
 * Configure event bindings on a user-provided input element.
 * @param inputElement
 */
    MdChipsCtrl.prototype.configureUserInput = function (inputElement) {
      this.userInputElement = inputElement;
      // Find the NgModelCtrl for the input element
      var ngModelCtrl = inputElement.controller('ngModel');
      // `.controller` will look in the parent as well.
      if (ngModelCtrl != this.ngModelCtrl) {
        this.userInputNgModelCtrl = ngModelCtrl;
      }
      var scope = this.$scope;
      var ctrl = this;
      // Run all of the events using evalAsync because a focus may fire a blur in the same digest loop
      var scopeApplyFn = function (event, fn) {
        scope.$evalAsync(angular.bind(ctrl, fn, event));
      };
      // Bind to keydown and focus events of input
      inputElement.attr({ tabindex: 0 }).on('keydown', function (event) {
        scopeApplyFn(event, ctrl.inputKeydown);
      }).on('focus', function (event) {
        scopeApplyFn(event, ctrl.onInputFocus);
      }).on('blur', function (event) {
        scopeApplyFn(event, ctrl.onInputBlur);
      });
    };
    MdChipsCtrl.prototype.configureAutocomplete = function (ctrl) {
      if (ctrl) {
        this.hasAutocomplete = true;
        ctrl.registerSelectedItemWatcher(angular.bind(this, function (item) {
          if (item) {
            this.appendChip(item);
            this.resetChipBuffer();
          }
        }));
        this.$element.find('input').on('focus', angular.bind(this, this.onInputFocus)).on('blur', angular.bind(this, this.onInputBlur));
      }
    };
    MdChipsCtrl.prototype.hasFocus = function () {
      return this.inputHasFocus || this.selectedChip >= 0;
    };
  }());
  (function () {
    'use strict';
    angular.module('material.components.chips').directive('mdChips', MdChips);
    /**
   * @ngdoc directive
   * @name mdChips
   * @module material.components.chips
   *
   * @description
   * `<md-chips>` is an input component for building lists of strings or objects. The list items are
   * displayed as 'chips'. This component can make use of an `<input>` element or an 
   * `<md-autocomplete>` element.
   *
   * ### Custom templates
   * A custom template may be provided to render the content of each chip. This is achieved by
   * specifying an `<md-chip-template>` element containing the custom content as a child of
   * `<md-chips>`.
   *
   * Note: Any attributes on
   * `<md-chip-template>` will be dropped as only the innerHTML is used for the chip template. The
   * variables `$chip` and `$index` are available in the scope of `<md-chip-template>`, representing
   * the chip object and its index in the list of chips, respectively.
   * To override the chip delete control, include an element (ideally a button) with the attribute
   * `md-chip-remove`. A click listener to remove the chip will be added automatically. The element
   * is also placed as a sibling to the chip content (on which there are also click listeners) to
   * avoid a nested ng-click situation.
   *
   * <h3> Pending Features </h3>
   * <ul style="padding-left:20px;">
   *
   *   <ul>Style
   *     <li>Colours for hover, press states (ripple?).</li>
   *   </ul>
   *
   *   <ul>Validation
   *     <li>allow a validation callback</li>
   *     <li>hilighting style for invalid chips</li>
   *   </ul>
   *
   *   <ul>Item mutation
   *     <li>Support `
   *       <md-chip-edit>` template, show/hide the edit element on tap/click? double tap/double
   *       click?
   *     </li>
   *   </ul>
   *
   *   <ul>Truncation and Disambiguation (?)
   *     <li>Truncate chip text where possible, but do not truncate entries such that two are
   *     indistinguishable.</li>
   *   </ul>
   *
   *   <ul>Drag and Drop
   *     <li>Drag and drop chips between related `<md-chips>` elements.
   *     </li>
   *   </ul>
   * </ul>
   *
   *  <span style="font-size:.8em;text-align:center">
   *    Warning: This component is a WORK IN PROGRESS. If you use it now,
   *    it will probably break on you in the future.
   *  </span>
   *
   * @param {string=|object=} ng-model A model to bind the list of items to
   * @param {string=} placeholder Placeholder text that will be forwarded to the input.
   * @param {string=} secondary-placeholder Placeholder text that will be forwarded to the input,
   *    displayed when there is at least on item in the list
   * @param {boolean=} readonly Disables list manipulation (deleting or adding list items), hiding
   *    the input and delete buttons
   * @param {expression} md-transform-chip An expression of form `myFunction($chip)` that when called
   *    expects one of the following return values:
   *    - an object representing the `$chip` input string
   *    - `undefined` to simply add the `$chip` input string, or
   *    - `null` to prevent the chip from being appended
   * @param {expression=} md-on-add An expression which will be called when a chip has been
   *    added.
   * @param {expression=} md-on-remove An expression which will be called when a chip has been
   *    removed.
   * @param {expression=} md-on-select An expression which will be called when a chip is selected.
   * @param {boolean} md-require-match If true, and the chips template contains an autocomplete,
   *    only allow selection of pre-defined chips (i.e. you cannot add new ones).
   * @param {string=} delete-hint A string read by screen readers instructing users that pressing
   *    the delete key will remove the chip.
   * @param {string=} delete-button-label A label for the delete button. Also hidden and read by
   *    screen readers.
   * @param {expression=} md-separator-keys An array of key codes used to separate chips.
   *
   * @usage
   * <hljs lang="html">
   *   <md-chips
   *       ng-model="myItems"
   *       placeholder="Add an item"
   *       readonly="isReadOnly">
   *   </md-chips>
   * </hljs>
   *
   */
    var MD_CHIPS_TEMPLATE = '      <md-chips-wrap          ng-if="!$mdChipsCtrl.readonly || $mdChipsCtrl.items.length > 0"          ng-keydown="$mdChipsCtrl.chipKeydown($event)"          ng-class="{ \'md-focused\': $mdChipsCtrl.hasFocus(), \'md-readonly\': !$mdChipsCtrl.ngModelCtrl }"          class="md-chips">        <md-chip ng-repeat="$chip in $mdChipsCtrl.items"            index="{{$index}}"            ng-class="{\'md-focused\': $mdChipsCtrl.selectedChip == $index, \'md-readonly\': $mdChipsCtrl.readonly}">          <div class="md-chip-content"              tabindex="-1"              aria-hidden="true"              ng-focus="!$mdChipsCtrl.readonly && $mdChipsCtrl.selectChip($index)"              md-chip-transclude="$mdChipsCtrl.chipContentsTemplate"></div>          <div ng-if="!$mdChipsCtrl.readonly"               class="md-chip-remove-container"               md-chip-transclude="$mdChipsCtrl.chipRemoveTemplate"></div>        </md-chip>        <div ng-if="!$mdChipsCtrl.readonly && $mdChipsCtrl.ngModelCtrl"            class="md-chip-input-container"            md-chip-transclude="$mdChipsCtrl.chipInputTemplate"></div>        </div>      </md-chips-wrap>';
    var CHIP_INPUT_TEMPLATE = '        <input            class="md-input"            tabindex="0"            placeholder="{{$mdChipsCtrl.getPlaceholder()}}"            aria-label="{{$mdChipsCtrl.getPlaceholder()}}"            ng-model="$mdChipsCtrl.chipBuffer"            ng-focus="$mdChipsCtrl.onInputFocus()"            ng-blur="$mdChipsCtrl.onInputBlur()"            ng-keydown="$mdChipsCtrl.inputKeydown($event)">';
    var CHIP_DEFAULT_TEMPLATE = '      <span>{{$chip}}</span>';
    var CHIP_REMOVE_TEMPLATE = '      <button          class="md-chip-remove"          ng-if="!$mdChipsCtrl.readonly"          ng-click="$mdChipsCtrl.removeChipAndFocusInput($$replacedScope.$index)"          type="button"          aria-hidden="true"          tabindex="-1">        <md-icon md-svg-icon="md-close"></md-icon>        <span class="md-visually-hidden">          {{$mdChipsCtrl.deleteButtonLabel}}        </span>      </button>';
    /**
   * MDChips Directive Definition
   */
    function MdChips($mdTheming, $mdUtil, $compile, $log, $timeout) {
      // Run our templates through $mdUtil.processTemplate() to allow custom start/end symbols
      var templates = getTemplates();
      return {
        template: function (element, attrs) {
          // Clone the element into an attribute. By prepending the attribute
          // name with '$', Angular won't write it into the DOM. The cloned
          // element propagates to the link function via the attrs argument,
          // where various contained-elements can be consumed.
          attrs['$mdUserTemplate'] = element.clone();
          return templates.chips;
        },
        require: ['mdChips'],
        restrict: 'E',
        controller: 'MdChipsCtrl',
        controllerAs: '$mdChipsCtrl',
        bindToController: true,
        compile: compile,
        scope: {
          readonly: '=readonly',
          placeholder: '@',
          secondaryPlaceholder: '@',
          transformChip: '&mdTransformChip',
          onAppend: '&mdOnAppend',
          onAdd: '&mdOnAdd',
          onRemove: '&mdOnRemove',
          onSelect: '&mdOnSelect',
          deleteHint: '@',
          deleteButtonLabel: '@',
          separatorKeys: '=?mdSeparatorKeys',
          requireMatch: '=?mdRequireMatch'
        }
      };
      /**
     * Builds the final template for `md-chips` and returns the postLink function.
     *
     * Building the template involves 3 key components:
     * static chips
     * chip template
     * input control
     *
     * If no `ng-model` is provided, only the static chip work needs to be done.
     *
     * If no user-passed `md-chip-template` exists, the default template is used. This resulting
     * template is appended to the chip content element.
     *
     * The remove button may be overridden by passing an element with an md-chip-remove attribute.
     *
     * If an `input` or `md-autocomplete` element is provided by the caller, it is set aside for
     * transclusion later. The transclusion happens in `postLink` as the parent scope is required.
     * If no user input is provided, a default one is appended to the input container node in the
     * template.
     *
     * Static Chips (i.e. `md-chip` elements passed from the caller) are gathered and set aside for
     * transclusion in the `postLink` function.
     *
     *
     * @param element
     * @param attr
     * @returns {Function}
     */
      function compile(element, attr) {
        // Grab the user template from attr and reset the attribute to null.
        var userTemplate = attr['$mdUserTemplate'];
        attr['$mdUserTemplate'] = null;
        // Set the chip remove, chip contents and chip input templates. The link function will put
        // them on the scope for transclusion later.
        var chipRemoveTemplate = getTemplateByQuery('md-chips>*[md-chip-remove]') || templates.remove, chipContentsTemplate = getTemplateByQuery('md-chips>md-chip-template') || templates.default, chipInputTemplate = getTemplateByQuery('md-chips>md-autocomplete') || getTemplateByQuery('md-chips>input') || templates.input, staticChips = userTemplate.find('md-chip');
        // Warn of malformed template. See #2545
        if (userTemplate[0].querySelector('md-chip-template>*[md-chip-remove]')) {
          $log.warn('invalid placement of md-chip-remove within md-chip-template.');
        }
        function getTemplateByQuery(query) {
          if (!attr.ngModel)
            return;
          var element = userTemplate[0].querySelector(query);
          return element && element.outerHTML;
        }
        /**
       * Configures controller and transcludes.
       */
        return function postLink(scope, element, attrs, controllers) {
          $mdUtil.initOptionalProperties(scope, attr);
          $mdTheming(element);
          var mdChipsCtrl = controllers[0];
          mdChipsCtrl.chipContentsTemplate = chipContentsTemplate;
          mdChipsCtrl.chipRemoveTemplate = chipRemoveTemplate;
          mdChipsCtrl.chipInputTemplate = chipInputTemplate;
          element.attr({
            'aria-hidden': true,
            tabindex: -1
          }).on('focus', function () {
            mdChipsCtrl.onFocus();
          });
          if (attr.ngModel) {
            mdChipsCtrl.configureNgModel(element.controller('ngModel'));
            // If an `md-transform-chip` attribute was set, tell the controller to use the expression
            // before appending chips.
            if (attrs.mdTransformChip)
              mdChipsCtrl.useTransformChipExpression();
            // If an `md-on-append` attribute was set, tell the controller to use the expression
            // when appending chips.
            //
            // DEPRECATED: Will remove in official 1.0 release
            if (attrs.mdOnAppend)
              mdChipsCtrl.useOnAppendExpression();
            // If an `md-on-add` attribute was set, tell the controller to use the expression
            // when adding chips.
            if (attrs.mdOnAdd)
              mdChipsCtrl.useOnAddExpression();
            // If an `md-on-remove` attribute was set, tell the controller to use the expression
            // when removing chips.
            if (attrs.mdOnRemove)
              mdChipsCtrl.useOnRemoveExpression();
            // If an `md-on-select` attribute was set, tell the controller to use the expression
            // when selecting chips.
            if (attrs.mdOnSelect)
              mdChipsCtrl.useOnSelectExpression();
            // The md-autocomplete and input elements won't be compiled until after this directive
            // is complete (due to their nested nature). Wait a tick before looking for them to
            // configure the controller.
            if (chipInputTemplate != templates.input) {
              // The autocomplete will not appear until the readonly attribute is not true (i.e.
              // false or undefined), so we have to watch the readonly and then on the next tick
              // after the chip transclusion has run, we can configure the autocomplete and user
              // input.
              scope.$watch('$mdChipsCtrl.readonly', function (readonly) {
                if (!readonly) {
                  $mdUtil.nextTick(function () {
                    if (chipInputTemplate.indexOf('<md-autocomplete') === 0)
                      mdChipsCtrl.configureAutocomplete(element.find('md-autocomplete').controller('mdAutocomplete'));
                    mdChipsCtrl.configureUserInput(element.find('input'));
                  });
                }
              });
            }
            // At the next tick, if we find an input, make sure it has the md-input class
            $mdUtil.nextTick(function () {
              var input = element.find('input');
              input && input.toggleClass('md-input', true);
            });
          }
          // Compile with the parent's scope and prepend any static chips to the wrapper.
          if (staticChips.length > 0) {
            var compiledStaticChips = $compile(staticChips.clone())(scope.$parent);
            $timeout(function () {
              element.find('md-chips-wrap').prepend(compiledStaticChips);
            });
          }
        };
      }
      function getTemplates() {
        return {
          chips: $mdUtil.processTemplate(MD_CHIPS_TEMPLATE),
          input: $mdUtil.processTemplate(CHIP_INPUT_TEMPLATE),
          default: $mdUtil.processTemplate(CHIP_DEFAULT_TEMPLATE),
          remove: $mdUtil.processTemplate(CHIP_REMOVE_TEMPLATE)
        };
      }
    }
    MdChips.$inject = [
      '$mdTheming',
      '$mdUtil',
      '$compile',
      '$log',
      '$timeout'
    ];
  }());
  (function () {
    'use strict';
    angular.module('material.components.chips').controller('MdContactChipsCtrl', MdContactChipsCtrl);
    /**
 * Controller for the MdContactChips component
 * @constructor
 */
    function MdContactChipsCtrl() {
      /** @type {Object} */
      this.selectedItem = null;
      /** @type {string} */
      this.searchText = '';
    }
    MdContactChipsCtrl.prototype.queryContact = function (searchText) {
      var results = this.contactQuery({ '$query': searchText });
      return this.filterSelected ? results.filter(angular.bind(this, this.filterSelectedContacts)) : results;
    };
    MdContactChipsCtrl.prototype.itemName = function (item) {
      return item[this.contactName];
    };
    MdContactChipsCtrl.prototype.filterSelectedContacts = function (contact) {
      return this.contacts.indexOf(contact) == -1;
    };
  }());
  (function () {
    'use strict';
    angular.module('material.components.chips').directive('mdContactChips', MdContactChips);
    /**
 * @ngdoc directive
 * @name mdContactChips
 * @module material.components.chips
 *
 * @description
 * `<md-contact-chips>` is an input component based on `md-chips` and makes use of an
 * `md-autocomplete` element. The component allows the caller to supply a query expression which
 * returns  a list of possible contacts. The user can select one of these and add it to the list of
 * chips.
 *
 * You may also use the `md-highlight-text` directive along with it's parameters to control the
 * appearance of the matched text inside of the contacts' autocomplete popup.
 *
 * @param {string=|object=} ng-model A model to bind the list of items to
 * @param {string=} placeholder Placeholder text that will be forwarded to the input.
 * @param {string=} secondary-placeholder Placeholder text that will be forwarded to the input,
 *    displayed when there is at least on item in the list
 * @param {expression} md-contacts An expression expected to return contacts matching the search
 *    test, `$query`.
 * @param {string} md-contact-name The field name of the contact object representing the
 *    contact's name.
 * @param {string} md-contact-email The field name of the contact object representing the
 *    contact's email address.
 * @param {string} md-contact-image The field name of the contact object representing the
 *    contact's image.
 *
 *
 * // The following attribute has been removed but may come back.
 * @param {expression=} filter-selected Whether to filter selected contacts from the list of
 *    suggestions shown in the autocomplete.
 *
 *
 *
 * @usage
 * <hljs lang="html">
 *   <md-contact-chips
 *       ng-model="ctrl.contacts"
 *       md-contacts="ctrl.querySearch($query)"
 *       md-contact-name="name"
 *       md-contact-image="image"
 *       md-contact-email="email"
 *       placeholder="To">
 *   </md-contact-chips>
 * </hljs>
 *
 */
    var MD_CONTACT_CHIPS_TEMPLATE = '      <md-chips class="md-contact-chips"          ng-model="$mdContactChipsCtrl.contacts"          md-require-match="$mdContactChipsCtrl.requireMatch"          md-autocomplete-snap>          <md-autocomplete              md-menu-class="md-contact-chips-suggestions"              md-selected-item="$mdContactChipsCtrl.selectedItem"              md-search-text="$mdContactChipsCtrl.searchText"              md-items="item in $mdContactChipsCtrl.queryContact($mdContactChipsCtrl.searchText)"              md-item-text="$mdContactChipsCtrl.itemName(item)"              md-no-cache="true"              md-autoselect              placeholder="{{$mdContactChipsCtrl.contacts.length == 0 ?                  $mdContactChipsCtrl.placeholder : $mdContactChipsCtrl.secondaryPlaceholder}}">            <div class="md-contact-suggestion">              <img                   ng-src="{{item[$mdContactChipsCtrl.contactImage]}}"                  alt="{{item[$mdContactChipsCtrl.contactName]}}"                  ng-if="item[$mdContactChipsCtrl.contactImage]" />              <span class="md-contact-name" md-highlight-text="$mdContactChipsCtrl.searchText"                    md-highlight-flags="{{$mdContactChipsCtrl.highlightFlags}}">                {{item[$mdContactChipsCtrl.contactName]}}              </span>              <span class="md-contact-email" >{{item[$mdContactChipsCtrl.contactEmail]}}</span>            </div>          </md-autocomplete>          <md-chip-template>            <div class="md-contact-avatar">              <img                   ng-src="{{$chip[$mdContactChipsCtrl.contactImage]}}"                  alt="{{$chip[$mdContactChipsCtrl.contactName]}}"                  ng-if="$chip[$mdContactChipsCtrl.contactImage]" />            </div>            <div class="md-contact-name">              {{$chip[$mdContactChipsCtrl.contactName]}}            </div>          </md-chip-template>      </md-chips>';
    /**
 * MDContactChips Directive Definition
 *
 * @param $mdTheming
 * @returns {*}
 * @ngInject
 */
    function MdContactChips($mdTheming, $mdUtil) {
      return {
        template: function (element, attrs) {
          return MD_CONTACT_CHIPS_TEMPLATE;
        },
        restrict: 'E',
        controller: 'MdContactChipsCtrl',
        controllerAs: '$mdContactChipsCtrl',
        bindToController: true,
        compile: compile,
        scope: {
          contactQuery: '&mdContacts',
          placeholder: '@',
          secondaryPlaceholder: '@',
          contactName: '@mdContactName',
          contactImage: '@mdContactImage',
          contactEmail: '@mdContactEmail',
          contacts: '=ngModel',
          requireMatch: '=?mdRequireMatch',
          highlightFlags: '@?mdHighlightFlags'
        }
      };
      function compile(element, attr) {
        return function postLink(scope, element, attrs, controllers) {
          $mdUtil.initOptionalProperties(scope, attr);
          $mdTheming(element);
          element.attr('tabindex', '-1');
        };
      }
    }
    MdContactChips.$inject = [
      '$mdTheming',
      '$mdUtil'
    ];
  }());
  (function () {
    'use strict';
    angular.module('material.components.icon').directive('mdIcon', [
      '$mdIcon',
      '$mdTheming',
      '$mdAria',
      mdIconDirective
    ]);
    /**
 * @ngdoc directive
 * @name mdIcon
 * @module material.components.icon
 *
 * @restrict E
 *
 * @description
 * The `md-icon` directive makes it easier to use vector-based icons in your app (as opposed to
 * raster-based icons types like PNG). The directive supports both icon fonts and SVG icons.
 *
 * Icons should be consider view-only elements that should not be used directly as buttons; instead nest a `<md-icon>`
 * inside a `md-button` to add hover and click features.
 *
 * ### Icon fonts
 * Icon fonts are a technique in which you use a font where the glyphs in the font are
 * your icons instead of text. Benefits include a straightforward way to bundle everything into a
 * single HTTP request, simple scaling, easy color changing, and more.
 *
 * `md-icon` lets you consume an icon font by letting you reference specific icons in that font
 * by name rather than character code.
 *
 * ### SVG
 * For SVGs, the problem with using `<img>` or a CSS `background-image` is that you can't take
 * advantage of some SVG features, such as styling specific parts of the icon with CSS or SVG
 * animation.
 *
 * `md-icon` makes it easier to use SVG icons by *inlining* the SVG into an `<svg>` element in the
 * document. The most straightforward way of referencing an SVG icon is via URL, just like a
 * traditional `<img>`. `$mdIconProvider`, as a convenience, lets you _name_ an icon so you can
 * reference it by name instead of URL throughout your templates.
 *
 * Additionally, you may not want to make separate HTTP requests for every icon, so you can bundle
 * your SVG icons together and pre-load them with $mdIconProvider as an icon set. An icon set can
 * also be given a name, which acts as a namespace for individual icons, so you can reference them
 * like `"social:cake"`.
 *
 * When using SVGs, both external SVGs (via URLs) or sets of SVGs [from icon sets] can be
 * easily loaded and used.When use font-icons, developers must following three (3) simple steps:
 *
 * <ol>
 * <li>Load the font library. e.g.<br/>
 *    &lt;link href="https://fonts.googleapis.com/icon?family=Material+Icons"
 *    rel="stylesheet"&gt;
 * </li>
 * <li> Use either (a) font-icon class names or (b) font ligatures to render the font glyph by using its textual name</li>
 * <li> Use &lt;md-icon md-font-icon="classname" /&gt; or <br/>
 *     use &lt;md-icon md-font-set="font library classname or alias"&gt; textual_name &lt;/md-icon&gt; or <br/>
 *     use &lt;md-icon md-font-set="font library classname or alias"&gt; numerical_character_reference &lt;/md-icon&gt;
 * </li>
 * </ol>
 *
 * Full details for these steps can be found:
 *
 * <ul>
 * <li>http://google.github.io/material-design-icons/</li>
 * <li>http://google.github.io/material-design-icons/#icon-font-for-the-web</li>
 * </ul>
 *
 * The Material Design icon style <code>.material-icons</code> and the icon font references are published in
 * Material Design Icons:
 *
 * <ul>
 * <li>http://www.google.com/design/icons/</li>
 * <li>https://www.google.com/design/icons/#ic_accessibility</li>
 * </ul>
 *
 * <h2 id="material_design_icons">Material Design Icons</h2>
 * Using the Material Design Icon-Selector, developers can easily and quickly search for a Material Design font-icon and
 * determine its textual name and character reference code. Click on any icon to see the slide-up information
 * panel with details regarding a SVG download or information on the font-icon usage.
 *
 * <a href="https://www.google.com/design/icons/#ic_accessibility" target="_blank" style="border-bottom:none;">
 * <img src="https://cloud.githubusercontent.com/assets/210413/7902490/fe8dd14c-0780-11e5-98fb-c821cc6475e6.png"
 *      aria-label="Material Design Icon-Selector" style="max-width:75%;padding-left:10%">
 * </a>
 *
 * <span class="image_caption">
 *  Click on the image above to link to the
 *  <a href="https://www.google.com/design/icons/#ic_accessibility" target="_blank">Material Design Icon-Selector</a>.
 * </span>
 *
 * @param {string} md-font-icon String name of CSS icon associated with the font-face will be used
 * to render the icon. Requires the fonts and the named CSS styles to be preloaded.
 * @param {string} md-font-set CSS style name associated with the font library; which will be assigned as
 * the class for the font-icon ligature. This value may also be an alias that is used to lookup the classname;
 * internally use `$mdIconProvider.fontSet(<alias>)` to determine the style name.
 * @param {string} md-svg-src String URL (or expression) used to load, cache, and display an
 *     external SVG.
 * @param {string} md-svg-icon md-svg-icon String name used for lookup of the icon from the internal cache;
 *     interpolated strings or expressions may also be used. Specific set names can be used with
 *     the syntax `<set name>:<icon name>`.<br/><br/>
 * To use icon sets, developers are required to pre-register the sets using the `$mdIconProvider` service.
 * @param {string=} aria-label Labels icon for accessibility. If an empty string is provided, icon
 * will be hidden from accessibility layer with `aria-hidden="true"`. If there's no aria-label on the icon
 * nor a label on the parent element, a warning will be logged to the console.
 * @param {string=} alt Labels icon for accessibility. If an empty string is provided, icon
 * will be hidden from accessibility layer with `aria-hidden="true"`. If there's no alt on the icon
 * nor a label on the parent element, a warning will be logged to the console.
 *
 * @usage
 * When using SVGs:
 * <hljs lang="html">
 *
 *  <!-- Icon ID; may contain optional icon set prefix; icons must registered using $mdIconProvider -->
 *  <md-icon md-svg-icon="social:android"    aria-label="android " ></md-icon>
 *
 *  <!-- Icon urls; may be preloaded in templateCache -->
 *  <md-icon md-svg-src="/android.svg"       aria-label="android " ></md-icon>
 *  <md-icon md-svg-src="{{ getAndroid() }}" aria-label="android " ></md-icon>
 *
 * </hljs>
 *
 * Use the <code>$mdIconProvider</code> to configure your application with
 * svg iconsets.
 *
 * <hljs lang="js">
 *  angular.module('appSvgIconSets', ['ngMaterial'])
 *    .controller('DemoCtrl', function($scope) {})
 *    .config(function($mdIconProvider) {
 *      $mdIconProvider
 *         .iconSet('social', 'img/icons/sets/social-icons.svg', 24)
 *         .defaultIconSet('img/icons/sets/core-icons.svg', 24);
 *     });
 * </hljs>
 *
 *
 * When using Font Icons with classnames:
 * <hljs lang="html">
 *
 *  <md-icon md-font-icon="android" aria-label="android" ></md-icon>
 *  <md-icon class="icon_home"      aria-label="Home"    ></md-icon>
 *
 * </hljs>
 *
 * When using Material Font Icons with ligatures:
 * <hljs lang="html">
 *  <!-- For Material Design Icons -->
 *  <!-- The class '.material-icons' is auto-added if a style has NOT been specified -->
 *  <md-icon> face </md-icon>
 *  <md-icon md-font-set="material-icons"> face </md-icon>
 *  <md-icon> #xE87C; </md-icon>
 *  <!-- The class '.material-icons' must be manually added if other styles are also specified-->
 *  <md-icon class="material-icons md-light md-48"> face </md-icon>
 * </hljs>
 *
 * When using other Font-Icon libraries:
 *
 * <hljs lang="js">
 *  // Specify a font-icon style alias
 *  angular.config(function($mdIconProvider) {
 *    $mdIconProvider.fontSet('fa', 'fontawesome');
 *  });
 * </hljs>
 *
 * <hljs lang="html">
 *  <md-icon md-font-set="fa">email</md-icon>
 * </hljs>
 *
 */
    function mdIconDirective($mdIcon, $mdTheming, $mdAria) {
      return {
        scope: {
          fontSet: '@mdFontSet',
          fontIcon: '@mdFontIcon',
          svgIcon: '@mdSvgIcon',
          svgSrc: '@mdSvgSrc'
        },
        restrict: 'E',
        link: postLink
      };
      /**
   * Directive postLink
   * Supports embedded SVGs, font-icons, & external SVGs
   */
      function postLink(scope, element, attr) {
        $mdTheming(element);
        prepareForFontIcon();
        // If using a font-icon, then the textual name of the icon itself
        // provides the aria-label.
        var label = attr.alt || scope.fontIcon || scope.svgIcon || element.text();
        var attrName = attr.$normalize(attr.$attr.mdSvgIcon || attr.$attr.mdSvgSrc || '');
        if (!attr['aria-label']) {
          if (label != '' && !parentsHaveText()) {
            $mdAria.expect(element, 'aria-label', label);
            $mdAria.expect(element, 'role', 'img');
          } else if (!element.text()) {
            // If not a font-icon with ligature, then
            // hide from the accessibility layer.
            $mdAria.expect(element, 'aria-hidden', 'true');
          }
        }
        if (attrName) {
          // Use either pre-configured SVG or URL source, respectively.
          attr.$observe(attrName, function (attrVal) {
            element.empty();
            if (attrVal) {
              $mdIcon(attrVal).then(function (svg) {
                element.append(svg);
              });
            }
          });
        }
        function parentsHaveText() {
          var parent = element.parent();
          if (parent.attr('aria-label') || parent.text()) {
            return true;
          } else if (parent.parent().attr('aria-label') || parent.parent().text()) {
            return true;
          }
          return false;
        }
        function prepareForFontIcon() {
          if (!scope.svgIcon && !scope.svgSrc) {
            if (scope.fontIcon) {
              element.addClass('md-font ' + scope.fontIcon);
            }
            element.addClass($mdIcon.fontSet(scope.fontSet));
          }
        }
      }
    }
  }());
  (function () {
    'use strict';
    angular.module('material.components.icon').provider('$mdIcon', MdIconProvider);
    /**
    * @ngdoc service
    * @name $mdIconProvider
    * @module material.components.icon
    *
    * @description
    * `$mdIconProvider` is used only to register icon IDs with URLs. These configuration features allow
    * icons and icon sets to be pre-registered and associated with source URLs **before** the `<md-icon />`
    * directives are compiled.
    *
    * If using font-icons, the developer is responsible for loading the fonts.
    *
    * If using SVGs, loading of the actual svg files are deferred to on-demand requests and are loaded
    * internally by the `$mdIcon` service using the `$http` service. When an SVG is requested by name/ID,
    * the `$mdIcon` service searches its registry for the associated source URL;
    * that URL is used to on-demand load and parse the SVG dynamically.
    *
    * @usage
    * <hljs lang="js">
    *   app.config(function($mdIconProvider) {
    *
    *     // Configure URLs for icons specified by [set:]id.
    *
    *     $mdIconProvider
    *          .defaultFontSet( 'fontawesome' )
    *          .defaultIconSet('my/app/icons.svg')       // Register a default set of SVG icons
    *          .iconSet('social', 'my/app/social.svg')   // Register a named icon set of SVGs
    *          .icon('android', 'my/app/android.svg')    // Register a specific icon (by name)
    *          .icon('work:chair', 'my/app/chair.svg');  // Register icon in a specific set
    *   });
    * </hljs>
    *
    * SVG icons and icon sets can be easily pre-loaded and cached using either (a) a build process or (b) a runtime
    * **startup** process (shown below):
    *
    * <hljs lang="js">
    *   app.config(function($mdIconProvider) {
    *
    *     // Register a default set of SVG icon definitions
    *     $mdIconProvider.defaultIconSet('my/app/icons.svg')
    *
    *   })
    *   .run(function($http, $templateCache){
    *
    *     // Pre-fetch icons sources by URL and cache in the $templateCache...
    *     // subsequent $http calls will look there first.
    *
    *     var urls = [ 'imy/app/icons.svg', 'img/icons/android.svg'];
    *
    *     angular.forEach(urls, function(url) {
    *       $http.get(url, {cache: $templateCache});
    *     });
    *
    *   });
    *
    * </hljs>
    *
    * NOTE: the loaded SVG data is subsequently cached internally for future requests.
    *
    */
    /**
    * @ngdoc method
    * @name $mdIconProvider#icon
    *
    * @description
    * Register a source URL for a specific icon name; the name may include optional 'icon set' name prefix.
    * These icons  will later be retrieved from the cache using `$mdIcon( <icon name> )`
    *
    * @param {string} id Icon name/id used to register the icon
    * @param {string} url specifies the external location for the data file. Used internally by `$http` to load the
    * data or as part of the lookup in `$templateCache` if pre-loading was configured.
    * @param {number=} viewBoxSize Sets the width and height the icon's viewBox.
    * It is ignored for icons with an existing viewBox. Default size is 24.
    *
    * @returns {obj} an `$mdIconProvider` reference; used to support method call chains for the API
    *
    * @usage
    * <hljs lang="js">
    *   app.config(function($mdIconProvider) {
    *
    *     // Configure URLs for icons specified by [set:]id.
    *
    *     $mdIconProvider
    *          .icon('android', 'my/app/android.svg')    // Register a specific icon (by name)
    *          .icon('work:chair', 'my/app/chair.svg');  // Register icon in a specific set
    *   });
    * </hljs>
    *
    */
    /**
    * @ngdoc method
    * @name $mdIconProvider#iconSet
    *
    * @description
    * Register a source URL for a 'named' set of icons; group of SVG definitions where each definition
    * has an icon id. Individual icons can be subsequently retrieved from this cached set using
    * `$mdIcon(<icon set name>:<icon name>)`
    *
    * @param {string} id Icon name/id used to register the iconset
    * @param {string} url specifies the external location for the data file. Used internally by `$http` to load the
    * data or as part of the lookup in `$templateCache` if pre-loading was configured.
    * @param {number=} viewBoxSize Sets the width and height of the viewBox of all icons in the set. 
    * It is ignored for icons with an existing viewBox. All icons in the icon set should be the same size.
    * Default value is 24.
    *
    * @returns {obj} an `$mdIconProvider` reference; used to support method call chains for the API
    *
    *
    * @usage
    * <hljs lang="js">
    *   app.config(function($mdIconProvider) {
    *
    *     // Configure URLs for icons specified by [set:]id.
    *
    *     $mdIconProvider
    *          .iconSet('social', 'my/app/social.svg')   // Register a named icon set
    *   });
    * </hljs>
    *
    */
    /**
    * @ngdoc method
    * @name $mdIconProvider#defaultIconSet
    *
    * @description
    * Register a source URL for the default 'named' set of icons. Unless explicitly registered,
    * subsequent lookups of icons will failover to search this 'default' icon set.
    * Icon can be retrieved from this cached, default set using `$mdIcon(<name>)`
    *
    * @param {string} url specifies the external location for the data file. Used internally by `$http` to load the
    * data or as part of the lookup in `$templateCache` if pre-loading was configured.
    * @param {number=} viewBoxSize Sets the width and height of the viewBox of all icons in the set. 
    * It is ignored for icons with an existing viewBox. All icons in the icon set should be the same size.
    * Default value is 24.
    *
    * @returns {obj} an `$mdIconProvider` reference; used to support method call chains for the API
    *
    * @usage
    * <hljs lang="js">
    *   app.config(function($mdIconProvider) {
    *
    *     // Configure URLs for icons specified by [set:]id.
    *
    *     $mdIconProvider
    *          .defaultIconSet( 'my/app/social.svg' )   // Register a default icon set
    *   });
    * </hljs>
    *
    */
    /**
   * @ngdoc method
   * @name $mdIconProvider#defaultFontSet
   *
   * @description
   * When using Font-Icons, Angular Material assumes the the Material Design icons will be used and automatically
   * configures the default font-set == 'material-icons'. Note that the font-set references the font-icon library
   * class style that should be applied to the `<md-icon>`.
   *
   * Configuring the default means that the attributes
   * `md-font-set="material-icons"` or `class="material-icons"` do not need to be explicitly declared on the
   * `<md-icon>` markup. For example:
   *
   *  `<md-icon> face </md-icon>`
   *  will render as
   *  `<span class="material-icons"> face </span>`, and
   *
   *  `<md-icon md-font-set="fa"> face </md-icon>`
   *  will render as
   *  `<span class="fa"> face </span>`
   *
   * @param {string} name of the font-library style that should be applied to the md-icon DOM element
   *
   * @usage
   * <hljs lang="js">
   *   app.config(function($mdIconProvider) {
   *     $mdIconProvider.defaultFontSet( 'fontawesome' );
   *   });
   * </hljs>
   *
   */
    /**
    * @ngdoc method
    * @name $mdIconProvider#defaultViewBoxSize
    *
    * @description
    * While `<md-icon />` markup can also be style with sizing CSS, this method configures
    * the default width **and** height used for all icons; unless overridden by specific CSS.
    * The default sizing is (24px, 24px).
    * @param {number=} viewBoxSize Sets the width and height of the viewBox for an icon or an icon set.
    * All icons in a set should be the same size. The default value is 24.
    *
    * @returns {obj} an `$mdIconProvider` reference; used to support method call chains for the API
    *
    * @usage
    * <hljs lang="js">
    *   app.config(function($mdIconProvider) {
    *
    *     // Configure URLs for icons specified by [set:]id.
    *
    *     $mdIconProvider
    *          .defaultViewBoxSize(36)   // Register a default icon size (width == height)
    *   });
    * </hljs>
    *
    */
    var config = {
        defaultViewBoxSize: 24,
        defaultFontSet: 'material-icons',
        fontSets: []
      };
    function MdIconProvider() {
    }
    MdIconProvider.prototype = {
      icon: function (id, url, viewBoxSize) {
        if (id.indexOf(':') == -1)
          id = '$default:' + id;
        config[id] = new ConfigurationItem(url, viewBoxSize);
        return this;
      },
      iconSet: function (id, url, viewBoxSize) {
        config[id] = new ConfigurationItem(url, viewBoxSize);
        return this;
      },
      defaultIconSet: function (url, viewBoxSize) {
        var setName = '$default';
        if (!config[setName]) {
          config[setName] = new ConfigurationItem(url, viewBoxSize);
        }
        config[setName].viewBoxSize = viewBoxSize || config.defaultViewBoxSize;
        return this;
      },
      defaultViewBoxSize: function (viewBoxSize) {
        config.defaultViewBoxSize = viewBoxSize;
        return this;
      },
      fontSet: function fontSet(alias, className) {
        config.fontSets.push({
          alias: alias,
          fontSet: className || alias
        });
        return this;
      },
      defaultFontSet: function defaultFontSet(className) {
        config.defaultFontSet = !className ? '' : className;
        return this;
      },
      defaultIconSize: function defaultIconSize(iconSize) {
        config.defaultIconSize = iconSize;
        return this;
      },
      preloadIcons: function ($templateCache) {
        var iconProvider = this;
        var svgRegistry = [
            {
              id: 'md-tabs-arrow',
              url: 'md-tabs-arrow.svg',
              svg: '<svg version="1.1" x="0px" y="0px" viewBox="0 0 24 24"><g><polygon points="15.4,7.4 14,6 8,12 14,18 15.4,16.6 10.8,12 "/></g></svg>'
            },
            {
              id: 'md-close',
              url: 'md-close.svg',
              svg: '<svg version="1.1" x="0px" y="0px" viewBox="0 0 24 24"><g><path d="M19 6.41l-1.41-1.41-5.59 5.59-5.59-5.59-1.41 1.41 5.59 5.59-5.59 5.59 1.41 1.41 5.59-5.59 5.59 5.59 1.41-1.41-5.59-5.59z"/></g></svg>'
            },
            {
              id: 'md-cancel',
              url: 'md-cancel.svg',
              svg: '<svg version="1.1" x="0px" y="0px" viewBox="0 0 24 24"><g><path d="M12 2c-5.53 0-10 4.47-10 10s4.47 10 10 10 10-4.47 10-10-4.47-10-10-10zm5 13.59l-1.41 1.41-3.59-3.59-3.59 3.59-1.41-1.41 3.59-3.59-3.59-3.59 1.41-1.41 3.59 3.59 3.59-3.59 1.41 1.41-3.59 3.59 3.59 3.59z"/></g></svg>'
            },
            {
              id: 'md-menu',
              url: 'md-menu.svg',
              svg: '<svg version="1.1" x="0px" y="0px" viewBox="0 0 24 24"><path d="M3,6H21V8H3V6M3,11H21V13H3V11M3,16H21V18H3V16Z" /></svg>'
            },
            {
              id: 'md-toggle-arrow',
              url: 'md-toggle-arrow-svg',
              svg: '<svg version="1.1" x="0px" y="0px" viewBox="0 0 48 48"><path d="M24 16l-12 12 2.83 2.83 9.17-9.17 9.17 9.17 2.83-2.83z"/><path d="M0 0h48v48h-48z" fill="none"/></svg>'
            },
            {
              id: 'md-calendar',
              url: 'md-calendar.svg',
              svg: '<svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24"><path d="M19 3h-1V1h-2v2H8V1H6v2H5c-1.11 0-1.99.9-1.99 2L3 19c0 1.1.89 2 2 2h14c1.1 0 2-.9 2-2V5c0-1.1-.9-2-2-2zm0 16H5V8h14v11zM7 10h5v5H7z"/></svg>'
            }
          ];
        svgRegistry.forEach(function (asset) {
          iconProvider.icon(asset.id, asset.url);
          $templateCache.put(asset.url, asset.svg);
        });
      },
      $get: [
        '$http',
        '$q',
        '$log',
        '$templateCache',
        function ($http, $q, $log, $templateCache) {
          this.preloadIcons($templateCache);
          return MdIconService(config, $http, $q, $log, $templateCache);
        }
      ]
    };
    /**
    *  Configuration item stored in the Icon registry; used for lookups
    *  to load if not already cached in the `loaded` cache
    */
    function ConfigurationItem(url, viewBoxSize) {
      this.url = url;
      this.viewBoxSize = viewBoxSize || config.defaultViewBoxSize;
    }
    /**
  * @ngdoc service
  * @name $mdIcon
  * @module material.components.icon
  *
  * @description
  * The `$mdIcon` service is a function used to lookup SVG icons.
  *
  * @param {string} id Query value for a unique Id or URL. If the argument is a URL, then the service will retrieve the icon element
  * from its internal cache or load the icon and cache it first. If the value is not a URL-type string, then an ID lookup is
  * performed. The Id may be a unique icon ID or may include an iconSet ID prefix.
  *
  * For the **id** query to work properly, this means that all id-to-URL mappings must have been previously configured
  * using the `$mdIconProvider`.
  *
  * @returns {obj} Clone of the initial SVG DOM element; which was created from the SVG markup in the SVG data file.
  *
  * @usage
  * <hljs lang="js">
  * function SomeDirective($mdIcon) {
  *
  *   // See if the icon has already been loaded, if not
  *   // then lookup the icon from the registry cache, load and cache
  *   // it for future requests.
  *   // NOTE: ID queries require configuration with $mdIconProvider
  *
  *   $mdIcon('android').then(function(iconEl)    { element.append(iconEl); });
  *   $mdIcon('work:chair').then(function(iconEl) { element.append(iconEl); });
  *
  *   // Load and cache the external SVG using a URL
  *
  *   $mdIcon('img/icons/android.svg').then(function(iconEl) {
  *     element.append(iconEl);
  *   });
  * };
  * </hljs>
  *
  * NOTE: The `<md-icon />  ` directive internally uses the `$mdIcon` service to query, loaded, and instantiate
  * SVG DOM elements.
  */
    /* @ngInject */
    function MdIconService(config, $http, $q, $log, $templateCache) {
      var iconCache = {};
      var urlRegex = /[-a-zA-Z0-9@:%_\+.~#?&//=]{2,256}\.[a-z]{2,4}\b(\/[-a-zA-Z0-9@:%_\+.~#?&//=]*)?/i;
      Icon.prototype = {
        clone: cloneSVG,
        prepare: prepareAndStyle
      };
      getIcon.fontSet = findRegisteredFontSet;
      // Publish service...
      return getIcon;
      /**
    * Actual $mdIcon service is essentially a lookup function
    */
      function getIcon(id) {
        id = id || '';
        // If already loaded and cached, use a clone of the cached icon.
        // Otherwise either load by URL, or lookup in the registry and then load by URL, and cache.
        if (iconCache[id])
          return $q.when(iconCache[id].clone());
        if (urlRegex.test(id))
          return loadByURL(id).then(cacheIcon(id));
        if (id.indexOf(':') == -1)
          id = '$default:' + id;
        var load = config[id] ? loadByID : loadFromIconSet;
        return load(id).then(cacheIcon(id));
      }
      /**
    * Lookup registered fontSet style using its alias...
    * If not found,
    */
      function findRegisteredFontSet(alias) {
        var useDefault = angular.isUndefined(alias) || !(alias && alias.length);
        if (useDefault)
          return config.defaultFontSet;
        var result = alias;
        angular.forEach(config.fontSets, function (it) {
          if (it.alias == alias)
            result = it.fontSet || result;
        });
        return result;
      }
      /**
    * Prepare and cache the loaded icon for the specified `id`
    */
      function cacheIcon(id) {
        return function updateCache(icon) {
          iconCache[id] = isIcon(icon) ? icon : new Icon(icon, config[id]);
          return iconCache[id].clone();
        };
      }
      /**
    * Lookup the configuration in the registry, if !registered throw an error
    * otherwise load the icon [on-demand] using the registered URL.
    *
    */
      function loadByID(id) {
        var iconConfig = config[id];
        return loadByURL(iconConfig.url).then(function (icon) {
          return new Icon(icon, iconConfig);
        });
      }
      /**
    *    Loads the file as XML and uses querySelector( <id> ) to find
    *    the desired node...
    */
      function loadFromIconSet(id) {
        var setName = id.substring(0, id.lastIndexOf(':')) || '$default';
        var iconSetConfig = config[setName];
        return !iconSetConfig ? announceIdNotFound(id) : loadByURL(iconSetConfig.url).then(extractFromSet);
        function extractFromSet(set) {
          var iconName = id.slice(id.lastIndexOf(':') + 1);
          var icon = set.querySelector('#' + iconName);
          return !icon ? announceIdNotFound(id) : new Icon(icon, iconSetConfig);
        }
        function announceIdNotFound(id) {
          var msg = 'icon ' + id + ' not found';
          $log.warn(msg);
          return $q.reject(msg || id);
        }
      }
      /**
    * Load the icon by URL (may use the $templateCache).
    * Extract the data for later conversion to Icon
    */
      function loadByURL(url) {
        return $http.get(url, { cache: $templateCache }).then(function (response) {
          return angular.element('<div>').append(response.data).find('svg')[0];
        }).catch(announceNotFound);
      }
      /**
    * Catch HTTP or generic errors not related to incorrect icon IDs.
    */
      function announceNotFound(err) {
        var msg = angular.isString(err) ? err : err.message || err.data || err.statusText;
        $log.warn(msg);
        return $q.reject(msg);
      }
      /**
    * Check target signature to see if it is an Icon instance.
    */
      function isIcon(target) {
        return angular.isDefined(target.element) && angular.isDefined(target.config);
      }
      /**
    *  Define the Icon class
    */
      function Icon(el, config) {
        if (el && el.tagName != 'svg') {
          el = angular.element('<svg xmlns="http://www.w3.org/2000/svg">').append(el)[0];
        }
        // Inject the namespace if not available...
        if (!el.getAttribute('xmlns')) {
          el.setAttribute('xmlns', 'http://www.w3.org/2000/svg');
        }
        this.element = el;
        this.config = config;
        this.prepare();
      }
      /**
    *  Prepare the DOM element that will be cached in the
    *  loaded iconCache store.
    */
      function prepareAndStyle() {
        var viewBoxSize = this.config ? this.config.viewBoxSize : config.defaultViewBoxSize;
        angular.forEach({
          'fit': '',
          'height': '100%',
          'width': '100%',
          'preserveAspectRatio': 'xMidYMid meet',
          'viewBox': this.element.getAttribute('viewBox') || '0 0 ' + viewBoxSize + ' ' + viewBoxSize
        }, function (val, attr) {
          this.element.setAttribute(attr, val);
        }, this);
        angular.forEach({
          'pointer-events': 'none',
          'display': 'block'
        }, function (val, style) {
          this.element.style[style] = val;
        }, this);
      }
      /**
    * Clone the Icon DOM element.
    */
      function cloneSVG() {
        return this.element.cloneNode(true);
      }
    }
    MdIconService.$inject = [
      'config',
      '$http',
      '$q',
      '$log',
      '$templateCache'
    ];
  }());
  (function () {
    'use strict';
    angular.module('material.components.menu').controller('mdMenuCtrl', MenuController);
    /**
 * @ngInject
 */
    function MenuController($mdMenu, $attrs, $element, $scope, $mdUtil, $timeout, $rootScope, $q) {
      var menuContainer;
      var self = this;
      var triggerElement;
      this.nestLevel = parseInt($attrs.mdNestLevel, 10) || 0;
      /**
   * Called by our linking fn to provide access to the menu-content
   * element removed during link
   */
      this.init = function init(setMenuContainer, opts) {
        opts = opts || {};
        menuContainer = setMenuContainer;
        // Default element for ARIA attributes has the ngClick or ngMouseenter expression
        triggerElement = $element[0].querySelector('[ng-click],[ng-mouseenter]');
        triggerElement.setAttribute('aria-expanded', 'false');
        this.isInMenuBar = opts.isInMenuBar;
        this.nestedMenus = $mdUtil.nodesToArray(menuContainer[0].querySelectorAll('.md-nested-menu'));
        menuContainer.on('$mdInterimElementRemove', function () {
          self.isOpen = false;
        });
        var menuContainerId = 'menu_container_' + $mdUtil.nextUid();
        menuContainer.attr('id', menuContainerId);
        angular.element(triggerElement).attr({
          'aria-owns': menuContainerId,
          'aria-haspopup': 'true'
        });
        $scope.$on('$destroy', this.disableHoverListener);
      };
      var openMenuTimeout, menuItems, deregisterScopeListeners = [];
      this.enableHoverListener = function () {
        deregisterScopeListeners.push($rootScope.$on('$mdMenuOpen', function (event, el) {
          if (menuContainer[0].contains(el[0])) {
            self.currentlyOpenMenu = el.controller('mdMenu');
            self.isAlreadyOpening = false;
            self.currentlyOpenMenu.registerContainerProxy(self.triggerContainerProxy.bind(self));
          }
        }));
        deregisterScopeListeners.push($rootScope.$on('$mdMenuClose', function (event, el) {
          if (menuContainer[0].contains(el[0])) {
            self.currentlyOpenMenu = undefined;
          }
        }));
        menuItems = angular.element($mdUtil.nodesToArray(menuContainer[0].children[0].children));
        menuItems.on('mouseenter', self.handleMenuItemHover);
        menuItems.on('mouseleave', self.handleMenuItemMouseLeave);
      };
      this.disableHoverListener = function () {
        while (deregisterScopeListeners.length) {
          deregisterScopeListeners.shift()();
        }
        menuItems && menuItems.off('mouseenter', self.handleMenuItemHover);
        menuItems && menuItems.off('mouseleave', self.handleMenuMouseLeave);
      };
      this.handleMenuItemHover = function (event) {
        if (self.isAlreadyOpening)
          return;
        var nestedMenu = event.target.querySelector('md-menu') || $mdUtil.getClosest(event.target, 'MD-MENU');
        openMenuTimeout = $timeout(function () {
          if (nestedMenu) {
            nestedMenu = angular.element(nestedMenu).controller('mdMenu');
          }
          if (self.currentlyOpenMenu && self.currentlyOpenMenu != nestedMenu) {
            var closeTo = self.nestLevel + 1;
            self.currentlyOpenMenu.close(true, { closeTo: closeTo });
          } else if (nestedMenu && !nestedMenu.isOpen && nestedMenu.open) {
            self.isAlreadyOpening = true;
            nestedMenu.open();
          }
        }, nestedMenu ? 100 : 250);
        var focusableTarget = event.currentTarget.querySelector('button:not([disabled])');
        focusableTarget && focusableTarget.focus();
      };
      this.handleMenuItemMouseLeave = function () {
        if (openMenuTimeout) {
          $timeout.cancel(openMenuTimeout);
          openMenuTimeout = undefined;
        }
      };
      /**
   * Uses the $mdMenu interim element service to open the menu contents
   */
      this.open = function openMenu(ev) {
        ev && ev.stopPropagation();
        ev && ev.preventDefault();
        if (self.isOpen)
          return;
        self.enableHoverListener();
        self.isOpen = true;
        triggerElement = triggerElement || (ev ? ev.target : $element[0]);
        triggerElement.setAttribute('aria-expanded', 'true');
        $scope.$emit('$mdMenuOpen', $element);
        $mdMenu.show({
          scope: $scope,
          mdMenuCtrl: self,
          nestLevel: self.nestLevel,
          element: menuContainer,
          target: triggerElement,
          preserveElement: true,
          parent: 'body'
        }).finally(function () {
          triggerElement.setAttribute('aria-expanded', 'false');
          self.disableHoverListener();
        });
      };
      // Expose a open function to the child scope for html to use
      $scope.$mdOpenMenu = this.open;
      $scope.$watch(function () {
        return self.isOpen;
      }, function (isOpen) {
        if (isOpen) {
          menuContainer.attr('aria-hidden', 'false');
          $element[0].classList.add('md-open');
          angular.forEach(self.nestedMenus, function (el) {
            el.classList.remove('md-open');
          });
        } else {
          menuContainer.attr('aria-hidden', 'true');
          $element[0].classList.remove('md-open');
        }
        $scope.$mdMenuIsOpen = self.isOpen;
      });
      this.focusMenuContainer = function focusMenuContainer() {
        var focusTarget = menuContainer[0].querySelector('[md-menu-focus-target]');
        if (!focusTarget)
          focusTarget = menuContainer[0].querySelector('.md-button');
        focusTarget.focus();
      };
      this.registerContainerProxy = function registerContainerProxy(handler) {
        this.containerProxy = handler;
      };
      this.triggerContainerProxy = function triggerContainerProxy(ev) {
        this.containerProxy && this.containerProxy(ev);
      };
      this.destroy = function () {
        return self.isOpen ? $mdMenu.destroy() : $q.when(false);
      };
      // Use the $mdMenu interim element service to close the menu contents
      this.close = function closeMenu(skipFocus, closeOpts) {
        if (!self.isOpen)
          return;
        self.isOpen = false;
        var eventDetails = angular.extend({}, closeOpts, { skipFocus: skipFocus });
        $scope.$emit('$mdMenuClose', $element, eventDetails);
        $mdMenu.hide(null, closeOpts);
        if (!skipFocus) {
          var el = self.restoreFocusTo || $element.find('button')[0];
          if (el instanceof angular.element)
            el = el[0];
          if (el)
            el.focus();
        }
      };
      /**
   * Build a nice object out of our string attribute which specifies the
   * target mode for left and top positioning
   */
      this.positionMode = function positionMode() {
        var attachment = ($attrs.mdPositionMode || 'target').split(' ');
        // If attachment is a single item, duplicate it for our second value.
        // ie. 'target' -> 'target target'
        if (attachment.length == 1) {
          attachment.push(attachment[0]);
        }
        return {
          left: attachment[0],
          top: attachment[1]
        };
      };
      /**
   * Build a nice object out of our string attribute which specifies
   * the offset of top and left in pixels.
   */
      this.offsets = function offsets() {
        var position = ($attrs.mdOffset || '0 0').split(' ').map(parseFloat);
        if (position.length == 2) {
          return {
            left: position[0],
            top: position[1]
          };
        } else if (position.length == 1) {
          return {
            top: position[0],
            left: position[0]
          };
        } else {
          throw Error('Invalid offsets specified. Please follow format <x, y> or <n>');
        }
      };
    }
    MenuController.$inject = [
      '$mdMenu',
      '$attrs',
      '$element',
      '$scope',
      '$mdUtil',
      '$timeout',
      '$rootScope',
      '$q'
    ];
  }());
  (function () {
    'use strict';
    /**
 * @ngdoc directive
 * @name mdMenu
 * @module material.components.menu
 * @restrict E
 * @description
 *
 * Menus are elements that open when clicked. They are useful for displaying
 * additional options within the context of an action.
 *
 * Every `md-menu` must specify exactly two child elements. The first element is what is
 * left in the DOM and is used to open the menu. This element is called the trigger element.
 * The trigger element's scope has access to `$mdOpenMenu($event)`
 * which it may call to open the menu. By passing $event as argument, the
 * corresponding event is stopped from propagating up the DOM-tree.
 *
 * The second element is the `md-menu-content` element which represents the
 * contents of the menu when it is open. Typically this will contain `md-menu-item`s,
 * but you can do custom content as well.
 *
 * <hljs lang="html">
 * <md-menu>
 *  <!-- Trigger element is a md-button with an icon -->
 *  <md-button ng-click="$mdOpenMenu($event)" class="md-icon-button" aria-label="Open sample menu">
 *    <md-icon md-svg-icon="call:phone"></md-icon>
 *  </md-button>
 *  <md-menu-content>
 *    <md-menu-item><md-button ng-click="doSomething()">Do Something</md-button></md-menu-item>
 *  </md-menu-content>
 * </md-menu>
 * </hljs>

 * ## Sizing Menus
 *
 * The width of the menu when it is open may be specified by specifying a `width`
 * attribute on the `md-menu-content` element.
 * See the [Material Design Spec](http://www.google.com/design/spec/components/menus.html#menus-specs)
 * for more information.
 *
 *
 * ## Aligning Menus
 *
 * When a menu opens, it is important that the content aligns with the trigger element.
 * Failure to align menus can result in jarring experiences for users as content
 * suddenly shifts. To help with this, `md-menu` provides serveral APIs to help
 * with alignment.
 *
 * ### Target Mode
 *
 * By default, `md-menu` will attempt to align the `md-menu-content` by aligning
 * designated child elements in both the trigger and the menu content.
 *
 * To specify the alignment element in the `trigger` you can use the `md-menu-origin`
 * attribute on a child element. If no `md-menu-origin` is specified, the `md-menu`
 * will be used as the origin element.
 *
 * Similarly, the `md-menu-content` may specify a `md-menu-align-target` for a
 * `md-menu-item` to specify the node that it should try and align with.
 *
 * In this example code, we specify an icon to be our origin element, and an
 * icon in our menu content to be our alignment target. This ensures that both
 * icons are aligned when the menu opens.
 *
 * <hljs lang="html">
 * <md-menu>
 *  <md-button ng-click="$mdOpenMenu($event)" class="md-icon-button" aria-label="Open some menu">
 *    <md-icon md-menu-origin md-svg-icon="call:phone"></md-icon>
 *  </md-button>
 *  <md-menu-content>
 *    <md-menu-item>
 *      <md-button ng-click="doSomething()" aria-label="Do something">
 *        <md-icon md-menu-align-target md-svg-icon="call:phone"></md-icon>
 *        Do Something
 *      </md-button>
 *    </md-menu-item>
 *  </md-menu-content>
 * </md-menu>
 * </hljs>
 *
 * Sometimes we want to specify alignment on the right side of an element, for example
 * if we have a menu on the right side a toolbar, we want to right align our menu content.
 *
 * We can specify the origin by using the `md-position-mode` attribute on both
 * the `x` and `y` axis. Right now only the `x-axis` has more than one option.
 * You may specify the default mode of `target target` or
 * `target-right target` to specify a right-oriented alignment target. See the
 * position section of the demos for more examples.
 *
 * ### Menu Offsets
 *
 * It is sometimes unavoidable to need to have a deeper level of control for
 * the positioning of a menu to ensure perfect alignment. `md-menu` provides
 * the `md-offset` attribute to allow pixel level specificty of adjusting the
 * exact positioning.
 *
 * This offset is provided in the format of `x y` or `n` where `n` will be used
 * in both the `x` and `y` axis.
 *
 * For example, to move a menu by `2px` from the top, we can use:
 * <hljs lang="html">
 * <md-menu md-offset="2 0">
 *   <!-- menu-content -->
 * </md-menu>
 * </hljs>

 * ### Preventing close
 *
 * Sometimes you would like to be able to click on a menu item without having the menu
 * close. To do this, ngMaterial exposes the `md-prevent-menu-close` attribute which
 * can be added to a button inside a menu to stop the menu from automatically closing.
 * You can then close the menu programatically by injecting `$mdMenu` and calling 
 * `$mdMenu.hide()`.
 *
 * <hljs lang="html">
 * <md-menu-item>
 *   <md-button ng-click="doSomething()" aria-label="Do something" md-prevent-menu-close="md-prevent-menu-close">
 *     <md-icon md-menu-align-target md-svg-icon="call:phone"></md-icon>
 *     Do Something
 *   </md-button>
 * </md-menu-item>
 * </hljs>
 *
 * @usage
 * <hljs lang="html">
 * <md-menu>
 *  <md-button ng-click="$mdOpenMenu($event)" class="md-icon-button">
 *    <md-icon md-svg-icon="call:phone"></md-icon>
 *  </md-button>
 *  <md-menu-content>
 *    <md-menu-item><md-button ng-click="doSomething()">Do Something</md-button></md-menu-item>
 *  </md-menu-content>
 * </md-menu>
 * </hljs>
 *
 * @param {string} md-position-mode The position mode in the form of
 *           `x`, `y`. Default value is `target`,`target`. Right now the `x` axis
 *           also suppports `target-right`.
 * @param {string} md-offset An offset to apply to the dropdown after positioning
 *           `x`, `y`. Default value is `0`,`0`.
 *
 */
    angular.module('material.components.menu').directive('mdMenu', MenuDirective);
    /**
 * @ngInject
 */
    function MenuDirective($mdUtil) {
      var INVALID_PREFIX = 'Invalid HTML for md-menu: ';
      return {
        restrict: 'E',
        require: [
          'mdMenu',
          '?^mdMenuBar'
        ],
        controller: 'mdMenuCtrl',
        scope: true,
        compile: compile
      };
      function compile(templateElement) {
        templateElement.addClass('md-menu');
        var triggerElement = templateElement.children()[0];
        if (!triggerElement.hasAttribute('ng-click')) {
          triggerElement = triggerElement.querySelector('[ng-click],[ng-mouseenter]') || triggerElement;
        }
        if (triggerElement && (triggerElement.nodeName == 'MD-BUTTON' || triggerElement.nodeName == 'BUTTON') && !triggerElement.hasAttribute('type')) {
          triggerElement.setAttribute('type', 'button');
        }
        if (templateElement.children().length != 2) {
          throw Error(INVALID_PREFIX + 'Expected two children elements.');
        }
        // Default element for ARIA attributes has the ngClick or ngMouseenter expression
        triggerElement && triggerElement.setAttribute('aria-haspopup', 'true');
        var nestedMenus = templateElement[0].querySelectorAll('md-menu');
        var nestingDepth = parseInt(templateElement[0].getAttribute('md-nest-level'), 10) || 0;
        if (nestedMenus) {
          angular.forEach($mdUtil.nodesToArray(nestedMenus), function (menuEl) {
            if (!menuEl.hasAttribute('md-position-mode')) {
              menuEl.setAttribute('md-position-mode', 'cascade');
            }
            menuEl.classList.add('md-nested-menu');
            menuEl.setAttribute('md-nest-level', nestingDepth + 1);
          });
        }
        return link;
      }
      function link(scope, element, attrs, ctrls) {
        var mdMenuCtrl = ctrls[0];
        var isInMenuBar = ctrls[1] != undefined;
        // Move everything into a md-menu-container and pass it to the controller
        var menuContainer = angular.element('<div class="md-open-menu-container md-whiteframe-z2"></div>');
        var menuContents = element.children()[1];
        if (!menuContents.hasAttribute('role')) {
          menuContents.setAttribute('role', 'menu');
        }
        menuContainer.append(menuContents);
        element.on('$destroy', function () {
          menuContainer.remove();
        });
        element.append(menuContainer);
        menuContainer[0].style.display = 'none';
        mdMenuCtrl.init(menuContainer, { isInMenuBar: isInMenuBar });
      }
    }
    MenuDirective.$inject = ['$mdUtil'];
  }());
  (function () {
    'use strict';
    angular.module('material.components.menu').provider('$mdMenu', MenuProvider);
    /*
 * Interim element provider for the menu.
 * Handles behavior for a menu while it is open, including:
 *    - handling animating the menu opening/closing
 *    - handling key/mouse events on the menu element
 *    - handling enabling/disabling scroll while the menu is open
 *    - handling redrawing during resizes and orientation changes
 *
 */
    function MenuProvider($$interimElementProvider) {
      var MENU_EDGE_MARGIN = 8;
      menuDefaultOptions.$inject = [
        '$mdUtil',
        '$mdTheming',
        '$mdConstant',
        '$document',
        '$window',
        '$q',
        '$$rAF',
        '$animateCss',
        '$animate'
      ];
      return $$interimElementProvider('$mdMenu').setDefaults({
        methods: ['target'],
        options: menuDefaultOptions
      });
      /* @ngInject */
      function menuDefaultOptions($mdUtil, $mdTheming, $mdConstant, $document, $window, $q, $$rAF, $animateCss, $animate) {
        var animator = $mdUtil.dom.animator;
        return {
          parent: 'body',
          onShow: onShow,
          onRemove: onRemove,
          hasBackdrop: true,
          disableParentScroll: true,
          skipCompile: true,
          preserveScope: true,
          skipHide: true,
          themable: true
        };
        /**
     * Show modal backdrop element...
     * @returns {function(): void} A function that removes this backdrop
     */
        function showBackdrop(scope, element, options) {
          if (options.nestLevel)
            return angular.noop;
          // If we are not within a dialog...
          if (options.disableParentScroll && !$mdUtil.getClosest(options.target, 'MD-DIALOG')) {
            // !! DO this before creating the backdrop; since disableScrollAround()
            //    configures the scroll offset; which is used by mdBackDrop postLink()
            options.restoreScroll = $mdUtil.disableScrollAround(options.element, options.parent);
          } else {
            options.disableParentScroll = false;
          }
          if (options.hasBackdrop) {
            options.backdrop = $mdUtil.createBackdrop(scope, 'md-menu-backdrop md-click-catcher');
            $animate.enter(options.backdrop, $document[0].body);
          }
          /**
       * Hide and destroys the backdrop created by showBackdrop()
       */
          return function hideBackdrop() {
            if (options.backdrop)
              options.backdrop.remove();
            if (options.disableParentScroll)
              options.restoreScroll();
          };
        }
        /**
     * Removing the menu element from the DOM and remove all associated evetn listeners
     * and backdrop
     */
        function onRemove(scope, element, opts) {
          opts.cleanupInteraction();
          opts.cleanupResizing();
          opts.hideBackdrop();
          // For navigation $destroy events, do a quick, non-animated removal,
          // but for normal closes (from clicks, etc) animate the removal
          return opts.$destroy === true ? detachAndClean() : animateRemoval().then(detachAndClean);
          /**
       * For normal closes, animate the removal.
       * For forced closes (like $destroy events), skip the animations
       */
          function animateRemoval() {
            return $animateCss(element, { addClass: 'md-leave' }).start();
          }
          /**
       * Detach the element
       */
          function detachAndClean() {
            element.removeClass('md-active');
            detachElement(element, opts);
            opts.alreadyOpen = false;
          }
        }
        /**
     * Inserts and configures the staged Menu element into the DOM, positioning it,
     * and wiring up various interaction events
     */
        function onShow(scope, element, opts) {
          sanitizeAndConfigure(opts);
          // Wire up theming on our menu element
          $mdTheming.inherit(opts.menuContentEl, opts.target);
          // Register various listeners to move menu on resize/orientation change
          opts.cleanupResizing = startRepositioningOnResize();
          opts.hideBackdrop = showBackdrop(scope, element, opts);
          // Return the promise for when our menu is done animating in
          return showMenu().then(function (response) {
            opts.alreadyOpen = true;
            opts.cleanupInteraction = activateInteraction();
            return response;
          });
          /**
       * Place the menu into the DOM and call positioning related functions
       */
          function showMenu() {
            opts.parent.append(element);
            element[0].style.display = '';
            return $q(function (resolve) {
              var position = calculateMenuPosition(element, opts);
              element.removeClass('md-leave');
              // Animate the menu scaling, and opacity [from its position origin (default == top-left)]
              // to normal scale.
              $animateCss(element, {
                addClass: 'md-active',
                from: animator.toCss(position),
                to: animator.toCss({ transform: '' })
              }).start().then(resolve);
            });
          }
          /**
       * Check for valid opts and set some sane defaults
       */
          function sanitizeAndConfigure() {
            if (!opts.target) {
              throw Error('$mdMenu.show() expected a target to animate from in options.target');
            }
            angular.extend(opts, {
              alreadyOpen: false,
              isRemoved: false,
              target: angular.element(opts.target),
              parent: angular.element(opts.parent),
              menuContentEl: angular.element(element[0].querySelector('md-menu-content'))
            });
          }
          /**
       * Configure various resize listeners for screen changes
       */
          function startRepositioningOnResize() {
            var repositionMenu = function (target, options) {
                return $$rAF.throttle(function () {
                  if (opts.isRemoved)
                    return;
                  var position = calculateMenuPosition(target, options);
                  target.css(animator.toCss(position));
                });
              }(element, opts);
            $window.addEventListener('resize', repositionMenu);
            $window.addEventListener('orientationchange', repositionMenu);
            return function stopRepositioningOnResize() {
              // Disable resizing handlers
              $window.removeEventListener('resize', repositionMenu);
              $window.removeEventListener('orientationchange', repositionMenu);
            };
          }
          /**
       * Activate interaction on the menu. Wire up keyboard listerns for
       * clicks, keypresses, backdrop closing, etc.
       */
          function activateInteraction() {
            element.addClass('md-clickable');
            // close on backdrop click
            if (opts.backdrop)
              opts.backdrop.on('click', onBackdropClick);
            // Wire up keyboard listeners.
            // - Close on escape,
            // - focus next item on down arrow,
            // - focus prev item on up
            opts.menuContentEl.on('keydown', onMenuKeyDown);
            opts.menuContentEl[0].addEventListener('click', captureClickListener, true);
            // kick off initial focus in the menu on the first element
            var focusTarget = opts.menuContentEl[0].querySelector('[md-menu-focus-target]');
            if (!focusTarget) {
              var firstChild = opts.menuContentEl[0].firstElementChild;
              focusTarget = firstChild && (firstChild.querySelector('.md-button:not([disabled])') || firstChild.firstElementChild);
            }
            focusTarget && focusTarget.focus();
            return function cleanupInteraction() {
              element.removeClass('md-clickable');
              if (opts.backdrop)
                opts.backdrop.off('click', onBackdropClick);
              opts.menuContentEl.off('keydown', onMenuKeyDown);
              opts.menuContentEl[0].removeEventListener('click', captureClickListener, true);
            };
            // ************************************
            // internal functions
            // ************************************
            function onMenuKeyDown(ev) {
              var handled;
              switch (ev.keyCode) {
              case $mdConstant.KEY_CODE.ESCAPE:
                opts.mdMenuCtrl.close(false, { closeAll: true });
                handled = true;
                break;
              case $mdConstant.KEY_CODE.UP_ARROW:
                if (!focusMenuItem(ev, opts.menuContentEl, opts, -1) && !opts.nestLevel) {
                  opts.mdMenuCtrl.triggerContainerProxy(ev);
                }
                handled = true;
                break;
              case $mdConstant.KEY_CODE.DOWN_ARROW:
                if (!focusMenuItem(ev, opts.menuContentEl, opts, 1) && !opts.nestLevel) {
                  opts.mdMenuCtrl.triggerContainerProxy(ev);
                }
                handled = true;
                break;
              case $mdConstant.KEY_CODE.LEFT_ARROW:
                if (opts.nestLevel) {
                  opts.mdMenuCtrl.close();
                } else {
                  opts.mdMenuCtrl.triggerContainerProxy(ev);
                }
                handled = true;
                break;
              case $mdConstant.KEY_CODE.RIGHT_ARROW:
                var parentMenu = $mdUtil.getClosest(ev.target, 'MD-MENU');
                if (parentMenu && parentMenu != opts.parent[0]) {
                  ev.target.click();
                } else {
                  opts.mdMenuCtrl.triggerContainerProxy(ev);
                }
                handled = true;
                break;
              }
              if (handled) {
                ev.preventDefault();
                ev.stopImmediatePropagation();
              }
            }
            function onBackdropClick(e) {
              e.preventDefault();
              e.stopPropagation();
              scope.$apply(function () {
                opts.mdMenuCtrl.close(true, { closeAll: true });
              });
            }
            // Close menu on menu item click, if said menu-item is not disabled
            function captureClickListener(e) {
              var target = e.target;
              // Traverse up the event until we get to the menuContentEl to see if
              // there is an ng-click and that the ng-click is not disabled
              do {
                if (target == opts.menuContentEl[0])
                  return;
                if ((hasAnyAttribute(target, [
                    'ng-click',
                    'ng-href',
                    'ui-sref'
                  ]) || target.nodeName == 'BUTTON' || target.nodeName == 'MD-BUTTON') && !hasAnyAttribute(target, ['md-prevent-menu-close'])) {
                  var closestMenu = $mdUtil.getClosest(target, 'MD-MENU');
                  if (!target.hasAttribute('disabled') && (!closestMenu || closestMenu == opts.parent[0])) {
                    close();
                  }
                  break;
                }
              } while (target = target.parentNode);
              function close() {
                scope.$apply(function () {
                  opts.mdMenuCtrl.close(true, { closeAll: true });
                });
              }
              function hasAnyAttribute(target, attrs) {
                if (!target)
                  return false;
                for (var i = 0, attr; attr = attrs[i]; ++i) {
                  var altForms = [
                      attr,
                      'data-' + attr,
                      'x-' + attr
                    ];
                  for (var j = 0, rawAttr; rawAttr = altForms[j]; ++j) {
                    if (target.hasAttribute(rawAttr)) {
                      return true;
                    }
                  }
                }
                return false;
              }
            }
            opts.menuContentEl[0].addEventListener('click', captureClickListener, true);
            return function cleanupInteraction() {
              element.removeClass('md-clickable');
              opts.menuContentEl.off('keydown');
              opts.menuContentEl[0].removeEventListener('click', captureClickListener, true);
            };
          }
        }
        /**
     * Takes a keypress event and focuses the next/previous menu
     * item from the emitting element
     * @param {event} e - The origin keypress event
     * @param {angular.element} menuEl - The menu element
     * @param {object} opts - The interim element options for the mdMenu
     * @param {number} direction - The direction to move in (+1 = next, -1 = prev)
     */
        function focusMenuItem(e, menuEl, opts, direction) {
          var currentItem = $mdUtil.getClosest(e.target, 'MD-MENU-ITEM');
          var items = $mdUtil.nodesToArray(menuEl[0].children);
          var currentIndex = items.indexOf(currentItem);
          // Traverse through our elements in the specified direction (+/-1) and try to
          // focus them until we find one that accepts focus
          var didFocus;
          for (var i = currentIndex + direction; i >= 0 && i < items.length; i = i + direction) {
            var focusTarget = items[i].querySelector('.md-button');
            didFocus = attemptFocus(focusTarget);
            if (didFocus) {
              break;
            }
          }
          return didFocus;
        }
        /**
     * Attempts to focus an element. Checks whether that element is the currently
     * focused element after attempting.
     * @param {HTMLElement} el - the element to attempt focus on
     * @returns {bool} - whether the element was successfully focused
     */
        function attemptFocus(el) {
          if (el && el.getAttribute('tabindex') != -1) {
            el.focus();
            return $document[0].activeElement == el;
          }
        }
        /**
     * Use browser to remove this element without triggering a $destroy event
     */
        function detachElement(element, opts) {
          if (!opts.preserveElement) {
            if (toNode(element).parentNode === toNode(opts.parent)) {
              toNode(opts.parent).removeChild(toNode(element));
            }
          } else {
            toNode(element).style.display = 'none';
          }
        }
        /**
     * Computes menu position and sets the style on the menu container
     * @param {HTMLElement} el - the menu container element
     * @param {object} opts - the interim element options object
     */
        function calculateMenuPosition(el, opts) {
          var containerNode = el[0], openMenuNode = el[0].firstElementChild, openMenuNodeRect = openMenuNode.getBoundingClientRect(), boundryNode = $document[0].body, boundryNodeRect = boundryNode.getBoundingClientRect();
          var menuStyle = $window.getComputedStyle(openMenuNode);
          var originNode = opts.target[0].querySelector('[md-menu-origin]') || opts.target[0], originNodeRect = originNode.getBoundingClientRect();
          var bounds = {
              left: boundryNodeRect.left + MENU_EDGE_MARGIN,
              top: Math.max(boundryNodeRect.top, 0) + MENU_EDGE_MARGIN,
              bottom: Math.max(boundryNodeRect.bottom, Math.max(boundryNodeRect.top, 0) + boundryNodeRect.height) - MENU_EDGE_MARGIN,
              right: boundryNodeRect.right - MENU_EDGE_MARGIN
            };
          var alignTarget, alignTargetRect = {
              top: 0,
              left: 0,
              right: 0,
              bottom: 0
            }, existingOffsets = {
              top: 0,
              left: 0,
              right: 0,
              bottom: 0
            };
          var positionMode = opts.mdMenuCtrl.positionMode();
          if (positionMode.top == 'target' || positionMode.left == 'target' || positionMode.left == 'target-right') {
            alignTarget = firstVisibleChild();
            if (alignTarget) {
              // TODO: Allow centering on an arbitrary node, for now center on first menu-item's child
              alignTarget = alignTarget.firstElementChild || alignTarget;
              alignTarget = alignTarget.querySelector('[md-menu-align-target]') || alignTarget;
              alignTargetRect = alignTarget.getBoundingClientRect();
              existingOffsets = {
                top: parseFloat(containerNode.style.top || 0),
                left: parseFloat(containerNode.style.left || 0)
              };
            }
          }
          var position = {};
          var transformOrigin = 'top ';
          switch (positionMode.top) {
          case 'target':
            position.top = existingOffsets.top + originNodeRect.top - alignTargetRect.top;
            break;
          case 'cascade':
            position.top = originNodeRect.top - parseFloat(menuStyle.paddingTop) - originNode.style.top;
            break;
          case 'bottom':
            position.top = originNodeRect.top + originNodeRect.height;
            break;
          default:
            throw new Error('Invalid target mode "' + positionMode.top + '" specified for md-menu on Y axis.');
          }
          switch (positionMode.left) {
          case 'target':
            position.left = existingOffsets.left + originNodeRect.left - alignTargetRect.left;
            transformOrigin += 'left';
            break;
          case 'target-right':
            position.left = originNodeRect.right - openMenuNodeRect.width + (openMenuNodeRect.right - alignTargetRect.right);
            transformOrigin += 'right';
            break;
          case 'cascade':
            var willFitRight = originNodeRect.right + openMenuNodeRect.width < bounds.right;
            position.left = willFitRight ? originNodeRect.right - originNode.style.left : originNodeRect.left - originNode.style.left - openMenuNodeRect.width;
            transformOrigin += willFitRight ? 'left' : 'right';
            break;
          case 'left':
            position.left = originNodeRect.left;
            transformOrigin += 'left';
            break;
          default:
            throw new Error('Invalid target mode "' + positionMode.left + '" specified for md-menu on X axis.');
          }
          var offsets = opts.mdMenuCtrl.offsets();
          position.top += offsets.top;
          position.left += offsets.left;
          clamp(position);
          var scaleX = Math.round(100 * Math.min(originNodeRect.width / containerNode.offsetWidth, 1)) / 100;
          var scaleY = Math.round(100 * Math.min(originNodeRect.height / containerNode.offsetHeight, 1)) / 100;
          return {
            top: Math.round(position.top),
            left: Math.round(position.left),
            transform: !opts.alreadyOpen ? $mdUtil.supplant('scale({0},{1})', [
              scaleX,
              scaleY
            ]) : undefined,
            transformOrigin: transformOrigin
          };
          /**
       * Clamps the repositioning of the menu within the confines of
       * bounding element (often the screen/body)
       */
          function clamp(pos) {
            pos.top = Math.max(Math.min(pos.top, bounds.bottom - containerNode.offsetHeight), bounds.top);
            pos.left = Math.max(Math.min(pos.left, bounds.right - containerNode.offsetWidth), bounds.left);
          }
          /**
       * Gets the first visible child in the openMenuNode
       * Necessary incase menu nodes are being dynamically hidden
       */
          function firstVisibleChild() {
            for (var i = 0; i < openMenuNode.children.length; ++i) {
              if ($window.getComputedStyle(openMenuNode.children[i]).display != 'none') {
                return openMenuNode.children[i];
              }
            }
          }
        }
      }
      function toNode(el) {
        if (el instanceof angular.element) {
          el = el[0];
        }
        return el;
      }
    }
    MenuProvider.$inject = ['$$interimElementProvider'];
  }());
  (function () {
    'use strict';
    angular.module('material.components.menuBar').controller('MenuBarController', MenuBarController);
    var BOUND_MENU_METHODS = [
        'handleKeyDown',
        'handleMenuHover',
        'scheduleOpenHoveredMenu',
        'cancelScheduledOpen'
      ];
    /**
 * @ngInject
 */
    function MenuBarController($scope, $rootScope, $element, $attrs, $mdConstant, $document, $mdUtil, $timeout) {
      this.$element = $element;
      this.$attrs = $attrs;
      this.$mdConstant = $mdConstant;
      this.$mdUtil = $mdUtil;
      this.$document = $document;
      this.$scope = $scope;
      this.$rootScope = $rootScope;
      this.$timeout = $timeout;
      var self = this;
      angular.forEach(BOUND_MENU_METHODS, function (methodName) {
        self[methodName] = angular.bind(self, self[methodName]);
      });
    }
    MenuBarController.$inject = [
      '$scope',
      '$rootScope',
      '$element',
      '$attrs',
      '$mdConstant',
      '$document',
      '$mdUtil',
      '$timeout'
    ];
    MenuBarController.prototype.init = function () {
      var $element = this.$element;
      var $mdUtil = this.$mdUtil;
      var $scope = this.$scope;
      var self = this;
      var deregisterFns = [];
      $element.on('keydown', this.handleKeyDown);
      this.parentToolbar = $mdUtil.getClosest($element, 'MD-TOOLBAR');
      deregisterFns.push(this.$rootScope.$on('$mdMenuOpen', function (event, el) {
        if (self.getMenus().indexOf(el[0]) != -1) {
          $element[0].classList.add('md-open');
          el[0].classList.add('md-open');
          self.currentlyOpenMenu = el.controller('mdMenu');
          self.currentlyOpenMenu.registerContainerProxy(self.handleKeyDown);
          self.enableOpenOnHover();
        }
      }));
      deregisterFns.push(this.$rootScope.$on('$mdMenuClose', function (event, el, opts) {
        var rootMenus = self.getMenus();
        if (rootMenus.indexOf(el[0]) != -1) {
          $element[0].classList.remove('md-open');
          el[0].classList.remove('md-open');
        }
        if ($element[0].contains(el[0])) {
          var parentMenu = el[0];
          while (parentMenu && rootMenus.indexOf(parentMenu) == -1) {
            parentMenu = $mdUtil.getClosest(parentMenu, 'MD-MENU', true);
          }
          if (parentMenu) {
            if (!opts.skipFocus)
              parentMenu.querySelector('button:not([disabled])').focus();
            self.currentlyOpenMenu = undefined;
            self.disableOpenOnHover();
            self.setKeyboardMode(true);
          }
        }
      }));
      $scope.$on('$destroy', function () {
        while (deregisterFns.length) {
          deregisterFns.shift()();
        }
      });
      this.setKeyboardMode(true);
    };
    MenuBarController.prototype.setKeyboardMode = function (enabled) {
      if (enabled)
        this.$element[0].classList.add('md-keyboard-mode');
      else
        this.$element[0].classList.remove('md-keyboard-mode');
    };
    MenuBarController.prototype.enableOpenOnHover = function () {
      if (this.openOnHoverEnabled)
        return;
      this.openOnHoverEnabled = true;
      var parentToolbar;
      if (parentToolbar = this.parentToolbar) {
        parentToolbar.dataset.mdRestoreStyle = parentToolbar.getAttribute('style');
        parentToolbar.style.position = 'relative';
        parentToolbar.style.zIndex = 100;
      }
      angular.element(this.getMenus()).on('mouseenter', this.handleMenuHover);
    };
    MenuBarController.prototype.handleMenuHover = function (e) {
      this.setKeyboardMode(false);
      if (this.openOnHoverEnabled) {
        this.scheduleOpenHoveredMenu(e);
      }
    };
    MenuBarController.prototype.disableOpenOnHover = function () {
      if (!this.openOnHoverEnabled)
        return;
      this.openOnHoverEnabled = false;
      var parentToolbar;
      if (parentToolbar = this.parentToolbar) {
        parentToolbar.setAttribute('style', parentToolbar.dataset.mdRestoreStyle || '');
      }
      angular.element(this.getMenus()).off('mouseenter', this.handleMenuHover);
    };
    MenuBarController.prototype.scheduleOpenHoveredMenu = function (e) {
      var menuEl = angular.element(e.currentTarget);
      var menuCtrl = menuEl.controller('mdMenu');
      this.setKeyboardMode(false);
      this.scheduleOpenMenu(menuCtrl);
    };
    MenuBarController.prototype.scheduleOpenMenu = function (menuCtrl) {
      var self = this;
      var $timeout = this.$timeout;
      if (menuCtrl != self.currentlyOpenMenu) {
        $timeout.cancel(self.pendingMenuOpen);
        self.pendingMenuOpen = $timeout(function () {
          self.pendingMenuOpen = undefined;
          if (self.currentlyOpenMenu) {
            self.currentlyOpenMenu.close(true, { closeAll: true });
          }
          menuCtrl.open();
        }, 200, false);
      }
    };
    MenuBarController.prototype.handleKeyDown = function (e) {
      var keyCodes = this.$mdConstant.KEY_CODE;
      var currentMenu = this.currentlyOpenMenu;
      var wasOpen = currentMenu && currentMenu.isOpen;
      this.setKeyboardMode(true);
      var handled, newMenu, newMenuCtrl;
      switch (e.keyCode) {
      case keyCodes.DOWN_ARROW:
        if (currentMenu) {
          currentMenu.focusMenuContainer();
        } else {
          this.openFocusedMenu();
        }
        handled = true;
        break;
      case keyCodes.UP_ARROW:
        currentMenu && currentMenu.close();
        handled = true;
        break;
      case keyCodes.LEFT_ARROW:
        newMenu = this.focusMenu(-1);
        if (wasOpen) {
          newMenuCtrl = angular.element(newMenu).controller('mdMenu');
          this.scheduleOpenMenu(newMenuCtrl);
        }
        handled = true;
        break;
      case keyCodes.RIGHT_ARROW:
        newMenu = this.focusMenu(+1);
        if (wasOpen) {
          newMenuCtrl = angular.element(newMenu).controller('mdMenu');
          this.scheduleOpenMenu(newMenuCtrl);
        }
        handled = true;
        break;
      }
      if (handled) {
        e && e.preventDefault && e.preventDefault();
        e && e.stopImmediatePropagation && e.stopImmediatePropagation();
      }
    };
    MenuBarController.prototype.focusMenu = function (direction) {
      var menus = this.getMenus();
      var focusedIndex = this.getFocusedMenuIndex();
      if (focusedIndex == -1) {
        focusedIndex = this.getOpenMenuIndex();
      }
      var changed = false;
      if (focusedIndex == -1) {
        focusedIndex = 0;
      } else if (direction < 0 && focusedIndex > 0 || direction > 0 && focusedIndex < menus.length - direction) {
        focusedIndex += direction;
        changed = true;
      }
      if (changed) {
        menus[focusedIndex].querySelector('button').focus();
        return menus[focusedIndex];
      }
    };
    MenuBarController.prototype.openFocusedMenu = function () {
      var menu = this.getFocusedMenu();
      menu && angular.element(menu).controller('mdMenu').open();
    };
    MenuBarController.prototype.getMenus = function () {
      var $element = this.$element;
      return this.$mdUtil.nodesToArray($element[0].children).filter(function (el) {
        return el.nodeName == 'MD-MENU';
      });
    };
    MenuBarController.prototype.getFocusedMenu = function () {
      return this.getMenus()[this.getFocusedMenuIndex()];
    };
    MenuBarController.prototype.getFocusedMenuIndex = function () {
      var $mdUtil = this.$mdUtil;
      var focusedEl = $mdUtil.getClosest(this.$document[0].activeElement, 'MD-MENU');
      if (!focusedEl)
        return -1;
      var focusedIndex = this.getMenus().indexOf(focusedEl);
      return focusedIndex;
    };
    MenuBarController.prototype.getOpenMenuIndex = function () {
      var menus = this.getMenus();
      for (var i = 0; i < menus.length; ++i) {
        if (menus[i].classList.contains('md-open'))
          return i;
      }
      return -1;
    };
  }());
  (function () {
    'use strict';
    /**
 * @ngdoc directive
 * @name mdMenuBar
 * @module material.components.menu-bar
 * @restrict E
 * @description
 *
 * Menu bars are containers that hold multiple menus. They change the behavior and appearence
 * of the `md-menu` directive to behave similar to an operating system provided menu.
 *
 * @usage
 * <hljs lang="html">
 * <md-menu-bar>
 *   <md-menu>
 *     <button ng-click="$mdOpenMenu()">
 *       File
 *     </button>
 *     <md-menu-content>
 *       <md-menu-item>
 *         <md-button ng-click="ctrl.sampleAction('share', $event)">
 *           Share...
 *         </md-button>
 *       </md-menu-item>
 *       <md-menu-divider></md-menu-divider>
 *       <md-menu-item>
 *       <md-menu-item>
 *         <md-menu>
 *           <md-button ng-click="$mdOpenMenu()">New</md-button>
 *           <md-menu-content>
 *             <md-menu-item><md-button ng-click="ctrl.sampleAction('New Document', $event)">Document</md-button></md-menu-item>
 *             <md-menu-item><md-button ng-click="ctrl.sampleAction('New Spreadsheet', $event)">Spreadsheet</md-button></md-menu-item>
 *             <md-menu-item><md-button ng-click="ctrl.sampleAction('New Presentation', $event)">Presentation</md-button></md-menu-item>
 *             <md-menu-item><md-button ng-click="ctrl.sampleAction('New Form', $event)">Form</md-button></md-menu-item>
 *             <md-menu-item><md-button ng-click="ctrl.sampleAction('New Drawing', $event)">Drawing</md-button></md-menu-item>
 *           </md-menu-content>
 *         </md-menu>
 *       </md-menu-item>
 *     </md-menu-content>
 *   </md-menu>
 * </md-menu-bar>
 * </hljs>
 *
 * ## Menu Bar Controls
 *
 * You may place `md-menu-items` that function as controls within menu bars.
 * There are two modes that are exposed via the `type` attribute of the `md-menu-item`.
 * `type="checkbox"` will function as a boolean control for the `ng-model` attribute of the
 * `md-menu-item`. `type="radio"` will function like a radio button, setting the `ngModel`
 * to the `string` value of the `value` attribute. If you need non-string values, you can use
 * `ng-value` to provide an expression (this is similar to how angular's native `input[type=radio]` works.
 *
 * <hljs lang="html">
 * <md-menu-bar>
 *  <md-menu>
 *    <button ng-click="$mdOpenMenu()">
 *      Sample Menu
 *    </button>
 *    <md-menu-content>
 *      <md-menu-item type="checkbox" ng-model="settings.allowChanges">Allow changes</md-menu-item>
 *      <md-menu-divider></md-menu-divider>
 *      <md-menu-item type="radio" ng-model="settings.mode" ng-value="1">Mode 1</md-menu-item>
 *      <md-menu-item type="radio" ng-model="settings.mode" ng-value="1">Mode 2</md-menu-item>
 *      <md-menu-item type="radio" ng-model="settings.mode" ng-value="1">Mode 3</md-menu-item>
 *    </md-menu-content>
 *  </md-menu>
 * </md-menu-bar>
 * </hljs>
 *
 *
 * ### Nesting Menus
 *
 * Menus may be nested within menu bars. This is commonly called cascading menus.
 * To nest a menu place the nested menu inside the content of the `md-menu-item`.
 * <hljs lang="html">
 * <md-menu-item>
 *   <md-menu>
 *     <button ng-click="$mdOpenMenu()">New</md-button>
 *     <md-menu-content>
 *       <md-menu-item><md-button ng-click="ctrl.sampleAction('New Document', $event)">Document</md-button></md-menu-item>
 *       <md-menu-item><md-button ng-click="ctrl.sampleAction('New Spreadsheet', $event)">Spreadsheet</md-button></md-menu-item>
 *       <md-menu-item><md-button ng-click="ctrl.sampleAction('New Presentation', $event)">Presentation</md-button></md-menu-item>
 *       <md-menu-item><md-button ng-click="ctrl.sampleAction('New Form', $event)">Form</md-button></md-menu-item>
 *       <md-menu-item><md-button ng-click="ctrl.sampleAction('New Drawing', $event)">Drawing</md-button></md-menu-item>
 *     </md-menu-content>
 *   </md-menu>
 * </md-menu-item>
 * </hljs>
 *
 */
    angular.module('material.components.menuBar').directive('mdMenuBar', MenuBarDirective);
    /**
 *
 * @ngInjdect
 */
    function MenuBarDirective($mdUtil, $mdTheming) {
      return {
        restrict: 'E',
        require: 'mdMenuBar',
        controller: 'MenuBarController',
        compile: function compile(templateEl, templateAttrs) {
          if (!templateAttrs.ariaRole) {
            templateEl[0].setAttribute('role', 'menubar');
          }
          angular.forEach(templateEl[0].children, function (menuEl) {
            if (menuEl.nodeName == 'MD-MENU') {
              if (!menuEl.hasAttribute('md-position-mode')) {
                menuEl.setAttribute('md-position-mode', 'left bottom');
                menuEl.querySelector('button,a').setAttribute('role', 'menuitem');
              }
              var contentEls = $mdUtil.nodesToArray(menuEl.querySelectorAll('md-menu-content'));
              angular.forEach(contentEls, function (contentEl) {
                contentEl.classList.add('md-menu-bar-menu');
                contentEl.classList.add('md-dense');
                if (!contentEl.hasAttribute('width')) {
                  contentEl.setAttribute('width', 5);
                }
              });
            }
          });
          return function postLink(scope, el, attrs, ctrl) {
            $mdTheming(scope, el);
            ctrl.init();
          };
        }
      };
    }
    MenuBarDirective.$inject = [
      '$mdUtil',
      '$mdTheming'
    ];
  }());
  (function () {
    'use strict';
    angular.module('material.components.menuBar').directive('mdMenuDivider', MenuDividerDirective);
    function MenuDividerDirective() {
      return {
        restrict: 'E',
        compile: function (templateEl, templateAttrs) {
          if (!templateAttrs.role) {
            templateEl[0].setAttribute('role', 'separator');
          }
        }
      };
    }
  }());
  (function () {
    'use strict';
    angular.module('material.components.menuBar').controller('MenuItemController', MenuItemController);
    /**
 * @ngInject
 */
    function MenuItemController($scope, $element, $attrs) {
      this.$element = $element;
      this.$attrs = $attrs;
      this.$scope = $scope;
    }
    MenuItemController.$inject = [
      '$scope',
      '$element',
      '$attrs'
    ];
    MenuItemController.prototype.init = function (ngModel) {
      var $element = this.$element;
      var $attrs = this.$attrs;
      this.ngModel = ngModel;
      if ($attrs.type == 'checkbox' || $attrs.type == 'radio') {
        this.mode = $attrs.type;
        this.iconEl = $element[0].children[0];
        this.buttonEl = $element[0].children[1];
        if (ngModel) {
          // Clear ngAria set attributes
          this.initClickListeners();
        }
      }
    };
    // ngAria auto sets attributes on a menu-item with a ngModel.
    // We don't want this because our content (buttons) get the focus
    // and set their own aria attributes appropritately. Having both
    // breaks NVDA / JAWS. This undeoes ngAria's attrs.
    MenuItemController.prototype.clearNgAria = function () {
      var el = this.$element[0];
      var clearAttrs = [
          'role',
          'tabindex',
          'aria-invalid',
          'aria-checked'
        ];
      angular.forEach(clearAttrs, function (attr) {
        el.removeAttribute(attr);
      });
    };
    MenuItemController.prototype.initClickListeners = function () {
      var self = this;
      var ngModel = this.ngModel;
      var $scope = this.$scope;
      var $attrs = this.$attrs;
      var $element = this.$element;
      var mode = this.mode;
      this.handleClick = angular.bind(this, this.handleClick);
      var icon = this.iconEl;
      var button = angular.element(this.buttonEl);
      var handleClick = this.handleClick;
      $attrs.$observe('disabled', setDisabled);
      setDisabled($attrs.disabled);
      ngModel.$render = function render() {
        self.clearNgAria();
        if (isSelected()) {
          icon.style.display = '';
          button.attr('aria-checked', 'true');
        } else {
          icon.style.display = 'none';
          button.attr('aria-checked', 'false');
        }
      };
      $scope.$$postDigest(ngModel.$render);
      function isSelected() {
        if (mode == 'radio') {
          var val = $attrs.ngValue ? $scope.$eval($attrs.ngValue) : $attrs.value;
          return ngModel.$modelValue == val;
        } else {
          return ngModel.$modelValue;
        }
      }
      function setDisabled(disabled) {
        if (disabled) {
          button.off('click', handleClick);
        } else {
          button.on('click', handleClick);
        }
      }
    };
    MenuItemController.prototype.handleClick = function (e) {
      var mode = this.mode;
      var ngModel = this.ngModel;
      var $attrs = this.$attrs;
      var newVal;
      if (mode == 'checkbox') {
        newVal = !ngModel.$modelValue;
      } else if (mode == 'radio') {
        newVal = $attrs.ngValue ? this.$scope.$eval($attrs.ngValue) : $attrs.value;
      }
      ngModel.$setViewValue(newVal);
      ngModel.$render();
    };
  }());
  (function () {
    'use strict';
    angular.module('material.components.menuBar').directive('mdMenuItem', MenuItemDirective);
    /**
  *
  * @ngInjdect
  */
    function MenuItemDirective() {
      return {
        require: [
          'mdMenuItem',
          '?ngModel'
        ],
        priority: 210,
        compile: function (templateEl, templateAttrs) {
          if (templateAttrs.type == 'checkbox' || templateAttrs.type == 'radio') {
            var text = templateEl[0].textContent;
            var buttonEl = angular.element('<md-button type="button"></md-button>');
            buttonEl.html(text);
            buttonEl.attr('tabindex', '0');
            templateEl.html('');
            templateEl.append(angular.element('<md-icon md-svg-icon="check"></md-icon>'));
            templateEl.append(buttonEl);
            templateEl[0].classList.add('md-indent');
            setDefault('role', templateAttrs.type == 'checkbox' ? 'menuitemcheckbox' : 'menuitemradio', buttonEl);
            angular.forEach(['ng-disabled'], moveAttrToButton);
          } else {
            setDefault('role', 'menuitem', templateEl[0].querySelector('md-button,button,a'));
          }
          return function (scope, el, attrs, ctrls) {
            var ctrl = ctrls[0];
            var ngModel = ctrls[1];
            ctrl.init(ngModel);
          };
          function setDefault(attr, val, el) {
            el = el || templateEl;
            if (el instanceof angular.element) {
              el = el[0];
            }
            if (!el.hasAttribute(attr)) {
              el.setAttribute(attr, val);
            }
          }
          function moveAttrToButton(attr) {
            if (templateEl[0].hasAttribute(attr)) {
              var val = templateEl[0].getAttribute(attr);
              buttonEl[0].setAttribute(attr, val);
              templateEl[0].removeAttribute(attr);
            }
          }
        },
        controller: 'MenuItemController'
      };
    }
  }());
  (function () {
    'use strict';
    /**
 * @ngdoc directive
 * @name mdTab
 * @module material.components.tabs
 *
 * @restrict E
 *
 * @description
 * Use the `<md-tab>` a nested directive used within `<md-tabs>` to specify a tab with a **label** and optional *view content*.
 *
 * If the `label` attribute is not specified, then an optional `<md-tab-label>` tag can be used to specify more
 * complex tab header markup. If neither the **label** nor the **md-tab-label** are specified, then the nested
 * markup of the `<md-tab>` is used as the tab header markup.
 *
 * Please note that if you use `<md-tab-label>`, your content **MUST** be wrapped in the `<md-tab-body>` tag.  This
 * is to define a clear separation between the tab content and the tab label.
 *
 * This container is used by the TabsController to show/hide the active tab's content view. This synchronization is
 * automatically managed by the internal TabsController whenever the tab selection changes. Selection changes can
 * be initiated via data binding changes, programmatic invocation, or user gestures.
 *
 * @param {string=} label Optional attribute to specify a simple string as the tab label
 * @param {boolean=} ng-disabled If present, disabled tab selection.
 * @param {expression=} md-on-deselect Expression to be evaluated after the tab has been de-selected.
 * @param {expression=} md-on-select Expression to be evaluated after the tab has been selected.
 * @param {boolean=} md-active When true, sets the active tab.  Note: There can only be one active tab at a time.
 *
 *
 * @usage
 *
 * <hljs lang="html">
 * <md-tab label="" disabled="" md-on-select="" md-on-deselect="" >
 *   <h3>My Tab content</h3>
 * </md-tab>
 *
 * <md-tab >
 *   <md-tab-label>
 *     <h3>My Tab content</h3>
 *   </md-tab-label>
 *   <md-tab-body>
 *     <p>
 *       Sed ut perspiciatis unde omnis iste natus error sit voluptatem accusantium doloremque laudantium,
 *       totam rem aperiam, eaque ipsa quae ab illo inventore veritatis et quasi architecto beatae vitae
 *       dicta sunt explicabo. Nemo enim ipsam voluptatem quia voluptas sit aspernatur aut odit aut fugit,
 *       sed quia consequuntur magni dolores eos qui ratione voluptatem sequi nesciunt.
 *     </p>
 *   </md-tab-body>
 * </md-tab>
 * </hljs>
 *
 */
    angular.module('material.components.tabs').directive('mdTab', MdTab);
    function MdTab() {
      return {
        require: '^?mdTabs',
        terminal: true,
        compile: function (element, attr) {
          var label = firstChild(element, 'md-tab-label'), body = firstChild(element, 'md-tab-body');
          if (label.length == 0) {
            label = angular.element('<md-tab-label></md-tab-label>');
            if (attr.label)
              label.text(attr.label);
            else
              label.append(element.contents());
            if (body.length == 0) {
              var contents = element.contents().detach();
              body = angular.element('<md-tab-body></md-tab-body>');
              body.append(contents);
            }
          }
          element.append(label);
          if (body.html())
            element.append(body);
          return postLink;
        },
        scope: {
          active: '=?mdActive',
          disabled: '=?ngDisabled',
          select: '&?mdOnSelect',
          deselect: '&?mdOnDeselect'
        }
      };
      function postLink(scope, element, attr, ctrl) {
        if (!ctrl)
          return;
        var index = ctrl.getTabElementIndex(element), body = firstChild(element, 'md-tab-body').remove(), label = firstChild(element, 'md-tab-label').remove(), data = ctrl.insertTab({
            scope: scope,
            parent: scope.$parent,
            index: index,
            element: element,
            template: body.html(),
            label: label.html()
          }, index);
        scope.select = scope.select || angular.noop;
        scope.deselect = scope.deselect || angular.noop;
        scope.$watch('active', function (active) {
          if (active)
            ctrl.select(data.getIndex());
        });
        scope.$watch('disabled', function () {
          ctrl.refreshIndex();
        });
        scope.$watch(function () {
          return ctrl.getTabElementIndex(element);
        }, function (newIndex) {
          data.index = newIndex;
          ctrl.updateTabOrder();
        });
        scope.$on('$destroy', function () {
          ctrl.removeTab(data);
        });
      }
      function firstChild(element, tagName) {
        var children = element[0].children;
        for (var i = 0, len = children.length; i < len; i++) {
          var child = children[i];
          if (child.tagName === tagName.toUpperCase())
            return angular.element(child);
        }
        return angular.element();
      }
    }
  }());
  (function () {
    'use strict';
    angular.module('material.components.tabs').directive('mdTabItem', MdTabItem);
    function MdTabItem() {
      return {
        require: '^?mdTabs',
        link: function link(scope, element, attr, ctrl) {
          if (!ctrl)
            return;
          ctrl.attachRipple(scope, element);
        }
      };
    }
  }());
  (function () {
    'use strict';
    angular.module('material.components.tabs').directive('mdTabLabel', MdTabLabel);
    function MdTabLabel() {
      return { terminal: true };
    }
  }());
  (function () {
    'use strict';
    angular.module('material.components.tabs').directive('mdTabScroll', MdTabScroll);
    function MdTabScroll($parse) {
      return {
        restrict: 'A',
        compile: function ($element, attr) {
          var fn = $parse(attr.mdTabScroll, null, true);
          return function ngEventHandler(scope, element) {
            element.on('mousewheel', function (event) {
              scope.$apply(function () {
                fn(scope, { $event: event });
              });
            });
          };
        }
      };
    }
    MdTabScroll.$inject = ['$parse'];
  }());
  (function () {
    'use strict';
    angular.module('material.components.tabs').controller('MdTabsController', MdTabsController);
    /**
 * @ngInject
 */
    function MdTabsController($scope, $element, $window, $mdConstant, $mdTabInkRipple, $mdUtil, $animateCss, $attrs, $compile, $mdTheming) {
      // define private properties
      var ctrl = this, locked = false, elements = getElements(), queue = [], destroyed = false, loaded = false;
      // define one-way bindings
      defineOneWayBinding('stretchTabs', handleStretchTabs);
      // define public properties with change handlers
      defineProperty('focusIndex', handleFocusIndexChange, ctrl.selectedIndex || 0);
      defineProperty('offsetLeft', handleOffsetChange, 0);
      defineProperty('hasContent', handleHasContent, false);
      defineProperty('maxTabWidth', handleMaxTabWidth, getMaxTabWidth());
      defineProperty('shouldPaginate', handleShouldPaginate, false);
      // define boolean attributes
      defineBooleanAttribute('noInkBar', handleInkBar);
      defineBooleanAttribute('dynamicHeight', handleDynamicHeight);
      defineBooleanAttribute('noPagination');
      defineBooleanAttribute('swipeContent');
      defineBooleanAttribute('noDisconnect');
      defineBooleanAttribute('autoselect');
      defineBooleanAttribute('centerTabs', handleCenterTabs, false);
      defineBooleanAttribute('enableDisconnect');
      // define public properties
      ctrl.scope = $scope;
      ctrl.parent = $scope.$parent;
      ctrl.tabs = [];
      ctrl.lastSelectedIndex = null;
      ctrl.hasFocus = false;
      ctrl.lastClick = true;
      ctrl.shouldCenterTabs = shouldCenterTabs();
      // define public methods
      ctrl.updatePagination = $mdUtil.debounce(updatePagination, 100);
      ctrl.redirectFocus = redirectFocus;
      ctrl.attachRipple = attachRipple;
      ctrl.insertTab = insertTab;
      ctrl.removeTab = removeTab;
      ctrl.select = select;
      ctrl.scroll = scroll;
      ctrl.nextPage = nextPage;
      ctrl.previousPage = previousPage;
      ctrl.keydown = keydown;
      ctrl.canPageForward = canPageForward;
      ctrl.canPageBack = canPageBack;
      ctrl.refreshIndex = refreshIndex;
      ctrl.incrementIndex = incrementIndex;
      ctrl.getTabElementIndex = getTabElementIndex;
      ctrl.updateInkBarStyles = $mdUtil.debounce(updateInkBarStyles, 100);
      ctrl.updateTabOrder = $mdUtil.debounce(updateTabOrder, 100);
      init();
      /**
   * Perform initialization for the controller, setup events and watcher(s)
   */
      function init() {
        ctrl.selectedIndex = ctrl.selectedIndex || 0;
        compileTemplate();
        configureWatchers();
        bindEvents();
        $mdTheming($element);
        $mdUtil.nextTick(function () {
          updateHeightFromContent();
          adjustOffset();
          updateInkBarStyles();
          ctrl.tabs[ctrl.selectedIndex] && ctrl.tabs[ctrl.selectedIndex].scope.select();
          loaded = true;
          updatePagination();
        });
      }
      /**
   * Compiles the template provided by the user.  This is passed as an attribute from the tabs
   * directive's template function.
   */
      function compileTemplate() {
        var template = $attrs.$mdTabsTemplate, element = angular.element(elements.data);
        element.html(template);
        $compile(element.contents())(ctrl.parent);
        delete $attrs.$mdTabsTemplate;
      }
      /**
   * Binds events used by the tabs component.
   */
      function bindEvents() {
        angular.element($window).on('resize', handleWindowResize);
        $scope.$on('$destroy', cleanup);
      }
      /**
   * Configure watcher(s) used by Tabs
   */
      function configureWatchers() {
        $scope.$watch('$mdTabsCtrl.selectedIndex', handleSelectedIndexChange);
      }
      /**
   * Creates a one-way binding manually rather than relying on Angular's isolated scope
   * @param key
   * @param handler
   */
      function defineOneWayBinding(key, handler) {
        var attr = $attrs.$normalize('md-' + key);
        if (handler)
          defineProperty(key, handler);
        $attrs.$observe(attr, function (newValue) {
          ctrl[key] = newValue;
        });
      }
      /**
   * Defines boolean attributes with default value set to true.  (ie. md-stretch-tabs with no value
   * will be treated as being truthy)
   * @param key
   * @param handler
   */
      function defineBooleanAttribute(key, handler) {
        var attr = $attrs.$normalize('md-' + key);
        if (handler)
          defineProperty(key, handler);
        if ($attrs.hasOwnProperty(attr))
          updateValue($attrs[attr]);
        $attrs.$observe(attr, updateValue);
        function updateValue(newValue) {
          ctrl[key] = newValue !== 'false';
        }
      }
      /**
   * Remove any events defined by this controller
   */
      function cleanup() {
        destroyed = true;
        angular.element($window).off('resize', handleWindowResize);
      }
      // Change handlers
      /**
   * Toggles stretch tabs class and updates inkbar when tab stretching changes
   * @param stretchTabs
   */
      function handleStretchTabs(stretchTabs) {
        angular.element(elements.wrapper).toggleClass('md-stretch-tabs', shouldStretchTabs());
        updateInkBarStyles();
      }
      function handleCenterTabs(newValue) {
        ctrl.shouldCenterTabs = shouldCenterTabs();
      }
      function handleMaxTabWidth(newWidth, oldWidth) {
        if (newWidth !== oldWidth) {
          $mdUtil.nextTick(ctrl.updateInkBarStyles);
        }
      }
      function handleShouldPaginate(newValue, oldValue) {
        if (newValue !== oldValue) {
          ctrl.maxTabWidth = getMaxTabWidth();
          ctrl.shouldCenterTabs = shouldCenterTabs();
          $mdUtil.nextTick(function () {
            ctrl.maxTabWidth = getMaxTabWidth();
            adjustOffset(ctrl.selectedIndex);
          });
        }
      }
      /**
   * Add/remove the `md-no-tab-content` class depending on `ctrl.hasContent`
   * @param hasContent
   */
      function handleHasContent(hasContent) {
        $element[hasContent ? 'removeClass' : 'addClass']('md-no-tab-content');
      }
      /**
   * Apply ctrl.offsetLeft to the paging element when it changes
   * @param left
   */
      function handleOffsetChange(left) {
        var newValue = ctrl.shouldCenterTabs ? '' : '-' + left + 'px';
        angular.element(elements.paging).css($mdConstant.CSS.TRANSFORM, 'translate3d(' + newValue + ', 0, 0)');
        $scope.$broadcast('$mdTabsPaginationChanged');
      }
      /**
   * Update the UI whenever `ctrl.focusIndex` is updated
   * @param newIndex
   * @param oldIndex
   */
      function handleFocusIndexChange(newIndex, oldIndex) {
        if (newIndex === oldIndex)
          return;
        if (!elements.tabs[newIndex])
          return;
        adjustOffset();
        redirectFocus();
      }
      /**
   * Update the UI whenever the selected index changes. Calls user-defined select/deselect methods.
   * @param newValue
   * @param oldValue
   */
      function handleSelectedIndexChange(newValue, oldValue) {
        if (newValue === oldValue)
          return;
        ctrl.selectedIndex = getNearestSafeIndex(newValue);
        ctrl.lastSelectedIndex = oldValue;
        ctrl.updateInkBarStyles();
        updateHeightFromContent();
        adjustOffset(newValue);
        $scope.$broadcast('$mdTabsChanged');
        ctrl.tabs[oldValue] && ctrl.tabs[oldValue].scope.deselect();
        ctrl.tabs[newValue] && ctrl.tabs[newValue].scope.select();
      }
      function getTabElementIndex(tabEl) {
        var tabs = $element[0].getElementsByTagName('md-tab');
        return Array.prototype.indexOf.call(tabs, tabEl[0]);
      }
      /**
   * Queues up a call to `handleWindowResize` when a resize occurs while the tabs component is
   * hidden.
   */
      function handleResizeWhenVisible() {
        // if there is already a watcher waiting for resize, do nothing
        if (handleResizeWhenVisible.watcher)
          return;
        // otherwise, we will abuse the $watch function to check for visible
        handleResizeWhenVisible.watcher = $scope.$watch(function () {
          // since we are checking for DOM size, we use $mdUtil.nextTick() to wait for after the DOM updates
          $mdUtil.nextTick(function () {
            // if the watcher has already run (ie. multiple digests in one cycle), do nothing
            if (!handleResizeWhenVisible.watcher)
              return;
            if ($element.prop('offsetParent')) {
              handleResizeWhenVisible.watcher();
              handleResizeWhenVisible.watcher = null;
              handleWindowResize();
            }
          }, false);
        });
      }
      // Event handlers / actions
      /**
   * Handle user keyboard interactions
   * @param event
   */
      function keydown(event) {
        switch (event.keyCode) {
        case $mdConstant.KEY_CODE.LEFT_ARROW:
          event.preventDefault();
          incrementIndex(-1, true);
          break;
        case $mdConstant.KEY_CODE.RIGHT_ARROW:
          event.preventDefault();
          incrementIndex(1, true);
          break;
        case $mdConstant.KEY_CODE.SPACE:
        case $mdConstant.KEY_CODE.ENTER:
          event.preventDefault();
          if (!locked)
            ctrl.selectedIndex = ctrl.focusIndex;
          break;
        }
        ctrl.lastClick = false;
      }
      /**
   * Update the selected index and trigger a click event on the original `md-tab` element in order
   * to fire user-added click events.
   * @param index
   */
      function select(index) {
        if (!locked)
          ctrl.focusIndex = ctrl.selectedIndex = index;
        ctrl.lastClick = true;
        // nextTick is required to prevent errors in user-defined click events
        $mdUtil.nextTick(function () {
          ctrl.tabs[index].element.triggerHandler('click');
        }, false);
      }
      /**
   * When pagination is on, this makes sure the selected index is in view.
   * @param event
   */
      function scroll(event) {
        if (!ctrl.shouldPaginate)
          return;
        event.preventDefault();
        ctrl.offsetLeft = fixOffset(ctrl.offsetLeft - event.wheelDelta);
      }
      /**
   * Slides the tabs over approximately one page forward.
   */
      function nextPage() {
        var viewportWidth = elements.canvas.clientWidth, totalWidth = viewportWidth + ctrl.offsetLeft, i, tab;
        for (i = 0; i < elements.tabs.length; i++) {
          tab = elements.tabs[i];
          if (tab.offsetLeft + tab.offsetWidth > totalWidth)
            break;
        }
        ctrl.offsetLeft = fixOffset(tab.offsetLeft);
      }
      /**
   * Slides the tabs over approximately one page backward.
   */
      function previousPage() {
        var i, tab;
        for (i = 0; i < elements.tabs.length; i++) {
          tab = elements.tabs[i];
          if (tab.offsetLeft + tab.offsetWidth >= ctrl.offsetLeft)
            break;
        }
        ctrl.offsetLeft = fixOffset(tab.offsetLeft + tab.offsetWidth - elements.canvas.clientWidth);
      }
      /**
   * Update size calculations when the window is resized.
   */
      function handleWindowResize() {
        ctrl.lastSelectedIndex = ctrl.selectedIndex;
        ctrl.offsetLeft = fixOffset(ctrl.offsetLeft);
        $mdUtil.nextTick(function () {
          ctrl.updateInkBarStyles();
          updatePagination();
        });
      }
      function handleInkBar(hide) {
        angular.element(elements.inkBar).toggleClass('ng-hide', hide);
      }
      /**
   * Toggle dynamic height class when value changes
   * @param value
   */
      function handleDynamicHeight(value) {
        $element.toggleClass('md-dynamic-height', value);
      }
      /**
   * Remove a tab from the data and select the nearest valid tab.
   * @param tabData
   */
      function removeTab(tabData) {
        if (destroyed)
          return;
        var selectedIndex = ctrl.selectedIndex, tab = ctrl.tabs.splice(tabData.getIndex(), 1)[0];
        refreshIndex();
        // when removing a tab, if the selected index did not change, we have to manually trigger the
        //   tab select/deselect events
        if (ctrl.selectedIndex === selectedIndex) {
          tab.scope.deselect();
          ctrl.tabs[ctrl.selectedIndex] && ctrl.tabs[ctrl.selectedIndex].scope.select();
        }
        $mdUtil.nextTick(function () {
          updatePagination();
          ctrl.offsetLeft = fixOffset(ctrl.offsetLeft);
        });
      }
      /**
   * Create an entry in the tabs array for a new tab at the specified index.
   * @param tabData
   * @param index
   * @returns {*}
   */
      function insertTab(tabData, index) {
        var hasLoaded = loaded;
        var proto = {
            getIndex: function () {
              return ctrl.tabs.indexOf(tab);
            },
            isActive: function () {
              return this.getIndex() === ctrl.selectedIndex;
            },
            isLeft: function () {
              return this.getIndex() < ctrl.selectedIndex;
            },
            isRight: function () {
              return this.getIndex() > ctrl.selectedIndex;
            },
            shouldRender: function () {
              return !ctrl.noDisconnect || this.isActive();
            },
            hasFocus: function () {
              return !ctrl.lastClick && ctrl.hasFocus && this.getIndex() === ctrl.focusIndex;
            },
            id: $mdUtil.nextUid()
          }, tab = angular.extend(proto, tabData);
        if (angular.isDefined(index)) {
          ctrl.tabs.splice(index, 0, tab);
        } else {
          ctrl.tabs.push(tab);
        }
        processQueue();
        updateHasContent();
        $mdUtil.nextTick(function () {
          updatePagination();
          // if autoselect is enabled, select the newly added tab
          if (hasLoaded && ctrl.autoselect)
            $mdUtil.nextTick(function () {
              $mdUtil.nextTick(function () {
                select(ctrl.tabs.indexOf(tab));
              });
            });
        });
        return tab;
      }
      // Getter methods
      /**
   * Gathers references to all of the DOM elements used by this controller.
   * @returns {{}}
   */
      function getElements() {
        var elements = {};
        // gather tab bar elements
        elements.wrapper = $element[0].getElementsByTagName('md-tabs-wrapper')[0];
        elements.data = $element[0].getElementsByTagName('md-tab-data')[0];
        elements.canvas = elements.wrapper.getElementsByTagName('md-tabs-canvas')[0];
        elements.paging = elements.canvas.getElementsByTagName('md-pagination-wrapper')[0];
        elements.tabs = elements.paging.getElementsByTagName('md-tab-item');
        elements.dummies = elements.canvas.getElementsByTagName('md-dummy-tab');
        elements.inkBar = elements.paging.getElementsByTagName('md-ink-bar')[0];
        // gather tab content elements
        elements.contentsWrapper = $element[0].getElementsByTagName('md-tabs-content-wrapper')[0];
        elements.contents = elements.contentsWrapper.getElementsByTagName('md-tab-content');
        return elements;
      }
      /**
   * Determines whether or not the left pagination arrow should be enabled.
   * @returns {boolean}
   */
      function canPageBack() {
        return ctrl.offsetLeft > 0;
      }
      /**
   * Determines whether or not the right pagination arrow should be enabled.
   * @returns {*|boolean}
   */
      function canPageForward() {
        var lastTab = elements.tabs[elements.tabs.length - 1];
        return lastTab && lastTab.offsetLeft + lastTab.offsetWidth > elements.canvas.clientWidth + ctrl.offsetLeft;
      }
      /**
   * Determines if the UI should stretch the tabs to fill the available space.
   * @returns {*}
   */
      function shouldStretchTabs() {
        switch (ctrl.stretchTabs) {
        case 'always':
          return true;
        case 'never':
          return false;
        default:
          return !ctrl.shouldPaginate && $window.matchMedia('(max-width: 600px)').matches;
        }
      }
      /**
   * Determines if the tabs should appear centered.
   * @returns {string|boolean}
   */
      function shouldCenterTabs() {
        return ctrl.centerTabs && !ctrl.shouldPaginate;
      }
      /**
   * Determines if pagination is necessary to display the tabs within the available space.
   * @returns {boolean}
   */
      function shouldPaginate() {
        if (ctrl.noPagination || !loaded)
          return false;
        var canvasWidth = $element.prop('clientWidth');
        angular.forEach(getElements().dummies, function (tab) {
          canvasWidth -= tab.offsetWidth;
        });
        return canvasWidth < 0;
      }
      /**
   * Finds the nearest tab index that is available.  This is primarily used for when the active
   * tab is removed.
   * @param newIndex
   * @returns {*}
   */
      function getNearestSafeIndex(newIndex) {
        if (newIndex === -1)
          return -1;
        var maxOffset = Math.max(ctrl.tabs.length - newIndex, newIndex), i, tab;
        for (i = 0; i <= maxOffset; i++) {
          tab = ctrl.tabs[newIndex + i];
          if (tab && tab.scope.disabled !== true)
            return tab.getIndex();
          tab = ctrl.tabs[newIndex - i];
          if (tab && tab.scope.disabled !== true)
            return tab.getIndex();
        }
        return newIndex;
      }
      // Utility methods
      /**
   * Defines a property using a getter and setter in order to trigger a change handler without
   * using `$watch` to observe changes.
   * @param key
   * @param handler
   * @param value
   */
      function defineProperty(key, handler, value) {
        Object.defineProperty(ctrl, key, {
          get: function () {
            return value;
          },
          set: function (newValue) {
            var oldValue = value;
            value = newValue;
            handler && handler(newValue, oldValue);
          }
        });
      }
      /**
   * Updates whether or not pagination should be displayed.
   */
      function updatePagination() {
        if (!shouldStretchTabs())
          updatePagingWidth();
        ctrl.maxTabWidth = getMaxTabWidth();
        ctrl.shouldPaginate = shouldPaginate();
      }
      function updatePagingWidth() {
        var width = 1;
        angular.forEach(getElements().dummies, function (element) {
          width += Math.ceil(element.offsetWidth);
        });
        angular.element(elements.paging).css('width', width + 'px');
      }
      function getMaxTabWidth() {
        return $element.prop('clientWidth');
      }
      /**
   * Re-orders the tabs and updates the selected and focus indexes to their new positions.
   * This is triggered by `tabDirective.js` when the user's tabs have been re-ordered.
   */
      function updateTabOrder() {
        var selectedItem = ctrl.tabs[ctrl.selectedIndex], focusItem = ctrl.tabs[ctrl.focusIndex];
        ctrl.tabs = ctrl.tabs.sort(function (a, b) {
          return a.index - b.index;
        });
        ctrl.selectedIndex = ctrl.tabs.indexOf(selectedItem);
        ctrl.focusIndex = ctrl.tabs.indexOf(focusItem);
      }
      /**
   * This moves the selected or focus index left or right.  This is used by the keydown handler.
   * @param inc
   */
      function incrementIndex(inc, focus) {
        var newIndex, key = focus ? 'focusIndex' : 'selectedIndex', index = ctrl[key];
        for (newIndex = index + inc; ctrl.tabs[newIndex] && ctrl.tabs[newIndex].scope.disabled; newIndex += inc) {
        }
        if (ctrl.tabs[newIndex]) {
          ctrl[key] = newIndex;
        }
      }
      /**
   * This is used to forward focus to dummy elements.  This method is necessary to avoid animation
   * issues when attempting to focus an item that is out of view.
   */
      function redirectFocus() {
        getElements().dummies[ctrl.focusIndex].focus();
      }
      /**
   * Forces the pagination to move the focused tab into view.
   */
      function adjustOffset(index) {
        if (index == null)
          index = ctrl.focusIndex;
        if (!elements.tabs[index])
          return;
        if (ctrl.shouldCenterTabs)
          return;
        var tab = elements.tabs[index], left = tab.offsetLeft, right = tab.offsetWidth + left;
        ctrl.offsetLeft = Math.max(ctrl.offsetLeft, fixOffset(right - elements.canvas.clientWidth + 32 * 2));
        ctrl.offsetLeft = Math.min(ctrl.offsetLeft, fixOffset(left));
      }
      /**
   * Iterates through all queued functions and clears the queue.  This is used for functions that
   * are called before the UI is ready, such as size calculations.
   */
      function processQueue() {
        queue.forEach(function (func) {
          $mdUtil.nextTick(func);
        });
        queue = [];
      }
      /**
   * Determines if the tab content area is needed.
   */
      function updateHasContent() {
        var hasContent = false;
        angular.forEach(ctrl.tabs, function (tab) {
          if (tab.template)
            hasContent = true;
        });
        ctrl.hasContent = hasContent;
      }
      /**
   * Moves the indexes to their nearest valid values.
   */
      function refreshIndex() {
        ctrl.selectedIndex = getNearestSafeIndex(ctrl.selectedIndex);
        ctrl.focusIndex = getNearestSafeIndex(ctrl.focusIndex);
      }
      /**
   * Calculates the content height of the current tab.
   * @returns {*}
   */
      function updateHeightFromContent() {
        if (!ctrl.dynamicHeight)
          return $element.css('height', '');
        if (!ctrl.tabs.length)
          return queue.push(updateHeightFromContent);
        var tabContent = elements.contents[ctrl.selectedIndex], contentHeight = tabContent ? tabContent.offsetHeight : 0, tabsHeight = elements.wrapper.offsetHeight, newHeight = contentHeight + tabsHeight, currentHeight = $element.prop('clientHeight');
        if (currentHeight === newHeight)
          return;
        // Adjusts calculations for when the buttons are bottom-aligned since this relies on absolute
        // positioning.  This should probably be cleaned up if a cleaner solution is possible.
        if ($element.attr('md-align-tabs') === 'bottom') {
          currentHeight -= tabsHeight;
          newHeight -= tabsHeight;
          // Need to include bottom border in these calculations
          if ($element.attr('md-border-bottom') !== undefined)
            ++currentHeight;
        }
        // Lock during animation so the user can't change tabs
        locked = true;
        var fromHeight = { height: currentHeight + 'px' }, toHeight = { height: newHeight + 'px' };
        // Set the height to the current, specific pixel height to fix a bug on iOS where the height
        // first animates to 0, then back to the proper height causing a visual glitch
        $element.css(fromHeight);
        // Animate the height from the old to the new
        $animateCss($element, {
          from: fromHeight,
          to: toHeight,
          easing: 'cubic-bezier(0.35, 0, 0.25, 1)',
          duration: 0.5
        }).start().done(function () {
          // Then (to fix the same iOS issue as above), disable transitions and remove the specific
          // pixel height so the height can size with browser width/content changes, etc.
          $element.css({
            transition: 'none',
            height: ''
          });
          // In the next tick, re-allow transitions (if we do it all at once, $element.css is "smart"
          // enough to batch it for us instead of doing it immediately, which undoes the original
          // transition: none)
          $mdUtil.nextTick(function () {
            $element.css('transition', '');
          });
          // And unlock so tab changes can occur
          locked = false;
        });
      }
      /**
   * Repositions the ink bar to the selected tab.
   * @returns {*}
   */
      function updateInkBarStyles() {
        if (!elements.tabs[ctrl.selectedIndex]) {
          angular.element(elements.inkBar).css({
            left: 'auto',
            right: 'auto'
          });
          return;
        }
        if (!ctrl.tabs.length)
          return queue.push(ctrl.updateInkBarStyles);
        // if the element is not visible, we will not be able to calculate sizes until it is
        // we should treat that as a resize event rather than just updating the ink bar
        if (!$element.prop('offsetParent'))
          return handleResizeWhenVisible();
        var index = ctrl.selectedIndex, totalWidth = elements.paging.offsetWidth, tab = elements.tabs[index], left = tab.offsetLeft, right = totalWidth - left - tab.offsetWidth, tabWidth;
        if (ctrl.shouldCenterTabs) {
          tabWidth = Array.prototype.slice.call(elements.tabs).reduce(function (value, element) {
            return value + element.offsetWidth;
          }, 0);
          if (totalWidth > tabWidth)
            $mdUtil.nextTick(updateInkBarStyles, false);
        }
        updateInkBarClassName();
        angular.element(elements.inkBar).css({
          left: left + 'px',
          right: right + 'px'
        });
      }
      /**
   * Adds left/right classes so that the ink bar will animate properly.
   */
      function updateInkBarClassName() {
        var newIndex = ctrl.selectedIndex, oldIndex = ctrl.lastSelectedIndex, ink = angular.element(elements.inkBar);
        if (!angular.isNumber(oldIndex))
          return;
        ink.toggleClass('md-left', newIndex < oldIndex).toggleClass('md-right', newIndex > oldIndex);
      }
      /**
   * Takes an offset value and makes sure that it is within the min/max allowed values.
   * @param value
   * @returns {*}
   */
      function fixOffset(value) {
        if (!elements.tabs.length || !ctrl.shouldPaginate)
          return 0;
        var lastTab = elements.tabs[elements.tabs.length - 1], totalWidth = lastTab.offsetLeft + lastTab.offsetWidth;
        value = Math.max(0, value);
        value = Math.min(totalWidth - elements.canvas.clientWidth, value);
        return value;
      }
      /**
   * Attaches a ripple to the tab item element.
   * @param scope
   * @param element
   */
      function attachRipple(scope, element) {
        var options = { colorElement: angular.element(elements.inkBar) };
        $mdTabInkRipple.attach(scope, element, options);
      }
    }
    MdTabsController.$inject = [
      '$scope',
      '$element',
      '$window',
      '$mdConstant',
      '$mdTabInkRipple',
      '$mdUtil',
      '$animateCss',
      '$attrs',
      '$compile',
      '$mdTheming'
    ];
  }());
  (function () {
    'use strict';
    /**
 * @ngdoc directive
 * @name mdTabs
 * @module material.components.tabs
 *
 * @restrict E
 *
 * @description
 * The `<md-tabs>` directive serves as the container for 1..n `<md-tab>` child directives to produces a Tabs components.
 * In turn, the nested `<md-tab>` directive is used to specify a tab label for the **header button** and a [optional] tab view
 * content that will be associated with each tab button.
 *
 * Below is the markup for its simplest usage:
 *
 *  <hljs lang="html">
 *  <md-tabs>
 *    <md-tab label="Tab #1"></md-tab>
 *    <md-tab label="Tab #2"></md-tab>
 *    <md-tab label="Tab #3"></md-tab>
 *  </md-tabs>
 *  </hljs>
 *
 * Tabs supports three (3) usage scenarios:
 *
 *  1. Tabs (buttons only)
 *  2. Tabs with internal view content
 *  3. Tabs with external view content
 *
 * **Tab-only** support is useful when tab buttons are used for custom navigation regardless of any other components, content, or views.
 * **Tabs with internal views** are the traditional usages where each tab has associated view content and the view switching is managed internally by the Tabs component.
 * **Tabs with external view content** is often useful when content associated with each tab is independently managed and data-binding notifications announce tab selection changes.
 *
 * Additional features also include:
 *
 * *  Content can include any markup.
 * *  If a tab is disabled while active/selected, then the next tab will be auto-selected.
 *
 * ### Explanation of tab stretching
 *
 * Initially, tabs will have an inherent size.  This size will either be defined by how much space is needed to accommodate their text or set by the user through CSS.  Calculations will be based on this size.
 *
 * On mobile devices, tabs will be expanded to fill the available horizontal space.  When this happens, all tabs will become the same size.
 *
 * On desktops, by default, stretching will never occur.
 *
 * This default behavior can be overridden through the `md-stretch-tabs` attribute.  Here is a table showing when stretching will occur:
 *
 * `md-stretch-tabs` | mobile    | desktop
 * ------------------|-----------|--------
 * `auto`            | stretched | ---
 * `always`          | stretched | stretched
 * `never`           | ---       | ---
 *
 * @param {integer=} md-selected Index of the active/selected tab
 * @param {boolean=} md-no-ink If present, disables ink ripple effects.
 * @param {boolean=} md-no-ink-bar If present, disables the selection ink bar.
 * @param {string=}  md-align-tabs Attribute to indicate position of tab buttons: `bottom` or `top`; default is `top`
 * @param {string=} md-stretch-tabs Attribute to indicate whether or not to stretch tabs: `auto`, `always`, or `never`; default is `auto`
 * @param {boolean=} md-dynamic-height When enabled, the tab wrapper will resize based on the contents of the selected tab
 * @param {boolean=} md-center-tabs When enabled, tabs will be centered provided there is no need for pagination
 * @param {boolean=} md-no-pagination When enabled, pagination will remain off
 * @param {boolean=} md-swipe-content When enabled, swipe gestures will be enabled for the content area to jump between tabs
 * @param {boolean=} md-enable-disconnect When enabled, scopes will be disconnected for tabs that are not being displayed.  This provides a performance boost, but may also cause unexpected issues and is not recommended for most users.
 * @param {boolean=} md-autoselect When present, any tabs added after the initial load will be automatically selected
 *
 * @usage
 * <hljs lang="html">
 * <md-tabs md-selected="selectedIndex" >
 *   <img ng-src="img/angular.png" class="centered">
 *   <md-tab
 *       ng-repeat="tab in tabs | orderBy:predicate:reversed"
 *       md-on-select="onTabSelected(tab)"
 *       md-on-deselect="announceDeselected(tab)"
 *       ng-disabled="tab.disabled">
 *     <md-tab-label>
 *       {{tab.title}}
 *       <img src="img/removeTab.png" ng-click="removeTab(tab)" class="delete">
 *     </md-tab-label>
 *     <md-tab-body>
 *       {{tab.content}}
 *     </md-tab-body>
 *   </md-tab>
 * </md-tabs>
 * </hljs>
 *
 */
    angular.module('material.components.tabs').directive('mdTabs', MdTabs);
    function MdTabs() {
      return {
        scope: { selectedIndex: '=?mdSelected' },
        template: function (element, attr) {
          attr['$mdTabsTemplate'] = element.html();
          return '' + '<md-tabs-wrapper> ' + '<md-tab-data></md-tab-data> ' + '<md-prev-button ' + 'tabindex="-1" ' + 'role="button" ' + 'aria-label="Previous Page" ' + 'aria-disabled="{{!$mdTabsCtrl.canPageBack()}}" ' + 'ng-class="{ \'md-disabled\': !$mdTabsCtrl.canPageBack() }" ' + 'ng-if="$mdTabsCtrl.shouldPaginate" ' + 'ng-click="$mdTabsCtrl.previousPage()"> ' + '<md-icon md-svg-icon="md-tabs-arrow"></md-icon> ' + '</md-prev-button> ' + '<md-next-button ' + 'tabindex="-1" ' + 'role="button" ' + 'aria-label="Next Page" ' + 'aria-disabled="{{!$mdTabsCtrl.canPageForward()}}" ' + 'ng-class="{ \'md-disabled\': !$mdTabsCtrl.canPageForward() }" ' + 'ng-if="$mdTabsCtrl.shouldPaginate" ' + 'ng-click="$mdTabsCtrl.nextPage()"> ' + '<md-icon md-svg-icon="md-tabs-arrow"></md-icon> ' + '</md-next-button> ' + '<md-tabs-canvas ' + 'tabindex="{{ $mdTabsCtrl.hasFocus ? -1 : 0 }}" ' + 'aria-activedescendant="tab-item-{{$mdTabsCtrl.tabs[$mdTabsCtrl.focusIndex].id}}" ' + 'ng-focus="$mdTabsCtrl.redirectFocus()" ' + 'ng-class="{ ' + '\'md-paginated\': $mdTabsCtrl.shouldPaginate, ' + '\'md-center-tabs\': $mdTabsCtrl.shouldCenterTabs ' + '}" ' + 'ng-keydown="$mdTabsCtrl.keydown($event)" ' + 'role="tablist"> ' + '<md-pagination-wrapper ' + 'ng-class="{ \'md-center-tabs\': $mdTabsCtrl.shouldCenterTabs }" ' + 'md-tab-scroll="$mdTabsCtrl.scroll($event)"> ' + '<md-tab-item ' + 'tabindex="-1" ' + 'class="md-tab" ' + 'style="max-width: {{ $mdTabsCtrl.maxTabWidth + \'px\' }}" ' + 'ng-repeat="tab in $mdTabsCtrl.tabs" ' + 'role="tab" ' + 'aria-controls="tab-content-{{::tab.id}}" ' + 'aria-selected="{{tab.isActive()}}" ' + 'aria-disabled="{{tab.scope.disabled || \'false\'}}" ' + 'ng-click="$mdTabsCtrl.select(tab.getIndex())" ' + 'ng-class="{ ' + '\'md-active\':    tab.isActive(), ' + '\'md-focused\':   tab.hasFocus(), ' + '\'md-disabled\':  tab.scope.disabled ' + '}" ' + 'ng-disabled="tab.scope.disabled" ' + 'md-swipe-left="$mdTabsCtrl.nextPage()" ' + 'md-swipe-right="$mdTabsCtrl.previousPage()" ' + 'md-tabs-template="::tab.label" ' + 'md-scope="::tab.parent"></md-tab-item> ' + '<md-ink-bar></md-ink-bar> ' + '</md-pagination-wrapper> ' + '<div class="md-visually-hidden md-dummy-wrapper"> ' + '<md-dummy-tab ' + 'class="md-tab" ' + 'tabindex="-1" ' + 'id="tab-item-{{::tab.id}}" ' + 'role="tab" ' + 'aria-controls="tab-content-{{::tab.id}}" ' + 'aria-selected="{{tab.isActive()}}" ' + 'aria-disabled="{{tab.scope.disabled || \'false\'}}" ' + 'ng-focus="$mdTabsCtrl.hasFocus = true" ' + 'ng-blur="$mdTabsCtrl.hasFocus = false" ' + 'ng-repeat="tab in $mdTabsCtrl.tabs" ' + 'md-tabs-template="::tab.label" ' + 'md-scope="::tab.parent"></md-dummy-tab> ' + '</div> ' + '</md-tabs-canvas> ' + '</md-tabs-wrapper> ' + '<md-tabs-content-wrapper ng-show="$mdTabsCtrl.hasContent && $mdTabsCtrl.selectedIndex >= 0"> ' + '<md-tab-content ' + 'id="tab-content-{{::tab.id}}" ' + 'role="tabpanel" ' + 'aria-labelledby="tab-item-{{::tab.id}}" ' + 'md-swipe-left="$mdTabsCtrl.swipeContent && $mdTabsCtrl.incrementIndex(1)" ' + 'md-swipe-right="$mdTabsCtrl.swipeContent && $mdTabsCtrl.incrementIndex(-1)" ' + 'ng-if="$mdTabsCtrl.hasContent" ' + 'ng-repeat="(index, tab) in $mdTabsCtrl.tabs" ' + 'ng-class="{ ' + '\'md-no-transition\': $mdTabsCtrl.lastSelectedIndex == null, ' + '\'md-active\':        tab.isActive(), ' + '\'md-left\':          tab.isLeft(), ' + '\'md-right\':         tab.isRight(), ' + '\'md-no-scroll\':     $mdTabsCtrl.dynamicHeight ' + '}"> ' + '<div ' + 'md-tabs-template="::tab.template" ' + 'md-connected-if="tab.isActive()" ' + 'md-scope="::tab.parent" ' + 'ng-if="$mdTabsCtrl.enableDisconnect || tab.shouldRender()"></div> ' + '</md-tab-content> ' + '</md-tabs-content-wrapper>';
        },
        controller: 'MdTabsController',
        controllerAs: '$mdTabsCtrl',
        bindToController: true
      };
    }
  }());
  (function () {
    'use strict';
    angular.module('material.components.tabs').directive('mdTabsTemplate', MdTabsTemplate);
    function MdTabsTemplate($compile, $mdUtil) {
      return {
        restrict: 'A',
        link: link,
        scope: {
          template: '=mdTabsTemplate',
          connected: '=?mdConnectedIf',
          compileScope: '=mdScope'
        },
        require: '^?mdTabs'
      };
      function link(scope, element, attr, ctrl) {
        if (!ctrl)
          return;
        var compileScope = ctrl.enableDisconnect ? scope.compileScope.$new() : scope.compileScope;
        element.html(scope.template);
        $compile(element.contents())(compileScope);
        element.on('DOMSubtreeModified', function () {
          ctrl.updatePagination();
          ctrl.updateInkBarStyles();
        });
        return $mdUtil.nextTick(handleScope);
        function handleScope() {
          scope.$watch('connected', function (value) {
            value === false ? disconnect() : reconnect();
          });
          scope.$on('$destroy', reconnect);
        }
        function disconnect() {
          if (ctrl.enableDisconnect)
            $mdUtil.disconnectScope(compileScope);
        }
        function reconnect() {
          if (ctrl.enableDisconnect)
            $mdUtil.reconnectScope(compileScope);
        }
      }
    }
    MdTabsTemplate.$inject = [
      '$compile',
      '$mdUtil'
    ];
  }());
  (function () {
    angular.module('material.core').constant('$MD_THEME_CSS', 'md-autocomplete.md-THEME_NAME-theme {  background: \'{{background-50}}\'; }  md-autocomplete.md-THEME_NAME-theme[disabled] {    background: \'{{background-100}}\'; }  md-autocomplete.md-THEME_NAME-theme button md-icon path {    fill: \'{{background-600}}\'; }  md-autocomplete.md-THEME_NAME-theme button:after {    background: \'{{background-600-0.3}}\'; }.md-autocomplete-suggestions-container.md-THEME_NAME-theme {  background: \'{{background-50}}\'; }  .md-autocomplete-suggestions-container.md-THEME_NAME-theme li {    color: \'{{background-900}}\'; }    .md-autocomplete-suggestions-container.md-THEME_NAME-theme li .highlight {      color: \'{{background-600}}\'; }    .md-autocomplete-suggestions-container.md-THEME_NAME-theme li:hover, .md-autocomplete-suggestions-container.md-THEME_NAME-theme li.selected {      background: \'{{background-200}}\'; }md-backdrop {  background-color: \'{{background-900-0.0}}\'; }  md-backdrop.md-opaque.md-THEME_NAME-theme {    background-color: \'{{background-900-1.0}}\'; }md-bottom-sheet.md-THEME_NAME-theme {  background-color: \'{{background-50}}\';  border-top-color: \'{{background-300}}\'; }  md-bottom-sheet.md-THEME_NAME-theme.md-list md-list-item {    color: \'{{foreground-1}}\'; }  md-bottom-sheet.md-THEME_NAME-theme .md-subheader {    background-color: \'{{background-50}}\'; }  md-bottom-sheet.md-THEME_NAME-theme .md-subheader {    color: \'{{foreground-1}}\'; }a.md-button.md-THEME_NAME-theme:not([disabled]):hover,.md-button.md-THEME_NAME-theme:not([disabled]):hover {  background-color: \'{{background-500-0.2}}\'; }a.md-button.md-THEME_NAME-theme:not([disabled]).md-focused,.md-button.md-THEME_NAME-theme:not([disabled]).md-focused {  background-color: \'{{background-500-0.2}}\'; }a.md-button.md-THEME_NAME-theme:not([disabled]).md-icon-button:hover,.md-button.md-THEME_NAME-theme:not([disabled]).md-icon-button:hover {  background-color: transparent; }a.md-button.md-THEME_NAME-theme.md-fab,.md-button.md-THEME_NAME-theme.md-fab {  background-color: \'{{accent-color}}\';  color: \'{{accent-contrast}}\'; }  a.md-button.md-THEME_NAME-theme.md-fab md-icon,  .md-button.md-THEME_NAME-theme.md-fab md-icon {    color: \'{{accent-contrast}}\'; }  a.md-button.md-THEME_NAME-theme.md-fab:not([disabled]):hover,  .md-button.md-THEME_NAME-theme.md-fab:not([disabled]):hover {    background-color: \'{{accent-color}}\'; }  a.md-button.md-THEME_NAME-theme.md-fab:not([disabled]).md-focused,  .md-button.md-THEME_NAME-theme.md-fab:not([disabled]).md-focused {    background-color: \'{{accent-A700}}\'; }a.md-button.md-THEME_NAME-theme.md-primary,.md-button.md-THEME_NAME-theme.md-primary {  color: \'{{primary-color}}\'; }  a.md-button.md-THEME_NAME-theme.md-primary.md-raised, a.md-button.md-THEME_NAME-theme.md-primary.md-fab,  .md-button.md-THEME_NAME-theme.md-primary.md-raised,  .md-button.md-THEME_NAME-theme.md-primary.md-fab {    color: \'{{primary-contrast}}\';    background-color: \'{{primary-color}}\'; }    a.md-button.md-THEME_NAME-theme.md-primary.md-raised:not([disabled]) md-icon, a.md-button.md-THEME_NAME-theme.md-primary.md-fab:not([disabled]) md-icon,    .md-button.md-THEME_NAME-theme.md-primary.md-raised:not([disabled]) md-icon,    .md-button.md-THEME_NAME-theme.md-primary.md-fab:not([disabled]) md-icon {      color: \'{{primary-contrast}}\'; }    a.md-button.md-THEME_NAME-theme.md-primary.md-raised:not([disabled]):hover, a.md-button.md-THEME_NAME-theme.md-primary.md-fab:not([disabled]):hover,    .md-button.md-THEME_NAME-theme.md-primary.md-raised:not([disabled]):hover,    .md-button.md-THEME_NAME-theme.md-primary.md-fab:not([disabled]):hover {      background-color: \'{{primary-color}}\'; }    a.md-button.md-THEME_NAME-theme.md-primary.md-raised:not([disabled]).md-focused, a.md-button.md-THEME_NAME-theme.md-primary.md-fab:not([disabled]).md-focused,    .md-button.md-THEME_NAME-theme.md-primary.md-raised:not([disabled]).md-focused,    .md-button.md-THEME_NAME-theme.md-primary.md-fab:not([disabled]).md-focused {      background-color: \'{{primary-600}}\'; }  a.md-button.md-THEME_NAME-theme.md-primary:not([disabled]) md-icon,  .md-button.md-THEME_NAME-theme.md-primary:not([disabled]) md-icon {    color: \'{{primary-color}}\'; }a.md-button.md-THEME_NAME-theme.md-fab,.md-button.md-THEME_NAME-theme.md-fab {  background-color: \'{{accent-color}}\';  color: \'{{accent-contrast}}\'; }  a.md-button.md-THEME_NAME-theme.md-fab:not([disabled]) .md-icon,  .md-button.md-THEME_NAME-theme.md-fab:not([disabled]) .md-icon {    color: \'{{accent-contrast}}\'; }  a.md-button.md-THEME_NAME-theme.md-fab:not([disabled]):hover,  .md-button.md-THEME_NAME-theme.md-fab:not([disabled]):hover {    background-color: \'{{accent-color}}\'; }  a.md-button.md-THEME_NAME-theme.md-fab:not([disabled]).md-focused,  .md-button.md-THEME_NAME-theme.md-fab:not([disabled]).md-focused {    background-color: \'{{accent-A700}}\'; }a.md-button.md-THEME_NAME-theme.md-raised,.md-button.md-THEME_NAME-theme.md-raised {  color: \'{{background-900}}\';  background-color: \'{{background-50}}\'; }  a.md-button.md-THEME_NAME-theme.md-raised:not([disabled]) .md-icon,  .md-button.md-THEME_NAME-theme.md-raised:not([disabled]) .md-icon {    color: \'{{background-contrast}}\'; }  a.md-button.md-THEME_NAME-theme.md-raised:not([disabled]):hover,  .md-button.md-THEME_NAME-theme.md-raised:not([disabled]):hover {    background-color: \'{{background-50}}\'; }  a.md-button.md-THEME_NAME-theme.md-raised:not([disabled]).md-focused,  .md-button.md-THEME_NAME-theme.md-raised:not([disabled]).md-focused {    background-color: \'{{background-200}}\'; }a.md-button.md-THEME_NAME-theme.md-warn,.md-button.md-THEME_NAME-theme.md-warn {  color: \'{{warn-color}}\'; }  a.md-button.md-THEME_NAME-theme.md-warn.md-raised, a.md-button.md-THEME_NAME-theme.md-warn.md-fab,  .md-button.md-THEME_NAME-theme.md-warn.md-raised,  .md-button.md-THEME_NAME-theme.md-warn.md-fab {    color: \'{{warn-contrast}}\';    background-color: \'{{warn-color}}\'; }    a.md-button.md-THEME_NAME-theme.md-warn.md-raised:not([disabled]) md-icon, a.md-button.md-THEME_NAME-theme.md-warn.md-fab:not([disabled]) md-icon,    .md-button.md-THEME_NAME-theme.md-warn.md-raised:not([disabled]) md-icon,    .md-button.md-THEME_NAME-theme.md-warn.md-fab:not([disabled]) md-icon {      color: \'{{warn-contrast}}\'; }    a.md-button.md-THEME_NAME-theme.md-warn.md-raised:not([disabled]):hover, a.md-button.md-THEME_NAME-theme.md-warn.md-fab:not([disabled]):hover,    .md-button.md-THEME_NAME-theme.md-warn.md-raised:not([disabled]):hover,    .md-button.md-THEME_NAME-theme.md-warn.md-fab:not([disabled]):hover {      background-color: \'{{warn-color}}\'; }    a.md-button.md-THEME_NAME-theme.md-warn.md-raised:not([disabled]).md-focused, a.md-button.md-THEME_NAME-theme.md-warn.md-fab:not([disabled]).md-focused,    .md-button.md-THEME_NAME-theme.md-warn.md-raised:not([disabled]).md-focused,    .md-button.md-THEME_NAME-theme.md-warn.md-fab:not([disabled]).md-focused {      background-color: \'{{warn-700}}\'; }  a.md-button.md-THEME_NAME-theme.md-warn:not([disabled]) md-icon,  .md-button.md-THEME_NAME-theme.md-warn:not([disabled]) md-icon {    color: \'{{warn-color}}\'; }a.md-button.md-THEME_NAME-theme.md-accent,.md-button.md-THEME_NAME-theme.md-accent {  color: \'{{accent-color}}\'; }  a.md-button.md-THEME_NAME-theme.md-accent.md-raised, a.md-button.md-THEME_NAME-theme.md-accent.md-fab,  .md-button.md-THEME_NAME-theme.md-accent.md-raised,  .md-button.md-THEME_NAME-theme.md-accent.md-fab {    color: \'{{accent-contrast}}\';    background-color: \'{{accent-color}}\'; }    a.md-button.md-THEME_NAME-theme.md-accent.md-raised:not([disabled]) md-icon, a.md-button.md-THEME_NAME-theme.md-accent.md-fab:not([disabled]) md-icon,    .md-button.md-THEME_NAME-theme.md-accent.md-raised:not([disabled]) md-icon,    .md-button.md-THEME_NAME-theme.md-accent.md-fab:not([disabled]) md-icon {      color: \'{{accent-contrast}}\'; }    a.md-button.md-THEME_NAME-theme.md-accent.md-raised:not([disabled]):hover, a.md-button.md-THEME_NAME-theme.md-accent.md-fab:not([disabled]):hover,    .md-button.md-THEME_NAME-theme.md-accent.md-raised:not([disabled]):hover,    .md-button.md-THEME_NAME-theme.md-accent.md-fab:not([disabled]):hover {      background-color: \'{{accent-color}}\'; }    a.md-button.md-THEME_NAME-theme.md-accent.md-raised:not([disabled]).md-focused, a.md-button.md-THEME_NAME-theme.md-accent.md-fab:not([disabled]).md-focused,    .md-button.md-THEME_NAME-theme.md-accent.md-raised:not([disabled]).md-focused,    .md-button.md-THEME_NAME-theme.md-accent.md-fab:not([disabled]).md-focused {      background-color: \'{{accent-700}}\'; }  a.md-button.md-THEME_NAME-theme.md-accent:not([disabled]) md-icon,  .md-button.md-THEME_NAME-theme.md-accent:not([disabled]) md-icon {    color: \'{{accent-color}}\'; }a.md-button.md-THEME_NAME-theme[disabled], a.md-button.md-THEME_NAME-theme.md-raised[disabled], a.md-button.md-THEME_NAME-theme.md-fab[disabled], a.md-button.md-THEME_NAME-theme.md-accent[disabled], a.md-button.md-THEME_NAME-theme.md-warn[disabled],.md-button.md-THEME_NAME-theme[disabled],.md-button.md-THEME_NAME-theme.md-raised[disabled],.md-button.md-THEME_NAME-theme.md-fab[disabled],.md-button.md-THEME_NAME-theme.md-accent[disabled],.md-button.md-THEME_NAME-theme.md-warn[disabled] {  color: \'{{foreground-3}}\' !important;  cursor: default; }  a.md-button.md-THEME_NAME-theme[disabled] md-icon, a.md-button.md-THEME_NAME-theme.md-raised[disabled] md-icon, a.md-button.md-THEME_NAME-theme.md-fab[disabled] md-icon, a.md-button.md-THEME_NAME-theme.md-accent[disabled] md-icon, a.md-button.md-THEME_NAME-theme.md-warn[disabled] md-icon,  .md-button.md-THEME_NAME-theme[disabled] md-icon,  .md-button.md-THEME_NAME-theme.md-raised[disabled] md-icon,  .md-button.md-THEME_NAME-theme.md-fab[disabled] md-icon,  .md-button.md-THEME_NAME-theme.md-accent[disabled] md-icon,  .md-button.md-THEME_NAME-theme.md-warn[disabled] md-icon {    color: \'{{foreground-3}}\'; }a.md-button.md-THEME_NAME-theme.md-raised[disabled], a.md-button.md-THEME_NAME-theme.md-fab[disabled],.md-button.md-THEME_NAME-theme.md-raised[disabled],.md-button.md-THEME_NAME-theme.md-fab[disabled] {  background-color: \'{{foreground-4}}\'; }a.md-button.md-THEME_NAME-theme[disabled],.md-button.md-THEME_NAME-theme[disabled] {  background-color: transparent; }md-card.md-THEME_NAME-theme {  background-color: \'{{background-color}}\';  border-radius: 2px; }  md-card.md-THEME_NAME-theme .md-card-image {    border-radius: 2px 2px 0 0; }  md-card.md-THEME_NAME-theme md-card-header md-card-avatar md-icon {    color: \'{{background-color}}\';    background-color: \'{{foreground-3}}\'; }  md-card.md-THEME_NAME-theme md-card-header md-card-header-text .md-subhead {    color: \'{{foreground-2}}\'; }  md-card.md-THEME_NAME-theme md-card-title md-card-title-text:not(:only-child) .md-subhead {    color: \'{{foreground-2}}\'; }md-checkbox.md-THEME_NAME-theme .md-ripple {  color: \'{{accent-600}}\'; }md-checkbox.md-THEME_NAME-theme.md-checked .md-ripple {  color: \'{{background-600}}\'; }md-checkbox.md-THEME_NAME-theme.md-checked.md-focused .md-container:before {  background-color: \'{{accent-color-0.26}}\'; }md-checkbox.md-THEME_NAME-theme .md-ink-ripple {  color: \'{{foreground-2}}\'; }md-checkbox.md-THEME_NAME-theme.md-checked .md-ink-ripple {  color: \'{{accent-color-0.87}}\'; }md-checkbox.md-THEME_NAME-theme .md-icon {  border-color: \'{{foreground-2}}\'; }md-checkbox.md-THEME_NAME-theme.md-checked .md-icon {  background-color: \'{{accent-color-0.87}}\'; }md-checkbox.md-THEME_NAME-theme.md-checked .md-icon:after {  border-color: \'{{accent-contrast-0.87}}\'; }md-checkbox.md-THEME_NAME-theme:not([disabled]).md-primary .md-ripple {  color: \'{{primary-600}}\'; }md-checkbox.md-THEME_NAME-theme:not([disabled]).md-primary.md-checked .md-ripple {  color: \'{{background-600}}\'; }md-checkbox.md-THEME_NAME-theme:not([disabled]).md-primary .md-ink-ripple {  color: \'{{foreground-2}}\'; }md-checkbox.md-THEME_NAME-theme:not([disabled]).md-primary.md-checked .md-ink-ripple {  color: \'{{primary-color-0.87}}\'; }md-checkbox.md-THEME_NAME-theme:not([disabled]).md-primary .md-icon {  border-color: \'{{foreground-2}}\'; }md-checkbox.md-THEME_NAME-theme:not([disabled]).md-primary.md-checked .md-icon {  background-color: \'{{primary-color-0.87}}\'; }md-checkbox.md-THEME_NAME-theme:not([disabled]).md-primary.md-checked.md-focused .md-container:before {  background-color: \'{{primary-color-0.26}}\'; }md-checkbox.md-THEME_NAME-theme:not([disabled]).md-primary.md-checked .md-icon:after {  border-color: \'{{primary-contrast-0.87}}\'; }md-checkbox.md-THEME_NAME-theme:not([disabled]).md-warn .md-ripple {  color: \'{{warn-600}}\'; }md-checkbox.md-THEME_NAME-theme:not([disabled]).md-warn .md-ink-ripple {  color: \'{{foreground-2}}\'; }md-checkbox.md-THEME_NAME-theme:not([disabled]).md-warn.md-checked .md-ink-ripple {  color: \'{{warn-color-0.87}}\'; }md-checkbox.md-THEME_NAME-theme:not([disabled]).md-warn .md-icon {  border-color: \'{{foreground-2}}\'; }md-checkbox.md-THEME_NAME-theme:not([disabled]).md-warn.md-checked .md-icon {  background-color: \'{{warn-color-0.87}}\'; }md-checkbox.md-THEME_NAME-theme:not([disabled]).md-warn.md-checked.md-focused:not([disabled]) .md-container:before {  background-color: \'{{warn-color-0.26}}\'; }md-checkbox.md-THEME_NAME-theme:not([disabled]).md-warn.md-checked .md-icon:after {  border-color: \'{{background-200}}\'; }md-checkbox.md-THEME_NAME-theme[disabled] .md-icon {  border-color: \'{{foreground-3}}\'; }md-checkbox.md-THEME_NAME-theme[disabled].md-checked .md-icon {  background-color: \'{{foreground-3}}\'; }md-checkbox.md-THEME_NAME-theme[disabled].md-checked .md-icon:after {  border-color: \'{{background-200}}\'; }md-checkbox.md-THEME_NAME-theme[disabled] .md-label {  color: \'{{foreground-3}}\'; }md-chips.md-THEME_NAME-theme .md-chips {  box-shadow: 0 1px \'{{background-300}}\'; }  md-chips.md-THEME_NAME-theme .md-chips.md-focused {    box-shadow: 0 2px \'{{primary-color}}\'; }md-chips.md-THEME_NAME-theme .md-chip {  background: \'{{background-300}}\';  color: \'{{background-800}}\'; }  md-chips.md-THEME_NAME-theme .md-chip.md-focused {    background: \'{{primary-color}}\';    color: \'{{primary-contrast}}\'; }    md-chips.md-THEME_NAME-theme .md-chip.md-focused md-icon {      color: \'{{primary-contrast}}\'; }md-chips.md-THEME_NAME-theme md-chip-remove .md-button md-icon path {  fill: \'{{background-500}}\'; }.md-contact-suggestion span.md-contact-email {  color: \'{{background-400}}\'; }md-content.md-THEME_NAME-theme {  color: \'{{foreground-1}}\';  background-color: \'{{background-color}}\'; }/** Theme styles for mdCalendar. */.md-calendar.md-THEME_NAME-theme {  color: \'{{foreground-1}}\'; }  .md-calendar.md-THEME_NAME-theme tr:last-child td {    border-bottom-color: \'{{background-200}}\'; }.md-THEME_NAME-theme .md-calendar-day-header {  background: \'{{background-hue-1}}\';  color: \'{{foreground-1}}\'; }.md-THEME_NAME-theme .md-calendar-date.md-calendar-date-today .md-calendar-date-selection-indicator {  border: 1px solid \'{{primary-500}}\'; }.md-THEME_NAME-theme .md-calendar-date.md-calendar-date-today.md-calendar-date-disabled {  color: \'{{primary-500-0.6}}\'; }.md-THEME_NAME-theme .md-calendar-date.md-focus .md-calendar-date-selection-indicator {  background: \'{{background-hue-1}}\'; }.md-THEME_NAME-theme .md-calendar-date-selection-indicator:hover {  background: \'{{background-hue-1}}\'; }.md-THEME_NAME-theme .md-calendar-date.md-calendar-selected-date .md-calendar-date-selection-indicator,.md-THEME_NAME-theme .md-calendar-date.md-focus.md-calendar-selected-date .md-calendar-date-selection-indicator {  background: \'{{primary-500}}\';  color: \'{{primary-500-contrast}}\';  border-color: transparent; }.md-THEME_NAME-theme .md-calendar-date-disabled,.md-THEME_NAME-theme .md-calendar-month-label-disabled {  color: \'{{foreground-3}}\'; }/** Theme styles for mdDatepicker. */md-datepicker.md-THEME_NAME-theme {  background: \'{{background-color}}\'; }.md-THEME_NAME-theme .md-datepicker-input {  color: \'{{background-contrast}}\';  background: \'{{background-color}}\'; }  .md-THEME_NAME-theme .md-datepicker-input::-webkit-input-placeholder, .md-THEME_NAME-theme .md-datepicker-input::-moz-placeholder, .md-THEME_NAME-theme .md-datepicker-input:-moz-placeholder, .md-THEME_NAME-theme .md-datepicker-input:-ms-input-placeholder {    color: "{{foreground-3}}"; }.md-THEME_NAME-theme .md-datepicker-input-container {  border-bottom-color: \'{{background-300}}\'; }  .md-THEME_NAME-theme .md-datepicker-input-container.md-datepicker-focused {    border-bottom-color: \'{{primary-500}}\'; }  .md-THEME_NAME-theme .md-datepicker-input-container.md-datepicker-invalid {    border-bottom-color: \'{{warn-A700}}\'; }.md-THEME_NAME-theme .md-datepicker-calendar-pane {  border-color: \'{{background-300}}\'; }.md-THEME_NAME-theme .md-datepicker-triangle-button .md-datepicker-expand-triangle {  border-top-color: \'{{foreground-3}}\'; }.md-THEME_NAME-theme .md-datepicker-triangle-button:hover .md-datepicker-expand-triangle {  border-top-color: \'{{foreground-2}}\'; }.md-THEME_NAME-theme .md-datepicker-open .md-datepicker-calendar-icon {  fill: \'{{primary-500}}\'; }.md-THEME_NAME-theme .md-datepicker-calendar,.md-THEME_NAME-theme .md-datepicker-input-mask-opaque {  background: \'{{background-color}}\'; }md-dialog.md-THEME_NAME-theme {  border-radius: 4px;  background-color: \'{{background-color}}\'; }  md-dialog.md-THEME_NAME-theme.md-content-overflow .md-actions, md-dialog.md-THEME_NAME-theme.md-content-overflow md-dialog-actions {    border-top-color: \'{{foreground-4}}\'; }md-divider.md-THEME_NAME-theme {  border-top-color: \'{{foreground-4}}\'; }.layout-row > md-divider.md-THEME_NAME-theme {  border-right-color: \'{{foreground-4}}\'; }md-icon.md-THEME_NAME-theme {  color: \'{{foreground-2}}\'; }  md-icon.md-THEME_NAME-theme.md-primary {    color: \'{{primary-color}}\'; }  md-icon.md-THEME_NAME-theme.md-accent {    color: \'{{accent-color}}\'; }  md-icon.md-THEME_NAME-theme.md-warn {    color: \'{{warn-color}}\'; }md-input-container.md-THEME_NAME-theme .md-input {  color: \'{{foreground-1}}\';  border-color: \'{{foreground-4}}\';  text-shadow: \'{{foreground-shadow}}\'; }  md-input-container.md-THEME_NAME-theme .md-input::-webkit-input-placeholder, md-input-container.md-THEME_NAME-theme .md-input::-moz-placeholder, md-input-container.md-THEME_NAME-theme .md-input:-moz-placeholder, md-input-container.md-THEME_NAME-theme .md-input:-ms-input-placeholder {    color: "{{foreground-3}}"; }md-input-container.md-THEME_NAME-theme > md-icon {  color: \'{{foreground-1}}\'; }md-input-container.md-THEME_NAME-theme label,md-input-container.md-THEME_NAME-theme .md-placeholder {  text-shadow: \'{{foreground-shadow}}\';  color: \'{{foreground-3}}\'; }md-input-container.md-THEME_NAME-theme ng-messages :not(.md-char-counter), md-input-container.md-THEME_NAME-theme [ng-messages] :not(.md-char-counter),md-input-container.md-THEME_NAME-theme ng-message :not(.md-char-counter), md-input-container.md-THEME_NAME-theme data-ng-message :not(.md-char-counter), md-input-container.md-THEME_NAME-theme x-ng-message :not(.md-char-counter),md-input-container.md-THEME_NAME-theme [ng-message] :not(.md-char-counter), md-input-container.md-THEME_NAME-theme [data-ng-message] :not(.md-char-counter), md-input-container.md-THEME_NAME-theme [x-ng-message] :not(.md-char-counter),md-input-container.md-THEME_NAME-theme [ng-message-exp] :not(.md-char-counter), md-input-container.md-THEME_NAME-theme [data-ng-message-exp] :not(.md-char-counter), md-input-container.md-THEME_NAME-theme [x-ng-message-exp] :not(.md-char-counter) {  color: \'{{warn-A700}}\'; }md-input-container.md-THEME_NAME-theme:not(.md-input-invalid).md-input-has-value label {  color: \'{{foreground-2}}\'; }md-input-container.md-THEME_NAME-theme:not(.md-input-invalid).md-input-focused .md-input {  border-color: \'{{primary-500}}\'; }md-input-container.md-THEME_NAME-theme:not(.md-input-invalid).md-input-focused label {  color: \'{{primary-500}}\'; }md-input-container.md-THEME_NAME-theme:not(.md-input-invalid).md-input-focused md-icon {  color: \'{{primary-500}}\'; }md-input-container.md-THEME_NAME-theme:not(.md-input-invalid).md-input-focused.md-accent .md-input {  border-color: \'{{accent-500}}\'; }md-input-container.md-THEME_NAME-theme:not(.md-input-invalid).md-input-focused.md-accent label {  color: \'{{accent-500}}\'; }md-input-container.md-THEME_NAME-theme:not(.md-input-invalid).md-input-focused.md-warn .md-input {  border-color: \'{{warn-A700}}\'; }md-input-container.md-THEME_NAME-theme:not(.md-input-invalid).md-input-focused.md-warn label {  color: \'{{warn-A700}}\'; }md-input-container.md-THEME_NAME-theme.md-input-invalid .md-input {  border-color: \'{{warn-A700}}\'; }md-input-container.md-THEME_NAME-theme.md-input-invalid.md-input-focused label {  color: \'{{warn-A700}}\'; }md-input-container.md-THEME_NAME-theme.md-input-invalid ng-message, md-input-container.md-THEME_NAME-theme.md-input-invalid data-ng-message, md-input-container.md-THEME_NAME-theme.md-input-invalid x-ng-message,md-input-container.md-THEME_NAME-theme.md-input-invalid [ng-message], md-input-container.md-THEME_NAME-theme.md-input-invalid [data-ng-message], md-input-container.md-THEME_NAME-theme.md-input-invalid [x-ng-message],md-input-container.md-THEME_NAME-theme.md-input-invalid [ng-message-exp], md-input-container.md-THEME_NAME-theme.md-input-invalid [data-ng-message-exp], md-input-container.md-THEME_NAME-theme.md-input-invalid [x-ng-message-exp],md-input-container.md-THEME_NAME-theme.md-input-invalid .md-char-counter {  color: \'{{warn-A700}}\'; }md-input-container.md-THEME_NAME-theme .md-input[disabled],md-input-container.md-THEME_NAME-theme .md-input [disabled] {  border-bottom-color: transparent;  color: \'{{foreground-3}}\';  background-image: linear-gradient(to right, "{{foreground-3}}" 0%, "{{foreground-3}}" 33%, transparent 0%);  background-image: -ms-linear-gradient(left, transparent 0%, "{{foreground-3}}" 100%); }md-list.md-THEME_NAME-theme md-list-item.md-2-line .md-list-item-text h3, md-list.md-THEME_NAME-theme md-list-item.md-2-line .md-list-item-text h4,md-list.md-THEME_NAME-theme md-list-item.md-3-line .md-list-item-text h3,md-list.md-THEME_NAME-theme md-list-item.md-3-line .md-list-item-text h4 {  color: \'{{foreground-1}}\'; }md-list.md-THEME_NAME-theme md-list-item.md-2-line .md-list-item-text p,md-list.md-THEME_NAME-theme md-list-item.md-3-line .md-list-item-text p {  color: \'{{foreground-2}}\'; }md-list.md-THEME_NAME-theme .md-proxy-focus.md-focused div.md-no-style {  background-color: \'{{background-100}}\'; }md-list.md-THEME_NAME-theme md-list-item > .md-avatar-icon {  background-color: \'{{foreground-3}}\';  color: \'{{background-color}}\'; }md-list.md-THEME_NAME-theme md-list-item > md-icon {  color: \'{{foreground-2}}\'; }  md-list.md-THEME_NAME-theme md-list-item > md-icon.md-highlight {    color: \'{{primary-color}}\'; }    md-list.md-THEME_NAME-theme md-list-item > md-icon.md-highlight.md-accent {      color: \'{{accent-color}}\'; }md-menu-content.md-THEME_NAME-theme {  background-color: \'{{background-color}}\'; }  md-menu-content.md-THEME_NAME-theme md-menu-divider {    background-color: \'{{foreground-4}}\'; }md-menu-bar.md-THEME_NAME-theme > button.md-button {  color: \'{{foreground-2}}\';  border-radius: 2px; }md-menu-bar.md-THEME_NAME-theme md-menu.md-open > button, md-menu-bar.md-THEME_NAME-theme md-menu > button:focus {  outline: none;  background: \'{{background-200}}\'; }md-menu-bar.md-THEME_NAME-theme.md-open:not(.md-keyboard-mode) md-menu:hover > button {  background-color: \'{{ background-500-0.2}}\'; }md-menu-bar.md-THEME_NAME-theme:not(.md-keyboard-mode):not(.md-open) md-menu button:hover,md-menu-bar.md-THEME_NAME-theme:not(.md-keyboard-mode):not(.md-open) md-menu button:focus {  background: transparent; }md-menu-content.md-THEME_NAME-theme .md-menu > .md-button:after {  color: \'{{foreground-2}}\'; }md-menu-content.md-THEME_NAME-theme .md-menu.md-open > .md-button {  background-color: \'{{ background-500-0.2}}\'; }md-toolbar.md-THEME_NAME-theme.md-menu-toolbar {  background-color: \'{{background-color}}\';  color: \'{{foreground-1}}\'; }  md-toolbar.md-THEME_NAME-theme.md-menu-toolbar md-toolbar-filler {    background-color: \'{{primary-color}}\';    color: \'{{primary-contrast}}\'; }    md-toolbar.md-THEME_NAME-theme.md-menu-toolbar md-toolbar-filler md-icon {      color: \'{{primary-contrast}}\'; }md-progress-circular.md-THEME_NAME-theme {  background-color: transparent; }  md-progress-circular.md-THEME_NAME-theme .md-inner .md-gap {    border-top-color: \'{{primary-color}}\';    border-bottom-color: \'{{primary-color}}\'; }  md-progress-circular.md-THEME_NAME-theme .md-inner .md-left .md-half-circle, md-progress-circular.md-THEME_NAME-theme .md-inner .md-right .md-half-circle {    border-top-color: \'{{primary-color}}\'; }  md-progress-circular.md-THEME_NAME-theme .md-inner .md-right .md-half-circle {    border-right-color: \'{{primary-color}}\'; }  md-progress-circular.md-THEME_NAME-theme .md-inner .md-left .md-half-circle {    border-left-color: \'{{primary-color}}\'; }  md-progress-circular.md-THEME_NAME-theme.md-warn .md-inner .md-gap {    border-top-color: \'{{warn-color}}\';    border-bottom-color: \'{{warn-color}}\'; }  md-progress-circular.md-THEME_NAME-theme.md-warn .md-inner .md-left .md-half-circle, md-progress-circular.md-THEME_NAME-theme.md-warn .md-inner .md-right .md-half-circle {    border-top-color: \'{{warn-color}}\'; }  md-progress-circular.md-THEME_NAME-theme.md-warn .md-inner .md-right .md-half-circle {    border-right-color: \'{{warn-color}}\'; }  md-progress-circular.md-THEME_NAME-theme.md-warn .md-inner .md-left .md-half-circle {    border-left-color: \'{{warn-color}}\'; }  md-progress-circular.md-THEME_NAME-theme.md-accent .md-inner .md-gap {    border-top-color: \'{{accent-color}}\';    border-bottom-color: \'{{accent-color}}\'; }  md-progress-circular.md-THEME_NAME-theme.md-accent .md-inner .md-left .md-half-circle, md-progress-circular.md-THEME_NAME-theme.md-accent .md-inner .md-right .md-half-circle {    border-top-color: \'{{accent-color}}\'; }  md-progress-circular.md-THEME_NAME-theme.md-accent .md-inner .md-right .md-half-circle {    border-right-color: \'{{accent-color}}\'; }  md-progress-circular.md-THEME_NAME-theme.md-accent .md-inner .md-left .md-half-circle {    border-left-color: \'{{accent-color}}\'; }md-radio-button.md-THEME_NAME-theme .md-off {  border-color: \'{{foreground-2}}\'; }md-radio-button.md-THEME_NAME-theme .md-on {  background-color: \'{{accent-color-0.87}}\'; }md-radio-button.md-THEME_NAME-theme.md-checked .md-off {  border-color: \'{{accent-color-0.87}}\'; }md-radio-button.md-THEME_NAME-theme.md-checked .md-ink-ripple {  color: \'{{accent-color-0.87}}\'; }md-radio-button.md-THEME_NAME-theme .md-container .md-ripple {  color: \'{{accent-600}}\'; }md-radio-group.md-THEME_NAME-theme:not([disabled]) .md-primary .md-on, md-radio-group.md-THEME_NAME-theme:not([disabled]).md-primary .md-on,md-radio-button.md-THEME_NAME-theme:not([disabled]) .md-primary .md-on,md-radio-button.md-THEME_NAME-theme:not([disabled]).md-primary .md-on {  background-color: \'{{primary-color-0.87}}\'; }md-radio-group.md-THEME_NAME-theme:not([disabled]) .md-primary .md-checked .md-off, md-radio-group.md-THEME_NAME-theme:not([disabled]) .md-primary.md-checked .md-off, md-radio-group.md-THEME_NAME-theme:not([disabled]).md-primary .md-checked .md-off, md-radio-group.md-THEME_NAME-theme:not([disabled]).md-primary.md-checked .md-off,md-radio-button.md-THEME_NAME-theme:not([disabled]) .md-primary .md-checked .md-off,md-radio-button.md-THEME_NAME-theme:not([disabled]) .md-primary.md-checked .md-off,md-radio-button.md-THEME_NAME-theme:not([disabled]).md-primary .md-checked .md-off,md-radio-button.md-THEME_NAME-theme:not([disabled]).md-primary.md-checked .md-off {  border-color: \'{{primary-color-0.87}}\'; }md-radio-group.md-THEME_NAME-theme:not([disabled]) .md-primary .md-checked .md-ink-ripple, md-radio-group.md-THEME_NAME-theme:not([disabled]) .md-primary.md-checked .md-ink-ripple, md-radio-group.md-THEME_NAME-theme:not([disabled]).md-primary .md-checked .md-ink-ripple, md-radio-group.md-THEME_NAME-theme:not([disabled]).md-primary.md-checked .md-ink-ripple,md-radio-button.md-THEME_NAME-theme:not([disabled]) .md-primary .md-checked .md-ink-ripple,md-radio-button.md-THEME_NAME-theme:not([disabled]) .md-primary.md-checked .md-ink-ripple,md-radio-button.md-THEME_NAME-theme:not([disabled]).md-primary .md-checked .md-ink-ripple,md-radio-button.md-THEME_NAME-theme:not([disabled]).md-primary.md-checked .md-ink-ripple {  color: \'{{primary-color-0.87}}\'; }md-radio-group.md-THEME_NAME-theme:not([disabled]) .md-primary .md-container .md-ripple, md-radio-group.md-THEME_NAME-theme:not([disabled]).md-primary .md-container .md-ripple,md-radio-button.md-THEME_NAME-theme:not([disabled]) .md-primary .md-container .md-ripple,md-radio-button.md-THEME_NAME-theme:not([disabled]).md-primary .md-container .md-ripple {  color: \'{{primary-600}}\'; }md-radio-group.md-THEME_NAME-theme:not([disabled]) .md-warn .md-on, md-radio-group.md-THEME_NAME-theme:not([disabled]).md-warn .md-on,md-radio-button.md-THEME_NAME-theme:not([disabled]) .md-warn .md-on,md-radio-button.md-THEME_NAME-theme:not([disabled]).md-warn .md-on {  background-color: \'{{warn-color-0.87}}\'; }md-radio-group.md-THEME_NAME-theme:not([disabled]) .md-warn .md-checked .md-off, md-radio-group.md-THEME_NAME-theme:not([disabled]) .md-warn.md-checked .md-off, md-radio-group.md-THEME_NAME-theme:not([disabled]).md-warn .md-checked .md-off, md-radio-group.md-THEME_NAME-theme:not([disabled]).md-warn.md-checked .md-off,md-radio-button.md-THEME_NAME-theme:not([disabled]) .md-warn .md-checked .md-off,md-radio-button.md-THEME_NAME-theme:not([disabled]) .md-warn.md-checked .md-off,md-radio-button.md-THEME_NAME-theme:not([disabled]).md-warn .md-checked .md-off,md-radio-button.md-THEME_NAME-theme:not([disabled]).md-warn.md-checked .md-off {  border-color: \'{{warn-color-0.87}}\'; }md-radio-group.md-THEME_NAME-theme:not([disabled]) .md-warn .md-checked .md-ink-ripple, md-radio-group.md-THEME_NAME-theme:not([disabled]) .md-warn.md-checked .md-ink-ripple, md-radio-group.md-THEME_NAME-theme:not([disabled]).md-warn .md-checked .md-ink-ripple, md-radio-group.md-THEME_NAME-theme:not([disabled]).md-warn.md-checked .md-ink-ripple,md-radio-button.md-THEME_NAME-theme:not([disabled]) .md-warn .md-checked .md-ink-ripple,md-radio-button.md-THEME_NAME-theme:not([disabled]) .md-warn.md-checked .md-ink-ripple,md-radio-button.md-THEME_NAME-theme:not([disabled]).md-warn .md-checked .md-ink-ripple,md-radio-button.md-THEME_NAME-theme:not([disabled]).md-warn.md-checked .md-ink-ripple {  color: \'{{warn-color-0.87}}\'; }md-radio-group.md-THEME_NAME-theme:not([disabled]) .md-warn .md-container .md-ripple, md-radio-group.md-THEME_NAME-theme:not([disabled]).md-warn .md-container .md-ripple,md-radio-button.md-THEME_NAME-theme:not([disabled]) .md-warn .md-container .md-ripple,md-radio-button.md-THEME_NAME-theme:not([disabled]).md-warn .md-container .md-ripple {  color: \'{{warn-600}}\'; }md-radio-group.md-THEME_NAME-theme[disabled],md-radio-button.md-THEME_NAME-theme[disabled] {  color: \'{{foreground-3}}\'; }  md-radio-group.md-THEME_NAME-theme[disabled] .md-container .md-off,  md-radio-button.md-THEME_NAME-theme[disabled] .md-container .md-off {    border-color: \'{{foreground-3}}\'; }  md-radio-group.md-THEME_NAME-theme[disabled] .md-container .md-on,  md-radio-button.md-THEME_NAME-theme[disabled] .md-container .md-on {    border-color: \'{{foreground-3}}\'; }md-radio-group.md-THEME_NAME-theme .md-checked .md-ink-ripple {  color: \'{{accent-color-0.26}}\'; }md-radio-group.md-THEME_NAME-theme.md-primary .md-checked:not([disabled]) .md-ink-ripple, md-radio-group.md-THEME_NAME-theme .md-checked:not([disabled]).md-primary .md-ink-ripple {  color: \'{{primary-color-0.26}}\'; }md-radio-group.md-THEME_NAME-theme .md-checked.md-primary .md-ink-ripple {  color: \'{{warn-color-0.26}}\'; }md-radio-group.md-THEME_NAME-theme.md-focused:not(:empty) .md-checked .md-container:before {  background-color: \'{{accent-color-0.26}}\'; }md-radio-group.md-THEME_NAME-theme.md-focused:not(:empty).md-primary .md-checked .md-container:before,md-radio-group.md-THEME_NAME-theme.md-focused:not(:empty) .md-checked.md-primary .md-container:before {  background-color: \'{{primary-color-0.26}}\'; }md-radio-group.md-THEME_NAME-theme.md-focused:not(:empty).md-warn .md-checked .md-container:before,md-radio-group.md-THEME_NAME-theme.md-focused:not(:empty) .md-checked.md-warn .md-container:before {  background-color: \'{{warn-color-0.26}}\'; }md-progress-linear.md-THEME_NAME-theme .md-container {  background-color: \'{{primary-100}}\'; }md-progress-linear.md-THEME_NAME-theme .md-bar {  background-color: \'{{primary-color}}\'; }md-progress-linear.md-THEME_NAME-theme.md-warn .md-container {  background-color: \'{{warn-100}}\'; }md-progress-linear.md-THEME_NAME-theme.md-warn .md-bar {  background-color: \'{{warn-color}}\'; }md-progress-linear.md-THEME_NAME-theme.md-accent .md-container {  background-color: \'{{accent-100}}\'; }md-progress-linear.md-THEME_NAME-theme.md-accent .md-bar {  background-color: \'{{accent-color}}\'; }md-progress-linear.md-THEME_NAME-theme[md-mode=buffer].md-warn .md-bar1 {  background-color: \'{{warn-100}}\'; }md-progress-linear.md-THEME_NAME-theme[md-mode=buffer].md-warn .md-dashed:before {  background: radial-gradient("{{warn-100}}" 0%, "{{warn-100}}" 16%, transparent 42%); }md-progress-linear.md-THEME_NAME-theme[md-mode=buffer].md-accent .md-bar1 {  background-color: \'{{accent-100}}\'; }md-progress-linear.md-THEME_NAME-theme[md-mode=buffer].md-accent .md-dashed:before {  background: radial-gradient("{{accent-100}}" 0%, "{{accent-100}}" 16%, transparent 42%); }md-select.md-THEME_NAME-theme[disabled] .md-select-value {  border-bottom-color: transparent;  background-image: linear-gradient(to right, "{{foreground-3}}" 0%, "{{foreground-3}}" 33%, transparent 0%);  background-image: -ms-linear-gradient(left, transparent 0%, "{{foreground-3}}" 100%); }md-select.md-THEME_NAME-theme .md-select-value {  border-bottom-color: \'{{foreground-4}}\'; }  md-select.md-THEME_NAME-theme .md-select-value.md-select-placeholder {    color: \'{{foreground-3}}\'; }md-select.md-THEME_NAME-theme.ng-invalid.ng-dirty .md-select-value {  color: \'{{warn-A700}}\' !important;  border-bottom-color: \'{{warn-A700}}\' !important; }md-select.md-THEME_NAME-theme:not([disabled]):focus .md-select-value {  border-bottom-color: \'{{primary-color}}\';  color: \'{{ foreground-1 }}\'; }  md-select.md-THEME_NAME-theme:not([disabled]):focus .md-select-value.md-select-placeholder {    color: \'{{ foreground-1 }}\'; }md-select.md-THEME_NAME-theme:not([disabled]):focus.md-accent .md-select-value {  border-bottom-color: \'{{accent-color}}\'; }md-select.md-THEME_NAME-theme:not([disabled]):focus.md-warn .md-select-value {  border-bottom-color: \'{{warn-color}}\'; }md-select.md-THEME_NAME-theme[disabled] .md-select-value {  color: \'{{foreground-3}}\'; }  md-select.md-THEME_NAME-theme[disabled] .md-select-value.md-select-placeholder {    color: \'{{foreground-3}}\'; }md-select-menu.md-THEME_NAME-theme md-option[disabled] {  color: \'{{foreground-3}}\'; }md-select-menu.md-THEME_NAME-theme md-optgroup {  color: \'{{foreground-2}}\'; }  md-select-menu.md-THEME_NAME-theme md-optgroup md-option {    color: \'{{foreground-1}}\'; }md-select-menu.md-THEME_NAME-theme md-option[selected] {  color: \'{{primary-500}}\'; }  md-select-menu.md-THEME_NAME-theme md-option[selected]:focus {    color: \'{{primary-600}}\'; }  md-select-menu.md-THEME_NAME-theme md-option[selected].md-accent {    color: \'{{accent-500}}\'; }    md-select-menu.md-THEME_NAME-theme md-option[selected].md-accent:focus {      color: \'{{accent-600}}\'; }md-select-menu.md-THEME_NAME-theme md-option:focus:not([disabled]):not([selected]) {  background: \'{{background-200}}\'; }md-slider.md-THEME_NAME-theme .md-track {  background-color: \'{{foreground-3}}\'; }md-slider.md-THEME_NAME-theme .md-track-ticks {  background-color: \'{{foreground-4}}\'; }md-slider.md-THEME_NAME-theme .md-focus-thumb {  background-color: \'{{foreground-2}}\'; }md-slider.md-THEME_NAME-theme .md-focus-ring {  background-color: \'{{accent-color}}\'; }md-slider.md-THEME_NAME-theme .md-disabled-thumb {  border-color: \'{{background-color}}\'; }md-slider.md-THEME_NAME-theme.md-min .md-thumb:after {  background-color: \'{{background-color}}\'; }md-slider.md-THEME_NAME-theme .md-track.md-track-fill {  background-color: \'{{accent-color}}\'; }md-slider.md-THEME_NAME-theme .md-thumb:after {  border-color: \'{{accent-color}}\';  background-color: \'{{accent-color}}\'; }md-slider.md-THEME_NAME-theme .md-sign {  background-color: \'{{accent-color}}\'; }  md-slider.md-THEME_NAME-theme .md-sign:after {    border-top-color: \'{{accent-color}}\'; }md-slider.md-THEME_NAME-theme .md-thumb-text {  color: \'{{accent-contrast}}\'; }md-slider.md-THEME_NAME-theme.md-warn .md-focus-ring {  background-color: \'{{warn-color}}\'; }md-slider.md-THEME_NAME-theme.md-warn .md-track.md-track-fill {  background-color: \'{{warn-color}}\'; }md-slider.md-THEME_NAME-theme.md-warn .md-thumb:after {  border-color: \'{{warn-color}}\';  background-color: \'{{warn-color}}\'; }md-slider.md-THEME_NAME-theme.md-warn .md-sign {  background-color: \'{{warn-color}}\'; }  md-slider.md-THEME_NAME-theme.md-warn .md-sign:after {    border-top-color: \'{{warn-color}}\'; }md-slider.md-THEME_NAME-theme.md-warn .md-thumb-text {  color: \'{{warn-contrast}}\'; }md-slider.md-THEME_NAME-theme.md-primary .md-focus-ring {  background-color: \'{{primary-color}}\'; }md-slider.md-THEME_NAME-theme.md-primary .md-track.md-track-fill {  background-color: \'{{primary-color}}\'; }md-slider.md-THEME_NAME-theme.md-primary .md-thumb:after {  border-color: \'{{primary-color}}\';  background-color: \'{{primary-color}}\'; }md-slider.md-THEME_NAME-theme.md-primary .md-sign {  background-color: \'{{primary-color}}\'; }  md-slider.md-THEME_NAME-theme.md-primary .md-sign:after {    border-top-color: \'{{primary-color}}\'; }md-slider.md-THEME_NAME-theme.md-primary .md-thumb-text {  color: \'{{primary-contrast}}\'; }md-slider.md-THEME_NAME-theme[disabled] .md-thumb:after {  border-color: \'{{foreground-3}}\'; }md-slider.md-THEME_NAME-theme[disabled]:not(.md-min) .md-thumb:after {  background-color: \'{{foreground-3}}\'; }md-sidenav.md-THEME_NAME-theme {  background-color: \'{{background-color}}\'; }.md-subheader.md-THEME_NAME-theme {  color: \'{{ foreground-2-0.23 }}\';  background-color: \'{{background-color}}\'; }  .md-subheader.md-THEME_NAME-theme.md-primary {    color: \'{{primary-color}}\'; }  .md-subheader.md-THEME_NAME-theme.md-accent {    color: \'{{accent-color}}\'; }  .md-subheader.md-THEME_NAME-theme.md-warn {    color: \'{{warn-color}}\'; }md-switch.md-THEME_NAME-theme .md-ink-ripple {  color: \'{{background-500}}\'; }md-switch.md-THEME_NAME-theme .md-thumb {  background-color: \'{{background-50}}\'; }md-switch.md-THEME_NAME-theme .md-bar {  background-color: \'{{background-500}}\'; }md-switch.md-THEME_NAME-theme.md-checked .md-ink-ripple {  color: \'{{accent-color}}\'; }md-switch.md-THEME_NAME-theme.md-checked .md-thumb {  background-color: \'{{accent-color}}\'; }md-switch.md-THEME_NAME-theme.md-checked .md-bar {  background-color: \'{{accent-color-0.5}}\'; }md-switch.md-THEME_NAME-theme.md-checked.md-focused .md-thumb:before {  background-color: \'{{accent-color-0.26}}\'; }md-switch.md-THEME_NAME-theme.md-checked.md-primary .md-ink-ripple {  color: \'{{primary-color}}\'; }md-switch.md-THEME_NAME-theme.md-checked.md-primary .md-thumb {  background-color: \'{{primary-color}}\'; }md-switch.md-THEME_NAME-theme.md-checked.md-primary .md-bar {  background-color: \'{{primary-color-0.5}}\'; }md-switch.md-THEME_NAME-theme.md-checked.md-primary.md-focused .md-thumb:before {  background-color: \'{{primary-color-0.26}}\'; }md-switch.md-THEME_NAME-theme.md-checked.md-warn .md-ink-ripple {  color: \'{{warn-color}}\'; }md-switch.md-THEME_NAME-theme.md-checked.md-warn .md-thumb {  background-color: \'{{warn-color}}\'; }md-switch.md-THEME_NAME-theme.md-checked.md-warn .md-bar {  background-color: \'{{warn-color-0.5}}\'; }md-switch.md-THEME_NAME-theme.md-checked.md-warn.md-focused .md-thumb:before {  background-color: \'{{warn-color-0.26}}\'; }md-switch.md-THEME_NAME-theme[disabled] .md-thumb {  background-color: \'{{background-400}}\'; }md-switch.md-THEME_NAME-theme[disabled] .md-bar {  background-color: \'{{foreground-4}}\'; }md-tabs.md-THEME_NAME-theme md-tabs-wrapper {  background-color: transparent;  border-color: \'{{foreground-4}}\'; }md-tabs.md-THEME_NAME-theme .md-paginator md-icon {  color: \'{{primary-color}}\'; }md-tabs.md-THEME_NAME-theme md-ink-bar {  color: \'{{accent-color}}\';  background: \'{{accent-color}}\'; }md-tabs.md-THEME_NAME-theme .md-tab {  color: \'{{foreground-2}}\'; }  md-tabs.md-THEME_NAME-theme .md-tab[disabled], md-tabs.md-THEME_NAME-theme .md-tab[disabled] md-icon {    color: \'{{foreground-3}}\'; }  md-tabs.md-THEME_NAME-theme .md-tab.md-active, md-tabs.md-THEME_NAME-theme .md-tab.md-active md-icon, md-tabs.md-THEME_NAME-theme .md-tab.md-focused, md-tabs.md-THEME_NAME-theme .md-tab.md-focused md-icon {    color: \'{{primary-color}}\'; }  md-tabs.md-THEME_NAME-theme .md-tab.md-focused {    background: \'{{primary-color-0.1}}\'; }  md-tabs.md-THEME_NAME-theme .md-tab .md-ripple-container {    color: \'{{accent-100}}\'; }md-tabs.md-THEME_NAME-theme.md-accent > md-tabs-wrapper {  background-color: \'{{accent-color}}\'; }  md-tabs.md-THEME_NAME-theme.md-accent > md-tabs-wrapper > md-tabs-canvas > md-pagination-wrapper > md-tab-item:not([disabled]) {    color: \'{{accent-100}}\'; }    md-tabs.md-THEME_NAME-theme.md-accent > md-tabs-wrapper > md-tabs-canvas > md-pagination-wrapper > md-tab-item:not([disabled]).md-active, md-tabs.md-THEME_NAME-theme.md-accent > md-tabs-wrapper > md-tabs-canvas > md-pagination-wrapper > md-tab-item:not([disabled]).md-active md-icon, md-tabs.md-THEME_NAME-theme.md-accent > md-tabs-wrapper > md-tabs-canvas > md-pagination-wrapper > md-tab-item:not([disabled]).md-focused, md-tabs.md-THEME_NAME-theme.md-accent > md-tabs-wrapper > md-tabs-canvas > md-pagination-wrapper > md-tab-item:not([disabled]).md-focused md-icon {      color: \'{{accent-contrast}}\'; }    md-tabs.md-THEME_NAME-theme.md-accent > md-tabs-wrapper > md-tabs-canvas > md-pagination-wrapper > md-tab-item:not([disabled]).md-focused {      background: \'{{accent-contrast-0.1}}\'; }  md-tabs.md-THEME_NAME-theme.md-accent > md-tabs-wrapper > md-tabs-canvas > md-pagination-wrapper > md-ink-bar {    color: \'{{primary-600-1}}\';    background: \'{{primary-600-1}}\'; }md-tabs.md-THEME_NAME-theme.md-primary > md-tabs-wrapper {  background-color: \'{{primary-color}}\'; }  md-tabs.md-THEME_NAME-theme.md-primary > md-tabs-wrapper > md-tabs-canvas > md-pagination-wrapper > md-tab-item:not([disabled]) {    color: \'{{primary-100}}\'; }    md-tabs.md-THEME_NAME-theme.md-primary > md-tabs-wrapper > md-tabs-canvas > md-pagination-wrapper > md-tab-item:not([disabled]).md-active, md-tabs.md-THEME_NAME-theme.md-primary > md-tabs-wrapper > md-tabs-canvas > md-pagination-wrapper > md-tab-item:not([disabled]).md-active md-icon, md-tabs.md-THEME_NAME-theme.md-primary > md-tabs-wrapper > md-tabs-canvas > md-pagination-wrapper > md-tab-item:not([disabled]).md-focused, md-tabs.md-THEME_NAME-theme.md-primary > md-tabs-wrapper > md-tabs-canvas > md-pagination-wrapper > md-tab-item:not([disabled]).md-focused md-icon {      color: \'{{primary-contrast}}\'; }    md-tabs.md-THEME_NAME-theme.md-primary > md-tabs-wrapper > md-tabs-canvas > md-pagination-wrapper > md-tab-item:not([disabled]).md-focused {      background: \'{{primary-contrast-0.1}}\'; }md-tabs.md-THEME_NAME-theme.md-warn > md-tabs-wrapper {  background-color: \'{{warn-color}}\'; }  md-tabs.md-THEME_NAME-theme.md-warn > md-tabs-wrapper > md-tabs-canvas > md-pagination-wrapper > md-tab-item:not([disabled]) {    color: \'{{warn-100}}\'; }    md-tabs.md-THEME_NAME-theme.md-warn > md-tabs-wrapper > md-tabs-canvas > md-pagination-wrapper > md-tab-item:not([disabled]).md-active, md-tabs.md-THEME_NAME-theme.md-warn > md-tabs-wrapper > md-tabs-canvas > md-pagination-wrapper > md-tab-item:not([disabled]).md-active md-icon, md-tabs.md-THEME_NAME-theme.md-warn > md-tabs-wrapper > md-tabs-canvas > md-pagination-wrapper > md-tab-item:not([disabled]).md-focused, md-tabs.md-THEME_NAME-theme.md-warn > md-tabs-wrapper > md-tabs-canvas > md-pagination-wrapper > md-tab-item:not([disabled]).md-focused md-icon {      color: \'{{warn-contrast}}\'; }    md-tabs.md-THEME_NAME-theme.md-warn > md-tabs-wrapper > md-tabs-canvas > md-pagination-wrapper > md-tab-item:not([disabled]).md-focused {      background: \'{{warn-contrast-0.1}}\'; }md-toolbar > md-tabs.md-THEME_NAME-theme > md-tabs-wrapper {  background-color: \'{{primary-color}}\'; }  md-toolbar > md-tabs.md-THEME_NAME-theme > md-tabs-wrapper > md-tabs-canvas > md-pagination-wrapper > md-tab-item:not([disabled]) {    color: \'{{primary-100}}\'; }    md-toolbar > md-tabs.md-THEME_NAME-theme > md-tabs-wrapper > md-tabs-canvas > md-pagination-wrapper > md-tab-item:not([disabled]).md-active, md-toolbar > md-tabs.md-THEME_NAME-theme > md-tabs-wrapper > md-tabs-canvas > md-pagination-wrapper > md-tab-item:not([disabled]).md-active md-icon, md-toolbar > md-tabs.md-THEME_NAME-theme > md-tabs-wrapper > md-tabs-canvas > md-pagination-wrapper > md-tab-item:not([disabled]).md-focused, md-toolbar > md-tabs.md-THEME_NAME-theme > md-tabs-wrapper > md-tabs-canvas > md-pagination-wrapper > md-tab-item:not([disabled]).md-focused md-icon {      color: \'{{primary-contrast}}\'; }    md-toolbar > md-tabs.md-THEME_NAME-theme > md-tabs-wrapper > md-tabs-canvas > md-pagination-wrapper > md-tab-item:not([disabled]).md-focused {      background: \'{{primary-contrast-0.1}}\'; }md-toolbar.md-accent > md-tabs.md-THEME_NAME-theme > md-tabs-wrapper {  background-color: \'{{accent-color}}\'; }  md-toolbar.md-accent > md-tabs.md-THEME_NAME-theme > md-tabs-wrapper > md-tabs-canvas > md-pagination-wrapper > md-tab-item:not([disabled]) {    color: \'{{accent-100}}\'; }    md-toolbar.md-accent > md-tabs.md-THEME_NAME-theme > md-tabs-wrapper > md-tabs-canvas > md-pagination-wrapper > md-tab-item:not([disabled]).md-active, md-toolbar.md-accent > md-tabs.md-THEME_NAME-theme > md-tabs-wrapper > md-tabs-canvas > md-pagination-wrapper > md-tab-item:not([disabled]).md-active md-icon, md-toolbar.md-accent > md-tabs.md-THEME_NAME-theme > md-tabs-wrapper > md-tabs-canvas > md-pagination-wrapper > md-tab-item:not([disabled]).md-focused, md-toolbar.md-accent > md-tabs.md-THEME_NAME-theme > md-tabs-wrapper > md-tabs-canvas > md-pagination-wrapper > md-tab-item:not([disabled]).md-focused md-icon {      color: \'{{accent-contrast}}\'; }    md-toolbar.md-accent > md-tabs.md-THEME_NAME-theme > md-tabs-wrapper > md-tabs-canvas > md-pagination-wrapper > md-tab-item:not([disabled]).md-focused {      background: \'{{accent-contrast-0.1}}\'; }  md-toolbar.md-accent > md-tabs.md-THEME_NAME-theme > md-tabs-wrapper > md-tabs-canvas > md-pagination-wrapper > md-ink-bar {    color: \'{{primary-600-1}}\';    background: \'{{primary-600-1}}\'; }md-toolbar.md-warn > md-tabs.md-THEME_NAME-theme > md-tabs-wrapper {  background-color: \'{{warn-color}}\'; }  md-toolbar.md-warn > md-tabs.md-THEME_NAME-theme > md-tabs-wrapper > md-tabs-canvas > md-pagination-wrapper > md-tab-item:not([disabled]) {    color: \'{{warn-100}}\'; }    md-toolbar.md-warn > md-tabs.md-THEME_NAME-theme > md-tabs-wrapper > md-tabs-canvas > md-pagination-wrapper > md-tab-item:not([disabled]).md-active, md-toolbar.md-warn > md-tabs.md-THEME_NAME-theme > md-tabs-wrapper > md-tabs-canvas > md-pagination-wrapper > md-tab-item:not([disabled]).md-active md-icon, md-toolbar.md-warn > md-tabs.md-THEME_NAME-theme > md-tabs-wrapper > md-tabs-canvas > md-pagination-wrapper > md-tab-item:not([disabled]).md-focused, md-toolbar.md-warn > md-tabs.md-THEME_NAME-theme > md-tabs-wrapper > md-tabs-canvas > md-pagination-wrapper > md-tab-item:not([disabled]).md-focused md-icon {      color: \'{{warn-contrast}}\'; }    md-toolbar.md-warn > md-tabs.md-THEME_NAME-theme > md-tabs-wrapper > md-tabs-canvas > md-pagination-wrapper > md-tab-item:not([disabled]).md-focused {      background: \'{{warn-contrast-0.1}}\'; }md-toast.md-THEME_NAME-theme .md-toast-content {  background-color: #323232;  color: \'{{background-50}}\'; }  md-toast.md-THEME_NAME-theme .md-toast-content .md-button {    color: \'{{background-50}}\'; }    md-toast.md-THEME_NAME-theme .md-toast-content .md-button.md-highlight {      color: \'{{primary-A200}}\'; }      md-toast.md-THEME_NAME-theme .md-toast-content .md-button.md-highlight.md-accent {        color: \'{{accent-A200}}\'; }      md-toast.md-THEME_NAME-theme .md-toast-content .md-button.md-highlight.md-warn {        color: \'{{warn-A200}}\'; }md-toolbar.md-THEME_NAME-theme:not(.md-menu-toolbar) {  background-color: \'{{primary-color}}\';  color: \'{{primary-contrast}}\'; }  md-toolbar.md-THEME_NAME-theme:not(.md-menu-toolbar) md-icon {    color: \'{{primary-contrast}}\'; }  md-toolbar.md-THEME_NAME-theme:not(.md-menu-toolbar) .md-button:not(.md-raised) {    color: \'{{primary-contrast}}\'; }  md-toolbar.md-THEME_NAME-theme:not(.md-menu-toolbar).md-accent {    background-color: \'{{accent-color}}\';    color: \'{{accent-contrast}}\'; }  md-toolbar.md-THEME_NAME-theme:not(.md-menu-toolbar).md-warn {    background-color: \'{{warn-color}}\';    color: \'{{warn-contrast}}\'; }md-tooltip.md-THEME_NAME-theme {  color: \'{{background-A100}}\'; }  md-tooltip.md-THEME_NAME-theme .md-content {    background-color: \'{{foreground-2}}\'; }');
  }());
}(window, window.angular));
;
window.ngMaterial = { version: { full: '1.0.0' } };
/**
 * @license AngularJS v1.2.32
 * (c) 2010-2014 Google, Inc. http://angularjs.org
 * License: MIT
 */
(function (window, angular, undefined) {
  'use strict';
  /**
 * @ngdoc module
 * @name ngTouch
 * @description
 *
 * # ngTouch
 *
 * The `ngTouch` module provides touch events and other helpers for touch-enabled devices.
 * The implementation is based on jQuery Mobile touch event handling
 * ([jquerymobile.com](http://jquerymobile.com/)).
 *
 *
 * See {@link ngTouch.$swipe `$swipe`} for usage.
 *
 * <div doc-module-components="ngTouch"></div>
 *
 */
  // define ngTouch module
  /* global -ngTouch */
  var ngTouch = angular.module('ngTouch', []);
  /* global ngTouch: false */
  /**
     * @ngdoc service
     * @name $swipe
     *
     * @description
     * The `$swipe` service is a service that abstracts the messier details of hold-and-drag swipe
     * behavior, to make implementing swipe-related directives more convenient.
     *
     * Requires the {@link ngTouch `ngTouch`} module to be installed.
     *
     * `$swipe` is used by the `ngSwipeLeft` and `ngSwipeRight` directives in `ngTouch`, and by
     * `ngCarousel` in a separate component.
     *
     * # Usage
     * The `$swipe` service is an object with a single method: `bind`. `bind` takes an element
     * which is to be watched for swipes, and an object with four handler functions. See the
     * documentation for `bind` below.
     */
  ngTouch.factory('$swipe', [function () {
      // The total distance in any direction before we make the call on swipe vs. scroll.
      var MOVE_BUFFER_RADIUS = 10;
      function getCoordinates(event) {
        var touches = event.touches && event.touches.length ? event.touches : [event];
        var e = event.changedTouches && event.changedTouches[0] || event.originalEvent && event.originalEvent.changedTouches && event.originalEvent.changedTouches[0] || touches[0].originalEvent || touches[0];
        return {
          x: e.clientX,
          y: e.clientY
        };
      }
      return {
        bind: function (element, eventHandlers) {
          // Absolute total movement, used to control swipe vs. scroll.
          var totalX, totalY;
          // Coordinates of the start position.
          var startCoords;
          // Last event's position.
          var lastPos;
          // Whether a swipe is active.
          var active = false;
          element.on('touchstart mousedown', function (event) {
            startCoords = getCoordinates(event);
            active = true;
            totalX = 0;
            totalY = 0;
            lastPos = startCoords;
            eventHandlers['start'] && eventHandlers['start'](startCoords, event);
          });
          element.on('touchcancel', function (event) {
            active = false;
            eventHandlers['cancel'] && eventHandlers['cancel'](event);
          });
          element.on('touchmove mousemove', function (event) {
            if (!active)
              return;
            // Android will send a touchcancel if it thinks we're starting to scroll.
            // So when the total distance (+ or - or both) exceeds 10px in either direction,
            // we either:
            // - On totalX > totalY, we send preventDefault() and treat this as a swipe.
            // - On totalY > totalX, we let the browser handle it as a scroll.
            if (!startCoords)
              return;
            var coords = getCoordinates(event);
            totalX += Math.abs(coords.x - lastPos.x);
            totalY += Math.abs(coords.y - lastPos.y);
            lastPos = coords;
            if (totalX < MOVE_BUFFER_RADIUS && totalY < MOVE_BUFFER_RADIUS) {
              return;
            }
            // One of totalX or totalY has exceeded the buffer, so decide on swipe vs. scroll.
            if (totalY > totalX) {
              // Allow native scrolling to take over.
              active = false;
              eventHandlers['cancel'] && eventHandlers['cancel'](event);
              return;
            } else {
              // Prevent the browser from scrolling.
              event.preventDefault();
              eventHandlers['move'] && eventHandlers['move'](coords, event);
            }
          });
          element.on('touchend mouseup', function (event) {
            if (!active)
              return;
            active = false;
            eventHandlers['end'] && eventHandlers['end'](getCoordinates(event), event);
          });
        }
      };
    }]);
  /* global ngTouch: false */
  /**
 * @ngdoc directive
 * @name ngClick
 *
 * @description
 * A more powerful replacement for the default ngClick designed to be used on touchscreen
 * devices. Most mobile browsers wait about 300ms after a tap-and-release before sending
 * the click event. This version handles them immediately, and then prevents the
 * following click event from propagating.
 *
 * Requires the {@link ngTouch `ngTouch`} module to be installed.
 *
 * This directive can fall back to using an ordinary click event, and so works on desktop
 * browsers as well as mobile.
 *
 * This directive also sets the CSS class `ng-click-active` while the element is being held
 * down (by a mouse click or touch) so you can restyle the depressed element if you wish.
 *
 * @element ANY
 * @param {expression} ngClick {@link guide/expression Expression} to evaluate
 * upon tap. (Event object is available as `$event`)
 *
 * @example
    <example module="ngClickExample" deps="angular-touch.js">
      <file name="index.html">
        <button ng-click="count = count + 1" ng-init="count=0">
          Increment
        </button>
        count: {{ count }}
      </file>
      <file name="script.js">
        angular.module('ngClickExample', ['ngTouch']);
      </file>
    </example>
 */
  ngTouch.config([
    '$provide',
    function ($provide) {
      $provide.decorator('ngClickDirective', [
        '$delegate',
        function ($delegate) {
          // drop the default ngClick directive
          $delegate.shift();
          return $delegate;
        }
      ]);
    }
  ]);
  ngTouch.directive('ngClick', [
    '$parse',
    '$timeout',
    '$rootElement',
    function ($parse, $timeout, $rootElement) {
      var TAP_DURATION = 750;
      // Shorter than 750ms is a tap, longer is a taphold or drag.
      var MOVE_TOLERANCE = 12;
      // 12px seems to work in most mobile browsers.
      var PREVENT_DURATION = 2500;
      // 2.5 seconds maximum from preventGhostClick call to click
      var CLICKBUSTER_THRESHOLD = 25;
      // 25 pixels in any dimension is the limit for busting clicks.
      var ACTIVE_CLASS_NAME = 'ng-click-active';
      var lastPreventedTime;
      var touchCoordinates;
      var lastLabelClickCoordinates;
      // TAP EVENTS AND GHOST CLICKS
      //
      // Why tap events?
      // Mobile browsers detect a tap, then wait a moment (usually ~300ms) to see if you're
      // double-tapping, and then fire a click event.
      //
      // This delay sucks and makes mobile apps feel unresponsive.
      // So we detect touchstart, touchmove, touchcancel and touchend ourselves and determine when
      // the user has tapped on something.
      //
      // What happens when the browser then generates a click event?
      // The browser, of course, also detects the tap and fires a click after a delay. This results in
      // tapping/clicking twice. We do "clickbusting" to prevent it.
      //
      // How does it work?
      // We attach global touchstart and click handlers, that run during the capture (early) phase.
      // So the sequence for a tap is:
      // - global touchstart: Sets an "allowable region" at the point touched.
      // - element's touchstart: Starts a touch
      // (- touchmove or touchcancel ends the touch, no click follows)
      // - element's touchend: Determines if the tap is valid (didn't move too far away, didn't hold
      //   too long) and fires the user's tap handler. The touchend also calls preventGhostClick().
      // - preventGhostClick() removes the allowable region the global touchstart created.
      // - The browser generates a click event.
      // - The global click handler catches the click, and checks whether it was in an allowable region.
      //     - If preventGhostClick was called, the region will have been removed, the click is busted.
      //     - If the region is still there, the click proceeds normally. Therefore clicks on links and
      //       other elements without ngTap on them work normally.
      //
      // This is an ugly, terrible hack!
      // Yeah, tell me about it. The alternatives are using the slow click events, or making our users
      // deal with the ghost clicks, so I consider this the least of evils. Fortunately Angular
      // encapsulates this ugly logic away from the user.
      //
      // Why not just put click handlers on the element?
      // We do that too, just to be sure. If the tap event caused the DOM to change,
      // it is possible another element is now in that position. To take account for these possibly
      // distinct elements, the handlers are global and care only about coordinates.
      // Checks if the coordinates are close enough to be within the region.
      function hit(x1, y1, x2, y2) {
        return Math.abs(x1 - x2) < CLICKBUSTER_THRESHOLD && Math.abs(y1 - y2) < CLICKBUSTER_THRESHOLD;
      }
      // Checks a list of allowable regions against a click location.
      // Returns true if the click should be allowed.
      // Splices out the allowable region from the list after it has been used.
      function checkAllowableRegions(touchCoordinates, x, y) {
        for (var i = 0; i < touchCoordinates.length; i += 2) {
          if (hit(touchCoordinates[i], touchCoordinates[i + 1], x, y)) {
            touchCoordinates.splice(i, i + 2);
            return true;  // allowable region
          }
        }
        return false;  // No allowable region; bust it.
      }
      // Global click handler that prevents the click if it's in a bustable zone and preventGhostClick
      // was called recently.
      function onClick(event) {
        if (Date.now() - lastPreventedTime > PREVENT_DURATION) {
          return;  // Too old.
        }
        var touches = event.touches && event.touches.length ? event.touches : [event];
        var x = touches[0].clientX;
        var y = touches[0].clientY;
        // Work around desktop Webkit quirk where clicking a label will fire two clicks (on the label
        // and on the input element). Depending on the exact browser, this second click we don't want
        // to bust has either (0,0), negative coordinates, or coordinates equal to triggering label
        // click event
        if (x < 1 && y < 1) {
          return;  // offscreen
        }
        if (lastLabelClickCoordinates && lastLabelClickCoordinates[0] === x && lastLabelClickCoordinates[1] === y) {
          return;  // input click triggered by label click
        }
        // reset label click coordinates on first subsequent click
        if (lastLabelClickCoordinates) {
          lastLabelClickCoordinates = null;
        }
        // remember label click coordinates to prevent click busting of trigger click event on input
        if (event.target.tagName.toLowerCase() === 'label') {
          lastLabelClickCoordinates = [
            x,
            y
          ];
        }
        // Look for an allowable region containing this click.
        // If we find one, that means it was created by touchstart and not removed by
        // preventGhostClick, so we don't bust it.
        if (checkAllowableRegions(touchCoordinates, x, y)) {
          return;
        }
        // If we didn't find an allowable region, bust the click.
        event.stopPropagation();
        event.preventDefault();
        // Blur focused form elements
        event.target && event.target.blur();
      }
      // Global touchstart handler that creates an allowable region for a click event.
      // This allowable region can be removed by preventGhostClick if we want to bust it.
      function onTouchStart(event) {
        var touches = event.touches && event.touches.length ? event.touches : [event];
        var x = touches[0].clientX;
        var y = touches[0].clientY;
        touchCoordinates.push(x, y);
        $timeout(function () {
          // Remove the allowable region.
          for (var i = 0; i < touchCoordinates.length; i += 2) {
            if (touchCoordinates[i] == x && touchCoordinates[i + 1] == y) {
              touchCoordinates.splice(i, i + 2);
              return;
            }
          }
        }, PREVENT_DURATION, false);
      }
      // On the first call, attaches some event handlers. Then whenever it gets called, it creates a
      // zone around the touchstart where clicks will get busted.
      function preventGhostClick(x, y) {
        if (!touchCoordinates) {
          $rootElement[0].addEventListener('click', onClick, true);
          $rootElement[0].addEventListener('touchstart', onTouchStart, true);
          touchCoordinates = [];
        }
        lastPreventedTime = Date.now();
        checkAllowableRegions(touchCoordinates, x, y);
      }
      // Actual linking function.
      return function (scope, element, attr) {
        var clickHandler = $parse(attr.ngClick), tapping = false, tapElement,
          // Used to blur the element after a tap.
          startTime,
          // Used to check if the tap was held too long.
          touchStartX, touchStartY;
        function resetState() {
          tapping = false;
          element.removeClass(ACTIVE_CLASS_NAME);
        }
        element.on('touchstart', function (event) {
          tapping = true;
          tapElement = event.target ? event.target : event.srcElement;
          // IE uses srcElement.
          // Hack for Safari, which can target text nodes instead of containers.
          if (tapElement.nodeType == 3) {
            tapElement = tapElement.parentNode;
          }
          element.addClass(ACTIVE_CLASS_NAME);
          startTime = Date.now();
          var touches = event.touches && event.touches.length ? event.touches : [event];
          var e = touches[0].originalEvent || touches[0];
          touchStartX = e.clientX;
          touchStartY = e.clientY;
        });
        element.on('touchmove', function (event) {
          resetState();
        });
        element.on('touchcancel', function (event) {
          resetState();
        });
        element.on('touchend', function (event) {
          var diff = Date.now() - startTime;
          var touches = event.changedTouches && event.changedTouches.length ? event.changedTouches : event.touches && event.touches.length ? event.touches : [event];
          var e = touches[0].originalEvent || touches[0];
          var x = e.clientX;
          var y = e.clientY;
          var dist = Math.sqrt(Math.pow(x - touchStartX, 2) + Math.pow(y - touchStartY, 2));
          if (tapping && diff < TAP_DURATION && dist < MOVE_TOLERANCE) {
            // Call preventGhostClick so the clickbuster will catch the corresponding click.
            preventGhostClick(x, y);
            // Blur the focused element (the button, probably) before firing the callback.
            // This doesn't work perfectly on Android Chrome, but seems to work elsewhere.
            // I couldn't get anything to work reliably on Android Chrome.
            if (tapElement) {
              tapElement.blur();
            }
            if (!angular.isDefined(attr.disabled) || attr.disabled === false) {
              element.triggerHandler('click', [event]);
            }
          }
          resetState();
        });
        // Hack for iOS Safari's benefit. It goes searching for onclick handlers and is liable to click
        // something else nearby.
        element.onclick = function (event) {
        };
        // Actual click handler.
        // There are three different kinds of clicks, only two of which reach this point.
        // - On desktop browsers without touch events, their clicks will always come here.
        // - On mobile browsers, the simulated "fast" click will call this.
        // - But the browser's follow-up slow click will be "busted" before it reaches this handler.
        // Therefore it's safe to use this directive on both mobile and desktop.
        element.on('click', function (event, touchend) {
          scope.$apply(function () {
            clickHandler(scope, { $event: touchend || event });
          });
        });
        element.on('mousedown', function (event) {
          element.addClass(ACTIVE_CLASS_NAME);
        });
        element.on('mousemove mouseup', function (event) {
          element.removeClass(ACTIVE_CLASS_NAME);
        });
      };
    }
  ]);
  /* global ngTouch: false */
  /**
 * @ngdoc directive
 * @name ngSwipeLeft
 *
 * @description
 * Specify custom behavior when an element is swiped to the left on a touchscreen device.
 * A leftward swipe is a quick, right-to-left slide of the finger.
 * Though ngSwipeLeft is designed for touch-based devices, it will work with a mouse click and drag
 * too.
 *
 * Requires the {@link ngTouch `ngTouch`} module to be installed.
 *
 * @element ANY
 * @param {expression} ngSwipeLeft {@link guide/expression Expression} to evaluate
 * upon left swipe. (Event object is available as `$event`)
 *
 * @example
    <example module="ngSwipeLeftExample" deps="angular-touch.js">
      <file name="index.html">
        <div ng-show="!showActions" ng-swipe-left="showActions = true">
          Some list content, like an email in the inbox
        </div>
        <div ng-show="showActions" ng-swipe-right="showActions = false">
          <button ng-click="reply()">Reply</button>
          <button ng-click="delete()">Delete</button>
        </div>
      </file>
      <file name="script.js">
        angular.module('ngSwipeLeftExample', ['ngTouch']);
      </file>
    </example>
 */
  /**
 * @ngdoc directive
 * @name ngSwipeRight
 *
 * @description
 * Specify custom behavior when an element is swiped to the right on a touchscreen device.
 * A rightward swipe is a quick, left-to-right slide of the finger.
 * Though ngSwipeRight is designed for touch-based devices, it will work with a mouse click and drag
 * too.
 *
 * Requires the {@link ngTouch `ngTouch`} module to be installed.
 *
 * @element ANY
 * @param {expression} ngSwipeRight {@link guide/expression Expression} to evaluate
 * upon right swipe. (Event object is available as `$event`)
 *
 * @example
    <example module="ngSwipeRightExample" deps="angular-touch.js">
      <file name="index.html">
        <div ng-show="!showActions" ng-swipe-left="showActions = true">
          Some list content, like an email in the inbox
        </div>
        <div ng-show="showActions" ng-swipe-right="showActions = false">
          <button ng-click="reply()">Reply</button>
          <button ng-click="delete()">Delete</button>
        </div>
      </file>
      <file name="script.js">
        angular.module('ngSwipeRightExample', ['ngTouch']);
      </file>
    </example>
 */
  function makeSwipeDirective(directiveName, direction, eventName) {
    ngTouch.directive(directiveName, [
      '$parse',
      '$swipe',
      function ($parse, $swipe) {
        // The maximum vertical delta for a swipe should be less than 75px.
        var MAX_VERTICAL_DISTANCE = 75;
        // Vertical distance should not be more than a fraction of the horizontal distance.
        var MAX_VERTICAL_RATIO = 0.3;
        // At least a 30px lateral motion is necessary for a swipe.
        var MIN_HORIZONTAL_DISTANCE = 30;
        return function (scope, element, attr) {
          var swipeHandler = $parse(attr[directiveName]);
          var startCoords, valid;
          function validSwipe(coords) {
            // Check that it's within the coordinates.
            // Absolute vertical distance must be within tolerances.
            // Horizontal distance, we take the current X - the starting X.
            // This is negative for leftward swipes and positive for rightward swipes.
            // After multiplying by the direction (-1 for left, +1 for right), legal swipes
            // (ie. same direction as the directive wants) will have a positive delta and
            // illegal ones a negative delta.
            // Therefore this delta must be positive, and larger than the minimum.
            if (!startCoords)
              return false;
            var deltaY = Math.abs(coords.y - startCoords.y);
            var deltaX = (coords.x - startCoords.x) * direction;
            return valid && deltaY < MAX_VERTICAL_DISTANCE && deltaX > 0 && deltaX > MIN_HORIZONTAL_DISTANCE && deltaY / deltaX < MAX_VERTICAL_RATIO;
          }
          $swipe.bind(element, {
            'start': function (coords, event) {
              startCoords = coords;
              valid = true;
            },
            'cancel': function (event) {
              valid = false;
            },
            'end': function (coords, event) {
              if (validSwipe(coords)) {
                scope.$apply(function () {
                  element.triggerHandler(eventName);
                  swipeHandler(scope, { $event: event });
                });
              }
            }
          });
        };
      }
    ]);
  }
  // Left is negative X-coordinate, right is positive.
  makeSwipeDirective('ngSwipeLeft', -1, 'swipeleft');
  makeSwipeDirective('ngSwipeRight', 1, 'swiperight');
}(window, window.angular));
/*
 * angular-ui-bootstrap
 * http://angular-ui.github.io/bootstrap/

 * Version: 0.14.0 - 2015-10-08
 * License: MIT
 */
angular.module('ui.bootstrap', [
  'ui.bootstrap.collapse',
  'ui.bootstrap.accordion',
  'ui.bootstrap.alert',
  'ui.bootstrap.buttons',
  'ui.bootstrap.carousel',
  'ui.bootstrap.dateparser',
  'ui.bootstrap.position',
  'ui.bootstrap.datepicker',
  'ui.bootstrap.dropdown',
  'ui.bootstrap.stackedMap',
  'ui.bootstrap.modal',
  'ui.bootstrap.pagination',
  'ui.bootstrap.tooltip',
  'ui.bootstrap.popover',
  'ui.bootstrap.progressbar',
  'ui.bootstrap.rating',
  'ui.bootstrap.tabs',
  'ui.bootstrap.timepicker',
  'ui.bootstrap.typeahead'
]);
angular.module('ui.bootstrap.collapse', []).directive('uibCollapse', [
  '$animate',
  '$injector',
  function ($animate, $injector) {
    var $animateCss = $injector.has('$animateCss') ? $injector.get('$animateCss') : null;
    return {
      link: function (scope, element, attrs) {
        function expand() {
          element.removeClass('collapse').addClass('collapsing').attr('aria-expanded', true).attr('aria-hidden', false);
          if ($animateCss) {
            $animateCss(element, {
              addClass: 'in',
              easing: 'ease',
              to: { height: element[0].scrollHeight + 'px' }
            }).start().done(expandDone);
          } else {
            $animate.addClass(element, 'in', { to: { height: element[0].scrollHeight + 'px' } }).then(expandDone);
          }
        }
        function expandDone() {
          element.removeClass('collapsing').addClass('collapse').css({ height: 'auto' });
        }
        function collapse() {
          if (!element.hasClass('collapse') && !element.hasClass('in')) {
            return collapseDone();
          }
          element.css({ height: element[0].scrollHeight + 'px' }).removeClass('collapse').addClass('collapsing').attr('aria-expanded', false).attr('aria-hidden', true);
          if ($animateCss) {
            $animateCss(element, {
              removeClass: 'in',
              to: { height: '0' }
            }).start().done(collapseDone);
          } else {
            $animate.removeClass(element, 'in', { to: { height: '0' } }).then(collapseDone);
          }
        }
        function collapseDone() {
          element.css({ height: '0' });
          // Required so that collapse works when animation is disabled
          element.removeClass('collapsing').addClass('collapse');
        }
        scope.$watch(attrs.uibCollapse, function (shouldCollapse) {
          if (shouldCollapse) {
            collapse();
          } else {
            expand();
          }
        });
      }
    };
  }
]);
/* Deprecated collapse below */
angular.module('ui.bootstrap.collapse').value('$collapseSuppressWarning', false).directive('collapse', [
  '$animate',
  '$injector',
  '$log',
  '$collapseSuppressWarning',
  function ($animate, $injector, $log, $collapseSuppressWarning) {
    var $animateCss = $injector.has('$animateCss') ? $injector.get('$animateCss') : null;
    return {
      link: function (scope, element, attrs) {
        if (!$collapseSuppressWarning) {
          $log.warn('collapse is now deprecated. Use uib-collapse instead.');
        }
        function expand() {
          element.removeClass('collapse').addClass('collapsing').attr('aria-expanded', true).attr('aria-hidden', false);
          if ($animateCss) {
            $animateCss(element, {
              addClass: 'in',
              easing: 'ease',
              to: { height: element[0].scrollHeight + 'px' }
            }).start().done(expandDone);
          } else {
            $animate.addClass(element, 'in', { to: { height: element[0].scrollHeight + 'px' } }).then(expandDone);
          }
        }
        function expandDone() {
          element.removeClass('collapsing').addClass('collapse').css({ height: 'auto' });
        }
        function collapse() {
          if (!element.hasClass('collapse') && !element.hasClass('in')) {
            return collapseDone();
          }
          element.css({ height: element[0].scrollHeight + 'px' }).removeClass('collapse').addClass('collapsing').attr('aria-expanded', false).attr('aria-hidden', true);
          if ($animateCss) {
            $animateCss(element, {
              removeClass: 'in',
              to: { height: '0' }
            }).start().done(collapseDone);
          } else {
            $animate.removeClass(element, 'in', { to: { height: '0' } }).then(collapseDone);
          }
        }
        function collapseDone() {
          element.css({ height: '0' });
          // Required so that collapse works when animation is disabled
          element.removeClass('collapsing').addClass('collapse');
        }
        scope.$watch(attrs.collapse, function (shouldCollapse) {
          if (shouldCollapse) {
            collapse();
          } else {
            expand();
          }
        });
      }
    };
  }
]);
angular.module('ui.bootstrap.accordion', ['ui.bootstrap.collapse']).constant('uibAccordionConfig', { closeOthers: true }).controller('UibAccordionController', [
  '$scope',
  '$attrs',
  'uibAccordionConfig',
  function ($scope, $attrs, accordionConfig) {
    // This array keeps track of the accordion groups
    this.groups = [];
    // Ensure that all the groups in this accordion are closed, unless close-others explicitly says not to
    this.closeOthers = function (openGroup) {
      var closeOthers = angular.isDefined($attrs.closeOthers) ? $scope.$eval($attrs.closeOthers) : accordionConfig.closeOthers;
      if (closeOthers) {
        angular.forEach(this.groups, function (group) {
          if (group !== openGroup) {
            group.isOpen = false;
          }
        });
      }
    };
    // This is called from the accordion-group directive to add itself to the accordion
    this.addGroup = function (groupScope) {
      var that = this;
      this.groups.push(groupScope);
      groupScope.$on('$destroy', function (event) {
        that.removeGroup(groupScope);
      });
    };
    // This is called from the accordion-group directive when to remove itself
    this.removeGroup = function (group) {
      var index = this.groups.indexOf(group);
      if (index !== -1) {
        this.groups.splice(index, 1);
      }
    };
  }
]).directive('uibAccordion', function () {
  return {
    controller: 'UibAccordionController',
    controllerAs: 'accordion',
    transclude: true,
    templateUrl: function (element, attrs) {
      return attrs.templateUrl || 'template/accordion/accordion.html';
    }
  };
}).directive('uibAccordionGroup', function () {
  return {
    require: '^uibAccordion',
    transclude: true,
    replace: true,
    templateUrl: function (element, attrs) {
      return attrs.templateUrl || 'template/accordion/accordion-group.html';
    },
    scope: {
      heading: '@',
      isOpen: '=?',
      isDisabled: '=?'
    },
    controller: function () {
      this.setHeading = function (element) {
        this.heading = element;
      };
    },
    link: function (scope, element, attrs, accordionCtrl) {
      accordionCtrl.addGroup(scope);
      scope.openClass = attrs.openClass || 'panel-open';
      scope.panelClass = attrs.panelClass;
      scope.$watch('isOpen', function (value) {
        element.toggleClass(scope.openClass, !!value);
        if (value) {
          accordionCtrl.closeOthers(scope);
        }
      });
      scope.toggleOpen = function ($event) {
        if (!scope.isDisabled) {
          if (!$event || $event.which === 32) {
            scope.isOpen = !scope.isOpen;
          }
        }
      };
    }
  };
}).directive('uibAccordionHeading', function () {
  return {
    transclude: true,
    template: '',
    replace: true,
    require: '^uibAccordionGroup',
    link: function (scope, element, attrs, accordionGroupCtrl, transclude) {
      // Pass the heading to the accordion-group controller
      // so that it can be transcluded into the right place in the template
      // [The second parameter to transclude causes the elements to be cloned so that they work in ng-repeat]
      accordionGroupCtrl.setHeading(transclude(scope, angular.noop));
    }
  };
}).directive('uibAccordionTransclude', function () {
  return {
    require: [
      '?^uibAccordionGroup',
      '?^accordionGroup'
    ],
    link: function (scope, element, attrs, controller) {
      controller = controller[0] ? controller[0] : controller[1];
      // Delete after we remove deprecation
      scope.$watch(function () {
        return controller[attrs.uibAccordionTransclude];
      }, function (heading) {
        if (heading) {
          element.find('span').html('');
          element.find('span').append(heading);
        }
      });
    }
  };
});
/* Deprecated accordion below */
angular.module('ui.bootstrap.accordion').value('$accordionSuppressWarning', false).controller('AccordionController', [
  '$scope',
  '$attrs',
  '$controller',
  '$log',
  '$accordionSuppressWarning',
  function ($scope, $attrs, $controller, $log, $accordionSuppressWarning) {
    if (!$accordionSuppressWarning) {
      $log.warn('AccordionController is now deprecated. Use UibAccordionController instead.');
    }
    return $controller('UibAccordionController', {
      $scope: $scope,
      $attrs: $attrs
    });
  }
]).directive('accordion', [
  '$log',
  '$accordionSuppressWarning',
  function ($log, $accordionSuppressWarning) {
    return {
      restrict: 'EA',
      controller: 'AccordionController',
      controllerAs: 'accordion',
      transclude: true,
      replace: false,
      templateUrl: function (element, attrs) {
        return attrs.templateUrl || 'template/accordion/accordion.html';
      },
      link: function () {
        if (!$accordionSuppressWarning) {
          $log.warn('accordion is now deprecated. Use uib-accordion instead.');
        }
      }
    };
  }
]).directive('accordionGroup', [
  '$log',
  '$accordionSuppressWarning',
  function ($log, $accordionSuppressWarning) {
    return {
      require: '^accordion',
      restrict: 'EA',
      transclude: true,
      replace: true,
      templateUrl: function (element, attrs) {
        return attrs.templateUrl || 'template/accordion/accordion-group.html';
      },
      scope: {
        heading: '@',
        isOpen: '=?',
        isDisabled: '=?'
      },
      controller: function () {
        this.setHeading = function (element) {
          this.heading = element;
        };
      },
      link: function (scope, element, attrs, accordionCtrl) {
        if (!$accordionSuppressWarning) {
          $log.warn('accordion-group is now deprecated. Use uib-accordion-group instead.');
        }
        accordionCtrl.addGroup(scope);
        scope.openClass = attrs.openClass || 'panel-open';
        scope.panelClass = attrs.panelClass;
        scope.$watch('isOpen', function (value) {
          element.toggleClass(scope.openClass, !!value);
          if (value) {
            accordionCtrl.closeOthers(scope);
          }
        });
        scope.toggleOpen = function ($event) {
          if (!scope.isDisabled) {
            if (!$event || $event.which === 32) {
              scope.isOpen = !scope.isOpen;
            }
          }
        };
      }
    };
  }
]).directive('accordionHeading', [
  '$log',
  '$accordionSuppressWarning',
  function ($log, $accordionSuppressWarning) {
    return {
      restrict: 'EA',
      transclude: true,
      template: '',
      replace: true,
      require: '^accordionGroup',
      link: function (scope, element, attr, accordionGroupCtrl, transclude) {
        if (!$accordionSuppressWarning) {
          $log.warn('accordion-heading is now deprecated. Use uib-accordion-heading instead.');
        }
        // Pass the heading to the accordion-group controller
        // so that it can be transcluded into the right place in the template
        // [The second parameter to transclude causes the elements to be cloned so that they work in ng-repeat]
        accordionGroupCtrl.setHeading(transclude(scope, angular.noop));
      }
    };
  }
]).directive('accordionTransclude', [
  '$log',
  '$accordionSuppressWarning',
  function ($log, $accordionSuppressWarning) {
    return {
      require: '^accordionGroup',
      link: function (scope, element, attr, controller) {
        if (!$accordionSuppressWarning) {
          $log.warn('accordion-transclude is now deprecated. Use uib-accordion-transclude instead.');
        }
        scope.$watch(function () {
          return controller[attr.accordionTransclude];
        }, function (heading) {
          if (heading) {
            element.find('span').html('');
            element.find('span').append(heading);
          }
        });
      }
    };
  }
]);
angular.module('ui.bootstrap.alert', []).controller('UibAlertController', [
  '$scope',
  '$attrs',
  '$timeout',
  function ($scope, $attrs, $timeout) {
    $scope.closeable = !!$attrs.close;
    if (angular.isDefined($attrs.dismissOnTimeout)) {
      $timeout(function () {
        $scope.close();
      }, parseInt($attrs.dismissOnTimeout, 10));
    }
  }
]).directive('uibAlert', function () {
  return {
    controller: 'UibAlertController',
    controllerAs: 'alert',
    templateUrl: function (element, attrs) {
      return attrs.templateUrl || 'template/alert/alert.html';
    },
    transclude: true,
    replace: true,
    scope: {
      type: '@',
      close: '&'
    }
  };
});
/* Deprecated alert below */
angular.module('ui.bootstrap.alert').value('$alertSuppressWarning', false).controller('AlertController', [
  '$scope',
  '$attrs',
  '$controller',
  '$log',
  '$alertSuppressWarning',
  function ($scope, $attrs, $controller, $log, $alertSuppressWarning) {
    if (!$alertSuppressWarning) {
      $log.warn('AlertController is now deprecated. Use UibAlertController instead.');
    }
    return $controller('UibAlertController', {
      $scope: $scope,
      $attrs: $attrs
    });
  }
]).directive('alert', [
  '$log',
  '$alertSuppressWarning',
  function ($log, $alertSuppressWarning) {
    return {
      controller: 'AlertController',
      controllerAs: 'alert',
      templateUrl: function (element, attrs) {
        return attrs.templateUrl || 'template/alert/alert.html';
      },
      transclude: true,
      replace: true,
      scope: {
        type: '@',
        close: '&'
      },
      link: function () {
        if (!$alertSuppressWarning) {
          $log.warn('alert is now deprecated. Use uib-alert instead.');
        }
      }
    };
  }
]);
angular.module('ui.bootstrap.buttons', []).constant('uibButtonConfig', {
  activeClass: 'active',
  toggleEvent: 'click'
}).controller('UibButtonsController', [
  'uibButtonConfig',
  function (buttonConfig) {
    this.activeClass = buttonConfig.activeClass || 'active';
    this.toggleEvent = buttonConfig.toggleEvent || 'click';
  }
]).directive('uibBtnRadio', function () {
  return {
    require: [
      'uibBtnRadio',
      'ngModel'
    ],
    controller: 'UibButtonsController',
    controllerAs: 'buttons',
    link: function (scope, element, attrs, ctrls) {
      var buttonsCtrl = ctrls[0], ngModelCtrl = ctrls[1];
      element.find('input').css({ display: 'none' });
      //model -> UI
      ngModelCtrl.$render = function () {
        element.toggleClass(buttonsCtrl.activeClass, angular.equals(ngModelCtrl.$modelValue, scope.$eval(attrs.uibBtnRadio)));
      };
      //ui->model
      element.on(buttonsCtrl.toggleEvent, function () {
        if (attrs.disabled) {
          return;
        }
        var isActive = element.hasClass(buttonsCtrl.activeClass);
        if (!isActive || angular.isDefined(attrs.uncheckable)) {
          scope.$apply(function () {
            ngModelCtrl.$setViewValue(isActive ? null : scope.$eval(attrs.uibBtnRadio));
            ngModelCtrl.$render();
          });
        }
      });
    }
  };
}).directive('uibBtnCheckbox', [
  '$document',
  function ($document) {
    return {
      require: [
        'uibBtnCheckbox',
        'ngModel'
      ],
      controller: 'UibButtonsController',
      controllerAs: 'button',
      link: function (scope, element, attrs, ctrls) {
        var buttonsCtrl = ctrls[0], ngModelCtrl = ctrls[1];
        element.find('input').css({ display: 'none' });
        function getTrueValue() {
          return getCheckboxValue(attrs.btnCheckboxTrue, true);
        }
        function getFalseValue() {
          return getCheckboxValue(attrs.btnCheckboxFalse, false);
        }
        function getCheckboxValue(attribute, defaultValue) {
          return angular.isDefined(attribute) ? scope.$eval(attribute) : defaultValue;
        }
        //model -> UI
        ngModelCtrl.$render = function () {
          element.toggleClass(buttonsCtrl.activeClass, angular.equals(ngModelCtrl.$modelValue, getTrueValue()));
        };
        //ui->model
        element.on(buttonsCtrl.toggleEvent, function () {
          if (attrs.disabled) {
            return;
          }
          scope.$apply(function () {
            ngModelCtrl.$setViewValue(element.hasClass(buttonsCtrl.activeClass) ? getFalseValue() : getTrueValue());
            ngModelCtrl.$render();
          });
        });
        //accessibility
        element.on('keypress', function (e) {
          if (attrs.disabled || e.which !== 32 || $document[0].activeElement !== element[0]) {
            return;
          }
          scope.$apply(function () {
            ngModelCtrl.$setViewValue(element.hasClass(buttonsCtrl.activeClass) ? getFalseValue() : getTrueValue());
            ngModelCtrl.$render();
          });
        });
      }
    };
  }
]);
/* Deprecated buttons below */
angular.module('ui.bootstrap.buttons').value('$buttonsSuppressWarning', false).controller('ButtonsController', [
  '$controller',
  '$log',
  '$buttonsSuppressWarning',
  function ($controller, $log, $buttonsSuppressWarning) {
    if (!$buttonsSuppressWarning) {
      $log.warn('ButtonsController is now deprecated. Use UibButtonsController instead.');
    }
    return $controller('UibButtonsController');
  }
]).directive('btnRadio', [
  '$log',
  '$buttonsSuppressWarning',
  function ($log, $buttonsSuppressWarning) {
    return {
      require: [
        'btnRadio',
        'ngModel'
      ],
      controller: 'ButtonsController',
      controllerAs: 'buttons',
      link: function (scope, element, attrs, ctrls) {
        if (!$buttonsSuppressWarning) {
          $log.warn('btn-radio is now deprecated. Use uib-btn-radio instead.');
        }
        var buttonsCtrl = ctrls[0], ngModelCtrl = ctrls[1];
        element.find('input').css({ display: 'none' });
        //model -> UI
        ngModelCtrl.$render = function () {
          element.toggleClass(buttonsCtrl.activeClass, angular.equals(ngModelCtrl.$modelValue, scope.$eval(attrs.btnRadio)));
        };
        //ui->model
        element.bind(buttonsCtrl.toggleEvent, function () {
          if (attrs.disabled) {
            return;
          }
          var isActive = element.hasClass(buttonsCtrl.activeClass);
          if (!isActive || angular.isDefined(attrs.uncheckable)) {
            scope.$apply(function () {
              ngModelCtrl.$setViewValue(isActive ? null : scope.$eval(attrs.btnRadio));
              ngModelCtrl.$render();
            });
          }
        });
      }
    };
  }
]).directive('btnCheckbox', [
  '$document',
  '$log',
  '$buttonsSuppressWarning',
  function ($document, $log, $buttonsSuppressWarning) {
    return {
      require: [
        'btnCheckbox',
        'ngModel'
      ],
      controller: 'ButtonsController',
      controllerAs: 'button',
      link: function (scope, element, attrs, ctrls) {
        if (!$buttonsSuppressWarning) {
          $log.warn('btn-checkbox is now deprecated. Use uib-btn-checkbox instead.');
        }
        var buttonsCtrl = ctrls[0], ngModelCtrl = ctrls[1];
        element.find('input').css({ display: 'none' });
        function getTrueValue() {
          return getCheckboxValue(attrs.btnCheckboxTrue, true);
        }
        function getFalseValue() {
          return getCheckboxValue(attrs.btnCheckboxFalse, false);
        }
        function getCheckboxValue(attributeValue, defaultValue) {
          var val = scope.$eval(attributeValue);
          return angular.isDefined(val) ? val : defaultValue;
        }
        //model -> UI
        ngModelCtrl.$render = function () {
          element.toggleClass(buttonsCtrl.activeClass, angular.equals(ngModelCtrl.$modelValue, getTrueValue()));
        };
        //ui->model
        element.bind(buttonsCtrl.toggleEvent, function () {
          if (attrs.disabled) {
            return;
          }
          scope.$apply(function () {
            ngModelCtrl.$setViewValue(element.hasClass(buttonsCtrl.activeClass) ? getFalseValue() : getTrueValue());
            ngModelCtrl.$render();
          });
        });
        //accessibility
        element.on('keypress', function (e) {
          if (attrs.disabled || e.which !== 32 || $document[0].activeElement !== element[0]) {
            return;
          }
          scope.$apply(function () {
            ngModelCtrl.$setViewValue(element.hasClass(buttonsCtrl.activeClass) ? getFalseValue() : getTrueValue());
            ngModelCtrl.$render();
          });
        });
      }
    };
  }
]);
/**    
 * @ngdoc overview    
 * @name ui.bootstrap.carousel    
 *    
 * @description   
 * AngularJS version of an image carousel.    
 *    
 */
angular.module('ui.bootstrap.carousel', []).controller('UibCarouselController', [
  '$scope',
  '$element',
  '$interval',
  '$animate',
  function ($scope, $element, $interval, $animate) {
    var self = this, slides = self.slides = $scope.slides = [], NEW_ANIMATE = angular.version.minor >= 4, NO_TRANSITION = 'uib-noTransition', SLIDE_DIRECTION = 'uib-slideDirection', currentIndex = -1, currentInterval, isPlaying;
    self.currentSlide = null;
    var destroyed = false;
    /* direction: "prev" or "next" */
    self.select = $scope.select = function (nextSlide, direction) {
      var nextIndex = $scope.indexOfSlide(nextSlide);
      //Decide direction if it's not given
      if (direction === undefined) {
        direction = nextIndex > self.getCurrentIndex() ? 'next' : 'prev';
      }
      //Prevent this user-triggered transition from occurring if there is already one in progress
      if (nextSlide && nextSlide !== self.currentSlide && !$scope.$currentTransition) {
        goNext(nextSlide, nextIndex, direction);
      }
    };
    function goNext(slide, index, direction) {
      // Scope has been destroyed, stop here.
      if (destroyed) {
        return;
      }
      angular.extend(slide, {
        direction: direction,
        active: true
      });
      angular.extend(self.currentSlide || {}, {
        direction: direction,
        active: false
      });
      if ($animate.enabled() && !$scope.noTransition && !$scope.$currentTransition && slide.$element && self.slides.length > 1) {
        slide.$element.data(SLIDE_DIRECTION, slide.direction);
        if (self.currentSlide && self.currentSlide.$element) {
          self.currentSlide.$element.data(SLIDE_DIRECTION, slide.direction);
        }
        $scope.$currentTransition = true;
        if (NEW_ANIMATE) {
          $animate.on('addClass', slide.$element, function (element, phase) {
            if (phase === 'close') {
              $scope.$currentTransition = null;
              $animate.off('addClass', element);
            }
          });
        } else {
          slide.$element.one('$animate:close', function closeFn() {
            $scope.$currentTransition = null;
          });
        }
      }
      self.currentSlide = slide;
      currentIndex = index;
      //every time you change slides, reset the timer
      restartTimer();
    }
    $scope.$on('$destroy', function () {
      destroyed = true;
    });
    function getSlideByIndex(index) {
      if (angular.isUndefined(slides[index].index)) {
        return slides[index];
      }
      var i, len = slides.length;
      for (i = 0; i < slides.length; ++i) {
        if (slides[i].index == index) {
          return slides[i];
        }
      }
    }
    self.getCurrentIndex = function () {
      if (self.currentSlide && angular.isDefined(self.currentSlide.index)) {
        return +self.currentSlide.index;
      }
      return currentIndex;
    };
    /* Allow outside people to call indexOf on slides array */
    $scope.indexOfSlide = function (slide) {
      return angular.isDefined(slide.index) ? +slide.index : slides.indexOf(slide);
    };
    $scope.next = function () {
      var newIndex = (self.getCurrentIndex() + 1) % slides.length;
      if (newIndex === 0 && $scope.noWrap()) {
        $scope.pause();
        return;
      }
      return self.select(getSlideByIndex(newIndex), 'next');
    };
    $scope.prev = function () {
      var newIndex = self.getCurrentIndex() - 1 < 0 ? slides.length - 1 : self.getCurrentIndex() - 1;
      if ($scope.noWrap() && newIndex === slides.length - 1) {
        $scope.pause();
        return;
      }
      return self.select(getSlideByIndex(newIndex), 'prev');
    };
    $scope.isActive = function (slide) {
      return self.currentSlide === slide;
    };
    $scope.$watch('interval', restartTimer);
    $scope.$watchCollection('slides', resetTransition);
    $scope.$on('$destroy', resetTimer);
    function restartTimer() {
      resetTimer();
      var interval = +$scope.interval;
      if (!isNaN(interval) && interval > 0) {
        currentInterval = $interval(timerFn, interval);
      }
    }
    function resetTimer() {
      if (currentInterval) {
        $interval.cancel(currentInterval);
        currentInterval = null;
      }
    }
    function timerFn() {
      var interval = +$scope.interval;
      if (isPlaying && !isNaN(interval) && interval > 0 && slides.length) {
        $scope.next();
      } else {
        $scope.pause();
      }
    }
    function resetTransition(slides) {
      if (!slides.length) {
        $scope.$currentTransition = null;
      }
    }
    $scope.play = function () {
      if (!isPlaying) {
        isPlaying = true;
        restartTimer();
      }
    };
    $scope.pause = function () {
      if (!$scope.noPause) {
        isPlaying = false;
        resetTimer();
      }
    };
    self.addSlide = function (slide, element) {
      slide.$element = element;
      slides.push(slide);
      //if this is the first slide or the slide is set to active, select it
      if (slides.length === 1 || slide.active) {
        self.select(slides[slides.length - 1]);
        if (slides.length === 1) {
          $scope.play();
        }
      } else {
        slide.active = false;
      }
    };
    self.removeSlide = function (slide) {
      if (angular.isDefined(slide.index)) {
        slides.sort(function (a, b) {
          return +a.index > +b.index;
        });
      }
      //get the index of the slide inside the carousel
      var index = slides.indexOf(slide);
      slides.splice(index, 1);
      if (slides.length > 0 && slide.active) {
        if (index >= slides.length) {
          self.select(slides[index - 1]);
        } else {
          self.select(slides[index]);
        }
      } else if (currentIndex > index) {
        currentIndex--;
      }
      //clean the currentSlide when no more slide
      if (slides.length === 0) {
        self.currentSlide = null;
      }
    };
    $scope.$watch('noTransition', function (noTransition) {
      $element.data(NO_TRANSITION, noTransition);
    });
  }
]).directive('uibCarousel', [function () {
    return {
      transclude: true,
      replace: true,
      controller: 'UibCarouselController',
      controllerAs: 'carousel',
      require: 'carousel',
      templateUrl: function (element, attrs) {
        return attrs.templateUrl || 'template/carousel/carousel.html';
      },
      scope: {
        interval: '=',
        noTransition: '=',
        noPause: '=',
        noWrap: '&'
      }
    };
  }]).directive('uibSlide', function () {
  return {
    require: '^uibCarousel',
    restrict: 'EA',
    transclude: true,
    replace: true,
    templateUrl: function (element, attrs) {
      return attrs.templateUrl || 'template/carousel/slide.html';
    },
    scope: {
      active: '=?',
      actual: '=?',
      index: '=?'
    },
    link: function (scope, element, attrs, carouselCtrl) {
      carouselCtrl.addSlide(scope, element);
      //when the scope is destroyed then remove the slide from the current slides array
      scope.$on('$destroy', function () {
        carouselCtrl.removeSlide(scope);
      });
      scope.$watch('active', function (active) {
        if (active) {
          carouselCtrl.select(scope);
        }
      });
    }
  };
}).animation('.item', [
  '$injector',
  '$animate',
  function ($injector, $animate) {
    var NO_TRANSITION = 'uib-noTransition', SLIDE_DIRECTION = 'uib-slideDirection', $animateCss = null;
    if ($injector.has('$animateCss')) {
      $animateCss = $injector.get('$animateCss');
    }
    function removeClass(element, className, callback) {
      element.removeClass(className);
      if (callback) {
        callback();
      }
    }
    return {
      beforeAddClass: function (element, className, done) {
        // Due to transclusion, noTransition property is on parent's scope
        if (className == 'active' && element.parent() && element.parent().parent() && !element.parent().parent().data(NO_TRANSITION)) {
          var stopped = false;
          var direction = element.data(SLIDE_DIRECTION);
          var directionClass = direction == 'next' ? 'left' : 'right';
          var removeClassFn = removeClass.bind(this, element, directionClass + ' ' + direction, done);
          element.addClass(direction);
          if ($animateCss) {
            $animateCss(element, { addClass: directionClass }).start().done(removeClassFn);
          } else {
            $animate.addClass(element, directionClass).then(function () {
              if (!stopped) {
                removeClassFn();
              }
              done();
            });
          }
          return function () {
            stopped = true;
          };
        }
        done();
      },
      beforeRemoveClass: function (element, className, done) {
        // Due to transclusion, noTransition property is on parent's scope
        if (className === 'active' && element.parent() && element.parent().parent() && !element.parent().parent().data(NO_TRANSITION)) {
          var stopped = false;
          var direction = element.data(SLIDE_DIRECTION);
          var directionClass = direction == 'next' ? 'left' : 'right';
          var removeClassFn = removeClass.bind(this, element, directionClass, done);
          if ($animateCss) {
            $animateCss(element, { addClass: directionClass }).start().done(removeClassFn);
          } else {
            $animate.addClass(element, directionClass).then(function () {
              if (!stopped) {
                removeClassFn();
              }
              done();
            });
          }
          return function () {
            stopped = true;
          };
        }
        done();
      }
    };
  }
]);
/* deprecated carousel below */
angular.module('ui.bootstrap.carousel').value('$carouselSuppressWarning', false).controller('CarouselController', [
  '$scope',
  '$element',
  '$controller',
  '$log',
  '$carouselSuppressWarning',
  function ($scope, $element, $controller, $log, $carouselSuppressWarning) {
    if (!$carouselSuppressWarning) {
      $log.warn('CarouselController is now deprecated. Use UibCarouselController instead.');
    }
    return $controller('UibCarouselController', {
      $scope: $scope,
      $element: $element
    });
  }
]).directive('carousel', [
  '$log',
  '$carouselSuppressWarning',
  function ($log, $carouselSuppressWarning) {
    return {
      transclude: true,
      replace: true,
      controller: 'CarouselController',
      controllerAs: 'carousel',
      require: 'carousel',
      templateUrl: function (element, attrs) {
        return attrs.templateUrl || 'template/carousel/carousel.html';
      },
      scope: {
        interval: '=',
        noTransition: '=',
        noPause: '=',
        noWrap: '&'
      },
      link: function () {
        if (!$carouselSuppressWarning) {
          $log.warn('carousel is now deprecated. Use uib-carousel instead.');
        }
      }
    };
  }
]).directive('slide', [
  '$log',
  '$carouselSuppressWarning',
  function ($log, $carouselSuppressWarning) {
    return {
      require: '^carousel',
      transclude: true,
      replace: true,
      templateUrl: function (element, attrs) {
        return attrs.templateUrl || 'template/carousel/slide.html';
      },
      scope: {
        active: '=?',
        actual: '=?',
        index: '=?'
      },
      link: function (scope, element, attrs, carouselCtrl) {
        if (!$carouselSuppressWarning) {
          $log.warn('slide is now deprecated. Use uib-slide instead.');
        }
        carouselCtrl.addSlide(scope, element);
        //when the scope is destroyed then remove the slide from the current slides array
        scope.$on('$destroy', function () {
          carouselCtrl.removeSlide(scope);
        });
        scope.$watch('active', function (active) {
          if (active) {
            carouselCtrl.select(scope);
          }
        });
      }
    };
  }
]);
angular.module('ui.bootstrap.dateparser', []).service('uibDateParser', [
  '$log',
  '$locale',
  'orderByFilter',
  function ($log, $locale, orderByFilter) {
    // Pulled from https://github.com/mbostock/d3/blob/master/src/format/requote.js
    var SPECIAL_CHARACTERS_REGEXP = /[\\\^\$\*\+\?\|\[\]\(\)\.\{\}]/g;
    var localeId;
    var formatCodeToRegex;
    this.init = function () {
      localeId = $locale.id;
      this.parsers = {};
      formatCodeToRegex = {
        'yyyy': {
          regex: '\\d{4}',
          apply: function (value) {
            this.year = +value;
          }
        },
        'yy': {
          regex: '\\d{2}',
          apply: function (value) {
            this.year = +value + 2000;
          }
        },
        'y': {
          regex: '\\d{1,4}',
          apply: function (value) {
            this.year = +value;
          }
        },
        'MMMM': {
          regex: $locale.DATETIME_FORMATS.MONTH.join('|'),
          apply: function (value) {
            this.month = $locale.DATETIME_FORMATS.MONTH.indexOf(value);
          }
        },
        'MMM': {
          regex: $locale.DATETIME_FORMATS.SHORTMONTH.join('|'),
          apply: function (value) {
            this.month = $locale.DATETIME_FORMATS.SHORTMONTH.indexOf(value);
          }
        },
        'MM': {
          regex: '0[1-9]|1[0-2]',
          apply: function (value) {
            this.month = value - 1;
          }
        },
        'M': {
          regex: '[1-9]|1[0-2]',
          apply: function (value) {
            this.month = value - 1;
          }
        },
        'dd': {
          regex: '[0-2][0-9]{1}|3[0-1]{1}',
          apply: function (value) {
            this.date = +value;
          }
        },
        'd': {
          regex: '[1-2]?[0-9]{1}|3[0-1]{1}',
          apply: function (value) {
            this.date = +value;
          }
        },
        'EEEE': { regex: $locale.DATETIME_FORMATS.DAY.join('|') },
        'EEE': { regex: $locale.DATETIME_FORMATS.SHORTDAY.join('|') },
        'HH': {
          regex: '(?:0|1)[0-9]|2[0-3]',
          apply: function (value) {
            this.hours = +value;
          }
        },
        'hh': {
          regex: '0[0-9]|1[0-2]',
          apply: function (value) {
            this.hours = +value;
          }
        },
        'H': {
          regex: '1?[0-9]|2[0-3]',
          apply: function (value) {
            this.hours = +value;
          }
        },
        'h': {
          regex: '[0-9]|1[0-2]',
          apply: function (value) {
            this.hours = +value;
          }
        },
        'mm': {
          regex: '[0-5][0-9]',
          apply: function (value) {
            this.minutes = +value;
          }
        },
        'm': {
          regex: '[0-9]|[1-5][0-9]',
          apply: function (value) {
            this.minutes = +value;
          }
        },
        'sss': {
          regex: '[0-9][0-9][0-9]',
          apply: function (value) {
            this.milliseconds = +value;
          }
        },
        'ss': {
          regex: '[0-5][0-9]',
          apply: function (value) {
            this.seconds = +value;
          }
        },
        's': {
          regex: '[0-9]|[1-5][0-9]',
          apply: function (value) {
            this.seconds = +value;
          }
        },
        'a': {
          regex: $locale.DATETIME_FORMATS.AMPMS.join('|'),
          apply: function (value) {
            if (this.hours === 12) {
              this.hours = 0;
            }
            if (value === 'PM') {
              this.hours += 12;
            }
          }
        }
      };
    };
    this.init();
    function createParser(format) {
      var map = [], regex = format.split('');
      angular.forEach(formatCodeToRegex, function (data, code) {
        var index = format.indexOf(code);
        if (index > -1) {
          format = format.split('');
          regex[index] = '(' + data.regex + ')';
          format[index] = '$';
          // Custom symbol to define consumed part of format
          for (var i = index + 1, n = index + code.length; i < n; i++) {
            regex[i] = '';
            format[i] = '$';
          }
          format = format.join('');
          map.push({
            index: index,
            apply: data.apply
          });
        }
      });
      return {
        regex: new RegExp('^' + regex.join('') + '$'),
        map: orderByFilter(map, 'index')
      };
    }
    this.parse = function (input, format, baseDate) {
      if (!angular.isString(input) || !format) {
        return input;
      }
      format = $locale.DATETIME_FORMATS[format] || format;
      format = format.replace(SPECIAL_CHARACTERS_REGEXP, '\\$&');
      if ($locale.id !== localeId) {
        this.init();
      }
      if (!this.parsers[format]) {
        this.parsers[format] = createParser(format);
      }
      var parser = this.parsers[format], regex = parser.regex, map = parser.map, results = input.match(regex);
      if (results && results.length) {
        var fields, dt;
        if (angular.isDate(baseDate) && !isNaN(baseDate.getTime())) {
          fields = {
            year: baseDate.getFullYear(),
            month: baseDate.getMonth(),
            date: baseDate.getDate(),
            hours: baseDate.getHours(),
            minutes: baseDate.getMinutes(),
            seconds: baseDate.getSeconds(),
            milliseconds: baseDate.getMilliseconds()
          };
        } else {
          if (baseDate) {
            $log.warn('dateparser:', 'baseDate is not a valid date');
          }
          fields = {
            year: 1900,
            month: 0,
            date: 1,
            hours: 0,
            minutes: 0,
            seconds: 0,
            milliseconds: 0
          };
        }
        for (var i = 1, n = results.length; i < n; i++) {
          var mapper = map[i - 1];
          if (mapper.apply) {
            mapper.apply.call(fields, results[i]);
          }
        }
        if (isValid(fields.year, fields.month, fields.date)) {
          dt = new Date(fields.year, fields.month, fields.date, fields.hours, fields.minutes, fields.seconds, fields.milliseconds || 0);
        }
        return dt;
      }
    };
    // Check if date is valid for specific month (and year for February).
    // Month: 0 = Jan, 1 = Feb, etc
    function isValid(year, month, date) {
      if (date < 1) {
        return false;
      }
      if (month === 1 && date > 28) {
        return date === 29 && (year % 4 === 0 && year % 100 !== 0 || year % 400 === 0);
      }
      if (month === 3 || month === 5 || month === 8 || month === 10) {
        return date < 31;
      }
      return true;
    }
  }
]);
/* Deprecated dateparser below */
angular.module('ui.bootstrap.dateparser').value('$dateParserSuppressWarning', false).service('dateParser', [
  '$log',
  '$dateParserSuppressWarning',
  'uibDateParser',
  function ($log, $dateParserSuppressWarning, uibDateParser) {
    if (!$dateParserSuppressWarning) {
      $log.warn('dateParser is now deprecated. Use uibDateParser instead.');
    }
    angular.extend(this, uibDateParser);
  }
]);
angular.module('ui.bootstrap.position', []).factory('$uibPosition', [
  '$document',
  '$window',
  function ($document, $window) {
    function getStyle(el, cssprop) {
      if (el.currentStyle) {
        //IE
        return el.currentStyle[cssprop];
      } else if ($window.getComputedStyle) {
        return $window.getComputedStyle(el)[cssprop];
      }
      // finally try and get inline style
      return el.style[cssprop];
    }
    /**
     * Checks if a given element is statically positioned
     * @param element - raw DOM element
     */
    function isStaticPositioned(element) {
      return (getStyle(element, 'position') || 'static') === 'static';
    }
    /**
     * returns the closest, non-statically positioned parentOffset of a given element
     * @param element
     */
    var parentOffsetEl = function (element) {
      var docDomEl = $document[0];
      var offsetParent = element.offsetParent || docDomEl;
      while (offsetParent && offsetParent !== docDomEl && isStaticPositioned(offsetParent)) {
        offsetParent = offsetParent.offsetParent;
      }
      return offsetParent || docDomEl;
    };
    return {
      position: function (element) {
        var elBCR = this.offset(element);
        var offsetParentBCR = {
            top: 0,
            left: 0
          };
        var offsetParentEl = parentOffsetEl(element[0]);
        if (offsetParentEl != $document[0]) {
          offsetParentBCR = this.offset(angular.element(offsetParentEl));
          offsetParentBCR.top += offsetParentEl.clientTop - offsetParentEl.scrollTop;
          offsetParentBCR.left += offsetParentEl.clientLeft - offsetParentEl.scrollLeft;
        }
        var boundingClientRect = element[0].getBoundingClientRect();
        return {
          width: boundingClientRect.width || element.prop('offsetWidth'),
          height: boundingClientRect.height || element.prop('offsetHeight'),
          top: elBCR.top - offsetParentBCR.top,
          left: elBCR.left - offsetParentBCR.left
        };
      },
      offset: function (element) {
        var boundingClientRect = element[0].getBoundingClientRect();
        return {
          width: boundingClientRect.width || element.prop('offsetWidth'),
          height: boundingClientRect.height || element.prop('offsetHeight'),
          top: boundingClientRect.top + ($window.pageYOffset || $document[0].documentElement.scrollTop),
          left: boundingClientRect.left + ($window.pageXOffset || $document[0].documentElement.scrollLeft)
        };
      },
      positionElements: function (hostEl, targetEl, positionStr, appendToBody) {
        var positionStrParts = positionStr.split('-');
        var pos0 = positionStrParts[0], pos1 = positionStrParts[1] || 'center';
        var hostElPos, targetElWidth, targetElHeight, targetElPos;
        hostElPos = appendToBody ? this.offset(hostEl) : this.position(hostEl);
        targetElWidth = targetEl.prop('offsetWidth');
        targetElHeight = targetEl.prop('offsetHeight');
        var shiftWidth = {
            center: function () {
              return hostElPos.left + hostElPos.width / 2 - targetElWidth / 2;
            },
            left: function () {
              return hostElPos.left;
            },
            right: function () {
              return hostElPos.left + hostElPos.width;
            }
          };
        var shiftHeight = {
            center: function () {
              return hostElPos.top + hostElPos.height / 2 - targetElHeight / 2;
            },
            top: function () {
              return hostElPos.top;
            },
            bottom: function () {
              return hostElPos.top + hostElPos.height;
            }
          };
        switch (pos0) {
        case 'right':
          targetElPos = {
            top: shiftHeight[pos1](),
            left: shiftWidth[pos0]()
          };
          break;
        case 'left':
          targetElPos = {
            top: shiftHeight[pos1](),
            left: hostElPos.left - targetElWidth
          };
          break;
        case 'bottom':
          targetElPos = {
            top: shiftHeight[pos0](),
            left: shiftWidth[pos1]()
          };
          break;
        default:
          targetElPos = {
            top: hostElPos.top - targetElHeight,
            left: shiftWidth[pos1]()
          };
          break;
        }
        return targetElPos;
      }
    };
  }
]);
/* Deprecated position below */
angular.module('ui.bootstrap.position').value('$positionSuppressWarning', false).service('$position', [
  '$log',
  '$positionSuppressWarning',
  '$uibPosition',
  function ($log, $positionSuppressWarning, $uibPosition) {
    if (!$positionSuppressWarning) {
      $log.warn('$position is now deprecated. Use $uibPosition instead.');
    }
    angular.extend(this, $uibPosition);
  }
]);
angular.module('ui.bootstrap.datepicker', [
  'ui.bootstrap.dateparser',
  'ui.bootstrap.position'
]).value('$datepickerSuppressError', false).constant('uibDatepickerConfig', {
  formatDay: 'dd',
  formatMonth: 'MMMM',
  formatYear: 'yyyy',
  formatDayHeader: 'EEE',
  formatDayTitle: 'MMMM yyyy',
  formatMonthTitle: 'yyyy',
  datepickerMode: 'day',
  minMode: 'day',
  maxMode: 'year',
  showWeeks: true,
  startingDay: 0,
  yearRange: 20,
  minDate: null,
  maxDate: null,
  shortcutPropagation: false
}).controller('UibDatepickerController', [
  '$scope',
  '$attrs',
  '$parse',
  '$interpolate',
  '$log',
  'dateFilter',
  'uibDatepickerConfig',
  '$datepickerSuppressError',
  function ($scope, $attrs, $parse, $interpolate, $log, dateFilter, datepickerConfig, $datepickerSuppressError) {
    var self = this, ngModelCtrl = { $setViewValue: angular.noop };
    // nullModelCtrl;
    // Modes chain
    this.modes = [
      'day',
      'month',
      'year'
    ];
    // Configuration attributes
    angular.forEach([
      'formatDay',
      'formatMonth',
      'formatYear',
      'formatDayHeader',
      'formatDayTitle',
      'formatMonthTitle',
      'showWeeks',
      'startingDay',
      'yearRange',
      'shortcutPropagation'
    ], function (key, index) {
      self[key] = angular.isDefined($attrs[key]) ? index < 6 ? $interpolate($attrs[key])($scope.$parent) : $scope.$parent.$eval($attrs[key]) : datepickerConfig[key];
    });
    // Watchable date attributes
    angular.forEach([
      'minDate',
      'maxDate'
    ], function (key) {
      if ($attrs[key]) {
        $scope.$parent.$watch($parse($attrs[key]), function (value) {
          self[key] = value ? new Date(value) : null;
          self.refreshView();
        });
      } else {
        self[key] = datepickerConfig[key] ? new Date(datepickerConfig[key]) : null;
      }
    });
    angular.forEach([
      'minMode',
      'maxMode'
    ], function (key) {
      if ($attrs[key]) {
        $scope.$parent.$watch($parse($attrs[key]), function (value) {
          self[key] = angular.isDefined(value) ? value : $attrs[key];
          $scope[key] = self[key];
          if (key == 'minMode' && self.modes.indexOf($scope.datepickerMode) < self.modes.indexOf(self[key]) || key == 'maxMode' && self.modes.indexOf($scope.datepickerMode) > self.modes.indexOf(self[key])) {
            $scope.datepickerMode = self[key];
          }
        });
      } else {
        self[key] = datepickerConfig[key] || null;
        $scope[key] = self[key];
      }
    });
    $scope.datepickerMode = $scope.datepickerMode || datepickerConfig.datepickerMode;
    $scope.uniqueId = 'datepicker-' + $scope.$id + '-' + Math.floor(Math.random() * 10000);
    if (angular.isDefined($attrs.initDate)) {
      this.activeDate = $scope.$parent.$eval($attrs.initDate) || new Date();
      $scope.$parent.$watch($attrs.initDate, function (initDate) {
        if (initDate && (ngModelCtrl.$isEmpty(ngModelCtrl.$modelValue) || ngModelCtrl.$invalid)) {
          self.activeDate = initDate;
          self.refreshView();
        }
      });
    } else {
      this.activeDate = new Date();
    }
    $scope.isActive = function (dateObject) {
      if (self.compare(dateObject.date, self.activeDate) === 0) {
        $scope.activeDateId = dateObject.uid;
        return true;
      }
      return false;
    };
    this.init = function (ngModelCtrl_) {
      ngModelCtrl = ngModelCtrl_;
      ngModelCtrl.$render = function () {
        self.render();
      };
    };
    this.render = function () {
      if (ngModelCtrl.$viewValue) {
        var date = new Date(ngModelCtrl.$viewValue), isValid = !isNaN(date);
        if (isValid) {
          this.activeDate = date;
        } else if (!$datepickerSuppressError) {
          $log.error('Datepicker directive: "ng-model" value must be a Date object, a number of milliseconds since 01.01.1970 or a string representing an RFC2822 or ISO 8601 date.');
        }
      }
      this.refreshView();
    };
    this.refreshView = function () {
      if (this.element) {
        this._refreshView();
        var date = ngModelCtrl.$viewValue ? new Date(ngModelCtrl.$viewValue) : null;
        ngModelCtrl.$setValidity('dateDisabled', !date || this.element && !this.isDisabled(date));
      }
    };
    this.createDateObject = function (date, format) {
      var model = ngModelCtrl.$viewValue ? new Date(ngModelCtrl.$viewValue) : null;
      return {
        date: date,
        label: dateFilter(date, format),
        selected: model && this.compare(date, model) === 0,
        disabled: this.isDisabled(date),
        current: this.compare(date, new Date()) === 0,
        customClass: this.customClass(date)
      };
    };
    this.isDisabled = function (date) {
      return this.minDate && this.compare(date, this.minDate) < 0 || this.maxDate && this.compare(date, this.maxDate) > 0 || $attrs.dateDisabled && $scope.dateDisabled({
        date: date,
        mode: $scope.datepickerMode
      });
    };
    this.customClass = function (date) {
      return $scope.customClass({
        date: date,
        mode: $scope.datepickerMode
      });
    };
    // Split array into smaller arrays
    this.split = function (arr, size) {
      var arrays = [];
      while (arr.length > 0) {
        arrays.push(arr.splice(0, size));
      }
      return arrays;
    };
    // Fix a hard-reprodusible bug with timezones
    // The bug depends on OS, browser, current timezone and current date
    // i.e.
    // var date = new Date(2014, 0, 1);
    // console.log(date.getFullYear(), date.getMonth(), date.getDate(), date.getHours());
    // can result in "2013 11 31 23" because of the bug.
    this.fixTimeZone = function (date) {
      var hours = date.getHours();
      date.setHours(hours === 23 ? hours + 2 : 0);
    };
    $scope.select = function (date) {
      if ($scope.datepickerMode === self.minMode) {
        var dt = ngModelCtrl.$viewValue ? new Date(ngModelCtrl.$viewValue) : new Date(0, 0, 0, 0, 0, 0, 0);
        dt.setFullYear(date.getFullYear(), date.getMonth(), date.getDate());
        ngModelCtrl.$setViewValue(dt);
        ngModelCtrl.$render();
      } else {
        self.activeDate = date;
        $scope.datepickerMode = self.modes[self.modes.indexOf($scope.datepickerMode) - 1];
      }
    };
    $scope.move = function (direction) {
      var year = self.activeDate.getFullYear() + direction * (self.step.years || 0), month = self.activeDate.getMonth() + direction * (self.step.months || 0);
      self.activeDate.setFullYear(year, month, 1);
      self.refreshView();
    };
    $scope.toggleMode = function (direction) {
      direction = direction || 1;
      if ($scope.datepickerMode === self.maxMode && direction === 1 || $scope.datepickerMode === self.minMode && direction === -1) {
        return;
      }
      $scope.datepickerMode = self.modes[self.modes.indexOf($scope.datepickerMode) + direction];
    };
    // Key event mapper
    $scope.keys = {
      13: 'enter',
      32: 'space',
      33: 'pageup',
      34: 'pagedown',
      35: 'end',
      36: 'home',
      37: 'left',
      38: 'up',
      39: 'right',
      40: 'down'
    };
    var focusElement = function () {
      self.element[0].focus();
    };
    // Listen for focus requests from popup directive
    $scope.$on('uib:datepicker.focus', focusElement);
    $scope.keydown = function (evt) {
      var key = $scope.keys[evt.which];
      if (!key || evt.shiftKey || evt.altKey) {
        return;
      }
      evt.preventDefault();
      if (!self.shortcutPropagation) {
        evt.stopPropagation();
      }
      if (key === 'enter' || key === 'space') {
        if (self.isDisabled(self.activeDate)) {
          return;  // do nothing
        }
        $scope.select(self.activeDate);
      } else if (evt.ctrlKey && (key === 'up' || key === 'down')) {
        $scope.toggleMode(key === 'up' ? 1 : -1);
      } else {
        self.handleKeyDown(key, evt);
        self.refreshView();
      }
    };
  }
]).controller('UibDaypickerController', [
  '$scope',
  '$element',
  'dateFilter',
  function (scope, $element, dateFilter) {
    var DAYS_IN_MONTH = [
        31,
        28,
        31,
        30,
        31,
        30,
        31,
        31,
        30,
        31,
        30,
        31
      ];
    this.step = { months: 1 };
    this.element = $element;
    function getDaysInMonth(year, month) {
      return month === 1 && year % 4 === 0 && (year % 100 !== 0 || year % 400 === 0) ? 29 : DAYS_IN_MONTH[month];
    }
    this.init = function (ctrl) {
      angular.extend(ctrl, this);
      scope.showWeeks = ctrl.showWeeks;
      ctrl.refreshView();
    };
    this.getDates = function (startDate, n) {
      var dates = new Array(n), current = new Date(startDate), i = 0, date;
      while (i < n) {
        date = new Date(current);
        this.fixTimeZone(date);
        dates[i++] = date;
        current.setDate(current.getDate() + 1);
      }
      return dates;
    };
    this._refreshView = function () {
      var year = this.activeDate.getFullYear(), month = this.activeDate.getMonth(), firstDayOfMonth = new Date(year, month, 1), difference = this.startingDay - firstDayOfMonth.getDay(), numDisplayedFromPreviousMonth = difference > 0 ? 7 - difference : -difference, firstDate = new Date(firstDayOfMonth);
      if (numDisplayedFromPreviousMonth > 0) {
        firstDate.setDate(-numDisplayedFromPreviousMonth + 1);
      }
      // 42 is the number of days on a six-month calendar
      var days = this.getDates(firstDate, 42);
      for (var i = 0; i < 42; i++) {
        days[i] = angular.extend(this.createDateObject(days[i], this.formatDay), {
          secondary: days[i].getMonth() !== month,
          uid: scope.uniqueId + '-' + i
        });
      }
      scope.labels = new Array(7);
      for (var j = 0; j < 7; j++) {
        scope.labels[j] = {
          abbr: dateFilter(days[j].date, this.formatDayHeader),
          full: dateFilter(days[j].date, 'EEEE')
        };
      }
      scope.title = dateFilter(this.activeDate, this.formatDayTitle);
      scope.rows = this.split(days, 7);
      if (scope.showWeeks) {
        scope.weekNumbers = [];
        var thursdayIndex = (4 + 7 - this.startingDay) % 7, numWeeks = scope.rows.length;
        for (var curWeek = 0; curWeek < numWeeks; curWeek++) {
          scope.weekNumbers.push(getISO8601WeekNumber(scope.rows[curWeek][thursdayIndex].date));
        }
      }
    };
    this.compare = function (date1, date2) {
      return new Date(date1.getFullYear(), date1.getMonth(), date1.getDate()) - new Date(date2.getFullYear(), date2.getMonth(), date2.getDate());
    };
    function getISO8601WeekNumber(date) {
      var checkDate = new Date(date);
      checkDate.setDate(checkDate.getDate() + 4 - (checkDate.getDay() || 7));
      // Thursday
      var time = checkDate.getTime();
      checkDate.setMonth(0);
      // Compare with Jan 1
      checkDate.setDate(1);
      return Math.floor(Math.round((time - checkDate) / 86400000) / 7) + 1;
    }
    this.handleKeyDown = function (key, evt) {
      var date = this.activeDate.getDate();
      if (key === 'left') {
        date = date - 1;  // up
      } else if (key === 'up') {
        date = date - 7;  // down
      } else if (key === 'right') {
        date = date + 1;  // down
      } else if (key === 'down') {
        date = date + 7;
      } else if (key === 'pageup' || key === 'pagedown') {
        var month = this.activeDate.getMonth() + (key === 'pageup' ? -1 : 1);
        this.activeDate.setMonth(month, 1);
        date = Math.min(getDaysInMonth(this.activeDate.getFullYear(), this.activeDate.getMonth()), date);
      } else if (key === 'home') {
        date = 1;
      } else if (key === 'end') {
        date = getDaysInMonth(this.activeDate.getFullYear(), this.activeDate.getMonth());
      }
      this.activeDate.setDate(date);
    };
  }
]).controller('UibMonthpickerController', [
  '$scope',
  '$element',
  'dateFilter',
  function (scope, $element, dateFilter) {
    this.step = { years: 1 };
    this.element = $element;
    this.init = function (ctrl) {
      angular.extend(ctrl, this);
      ctrl.refreshView();
    };
    this._refreshView = function () {
      var months = new Array(12), year = this.activeDate.getFullYear(), date;
      for (var i = 0; i < 12; i++) {
        date = new Date(year, i, 1);
        this.fixTimeZone(date);
        months[i] = angular.extend(this.createDateObject(date, this.formatMonth), { uid: scope.uniqueId + '-' + i });
      }
      scope.title = dateFilter(this.activeDate, this.formatMonthTitle);
      scope.rows = this.split(months, 3);
    };
    this.compare = function (date1, date2) {
      return new Date(date1.getFullYear(), date1.getMonth()) - new Date(date2.getFullYear(), date2.getMonth());
    };
    this.handleKeyDown = function (key, evt) {
      var date = this.activeDate.getMonth();
      if (key === 'left') {
        date = date - 1;  // up
      } else if (key === 'up') {
        date = date - 3;  // down
      } else if (key === 'right') {
        date = date + 1;  // down
      } else if (key === 'down') {
        date = date + 3;
      } else if (key === 'pageup' || key === 'pagedown') {
        var year = this.activeDate.getFullYear() + (key === 'pageup' ? -1 : 1);
        this.activeDate.setFullYear(year);
      } else if (key === 'home') {
        date = 0;
      } else if (key === 'end') {
        date = 11;
      }
      this.activeDate.setMonth(date);
    };
  }
]).controller('UibYearpickerController', [
  '$scope',
  '$element',
  'dateFilter',
  function (scope, $element, dateFilter) {
    var range;
    this.element = $element;
    function getStartingYear(year) {
      return parseInt((year - 1) / range, 10) * range + 1;
    }
    this.yearpickerInit = function () {
      range = this.yearRange;
      this.step = { years: range };
    };
    this._refreshView = function () {
      var years = new Array(range), date;
      for (var i = 0, start = getStartingYear(this.activeDate.getFullYear()); i < range; i++) {
        date = new Date(start + i, 0, 1);
        this.fixTimeZone(date);
        years[i] = angular.extend(this.createDateObject(date, this.formatYear), { uid: scope.uniqueId + '-' + i });
      }
      scope.title = [
        years[0].label,
        years[range - 1].label
      ].join(' - ');
      scope.rows = this.split(years, 5);
    };
    this.compare = function (date1, date2) {
      return date1.getFullYear() - date2.getFullYear();
    };
    this.handleKeyDown = function (key, evt) {
      var date = this.activeDate.getFullYear();
      if (key === 'left') {
        date = date - 1;  // up
      } else if (key === 'up') {
        date = date - 5;  // down
      } else if (key === 'right') {
        date = date + 1;  // down
      } else if (key === 'down') {
        date = date + 5;
      } else if (key === 'pageup' || key === 'pagedown') {
        date += (key === 'pageup' ? -1 : 1) * this.step.years;
      } else if (key === 'home') {
        date = getStartingYear(this.activeDate.getFullYear());
      } else if (key === 'end') {
        date = getStartingYear(this.activeDate.getFullYear()) + range - 1;
      }
      this.activeDate.setFullYear(date);
    };
  }
]).directive('uibDatepicker', function () {
  return {
    replace: true,
    templateUrl: function (element, attrs) {
      return attrs.templateUrl || 'template/datepicker/datepicker.html';
    },
    scope: {
      datepickerMode: '=?',
      dateDisabled: '&',
      customClass: '&',
      shortcutPropagation: '&?'
    },
    require: [
      'uibDatepicker',
      '^ngModel'
    ],
    controller: 'UibDatepickerController',
    controllerAs: 'datepicker',
    link: function (scope, element, attrs, ctrls) {
      var datepickerCtrl = ctrls[0], ngModelCtrl = ctrls[1];
      datepickerCtrl.init(ngModelCtrl);
    }
  };
}).directive('uibDaypicker', function () {
  return {
    replace: true,
    templateUrl: 'template/datepicker/day.html',
    require: [
      '^?uibDatepicker',
      'uibDaypicker',
      '^?datepicker'
    ],
    controller: 'UibDaypickerController',
    link: function (scope, element, attrs, ctrls) {
      var datepickerCtrl = ctrls[0] || ctrls[2], daypickerCtrl = ctrls[1];
      daypickerCtrl.init(datepickerCtrl);
    }
  };
}).directive('uibMonthpicker', function () {
  return {
    replace: true,
    templateUrl: 'template/datepicker/month.html',
    require: [
      '^?uibDatepicker',
      'uibMonthpicker',
      '^?datepicker'
    ],
    controller: 'UibMonthpickerController',
    link: function (scope, element, attrs, ctrls) {
      var datepickerCtrl = ctrls[0] || ctrls[2], monthpickerCtrl = ctrls[1];
      monthpickerCtrl.init(datepickerCtrl);
    }
  };
}).directive('uibYearpicker', function () {
  return {
    replace: true,
    templateUrl: 'template/datepicker/year.html',
    require: [
      '^?uibDatepicker',
      'uibYearpicker',
      '^?datepicker'
    ],
    controller: 'UibYearpickerController',
    link: function (scope, element, attrs, ctrls) {
      var ctrl = ctrls[0] || ctrls[2];
      angular.extend(ctrl, ctrls[1]);
      ctrl.yearpickerInit();
      ctrl.refreshView();
    }
  };
}).constant('uibDatepickerPopupConfig', {
  datepickerPopup: 'yyyy-MM-dd',
  datepickerPopupTemplateUrl: 'template/datepicker/popup.html',
  datepickerTemplateUrl: 'template/datepicker/datepicker.html',
  html5Types: {
    date: 'yyyy-MM-dd',
    'datetime-local': 'yyyy-MM-ddTHH:mm:ss.sss',
    'month': 'yyyy-MM'
  },
  currentText: 'Today',
  clearText: 'Clear',
  closeText: 'Done',
  closeOnDateSelection: true,
  appendToBody: false,
  showButtonBar: true,
  onOpenFocus: true
}).controller('UibDatepickerPopupController', [
  '$scope',
  '$element',
  '$attrs',
  '$compile',
  '$parse',
  '$document',
  '$rootScope',
  '$uibPosition',
  'dateFilter',
  'uibDateParser',
  'uibDatepickerPopupConfig',
  '$timeout',
  function (scope, element, attrs, $compile, $parse, $document, $rootScope, $position, dateFilter, dateParser, datepickerPopupConfig, $timeout) {
    var self = this;
    var cache = {}, isHtml5DateInput = false;
    var dateFormat, closeOnDateSelection, appendToBody, onOpenFocus, datepickerPopupTemplateUrl, datepickerTemplateUrl, popupEl, datepickerEl, ngModel, $popup;
    scope.watchData = {};
    this.init = function (_ngModel_) {
      ngModel = _ngModel_;
      closeOnDateSelection = angular.isDefined(attrs.closeOnDateSelection) ? scope.$parent.$eval(attrs.closeOnDateSelection) : datepickerPopupConfig.closeOnDateSelection;
      appendToBody = angular.isDefined(attrs.datepickerAppendToBody) ? scope.$parent.$eval(attrs.datepickerAppendToBody) : datepickerPopupConfig.appendToBody;
      onOpenFocus = angular.isDefined(attrs.onOpenFocus) ? scope.$parent.$eval(attrs.onOpenFocus) : datepickerPopupConfig.onOpenFocus;
      datepickerPopupTemplateUrl = angular.isDefined(attrs.datepickerPopupTemplateUrl) ? attrs.datepickerPopupTemplateUrl : datepickerPopupConfig.datepickerPopupTemplateUrl;
      datepickerTemplateUrl = angular.isDefined(attrs.datepickerTemplateUrl) ? attrs.datepickerTemplateUrl : datepickerPopupConfig.datepickerTemplateUrl;
      scope.showButtonBar = angular.isDefined(attrs.showButtonBar) ? scope.$parent.$eval(attrs.showButtonBar) : datepickerPopupConfig.showButtonBar;
      if (datepickerPopupConfig.html5Types[attrs.type]) {
        dateFormat = datepickerPopupConfig.html5Types[attrs.type];
        isHtml5DateInput = true;
      } else {
        dateFormat = attrs.datepickerPopup || datepickerPopupConfig.datepickerPopup;
        attrs.$observe('uibDatepickerPopup', function (value, oldValue) {
          var newDateFormat = value || datepickerPopupConfig.datepickerPopup;
          // Invalidate the $modelValue to ensure that formatters re-run
          // FIXME: Refactor when PR is merged: https://github.com/angular/angular.js/pull/10764
          if (newDateFormat !== dateFormat) {
            dateFormat = newDateFormat;
            ngModel.$modelValue = null;
            if (!dateFormat) {
              throw new Error('uibDatepickerPopup must have a date format specified.');
            }
          }
        });
      }
      if (!dateFormat) {
        throw new Error('uibDatepickerPopup must have a date format specified.');
      }
      if (isHtml5DateInput && attrs.datepickerPopup) {
        throw new Error('HTML5 date input types do not support custom formats.');
      }
      // popup element used to display calendar
      popupEl = angular.element('<div uib-datepicker-popup-wrap><div uib-datepicker></div></div>');
      popupEl.attr({
        'ng-model': 'date',
        'ng-change': 'dateSelection(date)',
        'template-url': datepickerPopupTemplateUrl
      });
      // datepicker element
      datepickerEl = angular.element(popupEl.children()[0]);
      datepickerEl.attr('template-url', datepickerTemplateUrl);
      if (isHtml5DateInput) {
        if (attrs.type === 'month') {
          datepickerEl.attr('datepicker-mode', '"month"');
          datepickerEl.attr('min-mode', 'month');
        }
      }
      if (attrs.datepickerOptions) {
        var options = scope.$parent.$eval(attrs.datepickerOptions);
        if (options && options.initDate) {
          scope.initDate = options.initDate;
          datepickerEl.attr('init-date', 'initDate');
          delete options.initDate;
        }
        angular.forEach(options, function (value, option) {
          datepickerEl.attr(cameltoDash(option), value);
        });
      }
      angular.forEach([
        'minMode',
        'maxMode',
        'minDate',
        'maxDate',
        'datepickerMode',
        'initDate',
        'shortcutPropagation'
      ], function (key) {
        if (attrs[key]) {
          var getAttribute = $parse(attrs[key]);
          scope.$parent.$watch(getAttribute, function (value) {
            scope.watchData[key] = value;
            if (key === 'minDate' || key === 'maxDate') {
              cache[key] = new Date(value);
            }
          });
          datepickerEl.attr(cameltoDash(key), 'watchData.' + key);
          // Propagate changes from datepicker to outside
          if (key === 'datepickerMode') {
            var setAttribute = getAttribute.assign;
            scope.$watch('watchData.' + key, function (value, oldvalue) {
              if (angular.isFunction(setAttribute) && value !== oldvalue) {
                setAttribute(scope.$parent, value);
              }
            });
          }
        }
      });
      if (attrs.dateDisabled) {
        datepickerEl.attr('date-disabled', 'dateDisabled({ date: date, mode: mode })');
      }
      if (attrs.showWeeks) {
        datepickerEl.attr('show-weeks', attrs.showWeeks);
      }
      if (attrs.customClass) {
        datepickerEl.attr('custom-class', 'customClass({ date: date, mode: mode })');
      }
      if (!isHtml5DateInput) {
        // Internal API to maintain the correct ng-invalid-[key] class
        ngModel.$$parserName = 'date';
        ngModel.$validators.date = validator;
        ngModel.$parsers.unshift(parseDate);
        ngModel.$formatters.push(function (value) {
          scope.date = value;
          return ngModel.$isEmpty(value) ? value : dateFilter(value, dateFormat);
        });
      } else {
        ngModel.$formatters.push(function (value) {
          scope.date = value;
          return value;
        });
      }
      // Detect changes in the view from the text box
      ngModel.$viewChangeListeners.push(function () {
        scope.date = dateParser.parse(ngModel.$viewValue, dateFormat, scope.date);
      });
      element.bind('keydown', inputKeydownBind);
      $popup = $compile(popupEl)(scope);
      // Prevent jQuery cache memory leak (template is now redundant after linking)
      popupEl.remove();
      if (appendToBody) {
        $document.find('body').append($popup);
      } else {
        element.after($popup);
      }
      scope.$on('$destroy', function () {
        if (scope.isOpen === true) {
          if (!$rootScope.$$phase) {
            scope.$apply(function () {
              scope.isOpen = false;
            });
          }
        }
        $popup.remove();
        element.unbind('keydown', inputKeydownBind);
        $document.unbind('click', documentClickBind);
      });
    };
    scope.getText = function (key) {
      return scope[key + 'Text'] || datepickerPopupConfig[key + 'Text'];
    };
    scope.isDisabled = function (date) {
      if (date === 'today') {
        date = new Date();
      }
      return scope.watchData.minDate && scope.compare(date, cache.minDate) < 0 || scope.watchData.maxDate && scope.compare(date, cache.maxDate) > 0;
    };
    scope.compare = function (date1, date2) {
      return new Date(date1.getFullYear(), date1.getMonth(), date1.getDate()) - new Date(date2.getFullYear(), date2.getMonth(), date2.getDate());
    };
    // Inner change
    scope.dateSelection = function (dt) {
      if (angular.isDefined(dt)) {
        scope.date = dt;
      }
      var date = scope.date ? dateFilter(scope.date, dateFormat) : null;
      // Setting to NULL is necessary for form validators to function
      element.val(date);
      ngModel.$setViewValue(date);
      if (closeOnDateSelection) {
        scope.isOpen = false;
        element[0].focus();
      }
    };
    scope.keydown = function (evt) {
      if (evt.which === 27) {
        scope.isOpen = false;
        element[0].focus();
      }
    };
    scope.select = function (date) {
      if (date === 'today') {
        var today = new Date();
        if (angular.isDate(scope.date)) {
          date = new Date(scope.date);
          date.setFullYear(today.getFullYear(), today.getMonth(), today.getDate());
        } else {
          date = new Date(today.setHours(0, 0, 0, 0));
        }
      }
      scope.dateSelection(date);
    };
    scope.close = function () {
      scope.isOpen = false;
      element[0].focus();
    };
    scope.$watch('isOpen', function (value) {
      if (value) {
        scope.position = appendToBody ? $position.offset(element) : $position.position(element);
        scope.position.top = scope.position.top + element.prop('offsetHeight');
        $timeout(function () {
          if (onOpenFocus) {
            scope.$broadcast('uib:datepicker.focus');
          }
          $document.bind('click', documentClickBind);
        }, 0, false);
      } else {
        $document.unbind('click', documentClickBind);
      }
    });
    function cameltoDash(string) {
      return string.replace(/([A-Z])/g, function ($1) {
        return '-' + $1.toLowerCase();
      });
    }
    function parseDate(viewValue) {
      if (angular.isNumber(viewValue)) {
        // presumably timestamp to date object
        viewValue = new Date(viewValue);
      }
      if (!viewValue) {
        return null;
      } else if (angular.isDate(viewValue) && !isNaN(viewValue)) {
        return viewValue;
      } else if (angular.isString(viewValue)) {
        var date = dateParser.parse(viewValue, dateFormat, scope.date);
        if (isNaN(date)) {
          return undefined;
        } else {
          return date;
        }
      } else {
        return undefined;
      }
    }
    function validator(modelValue, viewValue) {
      var value = modelValue || viewValue;
      if (!attrs.ngRequired && !value) {
        return true;
      }
      if (angular.isNumber(value)) {
        value = new Date(value);
      }
      if (!value) {
        return true;
      } else if (angular.isDate(value) && !isNaN(value)) {
        return true;
      } else if (angular.isString(value)) {
        var date = dateParser.parse(value, dateFormat);
        return !isNaN(date);
      } else {
        return false;
      }
    }
    function documentClickBind(event) {
      var popup = $popup[0];
      var dpContainsTarget = element[0].contains(event.target);
      // The popup node may not be an element node
      // In some browsers (IE) only element nodes have the 'contains' function
      var popupContainsTarget = popup.contains !== undefined && popup.contains(event.target);
      if (scope.isOpen && !(dpContainsTarget || popupContainsTarget)) {
        scope.$apply(function () {
          scope.isOpen = false;
        });
      }
    }
    function inputKeydownBind(evt) {
      if (evt.which === 27 && scope.isOpen) {
        evt.preventDefault();
        evt.stopPropagation();
        scope.$apply(function () {
          scope.isOpen = false;
        });
        element[0].focus();
      } else if (evt.which === 40 && !scope.isOpen) {
        evt.preventDefault();
        evt.stopPropagation();
        scope.$apply(function () {
          scope.isOpen = true;
        });
      }
    }
  }
]).directive('uibDatepickerPopup', function () {
  return {
    require: [
      'ngModel',
      'uibDatepickerPopup'
    ],
    controller: 'UibDatepickerPopupController',
    scope: {
      isOpen: '=?',
      currentText: '@',
      clearText: '@',
      closeText: '@',
      dateDisabled: '&',
      customClass: '&'
    },
    link: function (scope, element, attrs, ctrls) {
      var ngModel = ctrls[0], ctrl = ctrls[1];
      ctrl.init(ngModel);
    }
  };
}).directive('uibDatepickerPopupWrap', function () {
  return {
    replace: true,
    transclude: true,
    templateUrl: function (element, attrs) {
      return attrs.templateUrl || 'template/datepicker/popup.html';
    }
  };
});
/* Deprecated datepicker below */
angular.module('ui.bootstrap.datepicker').value('$datepickerSuppressWarning', false).controller('DatepickerController', [
  '$scope',
  '$attrs',
  '$log',
  '$datepickerSuppressWarning',
  '$controller',
  function ($scope, $attrs, $log, $datepickerSuppressWarning, $controller) {
    if (!$datepickerSuppressWarning) {
      $log.warn('DatepickerController is now deprecated. Use UibDatepickerController instead.');
    }
    return $controller('UibDatepickerController', {
      $scope: $scope,
      $attrs: $attrs
    });
  }
]).directive('datepicker', [
  '$log',
  '$datepickerSuppressWarning',
  function ($log, $datepickerSuppressWarning) {
    return {
      replace: true,
      templateUrl: function (element, attrs) {
        return attrs.templateUrl || 'template/datepicker/datepicker.html';
      },
      scope: {
        datepickerMode: '=?',
        dateDisabled: '&',
        customClass: '&',
        shortcutPropagation: '&?'
      },
      require: [
        'datepicker',
        '^ngModel'
      ],
      controller: 'DatepickerController',
      controllerAs: 'datepicker',
      link: function (scope, element, attrs, ctrls) {
        if (!$datepickerSuppressWarning) {
          $log.warn('datepicker is now deprecated. Use uib-datepicker instead.');
        }
        var datepickerCtrl = ctrls[0], ngModelCtrl = ctrls[1];
        datepickerCtrl.init(ngModelCtrl);
      }
    };
  }
]).directive('daypicker', [
  '$log',
  '$datepickerSuppressWarning',
  function ($log, $datepickerSuppressWarning) {
    return {
      replace: true,
      templateUrl: 'template/datepicker/day.html',
      require: [
        '^datepicker',
        'daypicker'
      ],
      controller: 'UibDaypickerController',
      link: function (scope, element, attrs, ctrls) {
        if (!$datepickerSuppressWarning) {
          $log.warn('daypicker is now deprecated. Use uib-daypicker instead.');
        }
        var datepickerCtrl = ctrls[0], daypickerCtrl = ctrls[1];
        daypickerCtrl.init(datepickerCtrl);
      }
    };
  }
]).directive('monthpicker', [
  '$log',
  '$datepickerSuppressWarning',
  function ($log, $datepickerSuppressWarning) {
    return {
      replace: true,
      templateUrl: 'template/datepicker/month.html',
      require: [
        '^datepicker',
        'monthpicker'
      ],
      controller: 'UibMonthpickerController',
      link: function (scope, element, attrs, ctrls) {
        if (!$datepickerSuppressWarning) {
          $log.warn('monthpicker is now deprecated. Use uib-monthpicker instead.');
        }
        var datepickerCtrl = ctrls[0], monthpickerCtrl = ctrls[1];
        monthpickerCtrl.init(datepickerCtrl);
      }
    };
  }
]).directive('yearpicker', [
  '$log',
  '$datepickerSuppressWarning',
  function ($log, $datepickerSuppressWarning) {
    return {
      replace: true,
      templateUrl: 'template/datepicker/year.html',
      require: [
        '^datepicker',
        'yearpicker'
      ],
      controller: 'UibYearpickerController',
      link: function (scope, element, attrs, ctrls) {
        if (!$datepickerSuppressWarning) {
          $log.warn('yearpicker is now deprecated. Use uib-yearpicker instead.');
        }
        var ctrl = ctrls[0];
        angular.extend(ctrl, ctrls[1]);
        ctrl.yearpickerInit();
        ctrl.refreshView();
      }
    };
  }
]).directive('datepickerPopup', [
  '$log',
  '$datepickerSuppressWarning',
  function ($log, $datepickerSuppressWarning) {
    return {
      require: [
        'ngModel',
        'datepickerPopup'
      ],
      controller: 'UibDatepickerPopupController',
      scope: {
        isOpen: '=?',
        currentText: '@',
        clearText: '@',
        closeText: '@',
        dateDisabled: '&',
        customClass: '&'
      },
      link: function (scope, element, attrs, ctrls) {
        if (!$datepickerSuppressWarning) {
          $log.warn('datepicker-popup is now deprecated. Use uib-datepicker-popup instead.');
        }
        var ngModel = ctrls[0], ctrl = ctrls[1];
        ctrl.init(ngModel);
      }
    };
  }
]).directive('datepickerPopupWrap', [
  '$log',
  '$datepickerSuppressWarning',
  function ($log, $datepickerSuppressWarning) {
    return {
      replace: true,
      transclude: true,
      templateUrl: function (element, attrs) {
        return attrs.templateUrl || 'template/datepicker/popup.html';
      },
      link: function () {
        if (!$datepickerSuppressWarning) {
          $log.warn('datepicker-popup-wrap is now deprecated. Use uib-datepicker-popup-wrap instead.');
        }
      }
    };
  }
]);
angular.module('ui.bootstrap.dropdown', ['ui.bootstrap.position']).constant('uibDropdownConfig', { openClass: 'open' }).service('uibDropdownService', [
  '$document',
  '$rootScope',
  function ($document, $rootScope) {
    var openScope = null;
    this.open = function (dropdownScope) {
      if (!openScope) {
        $document.bind('click', closeDropdown);
        $document.bind('keydown', keybindFilter);
      }
      if (openScope && openScope !== dropdownScope) {
        openScope.isOpen = false;
      }
      openScope = dropdownScope;
    };
    this.close = function (dropdownScope) {
      if (openScope === dropdownScope) {
        openScope = null;
        $document.unbind('click', closeDropdown);
        $document.unbind('keydown', keybindFilter);
      }
    };
    var closeDropdown = function (evt) {
      // This method may still be called during the same mouse event that
      // unbound this event handler. So check openScope before proceeding.
      if (!openScope) {
        return;
      }
      if (evt && openScope.getAutoClose() === 'disabled') {
        return;
      }
      var toggleElement = openScope.getToggleElement();
      if (evt && toggleElement && toggleElement[0].contains(evt.target)) {
        return;
      }
      var dropdownElement = openScope.getDropdownElement();
      if (evt && openScope.getAutoClose() === 'outsideClick' && dropdownElement && dropdownElement[0].contains(evt.target)) {
        return;
      }
      openScope.isOpen = false;
      if (!$rootScope.$$phase) {
        openScope.$apply();
      }
    };
    var keybindFilter = function (evt) {
      if (evt.which === 27) {
        openScope.focusToggleElement();
        closeDropdown();
      } else if (openScope.isKeynavEnabled() && /(38|40)/.test(evt.which) && openScope.isOpen) {
        evt.preventDefault();
        evt.stopPropagation();
        openScope.focusDropdownEntry(evt.which);
      }
    };
  }
]).controller('UibDropdownController', [
  '$scope',
  '$element',
  '$attrs',
  '$parse',
  'uibDropdownConfig',
  'uibDropdownService',
  '$animate',
  '$uibPosition',
  '$document',
  '$compile',
  '$templateRequest',
  function ($scope, $element, $attrs, $parse, dropdownConfig, uibDropdownService, $animate, $position, $document, $compile, $templateRequest) {
    var self = this, scope = $scope.$new(),
      // create a child scope so we are not polluting original one
      templateScope, openClass = dropdownConfig.openClass, getIsOpen, setIsOpen = angular.noop, toggleInvoker = $attrs.onToggle ? $parse($attrs.onToggle) : angular.noop, appendToBody = false, keynavEnabled = false, selectedOption = null;
    $element.addClass('dropdown');
    this.init = function () {
      if ($attrs.isOpen) {
        getIsOpen = $parse($attrs.isOpen);
        setIsOpen = getIsOpen.assign;
        $scope.$watch(getIsOpen, function (value) {
          scope.isOpen = !!value;
        });
      }
      appendToBody = angular.isDefined($attrs.dropdownAppendToBody);
      keynavEnabled = angular.isDefined($attrs.uibKeyboardNav);
      if (appendToBody && self.dropdownMenu) {
        $document.find('body').append(self.dropdownMenu);
        $element.on('$destroy', function handleDestroyEvent() {
          self.dropdownMenu.remove();
        });
      }
    };
    this.toggle = function (open) {
      return scope.isOpen = arguments.length ? !!open : !scope.isOpen;
    };
    // Allow other directives to watch status
    this.isOpen = function () {
      return scope.isOpen;
    };
    scope.getToggleElement = function () {
      return self.toggleElement;
    };
    scope.getAutoClose = function () {
      return $attrs.autoClose || 'always';  //or 'outsideClick' or 'disabled'
    };
    scope.getElement = function () {
      return $element;
    };
    scope.isKeynavEnabled = function () {
      return keynavEnabled;
    };
    scope.focusDropdownEntry = function (keyCode) {
      var elems = self.dropdownMenu ? angular.element(self.dropdownMenu).find('a') : angular.element($element).find('ul').eq(0).find('a');
      switch (keyCode) {
      case 40: {
          if (!angular.isNumber(self.selectedOption)) {
            self.selectedOption = 0;
          } else {
            self.selectedOption = self.selectedOption === elems.length - 1 ? self.selectedOption : self.selectedOption + 1;
          }
          break;
        }
      case 38: {
          if (!angular.isNumber(self.selectedOption)) {
            self.selectedOption = elems.length - 1;
          } else {
            self.selectedOption = self.selectedOption === 0 ? 0 : self.selectedOption - 1;
          }
          break;
        }
      }
      elems[self.selectedOption].focus();
    };
    scope.getDropdownElement = function () {
      return self.dropdownMenu;
    };
    scope.focusToggleElement = function () {
      if (self.toggleElement) {
        self.toggleElement[0].focus();
      }
    };
    scope.$watch('isOpen', function (isOpen, wasOpen) {
      if (appendToBody && self.dropdownMenu) {
        var pos = $position.positionElements($element, self.dropdownMenu, 'bottom-left', true);
        var css = {
            top: pos.top + 'px',
            display: isOpen ? 'block' : 'none'
          };
        var rightalign = self.dropdownMenu.hasClass('dropdown-menu-right');
        if (!rightalign) {
          css.left = pos.left + 'px';
          css.right = 'auto';
        } else {
          css.left = 'auto';
          css.right = window.innerWidth - (pos.left + $element.prop('offsetWidth')) + 'px';
        }
        self.dropdownMenu.css(css);
      }
      $animate[isOpen ? 'addClass' : 'removeClass']($element, openClass).then(function () {
        if (angular.isDefined(isOpen) && isOpen !== wasOpen) {
          toggleInvoker($scope, { open: !!isOpen });
        }
      });
      if (isOpen) {
        if (self.dropdownMenuTemplateUrl) {
          $templateRequest(self.dropdownMenuTemplateUrl).then(function (tplContent) {
            templateScope = scope.$new();
            $compile(tplContent.trim())(templateScope, function (dropdownElement) {
              var newEl = dropdownElement;
              self.dropdownMenu.replaceWith(newEl);
              self.dropdownMenu = newEl;
            });
          });
        }
        scope.focusToggleElement();
        uibDropdownService.open(scope);
      } else {
        if (self.dropdownMenuTemplateUrl) {
          if (templateScope) {
            templateScope.$destroy();
          }
          var newEl = angular.element('<ul class="dropdown-menu"></ul>');
          self.dropdownMenu.replaceWith(newEl);
          self.dropdownMenu = newEl;
        }
        uibDropdownService.close(scope);
        self.selectedOption = null;
      }
      if (angular.isFunction(setIsOpen)) {
        setIsOpen($scope, isOpen);
      }
    });
    $scope.$on('$locationChangeSuccess', function () {
      if (scope.getAutoClose() !== 'disabled') {
        scope.isOpen = false;
      }
    });
    var offDestroy = $scope.$on('$destroy', function () {
        scope.$destroy();
      });
    scope.$on('$destroy', offDestroy);
  }
]).directive('uibDropdown', function () {
  return {
    controller: 'UibDropdownController',
    link: function (scope, element, attrs, dropdownCtrl) {
      dropdownCtrl.init();
    }
  };
}).directive('uibDropdownMenu', function () {
  return {
    restrict: 'AC',
    require: '?^uibDropdown',
    link: function (scope, element, attrs, dropdownCtrl) {
      if (!dropdownCtrl || angular.isDefined(attrs.dropdownNested)) {
        return;
      }
      element.addClass('dropdown-menu');
      var tplUrl = attrs.templateUrl;
      if (tplUrl) {
        dropdownCtrl.dropdownMenuTemplateUrl = tplUrl;
      }
      if (!dropdownCtrl.dropdownMenu) {
        dropdownCtrl.dropdownMenu = element;
      }
    }
  };
}).directive('uibKeyboardNav', function () {
  return {
    restrict: 'A',
    require: '?^uibDropdown',
    link: function (scope, element, attrs, dropdownCtrl) {
      element.bind('keydown', function (e) {
        if ([
            38,
            40
          ].indexOf(e.which) !== -1) {
          e.preventDefault();
          e.stopPropagation();
          var elems = dropdownCtrl.dropdownMenu.find('a');
          switch (e.which) {
          case 40: {
              // Down
              if (!angular.isNumber(dropdownCtrl.selectedOption)) {
                dropdownCtrl.selectedOption = 0;
              } else {
                dropdownCtrl.selectedOption = dropdownCtrl.selectedOption === elems.length - 1 ? dropdownCtrl.selectedOption : dropdownCtrl.selectedOption + 1;
              }
              break;
            }
          case 38: {
              // Up
              if (!angular.isNumber(dropdownCtrl.selectedOption)) {
                dropdownCtrl.selectedOption = elems.length - 1;
              } else {
                dropdownCtrl.selectedOption = dropdownCtrl.selectedOption === 0 ? 0 : dropdownCtrl.selectedOption - 1;
              }
              break;
            }
          }
          elems[dropdownCtrl.selectedOption].focus();
        }
      });
    }
  };
}).directive('uibDropdownToggle', function () {
  return {
    require: '?^uibDropdown',
    link: function (scope, element, attrs, dropdownCtrl) {
      if (!dropdownCtrl) {
        return;
      }
      element.addClass('dropdown-toggle');
      dropdownCtrl.toggleElement = element;
      var toggleDropdown = function (event) {
        event.preventDefault();
        if (!element.hasClass('disabled') && !attrs.disabled) {
          scope.$apply(function () {
            dropdownCtrl.toggle();
          });
        }
      };
      element.bind('click', toggleDropdown);
      // WAI-ARIA
      element.attr({
        'aria-haspopup': true,
        'aria-expanded': false
      });
      scope.$watch(dropdownCtrl.isOpen, function (isOpen) {
        element.attr('aria-expanded', !!isOpen);
      });
      scope.$on('$destroy', function () {
        element.unbind('click', toggleDropdown);
      });
    }
  };
});
/* Deprecated dropdown below */
angular.module('ui.bootstrap.dropdown').value('$dropdownSuppressWarning', false).service('dropdownService', [
  '$log',
  '$dropdownSuppressWarning',
  'uibDropdownService',
  function ($log, $dropdownSuppressWarning, uibDropdownService) {
    if (!$dropdownSuppressWarning) {
      $log.warn('dropdownService is now deprecated. Use uibDropdownService instead.');
    }
    angular.extend(this, uibDropdownService);
  }
]).controller('DropdownController', [
  '$scope',
  '$element',
  '$attrs',
  '$log',
  '$dropdownSuppressWarning',
  '$controller',
  function ($scope, $element, $attrs, $log, $dropdownSuppressWarning, $controller) {
    if (!$dropdownSuppressWarning) {
      $log.warn('DropdownController is now deprecated. Use UibDropdownController instead.');
    }
    return $controller('UibDropdownController', {
      $scope: $scope,
      $element: $element,
      $attrs: $attrs
    });
  }
]).directive('dropdown', [
  '$log',
  '$dropdownSuppressWarning',
  function ($log, $dropdownSuppressWarning) {
    return {
      controller: 'DropdownController',
      link: function (scope, element, attrs, dropdownCtrl) {
        if (!$dropdownSuppressWarning) {
          $log.warn('dropdown is now deprecated. Use uib-dropdown instead.');
        }
        dropdownCtrl.init();
      }
    };
  }
]).directive('dropdownMenu', [
  '$log',
  '$dropdownSuppressWarning',
  function ($log, $dropdownSuppressWarning) {
    return {
      restrict: 'AC',
      require: '?^dropdown',
      link: function (scope, element, attrs, dropdownCtrl) {
        if (!dropdownCtrl) {
          return;
        }
        if (!$dropdownSuppressWarning) {
          $log.warn('dropdown-menu is now deprecated. Use uib-dropdown-menu instead.');
        }
        element.addClass('dropdown-menu');
        var tplUrl = attrs.templateUrl;
        if (tplUrl) {
          dropdownCtrl.dropdownMenuTemplateUrl = tplUrl;
        }
        if (!dropdownCtrl.dropdownMenu) {
          dropdownCtrl.dropdownMenu = element;
        }
      }
    };
  }
]).directive('keyboardNav', [
  '$log',
  '$dropdownSuppressWarning',
  function ($log, $dropdownSuppressWarning) {
    return {
      restrict: 'A',
      require: '?^dropdown',
      link: function (scope, element, attrs, dropdownCtrl) {
        if (!$dropdownSuppressWarning) {
          $log.warn('keyboard-nav is now deprecated. Use uib-keyboard-nav instead.');
        }
        element.bind('keydown', function (e) {
          if ([
              38,
              40
            ].indexOf(e.which) !== -1) {
            e.preventDefault();
            e.stopPropagation();
            var elems = dropdownCtrl.dropdownMenu.find('a');
            switch (e.which) {
            case 40: {
                // Down
                if (!angular.isNumber(dropdownCtrl.selectedOption)) {
                  dropdownCtrl.selectedOption = 0;
                } else {
                  dropdownCtrl.selectedOption = dropdownCtrl.selectedOption === elems.length - 1 ? dropdownCtrl.selectedOption : dropdownCtrl.selectedOption + 1;
                }
                break;
              }
            case 38: {
                // Up
                if (!angular.isNumber(dropdownCtrl.selectedOption)) {
                  dropdownCtrl.selectedOption = elems.length - 1;
                } else {
                  dropdownCtrl.selectedOption = dropdownCtrl.selectedOption === 0 ? 0 : dropdownCtrl.selectedOption - 1;
                }
                break;
              }
            }
            elems[dropdownCtrl.selectedOption].focus();
          }
        });
      }
    };
  }
]).directive('dropdownToggle', [
  '$log',
  '$dropdownSuppressWarning',
  function ($log, $dropdownSuppressWarning) {
    return {
      require: '?^dropdown',
      link: function (scope, element, attrs, dropdownCtrl) {
        if (!$dropdownSuppressWarning) {
          $log.warn('dropdown-toggle is now deprecated. Use uib-dropdown-toggle instead.');
        }
        if (!dropdownCtrl) {
          return;
        }
        element.addClass('dropdown-toggle');
        dropdownCtrl.toggleElement = element;
        var toggleDropdown = function (event) {
          event.preventDefault();
          if (!element.hasClass('disabled') && !attrs.disabled) {
            scope.$apply(function () {
              dropdownCtrl.toggle();
            });
          }
        };
        element.bind('click', toggleDropdown);
        // WAI-ARIA
        element.attr({
          'aria-haspopup': true,
          'aria-expanded': false
        });
        scope.$watch(dropdownCtrl.isOpen, function (isOpen) {
          element.attr('aria-expanded', !!isOpen);
        });
        scope.$on('$destroy', function () {
          element.unbind('click', toggleDropdown);
        });
      }
    };
  }
]);
angular.module('ui.bootstrap.stackedMap', []).factory('$$stackedMap', function () {
  return {
    createNew: function () {
      var stack = [];
      return {
        add: function (key, value) {
          stack.push({
            key: key,
            value: value
          });
        },
        get: function (key) {
          for (var i = 0; i < stack.length; i++) {
            if (key == stack[i].key) {
              return stack[i];
            }
          }
        },
        keys: function () {
          var keys = [];
          for (var i = 0; i < stack.length; i++) {
            keys.push(stack[i].key);
          }
          return keys;
        },
        top: function () {
          return stack[stack.length - 1];
        },
        remove: function (key) {
          var idx = -1;
          for (var i = 0; i < stack.length; i++) {
            if (key == stack[i].key) {
              idx = i;
              break;
            }
          }
          return stack.splice(idx, 1)[0];
        },
        removeTop: function () {
          return stack.splice(stack.length - 1, 1)[0];
        },
        length: function () {
          return stack.length;
        }
      };
    }
  };
});
angular.module('ui.bootstrap.modal', ['ui.bootstrap.stackedMap']).factory('$$multiMap', function () {
  return {
    createNew: function () {
      var map = {};
      return {
        entries: function () {
          return Object.keys(map).map(function (key) {
            return {
              key: key,
              value: map[key]
            };
          });
        },
        get: function (key) {
          return map[key];
        },
        hasKey: function (key) {
          return !!map[key];
        },
        keys: function () {
          return Object.keys(map);
        },
        put: function (key, value) {
          if (!map[key]) {
            map[key] = [];
          }
          map[key].push(value);
        },
        remove: function (key, value) {
          var values = map[key];
          if (!values) {
            return;
          }
          var idx = values.indexOf(value);
          if (idx !== -1) {
            values.splice(idx, 1);
          }
          if (!values.length) {
            delete map[key];
          }
        }
      };
    }
  };
}).directive('uibModalBackdrop', [
  '$animate',
  '$injector',
  '$uibModalStack',
  function ($animate, $injector, $modalStack) {
    var $animateCss = null;
    if ($injector.has('$animateCss')) {
      $animateCss = $injector.get('$animateCss');
    }
    return {
      replace: true,
      templateUrl: 'template/modal/backdrop.html',
      compile: function (tElement, tAttrs) {
        tElement.addClass(tAttrs.backdropClass);
        return linkFn;
      }
    };
    function linkFn(scope, element, attrs) {
      // Temporary fix for prefixing
      element.addClass('modal-backdrop');
      if (attrs.modalInClass) {
        if ($animateCss) {
          $animateCss(element, { addClass: attrs.modalInClass }).start();
        } else {
          $animate.addClass(element, attrs.modalInClass);
        }
        scope.$on($modalStack.NOW_CLOSING_EVENT, function (e, setIsAsync) {
          var done = setIsAsync();
          if ($animateCss) {
            $animateCss(element, { removeClass: attrs.modalInClass }).start().then(done);
          } else {
            $animate.removeClass(element, attrs.modalInClass).then(done);
          }
        });
      }
    }
  }
]).directive('uibModalWindow', [
  '$uibModalStack',
  '$q',
  '$animate',
  '$injector',
  function ($modalStack, $q, $animate, $injector) {
    var $animateCss = null;
    if ($injector.has('$animateCss')) {
      $animateCss = $injector.get('$animateCss');
    }
    return {
      scope: { index: '@' },
      replace: true,
      transclude: true,
      templateUrl: function (tElement, tAttrs) {
        return tAttrs.templateUrl || 'template/modal/window.html';
      },
      link: function (scope, element, attrs) {
        element.addClass(attrs.windowClass || '');
        element.addClass(attrs.windowTopClass || '');
        scope.size = attrs.size;
        scope.close = function (evt) {
          var modal = $modalStack.getTop();
          if (modal && modal.value.backdrop && modal.value.backdrop !== 'static' && evt.target === evt.currentTarget) {
            evt.preventDefault();
            evt.stopPropagation();
            $modalStack.dismiss(modal.key, 'backdrop click');
          }
        };
        // moved from template to fix issue #2280
        element.on('click', scope.close);
        // This property is only added to the scope for the purpose of detecting when this directive is rendered.
        // We can detect that by using this property in the template associated with this directive and then use
        // {@link Attribute#$observe} on it. For more details please see {@link TableColumnResize}.
        scope.$isRendered = true;
        // Deferred object that will be resolved when this modal is render.
        var modalRenderDeferObj = $q.defer();
        // Observe function will be called on next digest cycle after compilation, ensuring that the DOM is ready.
        // In order to use this way of finding whether DOM is ready, we need to observe a scope property used in modal's template.
        attrs.$observe('modalRender', function (value) {
          if (value == 'true') {
            modalRenderDeferObj.resolve();
          }
        });
        modalRenderDeferObj.promise.then(function () {
          var animationPromise = null;
          if (attrs.modalInClass) {
            if ($animateCss) {
              animationPromise = $animateCss(element, { addClass: attrs.modalInClass }).start();
            } else {
              animationPromise = $animate.addClass(element, attrs.modalInClass);
            }
            scope.$on($modalStack.NOW_CLOSING_EVENT, function (e, setIsAsync) {
              var done = setIsAsync();
              if ($animateCss) {
                $animateCss(element, { removeClass: attrs.modalInClass }).start().then(done);
              } else {
                $animate.removeClass(element, attrs.modalInClass).then(done);
              }
            });
          }
          $q.when(animationPromise).then(function () {
            var inputsWithAutofocus = element[0].querySelectorAll('[autofocus]');
            /**
             * Auto-focusing of a freshly-opened modal element causes any child elements
             * with the autofocus attribute to lose focus. This is an issue on touch
             * based devices which will show and then hide the onscreen keyboard.
             * Attempts to refocus the autofocus element via JavaScript will not reopen
             * the onscreen keyboard. Fixed by updated the focusing logic to only autofocus
             * the modal element if the modal does not contain an autofocus element.
             */
            if (inputsWithAutofocus.length) {
              inputsWithAutofocus[0].focus();
            } else {
              element[0].focus();
            }
          });
          // Notify {@link $modalStack} that modal is rendered.
          var modal = $modalStack.getTop();
          if (modal) {
            $modalStack.modalRendered(modal.key);
          }
        });
      }
    };
  }
]).directive('uibModalAnimationClass', [function () {
    return {
      compile: function (tElement, tAttrs) {
        if (tAttrs.modalAnimation) {
          tElement.addClass(tAttrs.uibModalAnimationClass);
        }
      }
    };
  }]).directive('uibModalTransclude', function () {
  return {
    link: function ($scope, $element, $attrs, controller, $transclude) {
      $transclude($scope.$parent, function (clone) {
        $element.empty();
        $element.append(clone);
      });
    }
  };
}).factory('$uibModalStack', [
  '$animate',
  '$timeout',
  '$document',
  '$compile',
  '$rootScope',
  '$q',
  '$injector',
  '$$multiMap',
  '$$stackedMap',
  function ($animate, $timeout, $document, $compile, $rootScope, $q, $injector, $$multiMap, $$stackedMap) {
    var $animateCss = null;
    if ($injector.has('$animateCss')) {
      $animateCss = $injector.get('$animateCss');
    }
    var OPENED_MODAL_CLASS = 'modal-open';
    var backdropDomEl, backdropScope;
    var openedWindows = $$stackedMap.createNew();
    var openedClasses = $$multiMap.createNew();
    var $modalStack = { NOW_CLOSING_EVENT: 'modal.stack.now-closing' };
    //Modal focus behavior
    var focusableElementList;
    var focusIndex = 0;
    var tababbleSelector = 'a[href], area[href], input:not([disabled]), ' + 'button:not([disabled]),select:not([disabled]), textarea:not([disabled]), ' + 'iframe, object, embed, *[tabindex], *[contenteditable=true]';
    function backdropIndex() {
      var topBackdropIndex = -1;
      var opened = openedWindows.keys();
      for (var i = 0; i < opened.length; i++) {
        if (openedWindows.get(opened[i]).value.backdrop) {
          topBackdropIndex = i;
        }
      }
      return topBackdropIndex;
    }
    $rootScope.$watch(backdropIndex, function (newBackdropIndex) {
      if (backdropScope) {
        backdropScope.index = newBackdropIndex;
      }
    });
    function removeModalWindow(modalInstance, elementToReceiveFocus) {
      var body = $document.find('body').eq(0);
      var modalWindow = openedWindows.get(modalInstance).value;
      //clean up the stack
      openedWindows.remove(modalInstance);
      removeAfterAnimate(modalWindow.modalDomEl, modalWindow.modalScope, function () {
        var modalBodyClass = modalWindow.openedClass || OPENED_MODAL_CLASS;
        openedClasses.remove(modalBodyClass, modalInstance);
        body.toggleClass(modalBodyClass, openedClasses.hasKey(modalBodyClass));
        toggleTopWindowClass(true);
      });
      checkRemoveBackdrop();
      //move focus to specified element if available, or else to body
      if (elementToReceiveFocus && elementToReceiveFocus.focus) {
        elementToReceiveFocus.focus();
      } else {
        body.focus();
      }
    }
    // Add or remove "windowTopClass" from the top window in the stack
    function toggleTopWindowClass(toggleSwitch) {
      var modalWindow;
      if (openedWindows.length() > 0) {
        modalWindow = openedWindows.top().value;
        modalWindow.modalDomEl.toggleClass(modalWindow.windowTopClass || '', toggleSwitch);
      }
    }
    function checkRemoveBackdrop() {
      //remove backdrop if no longer needed
      if (backdropDomEl && backdropIndex() == -1) {
        var backdropScopeRef = backdropScope;
        removeAfterAnimate(backdropDomEl, backdropScope, function () {
          backdropScopeRef = null;
        });
        backdropDomEl = undefined;
        backdropScope = undefined;
      }
    }
    function removeAfterAnimate(domEl, scope, done) {
      var asyncDeferred;
      var asyncPromise = null;
      var setIsAsync = function () {
        if (!asyncDeferred) {
          asyncDeferred = $q.defer();
          asyncPromise = asyncDeferred.promise;
        }
        return function asyncDone() {
          asyncDeferred.resolve();
        };
      };
      scope.$broadcast($modalStack.NOW_CLOSING_EVENT, setIsAsync);
      // Note that it's intentional that asyncPromise might be null.
      // That's when setIsAsync has not been called during the
      // NOW_CLOSING_EVENT broadcast.
      return $q.when(asyncPromise).then(afterAnimating);
      function afterAnimating() {
        if (afterAnimating.done) {
          return;
        }
        afterAnimating.done = true;
        if ($animateCss) {
          $animateCss(domEl, { event: 'leave' }).start().then(function () {
            domEl.remove();
          });
        } else {
          $animate.leave(domEl);
        }
        scope.$destroy();
        if (done) {
          done();
        }
      }
    }
    $document.bind('keydown', function (evt) {
      if (evt.isDefaultPrevented()) {
        return evt;
      }
      var modal = openedWindows.top();
      if (modal && modal.value.keyboard) {
        switch (evt.which) {
        case 27: {
            evt.preventDefault();
            $rootScope.$apply(function () {
              $modalStack.dismiss(modal.key, 'escape key press');
            });
            break;
          }
        case 9: {
            $modalStack.loadFocusElementList(modal);
            var focusChanged = false;
            if (evt.shiftKey) {
              if ($modalStack.isFocusInFirstItem(evt)) {
                focusChanged = $modalStack.focusLastFocusableElement();
              }
            } else {
              if ($modalStack.isFocusInLastItem(evt)) {
                focusChanged = $modalStack.focusFirstFocusableElement();
              }
            }
            if (focusChanged) {
              evt.preventDefault();
              evt.stopPropagation();
            }
            break;
          }
        }
      }
    });
    $modalStack.open = function (modalInstance, modal) {
      var modalOpener = $document[0].activeElement, modalBodyClass = modal.openedClass || OPENED_MODAL_CLASS;
      toggleTopWindowClass(false);
      openedWindows.add(modalInstance, {
        deferred: modal.deferred,
        renderDeferred: modal.renderDeferred,
        modalScope: modal.scope,
        backdrop: modal.backdrop,
        keyboard: modal.keyboard,
        openedClass: modal.openedClass,
        windowTopClass: modal.windowTopClass
      });
      openedClasses.put(modalBodyClass, modalInstance);
      var body = $document.find('body').eq(0), currBackdropIndex = backdropIndex();
      if (currBackdropIndex >= 0 && !backdropDomEl) {
        backdropScope = $rootScope.$new(true);
        backdropScope.index = currBackdropIndex;
        var angularBackgroundDomEl = angular.element('<div uib-modal-backdrop="modal-backdrop"></div>');
        angularBackgroundDomEl.attr('backdrop-class', modal.backdropClass);
        if (modal.animation) {
          angularBackgroundDomEl.attr('modal-animation', 'true');
        }
        backdropDomEl = $compile(angularBackgroundDomEl)(backdropScope);
        body.append(backdropDomEl);
      }
      var angularDomEl = angular.element('<div uib-modal-window="modal-window"></div>');
      angularDomEl.attr({
        'template-url': modal.windowTemplateUrl,
        'window-class': modal.windowClass,
        'window-top-class': modal.windowTopClass,
        'size': modal.size,
        'index': openedWindows.length() - 1,
        'animate': 'animate'
      }).html(modal.content);
      if (modal.animation) {
        angularDomEl.attr('modal-animation', 'true');
      }
      var modalDomEl = $compile(angularDomEl)(modal.scope);
      openedWindows.top().value.modalDomEl = modalDomEl;
      openedWindows.top().value.modalOpener = modalOpener;
      body.append(modalDomEl);
      body.addClass(modalBodyClass);
      $modalStack.clearFocusListCache();
    };
    function broadcastClosing(modalWindow, resultOrReason, closing) {
      return !modalWindow.value.modalScope.$broadcast('modal.closing', resultOrReason, closing).defaultPrevented;
    }
    $modalStack.close = function (modalInstance, result) {
      var modalWindow = openedWindows.get(modalInstance);
      if (modalWindow && broadcastClosing(modalWindow, result, true)) {
        modalWindow.value.modalScope.$$uibDestructionScheduled = true;
        modalWindow.value.deferred.resolve(result);
        removeModalWindow(modalInstance, modalWindow.value.modalOpener);
        return true;
      }
      return !modalWindow;
    };
    $modalStack.dismiss = function (modalInstance, reason) {
      var modalWindow = openedWindows.get(modalInstance);
      if (modalWindow && broadcastClosing(modalWindow, reason, false)) {
        modalWindow.value.modalScope.$$uibDestructionScheduled = true;
        modalWindow.value.deferred.reject(reason);
        removeModalWindow(modalInstance, modalWindow.value.modalOpener);
        return true;
      }
      return !modalWindow;
    };
    $modalStack.dismissAll = function (reason) {
      var topModal = this.getTop();
      while (topModal && this.dismiss(topModal.key, reason)) {
        topModal = this.getTop();
      }
    };
    $modalStack.getTop = function () {
      return openedWindows.top();
    };
    $modalStack.modalRendered = function (modalInstance) {
      var modalWindow = openedWindows.get(modalInstance);
      if (modalWindow) {
        modalWindow.value.renderDeferred.resolve();
      }
    };
    $modalStack.focusFirstFocusableElement = function () {
      if (focusableElementList.length > 0) {
        focusableElementList[0].focus();
        return true;
      }
      return false;
    };
    $modalStack.focusLastFocusableElement = function () {
      if (focusableElementList.length > 0) {
        focusableElementList[focusableElementList.length - 1].focus();
        return true;
      }
      return false;
    };
    $modalStack.isFocusInFirstItem = function (evt) {
      if (focusableElementList.length > 0) {
        return (evt.target || evt.srcElement) == focusableElementList[0];
      }
      return false;
    };
    $modalStack.isFocusInLastItem = function (evt) {
      if (focusableElementList.length > 0) {
        return (evt.target || evt.srcElement) == focusableElementList[focusableElementList.length - 1];
      }
      return false;
    };
    $modalStack.clearFocusListCache = function () {
      focusableElementList = [];
      focusIndex = 0;
    };
    $modalStack.loadFocusElementList = function (modalWindow) {
      if (focusableElementList === undefined || !focusableElementList.length0) {
        if (modalWindow) {
          var modalDomE1 = modalWindow.value.modalDomEl;
          if (modalDomE1 && modalDomE1.length) {
            focusableElementList = modalDomE1[0].querySelectorAll(tababbleSelector);
          }
        }
      }
    };
    return $modalStack;
  }
]).provider('$uibModal', function () {
  var $modalProvider = {
      options: {
        animation: true,
        backdrop: true,
        keyboard: true
      },
      $get: [
        '$injector',
        '$rootScope',
        '$q',
        '$templateRequest',
        '$controller',
        '$uibModalStack',
        function ($injector, $rootScope, $q, $templateRequest, $controller, $modalStack) {
          var $modal = {};
          function getTemplatePromise(options) {
            return options.template ? $q.when(options.template) : $templateRequest(angular.isFunction(options.templateUrl) ? options.templateUrl() : options.templateUrl);
          }
          function getResolvePromises(resolves) {
            var promisesArr = [];
            angular.forEach(resolves, function (value) {
              if (angular.isFunction(value) || angular.isArray(value)) {
                promisesArr.push($q.when($injector.invoke(value)));
              } else if (angular.isString(value)) {
                promisesArr.push($q.when($injector.get(value)));
              } else {
                promisesArr.push($q.when(value));
              }
            });
            return promisesArr;
          }
          var promiseChain = null;
          $modal.getPromiseChain = function () {
            return promiseChain;
          };
          $modal.open = function (modalOptions) {
            var modalResultDeferred = $q.defer();
            var modalOpenedDeferred = $q.defer();
            var modalRenderDeferred = $q.defer();
            //prepare an instance of a modal to be injected into controllers and returned to a caller
            var modalInstance = {
                result: modalResultDeferred.promise,
                opened: modalOpenedDeferred.promise,
                rendered: modalRenderDeferred.promise,
                close: function (result) {
                  return $modalStack.close(modalInstance, result);
                },
                dismiss: function (reason) {
                  return $modalStack.dismiss(modalInstance, reason);
                }
              };
            //merge and clean up options
            modalOptions = angular.extend({}, $modalProvider.options, modalOptions);
            modalOptions.resolve = modalOptions.resolve || {};
            //verify options
            if (!modalOptions.template && !modalOptions.templateUrl) {
              throw new Error('One of template or templateUrl options is required.');
            }
            var templateAndResolvePromise = $q.all([getTemplatePromise(modalOptions)].concat(getResolvePromises(modalOptions.resolve)));
            // Wait for the resolution of the existing promise chain.
            // Then switch to our own combined promise dependency (regardless of how the previous modal fared).
            // Then add to $modalStack and resolve opened.
            // Finally clean up the chain variable if no subsequent modal has overwritten it.
            var samePromise;
            samePromise = promiseChain = $q.all([promiseChain]).then(function () {
              return templateAndResolvePromise;
            }, function () {
              return templateAndResolvePromise;
            }).then(function resolveSuccess(tplAndVars) {
              var modalScope = (modalOptions.scope || $rootScope).$new();
              modalScope.$close = modalInstance.close;
              modalScope.$dismiss = modalInstance.dismiss;
              modalScope.$on('$destroy', function () {
                if (!modalScope.$$uibDestructionScheduled) {
                  modalScope.$dismiss('$uibUnscheduledDestruction');
                }
              });
              var ctrlInstance, ctrlLocals = {};
              var resolveIter = 1;
              //controllers
              if (modalOptions.controller) {
                ctrlLocals.$scope = modalScope;
                ctrlLocals.$modalInstance = modalInstance;
                angular.forEach(modalOptions.resolve, function (value, key) {
                  ctrlLocals[key] = tplAndVars[resolveIter++];
                });
                ctrlInstance = $controller(modalOptions.controller, ctrlLocals);
                if (modalOptions.controllerAs) {
                  if (modalOptions.bindToController) {
                    angular.extend(ctrlInstance, modalScope);
                  }
                  modalScope[modalOptions.controllerAs] = ctrlInstance;
                }
              }
              $modalStack.open(modalInstance, {
                scope: modalScope,
                deferred: modalResultDeferred,
                renderDeferred: modalRenderDeferred,
                content: tplAndVars[0],
                animation: modalOptions.animation,
                backdrop: modalOptions.backdrop,
                keyboard: modalOptions.keyboard,
                backdropClass: modalOptions.backdropClass,
                windowTopClass: modalOptions.windowTopClass,
                windowClass: modalOptions.windowClass,
                windowTemplateUrl: modalOptions.windowTemplateUrl,
                size: modalOptions.size,
                openedClass: modalOptions.openedClass
              });
              modalOpenedDeferred.resolve(true);
            }, function resolveError(reason) {
              modalOpenedDeferred.reject(reason);
              modalResultDeferred.reject(reason);
            }).finally(function () {
              if (promiseChain === samePromise) {
                promiseChain = null;
              }
            });
            return modalInstance;
          };
          return $modal;
        }
      ]
    };
  return $modalProvider;
});
/* deprecated modal below */
angular.module('ui.bootstrap.modal').value('$modalSuppressWarning', false).directive('modalBackdrop', [
  '$animate',
  '$injector',
  '$modalStack',
  '$log',
  '$modalSuppressWarning',
  function ($animate, $injector, $modalStack, $log, $modalSuppressWarning) {
    var $animateCss = null;
    if ($injector.has('$animateCss')) {
      $animateCss = $injector.get('$animateCss');
    }
    return {
      replace: true,
      templateUrl: 'template/modal/backdrop.html',
      compile: function (tElement, tAttrs) {
        tElement.addClass(tAttrs.backdropClass);
        return linkFn;
      }
    };
    function linkFn(scope, element, attrs) {
      if (!$modalSuppressWarning) {
        $log.warn('modal-backdrop is now deprecated. Use uib-modal-backdrop instead.');
      }
      element.addClass('modal-backdrop');
      if (attrs.modalInClass) {
        if ($animateCss) {
          $animateCss(element, { addClass: attrs.modalInClass }).start();
        } else {
          $animate.addClass(element, attrs.modalInClass);
        }
        scope.$on($modalStack.NOW_CLOSING_EVENT, function (e, setIsAsync) {
          var done = setIsAsync();
          if ($animateCss) {
            $animateCss(element, { removeClass: attrs.modalInClass }).start().then(done);
          } else {
            $animate.removeClass(element, attrs.modalInClass).then(done);
          }
        });
      }
    }
  }
]).directive('modalWindow', [
  '$modalStack',
  '$q',
  '$animate',
  '$injector',
  '$log',
  '$modalSuppressWarning',
  function ($modalStack, $q, $animate, $injector, $log, $modalSuppressWarning) {
    var $animateCss = null;
    if ($injector.has('$animateCss')) {
      $animateCss = $injector.get('$animateCss');
    }
    return {
      scope: { index: '@' },
      replace: true,
      transclude: true,
      templateUrl: function (tElement, tAttrs) {
        return tAttrs.templateUrl || 'template/modal/window.html';
      },
      link: function (scope, element, attrs) {
        if (!$modalSuppressWarning) {
          $log.warn('modal-window is now deprecated. Use uib-modal-window instead.');
        }
        element.addClass(attrs.windowClass || '');
        element.addClass(attrs.windowTopClass || '');
        scope.size = attrs.size;
        scope.close = function (evt) {
          var modal = $modalStack.getTop();
          if (modal && modal.value.backdrop && modal.value.backdrop !== 'static' && evt.target === evt.currentTarget) {
            evt.preventDefault();
            evt.stopPropagation();
            $modalStack.dismiss(modal.key, 'backdrop click');
          }
        };
        // moved from template to fix issue #2280
        element.on('click', scope.close);
        // This property is only added to the scope for the purpose of detecting when this directive is rendered.
        // We can detect that by using this property in the template associated with this directive and then use
        // {@link Attribute#$observe} on it. For more details please see {@link TableColumnResize}.
        scope.$isRendered = true;
        // Deferred object that will be resolved when this modal is render.
        var modalRenderDeferObj = $q.defer();
        // Observe function will be called on next digest cycle after compilation, ensuring that the DOM is ready.
        // In order to use this way of finding whether DOM is ready, we need to observe a scope property used in modal's template.
        attrs.$observe('modalRender', function (value) {
          if (value == 'true') {
            modalRenderDeferObj.resolve();
          }
        });
        modalRenderDeferObj.promise.then(function () {
          var animationPromise = null;
          if (attrs.modalInClass) {
            if ($animateCss) {
              animationPromise = $animateCss(element, { addClass: attrs.modalInClass }).start();
            } else {
              animationPromise = $animate.addClass(element, attrs.modalInClass);
            }
            scope.$on($modalStack.NOW_CLOSING_EVENT, function (e, setIsAsync) {
              var done = setIsAsync();
              if ($animateCss) {
                $animateCss(element, { removeClass: attrs.modalInClass }).start().then(done);
              } else {
                $animate.removeClass(element, attrs.modalInClass).then(done);
              }
            });
          }
          $q.when(animationPromise).then(function () {
            var inputsWithAutofocus = element[0].querySelectorAll('[autofocus]');
            /**
               * Auto-focusing of a freshly-opened modal element causes any child elements
               * with the autofocus attribute to lose focus. This is an issue on touch
               * based devices which will show and then hide the onscreen keyboard.
               * Attempts to refocus the autofocus element via JavaScript will not reopen
               * the onscreen keyboard. Fixed by updated the focusing logic to only autofocus
               * the modal element if the modal does not contain an autofocus element.
               */
            if (inputsWithAutofocus.length) {
              inputsWithAutofocus[0].focus();
            } else {
              element[0].focus();
            }
          });
          // Notify {@link $modalStack} that modal is rendered.
          var modal = $modalStack.getTop();
          if (modal) {
            $modalStack.modalRendered(modal.key);
          }
        });
      }
    };
  }
]).directive('modalAnimationClass', [
  '$log',
  '$modalSuppressWarning',
  function ($log, $modalSuppressWarning) {
    return {
      compile: function (tElement, tAttrs) {
        if (!$modalSuppressWarning) {
          $log.warn('modal-animation-class is now deprecated. Use uib-modal-animation-class instead.');
        }
        if (tAttrs.modalAnimation) {
          tElement.addClass(tAttrs.modalAnimationClass);
        }
      }
    };
  }
]).directive('modalTransclude', [
  '$log',
  '$modalSuppressWarning',
  function ($log, $modalSuppressWarning) {
    return {
      link: function ($scope, $element, $attrs, controller, $transclude) {
        if (!$modalSuppressWarning) {
          $log.warn('modal-transclude is now deprecated. Use uib-modal-transclude instead.');
        }
        $transclude($scope.$parent, function (clone) {
          $element.empty();
          $element.append(clone);
        });
      }
    };
  }
]).service('$modalStack', [
  '$animate',
  '$timeout',
  '$document',
  '$compile',
  '$rootScope',
  '$q',
  '$injector',
  '$$multiMap',
  '$$stackedMap',
  '$uibModalStack',
  '$log',
  '$modalSuppressWarning',
  function ($animate, $timeout, $document, $compile, $rootScope, $q, $injector, $$multiMap, $$stackedMap, $uibModalStack, $log, $modalSuppressWarning) {
    if (!$modalSuppressWarning) {
      $log.warn('$modalStack is now deprecated. Use $uibModalStack instead.');
    }
    angular.extend(this, $uibModalStack);
  }
]).provider('$modal', [
  '$uibModalProvider',
  function ($uibModalProvider) {
    angular.extend(this, $uibModalProvider);
    this.$get = [
      '$injector',
      '$log',
      '$modalSuppressWarning',
      function ($injector, $log, $modalSuppressWarning) {
        if (!$modalSuppressWarning) {
          $log.warn('$modal is now deprecated. Use $uibModal instead.');
        }
        return $injector.invoke($uibModalProvider.$get);
      }
    ];
  }
]);
angular.module('ui.bootstrap.pagination', []).controller('UibPaginationController', [
  '$scope',
  '$attrs',
  '$parse',
  function ($scope, $attrs, $parse) {
    var self = this, ngModelCtrl = { $setViewValue: angular.noop },
      // nullModelCtrl
      setNumPages = $attrs.numPages ? $parse($attrs.numPages).assign : angular.noop;
    this.init = function (ngModelCtrl_, config) {
      ngModelCtrl = ngModelCtrl_;
      this.config = config;
      ngModelCtrl.$render = function () {
        self.render();
      };
      if ($attrs.itemsPerPage) {
        $scope.$parent.$watch($parse($attrs.itemsPerPage), function (value) {
          self.itemsPerPage = parseInt(value, 10);
          $scope.totalPages = self.calculateTotalPages();
        });
      } else {
        this.itemsPerPage = config.itemsPerPage;
      }
      $scope.$watch('totalItems', function () {
        $scope.totalPages = self.calculateTotalPages();
      });
      $scope.$watch('totalPages', function (value) {
        setNumPages($scope.$parent, value);
        // Readonly variable
        if ($scope.page > value) {
          $scope.selectPage(value);
        } else {
          ngModelCtrl.$render();
        }
      });
    };
    this.calculateTotalPages = function () {
      var totalPages = this.itemsPerPage < 1 ? 1 : Math.ceil($scope.totalItems / this.itemsPerPage);
      return Math.max(totalPages || 0, 1);
    };
    this.render = function () {
      $scope.page = parseInt(ngModelCtrl.$viewValue, 10) || 1;
    };
    $scope.selectPage = function (page, evt) {
      if (evt) {
        evt.preventDefault();
      }
      var clickAllowed = !$scope.ngDisabled || !evt;
      if (clickAllowed && $scope.page !== page && page > 0 && page <= $scope.totalPages) {
        if (evt && evt.target) {
          evt.target.blur();
        }
        ngModelCtrl.$setViewValue(page);
        ngModelCtrl.$render();
      }
    };
    $scope.getText = function (key) {
      return $scope[key + 'Text'] || self.config[key + 'Text'];
    };
    $scope.noPrevious = function () {
      return $scope.page === 1;
    };
    $scope.noNext = function () {
      return $scope.page === $scope.totalPages;
    };
  }
]).constant('uibPaginationConfig', {
  itemsPerPage: 10,
  boundaryLinks: false,
  directionLinks: true,
  firstText: 'First',
  previousText: 'Previous',
  nextText: 'Next',
  lastText: 'Last',
  rotate: true
}).directive('uibPagination', [
  '$parse',
  'uibPaginationConfig',
  function ($parse, paginationConfig) {
    return {
      restrict: 'EA',
      scope: {
        totalItems: '=',
        firstText: '@',
        previousText: '@',
        nextText: '@',
        lastText: '@',
        ngDisabled: '='
      },
      require: [
        'uibPagination',
        '?ngModel'
      ],
      controller: 'UibPaginationController',
      controllerAs: 'pagination',
      templateUrl: function (element, attrs) {
        return attrs.templateUrl || 'template/pagination/pagination.html';
      },
      replace: true,
      link: function (scope, element, attrs, ctrls) {
        var paginationCtrl = ctrls[0], ngModelCtrl = ctrls[1];
        if (!ngModelCtrl) {
          return;  // do nothing if no ng-model
        }
        // Setup configuration parameters
        var maxSize = angular.isDefined(attrs.maxSize) ? scope.$parent.$eval(attrs.maxSize) : paginationConfig.maxSize, rotate = angular.isDefined(attrs.rotate) ? scope.$parent.$eval(attrs.rotate) : paginationConfig.rotate;
        scope.boundaryLinks = angular.isDefined(attrs.boundaryLinks) ? scope.$parent.$eval(attrs.boundaryLinks) : paginationConfig.boundaryLinks;
        scope.directionLinks = angular.isDefined(attrs.directionLinks) ? scope.$parent.$eval(attrs.directionLinks) : paginationConfig.directionLinks;
        paginationCtrl.init(ngModelCtrl, paginationConfig);
        if (attrs.maxSize) {
          scope.$parent.$watch($parse(attrs.maxSize), function (value) {
            maxSize = parseInt(value, 10);
            paginationCtrl.render();
          });
        }
        // Create page object used in template
        function makePage(number, text, isActive) {
          return {
            number: number,
            text: text,
            active: isActive
          };
        }
        function getPages(currentPage, totalPages) {
          var pages = [];
          // Default page limits
          var startPage = 1, endPage = totalPages;
          var isMaxSized = angular.isDefined(maxSize) && maxSize < totalPages;
          // recompute if maxSize
          if (isMaxSized) {
            if (rotate) {
              // Current page is displayed in the middle of the visible ones
              startPage = Math.max(currentPage - Math.floor(maxSize / 2), 1);
              endPage = startPage + maxSize - 1;
              // Adjust if limit is exceeded
              if (endPage > totalPages) {
                endPage = totalPages;
                startPage = endPage - maxSize + 1;
              }
            } else {
              // Visible pages are paginated with maxSize
              startPage = (Math.ceil(currentPage / maxSize) - 1) * maxSize + 1;
              // Adjust last page if limit is exceeded
              endPage = Math.min(startPage + maxSize - 1, totalPages);
            }
          }
          // Add page number links
          for (var number = startPage; number <= endPage; number++) {
            var page = makePage(number, number, number === currentPage);
            pages.push(page);
          }
          // Add links to move between page sets
          if (isMaxSized && !rotate) {
            if (startPage > 1) {
              var previousPageSet = makePage(startPage - 1, '...', false);
              pages.unshift(previousPageSet);
            }
            if (endPage < totalPages) {
              var nextPageSet = makePage(endPage + 1, '...', false);
              pages.push(nextPageSet);
            }
          }
          return pages;
        }
        var originalRender = paginationCtrl.render;
        paginationCtrl.render = function () {
          originalRender();
          if (scope.page > 0 && scope.page <= scope.totalPages) {
            scope.pages = getPages(scope.page, scope.totalPages);
          }
        };
      }
    };
  }
]).constant('uibPagerConfig', {
  itemsPerPage: 10,
  previousText: '\xab Previous',
  nextText: 'Next \xbb',
  align: true
}).directive('uibPager', [
  'uibPagerConfig',
  function (pagerConfig) {
    return {
      restrict: 'EA',
      scope: {
        totalItems: '=',
        previousText: '@',
        nextText: '@',
        ngDisabled: '='
      },
      require: [
        'uibPager',
        '?ngModel'
      ],
      controller: 'UibPaginationController',
      controllerAs: 'pagination',
      templateUrl: function (element, attrs) {
        return attrs.templateUrl || 'template/pagination/pager.html';
      },
      replace: true,
      link: function (scope, element, attrs, ctrls) {
        var paginationCtrl = ctrls[0], ngModelCtrl = ctrls[1];
        if (!ngModelCtrl) {
          return;  // do nothing if no ng-model
        }
        scope.align = angular.isDefined(attrs.align) ? scope.$parent.$eval(attrs.align) : pagerConfig.align;
        paginationCtrl.init(ngModelCtrl, pagerConfig);
      }
    };
  }
]);
/* Deprecated Pagination Below */
angular.module('ui.bootstrap.pagination').value('$paginationSuppressWarning', false).controller('PaginationController', [
  '$scope',
  '$attrs',
  '$parse',
  '$controller',
  '$element',
  '$log',
  '$paginationSuppressWarning',
  function ($scope, $attrs, $parse, $controller, $element, $log, $paginationSuppressWarning) {
    if (!$paginationSuppressWarning) {
      $log.warn('PaginationController is now deprecated. Use UibPaginationController instead.');
    }
    return $controller('UibPaginationController', {
      $scope: $scope,
      $element: $element,
      $attrs: $attrs
    });
  }
]).directive('pagination', [
  '$parse',
  'uibPaginationConfig',
  '$log',
  '$paginationSuppressWarning',
  function ($parse, paginationConfig, $log, $paginationSuppressWarning) {
    return {
      restrict: 'EA',
      scope: {
        totalItems: '=',
        firstText: '@',
        previousText: '@',
        nextText: '@',
        lastText: '@',
        ngDisabled: '='
      },
      require: [
        'pagination',
        '?ngModel'
      ],
      controller: 'PaginationController',
      controllerAs: 'pagination',
      templateUrl: function (element, attrs) {
        return attrs.templateUrl || 'template/pagination/pagination.html';
      },
      replace: true,
      link: function (scope, element, attrs, ctrls) {
        if (!$paginationSuppressWarning) {
          $log.warn('pagination is now deprecated. Use uib-pagination instead.');
        }
        var paginationCtrl = ctrls[0], ngModelCtrl = ctrls[1];
        if (!ngModelCtrl) {
          return;  // do nothing if no ng-model
        }
        // Setup configuration parameters
        var maxSize = angular.isDefined(attrs.maxSize) ? scope.$parent.$eval(attrs.maxSize) : paginationConfig.maxSize, rotate = angular.isDefined(attrs.rotate) ? scope.$parent.$eval(attrs.rotate) : paginationConfig.rotate;
        scope.boundaryLinks = angular.isDefined(attrs.boundaryLinks) ? scope.$parent.$eval(attrs.boundaryLinks) : paginationConfig.boundaryLinks;
        scope.directionLinks = angular.isDefined(attrs.directionLinks) ? scope.$parent.$eval(attrs.directionLinks) : paginationConfig.directionLinks;
        paginationCtrl.init(ngModelCtrl, paginationConfig);
        if (attrs.maxSize) {
          scope.$parent.$watch($parse(attrs.maxSize), function (value) {
            maxSize = parseInt(value, 10);
            paginationCtrl.render();
          });
        }
        // Create page object used in template
        function makePage(number, text, isActive) {
          return {
            number: number,
            text: text,
            active: isActive
          };
        }
        function getPages(currentPage, totalPages) {
          var pages = [];
          // Default page limits
          var startPage = 1, endPage = totalPages;
          var isMaxSized = angular.isDefined(maxSize) && maxSize < totalPages;
          // recompute if maxSize
          if (isMaxSized) {
            if (rotate) {
              // Current page is displayed in the middle of the visible ones
              startPage = Math.max(currentPage - Math.floor(maxSize / 2), 1);
              endPage = startPage + maxSize - 1;
              // Adjust if limit is exceeded
              if (endPage > totalPages) {
                endPage = totalPages;
                startPage = endPage - maxSize + 1;
              }
            } else {
              // Visible pages are paginated with maxSize
              startPage = (Math.ceil(currentPage / maxSize) - 1) * maxSize + 1;
              // Adjust last page if limit is exceeded
              endPage = Math.min(startPage + maxSize - 1, totalPages);
            }
          }
          // Add page number links
          for (var number = startPage; number <= endPage; number++) {
            var page = makePage(number, number, number === currentPage);
            pages.push(page);
          }
          // Add links to move between page sets
          if (isMaxSized && !rotate) {
            if (startPage > 1) {
              var previousPageSet = makePage(startPage - 1, '...', false);
              pages.unshift(previousPageSet);
            }
            if (endPage < totalPages) {
              var nextPageSet = makePage(endPage + 1, '...', false);
              pages.push(nextPageSet);
            }
          }
          return pages;
        }
        var originalRender = paginationCtrl.render;
        paginationCtrl.render = function () {
          originalRender();
          if (scope.page > 0 && scope.page <= scope.totalPages) {
            scope.pages = getPages(scope.page, scope.totalPages);
          }
        };
      }
    };
  }
]).directive('pager', [
  'uibPagerConfig',
  '$log',
  '$paginationSuppressWarning',
  function (pagerConfig, $log, $paginationSuppressWarning) {
    return {
      restrict: 'EA',
      scope: {
        totalItems: '=',
        previousText: '@',
        nextText: '@',
        ngDisabled: '='
      },
      require: [
        'pager',
        '?ngModel'
      ],
      controller: 'PaginationController',
      controllerAs: 'pagination',
      templateUrl: function (element, attrs) {
        return attrs.templateUrl || 'template/pagination/pager.html';
      },
      replace: true,
      link: function (scope, element, attrs, ctrls) {
        if (!$paginationSuppressWarning) {
          $log.warn('pager is now deprecated. Use uib-pager instead.');
        }
        var paginationCtrl = ctrls[0], ngModelCtrl = ctrls[1];
        if (!ngModelCtrl) {
          return;  // do nothing if no ng-model
        }
        scope.align = angular.isDefined(attrs.align) ? scope.$parent.$eval(attrs.align) : pagerConfig.align;
        paginationCtrl.init(ngModelCtrl, pagerConfig);
      }
    };
  }
]);
/**
 * The following features are still outstanding: animation as a
 * function, placement as a function, inside, support for more triggers than
 * just mouse enter/leave, html tooltips, and selector delegation.
 */
angular.module('ui.bootstrap.tooltip', [
  'ui.bootstrap.position',
  'ui.bootstrap.stackedMap'
]).provider('$uibTooltip', function () {
  // The default options tooltip and popover.
  var defaultOptions = {
      placement: 'top',
      animation: true,
      popupDelay: 0,
      popupCloseDelay: 500,
      useContentExp: false
    };
  // Default hide triggers for each show trigger
  var triggerMap = {
      'mouseenter': 'mouseleave',
      'click': 'click',
      'focus': 'blur',
      'none': ''
    };
  // The options specified to the provider globally.
  var globalOptions = {};
  /**
   * `options({})` allows global configuration of all tooltips in the
   * application.
   *
   *   var app = angular.module( 'App', ['ui.bootstrap.tooltip'], function( $tooltipProvider ) {
   *     // place tooltips left instead of top by default
   *     $tooltipProvider.options( { placement: 'left' } );
   *   });
   */
  this.options = function (value) {
    angular.extend(globalOptions, value);
  };
  /**
   * This allows you to extend the set of trigger mappings available. E.g.:
   *
   *   $tooltipProvider.setTriggers( 'openTrigger': 'closeTrigger' );
   */
  this.setTriggers = function setTriggers(triggers) {
    angular.extend(triggerMap, triggers);
  };
  /**
   * This is a helper function for translating camel-case to snake-case.
   */
  function snake_case(name) {
    var regexp = /[A-Z]/g;
    var separator = '-';
    return name.replace(regexp, function (letter, pos) {
      return (pos ? separator : '') + letter.toLowerCase();
    });
  }
  /**
   * Returns the actual instance of the $tooltip service.
   * TODO support multiple triggers
   */
  this.$get = [
    '$window',
    '$compile',
    '$timeout',
    '$document',
    '$uibPosition',
    '$interpolate',
    '$rootScope',
    '$parse',
    '$$stackedMap',
    function ($window, $compile, $timeout, $document, $position, $interpolate, $rootScope, $parse, $$stackedMap) {
      var openedTooltips = $$stackedMap.createNew();
      $document.on('keypress', function (e) {
        if (e.which === 27) {
          var last = openedTooltips.top();
          if (last) {
            last.value.close();
            openedTooltips.removeTop();
            last = null;
          }
        }
      });
      return function $tooltip(ttType, prefix, defaultTriggerShow, options) {
        options = angular.extend({}, defaultOptions, globalOptions, options);
        /**
       * Returns an object of show and hide triggers.
       *
       * If a trigger is supplied,
       * it is used to show the tooltip; otherwise, it will use the `trigger`
       * option passed to the `$tooltipProvider.options` method; else it will
       * default to the trigger supplied to this directive factory.
       *
       * The hide trigger is based on the show trigger. If the `trigger` option
       * was passed to the `$tooltipProvider.options` method, it will use the
       * mapped trigger from `triggerMap` or the passed trigger if the map is
       * undefined; otherwise, it uses the `triggerMap` value of the show
       * trigger; else it will just use the show trigger.
       */
        function getTriggers(trigger) {
          var show = (trigger || options.trigger || defaultTriggerShow).split(' ');
          var hide = show.map(function (trigger) {
              return triggerMap[trigger] || trigger;
            });
          return {
            show: show,
            hide: hide
          };
        }
        var directiveName = snake_case(ttType);
        var startSym = $interpolate.startSymbol();
        var endSym = $interpolate.endSymbol();
        var template = '<div ' + directiveName + '-popup ' + 'title="' + startSym + 'title' + endSym + '" ' + (options.useContentExp ? 'content-exp="contentExp()" ' : 'content="' + startSym + 'content' + endSym + '" ') + 'placement="' + startSym + 'placement' + endSym + '" ' + 'popup-class="' + startSym + 'popupClass' + endSym + '" ' + 'animation="animation" ' + 'is-open="isOpen"' + 'origin-scope="origScope" ' + 'style="visibility: hidden; display: block;"' + '>' + '</div>';
        return {
          compile: function (tElem, tAttrs) {
            var tooltipLinker = $compile(template);
            return function link(scope, element, attrs, tooltipCtrl) {
              var tooltip;
              var tooltipLinkedScope;
              var transitionTimeout;
              var popupTimeout;
              var positionTimeout;
              var appendToBody = angular.isDefined(options.appendToBody) ? options.appendToBody : false;
              var triggers = getTriggers(undefined);
              var hasEnableExp = angular.isDefined(attrs[prefix + 'Enable']);
              var ttScope = scope.$new(true);
              var repositionScheduled = false;
              var isOpenParse = angular.isDefined(attrs[prefix + 'IsOpen']) ? $parse(attrs[prefix + 'IsOpen']) : false;
              var contentParse = options.useContentExp ? $parse(attrs[ttType]) : false;
              var observers = [];
              var positionTooltip = function () {
                // check if tooltip exists and is not empty
                if (!tooltip || !tooltip.html()) {
                  return;
                }
                if (!positionTimeout) {
                  positionTimeout = $timeout(function () {
                    // Reset the positioning.
                    tooltip.css({
                      top: 0,
                      left: 0
                    });
                    // Now set the calculated positioning.
                    var ttCss = $position.positionElements(element, tooltip, ttScope.placement, appendToBody);
                    ttCss.top += 'px';
                    ttCss.left += 'px';
                    ttCss.visibility = 'visible';
                    tooltip.css(ttCss);
                    positionTimeout = null;
                  }, 0, false);
                }
              };
              // Set up the correct scope to allow transclusion later
              ttScope.origScope = scope;
              // By default, the tooltip is not open.
              // TODO add ability to start tooltip opened
              ttScope.isOpen = false;
              openedTooltips.add(ttScope, { close: hide });
              function toggleTooltipBind() {
                if (!ttScope.isOpen) {
                  showTooltipBind();
                } else {
                  hideTooltipBind();
                }
              }
              // Show the tooltip with delay if specified, otherwise show it immediately
              function showTooltipBind() {
                if (hasEnableExp && !scope.$eval(attrs[prefix + 'Enable'])) {
                  return;
                }
                prepareTooltip();
                if (ttScope.popupDelay) {
                  // Do nothing if the tooltip was already scheduled to pop-up.
                  // This happens if show is triggered multiple times before any hide is triggered.
                  if (!popupTimeout) {
                    popupTimeout = $timeout(show, ttScope.popupDelay, false);
                  }
                } else {
                  show();
                }
              }
              function hideTooltipBind() {
                hide();
              }
              // Show the tooltip popup element.
              function show() {
                if (popupTimeout) {
                  $timeout.cancel(popupTimeout);
                  popupTimeout = null;
                }
                // If there is a pending remove transition, we must cancel it, lest the
                // tooltip be mysteriously removed.
                if (transitionTimeout) {
                  $timeout.cancel(transitionTimeout);
                  transitionTimeout = null;
                }
                // Don't show empty tooltips.
                if (!ttScope.content) {
                  return angular.noop;
                }
                createTooltip();
                // And show the tooltip.
                ttScope.$evalAsync(function () {
                  ttScope.isOpen = true;
                  if (isOpenParse && angular.isFunction(isOpenParse.assign)) {
                    isOpenParse.assign(ttScope.origScope, ttScope.isOpen);
                  }
                  positionTooltip();
                });
              }
              // Hide the tooltip popup element.
              function hide() {
                if (!ttScope) {
                  return;
                }
                // First things first: we don't show it anymore.
                ttScope.$evalAsync(function () {
                  ttScope.isOpen = false;
                  if (isOpenParse && angular.isFunction(isOpenParse.assign)) {
                    isOpenParse.assign(ttScope.origScope, ttScope.isOpen);
                  }
                });
                //if tooltip is going to be shown after delay, we must cancel this
                $timeout.cancel(popupTimeout);
                popupTimeout = null;
                $timeout.cancel(positionTimeout);
                positionTimeout = null;
                // And now we remove it from the DOM. However, if we have animation, we
                // need to wait for it to expire beforehand.
                // FIXME: this is a placeholder for a port of the transitions library.
                if (ttScope.animation) {
                  if (!transitionTimeout) {
                    transitionTimeout = $timeout(removeTooltip, ttScope.popupCloseDelay);
                  }
                } else {
                  removeTooltip();
                }
              }
              function createTooltip() {
                // There can only be one tooltip element per directive shown at once.
                if (tooltip) {
                  return;
                }
                tooltipLinkedScope = ttScope.$new();
                tooltip = tooltipLinker(tooltipLinkedScope, function (tooltip) {
                  if (appendToBody) {
                    $document.find('body').append(tooltip);
                  } else {
                    element.after(tooltip);
                  }
                });
                prepObservers();
              }
              function removeTooltip() {
                unregisterObservers();
                transitionTimeout = null;
                if (tooltip) {
                  tooltip.remove();
                  tooltip = null;
                }
                if (tooltipLinkedScope) {
                  tooltipLinkedScope.$destroy();
                  tooltipLinkedScope = null;
                }
              }
              /**
             * Set the inital scope values. Once
             * the tooltip is created, the observers
             * will be added to keep things in synch.
             */
              function prepareTooltip() {
                ttScope.title = attrs[prefix + 'Title'];
                if (contentParse) {
                  ttScope.content = contentParse(scope);
                } else {
                  ttScope.content = attrs[ttType];
                }
                ttScope.popupClass = attrs[prefix + 'Class'];
                ttScope.placement = angular.isDefined(attrs[prefix + 'Placement']) ? attrs[prefix + 'Placement'] : options.placement;
                var delay = parseInt(attrs[prefix + 'PopupDelay'], 10);
                var closeDelay = parseInt(attrs[prefix + 'PopupCloseDelay'], 10);
                ttScope.popupDelay = !isNaN(delay) ? delay : options.popupDelay;
                ttScope.popupCloseDelay = !isNaN(closeDelay) ? closeDelay : options.popupCloseDelay;
              }
              ttScope.contentExp = function () {
                return ttScope.content;
              };
              /**
             * Observe the relevant attributes.
             */
              attrs.$observe('disabled', function (val) {
                if (popupTimeout && val) {
                  $timeout.cancel(popupTimeout);
                  popupTimeout = null;
                }
                if (val && ttScope.isOpen) {
                  hide();
                }
              });
              if (isOpenParse) {
                scope.$watch(isOpenParse, function (val) {
                  /*jshint -W018 */
                  if (!val === ttScope.isOpen) {
                    toggleTooltipBind();
                  }  /*jshint +W018 */
                });
              }
              function prepObservers() {
                observers.length = 0;
                if (contentParse) {
                  observers.push(scope.$watch(contentParse, function (val) {
                    ttScope.content = val;
                    if (!val && ttScope.isOpen) {
                      hide();
                    }
                  }));
                  observers.push(tooltipLinkedScope.$watch(function () {
                    if (!repositionScheduled) {
                      repositionScheduled = true;
                      tooltipLinkedScope.$$postDigest(function () {
                        repositionScheduled = false;
                        if (ttScope && ttScope.isOpen) {
                          positionTooltip();
                        }
                      });
                    }
                  }));
                } else {
                  observers.push(attrs.$observe(ttType, function (val) {
                    ttScope.content = val;
                    if (!val && ttScope.isOpen) {
                      hide();
                    } else {
                      positionTooltip();
                    }
                  }));
                }
                observers.push(attrs.$observe(prefix + 'Title', function (val) {
                  ttScope.title = val;
                  if (ttScope.isOpen) {
                    positionTooltip();
                  }
                }));
                observers.push(attrs.$observe(prefix + 'Placement', function (val) {
                  ttScope.placement = val ? val : options.placement;
                  if (ttScope.isOpen) {
                    positionTooltip();
                  }
                }));
              }
              function unregisterObservers() {
                if (observers.length) {
                  angular.forEach(observers, function (observer) {
                    observer();
                  });
                  observers.length = 0;
                }
              }
              var unregisterTriggers = function () {
                triggers.show.forEach(function (trigger) {
                  element.unbind(trigger, showTooltipBind);
                });
                triggers.hide.forEach(function (trigger) {
                  trigger.split(' ').forEach(function (hideTrigger) {
                    element[0].removeEventListener(hideTrigger, hideTooltipBind);
                  });
                });
              };
              function prepTriggers() {
                var val = attrs[prefix + 'Trigger'];
                unregisterTriggers();
                triggers = getTriggers(val);
                if (triggers.show !== 'none') {
                  triggers.show.forEach(function (trigger, idx) {
                    // Using raw addEventListener due to jqLite/jQuery bug - #4060
                    if (trigger === triggers.hide[idx]) {
                      element[0].addEventListener(trigger, toggleTooltipBind);
                    } else if (trigger) {
                      element[0].addEventListener(trigger, showTooltipBind);
                      triggers.hide[idx].split(' ').forEach(function (trigger) {
                        element[0].addEventListener(trigger, hideTooltipBind);
                      });
                    }
                    element.on('keypress', function (e) {
                      if (e.which === 27) {
                        hideTooltipBind();
                      }
                    });
                  });
                }
              }
              prepTriggers();
              var animation = scope.$eval(attrs[prefix + 'Animation']);
              ttScope.animation = angular.isDefined(animation) ? !!animation : options.animation;
              var appendToBodyVal = scope.$eval(attrs[prefix + 'AppendToBody']);
              appendToBody = angular.isDefined(appendToBodyVal) ? appendToBodyVal : appendToBody;
              // if a tooltip is attached to <body> we need to remove it on
              // location change as its parent scope will probably not be destroyed
              // by the change.
              if (appendToBody) {
                scope.$on('$locationChangeSuccess', function closeTooltipOnLocationChangeSuccess() {
                  if (ttScope.isOpen) {
                    hide();
                  }
                });
              }
              // Make sure tooltip is destroyed and removed.
              scope.$on('$destroy', function onDestroyTooltip() {
                $timeout.cancel(transitionTimeout);
                $timeout.cancel(popupTimeout);
                $timeout.cancel(positionTimeout);
                unregisterTriggers();
                removeTooltip();
                ttScope = null;
              });
            };
          }
        };
      };
    }
  ];
}).directive('uibTooltipTemplateTransclude', [
  '$animate',
  '$sce',
  '$compile',
  '$templateRequest',
  function ($animate, $sce, $compile, $templateRequest) {
    return {
      link: function (scope, elem, attrs) {
        var origScope = scope.$eval(attrs.tooltipTemplateTranscludeScope);
        var changeCounter = 0, currentScope, previousElement, currentElement;
        var cleanupLastIncludeContent = function () {
          if (previousElement) {
            previousElement.remove();
            previousElement = null;
          }
          if (currentScope) {
            currentScope.$destroy();
            currentScope = null;
          }
          if (currentElement) {
            $animate.leave(currentElement).then(function () {
              previousElement = null;
            });
            previousElement = currentElement;
            currentElement = null;
          }
        };
        scope.$watch($sce.parseAsResourceUrl(attrs.uibTooltipTemplateTransclude), function (src) {
          var thisChangeId = ++changeCounter;
          if (src) {
            //set the 2nd param to true to ignore the template request error so that the inner
            //contents and scope can be cleaned up.
            $templateRequest(src, true).then(function (response) {
              if (thisChangeId !== changeCounter) {
                return;
              }
              var newScope = origScope.$new();
              var template = response;
              var clone = $compile(template)(newScope, function (clone) {
                  cleanupLastIncludeContent();
                  $animate.enter(clone, elem);
                });
              currentScope = newScope;
              currentElement = clone;
              currentScope.$emit('$includeContentLoaded', src);
            }, function () {
              if (thisChangeId === changeCounter) {
                cleanupLastIncludeContent();
                scope.$emit('$includeContentError', src);
              }
            });
            scope.$emit('$includeContentRequested', src);
          } else {
            cleanupLastIncludeContent();
          }
        });
        scope.$on('$destroy', cleanupLastIncludeContent);
      }
    };
  }
]).directive('uibTooltipClasses', function () {
  return {
    restrict: 'A',
    link: function (scope, element, attrs) {
      if (scope.placement) {
        element.addClass(scope.placement);
      }
      if (scope.popupClass) {
        element.addClass(scope.popupClass);
      }
      if (scope.animation()) {
        element.addClass(attrs.tooltipAnimationClass);
      }
    }
  };
}).directive('uibTooltipPopup', function () {
  return {
    replace: true,
    scope: {
      content: '@',
      placement: '@',
      popupClass: '@',
      animation: '&',
      isOpen: '&'
    },
    templateUrl: 'template/tooltip/tooltip-popup.html',
    link: function (scope, element) {
      element.addClass('tooltip');
    }
  };
}).directive('uibTooltip', [
  '$uibTooltip',
  function ($uibTooltip) {
    return $uibTooltip('uibTooltip', 'tooltip', 'mouseenter');
  }
]).directive('uibTooltipTemplatePopup', function () {
  return {
    replace: true,
    scope: {
      contentExp: '&',
      placement: '@',
      popupClass: '@',
      animation: '&',
      isOpen: '&',
      originScope: '&'
    },
    templateUrl: 'template/tooltip/tooltip-template-popup.html',
    link: function (scope, element) {
      element.addClass('tooltip');
    }
  };
}).directive('uibTooltipTemplate', [
  '$uibTooltip',
  function ($uibTooltip) {
    return $uibTooltip('uibTooltipTemplate', 'tooltip', 'mouseenter', { useContentExp: true });
  }
]).directive('uibTooltipHtmlPopup', function () {
  return {
    replace: true,
    scope: {
      contentExp: '&',
      placement: '@',
      popupClass: '@',
      animation: '&',
      isOpen: '&'
    },
    templateUrl: 'template/tooltip/tooltip-html-popup.html',
    link: function (scope, element) {
      element.addClass('tooltip');
    }
  };
}).directive('uibTooltipHtml', [
  '$uibTooltip',
  function ($uibTooltip) {
    return $uibTooltip('uibTooltipHtml', 'tooltip', 'mouseenter', { useContentExp: true });
  }
]);
/* Deprecated tooltip below */
angular.module('ui.bootstrap.tooltip').value('$tooltipSuppressWarning', false).provider('$tooltip', [
  '$uibTooltipProvider',
  function ($uibTooltipProvider) {
    angular.extend(this, $uibTooltipProvider);
    this.$get = [
      '$log',
      '$tooltipSuppressWarning',
      '$injector',
      function ($log, $tooltipSuppressWarning, $injector) {
        if (!$tooltipSuppressWarning) {
          $log.warn('$tooltip is now deprecated. Use $uibTooltip instead.');
        }
        return $injector.invoke($uibTooltipProvider.$get);
      }
    ];
  }
]).directive('tooltipTemplateTransclude', [
  '$animate',
  '$sce',
  '$compile',
  '$templateRequest',
  '$log',
  '$tooltipSuppressWarning',
  function ($animate, $sce, $compile, $templateRequest, $log, $tooltipSuppressWarning) {
    return {
      link: function (scope, elem, attrs) {
        if (!$tooltipSuppressWarning) {
          $log.warn('tooltip-template-transclude is now deprecated. Use uib-tooltip-template-transclude instead.');
        }
        var origScope = scope.$eval(attrs.tooltipTemplateTranscludeScope);
        var changeCounter = 0, currentScope, previousElement, currentElement;
        var cleanupLastIncludeContent = function () {
          if (previousElement) {
            previousElement.remove();
            previousElement = null;
          }
          if (currentScope) {
            currentScope.$destroy();
            currentScope = null;
          }
          if (currentElement) {
            $animate.leave(currentElement).then(function () {
              previousElement = null;
            });
            previousElement = currentElement;
            currentElement = null;
          }
        };
        scope.$watch($sce.parseAsResourceUrl(attrs.tooltipTemplateTransclude), function (src) {
          var thisChangeId = ++changeCounter;
          if (src) {
            //set the 2nd param to true to ignore the template request error so that the inner
            //contents and scope can be cleaned up.
            $templateRequest(src, true).then(function (response) {
              if (thisChangeId !== changeCounter) {
                return;
              }
              var newScope = origScope.$new();
              var template = response;
              var clone = $compile(template)(newScope, function (clone) {
                  cleanupLastIncludeContent();
                  $animate.enter(clone, elem);
                });
              currentScope = newScope;
              currentElement = clone;
              currentScope.$emit('$includeContentLoaded', src);
            }, function () {
              if (thisChangeId === changeCounter) {
                cleanupLastIncludeContent();
                scope.$emit('$includeContentError', src);
              }
            });
            scope.$emit('$includeContentRequested', src);
          } else {
            cleanupLastIncludeContent();
          }
        });
        scope.$on('$destroy', cleanupLastIncludeContent);
      }
    };
  }
]).directive('tooltipClasses', [
  '$log',
  '$tooltipSuppressWarning',
  function ($log, $tooltipSuppressWarning) {
    return {
      restrict: 'A',
      link: function (scope, element, attrs) {
        if (!$tooltipSuppressWarning) {
          $log.warn('tooltip-classes is now deprecated. Use uib-tooltip-classes instead.');
        }
        if (scope.placement) {
          element.addClass(scope.placement);
        }
        if (scope.popupClass) {
          element.addClass(scope.popupClass);
        }
        if (scope.animation()) {
          element.addClass(attrs.tooltipAnimationClass);
        }
      }
    };
  }
]).directive('tooltipPopup', [
  '$log',
  '$tooltipSuppressWarning',
  function ($log, $tooltipSuppressWarning) {
    return {
      replace: true,
      scope: {
        content: '@',
        placement: '@',
        popupClass: '@',
        animation: '&',
        isOpen: '&'
      },
      templateUrl: 'template/tooltip/tooltip-popup.html',
      link: function (scope, element) {
        if (!$tooltipSuppressWarning) {
          $log.warn('tooltip-popup is now deprecated. Use uib-tooltip-popup instead.');
        }
        element.addClass('tooltip');
      }
    };
  }
]).directive('tooltip', [
  '$tooltip',
  function ($tooltip) {
    return $tooltip('tooltip', 'tooltip', 'mouseenter');
  }
]).directive('tooltipTemplatePopup', [
  '$log',
  '$tooltipSuppressWarning',
  function ($log, $tooltipSuppressWarning) {
    return {
      replace: true,
      scope: {
        contentExp: '&',
        placement: '@',
        popupClass: '@',
        animation: '&',
        isOpen: '&',
        originScope: '&'
      },
      templateUrl: 'template/tooltip/tooltip-template-popup.html',
      link: function (scope, element) {
        if (!$tooltipSuppressWarning) {
          $log.warn('tooltip-template-popup is now deprecated. Use uib-tooltip-template-popup instead.');
        }
        element.addClass('tooltip');
      }
    };
  }
]).directive('tooltipTemplate', [
  '$tooltip',
  function ($tooltip) {
    return $tooltip('tooltipTemplate', 'tooltip', 'mouseenter', { useContentExp: true });
  }
]).directive('tooltipHtmlPopup', [
  '$log',
  '$tooltipSuppressWarning',
  function ($log, $tooltipSuppressWarning) {
    return {
      replace: true,
      scope: {
        contentExp: '&',
        placement: '@',
        popupClass: '@',
        animation: '&',
        isOpen: '&'
      },
      templateUrl: 'template/tooltip/tooltip-html-popup.html',
      link: function (scope, element) {
        if (!$tooltipSuppressWarning) {
          $log.warn('tooltip-html-popup is now deprecated. Use uib-tooltip-html-popup instead.');
        }
        element.addClass('tooltip');
      }
    };
  }
]).directive('tooltipHtml', [
  '$tooltip',
  function ($tooltip) {
    return $tooltip('tooltipHtml', 'tooltip', 'mouseenter', { useContentExp: true });
  }
]);
/**
 * The following features are still outstanding: popup delay, animation as a
 * function, placement as a function, inside, support for more triggers than
 * just mouse enter/leave, and selector delegatation.
 */
angular.module('ui.bootstrap.popover', ['ui.bootstrap.tooltip']).directive('uibPopoverTemplatePopup', function () {
  return {
    replace: true,
    scope: {
      title: '@',
      contentExp: '&',
      placement: '@',
      popupClass: '@',
      animation: '&',
      isOpen: '&',
      originScope: '&'
    },
    templateUrl: 'template/popover/popover-template.html',
    link: function (scope, element) {
      element.addClass('popover');
    }
  };
}).directive('uibPopoverTemplate', [
  '$uibTooltip',
  function ($uibTooltip) {
    return $uibTooltip('uibPopoverTemplate', 'popover', 'click', { useContentExp: true });
  }
]).directive('uibPopoverHtmlPopup', function () {
  return {
    replace: true,
    scope: {
      contentExp: '&',
      title: '@',
      placement: '@',
      popupClass: '@',
      animation: '&',
      isOpen: '&'
    },
    templateUrl: 'template/popover/popover-html.html',
    link: function (scope, element) {
      element.addClass('popover');
    }
  };
}).directive('uibPopoverHtml', [
  '$uibTooltip',
  function ($uibTooltip) {
    return $uibTooltip('uibPopoverHtml', 'popover', 'click', { useContentExp: true });
  }
]).directive('uibPopoverPopup', function () {
  return {
    replace: true,
    scope: {
      title: '@',
      content: '@',
      placement: '@',
      popupClass: '@',
      animation: '&',
      isOpen: '&'
    },
    templateUrl: 'template/popover/popover.html',
    link: function (scope, element) {
      element.addClass('popover');
    }
  };
}).directive('uibPopover', [
  '$uibTooltip',
  function ($uibTooltip) {
    return $uibTooltip('uibPopover', 'popover', 'click');
  }
]);
/* Deprecated popover below */
angular.module('ui.bootstrap.popover').value('$popoverSuppressWarning', false).directive('popoverTemplatePopup', [
  '$log',
  '$popoverSuppressWarning',
  function ($log, $popoverSuppressWarning) {
    return {
      replace: true,
      scope: {
        title: '@',
        contentExp: '&',
        placement: '@',
        popupClass: '@',
        animation: '&',
        isOpen: '&',
        originScope: '&'
      },
      templateUrl: 'template/popover/popover-template.html',
      link: function (scope, element) {
        if (!$popoverSuppressWarning) {
          $log.warn('popover-template-popup is now deprecated. Use uib-popover-template-popup instead.');
        }
        element.addClass('popover');
      }
    };
  }
]).directive('popoverTemplate', [
  '$tooltip',
  function ($tooltip) {
    return $tooltip('popoverTemplate', 'popover', 'click', { useContentExp: true });
  }
]).directive('popoverHtmlPopup', [
  '$log',
  '$popoverSuppressWarning',
  function ($log, $popoverSuppressWarning) {
    return {
      replace: true,
      scope: {
        contentExp: '&',
        title: '@',
        placement: '@',
        popupClass: '@',
        animation: '&',
        isOpen: '&'
      },
      templateUrl: 'template/popover/popover-html.html',
      link: function (scope, element) {
        if (!$popoverSuppressWarning) {
          $log.warn('popover-html-popup is now deprecated. Use uib-popover-html-popup instead.');
        }
        element.addClass('popover');
      }
    };
  }
]).directive('popoverHtml', [
  '$tooltip',
  function ($tooltip) {
    return $tooltip('popoverHtml', 'popover', 'click', { useContentExp: true });
  }
]).directive('popoverPopup', [
  '$log',
  '$popoverSuppressWarning',
  function ($log, $popoverSuppressWarning) {
    return {
      replace: true,
      scope: {
        title: '@',
        content: '@',
        placement: '@',
        popupClass: '@',
        animation: '&',
        isOpen: '&'
      },
      templateUrl: 'template/popover/popover.html',
      link: function (scope, element) {
        if (!$popoverSuppressWarning) {
          $log.warn('popover-popup is now deprecated. Use uib-popover-popup instead.');
        }
        element.addClass('popover');
      }
    };
  }
]).directive('popover', [
  '$tooltip',
  function ($tooltip) {
    return $tooltip('popover', 'popover', 'click');
  }
]);
angular.module('ui.bootstrap.progressbar', []).constant('uibProgressConfig', {
  animate: true,
  max: 100
}).controller('UibProgressController', [
  '$scope',
  '$attrs',
  'uibProgressConfig',
  function ($scope, $attrs, progressConfig) {
    var self = this, animate = angular.isDefined($attrs.animate) ? $scope.$parent.$eval($attrs.animate) : progressConfig.animate;
    this.bars = [];
    $scope.max = angular.isDefined($scope.max) ? $scope.max : progressConfig.max;
    this.addBar = function (bar, element, attrs) {
      if (!animate) {
        element.css({ 'transition': 'none' });
      }
      this.bars.push(bar);
      bar.max = $scope.max;
      bar.title = attrs && angular.isDefined(attrs.title) ? attrs.title : 'progressbar';
      bar.$watch('value', function (value) {
        bar.recalculatePercentage();
      });
      bar.recalculatePercentage = function () {
        bar.percent = +(100 * bar.value / bar.max).toFixed(2);
        var totalPercentage = self.bars.reduce(function (total, bar) {
            return total + bar.percent;
          }, 0);
        if (totalPercentage > 100) {
          bar.percent -= totalPercentage - 100;
        }
      };
      bar.$on('$destroy', function () {
        element = null;
        self.removeBar(bar);
      });
    };
    this.removeBar = function (bar) {
      this.bars.splice(this.bars.indexOf(bar), 1);
    };
    $scope.$watch('max', function (max) {
      self.bars.forEach(function (bar) {
        bar.max = $scope.max;
        bar.recalculatePercentage();
      });
    });
  }
]).directive('uibProgress', function () {
  return {
    replace: true,
    transclude: true,
    controller: 'UibProgressController',
    require: 'uibProgress',
    scope: { max: '=?' },
    templateUrl: 'template/progressbar/progress.html'
  };
}).directive('uibBar', function () {
  return {
    replace: true,
    transclude: true,
    require: '^uibProgress',
    scope: {
      value: '=',
      type: '@'
    },
    templateUrl: 'template/progressbar/bar.html',
    link: function (scope, element, attrs, progressCtrl) {
      progressCtrl.addBar(scope, element, attrs);
    }
  };
}).directive('uibProgressbar', function () {
  return {
    replace: true,
    transclude: true,
    controller: 'UibProgressController',
    scope: {
      value: '=',
      max: '=?',
      type: '@'
    },
    templateUrl: 'template/progressbar/progressbar.html',
    link: function (scope, element, attrs, progressCtrl) {
      progressCtrl.addBar(scope, angular.element(element.children()[0]), { title: attrs.title });
    }
  };
});
/* Deprecated progressbar below */
angular.module('ui.bootstrap.progressbar').value('$progressSuppressWarning', false).controller('ProgressController', [
  '$scope',
  '$attrs',
  '$controller',
  '$log',
  '$progressSuppressWarning',
  function ($scope, $attrs, $controller, $log, $progressSuppressWarning) {
    if (!$progressSuppressWarning) {
      $log.warn('ProgressController is now deprecated. Use UibProgressController instead.');
    }
    return $controller('UibProgressController', {
      $scope: $scope,
      $attrs: $attrs
    });
  }
]).directive('progress', [
  '$log',
  '$progressSuppressWarning',
  function ($log, $progressSuppressWarning) {
    return {
      replace: true,
      transclude: true,
      controller: 'ProgressController',
      require: 'progress',
      scope: {
        max: '=?',
        title: '@?'
      },
      templateUrl: 'template/progressbar/progress.html',
      link: function () {
        if (!$progressSuppressWarning) {
          $log.warn('progress is now deprecated. Use uib-progress instead.');
        }
      }
    };
  }
]).directive('bar', [
  '$log',
  '$progressSuppressWarning',
  function ($log, $progressSuppressWarning) {
    return {
      replace: true,
      transclude: true,
      require: '^progress',
      scope: {
        value: '=',
        type: '@'
      },
      templateUrl: 'template/progressbar/bar.html',
      link: function (scope, element, attrs, progressCtrl) {
        if (!$progressSuppressWarning) {
          $log.warn('bar is now deprecated. Use uib-bar instead.');
        }
        progressCtrl.addBar(scope, element);
      }
    };
  }
]).directive('progressbar', [
  '$log',
  '$progressSuppressWarning',
  function ($log, $progressSuppressWarning) {
    return {
      replace: true,
      transclude: true,
      controller: 'ProgressController',
      scope: {
        value: '=',
        max: '=?',
        type: '@'
      },
      templateUrl: 'template/progressbar/progressbar.html',
      link: function (scope, element, attrs, progressCtrl) {
        if (!$progressSuppressWarning) {
          $log.warn('progressbar is now deprecated. Use uib-progressbar instead.');
        }
        progressCtrl.addBar(scope, angular.element(element.children()[0]), { title: attrs.title });
      }
    };
  }
]);
angular.module('ui.bootstrap.rating', []).constant('uibRatingConfig', {
  max: 5,
  stateOn: null,
  stateOff: null,
  titles: [
    'one',
    'two',
    'three',
    'four',
    'five'
  ]
}).controller('UibRatingController', [
  '$scope',
  '$attrs',
  'uibRatingConfig',
  function ($scope, $attrs, ratingConfig) {
    var ngModelCtrl = { $setViewValue: angular.noop };
    this.init = function (ngModelCtrl_) {
      ngModelCtrl = ngModelCtrl_;
      ngModelCtrl.$render = this.render;
      ngModelCtrl.$formatters.push(function (value) {
        if (angular.isNumber(value) && value << 0 !== value) {
          value = Math.round(value);
        }
        return value;
      });
      this.stateOn = angular.isDefined($attrs.stateOn) ? $scope.$parent.$eval($attrs.stateOn) : ratingConfig.stateOn;
      this.stateOff = angular.isDefined($attrs.stateOff) ? $scope.$parent.$eval($attrs.stateOff) : ratingConfig.stateOff;
      var tmpTitles = angular.isDefined($attrs.titles) ? $scope.$parent.$eval($attrs.titles) : ratingConfig.titles;
      this.titles = angular.isArray(tmpTitles) && tmpTitles.length > 0 ? tmpTitles : ratingConfig.titles;
      var ratingStates = angular.isDefined($attrs.ratingStates) ? $scope.$parent.$eval($attrs.ratingStates) : new Array(angular.isDefined($attrs.max) ? $scope.$parent.$eval($attrs.max) : ratingConfig.max);
      $scope.range = this.buildTemplateObjects(ratingStates);
    };
    this.buildTemplateObjects = function (states) {
      for (var i = 0, n = states.length; i < n; i++) {
        states[i] = angular.extend({ index: i }, {
          stateOn: this.stateOn,
          stateOff: this.stateOff,
          title: this.getTitle(i)
        }, states[i]);
      }
      return states;
    };
    this.getTitle = function (index) {
      if (index >= this.titles.length) {
        return index + 1;
      } else {
        return this.titles[index];
      }
    };
    $scope.rate = function (value) {
      if (!$scope.readonly && value >= 0 && value <= $scope.range.length) {
        ngModelCtrl.$setViewValue(ngModelCtrl.$viewValue === value ? 0 : value);
        ngModelCtrl.$render();
      }
    };
    $scope.enter = function (value) {
      if (!$scope.readonly) {
        $scope.value = value;
      }
      $scope.onHover({ value: value });
    };
    $scope.reset = function () {
      $scope.value = ngModelCtrl.$viewValue;
      $scope.onLeave();
    };
    $scope.onKeydown = function (evt) {
      if (/(37|38|39|40)/.test(evt.which)) {
        evt.preventDefault();
        evt.stopPropagation();
        $scope.rate($scope.value + (evt.which === 38 || evt.which === 39 ? 1 : -1));
      }
    };
    this.render = function () {
      $scope.value = ngModelCtrl.$viewValue;
    };
  }
]).directive('uibRating', function () {
  return {
    require: [
      'uibRating',
      'ngModel'
    ],
    scope: {
      readonly: '=?',
      onHover: '&',
      onLeave: '&'
    },
    controller: 'UibRatingController',
    templateUrl: 'template/rating/rating.html',
    replace: true,
    link: function (scope, element, attrs, ctrls) {
      var ratingCtrl = ctrls[0], ngModelCtrl = ctrls[1];
      ratingCtrl.init(ngModelCtrl);
    }
  };
});
/* Deprecated rating below */
angular.module('ui.bootstrap.rating').value('$ratingSuppressWarning', false).controller('RatingController', [
  '$scope',
  '$attrs',
  '$controller',
  '$log',
  '$ratingSuppressWarning',
  function ($scope, $attrs, $controller, $log, $ratingSuppressWarning) {
    if (!$ratingSuppressWarning) {
      $log.warn('RatingController is now deprecated. Use UibRatingController instead.');
    }
    return $controller('UibRatingController', {
      $scope: $scope,
      $attrs: $attrs
    });
  }
]).directive('rating', [
  '$log',
  '$ratingSuppressWarning',
  function ($log, $ratingSuppressWarning) {
    return {
      require: [
        'rating',
        'ngModel'
      ],
      scope: {
        readonly: '=?',
        onHover: '&',
        onLeave: '&'
      },
      controller: 'RatingController',
      templateUrl: 'template/rating/rating.html',
      replace: true,
      link: function (scope, element, attrs, ctrls) {
        if (!$ratingSuppressWarning) {
          $log.warn('rating is now deprecated. Use uib-rating instead.');
        }
        var ratingCtrl = ctrls[0], ngModelCtrl = ctrls[1];
        ratingCtrl.init(ngModelCtrl);
      }
    };
  }
]);
/**
 * @ngdoc overview
 * @name ui.bootstrap.tabs
 *
 * @description
 * AngularJS version of the tabs directive.
 */
angular.module('ui.bootstrap.tabs', []).controller('UibTabsetController', [
  '$scope',
  function ($scope) {
    var ctrl = this, tabs = ctrl.tabs = $scope.tabs = [];
    ctrl.select = function (selectedTab) {
      angular.forEach(tabs, function (tab) {
        if (tab.active && tab !== selectedTab) {
          tab.active = false;
          tab.onDeselect();
          selectedTab.selectCalled = false;
        }
      });
      selectedTab.active = true;
      // only call select if it has not already been called
      if (!selectedTab.selectCalled) {
        selectedTab.onSelect();
        selectedTab.selectCalled = true;
      }
    };
    ctrl.addTab = function addTab(tab) {
      tabs.push(tab);
      // we can't run the select function on the first tab
      // since that would select it twice
      if (tabs.length === 1 && tab.active !== false) {
        tab.active = true;
      } else if (tab.active) {
        ctrl.select(tab);
      } else {
        tab.active = false;
      }
    };
    ctrl.removeTab = function removeTab(tab) {
      var index = tabs.indexOf(tab);
      //Select a new tab if the tab to be removed is selected and not destroyed
      if (tab.active && tabs.length > 1 && !destroyed) {
        //If this is the last tab, select the previous tab. else, the next tab.
        var newActiveIndex = index == tabs.length - 1 ? index - 1 : index + 1;
        ctrl.select(tabs[newActiveIndex]);
      }
      tabs.splice(index, 1);
    };
    var destroyed;
    $scope.$on('$destroy', function () {
      destroyed = true;
    });
  }
]).directive('uibTabset', function () {
  return {
    restrict: 'EA',
    transclude: true,
    replace: true,
    scope: { type: '@' },
    controller: 'UibTabsetController',
    templateUrl: 'template/tabs/tabset.html',
    link: function (scope, element, attrs) {
      scope.vertical = angular.isDefined(attrs.vertical) ? scope.$parent.$eval(attrs.vertical) : false;
      scope.justified = angular.isDefined(attrs.justified) ? scope.$parent.$eval(attrs.justified) : false;
    }
  };
}).directive('uibTab', [
  '$parse',
  function ($parse) {
    return {
      require: '^uibTabset',
      restrict: 'EA',
      replace: true,
      templateUrl: 'template/tabs/tab.html',
      transclude: true,
      scope: {
        active: '=?',
        heading: '@',
        onSelect: '&select',
        onDeselect: '&deselect'
      },
      controller: function () {
      },
      link: function (scope, elm, attrs, tabsetCtrl, transclude) {
        scope.$watch('active', function (active) {
          if (active) {
            tabsetCtrl.select(scope);
          }
        });
        scope.disabled = false;
        if (attrs.disable) {
          scope.$parent.$watch($parse(attrs.disable), function (value) {
            scope.disabled = !!value;
          });
        }
        scope.select = function () {
          if (!scope.disabled) {
            scope.active = true;
          }
        };
        tabsetCtrl.addTab(scope);
        scope.$on('$destroy', function () {
          tabsetCtrl.removeTab(scope);
        });
        //We need to transclude later, once the content container is ready.
        //when this link happens, we're inside a tab heading.
        scope.$transcludeFn = transclude;
      }
    };
  }
]).directive('uibTabHeadingTransclude', function () {
  return {
    restrict: 'A',
    require: [
      '?^uibTab',
      '?^tab'
    ],
    link: function (scope, elm) {
      scope.$watch('headingElement', function updateHeadingElement(heading) {
        if (heading) {
          elm.html('');
          elm.append(heading);
        }
      });
    }
  };
}).directive('uibTabContentTransclude', function () {
  return {
    restrict: 'A',
    require: [
      '?^uibTabset',
      '?^tabset'
    ],
    link: function (scope, elm, attrs) {
      var tab = scope.$eval(attrs.uibTabContentTransclude);
      //Now our tab is ready to be transcluded: both the tab heading area
      //and the tab content area are loaded.  Transclude 'em both.
      tab.$transcludeFn(tab.$parent, function (contents) {
        angular.forEach(contents, function (node) {
          if (isTabHeading(node)) {
            //Let tabHeadingTransclude know.
            tab.headingElement = node;
          } else {
            elm.append(node);
          }
        });
      });
    }
  };
  function isTabHeading(node) {
    return node.tagName && (node.hasAttribute('tab-heading') || node.hasAttribute('data-tab-heading') || node.hasAttribute('x-tab-heading') || node.hasAttribute('uib-tab-heading') || node.hasAttribute('data-uib-tab-heading') || node.hasAttribute('x-uib-tab-heading') || node.tagName.toLowerCase() === 'tab-heading' || node.tagName.toLowerCase() === 'data-tab-heading' || node.tagName.toLowerCase() === 'x-tab-heading' || node.tagName.toLowerCase() === 'uib-tab-heading' || node.tagName.toLowerCase() === 'data-uib-tab-heading' || node.tagName.toLowerCase() === 'x-uib-tab-heading');
  }
});
/* deprecated tabs below */
angular.module('ui.bootstrap.tabs').value('$tabsSuppressWarning', false).controller('TabsetController', [
  '$scope',
  '$controller',
  '$log',
  '$tabsSuppressWarning',
  function ($scope, $controller, $log, $tabsSuppressWarning) {
    if (!$tabsSuppressWarning) {
      $log.warn('TabsetController is now deprecated. Use UibTabsetController instead.');
    }
    return $controller('UibTabsetController', { $scope: $scope });
  }
]).directive('tabset', [
  '$log',
  '$tabsSuppressWarning',
  function ($log, $tabsSuppressWarning) {
    return {
      restrict: 'EA',
      transclude: true,
      replace: true,
      scope: { type: '@' },
      controller: 'TabsetController',
      templateUrl: 'template/tabs/tabset.html',
      link: function (scope, element, attrs) {
        if (!$tabsSuppressWarning) {
          $log.warn('tabset is now deprecated. Use uib-tabset instead.');
        }
        scope.vertical = angular.isDefined(attrs.vertical) ? scope.$parent.$eval(attrs.vertical) : false;
        scope.justified = angular.isDefined(attrs.justified) ? scope.$parent.$eval(attrs.justified) : false;
      }
    };
  }
]).directive('tab', [
  '$parse',
  '$log',
  '$tabsSuppressWarning',
  function ($parse, $log, $tabsSuppressWarning) {
    return {
      require: '^tabset',
      restrict: 'EA',
      replace: true,
      templateUrl: 'template/tabs/tab.html',
      transclude: true,
      scope: {
        active: '=?',
        heading: '@',
        onSelect: '&select',
        onDeselect: '&deselect'
      },
      controller: function () {
      },
      link: function (scope, elm, attrs, tabsetCtrl, transclude) {
        if (!$tabsSuppressWarning) {
          $log.warn('tab is now deprecated. Use uib-tab instead.');
        }
        scope.$watch('active', function (active) {
          if (active) {
            tabsetCtrl.select(scope);
          }
        });
        scope.disabled = false;
        if (attrs.disable) {
          scope.$parent.$watch($parse(attrs.disable), function (value) {
            scope.disabled = !!value;
          });
        }
        scope.select = function () {
          if (!scope.disabled) {
            scope.active = true;
          }
        };
        tabsetCtrl.addTab(scope);
        scope.$on('$destroy', function () {
          tabsetCtrl.removeTab(scope);
        });
        //We need to transclude later, once the content container is ready.
        //when this link happens, we're inside a tab heading.
        scope.$transcludeFn = transclude;
      }
    };
  }
]).directive('tabHeadingTransclude', [
  '$log',
  '$tabsSuppressWarning',
  function ($log, $tabsSuppressWarning) {
    return {
      restrict: 'A',
      require: '^tab',
      link: function (scope, elm) {
        if (!$tabsSuppressWarning) {
          $log.warn('tab-heading-transclude is now deprecated. Use uib-tab-heading-transclude instead.');
        }
        scope.$watch('headingElement', function updateHeadingElement(heading) {
          if (heading) {
            elm.html('');
            elm.append(heading);
          }
        });
      }
    };
  }
]).directive('tabContentTransclude', [
  '$log',
  '$tabsSuppressWarning',
  function ($log, $tabsSuppressWarning) {
    return {
      restrict: 'A',
      require: '^tabset',
      link: function (scope, elm, attrs) {
        if (!$tabsSuppressWarning) {
          $log.warn('tab-content-transclude is now deprecated. Use uib-tab-content-transclude instead.');
        }
        var tab = scope.$eval(attrs.tabContentTransclude);
        //Now our tab is ready to be transcluded: both the tab heading area
        //and the tab content area are loaded.  Transclude 'em both.
        tab.$transcludeFn(tab.$parent, function (contents) {
          angular.forEach(contents, function (node) {
            if (isTabHeading(node)) {
              //Let tabHeadingTransclude know.
              tab.headingElement = node;
            } else {
              elm.append(node);
            }
          });
        });
      }
    };
    function isTabHeading(node) {
      return node.tagName && (node.hasAttribute('tab-heading') || node.hasAttribute('data-tab-heading') || node.hasAttribute('x-tab-heading') || node.tagName.toLowerCase() === 'tab-heading' || node.tagName.toLowerCase() === 'data-tab-heading' || node.tagName.toLowerCase() === 'x-tab-heading');
    }
  }
]);
angular.module('ui.bootstrap.timepicker', []).constant('uibTimepickerConfig', {
  hourStep: 1,
  minuteStep: 1,
  showMeridian: true,
  meridians: null,
  readonlyInput: false,
  mousewheel: true,
  arrowkeys: true,
  showSpinners: true
}).controller('UibTimepickerController', [
  '$scope',
  '$attrs',
  '$parse',
  '$log',
  '$locale',
  'uibTimepickerConfig',
  function ($scope, $attrs, $parse, $log, $locale, timepickerConfig) {
    var selected = new Date(), ngModelCtrl = { $setViewValue: angular.noop },
      // nullModelCtrl
      meridians = angular.isDefined($attrs.meridians) ? $scope.$parent.$eval($attrs.meridians) : timepickerConfig.meridians || $locale.DATETIME_FORMATS.AMPMS;
    this.init = function (ngModelCtrl_, inputs) {
      ngModelCtrl = ngModelCtrl_;
      ngModelCtrl.$render = this.render;
      ngModelCtrl.$formatters.unshift(function (modelValue) {
        return modelValue ? new Date(modelValue) : null;
      });
      var hoursInputEl = inputs.eq(0), minutesInputEl = inputs.eq(1);
      var mousewheel = angular.isDefined($attrs.mousewheel) ? $scope.$parent.$eval($attrs.mousewheel) : timepickerConfig.mousewheel;
      if (mousewheel) {
        this.setupMousewheelEvents(hoursInputEl, minutesInputEl);
      }
      var arrowkeys = angular.isDefined($attrs.arrowkeys) ? $scope.$parent.$eval($attrs.arrowkeys) : timepickerConfig.arrowkeys;
      if (arrowkeys) {
        this.setupArrowkeyEvents(hoursInputEl, minutesInputEl);
      }
      $scope.readonlyInput = angular.isDefined($attrs.readonlyInput) ? $scope.$parent.$eval($attrs.readonlyInput) : timepickerConfig.readonlyInput;
      this.setupInputEvents(hoursInputEl, minutesInputEl);
    };
    var hourStep = timepickerConfig.hourStep;
    if ($attrs.hourStep) {
      $scope.$parent.$watch($parse($attrs.hourStep), function (value) {
        hourStep = parseInt(value, 10);
      });
    }
    var minuteStep = timepickerConfig.minuteStep;
    if ($attrs.minuteStep) {
      $scope.$parent.$watch($parse($attrs.minuteStep), function (value) {
        minuteStep = parseInt(value, 10);
      });
    }
    var min;
    $scope.$parent.$watch($parse($attrs.min), function (value) {
      var dt = new Date(value);
      min = isNaN(dt) ? undefined : dt;
    });
    var max;
    $scope.$parent.$watch($parse($attrs.max), function (value) {
      var dt = new Date(value);
      max = isNaN(dt) ? undefined : dt;
    });
    $scope.noIncrementHours = function () {
      var incrementedSelected = addMinutes(selected, hourStep * 60);
      return incrementedSelected > max || incrementedSelected < selected && incrementedSelected < min;
    };
    $scope.noDecrementHours = function () {
      var decrementedSelected = addMinutes(selected, -hourStep * 60);
      return decrementedSelected < min || decrementedSelected > selected && decrementedSelected > max;
    };
    $scope.noIncrementMinutes = function () {
      var incrementedSelected = addMinutes(selected, minuteStep);
      return incrementedSelected > max || incrementedSelected < selected && incrementedSelected < min;
    };
    $scope.noDecrementMinutes = function () {
      var decrementedSelected = addMinutes(selected, -minuteStep);
      return decrementedSelected < min || decrementedSelected > selected && decrementedSelected > max;
    };
    $scope.noToggleMeridian = function () {
      if (selected.getHours() < 13) {
        return addMinutes(selected, 12 * 60) > max;
      } else {
        return addMinutes(selected, -12 * 60) < min;
      }
    };
    // 12H / 24H mode
    $scope.showMeridian = timepickerConfig.showMeridian;
    if ($attrs.showMeridian) {
      $scope.$parent.$watch($parse($attrs.showMeridian), function (value) {
        $scope.showMeridian = !!value;
        if (ngModelCtrl.$error.time) {
          // Evaluate from template
          var hours = getHoursFromTemplate(), minutes = getMinutesFromTemplate();
          if (angular.isDefined(hours) && angular.isDefined(minutes)) {
            selected.setHours(hours);
            refresh();
          }
        } else {
          updateTemplate();
        }
      });
    }
    // Get $scope.hours in 24H mode if valid
    function getHoursFromTemplate() {
      var hours = parseInt($scope.hours, 10);
      var valid = $scope.showMeridian ? hours > 0 && hours < 13 : hours >= 0 && hours < 24;
      if (!valid) {
        return undefined;
      }
      if ($scope.showMeridian) {
        if (hours === 12) {
          hours = 0;
        }
        if ($scope.meridian === meridians[1]) {
          hours = hours + 12;
        }
      }
      return hours;
    }
    function getMinutesFromTemplate() {
      var minutes = parseInt($scope.minutes, 10);
      return minutes >= 0 && minutes < 60 ? minutes : undefined;
    }
    function pad(value) {
      return angular.isDefined(value) && value.toString().length < 2 ? '0' + value : value.toString();
    }
    // Respond on mousewheel spin
    this.setupMousewheelEvents = function (hoursInputEl, minutesInputEl) {
      var isScrollingUp = function (e) {
        if (e.originalEvent) {
          e = e.originalEvent;
        }
        //pick correct delta variable depending on event
        var delta = e.wheelDelta ? e.wheelDelta : -e.deltaY;
        return e.detail || delta > 0;
      };
      hoursInputEl.bind('mousewheel wheel', function (e) {
        $scope.$apply(isScrollingUp(e) ? $scope.incrementHours() : $scope.decrementHours());
        e.preventDefault();
      });
      minutesInputEl.bind('mousewheel wheel', function (e) {
        $scope.$apply(isScrollingUp(e) ? $scope.incrementMinutes() : $scope.decrementMinutes());
        e.preventDefault();
      });
    };
    // Respond on up/down arrowkeys
    this.setupArrowkeyEvents = function (hoursInputEl, minutesInputEl) {
      hoursInputEl.bind('keydown', function (e) {
        if (e.which === 38) {
          // up
          e.preventDefault();
          $scope.incrementHours();
          $scope.$apply();
        } else if (e.which === 40) {
          // down
          e.preventDefault();
          $scope.decrementHours();
          $scope.$apply();
        }
      });
      minutesInputEl.bind('keydown', function (e) {
        if (e.which === 38) {
          // up
          e.preventDefault();
          $scope.incrementMinutes();
          $scope.$apply();
        } else if (e.which === 40) {
          // down
          e.preventDefault();
          $scope.decrementMinutes();
          $scope.$apply();
        }
      });
    };
    this.setupInputEvents = function (hoursInputEl, minutesInputEl) {
      if ($scope.readonlyInput) {
        $scope.updateHours = angular.noop;
        $scope.updateMinutes = angular.noop;
        return;
      }
      var invalidate = function (invalidHours, invalidMinutes) {
        ngModelCtrl.$setViewValue(null);
        ngModelCtrl.$setValidity('time', false);
        if (angular.isDefined(invalidHours)) {
          $scope.invalidHours = invalidHours;
        }
        if (angular.isDefined(invalidMinutes)) {
          $scope.invalidMinutes = invalidMinutes;
        }
      };
      $scope.updateHours = function () {
        var hours = getHoursFromTemplate(), minutes = getMinutesFromTemplate();
        if (angular.isDefined(hours) && angular.isDefined(minutes)) {
          selected.setHours(hours);
          if (selected < min || selected > max) {
            invalidate(true);
          } else {
            refresh('h');
          }
        } else {
          invalidate(true);
        }
      };
      hoursInputEl.bind('blur', function (e) {
        if (!$scope.invalidHours && $scope.hours < 10) {
          $scope.$apply(function () {
            $scope.hours = pad($scope.hours);
          });
        }
      });
      $scope.updateMinutes = function () {
        var minutes = getMinutesFromTemplate(), hours = getHoursFromTemplate();
        if (angular.isDefined(minutes) && angular.isDefined(hours)) {
          selected.setMinutes(minutes);
          if (selected < min || selected > max) {
            invalidate(undefined, true);
          } else {
            refresh('m');
          }
        } else {
          invalidate(undefined, true);
        }
      };
      minutesInputEl.bind('blur', function (e) {
        if (!$scope.invalidMinutes && $scope.minutes < 10) {
          $scope.$apply(function () {
            $scope.minutes = pad($scope.minutes);
          });
        }
      });
    };
    this.render = function () {
      var date = ngModelCtrl.$viewValue;
      if (isNaN(date)) {
        ngModelCtrl.$setValidity('time', false);
        $log.error('Timepicker directive: "ng-model" value must be a Date object, a number of milliseconds since 01.01.1970 or a string representing an RFC2822 or ISO 8601 date.');
      } else {
        if (date) {
          selected = date;
        }
        if (selected < min || selected > max) {
          ngModelCtrl.$setValidity('time', false);
          $scope.invalidHours = true;
          $scope.invalidMinutes = true;
        } else {
          makeValid();
        }
        updateTemplate();
      }
    };
    // Call internally when we know that model is valid.
    function refresh(keyboardChange) {
      makeValid();
      ngModelCtrl.$setViewValue(new Date(selected));
      updateTemplate(keyboardChange);
    }
    function makeValid() {
      ngModelCtrl.$setValidity('time', true);
      $scope.invalidHours = false;
      $scope.invalidMinutes = false;
    }
    function updateTemplate(keyboardChange) {
      var hours = selected.getHours(), minutes = selected.getMinutes();
      if ($scope.showMeridian) {
        hours = hours === 0 || hours === 12 ? 12 : hours % 12;  // Convert 24 to 12 hour system
      }
      $scope.hours = keyboardChange === 'h' ? hours : pad(hours);
      if (keyboardChange !== 'm') {
        $scope.minutes = pad(minutes);
      }
      $scope.meridian = selected.getHours() < 12 ? meridians[0] : meridians[1];
    }
    function addMinutes(date, minutes) {
      var dt = new Date(date.getTime() + minutes * 60000);
      var newDate = new Date(date);
      newDate.setHours(dt.getHours(), dt.getMinutes());
      return newDate;
    }
    function addMinutesToSelected(minutes) {
      selected = addMinutes(selected, minutes);
      refresh();
    }
    $scope.showSpinners = angular.isDefined($attrs.showSpinners) ? $scope.$parent.$eval($attrs.showSpinners) : timepickerConfig.showSpinners;
    $scope.incrementHours = function () {
      if (!$scope.noIncrementHours()) {
        addMinutesToSelected(hourStep * 60);
      }
    };
    $scope.decrementHours = function () {
      if (!$scope.noDecrementHours()) {
        addMinutesToSelected(-hourStep * 60);
      }
    };
    $scope.incrementMinutes = function () {
      if (!$scope.noIncrementMinutes()) {
        addMinutesToSelected(minuteStep);
      }
    };
    $scope.decrementMinutes = function () {
      if (!$scope.noDecrementMinutes()) {
        addMinutesToSelected(-minuteStep);
      }
    };
    $scope.toggleMeridian = function () {
      if (!$scope.noToggleMeridian()) {
        addMinutesToSelected(12 * 60 * (selected.getHours() < 12 ? 1 : -1));
      }
    };
  }
]).directive('uibTimepicker', function () {
  return {
    restrict: 'EA',
    require: [
      'uibTimepicker',
      '?^ngModel'
    ],
    controller: 'UibTimepickerController',
    controllerAs: 'timepicker',
    replace: true,
    scope: {},
    templateUrl: function (element, attrs) {
      return attrs.templateUrl || 'template/timepicker/timepicker.html';
    },
    link: function (scope, element, attrs, ctrls) {
      var timepickerCtrl = ctrls[0], ngModelCtrl = ctrls[1];
      if (ngModelCtrl) {
        timepickerCtrl.init(ngModelCtrl, element.find('input'));
      }
    }
  };
});
/* Deprecated timepicker below */
angular.module('ui.bootstrap.timepicker').value('$timepickerSuppressWarning', false).controller('TimepickerController', [
  '$scope',
  '$attrs',
  '$controller',
  '$log',
  '$timepickerSuppressWarning',
  function ($scope, $attrs, $controller, $log, $timepickerSuppressWarning) {
    if (!$timepickerSuppressWarning) {
      $log.warn('TimepickerController is now deprecated. Use UibTimepickerController instead.');
    }
    return $controller('UibTimepickerController', {
      $scope: $scope,
      $attrs: $attrs
    });
  }
]).directive('timepicker', [
  '$log',
  '$timepickerSuppressWarning',
  function ($log, $timepickerSuppressWarning) {
    return {
      restrict: 'EA',
      require: [
        'timepicker',
        '?^ngModel'
      ],
      controller: 'TimepickerController',
      controllerAs: 'timepicker',
      replace: true,
      scope: {},
      templateUrl: function (element, attrs) {
        return attrs.templateUrl || 'template/timepicker/timepicker.html';
      },
      link: function (scope, element, attrs, ctrls) {
        if (!$timepickerSuppressWarning) {
          $log.warn('timepicker is now deprecated. Use uib-timepicker instead.');
        }
        var timepickerCtrl = ctrls[0], ngModelCtrl = ctrls[1];
        if (ngModelCtrl) {
          timepickerCtrl.init(ngModelCtrl, element.find('input'));
        }
      }
    };
  }
]);
angular.module('ui.bootstrap.typeahead', ['ui.bootstrap.position']).factory('uibTypeaheadParser', [
  '$parse',
  function ($parse) {
    //                      00000111000000000000022200000000000000003333333333333330000000000044000
    var TYPEAHEAD_REGEXP = /^\s*([\s\S]+?)(?:\s+as\s+([\s\S]+?))?\s+for\s+(?:([\$\w][\$\w\d]*))\s+in\s+([\s\S]+?)$/;
    return {
      parse: function (input) {
        var match = input.match(TYPEAHEAD_REGEXP);
        if (!match) {
          throw new Error('Expected typeahead specification in form of "_modelValue_ (as _label_)? for _item_ in _collection_"' + ' but got "' + input + '".');
        }
        return {
          itemName: match[3],
          source: $parse(match[4]),
          viewMapper: $parse(match[2] || match[1]),
          modelMapper: $parse(match[1])
        };
      }
    };
  }
]).directive('uibTypeahead', [
  '$compile',
  '$parse',
  '$q',
  '$timeout',
  '$document',
  '$window',
  '$rootScope',
  '$uibPosition',
  'uibTypeaheadParser',
  function ($compile, $parse, $q, $timeout, $document, $window, $rootScope, $position, typeaheadParser) {
    var HOT_KEYS = [
        9,
        13,
        27,
        38,
        40
      ];
    var eventDebounceTime = 200;
    return {
      require: [
        'ngModel',
        '^?ngModelOptions'
      ],
      link: function (originalScope, element, attrs, ctrls) {
        var modelCtrl = ctrls[0];
        var ngModelOptions = ctrls[1];
        //SUPPORTED ATTRIBUTES (OPTIONS)
        //minimal no of characters that needs to be entered before typeahead kicks-in
        var minLength = originalScope.$eval(attrs.typeaheadMinLength);
        if (!minLength && minLength !== 0) {
          minLength = 1;
        }
        //minimal wait time after last character typed before typeahead kicks-in
        var waitTime = originalScope.$eval(attrs.typeaheadWaitMs) || 0;
        //should it restrict model values to the ones selected from the popup only?
        var isEditable = originalScope.$eval(attrs.typeaheadEditable) !== false;
        //binding to a variable that indicates if matches are being retrieved asynchronously
        var isLoadingSetter = $parse(attrs.typeaheadLoading).assign || angular.noop;
        //a callback executed when a match is selected
        var onSelectCallback = $parse(attrs.typeaheadOnSelect);
        //should it select highlighted popup value when losing focus?
        var isSelectOnBlur = angular.isDefined(attrs.typeaheadSelectOnBlur) ? originalScope.$eval(attrs.typeaheadSelectOnBlur) : false;
        //binding to a variable that indicates if there were no results after the query is completed
        var isNoResultsSetter = $parse(attrs.typeaheadNoResults).assign || angular.noop;
        var inputFormatter = attrs.typeaheadInputFormatter ? $parse(attrs.typeaheadInputFormatter) : undefined;
        var appendToBody = attrs.typeaheadAppendToBody ? originalScope.$eval(attrs.typeaheadAppendToBody) : false;
        var appendToElementId = attrs.typeaheadAppendToElementId || false;
        var focusFirst = originalScope.$eval(attrs.typeaheadFocusFirst) !== false;
        //If input matches an item of the list exactly, select it automatically
        var selectOnExact = attrs.typeaheadSelectOnExact ? originalScope.$eval(attrs.typeaheadSelectOnExact) : false;
        //INTERNAL VARIABLES
        //model setter executed upon match selection
        var parsedModel = $parse(attrs.ngModel);
        var invokeModelSetter = $parse(attrs.ngModel + '($$$p)');
        var $setModelValue = function (scope, newValue) {
          if (angular.isFunction(parsedModel(originalScope)) && ngModelOptions && ngModelOptions.$options && ngModelOptions.$options.getterSetter) {
            return invokeModelSetter(scope, { $$$p: newValue });
          } else {
            return parsedModel.assign(scope, newValue);
          }
        };
        //expressions used by typeahead
        var parserResult = typeaheadParser.parse(attrs.uibTypeahead);
        var hasFocus;
        //Used to avoid bug in iOS webview where iOS keyboard does not fire
        //mousedown & mouseup events
        //Issue #3699
        var selected;
        //create a child scope for the typeahead directive so we are not polluting original scope
        //with typeahead-specific data (matches, query etc.)
        var scope = originalScope.$new();
        var offDestroy = originalScope.$on('$destroy', function () {
            scope.$destroy();
          });
        scope.$on('$destroy', offDestroy);
        // WAI-ARIA
        var popupId = 'typeahead-' + scope.$id + '-' + Math.floor(Math.random() * 10000);
        element.attr({
          'aria-autocomplete': 'list',
          'aria-expanded': false,
          'aria-owns': popupId
        });
        //pop-up element used to display matches
        var popUpEl = angular.element('<div uib-typeahead-popup></div>');
        popUpEl.attr({
          id: popupId,
          matches: 'matches',
          active: 'activeIdx',
          select: 'select(activeIdx)',
          'move-in-progress': 'moveInProgress',
          query: 'query',
          position: 'position'
        });
        //custom item template
        if (angular.isDefined(attrs.typeaheadTemplateUrl)) {
          popUpEl.attr('template-url', attrs.typeaheadTemplateUrl);
        }
        if (angular.isDefined(attrs.typeaheadPopupTemplateUrl)) {
          popUpEl.attr('popup-template-url', attrs.typeaheadPopupTemplateUrl);
        }
        var resetMatches = function () {
          scope.matches = [];
          scope.activeIdx = -1;
          element.attr('aria-expanded', false);
        };
        var getMatchId = function (index) {
          return popupId + '-option-' + index;
        };
        // Indicate that the specified match is the active (pre-selected) item in the list owned by this typeahead.
        // This attribute is added or removed automatically when the `activeIdx` changes.
        scope.$watch('activeIdx', function (index) {
          if (index < 0) {
            element.removeAttr('aria-activedescendant');
          } else {
            element.attr('aria-activedescendant', getMatchId(index));
          }
        });
        var inputIsExactMatch = function (inputValue, index) {
          if (scope.matches.length > index && inputValue) {
            return inputValue.toUpperCase() === scope.matches[index].label.toUpperCase();
          }
          return false;
        };
        var getMatchesAsync = function (inputValue) {
          var locals = { $viewValue: inputValue };
          isLoadingSetter(originalScope, true);
          isNoResultsSetter(originalScope, false);
          $q.when(parserResult.source(originalScope, locals)).then(function (matches) {
            //it might happen that several async queries were in progress if a user were typing fast
            //but we are interested only in responses that correspond to the current view value
            var onCurrentRequest = inputValue === modelCtrl.$viewValue;
            if (onCurrentRequest && hasFocus) {
              if (matches && matches.length > 0) {
                scope.activeIdx = focusFirst ? 0 : -1;
                isNoResultsSetter(originalScope, false);
                scope.matches.length = 0;
                //transform labels
                for (var i = 0; i < matches.length; i++) {
                  locals[parserResult.itemName] = matches[i];
                  scope.matches.push({
                    id: getMatchId(i),
                    label: parserResult.viewMapper(scope, locals),
                    model: matches[i]
                  });
                }
                scope.query = inputValue;
                //position pop-up with matches - we need to re-calculate its position each time we are opening a window
                //with matches as a pop-up might be absolute-positioned and position of an input might have changed on a page
                //due to other elements being rendered
                recalculatePosition();
                element.attr('aria-expanded', true);
                //Select the single remaining option if user input matches
                if (selectOnExact && scope.matches.length === 1 && inputIsExactMatch(inputValue, 0)) {
                  scope.select(0);
                }
              } else {
                resetMatches();
                isNoResultsSetter(originalScope, true);
              }
            }
            if (onCurrentRequest) {
              isLoadingSetter(originalScope, false);
            }
          }, function () {
            resetMatches();
            isLoadingSetter(originalScope, false);
            isNoResultsSetter(originalScope, true);
          });
        };
        // bind events only if appendToBody params exist - performance feature
        if (appendToBody) {
          angular.element($window).bind('resize', fireRecalculating);
          $document.find('body').bind('scroll', fireRecalculating);
        }
        // Declare the timeout promise var outside the function scope so that stacked calls can be cancelled later
        var timeoutEventPromise;
        // Default progress type
        scope.moveInProgress = false;
        function fireRecalculating() {
          if (!scope.moveInProgress) {
            scope.moveInProgress = true;
            scope.$digest();
          }
          // Cancel previous timeout
          if (timeoutEventPromise) {
            $timeout.cancel(timeoutEventPromise);
          }
          // Debounced executing recalculate after events fired
          timeoutEventPromise = $timeout(function () {
            // if popup is visible
            if (scope.matches.length) {
              recalculatePosition();
            }
            scope.moveInProgress = false;
          }, eventDebounceTime);
        }
        // recalculate actual position and set new values to scope
        // after digest loop is popup in right position
        function recalculatePosition() {
          scope.position = appendToBody ? $position.offset(element) : $position.position(element);
          scope.position.top += element.prop('offsetHeight');
        }
        resetMatches();
        //we need to propagate user's query so we can higlight matches
        scope.query = undefined;
        //Declare the timeout promise var outside the function scope so that stacked calls can be cancelled later
        var timeoutPromise;
        var scheduleSearchWithTimeout = function (inputValue) {
          timeoutPromise = $timeout(function () {
            getMatchesAsync(inputValue);
          }, waitTime);
        };
        var cancelPreviousTimeout = function () {
          if (timeoutPromise) {
            $timeout.cancel(timeoutPromise);
          }
        };
        //plug into $parsers pipeline to open a typeahead on view changes initiated from DOM
        //$parsers kick-in on all the changes coming from the view as well as manually triggered by $setViewValue
        modelCtrl.$parsers.unshift(function (inputValue) {
          hasFocus = true;
          if (minLength === 0 || inputValue && inputValue.length >= minLength) {
            if (waitTime > 0) {
              cancelPreviousTimeout();
              scheduleSearchWithTimeout(inputValue);
            } else {
              getMatchesAsync(inputValue);
            }
          } else {
            isLoadingSetter(originalScope, false);
            cancelPreviousTimeout();
            resetMatches();
          }
          if (isEditable) {
            return inputValue;
          } else {
            if (!inputValue) {
              // Reset in case user had typed something previously.
              modelCtrl.$setValidity('editable', true);
              return null;
            } else {
              modelCtrl.$setValidity('editable', false);
              return undefined;
            }
          }
        });
        modelCtrl.$formatters.push(function (modelValue) {
          var candidateViewValue, emptyViewValue;
          var locals = {};
          // The validity may be set to false via $parsers (see above) if
          // the model is restricted to selected values. If the model
          // is set manually it is considered to be valid.
          if (!isEditable) {
            modelCtrl.$setValidity('editable', true);
          }
          if (inputFormatter) {
            locals.$model = modelValue;
            return inputFormatter(originalScope, locals);
          } else {
            //it might happen that we don't have enough info to properly render input value
            //we need to check for this situation and simply return model value if we can't apply custom formatting
            locals[parserResult.itemName] = modelValue;
            candidateViewValue = parserResult.viewMapper(originalScope, locals);
            locals[parserResult.itemName] = undefined;
            emptyViewValue = parserResult.viewMapper(originalScope, locals);
            return candidateViewValue !== emptyViewValue ? candidateViewValue : modelValue;
          }
        });
        scope.select = function (activeIdx) {
          //called from within the $digest() cycle
          var locals = {};
          var model, item;
          selected = true;
          locals[parserResult.itemName] = item = scope.matches[activeIdx].model;
          model = parserResult.modelMapper(originalScope, locals);
          $setModelValue(originalScope, model);
          modelCtrl.$setValidity('editable', true);
          modelCtrl.$setValidity('parse', true);
          onSelectCallback(originalScope, {
            $item: item,
            $model: model,
            $label: parserResult.viewMapper(originalScope, locals)
          });
          resetMatches();
          //return focus to the input element if a match was selected via a mouse click event
          // use timeout to avoid $rootScope:inprog error
          if (scope.$eval(attrs.typeaheadFocusOnSelect) !== false) {
            $timeout(function () {
              element[0].focus();
            }, 0, false);
          }
        };
        //bind keyboard events: arrows up(38) / down(40), enter(13) and tab(9), esc(27)
        element.bind('keydown', function (evt) {
          //typeahead is open and an "interesting" key was pressed
          if (scope.matches.length === 0 || HOT_KEYS.indexOf(evt.which) === -1) {
            return;
          }
          // if there's nothing selected (i.e. focusFirst) and enter or tab is hit, clear the results
          if (scope.activeIdx === -1 && (evt.which === 9 || evt.which === 13)) {
            resetMatches();
            scope.$digest();
            return;
          }
          evt.preventDefault();
          if (evt.which === 40) {
            scope.activeIdx = (scope.activeIdx + 1) % scope.matches.length;
            scope.$digest();
          } else if (evt.which === 38) {
            scope.activeIdx = (scope.activeIdx > 0 ? scope.activeIdx : scope.matches.length) - 1;
            scope.$digest();
          } else if (evt.which === 13 || evt.which === 9) {
            scope.$apply(function () {
              scope.select(scope.activeIdx);
            });
          } else if (evt.which === 27) {
            evt.stopPropagation();
            resetMatches();
            scope.$digest();
          }
        });
        element.bind('blur', function () {
          if (isSelectOnBlur && scope.matches.length && scope.activeIdx !== -1 && !selected) {
            selected = true;
            scope.$apply(function () {
              scope.select(scope.activeIdx);
            });
          }
          hasFocus = false;
          selected = false;
        });
        // Keep reference to click handler to unbind it.
        var dismissClickHandler = function (evt) {
          // Issue #3973
          // Firefox treats right click as a click on document
          if (element[0] !== evt.target && evt.which !== 3 && scope.matches.length !== 0) {
            resetMatches();
            if (!$rootScope.$$phase) {
              scope.$digest();
            }
          }
        };
        $document.bind('click', dismissClickHandler);
        originalScope.$on('$destroy', function () {
          $document.unbind('click', dismissClickHandler);
          if (appendToBody || appendToElementId) {
            $popup.remove();
          }
          // Prevent jQuery cache memory leak
          popUpEl.remove();
        });
        var $popup = $compile(popUpEl)(scope);
        if (appendToBody) {
          $document.find('body').append($popup);
        } else if (appendToElementId !== false) {
          angular.element($document[0].getElementById(appendToElementId)).append($popup);
        } else {
          element.after($popup);
        }
      }
    };
  }
]).directive('uibTypeaheadPopup', function () {
  return {
    scope: {
      matches: '=',
      query: '=',
      active: '=',
      position: '&',
      moveInProgress: '=',
      select: '&'
    },
    replace: true,
    templateUrl: function (element, attrs) {
      return attrs.popupTemplateUrl || 'template/typeahead/typeahead-popup.html';
    },
    link: function (scope, element, attrs) {
      scope.templateUrl = attrs.templateUrl;
      scope.isOpen = function () {
        return scope.matches.length > 0;
      };
      scope.isActive = function (matchIdx) {
        return scope.active == matchIdx;
      };
      scope.selectActive = function (matchIdx) {
        scope.active = matchIdx;
      };
      scope.selectMatch = function (activeIdx) {
        scope.select({ activeIdx: activeIdx });
      };
    }
  };
}).directive('uibTypeaheadMatch', [
  '$templateRequest',
  '$compile',
  '$parse',
  function ($templateRequest, $compile, $parse) {
    return {
      scope: {
        index: '=',
        match: '=',
        query: '='
      },
      link: function (scope, element, attrs) {
        var tplUrl = $parse(attrs.templateUrl)(scope.$parent) || 'template/typeahead/typeahead-match.html';
        $templateRequest(tplUrl).then(function (tplContent) {
          $compile(tplContent.trim())(scope, function (clonedElement) {
            element.replaceWith(clonedElement);
          });
        });
      }
    };
  }
]).filter('uibTypeaheadHighlight', [
  '$sce',
  '$injector',
  '$log',
  function ($sce, $injector, $log) {
    var isSanitizePresent;
    isSanitizePresent = $injector.has('$sanitize');
    function escapeRegexp(queryToEscape) {
      // Regex: capture the whole query string and replace it with the string that will be used to match
      // the results, for example if the capture is "a" the result will be \a
      return queryToEscape.replace(/([.?*+^$[\]\\(){}|-])/g, '\\$1');
    }
    function containsHtml(matchItem) {
      return /<.*>/g.test(matchItem);
    }
    return function (matchItem, query) {
      if (!isSanitizePresent && containsHtml(matchItem)) {
        $log.warn('Unsafe use of typeahead please use ngSanitize');  // Warn the user about the danger
      }
      matchItem = query ? ('' + matchItem).replace(new RegExp(escapeRegexp(query), 'gi'), '<strong>$&</strong>') : matchItem;
      // Replaces the capture string with a the same string inside of a "strong" tag
      if (!isSanitizePresent) {
        matchItem = $sce.trustAsHtml(matchItem);  // If $sanitize is not present we pack the string in a $sce object for the ng-bind-html directive
      }
      return matchItem;
    };
  }
]);
/* Deprecated typeahead below */
angular.module('ui.bootstrap.typeahead').value('$typeaheadSuppressWarning', false).service('typeaheadParser', [
  '$parse',
  'uibTypeaheadParser',
  '$log',
  '$typeaheadSuppressWarning',
  function ($parse, uibTypeaheadParser, $log, $typeaheadSuppressWarning) {
    if (!$typeaheadSuppressWarning) {
      $log.warn('typeaheadParser is now deprecated. Use uibTypeaheadParser instead.');
    }
    return uibTypeaheadParser;
  }
]).directive('typeahead', [
  '$compile',
  '$parse',
  '$q',
  '$timeout',
  '$document',
  '$window',
  '$rootScope',
  '$uibPosition',
  'typeaheadParser',
  '$log',
  '$typeaheadSuppressWarning',
  function ($compile, $parse, $q, $timeout, $document, $window, $rootScope, $position, typeaheadParser, $log, $typeaheadSuppressWarning) {
    var HOT_KEYS = [
        9,
        13,
        27,
        38,
        40
      ];
    var eventDebounceTime = 200;
    return {
      require: [
        'ngModel',
        '^?ngModelOptions'
      ],
      link: function (originalScope, element, attrs, ctrls) {
        if (!$typeaheadSuppressWarning) {
          $log.warn('typeahead is now deprecated. Use uib-typeahead instead.');
        }
        var modelCtrl = ctrls[0];
        var ngModelOptions = ctrls[1];
        //SUPPORTED ATTRIBUTES (OPTIONS)
        //minimal no of characters that needs to be entered before typeahead kicks-in
        var minLength = originalScope.$eval(attrs.typeaheadMinLength);
        if (!minLength && minLength !== 0) {
          minLength = 1;
        }
        //minimal wait time after last character typed before typeahead kicks-in
        var waitTime = originalScope.$eval(attrs.typeaheadWaitMs) || 0;
        //should it restrict model values to the ones selected from the popup only?
        var isEditable = originalScope.$eval(attrs.typeaheadEditable) !== false;
        //binding to a variable that indicates if matches are being retrieved asynchronously
        var isLoadingSetter = $parse(attrs.typeaheadLoading).assign || angular.noop;
        //a callback executed when a match is selected
        var onSelectCallback = $parse(attrs.typeaheadOnSelect);
        //should it select highlighted popup value when losing focus?
        var isSelectOnBlur = angular.isDefined(attrs.typeaheadSelectOnBlur) ? originalScope.$eval(attrs.typeaheadSelectOnBlur) : false;
        //binding to a variable that indicates if there were no results after the query is completed
        var isNoResultsSetter = $parse(attrs.typeaheadNoResults).assign || angular.noop;
        var inputFormatter = attrs.typeaheadInputFormatter ? $parse(attrs.typeaheadInputFormatter) : undefined;
        var appendToBody = attrs.typeaheadAppendToBody ? originalScope.$eval(attrs.typeaheadAppendToBody) : false;
        var appendToElementId = attrs.typeaheadAppendToElementId || false;
        var focusFirst = originalScope.$eval(attrs.typeaheadFocusFirst) !== false;
        //If input matches an item of the list exactly, select it automatically
        var selectOnExact = attrs.typeaheadSelectOnExact ? originalScope.$eval(attrs.typeaheadSelectOnExact) : false;
        //INTERNAL VARIABLES
        //model setter executed upon match selection
        var parsedModel = $parse(attrs.ngModel);
        var invokeModelSetter = $parse(attrs.ngModel + '($$$p)');
        var $setModelValue = function (scope, newValue) {
          if (angular.isFunction(parsedModel(originalScope)) && ngModelOptions && ngModelOptions.$options && ngModelOptions.$options.getterSetter) {
            return invokeModelSetter(scope, { $$$p: newValue });
          } else {
            return parsedModel.assign(scope, newValue);
          }
        };
        //expressions used by typeahead
        var parserResult = typeaheadParser.parse(attrs.typeahead);
        var hasFocus;
        //Used to avoid bug in iOS webview where iOS keyboard does not fire
        //mousedown & mouseup events
        //Issue #3699
        var selected;
        //create a child scope for the typeahead directive so we are not polluting original scope
        //with typeahead-specific data (matches, query etc.)
        var scope = originalScope.$new();
        var offDestroy = originalScope.$on('$destroy', function () {
            scope.$destroy();
          });
        scope.$on('$destroy', offDestroy);
        // WAI-ARIA
        var popupId = 'typeahead-' + scope.$id + '-' + Math.floor(Math.random() * 10000);
        element.attr({
          'aria-autocomplete': 'list',
          'aria-expanded': false,
          'aria-owns': popupId
        });
        //pop-up element used to display matches
        var popUpEl = angular.element('<div typeahead-popup></div>');
        popUpEl.attr({
          id: popupId,
          matches: 'matches',
          active: 'activeIdx',
          select: 'select(activeIdx)',
          'move-in-progress': 'moveInProgress',
          query: 'query',
          position: 'position'
        });
        //custom item template
        if (angular.isDefined(attrs.typeaheadTemplateUrl)) {
          popUpEl.attr('template-url', attrs.typeaheadTemplateUrl);
        }
        if (angular.isDefined(attrs.typeaheadPopupTemplateUrl)) {
          popUpEl.attr('popup-template-url', attrs.typeaheadPopupTemplateUrl);
        }
        var resetMatches = function () {
          scope.matches = [];
          scope.activeIdx = -1;
          element.attr('aria-expanded', false);
        };
        var getMatchId = function (index) {
          return popupId + '-option-' + index;
        };
        // Indicate that the specified match is the active (pre-selected) item in the list owned by this typeahead.
        // This attribute is added or removed automatically when the `activeIdx` changes.
        scope.$watch('activeIdx', function (index) {
          if (index < 0) {
            element.removeAttr('aria-activedescendant');
          } else {
            element.attr('aria-activedescendant', getMatchId(index));
          }
        });
        var inputIsExactMatch = function (inputValue, index) {
          if (scope.matches.length > index && inputValue) {
            return inputValue.toUpperCase() === scope.matches[index].label.toUpperCase();
          }
          return false;
        };
        var getMatchesAsync = function (inputValue) {
          var locals = { $viewValue: inputValue };
          isLoadingSetter(originalScope, true);
          isNoResultsSetter(originalScope, false);
          $q.when(parserResult.source(originalScope, locals)).then(function (matches) {
            //it might happen that several async queries were in progress if a user were typing fast
            //but we are interested only in responses that correspond to the current view value
            var onCurrentRequest = inputValue === modelCtrl.$viewValue;
            if (onCurrentRequest && hasFocus) {
              if (matches && matches.length > 0) {
                scope.activeIdx = focusFirst ? 0 : -1;
                isNoResultsSetter(originalScope, false);
                scope.matches.length = 0;
                //transform labels
                for (var i = 0; i < matches.length; i++) {
                  locals[parserResult.itemName] = matches[i];
                  scope.matches.push({
                    id: getMatchId(i),
                    label: parserResult.viewMapper(scope, locals),
                    model: matches[i]
                  });
                }
                scope.query = inputValue;
                //position pop-up with matches - we need to re-calculate its position each time we are opening a window
                //with matches as a pop-up might be absolute-positioned and position of an input might have changed on a page
                //due to other elements being rendered
                recalculatePosition();
                element.attr('aria-expanded', true);
                //Select the single remaining option if user input matches
                if (selectOnExact && scope.matches.length === 1 && inputIsExactMatch(inputValue, 0)) {
                  scope.select(0);
                }
              } else {
                resetMatches();
                isNoResultsSetter(originalScope, true);
              }
            }
            if (onCurrentRequest) {
              isLoadingSetter(originalScope, false);
            }
          }, function () {
            resetMatches();
            isLoadingSetter(originalScope, false);
            isNoResultsSetter(originalScope, true);
          });
        };
        // bind events only if appendToBody params exist - performance feature
        if (appendToBody) {
          angular.element($window).bind('resize', fireRecalculating);
          $document.find('body').bind('scroll', fireRecalculating);
        }
        // Declare the timeout promise var outside the function scope so that stacked calls can be cancelled later
        var timeoutEventPromise;
        // Default progress type
        scope.moveInProgress = false;
        function fireRecalculating() {
          if (!scope.moveInProgress) {
            scope.moveInProgress = true;
            scope.$digest();
          }
          // Cancel previous timeout
          if (timeoutEventPromise) {
            $timeout.cancel(timeoutEventPromise);
          }
          // Debounced executing recalculate after events fired
          timeoutEventPromise = $timeout(function () {
            // if popup is visible
            if (scope.matches.length) {
              recalculatePosition();
            }
            scope.moveInProgress = false;
          }, eventDebounceTime);
        }
        // recalculate actual position and set new values to scope
        // after digest loop is popup in right position
        function recalculatePosition() {
          scope.position = appendToBody ? $position.offset(element) : $position.position(element);
          scope.position.top += element.prop('offsetHeight');
        }
        resetMatches();
        //we need to propagate user's query so we can higlight matches
        scope.query = undefined;
        //Declare the timeout promise var outside the function scope so that stacked calls can be cancelled later
        var timeoutPromise;
        var scheduleSearchWithTimeout = function (inputValue) {
          timeoutPromise = $timeout(function () {
            getMatchesAsync(inputValue);
          }, waitTime);
        };
        var cancelPreviousTimeout = function () {
          if (timeoutPromise) {
            $timeout.cancel(timeoutPromise);
          }
        };
        //plug into $parsers pipeline to open a typeahead on view changes initiated from DOM
        //$parsers kick-in on all the changes coming from the view as well as manually triggered by $setViewValue
        modelCtrl.$parsers.unshift(function (inputValue) {
          hasFocus = true;
          if (minLength === 0 || inputValue && inputValue.length >= minLength) {
            if (waitTime > 0) {
              cancelPreviousTimeout();
              scheduleSearchWithTimeout(inputValue);
            } else {
              getMatchesAsync(inputValue);
            }
          } else {
            isLoadingSetter(originalScope, false);
            cancelPreviousTimeout();
            resetMatches();
          }
          if (isEditable) {
            return inputValue;
          } else {
            if (!inputValue) {
              // Reset in case user had typed something previously.
              modelCtrl.$setValidity('editable', true);
              return null;
            } else {
              modelCtrl.$setValidity('editable', false);
              return undefined;
            }
          }
        });
        modelCtrl.$formatters.push(function (modelValue) {
          var candidateViewValue, emptyViewValue;
          var locals = {};
          // The validity may be set to false via $parsers (see above) if
          // the model is restricted to selected values. If the model
          // is set manually it is considered to be valid.
          if (!isEditable) {
            modelCtrl.$setValidity('editable', true);
          }
          if (inputFormatter) {
            locals.$model = modelValue;
            return inputFormatter(originalScope, locals);
          } else {
            //it might happen that we don't have enough info to properly render input value
            //we need to check for this situation and simply return model value if we can't apply custom formatting
            locals[parserResult.itemName] = modelValue;
            candidateViewValue = parserResult.viewMapper(originalScope, locals);
            locals[parserResult.itemName] = undefined;
            emptyViewValue = parserResult.viewMapper(originalScope, locals);
            return candidateViewValue !== emptyViewValue ? candidateViewValue : modelValue;
          }
        });
        scope.select = function (activeIdx) {
          //called from within the $digest() cycle
          var locals = {};
          var model, item;
          selected = true;
          locals[parserResult.itemName] = item = scope.matches[activeIdx].model;
          model = parserResult.modelMapper(originalScope, locals);
          $setModelValue(originalScope, model);
          modelCtrl.$setValidity('editable', true);
          modelCtrl.$setValidity('parse', true);
          onSelectCallback(originalScope, {
            $item: item,
            $model: model,
            $label: parserResult.viewMapper(originalScope, locals)
          });
          resetMatches();
          //return focus to the input element if a match was selected via a mouse click event
          // use timeout to avoid $rootScope:inprog error
          if (scope.$eval(attrs.typeaheadFocusOnSelect) !== false) {
            $timeout(function () {
              element[0].focus();
            }, 0, false);
          }
        };
        //bind keyboard events: arrows up(38) / down(40), enter(13) and tab(9), esc(27)
        element.bind('keydown', function (evt) {
          //typeahead is open and an "interesting" key was pressed
          if (scope.matches.length === 0 || HOT_KEYS.indexOf(evt.which) === -1) {
            return;
          }
          // if there's nothing selected (i.e. focusFirst) and enter or tab is hit, clear the results
          if (scope.activeIdx === -1 && (evt.which === 9 || evt.which === 13)) {
            resetMatches();
            scope.$digest();
            return;
          }
          evt.preventDefault();
          if (evt.which === 40) {
            scope.activeIdx = (scope.activeIdx + 1) % scope.matches.length;
            scope.$digest();
          } else if (evt.which === 38) {
            scope.activeIdx = (scope.activeIdx > 0 ? scope.activeIdx : scope.matches.length) - 1;
            scope.$digest();
          } else if (evt.which === 13 || evt.which === 9) {
            scope.$apply(function () {
              scope.select(scope.activeIdx);
            });
          } else if (evt.which === 27) {
            evt.stopPropagation();
            resetMatches();
            scope.$digest();
          }
        });
        element.bind('blur', function () {
          if (isSelectOnBlur && scope.matches.length && scope.activeIdx !== -1 && !selected) {
            selected = true;
            scope.$apply(function () {
              scope.select(scope.activeIdx);
            });
          }
          hasFocus = false;
          selected = false;
        });
        // Keep reference to click handler to unbind it.
        var dismissClickHandler = function (evt) {
          // Issue #3973
          // Firefox treats right click as a click on document
          if (element[0] !== evt.target && evt.which !== 3 && scope.matches.length !== 0) {
            resetMatches();
            if (!$rootScope.$$phase) {
              scope.$digest();
            }
          }
        };
        $document.bind('click', dismissClickHandler);
        originalScope.$on('$destroy', function () {
          $document.unbind('click', dismissClickHandler);
          if (appendToBody || appendToElementId) {
            $popup.remove();
          }
          // Prevent jQuery cache memory leak
          popUpEl.remove();
        });
        var $popup = $compile(popUpEl)(scope);
        if (appendToBody) {
          $document.find('body').append($popup);
        } else if (appendToElementId !== false) {
          angular.element($document[0].getElementById(appendToElementId)).append($popup);
        } else {
          element.after($popup);
        }
      }
    };
  }
]).directive('typeaheadPopup', [
  '$typeaheadSuppressWarning',
  '$log',
  function ($typeaheadSuppressWarning, $log) {
    return {
      scope: {
        matches: '=',
        query: '=',
        active: '=',
        position: '&',
        moveInProgress: '=',
        select: '&'
      },
      replace: true,
      templateUrl: function (element, attrs) {
        return attrs.popupTemplateUrl || 'template/typeahead/typeahead-popup.html';
      },
      link: function (scope, element, attrs) {
        if (!$typeaheadSuppressWarning) {
          $log.warn('typeahead-popup is now deprecated. Use uib-typeahead-popup instead.');
        }
        scope.templateUrl = attrs.templateUrl;
        scope.isOpen = function () {
          return scope.matches.length > 0;
        };
        scope.isActive = function (matchIdx) {
          return scope.active == matchIdx;
        };
        scope.selectActive = function (matchIdx) {
          scope.active = matchIdx;
        };
        scope.selectMatch = function (activeIdx) {
          scope.select({ activeIdx: activeIdx });
        };
      }
    };
  }
]).directive('typeaheadMatch', [
  '$templateRequest',
  '$compile',
  '$parse',
  '$typeaheadSuppressWarning',
  '$log',
  function ($templateRequest, $compile, $parse, $typeaheadSuppressWarning, $log) {
    return {
      restrict: 'EA',
      scope: {
        index: '=',
        match: '=',
        query: '='
      },
      link: function (scope, element, attrs) {
        if (!$typeaheadSuppressWarning) {
          $log.warn('typeahead-match is now deprecated. Use uib-typeahead-match instead.');
        }
        var tplUrl = $parse(attrs.templateUrl)(scope.$parent) || 'template/typeahead/typeahead-match.html';
        $templateRequest(tplUrl).then(function (tplContent) {
          $compile(tplContent.trim())(scope, function (clonedElement) {
            element.replaceWith(clonedElement);
          });
        });
      }
    };
  }
]).filter('typeaheadHighlight', [
  '$sce',
  '$injector',
  '$log',
  '$typeaheadSuppressWarning',
  function ($sce, $injector, $log, $typeaheadSuppressWarning) {
    var isSanitizePresent;
    isSanitizePresent = $injector.has('$sanitize');
    function escapeRegexp(queryToEscape) {
      // Regex: capture the whole query string and replace it with the string that will be used to match
      // the results, for example if the capture is "a" the result will be \a
      return queryToEscape.replace(/([.?*+^$[\]\\(){}|-])/g, '\\$1');
    }
    function containsHtml(matchItem) {
      return /<.*>/g.test(matchItem);
    }
    return function (matchItem, query) {
      if (!$typeaheadSuppressWarning) {
        $log.warn('typeaheadHighlight is now deprecated. Use uibTypeaheadHighlight instead.');
      }
      if (!isSanitizePresent && containsHtml(matchItem)) {
        $log.warn('Unsafe use of typeahead please use ngSanitize');  // Warn the user about the danger
      }
      matchItem = query ? ('' + matchItem).replace(new RegExp(escapeRegexp(query), 'gi'), '<strong>$&</strong>') : matchItem;
      // Replaces the capture string with a the same string inside of a "strong" tag
      if (!isSanitizePresent) {
        matchItem = $sce.trustAsHtml(matchItem);  // If $sanitize is not present we pack the string in a $sce object for the ng-bind-html directive
      }
      return matchItem;
    };
  }
]);
!angular.$$csp() && angular.element(document).find('head').prepend('<style type="text/css">.ng-animate.item:not(.left):not(.right){-webkit-transition:0s ease-in-out left;transition:0s ease-in-out left}</style>');
/*
 * angular-ui-bootstrap
 * http://angular-ui.github.io/bootstrap/

 * Version: 0.14.0 - 2015-10-08
 * License: MIT
 */
angular.module('ui.bootstrap', [
  'ui.bootstrap.tpls',
  'ui.bootstrap.collapse',
  'ui.bootstrap.accordion',
  'ui.bootstrap.alert',
  'ui.bootstrap.buttons',
  'ui.bootstrap.carousel',
  'ui.bootstrap.dateparser',
  'ui.bootstrap.position',
  'ui.bootstrap.datepicker',
  'ui.bootstrap.dropdown',
  'ui.bootstrap.stackedMap',
  'ui.bootstrap.modal',
  'ui.bootstrap.pagination',
  'ui.bootstrap.tooltip',
  'ui.bootstrap.popover',
  'ui.bootstrap.progressbar',
  'ui.bootstrap.rating',
  'ui.bootstrap.tabs',
  'ui.bootstrap.timepicker',
  'ui.bootstrap.typeahead'
]);
angular.module('ui.bootstrap.tpls', [
  'template/accordion/accordion-group.html',
  'template/accordion/accordion.html',
  'template/alert/alert.html',
  'template/carousel/carousel.html',
  'template/carousel/slide.html',
  'template/datepicker/datepicker.html',
  'template/datepicker/day.html',
  'template/datepicker/month.html',
  'template/datepicker/popup.html',
  'template/datepicker/year.html',
  'template/modal/backdrop.html',
  'template/modal/window.html',
  'template/pagination/pager.html',
  'template/pagination/pagination.html',
  'template/tooltip/tooltip-html-popup.html',
  'template/tooltip/tooltip-popup.html',
  'template/tooltip/tooltip-template-popup.html',
  'template/popover/popover-html.html',
  'template/popover/popover-template.html',
  'template/popover/popover.html',
  'template/progressbar/bar.html',
  'template/progressbar/progress.html',
  'template/progressbar/progressbar.html',
  'template/rating/rating.html',
  'template/tabs/tab.html',
  'template/tabs/tabset.html',
  'template/timepicker/timepicker.html',
  'template/typeahead/typeahead-match.html',
  'template/typeahead/typeahead-popup.html'
]);
angular.module('ui.bootstrap.collapse', []).directive('uibCollapse', [
  '$animate',
  '$injector',
  function ($animate, $injector) {
    var $animateCss = $injector.has('$animateCss') ? $injector.get('$animateCss') : null;
    return {
      link: function (scope, element, attrs) {
        function expand() {
          element.removeClass('collapse').addClass('collapsing').attr('aria-expanded', true).attr('aria-hidden', false);
          if ($animateCss) {
            $animateCss(element, {
              addClass: 'in',
              easing: 'ease',
              to: { height: element[0].scrollHeight + 'px' }
            }).start().done(expandDone);
          } else {
            $animate.addClass(element, 'in', { to: { height: element[0].scrollHeight + 'px' } }).then(expandDone);
          }
        }
        function expandDone() {
          element.removeClass('collapsing').addClass('collapse').css({ height: 'auto' });
        }
        function collapse() {
          if (!element.hasClass('collapse') && !element.hasClass('in')) {
            return collapseDone();
          }
          element.css({ height: element[0].scrollHeight + 'px' }).removeClass('collapse').addClass('collapsing').attr('aria-expanded', false).attr('aria-hidden', true);
          if ($animateCss) {
            $animateCss(element, {
              removeClass: 'in',
              to: { height: '0' }
            }).start().done(collapseDone);
          } else {
            $animate.removeClass(element, 'in', { to: { height: '0' } }).then(collapseDone);
          }
        }
        function collapseDone() {
          element.css({ height: '0' });
          // Required so that collapse works when animation is disabled
          element.removeClass('collapsing').addClass('collapse');
        }
        scope.$watch(attrs.uibCollapse, function (shouldCollapse) {
          if (shouldCollapse) {
            collapse();
          } else {
            expand();
          }
        });
      }
    };
  }
]);
/* Deprecated collapse below */
angular.module('ui.bootstrap.collapse').value('$collapseSuppressWarning', false).directive('collapse', [
  '$animate',
  '$injector',
  '$log',
  '$collapseSuppressWarning',
  function ($animate, $injector, $log, $collapseSuppressWarning) {
    var $animateCss = $injector.has('$animateCss') ? $injector.get('$animateCss') : null;
    return {
      link: function (scope, element, attrs) {
        if (!$collapseSuppressWarning) {
          $log.warn('collapse is now deprecated. Use uib-collapse instead.');
        }
        function expand() {
          element.removeClass('collapse').addClass('collapsing').attr('aria-expanded', true).attr('aria-hidden', false);
          if ($animateCss) {
            $animateCss(element, {
              addClass: 'in',
              easing: 'ease',
              to: { height: element[0].scrollHeight + 'px' }
            }).start().done(expandDone);
          } else {
            $animate.addClass(element, 'in', { to: { height: element[0].scrollHeight + 'px' } }).then(expandDone);
          }
        }
        function expandDone() {
          element.removeClass('collapsing').addClass('collapse').css({ height: 'auto' });
        }
        function collapse() {
          if (!element.hasClass('collapse') && !element.hasClass('in')) {
            return collapseDone();
          }
          element.css({ height: element[0].scrollHeight + 'px' }).removeClass('collapse').addClass('collapsing').attr('aria-expanded', false).attr('aria-hidden', true);
          if ($animateCss) {
            $animateCss(element, {
              removeClass: 'in',
              to: { height: '0' }
            }).start().done(collapseDone);
          } else {
            $animate.removeClass(element, 'in', { to: { height: '0' } }).then(collapseDone);
          }
        }
        function collapseDone() {
          element.css({ height: '0' });
          // Required so that collapse works when animation is disabled
          element.removeClass('collapsing').addClass('collapse');
        }
        scope.$watch(attrs.collapse, function (shouldCollapse) {
          if (shouldCollapse) {
            collapse();
          } else {
            expand();
          }
        });
      }
    };
  }
]);
angular.module('ui.bootstrap.accordion', ['ui.bootstrap.collapse']).constant('uibAccordionConfig', { closeOthers: true }).controller('UibAccordionController', [
  '$scope',
  '$attrs',
  'uibAccordionConfig',
  function ($scope, $attrs, accordionConfig) {
    // This array keeps track of the accordion groups
    this.groups = [];
    // Ensure that all the groups in this accordion are closed, unless close-others explicitly says not to
    this.closeOthers = function (openGroup) {
      var closeOthers = angular.isDefined($attrs.closeOthers) ? $scope.$eval($attrs.closeOthers) : accordionConfig.closeOthers;
      if (closeOthers) {
        angular.forEach(this.groups, function (group) {
          if (group !== openGroup) {
            group.isOpen = false;
          }
        });
      }
    };
    // This is called from the accordion-group directive to add itself to the accordion
    this.addGroup = function (groupScope) {
      var that = this;
      this.groups.push(groupScope);
      groupScope.$on('$destroy', function (event) {
        that.removeGroup(groupScope);
      });
    };
    // This is called from the accordion-group directive when to remove itself
    this.removeGroup = function (group) {
      var index = this.groups.indexOf(group);
      if (index !== -1) {
        this.groups.splice(index, 1);
      }
    };
  }
]).directive('uibAccordion', function () {
  return {
    controller: 'UibAccordionController',
    controllerAs: 'accordion',
    transclude: true,
    templateUrl: function (element, attrs) {
      return attrs.templateUrl || 'template/accordion/accordion.html';
    }
  };
}).directive('uibAccordionGroup', function () {
  return {
    require: '^uibAccordion',
    transclude: true,
    replace: true,
    templateUrl: function (element, attrs) {
      return attrs.templateUrl || 'template/accordion/accordion-group.html';
    },
    scope: {
      heading: '@',
      isOpen: '=?',
      isDisabled: '=?'
    },
    controller: function () {
      this.setHeading = function (element) {
        this.heading = element;
      };
    },
    link: function (scope, element, attrs, accordionCtrl) {
      accordionCtrl.addGroup(scope);
      scope.openClass = attrs.openClass || 'panel-open';
      scope.panelClass = attrs.panelClass;
      scope.$watch('isOpen', function (value) {
        element.toggleClass(scope.openClass, !!value);
        if (value) {
          accordionCtrl.closeOthers(scope);
        }
      });
      scope.toggleOpen = function ($event) {
        if (!scope.isDisabled) {
          if (!$event || $event.which === 32) {
            scope.isOpen = !scope.isOpen;
          }
        }
      };
    }
  };
}).directive('uibAccordionHeading', function () {
  return {
    transclude: true,
    template: '',
    replace: true,
    require: '^uibAccordionGroup',
    link: function (scope, element, attrs, accordionGroupCtrl, transclude) {
      // Pass the heading to the accordion-group controller
      // so that it can be transcluded into the right place in the template
      // [The second parameter to transclude causes the elements to be cloned so that they work in ng-repeat]
      accordionGroupCtrl.setHeading(transclude(scope, angular.noop));
    }
  };
}).directive('uibAccordionTransclude', function () {
  return {
    require: [
      '?^uibAccordionGroup',
      '?^accordionGroup'
    ],
    link: function (scope, element, attrs, controller) {
      controller = controller[0] ? controller[0] : controller[1];
      // Delete after we remove deprecation
      scope.$watch(function () {
        return controller[attrs.uibAccordionTransclude];
      }, function (heading) {
        if (heading) {
          element.find('span').html('');
          element.find('span').append(heading);
        }
      });
    }
  };
});
/* Deprecated accordion below */
angular.module('ui.bootstrap.accordion').value('$accordionSuppressWarning', false).controller('AccordionController', [
  '$scope',
  '$attrs',
  '$controller',
  '$log',
  '$accordionSuppressWarning',
  function ($scope, $attrs, $controller, $log, $accordionSuppressWarning) {
    if (!$accordionSuppressWarning) {
      $log.warn('AccordionController is now deprecated. Use UibAccordionController instead.');
    }
    return $controller('UibAccordionController', {
      $scope: $scope,
      $attrs: $attrs
    });
  }
]).directive('accordion', [
  '$log',
  '$accordionSuppressWarning',
  function ($log, $accordionSuppressWarning) {
    return {
      restrict: 'EA',
      controller: 'AccordionController',
      controllerAs: 'accordion',
      transclude: true,
      replace: false,
      templateUrl: function (element, attrs) {
        return attrs.templateUrl || 'template/accordion/accordion.html';
      },
      link: function () {
        if (!$accordionSuppressWarning) {
          $log.warn('accordion is now deprecated. Use uib-accordion instead.');
        }
      }
    };
  }
]).directive('accordionGroup', [
  '$log',
  '$accordionSuppressWarning',
  function ($log, $accordionSuppressWarning) {
    return {
      require: '^accordion',
      restrict: 'EA',
      transclude: true,
      replace: true,
      templateUrl: function (element, attrs) {
        return attrs.templateUrl || 'template/accordion/accordion-group.html';
      },
      scope: {
        heading: '@',
        isOpen: '=?',
        isDisabled: '=?'
      },
      controller: function () {
        this.setHeading = function (element) {
          this.heading = element;
        };
      },
      link: function (scope, element, attrs, accordionCtrl) {
        if (!$accordionSuppressWarning) {
          $log.warn('accordion-group is now deprecated. Use uib-accordion-group instead.');
        }
        accordionCtrl.addGroup(scope);
        scope.openClass = attrs.openClass || 'panel-open';
        scope.panelClass = attrs.panelClass;
        scope.$watch('isOpen', function (value) {
          element.toggleClass(scope.openClass, !!value);
          if (value) {
            accordionCtrl.closeOthers(scope);
          }
        });
        scope.toggleOpen = function ($event) {
          if (!scope.isDisabled) {
            if (!$event || $event.which === 32) {
              scope.isOpen = !scope.isOpen;
            }
          }
        };
      }
    };
  }
]).directive('accordionHeading', [
  '$log',
  '$accordionSuppressWarning',
  function ($log, $accordionSuppressWarning) {
    return {
      restrict: 'EA',
      transclude: true,
      template: '',
      replace: true,
      require: '^accordionGroup',
      link: function (scope, element, attr, accordionGroupCtrl, transclude) {
        if (!$accordionSuppressWarning) {
          $log.warn('accordion-heading is now deprecated. Use uib-accordion-heading instead.');
        }
        // Pass the heading to the accordion-group controller
        // so that it can be transcluded into the right place in the template
        // [The second parameter to transclude causes the elements to be cloned so that they work in ng-repeat]
        accordionGroupCtrl.setHeading(transclude(scope, angular.noop));
      }
    };
  }
]).directive('accordionTransclude', [
  '$log',
  '$accordionSuppressWarning',
  function ($log, $accordionSuppressWarning) {
    return {
      require: '^accordionGroup',
      link: function (scope, element, attr, controller) {
        if (!$accordionSuppressWarning) {
          $log.warn('accordion-transclude is now deprecated. Use uib-accordion-transclude instead.');
        }
        scope.$watch(function () {
          return controller[attr.accordionTransclude];
        }, function (heading) {
          if (heading) {
            element.find('span').html('');
            element.find('span').append(heading);
          }
        });
      }
    };
  }
]);
angular.module('ui.bootstrap.alert', []).controller('UibAlertController', [
  '$scope',
  '$attrs',
  '$timeout',
  function ($scope, $attrs, $timeout) {
    $scope.closeable = !!$attrs.close;
    if (angular.isDefined($attrs.dismissOnTimeout)) {
      $timeout(function () {
        $scope.close();
      }, parseInt($attrs.dismissOnTimeout, 10));
    }
  }
]).directive('uibAlert', function () {
  return {
    controller: 'UibAlertController',
    controllerAs: 'alert',
    templateUrl: function (element, attrs) {
      return attrs.templateUrl || 'template/alert/alert.html';
    },
    transclude: true,
    replace: true,
    scope: {
      type: '@',
      close: '&'
    }
  };
});
/* Deprecated alert below */
angular.module('ui.bootstrap.alert').value('$alertSuppressWarning', false).controller('AlertController', [
  '$scope',
  '$attrs',
  '$controller',
  '$log',
  '$alertSuppressWarning',
  function ($scope, $attrs, $controller, $log, $alertSuppressWarning) {
    if (!$alertSuppressWarning) {
      $log.warn('AlertController is now deprecated. Use UibAlertController instead.');
    }
    return $controller('UibAlertController', {
      $scope: $scope,
      $attrs: $attrs
    });
  }
]).directive('alert', [
  '$log',
  '$alertSuppressWarning',
  function ($log, $alertSuppressWarning) {
    return {
      controller: 'AlertController',
      controllerAs: 'alert',
      templateUrl: function (element, attrs) {
        return attrs.templateUrl || 'template/alert/alert.html';
      },
      transclude: true,
      replace: true,
      scope: {
        type: '@',
        close: '&'
      },
      link: function () {
        if (!$alertSuppressWarning) {
          $log.warn('alert is now deprecated. Use uib-alert instead.');
        }
      }
    };
  }
]);
angular.module('ui.bootstrap.buttons', []).constant('uibButtonConfig', {
  activeClass: 'active',
  toggleEvent: 'click'
}).controller('UibButtonsController', [
  'uibButtonConfig',
  function (buttonConfig) {
    this.activeClass = buttonConfig.activeClass || 'active';
    this.toggleEvent = buttonConfig.toggleEvent || 'click';
  }
]).directive('uibBtnRadio', function () {
  return {
    require: [
      'uibBtnRadio',
      'ngModel'
    ],
    controller: 'UibButtonsController',
    controllerAs: 'buttons',
    link: function (scope, element, attrs, ctrls) {
      var buttonsCtrl = ctrls[0], ngModelCtrl = ctrls[1];
      element.find('input').css({ display: 'none' });
      //model -> UI
      ngModelCtrl.$render = function () {
        element.toggleClass(buttonsCtrl.activeClass, angular.equals(ngModelCtrl.$modelValue, scope.$eval(attrs.uibBtnRadio)));
      };
      //ui->model
      element.on(buttonsCtrl.toggleEvent, function () {
        if (attrs.disabled) {
          return;
        }
        var isActive = element.hasClass(buttonsCtrl.activeClass);
        if (!isActive || angular.isDefined(attrs.uncheckable)) {
          scope.$apply(function () {
            ngModelCtrl.$setViewValue(isActive ? null : scope.$eval(attrs.uibBtnRadio));
            ngModelCtrl.$render();
          });
        }
      });
    }
  };
}).directive('uibBtnCheckbox', [
  '$document',
  function ($document) {
    return {
      require: [
        'uibBtnCheckbox',
        'ngModel'
      ],
      controller: 'UibButtonsController',
      controllerAs: 'button',
      link: function (scope, element, attrs, ctrls) {
        var buttonsCtrl = ctrls[0], ngModelCtrl = ctrls[1];
        element.find('input').css({ display: 'none' });
        function getTrueValue() {
          return getCheckboxValue(attrs.btnCheckboxTrue, true);
        }
        function getFalseValue() {
          return getCheckboxValue(attrs.btnCheckboxFalse, false);
        }
        function getCheckboxValue(attribute, defaultValue) {
          return angular.isDefined(attribute) ? scope.$eval(attribute) : defaultValue;
        }
        //model -> UI
        ngModelCtrl.$render = function () {
          element.toggleClass(buttonsCtrl.activeClass, angular.equals(ngModelCtrl.$modelValue, getTrueValue()));
        };
        //ui->model
        element.on(buttonsCtrl.toggleEvent, function () {
          if (attrs.disabled) {
            return;
          }
          scope.$apply(function () {
            ngModelCtrl.$setViewValue(element.hasClass(buttonsCtrl.activeClass) ? getFalseValue() : getTrueValue());
            ngModelCtrl.$render();
          });
        });
        //accessibility
        element.on('keypress', function (e) {
          if (attrs.disabled || e.which !== 32 || $document[0].activeElement !== element[0]) {
            return;
          }
          scope.$apply(function () {
            ngModelCtrl.$setViewValue(element.hasClass(buttonsCtrl.activeClass) ? getFalseValue() : getTrueValue());
            ngModelCtrl.$render();
          });
        });
      }
    };
  }
]);
/* Deprecated buttons below */
angular.module('ui.bootstrap.buttons').value('$buttonsSuppressWarning', false).controller('ButtonsController', [
  '$controller',
  '$log',
  '$buttonsSuppressWarning',
  function ($controller, $log, $buttonsSuppressWarning) {
    if (!$buttonsSuppressWarning) {
      $log.warn('ButtonsController is now deprecated. Use UibButtonsController instead.');
    }
    return $controller('UibButtonsController');
  }
]).directive('btnRadio', [
  '$log',
  '$buttonsSuppressWarning',
  function ($log, $buttonsSuppressWarning) {
    return {
      require: [
        'btnRadio',
        'ngModel'
      ],
      controller: 'ButtonsController',
      controllerAs: 'buttons',
      link: function (scope, element, attrs, ctrls) {
        if (!$buttonsSuppressWarning) {
          $log.warn('btn-radio is now deprecated. Use uib-btn-radio instead.');
        }
        var buttonsCtrl = ctrls[0], ngModelCtrl = ctrls[1];
        element.find('input').css({ display: 'none' });
        //model -> UI
        ngModelCtrl.$render = function () {
          element.toggleClass(buttonsCtrl.activeClass, angular.equals(ngModelCtrl.$modelValue, scope.$eval(attrs.btnRadio)));
        };
        //ui->model
        element.bind(buttonsCtrl.toggleEvent, function () {
          if (attrs.disabled) {
            return;
          }
          var isActive = element.hasClass(buttonsCtrl.activeClass);
          if (!isActive || angular.isDefined(attrs.uncheckable)) {
            scope.$apply(function () {
              ngModelCtrl.$setViewValue(isActive ? null : scope.$eval(attrs.btnRadio));
              ngModelCtrl.$render();
            });
          }
        });
      }
    };
  }
]).directive('btnCheckbox', [
  '$document',
  '$log',
  '$buttonsSuppressWarning',
  function ($document, $log, $buttonsSuppressWarning) {
    return {
      require: [
        'btnCheckbox',
        'ngModel'
      ],
      controller: 'ButtonsController',
      controllerAs: 'button',
      link: function (scope, element, attrs, ctrls) {
        if (!$buttonsSuppressWarning) {
          $log.warn('btn-checkbox is now deprecated. Use uib-btn-checkbox instead.');
        }
        var buttonsCtrl = ctrls[0], ngModelCtrl = ctrls[1];
        element.find('input').css({ display: 'none' });
        function getTrueValue() {
          return getCheckboxValue(attrs.btnCheckboxTrue, true);
        }
        function getFalseValue() {
          return getCheckboxValue(attrs.btnCheckboxFalse, false);
        }
        function getCheckboxValue(attributeValue, defaultValue) {
          var val = scope.$eval(attributeValue);
          return angular.isDefined(val) ? val : defaultValue;
        }
        //model -> UI
        ngModelCtrl.$render = function () {
          element.toggleClass(buttonsCtrl.activeClass, angular.equals(ngModelCtrl.$modelValue, getTrueValue()));
        };
        //ui->model
        element.bind(buttonsCtrl.toggleEvent, function () {
          if (attrs.disabled) {
            return;
          }
          scope.$apply(function () {
            ngModelCtrl.$setViewValue(element.hasClass(buttonsCtrl.activeClass) ? getFalseValue() : getTrueValue());
            ngModelCtrl.$render();
          });
        });
        //accessibility
        element.on('keypress', function (e) {
          if (attrs.disabled || e.which !== 32 || $document[0].activeElement !== element[0]) {
            return;
          }
          scope.$apply(function () {
            ngModelCtrl.$setViewValue(element.hasClass(buttonsCtrl.activeClass) ? getFalseValue() : getTrueValue());
            ngModelCtrl.$render();
          });
        });
      }
    };
  }
]);
/**    
 * @ngdoc overview    
 * @name ui.bootstrap.carousel    
 *    
 * @description   
 * AngularJS version of an image carousel.    
 *    
 */
angular.module('ui.bootstrap.carousel', []).controller('UibCarouselController', [
  '$scope',
  '$element',
  '$interval',
  '$animate',
  function ($scope, $element, $interval, $animate) {
    var self = this, slides = self.slides = $scope.slides = [], NEW_ANIMATE = angular.version.minor >= 4, NO_TRANSITION = 'uib-noTransition', SLIDE_DIRECTION = 'uib-slideDirection', currentIndex = -1, currentInterval, isPlaying;
    self.currentSlide = null;
    var destroyed = false;
    /* direction: "prev" or "next" */
    self.select = $scope.select = function (nextSlide, direction) {
      var nextIndex = $scope.indexOfSlide(nextSlide);
      //Decide direction if it's not given
      if (direction === undefined) {
        direction = nextIndex > self.getCurrentIndex() ? 'next' : 'prev';
      }
      //Prevent this user-triggered transition from occurring if there is already one in progress
      if (nextSlide && nextSlide !== self.currentSlide && !$scope.$currentTransition) {
        goNext(nextSlide, nextIndex, direction);
      }
    };
    function goNext(slide, index, direction) {
      // Scope has been destroyed, stop here.
      if (destroyed) {
        return;
      }
      angular.extend(slide, {
        direction: direction,
        active: true
      });
      angular.extend(self.currentSlide || {}, {
        direction: direction,
        active: false
      });
      if ($animate.enabled() && !$scope.noTransition && !$scope.$currentTransition && slide.$element && self.slides.length > 1) {
        slide.$element.data(SLIDE_DIRECTION, slide.direction);
        if (self.currentSlide && self.currentSlide.$element) {
          self.currentSlide.$element.data(SLIDE_DIRECTION, slide.direction);
        }
        $scope.$currentTransition = true;
        if (NEW_ANIMATE) {
          $animate.on('addClass', slide.$element, function (element, phase) {
            if (phase === 'close') {
              $scope.$currentTransition = null;
              $animate.off('addClass', element);
            }
          });
        } else {
          slide.$element.one('$animate:close', function closeFn() {
            $scope.$currentTransition = null;
          });
        }
      }
      self.currentSlide = slide;
      currentIndex = index;
      //every time you change slides, reset the timer
      restartTimer();
    }
    $scope.$on('$destroy', function () {
      destroyed = true;
    });
    function getSlideByIndex(index) {
      if (angular.isUndefined(slides[index].index)) {
        return slides[index];
      }
      var i, len = slides.length;
      for (i = 0; i < slides.length; ++i) {
        if (slides[i].index == index) {
          return slides[i];
        }
      }
    }
    self.getCurrentIndex = function () {
      if (self.currentSlide && angular.isDefined(self.currentSlide.index)) {
        return +self.currentSlide.index;
      }
      return currentIndex;
    };
    /* Allow outside people to call indexOf on slides array */
    $scope.indexOfSlide = function (slide) {
      return angular.isDefined(slide.index) ? +slide.index : slides.indexOf(slide);
    };
    $scope.next = function () {
      var newIndex = (self.getCurrentIndex() + 1) % slides.length;
      if (newIndex === 0 && $scope.noWrap()) {
        $scope.pause();
        return;
      }
      return self.select(getSlideByIndex(newIndex), 'next');
    };
    $scope.prev = function () {
      var newIndex = self.getCurrentIndex() - 1 < 0 ? slides.length - 1 : self.getCurrentIndex() - 1;
      if ($scope.noWrap() && newIndex === slides.length - 1) {
        $scope.pause();
        return;
      }
      return self.select(getSlideByIndex(newIndex), 'prev');
    };
    $scope.isActive = function (slide) {
      return self.currentSlide === slide;
    };
    $scope.$watch('interval', restartTimer);
    $scope.$watchCollection('slides', resetTransition);
    $scope.$on('$destroy', resetTimer);
    function restartTimer() {
      resetTimer();
      var interval = +$scope.interval;
      if (!isNaN(interval) && interval > 0) {
        currentInterval = $interval(timerFn, interval);
      }
    }
    function resetTimer() {
      if (currentInterval) {
        $interval.cancel(currentInterval);
        currentInterval = null;
      }
    }
    function timerFn() {
      var interval = +$scope.interval;
      if (isPlaying && !isNaN(interval) && interval > 0 && slides.length) {
        $scope.next();
      } else {
        $scope.pause();
      }
    }
    function resetTransition(slides) {
      if (!slides.length) {
        $scope.$currentTransition = null;
      }
    }
    $scope.play = function () {
      if (!isPlaying) {
        isPlaying = true;
        restartTimer();
      }
    };
    $scope.pause = function () {
      if (!$scope.noPause) {
        isPlaying = false;
        resetTimer();
      }
    };
    self.addSlide = function (slide, element) {
      slide.$element = element;
      slides.push(slide);
      //if this is the first slide or the slide is set to active, select it
      if (slides.length === 1 || slide.active) {
        self.select(slides[slides.length - 1]);
        if (slides.length === 1) {
          $scope.play();
        }
      } else {
        slide.active = false;
      }
    };
    self.removeSlide = function (slide) {
      if (angular.isDefined(slide.index)) {
        slides.sort(function (a, b) {
          return +a.index > +b.index;
        });
      }
      //get the index of the slide inside the carousel
      var index = slides.indexOf(slide);
      slides.splice(index, 1);
      if (slides.length > 0 && slide.active) {
        if (index >= slides.length) {
          self.select(slides[index - 1]);
        } else {
          self.select(slides[index]);
        }
      } else if (currentIndex > index) {
        currentIndex--;
      }
      //clean the currentSlide when no more slide
      if (slides.length === 0) {
        self.currentSlide = null;
      }
    };
    $scope.$watch('noTransition', function (noTransition) {
      $element.data(NO_TRANSITION, noTransition);
    });
  }
]).directive('uibCarousel', [function () {
    return {
      transclude: true,
      replace: true,
      controller: 'UibCarouselController',
      controllerAs: 'carousel',
      require: 'carousel',
      templateUrl: function (element, attrs) {
        return attrs.templateUrl || 'template/carousel/carousel.html';
      },
      scope: {
        interval: '=',
        noTransition: '=',
        noPause: '=',
        noWrap: '&'
      }
    };
  }]).directive('uibSlide', function () {
  return {
    require: '^uibCarousel',
    restrict: 'EA',
    transclude: true,
    replace: true,
    templateUrl: function (element, attrs) {
      return attrs.templateUrl || 'template/carousel/slide.html';
    },
    scope: {
      active: '=?',
      actual: '=?',
      index: '=?'
    },
    link: function (scope, element, attrs, carouselCtrl) {
      carouselCtrl.addSlide(scope, element);
      //when the scope is destroyed then remove the slide from the current slides array
      scope.$on('$destroy', function () {
        carouselCtrl.removeSlide(scope);
      });
      scope.$watch('active', function (active) {
        if (active) {
          carouselCtrl.select(scope);
        }
      });
    }
  };
}).animation('.item', [
  '$injector',
  '$animate',
  function ($injector, $animate) {
    var NO_TRANSITION = 'uib-noTransition', SLIDE_DIRECTION = 'uib-slideDirection', $animateCss = null;
    if ($injector.has('$animateCss')) {
      $animateCss = $injector.get('$animateCss');
    }
    function removeClass(element, className, callback) {
      element.removeClass(className);
      if (callback) {
        callback();
      }
    }
    return {
      beforeAddClass: function (element, className, done) {
        // Due to transclusion, noTransition property is on parent's scope
        if (className == 'active' && element.parent() && element.parent().parent() && !element.parent().parent().data(NO_TRANSITION)) {
          var stopped = false;
          var direction = element.data(SLIDE_DIRECTION);
          var directionClass = direction == 'next' ? 'left' : 'right';
          var removeClassFn = removeClass.bind(this, element, directionClass + ' ' + direction, done);
          element.addClass(direction);
          if ($animateCss) {
            $animateCss(element, { addClass: directionClass }).start().done(removeClassFn);
          } else {
            $animate.addClass(element, directionClass).then(function () {
              if (!stopped) {
                removeClassFn();
              }
              done();
            });
          }
          return function () {
            stopped = true;
          };
        }
        done();
      },
      beforeRemoveClass: function (element, className, done) {
        // Due to transclusion, noTransition property is on parent's scope
        if (className === 'active' && element.parent() && element.parent().parent() && !element.parent().parent().data(NO_TRANSITION)) {
          var stopped = false;
          var direction = element.data(SLIDE_DIRECTION);
          var directionClass = direction == 'next' ? 'left' : 'right';
          var removeClassFn = removeClass.bind(this, element, directionClass, done);
          if ($animateCss) {
            $animateCss(element, { addClass: directionClass }).start().done(removeClassFn);
          } else {
            $animate.addClass(element, directionClass).then(function () {
              if (!stopped) {
                removeClassFn();
              }
              done();
            });
          }
          return function () {
            stopped = true;
          };
        }
        done();
      }
    };
  }
]);
/* deprecated carousel below */
angular.module('ui.bootstrap.carousel').value('$carouselSuppressWarning', false).controller('CarouselController', [
  '$scope',
  '$element',
  '$controller',
  '$log',
  '$carouselSuppressWarning',
  function ($scope, $element, $controller, $log, $carouselSuppressWarning) {
    if (!$carouselSuppressWarning) {
      $log.warn('CarouselController is now deprecated. Use UibCarouselController instead.');
    }
    return $controller('UibCarouselController', {
      $scope: $scope,
      $element: $element
    });
  }
]).directive('carousel', [
  '$log',
  '$carouselSuppressWarning',
  function ($log, $carouselSuppressWarning) {
    return {
      transclude: true,
      replace: true,
      controller: 'CarouselController',
      controllerAs: 'carousel',
      require: 'carousel',
      templateUrl: function (element, attrs) {
        return attrs.templateUrl || 'template/carousel/carousel.html';
      },
      scope: {
        interval: '=',
        noTransition: '=',
        noPause: '=',
        noWrap: '&'
      },
      link: function () {
        if (!$carouselSuppressWarning) {
          $log.warn('carousel is now deprecated. Use uib-carousel instead.');
        }
      }
    };
  }
]).directive('slide', [
  '$log',
  '$carouselSuppressWarning',
  function ($log, $carouselSuppressWarning) {
    return {
      require: '^carousel',
      transclude: true,
      replace: true,
      templateUrl: function (element, attrs) {
        return attrs.templateUrl || 'template/carousel/slide.html';
      },
      scope: {
        active: '=?',
        actual: '=?',
        index: '=?'
      },
      link: function (scope, element, attrs, carouselCtrl) {
        if (!$carouselSuppressWarning) {
          $log.warn('slide is now deprecated. Use uib-slide instead.');
        }
        carouselCtrl.addSlide(scope, element);
        //when the scope is destroyed then remove the slide from the current slides array
        scope.$on('$destroy', function () {
          carouselCtrl.removeSlide(scope);
        });
        scope.$watch('active', function (active) {
          if (active) {
            carouselCtrl.select(scope);
          }
        });
      }
    };
  }
]);
angular.module('ui.bootstrap.dateparser', []).service('uibDateParser', [
  '$log',
  '$locale',
  'orderByFilter',
  function ($log, $locale, orderByFilter) {
    // Pulled from https://github.com/mbostock/d3/blob/master/src/format/requote.js
    var SPECIAL_CHARACTERS_REGEXP = /[\\\^\$\*\+\?\|\[\]\(\)\.\{\}]/g;
    var localeId;
    var formatCodeToRegex;
    this.init = function () {
      localeId = $locale.id;
      this.parsers = {};
      formatCodeToRegex = {
        'yyyy': {
          regex: '\\d{4}',
          apply: function (value) {
            this.year = +value;
          }
        },
        'yy': {
          regex: '\\d{2}',
          apply: function (value) {
            this.year = +value + 2000;
          }
        },
        'y': {
          regex: '\\d{1,4}',
          apply: function (value) {
            this.year = +value;
          }
        },
        'MMMM': {
          regex: $locale.DATETIME_FORMATS.MONTH.join('|'),
          apply: function (value) {
            this.month = $locale.DATETIME_FORMATS.MONTH.indexOf(value);
          }
        },
        'MMM': {
          regex: $locale.DATETIME_FORMATS.SHORTMONTH.join('|'),
          apply: function (value) {
            this.month = $locale.DATETIME_FORMATS.SHORTMONTH.indexOf(value);
          }
        },
        'MM': {
          regex: '0[1-9]|1[0-2]',
          apply: function (value) {
            this.month = value - 1;
          }
        },
        'M': {
          regex: '[1-9]|1[0-2]',
          apply: function (value) {
            this.month = value - 1;
          }
        },
        'dd': {
          regex: '[0-2][0-9]{1}|3[0-1]{1}',
          apply: function (value) {
            this.date = +value;
          }
        },
        'd': {
          regex: '[1-2]?[0-9]{1}|3[0-1]{1}',
          apply: function (value) {
            this.date = +value;
          }
        },
        'EEEE': { regex: $locale.DATETIME_FORMATS.DAY.join('|') },
        'EEE': { regex: $locale.DATETIME_FORMATS.SHORTDAY.join('|') },
        'HH': {
          regex: '(?:0|1)[0-9]|2[0-3]',
          apply: function (value) {
            this.hours = +value;
          }
        },
        'hh': {
          regex: '0[0-9]|1[0-2]',
          apply: function (value) {
            this.hours = +value;
          }
        },
        'H': {
          regex: '1?[0-9]|2[0-3]',
          apply: function (value) {
            this.hours = +value;
          }
        },
        'h': {
          regex: '[0-9]|1[0-2]',
          apply: function (value) {
            this.hours = +value;
          }
        },
        'mm': {
          regex: '[0-5][0-9]',
          apply: function (value) {
            this.minutes = +value;
          }
        },
        'm': {
          regex: '[0-9]|[1-5][0-9]',
          apply: function (value) {
            this.minutes = +value;
          }
        },
        'sss': {
          regex: '[0-9][0-9][0-9]',
          apply: function (value) {
            this.milliseconds = +value;
          }
        },
        'ss': {
          regex: '[0-5][0-9]',
          apply: function (value) {
            this.seconds = +value;
          }
        },
        's': {
          regex: '[0-9]|[1-5][0-9]',
          apply: function (value) {
            this.seconds = +value;
          }
        },
        'a': {
          regex: $locale.DATETIME_FORMATS.AMPMS.join('|'),
          apply: function (value) {
            if (this.hours === 12) {
              this.hours = 0;
            }
            if (value === 'PM') {
              this.hours += 12;
            }
          }
        }
      };
    };
    this.init();
    function createParser(format) {
      var map = [], regex = format.split('');
      angular.forEach(formatCodeToRegex, function (data, code) {
        var index = format.indexOf(code);
        if (index > -1) {
          format = format.split('');
          regex[index] = '(' + data.regex + ')';
          format[index] = '$';
          // Custom symbol to define consumed part of format
          for (var i = index + 1, n = index + code.length; i < n; i++) {
            regex[i] = '';
            format[i] = '$';
          }
          format = format.join('');
          map.push({
            index: index,
            apply: data.apply
          });
        }
      });
      return {
        regex: new RegExp('^' + regex.join('') + '$'),
        map: orderByFilter(map, 'index')
      };
    }
    this.parse = function (input, format, baseDate) {
      if (!angular.isString(input) || !format) {
        return input;
      }
      format = $locale.DATETIME_FORMATS[format] || format;
      format = format.replace(SPECIAL_CHARACTERS_REGEXP, '\\$&');
      if ($locale.id !== localeId) {
        this.init();
      }
      if (!this.parsers[format]) {
        this.parsers[format] = createParser(format);
      }
      var parser = this.parsers[format], regex = parser.regex, map = parser.map, results = input.match(regex);
      if (results && results.length) {
        var fields, dt;
        if (angular.isDate(baseDate) && !isNaN(baseDate.getTime())) {
          fields = {
            year: baseDate.getFullYear(),
            month: baseDate.getMonth(),
            date: baseDate.getDate(),
            hours: baseDate.getHours(),
            minutes: baseDate.getMinutes(),
            seconds: baseDate.getSeconds(),
            milliseconds: baseDate.getMilliseconds()
          };
        } else {
          if (baseDate) {
            $log.warn('dateparser:', 'baseDate is not a valid date');
          }
          fields = {
            year: 1900,
            month: 0,
            date: 1,
            hours: 0,
            minutes: 0,
            seconds: 0,
            milliseconds: 0
          };
        }
        for (var i = 1, n = results.length; i < n; i++) {
          var mapper = map[i - 1];
          if (mapper.apply) {
            mapper.apply.call(fields, results[i]);
          }
        }
        if (isValid(fields.year, fields.month, fields.date)) {
          dt = new Date(fields.year, fields.month, fields.date, fields.hours, fields.minutes, fields.seconds, fields.milliseconds || 0);
        }
        return dt;
      }
    };
    // Check if date is valid for specific month (and year for February).
    // Month: 0 = Jan, 1 = Feb, etc
    function isValid(year, month, date) {
      if (date < 1) {
        return false;
      }
      if (month === 1 && date > 28) {
        return date === 29 && (year % 4 === 0 && year % 100 !== 0 || year % 400 === 0);
      }
      if (month === 3 || month === 5 || month === 8 || month === 10) {
        return date < 31;
      }
      return true;
    }
  }
]);
/* Deprecated dateparser below */
angular.module('ui.bootstrap.dateparser').value('$dateParserSuppressWarning', false).service('dateParser', [
  '$log',
  '$dateParserSuppressWarning',
  'uibDateParser',
  function ($log, $dateParserSuppressWarning, uibDateParser) {
    if (!$dateParserSuppressWarning) {
      $log.warn('dateParser is now deprecated. Use uibDateParser instead.');
    }
    angular.extend(this, uibDateParser);
  }
]);
angular.module('ui.bootstrap.position', []).factory('$uibPosition', [
  '$document',
  '$window',
  function ($document, $window) {
    function getStyle(el, cssprop) {
      if (el.currentStyle) {
        //IE
        return el.currentStyle[cssprop];
      } else if ($window.getComputedStyle) {
        return $window.getComputedStyle(el)[cssprop];
      }
      // finally try and get inline style
      return el.style[cssprop];
    }
    /**
     * Checks if a given element is statically positioned
     * @param element - raw DOM element
     */
    function isStaticPositioned(element) {
      return (getStyle(element, 'position') || 'static') === 'static';
    }
    /**
     * returns the closest, non-statically positioned parentOffset of a given element
     * @param element
     */
    var parentOffsetEl = function (element) {
      var docDomEl = $document[0];
      var offsetParent = element.offsetParent || docDomEl;
      while (offsetParent && offsetParent !== docDomEl && isStaticPositioned(offsetParent)) {
        offsetParent = offsetParent.offsetParent;
      }
      return offsetParent || docDomEl;
    };
    return {
      position: function (element) {
        var elBCR = this.offset(element);
        var offsetParentBCR = {
            top: 0,
            left: 0
          };
        var offsetParentEl = parentOffsetEl(element[0]);
        if (offsetParentEl != $document[0]) {
          offsetParentBCR = this.offset(angular.element(offsetParentEl));
          offsetParentBCR.top += offsetParentEl.clientTop - offsetParentEl.scrollTop;
          offsetParentBCR.left += offsetParentEl.clientLeft - offsetParentEl.scrollLeft;
        }
        var boundingClientRect = element[0].getBoundingClientRect();
        return {
          width: boundingClientRect.width || element.prop('offsetWidth'),
          height: boundingClientRect.height || element.prop('offsetHeight'),
          top: elBCR.top - offsetParentBCR.top,
          left: elBCR.left - offsetParentBCR.left
        };
      },
      offset: function (element) {
        var boundingClientRect = element[0].getBoundingClientRect();
        return {
          width: boundingClientRect.width || element.prop('offsetWidth'),
          height: boundingClientRect.height || element.prop('offsetHeight'),
          top: boundingClientRect.top + ($window.pageYOffset || $document[0].documentElement.scrollTop),
          left: boundingClientRect.left + ($window.pageXOffset || $document[0].documentElement.scrollLeft)
        };
      },
      positionElements: function (hostEl, targetEl, positionStr, appendToBody) {
        var positionStrParts = positionStr.split('-');
        var pos0 = positionStrParts[0], pos1 = positionStrParts[1] || 'center';
        var hostElPos, targetElWidth, targetElHeight, targetElPos;
        hostElPos = appendToBody ? this.offset(hostEl) : this.position(hostEl);
        targetElWidth = targetEl.prop('offsetWidth');
        targetElHeight = targetEl.prop('offsetHeight');
        var shiftWidth = {
            center: function () {
              return hostElPos.left + hostElPos.width / 2 - targetElWidth / 2;
            },
            left: function () {
              return hostElPos.left;
            },
            right: function () {
              return hostElPos.left + hostElPos.width;
            }
          };
        var shiftHeight = {
            center: function () {
              return hostElPos.top + hostElPos.height / 2 - targetElHeight / 2;
            },
            top: function () {
              return hostElPos.top;
            },
            bottom: function () {
              return hostElPos.top + hostElPos.height;
            }
          };
        switch (pos0) {
        case 'right':
          targetElPos = {
            top: shiftHeight[pos1](),
            left: shiftWidth[pos0]()
          };
          break;
        case 'left':
          targetElPos = {
            top: shiftHeight[pos1](),
            left: hostElPos.left - targetElWidth
          };
          break;
        case 'bottom':
          targetElPos = {
            top: shiftHeight[pos0](),
            left: shiftWidth[pos1]()
          };
          break;
        default:
          targetElPos = {
            top: hostElPos.top - targetElHeight,
            left: shiftWidth[pos1]()
          };
          break;
        }
        return targetElPos;
      }
    };
  }
]);
/* Deprecated position below */
angular.module('ui.bootstrap.position').value('$positionSuppressWarning', false).service('$position', [
  '$log',
  '$positionSuppressWarning',
  '$uibPosition',
  function ($log, $positionSuppressWarning, $uibPosition) {
    if (!$positionSuppressWarning) {
      $log.warn('$position is now deprecated. Use $uibPosition instead.');
    }
    angular.extend(this, $uibPosition);
  }
]);
angular.module('ui.bootstrap.datepicker', [
  'ui.bootstrap.dateparser',
  'ui.bootstrap.position'
]).value('$datepickerSuppressError', false).constant('uibDatepickerConfig', {
  formatDay: 'dd',
  formatMonth: 'MMMM',
  formatYear: 'yyyy',
  formatDayHeader: 'EEE',
  formatDayTitle: 'MMMM yyyy',
  formatMonthTitle: 'yyyy',
  datepickerMode: 'day',
  minMode: 'day',
  maxMode: 'year',
  showWeeks: true,
  startingDay: 0,
  yearRange: 20,
  minDate: null,
  maxDate: null,
  shortcutPropagation: false
}).controller('UibDatepickerController', [
  '$scope',
  '$attrs',
  '$parse',
  '$interpolate',
  '$log',
  'dateFilter',
  'uibDatepickerConfig',
  '$datepickerSuppressError',
  function ($scope, $attrs, $parse, $interpolate, $log, dateFilter, datepickerConfig, $datepickerSuppressError) {
    var self = this, ngModelCtrl = { $setViewValue: angular.noop };
    // nullModelCtrl;
    // Modes chain
    this.modes = [
      'day',
      'month',
      'year'
    ];
    // Configuration attributes
    angular.forEach([
      'formatDay',
      'formatMonth',
      'formatYear',
      'formatDayHeader',
      'formatDayTitle',
      'formatMonthTitle',
      'showWeeks',
      'startingDay',
      'yearRange',
      'shortcutPropagation'
    ], function (key, index) {
      self[key] = angular.isDefined($attrs[key]) ? index < 6 ? $interpolate($attrs[key])($scope.$parent) : $scope.$parent.$eval($attrs[key]) : datepickerConfig[key];
    });
    // Watchable date attributes
    angular.forEach([
      'minDate',
      'maxDate'
    ], function (key) {
      if ($attrs[key]) {
        $scope.$parent.$watch($parse($attrs[key]), function (value) {
          self[key] = value ? new Date(value) : null;
          self.refreshView();
        });
      } else {
        self[key] = datepickerConfig[key] ? new Date(datepickerConfig[key]) : null;
      }
    });
    angular.forEach([
      'minMode',
      'maxMode'
    ], function (key) {
      if ($attrs[key]) {
        $scope.$parent.$watch($parse($attrs[key]), function (value) {
          self[key] = angular.isDefined(value) ? value : $attrs[key];
          $scope[key] = self[key];
          if (key == 'minMode' && self.modes.indexOf($scope.datepickerMode) < self.modes.indexOf(self[key]) || key == 'maxMode' && self.modes.indexOf($scope.datepickerMode) > self.modes.indexOf(self[key])) {
            $scope.datepickerMode = self[key];
          }
        });
      } else {
        self[key] = datepickerConfig[key] || null;
        $scope[key] = self[key];
      }
    });
    $scope.datepickerMode = $scope.datepickerMode || datepickerConfig.datepickerMode;
    $scope.uniqueId = 'datepicker-' + $scope.$id + '-' + Math.floor(Math.random() * 10000);
    if (angular.isDefined($attrs.initDate)) {
      this.activeDate = $scope.$parent.$eval($attrs.initDate) || new Date();
      $scope.$parent.$watch($attrs.initDate, function (initDate) {
        if (initDate && (ngModelCtrl.$isEmpty(ngModelCtrl.$modelValue) || ngModelCtrl.$invalid)) {
          self.activeDate = initDate;
          self.refreshView();
        }
      });
    } else {
      this.activeDate = new Date();
    }
    $scope.isActive = function (dateObject) {
      if (self.compare(dateObject.date, self.activeDate) === 0) {
        $scope.activeDateId = dateObject.uid;
        return true;
      }
      return false;
    };
    this.init = function (ngModelCtrl_) {
      ngModelCtrl = ngModelCtrl_;
      ngModelCtrl.$render = function () {
        self.render();
      };
    };
    this.render = function () {
      if (ngModelCtrl.$viewValue) {
        var date = new Date(ngModelCtrl.$viewValue), isValid = !isNaN(date);
        if (isValid) {
          this.activeDate = date;
        } else if (!$datepickerSuppressError) {
          $log.error('Datepicker directive: "ng-model" value must be a Date object, a number of milliseconds since 01.01.1970 or a string representing an RFC2822 or ISO 8601 date.');
        }
      }
      this.refreshView();
    };
    this.refreshView = function () {
      if (this.element) {
        this._refreshView();
        var date = ngModelCtrl.$viewValue ? new Date(ngModelCtrl.$viewValue) : null;
        ngModelCtrl.$setValidity('dateDisabled', !date || this.element && !this.isDisabled(date));
      }
    };
    this.createDateObject = function (date, format) {
      var model = ngModelCtrl.$viewValue ? new Date(ngModelCtrl.$viewValue) : null;
      return {
        date: date,
        label: dateFilter(date, format),
        selected: model && this.compare(date, model) === 0,
        disabled: this.isDisabled(date),
        current: this.compare(date, new Date()) === 0,
        customClass: this.customClass(date)
      };
    };
    this.isDisabled = function (date) {
      return this.minDate && this.compare(date, this.minDate) < 0 || this.maxDate && this.compare(date, this.maxDate) > 0 || $attrs.dateDisabled && $scope.dateDisabled({
        date: date,
        mode: $scope.datepickerMode
      });
    };
    this.customClass = function (date) {
      return $scope.customClass({
        date: date,
        mode: $scope.datepickerMode
      });
    };
    // Split array into smaller arrays
    this.split = function (arr, size) {
      var arrays = [];
      while (arr.length > 0) {
        arrays.push(arr.splice(0, size));
      }
      return arrays;
    };
    // Fix a hard-reprodusible bug with timezones
    // The bug depends on OS, browser, current timezone and current date
    // i.e.
    // var date = new Date(2014, 0, 1);
    // console.log(date.getFullYear(), date.getMonth(), date.getDate(), date.getHours());
    // can result in "2013 11 31 23" because of the bug.
    this.fixTimeZone = function (date) {
      var hours = date.getHours();
      date.setHours(hours === 23 ? hours + 2 : 0);
    };
    $scope.select = function (date) {
      if ($scope.datepickerMode === self.minMode) {
        var dt = ngModelCtrl.$viewValue ? new Date(ngModelCtrl.$viewValue) : new Date(0, 0, 0, 0, 0, 0, 0);
        dt.setFullYear(date.getFullYear(), date.getMonth(), date.getDate());
        ngModelCtrl.$setViewValue(dt);
        ngModelCtrl.$render();
      } else {
        self.activeDate = date;
        $scope.datepickerMode = self.modes[self.modes.indexOf($scope.datepickerMode) - 1];
      }
    };
    $scope.move = function (direction) {
      var year = self.activeDate.getFullYear() + direction * (self.step.years || 0), month = self.activeDate.getMonth() + direction * (self.step.months || 0);
      self.activeDate.setFullYear(year, month, 1);
      self.refreshView();
    };
    $scope.toggleMode = function (direction) {
      direction = direction || 1;
      if ($scope.datepickerMode === self.maxMode && direction === 1 || $scope.datepickerMode === self.minMode && direction === -1) {
        return;
      }
      $scope.datepickerMode = self.modes[self.modes.indexOf($scope.datepickerMode) + direction];
    };
    // Key event mapper
    $scope.keys = {
      13: 'enter',
      32: 'space',
      33: 'pageup',
      34: 'pagedown',
      35: 'end',
      36: 'home',
      37: 'left',
      38: 'up',
      39: 'right',
      40: 'down'
    };
    var focusElement = function () {
      self.element[0].focus();
    };
    // Listen for focus requests from popup directive
    $scope.$on('uib:datepicker.focus', focusElement);
    $scope.keydown = function (evt) {
      var key = $scope.keys[evt.which];
      if (!key || evt.shiftKey || evt.altKey) {
        return;
      }
      evt.preventDefault();
      if (!self.shortcutPropagation) {
        evt.stopPropagation();
      }
      if (key === 'enter' || key === 'space') {
        if (self.isDisabled(self.activeDate)) {
          return;  // do nothing
        }
        $scope.select(self.activeDate);
      } else if (evt.ctrlKey && (key === 'up' || key === 'down')) {
        $scope.toggleMode(key === 'up' ? 1 : -1);
      } else {
        self.handleKeyDown(key, evt);
        self.refreshView();
      }
    };
  }
]).controller('UibDaypickerController', [
  '$scope',
  '$element',
  'dateFilter',
  function (scope, $element, dateFilter) {
    var DAYS_IN_MONTH = [
        31,
        28,
        31,
        30,
        31,
        30,
        31,
        31,
        30,
        31,
        30,
        31
      ];
    this.step = { months: 1 };
    this.element = $element;
    function getDaysInMonth(year, month) {
      return month === 1 && year % 4 === 0 && (year % 100 !== 0 || year % 400 === 0) ? 29 : DAYS_IN_MONTH[month];
    }
    this.init = function (ctrl) {
      angular.extend(ctrl, this);
      scope.showWeeks = ctrl.showWeeks;
      ctrl.refreshView();
    };
    this.getDates = function (startDate, n) {
      var dates = new Array(n), current = new Date(startDate), i = 0, date;
      while (i < n) {
        date = new Date(current);
        this.fixTimeZone(date);
        dates[i++] = date;
        current.setDate(current.getDate() + 1);
      }
      return dates;
    };
    this._refreshView = function () {
      var year = this.activeDate.getFullYear(), month = this.activeDate.getMonth(), firstDayOfMonth = new Date(year, month, 1), difference = this.startingDay - firstDayOfMonth.getDay(), numDisplayedFromPreviousMonth = difference > 0 ? 7 - difference : -difference, firstDate = new Date(firstDayOfMonth);
      if (numDisplayedFromPreviousMonth > 0) {
        firstDate.setDate(-numDisplayedFromPreviousMonth + 1);
      }
      // 42 is the number of days on a six-month calendar
      var days = this.getDates(firstDate, 42);
      for (var i = 0; i < 42; i++) {
        days[i] = angular.extend(this.createDateObject(days[i], this.formatDay), {
          secondary: days[i].getMonth() !== month,
          uid: scope.uniqueId + '-' + i
        });
      }
      scope.labels = new Array(7);
      for (var j = 0; j < 7; j++) {
        scope.labels[j] = {
          abbr: dateFilter(days[j].date, this.formatDayHeader),
          full: dateFilter(days[j].date, 'EEEE')
        };
      }
      scope.title = dateFilter(this.activeDate, this.formatDayTitle);
      scope.rows = this.split(days, 7);
      if (scope.showWeeks) {
        scope.weekNumbers = [];
        var thursdayIndex = (4 + 7 - this.startingDay) % 7, numWeeks = scope.rows.length;
        for (var curWeek = 0; curWeek < numWeeks; curWeek++) {
          scope.weekNumbers.push(getISO8601WeekNumber(scope.rows[curWeek][thursdayIndex].date));
        }
      }
    };
    this.compare = function (date1, date2) {
      return new Date(date1.getFullYear(), date1.getMonth(), date1.getDate()) - new Date(date2.getFullYear(), date2.getMonth(), date2.getDate());
    };
    function getISO8601WeekNumber(date) {
      var checkDate = new Date(date);
      checkDate.setDate(checkDate.getDate() + 4 - (checkDate.getDay() || 7));
      // Thursday
      var time = checkDate.getTime();
      checkDate.setMonth(0);
      // Compare with Jan 1
      checkDate.setDate(1);
      return Math.floor(Math.round((time - checkDate) / 86400000) / 7) + 1;
    }
    this.handleKeyDown = function (key, evt) {
      var date = this.activeDate.getDate();
      if (key === 'left') {
        date = date - 1;  // up
      } else if (key === 'up') {
        date = date - 7;  // down
      } else if (key === 'right') {
        date = date + 1;  // down
      } else if (key === 'down') {
        date = date + 7;
      } else if (key === 'pageup' || key === 'pagedown') {
        var month = this.activeDate.getMonth() + (key === 'pageup' ? -1 : 1);
        this.activeDate.setMonth(month, 1);
        date = Math.min(getDaysInMonth(this.activeDate.getFullYear(), this.activeDate.getMonth()), date);
      } else if (key === 'home') {
        date = 1;
      } else if (key === 'end') {
        date = getDaysInMonth(this.activeDate.getFullYear(), this.activeDate.getMonth());
      }
      this.activeDate.setDate(date);
    };
  }
]).controller('UibMonthpickerController', [
  '$scope',
  '$element',
  'dateFilter',
  function (scope, $element, dateFilter) {
    this.step = { years: 1 };
    this.element = $element;
    this.init = function (ctrl) {
      angular.extend(ctrl, this);
      ctrl.refreshView();
    };
    this._refreshView = function () {
      var months = new Array(12), year = this.activeDate.getFullYear(), date;
      for (var i = 0; i < 12; i++) {
        date = new Date(year, i, 1);
        this.fixTimeZone(date);
        months[i] = angular.extend(this.createDateObject(date, this.formatMonth), { uid: scope.uniqueId + '-' + i });
      }
      scope.title = dateFilter(this.activeDate, this.formatMonthTitle);
      scope.rows = this.split(months, 3);
    };
    this.compare = function (date1, date2) {
      return new Date(date1.getFullYear(), date1.getMonth()) - new Date(date2.getFullYear(), date2.getMonth());
    };
    this.handleKeyDown = function (key, evt) {
      var date = this.activeDate.getMonth();
      if (key === 'left') {
        date = date - 1;  // up
      } else if (key === 'up') {
        date = date - 3;  // down
      } else if (key === 'right') {
        date = date + 1;  // down
      } else if (key === 'down') {
        date = date + 3;
      } else if (key === 'pageup' || key === 'pagedown') {
        var year = this.activeDate.getFullYear() + (key === 'pageup' ? -1 : 1);
        this.activeDate.setFullYear(year);
      } else if (key === 'home') {
        date = 0;
      } else if (key === 'end') {
        date = 11;
      }
      this.activeDate.setMonth(date);
    };
  }
]).controller('UibYearpickerController', [
  '$scope',
  '$element',
  'dateFilter',
  function (scope, $element, dateFilter) {
    var range;
    this.element = $element;
    function getStartingYear(year) {
      return parseInt((year - 1) / range, 10) * range + 1;
    }
    this.yearpickerInit = function () {
      range = this.yearRange;
      this.step = { years: range };
    };
    this._refreshView = function () {
      var years = new Array(range), date;
      for (var i = 0, start = getStartingYear(this.activeDate.getFullYear()); i < range; i++) {
        date = new Date(start + i, 0, 1);
        this.fixTimeZone(date);
        years[i] = angular.extend(this.createDateObject(date, this.formatYear), { uid: scope.uniqueId + '-' + i });
      }
      scope.title = [
        years[0].label,
        years[range - 1].label
      ].join(' - ');
      scope.rows = this.split(years, 5);
    };
    this.compare = function (date1, date2) {
      return date1.getFullYear() - date2.getFullYear();
    };
    this.handleKeyDown = function (key, evt) {
      var date = this.activeDate.getFullYear();
      if (key === 'left') {
        date = date - 1;  // up
      } else if (key === 'up') {
        date = date - 5;  // down
      } else if (key === 'right') {
        date = date + 1;  // down
      } else if (key === 'down') {
        date = date + 5;
      } else if (key === 'pageup' || key === 'pagedown') {
        date += (key === 'pageup' ? -1 : 1) * this.step.years;
      } else if (key === 'home') {
        date = getStartingYear(this.activeDate.getFullYear());
      } else if (key === 'end') {
        date = getStartingYear(this.activeDate.getFullYear()) + range - 1;
      }
      this.activeDate.setFullYear(date);
    };
  }
]).directive('uibDatepicker', function () {
  return {
    replace: true,
    templateUrl: function (element, attrs) {
      return attrs.templateUrl || 'template/datepicker/datepicker.html';
    },
    scope: {
      datepickerMode: '=?',
      dateDisabled: '&',
      customClass: '&',
      shortcutPropagation: '&?'
    },
    require: [
      'uibDatepicker',
      '^ngModel'
    ],
    controller: 'UibDatepickerController',
    controllerAs: 'datepicker',
    link: function (scope, element, attrs, ctrls) {
      var datepickerCtrl = ctrls[0], ngModelCtrl = ctrls[1];
      datepickerCtrl.init(ngModelCtrl);
    }
  };
}).directive('uibDaypicker', function () {
  return {
    replace: true,
    templateUrl: 'template/datepicker/day.html',
    require: [
      '^?uibDatepicker',
      'uibDaypicker',
      '^?datepicker'
    ],
    controller: 'UibDaypickerController',
    link: function (scope, element, attrs, ctrls) {
      var datepickerCtrl = ctrls[0] || ctrls[2], daypickerCtrl = ctrls[1];
      daypickerCtrl.init(datepickerCtrl);
    }
  };
}).directive('uibMonthpicker', function () {
  return {
    replace: true,
    templateUrl: 'template/datepicker/month.html',
    require: [
      '^?uibDatepicker',
      'uibMonthpicker',
      '^?datepicker'
    ],
    controller: 'UibMonthpickerController',
    link: function (scope, element, attrs, ctrls) {
      var datepickerCtrl = ctrls[0] || ctrls[2], monthpickerCtrl = ctrls[1];
      monthpickerCtrl.init(datepickerCtrl);
    }
  };
}).directive('uibYearpicker', function () {
  return {
    replace: true,
    templateUrl: 'template/datepicker/year.html',
    require: [
      '^?uibDatepicker',
      'uibYearpicker',
      '^?datepicker'
    ],
    controller: 'UibYearpickerController',
    link: function (scope, element, attrs, ctrls) {
      var ctrl = ctrls[0] || ctrls[2];
      angular.extend(ctrl, ctrls[1]);
      ctrl.yearpickerInit();
      ctrl.refreshView();
    }
  };
}).constant('uibDatepickerPopupConfig', {
  datepickerPopup: 'yyyy-MM-dd',
  datepickerPopupTemplateUrl: 'template/datepicker/popup.html',
  datepickerTemplateUrl: 'template/datepicker/datepicker.html',
  html5Types: {
    date: 'yyyy-MM-dd',
    'datetime-local': 'yyyy-MM-ddTHH:mm:ss.sss',
    'month': 'yyyy-MM'
  },
  currentText: 'Today',
  clearText: 'Clear',
  closeText: 'Done',
  closeOnDateSelection: true,
  appendToBody: false,
  showButtonBar: true,
  onOpenFocus: true
}).controller('UibDatepickerPopupController', [
  '$scope',
  '$element',
  '$attrs',
  '$compile',
  '$parse',
  '$document',
  '$rootScope',
  '$uibPosition',
  'dateFilter',
  'uibDateParser',
  'uibDatepickerPopupConfig',
  '$timeout',
  function (scope, element, attrs, $compile, $parse, $document, $rootScope, $position, dateFilter, dateParser, datepickerPopupConfig, $timeout) {
    var self = this;
    var cache = {}, isHtml5DateInput = false;
    var dateFormat, closeOnDateSelection, appendToBody, onOpenFocus, datepickerPopupTemplateUrl, datepickerTemplateUrl, popupEl, datepickerEl, ngModel, $popup;
    scope.watchData = {};
    this.init = function (_ngModel_) {
      ngModel = _ngModel_;
      closeOnDateSelection = angular.isDefined(attrs.closeOnDateSelection) ? scope.$parent.$eval(attrs.closeOnDateSelection) : datepickerPopupConfig.closeOnDateSelection;
      appendToBody = angular.isDefined(attrs.datepickerAppendToBody) ? scope.$parent.$eval(attrs.datepickerAppendToBody) : datepickerPopupConfig.appendToBody;
      onOpenFocus = angular.isDefined(attrs.onOpenFocus) ? scope.$parent.$eval(attrs.onOpenFocus) : datepickerPopupConfig.onOpenFocus;
      datepickerPopupTemplateUrl = angular.isDefined(attrs.datepickerPopupTemplateUrl) ? attrs.datepickerPopupTemplateUrl : datepickerPopupConfig.datepickerPopupTemplateUrl;
      datepickerTemplateUrl = angular.isDefined(attrs.datepickerTemplateUrl) ? attrs.datepickerTemplateUrl : datepickerPopupConfig.datepickerTemplateUrl;
      scope.showButtonBar = angular.isDefined(attrs.showButtonBar) ? scope.$parent.$eval(attrs.showButtonBar) : datepickerPopupConfig.showButtonBar;
      if (datepickerPopupConfig.html5Types[attrs.type]) {
        dateFormat = datepickerPopupConfig.html5Types[attrs.type];
        isHtml5DateInput = true;
      } else {
        dateFormat = attrs.datepickerPopup || datepickerPopupConfig.datepickerPopup;
        attrs.$observe('uibDatepickerPopup', function (value, oldValue) {
          var newDateFormat = value || datepickerPopupConfig.datepickerPopup;
          // Invalidate the $modelValue to ensure that formatters re-run
          // FIXME: Refactor when PR is merged: https://github.com/angular/angular.js/pull/10764
          if (newDateFormat !== dateFormat) {
            dateFormat = newDateFormat;
            ngModel.$modelValue = null;
            if (!dateFormat) {
              throw new Error('uibDatepickerPopup must have a date format specified.');
            }
          }
        });
      }
      if (!dateFormat) {
        throw new Error('uibDatepickerPopup must have a date format specified.');
      }
      if (isHtml5DateInput && attrs.datepickerPopup) {
        throw new Error('HTML5 date input types do not support custom formats.');
      }
      // popup element used to display calendar
      popupEl = angular.element('<div uib-datepicker-popup-wrap><div uib-datepicker></div></div>');
      popupEl.attr({
        'ng-model': 'date',
        'ng-change': 'dateSelection(date)',
        'template-url': datepickerPopupTemplateUrl
      });
      // datepicker element
      datepickerEl = angular.element(popupEl.children()[0]);
      datepickerEl.attr('template-url', datepickerTemplateUrl);
      if (isHtml5DateInput) {
        if (attrs.type === 'month') {
          datepickerEl.attr('datepicker-mode', '"month"');
          datepickerEl.attr('min-mode', 'month');
        }
      }
      if (attrs.datepickerOptions) {
        var options = scope.$parent.$eval(attrs.datepickerOptions);
        if (options && options.initDate) {
          scope.initDate = options.initDate;
          datepickerEl.attr('init-date', 'initDate');
          delete options.initDate;
        }
        angular.forEach(options, function (value, option) {
          datepickerEl.attr(cameltoDash(option), value);
        });
      }
      angular.forEach([
        'minMode',
        'maxMode',
        'minDate',
        'maxDate',
        'datepickerMode',
        'initDate',
        'shortcutPropagation'
      ], function (key) {
        if (attrs[key]) {
          var getAttribute = $parse(attrs[key]);
          scope.$parent.$watch(getAttribute, function (value) {
            scope.watchData[key] = value;
            if (key === 'minDate' || key === 'maxDate') {
              cache[key] = new Date(value);
            }
          });
          datepickerEl.attr(cameltoDash(key), 'watchData.' + key);
          // Propagate changes from datepicker to outside
          if (key === 'datepickerMode') {
            var setAttribute = getAttribute.assign;
            scope.$watch('watchData.' + key, function (value, oldvalue) {
              if (angular.isFunction(setAttribute) && value !== oldvalue) {
                setAttribute(scope.$parent, value);
              }
            });
          }
        }
      });
      if (attrs.dateDisabled) {
        datepickerEl.attr('date-disabled', 'dateDisabled({ date: date, mode: mode })');
      }
      if (attrs.showWeeks) {
        datepickerEl.attr('show-weeks', attrs.showWeeks);
      }
      if (attrs.customClass) {
        datepickerEl.attr('custom-class', 'customClass({ date: date, mode: mode })');
      }
      if (!isHtml5DateInput) {
        // Internal API to maintain the correct ng-invalid-[key] class
        ngModel.$$parserName = 'date';
        ngModel.$validators.date = validator;
        ngModel.$parsers.unshift(parseDate);
        ngModel.$formatters.push(function (value) {
          scope.date = value;
          return ngModel.$isEmpty(value) ? value : dateFilter(value, dateFormat);
        });
      } else {
        ngModel.$formatters.push(function (value) {
          scope.date = value;
          return value;
        });
      }
      // Detect changes in the view from the text box
      ngModel.$viewChangeListeners.push(function () {
        scope.date = dateParser.parse(ngModel.$viewValue, dateFormat, scope.date);
      });
      element.bind('keydown', inputKeydownBind);
      $popup = $compile(popupEl)(scope);
      // Prevent jQuery cache memory leak (template is now redundant after linking)
      popupEl.remove();
      if (appendToBody) {
        $document.find('body').append($popup);
      } else {
        element.after($popup);
      }
      scope.$on('$destroy', function () {
        if (scope.isOpen === true) {
          if (!$rootScope.$$phase) {
            scope.$apply(function () {
              scope.isOpen = false;
            });
          }
        }
        $popup.remove();
        element.unbind('keydown', inputKeydownBind);
        $document.unbind('click', documentClickBind);
      });
    };
    scope.getText = function (key) {
      return scope[key + 'Text'] || datepickerPopupConfig[key + 'Text'];
    };
    scope.isDisabled = function (date) {
      if (date === 'today') {
        date = new Date();
      }
      return scope.watchData.minDate && scope.compare(date, cache.minDate) < 0 || scope.watchData.maxDate && scope.compare(date, cache.maxDate) > 0;
    };
    scope.compare = function (date1, date2) {
      return new Date(date1.getFullYear(), date1.getMonth(), date1.getDate()) - new Date(date2.getFullYear(), date2.getMonth(), date2.getDate());
    };
    // Inner change
    scope.dateSelection = function (dt) {
      if (angular.isDefined(dt)) {
        scope.date = dt;
      }
      var date = scope.date ? dateFilter(scope.date, dateFormat) : null;
      // Setting to NULL is necessary for form validators to function
      element.val(date);
      ngModel.$setViewValue(date);
      if (closeOnDateSelection) {
        scope.isOpen = false;
        element[0].focus();
      }
    };
    scope.keydown = function (evt) {
      if (evt.which === 27) {
        scope.isOpen = false;
        element[0].focus();
      }
    };
    scope.select = function (date) {
      if (date === 'today') {
        var today = new Date();
        if (angular.isDate(scope.date)) {
          date = new Date(scope.date);
          date.setFullYear(today.getFullYear(), today.getMonth(), today.getDate());
        } else {
          date = new Date(today.setHours(0, 0, 0, 0));
        }
      }
      scope.dateSelection(date);
    };
    scope.close = function () {
      scope.isOpen = false;
      element[0].focus();
    };
    scope.$watch('isOpen', function (value) {
      if (value) {
        scope.position = appendToBody ? $position.offset(element) : $position.position(element);
        scope.position.top = scope.position.top + element.prop('offsetHeight');
        $timeout(function () {
          if (onOpenFocus) {
            scope.$broadcast('uib:datepicker.focus');
          }
          $document.bind('click', documentClickBind);
        }, 0, false);
      } else {
        $document.unbind('click', documentClickBind);
      }
    });
    function cameltoDash(string) {
      return string.replace(/([A-Z])/g, function ($1) {
        return '-' + $1.toLowerCase();
      });
    }
    function parseDate(viewValue) {
      if (angular.isNumber(viewValue)) {
        // presumably timestamp to date object
        viewValue = new Date(viewValue);
      }
      if (!viewValue) {
        return null;
      } else if (angular.isDate(viewValue) && !isNaN(viewValue)) {
        return viewValue;
      } else if (angular.isString(viewValue)) {
        var date = dateParser.parse(viewValue, dateFormat, scope.date);
        if (isNaN(date)) {
          return undefined;
        } else {
          return date;
        }
      } else {
        return undefined;
      }
    }
    function validator(modelValue, viewValue) {
      var value = modelValue || viewValue;
      if (!attrs.ngRequired && !value) {
        return true;
      }
      if (angular.isNumber(value)) {
        value = new Date(value);
      }
      if (!value) {
        return true;
      } else if (angular.isDate(value) && !isNaN(value)) {
        return true;
      } else if (angular.isString(value)) {
        var date = dateParser.parse(value, dateFormat);
        return !isNaN(date);
      } else {
        return false;
      }
    }
    function documentClickBind(event) {
      var popup = $popup[0];
      var dpContainsTarget = element[0].contains(event.target);
      // The popup node may not be an element node
      // In some browsers (IE) only element nodes have the 'contains' function
      var popupContainsTarget = popup.contains !== undefined && popup.contains(event.target);
      if (scope.isOpen && !(dpContainsTarget || popupContainsTarget)) {
        scope.$apply(function () {
          scope.isOpen = false;
        });
      }
    }
    function inputKeydownBind(evt) {
      if (evt.which === 27 && scope.isOpen) {
        evt.preventDefault();
        evt.stopPropagation();
        scope.$apply(function () {
          scope.isOpen = false;
        });
        element[0].focus();
      } else if (evt.which === 40 && !scope.isOpen) {
        evt.preventDefault();
        evt.stopPropagation();
        scope.$apply(function () {
          scope.isOpen = true;
        });
      }
    }
  }
]).directive('uibDatepickerPopup', function () {
  return {
    require: [
      'ngModel',
      'uibDatepickerPopup'
    ],
    controller: 'UibDatepickerPopupController',
    scope: {
      isOpen: '=?',
      currentText: '@',
      clearText: '@',
      closeText: '@',
      dateDisabled: '&',
      customClass: '&'
    },
    link: function (scope, element, attrs, ctrls) {
      var ngModel = ctrls[0], ctrl = ctrls[1];
      ctrl.init(ngModel);
    }
  };
}).directive('uibDatepickerPopupWrap', function () {
  return {
    replace: true,
    transclude: true,
    templateUrl: function (element, attrs) {
      return attrs.templateUrl || 'template/datepicker/popup.html';
    }
  };
});
/* Deprecated datepicker below */
angular.module('ui.bootstrap.datepicker').value('$datepickerSuppressWarning', false).controller('DatepickerController', [
  '$scope',
  '$attrs',
  '$log',
  '$datepickerSuppressWarning',
  '$controller',
  function ($scope, $attrs, $log, $datepickerSuppressWarning, $controller) {
    if (!$datepickerSuppressWarning) {
      $log.warn('DatepickerController is now deprecated. Use UibDatepickerController instead.');
    }
    return $controller('UibDatepickerController', {
      $scope: $scope,
      $attrs: $attrs
    });
  }
]).directive('datepicker', [
  '$log',
  '$datepickerSuppressWarning',
  function ($log, $datepickerSuppressWarning) {
    return {
      replace: true,
      templateUrl: function (element, attrs) {
        return attrs.templateUrl || 'template/datepicker/datepicker.html';
      },
      scope: {
        datepickerMode: '=?',
        dateDisabled: '&',
        customClass: '&',
        shortcutPropagation: '&?'
      },
      require: [
        'datepicker',
        '^ngModel'
      ],
      controller: 'DatepickerController',
      controllerAs: 'datepicker',
      link: function (scope, element, attrs, ctrls) {
        if (!$datepickerSuppressWarning) {
          $log.warn('datepicker is now deprecated. Use uib-datepicker instead.');
        }
        var datepickerCtrl = ctrls[0], ngModelCtrl = ctrls[1];
        datepickerCtrl.init(ngModelCtrl);
      }
    };
  }
]).directive('daypicker', [
  '$log',
  '$datepickerSuppressWarning',
  function ($log, $datepickerSuppressWarning) {
    return {
      replace: true,
      templateUrl: 'template/datepicker/day.html',
      require: [
        '^datepicker',
        'daypicker'
      ],
      controller: 'UibDaypickerController',
      link: function (scope, element, attrs, ctrls) {
        if (!$datepickerSuppressWarning) {
          $log.warn('daypicker is now deprecated. Use uib-daypicker instead.');
        }
        var datepickerCtrl = ctrls[0], daypickerCtrl = ctrls[1];
        daypickerCtrl.init(datepickerCtrl);
      }
    };
  }
]).directive('monthpicker', [
  '$log',
  '$datepickerSuppressWarning',
  function ($log, $datepickerSuppressWarning) {
    return {
      replace: true,
      templateUrl: 'template/datepicker/month.html',
      require: [
        '^datepicker',
        'monthpicker'
      ],
      controller: 'UibMonthpickerController',
      link: function (scope, element, attrs, ctrls) {
        if (!$datepickerSuppressWarning) {
          $log.warn('monthpicker is now deprecated. Use uib-monthpicker instead.');
        }
        var datepickerCtrl = ctrls[0], monthpickerCtrl = ctrls[1];
        monthpickerCtrl.init(datepickerCtrl);
      }
    };
  }
]).directive('yearpicker', [
  '$log',
  '$datepickerSuppressWarning',
  function ($log, $datepickerSuppressWarning) {
    return {
      replace: true,
      templateUrl: 'template/datepicker/year.html',
      require: [
        '^datepicker',
        'yearpicker'
      ],
      controller: 'UibYearpickerController',
      link: function (scope, element, attrs, ctrls) {
        if (!$datepickerSuppressWarning) {
          $log.warn('yearpicker is now deprecated. Use uib-yearpicker instead.');
        }
        var ctrl = ctrls[0];
        angular.extend(ctrl, ctrls[1]);
        ctrl.yearpickerInit();
        ctrl.refreshView();
      }
    };
  }
]).directive('datepickerPopup', [
  '$log',
  '$datepickerSuppressWarning',
  function ($log, $datepickerSuppressWarning) {
    return {
      require: [
        'ngModel',
        'datepickerPopup'
      ],
      controller: 'UibDatepickerPopupController',
      scope: {
        isOpen: '=?',
        currentText: '@',
        clearText: '@',
        closeText: '@',
        dateDisabled: '&',
        customClass: '&'
      },
      link: function (scope, element, attrs, ctrls) {
        if (!$datepickerSuppressWarning) {
          $log.warn('datepicker-popup is now deprecated. Use uib-datepicker-popup instead.');
        }
        var ngModel = ctrls[0], ctrl = ctrls[1];
        ctrl.init(ngModel);
      }
    };
  }
]).directive('datepickerPopupWrap', [
  '$log',
  '$datepickerSuppressWarning',
  function ($log, $datepickerSuppressWarning) {
    return {
      replace: true,
      transclude: true,
      templateUrl: function (element, attrs) {
        return attrs.templateUrl || 'template/datepicker/popup.html';
      },
      link: function () {
        if (!$datepickerSuppressWarning) {
          $log.warn('datepicker-popup-wrap is now deprecated. Use uib-datepicker-popup-wrap instead.');
        }
      }
    };
  }
]);
angular.module('ui.bootstrap.dropdown', ['ui.bootstrap.position']).constant('uibDropdownConfig', { openClass: 'open' }).service('uibDropdownService', [
  '$document',
  '$rootScope',
  function ($document, $rootScope) {
    var openScope = null;
    this.open = function (dropdownScope) {
      if (!openScope) {
        $document.bind('click', closeDropdown);
        $document.bind('keydown', keybindFilter);
      }
      if (openScope && openScope !== dropdownScope) {
        openScope.isOpen = false;
      }
      openScope = dropdownScope;
    };
    this.close = function (dropdownScope) {
      if (openScope === dropdownScope) {
        openScope = null;
        $document.unbind('click', closeDropdown);
        $document.unbind('keydown', keybindFilter);
      }
    };
    var closeDropdown = function (evt) {
      // This method may still be called during the same mouse event that
      // unbound this event handler. So check openScope before proceeding.
      if (!openScope) {
        return;
      }
      if (evt && openScope.getAutoClose() === 'disabled') {
        return;
      }
      var toggleElement = openScope.getToggleElement();
      if (evt && toggleElement && toggleElement[0].contains(evt.target)) {
        return;
      }
      var dropdownElement = openScope.getDropdownElement();
      if (evt && openScope.getAutoClose() === 'outsideClick' && dropdownElement && dropdownElement[0].contains(evt.target)) {
        return;
      }
      openScope.isOpen = false;
      if (!$rootScope.$$phase) {
        openScope.$apply();
      }
    };
    var keybindFilter = function (evt) {
      if (evt.which === 27) {
        openScope.focusToggleElement();
        closeDropdown();
      } else if (openScope.isKeynavEnabled() && /(38|40)/.test(evt.which) && openScope.isOpen) {
        evt.preventDefault();
        evt.stopPropagation();
        openScope.focusDropdownEntry(evt.which);
      }
    };
  }
]).controller('UibDropdownController', [
  '$scope',
  '$element',
  '$attrs',
  '$parse',
  'uibDropdownConfig',
  'uibDropdownService',
  '$animate',
  '$uibPosition',
  '$document',
  '$compile',
  '$templateRequest',
  function ($scope, $element, $attrs, $parse, dropdownConfig, uibDropdownService, $animate, $position, $document, $compile, $templateRequest) {
    var self = this, scope = $scope.$new(),
      // create a child scope so we are not polluting original one
      templateScope, openClass = dropdownConfig.openClass, getIsOpen, setIsOpen = angular.noop, toggleInvoker = $attrs.onToggle ? $parse($attrs.onToggle) : angular.noop, appendToBody = false, keynavEnabled = false, selectedOption = null;
    $element.addClass('dropdown');
    this.init = function () {
      if ($attrs.isOpen) {
        getIsOpen = $parse($attrs.isOpen);
        setIsOpen = getIsOpen.assign;
        $scope.$watch(getIsOpen, function (value) {
          scope.isOpen = !!value;
        });
      }
      appendToBody = angular.isDefined($attrs.dropdownAppendToBody);
      keynavEnabled = angular.isDefined($attrs.uibKeyboardNav);
      if (appendToBody && self.dropdownMenu) {
        $document.find('body').append(self.dropdownMenu);
        $element.on('$destroy', function handleDestroyEvent() {
          self.dropdownMenu.remove();
        });
      }
    };
    this.toggle = function (open) {
      return scope.isOpen = arguments.length ? !!open : !scope.isOpen;
    };
    // Allow other directives to watch status
    this.isOpen = function () {
      return scope.isOpen;
    };
    scope.getToggleElement = function () {
      return self.toggleElement;
    };
    scope.getAutoClose = function () {
      return $attrs.autoClose || 'always';  //or 'outsideClick' or 'disabled'
    };
    scope.getElement = function () {
      return $element;
    };
    scope.isKeynavEnabled = function () {
      return keynavEnabled;
    };
    scope.focusDropdownEntry = function (keyCode) {
      var elems = self.dropdownMenu ? angular.element(self.dropdownMenu).find('a') : angular.element($element).find('ul').eq(0).find('a');
      switch (keyCode) {
      case 40: {
          if (!angular.isNumber(self.selectedOption)) {
            self.selectedOption = 0;
          } else {
            self.selectedOption = self.selectedOption === elems.length - 1 ? self.selectedOption : self.selectedOption + 1;
          }
          break;
        }
      case 38: {
          if (!angular.isNumber(self.selectedOption)) {
            self.selectedOption = elems.length - 1;
          } else {
            self.selectedOption = self.selectedOption === 0 ? 0 : self.selectedOption - 1;
          }
          break;
        }
      }
      elems[self.selectedOption].focus();
    };
    scope.getDropdownElement = function () {
      return self.dropdownMenu;
    };
    scope.focusToggleElement = function () {
      if (self.toggleElement) {
        self.toggleElement[0].focus();
      }
    };
    scope.$watch('isOpen', function (isOpen, wasOpen) {
      if (appendToBody && self.dropdownMenu) {
        var pos = $position.positionElements($element, self.dropdownMenu, 'bottom-left', true);
        var css = {
            top: pos.top + 'px',
            display: isOpen ? 'block' : 'none'
          };
        var rightalign = self.dropdownMenu.hasClass('dropdown-menu-right');
        if (!rightalign) {
          css.left = pos.left + 'px';
          css.right = 'auto';
        } else {
          css.left = 'auto';
          css.right = window.innerWidth - (pos.left + $element.prop('offsetWidth')) + 'px';
        }
        self.dropdownMenu.css(css);
      }
      $animate[isOpen ? 'addClass' : 'removeClass']($element, openClass).then(function () {
        if (angular.isDefined(isOpen) && isOpen !== wasOpen) {
          toggleInvoker($scope, { open: !!isOpen });
        }
      });
      if (isOpen) {
        if (self.dropdownMenuTemplateUrl) {
          $templateRequest(self.dropdownMenuTemplateUrl).then(function (tplContent) {
            templateScope = scope.$new();
            $compile(tplContent.trim())(templateScope, function (dropdownElement) {
              var newEl = dropdownElement;
              self.dropdownMenu.replaceWith(newEl);
              self.dropdownMenu = newEl;
            });
          });
        }
        scope.focusToggleElement();
        uibDropdownService.open(scope);
      } else {
        if (self.dropdownMenuTemplateUrl) {
          if (templateScope) {
            templateScope.$destroy();
          }
          var newEl = angular.element('<ul class="dropdown-menu"></ul>');
          self.dropdownMenu.replaceWith(newEl);
          self.dropdownMenu = newEl;
        }
        uibDropdownService.close(scope);
        self.selectedOption = null;
      }
      if (angular.isFunction(setIsOpen)) {
        setIsOpen($scope, isOpen);
      }
    });
    $scope.$on('$locationChangeSuccess', function () {
      if (scope.getAutoClose() !== 'disabled') {
        scope.isOpen = false;
      }
    });
    var offDestroy = $scope.$on('$destroy', function () {
        scope.$destroy();
      });
    scope.$on('$destroy', offDestroy);
  }
]).directive('uibDropdown', function () {
  return {
    controller: 'UibDropdownController',
    link: function (scope, element, attrs, dropdownCtrl) {
      dropdownCtrl.init();
    }
  };
}).directive('uibDropdownMenu', function () {
  return {
    restrict: 'AC',
    require: '?^uibDropdown',
    link: function (scope, element, attrs, dropdownCtrl) {
      if (!dropdownCtrl || angular.isDefined(attrs.dropdownNested)) {
        return;
      }
      element.addClass('dropdown-menu');
      var tplUrl = attrs.templateUrl;
      if (tplUrl) {
        dropdownCtrl.dropdownMenuTemplateUrl = tplUrl;
      }
      if (!dropdownCtrl.dropdownMenu) {
        dropdownCtrl.dropdownMenu = element;
      }
    }
  };
}).directive('uibKeyboardNav', function () {
  return {
    restrict: 'A',
    require: '?^uibDropdown',
    link: function (scope, element, attrs, dropdownCtrl) {
      element.bind('keydown', function (e) {
        if ([
            38,
            40
          ].indexOf(e.which) !== -1) {
          e.preventDefault();
          e.stopPropagation();
          var elems = dropdownCtrl.dropdownMenu.find('a');
          switch (e.which) {
          case 40: {
              // Down
              if (!angular.isNumber(dropdownCtrl.selectedOption)) {
                dropdownCtrl.selectedOption = 0;
              } else {
                dropdownCtrl.selectedOption = dropdownCtrl.selectedOption === elems.length - 1 ? dropdownCtrl.selectedOption : dropdownCtrl.selectedOption + 1;
              }
              break;
            }
          case 38: {
              // Up
              if (!angular.isNumber(dropdownCtrl.selectedOption)) {
                dropdownCtrl.selectedOption = elems.length - 1;
              } else {
                dropdownCtrl.selectedOption = dropdownCtrl.selectedOption === 0 ? 0 : dropdownCtrl.selectedOption - 1;
              }
              break;
            }
          }
          elems[dropdownCtrl.selectedOption].focus();
        }
      });
    }
  };
}).directive('uibDropdownToggle', function () {
  return {
    require: '?^uibDropdown',
    link: function (scope, element, attrs, dropdownCtrl) {
      if (!dropdownCtrl) {
        return;
      }
      element.addClass('dropdown-toggle');
      dropdownCtrl.toggleElement = element;
      var toggleDropdown = function (event) {
        event.preventDefault();
        if (!element.hasClass('disabled') && !attrs.disabled) {
          scope.$apply(function () {
            dropdownCtrl.toggle();
          });
        }
      };
      element.bind('click', toggleDropdown);
      // WAI-ARIA
      element.attr({
        'aria-haspopup': true,
        'aria-expanded': false
      });
      scope.$watch(dropdownCtrl.isOpen, function (isOpen) {
        element.attr('aria-expanded', !!isOpen);
      });
      scope.$on('$destroy', function () {
        element.unbind('click', toggleDropdown);
      });
    }
  };
});
/* Deprecated dropdown below */
angular.module('ui.bootstrap.dropdown').value('$dropdownSuppressWarning', false).service('dropdownService', [
  '$log',
  '$dropdownSuppressWarning',
  'uibDropdownService',
  function ($log, $dropdownSuppressWarning, uibDropdownService) {
    if (!$dropdownSuppressWarning) {
      $log.warn('dropdownService is now deprecated. Use uibDropdownService instead.');
    }
    angular.extend(this, uibDropdownService);
  }
]).controller('DropdownController', [
  '$scope',
  '$element',
  '$attrs',
  '$log',
  '$dropdownSuppressWarning',
  '$controller',
  function ($scope, $element, $attrs, $log, $dropdownSuppressWarning, $controller) {
    if (!$dropdownSuppressWarning) {
      $log.warn('DropdownController is now deprecated. Use UibDropdownController instead.');
    }
    return $controller('UibDropdownController', {
      $scope: $scope,
      $element: $element,
      $attrs: $attrs
    });
  }
]).directive('dropdown', [
  '$log',
  '$dropdownSuppressWarning',
  function ($log, $dropdownSuppressWarning) {
    return {
      controller: 'DropdownController',
      link: function (scope, element, attrs, dropdownCtrl) {
        if (!$dropdownSuppressWarning) {
          $log.warn('dropdown is now deprecated. Use uib-dropdown instead.');
        }
        dropdownCtrl.init();
      }
    };
  }
]).directive('dropdownMenu', [
  '$log',
  '$dropdownSuppressWarning',
  function ($log, $dropdownSuppressWarning) {
    return {
      restrict: 'AC',
      require: '?^dropdown',
      link: function (scope, element, attrs, dropdownCtrl) {
        if (!dropdownCtrl) {
          return;
        }
        if (!$dropdownSuppressWarning) {
          $log.warn('dropdown-menu is now deprecated. Use uib-dropdown-menu instead.');
        }
        element.addClass('dropdown-menu');
        var tplUrl = attrs.templateUrl;
        if (tplUrl) {
          dropdownCtrl.dropdownMenuTemplateUrl = tplUrl;
        }
        if (!dropdownCtrl.dropdownMenu) {
          dropdownCtrl.dropdownMenu = element;
        }
      }
    };
  }
]).directive('keyboardNav', [
  '$log',
  '$dropdownSuppressWarning',
  function ($log, $dropdownSuppressWarning) {
    return {
      restrict: 'A',
      require: '?^dropdown',
      link: function (scope, element, attrs, dropdownCtrl) {
        if (!$dropdownSuppressWarning) {
          $log.warn('keyboard-nav is now deprecated. Use uib-keyboard-nav instead.');
        }
        element.bind('keydown', function (e) {
          if ([
              38,
              40
            ].indexOf(e.which) !== -1) {
            e.preventDefault();
            e.stopPropagation();
            var elems = dropdownCtrl.dropdownMenu.find('a');
            switch (e.which) {
            case 40: {
                // Down
                if (!angular.isNumber(dropdownCtrl.selectedOption)) {
                  dropdownCtrl.selectedOption = 0;
                } else {
                  dropdownCtrl.selectedOption = dropdownCtrl.selectedOption === elems.length - 1 ? dropdownCtrl.selectedOption : dropdownCtrl.selectedOption + 1;
                }
                break;
              }
            case 38: {
                // Up
                if (!angular.isNumber(dropdownCtrl.selectedOption)) {
                  dropdownCtrl.selectedOption = elems.length - 1;
                } else {
                  dropdownCtrl.selectedOption = dropdownCtrl.selectedOption === 0 ? 0 : dropdownCtrl.selectedOption - 1;
                }
                break;
              }
            }
            elems[dropdownCtrl.selectedOption].focus();
          }
        });
      }
    };
  }
]).directive('dropdownToggle', [
  '$log',
  '$dropdownSuppressWarning',
  function ($log, $dropdownSuppressWarning) {
    return {
      require: '?^dropdown',
      link: function (scope, element, attrs, dropdownCtrl) {
        if (!$dropdownSuppressWarning) {
          $log.warn('dropdown-toggle is now deprecated. Use uib-dropdown-toggle instead.');
        }
        if (!dropdownCtrl) {
          return;
        }
        element.addClass('dropdown-toggle');
        dropdownCtrl.toggleElement = element;
        var toggleDropdown = function (event) {
          event.preventDefault();
          if (!element.hasClass('disabled') && !attrs.disabled) {
            scope.$apply(function () {
              dropdownCtrl.toggle();
            });
          }
        };
        element.bind('click', toggleDropdown);
        // WAI-ARIA
        element.attr({
          'aria-haspopup': true,
          'aria-expanded': false
        });
        scope.$watch(dropdownCtrl.isOpen, function (isOpen) {
          element.attr('aria-expanded', !!isOpen);
        });
        scope.$on('$destroy', function () {
          element.unbind('click', toggleDropdown);
        });
      }
    };
  }
]);
angular.module('ui.bootstrap.stackedMap', []).factory('$$stackedMap', function () {
  return {
    createNew: function () {
      var stack = [];
      return {
        add: function (key, value) {
          stack.push({
            key: key,
            value: value
          });
        },
        get: function (key) {
          for (var i = 0; i < stack.length; i++) {
            if (key == stack[i].key) {
              return stack[i];
            }
          }
        },
        keys: function () {
          var keys = [];
          for (var i = 0; i < stack.length; i++) {
            keys.push(stack[i].key);
          }
          return keys;
        },
        top: function () {
          return stack[stack.length - 1];
        },
        remove: function (key) {
          var idx = -1;
          for (var i = 0; i < stack.length; i++) {
            if (key == stack[i].key) {
              idx = i;
              break;
            }
          }
          return stack.splice(idx, 1)[0];
        },
        removeTop: function () {
          return stack.splice(stack.length - 1, 1)[0];
        },
        length: function () {
          return stack.length;
        }
      };
    }
  };
});
angular.module('ui.bootstrap.modal', ['ui.bootstrap.stackedMap']).factory('$$multiMap', function () {
  return {
    createNew: function () {
      var map = {};
      return {
        entries: function () {
          return Object.keys(map).map(function (key) {
            return {
              key: key,
              value: map[key]
            };
          });
        },
        get: function (key) {
          return map[key];
        },
        hasKey: function (key) {
          return !!map[key];
        },
        keys: function () {
          return Object.keys(map);
        },
        put: function (key, value) {
          if (!map[key]) {
            map[key] = [];
          }
          map[key].push(value);
        },
        remove: function (key, value) {
          var values = map[key];
          if (!values) {
            return;
          }
          var idx = values.indexOf(value);
          if (idx !== -1) {
            values.splice(idx, 1);
          }
          if (!values.length) {
            delete map[key];
          }
        }
      };
    }
  };
}).directive('uibModalBackdrop', [
  '$animate',
  '$injector',
  '$uibModalStack',
  function ($animate, $injector, $modalStack) {
    var $animateCss = null;
    if ($injector.has('$animateCss')) {
      $animateCss = $injector.get('$animateCss');
    }
    return {
      replace: true,
      templateUrl: 'template/modal/backdrop.html',
      compile: function (tElement, tAttrs) {
        tElement.addClass(tAttrs.backdropClass);
        return linkFn;
      }
    };
    function linkFn(scope, element, attrs) {
      // Temporary fix for prefixing
      element.addClass('modal-backdrop');
      if (attrs.modalInClass) {
        if ($animateCss) {
          $animateCss(element, { addClass: attrs.modalInClass }).start();
        } else {
          $animate.addClass(element, attrs.modalInClass);
        }
        scope.$on($modalStack.NOW_CLOSING_EVENT, function (e, setIsAsync) {
          var done = setIsAsync();
          if ($animateCss) {
            $animateCss(element, { removeClass: attrs.modalInClass }).start().then(done);
          } else {
            $animate.removeClass(element, attrs.modalInClass).then(done);
          }
        });
      }
    }
  }
]).directive('uibModalWindow', [
  '$uibModalStack',
  '$q',
  '$animate',
  '$injector',
  function ($modalStack, $q, $animate, $injector) {
    var $animateCss = null;
    if ($injector.has('$animateCss')) {
      $animateCss = $injector.get('$animateCss');
    }
    return {
      scope: { index: '@' },
      replace: true,
      transclude: true,
      templateUrl: function (tElement, tAttrs) {
        return tAttrs.templateUrl || 'template/modal/window.html';
      },
      link: function (scope, element, attrs) {
        element.addClass(attrs.windowClass || '');
        element.addClass(attrs.windowTopClass || '');
        scope.size = attrs.size;
        scope.close = function (evt) {
          var modal = $modalStack.getTop();
          if (modal && modal.value.backdrop && modal.value.backdrop !== 'static' && evt.target === evt.currentTarget) {
            evt.preventDefault();
            evt.stopPropagation();
            $modalStack.dismiss(modal.key, 'backdrop click');
          }
        };
        // moved from template to fix issue #2280
        element.on('click', scope.close);
        // This property is only added to the scope for the purpose of detecting when this directive is rendered.
        // We can detect that by using this property in the template associated with this directive and then use
        // {@link Attribute#$observe} on it. For more details please see {@link TableColumnResize}.
        scope.$isRendered = true;
        // Deferred object that will be resolved when this modal is render.
        var modalRenderDeferObj = $q.defer();
        // Observe function will be called on next digest cycle after compilation, ensuring that the DOM is ready.
        // In order to use this way of finding whether DOM is ready, we need to observe a scope property used in modal's template.
        attrs.$observe('modalRender', function (value) {
          if (value == 'true') {
            modalRenderDeferObj.resolve();
          }
        });
        modalRenderDeferObj.promise.then(function () {
          var animationPromise = null;
          if (attrs.modalInClass) {
            if ($animateCss) {
              animationPromise = $animateCss(element, { addClass: attrs.modalInClass }).start();
            } else {
              animationPromise = $animate.addClass(element, attrs.modalInClass);
            }
            scope.$on($modalStack.NOW_CLOSING_EVENT, function (e, setIsAsync) {
              var done = setIsAsync();
              if ($animateCss) {
                $animateCss(element, { removeClass: attrs.modalInClass }).start().then(done);
              } else {
                $animate.removeClass(element, attrs.modalInClass).then(done);
              }
            });
          }
          $q.when(animationPromise).then(function () {
            var inputsWithAutofocus = element[0].querySelectorAll('[autofocus]');
            /**
             * Auto-focusing of a freshly-opened modal element causes any child elements
             * with the autofocus attribute to lose focus. This is an issue on touch
             * based devices which will show and then hide the onscreen keyboard.
             * Attempts to refocus the autofocus element via JavaScript will not reopen
             * the onscreen keyboard. Fixed by updated the focusing logic to only autofocus
             * the modal element if the modal does not contain an autofocus element.
             */
            if (inputsWithAutofocus.length) {
              inputsWithAutofocus[0].focus();
            } else {
              element[0].focus();
            }
          });
          // Notify {@link $modalStack} that modal is rendered.
          var modal = $modalStack.getTop();
          if (modal) {
            $modalStack.modalRendered(modal.key);
          }
        });
      }
    };
  }
]).directive('uibModalAnimationClass', [function () {
    return {
      compile: function (tElement, tAttrs) {
        if (tAttrs.modalAnimation) {
          tElement.addClass(tAttrs.uibModalAnimationClass);
        }
      }
    };
  }]).directive('uibModalTransclude', function () {
  return {
    link: function ($scope, $element, $attrs, controller, $transclude) {
      $transclude($scope.$parent, function (clone) {
        $element.empty();
        $element.append(clone);
      });
    }
  };
}).factory('$uibModalStack', [
  '$animate',
  '$timeout',
  '$document',
  '$compile',
  '$rootScope',
  '$q',
  '$injector',
  '$$multiMap',
  '$$stackedMap',
  function ($animate, $timeout, $document, $compile, $rootScope, $q, $injector, $$multiMap, $$stackedMap) {
    var $animateCss = null;
    if ($injector.has('$animateCss')) {
      $animateCss = $injector.get('$animateCss');
    }
    var OPENED_MODAL_CLASS = 'modal-open';
    var backdropDomEl, backdropScope;
    var openedWindows = $$stackedMap.createNew();
    var openedClasses = $$multiMap.createNew();
    var $modalStack = { NOW_CLOSING_EVENT: 'modal.stack.now-closing' };
    //Modal focus behavior
    var focusableElementList;
    var focusIndex = 0;
    var tababbleSelector = 'a[href], area[href], input:not([disabled]), ' + 'button:not([disabled]),select:not([disabled]), textarea:not([disabled]), ' + 'iframe, object, embed, *[tabindex], *[contenteditable=true]';
    function backdropIndex() {
      var topBackdropIndex = -1;
      var opened = openedWindows.keys();
      for (var i = 0; i < opened.length; i++) {
        if (openedWindows.get(opened[i]).value.backdrop) {
          topBackdropIndex = i;
        }
      }
      return topBackdropIndex;
    }
    $rootScope.$watch(backdropIndex, function (newBackdropIndex) {
      if (backdropScope) {
        backdropScope.index = newBackdropIndex;
      }
    });
    function removeModalWindow(modalInstance, elementToReceiveFocus) {
      var body = $document.find('body').eq(0);
      var modalWindow = openedWindows.get(modalInstance).value;
      //clean up the stack
      openedWindows.remove(modalInstance);
      removeAfterAnimate(modalWindow.modalDomEl, modalWindow.modalScope, function () {
        var modalBodyClass = modalWindow.openedClass || OPENED_MODAL_CLASS;
        openedClasses.remove(modalBodyClass, modalInstance);
        body.toggleClass(modalBodyClass, openedClasses.hasKey(modalBodyClass));
        toggleTopWindowClass(true);
      });
      checkRemoveBackdrop();
      //move focus to specified element if available, or else to body
      if (elementToReceiveFocus && elementToReceiveFocus.focus) {
        elementToReceiveFocus.focus();
      } else {
        body.focus();
      }
    }
    // Add or remove "windowTopClass" from the top window in the stack
    function toggleTopWindowClass(toggleSwitch) {
      var modalWindow;
      if (openedWindows.length() > 0) {
        modalWindow = openedWindows.top().value;
        modalWindow.modalDomEl.toggleClass(modalWindow.windowTopClass || '', toggleSwitch);
      }
    }
    function checkRemoveBackdrop() {
      //remove backdrop if no longer needed
      if (backdropDomEl && backdropIndex() == -1) {
        var backdropScopeRef = backdropScope;
        removeAfterAnimate(backdropDomEl, backdropScope, function () {
          backdropScopeRef = null;
        });
        backdropDomEl = undefined;
        backdropScope = undefined;
      }
    }
    function removeAfterAnimate(domEl, scope, done) {
      var asyncDeferred;
      var asyncPromise = null;
      var setIsAsync = function () {
        if (!asyncDeferred) {
          asyncDeferred = $q.defer();
          asyncPromise = asyncDeferred.promise;
        }
        return function asyncDone() {
          asyncDeferred.resolve();
        };
      };
      scope.$broadcast($modalStack.NOW_CLOSING_EVENT, setIsAsync);
      // Note that it's intentional that asyncPromise might be null.
      // That's when setIsAsync has not been called during the
      // NOW_CLOSING_EVENT broadcast.
      return $q.when(asyncPromise).then(afterAnimating);
      function afterAnimating() {
        if (afterAnimating.done) {
          return;
        }
        afterAnimating.done = true;
        if ($animateCss) {
          $animateCss(domEl, { event: 'leave' }).start().then(function () {
            domEl.remove();
          });
        } else {
          $animate.leave(domEl);
        }
        scope.$destroy();
        if (done) {
          done();
        }
      }
    }
    $document.bind('keydown', function (evt) {
      if (evt.isDefaultPrevented()) {
        return evt;
      }
      var modal = openedWindows.top();
      if (modal && modal.value.keyboard) {
        switch (evt.which) {
        case 27: {
            evt.preventDefault();
            $rootScope.$apply(function () {
              $modalStack.dismiss(modal.key, 'escape key press');
            });
            break;
          }
        case 9: {
            $modalStack.loadFocusElementList(modal);
            var focusChanged = false;
            if (evt.shiftKey) {
              if ($modalStack.isFocusInFirstItem(evt)) {
                focusChanged = $modalStack.focusLastFocusableElement();
              }
            } else {
              if ($modalStack.isFocusInLastItem(evt)) {
                focusChanged = $modalStack.focusFirstFocusableElement();
              }
            }
            if (focusChanged) {
              evt.preventDefault();
              evt.stopPropagation();
            }
            break;
          }
        }
      }
    });
    $modalStack.open = function (modalInstance, modal) {
      var modalOpener = $document[0].activeElement, modalBodyClass = modal.openedClass || OPENED_MODAL_CLASS;
      toggleTopWindowClass(false);
      openedWindows.add(modalInstance, {
        deferred: modal.deferred,
        renderDeferred: modal.renderDeferred,
        modalScope: modal.scope,
        backdrop: modal.backdrop,
        keyboard: modal.keyboard,
        openedClass: modal.openedClass,
        windowTopClass: modal.windowTopClass
      });
      openedClasses.put(modalBodyClass, modalInstance);
      var body = $document.find('body').eq(0), currBackdropIndex = backdropIndex();
      if (currBackdropIndex >= 0 && !backdropDomEl) {
        backdropScope = $rootScope.$new(true);
        backdropScope.index = currBackdropIndex;
        var angularBackgroundDomEl = angular.element('<div uib-modal-backdrop="modal-backdrop"></div>');
        angularBackgroundDomEl.attr('backdrop-class', modal.backdropClass);
        if (modal.animation) {
          angularBackgroundDomEl.attr('modal-animation', 'true');
        }
        backdropDomEl = $compile(angularBackgroundDomEl)(backdropScope);
        body.append(backdropDomEl);
      }
      var angularDomEl = angular.element('<div uib-modal-window="modal-window"></div>');
      angularDomEl.attr({
        'template-url': modal.windowTemplateUrl,
        'window-class': modal.windowClass,
        'window-top-class': modal.windowTopClass,
        'size': modal.size,
        'index': openedWindows.length() - 1,
        'animate': 'animate'
      }).html(modal.content);
      if (modal.animation) {
        angularDomEl.attr('modal-animation', 'true');
      }
      var modalDomEl = $compile(angularDomEl)(modal.scope);
      openedWindows.top().value.modalDomEl = modalDomEl;
      openedWindows.top().value.modalOpener = modalOpener;
      body.append(modalDomEl);
      body.addClass(modalBodyClass);
      $modalStack.clearFocusListCache();
    };
    function broadcastClosing(modalWindow, resultOrReason, closing) {
      return !modalWindow.value.modalScope.$broadcast('modal.closing', resultOrReason, closing).defaultPrevented;
    }
    $modalStack.close = function (modalInstance, result) {
      var modalWindow = openedWindows.get(modalInstance);
      if (modalWindow && broadcastClosing(modalWindow, result, true)) {
        modalWindow.value.modalScope.$$uibDestructionScheduled = true;
        modalWindow.value.deferred.resolve(result);
        removeModalWindow(modalInstance, modalWindow.value.modalOpener);
        return true;
      }
      return !modalWindow;
    };
    $modalStack.dismiss = function (modalInstance, reason) {
      var modalWindow = openedWindows.get(modalInstance);
      if (modalWindow && broadcastClosing(modalWindow, reason, false)) {
        modalWindow.value.modalScope.$$uibDestructionScheduled = true;
        modalWindow.value.deferred.reject(reason);
        removeModalWindow(modalInstance, modalWindow.value.modalOpener);
        return true;
      }
      return !modalWindow;
    };
    $modalStack.dismissAll = function (reason) {
      var topModal = this.getTop();
      while (topModal && this.dismiss(topModal.key, reason)) {
        topModal = this.getTop();
      }
    };
    $modalStack.getTop = function () {
      return openedWindows.top();
    };
    $modalStack.modalRendered = function (modalInstance) {
      var modalWindow = openedWindows.get(modalInstance);
      if (modalWindow) {
        modalWindow.value.renderDeferred.resolve();
      }
    };
    $modalStack.focusFirstFocusableElement = function () {
      if (focusableElementList.length > 0) {
        focusableElementList[0].focus();
        return true;
      }
      return false;
    };
    $modalStack.focusLastFocusableElement = function () {
      if (focusableElementList.length > 0) {
        focusableElementList[focusableElementList.length - 1].focus();
        return true;
      }
      return false;
    };
    $modalStack.isFocusInFirstItem = function (evt) {
      if (focusableElementList.length > 0) {
        return (evt.target || evt.srcElement) == focusableElementList[0];
      }
      return false;
    };
    $modalStack.isFocusInLastItem = function (evt) {
      if (focusableElementList.length > 0) {
        return (evt.target || evt.srcElement) == focusableElementList[focusableElementList.length - 1];
      }
      return false;
    };
    $modalStack.clearFocusListCache = function () {
      focusableElementList = [];
      focusIndex = 0;
    };
    $modalStack.loadFocusElementList = function (modalWindow) {
      if (focusableElementList === undefined || !focusableElementList.length0) {
        if (modalWindow) {
          var modalDomE1 = modalWindow.value.modalDomEl;
          if (modalDomE1 && modalDomE1.length) {
            focusableElementList = modalDomE1[0].querySelectorAll(tababbleSelector);
          }
        }
      }
    };
    return $modalStack;
  }
]).provider('$uibModal', function () {
  var $modalProvider = {
      options: {
        animation: true,
        backdrop: true,
        keyboard: true
      },
      $get: [
        '$injector',
        '$rootScope',
        '$q',
        '$templateRequest',
        '$controller',
        '$uibModalStack',
        function ($injector, $rootScope, $q, $templateRequest, $controller, $modalStack) {
          var $modal = {};
          function getTemplatePromise(options) {
            return options.template ? $q.when(options.template) : $templateRequest(angular.isFunction(options.templateUrl) ? options.templateUrl() : options.templateUrl);
          }
          function getResolvePromises(resolves) {
            var promisesArr = [];
            angular.forEach(resolves, function (value) {
              if (angular.isFunction(value) || angular.isArray(value)) {
                promisesArr.push($q.when($injector.invoke(value)));
              } else if (angular.isString(value)) {
                promisesArr.push($q.when($injector.get(value)));
              } else {
                promisesArr.push($q.when(value));
              }
            });
            return promisesArr;
          }
          var promiseChain = null;
          $modal.getPromiseChain = function () {
            return promiseChain;
          };
          $modal.open = function (modalOptions) {
            var modalResultDeferred = $q.defer();
            var modalOpenedDeferred = $q.defer();
            var modalRenderDeferred = $q.defer();
            //prepare an instance of a modal to be injected into controllers and returned to a caller
            var modalInstance = {
                result: modalResultDeferred.promise,
                opened: modalOpenedDeferred.promise,
                rendered: modalRenderDeferred.promise,
                close: function (result) {
                  return $modalStack.close(modalInstance, result);
                },
                dismiss: function (reason) {
                  return $modalStack.dismiss(modalInstance, reason);
                }
              };
            //merge and clean up options
            modalOptions = angular.extend({}, $modalProvider.options, modalOptions);
            modalOptions.resolve = modalOptions.resolve || {};
            //verify options
            if (!modalOptions.template && !modalOptions.templateUrl) {
              throw new Error('One of template or templateUrl options is required.');
            }
            var templateAndResolvePromise = $q.all([getTemplatePromise(modalOptions)].concat(getResolvePromises(modalOptions.resolve)));
            // Wait for the resolution of the existing promise chain.
            // Then switch to our own combined promise dependency (regardless of how the previous modal fared).
            // Then add to $modalStack and resolve opened.
            // Finally clean up the chain variable if no subsequent modal has overwritten it.
            var samePromise;
            samePromise = promiseChain = $q.all([promiseChain]).then(function () {
              return templateAndResolvePromise;
            }, function () {
              return templateAndResolvePromise;
            }).then(function resolveSuccess(tplAndVars) {
              var modalScope = (modalOptions.scope || $rootScope).$new();
              modalScope.$close = modalInstance.close;
              modalScope.$dismiss = modalInstance.dismiss;
              modalScope.$on('$destroy', function () {
                if (!modalScope.$$uibDestructionScheduled) {
                  modalScope.$dismiss('$uibUnscheduledDestruction');
                }
              });
              var ctrlInstance, ctrlLocals = {};
              var resolveIter = 1;
              //controllers
              if (modalOptions.controller) {
                ctrlLocals.$scope = modalScope;
                ctrlLocals.$modalInstance = modalInstance;
                angular.forEach(modalOptions.resolve, function (value, key) {
                  ctrlLocals[key] = tplAndVars[resolveIter++];
                });
                ctrlInstance = $controller(modalOptions.controller, ctrlLocals);
                if (modalOptions.controllerAs) {
                  if (modalOptions.bindToController) {
                    angular.extend(ctrlInstance, modalScope);
                  }
                  modalScope[modalOptions.controllerAs] = ctrlInstance;
                }
              }
              $modalStack.open(modalInstance, {
                scope: modalScope,
                deferred: modalResultDeferred,
                renderDeferred: modalRenderDeferred,
                content: tplAndVars[0],
                animation: modalOptions.animation,
                backdrop: modalOptions.backdrop,
                keyboard: modalOptions.keyboard,
                backdropClass: modalOptions.backdropClass,
                windowTopClass: modalOptions.windowTopClass,
                windowClass: modalOptions.windowClass,
                windowTemplateUrl: modalOptions.windowTemplateUrl,
                size: modalOptions.size,
                openedClass: modalOptions.openedClass
              });
              modalOpenedDeferred.resolve(true);
            }, function resolveError(reason) {
              modalOpenedDeferred.reject(reason);
              modalResultDeferred.reject(reason);
            }).finally(function () {
              if (promiseChain === samePromise) {
                promiseChain = null;
              }
            });
            return modalInstance;
          };
          return $modal;
        }
      ]
    };
  return $modalProvider;
});
/* deprecated modal below */
angular.module('ui.bootstrap.modal').value('$modalSuppressWarning', false).directive('modalBackdrop', [
  '$animate',
  '$injector',
  '$modalStack',
  '$log',
  '$modalSuppressWarning',
  function ($animate, $injector, $modalStack, $log, $modalSuppressWarning) {
    var $animateCss = null;
    if ($injector.has('$animateCss')) {
      $animateCss = $injector.get('$animateCss');
    }
    return {
      replace: true,
      templateUrl: 'template/modal/backdrop.html',
      compile: function (tElement, tAttrs) {
        tElement.addClass(tAttrs.backdropClass);
        return linkFn;
      }
    };
    function linkFn(scope, element, attrs) {
      if (!$modalSuppressWarning) {
        $log.warn('modal-backdrop is now deprecated. Use uib-modal-backdrop instead.');
      }
      element.addClass('modal-backdrop');
      if (attrs.modalInClass) {
        if ($animateCss) {
          $animateCss(element, { addClass: attrs.modalInClass }).start();
        } else {
          $animate.addClass(element, attrs.modalInClass);
        }
        scope.$on($modalStack.NOW_CLOSING_EVENT, function (e, setIsAsync) {
          var done = setIsAsync();
          if ($animateCss) {
            $animateCss(element, { removeClass: attrs.modalInClass }).start().then(done);
          } else {
            $animate.removeClass(element, attrs.modalInClass).then(done);
          }
        });
      }
    }
  }
]).directive('modalWindow', [
  '$modalStack',
  '$q',
  '$animate',
  '$injector',
  '$log',
  '$modalSuppressWarning',
  function ($modalStack, $q, $animate, $injector, $log, $modalSuppressWarning) {
    var $animateCss = null;
    if ($injector.has('$animateCss')) {
      $animateCss = $injector.get('$animateCss');
    }
    return {
      scope: { index: '@' },
      replace: true,
      transclude: true,
      templateUrl: function (tElement, tAttrs) {
        return tAttrs.templateUrl || 'template/modal/window.html';
      },
      link: function (scope, element, attrs) {
        if (!$modalSuppressWarning) {
          $log.warn('modal-window is now deprecated. Use uib-modal-window instead.');
        }
        element.addClass(attrs.windowClass || '');
        element.addClass(attrs.windowTopClass || '');
        scope.size = attrs.size;
        scope.close = function (evt) {
          var modal = $modalStack.getTop();
          if (modal && modal.value.backdrop && modal.value.backdrop !== 'static' && evt.target === evt.currentTarget) {
            evt.preventDefault();
            evt.stopPropagation();
            $modalStack.dismiss(modal.key, 'backdrop click');
          }
        };
        // moved from template to fix issue #2280
        element.on('click', scope.close);
        // This property is only added to the scope for the purpose of detecting when this directive is rendered.
        // We can detect that by using this property in the template associated with this directive and then use
        // {@link Attribute#$observe} on it. For more details please see {@link TableColumnResize}.
        scope.$isRendered = true;
        // Deferred object that will be resolved when this modal is render.
        var modalRenderDeferObj = $q.defer();
        // Observe function will be called on next digest cycle after compilation, ensuring that the DOM is ready.
        // In order to use this way of finding whether DOM is ready, we need to observe a scope property used in modal's template.
        attrs.$observe('modalRender', function (value) {
          if (value == 'true') {
            modalRenderDeferObj.resolve();
          }
        });
        modalRenderDeferObj.promise.then(function () {
          var animationPromise = null;
          if (attrs.modalInClass) {
            if ($animateCss) {
              animationPromise = $animateCss(element, { addClass: attrs.modalInClass }).start();
            } else {
              animationPromise = $animate.addClass(element, attrs.modalInClass);
            }
            scope.$on($modalStack.NOW_CLOSING_EVENT, function (e, setIsAsync) {
              var done = setIsAsync();
              if ($animateCss) {
                $animateCss(element, { removeClass: attrs.modalInClass }).start().then(done);
              } else {
                $animate.removeClass(element, attrs.modalInClass).then(done);
              }
            });
          }
          $q.when(animationPromise).then(function () {
            var inputsWithAutofocus = element[0].querySelectorAll('[autofocus]');
            /**
               * Auto-focusing of a freshly-opened modal element causes any child elements
               * with the autofocus attribute to lose focus. This is an issue on touch
               * based devices which will show and then hide the onscreen keyboard.
               * Attempts to refocus the autofocus element via JavaScript will not reopen
               * the onscreen keyboard. Fixed by updated the focusing logic to only autofocus
               * the modal element if the modal does not contain an autofocus element.
               */
            if (inputsWithAutofocus.length) {
              inputsWithAutofocus[0].focus();
            } else {
              element[0].focus();
            }
          });
          // Notify {@link $modalStack} that modal is rendered.
          var modal = $modalStack.getTop();
          if (modal) {
            $modalStack.modalRendered(modal.key);
          }
        });
      }
    };
  }
]).directive('modalAnimationClass', [
  '$log',
  '$modalSuppressWarning',
  function ($log, $modalSuppressWarning) {
    return {
      compile: function (tElement, tAttrs) {
        if (!$modalSuppressWarning) {
          $log.warn('modal-animation-class is now deprecated. Use uib-modal-animation-class instead.');
        }
        if (tAttrs.modalAnimation) {
          tElement.addClass(tAttrs.modalAnimationClass);
        }
      }
    };
  }
]).directive('modalTransclude', [
  '$log',
  '$modalSuppressWarning',
  function ($log, $modalSuppressWarning) {
    return {
      link: function ($scope, $element, $attrs, controller, $transclude) {
        if (!$modalSuppressWarning) {
          $log.warn('modal-transclude is now deprecated. Use uib-modal-transclude instead.');
        }
        $transclude($scope.$parent, function (clone) {
          $element.empty();
          $element.append(clone);
        });
      }
    };
  }
]).service('$modalStack', [
  '$animate',
  '$timeout',
  '$document',
  '$compile',
  '$rootScope',
  '$q',
  '$injector',
  '$$multiMap',
  '$$stackedMap',
  '$uibModalStack',
  '$log',
  '$modalSuppressWarning',
  function ($animate, $timeout, $document, $compile, $rootScope, $q, $injector, $$multiMap, $$stackedMap, $uibModalStack, $log, $modalSuppressWarning) {
    if (!$modalSuppressWarning) {
      $log.warn('$modalStack is now deprecated. Use $uibModalStack instead.');
    }
    angular.extend(this, $uibModalStack);
  }
]).provider('$modal', [
  '$uibModalProvider',
  function ($uibModalProvider) {
    angular.extend(this, $uibModalProvider);
    this.$get = [
      '$injector',
      '$log',
      '$modalSuppressWarning',
      function ($injector, $log, $modalSuppressWarning) {
        if (!$modalSuppressWarning) {
          $log.warn('$modal is now deprecated. Use $uibModal instead.');
        }
        return $injector.invoke($uibModalProvider.$get);
      }
    ];
  }
]);
angular.module('ui.bootstrap.pagination', []).controller('UibPaginationController', [
  '$scope',
  '$attrs',
  '$parse',
  function ($scope, $attrs, $parse) {
    var self = this, ngModelCtrl = { $setViewValue: angular.noop },
      // nullModelCtrl
      setNumPages = $attrs.numPages ? $parse($attrs.numPages).assign : angular.noop;
    this.init = function (ngModelCtrl_, config) {
      ngModelCtrl = ngModelCtrl_;
      this.config = config;
      ngModelCtrl.$render = function () {
        self.render();
      };
      if ($attrs.itemsPerPage) {
        $scope.$parent.$watch($parse($attrs.itemsPerPage), function (value) {
          self.itemsPerPage = parseInt(value, 10);
          $scope.totalPages = self.calculateTotalPages();
        });
      } else {
        this.itemsPerPage = config.itemsPerPage;
      }
      $scope.$watch('totalItems', function () {
        $scope.totalPages = self.calculateTotalPages();
      });
      $scope.$watch('totalPages', function (value) {
        setNumPages($scope.$parent, value);
        // Readonly variable
        if ($scope.page > value) {
          $scope.selectPage(value);
        } else {
          ngModelCtrl.$render();
        }
      });
    };
    this.calculateTotalPages = function () {
      var totalPages = this.itemsPerPage < 1 ? 1 : Math.ceil($scope.totalItems / this.itemsPerPage);
      return Math.max(totalPages || 0, 1);
    };
    this.render = function () {
      $scope.page = parseInt(ngModelCtrl.$viewValue, 10) || 1;
    };
    $scope.selectPage = function (page, evt) {
      if (evt) {
        evt.preventDefault();
      }
      var clickAllowed = !$scope.ngDisabled || !evt;
      if (clickAllowed && $scope.page !== page && page > 0 && page <= $scope.totalPages) {
        if (evt && evt.target) {
          evt.target.blur();
        }
        ngModelCtrl.$setViewValue(page);
        ngModelCtrl.$render();
      }
    };
    $scope.getText = function (key) {
      return $scope[key + 'Text'] || self.config[key + 'Text'];
    };
    $scope.noPrevious = function () {
      return $scope.page === 1;
    };
    $scope.noNext = function () {
      return $scope.page === $scope.totalPages;
    };
  }
]).constant('uibPaginationConfig', {
  itemsPerPage: 10,
  boundaryLinks: false,
  directionLinks: true,
  firstText: 'First',
  previousText: 'Previous',
  nextText: 'Next',
  lastText: 'Last',
  rotate: true
}).directive('uibPagination', [
  '$parse',
  'uibPaginationConfig',
  function ($parse, paginationConfig) {
    return {
      restrict: 'EA',
      scope: {
        totalItems: '=',
        firstText: '@',
        previousText: '@',
        nextText: '@',
        lastText: '@',
        ngDisabled: '='
      },
      require: [
        'uibPagination',
        '?ngModel'
      ],
      controller: 'UibPaginationController',
      controllerAs: 'pagination',
      templateUrl: function (element, attrs) {
        return attrs.templateUrl || 'template/pagination/pagination.html';
      },
      replace: true,
      link: function (scope, element, attrs, ctrls) {
        var paginationCtrl = ctrls[0], ngModelCtrl = ctrls[1];
        if (!ngModelCtrl) {
          return;  // do nothing if no ng-model
        }
        // Setup configuration parameters
        var maxSize = angular.isDefined(attrs.maxSize) ? scope.$parent.$eval(attrs.maxSize) : paginationConfig.maxSize, rotate = angular.isDefined(attrs.rotate) ? scope.$parent.$eval(attrs.rotate) : paginationConfig.rotate;
        scope.boundaryLinks = angular.isDefined(attrs.boundaryLinks) ? scope.$parent.$eval(attrs.boundaryLinks) : paginationConfig.boundaryLinks;
        scope.directionLinks = angular.isDefined(attrs.directionLinks) ? scope.$parent.$eval(attrs.directionLinks) : paginationConfig.directionLinks;
        paginationCtrl.init(ngModelCtrl, paginationConfig);
        if (attrs.maxSize) {
          scope.$parent.$watch($parse(attrs.maxSize), function (value) {
            maxSize = parseInt(value, 10);
            paginationCtrl.render();
          });
        }
        // Create page object used in template
        function makePage(number, text, isActive) {
          return {
            number: number,
            text: text,
            active: isActive
          };
        }
        function getPages(currentPage, totalPages) {
          var pages = [];
          // Default page limits
          var startPage = 1, endPage = totalPages;
          var isMaxSized = angular.isDefined(maxSize) && maxSize < totalPages;
          // recompute if maxSize
          if (isMaxSized) {
            if (rotate) {
              // Current page is displayed in the middle of the visible ones
              startPage = Math.max(currentPage - Math.floor(maxSize / 2), 1);
              endPage = startPage + maxSize - 1;
              // Adjust if limit is exceeded
              if (endPage > totalPages) {
                endPage = totalPages;
                startPage = endPage - maxSize + 1;
              }
            } else {
              // Visible pages are paginated with maxSize
              startPage = (Math.ceil(currentPage / maxSize) - 1) * maxSize + 1;
              // Adjust last page if limit is exceeded
              endPage = Math.min(startPage + maxSize - 1, totalPages);
            }
          }
          // Add page number links
          for (var number = startPage; number <= endPage; number++) {
            var page = makePage(number, number, number === currentPage);
            pages.push(page);
          }
          // Add links to move between page sets
          if (isMaxSized && !rotate) {
            if (startPage > 1) {
              var previousPageSet = makePage(startPage - 1, '...', false);
              pages.unshift(previousPageSet);
            }
            if (endPage < totalPages) {
              var nextPageSet = makePage(endPage + 1, '...', false);
              pages.push(nextPageSet);
            }
          }
          return pages;
        }
        var originalRender = paginationCtrl.render;
        paginationCtrl.render = function () {
          originalRender();
          if (scope.page > 0 && scope.page <= scope.totalPages) {
            scope.pages = getPages(scope.page, scope.totalPages);
          }
        };
      }
    };
  }
]).constant('uibPagerConfig', {
  itemsPerPage: 10,
  previousText: '\xab Previous',
  nextText: 'Next \xbb',
  align: true
}).directive('uibPager', [
  'uibPagerConfig',
  function (pagerConfig) {
    return {
      restrict: 'EA',
      scope: {
        totalItems: '=',
        previousText: '@',
        nextText: '@',
        ngDisabled: '='
      },
      require: [
        'uibPager',
        '?ngModel'
      ],
      controller: 'UibPaginationController',
      controllerAs: 'pagination',
      templateUrl: function (element, attrs) {
        return attrs.templateUrl || 'template/pagination/pager.html';
      },
      replace: true,
      link: function (scope, element, attrs, ctrls) {
        var paginationCtrl = ctrls[0], ngModelCtrl = ctrls[1];
        if (!ngModelCtrl) {
          return;  // do nothing if no ng-model
        }
        scope.align = angular.isDefined(attrs.align) ? scope.$parent.$eval(attrs.align) : pagerConfig.align;
        paginationCtrl.init(ngModelCtrl, pagerConfig);
      }
    };
  }
]);
/* Deprecated Pagination Below */
angular.module('ui.bootstrap.pagination').value('$paginationSuppressWarning', false).controller('PaginationController', [
  '$scope',
  '$attrs',
  '$parse',
  '$controller',
  '$element',
  '$log',
  '$paginationSuppressWarning',
  function ($scope, $attrs, $parse, $controller, $element, $log, $paginationSuppressWarning) {
    if (!$paginationSuppressWarning) {
      $log.warn('PaginationController is now deprecated. Use UibPaginationController instead.');
    }
    return $controller('UibPaginationController', {
      $scope: $scope,
      $element: $element,
      $attrs: $attrs
    });
  }
]).directive('pagination', [
  '$parse',
  'uibPaginationConfig',
  '$log',
  '$paginationSuppressWarning',
  function ($parse, paginationConfig, $log, $paginationSuppressWarning) {
    return {
      restrict: 'EA',
      scope: {
        totalItems: '=',
        firstText: '@',
        previousText: '@',
        nextText: '@',
        lastText: '@',
        ngDisabled: '='
      },
      require: [
        'pagination',
        '?ngModel'
      ],
      controller: 'PaginationController',
      controllerAs: 'pagination',
      templateUrl: function (element, attrs) {
        return attrs.templateUrl || 'template/pagination/pagination.html';
      },
      replace: true,
      link: function (scope, element, attrs, ctrls) {
        if (!$paginationSuppressWarning) {
          $log.warn('pagination is now deprecated. Use uib-pagination instead.');
        }
        var paginationCtrl = ctrls[0], ngModelCtrl = ctrls[1];
        if (!ngModelCtrl) {
          return;  // do nothing if no ng-model
        }
        // Setup configuration parameters
        var maxSize = angular.isDefined(attrs.maxSize) ? scope.$parent.$eval(attrs.maxSize) : paginationConfig.maxSize, rotate = angular.isDefined(attrs.rotate) ? scope.$parent.$eval(attrs.rotate) : paginationConfig.rotate;
        scope.boundaryLinks = angular.isDefined(attrs.boundaryLinks) ? scope.$parent.$eval(attrs.boundaryLinks) : paginationConfig.boundaryLinks;
        scope.directionLinks = angular.isDefined(attrs.directionLinks) ? scope.$parent.$eval(attrs.directionLinks) : paginationConfig.directionLinks;
        paginationCtrl.init(ngModelCtrl, paginationConfig);
        if (attrs.maxSize) {
          scope.$parent.$watch($parse(attrs.maxSize), function (value) {
            maxSize = parseInt(value, 10);
            paginationCtrl.render();
          });
        }
        // Create page object used in template
        function makePage(number, text, isActive) {
          return {
            number: number,
            text: text,
            active: isActive
          };
        }
        function getPages(currentPage, totalPages) {
          var pages = [];
          // Default page limits
          var startPage = 1, endPage = totalPages;
          var isMaxSized = angular.isDefined(maxSize) && maxSize < totalPages;
          // recompute if maxSize
          if (isMaxSized) {
            if (rotate) {
              // Current page is displayed in the middle of the visible ones
              startPage = Math.max(currentPage - Math.floor(maxSize / 2), 1);
              endPage = startPage + maxSize - 1;
              // Adjust if limit is exceeded
              if (endPage > totalPages) {
                endPage = totalPages;
                startPage = endPage - maxSize + 1;
              }
            } else {
              // Visible pages are paginated with maxSize
              startPage = (Math.ceil(currentPage / maxSize) - 1) * maxSize + 1;
              // Adjust last page if limit is exceeded
              endPage = Math.min(startPage + maxSize - 1, totalPages);
            }
          }
          // Add page number links
          for (var number = startPage; number <= endPage; number++) {
            var page = makePage(number, number, number === currentPage);
            pages.push(page);
          }
          // Add links to move between page sets
          if (isMaxSized && !rotate) {
            if (startPage > 1) {
              var previousPageSet = makePage(startPage - 1, '...', false);
              pages.unshift(previousPageSet);
            }
            if (endPage < totalPages) {
              var nextPageSet = makePage(endPage + 1, '...', false);
              pages.push(nextPageSet);
            }
          }
          return pages;
        }
        var originalRender = paginationCtrl.render;
        paginationCtrl.render = function () {
          originalRender();
          if (scope.page > 0 && scope.page <= scope.totalPages) {
            scope.pages = getPages(scope.page, scope.totalPages);
          }
        };
      }
    };
  }
]).directive('pager', [
  'uibPagerConfig',
  '$log',
  '$paginationSuppressWarning',
  function (pagerConfig, $log, $paginationSuppressWarning) {
    return {
      restrict: 'EA',
      scope: {
        totalItems: '=',
        previousText: '@',
        nextText: '@',
        ngDisabled: '='
      },
      require: [
        'pager',
        '?ngModel'
      ],
      controller: 'PaginationController',
      controllerAs: 'pagination',
      templateUrl: function (element, attrs) {
        return attrs.templateUrl || 'template/pagination/pager.html';
      },
      replace: true,
      link: function (scope, element, attrs, ctrls) {
        if (!$paginationSuppressWarning) {
          $log.warn('pager is now deprecated. Use uib-pager instead.');
        }
        var paginationCtrl = ctrls[0], ngModelCtrl = ctrls[1];
        if (!ngModelCtrl) {
          return;  // do nothing if no ng-model
        }
        scope.align = angular.isDefined(attrs.align) ? scope.$parent.$eval(attrs.align) : pagerConfig.align;
        paginationCtrl.init(ngModelCtrl, pagerConfig);
      }
    };
  }
]);
/**
 * The following features are still outstanding: animation as a
 * function, placement as a function, inside, support for more triggers than
 * just mouse enter/leave, html tooltips, and selector delegation.
 */
angular.module('ui.bootstrap.tooltip', [
  'ui.bootstrap.position',
  'ui.bootstrap.stackedMap'
]).provider('$uibTooltip', function () {
  // The default options tooltip and popover.
  var defaultOptions = {
      placement: 'top',
      animation: true,
      popupDelay: 0,
      popupCloseDelay: 500,
      useContentExp: false
    };
  // Default hide triggers for each show trigger
  var triggerMap = {
      'mouseenter': 'mouseleave',
      'click': 'click',
      'focus': 'blur',
      'none': ''
    };
  // The options specified to the provider globally.
  var globalOptions = {};
  /**
   * `options({})` allows global configuration of all tooltips in the
   * application.
   *
   *   var app = angular.module( 'App', ['ui.bootstrap.tooltip'], function( $tooltipProvider ) {
   *     // place tooltips left instead of top by default
   *     $tooltipProvider.options( { placement: 'left' } );
   *   });
   */
  this.options = function (value) {
    angular.extend(globalOptions, value);
  };
  /**
   * This allows you to extend the set of trigger mappings available. E.g.:
   *
   *   $tooltipProvider.setTriggers( 'openTrigger': 'closeTrigger' );
   */
  this.setTriggers = function setTriggers(triggers) {
    angular.extend(triggerMap, triggers);
  };
  /**
   * This is a helper function for translating camel-case to snake-case.
   */
  function snake_case(name) {
    var regexp = /[A-Z]/g;
    var separator = '-';
    return name.replace(regexp, function (letter, pos) {
      return (pos ? separator : '') + letter.toLowerCase();
    });
  }
  /**
   * Returns the actual instance of the $tooltip service.
   * TODO support multiple triggers
   */
  this.$get = [
    '$window',
    '$compile',
    '$timeout',
    '$document',
    '$uibPosition',
    '$interpolate',
    '$rootScope',
    '$parse',
    '$$stackedMap',
    function ($window, $compile, $timeout, $document, $position, $interpolate, $rootScope, $parse, $$stackedMap) {
      var openedTooltips = $$stackedMap.createNew();
      $document.on('keypress', function (e) {
        if (e.which === 27) {
          var last = openedTooltips.top();
          if (last) {
            last.value.close();
            openedTooltips.removeTop();
            last = null;
          }
        }
      });
      return function $tooltip(ttType, prefix, defaultTriggerShow, options) {
        options = angular.extend({}, defaultOptions, globalOptions, options);
        /**
       * Returns an object of show and hide triggers.
       *
       * If a trigger is supplied,
       * it is used to show the tooltip; otherwise, it will use the `trigger`
       * option passed to the `$tooltipProvider.options` method; else it will
       * default to the trigger supplied to this directive factory.
       *
       * The hide trigger is based on the show trigger. If the `trigger` option
       * was passed to the `$tooltipProvider.options` method, it will use the
       * mapped trigger from `triggerMap` or the passed trigger if the map is
       * undefined; otherwise, it uses the `triggerMap` value of the show
       * trigger; else it will just use the show trigger.
       */
        function getTriggers(trigger) {
          var show = (trigger || options.trigger || defaultTriggerShow).split(' ');
          var hide = show.map(function (trigger) {
              return triggerMap[trigger] || trigger;
            });
          return {
            show: show,
            hide: hide
          };
        }
        var directiveName = snake_case(ttType);
        var startSym = $interpolate.startSymbol();
        var endSym = $interpolate.endSymbol();
        var template = '<div ' + directiveName + '-popup ' + 'title="' + startSym + 'title' + endSym + '" ' + (options.useContentExp ? 'content-exp="contentExp()" ' : 'content="' + startSym + 'content' + endSym + '" ') + 'placement="' + startSym + 'placement' + endSym + '" ' + 'popup-class="' + startSym + 'popupClass' + endSym + '" ' + 'animation="animation" ' + 'is-open="isOpen"' + 'origin-scope="origScope" ' + 'style="visibility: hidden; display: block;"' + '>' + '</div>';
        return {
          compile: function (tElem, tAttrs) {
            var tooltipLinker = $compile(template);
            return function link(scope, element, attrs, tooltipCtrl) {
              var tooltip;
              var tooltipLinkedScope;
              var transitionTimeout;
              var popupTimeout;
              var positionTimeout;
              var appendToBody = angular.isDefined(options.appendToBody) ? options.appendToBody : false;
              var triggers = getTriggers(undefined);
              var hasEnableExp = angular.isDefined(attrs[prefix + 'Enable']);
              var ttScope = scope.$new(true);
              var repositionScheduled = false;
              var isOpenParse = angular.isDefined(attrs[prefix + 'IsOpen']) ? $parse(attrs[prefix + 'IsOpen']) : false;
              var contentParse = options.useContentExp ? $parse(attrs[ttType]) : false;
              var observers = [];
              var positionTooltip = function () {
                // check if tooltip exists and is not empty
                if (!tooltip || !tooltip.html()) {
                  return;
                }
                if (!positionTimeout) {
                  positionTimeout = $timeout(function () {
                    // Reset the positioning.
                    tooltip.css({
                      top: 0,
                      left: 0
                    });
                    // Now set the calculated positioning.
                    var ttCss = $position.positionElements(element, tooltip, ttScope.placement, appendToBody);
                    ttCss.top += 'px';
                    ttCss.left += 'px';
                    ttCss.visibility = 'visible';
                    tooltip.css(ttCss);
                    positionTimeout = null;
                  }, 0, false);
                }
              };
              // Set up the correct scope to allow transclusion later
              ttScope.origScope = scope;
              // By default, the tooltip is not open.
              // TODO add ability to start tooltip opened
              ttScope.isOpen = false;
              openedTooltips.add(ttScope, { close: hide });
              function toggleTooltipBind() {
                if (!ttScope.isOpen) {
                  showTooltipBind();
                } else {
                  hideTooltipBind();
                }
              }
              // Show the tooltip with delay if specified, otherwise show it immediately
              function showTooltipBind() {
                if (hasEnableExp && !scope.$eval(attrs[prefix + 'Enable'])) {
                  return;
                }
                prepareTooltip();
                if (ttScope.popupDelay) {
                  // Do nothing if the tooltip was already scheduled to pop-up.
                  // This happens if show is triggered multiple times before any hide is triggered.
                  if (!popupTimeout) {
                    popupTimeout = $timeout(show, ttScope.popupDelay, false);
                  }
                } else {
                  show();
                }
              }
              function hideTooltipBind() {
                hide();
              }
              // Show the tooltip popup element.
              function show() {
                if (popupTimeout) {
                  $timeout.cancel(popupTimeout);
                  popupTimeout = null;
                }
                // If there is a pending remove transition, we must cancel it, lest the
                // tooltip be mysteriously removed.
                if (transitionTimeout) {
                  $timeout.cancel(transitionTimeout);
                  transitionTimeout = null;
                }
                // Don't show empty tooltips.
                if (!ttScope.content) {
                  return angular.noop;
                }
                createTooltip();
                // And show the tooltip.
                ttScope.$evalAsync(function () {
                  ttScope.isOpen = true;
                  if (isOpenParse && angular.isFunction(isOpenParse.assign)) {
                    isOpenParse.assign(ttScope.origScope, ttScope.isOpen);
                  }
                  positionTooltip();
                });
              }
              // Hide the tooltip popup element.
              function hide() {
                if (!ttScope) {
                  return;
                }
                // First things first: we don't show it anymore.
                ttScope.$evalAsync(function () {
                  ttScope.isOpen = false;
                  if (isOpenParse && angular.isFunction(isOpenParse.assign)) {
                    isOpenParse.assign(ttScope.origScope, ttScope.isOpen);
                  }
                });
                //if tooltip is going to be shown after delay, we must cancel this
                $timeout.cancel(popupTimeout);
                popupTimeout = null;
                $timeout.cancel(positionTimeout);
                positionTimeout = null;
                // And now we remove it from the DOM. However, if we have animation, we
                // need to wait for it to expire beforehand.
                // FIXME: this is a placeholder for a port of the transitions library.
                if (ttScope.animation) {
                  if (!transitionTimeout) {
                    transitionTimeout = $timeout(removeTooltip, ttScope.popupCloseDelay);
                  }
                } else {
                  removeTooltip();
                }
              }
              function createTooltip() {
                // There can only be one tooltip element per directive shown at once.
                if (tooltip) {
                  return;
                }
                tooltipLinkedScope = ttScope.$new();
                tooltip = tooltipLinker(tooltipLinkedScope, function (tooltip) {
                  if (appendToBody) {
                    $document.find('body').append(tooltip);
                  } else {
                    element.after(tooltip);
                  }
                });
                prepObservers();
              }
              function removeTooltip() {
                unregisterObservers();
                transitionTimeout = null;
                if (tooltip) {
                  tooltip.remove();
                  tooltip = null;
                }
                if (tooltipLinkedScope) {
                  tooltipLinkedScope.$destroy();
                  tooltipLinkedScope = null;
                }
              }
              /**
             * Set the inital scope values. Once
             * the tooltip is created, the observers
             * will be added to keep things in synch.
             */
              function prepareTooltip() {
                ttScope.title = attrs[prefix + 'Title'];
                if (contentParse) {
                  ttScope.content = contentParse(scope);
                } else {
                  ttScope.content = attrs[ttType];
                }
                ttScope.popupClass = attrs[prefix + 'Class'];
                ttScope.placement = angular.isDefined(attrs[prefix + 'Placement']) ? attrs[prefix + 'Placement'] : options.placement;
                var delay = parseInt(attrs[prefix + 'PopupDelay'], 10);
                var closeDelay = parseInt(attrs[prefix + 'PopupCloseDelay'], 10);
                ttScope.popupDelay = !isNaN(delay) ? delay : options.popupDelay;
                ttScope.popupCloseDelay = !isNaN(closeDelay) ? closeDelay : options.popupCloseDelay;
              }
              ttScope.contentExp = function () {
                return ttScope.content;
              };
              /**
             * Observe the relevant attributes.
             */
              attrs.$observe('disabled', function (val) {
                if (popupTimeout && val) {
                  $timeout.cancel(popupTimeout);
                  popupTimeout = null;
                }
                if (val && ttScope.isOpen) {
                  hide();
                }
              });
              if (isOpenParse) {
                scope.$watch(isOpenParse, function (val) {
                  /*jshint -W018 */
                  if (!val === ttScope.isOpen) {
                    toggleTooltipBind();
                  }  /*jshint +W018 */
                });
              }
              function prepObservers() {
                observers.length = 0;
                if (contentParse) {
                  observers.push(scope.$watch(contentParse, function (val) {
                    ttScope.content = val;
                    if (!val && ttScope.isOpen) {
                      hide();
                    }
                  }));
                  observers.push(tooltipLinkedScope.$watch(function () {
                    if (!repositionScheduled) {
                      repositionScheduled = true;
                      tooltipLinkedScope.$$postDigest(function () {
                        repositionScheduled = false;
                        if (ttScope && ttScope.isOpen) {
                          positionTooltip();
                        }
                      });
                    }
                  }));
                } else {
                  observers.push(attrs.$observe(ttType, function (val) {
                    ttScope.content = val;
                    if (!val && ttScope.isOpen) {
                      hide();
                    } else {
                      positionTooltip();
                    }
                  }));
                }
                observers.push(attrs.$observe(prefix + 'Title', function (val) {
                  ttScope.title = val;
                  if (ttScope.isOpen) {
                    positionTooltip();
                  }
                }));
                observers.push(attrs.$observe(prefix + 'Placement', function (val) {
                  ttScope.placement = val ? val : options.placement;
                  if (ttScope.isOpen) {
                    positionTooltip();
                  }
                }));
              }
              function unregisterObservers() {
                if (observers.length) {
                  angular.forEach(observers, function (observer) {
                    observer();
                  });
                  observers.length = 0;
                }
              }
              var unregisterTriggers = function () {
                triggers.show.forEach(function (trigger) {
                  element.unbind(trigger, showTooltipBind);
                });
                triggers.hide.forEach(function (trigger) {
                  trigger.split(' ').forEach(function (hideTrigger) {
                    element[0].removeEventListener(hideTrigger, hideTooltipBind);
                  });
                });
              };
              function prepTriggers() {
                var val = attrs[prefix + 'Trigger'];
                unregisterTriggers();
                triggers = getTriggers(val);
                if (triggers.show !== 'none') {
                  triggers.show.forEach(function (trigger, idx) {
                    // Using raw addEventListener due to jqLite/jQuery bug - #4060
                    if (trigger === triggers.hide[idx]) {
                      element[0].addEventListener(trigger, toggleTooltipBind);
                    } else if (trigger) {
                      element[0].addEventListener(trigger, showTooltipBind);
                      triggers.hide[idx].split(' ').forEach(function (trigger) {
                        element[0].addEventListener(trigger, hideTooltipBind);
                      });
                    }
                    element.on('keypress', function (e) {
                      if (e.which === 27) {
                        hideTooltipBind();
                      }
                    });
                  });
                }
              }
              prepTriggers();
              var animation = scope.$eval(attrs[prefix + 'Animation']);
              ttScope.animation = angular.isDefined(animation) ? !!animation : options.animation;
              var appendToBodyVal = scope.$eval(attrs[prefix + 'AppendToBody']);
              appendToBody = angular.isDefined(appendToBodyVal) ? appendToBodyVal : appendToBody;
              // if a tooltip is attached to <body> we need to remove it on
              // location change as its parent scope will probably not be destroyed
              // by the change.
              if (appendToBody) {
                scope.$on('$locationChangeSuccess', function closeTooltipOnLocationChangeSuccess() {
                  if (ttScope.isOpen) {
                    hide();
                  }
                });
              }
              // Make sure tooltip is destroyed and removed.
              scope.$on('$destroy', function onDestroyTooltip() {
                $timeout.cancel(transitionTimeout);
                $timeout.cancel(popupTimeout);
                $timeout.cancel(positionTimeout);
                unregisterTriggers();
                removeTooltip();
                ttScope = null;
              });
            };
          }
        };
      };
    }
  ];
}).directive('uibTooltipTemplateTransclude', [
  '$animate',
  '$sce',
  '$compile',
  '$templateRequest',
  function ($animate, $sce, $compile, $templateRequest) {
    return {
      link: function (scope, elem, attrs) {
        var origScope = scope.$eval(attrs.tooltipTemplateTranscludeScope);
        var changeCounter = 0, currentScope, previousElement, currentElement;
        var cleanupLastIncludeContent = function () {
          if (previousElement) {
            previousElement.remove();
            previousElement = null;
          }
          if (currentScope) {
            currentScope.$destroy();
            currentScope = null;
          }
          if (currentElement) {
            $animate.leave(currentElement).then(function () {
              previousElement = null;
            });
            previousElement = currentElement;
            currentElement = null;
          }
        };
        scope.$watch($sce.parseAsResourceUrl(attrs.uibTooltipTemplateTransclude), function (src) {
          var thisChangeId = ++changeCounter;
          if (src) {
            //set the 2nd param to true to ignore the template request error so that the inner
            //contents and scope can be cleaned up.
            $templateRequest(src, true).then(function (response) {
              if (thisChangeId !== changeCounter) {
                return;
              }
              var newScope = origScope.$new();
              var template = response;
              var clone = $compile(template)(newScope, function (clone) {
                  cleanupLastIncludeContent();
                  $animate.enter(clone, elem);
                });
              currentScope = newScope;
              currentElement = clone;
              currentScope.$emit('$includeContentLoaded', src);
            }, function () {
              if (thisChangeId === changeCounter) {
                cleanupLastIncludeContent();
                scope.$emit('$includeContentError', src);
              }
            });
            scope.$emit('$includeContentRequested', src);
          } else {
            cleanupLastIncludeContent();
          }
        });
        scope.$on('$destroy', cleanupLastIncludeContent);
      }
    };
  }
]).directive('uibTooltipClasses', function () {
  return {
    restrict: 'A',
    link: function (scope, element, attrs) {
      if (scope.placement) {
        element.addClass(scope.placement);
      }
      if (scope.popupClass) {
        element.addClass(scope.popupClass);
      }
      if (scope.animation()) {
        element.addClass(attrs.tooltipAnimationClass);
      }
    }
  };
}).directive('uibTooltipPopup', function () {
  return {
    replace: true,
    scope: {
      content: '@',
      placement: '@',
      popupClass: '@',
      animation: '&',
      isOpen: '&'
    },
    templateUrl: 'template/tooltip/tooltip-popup.html',
    link: function (scope, element) {
      element.addClass('tooltip');
    }
  };
}).directive('uibTooltip', [
  '$uibTooltip',
  function ($uibTooltip) {
    return $uibTooltip('uibTooltip', 'tooltip', 'mouseenter');
  }
]).directive('uibTooltipTemplatePopup', function () {
  return {
    replace: true,
    scope: {
      contentExp: '&',
      placement: '@',
      popupClass: '@',
      animation: '&',
      isOpen: '&',
      originScope: '&'
    },
    templateUrl: 'template/tooltip/tooltip-template-popup.html',
    link: function (scope, element) {
      element.addClass('tooltip');
    }
  };
}).directive('uibTooltipTemplate', [
  '$uibTooltip',
  function ($uibTooltip) {
    return $uibTooltip('uibTooltipTemplate', 'tooltip', 'mouseenter', { useContentExp: true });
  }
]).directive('uibTooltipHtmlPopup', function () {
  return {
    replace: true,
    scope: {
      contentExp: '&',
      placement: '@',
      popupClass: '@',
      animation: '&',
      isOpen: '&'
    },
    templateUrl: 'template/tooltip/tooltip-html-popup.html',
    link: function (scope, element) {
      element.addClass('tooltip');
    }
  };
}).directive('uibTooltipHtml', [
  '$uibTooltip',
  function ($uibTooltip) {
    return $uibTooltip('uibTooltipHtml', 'tooltip', 'mouseenter', { useContentExp: true });
  }
]);
/* Deprecated tooltip below */
angular.module('ui.bootstrap.tooltip').value('$tooltipSuppressWarning', false).provider('$tooltip', [
  '$uibTooltipProvider',
  function ($uibTooltipProvider) {
    angular.extend(this, $uibTooltipProvider);
    this.$get = [
      '$log',
      '$tooltipSuppressWarning',
      '$injector',
      function ($log, $tooltipSuppressWarning, $injector) {
        if (!$tooltipSuppressWarning) {
          $log.warn('$tooltip is now deprecated. Use $uibTooltip instead.');
        }
        return $injector.invoke($uibTooltipProvider.$get);
      }
    ];
  }
]).directive('tooltipTemplateTransclude', [
  '$animate',
  '$sce',
  '$compile',
  '$templateRequest',
  '$log',
  '$tooltipSuppressWarning',
  function ($animate, $sce, $compile, $templateRequest, $log, $tooltipSuppressWarning) {
    return {
      link: function (scope, elem, attrs) {
        if (!$tooltipSuppressWarning) {
          $log.warn('tooltip-template-transclude is now deprecated. Use uib-tooltip-template-transclude instead.');
        }
        var origScope = scope.$eval(attrs.tooltipTemplateTranscludeScope);
        var changeCounter = 0, currentScope, previousElement, currentElement;
        var cleanupLastIncludeContent = function () {
          if (previousElement) {
            previousElement.remove();
            previousElement = null;
          }
          if (currentScope) {
            currentScope.$destroy();
            currentScope = null;
          }
          if (currentElement) {
            $animate.leave(currentElement).then(function () {
              previousElement = null;
            });
            previousElement = currentElement;
            currentElement = null;
          }
        };
        scope.$watch($sce.parseAsResourceUrl(attrs.tooltipTemplateTransclude), function (src) {
          var thisChangeId = ++changeCounter;
          if (src) {
            //set the 2nd param to true to ignore the template request error so that the inner
            //contents and scope can be cleaned up.
            $templateRequest(src, true).then(function (response) {
              if (thisChangeId !== changeCounter) {
                return;
              }
              var newScope = origScope.$new();
              var template = response;
              var clone = $compile(template)(newScope, function (clone) {
                  cleanupLastIncludeContent();
                  $animate.enter(clone, elem);
                });
              currentScope = newScope;
              currentElement = clone;
              currentScope.$emit('$includeContentLoaded', src);
            }, function () {
              if (thisChangeId === changeCounter) {
                cleanupLastIncludeContent();
                scope.$emit('$includeContentError', src);
              }
            });
            scope.$emit('$includeContentRequested', src);
          } else {
            cleanupLastIncludeContent();
          }
        });
        scope.$on('$destroy', cleanupLastIncludeContent);
      }
    };
  }
]).directive('tooltipClasses', [
  '$log',
  '$tooltipSuppressWarning',
  function ($log, $tooltipSuppressWarning) {
    return {
      restrict: 'A',
      link: function (scope, element, attrs) {
        if (!$tooltipSuppressWarning) {
          $log.warn('tooltip-classes is now deprecated. Use uib-tooltip-classes instead.');
        }
        if (scope.placement) {
          element.addClass(scope.placement);
        }
        if (scope.popupClass) {
          element.addClass(scope.popupClass);
        }
        if (scope.animation()) {
          element.addClass(attrs.tooltipAnimationClass);
        }
      }
    };
  }
]).directive('tooltipPopup', [
  '$log',
  '$tooltipSuppressWarning',
  function ($log, $tooltipSuppressWarning) {
    return {
      replace: true,
      scope: {
        content: '@',
        placement: '@',
        popupClass: '@',
        animation: '&',
        isOpen: '&'
      },
      templateUrl: 'template/tooltip/tooltip-popup.html',
      link: function (scope, element) {
        if (!$tooltipSuppressWarning) {
          $log.warn('tooltip-popup is now deprecated. Use uib-tooltip-popup instead.');
        }
        element.addClass('tooltip');
      }
    };
  }
]).directive('tooltip', [
  '$tooltip',
  function ($tooltip) {
    return $tooltip('tooltip', 'tooltip', 'mouseenter');
  }
]).directive('tooltipTemplatePopup', [
  '$log',
  '$tooltipSuppressWarning',
  function ($log, $tooltipSuppressWarning) {
    return {
      replace: true,
      scope: {
        contentExp: '&',
        placement: '@',
        popupClass: '@',
        animation: '&',
        isOpen: '&',
        originScope: '&'
      },
      templateUrl: 'template/tooltip/tooltip-template-popup.html',
      link: function (scope, element) {
        if (!$tooltipSuppressWarning) {
          $log.warn('tooltip-template-popup is now deprecated. Use uib-tooltip-template-popup instead.');
        }
        element.addClass('tooltip');
      }
    };
  }
]).directive('tooltipTemplate', [
  '$tooltip',
  function ($tooltip) {
    return $tooltip('tooltipTemplate', 'tooltip', 'mouseenter', { useContentExp: true });
  }
]).directive('tooltipHtmlPopup', [
  '$log',
  '$tooltipSuppressWarning',
  function ($log, $tooltipSuppressWarning) {
    return {
      replace: true,
      scope: {
        contentExp: '&',
        placement: '@',
        popupClass: '@',
        animation: '&',
        isOpen: '&'
      },
      templateUrl: 'template/tooltip/tooltip-html-popup.html',
      link: function (scope, element) {
        if (!$tooltipSuppressWarning) {
          $log.warn('tooltip-html-popup is now deprecated. Use uib-tooltip-html-popup instead.');
        }
        element.addClass('tooltip');
      }
    };
  }
]).directive('tooltipHtml', [
  '$tooltip',
  function ($tooltip) {
    return $tooltip('tooltipHtml', 'tooltip', 'mouseenter', { useContentExp: true });
  }
]);
/**
 * The following features are still outstanding: popup delay, animation as a
 * function, placement as a function, inside, support for more triggers than
 * just mouse enter/leave, and selector delegatation.
 */
angular.module('ui.bootstrap.popover', ['ui.bootstrap.tooltip']).directive('uibPopoverTemplatePopup', function () {
  return {
    replace: true,
    scope: {
      title: '@',
      contentExp: '&',
      placement: '@',
      popupClass: '@',
      animation: '&',
      isOpen: '&',
      originScope: '&'
    },
    templateUrl: 'template/popover/popover-template.html',
    link: function (scope, element) {
      element.addClass('popover');
    }
  };
}).directive('uibPopoverTemplate', [
  '$uibTooltip',
  function ($uibTooltip) {
    return $uibTooltip('uibPopoverTemplate', 'popover', 'click', { useContentExp: true });
  }
]).directive('uibPopoverHtmlPopup', function () {
  return {
    replace: true,
    scope: {
      contentExp: '&',
      title: '@',
      placement: '@',
      popupClass: '@',
      animation: '&',
      isOpen: '&'
    },
    templateUrl: 'template/popover/popover-html.html',
    link: function (scope, element) {
      element.addClass('popover');
    }
  };
}).directive('uibPopoverHtml', [
  '$uibTooltip',
  function ($uibTooltip) {
    return $uibTooltip('uibPopoverHtml', 'popover', 'click', { useContentExp: true });
  }
]).directive('uibPopoverPopup', function () {
  return {
    replace: true,
    scope: {
      title: '@',
      content: '@',
      placement: '@',
      popupClass: '@',
      animation: '&',
      isOpen: '&'
    },
    templateUrl: 'template/popover/popover.html',
    link: function (scope, element) {
      element.addClass('popover');
    }
  };
}).directive('uibPopover', [
  '$uibTooltip',
  function ($uibTooltip) {
    return $uibTooltip('uibPopover', 'popover', 'click');
  }
]);
/* Deprecated popover below */
angular.module('ui.bootstrap.popover').value('$popoverSuppressWarning', false).directive('popoverTemplatePopup', [
  '$log',
  '$popoverSuppressWarning',
  function ($log, $popoverSuppressWarning) {
    return {
      replace: true,
      scope: {
        title: '@',
        contentExp: '&',
        placement: '@',
        popupClass: '@',
        animation: '&',
        isOpen: '&',
        originScope: '&'
      },
      templateUrl: 'template/popover/popover-template.html',
      link: function (scope, element) {
        if (!$popoverSuppressWarning) {
          $log.warn('popover-template-popup is now deprecated. Use uib-popover-template-popup instead.');
        }
        element.addClass('popover');
      }
    };
  }
]).directive('popoverTemplate', [
  '$tooltip',
  function ($tooltip) {
    return $tooltip('popoverTemplate', 'popover', 'click', { useContentExp: true });
  }
]).directive('popoverHtmlPopup', [
  '$log',
  '$popoverSuppressWarning',
  function ($log, $popoverSuppressWarning) {
    return {
      replace: true,
      scope: {
        contentExp: '&',
        title: '@',
        placement: '@',
        popupClass: '@',
        animation: '&',
        isOpen: '&'
      },
      templateUrl: 'template/popover/popover-html.html',
      link: function (scope, element) {
        if (!$popoverSuppressWarning) {
          $log.warn('popover-html-popup is now deprecated. Use uib-popover-html-popup instead.');
        }
        element.addClass('popover');
      }
    };
  }
]).directive('popoverHtml', [
  '$tooltip',
  function ($tooltip) {
    return $tooltip('popoverHtml', 'popover', 'click', { useContentExp: true });
  }
]).directive('popoverPopup', [
  '$log',
  '$popoverSuppressWarning',
  function ($log, $popoverSuppressWarning) {
    return {
      replace: true,
      scope: {
        title: '@',
        content: '@',
        placement: '@',
        popupClass: '@',
        animation: '&',
        isOpen: '&'
      },
      templateUrl: 'template/popover/popover.html',
      link: function (scope, element) {
        if (!$popoverSuppressWarning) {
          $log.warn('popover-popup is now deprecated. Use uib-popover-popup instead.');
        }
        element.addClass('popover');
      }
    };
  }
]).directive('popover', [
  '$tooltip',
  function ($tooltip) {
    return $tooltip('popover', 'popover', 'click');
  }
]);
angular.module('ui.bootstrap.progressbar', []).constant('uibProgressConfig', {
  animate: true,
  max: 100
}).controller('UibProgressController', [
  '$scope',
  '$attrs',
  'uibProgressConfig',
  function ($scope, $attrs, progressConfig) {
    var self = this, animate = angular.isDefined($attrs.animate) ? $scope.$parent.$eval($attrs.animate) : progressConfig.animate;
    this.bars = [];
    $scope.max = angular.isDefined($scope.max) ? $scope.max : progressConfig.max;
    this.addBar = function (bar, element, attrs) {
      if (!animate) {
        element.css({ 'transition': 'none' });
      }
      this.bars.push(bar);
      bar.max = $scope.max;
      bar.title = attrs && angular.isDefined(attrs.title) ? attrs.title : 'progressbar';
      bar.$watch('value', function (value) {
        bar.recalculatePercentage();
      });
      bar.recalculatePercentage = function () {
        bar.percent = +(100 * bar.value / bar.max).toFixed(2);
        var totalPercentage = self.bars.reduce(function (total, bar) {
            return total + bar.percent;
          }, 0);
        if (totalPercentage > 100) {
          bar.percent -= totalPercentage - 100;
        }
      };
      bar.$on('$destroy', function () {
        element = null;
        self.removeBar(bar);
      });
    };
    this.removeBar = function (bar) {
      this.bars.splice(this.bars.indexOf(bar), 1);
    };
    $scope.$watch('max', function (max) {
      self.bars.forEach(function (bar) {
        bar.max = $scope.max;
        bar.recalculatePercentage();
      });
    });
  }
]).directive('uibProgress', function () {
  return {
    replace: true,
    transclude: true,
    controller: 'UibProgressController',
    require: 'uibProgress',
    scope: { max: '=?' },
    templateUrl: 'template/progressbar/progress.html'
  };
}).directive('uibBar', function () {
  return {
    replace: true,
    transclude: true,
    require: '^uibProgress',
    scope: {
      value: '=',
      type: '@'
    },
    templateUrl: 'template/progressbar/bar.html',
    link: function (scope, element, attrs, progressCtrl) {
      progressCtrl.addBar(scope, element, attrs);
    }
  };
}).directive('uibProgressbar', function () {
  return {
    replace: true,
    transclude: true,
    controller: 'UibProgressController',
    scope: {
      value: '=',
      max: '=?',
      type: '@'
    },
    templateUrl: 'template/progressbar/progressbar.html',
    link: function (scope, element, attrs, progressCtrl) {
      progressCtrl.addBar(scope, angular.element(element.children()[0]), { title: attrs.title });
    }
  };
});
/* Deprecated progressbar below */
angular.module('ui.bootstrap.progressbar').value('$progressSuppressWarning', false).controller('ProgressController', [
  '$scope',
  '$attrs',
  '$controller',
  '$log',
  '$progressSuppressWarning',
  function ($scope, $attrs, $controller, $log, $progressSuppressWarning) {
    if (!$progressSuppressWarning) {
      $log.warn('ProgressController is now deprecated. Use UibProgressController instead.');
    }
    return $controller('UibProgressController', {
      $scope: $scope,
      $attrs: $attrs
    });
  }
]).directive('progress', [
  '$log',
  '$progressSuppressWarning',
  function ($log, $progressSuppressWarning) {
    return {
      replace: true,
      transclude: true,
      controller: 'ProgressController',
      require: 'progress',
      scope: {
        max: '=?',
        title: '@?'
      },
      templateUrl: 'template/progressbar/progress.html',
      link: function () {
        if (!$progressSuppressWarning) {
          $log.warn('progress is now deprecated. Use uib-progress instead.');
        }
      }
    };
  }
]).directive('bar', [
  '$log',
  '$progressSuppressWarning',
  function ($log, $progressSuppressWarning) {
    return {
      replace: true,
      transclude: true,
      require: '^progress',
      scope: {
        value: '=',
        type: '@'
      },
      templateUrl: 'template/progressbar/bar.html',
      link: function (scope, element, attrs, progressCtrl) {
        if (!$progressSuppressWarning) {
          $log.warn('bar is now deprecated. Use uib-bar instead.');
        }
        progressCtrl.addBar(scope, element);
      }
    };
  }
]).directive('progressbar', [
  '$log',
  '$progressSuppressWarning',
  function ($log, $progressSuppressWarning) {
    return {
      replace: true,
      transclude: true,
      controller: 'ProgressController',
      scope: {
        value: '=',
        max: '=?',
        type: '@'
      },
      templateUrl: 'template/progressbar/progressbar.html',
      link: function (scope, element, attrs, progressCtrl) {
        if (!$progressSuppressWarning) {
          $log.warn('progressbar is now deprecated. Use uib-progressbar instead.');
        }
        progressCtrl.addBar(scope, angular.element(element.children()[0]), { title: attrs.title });
      }
    };
  }
]);
angular.module('ui.bootstrap.rating', []).constant('uibRatingConfig', {
  max: 5,
  stateOn: null,
  stateOff: null,
  titles: [
    'one',
    'two',
    'three',
    'four',
    'five'
  ]
}).controller('UibRatingController', [
  '$scope',
  '$attrs',
  'uibRatingConfig',
  function ($scope, $attrs, ratingConfig) {
    var ngModelCtrl = { $setViewValue: angular.noop };
    this.init = function (ngModelCtrl_) {
      ngModelCtrl = ngModelCtrl_;
      ngModelCtrl.$render = this.render;
      ngModelCtrl.$formatters.push(function (value) {
        if (angular.isNumber(value) && value << 0 !== value) {
          value = Math.round(value);
        }
        return value;
      });
      this.stateOn = angular.isDefined($attrs.stateOn) ? $scope.$parent.$eval($attrs.stateOn) : ratingConfig.stateOn;
      this.stateOff = angular.isDefined($attrs.stateOff) ? $scope.$parent.$eval($attrs.stateOff) : ratingConfig.stateOff;
      var tmpTitles = angular.isDefined($attrs.titles) ? $scope.$parent.$eval($attrs.titles) : ratingConfig.titles;
      this.titles = angular.isArray(tmpTitles) && tmpTitles.length > 0 ? tmpTitles : ratingConfig.titles;
      var ratingStates = angular.isDefined($attrs.ratingStates) ? $scope.$parent.$eval($attrs.ratingStates) : new Array(angular.isDefined($attrs.max) ? $scope.$parent.$eval($attrs.max) : ratingConfig.max);
      $scope.range = this.buildTemplateObjects(ratingStates);
    };
    this.buildTemplateObjects = function (states) {
      for (var i = 0, n = states.length; i < n; i++) {
        states[i] = angular.extend({ index: i }, {
          stateOn: this.stateOn,
          stateOff: this.stateOff,
          title: this.getTitle(i)
        }, states[i]);
      }
      return states;
    };
    this.getTitle = function (index) {
      if (index >= this.titles.length) {
        return index + 1;
      } else {
        return this.titles[index];
      }
    };
    $scope.rate = function (value) {
      if (!$scope.readonly && value >= 0 && value <= $scope.range.length) {
        ngModelCtrl.$setViewValue(ngModelCtrl.$viewValue === value ? 0 : value);
        ngModelCtrl.$render();
      }
    };
    $scope.enter = function (value) {
      if (!$scope.readonly) {
        $scope.value = value;
      }
      $scope.onHover({ value: value });
    };
    $scope.reset = function () {
      $scope.value = ngModelCtrl.$viewValue;
      $scope.onLeave();
    };
    $scope.onKeydown = function (evt) {
      if (/(37|38|39|40)/.test(evt.which)) {
        evt.preventDefault();
        evt.stopPropagation();
        $scope.rate($scope.value + (evt.which === 38 || evt.which === 39 ? 1 : -1));
      }
    };
    this.render = function () {
      $scope.value = ngModelCtrl.$viewValue;
    };
  }
]).directive('uibRating', function () {
  return {
    require: [
      'uibRating',
      'ngModel'
    ],
    scope: {
      readonly: '=?',
      onHover: '&',
      onLeave: '&'
    },
    controller: 'UibRatingController',
    templateUrl: 'template/rating/rating.html',
    replace: true,
    link: function (scope, element, attrs, ctrls) {
      var ratingCtrl = ctrls[0], ngModelCtrl = ctrls[1];
      ratingCtrl.init(ngModelCtrl);
    }
  };
});
/* Deprecated rating below */
angular.module('ui.bootstrap.rating').value('$ratingSuppressWarning', false).controller('RatingController', [
  '$scope',
  '$attrs',
  '$controller',
  '$log',
  '$ratingSuppressWarning',
  function ($scope, $attrs, $controller, $log, $ratingSuppressWarning) {
    if (!$ratingSuppressWarning) {
      $log.warn('RatingController is now deprecated. Use UibRatingController instead.');
    }
    return $controller('UibRatingController', {
      $scope: $scope,
      $attrs: $attrs
    });
  }
]).directive('rating', [
  '$log',
  '$ratingSuppressWarning',
  function ($log, $ratingSuppressWarning) {
    return {
      require: [
        'rating',
        'ngModel'
      ],
      scope: {
        readonly: '=?',
        onHover: '&',
        onLeave: '&'
      },
      controller: 'RatingController',
      templateUrl: 'template/rating/rating.html',
      replace: true,
      link: function (scope, element, attrs, ctrls) {
        if (!$ratingSuppressWarning) {
          $log.warn('rating is now deprecated. Use uib-rating instead.');
        }
        var ratingCtrl = ctrls[0], ngModelCtrl = ctrls[1];
        ratingCtrl.init(ngModelCtrl);
      }
    };
  }
]);
/**
 * @ngdoc overview
 * @name ui.bootstrap.tabs
 *
 * @description
 * AngularJS version of the tabs directive.
 */
angular.module('ui.bootstrap.tabs', []).controller('UibTabsetController', [
  '$scope',
  function ($scope) {
    var ctrl = this, tabs = ctrl.tabs = $scope.tabs = [];
    ctrl.select = function (selectedTab) {
      angular.forEach(tabs, function (tab) {
        if (tab.active && tab !== selectedTab) {
          tab.active = false;
          tab.onDeselect();
          selectedTab.selectCalled = false;
        }
      });
      selectedTab.active = true;
      // only call select if it has not already been called
      if (!selectedTab.selectCalled) {
        selectedTab.onSelect();
        selectedTab.selectCalled = true;
      }
    };
    ctrl.addTab = function addTab(tab) {
      tabs.push(tab);
      // we can't run the select function on the first tab
      // since that would select it twice
      if (tabs.length === 1 && tab.active !== false) {
        tab.active = true;
      } else if (tab.active) {
        ctrl.select(tab);
      } else {
        tab.active = false;
      }
    };
    ctrl.removeTab = function removeTab(tab) {
      var index = tabs.indexOf(tab);
      //Select a new tab if the tab to be removed is selected and not destroyed
      if (tab.active && tabs.length > 1 && !destroyed) {
        //If this is the last tab, select the previous tab. else, the next tab.
        var newActiveIndex = index == tabs.length - 1 ? index - 1 : index + 1;
        ctrl.select(tabs[newActiveIndex]);
      }
      tabs.splice(index, 1);
    };
    var destroyed;
    $scope.$on('$destroy', function () {
      destroyed = true;
    });
  }
]).directive('uibTabset', function () {
  return {
    restrict: 'EA',
    transclude: true,
    replace: true,
    scope: { type: '@' },
    controller: 'UibTabsetController',
    templateUrl: 'template/tabs/tabset.html',
    link: function (scope, element, attrs) {
      scope.vertical = angular.isDefined(attrs.vertical) ? scope.$parent.$eval(attrs.vertical) : false;
      scope.justified = angular.isDefined(attrs.justified) ? scope.$parent.$eval(attrs.justified) : false;
    }
  };
}).directive('uibTab', [
  '$parse',
  function ($parse) {
    return {
      require: '^uibTabset',
      restrict: 'EA',
      replace: true,
      templateUrl: 'template/tabs/tab.html',
      transclude: true,
      scope: {
        active: '=?',
        heading: '@',
        onSelect: '&select',
        onDeselect: '&deselect'
      },
      controller: function () {
      },
      link: function (scope, elm, attrs, tabsetCtrl, transclude) {
        scope.$watch('active', function (active) {
          if (active) {
            tabsetCtrl.select(scope);
          }
        });
        scope.disabled = false;
        if (attrs.disable) {
          scope.$parent.$watch($parse(attrs.disable), function (value) {
            scope.disabled = !!value;
          });
        }
        scope.select = function () {
          if (!scope.disabled) {
            scope.active = true;
          }
        };
        tabsetCtrl.addTab(scope);
        scope.$on('$destroy', function () {
          tabsetCtrl.removeTab(scope);
        });
        //We need to transclude later, once the content container is ready.
        //when this link happens, we're inside a tab heading.
        scope.$transcludeFn = transclude;
      }
    };
  }
]).directive('uibTabHeadingTransclude', function () {
  return {
    restrict: 'A',
    require: [
      '?^uibTab',
      '?^tab'
    ],
    link: function (scope, elm) {
      scope.$watch('headingElement', function updateHeadingElement(heading) {
        if (heading) {
          elm.html('');
          elm.append(heading);
        }
      });
    }
  };
}).directive('uibTabContentTransclude', function () {
  return {
    restrict: 'A',
    require: [
      '?^uibTabset',
      '?^tabset'
    ],
    link: function (scope, elm, attrs) {
      var tab = scope.$eval(attrs.uibTabContentTransclude);
      //Now our tab is ready to be transcluded: both the tab heading area
      //and the tab content area are loaded.  Transclude 'em both.
      tab.$transcludeFn(tab.$parent, function (contents) {
        angular.forEach(contents, function (node) {
          if (isTabHeading(node)) {
            //Let tabHeadingTransclude know.
            tab.headingElement = node;
          } else {
            elm.append(node);
          }
        });
      });
    }
  };
  function isTabHeading(node) {
    return node.tagName && (node.hasAttribute('tab-heading') || node.hasAttribute('data-tab-heading') || node.hasAttribute('x-tab-heading') || node.hasAttribute('uib-tab-heading') || node.hasAttribute('data-uib-tab-heading') || node.hasAttribute('x-uib-tab-heading') || node.tagName.toLowerCase() === 'tab-heading' || node.tagName.toLowerCase() === 'data-tab-heading' || node.tagName.toLowerCase() === 'x-tab-heading' || node.tagName.toLowerCase() === 'uib-tab-heading' || node.tagName.toLowerCase() === 'data-uib-tab-heading' || node.tagName.toLowerCase() === 'x-uib-tab-heading');
  }
});
/* deprecated tabs below */
angular.module('ui.bootstrap.tabs').value('$tabsSuppressWarning', false).controller('TabsetController', [
  '$scope',
  '$controller',
  '$log',
  '$tabsSuppressWarning',
  function ($scope, $controller, $log, $tabsSuppressWarning) {
    if (!$tabsSuppressWarning) {
      $log.warn('TabsetController is now deprecated. Use UibTabsetController instead.');
    }
    return $controller('UibTabsetController', { $scope: $scope });
  }
]).directive('tabset', [
  '$log',
  '$tabsSuppressWarning',
  function ($log, $tabsSuppressWarning) {
    return {
      restrict: 'EA',
      transclude: true,
      replace: true,
      scope: { type: '@' },
      controller: 'TabsetController',
      templateUrl: 'template/tabs/tabset.html',
      link: function (scope, element, attrs) {
        if (!$tabsSuppressWarning) {
          $log.warn('tabset is now deprecated. Use uib-tabset instead.');
        }
        scope.vertical = angular.isDefined(attrs.vertical) ? scope.$parent.$eval(attrs.vertical) : false;
        scope.justified = angular.isDefined(attrs.justified) ? scope.$parent.$eval(attrs.justified) : false;
      }
    };
  }
]).directive('tab', [
  '$parse',
  '$log',
  '$tabsSuppressWarning',
  function ($parse, $log, $tabsSuppressWarning) {
    return {
      require: '^tabset',
      restrict: 'EA',
      replace: true,
      templateUrl: 'template/tabs/tab.html',
      transclude: true,
      scope: {
        active: '=?',
        heading: '@',
        onSelect: '&select',
        onDeselect: '&deselect'
      },
      controller: function () {
      },
      link: function (scope, elm, attrs, tabsetCtrl, transclude) {
        if (!$tabsSuppressWarning) {
          $log.warn('tab is now deprecated. Use uib-tab instead.');
        }
        scope.$watch('active', function (active) {
          if (active) {
            tabsetCtrl.select(scope);
          }
        });
        scope.disabled = false;
        if (attrs.disable) {
          scope.$parent.$watch($parse(attrs.disable), function (value) {
            scope.disabled = !!value;
          });
        }
        scope.select = function () {
          if (!scope.disabled) {
            scope.active = true;
          }
        };
        tabsetCtrl.addTab(scope);
        scope.$on('$destroy', function () {
          tabsetCtrl.removeTab(scope);
        });
        //We need to transclude later, once the content container is ready.
        //when this link happens, we're inside a tab heading.
        scope.$transcludeFn = transclude;
      }
    };
  }
]).directive('tabHeadingTransclude', [
  '$log',
  '$tabsSuppressWarning',
  function ($log, $tabsSuppressWarning) {
    return {
      restrict: 'A',
      require: '^tab',
      link: function (scope, elm) {
        if (!$tabsSuppressWarning) {
          $log.warn('tab-heading-transclude is now deprecated. Use uib-tab-heading-transclude instead.');
        }
        scope.$watch('headingElement', function updateHeadingElement(heading) {
          if (heading) {
            elm.html('');
            elm.append(heading);
          }
        });
      }
    };
  }
]).directive('tabContentTransclude', [
  '$log',
  '$tabsSuppressWarning',
  function ($log, $tabsSuppressWarning) {
    return {
      restrict: 'A',
      require: '^tabset',
      link: function (scope, elm, attrs) {
        if (!$tabsSuppressWarning) {
          $log.warn('tab-content-transclude is now deprecated. Use uib-tab-content-transclude instead.');
        }
        var tab = scope.$eval(attrs.tabContentTransclude);
        //Now our tab is ready to be transcluded: both the tab heading area
        //and the tab content area are loaded.  Transclude 'em both.
        tab.$transcludeFn(tab.$parent, function (contents) {
          angular.forEach(contents, function (node) {
            if (isTabHeading(node)) {
              //Let tabHeadingTransclude know.
              tab.headingElement = node;
            } else {
              elm.append(node);
            }
          });
        });
      }
    };
    function isTabHeading(node) {
      return node.tagName && (node.hasAttribute('tab-heading') || node.hasAttribute('data-tab-heading') || node.hasAttribute('x-tab-heading') || node.tagName.toLowerCase() === 'tab-heading' || node.tagName.toLowerCase() === 'data-tab-heading' || node.tagName.toLowerCase() === 'x-tab-heading');
    }
  }
]);
angular.module('ui.bootstrap.timepicker', []).constant('uibTimepickerConfig', {
  hourStep: 1,
  minuteStep: 1,
  showMeridian: true,
  meridians: null,
  readonlyInput: false,
  mousewheel: true,
  arrowkeys: true,
  showSpinners: true
}).controller('UibTimepickerController', [
  '$scope',
  '$attrs',
  '$parse',
  '$log',
  '$locale',
  'uibTimepickerConfig',
  function ($scope, $attrs, $parse, $log, $locale, timepickerConfig) {
    var selected = new Date(), ngModelCtrl = { $setViewValue: angular.noop },
      // nullModelCtrl
      meridians = angular.isDefined($attrs.meridians) ? $scope.$parent.$eval($attrs.meridians) : timepickerConfig.meridians || $locale.DATETIME_FORMATS.AMPMS;
    this.init = function (ngModelCtrl_, inputs) {
      ngModelCtrl = ngModelCtrl_;
      ngModelCtrl.$render = this.render;
      ngModelCtrl.$formatters.unshift(function (modelValue) {
        return modelValue ? new Date(modelValue) : null;
      });
      var hoursInputEl = inputs.eq(0), minutesInputEl = inputs.eq(1);
      var mousewheel = angular.isDefined($attrs.mousewheel) ? $scope.$parent.$eval($attrs.mousewheel) : timepickerConfig.mousewheel;
      if (mousewheel) {
        this.setupMousewheelEvents(hoursInputEl, minutesInputEl);
      }
      var arrowkeys = angular.isDefined($attrs.arrowkeys) ? $scope.$parent.$eval($attrs.arrowkeys) : timepickerConfig.arrowkeys;
      if (arrowkeys) {
        this.setupArrowkeyEvents(hoursInputEl, minutesInputEl);
      }
      $scope.readonlyInput = angular.isDefined($attrs.readonlyInput) ? $scope.$parent.$eval($attrs.readonlyInput) : timepickerConfig.readonlyInput;
      this.setupInputEvents(hoursInputEl, minutesInputEl);
    };
    var hourStep = timepickerConfig.hourStep;
    if ($attrs.hourStep) {
      $scope.$parent.$watch($parse($attrs.hourStep), function (value) {
        hourStep = parseInt(value, 10);
      });
    }
    var minuteStep = timepickerConfig.minuteStep;
    if ($attrs.minuteStep) {
      $scope.$parent.$watch($parse($attrs.minuteStep), function (value) {
        minuteStep = parseInt(value, 10);
      });
    }
    var min;
    $scope.$parent.$watch($parse($attrs.min), function (value) {
      var dt = new Date(value);
      min = isNaN(dt) ? undefined : dt;
    });
    var max;
    $scope.$parent.$watch($parse($attrs.max), function (value) {
      var dt = new Date(value);
      max = isNaN(dt) ? undefined : dt;
    });
    $scope.noIncrementHours = function () {
      var incrementedSelected = addMinutes(selected, hourStep * 60);
      return incrementedSelected > max || incrementedSelected < selected && incrementedSelected < min;
    };
    $scope.noDecrementHours = function () {
      var decrementedSelected = addMinutes(selected, -hourStep * 60);
      return decrementedSelected < min || decrementedSelected > selected && decrementedSelected > max;
    };
    $scope.noIncrementMinutes = function () {
      var incrementedSelected = addMinutes(selected, minuteStep);
      return incrementedSelected > max || incrementedSelected < selected && incrementedSelected < min;
    };
    $scope.noDecrementMinutes = function () {
      var decrementedSelected = addMinutes(selected, -minuteStep);
      return decrementedSelected < min || decrementedSelected > selected && decrementedSelected > max;
    };
    $scope.noToggleMeridian = function () {
      if (selected.getHours() < 13) {
        return addMinutes(selected, 12 * 60) > max;
      } else {
        return addMinutes(selected, -12 * 60) < min;
      }
    };
    // 12H / 24H mode
    $scope.showMeridian = timepickerConfig.showMeridian;
    if ($attrs.showMeridian) {
      $scope.$parent.$watch($parse($attrs.showMeridian), function (value) {
        $scope.showMeridian = !!value;
        if (ngModelCtrl.$error.time) {
          // Evaluate from template
          var hours = getHoursFromTemplate(), minutes = getMinutesFromTemplate();
          if (angular.isDefined(hours) && angular.isDefined(minutes)) {
            selected.setHours(hours);
            refresh();
          }
        } else {
          updateTemplate();
        }
      });
    }
    // Get $scope.hours in 24H mode if valid
    function getHoursFromTemplate() {
      var hours = parseInt($scope.hours, 10);
      var valid = $scope.showMeridian ? hours > 0 && hours < 13 : hours >= 0 && hours < 24;
      if (!valid) {
        return undefined;
      }
      if ($scope.showMeridian) {
        if (hours === 12) {
          hours = 0;
        }
        if ($scope.meridian === meridians[1]) {
          hours = hours + 12;
        }
      }
      return hours;
    }
    function getMinutesFromTemplate() {
      var minutes = parseInt($scope.minutes, 10);
      return minutes >= 0 && minutes < 60 ? minutes : undefined;
    }
    function pad(value) {
      return angular.isDefined(value) && value.toString().length < 2 ? '0' + value : value.toString();
    }
    // Respond on mousewheel spin
    this.setupMousewheelEvents = function (hoursInputEl, minutesInputEl) {
      var isScrollingUp = function (e) {
        if (e.originalEvent) {
          e = e.originalEvent;
        }
        //pick correct delta variable depending on event
        var delta = e.wheelDelta ? e.wheelDelta : -e.deltaY;
        return e.detail || delta > 0;
      };
      hoursInputEl.bind('mousewheel wheel', function (e) {
        $scope.$apply(isScrollingUp(e) ? $scope.incrementHours() : $scope.decrementHours());
        e.preventDefault();
      });
      minutesInputEl.bind('mousewheel wheel', function (e) {
        $scope.$apply(isScrollingUp(e) ? $scope.incrementMinutes() : $scope.decrementMinutes());
        e.preventDefault();
      });
    };
    // Respond on up/down arrowkeys
    this.setupArrowkeyEvents = function (hoursInputEl, minutesInputEl) {
      hoursInputEl.bind('keydown', function (e) {
        if (e.which === 38) {
          // up
          e.preventDefault();
          $scope.incrementHours();
          $scope.$apply();
        } else if (e.which === 40) {
          // down
          e.preventDefault();
          $scope.decrementHours();
          $scope.$apply();
        }
      });
      minutesInputEl.bind('keydown', function (e) {
        if (e.which === 38) {
          // up
          e.preventDefault();
          $scope.incrementMinutes();
          $scope.$apply();
        } else if (e.which === 40) {
          // down
          e.preventDefault();
          $scope.decrementMinutes();
          $scope.$apply();
        }
      });
    };
    this.setupInputEvents = function (hoursInputEl, minutesInputEl) {
      if ($scope.readonlyInput) {
        $scope.updateHours = angular.noop;
        $scope.updateMinutes = angular.noop;
        return;
      }
      var invalidate = function (invalidHours, invalidMinutes) {
        ngModelCtrl.$setViewValue(null);
        ngModelCtrl.$setValidity('time', false);
        if (angular.isDefined(invalidHours)) {
          $scope.invalidHours = invalidHours;
        }
        if (angular.isDefined(invalidMinutes)) {
          $scope.invalidMinutes = invalidMinutes;
        }
      };
      $scope.updateHours = function () {
        var hours = getHoursFromTemplate(), minutes = getMinutesFromTemplate();
        if (angular.isDefined(hours) && angular.isDefined(minutes)) {
          selected.setHours(hours);
          if (selected < min || selected > max) {
            invalidate(true);
          } else {
            refresh('h');
          }
        } else {
          invalidate(true);
        }
      };
      hoursInputEl.bind('blur', function (e) {
        if (!$scope.invalidHours && $scope.hours < 10) {
          $scope.$apply(function () {
            $scope.hours = pad($scope.hours);
          });
        }
      });
      $scope.updateMinutes = function () {
        var minutes = getMinutesFromTemplate(), hours = getHoursFromTemplate();
        if (angular.isDefined(minutes) && angular.isDefined(hours)) {
          selected.setMinutes(minutes);
          if (selected < min || selected > max) {
            invalidate(undefined, true);
          } else {
            refresh('m');
          }
        } else {
          invalidate(undefined, true);
        }
      };
      minutesInputEl.bind('blur', function (e) {
        if (!$scope.invalidMinutes && $scope.minutes < 10) {
          $scope.$apply(function () {
            $scope.minutes = pad($scope.minutes);
          });
        }
      });
    };
    this.render = function () {
      var date = ngModelCtrl.$viewValue;
      if (isNaN(date)) {
        ngModelCtrl.$setValidity('time', false);
        $log.error('Timepicker directive: "ng-model" value must be a Date object, a number of milliseconds since 01.01.1970 or a string representing an RFC2822 or ISO 8601 date.');
      } else {
        if (date) {
          selected = date;
        }
        if (selected < min || selected > max) {
          ngModelCtrl.$setValidity('time', false);
          $scope.invalidHours = true;
          $scope.invalidMinutes = true;
        } else {
          makeValid();
        }
        updateTemplate();
      }
    };
    // Call internally when we know that model is valid.
    function refresh(keyboardChange) {
      makeValid();
      ngModelCtrl.$setViewValue(new Date(selected));
      updateTemplate(keyboardChange);
    }
    function makeValid() {
      ngModelCtrl.$setValidity('time', true);
      $scope.invalidHours = false;
      $scope.invalidMinutes = false;
    }
    function updateTemplate(keyboardChange) {
      var hours = selected.getHours(), minutes = selected.getMinutes();
      if ($scope.showMeridian) {
        hours = hours === 0 || hours === 12 ? 12 : hours % 12;  // Convert 24 to 12 hour system
      }
      $scope.hours = keyboardChange === 'h' ? hours : pad(hours);
      if (keyboardChange !== 'm') {
        $scope.minutes = pad(minutes);
      }
      $scope.meridian = selected.getHours() < 12 ? meridians[0] : meridians[1];
    }
    function addMinutes(date, minutes) {
      var dt = new Date(date.getTime() + minutes * 60000);
      var newDate = new Date(date);
      newDate.setHours(dt.getHours(), dt.getMinutes());
      return newDate;
    }
    function addMinutesToSelected(minutes) {
      selected = addMinutes(selected, minutes);
      refresh();
    }
    $scope.showSpinners = angular.isDefined($attrs.showSpinners) ? $scope.$parent.$eval($attrs.showSpinners) : timepickerConfig.showSpinners;
    $scope.incrementHours = function () {
      if (!$scope.noIncrementHours()) {
        addMinutesToSelected(hourStep * 60);
      }
    };
    $scope.decrementHours = function () {
      if (!$scope.noDecrementHours()) {
        addMinutesToSelected(-hourStep * 60);
      }
    };
    $scope.incrementMinutes = function () {
      if (!$scope.noIncrementMinutes()) {
        addMinutesToSelected(minuteStep);
      }
    };
    $scope.decrementMinutes = function () {
      if (!$scope.noDecrementMinutes()) {
        addMinutesToSelected(-minuteStep);
      }
    };
    $scope.toggleMeridian = function () {
      if (!$scope.noToggleMeridian()) {
        addMinutesToSelected(12 * 60 * (selected.getHours() < 12 ? 1 : -1));
      }
    };
  }
]).directive('uibTimepicker', function () {
  return {
    restrict: 'EA',
    require: [
      'uibTimepicker',
      '?^ngModel'
    ],
    controller: 'UibTimepickerController',
    controllerAs: 'timepicker',
    replace: true,
    scope: {},
    templateUrl: function (element, attrs) {
      return attrs.templateUrl || 'template/timepicker/timepicker.html';
    },
    link: function (scope, element, attrs, ctrls) {
      var timepickerCtrl = ctrls[0], ngModelCtrl = ctrls[1];
      if (ngModelCtrl) {
        timepickerCtrl.init(ngModelCtrl, element.find('input'));
      }
    }
  };
});
/* Deprecated timepicker below */
angular.module('ui.bootstrap.timepicker').value('$timepickerSuppressWarning', false).controller('TimepickerController', [
  '$scope',
  '$attrs',
  '$controller',
  '$log',
  '$timepickerSuppressWarning',
  function ($scope, $attrs, $controller, $log, $timepickerSuppressWarning) {
    if (!$timepickerSuppressWarning) {
      $log.warn('TimepickerController is now deprecated. Use UibTimepickerController instead.');
    }
    return $controller('UibTimepickerController', {
      $scope: $scope,
      $attrs: $attrs
    });
  }
]).directive('timepicker', [
  '$log',
  '$timepickerSuppressWarning',
  function ($log, $timepickerSuppressWarning) {
    return {
      restrict: 'EA',
      require: [
        'timepicker',
        '?^ngModel'
      ],
      controller: 'TimepickerController',
      controllerAs: 'timepicker',
      replace: true,
      scope: {},
      templateUrl: function (element, attrs) {
        return attrs.templateUrl || 'template/timepicker/timepicker.html';
      },
      link: function (scope, element, attrs, ctrls) {
        if (!$timepickerSuppressWarning) {
          $log.warn('timepicker is now deprecated. Use uib-timepicker instead.');
        }
        var timepickerCtrl = ctrls[0], ngModelCtrl = ctrls[1];
        if (ngModelCtrl) {
          timepickerCtrl.init(ngModelCtrl, element.find('input'));
        }
      }
    };
  }
]);
angular.module('ui.bootstrap.typeahead', ['ui.bootstrap.position']).factory('uibTypeaheadParser', [
  '$parse',
  function ($parse) {
    //                      00000111000000000000022200000000000000003333333333333330000000000044000
    var TYPEAHEAD_REGEXP = /^\s*([\s\S]+?)(?:\s+as\s+([\s\S]+?))?\s+for\s+(?:([\$\w][\$\w\d]*))\s+in\s+([\s\S]+?)$/;
    return {
      parse: function (input) {
        var match = input.match(TYPEAHEAD_REGEXP);
        if (!match) {
          throw new Error('Expected typeahead specification in form of "_modelValue_ (as _label_)? for _item_ in _collection_"' + ' but got "' + input + '".');
        }
        return {
          itemName: match[3],
          source: $parse(match[4]),
          viewMapper: $parse(match[2] || match[1]),
          modelMapper: $parse(match[1])
        };
      }
    };
  }
]).directive('uibTypeahead', [
  '$compile',
  '$parse',
  '$q',
  '$timeout',
  '$document',
  '$window',
  '$rootScope',
  '$uibPosition',
  'uibTypeaheadParser',
  function ($compile, $parse, $q, $timeout, $document, $window, $rootScope, $position, typeaheadParser) {
    var HOT_KEYS = [
        9,
        13,
        27,
        38,
        40
      ];
    var eventDebounceTime = 200;
    return {
      require: [
        'ngModel',
        '^?ngModelOptions'
      ],
      link: function (originalScope, element, attrs, ctrls) {
        var modelCtrl = ctrls[0];
        var ngModelOptions = ctrls[1];
        //SUPPORTED ATTRIBUTES (OPTIONS)
        //minimal no of characters that needs to be entered before typeahead kicks-in
        var minLength = originalScope.$eval(attrs.typeaheadMinLength);
        if (!minLength && minLength !== 0) {
          minLength = 1;
        }
        //minimal wait time after last character typed before typeahead kicks-in
        var waitTime = originalScope.$eval(attrs.typeaheadWaitMs) || 0;
        //should it restrict model values to the ones selected from the popup only?
        var isEditable = originalScope.$eval(attrs.typeaheadEditable) !== false;
        //binding to a variable that indicates if matches are being retrieved asynchronously
        var isLoadingSetter = $parse(attrs.typeaheadLoading).assign || angular.noop;
        //a callback executed when a match is selected
        var onSelectCallback = $parse(attrs.typeaheadOnSelect);
        //should it select highlighted popup value when losing focus?
        var isSelectOnBlur = angular.isDefined(attrs.typeaheadSelectOnBlur) ? originalScope.$eval(attrs.typeaheadSelectOnBlur) : false;
        //binding to a variable that indicates if there were no results after the query is completed
        var isNoResultsSetter = $parse(attrs.typeaheadNoResults).assign || angular.noop;
        var inputFormatter = attrs.typeaheadInputFormatter ? $parse(attrs.typeaheadInputFormatter) : undefined;
        var appendToBody = attrs.typeaheadAppendToBody ? originalScope.$eval(attrs.typeaheadAppendToBody) : false;
        var appendToElementId = attrs.typeaheadAppendToElementId || false;
        var focusFirst = originalScope.$eval(attrs.typeaheadFocusFirst) !== false;
        //If input matches an item of the list exactly, select it automatically
        var selectOnExact = attrs.typeaheadSelectOnExact ? originalScope.$eval(attrs.typeaheadSelectOnExact) : false;
        //INTERNAL VARIABLES
        //model setter executed upon match selection
        var parsedModel = $parse(attrs.ngModel);
        var invokeModelSetter = $parse(attrs.ngModel + '($$$p)');
        var $setModelValue = function (scope, newValue) {
          if (angular.isFunction(parsedModel(originalScope)) && ngModelOptions && ngModelOptions.$options && ngModelOptions.$options.getterSetter) {
            return invokeModelSetter(scope, { $$$p: newValue });
          } else {
            return parsedModel.assign(scope, newValue);
          }
        };
        //expressions used by typeahead
        var parserResult = typeaheadParser.parse(attrs.uibTypeahead);
        var hasFocus;
        //Used to avoid bug in iOS webview where iOS keyboard does not fire
        //mousedown & mouseup events
        //Issue #3699
        var selected;
        //create a child scope for the typeahead directive so we are not polluting original scope
        //with typeahead-specific data (matches, query etc.)
        var scope = originalScope.$new();
        var offDestroy = originalScope.$on('$destroy', function () {
            scope.$destroy();
          });
        scope.$on('$destroy', offDestroy);
        // WAI-ARIA
        var popupId = 'typeahead-' + scope.$id + '-' + Math.floor(Math.random() * 10000);
        element.attr({
          'aria-autocomplete': 'list',
          'aria-expanded': false,
          'aria-owns': popupId
        });
        //pop-up element used to display matches
        var popUpEl = angular.element('<div uib-typeahead-popup></div>');
        popUpEl.attr({
          id: popupId,
          matches: 'matches',
          active: 'activeIdx',
          select: 'select(activeIdx)',
          'move-in-progress': 'moveInProgress',
          query: 'query',
          position: 'position'
        });
        //custom item template
        if (angular.isDefined(attrs.typeaheadTemplateUrl)) {
          popUpEl.attr('template-url', attrs.typeaheadTemplateUrl);
        }
        if (angular.isDefined(attrs.typeaheadPopupTemplateUrl)) {
          popUpEl.attr('popup-template-url', attrs.typeaheadPopupTemplateUrl);
        }
        var resetMatches = function () {
          scope.matches = [];
          scope.activeIdx = -1;
          element.attr('aria-expanded', false);
        };
        var getMatchId = function (index) {
          return popupId + '-option-' + index;
        };
        // Indicate that the specified match is the active (pre-selected) item in the list owned by this typeahead.
        // This attribute is added or removed automatically when the `activeIdx` changes.
        scope.$watch('activeIdx', function (index) {
          if (index < 0) {
            element.removeAttr('aria-activedescendant');
          } else {
            element.attr('aria-activedescendant', getMatchId(index));
          }
        });
        var inputIsExactMatch = function (inputValue, index) {
          if (scope.matches.length > index && inputValue) {
            return inputValue.toUpperCase() === scope.matches[index].label.toUpperCase();
          }
          return false;
        };
        var getMatchesAsync = function (inputValue) {
          var locals = { $viewValue: inputValue };
          isLoadingSetter(originalScope, true);
          isNoResultsSetter(originalScope, false);
          $q.when(parserResult.source(originalScope, locals)).then(function (matches) {
            //it might happen that several async queries were in progress if a user were typing fast
            //but we are interested only in responses that correspond to the current view value
            var onCurrentRequest = inputValue === modelCtrl.$viewValue;
            if (onCurrentRequest && hasFocus) {
              if (matches && matches.length > 0) {
                scope.activeIdx = focusFirst ? 0 : -1;
                isNoResultsSetter(originalScope, false);
                scope.matches.length = 0;
                //transform labels
                for (var i = 0; i < matches.length; i++) {
                  locals[parserResult.itemName] = matches[i];
                  scope.matches.push({
                    id: getMatchId(i),
                    label: parserResult.viewMapper(scope, locals),
                    model: matches[i]
                  });
                }
                scope.query = inputValue;
                //position pop-up with matches - we need to re-calculate its position each time we are opening a window
                //with matches as a pop-up might be absolute-positioned and position of an input might have changed on a page
                //due to other elements being rendered
                recalculatePosition();
                element.attr('aria-expanded', true);
                //Select the single remaining option if user input matches
                if (selectOnExact && scope.matches.length === 1 && inputIsExactMatch(inputValue, 0)) {
                  scope.select(0);
                }
              } else {
                resetMatches();
                isNoResultsSetter(originalScope, true);
              }
            }
            if (onCurrentRequest) {
              isLoadingSetter(originalScope, false);
            }
          }, function () {
            resetMatches();
            isLoadingSetter(originalScope, false);
            isNoResultsSetter(originalScope, true);
          });
        };
        // bind events only if appendToBody params exist - performance feature
        if (appendToBody) {
          angular.element($window).bind('resize', fireRecalculating);
          $document.find('body').bind('scroll', fireRecalculating);
        }
        // Declare the timeout promise var outside the function scope so that stacked calls can be cancelled later
        var timeoutEventPromise;
        // Default progress type
        scope.moveInProgress = false;
        function fireRecalculating() {
          if (!scope.moveInProgress) {
            scope.moveInProgress = true;
            scope.$digest();
          }
          // Cancel previous timeout
          if (timeoutEventPromise) {
            $timeout.cancel(timeoutEventPromise);
          }
          // Debounced executing recalculate after events fired
          timeoutEventPromise = $timeout(function () {
            // if popup is visible
            if (scope.matches.length) {
              recalculatePosition();
            }
            scope.moveInProgress = false;
          }, eventDebounceTime);
        }
        // recalculate actual position and set new values to scope
        // after digest loop is popup in right position
        function recalculatePosition() {
          scope.position = appendToBody ? $position.offset(element) : $position.position(element);
          scope.position.top += element.prop('offsetHeight');
        }
        resetMatches();
        //we need to propagate user's query so we can higlight matches
        scope.query = undefined;
        //Declare the timeout promise var outside the function scope so that stacked calls can be cancelled later
        var timeoutPromise;
        var scheduleSearchWithTimeout = function (inputValue) {
          timeoutPromise = $timeout(function () {
            getMatchesAsync(inputValue);
          }, waitTime);
        };
        var cancelPreviousTimeout = function () {
          if (timeoutPromise) {
            $timeout.cancel(timeoutPromise);
          }
        };
        //plug into $parsers pipeline to open a typeahead on view changes initiated from DOM
        //$parsers kick-in on all the changes coming from the view as well as manually triggered by $setViewValue
        modelCtrl.$parsers.unshift(function (inputValue) {
          hasFocus = true;
          if (minLength === 0 || inputValue && inputValue.length >= minLength) {
            if (waitTime > 0) {
              cancelPreviousTimeout();
              scheduleSearchWithTimeout(inputValue);
            } else {
              getMatchesAsync(inputValue);
            }
          } else {
            isLoadingSetter(originalScope, false);
            cancelPreviousTimeout();
            resetMatches();
          }
          if (isEditable) {
            return inputValue;
          } else {
            if (!inputValue) {
              // Reset in case user had typed something previously.
              modelCtrl.$setValidity('editable', true);
              return null;
            } else {
              modelCtrl.$setValidity('editable', false);
              return undefined;
            }
          }
        });
        modelCtrl.$formatters.push(function (modelValue) {
          var candidateViewValue, emptyViewValue;
          var locals = {};
          // The validity may be set to false via $parsers (see above) if
          // the model is restricted to selected values. If the model
          // is set manually it is considered to be valid.
          if (!isEditable) {
            modelCtrl.$setValidity('editable', true);
          }
          if (inputFormatter) {
            locals.$model = modelValue;
            return inputFormatter(originalScope, locals);
          } else {
            //it might happen that we don't have enough info to properly render input value
            //we need to check for this situation and simply return model value if we can't apply custom formatting
            locals[parserResult.itemName] = modelValue;
            candidateViewValue = parserResult.viewMapper(originalScope, locals);
            locals[parserResult.itemName] = undefined;
            emptyViewValue = parserResult.viewMapper(originalScope, locals);
            return candidateViewValue !== emptyViewValue ? candidateViewValue : modelValue;
          }
        });
        scope.select = function (activeIdx) {
          //called from within the $digest() cycle
          var locals = {};
          var model, item;
          selected = true;
          locals[parserResult.itemName] = item = scope.matches[activeIdx].model;
          model = parserResult.modelMapper(originalScope, locals);
          $setModelValue(originalScope, model);
          modelCtrl.$setValidity('editable', true);
          modelCtrl.$setValidity('parse', true);
          onSelectCallback(originalScope, {
            $item: item,
            $model: model,
            $label: parserResult.viewMapper(originalScope, locals)
          });
          resetMatches();
          //return focus to the input element if a match was selected via a mouse click event
          // use timeout to avoid $rootScope:inprog error
          if (scope.$eval(attrs.typeaheadFocusOnSelect) !== false) {
            $timeout(function () {
              element[0].focus();
            }, 0, false);
          }
        };
        //bind keyboard events: arrows up(38) / down(40), enter(13) and tab(9), esc(27)
        element.bind('keydown', function (evt) {
          //typeahead is open and an "interesting" key was pressed
          if (scope.matches.length === 0 || HOT_KEYS.indexOf(evt.which) === -1) {
            return;
          }
          // if there's nothing selected (i.e. focusFirst) and enter or tab is hit, clear the results
          if (scope.activeIdx === -1 && (evt.which === 9 || evt.which === 13)) {
            resetMatches();
            scope.$digest();
            return;
          }
          evt.preventDefault();
          if (evt.which === 40) {
            scope.activeIdx = (scope.activeIdx + 1) % scope.matches.length;
            scope.$digest();
          } else if (evt.which === 38) {
            scope.activeIdx = (scope.activeIdx > 0 ? scope.activeIdx : scope.matches.length) - 1;
            scope.$digest();
          } else if (evt.which === 13 || evt.which === 9) {
            scope.$apply(function () {
              scope.select(scope.activeIdx);
            });
          } else if (evt.which === 27) {
            evt.stopPropagation();
            resetMatches();
            scope.$digest();
          }
        });
        element.bind('blur', function () {
          if (isSelectOnBlur && scope.matches.length && scope.activeIdx !== -1 && !selected) {
            selected = true;
            scope.$apply(function () {
              scope.select(scope.activeIdx);
            });
          }
          hasFocus = false;
          selected = false;
        });
        // Keep reference to click handler to unbind it.
        var dismissClickHandler = function (evt) {
          // Issue #3973
          // Firefox treats right click as a click on document
          if (element[0] !== evt.target && evt.which !== 3 && scope.matches.length !== 0) {
            resetMatches();
            if (!$rootScope.$$phase) {
              scope.$digest();
            }
          }
        };
        $document.bind('click', dismissClickHandler);
        originalScope.$on('$destroy', function () {
          $document.unbind('click', dismissClickHandler);
          if (appendToBody || appendToElementId) {
            $popup.remove();
          }
          // Prevent jQuery cache memory leak
          popUpEl.remove();
        });
        var $popup = $compile(popUpEl)(scope);
        if (appendToBody) {
          $document.find('body').append($popup);
        } else if (appendToElementId !== false) {
          angular.element($document[0].getElementById(appendToElementId)).append($popup);
        } else {
          element.after($popup);
        }
      }
    };
  }
]).directive('uibTypeaheadPopup', function () {
  return {
    scope: {
      matches: '=',
      query: '=',
      active: '=',
      position: '&',
      moveInProgress: '=',
      select: '&'
    },
    replace: true,
    templateUrl: function (element, attrs) {
      return attrs.popupTemplateUrl || 'template/typeahead/typeahead-popup.html';
    },
    link: function (scope, element, attrs) {
      scope.templateUrl = attrs.templateUrl;
      scope.isOpen = function () {
        return scope.matches.length > 0;
      };
      scope.isActive = function (matchIdx) {
        return scope.active == matchIdx;
      };
      scope.selectActive = function (matchIdx) {
        scope.active = matchIdx;
      };
      scope.selectMatch = function (activeIdx) {
        scope.select({ activeIdx: activeIdx });
      };
    }
  };
}).directive('uibTypeaheadMatch', [
  '$templateRequest',
  '$compile',
  '$parse',
  function ($templateRequest, $compile, $parse) {
    return {
      scope: {
        index: '=',
        match: '=',
        query: '='
      },
      link: function (scope, element, attrs) {
        var tplUrl = $parse(attrs.templateUrl)(scope.$parent) || 'template/typeahead/typeahead-match.html';
        $templateRequest(tplUrl).then(function (tplContent) {
          $compile(tplContent.trim())(scope, function (clonedElement) {
            element.replaceWith(clonedElement);
          });
        });
      }
    };
  }
]).filter('uibTypeaheadHighlight', [
  '$sce',
  '$injector',
  '$log',
  function ($sce, $injector, $log) {
    var isSanitizePresent;
    isSanitizePresent = $injector.has('$sanitize');
    function escapeRegexp(queryToEscape) {
      // Regex: capture the whole query string and replace it with the string that will be used to match
      // the results, for example if the capture is "a" the result will be \a
      return queryToEscape.replace(/([.?*+^$[\]\\(){}|-])/g, '\\$1');
    }
    function containsHtml(matchItem) {
      return /<.*>/g.test(matchItem);
    }
    return function (matchItem, query) {
      if (!isSanitizePresent && containsHtml(matchItem)) {
        $log.warn('Unsafe use of typeahead please use ngSanitize');  // Warn the user about the danger
      }
      matchItem = query ? ('' + matchItem).replace(new RegExp(escapeRegexp(query), 'gi'), '<strong>$&</strong>') : matchItem;
      // Replaces the capture string with a the same string inside of a "strong" tag
      if (!isSanitizePresent) {
        matchItem = $sce.trustAsHtml(matchItem);  // If $sanitize is not present we pack the string in a $sce object for the ng-bind-html directive
      }
      return matchItem;
    };
  }
]);
/* Deprecated typeahead below */
angular.module('ui.bootstrap.typeahead').value('$typeaheadSuppressWarning', false).service('typeaheadParser', [
  '$parse',
  'uibTypeaheadParser',
  '$log',
  '$typeaheadSuppressWarning',
  function ($parse, uibTypeaheadParser, $log, $typeaheadSuppressWarning) {
    if (!$typeaheadSuppressWarning) {
      $log.warn('typeaheadParser is now deprecated. Use uibTypeaheadParser instead.');
    }
    return uibTypeaheadParser;
  }
]).directive('typeahead', [
  '$compile',
  '$parse',
  '$q',
  '$timeout',
  '$document',
  '$window',
  '$rootScope',
  '$uibPosition',
  'typeaheadParser',
  '$log',
  '$typeaheadSuppressWarning',
  function ($compile, $parse, $q, $timeout, $document, $window, $rootScope, $position, typeaheadParser, $log, $typeaheadSuppressWarning) {
    var HOT_KEYS = [
        9,
        13,
        27,
        38,
        40
      ];
    var eventDebounceTime = 200;
    return {
      require: [
        'ngModel',
        '^?ngModelOptions'
      ],
      link: function (originalScope, element, attrs, ctrls) {
        if (!$typeaheadSuppressWarning) {
          $log.warn('typeahead is now deprecated. Use uib-typeahead instead.');
        }
        var modelCtrl = ctrls[0];
        var ngModelOptions = ctrls[1];
        //SUPPORTED ATTRIBUTES (OPTIONS)
        //minimal no of characters that needs to be entered before typeahead kicks-in
        var minLength = originalScope.$eval(attrs.typeaheadMinLength);
        if (!minLength && minLength !== 0) {
          minLength = 1;
        }
        //minimal wait time after last character typed before typeahead kicks-in
        var waitTime = originalScope.$eval(attrs.typeaheadWaitMs) || 0;
        //should it restrict model values to the ones selected from the popup only?
        var isEditable = originalScope.$eval(attrs.typeaheadEditable) !== false;
        //binding to a variable that indicates if matches are being retrieved asynchronously
        var isLoadingSetter = $parse(attrs.typeaheadLoading).assign || angular.noop;
        //a callback executed when a match is selected
        var onSelectCallback = $parse(attrs.typeaheadOnSelect);
        //should it select highlighted popup value when losing focus?
        var isSelectOnBlur = angular.isDefined(attrs.typeaheadSelectOnBlur) ? originalScope.$eval(attrs.typeaheadSelectOnBlur) : false;
        //binding to a variable that indicates if there were no results after the query is completed
        var isNoResultsSetter = $parse(attrs.typeaheadNoResults).assign || angular.noop;
        var inputFormatter = attrs.typeaheadInputFormatter ? $parse(attrs.typeaheadInputFormatter) : undefined;
        var appendToBody = attrs.typeaheadAppendToBody ? originalScope.$eval(attrs.typeaheadAppendToBody) : false;
        var appendToElementId = attrs.typeaheadAppendToElementId || false;
        var focusFirst = originalScope.$eval(attrs.typeaheadFocusFirst) !== false;
        //If input matches an item of the list exactly, select it automatically
        var selectOnExact = attrs.typeaheadSelectOnExact ? originalScope.$eval(attrs.typeaheadSelectOnExact) : false;
        //INTERNAL VARIABLES
        //model setter executed upon match selection
        var parsedModel = $parse(attrs.ngModel);
        var invokeModelSetter = $parse(attrs.ngModel + '($$$p)');
        var $setModelValue = function (scope, newValue) {
          if (angular.isFunction(parsedModel(originalScope)) && ngModelOptions && ngModelOptions.$options && ngModelOptions.$options.getterSetter) {
            return invokeModelSetter(scope, { $$$p: newValue });
          } else {
            return parsedModel.assign(scope, newValue);
          }
        };
        //expressions used by typeahead
        var parserResult = typeaheadParser.parse(attrs.typeahead);
        var hasFocus;
        //Used to avoid bug in iOS webview where iOS keyboard does not fire
        //mousedown & mouseup events
        //Issue #3699
        var selected;
        //create a child scope for the typeahead directive so we are not polluting original scope
        //with typeahead-specific data (matches, query etc.)
        var scope = originalScope.$new();
        var offDestroy = originalScope.$on('$destroy', function () {
            scope.$destroy();
          });
        scope.$on('$destroy', offDestroy);
        // WAI-ARIA
        var popupId = 'typeahead-' + scope.$id + '-' + Math.floor(Math.random() * 10000);
        element.attr({
          'aria-autocomplete': 'list',
          'aria-expanded': false,
          'aria-owns': popupId
        });
        //pop-up element used to display matches
        var popUpEl = angular.element('<div typeahead-popup></div>');
        popUpEl.attr({
          id: popupId,
          matches: 'matches',
          active: 'activeIdx',
          select: 'select(activeIdx)',
          'move-in-progress': 'moveInProgress',
          query: 'query',
          position: 'position'
        });
        //custom item template
        if (angular.isDefined(attrs.typeaheadTemplateUrl)) {
          popUpEl.attr('template-url', attrs.typeaheadTemplateUrl);
        }
        if (angular.isDefined(attrs.typeaheadPopupTemplateUrl)) {
          popUpEl.attr('popup-template-url', attrs.typeaheadPopupTemplateUrl);
        }
        var resetMatches = function () {
          scope.matches = [];
          scope.activeIdx = -1;
          element.attr('aria-expanded', false);
        };
        var getMatchId = function (index) {
          return popupId + '-option-' + index;
        };
        // Indicate that the specified match is the active (pre-selected) item in the list owned by this typeahead.
        // This attribute is added or removed automatically when the `activeIdx` changes.
        scope.$watch('activeIdx', function (index) {
          if (index < 0) {
            element.removeAttr('aria-activedescendant');
          } else {
            element.attr('aria-activedescendant', getMatchId(index));
          }
        });
        var inputIsExactMatch = function (inputValue, index) {
          if (scope.matches.length > index && inputValue) {
            return inputValue.toUpperCase() === scope.matches[index].label.toUpperCase();
          }
          return false;
        };
        var getMatchesAsync = function (inputValue) {
          var locals = { $viewValue: inputValue };
          isLoadingSetter(originalScope, true);
          isNoResultsSetter(originalScope, false);
          $q.when(parserResult.source(originalScope, locals)).then(function (matches) {
            //it might happen that several async queries were in progress if a user were typing fast
            //but we are interested only in responses that correspond to the current view value
            var onCurrentRequest = inputValue === modelCtrl.$viewValue;
            if (onCurrentRequest && hasFocus) {
              if (matches && matches.length > 0) {
                scope.activeIdx = focusFirst ? 0 : -1;
                isNoResultsSetter(originalScope, false);
                scope.matches.length = 0;
                //transform labels
                for (var i = 0; i < matches.length; i++) {
                  locals[parserResult.itemName] = matches[i];
                  scope.matches.push({
                    id: getMatchId(i),
                    label: parserResult.viewMapper(scope, locals),
                    model: matches[i]
                  });
                }
                scope.query = inputValue;
                //position pop-up with matches - we need to re-calculate its position each time we are opening a window
                //with matches as a pop-up might be absolute-positioned and position of an input might have changed on a page
                //due to other elements being rendered
                recalculatePosition();
                element.attr('aria-expanded', true);
                //Select the single remaining option if user input matches
                if (selectOnExact && scope.matches.length === 1 && inputIsExactMatch(inputValue, 0)) {
                  scope.select(0);
                }
              } else {
                resetMatches();
                isNoResultsSetter(originalScope, true);
              }
            }
            if (onCurrentRequest) {
              isLoadingSetter(originalScope, false);
            }
          }, function () {
            resetMatches();
            isLoadingSetter(originalScope, false);
            isNoResultsSetter(originalScope, true);
          });
        };
        // bind events only if appendToBody params exist - performance feature
        if (appendToBody) {
          angular.element($window).bind('resize', fireRecalculating);
          $document.find('body').bind('scroll', fireRecalculating);
        }
        // Declare the timeout promise var outside the function scope so that stacked calls can be cancelled later
        var timeoutEventPromise;
        // Default progress type
        scope.moveInProgress = false;
        function fireRecalculating() {
          if (!scope.moveInProgress) {
            scope.moveInProgress = true;
            scope.$digest();
          }
          // Cancel previous timeout
          if (timeoutEventPromise) {
            $timeout.cancel(timeoutEventPromise);
          }
          // Debounced executing recalculate after events fired
          timeoutEventPromise = $timeout(function () {
            // if popup is visible
            if (scope.matches.length) {
              recalculatePosition();
            }
            scope.moveInProgress = false;
          }, eventDebounceTime);
        }
        // recalculate actual position and set new values to scope
        // after digest loop is popup in right position
        function recalculatePosition() {
          scope.position = appendToBody ? $position.offset(element) : $position.position(element);
          scope.position.top += element.prop('offsetHeight');
        }
        resetMatches();
        //we need to propagate user's query so we can higlight matches
        scope.query = undefined;
        //Declare the timeout promise var outside the function scope so that stacked calls can be cancelled later
        var timeoutPromise;
        var scheduleSearchWithTimeout = function (inputValue) {
          timeoutPromise = $timeout(function () {
            getMatchesAsync(inputValue);
          }, waitTime);
        };
        var cancelPreviousTimeout = function () {
          if (timeoutPromise) {
            $timeout.cancel(timeoutPromise);
          }
        };
        //plug into $parsers pipeline to open a typeahead on view changes initiated from DOM
        //$parsers kick-in on all the changes coming from the view as well as manually triggered by $setViewValue
        modelCtrl.$parsers.unshift(function (inputValue) {
          hasFocus = true;
          if (minLength === 0 || inputValue && inputValue.length >= minLength) {
            if (waitTime > 0) {
              cancelPreviousTimeout();
              scheduleSearchWithTimeout(inputValue);
            } else {
              getMatchesAsync(inputValue);
            }
          } else {
            isLoadingSetter(originalScope, false);
            cancelPreviousTimeout();
            resetMatches();
          }
          if (isEditable) {
            return inputValue;
          } else {
            if (!inputValue) {
              // Reset in case user had typed something previously.
              modelCtrl.$setValidity('editable', true);
              return null;
            } else {
              modelCtrl.$setValidity('editable', false);
              return undefined;
            }
          }
        });
        modelCtrl.$formatters.push(function (modelValue) {
          var candidateViewValue, emptyViewValue;
          var locals = {};
          // The validity may be set to false via $parsers (see above) if
          // the model is restricted to selected values. If the model
          // is set manually it is considered to be valid.
          if (!isEditable) {
            modelCtrl.$setValidity('editable', true);
          }
          if (inputFormatter) {
            locals.$model = modelValue;
            return inputFormatter(originalScope, locals);
          } else {
            //it might happen that we don't have enough info to properly render input value
            //we need to check for this situation and simply return model value if we can't apply custom formatting
            locals[parserResult.itemName] = modelValue;
            candidateViewValue = parserResult.viewMapper(originalScope, locals);
            locals[parserResult.itemName] = undefined;
            emptyViewValue = parserResult.viewMapper(originalScope, locals);
            return candidateViewValue !== emptyViewValue ? candidateViewValue : modelValue;
          }
        });
        scope.select = function (activeIdx) {
          //called from within the $digest() cycle
          var locals = {};
          var model, item;
          selected = true;
          locals[parserResult.itemName] = item = scope.matches[activeIdx].model;
          model = parserResult.modelMapper(originalScope, locals);
          $setModelValue(originalScope, model);
          modelCtrl.$setValidity('editable', true);
          modelCtrl.$setValidity('parse', true);
          onSelectCallback(originalScope, {
            $item: item,
            $model: model,
            $label: parserResult.viewMapper(originalScope, locals)
          });
          resetMatches();
          //return focus to the input element if a match was selected via a mouse click event
          // use timeout to avoid $rootScope:inprog error
          if (scope.$eval(attrs.typeaheadFocusOnSelect) !== false) {
            $timeout(function () {
              element[0].focus();
            }, 0, false);
          }
        };
        //bind keyboard events: arrows up(38) / down(40), enter(13) and tab(9), esc(27)
        element.bind('keydown', function (evt) {
          //typeahead is open and an "interesting" key was pressed
          if (scope.matches.length === 0 || HOT_KEYS.indexOf(evt.which) === -1) {
            return;
          }
          // if there's nothing selected (i.e. focusFirst) and enter or tab is hit, clear the results
          if (scope.activeIdx === -1 && (evt.which === 9 || evt.which === 13)) {
            resetMatches();
            scope.$digest();
            return;
          }
          evt.preventDefault();
          if (evt.which === 40) {
            scope.activeIdx = (scope.activeIdx + 1) % scope.matches.length;
            scope.$digest();
          } else if (evt.which === 38) {
            scope.activeIdx = (scope.activeIdx > 0 ? scope.activeIdx : scope.matches.length) - 1;
            scope.$digest();
          } else if (evt.which === 13 || evt.which === 9) {
            scope.$apply(function () {
              scope.select(scope.activeIdx);
            });
          } else if (evt.which === 27) {
            evt.stopPropagation();
            resetMatches();
            scope.$digest();
          }
        });
        element.bind('blur', function () {
          if (isSelectOnBlur && scope.matches.length && scope.activeIdx !== -1 && !selected) {
            selected = true;
            scope.$apply(function () {
              scope.select(scope.activeIdx);
            });
          }
          hasFocus = false;
          selected = false;
        });
        // Keep reference to click handler to unbind it.
        var dismissClickHandler = function (evt) {
          // Issue #3973
          // Firefox treats right click as a click on document
          if (element[0] !== evt.target && evt.which !== 3 && scope.matches.length !== 0) {
            resetMatches();
            if (!$rootScope.$$phase) {
              scope.$digest();
            }
          }
        };
        $document.bind('click', dismissClickHandler);
        originalScope.$on('$destroy', function () {
          $document.unbind('click', dismissClickHandler);
          if (appendToBody || appendToElementId) {
            $popup.remove();
          }
          // Prevent jQuery cache memory leak
          popUpEl.remove();
        });
        var $popup = $compile(popUpEl)(scope);
        if (appendToBody) {
          $document.find('body').append($popup);
        } else if (appendToElementId !== false) {
          angular.element($document[0].getElementById(appendToElementId)).append($popup);
        } else {
          element.after($popup);
        }
      }
    };
  }
]).directive('typeaheadPopup', [
  '$typeaheadSuppressWarning',
  '$log',
  function ($typeaheadSuppressWarning, $log) {
    return {
      scope: {
        matches: '=',
        query: '=',
        active: '=',
        position: '&',
        moveInProgress: '=',
        select: '&'
      },
      replace: true,
      templateUrl: function (element, attrs) {
        return attrs.popupTemplateUrl || 'template/typeahead/typeahead-popup.html';
      },
      link: function (scope, element, attrs) {
        if (!$typeaheadSuppressWarning) {
          $log.warn('typeahead-popup is now deprecated. Use uib-typeahead-popup instead.');
        }
        scope.templateUrl = attrs.templateUrl;
        scope.isOpen = function () {
          return scope.matches.length > 0;
        };
        scope.isActive = function (matchIdx) {
          return scope.active == matchIdx;
        };
        scope.selectActive = function (matchIdx) {
          scope.active = matchIdx;
        };
        scope.selectMatch = function (activeIdx) {
          scope.select({ activeIdx: activeIdx });
        };
      }
    };
  }
]).directive('typeaheadMatch', [
  '$templateRequest',
  '$compile',
  '$parse',
  '$typeaheadSuppressWarning',
  '$log',
  function ($templateRequest, $compile, $parse, $typeaheadSuppressWarning, $log) {
    return {
      restrict: 'EA',
      scope: {
        index: '=',
        match: '=',
        query: '='
      },
      link: function (scope, element, attrs) {
        if (!$typeaheadSuppressWarning) {
          $log.warn('typeahead-match is now deprecated. Use uib-typeahead-match instead.');
        }
        var tplUrl = $parse(attrs.templateUrl)(scope.$parent) || 'template/typeahead/typeahead-match.html';
        $templateRequest(tplUrl).then(function (tplContent) {
          $compile(tplContent.trim())(scope, function (clonedElement) {
            element.replaceWith(clonedElement);
          });
        });
      }
    };
  }
]).filter('typeaheadHighlight', [
  '$sce',
  '$injector',
  '$log',
  '$typeaheadSuppressWarning',
  function ($sce, $injector, $log, $typeaheadSuppressWarning) {
    var isSanitizePresent;
    isSanitizePresent = $injector.has('$sanitize');
    function escapeRegexp(queryToEscape) {
      // Regex: capture the whole query string and replace it with the string that will be used to match
      // the results, for example if the capture is "a" the result will be \a
      return queryToEscape.replace(/([.?*+^$[\]\\(){}|-])/g, '\\$1');
    }
    function containsHtml(matchItem) {
      return /<.*>/g.test(matchItem);
    }
    return function (matchItem, query) {
      if (!$typeaheadSuppressWarning) {
        $log.warn('typeaheadHighlight is now deprecated. Use uibTypeaheadHighlight instead.');
      }
      if (!isSanitizePresent && containsHtml(matchItem)) {
        $log.warn('Unsafe use of typeahead please use ngSanitize');  // Warn the user about the danger
      }
      matchItem = query ? ('' + matchItem).replace(new RegExp(escapeRegexp(query), 'gi'), '<strong>$&</strong>') : matchItem;
      // Replaces the capture string with a the same string inside of a "strong" tag
      if (!isSanitizePresent) {
        matchItem = $sce.trustAsHtml(matchItem);  // If $sanitize is not present we pack the string in a $sce object for the ng-bind-html directive
      }
      return matchItem;
    };
  }
]);
angular.module('template/accordion/accordion-group.html', []).run([
  '$templateCache',
  function ($templateCache) {
    $templateCache.put('template/accordion/accordion-group.html', '<div class="panel {{panelClass || \'panel-default\'}}">\n' + '  <div class="panel-heading" ng-keypress="toggleOpen($event)">\n' + '    <h4 class="panel-title">\n' + '      <a href tabindex="0" class="accordion-toggle" ng-click="toggleOpen()" uib-accordion-transclude="heading"><span ng-class="{\'text-muted\': isDisabled}">{{heading}}</span></a>\n' + '    </h4>\n' + '  </div>\n' + '  <div class="panel-collapse collapse" uib-collapse="!isOpen">\n' + '\t  <div class="panel-body" ng-transclude></div>\n' + '  </div>\n' + '</div>\n' + '');
  }
]);
angular.module('template/accordion/accordion.html', []).run([
  '$templateCache',
  function ($templateCache) {
    $templateCache.put('template/accordion/accordion.html', '<div class="panel-group" ng-transclude></div>');
  }
]);
angular.module('template/alert/alert.html', []).run([
  '$templateCache',
  function ($templateCache) {
    $templateCache.put('template/alert/alert.html', '<div class="alert" ng-class="[\'alert-\' + (type || \'warning\'), closeable ? \'alert-dismissible\' : null]" role="alert">\n' + '    <button ng-show="closeable" type="button" class="close" ng-click="close({$event: $event})">\n' + '        <span aria-hidden="true">&times;</span>\n' + '        <span class="sr-only">Close</span>\n' + '    </button>\n' + '    <div ng-transclude></div>\n' + '</div>\n' + '');
  }
]);
angular.module('template/carousel/carousel.html', []).run([
  '$templateCache',
  function ($templateCache) {
    $templateCache.put('template/carousel/carousel.html', '<div ng-mouseenter="pause()" ng-mouseleave="play()" class="carousel" ng-swipe-right="prev()" ng-swipe-left="next()">\n' + '  <div class="carousel-inner" ng-transclude></div>\n' + '  <a role="button" href class="left carousel-control" ng-click="prev()" ng-show="slides.length > 1">\n' + '    <span aria-hidden="true" class="glyphicon glyphicon-chevron-left"></span>\n' + '    <span class="sr-only">previous</span>\n' + '  </a>\n' + '  <a role="button" href class="right carousel-control" ng-click="next()" ng-show="slides.length > 1">\n' + '    <span aria-hidden="true" class="glyphicon glyphicon-chevron-right"></span>\n' + '    <span class="sr-only">next</span>\n' + '  </a>\n' + '  <ol class="carousel-indicators" ng-show="slides.length > 1">\n' + '    <li ng-repeat="slide in slides | orderBy:indexOfSlide track by $index" ng-class="{ active: isActive(slide) }" ng-click="select(slide)">\n' + '      <span class="sr-only">slide {{ $index + 1 }} of {{ slides.length }}<span ng-if="isActive(slide)">, currently active</span></span>\n' + '    </li>\n' + '  </ol>\n' + '</div>');
  }
]);
angular.module('template/carousel/slide.html', []).run([
  '$templateCache',
  function ($templateCache) {
    $templateCache.put('template/carousel/slide.html', '<div ng-class="{\n' + '    \'active\': active\n' + '  }" class="item text-center" ng-transclude></div>\n' + '');
  }
]);
angular.module('template/datepicker/datepicker.html', []).run([
  '$templateCache',
  function ($templateCache) {
    $templateCache.put('template/datepicker/datepicker.html', '<div ng-switch="datepickerMode" role="application" ng-keydown="keydown($event)">\n' + '  <uib-daypicker ng-switch-when="day" tabindex="0"></uib-daypicker>\n' + '  <uib-monthpicker ng-switch-when="month" tabindex="0"></uib-monthpicker>\n' + '  <uib-yearpicker ng-switch-when="year" tabindex="0"></uib-yearpicker>\n' + '</div>');
  }
]);
angular.module('template/datepicker/day.html', []).run([
  '$templateCache',
  function ($templateCache) {
    $templateCache.put('template/datepicker/day.html', '<table role="grid" aria-labelledby="{{::uniqueId}}-title" aria-activedescendant="{{activeDateId}}">\n' + '  <thead>\n' + '    <tr>\n' + '      <th><button type="button" class="btn btn-default btn-sm pull-left" ng-click="move(-1)" tabindex="-1"><i class="glyphicon glyphicon-chevron-left"></i></button></th>\n' + '      <th colspan="{{::5 + showWeeks}}"><button id="{{::uniqueId}}-title" role="heading" aria-live="assertive" aria-atomic="true" type="button" class="btn btn-default btn-sm" ng-click="toggleMode()" ng-disabled="datepickerMode === maxMode" tabindex="-1" style="width:100%;"><strong>{{title}}</strong></button></th>\n' + '      <th><button type="button" class="btn btn-default btn-sm pull-right" ng-click="move(1)" tabindex="-1"><i class="glyphicon glyphicon-chevron-right"></i></button></th>\n' + '    </tr>\n' + '    <tr>\n' + '      <th ng-if="showWeeks" class="text-center"></th>\n' + '      <th ng-repeat="label in ::labels track by $index" class="text-center"><small aria-label="{{::label.full}}">{{::label.abbr}}</small></th>\n' + '    </tr>\n' + '  </thead>\n' + '  <tbody>\n' + '    <tr ng-repeat="row in rows track by $index">\n' + '      <td ng-if="showWeeks" class="text-center h6"><em>{{ weekNumbers[$index] }}</em></td>\n' + '      <td ng-repeat="dt in row track by dt.date" class="text-center" role="gridcell" id="{{::dt.uid}}" ng-class="::dt.customClass">\n' + '        <button type="button" style="min-width:100%;" class="btn btn-default btn-sm" ng-class="{\'btn-info\': dt.selected, active: isActive(dt)}" ng-click="select(dt.date)" ng-disabled="dt.disabled" tabindex="-1"><span ng-class="::{\'text-muted\': dt.secondary, \'text-info\': dt.current}">{{::dt.label}}</span></button>\n' + '      </td>\n' + '    </tr>\n' + '  </tbody>\n' + '</table>\n' + '');
  }
]);
angular.module('template/datepicker/month.html', []).run([
  '$templateCache',
  function ($templateCache) {
    $templateCache.put('template/datepicker/month.html', '<table role="grid" aria-labelledby="{{::uniqueId}}-title" aria-activedescendant="{{activeDateId}}">\n' + '  <thead>\n' + '    <tr>\n' + '      <th><button type="button" class="btn btn-default btn-sm pull-left" ng-click="move(-1)" tabindex="-1"><i class="glyphicon glyphicon-chevron-left"></i></button></th>\n' + '      <th><button id="{{::uniqueId}}-title" role="heading" aria-live="assertive" aria-atomic="true" type="button" class="btn btn-default btn-sm" ng-click="toggleMode()" ng-disabled="datepickerMode === maxMode" tabindex="-1" style="width:100%;"><strong>{{title}}</strong></button></th>\n' + '      <th><button type="button" class="btn btn-default btn-sm pull-right" ng-click="move(1)" tabindex="-1"><i class="glyphicon glyphicon-chevron-right"></i></button></th>\n' + '    </tr>\n' + '  </thead>\n' + '  <tbody>\n' + '    <tr ng-repeat="row in rows track by $index">\n' + '      <td ng-repeat="dt in row track by dt.date" class="text-center" role="gridcell" id="{{::dt.uid}}" ng-class="::dt.customClass">\n' + '        <button type="button" style="min-width:100%;" class="btn btn-default" ng-class="{\'btn-info\': dt.selected, active: isActive(dt)}" ng-click="select(dt.date)" ng-disabled="dt.disabled" tabindex="-1"><span ng-class="::{\'text-info\': dt.current}">{{::dt.label}}</span></button>\n' + '      </td>\n' + '    </tr>\n' + '  </tbody>\n' + '</table>\n' + '');
  }
]);
angular.module('template/datepicker/popup.html', []).run([
  '$templateCache',
  function ($templateCache) {
    $templateCache.put('template/datepicker/popup.html', '<ul class="dropdown-menu" dropdown-nested ng-if="isOpen" style="display: block" ng-style="{top: position.top+\'px\', left: position.left+\'px\'}" ng-keydown="keydown($event)" ng-click="$event.stopPropagation()">\n' + '\t<li ng-transclude></li>\n' + '\t<li ng-if="showButtonBar" style="padding:10px 9px 2px">\n' + '\t\t<span class="btn-group pull-left">\n' + '\t\t\t<button type="button" class="btn btn-sm btn-info" ng-click="select(\'today\')" ng-disabled="isDisabled(\'today\')">{{ getText(\'current\') }}</button>\n' + '\t\t\t<button type="button" class="btn btn-sm btn-danger" ng-click="select(null)">{{ getText(\'clear\') }}</button>\n' + '\t\t</span>\n' + '\t\t<button type="button" class="btn btn-sm btn-success pull-right" ng-click="close()">{{ getText(\'close\') }}</button>\n' + '\t</li>\n' + '</ul>\n' + '');
  }
]);
angular.module('template/datepicker/year.html', []).run([
  '$templateCache',
  function ($templateCache) {
    $templateCache.put('template/datepicker/year.html', '<table role="grid" aria-labelledby="{{::uniqueId}}-title" aria-activedescendant="{{activeDateId}}">\n' + '  <thead>\n' + '    <tr>\n' + '      <th><button type="button" class="btn btn-default btn-sm pull-left" ng-click="move(-1)" tabindex="-1"><i class="glyphicon glyphicon-chevron-left"></i></button></th>\n' + '      <th colspan="3"><button id="{{::uniqueId}}-title" role="heading" aria-live="assertive" aria-atomic="true" type="button" class="btn btn-default btn-sm" ng-click="toggleMode()" ng-disabled="datepickerMode === maxMode" tabindex="-1" style="width:100%;"><strong>{{title}}</strong></button></th>\n' + '      <th><button type="button" class="btn btn-default btn-sm pull-right" ng-click="move(1)" tabindex="-1"><i class="glyphicon glyphicon-chevron-right"></i></button></th>\n' + '    </tr>\n' + '  </thead>\n' + '  <tbody>\n' + '    <tr ng-repeat="row in rows track by $index">\n' + '      <td ng-repeat="dt in row track by dt.date" class="text-center" role="gridcell" id="{{::dt.uid}}" ng-class="::dt.customClass">\n' + '        <button type="button" style="min-width:100%;" class="btn btn-default" ng-class="{\'btn-info\': dt.selected, active: isActive(dt)}" ng-click="select(dt.date)" ng-disabled="dt.disabled" tabindex="-1"><span ng-class="::{\'text-info\': dt.current}">{{::dt.label}}</span></button>\n' + '      </td>\n' + '    </tr>\n' + '  </tbody>\n' + '</table>\n' + '');
  }
]);
angular.module('template/modal/backdrop.html', []).run([
  '$templateCache',
  function ($templateCache) {
    $templateCache.put('template/modal/backdrop.html', '<div uib-modal-animation-class="fade"\n' + '     modal-in-class="in"\n' + '     ng-style="{\'z-index\': 1040 + (index && 1 || 0) + index*10}"\n' + '></div>\n' + '');
  }
]);
angular.module('template/modal/window.html', []).run([
  '$templateCache',
  function ($templateCache) {
    $templateCache.put('template/modal/window.html', '<div modal-render="{{$isRendered}}" tabindex="-1" role="dialog" class="modal"\n' + '    uib-modal-animation-class="fade"\n' + '    modal-in-class="in"\n' + '    ng-style="{\'z-index\': 1050 + index*10, display: \'block\'}">\n' + '    <div class="modal-dialog" ng-class="size ? \'modal-\' + size : \'\'"><div class="modal-content" uib-modal-transclude></div></div>\n' + '</div>\n' + '');
  }
]);
angular.module('template/pagination/pager.html', []).run([
  '$templateCache',
  function ($templateCache) {
    $templateCache.put('template/pagination/pager.html', '<ul class="pager">\n' + '  <li ng-class="{disabled: noPrevious()||ngDisabled, previous: align}"><a href ng-click="selectPage(page - 1, $event)">{{::getText(\'previous\')}}</a></li>\n' + '  <li ng-class="{disabled: noNext()||ngDisabled, next: align}"><a href ng-click="selectPage(page + 1, $event)">{{::getText(\'next\')}}</a></li>\n' + '</ul>\n' + '');
  }
]);
angular.module('template/pagination/pagination.html', []).run([
  '$templateCache',
  function ($templateCache) {
    $templateCache.put('template/pagination/pagination.html', '<ul class="pagination">\n' + '  <li ng-if="::boundaryLinks" ng-class="{disabled: noPrevious()||ngDisabled}" class="pagination-first"><a href ng-click="selectPage(1, $event)">{{::getText(\'first\')}}</a></li>\n' + '  <li ng-if="::directionLinks" ng-class="{disabled: noPrevious()||ngDisabled}" class="pagination-prev"><a href ng-click="selectPage(page - 1, $event)">{{::getText(\'previous\')}}</a></li>\n' + '  <li ng-repeat="page in pages track by $index" ng-class="{active: page.active,disabled: ngDisabled&&!page.active}" class="pagination-page"><a href ng-click="selectPage(page.number, $event)">{{page.text}}</a></li>\n' + '  <li ng-if="::directionLinks" ng-class="{disabled: noNext()||ngDisabled}" class="pagination-next"><a href ng-click="selectPage(page + 1, $event)">{{::getText(\'next\')}}</a></li>\n' + '  <li ng-if="::boundaryLinks" ng-class="{disabled: noNext()||ngDisabled}" class="pagination-last"><a href ng-click="selectPage(totalPages, $event)">{{::getText(\'last\')}}</a></li>\n' + '</ul>\n' + '');
  }
]);
angular.module('template/tooltip/tooltip-html-popup.html', []).run([
  '$templateCache',
  function ($templateCache) {
    $templateCache.put('template/tooltip/tooltip-html-popup.html', '<div\n' + '  tooltip-animation-class="fade"\n' + '  uib-tooltip-classes\n' + '  ng-class="{ in: isOpen() }">\n' + '  <div class="tooltip-arrow"></div>\n' + '  <div class="tooltip-inner" ng-bind-html="contentExp()"></div>\n' + '</div>\n' + '');
  }
]);
angular.module('template/tooltip/tooltip-html-unsafe-popup.html', []).run([
  '$templateCache',
  function ($templateCache) {
    $templateCache.put('template/tooltip/tooltip-html-unsafe-popup.html', '<div class="tooltip"\n' + '  tooltip-animation-class="fade"\n' + '  tooltip-classes\n' + '  ng-class="{ in: isOpen() }">\n' + '  <div class="tooltip-arrow"></div>\n' + '  <div class="tooltip-inner" bind-html-unsafe="content"></div>\n' + '</div>\n' + '');
  }
]);
angular.module('template/tooltip/tooltip-popup.html', []).run([
  '$templateCache',
  function ($templateCache) {
    $templateCache.put('template/tooltip/tooltip-popup.html', '<div\n' + '  tooltip-animation-class="fade"\n' + '  uib-tooltip-classes\n' + '  ng-class="{ in: isOpen() }">\n' + '  <div class="tooltip-arrow"></div>\n' + '  <div class="tooltip-inner" ng-bind="content"></div>\n' + '</div>\n' + '');
  }
]);
angular.module('template/tooltip/tooltip-template-popup.html', []).run([
  '$templateCache',
  function ($templateCache) {
    $templateCache.put('template/tooltip/tooltip-template-popup.html', '<div\n' + '  tooltip-animation-class="fade"\n' + '  uib-tooltip-classes\n' + '  ng-class="{ in: isOpen() }">\n' + '  <div class="tooltip-arrow"></div>\n' + '  <div class="tooltip-inner"\n' + '    uib-tooltip-template-transclude="contentExp()"\n' + '    tooltip-template-transclude-scope="originScope()"></div>\n' + '</div>\n' + '');
  }
]);
angular.module('template/popover/popover-html.html', []).run([
  '$templateCache',
  function ($templateCache) {
    $templateCache.put('template/popover/popover-html.html', '<div tooltip-animation-class="fade"\n' + '  uib-tooltip-classes\n' + '  ng-class="{ in: isOpen() }">\n' + '  <div class="arrow"></div>\n' + '\n' + '  <div class="popover-inner">\n' + '      <h3 class="popover-title" ng-bind="title" ng-if="title"></h3>\n' + '      <div class="popover-content" ng-bind-html="contentExp()"></div>\n' + '  </div>\n' + '</div>\n' + '');
  }
]);
angular.module('template/popover/popover-template.html', []).run([
  '$templateCache',
  function ($templateCache) {
    $templateCache.put('template/popover/popover-template.html', '<div tooltip-animation-class="fade"\n' + '  uib-tooltip-classes\n' + '  ng-class="{ in: isOpen() }">\n' + '  <div class="arrow"></div>\n' + '\n' + '  <div class="popover-inner">\n' + '      <h3 class="popover-title" ng-bind="title" ng-if="title"></h3>\n' + '      <div class="popover-content"\n' + '        uib-tooltip-template-transclude="contentExp()"\n' + '        tooltip-template-transclude-scope="originScope()"></div>\n' + '  </div>\n' + '</div>\n' + '');
  }
]);
angular.module('template/popover/popover.html', []).run([
  '$templateCache',
  function ($templateCache) {
    $templateCache.put('template/popover/popover.html', '<div tooltip-animation-class="fade"\n' + '  uib-tooltip-classes\n' + '  ng-class="{ in: isOpen() }">\n' + '  <div class="arrow"></div>\n' + '\n' + '  <div class="popover-inner">\n' + '      <h3 class="popover-title" ng-bind="title" ng-if="title"></h3>\n' + '      <div class="popover-content" ng-bind="content"></div>\n' + '  </div>\n' + '</div>\n' + '');
  }
]);
angular.module('template/progressbar/bar.html', []).run([
  '$templateCache',
  function ($templateCache) {
    $templateCache.put('template/progressbar/bar.html', '<div class="progress-bar" ng-class="type && \'progress-bar-\' + type" role="progressbar" aria-valuenow="{{value}}" aria-valuemin="0" aria-valuemax="{{max}}" ng-style="{width: (percent < 100 ? percent : 100) + \'%\'}" aria-valuetext="{{percent | number:0}}%" aria-labelledby="{{::title}}" style="min-width: 0;" ng-transclude></div>\n' + '');
  }
]);
angular.module('template/progressbar/progress.html', []).run([
  '$templateCache',
  function ($templateCache) {
    $templateCache.put('template/progressbar/progress.html', '<div class="progress" ng-transclude aria-labelledby="{{::title}}"></div>');
  }
]);
angular.module('template/progressbar/progressbar.html', []).run([
  '$templateCache',
  function ($templateCache) {
    $templateCache.put('template/progressbar/progressbar.html', '<div class="progress">\n' + '  <div class="progress-bar" ng-class="type && \'progress-bar-\' + type" role="progressbar" aria-valuenow="{{value}}" aria-valuemin="0" aria-valuemax="{{max}}" ng-style="{width: (percent < 100 ? percent : 100) + \'%\'}" aria-valuetext="{{percent | number:0}}%" aria-labelledby="{{::title}}" style="min-width: 0;" ng-transclude></div>\n' + '</div>\n' + '');
  }
]);
angular.module('template/rating/rating.html', []).run([
  '$templateCache',
  function ($templateCache) {
    $templateCache.put('template/rating/rating.html', '<span ng-mouseleave="reset()" ng-keydown="onKeydown($event)" tabindex="0" role="slider" aria-valuemin="0" aria-valuemax="{{range.length}}" aria-valuenow="{{value}}">\n' + '    <span ng-repeat-start="r in range track by $index" class="sr-only">({{ $index < value ? \'*\' : \' \' }})</span>\n' + '    <i ng-repeat-end ng-mouseenter="enter($index + 1)" ng-click="rate($index + 1)" class="glyphicon" ng-class="$index < value && (r.stateOn || \'glyphicon-star\') || (r.stateOff || \'glyphicon-star-empty\')" ng-attr-title="{{r.title}}" aria-valuetext="{{r.title}}"></i>\n' + '</span>\n' + '');
  }
]);
angular.module('template/tabs/tab.html', []).run([
  '$templateCache',
  function ($templateCache) {
    $templateCache.put('template/tabs/tab.html', '<li ng-class="{active: active, disabled: disabled}">\n' + '  <a href ng-click="select()" uib-tab-heading-transclude>{{heading}}</a>\n' + '</li>\n' + '');
  }
]);
angular.module('template/tabs/tabset.html', []).run([
  '$templateCache',
  function ($templateCache) {
    $templateCache.put('template/tabs/tabset.html', '<div>\n' + '  <ul class="nav nav-{{type || \'tabs\'}}" ng-class="{\'nav-stacked\': vertical, \'nav-justified\': justified}" ng-transclude></ul>\n' + '  <div class="tab-content">\n' + '    <div class="tab-pane" \n' + '         ng-repeat="tab in tabs" \n' + '         ng-class="{active: tab.active}"\n' + '         uib-tab-content-transclude="tab">\n' + '    </div>\n' + '  </div>\n' + '</div>\n' + '');
  }
]);
angular.module('template/timepicker/timepicker.html', []).run([
  '$templateCache',
  function ($templateCache) {
    $templateCache.put('template/timepicker/timepicker.html', '<table>\n' + '  <tbody>\n' + '    <tr class="text-center" ng-show="::showSpinners">\n' + '      <td><a ng-click="incrementHours()" ng-class="{disabled: noIncrementHours()}" class="btn btn-link"><span class="glyphicon glyphicon-chevron-up"></span></a></td>\n' + '      <td>&nbsp;</td>\n' + '      <td><a ng-click="incrementMinutes()" ng-class="{disabled: noIncrementMinutes()}" class="btn btn-link"><span class="glyphicon glyphicon-chevron-up"></span></a></td>\n' + '      <td ng-show="showMeridian"></td>\n' + '    </tr>\n' + '    <tr>\n' + '      <td class="form-group" ng-class="{\'has-error\': invalidHours}">\n' + '        <input style="width:50px;" type="text" ng-model="hours" ng-change="updateHours()" class="form-control text-center" ng-readonly="::readonlyInput" maxlength="2">\n' + '      </td>\n' + '      <td>:</td>\n' + '      <td class="form-group" ng-class="{\'has-error\': invalidMinutes}">\n' + '        <input style="width:50px;" type="text" ng-model="minutes" ng-change="updateMinutes()" class="form-control text-center" ng-readonly="::readonlyInput" maxlength="2">\n' + '      </td>\n' + '      <td ng-show="showMeridian"><button type="button" ng-class="{disabled: noToggleMeridian()}" class="btn btn-default text-center" ng-click="toggleMeridian()">{{meridian}}</button></td>\n' + '    </tr>\n' + '    <tr class="text-center" ng-show="::showSpinners">\n' + '      <td><a ng-click="decrementHours()" ng-class="{disabled: noDecrementHours()}" class="btn btn-link"><span class="glyphicon glyphicon-chevron-down"></span></a></td>\n' + '      <td>&nbsp;</td>\n' + '      <td><a ng-click="decrementMinutes()" ng-class="{disabled: noDecrementMinutes()}" class="btn btn-link"><span class="glyphicon glyphicon-chevron-down"></span></a></td>\n' + '      <td ng-show="showMeridian"></td>\n' + '    </tr>\n' + '  </tbody>\n' + '</table>\n' + '');
  }
]);
angular.module('template/typeahead/typeahead-match.html', []).run([
  '$templateCache',
  function ($templateCache) {
    $templateCache.put('template/typeahead/typeahead-match.html', '<a href tabindex="-1" ng-bind-html="match.label | uibTypeaheadHighlight:query"></a>\n' + '');
  }
]);
angular.module('template/typeahead/typeahead-popup.html', []).run([
  '$templateCache',
  function ($templateCache) {
    $templateCache.put('template/typeahead/typeahead-popup.html', '<ul class="dropdown-menu" ng-show="isOpen() && !moveInProgress" ng-style="{top: position().top+\'px\', left: position().left+\'px\'}" style="display: block;" role="listbox" aria-hidden="{{!isOpen()}}">\n' + '    <li ng-repeat="match in matches track by $index" ng-class="{active: isActive($index) }" ng-mouseenter="selectActive($index)" ng-click="selectMatch($index)" role="option" id="{{::match.id}}">\n' + '        <div uib-typeahead-match index="$index" match="match" query="query" template-url="templateUrl"></div>\n' + '    </li>\n' + '</ul>\n' + '');
  }
]);
!angular.$$csp() && angular.element(document).find('head').prepend('<style type="text/css">.ng-animate.item:not(.left):not(.right){-webkit-transition:0s ease-in-out left;transition:0s ease-in-out left}</style>');
/**
 * @preserve FastClick: polyfill to remove click delays on browsers with touch UIs.
 *
 * @version 0.6.9
 * @codingstandard ftlabs-jsv2
 * @copyright The Financial Times Limited [All Rights Reserved]
 * @license MIT License (see LICENSE.txt)
 */
/*jslint browser:true, node:true*/
/*global define, Event, Node*/
/**
 * Instantiate fast-clicking listeners on the specificed layer.
 *
 * @constructor
 * @param {Element} layer The layer to listen on
 */
function FastClick(layer) {
  'use strict';
  var oldOnClick, self = this;
  /**
	 * Whether a click is currently being tracked.
	 *
	 * @type boolean
	 */
  this.trackingClick = false;
  /**
	 * Timestamp for when when click tracking started.
	 *
	 * @type number
	 */
  this.trackingClickStart = 0;
  /**
	 * The element being tracked for a click.
	 *
	 * @type EventTarget
	 */
  this.targetElement = null;
  /**
	 * X-coordinate of touch start event.
	 *
	 * @type number
	 */
  this.touchStartX = 0;
  /**
	 * Y-coordinate of touch start event.
	 *
	 * @type number
	 */
  this.touchStartY = 0;
  /**
	 * ID of the last touch, retrieved from Touch.identifier.
	 *
	 * @type number
	 */
  this.lastTouchIdentifier = 0;
  /**
	 * Touchmove boundary, beyond which a click will be cancelled.
	 *
	 * @type number
	 */
  this.touchBoundary = 10;
  /**
	 * The FastClick layer.
	 *
	 * @type Element
	 */
  this.layer = layer;
  if (!layer || !layer.nodeType) {
    throw new TypeError('Layer must be a document node');
  }
  /** @type function() */
  this.onClick = function () {
    return FastClick.prototype.onClick.apply(self, arguments);
  };
  /** @type function() */
  this.onMouse = function () {
    return FastClick.prototype.onMouse.apply(self, arguments);
  };
  /** @type function() */
  this.onTouchStart = function () {
    return FastClick.prototype.onTouchStart.apply(self, arguments);
  };
  /** @type function() */
  this.onTouchMove = function () {
    return FastClick.prototype.onTouchMove.apply(self, arguments);
  };
  /** @type function() */
  this.onTouchEnd = function () {
    return FastClick.prototype.onTouchEnd.apply(self, arguments);
  };
  /** @type function() */
  this.onTouchCancel = function () {
    return FastClick.prototype.onTouchCancel.apply(self, arguments);
  };
  if (FastClick.notNeeded(layer)) {
    return;
  }
  // Set up event handlers as required
  if (this.deviceIsAndroid) {
    layer.addEventListener('mouseover', this.onMouse, true);
    layer.addEventListener('mousedown', this.onMouse, true);
    layer.addEventListener('mouseup', this.onMouse, true);
  }
  layer.addEventListener('click', this.onClick, true);
  layer.addEventListener('touchstart', this.onTouchStart, false);
  layer.addEventListener('touchmove', this.onTouchMove, false);
  layer.addEventListener('touchend', this.onTouchEnd, false);
  layer.addEventListener('touchcancel', this.onTouchCancel, false);
  // Hack is required for browsers that don't support Event#stopImmediatePropagation (e.g. Android 2)
  // which is how FastClick normally stops click events bubbling to callbacks registered on the FastClick
  // layer when they are cancelled.
  if (!Event.prototype.stopImmediatePropagation) {
    layer.removeEventListener = function (type, callback, capture) {
      var rmv = Node.prototype.removeEventListener;
      if (type === 'click') {
        rmv.call(layer, type, callback.hijacked || callback, capture);
      } else {
        rmv.call(layer, type, callback, capture);
      }
    };
    layer.addEventListener = function (type, callback, capture) {
      var adv = Node.prototype.addEventListener;
      if (type === 'click') {
        adv.call(layer, type, callback.hijacked || (callback.hijacked = function (event) {
          if (!event.propagationStopped) {
            callback(event);
          }
        }), capture);
      } else {
        adv.call(layer, type, callback, capture);
      }
    };
  }
  // If a handler is already declared in the element's onclick attribute, it will be fired before
  // FastClick's onClick handler. Fix this by pulling out the user-defined handler function and
  // adding it as listener.
  if (typeof layer.onclick === 'function') {
    // Android browser on at least 3.2 requires a new reference to the function in layer.onclick
    // - the old one won't work if passed to addEventListener directly.
    oldOnClick = layer.onclick;
    layer.addEventListener('click', function (event) {
      oldOnClick(event);
    }, false);
    layer.onclick = null;
  }
}
/**
 * Android requires exceptions.
 *
 * @type boolean
 */
FastClick.prototype.deviceIsAndroid = navigator.userAgent.indexOf('Android') > 0;
/**
 * iOS requires exceptions.
 *
 * @type boolean
 */
FastClick.prototype.deviceIsIOS = /iP(ad|hone|od)/.test(navigator.userAgent);
/**
 * iOS 4 requires an exception for select elements.
 *
 * @type boolean
 */
FastClick.prototype.deviceIsIOS4 = FastClick.prototype.deviceIsIOS && /OS 4_\d(_\d)?/.test(navigator.userAgent);
/**
 * iOS 6.0(+?) requires the target element to be manually derived
 *
 * @type boolean
 */
FastClick.prototype.deviceIsIOSWithBadTarget = FastClick.prototype.deviceIsIOS && /OS ([6-9]|\d{2})_\d/.test(navigator.userAgent);
/**
 * Determine whether a given element requires a native click.
 *
 * @param {EventTarget|Element} target Target DOM element
 * @returns {boolean} Returns true if the element needs a native click
 */
FastClick.prototype.needsClick = function (target) {
  'use strict';
  switch (target.nodeName.toLowerCase()) {
  // Don't send a synthetic click to disabled inputs (issue #62)
  case 'button':
  case 'select':
  case 'textarea':
    if (target.disabled) {
      return true;
    }
    break;
  case 'input':
    // File inputs need real clicks on iOS 6 due to a browser bug (issue #68)
    if (this.deviceIsIOS && target.type === 'file' || target.disabled) {
      return true;
    }
    break;
  case 'label':
  case 'video':
    return true;
  }
  return /\bneedsclick\b/.test(target.className);
};
/**
 * Determine whether a given element requires a call to focus to simulate click into element.
 *
 * @param {EventTarget|Element} target Target DOM element
 * @returns {boolean} Returns true if the element requires a call to focus to simulate native click.
 */
FastClick.prototype.needsFocus = function (target) {
  'use strict';
  switch (target.nodeName.toLowerCase()) {
  case 'textarea':
  case 'select':
    return true;
  case 'input':
    switch (target.type) {
    case 'button':
    case 'checkbox':
    case 'file':
    case 'image':
    case 'radio':
    case 'submit':
      return false;
    }
    // No point in attempting to focus disabled inputs
    return !target.disabled && !target.readOnly;
  default:
    return /\bneedsfocus\b/.test(target.className);
  }
};
/**
 * Send a click event to the specified element.
 *
 * @param {EventTarget|Element} targetElement
 * @param {Event} event
 */
FastClick.prototype.sendClick = function (targetElement, event) {
  'use strict';
  var clickEvent, touch;
  // On some Android devices activeElement needs to be blurred otherwise the synthetic click will have no effect (#24)
  if (document.activeElement && document.activeElement !== targetElement) {
    document.activeElement.blur();
  }
  touch = event.changedTouches[0];
  // Synthesise a click event, with an extra attribute so it can be tracked
  clickEvent = document.createEvent('MouseEvents');
  clickEvent.initMouseEvent('click', true, true, window, 1, touch.screenX, touch.screenY, touch.clientX, touch.clientY, false, false, false, false, 0, null);
  clickEvent.forwardedTouchEvent = true;
  targetElement.dispatchEvent(clickEvent);
};
/**
 * @param {EventTarget|Element} targetElement
 */
FastClick.prototype.focus = function (targetElement) {
  'use strict';
  var length;
  if (this.deviceIsIOS && targetElement.setSelectionRange) {
    length = targetElement.value.length;
    targetElement.setSelectionRange(length, length);
  } else {
    targetElement.focus();
  }
};
/**
 * Check whether the given target element is a child of a scrollable layer and if so, set a flag on it.
 *
 * @param {EventTarget|Element} targetElement
 */
FastClick.prototype.updateScrollParent = function (targetElement) {
  'use strict';
  var scrollParent, parentElement;
  scrollParent = targetElement.fastClickScrollParent;
  // Attempt to discover whether the target element is contained within a scrollable layer. Re-check if the
  // target element was moved to another parent.
  if (!scrollParent || !scrollParent.contains(targetElement)) {
    parentElement = targetElement;
    do {
      if (parentElement.scrollHeight > parentElement.offsetHeight) {
        scrollParent = parentElement;
        targetElement.fastClickScrollParent = parentElement;
        break;
      }
      parentElement = parentElement.parentElement;
    } while (parentElement);
  }
  // Always update the scroll top tracker if possible.
  if (scrollParent) {
    scrollParent.fastClickLastScrollTop = scrollParent.scrollTop;
  }
};
/**
 * @param {EventTarget} targetElement
 * @returns {Element|EventTarget}
 */
FastClick.prototype.getTargetElementFromEventTarget = function (eventTarget) {
  'use strict';
  // On some older browsers (notably Safari on iOS 4.1 - see issue #56) the event target may be a text node.
  if (eventTarget.nodeType === Node.TEXT_NODE) {
    return eventTarget.parentNode;
  }
  return eventTarget;
};
/**
 * On touch start, record the position and scroll offset.
 *
 * @param {Event} event
 * @returns {boolean}
 */
FastClick.prototype.onTouchStart = function (event) {
  'use strict';
  var targetElement, touch, selection;
  // Ignore multiple touches, otherwise pinch-to-zoom is prevented if both fingers are on the FastClick element (issue #111).
  if (event.targetTouches.length > 1) {
    return true;
  }
  targetElement = this.getTargetElementFromEventTarget(event.target);
  touch = event.targetTouches[0];
  if (this.deviceIsIOS) {
    // Only trusted events will deselect text on iOS (issue #49)
    selection = window.getSelection();
    if (selection.rangeCount && !selection.isCollapsed) {
      return true;
    }
    if (!this.deviceIsIOS4) {
      // Weird things happen on iOS when an alert or confirm dialog is opened from a click event callback (issue #23):
      // when the user next taps anywhere else on the page, new touchstart and touchend events are dispatched
      // with the same identifier as the touch event that previously triggered the click that triggered the alert.
      // Sadly, there is an issue on iOS 4 that causes some normal touch events to have the same identifier as an
      // immediately preceeding touch event (issue #52), so this fix is unavailable on that platform.
      if (touch.identifier === this.lastTouchIdentifier) {
        event.preventDefault();
        return false;
      }
      this.lastTouchIdentifier = touch.identifier;
      // If the target element is a child of a scrollable layer (using -webkit-overflow-scrolling: touch) and:
      // 1) the user does a fling scroll on the scrollable layer
      // 2) the user stops the fling scroll with another tap
      // then the event.target of the last 'touchend' event will be the element that was under the user's finger
      // when the fling scroll was started, causing FastClick to send a click event to that layer - unless a check
      // is made to ensure that a parent layer was not scrolled before sending a synthetic click (issue #42).
      this.updateScrollParent(targetElement);
    }
  }
  this.trackingClick = true;
  this.trackingClickStart = event.timeStamp;
  this.targetElement = targetElement;
  this.touchStartX = touch.pageX;
  this.touchStartY = touch.pageY;
  // Prevent phantom clicks on fast double-tap (issue #36)
  if (event.timeStamp - this.lastClickTime < 200) {
    event.preventDefault();
  }
  return true;
};
/**
 * Based on a touchmove event object, check whether the touch has moved past a boundary since it started.
 *
 * @param {Event} event
 * @returns {boolean}
 */
FastClick.prototype.touchHasMoved = function (event) {
  'use strict';
  var touch = event.changedTouches[0], boundary = this.touchBoundary;
  if (Math.abs(touch.pageX - this.touchStartX) > boundary || Math.abs(touch.pageY - this.touchStartY) > boundary) {
    return true;
  }
  return false;
};
/**
 * Update the last position.
 *
 * @param {Event} event
 * @returns {boolean}
 */
FastClick.prototype.onTouchMove = function (event) {
  'use strict';
  if (!this.trackingClick) {
    return true;
  }
  // If the touch has moved, cancel the click tracking
  if (this.targetElement !== this.getTargetElementFromEventTarget(event.target) || this.touchHasMoved(event)) {
    this.trackingClick = false;
    this.targetElement = null;
  }
  return true;
};
/**
 * Attempt to find the labelled control for the given label element.
 *
 * @param {EventTarget|HTMLLabelElement} labelElement
 * @returns {Element|null}
 */
FastClick.prototype.findControl = function (labelElement) {
  'use strict';
  // Fast path for newer browsers supporting the HTML5 control attribute
  if (labelElement.control !== undefined) {
    return labelElement.control;
  }
  // All browsers under test that support touch events also support the HTML5 htmlFor attribute
  if (labelElement.htmlFor) {
    return document.getElementById(labelElement.htmlFor);
  }
  // If no for attribute exists, attempt to retrieve the first labellable descendant element
  // the list of which is defined here: http://www.w3.org/TR/html5/forms.html#category-label
  return labelElement.querySelector('button, input:not([type=hidden]), keygen, meter, output, progress, select, textarea');
};
/**
 * On touch end, determine whether to send a click event at once.
 *
 * @param {Event} event
 * @returns {boolean}
 */
FastClick.prototype.onTouchEnd = function (event) {
  'use strict';
  var forElement, trackingClickStart, targetTagName, scrollParent, touch, targetElement = this.targetElement;
  if (!this.trackingClick) {
    return true;
  }
  // Prevent phantom clicks on fast double-tap (issue #36)
  if (event.timeStamp - this.lastClickTime < 200) {
    this.cancelNextClick = true;
    return true;
  }
  this.lastClickTime = event.timeStamp;
  trackingClickStart = this.trackingClickStart;
  this.trackingClick = false;
  this.trackingClickStart = 0;
  // On some iOS devices, the targetElement supplied with the event is invalid if the layer
  // is performing a transition or scroll, and has to be re-detected manually. Note that
  // for this to function correctly, it must be called *after* the event target is checked!
  // See issue #57; also filed as rdar://13048589 .
  if (this.deviceIsIOSWithBadTarget) {
    touch = event.changedTouches[0];
    // In certain cases arguments of elementFromPoint can be negative, so prevent setting targetElement to null
    targetElement = document.elementFromPoint(touch.pageX - window.pageXOffset, touch.pageY - window.pageYOffset) || targetElement;
    targetElement.fastClickScrollParent = this.targetElement.fastClickScrollParent;
  }
  targetTagName = targetElement.tagName.toLowerCase();
  if (targetTagName === 'label') {
    forElement = this.findControl(targetElement);
    if (forElement) {
      this.focus(targetElement);
      if (this.deviceIsAndroid) {
        return false;
      }
      targetElement = forElement;
    }
  } else if (this.needsFocus(targetElement)) {
    // Case 1: If the touch started a while ago (best guess is 100ms based on tests for issue #36) then focus will be triggered anyway. Return early and unset the target element reference so that the subsequent click will be allowed through.
    // Case 2: Without this exception for input elements tapped when the document is contained in an iframe, then any inputted text won't be visible even though the value attribute is updated as the user types (issue #37).
    if (event.timeStamp - trackingClickStart > 100 || this.deviceIsIOS && window.top !== window && targetTagName === 'input') {
      this.targetElement = null;
      return false;
    }
    this.focus(targetElement);
    // Select elements need the event to go through on iOS 4, otherwise the selector menu won't open.
    if (!this.deviceIsIOS4 || targetTagName !== 'select') {
      this.targetElement = null;
      event.preventDefault();
    }
    return false;
  }
  if (this.deviceIsIOS && !this.deviceIsIOS4) {
    // Don't send a synthetic click event if the target element is contained within a parent layer that was scrolled
    // and this tap is being used to stop the scrolling (usually initiated by a fling - issue #42).
    scrollParent = targetElement.fastClickScrollParent;
    if (scrollParent && scrollParent.fastClickLastScrollTop !== scrollParent.scrollTop) {
      return true;
    }
  }
  // Prevent the actual click from going though - unless the target node is marked as requiring
  // real clicks or if it is in the whitelist in which case only non-programmatic clicks are permitted.
  if (!this.needsClick(targetElement)) {
    event.preventDefault();
    this.sendClick(targetElement, event);
  }
  return false;
};
/**
 * On touch cancel, stop tracking the click.
 *
 * @returns {void}
 */
FastClick.prototype.onTouchCancel = function () {
  'use strict';
  this.trackingClick = false;
  this.targetElement = null;
};
/**
 * Determine mouse events which should be permitted.
 *
 * @param {Event} event
 * @returns {boolean}
 */
FastClick.prototype.onMouse = function (event) {
  'use strict';
  // If a target element was never set (because a touch event was never fired) allow the event
  if (!this.targetElement) {
    return true;
  }
  if (event.forwardedTouchEvent) {
    return true;
  }
  // Programmatically generated events targeting a specific element should be permitted
  if (!event.cancelable) {
    return true;
  }
  // Derive and check the target element to see whether the mouse event needs to be permitted;
  // unless explicitly enabled, prevent non-touch click events from triggering actions,
  // to prevent ghost/doubleclicks.
  if (!this.needsClick(this.targetElement) || this.cancelNextClick) {
    // Prevent any user-added listeners declared on FastClick element from being fired.
    if (event.stopImmediatePropagation) {
      event.stopImmediatePropagation();
    } else {
      // Part of the hack for browsers that don't support Event#stopImmediatePropagation (e.g. Android 2)
      event.propagationStopped = true;
    }
    // Cancel the event
    event.stopPropagation();
    event.preventDefault();
    return false;
  }
  // If the mouse event is permitted, return true for the action to go through.
  return true;
};
/**
 * On actual clicks, determine whether this is a touch-generated click, a click action occurring
 * naturally after a delay after a touch (which needs to be cancelled to avoid duplication), or
 * an actual click which should be permitted.
 *
 * @param {Event} event
 * @returns {boolean}
 */
FastClick.prototype.onClick = function (event) {
  'use strict';
  var permitted;
  // It's possible for another FastClick-like library delivered with third-party code to fire a click event before FastClick does (issue #44). In that case, set the click-tracking flag back to false and return early. This will cause onTouchEnd to return early.
  if (this.trackingClick) {
    this.targetElement = null;
    this.trackingClick = false;
    return true;
  }
  // Very odd behaviour on iOS (issue #18): if a submit element is present inside a form and the user hits enter in the iOS simulator or clicks the Go button on the pop-up OS keyboard the a kind of 'fake' click event will be triggered with the submit-type input element as the target.
  if (event.target.type === 'submit' && event.detail === 0) {
    return true;
  }
  permitted = this.onMouse(event);
  // Only unset targetElement if the click is not permitted. This will ensure that the check for !targetElement in onMouse fails and the browser's click doesn't go through.
  if (!permitted) {
    this.targetElement = null;
  }
  // If clicks are permitted, return true for the action to go through.
  return permitted;
};
/**
 * Remove all FastClick's event listeners.
 *
 * @returns {void}
 */
FastClick.prototype.destroy = function () {
  'use strict';
  var layer = this.layer;
  if (this.deviceIsAndroid) {
    layer.removeEventListener('mouseover', this.onMouse, true);
    layer.removeEventListener('mousedown', this.onMouse, true);
    layer.removeEventListener('mouseup', this.onMouse, true);
  }
  layer.removeEventListener('click', this.onClick, true);
  layer.removeEventListener('touchstart', this.onTouchStart, false);
  layer.removeEventListener('touchmove', this.onTouchMove, false);
  layer.removeEventListener('touchend', this.onTouchEnd, false);
  layer.removeEventListener('touchcancel', this.onTouchCancel, false);
};
/**
 * Check whether FastClick is needed.
 *
 * @param {Element} layer The layer to listen on
 */
FastClick.notNeeded = function (layer) {
  'use strict';
  var metaViewport;
  // Devices that don't support touch don't need FastClick
  if (typeof window.ontouchstart === 'undefined') {
    return true;
  }
  if (/Chrome\/[0-9]+/.test(navigator.userAgent)) {
    // Chrome on Android with user-scalable="no" doesn't need FastClick (issue #89)
    if (FastClick.prototype.deviceIsAndroid) {
      metaViewport = document.querySelector('meta[name=viewport]');
      if (metaViewport && metaViewport.content.indexOf('user-scalable=no') !== -1) {
        return true;
      }  // Chrome desktop doesn't need FastClick (issue #15)
    } else {
      return true;
    }
  }
  // IE10 with -ms-touch-action: none, which disables double-tap-to-zoom (issue #97)
  if (layer.style.msTouchAction === 'none') {
    return true;
  }
  return false;
};
/**
 * Factory method for creating a FastClick object
 *
 * @param {Element} layer The layer to listen on
 */
FastClick.attach = function (layer) {
  'use strict';
  return new FastClick(layer);
};
if (typeof define !== 'undefined' && define.amd) {
  // AMD. Register as an anonymous module.
  define(function () {
    'use strict';
    return FastClick;
  });
} else if (typeof module !== 'undefined' && module.exports) {
  module.exports = FastClick.attach;
  module.exports.FastClick = FastClick;
} else {
  window.FastClick = FastClick;
}
/*

 Software License Agreement (BSD License)
 http://taffydb.com
 Copyright (c)
 All rights reserved.


 Redistribution and use of this software in source and binary forms, with or without modification, are permitted provided that the following condition is met:

 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 */
/*jslint        browser : true, continue : true,
 devel  : true, indent  : 2,    maxerr   : 500,
 newcap : true, nomen   : true, plusplus : true,
 regexp : true, sloppy  : true, vars     : false,
 white  : true
 */
// BUILD 193d48d, modified by mmikowski to pass jslint
// Setup TAFFY name space to return an object with methods
var TAFFY, exports, T;
(function () {
  'use strict';
  var typeList, makeTest, idx, typeKey, version, TC, idpad, cmax, API, protectJSON, each, eachin, isIndexable, returnFilter, runFilters, numcharsplit, orderByCol, run, intersection, filter, makeCid, safeForJson, isRegexp, sortArgs;
  ;
  if (!TAFFY) {
    // TC = Counter for Taffy DBs on page, used for unique IDs
    // cmax = size of charnumarray conversion cache
    // idpad = zeros to pad record IDs with
    version = '2.7';
    TC = 1;
    idpad = '000000';
    cmax = 1000;
    API = {};
    sortArgs = function (args) {
      var v = Array.prototype.slice.call(args);
      return v.sort();
    };
    protectJSON = function (t) {
      // ****************************************
      // *
      // * Takes: a variable
      // * Returns: the variable if object/array or the parsed variable if JSON
      // *
      // ****************************************
      if (TAFFY.isArray(t) || TAFFY.isObject(t)) {
        return t;
      } else {
        return JSON.parse(t);
      }
    };
    // gracefully stolen from underscore.js
    intersection = function (array1, array2) {
      return filter(array1, function (item) {
        return array2.indexOf(item) >= 0;
      });
    };
    // gracefully stolen from underscore.js
    filter = function (obj, iterator, context) {
      var results = [];
      if (obj == null)
        return results;
      if (Array.prototype.filter && obj.filter === Array.prototype.filter)
        return obj.filter(iterator, context);
      each(obj, function (value, index, list) {
        if (iterator.call(context, value, index, list))
          results[results.length] = value;
      });
      return results;
    };
    isRegexp = function (aObj) {
      return Object.prototype.toString.call(aObj) === '[object RegExp]';
    };
    safeForJson = function (aObj) {
      var myResult = T.isArray(aObj) ? [] : T.isObject(aObj) ? {} : null;
      if (aObj === null)
        return aObj;
      for (var i in aObj) {
        myResult[i] = isRegexp(aObj[i]) ? aObj[i].toString() : T.isArray(aObj[i]) || T.isObject(aObj[i]) ? safeForJson(aObj[i]) : aObj[i];
      }
      return myResult;
    };
    makeCid = function (aContext) {
      var myCid = JSON.stringify(aContext);
      if (myCid.match(/regex/) === null)
        return myCid;
      return JSON.stringify(safeForJson(aContext));
    };
    each = function (a, fun, u) {
      var r, i, x, y;
      // ****************************************
      // *
      // * Takes:
      // * a = an object/value or an array of objects/values
      // * f = a function
      // * u = optional flag to describe how to handle undefined values
      //   in array of values. True: pass them to the functions,
      //   False: skip. Default False;
      // * Purpose: Used to loop over arrays
      // *
      // ****************************************
      if (a && (T.isArray(a) && a.length === 1 || !T.isArray(a))) {
        fun(T.isArray(a) ? a[0] : a, 0);
      } else {
        for (r, i, x = 0, a = T.isArray(a) ? a : [a], y = a.length; x < y; x++) {
          i = a[x];
          if (!T.isUndefined(i) || (u || false)) {
            r = fun(i, x);
            if (r === T.EXIT) {
              break;
            }
          }
        }
      }
    };
    eachin = function (o, fun) {
      // ****************************************
      // *
      // * Takes:
      // * o = an object
      // * f = a function
      // * Purpose: Used to loop over objects
      // *
      // ****************************************
      var x = 0, r, i;
      for (i in o) {
        if (o.hasOwnProperty(i)) {
          r = fun(o[i], i, x++);
          if (r === T.EXIT) {
            break;
          }
        }
      }
    };
    API.extend = function (m, f) {
      // ****************************************
      // *
      // * Takes: method name, function
      // * Purpose: Add a custom method to the API
      // *
      // ****************************************
      API[m] = function () {
        return f.apply(this, sortArgs(arguments));
      };
    };
    isIndexable = function (f) {
      var i;
      // Check to see if record ID
      if (T.isString(f) && /[t][0-9]*[r][0-9]*/i.test(f)) {
        return true;
      }
      // Check to see if record
      if (T.isObject(f) && f.___id && f.___s) {
        return true;
      }
      // Check to see if array of indexes
      if (T.isArray(f)) {
        i = true;
        each(f, function (r) {
          if (!isIndexable(r)) {
            i = false;
            return TAFFY.EXIT;
          }
        });
        return i;
      }
      return false;
    };
    runFilters = function (r, filter) {
      // ****************************************
      // *
      // * Takes: takes a record and a collection of filters
      // * Returns: true if the record matches, false otherwise
      // ****************************************
      var match = true;
      each(filter, function (mf) {
        switch (T.typeOf(mf)) {
        case 'function':
          // run function
          if (!mf.apply(r)) {
            match = false;
            return TAFFY.EXIT;
          }
          break;
        case 'array':
          // loop array and treat like a SQL or
          match = mf.length === 1 ? runFilters(r, mf[0]) : mf.length === 2 ? runFilters(r, mf[0]) || runFilters(r, mf[1]) : mf.length === 3 ? runFilters(r, mf[0]) || runFilters(r, mf[1]) || runFilters(r, mf[2]) : mf.length === 4 ? runFilters(r, mf[0]) || runFilters(r, mf[1]) || runFilters(r, mf[2]) || runFilters(r, mf[3]) : false;
          if (mf.length > 4) {
            each(mf, function (f) {
              if (runFilters(r, f)) {
                match = true;
              }
            });
          }
          break;
        }
      });
      return match;
    };
    returnFilter = function (f) {
      // ****************************************
      // *
      // * Takes: filter object
      // * Returns: a filter function
      // * Purpose: Take a filter object and return a function that can be used to compare
      // * a TaffyDB record to see if the record matches a query
      // ****************************************
      var nf = [];
      if (T.isString(f) && /[t][0-9]*[r][0-9]*/i.test(f)) {
        f = { ___id: f };
      }
      if (T.isArray(f)) {
        // if we are working with an array
        each(f, function (r) {
          // loop the array and return a filter func for each value
          nf.push(returnFilter(r));
        });
        // now build a func to loop over the filters and return true if ANY of the filters match
        // This handles logical OR expressions
        f = function () {
          var that = this, match = false;
          each(nf, function (f) {
            if (runFilters(that, f)) {
              match = true;
            }
          });
          return match;
        };
        return f;
      }
      // if we are dealing with an Object
      if (T.isObject(f)) {
        if (T.isObject(f) && f.___id && f.___s) {
          f = { ___id: f.___id };
        }
        // Loop over each value on the object to prep match type and match value
        eachin(f, function (v, i) {
          // default match type to IS/Equals
          if (!T.isObject(v)) {
            v = { 'is': v };
          }
          // loop over each value on the value object  - if any
          eachin(v, function (mtest, s) {
            // s = match type, e.g. is, hasAll, like, etc
            var c = [], looper;
            // function to loop and apply filter
            looper = s === 'hasAll' ? function (mtest, func) {
              func(mtest);
            } : each;
            // loop over each test
            looper(mtest, function (mtest) {
              // su = match success
              // f = match false
              var su = true, f = false, matchFunc;
              // push a function onto the filter collection to do the matching
              matchFunc = function () {
                // get the value from the record
                var mvalue = this[i], eqeq = '==', bangeq = '!=', eqeqeq = '===', lt = '<', gt = '>', lteq = '<=', gteq = '>=', bangeqeq = '!==', r;
                ;
                if (typeof mvalue === 'undefined') {
                  return false;
                }
                if (s.indexOf('!') === 0 && s !== bangeq && s !== bangeqeq) {
                  // if the filter name starts with ! as in '!is' then reverse the match logic and remove the !
                  su = false;
                  s = s.substring(1, s.length);
                }
                // get the match results based on the s/match type
                /*jslint eqeq : true */
                r = s === 'regex' ? mtest.test(mvalue) : s === 'lt' || s === lt ? mvalue < mtest : s === 'gt' || s === gt ? mvalue > mtest : s === 'lte' || s === lteq ? mvalue <= mtest : s === 'gte' || s === gteq ? mvalue >= mtest : s === 'left' ? mvalue.indexOf(mtest) === 0 : s === 'leftnocase' ? mvalue.toLowerCase().indexOf(mtest.toLowerCase()) === 0 : s === 'right' ? mvalue.substring(mvalue.length - mtest.length) === mtest : s === 'rightnocase' ? mvalue.toLowerCase().substring(mvalue.length - mtest.length) === mtest.toLowerCase() : s === 'like' ? mvalue.indexOf(mtest) >= 0 : s === 'likenocase' ? mvalue.toLowerCase().indexOf(mtest.toLowerCase()) >= 0 : s === eqeqeq || s === 'is' ? mvalue === mtest : s === eqeq ? mvalue == mtest : s === bangeqeq ? mvalue !== mtest : s === bangeq ? mvalue != mtest : s === 'isnocase' ? mvalue.toLowerCase ? mvalue.toLowerCase() === mtest.toLowerCase() : mvalue === mtest : s === 'has' ? T.has(mvalue, mtest) : s === 'hasall' ? T.hasAll(mvalue, mtest) : s === 'contains' ? TAFFY.isArray(mvalue) && mvalue.indexOf(mtest) > -1 : s.indexOf('is') === -1 && !TAFFY.isNull(mvalue) && !TAFFY.isUndefined(mvalue) && !TAFFY.isObject(mtest) && !TAFFY.isArray(mtest) ? mtest === mvalue[s] : T[s] && T.isFunction(T[s]) && s.indexOf('is') === 0 ? T[s](mvalue) === mtest : T[s] && T.isFunction(T[s]) ? T[s](mvalue, mtest) : false;
                /*jslint eqeq : false */
                r = r && !su ? false : !r && !su ? true : r;
                return r;
              };
              c.push(matchFunc);
            });
            // if only one filter in the collection push it onto the filter list without the array
            if (c.length === 1) {
              nf.push(c[0]);
            } else {
              // else build a function to loop over all the filters and return true only if ALL match
              // this is a logical AND
              nf.push(function () {
                var that = this, match = false;
                each(c, function (f) {
                  if (f.apply(that)) {
                    match = true;
                  }
                });
                return match;
              });
            }
          });
        });
        // finally return a single function that wraps all the other functions and will run a query
        // where all functions have to return true for a record to appear in a query result
        f = function () {
          var that = this, match = true;
          // faster if less than  4 functions
          match = nf.length === 1 && !nf[0].apply(that) ? false : nf.length === 2 && (!nf[0].apply(that) || !nf[1].apply(that)) ? false : nf.length === 3 && (!nf[0].apply(that) || !nf[1].apply(that) || !nf[2].apply(that)) ? false : nf.length === 4 && (!nf[0].apply(that) || !nf[1].apply(that) || !nf[2].apply(that) || !nf[3].apply(that)) ? false : true;
          if (nf.length > 4) {
            each(nf, function (f) {
              if (!runFilters(that, f)) {
                match = false;
              }
            });
          }
          return match;
        };
        return f;
      }
      // if function
      if (T.isFunction(f)) {
        return f;
      }
    };
    orderByCol = function (ar, o) {
      // ****************************************
      // *
      // * Takes: takes an array and a sort object
      // * Returns: the array sorted
      // * Purpose: Accept filters such as "[col], [col2]" or "[col] desc" and sort on those columns
      // *
      // ****************************************
      var sortFunc = function (a, b) {
        // function to pass to the native array.sort to sort an array
        var r = 0;
        T.each(o, function (sd) {
          // loop over the sort instructions
          // get the column name
          var o, col, dir, c, d;
          o = sd.split(' ');
          col = o[0];
          // get the direction
          dir = o.length === 1 ? 'logical' : o[1];
          if (dir === 'logical') {
            // if dir is logical than grab the charnum arrays for the two values we are looking at
            c = numcharsplit(a[col]);
            d = numcharsplit(b[col]);
            // loop over the charnumarrays until one value is higher than the other
            T.each(c.length <= d.length ? c : d, function (x, i) {
              if (c[i] < d[i]) {
                r = -1;
                return TAFFY.EXIT;
              } else if (c[i] > d[i]) {
                r = 1;
                return TAFFY.EXIT;
              }
            });
          } else if (dir === 'logicaldesc') {
            // if logicaldesc than grab the charnum arrays for the two values we are looking at
            c = numcharsplit(a[col]);
            d = numcharsplit(b[col]);
            // loop over the charnumarrays until one value is lower than the other
            T.each(c.length <= d.length ? c : d, function (x, i) {
              if (c[i] > d[i]) {
                r = -1;
                return TAFFY.EXIT;
              } else if (c[i] < d[i]) {
                r = 1;
                return TAFFY.EXIT;
              }
            });
          } else if (dir === 'asec' && a[col] < b[col]) {
            // if asec - default - check to see which is higher
            r = -1;
            return T.EXIT;
          } else if (dir === 'asec' && a[col] > b[col]) {
            // if asec - default - check to see which is higher
            r = 1;
            return T.EXIT;
          } else if (dir === 'desc' && a[col] > b[col]) {
            // if desc check to see which is lower
            r = -1;
            return T.EXIT;
          } else if (dir === 'desc' && a[col] < b[col]) {
            // if desc check to see which is lower
            r = 1;
            return T.EXIT;
          }
          // if r is still 0 and we are doing a logical sort than look to see if one array is longer than the other
          if (r === 0 && dir === 'logical' && c.length < d.length) {
            r = -1;
          } else if (r === 0 && dir === 'logical' && c.length > d.length) {
            r = 1;
          } else if (r === 0 && dir === 'logicaldesc' && c.length > d.length) {
            r = -1;
          } else if (r === 0 && dir === 'logicaldesc' && c.length < d.length) {
            r = 1;
          }
          if (r !== 0) {
            return T.EXIT;
          }
        });
        return r;
      };
      // call the sort function and return the newly sorted array
      return ar && ar.push ? ar.sort(sortFunc) : ar;
    };
    // ****************************************
    // *
    // * Takes: a string containing numbers and letters and turn it into an array
    // * Returns: return an array of numbers and letters
    // * Purpose: Used for logical sorting. String Example: 12ABC results: [12,'ABC']
    // ****************************************
    (function () {
      // creates a cache for numchar conversions
      var cache = {}, cachcounter = 0;
      // creates the numcharsplit function
      numcharsplit = function (thing) {
        // if over 1000 items exist in the cache, clear it and start over
        if (cachcounter > cmax) {
          cache = {};
          cachcounter = 0;
        }
        // if a cache can be found for a numchar then return its array value
        return cache['_' + thing] || function () {
          // otherwise do the conversion
          // make sure it is a string and setup so other variables
          var nthing = String(thing), na = [], rv = '_', rt = '', x, xx, c;
          // loop over the string char by char
          for (x = 0, xx = nthing.length; x < xx; x++) {
            // take the char at each location
            c = nthing.charCodeAt(x);
            // check to see if it is a valid number char and append it to the array.
            // if last char was a string push the string to the charnum array
            if (c >= 48 && c <= 57 || c === 46) {
              if (rt !== 'n') {
                rt = 'n';
                na.push(rv.toLowerCase());
                rv = '';
              }
              rv = rv + nthing.charAt(x);
            } else {
              // check to see if it is a valid string char and append to string
              // if last char was a number push the whole number to the charnum array
              if (rt !== 's') {
                rt = 's';
                na.push(parseFloat(rv));
                rv = '';
              }
              rv = rv + nthing.charAt(x);
            }
          }
          // once done, push the last value to the charnum array and remove the first uneeded item
          na.push(rt === 'n' ? parseFloat(rv) : rv.toLowerCase());
          na.shift();
          // add to cache
          cache['_' + thing] = na;
          cachcounter++;
          // return charnum array
          return na;
        }();
      };
    }());
    // ****************************************
    // *
    // * Runs a query
    // ****************************************
    run = function () {
      this.context({ results: this.getDBI().query(this.context()) });
    };
    API.extend('filter', function () {
      // ****************************************
      // *
      // * Takes: takes unlimited filter objects as arguments
      // * Returns: method collection
      // * Purpose: Take filters as objects and cache functions for later lookup when a query is run
      // ****************************************
      var nc = TAFFY.mergeObj(this.context(), { run: null }), nq = [];
      ;
      each(nc.q, function (v) {
        nq.push(v);
      });
      nc.q = nq;
      // Hadnle passing of ___ID or a record on lookup.
      each(sortArgs(arguments), function (f) {
        nc.q.push(returnFilter(f));
        nc.filterRaw.push(f);
      });
      return this.getroot(nc);
    });
    API.extend('order', function (o) {
      // ****************************************
      // *
      // * Purpose: takes a string and creates an array of order instructions to be used with a query
      // ****************************************
      o = o.split(',');
      var x = [], nc;
      each(o, function (r) {
        x.push(r.replace(/^\s*/, '').replace(/\s*$/, ''));
      });
      nc = TAFFY.mergeObj(this.context(), { sort: null });
      nc.order = x;
      return this.getroot(nc);
    });
    API.extend('limit', function (n) {
      // ****************************************
      // *
      // * Purpose: takes a limit number to limit the number of rows returned by a query. Will update the results
      // * of a query
      // ****************************************
      var nc = TAFFY.mergeObj(this.context(), {}), limitedresults;
      ;
      nc.limit = n;
      if (nc.run && nc.sort) {
        limitedresults = [];
        each(nc.results, function (i, x) {
          if (x + 1 > n) {
            return TAFFY.EXIT;
          }
          limitedresults.push(i);
        });
        nc.results = limitedresults;
      }
      return this.getroot(nc);
    });
    API.extend('start', function (n) {
      // ****************************************
      // *
      // * Purpose: takes a limit number to limit the number of rows returned by a query. Will update the results
      // * of a query
      // ****************************************
      var nc = TAFFY.mergeObj(this.context(), {}), limitedresults;
      ;
      nc.start = n;
      if (nc.run && nc.sort && !nc.limit) {
        limitedresults = [];
        each(nc.results, function (i, x) {
          if (x + 1 > n) {
            limitedresults.push(i);
          }
        });
        nc.results = limitedresults;
      } else {
        nc = TAFFY.mergeObj(this.context(), {
          run: null,
          start: n
        });
      }
      return this.getroot(nc);
    });
    API.extend('update', function (arg0, arg1, arg2) {
      // ****************************************
      // *
      // * Takes: a object and passes it off DBI update method for all matched records
      // ****************************************
      var runEvent = true, o = {}, args = arguments, that;
      if (TAFFY.isString(arg0) && (arguments.length === 2 || arguments.length === 3)) {
        o[arg0] = arg1;
        if (arguments.length === 3) {
          runEvent = arg2;
        }
      } else {
        o = arg0;
        if (args.length === 2) {
          runEvent = arg1;
        }
      }
      that = this;
      run.call(this);
      each(this.context().results, function (r) {
        var c = o;
        if (TAFFY.isFunction(c)) {
          c = c.apply(TAFFY.mergeObj(r, {}));
        } else {
          if (T.isFunction(c)) {
            c = c(TAFFY.mergeObj(r, {}));
          }
        }
        if (TAFFY.isObject(c)) {
          that.getDBI().update(r.___id, c, runEvent);
        }
      });
      if (this.context().results.length) {
        this.context({ run: null });
      }
      return this;
    });
    API.extend('remove', function (runEvent) {
      // ****************************************
      // *
      // * Purpose: removes records from the DB via the remove and removeCommit DBI methods
      // ****************************************
      var that = this, c = 0;
      run.call(this);
      each(this.context().results, function (r) {
        that.getDBI().remove(r.___id);
        c++;
      });
      if (this.context().results.length) {
        this.context({ run: null });
        that.getDBI().removeCommit(runEvent);
      }
      return c;
    });
    API.extend('count', function () {
      // ****************************************
      // *
      // * Returns: The length of a query result
      // ****************************************
      run.call(this);
      return this.context().results.length;
    });
    API.extend('callback', function (f, delay) {
      // ****************************************
      // *
      // * Returns null;
      // * Runs a function on return of run.call
      // ****************************************
      if (f) {
        var that = this;
        setTimeout(function () {
          run.call(that);
          f.call(that.getroot(that.context()));
        }, delay || 0);
      }
      return null;
    });
    API.extend('get', function () {
      // ****************************************
      // *
      // * Returns: An array of all matching records
      // ****************************************
      run.call(this);
      return this.context().results;
    });
    API.extend('stringify', function () {
      // ****************************************
      // *
      // * Returns: An JSON string of all matching records
      // ****************************************
      return JSON.stringify(this.get());
    });
    API.extend('first', function () {
      // ****************************************
      // *
      // * Returns: The first matching record
      // ****************************************
      run.call(this);
      return this.context().results[0] || false;
    });
    API.extend('last', function () {
      // ****************************************
      // *
      // * Returns: The last matching record
      // ****************************************
      run.call(this);
      return this.context().results[this.context().results.length - 1] || false;
    });
    API.extend('sum', function () {
      // ****************************************
      // *
      // * Takes: column to sum up
      // * Returns: Sums the values of a column
      // ****************************************
      var total = 0, that = this;
      run.call(that);
      each(sortArgs(arguments), function (c) {
        each(that.context().results, function (r) {
          total = total + (r[c] || 0);
        });
      });
      return total;
    });
    API.extend('min', function (c) {
      // ****************************************
      // *
      // * Takes: column to find min
      // * Returns: the lowest value
      // ****************************************
      var lowest = null;
      run.call(this);
      each(this.context().results, function (r) {
        if (lowest === null || r[c] < lowest) {
          lowest = r[c];
        }
      });
      return lowest;
    });
    //  Taffy innerJoin Extension (OCD edition)
    //  =======================================
    //
    //  How to Use
    //  **********
    //
    //  left_table.innerJoin( right_table, condition1 <,... conditionN> )
    //
    //  A condition can take one of 2 forms:
    //
    //    1. An ARRAY with 2 or 3 values:
    //    A column name from the left table, an optional comparison string,
    //    and column name from the right table.  The condition passes if the test
    //    indicated is true.   If the condition string is omitted, '===' is assumed.
    //    EXAMPLES: [ 'last_used_time', '>=', 'current_use_time' ], [ 'user_id','id' ]
    //
    //    2. A FUNCTION:
    //    The function receives a left table row and right table row during the
    //    cartesian join.  If the function returns true for the rows considered,
    //    the merged row is included in the result set.
    //    EXAMPLE: function (l,r){ return l.name === r.label; }
    //
    //  Conditions are considered in the order they are presented.  Therefore the best
    //  performance is realized when the least expensive and highest prune-rate
    //  conditions are placed first, since if they return false Taffy skips any
    //  further condition tests.
    //
    //  Other notes
    //  ***********
    //
    //  This code passes jslint with the exception of 2 warnings about
    //  the '==' and '!=' lines.  We can't do anything about that short of
    //  deleting the lines.
    //
    //  Credits
    //  *******
    //
    //  Heavily based upon the work of Ian Toltz.
    //  Revisions to API by Michael Mikowski.
    //  Code convention per standards in http://manning.com/mikowski
    (function () {
      var innerJoinFunction = function () {
          var fnCompareList, fnCombineRow, fnMain;
          fnCompareList = function (left_row, right_row, arg_list) {
            var data_lt, data_rt, op_code, error;
            if (arg_list.length === 2) {
              data_lt = left_row[arg_list[0]];
              op_code = '===';
              data_rt = right_row[arg_list[1]];
            } else {
              data_lt = left_row[arg_list[0]];
              op_code = arg_list[1];
              data_rt = right_row[arg_list[2]];
            }
            /*jslint eqeq : true */
            switch (op_code) {
            case '===':
              return data_lt === data_rt;
            case '!==':
              return data_lt !== data_rt;
            case '<':
              return data_lt < data_rt;
            case '>':
              return data_lt > data_rt;
            case '<=':
              return data_lt <= data_rt;
            case '>=':
              return data_lt >= data_rt;
            case '==':
              return data_lt == data_rt;
            case '!=':
              return data_lt != data_rt;
            default:
              throw String(op_code) + ' is not supported';
            }  // 'jslint eqeq : false'  here results in
               // "Unreachable '/*jslint' after 'return'".
               // We don't need it though, as the rule exception
               // is discarded at the end of this functional scope
          };
          fnCombineRow = function (left_row, right_row) {
            var out_map = {}, i, prefix;
            for (i in left_row) {
              if (left_row.hasOwnProperty(i)) {
                out_map[i] = left_row[i];
              }
            }
            for (i in right_row) {
              if (right_row.hasOwnProperty(i) && i !== '___id' && i !== '___s') {
                prefix = !TAFFY.isUndefined(out_map[i]) ? 'right_' : '';
                out_map[prefix + String(i)] = right_row[i];
              }
            }
            return out_map;
          };
          fnMain = function (table) {
            var right_table, i, arg_list = arguments, arg_length = arg_list.length, result_list = [];
            ;
            if (typeof table.filter !== 'function') {
              if (table.TAFFY) {
                right_table = table();
              } else {
                throw 'TAFFY DB or result not supplied';
              }
            } else {
              right_table = table;
            }
            this.context({ results: this.getDBI().query(this.context()) });
            TAFFY.each(this.context().results, function (left_row) {
              right_table.each(function (right_row) {
                var arg_data, is_ok = true;
                CONDITION:
                  for (i = 1; i < arg_length; i++) {
                    arg_data = arg_list[i];
                    if (typeof arg_data === 'function') {
                      is_ok = arg_data(left_row, right_row);
                    } else if (typeof arg_data === 'object' && arg_data.length) {
                      is_ok = fnCompareList(left_row, right_row, arg_data);
                    } else {
                      is_ok = false;
                    }
                    if (!is_ok) {
                      break CONDITION;
                    }  // short circuit
                  }
                if (is_ok) {
                  result_list.push(fnCombineRow(left_row, right_row));
                }
              });
            });
            return TAFFY(result_list)();
          };
          return fnMain;
        }();
      API.extend('join', innerJoinFunction);
    }());
    API.extend('max', function (c) {
      // ****************************************
      // *
      // * Takes: column to find max
      // * Returns: the highest value
      // ****************************************
      var highest = null;
      run.call(this);
      each(this.context().results, function (r) {
        if (highest === null || r[c] > highest) {
          highest = r[c];
        }
      });
      return highest;
    });
    API.extend('select', function () {
      // ****************************************
      // *
      // * Takes: columns to select values into an array
      // * Returns: array of values
      // * Note if more than one column is given an array of arrays is returned
      // ****************************************
      var ra = [], args = sortArgs(arguments);
      run.call(this);
      if (arguments.length === 1) {
        each(this.context().results, function (r) {
          ra.push(r[args[0]]);
        });
      } else {
        each(this.context().results, function (r) {
          var row = [];
          each(args, function (c) {
            row.push(r[c]);
          });
          ra.push(row);
        });
      }
      return ra;
    });
    API.extend('distinct', function () {
      // ****************************************
      // *
      // * Takes: columns to select unique alues into an array
      // * Returns: array of values
      // * Note if more than one column is given an array of arrays is returned
      // ****************************************
      var ra = [], args = sortArgs(arguments);
      run.call(this);
      if (arguments.length === 1) {
        each(this.context().results, function (r) {
          var v = r[args[0]], dup = false;
          each(ra, function (d) {
            if (v === d) {
              dup = true;
              return TAFFY.EXIT;
            }
          });
          if (!dup) {
            ra.push(v);
          }
        });
      } else {
        each(this.context().results, function (r) {
          var row = [], dup = false;
          each(args, function (c) {
            row.push(r[c]);
          });
          each(ra, function (d) {
            var ldup = true;
            each(args, function (c, i) {
              if (row[i] !== d[i]) {
                ldup = false;
                return TAFFY.EXIT;
              }
            });
            if (ldup) {
              dup = true;
              return TAFFY.EXIT;
            }
          });
          if (!dup) {
            ra.push(row);
          }
        });
      }
      return ra;
    });
    API.extend('supplant', function (template, returnarray) {
      // ****************************************
      // *
      // * Takes: a string template formated with key to be replaced with values from the rows, flag to determine if we want array of strings
      // * Returns: array of values or a string
      // ****************************************
      var ra = [];
      run.call(this);
      each(this.context().results, function (r) {
        // TODO: The curly braces used to be unescaped
        ra.push(template.replace(/\{([^\{\}]*)\}/g, function (a, b) {
          var v = r[b];
          return typeof v === 'string' || typeof v === 'number' ? v : a;
        }));
      });
      return !returnarray ? ra.join('') : ra;
    });
    API.extend('each', function (m) {
      // ****************************************
      // *
      // * Takes: a function
      // * Purpose: loops over every matching record and applies the function
      // ****************************************
      run.call(this);
      each(this.context().results, m);
      return this;
    });
    API.extend('map', function (m) {
      // ****************************************
      // *
      // * Takes: a function
      // * Purpose: loops over every matching record and applies the function, returing the results in an array
      // ****************************************
      var ra = [];
      run.call(this);
      each(this.context().results, function (r) {
        ra.push(m(r));
      });
      return ra;
    });
    T = function (d) {
      // ****************************************
      // *
      // * T is the main TAFFY object
      // * Takes: an array of objects or JSON
      // * Returns a new TAFFYDB
      // ****************************************
      var TOb = [], ID = {}, RC = 1, settings = {
          template: false,
          onInsert: false,
          onUpdate: false,
          onRemove: false,
          onDBChange: false,
          storageName: false,
          forcePropertyCase: null,
          delayDiskUpdates: 0,
          cacheSize: 100,
          name: ''
        }, dm = new Date(), CacheCount = 0, CacheClear = 0, Cache = {}, DBI, runIndexes, root;
      ;
      // ****************************************
      // *
      // * TOb = this database
      // * ID = collection of the record IDs and locations within the DB, used for fast lookups
      // * RC = record counter, used for creating IDs
      // * settings.template = the template to merge all new records with
      // * settings.onInsert = event given a copy of the newly inserted record
      // * settings.onUpdate = event given the original record, the changes, and the new record
      // * settings.onRemove = event given the removed record
      // * settings.forcePropertyCase = on insert force the proprty case to be lower or upper. default lower, null/undefined will leave case as is
      // * dm = the modify date of the database, used for query caching
      // ****************************************
      runIndexes = function (indexes) {
        // ****************************************
        // *
        // * Takes: a collection of indexes
        // * Returns: collection with records matching indexed filters
        // ****************************************
        var records = [], UniqueEnforce = false;
        if (indexes.length === 0) {
          return TOb;
        }
        each(indexes, function (f) {
          // Check to see if record ID
          if (T.isString(f) && /[t][0-9]*[r][0-9]*/i.test(f) && TOb[ID[f]]) {
            records.push(TOb[ID[f]]);
            UniqueEnforce = true;
          }
          // Check to see if record
          if (T.isObject(f) && f.___id && f.___s && TOb[ID[f.___id]]) {
            records.push(TOb[ID[f.___id]]);
            UniqueEnforce = true;
          }
          // Check to see if array of indexes
          if (T.isArray(f)) {
            each(f, function (r) {
              each(runIndexes(r), function (rr) {
                records.push(rr);
              });
            });
          }
        });
        if (UniqueEnforce && records.length > 1) {
          records = [];
        }
        return records;
      };
      DBI = {
        dm: function (nd) {
          // ****************************************
          // *
          // * Takes: an optional new modify date
          // * Purpose: used to get and set the DB modify date
          // ****************************************
          if (nd) {
            dm = nd;
            Cache = {};
            CacheCount = 0;
            CacheClear = 0;
          }
          if (settings.onDBChange) {
            setTimeout(function () {
              settings.onDBChange.call(TOb);
            }, 0);
          }
          if (settings.delayDiskUpdates === 0) {
            if (settings.storageName) {
              setTimeout(function () {
                var time = new Date().getTime();
                localStorage.setItem('taffy_' + settings.storageName, JSON.stringify(TOb));
              });
            }
          }
          return dm;
        },
        delayDiskUpdates: function () {
          settings.delayDiskUpdates++;
        },
        flushDiskUpdates: function () {
          settings.delayDiskUpdates--;
          if (settings.delayDiskUpdates === 0) {
            if (settings.storageName) {
              setTimeout(function () {
                var time = new Date().getTime();
                localStorage.setItem('taffy_' + settings.storageName, JSON.stringify(TOb));
              });
            }
          }
        },
        insert: function (i, runEvent) {
          // ****************************************
          // *
          // * Takes: a new record to insert
          // * Purpose: merge the object with the template, add an ID, insert into DB, call insert event
          // ****************************************
          var columns = [], records = [], input = protectJSON(i);
          ;
          each(input, function (v, i) {
            var nv, o;
            if (T.isArray(v) && i === 0) {
              each(v, function (av) {
                columns.push(settings.forcePropertyCase === 'lower' ? av.toLowerCase() : settings.forcePropertyCase === 'upper' ? av.toUpperCase() : av);
              });
              return true;
            } else if (T.isArray(v)) {
              nv = {};
              each(v, function (av, ai) {
                nv[columns[ai]] = av;
              });
              v = nv;
            } else if (T.isObject(v) && settings.forcePropertyCase) {
              o = {};
              eachin(v, function (av, ai) {
                o[settings.forcePropertyCase === 'lower' ? ai.toLowerCase() : settings.forcePropertyCase === 'upper' ? ai.toUpperCase() : ai] = v[ai];
              });
              v = o;
            }
            RC++;
            v.___id = 'T' + String(idpad + TC).slice(-6) + 'R' + String(idpad + RC).slice(-6);
            v.___s = true;
            records.push(v.___id);
            if (settings.template) {
              v = T.mergeObj(settings.template, v);
            }
            TOb.push(v);
            ID[v.___id] = TOb.length - 1;
            if (settings.onInsert && (runEvent || TAFFY.isUndefined(runEvent))) {
              settings.onInsert.call(v);
            }
            DBI.dm(new Date());
          });
          return root(records);
        },
        sort: function (o) {
          // ****************************************
          // *
          // * Purpose: Change the sort order of the DB itself and reset the ID bucket
          // ****************************************
          TOb = orderByCol(TOb, o.split(','));
          ID = {};
          each(TOb, function (r, i) {
            ID[r.___id] = i;
          });
          DBI.dm(new Date());
          return true;
        },
        update: function (id, changes, runEvent) {
          // ****************************************
          // *
          // * Takes: the ID of record being changed and the changes
          // * Purpose: Update a record and change some or all values, call the on update method
          // ****************************************
          var nc = {}, or, nr, tc, hasChange;
          if (settings.forcePropertyCase) {
            eachin(changes, function (v, p) {
              nc[settings.forcePropertyCase === 'lower' ? p.toLowerCase() : settings.forcePropertyCase === 'upper' ? p.toUpperCase() : p] = v;
            });
            changes = nc;
          }
          or = TOb[ID[id]];
          nr = T.mergeObj(or, changes);
          tc = {};
          hasChange = false;
          eachin(nr, function (v, i) {
            if (TAFFY.isUndefined(or[i]) || or[i] !== v) {
              tc[i] = v;
              hasChange = true;
            }
          });
          if (hasChange) {
            if (settings.onUpdate && (runEvent || TAFFY.isUndefined(runEvent))) {
              settings.onUpdate.call(nr, TOb[ID[id]], tc);
            }
            TOb[ID[id]] = nr;
            DBI.dm(new Date());
          }
        },
        remove: function (id) {
          // ****************************************
          // *
          // * Takes: the ID of record to be removed
          // * Purpose: remove a record, changes its ___s value to false
          // ****************************************
          TOb[ID[id]].___s = false;
        },
        removeCommit: function (runEvent) {
          var x;
          // ****************************************
          // *
          // *
          // * Purpose: loop over all records and remove records with ___s = false, call onRemove event, clear ID
          // ****************************************
          for (x = TOb.length - 1; x > -1; x--) {
            if (!TOb[x].___s) {
              if (settings.onRemove && (runEvent || TAFFY.isUndefined(runEvent))) {
                settings.onRemove.call(TOb[x]);
              }
              ID[TOb[x].___id] = undefined;
              TOb.splice(x, 1);
            }
          }
          ID = {};
          each(TOb, function (r, i) {
            ID[r.___id] = i;
          });
          DBI.dm(new Date());
        },
        query: function (context) {
          // ****************************************
          // *
          // * Takes: the context object for a query and either returns a cache result or a new query result
          // ****************************************
          var returnq, cid, results, indexed, limitq, ni;
          if (settings.cacheSize) {
            cid = '';
            each(context.filterRaw, function (r) {
              if (T.isFunction(r)) {
                cid = 'nocache';
                return TAFFY.EXIT;
              }
            });
            if (cid === '') {
              cid = makeCid(T.mergeObj(context, {
                q: false,
                run: false,
                sort: false
              }));
            }
          }
          // Run a new query if there are no results or the run date has been cleared
          if (!context.results || !context.run || context.run && DBI.dm() > context.run) {
            results = [];
            // check Cache
            if (settings.cacheSize && Cache[cid]) {
              Cache[cid].i = CacheCount++;
              return Cache[cid].results;
            } else {
              // if no filter, return DB
              if (context.q.length === 0 && context.index.length === 0) {
                each(TOb, function (r) {
                  results.push(r);
                });
                returnq = results;
              } else {
                // use indexes
                indexed = runIndexes(context.index);
                // run filters
                each(indexed, function (r) {
                  // Run filter to see if record matches query
                  if (context.q.length === 0 || runFilters(r, context.q)) {
                    results.push(r);
                  }
                });
                returnq = results;
              }
            }
          } else {
            // If query exists and run has not been cleared return the cache results
            returnq = context.results;
          }
          // If a custom order array exists and the run has been clear or the sort has been cleared
          if (context.order.length > 0 && (!context.run || !context.sort)) {
            // order the results
            returnq = orderByCol(returnq, context.order);
          }
          // If a limit on the number of results exists and it is less than the returned results, limit results
          if (returnq.length && (context.limit && context.limit < returnq.length || context.start)) {
            limitq = [];
            each(returnq, function (r, i) {
              if (!context.start || context.start && i + 1 >= context.start) {
                if (context.limit) {
                  ni = context.start ? i + 1 - context.start : i;
                  if (ni < context.limit) {
                    limitq.push(r);
                  } else if (ni > context.limit) {
                    return TAFFY.EXIT;
                  }
                } else {
                  limitq.push(r);
                }
              }
            });
            returnq = limitq;
          }
          // update cache
          if (settings.cacheSize && cid !== 'nocache') {
            CacheClear++;
            setTimeout(function () {
              var bCounter, nc;
              if (CacheClear >= settings.cacheSize * 2) {
                CacheClear = 0;
                bCounter = CacheCount - settings.cacheSize;
                nc = {};
                eachin(function (r, k) {
                  if (r.i >= bCounter) {
                    nc[k] = r;
                  }
                });
                Cache = nc;
              }
            }, 0);
            Cache[cid] = {
              i: CacheCount++,
              results: returnq
            };
          }
          return returnq;
        }
      };
      root = function () {
        var iAPI, context;
        // ****************************************
        // *
        // * The root function that gets returned when a new DB is created
        // * Takes: unlimited filter arguments and creates filters to be run when a query is called
        // ****************************************
        // ****************************************
        // *
        // * iAPI is the the method collection valiable when a query has been started by calling dbname
        // * Certain methods are or are not avaliable once you have started a query such as insert -- you can only insert into root
        // ****************************************
        iAPI = TAFFY.mergeObj(TAFFY.mergeObj(API, { insert: undefined }), {
          getDBI: function () {
            return DBI;
          },
          getroot: function (c) {
            return root.call(c);
          },
          context: function (n) {
            // ****************************************
            // *
            // * The context contains all the information to manage a query including filters, limits, and sorts
            // ****************************************
            if (n) {
              context = TAFFY.mergeObj(context, n.hasOwnProperty('results') ? TAFFY.mergeObj(n, {
                run: new Date(),
                sort: new Date()
              }) : n);
            }
            return context;
          },
          extend: undefined
        });
        context = this && this.q ? this : {
          limit: false,
          start: false,
          q: [],
          filterRaw: [],
          index: [],
          order: [],
          results: false,
          run: null,
          sort: null,
          settings: settings
        };
        // ****************************************
        // *
        // * Call the query method to setup a new query
        // ****************************************
        each(sortArgs(arguments), function (f) {
          if (isIndexable(f)) {
            context.index.push(f);
          } else {
            context.q.push(returnFilter(f));
          }
          context.filterRaw.push(f);
        });
        return iAPI;
      };
      // ****************************************
      // *
      // * If new records have been passed on creation of the DB either as JSON or as an array/object, insert them
      // ****************************************
      TC++;
      if (d) {
        DBI.insert(d);
      }
      root.insert = DBI.insert;
      root.delayDiskUpdates = DBI.delayDiskUpdates;
      root.flushDiskUpdates = DBI.flushDiskUpdates;
      root.merge = function (i, key, runEvent) {
        var search = {}, finalSearch = [], obj = {};
        ;
        runEvent = runEvent || false;
        key = key || 'id';
        each(i, function (o) {
          var existingObject;
          search[key] = o[key];
          finalSearch.push(o[key]);
          existingObject = root(search).first();
          if (existingObject) {
            DBI.update(existingObject.___id, o, runEvent);
          } else {
            DBI.insert(o, runEvent);
          }
        });
        obj[key] = finalSearch;
        return root(obj);
      };
      root.TAFFY = true;
      root.sort = DBI.sort;
      // ****************************************
      // *
      // * These are the methods that can be accessed on off the root DB function. Example dbname.insert;
      // ****************************************
      root.settings = function (n) {
        // ****************************************
        // *
        // * Getting and setting for this DB's settings/events
        // ****************************************
        if (n) {
          settings = TAFFY.mergeObj(settings, n);
          if (n.template) {
            root().update(n.template);
          }
        }
        return settings;
      };
      // ****************************************
      // *
      // * These are the methods that can be accessed on off the root DB function. Example dbname.insert;
      // ****************************************
      root.store = function (n) {
        // ****************************************
        // *
        // * Setup localstorage for this DB on a given name
        // * Pull data into the DB as needed
        // ****************************************
        var r = false, i;
        if (localStorage) {
          if (n) {
            i = localStorage.getItem('taffy_' + n);
            if (i && i.length > 0) {
              root.insert(i);
              r = true;
            }
            if (TOb.length > 0) {
              setTimeout(function () {
                localStorage.setItem('taffy_' + settings.storageName, JSON.stringify(TOb));
              });
            }
          }
          root.settings({ storageName: n });
        }
        return root;
      };
      // ****************************************
      // *
      // * Return root on DB creation and start having fun
      // ****************************************
      return root;
    };
    // ****************************************
    // *
    // * Sets the global TAFFY object
    // ****************************************
    TAFFY = T;
    // ****************************************
    // *
    // * Create public each method
    // *
    // ****************************************
    T.each = each;
    // ****************************************
    // *
    // * Create public eachin method
    // *
    // ****************************************
    T.eachin = eachin;
    // ****************************************
    // *
    // * Create public extend method
    // * Add a custom method to the API
    // *
    // ****************************************
    T.extend = API.extend;
    // ****************************************
    // *
    // * Creates TAFFY.EXIT value that can be returned to stop an each loop
    // *
    // ****************************************
    TAFFY.EXIT = 'TAFFYEXIT';
    // ****************************************
    // *
    // * Create public utility mergeObj method
    // * Return a new object where items from obj2
    // * have replaced or been added to the items in
    // * obj1
    // * Purpose: Used to combine objs
    // *
    // ****************************************
    TAFFY.mergeObj = function (ob1, ob2) {
      var c = {};
      eachin(ob1, function (v, n) {
        c[n] = ob1[n];
      });
      eachin(ob2, function (v, n) {
        c[n] = ob2[n];
      });
      return c;
    };
    // ****************************************
    // *
    // * Create public utility has method
    // * Returns true if a complex object, array
    // * or taffy collection contains the material
    // * provided in the second argument
    // * Purpose: Used to comare objects
    // *
    // ****************************************
    TAFFY.has = function (var1, var2) {
      var re = false, n;
      if (var1.TAFFY) {
        re = var1(var2);
        if (re.length > 0) {
          return true;
        } else {
          return false;
        }
      } else {
        switch (T.typeOf(var1)) {
        case 'object':
          if (T.isObject(var2)) {
            eachin(var2, function (v, n) {
              if (re === true && !T.isUndefined(var1[n]) && var1.hasOwnProperty(n)) {
                re = T.has(var1[n], var2[n]);
              } else {
                re = false;
                return TAFFY.EXIT;
              }
            });
          } else if (T.isArray(var2)) {
            each(var2, function (v, n) {
              re = T.has(var1, var2[n]);
              if (re) {
                return TAFFY.EXIT;
              }
            });
          } else if (T.isString(var2)) {
            if (!TAFFY.isUndefined(var1[var2])) {
              return true;
            } else {
              return false;
            }
          }
          return re;
        case 'array':
          if (T.isObject(var2)) {
            each(var1, function (v, i) {
              re = T.has(var1[i], var2);
              if (re === true) {
                return TAFFY.EXIT;
              }
            });
          } else if (T.isArray(var2)) {
            each(var2, function (v2, i2) {
              each(var1, function (v1, i1) {
                re = T.has(var1[i1], var2[i2]);
                if (re === true) {
                  return TAFFY.EXIT;
                }
              });
              if (re === true) {
                return TAFFY.EXIT;
              }
            });
          } else if (T.isString(var2) || T.isNumber(var2)) {
            re = false;
            for (n = 0; n < var1.length; n++) {
              re = T.has(var1[n], var2);
              if (re) {
                return true;
              }
            }
          }
          return re;
        case 'string':
          if (T.isString(var2) && var2 === var1) {
            return true;
          }
          break;
        default:
          if (T.typeOf(var1) === T.typeOf(var2) && var1 === var2) {
            return true;
          }
          break;
        }
      }
      return false;
    };
    // ****************************************
    // *
    // * Create public utility hasAll method
    // * Returns true if a complex object, array
    // * or taffy collection contains the material
    // * provided in the call - for arrays it must
    // * contain all the material in each array item
    // * Purpose: Used to comare objects
    // *
    // ****************************************
    TAFFY.hasAll = function (var1, var2) {
      var T = TAFFY, ar;
      if (T.isArray(var2)) {
        ar = true;
        each(var2, function (v) {
          ar = T.has(var1, v);
          if (ar === false) {
            return TAFFY.EXIT;
          }
        });
        return ar;
      } else {
        return T.has(var1, var2);
      }
    };
    // ****************************************
    // *
    // * typeOf Fixed in JavaScript as public utility
    // *
    // ****************************************
    TAFFY.typeOf = function (v) {
      var s = typeof v;
      if (s === 'object') {
        if (v) {
          if (typeof v.length === 'number' && !v.propertyIsEnumerable('length')) {
            s = 'array';
          }
        } else {
          s = 'null';
        }
      }
      return s;
    };
    // ****************************************
    // *
    // * Create public utility getObjectKeys method
    // * Returns an array of an objects keys
    // * Purpose: Used to get the keys for an object
    // *
    // ****************************************
    TAFFY.getObjectKeys = function (ob) {
      var kA = [];
      eachin(ob, function (n, h) {
        kA.push(h);
      });
      kA.sort();
      return kA;
    };
    // ****************************************
    // *
    // * Create public utility isSameArray
    // * Returns an array of an objects keys
    // * Purpose: Used to get the keys for an object
    // *
    // ****************************************
    TAFFY.isSameArray = function (ar1, ar2) {
      return TAFFY.isArray(ar1) && TAFFY.isArray(ar2) && ar1.join(',') === ar2.join(',') ? true : false;
    };
    // ****************************************
    // *
    // * Create public utility isSameObject method
    // * Returns true if objects contain the same
    // * material or false if they do not
    // * Purpose: Used to comare objects
    // *
    // ****************************************
    TAFFY.isSameObject = function (ob1, ob2) {
      var T = TAFFY, rv = true;
      if (T.isObject(ob1) && T.isObject(ob2)) {
        if (T.isSameArray(T.getObjectKeys(ob1), T.getObjectKeys(ob2))) {
          eachin(ob1, function (v, n) {
            if (!(T.isObject(ob1[n]) && T.isObject(ob2[n]) && T.isSameObject(ob1[n], ob2[n]) || T.isArray(ob1[n]) && T.isArray(ob2[n]) && T.isSameArray(ob1[n], ob2[n]) || ob1[n] === ob2[n])) {
              rv = false;
              return TAFFY.EXIT;
            }
          });
        } else {
          rv = false;
        }
      } else {
        rv = false;
      }
      return rv;
    };
    // ****************************************
    // *
    // * Create public utility is[DataType] methods
    // * Return true if obj is datatype, false otherwise
    // * Purpose: Used to determine if arguments are of certain data type
    // *
    // * mmikowski 2012-08-06 refactored to make much less "magical":
    // *   fewer closures and passes jslint
    // *
    // ****************************************
    typeList = [
      'String',
      'Number',
      'Object',
      'Array',
      'Boolean',
      'Null',
      'Function',
      'Undefined'
    ];
    makeTest = function (thisKey) {
      return function (data) {
        return TAFFY.typeOf(data) === thisKey.toLowerCase() ? true : false;
      };
    };
    for (idx = 0; idx < typeList.length; idx++) {
      typeKey = typeList[idx];
      TAFFY['is' + typeKey] = makeTest(typeKey);
    }
  }
}());
if (typeof exports === 'object') {
  exports.taffy = TAFFY;
}
'use strict';
/**
 * Bindonce - Zero watches binding for AngularJs
 * @version v0.2.2 - 2013-05-07
 * @link https://github.com/Pasvaz/bindonce
 * @author Pasquale Vazzana <pasqualevazzana@gmail.com>
 * @license MIT License, http://www.opensource.org/licenses/MIT
 */
angular.module('pasvaz.bindonce', []).directive('bindonce', function () {
  var toBoolean = function (value) {
    if (value && value.length !== 0) {
      var v = angular.lowercase('' + value);
      value = !(v == 'f' || v == '0' || v == 'false' || v == 'no' || v == 'n' || v == '[]');
    } else {
      value = false;
    }
    return value;
  };
  var msie = parseInt((/msie (\d+)/.exec(angular.lowercase(navigator.userAgent)) || [])[1], 10);
  if (isNaN(msie)) {
    msie = parseInt((/trident\/.*; rv:(\d+)/.exec(angular.lowercase(navigator.userAgent)) || [])[1], 10);
  }
  var bindonceDirective = {
      restrict: 'AM',
      controller: [
        '$scope',
        '$element',
        '$attrs',
        '$interpolate',
        function ($scope, $element, $attrs, $interpolate) {
          var showHideBinder = function (elm, attr, value) {
            var show = attr == 'show' ? '' : 'none';
            var hide = attr == 'hide' ? '' : 'none';
            elm.css('display', toBoolean(value) ? show : hide);
          };
          var classBinder = function (elm, value) {
            if (angular.isObject(value) && !angular.isArray(value)) {
              var results = [];
              angular.forEach(value, function (value, index) {
                if (value)
                  results.push(index);
              });
              value = results;
            }
            if (value) {
              elm.addClass(angular.isArray(value) ? value.join(' ') : value);
            }
          };
          var ctrl = {
              watcherRemover: undefined,
              binders: [],
              group: $attrs.boName,
              element: $element,
              ran: false,
              addBinder: function (binder) {
                this.binders.push(binder);
                // In case of late binding (when using the directive bo-name/bo-parent)
                // it happens only when you use nested bindonce, if the bo-children
                // are not dom children the linking can follow another order
                if (this.ran) {
                  this.runBinders();
                }
              },
              setupWatcher: function (bindonceValue) {
                var that = this;
                this.watcherRemover = $scope.$watch(bindonceValue, function (newValue) {
                  if (newValue == undefined)
                    return;
                  that.removeWatcher();
                  that.runBinders();
                }, true);
              },
              removeWatcher: function () {
                if (this.watcherRemover != undefined) {
                  this.watcherRemover();
                  this.watcherRemover = undefined;
                }
              },
              runBinders: function () {
                var i, max;
                for (i = 0, max = this.binders.length; i < max; i++) {
                  var binder = this.binders[i];
                  if (this.group && this.group != binder.group)
                    continue;
                  var value = binder.scope.$eval(binder.interpolate ? $interpolate(binder.value) : binder.value);
                  switch (binder.attr) {
                  case 'if':
                    if (toBoolean(value)) {
                      binder.transclude(binder.scope.$new(), function (clone) {
                        var parent = binder.element.parent();
                        var afterNode = binder.element && binder.element[binder.element.length - 1];
                        var parentNode = parent && parent[0] || afterNode && afterNode.parentNode;
                        var afterNextSibling = afterNode && afterNode.nextSibling || null;
                        angular.forEach(clone, function (node) {
                          parentNode.insertBefore(node, afterNextSibling);
                        });
                      });
                    }
                    break;
                  case 'hide':
                  case 'show':
                    showHideBinder(binder.element, binder.attr, value);
                    break;
                  case 'class':
                    classBinder(binder.element, value);
                    break;
                  case 'text':
                    binder.element.text(value);
                    break;
                  case 'html':
                    binder.element.html(value);
                    break;
                  case 'style':
                    binder.element.css(value);
                    break;
                  case 'src':
                    binder.element.attr(binder.attr, value);
                    if (msie)
                      binder.element.prop('src', value);
                  case 'attr':
                    angular.forEach(binder.attrs, function (attrValue, attrKey) {
                      var newAttr, newValue;
                      if (attrKey.match(/^boAttr./) && binder.attrs[attrKey]) {
                        newAttr = attrKey.replace(/^boAttr/, '').replace(/([a-z])([A-Z])/g, '$1-$2').toLowerCase();
                        newValue = binder.scope.$eval(binder.attrs[attrKey]);
                        binder.element.attr(newAttr, newValue);
                      }
                    });
                    break;
                  case 'href':
                  case 'alt':
                  case 'title':
                  case 'id':
                  case 'value':
                    binder.element.attr(binder.attr, value);
                    break;
                  }
                }
                this.ran = true;
                this.binders = [];
              }
            };
          return ctrl;
        }
      ],
      link: function (scope, elm, attrs, bindonceController) {
        var value = attrs.bindonce ? scope.$eval(attrs.bindonce) : true;
        if (value != undefined) {
          bindonceController.runBinders();
        } else {
          bindonceController.setupWatcher(attrs.bindonce);
          elm.bind('$destroy', bindonceController.removeWatcher);
        }
      }
    };
  return bindonceDirective;
});
angular.forEach([
  {
    directiveName: 'boShow',
    attribute: 'show'
  },
  {
    directiveName: 'boIf',
    attribute: 'if',
    transclude: 'element',
    terminal: true,
    priority: 1000
  },
  {
    directiveName: 'boHide',
    attribute: 'hide'
  },
  {
    directiveName: 'boClass',
    attribute: 'class'
  },
  {
    directiveName: 'boText',
    attribute: 'text'
  },
  {
    directiveName: 'boHtml',
    attribute: 'html'
  },
  {
    directiveName: 'boSrcI',
    attribute: 'src',
    interpolate: true
  },
  {
    directiveName: 'boSrc',
    attribute: 'src'
  },
  {
    directiveName: 'boHrefI',
    attribute: 'href',
    interpolate: true
  },
  {
    directiveName: 'boHref',
    attribute: 'href'
  },
  {
    directiveName: 'boAlt',
    attribute: 'alt'
  },
  {
    directiveName: 'boTitle',
    attribute: 'title'
  },
  {
    directiveName: 'boId',
    attribute: 'id'
  },
  {
    directiveName: 'boStyle',
    attribute: 'style'
  },
  {
    directiveName: 'boValue',
    attribute: 'value'
  },
  {
    directiveName: 'boAttr',
    attribute: 'attr'
  }
], function (boDirective) {
  var childPriority = 200;
  return angular.module('pasvaz.bindonce').directive(boDirective.directiveName, function () {
    var bindonceDirective = {
        priority: boDirective.priority || childPriority,
        transclude: boDirective.transclude || false,
        terminal: boDirective.terminal || false,
        require: '^bindonce',
        compile: function (tElement, tAttrs, transclude) {
          return function (scope, elm, attrs, bindonceController) {
            var name = attrs.boParent;
            if (name && bindonceController.group != name) {
              var element = bindonceController.element.parent();
              bindonceController = undefined;
              var parentValue;
              while (element[0].nodeType != 9 && element.length) {
                if ((parentValue = element.data('$bindonceController')) && parentValue.group == name) {
                  bindonceController = parentValue;
                  break;
                }
                element = element.parent();
              }
              if (!bindonceController) {
                throw Error('No bindonce controller: ' + name);
              }
            }
            bindonceController.addBinder({
              element: elm,
              attr: boDirective.attribute,
              attrs: attrs,
              value: attrs[boDirective.directiveName],
              interpolate: boDirective.interpolate,
              group: name,
              transclude: transclude,
              scope: scope
            });
          };
        }
      };
    return bindonceDirective;
  });
});
/*! jQuery UI - v1.10.4 - 2014-04-02
* http://jqueryui.com
* Includes: jquery.ui.core.js, jquery.ui.widget.js, jquery.ui.mouse.js, jquery.ui.position.js, jquery.ui.accordion.js, jquery.ui.autocomplete.js, jquery.ui.button.js, jquery.ui.datepicker.js, jquery.ui.dialog.js, jquery.ui.draggable.js, jquery.ui.droppable.js, jquery.ui.effect.js, jquery.ui.effect-blind.js, jquery.ui.effect-bounce.js, jquery.ui.effect-clip.js, jquery.ui.effect-drop.js, jquery.ui.effect-explode.js, jquery.ui.effect-fade.js, jquery.ui.effect-fold.js, jquery.ui.effect-highlight.js, jquery.ui.effect-pulsate.js, jquery.ui.effect-scale.js, jquery.ui.effect-shake.js, jquery.ui.effect-slide.js, jquery.ui.effect-transfer.js, jquery.ui.menu.js, jquery.ui.progressbar.js, jquery.ui.resizable.js, jquery.ui.selectable.js, jquery.ui.slider.js, jquery.ui.sortable.js, jquery.ui.spinner.js, jquery.ui.tabs.js, jquery.ui.tooltip.js
* Copyright 2014 jQuery Foundation and other contributors; Licensed MIT */
(function ($, undefined) {
  var uuid = 0, runiqueId = /^ui-id-\d+$/;
  // $.ui might exist from components with no dependencies, e.g., $.ui.position
  $.ui = $.ui || {};
  $.extend($.ui, {
    version: '1.10.4',
    keyCode: {
      BACKSPACE: 8,
      COMMA: 188,
      DELETE: 46,
      DOWN: 40,
      END: 35,
      ENTER: 13,
      ESCAPE: 27,
      HOME: 36,
      LEFT: 37,
      NUMPAD_ADD: 107,
      NUMPAD_DECIMAL: 110,
      NUMPAD_DIVIDE: 111,
      NUMPAD_ENTER: 108,
      NUMPAD_MULTIPLY: 106,
      NUMPAD_SUBTRACT: 109,
      PAGE_DOWN: 34,
      PAGE_UP: 33,
      PERIOD: 190,
      RIGHT: 39,
      SPACE: 32,
      TAB: 9,
      UP: 38
    }
  });
  // plugins
  $.fn.extend({
    focus: function (orig) {
      return function (delay, fn) {
        return typeof delay === 'number' ? this.each(function () {
          var elem = this;
          setTimeout(function () {
            $(elem).focus();
            if (fn) {
              fn.call(elem);
            }
          }, delay);
        }) : orig.apply(this, arguments);
      };
    }($.fn.focus),
    scrollParent: function () {
      var scrollParent;
      if ($.ui.ie && /(static|relative)/.test(this.css('position')) || /absolute/.test(this.css('position'))) {
        scrollParent = this.parents().filter(function () {
          return /(relative|absolute|fixed)/.test($.css(this, 'position')) && /(auto|scroll)/.test($.css(this, 'overflow') + $.css(this, 'overflow-y') + $.css(this, 'overflow-x'));
        }).eq(0);
      } else {
        scrollParent = this.parents().filter(function () {
          return /(auto|scroll)/.test($.css(this, 'overflow') + $.css(this, 'overflow-y') + $.css(this, 'overflow-x'));
        }).eq(0);
      }
      return /fixed/.test(this.css('position')) || !scrollParent.length ? $(document) : scrollParent;
    },
    zIndex: function (zIndex) {
      if (zIndex !== undefined) {
        return this.css('zIndex', zIndex);
      }
      if (this.length) {
        var elem = $(this[0]), position, value;
        while (elem.length && elem[0] !== document) {
          // Ignore z-index if position is set to a value where z-index is ignored by the browser
          // This makes behavior of this function consistent across browsers
          // WebKit always returns auto if the element is positioned
          position = elem.css('position');
          if (position === 'absolute' || position === 'relative' || position === 'fixed') {
            // IE returns 0 when zIndex is not specified
            // other browsers return a string
            // we ignore the case of nested elements with an explicit value of 0
            // <div style="z-index: -10;"><div style="z-index: 0;"></div></div>
            value = parseInt(elem.css('zIndex'), 10);
            if (!isNaN(value) && value !== 0) {
              return value;
            }
          }
          elem = elem.parent();
        }
      }
      return 0;
    },
    uniqueId: function () {
      return this.each(function () {
        if (!this.id) {
          this.id = 'ui-id-' + ++uuid;
        }
      });
    },
    removeUniqueId: function () {
      return this.each(function () {
        if (runiqueId.test(this.id)) {
          $(this).removeAttr('id');
        }
      });
    }
  });
  // selectors
  function focusable(element, isTabIndexNotNaN) {
    var map, mapName, img, nodeName = element.nodeName.toLowerCase();
    if ('area' === nodeName) {
      map = element.parentNode;
      mapName = map.name;
      if (!element.href || !mapName || map.nodeName.toLowerCase() !== 'map') {
        return false;
      }
      img = $('img[usemap=#' + mapName + ']')[0];
      return !!img && visible(img);
    }
    return (/input|select|textarea|button|object/.test(nodeName) ? !element.disabled : 'a' === nodeName ? element.href || isTabIndexNotNaN : isTabIndexNotNaN) && visible(element);
  }
  function visible(element) {
    return $.expr.filters.visible(element) && !$(element).parents().addBack().filter(function () {
      return $.css(this, 'visibility') === 'hidden';
    }).length;
  }
  $.extend($.expr[':'], {
    data: $.expr.createPseudo ? $.expr.createPseudo(function (dataName) {
      return function (elem) {
        return !!$.data(elem, dataName);
      };
    }) : function (elem, i, match) {
      return !!$.data(elem, match[3]);
    },
    focusable: function (element) {
      return focusable(element, !isNaN($.attr(element, 'tabindex')));
    },
    tabbable: function (element) {
      var tabIndex = $.attr(element, 'tabindex'), isTabIndexNaN = isNaN(tabIndex);
      return (isTabIndexNaN || tabIndex >= 0) && focusable(element, !isTabIndexNaN);
    }
  });
  // support: jQuery <1.8
  if (!$('<a>').outerWidth(1).jquery) {
    $.each([
      'Width',
      'Height'
    ], function (i, name) {
      var side = name === 'Width' ? [
          'Left',
          'Right'
        ] : [
          'Top',
          'Bottom'
        ], type = name.toLowerCase(), orig = {
          innerWidth: $.fn.innerWidth,
          innerHeight: $.fn.innerHeight,
          outerWidth: $.fn.outerWidth,
          outerHeight: $.fn.outerHeight
        };
      function reduce(elem, size, border, margin) {
        $.each(side, function () {
          size -= parseFloat($.css(elem, 'padding' + this)) || 0;
          if (border) {
            size -= parseFloat($.css(elem, 'border' + this + 'Width')) || 0;
          }
          if (margin) {
            size -= parseFloat($.css(elem, 'margin' + this)) || 0;
          }
        });
        return size;
      }
      $.fn['inner' + name] = function (size) {
        if (size === undefined) {
          return orig['inner' + name].call(this);
        }
        return this.each(function () {
          $(this).css(type, reduce(this, size) + 'px');
        });
      };
      $.fn['outer' + name] = function (size, margin) {
        if (typeof size !== 'number') {
          return orig['outer' + name].call(this, size);
        }
        return this.each(function () {
          $(this).css(type, reduce(this, size, true, margin) + 'px');
        });
      };
    });
  }
  // support: jQuery <1.8
  if (!$.fn.addBack) {
    $.fn.addBack = function (selector) {
      return this.add(selector == null ? this.prevObject : this.prevObject.filter(selector));
    };
  }
  // support: jQuery 1.6.1, 1.6.2 (http://bugs.jquery.com/ticket/9413)
  if ($('<a>').data('a-b', 'a').removeData('a-b').data('a-b')) {
    $.fn.removeData = function (removeData) {
      return function (key) {
        if (arguments.length) {
          return removeData.call(this, $.camelCase(key));
        } else {
          return removeData.call(this);
        }
      };
    }($.fn.removeData);
  }
  // deprecated
  $.ui.ie = !!/msie [\w.]+/.exec(navigator.userAgent.toLowerCase());
  $.support.selectstart = 'onselectstart' in document.createElement('div');
  $.fn.extend({
    disableSelection: function () {
      return this.bind(($.support.selectstart ? 'selectstart' : 'mousedown') + '.ui-disableSelection', function (event) {
        event.preventDefault();
      });
    },
    enableSelection: function () {
      return this.unbind('.ui-disableSelection');
    }
  });
  $.extend($.ui, {
    plugin: {
      add: function (module, option, set) {
        var i, proto = $.ui[module].prototype;
        for (i in set) {
          proto.plugins[i] = proto.plugins[i] || [];
          proto.plugins[i].push([
            option,
            set[i]
          ]);
        }
      },
      call: function (instance, name, args) {
        var i, set = instance.plugins[name];
        if (!set || !instance.element[0].parentNode || instance.element[0].parentNode.nodeType === 11) {
          return;
        }
        for (i = 0; i < set.length; i++) {
          if (instance.options[set[i][0]]) {
            set[i][1].apply(instance.element, args);
          }
        }
      }
    },
    hasScroll: function (el, a) {
      //If overflow is hidden, the element might have extra content, but the user wants to hide it
      if ($(el).css('overflow') === 'hidden') {
        return false;
      }
      var scroll = a && a === 'left' ? 'scrollLeft' : 'scrollTop', has = false;
      if (el[scroll] > 0) {
        return true;
      }
      // TODO: determine which cases actually cause this to happen
      // if the element doesn't have the scroll set, see if it's possible to
      // set the scroll
      el[scroll] = 1;
      has = el[scroll] > 0;
      el[scroll] = 0;
      return has;
    }
  });
}(jQuery));
(function ($, undefined) {
  var uuid = 0, slice = Array.prototype.slice, _cleanData = $.cleanData;
  $.cleanData = function (elems) {
    for (var i = 0, elem; (elem = elems[i]) != null; i++) {
      try {
        $(elem).triggerHandler('remove');  // http://bugs.jquery.com/ticket/8235
      } catch (e) {
      }
    }
    _cleanData(elems);
  };
  $.widget = function (name, base, prototype) {
    var fullName, existingConstructor, constructor, basePrototype,
      // proxiedPrototype allows the provided prototype to remain unmodified
      // so that it can be used as a mixin for multiple widgets (#8876)
      proxiedPrototype = {}, namespace = name.split('.')[0];
    name = name.split('.')[1];
    fullName = namespace + '-' + name;
    if (!prototype) {
      prototype = base;
      base = $.Widget;
    }
    // create selector for plugin
    $.expr[':'][fullName.toLowerCase()] = function (elem) {
      return !!$.data(elem, fullName);
    };
    $[namespace] = $[namespace] || {};
    existingConstructor = $[namespace][name];
    constructor = $[namespace][name] = function (options, element) {
      // allow instantiation without "new" keyword
      if (!this._createWidget) {
        return new constructor(options, element);
      }
      // allow instantiation without initializing for simple inheritance
      // must use "new" keyword (the code above always passes args)
      if (arguments.length) {
        this._createWidget(options, element);
      }
    };
    // extend with the existing constructor to carry over any static properties
    $.extend(constructor, existingConstructor, {
      version: prototype.version,
      _proto: $.extend({}, prototype),
      _childConstructors: []
    });
    basePrototype = new base();
    // we need to make the options hash a property directly on the new instance
    // otherwise we'll modify the options hash on the prototype that we're
    // inheriting from
    basePrototype.options = $.widget.extend({}, basePrototype.options);
    $.each(prototype, function (prop, value) {
      if (!$.isFunction(value)) {
        proxiedPrototype[prop] = value;
        return;
      }
      proxiedPrototype[prop] = function () {
        var _super = function () {
            return base.prototype[prop].apply(this, arguments);
          }, _superApply = function (args) {
            return base.prototype[prop].apply(this, args);
          };
        return function () {
          var __super = this._super, __superApply = this._superApply, returnValue;
          this._super = _super;
          this._superApply = _superApply;
          returnValue = value.apply(this, arguments);
          this._super = __super;
          this._superApply = __superApply;
          return returnValue;
        };
      }();
    });
    constructor.prototype = $.widget.extend(basePrototype, { widgetEventPrefix: existingConstructor ? basePrototype.widgetEventPrefix || name : name }, proxiedPrototype, {
      constructor: constructor,
      namespace: namespace,
      widgetName: name,
      widgetFullName: fullName
    });
    // If this widget is being redefined then we need to find all widgets that
    // are inheriting from it and redefine all of them so that they inherit from
    // the new version of this widget. We're essentially trying to replace one
    // level in the prototype chain.
    if (existingConstructor) {
      $.each(existingConstructor._childConstructors, function (i, child) {
        var childPrototype = child.prototype;
        // redefine the child widget using the same prototype that was
        // originally used, but inherit from the new version of the base
        $.widget(childPrototype.namespace + '.' + childPrototype.widgetName, constructor, child._proto);
      });
      // remove the list of existing child constructors from the old constructor
      // so the old child constructors can be garbage collected
      delete existingConstructor._childConstructors;
    } else {
      base._childConstructors.push(constructor);
    }
    $.widget.bridge(name, constructor);
  };
  $.widget.extend = function (target) {
    var input = slice.call(arguments, 1), inputIndex = 0, inputLength = input.length, key, value;
    for (; inputIndex < inputLength; inputIndex++) {
      for (key in input[inputIndex]) {
        value = input[inputIndex][key];
        if (input[inputIndex].hasOwnProperty(key) && value !== undefined) {
          // Clone objects
          if ($.isPlainObject(value)) {
            target[key] = $.isPlainObject(target[key]) ? $.widget.extend({}, target[key], value) : $.widget.extend({}, value);  // Copy everything else by reference
          } else {
            target[key] = value;
          }
        }
      }
    }
    return target;
  };
  $.widget.bridge = function (name, object) {
    var fullName = object.prototype.widgetFullName || name;
    $.fn[name] = function (options) {
      var isMethodCall = typeof options === 'string', args = slice.call(arguments, 1), returnValue = this;
      // allow multiple hashes to be passed on init
      options = !isMethodCall && args.length ? $.widget.extend.apply(null, [options].concat(args)) : options;
      if (isMethodCall) {
        this.each(function () {
          var methodValue, instance = $.data(this, fullName);
          if (!instance) {
            return $.error('cannot call methods on ' + name + ' prior to initialization; ' + 'attempted to call method \'' + options + '\'');
          }
          if (!$.isFunction(instance[options]) || options.charAt(0) === '_') {
            return $.error('no such method \'' + options + '\' for ' + name + ' widget instance');
          }
          methodValue = instance[options].apply(instance, args);
          if (methodValue !== instance && methodValue !== undefined) {
            returnValue = methodValue && methodValue.jquery ? returnValue.pushStack(methodValue.get()) : methodValue;
            return false;
          }
        });
      } else {
        this.each(function () {
          var instance = $.data(this, fullName);
          if (instance) {
            instance.option(options || {})._init();
          } else {
            $.data(this, fullName, new object(options, this));
          }
        });
      }
      return returnValue;
    };
  };
  $.Widget = function () {
  };
  $.Widget._childConstructors = [];
  $.Widget.prototype = {
    widgetName: 'widget',
    widgetEventPrefix: '',
    defaultElement: '<div>',
    options: {
      disabled: false,
      create: null
    },
    _createWidget: function (options, element) {
      element = $(element || this.defaultElement || this)[0];
      this.element = $(element);
      this.uuid = uuid++;
      this.eventNamespace = '.' + this.widgetName + this.uuid;
      this.options = $.widget.extend({}, this.options, this._getCreateOptions(), options);
      this.bindings = $();
      this.hoverable = $();
      this.focusable = $();
      if (element !== this) {
        $.data(element, this.widgetFullName, this);
        this._on(true, this.element, {
          remove: function (event) {
            if (event.target === element) {
              this.destroy();
            }
          }
        });
        this.document = $(element.style ? element.ownerDocument : element.document || element);
        this.window = $(this.document[0].defaultView || this.document[0].parentWindow);
      }
      this._create();
      this._trigger('create', null, this._getCreateEventData());
      this._init();
    },
    _getCreateOptions: $.noop,
    _getCreateEventData: $.noop,
    _create: $.noop,
    _init: $.noop,
    destroy: function () {
      this._destroy();
      // we can probably remove the unbind calls in 2.0
      // all event bindings should go through this._on()
      this.element.unbind(this.eventNamespace).removeData(this.widgetName).removeData(this.widgetFullName).removeData($.camelCase(this.widgetFullName));
      this.widget().unbind(this.eventNamespace).removeAttr('aria-disabled').removeClass(this.widgetFullName + '-disabled ' + 'ui-state-disabled');
      // clean up events and states
      this.bindings.unbind(this.eventNamespace);
      this.hoverable.removeClass('ui-state-hover');
      this.focusable.removeClass('ui-state-focus');
    },
    _destroy: $.noop,
    widget: function () {
      return this.element;
    },
    option: function (key, value) {
      var options = key, parts, curOption, i;
      if (arguments.length === 0) {
        // don't return a reference to the internal hash
        return $.widget.extend({}, this.options);
      }
      if (typeof key === 'string') {
        // handle nested keys, e.g., "foo.bar" => { foo: { bar: ___ } }
        options = {};
        parts = key.split('.');
        key = parts.shift();
        if (parts.length) {
          curOption = options[key] = $.widget.extend({}, this.options[key]);
          for (i = 0; i < parts.length - 1; i++) {
            curOption[parts[i]] = curOption[parts[i]] || {};
            curOption = curOption[parts[i]];
          }
          key = parts.pop();
          if (arguments.length === 1) {
            return curOption[key] === undefined ? null : curOption[key];
          }
          curOption[key] = value;
        } else {
          if (arguments.length === 1) {
            return this.options[key] === undefined ? null : this.options[key];
          }
          options[key] = value;
        }
      }
      this._setOptions(options);
      return this;
    },
    _setOptions: function (options) {
      var key;
      for (key in options) {
        this._setOption(key, options[key]);
      }
      return this;
    },
    _setOption: function (key, value) {
      this.options[key] = value;
      if (key === 'disabled') {
        this.widget().toggleClass(this.widgetFullName + '-disabled ui-state-disabled', !!value).attr('aria-disabled', value);
        this.hoverable.removeClass('ui-state-hover');
        this.focusable.removeClass('ui-state-focus');
      }
      return this;
    },
    enable: function () {
      return this._setOption('disabled', false);
    },
    disable: function () {
      return this._setOption('disabled', true);
    },
    _on: function (suppressDisabledCheck, element, handlers) {
      var delegateElement, instance = this;
      // no suppressDisabledCheck flag, shuffle arguments
      if (typeof suppressDisabledCheck !== 'boolean') {
        handlers = element;
        element = suppressDisabledCheck;
        suppressDisabledCheck = false;
      }
      // no element argument, shuffle and use this.element
      if (!handlers) {
        handlers = element;
        element = this.element;
        delegateElement = this.widget();
      } else {
        // accept selectors, DOM elements
        element = delegateElement = $(element);
        this.bindings = this.bindings.add(element);
      }
      $.each(handlers, function (event, handler) {
        function handlerProxy() {
          // allow widgets to customize the disabled handling
          // - disabled as an array instead of boolean
          // - disabled class as method for disabling individual parts
          if (!suppressDisabledCheck && (instance.options.disabled === true || $(this).hasClass('ui-state-disabled'))) {
            return;
          }
          return (typeof handler === 'string' ? instance[handler] : handler).apply(instance, arguments);
        }
        // copy the guid so direct unbinding works
        if (typeof handler !== 'string') {
          handlerProxy.guid = handler.guid = handler.guid || handlerProxy.guid || $.guid++;
        }
        var match = event.match(/^(\w+)\s*(.*)$/), eventName = match[1] + instance.eventNamespace, selector = match[2];
        if (selector) {
          delegateElement.delegate(selector, eventName, handlerProxy);
        } else {
          element.bind(eventName, handlerProxy);
        }
      });
    },
    _off: function (element, eventName) {
      eventName = (eventName || '').split(' ').join(this.eventNamespace + ' ') + this.eventNamespace;
      element.unbind(eventName).undelegate(eventName);
    },
    _delay: function (handler, delay) {
      function handlerProxy() {
        return (typeof handler === 'string' ? instance[handler] : handler).apply(instance, arguments);
      }
      var instance = this;
      return setTimeout(handlerProxy, delay || 0);
    },
    _hoverable: function (element) {
      this.hoverable = this.hoverable.add(element);
      this._on(element, {
        mouseenter: function (event) {
          $(event.currentTarget).addClass('ui-state-hover');
        },
        mouseleave: function (event) {
          $(event.currentTarget).removeClass('ui-state-hover');
        }
      });
    },
    _focusable: function (element) {
      this.focusable = this.focusable.add(element);
      this._on(element, {
        focusin: function (event) {
          $(event.currentTarget).addClass('ui-state-focus');
        },
        focusout: function (event) {
          $(event.currentTarget).removeClass('ui-state-focus');
        }
      });
    },
    _trigger: function (type, event, data) {
      var prop, orig, callback = this.options[type];
      data = data || {};
      event = $.Event(event);
      event.type = (type === this.widgetEventPrefix ? type : this.widgetEventPrefix + type).toLowerCase();
      // the original event may come from any element
      // so we need to reset the target on the new event
      event.target = this.element[0];
      // copy original event properties over to the new event
      orig = event.originalEvent;
      if (orig) {
        for (prop in orig) {
          if (!(prop in event)) {
            event[prop] = orig[prop];
          }
        }
      }
      this.element.trigger(event, data);
      return !($.isFunction(callback) && callback.apply(this.element[0], [event].concat(data)) === false || event.isDefaultPrevented());
    }
  };
  $.each({
    show: 'fadeIn',
    hide: 'fadeOut'
  }, function (method, defaultEffect) {
    $.Widget.prototype['_' + method] = function (element, options, callback) {
      if (typeof options === 'string') {
        options = { effect: options };
      }
      var hasOptions, effectName = !options ? method : options === true || typeof options === 'number' ? defaultEffect : options.effect || defaultEffect;
      options = options || {};
      if (typeof options === 'number') {
        options = { duration: options };
      }
      hasOptions = !$.isEmptyObject(options);
      options.complete = callback;
      if (options.delay) {
        element.delay(options.delay);
      }
      if (hasOptions && $.effects && $.effects.effect[effectName]) {
        element[method](options);
      } else if (effectName !== method && element[effectName]) {
        element[effectName](options.duration, options.easing, callback);
      } else {
        element.queue(function (next) {
          $(this)[method]();
          if (callback) {
            callback.call(element[0]);
          }
          next();
        });
      }
    };
  });
}(jQuery));
(function ($, undefined) {
  var mouseHandled = false;
  $(document).mouseup(function () {
    mouseHandled = false;
  });
  $.widget('ui.mouse', {
    version: '1.10.4',
    options: {
      cancel: 'input,textarea,button,select,option',
      distance: 1,
      delay: 0
    },
    _mouseInit: function () {
      var that = this;
      this.element.bind('mousedown.' + this.widgetName, function (event) {
        return that._mouseDown(event);
      }).bind('click.' + this.widgetName, function (event) {
        if (true === $.data(event.target, that.widgetName + '.preventClickEvent')) {
          $.removeData(event.target, that.widgetName + '.preventClickEvent');
          event.stopImmediatePropagation();
          return false;
        }
      });
      this.started = false;
    },
    _mouseDestroy: function () {
      this.element.unbind('.' + this.widgetName);
      if (this._mouseMoveDelegate) {
        $(document).unbind('mousemove.' + this.widgetName, this._mouseMoveDelegate).unbind('mouseup.' + this.widgetName, this._mouseUpDelegate);
      }
    },
    _mouseDown: function (event) {
      // don't let more than one widget handle mouseStart
      if (mouseHandled) {
        return;
      }
      // we may have missed mouseup (out of window)
      this._mouseStarted && this._mouseUp(event);
      this._mouseDownEvent = event;
      var that = this, btnIsLeft = event.which === 1,
        // event.target.nodeName works around a bug in IE 8 with
        // disabled inputs (#7620)
        elIsCancel = typeof this.options.cancel === 'string' && event.target.nodeName ? $(event.target).closest(this.options.cancel).length : false;
      if (!btnIsLeft || elIsCancel || !this._mouseCapture(event)) {
        return true;
      }
      this.mouseDelayMet = !this.options.delay;
      if (!this.mouseDelayMet) {
        this._mouseDelayTimer = setTimeout(function () {
          that.mouseDelayMet = true;
        }, this.options.delay);
      }
      if (this._mouseDistanceMet(event) && this._mouseDelayMet(event)) {
        this._mouseStarted = this._mouseStart(event) !== false;
        if (!this._mouseStarted) {
          event.preventDefault();
          return true;
        }
      }
      // Click event may never have fired (Gecko & Opera)
      if (true === $.data(event.target, this.widgetName + '.preventClickEvent')) {
        $.removeData(event.target, this.widgetName + '.preventClickEvent');
      }
      // these delegates are required to keep context
      this._mouseMoveDelegate = function (event) {
        return that._mouseMove(event);
      };
      this._mouseUpDelegate = function (event) {
        return that._mouseUp(event);
      };
      $(document).bind('mousemove.' + this.widgetName, this._mouseMoveDelegate).bind('mouseup.' + this.widgetName, this._mouseUpDelegate);
      event.preventDefault();
      mouseHandled = true;
      return true;
    },
    _mouseMove: function (event) {
      // IE mouseup check - mouseup happened when mouse was out of window
      if ($.ui.ie && (!document.documentMode || document.documentMode < 9) && !event.button) {
        return this._mouseUp(event);
      }
      if (this._mouseStarted) {
        this._mouseDrag(event);
        return event.preventDefault();
      }
      if (this._mouseDistanceMet(event) && this._mouseDelayMet(event)) {
        this._mouseStarted = this._mouseStart(this._mouseDownEvent, event) !== false;
        this._mouseStarted ? this._mouseDrag(event) : this._mouseUp(event);
      }
      return !this._mouseStarted;
    },
    _mouseUp: function (event) {
      $(document).unbind('mousemove.' + this.widgetName, this._mouseMoveDelegate).unbind('mouseup.' + this.widgetName, this._mouseUpDelegate);
      if (this._mouseStarted) {
        this._mouseStarted = false;
        if (event.target === this._mouseDownEvent.target) {
          $.data(event.target, this.widgetName + '.preventClickEvent', true);
        }
        this._mouseStop(event);
      }
      return false;
    },
    _mouseDistanceMet: function (event) {
      return Math.max(Math.abs(this._mouseDownEvent.pageX - event.pageX), Math.abs(this._mouseDownEvent.pageY - event.pageY)) >= this.options.distance;
    },
    _mouseDelayMet: function () {
      return this.mouseDelayMet;
    },
    _mouseStart: function () {
    },
    _mouseDrag: function () {
    },
    _mouseStop: function () {
    },
    _mouseCapture: function () {
      return true;
    }
  });
}(jQuery));
(function ($, undefined) {
  $.ui = $.ui || {};
  var cachedScrollbarWidth, max = Math.max, abs = Math.abs, round = Math.round, rhorizontal = /left|center|right/, rvertical = /top|center|bottom/, roffset = /[\+\-]\d+(\.[\d]+)?%?/, rposition = /^\w+/, rpercent = /%$/, _position = $.fn.position;
  function getOffsets(offsets, width, height) {
    return [
      parseFloat(offsets[0]) * (rpercent.test(offsets[0]) ? width / 100 : 1),
      parseFloat(offsets[1]) * (rpercent.test(offsets[1]) ? height / 100 : 1)
    ];
  }
  function parseCss(element, property) {
    return parseInt($.css(element, property), 10) || 0;
  }
  function getDimensions(elem) {
    var raw = elem[0];
    if (raw.nodeType === 9) {
      return {
        width: elem.width(),
        height: elem.height(),
        offset: {
          top: 0,
          left: 0
        }
      };
    }
    if ($.isWindow(raw)) {
      return {
        width: elem.width(),
        height: elem.height(),
        offset: {
          top: elem.scrollTop(),
          left: elem.scrollLeft()
        }
      };
    }
    if (raw.preventDefault) {
      return {
        width: 0,
        height: 0,
        offset: {
          top: raw.pageY,
          left: raw.pageX
        }
      };
    }
    return {
      width: elem.outerWidth(),
      height: elem.outerHeight(),
      offset: elem.offset()
    };
  }
  $.position = {
    scrollbarWidth: function () {
      if (cachedScrollbarWidth !== undefined) {
        return cachedScrollbarWidth;
      }
      var w1, w2, div = $('<div style=\'display:block;position:absolute;width:50px;height:50px;overflow:hidden;\'><div style=\'height:100px;width:auto;\'></div></div>'), innerDiv = div.children()[0];
      $('body').append(div);
      w1 = innerDiv.offsetWidth;
      div.css('overflow', 'scroll');
      w2 = innerDiv.offsetWidth;
      if (w1 === w2) {
        w2 = div[0].clientWidth;
      }
      div.remove();
      return cachedScrollbarWidth = w1 - w2;
    },
    getScrollInfo: function (within) {
      var overflowX = within.isWindow || within.isDocument ? '' : within.element.css('overflow-x'), overflowY = within.isWindow || within.isDocument ? '' : within.element.css('overflow-y'), hasOverflowX = overflowX === 'scroll' || overflowX === 'auto' && within.width < within.element[0].scrollWidth, hasOverflowY = overflowY === 'scroll' || overflowY === 'auto' && within.height < within.element[0].scrollHeight;
      return {
        width: hasOverflowY ? $.position.scrollbarWidth() : 0,
        height: hasOverflowX ? $.position.scrollbarWidth() : 0
      };
    },
    getWithinInfo: function (element) {
      var withinElement = $(element || window), isWindow = $.isWindow(withinElement[0]), isDocument = !!withinElement[0] && withinElement[0].nodeType === 9;
      return {
        element: withinElement,
        isWindow: isWindow,
        isDocument: isDocument,
        offset: withinElement.offset() || {
          left: 0,
          top: 0
        },
        scrollLeft: withinElement.scrollLeft(),
        scrollTop: withinElement.scrollTop(),
        width: isWindow ? withinElement.width() : withinElement.outerWidth(),
        height: isWindow ? withinElement.height() : withinElement.outerHeight()
      };
    }
  };
  $.fn.position = function (options) {
    if (!options || !options.of) {
      return _position.apply(this, arguments);
    }
    // make a copy, we don't want to modify arguments
    options = $.extend({}, options);
    var atOffset, targetWidth, targetHeight, targetOffset, basePosition, dimensions, target = $(options.of), within = $.position.getWithinInfo(options.within), scrollInfo = $.position.getScrollInfo(within), collision = (options.collision || 'flip').split(' '), offsets = {};
    dimensions = getDimensions(target);
    if (target[0].preventDefault) {
      // force left top to allow flipping
      options.at = 'left top';
    }
    targetWidth = dimensions.width;
    targetHeight = dimensions.height;
    targetOffset = dimensions.offset;
    // clone to reuse original targetOffset later
    basePosition = $.extend({}, targetOffset);
    // force my and at to have valid horizontal and vertical positions
    // if a value is missing or invalid, it will be converted to center
    $.each([
      'my',
      'at'
    ], function () {
      var pos = (options[this] || '').split(' '), horizontalOffset, verticalOffset;
      if (pos.length === 1) {
        pos = rhorizontal.test(pos[0]) ? pos.concat(['center']) : rvertical.test(pos[0]) ? ['center'].concat(pos) : [
          'center',
          'center'
        ];
      }
      pos[0] = rhorizontal.test(pos[0]) ? pos[0] : 'center';
      pos[1] = rvertical.test(pos[1]) ? pos[1] : 'center';
      // calculate offsets
      horizontalOffset = roffset.exec(pos[0]);
      verticalOffset = roffset.exec(pos[1]);
      offsets[this] = [
        horizontalOffset ? horizontalOffset[0] : 0,
        verticalOffset ? verticalOffset[0] : 0
      ];
      // reduce to just the positions without the offsets
      options[this] = [
        rposition.exec(pos[0])[0],
        rposition.exec(pos[1])[0]
      ];
    });
    // normalize collision option
    if (collision.length === 1) {
      collision[1] = collision[0];
    }
    if (options.at[0] === 'right') {
      basePosition.left += targetWidth;
    } else if (options.at[0] === 'center') {
      basePosition.left += targetWidth / 2;
    }
    if (options.at[1] === 'bottom') {
      basePosition.top += targetHeight;
    } else if (options.at[1] === 'center') {
      basePosition.top += targetHeight / 2;
    }
    atOffset = getOffsets(offsets.at, targetWidth, targetHeight);
    basePosition.left += atOffset[0];
    basePosition.top += atOffset[1];
    return this.each(function () {
      var collisionPosition, using, elem = $(this), elemWidth = elem.outerWidth(), elemHeight = elem.outerHeight(), marginLeft = parseCss(this, 'marginLeft'), marginTop = parseCss(this, 'marginTop'), collisionWidth = elemWidth + marginLeft + parseCss(this, 'marginRight') + scrollInfo.width, collisionHeight = elemHeight + marginTop + parseCss(this, 'marginBottom') + scrollInfo.height, position = $.extend({}, basePosition), myOffset = getOffsets(offsets.my, elem.outerWidth(), elem.outerHeight());
      if (options.my[0] === 'right') {
        position.left -= elemWidth;
      } else if (options.my[0] === 'center') {
        position.left -= elemWidth / 2;
      }
      if (options.my[1] === 'bottom') {
        position.top -= elemHeight;
      } else if (options.my[1] === 'center') {
        position.top -= elemHeight / 2;
      }
      position.left += myOffset[0];
      position.top += myOffset[1];
      // if the browser doesn't support fractions, then round for consistent results
      if (!$.support.offsetFractions) {
        position.left = round(position.left);
        position.top = round(position.top);
      }
      collisionPosition = {
        marginLeft: marginLeft,
        marginTop: marginTop
      };
      $.each([
        'left',
        'top'
      ], function (i, dir) {
        if ($.ui.position[collision[i]]) {
          $.ui.position[collision[i]][dir](position, {
            targetWidth: targetWidth,
            targetHeight: targetHeight,
            elemWidth: elemWidth,
            elemHeight: elemHeight,
            collisionPosition: collisionPosition,
            collisionWidth: collisionWidth,
            collisionHeight: collisionHeight,
            offset: [
              atOffset[0] + myOffset[0],
              atOffset[1] + myOffset[1]
            ],
            my: options.my,
            at: options.at,
            within: within,
            elem: elem
          });
        }
      });
      if (options.using) {
        // adds feedback as second argument to using callback, if present
        using = function (props) {
          var left = targetOffset.left - position.left, right = left + targetWidth - elemWidth, top = targetOffset.top - position.top, bottom = top + targetHeight - elemHeight, feedback = {
              target: {
                element: target,
                left: targetOffset.left,
                top: targetOffset.top,
                width: targetWidth,
                height: targetHeight
              },
              element: {
                element: elem,
                left: position.left,
                top: position.top,
                width: elemWidth,
                height: elemHeight
              },
              horizontal: right < 0 ? 'left' : left > 0 ? 'right' : 'center',
              vertical: bottom < 0 ? 'top' : top > 0 ? 'bottom' : 'middle'
            };
          if (targetWidth < elemWidth && abs(left + right) < targetWidth) {
            feedback.horizontal = 'center';
          }
          if (targetHeight < elemHeight && abs(top + bottom) < targetHeight) {
            feedback.vertical = 'middle';
          }
          if (max(abs(left), abs(right)) > max(abs(top), abs(bottom))) {
            feedback.important = 'horizontal';
          } else {
            feedback.important = 'vertical';
          }
          options.using.call(this, props, feedback);
        };
      }
      elem.offset($.extend(position, { using: using }));
    });
  };
  $.ui.position = {
    fit: {
      left: function (position, data) {
        var within = data.within, withinOffset = within.isWindow ? within.scrollLeft : within.offset.left, outerWidth = within.width, collisionPosLeft = position.left - data.collisionPosition.marginLeft, overLeft = withinOffset - collisionPosLeft, overRight = collisionPosLeft + data.collisionWidth - outerWidth - withinOffset, newOverRight;
        // element is wider than within
        if (data.collisionWidth > outerWidth) {
          // element is initially over the left side of within
          if (overLeft > 0 && overRight <= 0) {
            newOverRight = position.left + overLeft + data.collisionWidth - outerWidth - withinOffset;
            position.left += overLeft - newOverRight;  // element is initially over right side of within
          } else if (overRight > 0 && overLeft <= 0) {
            position.left = withinOffset;  // element is initially over both left and right sides of within
          } else {
            if (overLeft > overRight) {
              position.left = withinOffset + outerWidth - data.collisionWidth;
            } else {
              position.left = withinOffset;
            }
          }  // too far left -> align with left edge
        } else if (overLeft > 0) {
          position.left += overLeft;  // too far right -> align with right edge
        } else if (overRight > 0) {
          position.left -= overRight;  // adjust based on position and margin
        } else {
          position.left = max(position.left - collisionPosLeft, position.left);
        }
      },
      top: function (position, data) {
        var within = data.within, withinOffset = within.isWindow ? within.scrollTop : within.offset.top, outerHeight = data.within.height, collisionPosTop = position.top - data.collisionPosition.marginTop, overTop = withinOffset - collisionPosTop, overBottom = collisionPosTop + data.collisionHeight - outerHeight - withinOffset, newOverBottom;
        // element is taller than within
        if (data.collisionHeight > outerHeight) {
          // element is initially over the top of within
          if (overTop > 0 && overBottom <= 0) {
            newOverBottom = position.top + overTop + data.collisionHeight - outerHeight - withinOffset;
            position.top += overTop - newOverBottom;  // element is initially over bottom of within
          } else if (overBottom > 0 && overTop <= 0) {
            position.top = withinOffset;  // element is initially over both top and bottom of within
          } else {
            if (overTop > overBottom) {
              position.top = withinOffset + outerHeight - data.collisionHeight;
            } else {
              position.top = withinOffset;
            }
          }  // too far up -> align with top
        } else if (overTop > 0) {
          position.top += overTop;  // too far down -> align with bottom edge
        } else if (overBottom > 0) {
          position.top -= overBottom;  // adjust based on position and margin
        } else {
          position.top = max(position.top - collisionPosTop, position.top);
        }
      }
    },
    flip: {
      left: function (position, data) {
        var within = data.within, withinOffset = within.offset.left + within.scrollLeft, outerWidth = within.width, offsetLeft = within.isWindow ? within.scrollLeft : within.offset.left, collisionPosLeft = position.left - data.collisionPosition.marginLeft, overLeft = collisionPosLeft - offsetLeft, overRight = collisionPosLeft + data.collisionWidth - outerWidth - offsetLeft, myOffset = data.my[0] === 'left' ? -data.elemWidth : data.my[0] === 'right' ? data.elemWidth : 0, atOffset = data.at[0] === 'left' ? data.targetWidth : data.at[0] === 'right' ? -data.targetWidth : 0, offset = -2 * data.offset[0], newOverRight, newOverLeft;
        if (overLeft < 0) {
          newOverRight = position.left + myOffset + atOffset + offset + data.collisionWidth - outerWidth - withinOffset;
          if (newOverRight < 0 || newOverRight < abs(overLeft)) {
            position.left += myOffset + atOffset + offset;
          }
        } else if (overRight > 0) {
          newOverLeft = position.left - data.collisionPosition.marginLeft + myOffset + atOffset + offset - offsetLeft;
          if (newOverLeft > 0 || abs(newOverLeft) < overRight) {
            position.left += myOffset + atOffset + offset;
          }
        }
      },
      top: function (position, data) {
        var within = data.within, withinOffset = within.offset.top + within.scrollTop, outerHeight = within.height, offsetTop = within.isWindow ? within.scrollTop : within.offset.top, collisionPosTop = position.top - data.collisionPosition.marginTop, overTop = collisionPosTop - offsetTop, overBottom = collisionPosTop + data.collisionHeight - outerHeight - offsetTop, top = data.my[1] === 'top', myOffset = top ? -data.elemHeight : data.my[1] === 'bottom' ? data.elemHeight : 0, atOffset = data.at[1] === 'top' ? data.targetHeight : data.at[1] === 'bottom' ? -data.targetHeight : 0, offset = -2 * data.offset[1], newOverTop, newOverBottom;
        if (overTop < 0) {
          newOverBottom = position.top + myOffset + atOffset + offset + data.collisionHeight - outerHeight - withinOffset;
          if (position.top + myOffset + atOffset + offset > overTop && (newOverBottom < 0 || newOverBottom < abs(overTop))) {
            position.top += myOffset + atOffset + offset;
          }
        } else if (overBottom > 0) {
          newOverTop = position.top - data.collisionPosition.marginTop + myOffset + atOffset + offset - offsetTop;
          if (position.top + myOffset + atOffset + offset > overBottom && (newOverTop > 0 || abs(newOverTop) < overBottom)) {
            position.top += myOffset + atOffset + offset;
          }
        }
      }
    },
    flipfit: {
      left: function () {
        $.ui.position.flip.left.apply(this, arguments);
        $.ui.position.fit.left.apply(this, arguments);
      },
      top: function () {
        $.ui.position.flip.top.apply(this, arguments);
        $.ui.position.fit.top.apply(this, arguments);
      }
    }
  };
  // fraction support test
  (function () {
    var testElement, testElementParent, testElementStyle, offsetLeft, i, body = document.getElementsByTagName('body')[0], div = document.createElement('div');
    //Create a "fake body" for testing based on method used in jQuery.support
    testElement = document.createElement(body ? 'div' : 'body');
    testElementStyle = {
      visibility: 'hidden',
      width: 0,
      height: 0,
      border: 0,
      margin: 0,
      background: 'none'
    };
    if (body) {
      $.extend(testElementStyle, {
        position: 'absolute',
        left: '-1000px',
        top: '-1000px'
      });
    }
    for (i in testElementStyle) {
      testElement.style[i] = testElementStyle[i];
    }
    testElement.appendChild(div);
    testElementParent = body || document.documentElement;
    testElementParent.insertBefore(testElement, testElementParent.firstChild);
    div.style.cssText = 'position: absolute; left: 10.7432222px;';
    offsetLeft = $(div).offset().left;
    $.support.offsetFractions = offsetLeft > 10 && offsetLeft < 11;
    testElement.innerHTML = '';
    testElementParent.removeChild(testElement);
  }());
}(jQuery));
(function ($, undefined) {
  var uid = 0, hideProps = {}, showProps = {};
  hideProps.height = hideProps.paddingTop = hideProps.paddingBottom = hideProps.borderTopWidth = hideProps.borderBottomWidth = 'hide';
  showProps.height = showProps.paddingTop = showProps.paddingBottom = showProps.borderTopWidth = showProps.borderBottomWidth = 'show';
  $.widget('ui.accordion', {
    version: '1.10.4',
    options: {
      active: 0,
      animate: {},
      collapsible: false,
      event: 'click',
      header: '> li > :first-child,> :not(li):even',
      heightStyle: 'auto',
      icons: {
        activeHeader: 'ui-icon-triangle-1-s',
        header: 'ui-icon-triangle-1-e'
      },
      activate: null,
      beforeActivate: null
    },
    _create: function () {
      var options = this.options;
      this.prevShow = this.prevHide = $();
      this.element.addClass('ui-accordion ui-widget ui-helper-reset').attr('role', 'tablist');
      // don't allow collapsible: false and active: false / null
      if (!options.collapsible && (options.active === false || options.active == null)) {
        options.active = 0;
      }
      this._processPanels();
      // handle negative values
      if (options.active < 0) {
        options.active += this.headers.length;
      }
      this._refresh();
    },
    _getCreateEventData: function () {
      return {
        header: this.active,
        panel: !this.active.length ? $() : this.active.next(),
        content: !this.active.length ? $() : this.active.next()
      };
    },
    _createIcons: function () {
      var icons = this.options.icons;
      if (icons) {
        $('<span>').addClass('ui-accordion-header-icon ui-icon ' + icons.header).prependTo(this.headers);
        this.active.children('.ui-accordion-header-icon').removeClass(icons.header).addClass(icons.activeHeader);
        this.headers.addClass('ui-accordion-icons');
      }
    },
    _destroyIcons: function () {
      this.headers.removeClass('ui-accordion-icons').children('.ui-accordion-header-icon').remove();
    },
    _destroy: function () {
      var contents;
      // clean up main element
      this.element.removeClass('ui-accordion ui-widget ui-helper-reset').removeAttr('role');
      // clean up headers
      this.headers.removeClass('ui-accordion-header ui-accordion-header-active ui-helper-reset ui-state-default ui-corner-all ui-state-active ui-state-disabled ui-corner-top').removeAttr('role').removeAttr('aria-expanded').removeAttr('aria-selected').removeAttr('aria-controls').removeAttr('tabIndex').each(function () {
        if (/^ui-accordion/.test(this.id)) {
          this.removeAttribute('id');
        }
      });
      this._destroyIcons();
      // clean up content panels
      contents = this.headers.next().css('display', '').removeAttr('role').removeAttr('aria-hidden').removeAttr('aria-labelledby').removeClass('ui-helper-reset ui-widget-content ui-corner-bottom ui-accordion-content ui-accordion-content-active ui-state-disabled').each(function () {
        if (/^ui-accordion/.test(this.id)) {
          this.removeAttribute('id');
        }
      });
      if (this.options.heightStyle !== 'content') {
        contents.css('height', '');
      }
    },
    _setOption: function (key, value) {
      if (key === 'active') {
        // _activate() will handle invalid values and update this.options
        this._activate(value);
        return;
      }
      if (key === 'event') {
        if (this.options.event) {
          this._off(this.headers, this.options.event);
        }
        this._setupEvents(value);
      }
      this._super(key, value);
      // setting collapsible: false while collapsed; open first panel
      if (key === 'collapsible' && !value && this.options.active === false) {
        this._activate(0);
      }
      if (key === 'icons') {
        this._destroyIcons();
        if (value) {
          this._createIcons();
        }
      }
      // #5332 - opacity doesn't cascade to positioned elements in IE
      // so we need to add the disabled class to the headers and panels
      if (key === 'disabled') {
        this.headers.add(this.headers.next()).toggleClass('ui-state-disabled', !!value);
      }
    },
    _keydown: function (event) {
      if (event.altKey || event.ctrlKey) {
        return;
      }
      var keyCode = $.ui.keyCode, length = this.headers.length, currentIndex = this.headers.index(event.target), toFocus = false;
      switch (event.keyCode) {
      case keyCode.RIGHT:
      case keyCode.DOWN:
        toFocus = this.headers[(currentIndex + 1) % length];
        break;
      case keyCode.LEFT:
      case keyCode.UP:
        toFocus = this.headers[(currentIndex - 1 + length) % length];
        break;
      case keyCode.SPACE:
      case keyCode.ENTER:
        this._eventHandler(event);
        break;
      case keyCode.HOME:
        toFocus = this.headers[0];
        break;
      case keyCode.END:
        toFocus = this.headers[length - 1];
        break;
      }
      if (toFocus) {
        $(event.target).attr('tabIndex', -1);
        $(toFocus).attr('tabIndex', 0);
        toFocus.focus();
        event.preventDefault();
      }
    },
    _panelKeyDown: function (event) {
      if (event.keyCode === $.ui.keyCode.UP && event.ctrlKey) {
        $(event.currentTarget).prev().focus();
      }
    },
    refresh: function () {
      var options = this.options;
      this._processPanels();
      // was collapsed or no panel
      if (options.active === false && options.collapsible === true || !this.headers.length) {
        options.active = false;
        this.active = $();  // active false only when collapsible is true
      } else if (options.active === false) {
        this._activate(0);  // was active, but active panel is gone
      } else if (this.active.length && !$.contains(this.element[0], this.active[0])) {
        // all remaining panel are disabled
        if (this.headers.length === this.headers.find('.ui-state-disabled').length) {
          options.active = false;
          this.active = $();  // activate previous panel
        } else {
          this._activate(Math.max(0, options.active - 1));
        }  // was active, active panel still exists
      } else {
        // make sure active index is correct
        options.active = this.headers.index(this.active);
      }
      this._destroyIcons();
      this._refresh();
    },
    _processPanels: function () {
      this.headers = this.element.find(this.options.header).addClass('ui-accordion-header ui-helper-reset ui-state-default ui-corner-all');
      this.headers.next().addClass('ui-accordion-content ui-helper-reset ui-widget-content ui-corner-bottom').filter(':not(.ui-accordion-content-active)').hide();
    },
    _refresh: function () {
      var maxHeight, options = this.options, heightStyle = options.heightStyle, parent = this.element.parent(), accordionId = this.accordionId = 'ui-accordion-' + (this.element.attr('id') || ++uid);
      this.active = this._findActive(options.active).addClass('ui-accordion-header-active ui-state-active ui-corner-top').removeClass('ui-corner-all');
      this.active.next().addClass('ui-accordion-content-active').show();
      this.headers.attr('role', 'tab').each(function (i) {
        var header = $(this), headerId = header.attr('id'), panel = header.next(), panelId = panel.attr('id');
        if (!headerId) {
          headerId = accordionId + '-header-' + i;
          header.attr('id', headerId);
        }
        if (!panelId) {
          panelId = accordionId + '-panel-' + i;
          panel.attr('id', panelId);
        }
        header.attr('aria-controls', panelId);
        panel.attr('aria-labelledby', headerId);
      }).next().attr('role', 'tabpanel');
      this.headers.not(this.active).attr({
        'aria-selected': 'false',
        'aria-expanded': 'false',
        tabIndex: -1
      }).next().attr({ 'aria-hidden': 'true' }).hide();
      // make sure at least one header is in the tab order
      if (!this.active.length) {
        this.headers.eq(0).attr('tabIndex', 0);
      } else {
        this.active.attr({
          'aria-selected': 'true',
          'aria-expanded': 'true',
          tabIndex: 0
        }).next().attr({ 'aria-hidden': 'false' });
      }
      this._createIcons();
      this._setupEvents(options.event);
      if (heightStyle === 'fill') {
        maxHeight = parent.height();
        this.element.siblings(':visible').each(function () {
          var elem = $(this), position = elem.css('position');
          if (position === 'absolute' || position === 'fixed') {
            return;
          }
          maxHeight -= elem.outerHeight(true);
        });
        this.headers.each(function () {
          maxHeight -= $(this).outerHeight(true);
        });
        this.headers.next().each(function () {
          $(this).height(Math.max(0, maxHeight - $(this).innerHeight() + $(this).height()));
        }).css('overflow', 'auto');
      } else if (heightStyle === 'auto') {
        maxHeight = 0;
        this.headers.next().each(function () {
          maxHeight = Math.max(maxHeight, $(this).css('height', '').height());
        }).height(maxHeight);
      }
    },
    _activate: function (index) {
      var active = this._findActive(index)[0];
      // trying to activate the already active panel
      if (active === this.active[0]) {
        return;
      }
      // trying to collapse, simulate a click on the currently active header
      active = active || this.active[0];
      this._eventHandler({
        target: active,
        currentTarget: active,
        preventDefault: $.noop
      });
    },
    _findActive: function (selector) {
      return typeof selector === 'number' ? this.headers.eq(selector) : $();
    },
    _setupEvents: function (event) {
      var events = { keydown: '_keydown' };
      if (event) {
        $.each(event.split(' '), function (index, eventName) {
          events[eventName] = '_eventHandler';
        });
      }
      this._off(this.headers.add(this.headers.next()));
      this._on(this.headers, events);
      this._on(this.headers.next(), { keydown: '_panelKeyDown' });
      this._hoverable(this.headers);
      this._focusable(this.headers);
    },
    _eventHandler: function (event) {
      var options = this.options, active = this.active, clicked = $(event.currentTarget), clickedIsActive = clicked[0] === active[0], collapsing = clickedIsActive && options.collapsible, toShow = collapsing ? $() : clicked.next(), toHide = active.next(), eventData = {
          oldHeader: active,
          oldPanel: toHide,
          newHeader: collapsing ? $() : clicked,
          newPanel: toShow
        };
      event.preventDefault();
      if (clickedIsActive && !options.collapsible || this._trigger('beforeActivate', event, eventData) === false) {
        return;
      }
      options.active = collapsing ? false : this.headers.index(clicked);
      // when the call to ._toggle() comes after the class changes
      // it causes a very odd bug in IE 8 (see #6720)
      this.active = clickedIsActive ? $() : clicked;
      this._toggle(eventData);
      // switch classes
      // corner classes on the previously active header stay after the animation
      active.removeClass('ui-accordion-header-active ui-state-active');
      if (options.icons) {
        active.children('.ui-accordion-header-icon').removeClass(options.icons.activeHeader).addClass(options.icons.header);
      }
      if (!clickedIsActive) {
        clicked.removeClass('ui-corner-all').addClass('ui-accordion-header-active ui-state-active ui-corner-top');
        if (options.icons) {
          clicked.children('.ui-accordion-header-icon').removeClass(options.icons.header).addClass(options.icons.activeHeader);
        }
        clicked.next().addClass('ui-accordion-content-active');
      }
    },
    _toggle: function (data) {
      var toShow = data.newPanel, toHide = this.prevShow.length ? this.prevShow : data.oldPanel;
      // handle activating a panel during the animation for another activation
      this.prevShow.add(this.prevHide).stop(true, true);
      this.prevShow = toShow;
      this.prevHide = toHide;
      if (this.options.animate) {
        this._animate(toShow, toHide, data);
      } else {
        toHide.hide();
        toShow.show();
        this._toggleComplete(data);
      }
      toHide.attr({ 'aria-hidden': 'true' });
      toHide.prev().attr('aria-selected', 'false');
      // if we're switching panels, remove the old header from the tab order
      // if we're opening from collapsed state, remove the previous header from the tab order
      // if we're collapsing, then keep the collapsing header in the tab order
      if (toShow.length && toHide.length) {
        toHide.prev().attr({
          'tabIndex': -1,
          'aria-expanded': 'false'
        });
      } else if (toShow.length) {
        this.headers.filter(function () {
          return $(this).attr('tabIndex') === 0;
        }).attr('tabIndex', -1);
      }
      toShow.attr('aria-hidden', 'false').prev().attr({
        'aria-selected': 'true',
        tabIndex: 0,
        'aria-expanded': 'true'
      });
    },
    _animate: function (toShow, toHide, data) {
      var total, easing, duration, that = this, adjust = 0, down = toShow.length && (!toHide.length || toShow.index() < toHide.index()), animate = this.options.animate || {}, options = down && animate.down || animate, complete = function () {
          that._toggleComplete(data);
        };
      if (typeof options === 'number') {
        duration = options;
      }
      if (typeof options === 'string') {
        easing = options;
      }
      // fall back from options to animation in case of partial down settings
      easing = easing || options.easing || animate.easing;
      duration = duration || options.duration || animate.duration;
      if (!toHide.length) {
        return toShow.animate(showProps, duration, easing, complete);
      }
      if (!toShow.length) {
        return toHide.animate(hideProps, duration, easing, complete);
      }
      total = toShow.show().outerHeight();
      toHide.animate(hideProps, {
        duration: duration,
        easing: easing,
        step: function (now, fx) {
          fx.now = Math.round(now);
        }
      });
      toShow.hide().animate(showProps, {
        duration: duration,
        easing: easing,
        complete: complete,
        step: function (now, fx) {
          fx.now = Math.round(now);
          if (fx.prop !== 'height') {
            adjust += fx.now;
          } else if (that.options.heightStyle !== 'content') {
            fx.now = Math.round(total - toHide.outerHeight() - adjust);
            adjust = 0;
          }
        }
      });
    },
    _toggleComplete: function (data) {
      var toHide = data.oldPanel;
      toHide.removeClass('ui-accordion-content-active').prev().removeClass('ui-corner-top').addClass('ui-corner-all');
      // Work around for rendering bug in IE (#5421)
      if (toHide.length) {
        toHide.parent()[0].className = toHide.parent()[0].className;
      }
      this._trigger('activate', null, data);
    }
  });
}(jQuery));
(function ($, undefined) {
  $.widget('ui.autocomplete', {
    version: '1.10.4',
    defaultElement: '<input>',
    options: {
      appendTo: null,
      autoFocus: false,
      delay: 300,
      minLength: 1,
      position: {
        my: 'left top',
        at: 'left bottom',
        collision: 'none'
      },
      source: null,
      change: null,
      close: null,
      focus: null,
      open: null,
      response: null,
      search: null,
      select: null
    },
    requestIndex: 0,
    pending: 0,
    _create: function () {
      // Some browsers only repeat keydown events, not keypress events,
      // so we use the suppressKeyPress flag to determine if we've already
      // handled the keydown event. #7269
      // Unfortunately the code for & in keypress is the same as the up arrow,
      // so we use the suppressKeyPressRepeat flag to avoid handling keypress
      // events when we know the keydown event was used to modify the
      // search term. #7799
      var suppressKeyPress, suppressKeyPressRepeat, suppressInput, nodeName = this.element[0].nodeName.toLowerCase(), isTextarea = nodeName === 'textarea', isInput = nodeName === 'input';
      this.isMultiLine = isTextarea ? true : isInput ? false : this.element.prop('isContentEditable');
      this.valueMethod = this.element[isTextarea || isInput ? 'val' : 'text'];
      this.isNewMenu = true;
      this.element.addClass('ui-autocomplete-input').attr('autocomplete', 'off');
      this._on(this.element, {
        keydown: function (event) {
          if (this.element.prop('readOnly')) {
            suppressKeyPress = true;
            suppressInput = true;
            suppressKeyPressRepeat = true;
            return;
          }
          suppressKeyPress = false;
          suppressInput = false;
          suppressKeyPressRepeat = false;
          var keyCode = $.ui.keyCode;
          switch (event.keyCode) {
          case keyCode.PAGE_UP:
            suppressKeyPress = true;
            this._move('previousPage', event);
            break;
          case keyCode.PAGE_DOWN:
            suppressKeyPress = true;
            this._move('nextPage', event);
            break;
          case keyCode.UP:
            suppressKeyPress = true;
            this._keyEvent('previous', event);
            break;
          case keyCode.DOWN:
            suppressKeyPress = true;
            this._keyEvent('next', event);
            break;
          case keyCode.ENTER:
          case keyCode.NUMPAD_ENTER:
            // when menu is open and has focus
            if (this.menu.active) {
              // #6055 - Opera still allows the keypress to occur
              // which causes forms to submit
              suppressKeyPress = true;
              event.preventDefault();
              this.menu.select(event);
            }
            break;
          case keyCode.TAB:
            if (this.menu.active) {
              this.menu.select(event);
            }
            break;
          case keyCode.ESCAPE:
            if (this.menu.element.is(':visible')) {
              this._value(this.term);
              this.close(event);
              // Different browsers have different default behavior for escape
              // Single press can mean undo or clear
              // Double press in IE means clear the whole form
              event.preventDefault();
            }
            break;
          default:
            suppressKeyPressRepeat = true;
            // search timeout should be triggered before the input value is changed
            this._searchTimeout(event);
            break;
          }
        },
        keypress: function (event) {
          if (suppressKeyPress) {
            suppressKeyPress = false;
            if (!this.isMultiLine || this.menu.element.is(':visible')) {
              event.preventDefault();
            }
            return;
          }
          if (suppressKeyPressRepeat) {
            return;
          }
          // replicate some key handlers to allow them to repeat in Firefox and Opera
          var keyCode = $.ui.keyCode;
          switch (event.keyCode) {
          case keyCode.PAGE_UP:
            this._move('previousPage', event);
            break;
          case keyCode.PAGE_DOWN:
            this._move('nextPage', event);
            break;
          case keyCode.UP:
            this._keyEvent('previous', event);
            break;
          case keyCode.DOWN:
            this._keyEvent('next', event);
            break;
          }
        },
        input: function (event) {
          if (suppressInput) {
            suppressInput = false;
            event.preventDefault();
            return;
          }
          this._searchTimeout(event);
        },
        focus: function () {
          this.selectedItem = null;
          this.previous = this._value();
        },
        blur: function (event) {
          if (this.cancelBlur) {
            delete this.cancelBlur;
            return;
          }
          clearTimeout(this.searching);
          this.close(event);
          this._change(event);
        }
      });
      this._initSource();
      this.menu = $('<ul>').addClass('ui-autocomplete ui-front').appendTo(this._appendTo()).menu({ role: null }).hide().data('ui-menu');
      this._on(this.menu.element, {
        mousedown: function (event) {
          // prevent moving focus out of the text field
          event.preventDefault();
          // IE doesn't prevent moving focus even with event.preventDefault()
          // so we set a flag to know when we should ignore the blur event
          this.cancelBlur = true;
          this._delay(function () {
            delete this.cancelBlur;
          });
          // clicking on the scrollbar causes focus to shift to the body
          // but we can't detect a mouseup or a click immediately afterward
          // so we have to track the next mousedown and close the menu if
          // the user clicks somewhere outside of the autocomplete
          var menuElement = this.menu.element[0];
          if (!$(event.target).closest('.ui-menu-item').length) {
            this._delay(function () {
              var that = this;
              this.document.one('mousedown', function (event) {
                if (event.target !== that.element[0] && event.target !== menuElement && !$.contains(menuElement, event.target)) {
                  that.close();
                }
              });
            });
          }
        },
        menufocus: function (event, ui) {
          // support: Firefox
          // Prevent accidental activation of menu items in Firefox (#7024 #9118)
          if (this.isNewMenu) {
            this.isNewMenu = false;
            if (event.originalEvent && /^mouse/.test(event.originalEvent.type)) {
              this.menu.blur();
              this.document.one('mousemove', function () {
                $(event.target).trigger(event.originalEvent);
              });
              return;
            }
          }
          var item = ui.item.data('ui-autocomplete-item');
          if (false !== this._trigger('focus', event, { item: item })) {
            // use value to match what will end up in the input, if it was a key event
            if (event.originalEvent && /^key/.test(event.originalEvent.type)) {
              this._value(item.value);
            }
          } else {
            // Normally the input is populated with the item's value as the
            // menu is navigated, causing screen readers to notice a change and
            // announce the item. Since the focus event was canceled, this doesn't
            // happen, so we update the live region so that screen readers can
            // still notice the change and announce it.
            this.liveRegion.text(item.value);
          }
        },
        menuselect: function (event, ui) {
          var item = ui.item.data('ui-autocomplete-item'), previous = this.previous;
          // only trigger when focus was lost (click on menu)
          if (this.element[0] !== this.document[0].activeElement) {
            this.element.focus();
            this.previous = previous;
            // #6109 - IE triggers two focus events and the second
            // is asynchronous, so we need to reset the previous
            // term synchronously and asynchronously :-(
            this._delay(function () {
              this.previous = previous;
              this.selectedItem = item;
            });
          }
          if (false !== this._trigger('select', event, { item: item })) {
            this._value(item.value);
          }
          // reset the term after the select event
          // this allows custom select handling to work properly
          this.term = this._value();
          this.close(event);
          this.selectedItem = item;
        }
      });
      this.liveRegion = $('<span>', {
        role: 'status',
        'aria-live': 'polite'
      }).addClass('ui-helper-hidden-accessible').insertBefore(this.element);
      // turning off autocomplete prevents the browser from remembering the
      // value when navigating through history, so we re-enable autocomplete
      // if the page is unloaded before the widget is destroyed. #7790
      this._on(this.window, {
        beforeunload: function () {
          this.element.removeAttr('autocomplete');
        }
      });
    },
    _destroy: function () {
      clearTimeout(this.searching);
      this.element.removeClass('ui-autocomplete-input').removeAttr('autocomplete');
      this.menu.element.remove();
      this.liveRegion.remove();
    },
    _setOption: function (key, value) {
      this._super(key, value);
      if (key === 'source') {
        this._initSource();
      }
      if (key === 'appendTo') {
        this.menu.element.appendTo(this._appendTo());
      }
      if (key === 'disabled' && value && this.xhr) {
        this.xhr.abort();
      }
    },
    _appendTo: function () {
      var element = this.options.appendTo;
      if (element) {
        element = element.jquery || element.nodeType ? $(element) : this.document.find(element).eq(0);
      }
      if (!element) {
        element = this.element.closest('.ui-front');
      }
      if (!element.length) {
        element = this.document[0].body;
      }
      return element;
    },
    _initSource: function () {
      var array, url, that = this;
      if ($.isArray(this.options.source)) {
        array = this.options.source;
        this.source = function (request, response) {
          response($.ui.autocomplete.filter(array, request.term));
        };
      } else if (typeof this.options.source === 'string') {
        url = this.options.source;
        this.source = function (request, response) {
          if (that.xhr) {
            that.xhr.abort();
          }
          that.xhr = $.ajax({
            url: url,
            data: request,
            dataType: 'json',
            success: function (data) {
              response(data);
            },
            error: function () {
              response([]);
            }
          });
        };
      } else {
        this.source = this.options.source;
      }
    },
    _searchTimeout: function (event) {
      clearTimeout(this.searching);
      this.searching = this._delay(function () {
        // only search if the value has changed
        if (this.term !== this._value()) {
          this.selectedItem = null;
          this.search(null, event);
        }
      }, this.options.delay);
    },
    search: function (value, event) {
      value = value != null ? value : this._value();
      // always save the actual value, not the one passed as an argument
      this.term = this._value();
      if (value.length < this.options.minLength) {
        return this.close(event);
      }
      if (this._trigger('search', event) === false) {
        return;
      }
      return this._search(value);
    },
    _search: function (value) {
      this.pending++;
      this.element.addClass('ui-autocomplete-loading');
      this.cancelSearch = false;
      this.source({ term: value }, this._response());
    },
    _response: function () {
      var index = ++this.requestIndex;
      return $.proxy(function (content) {
        if (index === this.requestIndex) {
          this.__response(content);
        }
        this.pending--;
        if (!this.pending) {
          this.element.removeClass('ui-autocomplete-loading');
        }
      }, this);
    },
    __response: function (content) {
      if (content) {
        content = this._normalize(content);
      }
      this._trigger('response', null, { content: content });
      if (!this.options.disabled && content && content.length && !this.cancelSearch) {
        this._suggest(content);
        this._trigger('open');
      } else {
        // use ._close() instead of .close() so we don't cancel future searches
        this._close();
      }
    },
    close: function (event) {
      this.cancelSearch = true;
      this._close(event);
    },
    _close: function (event) {
      if (this.menu.element.is(':visible')) {
        this.menu.element.hide();
        this.menu.blur();
        this.isNewMenu = true;
        this._trigger('close', event);
      }
    },
    _change: function (event) {
      if (this.previous !== this._value()) {
        this._trigger('change', event, { item: this.selectedItem });
      }
    },
    _normalize: function (items) {
      // assume all items have the right format when the first item is complete
      if (items.length && items[0].label && items[0].value) {
        return items;
      }
      return $.map(items, function (item) {
        if (typeof item === 'string') {
          return {
            label: item,
            value: item
          };
        }
        return $.extend({
          label: item.label || item.value,
          value: item.value || item.label
        }, item);
      });
    },
    _suggest: function (items) {
      var ul = this.menu.element.empty();
      this._renderMenu(ul, items);
      this.isNewMenu = true;
      this.menu.refresh();
      // size and position menu
      ul.show();
      this._resizeMenu();
      ul.position($.extend({ of: this.element }, this.options.position));
      if (this.options.autoFocus) {
        this.menu.next();
      }
    },
    _resizeMenu: function () {
      var ul = this.menu.element;
      ul.outerWidth(Math.max(ul.width('').outerWidth() + 1, this.element.outerWidth()));
    },
    _renderMenu: function (ul, items) {
      var that = this;
      $.each(items, function (index, item) {
        that._renderItemData(ul, item);
      });
    },
    _renderItemData: function (ul, item) {
      return this._renderItem(ul, item).data('ui-autocomplete-item', item);
    },
    _renderItem: function (ul, item) {
      return $('<li>').append($('<a>').text(item.label)).appendTo(ul);
    },
    _move: function (direction, event) {
      if (!this.menu.element.is(':visible')) {
        this.search(null, event);
        return;
      }
      if (this.menu.isFirstItem() && /^previous/.test(direction) || this.menu.isLastItem() && /^next/.test(direction)) {
        this._value(this.term);
        this.menu.blur();
        return;
      }
      this.menu[direction](event);
    },
    widget: function () {
      return this.menu.element;
    },
    _value: function () {
      return this.valueMethod.apply(this.element, arguments);
    },
    _keyEvent: function (keyEvent, event) {
      if (!this.isMultiLine || this.menu.element.is(':visible')) {
        this._move(keyEvent, event);
        // prevents moving cursor to beginning/end of the text field in some browsers
        event.preventDefault();
      }
    }
  });
  $.extend($.ui.autocomplete, {
    escapeRegex: function (value) {
      return value.replace(/[\-\[\]{}()*+?.,\\\^$|#\s]/g, '\\$&');
    },
    filter: function (array, term) {
      var matcher = new RegExp($.ui.autocomplete.escapeRegex(term), 'i');
      return $.grep(array, function (value) {
        return matcher.test(value.label || value.value || value);
      });
    }
  });
  // live region extension, adding a `messages` option
  // NOTE: This is an experimental API. We are still investigating
  // a full solution for string manipulation and internationalization.
  $.widget('ui.autocomplete', $.ui.autocomplete, {
    options: {
      messages: {
        noResults: 'No search results.',
        results: function (amount) {
          return amount + (amount > 1 ? ' results are' : ' result is') + ' available, use up and down arrow keys to navigate.';
        }
      }
    },
    __response: function (content) {
      var message;
      this._superApply(arguments);
      if (this.options.disabled || this.cancelSearch) {
        return;
      }
      if (content && content.length) {
        message = this.options.messages.results(content.length);
      } else {
        message = this.options.messages.noResults;
      }
      this.liveRegion.text(message);
    }
  });
}(jQuery));
(function ($, undefined) {
  var lastActive, baseClasses = 'ui-button ui-widget ui-state-default ui-corner-all', typeClasses = 'ui-button-icons-only ui-button-icon-only ui-button-text-icons ui-button-text-icon-primary ui-button-text-icon-secondary ui-button-text-only', formResetHandler = function () {
      var form = $(this);
      setTimeout(function () {
        form.find(':ui-button').button('refresh');
      }, 1);
    }, radioGroup = function (radio) {
      var name = radio.name, form = radio.form, radios = $([]);
      if (name) {
        name = name.replace(/'/g, '\\\'');
        if (form) {
          radios = $(form).find('[name=\'' + name + '\']');
        } else {
          radios = $('[name=\'' + name + '\']', radio.ownerDocument).filter(function () {
            return !this.form;
          });
        }
      }
      return radios;
    };
  $.widget('ui.button', {
    version: '1.10.4',
    defaultElement: '<button>',
    options: {
      disabled: null,
      text: true,
      label: null,
      icons: {
        primary: null,
        secondary: null
      }
    },
    _create: function () {
      this.element.closest('form').unbind('reset' + this.eventNamespace).bind('reset' + this.eventNamespace, formResetHandler);
      if (typeof this.options.disabled !== 'boolean') {
        this.options.disabled = !!this.element.prop('disabled');
      } else {
        this.element.prop('disabled', this.options.disabled);
      }
      this._determineButtonType();
      this.hasTitle = !!this.buttonElement.attr('title');
      var that = this, options = this.options, toggleButton = this.type === 'checkbox' || this.type === 'radio', activeClass = !toggleButton ? 'ui-state-active' : '';
      if (options.label === null) {
        options.label = this.type === 'input' ? this.buttonElement.val() : this.buttonElement.html();
      }
      this._hoverable(this.buttonElement);
      this.buttonElement.addClass(baseClasses).attr('role', 'button').bind('mouseenter' + this.eventNamespace, function () {
        if (options.disabled) {
          return;
        }
        if (this === lastActive) {
          $(this).addClass('ui-state-active');
        }
      }).bind('mouseleave' + this.eventNamespace, function () {
        if (options.disabled) {
          return;
        }
        $(this).removeClass(activeClass);
      }).bind('click' + this.eventNamespace, function (event) {
        if (options.disabled) {
          event.preventDefault();
          event.stopImmediatePropagation();
        }
      });
      // Can't use _focusable() because the element that receives focus
      // and the element that gets the ui-state-focus class are different
      this._on({
        focus: function () {
          this.buttonElement.addClass('ui-state-focus');
        },
        blur: function () {
          this.buttonElement.removeClass('ui-state-focus');
        }
      });
      if (toggleButton) {
        this.element.bind('change' + this.eventNamespace, function () {
          that.refresh();
        });
      }
      if (this.type === 'checkbox') {
        this.buttonElement.bind('click' + this.eventNamespace, function () {
          if (options.disabled) {
            return false;
          }
        });
      } else if (this.type === 'radio') {
        this.buttonElement.bind('click' + this.eventNamespace, function () {
          if (options.disabled) {
            return false;
          }
          $(this).addClass('ui-state-active');
          that.buttonElement.attr('aria-pressed', 'true');
          var radio = that.element[0];
          radioGroup(radio).not(radio).map(function () {
            return $(this).button('widget')[0];
          }).removeClass('ui-state-active').attr('aria-pressed', 'false');
        });
      } else {
        this.buttonElement.bind('mousedown' + this.eventNamespace, function () {
          if (options.disabled) {
            return false;
          }
          $(this).addClass('ui-state-active');
          lastActive = this;
          that.document.one('mouseup', function () {
            lastActive = null;
          });
        }).bind('mouseup' + this.eventNamespace, function () {
          if (options.disabled) {
            return false;
          }
          $(this).removeClass('ui-state-active');
        }).bind('keydown' + this.eventNamespace, function (event) {
          if (options.disabled) {
            return false;
          }
          if (event.keyCode === $.ui.keyCode.SPACE || event.keyCode === $.ui.keyCode.ENTER) {
            $(this).addClass('ui-state-active');
          }
        }).bind('keyup' + this.eventNamespace + ' blur' + this.eventNamespace, function () {
          $(this).removeClass('ui-state-active');
        });
        if (this.buttonElement.is('a')) {
          this.buttonElement.keyup(function (event) {
            if (event.keyCode === $.ui.keyCode.SPACE) {
              // TODO pass through original event correctly (just as 2nd argument doesn't work)
              $(this).click();
            }
          });
        }
      }
      // TODO: pull out $.Widget's handling for the disabled option into
      // $.Widget.prototype._setOptionDisabled so it's easy to proxy and can
      // be overridden by individual plugins
      this._setOption('disabled', options.disabled);
      this._resetButton();
    },
    _determineButtonType: function () {
      var ancestor, labelSelector, checked;
      if (this.element.is('[type=checkbox]')) {
        this.type = 'checkbox';
      } else if (this.element.is('[type=radio]')) {
        this.type = 'radio';
      } else if (this.element.is('input')) {
        this.type = 'input';
      } else {
        this.type = 'button';
      }
      if (this.type === 'checkbox' || this.type === 'radio') {
        // we don't search against the document in case the element
        // is disconnected from the DOM
        ancestor = this.element.parents().last();
        labelSelector = 'label[for=\'' + this.element.attr('id') + '\']';
        this.buttonElement = ancestor.find(labelSelector);
        if (!this.buttonElement.length) {
          ancestor = ancestor.length ? ancestor.siblings() : this.element.siblings();
          this.buttonElement = ancestor.filter(labelSelector);
          if (!this.buttonElement.length) {
            this.buttonElement = ancestor.find(labelSelector);
          }
        }
        this.element.addClass('ui-helper-hidden-accessible');
        checked = this.element.is(':checked');
        if (checked) {
          this.buttonElement.addClass('ui-state-active');
        }
        this.buttonElement.prop('aria-pressed', checked);
      } else {
        this.buttonElement = this.element;
      }
    },
    widget: function () {
      return this.buttonElement;
    },
    _destroy: function () {
      this.element.removeClass('ui-helper-hidden-accessible');
      this.buttonElement.removeClass(baseClasses + ' ui-state-active ' + typeClasses).removeAttr('role').removeAttr('aria-pressed').html(this.buttonElement.find('.ui-button-text').html());
      if (!this.hasTitle) {
        this.buttonElement.removeAttr('title');
      }
    },
    _setOption: function (key, value) {
      this._super(key, value);
      if (key === 'disabled') {
        this.element.prop('disabled', !!value);
        if (value) {
          this.buttonElement.removeClass('ui-state-focus');
        }
        return;
      }
      this._resetButton();
    },
    refresh: function () {
      //See #8237 & #8828
      var isDisabled = this.element.is('input, button') ? this.element.is(':disabled') : this.element.hasClass('ui-button-disabled');
      if (isDisabled !== this.options.disabled) {
        this._setOption('disabled', isDisabled);
      }
      if (this.type === 'radio') {
        radioGroup(this.element[0]).each(function () {
          if ($(this).is(':checked')) {
            $(this).button('widget').addClass('ui-state-active').attr('aria-pressed', 'true');
          } else {
            $(this).button('widget').removeClass('ui-state-active').attr('aria-pressed', 'false');
          }
        });
      } else if (this.type === 'checkbox') {
        if (this.element.is(':checked')) {
          this.buttonElement.addClass('ui-state-active').attr('aria-pressed', 'true');
        } else {
          this.buttonElement.removeClass('ui-state-active').attr('aria-pressed', 'false');
        }
      }
    },
    _resetButton: function () {
      if (this.type === 'input') {
        if (this.options.label) {
          this.element.val(this.options.label);
        }
        return;
      }
      var buttonElement = this.buttonElement.removeClass(typeClasses), buttonText = $('<span></span>', this.document[0]).addClass('ui-button-text').html(this.options.label).appendTo(buttonElement.empty()).text(), icons = this.options.icons, multipleIcons = icons.primary && icons.secondary, buttonClasses = [];
      if (icons.primary || icons.secondary) {
        if (this.options.text) {
          buttonClasses.push('ui-button-text-icon' + (multipleIcons ? 's' : icons.primary ? '-primary' : '-secondary'));
        }
        if (icons.primary) {
          buttonElement.prepend('<span class=\'ui-button-icon-primary ui-icon ' + icons.primary + '\'></span>');
        }
        if (icons.secondary) {
          buttonElement.append('<span class=\'ui-button-icon-secondary ui-icon ' + icons.secondary + '\'></span>');
        }
        if (!this.options.text) {
          buttonClasses.push(multipleIcons ? 'ui-button-icons-only' : 'ui-button-icon-only');
          if (!this.hasTitle) {
            buttonElement.attr('title', $.trim(buttonText));
          }
        }
      } else {
        buttonClasses.push('ui-button-text-only');
      }
      buttonElement.addClass(buttonClasses.join(' '));
    }
  });
  $.widget('ui.buttonset', {
    version: '1.10.4',
    options: { items: 'button, input[type=button], input[type=submit], input[type=reset], input[type=checkbox], input[type=radio], a, :data(ui-button)' },
    _create: function () {
      this.element.addClass('ui-buttonset');
    },
    _init: function () {
      this.refresh();
    },
    _setOption: function (key, value) {
      if (key === 'disabled') {
        this.buttons.button('option', key, value);
      }
      this._super(key, value);
    },
    refresh: function () {
      var rtl = this.element.css('direction') === 'rtl';
      this.buttons = this.element.find(this.options.items).filter(':ui-button').button('refresh').end().not(':ui-button').button().end().map(function () {
        return $(this).button('widget')[0];
      }).removeClass('ui-corner-all ui-corner-left ui-corner-right').filter(':first').addClass(rtl ? 'ui-corner-right' : 'ui-corner-left').end().filter(':last').addClass(rtl ? 'ui-corner-left' : 'ui-corner-right').end().end();
    },
    _destroy: function () {
      this.element.removeClass('ui-buttonset');
      this.buttons.map(function () {
        return $(this).button('widget')[0];
      }).removeClass('ui-corner-left ui-corner-right').end().button('destroy');
    }
  });
}(jQuery));
(function ($, undefined) {
  $.extend($.ui, { datepicker: { version: '1.10.4' } });
  var PROP_NAME = 'datepicker', instActive;
  /* Date picker manager.
   Use the singleton instance of this class, $.datepicker, to interact with the date picker.
   Settings for (groups of) date pickers are maintained in an instance object,
   allowing multiple different settings on the same page. */
  function Datepicker() {
    this._curInst = null;
    // The current instance in use
    this._keyEvent = false;
    // If the last event was a key event
    this._disabledInputs = [];
    // List of date picker inputs that have been disabled
    this._datepickerShowing = false;
    // True if the popup picker is showing , false if not
    this._inDialog = false;
    // True if showing within a "dialog", false if not
    this._mainDivId = 'ui-datepicker-div';
    // The ID of the main datepicker division
    this._inlineClass = 'ui-datepicker-inline';
    // The name of the inline marker class
    this._appendClass = 'ui-datepicker-append';
    // The name of the append marker class
    this._triggerClass = 'ui-datepicker-trigger';
    // The name of the trigger marker class
    this._dialogClass = 'ui-datepicker-dialog';
    // The name of the dialog marker class
    this._disableClass = 'ui-datepicker-disabled';
    // The name of the disabled covering marker class
    this._unselectableClass = 'ui-datepicker-unselectable';
    // The name of the unselectable cell marker class
    this._currentClass = 'ui-datepicker-current-day';
    // The name of the current day marker class
    this._dayOverClass = 'ui-datepicker-days-cell-over';
    // The name of the day hover marker class
    this.regional = [];
    // Available regional settings, indexed by language code
    this.regional[''] = {
      closeText: 'Done',
      prevText: 'Prev',
      nextText: 'Next',
      currentText: 'Today',
      monthNames: [
        'January',
        'February',
        'March',
        'April',
        'May',
        'June',
        'July',
        'August',
        'September',
        'October',
        'November',
        'December'
      ],
      monthNamesShort: [
        'Jan',
        'Feb',
        'Mar',
        'Apr',
        'May',
        'Jun',
        'Jul',
        'Aug',
        'Sep',
        'Oct',
        'Nov',
        'Dec'
      ],
      dayNames: [
        'Sunday',
        'Monday',
        'Tuesday',
        'Wednesday',
        'Thursday',
        'Friday',
        'Saturday'
      ],
      dayNamesShort: [
        'Sun',
        'Mon',
        'Tue',
        'Wed',
        'Thu',
        'Fri',
        'Sat'
      ],
      dayNamesMin: [
        'Su',
        'Mo',
        'Tu',
        'We',
        'Th',
        'Fr',
        'Sa'
      ],
      weekHeader: 'Wk',
      dateFormat: 'mm/dd/yy',
      firstDay: 0,
      isRTL: false,
      showMonthAfterYear: false,
      yearSuffix: ''
    };
    this._defaults = {
      showOn: 'focus',
      showAnim: 'fadeIn',
      showOptions: {},
      defaultDate: null,
      appendText: '',
      buttonText: '...',
      buttonImage: '',
      buttonImageOnly: false,
      hideIfNoPrevNext: false,
      navigationAsDateFormat: false,
      gotoCurrent: false,
      changeMonth: false,
      changeYear: false,
      yearRange: 'c-10:c+10',
      showOtherMonths: false,
      selectOtherMonths: false,
      showWeek: false,
      calculateWeek: this.iso8601Week,
      shortYearCutoff: '+10',
      minDate: null,
      maxDate: null,
      duration: 'fast',
      beforeShowDay: null,
      beforeShow: null,
      onSelect: null,
      onChangeMonthYear: null,
      onClose: null,
      numberOfMonths: 1,
      showCurrentAtPos: 0,
      stepMonths: 1,
      stepBigMonths: 12,
      altField: '',
      altFormat: '',
      constrainInput: true,
      showButtonPanel: false,
      autoSize: false,
      disabled: false
    };
    $.extend(this._defaults, this.regional['']);
    this.dpDiv = bindHover($('<div id=\'' + this._mainDivId + '\' class=\'ui-datepicker ui-widget ui-widget-content ui-helper-clearfix ui-corner-all\'></div>'));
  }
  $.extend(Datepicker.prototype, {
    markerClassName: 'hasDatepicker',
    maxRows: 4,
    _widgetDatepicker: function () {
      return this.dpDiv;
    },
    setDefaults: function (settings) {
      extendRemove(this._defaults, settings || {});
      return this;
    },
    _attachDatepicker: function (target, settings) {
      var nodeName, inline, inst;
      nodeName = target.nodeName.toLowerCase();
      inline = nodeName === 'div' || nodeName === 'span';
      if (!target.id) {
        this.uuid += 1;
        target.id = 'dp' + this.uuid;
      }
      inst = this._newInst($(target), inline);
      inst.settings = $.extend({}, settings || {});
      if (nodeName === 'input') {
        this._connectDatepicker(target, inst);
      } else if (inline) {
        this._inlineDatepicker(target, inst);
      }
    },
    _newInst: function (target, inline) {
      var id = target[0].id.replace(/([^A-Za-z0-9_\-])/g, '\\\\$1');
      // escape jQuery meta chars
      return {
        id: id,
        input: target,
        selectedDay: 0,
        selectedMonth: 0,
        selectedYear: 0,
        drawMonth: 0,
        drawYear: 0,
        inline: inline,
        dpDiv: !inline ? this.dpDiv : bindHover($('<div class=\'' + this._inlineClass + ' ui-datepicker ui-widget ui-widget-content ui-helper-clearfix ui-corner-all\'></div>'))
      };
    },
    _connectDatepicker: function (target, inst) {
      var input = $(target);
      inst.append = $([]);
      inst.trigger = $([]);
      if (input.hasClass(this.markerClassName)) {
        return;
      }
      this._attachments(input, inst);
      input.addClass(this.markerClassName).keydown(this._doKeyDown).keypress(this._doKeyPress).keyup(this._doKeyUp);
      this._autoSize(inst);
      $.data(target, PROP_NAME, inst);
      //If disabled option is true, disable the datepicker once it has been attached to the input (see ticket #5665)
      if (inst.settings.disabled) {
        this._disableDatepicker(target);
      }
    },
    _attachments: function (input, inst) {
      var showOn, buttonText, buttonImage, appendText = this._get(inst, 'appendText'), isRTL = this._get(inst, 'isRTL');
      if (inst.append) {
        inst.append.remove();
      }
      if (appendText) {
        inst.append = $('<span class=\'' + this._appendClass + '\'>' + appendText + '</span>');
        input[isRTL ? 'before' : 'after'](inst.append);
      }
      input.unbind('focus', this._showDatepicker);
      if (inst.trigger) {
        inst.trigger.remove();
      }
      showOn = this._get(inst, 'showOn');
      if (showOn === 'focus' || showOn === 'both') {
        // pop-up date picker when in the marked field
        input.focus(this._showDatepicker);
      }
      if (showOn === 'button' || showOn === 'both') {
        // pop-up date picker when button clicked
        buttonText = this._get(inst, 'buttonText');
        buttonImage = this._get(inst, 'buttonImage');
        inst.trigger = $(this._get(inst, 'buttonImageOnly') ? $('<img/>').addClass(this._triggerClass).attr({
          src: buttonImage,
          alt: buttonText,
          title: buttonText
        }) : $('<button type=\'button\'></button>').addClass(this._triggerClass).html(!buttonImage ? buttonText : $('<img/>').attr({
          src: buttonImage,
          alt: buttonText,
          title: buttonText
        })));
        input[isRTL ? 'before' : 'after'](inst.trigger);
        inst.trigger.click(function () {
          if ($.datepicker._datepickerShowing && $.datepicker._lastInput === input[0]) {
            $.datepicker._hideDatepicker();
          } else if ($.datepicker._datepickerShowing && $.datepicker._lastInput !== input[0]) {
            $.datepicker._hideDatepicker();
            $.datepicker._showDatepicker(input[0]);
          } else {
            $.datepicker._showDatepicker(input[0]);
          }
          return false;
        });
      }
    },
    _autoSize: function (inst) {
      if (this._get(inst, 'autoSize') && !inst.inline) {
        var findMax, max, maxI, i, date = new Date(2009, 12 - 1, 20),
          // Ensure double digits
          dateFormat = this._get(inst, 'dateFormat');
        if (dateFormat.match(/[DM]/)) {
          findMax = function (names) {
            max = 0;
            maxI = 0;
            for (i = 0; i < names.length; i++) {
              if (names[i].length > max) {
                max = names[i].length;
                maxI = i;
              }
            }
            return maxI;
          };
          date.setMonth(findMax(this._get(inst, dateFormat.match(/MM/) ? 'monthNames' : 'monthNamesShort')));
          date.setDate(findMax(this._get(inst, dateFormat.match(/DD/) ? 'dayNames' : 'dayNamesShort')) + 20 - date.getDay());
        }
        inst.input.attr('size', this._formatDate(inst, date).length);
      }
    },
    _inlineDatepicker: function (target, inst) {
      var divSpan = $(target);
      if (divSpan.hasClass(this.markerClassName)) {
        return;
      }
      divSpan.addClass(this.markerClassName).append(inst.dpDiv);
      $.data(target, PROP_NAME, inst);
      this._setDate(inst, this._getDefaultDate(inst), true);
      this._updateDatepicker(inst);
      this._updateAlternate(inst);
      //If disabled option is true, disable the datepicker before showing it (see ticket #5665)
      if (inst.settings.disabled) {
        this._disableDatepicker(target);
      }
      // Set display:block in place of inst.dpDiv.show() which won't work on disconnected elements
      // http://bugs.jqueryui.com/ticket/7552 - A Datepicker created on a detached div has zero height
      inst.dpDiv.css('display', 'block');
    },
    _dialogDatepicker: function (input, date, onSelect, settings, pos) {
      var id, browserWidth, browserHeight, scrollX, scrollY, inst = this._dialogInst;
      // internal instance
      if (!inst) {
        this.uuid += 1;
        id = 'dp' + this.uuid;
        this._dialogInput = $('<input type=\'text\' id=\'' + id + '\' style=\'position: absolute; top: -100px; width: 0px;\'/>');
        this._dialogInput.keydown(this._doKeyDown);
        $('body').append(this._dialogInput);
        inst = this._dialogInst = this._newInst(this._dialogInput, false);
        inst.settings = {};
        $.data(this._dialogInput[0], PROP_NAME, inst);
      }
      extendRemove(inst.settings, settings || {});
      date = date && date.constructor === Date ? this._formatDate(inst, date) : date;
      this._dialogInput.val(date);
      this._pos = pos ? pos.length ? pos : [
        pos.pageX,
        pos.pageY
      ] : null;
      if (!this._pos) {
        browserWidth = document.documentElement.clientWidth;
        browserHeight = document.documentElement.clientHeight;
        scrollX = document.documentElement.scrollLeft || document.body.scrollLeft;
        scrollY = document.documentElement.scrollTop || document.body.scrollTop;
        this._pos = [
          browserWidth / 2 - 100 + scrollX,
          browserHeight / 2 - 150 + scrollY
        ];
      }
      // move input on screen for focus, but hidden behind dialog
      this._dialogInput.css('left', this._pos[0] + 20 + 'px').css('top', this._pos[1] + 'px');
      inst.settings.onSelect = onSelect;
      this._inDialog = true;
      this.dpDiv.addClass(this._dialogClass);
      this._showDatepicker(this._dialogInput[0]);
      if ($.blockUI) {
        $.blockUI(this.dpDiv);
      }
      $.data(this._dialogInput[0], PROP_NAME, inst);
      return this;
    },
    _destroyDatepicker: function (target) {
      var nodeName, $target = $(target), inst = $.data(target, PROP_NAME);
      if (!$target.hasClass(this.markerClassName)) {
        return;
      }
      nodeName = target.nodeName.toLowerCase();
      $.removeData(target, PROP_NAME);
      if (nodeName === 'input') {
        inst.append.remove();
        inst.trigger.remove();
        $target.removeClass(this.markerClassName).unbind('focus', this._showDatepicker).unbind('keydown', this._doKeyDown).unbind('keypress', this._doKeyPress).unbind('keyup', this._doKeyUp);
      } else if (nodeName === 'div' || nodeName === 'span') {
        $target.removeClass(this.markerClassName).empty();
      }
    },
    _enableDatepicker: function (target) {
      var nodeName, inline, $target = $(target), inst = $.data(target, PROP_NAME);
      if (!$target.hasClass(this.markerClassName)) {
        return;
      }
      nodeName = target.nodeName.toLowerCase();
      if (nodeName === 'input') {
        target.disabled = false;
        inst.trigger.filter('button').each(function () {
          this.disabled = false;
        }).end().filter('img').css({
          opacity: '1.0',
          cursor: ''
        });
      } else if (nodeName === 'div' || nodeName === 'span') {
        inline = $target.children('.' + this._inlineClass);
        inline.children().removeClass('ui-state-disabled');
        inline.find('select.ui-datepicker-month, select.ui-datepicker-year').prop('disabled', false);
      }
      this._disabledInputs = $.map(this._disabledInputs, function (value) {
        return value === target ? null : value;
      });  // delete entry
    },
    _disableDatepicker: function (target) {
      var nodeName, inline, $target = $(target), inst = $.data(target, PROP_NAME);
      if (!$target.hasClass(this.markerClassName)) {
        return;
      }
      nodeName = target.nodeName.toLowerCase();
      if (nodeName === 'input') {
        target.disabled = true;
        inst.trigger.filter('button').each(function () {
          this.disabled = true;
        }).end().filter('img').css({
          opacity: '0.5',
          cursor: 'default'
        });
      } else if (nodeName === 'div' || nodeName === 'span') {
        inline = $target.children('.' + this._inlineClass);
        inline.children().addClass('ui-state-disabled');
        inline.find('select.ui-datepicker-month, select.ui-datepicker-year').prop('disabled', true);
      }
      this._disabledInputs = $.map(this._disabledInputs, function (value) {
        return value === target ? null : value;
      });
      // delete entry
      this._disabledInputs[this._disabledInputs.length] = target;
    },
    _isDisabledDatepicker: function (target) {
      if (!target) {
        return false;
      }
      for (var i = 0; i < this._disabledInputs.length; i++) {
        if (this._disabledInputs[i] === target) {
          return true;
        }
      }
      return false;
    },
    _getInst: function (target) {
      try {
        return $.data(target, PROP_NAME);
      } catch (err) {
        throw 'Missing instance data for this datepicker';
      }
    },
    _optionDatepicker: function (target, name, value) {
      var settings, date, minDate, maxDate, inst = this._getInst(target);
      if (arguments.length === 2 && typeof name === 'string') {
        return name === 'defaults' ? $.extend({}, $.datepicker._defaults) : inst ? name === 'all' ? $.extend({}, inst.settings) : this._get(inst, name) : null;
      }
      settings = name || {};
      if (typeof name === 'string') {
        settings = {};
        settings[name] = value;
      }
      if (inst) {
        if (this._curInst === inst) {
          this._hideDatepicker();
        }
        date = this._getDateDatepicker(target, true);
        minDate = this._getMinMaxDate(inst, 'min');
        maxDate = this._getMinMaxDate(inst, 'max');
        extendRemove(inst.settings, settings);
        // reformat the old minDate/maxDate values if dateFormat changes and a new minDate/maxDate isn't provided
        if (minDate !== null && settings.dateFormat !== undefined && settings.minDate === undefined) {
          inst.settings.minDate = this._formatDate(inst, minDate);
        }
        if (maxDate !== null && settings.dateFormat !== undefined && settings.maxDate === undefined) {
          inst.settings.maxDate = this._formatDate(inst, maxDate);
        }
        if ('disabled' in settings) {
          if (settings.disabled) {
            this._disableDatepicker(target);
          } else {
            this._enableDatepicker(target);
          }
        }
        this._attachments($(target), inst);
        this._autoSize(inst);
        this._setDate(inst, date);
        this._updateAlternate(inst);
        this._updateDatepicker(inst);
      }
    },
    _changeDatepicker: function (target, name, value) {
      this._optionDatepicker(target, name, value);
    },
    _refreshDatepicker: function (target) {
      var inst = this._getInst(target);
      if (inst) {
        this._updateDatepicker(inst);
      }
    },
    _setDateDatepicker: function (target, date) {
      var inst = this._getInst(target);
      if (inst) {
        this._setDate(inst, date);
        this._updateDatepicker(inst);
        this._updateAlternate(inst);
      }
    },
    _getDateDatepicker: function (target, noDefault) {
      var inst = this._getInst(target);
      if (inst && !inst.inline) {
        this._setDateFromField(inst, noDefault);
      }
      return inst ? this._getDate(inst) : null;
    },
    _doKeyDown: function (event) {
      var onSelect, dateStr, sel, inst = $.datepicker._getInst(event.target), handled = true, isRTL = inst.dpDiv.is('.ui-datepicker-rtl');
      inst._keyEvent = true;
      if ($.datepicker._datepickerShowing) {
        switch (event.keyCode) {
        case 9:
          $.datepicker._hideDatepicker();
          handled = false;
          break;
        // hide on tab out
        case 13:
          sel = $('td.' + $.datepicker._dayOverClass + ':not(.' + $.datepicker._currentClass + ')', inst.dpDiv);
          if (sel[0]) {
            $.datepicker._selectDay(event.target, inst.selectedMonth, inst.selectedYear, sel[0]);
          }
          onSelect = $.datepicker._get(inst, 'onSelect');
          if (onSelect) {
            dateStr = $.datepicker._formatDate(inst);
            // trigger custom callback
            onSelect.apply(inst.input ? inst.input[0] : null, [
              dateStr,
              inst
            ]);
          } else {
            $.datepicker._hideDatepicker();
          }
          return false;
        // don't submit the form
        case 27:
          $.datepicker._hideDatepicker();
          break;
        // hide on escape
        case 33:
          $.datepicker._adjustDate(event.target, event.ctrlKey ? -$.datepicker._get(inst, 'stepBigMonths') : -$.datepicker._get(inst, 'stepMonths'), 'M');
          break;
        // previous month/year on page up/+ ctrl
        case 34:
          $.datepicker._adjustDate(event.target, event.ctrlKey ? +$.datepicker._get(inst, 'stepBigMonths') : +$.datepicker._get(inst, 'stepMonths'), 'M');
          break;
        // next month/year on page down/+ ctrl
        case 35:
          if (event.ctrlKey || event.metaKey) {
            $.datepicker._clearDate(event.target);
          }
          handled = event.ctrlKey || event.metaKey;
          break;
        // clear on ctrl or command +end
        case 36:
          if (event.ctrlKey || event.metaKey) {
            $.datepicker._gotoToday(event.target);
          }
          handled = event.ctrlKey || event.metaKey;
          break;
        // current on ctrl or command +home
        case 37:
          if (event.ctrlKey || event.metaKey) {
            $.datepicker._adjustDate(event.target, isRTL ? +1 : -1, 'D');
          }
          handled = event.ctrlKey || event.metaKey;
          // -1 day on ctrl or command +left
          if (event.originalEvent.altKey) {
            $.datepicker._adjustDate(event.target, event.ctrlKey ? -$.datepicker._get(inst, 'stepBigMonths') : -$.datepicker._get(inst, 'stepMonths'), 'M');
          }
          // next month/year on alt +left on Mac
          break;
        case 38:
          if (event.ctrlKey || event.metaKey) {
            $.datepicker._adjustDate(event.target, -7, 'D');
          }
          handled = event.ctrlKey || event.metaKey;
          break;
        // -1 week on ctrl or command +up
        case 39:
          if (event.ctrlKey || event.metaKey) {
            $.datepicker._adjustDate(event.target, isRTL ? -1 : +1, 'D');
          }
          handled = event.ctrlKey || event.metaKey;
          // +1 day on ctrl or command +right
          if (event.originalEvent.altKey) {
            $.datepicker._adjustDate(event.target, event.ctrlKey ? +$.datepicker._get(inst, 'stepBigMonths') : +$.datepicker._get(inst, 'stepMonths'), 'M');
          }
          // next month/year on alt +right
          break;
        case 40:
          if (event.ctrlKey || event.metaKey) {
            $.datepicker._adjustDate(event.target, +7, 'D');
          }
          handled = event.ctrlKey || event.metaKey;
          break;
        // +1 week on ctrl or command +down
        default:
          handled = false;
        }
      } else if (event.keyCode === 36 && event.ctrlKey) {
        // display the date picker on ctrl+home
        $.datepicker._showDatepicker(this);
      } else {
        handled = false;
      }
      if (handled) {
        event.preventDefault();
        event.stopPropagation();
      }
    },
    _doKeyPress: function (event) {
      var chars, chr, inst = $.datepicker._getInst(event.target);
      if ($.datepicker._get(inst, 'constrainInput')) {
        chars = $.datepicker._possibleChars($.datepicker._get(inst, 'dateFormat'));
        chr = String.fromCharCode(event.charCode == null ? event.keyCode : event.charCode);
        return event.ctrlKey || event.metaKey || (chr < ' ' || !chars || chars.indexOf(chr) > -1);
      }
    },
    _doKeyUp: function (event) {
      var date, inst = $.datepicker._getInst(event.target);
      if (inst.input.val() !== inst.lastVal) {
        try {
          date = $.datepicker.parseDate($.datepicker._get(inst, 'dateFormat'), inst.input ? inst.input.val() : null, $.datepicker._getFormatConfig(inst));
          if (date) {
            // only if valid
            $.datepicker._setDateFromField(inst);
            $.datepicker._updateAlternate(inst);
            $.datepicker._updateDatepicker(inst);
          }
        } catch (err) {
        }
      }
      return true;
    },
    _showDatepicker: function (input) {
      input = input.target || input;
      if (input.nodeName.toLowerCase() !== 'input') {
        // find from button/image trigger
        input = $('input', input.parentNode)[0];
      }
      if ($.datepicker._isDisabledDatepicker(input) || $.datepicker._lastInput === input) {
        // already here
        return;
      }
      var inst, beforeShow, beforeShowSettings, isFixed, offset, showAnim, duration;
      inst = $.datepicker._getInst(input);
      if ($.datepicker._curInst && $.datepicker._curInst !== inst) {
        $.datepicker._curInst.dpDiv.stop(true, true);
        if (inst && $.datepicker._datepickerShowing) {
          $.datepicker._hideDatepicker($.datepicker._curInst.input[0]);
        }
      }
      beforeShow = $.datepicker._get(inst, 'beforeShow');
      beforeShowSettings = beforeShow ? beforeShow.apply(input, [
        input,
        inst
      ]) : {};
      if (beforeShowSettings === false) {
        return;
      }
      extendRemove(inst.settings, beforeShowSettings);
      inst.lastVal = null;
      $.datepicker._lastInput = input;
      $.datepicker._setDateFromField(inst);
      if ($.datepicker._inDialog) {
        // hide cursor
        input.value = '';
      }
      if (!$.datepicker._pos) {
        // position below input
        $.datepicker._pos = $.datepicker._findPos(input);
        $.datepicker._pos[1] += input.offsetHeight;  // add the height
      }
      isFixed = false;
      $(input).parents().each(function () {
        isFixed |= $(this).css('position') === 'fixed';
        return !isFixed;
      });
      offset = {
        left: $.datepicker._pos[0],
        top: $.datepicker._pos[1]
      };
      $.datepicker._pos = null;
      //to avoid flashes on Firefox
      inst.dpDiv.empty();
      // determine sizing offscreen
      inst.dpDiv.css({
        position: 'absolute',
        display: 'block',
        top: '-1000px'
      });
      $.datepicker._updateDatepicker(inst);
      // fix width for dynamic number of date pickers
      // and adjust position before showing
      offset = $.datepicker._checkOffset(inst, offset, isFixed);
      inst.dpDiv.css({
        position: $.datepicker._inDialog && $.blockUI ? 'static' : isFixed ? 'fixed' : 'absolute',
        display: 'none',
        left: offset.left + 'px',
        top: offset.top + 'px'
      });
      if (!inst.inline) {
        showAnim = $.datepicker._get(inst, 'showAnim');
        duration = $.datepicker._get(inst, 'duration');
        inst.dpDiv.zIndex($(input).zIndex() + 1);
        $.datepicker._datepickerShowing = true;
        if ($.effects && $.effects.effect[showAnim]) {
          inst.dpDiv.show(showAnim, $.datepicker._get(inst, 'showOptions'), duration);
        } else {
          inst.dpDiv[showAnim || 'show'](showAnim ? duration : null);
        }
        if ($.datepicker._shouldFocusInput(inst)) {
          inst.input.focus();
        }
        $.datepicker._curInst = inst;
      }
    },
    _updateDatepicker: function (inst) {
      this.maxRows = 4;
      //Reset the max number of rows being displayed (see #7043)
      instActive = inst;
      // for delegate hover events
      inst.dpDiv.empty().append(this._generateHTML(inst));
      this._attachHandlers(inst);
      inst.dpDiv.find('.' + this._dayOverClass + ' a').mouseover();
      var origyearshtml, numMonths = this._getNumberOfMonths(inst), cols = numMonths[1], width = 17;
      inst.dpDiv.removeClass('ui-datepicker-multi-2 ui-datepicker-multi-3 ui-datepicker-multi-4').width('');
      if (cols > 1) {
        inst.dpDiv.addClass('ui-datepicker-multi-' + cols).css('width', width * cols + 'em');
      }
      inst.dpDiv[(numMonths[0] !== 1 || numMonths[1] !== 1 ? 'add' : 'remove') + 'Class']('ui-datepicker-multi');
      inst.dpDiv[(this._get(inst, 'isRTL') ? 'add' : 'remove') + 'Class']('ui-datepicker-rtl');
      if (inst === $.datepicker._curInst && $.datepicker._datepickerShowing && $.datepicker._shouldFocusInput(inst)) {
        inst.input.focus();
      }
      // deffered render of the years select (to avoid flashes on Firefox)
      if (inst.yearshtml) {
        origyearshtml = inst.yearshtml;
        setTimeout(function () {
          //assure that inst.yearshtml didn't change.
          if (origyearshtml === inst.yearshtml && inst.yearshtml) {
            inst.dpDiv.find('select.ui-datepicker-year:first').replaceWith(inst.yearshtml);
          }
          origyearshtml = inst.yearshtml = null;
        }, 0);
      }
    },
    _shouldFocusInput: function (inst) {
      return inst.input && inst.input.is(':visible') && !inst.input.is(':disabled') && !inst.input.is(':focus');
    },
    _checkOffset: function (inst, offset, isFixed) {
      var dpWidth = inst.dpDiv.outerWidth(), dpHeight = inst.dpDiv.outerHeight(), inputWidth = inst.input ? inst.input.outerWidth() : 0, inputHeight = inst.input ? inst.input.outerHeight() : 0, viewWidth = document.documentElement.clientWidth + (isFixed ? 0 : $(document).scrollLeft()), viewHeight = document.documentElement.clientHeight + (isFixed ? 0 : $(document).scrollTop());
      offset.left -= this._get(inst, 'isRTL') ? dpWidth - inputWidth : 0;
      offset.left -= isFixed && offset.left === inst.input.offset().left ? $(document).scrollLeft() : 0;
      offset.top -= isFixed && offset.top === inst.input.offset().top + inputHeight ? $(document).scrollTop() : 0;
      // now check if datepicker is showing outside window viewport - move to a better place if so.
      offset.left -= Math.min(offset.left, offset.left + dpWidth > viewWidth && viewWidth > dpWidth ? Math.abs(offset.left + dpWidth - viewWidth) : 0);
      offset.top -= Math.min(offset.top, offset.top + dpHeight > viewHeight && viewHeight > dpHeight ? Math.abs(dpHeight + inputHeight) : 0);
      return offset;
    },
    _findPos: function (obj) {
      var position, inst = this._getInst(obj), isRTL = this._get(inst, 'isRTL');
      while (obj && (obj.type === 'hidden' || obj.nodeType !== 1 || $.expr.filters.hidden(obj))) {
        obj = obj[isRTL ? 'previousSibling' : 'nextSibling'];
      }
      position = $(obj).offset();
      return [
        position.left,
        position.top
      ];
    },
    _hideDatepicker: function (input) {
      var showAnim, duration, postProcess, onClose, inst = this._curInst;
      if (!inst || input && inst !== $.data(input, PROP_NAME)) {
        return;
      }
      if (this._datepickerShowing) {
        showAnim = this._get(inst, 'showAnim');
        duration = this._get(inst, 'duration');
        postProcess = function () {
          $.datepicker._tidyDialog(inst);
        };
        // DEPRECATED: after BC for 1.8.x $.effects[ showAnim ] is not needed
        if ($.effects && ($.effects.effect[showAnim] || $.effects[showAnim])) {
          inst.dpDiv.hide(showAnim, $.datepicker._get(inst, 'showOptions'), duration, postProcess);
        } else {
          inst.dpDiv[showAnim === 'slideDown' ? 'slideUp' : showAnim === 'fadeIn' ? 'fadeOut' : 'hide'](showAnim ? duration : null, postProcess);
        }
        if (!showAnim) {
          postProcess();
        }
        this._datepickerShowing = false;
        onClose = this._get(inst, 'onClose');
        if (onClose) {
          onClose.apply(inst.input ? inst.input[0] : null, [
            inst.input ? inst.input.val() : '',
            inst
          ]);
        }
        this._lastInput = null;
        if (this._inDialog) {
          this._dialogInput.css({
            position: 'absolute',
            left: '0',
            top: '-100px'
          });
          if ($.blockUI) {
            $.unblockUI();
            $('body').append(this.dpDiv);
          }
        }
        this._inDialog = false;
      }
    },
    _tidyDialog: function (inst) {
      inst.dpDiv.removeClass(this._dialogClass).unbind('.ui-datepicker-calendar');
    },
    _checkExternalClick: function (event) {
      if (!$.datepicker._curInst) {
        return;
      }
      var $target = $(event.target), inst = $.datepicker._getInst($target[0]);
      if ($target[0].id !== $.datepicker._mainDivId && $target.parents('#' + $.datepicker._mainDivId).length === 0 && !$target.hasClass($.datepicker.markerClassName) && !$target.closest('.' + $.datepicker._triggerClass).length && $.datepicker._datepickerShowing && !($.datepicker._inDialog && $.blockUI) || $target.hasClass($.datepicker.markerClassName) && $.datepicker._curInst !== inst) {
        $.datepicker._hideDatepicker();
      }
    },
    _adjustDate: function (id, offset, period) {
      var target = $(id), inst = this._getInst(target[0]);
      if (this._isDisabledDatepicker(target[0])) {
        return;
      }
      this._adjustInstDate(inst, offset + (period === 'M' ? this._get(inst, 'showCurrentAtPos') : 0), period);
      this._updateDatepicker(inst);
    },
    _gotoToday: function (id) {
      var date, target = $(id), inst = this._getInst(target[0]);
      if (this._get(inst, 'gotoCurrent') && inst.currentDay) {
        inst.selectedDay = inst.currentDay;
        inst.drawMonth = inst.selectedMonth = inst.currentMonth;
        inst.drawYear = inst.selectedYear = inst.currentYear;
      } else {
        date = new Date();
        inst.selectedDay = date.getDate();
        inst.drawMonth = inst.selectedMonth = date.getMonth();
        inst.drawYear = inst.selectedYear = date.getFullYear();
      }
      this._notifyChange(inst);
      this._adjustDate(target);
    },
    _selectMonthYear: function (id, select, period) {
      var target = $(id), inst = this._getInst(target[0]);
      inst['selected' + (period === 'M' ? 'Month' : 'Year')] = inst['draw' + (period === 'M' ? 'Month' : 'Year')] = parseInt(select.options[select.selectedIndex].value, 10);
      this._notifyChange(inst);
      this._adjustDate(target);
    },
    _selectDay: function (id, month, year, td) {
      var inst, target = $(id);
      if ($(td).hasClass(this._unselectableClass) || this._isDisabledDatepicker(target[0])) {
        return;
      }
      inst = this._getInst(target[0]);
      inst.selectedDay = inst.currentDay = $('a', td).html();
      inst.selectedMonth = inst.currentMonth = month;
      inst.selectedYear = inst.currentYear = year;
      this._selectDate(id, this._formatDate(inst, inst.currentDay, inst.currentMonth, inst.currentYear));
    },
    _clearDate: function (id) {
      var target = $(id);
      this._selectDate(target, '');
    },
    _selectDate: function (id, dateStr) {
      var onSelect, target = $(id), inst = this._getInst(target[0]);
      dateStr = dateStr != null ? dateStr : this._formatDate(inst);
      if (inst.input) {
        inst.input.val(dateStr);
      }
      this._updateAlternate(inst);
      onSelect = this._get(inst, 'onSelect');
      if (onSelect) {
        onSelect.apply(inst.input ? inst.input[0] : null, [
          dateStr,
          inst
        ]);  // trigger custom callback
      } else if (inst.input) {
        inst.input.trigger('change');  // fire the change event
      }
      if (inst.inline) {
        this._updateDatepicker(inst);
      } else {
        this._hideDatepicker();
        this._lastInput = inst.input[0];
        if (typeof inst.input[0] !== 'object') {
          inst.input.focus();  // restore focus
        }
        this._lastInput = null;
      }
    },
    _updateAlternate: function (inst) {
      var altFormat, date, dateStr, altField = this._get(inst, 'altField');
      if (altField) {
        // update alternate field too
        altFormat = this._get(inst, 'altFormat') || this._get(inst, 'dateFormat');
        date = this._getDate(inst);
        dateStr = this.formatDate(altFormat, date, this._getFormatConfig(inst));
        $(altField).each(function () {
          $(this).val(dateStr);
        });
      }
    },
    noWeekends: function (date) {
      var day = date.getDay();
      return [
        day > 0 && day < 6,
        ''
      ];
    },
    iso8601Week: function (date) {
      var time, checkDate = new Date(date.getTime());
      // Find Thursday of this week starting on Monday
      checkDate.setDate(checkDate.getDate() + 4 - (checkDate.getDay() || 7));
      time = checkDate.getTime();
      checkDate.setMonth(0);
      // Compare with Jan 1
      checkDate.setDate(1);
      return Math.floor(Math.round((time - checkDate) / 86400000) / 7) + 1;
    },
    parseDate: function (format, value, settings) {
      if (format == null || value == null) {
        throw 'Invalid arguments';
      }
      value = typeof value === 'object' ? value.toString() : value + '';
      if (value === '') {
        return null;
      }
      var iFormat, dim, extra, iValue = 0, shortYearCutoffTemp = (settings ? settings.shortYearCutoff : null) || this._defaults.shortYearCutoff, shortYearCutoff = typeof shortYearCutoffTemp !== 'string' ? shortYearCutoffTemp : new Date().getFullYear() % 100 + parseInt(shortYearCutoffTemp, 10), dayNamesShort = (settings ? settings.dayNamesShort : null) || this._defaults.dayNamesShort, dayNames = (settings ? settings.dayNames : null) || this._defaults.dayNames, monthNamesShort = (settings ? settings.monthNamesShort : null) || this._defaults.monthNamesShort, monthNames = (settings ? settings.monthNames : null) || this._defaults.monthNames, year = -1, month = -1, day = -1, doy = -1, literal = false, date,
        // Check whether a format character is doubled
        lookAhead = function (match) {
          var matches = iFormat + 1 < format.length && format.charAt(iFormat + 1) === match;
          if (matches) {
            iFormat++;
          }
          return matches;
        },
        // Extract a number from the string value
        getNumber = function (match) {
          var isDoubled = lookAhead(match), size = match === '@' ? 14 : match === '!' ? 20 : match === 'y' && isDoubled ? 4 : match === 'o' ? 3 : 2, digits = new RegExp('^\\d{1,' + size + '}'), num = value.substring(iValue).match(digits);
          if (!num) {
            throw 'Missing number at position ' + iValue;
          }
          iValue += num[0].length;
          return parseInt(num[0], 10);
        },
        // Extract a name from the string value and convert to an index
        getName = function (match, shortNames, longNames) {
          var index = -1, names = $.map(lookAhead(match) ? longNames : shortNames, function (v, k) {
              return [[
                  k,
                  v
                ]];
            }).sort(function (a, b) {
              return -(a[1].length - b[1].length);
            });
          $.each(names, function (i, pair) {
            var name = pair[1];
            if (value.substr(iValue, name.length).toLowerCase() === name.toLowerCase()) {
              index = pair[0];
              iValue += name.length;
              return false;
            }
          });
          if (index !== -1) {
            return index + 1;
          } else {
            throw 'Unknown name at position ' + iValue;
          }
        },
        // Confirm that a literal character matches the string value
        checkLiteral = function () {
          if (value.charAt(iValue) !== format.charAt(iFormat)) {
            throw 'Unexpected literal at position ' + iValue;
          }
          iValue++;
        };
      for (iFormat = 0; iFormat < format.length; iFormat++) {
        if (literal) {
          if (format.charAt(iFormat) === '\'' && !lookAhead('\'')) {
            literal = false;
          } else {
            checkLiteral();
          }
        } else {
          switch (format.charAt(iFormat)) {
          case 'd':
            day = getNumber('d');
            break;
          case 'D':
            getName('D', dayNamesShort, dayNames);
            break;
          case 'o':
            doy = getNumber('o');
            break;
          case 'm':
            month = getNumber('m');
            break;
          case 'M':
            month = getName('M', monthNamesShort, monthNames);
            break;
          case 'y':
            year = getNumber('y');
            break;
          case '@':
            date = new Date(getNumber('@'));
            year = date.getFullYear();
            month = date.getMonth() + 1;
            day = date.getDate();
            break;
          case '!':
            date = new Date((getNumber('!') - this._ticksTo1970) / 10000);
            year = date.getFullYear();
            month = date.getMonth() + 1;
            day = date.getDate();
            break;
          case '\'':
            if (lookAhead('\'')) {
              checkLiteral();
            } else {
              literal = true;
            }
            break;
          default:
            checkLiteral();
          }
        }
      }
      if (iValue < value.length) {
        extra = value.substr(iValue);
        if (!/^\s+/.test(extra)) {
          throw 'Extra/unparsed characters found in date: ' + extra;
        }
      }
      if (year === -1) {
        year = new Date().getFullYear();
      } else if (year < 100) {
        year += new Date().getFullYear() - new Date().getFullYear() % 100 + (year <= shortYearCutoff ? 0 : -100);
      }
      if (doy > -1) {
        month = 1;
        day = doy;
        do {
          dim = this._getDaysInMonth(year, month - 1);
          if (day <= dim) {
            break;
          }
          month++;
          day -= dim;
        } while (true);
      }
      date = this._daylightSavingAdjust(new Date(year, month - 1, day));
      if (date.getFullYear() !== year || date.getMonth() + 1 !== month || date.getDate() !== day) {
        throw 'Invalid date';  // E.g. 31/02/00
      }
      return date;
    },
    ATOM: 'yy-mm-dd',
    COOKIE: 'D, dd M yy',
    ISO_8601: 'yy-mm-dd',
    RFC_822: 'D, d M y',
    RFC_850: 'DD, dd-M-y',
    RFC_1036: 'D, d M y',
    RFC_1123: 'D, d M yy',
    RFC_2822: 'D, d M yy',
    RSS: 'D, d M y',
    TICKS: '!',
    TIMESTAMP: '@',
    W3C: 'yy-mm-dd',
    _ticksTo1970: ((1970 - 1) * 365 + Math.floor(1970 / 4) - Math.floor(1970 / 100) + Math.floor(1970 / 400)) * 24 * 60 * 60 * 10000000,
    formatDate: function (format, date, settings) {
      if (!date) {
        return '';
      }
      var iFormat, dayNamesShort = (settings ? settings.dayNamesShort : null) || this._defaults.dayNamesShort, dayNames = (settings ? settings.dayNames : null) || this._defaults.dayNames, monthNamesShort = (settings ? settings.monthNamesShort : null) || this._defaults.monthNamesShort, monthNames = (settings ? settings.monthNames : null) || this._defaults.monthNames,
        // Check whether a format character is doubled
        lookAhead = function (match) {
          var matches = iFormat + 1 < format.length && format.charAt(iFormat + 1) === match;
          if (matches) {
            iFormat++;
          }
          return matches;
        },
        // Format a number, with leading zero if necessary
        formatNumber = function (match, value, len) {
          var num = '' + value;
          if (lookAhead(match)) {
            while (num.length < len) {
              num = '0' + num;
            }
          }
          return num;
        },
        // Format a name, short or long as requested
        formatName = function (match, value, shortNames, longNames) {
          return lookAhead(match) ? longNames[value] : shortNames[value];
        }, output = '', literal = false;
      if (date) {
        for (iFormat = 0; iFormat < format.length; iFormat++) {
          if (literal) {
            if (format.charAt(iFormat) === '\'' && !lookAhead('\'')) {
              literal = false;
            } else {
              output += format.charAt(iFormat);
            }
          } else {
            switch (format.charAt(iFormat)) {
            case 'd':
              output += formatNumber('d', date.getDate(), 2);
              break;
            case 'D':
              output += formatName('D', date.getDay(), dayNamesShort, dayNames);
              break;
            case 'o':
              output += formatNumber('o', Math.round((new Date(date.getFullYear(), date.getMonth(), date.getDate()).getTime() - new Date(date.getFullYear(), 0, 0).getTime()) / 86400000), 3);
              break;
            case 'm':
              output += formatNumber('m', date.getMonth() + 1, 2);
              break;
            case 'M':
              output += formatName('M', date.getMonth(), monthNamesShort, monthNames);
              break;
            case 'y':
              output += lookAhead('y') ? date.getFullYear() : (date.getYear() % 100 < 10 ? '0' : '') + date.getYear() % 100;
              break;
            case '@':
              output += date.getTime();
              break;
            case '!':
              output += date.getTime() * 10000 + this._ticksTo1970;
              break;
            case '\'':
              if (lookAhead('\'')) {
                output += '\'';
              } else {
                literal = true;
              }
              break;
            default:
              output += format.charAt(iFormat);
            }
          }
        }
      }
      return output;
    },
    _possibleChars: function (format) {
      var iFormat, chars = '', literal = false,
        // Check whether a format character is doubled
        lookAhead = function (match) {
          var matches = iFormat + 1 < format.length && format.charAt(iFormat + 1) === match;
          if (matches) {
            iFormat++;
          }
          return matches;
        };
      for (iFormat = 0; iFormat < format.length; iFormat++) {
        if (literal) {
          if (format.charAt(iFormat) === '\'' && !lookAhead('\'')) {
            literal = false;
          } else {
            chars += format.charAt(iFormat);
          }
        } else {
          switch (format.charAt(iFormat)) {
          case 'd':
          case 'm':
          case 'y':
          case '@':
            chars += '0123456789';
            break;
          case 'D':
          case 'M':
            return null;
          // Accept anything
          case '\'':
            if (lookAhead('\'')) {
              chars += '\'';
            } else {
              literal = true;
            }
            break;
          default:
            chars += format.charAt(iFormat);
          }
        }
      }
      return chars;
    },
    _get: function (inst, name) {
      return inst.settings[name] !== undefined ? inst.settings[name] : this._defaults[name];
    },
    _setDateFromField: function (inst, noDefault) {
      if (inst.input.val() === inst.lastVal) {
        return;
      }
      var dateFormat = this._get(inst, 'dateFormat'), dates = inst.lastVal = inst.input ? inst.input.val() : null, defaultDate = this._getDefaultDate(inst), date = defaultDate, settings = this._getFormatConfig(inst);
      try {
        date = this.parseDate(dateFormat, dates, settings) || defaultDate;
      } catch (event) {
        dates = noDefault ? '' : dates;
      }
      inst.selectedDay = date.getDate();
      inst.drawMonth = inst.selectedMonth = date.getMonth();
      inst.drawYear = inst.selectedYear = date.getFullYear();
      inst.currentDay = dates ? date.getDate() : 0;
      inst.currentMonth = dates ? date.getMonth() : 0;
      inst.currentYear = dates ? date.getFullYear() : 0;
      this._adjustInstDate(inst);
    },
    _getDefaultDate: function (inst) {
      return this._restrictMinMax(inst, this._determineDate(inst, this._get(inst, 'defaultDate'), new Date()));
    },
    _determineDate: function (inst, date, defaultDate) {
      var offsetNumeric = function (offset) {
          var date = new Date();
          date.setDate(date.getDate() + offset);
          return date;
        }, offsetString = function (offset) {
          try {
            return $.datepicker.parseDate($.datepicker._get(inst, 'dateFormat'), offset, $.datepicker._getFormatConfig(inst));
          } catch (e) {
          }
          var date = (offset.toLowerCase().match(/^c/) ? $.datepicker._getDate(inst) : null) || new Date(), year = date.getFullYear(), month = date.getMonth(), day = date.getDate(), pattern = /([+\-]?[0-9]+)\s*(d|D|w|W|m|M|y|Y)?/g, matches = pattern.exec(offset);
          while (matches) {
            switch (matches[2] || 'd') {
            case 'd':
            case 'D':
              day += parseInt(matches[1], 10);
              break;
            case 'w':
            case 'W':
              day += parseInt(matches[1], 10) * 7;
              break;
            case 'm':
            case 'M':
              month += parseInt(matches[1], 10);
              day = Math.min(day, $.datepicker._getDaysInMonth(year, month));
              break;
            case 'y':
            case 'Y':
              year += parseInt(matches[1], 10);
              day = Math.min(day, $.datepicker._getDaysInMonth(year, month));
              break;
            }
            matches = pattern.exec(offset);
          }
          return new Date(year, month, day);
        }, newDate = date == null || date === '' ? defaultDate : typeof date === 'string' ? offsetString(date) : typeof date === 'number' ? isNaN(date) ? defaultDate : offsetNumeric(date) : new Date(date.getTime());
      newDate = newDate && newDate.toString() === 'Invalid Date' ? defaultDate : newDate;
      if (newDate) {
        newDate.setHours(0);
        newDate.setMinutes(0);
        newDate.setSeconds(0);
        newDate.setMilliseconds(0);
      }
      return this._daylightSavingAdjust(newDate);
    },
    _daylightSavingAdjust: function (date) {
      if (!date) {
        return null;
      }
      date.setHours(date.getHours() > 12 ? date.getHours() + 2 : 0);
      return date;
    },
    _setDate: function (inst, date, noChange) {
      var clear = !date, origMonth = inst.selectedMonth, origYear = inst.selectedYear, newDate = this._restrictMinMax(inst, this._determineDate(inst, date, new Date()));
      inst.selectedDay = inst.currentDay = newDate.getDate();
      inst.drawMonth = inst.selectedMonth = inst.currentMonth = newDate.getMonth();
      inst.drawYear = inst.selectedYear = inst.currentYear = newDate.getFullYear();
      if ((origMonth !== inst.selectedMonth || origYear !== inst.selectedYear) && !noChange) {
        this._notifyChange(inst);
      }
      this._adjustInstDate(inst);
      if (inst.input) {
        inst.input.val(clear ? '' : this._formatDate(inst));
      }
    },
    _getDate: function (inst) {
      var startDate = !inst.currentYear || inst.input && inst.input.val() === '' ? null : this._daylightSavingAdjust(new Date(inst.currentYear, inst.currentMonth, inst.currentDay));
      return startDate;
    },
    _attachHandlers: function (inst) {
      var stepMonths = this._get(inst, 'stepMonths'), id = '#' + inst.id.replace(/\\\\/g, '\\');
      inst.dpDiv.find('[data-handler]').map(function () {
        var handler = {
            prev: function () {
              $.datepicker._adjustDate(id, -stepMonths, 'M');
            },
            next: function () {
              $.datepicker._adjustDate(id, +stepMonths, 'M');
            },
            hide: function () {
              $.datepicker._hideDatepicker();
            },
            today: function () {
              $.datepicker._gotoToday(id);
            },
            selectDay: function () {
              $.datepicker._selectDay(id, +this.getAttribute('data-month'), +this.getAttribute('data-year'), this);
              return false;
            },
            selectMonth: function () {
              $.datepicker._selectMonthYear(id, this, 'M');
              return false;
            },
            selectYear: function () {
              $.datepicker._selectMonthYear(id, this, 'Y');
              return false;
            }
          };
        $(this).bind(this.getAttribute('data-event'), handler[this.getAttribute('data-handler')]);
      });
    },
    _generateHTML: function (inst) {
      var maxDraw, prevText, prev, nextText, next, currentText, gotoDate, controls, buttonPanel, firstDay, showWeek, dayNames, dayNamesMin, monthNames, monthNamesShort, beforeShowDay, showOtherMonths, selectOtherMonths, defaultDate, html, dow, row, group, col, selectedDate, cornerClass, calender, thead, day, daysInMonth, leadDays, curRows, numRows, printDate, dRow, tbody, daySettings, otherMonth, unselectable, tempDate = new Date(), today = this._daylightSavingAdjust(new Date(tempDate.getFullYear(), tempDate.getMonth(), tempDate.getDate())),
        // clear time
        isRTL = this._get(inst, 'isRTL'), showButtonPanel = this._get(inst, 'showButtonPanel'), hideIfNoPrevNext = this._get(inst, 'hideIfNoPrevNext'), navigationAsDateFormat = this._get(inst, 'navigationAsDateFormat'), numMonths = this._getNumberOfMonths(inst), showCurrentAtPos = this._get(inst, 'showCurrentAtPos'), stepMonths = this._get(inst, 'stepMonths'), isMultiMonth = numMonths[0] !== 1 || numMonths[1] !== 1, currentDate = this._daylightSavingAdjust(!inst.currentDay ? new Date(9999, 9, 9) : new Date(inst.currentYear, inst.currentMonth, inst.currentDay)), minDate = this._getMinMaxDate(inst, 'min'), maxDate = this._getMinMaxDate(inst, 'max'), drawMonth = inst.drawMonth - showCurrentAtPos, drawYear = inst.drawYear;
      if (drawMonth < 0) {
        drawMonth += 12;
        drawYear--;
      }
      if (maxDate) {
        maxDraw = this._daylightSavingAdjust(new Date(maxDate.getFullYear(), maxDate.getMonth() - numMonths[0] * numMonths[1] + 1, maxDate.getDate()));
        maxDraw = minDate && maxDraw < minDate ? minDate : maxDraw;
        while (this._daylightSavingAdjust(new Date(drawYear, drawMonth, 1)) > maxDraw) {
          drawMonth--;
          if (drawMonth < 0) {
            drawMonth = 11;
            drawYear--;
          }
        }
      }
      inst.drawMonth = drawMonth;
      inst.drawYear = drawYear;
      prevText = this._get(inst, 'prevText');
      prevText = !navigationAsDateFormat ? prevText : this.formatDate(prevText, this._daylightSavingAdjust(new Date(drawYear, drawMonth - stepMonths, 1)), this._getFormatConfig(inst));
      prev = this._canAdjustMonth(inst, -1, drawYear, drawMonth) ? '<a class=\'ui-datepicker-prev ui-corner-all\' data-handler=\'prev\' data-event=\'click\'' + ' title=\'' + prevText + '\'><span class=\'ui-icon ui-icon-circle-triangle-' + (isRTL ? 'e' : 'w') + '\'>' + prevText + '</span></a>' : hideIfNoPrevNext ? '' : '<a class=\'ui-datepicker-prev ui-corner-all ui-state-disabled\' title=\'' + prevText + '\'><span class=\'ui-icon ui-icon-circle-triangle-' + (isRTL ? 'e' : 'w') + '\'>' + prevText + '</span></a>';
      nextText = this._get(inst, 'nextText');
      nextText = !navigationAsDateFormat ? nextText : this.formatDate(nextText, this._daylightSavingAdjust(new Date(drawYear, drawMonth + stepMonths, 1)), this._getFormatConfig(inst));
      next = this._canAdjustMonth(inst, +1, drawYear, drawMonth) ? '<a class=\'ui-datepicker-next ui-corner-all\' data-handler=\'next\' data-event=\'click\'' + ' title=\'' + nextText + '\'><span class=\'ui-icon ui-icon-circle-triangle-' + (isRTL ? 'w' : 'e') + '\'>' + nextText + '</span></a>' : hideIfNoPrevNext ? '' : '<a class=\'ui-datepicker-next ui-corner-all ui-state-disabled\' title=\'' + nextText + '\'><span class=\'ui-icon ui-icon-circle-triangle-' + (isRTL ? 'w' : 'e') + '\'>' + nextText + '</span></a>';
      currentText = this._get(inst, 'currentText');
      gotoDate = this._get(inst, 'gotoCurrent') && inst.currentDay ? currentDate : today;
      currentText = !navigationAsDateFormat ? currentText : this.formatDate(currentText, gotoDate, this._getFormatConfig(inst));
      controls = !inst.inline ? '<button type=\'button\' class=\'ui-datepicker-close ui-state-default ui-priority-primary ui-corner-all\' data-handler=\'hide\' data-event=\'click\'>' + this._get(inst, 'closeText') + '</button>' : '';
      buttonPanel = showButtonPanel ? '<div class=\'ui-datepicker-buttonpane ui-widget-content\'>' + (isRTL ? controls : '') + (this._isInRange(inst, gotoDate) ? '<button type=\'button\' class=\'ui-datepicker-current ui-state-default ui-priority-secondary ui-corner-all\' data-handler=\'today\' data-event=\'click\'' + '>' + currentText + '</button>' : '') + (isRTL ? '' : controls) + '</div>' : '';
      firstDay = parseInt(this._get(inst, 'firstDay'), 10);
      firstDay = isNaN(firstDay) ? 0 : firstDay;
      showWeek = this._get(inst, 'showWeek');
      dayNames = this._get(inst, 'dayNames');
      dayNamesMin = this._get(inst, 'dayNamesMin');
      monthNames = this._get(inst, 'monthNames');
      monthNamesShort = this._get(inst, 'monthNamesShort');
      beforeShowDay = this._get(inst, 'beforeShowDay');
      showOtherMonths = this._get(inst, 'showOtherMonths');
      selectOtherMonths = this._get(inst, 'selectOtherMonths');
      defaultDate = this._getDefaultDate(inst);
      html = '';
      dow;
      for (row = 0; row < numMonths[0]; row++) {
        group = '';
        this.maxRows = 4;
        for (col = 0; col < numMonths[1]; col++) {
          selectedDate = this._daylightSavingAdjust(new Date(drawYear, drawMonth, inst.selectedDay));
          cornerClass = ' ui-corner-all';
          calender = '';
          if (isMultiMonth) {
            calender += '<div class=\'ui-datepicker-group';
            if (numMonths[1] > 1) {
              switch (col) {
              case 0:
                calender += ' ui-datepicker-group-first';
                cornerClass = ' ui-corner-' + (isRTL ? 'right' : 'left');
                break;
              case numMonths[1] - 1:
                calender += ' ui-datepicker-group-last';
                cornerClass = ' ui-corner-' + (isRTL ? 'left' : 'right');
                break;
              default:
                calender += ' ui-datepicker-group-middle';
                cornerClass = '';
                break;
              }
            }
            calender += '\'>';
          }
          calender += '<div class=\'ui-datepicker-header ui-widget-header ui-helper-clearfix' + cornerClass + '\'>' + (/all|left/.test(cornerClass) && row === 0 ? isRTL ? next : prev : '') + (/all|right/.test(cornerClass) && row === 0 ? isRTL ? prev : next : '') + this._generateMonthYearHeader(inst, drawMonth, drawYear, minDate, maxDate, row > 0 || col > 0, monthNames, monthNamesShort) + '</div><table class=\'ui-datepicker-calendar\'><thead>' + '<tr>';
          thead = showWeek ? '<th class=\'ui-datepicker-week-col\'>' + this._get(inst, 'weekHeader') + '</th>' : '';
          for (dow = 0; dow < 7; dow++) {
            // days of the week
            day = (dow + firstDay) % 7;
            thead += '<th' + ((dow + firstDay + 6) % 7 >= 5 ? ' class=\'ui-datepicker-week-end\'' : '') + '>' + '<span title=\'' + dayNames[day] + '\'>' + dayNamesMin[day] + '</span></th>';
          }
          calender += thead + '</tr></thead><tbody>';
          daysInMonth = this._getDaysInMonth(drawYear, drawMonth);
          if (drawYear === inst.selectedYear && drawMonth === inst.selectedMonth) {
            inst.selectedDay = Math.min(inst.selectedDay, daysInMonth);
          }
          leadDays = (this._getFirstDayOfMonth(drawYear, drawMonth) - firstDay + 7) % 7;
          curRows = Math.ceil((leadDays + daysInMonth) / 7);
          // calculate the number of rows to generate
          numRows = isMultiMonth ? this.maxRows > curRows ? this.maxRows : curRows : curRows;
          //If multiple months, use the higher number of rows (see #7043)
          this.maxRows = numRows;
          printDate = this._daylightSavingAdjust(new Date(drawYear, drawMonth, 1 - leadDays));
          for (dRow = 0; dRow < numRows; dRow++) {
            // create date picker rows
            calender += '<tr>';
            tbody = !showWeek ? '' : '<td class=\'ui-datepicker-week-col\'>' + this._get(inst, 'calculateWeek')(printDate) + '</td>';
            for (dow = 0; dow < 7; dow++) {
              // create date picker days
              daySettings = beforeShowDay ? beforeShowDay.apply(inst.input ? inst.input[0] : null, [printDate]) : [
                true,
                ''
              ];
              otherMonth = printDate.getMonth() !== drawMonth;
              unselectable = otherMonth && !selectOtherMonths || !daySettings[0] || minDate && printDate < minDate || maxDate && printDate > maxDate;
              tbody += '<td class=\'' + ((dow + firstDay + 6) % 7 >= 5 ? ' ui-datepicker-week-end' : '') + (otherMonth ? ' ui-datepicker-other-month' : '') + (printDate.getTime() === selectedDate.getTime() && drawMonth === inst.selectedMonth && inst._keyEvent || defaultDate.getTime() === printDate.getTime() && defaultDate.getTime() === selectedDate.getTime() ? ' ' + this._dayOverClass : '') + (unselectable ? ' ' + this._unselectableClass + ' ui-state-disabled' : '') + (otherMonth && !showOtherMonths ? '' : ' ' + daySettings[1] + (printDate.getTime() === currentDate.getTime() ? ' ' + this._currentClass : '') + (printDate.getTime() === today.getTime() ? ' ui-datepicker-today' : '')) + '\'' + ((!otherMonth || showOtherMonths) && daySettings[2] ? ' title=\'' + daySettings[2].replace(/'/g, '&#39;') + '\'' : '') + (unselectable ? '' : ' data-handler=\'selectDay\' data-event=\'click\' data-month=\'' + printDate.getMonth() + '\' data-year=\'' + printDate.getFullYear() + '\'') + '>' + (otherMonth && !showOtherMonths ? '&#xa0;' : unselectable ? '<span class=\'ui-state-default\'>' + printDate.getDate() + '</span>' : '<a class=\'ui-state-default' + (printDate.getTime() === today.getTime() ? ' ui-state-highlight' : '') + (printDate.getTime() === currentDate.getTime() ? ' ui-state-active' : '') + (otherMonth ? ' ui-priority-secondary' : '') + '\' href=\'#\'>' + printDate.getDate() + '</a>') + '</td>';
              // display selectable date
              printDate.setDate(printDate.getDate() + 1);
              printDate = this._daylightSavingAdjust(printDate);
            }
            calender += tbody + '</tr>';
          }
          drawMonth++;
          if (drawMonth > 11) {
            drawMonth = 0;
            drawYear++;
          }
          calender += '</tbody></table>' + (isMultiMonth ? '</div>' + (numMonths[0] > 0 && col === numMonths[1] - 1 ? '<div class=\'ui-datepicker-row-break\'></div>' : '') : '');
          group += calender;
        }
        html += group;
      }
      html += buttonPanel;
      inst._keyEvent = false;
      return html;
    },
    _generateMonthYearHeader: function (inst, drawMonth, drawYear, minDate, maxDate, secondary, monthNames, monthNamesShort) {
      var inMinYear, inMaxYear, month, years, thisYear, determineYear, year, endYear, changeMonth = this._get(inst, 'changeMonth'), changeYear = this._get(inst, 'changeYear'), showMonthAfterYear = this._get(inst, 'showMonthAfterYear'), html = '<div class=\'ui-datepicker-title\'>', monthHtml = '';
      // month selection
      if (secondary || !changeMonth) {
        monthHtml += '<span class=\'ui-datepicker-month\'>' + monthNames[drawMonth] + '</span>';
      } else {
        inMinYear = minDate && minDate.getFullYear() === drawYear;
        inMaxYear = maxDate && maxDate.getFullYear() === drawYear;
        monthHtml += '<select class=\'ui-datepicker-month\' data-handler=\'selectMonth\' data-event=\'change\'>';
        for (month = 0; month < 12; month++) {
          if ((!inMinYear || month >= minDate.getMonth()) && (!inMaxYear || month <= maxDate.getMonth())) {
            monthHtml += '<option value=\'' + month + '\'' + (month === drawMonth ? ' selected=\'selected\'' : '') + '>' + monthNamesShort[month] + '</option>';
          }
        }
        monthHtml += '</select>';
      }
      if (!showMonthAfterYear) {
        html += monthHtml + (secondary || !(changeMonth && changeYear) ? '&#xa0;' : '');
      }
      // year selection
      if (!inst.yearshtml) {
        inst.yearshtml = '';
        if (secondary || !changeYear) {
          html += '<span class=\'ui-datepicker-year\'>' + drawYear + '</span>';
        } else {
          // determine range of years to display
          years = this._get(inst, 'yearRange').split(':');
          thisYear = new Date().getFullYear();
          determineYear = function (value) {
            var year = value.match(/c[+\-].*/) ? drawYear + parseInt(value.substring(1), 10) : value.match(/[+\-].*/) ? thisYear + parseInt(value, 10) : parseInt(value, 10);
            return isNaN(year) ? thisYear : year;
          };
          year = determineYear(years[0]);
          endYear = Math.max(year, determineYear(years[1] || ''));
          year = minDate ? Math.max(year, minDate.getFullYear()) : year;
          endYear = maxDate ? Math.min(endYear, maxDate.getFullYear()) : endYear;
          inst.yearshtml += '<select class=\'ui-datepicker-year\' data-handler=\'selectYear\' data-event=\'change\'>';
          for (; year <= endYear; year++) {
            inst.yearshtml += '<option value=\'' + year + '\'' + (year === drawYear ? ' selected=\'selected\'' : '') + '>' + year + '</option>';
          }
          inst.yearshtml += '</select>';
          html += inst.yearshtml;
          inst.yearshtml = null;
        }
      }
      html += this._get(inst, 'yearSuffix');
      if (showMonthAfterYear) {
        html += (secondary || !(changeMonth && changeYear) ? '&#xa0;' : '') + monthHtml;
      }
      html += '</div>';
      // Close datepicker_header
      return html;
    },
    _adjustInstDate: function (inst, offset, period) {
      var year = inst.drawYear + (period === 'Y' ? offset : 0), month = inst.drawMonth + (period === 'M' ? offset : 0), day = Math.min(inst.selectedDay, this._getDaysInMonth(year, month)) + (period === 'D' ? offset : 0), date = this._restrictMinMax(inst, this._daylightSavingAdjust(new Date(year, month, day)));
      inst.selectedDay = date.getDate();
      inst.drawMonth = inst.selectedMonth = date.getMonth();
      inst.drawYear = inst.selectedYear = date.getFullYear();
      if (period === 'M' || period === 'Y') {
        this._notifyChange(inst);
      }
    },
    _restrictMinMax: function (inst, date) {
      var minDate = this._getMinMaxDate(inst, 'min'), maxDate = this._getMinMaxDate(inst, 'max'), newDate = minDate && date < minDate ? minDate : date;
      return maxDate && newDate > maxDate ? maxDate : newDate;
    },
    _notifyChange: function (inst) {
      var onChange = this._get(inst, 'onChangeMonthYear');
      if (onChange) {
        onChange.apply(inst.input ? inst.input[0] : null, [
          inst.selectedYear,
          inst.selectedMonth + 1,
          inst
        ]);
      }
    },
    _getNumberOfMonths: function (inst) {
      var numMonths = this._get(inst, 'numberOfMonths');
      return numMonths == null ? [
        1,
        1
      ] : typeof numMonths === 'number' ? [
        1,
        numMonths
      ] : numMonths;
    },
    _getMinMaxDate: function (inst, minMax) {
      return this._determineDate(inst, this._get(inst, minMax + 'Date'), null);
    },
    _getDaysInMonth: function (year, month) {
      return 32 - this._daylightSavingAdjust(new Date(year, month, 32)).getDate();
    },
    _getFirstDayOfMonth: function (year, month) {
      return new Date(year, month, 1).getDay();
    },
    _canAdjustMonth: function (inst, offset, curYear, curMonth) {
      var numMonths = this._getNumberOfMonths(inst), date = this._daylightSavingAdjust(new Date(curYear, curMonth + (offset < 0 ? offset : numMonths[0] * numMonths[1]), 1));
      if (offset < 0) {
        date.setDate(this._getDaysInMonth(date.getFullYear(), date.getMonth()));
      }
      return this._isInRange(inst, date);
    },
    _isInRange: function (inst, date) {
      var yearSplit, currentYear, minDate = this._getMinMaxDate(inst, 'min'), maxDate = this._getMinMaxDate(inst, 'max'), minYear = null, maxYear = null, years = this._get(inst, 'yearRange');
      if (years) {
        yearSplit = years.split(':');
        currentYear = new Date().getFullYear();
        minYear = parseInt(yearSplit[0], 10);
        maxYear = parseInt(yearSplit[1], 10);
        if (yearSplit[0].match(/[+\-].*/)) {
          minYear += currentYear;
        }
        if (yearSplit[1].match(/[+\-].*/)) {
          maxYear += currentYear;
        }
      }
      return (!minDate || date.getTime() >= minDate.getTime()) && (!maxDate || date.getTime() <= maxDate.getTime()) && (!minYear || date.getFullYear() >= minYear) && (!maxYear || date.getFullYear() <= maxYear);
    },
    _getFormatConfig: function (inst) {
      var shortYearCutoff = this._get(inst, 'shortYearCutoff');
      shortYearCutoff = typeof shortYearCutoff !== 'string' ? shortYearCutoff : new Date().getFullYear() % 100 + parseInt(shortYearCutoff, 10);
      return {
        shortYearCutoff: shortYearCutoff,
        dayNamesShort: this._get(inst, 'dayNamesShort'),
        dayNames: this._get(inst, 'dayNames'),
        monthNamesShort: this._get(inst, 'monthNamesShort'),
        monthNames: this._get(inst, 'monthNames')
      };
    },
    _formatDate: function (inst, day, month, year) {
      if (!day) {
        inst.currentDay = inst.selectedDay;
        inst.currentMonth = inst.selectedMonth;
        inst.currentYear = inst.selectedYear;
      }
      var date = day ? typeof day === 'object' ? day : this._daylightSavingAdjust(new Date(year, month, day)) : this._daylightSavingAdjust(new Date(inst.currentYear, inst.currentMonth, inst.currentDay));
      return this.formatDate(this._get(inst, 'dateFormat'), date, this._getFormatConfig(inst));
    }
  });
  /*
 * Bind hover events for datepicker elements.
 * Done via delegate so the binding only occurs once in the lifetime of the parent div.
 * Global instActive, set by _updateDatepicker allows the handlers to find their way back to the active picker.
 */
  function bindHover(dpDiv) {
    var selector = 'button, .ui-datepicker-prev, .ui-datepicker-next, .ui-datepicker-calendar td a';
    return dpDiv.delegate(selector, 'mouseout', function () {
      $(this).removeClass('ui-state-hover');
      if (this.className.indexOf('ui-datepicker-prev') !== -1) {
        $(this).removeClass('ui-datepicker-prev-hover');
      }
      if (this.className.indexOf('ui-datepicker-next') !== -1) {
        $(this).removeClass('ui-datepicker-next-hover');
      }
    }).delegate(selector, 'mouseover', function () {
      if (!$.datepicker._isDisabledDatepicker(instActive.inline ? dpDiv.parent()[0] : instActive.input[0])) {
        $(this).parents('.ui-datepicker-calendar').find('a').removeClass('ui-state-hover');
        $(this).addClass('ui-state-hover');
        if (this.className.indexOf('ui-datepicker-prev') !== -1) {
          $(this).addClass('ui-datepicker-prev-hover');
        }
        if (this.className.indexOf('ui-datepicker-next') !== -1) {
          $(this).addClass('ui-datepicker-next-hover');
        }
      }
    });
  }
  /* jQuery extend now ignores nulls! */
  function extendRemove(target, props) {
    $.extend(target, props);
    for (var name in props) {
      if (props[name] == null) {
        target[name] = props[name];
      }
    }
    return target;
  }
  /* Invoke the datepicker functionality.
   @param  options  string - a command, optionally followed by additional parameters or
					Object - settings for attaching new datepicker functionality
   @return  jQuery object */
  $.fn.datepicker = function (options) {
    /* Verify an empty collection wasn't passed - Fixes #6976 */
    if (!this.length) {
      return this;
    }
    /* Initialise the date picker. */
    if (!$.datepicker.initialized) {
      $(document).mousedown($.datepicker._checkExternalClick);
      $.datepicker.initialized = true;
    }
    /* Append datepicker main container to body if not exist. */
    if ($('#' + $.datepicker._mainDivId).length === 0) {
      $('body').append($.datepicker.dpDiv);
    }
    var otherArgs = Array.prototype.slice.call(arguments, 1);
    if (typeof options === 'string' && (options === 'isDisabled' || options === 'getDate' || options === 'widget')) {
      return $.datepicker['_' + options + 'Datepicker'].apply($.datepicker, [this[0]].concat(otherArgs));
    }
    if (options === 'option' && arguments.length === 2 && typeof arguments[1] === 'string') {
      return $.datepicker['_' + options + 'Datepicker'].apply($.datepicker, [this[0]].concat(otherArgs));
    }
    return this.each(function () {
      typeof options === 'string' ? $.datepicker['_' + options + 'Datepicker'].apply($.datepicker, [this].concat(otherArgs)) : $.datepicker._attachDatepicker(this, options);
    });
  };
  $.datepicker = new Datepicker();
  // singleton instance
  $.datepicker.initialized = false;
  $.datepicker.uuid = new Date().getTime();
  $.datepicker.version = '1.10.4';
}(jQuery));
(function ($, undefined) {
  var sizeRelatedOptions = {
      buttons: true,
      height: true,
      maxHeight: true,
      maxWidth: true,
      minHeight: true,
      minWidth: true,
      width: true
    }, resizableRelatedOptions = {
      maxHeight: true,
      maxWidth: true,
      minHeight: true,
      minWidth: true
    };
  $.widget('ui.dialog', {
    version: '1.10.4',
    options: {
      appendTo: 'body',
      autoOpen: true,
      buttons: [],
      closeOnEscape: true,
      closeText: 'close',
      dialogClass: '',
      draggable: true,
      hide: null,
      height: 'auto',
      maxHeight: null,
      maxWidth: null,
      minHeight: 150,
      minWidth: 150,
      modal: false,
      position: {
        my: 'center',
        at: 'center',
        of: window,
        collision: 'fit',
        using: function (pos) {
          var topOffset = $(this).css(pos).offset().top;
          if (topOffset < 0) {
            $(this).css('top', pos.top - topOffset);
          }
        }
      },
      resizable: true,
      show: null,
      title: null,
      width: 300,
      beforeClose: null,
      close: null,
      drag: null,
      dragStart: null,
      dragStop: null,
      focus: null,
      open: null,
      resize: null,
      resizeStart: null,
      resizeStop: null
    },
    _create: function () {
      this.originalCss = {
        display: this.element[0].style.display,
        width: this.element[0].style.width,
        minHeight: this.element[0].style.minHeight,
        maxHeight: this.element[0].style.maxHeight,
        height: this.element[0].style.height
      };
      this.originalPosition = {
        parent: this.element.parent(),
        index: this.element.parent().children().index(this.element)
      };
      this.originalTitle = this.element.attr('title');
      this.options.title = this.options.title || this.originalTitle;
      this._createWrapper();
      this.element.show().removeAttr('title').addClass('ui-dialog-content ui-widget-content').appendTo(this.uiDialog);
      this._createTitlebar();
      this._createButtonPane();
      if (this.options.draggable && $.fn.draggable) {
        this._makeDraggable();
      }
      if (this.options.resizable && $.fn.resizable) {
        this._makeResizable();
      }
      this._isOpen = false;
    },
    _init: function () {
      if (this.options.autoOpen) {
        this.open();
      }
    },
    _appendTo: function () {
      var element = this.options.appendTo;
      if (element && (element.jquery || element.nodeType)) {
        return $(element);
      }
      return this.document.find(element || 'body').eq(0);
    },
    _destroy: function () {
      var next, originalPosition = this.originalPosition;
      this._destroyOverlay();
      this.element.removeUniqueId().removeClass('ui-dialog-content ui-widget-content').css(this.originalCss).detach();
      this.uiDialog.stop(true, true).remove();
      if (this.originalTitle) {
        this.element.attr('title', this.originalTitle);
      }
      next = originalPosition.parent.children().eq(originalPosition.index);
      // Don't try to place the dialog next to itself (#8613)
      if (next.length && next[0] !== this.element[0]) {
        next.before(this.element);
      } else {
        originalPosition.parent.append(this.element);
      }
    },
    widget: function () {
      return this.uiDialog;
    },
    disable: $.noop,
    enable: $.noop,
    close: function (event) {
      var activeElement, that = this;
      if (!this._isOpen || this._trigger('beforeClose', event) === false) {
        return;
      }
      this._isOpen = false;
      this._destroyOverlay();
      if (!this.opener.filter(':focusable').focus().length) {
        // support: IE9
        // IE9 throws an "Unspecified error" accessing document.activeElement from an <iframe>
        try {
          activeElement = this.document[0].activeElement;
          // Support: IE9, IE10
          // If the <body> is blurred, IE will switch windows, see #4520
          if (activeElement && activeElement.nodeName.toLowerCase() !== 'body') {
            // Hiding a focused element doesn't trigger blur in WebKit
            // so in case we have nothing to focus on, explicitly blur the active element
            // https://bugs.webkit.org/show_bug.cgi?id=47182
            $(activeElement).blur();
          }
        } catch (error) {
        }
      }
      this._hide(this.uiDialog, this.options.hide, function () {
        that._trigger('close', event);
      });
    },
    isOpen: function () {
      return this._isOpen;
    },
    moveToTop: function () {
      this._moveToTop();
    },
    _moveToTop: function (event, silent) {
      var moved = !!this.uiDialog.nextAll(':visible').insertBefore(this.uiDialog).length;
      if (moved && !silent) {
        this._trigger('focus', event);
      }
      return moved;
    },
    open: function () {
      var that = this;
      if (this._isOpen) {
        if (this._moveToTop()) {
          this._focusTabbable();
        }
        return;
      }
      this._isOpen = true;
      this.opener = $(this.document[0].activeElement);
      this._size();
      this._position();
      this._createOverlay();
      this._moveToTop(null, true);
      this._show(this.uiDialog, this.options.show, function () {
        that._focusTabbable();
        that._trigger('focus');
      });
      this._trigger('open');
    },
    _focusTabbable: function () {
      // Set focus to the first match:
      // 1. First element inside the dialog matching [autofocus]
      // 2. Tabbable element inside the content element
      // 3. Tabbable element inside the buttonpane
      // 4. The close button
      // 5. The dialog itself
      var hasFocus = this.element.find('[autofocus]');
      if (!hasFocus.length) {
        hasFocus = this.element.find(':tabbable');
      }
      if (!hasFocus.length) {
        hasFocus = this.uiDialogButtonPane.find(':tabbable');
      }
      if (!hasFocus.length) {
        hasFocus = this.uiDialogTitlebarClose.filter(':tabbable');
      }
      if (!hasFocus.length) {
        hasFocus = this.uiDialog;
      }
      hasFocus.eq(0).focus();
    },
    _keepFocus: function (event) {
      function checkFocus() {
        var activeElement = this.document[0].activeElement, isActive = this.uiDialog[0] === activeElement || $.contains(this.uiDialog[0], activeElement);
        if (!isActive) {
          this._focusTabbable();
        }
      }
      event.preventDefault();
      checkFocus.call(this);
      // support: IE
      // IE <= 8 doesn't prevent moving focus even with event.preventDefault()
      // so we check again later
      this._delay(checkFocus);
    },
    _createWrapper: function () {
      this.uiDialog = $('<div>').addClass('ui-dialog ui-widget ui-widget-content ui-corner-all ui-front ' + this.options.dialogClass).hide().attr({
        tabIndex: -1,
        role: 'dialog'
      }).appendTo(this._appendTo());
      this._on(this.uiDialog, {
        keydown: function (event) {
          if (this.options.closeOnEscape && !event.isDefaultPrevented() && event.keyCode && event.keyCode === $.ui.keyCode.ESCAPE) {
            event.preventDefault();
            this.close(event);
            return;
          }
          // prevent tabbing out of dialogs
          if (event.keyCode !== $.ui.keyCode.TAB) {
            return;
          }
          var tabbables = this.uiDialog.find(':tabbable'), first = tabbables.filter(':first'), last = tabbables.filter(':last');
          if ((event.target === last[0] || event.target === this.uiDialog[0]) && !event.shiftKey) {
            first.focus(1);
            event.preventDefault();
          } else if ((event.target === first[0] || event.target === this.uiDialog[0]) && event.shiftKey) {
            last.focus(1);
            event.preventDefault();
          }
        },
        mousedown: function (event) {
          if (this._moveToTop(event)) {
            this._focusTabbable();
          }
        }
      });
      // We assume that any existing aria-describedby attribute means
      // that the dialog content is marked up properly
      // otherwise we brute force the content as the description
      if (!this.element.find('[aria-describedby]').length) {
        this.uiDialog.attr({ 'aria-describedby': this.element.uniqueId().attr('id') });
      }
    },
    _createTitlebar: function () {
      var uiDialogTitle;
      this.uiDialogTitlebar = $('<div>').addClass('ui-dialog-titlebar ui-widget-header ui-corner-all ui-helper-clearfix').prependTo(this.uiDialog);
      this._on(this.uiDialogTitlebar, {
        mousedown: function (event) {
          // Don't prevent click on close button (#8838)
          // Focusing a dialog that is partially scrolled out of view
          // causes the browser to scroll it into view, preventing the click event
          if (!$(event.target).closest('.ui-dialog-titlebar-close')) {
            // Dialog isn't getting focus when dragging (#8063)
            this.uiDialog.focus();
          }
        }
      });
      // support: IE
      // Use type="button" to prevent enter keypresses in textboxes from closing the
      // dialog in IE (#9312)
      this.uiDialogTitlebarClose = $('<button type=\'button\'></button>').button({
        label: this.options.closeText,
        icons: { primary: 'ui-icon-closethick' },
        text: false
      }).addClass('ui-dialog-titlebar-close').appendTo(this.uiDialogTitlebar);
      this._on(this.uiDialogTitlebarClose, {
        click: function (event) {
          event.preventDefault();
          this.close(event);
        }
      });
      uiDialogTitle = $('<span>').uniqueId().addClass('ui-dialog-title').prependTo(this.uiDialogTitlebar);
      this._title(uiDialogTitle);
      this.uiDialog.attr({ 'aria-labelledby': uiDialogTitle.attr('id') });
    },
    _title: function (title) {
      if (!this.options.title) {
        title.html('&#160;');
      }
      title.text(this.options.title);
    },
    _createButtonPane: function () {
      this.uiDialogButtonPane = $('<div>').addClass('ui-dialog-buttonpane ui-widget-content ui-helper-clearfix');
      this.uiButtonSet = $('<div>').addClass('ui-dialog-buttonset').appendTo(this.uiDialogButtonPane);
      this._createButtons();
    },
    _createButtons: function () {
      var that = this, buttons = this.options.buttons;
      // if we already have a button pane, remove it
      this.uiDialogButtonPane.remove();
      this.uiButtonSet.empty();
      if ($.isEmptyObject(buttons) || $.isArray(buttons) && !buttons.length) {
        this.uiDialog.removeClass('ui-dialog-buttons');
        return;
      }
      $.each(buttons, function (name, props) {
        var click, buttonOptions;
        props = $.isFunction(props) ? {
          click: props,
          text: name
        } : props;
        // Default to a non-submitting button
        props = $.extend({ type: 'button' }, props);
        // Change the context for the click callback to be the main element
        click = props.click;
        props.click = function () {
          click.apply(that.element[0], arguments);
        };
        buttonOptions = {
          icons: props.icons,
          text: props.showText
        };
        delete props.icons;
        delete props.showText;
        $('<button></button>', props).button(buttonOptions).appendTo(that.uiButtonSet);
      });
      this.uiDialog.addClass('ui-dialog-buttons');
      this.uiDialogButtonPane.appendTo(this.uiDialog);
    },
    _makeDraggable: function () {
      var that = this, options = this.options;
      function filteredUi(ui) {
        return {
          position: ui.position,
          offset: ui.offset
        };
      }
      this.uiDialog.draggable({
        cancel: '.ui-dialog-content, .ui-dialog-titlebar-close',
        handle: '.ui-dialog-titlebar',
        containment: 'document',
        start: function (event, ui) {
          $(this).addClass('ui-dialog-dragging');
          that._blockFrames();
          that._trigger('dragStart', event, filteredUi(ui));
        },
        drag: function (event, ui) {
          that._trigger('drag', event, filteredUi(ui));
        },
        stop: function (event, ui) {
          options.position = [
            ui.position.left - that.document.scrollLeft(),
            ui.position.top - that.document.scrollTop()
          ];
          $(this).removeClass('ui-dialog-dragging');
          that._unblockFrames();
          that._trigger('dragStop', event, filteredUi(ui));
        }
      });
    },
    _makeResizable: function () {
      var that = this, options = this.options, handles = options.resizable,
        // .ui-resizable has position: relative defined in the stylesheet
        // but dialogs have to use absolute or fixed positioning
        position = this.uiDialog.css('position'), resizeHandles = typeof handles === 'string' ? handles : 'n,e,s,w,se,sw,ne,nw';
      function filteredUi(ui) {
        return {
          originalPosition: ui.originalPosition,
          originalSize: ui.originalSize,
          position: ui.position,
          size: ui.size
        };
      }
      this.uiDialog.resizable({
        cancel: '.ui-dialog-content',
        containment: 'document',
        alsoResize: this.element,
        maxWidth: options.maxWidth,
        maxHeight: options.maxHeight,
        minWidth: options.minWidth,
        minHeight: this._minHeight(),
        handles: resizeHandles,
        start: function (event, ui) {
          $(this).addClass('ui-dialog-resizing');
          that._blockFrames();
          that._trigger('resizeStart', event, filteredUi(ui));
        },
        resize: function (event, ui) {
          that._trigger('resize', event, filteredUi(ui));
        },
        stop: function (event, ui) {
          options.height = $(this).height();
          options.width = $(this).width();
          $(this).removeClass('ui-dialog-resizing');
          that._unblockFrames();
          that._trigger('resizeStop', event, filteredUi(ui));
        }
      }).css('position', position);
    },
    _minHeight: function () {
      var options = this.options;
      return options.height === 'auto' ? options.minHeight : Math.min(options.minHeight, options.height);
    },
    _position: function () {
      // Need to show the dialog to get the actual offset in the position plugin
      var isVisible = this.uiDialog.is(':visible');
      if (!isVisible) {
        this.uiDialog.show();
      }
      this.uiDialog.position(this.options.position);
      if (!isVisible) {
        this.uiDialog.hide();
      }
    },
    _setOptions: function (options) {
      var that = this, resize = false, resizableOptions = {};
      $.each(options, function (key, value) {
        that._setOption(key, value);
        if (key in sizeRelatedOptions) {
          resize = true;
        }
        if (key in resizableRelatedOptions) {
          resizableOptions[key] = value;
        }
      });
      if (resize) {
        this._size();
        this._position();
      }
      if (this.uiDialog.is(':data(ui-resizable)')) {
        this.uiDialog.resizable('option', resizableOptions);
      }
    },
    _setOption: function (key, value) {
      var isDraggable, isResizable, uiDialog = this.uiDialog;
      if (key === 'dialogClass') {
        uiDialog.removeClass(this.options.dialogClass).addClass(value);
      }
      if (key === 'disabled') {
        return;
      }
      this._super(key, value);
      if (key === 'appendTo') {
        this.uiDialog.appendTo(this._appendTo());
      }
      if (key === 'buttons') {
        this._createButtons();
      }
      if (key === 'closeText') {
        this.uiDialogTitlebarClose.button({ label: '' + value });
      }
      if (key === 'draggable') {
        isDraggable = uiDialog.is(':data(ui-draggable)');
        if (isDraggable && !value) {
          uiDialog.draggable('destroy');
        }
        if (!isDraggable && value) {
          this._makeDraggable();
        }
      }
      if (key === 'position') {
        this._position();
      }
      if (key === 'resizable') {
        // currently resizable, becoming non-resizable
        isResizable = uiDialog.is(':data(ui-resizable)');
        if (isResizable && !value) {
          uiDialog.resizable('destroy');
        }
        // currently resizable, changing handles
        if (isResizable && typeof value === 'string') {
          uiDialog.resizable('option', 'handles', value);
        }
        // currently non-resizable, becoming resizable
        if (!isResizable && value !== false) {
          this._makeResizable();
        }
      }
      if (key === 'title') {
        this._title(this.uiDialogTitlebar.find('.ui-dialog-title'));
      }
    },
    _size: function () {
      // If the user has resized the dialog, the .ui-dialog and .ui-dialog-content
      // divs will both have width and height set, so we need to reset them
      var nonContentHeight, minContentHeight, maxContentHeight, options = this.options;
      // Reset content sizing
      this.element.show().css({
        width: 'auto',
        minHeight: 0,
        maxHeight: 'none',
        height: 0
      });
      if (options.minWidth > options.width) {
        options.width = options.minWidth;
      }
      // reset wrapper sizing
      // determine the height of all the non-content elements
      nonContentHeight = this.uiDialog.css({
        height: 'auto',
        width: options.width
      }).outerHeight();
      minContentHeight = Math.max(0, options.minHeight - nonContentHeight);
      maxContentHeight = typeof options.maxHeight === 'number' ? Math.max(0, options.maxHeight - nonContentHeight) : 'none';
      if (options.height === 'auto') {
        this.element.css({
          minHeight: minContentHeight,
          maxHeight: maxContentHeight,
          height: 'auto'
        });
      } else {
        this.element.height(Math.max(0, options.height - nonContentHeight));
      }
      if (this.uiDialog.is(':data(ui-resizable)')) {
        this.uiDialog.resizable('option', 'minHeight', this._minHeight());
      }
    },
    _blockFrames: function () {
      this.iframeBlocks = this.document.find('iframe').map(function () {
        var iframe = $(this);
        return $('<div>').css({
          position: 'absolute',
          width: iframe.outerWidth(),
          height: iframe.outerHeight()
        }).appendTo(iframe.parent()).offset(iframe.offset())[0];
      });
    },
    _unblockFrames: function () {
      if (this.iframeBlocks) {
        this.iframeBlocks.remove();
        delete this.iframeBlocks;
      }
    },
    _allowInteraction: function (event) {
      if ($(event.target).closest('.ui-dialog').length) {
        return true;
      }
      // TODO: Remove hack when datepicker implements
      // the .ui-front logic (#8989)
      return !!$(event.target).closest('.ui-datepicker').length;
    },
    _createOverlay: function () {
      if (!this.options.modal) {
        return;
      }
      var that = this, widgetFullName = this.widgetFullName;
      if (!$.ui.dialog.overlayInstances) {
        // Prevent use of anchors and inputs.
        // We use a delay in case the overlay is created from an
        // event that we're going to be cancelling. (#2804)
        this._delay(function () {
          // Handle .dialog().dialog("close") (#4065)
          if ($.ui.dialog.overlayInstances) {
            this.document.bind('focusin.dialog', function (event) {
              if (!that._allowInteraction(event)) {
                event.preventDefault();
                $('.ui-dialog:visible:last .ui-dialog-content').data(widgetFullName)._focusTabbable();
              }
            });
          }
        });
      }
      this.overlay = $('<div>').addClass('ui-widget-overlay ui-front').appendTo(this._appendTo());
      this._on(this.overlay, { mousedown: '_keepFocus' });
      $.ui.dialog.overlayInstances++;
    },
    _destroyOverlay: function () {
      if (!this.options.modal) {
        return;
      }
      if (this.overlay) {
        $.ui.dialog.overlayInstances--;
        if (!$.ui.dialog.overlayInstances) {
          this.document.unbind('focusin.dialog');
        }
        this.overlay.remove();
        this.overlay = null;
      }
    }
  });
  $.ui.dialog.overlayInstances = 0;
  // DEPRECATED
  if ($.uiBackCompat !== false) {
    // position option with array notation
    // just override with old implementation
    $.widget('ui.dialog', $.ui.dialog, {
      _position: function () {
        var position = this.options.position, myAt = [], offset = [
            0,
            0
          ], isVisible;
        if (position) {
          if (typeof position === 'string' || typeof position === 'object' && '0' in position) {
            myAt = position.split ? position.split(' ') : [
              position[0],
              position[1]
            ];
            if (myAt.length === 1) {
              myAt[1] = myAt[0];
            }
            $.each([
              'left',
              'top'
            ], function (i, offsetPosition) {
              if (+myAt[i] === myAt[i]) {
                offset[i] = myAt[i];
                myAt[i] = offsetPosition;
              }
            });
            position = {
              my: myAt[0] + (offset[0] < 0 ? offset[0] : '+' + offset[0]) + ' ' + myAt[1] + (offset[1] < 0 ? offset[1] : '+' + offset[1]),
              at: myAt.join(' ')
            };
          }
          position = $.extend({}, $.ui.dialog.prototype.options.position, position);
        } else {
          position = $.ui.dialog.prototype.options.position;
        }
        // need to show the dialog to get the actual offset in the position plugin
        isVisible = this.uiDialog.is(':visible');
        if (!isVisible) {
          this.uiDialog.show();
        }
        this.uiDialog.position(position);
        if (!isVisible) {
          this.uiDialog.hide();
        }
      }
    });
  }
}(jQuery));
(function ($, undefined) {
  $.widget('ui.draggable', $.ui.mouse, {
    version: '1.10.4',
    widgetEventPrefix: 'drag',
    options: {
      addClasses: true,
      appendTo: 'parent',
      axis: false,
      connectToSortable: false,
      containment: false,
      cursor: 'auto',
      cursorAt: false,
      grid: false,
      handle: false,
      helper: 'original',
      iframeFix: false,
      opacity: false,
      refreshPositions: false,
      revert: false,
      revertDuration: 500,
      scope: 'default',
      scroll: true,
      scrollSensitivity: 20,
      scrollSpeed: 20,
      snap: false,
      snapMode: 'both',
      snapTolerance: 20,
      stack: false,
      zIndex: false,
      drag: null,
      start: null,
      stop: null
    },
    _create: function () {
      if (this.options.helper === 'original' && !/^(?:r|a|f)/.test(this.element.css('position'))) {
        this.element[0].style.position = 'relative';
      }
      if (this.options.addClasses) {
        this.element.addClass('ui-draggable');
      }
      if (this.options.disabled) {
        this.element.addClass('ui-draggable-disabled');
      }
      this._mouseInit();
    },
    _destroy: function () {
      this.element.removeClass('ui-draggable ui-draggable-dragging ui-draggable-disabled');
      this._mouseDestroy();
    },
    _mouseCapture: function (event) {
      var o = this.options;
      // among others, prevent a drag on a resizable-handle
      if (this.helper || o.disabled || $(event.target).closest('.ui-resizable-handle').length > 0) {
        return false;
      }
      //Quit if we're not on a valid handle
      this.handle = this._getHandle(event);
      if (!this.handle) {
        return false;
      }
      $(o.iframeFix === true ? 'iframe' : o.iframeFix).each(function () {
        $('<div class=\'ui-draggable-iframeFix\' style=\'background: #fff;\'></div>').css({
          width: this.offsetWidth + 'px',
          height: this.offsetHeight + 'px',
          position: 'absolute',
          opacity: '0.001',
          zIndex: 1000
        }).css($(this).offset()).appendTo('body');
      });
      return true;
    },
    _mouseStart: function (event) {
      var o = this.options;
      //Create and append the visible helper
      this.helper = this._createHelper(event);
      this.helper.addClass('ui-draggable-dragging');
      //Cache the helper size
      this._cacheHelperProportions();
      //If ddmanager is used for droppables, set the global draggable
      if ($.ui.ddmanager) {
        $.ui.ddmanager.current = this;
      }
      /*
		 * - Position generation -
		 * This block generates everything position related - it's the core of draggables.
		 */
      //Cache the margins of the original element
      this._cacheMargins();
      //Store the helper's css position
      this.cssPosition = this.helper.css('position');
      this.scrollParent = this.helper.scrollParent();
      this.offsetParent = this.helper.offsetParent();
      this.offsetParentCssPosition = this.offsetParent.css('position');
      //The element's absolute position on the page minus margins
      this.offset = this.positionAbs = this.element.offset();
      this.offset = {
        top: this.offset.top - this.margins.top,
        left: this.offset.left - this.margins.left
      };
      //Reset scroll cache
      this.offset.scroll = false;
      $.extend(this.offset, {
        click: {
          left: event.pageX - this.offset.left,
          top: event.pageY - this.offset.top
        },
        parent: this._getParentOffset(),
        relative: this._getRelativeOffset()
      });
      //Generate the original position
      this.originalPosition = this.position = this._generatePosition(event);
      this.originalPageX = event.pageX;
      this.originalPageY = event.pageY;
      //Adjust the mouse offset relative to the helper if "cursorAt" is supplied
      o.cursorAt && this._adjustOffsetFromHelper(o.cursorAt);
      //Set a containment if given in the options
      this._setContainment();
      //Trigger event + callbacks
      if (this._trigger('start', event) === false) {
        this._clear();
        return false;
      }
      //Recache the helper size
      this._cacheHelperProportions();
      //Prepare the droppable offsets
      if ($.ui.ddmanager && !o.dropBehaviour) {
        $.ui.ddmanager.prepareOffsets(this, event);
      }
      this._mouseDrag(event, true);
      //Execute the drag once - this causes the helper not to be visible before getting its correct position
      //If the ddmanager is used for droppables, inform the manager that dragging has started (see #5003)
      if ($.ui.ddmanager) {
        $.ui.ddmanager.dragStart(this, event);
      }
      return true;
    },
    _mouseDrag: function (event, noPropagation) {
      // reset any necessary cached properties (see #5009)
      if (this.offsetParentCssPosition === 'fixed') {
        this.offset.parent = this._getParentOffset();
      }
      //Compute the helpers position
      this.position = this._generatePosition(event);
      this.positionAbs = this._convertPositionTo('absolute');
      //Call plugins and callbacks and use the resulting position if something is returned
      if (!noPropagation) {
        var ui = this._uiHash();
        if (this._trigger('drag', event, ui) === false) {
          this._mouseUp({});
          return false;
        }
        this.position = ui.position;
      }
      if (!this.options.axis || this.options.axis !== 'y') {
        this.helper[0].style.left = this.position.left + 'px';
      }
      if (!this.options.axis || this.options.axis !== 'x') {
        this.helper[0].style.top = this.position.top + 'px';
      }
      if ($.ui.ddmanager) {
        $.ui.ddmanager.drag(this, event);
      }
      return false;
    },
    _mouseStop: function (event) {
      //If we are using droppables, inform the manager about the drop
      var that = this, dropped = false;
      if ($.ui.ddmanager && !this.options.dropBehaviour) {
        dropped = $.ui.ddmanager.drop(this, event);
      }
      //if a drop comes from outside (a sortable)
      if (this.dropped) {
        dropped = this.dropped;
        this.dropped = false;
      }
      //if the original element is no longer in the DOM don't bother to continue (see #8269)
      if (this.options.helper === 'original' && !$.contains(this.element[0].ownerDocument, this.element[0])) {
        return false;
      }
      if (this.options.revert === 'invalid' && !dropped || this.options.revert === 'valid' && dropped || this.options.revert === true || $.isFunction(this.options.revert) && this.options.revert.call(this.element, dropped)) {
        $(this.helper).animate(this.originalPosition, parseInt(this.options.revertDuration, 10), function () {
          if (that._trigger('stop', event) !== false) {
            that._clear();
          }
        });
      } else {
        if (this._trigger('stop', event) !== false) {
          this._clear();
        }
      }
      return false;
    },
    _mouseUp: function (event) {
      //Remove frame helpers
      $('div.ui-draggable-iframeFix').each(function () {
        this.parentNode.removeChild(this);
      });
      //If the ddmanager is used for droppables, inform the manager that dragging has stopped (see #5003)
      if ($.ui.ddmanager) {
        $.ui.ddmanager.dragStop(this, event);
      }
      return $.ui.mouse.prototype._mouseUp.call(this, event);
    },
    cancel: function () {
      if (this.helper.is('.ui-draggable-dragging')) {
        this._mouseUp({});
      } else {
        this._clear();
      }
      return this;
    },
    _getHandle: function (event) {
      return this.options.handle ? !!$(event.target).closest(this.element.find(this.options.handle)).length : true;
    },
    _createHelper: function (event) {
      var o = this.options, helper = $.isFunction(o.helper) ? $(o.helper.apply(this.element[0], [event])) : o.helper === 'clone' ? this.element.clone().removeAttr('id') : this.element;
      if (!helper.parents('body').length) {
        helper.appendTo(o.appendTo === 'parent' ? this.element[0].parentNode : o.appendTo);
      }
      if (helper[0] !== this.element[0] && !/(fixed|absolute)/.test(helper.css('position'))) {
        helper.css('position', 'absolute');
      }
      return helper;
    },
    _adjustOffsetFromHelper: function (obj) {
      if (typeof obj === 'string') {
        obj = obj.split(' ');
      }
      if ($.isArray(obj)) {
        obj = {
          left: +obj[0],
          top: +obj[1] || 0
        };
      }
      if ('left' in obj) {
        this.offset.click.left = obj.left + this.margins.left;
      }
      if ('right' in obj) {
        this.offset.click.left = this.helperProportions.width - obj.right + this.margins.left;
      }
      if ('top' in obj) {
        this.offset.click.top = obj.top + this.margins.top;
      }
      if ('bottom' in obj) {
        this.offset.click.top = this.helperProportions.height - obj.bottom + this.margins.top;
      }
    },
    _getParentOffset: function () {
      //Get the offsetParent and cache its position
      var po = this.offsetParent.offset();
      // This is a special case where we need to modify a offset calculated on start, since the following happened:
      // 1. The position of the helper is absolute, so it's position is calculated based on the next positioned parent
      // 2. The actual offset parent is a child of the scroll parent, and the scroll parent isn't the document, which means that
      //    the scroll is included in the initial calculation of the offset of the parent, and never recalculated upon drag
      if (this.cssPosition === 'absolute' && this.scrollParent[0] !== document && $.contains(this.scrollParent[0], this.offsetParent[0])) {
        po.left += this.scrollParent.scrollLeft();
        po.top += this.scrollParent.scrollTop();
      }
      //This needs to be actually done for all browsers, since pageX/pageY includes this information
      //Ugly IE fix
      if (this.offsetParent[0] === document.body || this.offsetParent[0].tagName && this.offsetParent[0].tagName.toLowerCase() === 'html' && $.ui.ie) {
        po = {
          top: 0,
          left: 0
        };
      }
      return {
        top: po.top + (parseInt(this.offsetParent.css('borderTopWidth'), 10) || 0),
        left: po.left + (parseInt(this.offsetParent.css('borderLeftWidth'), 10) || 0)
      };
    },
    _getRelativeOffset: function () {
      if (this.cssPosition === 'relative') {
        var p = this.element.position();
        return {
          top: p.top - (parseInt(this.helper.css('top'), 10) || 0) + this.scrollParent.scrollTop(),
          left: p.left - (parseInt(this.helper.css('left'), 10) || 0) + this.scrollParent.scrollLeft()
        };
      } else {
        return {
          top: 0,
          left: 0
        };
      }
    },
    _cacheMargins: function () {
      this.margins = {
        left: parseInt(this.element.css('marginLeft'), 10) || 0,
        top: parseInt(this.element.css('marginTop'), 10) || 0,
        right: parseInt(this.element.css('marginRight'), 10) || 0,
        bottom: parseInt(this.element.css('marginBottom'), 10) || 0
      };
    },
    _cacheHelperProportions: function () {
      this.helperProportions = {
        width: this.helper.outerWidth(),
        height: this.helper.outerHeight()
      };
    },
    _setContainment: function () {
      var over, c, ce, o = this.options;
      if (!o.containment) {
        this.containment = null;
        return;
      }
      if (o.containment === 'window') {
        this.containment = [
          $(window).scrollLeft() - this.offset.relative.left - this.offset.parent.left,
          $(window).scrollTop() - this.offset.relative.top - this.offset.parent.top,
          $(window).scrollLeft() + $(window).width() - this.helperProportions.width - this.margins.left,
          $(window).scrollTop() + ($(window).height() || document.body.parentNode.scrollHeight) - this.helperProportions.height - this.margins.top
        ];
        return;
      }
      if (o.containment === 'document') {
        this.containment = [
          0,
          0,
          $(document).width() - this.helperProportions.width - this.margins.left,
          ($(document).height() || document.body.parentNode.scrollHeight) - this.helperProportions.height - this.margins.top
        ];
        return;
      }
      if (o.containment.constructor === Array) {
        this.containment = o.containment;
        return;
      }
      if (o.containment === 'parent') {
        o.containment = this.helper[0].parentNode;
      }
      c = $(o.containment);
      ce = c[0];
      if (!ce) {
        return;
      }
      over = c.css('overflow') !== 'hidden';
      this.containment = [
        (parseInt(c.css('borderLeftWidth'), 10) || 0) + (parseInt(c.css('paddingLeft'), 10) || 0),
        (parseInt(c.css('borderTopWidth'), 10) || 0) + (parseInt(c.css('paddingTop'), 10) || 0),
        (over ? Math.max(ce.scrollWidth, ce.offsetWidth) : ce.offsetWidth) - (parseInt(c.css('borderRightWidth'), 10) || 0) - (parseInt(c.css('paddingRight'), 10) || 0) - this.helperProportions.width - this.margins.left - this.margins.right,
        (over ? Math.max(ce.scrollHeight, ce.offsetHeight) : ce.offsetHeight) - (parseInt(c.css('borderBottomWidth'), 10) || 0) - (parseInt(c.css('paddingBottom'), 10) || 0) - this.helperProportions.height - this.margins.top - this.margins.bottom
      ];
      this.relative_container = c;
    },
    _convertPositionTo: function (d, pos) {
      if (!pos) {
        pos = this.position;
      }
      var mod = d === 'absolute' ? 1 : -1, scroll = this.cssPosition === 'absolute' && !(this.scrollParent[0] !== document && $.contains(this.scrollParent[0], this.offsetParent[0])) ? this.offsetParent : this.scrollParent;
      //Cache the scroll
      if (!this.offset.scroll) {
        this.offset.scroll = {
          top: scroll.scrollTop(),
          left: scroll.scrollLeft()
        };
      }
      return {
        top: pos.top + this.offset.relative.top * mod + this.offset.parent.top * mod - (this.cssPosition === 'fixed' ? -this.scrollParent.scrollTop() : this.offset.scroll.top) * mod,
        left: pos.left + this.offset.relative.left * mod + this.offset.parent.left * mod - (this.cssPosition === 'fixed' ? -this.scrollParent.scrollLeft() : this.offset.scroll.left) * mod
      };
    },
    _generatePosition: function (event) {
      var containment, co, top, left, o = this.options, scroll = this.cssPosition === 'absolute' && !(this.scrollParent[0] !== document && $.contains(this.scrollParent[0], this.offsetParent[0])) ? this.offsetParent : this.scrollParent, pageX = event.pageX, pageY = event.pageY;
      //Cache the scroll
      if (!this.offset.scroll) {
        this.offset.scroll = {
          top: scroll.scrollTop(),
          left: scroll.scrollLeft()
        };
      }
      /*
		 * - Position constraining -
		 * Constrain the position to a mix of grid, containment.
		 */
      // If we are not dragging yet, we won't check for options
      if (this.originalPosition) {
        if (this.containment) {
          if (this.relative_container) {
            co = this.relative_container.offset();
            containment = [
              this.containment[0] + co.left,
              this.containment[1] + co.top,
              this.containment[2] + co.left,
              this.containment[3] + co.top
            ];
          } else {
            containment = this.containment;
          }
          if (event.pageX - this.offset.click.left < containment[0]) {
            pageX = containment[0] + this.offset.click.left;
          }
          if (event.pageY - this.offset.click.top < containment[1]) {
            pageY = containment[1] + this.offset.click.top;
          }
          if (event.pageX - this.offset.click.left > containment[2]) {
            pageX = containment[2] + this.offset.click.left;
          }
          if (event.pageY - this.offset.click.top > containment[3]) {
            pageY = containment[3] + this.offset.click.top;
          }
        }
        if (o.grid) {
          //Check for grid elements set to 0 to prevent divide by 0 error causing invalid argument errors in IE (see ticket #6950)
          top = o.grid[1] ? this.originalPageY + Math.round((pageY - this.originalPageY) / o.grid[1]) * o.grid[1] : this.originalPageY;
          pageY = containment ? top - this.offset.click.top >= containment[1] || top - this.offset.click.top > containment[3] ? top : top - this.offset.click.top >= containment[1] ? top - o.grid[1] : top + o.grid[1] : top;
          left = o.grid[0] ? this.originalPageX + Math.round((pageX - this.originalPageX) / o.grid[0]) * o.grid[0] : this.originalPageX;
          pageX = containment ? left - this.offset.click.left >= containment[0] || left - this.offset.click.left > containment[2] ? left : left - this.offset.click.left >= containment[0] ? left - o.grid[0] : left + o.grid[0] : left;
        }
      }
      return {
        top: pageY - this.offset.click.top - this.offset.relative.top - this.offset.parent.top + (this.cssPosition === 'fixed' ? -this.scrollParent.scrollTop() : this.offset.scroll.top),
        left: pageX - this.offset.click.left - this.offset.relative.left - this.offset.parent.left + (this.cssPosition === 'fixed' ? -this.scrollParent.scrollLeft() : this.offset.scroll.left)
      };
    },
    _clear: function () {
      this.helper.removeClass('ui-draggable-dragging');
      if (this.helper[0] !== this.element[0] && !this.cancelHelperRemoval) {
        this.helper.remove();
      }
      this.helper = null;
      this.cancelHelperRemoval = false;
    },
    _trigger: function (type, event, ui) {
      ui = ui || this._uiHash();
      $.ui.plugin.call(this, type, [
        event,
        ui
      ]);
      //The absolute position has to be recalculated after plugins
      if (type === 'drag') {
        this.positionAbs = this._convertPositionTo('absolute');
      }
      return $.Widget.prototype._trigger.call(this, type, event, ui);
    },
    plugins: {},
    _uiHash: function () {
      return {
        helper: this.helper,
        position: this.position,
        originalPosition: this.originalPosition,
        offset: this.positionAbs
      };
    }
  });
  $.ui.plugin.add('draggable', 'connectToSortable', {
    start: function (event, ui) {
      var inst = $(this).data('ui-draggable'), o = inst.options, uiSortable = $.extend({}, ui, { item: inst.element });
      inst.sortables = [];
      $(o.connectToSortable).each(function () {
        var sortable = $.data(this, 'ui-sortable');
        if (sortable && !sortable.options.disabled) {
          inst.sortables.push({
            instance: sortable,
            shouldRevert: sortable.options.revert
          });
          sortable.refreshPositions();
          // Call the sortable's refreshPositions at drag start to refresh the containerCache since the sortable container cache is used in drag and needs to be up to date (this will ensure it's initialised as well as being kept in step with any changes that might have happened on the page).
          sortable._trigger('activate', event, uiSortable);
        }
      });
    },
    stop: function (event, ui) {
      //If we are still over the sortable, we fake the stop event of the sortable, but also remove helper
      var inst = $(this).data('ui-draggable'), uiSortable = $.extend({}, ui, { item: inst.element });
      $.each(inst.sortables, function () {
        if (this.instance.isOver) {
          this.instance.isOver = 0;
          inst.cancelHelperRemoval = true;
          //Don't remove the helper in the draggable instance
          this.instance.cancelHelperRemoval = false;
          //Remove it in the sortable instance (so sortable plugins like revert still work)
          //The sortable revert is supported, and we have to set a temporary dropped variable on the draggable to support revert: "valid/invalid"
          if (this.shouldRevert) {
            this.instance.options.revert = this.shouldRevert;
          }
          //Trigger the stop of the sortable
          this.instance._mouseStop(event);
          this.instance.options.helper = this.instance.options._helper;
          //If the helper has been the original item, restore properties in the sortable
          if (inst.options.helper === 'original') {
            this.instance.currentItem.css({
              top: 'auto',
              left: 'auto'
            });
          }
        } else {
          this.instance.cancelHelperRemoval = false;
          //Remove the helper in the sortable instance
          this.instance._trigger('deactivate', event, uiSortable);
        }
      });
    },
    drag: function (event, ui) {
      var inst = $(this).data('ui-draggable'), that = this;
      $.each(inst.sortables, function () {
        var innermostIntersecting = false, thisSortable = this;
        //Copy over some variables to allow calling the sortable's native _intersectsWith
        this.instance.positionAbs = inst.positionAbs;
        this.instance.helperProportions = inst.helperProportions;
        this.instance.offset.click = inst.offset.click;
        if (this.instance._intersectsWith(this.instance.containerCache)) {
          innermostIntersecting = true;
          $.each(inst.sortables, function () {
            this.instance.positionAbs = inst.positionAbs;
            this.instance.helperProportions = inst.helperProportions;
            this.instance.offset.click = inst.offset.click;
            if (this !== thisSortable && this.instance._intersectsWith(this.instance.containerCache) && $.contains(thisSortable.instance.element[0], this.instance.element[0])) {
              innermostIntersecting = false;
            }
            return innermostIntersecting;
          });
        }
        if (innermostIntersecting) {
          //If it intersects, we use a little isOver variable and set it once, so our move-in stuff gets fired only once
          if (!this.instance.isOver) {
            this.instance.isOver = 1;
            //Now we fake the start of dragging for the sortable instance,
            //by cloning the list group item, appending it to the sortable and using it as inst.currentItem
            //We can then fire the start event of the sortable with our passed browser event, and our own helper (so it doesn't create a new one)
            this.instance.currentItem = $(that).clone().removeAttr('id').appendTo(this.instance.element).data('ui-sortable-item', true);
            this.instance.options._helper = this.instance.options.helper;
            //Store helper option to later restore it
            this.instance.options.helper = function () {
              return ui.helper[0];
            };
            event.target = this.instance.currentItem[0];
            this.instance._mouseCapture(event, true);
            this.instance._mouseStart(event, true, true);
            //Because the browser event is way off the new appended portlet, we modify a couple of variables to reflect the changes
            this.instance.offset.click.top = inst.offset.click.top;
            this.instance.offset.click.left = inst.offset.click.left;
            this.instance.offset.parent.left -= inst.offset.parent.left - this.instance.offset.parent.left;
            this.instance.offset.parent.top -= inst.offset.parent.top - this.instance.offset.parent.top;
            inst._trigger('toSortable', event);
            inst.dropped = this.instance.element;
            //draggable revert needs that
            //hack so receive/update callbacks work (mostly)
            inst.currentItem = inst.element;
            this.instance.fromOutside = inst;
          }
          //Provided we did all the previous steps, we can fire the drag event of the sortable on every draggable drag, when it intersects with the sortable
          if (this.instance.currentItem) {
            this.instance._mouseDrag(event);
          }
        } else {
          //If it doesn't intersect with the sortable, and it intersected before,
          //we fake the drag stop of the sortable, but make sure it doesn't remove the helper by using cancelHelperRemoval
          if (this.instance.isOver) {
            this.instance.isOver = 0;
            this.instance.cancelHelperRemoval = true;
            //Prevent reverting on this forced stop
            this.instance.options.revert = false;
            // The out event needs to be triggered independently
            this.instance._trigger('out', event, this.instance._uiHash(this.instance));
            this.instance._mouseStop(event, true);
            this.instance.options.helper = this.instance.options._helper;
            //Now we remove our currentItem, the list group clone again, and the placeholder, and animate the helper back to it's original size
            this.instance.currentItem.remove();
            if (this.instance.placeholder) {
              this.instance.placeholder.remove();
            }
            inst._trigger('fromSortable', event);
            inst.dropped = false;  //draggable revert needs that
          }
        }
      });
    }
  });
  $.ui.plugin.add('draggable', 'cursor', {
    start: function () {
      var t = $('body'), o = $(this).data('ui-draggable').options;
      if (t.css('cursor')) {
        o._cursor = t.css('cursor');
      }
      t.css('cursor', o.cursor);
    },
    stop: function () {
      var o = $(this).data('ui-draggable').options;
      if (o._cursor) {
        $('body').css('cursor', o._cursor);
      }
    }
  });
  $.ui.plugin.add('draggable', 'opacity', {
    start: function (event, ui) {
      var t = $(ui.helper), o = $(this).data('ui-draggable').options;
      if (t.css('opacity')) {
        o._opacity = t.css('opacity');
      }
      t.css('opacity', o.opacity);
    },
    stop: function (event, ui) {
      var o = $(this).data('ui-draggable').options;
      if (o._opacity) {
        $(ui.helper).css('opacity', o._opacity);
      }
    }
  });
  $.ui.plugin.add('draggable', 'scroll', {
    start: function () {
      var i = $(this).data('ui-draggable');
      if (i.scrollParent[0] !== document && i.scrollParent[0].tagName !== 'HTML') {
        i.overflowOffset = i.scrollParent.offset();
      }
    },
    drag: function (event) {
      var i = $(this).data('ui-draggable'), o = i.options, scrolled = false;
      if (i.scrollParent[0] !== document && i.scrollParent[0].tagName !== 'HTML') {
        if (!o.axis || o.axis !== 'x') {
          if (i.overflowOffset.top + i.scrollParent[0].offsetHeight - event.pageY < o.scrollSensitivity) {
            i.scrollParent[0].scrollTop = scrolled = i.scrollParent[0].scrollTop + o.scrollSpeed;
          } else if (event.pageY - i.overflowOffset.top < o.scrollSensitivity) {
            i.scrollParent[0].scrollTop = scrolled = i.scrollParent[0].scrollTop - o.scrollSpeed;
          }
        }
        if (!o.axis || o.axis !== 'y') {
          if (i.overflowOffset.left + i.scrollParent[0].offsetWidth - event.pageX < o.scrollSensitivity) {
            i.scrollParent[0].scrollLeft = scrolled = i.scrollParent[0].scrollLeft + o.scrollSpeed;
          } else if (event.pageX - i.overflowOffset.left < o.scrollSensitivity) {
            i.scrollParent[0].scrollLeft = scrolled = i.scrollParent[0].scrollLeft - o.scrollSpeed;
          }
        }
      } else {
        if (!o.axis || o.axis !== 'x') {
          if (event.pageY - $(document).scrollTop() < o.scrollSensitivity) {
            scrolled = $(document).scrollTop($(document).scrollTop() - o.scrollSpeed);
          } else if ($(window).height() - (event.pageY - $(document).scrollTop()) < o.scrollSensitivity) {
            scrolled = $(document).scrollTop($(document).scrollTop() + o.scrollSpeed);
          }
        }
        if (!o.axis || o.axis !== 'y') {
          if (event.pageX - $(document).scrollLeft() < o.scrollSensitivity) {
            scrolled = $(document).scrollLeft($(document).scrollLeft() - o.scrollSpeed);
          } else if ($(window).width() - (event.pageX - $(document).scrollLeft()) < o.scrollSensitivity) {
            scrolled = $(document).scrollLeft($(document).scrollLeft() + o.scrollSpeed);
          }
        }
      }
      if (scrolled !== false && $.ui.ddmanager && !o.dropBehaviour) {
        $.ui.ddmanager.prepareOffsets(i, event);
      }
    }
  });
  $.ui.plugin.add('draggable', 'snap', {
    start: function () {
      var i = $(this).data('ui-draggable'), o = i.options;
      i.snapElements = [];
      $(o.snap.constructor !== String ? o.snap.items || ':data(ui-draggable)' : o.snap).each(function () {
        var $t = $(this), $o = $t.offset();
        if (this !== i.element[0]) {
          i.snapElements.push({
            item: this,
            width: $t.outerWidth(),
            height: $t.outerHeight(),
            top: $o.top,
            left: $o.left
          });
        }
      });
    },
    drag: function (event, ui) {
      var ts, bs, ls, rs, l, r, t, b, i, first, inst = $(this).data('ui-draggable'), o = inst.options, d = o.snapTolerance, x1 = ui.offset.left, x2 = x1 + inst.helperProportions.width, y1 = ui.offset.top, y2 = y1 + inst.helperProportions.height;
      for (i = inst.snapElements.length - 1; i >= 0; i--) {
        l = inst.snapElements[i].left;
        r = l + inst.snapElements[i].width;
        t = inst.snapElements[i].top;
        b = t + inst.snapElements[i].height;
        if (x2 < l - d || x1 > r + d || y2 < t - d || y1 > b + d || !$.contains(inst.snapElements[i].item.ownerDocument, inst.snapElements[i].item)) {
          if (inst.snapElements[i].snapping) {
            inst.options.snap.release && inst.options.snap.release.call(inst.element, event, $.extend(inst._uiHash(), { snapItem: inst.snapElements[i].item }));
          }
          inst.snapElements[i].snapping = false;
          continue;
        }
        if (o.snapMode !== 'inner') {
          ts = Math.abs(t - y2) <= d;
          bs = Math.abs(b - y1) <= d;
          ls = Math.abs(l - x2) <= d;
          rs = Math.abs(r - x1) <= d;
          if (ts) {
            ui.position.top = inst._convertPositionTo('relative', {
              top: t - inst.helperProportions.height,
              left: 0
            }).top - inst.margins.top;
          }
          if (bs) {
            ui.position.top = inst._convertPositionTo('relative', {
              top: b,
              left: 0
            }).top - inst.margins.top;
          }
          if (ls) {
            ui.position.left = inst._convertPositionTo('relative', {
              top: 0,
              left: l - inst.helperProportions.width
            }).left - inst.margins.left;
          }
          if (rs) {
            ui.position.left = inst._convertPositionTo('relative', {
              top: 0,
              left: r
            }).left - inst.margins.left;
          }
        }
        first = ts || bs || ls || rs;
        if (o.snapMode !== 'outer') {
          ts = Math.abs(t - y1) <= d;
          bs = Math.abs(b - y2) <= d;
          ls = Math.abs(l - x1) <= d;
          rs = Math.abs(r - x2) <= d;
          if (ts) {
            ui.position.top = inst._convertPositionTo('relative', {
              top: t,
              left: 0
            }).top - inst.margins.top;
          }
          if (bs) {
            ui.position.top = inst._convertPositionTo('relative', {
              top: b - inst.helperProportions.height,
              left: 0
            }).top - inst.margins.top;
          }
          if (ls) {
            ui.position.left = inst._convertPositionTo('relative', {
              top: 0,
              left: l
            }).left - inst.margins.left;
          }
          if (rs) {
            ui.position.left = inst._convertPositionTo('relative', {
              top: 0,
              left: r - inst.helperProportions.width
            }).left - inst.margins.left;
          }
        }
        if (!inst.snapElements[i].snapping && (ts || bs || ls || rs || first)) {
          inst.options.snap.snap && inst.options.snap.snap.call(inst.element, event, $.extend(inst._uiHash(), { snapItem: inst.snapElements[i].item }));
        }
        inst.snapElements[i].snapping = ts || bs || ls || rs || first;
      }
    }
  });
  $.ui.plugin.add('draggable', 'stack', {
    start: function () {
      var min, o = this.data('ui-draggable').options, group = $.makeArray($(o.stack)).sort(function (a, b) {
          return (parseInt($(a).css('zIndex'), 10) || 0) - (parseInt($(b).css('zIndex'), 10) || 0);
        });
      if (!group.length) {
        return;
      }
      min = parseInt($(group[0]).css('zIndex'), 10) || 0;
      $(group).each(function (i) {
        $(this).css('zIndex', min + i);
      });
      this.css('zIndex', min + group.length);
    }
  });
  $.ui.plugin.add('draggable', 'zIndex', {
    start: function (event, ui) {
      var t = $(ui.helper), o = $(this).data('ui-draggable').options;
      if (t.css('zIndex')) {
        o._zIndex = t.css('zIndex');
      }
      t.css('zIndex', o.zIndex);
    },
    stop: function (event, ui) {
      var o = $(this).data('ui-draggable').options;
      if (o._zIndex) {
        $(ui.helper).css('zIndex', o._zIndex);
      }
    }
  });
}(jQuery));
(function ($, undefined) {
  function isOverAxis(x, reference, size) {
    return x > reference && x < reference + size;
  }
  $.widget('ui.droppable', {
    version: '1.10.4',
    widgetEventPrefix: 'drop',
    options: {
      accept: '*',
      activeClass: false,
      addClasses: true,
      greedy: false,
      hoverClass: false,
      scope: 'default',
      tolerance: 'intersect',
      activate: null,
      deactivate: null,
      drop: null,
      out: null,
      over: null
    },
    _create: function () {
      var proportions, o = this.options, accept = o.accept;
      this.isover = false;
      this.isout = true;
      this.accept = $.isFunction(accept) ? accept : function (d) {
        return d.is(accept);
      };
      this.proportions = function () {
        if (arguments.length) {
          // Store the droppable's proportions
          proportions = arguments[0];
        } else {
          // Retrieve or derive the droppable's proportions
          return proportions ? proportions : proportions = {
            width: this.element[0].offsetWidth,
            height: this.element[0].offsetHeight
          };
        }
      };
      // Add the reference and positions to the manager
      $.ui.ddmanager.droppables[o.scope] = $.ui.ddmanager.droppables[o.scope] || [];
      $.ui.ddmanager.droppables[o.scope].push(this);
      o.addClasses && this.element.addClass('ui-droppable');
    },
    _destroy: function () {
      var i = 0, drop = $.ui.ddmanager.droppables[this.options.scope];
      for (; i < drop.length; i++) {
        if (drop[i] === this) {
          drop.splice(i, 1);
        }
      }
      this.element.removeClass('ui-droppable ui-droppable-disabled');
    },
    _setOption: function (key, value) {
      if (key === 'accept') {
        this.accept = $.isFunction(value) ? value : function (d) {
          return d.is(value);
        };
      }
      $.Widget.prototype._setOption.apply(this, arguments);
    },
    _activate: function (event) {
      var draggable = $.ui.ddmanager.current;
      if (this.options.activeClass) {
        this.element.addClass(this.options.activeClass);
      }
      if (draggable) {
        this._trigger('activate', event, this.ui(draggable));
      }
    },
    _deactivate: function (event) {
      var draggable = $.ui.ddmanager.current;
      if (this.options.activeClass) {
        this.element.removeClass(this.options.activeClass);
      }
      if (draggable) {
        this._trigger('deactivate', event, this.ui(draggable));
      }
    },
    _over: function (event) {
      var draggable = $.ui.ddmanager.current;
      // Bail if draggable and droppable are same element
      if (!draggable || (draggable.currentItem || draggable.element)[0] === this.element[0]) {
        return;
      }
      if (this.accept.call(this.element[0], draggable.currentItem || draggable.element)) {
        if (this.options.hoverClass) {
          this.element.addClass(this.options.hoverClass);
        }
        this._trigger('over', event, this.ui(draggable));
      }
    },
    _out: function (event) {
      var draggable = $.ui.ddmanager.current;
      // Bail if draggable and droppable are same element
      if (!draggable || (draggable.currentItem || draggable.element)[0] === this.element[0]) {
        return;
      }
      if (this.accept.call(this.element[0], draggable.currentItem || draggable.element)) {
        if (this.options.hoverClass) {
          this.element.removeClass(this.options.hoverClass);
        }
        this._trigger('out', event, this.ui(draggable));
      }
    },
    _drop: function (event, custom) {
      var draggable = custom || $.ui.ddmanager.current, childrenIntersection = false;
      // Bail if draggable and droppable are same element
      if (!draggable || (draggable.currentItem || draggable.element)[0] === this.element[0]) {
        return false;
      }
      this.element.find(':data(ui-droppable)').not('.ui-draggable-dragging').each(function () {
        var inst = $.data(this, 'ui-droppable');
        if (inst.options.greedy && !inst.options.disabled && inst.options.scope === draggable.options.scope && inst.accept.call(inst.element[0], draggable.currentItem || draggable.element) && $.ui.intersect(draggable, $.extend(inst, { offset: inst.element.offset() }), inst.options.tolerance)) {
          childrenIntersection = true;
          return false;
        }
      });
      if (childrenIntersection) {
        return false;
      }
      if (this.accept.call(this.element[0], draggable.currentItem || draggable.element)) {
        if (this.options.activeClass) {
          this.element.removeClass(this.options.activeClass);
        }
        if (this.options.hoverClass) {
          this.element.removeClass(this.options.hoverClass);
        }
        this._trigger('drop', event, this.ui(draggable));
        return this.element;
      }
      return false;
    },
    ui: function (c) {
      return {
        draggable: c.currentItem || c.element,
        helper: c.helper,
        position: c.position,
        offset: c.positionAbs
      };
    }
  });
  $.ui.intersect = function (draggable, droppable, toleranceMode) {
    if (!droppable.offset) {
      return false;
    }
    var draggableLeft, draggableTop, x1 = (draggable.positionAbs || draggable.position.absolute).left, y1 = (draggable.positionAbs || draggable.position.absolute).top, x2 = x1 + draggable.helperProportions.width, y2 = y1 + draggable.helperProportions.height, l = droppable.offset.left, t = droppable.offset.top, r = l + droppable.proportions().width, b = t + droppable.proportions().height;
    switch (toleranceMode) {
    case 'fit':
      return l <= x1 && x2 <= r && t <= y1 && y2 <= b;
    case 'intersect':
      return l < x1 + draggable.helperProportions.width / 2 && x2 - draggable.helperProportions.width / 2 < r && t < y1 + draggable.helperProportions.height / 2 && y2 - draggable.helperProportions.height / 2 < b;
    // Top Half
    case 'pointer':
      draggableLeft = (draggable.positionAbs || draggable.position.absolute).left + (draggable.clickOffset || draggable.offset.click).left;
      draggableTop = (draggable.positionAbs || draggable.position.absolute).top + (draggable.clickOffset || draggable.offset.click).top;
      return isOverAxis(draggableTop, t, droppable.proportions().height) && isOverAxis(draggableLeft, l, droppable.proportions().width);
    case 'touch':
      return (y1 >= t && y1 <= b || y2 >= t && y2 <= b || y1 < t && y2 > b) && (x1 >= l && x1 <= r || x2 >= l && x2 <= r || x1 < l && x2 > r);
    default:
      return false;
    }
  };
  /*
	This manager tracks offsets of draggables and droppables
*/
  $.ui.ddmanager = {
    current: null,
    droppables: { 'default': [] },
    prepareOffsets: function (t, event) {
      var i, j, m = $.ui.ddmanager.droppables[t.options.scope] || [], type = event ? event.type : null,
        // workaround for #2317
        list = (t.currentItem || t.element).find(':data(ui-droppable)').addBack();
      droppablesLoop:
        for (i = 0; i < m.length; i++) {
          //No disabled and non-accepted
          if (m[i].options.disabled || t && !m[i].accept.call(m[i].element[0], t.currentItem || t.element)) {
            continue;
          }
          // Filter out elements in the current dragged item
          for (j = 0; j < list.length; j++) {
            if (list[j] === m[i].element[0]) {
              m[i].proportions().height = 0;
              continue droppablesLoop;
            }
          }
          m[i].visible = m[i].element.css('display') !== 'none';
          if (!m[i].visible) {
            continue;
          }
          //Activate the droppable if used directly from draggables
          if (type === 'mousedown') {
            m[i]._activate.call(m[i], event);
          }
          m[i].offset = m[i].element.offset();
          m[i].proportions({
            width: m[i].element[0].offsetWidth,
            height: m[i].element[0].offsetHeight
          });
        }
    },
    drop: function (draggable, event) {
      var dropped = false;
      // Create a copy of the droppables in case the list changes during the drop (#9116)
      $.each(($.ui.ddmanager.droppables[draggable.options.scope] || []).slice(), function () {
        if (!this.options) {
          return;
        }
        if (!this.options.disabled && this.visible && $.ui.intersect(draggable, this, this.options.tolerance)) {
          dropped = this._drop.call(this, event) || dropped;
        }
        if (!this.options.disabled && this.visible && this.accept.call(this.element[0], draggable.currentItem || draggable.element)) {
          this.isout = true;
          this.isover = false;
          this._deactivate.call(this, event);
        }
      });
      return dropped;
    },
    dragStart: function (draggable, event) {
      //Listen for scrolling so that if the dragging causes scrolling the position of the droppables can be recalculated (see #5003)
      draggable.element.parentsUntil('body').bind('scroll.droppable', function () {
        if (!draggable.options.refreshPositions) {
          $.ui.ddmanager.prepareOffsets(draggable, event);
        }
      });
    },
    drag: function (draggable, event) {
      //If you have a highly dynamic page, you might try this option. It renders positions every time you move the mouse.
      if (draggable.options.refreshPositions) {
        $.ui.ddmanager.prepareOffsets(draggable, event);
      }
      //Run through all droppables and check their positions based on specific tolerance options
      $.each($.ui.ddmanager.droppables[draggable.options.scope] || [], function () {
        if (this.options.disabled || this.greedyChild || !this.visible) {
          return;
        }
        var parentInstance, scope, parent, intersects = $.ui.intersect(draggable, this, this.options.tolerance), c = !intersects && this.isover ? 'isout' : intersects && !this.isover ? 'isover' : null;
        if (!c) {
          return;
        }
        if (this.options.greedy) {
          // find droppable parents with same scope
          scope = this.options.scope;
          parent = this.element.parents(':data(ui-droppable)').filter(function () {
            return $.data(this, 'ui-droppable').options.scope === scope;
          });
          if (parent.length) {
            parentInstance = $.data(parent[0], 'ui-droppable');
            parentInstance.greedyChild = c === 'isover';
          }
        }
        // we just moved into a greedy child
        if (parentInstance && c === 'isover') {
          parentInstance.isover = false;
          parentInstance.isout = true;
          parentInstance._out.call(parentInstance, event);
        }
        this[c] = true;
        this[c === 'isout' ? 'isover' : 'isout'] = false;
        this[c === 'isover' ? '_over' : '_out'].call(this, event);
        // we just moved out of a greedy child
        if (parentInstance && c === 'isout') {
          parentInstance.isout = false;
          parentInstance.isover = true;
          parentInstance._over.call(parentInstance, event);
        }
      });
    },
    dragStop: function (draggable, event) {
      draggable.element.parentsUntil('body').unbind('scroll.droppable');
      //Call prepareOffsets one final time since IE does not fire return scroll events when overflow was caused by drag (see #5003)
      if (!draggable.options.refreshPositions) {
        $.ui.ddmanager.prepareOffsets(draggable, event);
      }
    }
  };
}(jQuery));
(function ($, undefined) {
  var dataSpace = 'ui-effects-';
  $.effects = { effect: {} };
  /*!
 * jQuery Color Animations v2.1.2
 * https://github.com/jquery/jquery-color
 *
 * Copyright 2013 jQuery Foundation and other contributors
 * Released under the MIT license.
 * http://jquery.org/license
 *
 * Date: Wed Jan 16 08:47:09 2013 -0600
 */
  (function (jQuery, undefined) {
    var stepHooks = 'backgroundColor borderBottomColor borderLeftColor borderRightColor borderTopColor color columnRuleColor outlineColor textDecorationColor textEmphasisColor',
      // plusequals test for += 100 -= 100
      rplusequals = /^([\-+])=\s*(\d+\.?\d*)/,
      // a set of RE's that can match strings and generate color tuples.
      stringParsers = [
        {
          re: /rgba?\(\s*(\d{1,3})\s*,\s*(\d{1,3})\s*,\s*(\d{1,3})\s*(?:,\s*(\d?(?:\.\d+)?)\s*)?\)/,
          parse: function (execResult) {
            return [
              execResult[1],
              execResult[2],
              execResult[3],
              execResult[4]
            ];
          }
        },
        {
          re: /rgba?\(\s*(\d+(?:\.\d+)?)\%\s*,\s*(\d+(?:\.\d+)?)\%\s*,\s*(\d+(?:\.\d+)?)\%\s*(?:,\s*(\d?(?:\.\d+)?)\s*)?\)/,
          parse: function (execResult) {
            return [
              execResult[1] * 2.55,
              execResult[2] * 2.55,
              execResult[3] * 2.55,
              execResult[4]
            ];
          }
        },
        {
          re: /#([a-f0-9]{2})([a-f0-9]{2})([a-f0-9]{2})/,
          parse: function (execResult) {
            return [
              parseInt(execResult[1], 16),
              parseInt(execResult[2], 16),
              parseInt(execResult[3], 16)
            ];
          }
        },
        {
          re: /#([a-f0-9])([a-f0-9])([a-f0-9])/,
          parse: function (execResult) {
            return [
              parseInt(execResult[1] + execResult[1], 16),
              parseInt(execResult[2] + execResult[2], 16),
              parseInt(execResult[3] + execResult[3], 16)
            ];
          }
        },
        {
          re: /hsla?\(\s*(\d+(?:\.\d+)?)\s*,\s*(\d+(?:\.\d+)?)\%\s*,\s*(\d+(?:\.\d+)?)\%\s*(?:,\s*(\d?(?:\.\d+)?)\s*)?\)/,
          space: 'hsla',
          parse: function (execResult) {
            return [
              execResult[1],
              execResult[2] / 100,
              execResult[3] / 100,
              execResult[4]
            ];
          }
        }
      ],
      // jQuery.Color( )
      color = jQuery.Color = function (color, green, blue, alpha) {
        return new jQuery.Color.fn.parse(color, green, blue, alpha);
      }, spaces = {
        rgba: {
          props: {
            red: {
              idx: 0,
              type: 'byte'
            },
            green: {
              idx: 1,
              type: 'byte'
            },
            blue: {
              idx: 2,
              type: 'byte'
            }
          }
        },
        hsla: {
          props: {
            hue: {
              idx: 0,
              type: 'degrees'
            },
            saturation: {
              idx: 1,
              type: 'percent'
            },
            lightness: {
              idx: 2,
              type: 'percent'
            }
          }
        }
      }, propTypes = {
        'byte': {
          floor: true,
          max: 255
        },
        'percent': { max: 1 },
        'degrees': {
          mod: 360,
          floor: true
        }
      }, support = color.support = {},
      // element for support tests
      supportElem = jQuery('<p>')[0],
      // colors = jQuery.Color.names
      colors,
      // local aliases of functions called often
      each = jQuery.each;
    // determine rgba support immediately
    supportElem.style.cssText = 'background-color:rgba(1,1,1,.5)';
    support.rgba = supportElem.style.backgroundColor.indexOf('rgba') > -1;
    // define cache name and alpha properties
    // for rgba and hsla spaces
    each(spaces, function (spaceName, space) {
      space.cache = '_' + spaceName;
      space.props.alpha = {
        idx: 3,
        type: 'percent',
        def: 1
      };
    });
    function clamp(value, prop, allowEmpty) {
      var type = propTypes[prop.type] || {};
      if (value == null) {
        return allowEmpty || !prop.def ? null : prop.def;
      }
      // ~~ is an short way of doing floor for positive numbers
      value = type.floor ? ~~value : parseFloat(value);
      // IE will pass in empty strings as value for alpha,
      // which will hit this case
      if (isNaN(value)) {
        return prop.def;
      }
      if (type.mod) {
        // we add mod before modding to make sure that negatives values
        // get converted properly: -10 -> 350
        return (value + type.mod) % type.mod;
      }
      // for now all property types without mod have min and max
      return 0 > value ? 0 : type.max < value ? type.max : value;
    }
    function stringParse(string) {
      var inst = color(), rgba = inst._rgba = [];
      string = string.toLowerCase();
      each(stringParsers, function (i, parser) {
        var parsed, match = parser.re.exec(string), values = match && parser.parse(match), spaceName = parser.space || 'rgba';
        if (values) {
          parsed = inst[spaceName](values);
          // if this was an rgba parse the assignment might happen twice
          // oh well....
          inst[spaces[spaceName].cache] = parsed[spaces[spaceName].cache];
          rgba = inst._rgba = parsed._rgba;
          // exit each( stringParsers ) here because we matched
          return false;
        }
      });
      // Found a stringParser that handled it
      if (rgba.length) {
        // if this came from a parsed string, force "transparent" when alpha is 0
        // chrome, (and maybe others) return "transparent" as rgba(0,0,0,0)
        if (rgba.join() === '0,0,0,0') {
          jQuery.extend(rgba, colors.transparent);
        }
        return inst;
      }
      // named colors
      return colors[string];
    }
    color.fn = jQuery.extend(color.prototype, {
      parse: function (red, green, blue, alpha) {
        if (red === undefined) {
          this._rgba = [
            null,
            null,
            null,
            null
          ];
          return this;
        }
        if (red.jquery || red.nodeType) {
          red = jQuery(red).css(green);
          green = undefined;
        }
        var inst = this, type = jQuery.type(red), rgba = this._rgba = [];
        // more than 1 argument specified - assume ( red, green, blue, alpha )
        if (green !== undefined) {
          red = [
            red,
            green,
            blue,
            alpha
          ];
          type = 'array';
        }
        if (type === 'string') {
          return this.parse(stringParse(red) || colors._default);
        }
        if (type === 'array') {
          each(spaces.rgba.props, function (key, prop) {
            rgba[prop.idx] = clamp(red[prop.idx], prop);
          });
          return this;
        }
        if (type === 'object') {
          if (red instanceof color) {
            each(spaces, function (spaceName, space) {
              if (red[space.cache]) {
                inst[space.cache] = red[space.cache].slice();
              }
            });
          } else {
            each(spaces, function (spaceName, space) {
              var cache = space.cache;
              each(space.props, function (key, prop) {
                // if the cache doesn't exist, and we know how to convert
                if (!inst[cache] && space.to) {
                  // if the value was null, we don't need to copy it
                  // if the key was alpha, we don't need to copy it either
                  if (key === 'alpha' || red[key] == null) {
                    return;
                  }
                  inst[cache] = space.to(inst._rgba);
                }
                // this is the only case where we allow nulls for ALL properties.
                // call clamp with alwaysAllowEmpty
                inst[cache][prop.idx] = clamp(red[key], prop, true);
              });
              // everything defined but alpha?
              if (inst[cache] && jQuery.inArray(null, inst[cache].slice(0, 3)) < 0) {
                // use the default of 1
                inst[cache][3] = 1;
                if (space.from) {
                  inst._rgba = space.from(inst[cache]);
                }
              }
            });
          }
          return this;
        }
      },
      is: function (compare) {
        var is = color(compare), same = true, inst = this;
        each(spaces, function (_, space) {
          var localCache, isCache = is[space.cache];
          if (isCache) {
            localCache = inst[space.cache] || space.to && space.to(inst._rgba) || [];
            each(space.props, function (_, prop) {
              if (isCache[prop.idx] != null) {
                same = isCache[prop.idx] === localCache[prop.idx];
                return same;
              }
            });
          }
          return same;
        });
        return same;
      },
      _space: function () {
        var used = [], inst = this;
        each(spaces, function (spaceName, space) {
          if (inst[space.cache]) {
            used.push(spaceName);
          }
        });
        return used.pop();
      },
      transition: function (other, distance) {
        var end = color(other), spaceName = end._space(), space = spaces[spaceName], startColor = this.alpha() === 0 ? color('transparent') : this, start = startColor[space.cache] || space.to(startColor._rgba), result = start.slice();
        end = end[space.cache];
        each(space.props, function (key, prop) {
          var index = prop.idx, startValue = start[index], endValue = end[index], type = propTypes[prop.type] || {};
          // if null, don't override start value
          if (endValue === null) {
            return;
          }
          // if null - use end
          if (startValue === null) {
            result[index] = endValue;
          } else {
            if (type.mod) {
              if (endValue - startValue > type.mod / 2) {
                startValue += type.mod;
              } else if (startValue - endValue > type.mod / 2) {
                startValue -= type.mod;
              }
            }
            result[index] = clamp((endValue - startValue) * distance + startValue, prop);
          }
        });
        return this[spaceName](result);
      },
      blend: function (opaque) {
        // if we are already opaque - return ourself
        if (this._rgba[3] === 1) {
          return this;
        }
        var rgb = this._rgba.slice(), a = rgb.pop(), blend = color(opaque)._rgba;
        return color(jQuery.map(rgb, function (v, i) {
          return (1 - a) * blend[i] + a * v;
        }));
      },
      toRgbaString: function () {
        var prefix = 'rgba(', rgba = jQuery.map(this._rgba, function (v, i) {
            return v == null ? i > 2 ? 1 : 0 : v;
          });
        if (rgba[3] === 1) {
          rgba.pop();
          prefix = 'rgb(';
        }
        return prefix + rgba.join() + ')';
      },
      toHslaString: function () {
        var prefix = 'hsla(', hsla = jQuery.map(this.hsla(), function (v, i) {
            if (v == null) {
              v = i > 2 ? 1 : 0;
            }
            // catch 1 and 2
            if (i && i < 3) {
              v = Math.round(v * 100) + '%';
            }
            return v;
          });
        if (hsla[3] === 1) {
          hsla.pop();
          prefix = 'hsl(';
        }
        return prefix + hsla.join() + ')';
      },
      toHexString: function (includeAlpha) {
        var rgba = this._rgba.slice(), alpha = rgba.pop();
        if (includeAlpha) {
          rgba.push(~~(alpha * 255));
        }
        return '#' + jQuery.map(rgba, function (v) {
          // default to 0 when nulls exist
          v = (v || 0).toString(16);
          return v.length === 1 ? '0' + v : v;
        }).join('');
      },
      toString: function () {
        return this._rgba[3] === 0 ? 'transparent' : this.toRgbaString();
      }
    });
    color.fn.parse.prototype = color.fn;
    // hsla conversions adapted from:
    // https://code.google.com/p/maashaack/source/browse/packages/graphics/trunk/src/graphics/colors/HUE2RGB.as?r=5021
    function hue2rgb(p, q, h) {
      h = (h + 1) % 1;
      if (h * 6 < 1) {
        return p + (q - p) * h * 6;
      }
      if (h * 2 < 1) {
        return q;
      }
      if (h * 3 < 2) {
        return p + (q - p) * (2 / 3 - h) * 6;
      }
      return p;
    }
    spaces.hsla.to = function (rgba) {
      if (rgba[0] == null || rgba[1] == null || rgba[2] == null) {
        return [
          null,
          null,
          null,
          rgba[3]
        ];
      }
      var r = rgba[0] / 255, g = rgba[1] / 255, b = rgba[2] / 255, a = rgba[3], max = Math.max(r, g, b), min = Math.min(r, g, b), diff = max - min, add = max + min, l = add * 0.5, h, s;
      if (min === max) {
        h = 0;
      } else if (r === max) {
        h = 60 * (g - b) / diff + 360;
      } else if (g === max) {
        h = 60 * (b - r) / diff + 120;
      } else {
        h = 60 * (r - g) / diff + 240;
      }
      // chroma (diff) == 0 means greyscale which, by definition, saturation = 0%
      // otherwise, saturation is based on the ratio of chroma (diff) to lightness (add)
      if (diff === 0) {
        s = 0;
      } else if (l <= 0.5) {
        s = diff / add;
      } else {
        s = diff / (2 - add);
      }
      return [
        Math.round(h) % 360,
        s,
        l,
        a == null ? 1 : a
      ];
    };
    spaces.hsla.from = function (hsla) {
      if (hsla[0] == null || hsla[1] == null || hsla[2] == null) {
        return [
          null,
          null,
          null,
          hsla[3]
        ];
      }
      var h = hsla[0] / 360, s = hsla[1], l = hsla[2], a = hsla[3], q = l <= 0.5 ? l * (1 + s) : l + s - l * s, p = 2 * l - q;
      return [
        Math.round(hue2rgb(p, q, h + 1 / 3) * 255),
        Math.round(hue2rgb(p, q, h) * 255),
        Math.round(hue2rgb(p, q, h - 1 / 3) * 255),
        a
      ];
    };
    each(spaces, function (spaceName, space) {
      var props = space.props, cache = space.cache, to = space.to, from = space.from;
      // makes rgba() and hsla()
      color.fn[spaceName] = function (value) {
        // generate a cache for this space if it doesn't exist
        if (to && !this[cache]) {
          this[cache] = to(this._rgba);
        }
        if (value === undefined) {
          return this[cache].slice();
        }
        var ret, type = jQuery.type(value), arr = type === 'array' || type === 'object' ? value : arguments, local = this[cache].slice();
        each(props, function (key, prop) {
          var val = arr[type === 'object' ? key : prop.idx];
          if (val == null) {
            val = local[prop.idx];
          }
          local[prop.idx] = clamp(val, prop);
        });
        if (from) {
          ret = color(from(local));
          ret[cache] = local;
          return ret;
        } else {
          return color(local);
        }
      };
      // makes red() green() blue() alpha() hue() saturation() lightness()
      each(props, function (key, prop) {
        // alpha is included in more than one space
        if (color.fn[key]) {
          return;
        }
        color.fn[key] = function (value) {
          var vtype = jQuery.type(value), fn = key === 'alpha' ? this._hsla ? 'hsla' : 'rgba' : spaceName, local = this[fn](), cur = local[prop.idx], match;
          if (vtype === 'undefined') {
            return cur;
          }
          if (vtype === 'function') {
            value = value.call(this, cur);
            vtype = jQuery.type(value);
          }
          if (value == null && prop.empty) {
            return this;
          }
          if (vtype === 'string') {
            match = rplusequals.exec(value);
            if (match) {
              value = cur + parseFloat(match[2]) * (match[1] === '+' ? 1 : -1);
            }
          }
          local[prop.idx] = value;
          return this[fn](local);
        };
      });
    });
    // add cssHook and .fx.step function for each named hook.
    // accept a space separated string of properties
    color.hook = function (hook) {
      var hooks = hook.split(' ');
      each(hooks, function (i, hook) {
        jQuery.cssHooks[hook] = {
          set: function (elem, value) {
            var parsed, curElem, backgroundColor = '';
            if (value !== 'transparent' && (jQuery.type(value) !== 'string' || (parsed = stringParse(value)))) {
              value = color(parsed || value);
              if (!support.rgba && value._rgba[3] !== 1) {
                curElem = hook === 'backgroundColor' ? elem.parentNode : elem;
                while ((backgroundColor === '' || backgroundColor === 'transparent') && curElem && curElem.style) {
                  try {
                    backgroundColor = jQuery.css(curElem, 'backgroundColor');
                    curElem = curElem.parentNode;
                  } catch (e) {
                  }
                }
                value = value.blend(backgroundColor && backgroundColor !== 'transparent' ? backgroundColor : '_default');
              }
              value = value.toRgbaString();
            }
            try {
              elem.style[hook] = value;
            } catch (e) {
            }
          }
        };
        jQuery.fx.step[hook] = function (fx) {
          if (!fx.colorInit) {
            fx.start = color(fx.elem, hook);
            fx.end = color(fx.end);
            fx.colorInit = true;
          }
          jQuery.cssHooks[hook].set(fx.elem, fx.start.transition(fx.end, fx.pos));
        };
      });
    };
    color.hook(stepHooks);
    jQuery.cssHooks.borderColor = {
      expand: function (value) {
        var expanded = {};
        each([
          'Top',
          'Right',
          'Bottom',
          'Left'
        ], function (i, part) {
          expanded['border' + part + 'Color'] = value;
        });
        return expanded;
      }
    };
    // Basic color names only.
    // Usage of any of the other color names requires adding yourself or including
    // jquery.color.svg-names.js.
    colors = jQuery.Color.names = {
      aqua: '#00ffff',
      black: '#000000',
      blue: '#0000ff',
      fuchsia: '#ff00ff',
      gray: '#808080',
      green: '#008000',
      lime: '#00ff00',
      maroon: '#800000',
      navy: '#000080',
      olive: '#808000',
      purple: '#800080',
      red: '#ff0000',
      silver: '#c0c0c0',
      teal: '#008080',
      white: '#ffffff',
      yellow: '#ffff00',
      transparent: [
        null,
        null,
        null,
        0
      ],
      _default: '#ffffff'
    };
  }(jQuery));
  /******************************************************************************/
  /****************************** CLASS ANIMATIONS ******************************/
  /******************************************************************************/
  (function () {
    var classAnimationActions = [
        'add',
        'remove',
        'toggle'
      ], shorthandStyles = {
        border: 1,
        borderBottom: 1,
        borderColor: 1,
        borderLeft: 1,
        borderRight: 1,
        borderTop: 1,
        borderWidth: 1,
        margin: 1,
        padding: 1
      };
    $.each([
      'borderLeftStyle',
      'borderRightStyle',
      'borderBottomStyle',
      'borderTopStyle'
    ], function (_, prop) {
      $.fx.step[prop] = function (fx) {
        if (fx.end !== 'none' && !fx.setAttr || fx.pos === 1 && !fx.setAttr) {
          jQuery.style(fx.elem, prop, fx.end);
          fx.setAttr = true;
        }
      };
    });
    function getElementStyles(elem) {
      var key, len, style = elem.ownerDocument.defaultView ? elem.ownerDocument.defaultView.getComputedStyle(elem, null) : elem.currentStyle, styles = {};
      if (style && style.length && style[0] && style[style[0]]) {
        len = style.length;
        while (len--) {
          key = style[len];
          if (typeof style[key] === 'string') {
            styles[$.camelCase(key)] = style[key];
          }
        }  // support: Opera, IE <9
      } else {
        for (key in style) {
          if (typeof style[key] === 'string') {
            styles[key] = style[key];
          }
        }
      }
      return styles;
    }
    function styleDifference(oldStyle, newStyle) {
      var diff = {}, name, value;
      for (name in newStyle) {
        value = newStyle[name];
        if (oldStyle[name] !== value) {
          if (!shorthandStyles[name]) {
            if ($.fx.step[name] || !isNaN(parseFloat(value))) {
              diff[name] = value;
            }
          }
        }
      }
      return diff;
    }
    // support: jQuery <1.8
    if (!$.fn.addBack) {
      $.fn.addBack = function (selector) {
        return this.add(selector == null ? this.prevObject : this.prevObject.filter(selector));
      };
    }
    $.effects.animateClass = function (value, duration, easing, callback) {
      var o = $.speed(duration, easing, callback);
      return this.queue(function () {
        var animated = $(this), baseClass = animated.attr('class') || '', applyClassChange, allAnimations = o.children ? animated.find('*').addBack() : animated;
        // map the animated objects to store the original styles.
        allAnimations = allAnimations.map(function () {
          var el = $(this);
          return {
            el: el,
            start: getElementStyles(this)
          };
        });
        // apply class change
        applyClassChange = function () {
          $.each(classAnimationActions, function (i, action) {
            if (value[action]) {
              animated[action + 'Class'](value[action]);
            }
          });
        };
        applyClassChange();
        // map all animated objects again - calculate new styles and diff
        allAnimations = allAnimations.map(function () {
          this.end = getElementStyles(this.el[0]);
          this.diff = styleDifference(this.start, this.end);
          return this;
        });
        // apply original class
        animated.attr('class', baseClass);
        // map all animated objects again - this time collecting a promise
        allAnimations = allAnimations.map(function () {
          var styleInfo = this, dfd = $.Deferred(), opts = $.extend({}, o, {
              queue: false,
              complete: function () {
                dfd.resolve(styleInfo);
              }
            });
          this.el.animate(this.diff, opts);
          return dfd.promise();
        });
        // once all animations have completed:
        $.when.apply($, allAnimations.get()).done(function () {
          // set the final class
          applyClassChange();
          // for each animated element,
          // clear all css properties that were animated
          $.each(arguments, function () {
            var el = this.el;
            $.each(this.diff, function (key) {
              el.css(key, '');
            });
          });
          // this is guarnteed to be there if you use jQuery.speed()
          // it also handles dequeuing the next anim...
          o.complete.call(animated[0]);
        });
      });
    };
    $.fn.extend({
      addClass: function (orig) {
        return function (classNames, speed, easing, callback) {
          return speed ? $.effects.animateClass.call(this, { add: classNames }, speed, easing, callback) : orig.apply(this, arguments);
        };
      }($.fn.addClass),
      removeClass: function (orig) {
        return function (classNames, speed, easing, callback) {
          return arguments.length > 1 ? $.effects.animateClass.call(this, { remove: classNames }, speed, easing, callback) : orig.apply(this, arguments);
        };
      }($.fn.removeClass),
      toggleClass: function (orig) {
        return function (classNames, force, speed, easing, callback) {
          if (typeof force === 'boolean' || force === undefined) {
            if (!speed) {
              // without speed parameter
              return orig.apply(this, arguments);
            } else {
              return $.effects.animateClass.call(this, force ? { add: classNames } : { remove: classNames }, speed, easing, callback);
            }
          } else {
            // without force parameter
            return $.effects.animateClass.call(this, { toggle: classNames }, force, speed, easing);
          }
        };
      }($.fn.toggleClass),
      switchClass: function (remove, add, speed, easing, callback) {
        return $.effects.animateClass.call(this, {
          add: add,
          remove: remove
        }, speed, easing, callback);
      }
    });
  }());
  /******************************************************************************/
  /*********************************** EFFECTS **********************************/
  /******************************************************************************/
  (function () {
    $.extend($.effects, {
      version: '1.10.4',
      save: function (element, set) {
        for (var i = 0; i < set.length; i++) {
          if (set[i] !== null) {
            element.data(dataSpace + set[i], element[0].style[set[i]]);
          }
        }
      },
      restore: function (element, set) {
        var val, i;
        for (i = 0; i < set.length; i++) {
          if (set[i] !== null) {
            val = element.data(dataSpace + set[i]);
            // support: jQuery 1.6.2
            // http://bugs.jquery.com/ticket/9917
            // jQuery 1.6.2 incorrectly returns undefined for any falsy value.
            // We can't differentiate between "" and 0 here, so we just assume
            // empty string since it's likely to be a more common value...
            if (val === undefined) {
              val = '';
            }
            element.css(set[i], val);
          }
        }
      },
      setMode: function (el, mode) {
        if (mode === 'toggle') {
          mode = el.is(':hidden') ? 'show' : 'hide';
        }
        return mode;
      },
      getBaseline: function (origin, original) {
        var y, x;
        switch (origin[0]) {
        case 'top':
          y = 0;
          break;
        case 'middle':
          y = 0.5;
          break;
        case 'bottom':
          y = 1;
          break;
        default:
          y = origin[0] / original.height;
        }
        switch (origin[1]) {
        case 'left':
          x = 0;
          break;
        case 'center':
          x = 0.5;
          break;
        case 'right':
          x = 1;
          break;
        default:
          x = origin[1] / original.width;
        }
        return {
          x: x,
          y: y
        };
      },
      createWrapper: function (element) {
        // if the element is already wrapped, return it
        if (element.parent().is('.ui-effects-wrapper')) {
          return element.parent();
        }
        // wrap the element
        var props = {
            width: element.outerWidth(true),
            height: element.outerHeight(true),
            'float': element.css('float')
          }, wrapper = $('<div></div>').addClass('ui-effects-wrapper').css({
            fontSize: '100%',
            background: 'transparent',
            border: 'none',
            margin: 0,
            padding: 0
          }),
          // Store the size in case width/height are defined in % - Fixes #5245
          size = {
            width: element.width(),
            height: element.height()
          }, active = document.activeElement;
        // support: Firefox
        // Firefox incorrectly exposes anonymous content
        // https://bugzilla.mozilla.org/show_bug.cgi?id=561664
        try {
          active.id;
        } catch (e) {
          active = document.body;
        }
        element.wrap(wrapper);
        // Fixes #7595 - Elements lose focus when wrapped.
        if (element[0] === active || $.contains(element[0], active)) {
          $(active).focus();
        }
        wrapper = element.parent();
        //Hotfix for jQuery 1.4 since some change in wrap() seems to actually lose the reference to the wrapped element
        // transfer positioning properties to the wrapper
        if (element.css('position') === 'static') {
          wrapper.css({ position: 'relative' });
          element.css({ position: 'relative' });
        } else {
          $.extend(props, {
            position: element.css('position'),
            zIndex: element.css('z-index')
          });
          $.each([
            'top',
            'left',
            'bottom',
            'right'
          ], function (i, pos) {
            props[pos] = element.css(pos);
            if (isNaN(parseInt(props[pos], 10))) {
              props[pos] = 'auto';
            }
          });
          element.css({
            position: 'relative',
            top: 0,
            left: 0,
            right: 'auto',
            bottom: 'auto'
          });
        }
        element.css(size);
        return wrapper.css(props).show();
      },
      removeWrapper: function (element) {
        var active = document.activeElement;
        if (element.parent().is('.ui-effects-wrapper')) {
          element.parent().replaceWith(element);
          // Fixes #7595 - Elements lose focus when wrapped.
          if (element[0] === active || $.contains(element[0], active)) {
            $(active).focus();
          }
        }
        return element;
      },
      setTransition: function (element, list, factor, value) {
        value = value || {};
        $.each(list, function (i, x) {
          var unit = element.cssUnit(x);
          if (unit[0] > 0) {
            value[x] = unit[0] * factor + unit[1];
          }
        });
        return value;
      }
    });
    // return an effect options object for the given parameters:
    function _normalizeArguments(effect, options, speed, callback) {
      // allow passing all options as the first parameter
      if ($.isPlainObject(effect)) {
        options = effect;
        effect = effect.effect;
      }
      // convert to an object
      effect = { effect: effect };
      // catch (effect, null, ...)
      if (options == null) {
        options = {};
      }
      // catch (effect, callback)
      if ($.isFunction(options)) {
        callback = options;
        speed = null;
        options = {};
      }
      // catch (effect, speed, ?)
      if (typeof options === 'number' || $.fx.speeds[options]) {
        callback = speed;
        speed = options;
        options = {};
      }
      // catch (effect, options, callback)
      if ($.isFunction(speed)) {
        callback = speed;
        speed = null;
      }
      // add options to effect
      if (options) {
        $.extend(effect, options);
      }
      speed = speed || options.duration;
      effect.duration = $.fx.off ? 0 : typeof speed === 'number' ? speed : speed in $.fx.speeds ? $.fx.speeds[speed] : $.fx.speeds._default;
      effect.complete = callback || options.complete;
      return effect;
    }
    function standardAnimationOption(option) {
      // Valid standard speeds (nothing, number, named speed)
      if (!option || typeof option === 'number' || $.fx.speeds[option]) {
        return true;
      }
      // Invalid strings - treat as "normal" speed
      if (typeof option === 'string' && !$.effects.effect[option]) {
        return true;
      }
      // Complete callback
      if ($.isFunction(option)) {
        return true;
      }
      // Options hash (but not naming an effect)
      if (typeof option === 'object' && !option.effect) {
        return true;
      }
      // Didn't match any standard API
      return false;
    }
    $.fn.extend({
      effect: function () {
        var args = _normalizeArguments.apply(this, arguments), mode = args.mode, queue = args.queue, effectMethod = $.effects.effect[args.effect];
        if ($.fx.off || !effectMethod) {
          // delegate to the original method (e.g., .show()) if possible
          if (mode) {
            return this[mode](args.duration, args.complete);
          } else {
            return this.each(function () {
              if (args.complete) {
                args.complete.call(this);
              }
            });
          }
        }
        function run(next) {
          var elem = $(this), complete = args.complete, mode = args.mode;
          function done() {
            if ($.isFunction(complete)) {
              complete.call(elem[0]);
            }
            if ($.isFunction(next)) {
              next();
            }
          }
          // If the element already has the correct final state, delegate to
          // the core methods so the internal tracking of "olddisplay" works.
          if (elem.is(':hidden') ? mode === 'hide' : mode === 'show') {
            elem[mode]();
            done();
          } else {
            effectMethod.call(elem[0], args, done);
          }
        }
        return queue === false ? this.each(run) : this.queue(queue || 'fx', run);
      },
      show: function (orig) {
        return function (option) {
          if (standardAnimationOption(option)) {
            return orig.apply(this, arguments);
          } else {
            var args = _normalizeArguments.apply(this, arguments);
            args.mode = 'show';
            return this.effect.call(this, args);
          }
        };
      }($.fn.show),
      hide: function (orig) {
        return function (option) {
          if (standardAnimationOption(option)) {
            return orig.apply(this, arguments);
          } else {
            var args = _normalizeArguments.apply(this, arguments);
            args.mode = 'hide';
            return this.effect.call(this, args);
          }
        };
      }($.fn.hide),
      toggle: function (orig) {
        return function (option) {
          if (standardAnimationOption(option) || typeof option === 'boolean') {
            return orig.apply(this, arguments);
          } else {
            var args = _normalizeArguments.apply(this, arguments);
            args.mode = 'toggle';
            return this.effect.call(this, args);
          }
        };
      }($.fn.toggle),
      cssUnit: function (key) {
        var style = this.css(key), val = [];
        $.each([
          'em',
          'px',
          '%',
          'pt'
        ], function (i, unit) {
          if (style.indexOf(unit) > 0) {
            val = [
              parseFloat(style),
              unit
            ];
          }
        });
        return val;
      }
    });
  }());
  /******************************************************************************/
  /*********************************** EASING ***********************************/
  /******************************************************************************/
  (function () {
    // based on easing equations from Robert Penner (http://www.robertpenner.com/easing)
    var baseEasings = {};
    $.each([
      'Quad',
      'Cubic',
      'Quart',
      'Quint',
      'Expo'
    ], function (i, name) {
      baseEasings[name] = function (p) {
        return Math.pow(p, i + 2);
      };
    });
    $.extend(baseEasings, {
      Sine: function (p) {
        return 1 - Math.cos(p * Math.PI / 2);
      },
      Circ: function (p) {
        return 1 - Math.sqrt(1 - p * p);
      },
      Elastic: function (p) {
        return p === 0 || p === 1 ? p : -Math.pow(2, 8 * (p - 1)) * Math.sin(((p - 1) * 80 - 7.5) * Math.PI / 15);
      },
      Back: function (p) {
        return p * p * (3 * p - 2);
      },
      Bounce: function (p) {
        var pow2, bounce = 4;
        while (p < ((pow2 = Math.pow(2, --bounce)) - 1) / 11) {
        }
        return 1 / Math.pow(4, 3 - bounce) - 7.5625 * Math.pow((pow2 * 3 - 2) / 22 - p, 2);
      }
    });
    $.each(baseEasings, function (name, easeIn) {
      $.easing['easeIn' + name] = easeIn;
      $.easing['easeOut' + name] = function (p) {
        return 1 - easeIn(1 - p);
      };
      $.easing['easeInOut' + name] = function (p) {
        return p < 0.5 ? easeIn(p * 2) / 2 : 1 - easeIn(p * -2 + 2) / 2;
      };
    });
  }());
}(jQuery));
(function ($, undefined) {
  var rvertical = /up|down|vertical/, rpositivemotion = /up|left|vertical|horizontal/;
  $.effects.effect.blind = function (o, done) {
    // Create element
    var el = $(this), props = [
        'position',
        'top',
        'bottom',
        'left',
        'right',
        'height',
        'width'
      ], mode = $.effects.setMode(el, o.mode || 'hide'), direction = o.direction || 'up', vertical = rvertical.test(direction), ref = vertical ? 'height' : 'width', ref2 = vertical ? 'top' : 'left', motion = rpositivemotion.test(direction), animation = {}, show = mode === 'show', wrapper, distance, margin;
    // if already wrapped, the wrapper's properties are my property. #6245
    if (el.parent().is('.ui-effects-wrapper')) {
      $.effects.save(el.parent(), props);
    } else {
      $.effects.save(el, props);
    }
    el.show();
    wrapper = $.effects.createWrapper(el).css({ overflow: 'hidden' });
    distance = wrapper[ref]();
    margin = parseFloat(wrapper.css(ref2)) || 0;
    animation[ref] = show ? distance : 0;
    if (!motion) {
      el.css(vertical ? 'bottom' : 'right', 0).css(vertical ? 'top' : 'left', 'auto').css({ position: 'absolute' });
      animation[ref2] = show ? margin : distance + margin;
    }
    // start at 0 if we are showing
    if (show) {
      wrapper.css(ref, 0);
      if (!motion) {
        wrapper.css(ref2, margin + distance);
      }
    }
    // Animate
    wrapper.animate(animation, {
      duration: o.duration,
      easing: o.easing,
      queue: false,
      complete: function () {
        if (mode === 'hide') {
          el.hide();
        }
        $.effects.restore(el, props);
        $.effects.removeWrapper(el);
        done();
      }
    });
  };
}(jQuery));
(function ($, undefined) {
  $.effects.effect.bounce = function (o, done) {
    var el = $(this), props = [
        'position',
        'top',
        'bottom',
        'left',
        'right',
        'height',
        'width'
      ],
      // defaults:
      mode = $.effects.setMode(el, o.mode || 'effect'), hide = mode === 'hide', show = mode === 'show', direction = o.direction || 'up', distance = o.distance, times = o.times || 5,
      // number of internal animations
      anims = times * 2 + (show || hide ? 1 : 0), speed = o.duration / anims, easing = o.easing,
      // utility:
      ref = direction === 'up' || direction === 'down' ? 'top' : 'left', motion = direction === 'up' || direction === 'left', i, upAnim, downAnim,
      // we will need to re-assemble the queue to stack our animations in place
      queue = el.queue(), queuelen = queue.length;
    // Avoid touching opacity to prevent clearType and PNG issues in IE
    if (show || hide) {
      props.push('opacity');
    }
    $.effects.save(el, props);
    el.show();
    $.effects.createWrapper(el);
    // Create Wrapper
    // default distance for the BIGGEST bounce is the outer Distance / 3
    if (!distance) {
      distance = el[ref === 'top' ? 'outerHeight' : 'outerWidth']() / 3;
    }
    if (show) {
      downAnim = { opacity: 1 };
      downAnim[ref] = 0;
      // if we are showing, force opacity 0 and set the initial position
      // then do the "first" animation
      el.css('opacity', 0).css(ref, motion ? -distance * 2 : distance * 2).animate(downAnim, speed, easing);
    }
    // start at the smallest distance if we are hiding
    if (hide) {
      distance = distance / Math.pow(2, times - 1);
    }
    downAnim = {};
    downAnim[ref] = 0;
    // Bounces up/down/left/right then back to 0 -- times * 2 animations happen here
    for (i = 0; i < times; i++) {
      upAnim = {};
      upAnim[ref] = (motion ? '-=' : '+=') + distance;
      el.animate(upAnim, speed, easing).animate(downAnim, speed, easing);
      distance = hide ? distance * 2 : distance / 2;
    }
    // Last Bounce when Hiding
    if (hide) {
      upAnim = { opacity: 0 };
      upAnim[ref] = (motion ? '-=' : '+=') + distance;
      el.animate(upAnim, speed, easing);
    }
    el.queue(function () {
      if (hide) {
        el.hide();
      }
      $.effects.restore(el, props);
      $.effects.removeWrapper(el);
      done();
    });
    // inject all the animations we just queued to be first in line (after "inprogress")
    if (queuelen > 1) {
      queue.splice.apply(queue, [
        1,
        0
      ].concat(queue.splice(queuelen, anims + 1)));
    }
    el.dequeue();
  };
}(jQuery));
(function ($, undefined) {
  $.effects.effect.clip = function (o, done) {
    // Create element
    var el = $(this), props = [
        'position',
        'top',
        'bottom',
        'left',
        'right',
        'height',
        'width'
      ], mode = $.effects.setMode(el, o.mode || 'hide'), show = mode === 'show', direction = o.direction || 'vertical', vert = direction === 'vertical', size = vert ? 'height' : 'width', position = vert ? 'top' : 'left', animation = {}, wrapper, animate, distance;
    // Save & Show
    $.effects.save(el, props);
    el.show();
    // Create Wrapper
    wrapper = $.effects.createWrapper(el).css({ overflow: 'hidden' });
    animate = el[0].tagName === 'IMG' ? wrapper : el;
    distance = animate[size]();
    // Shift
    if (show) {
      animate.css(size, 0);
      animate.css(position, distance / 2);
    }
    // Create Animation Object:
    animation[size] = show ? distance : 0;
    animation[position] = show ? 0 : distance / 2;
    // Animate
    animate.animate(animation, {
      queue: false,
      duration: o.duration,
      easing: o.easing,
      complete: function () {
        if (!show) {
          el.hide();
        }
        $.effects.restore(el, props);
        $.effects.removeWrapper(el);
        done();
      }
    });
  };
}(jQuery));
(function ($, undefined) {
  $.effects.effect.drop = function (o, done) {
    var el = $(this), props = [
        'position',
        'top',
        'bottom',
        'left',
        'right',
        'opacity',
        'height',
        'width'
      ], mode = $.effects.setMode(el, o.mode || 'hide'), show = mode === 'show', direction = o.direction || 'left', ref = direction === 'up' || direction === 'down' ? 'top' : 'left', motion = direction === 'up' || direction === 'left' ? 'pos' : 'neg', animation = { opacity: show ? 1 : 0 }, distance;
    // Adjust
    $.effects.save(el, props);
    el.show();
    $.effects.createWrapper(el);
    distance = o.distance || el[ref === 'top' ? 'outerHeight' : 'outerWidth'](true) / 2;
    if (show) {
      el.css('opacity', 0).css(ref, motion === 'pos' ? -distance : distance);
    }
    // Animation
    animation[ref] = (show ? motion === 'pos' ? '+=' : '-=' : motion === 'pos' ? '-=' : '+=') + distance;
    // Animate
    el.animate(animation, {
      queue: false,
      duration: o.duration,
      easing: o.easing,
      complete: function () {
        if (mode === 'hide') {
          el.hide();
        }
        $.effects.restore(el, props);
        $.effects.removeWrapper(el);
        done();
      }
    });
  };
}(jQuery));
(function ($, undefined) {
  $.effects.effect.explode = function (o, done) {
    var rows = o.pieces ? Math.round(Math.sqrt(o.pieces)) : 3, cells = rows, el = $(this), mode = $.effects.setMode(el, o.mode || 'hide'), show = mode === 'show',
      // show and then visibility:hidden the element before calculating offset
      offset = el.show().css('visibility', 'hidden').offset(),
      // width and height of a piece
      width = Math.ceil(el.outerWidth() / cells), height = Math.ceil(el.outerHeight() / rows), pieces = [],
      // loop
      i, j, left, top, mx, my;
    // children animate complete:
    function childComplete() {
      pieces.push(this);
      if (pieces.length === rows * cells) {
        animComplete();
      }
    }
    // clone the element for each row and cell.
    for (i = 0; i < rows; i++) {
      // ===>
      top = offset.top + i * height;
      my = i - (rows - 1) / 2;
      for (j = 0; j < cells; j++) {
        // |||
        left = offset.left + j * width;
        mx = j - (cells - 1) / 2;
        // Create a clone of the now hidden main element that will be absolute positioned
        // within a wrapper div off the -left and -top equal to size of our pieces
        el.clone().appendTo('body').wrap('<div></div>').css({
          position: 'absolute',
          visibility: 'visible',
          left: -j * width,
          top: -i * height
        }).parent().addClass('ui-effects-explode').css({
          position: 'absolute',
          overflow: 'hidden',
          width: width,
          height: height,
          left: left + (show ? mx * width : 0),
          top: top + (show ? my * height : 0),
          opacity: show ? 0 : 1
        }).animate({
          left: left + (show ? 0 : mx * width),
          top: top + (show ? 0 : my * height),
          opacity: show ? 1 : 0
        }, o.duration || 500, o.easing, childComplete);
      }
    }
    function animComplete() {
      el.css({ visibility: 'visible' });
      $(pieces).remove();
      if (!show) {
        el.hide();
      }
      done();
    }
  };
}(jQuery));
(function ($, undefined) {
  $.effects.effect.fade = function (o, done) {
    var el = $(this), mode = $.effects.setMode(el, o.mode || 'toggle');
    el.animate({ opacity: mode }, {
      queue: false,
      duration: o.duration,
      easing: o.easing,
      complete: done
    });
  };
}(jQuery));
(function ($, undefined) {
  $.effects.effect.fold = function (o, done) {
    // Create element
    var el = $(this), props = [
        'position',
        'top',
        'bottom',
        'left',
        'right',
        'height',
        'width'
      ], mode = $.effects.setMode(el, o.mode || 'hide'), show = mode === 'show', hide = mode === 'hide', size = o.size || 15, percent = /([0-9]+)%/.exec(size), horizFirst = !!o.horizFirst, widthFirst = show !== horizFirst, ref = widthFirst ? [
        'width',
        'height'
      ] : [
        'height',
        'width'
      ], duration = o.duration / 2, wrapper, distance, animation1 = {}, animation2 = {};
    $.effects.save(el, props);
    el.show();
    // Create Wrapper
    wrapper = $.effects.createWrapper(el).css({ overflow: 'hidden' });
    distance = widthFirst ? [
      wrapper.width(),
      wrapper.height()
    ] : [
      wrapper.height(),
      wrapper.width()
    ];
    if (percent) {
      size = parseInt(percent[1], 10) / 100 * distance[hide ? 0 : 1];
    }
    if (show) {
      wrapper.css(horizFirst ? {
        height: 0,
        width: size
      } : {
        height: size,
        width: 0
      });
    }
    // Animation
    animation1[ref[0]] = show ? distance[0] : size;
    animation2[ref[1]] = show ? distance[1] : 0;
    // Animate
    wrapper.animate(animation1, duration, o.easing).animate(animation2, duration, o.easing, function () {
      if (hide) {
        el.hide();
      }
      $.effects.restore(el, props);
      $.effects.removeWrapper(el);
      done();
    });
  };
}(jQuery));
(function ($, undefined) {
  $.effects.effect.highlight = function (o, done) {
    var elem = $(this), props = [
        'backgroundImage',
        'backgroundColor',
        'opacity'
      ], mode = $.effects.setMode(elem, o.mode || 'show'), animation = { backgroundColor: elem.css('backgroundColor') };
    if (mode === 'hide') {
      animation.opacity = 0;
    }
    $.effects.save(elem, props);
    elem.show().css({
      backgroundImage: 'none',
      backgroundColor: o.color || '#ffff99'
    }).animate(animation, {
      queue: false,
      duration: o.duration,
      easing: o.easing,
      complete: function () {
        if (mode === 'hide') {
          elem.hide();
        }
        $.effects.restore(elem, props);
        done();
      }
    });
  };
}(jQuery));
(function ($, undefined) {
  $.effects.effect.pulsate = function (o, done) {
    var elem = $(this), mode = $.effects.setMode(elem, o.mode || 'show'), show = mode === 'show', hide = mode === 'hide', showhide = show || mode === 'hide',
      // showing or hiding leaves of the "last" animation
      anims = (o.times || 5) * 2 + (showhide ? 1 : 0), duration = o.duration / anims, animateTo = 0, queue = elem.queue(), queuelen = queue.length, i;
    if (show || !elem.is(':visible')) {
      elem.css('opacity', 0).show();
      animateTo = 1;
    }
    // anims - 1 opacity "toggles"
    for (i = 1; i < anims; i++) {
      elem.animate({ opacity: animateTo }, duration, o.easing);
      animateTo = 1 - animateTo;
    }
    elem.animate({ opacity: animateTo }, duration, o.easing);
    elem.queue(function () {
      if (hide) {
        elem.hide();
      }
      done();
    });
    // We just queued up "anims" animations, we need to put them next in the queue
    if (queuelen > 1) {
      queue.splice.apply(queue, [
        1,
        0
      ].concat(queue.splice(queuelen, anims + 1)));
    }
    elem.dequeue();
  };
}(jQuery));
(function ($, undefined) {
  $.effects.effect.puff = function (o, done) {
    var elem = $(this), mode = $.effects.setMode(elem, o.mode || 'hide'), hide = mode === 'hide', percent = parseInt(o.percent, 10) || 150, factor = percent / 100, original = {
        height: elem.height(),
        width: elem.width(),
        outerHeight: elem.outerHeight(),
        outerWidth: elem.outerWidth()
      };
    $.extend(o, {
      effect: 'scale',
      queue: false,
      fade: true,
      mode: mode,
      complete: done,
      percent: hide ? percent : 100,
      from: hide ? original : {
        height: original.height * factor,
        width: original.width * factor,
        outerHeight: original.outerHeight * factor,
        outerWidth: original.outerWidth * factor
      }
    });
    elem.effect(o);
  };
  $.effects.effect.scale = function (o, done) {
    // Create element
    var el = $(this), options = $.extend(true, {}, o), mode = $.effects.setMode(el, o.mode || 'effect'), percent = parseInt(o.percent, 10) || (parseInt(o.percent, 10) === 0 ? 0 : mode === 'hide' ? 0 : 100), direction = o.direction || 'both', origin = o.origin, original = {
        height: el.height(),
        width: el.width(),
        outerHeight: el.outerHeight(),
        outerWidth: el.outerWidth()
      }, factor = {
        y: direction !== 'horizontal' ? percent / 100 : 1,
        x: direction !== 'vertical' ? percent / 100 : 1
      };
    // We are going to pass this effect to the size effect:
    options.effect = 'size';
    options.queue = false;
    options.complete = done;
    // Set default origin and restore for show/hide
    if (mode !== 'effect') {
      options.origin = origin || [
        'middle',
        'center'
      ];
      options.restore = true;
    }
    options.from = o.from || (mode === 'show' ? {
      height: 0,
      width: 0,
      outerHeight: 0,
      outerWidth: 0
    } : original);
    options.to = {
      height: original.height * factor.y,
      width: original.width * factor.x,
      outerHeight: original.outerHeight * factor.y,
      outerWidth: original.outerWidth * factor.x
    };
    // Fade option to support puff
    if (options.fade) {
      if (mode === 'show') {
        options.from.opacity = 0;
        options.to.opacity = 1;
      }
      if (mode === 'hide') {
        options.from.opacity = 1;
        options.to.opacity = 0;
      }
    }
    // Animate
    el.effect(options);
  };
  $.effects.effect.size = function (o, done) {
    // Create element
    var original, baseline, factor, el = $(this), props0 = [
        'position',
        'top',
        'bottom',
        'left',
        'right',
        'width',
        'height',
        'overflow',
        'opacity'
      ],
      // Always restore
      props1 = [
        'position',
        'top',
        'bottom',
        'left',
        'right',
        'overflow',
        'opacity'
      ],
      // Copy for children
      props2 = [
        'width',
        'height',
        'overflow'
      ], cProps = ['fontSize'], vProps = [
        'borderTopWidth',
        'borderBottomWidth',
        'paddingTop',
        'paddingBottom'
      ], hProps = [
        'borderLeftWidth',
        'borderRightWidth',
        'paddingLeft',
        'paddingRight'
      ],
      // Set options
      mode = $.effects.setMode(el, o.mode || 'effect'), restore = o.restore || mode !== 'effect', scale = o.scale || 'both', origin = o.origin || [
        'middle',
        'center'
      ], position = el.css('position'), props = restore ? props0 : props1, zero = {
        height: 0,
        width: 0,
        outerHeight: 0,
        outerWidth: 0
      };
    if (mode === 'show') {
      el.show();
    }
    original = {
      height: el.height(),
      width: el.width(),
      outerHeight: el.outerHeight(),
      outerWidth: el.outerWidth()
    };
    if (o.mode === 'toggle' && mode === 'show') {
      el.from = o.to || zero;
      el.to = o.from || original;
    } else {
      el.from = o.from || (mode === 'show' ? zero : original);
      el.to = o.to || (mode === 'hide' ? zero : original);
    }
    // Set scaling factor
    factor = {
      from: {
        y: el.from.height / original.height,
        x: el.from.width / original.width
      },
      to: {
        y: el.to.height / original.height,
        x: el.to.width / original.width
      }
    };
    // Scale the css box
    if (scale === 'box' || scale === 'both') {
      // Vertical props scaling
      if (factor.from.y !== factor.to.y) {
        props = props.concat(vProps);
        el.from = $.effects.setTransition(el, vProps, factor.from.y, el.from);
        el.to = $.effects.setTransition(el, vProps, factor.to.y, el.to);
      }
      // Horizontal props scaling
      if (factor.from.x !== factor.to.x) {
        props = props.concat(hProps);
        el.from = $.effects.setTransition(el, hProps, factor.from.x, el.from);
        el.to = $.effects.setTransition(el, hProps, factor.to.x, el.to);
      }
    }
    // Scale the content
    if (scale === 'content' || scale === 'both') {
      // Vertical props scaling
      if (factor.from.y !== factor.to.y) {
        props = props.concat(cProps).concat(props2);
        el.from = $.effects.setTransition(el, cProps, factor.from.y, el.from);
        el.to = $.effects.setTransition(el, cProps, factor.to.y, el.to);
      }
    }
    $.effects.save(el, props);
    el.show();
    $.effects.createWrapper(el);
    el.css('overflow', 'hidden').css(el.from);
    // Adjust
    if (origin) {
      // Calculate baseline shifts
      baseline = $.effects.getBaseline(origin, original);
      el.from.top = (original.outerHeight - el.outerHeight()) * baseline.y;
      el.from.left = (original.outerWidth - el.outerWidth()) * baseline.x;
      el.to.top = (original.outerHeight - el.to.outerHeight) * baseline.y;
      el.to.left = (original.outerWidth - el.to.outerWidth) * baseline.x;
    }
    el.css(el.from);
    // set top & left
    // Animate
    if (scale === 'content' || scale === 'both') {
      // Scale the children
      // Add margins/font-size
      vProps = vProps.concat([
        'marginTop',
        'marginBottom'
      ]).concat(cProps);
      hProps = hProps.concat([
        'marginLeft',
        'marginRight'
      ]);
      props2 = props0.concat(vProps).concat(hProps);
      el.find('*[width]').each(function () {
        var child = $(this), c_original = {
            height: child.height(),
            width: child.width(),
            outerHeight: child.outerHeight(),
            outerWidth: child.outerWidth()
          };
        if (restore) {
          $.effects.save(child, props2);
        }
        child.from = {
          height: c_original.height * factor.from.y,
          width: c_original.width * factor.from.x,
          outerHeight: c_original.outerHeight * factor.from.y,
          outerWidth: c_original.outerWidth * factor.from.x
        };
        child.to = {
          height: c_original.height * factor.to.y,
          width: c_original.width * factor.to.x,
          outerHeight: c_original.height * factor.to.y,
          outerWidth: c_original.width * factor.to.x
        };
        // Vertical props scaling
        if (factor.from.y !== factor.to.y) {
          child.from = $.effects.setTransition(child, vProps, factor.from.y, child.from);
          child.to = $.effects.setTransition(child, vProps, factor.to.y, child.to);
        }
        // Horizontal props scaling
        if (factor.from.x !== factor.to.x) {
          child.from = $.effects.setTransition(child, hProps, factor.from.x, child.from);
          child.to = $.effects.setTransition(child, hProps, factor.to.x, child.to);
        }
        // Animate children
        child.css(child.from);
        child.animate(child.to, o.duration, o.easing, function () {
          // Restore children
          if (restore) {
            $.effects.restore(child, props2);
          }
        });
      });
    }
    // Animate
    el.animate(el.to, {
      queue: false,
      duration: o.duration,
      easing: o.easing,
      complete: function () {
        if (el.to.opacity === 0) {
          el.css('opacity', el.from.opacity);
        }
        if (mode === 'hide') {
          el.hide();
        }
        $.effects.restore(el, props);
        if (!restore) {
          // we need to calculate our new positioning based on the scaling
          if (position === 'static') {
            el.css({
              position: 'relative',
              top: el.to.top,
              left: el.to.left
            });
          } else {
            $.each([
              'top',
              'left'
            ], function (idx, pos) {
              el.css(pos, function (_, str) {
                var val = parseInt(str, 10), toRef = idx ? el.to.left : el.to.top;
                // if original was "auto", recalculate the new value from wrapper
                if (str === 'auto') {
                  return toRef + 'px';
                }
                return val + toRef + 'px';
              });
            });
          }
        }
        $.effects.removeWrapper(el);
        done();
      }
    });
  };
}(jQuery));
(function ($, undefined) {
  $.effects.effect.shake = function (o, done) {
    var el = $(this), props = [
        'position',
        'top',
        'bottom',
        'left',
        'right',
        'height',
        'width'
      ], mode = $.effects.setMode(el, o.mode || 'effect'), direction = o.direction || 'left', distance = o.distance || 20, times = o.times || 3, anims = times * 2 + 1, speed = Math.round(o.duration / anims), ref = direction === 'up' || direction === 'down' ? 'top' : 'left', positiveMotion = direction === 'up' || direction === 'left', animation = {}, animation1 = {}, animation2 = {}, i,
      // we will need to re-assemble the queue to stack our animations in place
      queue = el.queue(), queuelen = queue.length;
    $.effects.save(el, props);
    el.show();
    $.effects.createWrapper(el);
    // Animation
    animation[ref] = (positiveMotion ? '-=' : '+=') + distance;
    animation1[ref] = (positiveMotion ? '+=' : '-=') + distance * 2;
    animation2[ref] = (positiveMotion ? '-=' : '+=') + distance * 2;
    // Animate
    el.animate(animation, speed, o.easing);
    // Shakes
    for (i = 1; i < times; i++) {
      el.animate(animation1, speed, o.easing).animate(animation2, speed, o.easing);
    }
    el.animate(animation1, speed, o.easing).animate(animation, speed / 2, o.easing).queue(function () {
      if (mode === 'hide') {
        el.hide();
      }
      $.effects.restore(el, props);
      $.effects.removeWrapper(el);
      done();
    });
    // inject all the animations we just queued to be first in line (after "inprogress")
    if (queuelen > 1) {
      queue.splice.apply(queue, [
        1,
        0
      ].concat(queue.splice(queuelen, anims + 1)));
    }
    el.dequeue();
  };
}(jQuery));
(function ($, undefined) {
  $.effects.effect.slide = function (o, done) {
    // Create element
    var el = $(this), props = [
        'position',
        'top',
        'bottom',
        'left',
        'right',
        'width',
        'height'
      ], mode = $.effects.setMode(el, o.mode || 'show'), show = mode === 'show', direction = o.direction || 'left', ref = direction === 'up' || direction === 'down' ? 'top' : 'left', positiveMotion = direction === 'up' || direction === 'left', distance, animation = {};
    // Adjust
    $.effects.save(el, props);
    el.show();
    distance = o.distance || el[ref === 'top' ? 'outerHeight' : 'outerWidth'](true);
    $.effects.createWrapper(el).css({ overflow: 'hidden' });
    if (show) {
      el.css(ref, positiveMotion ? isNaN(distance) ? '-' + distance : -distance : distance);
    }
    // Animation
    animation[ref] = (show ? positiveMotion ? '+=' : '-=' : positiveMotion ? '-=' : '+=') + distance;
    // Animate
    el.animate(animation, {
      queue: false,
      duration: o.duration,
      easing: o.easing,
      complete: function () {
        if (mode === 'hide') {
          el.hide();
        }
        $.effects.restore(el, props);
        $.effects.removeWrapper(el);
        done();
      }
    });
  };
}(jQuery));
(function ($, undefined) {
  $.effects.effect.transfer = function (o, done) {
    var elem = $(this), target = $(o.to), targetFixed = target.css('position') === 'fixed', body = $('body'), fixTop = targetFixed ? body.scrollTop() : 0, fixLeft = targetFixed ? body.scrollLeft() : 0, endPosition = target.offset(), animation = {
        top: endPosition.top - fixTop,
        left: endPosition.left - fixLeft,
        height: target.innerHeight(),
        width: target.innerWidth()
      }, startPosition = elem.offset(), transfer = $('<div class=\'ui-effects-transfer\'></div>').appendTo(document.body).addClass(o.className).css({
        top: startPosition.top - fixTop,
        left: startPosition.left - fixLeft,
        height: elem.innerHeight(),
        width: elem.innerWidth(),
        position: targetFixed ? 'fixed' : 'absolute'
      }).animate(animation, o.duration, o.easing, function () {
        transfer.remove();
        done();
      });
  };
}(jQuery));
(function ($, undefined) {
  $.widget('ui.menu', {
    version: '1.10.4',
    defaultElement: '<ul>',
    delay: 300,
    options: {
      icons: { submenu: 'ui-icon-carat-1-e' },
      menus: 'ul',
      position: {
        my: 'left top',
        at: 'right top'
      },
      role: 'menu',
      blur: null,
      focus: null,
      select: null
    },
    _create: function () {
      this.activeMenu = this.element;
      // flag used to prevent firing of the click handler
      // as the event bubbles up through nested menus
      this.mouseHandled = false;
      this.element.uniqueId().addClass('ui-menu ui-widget ui-widget-content ui-corner-all').toggleClass('ui-menu-icons', !!this.element.find('.ui-icon').length).attr({
        role: this.options.role,
        tabIndex: 0
      }).bind('click' + this.eventNamespace, $.proxy(function (event) {
        if (this.options.disabled) {
          event.preventDefault();
        }
      }, this));
      if (this.options.disabled) {
        this.element.addClass('ui-state-disabled').attr('aria-disabled', 'true');
      }
      this._on({
        'mousedown .ui-menu-item > a': function (event) {
          event.preventDefault();
        },
        'click .ui-state-disabled > a': function (event) {
          event.preventDefault();
        },
        'click .ui-menu-item:has(a)': function (event) {
          var target = $(event.target).closest('.ui-menu-item');
          if (!this.mouseHandled && target.not('.ui-state-disabled').length) {
            this.select(event);
            // Only set the mouseHandled flag if the event will bubble, see #9469.
            if (!event.isPropagationStopped()) {
              this.mouseHandled = true;
            }
            // Open submenu on click
            if (target.has('.ui-menu').length) {
              this.expand(event);
            } else if (!this.element.is(':focus') && $(this.document[0].activeElement).closest('.ui-menu').length) {
              // Redirect focus to the menu
              this.element.trigger('focus', [true]);
              // If the active item is on the top level, let it stay active.
              // Otherwise, blur the active item since it is no longer visible.
              if (this.active && this.active.parents('.ui-menu').length === 1) {
                clearTimeout(this.timer);
              }
            }
          }
        },
        'mouseenter .ui-menu-item': function (event) {
          var target = $(event.currentTarget);
          // Remove ui-state-active class from siblings of the newly focused menu item
          // to avoid a jump caused by adjacent elements both having a class with a border
          target.siblings().children('.ui-state-active').removeClass('ui-state-active');
          this.focus(event, target);
        },
        mouseleave: 'collapseAll',
        'mouseleave .ui-menu': 'collapseAll',
        focus: function (event, keepActiveItem) {
          // If there's already an active item, keep it active
          // If not, activate the first item
          var item = this.active || this.element.children('.ui-menu-item').eq(0);
          if (!keepActiveItem) {
            this.focus(event, item);
          }
        },
        blur: function (event) {
          this._delay(function () {
            if (!$.contains(this.element[0], this.document[0].activeElement)) {
              this.collapseAll(event);
            }
          });
        },
        keydown: '_keydown'
      });
      this.refresh();
      // Clicks outside of a menu collapse any open menus
      this._on(this.document, {
        click: function (event) {
          if (!$(event.target).closest('.ui-menu').length) {
            this.collapseAll(event);
          }
          // Reset the mouseHandled flag
          this.mouseHandled = false;
        }
      });
    },
    _destroy: function () {
      // Destroy (sub)menus
      this.element.removeAttr('aria-activedescendant').find('.ui-menu').addBack().removeClass('ui-menu ui-widget ui-widget-content ui-corner-all ui-menu-icons').removeAttr('role').removeAttr('tabIndex').removeAttr('aria-labelledby').removeAttr('aria-expanded').removeAttr('aria-hidden').removeAttr('aria-disabled').removeUniqueId().show();
      // Destroy menu items
      this.element.find('.ui-menu-item').removeClass('ui-menu-item').removeAttr('role').removeAttr('aria-disabled').children('a').removeUniqueId().removeClass('ui-corner-all ui-state-hover').removeAttr('tabIndex').removeAttr('role').removeAttr('aria-haspopup').children().each(function () {
        var elem = $(this);
        if (elem.data('ui-menu-submenu-carat')) {
          elem.remove();
        }
      });
      // Destroy menu dividers
      this.element.find('.ui-menu-divider').removeClass('ui-menu-divider ui-widget-content');
    },
    _keydown: function (event) {
      var match, prev, character, skip, regex, preventDefault = true;
      function escape(value) {
        return value.replace(/[\-\[\]{}()*+?.,\\\^$|#\s]/g, '\\$&');
      }
      switch (event.keyCode) {
      case $.ui.keyCode.PAGE_UP:
        this.previousPage(event);
        break;
      case $.ui.keyCode.PAGE_DOWN:
        this.nextPage(event);
        break;
      case $.ui.keyCode.HOME:
        this._move('first', 'first', event);
        break;
      case $.ui.keyCode.END:
        this._move('last', 'last', event);
        break;
      case $.ui.keyCode.UP:
        this.previous(event);
        break;
      case $.ui.keyCode.DOWN:
        this.next(event);
        break;
      case $.ui.keyCode.LEFT:
        this.collapse(event);
        break;
      case $.ui.keyCode.RIGHT:
        if (this.active && !this.active.is('.ui-state-disabled')) {
          this.expand(event);
        }
        break;
      case $.ui.keyCode.ENTER:
      case $.ui.keyCode.SPACE:
        this._activate(event);
        break;
      case $.ui.keyCode.ESCAPE:
        this.collapse(event);
        break;
      default:
        preventDefault = false;
        prev = this.previousFilter || '';
        character = String.fromCharCode(event.keyCode);
        skip = false;
        clearTimeout(this.filterTimer);
        if (character === prev) {
          skip = true;
        } else {
          character = prev + character;
        }
        regex = new RegExp('^' + escape(character), 'i');
        match = this.activeMenu.children('.ui-menu-item').filter(function () {
          return regex.test($(this).children('a').text());
        });
        match = skip && match.index(this.active.next()) !== -1 ? this.active.nextAll('.ui-menu-item') : match;
        // If no matches on the current filter, reset to the last character pressed
        // to move down the menu to the first item that starts with that character
        if (!match.length) {
          character = String.fromCharCode(event.keyCode);
          regex = new RegExp('^' + escape(character), 'i');
          match = this.activeMenu.children('.ui-menu-item').filter(function () {
            return regex.test($(this).children('a').text());
          });
        }
        if (match.length) {
          this.focus(event, match);
          if (match.length > 1) {
            this.previousFilter = character;
            this.filterTimer = this._delay(function () {
              delete this.previousFilter;
            }, 1000);
          } else {
            delete this.previousFilter;
          }
        } else {
          delete this.previousFilter;
        }
      }
      if (preventDefault) {
        event.preventDefault();
      }
    },
    _activate: function (event) {
      if (!this.active.is('.ui-state-disabled')) {
        if (this.active.children('a[aria-haspopup=\'true\']').length) {
          this.expand(event);
        } else {
          this.select(event);
        }
      }
    },
    refresh: function () {
      var menus, icon = this.options.icons.submenu, submenus = this.element.find(this.options.menus);
      this.element.toggleClass('ui-menu-icons', !!this.element.find('.ui-icon').length);
      // Initialize nested menus
      submenus.filter(':not(.ui-menu)').addClass('ui-menu ui-widget ui-widget-content ui-corner-all').hide().attr({
        role: this.options.role,
        'aria-hidden': 'true',
        'aria-expanded': 'false'
      }).each(function () {
        var menu = $(this), item = menu.prev('a'), submenuCarat = $('<span>').addClass('ui-menu-icon ui-icon ' + icon).data('ui-menu-submenu-carat', true);
        item.attr('aria-haspopup', 'true').prepend(submenuCarat);
        menu.attr('aria-labelledby', item.attr('id'));
      });
      menus = submenus.add(this.element);
      // Don't refresh list items that are already adapted
      menus.children(':not(.ui-menu-item):has(a)').addClass('ui-menu-item').attr('role', 'presentation').children('a').uniqueId().addClass('ui-corner-all').attr({
        tabIndex: -1,
        role: this._itemRole()
      });
      // Initialize unlinked menu-items containing spaces and/or dashes only as dividers
      menus.children(':not(.ui-menu-item)').each(function () {
        var item = $(this);
        // hyphen, em dash, en dash
        if (!/[^\-\u2014\u2013\s]/.test(item.text())) {
          item.addClass('ui-widget-content ui-menu-divider');
        }
      });
      // Add aria-disabled attribute to any disabled menu item
      menus.children('.ui-state-disabled').attr('aria-disabled', 'true');
      // If the active item has been removed, blur the menu
      if (this.active && !$.contains(this.element[0], this.active[0])) {
        this.blur();
      }
    },
    _itemRole: function () {
      return {
        menu: 'menuitem',
        listbox: 'option'
      }[this.options.role];
    },
    _setOption: function (key, value) {
      if (key === 'icons') {
        this.element.find('.ui-menu-icon').removeClass(this.options.icons.submenu).addClass(value.submenu);
      }
      this._super(key, value);
    },
    focus: function (event, item) {
      var nested, focused;
      this.blur(event, event && event.type === 'focus');
      this._scrollIntoView(item);
      this.active = item.first();
      focused = this.active.children('a').addClass('ui-state-focus');
      // Only update aria-activedescendant if there's a role
      // otherwise we assume focus is managed elsewhere
      if (this.options.role) {
        this.element.attr('aria-activedescendant', focused.attr('id'));
      }
      // Highlight active parent menu item, if any
      this.active.parent().closest('.ui-menu-item').children('a:first').addClass('ui-state-active');
      if (event && event.type === 'keydown') {
        this._close();
      } else {
        this.timer = this._delay(function () {
          this._close();
        }, this.delay);
      }
      nested = item.children('.ui-menu');
      if (nested.length && event && /^mouse/.test(event.type)) {
        this._startOpening(nested);
      }
      this.activeMenu = item.parent();
      this._trigger('focus', event, { item: item });
    },
    _scrollIntoView: function (item) {
      var borderTop, paddingTop, offset, scroll, elementHeight, itemHeight;
      if (this._hasScroll()) {
        borderTop = parseFloat($.css(this.activeMenu[0], 'borderTopWidth')) || 0;
        paddingTop = parseFloat($.css(this.activeMenu[0], 'paddingTop')) || 0;
        offset = item.offset().top - this.activeMenu.offset().top - borderTop - paddingTop;
        scroll = this.activeMenu.scrollTop();
        elementHeight = this.activeMenu.height();
        itemHeight = item.height();
        if (offset < 0) {
          this.activeMenu.scrollTop(scroll + offset);
        } else if (offset + itemHeight > elementHeight) {
          this.activeMenu.scrollTop(scroll + offset - elementHeight + itemHeight);
        }
      }
    },
    blur: function (event, fromFocus) {
      if (!fromFocus) {
        clearTimeout(this.timer);
      }
      if (!this.active) {
        return;
      }
      this.active.children('a').removeClass('ui-state-focus');
      this.active = null;
      this._trigger('blur', event, { item: this.active });
    },
    _startOpening: function (submenu) {
      clearTimeout(this.timer);
      // Don't open if already open fixes a Firefox bug that caused a .5 pixel
      // shift in the submenu position when mousing over the carat icon
      if (submenu.attr('aria-hidden') !== 'true') {
        return;
      }
      this.timer = this._delay(function () {
        this._close();
        this._open(submenu);
      }, this.delay);
    },
    _open: function (submenu) {
      var position = $.extend({ of: this.active }, this.options.position);
      clearTimeout(this.timer);
      this.element.find('.ui-menu').not(submenu.parents('.ui-menu')).hide().attr('aria-hidden', 'true');
      submenu.show().removeAttr('aria-hidden').attr('aria-expanded', 'true').position(position);
    },
    collapseAll: function (event, all) {
      clearTimeout(this.timer);
      this.timer = this._delay(function () {
        // If we were passed an event, look for the submenu that contains the event
        var currentMenu = all ? this.element : $(event && event.target).closest(this.element.find('.ui-menu'));
        // If we found no valid submenu ancestor, use the main menu to close all sub menus anyway
        if (!currentMenu.length) {
          currentMenu = this.element;
        }
        this._close(currentMenu);
        this.blur(event);
        this.activeMenu = currentMenu;
      }, this.delay);
    },
    _close: function (startMenu) {
      if (!startMenu) {
        startMenu = this.active ? this.active.parent() : this.element;
      }
      startMenu.find('.ui-menu').hide().attr('aria-hidden', 'true').attr('aria-expanded', 'false').end().find('a.ui-state-active').removeClass('ui-state-active');
    },
    collapse: function (event) {
      var newItem = this.active && this.active.parent().closest('.ui-menu-item', this.element);
      if (newItem && newItem.length) {
        this._close();
        this.focus(event, newItem);
      }
    },
    expand: function (event) {
      var newItem = this.active && this.active.children('.ui-menu ').children('.ui-menu-item').first();
      if (newItem && newItem.length) {
        this._open(newItem.parent());
        // Delay so Firefox will not hide activedescendant change in expanding submenu from AT
        this._delay(function () {
          this.focus(event, newItem);
        });
      }
    },
    next: function (event) {
      this._move('next', 'first', event);
    },
    previous: function (event) {
      this._move('prev', 'last', event);
    },
    isFirstItem: function () {
      return this.active && !this.active.prevAll('.ui-menu-item').length;
    },
    isLastItem: function () {
      return this.active && !this.active.nextAll('.ui-menu-item').length;
    },
    _move: function (direction, filter, event) {
      var next;
      if (this.active) {
        if (direction === 'first' || direction === 'last') {
          next = this.active[direction === 'first' ? 'prevAll' : 'nextAll']('.ui-menu-item').eq(-1);
        } else {
          next = this.active[direction + 'All']('.ui-menu-item').eq(0);
        }
      }
      if (!next || !next.length || !this.active) {
        next = this.activeMenu.children('.ui-menu-item')[filter]();
      }
      this.focus(event, next);
    },
    nextPage: function (event) {
      var item, base, height;
      if (!this.active) {
        this.next(event);
        return;
      }
      if (this.isLastItem()) {
        return;
      }
      if (this._hasScroll()) {
        base = this.active.offset().top;
        height = this.element.height();
        this.active.nextAll('.ui-menu-item').each(function () {
          item = $(this);
          return item.offset().top - base - height < 0;
        });
        this.focus(event, item);
      } else {
        this.focus(event, this.activeMenu.children('.ui-menu-item')[!this.active ? 'first' : 'last']());
      }
    },
    previousPage: function (event) {
      var item, base, height;
      if (!this.active) {
        this.next(event);
        return;
      }
      if (this.isFirstItem()) {
        return;
      }
      if (this._hasScroll()) {
        base = this.active.offset().top;
        height = this.element.height();
        this.active.prevAll('.ui-menu-item').each(function () {
          item = $(this);
          return item.offset().top - base + height > 0;
        });
        this.focus(event, item);
      } else {
        this.focus(event, this.activeMenu.children('.ui-menu-item').first());
      }
    },
    _hasScroll: function () {
      return this.element.outerHeight() < this.element.prop('scrollHeight');
    },
    select: function (event) {
      // TODO: It should never be possible to not have an active item at this
      // point, but the tests don't trigger mouseenter before click.
      this.active = this.active || $(event.target).closest('.ui-menu-item');
      var ui = { item: this.active };
      if (!this.active.has('.ui-menu').length) {
        this.collapseAll(event, true);
      }
      this._trigger('select', event, ui);
    }
  });
}(jQuery));
(function ($, undefined) {
  $.widget('ui.progressbar', {
    version: '1.10.4',
    options: {
      max: 100,
      value: 0,
      change: null,
      complete: null
    },
    min: 0,
    _create: function () {
      // Constrain initial value
      this.oldValue = this.options.value = this._constrainedValue();
      this.element.addClass('ui-progressbar ui-widget ui-widget-content ui-corner-all').attr({
        role: 'progressbar',
        'aria-valuemin': this.min
      });
      this.valueDiv = $('<div class=\'ui-progressbar-value ui-widget-header ui-corner-left\'></div>').appendTo(this.element);
      this._refreshValue();
    },
    _destroy: function () {
      this.element.removeClass('ui-progressbar ui-widget ui-widget-content ui-corner-all').removeAttr('role').removeAttr('aria-valuemin').removeAttr('aria-valuemax').removeAttr('aria-valuenow');
      this.valueDiv.remove();
    },
    value: function (newValue) {
      if (newValue === undefined) {
        return this.options.value;
      }
      this.options.value = this._constrainedValue(newValue);
      this._refreshValue();
    },
    _constrainedValue: function (newValue) {
      if (newValue === undefined) {
        newValue = this.options.value;
      }
      this.indeterminate = newValue === false;
      // sanitize value
      if (typeof newValue !== 'number') {
        newValue = 0;
      }
      return this.indeterminate ? false : Math.min(this.options.max, Math.max(this.min, newValue));
    },
    _setOptions: function (options) {
      // Ensure "value" option is set after other values (like max)
      var value = options.value;
      delete options.value;
      this._super(options);
      this.options.value = this._constrainedValue(value);
      this._refreshValue();
    },
    _setOption: function (key, value) {
      if (key === 'max') {
        // Don't allow a max less than min
        value = Math.max(this.min, value);
      }
      this._super(key, value);
    },
    _percentage: function () {
      return this.indeterminate ? 100 : 100 * (this.options.value - this.min) / (this.options.max - this.min);
    },
    _refreshValue: function () {
      var value = this.options.value, percentage = this._percentage();
      this.valueDiv.toggle(this.indeterminate || value > this.min).toggleClass('ui-corner-right', value === this.options.max).width(percentage.toFixed(0) + '%');
      this.element.toggleClass('ui-progressbar-indeterminate', this.indeterminate);
      if (this.indeterminate) {
        this.element.removeAttr('aria-valuenow');
        if (!this.overlayDiv) {
          this.overlayDiv = $('<div class=\'ui-progressbar-overlay\'></div>').appendTo(this.valueDiv);
        }
      } else {
        this.element.attr({
          'aria-valuemax': this.options.max,
          'aria-valuenow': value
        });
        if (this.overlayDiv) {
          this.overlayDiv.remove();
          this.overlayDiv = null;
        }
      }
      if (this.oldValue !== value) {
        this.oldValue = value;
        this._trigger('change');
      }
      if (value === this.options.max) {
        this._trigger('complete');
      }
    }
  });
}(jQuery));
(function ($, undefined) {
  function num(v) {
    return parseInt(v, 10) || 0;
  }
  function isNumber(value) {
    return !isNaN(parseInt(value, 10));
  }
  $.widget('ui.resizable', $.ui.mouse, {
    version: '1.10.4',
    widgetEventPrefix: 'resize',
    options: {
      alsoResize: false,
      animate: false,
      animateDuration: 'slow',
      animateEasing: 'swing',
      aspectRatio: false,
      autoHide: false,
      containment: false,
      ghost: false,
      grid: false,
      handles: 'e,s,se',
      helper: false,
      maxHeight: null,
      maxWidth: null,
      minHeight: 10,
      minWidth: 10,
      zIndex: 90,
      resize: null,
      start: null,
      stop: null
    },
    _create: function () {
      var n, i, handle, axis, hname, that = this, o = this.options;
      this.element.addClass('ui-resizable');
      $.extend(this, {
        _aspectRatio: !!o.aspectRatio,
        aspectRatio: o.aspectRatio,
        originalElement: this.element,
        _proportionallyResizeElements: [],
        _helper: o.helper || o.ghost || o.animate ? o.helper || 'ui-resizable-helper' : null
      });
      //Wrap the element if it cannot hold child nodes
      if (this.element[0].nodeName.match(/canvas|textarea|input|select|button|img/i)) {
        //Create a wrapper element and set the wrapper to the new current internal element
        this.element.wrap($('<div class=\'ui-wrapper\' style=\'overflow: hidden;\'></div>').css({
          position: this.element.css('position'),
          width: this.element.outerWidth(),
          height: this.element.outerHeight(),
          top: this.element.css('top'),
          left: this.element.css('left')
        }));
        //Overwrite the original this.element
        this.element = this.element.parent().data('ui-resizable', this.element.data('ui-resizable'));
        this.elementIsWrapper = true;
        //Move margins to the wrapper
        this.element.css({
          marginLeft: this.originalElement.css('marginLeft'),
          marginTop: this.originalElement.css('marginTop'),
          marginRight: this.originalElement.css('marginRight'),
          marginBottom: this.originalElement.css('marginBottom')
        });
        this.originalElement.css({
          marginLeft: 0,
          marginTop: 0,
          marginRight: 0,
          marginBottom: 0
        });
        //Prevent Safari textarea resize
        this.originalResizeStyle = this.originalElement.css('resize');
        this.originalElement.css('resize', 'none');
        //Push the actual element to our proportionallyResize internal array
        this._proportionallyResizeElements.push(this.originalElement.css({
          position: 'static',
          zoom: 1,
          display: 'block'
        }));
        // avoid IE jump (hard set the margin)
        this.originalElement.css({ margin: this.originalElement.css('margin') });
        // fix handlers offset
        this._proportionallyResize();
      }
      this.handles = o.handles || (!$('.ui-resizable-handle', this.element).length ? 'e,s,se' : {
        n: '.ui-resizable-n',
        e: '.ui-resizable-e',
        s: '.ui-resizable-s',
        w: '.ui-resizable-w',
        se: '.ui-resizable-se',
        sw: '.ui-resizable-sw',
        ne: '.ui-resizable-ne',
        nw: '.ui-resizable-nw'
      });
      if (this.handles.constructor === String) {
        if (this.handles === 'all') {
          this.handles = 'n,e,s,w,se,sw,ne,nw';
        }
        n = this.handles.split(',');
        this.handles = {};
        for (i = 0; i < n.length; i++) {
          handle = $.trim(n[i]);
          hname = 'ui-resizable-' + handle;
          axis = $('<div class=\'ui-resizable-handle ' + hname + '\'></div>');
          // Apply zIndex to all handles - see #7960
          axis.css({ zIndex: o.zIndex });
          //TODO : What's going on here?
          if ('se' === handle) {
            axis.addClass('ui-icon ui-icon-gripsmall-diagonal-se');
          }
          //Insert into internal handles object and append to element
          this.handles[handle] = '.ui-resizable-' + handle;
          this.element.append(axis);
        }
      }
      this._renderAxis = function (target) {
        var i, axis, padPos, padWrapper;
        target = target || this.element;
        for (i in this.handles) {
          if (this.handles[i].constructor === String) {
            this.handles[i] = $(this.handles[i], this.element).show();
          }
          //Apply pad to wrapper element, needed to fix axis position (textarea, inputs, scrolls)
          if (this.elementIsWrapper && this.originalElement[0].nodeName.match(/textarea|input|select|button/i)) {
            axis = $(this.handles[i], this.element);
            //Checking the correct pad and border
            padWrapper = /sw|ne|nw|se|n|s/.test(i) ? axis.outerHeight() : axis.outerWidth();
            //The padding type i have to apply...
            padPos = [
              'padding',
              /ne|nw|n/.test(i) ? 'Top' : /se|sw|s/.test(i) ? 'Bottom' : /^e$/.test(i) ? 'Right' : 'Left'
            ].join('');
            target.css(padPos, padWrapper);
            this._proportionallyResize();
          }
          //TODO: What's that good for? There's not anything to be executed left
          if (!$(this.handles[i]).length) {
            continue;
          }
        }
      };
      //TODO: make renderAxis a prototype function
      this._renderAxis(this.element);
      this._handles = $('.ui-resizable-handle', this.element).disableSelection();
      //Matching axis name
      this._handles.mouseover(function () {
        if (!that.resizing) {
          if (this.className) {
            axis = this.className.match(/ui-resizable-(se|sw|ne|nw|n|e|s|w)/i);
          }
          //Axis, default = se
          that.axis = axis && axis[1] ? axis[1] : 'se';
        }
      });
      //If we want to auto hide the elements
      if (o.autoHide) {
        this._handles.hide();
        $(this.element).addClass('ui-resizable-autohide').mouseenter(function () {
          if (o.disabled) {
            return;
          }
          $(this).removeClass('ui-resizable-autohide');
          that._handles.show();
        }).mouseleave(function () {
          if (o.disabled) {
            return;
          }
          if (!that.resizing) {
            $(this).addClass('ui-resizable-autohide');
            that._handles.hide();
          }
        });
      }
      //Initialize the mouse interaction
      this._mouseInit();
    },
    _destroy: function () {
      this._mouseDestroy();
      var wrapper, _destroy = function (exp) {
          $(exp).removeClass('ui-resizable ui-resizable-disabled ui-resizable-resizing').removeData('resizable').removeData('ui-resizable').unbind('.resizable').find('.ui-resizable-handle').remove();
        };
      //TODO: Unwrap at same DOM position
      if (this.elementIsWrapper) {
        _destroy(this.element);
        wrapper = this.element;
        this.originalElement.css({
          position: wrapper.css('position'),
          width: wrapper.outerWidth(),
          height: wrapper.outerHeight(),
          top: wrapper.css('top'),
          left: wrapper.css('left')
        }).insertAfter(wrapper);
        wrapper.remove();
      }
      this.originalElement.css('resize', this.originalResizeStyle);
      _destroy(this.originalElement);
      return this;
    },
    _mouseCapture: function (event) {
      var i, handle, capture = false;
      for (i in this.handles) {
        handle = $(this.handles[i])[0];
        if (handle === event.target || $.contains(handle, event.target)) {
          capture = true;
        }
      }
      return !this.options.disabled && capture;
    },
    _mouseStart: function (event) {
      var curleft, curtop, cursor, o = this.options, iniPos = this.element.position(), el = this.element;
      this.resizing = true;
      // bugfix for http://dev.jquery.com/ticket/1749
      if (/absolute/.test(el.css('position'))) {
        el.css({
          position: 'absolute',
          top: el.css('top'),
          left: el.css('left')
        });
      } else if (el.is('.ui-draggable')) {
        el.css({
          position: 'absolute',
          top: iniPos.top,
          left: iniPos.left
        });
      }
      this._renderProxy();
      curleft = num(this.helper.css('left'));
      curtop = num(this.helper.css('top'));
      if (o.containment) {
        curleft += $(o.containment).scrollLeft() || 0;
        curtop += $(o.containment).scrollTop() || 0;
      }
      //Store needed variables
      this.offset = this.helper.offset();
      this.position = {
        left: curleft,
        top: curtop
      };
      this.size = this._helper ? {
        width: this.helper.width(),
        height: this.helper.height()
      } : {
        width: el.width(),
        height: el.height()
      };
      this.originalSize = this._helper ? {
        width: el.outerWidth(),
        height: el.outerHeight()
      } : {
        width: el.width(),
        height: el.height()
      };
      this.originalPosition = {
        left: curleft,
        top: curtop
      };
      this.sizeDiff = {
        width: el.outerWidth() - el.width(),
        height: el.outerHeight() - el.height()
      };
      this.originalMousePosition = {
        left: event.pageX,
        top: event.pageY
      };
      //Aspect Ratio
      this.aspectRatio = typeof o.aspectRatio === 'number' ? o.aspectRatio : this.originalSize.width / this.originalSize.height || 1;
      cursor = $('.ui-resizable-' + this.axis).css('cursor');
      $('body').css('cursor', cursor === 'auto' ? this.axis + '-resize' : cursor);
      el.addClass('ui-resizable-resizing');
      this._propagate('start', event);
      return true;
    },
    _mouseDrag: function (event) {
      //Increase performance, avoid regex
      var data, el = this.helper, props = {}, smp = this.originalMousePosition, a = this.axis, prevTop = this.position.top, prevLeft = this.position.left, prevWidth = this.size.width, prevHeight = this.size.height, dx = event.pageX - smp.left || 0, dy = event.pageY - smp.top || 0, trigger = this._change[a];
      if (!trigger) {
        return false;
      }
      // Calculate the attrs that will be change
      data = trigger.apply(this, [
        event,
        dx,
        dy
      ]);
      // Put this in the mouseDrag handler since the user can start pressing shift while resizing
      this._updateVirtualBoundaries(event.shiftKey);
      if (this._aspectRatio || event.shiftKey) {
        data = this._updateRatio(data, event);
      }
      data = this._respectSize(data, event);
      this._updateCache(data);
      // plugins callbacks need to be called first
      this._propagate('resize', event);
      if (this.position.top !== prevTop) {
        props.top = this.position.top + 'px';
      }
      if (this.position.left !== prevLeft) {
        props.left = this.position.left + 'px';
      }
      if (this.size.width !== prevWidth) {
        props.width = this.size.width + 'px';
      }
      if (this.size.height !== prevHeight) {
        props.height = this.size.height + 'px';
      }
      el.css(props);
      if (!this._helper && this._proportionallyResizeElements.length) {
        this._proportionallyResize();
      }
      // Call the user callback if the element was resized
      if (!$.isEmptyObject(props)) {
        this._trigger('resize', event, this.ui());
      }
      return false;
    },
    _mouseStop: function (event) {
      this.resizing = false;
      var pr, ista, soffseth, soffsetw, s, left, top, o = this.options, that = this;
      if (this._helper) {
        pr = this._proportionallyResizeElements;
        ista = pr.length && /textarea/i.test(pr[0].nodeName);
        soffseth = ista && $.ui.hasScroll(pr[0], 'left') ? 0 : that.sizeDiff.height;
        soffsetw = ista ? 0 : that.sizeDiff.width;
        s = {
          width: that.helper.width() - soffsetw,
          height: that.helper.height() - soffseth
        };
        left = parseInt(that.element.css('left'), 10) + (that.position.left - that.originalPosition.left) || null;
        top = parseInt(that.element.css('top'), 10) + (that.position.top - that.originalPosition.top) || null;
        if (!o.animate) {
          this.element.css($.extend(s, {
            top: top,
            left: left
          }));
        }
        that.helper.height(that.size.height);
        that.helper.width(that.size.width);
        if (this._helper && !o.animate) {
          this._proportionallyResize();
        }
      }
      $('body').css('cursor', 'auto');
      this.element.removeClass('ui-resizable-resizing');
      this._propagate('stop', event);
      if (this._helper) {
        this.helper.remove();
      }
      return false;
    },
    _updateVirtualBoundaries: function (forceAspectRatio) {
      var pMinWidth, pMaxWidth, pMinHeight, pMaxHeight, b, o = this.options;
      b = {
        minWidth: isNumber(o.minWidth) ? o.minWidth : 0,
        maxWidth: isNumber(o.maxWidth) ? o.maxWidth : Infinity,
        minHeight: isNumber(o.minHeight) ? o.minHeight : 0,
        maxHeight: isNumber(o.maxHeight) ? o.maxHeight : Infinity
      };
      if (this._aspectRatio || forceAspectRatio) {
        // We want to create an enclosing box whose aspect ration is the requested one
        // First, compute the "projected" size for each dimension based on the aspect ratio and other dimension
        pMinWidth = b.minHeight * this.aspectRatio;
        pMinHeight = b.minWidth / this.aspectRatio;
        pMaxWidth = b.maxHeight * this.aspectRatio;
        pMaxHeight = b.maxWidth / this.aspectRatio;
        if (pMinWidth > b.minWidth) {
          b.minWidth = pMinWidth;
        }
        if (pMinHeight > b.minHeight) {
          b.minHeight = pMinHeight;
        }
        if (pMaxWidth < b.maxWidth) {
          b.maxWidth = pMaxWidth;
        }
        if (pMaxHeight < b.maxHeight) {
          b.maxHeight = pMaxHeight;
        }
      }
      this._vBoundaries = b;
    },
    _updateCache: function (data) {
      this.offset = this.helper.offset();
      if (isNumber(data.left)) {
        this.position.left = data.left;
      }
      if (isNumber(data.top)) {
        this.position.top = data.top;
      }
      if (isNumber(data.height)) {
        this.size.height = data.height;
      }
      if (isNumber(data.width)) {
        this.size.width = data.width;
      }
    },
    _updateRatio: function (data) {
      var cpos = this.position, csize = this.size, a = this.axis;
      if (isNumber(data.height)) {
        data.width = data.height * this.aspectRatio;
      } else if (isNumber(data.width)) {
        data.height = data.width / this.aspectRatio;
      }
      if (a === 'sw') {
        data.left = cpos.left + (csize.width - data.width);
        data.top = null;
      }
      if (a === 'nw') {
        data.top = cpos.top + (csize.height - data.height);
        data.left = cpos.left + (csize.width - data.width);
      }
      return data;
    },
    _respectSize: function (data) {
      var o = this._vBoundaries, a = this.axis, ismaxw = isNumber(data.width) && o.maxWidth && o.maxWidth < data.width, ismaxh = isNumber(data.height) && o.maxHeight && o.maxHeight < data.height, isminw = isNumber(data.width) && o.minWidth && o.minWidth > data.width, isminh = isNumber(data.height) && o.minHeight && o.minHeight > data.height, dw = this.originalPosition.left + this.originalSize.width, dh = this.position.top + this.size.height, cw = /sw|nw|w/.test(a), ch = /nw|ne|n/.test(a);
      if (isminw) {
        data.width = o.minWidth;
      }
      if (isminh) {
        data.height = o.minHeight;
      }
      if (ismaxw) {
        data.width = o.maxWidth;
      }
      if (ismaxh) {
        data.height = o.maxHeight;
      }
      if (isminw && cw) {
        data.left = dw - o.minWidth;
      }
      if (ismaxw && cw) {
        data.left = dw - o.maxWidth;
      }
      if (isminh && ch) {
        data.top = dh - o.minHeight;
      }
      if (ismaxh && ch) {
        data.top = dh - o.maxHeight;
      }
      // fixing jump error on top/left - bug #2330
      if (!data.width && !data.height && !data.left && data.top) {
        data.top = null;
      } else if (!data.width && !data.height && !data.top && data.left) {
        data.left = null;
      }
      return data;
    },
    _proportionallyResize: function () {
      if (!this._proportionallyResizeElements.length) {
        return;
      }
      var i, j, borders, paddings, prel, element = this.helper || this.element;
      for (i = 0; i < this._proportionallyResizeElements.length; i++) {
        prel = this._proportionallyResizeElements[i];
        if (!this.borderDif) {
          this.borderDif = [];
          borders = [
            prel.css('borderTopWidth'),
            prel.css('borderRightWidth'),
            prel.css('borderBottomWidth'),
            prel.css('borderLeftWidth')
          ];
          paddings = [
            prel.css('paddingTop'),
            prel.css('paddingRight'),
            prel.css('paddingBottom'),
            prel.css('paddingLeft')
          ];
          for (j = 0; j < borders.length; j++) {
            this.borderDif[j] = (parseInt(borders[j], 10) || 0) + (parseInt(paddings[j], 10) || 0);
          }
        }
        prel.css({
          height: element.height() - this.borderDif[0] - this.borderDif[2] || 0,
          width: element.width() - this.borderDif[1] - this.borderDif[3] || 0
        });
      }
    },
    _renderProxy: function () {
      var el = this.element, o = this.options;
      this.elementOffset = el.offset();
      if (this._helper) {
        this.helper = this.helper || $('<div style=\'overflow:hidden;\'></div>');
        this.helper.addClass(this._helper).css({
          width: this.element.outerWidth() - 1,
          height: this.element.outerHeight() - 1,
          position: 'absolute',
          left: this.elementOffset.left + 'px',
          top: this.elementOffset.top + 'px',
          zIndex: ++o.zIndex
        });
        this.helper.appendTo('body').disableSelection();
      } else {
        this.helper = this.element;
      }
    },
    _change: {
      e: function (event, dx) {
        return { width: this.originalSize.width + dx };
      },
      w: function (event, dx) {
        var cs = this.originalSize, sp = this.originalPosition;
        return {
          left: sp.left + dx,
          width: cs.width - dx
        };
      },
      n: function (event, dx, dy) {
        var cs = this.originalSize, sp = this.originalPosition;
        return {
          top: sp.top + dy,
          height: cs.height - dy
        };
      },
      s: function (event, dx, dy) {
        return { height: this.originalSize.height + dy };
      },
      se: function (event, dx, dy) {
        return $.extend(this._change.s.apply(this, arguments), this._change.e.apply(this, [
          event,
          dx,
          dy
        ]));
      },
      sw: function (event, dx, dy) {
        return $.extend(this._change.s.apply(this, arguments), this._change.w.apply(this, [
          event,
          dx,
          dy
        ]));
      },
      ne: function (event, dx, dy) {
        return $.extend(this._change.n.apply(this, arguments), this._change.e.apply(this, [
          event,
          dx,
          dy
        ]));
      },
      nw: function (event, dx, dy) {
        return $.extend(this._change.n.apply(this, arguments), this._change.w.apply(this, [
          event,
          dx,
          dy
        ]));
      }
    },
    _propagate: function (n, event) {
      $.ui.plugin.call(this, n, [
        event,
        this.ui()
      ]);
      n !== 'resize' && this._trigger(n, event, this.ui());
    },
    plugins: {},
    ui: function () {
      return {
        originalElement: this.originalElement,
        element: this.element,
        helper: this.helper,
        position: this.position,
        size: this.size,
        originalSize: this.originalSize,
        originalPosition: this.originalPosition
      };
    }
  });
  /*
 * Resizable Extensions
 */
  $.ui.plugin.add('resizable', 'animate', {
    stop: function (event) {
      var that = $(this).data('ui-resizable'), o = that.options, pr = that._proportionallyResizeElements, ista = pr.length && /textarea/i.test(pr[0].nodeName), soffseth = ista && $.ui.hasScroll(pr[0], 'left') ? 0 : that.sizeDiff.height, soffsetw = ista ? 0 : that.sizeDiff.width, style = {
          width: that.size.width - soffsetw,
          height: that.size.height - soffseth
        }, left = parseInt(that.element.css('left'), 10) + (that.position.left - that.originalPosition.left) || null, top = parseInt(that.element.css('top'), 10) + (that.position.top - that.originalPosition.top) || null;
      that.element.animate($.extend(style, top && left ? {
        top: top,
        left: left
      } : {}), {
        duration: o.animateDuration,
        easing: o.animateEasing,
        step: function () {
          var data = {
              width: parseInt(that.element.css('width'), 10),
              height: parseInt(that.element.css('height'), 10),
              top: parseInt(that.element.css('top'), 10),
              left: parseInt(that.element.css('left'), 10)
            };
          if (pr && pr.length) {
            $(pr[0]).css({
              width: data.width,
              height: data.height
            });
          }
          // propagating resize, and updating values for each animation step
          that._updateCache(data);
          that._propagate('resize', event);
        }
      });
    }
  });
  $.ui.plugin.add('resizable', 'containment', {
    start: function () {
      var element, p, co, ch, cw, width, height, that = $(this).data('ui-resizable'), o = that.options, el = that.element, oc = o.containment, ce = oc instanceof $ ? oc.get(0) : /parent/.test(oc) ? el.parent().get(0) : oc;
      if (!ce) {
        return;
      }
      that.containerElement = $(ce);
      if (/document/.test(oc) || oc === document) {
        that.containerOffset = {
          left: 0,
          top: 0
        };
        that.containerPosition = {
          left: 0,
          top: 0
        };
        that.parentData = {
          element: $(document),
          left: 0,
          top: 0,
          width: $(document).width(),
          height: $(document).height() || document.body.parentNode.scrollHeight
        };
      }  // i'm a node, so compute top, left, right, bottom
      else {
        element = $(ce);
        p = [];
        $([
          'Top',
          'Right',
          'Left',
          'Bottom'
        ]).each(function (i, name) {
          p[i] = num(element.css('padding' + name));
        });
        that.containerOffset = element.offset();
        that.containerPosition = element.position();
        that.containerSize = {
          height: element.innerHeight() - p[3],
          width: element.innerWidth() - p[1]
        };
        co = that.containerOffset;
        ch = that.containerSize.height;
        cw = that.containerSize.width;
        width = $.ui.hasScroll(ce, 'left') ? ce.scrollWidth : cw;
        height = $.ui.hasScroll(ce) ? ce.scrollHeight : ch;
        that.parentData = {
          element: ce,
          left: co.left,
          top: co.top,
          width: width,
          height: height
        };
      }
    },
    resize: function (event) {
      var woset, hoset, isParent, isOffsetRelative, that = $(this).data('ui-resizable'), o = that.options, co = that.containerOffset, cp = that.position, pRatio = that._aspectRatio || event.shiftKey, cop = {
          top: 0,
          left: 0
        }, ce = that.containerElement;
      if (ce[0] !== document && /static/.test(ce.css('position'))) {
        cop = co;
      }
      if (cp.left < (that._helper ? co.left : 0)) {
        that.size.width = that.size.width + (that._helper ? that.position.left - co.left : that.position.left - cop.left);
        if (pRatio) {
          that.size.height = that.size.width / that.aspectRatio;
        }
        that.position.left = o.helper ? co.left : 0;
      }
      if (cp.top < (that._helper ? co.top : 0)) {
        that.size.height = that.size.height + (that._helper ? that.position.top - co.top : that.position.top);
        if (pRatio) {
          that.size.width = that.size.height * that.aspectRatio;
        }
        that.position.top = that._helper ? co.top : 0;
      }
      that.offset.left = that.parentData.left + that.position.left;
      that.offset.top = that.parentData.top + that.position.top;
      woset = Math.abs((that._helper ? that.offset.left - cop.left : that.offset.left - cop.left) + that.sizeDiff.width);
      hoset = Math.abs((that._helper ? that.offset.top - cop.top : that.offset.top - co.top) + that.sizeDiff.height);
      isParent = that.containerElement.get(0) === that.element.parent().get(0);
      isOffsetRelative = /relative|absolute/.test(that.containerElement.css('position'));
      if (isParent && isOffsetRelative) {
        woset -= Math.abs(that.parentData.left);
      }
      if (woset + that.size.width >= that.parentData.width) {
        that.size.width = that.parentData.width - woset;
        if (pRatio) {
          that.size.height = that.size.width / that.aspectRatio;
        }
      }
      if (hoset + that.size.height >= that.parentData.height) {
        that.size.height = that.parentData.height - hoset;
        if (pRatio) {
          that.size.width = that.size.height * that.aspectRatio;
        }
      }
    },
    stop: function () {
      var that = $(this).data('ui-resizable'), o = that.options, co = that.containerOffset, cop = that.containerPosition, ce = that.containerElement, helper = $(that.helper), ho = helper.offset(), w = helper.outerWidth() - that.sizeDiff.width, h = helper.outerHeight() - that.sizeDiff.height;
      if (that._helper && !o.animate && /relative/.test(ce.css('position'))) {
        $(this).css({
          left: ho.left - cop.left - co.left,
          width: w,
          height: h
        });
      }
      if (that._helper && !o.animate && /static/.test(ce.css('position'))) {
        $(this).css({
          left: ho.left - cop.left - co.left,
          width: w,
          height: h
        });
      }
    }
  });
  $.ui.plugin.add('resizable', 'alsoResize', {
    start: function () {
      var that = $(this).data('ui-resizable'), o = that.options, _store = function (exp) {
          $(exp).each(function () {
            var el = $(this);
            el.data('ui-resizable-alsoresize', {
              width: parseInt(el.width(), 10),
              height: parseInt(el.height(), 10),
              left: parseInt(el.css('left'), 10),
              top: parseInt(el.css('top'), 10)
            });
          });
        };
      if (typeof o.alsoResize === 'object' && !o.alsoResize.parentNode) {
        if (o.alsoResize.length) {
          o.alsoResize = o.alsoResize[0];
          _store(o.alsoResize);
        } else {
          $.each(o.alsoResize, function (exp) {
            _store(exp);
          });
        }
      } else {
        _store(o.alsoResize);
      }
    },
    resize: function (event, ui) {
      var that = $(this).data('ui-resizable'), o = that.options, os = that.originalSize, op = that.originalPosition, delta = {
          height: that.size.height - os.height || 0,
          width: that.size.width - os.width || 0,
          top: that.position.top - op.top || 0,
          left: that.position.left - op.left || 0
        }, _alsoResize = function (exp, c) {
          $(exp).each(function () {
            var el = $(this), start = $(this).data('ui-resizable-alsoresize'), style = {}, css = c && c.length ? c : el.parents(ui.originalElement[0]).length ? [
                'width',
                'height'
              ] : [
                'width',
                'height',
                'top',
                'left'
              ];
            $.each(css, function (i, prop) {
              var sum = (start[prop] || 0) + (delta[prop] || 0);
              if (sum && sum >= 0) {
                style[prop] = sum || null;
              }
            });
            el.css(style);
          });
        };
      if (typeof o.alsoResize === 'object' && !o.alsoResize.nodeType) {
        $.each(o.alsoResize, function (exp, c) {
          _alsoResize(exp, c);
        });
      } else {
        _alsoResize(o.alsoResize);
      }
    },
    stop: function () {
      $(this).removeData('resizable-alsoresize');
    }
  });
  $.ui.plugin.add('resizable', 'ghost', {
    start: function () {
      var that = $(this).data('ui-resizable'), o = that.options, cs = that.size;
      that.ghost = that.originalElement.clone();
      that.ghost.css({
        opacity: 0.25,
        display: 'block',
        position: 'relative',
        height: cs.height,
        width: cs.width,
        margin: 0,
        left: 0,
        top: 0
      }).addClass('ui-resizable-ghost').addClass(typeof o.ghost === 'string' ? o.ghost : '');
      that.ghost.appendTo(that.helper);
    },
    resize: function () {
      var that = $(this).data('ui-resizable');
      if (that.ghost) {
        that.ghost.css({
          position: 'relative',
          height: that.size.height,
          width: that.size.width
        });
      }
    },
    stop: function () {
      var that = $(this).data('ui-resizable');
      if (that.ghost && that.helper) {
        that.helper.get(0).removeChild(that.ghost.get(0));
      }
    }
  });
  $.ui.plugin.add('resizable', 'grid', {
    resize: function () {
      var that = $(this).data('ui-resizable'), o = that.options, cs = that.size, os = that.originalSize, op = that.originalPosition, a = that.axis, grid = typeof o.grid === 'number' ? [
          o.grid,
          o.grid
        ] : o.grid, gridX = grid[0] || 1, gridY = grid[1] || 1, ox = Math.round((cs.width - os.width) / gridX) * gridX, oy = Math.round((cs.height - os.height) / gridY) * gridY, newWidth = os.width + ox, newHeight = os.height + oy, isMaxWidth = o.maxWidth && o.maxWidth < newWidth, isMaxHeight = o.maxHeight && o.maxHeight < newHeight, isMinWidth = o.minWidth && o.minWidth > newWidth, isMinHeight = o.minHeight && o.minHeight > newHeight;
      o.grid = grid;
      if (isMinWidth) {
        newWidth = newWidth + gridX;
      }
      if (isMinHeight) {
        newHeight = newHeight + gridY;
      }
      if (isMaxWidth) {
        newWidth = newWidth - gridX;
      }
      if (isMaxHeight) {
        newHeight = newHeight - gridY;
      }
      if (/^(se|s|e)$/.test(a)) {
        that.size.width = newWidth;
        that.size.height = newHeight;
      } else if (/^(ne)$/.test(a)) {
        that.size.width = newWidth;
        that.size.height = newHeight;
        that.position.top = op.top - oy;
      } else if (/^(sw)$/.test(a)) {
        that.size.width = newWidth;
        that.size.height = newHeight;
        that.position.left = op.left - ox;
      } else {
        if (newHeight - gridY > 0) {
          that.size.height = newHeight;
          that.position.top = op.top - oy;
        } else {
          that.size.height = gridY;
          that.position.top = op.top + os.height - gridY;
        }
        if (newWidth - gridX > 0) {
          that.size.width = newWidth;
          that.position.left = op.left - ox;
        } else {
          that.size.width = gridX;
          that.position.left = op.left + os.width - gridX;
        }
      }
    }
  });
}(jQuery));
(function ($, undefined) {
  $.widget('ui.selectable', $.ui.mouse, {
    version: '1.10.4',
    options: {
      appendTo: 'body',
      autoRefresh: true,
      distance: 0,
      filter: '*',
      tolerance: 'touch',
      selected: null,
      selecting: null,
      start: null,
      stop: null,
      unselected: null,
      unselecting: null
    },
    _create: function () {
      var selectees, that = this;
      this.element.addClass('ui-selectable');
      this.dragged = false;
      // cache selectee children based on filter
      this.refresh = function () {
        selectees = $(that.options.filter, that.element[0]);
        selectees.addClass('ui-selectee');
        selectees.each(function () {
          var $this = $(this), pos = $this.offset();
          $.data(this, 'selectable-item', {
            element: this,
            $element: $this,
            left: pos.left,
            top: pos.top,
            right: pos.left + $this.outerWidth(),
            bottom: pos.top + $this.outerHeight(),
            startselected: false,
            selected: $this.hasClass('ui-selected'),
            selecting: $this.hasClass('ui-selecting'),
            unselecting: $this.hasClass('ui-unselecting')
          });
        });
      };
      this.refresh();
      this.selectees = selectees.addClass('ui-selectee');
      this._mouseInit();
      this.helper = $('<div class=\'ui-selectable-helper\'></div>');
    },
    _destroy: function () {
      this.selectees.removeClass('ui-selectee').removeData('selectable-item');
      this.element.removeClass('ui-selectable ui-selectable-disabled');
      this._mouseDestroy();
    },
    _mouseStart: function (event) {
      var that = this, options = this.options;
      this.opos = [
        event.pageX,
        event.pageY
      ];
      if (this.options.disabled) {
        return;
      }
      this.selectees = $(options.filter, this.element[0]);
      this._trigger('start', event);
      $(options.appendTo).append(this.helper);
      // position helper (lasso)
      this.helper.css({
        'left': event.pageX,
        'top': event.pageY,
        'width': 0,
        'height': 0
      });
      if (options.autoRefresh) {
        this.refresh();
      }
      this.selectees.filter('.ui-selected').each(function () {
        var selectee = $.data(this, 'selectable-item');
        selectee.startselected = true;
        if (!event.metaKey && !event.ctrlKey) {
          selectee.$element.removeClass('ui-selected');
          selectee.selected = false;
          selectee.$element.addClass('ui-unselecting');
          selectee.unselecting = true;
          // selectable UNSELECTING callback
          that._trigger('unselecting', event, { unselecting: selectee.element });
        }
      });
      $(event.target).parents().addBack().each(function () {
        var doSelect, selectee = $.data(this, 'selectable-item');
        if (selectee) {
          doSelect = !event.metaKey && !event.ctrlKey || !selectee.$element.hasClass('ui-selected');
          selectee.$element.removeClass(doSelect ? 'ui-unselecting' : 'ui-selected').addClass(doSelect ? 'ui-selecting' : 'ui-unselecting');
          selectee.unselecting = !doSelect;
          selectee.selecting = doSelect;
          selectee.selected = doSelect;
          // selectable (UN)SELECTING callback
          if (doSelect) {
            that._trigger('selecting', event, { selecting: selectee.element });
          } else {
            that._trigger('unselecting', event, { unselecting: selectee.element });
          }
          return false;
        }
      });
    },
    _mouseDrag: function (event) {
      this.dragged = true;
      if (this.options.disabled) {
        return;
      }
      var tmp, that = this, options = this.options, x1 = this.opos[0], y1 = this.opos[1], x2 = event.pageX, y2 = event.pageY;
      if (x1 > x2) {
        tmp = x2;
        x2 = x1;
        x1 = tmp;
      }
      if (y1 > y2) {
        tmp = y2;
        y2 = y1;
        y1 = tmp;
      }
      this.helper.css({
        left: x1,
        top: y1,
        width: x2 - x1,
        height: y2 - y1
      });
      this.selectees.each(function () {
        var selectee = $.data(this, 'selectable-item'), hit = false;
        //prevent helper from being selected if appendTo: selectable
        if (!selectee || selectee.element === that.element[0]) {
          return;
        }
        if (options.tolerance === 'touch') {
          hit = !(selectee.left > x2 || selectee.right < x1 || selectee.top > y2 || selectee.bottom < y1);
        } else if (options.tolerance === 'fit') {
          hit = selectee.left > x1 && selectee.right < x2 && selectee.top > y1 && selectee.bottom < y2;
        }
        if (hit) {
          // SELECT
          if (selectee.selected) {
            selectee.$element.removeClass('ui-selected');
            selectee.selected = false;
          }
          if (selectee.unselecting) {
            selectee.$element.removeClass('ui-unselecting');
            selectee.unselecting = false;
          }
          if (!selectee.selecting) {
            selectee.$element.addClass('ui-selecting');
            selectee.selecting = true;
            // selectable SELECTING callback
            that._trigger('selecting', event, { selecting: selectee.element });
          }
        } else {
          // UNSELECT
          if (selectee.selecting) {
            if ((event.metaKey || event.ctrlKey) && selectee.startselected) {
              selectee.$element.removeClass('ui-selecting');
              selectee.selecting = false;
              selectee.$element.addClass('ui-selected');
              selectee.selected = true;
            } else {
              selectee.$element.removeClass('ui-selecting');
              selectee.selecting = false;
              if (selectee.startselected) {
                selectee.$element.addClass('ui-unselecting');
                selectee.unselecting = true;
              }
              // selectable UNSELECTING callback
              that._trigger('unselecting', event, { unselecting: selectee.element });
            }
          }
          if (selectee.selected) {
            if (!event.metaKey && !event.ctrlKey && !selectee.startselected) {
              selectee.$element.removeClass('ui-selected');
              selectee.selected = false;
              selectee.$element.addClass('ui-unselecting');
              selectee.unselecting = true;
              // selectable UNSELECTING callback
              that._trigger('unselecting', event, { unselecting: selectee.element });
            }
          }
        }
      });
      return false;
    },
    _mouseStop: function (event) {
      var that = this;
      this.dragged = false;
      $('.ui-unselecting', this.element[0]).each(function () {
        var selectee = $.data(this, 'selectable-item');
        selectee.$element.removeClass('ui-unselecting');
        selectee.unselecting = false;
        selectee.startselected = false;
        that._trigger('unselected', event, { unselected: selectee.element });
      });
      $('.ui-selecting', this.element[0]).each(function () {
        var selectee = $.data(this, 'selectable-item');
        selectee.$element.removeClass('ui-selecting').addClass('ui-selected');
        selectee.selecting = false;
        selectee.selected = true;
        selectee.startselected = true;
        that._trigger('selected', event, { selected: selectee.element });
      });
      this._trigger('stop', event);
      this.helper.remove();
      return false;
    }
  });
}(jQuery));
(function ($, undefined) {
  // number of pages in a slider
  // (how many times can you page up/down to go through the whole range)
  var numPages = 5;
  $.widget('ui.slider', $.ui.mouse, {
    version: '1.10.4',
    widgetEventPrefix: 'slide',
    options: {
      animate: false,
      distance: 0,
      max: 100,
      min: 0,
      orientation: 'horizontal',
      range: false,
      step: 1,
      value: 0,
      values: null,
      change: null,
      slide: null,
      start: null,
      stop: null
    },
    _create: function () {
      this._keySliding = false;
      this._mouseSliding = false;
      this._animateOff = true;
      this._handleIndex = null;
      this._detectOrientation();
      this._mouseInit();
      this.element.addClass('ui-slider' + ' ui-slider-' + this.orientation + ' ui-widget' + ' ui-widget-content' + ' ui-corner-all');
      this._refresh();
      this._setOption('disabled', this.options.disabled);
      this._animateOff = false;
    },
    _refresh: function () {
      this._createRange();
      this._createHandles();
      this._setupEvents();
      this._refreshValue();
    },
    _createHandles: function () {
      var i, handleCount, options = this.options, existingHandles = this.element.find('.ui-slider-handle').addClass('ui-state-default ui-corner-all'), handle = '<a class=\'ui-slider-handle ui-state-default ui-corner-all\' href=\'#\'></a>', handles = [];
      handleCount = options.values && options.values.length || 1;
      if (existingHandles.length > handleCount) {
        existingHandles.slice(handleCount).remove();
        existingHandles = existingHandles.slice(0, handleCount);
      }
      for (i = existingHandles.length; i < handleCount; i++) {
        handles.push(handle);
      }
      this.handles = existingHandles.add($(handles.join('')).appendTo(this.element));
      this.handle = this.handles.eq(0);
      this.handles.each(function (i) {
        $(this).data('ui-slider-handle-index', i);
      });
    },
    _createRange: function () {
      var options = this.options, classes = '';
      if (options.range) {
        if (options.range === true) {
          if (!options.values) {
            options.values = [
              this._valueMin(),
              this._valueMin()
            ];
          } else if (options.values.length && options.values.length !== 2) {
            options.values = [
              options.values[0],
              options.values[0]
            ];
          } else if ($.isArray(options.values)) {
            options.values = options.values.slice(0);
          }
        }
        if (!this.range || !this.range.length) {
          this.range = $('<div></div>').appendTo(this.element);
          classes = 'ui-slider-range' + ' ui-widget-header ui-corner-all';
        } else {
          this.range.removeClass('ui-slider-range-min ui-slider-range-max').css({
            'left': '',
            'bottom': ''
          });
        }
        this.range.addClass(classes + (options.range === 'min' || options.range === 'max' ? ' ui-slider-range-' + options.range : ''));
      } else {
        if (this.range) {
          this.range.remove();
        }
        this.range = null;
      }
    },
    _setupEvents: function () {
      var elements = this.handles.add(this.range).filter('a');
      this._off(elements);
      this._on(elements, this._handleEvents);
      this._hoverable(elements);
      this._focusable(elements);
    },
    _destroy: function () {
      this.handles.remove();
      if (this.range) {
        this.range.remove();
      }
      this.element.removeClass('ui-slider' + ' ui-slider-horizontal' + ' ui-slider-vertical' + ' ui-widget' + ' ui-widget-content' + ' ui-corner-all');
      this._mouseDestroy();
    },
    _mouseCapture: function (event) {
      var position, normValue, distance, closestHandle, index, allowed, offset, mouseOverHandle, that = this, o = this.options;
      if (o.disabled) {
        return false;
      }
      this.elementSize = {
        width: this.element.outerWidth(),
        height: this.element.outerHeight()
      };
      this.elementOffset = this.element.offset();
      position = {
        x: event.pageX,
        y: event.pageY
      };
      normValue = this._normValueFromMouse(position);
      distance = this._valueMax() - this._valueMin() + 1;
      this.handles.each(function (i) {
        var thisDistance = Math.abs(normValue - that.values(i));
        if (distance > thisDistance || distance === thisDistance && (i === that._lastChangedValue || that.values(i) === o.min)) {
          distance = thisDistance;
          closestHandle = $(this);
          index = i;
        }
      });
      allowed = this._start(event, index);
      if (allowed === false) {
        return false;
      }
      this._mouseSliding = true;
      this._handleIndex = index;
      closestHandle.addClass('ui-state-active').focus();
      offset = closestHandle.offset();
      mouseOverHandle = !$(event.target).parents().addBack().is('.ui-slider-handle');
      this._clickOffset = mouseOverHandle ? {
        left: 0,
        top: 0
      } : {
        left: event.pageX - offset.left - closestHandle.width() / 2,
        top: event.pageY - offset.top - closestHandle.height() / 2 - (parseInt(closestHandle.css('borderTopWidth'), 10) || 0) - (parseInt(closestHandle.css('borderBottomWidth'), 10) || 0) + (parseInt(closestHandle.css('marginTop'), 10) || 0)
      };
      if (!this.handles.hasClass('ui-state-hover')) {
        this._slide(event, index, normValue);
      }
      this._animateOff = true;
      return true;
    },
    _mouseStart: function () {
      return true;
    },
    _mouseDrag: function (event) {
      var position = {
          x: event.pageX,
          y: event.pageY
        }, normValue = this._normValueFromMouse(position);
      this._slide(event, this._handleIndex, normValue);
      return false;
    },
    _mouseStop: function (event) {
      this.handles.removeClass('ui-state-active');
      this._mouseSliding = false;
      this._stop(event, this._handleIndex);
      this._change(event, this._handleIndex);
      this._handleIndex = null;
      this._clickOffset = null;
      this._animateOff = false;
      return false;
    },
    _detectOrientation: function () {
      this.orientation = this.options.orientation === 'vertical' ? 'vertical' : 'horizontal';
    },
    _normValueFromMouse: function (position) {
      var pixelTotal, pixelMouse, percentMouse, valueTotal, valueMouse;
      if (this.orientation === 'horizontal') {
        pixelTotal = this.elementSize.width;
        pixelMouse = position.x - this.elementOffset.left - (this._clickOffset ? this._clickOffset.left : 0);
      } else {
        pixelTotal = this.elementSize.height;
        pixelMouse = position.y - this.elementOffset.top - (this._clickOffset ? this._clickOffset.top : 0);
      }
      percentMouse = pixelMouse / pixelTotal;
      if (percentMouse > 1) {
        percentMouse = 1;
      }
      if (percentMouse < 0) {
        percentMouse = 0;
      }
      if (this.orientation === 'vertical') {
        percentMouse = 1 - percentMouse;
      }
      valueTotal = this._valueMax() - this._valueMin();
      valueMouse = this._valueMin() + percentMouse * valueTotal;
      return this._trimAlignValue(valueMouse);
    },
    _start: function (event, index) {
      var uiHash = {
          handle: this.handles[index],
          value: this.value()
        };
      if (this.options.values && this.options.values.length) {
        uiHash.value = this.values(index);
        uiHash.values = this.values();
      }
      return this._trigger('start', event, uiHash);
    },
    _slide: function (event, index, newVal) {
      var otherVal, newValues, allowed;
      if (this.options.values && this.options.values.length) {
        otherVal = this.values(index ? 0 : 1);
        if (this.options.values.length === 2 && this.options.range === true && (index === 0 && newVal > otherVal || index === 1 && newVal < otherVal)) {
          newVal = otherVal;
        }
        if (newVal !== this.values(index)) {
          newValues = this.values();
          newValues[index] = newVal;
          // A slide can be canceled by returning false from the slide callback
          allowed = this._trigger('slide', event, {
            handle: this.handles[index],
            value: newVal,
            values: newValues
          });
          otherVal = this.values(index ? 0 : 1);
          if (allowed !== false) {
            this.values(index, newVal);
          }
        }
      } else {
        if (newVal !== this.value()) {
          // A slide can be canceled by returning false from the slide callback
          allowed = this._trigger('slide', event, {
            handle: this.handles[index],
            value: newVal
          });
          if (allowed !== false) {
            this.value(newVal);
          }
        }
      }
    },
    _stop: function (event, index) {
      var uiHash = {
          handle: this.handles[index],
          value: this.value()
        };
      if (this.options.values && this.options.values.length) {
        uiHash.value = this.values(index);
        uiHash.values = this.values();
      }
      this._trigger('stop', event, uiHash);
    },
    _change: function (event, index) {
      if (!this._keySliding && !this._mouseSliding) {
        var uiHash = {
            handle: this.handles[index],
            value: this.value()
          };
        if (this.options.values && this.options.values.length) {
          uiHash.value = this.values(index);
          uiHash.values = this.values();
        }
        //store the last changed value index for reference when handles overlap
        this._lastChangedValue = index;
        this._trigger('change', event, uiHash);
      }
    },
    value: function (newValue) {
      if (arguments.length) {
        this.options.value = this._trimAlignValue(newValue);
        this._refreshValue();
        this._change(null, 0);
        return;
      }
      return this._value();
    },
    values: function (index, newValue) {
      var vals, newValues, i;
      if (arguments.length > 1) {
        this.options.values[index] = this._trimAlignValue(newValue);
        this._refreshValue();
        this._change(null, index);
        return;
      }
      if (arguments.length) {
        if ($.isArray(arguments[0])) {
          vals = this.options.values;
          newValues = arguments[0];
          for (i = 0; i < vals.length; i += 1) {
            vals[i] = this._trimAlignValue(newValues[i]);
            this._change(null, i);
          }
          this._refreshValue();
        } else {
          if (this.options.values && this.options.values.length) {
            return this._values(index);
          } else {
            return this.value();
          }
        }
      } else {
        return this._values();
      }
    },
    _setOption: function (key, value) {
      var i, valsLength = 0;
      if (key === 'range' && this.options.range === true) {
        if (value === 'min') {
          this.options.value = this._values(0);
          this.options.values = null;
        } else if (value === 'max') {
          this.options.value = this._values(this.options.values.length - 1);
          this.options.values = null;
        }
      }
      if ($.isArray(this.options.values)) {
        valsLength = this.options.values.length;
      }
      $.Widget.prototype._setOption.apply(this, arguments);
      switch (key) {
      case 'orientation':
        this._detectOrientation();
        this.element.removeClass('ui-slider-horizontal ui-slider-vertical').addClass('ui-slider-' + this.orientation);
        this._refreshValue();
        break;
      case 'value':
        this._animateOff = true;
        this._refreshValue();
        this._change(null, 0);
        this._animateOff = false;
        break;
      case 'values':
        this._animateOff = true;
        this._refreshValue();
        for (i = 0; i < valsLength; i += 1) {
          this._change(null, i);
        }
        this._animateOff = false;
        break;
      case 'min':
      case 'max':
        this._animateOff = true;
        this._refreshValue();
        this._animateOff = false;
        break;
      case 'range':
        this._animateOff = true;
        this._refresh();
        this._animateOff = false;
        break;
      }
    },
    _value: function () {
      var val = this.options.value;
      val = this._trimAlignValue(val);
      return val;
    },
    _values: function (index) {
      var val, vals, i;
      if (arguments.length) {
        val = this.options.values[index];
        val = this._trimAlignValue(val);
        return val;
      } else if (this.options.values && this.options.values.length) {
        // .slice() creates a copy of the array
        // this copy gets trimmed by min and max and then returned
        vals = this.options.values.slice();
        for (i = 0; i < vals.length; i += 1) {
          vals[i] = this._trimAlignValue(vals[i]);
        }
        return vals;
      } else {
        return [];
      }
    },
    _trimAlignValue: function (val) {
      if (val <= this._valueMin()) {
        return this._valueMin();
      }
      if (val >= this._valueMax()) {
        return this._valueMax();
      }
      var step = this.options.step > 0 ? this.options.step : 1, valModStep = (val - this._valueMin()) % step, alignValue = val - valModStep;
      if (Math.abs(valModStep) * 2 >= step) {
        alignValue += valModStep > 0 ? step : -step;
      }
      // Since JavaScript has problems with large floats, round
      // the final value to 5 digits after the decimal point (see #4124)
      return parseFloat(alignValue.toFixed(5));
    },
    _valueMin: function () {
      return this.options.min;
    },
    _valueMax: function () {
      return this.options.max;
    },
    _refreshValue: function () {
      var lastValPercent, valPercent, value, valueMin, valueMax, oRange = this.options.range, o = this.options, that = this, animate = !this._animateOff ? o.animate : false, _set = {};
      if (this.options.values && this.options.values.length) {
        this.handles.each(function (i) {
          valPercent = (that.values(i) - that._valueMin()) / (that._valueMax() - that._valueMin()) * 100;
          _set[that.orientation === 'horizontal' ? 'left' : 'bottom'] = valPercent + '%';
          $(this).stop(1, 1)[animate ? 'animate' : 'css'](_set, o.animate);
          if (that.options.range === true) {
            if (that.orientation === 'horizontal') {
              if (i === 0) {
                that.range.stop(1, 1)[animate ? 'animate' : 'css']({ left: valPercent + '%' }, o.animate);
              }
              if (i === 1) {
                that.range[animate ? 'animate' : 'css']({ width: valPercent - lastValPercent + '%' }, {
                  queue: false,
                  duration: o.animate
                });
              }
            } else {
              if (i === 0) {
                that.range.stop(1, 1)[animate ? 'animate' : 'css']({ bottom: valPercent + '%' }, o.animate);
              }
              if (i === 1) {
                that.range[animate ? 'animate' : 'css']({ height: valPercent - lastValPercent + '%' }, {
                  queue: false,
                  duration: o.animate
                });
              }
            }
          }
          lastValPercent = valPercent;
        });
      } else {
        value = this.value();
        valueMin = this._valueMin();
        valueMax = this._valueMax();
        valPercent = valueMax !== valueMin ? (value - valueMin) / (valueMax - valueMin) * 100 : 0;
        _set[this.orientation === 'horizontal' ? 'left' : 'bottom'] = valPercent + '%';
        this.handle.stop(1, 1)[animate ? 'animate' : 'css'](_set, o.animate);
        if (oRange === 'min' && this.orientation === 'horizontal') {
          this.range.stop(1, 1)[animate ? 'animate' : 'css']({ width: valPercent + '%' }, o.animate);
        }
        if (oRange === 'max' && this.orientation === 'horizontal') {
          this.range[animate ? 'animate' : 'css']({ width: 100 - valPercent + '%' }, {
            queue: false,
            duration: o.animate
          });
        }
        if (oRange === 'min' && this.orientation === 'vertical') {
          this.range.stop(1, 1)[animate ? 'animate' : 'css']({ height: valPercent + '%' }, o.animate);
        }
        if (oRange === 'max' && this.orientation === 'vertical') {
          this.range[animate ? 'animate' : 'css']({ height: 100 - valPercent + '%' }, {
            queue: false,
            duration: o.animate
          });
        }
      }
    },
    _handleEvents: {
      keydown: function (event) {
        var allowed, curVal, newVal, step, index = $(event.target).data('ui-slider-handle-index');
        switch (event.keyCode) {
        case $.ui.keyCode.HOME:
        case $.ui.keyCode.END:
        case $.ui.keyCode.PAGE_UP:
        case $.ui.keyCode.PAGE_DOWN:
        case $.ui.keyCode.UP:
        case $.ui.keyCode.RIGHT:
        case $.ui.keyCode.DOWN:
        case $.ui.keyCode.LEFT:
          event.preventDefault();
          if (!this._keySliding) {
            this._keySliding = true;
            $(event.target).addClass('ui-state-active');
            allowed = this._start(event, index);
            if (allowed === false) {
              return;
            }
          }
          break;
        }
        step = this.options.step;
        if (this.options.values && this.options.values.length) {
          curVal = newVal = this.values(index);
        } else {
          curVal = newVal = this.value();
        }
        switch (event.keyCode) {
        case $.ui.keyCode.HOME:
          newVal = this._valueMin();
          break;
        case $.ui.keyCode.END:
          newVal = this._valueMax();
          break;
        case $.ui.keyCode.PAGE_UP:
          newVal = this._trimAlignValue(curVal + (this._valueMax() - this._valueMin()) / numPages);
          break;
        case $.ui.keyCode.PAGE_DOWN:
          newVal = this._trimAlignValue(curVal - (this._valueMax() - this._valueMin()) / numPages);
          break;
        case $.ui.keyCode.UP:
        case $.ui.keyCode.RIGHT:
          if (curVal === this._valueMax()) {
            return;
          }
          newVal = this._trimAlignValue(curVal + step);
          break;
        case $.ui.keyCode.DOWN:
        case $.ui.keyCode.LEFT:
          if (curVal === this._valueMin()) {
            return;
          }
          newVal = this._trimAlignValue(curVal - step);
          break;
        }
        this._slide(event, index, newVal);
      },
      click: function (event) {
        event.preventDefault();
      },
      keyup: function (event) {
        var index = $(event.target).data('ui-slider-handle-index');
        if (this._keySliding) {
          this._keySliding = false;
          this._stop(event, index);
          this._change(event, index);
          $(event.target).removeClass('ui-state-active');
        }
      }
    }
  });
}(jQuery));
(function ($, undefined) {
  function isOverAxis(x, reference, size) {
    return x > reference && x < reference + size;
  }
  function isFloating(item) {
    return /left|right/.test(item.css('float')) || /inline|table-cell/.test(item.css('display'));
  }
  $.widget('ui.sortable', $.ui.mouse, {
    version: '1.10.4',
    widgetEventPrefix: 'sort',
    ready: false,
    options: {
      appendTo: 'parent',
      axis: false,
      connectWith: false,
      containment: false,
      cursor: 'auto',
      cursorAt: false,
      dropOnEmpty: true,
      forcePlaceholderSize: false,
      forceHelperSize: false,
      grid: false,
      handle: false,
      helper: 'original',
      items: '> *',
      opacity: false,
      placeholder: false,
      revert: false,
      scroll: true,
      scrollSensitivity: 20,
      scrollSpeed: 20,
      scope: 'default',
      tolerance: 'intersect',
      zIndex: 1000,
      activate: null,
      beforeStop: null,
      change: null,
      deactivate: null,
      out: null,
      over: null,
      receive: null,
      remove: null,
      sort: null,
      start: null,
      stop: null,
      update: null
    },
    _create: function () {
      var o = this.options;
      this.containerCache = {};
      this.element.addClass('ui-sortable');
      //Get the items
      this.refresh();
      //Let's determine if the items are being displayed horizontally
      this.floating = this.items.length ? o.axis === 'x' || isFloating(this.items[0].item) : false;
      //Let's determine the parent's offset
      this.offset = this.element.offset();
      //Initialize mouse events for interaction
      this._mouseInit();
      //We're ready to go
      this.ready = true;
    },
    _destroy: function () {
      this.element.removeClass('ui-sortable ui-sortable-disabled');
      this._mouseDestroy();
      for (var i = this.items.length - 1; i >= 0; i--) {
        this.items[i].item.removeData(this.widgetName + '-item');
      }
      return this;
    },
    _setOption: function (key, value) {
      if (key === 'disabled') {
        this.options[key] = value;
        this.widget().toggleClass('ui-sortable-disabled', !!value);
      } else {
        // Don't call widget base _setOption for disable as it adds ui-state-disabled class
        $.Widget.prototype._setOption.apply(this, arguments);
      }
    },
    _mouseCapture: function (event, overrideHandle) {
      var currentItem = null, validHandle = false, that = this;
      if (this.reverting) {
        return false;
      }
      if (this.options.disabled || this.options.type === 'static') {
        return false;
      }
      //We have to refresh the items data once first
      this._refreshItems(event);
      //Find out if the clicked node (or one of its parents) is a actual item in this.items
      $(event.target).parents().each(function () {
        if ($.data(this, that.widgetName + '-item') === that) {
          currentItem = $(this);
          return false;
        }
      });
      if ($.data(event.target, that.widgetName + '-item') === that) {
        currentItem = $(event.target);
      }
      if (!currentItem) {
        return false;
      }
      if (this.options.handle && !overrideHandle) {
        $(this.options.handle, currentItem).find('*').addBack().each(function () {
          if (this === event.target) {
            validHandle = true;
          }
        });
        if (!validHandle) {
          return false;
        }
      }
      this.currentItem = currentItem;
      this._removeCurrentsFromItems();
      return true;
    },
    _mouseStart: function (event, overrideHandle, noActivation) {
      var i, body, o = this.options;
      this.currentContainer = this;
      //We only need to call refreshPositions, because the refreshItems call has been moved to mouseCapture
      this.refreshPositions();
      //Create and append the visible helper
      this.helper = this._createHelper(event);
      //Cache the helper size
      this._cacheHelperProportions();
      /*
		 * - Position generation -
		 * This block generates everything position related - it's the core of draggables.
		 */
      //Cache the margins of the original element
      this._cacheMargins();
      //Get the next scrolling parent
      this.scrollParent = this.helper.scrollParent();
      //The element's absolute position on the page minus margins
      this.offset = this.currentItem.offset();
      this.offset = {
        top: this.offset.top - this.margins.top,
        left: this.offset.left - this.margins.left
      };
      $.extend(this.offset, {
        click: {
          left: event.pageX - this.offset.left,
          top: event.pageY - this.offset.top
        },
        parent: this._getParentOffset(),
        relative: this._getRelativeOffset()
      });
      // Only after we got the offset, we can change the helper's position to absolute
      // TODO: Still need to figure out a way to make relative sorting possible
      this.helper.css('position', 'absolute');
      this.cssPosition = this.helper.css('position');
      //Generate the original position
      this.originalPosition = this._generatePosition(event);
      this.originalPageX = event.pageX;
      this.originalPageY = event.pageY;
      //Adjust the mouse offset relative to the helper if "cursorAt" is supplied
      o.cursorAt && this._adjustOffsetFromHelper(o.cursorAt);
      //Cache the former DOM position
      this.domPosition = {
        prev: this.currentItem.prev()[0],
        parent: this.currentItem.parent()[0]
      };
      //If the helper is not the original, hide the original so it's not playing any role during the drag, won't cause anything bad this way
      if (this.helper[0] !== this.currentItem[0]) {
        this.currentItem.hide();
      }
      //Create the placeholder
      this._createPlaceholder();
      //Set a containment if given in the options
      if (o.containment) {
        this._setContainment();
      }
      if (o.cursor && o.cursor !== 'auto') {
        // cursor option
        body = this.document.find('body');
        // support: IE
        this.storedCursor = body.css('cursor');
        body.css('cursor', o.cursor);
        this.storedStylesheet = $('<style>*{ cursor: ' + o.cursor + ' !important; }</style>').appendTo(body);
      }
      if (o.opacity) {
        // opacity option
        if (this.helper.css('opacity')) {
          this._storedOpacity = this.helper.css('opacity');
        }
        this.helper.css('opacity', o.opacity);
      }
      if (o.zIndex) {
        // zIndex option
        if (this.helper.css('zIndex')) {
          this._storedZIndex = this.helper.css('zIndex');
        }
        this.helper.css('zIndex', o.zIndex);
      }
      //Prepare scrolling
      if (this.scrollParent[0] !== document && this.scrollParent[0].tagName !== 'HTML') {
        this.overflowOffset = this.scrollParent.offset();
      }
      //Call callbacks
      this._trigger('start', event, this._uiHash());
      //Recache the helper size
      if (!this._preserveHelperProportions) {
        this._cacheHelperProportions();
      }
      //Post "activate" events to possible containers
      if (!noActivation) {
        for (i = this.containers.length - 1; i >= 0; i--) {
          this.containers[i]._trigger('activate', event, this._uiHash(this));
        }
      }
      //Prepare possible droppables
      if ($.ui.ddmanager) {
        $.ui.ddmanager.current = this;
      }
      if ($.ui.ddmanager && !o.dropBehaviour) {
        $.ui.ddmanager.prepareOffsets(this, event);
      }
      this.dragging = true;
      this.helper.addClass('ui-sortable-helper');
      this._mouseDrag(event);
      //Execute the drag once - this causes the helper not to be visible before getting its correct position
      return true;
    },
    _mouseDrag: function (event) {
      var i, item, itemElement, intersection, o = this.options, scrolled = false;
      //Compute the helpers position
      this.position = this._generatePosition(event);
      this.positionAbs = this._convertPositionTo('absolute');
      if (!this.lastPositionAbs) {
        this.lastPositionAbs = this.positionAbs;
      }
      //Do scrolling
      if (this.options.scroll) {
        if (this.scrollParent[0] !== document && this.scrollParent[0].tagName !== 'HTML') {
          if (this.overflowOffset.top + this.scrollParent[0].offsetHeight - event.pageY < o.scrollSensitivity) {
            this.scrollParent[0].scrollTop = scrolled = this.scrollParent[0].scrollTop + o.scrollSpeed;
          } else if (event.pageY - this.overflowOffset.top < o.scrollSensitivity) {
            this.scrollParent[0].scrollTop = scrolled = this.scrollParent[0].scrollTop - o.scrollSpeed;
          }
          if (this.overflowOffset.left + this.scrollParent[0].offsetWidth - event.pageX < o.scrollSensitivity) {
            this.scrollParent[0].scrollLeft = scrolled = this.scrollParent[0].scrollLeft + o.scrollSpeed;
          } else if (event.pageX - this.overflowOffset.left < o.scrollSensitivity) {
            this.scrollParent[0].scrollLeft = scrolled = this.scrollParent[0].scrollLeft - o.scrollSpeed;
          }
        } else {
          if (event.pageY - $(document).scrollTop() < o.scrollSensitivity) {
            scrolled = $(document).scrollTop($(document).scrollTop() - o.scrollSpeed);
          } else if ($(window).height() - (event.pageY - $(document).scrollTop()) < o.scrollSensitivity) {
            scrolled = $(document).scrollTop($(document).scrollTop() + o.scrollSpeed);
          }
          if (event.pageX - $(document).scrollLeft() < o.scrollSensitivity) {
            scrolled = $(document).scrollLeft($(document).scrollLeft() - o.scrollSpeed);
          } else if ($(window).width() - (event.pageX - $(document).scrollLeft()) < o.scrollSensitivity) {
            scrolled = $(document).scrollLeft($(document).scrollLeft() + o.scrollSpeed);
          }
        }
        if (scrolled !== false && $.ui.ddmanager && !o.dropBehaviour) {
          $.ui.ddmanager.prepareOffsets(this, event);
        }
      }
      //Regenerate the absolute position used for position checks
      this.positionAbs = this._convertPositionTo('absolute');
      //Set the helper position
      if (!this.options.axis || this.options.axis !== 'y') {
        this.helper[0].style.left = this.position.left + 'px';
      }
      if (!this.options.axis || this.options.axis !== 'x') {
        this.helper[0].style.top = this.position.top + 'px';
      }
      //Rearrange
      for (i = this.items.length - 1; i >= 0; i--) {
        //Cache variables and intersection, continue if no intersection
        item = this.items[i];
        itemElement = item.item[0];
        intersection = this._intersectsWithPointer(item);
        if (!intersection) {
          continue;
        }
        // Only put the placeholder inside the current Container, skip all
        // items from other containers. This works because when moving
        // an item from one container to another the
        // currentContainer is switched before the placeholder is moved.
        //
        // Without this, moving items in "sub-sortables" can cause
        // the placeholder to jitter beetween the outer and inner container.
        if (item.instance !== this.currentContainer) {
          continue;
        }
        // cannot intersect with itself
        // no useless actions that have been done before
        // no action if the item moved is the parent of the item checked
        if (itemElement !== this.currentItem[0] && this.placeholder[intersection === 1 ? 'next' : 'prev']()[0] !== itemElement && !$.contains(this.placeholder[0], itemElement) && (this.options.type === 'semi-dynamic' ? !$.contains(this.element[0], itemElement) : true)) {
          this.direction = intersection === 1 ? 'down' : 'up';
          if (this.options.tolerance === 'pointer' || this._intersectsWithSides(item)) {
            this._rearrange(event, item);
          } else {
            break;
          }
          this._trigger('change', event, this._uiHash());
          break;
        }
      }
      //Post events to containers
      this._contactContainers(event);
      //Interconnect with droppables
      if ($.ui.ddmanager) {
        $.ui.ddmanager.drag(this, event);
      }
      //Call callbacks
      this._trigger('sort', event, this._uiHash());
      this.lastPositionAbs = this.positionAbs;
      return false;
    },
    _mouseStop: function (event, noPropagation) {
      if (!event) {
        return;
      }
      //If we are using droppables, inform the manager about the drop
      if ($.ui.ddmanager && !this.options.dropBehaviour) {
        $.ui.ddmanager.drop(this, event);
      }
      if (this.options.revert) {
        var that = this, cur = this.placeholder.offset(), axis = this.options.axis, animation = {};
        if (!axis || axis === 'x') {
          animation.left = cur.left - this.offset.parent.left - this.margins.left + (this.offsetParent[0] === document.body ? 0 : this.offsetParent[0].scrollLeft);
        }
        if (!axis || axis === 'y') {
          animation.top = cur.top - this.offset.parent.top - this.margins.top + (this.offsetParent[0] === document.body ? 0 : this.offsetParent[0].scrollTop);
        }
        this.reverting = true;
        $(this.helper).animate(animation, parseInt(this.options.revert, 10) || 500, function () {
          that._clear(event);
        });
      } else {
        this._clear(event, noPropagation);
      }
      return false;
    },
    cancel: function () {
      if (this.dragging) {
        this._mouseUp({ target: null });
        if (this.options.helper === 'original') {
          this.currentItem.css(this._storedCSS).removeClass('ui-sortable-helper');
        } else {
          this.currentItem.show();
        }
        //Post deactivating events to containers
        for (var i = this.containers.length - 1; i >= 0; i--) {
          this.containers[i]._trigger('deactivate', null, this._uiHash(this));
          if (this.containers[i].containerCache.over) {
            this.containers[i]._trigger('out', null, this._uiHash(this));
            this.containers[i].containerCache.over = 0;
          }
        }
      }
      if (this.placeholder) {
        //$(this.placeholder[0]).remove(); would have been the jQuery way - unfortunately, it unbinds ALL events from the original node!
        if (this.placeholder[0].parentNode) {
          this.placeholder[0].parentNode.removeChild(this.placeholder[0]);
        }
        if (this.options.helper !== 'original' && this.helper && this.helper[0].parentNode) {
          this.helper.remove();
        }
        $.extend(this, {
          helper: null,
          dragging: false,
          reverting: false,
          _noFinalSort: null
        });
        if (this.domPosition.prev) {
          $(this.domPosition.prev).after(this.currentItem);
        } else {
          $(this.domPosition.parent).prepend(this.currentItem);
        }
      }
      return this;
    },
    serialize: function (o) {
      var items = this._getItemsAsjQuery(o && o.connected), str = [];
      o = o || {};
      $(items).each(function () {
        var res = ($(o.item || this).attr(o.attribute || 'id') || '').match(o.expression || /(.+)[\-=_](.+)/);
        if (res) {
          str.push((o.key || res[1] + '[]') + '=' + (o.key && o.expression ? res[1] : res[2]));
        }
      });
      if (!str.length && o.key) {
        str.push(o.key + '=');
      }
      return str.join('&');
    },
    toArray: function (o) {
      var items = this._getItemsAsjQuery(o && o.connected), ret = [];
      o = o || {};
      items.each(function () {
        ret.push($(o.item || this).attr(o.attribute || 'id') || '');
      });
      return ret;
    },
    _intersectsWith: function (item) {
      var x1 = this.positionAbs.left, x2 = x1 + this.helperProportions.width, y1 = this.positionAbs.top, y2 = y1 + this.helperProportions.height, l = item.left, r = l + item.width, t = item.top, b = t + item.height, dyClick = this.offset.click.top, dxClick = this.offset.click.left, isOverElementHeight = this.options.axis === 'x' || y1 + dyClick > t && y1 + dyClick < b, isOverElementWidth = this.options.axis === 'y' || x1 + dxClick > l && x1 + dxClick < r, isOverElement = isOverElementHeight && isOverElementWidth;
      if (this.options.tolerance === 'pointer' || this.options.forcePointerForContainers || this.options.tolerance !== 'pointer' && this.helperProportions[this.floating ? 'width' : 'height'] > item[this.floating ? 'width' : 'height']) {
        return isOverElement;
      } else {
        return l < x1 + this.helperProportions.width / 2 && x2 - this.helperProportions.width / 2 < r && t < y1 + this.helperProportions.height / 2 && y2 - this.helperProportions.height / 2 < b;  // Top Half
      }
    },
    _intersectsWithPointer: function (item) {
      var isOverElementHeight = this.options.axis === 'x' || isOverAxis(this.positionAbs.top + this.offset.click.top, item.top, item.height), isOverElementWidth = this.options.axis === 'y' || isOverAxis(this.positionAbs.left + this.offset.click.left, item.left, item.width), isOverElement = isOverElementHeight && isOverElementWidth, verticalDirection = this._getDragVerticalDirection(), horizontalDirection = this._getDragHorizontalDirection();
      if (!isOverElement) {
        return false;
      }
      return this.floating ? horizontalDirection && horizontalDirection === 'right' || verticalDirection === 'down' ? 2 : 1 : verticalDirection && (verticalDirection === 'down' ? 2 : 1);
    },
    _intersectsWithSides: function (item) {
      var isOverBottomHalf = isOverAxis(this.positionAbs.top + this.offset.click.top, item.top + item.height / 2, item.height), isOverRightHalf = isOverAxis(this.positionAbs.left + this.offset.click.left, item.left + item.width / 2, item.width), verticalDirection = this._getDragVerticalDirection(), horizontalDirection = this._getDragHorizontalDirection();
      if (this.floating && horizontalDirection) {
        return horizontalDirection === 'right' && isOverRightHalf || horizontalDirection === 'left' && !isOverRightHalf;
      } else {
        return verticalDirection && (verticalDirection === 'down' && isOverBottomHalf || verticalDirection === 'up' && !isOverBottomHalf);
      }
    },
    _getDragVerticalDirection: function () {
      var delta = this.positionAbs.top - this.lastPositionAbs.top;
      return delta !== 0 && (delta > 0 ? 'down' : 'up');
    },
    _getDragHorizontalDirection: function () {
      var delta = this.positionAbs.left - this.lastPositionAbs.left;
      return delta !== 0 && (delta > 0 ? 'right' : 'left');
    },
    refresh: function (event) {
      this._refreshItems(event);
      this.refreshPositions();
      return this;
    },
    _connectWith: function () {
      var options = this.options;
      return options.connectWith.constructor === String ? [options.connectWith] : options.connectWith;
    },
    _getItemsAsjQuery: function (connected) {
      var i, j, cur, inst, items = [], queries = [], connectWith = this._connectWith();
      if (connectWith && connected) {
        for (i = connectWith.length - 1; i >= 0; i--) {
          cur = $(connectWith[i]);
          for (j = cur.length - 1; j >= 0; j--) {
            inst = $.data(cur[j], this.widgetFullName);
            if (inst && inst !== this && !inst.options.disabled) {
              queries.push([
                $.isFunction(inst.options.items) ? inst.options.items.call(inst.element) : $(inst.options.items, inst.element).not('.ui-sortable-helper').not('.ui-sortable-placeholder'),
                inst
              ]);
            }
          }
        }
      }
      queries.push([
        $.isFunction(this.options.items) ? this.options.items.call(this.element, null, {
          options: this.options,
          item: this.currentItem
        }) : $(this.options.items, this.element).not('.ui-sortable-helper').not('.ui-sortable-placeholder'),
        this
      ]);
      function addItems() {
        items.push(this);
      }
      for (i = queries.length - 1; i >= 0; i--) {
        queries[i][0].each(addItems);
      }
      return $(items);
    },
    _removeCurrentsFromItems: function () {
      var list = this.currentItem.find(':data(' + this.widgetName + '-item)');
      this.items = $.grep(this.items, function (item) {
        for (var j = 0; j < list.length; j++) {
          if (list[j] === item.item[0]) {
            return false;
          }
        }
        return true;
      });
    },
    _refreshItems: function (event) {
      this.items = [];
      this.containers = [this];
      var i, j, cur, inst, targetData, _queries, item, queriesLength, items = this.items, queries = [[
            $.isFunction(this.options.items) ? this.options.items.call(this.element[0], event, { item: this.currentItem }) : $(this.options.items, this.element),
            this
          ]], connectWith = this._connectWith();
      if (connectWith && this.ready) {
        //Shouldn't be run the first time through due to massive slow-down
        for (i = connectWith.length - 1; i >= 0; i--) {
          cur = $(connectWith[i]);
          for (j = cur.length - 1; j >= 0; j--) {
            inst = $.data(cur[j], this.widgetFullName);
            if (inst && inst !== this && !inst.options.disabled) {
              queries.push([
                $.isFunction(inst.options.items) ? inst.options.items.call(inst.element[0], event, { item: this.currentItem }) : $(inst.options.items, inst.element),
                inst
              ]);
              this.containers.push(inst);
            }
          }
        }
      }
      for (i = queries.length - 1; i >= 0; i--) {
        targetData = queries[i][1];
        _queries = queries[i][0];
        for (j = 0, queriesLength = _queries.length; j < queriesLength; j++) {
          item = $(_queries[j]);
          item.data(this.widgetName + '-item', targetData);
          // Data for target checking (mouse manager)
          items.push({
            item: item,
            instance: targetData,
            width: 0,
            height: 0,
            left: 0,
            top: 0
          });
        }
      }
    },
    refreshPositions: function (fast) {
      //This has to be redone because due to the item being moved out/into the offsetParent, the offsetParent's position will change
      if (this.offsetParent && this.helper) {
        this.offset.parent = this._getParentOffset();
      }
      var i, item, t, p;
      for (i = this.items.length - 1; i >= 0; i--) {
        item = this.items[i];
        //We ignore calculating positions of all connected containers when we're not over them
        if (item.instance !== this.currentContainer && this.currentContainer && item.item[0] !== this.currentItem[0]) {
          continue;
        }
        t = this.options.toleranceElement ? $(this.options.toleranceElement, item.item) : item.item;
        if (!fast) {
          item.width = t.outerWidth();
          item.height = t.outerHeight();
        }
        p = t.offset();
        item.left = p.left;
        item.top = p.top;
      }
      if (this.options.custom && this.options.custom.refreshContainers) {
        this.options.custom.refreshContainers.call(this);
      } else {
        for (i = this.containers.length - 1; i >= 0; i--) {
          p = this.containers[i].element.offset();
          this.containers[i].containerCache.left = p.left;
          this.containers[i].containerCache.top = p.top;
          this.containers[i].containerCache.width = this.containers[i].element.outerWidth();
          this.containers[i].containerCache.height = this.containers[i].element.outerHeight();
        }
      }
      return this;
    },
    _createPlaceholder: function (that) {
      that = that || this;
      var className, o = that.options;
      if (!o.placeholder || o.placeholder.constructor === String) {
        className = o.placeholder;
        o.placeholder = {
          element: function () {
            var nodeName = that.currentItem[0].nodeName.toLowerCase(), element = $('<' + nodeName + '>', that.document[0]).addClass(className || that.currentItem[0].className + ' ui-sortable-placeholder').removeClass('ui-sortable-helper');
            if (nodeName === 'tr') {
              that.currentItem.children().each(function () {
                $('<td>&#160;</td>', that.document[0]).attr('colspan', $(this).attr('colspan') || 1).appendTo(element);
              });
            } else if (nodeName === 'img') {
              element.attr('src', that.currentItem.attr('src'));
            }
            if (!className) {
              element.css('visibility', 'hidden');
            }
            return element;
          },
          update: function (container, p) {
            // 1. If a className is set as 'placeholder option, we don't force sizes - the class is responsible for that
            // 2. The option 'forcePlaceholderSize can be enabled to force it even if a class name is specified
            if (className && !o.forcePlaceholderSize) {
              return;
            }
            //If the element doesn't have a actual height by itself (without styles coming from a stylesheet), it receives the inline height from the dragged item
            if (!p.height()) {
              p.height(that.currentItem.innerHeight() - parseInt(that.currentItem.css('paddingTop') || 0, 10) - parseInt(that.currentItem.css('paddingBottom') || 0, 10));
            }
            if (!p.width()) {
              p.width(that.currentItem.innerWidth() - parseInt(that.currentItem.css('paddingLeft') || 0, 10) - parseInt(that.currentItem.css('paddingRight') || 0, 10));
            }
          }
        };
      }
      //Create the placeholder
      that.placeholder = $(o.placeholder.element.call(that.element, that.currentItem));
      //Append it after the actual current item
      that.currentItem.after(that.placeholder);
      //Update the size of the placeholder (TODO: Logic to fuzzy, see line 316/317)
      o.placeholder.update(that, that.placeholder);
    },
    _contactContainers: function (event) {
      var i, j, dist, itemWithLeastDistance, posProperty, sizeProperty, base, cur, nearBottom, floating, innermostContainer = null, innermostIndex = null;
      // get innermost container that intersects with item
      for (i = this.containers.length - 1; i >= 0; i--) {
        // never consider a container that's located within the item itself
        if ($.contains(this.currentItem[0], this.containers[i].element[0])) {
          continue;
        }
        if (this._intersectsWith(this.containers[i].containerCache)) {
          // if we've already found a container and it's more "inner" than this, then continue
          if (innermostContainer && $.contains(this.containers[i].element[0], innermostContainer.element[0])) {
            continue;
          }
          innermostContainer = this.containers[i];
          innermostIndex = i;
        } else {
          // container doesn't intersect. trigger "out" event if necessary
          if (this.containers[i].containerCache.over) {
            this.containers[i]._trigger('out', event, this._uiHash(this));
            this.containers[i].containerCache.over = 0;
          }
        }
      }
      // if no intersecting containers found, return
      if (!innermostContainer) {
        return;
      }
      // move the item into the container if it's not there already
      if (this.containers.length === 1) {
        if (!this.containers[innermostIndex].containerCache.over) {
          this.containers[innermostIndex]._trigger('over', event, this._uiHash(this));
          this.containers[innermostIndex].containerCache.over = 1;
        }
      } else {
        //When entering a new container, we will find the item with the least distance and append our item near it
        dist = 10000;
        itemWithLeastDistance = null;
        floating = innermostContainer.floating || isFloating(this.currentItem);
        posProperty = floating ? 'left' : 'top';
        sizeProperty = floating ? 'width' : 'height';
        base = this.positionAbs[posProperty] + this.offset.click[posProperty];
        for (j = this.items.length - 1; j >= 0; j--) {
          if (!$.contains(this.containers[innermostIndex].element[0], this.items[j].item[0])) {
            continue;
          }
          if (this.items[j].item[0] === this.currentItem[0]) {
            continue;
          }
          if (floating && !isOverAxis(this.positionAbs.top + this.offset.click.top, this.items[j].top, this.items[j].height)) {
            continue;
          }
          cur = this.items[j].item.offset()[posProperty];
          nearBottom = false;
          if (Math.abs(cur - base) > Math.abs(cur + this.items[j][sizeProperty] - base)) {
            nearBottom = true;
            cur += this.items[j][sizeProperty];
          }
          if (Math.abs(cur - base) < dist) {
            dist = Math.abs(cur - base);
            itemWithLeastDistance = this.items[j];
            this.direction = nearBottom ? 'up' : 'down';
          }
        }
        //Check if dropOnEmpty is enabled
        if (!itemWithLeastDistance && !this.options.dropOnEmpty) {
          return;
        }
        if (this.currentContainer === this.containers[innermostIndex]) {
          return;
        }
        itemWithLeastDistance ? this._rearrange(event, itemWithLeastDistance, null, true) : this._rearrange(event, null, this.containers[innermostIndex].element, true);
        this._trigger('change', event, this._uiHash());
        this.containers[innermostIndex]._trigger('change', event, this._uiHash(this));
        this.currentContainer = this.containers[innermostIndex];
        //Update the placeholder
        this.options.placeholder.update(this.currentContainer, this.placeholder);
        this.containers[innermostIndex]._trigger('over', event, this._uiHash(this));
        this.containers[innermostIndex].containerCache.over = 1;
      }
    },
    _createHelper: function (event) {
      var o = this.options, helper = $.isFunction(o.helper) ? $(o.helper.apply(this.element[0], [
          event,
          this.currentItem
        ])) : o.helper === 'clone' ? this.currentItem.clone() : this.currentItem;
      //Add the helper to the DOM if that didn't happen already
      if (!helper.parents('body').length) {
        $(o.appendTo !== 'parent' ? o.appendTo : this.currentItem[0].parentNode)[0].appendChild(helper[0]);
      }
      if (helper[0] === this.currentItem[0]) {
        this._storedCSS = {
          width: this.currentItem[0].style.width,
          height: this.currentItem[0].style.height,
          position: this.currentItem.css('position'),
          top: this.currentItem.css('top'),
          left: this.currentItem.css('left')
        };
      }
      if (!helper[0].style.width || o.forceHelperSize) {
        helper.width(this.currentItem.width());
      }
      if (!helper[0].style.height || o.forceHelperSize) {
        helper.height(this.currentItem.height());
      }
      return helper;
    },
    _adjustOffsetFromHelper: function (obj) {
      if (typeof obj === 'string') {
        obj = obj.split(' ');
      }
      if ($.isArray(obj)) {
        obj = {
          left: +obj[0],
          top: +obj[1] || 0
        };
      }
      if ('left' in obj) {
        this.offset.click.left = obj.left + this.margins.left;
      }
      if ('right' in obj) {
        this.offset.click.left = this.helperProportions.width - obj.right + this.margins.left;
      }
      if ('top' in obj) {
        this.offset.click.top = obj.top + this.margins.top;
      }
      if ('bottom' in obj) {
        this.offset.click.top = this.helperProportions.height - obj.bottom + this.margins.top;
      }
    },
    _getParentOffset: function () {
      //Get the offsetParent and cache its position
      this.offsetParent = this.helper.offsetParent();
      var po = this.offsetParent.offset();
      // This is a special case where we need to modify a offset calculated on start, since the following happened:
      // 1. The position of the helper is absolute, so it's position is calculated based on the next positioned parent
      // 2. The actual offset parent is a child of the scroll parent, and the scroll parent isn't the document, which means that
      //    the scroll is included in the initial calculation of the offset of the parent, and never recalculated upon drag
      if (this.cssPosition === 'absolute' && this.scrollParent[0] !== document && $.contains(this.scrollParent[0], this.offsetParent[0])) {
        po.left += this.scrollParent.scrollLeft();
        po.top += this.scrollParent.scrollTop();
      }
      // This needs to be actually done for all browsers, since pageX/pageY includes this information
      // with an ugly IE fix
      if (this.offsetParent[0] === document.body || this.offsetParent[0].tagName && this.offsetParent[0].tagName.toLowerCase() === 'html' && $.ui.ie) {
        po = {
          top: 0,
          left: 0
        };
      }
      return {
        top: po.top + (parseInt(this.offsetParent.css('borderTopWidth'), 10) || 0),
        left: po.left + (parseInt(this.offsetParent.css('borderLeftWidth'), 10) || 0)
      };
    },
    _getRelativeOffset: function () {
      if (this.cssPosition === 'relative') {
        var p = this.currentItem.position();
        return {
          top: p.top - (parseInt(this.helper.css('top'), 10) || 0) + this.scrollParent.scrollTop(),
          left: p.left - (parseInt(this.helper.css('left'), 10) || 0) + this.scrollParent.scrollLeft()
        };
      } else {
        return {
          top: 0,
          left: 0
        };
      }
    },
    _cacheMargins: function () {
      this.margins = {
        left: parseInt(this.currentItem.css('marginLeft'), 10) || 0,
        top: parseInt(this.currentItem.css('marginTop'), 10) || 0
      };
    },
    _cacheHelperProportions: function () {
      this.helperProportions = {
        width: this.helper.outerWidth(),
        height: this.helper.outerHeight()
      };
    },
    _setContainment: function () {
      var ce, co, over, o = this.options;
      if (o.containment === 'parent') {
        o.containment = this.helper[0].parentNode;
      }
      if (o.containment === 'document' || o.containment === 'window') {
        this.containment = [
          0 - this.offset.relative.left - this.offset.parent.left,
          0 - this.offset.relative.top - this.offset.parent.top,
          $(o.containment === 'document' ? document : window).width() - this.helperProportions.width - this.margins.left,
          ($(o.containment === 'document' ? document : window).height() || document.body.parentNode.scrollHeight) - this.helperProportions.height - this.margins.top
        ];
      }
      if (!/^(document|window|parent)$/.test(o.containment)) {
        ce = $(o.containment)[0];
        co = $(o.containment).offset();
        over = $(ce).css('overflow') !== 'hidden';
        this.containment = [
          co.left + (parseInt($(ce).css('borderLeftWidth'), 10) || 0) + (parseInt($(ce).css('paddingLeft'), 10) || 0) - this.margins.left,
          co.top + (parseInt($(ce).css('borderTopWidth'), 10) || 0) + (parseInt($(ce).css('paddingTop'), 10) || 0) - this.margins.top,
          co.left + (over ? Math.max(ce.scrollWidth, ce.offsetWidth) : ce.offsetWidth) - (parseInt($(ce).css('borderLeftWidth'), 10) || 0) - (parseInt($(ce).css('paddingRight'), 10) || 0) - this.helperProportions.width - this.margins.left,
          co.top + (over ? Math.max(ce.scrollHeight, ce.offsetHeight) : ce.offsetHeight) - (parseInt($(ce).css('borderTopWidth'), 10) || 0) - (parseInt($(ce).css('paddingBottom'), 10) || 0) - this.helperProportions.height - this.margins.top
        ];
      }
    },
    _convertPositionTo: function (d, pos) {
      if (!pos) {
        pos = this.position;
      }
      var mod = d === 'absolute' ? 1 : -1, scroll = this.cssPosition === 'absolute' && !(this.scrollParent[0] !== document && $.contains(this.scrollParent[0], this.offsetParent[0])) ? this.offsetParent : this.scrollParent, scrollIsRootNode = /(html|body)/i.test(scroll[0].tagName);
      return {
        top: pos.top + this.offset.relative.top * mod + this.offset.parent.top * mod - (this.cssPosition === 'fixed' ? -this.scrollParent.scrollTop() : scrollIsRootNode ? 0 : scroll.scrollTop()) * mod,
        left: pos.left + this.offset.relative.left * mod + this.offset.parent.left * mod - (this.cssPosition === 'fixed' ? -this.scrollParent.scrollLeft() : scrollIsRootNode ? 0 : scroll.scrollLeft()) * mod
      };
    },
    _generatePosition: function (event) {
      var top, left, o = this.options, pageX = event.pageX, pageY = event.pageY, scroll = this.cssPosition === 'absolute' && !(this.scrollParent[0] !== document && $.contains(this.scrollParent[0], this.offsetParent[0])) ? this.offsetParent : this.scrollParent, scrollIsRootNode = /(html|body)/i.test(scroll[0].tagName);
      // This is another very weird special case that only happens for relative elements:
      // 1. If the css position is relative
      // 2. and the scroll parent is the document or similar to the offset parent
      // we have to refresh the relative offset during the scroll so there are no jumps
      if (this.cssPosition === 'relative' && !(this.scrollParent[0] !== document && this.scrollParent[0] !== this.offsetParent[0])) {
        this.offset.relative = this._getRelativeOffset();
      }
      /*
		 * - Position constraining -
		 * Constrain the position to a mix of grid, containment.
		 */
      if (this.originalPosition) {
        //If we are not dragging yet, we won't check for options
        if (this.containment) {
          if (event.pageX - this.offset.click.left < this.containment[0]) {
            pageX = this.containment[0] + this.offset.click.left;
          }
          if (event.pageY - this.offset.click.top < this.containment[1]) {
            pageY = this.containment[1] + this.offset.click.top;
          }
          if (event.pageX - this.offset.click.left > this.containment[2]) {
            pageX = this.containment[2] + this.offset.click.left;
          }
          if (event.pageY - this.offset.click.top > this.containment[3]) {
            pageY = this.containment[3] + this.offset.click.top;
          }
        }
        if (o.grid) {
          top = this.originalPageY + Math.round((pageY - this.originalPageY) / o.grid[1]) * o.grid[1];
          pageY = this.containment ? top - this.offset.click.top >= this.containment[1] && top - this.offset.click.top <= this.containment[3] ? top : top - this.offset.click.top >= this.containment[1] ? top - o.grid[1] : top + o.grid[1] : top;
          left = this.originalPageX + Math.round((pageX - this.originalPageX) / o.grid[0]) * o.grid[0];
          pageX = this.containment ? left - this.offset.click.left >= this.containment[0] && left - this.offset.click.left <= this.containment[2] ? left : left - this.offset.click.left >= this.containment[0] ? left - o.grid[0] : left + o.grid[0] : left;
        }
      }
      return {
        top: pageY - this.offset.click.top - this.offset.relative.top - this.offset.parent.top + (this.cssPosition === 'fixed' ? -this.scrollParent.scrollTop() : scrollIsRootNode ? 0 : scroll.scrollTop()),
        left: pageX - this.offset.click.left - this.offset.relative.left - this.offset.parent.left + (this.cssPosition === 'fixed' ? -this.scrollParent.scrollLeft() : scrollIsRootNode ? 0 : scroll.scrollLeft())
      };
    },
    _rearrange: function (event, i, a, hardRefresh) {
      a ? a[0].appendChild(this.placeholder[0]) : i.item[0].parentNode.insertBefore(this.placeholder[0], this.direction === 'down' ? i.item[0] : i.item[0].nextSibling);
      //Various things done here to improve the performance:
      // 1. we create a setTimeout, that calls refreshPositions
      // 2. on the instance, we have a counter variable, that get's higher after every append
      // 3. on the local scope, we copy the counter variable, and check in the timeout, if it's still the same
      // 4. this lets only the last addition to the timeout stack through
      this.counter = this.counter ? ++this.counter : 1;
      var counter = this.counter;
      this._delay(function () {
        if (counter === this.counter) {
          this.refreshPositions(!hardRefresh);  //Precompute after each DOM insertion, NOT on mousemove
        }
      });
    },
    _clear: function (event, noPropagation) {
      this.reverting = false;
      // We delay all events that have to be triggered to after the point where the placeholder has been removed and
      // everything else normalized again
      var i, delayedTriggers = [];
      // We first have to update the dom position of the actual currentItem
      // Note: don't do it if the current item is already removed (by a user), or it gets reappended (see #4088)
      if (!this._noFinalSort && this.currentItem.parent().length) {
        this.placeholder.before(this.currentItem);
      }
      this._noFinalSort = null;
      if (this.helper[0] === this.currentItem[0]) {
        for (i in this._storedCSS) {
          if (this._storedCSS[i] === 'auto' || this._storedCSS[i] === 'static') {
            this._storedCSS[i] = '';
          }
        }
        this.currentItem.css(this._storedCSS).removeClass('ui-sortable-helper');
      } else {
        this.currentItem.show();
      }
      if (this.fromOutside && !noPropagation) {
        delayedTriggers.push(function (event) {
          this._trigger('receive', event, this._uiHash(this.fromOutside));
        });
      }
      if ((this.fromOutside || this.domPosition.prev !== this.currentItem.prev().not('.ui-sortable-helper')[0] || this.domPosition.parent !== this.currentItem.parent()[0]) && !noPropagation) {
        delayedTriggers.push(function (event) {
          this._trigger('update', event, this._uiHash());
        });  //Trigger update callback if the DOM position has changed
      }
      // Check if the items Container has Changed and trigger appropriate
      // events.
      if (this !== this.currentContainer) {
        if (!noPropagation) {
          delayedTriggers.push(function (event) {
            this._trigger('remove', event, this._uiHash());
          });
          delayedTriggers.push(function (c) {
            return function (event) {
              c._trigger('receive', event, this._uiHash(this));
            };
          }.call(this, this.currentContainer));
          delayedTriggers.push(function (c) {
            return function (event) {
              c._trigger('update', event, this._uiHash(this));
            };
          }.call(this, this.currentContainer));
        }
      }
      //Post events to containers
      function delayEvent(type, instance, container) {
        return function (event) {
          container._trigger(type, event, instance._uiHash(instance));
        };
      }
      for (i = this.containers.length - 1; i >= 0; i--) {
        if (!noPropagation) {
          delayedTriggers.push(delayEvent('deactivate', this, this.containers[i]));
        }
        if (this.containers[i].containerCache.over) {
          delayedTriggers.push(delayEvent('out', this, this.containers[i]));
          this.containers[i].containerCache.over = 0;
        }
      }
      //Do what was originally in plugins
      if (this.storedCursor) {
        this.document.find('body').css('cursor', this.storedCursor);
        this.storedStylesheet.remove();
      }
      if (this._storedOpacity) {
        this.helper.css('opacity', this._storedOpacity);
      }
      if (this._storedZIndex) {
        this.helper.css('zIndex', this._storedZIndex === 'auto' ? '' : this._storedZIndex);
      }
      this.dragging = false;
      if (this.cancelHelperRemoval) {
        if (!noPropagation) {
          this._trigger('beforeStop', event, this._uiHash());
          for (i = 0; i < delayedTriggers.length; i++) {
            delayedTriggers[i].call(this, event);
          }
          //Trigger all delayed events
          this._trigger('stop', event, this._uiHash());
        }
        this.fromOutside = false;
        return false;
      }
      if (!noPropagation) {
        this._trigger('beforeStop', event, this._uiHash());
      }
      //$(this.placeholder[0]).remove(); would have been the jQuery way - unfortunately, it unbinds ALL events from the original node!
      this.placeholder[0].parentNode.removeChild(this.placeholder[0]);
      if (this.helper[0] !== this.currentItem[0]) {
        this.helper.remove();
      }
      this.helper = null;
      if (!noPropagation) {
        for (i = 0; i < delayedTriggers.length; i++) {
          delayedTriggers[i].call(this, event);
        }
        //Trigger all delayed events
        this._trigger('stop', event, this._uiHash());
      }
      this.fromOutside = false;
      return true;
    },
    _trigger: function () {
      if ($.Widget.prototype._trigger.apply(this, arguments) === false) {
        this.cancel();
      }
    },
    _uiHash: function (_inst) {
      var inst = _inst || this;
      return {
        helper: inst.helper,
        placeholder: inst.placeholder || $([]),
        position: inst.position,
        originalPosition: inst.originalPosition,
        offset: inst.positionAbs,
        item: inst.currentItem,
        sender: _inst ? _inst.element : null
      };
    }
  });
}(jQuery));
(function ($) {
  function modifier(fn) {
    return function () {
      var previous = this.element.val();
      fn.apply(this, arguments);
      this._refresh();
      if (previous !== this.element.val()) {
        this._trigger('change');
      }
    };
  }
  $.widget('ui.spinner', {
    version: '1.10.4',
    defaultElement: '<input>',
    widgetEventPrefix: 'spin',
    options: {
      culture: null,
      icons: {
        down: 'ui-icon-triangle-1-s',
        up: 'ui-icon-triangle-1-n'
      },
      incremental: true,
      max: null,
      min: null,
      numberFormat: null,
      page: 10,
      step: 1,
      change: null,
      spin: null,
      start: null,
      stop: null
    },
    _create: function () {
      // handle string values that need to be parsed
      this._setOption('max', this.options.max);
      this._setOption('min', this.options.min);
      this._setOption('step', this.options.step);
      // Only format if there is a value, prevents the field from being marked
      // as invalid in Firefox, see #9573.
      if (this.value() !== '') {
        // Format the value, but don't constrain.
        this._value(this.element.val(), true);
      }
      this._draw();
      this._on(this._events);
      this._refresh();
      // turning off autocomplete prevents the browser from remembering the
      // value when navigating through history, so we re-enable autocomplete
      // if the page is unloaded before the widget is destroyed. #7790
      this._on(this.window, {
        beforeunload: function () {
          this.element.removeAttr('autocomplete');
        }
      });
    },
    _getCreateOptions: function () {
      var options = {}, element = this.element;
      $.each([
        'min',
        'max',
        'step'
      ], function (i, option) {
        var value = element.attr(option);
        if (value !== undefined && value.length) {
          options[option] = value;
        }
      });
      return options;
    },
    _events: {
      keydown: function (event) {
        if (this._start(event) && this._keydown(event)) {
          event.preventDefault();
        }
      },
      keyup: '_stop',
      focus: function () {
        this.previous = this.element.val();
      },
      blur: function (event) {
        if (this.cancelBlur) {
          delete this.cancelBlur;
          return;
        }
        this._stop();
        this._refresh();
        if (this.previous !== this.element.val()) {
          this._trigger('change', event);
        }
      },
      mousewheel: function (event, delta) {
        if (!delta) {
          return;
        }
        if (!this.spinning && !this._start(event)) {
          return false;
        }
        this._spin((delta > 0 ? 1 : -1) * this.options.step, event);
        clearTimeout(this.mousewheelTimer);
        this.mousewheelTimer = this._delay(function () {
          if (this.spinning) {
            this._stop(event);
          }
        }, 100);
        event.preventDefault();
      },
      'mousedown .ui-spinner-button': function (event) {
        var previous;
        // We never want the buttons to have focus; whenever the user is
        // interacting with the spinner, the focus should be on the input.
        // If the input is focused then this.previous is properly set from
        // when the input first received focus. If the input is not focused
        // then we need to set this.previous based on the value before spinning.
        previous = this.element[0] === this.document[0].activeElement ? this.previous : this.element.val();
        function checkFocus() {
          var isActive = this.element[0] === this.document[0].activeElement;
          if (!isActive) {
            this.element.focus();
            this.previous = previous;
            // support: IE
            // IE sets focus asynchronously, so we need to check if focus
            // moved off of the input because the user clicked on the button.
            this._delay(function () {
              this.previous = previous;
            });
          }
        }
        // ensure focus is on (or stays on) the text field
        event.preventDefault();
        checkFocus.call(this);
        // support: IE
        // IE doesn't prevent moving focus even with event.preventDefault()
        // so we set a flag to know when we should ignore the blur event
        // and check (again) if focus moved off of the input.
        this.cancelBlur = true;
        this._delay(function () {
          delete this.cancelBlur;
          checkFocus.call(this);
        });
        if (this._start(event) === false) {
          return;
        }
        this._repeat(null, $(event.currentTarget).hasClass('ui-spinner-up') ? 1 : -1, event);
      },
      'mouseup .ui-spinner-button': '_stop',
      'mouseenter .ui-spinner-button': function (event) {
        // button will add ui-state-active if mouse was down while mouseleave and kept down
        if (!$(event.currentTarget).hasClass('ui-state-active')) {
          return;
        }
        if (this._start(event) === false) {
          return false;
        }
        this._repeat(null, $(event.currentTarget).hasClass('ui-spinner-up') ? 1 : -1, event);
      },
      'mouseleave .ui-spinner-button': '_stop'
    },
    _draw: function () {
      var uiSpinner = this.uiSpinner = this.element.addClass('ui-spinner-input').attr('autocomplete', 'off').wrap(this._uiSpinnerHtml()).parent().append(this._buttonHtml());
      this.element.attr('role', 'spinbutton');
      // button bindings
      this.buttons = uiSpinner.find('.ui-spinner-button').attr('tabIndex', -1).button().removeClass('ui-corner-all');
      // IE 6 doesn't understand height: 50% for the buttons
      // unless the wrapper has an explicit height
      if (this.buttons.height() > Math.ceil(uiSpinner.height() * 0.5) && uiSpinner.height() > 0) {
        uiSpinner.height(uiSpinner.height());
      }
      // disable spinner if element was already disabled
      if (this.options.disabled) {
        this.disable();
      }
    },
    _keydown: function (event) {
      var options = this.options, keyCode = $.ui.keyCode;
      switch (event.keyCode) {
      case keyCode.UP:
        this._repeat(null, 1, event);
        return true;
      case keyCode.DOWN:
        this._repeat(null, -1, event);
        return true;
      case keyCode.PAGE_UP:
        this._repeat(null, options.page, event);
        return true;
      case keyCode.PAGE_DOWN:
        this._repeat(null, -options.page, event);
        return true;
      }
      return false;
    },
    _uiSpinnerHtml: function () {
      return '<span class=\'ui-spinner ui-widget ui-widget-content ui-corner-all\'></span>';
    },
    _buttonHtml: function () {
      return '' + '<a class=\'ui-spinner-button ui-spinner-up ui-corner-tr\'>' + '<span class=\'ui-icon ' + this.options.icons.up + '\'>&#9650;</span>' + '</a>' + '<a class=\'ui-spinner-button ui-spinner-down ui-corner-br\'>' + '<span class=\'ui-icon ' + this.options.icons.down + '\'>&#9660;</span>' + '</a>';
    },
    _start: function (event) {
      if (!this.spinning && this._trigger('start', event) === false) {
        return false;
      }
      if (!this.counter) {
        this.counter = 1;
      }
      this.spinning = true;
      return true;
    },
    _repeat: function (i, steps, event) {
      i = i || 500;
      clearTimeout(this.timer);
      this.timer = this._delay(function () {
        this._repeat(40, steps, event);
      }, i);
      this._spin(steps * this.options.step, event);
    },
    _spin: function (step, event) {
      var value = this.value() || 0;
      if (!this.counter) {
        this.counter = 1;
      }
      value = this._adjustValue(value + step * this._increment(this.counter));
      if (!this.spinning || this._trigger('spin', event, { value: value }) !== false) {
        this._value(value);
        this.counter++;
      }
    },
    _increment: function (i) {
      var incremental = this.options.incremental;
      if (incremental) {
        return $.isFunction(incremental) ? incremental(i) : Math.floor(i * i * i / 50000 - i * i / 500 + 17 * i / 200 + 1);
      }
      return 1;
    },
    _precision: function () {
      var precision = this._precisionOf(this.options.step);
      if (this.options.min !== null) {
        precision = Math.max(precision, this._precisionOf(this.options.min));
      }
      return precision;
    },
    _precisionOf: function (num) {
      var str = num.toString(), decimal = str.indexOf('.');
      return decimal === -1 ? 0 : str.length - decimal - 1;
    },
    _adjustValue: function (value) {
      var base, aboveMin, options = this.options;
      // make sure we're at a valid step
      // - find out where we are relative to the base (min or 0)
      base = options.min !== null ? options.min : 0;
      aboveMin = value - base;
      // - round to the nearest step
      aboveMin = Math.round(aboveMin / options.step) * options.step;
      // - rounding is based on 0, so adjust back to our base
      value = base + aboveMin;
      // fix precision from bad JS floating point math
      value = parseFloat(value.toFixed(this._precision()));
      // clamp the value
      if (options.max !== null && value > options.max) {
        return options.max;
      }
      if (options.min !== null && value < options.min) {
        return options.min;
      }
      return value;
    },
    _stop: function (event) {
      if (!this.spinning) {
        return;
      }
      clearTimeout(this.timer);
      clearTimeout(this.mousewheelTimer);
      this.counter = 0;
      this.spinning = false;
      this._trigger('stop', event);
    },
    _setOption: function (key, value) {
      if (key === 'culture' || key === 'numberFormat') {
        var prevValue = this._parse(this.element.val());
        this.options[key] = value;
        this.element.val(this._format(prevValue));
        return;
      }
      if (key === 'max' || key === 'min' || key === 'step') {
        if (typeof value === 'string') {
          value = this._parse(value);
        }
      }
      if (key === 'icons') {
        this.buttons.first().find('.ui-icon').removeClass(this.options.icons.up).addClass(value.up);
        this.buttons.last().find('.ui-icon').removeClass(this.options.icons.down).addClass(value.down);
      }
      this._super(key, value);
      if (key === 'disabled') {
        if (value) {
          this.element.prop('disabled', true);
          this.buttons.button('disable');
        } else {
          this.element.prop('disabled', false);
          this.buttons.button('enable');
        }
      }
    },
    _setOptions: modifier(function (options) {
      this._super(options);
      this._value(this.element.val());
    }),
    _parse: function (val) {
      if (typeof val === 'string' && val !== '') {
        val = window.Globalize && this.options.numberFormat ? Globalize.parseFloat(val, 10, this.options.culture) : +val;
      }
      return val === '' || isNaN(val) ? null : val;
    },
    _format: function (value) {
      if (value === '') {
        return '';
      }
      return window.Globalize && this.options.numberFormat ? Globalize.format(value, this.options.numberFormat, this.options.culture) : value;
    },
    _refresh: function () {
      this.element.attr({
        'aria-valuemin': this.options.min,
        'aria-valuemax': this.options.max,
        'aria-valuenow': this._parse(this.element.val())
      });
    },
    _value: function (value, allowAny) {
      var parsed;
      if (value !== '') {
        parsed = this._parse(value);
        if (parsed !== null) {
          if (!allowAny) {
            parsed = this._adjustValue(parsed);
          }
          value = this._format(parsed);
        }
      }
      this.element.val(value);
      this._refresh();
    },
    _destroy: function () {
      this.element.removeClass('ui-spinner-input').prop('disabled', false).removeAttr('autocomplete').removeAttr('role').removeAttr('aria-valuemin').removeAttr('aria-valuemax').removeAttr('aria-valuenow');
      this.uiSpinner.replaceWith(this.element);
    },
    stepUp: modifier(function (steps) {
      this._stepUp(steps);
    }),
    _stepUp: function (steps) {
      if (this._start()) {
        this._spin((steps || 1) * this.options.step);
        this._stop();
      }
    },
    stepDown: modifier(function (steps) {
      this._stepDown(steps);
    }),
    _stepDown: function (steps) {
      if (this._start()) {
        this._spin((steps || 1) * -this.options.step);
        this._stop();
      }
    },
    pageUp: modifier(function (pages) {
      this._stepUp((pages || 1) * this.options.page);
    }),
    pageDown: modifier(function (pages) {
      this._stepDown((pages || 1) * this.options.page);
    }),
    value: function (newVal) {
      if (!arguments.length) {
        return this._parse(this.element.val());
      }
      modifier(this._value).call(this, newVal);
    },
    widget: function () {
      return this.uiSpinner;
    }
  });
}(jQuery));
(function ($, undefined) {
  var tabId = 0, rhash = /#.*$/;
  function getNextTabId() {
    return ++tabId;
  }
  function isLocal(anchor) {
    // support: IE7
    // IE7 doesn't normalize the href property when set via script (#9317)
    anchor = anchor.cloneNode(false);
    return anchor.hash.length > 1 && decodeURIComponent(anchor.href.replace(rhash, '')) === decodeURIComponent(location.href.replace(rhash, ''));
  }
  $.widget('ui.tabs', {
    version: '1.10.4',
    delay: 300,
    options: {
      active: null,
      collapsible: false,
      event: 'click',
      heightStyle: 'content',
      hide: null,
      show: null,
      activate: null,
      beforeActivate: null,
      beforeLoad: null,
      load: null
    },
    _create: function () {
      var that = this, options = this.options;
      this.running = false;
      this.element.addClass('ui-tabs ui-widget ui-widget-content ui-corner-all').toggleClass('ui-tabs-collapsible', options.collapsible).delegate('.ui-tabs-nav > li', 'mousedown' + this.eventNamespace, function (event) {
        if ($(this).is('.ui-state-disabled')) {
          event.preventDefault();
        }
      }).delegate('.ui-tabs-anchor', 'focus' + this.eventNamespace, function () {
        if ($(this).closest('li').is('.ui-state-disabled')) {
          this.blur();
        }
      });
      this._processTabs();
      options.active = this._initialActive();
      // Take disabling tabs via class attribute from HTML
      // into account and update option properly.
      if ($.isArray(options.disabled)) {
        options.disabled = $.unique(options.disabled.concat($.map(this.tabs.filter('.ui-state-disabled'), function (li) {
          return that.tabs.index(li);
        }))).sort();
      }
      // check for length avoids error when initializing empty list
      if (this.options.active !== false && this.anchors.length) {
        this.active = this._findActive(options.active);
      } else {
        this.active = $();
      }
      this._refresh();
      if (this.active.length) {
        this.load(options.active);
      }
    },
    _initialActive: function () {
      var active = this.options.active, collapsible = this.options.collapsible, locationHash = location.hash.substring(1);
      if (active === null) {
        // check the fragment identifier in the URL
        if (locationHash) {
          this.tabs.each(function (i, tab) {
            if ($(tab).attr('aria-controls') === locationHash) {
              active = i;
              return false;
            }
          });
        }
        // check for a tab marked active via a class
        if (active === null) {
          active = this.tabs.index(this.tabs.filter('.ui-tabs-active'));
        }
        // no active tab, set to false
        if (active === null || active === -1) {
          active = this.tabs.length ? 0 : false;
        }
      }
      // handle numbers: negative, out of range
      if (active !== false) {
        active = this.tabs.index(this.tabs.eq(active));
        if (active === -1) {
          active = collapsible ? false : 0;
        }
      }
      // don't allow collapsible: false and active: false
      if (!collapsible && active === false && this.anchors.length) {
        active = 0;
      }
      return active;
    },
    _getCreateEventData: function () {
      return {
        tab: this.active,
        panel: !this.active.length ? $() : this._getPanelForTab(this.active)
      };
    },
    _tabKeydown: function (event) {
      var focusedTab = $(this.document[0].activeElement).closest('li'), selectedIndex = this.tabs.index(focusedTab), goingForward = true;
      if (this._handlePageNav(event)) {
        return;
      }
      switch (event.keyCode) {
      case $.ui.keyCode.RIGHT:
      case $.ui.keyCode.DOWN:
        selectedIndex++;
        break;
      case $.ui.keyCode.UP:
      case $.ui.keyCode.LEFT:
        goingForward = false;
        selectedIndex--;
        break;
      case $.ui.keyCode.END:
        selectedIndex = this.anchors.length - 1;
        break;
      case $.ui.keyCode.HOME:
        selectedIndex = 0;
        break;
      case $.ui.keyCode.SPACE:
        // Activate only, no collapsing
        event.preventDefault();
        clearTimeout(this.activating);
        this._activate(selectedIndex);
        return;
      case $.ui.keyCode.ENTER:
        // Toggle (cancel delayed activation, allow collapsing)
        event.preventDefault();
        clearTimeout(this.activating);
        // Determine if we should collapse or activate
        this._activate(selectedIndex === this.options.active ? false : selectedIndex);
        return;
      default:
        return;
      }
      // Focus the appropriate tab, based on which key was pressed
      event.preventDefault();
      clearTimeout(this.activating);
      selectedIndex = this._focusNextTab(selectedIndex, goingForward);
      // Navigating with control key will prevent automatic activation
      if (!event.ctrlKey) {
        // Update aria-selected immediately so that AT think the tab is already selected.
        // Otherwise AT may confuse the user by stating that they need to activate the tab,
        // but the tab will already be activated by the time the announcement finishes.
        focusedTab.attr('aria-selected', 'false');
        this.tabs.eq(selectedIndex).attr('aria-selected', 'true');
        this.activating = this._delay(function () {
          this.option('active', selectedIndex);
        }, this.delay);
      }
    },
    _panelKeydown: function (event) {
      if (this._handlePageNav(event)) {
        return;
      }
      // Ctrl+up moves focus to the current tab
      if (event.ctrlKey && event.keyCode === $.ui.keyCode.UP) {
        event.preventDefault();
        this.active.focus();
      }
    },
    _handlePageNav: function (event) {
      if (event.altKey && event.keyCode === $.ui.keyCode.PAGE_UP) {
        this._activate(this._focusNextTab(this.options.active - 1, false));
        return true;
      }
      if (event.altKey && event.keyCode === $.ui.keyCode.PAGE_DOWN) {
        this._activate(this._focusNextTab(this.options.active + 1, true));
        return true;
      }
    },
    _findNextTab: function (index, goingForward) {
      var lastTabIndex = this.tabs.length - 1;
      function constrain() {
        if (index > lastTabIndex) {
          index = 0;
        }
        if (index < 0) {
          index = lastTabIndex;
        }
        return index;
      }
      while ($.inArray(constrain(), this.options.disabled) !== -1) {
        index = goingForward ? index + 1 : index - 1;
      }
      return index;
    },
    _focusNextTab: function (index, goingForward) {
      index = this._findNextTab(index, goingForward);
      this.tabs.eq(index).focus();
      return index;
    },
    _setOption: function (key, value) {
      if (key === 'active') {
        // _activate() will handle invalid values and update this.options
        this._activate(value);
        return;
      }
      if (key === 'disabled') {
        // don't use the widget factory's disabled handling
        this._setupDisabled(value);
        return;
      }
      this._super(key, value);
      if (key === 'collapsible') {
        this.element.toggleClass('ui-tabs-collapsible', value);
        // Setting collapsible: false while collapsed; open first panel
        if (!value && this.options.active === false) {
          this._activate(0);
        }
      }
      if (key === 'event') {
        this._setupEvents(value);
      }
      if (key === 'heightStyle') {
        this._setupHeightStyle(value);
      }
    },
    _tabId: function (tab) {
      return tab.attr('aria-controls') || 'ui-tabs-' + getNextTabId();
    },
    _sanitizeSelector: function (hash) {
      return hash ? hash.replace(/[!"$%&'()*+,.\/:;<=>?@\[\]\^`{|}~]/g, '\\$&') : '';
    },
    refresh: function () {
      var options = this.options, lis = this.tablist.children(':has(a[href])');
      // get disabled tabs from class attribute from HTML
      // this will get converted to a boolean if needed in _refresh()
      options.disabled = $.map(lis.filter('.ui-state-disabled'), function (tab) {
        return lis.index(tab);
      });
      this._processTabs();
      // was collapsed or no tabs
      if (options.active === false || !this.anchors.length) {
        options.active = false;
        this.active = $();  // was active, but active tab is gone
      } else if (this.active.length && !$.contains(this.tablist[0], this.active[0])) {
        // all remaining tabs are disabled
        if (this.tabs.length === options.disabled.length) {
          options.active = false;
          this.active = $();  // activate previous tab
        } else {
          this._activate(this._findNextTab(Math.max(0, options.active - 1), false));
        }  // was active, active tab still exists
      } else {
        // make sure active index is correct
        options.active = this.tabs.index(this.active);
      }
      this._refresh();
    },
    _refresh: function () {
      this._setupDisabled(this.options.disabled);
      this._setupEvents(this.options.event);
      this._setupHeightStyle(this.options.heightStyle);
      this.tabs.not(this.active).attr({
        'aria-selected': 'false',
        tabIndex: -1
      });
      this.panels.not(this._getPanelForTab(this.active)).hide().attr({
        'aria-expanded': 'false',
        'aria-hidden': 'true'
      });
      // Make sure one tab is in the tab order
      if (!this.active.length) {
        this.tabs.eq(0).attr('tabIndex', 0);
      } else {
        this.active.addClass('ui-tabs-active ui-state-active').attr({
          'aria-selected': 'true',
          tabIndex: 0
        });
        this._getPanelForTab(this.active).show().attr({
          'aria-expanded': 'true',
          'aria-hidden': 'false'
        });
      }
    },
    _processTabs: function () {
      var that = this;
      this.tablist = this._getList().addClass('ui-tabs-nav ui-helper-reset ui-helper-clearfix ui-widget-header ui-corner-all').attr('role', 'tablist');
      this.tabs = this.tablist.find('> li:has(a[href])').addClass('ui-state-default ui-corner-top').attr({
        role: 'tab',
        tabIndex: -1
      });
      this.anchors = this.tabs.map(function () {
        return $('a', this)[0];
      }).addClass('ui-tabs-anchor').attr({
        role: 'presentation',
        tabIndex: -1
      });
      this.panels = $();
      this.anchors.each(function (i, anchor) {
        var selector, panel, panelId, anchorId = $(anchor).uniqueId().attr('id'), tab = $(anchor).closest('li'), originalAriaControls = tab.attr('aria-controls');
        // inline tab
        if (isLocal(anchor)) {
          selector = anchor.hash;
          panel = that.element.find(that._sanitizeSelector(selector));  // remote tab
        } else {
          panelId = that._tabId(tab);
          selector = '#' + panelId;
          panel = that.element.find(selector);
          if (!panel.length) {
            panel = that._createPanel(panelId);
            panel.insertAfter(that.panels[i - 1] || that.tablist);
          }
          panel.attr('aria-live', 'polite');
        }
        if (panel.length) {
          that.panels = that.panels.add(panel);
        }
        if (originalAriaControls) {
          tab.data('ui-tabs-aria-controls', originalAriaControls);
        }
        tab.attr({
          'aria-controls': selector.substring(1),
          'aria-labelledby': anchorId
        });
        panel.attr('aria-labelledby', anchorId);
      });
      this.panels.addClass('ui-tabs-panel ui-widget-content ui-corner-bottom').attr('role', 'tabpanel');
    },
    _getList: function () {
      return this.tablist || this.element.find('ol,ul').eq(0);
    },
    _createPanel: function (id) {
      return $('<div>').attr('id', id).addClass('ui-tabs-panel ui-widget-content ui-corner-bottom').data('ui-tabs-destroy', true);
    },
    _setupDisabled: function (disabled) {
      if ($.isArray(disabled)) {
        if (!disabled.length) {
          disabled = false;
        } else if (disabled.length === this.anchors.length) {
          disabled = true;
        }
      }
      // disable tabs
      for (var i = 0, li; li = this.tabs[i]; i++) {
        if (disabled === true || $.inArray(i, disabled) !== -1) {
          $(li).addClass('ui-state-disabled').attr('aria-disabled', 'true');
        } else {
          $(li).removeClass('ui-state-disabled').removeAttr('aria-disabled');
        }
      }
      this.options.disabled = disabled;
    },
    _setupEvents: function (event) {
      var events = {
          click: function (event) {
            event.preventDefault();
          }
        };
      if (event) {
        $.each(event.split(' '), function (index, eventName) {
          events[eventName] = '_eventHandler';
        });
      }
      this._off(this.anchors.add(this.tabs).add(this.panels));
      this._on(this.anchors, events);
      this._on(this.tabs, { keydown: '_tabKeydown' });
      this._on(this.panels, { keydown: '_panelKeydown' });
      this._focusable(this.tabs);
      this._hoverable(this.tabs);
    },
    _setupHeightStyle: function (heightStyle) {
      var maxHeight, parent = this.element.parent();
      if (heightStyle === 'fill') {
        maxHeight = parent.height();
        maxHeight -= this.element.outerHeight() - this.element.height();
        this.element.siblings(':visible').each(function () {
          var elem = $(this), position = elem.css('position');
          if (position === 'absolute' || position === 'fixed') {
            return;
          }
          maxHeight -= elem.outerHeight(true);
        });
        this.element.children().not(this.panels).each(function () {
          maxHeight -= $(this).outerHeight(true);
        });
        this.panels.each(function () {
          $(this).height(Math.max(0, maxHeight - $(this).innerHeight() + $(this).height()));
        }).css('overflow', 'auto');
      } else if (heightStyle === 'auto') {
        maxHeight = 0;
        this.panels.each(function () {
          maxHeight = Math.max(maxHeight, $(this).height('').height());
        }).height(maxHeight);
      }
    },
    _eventHandler: function (event) {
      var options = this.options, active = this.active, anchor = $(event.currentTarget), tab = anchor.closest('li'), clickedIsActive = tab[0] === active[0], collapsing = clickedIsActive && options.collapsible, toShow = collapsing ? $() : this._getPanelForTab(tab), toHide = !active.length ? $() : this._getPanelForTab(active), eventData = {
          oldTab: active,
          oldPanel: toHide,
          newTab: collapsing ? $() : tab,
          newPanel: toShow
        };
      event.preventDefault();
      if (tab.hasClass('ui-state-disabled') || tab.hasClass('ui-tabs-loading') || this.running || clickedIsActive && !options.collapsible || this._trigger('beforeActivate', event, eventData) === false) {
        return;
      }
      options.active = collapsing ? false : this.tabs.index(tab);
      this.active = clickedIsActive ? $() : tab;
      if (this.xhr) {
        this.xhr.abort();
      }
      if (!toHide.length && !toShow.length) {
        $.error('jQuery UI Tabs: Mismatching fragment identifier.');
      }
      if (toShow.length) {
        this.load(this.tabs.index(tab), event);
      }
      this._toggle(event, eventData);
    },
    _toggle: function (event, eventData) {
      var that = this, toShow = eventData.newPanel, toHide = eventData.oldPanel;
      this.running = true;
      function complete() {
        that.running = false;
        that._trigger('activate', event, eventData);
      }
      function show() {
        eventData.newTab.closest('li').addClass('ui-tabs-active ui-state-active');
        if (toShow.length && that.options.show) {
          that._show(toShow, that.options.show, complete);
        } else {
          toShow.show();
          complete();
        }
      }
      // start out by hiding, then showing, then completing
      if (toHide.length && this.options.hide) {
        this._hide(toHide, this.options.hide, function () {
          eventData.oldTab.closest('li').removeClass('ui-tabs-active ui-state-active');
          show();
        });
      } else {
        eventData.oldTab.closest('li').removeClass('ui-tabs-active ui-state-active');
        toHide.hide();
        show();
      }
      toHide.attr({
        'aria-expanded': 'false',
        'aria-hidden': 'true'
      });
      eventData.oldTab.attr('aria-selected', 'false');
      // If we're switching tabs, remove the old tab from the tab order.
      // If we're opening from collapsed state, remove the previous tab from the tab order.
      // If we're collapsing, then keep the collapsing tab in the tab order.
      if (toShow.length && toHide.length) {
        eventData.oldTab.attr('tabIndex', -1);
      } else if (toShow.length) {
        this.tabs.filter(function () {
          return $(this).attr('tabIndex') === 0;
        }).attr('tabIndex', -1);
      }
      toShow.attr({
        'aria-expanded': 'true',
        'aria-hidden': 'false'
      });
      eventData.newTab.attr({
        'aria-selected': 'true',
        tabIndex: 0
      });
    },
    _activate: function (index) {
      var anchor, active = this._findActive(index);
      // trying to activate the already active panel
      if (active[0] === this.active[0]) {
        return;
      }
      // trying to collapse, simulate a click on the current active header
      if (!active.length) {
        active = this.active;
      }
      anchor = active.find('.ui-tabs-anchor')[0];
      this._eventHandler({
        target: anchor,
        currentTarget: anchor,
        preventDefault: $.noop
      });
    },
    _findActive: function (index) {
      return index === false ? $() : this.tabs.eq(index);
    },
    _getIndex: function (index) {
      // meta-function to give users option to provide a href string instead of a numerical index.
      if (typeof index === 'string') {
        index = this.anchors.index(this.anchors.filter('[href$=\'' + index + '\']'));
      }
      return index;
    },
    _destroy: function () {
      if (this.xhr) {
        this.xhr.abort();
      }
      this.element.removeClass('ui-tabs ui-widget ui-widget-content ui-corner-all ui-tabs-collapsible');
      this.tablist.removeClass('ui-tabs-nav ui-helper-reset ui-helper-clearfix ui-widget-header ui-corner-all').removeAttr('role');
      this.anchors.removeClass('ui-tabs-anchor').removeAttr('role').removeAttr('tabIndex').removeUniqueId();
      this.tabs.add(this.panels).each(function () {
        if ($.data(this, 'ui-tabs-destroy')) {
          $(this).remove();
        } else {
          $(this).removeClass('ui-state-default ui-state-active ui-state-disabled ' + 'ui-corner-top ui-corner-bottom ui-widget-content ui-tabs-active ui-tabs-panel').removeAttr('tabIndex').removeAttr('aria-live').removeAttr('aria-busy').removeAttr('aria-selected').removeAttr('aria-labelledby').removeAttr('aria-hidden').removeAttr('aria-expanded').removeAttr('role');
        }
      });
      this.tabs.each(function () {
        var li = $(this), prev = li.data('ui-tabs-aria-controls');
        if (prev) {
          li.attr('aria-controls', prev).removeData('ui-tabs-aria-controls');
        } else {
          li.removeAttr('aria-controls');
        }
      });
      this.panels.show();
      if (this.options.heightStyle !== 'content') {
        this.panels.css('height', '');
      }
    },
    enable: function (index) {
      var disabled = this.options.disabled;
      if (disabled === false) {
        return;
      }
      if (index === undefined) {
        disabled = false;
      } else {
        index = this._getIndex(index);
        if ($.isArray(disabled)) {
          disabled = $.map(disabled, function (num) {
            return num !== index ? num : null;
          });
        } else {
          disabled = $.map(this.tabs, function (li, num) {
            return num !== index ? num : null;
          });
        }
      }
      this._setupDisabled(disabled);
    },
    disable: function (index) {
      var disabled = this.options.disabled;
      if (disabled === true) {
        return;
      }
      if (index === undefined) {
        disabled = true;
      } else {
        index = this._getIndex(index);
        if ($.inArray(index, disabled) !== -1) {
          return;
        }
        if ($.isArray(disabled)) {
          disabled = $.merge([index], disabled).sort();
        } else {
          disabled = [index];
        }
      }
      this._setupDisabled(disabled);
    },
    load: function (index, event) {
      index = this._getIndex(index);
      var that = this, tab = this.tabs.eq(index), anchor = tab.find('.ui-tabs-anchor'), panel = this._getPanelForTab(tab), eventData = {
          tab: tab,
          panel: panel
        };
      // not remote
      if (isLocal(anchor[0])) {
        return;
      }
      this.xhr = $.ajax(this._ajaxSettings(anchor, event, eventData));
      // support: jQuery <1.8
      // jQuery <1.8 returns false if the request is canceled in beforeSend,
      // but as of 1.8, $.ajax() always returns a jqXHR object.
      if (this.xhr && this.xhr.statusText !== 'canceled') {
        tab.addClass('ui-tabs-loading');
        panel.attr('aria-busy', 'true');
        this.xhr.success(function (response) {
          // support: jQuery <1.8
          // http://bugs.jquery.com/ticket/11778
          setTimeout(function () {
            panel.html(response);
            that._trigger('load', event, eventData);
          }, 1);
        }).complete(function (jqXHR, status) {
          // support: jQuery <1.8
          // http://bugs.jquery.com/ticket/11778
          setTimeout(function () {
            if (status === 'abort') {
              that.panels.stop(false, true);
            }
            tab.removeClass('ui-tabs-loading');
            panel.removeAttr('aria-busy');
            if (jqXHR === that.xhr) {
              delete that.xhr;
            }
          }, 1);
        });
      }
    },
    _ajaxSettings: function (anchor, event, eventData) {
      var that = this;
      return {
        url: anchor.attr('href'),
        beforeSend: function (jqXHR, settings) {
          return that._trigger('beforeLoad', event, $.extend({
            jqXHR: jqXHR,
            ajaxSettings: settings
          }, eventData));
        }
      };
    },
    _getPanelForTab: function (tab) {
      var id = $(tab).attr('aria-controls');
      return this.element.find(this._sanitizeSelector('#' + id));
    }
  });
}(jQuery));
(function ($) {
  var increments = 0;
  function addDescribedBy(elem, id) {
    var describedby = (elem.attr('aria-describedby') || '').split(/\s+/);
    describedby.push(id);
    elem.data('ui-tooltip-id', id).attr('aria-describedby', $.trim(describedby.join(' ')));
  }
  function removeDescribedBy(elem) {
    var id = elem.data('ui-tooltip-id'), describedby = (elem.attr('aria-describedby') || '').split(/\s+/), index = $.inArray(id, describedby);
    if (index !== -1) {
      describedby.splice(index, 1);
    }
    elem.removeData('ui-tooltip-id');
    describedby = $.trim(describedby.join(' '));
    if (describedby) {
      elem.attr('aria-describedby', describedby);
    } else {
      elem.removeAttr('aria-describedby');
    }
  }
  $.widget('ui.tooltip', {
    version: '1.10.4',
    options: {
      content: function () {
        // support: IE<9, Opera in jQuery <1.7
        // .text() can't accept undefined, so coerce to a string
        var title = $(this).attr('title') || '';
        // Escape title, since we're going from an attribute to raw HTML
        return $('<a>').text(title).html();
      },
      hide: true,
      items: '[title]:not([disabled])',
      position: {
        my: 'left top+15',
        at: 'left bottom',
        collision: 'flipfit flip'
      },
      show: true,
      tooltipClass: null,
      track: false,
      close: null,
      open: null
    },
    _create: function () {
      this._on({
        mouseover: 'open',
        focusin: 'open'
      });
      // IDs of generated tooltips, needed for destroy
      this.tooltips = {};
      // IDs of parent tooltips where we removed the title attribute
      this.parents = {};
      if (this.options.disabled) {
        this._disable();
      }
    },
    _setOption: function (key, value) {
      var that = this;
      if (key === 'disabled') {
        this[value ? '_disable' : '_enable']();
        this.options[key] = value;
        // disable element style changes
        return;
      }
      this._super(key, value);
      if (key === 'content') {
        $.each(this.tooltips, function (id, element) {
          that._updateContent(element);
        });
      }
    },
    _disable: function () {
      var that = this;
      // close open tooltips
      $.each(this.tooltips, function (id, element) {
        var event = $.Event('blur');
        event.target = event.currentTarget = element[0];
        that.close(event, true);
      });
      // remove title attributes to prevent native tooltips
      this.element.find(this.options.items).addBack().each(function () {
        var element = $(this);
        if (element.is('[title]')) {
          element.data('ui-tooltip-title', element.attr('title')).attr('title', '');
        }
      });
    },
    _enable: function () {
      // restore title attributes
      this.element.find(this.options.items).addBack().each(function () {
        var element = $(this);
        if (element.data('ui-tooltip-title')) {
          element.attr('title', element.data('ui-tooltip-title'));
        }
      });
    },
    open: function (event) {
      var that = this, target = $(event ? event.target : this.element).closest(this.options.items);
      // No element to show a tooltip for or the tooltip is already open
      if (!target.length || target.data('ui-tooltip-id')) {
        return;
      }
      if (target.attr('title')) {
        target.data('ui-tooltip-title', target.attr('title'));
      }
      target.data('ui-tooltip-open', true);
      // kill parent tooltips, custom or native, for hover
      if (event && event.type === 'mouseover') {
        target.parents().each(function () {
          var parent = $(this), blurEvent;
          if (parent.data('ui-tooltip-open')) {
            blurEvent = $.Event('blur');
            blurEvent.target = blurEvent.currentTarget = this;
            that.close(blurEvent, true);
          }
          if (parent.attr('title')) {
            parent.uniqueId();
            that.parents[this.id] = {
              element: this,
              title: parent.attr('title')
            };
            parent.attr('title', '');
          }
        });
      }
      this._updateContent(target, event);
    },
    _updateContent: function (target, event) {
      var content, contentOption = this.options.content, that = this, eventType = event ? event.type : null;
      if (typeof contentOption === 'string') {
        return this._open(event, target, contentOption);
      }
      content = contentOption.call(target[0], function (response) {
        // ignore async response if tooltip was closed already
        if (!target.data('ui-tooltip-open')) {
          return;
        }
        // IE may instantly serve a cached response for ajax requests
        // delay this call to _open so the other call to _open runs first
        that._delay(function () {
          // jQuery creates a special event for focusin when it doesn't
          // exist natively. To improve performance, the native event
          // object is reused and the type is changed. Therefore, we can't
          // rely on the type being correct after the event finished
          // bubbling, so we set it back to the previous value. (#8740)
          if (event) {
            event.type = eventType;
          }
          this._open(event, target, response);
        });
      });
      if (content) {
        this._open(event, target, content);
      }
    },
    _open: function (event, target, content) {
      var tooltip, events, delayedShow, positionOption = $.extend({}, this.options.position);
      if (!content) {
        return;
      }
      // Content can be updated multiple times. If the tooltip already
      // exists, then just update the content and bail.
      tooltip = this._find(target);
      if (tooltip.length) {
        tooltip.find('.ui-tooltip-content').html(content);
        return;
      }
      // if we have a title, clear it to prevent the native tooltip
      // we have to check first to avoid defining a title if none exists
      // (we don't want to cause an element to start matching [title])
      //
      // We use removeAttr only for key events, to allow IE to export the correct
      // accessible attributes. For mouse events, set to empty string to avoid
      // native tooltip showing up (happens only when removing inside mouseover).
      if (target.is('[title]')) {
        if (event && event.type === 'mouseover') {
          target.attr('title', '');
        } else {
          target.removeAttr('title');
        }
      }
      tooltip = this._tooltip(target);
      addDescribedBy(target, tooltip.attr('id'));
      tooltip.find('.ui-tooltip-content').html(content);
      function position(event) {
        positionOption.of = event;
        if (tooltip.is(':hidden')) {
          return;
        }
        tooltip.position(positionOption);
      }
      if (this.options.track && event && /^mouse/.test(event.type)) {
        this._on(this.document, { mousemove: position });
        // trigger once to override element-relative positioning
        position(event);
      } else {
        tooltip.position($.extend({ of: target }, this.options.position));
      }
      tooltip.hide();
      this._show(tooltip, this.options.show);
      // Handle tracking tooltips that are shown with a delay (#8644). As soon
      // as the tooltip is visible, position the tooltip using the most recent
      // event.
      if (this.options.show && this.options.show.delay) {
        delayedShow = this.delayedShow = setInterval(function () {
          if (tooltip.is(':visible')) {
            position(positionOption.of);
            clearInterval(delayedShow);
          }
        }, $.fx.interval);
      }
      this._trigger('open', event, { tooltip: tooltip });
      events = {
        keyup: function (event) {
          if (event.keyCode === $.ui.keyCode.ESCAPE) {
            var fakeEvent = $.Event(event);
            fakeEvent.currentTarget = target[0];
            this.close(fakeEvent, true);
          }
        },
        remove: function () {
          this._removeTooltip(tooltip);
        }
      };
      if (!event || event.type === 'mouseover') {
        events.mouseleave = 'close';
      }
      if (!event || event.type === 'focusin') {
        events.focusout = 'close';
      }
      this._on(true, target, events);
    },
    close: function (event) {
      var that = this, target = $(event ? event.currentTarget : this.element), tooltip = this._find(target);
      // disabling closes the tooltip, so we need to track when we're closing
      // to avoid an infinite loop in case the tooltip becomes disabled on close
      if (this.closing) {
        return;
      }
      // Clear the interval for delayed tracking tooltips
      clearInterval(this.delayedShow);
      // only set title if we had one before (see comment in _open())
      if (target.data('ui-tooltip-title')) {
        target.attr('title', target.data('ui-tooltip-title'));
      }
      removeDescribedBy(target);
      tooltip.stop(true);
      this._hide(tooltip, this.options.hide, function () {
        that._removeTooltip($(this));
      });
      target.removeData('ui-tooltip-open');
      this._off(target, 'mouseleave focusout keyup');
      // Remove 'remove' binding only on delegated targets
      if (target[0] !== this.element[0]) {
        this._off(target, 'remove');
      }
      this._off(this.document, 'mousemove');
      if (event && event.type === 'mouseleave') {
        $.each(this.parents, function (id, parent) {
          $(parent.element).attr('title', parent.title);
          delete that.parents[id];
        });
      }
      this.closing = true;
      this._trigger('close', event, { tooltip: tooltip });
      this.closing = false;
    },
    _tooltip: function (element) {
      var id = 'ui-tooltip-' + increments++, tooltip = $('<div>').attr({
          id: id,
          role: 'tooltip'
        }).addClass('ui-tooltip ui-widget ui-corner-all ui-widget-content ' + (this.options.tooltipClass || ''));
      $('<div>').addClass('ui-tooltip-content').appendTo(tooltip);
      tooltip.appendTo(this.document[0].body);
      this.tooltips[id] = element;
      return tooltip;
    },
    _find: function (target) {
      var id = target.data('ui-tooltip-id');
      return id ? $('#' + id) : $();
    },
    _removeTooltip: function (tooltip) {
      tooltip.remove();
      delete this.tooltips[tooltip.attr('id')];
    },
    _destroy: function () {
      var that = this;
      // close open tooltips
      $.each(this.tooltips, function (id, element) {
        // Delegate to close method to handle common cleanup
        var event = $.Event('blur');
        event.target = event.currentTarget = element[0];
        that.close(event, true);
        // Remove immediately; destroying an open tooltip doesn't use the
        // hide animation
        $('#' + id).remove();
        // Restore the title
        if (element.data('ui-tooltip-title')) {
          element.attr('title', element.data('ui-tooltip-title'));
          element.removeData('ui-tooltip-title');
        }
      });
    }
  });
}(jQuery));
/*
*  AngularJs Fullcalendar Wrapper for the JQuery FullCalendar
*  API @ http://arshaw.com/fullcalendar/
*
*  Angular Calendar Directive that takes in the [eventSources] nested array object as the ng-model and watches it deeply changes.
*       Can also take in multiple event urls as a source object(s) and feed the events per view.
*       The calendar will watch any eventSource array and update itself when a change is made.
*
*/
angular.module('ui.calendar', []).constant('uiCalendarConfig', {}).controller('uiCalendarCtrl', [
  '$scope',
  '$timeout',
  function ($scope, $timeout) {
    var sourceSerialId = 1, eventSerialId = 1, sources = $scope.eventSources, extraEventSignature = $scope.calendarWatchEvent ? $scope.calendarWatchEvent : angular.noop, wrapFunctionWithScopeApply = function (functionToWrap) {
        var wrapper;
        if (functionToWrap) {
          wrapper = function () {
            // This happens outside of angular context so we need to wrap it in a timeout which has an implied apply.
            // In this way the function will be safely executed on the next digest.
            var args = arguments;
            $timeout(function () {
              functionToWrap.apply(this, args);
            });
          };
        }
        return wrapper;
      };
    this.eventsFingerprint = function (e) {
      if (!e.__uiCalId) {
        e.__uiCalId = eventSerialId++;
      }
      // This extracts all the information we need from the event. http://jsperf.com/angular-calendar-events-fingerprint/3
      return '' + e.__uiCalId + (e.id || '') + (e.title || '') + (e.url || '') + (+e.start || '') + (+e.end || '') + (e.allDay || '') + (e.className || '') + extraEventSignature(e) || '';
    };
    this.sourcesFingerprint = function (source) {
      return source.__id || (source.__id = sourceSerialId++);
    };
    this.allEvents = function () {
      // return sources.flatten(); but we don't have flatten
      var arraySources = [];
      for (var i = 0, srcLen = sources.length; i < srcLen; i++) {
        var source = sources[i];
        if (angular.isArray(source)) {
          // event source as array
          arraySources.push(source);
        } else if (angular.isObject(source) && angular.isArray(source.events)) {
          // event source as object, ie extended form
          var extEvent = {};
          for (var key in source) {
            if (key !== '_uiCalId' && key !== 'events') {
              extEvent[key] = source[key];
            }
          }
          for (var eI = 0; eI < source.events.length; eI++) {
            angular.extend(source.events[eI], extEvent);
          }
          arraySources.push(source.events);
        }
      }
      return Array.prototype.concat.apply([], arraySources);
    };
    // Track changes in array by assigning id tokens to each element and watching the scope for changes in those tokens
    // arguments:
    //  arraySource array of function that returns array of objects to watch
    //  tokenFn function(object) that returns the token for a given object
    this.changeWatcher = function (arraySource, tokenFn) {
      var self;
      var getTokens = function () {
        var array = angular.isFunction(arraySource) ? arraySource() : arraySource;
        var result = [], token, el;
        for (var i = 0, n = array.length; i < n; i++) {
          el = array[i];
          token = tokenFn(el);
          map[token] = el;
          result.push(token);
        }
        return result;
      };
      // returns elements in that are in a but not in b
      // subtractAsSets([4, 5, 6], [4, 5, 7]) => [6]
      var subtractAsSets = function (a, b) {
        var result = [], inB = {}, i, n;
        for (i = 0, n = b.length; i < n; i++) {
          inB[b[i]] = true;
        }
        for (i = 0, n = a.length; i < n; i++) {
          if (!inB[a[i]]) {
            result.push(a[i]);
          }
        }
        return result;
      };
      // Map objects to tokens and vice-versa
      var map = {};
      var applyChanges = function (newTokens, oldTokens) {
        var i, n, el, token;
        var replacedTokens = {};
        var removedTokens = subtractAsSets(oldTokens, newTokens);
        for (i = 0, n = removedTokens.length; i < n; i++) {
          var removedToken = removedTokens[i];
          el = map[removedToken];
          delete map[removedToken];
          var newToken = tokenFn(el);
          // if the element wasn't removed but simply got a new token, its old token will be different from the current one
          if (newToken === removedToken) {
            self.onRemoved(el);
          } else {
            replacedTokens[newToken] = removedToken;
            self.onChanged(el);
          }
        }
        var addedTokens = subtractAsSets(newTokens, oldTokens);
        for (i = 0, n = addedTokens.length; i < n; i++) {
          token = addedTokens[i];
          el = map[token];
          if (!replacedTokens[token]) {
            self.onAdded(el);
          }
        }
      };
      return self = {
        subscribe: function (scope, onChanged) {
          scope.$watch(getTokens, function (newTokens, oldTokens) {
            if (!onChanged || onChanged(newTokens, oldTokens) !== false) {
              applyChanges(newTokens, oldTokens);
            }
          }, true);
        },
        onAdded: angular.noop,
        onChanged: angular.noop,
        onRemoved: angular.noop
      };
    };
    this.getFullCalendarConfig = function (calendarSettings, uiCalendarConfig) {
      var config = {};
      angular.extend(config, uiCalendarConfig);
      angular.extend(config, calendarSettings);
      angular.forEach(config, function (value, key) {
        if (typeof value === 'function') {
          config[key] = wrapFunctionWithScopeApply(config[key]);
        }
      });
      return config;
    };
  }
]).directive('uiCalendar', [
  'uiCalendarConfig',
  '$locale',
  function (uiCalendarConfig, $locale) {
    // Configure to use locale names by default
    var tValues = function (data) {
      // convert {0: "Jan", 1: "Feb", ...} to ["Jan", "Feb", ...]
      var r, k;
      r = [];
      for (k in data) {
        r[k] = data[k];
      }
      return r;
    };
    var dtf = $locale.DATETIME_FORMATS;
    uiCalendarConfig = angular.extend({
      monthNames: tValues(dtf.MONTH),
      monthNamesShort: tValues(dtf.SHORTMONTH),
      dayNames: tValues(dtf.DAY),
      dayNamesShort: tValues(dtf.SHORTDAY)
    }, uiCalendarConfig || {});
    return {
      restrict: 'A',
      scope: {
        eventSources: '=ngModel',
        calendarWatchEvent: '&'
      },
      controller: 'uiCalendarCtrl',
      link: function (scope, elm, attrs, controller) {
        var sources = scope.eventSources, sourcesChanged = false, eventSourcesWatcher = controller.changeWatcher(sources, controller.sourcesFingerprint), eventsWatcher = controller.changeWatcher(controller.allEvents, controller.eventsFingerprint), options = null;
        function getOptions() {
          var calendarSettings = attrs.uiCalendar ? scope.$parent.$eval(attrs.uiCalendar) : {}, fullCalendarConfig;
          fullCalendarConfig = controller.getFullCalendarConfig(calendarSettings, uiCalendarConfig);
          options = { eventSources: sources };
          angular.extend(options, fullCalendarConfig);
          var options2 = {};
          for (var o in options) {
            if (o !== 'eventSources') {
              options2[o] = options[o];
            }
          }
          return JSON.stringify(options2);
        }
        scope.destroy = function () {
          if (attrs.calendar) {
            scope.calendar = scope.$parent[attrs.calendar] = elm.html('');
          } else {
            scope.calendar = elm.html('');
          }
        };
        scope.init = function () {
          scope.calendar.fullCalendar(options);
        };
        eventSourcesWatcher.onAdded = function (source) {
          scope.calendar.fullCalendar('addEventSource', source);
          sourcesChanged = true;
        };
        eventSourcesWatcher.onRemoved = function (source) {
          scope.calendar.fullCalendar('removeEventSource', source);
          sourcesChanged = true;
        };
        eventsWatcher.onAdded = function (event) {
          scope.calendar.fullCalendar('renderEvent', event);
        };
        eventsWatcher.onRemoved = function (event) {
          scope.calendar.fullCalendar('removeEvents', function (e) {
            return e === event;
          });
        };
        eventsWatcher.onChanged = function (event) {
          scope.calendar.fullCalendar('updateEvent', event);
        };
        eventSourcesWatcher.subscribe(scope);
        eventsWatcher.subscribe(scope, function (newTokens, oldTokens) {
          if (sourcesChanged === true) {
            sourcesChanged = false;
            // prevent incremental updates in this case
            return false;
          }
        });
        scope.$watch(getOptions, function (newO, oldO) {
          scope.destroy();
          scope.init();
        });
      }
    };
  }
]);
/*!
 * FullCalendar v1.6.7
 * Docs & License: http://arshaw.com/fullcalendar/
 * (c) 2013 Adam Shaw
 */
/*
 * Use fullcalendar.css for basic styling.
 * For event drag & drop, requires jQuery UI draggable.
 * For event resizing, requires jQuery UI resizable.
 */
(function ($, undefined) {
  ;
  ;
  var defaults = {
      defaultView: 'month',
      aspectRatio: 1.35,
      header: {
        left: 'title',
        center: '',
        right: 'today prev,next'
      },
      weekends: true,
      weekNumbers: false,
      weekNumberCalculation: 'iso',
      weekNumberTitle: 'W',
      allDayDefault: true,
      ignoreTimezone: true,
      lazyFetching: true,
      startParam: 'start',
      endParam: 'end',
      titleFormat: {
        month: 'MMMM yyyy',
        week: 'MMM d[ yyyy]{ \'&#8212;\'[ MMM] d yyyy}',
        day: 'dddd, MMM d, yyyy'
      },
      columnFormat: {
        month: 'ddd',
        week: 'ddd M/d',
        day: 'dddd M/d'
      },
      timeFormat: { '': 'h(:mm)t' },
      isRTL: false,
      firstDay: 0,
      monthNames: [
        'January',
        'February',
        'March',
        'April',
        'May',
        'June',
        'July',
        'August',
        'September',
        'October',
        'November',
        'December'
      ],
      monthNamesShort: [
        'Jan',
        'Feb',
        'Mar',
        'Apr',
        'May',
        'Jun',
        'Jul',
        'Aug',
        'Sep',
        'Oct',
        'Nov',
        'Dec'
      ],
      dayNames: [
        'Sunday',
        'Monday',
        'Tuesday',
        'Wednesday',
        'Thursday',
        'Friday',
        'Saturday'
      ],
      dayNamesShort: [
        'Sun',
        'Mon',
        'Tue',
        'Wed',
        'Thu',
        'Fri',
        'Sat'
      ],
      buttonText: {
        prev: '<span class=\'fc-text-arrow\'>&lsaquo;</span>',
        next: '<span class=\'fc-text-arrow\'>&rsaquo;</span>',
        prevYear: '<span class=\'fc-text-arrow\'>&laquo;</span>',
        nextYear: '<span class=\'fc-text-arrow\'>&raquo;</span>',
        today: 'today',
        month: 'month',
        week: 'week',
        day: 'day'
      },
      theme: false,
      buttonIcons: {
        prev: 'circle-triangle-w',
        next: 'circle-triangle-e'
      },
      unselectAuto: true,
      dropAccept: '*',
      handleWindowResize: true
    };
  // right-to-left defaults
  var rtlDefaults = {
      header: {
        left: 'next,prev today',
        center: '',
        right: 'title'
      },
      buttonText: {
        prev: '<span class=\'fc-text-arrow\'>&rsaquo;</span>',
        next: '<span class=\'fc-text-arrow\'>&lsaquo;</span>',
        prevYear: '<span class=\'fc-text-arrow\'>&raquo;</span>',
        nextYear: '<span class=\'fc-text-arrow\'>&laquo;</span>'
      },
      buttonIcons: {
        prev: 'circle-triangle-e',
        next: 'circle-triangle-w'
      }
    };
  ;
  ;
  var fc = $.fullCalendar = { version: '1.6.7' };
  var fcViews = fc.views = {};
  $.fn.fullCalendar = function (options) {
    // method calling
    if (typeof options == 'string') {
      var args = Array.prototype.slice.call(arguments, 1);
      var res;
      this.each(function () {
        var calendar = $.data(this, 'fullCalendar');
        if (calendar && $.isFunction(calendar[options])) {
          var r = calendar[options].apply(calendar, args);
          if (res === undefined) {
            res = r;
          }
          if (options == 'destroy') {
            $.removeData(this, 'fullCalendar');
          }
        }
      });
      if (res !== undefined) {
        return res;
      }
      return this;
    }
    options = options || {};
    // would like to have this logic in EventManager, but needs to happen before options are recursively extended
    var eventSources = options.eventSources || [];
    delete options.eventSources;
    if (options.events) {
      eventSources.push(options.events);
      delete options.events;
    }
    options = $.extend(true, {}, defaults, options.isRTL || options.isRTL === undefined && defaults.isRTL ? rtlDefaults : {}, options);
    this.each(function (i, _element) {
      var element = $(_element);
      var calendar = new Calendar(element, options, eventSources);
      element.data('fullCalendar', calendar);
      // TODO: look into memory leak implications
      calendar.render();
    });
    return this;
  };
  // function for adding/overriding defaults
  function setDefaults(d) {
    $.extend(true, defaults, d);
  }
  ;
  ;
  function Calendar(element, options, eventSources) {
    var t = this;
    // exports
    t.options = options;
    t.render = render;
    t.destroy = destroy;
    t.refetchEvents = refetchEvents;
    t.reportEvents = reportEvents;
    t.reportEventChange = reportEventChange;
    t.rerenderEvents = rerenderEvents;
    t.changeView = changeView;
    t.select = select;
    t.unselect = unselect;
    t.prev = prev;
    t.next = next;
    t.prevYear = prevYear;
    t.nextYear = nextYear;
    t.today = today;
    t.gotoDate = gotoDate;
    t.incrementDate = incrementDate;
    t.formatDate = function (format, date) {
      return formatDate(format, date, options);
    };
    t.formatDates = function (format, date1, date2) {
      return formatDates(format, date1, date2, options);
    };
    t.getDate = getDate;
    t.getView = getView;
    t.option = option;
    t.trigger = trigger;
    // imports
    EventManager.call(t, options, eventSources);
    var isFetchNeeded = t.isFetchNeeded;
    var fetchEvents = t.fetchEvents;
    // locals
    var _element = element[0];
    var header;
    var headerElement;
    var content;
    var tm;
    // for making theme classes
    var currentView;
    var elementOuterWidth;
    var suggestedViewHeight;
    var resizeUID = 0;
    var ignoreWindowResize = 0;
    var date = new Date();
    var events = [];
    var _dragElement;
    /* Main Rendering
	-----------------------------------------------------------------------------*/
    setYMD(date, options.year, options.month, options.date);
    function render(inc) {
      if (!content) {
        initialRender();
      } else if (elementVisible()) {
        // mainly for the public API
        calcSize();
        _renderView(inc);
      }
    }
    function initialRender() {
      tm = options.theme ? 'ui' : 'fc';
      element.addClass('fc');
      if (options.isRTL) {
        element.addClass('fc-rtl');
      } else {
        element.addClass('fc-ltr');
      }
      if (options.theme) {
        element.addClass('ui-widget');
      }
      content = $('<div class=\'fc-content\' style=\'position:relative\'/>').prependTo(element);
      header = new Header(t, options);
      headerElement = header.render();
      if (headerElement) {
        element.prepend(headerElement);
      }
      changeView(options.defaultView);
      if (options.handleWindowResize) {
        $(window).resize(windowResize);
      }
      // needed for IE in a 0x0 iframe, b/c when it is resized, never triggers a windowResize
      if (!bodyVisible()) {
        lateRender();
      }
    }
    // called when we know the calendar couldn't be rendered when it was initialized,
    // but we think it's ready now
    function lateRender() {
      setTimeout(function () {
        // IE7 needs this so dimensions are calculated correctly
        if (!currentView.start && bodyVisible()) {
          // !currentView.start makes sure this never happens more than once
          renderView();
        }
      }, 0);
    }
    function destroy() {
      if (currentView) {
        trigger('viewDestroy', currentView, currentView, currentView.element);
        currentView.triggerEventDestroy();
      }
      $(window).unbind('resize', windowResize);
      header.destroy();
      content.remove();
      element.removeClass('fc fc-rtl ui-widget');
    }
    function elementVisible() {
      return element.is(':visible');
    }
    function bodyVisible() {
      return $('body').is(':visible');
    }
    /* View Rendering
	-----------------------------------------------------------------------------*/
    function changeView(newViewName) {
      if (!currentView || newViewName != currentView.name) {
        _changeView(newViewName);
      }
    }
    function _changeView(newViewName) {
      ignoreWindowResize++;
      if (currentView) {
        trigger('viewDestroy', currentView, currentView, currentView.element);
        unselect();
        currentView.triggerEventDestroy();
        // trigger 'eventDestroy' for each event
        freezeContentHeight();
        currentView.element.remove();
        header.deactivateButton(currentView.name);
      }
      header.activateButton(newViewName);
      currentView = new fcViews[newViewName]($('<div class=\'fc-view fc-view-' + newViewName + '\' style=\'position:relative\'/>').appendTo(content), t);
      renderView();
      unfreezeContentHeight();
      ignoreWindowResize--;
    }
    function renderView(inc) {
      if (!currentView.start || inc || date < currentView.start || date >= currentView.end) {
        if (elementVisible()) {
          _renderView(inc);
        }
      }
    }
    function _renderView(inc) {
      // assumes elementVisible
      ignoreWindowResize++;
      if (currentView.start) {
        // already been rendered?
        trigger('viewDestroy', currentView, currentView, currentView.element);
        unselect();
        clearEvents();
      }
      freezeContentHeight();
      currentView.render(date, inc || 0);
      // the view's render method ONLY renders the skeleton, nothing else
      setSize();
      unfreezeContentHeight();
      (currentView.afterRender || noop)();
      updateTitle();
      updateTodayButton();
      trigger('viewRender', currentView, currentView, currentView.element);
      currentView.trigger('viewDisplay', _element);
      // deprecated
      ignoreWindowResize--;
      getAndRenderEvents();
    }
    /* Resizing
	-----------------------------------------------------------------------------*/
    function updateSize() {
      if (elementVisible()) {
        unselect();
        clearEvents();
        calcSize();
        setSize();
        renderEvents();
      }
    }
    function calcSize() {
      // assumes elementVisible
      if (options.contentHeight) {
        suggestedViewHeight = options.contentHeight;
      } else if (options.height) {
        suggestedViewHeight = options.height - (headerElement ? headerElement.height() : 0) - vsides(content);
      } else {
        suggestedViewHeight = Math.round(content.width() / Math.max(options.aspectRatio, 0.5));
      }
    }
    function setSize() {
      // assumes elementVisible
      if (suggestedViewHeight === undefined) {
        calcSize();  // for first time
                     // NOTE: we don't want to recalculate on every renderView because
                     // it could result in oscillating heights due to scrollbars.
      }
      ignoreWindowResize++;
      currentView.setHeight(suggestedViewHeight);
      currentView.setWidth(content.width());
      ignoreWindowResize--;
      elementOuterWidth = element.outerWidth();
    }
    function windowResize() {
      if (!ignoreWindowResize) {
        if (currentView.start) {
          // view has already been rendered
          var uid = ++resizeUID;
          setTimeout(function () {
            // add a delay
            if (uid == resizeUID && !ignoreWindowResize && elementVisible()) {
              if (elementOuterWidth != (elementOuterWidth = element.outerWidth())) {
                ignoreWindowResize++;
                // in case the windowResize callback changes the height
                updateSize();
                currentView.trigger('windowResize', _element);
                ignoreWindowResize--;
              }
            }
          }, 200);
        } else {
          // calendar must have been initialized in a 0x0 iframe that has just been resized
          lateRender();
        }
      }
    }
    /* Event Fetching/Rendering
	-----------------------------------------------------------------------------*/
    // TODO: going forward, most of this stuff should be directly handled by the view
    function refetchEvents() {
      // can be called as an API method
      clearEvents();
      fetchAndRenderEvents();
    }
    function rerenderEvents(modifiedEventID) {
      // can be called as an API method
      clearEvents();
      renderEvents(modifiedEventID);
    }
    function renderEvents(modifiedEventID) {
      // TODO: remove modifiedEventID hack
      if (elementVisible()) {
        currentView.setEventData(events);
        // for View.js, TODO: unify with renderEvents
        currentView.renderEvents(events, modifiedEventID);
        // actually render the DOM elements
        currentView.trigger('eventAfterAllRender');
      }
    }
    function clearEvents() {
      currentView.triggerEventDestroy();
      // trigger 'eventDestroy' for each event
      currentView.clearEvents();
      // actually remove the DOM elements
      currentView.clearEventData();  // for View.js, TODO: unify with clearEvents
    }
    function getAndRenderEvents() {
      if (!options.lazyFetching || isFetchNeeded(currentView.visStart, currentView.visEnd)) {
        fetchAndRenderEvents();
      } else {
        renderEvents();
      }
    }
    function fetchAndRenderEvents() {
      fetchEvents(currentView.visStart, currentView.visEnd);  // ... will call reportEvents
                                                              // ... which will call renderEvents
    }
    // called when event data arrives
    function reportEvents(_events) {
      events = _events;
      renderEvents();
    }
    // called when a single event's data has been changed
    function reportEventChange(eventID) {
      rerenderEvents(eventID);
    }
    /* Header Updating
	-----------------------------------------------------------------------------*/
    function updateTitle() {
      header.updateTitle(currentView.title);
    }
    function updateTodayButton() {
      var today = new Date();
      if (today >= currentView.start && today < currentView.end) {
        header.disableButton('today');
      } else {
        header.enableButton('today');
      }
    }
    /* Selection
	-----------------------------------------------------------------------------*/
    function select(start, end, allDay) {
      currentView.select(start, end, allDay === undefined ? true : allDay);
    }
    function unselect() {
      // safe to be called before renderView
      if (currentView) {
        currentView.unselect();
      }
    }
    /* Date
	-----------------------------------------------------------------------------*/
    function prev() {
      renderView(-1);
    }
    function next() {
      renderView(1);
    }
    function prevYear() {
      addYears(date, -1);
      renderView();
    }
    function nextYear() {
      addYears(date, 1);
      renderView();
    }
    function today() {
      date = new Date();
      renderView();
    }
    function gotoDate(year, month, dateOfMonth) {
      if (year instanceof Date) {
        date = cloneDate(year);  // provided 1 argument, a Date
      } else {
        setYMD(date, year, month, dateOfMonth);
      }
      renderView();
    }
    function incrementDate(years, months, days) {
      if (years !== undefined) {
        addYears(date, years);
      }
      if (months !== undefined) {
        addMonths(date, months);
      }
      if (days !== undefined) {
        addDays(date, days);
      }
      renderView();
    }
    function getDate() {
      return cloneDate(date);
    }
    /* Height "Freezing"
	-----------------------------------------------------------------------------*/
    function freezeContentHeight() {
      content.css({
        width: '100%',
        height: content.height(),
        overflow: 'hidden'
      });
    }
    function unfreezeContentHeight() {
      content.css({
        width: '',
        height: '',
        overflow: ''
      });
    }
    /* Misc
	-----------------------------------------------------------------------------*/
    function getView() {
      return currentView;
    }
    function option(name, value) {
      if (value === undefined) {
        return options[name];
      }
      if (name == 'height' || name == 'contentHeight' || name == 'aspectRatio') {
        options[name] = value;
        updateSize();
      }
    }
    function trigger(name, thisObj) {
      if (options[name]) {
        return options[name].apply(thisObj || _element, Array.prototype.slice.call(arguments, 2));
      }
    }
    /* External Dragging
	------------------------------------------------------------------------*/
    if (options.droppable) {
      $(document).bind('dragstart', function (ev, ui) {
        var _e = ev.target;
        var e = $(_e);
        if (!e.parents('.fc').length) {
          // not already inside a calendar
          var accept = options.dropAccept;
          if ($.isFunction(accept) ? accept.call(_e, e) : e.is(accept)) {
            _dragElement = _e;
            currentView.dragStart(_dragElement, ev, ui);
          }
        }
      }).bind('dragstop', function (ev, ui) {
        if (_dragElement) {
          currentView.dragStop(_dragElement, ev, ui);
          _dragElement = null;
        }
      });
    }
  }
  ;
  ;
  function Header(calendar, options) {
    var t = this;
    // exports
    t.render = render;
    t.destroy = destroy;
    t.updateTitle = updateTitle;
    t.activateButton = activateButton;
    t.deactivateButton = deactivateButton;
    t.disableButton = disableButton;
    t.enableButton = enableButton;
    // locals
    var element = $([]);
    var tm;
    function render() {
      tm = options.theme ? 'ui' : 'fc';
      var sections = options.header;
      if (sections) {
        element = $('<table class=\'fc-header\' style=\'width:100%\'/>').append($('<tr/>').append(renderSection('left')).append(renderSection('center')).append(renderSection('right')));
        return element;
      }
    }
    function destroy() {
      element.remove();
    }
    function renderSection(position) {
      var e = $('<td class=\'fc-header-' + position + '\'/>');
      var buttonStr = options.header[position];
      if (buttonStr) {
        $.each(buttonStr.split(' '), function (i) {
          if (i > 0) {
            e.append('<span class=\'fc-header-space\'/>');
          }
          var prevButton;
          $.each(this.split(','), function (j, buttonName) {
            if (buttonName == 'title') {
              e.append('<span class=\'fc-header-title\'><h2>&nbsp;</h2></span>');
              if (prevButton) {
                prevButton.addClass(tm + '-corner-right');
              }
              prevButton = null;
            } else {
              var buttonClick;
              if (calendar[buttonName]) {
                buttonClick = calendar[buttonName];  // calendar method
              } else if (fcViews[buttonName]) {
                buttonClick = function () {
                  button.removeClass(tm + '-state-hover');
                  // forget why
                  calendar.changeView(buttonName);
                };
              }
              if (buttonClick) {
                var icon = options.theme ? smartProperty(options.buttonIcons, buttonName) : null;
                // why are we using smartProperty here?
                var text = smartProperty(options.buttonText, buttonName);
                // why are we using smartProperty here?
                var button = $('<span class=\'fc-button fc-button-' + buttonName + ' ' + tm + '-state-default\'>' + (icon ? '<span class=\'fc-icon-wrap\'>' + '<span class=\'ui-icon ui-icon-' + icon + '\'/>' + '</span>' : text) + '</span>').click(function () {
                    if (!button.hasClass(tm + '-state-disabled')) {
                      buttonClick();
                    }
                  }).mousedown(function () {
                    button.not('.' + tm + '-state-active').not('.' + tm + '-state-disabled').addClass(tm + '-state-down');
                  }).mouseup(function () {
                    button.removeClass(tm + '-state-down');
                  }).hover(function () {
                    button.not('.' + tm + '-state-active').not('.' + tm + '-state-disabled').addClass(tm + '-state-hover');
                  }, function () {
                    button.removeClass(tm + '-state-hover').removeClass(tm + '-state-down');
                  }).appendTo(e);
                disableTextSelection(button);
                if (!prevButton) {
                  button.addClass(tm + '-corner-left');
                }
                prevButton = button;
              }
            }
          });
          if (prevButton) {
            prevButton.addClass(tm + '-corner-right');
          }
        });
      }
      return e;
    }
    function updateTitle(html) {
      element.find('h2').html(html);
    }
    function activateButton(buttonName) {
      element.find('span.fc-button-' + buttonName).addClass(tm + '-state-active');
    }
    function deactivateButton(buttonName) {
      element.find('span.fc-button-' + buttonName).removeClass(tm + '-state-active');
    }
    function disableButton(buttonName) {
      element.find('span.fc-button-' + buttonName).addClass(tm + '-state-disabled');
    }
    function enableButton(buttonName) {
      element.find('span.fc-button-' + buttonName).removeClass(tm + '-state-disabled');
    }
  }
  ;
  ;
  fc.sourceNormalizers = [];
  fc.sourceFetchers = [];
  var ajaxDefaults = {
      dataType: 'json',
      cache: false
    };
  var eventGUID = 1;
  function EventManager(options, _sources) {
    var t = this;
    // exports
    t.isFetchNeeded = isFetchNeeded;
    t.fetchEvents = fetchEvents;
    t.addEventSource = addEventSource;
    t.removeEventSource = removeEventSource;
    t.updateEvent = updateEvent;
    t.renderEvent = renderEvent;
    t.removeEvents = removeEvents;
    t.clientEvents = clientEvents;
    t.normalizeEvent = normalizeEvent;
    // imports
    var trigger = t.trigger;
    var getView = t.getView;
    var reportEvents = t.reportEvents;
    // locals
    var stickySource = { events: [] };
    var sources = [stickySource];
    var rangeStart, rangeEnd;
    var currentFetchID = 0;
    var pendingSourceCnt = 0;
    var loadingLevel = 0;
    var cache = [];
    for (var i = 0; i < _sources.length; i++) {
      _addEventSource(_sources[i]);
    }
    /* Fetching
	-----------------------------------------------------------------------------*/
    function isFetchNeeded(start, end) {
      return !rangeStart || start < rangeStart || end > rangeEnd;
    }
    function fetchEvents(start, end) {
      rangeStart = start;
      rangeEnd = end;
      cache = [];
      var fetchID = ++currentFetchID;
      var len = sources.length;
      pendingSourceCnt = len;
      for (var i = 0; i < len; i++) {
        fetchEventSource(sources[i], fetchID);
      }
    }
    function fetchEventSource(source, fetchID) {
      _fetchEventSource(source, function (events) {
        if (fetchID == currentFetchID) {
          if (events) {
            if (options.eventDataTransform) {
              events = $.map(events, options.eventDataTransform);
            }
            if (source.eventDataTransform) {
              events = $.map(events, source.eventDataTransform);
            }
            // TODO: this technique is not ideal for static array event sources.
            //  For arrays, we'll want to process all events right in the beginning, then never again.
            for (var i = 0; i < events.length; i++) {
              events[i].source = source;
              normalizeEvent(events[i]);
            }
            cache = cache.concat(events);
          }
          pendingSourceCnt--;
          if (!pendingSourceCnt) {
            reportEvents(cache);
          }
        }
      });
    }
    function _fetchEventSource(source, callback) {
      var i;
      var fetchers = fc.sourceFetchers;
      var res;
      for (i = 0; i < fetchers.length; i++) {
        res = fetchers[i](source, rangeStart, rangeEnd, callback);
        if (res === true) {
          // the fetcher is in charge. made its own async request
          return;
        } else if (typeof res == 'object') {
          // the fetcher returned a new source. process it
          _fetchEventSource(res, callback);
          return;
        }
      }
      var events = source.events;
      if (events) {
        if ($.isFunction(events)) {
          pushLoading();
          events(cloneDate(rangeStart), cloneDate(rangeEnd), function (events) {
            callback(events);
            popLoading();
          });
        } else if ($.isArray(events)) {
          callback(events);
        } else {
          callback();
        }
      } else {
        var url = source.url;
        if (url) {
          var success = source.success;
          var error = source.error;
          var complete = source.complete;
          // retrieve any outbound GET/POST $.ajax data from the options
          var customData;
          if ($.isFunction(source.data)) {
            // supplied as a function that returns a key/value object
            customData = source.data();
          } else {
            // supplied as a straight key/value object
            customData = source.data;
          }
          // use a copy of the custom data so we can modify the parameters
          // and not affect the passed-in object.
          var data = $.extend({}, customData || {});
          var startParam = firstDefined(source.startParam, options.startParam);
          var endParam = firstDefined(source.endParam, options.endParam);
          if (startParam) {
            data[startParam] = Math.round(+rangeStart / 1000);
          }
          if (endParam) {
            data[endParam] = Math.round(+rangeEnd / 1000);
          }
          pushLoading();
          $.ajax($.extend({}, ajaxDefaults, source, {
            data: data,
            success: function (events) {
              events = events || [];
              var res = applyAll(success, this, arguments);
              if ($.isArray(res)) {
                events = res;
              }
              callback(events);
            },
            error: function () {
              applyAll(error, this, arguments);
              callback();
            },
            complete: function () {
              applyAll(complete, this, arguments);
              popLoading();
            }
          }));
        } else {
          callback();
        }
      }
    }
    /* Sources
	-----------------------------------------------------------------------------*/
    function addEventSource(source) {
      source = _addEventSource(source);
      if (source) {
        pendingSourceCnt++;
        fetchEventSource(source, currentFetchID);  // will eventually call reportEvents
      }
    }
    function _addEventSource(source) {
      if ($.isFunction(source) || $.isArray(source)) {
        source = { events: source };
      } else if (typeof source == 'string') {
        source = { url: source };
      }
      if (typeof source == 'object') {
        normalizeSource(source);
        sources.push(source);
        return source;
      }
    }
    function removeEventSource(source) {
      sources = $.grep(sources, function (src) {
        return !isSourcesEqual(src, source);
      });
      // remove all client events from that source
      cache = $.grep(cache, function (e) {
        return !isSourcesEqual(e.source, source);
      });
      reportEvents(cache);
    }
    /* Manipulation
	-----------------------------------------------------------------------------*/
    function updateEvent(event) {
      // update an existing event
      var i, len = cache.length, e, defaultEventEnd = getView().defaultEventEnd,
        // getView???
        startDelta = event.start - event._start, endDelta = event.end ? event.end - (event._end || defaultEventEnd(event)) : 0;
      // was null and event was just resized
      for (i = 0; i < len; i++) {
        e = cache[i];
        if (e._id == event._id && e != event) {
          e.start = new Date(+e.start + startDelta);
          if (event.end) {
            if (e.end) {
              e.end = new Date(+e.end + endDelta);
            } else {
              e.end = new Date(+defaultEventEnd(e) + endDelta);
            }
          } else {
            e.end = null;
          }
          e.title = event.title;
          e.url = event.url;
          e.allDay = event.allDay;
          e.className = event.className;
          e.editable = event.editable;
          e.color = event.color;
          e.backgroundColor = event.backgroundColor;
          e.borderColor = event.borderColor;
          e.textColor = event.textColor;
          normalizeEvent(e);
        }
      }
      normalizeEvent(event);
      reportEvents(cache);
    }
    function renderEvent(event, stick) {
      normalizeEvent(event);
      if (!event.source) {
        if (stick) {
          stickySource.events.push(event);
          event.source = stickySource;
        }
        cache.push(event);
      }
      reportEvents(cache);
    }
    function removeEvents(filter) {
      if (!filter) {
        // remove all
        cache = [];
        // clear all array sources
        for (var i = 0; i < sources.length; i++) {
          if ($.isArray(sources[i].events)) {
            sources[i].events = [];
          }
        }
      } else {
        if (!$.isFunction(filter)) {
          // an event ID
          var id = filter + '';
          filter = function (e) {
            return e._id == id;
          };
        }
        cache = $.grep(cache, filter, true);
        // remove events from array sources
        for (var i = 0; i < sources.length; i++) {
          if ($.isArray(sources[i].events)) {
            sources[i].events = $.grep(sources[i].events, filter, true);
          }
        }
      }
      reportEvents(cache);
    }
    function clientEvents(filter) {
      if ($.isFunction(filter)) {
        return $.grep(cache, filter);
      } else if (filter) {
        // an event ID
        filter += '';
        return $.grep(cache, function (e) {
          return e._id == filter;
        });
      }
      return cache;  // else, return all
    }
    /* Loading State
	-----------------------------------------------------------------------------*/
    function pushLoading() {
      if (!loadingLevel++) {
        trigger('loading', null, true, getView());
      }
    }
    function popLoading() {
      if (!--loadingLevel) {
        trigger('loading', null, false, getView());
      }
    }
    /* Event Normalization
	-----------------------------------------------------------------------------*/
    function normalizeEvent(event) {
      var source = event.source || {};
      var ignoreTimezone = firstDefined(source.ignoreTimezone, options.ignoreTimezone);
      event._id = event._id || (event.id === undefined ? '_fc' + eventGUID++ : event.id + '');
      if (event.date) {
        if (!event.start) {
          event.start = event.date;
        }
        delete event.date;
      }
      event._start = cloneDate(event.start = parseDate(event.start, ignoreTimezone));
      event.end = parseDate(event.end, ignoreTimezone);
      if (event.end && event.end <= event.start) {
        event.end = null;
      }
      event._end = event.end ? cloneDate(event.end) : null;
      if (event.allDay === undefined) {
        event.allDay = firstDefined(source.allDayDefault, options.allDayDefault);
      }
      if (event.className) {
        if (typeof event.className == 'string') {
          event.className = event.className.split(/\s+/);
        }
      } else {
        event.className = [];
      }  // TODO: if there is no start date, return false to indicate an invalid event
    }
    /* Utils
	------------------------------------------------------------------------------*/
    function normalizeSource(source) {
      if (source.className) {
        // TODO: repeat code, same code for event classNames
        if (typeof source.className == 'string') {
          source.className = source.className.split(/\s+/);
        }
      } else {
        source.className = [];
      }
      var normalizers = fc.sourceNormalizers;
      for (var i = 0; i < normalizers.length; i++) {
        normalizers[i](source);
      }
    }
    function isSourcesEqual(source1, source2) {
      return source1 && source2 && getSourcePrimitive(source1) == getSourcePrimitive(source2);
    }
    function getSourcePrimitive(source) {
      return (typeof source == 'object' ? source.googleCalendarId || source.events || source.url : '') || source;
    }
  }
  ;
  ;
  fc.addDays = addDays;
  fc.cloneDate = cloneDate;
  fc.parseDate = parseDate;
  fc.parseISO8601 = parseISO8601;
  fc.parseTime = parseTime;
  fc.formatDate = formatDate;
  fc.formatDates = formatDates;
  /* Date Math
-----------------------------------------------------------------------------*/
  var dayIDs = [
      'sun',
      'mon',
      'tue',
      'wed',
      'thu',
      'fri',
      'sat'
    ], DAY_MS = 86400000, HOUR_MS = 3600000, MINUTE_MS = 60000;
  function addYears(d, n, keepTime) {
    d.setFullYear(d.getFullYear() + n);
    if (!keepTime) {
      clearTime(d);
    }
    return d;
  }
  function addMonths(d, n, keepTime) {
    // prevents day overflow/underflow
    if (+d) {
      // prevent infinite looping on invalid dates
      var m = d.getMonth() + n, check = cloneDate(d);
      check.setDate(1);
      check.setMonth(m);
      d.setMonth(m);
      if (!keepTime) {
        clearTime(d);
      }
      while (d.getMonth() != check.getMonth()) {
        d.setDate(d.getDate() + (d < check ? 1 : -1));
      }
    }
    return d;
  }
  function addDays(d, n, keepTime) {
    // deals with daylight savings
    if (+d) {
      var dd = d.getDate() + n, check = cloneDate(d);
      check.setHours(9);
      // set to middle of day
      check.setDate(dd);
      d.setDate(dd);
      if (!keepTime) {
        clearTime(d);
      }
      fixDate(d, check);
    }
    return d;
  }
  function fixDate(d, check) {
    // force d to be on check's YMD, for daylight savings purposes
    if (+d) {
      // prevent infinite looping on invalid dates
      while (d.getDate() != check.getDate()) {
        d.setTime(+d + (d < check ? 1 : -1) * HOUR_MS);
      }
    }
  }
  function addMinutes(d, n) {
    d.setMinutes(d.getMinutes() + n);
    return d;
  }
  function clearTime(d) {
    d.setHours(0);
    d.setMinutes(0);
    d.setSeconds(0);
    d.setMilliseconds(0);
    return d;
  }
  function cloneDate(d, dontKeepTime) {
    if (dontKeepTime) {
      return clearTime(new Date(+d));
    }
    return new Date(+d);
  }
  function zeroDate() {
    // returns a Date with time 00:00:00 and dateOfMonth=1
    var i = 0, d;
    do {
      d = new Date(1970, i++, 1);
    } while (d.getHours());
    // != 0
    return d;
  }
  function dayDiff(d1, d2) {
    // d1 - d2
    return Math.round((cloneDate(d1, true) - cloneDate(d2, true)) / DAY_MS);
  }
  function setYMD(date, y, m, d) {
    if (y !== undefined && y != date.getFullYear()) {
      date.setDate(1);
      date.setMonth(0);
      date.setFullYear(y);
    }
    if (m !== undefined && m != date.getMonth()) {
      date.setDate(1);
      date.setMonth(m);
    }
    if (d !== undefined) {
      date.setDate(d);
    }
  }
  /* Date Parsing
-----------------------------------------------------------------------------*/
  function parseDate(s, ignoreTimezone) {
    // ignoreTimezone defaults to true
    if (typeof s == 'object') {
      // already a Date object
      return s;
    }
    if (typeof s == 'number') {
      // a UNIX timestamp
      return new Date(s * 1000);
    }
    if (typeof s == 'string') {
      if (s.match(/^\d+(\.\d+)?$/)) {
        // a UNIX timestamp
        return new Date(parseFloat(s) * 1000);
      }
      if (ignoreTimezone === undefined) {
        ignoreTimezone = true;
      }
      return parseISO8601(s, ignoreTimezone) || (s ? new Date(s) : null);
    }
    // TODO: never return invalid dates (like from new Date(<string>)), return null instead
    return null;
  }
  function parseISO8601(s, ignoreTimezone) {
    // ignoreTimezone defaults to false
    // derived from http://delete.me.uk/2005/03/iso8601.html
    // TODO: for a know glitch/feature, read tests/issue_206_parseDate_dst.html
    var m = s.match(/^([0-9]{4})(-([0-9]{2})(-([0-9]{2})([T ]([0-9]{2}):([0-9]{2})(:([0-9]{2})(\.([0-9]+))?)?(Z|(([-+])([0-9]{2})(:?([0-9]{2}))?))?)?)?)?$/);
    if (!m) {
      return null;
    }
    var date = new Date(m[1], 0, 1);
    if (ignoreTimezone || !m[13]) {
      var check = new Date(m[1], 0, 1, 9, 0);
      if (m[3]) {
        date.setMonth(m[3] - 1);
        check.setMonth(m[3] - 1);
      }
      if (m[5]) {
        date.setDate(m[5]);
        check.setDate(m[5]);
      }
      fixDate(date, check);
      if (m[7]) {
        date.setHours(m[7]);
      }
      if (m[8]) {
        date.setMinutes(m[8]);
      }
      if (m[10]) {
        date.setSeconds(m[10]);
      }
      if (m[12]) {
        date.setMilliseconds(Number('0.' + m[12]) * 1000);
      }
      fixDate(date, check);
    } else {
      date.setUTCFullYear(m[1], m[3] ? m[3] - 1 : 0, m[5] || 1);
      date.setUTCHours(m[7] || 0, m[8] || 0, m[10] || 0, m[12] ? Number('0.' + m[12]) * 1000 : 0);
      if (m[14]) {
        var offset = Number(m[16]) * 60 + (m[18] ? Number(m[18]) : 0);
        offset *= m[15] == '-' ? 1 : -1;
        date = new Date(+date + offset * 60 * 1000);
      }
    }
    return date;
  }
  function parseTime(s) {
    // returns minutes since start of day
    if (typeof s == 'number') {
      // an hour
      return s * 60;
    }
    if (typeof s == 'object') {
      // a Date object
      return s.getHours() * 60 + s.getMinutes();
    }
    var m = s.match(/(\d+)(?::(\d+))?\s*(\w+)?/);
    if (m) {
      var h = parseInt(m[1], 10);
      if (m[3]) {
        h %= 12;
        if (m[3].toLowerCase().charAt(0) == 'p') {
          h += 12;
        }
      }
      return h * 60 + (m[2] ? parseInt(m[2], 10) : 0);
    }
  }
  /* Date Formatting
-----------------------------------------------------------------------------*/
  // TODO: use same function formatDate(date, [date2], format, [options])
  function formatDate(date, format, options) {
    return formatDates(date, null, format, options);
  }
  function formatDates(date1, date2, format, options) {
    options = options || defaults;
    var date = date1, otherDate = date2, i, len = format.length, c, i2, formatter, res = '';
    for (i = 0; i < len; i++) {
      c = format.charAt(i);
      if (c == '\'') {
        for (i2 = i + 1; i2 < len; i2++) {
          if (format.charAt(i2) == '\'') {
            if (date) {
              if (i2 == i + 1) {
                res += '\'';
              } else {
                res += format.substring(i + 1, i2);
              }
              i = i2;
            }
            break;
          }
        }
      } else if (c == '(') {
        for (i2 = i + 1; i2 < len; i2++) {
          if (format.charAt(i2) == ')') {
            var subres = formatDate(date, format.substring(i + 1, i2), options);
            if (parseInt(subres.replace(/\D/, ''), 10)) {
              res += subres;
            }
            i = i2;
            break;
          }
        }
      } else if (c == '[') {
        for (i2 = i + 1; i2 < len; i2++) {
          if (format.charAt(i2) == ']') {
            var subformat = format.substring(i + 1, i2);
            var subres = formatDate(date, subformat, options);
            if (subres != formatDate(otherDate, subformat, options)) {
              res += subres;
            }
            i = i2;
            break;
          }
        }
      } else if (c == '{') {
        date = date2;
        otherDate = date1;
      } else if (c == '}') {
        date = date1;
        otherDate = date2;
      } else {
        for (i2 = len; i2 > i; i2--) {
          if (formatter = dateFormatters[format.substring(i, i2)]) {
            if (date) {
              res += formatter(date, options);
            }
            i = i2 - 1;
            break;
          }
        }
        if (i2 == i) {
          if (date) {
            res += c;
          }
        }
      }
    }
    return res;
  }
  ;
  var dateFormatters = {
      s: function (d) {
        return d.getSeconds();
      },
      ss: function (d) {
        return zeroPad(d.getSeconds());
      },
      m: function (d) {
        return d.getMinutes();
      },
      mm: function (d) {
        return zeroPad(d.getMinutes());
      },
      h: function (d) {
        return d.getHours() % 12 || 12;
      },
      hh: function (d) {
        return zeroPad(d.getHours() % 12 || 12);
      },
      H: function (d) {
        return d.getHours();
      },
      HH: function (d) {
        return zeroPad(d.getHours());
      },
      d: function (d) {
        return d.getDate();
      },
      dd: function (d) {
        return zeroPad(d.getDate());
      },
      ddd: function (d, o) {
        return o.dayNamesShort[d.getDay()];
      },
      dddd: function (d, o) {
        return o.dayNames[d.getDay()];
      },
      M: function (d) {
        return d.getMonth() + 1;
      },
      MM: function (d) {
        return zeroPad(d.getMonth() + 1);
      },
      MMM: function (d, o) {
        return o.monthNamesShort[d.getMonth()];
      },
      MMMM: function (d, o) {
        return o.monthNames[d.getMonth()];
      },
      yy: function (d) {
        return (d.getFullYear() + '').substring(2);
      },
      yyyy: function (d) {
        return d.getFullYear();
      },
      t: function (d) {
        return d.getHours() < 12 ? 'a' : 'p';
      },
      tt: function (d) {
        return d.getHours() < 12 ? 'am' : 'pm';
      },
      T: function (d) {
        return d.getHours() < 12 ? 'A' : 'P';
      },
      TT: function (d) {
        return d.getHours() < 12 ? 'AM' : 'PM';
      },
      u: function (d) {
        return formatDate(d, 'yyyy-MM-dd\'T\'HH:mm:ss\'Z\'');
      },
      S: function (d) {
        var date = d.getDate();
        if (date > 10 && date < 20) {
          return 'th';
        }
        return [
          'st',
          'nd',
          'rd'
        ][date % 10 - 1] || 'th';
      },
      w: function (d, o) {
        // local
        return o.weekNumberCalculation(d);
      },
      W: function (d) {
        // ISO
        return iso8601Week(d);
      }
    };
  fc.dateFormatters = dateFormatters;
  /* thanks jQuery UI (https://github.com/jquery/jquery-ui/blob/master/ui/jquery.ui.datepicker.js)
 * 
 * Set as calculateWeek to determine the week of the year based on the ISO 8601 definition.
 * `date` - the date to get the week for
 * `number` - the number of the week within the year that contains this date
 */
  function iso8601Week(date) {
    var time;
    var checkDate = new Date(date.getTime());
    // Find Thursday of this week starting on Monday
    checkDate.setDate(checkDate.getDate() + 4 - (checkDate.getDay() || 7));
    time = checkDate.getTime();
    checkDate.setMonth(0);
    // Compare with Jan 1
    checkDate.setDate(1);
    return Math.floor(Math.round((time - checkDate) / 86400000) / 7) + 1;
  }
  ;
  ;
  fc.applyAll = applyAll;
  /* Event Date Math
-----------------------------------------------------------------------------*/
  function exclEndDay(event) {
    if (event.end) {
      return _exclEndDay(event.end, event.allDay);
    } else {
      return addDays(cloneDate(event.start), 1);
    }
  }
  function _exclEndDay(end, allDay) {
    end = cloneDate(end);
    return allDay || end.getHours() || end.getMinutes() ? addDays(end, 1) : clearTime(end);  // why don't we check for seconds/ms too?
  }
  /* Event Element Binding
-----------------------------------------------------------------------------*/
  function lazySegBind(container, segs, bindHandlers) {
    container.unbind('mouseover').mouseover(function (ev) {
      var parent = ev.target, e, i, seg;
      while (parent != this) {
        e = parent;
        parent = parent.parentNode;
      }
      if ((i = e._fci) !== undefined) {
        e._fci = undefined;
        seg = segs[i];
        bindHandlers(seg.event, seg.element, seg);
        $(ev.target).trigger(ev);
      }
      ev.stopPropagation();
    });
  }
  /* Element Dimensions
-----------------------------------------------------------------------------*/
  function setOuterWidth(element, width, includeMargins) {
    for (var i = 0, e; i < element.length; i++) {
      e = $(element[i]);
      e.width(Math.max(0, width - hsides(e, includeMargins)));
    }
  }
  function setOuterHeight(element, height, includeMargins) {
    for (var i = 0, e; i < element.length; i++) {
      e = $(element[i]);
      e.height(Math.max(0, height - vsides(e, includeMargins)));
    }
  }
  function hsides(element, includeMargins) {
    return hpadding(element) + hborders(element) + (includeMargins ? hmargins(element) : 0);
  }
  function hpadding(element) {
    return (parseFloat($.css(element[0], 'paddingLeft', true)) || 0) + (parseFloat($.css(element[0], 'paddingRight', true)) || 0);
  }
  function hmargins(element) {
    return (parseFloat($.css(element[0], 'marginLeft', true)) || 0) + (parseFloat($.css(element[0], 'marginRight', true)) || 0);
  }
  function hborders(element) {
    return (parseFloat($.css(element[0], 'borderLeftWidth', true)) || 0) + (parseFloat($.css(element[0], 'borderRightWidth', true)) || 0);
  }
  function vsides(element, includeMargins) {
    return vpadding(element) + vborders(element) + (includeMargins ? vmargins(element) : 0);
  }
  function vpadding(element) {
    return (parseFloat($.css(element[0], 'paddingTop', true)) || 0) + (parseFloat($.css(element[0], 'paddingBottom', true)) || 0);
  }
  function vmargins(element) {
    return (parseFloat($.css(element[0], 'marginTop', true)) || 0) + (parseFloat($.css(element[0], 'marginBottom', true)) || 0);
  }
  function vborders(element) {
    return (parseFloat($.css(element[0], 'borderTopWidth', true)) || 0) + (parseFloat($.css(element[0], 'borderBottomWidth', true)) || 0);
  }
  /* Misc Utils
-----------------------------------------------------------------------------*/
  //TODO: arraySlice
  //TODO: isFunction, grep ?
  function noop() {
  }
  function dateCompare(a, b) {
    return a - b;
  }
  function arrayMax(a) {
    return Math.max.apply(Math, a);
  }
  function zeroPad(n) {
    return (n < 10 ? '0' : '') + n;
  }
  function smartProperty(obj, name) {
    // get a camel-cased/namespaced property of an object
    if (obj[name] !== undefined) {
      return obj[name];
    }
    var parts = name.split(/(?=[A-Z])/), i = parts.length - 1, res;
    for (; i >= 0; i--) {
      res = obj[parts[i].toLowerCase()];
      if (res !== undefined) {
        return res;
      }
    }
    return obj[''];
  }
  function htmlEscape(s) {
    return s.replace(/&/g, '&amp;').replace(/</g, '&lt;').replace(/>/g, '&gt;').replace(/'/g, '&#039;').replace(/"/g, '&quot;').replace(/\n/g, '<br />');
  }
  function disableTextSelection(element) {
    element.attr('unselectable', 'on').css('MozUserSelect', 'none').bind('selectstart.ui', function () {
      return false;
    });
  }
  /*
function enableTextSelection(element) {
	element
		.attr('unselectable', 'off')
		.css('MozUserSelect', '')
		.unbind('selectstart.ui');
}
*/
  function markFirstLast(e) {
    e.children().removeClass('fc-first fc-last').filter(':first-child').addClass('fc-first').end().filter(':last-child').addClass('fc-last');
  }
  function setDayID(cell, date) {
    cell.each(function (i, _cell) {
      _cell.className = _cell.className.replace(/^fc-\w*/, 'fc-' + dayIDs[date.getDay()]);  // TODO: make a way that doesn't rely on order of classes
    });
  }
  function getSkinCss(event, opt) {
    var source = event.source || {};
    var eventColor = event.color;
    var sourceColor = source.color;
    var optionColor = opt('eventColor');
    var backgroundColor = event.backgroundColor || eventColor || source.backgroundColor || sourceColor || opt('eventBackgroundColor') || optionColor;
    var borderColor = event.borderColor || eventColor || source.borderColor || sourceColor || opt('eventBorderColor') || optionColor;
    var textColor = event.textColor || source.textColor || opt('eventTextColor');
    var statements = [];
    if (backgroundColor) {
      statements.push('background-color:' + backgroundColor);
    }
    if (borderColor) {
      statements.push('border-color:' + borderColor);
    }
    if (textColor) {
      statements.push('color:' + textColor);
    }
    return statements.join(';');
  }
  function applyAll(functions, thisObj, args) {
    if ($.isFunction(functions)) {
      functions = [functions];
    }
    if (functions) {
      var i;
      var ret;
      for (i = 0; i < functions.length; i++) {
        ret = functions[i].apply(thisObj, args) || ret;
      }
      return ret;
    }
  }
  function firstDefined() {
    for (var i = 0; i < arguments.length; i++) {
      if (arguments[i] !== undefined) {
        return arguments[i];
      }
    }
  }
  ;
  ;
  fcViews.month = MonthView;
  function MonthView(element, calendar) {
    var t = this;
    // exports
    t.render = render;
    // imports
    BasicView.call(t, element, calendar, 'month');
    var opt = t.opt;
    var renderBasic = t.renderBasic;
    var skipHiddenDays = t.skipHiddenDays;
    var getCellsPerWeek = t.getCellsPerWeek;
    var formatDate = calendar.formatDate;
    function render(date, delta) {
      if (delta) {
        addMonths(date, delta);
        date.setDate(1);
      }
      var firstDay = opt('firstDay');
      var start = cloneDate(date, true);
      start.setDate(1);
      var end = addMonths(cloneDate(start), 1);
      var visStart = cloneDate(start);
      addDays(visStart, -((visStart.getDay() - firstDay + 7) % 7));
      skipHiddenDays(visStart);
      var visEnd = cloneDate(end);
      addDays(visEnd, (7 - visEnd.getDay() + firstDay) % 7);
      skipHiddenDays(visEnd, -1, true);
      var colCnt = getCellsPerWeek();
      var rowCnt = Math.round(dayDiff(visEnd, visStart) / 7);
      // should be no need for Math.round
      if (opt('weekMode') == 'fixed') {
        addDays(visEnd, (6 - rowCnt) * 7);
        // add weeks to make up for it
        rowCnt = 6;
      }
      t.title = formatDate(start, opt('titleFormat'));
      t.start = start;
      t.end = end;
      t.visStart = visStart;
      t.visEnd = visEnd;
      renderBasic(rowCnt, colCnt, true);
    }
  }
  ;
  ;
  fcViews.basicWeek = BasicWeekView;
  function BasicWeekView(element, calendar) {
    var t = this;
    // exports
    t.render = render;
    // imports
    BasicView.call(t, element, calendar, 'basicWeek');
    var opt = t.opt;
    var renderBasic = t.renderBasic;
    var skipHiddenDays = t.skipHiddenDays;
    var getCellsPerWeek = t.getCellsPerWeek;
    var formatDates = calendar.formatDates;
    function render(date, delta) {
      if (delta) {
        addDays(date, delta * 7);
      }
      var start = addDays(cloneDate(date), -((date.getDay() - opt('firstDay') + 7) % 7));
      var end = addDays(cloneDate(start), 7);
      var visStart = cloneDate(start);
      skipHiddenDays(visStart);
      var visEnd = cloneDate(end);
      skipHiddenDays(visEnd, -1, true);
      var colCnt = getCellsPerWeek();
      t.start = start;
      t.end = end;
      t.visStart = visStart;
      t.visEnd = visEnd;
      t.title = formatDates(visStart, addDays(cloneDate(visEnd), -1), opt('titleFormat'));
      renderBasic(1, colCnt, false);
    }
  }
  ;
  ;
  fcViews.basicDay = BasicDayView;
  function BasicDayView(element, calendar) {
    var t = this;
    // exports
    t.render = render;
    // imports
    BasicView.call(t, element, calendar, 'basicDay');
    var opt = t.opt;
    var renderBasic = t.renderBasic;
    var skipHiddenDays = t.skipHiddenDays;
    var formatDate = calendar.formatDate;
    function render(date, delta) {
      if (delta) {
        addDays(date, delta);
      }
      skipHiddenDays(date, delta < 0 ? -1 : 1);
      var start = cloneDate(date, true);
      var end = addDays(cloneDate(start), 1);
      t.title = formatDate(date, opt('titleFormat'));
      t.start = t.visStart = start;
      t.end = t.visEnd = end;
      renderBasic(1, 1, false);
    }
  }
  ;
  ;
  setDefaults({ weekMode: 'fixed' });
  function BasicView(element, calendar, viewName) {
    var t = this;
    // exports
    t.renderBasic = renderBasic;
    t.setHeight = setHeight;
    t.setWidth = setWidth;
    t.renderDayOverlay = renderDayOverlay;
    t.defaultSelectionEnd = defaultSelectionEnd;
    t.renderSelection = renderSelection;
    t.clearSelection = clearSelection;
    t.reportDayClick = reportDayClick;
    // for selection (kinda hacky)
    t.dragStart = dragStart;
    t.dragStop = dragStop;
    t.defaultEventEnd = defaultEventEnd;
    t.getHoverListener = function () {
      return hoverListener;
    };
    t.colLeft = colLeft;
    t.colRight = colRight;
    t.colContentLeft = colContentLeft;
    t.colContentRight = colContentRight;
    t.getIsCellAllDay = function () {
      return true;
    };
    t.allDayRow = allDayRow;
    t.getRowCnt = function () {
      return rowCnt;
    };
    t.getColCnt = function () {
      return colCnt;
    };
    t.getColWidth = function () {
      return colWidth;
    };
    t.getDaySegmentContainer = function () {
      return daySegmentContainer;
    };
    // imports
    View.call(t, element, calendar, viewName);
    OverlayManager.call(t);
    SelectionManager.call(t);
    BasicEventRenderer.call(t);
    var opt = t.opt;
    var trigger = t.trigger;
    var renderOverlay = t.renderOverlay;
    var clearOverlays = t.clearOverlays;
    var daySelectionMousedown = t.daySelectionMousedown;
    var cellToDate = t.cellToDate;
    var dateToCell = t.dateToCell;
    var rangeToSegments = t.rangeToSegments;
    var formatDate = calendar.formatDate;
    // locals
    var table;
    var head;
    var headCells;
    var body;
    var bodyRows;
    var bodyCells;
    var bodyFirstCells;
    var firstRowCellInners;
    var firstRowCellContentInners;
    var daySegmentContainer;
    var viewWidth;
    var viewHeight;
    var colWidth;
    var weekNumberWidth;
    var rowCnt, colCnt;
    var showNumbers;
    var coordinateGrid;
    var hoverListener;
    var colPositions;
    var colContentPositions;
    var tm;
    var colFormat;
    var showWeekNumbers;
    var weekNumberTitle;
    var weekNumberFormat;
    /* Rendering
	------------------------------------------------------------*/
    disableTextSelection(element.addClass('fc-grid'));
    function renderBasic(_rowCnt, _colCnt, _showNumbers) {
      rowCnt = _rowCnt;
      colCnt = _colCnt;
      showNumbers = _showNumbers;
      updateOptions();
      if (!body) {
        buildEventContainer();
      }
      buildTable();
    }
    function updateOptions() {
      tm = opt('theme') ? 'ui' : 'fc';
      colFormat = opt('columnFormat');
      // week # options. (TODO: bad, logic also in other views)
      showWeekNumbers = opt('weekNumbers');
      weekNumberTitle = opt('weekNumberTitle');
      if (opt('weekNumberCalculation') != 'iso') {
        weekNumberFormat = 'w';
      } else {
        weekNumberFormat = 'W';
      }
    }
    function buildEventContainer() {
      daySegmentContainer = $('<div class=\'fc-event-container\' style=\'position:absolute;z-index:8;top:0;left:0\'/>').appendTo(element);
    }
    function buildTable() {
      var html = buildTableHTML();
      if (table) {
        table.remove();
      }
      table = $(html).appendTo(element);
      head = table.find('thead');
      headCells = head.find('.fc-day-header');
      body = table.find('tbody');
      bodyRows = body.find('tr');
      bodyCells = body.find('.fc-day');
      bodyFirstCells = bodyRows.find('td:first-child');
      firstRowCellInners = bodyRows.eq(0).find('.fc-day > div');
      firstRowCellContentInners = bodyRows.eq(0).find('.fc-day-content > div');
      markFirstLast(head.add(head.find('tr')));
      // marks first+last tr/th's
      markFirstLast(bodyRows);
      // marks first+last td's
      bodyRows.eq(0).addClass('fc-first');
      bodyRows.filter(':last').addClass('fc-last');
      bodyCells.each(function (i, _cell) {
        var date = cellToDate(Math.floor(i / colCnt), i % colCnt);
        trigger('dayRender', t, date, $(_cell));
      });
      dayBind(bodyCells);
    }
    /* HTML Building
	-----------------------------------------------------------*/
    function buildTableHTML() {
      var html = '<table class=\'fc-border-separate\' style=\'width:100%\' cellspacing=\'0\'>' + buildHeadHTML() + buildBodyHTML() + '</table>';
      return html;
    }
    function buildHeadHTML() {
      var headerClass = tm + '-widget-header';
      var html = '';
      var col;
      var date;
      html += '<thead><tr>';
      if (showWeekNumbers) {
        html += '<th class=\'fc-week-number ' + headerClass + '\'>' + htmlEscape(weekNumberTitle) + '</th>';
      }
      for (col = 0; col < colCnt; col++) {
        date = cellToDate(0, col);
        html += '<th class=\'fc-day-header fc-' + dayIDs[date.getDay()] + ' ' + headerClass + '\'>' + htmlEscape(formatDate(date, colFormat)) + '</th>';
      }
      html += '</tr></thead>';
      return html;
    }
    function buildBodyHTML() {
      var contentClass = tm + '-widget-content';
      var html = '';
      var row;
      var col;
      var date;
      html += '<tbody>';
      for (row = 0; row < rowCnt; row++) {
        html += '<tr class=\'fc-week\'>';
        if (showWeekNumbers) {
          date = cellToDate(row, 0);
          html += '<td class=\'fc-week-number ' + contentClass + '\'>' + '<div>' + htmlEscape(formatDate(date, weekNumberFormat)) + '</div>' + '</td>';
        }
        for (col = 0; col < colCnt; col++) {
          date = cellToDate(row, col);
          html += buildCellHTML(date);
        }
        html += '</tr>';
      }
      html += '</tbody>';
      return html;
    }
    function buildCellHTML(date) {
      var contentClass = tm + '-widget-content';
      var month = t.start.getMonth();
      var today = clearTime(new Date());
      var html = '';
      var classNames = [
          'fc-day',
          'fc-' + dayIDs[date.getDay()],
          contentClass
        ];
      if (date.getMonth() != month) {
        classNames.push('fc-other-month');
      }
      if (+date == +today) {
        classNames.push('fc-today', tm + '-state-highlight');
      } else if (date < today) {
        classNames.push('fc-past');
      } else {
        classNames.push('fc-future');
      }
      html += '<td' + ' class=\'' + classNames.join(' ') + '\'' + ' data-date=\'' + formatDate(date, 'yyyy-MM-dd') + '\'' + '>' + '<div>';
      if (showNumbers) {
        html += '<div class=\'fc-day-number\'>' + date.getDate() + '</div>';
      }
      html += '<div class=\'fc-day-content\'>' + '<div style=\'position:relative\'>&nbsp;</div>' + '</div>' + '</div>' + '</td>';
      return html;
    }
    /* Dimensions
	-----------------------------------------------------------*/
    function setHeight(height) {
      viewHeight = height;
      var bodyHeight = viewHeight - head.height();
      var rowHeight;
      var rowHeightLast;
      var cell;
      if (opt('weekMode') == 'variable') {
        rowHeight = rowHeightLast = Math.floor(bodyHeight / (rowCnt == 1 ? 2 : 6));
      } else {
        rowHeight = Math.floor(bodyHeight / rowCnt);
        rowHeightLast = bodyHeight - rowHeight * (rowCnt - 1);
      }
      bodyFirstCells.each(function (i, _cell) {
        if (i < rowCnt) {
          cell = $(_cell);
          cell.find('> div').css('min-height', (i == rowCnt - 1 ? rowHeightLast : rowHeight) - vsides(cell));
        }
      });
    }
    function setWidth(width) {
      viewWidth = width;
      colPositions.clear();
      colContentPositions.clear();
      weekNumberWidth = 0;
      if (showWeekNumbers) {
        weekNumberWidth = head.find('th.fc-week-number').outerWidth();
      }
      colWidth = Math.floor((viewWidth - weekNumberWidth) / colCnt);
      setOuterWidth(headCells.slice(0, -1), colWidth);
    }
    /* Day clicking and binding
	-----------------------------------------------------------*/
    function dayBind(days) {
      days.click(dayClick).mousedown(daySelectionMousedown);
    }
    function dayClick(ev) {
      if (!opt('selectable')) {
        // if selectable, SelectionManager will worry about dayClick
        var date = parseISO8601($(this).data('date'));
        trigger('dayClick', this, date, true, ev);
      }
    }
    /* Semi-transparent Overlay Helpers
	------------------------------------------------------*/
    // TODO: should be consolidated with AgendaView's methods
    function renderDayOverlay(overlayStart, overlayEnd, refreshCoordinateGrid) {
      // overlayEnd is exclusive
      if (refreshCoordinateGrid) {
        coordinateGrid.build();
      }
      var segments = rangeToSegments(overlayStart, overlayEnd);
      for (var i = 0; i < segments.length; i++) {
        var segment = segments[i];
        dayBind(renderCellOverlay(segment.row, segment.leftCol, segment.row, segment.rightCol));
      }
    }
    function renderCellOverlay(row0, col0, row1, col1) {
      // row1,col1 is inclusive
      var rect = coordinateGrid.rect(row0, col0, row1, col1, element);
      return renderOverlay(rect, element);
    }
    /* Selection
	-----------------------------------------------------------------------*/
    function defaultSelectionEnd(startDate, allDay) {
      return cloneDate(startDate);
    }
    function renderSelection(startDate, endDate, allDay) {
      renderDayOverlay(startDate, addDays(cloneDate(endDate), 1), true);  // rebuild every time???
    }
    function clearSelection() {
      clearOverlays();
    }
    function reportDayClick(date, allDay, ev) {
      var cell = dateToCell(date);
      var _element = bodyCells[cell.row * colCnt + cell.col];
      trigger('dayClick', _element, date, allDay, ev);
    }
    /* External Dragging
	-----------------------------------------------------------------------*/
    function dragStart(_dragElement, ev, ui) {
      hoverListener.start(function (cell) {
        clearOverlays();
        if (cell) {
          renderCellOverlay(cell.row, cell.col, cell.row, cell.col);
        }
      }, ev);
    }
    function dragStop(_dragElement, ev, ui) {
      var cell = hoverListener.stop();
      clearOverlays();
      if (cell) {
        var d = cellToDate(cell);
        trigger('drop', _dragElement, d, true, ev, ui);
      }
    }
    /* Utilities
	--------------------------------------------------------*/
    function defaultEventEnd(event) {
      return cloneDate(event.start);
    }
    coordinateGrid = new CoordinateGrid(function (rows, cols) {
      var e, n, p;
      headCells.each(function (i, _e) {
        e = $(_e);
        n = e.offset().left;
        if (i) {
          p[1] = n;
        }
        p = [n];
        cols[i] = p;
      });
      p[1] = n + e.outerWidth();
      bodyRows.each(function (i, _e) {
        if (i < rowCnt) {
          e = $(_e);
          n = e.offset().top;
          if (i) {
            p[1] = n;
          }
          p = [n];
          rows[i] = p;
        }
      });
      p[1] = n + e.outerHeight();
    });
    hoverListener = new HoverListener(coordinateGrid);
    colPositions = new HorizontalPositionCache(function (col) {
      return firstRowCellInners.eq(col);
    });
    colContentPositions = new HorizontalPositionCache(function (col) {
      return firstRowCellContentInners.eq(col);
    });
    function colLeft(col) {
      return colPositions.left(col);
    }
    function colRight(col) {
      return colPositions.right(col);
    }
    function colContentLeft(col) {
      return colContentPositions.left(col);
    }
    function colContentRight(col) {
      return colContentPositions.right(col);
    }
    function allDayRow(i) {
      return bodyRows.eq(i);
    }
  }
  ;
  ;
  function BasicEventRenderer() {
    var t = this;
    // exports
    t.renderEvents = renderEvents;
    t.clearEvents = clearEvents;
    // imports
    DayEventRenderer.call(t);
    function renderEvents(events, modifiedEventId) {
      t.renderDayEvents(events, modifiedEventId);
    }
    function clearEvents() {
      t.getDaySegmentContainer().empty();
    }  // TODO: have this class (and AgendaEventRenderer) be responsible for creating the event container div
  }
  ;
  ;
  fcViews.agendaWeek = AgendaWeekView;
  function AgendaWeekView(element, calendar) {
    var t = this;
    // exports
    t.render = render;
    // imports
    AgendaView.call(t, element, calendar, 'agendaWeek');
    var opt = t.opt;
    var renderAgenda = t.renderAgenda;
    var skipHiddenDays = t.skipHiddenDays;
    var getCellsPerWeek = t.getCellsPerWeek;
    var formatDates = calendar.formatDates;
    function render(date, delta) {
      if (delta) {
        addDays(date, delta * 7);
      }
      var start = addDays(cloneDate(date), -((date.getDay() - opt('firstDay') + 7) % 7));
      var end = addDays(cloneDate(start), 7);
      var visStart = cloneDate(start);
      skipHiddenDays(visStart);
      var visEnd = cloneDate(end);
      skipHiddenDays(visEnd, -1, true);
      var colCnt = getCellsPerWeek();
      t.title = formatDates(visStart, addDays(cloneDate(visEnd), -1), opt('titleFormat'));
      t.start = start;
      t.end = end;
      t.visStart = visStart;
      t.visEnd = visEnd;
      renderAgenda(colCnt);
    }
  }
  ;
  ;
  fcViews.agendaDay = AgendaDayView;
  function AgendaDayView(element, calendar) {
    var t = this;
    // exports
    t.render = render;
    // imports
    AgendaView.call(t, element, calendar, 'agendaDay');
    var opt = t.opt;
    var renderAgenda = t.renderAgenda;
    var skipHiddenDays = t.skipHiddenDays;
    var formatDate = calendar.formatDate;
    function render(date, delta) {
      if (delta) {
        addDays(date, delta);
      }
      skipHiddenDays(date, delta < 0 ? -1 : 1);
      var start = cloneDate(date, true);
      var end = addDays(cloneDate(start), 1);
      t.title = formatDate(date, opt('titleFormat'));
      t.start = t.visStart = start;
      t.end = t.visEnd = end;
      renderAgenda(1);
    }
  }
  ;
  ;
  setDefaults({
    allDaySlot: true,
    allDayText: 'all-day',
    firstHour: 6,
    slotMinutes: 30,
    defaultEventMinutes: 120,
    axisFormat: 'h(:mm)tt',
    timeFormat: { agenda: 'h:mm{ - h:mm}' },
    dragOpacity: { agenda: 0.5 },
    minTime: 0,
    maxTime: 24,
    slotEventOverlap: true
  });
  // TODO: make it work in quirks mode (event corners, all-day height)
  // TODO: test liquid width, especially in IE6
  function AgendaView(element, calendar, viewName) {
    var t = this;
    // exports
    t.renderAgenda = renderAgenda;
    t.setWidth = setWidth;
    t.setHeight = setHeight;
    t.afterRender = afterRender;
    t.defaultEventEnd = defaultEventEnd;
    t.timePosition = timePosition;
    t.getIsCellAllDay = getIsCellAllDay;
    t.allDayRow = getAllDayRow;
    t.getCoordinateGrid = function () {
      return coordinateGrid;
    };
    // specifically for AgendaEventRenderer
    t.getHoverListener = function () {
      return hoverListener;
    };
    t.colLeft = colLeft;
    t.colRight = colRight;
    t.colContentLeft = colContentLeft;
    t.colContentRight = colContentRight;
    t.getDaySegmentContainer = function () {
      return daySegmentContainer;
    };
    t.getSlotSegmentContainer = function () {
      return slotSegmentContainer;
    };
    t.getMinMinute = function () {
      return minMinute;
    };
    t.getMaxMinute = function () {
      return maxMinute;
    };
    t.getSlotContainer = function () {
      return slotContainer;
    };
    t.getRowCnt = function () {
      return 1;
    };
    t.getColCnt = function () {
      return colCnt;
    };
    t.getColWidth = function () {
      return colWidth;
    };
    t.getSnapHeight = function () {
      return snapHeight;
    };
    t.getSnapMinutes = function () {
      return snapMinutes;
    };
    t.defaultSelectionEnd = defaultSelectionEnd;
    t.renderDayOverlay = renderDayOverlay;
    t.renderSelection = renderSelection;
    t.clearSelection = clearSelection;
    t.reportDayClick = reportDayClick;
    // selection mousedown hack
    t.dragStart = dragStart;
    t.dragStop = dragStop;
    // imports
    View.call(t, element, calendar, viewName);
    OverlayManager.call(t);
    SelectionManager.call(t);
    AgendaEventRenderer.call(t);
    var opt = t.opt;
    var trigger = t.trigger;
    var renderOverlay = t.renderOverlay;
    var clearOverlays = t.clearOverlays;
    var reportSelection = t.reportSelection;
    var unselect = t.unselect;
    var daySelectionMousedown = t.daySelectionMousedown;
    var slotSegHtml = t.slotSegHtml;
    var cellToDate = t.cellToDate;
    var dateToCell = t.dateToCell;
    var rangeToSegments = t.rangeToSegments;
    var formatDate = calendar.formatDate;
    // locals
    var dayTable;
    var dayHead;
    var dayHeadCells;
    var dayBody;
    var dayBodyCells;
    var dayBodyCellInners;
    var dayBodyCellContentInners;
    var dayBodyFirstCell;
    var dayBodyFirstCellStretcher;
    var slotLayer;
    var daySegmentContainer;
    var allDayTable;
    var allDayRow;
    var slotScroller;
    var slotContainer;
    var slotSegmentContainer;
    var slotTable;
    var selectionHelper;
    var viewWidth;
    var viewHeight;
    var axisWidth;
    var colWidth;
    var gutterWidth;
    var slotHeight;
    // TODO: what if slotHeight changes? (see issue 650)
    var snapMinutes;
    var snapRatio;
    // ratio of number of "selection" slots to normal slots. (ex: 1, 2, 4)
    var snapHeight;
    // holds the pixel hight of a "selection" slot
    var colCnt;
    var slotCnt;
    var coordinateGrid;
    var hoverListener;
    var colPositions;
    var colContentPositions;
    var slotTopCache = {};
    var tm;
    var rtl;
    var minMinute, maxMinute;
    var colFormat;
    var showWeekNumbers;
    var weekNumberTitle;
    var weekNumberFormat;
    /* Rendering
	-----------------------------------------------------------------------------*/
    disableTextSelection(element.addClass('fc-agenda'));
    function renderAgenda(c) {
      colCnt = c;
      updateOptions();
      if (!dayTable) {
        // first time rendering?
        buildSkeleton();  // builds day table, slot area, events containers
      } else {
        buildDayTable();  // rebuilds day table
      }
    }
    function updateOptions() {
      tm = opt('theme') ? 'ui' : 'fc';
      rtl = opt('isRTL');
      minMinute = parseTime(opt('minTime'));
      maxMinute = parseTime(opt('maxTime'));
      colFormat = opt('columnFormat');
      // week # options. (TODO: bad, logic also in other views)
      showWeekNumbers = opt('weekNumbers');
      weekNumberTitle = opt('weekNumberTitle');
      if (opt('weekNumberCalculation') != 'iso') {
        weekNumberFormat = 'w';
      } else {
        weekNumberFormat = 'W';
      }
      snapMinutes = opt('snapMinutes') || opt('slotMinutes');
    }
    /* Build DOM
	-----------------------------------------------------------------------*/
    function buildSkeleton() {
      var headerClass = tm + '-widget-header';
      var contentClass = tm + '-widget-content';
      var s;
      var d;
      var i;
      var maxd;
      var minutes;
      var slotNormal = opt('slotMinutes') % 15 == 0;
      buildDayTable();
      slotLayer = $('<div style=\'position:absolute;z-index:2;left:0;width:100%\'/>').appendTo(element);
      if (opt('allDaySlot')) {
        daySegmentContainer = $('<div class=\'fc-event-container\' style=\'position:absolute;z-index:8;top:0;left:0\'/>').appendTo(slotLayer);
        s = '<table style=\'width:100%\' class=\'fc-agenda-allday\' cellspacing=\'0\'>' + '<tr>' + '<th class=\'' + headerClass + ' fc-agenda-axis\'>' + opt('allDayText') + '</th>' + '<td>' + '<div class=\'fc-day-content\'><div style=\'position:relative\'/></div>' + '</td>' + '<th class=\'' + headerClass + ' fc-agenda-gutter\'>&nbsp;</th>' + '</tr>' + '</table>';
        allDayTable = $(s).appendTo(slotLayer);
        allDayRow = allDayTable.find('tr');
        dayBind(allDayRow.find('td'));
        slotLayer.append('<div class=\'fc-agenda-divider ' + headerClass + '\'>' + '<div class=\'fc-agenda-divider-inner\'/>' + '</div>');
      } else {
        daySegmentContainer = $([]);  // in jQuery 1.4, we can just do $()
      }
      slotScroller = $('<div style=\'position:absolute;width:100%;overflow-x:hidden;overflow-y:auto\'/>').appendTo(slotLayer);
      slotContainer = $('<div style=\'position:relative;width:100%;overflow:hidden\'/>').appendTo(slotScroller);
      slotSegmentContainer = $('<div class=\'fc-event-container\' style=\'position:absolute;z-index:8;top:0;left:0\'/>').appendTo(slotContainer);
      s = '<table class=\'fc-agenda-slots\' style=\'width:100%\' cellspacing=\'0\'>' + '<tbody>';
      d = zeroDate();
      maxd = addMinutes(cloneDate(d), maxMinute);
      addMinutes(d, minMinute);
      slotCnt = 0;
      for (i = 0; d < maxd; i++) {
        minutes = d.getMinutes();
        s += '<tr class=\'fc-slot' + i + ' ' + (!minutes ? '' : 'fc-minor') + '\'>' + '<th class=\'fc-agenda-axis ' + headerClass + '\'>' + (!slotNormal || !minutes ? formatDate(d, opt('axisFormat')) : '&nbsp;') + '</th>' + '<td class=\'' + contentClass + '\'>' + '<div style=\'position:relative\'>&nbsp;</div>' + '</td>' + '</tr>';
        addMinutes(d, opt('slotMinutes'));
        slotCnt++;
      }
      s += '</tbody>' + '</table>';
      slotTable = $(s).appendTo(slotContainer);
      slotBind(slotTable.find('td'));
    }
    /* Build Day Table
	-----------------------------------------------------------------------*/
    function buildDayTable() {
      var html = buildDayTableHTML();
      if (dayTable) {
        dayTable.remove();
      }
      dayTable = $(html).appendTo(element);
      dayHead = dayTable.find('thead');
      dayHeadCells = dayHead.find('th').slice(1, -1);
      // exclude gutter
      dayBody = dayTable.find('tbody');
      dayBodyCells = dayBody.find('td').slice(0, -1);
      // exclude gutter
      dayBodyCellInners = dayBodyCells.find('> div');
      dayBodyCellContentInners = dayBodyCells.find('.fc-day-content > div');
      dayBodyFirstCell = dayBodyCells.eq(0);
      dayBodyFirstCellStretcher = dayBodyCellInners.eq(0);
      markFirstLast(dayHead.add(dayHead.find('tr')));
      markFirstLast(dayBody.add(dayBody.find('tr')));  // TODO: now that we rebuild the cells every time, we should call dayRender
    }
    function buildDayTableHTML() {
      var html = '<table style=\'width:100%\' class=\'fc-agenda-days fc-border-separate\' cellspacing=\'0\'>' + buildDayTableHeadHTML() + buildDayTableBodyHTML() + '</table>';
      return html;
    }
    function buildDayTableHeadHTML() {
      var headerClass = tm + '-widget-header';
      var date;
      var html = '';
      var weekText;
      var col;
      html += '<thead>' + '<tr>';
      if (showWeekNumbers) {
        date = cellToDate(0, 0);
        weekText = formatDate(date, weekNumberFormat);
        if (rtl) {
          weekText += weekNumberTitle;
        } else {
          weekText = weekNumberTitle + weekText;
        }
        html += '<th class=\'fc-agenda-axis fc-week-number ' + headerClass + '\'>' + htmlEscape(weekText) + '</th>';
      } else {
        html += '<th class=\'fc-agenda-axis ' + headerClass + '\'>&nbsp;</th>';
      }
      for (col = 0; col < colCnt; col++) {
        date = cellToDate(0, col);
        html += '<th class=\'fc-' + dayIDs[date.getDay()] + ' fc-col' + col + ' ' + headerClass + '\'>' + htmlEscape(formatDate(date, colFormat)) + '</th>';
      }
      html += '<th class=\'fc-agenda-gutter ' + headerClass + '\'>&nbsp;</th>' + '</tr>' + '</thead>';
      return html;
    }
    function buildDayTableBodyHTML() {
      var headerClass = tm + '-widget-header';
      // TODO: make these when updateOptions() called
      var contentClass = tm + '-widget-content';
      var date;
      var today = clearTime(new Date());
      var col;
      var cellsHTML;
      var cellHTML;
      var classNames;
      var html = '';
      html += '<tbody>' + '<tr>' + '<th class=\'fc-agenda-axis ' + headerClass + '\'>&nbsp;</th>';
      cellsHTML = '';
      for (col = 0; col < colCnt; col++) {
        date = cellToDate(0, col);
        classNames = [
          'fc-col' + col,
          'fc-' + dayIDs[date.getDay()],
          contentClass
        ];
        if (+date == +today) {
          classNames.push(tm + '-state-highlight', 'fc-today');
        } else if (date < today) {
          classNames.push('fc-past');
        } else {
          classNames.push('fc-future');
        }
        cellHTML = '<td class=\'' + classNames.join(' ') + '\'>' + '<div>' + '<div class=\'fc-day-content\'>' + '<div style=\'position:relative\'>&nbsp;</div>' + '</div>' + '</div>' + '</td>';
        cellsHTML += cellHTML;
      }
      html += cellsHTML;
      html += '<td class=\'fc-agenda-gutter ' + contentClass + '\'>&nbsp;</td>' + '</tr>' + '</tbody>';
      return html;
    }
    // TODO: data-date on the cells
    /* Dimensions
	-----------------------------------------------------------------------*/
    function setHeight(height) {
      if (height === undefined) {
        height = viewHeight;
      }
      viewHeight = height;
      slotTopCache = {};
      var headHeight = dayBody.position().top;
      var allDayHeight = slotScroller.position().top;
      // including divider
      var bodyHeight = Math.min(height - headHeight, slotTable.height() + allDayHeight + 1);
      dayBodyFirstCellStretcher.height(bodyHeight - vsides(dayBodyFirstCell));
      slotLayer.css('top', headHeight);
      slotScroller.height(bodyHeight - allDayHeight - 1);
      // the stylesheet guarantees that the first row has no border.
      // this allows .height() to work well cross-browser.
      slotHeight = slotTable.find('tr:first').height() + 1;
      // +1 for bottom border
      snapRatio = opt('slotMinutes') / snapMinutes;
      snapHeight = slotHeight / snapRatio;
    }
    function setWidth(width) {
      viewWidth = width;
      colPositions.clear();
      colContentPositions.clear();
      var axisFirstCells = dayHead.find('th:first');
      if (allDayTable) {
        axisFirstCells = axisFirstCells.add(allDayTable.find('th:first'));
      }
      axisFirstCells = axisFirstCells.add(slotTable.find('th:first'));
      axisWidth = 0;
      setOuterWidth(axisFirstCells.width('').each(function (i, _cell) {
        axisWidth = Math.max(axisWidth, $(_cell).outerWidth());
      }), axisWidth);
      var gutterCells = dayTable.find('.fc-agenda-gutter');
      if (allDayTable) {
        gutterCells = gutterCells.add(allDayTable.find('th.fc-agenda-gutter'));
      }
      var slotTableWidth = slotScroller[0].clientWidth;
      // needs to be done after axisWidth (for IE7)
      gutterWidth = slotScroller.width() - slotTableWidth;
      if (gutterWidth) {
        setOuterWidth(gutterCells, gutterWidth);
        gutterCells.show().prev().removeClass('fc-last');
      } else {
        gutterCells.hide().prev().addClass('fc-last');
      }
      colWidth = Math.floor((slotTableWidth - axisWidth) / colCnt);
      setOuterWidth(dayHeadCells.slice(0, -1), colWidth);
    }
    /* Scrolling
	-----------------------------------------------------------------------*/
    function resetScroll() {
      var d0 = zeroDate();
      var scrollDate = cloneDate(d0);
      scrollDate.setHours(opt('firstHour'));
      var top = timePosition(d0, scrollDate) + 1;
      // +1 for the border
      function scroll() {
        slotScroller.scrollTop(top);
      }
      scroll();
      setTimeout(scroll, 0);  // overrides any previous scroll state made by the browser
    }
    function afterRender() {
      // after the view has been freshly rendered and sized
      resetScroll();
    }
    /* Slot/Day clicking and binding
	-----------------------------------------------------------------------*/
    function dayBind(cells) {
      cells.click(slotClick).mousedown(daySelectionMousedown);
    }
    function slotBind(cells) {
      cells.click(slotClick).mousedown(slotSelectionMousedown);
    }
    function slotClick(ev) {
      if (!opt('selectable')) {
        // if selectable, SelectionManager will worry about dayClick
        var col = Math.min(colCnt - 1, Math.floor((ev.pageX - dayTable.offset().left - axisWidth) / colWidth));
        var date = cellToDate(0, col);
        var rowMatch = this.parentNode.className.match(/fc-slot(\d+)/);
        // TODO: maybe use data
        if (rowMatch) {
          var mins = parseInt(rowMatch[1]) * opt('slotMinutes');
          var hours = Math.floor(mins / 60);
          date.setHours(hours);
          date.setMinutes(mins % 60 + minMinute);
          trigger('dayClick', dayBodyCells[col], date, false, ev);
        } else {
          trigger('dayClick', dayBodyCells[col], date, true, ev);
        }
      }
    }
    /* Semi-transparent Overlay Helpers
	-----------------------------------------------------*/
    // TODO: should be consolidated with BasicView's methods
    function renderDayOverlay(overlayStart, overlayEnd, refreshCoordinateGrid) {
      // overlayEnd is exclusive
      if (refreshCoordinateGrid) {
        coordinateGrid.build();
      }
      var segments = rangeToSegments(overlayStart, overlayEnd);
      for (var i = 0; i < segments.length; i++) {
        var segment = segments[i];
        dayBind(renderCellOverlay(segment.row, segment.leftCol, segment.row, segment.rightCol));
      }
    }
    function renderCellOverlay(row0, col0, row1, col1) {
      // only for all-day?
      var rect = coordinateGrid.rect(row0, col0, row1, col1, slotLayer);
      return renderOverlay(rect, slotLayer);
    }
    function renderSlotOverlay(overlayStart, overlayEnd) {
      for (var i = 0; i < colCnt; i++) {
        var dayStart = cellToDate(0, i);
        var dayEnd = addDays(cloneDate(dayStart), 1);
        var stretchStart = new Date(Math.max(dayStart, overlayStart));
        var stretchEnd = new Date(Math.min(dayEnd, overlayEnd));
        if (stretchStart < stretchEnd) {
          var rect = coordinateGrid.rect(0, i, 0, i, slotContainer);
          // only use it for horizontal coords
          var top = timePosition(dayStart, stretchStart);
          var bottom = timePosition(dayStart, stretchEnd);
          rect.top = top;
          rect.height = bottom - top;
          slotBind(renderOverlay(rect, slotContainer));
        }
      }
    }
    /* Coordinate Utilities
	-----------------------------------------------------------------------------*/
    coordinateGrid = new CoordinateGrid(function (rows, cols) {
      var e, n, p;
      dayHeadCells.each(function (i, _e) {
        e = $(_e);
        n = e.offset().left;
        if (i) {
          p[1] = n;
        }
        p = [n];
        cols[i] = p;
      });
      p[1] = n + e.outerWidth();
      if (opt('allDaySlot')) {
        e = allDayRow;
        n = e.offset().top;
        rows[0] = [
          n,
          n + e.outerHeight()
        ];
      }
      var slotTableTop = slotContainer.offset().top;
      var slotScrollerTop = slotScroller.offset().top;
      var slotScrollerBottom = slotScrollerTop + slotScroller.outerHeight();
      function constrain(n) {
        return Math.max(slotScrollerTop, Math.min(slotScrollerBottom, n));
      }
      for (var i = 0; i < slotCnt * snapRatio; i++) {
        // adapt slot count to increased/decreased selection slot count
        rows.push([
          constrain(slotTableTop + snapHeight * i),
          constrain(slotTableTop + snapHeight * (i + 1))
        ]);
      }
    });
    hoverListener = new HoverListener(coordinateGrid);
    colPositions = new HorizontalPositionCache(function (col) {
      return dayBodyCellInners.eq(col);
    });
    colContentPositions = new HorizontalPositionCache(function (col) {
      return dayBodyCellContentInners.eq(col);
    });
    function colLeft(col) {
      return colPositions.left(col);
    }
    function colContentLeft(col) {
      return colContentPositions.left(col);
    }
    function colRight(col) {
      return colPositions.right(col);
    }
    function colContentRight(col) {
      return colContentPositions.right(col);
    }
    function getIsCellAllDay(cell) {
      return opt('allDaySlot') && !cell.row;
    }
    function realCellToDate(cell) {
      // ugh "real" ... but blame it on our abuse of the "cell" system
      var d = cellToDate(0, cell.col);
      var slotIndex = cell.row;
      if (opt('allDaySlot')) {
        slotIndex--;
      }
      if (slotIndex >= 0) {
        addMinutes(d, minMinute + slotIndex * snapMinutes);
      }
      return d;
    }
    // get the Y coordinate of the given time on the given day (both Date objects)
    function timePosition(day, time) {
      // both date objects. day holds 00:00 of current day
      day = cloneDate(day, true);
      if (time < addMinutes(cloneDate(day), minMinute)) {
        return 0;
      }
      if (time >= addMinutes(cloneDate(day), maxMinute)) {
        return slotTable.height();
      }
      var slotMinutes = opt('slotMinutes'), minutes = time.getHours() * 60 + time.getMinutes() - minMinute, slotI = Math.floor(minutes / slotMinutes), slotTop = slotTopCache[slotI];
      if (slotTop === undefined) {
        slotTop = slotTopCache[slotI] = slotTable.find('tr').eq(slotI).find('td div')[0].offsetTop;  // .eq() is faster than ":eq()" selector
                                                                                                     // [0].offsetTop is faster than .position().top (do we really need this optimization?)
                                                                                                     // a better optimization would be to cache all these divs
      }
      return Math.max(0, Math.round(slotTop - 1 + slotHeight * (minutes % slotMinutes / slotMinutes)));
    }
    function getAllDayRow(index) {
      return allDayRow;
    }
    function defaultEventEnd(event) {
      var start = cloneDate(event.start);
      if (event.allDay) {
        return start;
      }
      return addMinutes(start, opt('defaultEventMinutes'));
    }
    /* Selection
	---------------------------------------------------------------------------------*/
    function defaultSelectionEnd(startDate, allDay) {
      if (allDay) {
        return cloneDate(startDate);
      }
      return addMinutes(cloneDate(startDate), opt('slotMinutes'));
    }
    function renderSelection(startDate, endDate, allDay) {
      // only for all-day
      if (allDay) {
        if (opt('allDaySlot')) {
          renderDayOverlay(startDate, addDays(cloneDate(endDate), 1), true);
        }
      } else {
        renderSlotSelection(startDate, endDate);
      }
    }
    function renderSlotSelection(startDate, endDate) {
      var helperOption = opt('selectHelper');
      coordinateGrid.build();
      if (helperOption) {
        var col = dateToCell(startDate).col;
        if (col >= 0 && col < colCnt) {
          // only works when times are on same day
          var rect = coordinateGrid.rect(0, col, 0, col, slotContainer);
          // only for horizontal coords
          var top = timePosition(startDate, startDate);
          var bottom = timePosition(startDate, endDate);
          if (bottom > top) {
            // protect against selections that are entirely before or after visible range
            rect.top = top;
            rect.height = bottom - top;
            rect.left += 2;
            rect.width -= 5;
            if ($.isFunction(helperOption)) {
              var helperRes = helperOption(startDate, endDate);
              if (helperRes) {
                rect.position = 'absolute';
                selectionHelper = $(helperRes).css(rect).appendTo(slotContainer);
              }
            } else {
              rect.isStart = true;
              // conside rect a "seg" now
              rect.isEnd = true;
              //
              selectionHelper = $(slotSegHtml({
                title: '',
                start: startDate,
                end: endDate,
                className: ['fc-select-helper'],
                editable: false
              }, rect));
              selectionHelper.css('opacity', opt('dragOpacity'));
            }
            if (selectionHelper) {
              slotBind(selectionHelper);
              slotContainer.append(selectionHelper);
              setOuterWidth(selectionHelper, rect.width, true);
              // needs to be after appended
              setOuterHeight(selectionHelper, rect.height, true);
            }
          }
        }
      } else {
        renderSlotOverlay(startDate, endDate);
      }
    }
    function clearSelection() {
      clearOverlays();
      if (selectionHelper) {
        selectionHelper.remove();
        selectionHelper = null;
      }
    }
    function slotSelectionMousedown(ev) {
      if (ev.which == 1 && opt('selectable')) {
        // ev.which==1 means left mouse button
        unselect(ev);
        var dates;
        hoverListener.start(function (cell, origCell) {
          clearSelection();
          if (cell && cell.col == origCell.col && !getIsCellAllDay(cell)) {
            var d1 = realCellToDate(origCell);
            var d2 = realCellToDate(cell);
            dates = [
              d1,
              addMinutes(cloneDate(d1), snapMinutes),
              d2,
              addMinutes(cloneDate(d2), snapMinutes)
            ].sort(dateCompare);
            renderSlotSelection(dates[0], dates[3]);
          } else {
            dates = null;
          }
        }, ev);
        $(document).one('mouseup', function (ev) {
          hoverListener.stop();
          if (dates) {
            if (+dates[0] == +dates[1]) {
              reportDayClick(dates[0], false, ev);
            }
            reportSelection(dates[0], dates[3], false, ev);
          }
        });
      }
    }
    function reportDayClick(date, allDay, ev) {
      trigger('dayClick', dayBodyCells[dateToCell(date).col], date, allDay, ev);
    }
    /* External Dragging
	--------------------------------------------------------------------------------*/
    function dragStart(_dragElement, ev, ui) {
      hoverListener.start(function (cell) {
        clearOverlays();
        if (cell) {
          if (getIsCellAllDay(cell)) {
            renderCellOverlay(cell.row, cell.col, cell.row, cell.col);
          } else {
            var d1 = realCellToDate(cell);
            var d2 = addMinutes(cloneDate(d1), opt('defaultEventMinutes'));
            renderSlotOverlay(d1, d2);
          }
        }
      }, ev);
    }
    function dragStop(_dragElement, ev, ui) {
      var cell = hoverListener.stop();
      clearOverlays();
      if (cell) {
        trigger('drop', _dragElement, realCellToDate(cell), getIsCellAllDay(cell), ev, ui);
      }
    }
  }
  ;
  ;
  function AgendaEventRenderer() {
    var t = this;
    // exports
    t.renderEvents = renderEvents;
    t.clearEvents = clearEvents;
    t.slotSegHtml = slotSegHtml;
    // imports
    DayEventRenderer.call(t);
    var opt = t.opt;
    var trigger = t.trigger;
    var isEventDraggable = t.isEventDraggable;
    var isEventResizable = t.isEventResizable;
    var eventEnd = t.eventEnd;
    var eventElementHandlers = t.eventElementHandlers;
    var setHeight = t.setHeight;
    var getDaySegmentContainer = t.getDaySegmentContainer;
    var getSlotSegmentContainer = t.getSlotSegmentContainer;
    var getHoverListener = t.getHoverListener;
    var getMaxMinute = t.getMaxMinute;
    var getMinMinute = t.getMinMinute;
    var timePosition = t.timePosition;
    var getIsCellAllDay = t.getIsCellAllDay;
    var colContentLeft = t.colContentLeft;
    var colContentRight = t.colContentRight;
    var cellToDate = t.cellToDate;
    var getColCnt = t.getColCnt;
    var getColWidth = t.getColWidth;
    var getSnapHeight = t.getSnapHeight;
    var getSnapMinutes = t.getSnapMinutes;
    var getSlotContainer = t.getSlotContainer;
    var reportEventElement = t.reportEventElement;
    var showEvents = t.showEvents;
    var hideEvents = t.hideEvents;
    var eventDrop = t.eventDrop;
    var eventResize = t.eventResize;
    var renderDayOverlay = t.renderDayOverlay;
    var clearOverlays = t.clearOverlays;
    var renderDayEvents = t.renderDayEvents;
    var calendar = t.calendar;
    var formatDate = calendar.formatDate;
    var formatDates = calendar.formatDates;
    // overrides
    t.draggableDayEvent = draggableDayEvent;
    /* Rendering
	----------------------------------------------------------------------------*/
    function renderEvents(events, modifiedEventId) {
      var i, len = events.length, dayEvents = [], slotEvents = [];
      for (i = 0; i < len; i++) {
        if (events[i].allDay) {
          dayEvents.push(events[i]);
        } else {
          slotEvents.push(events[i]);
        }
      }
      if (opt('allDaySlot')) {
        renderDayEvents(dayEvents, modifiedEventId);
        setHeight();  // no params means set to viewHeight
      }
      renderSlotSegs(compileSlotSegs(slotEvents), modifiedEventId);
    }
    function clearEvents() {
      getDaySegmentContainer().empty();
      getSlotSegmentContainer().empty();
    }
    function compileSlotSegs(events) {
      var colCnt = getColCnt(), minMinute = getMinMinute(), maxMinute = getMaxMinute(), d, visEventEnds = $.map(events, slotEventEnd), i, j, seg, colSegs, segs = [];
      for (i = 0; i < colCnt; i++) {
        d = cellToDate(0, i);
        addMinutes(d, minMinute);
        colSegs = sliceSegs(events, visEventEnds, d, addMinutes(cloneDate(d), maxMinute - minMinute));
        colSegs = placeSlotSegs(colSegs);
        // returns a new order
        for (j = 0; j < colSegs.length; j++) {
          seg = colSegs[j];
          seg.col = i;
          segs.push(seg);
        }
      }
      return segs;
    }
    function sliceSegs(events, visEventEnds, start, end) {
      var segs = [], i, len = events.length, event, eventStart, eventEnd, segStart, segEnd, isStart, isEnd;
      for (i = 0; i < len; i++) {
        event = events[i];
        eventStart = event.start;
        eventEnd = visEventEnds[i];
        if (eventEnd > start && eventStart < end) {
          if (eventStart < start) {
            segStart = cloneDate(start);
            isStart = false;
          } else {
            segStart = eventStart;
            isStart = true;
          }
          if (eventEnd > end) {
            segEnd = cloneDate(end);
            isEnd = false;
          } else {
            segEnd = eventEnd;
            isEnd = true;
          }
          segs.push({
            event: event,
            start: segStart,
            end: segEnd,
            isStart: isStart,
            isEnd: isEnd
          });
        }
      }
      return segs.sort(compareSlotSegs);
    }
    function slotEventEnd(event) {
      if (event.end) {
        return cloneDate(event.end);
      } else {
        return addMinutes(cloneDate(event.start), opt('defaultEventMinutes'));
      }
    }
    // renders events in the 'time slots' at the bottom
    // TODO: when we refactor this, when user returns `false` eventRender, don't have empty space
    // TODO: refactor will include using pixels to detect collisions instead of dates (handy for seg cmp)
    function renderSlotSegs(segs, modifiedEventId) {
      var i, segCnt = segs.length, seg, event, top, bottom, columnLeft, columnRight, columnWidth, width, left, right, html = '', eventElements, eventElement, triggerRes, titleElement, height, slotSegmentContainer = getSlotSegmentContainer(), isRTL = opt('isRTL');
      // calculate position/dimensions, create html
      for (i = 0; i < segCnt; i++) {
        seg = segs[i];
        event = seg.event;
        top = timePosition(seg.start, seg.start);
        bottom = timePosition(seg.start, seg.end);
        columnLeft = colContentLeft(seg.col);
        columnRight = colContentRight(seg.col);
        columnWidth = columnRight - columnLeft;
        // shave off space on right near scrollbars (2.5%)
        // TODO: move this to CSS somehow
        columnRight -= columnWidth * 0.025;
        columnWidth = columnRight - columnLeft;
        width = columnWidth * (seg.forwardCoord - seg.backwardCoord);
        if (opt('slotEventOverlap')) {
          // double the width while making sure resize handle is visible
          // (assumed to be 20px wide)
          width = Math.max((width - 20 / 2) * 2, width);
        }
        if (isRTL) {
          right = columnRight - seg.backwardCoord * columnWidth;
          left = right - width;
        } else {
          left = columnLeft + seg.backwardCoord * columnWidth;
          right = left + width;
        }
        // make sure horizontal coordinates are in bounds
        left = Math.max(left, columnLeft);
        right = Math.min(right, columnRight);
        width = right - left;
        seg.top = top;
        seg.left = left;
        seg.outerWidth = width;
        seg.outerHeight = bottom - top;
        html += slotSegHtml(event, seg);
      }
      slotSegmentContainer[0].innerHTML = html;
      // faster than html()
      eventElements = slotSegmentContainer.children();
      // retrieve elements, run through eventRender callback, bind event handlers
      for (i = 0; i < segCnt; i++) {
        seg = segs[i];
        event = seg.event;
        eventElement = $(eventElements[i]);
        // faster than eq()
        triggerRes = trigger('eventRender', event, event, eventElement);
        if (triggerRes === false) {
          eventElement.remove();
        } else {
          if (triggerRes && triggerRes !== true) {
            eventElement.remove();
            eventElement = $(triggerRes).css({
              position: 'absolute',
              top: seg.top,
              left: seg.left
            }).appendTo(slotSegmentContainer);
          }
          seg.element = eventElement;
          if (event._id === modifiedEventId) {
            bindSlotSeg(event, eventElement, seg);
          } else {
            eventElement[0]._fci = i;  // for lazySegBind
          }
          reportEventElement(event, eventElement);
        }
      }
      lazySegBind(slotSegmentContainer, segs, bindSlotSeg);
      // record event sides and title positions
      for (i = 0; i < segCnt; i++) {
        seg = segs[i];
        if (eventElement = seg.element) {
          seg.vsides = vsides(eventElement, true);
          seg.hsides = hsides(eventElement, true);
          titleElement = eventElement.find('.fc-event-title');
          if (titleElement.length) {
            seg.contentTop = titleElement[0].offsetTop;
          }
        }
      }
      // set all positions/dimensions at once
      for (i = 0; i < segCnt; i++) {
        seg = segs[i];
        if (eventElement = seg.element) {
          eventElement[0].style.width = Math.max(0, seg.outerWidth - seg.hsides) + 'px';
          height = Math.max(0, seg.outerHeight - seg.vsides);
          eventElement[0].style.height = height + 'px';
          event = seg.event;
          if (seg.contentTop !== undefined && height - seg.contentTop < 10) {
            // not enough room for title, put it in the time (TODO: maybe make both display:inline instead)
            eventElement.find('div.fc-event-time').text(formatDate(event.start, opt('timeFormat')) + ' - ' + event.title);
            eventElement.find('div.fc-event-title').remove();
          }
          trigger('eventAfterRender', event, event, eventElement);
        }
      }
    }
    function slotSegHtml(event, seg) {
      var html = '<';
      var url = event.url;
      var skinCss = getSkinCss(event, opt);
      var classes = [
          'fc-event',
          'fc-event-vert'
        ];
      if (isEventDraggable(event)) {
        classes.push('fc-event-draggable');
      }
      if (seg.isStart) {
        classes.push('fc-event-start');
      }
      if (seg.isEnd) {
        classes.push('fc-event-end');
      }
      classes = classes.concat(event.className);
      if (event.source) {
        classes = classes.concat(event.source.className || []);
      }
      if (url) {
        html += 'a href=\'' + htmlEscape(event.url) + '\'';
      } else {
        html += 'div';
      }
      html += ' class=\'' + classes.join(' ') + '\'' + ' style=' + '\'' + 'position:absolute;' + 'top:' + seg.top + 'px;' + 'left:' + seg.left + 'px;' + skinCss + '\'' + '>' + '<div class=\'fc-event-inner\'>' + '<div class=\'fc-event-time\'>' + htmlEscape(formatDates(event.start, event.end, opt('timeFormat'))) + '</div>' + '<div class=\'fc-event-title\'>' + htmlEscape(event.title || '') + '</div>' + '</div>' + '<div class=\'fc-event-bg\'></div>';
      if (seg.isEnd && isEventResizable(event)) {
        html += '<div class=\'ui-resizable-handle ui-resizable-s\'>=</div>';
      }
      html += '</' + (url ? 'a' : 'div') + '>';
      return html;
    }
    function bindSlotSeg(event, eventElement, seg) {
      var timeElement = eventElement.find('div.fc-event-time');
      if (isEventDraggable(event)) {
        draggableSlotEvent(event, eventElement, timeElement);
      }
      if (seg.isEnd && isEventResizable(event)) {
        resizableSlotEvent(event, eventElement, timeElement);
      }
      eventElementHandlers(event, eventElement);
    }
    /* Dragging
	-----------------------------------------------------------------------------------*/
    // when event starts out FULL-DAY
    // overrides DayEventRenderer's version because it needs to account for dragging elements
    // to and from the slot area.
    function draggableDayEvent(event, eventElement, seg) {
      var isStart = seg.isStart;
      var origWidth;
      var revert;
      var allDay = true;
      var dayDelta;
      var hoverListener = getHoverListener();
      var colWidth = getColWidth();
      var snapHeight = getSnapHeight();
      var snapMinutes = getSnapMinutes();
      var minMinute = getMinMinute();
      eventElement.draggable({
        opacity: opt('dragOpacity', 'month'),
        revertDuration: opt('dragRevertDuration'),
        start: function (ev, ui) {
          trigger('eventDragStart', eventElement, event, ev, ui);
          hideEvents(event, eventElement);
          origWidth = eventElement.width();
          hoverListener.start(function (cell, origCell) {
            clearOverlays();
            if (cell) {
              revert = false;
              var origDate = cellToDate(0, origCell.col);
              var date = cellToDate(0, cell.col);
              dayDelta = dayDiff(date, origDate);
              if (!cell.row) {
                // on full-days
                renderDayOverlay(addDays(cloneDate(event.start), dayDelta), addDays(exclEndDay(event), dayDelta));
                resetElement();
              } else {
                // mouse is over bottom slots
                if (isStart) {
                  if (allDay) {
                    // convert event to temporary slot-event
                    eventElement.width(colWidth - 10);
                    // don't use entire width
                    setOuterHeight(eventElement, snapHeight * Math.round((event.end ? (event.end - event.start) / MINUTE_MS : opt('defaultEventMinutes')) / snapMinutes));
                    eventElement.draggable('option', 'grid', [
                      colWidth,
                      1
                    ]);
                    allDay = false;
                  }
                } else {
                  revert = true;
                }
              }
              revert = revert || allDay && !dayDelta;
            } else {
              resetElement();
              revert = true;
            }
            eventElement.draggable('option', 'revert', revert);
          }, ev, 'drag');
        },
        stop: function (ev, ui) {
          hoverListener.stop();
          clearOverlays();
          trigger('eventDragStop', eventElement, event, ev, ui);
          if (revert) {
            // hasn't moved or is out of bounds (draggable has already reverted)
            resetElement();
            eventElement.css('filter', '');
            // clear IE opacity side-effects
            showEvents(event, eventElement);
          } else {
            // changed!
            var minuteDelta = 0;
            if (!allDay) {
              minuteDelta = Math.round((eventElement.offset().top - getSlotContainer().offset().top) / snapHeight) * snapMinutes + minMinute - (event.start.getHours() * 60 + event.start.getMinutes());
            }
            eventDrop(this, event, dayDelta, minuteDelta, allDay, ev, ui);
          }
        }
      });
      function resetElement() {
        if (!allDay) {
          eventElement.width(origWidth).height('').draggable('option', 'grid', null);
          allDay = true;
        }
      }
    }
    // when event starts out IN TIMESLOTS
    function draggableSlotEvent(event, eventElement, timeElement) {
      var coordinateGrid = t.getCoordinateGrid();
      var colCnt = getColCnt();
      var colWidth = getColWidth();
      var snapHeight = getSnapHeight();
      var snapMinutes = getSnapMinutes();
      // states
      var origPosition;
      // original position of the element, not the mouse
      var origCell;
      var isInBounds, prevIsInBounds;
      var isAllDay, prevIsAllDay;
      var colDelta, prevColDelta;
      var dayDelta;
      // derived from colDelta
      var minuteDelta, prevMinuteDelta;
      eventElement.draggable({
        scroll: false,
        grid: [
          colWidth,
          snapHeight
        ],
        axis: colCnt == 1 ? 'y' : false,
        opacity: opt('dragOpacity'),
        revertDuration: opt('dragRevertDuration'),
        start: function (ev, ui) {
          trigger('eventDragStart', eventElement, event, ev, ui);
          hideEvents(event, eventElement);
          coordinateGrid.build();
          // initialize states
          origPosition = eventElement.position();
          origCell = coordinateGrid.cell(ev.pageX, ev.pageY);
          isInBounds = prevIsInBounds = true;
          isAllDay = prevIsAllDay = getIsCellAllDay(origCell);
          colDelta = prevColDelta = 0;
          dayDelta = 0;
          minuteDelta = prevMinuteDelta = 0;
        },
        drag: function (ev, ui) {
          // NOTE: this `cell` value is only useful for determining in-bounds and all-day.
          // Bad for anything else due to the discrepancy between the mouse position and the
          // element position while snapping. (problem revealed in PR #55)
          //
          // PS- the problem exists for draggableDayEvent() when dragging an all-day event to a slot event.
          // We should overhaul the dragging system and stop relying on jQuery UI.
          var cell = coordinateGrid.cell(ev.pageX, ev.pageY);
          // update states
          isInBounds = !!cell;
          if (isInBounds) {
            isAllDay = getIsCellAllDay(cell);
            // calculate column delta
            colDelta = Math.round((ui.position.left - origPosition.left) / colWidth);
            if (colDelta != prevColDelta) {
              // calculate the day delta based off of the original clicked column and the column delta
              var origDate = cellToDate(0, origCell.col);
              var col = origCell.col + colDelta;
              col = Math.max(0, col);
              col = Math.min(colCnt - 1, col);
              var date = cellToDate(0, col);
              dayDelta = dayDiff(date, origDate);
            }
            // calculate minute delta (only if over slots)
            if (!isAllDay) {
              minuteDelta = Math.round((ui.position.top - origPosition.top) / snapHeight) * snapMinutes;
            }
          }
          // any state changes?
          if (isInBounds != prevIsInBounds || isAllDay != prevIsAllDay || colDelta != prevColDelta || minuteDelta != prevMinuteDelta) {
            updateUI();
            // update previous states for next time
            prevIsInBounds = isInBounds;
            prevIsAllDay = isAllDay;
            prevColDelta = colDelta;
            prevMinuteDelta = minuteDelta;
          }
          // if out-of-bounds, revert when done, and vice versa.
          eventElement.draggable('option', 'revert', !isInBounds);
        },
        stop: function (ev, ui) {
          clearOverlays();
          trigger('eventDragStop', eventElement, event, ev, ui);
          if (isInBounds && (isAllDay || dayDelta || minuteDelta)) {
            // changed!
            eventDrop(this, event, dayDelta, isAllDay ? 0 : minuteDelta, isAllDay, ev, ui);
          } else {
            // either no change or out-of-bounds (draggable has already reverted)
            // reset states for next time, and for updateUI()
            isInBounds = true;
            isAllDay = false;
            colDelta = 0;
            dayDelta = 0;
            minuteDelta = 0;
            updateUI();
            eventElement.css('filter', '');
            // clear IE opacity side-effects
            // sometimes fast drags make event revert to wrong position, so reset.
            // also, if we dragged the element out of the area because of snapping,
            // but the *mouse* is still in bounds, we need to reset the position.
            eventElement.css(origPosition);
            showEvents(event, eventElement);
          }
        }
      });
      function updateUI() {
        clearOverlays();
        if (isInBounds) {
          if (isAllDay) {
            timeElement.hide();
            eventElement.draggable('option', 'grid', null);
            // disable grid snapping
            renderDayOverlay(addDays(cloneDate(event.start), dayDelta), addDays(exclEndDay(event), dayDelta));
          } else {
            updateTimeText(minuteDelta);
            timeElement.css('display', '');
            // show() was causing display=inline
            eventElement.draggable('option', 'grid', [
              colWidth,
              snapHeight
            ]);  // re-enable grid snapping
          }
        }
      }
      function updateTimeText(minuteDelta) {
        var newStart = addMinutes(cloneDate(event.start), minuteDelta);
        var newEnd;
        if (event.end) {
          newEnd = addMinutes(cloneDate(event.end), minuteDelta);
        }
        timeElement.text(formatDates(newStart, newEnd, opt('timeFormat')));
      }
    }
    /* Resizing
	--------------------------------------------------------------------------------------*/
    function resizableSlotEvent(event, eventElement, timeElement) {
      var snapDelta, prevSnapDelta;
      var snapHeight = getSnapHeight();
      var snapMinutes = getSnapMinutes();
      eventElement.resizable({
        handles: { s: '.ui-resizable-handle' },
        grid: snapHeight,
        start: function (ev, ui) {
          snapDelta = prevSnapDelta = 0;
          hideEvents(event, eventElement);
          trigger('eventResizeStart', this, event, ev, ui);
        },
        resize: function (ev, ui) {
          // don't rely on ui.size.height, doesn't take grid into account
          snapDelta = Math.round((Math.max(snapHeight, eventElement.height()) - ui.originalSize.height) / snapHeight);
          if (snapDelta != prevSnapDelta) {
            timeElement.text(formatDates(event.start, !snapDelta && !event.end ? null : addMinutes(eventEnd(event), snapMinutes * snapDelta), opt('timeFormat')));
            prevSnapDelta = snapDelta;
          }
        },
        stop: function (ev, ui) {
          trigger('eventResizeStop', this, event, ev, ui);
          if (snapDelta) {
            eventResize(this, event, 0, snapMinutes * snapDelta, ev, ui);
          } else {
            showEvents(event, eventElement);  // BUG: if event was really short, need to put title back in span
          }
        }
      });
    }
  }
  /* Agenda Event Segment Utilities
-----------------------------------------------------------------------------*/
  // Sets the seg.backwardCoord and seg.forwardCoord on each segment and returns a new
  // list in the order they should be placed into the DOM (an implicit z-index).
  function placeSlotSegs(segs) {
    var levels = buildSlotSegLevels(segs);
    var level0 = levels[0];
    var i;
    computeForwardSlotSegs(levels);
    if (level0) {
      for (i = 0; i < level0.length; i++) {
        computeSlotSegPressures(level0[i]);
      }
      for (i = 0; i < level0.length; i++) {
        computeSlotSegCoords(level0[i], 0, 0);
      }
    }
    return flattenSlotSegLevels(levels);
  }
  // Builds an array of segments "levels". The first level will be the leftmost tier of segments
  // if the calendar is left-to-right, or the rightmost if the calendar is right-to-left.
  function buildSlotSegLevels(segs) {
    var levels = [];
    var i, seg;
    var j;
    for (i = 0; i < segs.length; i++) {
      seg = segs[i];
      // go through all the levels and stop on the first level where there are no collisions
      for (j = 0; j < levels.length; j++) {
        if (!computeSlotSegCollisions(seg, levels[j]).length) {
          break;
        }
      }
      (levels[j] || (levels[j] = [])).push(seg);
    }
    return levels;
  }
  // For every segment, figure out the other segments that are in subsequent
  // levels that also occupy the same vertical space. Accumulate in seg.forwardSegs
  function computeForwardSlotSegs(levels) {
    var i, level;
    var j, seg;
    var k;
    for (i = 0; i < levels.length; i++) {
      level = levels[i];
      for (j = 0; j < level.length; j++) {
        seg = level[j];
        seg.forwardSegs = [];
        for (k = i + 1; k < levels.length; k++) {
          computeSlotSegCollisions(seg, levels[k], seg.forwardSegs);
        }
      }
    }
  }
  // Figure out which path forward (via seg.forwardSegs) results in the longest path until
  // the furthest edge is reached. The number of segments in this path will be seg.forwardPressure
  function computeSlotSegPressures(seg) {
    var forwardSegs = seg.forwardSegs;
    var forwardPressure = 0;
    var i, forwardSeg;
    if (seg.forwardPressure === undefined) {
      // not already computed
      for (i = 0; i < forwardSegs.length; i++) {
        forwardSeg = forwardSegs[i];
        // figure out the child's maximum forward path
        computeSlotSegPressures(forwardSeg);
        // either use the existing maximum, or use the child's forward pressure
        // plus one (for the forwardSeg itself)
        forwardPressure = Math.max(forwardPressure, 1 + forwardSeg.forwardPressure);
      }
      seg.forwardPressure = forwardPressure;
    }
  }
  // Calculate seg.forwardCoord and seg.backwardCoord for the segment, where both values range
  // from 0 to 1. If the calendar is left-to-right, the seg.backwardCoord maps to "left" and
  // seg.forwardCoord maps to "right" (via percentage). Vice-versa if the calendar is right-to-left.
  //
  // The segment might be part of a "series", which means consecutive segments with the same pressure
  // who's width is unknown until an edge has been hit. `seriesBackwardPressure` is the number of
  // segments behind this one in the current series, and `seriesBackwardCoord` is the starting
  // coordinate of the first segment in the series.
  function computeSlotSegCoords(seg, seriesBackwardPressure, seriesBackwardCoord) {
    var forwardSegs = seg.forwardSegs;
    var i;
    if (seg.forwardCoord === undefined) {
      // not already computed
      if (!forwardSegs.length) {
        // if there are no forward segments, this segment should butt up against the edge
        seg.forwardCoord = 1;
      } else {
        // sort highest pressure first
        forwardSegs.sort(compareForwardSlotSegs);
        // this segment's forwardCoord will be calculated from the backwardCoord of the
        // highest-pressure forward segment.
        computeSlotSegCoords(forwardSegs[0], seriesBackwardPressure + 1, seriesBackwardCoord);
        seg.forwardCoord = forwardSegs[0].backwardCoord;
      }
      // calculate the backwardCoord from the forwardCoord. consider the series
      seg.backwardCoord = seg.forwardCoord - (seg.forwardCoord - seriesBackwardCoord) / (seriesBackwardPressure + 1);
      // # of segments in the series
      // use this segment's coordinates to computed the coordinates of the less-pressurized
      // forward segments
      for (i = 0; i < forwardSegs.length; i++) {
        computeSlotSegCoords(forwardSegs[i], 0, seg.forwardCoord);
      }
    }
  }
  // Outputs a flat array of segments, from lowest to highest level
  function flattenSlotSegLevels(levels) {
    var segs = [];
    var i, level;
    var j;
    for (i = 0; i < levels.length; i++) {
      level = levels[i];
      for (j = 0; j < level.length; j++) {
        segs.push(level[j]);
      }
    }
    return segs;
  }
  // Find all the segments in `otherSegs` that vertically collide with `seg`.
  // Append into an optionally-supplied `results` array and return.
  function computeSlotSegCollisions(seg, otherSegs, results) {
    results = results || [];
    for (var i = 0; i < otherSegs.length; i++) {
      if (isSlotSegCollision(seg, otherSegs[i])) {
        results.push(otherSegs[i]);
      }
    }
    return results;
  }
  // Do these segments occupy the same vertical space?
  function isSlotSegCollision(seg1, seg2) {
    return seg1.end > seg2.start && seg1.start < seg2.end;
  }
  // A cmp function for determining which forward segment to rely on more when computing coordinates.
  function compareForwardSlotSegs(seg1, seg2) {
    // put higher-pressure first
    return seg2.forwardPressure - seg1.forwardPressure || (seg1.backwardCoord || 0) - (seg2.backwardCoord || 0) || compareSlotSegs(seg1, seg2);
  }
  // A cmp function for determining which segment should be closer to the initial edge
  // (the left edge on a left-to-right calendar).
  function compareSlotSegs(seg1, seg2) {
    return seg1.start - seg2.start || seg2.end - seg2.start - (seg1.end - seg1.start) || (seg1.event.title || '').localeCompare(seg2.event.title);  // tie? alphabetically by title
  }
  ;
  ;
  function View(element, calendar, viewName) {
    var t = this;
    // exports
    t.element = element;
    t.calendar = calendar;
    t.name = viewName;
    t.opt = opt;
    t.trigger = trigger;
    t.isEventDraggable = isEventDraggable;
    t.isEventResizable = isEventResizable;
    t.setEventData = setEventData;
    t.clearEventData = clearEventData;
    t.eventEnd = eventEnd;
    t.reportEventElement = reportEventElement;
    t.triggerEventDestroy = triggerEventDestroy;
    t.eventElementHandlers = eventElementHandlers;
    t.showEvents = showEvents;
    t.hideEvents = hideEvents;
    t.eventDrop = eventDrop;
    t.eventResize = eventResize;
    // t.title
    // t.start, t.end
    // t.visStart, t.visEnd
    // imports
    var defaultEventEnd = t.defaultEventEnd;
    var normalizeEvent = calendar.normalizeEvent;
    // in EventManager
    var reportEventChange = calendar.reportEventChange;
    // locals
    var eventsByID = {};
    // eventID mapped to array of events (there can be multiple b/c of repeating events)
    var eventElementsByID = {};
    // eventID mapped to array of jQuery elements
    var eventElementCouples = [];
    // array of objects, { event, element } // TODO: unify with segment system
    var options = calendar.options;
    function opt(name, viewNameOverride) {
      var v = options[name];
      if ($.isPlainObject(v)) {
        return smartProperty(v, viewNameOverride || viewName);
      }
      return v;
    }
    function trigger(name, thisObj) {
      return calendar.trigger.apply(calendar, [
        name,
        thisObj || t
      ].concat(Array.prototype.slice.call(arguments, 2), [t]));
    }
    /* Event Editable Boolean Calculations
	------------------------------------------------------------------------------*/
    function isEventDraggable(event) {
      var source = event.source || {};
      return firstDefined(event.startEditable, source.startEditable, opt('eventStartEditable'), event.editable, source.editable, opt('editable')) && !opt('disableDragging');  // deprecated
    }
    function isEventResizable(event) {
      // but also need to make sure the seg.isEnd == true
      var source = event.source || {};
      return firstDefined(event.durationEditable, source.durationEditable, opt('eventDurationEditable'), event.editable, source.editable, opt('editable')) && !opt('disableResizing');  // deprecated
    }
    /* Event Data
	------------------------------------------------------------------------------*/
    function setEventData(events) {
      // events are already normalized at this point
      eventsByID = {};
      var i, len = events.length, event;
      for (i = 0; i < len; i++) {
        event = events[i];
        if (eventsByID[event._id]) {
          eventsByID[event._id].push(event);
        } else {
          eventsByID[event._id] = [event];
        }
      }
    }
    function clearEventData() {
      eventsByID = {};
      eventElementsByID = {};
      eventElementCouples = [];
    }
    // returns a Date object for an event's end
    function eventEnd(event) {
      return event.end ? cloneDate(event.end) : defaultEventEnd(event);
    }
    /* Event Elements
	------------------------------------------------------------------------------*/
    // report when view creates an element for an event
    function reportEventElement(event, element) {
      eventElementCouples.push({
        event: event,
        element: element
      });
      if (eventElementsByID[event._id]) {
        eventElementsByID[event._id].push(element);
      } else {
        eventElementsByID[event._id] = [element];
      }
    }
    function triggerEventDestroy() {
      $.each(eventElementCouples, function (i, couple) {
        t.trigger('eventDestroy', couple.event, couple.event, couple.element);
      });
    }
    // attaches eventClick, eventMouseover, eventMouseout
    function eventElementHandlers(event, eventElement) {
      eventElement.click(function (ev) {
        if (!eventElement.hasClass('ui-draggable-dragging') && !eventElement.hasClass('ui-resizable-resizing')) {
          return trigger('eventClick', this, event, ev);
        }
      }).hover(function (ev) {
        trigger('eventMouseover', this, event, ev);
      }, function (ev) {
        trigger('eventMouseout', this, event, ev);
      });  // TODO: don't fire eventMouseover/eventMouseout *while* dragging is occuring (on subject element)
           // TODO: same for resizing
    }
    function showEvents(event, exceptElement) {
      eachEventElement(event, exceptElement, 'show');
    }
    function hideEvents(event, exceptElement) {
      eachEventElement(event, exceptElement, 'hide');
    }
    function eachEventElement(event, exceptElement, funcName) {
      // NOTE: there may be multiple events per ID (repeating events)
      // and multiple segments per event
      var elements = eventElementsByID[event._id], i, len = elements.length;
      for (i = 0; i < len; i++) {
        if (!exceptElement || elements[i][0] != exceptElement[0]) {
          elements[i][funcName]();
        }
      }
    }
    /* Event Modification Reporting
	---------------------------------------------------------------------------------*/
    function eventDrop(e, event, dayDelta, minuteDelta, allDay, ev, ui) {
      var oldAllDay = event.allDay;
      var eventId = event._id;
      moveEvents(eventsByID[eventId], dayDelta, minuteDelta, allDay);
      trigger('eventDrop', e, event, dayDelta, minuteDelta, allDay, function () {
        // TODO: investigate cases where this inverse technique might not work
        moveEvents(eventsByID[eventId], -dayDelta, -minuteDelta, oldAllDay);
        reportEventChange(eventId);
      }, ev, ui);
      reportEventChange(eventId);
    }
    function eventResize(e, event, dayDelta, minuteDelta, ev, ui) {
      var eventId = event._id;
      elongateEvents(eventsByID[eventId], dayDelta, minuteDelta);
      trigger('eventResize', e, event, dayDelta, minuteDelta, function () {
        // TODO: investigate cases where this inverse technique might not work
        elongateEvents(eventsByID[eventId], -dayDelta, -minuteDelta);
        reportEventChange(eventId);
      }, ev, ui);
      reportEventChange(eventId);
    }
    /* Event Modification Math
	---------------------------------------------------------------------------------*/
    function moveEvents(events, dayDelta, minuteDelta, allDay) {
      minuteDelta = minuteDelta || 0;
      for (var e, len = events.length, i = 0; i < len; i++) {
        e = events[i];
        if (allDay !== undefined) {
          e.allDay = allDay;
        }
        addMinutes(addDays(e.start, dayDelta, true), minuteDelta);
        if (e.end) {
          e.end = addMinutes(addDays(e.end, dayDelta, true), minuteDelta);
        }
        normalizeEvent(e, options);
      }
    }
    function elongateEvents(events, dayDelta, minuteDelta) {
      minuteDelta = minuteDelta || 0;
      for (var e, len = events.length, i = 0; i < len; i++) {
        e = events[i];
        e.end = addMinutes(addDays(eventEnd(e), dayDelta, true), minuteDelta);
        normalizeEvent(e, options);
      }
    }
    // ====================================================================================================
    // Utilities for day "cells"
    // ====================================================================================================
    // The "basic" views are completely made up of day cells.
    // The "agenda" views have day cells at the top "all day" slot.
    // This was the obvious common place to put these utilities, but they should be abstracted out into
    // a more meaningful class (like DayEventRenderer).
    // ====================================================================================================
    // For determining how a given "cell" translates into a "date":
    //
    // 1. Convert the "cell" (row and column) into a "cell offset" (the # of the cell, cronologically from the first).
    //    Keep in mind that column indices are inverted with isRTL. This is taken into account.
    //
    // 2. Convert the "cell offset" to a "day offset" (the # of days since the first visible day in the view).
    //
    // 3. Convert the "day offset" into a "date" (a JavaScript Date object).
    //
    // The reverse transformation happens when transforming a date into a cell.
    // exports
    t.isHiddenDay = isHiddenDay;
    t.skipHiddenDays = skipHiddenDays;
    t.getCellsPerWeek = getCellsPerWeek;
    t.dateToCell = dateToCell;
    t.dateToDayOffset = dateToDayOffset;
    t.dayOffsetToCellOffset = dayOffsetToCellOffset;
    t.cellOffsetToCell = cellOffsetToCell;
    t.cellToDate = cellToDate;
    t.cellToCellOffset = cellToCellOffset;
    t.cellOffsetToDayOffset = cellOffsetToDayOffset;
    t.dayOffsetToDate = dayOffsetToDate;
    t.rangeToSegments = rangeToSegments;
    // internals
    var hiddenDays = opt('hiddenDays') || [];
    // array of day-of-week indices that are hidden
    var isHiddenDayHash = [];
    // is the day-of-week hidden? (hash with day-of-week-index -> bool)
    var cellsPerWeek;
    var dayToCellMap = [];
    // hash from dayIndex -> cellIndex, for one week
    var cellToDayMap = [];
    // hash from cellIndex -> dayIndex, for one week
    var isRTL = opt('isRTL');
    // initialize important internal variables
    (function () {
      if (opt('weekends') === false) {
        hiddenDays.push(0, 6);  // 0=sunday, 6=saturday
      }
      // Loop through a hypothetical week and determine which
      // days-of-week are hidden. Record in both hashes (one is the reverse of the other).
      for (var dayIndex = 0, cellIndex = 0; dayIndex < 7; dayIndex++) {
        dayToCellMap[dayIndex] = cellIndex;
        isHiddenDayHash[dayIndex] = $.inArray(dayIndex, hiddenDays) != -1;
        if (!isHiddenDayHash[dayIndex]) {
          cellToDayMap[cellIndex] = dayIndex;
          cellIndex++;
        }
      }
      cellsPerWeek = cellIndex;
      if (!cellsPerWeek) {
        throw 'invalid hiddenDays';  // all days were hidden? bad.
      }
    }());
    // Is the current day hidden?
    // `day` is a day-of-week index (0-6), or a Date object
    function isHiddenDay(day) {
      if (typeof day == 'object') {
        day = day.getDay();
      }
      return isHiddenDayHash[day];
    }
    function getCellsPerWeek() {
      return cellsPerWeek;
    }
    // Keep incrementing the current day until it is no longer a hidden day.
    // If the initial value of `date` is not a hidden day, don't do anything.
    // Pass `isExclusive` as `true` if you are dealing with an end date.
    // `inc` defaults to `1` (increment one day forward each time)
    function skipHiddenDays(date, inc, isExclusive) {
      inc = inc || 1;
      while (isHiddenDayHash[(date.getDay() + (isExclusive ? inc : 0) + 7) % 7]) {
        addDays(date, inc);
      }
    }
    //
    // TRANSFORMATIONS: cell -> cell offset -> day offset -> date
    //
    // cell -> date (combines all transformations)
    // Possible arguments:
    // - row, col
    // - { row:#, col: # }
    function cellToDate() {
      var cellOffset = cellToCellOffset.apply(null, arguments);
      var dayOffset = cellOffsetToDayOffset(cellOffset);
      var date = dayOffsetToDate(dayOffset);
      return date;
    }
    // cell -> cell offset
    // Possible arguments:
    // - row, col
    // - { row:#, col:# }
    function cellToCellOffset(row, col) {
      var colCnt = t.getColCnt();
      // rtl variables. wish we could pre-populate these. but where?
      var dis = isRTL ? -1 : 1;
      var dit = isRTL ? colCnt - 1 : 0;
      if (typeof row == 'object') {
        col = row.col;
        row = row.row;
      }
      var cellOffset = row * colCnt + (col * dis + dit);
      // column, adjusted for RTL (dis & dit)
      return cellOffset;
    }
    // cell offset -> day offset
    function cellOffsetToDayOffset(cellOffset) {
      var day0 = t.visStart.getDay();
      // first date's day of week
      cellOffset += dayToCellMap[day0];
      // normlize cellOffset to beginning-of-week
      return Math.floor(cellOffset / cellsPerWeek) * 7 + cellToDayMap[(cellOffset % cellsPerWeek + cellsPerWeek) % cellsPerWeek] - day0;  // adjustment for beginning-of-week normalization
    }
    // day offset -> date (JavaScript Date object)
    function dayOffsetToDate(dayOffset) {
      var date = cloneDate(t.visStart);
      addDays(date, dayOffset);
      return date;
    }
    //
    // TRANSFORMATIONS: date -> day offset -> cell offset -> cell
    //
    // date -> cell (combines all transformations)
    function dateToCell(date) {
      var dayOffset = dateToDayOffset(date);
      var cellOffset = dayOffsetToCellOffset(dayOffset);
      var cell = cellOffsetToCell(cellOffset);
      return cell;
    }
    // date -> day offset
    function dateToDayOffset(date) {
      return dayDiff(date, t.visStart);
    }
    // day offset -> cell offset
    function dayOffsetToCellOffset(dayOffset) {
      var day0 = t.visStart.getDay();
      // first date's day of week
      dayOffset += day0;
      // normalize dayOffset to beginning-of-week
      return Math.floor(dayOffset / 7) * cellsPerWeek + dayToCellMap[(dayOffset % 7 + 7) % 7] - dayToCellMap[day0];  // adjustment for beginning-of-week normalization
    }
    // cell offset -> cell (object with row & col keys)
    function cellOffsetToCell(cellOffset) {
      var colCnt = t.getColCnt();
      // rtl variables. wish we could pre-populate these. but where?
      var dis = isRTL ? -1 : 1;
      var dit = isRTL ? colCnt - 1 : 0;
      var row = Math.floor(cellOffset / colCnt);
      var col = (cellOffset % colCnt + colCnt) % colCnt * dis + dit;
      // column, adjusted for RTL (dis & dit)
      return {
        row: row,
        col: col
      };
    }
    //
    // Converts a date range into an array of segment objects.
    // "Segments" are horizontal stretches of time, sliced up by row.
    // A segment object has the following properties:
    // - row
    // - cols
    // - isStart
    // - isEnd
    //
    function rangeToSegments(startDate, endDate) {
      var rowCnt = t.getRowCnt();
      var colCnt = t.getColCnt();
      var segments = [];
      // array of segments to return
      // day offset for given date range
      var rangeDayOffsetStart = dateToDayOffset(startDate);
      var rangeDayOffsetEnd = dateToDayOffset(endDate);
      // exclusive
      // first and last cell offset for the given date range
      // "last" implies inclusivity
      var rangeCellOffsetFirst = dayOffsetToCellOffset(rangeDayOffsetStart);
      var rangeCellOffsetLast = dayOffsetToCellOffset(rangeDayOffsetEnd) - 1;
      // loop through all the rows in the view
      for (var row = 0; row < rowCnt; row++) {
        // first and last cell offset for the row
        var rowCellOffsetFirst = row * colCnt;
        var rowCellOffsetLast = rowCellOffsetFirst + colCnt - 1;
        // get the segment's cell offsets by constraining the range's cell offsets to the bounds of the row
        var segmentCellOffsetFirst = Math.max(rangeCellOffsetFirst, rowCellOffsetFirst);
        var segmentCellOffsetLast = Math.min(rangeCellOffsetLast, rowCellOffsetLast);
        // make sure segment's offsets are valid and in view
        if (segmentCellOffsetFirst <= segmentCellOffsetLast) {
          // translate to cells
          var segmentCellFirst = cellOffsetToCell(segmentCellOffsetFirst);
          var segmentCellLast = cellOffsetToCell(segmentCellOffsetLast);
          // view might be RTL, so order by leftmost column
          var cols = [
              segmentCellFirst.col,
              segmentCellLast.col
            ].sort();
          // Determine if segment's first/last cell is the beginning/end of the date range.
          // We need to compare "day offset" because "cell offsets" are often ambiguous and
          // can translate to multiple days, and an edge case reveals itself when we the
          // range's first cell is hidden (we don't want isStart to be true).
          var isStart = cellOffsetToDayOffset(segmentCellOffsetFirst) == rangeDayOffsetStart;
          var isEnd = cellOffsetToDayOffset(segmentCellOffsetLast) + 1 == rangeDayOffsetEnd;
          // +1 for comparing exclusively
          segments.push({
            row: row,
            leftCol: cols[0],
            rightCol: cols[1],
            isStart: isStart,
            isEnd: isEnd
          });
        }
      }
      return segments;
    }
  }
  ;
  ;
  function DayEventRenderer() {
    var t = this;
    // exports
    t.renderDayEvents = renderDayEvents;
    t.draggableDayEvent = draggableDayEvent;
    // made public so that subclasses can override
    t.resizableDayEvent = resizableDayEvent;
    // "
    // imports
    var opt = t.opt;
    var trigger = t.trigger;
    var isEventDraggable = t.isEventDraggable;
    var isEventResizable = t.isEventResizable;
    var eventEnd = t.eventEnd;
    var reportEventElement = t.reportEventElement;
    var eventElementHandlers = t.eventElementHandlers;
    var showEvents = t.showEvents;
    var hideEvents = t.hideEvents;
    var eventDrop = t.eventDrop;
    var eventResize = t.eventResize;
    var getRowCnt = t.getRowCnt;
    var getColCnt = t.getColCnt;
    var getColWidth = t.getColWidth;
    var allDayRow = t.allDayRow;
    // TODO: rename
    var colLeft = t.colLeft;
    var colRight = t.colRight;
    var colContentLeft = t.colContentLeft;
    var colContentRight = t.colContentRight;
    var dateToCell = t.dateToCell;
    var getDaySegmentContainer = t.getDaySegmentContainer;
    var formatDates = t.calendar.formatDates;
    var renderDayOverlay = t.renderDayOverlay;
    var clearOverlays = t.clearOverlays;
    var clearSelection = t.clearSelection;
    var getHoverListener = t.getHoverListener;
    var rangeToSegments = t.rangeToSegments;
    var cellToDate = t.cellToDate;
    var cellToCellOffset = t.cellToCellOffset;
    var cellOffsetToDayOffset = t.cellOffsetToDayOffset;
    var dateToDayOffset = t.dateToDayOffset;
    var dayOffsetToCellOffset = t.dayOffsetToCellOffset;
    // Render `events` onto the calendar, attach mouse event handlers, and call the `eventAfterRender` callback for each.
    // Mouse event will be lazily applied, except if the event has an ID of `modifiedEventId`.
    // Can only be called when the event container is empty (because it wipes out all innerHTML).
    function renderDayEvents(events, modifiedEventId) {
      // do the actual rendering. Receive the intermediate "segment" data structures.
      var segments = _renderDayEvents(events, false, true);
      // report the elements to the View, for general drag/resize utilities
      segmentElementEach(segments, function (segment, element) {
        reportEventElement(segment.event, element);
      });
      // attach mouse handlers
      attachHandlers(segments, modifiedEventId);
      // call `eventAfterRender` callback for each event
      segmentElementEach(segments, function (segment, element) {
        trigger('eventAfterRender', segment.event, segment.event, element);
      });
    }
    // Render an event on the calendar, but don't report them anywhere, and don't attach mouse handlers.
    // Append this event element to the event container, which might already be populated with events.
    // If an event's segment will have row equal to `adjustRow`, then explicitly set its top coordinate to `adjustTop`.
    // This hack is used to maintain continuity when user is manually resizing an event.
    // Returns an array of DOM elements for the event.
    function renderTempDayEvent(event, adjustRow, adjustTop) {
      // actually render the event. `true` for appending element to container.
      // Recieve the intermediate "segment" data structures.
      var segments = _renderDayEvents([event], true, false);
      var elements = [];
      // Adjust certain elements' top coordinates
      segmentElementEach(segments, function (segment, element) {
        if (segment.row === adjustRow) {
          element.css('top', adjustTop);
        }
        elements.push(element[0]);  // accumulate DOM nodes
      });
      return elements;
    }
    // Render events onto the calendar. Only responsible for the VISUAL aspect.
    // Not responsible for attaching handlers or calling callbacks.
    // Set `doAppend` to `true` for rendering elements without clearing the existing container.
    // Set `doRowHeights` to allow setting the height of each row, to compensate for vertical event overflow.
    function _renderDayEvents(events, doAppend, doRowHeights) {
      // where the DOM nodes will eventually end up
      var finalContainer = getDaySegmentContainer();
      // the container where the initial HTML will be rendered.
      // If `doAppend`==true, uses a temporary container.
      var renderContainer = doAppend ? $('<div/>') : finalContainer;
      var segments = buildSegments(events);
      var html;
      var elements;
      // calculate the desired `left` and `width` properties on each segment object
      calculateHorizontals(segments);
      // build the HTML string. relies on `left` property
      html = buildHTML(segments);
      // render the HTML. innerHTML is considerably faster than jQuery's .html()
      renderContainer[0].innerHTML = html;
      // retrieve the individual elements
      elements = renderContainer.children();
      // if we were appending, and thus using a temporary container,
      // re-attach elements to the real container.
      if (doAppend) {
        finalContainer.append(elements);
      }
      // assigns each element to `segment.event`, after filtering them through user callbacks
      resolveElements(segments, elements);
      // Calculate the left and right padding+margin for each element.
      // We need this for setting each element's desired outer width, because of the W3C box model.
      // It's important we do this in a separate pass from acually setting the width on the DOM elements
      // because alternating reading/writing dimensions causes reflow for every iteration.
      segmentElementEach(segments, function (segment, element) {
        segment.hsides = hsides(element, true);  // include margins = `true`
      });
      // Set the width of each element
      segmentElementEach(segments, function (segment, element) {
        element.width(Math.max(0, segment.outerWidth - segment.hsides));
      });
      // Grab each element's outerHeight (setVerticals uses this).
      // To get an accurate reading, it's important to have each element's width explicitly set already.
      segmentElementEach(segments, function (segment, element) {
        segment.outerHeight = element.outerHeight(true);  // include margins = `true`
      });
      // Set the top coordinate on each element (requires segment.outerHeight)
      setVerticals(segments, doRowHeights);
      return segments;
    }
    // Generate an array of "segments" for all events.
    function buildSegments(events) {
      var segments = [];
      for (var i = 0; i < events.length; i++) {
        var eventSegments = buildSegmentsForEvent(events[i]);
        segments.push.apply(segments, eventSegments);  // append an array to an array
      }
      return segments;
    }
    // Generate an array of segments for a single event.
    // A "segment" is the same data structure that View.rangeToSegments produces,
    // with the addition of the `event` property being set to reference the original event.
    function buildSegmentsForEvent(event) {
      var startDate = event.start;
      var endDate = exclEndDay(event);
      var segments = rangeToSegments(startDate, endDate);
      for (var i = 0; i < segments.length; i++) {
        segments[i].event = event;
      }
      return segments;
    }
    // Sets the `left` and `outerWidth` property of each segment.
    // These values are the desired dimensions for the eventual DOM elements.
    function calculateHorizontals(segments) {
      var isRTL = opt('isRTL');
      for (var i = 0; i < segments.length; i++) {
        var segment = segments[i];
        // Determine functions used for calulating the elements left/right coordinates,
        // depending on whether the view is RTL or not.
        // NOTE:
        // colLeft/colRight returns the coordinate butting up the edge of the cell.
        // colContentLeft/colContentRight is indented a little bit from the edge.
        var leftFunc = (isRTL ? segment.isEnd : segment.isStart) ? colContentLeft : colLeft;
        var rightFunc = (isRTL ? segment.isStart : segment.isEnd) ? colContentRight : colRight;
        var left = leftFunc(segment.leftCol);
        var right = rightFunc(segment.rightCol);
        segment.left = left;
        segment.outerWidth = right - left;
      }
    }
    // Build a concatenated HTML string for an array of segments
    function buildHTML(segments) {
      var html = '';
      for (var i = 0; i < segments.length; i++) {
        html += buildHTMLForSegment(segments[i]);
      }
      return html;
    }
    // Build an HTML string for a single segment.
    // Relies on the following properties:
    // - `segment.event` (from `buildSegmentsForEvent`)
    // - `segment.left` (from `calculateHorizontals`)
    function buildHTMLForSegment(segment) {
      var html = '';
      var isRTL = opt('isRTL');
      var event = segment.event;
      var url = event.url;
      // generate the list of CSS classNames
      var classNames = [
          'fc-event',
          'fc-event-hori'
        ];
      if (isEventDraggable(event)) {
        classNames.push('fc-event-draggable');
      }
      if (segment.isStart) {
        classNames.push('fc-event-start');
      }
      if (segment.isEnd) {
        classNames.push('fc-event-end');
      }
      // use the event's configured classNames
      // guaranteed to be an array via `normalizeEvent`
      classNames = classNames.concat(event.className);
      if (event.source) {
        // use the event's source's classNames, if specified
        classNames = classNames.concat(event.source.className || []);
      }
      // generate a semicolon delimited CSS string for any of the "skin" properties
      // of the event object (`backgroundColor`, `borderColor` and such)
      var skinCss = getSkinCss(event, opt);
      if (url) {
        html += '<a href=\'' + htmlEscape(url) + '\'';
      } else {
        html += '<div';
      }
      html += ' class=\'' + classNames.join(' ') + '\'' + ' style=' + '\'' + 'position:absolute;' + 'left:' + segment.left + 'px;' + skinCss + '\'' + '>' + '<div class=\'fc-event-inner\'>';
      if (!event.allDay && segment.isStart) {
        html += '<span class=\'fc-event-time\'>' + htmlEscape(formatDates(event.start, event.end, opt('timeFormat'))) + '</span>';
      }
      html += '<span class=\'fc-event-title\'>' + htmlEscape(event.title || '') + '</span>' + '</div>';
      if (segment.isEnd && isEventResizable(event)) {
        html += '<div class=\'ui-resizable-handle ui-resizable-' + (isRTL ? 'w' : 'e') + '\'>' + '&nbsp;&nbsp;&nbsp;' + '</div>';
      }
      html += '</' + (url ? 'a' : 'div') + '>';
      // TODO:
      // When these elements are initially rendered, they will be briefly visibile on the screen,
      // even though their widths/heights are not set.
      // SOLUTION: initially set them as visibility:hidden ?
      return html;
    }
    // Associate each segment (an object) with an element (a jQuery object),
    // by setting each `segment.element`.
    // Run each element through the `eventRender` filter, which allows developers to
    // modify an existing element, supply a new one, or cancel rendering.
    function resolveElements(segments, elements) {
      for (var i = 0; i < segments.length; i++) {
        var segment = segments[i];
        var event = segment.event;
        var element = elements.eq(i);
        // call the trigger with the original element
        var triggerRes = trigger('eventRender', event, event, element);
        if (triggerRes === false) {
          // if `false`, remove the event from the DOM and don't assign it to `segment.event`
          element.remove();
        } else {
          if (triggerRes && triggerRes !== true) {
            // the trigger returned a new element, but not `true` (which means keep the existing element)
            // re-assign the important CSS dimension properties that were already assigned in `buildHTMLForSegment`
            triggerRes = $(triggerRes).css({
              position: 'absolute',
              left: segment.left
            });
            element.replaceWith(triggerRes);
            element = triggerRes;
          }
          segment.element = element;
        }
      }
    }
    /* Top-coordinate Methods
	-------------------------------------------------------------------------------------------------*/
    // Sets the "top" CSS property for each element.
    // If `doRowHeights` is `true`, also sets each row's first cell to an explicit height,
    // so that if elements vertically overflow, the cell expands vertically to compensate.
    function setVerticals(segments, doRowHeights) {
      var rowContentHeights = calculateVerticals(segments);
      // also sets segment.top
      var rowContentElements = getRowContentElements();
      // returns 1 inner div per row
      var rowContentTops = [];
      // Set each row's height by setting height of first inner div
      if (doRowHeights) {
        for (var i = 0; i < rowContentElements.length; i++) {
          rowContentElements[i].height(rowContentHeights[i]);
        }
      }
      // Get each row's top, relative to the views's origin.
      // Important to do this after setting each row's height.
      for (var i = 0; i < rowContentElements.length; i++) {
        rowContentTops.push(rowContentElements[i].position().top);
      }
      // Set each segment element's CSS "top" property.
      // Each segment object has a "top" property, which is relative to the row's top, but...
      segmentElementEach(segments, function (segment, element) {
        element.css('top', rowContentTops[segment.row] + segment.top);
      });
    }
    // Calculate the "top" coordinate for each segment, relative to the "top" of the row.
    // Also, return an array that contains the "content" height for each row
    // (the height displaced by the vertically stacked events in the row).
    // Requires segments to have their `outerHeight` property already set.
    function calculateVerticals(segments) {
      var rowCnt = getRowCnt();
      var colCnt = getColCnt();
      var rowContentHeights = [];
      // content height for each row
      var segmentRows = buildSegmentRows(segments);
      // an array of segment arrays, one for each row
      for (var rowI = 0; rowI < rowCnt; rowI++) {
        var segmentRow = segmentRows[rowI];
        // an array of running total heights for each column.
        // initialize with all zeros.
        var colHeights = [];
        for (var colI = 0; colI < colCnt; colI++) {
          colHeights.push(0);
        }
        // loop through every segment
        for (var segmentI = 0; segmentI < segmentRow.length; segmentI++) {
          var segment = segmentRow[segmentI];
          // find the segment's top coordinate by looking at the max height
          // of all the columns the segment will be in.
          segment.top = arrayMax(colHeights.slice(segment.leftCol, segment.rightCol + 1));
          // adjust the columns to account for the segment's height
          for (var colI = segment.leftCol; colI <= segment.rightCol; colI++) {
            colHeights[colI] = segment.top + segment.outerHeight;
          }
        }
        // the tallest column in the row should be the "content height"
        rowContentHeights.push(arrayMax(colHeights));
      }
      return rowContentHeights;
    }
    // Build an array of segment arrays, each representing the segments that will
    // be in a row of the grid, sorted by which event should be closest to the top.
    function buildSegmentRows(segments) {
      var rowCnt = getRowCnt();
      var segmentRows = [];
      var segmentI;
      var segment;
      var rowI;
      // group segments by row
      for (segmentI = 0; segmentI < segments.length; segmentI++) {
        segment = segments[segmentI];
        rowI = segment.row;
        if (segment.element) {
          // was rendered?
          if (segmentRows[rowI]) {
            // already other segments. append to array
            segmentRows[rowI].push(segment);
          } else {
            // first segment in row. create new array
            segmentRows[rowI] = [segment];
          }
        }
      }
      // sort each row
      for (rowI = 0; rowI < rowCnt; rowI++) {
        segmentRows[rowI] = sortSegmentRow(segmentRows[rowI] || []);
      }
      return segmentRows;
    }
    // Sort an array of segments according to which segment should appear closest to the top
    function sortSegmentRow(segments) {
      var sortedSegments = [];
      // build the subrow array
      var subrows = buildSegmentSubrows(segments);
      // flatten it
      for (var i = 0; i < subrows.length; i++) {
        sortedSegments.push.apply(sortedSegments, subrows[i]);  // append an array to an array
      }
      return sortedSegments;
    }
    // Take an array of segments, which are all assumed to be in the same row,
    // and sort into subrows.
    function buildSegmentSubrows(segments) {
      // Give preference to elements with certain criteria, so they have
      // a chance to be closer to the top.
      segments.sort(compareDaySegments);
      var subrows = [];
      for (var i = 0; i < segments.length; i++) {
        var segment = segments[i];
        // loop through subrows, starting with the topmost, until the segment
        // doesn't collide with other segments.
        for (var j = 0; j < subrows.length; j++) {
          if (!isDaySegmentCollision(segment, subrows[j])) {
            break;
          }
        }
        // `j` now holds the desired subrow index
        if (subrows[j]) {
          subrows[j].push(segment);
        } else {
          subrows[j] = [segment];
        }
      }
      return subrows;
    }
    // Return an array of jQuery objects for the placeholder content containers of each row.
    // The content containers don't actually contain anything, but their dimensions should match
    // the events that are overlaid on top.
    function getRowContentElements() {
      var i;
      var rowCnt = getRowCnt();
      var rowDivs = [];
      for (i = 0; i < rowCnt; i++) {
        rowDivs[i] = allDayRow(i).find('div.fc-day-content > div');
      }
      return rowDivs;
    }
    /* Mouse Handlers
	---------------------------------------------------------------------------------------------------*/
    // TODO: better documentation!
    function attachHandlers(segments, modifiedEventId) {
      var segmentContainer = getDaySegmentContainer();
      segmentElementEach(segments, function (segment, element, i) {
        var event = segment.event;
        if (event._id === modifiedEventId) {
          bindDaySeg(event, element, segment);
        } else {
          element[0]._fci = i;  // for lazySegBind
        }
      });
      lazySegBind(segmentContainer, segments, bindDaySeg);
    }
    function bindDaySeg(event, eventElement, segment) {
      if (isEventDraggable(event)) {
        t.draggableDayEvent(event, eventElement, segment);  // use `t` so subclasses can override
      }
      if (segment.isEnd && isEventResizable(event)) {
        t.resizableDayEvent(event, eventElement, segment);  // use `t` so subclasses can override
      }
      // attach all other handlers.
      // needs to be after, because resizableDayEvent might stopImmediatePropagation on click
      eventElementHandlers(event, eventElement);
    }
    function draggableDayEvent(event, eventElement) {
      var hoverListener = getHoverListener();
      var dayDelta;
      eventElement.draggable({
        delay: 50,
        opacity: opt('dragOpacity'),
        revertDuration: opt('dragRevertDuration'),
        start: function (ev, ui) {
          trigger('eventDragStart', eventElement, event, ev, ui);
          hideEvents(event, eventElement);
          hoverListener.start(function (cell, origCell, rowDelta, colDelta) {
            eventElement.draggable('option', 'revert', !cell || !rowDelta && !colDelta);
            clearOverlays();
            if (cell) {
              var origDate = cellToDate(origCell);
              var date = cellToDate(cell);
              dayDelta = dayDiff(date, origDate);
              renderDayOverlay(addDays(cloneDate(event.start), dayDelta), addDays(exclEndDay(event), dayDelta));
            } else {
              dayDelta = 0;
            }
          }, ev, 'drag');
        },
        stop: function (ev, ui) {
          hoverListener.stop();
          clearOverlays();
          trigger('eventDragStop', eventElement, event, ev, ui);
          if (dayDelta) {
            eventDrop(this, event, dayDelta, 0, event.allDay, ev, ui);
          } else {
            eventElement.css('filter', '');
            // clear IE opacity side-effects
            showEvents(event, eventElement);
          }
        }
      });
    }
    function resizableDayEvent(event, element, segment) {
      var isRTL = opt('isRTL');
      var direction = isRTL ? 'w' : 'e';
      var handle = element.find('.ui-resizable-' + direction);
      // TODO: stop using this class because we aren't using jqui for this
      var isResizing = false;
      // TODO: look into using jquery-ui mouse widget for this stuff
      disableTextSelection(element);
      // prevent native <a> selection for IE
      element.mousedown(function (ev) {
        // prevent native <a> selection for others
        ev.preventDefault();
      }).click(function (ev) {
        if (isResizing) {
          ev.preventDefault();
          // prevent link from being visited (only method that worked in IE6)
          ev.stopImmediatePropagation();  // prevent fullcalendar eventClick handler from being called
                                          // (eventElementHandlers needs to be bound after resizableDayEvent)
        }
      });
      handle.mousedown(function (ev) {
        if (ev.which != 1) {
          return;  // needs to be left mouse button
        }
        isResizing = true;
        var hoverListener = getHoverListener();
        var rowCnt = getRowCnt();
        var colCnt = getColCnt();
        var elementTop = element.css('top');
        var dayDelta;
        var helpers;
        var eventCopy = $.extend({}, event);
        var minCellOffset = dayOffsetToCellOffset(dateToDayOffset(event.start));
        clearSelection();
        $('body').css('cursor', direction + '-resize').one('mouseup', mouseup);
        trigger('eventResizeStart', this, event, ev);
        hoverListener.start(function (cell, origCell) {
          if (cell) {
            var origCellOffset = cellToCellOffset(origCell);
            var cellOffset = cellToCellOffset(cell);
            // don't let resizing move earlier than start date cell
            cellOffset = Math.max(cellOffset, minCellOffset);
            dayDelta = cellOffsetToDayOffset(cellOffset) - cellOffsetToDayOffset(origCellOffset);
            if (dayDelta) {
              eventCopy.end = addDays(eventEnd(event), dayDelta, true);
              var oldHelpers = helpers;
              helpers = renderTempDayEvent(eventCopy, segment.row, elementTop);
              helpers = $(helpers);
              // turn array into a jQuery object
              helpers.find('*').css('cursor', direction + '-resize');
              if (oldHelpers) {
                oldHelpers.remove();
              }
              hideEvents(event);
            } else {
              if (helpers) {
                showEvents(event);
                helpers.remove();
                helpers = null;
              }
            }
            clearOverlays();
            renderDayOverlay(event.start, addDays(exclEndDay(event), dayDelta));
          }
        }, ev);
        function mouseup(ev) {
          trigger('eventResizeStop', this, event, ev);
          $('body').css('cursor', '');
          hoverListener.stop();
          clearOverlays();
          if (dayDelta) {
            eventResize(this, event, dayDelta, 0, ev);  // event redraw will clear helpers
          }
          // otherwise, the drag handler already restored the old events
          setTimeout(function () {
            // make this happen after the element's click event
            isResizing = false;
          }, 0);
        }
      });
    }
  }
  /* Generalized Segment Utilities
-------------------------------------------------------------------------------------------------*/
  function isDaySegmentCollision(segment, otherSegments) {
    for (var i = 0; i < otherSegments.length; i++) {
      var otherSegment = otherSegments[i];
      if (otherSegment.leftCol <= segment.rightCol && otherSegment.rightCol >= segment.leftCol) {
        return true;
      }
    }
    return false;
  }
  function segmentElementEach(segments, callback) {
    // TODO: use in AgendaView?
    for (var i = 0; i < segments.length; i++) {
      var segment = segments[i];
      var element = segment.element;
      if (element) {
        callback(segment, element, i);
      }
    }
  }
  // A cmp function for determining which segments should appear higher up
  function compareDaySegments(a, b) {
    return b.rightCol - b.leftCol - (a.rightCol - a.leftCol) || b.event.allDay - a.event.allDay || a.event.start - b.event.start || (a.event.title || '').localeCompare(b.event.title);
  }
  ;
  ;
  //BUG: unselect needs to be triggered when events are dragged+dropped
  function SelectionManager() {
    var t = this;
    // exports
    t.select = select;
    t.unselect = unselect;
    t.reportSelection = reportSelection;
    t.daySelectionMousedown = daySelectionMousedown;
    // imports
    var opt = t.opt;
    var trigger = t.trigger;
    var defaultSelectionEnd = t.defaultSelectionEnd;
    var renderSelection = t.renderSelection;
    var clearSelection = t.clearSelection;
    // locals
    var selected = false;
    // unselectAuto
    if (opt('selectable') && opt('unselectAuto')) {
      $(document).mousedown(function (ev) {
        var ignore = opt('unselectCancel');
        if (ignore) {
          if ($(ev.target).parents(ignore).length) {
            // could be optimized to stop after first match
            return;
          }
        }
        unselect(ev);
      });
    }
    function select(startDate, endDate, allDay) {
      unselect();
      if (!endDate) {
        endDate = defaultSelectionEnd(startDate, allDay);
      }
      renderSelection(startDate, endDate, allDay);
      reportSelection(startDate, endDate, allDay);
    }
    function unselect(ev) {
      if (selected) {
        selected = false;
        clearSelection();
        trigger('unselect', null, ev);
      }
    }
    function reportSelection(startDate, endDate, allDay, ev) {
      selected = true;
      trigger('select', null, startDate, endDate, allDay, ev);
    }
    function daySelectionMousedown(ev) {
      // not really a generic manager method, oh well
      var cellToDate = t.cellToDate;
      var getIsCellAllDay = t.getIsCellAllDay;
      var hoverListener = t.getHoverListener();
      var reportDayClick = t.reportDayClick;
      // this is hacky and sort of weird
      if (ev.which == 1 && opt('selectable')) {
        // which==1 means left mouse button
        unselect(ev);
        var _mousedownElement = this;
        var dates;
        hoverListener.start(function (cell, origCell) {
          // TODO: maybe put cellToDate/getIsCellAllDay info in cell
          clearSelection();
          if (cell && getIsCellAllDay(cell)) {
            dates = [
              cellToDate(origCell),
              cellToDate(cell)
            ].sort(dateCompare);
            renderSelection(dates[0], dates[1], true);
          } else {
            dates = null;
          }
        }, ev);
        $(document).one('mouseup', function (ev) {
          hoverListener.stop();
          if (dates) {
            if (+dates[0] == +dates[1]) {
              reportDayClick(dates[0], true, ev);
            }
            reportSelection(dates[0], dates[1], true, ev);
          }
        });
      }
    }
  }
  ;
  ;
  function OverlayManager() {
    var t = this;
    // exports
    t.renderOverlay = renderOverlay;
    t.clearOverlays = clearOverlays;
    // locals
    var usedOverlays = [];
    var unusedOverlays = [];
    function renderOverlay(rect, parent) {
      var e = unusedOverlays.shift();
      if (!e) {
        e = $('<div class=\'fc-cell-overlay\' style=\'position:absolute;z-index:3\'/>');
      }
      if (e[0].parentNode != parent[0]) {
        e.appendTo(parent);
      }
      usedOverlays.push(e.css(rect).show());
      return e;
    }
    function clearOverlays() {
      var e;
      while (e = usedOverlays.shift()) {
        unusedOverlays.push(e.hide().unbind());
      }
    }
  }
  ;
  ;
  function CoordinateGrid(buildFunc) {
    var t = this;
    var rows;
    var cols;
    t.build = function () {
      rows = [];
      cols = [];
      buildFunc(rows, cols);
    };
    t.cell = function (x, y) {
      var rowCnt = rows.length;
      var colCnt = cols.length;
      var i, r = -1, c = -1;
      for (i = 0; i < rowCnt; i++) {
        if (y >= rows[i][0] && y < rows[i][1]) {
          r = i;
          break;
        }
      }
      for (i = 0; i < colCnt; i++) {
        if (x >= cols[i][0] && x < cols[i][1]) {
          c = i;
          break;
        }
      }
      return r >= 0 && c >= 0 ? {
        row: r,
        col: c
      } : null;
    };
    t.rect = function (row0, col0, row1, col1, originElement) {
      // row1,col1 is inclusive
      var origin = originElement.offset();
      return {
        top: rows[row0][0] - origin.top,
        left: cols[col0][0] - origin.left,
        width: cols[col1][1] - cols[col0][0],
        height: rows[row1][1] - rows[row0][0]
      };
    };
  }
  ;
  ;
  function HoverListener(coordinateGrid) {
    var t = this;
    var bindType;
    var change;
    var firstCell;
    var cell;
    t.start = function (_change, ev, _bindType) {
      change = _change;
      firstCell = cell = null;
      coordinateGrid.build();
      mouse(ev);
      bindType = _bindType || 'mousemove';
      $(document).bind(bindType, mouse);
    };
    function mouse(ev) {
      _fixUIEvent(ev);
      // see below
      var newCell = coordinateGrid.cell(ev.pageX, ev.pageY);
      if (!newCell != !cell || newCell && (newCell.row != cell.row || newCell.col != cell.col)) {
        if (newCell) {
          if (!firstCell) {
            firstCell = newCell;
          }
          change(newCell, firstCell, newCell.row - firstCell.row, newCell.col - firstCell.col);
        } else {
          change(newCell, firstCell);
        }
        cell = newCell;
      }
    }
    t.stop = function () {
      $(document).unbind(bindType, mouse);
      return cell;
    };
  }
  // this fix was only necessary for jQuery UI 1.8.16 (and jQuery 1.7 or 1.7.1)
  // upgrading to jQuery UI 1.8.17 (and using either jQuery 1.7 or 1.7.1) fixed the problem
  // but keep this in here for 1.8.16 users
  // and maybe remove it down the line
  function _fixUIEvent(event) {
    // for issue 1168
    if (event.pageX === undefined) {
      event.pageX = event.originalEvent.pageX;
      event.pageY = event.originalEvent.pageY;
    }
  }
  ;
  ;
  function HorizontalPositionCache(getElement) {
    var t = this, elements = {}, lefts = {}, rights = {};
    function e(i) {
      return elements[i] = elements[i] || getElement(i);
    }
    t.left = function (i) {
      return lefts[i] = lefts[i] === undefined ? e(i).position().left : lefts[i];
    };
    t.right = function (i) {
      return rights[i] = rights[i] === undefined ? t.left(i) + e(i).width() : rights[i];
    };
    t.clear = function () {
      elements = {};
      lefts = {};
      rights = {};
    };
  }
  ;
  ;
}(jQuery));
/*!
 * FullCalendar v1.6.7 Google Calendar Plugin
 * Docs & License: http://arshaw.com/fullcalendar/
 * (c) 2013 Adam Shaw
 */
(function ($) {
  var API_BASE = 'https://www.googleapis.com/calendar/v3/calendars';
  var fc = $.fullCalendar;
  var formatDate = fc.formatDate;
  var parseISO8601 = fc.parseISO8601;
  var cloneDate = fc.cloneDate;
  var addDays = fc.addDays;
  var applyAll = fc.applyAll;
  fc.sourceNormalizers.push(function (sourceOptions) {
    var googleCalendarId = sourceOptions.googleCalendarId;
    var url = sourceOptions.url;
    var match;
    // if the Google Calendar ID hasn't been explicitly defined
    if (!googleCalendarId && url) {
      // detect if the ID was specified as a single string
      if (match = /^[^\/]+@([^\/]+\.)?calendar\.google\.com$/.test(url)) {
        googleCalendarId = url;
      }  // try to scrape it out of a V1 or V3 API feed URL
      else if ((match = /^https:\/\/www.googleapis.com\/calendar\/v3\/calendars\/([^\/]*)/.exec(url)) || (match = /^https?:\/\/www.google.com\/calendar\/feeds\/([^\/]*)/.exec(url))) {
        googleCalendarId = decodeURIComponent(match[1]);
      }
      if (googleCalendarId) {
        sourceOptions.googleCalendarId = googleCalendarId;
      }
    }
    if (googleCalendarId) {
      // is this a Google Calendar?
      // make each Google Calendar source uneditable by default
      if (sourceOptions.editable == null) {
        sourceOptions.editable = false;
      }
      // We want removeEventSource to work, but it won't know about the googleCalendarId primitive.
      // Shoehorn it into the url, which will function as the unique primitive. Won't cause side effects.
      // This hack is obsolete since 1.6.7, but keep it so this plugin file is compatible with old versions.
      sourceOptions.url = googleCalendarId;
    }
  });
  fc.sourceFetchers.push(function (sourceOptions, start, end) {
    if (sourceOptions.googleCalendarId) {
      return transformOptions(sourceOptions, start, end);
    }
  });
  function transformOptions(sourceOptions, start, end) {
    var url = API_BASE + '/' + encodeURIComponent(sourceOptions.googleCalendarId) + '/events?callback=?';
    // jsonp
    var apiKey = sourceOptions.googleCalendarApiKey;
    var ctz = sourceOptions.currentTimezone;
    var success = sourceOptions.success;
    var data;
    function reportError(message, apiErrorObjs) {
      var errorObjs = apiErrorObjs || [{ message: message }];
      // to be passed into error handlers
      var consoleObj = window.console;
      var consoleWarnFunc = consoleObj ? consoleObj.warn || consoleObj.log : null;
      // call error handlers
      (sourceOptions.googleCalendarError || $.noop).apply(null, errorObjs);
      // print error to debug console
      if (consoleWarnFunc) {
        consoleWarnFunc.apply(consoleObj, [message].concat(apiErrorObjs || []));
      }
    }
    // The API expects an ISO8601 datetime with a time and timezone part.
    // Since the calendar's timezone offset isn't always known, request the date in UTC and pad it by a day on each
    // side, guaranteeing we will receive all events in the desired range, albeit a superset.
    start = addDays(cloneDate(start), -1);
    end = addDays(cloneDate(end), 1);
    if (!apiKey) {
      reportError('Specify a googleCalendarApiKey. See http://fullcalendar.io/docs1/google_calendar/');
      return {};  // an empty source to use instead. won't fetch anything.
    }
    data = $.extend({}, sourceOptions.data || {}, {
      key: apiKey,
      timeMin: formatDate(start, 'u'),
      timeMax: formatDate(end, 'u'),
      singleEvents: true,
      maxResults: 9999
    });
    if (ctz) {
      data.timeZone = ctz = ctz.replace(' ', '_');
    }
    return $.extend({}, sourceOptions, {
      googleCalendarId: null,
      url: url,
      data: data,
      startParam: false,
      endParam: false,
      success: function (data) {
        var events = [];
        var successArgs;
        var successRes;
        if (data.error) {
          reportError('Google Calendar API: ' + data.error.message, data.error.errors);
        } else if (data.items) {
          $.each(data.items, function (i, entry) {
            var allDay = !entry.start.dateTime && !entry.end.dateTime;
            var start = parseISO8601(entry.start.dateTime || entry.start.date, true);
            var end = parseISO8601(entry.end.dateTime || entry.end.date, true);
            var url = entry.htmlLink;
            if (allDay) {
              addDays(end, -1);  // make inclusive
            }
            if (ctz) {
              url += (url.indexOf('?') == -1 ? '?' : '&') + 'ctz=' + ctz;
            }
            events.push({
              id: entry.id,
              title: entry.summary,
              allDay: allDay,
              start: start,
              end: end,
              url: url,
              location: entry.location,
              description: entry.description
            });
          });
          // call the success handler(s) and allow it to return a new events array
          successArgs = [events].concat(Array.prototype.slice.call(arguments, 1));
          // forward other jq args
          successRes = applyAll(success, this, successArgs);
          if ($.isArray(successRes)) {
            return successRes;
          }
        }
        return events;
      }
    });
  }
  // legacy
  fc.gcalFeed = function (url, sourceOptions) {
    return $.extend({}, sourceOptions, { url: url });
  };
}(jQuery));
/*
 Copyright 2011-2013 Abdulla Abdurakhmanov
 Original sources are available at https://code.google.com/p/x2js/

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 */
(function (root, factory) {
  if (typeof define === 'function' && define.amd) {
    define([], factory);
  } else if (typeof exports === 'object') {
    module.exports = factory();
  } else {
    root.X2JS = factory();
  }
}(this, function () {
  return function (config) {
    'use strict';
    var VERSION = '1.2.0';
    config = config || {};
    initConfigDefaults();
    initRequiredPolyfills();
    function initConfigDefaults() {
      if (config.escapeMode === undefined) {
        config.escapeMode = true;
      }
      config.attributePrefix = config.attributePrefix || '_';
      config.arrayAccessForm = config.arrayAccessForm || 'none';
      config.emptyNodeForm = config.emptyNodeForm || 'text';
      if (config.enableToStringFunc === undefined) {
        config.enableToStringFunc = true;
      }
      config.arrayAccessFormPaths = config.arrayAccessFormPaths || [];
      if (config.skipEmptyTextNodesForObj === undefined) {
        config.skipEmptyTextNodesForObj = true;
      }
      if (config.stripWhitespaces === undefined) {
        config.stripWhitespaces = true;
      }
      config.datetimeAccessFormPaths = config.datetimeAccessFormPaths || [];
      if (config.useDoubleQuotes === undefined) {
        config.useDoubleQuotes = false;
      }
      config.xmlElementsFilter = config.xmlElementsFilter || [];
      config.jsonPropertiesFilter = config.jsonPropertiesFilter || [];
      if (config.keepCData === undefined) {
        config.keepCData = false;
      }
    }
    var DOMNodeTypes = {
        ELEMENT_NODE: 1,
        TEXT_NODE: 3,
        CDATA_SECTION_NODE: 4,
        COMMENT_NODE: 8,
        DOCUMENT_NODE: 9
      };
    function initRequiredPolyfills() {
    }
    function getNodeLocalName(node) {
      var nodeLocalName = node.localName;
      if (nodeLocalName == null)
        // Yeah, this is IE!! 
        nodeLocalName = node.baseName;
      if (nodeLocalName == null || nodeLocalName == '')
        // =="" is IE too
        nodeLocalName = node.nodeName;
      return nodeLocalName;
    }
    function getNodePrefix(node) {
      return node.prefix;
    }
    function escapeXmlChars(str) {
      if (typeof str == 'string')
        return str.replace(/&/g, '&amp;').replace(/</g, '&lt;').replace(/>/g, '&gt;').replace(/"/g, '&quot;').replace(/'/g, '&apos;');
      else
        return str;
    }
    function unescapeXmlChars(str) {
      return str.replace(/&lt;/g, '<').replace(/&gt;/g, '>').replace(/&quot;/g, '"').replace(/&apos;/g, '\'').replace(/&amp;/g, '&');
    }
    function checkInStdFiltersArrayForm(stdFiltersArrayForm, obj, name, path) {
      var idx = 0;
      for (; idx < stdFiltersArrayForm.length; idx++) {
        var filterPath = stdFiltersArrayForm[idx];
        if (typeof filterPath === 'string') {
          if (filterPath == path)
            break;
        } else if (filterPath instanceof RegExp) {
          if (filterPath.test(path))
            break;
        } else if (typeof filterPath === 'function') {
          if (filterPath(obj, name, path))
            break;
        }
      }
      return idx != stdFiltersArrayForm.length;
    }
    function toArrayAccessForm(obj, childName, path) {
      switch (config.arrayAccessForm) {
      case 'property':
        if (!(obj[childName] instanceof Array))
          obj[childName + '_asArray'] = [obj[childName]];
        else
          obj[childName + '_asArray'] = obj[childName];
        break;  /*case "none":
					break;*/
      }
      if (!(obj[childName] instanceof Array) && config.arrayAccessFormPaths.length > 0) {
        if (checkInStdFiltersArrayForm(config.arrayAccessFormPaths, obj, childName, path)) {
          obj[childName] = [obj[childName]];
        }
      }
    }
    function fromXmlDateTime(prop) {
      // Implementation based up on http://stackoverflow.com/questions/8178598/xml-datetime-to-javascript-date-object
      // Improved to support full spec and optional parts
      var bits = prop.split(/[-T:+Z]/g);
      var d = new Date(bits[0], bits[1] - 1, bits[2]);
      var secondBits = bits[5].split('.');
      d.setHours(bits[3], bits[4], secondBits[0]);
      if (secondBits.length > 1)
        d.setMilliseconds(secondBits[1]);
      // Get supplied time zone offset in minutes
      if (bits[6] && bits[7]) {
        var offsetMinutes = bits[6] * 60 + Number(bits[7]);
        var sign = /\d\d-\d\d:\d\d$/.test(prop) ? '-' : '+';
        // Apply the sign
        offsetMinutes = 0 + (sign == '-' ? -1 * offsetMinutes : offsetMinutes);
        // Apply offset and local timezone
        d.setMinutes(d.getMinutes() - offsetMinutes - d.getTimezoneOffset());
      } else if (prop.indexOf('Z', prop.length - 1) !== -1) {
        d = new Date(Date.UTC(d.getFullYear(), d.getMonth(), d.getDate(), d.getHours(), d.getMinutes(), d.getSeconds(), d.getMilliseconds()));
      }
      // d is now a local time equivalent to the supplied time
      return d;
    }
    function checkFromXmlDateTimePaths(value, childName, fullPath) {
      if (config.datetimeAccessFormPaths.length > 0) {
        var path = fullPath.split('.#')[0];
        if (checkInStdFiltersArrayForm(config.datetimeAccessFormPaths, value, childName, path)) {
          return fromXmlDateTime(value);
        } else
          return value;
      } else
        return value;
    }
    function checkXmlElementsFilter(obj, childType, childName, childPath) {
      if (childType == DOMNodeTypes.ELEMENT_NODE && config.xmlElementsFilter.length > 0) {
        return checkInStdFiltersArrayForm(config.xmlElementsFilter, obj, childName, childPath);
      } else
        return true;
    }
    function parseDOMChildren(node, path) {
      if (node.nodeType == DOMNodeTypes.DOCUMENT_NODE) {
        var result = new Object();
        var nodeChildren = node.childNodes;
        // Alternative for firstElementChild which is not supported in some environments
        for (var cidx = 0; cidx < nodeChildren.length; cidx++) {
          var child = nodeChildren.item(cidx);
          if (child.nodeType == DOMNodeTypes.ELEMENT_NODE) {
            var childName = getNodeLocalName(child);
            result[childName] = parseDOMChildren(child, childName);
          }
        }
        return result;
      } else if (node.nodeType == DOMNodeTypes.ELEMENT_NODE) {
        var result = new Object();
        result.__cnt = 0;
        var nodeChildren = node.childNodes;
        // Children nodes
        for (var cidx = 0; cidx < nodeChildren.length; cidx++) {
          var child = nodeChildren.item(cidx);
          // nodeChildren[cidx];
          var childName = getNodeLocalName(child);
          if (child.nodeType != DOMNodeTypes.COMMENT_NODE) {
            var childPath = path + '.' + childName;
            if (checkXmlElementsFilter(result, child.nodeType, childName, childPath)) {
              result.__cnt++;
              if (result[childName] == null) {
                result[childName] = parseDOMChildren(child, childPath);
                toArrayAccessForm(result, childName, childPath);
              } else {
                if (result[childName] != null) {
                  if (!(result[childName] instanceof Array)) {
                    result[childName] = [result[childName]];
                    toArrayAccessForm(result, childName, childPath);
                  }
                }
                result[childName][result[childName].length] = parseDOMChildren(child, childPath);
              }
            }
          }
        }
        // Attributes
        for (var aidx = 0; aidx < node.attributes.length; aidx++) {
          var attr = node.attributes.item(aidx);
          // [aidx];
          result.__cnt++;
          result[config.attributePrefix + attr.name] = attr.value;
        }
        // Node namespace prefix
        var nodePrefix = getNodePrefix(node);
        if (nodePrefix != null && nodePrefix != '') {
          result.__cnt++;
          result.__prefix = nodePrefix;
        }
        if (result['#text'] != null) {
          result.__text = result['#text'];
          if (result.__text instanceof Array) {
            result.__text = result.__text.join('\n');
          }
          //if(config.escapeMode)
          //	result.__text = unescapeXmlChars(result.__text);
          if (config.stripWhitespaces)
            result.__text = result.__text.trim();
          delete result['#text'];
          if (config.arrayAccessForm == 'property')
            delete result['#text_asArray'];
          result.__text = checkFromXmlDateTimePaths(result.__text, childName, path + '.' + childName);
        }
        if (result['#cdata-section'] != null) {
          result.__cdata = result['#cdata-section'];
          delete result['#cdata-section'];
          if (config.arrayAccessForm == 'property')
            delete result['#cdata-section_asArray'];
        }
        if (result.__cnt == 0 && config.emptyNodeForm == 'text') {
          result = '';
        } else if (result.__cnt == 1 && result.__text != null) {
          result = result.__text;
        } else if (result.__cnt == 1 && result.__cdata != null && !config.keepCData) {
          result = result.__cdata;
        } else if (result.__cnt > 1 && result.__text != null && config.skipEmptyTextNodesForObj) {
          if (config.stripWhitespaces && result.__text == '' || result.__text.trim() == '') {
            delete result.__text;
          }
        }
        delete result.__cnt;
        if (config.enableToStringFunc && (result.__text != null || result.__cdata != null)) {
          result.toString = function () {
            return (this.__text != null ? this.__text : '') + (this.__cdata != null ? this.__cdata : '');
          };
        }
        return result;
      } else if (node.nodeType == DOMNodeTypes.TEXT_NODE || node.nodeType == DOMNodeTypes.CDATA_SECTION_NODE) {
        return node.nodeValue;
      }
    }
    function startTag(jsonObj, element, attrList, closed) {
      var resultStr = '<' + (jsonObj != null && jsonObj.__prefix != null ? jsonObj.__prefix + ':' : '') + element;
      if (attrList != null) {
        for (var aidx = 0; aidx < attrList.length; aidx++) {
          var attrName = attrList[aidx];
          var attrVal = jsonObj[attrName];
          if (config.escapeMode)
            attrVal = escapeXmlChars(attrVal);
          resultStr += ' ' + attrName.substr(config.attributePrefix.length) + '=';
          if (config.useDoubleQuotes)
            resultStr += '"' + attrVal + '"';
          else
            resultStr += '\'' + attrVal + '\'';
        }
      }
      if (!closed)
        resultStr += '>';
      else
        resultStr += '/>';
      return resultStr;
    }
    function endTag(jsonObj, elementName) {
      return '</' + (jsonObj.__prefix != null ? jsonObj.__prefix + ':' : '') + elementName + '>';
    }
    function endsWith(str, suffix) {
      return str.indexOf(suffix, str.length - suffix.length) !== -1;
    }
    function jsonXmlSpecialElem(jsonObj, jsonObjField) {
      if (config.arrayAccessForm == 'property' && endsWith(jsonObjField.toString(), '_asArray') || jsonObjField.toString().indexOf(config.attributePrefix) == 0 || jsonObjField.toString().indexOf('__') == 0 || jsonObj[jsonObjField] instanceof Function)
        return true;
      else
        return false;
    }
    function jsonXmlElemCount(jsonObj) {
      var elementsCnt = 0;
      if (jsonObj instanceof Object) {
        for (var it in jsonObj) {
          if (jsonXmlSpecialElem(jsonObj, it))
            continue;
          elementsCnt++;
        }
      }
      return elementsCnt;
    }
    function checkJsonObjPropertiesFilter(jsonObj, propertyName, jsonObjPath) {
      return config.jsonPropertiesFilter.length == 0 || jsonObjPath == '' || checkInStdFiltersArrayForm(config.jsonPropertiesFilter, jsonObj, propertyName, jsonObjPath);
    }
    function parseJSONAttributes(jsonObj) {
      var attrList = [];
      if (jsonObj instanceof Object) {
        for (var ait in jsonObj) {
          if (ait.toString().indexOf('__') == -1 && ait.toString().indexOf(config.attributePrefix) == 0) {
            attrList.push(ait);
          }
        }
      }
      return attrList;
    }
    function parseJSONTextAttrs(jsonTxtObj) {
      var result = '';
      if (jsonTxtObj.__cdata != null) {
        result += '<![CDATA[' + jsonTxtObj.__cdata + ']]>';
      }
      if (jsonTxtObj.__text != null) {
        if (config.escapeMode)
          result += escapeXmlChars(jsonTxtObj.__text);
        else
          result += jsonTxtObj.__text;
      }
      return result;
    }
    function parseJSONTextObject(jsonTxtObj) {
      var result = '';
      if (jsonTxtObj instanceof Object) {
        result += parseJSONTextAttrs(jsonTxtObj);
      } else if (jsonTxtObj != null) {
        if (config.escapeMode)
          result += escapeXmlChars(jsonTxtObj);
        else
          result += jsonTxtObj;
      }
      return result;
    }
    function getJsonPropertyPath(jsonObjPath, jsonPropName) {
      if (jsonObjPath === '') {
        return jsonPropName;
      } else
        return jsonObjPath + '.' + jsonPropName;
    }
    function parseJSONArray(jsonArrRoot, jsonArrObj, attrList, jsonObjPath) {
      var result = '';
      if (jsonArrRoot.length == 0) {
        result += startTag(jsonArrRoot, jsonArrObj, attrList, true);
      } else {
        for (var arIdx = 0; arIdx < jsonArrRoot.length; arIdx++) {
          result += startTag(jsonArrRoot[arIdx], jsonArrObj, parseJSONAttributes(jsonArrRoot[arIdx]), false);
          result += parseJSONObject(jsonArrRoot[arIdx], getJsonPropertyPath(jsonObjPath, jsonArrObj));
          result += endTag(jsonArrRoot[arIdx], jsonArrObj);
        }
      }
      return result;
    }
    function parseJSONObject(jsonObj, jsonObjPath) {
      var result = '';
      var elementsCnt = jsonXmlElemCount(jsonObj);
      if (elementsCnt > 0) {
        for (var it in jsonObj) {
          if (jsonXmlSpecialElem(jsonObj, it) || jsonObjPath != '' && !checkJsonObjPropertiesFilter(jsonObj, it, getJsonPropertyPath(jsonObjPath, it)))
            continue;
          var subObj = jsonObj[it];
          var attrList = parseJSONAttributes(subObj);
          if (subObj == null || subObj == undefined) {
            result += startTag(subObj, it, attrList, true);
          } else if (subObj instanceof Object) {
            if (subObj instanceof Array) {
              result += parseJSONArray(subObj, it, attrList, jsonObjPath);
            } else if (subObj instanceof Date) {
              result += startTag(subObj, it, attrList, false);
              result += subObj.toISOString();
              result += endTag(subObj, it);
            } else {
              var subObjElementsCnt = jsonXmlElemCount(subObj);
              if (subObjElementsCnt > 0 || subObj.__text != null || subObj.__cdata != null) {
                result += startTag(subObj, it, attrList, false);
                result += parseJSONObject(subObj, getJsonPropertyPath(jsonObjPath, it));
                result += endTag(subObj, it);
              } else {
                result += startTag(subObj, it, attrList, true);
              }
            }
          } else {
            result += startTag(subObj, it, attrList, false);
            result += parseJSONTextObject(subObj);
            result += endTag(subObj, it);
          }
        }
      }
      result += parseJSONTextObject(jsonObj);
      return result;
    }
    this.parseXmlString = function (xmlDocStr) {
      var isIEParser = window.ActiveXObject || 'ActiveXObject' in window;
      if (xmlDocStr === undefined) {
        return null;
      }
      var xmlDoc;
      if (window.DOMParser) {
        var parser = new window.DOMParser();
        var parsererrorNS = null;
        // IE9+ now is here
        if (!isIEParser) {
          try {
            parsererrorNS = parser.parseFromString('INVALID', 'text/xml').getElementsByTagName('parsererror')[0].namespaceURI;
          } catch (err) {
            parsererrorNS = null;
          }
        }
        try {
          xmlDoc = parser.parseFromString(xmlDocStr, 'text/xml');
          if (parsererrorNS != null && xmlDoc.getElementsByTagNameNS(parsererrorNS, 'parsererror').length > 0) {
            //throw new Error('Error parsing XML: '+xmlDocStr);
            xmlDoc = null;
          }
        } catch (err) {
          xmlDoc = null;
        }
      } else {
        // IE :(
        if (xmlDocStr.indexOf('<?') == 0) {
          xmlDocStr = xmlDocStr.substr(xmlDocStr.indexOf('?>') + 2);
        }
        xmlDoc = new ActiveXObject('Microsoft.XMLDOM');
        xmlDoc.async = 'false';
        xmlDoc.loadXML(xmlDocStr);
      }
      return xmlDoc;
    };
    this.asArray = function (prop) {
      if (prop === undefined || prop == null)
        return [];
      else if (prop instanceof Array)
        return prop;
      else
        return [prop];
    };
    this.toXmlDateTime = function (dt) {
      if (dt instanceof Date)
        return dt.toISOString();
      else if (typeof dt === 'number')
        return new Date(dt).toISOString();
      else
        return null;
    };
    this.asDateTime = function (prop) {
      if (typeof prop == 'string') {
        return fromXmlDateTime(prop);
      } else
        return prop;
    };
    this.xml2json = function (xmlDoc) {
      return parseDOMChildren(xmlDoc);
    };
    this.xml_str2json = function (xmlDocStr) {
      var xmlDoc = this.parseXmlString(xmlDocStr);
      if (xmlDoc != null)
        return this.xml2json(xmlDoc);
      else
        return null;
    };
    this.json2xml_str = function (jsonObj) {
      return parseJSONObject(jsonObj, '');
    };
    this.json2xml = function (jsonObj) {
      var xmlDocStr = this.json2xml_str(jsonObj);
      return this.parseXmlString(xmlDocStr);
    };
    this.getVersion = function () {
      return VERSION;
    };
  };
}));
/*!
 * Signature Pad v1.3.2
 * https://github.com/szimek/signature_pad
 *
 * Copyright 2013 Szymon Nowak
 * Released under the MIT license
 *
 * The main idea and some parts of the code (e.g. drawing variable width Bézier curve) are taken from:
 * http://corner.squareup.com/2012/07/smoother-signatures.html
 *
 * Implementation of interpolation using cubic Bézier curves is taken from:
 * http://benknowscode.wordpress.com/2012/09/14/path-interpolation-using-cubic-bezier-and-control-point-estimation-in-javascript
 *
 * Algorithm for approximated length of a Bézier curve is taken from:
 * http://www.lemoda.net/maths/bezier-length/index.html
 *
 */
var SignaturePad = function (document) {
    'use strict';
    var SignaturePad = function (canvas, options) {
      var self = this, opts = options || {};
      this.velocityFilterWeight = opts.velocityFilterWeight || 0.7;
      this.minWidth = opts.minWidth || 0.5;
      this.maxWidth = opts.maxWidth || 2.5;
      this.dotSize = opts.dotSize || function () {
        return (this.minWidth + this.maxWidth) / 2;
      };
      this.penColor = opts.penColor || 'black';
      this.backgroundColor = opts.backgroundColor || 'rgba(0,0,0,0)';
      this.onEnd = opts.onEnd;
      this.onBegin = opts.onBegin;
      this._canvas = canvas;
      this._ctx = canvas.getContext('2d');
      this.clear();
      this._handleMouseEvents();
      this._handleTouchEvents();
    };
    SignaturePad.prototype.clear = function () {
      var ctx = this._ctx, canvas = this._canvas;
      ctx.fillStyle = this.backgroundColor;
      ctx.clearRect(0, 0, canvas.width, canvas.height);
      ctx.fillRect(0, 0, canvas.width, canvas.height);
      this._reset();
    };
    SignaturePad.prototype.toDataURL = function (imageType, quality) {
      var canvas = this._canvas;
      return canvas.toDataURL.apply(canvas, arguments);
    };
    SignaturePad.prototype.fromDataURL = function (dataUrl) {
      var self = this, image = new Image();
      this._reset();
      image.src = dataUrl;
      image.onload = function () {
        self._ctx.drawImage(image, 0, 0, self._canvas.width, self._canvas.height);
      };
      this._isEmpty = false;
    };
    SignaturePad.prototype._strokeUpdate = function (event) {
      var point = this._createPoint(event);
      this._addPoint(point);
    };
    SignaturePad.prototype._strokeBegin = function (event) {
      this._reset();
      this._strokeUpdate(event);
      if (typeof this.onBegin === 'function') {
        this.onBegin(event);
      }
    };
    SignaturePad.prototype._strokeDraw = function (point) {
      var ctx = this._ctx, dotSize = typeof this.dotSize === 'function' ? this.dotSize() : this.dotSize;
      ctx.beginPath();
      this._drawPoint(point.x, point.y, dotSize);
      ctx.closePath();
      ctx.fill();
    };
    SignaturePad.prototype._strokeEnd = function (event) {
      var canDrawCurve = this.points.length > 2, point = this.points[0];
      if (!canDrawCurve && point) {
        this._strokeDraw(point);
      }
      if (typeof this.onEnd === 'function') {
        this.onEnd(event);
      }
    };
    SignaturePad.prototype._handleMouseEvents = function () {
      var self = this;
      this._mouseButtonDown = false;
      this._canvas.addEventListener('mousedown', function (event) {
        if (event.which === 1) {
          self._mouseButtonDown = true;
          self._strokeBegin(event);
        }
      });
      this._canvas.addEventListener('mousemove', function (event) {
        if (self._mouseButtonDown) {
          self._strokeUpdate(event);
        }
      });
      document.addEventListener('mouseup', function (event) {
        if (event.which === 1 && self._mouseButtonDown) {
          self._mouseButtonDown = false;
          self._strokeEnd(event);
        }
      });
    };
    SignaturePad.prototype._handleTouchEvents = function () {
      var self = this;
      // Pass touch events to canvas element on mobile IE.
      this._canvas.style.msTouchAction = 'none';
      this._canvas.addEventListener('touchstart', function (event) {
        var touch = event.changedTouches[0];
        self._strokeBegin(touch);
      });
      this._canvas.addEventListener('touchmove', function (event) {
        // Prevent scrolling.
        event.preventDefault();
        var touch = event.changedTouches[0];
        self._strokeUpdate(touch);
      });
      document.addEventListener('touchend', function (event) {
        var wasCanvasTouched = event.target === self._canvas;
        if (wasCanvasTouched) {
          self._strokeEnd(event);
        }
      });
    };
    SignaturePad.prototype.isEmpty = function () {
      return this._isEmpty;
    };
    SignaturePad.prototype._reset = function () {
      this.points = [];
      this._lastVelocity = 0;
      this._lastWidth = (this.minWidth + this.maxWidth) / 2;
      this._isEmpty = true;
      this._ctx.fillStyle = this.penColor;
    };
    SignaturePad.prototype._createPoint = function (event) {
      var rect = this._canvas.getBoundingClientRect();
      return new Point(event.clientX - rect.left, event.clientY - rect.top);
    };
    SignaturePad.prototype._addPoint = function (point) {
      var points = this.points, c2, c3, curve, tmp;
      points.push(point);
      if (points.length > 2) {
        // To reduce the initial lag make it work with 3 points
        // by copying the first point to the beginning.
        if (points.length === 3)
          points.unshift(points[0]);
        tmp = this._calculateCurveControlPoints(points[0], points[1], points[2]);
        c2 = tmp.c2;
        tmp = this._calculateCurveControlPoints(points[1], points[2], points[3]);
        c3 = tmp.c1;
        curve = new Bezier(points[1], c2, c3, points[2]);
        this._addCurve(curve);
        // Remove the first element from the list,
        // so that we always have no more than 4 points in points array.
        points.shift();
      }
    };
    SignaturePad.prototype._calculateCurveControlPoints = function (s1, s2, s3) {
      var dx1 = s1.x - s2.x, dy1 = s1.y - s2.y, dx2 = s2.x - s3.x, dy2 = s2.y - s3.y, m1 = {
          x: (s1.x + s2.x) / 2,
          y: (s1.y + s2.y) / 2
        }, m2 = {
          x: (s2.x + s3.x) / 2,
          y: (s2.y + s3.y) / 2
        }, l1 = Math.sqrt(dx1 * dx1 + dy1 * dy1), l2 = Math.sqrt(dx2 * dx2 + dy2 * dy2), dxm = m1.x - m2.x, dym = m1.y - m2.y, k = l2 / (l1 + l2), cm = {
          x: m2.x + dxm * k,
          y: m2.y + dym * k
        }, tx = s2.x - cm.x, ty = s2.y - cm.y;
      return {
        c1: new Point(m1.x + tx, m1.y + ty),
        c2: new Point(m2.x + tx, m2.y + ty)
      };
    };
    SignaturePad.prototype._addCurve = function (curve) {
      var startPoint = curve.startPoint, endPoint = curve.endPoint, velocity, newWidth;
      velocity = endPoint.velocityFrom(startPoint);
      velocity = this.velocityFilterWeight * velocity + (1 - this.velocityFilterWeight) * this._lastVelocity;
      newWidth = this._strokeWidth(velocity);
      this._drawCurve(curve, this._lastWidth, newWidth);
      this._lastVelocity = velocity;
      this._lastWidth = newWidth;
    };
    SignaturePad.prototype._drawPoint = function (x, y, size) {
      var ctx = this._ctx;
      ctx.moveTo(x, y);
      ctx.arc(x, y, size, 0, 2 * Math.PI, false);
      this._isEmpty = false;
    };
    SignaturePad.prototype._drawCurve = function (curve, startWidth, endWidth) {
      var ctx = this._ctx, widthDelta = endWidth - startWidth, drawSteps, width, i, t, tt, ttt, u, uu, uuu, x, y;
      drawSteps = Math.floor(curve.length());
      ctx.beginPath();
      for (i = 0; i < drawSteps; i++) {
        // Calculate the Bezier (x, y) coordinate for this step.
        t = i / drawSteps;
        tt = t * t;
        ttt = tt * t;
        u = 1 - t;
        uu = u * u;
        uuu = uu * u;
        x = uuu * curve.startPoint.x;
        x += 3 * uu * t * curve.control1.x;
        x += 3 * u * tt * curve.control2.x;
        x += ttt * curve.endPoint.x;
        y = uuu * curve.startPoint.y;
        y += 3 * uu * t * curve.control1.y;
        y += 3 * u * tt * curve.control2.y;
        y += ttt * curve.endPoint.y;
        width = startWidth + ttt * widthDelta;
        this._drawPoint(x, y, width);
      }
      ctx.closePath();
      ctx.fill();
    };
    SignaturePad.prototype._strokeWidth = function (velocity) {
      return Math.max(this.maxWidth / (velocity + 1), this.minWidth);
    };
    var Point = function (x, y, time) {
      this.x = x;
      this.y = y;
      this.time = time || new Date().getTime();
    };
    Point.prototype.velocityFrom = function (start) {
      return this.time !== start.time ? this.distanceTo(start) / (this.time - start.time) : 1;
    };
    Point.prototype.distanceTo = function (start) {
      return Math.sqrt(Math.pow(this.x - start.x, 2) + Math.pow(this.y - start.y, 2));
    };
    var Bezier = function (startPoint, control1, control2, endPoint) {
      this.startPoint = startPoint;
      this.control1 = control1;
      this.control2 = control2;
      this.endPoint = endPoint;
    };
    // Returns approximated length.
    Bezier.prototype.length = function () {
      var steps = 10, length = 0, i, t, cx, cy, px, py, xdiff, ydiff;
      for (i = 0; i <= steps; i++) {
        t = i / steps;
        cx = this._point(t, this.startPoint.x, this.control1.x, this.control2.x, this.endPoint.x);
        cy = this._point(t, this.startPoint.y, this.control1.y, this.control2.y, this.endPoint.y);
        if (i > 0) {
          xdiff = cx - px;
          ydiff = cy - py;
          length += Math.sqrt(xdiff * xdiff + ydiff * ydiff);
        }
        px = cx;
        py = cy;
      }
      return length;
    };
    Bezier.prototype._point = function (t, start, c1, c2, end) {
      return start * (1 - t) * (1 - t) * (1 - t) + 3 * c1 * (1 - t) * (1 - t) * t + 3 * c2 * (1 - t) * t * t + end * t * t * t;
    };
    return SignaturePad;
  }(document);
/*! Hammer.JS - v2.0.7 - 2016-04-22
 * http://hammerjs.github.io/
 *
 * Copyright (c) 2016 Jorik Tangelder;
 * Licensed under the MIT license */
(function (window, document, exportName, undefined) {
  'use strict';
  var VENDOR_PREFIXES = [
      '',
      'webkit',
      'Moz',
      'MS',
      'ms',
      'o'
    ];
  var TEST_ELEMENT = document.createElement('div');
  var TYPE_FUNCTION = 'function';
  var round = Math.round;
  var abs = Math.abs;
  var now = Date.now;
  /**
 * set a timeout with a given scope
 * @param {Function} fn
 * @param {Number} timeout
 * @param {Object} context
 * @returns {number}
 */
  function setTimeoutContext(fn, timeout, context) {
    return setTimeout(bindFn(fn, context), timeout);
  }
  /**
 * if the argument is an array, we want to execute the fn on each entry
 * if it aint an array we don't want to do a thing.
 * this is used by all the methods that accept a single and array argument.
 * @param {*|Array} arg
 * @param {String} fn
 * @param {Object} [context]
 * @returns {Boolean}
 */
  function invokeArrayArg(arg, fn, context) {
    if (Array.isArray(arg)) {
      each(arg, context[fn], context);
      return true;
    }
    return false;
  }
  /**
 * walk objects and arrays
 * @param {Object} obj
 * @param {Function} iterator
 * @param {Object} context
 */
  function each(obj, iterator, context) {
    var i;
    if (!obj) {
      return;
    }
    if (obj.forEach) {
      obj.forEach(iterator, context);
    } else if (obj.length !== undefined) {
      i = 0;
      while (i < obj.length) {
        iterator.call(context, obj[i], i, obj);
        i++;
      }
    } else {
      for (i in obj) {
        obj.hasOwnProperty(i) && iterator.call(context, obj[i], i, obj);
      }
    }
  }
  /**
 * wrap a method with a deprecation warning and stack trace
 * @param {Function} method
 * @param {String} name
 * @param {String} message
 * @returns {Function} A new function wrapping the supplied method.
 */
  function deprecate(method, name, message) {
    var deprecationMessage = 'DEPRECATED METHOD: ' + name + '\n' + message + ' AT \n';
    return function () {
      var e = new Error('get-stack-trace');
      var stack = e && e.stack ? e.stack.replace(/^[^\(]+?[\n$]/gm, '').replace(/^\s+at\s+/gm, '').replace(/^Object.<anonymous>\s*\(/gm, '{anonymous}()@') : 'Unknown Stack Trace';
      var log = window.console && (window.console.warn || window.console.log);
      if (log) {
        log.call(window.console, deprecationMessage, stack);
      }
      return method.apply(this, arguments);
    };
  }
  /**
 * extend object.
 * means that properties in dest will be overwritten by the ones in src.
 * @param {Object} target
 * @param {...Object} objects_to_assign
 * @returns {Object} target
 */
  var assign;
  if (typeof Object.assign !== 'function') {
    assign = function assign(target) {
      if (target === undefined || target === null) {
        throw new TypeError('Cannot convert undefined or null to object');
      }
      var output = Object(target);
      for (var index = 1; index < arguments.length; index++) {
        var source = arguments[index];
        if (source !== undefined && source !== null) {
          for (var nextKey in source) {
            if (source.hasOwnProperty(nextKey)) {
              output[nextKey] = source[nextKey];
            }
          }
        }
      }
      return output;
    };
  } else {
    assign = Object.assign;
  }
  /**
 * extend object.
 * means that properties in dest will be overwritten by the ones in src.
 * @param {Object} dest
 * @param {Object} src
 * @param {Boolean} [merge=false]
 * @returns {Object} dest
 */
  var extend = deprecate(function extend(dest, src, merge) {
      var keys = Object.keys(src);
      var i = 0;
      while (i < keys.length) {
        if (!merge || merge && dest[keys[i]] === undefined) {
          dest[keys[i]] = src[keys[i]];
        }
        i++;
      }
      return dest;
    }, 'extend', 'Use `assign`.');
  /**
 * merge the values from src in the dest.
 * means that properties that exist in dest will not be overwritten by src
 * @param {Object} dest
 * @param {Object} src
 * @returns {Object} dest
 */
  var merge = deprecate(function merge(dest, src) {
      return extend(dest, src, true);
    }, 'merge', 'Use `assign`.');
  /**
 * simple class inheritance
 * @param {Function} child
 * @param {Function} base
 * @param {Object} [properties]
 */
  function inherit(child, base, properties) {
    var baseP = base.prototype, childP;
    childP = child.prototype = Object.create(baseP);
    childP.constructor = child;
    childP._super = baseP;
    if (properties) {
      assign(childP, properties);
    }
  }
  /**
 * simple function bind
 * @param {Function} fn
 * @param {Object} context
 * @returns {Function}
 */
  function bindFn(fn, context) {
    return function boundFn() {
      return fn.apply(context, arguments);
    };
  }
  /**
 * let a boolean value also be a function that must return a boolean
 * this first item in args will be used as the context
 * @param {Boolean|Function} val
 * @param {Array} [args]
 * @returns {Boolean}
 */
  function boolOrFn(val, args) {
    if (typeof val == TYPE_FUNCTION) {
      return val.apply(args ? args[0] || undefined : undefined, args);
    }
    return val;
  }
  /**
 * use the val2 when val1 is undefined
 * @param {*} val1
 * @param {*} val2
 * @returns {*}
 */
  function ifUndefined(val1, val2) {
    return val1 === undefined ? val2 : val1;
  }
  /**
 * addEventListener with multiple events at once
 * @param {EventTarget} target
 * @param {String} types
 * @param {Function} handler
 */
  function addEventListeners(target, types, handler) {
    each(splitStr(types), function (type) {
      target.addEventListener(type, handler, false);
    });
  }
  /**
 * removeEventListener with multiple events at once
 * @param {EventTarget} target
 * @param {String} types
 * @param {Function} handler
 */
  function removeEventListeners(target, types, handler) {
    each(splitStr(types), function (type) {
      target.removeEventListener(type, handler, false);
    });
  }
  /**
 * find if a node is in the given parent
 * @method hasParent
 * @param {HTMLElement} node
 * @param {HTMLElement} parent
 * @return {Boolean} found
 */
  function hasParent(node, parent) {
    while (node) {
      if (node == parent) {
        return true;
      }
      node = node.parentNode;
    }
    return false;
  }
  /**
 * small indexOf wrapper
 * @param {String} str
 * @param {String} find
 * @returns {Boolean} found
 */
  function inStr(str, find) {
    return str.indexOf(find) > -1;
  }
  /**
 * split string on whitespace
 * @param {String} str
 * @returns {Array} words
 */
  function splitStr(str) {
    return str.trim().split(/\s+/g);
  }
  /**
 * find if a array contains the object using indexOf or a simple polyFill
 * @param {Array} src
 * @param {String} find
 * @param {String} [findByKey]
 * @return {Boolean|Number} false when not found, or the index
 */
  function inArray(src, find, findByKey) {
    if (src.indexOf && !findByKey) {
      return src.indexOf(find);
    } else {
      var i = 0;
      while (i < src.length) {
        if (findByKey && src[i][findByKey] == find || !findByKey && src[i] === find) {
          return i;
        }
        i++;
      }
      return -1;
    }
  }
  /**
 * convert array-like objects to real arrays
 * @param {Object} obj
 * @returns {Array}
 */
  function toArray(obj) {
    return Array.prototype.slice.call(obj, 0);
  }
  /**
 * unique array with objects based on a key (like 'id') or just by the array's value
 * @param {Array} src [{id:1},{id:2},{id:1}]
 * @param {String} [key]
 * @param {Boolean} [sort=False]
 * @returns {Array} [{id:1},{id:2}]
 */
  function uniqueArray(src, key, sort) {
    var results = [];
    var values = [];
    var i = 0;
    while (i < src.length) {
      var val = key ? src[i][key] : src[i];
      if (inArray(values, val) < 0) {
        results.push(src[i]);
      }
      values[i] = val;
      i++;
    }
    if (sort) {
      if (!key) {
        results = results.sort();
      } else {
        results = results.sort(function sortUniqueArray(a, b) {
          return a[key] > b[key];
        });
      }
    }
    return results;
  }
  /**
 * get the prefixed property
 * @param {Object} obj
 * @param {String} property
 * @returns {String|Undefined} prefixed
 */
  function prefixed(obj, property) {
    var prefix, prop;
    var camelProp = property[0].toUpperCase() + property.slice(1);
    var i = 0;
    while (i < VENDOR_PREFIXES.length) {
      prefix = VENDOR_PREFIXES[i];
      prop = prefix ? prefix + camelProp : property;
      if (prop in obj) {
        return prop;
      }
      i++;
    }
    return undefined;
  }
  /**
 * get a unique id
 * @returns {number} uniqueId
 */
  var _uniqueId = 1;
  function uniqueId() {
    return _uniqueId++;
  }
  /**
 * get the window object of an element
 * @param {HTMLElement} element
 * @returns {DocumentView|Window}
 */
  function getWindowForElement(element) {
    var doc = element.ownerDocument || element;
    return doc.defaultView || doc.parentWindow || window;
  }
  var MOBILE_REGEX = /mobile|tablet|ip(ad|hone|od)|android/i;
  var SUPPORT_TOUCH = 'ontouchstart' in window;
  var SUPPORT_POINTER_EVENTS = prefixed(window, 'PointerEvent') !== undefined;
  var SUPPORT_ONLY_TOUCH = SUPPORT_TOUCH && MOBILE_REGEX.test(navigator.userAgent);
  var INPUT_TYPE_TOUCH = 'touch';
  var INPUT_TYPE_PEN = 'pen';
  var INPUT_TYPE_MOUSE = 'mouse';
  var INPUT_TYPE_KINECT = 'kinect';
  var COMPUTE_INTERVAL = 25;
  var INPUT_START = 1;
  var INPUT_MOVE = 2;
  var INPUT_END = 4;
  var INPUT_CANCEL = 8;
  var DIRECTION_NONE = 1;
  var DIRECTION_LEFT = 2;
  var DIRECTION_RIGHT = 4;
  var DIRECTION_UP = 8;
  var DIRECTION_DOWN = 16;
  var DIRECTION_HORIZONTAL = DIRECTION_LEFT | DIRECTION_RIGHT;
  var DIRECTION_VERTICAL = DIRECTION_UP | DIRECTION_DOWN;
  var DIRECTION_ALL = DIRECTION_HORIZONTAL | DIRECTION_VERTICAL;
  var PROPS_XY = [
      'x',
      'y'
    ];
  var PROPS_CLIENT_XY = [
      'clientX',
      'clientY'
    ];
  /**
 * create new input type manager
 * @param {Manager} manager
 * @param {Function} callback
 * @returns {Input}
 * @constructor
 */
  function Input(manager, callback) {
    var self = this;
    this.manager = manager;
    this.callback = callback;
    this.element = manager.element;
    this.target = manager.options.inputTarget;
    // smaller wrapper around the handler, for the scope and the enabled state of the manager,
    // so when disabled the input events are completely bypassed.
    this.domHandler = function (ev) {
      if (boolOrFn(manager.options.enable, [manager])) {
        self.handler(ev);
      }
    };
    this.init();
  }
  Input.prototype = {
    handler: function () {
    },
    init: function () {
      this.evEl && addEventListeners(this.element, this.evEl, this.domHandler);
      this.evTarget && addEventListeners(this.target, this.evTarget, this.domHandler);
      this.evWin && addEventListeners(getWindowForElement(this.element), this.evWin, this.domHandler);
    },
    destroy: function () {
      this.evEl && removeEventListeners(this.element, this.evEl, this.domHandler);
      this.evTarget && removeEventListeners(this.target, this.evTarget, this.domHandler);
      this.evWin && removeEventListeners(getWindowForElement(this.element), this.evWin, this.domHandler);
    }
  };
  /**
 * create new input type manager
 * called by the Manager constructor
 * @param {Hammer} manager
 * @returns {Input}
 */
  function createInputInstance(manager) {
    var Type;
    var inputClass = manager.options.inputClass;
    if (inputClass) {
      Type = inputClass;
    } else if (SUPPORT_POINTER_EVENTS) {
      Type = PointerEventInput;
    } else if (SUPPORT_ONLY_TOUCH) {
      Type = TouchInput;
    } else if (!SUPPORT_TOUCH) {
      Type = MouseInput;
    } else {
      Type = TouchMouseInput;
    }
    return new Type(manager, inputHandler);
  }
  /**
 * handle input events
 * @param {Manager} manager
 * @param {String} eventType
 * @param {Object} input
 */
  function inputHandler(manager, eventType, input) {
    var pointersLen = input.pointers.length;
    var changedPointersLen = input.changedPointers.length;
    var isFirst = eventType & INPUT_START && pointersLen - changedPointersLen === 0;
    var isFinal = eventType & (INPUT_END | INPUT_CANCEL) && pointersLen - changedPointersLen === 0;
    input.isFirst = !!isFirst;
    input.isFinal = !!isFinal;
    if (isFirst) {
      manager.session = {};
    }
    // source event is the normalized value of the domEvents
    // like 'touchstart, mouseup, pointerdown'
    input.eventType = eventType;
    // compute scale, rotation etc
    computeInputData(manager, input);
    // emit secret event
    manager.emit('hammer.input', input);
    manager.recognize(input);
    manager.session.prevInput = input;
  }
  /**
 * extend the data with some usable properties like scale, rotate, velocity etc
 * @param {Object} manager
 * @param {Object} input
 */
  function computeInputData(manager, input) {
    var session = manager.session;
    var pointers = input.pointers;
    var pointersLength = pointers.length;
    // store the first input to calculate the distance and direction
    if (!session.firstInput) {
      session.firstInput = simpleCloneInputData(input);
    }
    // to compute scale and rotation we need to store the multiple touches
    if (pointersLength > 1 && !session.firstMultiple) {
      session.firstMultiple = simpleCloneInputData(input);
    } else if (pointersLength === 1) {
      session.firstMultiple = false;
    }
    var firstInput = session.firstInput;
    var firstMultiple = session.firstMultiple;
    var offsetCenter = firstMultiple ? firstMultiple.center : firstInput.center;
    var center = input.center = getCenter(pointers);
    input.timeStamp = now();
    input.deltaTime = input.timeStamp - firstInput.timeStamp;
    input.angle = getAngle(offsetCenter, center);
    input.distance = getDistance(offsetCenter, center);
    computeDeltaXY(session, input);
    input.offsetDirection = getDirection(input.deltaX, input.deltaY);
    var overallVelocity = getVelocity(input.deltaTime, input.deltaX, input.deltaY);
    input.overallVelocityX = overallVelocity.x;
    input.overallVelocityY = overallVelocity.y;
    input.overallVelocity = abs(overallVelocity.x) > abs(overallVelocity.y) ? overallVelocity.x : overallVelocity.y;
    input.scale = firstMultiple ? getScale(firstMultiple.pointers, pointers) : 1;
    input.rotation = firstMultiple ? getRotation(firstMultiple.pointers, pointers) : 0;
    input.maxPointers = !session.prevInput ? input.pointers.length : input.pointers.length > session.prevInput.maxPointers ? input.pointers.length : session.prevInput.maxPointers;
    computeIntervalInputData(session, input);
    // find the correct target
    var target = manager.element;
    if (hasParent(input.srcEvent.target, target)) {
      target = input.srcEvent.target;
    }
    input.target = target;
  }
  function computeDeltaXY(session, input) {
    var center = input.center;
    var offset = session.offsetDelta || {};
    var prevDelta = session.prevDelta || {};
    var prevInput = session.prevInput || {};
    if (input.eventType === INPUT_START || prevInput.eventType === INPUT_END) {
      prevDelta = session.prevDelta = {
        x: prevInput.deltaX || 0,
        y: prevInput.deltaY || 0
      };
      offset = session.offsetDelta = {
        x: center.x,
        y: center.y
      };
    }
    input.deltaX = prevDelta.x + (center.x - offset.x);
    input.deltaY = prevDelta.y + (center.y - offset.y);
  }
  /**
 * velocity is calculated every x ms
 * @param {Object} session
 * @param {Object} input
 */
  function computeIntervalInputData(session, input) {
    var last = session.lastInterval || input, deltaTime = input.timeStamp - last.timeStamp, velocity, velocityX, velocityY, direction;
    if (input.eventType != INPUT_CANCEL && (deltaTime > COMPUTE_INTERVAL || last.velocity === undefined)) {
      var deltaX = input.deltaX - last.deltaX;
      var deltaY = input.deltaY - last.deltaY;
      var v = getVelocity(deltaTime, deltaX, deltaY);
      velocityX = v.x;
      velocityY = v.y;
      velocity = abs(v.x) > abs(v.y) ? v.x : v.y;
      direction = getDirection(deltaX, deltaY);
      session.lastInterval = input;
    } else {
      // use latest velocity info if it doesn't overtake a minimum period
      velocity = last.velocity;
      velocityX = last.velocityX;
      velocityY = last.velocityY;
      direction = last.direction;
    }
    input.velocity = velocity;
    input.velocityX = velocityX;
    input.velocityY = velocityY;
    input.direction = direction;
  }
  /**
 * create a simple clone from the input used for storage of firstInput and firstMultiple
 * @param {Object} input
 * @returns {Object} clonedInputData
 */
  function simpleCloneInputData(input) {
    // make a simple copy of the pointers because we will get a reference if we don't
    // we only need clientXY for the calculations
    var pointers = [];
    var i = 0;
    while (i < input.pointers.length) {
      pointers[i] = {
        clientX: round(input.pointers[i].clientX),
        clientY: round(input.pointers[i].clientY)
      };
      i++;
    }
    return {
      timeStamp: now(),
      pointers: pointers,
      center: getCenter(pointers),
      deltaX: input.deltaX,
      deltaY: input.deltaY
    };
  }
  /**
 * get the center of all the pointers
 * @param {Array} pointers
 * @return {Object} center contains `x` and `y` properties
 */
  function getCenter(pointers) {
    var pointersLength = pointers.length;
    // no need to loop when only one touch
    if (pointersLength === 1) {
      return {
        x: round(pointers[0].clientX),
        y: round(pointers[0].clientY)
      };
    }
    var x = 0, y = 0, i = 0;
    while (i < pointersLength) {
      x += pointers[i].clientX;
      y += pointers[i].clientY;
      i++;
    }
    return {
      x: round(x / pointersLength),
      y: round(y / pointersLength)
    };
  }
  /**
 * calculate the velocity between two points. unit is in px per ms.
 * @param {Number} deltaTime
 * @param {Number} x
 * @param {Number} y
 * @return {Object} velocity `x` and `y`
 */
  function getVelocity(deltaTime, x, y) {
    return {
      x: x / deltaTime || 0,
      y: y / deltaTime || 0
    };
  }
  /**
 * get the direction between two points
 * @param {Number} x
 * @param {Number} y
 * @return {Number} direction
 */
  function getDirection(x, y) {
    if (x === y) {
      return DIRECTION_NONE;
    }
    if (abs(x) >= abs(y)) {
      return x < 0 ? DIRECTION_LEFT : DIRECTION_RIGHT;
    }
    return y < 0 ? DIRECTION_UP : DIRECTION_DOWN;
  }
  /**
 * calculate the absolute distance between two points
 * @param {Object} p1 {x, y}
 * @param {Object} p2 {x, y}
 * @param {Array} [props] containing x and y keys
 * @return {Number} distance
 */
  function getDistance(p1, p2, props) {
    if (!props) {
      props = PROPS_XY;
    }
    var x = p2[props[0]] - p1[props[0]], y = p2[props[1]] - p1[props[1]];
    return Math.sqrt(x * x + y * y);
  }
  /**
 * calculate the angle between two coordinates
 * @param {Object} p1
 * @param {Object} p2
 * @param {Array} [props] containing x and y keys
 * @return {Number} angle
 */
  function getAngle(p1, p2, props) {
    if (!props) {
      props = PROPS_XY;
    }
    var x = p2[props[0]] - p1[props[0]], y = p2[props[1]] - p1[props[1]];
    return Math.atan2(y, x) * 180 / Math.PI;
  }
  /**
 * calculate the rotation degrees between two pointersets
 * @param {Array} start array of pointers
 * @param {Array} end array of pointers
 * @return {Number} rotation
 */
  function getRotation(start, end) {
    return getAngle(end[1], end[0], PROPS_CLIENT_XY) + getAngle(start[1], start[0], PROPS_CLIENT_XY);
  }
  /**
 * calculate the scale factor between two pointersets
 * no scale is 1, and goes down to 0 when pinched together, and bigger when pinched out
 * @param {Array} start array of pointers
 * @param {Array} end array of pointers
 * @return {Number} scale
 */
  function getScale(start, end) {
    return getDistance(end[0], end[1], PROPS_CLIENT_XY) / getDistance(start[0], start[1], PROPS_CLIENT_XY);
  }
  var MOUSE_INPUT_MAP = {
      mousedown: INPUT_START,
      mousemove: INPUT_MOVE,
      mouseup: INPUT_END
    };
  var MOUSE_ELEMENT_EVENTS = 'mousedown';
  var MOUSE_WINDOW_EVENTS = 'mousemove mouseup';
  /**
 * Mouse events input
 * @constructor
 * @extends Input
 */
  function MouseInput() {
    this.evEl = MOUSE_ELEMENT_EVENTS;
    this.evWin = MOUSE_WINDOW_EVENTS;
    this.pressed = false;
    // mousedown state
    Input.apply(this, arguments);
  }
  inherit(MouseInput, Input, {
    handler: function MEhandler(ev) {
      var eventType = MOUSE_INPUT_MAP[ev.type];
      // on start we want to have the left mouse button down
      if (eventType & INPUT_START && ev.button === 0) {
        this.pressed = true;
      }
      if (eventType & INPUT_MOVE && ev.which !== 1) {
        eventType = INPUT_END;
      }
      // mouse must be down
      if (!this.pressed) {
        return;
      }
      if (eventType & INPUT_END) {
        this.pressed = false;
      }
      this.callback(this.manager, eventType, {
        pointers: [ev],
        changedPointers: [ev],
        pointerType: INPUT_TYPE_MOUSE,
        srcEvent: ev
      });
    }
  });
  var POINTER_INPUT_MAP = {
      pointerdown: INPUT_START,
      pointermove: INPUT_MOVE,
      pointerup: INPUT_END,
      pointercancel: INPUT_CANCEL,
      pointerout: INPUT_CANCEL
    };
  // in IE10 the pointer types is defined as an enum
  var IE10_POINTER_TYPE_ENUM = {
      2: INPUT_TYPE_TOUCH,
      3: INPUT_TYPE_PEN,
      4: INPUT_TYPE_MOUSE,
      5: INPUT_TYPE_KINECT
    };
  var POINTER_ELEMENT_EVENTS = 'pointerdown';
  var POINTER_WINDOW_EVENTS = 'pointermove pointerup pointercancel';
  // IE10 has prefixed support, and case-sensitive
  if (window.MSPointerEvent && !window.PointerEvent) {
    POINTER_ELEMENT_EVENTS = 'MSPointerDown';
    POINTER_WINDOW_EVENTS = 'MSPointerMove MSPointerUp MSPointerCancel';
  }
  /**
 * Pointer events input
 * @constructor
 * @extends Input
 */
  function PointerEventInput() {
    this.evEl = POINTER_ELEMENT_EVENTS;
    this.evWin = POINTER_WINDOW_EVENTS;
    Input.apply(this, arguments);
    this.store = this.manager.session.pointerEvents = [];
  }
  inherit(PointerEventInput, Input, {
    handler: function PEhandler(ev) {
      var store = this.store;
      var removePointer = false;
      var eventTypeNormalized = ev.type.toLowerCase().replace('ms', '');
      var eventType = POINTER_INPUT_MAP[eventTypeNormalized];
      var pointerType = IE10_POINTER_TYPE_ENUM[ev.pointerType] || ev.pointerType;
      var isTouch = pointerType == INPUT_TYPE_TOUCH;
      // get index of the event in the store
      var storeIndex = inArray(store, ev.pointerId, 'pointerId');
      // start and mouse must be down
      if (eventType & INPUT_START && (ev.button === 0 || isTouch)) {
        if (storeIndex < 0) {
          store.push(ev);
          storeIndex = store.length - 1;
        }
      } else if (eventType & (INPUT_END | INPUT_CANCEL)) {
        removePointer = true;
      }
      // it not found, so the pointer hasn't been down (so it's probably a hover)
      if (storeIndex < 0) {
        return;
      }
      // update the event in the store
      store[storeIndex] = ev;
      this.callback(this.manager, eventType, {
        pointers: store,
        changedPointers: [ev],
        pointerType: pointerType,
        srcEvent: ev
      });
      if (removePointer) {
        // remove from the store
        store.splice(storeIndex, 1);
      }
    }
  });
  var SINGLE_TOUCH_INPUT_MAP = {
      touchstart: INPUT_START,
      touchmove: INPUT_MOVE,
      touchend: INPUT_END,
      touchcancel: INPUT_CANCEL
    };
  var SINGLE_TOUCH_TARGET_EVENTS = 'touchstart';
  var SINGLE_TOUCH_WINDOW_EVENTS = 'touchstart touchmove touchend touchcancel';
  /**
 * Touch events input
 * @constructor
 * @extends Input
 */
  function SingleTouchInput() {
    this.evTarget = SINGLE_TOUCH_TARGET_EVENTS;
    this.evWin = SINGLE_TOUCH_WINDOW_EVENTS;
    this.started = false;
    Input.apply(this, arguments);
  }
  inherit(SingleTouchInput, Input, {
    handler: function TEhandler(ev) {
      var type = SINGLE_TOUCH_INPUT_MAP[ev.type];
      // should we handle the touch events?
      if (type === INPUT_START) {
        this.started = true;
      }
      if (!this.started) {
        return;
      }
      var touches = normalizeSingleTouches.call(this, ev, type);
      // when done, reset the started state
      if (type & (INPUT_END | INPUT_CANCEL) && touches[0].length - touches[1].length === 0) {
        this.started = false;
      }
      this.callback(this.manager, type, {
        pointers: touches[0],
        changedPointers: touches[1],
        pointerType: INPUT_TYPE_TOUCH,
        srcEvent: ev
      });
    }
  });
  /**
 * @this {TouchInput}
 * @param {Object} ev
 * @param {Number} type flag
 * @returns {undefined|Array} [all, changed]
 */
  function normalizeSingleTouches(ev, type) {
    var all = toArray(ev.touches);
    var changed = toArray(ev.changedTouches);
    if (type & (INPUT_END | INPUT_CANCEL)) {
      all = uniqueArray(all.concat(changed), 'identifier', true);
    }
    return [
      all,
      changed
    ];
  }
  var TOUCH_INPUT_MAP = {
      touchstart: INPUT_START,
      touchmove: INPUT_MOVE,
      touchend: INPUT_END,
      touchcancel: INPUT_CANCEL
    };
  var TOUCH_TARGET_EVENTS = 'touchstart touchmove touchend touchcancel';
  /**
 * Multi-user touch events input
 * @constructor
 * @extends Input
 */
  function TouchInput() {
    this.evTarget = TOUCH_TARGET_EVENTS;
    this.targetIds = {};
    Input.apply(this, arguments);
  }
  inherit(TouchInput, Input, {
    handler: function MTEhandler(ev) {
      var type = TOUCH_INPUT_MAP[ev.type];
      var touches = getTouches.call(this, ev, type);
      if (!touches) {
        return;
      }
      this.callback(this.manager, type, {
        pointers: touches[0],
        changedPointers: touches[1],
        pointerType: INPUT_TYPE_TOUCH,
        srcEvent: ev
      });
    }
  });
  /**
 * @this {TouchInput}
 * @param {Object} ev
 * @param {Number} type flag
 * @returns {undefined|Array} [all, changed]
 */
  function getTouches(ev, type) {
    var allTouches = toArray(ev.touches);
    var targetIds = this.targetIds;
    // when there is only one touch, the process can be simplified
    if (type & (INPUT_START | INPUT_MOVE) && allTouches.length === 1) {
      targetIds[allTouches[0].identifier] = true;
      return [
        allTouches,
        allTouches
      ];
    }
    var i, targetTouches, changedTouches = toArray(ev.changedTouches), changedTargetTouches = [], target = this.target;
    // get target touches from touches
    targetTouches = allTouches.filter(function (touch) {
      return hasParent(touch.target, target);
    });
    // collect touches
    if (type === INPUT_START) {
      i = 0;
      while (i < targetTouches.length) {
        targetIds[targetTouches[i].identifier] = true;
        i++;
      }
    }
    // filter changed touches to only contain touches that exist in the collected target ids
    i = 0;
    while (i < changedTouches.length) {
      if (targetIds[changedTouches[i].identifier]) {
        changedTargetTouches.push(changedTouches[i]);
      }
      // cleanup removed touches
      if (type & (INPUT_END | INPUT_CANCEL)) {
        delete targetIds[changedTouches[i].identifier];
      }
      i++;
    }
    if (!changedTargetTouches.length) {
      return;
    }
    return [
      uniqueArray(targetTouches.concat(changedTargetTouches), 'identifier', true),
      changedTargetTouches
    ];
  }
  /**
 * Combined touch and mouse input
 *
 * Touch has a higher priority then mouse, and while touching no mouse events are allowed.
 * This because touch devices also emit mouse events while doing a touch.
 *
 * @constructor
 * @extends Input
 */
  var DEDUP_TIMEOUT = 2500;
  var DEDUP_DISTANCE = 25;
  function TouchMouseInput() {
    Input.apply(this, arguments);
    var handler = bindFn(this.handler, this);
    this.touch = new TouchInput(this.manager, handler);
    this.mouse = new MouseInput(this.manager, handler);
    this.primaryTouch = null;
    this.lastTouches = [];
  }
  inherit(TouchMouseInput, Input, {
    handler: function TMEhandler(manager, inputEvent, inputData) {
      var isTouch = inputData.pointerType == INPUT_TYPE_TOUCH, isMouse = inputData.pointerType == INPUT_TYPE_MOUSE;
      if (isMouse && inputData.sourceCapabilities && inputData.sourceCapabilities.firesTouchEvents) {
        return;
      }
      // when we're in a touch event, record touches to  de-dupe synthetic mouse event
      if (isTouch) {
        recordTouches.call(this, inputEvent, inputData);
      } else if (isMouse && isSyntheticEvent.call(this, inputData)) {
        return;
      }
      this.callback(manager, inputEvent, inputData);
    },
    destroy: function destroy() {
      this.touch.destroy();
      this.mouse.destroy();
    }
  });
  function recordTouches(eventType, eventData) {
    if (eventType & INPUT_START) {
      this.primaryTouch = eventData.changedPointers[0].identifier;
      setLastTouch.call(this, eventData);
    } else if (eventType & (INPUT_END | INPUT_CANCEL)) {
      setLastTouch.call(this, eventData);
    }
  }
  function setLastTouch(eventData) {
    var touch = eventData.changedPointers[0];
    if (touch.identifier === this.primaryTouch) {
      var lastTouch = {
          x: touch.clientX,
          y: touch.clientY
        };
      this.lastTouches.push(lastTouch);
      var lts = this.lastTouches;
      var removeLastTouch = function () {
        var i = lts.indexOf(lastTouch);
        if (i > -1) {
          lts.splice(i, 1);
        }
      };
      setTimeout(removeLastTouch, DEDUP_TIMEOUT);
    }
  }
  function isSyntheticEvent(eventData) {
    var x = eventData.srcEvent.clientX, y = eventData.srcEvent.clientY;
    for (var i = 0; i < this.lastTouches.length; i++) {
      var t = this.lastTouches[i];
      var dx = Math.abs(x - t.x), dy = Math.abs(y - t.y);
      if (dx <= DEDUP_DISTANCE && dy <= DEDUP_DISTANCE) {
        return true;
      }
    }
    return false;
  }
  var PREFIXED_TOUCH_ACTION = prefixed(TEST_ELEMENT.style, 'touchAction');
  var NATIVE_TOUCH_ACTION = PREFIXED_TOUCH_ACTION !== undefined;
  // magical touchAction value
  var TOUCH_ACTION_COMPUTE = 'compute';
  var TOUCH_ACTION_AUTO = 'auto';
  var TOUCH_ACTION_MANIPULATION = 'manipulation';
  // not implemented
  var TOUCH_ACTION_NONE = 'none';
  var TOUCH_ACTION_PAN_X = 'pan-x';
  var TOUCH_ACTION_PAN_Y = 'pan-y';
  var TOUCH_ACTION_MAP = getTouchActionProps();
  /**
 * Touch Action
 * sets the touchAction property or uses the js alternative
 * @param {Manager} manager
 * @param {String} value
 * @constructor
 */
  function TouchAction(manager, value) {
    this.manager = manager;
    this.set(value);
  }
  TouchAction.prototype = {
    set: function (value) {
      // find out the touch-action by the event handlers
      if (value == TOUCH_ACTION_COMPUTE) {
        value = this.compute();
      }
      if (NATIVE_TOUCH_ACTION && this.manager.element.style && TOUCH_ACTION_MAP[value]) {
        this.manager.element.style[PREFIXED_TOUCH_ACTION] = value;
      }
      this.actions = value.toLowerCase().trim();
    },
    update: function () {
      this.set(this.manager.options.touchAction);
    },
    compute: function () {
      var actions = [];
      each(this.manager.recognizers, function (recognizer) {
        if (boolOrFn(recognizer.options.enable, [recognizer])) {
          actions = actions.concat(recognizer.getTouchAction());
        }
      });
      return cleanTouchActions(actions.join(' '));
    },
    preventDefaults: function (input) {
      var srcEvent = input.srcEvent;
      var direction = input.offsetDirection;
      // if the touch action did prevented once this session
      if (this.manager.session.prevented) {
        srcEvent.preventDefault();
        return;
      }
      var actions = this.actions;
      var hasNone = inStr(actions, TOUCH_ACTION_NONE) && !TOUCH_ACTION_MAP[TOUCH_ACTION_NONE];
      var hasPanY = inStr(actions, TOUCH_ACTION_PAN_Y) && !TOUCH_ACTION_MAP[TOUCH_ACTION_PAN_Y];
      var hasPanX = inStr(actions, TOUCH_ACTION_PAN_X) && !TOUCH_ACTION_MAP[TOUCH_ACTION_PAN_X];
      if (hasNone) {
        //do not prevent defaults if this is a tap gesture
        var isTapPointer = input.pointers.length === 1;
        var isTapMovement = input.distance < 2;
        var isTapTouchTime = input.deltaTime < 250;
        if (isTapPointer && isTapMovement && isTapTouchTime) {
          return;
        }
      }
      if (hasPanX && hasPanY) {
        // `pan-x pan-y` means browser handles all scrolling/panning, do not prevent
        return;
      }
      if (hasNone || hasPanY && direction & DIRECTION_HORIZONTAL || hasPanX && direction & DIRECTION_VERTICAL) {
        return this.preventSrc(srcEvent);
      }
    },
    preventSrc: function (srcEvent) {
      this.manager.session.prevented = true;
      srcEvent.preventDefault();
    }
  };
  /**
 * when the touchActions are collected they are not a valid value, so we need to clean things up. *
 * @param {String} actions
 * @returns {*}
 */
  function cleanTouchActions(actions) {
    // none
    if (inStr(actions, TOUCH_ACTION_NONE)) {
      return TOUCH_ACTION_NONE;
    }
    var hasPanX = inStr(actions, TOUCH_ACTION_PAN_X);
    var hasPanY = inStr(actions, TOUCH_ACTION_PAN_Y);
    // if both pan-x and pan-y are set (different recognizers
    // for different directions, e.g. horizontal pan but vertical swipe?)
    // we need none (as otherwise with pan-x pan-y combined none of these
    // recognizers will work, since the browser would handle all panning
    if (hasPanX && hasPanY) {
      return TOUCH_ACTION_NONE;
    }
    // pan-x OR pan-y
    if (hasPanX || hasPanY) {
      return hasPanX ? TOUCH_ACTION_PAN_X : TOUCH_ACTION_PAN_Y;
    }
    // manipulation
    if (inStr(actions, TOUCH_ACTION_MANIPULATION)) {
      return TOUCH_ACTION_MANIPULATION;
    }
    return TOUCH_ACTION_AUTO;
  }
  function getTouchActionProps() {
    if (!NATIVE_TOUCH_ACTION) {
      return false;
    }
    var touchMap = {};
    var cssSupports = window.CSS && window.CSS.supports;
    [
      'auto',
      'manipulation',
      'pan-y',
      'pan-x',
      'pan-x pan-y',
      'none'
    ].forEach(function (val) {
      // If css.supports is not supported but there is native touch-action assume it supports
      // all values. This is the case for IE 10 and 11.
      touchMap[val] = cssSupports ? window.CSS.supports('touch-action', val) : true;
    });
    return touchMap;
  }
  /**
 * Recognizer flow explained; *
 * All recognizers have the initial state of POSSIBLE when a input session starts.
 * The definition of a input session is from the first input until the last input, with all it's movement in it. *
 * Example session for mouse-input: mousedown -> mousemove -> mouseup
 *
 * On each recognizing cycle (see Manager.recognize) the .recognize() method is executed
 * which determines with state it should be.
 *
 * If the recognizer has the state FAILED, CANCELLED or RECOGNIZED (equals ENDED), it is reset to
 * POSSIBLE to give it another change on the next cycle.
 *
 *               Possible
 *                  |
 *            +-----+---------------+
 *            |                     |
 *      +-----+-----+               |
 *      |           |               |
 *   Failed      Cancelled          |
 *                          +-------+------+
 *                          |              |
 *                      Recognized       Began
 *                                         |
 *                                      Changed
 *                                         |
 *                                  Ended/Recognized
 */
  var STATE_POSSIBLE = 1;
  var STATE_BEGAN = 2;
  var STATE_CHANGED = 4;
  var STATE_ENDED = 8;
  var STATE_RECOGNIZED = STATE_ENDED;
  var STATE_CANCELLED = 16;
  var STATE_FAILED = 32;
  /**
 * Recognizer
 * Every recognizer needs to extend from this class.
 * @constructor
 * @param {Object} options
 */
  function Recognizer(options) {
    this.options = assign({}, this.defaults, options || {});
    this.id = uniqueId();
    this.manager = null;
    // default is enable true
    this.options.enable = ifUndefined(this.options.enable, true);
    this.state = STATE_POSSIBLE;
    this.simultaneous = {};
    this.requireFail = [];
  }
  Recognizer.prototype = {
    defaults: {},
    set: function (options) {
      assign(this.options, options);
      // also update the touchAction, in case something changed about the directions/enabled state
      this.manager && this.manager.touchAction.update();
      return this;
    },
    recognizeWith: function (otherRecognizer) {
      if (invokeArrayArg(otherRecognizer, 'recognizeWith', this)) {
        return this;
      }
      var simultaneous = this.simultaneous;
      otherRecognizer = getRecognizerByNameIfManager(otherRecognizer, this);
      if (!simultaneous[otherRecognizer.id]) {
        simultaneous[otherRecognizer.id] = otherRecognizer;
        otherRecognizer.recognizeWith(this);
      }
      return this;
    },
    dropRecognizeWith: function (otherRecognizer) {
      if (invokeArrayArg(otherRecognizer, 'dropRecognizeWith', this)) {
        return this;
      }
      otherRecognizer = getRecognizerByNameIfManager(otherRecognizer, this);
      delete this.simultaneous[otherRecognizer.id];
      return this;
    },
    requireFailure: function (otherRecognizer) {
      if (invokeArrayArg(otherRecognizer, 'requireFailure', this)) {
        return this;
      }
      var requireFail = this.requireFail;
      otherRecognizer = getRecognizerByNameIfManager(otherRecognizer, this);
      if (inArray(requireFail, otherRecognizer) === -1) {
        requireFail.push(otherRecognizer);
        otherRecognizer.requireFailure(this);
      }
      return this;
    },
    dropRequireFailure: function (otherRecognizer) {
      if (invokeArrayArg(otherRecognizer, 'dropRequireFailure', this)) {
        return this;
      }
      otherRecognizer = getRecognizerByNameIfManager(otherRecognizer, this);
      var index = inArray(this.requireFail, otherRecognizer);
      if (index > -1) {
        this.requireFail.splice(index, 1);
      }
      return this;
    },
    hasRequireFailures: function () {
      return this.requireFail.length > 0;
    },
    canRecognizeWith: function (otherRecognizer) {
      return !!this.simultaneous[otherRecognizer.id];
    },
    emit: function (input) {
      var self = this;
      var state = this.state;
      function emit(event) {
        self.manager.emit(event, input);
      }
      // 'panstart' and 'panmove'
      if (state < STATE_ENDED) {
        emit(self.options.event + stateStr(state));
      }
      emit(self.options.event);
      // simple 'eventName' events
      if (input.additionalEvent) {
        // additional event(panleft, panright, pinchin, pinchout...)
        emit(input.additionalEvent);
      }
      // panend and pancancel
      if (state >= STATE_ENDED) {
        emit(self.options.event + stateStr(state));
      }
    },
    tryEmit: function (input) {
      if (this.canEmit()) {
        return this.emit(input);
      }
      // it's failing anyway
      this.state = STATE_FAILED;
    },
    canEmit: function () {
      var i = 0;
      while (i < this.requireFail.length) {
        if (!(this.requireFail[i].state & (STATE_FAILED | STATE_POSSIBLE))) {
          return false;
        }
        i++;
      }
      return true;
    },
    recognize: function (inputData) {
      // make a new copy of the inputData
      // so we can change the inputData without messing up the other recognizers
      var inputDataClone = assign({}, inputData);
      // is is enabled and allow recognizing?
      if (!boolOrFn(this.options.enable, [
          this,
          inputDataClone
        ])) {
        this.reset();
        this.state = STATE_FAILED;
        return;
      }
      // reset when we've reached the end
      if (this.state & (STATE_RECOGNIZED | STATE_CANCELLED | STATE_FAILED)) {
        this.state = STATE_POSSIBLE;
      }
      this.state = this.process(inputDataClone);
      // the recognizer has recognized a gesture
      // so trigger an event
      if (this.state & (STATE_BEGAN | STATE_CHANGED | STATE_ENDED | STATE_CANCELLED)) {
        this.tryEmit(inputDataClone);
      }
    },
    process: function (inputData) {
    },
    getTouchAction: function () {
    },
    reset: function () {
    }
  };
  /**
 * get a usable string, used as event postfix
 * @param {Const} state
 * @returns {String} state
 */
  function stateStr(state) {
    if (state & STATE_CANCELLED) {
      return 'cancel';
    } else if (state & STATE_ENDED) {
      return 'end';
    } else if (state & STATE_CHANGED) {
      return 'move';
    } else if (state & STATE_BEGAN) {
      return 'start';
    }
    return '';
  }
  /**
 * direction cons to string
 * @param {Const} direction
 * @returns {String}
 */
  function directionStr(direction) {
    if (direction == DIRECTION_DOWN) {
      return 'down';
    } else if (direction == DIRECTION_UP) {
      return 'up';
    } else if (direction == DIRECTION_LEFT) {
      return 'left';
    } else if (direction == DIRECTION_RIGHT) {
      return 'right';
    }
    return '';
  }
  /**
 * get a recognizer by name if it is bound to a manager
 * @param {Recognizer|String} otherRecognizer
 * @param {Recognizer} recognizer
 * @returns {Recognizer}
 */
  function getRecognizerByNameIfManager(otherRecognizer, recognizer) {
    var manager = recognizer.manager;
    if (manager) {
      return manager.get(otherRecognizer);
    }
    return otherRecognizer;
  }
  /**
 * This recognizer is just used as a base for the simple attribute recognizers.
 * @constructor
 * @extends Recognizer
 */
  function AttrRecognizer() {
    Recognizer.apply(this, arguments);
  }
  inherit(AttrRecognizer, Recognizer, {
    defaults: { pointers: 1 },
    attrTest: function (input) {
      var optionPointers = this.options.pointers;
      return optionPointers === 0 || input.pointers.length === optionPointers;
    },
    process: function (input) {
      var state = this.state;
      var eventType = input.eventType;
      var isRecognized = state & (STATE_BEGAN | STATE_CHANGED);
      var isValid = this.attrTest(input);
      // on cancel input and we've recognized before, return STATE_CANCELLED
      if (isRecognized && (eventType & INPUT_CANCEL || !isValid)) {
        return state | STATE_CANCELLED;
      } else if (isRecognized || isValid) {
        if (eventType & INPUT_END) {
          return state | STATE_ENDED;
        } else if (!(state & STATE_BEGAN)) {
          return STATE_BEGAN;
        }
        return state | STATE_CHANGED;
      }
      return STATE_FAILED;
    }
  });
  /**
 * Pan
 * Recognized when the pointer is down and moved in the allowed direction.
 * @constructor
 * @extends AttrRecognizer
 */
  function PanRecognizer() {
    AttrRecognizer.apply(this, arguments);
    this.pX = null;
    this.pY = null;
  }
  inherit(PanRecognizer, AttrRecognizer, {
    defaults: {
      event: 'pan',
      threshold: 10,
      pointers: 1,
      direction: DIRECTION_ALL
    },
    getTouchAction: function () {
      var direction = this.options.direction;
      var actions = [];
      if (direction & DIRECTION_HORIZONTAL) {
        actions.push(TOUCH_ACTION_PAN_Y);
      }
      if (direction & DIRECTION_VERTICAL) {
        actions.push(TOUCH_ACTION_PAN_X);
      }
      return actions;
    },
    directionTest: function (input) {
      var options = this.options;
      var hasMoved = true;
      var distance = input.distance;
      var direction = input.direction;
      var x = input.deltaX;
      var y = input.deltaY;
      // lock to axis?
      if (!(direction & options.direction)) {
        if (options.direction & DIRECTION_HORIZONTAL) {
          direction = x === 0 ? DIRECTION_NONE : x < 0 ? DIRECTION_LEFT : DIRECTION_RIGHT;
          hasMoved = x != this.pX;
          distance = Math.abs(input.deltaX);
        } else {
          direction = y === 0 ? DIRECTION_NONE : y < 0 ? DIRECTION_UP : DIRECTION_DOWN;
          hasMoved = y != this.pY;
          distance = Math.abs(input.deltaY);
        }
      }
      input.direction = direction;
      return hasMoved && distance > options.threshold && direction & options.direction;
    },
    attrTest: function (input) {
      return AttrRecognizer.prototype.attrTest.call(this, input) && (this.state & STATE_BEGAN || !(this.state & STATE_BEGAN) && this.directionTest(input));
    },
    emit: function (input) {
      this.pX = input.deltaX;
      this.pY = input.deltaY;
      var direction = directionStr(input.direction);
      if (direction) {
        input.additionalEvent = this.options.event + direction;
      }
      this._super.emit.call(this, input);
    }
  });
  /**
 * Pinch
 * Recognized when two or more pointers are moving toward (zoom-in) or away from each other (zoom-out).
 * @constructor
 * @extends AttrRecognizer
 */
  function PinchRecognizer() {
    AttrRecognizer.apply(this, arguments);
  }
  inherit(PinchRecognizer, AttrRecognizer, {
    defaults: {
      event: 'pinch',
      threshold: 0,
      pointers: 2
    },
    getTouchAction: function () {
      return [TOUCH_ACTION_NONE];
    },
    attrTest: function (input) {
      return this._super.attrTest.call(this, input) && (Math.abs(input.scale - 1) > this.options.threshold || this.state & STATE_BEGAN);
    },
    emit: function (input) {
      if (input.scale !== 1) {
        var inOut = input.scale < 1 ? 'in' : 'out';
        input.additionalEvent = this.options.event + inOut;
      }
      this._super.emit.call(this, input);
    }
  });
  /**
 * Press
 * Recognized when the pointer is down for x ms without any movement.
 * @constructor
 * @extends Recognizer
 */
  function PressRecognizer() {
    Recognizer.apply(this, arguments);
    this._timer = null;
    this._input = null;
  }
  inherit(PressRecognizer, Recognizer, {
    defaults: {
      event: 'press',
      pointers: 1,
      time: 251,
      threshold: 9
    },
    getTouchAction: function () {
      return [TOUCH_ACTION_AUTO];
    },
    process: function (input) {
      var options = this.options;
      var validPointers = input.pointers.length === options.pointers;
      var validMovement = input.distance < options.threshold;
      var validTime = input.deltaTime > options.time;
      this._input = input;
      // we only allow little movement
      // and we've reached an end event, so a tap is possible
      if (!validMovement || !validPointers || input.eventType & (INPUT_END | INPUT_CANCEL) && !validTime) {
        this.reset();
      } else if (input.eventType & INPUT_START) {
        this.reset();
        this._timer = setTimeoutContext(function () {
          this.state = STATE_RECOGNIZED;
          this.tryEmit();
        }, options.time, this);
      } else if (input.eventType & INPUT_END) {
        return STATE_RECOGNIZED;
      }
      return STATE_FAILED;
    },
    reset: function () {
      clearTimeout(this._timer);
    },
    emit: function (input) {
      if (this.state !== STATE_RECOGNIZED) {
        return;
      }
      if (input && input.eventType & INPUT_END) {
        this.manager.emit(this.options.event + 'up', input);
      } else {
        this._input.timeStamp = now();
        this.manager.emit(this.options.event, this._input);
      }
    }
  });
  /**
 * Rotate
 * Recognized when two or more pointer are moving in a circular motion.
 * @constructor
 * @extends AttrRecognizer
 */
  function RotateRecognizer() {
    AttrRecognizer.apply(this, arguments);
  }
  inherit(RotateRecognizer, AttrRecognizer, {
    defaults: {
      event: 'rotate',
      threshold: 0,
      pointers: 2
    },
    getTouchAction: function () {
      return [TOUCH_ACTION_NONE];
    },
    attrTest: function (input) {
      return this._super.attrTest.call(this, input) && (Math.abs(input.rotation) > this.options.threshold || this.state & STATE_BEGAN);
    }
  });
  /**
 * Swipe
 * Recognized when the pointer is moving fast (velocity), with enough distance in the allowed direction.
 * @constructor
 * @extends AttrRecognizer
 */
  function SwipeRecognizer() {
    AttrRecognizer.apply(this, arguments);
  }
  inherit(SwipeRecognizer, AttrRecognizer, {
    defaults: {
      event: 'swipe',
      threshold: 10,
      velocity: 0.3,
      direction: DIRECTION_HORIZONTAL | DIRECTION_VERTICAL,
      pointers: 1
    },
    getTouchAction: function () {
      return PanRecognizer.prototype.getTouchAction.call(this);
    },
    attrTest: function (input) {
      var direction = this.options.direction;
      var velocity;
      if (direction & (DIRECTION_HORIZONTAL | DIRECTION_VERTICAL)) {
        velocity = input.overallVelocity;
      } else if (direction & DIRECTION_HORIZONTAL) {
        velocity = input.overallVelocityX;
      } else if (direction & DIRECTION_VERTICAL) {
        velocity = input.overallVelocityY;
      }
      return this._super.attrTest.call(this, input) && direction & input.offsetDirection && input.distance > this.options.threshold && input.maxPointers == this.options.pointers && abs(velocity) > this.options.velocity && input.eventType & INPUT_END;
    },
    emit: function (input) {
      var direction = directionStr(input.offsetDirection);
      if (direction) {
        this.manager.emit(this.options.event + direction, input);
      }
      this.manager.emit(this.options.event, input);
    }
  });
  /**
 * A tap is ecognized when the pointer is doing a small tap/click. Multiple taps are recognized if they occur
 * between the given interval and position. The delay option can be used to recognize multi-taps without firing
 * a single tap.
 *
 * The eventData from the emitted event contains the property `tapCount`, which contains the amount of
 * multi-taps being recognized.
 * @constructor
 * @extends Recognizer
 */
  function TapRecognizer() {
    Recognizer.apply(this, arguments);
    // previous time and center,
    // used for tap counting
    this.pTime = false;
    this.pCenter = false;
    this._timer = null;
    this._input = null;
    this.count = 0;
  }
  inherit(TapRecognizer, Recognizer, {
    defaults: {
      event: 'tap',
      pointers: 1,
      taps: 1,
      interval: 300,
      time: 250,
      threshold: 9,
      posThreshold: 10
    },
    getTouchAction: function () {
      return [TOUCH_ACTION_MANIPULATION];
    },
    process: function (input) {
      var options = this.options;
      var validPointers = input.pointers.length === options.pointers;
      var validMovement = input.distance < options.threshold;
      var validTouchTime = input.deltaTime < options.time;
      this.reset();
      if (input.eventType & INPUT_START && this.count === 0) {
        return this.failTimeout();
      }
      // we only allow little movement
      // and we've reached an end event, so a tap is possible
      if (validMovement && validTouchTime && validPointers) {
        if (input.eventType != INPUT_END) {
          return this.failTimeout();
        }
        var validInterval = this.pTime ? input.timeStamp - this.pTime < options.interval : true;
        var validMultiTap = !this.pCenter || getDistance(this.pCenter, input.center) < options.posThreshold;
        this.pTime = input.timeStamp;
        this.pCenter = input.center;
        if (!validMultiTap || !validInterval) {
          this.count = 1;
        } else {
          this.count += 1;
        }
        this._input = input;
        // if tap count matches we have recognized it,
        // else it has began recognizing...
        var tapCount = this.count % options.taps;
        if (tapCount === 0) {
          // no failing requirements, immediately trigger the tap event
          // or wait as long as the multitap interval to trigger
          if (!this.hasRequireFailures()) {
            return STATE_RECOGNIZED;
          } else {
            this._timer = setTimeoutContext(function () {
              this.state = STATE_RECOGNIZED;
              this.tryEmit();
            }, options.interval, this);
            return STATE_BEGAN;
          }
        }
      }
      return STATE_FAILED;
    },
    failTimeout: function () {
      this._timer = setTimeoutContext(function () {
        this.state = STATE_FAILED;
      }, this.options.interval, this);
      return STATE_FAILED;
    },
    reset: function () {
      clearTimeout(this._timer);
    },
    emit: function () {
      if (this.state == STATE_RECOGNIZED) {
        this._input.tapCount = this.count;
        this.manager.emit(this.options.event, this._input);
      }
    }
  });
  /**
 * Simple way to create a manager with a default set of recognizers.
 * @param {HTMLElement} element
 * @param {Object} [options]
 * @constructor
 */
  function Hammer(element, options) {
    options = options || {};
    options.recognizers = ifUndefined(options.recognizers, Hammer.defaults.preset);
    return new Manager(element, options);
  }
  /**
 * @const {string}
 */
  Hammer.VERSION = '2.0.7';
  /**
 * default settings
 * @namespace
 */
  Hammer.defaults = {
    domEvents: false,
    touchAction: TOUCH_ACTION_COMPUTE,
    enable: true,
    inputTarget: null,
    inputClass: null,
    preset: [
      [
        RotateRecognizer,
        { enable: false }
      ],
      [
        PinchRecognizer,
        { enable: false },
        ['rotate']
      ],
      [
        SwipeRecognizer,
        { direction: DIRECTION_HORIZONTAL }
      ],
      [
        PanRecognizer,
        { direction: DIRECTION_HORIZONTAL },
        ['swipe']
      ],
      [TapRecognizer],
      [
        TapRecognizer,
        {
          event: 'doubletap',
          taps: 2
        },
        ['tap']
      ],
      [PressRecognizer]
    ],
    cssProps: {
      userSelect: 'none',
      touchSelect: 'none',
      touchCallout: 'none',
      contentZooming: 'none',
      userDrag: 'none',
      tapHighlightColor: 'rgba(0,0,0,0)'
    }
  };
  var STOP = 1;
  var FORCED_STOP = 2;
  /**
 * Manager
 * @param {HTMLElement} element
 * @param {Object} [options]
 * @constructor
 */
  function Manager(element, options) {
    this.options = assign({}, Hammer.defaults, options || {});
    this.options.inputTarget = this.options.inputTarget || element;
    this.handlers = {};
    this.session = {};
    this.recognizers = [];
    this.oldCssProps = {};
    this.element = element;
    this.input = createInputInstance(this);
    this.touchAction = new TouchAction(this, this.options.touchAction);
    toggleCssProps(this, true);
    each(this.options.recognizers, function (item) {
      var recognizer = this.add(new item[0](item[1]));
      item[2] && recognizer.recognizeWith(item[2]);
      item[3] && recognizer.requireFailure(item[3]);
    }, this);
  }
  Manager.prototype = {
    set: function (options) {
      assign(this.options, options);
      // Options that need a little more setup
      if (options.touchAction) {
        this.touchAction.update();
      }
      if (options.inputTarget) {
        // Clean up existing event listeners and reinitialize
        this.input.destroy();
        this.input.target = options.inputTarget;
        this.input.init();
      }
      return this;
    },
    stop: function (force) {
      this.session.stopped = force ? FORCED_STOP : STOP;
    },
    recognize: function (inputData) {
      var session = this.session;
      if (session.stopped) {
        return;
      }
      // run the touch-action polyfill
      this.touchAction.preventDefaults(inputData);
      var recognizer;
      var recognizers = this.recognizers;
      // this holds the recognizer that is being recognized.
      // so the recognizer's state needs to be BEGAN, CHANGED, ENDED or RECOGNIZED
      // if no recognizer is detecting a thing, it is set to `null`
      var curRecognizer = session.curRecognizer;
      // reset when the last recognizer is recognized
      // or when we're in a new session
      if (!curRecognizer || curRecognizer && curRecognizer.state & STATE_RECOGNIZED) {
        curRecognizer = session.curRecognizer = null;
      }
      var i = 0;
      while (i < recognizers.length) {
        recognizer = recognizers[i];
        // find out if we are allowed try to recognize the input for this one.
        // 1.   allow if the session is NOT forced stopped (see the .stop() method)
        // 2.   allow if we still haven't recognized a gesture in this session, or the this recognizer is the one
        //      that is being recognized.
        // 3.   allow if the recognizer is allowed to run simultaneous with the current recognized recognizer.
        //      this can be setup with the `recognizeWith()` method on the recognizer.
        if (session.stopped !== FORCED_STOP && (!curRecognizer || recognizer == curRecognizer || recognizer.canRecognizeWith(curRecognizer))) {
          // 3
          recognizer.recognize(inputData);
        } else {
          recognizer.reset();
        }
        // if the recognizer has been recognizing the input as a valid gesture, we want to store this one as the
        // current active recognizer. but only if we don't already have an active recognizer
        if (!curRecognizer && recognizer.state & (STATE_BEGAN | STATE_CHANGED | STATE_ENDED)) {
          curRecognizer = session.curRecognizer = recognizer;
        }
        i++;
      }
    },
    get: function (recognizer) {
      if (recognizer instanceof Recognizer) {
        return recognizer;
      }
      var recognizers = this.recognizers;
      for (var i = 0; i < recognizers.length; i++) {
        if (recognizers[i].options.event == recognizer) {
          return recognizers[i];
        }
      }
      return null;
    },
    add: function (recognizer) {
      if (invokeArrayArg(recognizer, 'add', this)) {
        return this;
      }
      // remove existing
      var existing = this.get(recognizer.options.event);
      if (existing) {
        this.remove(existing);
      }
      this.recognizers.push(recognizer);
      recognizer.manager = this;
      this.touchAction.update();
      return recognizer;
    },
    remove: function (recognizer) {
      if (invokeArrayArg(recognizer, 'remove', this)) {
        return this;
      }
      recognizer = this.get(recognizer);
      // let's make sure this recognizer exists
      if (recognizer) {
        var recognizers = this.recognizers;
        var index = inArray(recognizers, recognizer);
        if (index !== -1) {
          recognizers.splice(index, 1);
          this.touchAction.update();
        }
      }
      return this;
    },
    on: function (events, handler) {
      if (events === undefined) {
        return;
      }
      if (handler === undefined) {
        return;
      }
      var handlers = this.handlers;
      each(splitStr(events), function (event) {
        handlers[event] = handlers[event] || [];
        handlers[event].push(handler);
      });
      return this;
    },
    off: function (events, handler) {
      if (events === undefined) {
        return;
      }
      var handlers = this.handlers;
      each(splitStr(events), function (event) {
        if (!handler) {
          delete handlers[event];
        } else {
          handlers[event] && handlers[event].splice(inArray(handlers[event], handler), 1);
        }
      });
      return this;
    },
    emit: function (event, data) {
      // we also want to trigger dom events
      if (this.options.domEvents) {
        triggerDomEvent(event, data);
      }
      // no handlers, so skip it all
      var handlers = this.handlers[event] && this.handlers[event].slice();
      if (!handlers || !handlers.length) {
        return;
      }
      data.type = event;
      data.preventDefault = function () {
        data.srcEvent.preventDefault();
      };
      var i = 0;
      while (i < handlers.length) {
        handlers[i](data);
        i++;
      }
    },
    destroy: function () {
      this.element && toggleCssProps(this, false);
      this.handlers = {};
      this.session = {};
      this.input.destroy();
      this.element = null;
    }
  };
  /**
 * add/remove the css properties as defined in manager.options.cssProps
 * @param {Manager} manager
 * @param {Boolean} add
 */
  function toggleCssProps(manager, add) {
    var element = manager.element;
    if (!element.style) {
      return;
    }
    var prop;
    each(manager.options.cssProps, function (value, name) {
      prop = prefixed(element.style, name);
      if (add) {
        manager.oldCssProps[prop] = element.style[prop];
        element.style[prop] = value;
      } else {
        element.style[prop] = manager.oldCssProps[prop] || '';
      }
    });
    if (!add) {
      manager.oldCssProps = {};
    }
  }
  /**
 * trigger dom event
 * @param {String} event
 * @param {Object} data
 */
  function triggerDomEvent(event, data) {
    var gestureEvent = document.createEvent('Event');
    gestureEvent.initEvent(event, true, true);
    gestureEvent.gesture = data;
    data.target.dispatchEvent(gestureEvent);
  }
  assign(Hammer, {
    INPUT_START: INPUT_START,
    INPUT_MOVE: INPUT_MOVE,
    INPUT_END: INPUT_END,
    INPUT_CANCEL: INPUT_CANCEL,
    STATE_POSSIBLE: STATE_POSSIBLE,
    STATE_BEGAN: STATE_BEGAN,
    STATE_CHANGED: STATE_CHANGED,
    STATE_ENDED: STATE_ENDED,
    STATE_RECOGNIZED: STATE_RECOGNIZED,
    STATE_CANCELLED: STATE_CANCELLED,
    STATE_FAILED: STATE_FAILED,
    DIRECTION_NONE: DIRECTION_NONE,
    DIRECTION_LEFT: DIRECTION_LEFT,
    DIRECTION_RIGHT: DIRECTION_RIGHT,
    DIRECTION_UP: DIRECTION_UP,
    DIRECTION_DOWN: DIRECTION_DOWN,
    DIRECTION_HORIZONTAL: DIRECTION_HORIZONTAL,
    DIRECTION_VERTICAL: DIRECTION_VERTICAL,
    DIRECTION_ALL: DIRECTION_ALL,
    Manager: Manager,
    Input: Input,
    TouchAction: TouchAction,
    TouchInput: TouchInput,
    MouseInput: MouseInput,
    PointerEventInput: PointerEventInput,
    TouchMouseInput: TouchMouseInput,
    SingleTouchInput: SingleTouchInput,
    Recognizer: Recognizer,
    AttrRecognizer: AttrRecognizer,
    Tap: TapRecognizer,
    Pan: PanRecognizer,
    Swipe: SwipeRecognizer,
    Pinch: PinchRecognizer,
    Rotate: RotateRecognizer,
    Press: PressRecognizer,
    on: addEventListeners,
    off: removeEventListeners,
    each: each,
    merge: merge,
    extend: extend,
    assign: assign,
    inherit: inherit,
    bindFn: bindFn,
    prefixed: prefixed
  });
  // this prevents errors when Hammer is loaded in the presence of an AMD
  //  style loader but by script tag, not by the loader.
  var freeGlobal = typeof window !== 'undefined' ? window : typeof self !== 'undefined' ? self : {};
  // jshint ignore:line
  freeGlobal.Hammer = Hammer;
  if (typeof define === 'function' && define.amd) {
    define(function () {
      return Hammer;
    });
  } else if (typeof module != 'undefined' && module.exports) {
    module.exports = Hammer;
  } else {
    window[exportName] = Hammer;
  }
}(window, document, 'Hammer'));
/*
 * angular-hammer v1.0.3
 * (c) 2013 Monospaced http://monospaced.com
 * License: MIT
 */
(function (window, angular, Hammer) {
  var hmTouchEvents = angular.module('hmTouchEvents', []), hmGestures = [
      'hmHold:hold',
      'hmTap:tap',
      'hmDoubletap:doubletap',
      'hmDrag:drag',
      'hmDragstart:dragstart',
      'hmDragend:dragend',
      'hmDragup:dragup',
      'hmDragdown:dragdown',
      'hmDragleft:dragleft',
      'hmDragright:dragright',
      'hmSwipe:swipe',
      'hmSwipeup:swipeup',
      'hmSwipedown:swipedown',
      'hmSwipeleft:swipeleft',
      'hmSwiperight:swiperight',
      'hmTransform:transform',
      'hmTransformstart:transformstart',
      'hmTransformend:transformend',
      'hmRotate:rotate',
      'hmPinch:pinch',
      'hmPinchin:pinchin',
      'hmPinchout:pinchout',
      'hmTouch:touch',
      'hmRelease:release'
    ];
  angular.forEach(hmGestures, function (name) {
    var directive = name.split(':'), directiveName = directive[0], eventName = directive[1];
    hmTouchEvents.directive(directiveName, [
      '$parse',
      '$window',
      function ($parse, $window) {
        return {
          restrict: 'A, C',
          link: function (scope, element, attr) {
            var expr = $parse(attr[directiveName]), fn = function (event) {
                scope.$apply(function () {
                  expr(scope, { $event: event });
                });
              }, opts = $parse(attr['hmOptions'])(scope, {}), hammer;
            if (typeof Hammer === 'undefined' || !$window.addEventListener) {
              // fallback to mouse events where appropriate
              if (directiveName === 'hmTap') {
                element.bind('click', fn);
              }
              if (directiveName === 'hmDoubletap') {
                element.bind('dblclick', fn);
              }
              return;
            }
            // don't create multiple Hammer instances per element
            if (!(hammer = element.data('hammer'))) {
              hammer = Hammer(element[0], opts);
              element.data('hammer', hammer);
            }
            // bind Hammer touch event
            hammer.on(eventName, fn);
            // unbind Hammer touch event
            scope.$on('$destroy', function () {
              hammer.off(eventName, fn);
            });
          }
        };
      }
    ]);
  });
}(window, window.angular, window.Hammer));
/**
 * @license videogular v1.4.4 http://videogular.com
 * Two Fucking Developers http://twofuckingdevelopers.com
 * License: MIT
 */
'use strict';
angular.module('com.2fdevs.videogular', ['ngSanitize']).run([
  '$templateCache',
  function ($templateCache) {
    $templateCache.put('vg-templates/vg-media-video', '<video></video>');
    $templateCache.put('vg-templates/vg-media-audio', '<audio></audio>');
    // Support for browsers that doesn't have .bind()
    if (!Function.prototype.bind) {
      Function.prototype.bind = function (oThis) {
        if (typeof this !== 'function') {
          // closest thing possible to the ECMAScript 5
          // internal IsCallable function
          throw new TypeError('Function.prototype.bind - what is trying to be bound is not callable');
        }
        var aArgs = Array.prototype.slice.call(arguments, 1), fToBind = this, fNOP = function () {
          }, fBound = function () {
            return fToBind.apply(this instanceof fNOP ? this : oThis, aArgs.concat(Array.prototype.slice.call(arguments)));
          };
        fNOP.prototype = this.prototype;
        fBound.prototype = new fNOP();
        return fBound;
      };
    }
  }
]);
/**
 * @ngdoc service
 * @name com.2fdevs.videogular.constant:VG_STATES
 *
 * @description
 * Possible video states:
 *  - VG_STATES.PLAY: "play"
 *  - VG_STATES.PAUSE: "pause"
 *  - VG_STATES.STOP: "stop"
 **/
/**
 * @ngdoc service
 * @name com.2fdevs.videogular.constant:VG_VOLUME_KEY
 *
 * @description localStorage key name for persistent video play volume on a domain.
 **/
'use strict';
angular.module('com.2fdevs.videogular').constant('VG_STATES', {
  PLAY: 'play',
  PAUSE: 'pause',
  STOP: 'stop'
}).constant('VG_VOLUME_KEY', 'videogularVolume');
'use strict';
/**
 * @ngdoc controller
 * @name com.2fdevs.videogular.controller:vgController
 * @description
 * Videogular controller.
 * This controller offers a public API:
 *
 * Methods
 * - play(): Plays media.
 * - pause(): Pause media.
 * - stop(): Stops media.
 * - playPause(): Toggles play and pause.
 * - seekTime(value, byPercent): Seeks to a specified time position. Param value must be an integer representing the target position in seconds or a percentage. By default seekTime seeks by seconds, if you want to seek by percentage just pass byPercent to true.
 * - setVolume(volume): Sets volume. Param volume must be a float number with a value between 0 and 1.
 * - setPlayback(playback): Sets playback. Param plaback must be a float number with a value between 0 and 2.
 * - setState(state): Sets a new state. Param state mus be an string with 'play', 'pause' or 'stop'. This method only changes the state of the player, but doesn't plays, pauses or stops the media file.
 * - toggleFullScreen(): Toggles between fullscreen and normal mode.
 * - updateTheme(css-url): Removes previous CSS theme and sets a new one.
 * - clearMedia(): Cleans the current media file.
 * - changeSource(array): Updates current media source. Param `array` must be an array of media source objects or a simple URL string.
 * A media source is an object with two properties `src` and `type`. The `src` property must contains a trustful url resource.
 * <pre>{src: $sce.trustAsResourceUrl("http://static.videogular.com/assets/videos/videogular.mp4"), type: "video/mp4"}</pre>
 *
 * Properties
 * - config: String with a url to JSON config file.
 * - isReady: Boolean value with current player initialization state.
 * - isBuffering: Boolean value to know if player is buffering media.
 * - isCompleted: Boolean value to know if current media file has been completed.
 * - isLive: Boolean value to know if current media file is a Live Streaming.
 * - playsInline: Boolean value to know if Videogular is using inline playing or not.
 * - nativeFullscreen: Boolean value to know if Videogular if fullscreen mode will use native mode or emulated mode.
 * - mediaElement: Reference to video/audio object.
 * - videogularElement: Reference to videogular tag.
 * - sources: Array with current sources or a simple URL string.
 * - tracks: Array with current tracks.
 * - cuePoints: Object containing a list of timelines with cue points. Each property in the object represents a timeline, which is an Array of objects with the next definition:
 * <pre>{
 *    timeLapse:{
 *      start: 0,
 *      end: 10
 *    },
 *    onEnter: callback(currentTime, timeLapse, params),
 *    onLeave: callback(currentTime, timeLapse, params),
 *    onUpdate: callback(currentTime, timeLapse, params),
 *    onComplete: callback(currentTime, timeLapse, params),
 *    params: {
 *      // Custom object with desired structure and data
 *    }
 * }</pre>
 *
 *    * **timeLapse:** Object with start and end properties to define in seconds when this timeline is active.\n
 *    * **onEnter:** Callback function that will be called when progress reaches a cue point or being outside a cue point user seeks to a cue point manually.
 *    * **onLeave:** Callback function that will be called when user seeks and the new time doesn't reach to the timeLapse.start property.
 *    * **onUpdate:** Callback function that will be called when the progress is in the middle of timeLapse.start and timeLapse.end.
 *    * **onComplete:** Callback function that will be called when the progress is bigger than timeLapse.end.
 *    * **params:** Custom object with data to pass to the callbacks.
 *
 * - isFullScreen: Boolean value to know if we’re in fullscreen mode.
 * - currentState: String value with “play”, “pause” or “stop”.
 * - currentTime: Number value with current media time progress.
 * - totalTime: Number value with total media time.
 * - timeLeft: Number value with current media time left.
 * - volume: Number value with current volume between 0 and 1.
 * - playback: Number value with current playback between 0 and 2.
 * - bufferEnd: Number value with latest buffer point in milliseconds.
 * - buffered: Array of TimeRanges objects that represents current buffer state.
 *
 */
angular.module('com.2fdevs.videogular').controller('vgController', [
  '$scope',
  '$window',
  'vgConfigLoader',
  'vgFullscreen',
  'VG_UTILS',
  'VG_STATES',
  'VG_VOLUME_KEY',
  function ($scope, $window, vgConfigLoader, vgFullscreen, VG_UTILS, VG_STATES, VG_VOLUME_KEY) {
    var currentTheme = null;
    var isFullScreenPressed = false;
    var isMetaDataLoaded = false;
    var hasStartTimePlayed = false;
    var isVirtualClip = false;
    // PUBLIC $API
    this.videogularElement = null;
    this.clearMedia = function () {
      this.mediaElement[0].src = '';
      this.mediaElement[0].removeEventListener('canplay', this.onCanPlay.bind(this), false);
      this.mediaElement[0].removeEventListener('loadedmetadata', this.onLoadMetaData.bind(this), false);
      this.mediaElement[0].removeEventListener('waiting', this.onStartBuffering.bind(this), false);
      this.mediaElement[0].removeEventListener('ended', this.onComplete.bind(this), false);
      this.mediaElement[0].removeEventListener('playing', this.onStartPlaying.bind(this), false);
      this.mediaElement[0].removeEventListener('play', this.onPlay.bind(this), false);
      this.mediaElement[0].removeEventListener('pause', this.onPause.bind(this), false);
      this.mediaElement[0].removeEventListener('volumechange', this.onVolumeChange.bind(this), false);
      this.mediaElement[0].removeEventListener('playbackchange', this.onPlaybackChange.bind(this), false);
      this.mediaElement[0].removeEventListener('timeupdate', this.onUpdateTime.bind(this), false);
      this.mediaElement[0].removeEventListener('progress', this.onProgress.bind(this), false);
      this.mediaElement[0].removeEventListener('seeking', this.onSeeking.bind(this), false);
      this.mediaElement[0].removeEventListener('seeked', this.onSeeked.bind(this), false);
      this.mediaElement[0].removeEventListener('error', this.onVideoError.bind(this), false);
    };
    this.onRouteChange = function () {
      if (this.clearMediaOnNavigate === undefined || this.clearMediaOnNavigate === true) {
        this.clearMedia();
      }
    };
    this.onCanPlay = function (evt) {
      this.isBuffering = false;
      $scope.$parent.$digest($scope.vgCanPlay({ $event: evt }));
      if (!hasStartTimePlayed && (this.startTime > 0 || this.startTime === 0)) {
        this.seekTime(this.startTime);
        hasStartTimePlayed = true;
      }
    };
    this.onVideoReady = function () {
      this.isReady = true;
      this.autoPlay = $scope.vgAutoPlay;
      this.playsInline = $scope.vgPlaysInline;
      this.nativeFullscreen = $scope.vgNativeFullscreen || true;
      this.cuePoints = $scope.vgCuePoints;
      this.startTime = $scope.vgStartTime;
      this.virtualClipDuration = $scope.vgVirtualClipDuration;
      this.clearMediaOnNavigate = $scope.vgClearMediaOnNavigate || true;
      this.currentState = VG_STATES.STOP;
      isMetaDataLoaded = true;
      isVirtualClip = this.startTime >= 0 && this.virtualClipDuration > 0;
      //Set media volume from localStorage if available
      if (VG_UTILS.supportsLocalStorage()) {
        //Default to 100% volume if local storage setting does not exist.
        this.setVolume(parseFloat($window.localStorage.getItem(VG_VOLUME_KEY) || '1'));
      }
      if ($scope.vgConfig) {
        vgConfigLoader.loadConfig($scope.vgConfig).then(this.onLoadConfig.bind(this));
      } else {
        $scope.vgPlayerReady({ $API: this });
      }
    };
    this.onLoadConfig = function (config) {
      this.config = config;
      $scope.vgTheme = this.config.theme;
      $scope.vgAutoPlay = this.config.autoPlay;
      $scope.vgPlaysInline = this.config.playsInline;
      $scope.vgNativeFullscreen = this.config.nativeFullscreen;
      $scope.vgCuePoints = this.config.cuePoints;
      $scope.vgClearMediaOnNavigate = this.config.clearMediaOnNavigate;
      $scope.vgStartTime = this.config.startTime;
      $scope.vgVirtualClipDuration = this.config.virtualClipDuration;
      isVirtualClip = $scope.vgStartTime >= 0 && $scope.vgVirtualClipDuration > 0;
      $scope.vgPlayerReady({ $API: this });
    };
    this.onLoadMetaData = function (evt) {
      this.isBuffering = false;
      this.onUpdateTime(evt);
    };
    this.onProgress = function (event) {
      this.updateBuffer(event);
      $scope.$parent.$digest();
    };
    this.updateBuffer = function getBuffer(event) {
      if (event.target.buffered.length) {
        this.buffered = event.target.buffered;
        this.bufferEnd = 1000 * event.target.buffered.end(event.target.buffered.length - 1);
        // Avoid bufferEnd overflow by virtual clips
        if (this.bufferEnd > this.totalTime)
          this.bufferEnd = this.totalTime;
      }
    };
    this.onUpdateTime = function (event) {
      var targetTime = 1000 * event.target.currentTime;
      this.updateBuffer(event);
      if (event.target.duration != Infinity && event.target.duration != null && event.target.duration != undefined && event.target.duration != 1.7976931348623157e+308) {
        // Fake the duration and current time for virtual clips
        if (isVirtualClip) {
          if (hasStartTimePlayed && (event.target.currentTime < this.startTime || event.target.currentTime - this.startTime > this.virtualClipDuration)) {
            this.onComplete();
          } else {
            this.currentTime = Math.max(0, targetTime - 1000 * this.startTime);
            this.totalTime = 1000 * this.virtualClipDuration;
            this.timeLeft = 1000 * this.virtualClipDuration - this.currentTime;
          }
        } else {
          this.currentTime = targetTime;
          this.totalTime = 1000 * event.target.duration;
          this.timeLeft = 1000 * (event.target.duration - event.target.currentTime);
        }
        this.isLive = false;
      } else {
        // It's a live streaming without and end
        this.currentTime = targetTime;
        this.isLive = true;
      }
      var targetSeconds = isVirtualClip ? this.currentTime / 1000 : event.target.currentTime;
      var targetDuration = isVirtualClip ? this.totalTime / 1000 : event.target.duration;
      if (this.cuePoints) {
        this.checkCuePoints(targetSeconds);
      }
      $scope.vgUpdateTime({
        $currentTime: targetSeconds,
        $duration: targetDuration
      });
      // Safe apply just in case we're calling from a non-event
      if ($scope.$$phase != '$apply' && $scope.$$phase != '$digest') {
        $scope.$parent.$digest();
      }
    };
    this.checkCuePoints = function checkCuePoints(currentTime) {
      for (var tl in this.cuePoints) {
        for (var i = 0, l = this.cuePoints[tl].length; i < l; i++) {
          var cp = this.cuePoints[tl][i];
          var currentSecond = parseInt(currentTime, 10);
          var start = parseInt(cp.timeLapse.start, 10);
          // If timeLapse.end is not defined we set it as 1 second length
          if (!cp.timeLapse.end)
            cp.timeLapse.end = cp.timeLapse.start + 1;
          if (currentTime < cp.timeLapse.end)
            cp.$$isCompleted = false;
          // Fire the onEnter event once reach to the cue point
          if (!cp.$$isDirty && currentSecond === start && typeof cp.onEnter == 'function') {
            cp.onEnter(currentTime, cp.timeLapse, cp.params);
            cp.$$isDirty = true;
          }
          // Check if we've been reached to the cue point
          if (currentTime > cp.timeLapse.start) {
            // We're in the timelapse
            if (currentTime < cp.timeLapse.end) {
              // Trigger onUpdate each time we enter here
              if (cp.onUpdate)
                cp.onUpdate(currentTime, cp.timeLapse, cp.params);
              // Trigger onEnter if we enter on the cue point by manually seeking
              if (!cp.$$isDirty && typeof cp.onEnter === 'function') {
                cp.onEnter(currentTime, cp.timeLapse, cp.params);
              }
            }
            // We've been passed the cue point
            if (currentTime >= cp.timeLapse.end) {
              if (cp.onComplete && !cp.$$isCompleted) {
                cp.$$isCompleted = true;
                cp.onComplete(currentTime, cp.timeLapse, cp.params);
              }
            }
            cp.$$isDirty = true;
          } else {
            if (cp.onLeave && cp.$$isDirty) {
              cp.onLeave(currentTime, cp.timeLapse, cp.params);
            }
            cp.$$isDirty = false;
          }
        }
      }
    };
    this.onPlay = function () {
      this.setState(VG_STATES.PLAY);
      $scope.$parent.$digest();
    };
    this.onPause = function () {
      var currentTime = isVirtualClip ? this.currentTime : this.mediaElement[0].currentTime;
      if (currentTime == 0) {
        this.setState(VG_STATES.STOP);
      } else {
        this.setState(VG_STATES.PAUSE);
      }
      $scope.$parent.$digest();
    };
    this.onVolumeChange = function () {
      this.volume = this.mediaElement[0].volume;
      $scope.$parent.$digest();
    };
    this.onPlaybackChange = function () {
      this.playback = this.mediaElement[0].playbackRate;
      $scope.$parent.$digest();
    };
    this.onSeeking = function (event) {
      $scope.vgSeeking({
        $currentTime: event.target.currentTime,
        $duration: event.target.duration
      });
    };
    this.onSeeked = function (event) {
      $scope.vgSeeked({
        $currentTime: event.target.currentTime,
        $duration: event.target.duration
      });
    };
    this.seekTime = function (value, byPercent) {
      var second;
      if (byPercent) {
        if (isVirtualClip) {
          value = Math.max(0, Math.min(value, 100));
          second = value * this.virtualClipDuration / 100;
          this.mediaElement[0].currentTime = this.startTime + second;
        } else {
          second = value * this.mediaElement[0].duration / 100;
          this.mediaElement[0].currentTime = second;
        }
      } else {
        if (isVirtualClip) {
          var durationPercent = value / this.mediaElement[0].duration;
          second = !hasStartTimePlayed ? 0 : this.virtualClipDuration * durationPercent;
          this.mediaElement[0].currentTime = !hasStartTimePlayed ? this.startTime : this.startTime + second;
        } else {
          second = value;
          this.mediaElement[0].currentTime = second;
        }
      }
      this.currentTime = 1000 * second;
    };
    this.playPause = function () {
      if (this.mediaElement[0].paused) {
        this.play();
      } else {
        this.pause();
      }
    };
    this.setState = function (newState) {
      if (newState && newState != this.currentState) {
        $scope.vgUpdateState({ $state: newState });
        this.currentState = newState;
      }
      return this.currentState;
    };
    this.play = function () {
      this.mediaElement[0].play();
      this.setState(VG_STATES.PLAY);
    };
    this.pause = function () {
      this.mediaElement[0].pause();
      this.setState(VG_STATES.PAUSE);
    };
    this.stop = function () {
      try {
        this.mediaElement[0].pause();
        var targetTime = isVirtualClip ? this.startTime : 0;
        this.mediaElement[0].currentTime = targetTime;
        this.currentTime = targetTime;
        this.buffered = [];
        this.bufferEnd = 0;
        this.setState(VG_STATES.STOP);
      } catch (e) {
        return e;
      }
    };
    this.toggleFullScreen = function () {
      // There is no native full screen support or we want to play inline
      if (!vgFullscreen.isAvailable || !this.nativeFullscreen) {
        if (this.isFullScreen) {
          this.videogularElement.removeClass('fullscreen');
          this.videogularElement.css('z-index', 'auto');
        } else {
          this.videogularElement.addClass('fullscreen');
          this.videogularElement.css('z-index', VG_UTILS.getZIndex());
        }
        this.isFullScreen = !this.isFullScreen;
      }  // Perform native full screen support
      else {
        if (this.isFullScreen) {
          if (!VG_UTILS.isMobileDevice()) {
            vgFullscreen.exit();
          }
        } else {
          // On mobile devices we should make fullscreen only the video object
          if (VG_UTILS.isMobileDevice()) {
            // On iOS we should check if user pressed before fullscreen button
            // and also if metadata is loaded
            if (VG_UTILS.isiOSDevice()) {
              if (isMetaDataLoaded) {
                this.enterElementInFullScreen(this.mediaElement[0]);
              } else {
                isFullScreenPressed = true;
                this.play();
              }
            } else {
              this.enterElementInFullScreen(this.mediaElement[0]);
            }
          } else {
            this.enterElementInFullScreen(this.videogularElement[0]);
          }
        }
      }
    };
    this.enterElementInFullScreen = function (element) {
      vgFullscreen.request(element);
    };
    this.changeSource = function (newValue) {
      $scope.vgChangeSource({ $source: newValue });
    };
    this.setVolume = function (newVolume) {
      newVolume = Math.max(Math.min(newVolume, 1), 0);
      $scope.vgUpdateVolume({ $volume: newVolume });
      this.mediaElement[0].volume = newVolume;
      this.volume = newVolume;
      //Push volume updates to localStorage so that future instances resume volume
      if (VG_UTILS.supportsLocalStorage()) {
        //TODO: Improvement: concat key with current page or "video player id" to create separate stored volumes.
        $window.localStorage.setItem(VG_VOLUME_KEY, newVolume.toString());
      }
    };
    this.setPlayback = function (newPlayback) {
      $scope.vgUpdatePlayback({ $playBack: newPlayback });
      this.mediaElement[0].playbackRate = newPlayback;
      this.playback = newPlayback;
    };
    this.updateTheme = function (value) {
      var links = document.getElementsByTagName('link');
      var i;
      var l;
      // Remove previous theme
      if (currentTheme) {
        for (i = 0, l = links.length; i < l; i++) {
          if (links[i].outerHTML.indexOf(currentTheme) >= 0) {
            links[i].parentNode.removeChild(links[i]);
            break;
          }
        }
      }
      if (value) {
        var headElem = angular.element(document).find('head');
        var exists = false;
        // Look if theme already exists
        for (i = 0, l = links.length; i < l; i++) {
          exists = links[i].outerHTML.indexOf(value) >= 0;
          if (exists)
            break;
        }
        if (!exists) {
          headElem.append('<link rel=\'stylesheet\' href=\'' + value + '\'>');
        }
        currentTheme = value;
      }
    };
    this.onStartBuffering = function (event) {
      this.isBuffering = true;
      $scope.$parent.$digest();
    };
    this.onStartPlaying = function (event) {
      this.isBuffering = false;
      $scope.$parent.$digest();
    };
    this.onComplete = function (event) {
      $scope.vgComplete();
      this.setState(VG_STATES.STOP);
      this.isCompleted = true;
      if (isVirtualClip) {
        this.stop();
      }
      $scope.$parent.$digest();
    };
    this.onVideoError = function (event) {
      $scope.vgError({ $event: event });
    };
    this.addListeners = function () {
      this.mediaElement[0].addEventListener('canplay', this.onCanPlay.bind(this), false);
      this.mediaElement[0].addEventListener('loadedmetadata', this.onLoadMetaData.bind(this), false);
      this.mediaElement[0].addEventListener('waiting', this.onStartBuffering.bind(this), false);
      this.mediaElement[0].addEventListener('ended', this.onComplete.bind(this), false);
      this.mediaElement[0].addEventListener('playing', this.onStartPlaying.bind(this), false);
      this.mediaElement[0].addEventListener('play', this.onPlay.bind(this), false);
      this.mediaElement[0].addEventListener('pause', this.onPause.bind(this), false);
      this.mediaElement[0].addEventListener('volumechange', this.onVolumeChange.bind(this), false);
      this.mediaElement[0].addEventListener('playbackchange', this.onPlaybackChange.bind(this), false);
      this.mediaElement[0].addEventListener('timeupdate', this.onUpdateTime.bind(this), false);
      this.mediaElement[0].addEventListener('progress', this.onProgress.bind(this), false);
      this.mediaElement[0].addEventListener('seeking', this.onSeeking.bind(this), false);
      this.mediaElement[0].addEventListener('seeked', this.onSeeked.bind(this), false);
      this.mediaElement[0].addEventListener('error', this.onVideoError.bind(this), false);
    };
    this.init = function () {
      this.isReady = false;
      this.isCompleted = false;
      this.buffered = [];
      this.bufferEnd = 0;
      this.currentTime = 0;
      this.totalTime = 0;
      this.timeLeft = 0;
      this.isLive = false;
      this.isFullScreen = false;
      this.playback = 1;
      this.isConfig = $scope.vgConfig != undefined;
      this.mediaElement = [{
          play: function () {
          },
          pause: function () {
          },
          stop: function () {
          },
          addEventListener: function () {
          },
          removeEventListener: function () {
          }
        }];
      if (vgFullscreen.isAvailable) {
        this.isFullScreen = vgFullscreen.isFullScreen();
      }
      this.updateTheme($scope.vgTheme);
      this.addBindings();
      if (vgFullscreen.isAvailable) {
        document.addEventListener(vgFullscreen.onchange, this.onFullScreenChange.bind(this));
      }
    };
    this.onUpdateTheme = function onUpdateTheme(newValue) {
      this.updateTheme(newValue);
    };
    this.onUpdateAutoPlay = function onUpdateAutoPlay(newValue) {
      if (newValue && !this.autoPlay) {
        this.autoPlay = newValue;
        this.play(this);
      }
    };
    this.onUpdateStartTime = function onUpdateStartTime(newValue) {
      if (newValue && newValue != this.startTime) {
        this.mediaElement[0].currentTime = newValue;
        this.startTime = newValue;
        isVirtualClip = this.startTime >= 0 && this.virtualClipDuration > 0;
        var fakeEvent = { target: this.mediaElement[0] };
        this.onUpdateTime(fakeEvent, true);
      }
    };
    this.onUpdateVirtualClipDuration = function onUpdateVirtualClipDuration(newValue) {
      if (newValue && newValue != this.virtualClipDuration) {
        this.virtualClipDuration = newValue;
        isVirtualClip = this.startTime >= 0 && this.virtualClipDuration > 0;
        var fakeEvent = { target: this.mediaElement[0] };
        this.onUpdateTime(fakeEvent, true);
      }
    };
    this.onUpdatePlaysInline = function onUpdatePlaysInline(newValue) {
      this.playsInline = newValue;
    };
    this.onUpdateNativeFullscreen = function onUpdateNativeFullscreen(newValue) {
      if (newValue == undefined)
        newValue = true;
      this.nativeFullscreen = newValue;
    };
    this.onUpdateCuePoints = function onUpdateCuePoints(newValue) {
      this.cuePoints = newValue;
      this.checkCuePoints(this.currentTime);
    };
    this.onUpdateClearMediaOnNavigate = function onUpdateClearMediaOnNavigate(newValue) {
      this.clearMediaOnNavigate = newValue;
    };
    this.addBindings = function () {
      $scope.$watch('vgTheme', this.onUpdateTheme.bind(this));
      $scope.$watch('vgAutoPlay', this.onUpdateAutoPlay.bind(this));
      $scope.$watch('vgStartTime', this.onUpdateStartTime.bind(this));
      $scope.$watch('vgVirtualClipDuration', this.onUpdateVirtualClipDuration.bind(this));
      $scope.$watch('vgPlaysInline', this.onUpdatePlaysInline.bind(this));
      $scope.$watch('vgNativeFullscreen', this.onUpdateNativeFullscreen.bind(this));
      $scope.$watch('vgCuePoints', this.onUpdateCuePoints.bind(this));
      $scope.$watch('vgClearMediaOnNavigate', this.onUpdateClearMediaOnNavigate.bind(this));
    };
    this.onFullScreenChange = function (event) {
      this.isFullScreen = vgFullscreen.isFullScreen();
      $scope.$parent.$digest();
    };
    // Empty mediaElement on destroy to avoid that Chrome downloads video even when it's not present
    $scope.$on('$destroy', this.clearMedia.bind(this));
    // Empty mediaElement when router changes
    $scope.$on('$routeChangeStart', this.onRouteChange.bind(this));
    this.init();
  }
]);
/**
 * @ngdoc directive
 * @name com.2fdevs.videogular.directive:vgCrossorigin
 * @restrict A
 * @description
 * Optional directive for `vg-media` to add or remove a crossorigin policy to the video object. Possible values are: "anonymous" and "use-credentials".
 * This feature should be enabled if you want to have your subtitles or video files on a different domain than the video player. Additionally you need
 * to add CORS policies to your video and track files to your server to make it work.
 *
 */
'use strict';
angular.module('com.2fdevs.videogular').directive('vgCrossorigin', [function () {
    return {
      restrict: 'A',
      require: '^videogular',
      link: {
        pre: function (scope, elem, attr, API) {
          var crossorigin;
          scope.setCrossorigin = function setCrossorigin(value) {
            if (value) {
              API.mediaElement.attr('crossorigin', value);
            } else {
              API.mediaElement.removeAttr('crossorigin');
            }
          };
          if (API.isConfig) {
            scope.$watch(function () {
              return API.config;
            }, function () {
              if (API.config) {
                scope.setCrossorigin(API.config.crossorigin);
              }
            });
          } else {
            scope.$watch(attr.vgCrossorigin, function (newValue, oldValue) {
              if ((!crossorigin || newValue != oldValue) && newValue) {
                crossorigin = newValue;
                scope.setCrossorigin(crossorigin);
              } else {
                scope.setCrossorigin();
              }
            });
          }
        }
      }
    };
  }]);
/**
 * @ngdoc directive
 * @name com.2fdevs.videogular.directive:vgLoop
 * @restrict A
 * @description
 * Optional directive for `vg-media` to add or remove loop in media files. Possible values are: "true" and "false"
 *
 */
'use strict';
angular.module('com.2fdevs.videogular').directive('vgLoop', [function () {
    return {
      restrict: 'A',
      require: '^videogular',
      link: {
        pre: function (scope, elem, attr, API) {
          var loop;
          scope.setLoop = function setLoop(value) {
            if (value) {
              API.mediaElement.attr('loop', value);
            } else {
              API.mediaElement.removeAttr('loop');
            }
          };
          if (API.isConfig) {
            scope.$watch(function () {
              return API.config;
            }, function () {
              if (API.config) {
                scope.setLoop(API.config.loop);
              }
            });
          } else {
            scope.$watch(attr.vgLoop, function (newValue, oldValue) {
              if ((!loop || newValue != oldValue) && newValue) {
                loop = newValue;
                scope.setLoop(loop);
              } else {
                scope.setLoop();
              }
            });
          }
        }
      }
    };
  }]);
/**
 * @ngdoc directive
 * @name com.2fdevs.videogular.direcitve:vgMedia
 * @restrict E
 * @description
 * Directive to add a source of videos or audios. This directive will create a &lt;video&gt; or &lt;audio&gt; tag and usually will be above plugin tags.
 *
 * @param {array} vgSrc Bindable array with a list of media sources or a simple url string. A media source is an object with two properties `src` and `type`. The `src` property must contains a trustful url resource.
 * @param {string} vgType String with "video" or "audio" values to set a <video> or <audio> tag inside <vg-media>.
 * <pre>
 * {
 *    src: $sce.trustAsResourceUrl("path/to/video/videogular.mp4"),
 *    type: "video/mp4"
 * }
 * </pre>
 *
 */
'use strict';
angular.module('com.2fdevs.videogular').directive('vgMedia', [
  '$timeout',
  'VG_UTILS',
  'VG_STATES',
  function ($timeout, VG_UTILS, VG_STATES) {
    return {
      restrict: 'E',
      require: '^videogular',
      templateUrl: function (elem, attrs) {
        var vgType = attrs.vgType || 'video';
        return attrs.vgTemplate || 'vg-templates/vg-media-' + vgType;
      },
      scope: {
        vgSrc: '=?',
        vgType: '=?'
      },
      link: function (scope, elem, attrs, API) {
        var sources;
        // what type of media do we want? defaults to 'video'
        if (!attrs.vgType || attrs.vgType === 'video') {
          attrs.vgType = 'video';
        } else {
          attrs.vgType = 'audio';
        }
        // FUNCTIONS
        scope.onChangeSource = function onChangeSource(newValue, oldValue) {
          if ((!sources || newValue != oldValue) && newValue) {
            sources = newValue;
            if (API.currentState !== VG_STATES.PLAY) {
              API.currentState = VG_STATES.STOP;
            }
            API.sources = sources;
            scope.changeSource();
          }
        };
        scope.changeSource = function changeSource() {
          if (angular.isArray(sources)) {
            var canPlay = '';
            // It's a cool browser
            if (API.mediaElement[0].canPlayType) {
              for (var i = 0, l = sources.length; i < l; i++) {
                canPlay = API.mediaElement[0].canPlayType(sources[i].type);
                if (canPlay == 'maybe' || canPlay == 'probably') {
                  API.mediaElement.attr('src', sources[i].src);
                  API.mediaElement.attr('type', sources[i].type);
                  //Trigger vgChangeSource($source) API callback in vgController
                  API.changeSource(sources[i]);
                  break;
                }
              }
            }  // It's a crappy browser and it doesn't deserve any respect
            else {
              // Get H264 or the first one
              API.mediaElement.attr('src', sources[0].src);
              API.mediaElement.attr('type', sources[0].type);
              //Trigger vgChangeSource($source) API callback in vgController
              API.changeSource(sources[0]);
            }
          } else {
            API.mediaElement.attr('src', sources);
            //Trigger vgChangeSource($source) API callback in vgController
            API.changeSource(sources);
          }
          // Android 2.3 support: https://github.com/2fdevs/videogular/issues/187
          if (VG_UTILS.isMobileDevice())
            API.mediaElement[0].load();
          $timeout(function () {
            if (API.autoPlay && (VG_UTILS.isCordova() || !VG_UTILS.isMobileDevice())) {
              API.play();
            }
          });
          if (canPlay == '') {
            API.onVideoError();
          }
        };
        // INIT
        API.mediaElement = elem.find(attrs.vgType);
        API.sources = scope.vgSrc;
        API.addListeners();
        API.onVideoReady();
        scope.$watch('vgSrc', scope.onChangeSource);
        scope.$watch(function () {
          return API.sources;
        }, scope.onChangeSource);
        scope.$watch(function () {
          return API.playsInline;
        }, function (newValue, oldValue) {
          if (newValue)
            API.mediaElement.attr('webkit-playsinline', '');
          else
            API.mediaElement.removeAttr('webkit-playsinline');
        });
        if (API.isConfig) {
          scope.$watch(function () {
            return API.config;
          }, function () {
            if (API.config) {
              scope.vgSrc = API.config.sources;
            }
          });
        }
      }
    };
  }
]);
/**
 * @ngdoc directive
 * @name com.2fdevs.videogular.directive:vgNativeControls
 * @restrict A
 * @description
 * Optional directive for `vg-media` to add or remove the native controls. Possible values are: "true" and "false"
 *
 */
'use strict';
angular.module('com.2fdevs.videogular').directive('vgNativeControls', [function () {
    return {
      restrict: 'A',
      require: '^videogular',
      link: {
        pre: function (scope, elem, attr, API) {
          var controls;
          scope.setControls = function setControls(value) {
            if (value) {
              API.mediaElement.attr('controls', value);
            } else {
              API.mediaElement.removeAttr('controls');
            }
          };
          if (API.isConfig) {
            scope.$watch(function () {
              return API.config;
            }, function () {
              if (API.config) {
                scope.setControls(API.config.controls);
              }
            });
          } else {
            scope.$watch(attr.vgNativeControls, function (newValue, oldValue) {
              if ((!controls || newValue != oldValue) && newValue) {
                controls = newValue;
                scope.setControls(controls);
              } else {
                scope.setControls();
              }
            });
          }
        }
      }
    };
  }]);
/**
 * @ngdoc directive
 * @name com.2fdevs.videogular.directive:vgPreload
 * @restrict A
 * @description
 * Optional directive for `vg-media` to preload media files. Possible values are: "auto", "none" and "preload"
 *
 */
'use strict';
angular.module('com.2fdevs.videogular').directive('vgPreload', [function () {
    return {
      restrict: 'A',
      require: '^videogular',
      link: {
        pre: function (scope, elem, attr, API) {
          var preload;
          scope.setPreload = function setPreload(value) {
            if (value) {
              API.mediaElement.attr('preload', value);
            } else {
              API.mediaElement.removeAttr('preload');
            }
          };
          if (API.isConfig) {
            scope.$watch(function () {
              return API.config;
            }, function () {
              if (API.config) {
                scope.setPreload(API.config.preload);
              }
            });
          } else {
            scope.$watch(attr.vgPreload, function (newValue, oldValue) {
              if ((!preload || newValue != oldValue) && newValue) {
                preload = newValue;
                scope.setPreload(preload);
              } else {
                scope.setPreload();
              }
            });
          }
        }
      }
    };
  }]);
/**
 * @ngdoc directive
 * @name com.2fdevs.videogular.directive:vgTracks
 * @restrict A
 * @description
 * Optional directive for `vg-media` to add a list of tracks.
 *
 * vgTracks Bindable array with a list of subtitles sources. A track source is an object with five properties: src, kind, srclang, label and default.
 * <pre>
 * {
 *    src: "assets/subs/pale-blue-dot.vtt",
 *    kind: "subtitles",
 *    srclang: "en",
 *    label: "English",
 *    default: "true/false"
 * }
 * </pre>
 */
'use strict';
angular.module('com.2fdevs.videogular').directive('vgTracks', [function () {
    return {
      restrict: 'A',
      require: '^videogular',
      link: {
        pre: function (scope, elem, attr, API) {
          var isMetaDataLoaded = false;
          var tracks;
          var i;
          var l;
          scope.onLoadMetaData = function () {
            isMetaDataLoaded = true;
            scope.updateTracks();
          };
          scope.updateTracks = function () {
            // Remove previous tracks
            var oldTracks = API.mediaElement.children();
            for (i = 0, l = oldTracks.length; i < l; i++) {
              if (oldTracks[i].remove) {
                oldTracks[i].remove();
              }
            }
            // Add new tracks
            if (tracks) {
              for (i = 0, l = tracks.length; i < l; i++) {
                var track = document.createElement('track');
                for (var prop in tracks[i]) {
                  track[prop] = tracks[i][prop];
                }
                track.addEventListener('load', scope.onLoadTrack.bind(scope, track));
                API.mediaElement[0].appendChild(track);
              }
            }
          };
          scope.onLoadTrack = function (track) {
            if (track.default)
              track.mode = 'showing';
            else
              track.mode = 'hidden';
            for (var i = 0, l = API.mediaElement[0].textTracks.length; i < l; i++) {
              if (track.label == API.mediaElement[0].textTracks[i].label) {
                if (track.default) {
                  API.mediaElement[0].textTracks[i].mode = 'showing';
                } else {
                  API.mediaElement[0].textTracks[i].mode = 'disabled';
                }
              }
            }
            track.removeEventListener('load', scope.onLoadTrack.bind(scope, track));
          };
          scope.setTracks = function setTracks(value) {
            // Add tracks to the API to have it available for other plugins (like controls)
            tracks = value;
            API.tracks = value;
            if (isMetaDataLoaded) {
              scope.updateTracks();
            } else {
              API.mediaElement[0].addEventListener('loadedmetadata', scope.onLoadMetaData.bind(scope), false);
            }
          };
          if (API.isConfig) {
            scope.$watch(function () {
              return API.config;
            }, function () {
              if (API.config) {
                scope.setTracks(API.config.tracks);
              }
            });
          } else {
            scope.$watch(attr.vgTracks, function (newValue, oldValue) {
              if (!tracks || newValue != oldValue) {
                scope.setTracks(newValue);
              }
            }, true);
          }
        }
      }
    };
  }]);
/**
 * @ngdoc directive
 * @name com.2fdevs.videogular.directive:videogular
 * @restrict E
 * @description
 * Main directive that must wrap a &lt;vg-media&gt; tag and all plugins.
 *
 * &lt;video&gt; tag usually will be above plugin tags, that's because plugins should be in a layer over the &lt;video&gt;.
 *
 * @param {string} vgTheme String with a scope name variable. This directive will inject a CSS link in the header of your page.
 * **This parameter is required.**
 *
 * @param {boolean} [vgPlaysInline=false] vgPlaysInline Boolean value or a String with a scope name variable to use native fullscreen (default) or set fullscreen inside browser (true).
 *
 * @param {boolean} [vgClearMediaOnNavigate=true] vgClearMediaOnNavigate Boolean value or a String with a scope name variable to reset the video player when user navigates.
 *
 * This is useful to allow continuous playback between different routes.
 *
 * @param {boolean} [vgAutoPlay=false] vgAutoPlay Boolean value or a String with a scope name variable to auto start playing video when it is initialized.
 *
 * **This parameter is disabled in mobile devices** because user must click on content to prevent consuming mobile data plans.
 *
 * @param {boolean} [vgStartTime=-1] vgStartTime Number value or a String with a scope name variable to start playing the video at a certain time.
 *
 * @param {boolean} [vgVirtualClipDuration=-1] vgVirtualClipDuration Number value or a String with a scope name variable for a length to limit the video playback to.
 *
 * @param {object} vgCuePoints Bindable object containing a list of timelines with cue points objects. A timeline is an array of objects with the following properties:
 * - `timeLapse` is an object with two properties `start` and `end` representing in seconds the period for this cue points.
 * - `onEnter` callback called when user enters on a cue point. callback(currentTime, timeLapse, params)
 * - `onLeave` callback called when user seeks backwards and leave the current cue point or a completed cue point. callback(currentTime, timeLapse, params)
 * - `onUpdate` callback called when the current time is between timeLapse.start and timeLapse.end. callback(currentTime, timeLapse, params)
 * - `onComplete` callback called when the user seek forward or the current time passes timeLapse.end property. callback(currentTime, timeLapse, params)
 * - `params` an object with values available to receive in the callback..
 *
 * @param {function} vgConfig String with a url to a config file. Config file's must be a JSON file object with the following structure:
 * <pre>
 {
   "controls": false,
   "loop": false,
   "autoplay": false,
   "startTime": -1,
   "virtualClipDuration": -1,
   "preload": "auto",
   "theme": "path/to/videogular.css",
   "sources": [
     {
       "src": "path/to/videogular.mp4",
       "type": "video/mp4"
     },
     {
       "src": "path/to/videogular.webm",
       "type": "video/webm"
     },
     {
       "src": "path/to/videogular.ogg",
       "type": "video/ogg"
     }
   ],
   "tracks": [
     {
       "src": "path/to/pale-blue-dot.vtt",
       "kind": "subtitles",
       "srclang": "en",
       "label": "English",
       "default": ""
     }
   ],
   "plugins": {
     "controls": {
       "autohide": true,
       "autohideTime": 3000
     },
     "poster": {
       "url": "path/to/earth.png"
     },
     "ima-ads": {
       "companion": "companionAd",
       "companionSize": [728, 90],
       "network": "6062",
       "unitPath": "iab_vast_samples",
       "adTagUrl": "http://pubads.g.doubleclick.net/gampad/ads?sz=640x480&iu=%2F3510761%2FadRulesSampleTags&ciu_szs=160x600%2C300x250%2C728x90&cust_params=adrule%3Dpremidpostpodandbumpers&impl=s&gdfp_req=1&env=vp&ad_rule=1&vid=47570401&cmsid=481&output=xml_vast2&unviewed_position_start=1&url=[referrer_url]&correlator=[timestamp]",
       "skipButton": "<div class='skipButton'>skip ad</div>"
     },
     "analytics": {
       "category": "Videogular",
       "label": "Main",
       "events": {
         "ready": true,
         "play": true,
         "pause": true,
         "stop": true,
         "complete": true,
         "progress": 10
       }
     }
   }
 }
 * </pre>
 * @param {function} vgCanPlay Function name in controller's scope to call when video is able to begin playback
 * @param {function} vgComplete Function name in controller's scope to call when video have been completed.
 * @param {function} vgUpdateVolume Function name in controller's scope to call when volume changes. Receives a param with the new volume.
 * @param {function} vgUpdatePlayback Function name in controller's scope to call when playback changes. Receives a param with the new playback rate.
 * @param {function} vgUpdateTime Function name in controller's scope to call when video playback time is updated. Receives two params with current time and duration in milliseconds.
 * @param {function} vgUpdateState Function name in controller's scope to call when video state changes. Receives a param with the new state. Possible values are "play", "stop" or "pause".
 * @param {function} vgPlayerReady Function name in controller's scope to call when video have been initialized. Receives a param with the videogular API.
 * @param {function} vgChangeSource Function name in controller's scope to change current video source. Receives a param with the new video.
 * @param {function} vgPlaysInline Boolean to play video inline. Generally used in mobile devices.
 * @param {function} vgNativeFullscreen Boolean to disable native fullscreen.
 * @param {function} vgSeeking Function name in controller's scope to call when the video has finished jumping to a new time. Receives a param with the seeked time and duration in seconds.
 * @param {function} vgSeeked Function name in controller's scope to call when the video is jumping to a new time. Receives two params with the seeked time and duration in seconds.
 * @param {function} vgError Function name in controller's scope to receive an error from video object. Receives a param with the error event.
 * This is a free parameter and it could be values like "new.mp4", "320" or "sd". This will allow you to use this to change a video or video quality.
 * This callback will not change the video, you should do that by updating your sources scope variable.
 *
 */
'use strict';
angular.module('com.2fdevs.videogular').directive('videogular', [function () {
    return {
      restrict: 'EA',
      scope: {
        vgTheme: '=?',
        vgAutoPlay: '=?',
        vgStartTime: '=?',
        vgVirtualClipDuration: '=?',
        vgPlaysInline: '=?',
        vgNativeFullscreen: '=?',
        vgClearMediaOnNavigate: '=?',
        vgCuePoints: '=?',
        vgConfig: '@',
        vgCanPlay: '&',
        vgComplete: '&',
        vgUpdateVolume: '&',
        vgUpdatePlayback: '&',
        vgUpdateTime: '&',
        vgUpdateState: '&',
        vgPlayerReady: '&',
        vgChangeSource: '&',
        vgSeeking: '&',
        vgSeeked: '&',
        vgError: '&'
      },
      controller: 'vgController',
      controllerAs: 'API',
      link: {
        pre: function (scope, elem, attr, controller) {
          controller.videogularElement = angular.element(elem);
        }
      }
    };
  }]);
/**
 * @ngdoc service
 * @name com.2fdevs.videogular.service:vgConfigLoader
 *
 * @description
 * Config loader service:
 *
 * vgConfigLoader.loadConfig(url): Param `url` is a url to a config JSON.
 **/
'use strict';
angular.module('com.2fdevs.videogular').service('vgConfigLoader', [
  '$http',
  '$q',
  '$sce',
  function ($http, $q, $sce) {
    this.loadConfig = function loadConfig(url) {
      var deferred = $q.defer();
      $http({
        method: 'GET',
        url: url
      }).then(function success(response) {
        var result = response.data;
        for (var i = 0, l = result.sources.length; i < l; i++) {
          result.sources[i].src = $sce.trustAsResourceUrl(result.sources[i].src);
        }
        deferred.resolve(result);
      }, function reject() {
        deferred.reject();
      });
      return deferred.promise;
    };
  }
]);
/**
 * @ngdoc service
 * @name com.2fdevs.videogular.service:vgFullscreen
 *
 * @description
 * Native fullscreen polyfill service.
 *
 *    * vgFullscreen.onchange: String with the onchange event name.
 *    * vgFullscreen.onerror: String with the onerror event name.
 *    * vgFullscreen.isAvailable: Boolean with fullscreen availability.
 *    * vgFullscreen.isFullScreen: Boolean with current view mode.
 *    * vgFullscreen.exit: Exit fullscreen function.
 *    * vgFullscreen.request: Request for fullscreen access function.
 **/
'use strict';
angular.module('com.2fdevs.videogular').service('vgFullscreen', [
  'VG_UTILS',
  function (VG_UTILS) {
    // Native fullscreen polyfill
    var element;
    var polyfill = null;
    var APIs = {
        w3: {
          enabled: 'fullscreenEnabled',
          element: 'fullscreenElement',
          request: 'requestFullscreen',
          exit: 'exitFullscreen',
          onchange: 'fullscreenchange',
          onerror: 'fullscreenerror'
        },
        newWebkit: {
          enabled: 'webkitFullscreenEnabled',
          element: 'webkitFullscreenElement',
          request: 'webkitRequestFullscreen',
          exit: 'webkitExitFullscreen',
          onchange: 'webkitfullscreenchange',
          onerror: 'webkitfullscreenerror'
        },
        oldWebkit: {
          enabled: 'webkitIsFullScreen',
          element: 'webkitCurrentFullScreenElement',
          request: 'webkitRequestFullScreen',
          exit: 'webkitCancelFullScreen',
          onchange: 'webkitfullscreenchange',
          onerror: 'webkitfullscreenerror'
        },
        moz: {
          enabled: 'mozFullScreen',
          element: 'mozFullScreenElement',
          request: 'mozRequestFullScreen',
          exit: 'mozCancelFullScreen',
          onchange: 'mozfullscreenchange',
          onerror: 'mozfullscreenerror'
        },
        ios: {
          enabled: 'webkitFullscreenEnabled',
          element: 'webkitFullscreenElement',
          request: 'webkitEnterFullscreen',
          exit: 'webkitExitFullscreen',
          onchange: 'webkitfullscreenchange',
          onerror: 'webkitfullscreenerror'
        },
        ms: {
          enabled: 'msFullscreenEnabled',
          element: 'msFullscreenElement',
          request: 'msRequestFullscreen',
          exit: 'msExitFullscreen',
          onchange: 'MSFullscreenChange',
          onerror: 'MSFullscreenError'
        }
      };
    for (var browser in APIs) {
      if (APIs[browser].enabled in document) {
        polyfill = APIs[browser];
        break;
      }
    }
    // Override APIs on iOS
    if (VG_UTILS.isiOSDevice()) {
      polyfill = APIs.ios;
    }
    function isFullScreen() {
      var result = false;
      if (element) {
        result = document[polyfill.element] != null || element.webkitDisplayingFullscreen;
      } else {
        result = document[polyfill.element] != null;
      }
      return result;
    }
    this.isAvailable = polyfill != null;
    if (polyfill) {
      this.onchange = polyfill.onchange;
      this.onerror = polyfill.onerror;
      this.isFullScreen = isFullScreen;
      this.exit = function () {
        document[polyfill.exit]();
      };
      this.request = function (elem) {
        element = elem;
        element[polyfill.request]();
      };
    }
  }
]);
'use strict';
angular.module('com.2fdevs.videogular').service('VG_UTILS', [
  '$window',
  function ($window) {
    this.fixEventOffset = function ($event) {
      /**
             * There's no offsetX in Firefox, so we fix that.
             * Solution provided by Jack Moore in this post:
             * http://www.jacklmoore.com/notes/mouse-position/
             * @param $event
             * @returns {*}
             */
      var matchedFF = navigator.userAgent.match(/Firefox\/(\d+)/i);
      if (matchedFF && Number.parseInt(matchedFF.pop()) < 39) {
        var style = $event.currentTarget.currentStyle || window.getComputedStyle($event.target, null);
        var borderLeftWidth = parseInt(style['borderLeftWidth'], 10);
        var borderTopWidth = parseInt(style['borderTopWidth'], 10);
        var rect = $event.currentTarget.getBoundingClientRect();
        var offsetX = $event.clientX - borderLeftWidth - rect.left;
        var offsetY = $event.clientY - borderTopWidth - rect.top;
        $event.offsetX = offsetX;
        $event.offsetY = offsetY;
      }
      return $event;
    };
    /**
         * Inspired by Paul Irish
         * https://gist.github.com/paulirish/211209
         * @returns {number}
         */
    this.getZIndex = function () {
      var zIndex = 1;
      var elementZIndex;
      var tags = document.getElementsByTagName('*');
      for (var i = 0, l = tags.length; i < l; i++) {
        elementZIndex = parseInt(window.getComputedStyle(tags[i])['z-index']);
        if (elementZIndex > zIndex) {
          zIndex = elementZIndex + 1;
        }
      }
      return zIndex;
    };
    // Very simple mobile detection, not 100% reliable
    this.isMobileDevice = function () {
      return typeof window.orientation !== 'undefined' || navigator.userAgent.indexOf('IEMobile') !== -1;
    };
    this.isiOSDevice = function () {
      return navigator.userAgent.match(/ip(hone|ad|od)/i) && !navigator.userAgent.match(/(iemobile)[\/\s]?([\w\.]*)/i);
    };
    this.isCordova = function () {
      return document.URL.indexOf('http://') === -1 && document.URL.indexOf('https://') === -1;
    };
    /**
         * Test the browser's support for HTML5 localStorage.
         * @returns {boolean}
         */
    this.supportsLocalStorage = function () {
      var testKey = 'videogular-test-key';
      try {
        var storage = $window.sessionStorage;
        storage.setItem(testKey, '1');
        storage.removeItem(testKey);
        return 'localStorage' in $window && $window['localStorage'] !== null;
      } catch (e) {
        return false;
      }
    };
  }
]);
/**
 * @license videogular v1.4.4 http://videogular.com
 * Two Fucking Developers http://twofuckingdevelopers.com
 * License: MIT
 */
/**
 * @ngdoc directive
 * @name com.2fdevs.videogular.plugins.poster.directive:vgPoster
 * @restrict E
 * @description
 * Shows an image when player hasn't been played or has been completed a video.
 * <pre>
 * <videogular vg-theme="config.theme.url" vg-autoplay="config.autoPlay">
 *    <vg-media vg-src="sources"></vg-media>
 *
 *    <vg-poster vg-url='config.plugins.poster.url'></vg-poster>
 * </videogular>
 * </pre>
 *
 * @param {string} vgUrl String with a scope name variable. URL to an image supported by the img tag.
 * **This parameter is required.**
 *
 *
 */
'use strict';
angular.module('com.2fdevs.videogular.plugins.poster', []).run([
  '$templateCache',
  function ($templateCache) {
    $templateCache.put('vg-templates/vg-poster', '<img ng-src="{{vgUrl}}" ng-class="API.currentState" role="presentation" alt="">');
  }
]).directive('vgPoster', [function () {
    return {
      restrict: 'E',
      require: '^videogular',
      scope: { vgUrl: '=?' },
      templateUrl: function (elem, attrs) {
        return attrs.vgTemplate || 'vg-templates/vg-poster';
      },
      link: function (scope, elem, attr, API) {
        scope.API = API;
        if (API.isConfig) {
          scope.$watch('API.config', function () {
            if (scope.API.config) {
              scope.vgUrl = scope.API.config.plugins.poster.url;
            }
          });
        }
      }
    };
  }]);
/* 
 * Angular JS Multi Select
 * Creates a dropdown-like button with checkboxes. 
 *
 * Project started on: Tue, 14 Jan 2014 - 5:18:02 PM
 * Current version: 2.0.2
 * 
 * Released under the MIT License
 * --------------------------------------------------------------------------------
 * The MIT License (MIT)
 *
 * Copyright (c) 2014 Ignatius Steven (https://github.com/isteven)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy 
 * of this software and associated documentation files (the "Software"), to deal 
 * in the Software without restriction, including without limitation the rights 
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
 * copies of the Software, and to permit persons to whom the Software is 
 * furnished to do so, subject to the following conditions: 
 *
 * The above copyright notice and this permission notice shall be included in all 
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
 * SOFTWARE.
 * --------------------------------------------------------------------------------
 */
angular.module('multi-select', ['ng']).directive('multiSelect', [
  '$sce',
  '$timeout',
  function ($sce, $timeout) {
    return {
      restrict: 'AE',
      replace: true,
      scope: {
        inputModel: '=',
        outputModel: '=',
        buttonLabel: '@',
        defaultLabel: '@',
        directiveId: '@',
        helperElements: '@',
        isDisabled: '=',
        itemLabel: '@',
        maxLabels: '@',
        orientation: '@',
        selectionMode: '@',
        tickProperty: '@',
        disableProperty: '@',
        groupProperty: '@',
        maxHeight: '@',
        onClose: '&',
        onItemClick: '&',
        onOpen: '&'
      },
      template: '<span class="multiSelect inlineBlock">' + '<button type="button" class="button multiSelectButton" ng-click="toggleCheckboxes( $event ); refreshSelectedItems(); refreshButton();" ng-bind-html="varButtonLabel">' + '</button>' + '<div class="checkboxLayer">' + '<form>' + '<div class="helperContainer" ng-if="displayHelper( \'filter\' ) || displayHelper( \'all\' ) || displayHelper( \'none\' ) || displayHelper( \'reset\' )">' + '<div class="line" ng-if="displayHelper( \'all\' ) || displayHelper( \'none\' ) || displayHelper( \'reset\' )">' + '<button type="button" ng-click="select( \'all\',   $event );"    class="helperButton" ng-if="!isDisabled && displayHelper( \'all\' )">   &#10003;&nbsp; Select All</button> ' + '<button type="button" ng-click="select( \'none\',  $event );"   class="helperButton" ng-if="!isDisabled && displayHelper( \'none\' )">  &times;&nbsp; Select None</button>' + '<button type="button" ng-click="select( \'reset\', $event );"  class="helperButton" ng-if="!isDisabled && displayHelper( \'reset\' )" style="float:right">&#8630;&nbsp; Reset</button>' + '</div>' + '<div class="line" style="position:relative" ng-if="displayHelper( \'filter\' )">' + '<input placeholder="Search..." type="text" ng-click="select( \'filter\', $event )" ng-model="inputLabel.labelFilter" ng-change="updateFilter();$scope.getFormElements();" class="inputFilter" />' + '<button type="button" class="clearButton" ng-click="inputLabel.labelFilter=\'\';updateFilter();prepareGrouping();prepareIndex();select( \'clear\', $event )">&times;</button> ' + '</div>' + '</div>' + '<div class="checkBoxContainer" style="{{setHeight();}}">' + '<div ng-repeat="item in filteredModel | filter:removeGroupEndMarker" class="multiSelectItem"' + 'ng-class="{selected: item[ tickProperty ], horizontal: orientationH, vertical: orientationV, multiSelectGroup:item[ groupProperty ], disabled:itemIsDisabled( item )}"' + 'ng-click="syncItems( item, $event, $index );"' + 'ng-mouseleave="removeFocusStyle( tabIndex );">' + '<div class="acol" ng-if="item[ spacingProperty ] > 0" ng-repeat="i in numberToArray( item[ spacingProperty ] ) track by $index">&nbsp;</div>' + '<div class="acol">' + '<label>' + '<input class="checkbox focusable" type="checkbox" ng-disabled="itemIsDisabled( item )" ng-checked="item[ tickProperty ]" ng-click="syncItems( item, $event, $index )" />' + '<span ng-class="{disabled:itemIsDisabled( item )}" ng-bind-html="writeLabel( item, \'itemLabel\' )"></span>' + '</label>' + '</div>' + '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;' + '<span class="tickMark" ng-if="item[ groupProperty ] !== true && item[ tickProperty ] === true">&#10004;</span>' + '</div>' + '</div>' + '</form>' + '</div>' + '</span>',
      link: function ($scope, element, attrs) {
        $scope.backUp = [];
        $scope.varButtonLabel = '';
        $scope.scrolled = false;
        $scope.spacingProperty = '';
        $scope.indexProperty = '';
        $scope.checkBoxLayer = '';
        $scope.orientationH = false;
        $scope.orientationV = true;
        $scope.filteredModel = [];
        $scope.inputLabel = { labelFilter: '' };
        $scope.selectedItems = [];
        $scope.formElements = [];
        $scope.tabIndex = 0;
        $scope.clickedItem = null;
        prevTabIndex = 0;
        helperItems = [];
        helperItemsLength = 0;
        // If user specify a height, call this function
        $scope.setHeight = function () {
          if (typeof $scope.maxHeight !== 'undefined') {
            return 'max-height: ' + $scope.maxHeight + '; overflow-y:scroll';
          }
        };
        // A little hack so that AngularJS ng-repeat can loop using start and end index like a normal loop
        // http://stackoverflow.com/questions/16824853/way-to-ng-repeat-defined-number-of-times-instead-of-repeating-over-array
        $scope.numberToArray = function (num) {
          return new Array(num);
        };
        $scope.updateFilter = function () {
          // we check by looping from end of array
          $scope.filteredModel = [];
          var i = 0;
          if (typeof $scope.inputModel === 'undefined') {
            return [];
          }
          for (i = $scope.inputModel.length - 1; i >= 0; i--) {
            // if it's group end
            if (typeof $scope.inputModel[i][$scope.groupProperty] !== 'undefined' && $scope.inputModel[i][$scope.groupProperty] === false) {
              $scope.filteredModel.push($scope.inputModel[i]);
            }
            // if it's data 
            var gotData = false;
            if (typeof $scope.inputModel[i][$scope.groupProperty] === 'undefined') {
              for (var key in $scope.inputModel[i]) {
                // if filter string is in one of object property                            
                if (typeof $scope.inputModel[i][key] !== 'boolean' && String($scope.inputModel[i][key]).toUpperCase().indexOf($scope.inputLabel.labelFilter.toUpperCase()) >= 0) {
                  gotData = true;
                  break;
                }
              }
              if (gotData === true) {
                // push
                $scope.filteredModel.push($scope.inputModel[i]);
              }
            }
            // if it's group start
            if (typeof $scope.inputModel[i][$scope.groupProperty] !== 'undefined' && $scope.inputModel[i][$scope.groupProperty] === true) {
              if (typeof $scope.filteredModel[$scope.filteredModel.length - 1][$scope.groupProperty] !== 'undefined' && $scope.filteredModel[$scope.filteredModel.length - 1][$scope.groupProperty] === false) {
                $scope.filteredModel.pop();
              } else {
                $scope.filteredModel.push($scope.inputModel[i]);
              }
            }
          }
          $scope.filteredModel.reverse();
          $timeout(function () {
            $scope.getFormElements();
          }, 0);
        };
        // List all the input elements.
        // This function will be called everytime the filter is updated. Not good for performance, but oh well..
        $scope.getFormElements = function () {
          $scope.formElements = [];
          for (var i = 0; i < element[0].getElementsByTagName('FORM')[0].elements.length; i++) {
            $scope.formElements.push(element[0].getElementsByTagName('FORM')[0].elements[i]);
          }
        };
        // check if an item has $scope.groupProperty (be it true or false)
        $scope.isGroupMarker = function (item, type) {
          if (typeof item[$scope.groupProperty] !== 'undefined' && item[$scope.groupProperty] === type)
            return true;
          return false;
        };
        $scope.removeGroupEndMarker = function (item) {
          if (typeof item[$scope.groupProperty] !== 'undefined' && item[$scope.groupProperty] === false)
            return false;
          return true;
        };
        // Show or hide a helper element 
        $scope.displayHelper = function (elementString) {
          if (attrs.selectionMode && $scope.selectionMode.toUpperCase() === 'SINGLE') {
            switch (elementString.toUpperCase()) {
            case 'ALL':
              return false;
              break;
            case 'NONE':
              return false;
              break;
            case 'RESET':
              if (typeof attrs.helperElements === 'undefined') {
                return true;
              } else if (attrs.helperElements && $scope.helperElements.toUpperCase().indexOf('RESET') >= 0) {
                return true;
              }
              break;
            case 'FILTER':
              if (typeof attrs.helperElements === 'undefined') {
                return true;
              }
              if (attrs.helperElements && $scope.helperElements.toUpperCase().indexOf('FILTER') >= 0) {
                return true;
              }
              break;
            default:
              break;
            }
            return false;
          } else {
            if (typeof attrs.helperElements === 'undefined') {
              return true;
            }
            if (attrs.helperElements && $scope.helperElements.toUpperCase().indexOf(elementString.toUpperCase()) >= 0) {
              return true;
            }
            return false;
          }
        };
        // call this function when an item is clicked
        $scope.syncItems = function (item, e, ng_repeat_index) {
          e.preventDefault();
          e.stopPropagation();
          // if it's globaly disabled, then don't do anything
          if (typeof attrs.disableProperty !== 'undefined' && item[$scope.disableProperty] === true) {
            return false;
          }
          // don't change disabled items
          if (typeof attrs.isDisabled !== 'undefined' && $scope.isDisabled === true) {
            return false;
          }
          // we don't care about end of group markers
          if (typeof item[$scope.groupProperty] !== 'undefined' && item[$scope.groupProperty] === false) {
            return false;
          }
          index = $scope.filteredModel.indexOf(item);
          // process items if the start of group marker is clicked ( only for multiple selection! )
          // if, in a group, there are items which are not selected, then they all will be selected
          // if, in a group, all items are selected, then they all will be de-selected                
          if (typeof item[$scope.groupProperty] !== 'undefined' && item[$scope.groupProperty] === true) {
            if (attrs.selectionMode && $scope.selectionMode.toUpperCase() === 'SINGLE') {
              return false;
            }
            var i, j, k;
            var startIndex = 0;
            var endIndex = $scope.filteredModel.length - 1;
            var tempArr = [];
            var nestLevel = 0;
            for (i = index; i < $scope.filteredModel.length; i++) {
              if (nestLevel === 0 && i > index) {
                break;
              }
              // if group start
              if (typeof $scope.filteredModel[i][$scope.groupProperty] !== 'undefined' && $scope.filteredModel[i][$scope.groupProperty] === true) {
                // To cater multi level grouping
                if (tempArr.length === 0) {
                  startIndex = i + 1;
                }
                nestLevel = nestLevel + 1;
              }  // if group end
              else if (typeof $scope.filteredModel[i][$scope.groupProperty] !== 'undefined' && $scope.filteredModel[i][$scope.groupProperty] === false) {
                nestLevel = nestLevel - 1;
                // cek if all are ticked or not                            
                if (tempArr.length > 0 && nestLevel === 0) {
                  var allTicked = true;
                  endIndex = i;
                  for (j = 0; j < tempArr.length; j++) {
                    if (typeof tempArr[j][$scope.tickProperty] !== 'undefined' && tempArr[j][$scope.tickProperty] === false) {
                      allTicked = false;
                      break;
                    }
                  }
                  if (allTicked === true) {
                    for (j = startIndex; j <= endIndex; j++) {
                      if (typeof $scope.filteredModel[j][$scope.groupProperty] === 'undefined') {
                        if (typeof attrs.disableProperty === 'undefined') {
                          $scope.filteredModel[j][$scope.tickProperty] = false;
                          // we refresh input model as well
                          inputModelIndex = $scope.filteredModel[j][$scope.indexProperty];
                          $scope.inputModel[inputModelIndex][$scope.tickProperty] = false;
                        } else if ($scope.filteredModel[j][$scope.disableProperty] !== true) {
                          $scope.filteredModel[j][$scope.tickProperty] = false;
                          // we refresh input model as well
                          inputModelIndex = $scope.filteredModel[j][$scope.indexProperty];
                          $scope.inputModel[inputModelIndex][$scope.tickProperty] = false;
                        }
                      }
                    }
                  } else {
                    for (j = startIndex; j <= endIndex; j++) {
                      if (typeof $scope.filteredModel[j][$scope.groupProperty] === 'undefined') {
                        if (typeof attrs.disableProperty === 'undefined') {
                          $scope.filteredModel[j][$scope.tickProperty] = true;
                          // we refresh input model as well
                          inputModelIndex = $scope.filteredModel[j][$scope.indexProperty];
                          $scope.inputModel[inputModelIndex][$scope.tickProperty] = true;
                        } else if ($scope.filteredModel[j][$scope.disableProperty] !== true) {
                          $scope.filteredModel[j][$scope.tickProperty] = true;
                          // we refresh input model as well
                          inputModelIndex = $scope.filteredModel[j][$scope.indexProperty];
                          $scope.inputModel[inputModelIndex][$scope.tickProperty] = true;
                        }
                      }
                    }
                  }
                }
              }  // if data
              else {
                tempArr.push($scope.filteredModel[i]);
              }
            }
          }  // single item click
          else {
            // If it's single selection mode
            if (attrs.selectionMode && $scope.selectionMode.toUpperCase() === 'SINGLE') {
              // first, set everything to false
              for (i = 0; i < $scope.filteredModel.length; i++) {
                $scope.filteredModel[i][$scope.tickProperty] = false;
              }
              for (i = 0; i < $scope.inputModel.length; i++) {
                $scope.inputModel[i][$scope.tickProperty] = false;
              }
              // then set the clicked item to true
              $scope.filteredModel[index][$scope.tickProperty] = true;
              $scope.toggleCheckboxes(e);
            }  // Multiple
            else {
              $scope.filteredModel[index][$scope.tickProperty] = !$scope.filteredModel[index][$scope.tickProperty];
            }
            // we refresh input model as well
            inputModelIndex = $scope.filteredModel[index][$scope.indexProperty];
            $scope.inputModel[inputModelIndex][$scope.tickProperty] = $scope.filteredModel[index][$scope.tickProperty];
          }
          $scope.clickedItem = angular.copy(item);
          // We update the index here
          prevTabIndex = $scope.tabIndex;
          $scope.tabIndex = ng_repeat_index + helperItemsLength;
          // Set focus on the hidden checkbox 
          e.target.focus();
          // set & remove CSS style
          $scope.removeFocusStyle(prevTabIndex);
          $scope.setFocusStyle($scope.tabIndex);
        };
        // update $scope.selectedItems
        // this variable is used in $scope.outputModel and to refresh the button label
        $scope.refreshSelectedItems = function () {
          $scope.selectedItems = [];
          angular.forEach($scope.inputModel, function (value, key) {
            if (typeof value !== 'undefined') {
              if (typeof value[$scope.groupProperty] === 'undefined') {
                if (value[$scope.tickProperty] === true) {
                  $scope.selectedItems.push(value);
                }
              }
            }
          });
        };
        // refresh output model as well
        $scope.refreshOutputModel = function () {
          if (typeof attrs.outputModel !== 'undefined') {
            $scope.outputModel = angular.copy($scope.selectedItems);
            angular.forEach($scope.outputModel, function (value, key) {
              // remove the index number and spacing number from output model
              delete value[$scope.indexProperty];
              delete value[$scope.spacingProperty];
            });
          }
        };
        // refresh button label
        $scope.refreshButton = function () {
          $scope.varButtonLabel = '';
          ctr = 0;
          // refresh button label...
          if ($scope.selectedItems.length === 0) {
            // https://github.com/isteven/angular-multi-select/pull/19                    
            $scope.varButtonLabel = typeof $scope.defaultLabel !== 'undefined' ? $scope.defaultLabel : 'None selected';
          } else {
            var tempMaxLabels = $scope.selectedItems.length;
            if (typeof $scope.maxLabels !== 'undefined' && $scope.maxLabels !== '') {
              tempMaxLabels = $scope.maxLabels;
            }
            // if max amount of labels displayed..
            if ($scope.selectedItems.length > tempMaxLabels) {
              $scope.more = true;
            } else {
              $scope.more = false;
            }
            angular.forEach($scope.selectedItems, function (value, key) {
              if (typeof value !== 'undefined') {
                if (ctr < tempMaxLabels) {
                  $scope.varButtonLabel += ($scope.varButtonLabel.length > 0 ? '</div>, <div class="buttonLabel">' : '<div class="buttonLabel">') + $scope.writeLabel(value, 'buttonLabel');
                }
                ctr++;
              }
            });
            if ($scope.more === true) {
              // https://github.com/isteven/angular-multi-select/pull/16
              if (tempMaxLabels > 0) {
                $scope.varButtonLabel += ', ... ';
              }
              $scope.varButtonLabel += '(Total: ' + $scope.selectedItems.length + ')';
            }
          }
          $scope.varButtonLabel = $sce.trustAsHtml($scope.varButtonLabel + '<span class="caret"></span>');
        };
        // Check if a checkbox is disabled or enabled. It will check the granular control (disableProperty) and global control (isDisabled)
        // Take note that the granular control has higher priority.
        $scope.itemIsDisabled = function (item) {
          if (typeof attrs.disableProperty !== 'undefined' && item[$scope.disableProperty] === true) {
            return true;
          } else {
            if ($scope.isDisabled === true) {
              return true;
            } else {
              return false;
            }
          }
        };
        // A simple function to parse the item label settings
        $scope.writeLabel = function (item, type) {
          var label = '';
          var temp = $scope[type].split(' ');
          angular.forEach(temp, function (value2, key2) {
            if (typeof value2 !== 'undefined') {
              angular.forEach(item, function (value1, key1) {
                if (key1 == value2) {
                  label += '&nbsp;' + value1;
                }
              });
            }
          });
          if (type.toUpperCase() === 'BUTTONLABEL') {
            return label;
          }
          return $sce.trustAsHtml(label);
        };
        // UI operations to show/hide checkboxes based on click event..
        $scope.toggleCheckboxes = function (e) {
          // We grab the checkboxLayer
          $scope.checkBoxLayer = element.children()[1];
          // We grab the button
          clickedEl = element.children()[0];
          // Just to make sure.. had a bug where key events were recorded twice
          angular.element(document).unbind('click', $scope.externalClickListener);
          angular.element(document).unbind('keydown', $scope.keyboardListener);
          // clear filter
          $scope.inputLabel.labelFilter = '';
          $scope.updateFilter();
          // close if ESC key is pressed.
          if (e.keyCode === 27) {
            angular.element($scope.checkBoxLayer).removeClass('show');
            angular.element(clickedEl).removeClass('buttonClicked');
            angular.element(document).unbind('click', $scope.externalClickListener);
            angular.element(document).unbind('keydown', $scope.keyboardListener);
            // clear the focused element;
            $scope.removeFocusStyle($scope.tabIndex);
            // close callback
            $scope.onClose({ data: element });
            return true;
          }
          // The idea below was taken from another multi-select directive - https://github.com/amitava82/angular-multiselect 
          // His version is awesome if you need a more simple multi-select approach.
          // close
          if (angular.element($scope.checkBoxLayer).hasClass('show')) {
            angular.element($scope.checkBoxLayer).removeClass('show');
            angular.element(clickedEl).removeClass('buttonClicked');
            angular.element(document).unbind('click', $scope.externalClickListener);
            angular.element(document).unbind('keydown', $scope.keyboardListener);
            // clear the focused element;
            $scope.removeFocusStyle($scope.tabIndex);
            // close callback
            $scope.onClose({ data: element });
          }  // open
          else {
            helperItems = [];
            helperItemsLength = 0;
            angular.element($scope.checkBoxLayer).addClass('show');
            angular.element(clickedEl).addClass('buttonClicked');
            angular.element(document).bind('click', $scope.externalClickListener);
            angular.element(document).bind('keydown', $scope.keyboardListener);
            // to get the initial tab index, depending on how many helper elements we have. 
            // priority is to always focus it on the input filter 
            $scope.getFormElements();
            $scope.tabIndex = 0;
            var helperContainer = angular.element(element[0].querySelector('.helperContainer'))[0];
            if (typeof helperContainer !== 'undefined') {
              for (i = 0; i < helperContainer.getElementsByTagName('BUTTON').length; i++) {
                helperItems[i] = helperContainer.getElementsByTagName('BUTTON')[i];
              }
              helperItemsLength = helperItems.length + helperContainer.getElementsByTagName('INPUT').length;
            }
            // focus on the filter element on open. 
            if (element[0].querySelector('.inputFilter')) {
              element[0].querySelector('.inputFilter').focus();
              $scope.tabIndex = $scope.tabIndex + helperItemsLength - 2;
            }  // if there's no filter then just focus on the first checkbox item
            else {
              $scope.formElements[$scope.tabIndex].focus();
            }
            // open callback
            $scope.onOpen({ data: element });
          }
        };
        // handle clicks outside the button / multi select layer
        $scope.externalClickListener = function (e) {
          targetsArr = element.find(e.target.tagName);
          for (var i = 0; i < targetsArr.length; i++) {
            if (e.target == targetsArr[i]) {
              return;
            }
          }
          angular.element($scope.checkBoxLayer.previousSibling).removeClass('buttonClicked');
          angular.element($scope.checkBoxLayer).removeClass('show');
          angular.element(document).unbind('click', $scope.externalClickListener);
          angular.element(document).unbind('keydown', $scope.keyboardListener);
          // close callback                
          $timeout(function () {
            $scope.onClose({ data: element });
          }, 0);
        };
        // traverse up to find the button tag
        // http://stackoverflow.com/questions/7332179/how-to-recursively-search-all-parentnodes
        $scope.findUpTag = function (el, tag, className) {
          while (el.parentNode) {
            el = el.parentNode;
            if (typeof el.tagName !== 'undefined') {
              if (el.tagName.toUpperCase() === tag.toUpperCase() && el.className.indexOf(className) > -1) {
                return el;
              }
            }
          }
          return null;
        };
        // select All / select None / reset buttons
        $scope.select = function (type, e) {
          helperIndex = helperItems.indexOf(e.target);
          $scope.tabIndex = helperIndex;
          switch (type.toUpperCase()) {
          case 'ALL':
            angular.forEach($scope.filteredModel, function (value, key) {
              if (typeof value !== 'undefined' && value[$scope.disableProperty] !== true) {
                if (typeof value[$scope.groupProperty] === 'undefined') {
                  value[$scope.tickProperty] = true;
                }
              }
            });
            break;
          case 'NONE':
            angular.forEach($scope.filteredModel, function (value, key) {
              if (typeof value !== 'undefined' && value[$scope.disableProperty] !== true) {
                if (typeof value[$scope.groupProperty] === 'undefined') {
                  value[$scope.tickProperty] = false;
                }
              }
            });
            break;
          case 'RESET':
            angular.forEach($scope.filteredModel, function (value, key) {
              if (typeof value[$scope.groupProperty] === 'undefined' && typeof value !== 'undefined' && value[$scope.disableProperty] !== true) {
                temp = value[$scope.indexProperty];
                value[$scope.tickProperty] = $scope.backUp[temp][$scope.tickProperty];
              }
            });
            break;
          case 'CLEAR':
            $scope.tabIndex = $scope.tabIndex + 1;
            break;
          case 'FILTER':
            $scope.tabIndex = helperItems.length - 1;
            break;
          default:
          }
        };
        // just to create a random variable name                
        genRandomString = function (length) {
          var possible = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz';
          var temp = '';
          for (var i = 0; i < length; i++) {
            temp += possible.charAt(Math.floor(Math.random() * possible.length));
          }
          return temp;
        };
        // count leading spaces
        $scope.prepareGrouping = function () {
          var spacing = 0;
          angular.forEach($scope.filteredModel, function (value, key) {
            value[$scope.spacingProperty] = spacing;
            if (value[$scope.groupProperty] === true) {
              spacing += 2;
            } else if (value[$scope.groupProperty] === false) {
              spacing -= 2;
            }
          });
        };
        // prepare original index
        $scope.prepareIndex = function () {
          ctr = 0;
          angular.forEach($scope.filteredModel, function (value, key) {
            value[$scope.indexProperty] = ctr;
            ctr++;
          });
        };
        // navigate using up and down arrow
        $scope.keyboardListener = function (e) {
          var key = e.keyCode ? e.keyCode : e.which;
          var isNavigationKey = false;
          // ESC key (close)
          if (key === 27) {
            $scope.toggleCheckboxes(e);
          }  // next element ( tab, down & right key )                    
          else if (key === 40 || key === 39 || !e.shiftKey && key == 9) {
            isNavigationKey = true;
            prevTabIndex = $scope.tabIndex;
            $scope.tabIndex++;
            if ($scope.tabIndex > $scope.formElements.length - 1) {
              $scope.tabIndex = 0;
              prevTabIndex = $scope.formElements.length - 1;
            }
            while ($scope.formElements[$scope.tabIndex].disabled === true) {
              $scope.tabIndex++;
              if ($scope.tabIndex > $scope.formElements.length - 1) {
                $scope.tabIndex = 0;
              }
            }
          }  // prev element ( shift+tab, up & left key )
          else if (key === 38 || key === 37 || e.shiftKey && key == 9) {
            isNavigationKey = true;
            prevTabIndex = $scope.tabIndex;
            $scope.tabIndex--;
            if ($scope.tabIndex < 0) {
              $scope.tabIndex = $scope.formElements.length - 1;
              prevTabIndex = 0;
            }
            while ($scope.formElements[$scope.tabIndex].disabled === true) {
              $scope.tabIndex--;
              if ($scope.tabIndex < 0) {
                $scope.tabIndex = $scope.formElements.length - 1;
              }
            }
          }
          if (isNavigationKey === true) {
            e.preventDefault();
            e.stopPropagation();
            // set focus on the checkbox
            $scope.formElements[$scope.tabIndex].focus();
            // css styling
            var actEl = document.activeElement;
            if (actEl.type.toUpperCase() === 'CHECKBOX') {
              $scope.setFocusStyle($scope.tabIndex);
              $scope.removeFocusStyle(prevTabIndex);
            } else {
              $scope.removeFocusStyle(prevTabIndex);
              $scope.removeFocusStyle(helperItemsLength);
              $scope.removeFocusStyle($scope.formElements.length - 1);
            }
          }
          isNavigationKey = false;
        };
        // set (add) CSS style on selected row
        $scope.setFocusStyle = function (tabIndex) {
          angular.element($scope.formElements[tabIndex]).parent().parent().parent().addClass('multiSelectFocus');
        };
        // remove CSS style on selected row
        $scope.removeFocusStyle = function (tabIndex) {
          angular.element($scope.formElements[tabIndex]).parent().parent().parent().removeClass('multiSelectFocus');
        };
        ///////////////////////////////////////////////////////
        //
        // Logic starts here, initiated by watch 1 & watch 2.
        //
        ///////////////////////////////////////////////////////
        var tempStr = genRandomString(5);
        $scope.indexProperty = 'idx_' + tempStr;
        $scope.spacingProperty = 'spc_' + tempStr;
        // set orientation css            
        if (typeof attrs.orientation !== 'undefined') {
          if (attrs.orientation.toUpperCase() === 'HORIZONTAL') {
            $scope.orientationH = true;
            $scope.orientationV = false;
          } else {
            $scope.orientationH = false;
            $scope.orientationV = true;
          }
        }
        // watch1, for changes in input model property
        // updates multi-select when user select/deselect a single checkbox programatically
        // https://github.com/isteven/angular-multi-select/issues/8
        $scope.$watch('inputModel', function (newVal) {
          if (newVal) {
            $scope.refreshSelectedItems();
            $scope.refreshOutputModel();
            $scope.refreshButton();
            if ($scope.clickedItem !== null) {
              $timeout(function () {
                $scope.onItemClick({ data: $scope.clickedItem });
                $scope.clickedItem = null;
              }, 0);
            }
          }
        }, true);
        // watch2 for changes in input model as a whole
        // this on updates the multi-select when a user load a whole new input-model. We also update the $scope.backUp variable
        $scope.$watch('inputModel', function (newVal) {
          if (newVal) {
            $scope.backUp = angular.copy($scope.inputModel);
            $scope.updateFilter();
            $scope.prepareGrouping();
            $scope.prepareIndex();
            $scope.refreshSelectedItems();
            $scope.refreshOutputModel();
            $scope.refreshButton();
          }
        });
        // watch for changes in directive state (disabled or enabled)
        $scope.$watch('isDisabled', function (newVal) {
          $scope.isDisabled = newVal;
        });
        // this is for touch enabled devices. We don't want to hide checkboxes on scroll. 
        angular.element(document).bind('touchstart', function (e) {
          $scope.$apply(function () {
            $scope.scrolled = false;
          });
        });
        // also for touch enabled devices
        angular.element(document).bind('touchmove', function (e) {
          $scope.$apply(function () {
            $scope.scrolled = true;
          });
        });
        // for IE8, perhaps. Not sure if this is really executed.
        if (!Array.prototype.indexOf) {
          Array.prototype.indexOf = function (what, i) {
            i = i || 0;
            var L = this.length;
            while (i < L) {
              if (this[i] === what)
                return i;
              ++i;
            }
            return -1;
          };
        }
      }
    };
  }
]);
/*!
 * angular-translate - v2.6.1 - 2015-03-01
 * http://github.com/angular-translate/angular-translate
 * Copyright (c) 2015 ; Licensed MIT
 */
/**
 * @ngdoc overview
 * @name pascalprecht.translate
 *
 * @description
 * The main module which holds everything together.
 */
angular.module('pascalprecht.translate', ['ng']).run([
  '$translate',
  function ($translate) {
    var key = $translate.storageKey(), storage = $translate.storage();
    var fallbackFromIncorrectStorageValue = function () {
      var preferred = $translate.preferredLanguage();
      if (angular.isString(preferred)) {
        $translate.use(preferred);  // $translate.use() will also remember the language.
                                    // So, we don't need to call storage.put() here.
      } else {
        storage.put(key, $translate.use());
      }
    };
    if (storage) {
      if (!storage.get(key)) {
        fallbackFromIncorrectStorageValue();
      } else {
        $translate.use(storage.get(key))['catch'](fallbackFromIncorrectStorageValue);
      }
    } else if (angular.isString($translate.preferredLanguage())) {
      $translate.use($translate.preferredLanguage());
    }
  }
]);
/**
 * @ngdoc object
 * @name pascalprecht.translate.$translateProvider
 * @description
 *
 * $translateProvider allows developers to register translation-tables, asynchronous loaders
 * and similar to configure translation behavior directly inside of a module.
 *
 */
angular.module('pascalprecht.translate').provider('$translate', [
  '$STORAGE_KEY',
  '$windowProvider',
  function ($STORAGE_KEY, $windowProvider) {
    var $translationTable = {}, $preferredLanguage, $availableLanguageKeys = [], $languageKeyAliases, $fallbackLanguage, $fallbackWasString, $uses, $nextLang, $storageFactory, $storageKey = $STORAGE_KEY, $storagePrefix, $missingTranslationHandlerFactory, $interpolationFactory, $interpolatorFactories = [], $interpolationSanitizationStrategy = false, $loaderFactory, $cloakClassName = 'translate-cloak', $loaderOptions, $notFoundIndicatorLeft, $notFoundIndicatorRight, $postCompilingEnabled = false, NESTED_OBJECT_DELIMITER = '.', loaderCache, directivePriority = 0;
    var version = '2.6.1';
    // tries to determine the browsers language
    var getFirstBrowserLanguage = function () {
      var nav = $windowProvider.$get().navigator, browserLanguagePropertyKeys = [
          'language',
          'browserLanguage',
          'systemLanguage',
          'userLanguage'
        ], i, language;
      // support for HTML 5.1 "navigator.languages"
      if (angular.isArray(nav.languages)) {
        for (i = 0; i < nav.languages.length; i++) {
          language = nav.languages[i];
          if (language && language.length) {
            return language;
          }
        }
      }
      // support for other well known properties in browsers
      for (i = 0; i < browserLanguagePropertyKeys.length; i++) {
        language = nav[browserLanguagePropertyKeys[i]];
        if (language && language.length) {
          return language;
        }
      }
      return null;
    };
    getFirstBrowserLanguage.displayName = 'angular-translate/service: getFirstBrowserLanguage';
    // tries to determine the browsers locale
    var getLocale = function () {
      return (getFirstBrowserLanguage() || '').split('-').join('_');
    };
    getLocale.displayName = 'angular-translate/service: getLocale';
    /**
   * @name indexOf
   * @private
   *
   * @description
   * indexOf polyfill. Kinda sorta.
   *
   * @param {array} array Array to search in.
   * @param {string} searchElement Element to search for.
   *
   * @returns {int} Index of search element.
   */
    var indexOf = function (array, searchElement) {
      for (var i = 0, len = array.length; i < len; i++) {
        if (array[i] === searchElement) {
          return i;
        }
      }
      return -1;
    };
    /**
   * @name trim
   * @private
   *
   * @description
   * trim polyfill
   *
   * @returns {string} The string stripped of whitespace from both ends
   */
    var trim = function () {
      return this.replace(/^\s+|\s+$/g, '');
    };
    var negotiateLocale = function (preferred) {
      var avail = [], locale = angular.lowercase(preferred), i = 0, n = $availableLanguageKeys.length;
      for (; i < n; i++) {
        avail.push(angular.lowercase($availableLanguageKeys[i]));
      }
      if (indexOf(avail, locale) > -1) {
        return preferred;
      }
      if ($languageKeyAliases) {
        var alias;
        for (var langKeyAlias in $languageKeyAliases) {
          var hasWildcardKey = false;
          var hasExactKey = Object.prototype.hasOwnProperty.call($languageKeyAliases, langKeyAlias) && angular.lowercase(langKeyAlias) === angular.lowercase(preferred);
          if (langKeyAlias.slice(-1) === '*') {
            hasWildcardKey = langKeyAlias.slice(0, -1) === preferred.slice(0, langKeyAlias.length - 1);
          }
          if (hasExactKey || hasWildcardKey) {
            alias = $languageKeyAliases[langKeyAlias];
            if (indexOf(avail, angular.lowercase(alias)) > -1) {
              return alias;
            }
          }
        }
      }
      var parts = preferred.split('_');
      if (parts.length > 1 && indexOf(avail, angular.lowercase(parts[0])) > -1) {
        return parts[0];
      }
      // If everything fails, just return the preferred, unchanged.
      return preferred;
    };
    /**
   * @ngdoc function
   * @name pascalprecht.translate.$translateProvider#translations
   * @methodOf pascalprecht.translate.$translateProvider
   *
   * @description
   * Registers a new translation table for specific language key.
   *
   * To register a translation table for specific language, pass a defined language
   * key as first parameter.
   *
   * <pre>
   *  // register translation table for language: 'de_DE'
   *  $translateProvider.translations('de_DE', {
   *    'GREETING': 'Hallo Welt!'
   *  });
   *
   *  // register another one
   *  $translateProvider.translations('en_US', {
   *    'GREETING': 'Hello world!'
   *  });
   * </pre>
   *
   * When registering multiple translation tables for for the same language key,
   * the actual translation table gets extended. This allows you to define module
   * specific translation which only get added, once a specific module is loaded in
   * your app.
   *
   * Invoking this method with no arguments returns the translation table which was
   * registered with no language key. Invoking it with a language key returns the
   * related translation table.
   *
   * @param {string} key A language key.
   * @param {object} translationTable A plain old JavaScript object that represents a translation table.
   *
   */
    var translations = function (langKey, translationTable) {
      if (!langKey && !translationTable) {
        return $translationTable;
      }
      if (langKey && !translationTable) {
        if (angular.isString(langKey)) {
          return $translationTable[langKey];
        }
      } else {
        if (!angular.isObject($translationTable[langKey])) {
          $translationTable[langKey] = {};
        }
        angular.extend($translationTable[langKey], flatObject(translationTable));
      }
      return this;
    };
    this.translations = translations;
    /**
   * @ngdoc function
   * @name pascalprecht.translate.$translateProvider#cloakClassName
   * @methodOf pascalprecht.translate.$translateProvider
   *
   * @description
   *
   * Let's you change the class name for `translate-cloak` directive.
   * Default class name is `translate-cloak`.
   *
   * @param {string} name translate-cloak class name
   */
    this.cloakClassName = function (name) {
      if (!name) {
        return $cloakClassName;
      }
      $cloakClassName = name;
      return this;
    };
    /**
   * @name flatObject
   * @private
   *
   * @description
   * Flats an object. This function is used to flatten given translation data with
   * namespaces, so they are later accessible via dot notation.
   */
    var flatObject = function (data, path, result, prevKey) {
      var key, keyWithPath, keyWithShortPath, val;
      if (!path) {
        path = [];
      }
      if (!result) {
        result = {};
      }
      for (key in data) {
        if (!Object.prototype.hasOwnProperty.call(data, key)) {
          continue;
        }
        val = data[key];
        if (angular.isObject(val)) {
          flatObject(val, path.concat(key), result, key);
        } else {
          keyWithPath = path.length ? '' + path.join(NESTED_OBJECT_DELIMITER) + NESTED_OBJECT_DELIMITER + key : key;
          if (path.length && key === prevKey) {
            // Create shortcut path (foo.bar == foo.bar.bar)
            keyWithShortPath = '' + path.join(NESTED_OBJECT_DELIMITER);
            // Link it to original path
            result[keyWithShortPath] = '@:' + keyWithPath;
          }
          result[keyWithPath] = val;
        }
      }
      return result;
    };
    /**
   * @ngdoc function
   * @name pascalprecht.translate.$translateProvider#addInterpolation
   * @methodOf pascalprecht.translate.$translateProvider
   *
   * @description
   * Adds interpolation services to angular-translate, so it can manage them.
   *
   * @param {object} factory Interpolation service factory
   */
    this.addInterpolation = function (factory) {
      $interpolatorFactories.push(factory);
      return this;
    };
    /**
   * @ngdoc function
   * @name pascalprecht.translate.$translateProvider#useMessageFormatInterpolation
   * @methodOf pascalprecht.translate.$translateProvider
   *
   * @description
   * Tells angular-translate to use interpolation functionality of messageformat.js.
   * This is useful when having high level pluralization and gender selection.
   */
    this.useMessageFormatInterpolation = function () {
      return this.useInterpolation('$translateMessageFormatInterpolation');
    };
    /**
   * @ngdoc function
   * @name pascalprecht.translate.$translateProvider#useInterpolation
   * @methodOf pascalprecht.translate.$translateProvider
   *
   * @description
   * Tells angular-translate which interpolation style to use as default, application-wide.
   * Simply pass a factory/service name. The interpolation service has to implement
   * the correct interface.
   *
   * @param {string} factory Interpolation service name.
   */
    this.useInterpolation = function (factory) {
      $interpolationFactory = factory;
      return this;
    };
    /**
   * @ngdoc function
   * @name pascalprecht.translate.$translateProvider#useSanitizeStrategy
   * @methodOf pascalprecht.translate.$translateProvider
   *
   * @description
   * Simply sets a sanitation strategy type.
   *
   * @param {string} value Strategy type.
   */
    this.useSanitizeValueStrategy = function (value) {
      $interpolationSanitizationStrategy = value;
      return this;
    };
    /**
   * @ngdoc function
   * @name pascalprecht.translate.$translateProvider#preferredLanguage
   * @methodOf pascalprecht.translate.$translateProvider
   *
   * @description
   * Tells the module which of the registered translation tables to use for translation
   * at initial startup by passing a language key. Similar to `$translateProvider#use`
   * only that it says which language to **prefer**.
   *
   * @param {string} langKey A language key.
   *
   */
    this.preferredLanguage = function (langKey) {
      setupPreferredLanguage(langKey);
      return this;
    };
    var setupPreferredLanguage = function (langKey) {
      if (langKey) {
        $preferredLanguage = langKey;
      }
      return $preferredLanguage;
    };
    /**
   * @ngdoc function
   * @name pascalprecht.translate.$translateProvider#translationNotFoundIndicator
   * @methodOf pascalprecht.translate.$translateProvider
   *
   * @description
   * Sets an indicator which is used when a translation isn't found. E.g. when
   * setting the indicator as 'X' and one tries to translate a translation id
   * called `NOT_FOUND`, this will result in `X NOT_FOUND X`.
   *
   * Internally this methods sets a left indicator and a right indicator using
   * `$translateProvider.translationNotFoundIndicatorLeft()` and
   * `$translateProvider.translationNotFoundIndicatorRight()`.
   *
   * **Note**: These methods automatically add a whitespace between the indicators
   * and the translation id.
   *
   * @param {string} indicator An indicator, could be any string.
   */
    this.translationNotFoundIndicator = function (indicator) {
      this.translationNotFoundIndicatorLeft(indicator);
      this.translationNotFoundIndicatorRight(indicator);
      return this;
    };
    /**
   * ngdoc function
   * @name pascalprecht.translate.$translateProvider#translationNotFoundIndicatorLeft
   * @methodOf pascalprecht.translate.$translateProvider
   *
   * @description
   * Sets an indicator which is used when a translation isn't found left to the
   * translation id.
   *
   * @param {string} indicator An indicator.
   */
    this.translationNotFoundIndicatorLeft = function (indicator) {
      if (!indicator) {
        return $notFoundIndicatorLeft;
      }
      $notFoundIndicatorLeft = indicator;
      return this;
    };
    /**
   * ngdoc function
   * @name pascalprecht.translate.$translateProvider#translationNotFoundIndicatorLeft
   * @methodOf pascalprecht.translate.$translateProvider
   *
   * @description
   * Sets an indicator which is used when a translation isn't found right to the
   * translation id.
   *
   * @param {string} indicator An indicator.
   */
    this.translationNotFoundIndicatorRight = function (indicator) {
      if (!indicator) {
        return $notFoundIndicatorRight;
      }
      $notFoundIndicatorRight = indicator;
      return this;
    };
    /**
   * @ngdoc function
   * @name pascalprecht.translate.$translateProvider#fallbackLanguage
   * @methodOf pascalprecht.translate.$translateProvider
   *
   * @description
   * Tells the module which of the registered translation tables to use when missing translations
   * at initial startup by passing a language key. Similar to `$translateProvider#use`
   * only that it says which language to **fallback**.
   *
   * @param {string||array} langKey A language key.
   *
   */
    this.fallbackLanguage = function (langKey) {
      fallbackStack(langKey);
      return this;
    };
    var fallbackStack = function (langKey) {
      if (langKey) {
        if (angular.isString(langKey)) {
          $fallbackWasString = true;
          $fallbackLanguage = [langKey];
        } else if (angular.isArray(langKey)) {
          $fallbackWasString = false;
          $fallbackLanguage = langKey;
        }
        if (angular.isString($preferredLanguage) && indexOf($fallbackLanguage, $preferredLanguage) < 0) {
          $fallbackLanguage.push($preferredLanguage);
        }
        return this;
      } else {
        if ($fallbackWasString) {
          return $fallbackLanguage[0];
        } else {
          return $fallbackLanguage;
        }
      }
    };
    /**
   * @ngdoc function
   * @name pascalprecht.translate.$translateProvider#use
   * @methodOf pascalprecht.translate.$translateProvider
   *
   * @description
   * Set which translation table to use for translation by given language key. When
   * trying to 'use' a language which isn't provided, it'll throw an error.
   *
   * You actually don't have to use this method since `$translateProvider#preferredLanguage`
   * does the job too.
   *
   * @param {string} langKey A language key.
   */
    this.use = function (langKey) {
      if (langKey) {
        if (!$translationTable[langKey] && !$loaderFactory) {
          // only throw an error, when not loading translation data asynchronously
          throw new Error('$translateProvider couldn\'t find translationTable for langKey: \'' + langKey + '\'');
        }
        $uses = langKey;
        return this;
      }
      return $uses;
    };
    /**
   * @ngdoc function
   * @name pascalprecht.translate.$translateProvider#storageKey
   * @methodOf pascalprecht.translate.$translateProvider
   *
   * @description
   * Tells the module which key must represent the choosed language by a user in the storage.
   *
   * @param {string} key A key for the storage.
   */
    var storageKey = function (key) {
      if (!key) {
        if ($storagePrefix) {
          return $storagePrefix + $storageKey;
        }
        return $storageKey;
      }
      $storageKey = key;
    };
    this.storageKey = storageKey;
    /**
   * @ngdoc function
   * @name pascalprecht.translate.$translateProvider#useUrlLoader
   * @methodOf pascalprecht.translate.$translateProvider
   *
   * @description
   * Tells angular-translate to use `$translateUrlLoader` extension service as loader.
   *
   * @param {string} url Url
   * @param {Object=} options Optional configuration object
   */
    this.useUrlLoader = function (url, options) {
      return this.useLoader('$translateUrlLoader', angular.extend({ url: url }, options));
    };
    /**
   * @ngdoc function
   * @name pascalprecht.translate.$translateProvider#useStaticFilesLoader
   * @methodOf pascalprecht.translate.$translateProvider
   *
   * @description
   * Tells angular-translate to use `$translateStaticFilesLoader` extension service as loader.
   *
   * @param {Object=} options Optional configuration object
   */
    this.useStaticFilesLoader = function (options) {
      return this.useLoader('$translateStaticFilesLoader', options);
    };
    /**
   * @ngdoc function
   * @name pascalprecht.translate.$translateProvider#useLoader
   * @methodOf pascalprecht.translate.$translateProvider
   *
   * @description
   * Tells angular-translate to use any other service as loader.
   *
   * @param {string} loaderFactory Factory name to use
   * @param {Object=} options Optional configuration object
   */
    this.useLoader = function (loaderFactory, options) {
      $loaderFactory = loaderFactory;
      $loaderOptions = options || {};
      return this;
    };
    /**
   * @ngdoc function
   * @name pascalprecht.translate.$translateProvider#useLocalStorage
   * @methodOf pascalprecht.translate.$translateProvider
   *
   * @description
   * Tells angular-translate to use `$translateLocalStorage` service as storage layer.
   *
   */
    this.useLocalStorage = function () {
      return this.useStorage('$translateLocalStorage');
    };
    /**
   * @ngdoc function
   * @name pascalprecht.translate.$translateProvider#useCookieStorage
   * @methodOf pascalprecht.translate.$translateProvider
   *
   * @description
   * Tells angular-translate to use `$translateCookieStorage` service as storage layer.
   */
    this.useCookieStorage = function () {
      return this.useStorage('$translateCookieStorage');
    };
    /**
   * @ngdoc function
   * @name pascalprecht.translate.$translateProvider#useStorage
   * @methodOf pascalprecht.translate.$translateProvider
   *
   * @description
   * Tells angular-translate to use custom service as storage layer.
   */
    this.useStorage = function (storageFactory) {
      $storageFactory = storageFactory;
      return this;
    };
    /**
   * @ngdoc function
   * @name pascalprecht.translate.$translateProvider#storagePrefix
   * @methodOf pascalprecht.translate.$translateProvider
   *
   * @description
   * Sets prefix for storage key.
   *
   * @param {string} prefix Storage key prefix
   */
    this.storagePrefix = function (prefix) {
      if (!prefix) {
        return prefix;
      }
      $storagePrefix = prefix;
      return this;
    };
    /**
   * @ngdoc function
   * @name pascalprecht.translate.$translateProvider#useMissingTranslationHandlerLog
   * @methodOf pascalprecht.translate.$translateProvider
   *
   * @description
   * Tells angular-translate to use built-in log handler when trying to translate
   * a translation Id which doesn't exist.
   *
   * This is actually a shortcut method for `useMissingTranslationHandler()`.
   *
   */
    this.useMissingTranslationHandlerLog = function () {
      return this.useMissingTranslationHandler('$translateMissingTranslationHandlerLog');
    };
    /**
   * @ngdoc function
   * @name pascalprecht.translate.$translateProvider#useMissingTranslationHandler
   * @methodOf pascalprecht.translate.$translateProvider
   *
   * @description
   * Expects a factory name which later gets instantiated with `$injector`.
   * This method can be used to tell angular-translate to use a custom
   * missingTranslationHandler. Just build a factory which returns a function
   * and expects a translation id as argument.
   *
   * Example:
   * <pre>
   *  app.config(function ($translateProvider) {
   *    $translateProvider.useMissingTranslationHandler('customHandler');
   *  });
   *
   *  app.factory('customHandler', function (dep1, dep2) {
   *    return function (translationId) {
   *      // something with translationId and dep1 and dep2
   *    };
   *  });
   * </pre>
   *
   * @param {string} factory Factory name
   */
    this.useMissingTranslationHandler = function (factory) {
      $missingTranslationHandlerFactory = factory;
      return this;
    };
    /**
   * @ngdoc function
   * @name pascalprecht.translate.$translateProvider#usePostCompiling
   * @methodOf pascalprecht.translate.$translateProvider
   *
   * @description
   * If post compiling is enabled, all translated values will be processed
   * again with AngularJS' $compile.
   *
   * Example:
   * <pre>
   *  app.config(function ($translateProvider) {
   *    $translateProvider.usePostCompiling(true);
   *  });
   * </pre>
   *
   * @param {string} factory Factory name
   */
    this.usePostCompiling = function (value) {
      $postCompilingEnabled = !!value;
      return this;
    };
    /**
   * @ngdoc function
   * @name pascalprecht.translate.$translateProvider#determinePreferredLanguage
   * @methodOf pascalprecht.translate.$translateProvider
   *
   * @description
   * Tells angular-translate to try to determine on its own which language key
   * to set as preferred language. When `fn` is given, angular-translate uses it
   * to determine a language key, otherwise it uses the built-in `getLocale()`
   * method.
   *
   * The `getLocale()` returns a language key in the format `[lang]_[country]` or
   * `[lang]` depending on what the browser provides.
   *
   * Use this method at your own risk, since not all browsers return a valid
   * locale.
   *
   * @param {object=} fn Function to determine a browser's locale
   */
    this.determinePreferredLanguage = function (fn) {
      var locale = fn && angular.isFunction(fn) ? fn() : getLocale();
      if (!$availableLanguageKeys.length) {
        $preferredLanguage = locale;
      } else {
        $preferredLanguage = negotiateLocale(locale);
      }
      return this;
    };
    /**
   * @ngdoc function
   * @name pascalprecht.translate.$translateProvider#registerAvailableLanguageKeys
   * @methodOf pascalprecht.translate.$translateProvider
   *
   * @description
   * Registers a set of language keys the app will work with. Use this method in
   * combination with
   * {@link pascalprecht.translate.$translateProvider#determinePreferredLanguage determinePreferredLanguage}.
   * When available languages keys are registered, angular-translate
   * tries to find the best fitting language key depending on the browsers locale,
   * considering your language key convention.
   *
   * @param {object} languageKeys Array of language keys the your app will use
   * @param {object=} aliases Alias map.
   */
    this.registerAvailableLanguageKeys = function (languageKeys, aliases) {
      if (languageKeys) {
        $availableLanguageKeys = languageKeys;
        if (aliases) {
          $languageKeyAliases = aliases;
        }
        return this;
      }
      return $availableLanguageKeys;
    };
    /**
   * @ngdoc function
   * @name pascalprecht.translate.$translateProvider#useLoaderCache
   * @methodOf pascalprecht.translate.$translateProvider
   *
   * @description
   * Registers a cache for internal $http based loaders.
   * {@link pascalprecht.translate.$translateProvider#determinePreferredLanguage determinePreferredLanguage}.
   * When false the cache will be disabled (default). When true or undefined
   * the cache will be a default (see $cacheFactory). When an object it will
   * be treat as a cache object itself: the usage is $http({cache: cache})
   *
   * @param {object} cache boolean, string or cache-object
   */
    this.useLoaderCache = function (cache) {
      if (cache === false) {
        // disable cache
        loaderCache = undefined;
      } else if (cache === true) {
        // enable cache using AJS defaults
        loaderCache = true;
      } else if (typeof cache === 'undefined') {
        // enable cache using default
        loaderCache = '$translationCache';
      } else if (cache) {
        // enable cache using given one (see $cacheFactory)
        loaderCache = cache;
      }
      return this;
    };
    /**
   * @ngdoc function
   * @name pascalprecht.translate.$translateProvider#directivePriority
   * @methodOf pascalprecht.translate.$translateProvider
   *
   * @description
   * Sets the default priority of the translate directive. The standard value is `0`.
   * Calling this function without an argument will return the current value.
   *
   * @param {number} priority for the translate-directive
   */
    this.directivePriority = function (priority) {
      if (priority === undefined) {
        // getter
        return directivePriority;
      } else {
        // setter with chaining
        directivePriority = priority;
        return this;
      }
    };
    /**
   * @ngdoc object
   * @name pascalprecht.translate.$translate
   * @requires $interpolate
   * @requires $log
   * @requires $rootScope
   * @requires $q
   *
   * @description
   * The `$translate` service is the actual core of angular-translate. It expects a translation id
   * and optional interpolate parameters to translate contents.
   *
   * <pre>
   *  $translate('HEADLINE_TEXT').then(function (translation) {
   *    $scope.translatedText = translation;
   *  });
   * </pre>
   *
   * @param {string|array} translationId A token which represents a translation id
   *                                     This can be optionally an array of translation ids which
   *                                     results that the function returns an object where each key
   *                                     is the translation id and the value the translation.
   * @param {object=} interpolateParams An object hash for dynamic values
   * @param {string} interpolationId The id of the interpolation to use
   * @returns {object} promise
   */
    this.$get = [
      '$log',
      '$injector',
      '$rootScope',
      '$q',
      function ($log, $injector, $rootScope, $q) {
        var Storage, defaultInterpolator = $injector.get($interpolationFactory || '$translateDefaultInterpolation'), pendingLoader = false, interpolatorHashMap = {}, langPromises = {}, fallbackIndex, startFallbackIteration;
        var $translate = function (translationId, interpolateParams, interpolationId, defaultTranslationText) {
          // Duck detection: If the first argument is an array, a bunch of translations was requested.
          // The result is an object.
          if (angular.isArray(translationId)) {
            // Inspired by Q.allSettled by Kris Kowal
            // https://github.com/kriskowal/q/blob/b0fa72980717dc202ffc3cbf03b936e10ebbb9d7/q.js#L1553-1563
            // This transforms all promises regardless resolved or rejected
            var translateAll = function (translationIds) {
              var results = {};
              // storing the actual results
              var promises = [];
              // promises to wait for
              // Wraps the promise a) being always resolved and b) storing the link id->value
              var translate = function (translationId) {
                var deferred = $q.defer();
                var regardless = function (value) {
                  results[translationId] = value;
                  deferred.resolve([
                    translationId,
                    value
                  ]);
                };
                // we don't care whether the promise was resolved or rejected; just store the values
                $translate(translationId, interpolateParams, interpolationId, defaultTranslationText).then(regardless, regardless);
                return deferred.promise;
              };
              for (var i = 0, c = translationIds.length; i < c; i++) {
                promises.push(translate(translationIds[i]));
              }
              // wait for all (including storing to results)
              return $q.all(promises).then(function () {
                // return the results
                return results;
              });
            };
            return translateAll(translationId);
          }
          var deferred = $q.defer();
          // trim off any whitespace
          if (translationId) {
            translationId = trim.apply(translationId);
          }
          var promiseToWaitFor = function () {
              var promise = $preferredLanguage ? langPromises[$preferredLanguage] : langPromises[$uses];
              fallbackIndex = 0;
              if ($storageFactory && !promise) {
                // looks like there's no pending promise for $preferredLanguage or
                // $uses. Maybe there's one pending for a language that comes from
                // storage.
                var langKey = Storage.get($storageKey);
                promise = langPromises[langKey];
                if ($fallbackLanguage && $fallbackLanguage.length) {
                  var index = indexOf($fallbackLanguage, langKey);
                  // maybe the language from storage is also defined as fallback language
                  // we increase the fallback language index to not search in that language
                  // as fallback, since it's probably the first used language
                  // in that case the index starts after the first element
                  fallbackIndex = index === 0 ? 1 : 0;
                  // but we can make sure to ALWAYS fallback to preferred language at least
                  if (indexOf($fallbackLanguage, $preferredLanguage) < 0) {
                    $fallbackLanguage.push($preferredLanguage);
                  }
                }
              }
              return promise;
            }();
          if (!promiseToWaitFor) {
            // no promise to wait for? okay. Then there's no loader registered
            // nor is a one pending for language that comes from storage.
            // We can just translate.
            determineTranslation(translationId, interpolateParams, interpolationId, defaultTranslationText).then(deferred.resolve, deferred.reject);
          } else {
            promiseToWaitFor.then(function () {
              determineTranslation(translationId, interpolateParams, interpolationId, defaultTranslationText).then(deferred.resolve, deferred.reject);
            }, deferred.reject);
          }
          return deferred.promise;
        };
        /**
       * @name applyNotFoundIndicators
       * @private
       *
       * @description
       * Applies not fount indicators to given translation id, if needed.
       * This function gets only executed, if a translation id doesn't exist,
       * which is why a translation id is expected as argument.
       *
       * @param {string} translationId Translation id.
       * @returns {string} Same as given translation id but applied with not found
       * indicators.
       */
        var applyNotFoundIndicators = function (translationId) {
          // applying notFoundIndicators
          if ($notFoundIndicatorLeft) {
            translationId = [
              $notFoundIndicatorLeft,
              translationId
            ].join(' ');
          }
          if ($notFoundIndicatorRight) {
            translationId = [
              translationId,
              $notFoundIndicatorRight
            ].join(' ');
          }
          return translationId;
        };
        /**
       * @name useLanguage
       * @private
       *
       * @description
       * Makes actual use of a language by setting a given language key as used
       * language and informs registered interpolators to also use the given
       * key as locale.
       *
       * @param {key} Locale key.
       */
        var useLanguage = function (key) {
          $uses = key;
          $rootScope.$emit('$translateChangeSuccess', { language: key });
          if ($storageFactory) {
            Storage.put($translate.storageKey(), $uses);
          }
          // inform default interpolator
          defaultInterpolator.setLocale($uses);
          // inform all others too!
          angular.forEach(interpolatorHashMap, function (interpolator, id) {
            interpolatorHashMap[id].setLocale($uses);
          });
          $rootScope.$emit('$translateChangeEnd', { language: key });
        };
        /**
       * @name loadAsync
       * @private
       *
       * @description
       * Kicks of registered async loader using `$injector` and applies existing
       * loader options. When resolved, it updates translation tables accordingly
       * or rejects with given language key.
       *
       * @param {string} key Language key.
       * @return {Promise} A promise.
       */
        var loadAsync = function (key) {
          if (!key) {
            throw 'No language key specified for loading.';
          }
          var deferred = $q.defer();
          $rootScope.$emit('$translateLoadingStart', { language: key });
          pendingLoader = true;
          var cache = loaderCache;
          if (typeof cache === 'string') {
            // getting on-demand instance of loader
            cache = $injector.get(cache);
          }
          var loaderOptions = angular.extend({}, $loaderOptions, {
              key: key,
              $http: angular.extend({}, { cache: cache }, $loaderOptions.$http)
            });
          $injector.get($loaderFactory)(loaderOptions).then(function (data) {
            var translationTable = {};
            $rootScope.$emit('$translateLoadingSuccess', { language: key });
            if (angular.isArray(data)) {
              angular.forEach(data, function (table) {
                angular.extend(translationTable, flatObject(table));
              });
            } else {
              angular.extend(translationTable, flatObject(data));
            }
            pendingLoader = false;
            deferred.resolve({
              key: key,
              table: translationTable
            });
            $rootScope.$emit('$translateLoadingEnd', { language: key });
          }, function (key) {
            $rootScope.$emit('$translateLoadingError', { language: key });
            deferred.reject(key);
            $rootScope.$emit('$translateLoadingEnd', { language: key });
          });
          return deferred.promise;
        };
        if ($storageFactory) {
          Storage = $injector.get($storageFactory);
          if (!Storage.get || !Storage.put) {
            throw new Error('Couldn\'t use storage \'' + $storageFactory + '\', missing get() or put() method!');
          }
        }
        // apply additional settings
        if (angular.isFunction(defaultInterpolator.useSanitizeValueStrategy)) {
          defaultInterpolator.useSanitizeValueStrategy($interpolationSanitizationStrategy);
        }
        // if we have additional interpolations that were added via
        // $translateProvider.addInterpolation(), we have to map'em
        if ($interpolatorFactories.length) {
          angular.forEach($interpolatorFactories, function (interpolatorFactory) {
            var interpolator = $injector.get(interpolatorFactory);
            // setting initial locale for each interpolation service
            interpolator.setLocale($preferredLanguage || $uses);
            // apply additional settings
            if (angular.isFunction(interpolator.useSanitizeValueStrategy)) {
              interpolator.useSanitizeValueStrategy($interpolationSanitizationStrategy);
            }
            // make'em recognizable through id
            interpolatorHashMap[interpolator.getInterpolationIdentifier()] = interpolator;
          });
        }
        /**
       * @name getTranslationTable
       * @private
       *
       * @description
       * Returns a promise that resolves to the translation table
       * or is rejected if an error occurred.
       *
       * @param langKey
       * @returns {Q.promise}
       */
        var getTranslationTable = function (langKey) {
          var deferred = $q.defer();
          if (Object.prototype.hasOwnProperty.call($translationTable, langKey)) {
            deferred.resolve($translationTable[langKey]);
          } else if (langPromises[langKey]) {
            langPromises[langKey].then(function (data) {
              translations(data.key, data.table);
              deferred.resolve(data.table);
            }, deferred.reject);
          } else {
            deferred.reject();
          }
          return deferred.promise;
        };
        /**
       * @name getFallbackTranslation
       * @private
       *
       * @description
       * Returns a promise that will resolve to the translation
       * or be rejected if no translation was found for the language.
       * This function is currently only used for fallback language translation.
       *
       * @param langKey The language to translate to.
       * @param translationId
       * @param interpolateParams
       * @param Interpolator
       * @returns {Q.promise}
       */
        var getFallbackTranslation = function (langKey, translationId, interpolateParams, Interpolator) {
          var deferred = $q.defer();
          getTranslationTable(langKey).then(function (translationTable) {
            if (Object.prototype.hasOwnProperty.call(translationTable, translationId)) {
              Interpolator.setLocale(langKey);
              var translation = translationTable[translationId];
              if (translation.substr(0, 2) === '@:') {
                getFallbackTranslation(langKey, translation.substr(2), interpolateParams, Interpolator).then(deferred.resolve, deferred.reject);
              } else {
                deferred.resolve(Interpolator.interpolate(translationTable[translationId], interpolateParams));
              }
              Interpolator.setLocale($uses);
            } else {
              deferred.reject();
            }
          }, deferred.reject);
          return deferred.promise;
        };
        /**
       * @name getFallbackTranslationInstant
       * @private
       *
       * @description
       * Returns a translation
       * This function is currently only used for fallback language translation.
       *
       * @param langKey The language to translate to.
       * @param translationId
       * @param interpolateParams
       * @param Interpolator
       * @returns {string} translation
       */
        var getFallbackTranslationInstant = function (langKey, translationId, interpolateParams, Interpolator) {
          var result, translationTable = $translationTable[langKey];
          if (translationTable && Object.prototype.hasOwnProperty.call(translationTable, translationId)) {
            Interpolator.setLocale(langKey);
            result = Interpolator.interpolate(translationTable[translationId], interpolateParams);
            if (result.substr(0, 2) === '@:') {
              return getFallbackTranslationInstant(langKey, result.substr(2), interpolateParams, Interpolator);
            }
            Interpolator.setLocale($uses);
          }
          return result;
        };
        /**
       * @name translateByHandler
       * @private
       *
       * Translate by missing translation handler.
       *
       * @param translationId
       * @returns translation created by $missingTranslationHandler or translationId is $missingTranslationHandler is
       * absent
       */
        var translateByHandler = function (translationId) {
          // If we have a handler factory - we might also call it here to determine if it provides
          // a default text for a translationid that can't be found anywhere in our tables
          if ($missingTranslationHandlerFactory) {
            var resultString = $injector.get($missingTranslationHandlerFactory)(translationId, $uses);
            if (resultString !== undefined) {
              return resultString;
            } else {
              return translationId;
            }
          } else {
            return translationId;
          }
        };
        /**
       * @name resolveForFallbackLanguage
       * @private
       *
       * Recursive helper function for fallbackTranslation that will sequentially look
       * for a translation in the fallbackLanguages starting with fallbackLanguageIndex.
       *
       * @param fallbackLanguageIndex
       * @param translationId
       * @param interpolateParams
       * @param Interpolator
       * @returns {Q.promise} Promise that will resolve to the translation.
       */
        var resolveForFallbackLanguage = function (fallbackLanguageIndex, translationId, interpolateParams, Interpolator, defaultTranslationText) {
          var deferred = $q.defer();
          if (fallbackLanguageIndex < $fallbackLanguage.length) {
            var langKey = $fallbackLanguage[fallbackLanguageIndex];
            getFallbackTranslation(langKey, translationId, interpolateParams, Interpolator).then(deferred.resolve, function () {
              // Look in the next fallback language for a translation.
              // It delays the resolving by passing another promise to resolve.
              resolveForFallbackLanguage(fallbackLanguageIndex + 1, translationId, interpolateParams, Interpolator, defaultTranslationText).then(deferred.resolve);
            });
          } else {
            // No translation found in any fallback language
            // if a default translation text is set in the directive, then return this as a result
            if (defaultTranslationText) {
              deferred.resolve(defaultTranslationText);
            } else {
              // if no default translation is set and an error handler is defined, send it to the handler
              // and then return the result
              deferred.resolve(translateByHandler(translationId));
            }
          }
          return deferred.promise;
        };
        /**
       * @name resolveForFallbackLanguageInstant
       * @private
       *
       * Recursive helper function for fallbackTranslation that will sequentially look
       * for a translation in the fallbackLanguages starting with fallbackLanguageIndex.
       *
       * @param fallbackLanguageIndex
       * @param translationId
       * @param interpolateParams
       * @param Interpolator
       * @returns {string} translation
       */
        var resolveForFallbackLanguageInstant = function (fallbackLanguageIndex, translationId, interpolateParams, Interpolator) {
          var result;
          if (fallbackLanguageIndex < $fallbackLanguage.length) {
            var langKey = $fallbackLanguage[fallbackLanguageIndex];
            result = getFallbackTranslationInstant(langKey, translationId, interpolateParams, Interpolator);
            if (!result) {
              result = resolveForFallbackLanguageInstant(fallbackLanguageIndex + 1, translationId, interpolateParams, Interpolator);
            }
          }
          return result;
        };
        /**
       * Translates with the usage of the fallback languages.
       *
       * @param translationId
       * @param interpolateParams
       * @param Interpolator
       * @returns {Q.promise} Promise, that resolves to the translation.
       */
        var fallbackTranslation = function (translationId, interpolateParams, Interpolator, defaultTranslationText) {
          // Start with the fallbackLanguage with index 0
          return resolveForFallbackLanguage(startFallbackIteration > 0 ? startFallbackIteration : fallbackIndex, translationId, interpolateParams, Interpolator, defaultTranslationText);
        };
        /**
       * Translates with the usage of the fallback languages.
       *
       * @param translationId
       * @param interpolateParams
       * @param Interpolator
       * @returns {String} translation
       */
        var fallbackTranslationInstant = function (translationId, interpolateParams, Interpolator) {
          // Start with the fallbackLanguage with index 0
          return resolveForFallbackLanguageInstant(startFallbackIteration > 0 ? startFallbackIteration : fallbackIndex, translationId, interpolateParams, Interpolator);
        };
        var determineTranslation = function (translationId, interpolateParams, interpolationId, defaultTranslationText) {
          var deferred = $q.defer();
          var table = $uses ? $translationTable[$uses] : $translationTable, Interpolator = interpolationId ? interpolatorHashMap[interpolationId] : defaultInterpolator;
          // if the translation id exists, we can just interpolate it
          if (table && Object.prototype.hasOwnProperty.call(table, translationId)) {
            var translation = table[translationId];
            // If using link, rerun $translate with linked translationId and return it
            if (translation.substr(0, 2) === '@:') {
              $translate(translation.substr(2), interpolateParams, interpolationId, defaultTranslationText).then(deferred.resolve, deferred.reject);
            } else {
              deferred.resolve(Interpolator.interpolate(translation, interpolateParams));
            }
          } else {
            var missingTranslationHandlerTranslation;
            // for logging purposes only (as in $translateMissingTranslationHandlerLog), value is not returned to promise
            if ($missingTranslationHandlerFactory && !pendingLoader) {
              missingTranslationHandlerTranslation = translateByHandler(translationId);
            }
            // since we couldn't translate the inital requested translation id,
            // we try it now with one or more fallback languages, if fallback language(s) is
            // configured.
            if ($uses && $fallbackLanguage && $fallbackLanguage.length) {
              fallbackTranslation(translationId, interpolateParams, Interpolator, defaultTranslationText).then(function (translation) {
                deferred.resolve(translation);
              }, function (_translationId) {
                deferred.reject(applyNotFoundIndicators(_translationId));
              });
            } else if ($missingTranslationHandlerFactory && !pendingLoader && missingTranslationHandlerTranslation) {
              // looks like the requested translation id doesn't exists.
              // Now, if there is a registered handler for missing translations and no
              // asyncLoader is pending, we execute the handler
              if (defaultTranslationText) {
                deferred.resolve(defaultTranslationText);
              } else {
                deferred.resolve(missingTranslationHandlerTranslation);
              }
            } else {
              if (defaultTranslationText) {
                deferred.resolve(defaultTranslationText);
              } else {
                deferred.reject(applyNotFoundIndicators(translationId));
              }
            }
          }
          return deferred.promise;
        };
        var determineTranslationInstant = function (translationId, interpolateParams, interpolationId) {
          var result, table = $uses ? $translationTable[$uses] : $translationTable, Interpolator = defaultInterpolator;
          // if the interpolation id exists use custom interpolator
          if (interpolatorHashMap && Object.prototype.hasOwnProperty.call(interpolatorHashMap, interpolationId)) {
            Interpolator = interpolatorHashMap[interpolationId];
          }
          // if the translation id exists, we can just interpolate it
          if (table && Object.prototype.hasOwnProperty.call(table, translationId)) {
            var translation = table[translationId];
            // If using link, rerun $translate with linked translationId and return it
            if (translation.substr(0, 2) === '@:') {
              result = determineTranslationInstant(translation.substr(2), interpolateParams, interpolationId);
            } else {
              result = Interpolator.interpolate(translation, interpolateParams);
            }
          } else {
            var missingTranslationHandlerTranslation;
            // for logging purposes only (as in $translateMissingTranslationHandlerLog), value is not returned to promise
            if ($missingTranslationHandlerFactory && !pendingLoader) {
              missingTranslationHandlerTranslation = translateByHandler(translationId);
            }
            // since we couldn't translate the inital requested translation id,
            // we try it now with one or more fallback languages, if fallback language(s) is
            // configured.
            if ($uses && $fallbackLanguage && $fallbackLanguage.length) {
              fallbackIndex = 0;
              result = fallbackTranslationInstant(translationId, interpolateParams, Interpolator);
            } else if ($missingTranslationHandlerFactory && !pendingLoader && missingTranslationHandlerTranslation) {
              // looks like the requested translation id doesn't exists.
              // Now, if there is a registered handler for missing translations and no
              // asyncLoader is pending, we execute the handler
              result = missingTranslationHandlerTranslation;
            } else {
              result = applyNotFoundIndicators(translationId);
            }
          }
          return result;
        };
        /**
       * @ngdoc function
       * @name pascalprecht.translate.$translate#preferredLanguage
       * @methodOf pascalprecht.translate.$translate
       *
       * @description
       * Returns the language key for the preferred language.
       *
       * @param {string} langKey language String or Array to be used as preferredLanguage (changing at runtime)
       *
       * @return {string} preferred language key
       */
        $translate.preferredLanguage = function (langKey) {
          if (langKey) {
            setupPreferredLanguage(langKey);
          }
          return $preferredLanguage;
        };
        /**
       * @ngdoc function
       * @name pascalprecht.translate.$translate#cloakClassName
       * @methodOf pascalprecht.translate.$translate
       *
       * @description
       * Returns the configured class name for `translate-cloak` directive.
       *
       * @return {string} cloakClassName
       */
        $translate.cloakClassName = function () {
          return $cloakClassName;
        };
        /**
       * @ngdoc function
       * @name pascalprecht.translate.$translate#fallbackLanguage
       * @methodOf pascalprecht.translate.$translate
       *
       * @description
       * Returns the language key for the fallback languages or sets a new fallback stack.
       *
       * @param {string=} langKey language String or Array of fallback languages to be used (to change stack at runtime)
       *
       * @return {string||array} fallback language key
       */
        $translate.fallbackLanguage = function (langKey) {
          if (langKey !== undefined && langKey !== null) {
            fallbackStack(langKey);
            // as we might have an async loader initiated and a new translation language might have been defined
            // we need to add the promise to the stack also. So - iterate.
            if ($loaderFactory) {
              if ($fallbackLanguage && $fallbackLanguage.length) {
                for (var i = 0, len = $fallbackLanguage.length; i < len; i++) {
                  if (!langPromises[$fallbackLanguage[i]]) {
                    langPromises[$fallbackLanguage[i]] = loadAsync($fallbackLanguage[i]);
                  }
                }
              }
            }
            $translate.use($translate.use());
          }
          if ($fallbackWasString) {
            return $fallbackLanguage[0];
          } else {
            return $fallbackLanguage;
          }
        };
        /**
       * @ngdoc function
       * @name pascalprecht.translate.$translate#useFallbackLanguage
       * @methodOf pascalprecht.translate.$translate
       *
       * @description
       * Sets the first key of the fallback language stack to be used for translation.
       * Therefore all languages in the fallback array BEFORE this key will be skipped!
       *
       * @param {string=} langKey Contains the langKey the iteration shall start with. Set to false if you want to
       * get back to the whole stack
       */
        $translate.useFallbackLanguage = function (langKey) {
          if (langKey !== undefined && langKey !== null) {
            if (!langKey) {
              startFallbackIteration = 0;
            } else {
              var langKeyPosition = indexOf($fallbackLanguage, langKey);
              if (langKeyPosition > -1) {
                startFallbackIteration = langKeyPosition;
              }
            }
          }
        };
        /**
       * @ngdoc function
       * @name pascalprecht.translate.$translate#proposedLanguage
       * @methodOf pascalprecht.translate.$translate
       *
       * @description
       * Returns the language key of language that is currently loaded asynchronously.
       *
       * @return {string} language key
       */
        $translate.proposedLanguage = function () {
          return $nextLang;
        };
        /**
       * @ngdoc function
       * @name pascalprecht.translate.$translate#storage
       * @methodOf pascalprecht.translate.$translate
       *
       * @description
       * Returns registered storage.
       *
       * @return {object} Storage
       */
        $translate.storage = function () {
          return Storage;
        };
        /**
       * @ngdoc function
       * @name pascalprecht.translate.$translate#use
       * @methodOf pascalprecht.translate.$translate
       *
       * @description
       * Tells angular-translate which language to use by given language key. This method is
       * used to change language at runtime. It also takes care of storing the language
       * key in a configured store to let your app remember the choosed language.
       *
       * When trying to 'use' a language which isn't available it tries to load it
       * asynchronously with registered loaders.
       *
       * Returns promise object with loaded language file data
       * @example
       * $translate.use("en_US").then(function(data){
       *   $scope.text = $translate("HELLO");
       * });
       *
       * @param {string} key Language key
       * @return {string} Language key
       */
        $translate.use = function (key) {
          if (!key) {
            return $uses;
          }
          var deferred = $q.defer();
          $rootScope.$emit('$translateChangeStart', { language: key });
          // Try to get the aliased language key
          var aliasedKey = negotiateLocale(key);
          if (aliasedKey) {
            key = aliasedKey;
          }
          // if there isn't a translation table for the language we've requested,
          // we load it asynchronously
          if (!$translationTable[key] && $loaderFactory && !langPromises[key]) {
            $nextLang = key;
            langPromises[key] = loadAsync(key).then(function (translation) {
              translations(translation.key, translation.table);
              deferred.resolve(translation.key);
              useLanguage(translation.key);
              if ($nextLang === key) {
                $nextLang = undefined;
              }
              return translation;
            }, function (key) {
              if ($nextLang === key) {
                $nextLang = undefined;
              }
              $rootScope.$emit('$translateChangeError', { language: key });
              deferred.reject(key);
              $rootScope.$emit('$translateChangeEnd', { language: key });
            });
          } else {
            deferred.resolve(key);
            useLanguage(key);
          }
          return deferred.promise;
        };
        /**
       * @ngdoc function
       * @name pascalprecht.translate.$translate#storageKey
       * @methodOf pascalprecht.translate.$translate
       *
       * @description
       * Returns the key for the storage.
       *
       * @return {string} storage key
       */
        $translate.storageKey = function () {
          return storageKey();
        };
        /**
       * @ngdoc function
       * @name pascalprecht.translate.$translate#isPostCompilingEnabled
       * @methodOf pascalprecht.translate.$translate
       *
       * @description
       * Returns whether post compiling is enabled or not
       *
       * @return {bool} storage key
       */
        $translate.isPostCompilingEnabled = function () {
          return $postCompilingEnabled;
        };
        /**
       * @ngdoc function
       * @name pascalprecht.translate.$translate#refresh
       * @methodOf pascalprecht.translate.$translate
       *
       * @description
       * Refreshes a translation table pointed by the given langKey. If langKey is not specified,
       * the module will drop all existent translation tables and load new version of those which
       * are currently in use.
       *
       * Refresh means that the module will drop target translation table and try to load it again.
       *
       * In case there are no loaders registered the refresh() method will throw an Error.
       *
       * If the module is able to refresh translation tables refresh() method will broadcast
       * $translateRefreshStart and $translateRefreshEnd events.
       *
       * @example
       * // this will drop all currently existent translation tables and reload those which are
       * // currently in use
       * $translate.refresh();
       * // this will refresh a translation table for the en_US language
       * $translate.refresh('en_US');
       *
       * @param {string} langKey A language key of the table, which has to be refreshed
       *
       * @return {promise} Promise, which will be resolved in case a translation tables refreshing
       * process is finished successfully, and reject if not.
       */
        $translate.refresh = function (langKey) {
          if (!$loaderFactory) {
            throw new Error('Couldn\'t refresh translation table, no loader registered!');
          }
          var deferred = $q.defer();
          function resolve() {
            deferred.resolve();
            $rootScope.$emit('$translateRefreshEnd', { language: langKey });
          }
          function reject() {
            deferred.reject();
            $rootScope.$emit('$translateRefreshEnd', { language: langKey });
          }
          $rootScope.$emit('$translateRefreshStart', { language: langKey });
          if (!langKey) {
            // if there's no language key specified we refresh ALL THE THINGS!
            var tables = [], loadingKeys = {};
            // reload registered fallback languages
            if ($fallbackLanguage && $fallbackLanguage.length) {
              for (var i = 0, len = $fallbackLanguage.length; i < len; i++) {
                tables.push(loadAsync($fallbackLanguage[i]));
                loadingKeys[$fallbackLanguage[i]] = true;
              }
            }
            // reload currently used language
            if ($uses && !loadingKeys[$uses]) {
              tables.push(loadAsync($uses));
            }
            $q.all(tables).then(function (tableData) {
              angular.forEach(tableData, function (data) {
                if ($translationTable[data.key]) {
                  delete $translationTable[data.key];
                }
                translations(data.key, data.table);
              });
              if ($uses) {
                useLanguage($uses);
              }
              resolve();
            });
          } else if ($translationTable[langKey]) {
            loadAsync(langKey).then(function (data) {
              translations(data.key, data.table);
              if (langKey === $uses) {
                useLanguage($uses);
              }
              resolve();
            }, reject);
          } else {
            reject();
          }
          return deferred.promise;
        };
        /**
       * @ngdoc function
       * @name pascalprecht.translate.$translate#instant
       * @methodOf pascalprecht.translate.$translate
       *
       * @description
       * Returns a translation instantly from the internal state of loaded translation. All rules
       * regarding the current language, the preferred language of even fallback languages will be
       * used except any promise handling. If a language was not found, an asynchronous loading
       * will be invoked in the background.
       *
       * @param {string|array} translationId A token which represents a translation id
       *                                     This can be optionally an array of translation ids which
       *                                     results that the function's promise returns an object where
       *                                     each key is the translation id and the value the translation.
       * @param {object} interpolateParams Params
       * @param {string} interpolationId The id of the interpolation to use
       *
       * @return {string} translation
       */
        $translate.instant = function (translationId, interpolateParams, interpolationId) {
          // Detect undefined and null values to shorten the execution and prevent exceptions
          if (translationId === null || angular.isUndefined(translationId)) {
            return translationId;
          }
          // Duck detection: If the first argument is an array, a bunch of translations was requested.
          // The result is an object.
          if (angular.isArray(translationId)) {
            var results = {};
            for (var i = 0, c = translationId.length; i < c; i++) {
              results[translationId[i]] = $translate.instant(translationId[i], interpolateParams, interpolationId);
            }
            return results;
          }
          // We discarded unacceptable values. So we just need to verify if translationId is empty String
          if (angular.isString(translationId) && translationId.length < 1) {
            return translationId;
          }
          // trim off any whitespace
          if (translationId) {
            translationId = trim.apply(translationId);
          }
          var result, possibleLangKeys = [];
          if ($preferredLanguage) {
            possibleLangKeys.push($preferredLanguage);
          }
          if ($uses) {
            possibleLangKeys.push($uses);
          }
          if ($fallbackLanguage && $fallbackLanguage.length) {
            possibleLangKeys = possibleLangKeys.concat($fallbackLanguage);
          }
          for (var j = 0, d = possibleLangKeys.length; j < d; j++) {
            var possibleLangKey = possibleLangKeys[j];
            if ($translationTable[possibleLangKey]) {
              if (typeof $translationTable[possibleLangKey][translationId] !== 'undefined') {
                result = determineTranslationInstant(translationId, interpolateParams, interpolationId);
              } else if ($notFoundIndicatorLeft || $notFoundIndicatorRight) {
                result = applyNotFoundIndicators(translationId);
              }
            }
            if (typeof result !== 'undefined') {
              break;
            }
          }
          if (!result && result !== '') {
            // Return translation of default interpolator if not found anything.
            result = defaultInterpolator.interpolate(translationId, interpolateParams);
            if ($missingTranslationHandlerFactory && !pendingLoader) {
              result = translateByHandler(translationId);
            }
          }
          return result;
        };
        /**
       * @ngdoc function
       * @name pascalprecht.translate.$translate#versionInfo
       * @methodOf pascalprecht.translate.$translate
       *
       * @description
       * Returns the current version information for the angular-translate library
       *
       * @return {string} angular-translate version
       */
        $translate.versionInfo = function () {
          return version;
        };
        /**
       * @ngdoc function
       * @name pascalprecht.translate.$translate#loaderCache
       * @methodOf pascalprecht.translate.$translate
       *
       * @description
       * Returns the defined loaderCache.
       *
       * @return {boolean|string|object} current value of loaderCache
       */
        $translate.loaderCache = function () {
          return loaderCache;
        };
        // internal purpose only
        $translate.directivePriority = function () {
          return directivePriority;
        };
        if ($loaderFactory) {
          // If at least one async loader is defined and there are no
          // (default) translations available we should try to load them.
          if (angular.equals($translationTable, {})) {
            $translate.use($translate.use());
          }
          // Also, if there are any fallback language registered, we start
          // loading them asynchronously as soon as we can.
          if ($fallbackLanguage && $fallbackLanguage.length) {
            var processAsyncResult = function (translation) {
              translations(translation.key, translation.table);
              $rootScope.$emit('$translateChangeEnd', { language: translation.key });
              return translation;
            };
            for (var i = 0, len = $fallbackLanguage.length; i < len; i++) {
              langPromises[$fallbackLanguage[i]] = loadAsync($fallbackLanguage[i]).then(processAsyncResult);
            }
          }
        }
        return $translate;
      }
    ];
  }
]);
/**
 * @ngdoc object
 * @name pascalprecht.translate.$translateDefaultInterpolation
 * @requires $interpolate
 *
 * @description
 * Uses angular's `$interpolate` services to interpolate strings against some values.
 *
 * @return {object} $translateInterpolator Interpolator service
 */
angular.module('pascalprecht.translate').factory('$translateDefaultInterpolation', [
  '$interpolate',
  function ($interpolate) {
    var $translateInterpolator = {}, $locale, $identifier = 'default', $sanitizeValueStrategy = null,
      // map of all sanitize strategies
      sanitizeValueStrategies = {
        escaped: function (params) {
          var result = {};
          for (var key in params) {
            if (Object.prototype.hasOwnProperty.call(params, key)) {
              if (angular.isNumber(params[key])) {
                result[key] = params[key];
              } else {
                result[key] = angular.element('<div></div>').text(params[key]).html();
              }
            }
          }
          return result;
        }
      };
    var sanitizeParams = function (params) {
      var result;
      if (angular.isFunction(sanitizeValueStrategies[$sanitizeValueStrategy])) {
        result = sanitizeValueStrategies[$sanitizeValueStrategy](params);
      } else {
        result = params;
      }
      return result;
    };
    /**
   * @ngdoc function
   * @name pascalprecht.translate.$translateDefaultInterpolation#setLocale
   * @methodOf pascalprecht.translate.$translateDefaultInterpolation
   *
   * @description
   * Sets current locale (this is currently not use in this interpolation).
   *
   * @param {string} locale Language key or locale.
   */
    $translateInterpolator.setLocale = function (locale) {
      $locale = locale;
    };
    /**
   * @ngdoc function
   * @name pascalprecht.translate.$translateDefaultInterpolation#getInterpolationIdentifier
   * @methodOf pascalprecht.translate.$translateDefaultInterpolation
   *
   * @description
   * Returns an identifier for this interpolation service.
   *
   * @returns {string} $identifier
   */
    $translateInterpolator.getInterpolationIdentifier = function () {
      return $identifier;
    };
    $translateInterpolator.useSanitizeValueStrategy = function (value) {
      $sanitizeValueStrategy = value;
      return this;
    };
    /**
   * @ngdoc function
   * @name pascalprecht.translate.$translateDefaultInterpolation#interpolate
   * @methodOf pascalprecht.translate.$translateDefaultInterpolation
   *
   * @description
   * Interpolates given string agains given interpolate params using angulars
   * `$interpolate` service.
   *
   * @returns {string} interpolated string.
   */
    $translateInterpolator.interpolate = function (string, interpolateParams) {
      if ($sanitizeValueStrategy) {
        interpolateParams = sanitizeParams(interpolateParams);
      }
      return $interpolate(string)(interpolateParams || {});
    };
    return $translateInterpolator;
  }
]);
angular.module('pascalprecht.translate').constant('$STORAGE_KEY', 'NG_TRANSLATE_LANG_KEY');
angular.module('pascalprecht.translate').directive('translate', [
  '$translate',
  '$q',
  '$interpolate',
  '$compile',
  '$parse',
  '$rootScope',
  function ($translate, $q, $interpolate, $compile, $parse, $rootScope) {
    /**
   * @name trim
   * @private
   *
   * @description
   * trim polyfill
   *
   * @returns {string} The string stripped of whitespace from both ends
   */
    var trim = function () {
      return this.replace(/^\s+|\s+$/g, '');
    };
    return {
      restrict: 'AE',
      scope: true,
      priority: $translate.directivePriority(),
      compile: function (tElement, tAttr) {
        var translateValuesExist = tAttr.translateValues ? tAttr.translateValues : undefined;
        var translateInterpolation = tAttr.translateInterpolation ? tAttr.translateInterpolation : undefined;
        var translateValueExist = tElement[0].outerHTML.match(/translate-value-+/i);
        var interpolateRegExp = '^(.*)(' + $interpolate.startSymbol() + '.*' + $interpolate.endSymbol() + ')(.*)', watcherRegExp = '^(.*)' + $interpolate.startSymbol() + '(.*)' + $interpolate.endSymbol() + '(.*)';
        return function linkFn(scope, iElement, iAttr) {
          scope.interpolateParams = {};
          scope.preText = '';
          scope.postText = '';
          var translationIds = {};
          // Ensures any change of the attribute "translate" containing the id will
          // be re-stored to the scope's "translationId".
          // If the attribute has no content, the element's text value (white spaces trimmed off) will be used.
          var observeElementTranslation = function (translationId) {
            // Remove any old watcher
            if (angular.isFunction(observeElementTranslation._unwatchOld)) {
              observeElementTranslation._unwatchOld();
              observeElementTranslation._unwatchOld = undefined;
            }
            if (angular.equals(translationId, '') || !angular.isDefined(translationId)) {
              // Resolve translation id by inner html if required
              var interpolateMatches = trim.apply(iElement.text()).match(interpolateRegExp);
              // Interpolate translation id if required
              if (angular.isArray(interpolateMatches)) {
                scope.preText = interpolateMatches[1];
                scope.postText = interpolateMatches[3];
                translationIds.translate = $interpolate(interpolateMatches[2])(scope.$parent);
                var watcherMatches = iElement.text().match(watcherRegExp);
                if (angular.isArray(watcherMatches) && watcherMatches[2] && watcherMatches[2].length) {
                  observeElementTranslation._unwatchOld = scope.$watch(watcherMatches[2], function (newValue) {
                    translationIds.translate = newValue;
                    updateTranslations();
                  });
                }
              } else {
                translationIds.translate = iElement.text().replace(/^\s+|\s+$/g, '');
              }
            } else {
              translationIds.translate = translationId;
            }
            updateTranslations();
          };
          var observeAttributeTranslation = function (translateAttr) {
            iAttr.$observe(translateAttr, function (translationId) {
              translationIds[translateAttr] = translationId;
              updateTranslations();
            });
          };
          var firstAttributeChangedEvent = true;
          iAttr.$observe('translate', function (translationId) {
            if (typeof translationId === 'undefined') {
              // case of element "<translate>xyz</translate>"
              observeElementTranslation('');
            } else {
              // case of regular attribute
              if (translationId !== '' || !firstAttributeChangedEvent) {
                translationIds.translate = translationId;
                updateTranslations();
              }
            }
            firstAttributeChangedEvent = false;
          });
          for (var translateAttr in iAttr) {
            if (iAttr.hasOwnProperty(translateAttr) && translateAttr.substr(0, 13) === 'translateAttr') {
              observeAttributeTranslation(translateAttr);
            }
          }
          iAttr.$observe('translateDefault', function (value) {
            scope.defaultText = value;
          });
          if (translateValuesExist) {
            iAttr.$observe('translateValues', function (interpolateParams) {
              if (interpolateParams) {
                scope.$parent.$watch(function () {
                  angular.extend(scope.interpolateParams, $parse(interpolateParams)(scope.$parent));
                });
              }
            });
          }
          if (translateValueExist) {
            var observeValueAttribute = function (attrName) {
              iAttr.$observe(attrName, function (value) {
                var attributeName = angular.lowercase(attrName.substr(14, 1)) + attrName.substr(15);
                scope.interpolateParams[attributeName] = value;
              });
            };
            for (var attr in iAttr) {
              if (Object.prototype.hasOwnProperty.call(iAttr, attr) && attr.substr(0, 14) === 'translateValue' && attr !== 'translateValues') {
                observeValueAttribute(attr);
              }
            }
          }
          // Master update function
          var updateTranslations = function () {
            for (var key in translationIds) {
              if (translationIds.hasOwnProperty(key)) {
                updateTranslation(key, translationIds[key], scope, scope.interpolateParams, scope.defaultText);
              }
            }
          };
          // Put translation processing function outside loop
          var updateTranslation = function (translateAttr, translationId, scope, interpolateParams, defaultTranslationText) {
            if (translationId) {
              $translate(translationId, interpolateParams, translateInterpolation, defaultTranslationText).then(function (translation) {
                applyTranslation(translation, scope, true, translateAttr);
              }, function (translationId) {
                applyTranslation(translationId, scope, false, translateAttr);
              });
            } else {
              // as an empty string cannot be translated, we can solve this using successful=false
              applyTranslation(translationId, scope, false, translateAttr);
            }
          };
          var applyTranslation = function (value, scope, successful, translateAttr) {
            if (translateAttr === 'translate') {
              // default translate into innerHTML
              if (!successful && typeof scope.defaultText !== 'undefined') {
                value = scope.defaultText;
              }
              iElement.html(scope.preText + value + scope.postText);
              var globallyEnabled = $translate.isPostCompilingEnabled();
              var locallyDefined = typeof tAttr.translateCompile !== 'undefined';
              var locallyEnabled = locallyDefined && tAttr.translateCompile !== 'false';
              if (globallyEnabled && !locallyDefined || locallyEnabled) {
                $compile(iElement.contents())(scope);
              }
            } else {
              // translate attribute
              if (!successful && typeof scope.defaultText !== 'undefined') {
                value = scope.defaultText;
              }
              var attributeName = iAttr.$attr[translateAttr].substr(15);
              iElement.attr(attributeName, value);
            }
          };
          scope.$watch('interpolateParams', updateTranslations, true);
          // Ensures the text will be refreshed after the current language was changed
          // w/ $translate.use(...)
          var unbind = $rootScope.$on('$translateChangeSuccess', updateTranslations);
          // ensure translation will be looked up at least one
          if (iElement.text().length) {
            observeElementTranslation('');
          }
          updateTranslations();
          scope.$on('$destroy', unbind);
        };
      }
    };
  }
]);
angular.module('pascalprecht.translate').directive('translateCloak', [
  '$rootScope',
  '$translate',
  function ($rootScope, $translate) {
    return {
      compile: function (tElement) {
        var applyCloak = function () {
            tElement.addClass($translate.cloakClassName());
          }, removeCloak = function () {
            tElement.removeClass($translate.cloakClassName());
          }, removeListener = $rootScope.$on('$translateChangeEnd', function () {
            removeCloak();
            removeListener();
            removeListener = null;
          });
        applyCloak();
        return function linkFn(scope, iElement, iAttr) {
          // Register a watcher for the defined translation allowing a fine tuned cloak
          if (iAttr.translateCloak && iAttr.translateCloak.length) {
            iAttr.$observe('translateCloak', function (translationId) {
              $translate(translationId).then(removeCloak, applyCloak);
            });
          }
        };
      }
    };
  }
]);
angular.module('pascalprecht.translate').filter('translate', [
  '$parse',
  '$translate',
  function ($parse, $translate) {
    var translateFilter = function (translationId, interpolateParams, interpolation) {
      if (!angular.isObject(interpolateParams)) {
        interpolateParams = $parse(interpolateParams)(this);
      }
      return $translate.instant(translationId, interpolateParams, interpolation);
    };
    // Since AngularJS 1.3, filters which are not stateless (depending at the scope)
    // have to explicit define this behavior.
    translateFilter.$stateful = true;
    return translateFilter;
  }
]);
//    PouchDB 3.4.0
//    
//    (c) 2012-2015 Dale Harvey and the PouchDB team
//    PouchDB may be freely distributed under the Apache license, version 2.0.
//    For all details and documentation:
//    http://pouchdb.com
!function (e) {
  if ('object' == typeof exports && 'undefined' != typeof module)
    module.exports = e();
  else if ('function' == typeof define && define.amd)
    define([], e);
  else {
    var f;
    'undefined' != typeof window ? f = window : 'undefined' != typeof global ? f = global : 'undefined' != typeof self && (f = self), f.PouchDB = e();
  }
}(function () {
  var define, module, exports;
  return function e(t, n, r) {
    function s(o, u) {
      if (!n[o]) {
        if (!t[o]) {
          var a = typeof require == 'function' && require;
          if (!u && a)
            return a(o, !0);
          if (i)
            return i(o, !0);
          var f = new Error('Cannot find module \'' + o + '\'');
          throw f.code = 'MODULE_NOT_FOUND', f;
        }
        var l = n[o] = { exports: {} };
        t[o][0].call(l.exports, function (e) {
          var n = t[o][1][e];
          return s(n ? n : e);
        }, l, l.exports, e, t, n, r);
      }
      return n[o].exports;
    }
    var i = typeof require == 'function' && require;
    for (var o = 0; o < r.length; o++)
      s(r[o]);
    return s;
  }({
    1: [
      function (_dereq_, module, exports) {
        'use strict';
        var utils = _dereq_(35);
        var merge = _dereq_(30);
        var errors = _dereq_(20);
        var EventEmitter = _dereq_(39).EventEmitter;
        var upsert = _dereq_(26);
        var Changes = _dereq_(14);
        var Promise = utils.Promise;
        /*
 * A generic pouch adapter
 */
        // returns first element of arr satisfying callback predicate
        function arrayFirst(arr, callback) {
          for (var i = 0; i < arr.length; i++) {
            if (callback(arr[i], i) === true) {
              return arr[i];
            }
          }
          return false;
        }
        // Wrapper for functions that call the bulkdocs api with a single doc,
        // if the first result is an error, return an error
        function yankError(callback) {
          return function (err, results) {
            if (err || results[0] && results[0].error) {
              callback(err || results[0]);
            } else {
              callback(null, results.length ? results[0] : results);
            }
          };
        }
        // for every node in a revision tree computes its distance from the closest
        // leaf
        function computeHeight(revs) {
          var height = {};
          var edges = [];
          merge.traverseRevTree(revs, function (isLeaf, pos, id, prnt) {
            var rev = pos + '-' + id;
            if (isLeaf) {
              height[rev] = 0;
            }
            if (prnt !== undefined) {
              edges.push({
                from: prnt,
                to: rev
              });
            }
            return rev;
          });
          edges.reverse();
          edges.forEach(function (edge) {
            if (height[edge.from] === undefined) {
              height[edge.from] = 1 + height[edge.to];
            } else {
              height[edge.from] = Math.min(height[edge.from], 1 + height[edge.to]);
            }
          });
          return height;
        }
        function allDocsKeysQuery(api, opts, callback) {
          var keys = 'limit' in opts ? opts.keys.slice(opts.skip, opts.limit + opts.skip) : opts.skip > 0 ? opts.keys.slice(opts.skip) : opts.keys;
          if (opts.descending) {
            keys.reverse();
          }
          if (!keys.length) {
            return api._allDocs({ limit: 0 }, callback);
          }
          var finalResults = { offset: opts.skip };
          return Promise.all(keys.map(function (key) {
            var subOpts = utils.extend(true, {
                key: key,
                deleted: 'ok'
              }, opts);
            [
              'limit',
              'skip',
              'keys'
            ].forEach(function (optKey) {
              delete subOpts[optKey];
            });
            return new Promise(function (resolve, reject) {
              api._allDocs(subOpts, function (err, res) {
                if (err) {
                  return reject(err);
                }
                finalResults.total_rows = res.total_rows;
                resolve(res.rows[0] || {
                  key: key,
                  error: 'not_found'
                });
              });
            });
          })).then(function (results) {
            finalResults.rows = results;
            return finalResults;
          });
        }
        utils.inherits(AbstractPouchDB, EventEmitter);
        module.exports = AbstractPouchDB;
        function AbstractPouchDB() {
          var self = this;
          EventEmitter.call(this);
          var listeners = 0, changes;
          var eventNames = [
              'change',
              'delete',
              'create',
              'update'
            ];
          this.on('newListener', function (eventName) {
            if (~eventNames.indexOf(eventName)) {
              if (listeners) {
                listeners++;
                return;
              } else {
                listeners++;
              }
            } else {
              return;
            }
            var lastChange = 0;
            changes = this.changes({
              conflicts: true,
              include_docs: true,
              continuous: true,
              since: 'now',
              onChange: function (change) {
                if (change.seq <= lastChange) {
                  return;
                }
                lastChange = change.seq;
                self.emit('change', change);
                if (change.doc._deleted) {
                  self.emit('delete', change);
                } else if (change.doc._rev.split('-')[0] === '1') {
                  self.emit('create', change);
                } else {
                  self.emit('update', change);
                }
              }
            });
          });
          this.on('removeListener', function (eventName) {
            if (~eventNames.indexOf(eventName)) {
              listeners--;
              if (listeners) {
                return;
              }
            } else {
              return;
            }
            changes.cancel();
          });
        }
        AbstractPouchDB.prototype.post = utils.adapterFun('post', function (doc, opts, callback) {
          if (typeof opts === 'function') {
            callback = opts;
            opts = {};
          }
          if (typeof doc !== 'object' || Array.isArray(doc)) {
            return callback(errors.error(errors.NOT_AN_OBJECT));
          }
          this.bulkDocs({ docs: [doc] }, opts, yankError(callback));
        });
        AbstractPouchDB.prototype.put = utils.adapterFun('put', utils.getArguments(function (args) {
          var temp, temptype, opts, callback;
          var doc = args.shift();
          var id = '_id' in doc;
          if (typeof doc !== 'object' || Array.isArray(doc)) {
            callback = args.pop();
            return callback(errors.error(errors.NOT_AN_OBJECT));
          }
          doc = utils.clone(doc);
          while (true) {
            temp = args.shift();
            temptype = typeof temp;
            if (temptype === 'string' && !id) {
              doc._id = temp;
              id = true;
            } else if (temptype === 'string' && id && !('_rev' in doc)) {
              doc._rev = temp;
            } else if (temptype === 'object') {
              opts = temp;
            } else if (temptype === 'function') {
              callback = temp;
            }
            if (!args.length) {
              break;
            }
          }
          opts = opts || {};
          var error = utils.invalidIdError(doc._id);
          if (error) {
            return callback(error);
          }
          if (utils.isLocalId(doc._id) && typeof this._putLocal === 'function') {
            if (doc._deleted) {
              return this._removeLocal(doc, callback);
            } else {
              return this._putLocal(doc, callback);
            }
          }
          this.bulkDocs({ docs: [doc] }, opts, yankError(callback));
        }));
        AbstractPouchDB.prototype.putAttachment = utils.adapterFun('putAttachment', function (docId, attachmentId, rev, blob, type, callback) {
          var api = this;
          if (typeof type === 'function') {
            callback = type;
            type = blob;
            blob = rev;
            rev = null;
          }
          if (typeof type === 'undefined') {
            type = blob;
            blob = rev;
            rev = null;
          }
          function createAttachment(doc) {
            doc._attachments = doc._attachments || {};
            doc._attachments[attachmentId] = {
              content_type: type,
              data: blob
            };
            return api.put(doc);
          }
          return api.get(docId).then(function (doc) {
            if (doc._rev !== rev) {
              throw errors.error(errors.REV_CONFLICT);
            }
            return createAttachment(doc);
          }, function (err) {
            // create new doc
            if (err.reason === errors.MISSING_DOC.message) {
              return createAttachment({ _id: docId });
            } else {
              throw err;
            }
          });
        });
        AbstractPouchDB.prototype.removeAttachment = utils.adapterFun('removeAttachment', function (docId, attachmentId, rev, callback) {
          var self = this;
          self.get(docId, function (err, obj) {
            if (err) {
              callback(err);
              return;
            }
            if (obj._rev !== rev) {
              callback(errors.error(errors.REV_CONFLICT));
              return;
            }
            if (!obj._attachments) {
              return callback();
            }
            delete obj._attachments[attachmentId];
            if (Object.keys(obj._attachments).length === 0) {
              delete obj._attachments;
            }
            self.put(obj, callback);
          });
        });
        AbstractPouchDB.prototype.remove = utils.adapterFun('remove', function (docOrId, optsOrRev, opts, callback) {
          var doc;
          if (typeof optsOrRev === 'string') {
            // id, rev, opts, callback style
            doc = {
              _id: docOrId,
              _rev: optsOrRev
            };
            if (typeof opts === 'function') {
              callback = opts;
              opts = {};
            }
          } else {
            // doc, opts, callback style
            doc = docOrId;
            if (typeof optsOrRev === 'function') {
              callback = optsOrRev;
              opts = {};
            } else {
              callback = opts;
              opts = optsOrRev;
            }
          }
          opts = utils.clone(opts || {});
          opts.was_delete = true;
          var newDoc = {
              _id: doc._id,
              _rev: doc._rev || opts.rev
            };
          newDoc._deleted = true;
          if (utils.isLocalId(newDoc._id) && typeof this._removeLocal === 'function') {
            return this._removeLocal(doc, callback);
          }
          this.bulkDocs({ docs: [newDoc] }, opts, yankError(callback));
        });
        AbstractPouchDB.prototype.revsDiff = utils.adapterFun('revsDiff', function (req, opts, callback) {
          if (typeof opts === 'function') {
            callback = opts;
            opts = {};
          }
          opts = utils.clone(opts);
          var ids = Object.keys(req);
          if (!ids.length) {
            return callback(null, {});
          }
          var count = 0;
          var missing = new utils.Map();
          function addToMissing(id, revId) {
            if (!missing.has(id)) {
              missing.set(id, { missing: [] });
            }
            missing.get(id).missing.push(revId);
          }
          function processDoc(id, rev_tree) {
            // Is this fast enough? Maybe we should switch to a set simulated by a map
            var missingForId = req[id].slice(0);
            merge.traverseRevTree(rev_tree, function (isLeaf, pos, revHash, ctx, opts) {
              var rev = pos + '-' + revHash;
              var idx = missingForId.indexOf(rev);
              if (idx === -1) {
                return;
              }
              missingForId.splice(idx, 1);
              if (opts.status !== 'available') {
                addToMissing(id, rev);
              }
            });
            // Traversing the tree is synchronous, so now `missingForId` contains
            // revisions that were not found in the tree
            missingForId.forEach(function (rev) {
              addToMissing(id, rev);
            });
          }
          ids.map(function (id) {
            this._getRevisionTree(id, function (err, rev_tree) {
              if (err && err.status === 404 && err.message === 'missing') {
                missing.set(id, { missing: req[id] });
              } else if (err) {
                return callback(err);
              } else {
                processDoc(id, rev_tree);
              }
              if (++count === ids.length) {
                // convert LazyMap to object
                var missingObj = {};
                missing.forEach(function (value, key) {
                  missingObj[key] = value;
                });
                return callback(null, missingObj);
              }
            });
          }, this);
        });
        // compact one document and fire callback
        // by compacting we mean removing all revisions which
        // are further from the leaf in revision tree than max_height
        AbstractPouchDB.prototype.compactDocument = utils.adapterFun('compactDocument', function (docId, maxHeight, callback) {
          var self = this;
          this._getRevisionTree(docId, function (err, revTree) {
            if (err) {
              return callback(err);
            }
            var height = computeHeight(revTree);
            var candidates = [];
            var revs = [];
            Object.keys(height).forEach(function (rev) {
              if (height[rev] > maxHeight) {
                candidates.push(rev);
              }
            });
            merge.traverseRevTree(revTree, function (isLeaf, pos, revHash, ctx, opts) {
              var rev = pos + '-' + revHash;
              if (opts.status === 'available' && candidates.indexOf(rev) !== -1) {
                revs.push(rev);
              }
            });
            self._doCompaction(docId, revs, callback);
          });
        });
        // compact the whole database using single document
        // compaction
        AbstractPouchDB.prototype.compact = utils.adapterFun('compact', function (opts, callback) {
          if (typeof opts === 'function') {
            callback = opts;
            opts = {};
          }
          var self = this;
          opts = utils.clone(opts || {});
          self.get('_local/compaction')['catch'](function () {
            return false;
          }).then(function (doc) {
            if (typeof self._compact === 'function') {
              if (doc && doc.last_seq) {
                opts.last_seq = doc.last_seq;
              }
              return self._compact(opts, callback);
            }
          });
        });
        AbstractPouchDB.prototype._compact = function (opts, callback) {
          var self = this;
          var changesOpts = {
              returnDocs: false,
              last_seq: opts.last_seq || 0
            };
          var promises = [];
          function onChange(row) {
            promises.push(self.compactDocument(row.id, 0));
          }
          function onComplete(resp) {
            var lastSeq = resp.last_seq;
            Promise.all(promises).then(function () {
              return upsert(self, '_local/compaction', function deltaFunc(doc) {
                if (!doc.last_seq || doc.last_seq < lastSeq) {
                  doc.last_seq = lastSeq;
                  return doc;
                }
                return false;  // somebody else got here first, don't update
              });
            }).then(function () {
              callback(null, { ok: true });
            })['catch'](callback);
          }
          self.changes(changesOpts).on('change', onChange).on('complete', onComplete).on('error', callback);
        };
        /* Begin api wrappers. Specific functionality to storage belongs in the 
   _[method] */
        AbstractPouchDB.prototype.get = utils.adapterFun('get', function (id, opts, callback) {
          if (typeof opts === 'function') {
            callback = opts;
            opts = {};
          }
          if (typeof id !== 'string') {
            return callback(errors.error(errors.INVALID_ID));
          }
          if (utils.isLocalId(id) && typeof this._getLocal === 'function') {
            return this._getLocal(id, callback);
          }
          var leaves = [], self = this;
          function finishOpenRevs() {
            var result = [];
            var count = leaves.length;
            if (!count) {
              return callback(null, result);
            }
            // order with open_revs is unspecified
            leaves.forEach(function (leaf) {
              self.get(id, {
                rev: leaf,
                revs: opts.revs,
                attachments: opts.attachments
              }, function (err, doc) {
                if (!err) {
                  result.push({ ok: doc });
                } else {
                  result.push({ missing: leaf });
                }
                count--;
                if (!count) {
                  callback(null, result);
                }
              });
            });
          }
          if (opts.open_revs) {
            if (opts.open_revs === 'all') {
              this._getRevisionTree(id, function (err, rev_tree) {
                if (err) {
                  return callback(err);
                }
                leaves = merge.collectLeaves(rev_tree).map(function (leaf) {
                  return leaf.rev;
                });
                finishOpenRevs();
              });
            } else {
              if (Array.isArray(opts.open_revs)) {
                leaves = opts.open_revs;
                for (var i = 0; i < leaves.length; i++) {
                  var l = leaves[i];
                  // looks like it's the only thing couchdb checks
                  if (!(typeof l === 'string' && /^\d+-/.test(l))) {
                    return callback(errors.error(errors.INVALID_REV));
                  }
                }
                finishOpenRevs();
              } else {
                return callback(errors.error(errors.UNKNOWN_ERROR, 'function_clause'));
              }
            }
            return;  // open_revs does not like other options
          }
          return this._get(id, opts, function (err, result) {
            opts = utils.clone(opts);
            if (err) {
              return callback(err);
            }
            var doc = result.doc;
            var metadata = result.metadata;
            var ctx = result.ctx;
            if (opts.conflicts) {
              var conflicts = merge.collectConflicts(metadata);
              if (conflicts.length) {
                doc._conflicts = conflicts;
              }
            }
            if (utils.isDeleted(metadata, doc._rev)) {
              doc._deleted = true;
            }
            if (opts.revs || opts.revs_info) {
              var paths = merge.rootToLeaf(metadata.rev_tree);
              var path = arrayFirst(paths, function (arr) {
                  return arr.ids.map(function (x) {
                    return x.id;
                  }).indexOf(doc._rev.split('-')[1]) !== -1;
                });
              var indexOfRev = path.ids.map(function (x) {
                  return x.id;
                }).indexOf(doc._rev.split('-')[1]) + 1;
              var howMany = path.ids.length - indexOfRev;
              path.ids.splice(indexOfRev, howMany);
              path.ids.reverse();
              if (opts.revs) {
                doc._revisions = {
                  start: path.pos + path.ids.length - 1,
                  ids: path.ids.map(function (rev) {
                    return rev.id;
                  })
                };
              }
              if (opts.revs_info) {
                var pos = path.pos + path.ids.length;
                doc._revs_info = path.ids.map(function (rev) {
                  pos--;
                  return {
                    rev: pos + '-' + rev.id,
                    status: rev.opts.status
                  };
                });
              }
            }
            if (opts.local_seq) {
              utils.info('The "local_seq" option is deprecated and will be removed');
              doc._local_seq = result.metadata.seq;
            }
            if (opts.attachments && doc._attachments) {
              var attachments = doc._attachments;
              var count = Object.keys(attachments).length;
              if (count === 0) {
                return callback(null, doc);
              }
              Object.keys(attachments).forEach(function (key) {
                this._getAttachment(attachments[key], {
                  encode: true,
                  ctx: ctx
                }, function (err, data) {
                  var att = doc._attachments[key];
                  att.data = data;
                  delete att.stub;
                  delete att.length;
                  if (!--count) {
                    callback(null, doc);
                  }
                });
              }, self);
            } else {
              if (doc._attachments) {
                for (var key in doc._attachments) {
                  if (doc._attachments.hasOwnProperty(key)) {
                    doc._attachments[key].stub = true;
                  }
                }
              }
              callback(null, doc);
            }
          });
        });
        AbstractPouchDB.prototype.getAttachment = utils.adapterFun('getAttachment', function (docId, attachmentId, opts, callback) {
          var self = this;
          if (opts instanceof Function) {
            callback = opts;
            opts = {};
          }
          opts = utils.clone(opts);
          this._get(docId, opts, function (err, res) {
            if (err) {
              return callback(err);
            }
            if (res.doc._attachments && res.doc._attachments[attachmentId]) {
              opts.ctx = res.ctx;
              self._getAttachment(res.doc._attachments[attachmentId], opts, callback);
            } else {
              return callback(errors.error(errors.MISSING_DOC));
            }
          });
        });
        AbstractPouchDB.prototype.allDocs = utils.adapterFun('allDocs', function (opts, callback) {
          if (typeof opts === 'function') {
            callback = opts;
            opts = {};
          }
          opts = utils.clone(opts);
          opts.skip = typeof opts.skip !== 'undefined' ? opts.skip : 0;
          if ('keys' in opts) {
            if (!Array.isArray(opts.keys)) {
              return callback(new TypeError('options.keys must be an array'));
            }
            var incompatibleOpt = [
                'startkey',
                'endkey',
                'key'
              ].filter(function (incompatibleOpt) {
                return incompatibleOpt in opts;
              })[0];
            if (incompatibleOpt) {
              callback(errors.error(errors.QUERY_PARSE_ERROR, 'Query parameter `' + incompatibleOpt + '` is not compatible with multi-get'));
              return;
            }
            if (this.type() !== 'http') {
              return allDocsKeysQuery(this, opts, callback);
            }
          }
          return this._allDocs(opts, callback);
        });
        AbstractPouchDB.prototype.changes = function (opts, callback) {
          if (typeof opts === 'function') {
            callback = opts;
            opts = {};
          }
          return new Changes(this, opts, callback);
        };
        AbstractPouchDB.prototype.close = utils.adapterFun('close', function (callback) {
          this._closed = true;
          return this._close(callback);
        });
        AbstractPouchDB.prototype.info = utils.adapterFun('info', function (callback) {
          var self = this;
          this._info(function (err, info) {
            if (err) {
              return callback(err);
            }
            // assume we know better than the adapter, unless it informs us
            info.db_name = info.db_name || self._db_name;
            info.auto_compaction = !!(self.auto_compaction && self.type() !== 'http');
            callback(null, info);
          });
        });
        AbstractPouchDB.prototype.id = utils.adapterFun('id', function (callback) {
          return this._id(callback);
        });
        AbstractPouchDB.prototype.type = function () {
          return typeof this._type === 'function' ? this._type() : this.adapter;
        };
        AbstractPouchDB.prototype.bulkDocs = utils.adapterFun('bulkDocs', function (req, opts, callback) {
          if (typeof opts === 'function') {
            callback = opts;
            opts = {};
          }
          opts = utils.clone(opts);
          if (Array.isArray(req)) {
            req = { docs: req };
          }
          if (!req || !req.docs || !Array.isArray(req.docs)) {
            return callback(errors.error(errors.MISSING_BULK_DOCS));
          }
          for (var i = 0; i < req.docs.length; ++i) {
            if (typeof req.docs[i] !== 'object' || Array.isArray(req.docs[i])) {
              return callback(errors.error(errors.NOT_AN_OBJECT));
            }
          }
          req = utils.clone(req);
          if (!('new_edits' in opts)) {
            if ('new_edits' in req) {
              opts.new_edits = req.new_edits;
            } else {
              opts.new_edits = true;
            }
          }
          if (!opts.new_edits && this.type() !== 'http') {
            // ensure revisions of the same doc are sorted, so that
            // the local adapter processes them correctly (#2935)
            req.docs.sort(function (a, b) {
              var idCompare = utils.compare(a._id, b._id);
              if (idCompare !== 0) {
                return idCompare;
              }
              var aStart = a._revisions ? a._revisions.start : 0;
              var bStart = b._revisions ? b._revisions.start : 0;
              return utils.compare(aStart, bStart);
            });
          }
          req.docs.forEach(function (doc) {
            if (doc._deleted) {
              delete doc._attachments;  // ignore atts for deleted docs
            }
          });
          return this._bulkDocs(req, opts, function (err, res) {
            if (err) {
              return callback(err);
            }
            if (!opts.new_edits) {
              // this is what couch does when new_edits is false
              res = res.filter(function (x) {
                return x.error;
              });
            }
            callback(null, res);
          });
        });
        AbstractPouchDB.prototype.registerDependentDatabase = utils.adapterFun('registerDependentDatabase', function (dependentDb, callback) {
          var depDB = new this.constructor(dependentDb, this.__opts || {});
          function diffFun(doc) {
            doc.dependentDbs = doc.dependentDbs || {};
            if (doc.dependentDbs[dependentDb]) {
              return false;  // no update required
            }
            doc.dependentDbs[dependentDb] = true;
            return doc;
          }
          upsert(this, '_local/_pouch_dependentDbs', diffFun, function (err) {
            if (err) {
              return callback(err);
            }
            return callback(null, { db: depDB });
          });
        });
      },
      {
        '14': 14,
        '20': 20,
        '26': 26,
        '30': 30,
        '35': 35,
        '39': 39
      }
    ],
    2: [
      function (_dereq_, module, exports) {
        (function (process) {
          'use strict';
          var CHANGES_BATCH_SIZE = 25;
          // according to http://stackoverflow.com/a/417184/680742,
          // the de factor URL length limit is 2000 characters.
          // but since most of our measurements don't take the full
          // URL into account, we fudge it a bit.
          // TODO: we could measure the full URL to enforce exactly 2000 chars
          var MAX_URL_LENGTH = 1800;
          var utils = _dereq_(35);
          var errors = _dereq_(20);
          var log = _dereq_(41)('pouchdb:http');
          var isBrowser = typeof process === 'undefined' || process.browser;
          var buffer = _dereq_(19);
          function encodeDocId(id) {
            if (/^_design/.test(id)) {
              return '_design/' + encodeURIComponent(id.slice(8));
            }
            if (/^_local/.test(id)) {
              return '_local/' + encodeURIComponent(id.slice(7));
            }
            return encodeURIComponent(id);
          }
          function preprocessAttachments(doc) {
            if (!doc._attachments || !Object.keys(doc._attachments)) {
              return utils.Promise.resolve();
            }
            return utils.Promise.all(Object.keys(doc._attachments).map(function (key) {
              var attachment = doc._attachments[key];
              if (attachment.data && typeof attachment.data !== 'string') {
                if (isBrowser) {
                  return new utils.Promise(function (resolve) {
                    utils.readAsBinaryString(attachment.data, function (binary) {
                      attachment.data = utils.btoa(binary);
                      resolve();
                    });
                  });
                } else {
                  attachment.data = attachment.data.toString('base64');
                }
              }
            }));
          }
          // Get all the information you possibly can about the URI given by name and
          // return it as a suitable object.
          function getHost(name, opts) {
            // If the given name contains "http:"
            if (/http(s?):/.test(name)) {
              // Prase the URI into all its little bits
              var uri = utils.parseUri(name);
              // Store the fact that it is a remote URI
              uri.remote = true;
              // Store the user and password as a separate auth object
              if (uri.user || uri.password) {
                uri.auth = {
                  username: uri.user,
                  password: uri.password
                };
              }
              // Split the path part of the URI into parts using '/' as the delimiter
              // after removing any leading '/' and any trailing '/'
              var parts = uri.path.replace(/(^\/|\/$)/g, '').split('/');
              // Store the first part as the database name and remove it from the parts
              // array
              uri.db = parts.pop();
              // Restore the path by joining all the remaining parts (all the parts
              // except for the database name) with '/'s
              uri.path = parts.join('/');
              opts = opts || {};
              opts = utils.clone(opts);
              uri.headers = opts.headers || opts.ajax && opts.ajax.headers || {};
              if (opts.auth || uri.auth) {
                var nAuth = opts.auth || uri.auth;
                var token = utils.btoa(nAuth.username + ':' + nAuth.password);
                uri.headers.Authorization = 'Basic ' + token;
              }
              if (opts.headers) {
                uri.headers = opts.headers;
              }
              return uri;
            }
            // If the given name does not contain 'http:' then return a very basic object
            // with no host, the current path, the given name as the database name and no
            // username/password
            return {
              host: '',
              path: '/',
              db: name,
              auth: false
            };
          }
          // Generate a URL with the host data given by opts and the given path
          function genDBUrl(opts, path) {
            return genUrl(opts, opts.db + '/' + path);
          }
          // Generate a URL with the host data given by opts and the given path
          function genUrl(opts, path) {
            if (opts.remote) {
              // If the host already has a path, then we need to have a path delimiter
              // Otherwise, the path delimiter is the empty string
              var pathDel = !opts.path ? '' : '/';
              // If the host already has a path, then we need to have a path delimiter
              // Otherwise, the path delimiter is the empty string
              return opts.protocol + '://' + opts.host + ':' + opts.port + '/' + opts.path + pathDel + path;
            }
            return '/' + path;
          }
          // Implements the PouchDB API for dealing with CouchDB instances over HTTP
          function HttpPouch(opts, callback) {
            // The functions that will be publicly available for HttpPouch
            var api = this;
            api.getHost = opts.getHost ? opts.getHost : getHost;
            // Parse the URI given by opts.name into an easy-to-use object
            var host = api.getHost(opts.name, opts);
            // Generate the database URL based on the host
            var dbUrl = genDBUrl(host, '');
            api.getUrl = function () {
              return dbUrl;
            };
            api.getHeaders = function () {
              return utils.clone(host.headers);
            };
            var ajaxOpts = opts.ajax || {};
            opts = utils.clone(opts);
            function ajax(options, callback) {
              var reqOpts = utils.extend(true, utils.clone(ajaxOpts), options);
              log(reqOpts.method + ' ' + reqOpts.url);
              return utils.ajax(reqOpts, callback);
            }
            // Create a new CouchDB database based on the given opts
            var createDB = function () {
              ajax({
                headers: host.headers,
                method: 'PUT',
                url: dbUrl
              }, function (err) {
                // If we get an "Unauthorized" error
                if (err && err.status === 401) {
                  // Test if the database already exists
                  ajax({
                    headers: host.headers,
                    method: 'HEAD',
                    url: dbUrl
                  }, function (err) {
                    // If there is still an error
                    if (err) {
                      // Give the error to the callback to deal with
                      callback(err);
                    } else {
                      // Continue as if there had been no errors
                      callback(null, api);
                    }
                  });  // If there were no errros or if the only error is "Precondition Failed"
                       // (note: "Precondition Failed" occurs when we try to create a database
                       // that already exists)
                } else if (!err || err.status === 412) {
                  // Continue as if there had been no errors
                  callback(null, api);
                } else {
                  callback(err);
                }
              });
            };
            if (!opts.skipSetup) {
              ajax({
                headers: host.headers,
                method: 'GET',
                url: dbUrl
              }, function (err) {
                //check if the db exists
                if (err) {
                  if (err.status === 404) {
                    utils.explain404('PouchDB is just detecting if the remote DB exists.');
                    //if it doesn't, create it
                    createDB();
                  } else {
                    callback(err);
                  }
                } else {
                  //go do stuff with the db
                  callback(null, api);
                }
              });
            }
            api.type = function () {
              return 'http';
            };
            api.id = utils.adapterFun('id', function (callback) {
              ajax({
                headers: host.headers,
                method: 'GET',
                url: genUrl(host, '')
              }, function (err, result) {
                var uuid = result && result.uuid ? result.uuid + host.db : genDBUrl(host, '');
                callback(null, uuid);
              });
            });
            api.request = utils.adapterFun('request', function (options, callback) {
              options.headers = host.headers;
              options.url = genDBUrl(host, options.url);
              ajax(options, callback);
            });
            // Sends a POST request to the host calling the couchdb _compact function
            //    version: The version of CouchDB it is running
            api.compact = utils.adapterFun('compact', function (opts, callback) {
              if (typeof opts === 'function') {
                callback = opts;
                opts = {};
              }
              opts = utils.clone(opts);
              ajax({
                headers: host.headers,
                url: genDBUrl(host, '_compact'),
                method: 'POST'
              }, function () {
                function ping() {
                  api.info(function (err, res) {
                    if (!res.compact_running) {
                      callback(null, { ok: true });
                    } else {
                      setTimeout(ping, opts.interval || 200);
                    }
                  });
                }
                // Ping the http if it's finished compaction
                if (typeof callback === 'function') {
                  ping();
                }
              });
            });
            // Calls GET on the host, which gets back a JSON string containing
            //    couchdb: A welcome string
            //    version: The version of CouchDB it is running
            api._info = function (callback) {
              ajax({
                headers: host.headers,
                method: 'GET',
                url: genDBUrl(host, '')
              }, function (err, res) {
                if (err) {
                  callback(err);
                } else {
                  res.host = genDBUrl(host, '');
                  callback(null, res);
                }
              });
            };
            // Get the document with the given id from the database given by host.
            // The id could be solely the _id in the database, or it may be a
            // _design/ID or _local/ID path
            api.get = utils.adapterFun('get', function (id, opts, callback) {
              // If no options were given, set the callback to the second parameter
              if (typeof opts === 'function') {
                callback = opts;
                opts = {};
              }
              opts = utils.clone(opts);
              if (opts.auto_encode === undefined) {
                opts.auto_encode = true;
              }
              // List of parameters to add to the GET request
              var params = [];
              // If it exists, add the opts.revs value to the list of parameters.
              // If revs=true then the resulting JSON will include a field
              // _revisions containing an array of the revision IDs.
              if (opts.revs) {
                params.push('revs=true');
              }
              // If it exists, add the opts.revs_info value to the list of parameters.
              // If revs_info=true then the resulting JSON will include the field
              // _revs_info containing an array of objects in which each object
              // representing an available revision.
              if (opts.revs_info) {
                params.push('revs_info=true');
              }
              if (opts.local_seq) {
                params.push('local_seq=true');
              }
              // If it exists, add the opts.open_revs value to the list of parameters.
              // If open_revs=all then the resulting JSON will include all the leaf
              // revisions. If open_revs=["rev1", "rev2",...] then the resulting JSON
              // will contain an array of objects containing data of all revisions
              if (opts.open_revs) {
                if (opts.open_revs !== 'all') {
                  opts.open_revs = JSON.stringify(opts.open_revs);
                }
                params.push('open_revs=' + opts.open_revs);
              }
              // If it exists, add the opts.attachments value to the list of parameters.
              // If attachments=true the resulting JSON will include the base64-encoded
              // contents in the "data" property of each attachment.
              if (opts.attachments) {
                params.push('attachments=true');
              }
              // If it exists, add the opts.rev value to the list of parameters.
              // If rev is given a revision number then get the specified revision.
              if (opts.rev) {
                params.push('rev=' + opts.rev);
              }
              // If it exists, add the opts.conflicts value to the list of parameters.
              // If conflicts=true then the resulting JSON will include the field
              // _conflicts containing all the conflicting revisions.
              if (opts.conflicts) {
                params.push('conflicts=' + opts.conflicts);
              }
              // Format the list of parameters into a valid URI query string
              params = params.join('&');
              params = params === '' ? '' : '?' + params;
              if (opts.auto_encode) {
                id = encodeDocId(id);
              }
              // Set the options for the ajax call
              var options = {
                  headers: host.headers,
                  method: 'GET',
                  url: genDBUrl(host, id + params)
                };
              var getRequestAjaxOpts = opts.ajax || {};
              utils.extend(true, options, getRequestAjaxOpts);
              // If the given id contains at least one '/' and the part before the '/'
              // is NOT "_design" and is NOT "_local"
              // OR
              // If the given id contains at least two '/' and the part before the first
              // '/' is "_design".
              // TODO This second condition seems strange since if parts[0] === '_design'
              // then we already know that parts[0] !== '_local'.
              var parts = id.split('/');
              if (parts.length > 1 && parts[0] !== '_design' && parts[0] !== '_local' || parts.length > 2 && parts[0] === '_design' && parts[0] !== '_local') {
                // Binary is expected back from the server
                options.binary = true;
              }
              // Get the document
              ajax(options, function (err, doc, xhr) {
                // If the document does not exist, send an error to the callback
                if (err) {
                  return callback(err);
                }
                // Send the document to the callback
                callback(null, doc, xhr);
              });
            });
            // Delete the document given by doc from the database given by host.
            api.remove = utils.adapterFun('remove', function (docOrId, optsOrRev, opts, callback) {
              var doc;
              if (typeof optsOrRev === 'string') {
                // id, rev, opts, callback style
                doc = {
                  _id: docOrId,
                  _rev: optsOrRev
                };
                if (typeof opts === 'function') {
                  callback = opts;
                  opts = {};
                }
              } else {
                // doc, opts, callback style
                doc = docOrId;
                if (typeof optsOrRev === 'function') {
                  callback = optsOrRev;
                  opts = {};
                } else {
                  callback = opts;
                  opts = optsOrRev;
                }
              }
              var rev = doc._rev || opts.rev;
              // Delete the document
              ajax({
                headers: host.headers,
                method: 'DELETE',
                url: genDBUrl(host, encodeDocId(doc._id)) + '?rev=' + rev
              }, callback);
            });
            function encodeAttachmentId(attachmentId) {
              return attachmentId.split('/').map(encodeURIComponent).join('/');
            }
            // Get the attachment
            api.getAttachment = utils.adapterFun('getAttachment', function (docId, attachmentId, opts, callback) {
              if (typeof opts === 'function') {
                callback = opts;
                opts = {};
              }
              opts = utils.clone(opts);
              if (opts.auto_encode === undefined) {
                opts.auto_encode = true;
              }
              if (opts.auto_encode) {
                docId = encodeDocId(docId);
              }
              opts.auto_encode = false;
              api.get(docId + '/' + encodeAttachmentId(attachmentId), opts, callback);
            });
            // Remove the attachment given by the id and rev
            api.removeAttachment = utils.adapterFun('removeAttachment', function (docId, attachmentId, rev, callback) {
              var url = genDBUrl(host, encodeDocId(docId) + '/' + encodeAttachmentId(attachmentId)) + '?rev=' + rev;
              ajax({
                headers: host.headers,
                method: 'DELETE',
                url: url
              }, callback);
            });
            // Add the attachment given by blob and its contentType property
            // to the document with the given id, the revision given by rev, and
            // add it to the database given by host.
            api.putAttachment = utils.adapterFun('putAttachment', function (docId, attachmentId, rev, blob, type, callback) {
              if (typeof type === 'function') {
                callback = type;
                type = blob;
                blob = rev;
                rev = null;
              }
              if (typeof type === 'undefined') {
                type = blob;
                blob = rev;
                rev = null;
              }
              var id = encodeDocId(docId) + '/' + encodeAttachmentId(attachmentId);
              var url = genDBUrl(host, id);
              if (rev) {
                url += '?rev=' + rev;
              }
              if (typeof blob === 'string') {
                var binary;
                try {
                  binary = utils.atob(blob);
                } catch (err) {
                  // it's not base64-encoded, so throw error
                  return callback(errors.error(errors.BAD_ARG, 'Attachments need to be base64 encoded'));
                }
                if (isBrowser) {
                  blob = utils.createBlob([utils.fixBinary(binary)], { type: type });
                } else {
                  blob = binary ? new buffer(binary, 'binary') : '';
                }
              }
              var opts = {
                  headers: utils.clone(host.headers),
                  method: 'PUT',
                  url: url,
                  processData: false,
                  body: blob,
                  timeout: 60000
                };
              opts.headers['Content-Type'] = type;
              // Add the attachment
              ajax(opts, callback);
            });
            // Add the document given by doc (in JSON string format) to the database
            // given by host. This fails if the doc has no _id field.
            api.put = utils.adapterFun('put', utils.getArguments(function (args) {
              var temp, temptype, opts;
              var doc = args.shift();
              var id = '_id' in doc;
              var callback = args.pop();
              if (typeof doc !== 'object' || Array.isArray(doc)) {
                return callback(errors.error(errors.NOT_AN_OBJECT));
              }
              doc = utils.clone(doc);
              preprocessAttachments(doc).then(function () {
                while (true) {
                  temp = args.shift();
                  temptype = typeof temp;
                  if (temptype === 'string' && !id) {
                    doc._id = temp;
                    id = true;
                  } else if (temptype === 'string' && id && !('_rev' in doc)) {
                    doc._rev = temp;
                  } else if (temptype === 'object') {
                    opts = utils.clone(temp);
                  }
                  if (!args.length) {
                    break;
                  }
                }
                opts = opts || {};
                var error = utils.invalidIdError(doc._id);
                if (error) {
                  throw error;
                }
                // List of parameter to add to the PUT request
                var params = [];
                // If it exists, add the opts.new_edits value to the list of parameters.
                // If new_edits = false then the database will NOT assign this document a
                // new revision number
                if (opts && typeof opts.new_edits !== 'undefined') {
                  params.push('new_edits=' + opts.new_edits);
                }
                // Format the list of parameters into a valid URI query string
                params = params.join('&');
                if (params !== '') {
                  params = '?' + params;
                }
                // Add the document
                ajax({
                  headers: host.headers,
                  method: 'PUT',
                  url: genDBUrl(host, encodeDocId(doc._id)) + params,
                  body: doc
                }, function (err, res) {
                  if (err) {
                    return callback(err);
                  }
                  res.ok = true;
                  callback(null, res);
                });
              })['catch'](callback);
            }));
            // Add the document given by doc (in JSON string format) to the database
            // given by host. This does not assume that doc is a new document 
            // (i.e. does not have a _id or a _rev field.)
            api.post = utils.adapterFun('post', function (doc, opts, callback) {
              // If no options were given, set the callback to be the second parameter
              if (typeof opts === 'function') {
                callback = opts;
                opts = {};
              }
              opts = utils.clone(opts);
              if (typeof doc !== 'object') {
                return callback(errors.error(errors.NOT_AN_OBJECT));
              }
              if (!('_id' in doc)) {
                doc._id = utils.uuid();
              }
              api.put(doc, opts, function (err, res) {
                if (err) {
                  return callback(err);
                }
                res.ok = true;
                callback(null, res);
              });
            });
            // Update/create multiple documents given by req in the database
            // given by host.
            api._bulkDocs = function (req, opts, callback) {
              // If opts.new_edits exists add it to the document data to be
              // send to the database.
              // If new_edits=false then it prevents the database from creating
              // new revision numbers for the documents. Instead it just uses
              // the old ones. This is used in database replication.
              if (typeof opts.new_edits !== 'undefined') {
                req.new_edits = opts.new_edits;
              }
              utils.Promise.all(req.docs.map(preprocessAttachments)).then(function () {
                // Update/create the documents
                ajax({
                  headers: host.headers,
                  method: 'POST',
                  url: genDBUrl(host, '_bulk_docs'),
                  body: req
                }, function (err, results) {
                  if (err) {
                    return callback(err);
                  }
                  results.forEach(function (result) {
                    result.ok = true;  // smooths out cloudant not adding this
                  });
                  callback(null, results);
                });
              })['catch'](callback);
            };
            // Get a listing of the documents in the database given
            // by host and ordered by increasing id.
            api.allDocs = utils.adapterFun('allDocs', function (opts, callback) {
              if (typeof opts === 'function') {
                callback = opts;
                opts = {};
              }
              opts = utils.clone(opts);
              // List of parameters to add to the GET request
              var params = [];
              var body;
              var method = 'GET';
              if (opts.conflicts) {
                params.push('conflicts=true');
              }
              // If opts.descending is truthy add it to params
              if (opts.descending) {
                params.push('descending=true');
              }
              // If opts.include_docs exists, add the include_docs value to the
              // list of parameters.
              // If include_docs=true then include the associated document with each
              // result.
              if (opts.include_docs) {
                params.push('include_docs=true');
              }
              if (opts.attachments) {
                // added in CouchDB 1.6.0
                params.push('attachments=true');
              }
              if (opts.key) {
                params.push('key=' + encodeURIComponent(JSON.stringify(opts.key)));
              }
              // If opts.startkey exists, add the startkey value to the list of
              // parameters.
              // If startkey is given then the returned list of documents will
              // start with the document whose id is startkey.
              if (opts.startkey) {
                params.push('startkey=' + encodeURIComponent(JSON.stringify(opts.startkey)));
              }
              // If opts.endkey exists, add the endkey value to the list of parameters.
              // If endkey is given then the returned list of docuemnts will
              // end with the document whose id is endkey.
              if (opts.endkey) {
                params.push('endkey=' + encodeURIComponent(JSON.stringify(opts.endkey)));
              }
              if (typeof opts.inclusive_end !== 'undefined') {
                params.push('inclusive_end=' + !!opts.inclusive_end);
              }
              // If opts.limit exists, add the limit value to the parameter list.
              if (typeof opts.limit !== 'undefined') {
                params.push('limit=' + opts.limit);
              }
              if (typeof opts.skip !== 'undefined') {
                params.push('skip=' + opts.skip);
              }
              // Format the list of parameters into a valid URI query string
              params = params.join('&');
              if (params !== '') {
                params = '?' + params;
              }
              if (typeof opts.keys !== 'undefined') {
                var keysAsString = 'keys=' + encodeURIComponent(JSON.stringify(opts.keys));
                if (keysAsString.length + params.length + 1 <= MAX_URL_LENGTH) {
                  // If the keys are short enough, do a GET. we do this to work around
                  // Safari not understanding 304s on POSTs (see issue #1239)
                  params += (params.indexOf('?') !== -1 ? '&' : '?') + keysAsString;
                } else {
                  // If keys are too long, issue a POST request to circumvent GET
                  // query string limits
                  // see http://wiki.apache.org/couchdb/HTTP_view_API#Querying_Options
                  method = 'POST';
                  body = JSON.stringify({ keys: opts.keys });
                }
              }
              // Get the document listing
              ajax({
                headers: host.headers,
                method: method,
                url: genDBUrl(host, '_all_docs' + params),
                body: body
              }, callback);
            });
            // Get a list of changes made to documents in the database given by host.
            // TODO According to the README, there should be two other methods here,
            // api.changes.addListener and api.changes.removeListener.
            api._changes = function (opts) {
              // We internally page the results of a changes request, this means
              // if there is a large set of changes to be returned we can start
              // processing them quicker instead of waiting on the entire
              // set of changes to return and attempting to process them at once
              var batchSize = 'batch_size' in opts ? opts.batch_size : CHANGES_BATCH_SIZE;
              opts = utils.clone(opts);
              opts.timeout = opts.timeout || 30 * 1000;
              // We give a 5 second buffer for CouchDB changes to respond with
              // an ok timeout
              var params = { timeout: opts.timeout - 5 * 1000 };
              var limit = typeof opts.limit !== 'undefined' ? opts.limit : false;
              if (limit === 0) {
                limit = 1;
              }
              var returnDocs;
              if ('returnDocs' in opts) {
                returnDocs = opts.returnDocs;
              } else {
                returnDocs = true;
              }
              //
              var leftToFetch = limit;
              if (opts.style) {
                params.style = opts.style;
              }
              if (opts.include_docs || opts.filter && typeof opts.filter === 'function') {
                params.include_docs = true;
              }
              if (opts.attachments) {
                params.attachments = true;
              }
              if (opts.continuous) {
                params.feed = 'longpoll';
              }
              if (opts.conflicts) {
                params.conflicts = true;
              }
              if (opts.descending) {
                params.descending = true;
              }
              if (opts.filter && typeof opts.filter === 'string') {
                params.filter = opts.filter;
                if (opts.filter === '_view' && opts.view && typeof opts.view === 'string') {
                  params.view = opts.view;
                }
              }
              // If opts.query_params exists, pass it through to the changes request.
              // These parameters may be used by the filter on the source database.
              if (opts.query_params && typeof opts.query_params === 'object') {
                for (var param_name in opts.query_params) {
                  if (opts.query_params.hasOwnProperty(param_name)) {
                    params[param_name] = opts.query_params[param_name];
                  }
                }
              }
              var method = 'GET';
              var body;
              if (opts.doc_ids) {
                // set this automagically for the user; it's annoying that couchdb
                // requires both a "filter" and a "doc_ids" param.
                params.filter = '_doc_ids';
                var docIdsJson = JSON.stringify(opts.doc_ids);
                if (docIdsJson.length < MAX_URL_LENGTH) {
                  params.doc_ids = docIdsJson;
                } else {
                  // anything greater than ~2000 is unsafe for gets, so
                  // use POST instead
                  method = 'POST';
                  body = { doc_ids: opts.doc_ids };
                }
              }
              if (opts.continuous && api._useSSE) {
                return api.sse(opts, params, returnDocs);
              }
              var xhr;
              var lastFetchedSeq;
              // Get all the changes starting wtih the one immediately after the
              // sequence number given by since.
              var fetch = function (since, callback) {
                if (opts.aborted) {
                  return;
                }
                params.since = since;
                if (typeof params.since === 'object') {
                  params.since = JSON.stringify(params.since);
                }
                if (opts.descending) {
                  if (limit) {
                    params.limit = leftToFetch;
                  }
                } else {
                  params.limit = !limit || leftToFetch > batchSize ? batchSize : leftToFetch;
                }
                var paramStr = '?' + Object.keys(params).map(function (k) {
                    return k + '=' + params[k];
                  }).join('&');
                // Set the options for the ajax call
                var xhrOpts = {
                    headers: host.headers,
                    method: method,
                    url: genDBUrl(host, '_changes' + paramStr),
                    timeout: opts.timeout,
                    body: body
                  };
                lastFetchedSeq = since;
                if (opts.aborted) {
                  return;
                }
                // Get the changes
                xhr = ajax(xhrOpts, callback);
              };
              // If opts.since exists, get all the changes from the sequence
              // number given by opts.since. Otherwise, get all the changes
              // from the sequence number 0.
              var fetchTimeout = 10;
              var fetchRetryCount = 0;
              var results = { results: [] };
              var fetched = function (err, res) {
                if (opts.aborted) {
                  return;
                }
                var raw_results_length = 0;
                // If the result of the ajax call (res) contains changes (res.results)
                if (res && res.results) {
                  raw_results_length = res.results.length;
                  results.last_seq = res.last_seq;
                  // For each change
                  var req = {};
                  req.query = opts.query_params;
                  res.results = res.results.filter(function (c) {
                    leftToFetch--;
                    var ret = utils.filterChange(opts)(c);
                    if (ret) {
                      if (returnDocs) {
                        results.results.push(c);
                      }
                      utils.call(opts.onChange, c);
                    }
                    return ret;
                  });
                } else if (err) {
                  // In case of an error, stop listening for changes and call
                  // opts.complete
                  opts.aborted = true;
                  utils.call(opts.complete, err);
                  return;
                }
                // The changes feed may have timed out with no results
                // if so reuse last update sequence
                if (res && res.last_seq) {
                  lastFetchedSeq = res.last_seq;
                }
                var finished = limit && leftToFetch <= 0 || res && raw_results_length < batchSize || opts.descending;
                if (opts.continuous && !(limit && leftToFetch <= 0) || !finished) {
                  // Increase retry delay exponentially as long as errors persist
                  if (err) {
                    fetchRetryCount += 1;
                  } else {
                    fetchRetryCount = 0;
                  }
                  var timeoutMultiplier = 1 << fetchRetryCount;
                  var retryWait = fetchTimeout * timeoutMultiplier;
                  var maximumWait = opts.maximumWait || 30000;
                  if (retryWait > maximumWait) {
                    utils.call(opts.complete, err || errors.error(errors.UNKNOWN_ERROR));
                    return;
                  }
                  // Queue a call to fetch again with the newest sequence number
                  setTimeout(function () {
                    fetch(lastFetchedSeq, fetched);
                  }, retryWait);
                } else {
                  // We're done, call the callback
                  utils.call(opts.complete, null, results);
                }
              };
              fetch(opts.since || 0, fetched);
              // Return a method to cancel this method from processing any more
              return {
                cancel: function () {
                  opts.aborted = true;
                  if (xhr) {
                    xhr.abort();
                  }
                }
              };
            };
            api.sse = function (opts, params, returnDocs) {
              params.feed = 'eventsource';
              params.since = opts.since || 0;
              params.limit = opts.limit;
              delete params.timeout;
              var paramStr = '?' + Object.keys(params).map(function (k) {
                  return k + '=' + params[k];
                }).join('&');
              var url = genDBUrl(host, '_changes' + paramStr);
              var source = new EventSource(url);
              var results = {
                  results: [],
                  last_seq: false
                };
              var dispatched = false;
              var open = false;
              source.addEventListener('message', msgHandler, false);
              source.onopen = function () {
                open = true;
              };
              source.onerror = errHandler;
              return {
                cancel: function () {
                  if (dispatched) {
                    return dispatched.cancel();
                  }
                  source.removeEventListener('message', msgHandler, false);
                  source.close();
                }
              };
              function msgHandler(e) {
                var data = JSON.parse(e.data);
                if (returnDocs) {
                  results.results.push(data);
                }
                results.last_seq = data.seq;
                utils.call(opts.onChange, data);
              }
              function errHandler(err) {
                source.removeEventListener('message', msgHandler, false);
                if (open === false) {
                  // errored before it opened
                  // likely doesn't support EventSource
                  api._useSSE = false;
                  dispatched = api._changes(opts);
                  return;
                }
                source.close();
                utils.call(opts.complete, err);
              }
            };
            api._useSSE = false;
            // Currently disabled due to failing chrome tests in saucelabs
            // api._useSSE = typeof global.EventSource === 'function';
            // Given a set of document/revision IDs (given by req), tets the subset of
            // those that do NOT correspond to revisions stored in the database.
            // See http://wiki.apache.org/couchdb/HttpPostRevsDiff
            api.revsDiff = utils.adapterFun('revsDiff', function (req, opts, callback) {
              // If no options were given, set the callback to be the second parameter
              if (typeof opts === 'function') {
                callback = opts;
                opts = {};
              }
              // Get the missing document/revision IDs
              ajax({
                headers: host.headers,
                method: 'POST',
                url: genDBUrl(host, '_revs_diff'),
                body: JSON.stringify(req)
              }, callback);
            });
            api._close = function (callback) {
              callback();
            };
            api.destroy = utils.adapterFun('destroy', function (callback) {
              ajax({
                url: genDBUrl(host, ''),
                method: 'DELETE',
                headers: host.headers
              }, function (err, resp) {
                if (err) {
                  api.emit('error', err);
                  callback(err);
                } else {
                  api.emit('destroyed');
                  callback(null, resp);
                }
              });
            });
          }
          // Delete the HttpPouch specified by the given name.
          HttpPouch.destroy = utils.toPromise(function (name, opts, callback) {
            var host = getHost(name, opts);
            opts = opts || {};
            if (typeof opts === 'function') {
              callback = opts;
              opts = {};
            }
            opts = utils.clone(opts);
            opts.headers = host.headers;
            opts.method = 'DELETE';
            opts.url = genDBUrl(host, '');
            var ajaxOpts = opts.ajax || {};
            opts = utils.extend({}, opts, ajaxOpts);
            utils.ajax(opts, callback);
          });
          // HttpPouch is a valid adapter.
          HttpPouch.valid = function () {
            return true;
          };
          module.exports = HttpPouch;
        }.call(this, _dereq_(40)));
      },
      {
        '19': 19,
        '20': 20,
        '35': 35,
        '40': 40,
        '41': 41
      }
    ],
    3: [
      function (_dereq_, module, exports) {
        'use strict';
        var merge = _dereq_(30);
        var errors = _dereq_(20);
        var idbUtils = _dereq_(7);
        var idbConstants = _dereq_(6);
        var ATTACH_STORE = idbConstants.ATTACH_STORE;
        var BY_SEQ_STORE = idbConstants.BY_SEQ_STORE;
        var DOC_STORE = idbConstants.DOC_STORE;
        var decodeDoc = idbUtils.decodeDoc;
        var decodeMetadata = idbUtils.decodeMetadata;
        var fetchAttachmentsIfNecessary = idbUtils.fetchAttachmentsIfNecessary;
        var postProcessAttachments = idbUtils.postProcessAttachments;
        var openTransactionSafely = idbUtils.openTransactionSafely;
        function createKeyRange(start, end, inclusiveEnd, key, descending) {
          try {
            if (start && end) {
              if (descending) {
                return IDBKeyRange.bound(end, start, !inclusiveEnd, false);
              } else {
                return IDBKeyRange.bound(start, end, false, !inclusiveEnd);
              }
            } else if (start) {
              if (descending) {
                return IDBKeyRange.upperBound(start);
              } else {
                return IDBKeyRange.lowerBound(start);
              }
            } else if (end) {
              if (descending) {
                return IDBKeyRange.lowerBound(end, !inclusiveEnd);
              } else {
                return IDBKeyRange.upperBound(end, !inclusiveEnd);
              }
            } else if (key) {
              return IDBKeyRange.only(key);
            }
          } catch (e) {
            return { error: e };
          }
          return null;
        }
        function handleKeyRangeError(api, opts, err, callback) {
          if (err.name === 'DataError' && err.code === 0) {
            // data error, start is less than end
            return callback(null, {
              total_rows: api._meta.docCount,
              offset: opts.skip,
              rows: []
            });
          }
          callback(errors.error(errors.IDB_ERROR, err.name, err.message));
        }
        function idbAllDocs(opts, api, idb, callback) {
          function allDocsQuery(opts, callback) {
            var start = 'startkey' in opts ? opts.startkey : false;
            var end = 'endkey' in opts ? opts.endkey : false;
            var key = 'key' in opts ? opts.key : false;
            var skip = opts.skip || 0;
            var limit = typeof opts.limit === 'number' ? opts.limit : -1;
            var inclusiveEnd = opts.inclusive_end !== false;
            var descending = 'descending' in opts && opts.descending ? 'prev' : null;
            var keyRange = createKeyRange(start, end, inclusiveEnd, key, descending);
            if (keyRange && keyRange.error) {
              return handleKeyRangeError(api, opts, keyRange.error, callback);
            }
            var stores = [
                DOC_STORE,
                BY_SEQ_STORE
              ];
            if (opts.attachments) {
              stores.push(ATTACH_STORE);
            }
            var txnResult = openTransactionSafely(idb, stores, 'readonly');
            if (txnResult.error) {
              return callback(txnResult.error);
            }
            var txn = txnResult.txn;
            var docStore = txn.objectStore(DOC_STORE);
            var seqStore = txn.objectStore(BY_SEQ_STORE);
            var cursor = descending ? docStore.openCursor(keyRange, descending) : docStore.openCursor(keyRange);
            var docIdRevIndex = seqStore.index('_doc_id_rev');
            var results = [];
            var docCount = 0;
            // if the user specifies include_docs=true, then we don't
            // want to block the main cursor while we're fetching the doc
            function fetchDocAsynchronously(metadata, row, winningRev) {
              var key = metadata.id + '::' + winningRev;
              docIdRevIndex.get(key).onsuccess = function onGetDoc(e) {
                row.doc = decodeDoc(e.target.result);
                if (opts.conflicts) {
                  row.doc._conflicts = merge.collectConflicts(metadata);
                }
                fetchAttachmentsIfNecessary(row.doc, opts, txn);
              };
            }
            function allDocsInner(cursor, winningRev, metadata) {
              var row = {
                  id: metadata.id,
                  key: metadata.id,
                  value: { rev: winningRev }
                };
              var deleted = metadata.deleted;
              if (opts.deleted === 'ok') {
                results.push(row);
                // deleted docs are okay with "keys" requests
                if (deleted) {
                  row.value.deleted = true;
                  row.doc = null;
                } else if (opts.include_docs) {
                  fetchDocAsynchronously(metadata, row, winningRev);
                }
              } else if (!deleted && skip-- <= 0) {
                results.push(row);
                if (opts.include_docs) {
                  fetchDocAsynchronously(metadata, row, winningRev);
                }
                if (--limit === 0) {
                  return;
                }
              }
              cursor['continue']();
            }
            function onGetCursor(e) {
              docCount = api._meta.docCount;
              // do this within the txn for consistency
              var cursor = e.target.result;
              if (!cursor) {
                return;
              }
              var metadata = decodeMetadata(cursor.value);
              var winningRev = metadata.winningRev;
              allDocsInner(cursor, winningRev, metadata);
            }
            function onResultsReady() {
              callback(null, {
                total_rows: docCount,
                offset: opts.skip,
                rows: results
              });
            }
            function onTxnComplete() {
              if (opts.attachments) {
                postProcessAttachments(results).then(onResultsReady);
              } else {
                onResultsReady();
              }
            }
            txn.oncomplete = onTxnComplete;
            cursor.onsuccess = onGetCursor;
          }
          function allDocs(opts, callback) {
            if (opts.limit === 0) {
              return callback(null, {
                total_rows: api._meta.docCount,
                offset: opts.skip,
                rows: []
              });
            }
            allDocsQuery(opts, callback);
          }
          allDocs(opts, callback);
        }
        module.exports = idbAllDocs;
      },
      {
        '20': 20,
        '30': 30,
        '6': 6,
        '7': 7
      }
    ],
    4: [
      function (_dereq_, module, exports) {
        'use strict';
        var utils = _dereq_(35);
        var idbConstants = _dereq_(6);
        var DETECT_BLOB_SUPPORT_STORE = idbConstants.DETECT_BLOB_SUPPORT_STORE;
        //
        // Detect blob support. Chrome didn't support it until version 38.
        // In version 37 they had a broken version where PNGs (and possibly
        // other binary types) aren't stored correctly, because when you fetch
        // them, the content type is always null.
        //
        // Furthermore, they have some outstanding bugs where blobs occasionally
        // are read by FileReader as null, or by ajax as 404s.
        //
        // Sadly we use the 404 bug to detect the FileReader bug, so if they
        // get fixed independently and released in different versions of Chrome,
        // then the bug could come back. So it's worthwhile to watch these issues:
        // 404 bug: https://code.google.com/p/chromium/issues/detail?id=447916
        // FileReader bug: https://code.google.com/p/chromium/issues/detail?id=447836
        //
        function checkBlobSupport(txn, idb) {
          return new utils.Promise(function (resolve, reject) {
            var blob = utils.createBlob([''], { type: 'image/png' });
            txn.objectStore(DETECT_BLOB_SUPPORT_STORE).put(blob, 'key');
            txn.oncomplete = function () {
              // have to do it in a separate transaction, else the correct
              // content type is always returned
              var blobTxn = idb.transaction([DETECT_BLOB_SUPPORT_STORE], 'readwrite');
              var getBlobReq = blobTxn.objectStore(DETECT_BLOB_SUPPORT_STORE).get('key');
              getBlobReq.onerror = reject;
              getBlobReq.onsuccess = function (e) {
                var storedBlob = e.target.result;
                var url = URL.createObjectURL(storedBlob);
                utils.ajax({
                  url: url,
                  cache: true,
                  binary: true
                }, function (err, res) {
                  if (err && err.status === 405) {
                    // firefox won't let us do that. but firefox doesn't
                    // have the blob type bug that Chrome does, so that's ok
                    resolve(true);
                  } else {
                    resolve(!!(res && res.type === 'image/png'));
                    if (err && err.status === 404) {
                      utils.explain404('PouchDB is just detecting blob URL support.');
                    }
                  }
                  URL.revokeObjectURL(url);
                });
              };
            };
          })['catch'](function () {
            return false;  // error, so assume unsupported
          });
        }
        module.exports = checkBlobSupport;
      },
      {
        '35': 35,
        '6': 6
      }
    ],
    5: [
      function (_dereq_, module, exports) {
        'use strict';
        var utils = _dereq_(35);
        var errors = _dereq_(20);
        var idbUtils = _dereq_(7);
        var idbConstants = _dereq_(6);
        var ATTACH_AND_SEQ_STORE = idbConstants.ATTACH_AND_SEQ_STORE;
        var ATTACH_STORE = idbConstants.ATTACH_STORE;
        var BY_SEQ_STORE = idbConstants.BY_SEQ_STORE;
        var DOC_STORE = idbConstants.DOC_STORE;
        var LOCAL_STORE = idbConstants.LOCAL_STORE;
        var META_STORE = idbConstants.META_STORE;
        var compactRevs = idbUtils.compactRevs;
        var decodeMetadata = idbUtils.decodeMetadata;
        var encodeMetadata = idbUtils.encodeMetadata;
        var idbError = idbUtils.idbError;
        var openTransactionSafely = idbUtils.openTransactionSafely;
        function idbBulkDocs(req, opts, api, idb, Changes, callback) {
          var docInfos = req.docs;
          var txn;
          var docStore;
          var bySeqStore;
          var attachStore;
          var attachAndSeqStore;
          var docInfoError;
          var docCountDelta = 0;
          for (var i = 0, len = docInfos.length; i < len; i++) {
            var doc = docInfos[i];
            if (doc._id && utils.isLocalId(doc._id)) {
              continue;
            }
            doc = docInfos[i] = utils.parseDoc(doc, opts.new_edits);
            if (doc.error && !docInfoError) {
              docInfoError = doc;
            }
          }
          if (docInfoError) {
            return callback(docInfoError);
          }
          var results = new Array(docInfos.length);
          var fetchedDocs = new utils.Map();
          var preconditionErrored = false;
          var blobType = api._meta.blobSupport ? 'blob' : 'base64';
          utils.preprocessAttachments(docInfos, blobType, function (err) {
            if (err) {
              return callback(err);
            }
            startTransaction();
          });
          function startTransaction() {
            var stores = [
                DOC_STORE,
                BY_SEQ_STORE,
                ATTACH_STORE,
                META_STORE,
                LOCAL_STORE,
                ATTACH_AND_SEQ_STORE
              ];
            var txnResult = openTransactionSafely(idb, stores, 'readwrite');
            if (txnResult.error) {
              return callback(txnResult.error);
            }
            txn = txnResult.txn;
            txn.onerror = idbError(callback);
            txn.ontimeout = idbError(callback);
            txn.oncomplete = complete;
            docStore = txn.objectStore(DOC_STORE);
            bySeqStore = txn.objectStore(BY_SEQ_STORE);
            attachStore = txn.objectStore(ATTACH_STORE);
            attachAndSeqStore = txn.objectStore(ATTACH_AND_SEQ_STORE);
            verifyAttachments(function (err) {
              if (err) {
                preconditionErrored = true;
                return callback(err);
              }
              fetchExistingDocs();
            });
          }
          function processDocs() {
            utils.processDocs(docInfos, api, fetchedDocs, txn, results, writeDoc, opts);
          }
          function fetchExistingDocs() {
            if (!docInfos.length) {
              return;
            }
            var numFetched = 0;
            function checkDone() {
              if (++numFetched === docInfos.length) {
                processDocs();
              }
            }
            function readMetadata(event) {
              var metadata = decodeMetadata(event.target.result);
              if (metadata) {
                fetchedDocs.set(metadata.id, metadata);
              }
              checkDone();
            }
            for (var i = 0, len = docInfos.length; i < len; i++) {
              var docInfo = docInfos[i];
              if (docInfo._id && utils.isLocalId(docInfo._id)) {
                checkDone();
                // skip local docs
                continue;
              }
              var req = docStore.get(docInfo.metadata.id);
              req.onsuccess = readMetadata;
            }
          }
          function complete() {
            if (preconditionErrored) {
              return;
            }
            Changes.notify(api._meta.name);
            api._meta.docCount += docCountDelta;
            callback(null, results);
          }
          function verifyAttachment(digest, callback) {
            var req = attachStore.get(digest);
            req.onsuccess = function (e) {
              if (!e.target.result) {
                var err = errors.error(errors.MISSING_STUB, 'unknown stub attachment with digest ' + digest);
                err.status = 412;
                callback(err);
              } else {
                callback();
              }
            };
          }
          function verifyAttachments(finish) {
            var digests = [];
            docInfos.forEach(function (docInfo) {
              if (docInfo.data && docInfo.data._attachments) {
                Object.keys(docInfo.data._attachments).forEach(function (filename) {
                  var att = docInfo.data._attachments[filename];
                  if (att.stub) {
                    digests.push(att.digest);
                  }
                });
              }
            });
            if (!digests.length) {
              return finish();
            }
            var numDone = 0;
            var err;
            function checkDone() {
              if (++numDone === digests.length) {
                finish(err);
              }
            }
            digests.forEach(function (digest) {
              verifyAttachment(digest, function (attErr) {
                if (attErr && !err) {
                  err = attErr;
                }
                checkDone();
              });
            });
          }
          function writeDoc(docInfo, winningRev, winningRevIsDeleted, newRevIsDeleted, isUpdate, delta, resultsIdx, callback) {
            docCountDelta += delta;
            var doc = docInfo.data;
            doc._id = docInfo.metadata.id;
            doc._rev = docInfo.metadata.rev;
            if (newRevIsDeleted) {
              doc._deleted = true;
            }
            var hasAttachments = doc._attachments && Object.keys(doc._attachments).length;
            if (hasAttachments) {
              return writeAttachments(docInfo, winningRev, winningRevIsDeleted, isUpdate, resultsIdx, callback);
            }
            finishDoc(docInfo, winningRev, winningRevIsDeleted, isUpdate, resultsIdx, callback);
          }
          function autoCompact(docInfo) {
            var revsToDelete = utils.compactTree(docInfo.metadata);
            compactRevs(revsToDelete, docInfo.metadata.id, txn);
          }
          function finishDoc(docInfo, winningRev, winningRevIsDeleted, isUpdate, resultsIdx, callback) {
            var doc = docInfo.data;
            var metadata = docInfo.metadata;
            doc._doc_id_rev = metadata.id + '::' + metadata.rev;
            delete doc._id;
            delete doc._rev;
            function afterPutDoc(e) {
              if (isUpdate && api.auto_compaction) {
                autoCompact(docInfo);
              }
              metadata.seq = e.target.result;
              // Current _rev is calculated from _rev_tree on read
              delete metadata.rev;
              var metadataToStore = encodeMetadata(metadata, winningRev, winningRevIsDeleted);
              var metaDataReq = docStore.put(metadataToStore);
              metaDataReq.onsuccess = afterPutMetadata;
            }
            function afterPutDocError(e) {
              // ConstraintError, need to update, not put (see #1638 for details)
              e.preventDefault();
              // avoid transaction abort
              e.stopPropagation();
              // avoid transaction onerror
              var index = bySeqStore.index('_doc_id_rev');
              var getKeyReq = index.getKey(doc._doc_id_rev);
              getKeyReq.onsuccess = function (e) {
                var putReq = bySeqStore.put(doc, e.target.result);
                putReq.onsuccess = afterPutDoc;
              };
            }
            function afterPutMetadata() {
              results[resultsIdx] = {
                ok: true,
                id: metadata.id,
                rev: winningRev
              };
              fetchedDocs.set(docInfo.metadata.id, docInfo.metadata);
              insertAttachmentMappings(docInfo, metadata.seq, callback);
            }
            var putReq = bySeqStore.put(doc);
            putReq.onsuccess = afterPutDoc;
            putReq.onerror = afterPutDocError;
          }
          function writeAttachments(docInfo, winningRev, winningRevIsDeleted, isUpdate, resultsIdx, callback) {
            var doc = docInfo.data;
            var numDone = 0;
            var attachments = Object.keys(doc._attachments);
            function collectResults() {
              if (numDone === attachments.length) {
                finishDoc(docInfo, winningRev, winningRevIsDeleted, isUpdate, resultsIdx, callback);
              }
            }
            function attachmentSaved() {
              numDone++;
              collectResults();
            }
            attachments.forEach(function (key) {
              var att = docInfo.data._attachments[key];
              if (!att.stub) {
                var data = att.data;
                delete att.data;
                var digest = att.digest;
                saveAttachment(digest, data, attachmentSaved);
              } else {
                numDone++;
                collectResults();
              }
            });
          }
          // map seqs to attachment digests, which
          // we will need later during compaction
          function insertAttachmentMappings(docInfo, seq, callback) {
            var attsAdded = 0;
            var attsToAdd = Object.keys(docInfo.data._attachments || {});
            if (!attsToAdd.length) {
              return callback();
            }
            function checkDone() {
              if (++attsAdded === attsToAdd.length) {
                callback();
              }
            }
            function add(att) {
              var digest = docInfo.data._attachments[att].digest;
              var req = attachAndSeqStore.put({
                  seq: seq,
                  digestSeq: digest + '::' + seq
                });
              req.onsuccess = checkDone;
              req.onerror = function (e) {
                // this callback is for a constaint error, which we ignore
                // because this docid/rev has already been associated with
                // the digest (e.g. when new_edits == false)
                e.preventDefault();
                // avoid transaction abort
                e.stopPropagation();
                // avoid transaction onerror
                checkDone();
              };
            }
            for (var i = 0; i < attsToAdd.length; i++) {
              add(attsToAdd[i]);  // do in parallel
            }
          }
          function saveAttachment(digest, data, callback) {
            var getKeyReq = attachStore.count(digest);
            getKeyReq.onsuccess = function (e) {
              var count = e.target.result;
              if (count) {
                return callback();  // already exists
              }
              var newAtt = {
                  digest: digest,
                  body: data
                };
              var putReq = attachStore.put(newAtt);
              putReq.onsuccess = callback;
            };
          }
        }
        module.exports = idbBulkDocs;
      },
      {
        '20': 20,
        '35': 35,
        '6': 6,
        '7': 7
      }
    ],
    6: [
      function (_dereq_, module, exports) {
        'use strict';
        // IndexedDB requires a versioned database structure, so we use the
        // version here to manage migrations.
        exports.ADAPTER_VERSION = 5;
        // The object stores created for each database
        // DOC_STORE stores the document meta data, its revision history and state
        // Keyed by document id
        exports.DOC_STORE = 'document-store';
        // BY_SEQ_STORE stores a particular version of a document, keyed by its
        // sequence id
        exports.BY_SEQ_STORE = 'by-sequence';
        // Where we store attachments
        exports.ATTACH_STORE = 'attach-store';
        // Where we store many-to-many relations
        // between attachment digests and seqs
        exports.ATTACH_AND_SEQ_STORE = 'attach-seq-store';
        // Where we store database-wide meta data in a single record
        // keyed by id: META_STORE
        exports.META_STORE = 'meta-store';
        // Where we store local documents
        exports.LOCAL_STORE = 'local-store';
        // Where we detect blob support
        exports.DETECT_BLOB_SUPPORT_STORE = 'detect-blob-support';
      },
      {}
    ],
    7: [
      function (_dereq_, module, exports) {
        (function (process) {
          'use strict';
          var errors = _dereq_(20);
          var utils = _dereq_(35);
          var constants = _dereq_(6);
          function tryCode(fun, that, args) {
            try {
              fun.apply(that, args);
            } catch (err) {
              // shouldn't happen
              if (typeof PouchDB !== 'undefined') {
                PouchDB.emit('error', err);
              }
            }
          }
          exports.taskQueue = {
            running: false,
            queue: []
          };
          exports.applyNext = function () {
            if (exports.taskQueue.running || !exports.taskQueue.queue.length) {
              return;
            }
            exports.taskQueue.running = true;
            var item = exports.taskQueue.queue.shift();
            item.action(function (err, res) {
              tryCode(item.callback, this, [
                err,
                res
              ]);
              exports.taskQueue.running = false;
              process.nextTick(exports.applyNext);
            });
          };
          exports.idbError = function (callback) {
            return function (event) {
              var message = event.target && event.target.error && event.target.error.name || event.target;
              callback(errors.error(errors.IDB_ERROR, message, event.type));
            };
          };
          // Unfortunately, the metadata has to be stringified
          // when it is put into the database, because otherwise
          // IndexedDB can throw errors for deeply-nested objects.
          // Originally we just used JSON.parse/JSON.stringify; now
          // we use this custom vuvuzela library that avoids recursion.
          // If we could do it all over again, we'd probably use a
          // format for the revision trees other than JSON.
          exports.encodeMetadata = function (metadata, winningRev, deleted) {
            return {
              data: utils.safeJsonStringify(metadata),
              winningRev: winningRev,
              deletedOrLocal: deleted ? '1' : '0',
              seq: metadata.seq,
              id: metadata.id
            };
          };
          exports.decodeMetadata = function (storedObject) {
            if (!storedObject) {
              return null;
            }
            var metadata = utils.safeJsonParse(storedObject.data);
            metadata.winningRev = storedObject.winningRev;
            metadata.deleted = storedObject.deletedOrLocal === '1';
            metadata.seq = storedObject.seq;
            return metadata;
          };
          // read the doc back out from the database. we don't store the
          // _id or _rev because we already have _doc_id_rev.
          exports.decodeDoc = function (doc) {
            if (!doc) {
              return doc;
            }
            var idx = utils.lastIndexOf(doc._doc_id_rev, ':');
            doc._id = doc._doc_id_rev.substring(0, idx - 1);
            doc._rev = doc._doc_id_rev.substring(idx + 1);
            delete doc._doc_id_rev;
            return doc;
          };
          // Read a blob from the database, encoding as necessary
          // and translating from base64 if the IDB doesn't support
          // native Blobs
          exports.readBlobData = function (body, type, encode, callback) {
            if (encode) {
              if (!body) {
                callback('');
              } else if (typeof body !== 'string') {
                // we have blob support
                utils.readAsBinaryString(body, function (binary) {
                  callback(utils.btoa(binary));
                });
              } else {
                // no blob support
                callback(body);
              }
            } else {
              if (!body) {
                callback(utils.createBlob([''], { type: type }));
              } else if (typeof body !== 'string') {
                // we have blob support
                callback(body);
              } else {
                // no blob support
                body = utils.fixBinary(atob(body));
                callback(utils.createBlob([body], { type: type }));
              }
            }
          };
          exports.fetchAttachmentsIfNecessary = function (doc, opts, txn, cb) {
            var attachments = Object.keys(doc._attachments || {});
            if (!attachments.length) {
              return cb && cb();
            }
            var numDone = 0;
            function checkDone() {
              if (++numDone === attachments.length && cb) {
                cb();
              }
            }
            function fetchAttachment(doc, att) {
              var attObj = doc._attachments[att];
              var digest = attObj.digest;
              var req = txn.objectStore(constants.ATTACH_STORE).get(digest);
              req.onsuccess = function (e) {
                attObj.body = e.target.result.body;
                checkDone();
              };
            }
            attachments.forEach(function (att) {
              if (opts.attachments && opts.include_docs) {
                fetchAttachment(doc, att);
              } else {
                doc._attachments[att].stub = true;
                checkDone();
              }
            });
          };
          // IDB-specific postprocessing necessary because
          // we don't know whether we stored a true Blob or
          // a base64-encoded string, and if it's a Blob it
          // needs to be read outside of the transaction context
          exports.postProcessAttachments = function (results) {
            return utils.Promise.all(results.map(function (row) {
              if (row.doc && row.doc._attachments) {
                var attNames = Object.keys(row.doc._attachments);
                return utils.Promise.all(attNames.map(function (att) {
                  var attObj = row.doc._attachments[att];
                  if (!('body' in attObj)) {
                    // already processed
                    return;
                  }
                  var body = attObj.body;
                  var type = attObj.content_type;
                  return new utils.Promise(function (resolve) {
                    exports.readBlobData(body, type, true, function (base64) {
                      row.doc._attachments[att] = utils.extend(utils.pick(attObj, [
                        'digest',
                        'content_type'
                      ]), { data: base64 });
                      resolve();
                    });
                  });
                }));
              }
            }));
          };
          exports.compactRevs = function (revs, docId, txn) {
            var possiblyOrphanedDigests = [];
            var seqStore = txn.objectStore(constants.BY_SEQ_STORE);
            var attStore = txn.objectStore(constants.ATTACH_STORE);
            var attAndSeqStore = txn.objectStore(constants.ATTACH_AND_SEQ_STORE);
            var count = revs.length;
            function checkDone() {
              count--;
              if (!count) {
                // done processing all revs
                deleteOrphanedAttachments();
              }
            }
            function deleteOrphanedAttachments() {
              if (!possiblyOrphanedDigests.length) {
                return;
              }
              possiblyOrphanedDigests.forEach(function (digest) {
                var countReq = attAndSeqStore.index('digestSeq').count(IDBKeyRange.bound(digest + '::', digest + '::\uffff', false, false));
                countReq.onsuccess = function (e) {
                  var count = e.target.result;
                  if (!count) {
                    // orphaned
                    attStore['delete'](digest);
                  }
                };
              });
            }
            revs.forEach(function (rev) {
              var index = seqStore.index('_doc_id_rev');
              var key = docId + '::' + rev;
              index.getKey(key).onsuccess = function (e) {
                var seq = e.target.result;
                if (typeof seq !== 'number') {
                  return checkDone();
                }
                seqStore['delete'](seq);
                var cursor = attAndSeqStore.index('seq').openCursor(IDBKeyRange.only(seq));
                cursor.onsuccess = function (event) {
                  var cursor = event.target.result;
                  if (cursor) {
                    var digest = cursor.value.digestSeq.split('::')[0];
                    possiblyOrphanedDigests.push(digest);
                    attAndSeqStore['delete'](cursor.primaryKey);
                    cursor['continue']();
                  } else {
                    // done
                    checkDone();
                  }
                };
              };
            });
          };
          exports.openTransactionSafely = function (idb, stores, mode) {
            try {
              return { txn: idb.transaction(stores, mode) };
            } catch (err) {
              return { error: err };
            }
          };
        }.call(this, _dereq_(40)));
      },
      {
        '20': 20,
        '35': 35,
        '40': 40,
        '6': 6
      }
    ],
    8: [
      function (_dereq_, module, exports) {
        (function (process) {
          'use strict';
          var utils = _dereq_(35);
          var merge = _dereq_(30);
          var errors = _dereq_(20);
          var idbUtils = _dereq_(7);
          var idbConstants = _dereq_(6);
          var idbBulkDocs = _dereq_(5);
          var idbAllDocs = _dereq_(3);
          var checkBlobSupport = _dereq_(4);
          var ADAPTER_VERSION = idbConstants.ADAPTER_VERSION;
          var ATTACH_AND_SEQ_STORE = idbConstants.ATTACH_AND_SEQ_STORE;
          var ATTACH_STORE = idbConstants.ATTACH_STORE;
          var BY_SEQ_STORE = idbConstants.BY_SEQ_STORE;
          var DETECT_BLOB_SUPPORT_STORE = idbConstants.DETECT_BLOB_SUPPORT_STORE;
          var DOC_STORE = idbConstants.DOC_STORE;
          var LOCAL_STORE = idbConstants.LOCAL_STORE;
          var META_STORE = idbConstants.META_STORE;
          var applyNext = idbUtils.applyNext;
          var compactRevs = idbUtils.compactRevs;
          var decodeDoc = idbUtils.decodeDoc;
          var decodeMetadata = idbUtils.decodeMetadata;
          var encodeMetadata = idbUtils.encodeMetadata;
          var fetchAttachmentsIfNecessary = idbUtils.fetchAttachmentsIfNecessary;
          var idbError = idbUtils.idbError;
          var postProcessAttachments = idbUtils.postProcessAttachments;
          var readBlobData = idbUtils.readBlobData;
          var taskQueue = idbUtils.taskQueue;
          var openTransactionSafely = idbUtils.openTransactionSafely;
          var cachedDBs = {};
          var blobSupportPromise;
          function IdbPouch(opts, callback) {
            var api = this;
            taskQueue.queue.push({
              action: function (thisCallback) {
                init(api, opts, thisCallback);
              },
              callback: callback
            });
            applyNext();
          }
          function init(api, opts, callback) {
            var dbName = opts.name;
            var idb = null;
            api._meta = null;
            // called when creating a fresh new database
            function createSchema(db) {
              var docStore = db.createObjectStore(DOC_STORE, { keyPath: 'id' });
              db.createObjectStore(BY_SEQ_STORE, { autoIncrement: true }).createIndex('_doc_id_rev', '_doc_id_rev', { unique: true });
              db.createObjectStore(ATTACH_STORE, { keyPath: 'digest' });
              db.createObjectStore(META_STORE, {
                keyPath: 'id',
                autoIncrement: false
              });
              db.createObjectStore(DETECT_BLOB_SUPPORT_STORE);
              // added in v2
              docStore.createIndex('deletedOrLocal', 'deletedOrLocal', { unique: false });
              // added in v3
              db.createObjectStore(LOCAL_STORE, { keyPath: '_id' });
              // added in v4
              var attAndSeqStore = db.createObjectStore(ATTACH_AND_SEQ_STORE, { autoIncrement: true });
              attAndSeqStore.createIndex('seq', 'seq');
              attAndSeqStore.createIndex('digestSeq', 'digestSeq', { unique: true });
            }
            // migration to version 2
            // unfortunately "deletedOrLocal" is a misnomer now that we no longer
            // store local docs in the main doc-store, but whaddyagonnado
            function addDeletedOrLocalIndex(txn, callback) {
              var docStore = txn.objectStore(DOC_STORE);
              docStore.createIndex('deletedOrLocal', 'deletedOrLocal', { unique: false });
              docStore.openCursor().onsuccess = function (event) {
                var cursor = event.target.result;
                if (cursor) {
                  var metadata = cursor.value;
                  var deleted = utils.isDeleted(metadata);
                  metadata.deletedOrLocal = deleted ? '1' : '0';
                  docStore.put(metadata);
                  cursor['continue']();
                } else {
                  callback();
                }
              };
            }
            // migration to version 3 (part 1)
            function createLocalStoreSchema(db) {
              db.createObjectStore(LOCAL_STORE, { keyPath: '_id' }).createIndex('_doc_id_rev', '_doc_id_rev', { unique: true });
            }
            // migration to version 3 (part 2)
            function migrateLocalStore(txn, cb) {
              var localStore = txn.objectStore(LOCAL_STORE);
              var docStore = txn.objectStore(DOC_STORE);
              var seqStore = txn.objectStore(BY_SEQ_STORE);
              var cursor = docStore.openCursor();
              cursor.onsuccess = function (event) {
                var cursor = event.target.result;
                if (cursor) {
                  var metadata = cursor.value;
                  var docId = metadata.id;
                  var local = utils.isLocalId(docId);
                  var rev = merge.winningRev(metadata);
                  if (local) {
                    var docIdRev = docId + '::' + rev;
                    // remove all seq entries
                    // associated with this docId
                    var start = docId + '::';
                    var end = docId + '::~';
                    var index = seqStore.index('_doc_id_rev');
                    var range = IDBKeyRange.bound(start, end, false, false);
                    var seqCursor = index.openCursor(range);
                    seqCursor.onsuccess = function (e) {
                      seqCursor = e.target.result;
                      if (!seqCursor) {
                        // done
                        docStore['delete'](cursor.primaryKey);
                        cursor['continue']();
                      } else {
                        var data = seqCursor.value;
                        if (data._doc_id_rev === docIdRev) {
                          localStore.put(data);
                        }
                        seqStore['delete'](seqCursor.primaryKey);
                        seqCursor['continue']();
                      }
                    };
                  } else {
                    cursor['continue']();
                  }
                } else if (cb) {
                  cb();
                }
              };
            }
            // migration to version 4 (part 1)
            function addAttachAndSeqStore(db) {
              var attAndSeqStore = db.createObjectStore(ATTACH_AND_SEQ_STORE, { autoIncrement: true });
              attAndSeqStore.createIndex('seq', 'seq');
              attAndSeqStore.createIndex('digestSeq', 'digestSeq', { unique: true });
            }
            // migration to version 4 (part 2)
            function migrateAttsAndSeqs(txn, callback) {
              var seqStore = txn.objectStore(BY_SEQ_STORE);
              var attStore = txn.objectStore(ATTACH_STORE);
              var attAndSeqStore = txn.objectStore(ATTACH_AND_SEQ_STORE);
              // need to actually populate the table. this is the expensive part,
              // so as an optimization, check first that this database even
              // contains attachments
              var req = attStore.count();
              req.onsuccess = function (e) {
                var count = e.target.result;
                if (!count) {
                  return callback();  // done
                }
                seqStore.openCursor().onsuccess = function (e) {
                  var cursor = e.target.result;
                  if (!cursor) {
                    return callback();  // done
                  }
                  var doc = cursor.value;
                  var seq = cursor.primaryKey;
                  var atts = Object.keys(doc._attachments || {});
                  var digestMap = {};
                  for (var j = 0; j < atts.length; j++) {
                    var att = doc._attachments[atts[j]];
                    digestMap[att.digest] = true;  // uniq digests, just in case
                  }
                  var digests = Object.keys(digestMap);
                  for (j = 0; j < digests.length; j++) {
                    var digest = digests[j];
                    attAndSeqStore.put({
                      seq: seq,
                      digestSeq: digest + '::' + seq
                    });
                  }
                  cursor['continue']();
                };
              };
            }
            // migration to version 5
            // Instead of relying on on-the-fly migration of metadata,
            // this brings the doc-store to its modern form:
            // - metadata.winningrev
            // - metadata.seq
            // - stringify the metadata when storing it
            function migrateMetadata(txn) {
              function decodeMetadataCompat(storedObject) {
                if (!storedObject.data) {
                  // old format, when we didn't store it stringified
                  storedObject.deleted = storedObject.deletedOrLocal === '1';
                  return storedObject;
                }
                return decodeMetadata(storedObject);
              }
              // ensure that every metadata has a winningRev and seq,
              // which was previously created on-the-fly but better to migrate
              var bySeqStore = txn.objectStore(BY_SEQ_STORE);
              var docStore = txn.objectStore(DOC_STORE);
              var cursor = docStore.openCursor();
              cursor.onsuccess = function (e) {
                var cursor = e.target.result;
                if (!cursor) {
                  return;  // done
                }
                var metadata = decodeMetadataCompat(cursor.value);
                metadata.winningRev = metadata.winningRev || merge.winningRev(metadata);
                function fetchMetadataSeq() {
                  // metadata.seq was added post-3.2.0, so if it's missing,
                  // we need to fetch it manually
                  var start = metadata.id + '::';
                  var end = metadata.id + '::\uffff';
                  var req = bySeqStore.index('_doc_id_rev').openCursor(IDBKeyRange.bound(start, end));
                  var metadataSeq = 0;
                  req.onsuccess = function (e) {
                    var cursor = e.target.result;
                    if (!cursor) {
                      metadata.seq = metadataSeq;
                      return onGetMetadataSeq();
                    }
                    var seq = cursor.primaryKey;
                    if (seq > metadataSeq) {
                      metadataSeq = seq;
                    }
                    cursor['continue']();
                  };
                }
                function onGetMetadataSeq() {
                  var metadataToStore = encodeMetadata(metadata, metadata.winningRev, metadata.deleted);
                  var req = docStore.put(metadataToStore);
                  req.onsuccess = function () {
                    cursor['continue']();
                  };
                }
                if (metadata.seq) {
                  return onGetMetadataSeq();
                }
                fetchMetadataSeq();
              };
            }
            api.type = function () {
              return 'idb';
            };
            api._id = utils.toPromise(function (callback) {
              callback(null, api._meta.instanceId);
            });
            api._bulkDocs = function idb_bulkDocs(req, opts, callback) {
              idbBulkDocs(req, opts, api, idb, IdbPouch.Changes, callback);
            };
            // First we look up the metadata in the ids database, then we fetch the
            // current revision(s) from the by sequence store
            api._get = function idb_get(id, opts, callback) {
              var doc;
              var metadata;
              var err;
              var txn;
              opts = utils.clone(opts);
              if (opts.ctx) {
                txn = opts.ctx;
              } else {
                var txnResult = openTransactionSafely(idb, [
                    DOC_STORE,
                    BY_SEQ_STORE,
                    ATTACH_STORE
                  ], 'readonly');
                if (txnResult.error) {
                  return callback(txnResult.error);
                }
                txn = txnResult.txn;
              }
              function finish() {
                callback(err, {
                  doc: doc,
                  metadata: metadata,
                  ctx: txn
                });
              }
              txn.objectStore(DOC_STORE).get(id).onsuccess = function (e) {
                metadata = decodeMetadata(e.target.result);
                // we can determine the result here if:
                // 1. there is no such document
                // 2. the document is deleted and we don't ask about specific rev
                // When we ask with opts.rev we expect the answer to be either
                // doc (possibly with _deleted=true) or missing error
                if (!metadata) {
                  err = errors.error(errors.MISSING_DOC, 'missing');
                  return finish();
                }
                if (utils.isDeleted(metadata) && !opts.rev) {
                  err = errors.error(errors.MISSING_DOC, 'deleted');
                  return finish();
                }
                var objectStore = txn.objectStore(BY_SEQ_STORE);
                var rev = opts.rev || metadata.winningRev;
                var key = metadata.id + '::' + rev;
                objectStore.index('_doc_id_rev').get(key).onsuccess = function (e) {
                  doc = e.target.result;
                  if (doc) {
                    doc = decodeDoc(doc);
                  }
                  if (!doc) {
                    err = errors.error(errors.MISSING_DOC, 'missing');
                    return finish();
                  }
                  finish();
                };
              };
            };
            api._getAttachment = function (attachment, opts, callback) {
              var txn;
              opts = utils.clone(opts);
              if (opts.ctx) {
                txn = opts.ctx;
              } else {
                var txnResult = openTransactionSafely(idb, [
                    DOC_STORE,
                    BY_SEQ_STORE,
                    ATTACH_STORE
                  ], 'readonly');
                if (txnResult.error) {
                  return callback(txnResult.error);
                }
                txn = txnResult.txn;
              }
              var digest = attachment.digest;
              var type = attachment.content_type;
              txn.objectStore(ATTACH_STORE).get(digest).onsuccess = function (e) {
                var body = e.target.result.body;
                readBlobData(body, type, opts.encode, function (blobData) {
                  callback(null, blobData);
                });
              };
            };
            api._info = function idb_info(callback) {
              if (idb === null || !cachedDBs[dbName]) {
                var error = new Error('db isn\'t open');
                error.id = 'idbNull';
                return callback(error);
              }
              var updateSeq;
              var docCount;
              var txnResult = openTransactionSafely(idb, [BY_SEQ_STORE], 'readonly');
              if (txnResult.error) {
                return callback(txnResult.error);
              }
              var txn = txnResult.txn;
              var cursor = txn.objectStore(BY_SEQ_STORE).openCursor(null, 'prev');
              cursor.onsuccess = function (event) {
                var cursor = event.target.result;
                updateSeq = cursor ? cursor.key : 0;
                // count within the same txn for consistency
                docCount = api._meta.docCount;
              };
              txn.oncomplete = function () {
                callback(null, {
                  doc_count: docCount,
                  update_seq: updateSeq,
                  idb_attachment_format: api._meta.blobSupport ? 'binary' : 'base64'
                });
              };
            };
            api._allDocs = function idb_allDocs(opts, callback) {
              idbAllDocs(opts, api, idb, callback);
            };
            api._changes = function (opts) {
              opts = utils.clone(opts);
              if (opts.continuous) {
                var id = dbName + ':' + utils.uuid();
                IdbPouch.Changes.addListener(dbName, id, api, opts);
                IdbPouch.Changes.notify(dbName);
                return {
                  cancel: function () {
                    IdbPouch.Changes.removeListener(dbName, id);
                  }
                };
              }
              var docIds = opts.doc_ids && new utils.Set(opts.doc_ids);
              var descending = opts.descending ? 'prev' : null;
              opts.since = opts.since || 0;
              var lastSeq = opts.since;
              var limit = 'limit' in opts ? opts.limit : -1;
              if (limit === 0) {
                limit = 1;  // per CouchDB _changes spec
              }
              var returnDocs;
              if ('returnDocs' in opts) {
                returnDocs = opts.returnDocs;
              } else {
                returnDocs = true;
              }
              var results = [];
              var numResults = 0;
              var filter = utils.filterChange(opts);
              var docIdsToMetadata = new utils.Map();
              var txn;
              var bySeqStore;
              var docStore;
              function onGetCursor(cursor) {
                var doc = decodeDoc(cursor.value);
                var seq = cursor.key;
                if (docIds && !docIds.has(doc._id)) {
                  return cursor['continue']();
                }
                var metadata;
                function onGetMetadata() {
                  if (metadata.seq !== seq) {
                    // some other seq is later
                    return cursor['continue']();
                  }
                  lastSeq = seq;
                  if (metadata.winningRev === doc._rev) {
                    return onGetWinningDoc(doc);
                  }
                  fetchWinningDoc();
                }
                function fetchWinningDoc() {
                  var docIdRev = doc._id + '::' + metadata.winningRev;
                  var req = bySeqStore.index('_doc_id_rev').openCursor(IDBKeyRange.bound(docIdRev, docIdRev + '\uffff'));
                  req.onsuccess = function (e) {
                    onGetWinningDoc(decodeDoc(e.target.result.value));
                  };
                }
                function onGetWinningDoc(winningDoc) {
                  var change = opts.processChange(winningDoc, metadata, opts);
                  change.seq = metadata.seq;
                  if (filter(change)) {
                    numResults++;
                    if (returnDocs) {
                      results.push(change);
                    }
                    // process the attachment immediately
                    // for the benefit of live listeners
                    if (opts.attachments && opts.include_docs) {
                      fetchAttachmentsIfNecessary(winningDoc, opts, txn, function () {
                        postProcessAttachments([change]).then(function () {
                          opts.onChange(change);
                        });
                      });
                    } else {
                      opts.onChange(change);
                    }
                  }
                  if (numResults !== limit) {
                    cursor['continue']();
                  }
                }
                metadata = docIdsToMetadata.get(doc._id);
                if (metadata) {
                  // cached
                  return onGetMetadata();
                }
                // metadata not cached, have to go fetch it
                docStore.get(doc._id).onsuccess = function (event) {
                  metadata = decodeMetadata(event.target.result);
                  docIdsToMetadata.set(doc._id, metadata);
                  onGetMetadata();
                };
              }
              function onsuccess(event) {
                var cursor = event.target.result;
                if (!cursor) {
                  return;
                }
                onGetCursor(cursor);
              }
              function fetchChanges() {
                var objectStores = [
                    DOC_STORE,
                    BY_SEQ_STORE
                  ];
                if (opts.attachments) {
                  objectStores.push(ATTACH_STORE);
                }
                var txnResult = openTransactionSafely(idb, objectStores, 'readonly');
                if (txnResult.error) {
                  return opts.complete(txnResult.error);
                }
                txn = txnResult.txn;
                txn.onerror = idbError(opts.complete);
                txn.oncomplete = onTxnComplete;
                bySeqStore = txn.objectStore(BY_SEQ_STORE);
                docStore = txn.objectStore(DOC_STORE);
                var req;
                if (descending) {
                  req = bySeqStore.openCursor(null, descending);
                } else {
                  req = bySeqStore.openCursor(IDBKeyRange.lowerBound(opts.since, true));
                }
                req.onsuccess = onsuccess;
              }
              fetchChanges();
              function onTxnComplete() {
                function finish() {
                  opts.complete(null, {
                    results: results,
                    last_seq: lastSeq
                  });
                }
                if (!opts.continuous && opts.attachments) {
                  // cannot guarantee that postProcessing was already done,
                  // so do it again
                  postProcessAttachments(results).then(finish);
                } else {
                  finish();
                }
              }
            };
            api._close = function (callback) {
              if (idb === null) {
                return callback(errors.error(errors.NOT_OPEN));
              }
              // https://developer.mozilla.org/en-US/docs/IndexedDB/IDBDatabase#close
              // "Returns immediately and closes the connection in a separate thread..."
              idb.close();
              delete cachedDBs[dbName];
              idb = null;
              callback();
            };
            api._getRevisionTree = function (docId, callback) {
              var txnResult = openTransactionSafely(idb, [DOC_STORE], 'readonly');
              if (txnResult.error) {
                return callback(txnResult.error);
              }
              var txn = txnResult.txn;
              var req = txn.objectStore(DOC_STORE).get(docId);
              req.onsuccess = function (event) {
                var doc = decodeMetadata(event.target.result);
                if (!doc) {
                  callback(errors.error(errors.MISSING_DOC));
                } else {
                  callback(null, doc.rev_tree);
                }
              };
            };
            // This function removes revisions of document docId
            // which are listed in revs and sets this document
            // revision to to rev_tree
            api._doCompaction = function (docId, revs, callback) {
              var stores = [
                  DOC_STORE,
                  BY_SEQ_STORE,
                  ATTACH_STORE,
                  ATTACH_AND_SEQ_STORE
                ];
              var txnResult = openTransactionSafely(idb, stores, 'readwrite');
              if (txnResult.error) {
                return callback(txnResult.error);
              }
              var txn = txnResult.txn;
              var docStore = txn.objectStore(DOC_STORE);
              docStore.get(docId).onsuccess = function (event) {
                var metadata = decodeMetadata(event.target.result);
                merge.traverseRevTree(metadata.rev_tree, function (isLeaf, pos, revHash, ctx, opts) {
                  var rev = pos + '-' + revHash;
                  if (revs.indexOf(rev) !== -1) {
                    opts.status = 'missing';
                  }
                });
                compactRevs(revs, docId, txn);
                var winningRev = metadata.winningRev;
                var deleted = metadata.deleted;
                txn.objectStore(DOC_STORE).put(encodeMetadata(metadata, winningRev, deleted));
              };
              txn.onerror = idbError(callback);
              txn.oncomplete = function () {
                utils.call(callback);
              };
            };
            api._getLocal = function (id, callback) {
              var txnResult = openTransactionSafely(idb, [LOCAL_STORE], 'readonly');
              if (txnResult.error) {
                return callback(txnResult.error);
              }
              var tx = txnResult.txn;
              var req = tx.objectStore(LOCAL_STORE).get(id);
              req.onerror = idbError(callback);
              req.onsuccess = function (e) {
                var doc = e.target.result;
                if (!doc) {
                  callback(errors.error(errors.MISSING_DOC));
                } else {
                  delete doc['_doc_id_rev'];
                  // for backwards compat
                  callback(null, doc);
                }
              };
            };
            api._putLocal = function (doc, opts, callback) {
              if (typeof opts === 'function') {
                callback = opts;
                opts = {};
              }
              delete doc._revisions;
              // ignore this, trust the rev
              var oldRev = doc._rev;
              var id = doc._id;
              if (!oldRev) {
                doc._rev = '0-1';
              } else {
                doc._rev = '0-' + (parseInt(oldRev.split('-')[1], 10) + 1);
              }
              var tx = opts.ctx;
              var ret;
              if (!tx) {
                var txnResult = openTransactionSafely(idb, [LOCAL_STORE], 'readwrite');
                if (txnResult.error) {
                  return callback(txnResult.error);
                }
                tx = txnResult.txn;
                tx.onerror = idbError(callback);
                tx.oncomplete = function () {
                  if (ret) {
                    callback(null, ret);
                  }
                };
              }
              var oStore = tx.objectStore(LOCAL_STORE);
              var req;
              if (oldRev) {
                req = oStore.get(id);
                req.onsuccess = function (e) {
                  var oldDoc = e.target.result;
                  if (!oldDoc || oldDoc._rev !== oldRev) {
                    callback(errors.error(errors.REV_CONFLICT));
                  } else {
                    // update
                    var req = oStore.put(doc);
                    req.onsuccess = function () {
                      ret = {
                        ok: true,
                        id: doc._id,
                        rev: doc._rev
                      };
                      if (opts.ctx) {
                        // return immediately
                        callback(null, ret);
                      }
                    };
                  }
                };
              } else {
                // new doc
                req = oStore.add(doc);
                req.onerror = function (e) {
                  // constraint error, already exists
                  callback(errors.error(errors.REV_CONFLICT));
                  e.preventDefault();
                  // avoid transaction abort
                  e.stopPropagation();  // avoid transaction onerror
                };
                req.onsuccess = function () {
                  ret = {
                    ok: true,
                    id: doc._id,
                    rev: doc._rev
                  };
                  if (opts.ctx) {
                    // return immediately
                    callback(null, ret);
                  }
                };
              }
            };
            api._removeLocal = function (doc, callback) {
              var txnResult = openTransactionSafely(idb, [LOCAL_STORE], 'readwrite');
              if (txnResult.error) {
                return callback(txnResult.error);
              }
              var tx = txnResult.txn;
              var ret;
              tx.oncomplete = function () {
                if (ret) {
                  callback(null, ret);
                }
              };
              var id = doc._id;
              var oStore = tx.objectStore(LOCAL_STORE);
              var req = oStore.get(id);
              req.onerror = idbError(callback);
              req.onsuccess = function (e) {
                var oldDoc = e.target.result;
                if (!oldDoc || oldDoc._rev !== doc._rev) {
                  callback(errors.error(errors.MISSING_DOC));
                } else {
                  oStore['delete'](id);
                  ret = {
                    ok: true,
                    id: id,
                    rev: '0-0'
                  };
                }
              };
            };
            var cached = cachedDBs[dbName];
            if (cached) {
              idb = cached.idb;
              api._meta = cached.global;
              process.nextTick(function () {
                callback(null, api);
              });
              return;
            }
            var req = indexedDB.open(dbName, ADAPTER_VERSION);
            if (!('openReqList' in IdbPouch)) {
              IdbPouch.openReqList = {};
            }
            IdbPouch.openReqList[dbName] = req;
            req.onupgradeneeded = function (e) {
              var db = e.target.result;
              if (e.oldVersion < 1) {
                return createSchema(db);  // new db, initial schema
              }
              // do migrations
              var txn = e.currentTarget.transaction;
              // these migrations have to be done in this function, before
              // control is returned to the event loop, because IndexedDB
              if (e.oldVersion < 3) {
                createLocalStoreSchema(db);  // v2 -> v3
              }
              if (e.oldVersion < 4) {
                addAttachAndSeqStore(db);  // v3 -> v4
              }
              var migrations = [
                  addDeletedOrLocalIndex,
                  migrateLocalStore,
                  migrateAttsAndSeqs,
                  migrateMetadata
                ];
              var i = e.oldVersion;
              function next() {
                var migration = migrations[i - 1];
                i++;
                if (migration) {
                  migration(txn, next);
                }
              }
              next();
            };
            req.onsuccess = function (e) {
              idb = e.target.result;
              idb.onversionchange = function () {
                idb.close();
                delete cachedDBs[dbName];
              };
              idb.onabort = function () {
                idb.close();
                delete cachedDBs[dbName];
              };
              var txn = idb.transaction([
                  META_STORE,
                  DETECT_BLOB_SUPPORT_STORE,
                  DOC_STORE
                ], 'readwrite');
              var req = txn.objectStore(META_STORE).get(META_STORE);
              var blobSupport = null;
              var docCount = null;
              var instanceId = null;
              req.onsuccess = function (e) {
                var checkSetupComplete = function () {
                  if (blobSupport === null || docCount === null || instanceId === null) {
                    return;
                  } else {
                    api._meta = {
                      name: dbName,
                      instanceId: instanceId,
                      blobSupport: blobSupport,
                      docCount: docCount
                    };
                    cachedDBs[dbName] = {
                      idb: idb,
                      global: api._meta
                    };
                    callback(null, api);
                  }
                };
                //
                // fetch/store the id
                //
                var meta = e.target.result || { id: META_STORE };
                if (dbName + '_id' in meta) {
                  instanceId = meta[dbName + '_id'];
                  checkSetupComplete();
                } else {
                  instanceId = utils.uuid();
                  meta[dbName + '_id'] = instanceId;
                  txn.objectStore(META_STORE).put(meta).onsuccess = function () {
                    checkSetupComplete();
                  };
                }
                //
                // check blob support
                //
                if (!blobSupportPromise) {
                  // make sure blob support is only checked once
                  blobSupportPromise = checkBlobSupport(txn, idb);
                }
                blobSupportPromise.then(function (val) {
                  blobSupport = val;
                  checkSetupComplete();
                });
                //
                // count docs
                //
                var index = txn.objectStore(DOC_STORE).index('deletedOrLocal');
                index.count(IDBKeyRange.only('0')).onsuccess = function (e) {
                  docCount = e.target.result;
                  checkSetupComplete();
                };
              };
            };
            req.onerror = idbError(callback);
          }
          IdbPouch.valid = function () {
            // Issue #2533, we finally gave up on doing bug
            // detection instead of browser sniffing. Safari brought us
            // to our knees.
            var isSafari = typeof openDatabase !== 'undefined' && /(Safari|iPhone|iPad|iPod)/.test(navigator.userAgent) && !/Chrome/.test(navigator.userAgent);
            // some outdated implementations of IDB that appear on Samsung
            // and HTC Android devices <4.4 are missing IDBKeyRange
            return !isSafari && typeof indexedDB !== 'undefined' && typeof IDBKeyRange !== 'undefined';
          };
          function destroy(name, opts, callback) {
            if (!('openReqList' in IdbPouch)) {
              IdbPouch.openReqList = {};
            }
            IdbPouch.Changes.removeAllListeners(name);
            //Close open request for "name" database to fix ie delay.
            if (IdbPouch.openReqList[name] && IdbPouch.openReqList[name].result) {
              IdbPouch.openReqList[name].result.close();
              delete cachedDBs[name];
            }
            var req = indexedDB.deleteDatabase(name);
            req.onsuccess = function () {
              //Remove open request from the list.
              if (IdbPouch.openReqList[name]) {
                IdbPouch.openReqList[name] = null;
              }
              if (utils.hasLocalStorage() && name in localStorage) {
                delete localStorage[name];
              }
              callback(null, { 'ok': true });
            };
            req.onerror = idbError(callback);
          }
          IdbPouch.destroy = utils.toPromise(function (name, opts, callback) {
            taskQueue.queue.push({
              action: function (thisCallback) {
                destroy(name, opts, thisCallback);
              },
              callback: callback
            });
            applyNext();
          });
          IdbPouch.Changes = new utils.Changes();
          module.exports = IdbPouch;
        }.call(this, _dereq_(40)));
      },
      {
        '20': 20,
        '3': 3,
        '30': 30,
        '35': 35,
        '4': 4,
        '40': 40,
        '5': 5,
        '6': 6,
        '7': 7
      }
    ],
    9: [
      function (_dereq_, module, exports) {
        module.exports = [
          'idb',
          'websql'
        ];
      },
      {}
    ],
    10: [
      function (_dereq_, module, exports) {
        'use strict';
        var utils = _dereq_(35);
        var errors = _dereq_(20);
        var websqlUtils = _dereq_(12);
        var websqlConstants = _dereq_(11);
        var DOC_STORE = websqlConstants.DOC_STORE;
        var BY_SEQ_STORE = websqlConstants.BY_SEQ_STORE;
        var ATTACH_STORE = websqlConstants.ATTACH_STORE;
        var ATTACH_AND_SEQ_STORE = websqlConstants.ATTACH_AND_SEQ_STORE;
        var select = websqlUtils.select;
        var stringifyDoc = websqlUtils.stringifyDoc;
        var compactRevs = websqlUtils.compactRevs;
        var unknownError = websqlUtils.unknownError;
        function websqlBulkDocs(req, opts, api, db, Changes, callback) {
          var newEdits = opts.new_edits;
          var userDocs = req.docs;
          // Parse the docs, give them a sequence number for the result
          var docInfos = userDocs.map(function (doc) {
              if (doc._id && utils.isLocalId(doc._id)) {
                return doc;
              }
              var newDoc = utils.parseDoc(doc, newEdits);
              return newDoc;
            });
          var docInfoErrors = docInfos.filter(function (docInfo) {
              return docInfo.error;
            });
          if (docInfoErrors.length) {
            return callback(docInfoErrors[0]);
          }
          var tx;
          var results = new Array(docInfos.length);
          var fetchedDocs = new utils.Map();
          var preconditionErrored;
          function complete() {
            if (preconditionErrored) {
              return callback(preconditionErrored);
            }
            Changes.notify(api._name);
            api._docCount = -1;
            // invalidate
            callback(null, results);
          }
          function verifyAttachment(digest, callback) {
            var sql = 'SELECT count(*) as cnt FROM ' + ATTACH_STORE + ' WHERE digest=?';
            tx.executeSql(sql, [digest], function (tx, result) {
              if (result.rows.item(0).cnt === 0) {
                var err = errors.error(errors.MISSING_STUB, 'unknown stub attachment with digest ' + digest);
                callback(err);
              } else {
                callback();
              }
            });
          }
          function verifyAttachments(finish) {
            var digests = [];
            docInfos.forEach(function (docInfo) {
              if (docInfo.data && docInfo.data._attachments) {
                Object.keys(docInfo.data._attachments).forEach(function (filename) {
                  var att = docInfo.data._attachments[filename];
                  if (att.stub) {
                    digests.push(att.digest);
                  }
                });
              }
            });
            if (!digests.length) {
              return finish();
            }
            var numDone = 0;
            var err;
            function checkDone() {
              if (++numDone === digests.length) {
                finish(err);
              }
            }
            digests.forEach(function (digest) {
              verifyAttachment(digest, function (attErr) {
                if (attErr && !err) {
                  err = attErr;
                }
                checkDone();
              });
            });
          }
          function writeDoc(docInfo, winningRev, winningRevIsDeleted, newRevIsDeleted, isUpdate, delta, resultsIdx, callback) {
            function finish() {
              var data = docInfo.data;
              var deletedInt = newRevIsDeleted ? 1 : 0;
              var id = data._id;
              var rev = data._rev;
              var json = stringifyDoc(data);
              var sql = 'INSERT INTO ' + BY_SEQ_STORE + ' (doc_id, rev, json, deleted) VALUES (?, ?, ?, ?);';
              var sqlArgs = [
                  id,
                  rev,
                  json,
                  deletedInt
                ];
              // map seqs to attachment digests, which
              // we will need later during compaction
              function insertAttachmentMappings(seq, callback) {
                var attsAdded = 0;
                var attsToAdd = Object.keys(data._attachments || {});
                if (!attsToAdd.length) {
                  return callback();
                }
                function checkDone() {
                  if (++attsAdded === attsToAdd.length) {
                    callback();
                  }
                  return false;  // ack handling a constraint error
                }
                function add(att) {
                  var sql = 'INSERT INTO ' + ATTACH_AND_SEQ_STORE + ' (digest, seq) VALUES (?,?)';
                  var sqlArgs = [
                      data._attachments[att].digest,
                      seq
                    ];
                  tx.executeSql(sql, sqlArgs, checkDone, checkDone);  // second callback is for a constaint error, which we ignore
                                                                      // because this docid/rev has already been associated with
                                                                      // the digest (e.g. when new_edits == false)
                }
                for (var i = 0; i < attsToAdd.length; i++) {
                  add(attsToAdd[i]);  // do in parallel
                }
              }
              tx.executeSql(sql, sqlArgs, function (tx, result) {
                var seq = result.insertId;
                insertAttachmentMappings(seq, function () {
                  dataWritten(tx, seq);
                });
              }, function () {
                // constraint error, recover by updating instead (see #1638)
                var fetchSql = select('seq', BY_SEQ_STORE, null, 'doc_id=? AND rev=?');
                tx.executeSql(fetchSql, [
                  id,
                  rev
                ], function (tx, res) {
                  var seq = res.rows.item(0).seq;
                  var sql = 'UPDATE ' + BY_SEQ_STORE + ' SET json=?, deleted=? WHERE doc_id=? AND rev=?;';
                  var sqlArgs = [
                      json,
                      deletedInt,
                      id,
                      rev
                    ];
                  tx.executeSql(sql, sqlArgs, function (tx) {
                    insertAttachmentMappings(seq, function () {
                      dataWritten(tx, seq);
                    });
                  });
                });
                return false;  // ack that we've handled the error
              });
            }
            function collectResults(attachmentErr) {
              if (!err) {
                if (attachmentErr) {
                  err = attachmentErr;
                  callback(err);
                } else if (recv === attachments.length) {
                  finish();
                }
              }
            }
            var err = null;
            var recv = 0;
            docInfo.data._id = docInfo.metadata.id;
            docInfo.data._rev = docInfo.metadata.rev;
            var attachments = Object.keys(docInfo.data._attachments || {});
            if (newRevIsDeleted) {
              docInfo.data._deleted = true;
            }
            function attachmentSaved(err) {
              recv++;
              collectResults(err);
            }
            attachments.forEach(function (key) {
              var att = docInfo.data._attachments[key];
              if (!att.stub) {
                var data = att.data;
                delete att.data;
                var digest = att.digest;
                saveAttachment(digest, data, attachmentSaved);
              } else {
                recv++;
                collectResults();
              }
            });
            if (!attachments.length) {
              finish();
            }
            function autoCompact() {
              if (!isUpdate || !api.auto_compaction) {
                return;  // nothing to do
              }
              var id = docInfo.metadata.id;
              var revsToDelete = utils.compactTree(docInfo.metadata);
              compactRevs(revsToDelete, id, tx);
            }
            function dataWritten(tx, seq) {
              autoCompact();
              docInfo.metadata.seq = seq;
              delete docInfo.metadata.rev;
              var sql = isUpdate ? 'UPDATE ' + DOC_STORE + ' SET json=?, max_seq=?, winningseq=' + '(SELECT seq FROM ' + BY_SEQ_STORE + ' WHERE doc_id=' + DOC_STORE + '.id AND rev=?) WHERE id=?' : 'INSERT INTO ' + DOC_STORE + ' (id, winningseq, max_seq, json) VALUES (?,?,?,?);';
              var metadataStr = utils.safeJsonStringify(docInfo.metadata);
              var id = docInfo.metadata.id;
              var params = isUpdate ? [
                  metadataStr,
                  seq,
                  winningRev,
                  id
                ] : [
                  id,
                  seq,
                  seq,
                  metadataStr
                ];
              tx.executeSql(sql, params, function () {
                results[resultsIdx] = {
                  ok: true,
                  id: docInfo.metadata.id,
                  rev: winningRev
                };
                fetchedDocs.set(id, docInfo.metadata);
                callback();
              });
            }
          }
          function processDocs() {
            utils.processDocs(docInfos, api, fetchedDocs, tx, results, writeDoc, opts);
          }
          function fetchExistingDocs(callback) {
            if (!docInfos.length) {
              return callback();
            }
            var numFetched = 0;
            function checkDone() {
              if (++numFetched === docInfos.length) {
                callback();
              }
            }
            docInfos.forEach(function (docInfo) {
              if (docInfo._id && utils.isLocalId(docInfo._id)) {
                return checkDone();  // skip local docs
              }
              var id = docInfo.metadata.id;
              tx.executeSql('SELECT json FROM ' + DOC_STORE + ' WHERE id = ?', [id], function (tx, result) {
                if (result.rows.length) {
                  var metadata = utils.safeJsonParse(result.rows.item(0).json);
                  fetchedDocs.set(id, metadata);
                }
                checkDone();
              });
            });
          }
          function saveAttachment(digest, data, callback) {
            var sql = 'SELECT digest FROM ' + ATTACH_STORE + ' WHERE digest=?';
            tx.executeSql(sql, [digest], function (tx, result) {
              if (result.rows.length) {
                // attachment already exists
                return callback();
              }
              // we could just insert before selecting and catch the error,
              // but my hunch is that it's cheaper not to serialize the blob
              // from JS to C if we don't have to (TODO: confirm this)
              sql = 'INSERT INTO ' + ATTACH_STORE + ' (digest, body, escaped) VALUES (?,?,1)';
              tx.executeSql(sql, [
                digest,
                websqlUtils.escapeBlob(data)
              ], function () {
                callback();
              }, function () {
                // ignore constaint errors, means it already exists
                callback();
                return false;  // ack we handled the error
              });
            });
          }
          utils.preprocessAttachments(docInfos, 'binary', function (err) {
            if (err) {
              return callback(err);
            }
            db.transaction(function (txn) {
              tx = txn;
              verifyAttachments(function (err) {
                if (err) {
                  preconditionErrored = err;
                } else {
                  fetchExistingDocs(processDocs);
                }
              });
            }, unknownError(callback), complete);
          });
        }
        module.exports = websqlBulkDocs;
      },
      {
        '11': 11,
        '12': 12,
        '20': 20,
        '35': 35
      }
    ],
    11: [
      function (_dereq_, module, exports) {
        'use strict';
        function quote(str) {
          return '\'' + str + '\'';
        }
        exports.ADAPTER_VERSION = 7;
        // used to manage migrations
        // The object stores created for each database
        // DOC_STORE stores the document meta data, its revision history and state
        exports.DOC_STORE = quote('document-store');
        // BY_SEQ_STORE stores a particular version of a document, keyed by its
        // sequence id
        exports.BY_SEQ_STORE = quote('by-sequence');
        // Where we store attachments
        exports.ATTACH_STORE = quote('attach-store');
        exports.LOCAL_STORE = quote('local-store');
        exports.META_STORE = quote('metadata-store');
        // where we store many-to-many relations between attachment
        // digests and seqs
        exports.ATTACH_AND_SEQ_STORE = quote('attach-seq-store');
      },
      {}
    ],
    12: [
      function (_dereq_, module, exports) {
        'use strict';
        var utils = _dereq_(35);
        var errors = _dereq_(20);
        var websqlConstants = _dereq_(11);
        var BY_SEQ_STORE = websqlConstants.BY_SEQ_STORE;
        var ATTACH_STORE = websqlConstants.ATTACH_STORE;
        var ATTACH_AND_SEQ_STORE = websqlConstants.ATTACH_AND_SEQ_STORE;
        // escapeBlob and unescapeBlob are workarounds for a websql bug:
        // https://code.google.com/p/chromium/issues/detail?id=422690
        // https://bugs.webkit.org/show_bug.cgi?id=137637
        // The goal is to never actually insert the \u0000 character
        // in the database.
        function escapeBlob(str) {
          return str.replace(/\u0002/g, '\x02\x02').replace(/\u0001/g, '\x01\x02').replace(/\u0000/g, '\x01\x01');
        }
        function unescapeBlob(str) {
          return str.replace(/\u0001\u0001/g, '\0').replace(/\u0001\u0002/g, '\x01').replace(/\u0002\u0002/g, '\x02');
        }
        function stringifyDoc(doc) {
          // don't bother storing the id/rev. it uses lots of space,
          // in persistent map/reduce especially
          delete doc._id;
          delete doc._rev;
          return JSON.stringify(doc);
        }
        function unstringifyDoc(doc, id, rev) {
          doc = JSON.parse(doc);
          doc._id = id;
          doc._rev = rev;
          return doc;
        }
        // question mark groups IN queries, e.g. 3 -> '(?,?,?)'
        function qMarks(num) {
          var s = '(';
          while (num--) {
            s += '?';
            if (num) {
              s += ',';
            }
          }
          return s + ')';
        }
        function select(selector, table, joiner, where, orderBy) {
          return 'SELECT ' + selector + ' FROM ' + (typeof table === 'string' ? table : table.join(' JOIN ')) + (joiner ? ' ON ' + joiner : '') + (where ? ' WHERE ' + (typeof where === 'string' ? where : where.join(' AND ')) : '') + (orderBy ? ' ORDER BY ' + orderBy : '');
        }
        function compactRevs(revs, docId, tx) {
          if (!revs.length) {
            return;
          }
          var numDone = 0;
          var seqs = [];
          function checkDone() {
            if (++numDone === revs.length) {
              // done
              deleteOrphans();
            }
          }
          function deleteOrphans() {
            // find orphaned attachment digests
            if (!seqs.length) {
              return;
            }
            var sql = 'SELECT DISTINCT digest AS digest FROM ' + ATTACH_AND_SEQ_STORE + ' WHERE seq IN ' + qMarks(seqs.length);
            tx.executeSql(sql, seqs, function (tx, res) {
              var digestsToCheck = [];
              for (var i = 0; i < res.rows.length; i++) {
                digestsToCheck.push(res.rows.item(i).digest);
              }
              if (!digestsToCheck.length) {
                return;
              }
              var sql = 'DELETE FROM ' + ATTACH_AND_SEQ_STORE + ' WHERE seq IN (' + seqs.map(function () {
                  return '?';
                }).join(',') + ')';
              tx.executeSql(sql, seqs, function (tx) {
                var sql = 'SELECT digest FROM ' + ATTACH_AND_SEQ_STORE + ' WHERE digest IN (' + digestsToCheck.map(function () {
                    return '?';
                  }).join(',') + ')';
                tx.executeSql(sql, digestsToCheck, function (tx, res) {
                  var nonOrphanedDigests = new utils.Set();
                  for (var i = 0; i < res.rows.length; i++) {
                    nonOrphanedDigests.add(res.rows.item(i).digest);
                  }
                  digestsToCheck.forEach(function (digest) {
                    if (nonOrphanedDigests.has(digest)) {
                      return;
                    }
                    tx.executeSql('DELETE FROM ' + ATTACH_AND_SEQ_STORE + ' WHERE digest=?', [digest]);
                    tx.executeSql('DELETE FROM ' + ATTACH_STORE + ' WHERE digest=?', [digest]);
                  });
                });
              });
            });
          }
          // update by-seq and attach stores in parallel
          revs.forEach(function (rev) {
            var sql = 'SELECT seq FROM ' + BY_SEQ_STORE + ' WHERE doc_id=? AND rev=?';
            tx.executeSql(sql, [
              docId,
              rev
            ], function (tx, res) {
              if (!res.rows.length) {
                // already deleted
                return checkDone();
              }
              var seq = res.rows.item(0).seq;
              seqs.push(seq);
              tx.executeSql('DELETE FROM ' + BY_SEQ_STORE + ' WHERE seq=?', [seq], checkDone);
            });
          });
        }
        function unknownError(callback) {
          return function (event) {
            // event may actually be a SQLError object, so report is as such
            var errorNameMatch = event && event.constructor.toString().match(/function ([^\(]+)/);
            var errorName = errorNameMatch && errorNameMatch[1] || event.type;
            var errorReason = event.target || event.message;
            callback(errors.error(errors.WSQ_ERROR, errorReason, errorName));
          };
        }
        function getSize(opts) {
          if ('size' in opts) {
            // triggers immediate popup in iOS, fixes #2347
            // e.g. 5000001 asks for 5 MB, 10000001 asks for 10 MB,
            return opts.size * 1000000;
          }
          // In iOS, doesn't matter as long as it's <= 5000000.
          // Except that if you request too much, our tests fail
          // because of the native "do you accept?" popup.
          // In Android <=4.3, this value is actually used as an
          // honest-to-god ceiling for data, so we need to
          // set it to a decently high number.
          var isAndroid = /Android/.test(window.navigator.userAgent);
          return isAndroid ? 5000000 : 1;  // in PhantomJS, if you use 0 it will crash
        }
        function createOpenDBFunction() {
          if (typeof sqlitePlugin !== 'undefined') {
            // The SQLite Plugin started deviating pretty heavily from the
            // standard openDatabase() function, as they started adding more features.
            // It's better to just use their "new" format and pass in a big ol'
            // options object.
            return sqlitePlugin.openDatabase.bind(sqlitePlugin);
          }
          if (typeof openDatabase !== 'undefined') {
            return function openDB(opts) {
              // Traditional WebSQL API
              return openDatabase(opts.name, opts.version, opts.description, opts.size);
            };
          }
        }
        var cachedDatabases = {};
        function openDB(opts) {
          var openDBFunction = createOpenDBFunction();
          var db = cachedDatabases[opts.name];
          if (!db) {
            db = cachedDatabases[opts.name] = openDBFunction(opts);
            db._sqlitePlugin = typeof sqlitePlugin !== 'undefined';
          }
          return db;
        }
        function valid() {
          // SQLitePlugin leaks this global object, which we can use
          // to detect if it's installed or not. The benefit is that it's
          // declared immediately, before the 'deviceready' event has fired.
          return typeof openDatabase !== 'undefined' || typeof SQLitePlugin !== 'undefined';
        }
        module.exports = {
          escapeBlob: escapeBlob,
          unescapeBlob: unescapeBlob,
          stringifyDoc: stringifyDoc,
          unstringifyDoc: unstringifyDoc,
          qMarks: qMarks,
          select: select,
          compactRevs: compactRevs,
          unknownError: unknownError,
          getSize: getSize,
          openDB: openDB,
          valid: valid
        };
      },
      {
        '11': 11,
        '20': 20,
        '35': 35
      }
    ],
    13: [
      function (_dereq_, module, exports) {
        'use strict';
        var utils = _dereq_(35);
        var merge = _dereq_(30);
        var errors = _dereq_(20);
        var parseHexString = _dereq_(23);
        var websqlConstants = _dereq_(11);
        var websqlUtils = _dereq_(12);
        var websqlBulkDocs = _dereq_(10);
        var ADAPTER_VERSION = websqlConstants.ADAPTER_VERSION;
        var DOC_STORE = websqlConstants.DOC_STORE;
        var BY_SEQ_STORE = websqlConstants.BY_SEQ_STORE;
        var ATTACH_STORE = websqlConstants.ATTACH_STORE;
        var LOCAL_STORE = websqlConstants.LOCAL_STORE;
        var META_STORE = websqlConstants.META_STORE;
        var ATTACH_AND_SEQ_STORE = websqlConstants.ATTACH_AND_SEQ_STORE;
        var qMarks = websqlUtils.qMarks;
        var stringifyDoc = websqlUtils.stringifyDoc;
        var unstringifyDoc = websqlUtils.unstringifyDoc;
        var select = websqlUtils.select;
        var compactRevs = websqlUtils.compactRevs;
        var unknownError = websqlUtils.unknownError;
        var getSize = websqlUtils.getSize;
        var openDB = websqlUtils.openDB;
        function fetchAttachmentsIfNecessary(doc, opts, api, txn, cb) {
          var attachments = Object.keys(doc._attachments || {});
          if (!attachments.length) {
            return cb && cb();
          }
          var numDone = 0;
          function checkDone() {
            if (++numDone === attachments.length && cb) {
              cb();
            }
          }
          function fetchAttachment(doc, att) {
            var attObj = doc._attachments[att];
            var attOpts = {
                encode: true,
                ctx: txn
              };
            api._getAttachment(attObj, attOpts, function (_, base64) {
              doc._attachments[att] = utils.extend(utils.pick(attObj, [
                'digest',
                'content_type'
              ]), { data: base64 });
              checkDone();
            });
          }
          attachments.forEach(function (att) {
            if (opts.attachments && opts.include_docs) {
              fetchAttachment(doc, att);
            } else {
              doc._attachments[att].stub = true;
              checkDone();
            }
          });
        }
        var POUCH_VERSION = 1;
        // these indexes cover the ground for most allDocs queries
        var BY_SEQ_STORE_DELETED_INDEX_SQL = 'CREATE INDEX IF NOT EXISTS \'by-seq-deleted-idx\' ON ' + BY_SEQ_STORE + ' (seq, deleted)';
        var BY_SEQ_STORE_DOC_ID_REV_INDEX_SQL = 'CREATE UNIQUE INDEX IF NOT EXISTS \'by-seq-doc-id-rev\' ON ' + BY_SEQ_STORE + ' (doc_id, rev)';
        var DOC_STORE_WINNINGSEQ_INDEX_SQL = 'CREATE INDEX IF NOT EXISTS \'doc-winningseq-idx\' ON ' + DOC_STORE + ' (winningseq)';
        var ATTACH_AND_SEQ_STORE_SEQ_INDEX_SQL = 'CREATE INDEX IF NOT EXISTS \'attach-seq-seq-idx\' ON ' + ATTACH_AND_SEQ_STORE + ' (seq)';
        var ATTACH_AND_SEQ_STORE_ATTACH_INDEX_SQL = 'CREATE UNIQUE INDEX IF NOT EXISTS \'attach-seq-digest-idx\' ON ' + ATTACH_AND_SEQ_STORE + ' (digest, seq)';
        var DOC_STORE_AND_BY_SEQ_JOINER = BY_SEQ_STORE + '.seq = ' + DOC_STORE + '.winningseq';
        var SELECT_DOCS = BY_SEQ_STORE + '.seq AS seq, ' + BY_SEQ_STORE + '.deleted AS deleted, ' + BY_SEQ_STORE + '.json AS data, ' + BY_SEQ_STORE + '.rev AS rev, ' + DOC_STORE + '.json AS metadata';
        function WebSqlPouch(opts, callback) {
          var api = this;
          var instanceId = null;
          var size = getSize(opts);
          var idRequests = [];
          var encoding;
          api._docCount = -1;
          // cache sqlite count(*) for performance
          api._name = opts.name;
          var db = openDB({
              name: api._name,
              version: POUCH_VERSION,
              description: api._name,
              size: size,
              location: opts.location,
              createFromLocation: opts.createFromLocation
            });
          if (!db) {
            return callback(errors.error(errors.UNKNOWN_ERROR));
          } else if (typeof db.readTransaction !== 'function') {
            // doesn't exist in sqlite plugin
            db.readTransaction = db.transaction;
          }
          function dbCreated() {
            // note the db name in case the browser upgrades to idb
            if (utils.hasLocalStorage()) {
              window.localStorage['_pouch__websqldb_' + api._name] = true;
            }
            callback(null, api);
          }
          // In this migration, we added the 'deleted' and 'local' columns to the
          // by-seq and doc store tables.
          // To preserve existing user data, we re-process all the existing JSON
          // and add these values.
          // Called migration2 because it corresponds to adapter version (db_version) #2
          function runMigration2(tx, callback) {
            // index used for the join in the allDocs query
            tx.executeSql(DOC_STORE_WINNINGSEQ_INDEX_SQL);
            tx.executeSql('ALTER TABLE ' + BY_SEQ_STORE + ' ADD COLUMN deleted TINYINT(1) DEFAULT 0', [], function () {
              tx.executeSql(BY_SEQ_STORE_DELETED_INDEX_SQL);
              tx.executeSql('ALTER TABLE ' + DOC_STORE + ' ADD COLUMN local TINYINT(1) DEFAULT 0', [], function () {
                tx.executeSql('CREATE INDEX IF NOT EXISTS \'doc-store-local-idx\' ON ' + DOC_STORE + ' (local, id)');
                var sql = 'SELECT ' + DOC_STORE + '.winningseq AS seq, ' + DOC_STORE + '.json AS metadata FROM ' + BY_SEQ_STORE + ' JOIN ' + DOC_STORE + ' ON ' + BY_SEQ_STORE + '.seq = ' + DOC_STORE + '.winningseq';
                tx.executeSql(sql, [], function (tx, result) {
                  var deleted = [];
                  var local = [];
                  for (var i = 0; i < result.rows.length; i++) {
                    var item = result.rows.item(i);
                    var seq = item.seq;
                    var metadata = JSON.parse(item.metadata);
                    if (utils.isDeleted(metadata)) {
                      deleted.push(seq);
                    }
                    if (utils.isLocalId(metadata.id)) {
                      local.push(metadata.id);
                    }
                  }
                  tx.executeSql('UPDATE ' + DOC_STORE + 'SET local = 1 WHERE id IN ' + qMarks(local.length), local, function () {
                    tx.executeSql('UPDATE ' + BY_SEQ_STORE + ' SET deleted = 1 WHERE seq IN ' + qMarks(deleted.length), deleted, callback);
                  });
                });
              });
            });
          }
          // in this migration, we make all the local docs unversioned
          function runMigration3(tx, callback) {
            var local = 'CREATE TABLE IF NOT EXISTS ' + LOCAL_STORE + ' (id UNIQUE, rev, json)';
            tx.executeSql(local, [], function () {
              var sql = 'SELECT ' + DOC_STORE + '.id AS id, ' + BY_SEQ_STORE + '.json AS data ' + 'FROM ' + BY_SEQ_STORE + ' JOIN ' + DOC_STORE + ' ON ' + BY_SEQ_STORE + '.seq = ' + DOC_STORE + '.winningseq WHERE local = 1';
              tx.executeSql(sql, [], function (tx, res) {
                var rows = [];
                for (var i = 0; i < res.rows.length; i++) {
                  rows.push(res.rows.item(i));
                }
                function doNext() {
                  if (!rows.length) {
                    return callback(tx);
                  }
                  var row = rows.shift();
                  var rev = JSON.parse(row.data)._rev;
                  tx.executeSql('INSERT INTO ' + LOCAL_STORE + ' (id, rev, json) VALUES (?,?,?)', [
                    row.id,
                    rev,
                    row.data
                  ], function (tx) {
                    tx.executeSql('DELETE FROM ' + DOC_STORE + ' WHERE id=?', [row.id], function (tx) {
                      tx.executeSql('DELETE FROM ' + BY_SEQ_STORE + ' WHERE seq=?', [row.seq], function () {
                        doNext();
                      });
                    });
                  });
                }
                doNext();
              });
            });
          }
          // in this migration, we remove doc_id_rev and just use rev
          function runMigration4(tx, callback) {
            function updateRows(rows) {
              function doNext() {
                if (!rows.length) {
                  return callback(tx);
                }
                var row = rows.shift();
                var doc_id_rev = parseHexString(row.hex, encoding);
                var idx = doc_id_rev.lastIndexOf('::');
                var doc_id = doc_id_rev.substring(0, idx);
                var rev = doc_id_rev.substring(idx + 2);
                var sql = 'UPDATE ' + BY_SEQ_STORE + ' SET doc_id=?, rev=? WHERE doc_id_rev=?';
                tx.executeSql(sql, [
                  doc_id,
                  rev,
                  doc_id_rev
                ], function () {
                  doNext();
                });
              }
              doNext();
            }
            var sql = 'ALTER TABLE ' + BY_SEQ_STORE + ' ADD COLUMN doc_id';
            tx.executeSql(sql, [], function (tx) {
              var sql = 'ALTER TABLE ' + BY_SEQ_STORE + ' ADD COLUMN rev';
              tx.executeSql(sql, [], function (tx) {
                tx.executeSql(BY_SEQ_STORE_DOC_ID_REV_INDEX_SQL, [], function (tx) {
                  var sql = 'SELECT hex(doc_id_rev) as hex FROM ' + BY_SEQ_STORE;
                  tx.executeSql(sql, [], function (tx, res) {
                    var rows = [];
                    for (var i = 0; i < res.rows.length; i++) {
                      rows.push(res.rows.item(i));
                    }
                    updateRows(rows);
                  });
                });
              });
            });
          }
          // in this migration, we add the attach_and_seq table
          // for issue #2818
          function runMigration5(tx, callback) {
            function migrateAttsAndSeqs(tx) {
              // need to actually populate the table. this is the expensive part,
              // so as an optimization, check first that this database even
              // contains attachments
              var sql = 'SELECT COUNT(*) AS cnt FROM ' + ATTACH_STORE;
              tx.executeSql(sql, [], function (tx, res) {
                var count = res.rows.item(0).cnt;
                if (!count) {
                  return callback(tx);
                }
                var offset = 0;
                var pageSize = 10;
                function nextPage() {
                  var sql = select(SELECT_DOCS + ', ' + DOC_STORE + '.id AS id', [
                      DOC_STORE,
                      BY_SEQ_STORE
                    ], DOC_STORE_AND_BY_SEQ_JOINER, null, DOC_STORE + '.id ');
                  sql += ' LIMIT ' + pageSize + ' OFFSET ' + offset;
                  offset += pageSize;
                  tx.executeSql(sql, [], function (tx, res) {
                    if (!res.rows.length) {
                      return callback(tx);
                    }
                    var digestSeqs = {};
                    function addDigestSeq(digest, seq) {
                      // uniq digest/seq pairs, just in case there are dups
                      var seqs = digestSeqs[digest] = digestSeqs[digest] || [];
                      if (seqs.indexOf(seq) === -1) {
                        seqs.push(seq);
                      }
                    }
                    for (var i = 0; i < res.rows.length; i++) {
                      var row = res.rows.item(i);
                      var doc = unstringifyDoc(row.data, row.id, row.rev);
                      var atts = Object.keys(doc._attachments || {});
                      for (var j = 0; j < atts.length; j++) {
                        var att = doc._attachments[atts[j]];
                        addDigestSeq(att.digest, row.seq);
                      }
                    }
                    var digestSeqPairs = [];
                    Object.keys(digestSeqs).forEach(function (digest) {
                      var seqs = digestSeqs[digest];
                      seqs.forEach(function (seq) {
                        digestSeqPairs.push([
                          digest,
                          seq
                        ]);
                      });
                    });
                    if (!digestSeqPairs.length) {
                      return nextPage();
                    }
                    var numDone = 0;
                    digestSeqPairs.forEach(function (pair) {
                      var sql = 'INSERT INTO ' + ATTACH_AND_SEQ_STORE + ' (digest, seq) VALUES (?,?)';
                      tx.executeSql(sql, pair, function () {
                        if (++numDone === digestSeqPairs.length) {
                          nextPage();
                        }
                      });
                    });
                  });
                }
                nextPage();
              });
            }
            var attachAndRev = 'CREATE TABLE IF NOT EXISTS ' + ATTACH_AND_SEQ_STORE + ' (digest, seq INTEGER)';
            tx.executeSql(attachAndRev, [], function (tx) {
              tx.executeSql(ATTACH_AND_SEQ_STORE_ATTACH_INDEX_SQL, [], function (tx) {
                tx.executeSql(ATTACH_AND_SEQ_STORE_SEQ_INDEX_SQL, [], migrateAttsAndSeqs);
              });
            });
          }
          // in this migration, we use escapeBlob() and unescapeBlob()
          // instead of reading out the binary as HEX, which is slow
          function runMigration6(tx, callback) {
            var sql = 'ALTER TABLE ' + ATTACH_STORE + ' ADD COLUMN escaped TINYINT(1) DEFAULT 0';
            tx.executeSql(sql, [], callback);
          }
          // issue #3136, in this migration we need a "latest seq" as well
          // as the "winning seq" in the doc store
          function runMigration7(tx, callback) {
            var sql = 'ALTER TABLE ' + DOC_STORE + ' ADD COLUMN max_seq INTEGER';
            tx.executeSql(sql, [], function (tx) {
              var sql = 'UPDATE ' + DOC_STORE + ' SET max_seq=(SELECT MAX(seq) FROM ' + BY_SEQ_STORE + ' WHERE doc_id=id)';
              tx.executeSql(sql, [], function (tx) {
                // add unique index after filling, else we'll get a constraint
                // error when we do the ALTER TABLE
                var sql = 'CREATE UNIQUE INDEX IF NOT EXISTS \'doc-max-seq-idx\' ON ' + DOC_STORE + ' (max_seq)';
                tx.executeSql(sql, [], callback);
              });
            });
          }
          function checkEncoding(tx, cb) {
            // UTF-8 on chrome/android, UTF-16 on safari < 7.1
            tx.executeSql('SELECT HEX("a") AS hex', [], function (tx, res) {
              var hex = res.rows.item(0).hex;
              encoding = hex.length === 2 ? 'UTF-8' : 'UTF-16';
              cb();
            });
          }
          function onGetInstanceId() {
            while (idRequests.length > 0) {
              var idCallback = idRequests.pop();
              idCallback(null, instanceId);
            }
          }
          function onGetVersion(tx, dbVersion) {
            if (dbVersion === 0) {
              // initial schema
              var meta = 'CREATE TABLE IF NOT EXISTS ' + META_STORE + ' (dbid, db_version INTEGER)';
              var attach = 'CREATE TABLE IF NOT EXISTS ' + ATTACH_STORE + ' (digest UNIQUE, escaped TINYINT(1), body BLOB)';
              var attachAndRev = 'CREATE TABLE IF NOT EXISTS ' + ATTACH_AND_SEQ_STORE + ' (digest, seq INTEGER)';
              // TODO: migrate winningseq to INTEGER
              var doc = 'CREATE TABLE IF NOT EXISTS ' + DOC_STORE + ' (id unique, json, winningseq, max_seq INTEGER UNIQUE)';
              var seq = 'CREATE TABLE IF NOT EXISTS ' + BY_SEQ_STORE + ' (seq INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT, ' + 'json, deleted TINYINT(1), doc_id, rev)';
              var local = 'CREATE TABLE IF NOT EXISTS ' + LOCAL_STORE + ' (id UNIQUE, rev, json)';
              // creates
              tx.executeSql(attach);
              tx.executeSql(local);
              tx.executeSql(attachAndRev, [], function () {
                tx.executeSql(ATTACH_AND_SEQ_STORE_SEQ_INDEX_SQL);
                tx.executeSql(ATTACH_AND_SEQ_STORE_ATTACH_INDEX_SQL);
              });
              tx.executeSql(doc, [], function () {
                tx.executeSql(DOC_STORE_WINNINGSEQ_INDEX_SQL);
                tx.executeSql(seq, [], function () {
                  tx.executeSql(BY_SEQ_STORE_DELETED_INDEX_SQL);
                  tx.executeSql(BY_SEQ_STORE_DOC_ID_REV_INDEX_SQL);
                  tx.executeSql(meta, [], function () {
                    // mark the db version, and new dbid
                    var initSeq = 'INSERT INTO ' + META_STORE + ' (db_version, dbid) VALUES (?,?)';
                    instanceId = utils.uuid();
                    var initSeqArgs = [
                        ADAPTER_VERSION,
                        instanceId
                      ];
                    tx.executeSql(initSeq, initSeqArgs, function () {
                      onGetInstanceId();
                    });
                  });
                });
              });
            } else {
              // version > 0
              var setupDone = function () {
                var migrated = dbVersion < ADAPTER_VERSION;
                if (migrated) {
                  // update the db version within this transaction
                  tx.executeSql('UPDATE ' + META_STORE + ' SET db_version = ' + ADAPTER_VERSION);
                }
                // notify db.id() callers
                var sql = 'SELECT dbid FROM ' + META_STORE;
                tx.executeSql(sql, [], function (tx, result) {
                  instanceId = result.rows.item(0).dbid;
                  onGetInstanceId();
                });
              };
              // would love to use promises here, but then websql
              // ends the transaction early
              var tasks = [
                  runMigration2,
                  runMigration3,
                  runMigration4,
                  runMigration5,
                  runMigration6,
                  runMigration7,
                  setupDone
                ];
              // run each migration sequentially
              var i = dbVersion;
              var nextMigration = function (tx) {
                tasks[i - 1](tx, nextMigration);
                i++;
              };
              nextMigration(tx);
            }
          }
          function setup() {
            db.transaction(function (tx) {
              // first check the encoding
              checkEncoding(tx, function () {
                // then get the version
                fetchVersion(tx);
              });
            }, unknownError(callback), dbCreated);
          }
          function fetchVersion(tx) {
            var sql = 'SELECT sql FROM sqlite_master WHERE tbl_name = ' + META_STORE;
            tx.executeSql(sql, [], function (tx, result) {
              if (!result.rows.length) {
                // database hasn't even been created yet (version 0)
                onGetVersion(tx, 0);
              } else if (!/db_version/.test(result.rows.item(0).sql)) {
                // table was created, but without the new db_version column,
                // so add it.
                tx.executeSql('ALTER TABLE ' + META_STORE + ' ADD COLUMN db_version INTEGER', [], function () {
                  // before version 2, this column didn't even exist
                  onGetVersion(tx, 1);
                });
              } else {
                // column exists, we can safely get it
                tx.executeSql('SELECT db_version FROM ' + META_STORE, [], function (tx, result) {
                  var dbVersion = result.rows.item(0).db_version;
                  onGetVersion(tx, dbVersion);
                });
              }
            });
          }
          if (utils.isCordova()) {
            //to wait until custom api is made in pouch.adapters before doing setup
            window.addEventListener(api._name + '_pouch', function cordova_init() {
              window.removeEventListener(api._name + '_pouch', cordova_init, false);
              setup();
            }, false);
          } else {
            setup();
          }
          api.type = function () {
            return 'websql';
          };
          api._id = utils.toPromise(function (callback) {
            callback(null, instanceId);
          });
          api._info = function (callback) {
            db.readTransaction(function (tx) {
              countDocs(tx, function (docCount) {
                var sql = 'SELECT MAX(seq) AS seq FROM ' + BY_SEQ_STORE;
                tx.executeSql(sql, [], function (tx, res) {
                  var updateSeq = res.rows.item(0).seq || 0;
                  callback(null, {
                    doc_count: docCount,
                    update_seq: updateSeq,
                    sqlite_plugin: db._sqlitePlugin,
                    websql_encoding: encoding
                  });
                });
              });
            }, unknownError(callback));
          };
          api._bulkDocs = function (req, opts, callback) {
            websqlBulkDocs(req, opts, api, db, WebSqlPouch.Changes, callback);
          };
          api._get = function (id, opts, callback) {
            opts = utils.clone(opts);
            var doc;
            var metadata;
            var err;
            if (!opts.ctx) {
              db.readTransaction(function (txn) {
                opts.ctx = txn;
                api._get(id, opts, callback);
              });
              return;
            }
            var tx = opts.ctx;
            function finish() {
              callback(err, {
                doc: doc,
                metadata: metadata,
                ctx: tx
              });
            }
            var sql;
            var sqlArgs;
            if (opts.rev) {
              sql = select(SELECT_DOCS, [
                DOC_STORE,
                BY_SEQ_STORE
              ], DOC_STORE + '.id=' + BY_SEQ_STORE + '.doc_id', [
                BY_SEQ_STORE + '.doc_id=?',
                BY_SEQ_STORE + '.rev=?'
              ]);
              sqlArgs = [
                id,
                opts.rev
              ];
            } else {
              sql = select(SELECT_DOCS, [
                DOC_STORE,
                BY_SEQ_STORE
              ], DOC_STORE_AND_BY_SEQ_JOINER, DOC_STORE + '.id=?');
              sqlArgs = [id];
            }
            tx.executeSql(sql, sqlArgs, function (a, results) {
              if (!results.rows.length) {
                err = errors.error(errors.MISSING_DOC, 'missing');
                return finish();
              }
              var item = results.rows.item(0);
              metadata = utils.safeJsonParse(item.metadata);
              if (item.deleted && !opts.rev) {
                err = errors.error(errors.MISSING_DOC, 'deleted');
                return finish();
              }
              doc = unstringifyDoc(item.data, metadata.id, item.rev);
              finish();
            });
          };
          function countDocs(tx, callback) {
            if (api._docCount !== -1) {
              return callback(api._docCount);
            }
            // count the total rows
            var sql = select('COUNT(' + DOC_STORE + '.id) AS \'num\'', [
                DOC_STORE,
                BY_SEQ_STORE
              ], DOC_STORE_AND_BY_SEQ_JOINER, BY_SEQ_STORE + '.deleted=0');
            tx.executeSql(sql, [], function (tx, result) {
              api._docCount = result.rows.item(0).num;
              callback(api._docCount);
            });
          }
          api._allDocs = function (opts, callback) {
            var results = [];
            var totalRows;
            var start = 'startkey' in opts ? opts.startkey : false;
            var end = 'endkey' in opts ? opts.endkey : false;
            var key = 'key' in opts ? opts.key : false;
            var descending = 'descending' in opts ? opts.descending : false;
            var limit = 'limit' in opts ? opts.limit : -1;
            var offset = 'skip' in opts ? opts.skip : 0;
            var inclusiveEnd = opts.inclusive_end !== false;
            var sqlArgs = [];
            var criteria = [];
            if (key !== false) {
              criteria.push(DOC_STORE + '.id = ?');
              sqlArgs.push(key);
            } else if (start !== false || end !== false) {
              if (start !== false) {
                criteria.push(DOC_STORE + '.id ' + (descending ? '<=' : '>=') + ' ?');
                sqlArgs.push(start);
              }
              if (end !== false) {
                var comparator = descending ? '>' : '<';
                if (inclusiveEnd) {
                  comparator += '=';
                }
                criteria.push(DOC_STORE + '.id ' + comparator + ' ?');
                sqlArgs.push(end);
              }
              if (key !== false) {
                criteria.push(DOC_STORE + '.id = ?');
                sqlArgs.push(key);
              }
            }
            if (opts.deleted !== 'ok') {
              // report deleted if keys are specified
              criteria.push(BY_SEQ_STORE + '.deleted = 0');
            }
            db.readTransaction(function (tx) {
              // first count up the total rows
              countDocs(tx, function (count) {
                totalRows = count;
                if (limit === 0) {
                  return;
                }
                // then actually fetch the documents
                var sql = select(SELECT_DOCS, [
                    DOC_STORE,
                    BY_SEQ_STORE
                  ], DOC_STORE_AND_BY_SEQ_JOINER, criteria, DOC_STORE + '.id ' + (descending ? 'DESC' : 'ASC'));
                sql += ' LIMIT ' + limit + ' OFFSET ' + offset;
                tx.executeSql(sql, sqlArgs, function (tx, result) {
                  for (var i = 0, l = result.rows.length; i < l; i++) {
                    var item = result.rows.item(i);
                    var metadata = utils.safeJsonParse(item.metadata);
                    var id = metadata.id;
                    var data = unstringifyDoc(item.data, id, item.rev);
                    var winningRev = data._rev;
                    var doc = {
                        id: id,
                        key: id,
                        value: { rev: winningRev }
                      };
                    if (opts.include_docs) {
                      doc.doc = data;
                      doc.doc._rev = winningRev;
                      if (opts.conflicts) {
                        doc.doc._conflicts = merge.collectConflicts(metadata);
                      }
                      fetchAttachmentsIfNecessary(doc.doc, opts, api, tx);
                    }
                    if (item.deleted) {
                      if (opts.deleted === 'ok') {
                        doc.value.deleted = true;
                        doc.doc = null;
                      } else {
                        continue;
                      }
                    }
                    results.push(doc);
                  }
                });
              });
            }, unknownError(callback), function () {
              callback(null, {
                total_rows: totalRows,
                offset: opts.skip,
                rows: results
              });
            });
          };
          api._changes = function (opts) {
            opts = utils.clone(opts);
            if (opts.continuous) {
              var id = api._name + ':' + utils.uuid();
              WebSqlPouch.Changes.addListener(api._name, id, api, opts);
              WebSqlPouch.Changes.notify(api._name);
              return {
                cancel: function () {
                  WebSqlPouch.Changes.removeListener(api._name, id);
                }
              };
            }
            var descending = opts.descending;
            // Ignore the `since` parameter when `descending` is true
            opts.since = opts.since && !descending ? opts.since : 0;
            var limit = 'limit' in opts ? opts.limit : -1;
            if (limit === 0) {
              limit = 1;  // per CouchDB _changes spec
            }
            var returnDocs;
            if ('returnDocs' in opts) {
              returnDocs = opts.returnDocs;
            } else {
              returnDocs = true;
            }
            var results = [];
            var numResults = 0;
            function fetchChanges() {
              var selectStmt = DOC_STORE + '.json AS metadata, ' + DOC_STORE + '.max_seq AS maxSeq, ' + BY_SEQ_STORE + '.json AS winningDoc, ' + BY_SEQ_STORE + '.rev AS winningRev ';
              var from = DOC_STORE + ' JOIN ' + BY_SEQ_STORE;
              var joiner = DOC_STORE + '.id=' + BY_SEQ_STORE + '.doc_id' + ' AND ' + DOC_STORE + '.winningseq=' + BY_SEQ_STORE + '.seq';
              var criteria = ['maxSeq > ?'];
              var sqlArgs = [opts.since];
              if (opts.doc_ids) {
                criteria.push(DOC_STORE + '.id IN ' + qMarks(opts.doc_ids.length));
                sqlArgs = sqlArgs.concat(opts.doc_ids);
              }
              var orderBy = 'maxSeq ' + (descending ? 'DESC' : 'ASC');
              var sql = select(selectStmt, from, joiner, criteria, orderBy);
              var filter = utils.filterChange(opts);
              if (!opts.view && !opts.filter) {
                // we can just limit in the query
                sql += ' LIMIT ' + limit;
              }
              var lastSeq = opts.since || 0;
              db.readTransaction(function (tx) {
                tx.executeSql(sql, sqlArgs, function (tx, result) {
                  function reportChange(change) {
                    return function () {
                      opts.onChange(change);
                    };
                  }
                  for (var i = 0, l = result.rows.length; i < l; i++) {
                    var item = result.rows.item(i);
                    var metadata = utils.safeJsonParse(item.metadata);
                    lastSeq = item.maxSeq;
                    var doc = unstringifyDoc(item.winningDoc, metadata.id, item.winningRev);
                    var change = opts.processChange(doc, metadata, opts);
                    change.seq = item.maxSeq;
                    if (filter(change)) {
                      numResults++;
                      if (returnDocs) {
                        results.push(change);
                      }
                      // process the attachment immediately
                      // for the benefit of live listeners
                      if (opts.attachments && opts.include_docs) {
                        fetchAttachmentsIfNecessary(doc, opts, api, tx, reportChange(change));
                      } else {
                        reportChange(change)();
                      }
                    }
                    if (numResults === limit) {
                      break;
                    }
                  }
                });
              }, unknownError(opts.complete), function () {
                if (!opts.continuous) {
                  opts.complete(null, {
                    results: results,
                    last_seq: lastSeq
                  });
                }
              });
            }
            fetchChanges();
          };
          api._close = function (callback) {
            //WebSQL databases do not need to be closed
            callback();
          };
          api._getAttachment = function (attachment, opts, callback) {
            var res;
            var tx = opts.ctx;
            var digest = attachment.digest;
            var type = attachment.content_type;
            var sql = 'SELECT escaped, ' + 'CASE WHEN escaped = 1 THEN body ELSE HEX(body) END AS body FROM ' + ATTACH_STORE + ' WHERE digest=?';
            tx.executeSql(sql, [digest], function (tx, result) {
              // websql has a bug where \u0000 causes early truncation in strings
              // and blobs. to work around this, we used to use the hex() function,
              // but that's not performant. after migration 6, we remove \u0000
              // and add it back in afterwards
              var item = result.rows.item(0);
              var data = item.escaped ? websqlUtils.unescapeBlob(item.body) : parseHexString(item.body, encoding);
              if (opts.encode) {
                res = btoa(data);
              } else {
                data = utils.fixBinary(data);
                res = utils.createBlob([data], { type: type });
              }
              callback(null, res);
            });
          };
          api._getRevisionTree = function (docId, callback) {
            db.readTransaction(function (tx) {
              var sql = 'SELECT json AS metadata FROM ' + DOC_STORE + ' WHERE id = ?';
              tx.executeSql(sql, [docId], function (tx, result) {
                if (!result.rows.length) {
                  callback(errors.error(errors.MISSING_DOC));
                } else {
                  var data = utils.safeJsonParse(result.rows.item(0).metadata);
                  callback(null, data.rev_tree);
                }
              });
            });
          };
          api._doCompaction = function (docId, revs, callback) {
            if (!revs.length) {
              return callback();
            }
            db.transaction(function (tx) {
              // update doc store
              var sql = 'SELECT json AS metadata FROM ' + DOC_STORE + ' WHERE id = ?';
              tx.executeSql(sql, [docId], function (tx, result) {
                var metadata = utils.safeJsonParse(result.rows.item(0).metadata);
                merge.traverseRevTree(metadata.rev_tree, function (isLeaf, pos, revHash, ctx, opts) {
                  var rev = pos + '-' + revHash;
                  if (revs.indexOf(rev) !== -1) {
                    opts.status = 'missing';
                  }
                });
                var sql = 'UPDATE ' + DOC_STORE + ' SET json = ? WHERE id = ?';
                tx.executeSql(sql, [
                  utils.safeJsonStringify(metadata),
                  docId
                ]);
              });
              compactRevs(revs, docId, tx);
            }, unknownError(callback), function () {
              callback();
            });
          };
          api._getLocal = function (id, callback) {
            db.readTransaction(function (tx) {
              var sql = 'SELECT json, rev FROM ' + LOCAL_STORE + ' WHERE id=?';
              tx.executeSql(sql, [id], function (tx, res) {
                if (res.rows.length) {
                  var item = res.rows.item(0);
                  var doc = unstringifyDoc(item.json, id, item.rev);
                  callback(null, doc);
                } else {
                  callback(errors.error(errors.MISSING_DOC));
                }
              });
            });
          };
          api._putLocal = function (doc, opts, callback) {
            if (typeof opts === 'function') {
              callback = opts;
              opts = {};
            }
            delete doc._revisions;
            // ignore this, trust the rev
            var oldRev = doc._rev;
            var id = doc._id;
            var newRev;
            if (!oldRev) {
              newRev = doc._rev = '0-1';
            } else {
              newRev = doc._rev = '0-' + (parseInt(oldRev.split('-')[1], 10) + 1);
            }
            var json = stringifyDoc(doc);
            var ret;
            function putLocal(tx) {
              var sql;
              var values;
              if (oldRev) {
                sql = 'UPDATE ' + LOCAL_STORE + ' SET rev=?, json=? ' + 'WHERE id=? AND rev=?';
                values = [
                  newRev,
                  json,
                  id,
                  oldRev
                ];
              } else {
                sql = 'INSERT INTO ' + LOCAL_STORE + ' (id, rev, json) VALUES (?,?,?)';
                values = [
                  id,
                  newRev,
                  json
                ];
              }
              tx.executeSql(sql, values, function (tx, res) {
                if (res.rowsAffected) {
                  ret = {
                    ok: true,
                    id: id,
                    rev: newRev
                  };
                  if (opts.ctx) {
                    // return immediately
                    callback(null, ret);
                  }
                } else {
                  callback(errors.error(errors.REV_CONFLICT));
                }
              }, function () {
                callback(errors.error(errors.REV_CONFLICT));
                return false;  // ack that we handled the error
              });
            }
            if (opts.ctx) {
              putLocal(opts.ctx);
            } else {
              db.transaction(function (tx) {
                putLocal(tx);
              }, unknownError(callback), function () {
                if (ret) {
                  callback(null, ret);
                }
              });
            }
          };
          api._removeLocal = function (doc, callback) {
            var ret;
            db.transaction(function (tx) {
              var sql = 'DELETE FROM ' + LOCAL_STORE + ' WHERE id=? AND rev=?';
              var params = [
                  doc._id,
                  doc._rev
                ];
              tx.executeSql(sql, params, function (tx, res) {
                if (!res.rowsAffected) {
                  return callback(errors.error(errors.MISSING_DOC));
                }
                ret = {
                  ok: true,
                  id: doc._id,
                  rev: '0-0'
                };
              });
            }, unknownError(callback), function () {
              if (ret) {
                callback(null, ret);
              }
            });
          };
        }
        WebSqlPouch.valid = websqlUtils.valid;
        WebSqlPouch.destroy = utils.toPromise(function (name, opts, callback) {
          WebSqlPouch.Changes.removeAllListeners(name);
          var size = getSize(opts);
          var db = openDB({
              name: name,
              version: POUCH_VERSION,
              description: name,
              size: size,
              location: opts.location,
              createFromLocation: opts.createFromLocation
            });
          db.transaction(function (tx) {
            var stores = [
                DOC_STORE,
                BY_SEQ_STORE,
                ATTACH_STORE,
                META_STORE,
                LOCAL_STORE,
                ATTACH_AND_SEQ_STORE
              ];
            stores.forEach(function (store) {
              tx.executeSql('DROP TABLE IF EXISTS ' + store, []);
            });
          }, unknownError(callback), function () {
            if (utils.hasLocalStorage()) {
              delete window.localStorage['_pouch__websqldb_' + name];
              delete window.localStorage[name];
            }
            callback(null, { 'ok': true });
          });
        });
        WebSqlPouch.Changes = new utils.Changes();
        module.exports = WebSqlPouch;
      },
      {
        '10': 10,
        '11': 11,
        '12': 12,
        '20': 20,
        '23': 23,
        '30': 30,
        '35': 35
      }
    ],
    14: [
      function (_dereq_, module, exports) {
        'use strict';
        var utils = _dereq_(35);
        var merge = _dereq_(30);
        var errors = _dereq_(20);
        var EE = _dereq_(39).EventEmitter;
        var evalFilter = _dereq_(28);
        var evalView = _dereq_(29);
        module.exports = Changes;
        utils.inherits(Changes, EE);
        function Changes(db, opts, callback) {
          EE.call(this);
          var self = this;
          this.db = db;
          opts = opts ? utils.clone(opts) : {};
          var oldComplete = callback || opts.complete || function () {
            };
          var complete = opts.complete = utils.once(function (err, resp) {
              if (err) {
                self.emit('error', err);
              } else {
                self.emit('complete', resp);
              }
              self.removeAllListeners();
              db.removeListener('destroyed', onDestroy);
            });
          if (oldComplete) {
            self.on('complete', function (resp) {
              oldComplete(null, resp);
            });
            self.on('error', function (err) {
              oldComplete(err);
            });
          }
          var oldOnChange = opts.onChange;
          if (oldOnChange) {
            self.on('change', oldOnChange);
          }
          function onDestroy() {
            self.cancel();
          }
          db.once('destroyed', onDestroy);
          opts.onChange = function (change) {
            if (opts.isCancelled) {
              return;
            }
            self.emit('change', change);
            if (self.startSeq && self.startSeq <= change.seq) {
              self.emit('uptodate');
              self.startSeq = false;
            }
            if (change.deleted) {
              self.emit('delete', change);
            } else if (change.changes.length === 1 && change.changes[0].rev.slice(0, 2) === '1-') {
              self.emit('create', change);
            } else {
              self.emit('update', change);
            }
          };
          var promise = new utils.Promise(function (fulfill, reject) {
              opts.complete = function (err, res) {
                if (err) {
                  reject(err);
                } else {
                  fulfill(res);
                }
              };
            });
          self.once('cancel', function () {
            if (oldOnChange) {
              self.removeListener('change', oldOnChange);
            }
            opts.complete(null, { status: 'cancelled' });
          });
          this.then = promise.then.bind(promise);
          this['catch'] = promise['catch'].bind(promise);
          this.then(function (result) {
            complete(null, result);
          }, complete);
          if (!db.taskqueue.isReady) {
            db.taskqueue.addTask(function () {
              if (self.isCancelled) {
                self.emit('cancel');
              } else {
                self.doChanges(opts);
              }
            });
          } else {
            self.doChanges(opts);
          }
        }
        Changes.prototype.cancel = function () {
          this.isCancelled = true;
          if (this.db.taskqueue.isReady) {
            this.emit('cancel');
          }
        };
        function processChange(doc, metadata, opts) {
          var changeList = [{ rev: doc._rev }];
          if (opts.style === 'all_docs') {
            changeList = merge.collectLeaves(metadata.rev_tree).map(function (x) {
              return { rev: x.rev };
            });
          }
          var change = {
              id: metadata.id,
              changes: changeList,
              doc: doc
            };
          if (utils.isDeleted(metadata, doc._rev)) {
            change.deleted = true;
          }
          if (opts.conflicts) {
            change.doc._conflicts = merge.collectConflicts(metadata);
            if (!change.doc._conflicts.length) {
              delete change.doc._conflicts;
            }
          }
          return change;
        }
        Changes.prototype.doChanges = function (opts) {
          var self = this;
          var callback = opts.complete;
          opts = utils.clone(opts);
          if ('live' in opts && !('continuous' in opts)) {
            opts.continuous = opts.live;
          }
          opts.processChange = processChange;
          if (opts.since === 'latest') {
            opts.since = 'now';
          }
          if (!opts.since) {
            opts.since = 0;
          }
          if (opts.since === 'now') {
            this.db.info().then(function (info) {
              if (self.isCancelled) {
                callback(null, { status: 'cancelled' });
                return;
              }
              opts.since = info.update_seq;
              self.doChanges(opts);
            }, callback);
            return;
          }
          if (opts.continuous && opts.since !== 'now') {
            this.db.info().then(function (info) {
              self.startSeq = info.update_seq;
            }, function (err) {
              if (err.id === 'idbNull') {
                //db closed before this returned
                //thats ok
                return;
              }
              throw err;
            });
          }
          if (this.db.type() !== 'http' && opts.filter && typeof opts.filter === 'string' && !opts.doc_ids) {
            return this.filterChanges(opts);
          }
          if (!('descending' in opts)) {
            opts.descending = false;
          }
          // 0 and 1 should return 1 document
          opts.limit = opts.limit === 0 ? 1 : opts.limit;
          opts.complete = callback;
          var newPromise = this.db._changes(opts);
          if (newPromise && typeof newPromise.cancel === 'function') {
            var cancel = self.cancel;
            self.cancel = utils.getArguments(function (args) {
              newPromise.cancel();
              cancel.apply(this, args);
            });
          }
        };
        Changes.prototype.filterChanges = function (opts) {
          var self = this;
          var callback = opts.complete;
          if (opts.filter === '_view') {
            if (!opts.view || typeof opts.view !== 'string') {
              var err = errors.error(errors.BAD_REQUEST, '`view` filter parameter is not provided.');
              callback(err);
              return;
            }
            // fetch a view from a design doc, make it behave like a filter
            var viewName = opts.view.split('/');
            this.db.get('_design/' + viewName[0], function (err, ddoc) {
              if (self.isCancelled) {
                callback(null, { status: 'cancelled' });
                return;
              }
              if (err) {
                callback(errors.generateErrorFromResponse(err));
                return;
              }
              if (ddoc && ddoc.views && ddoc.views[viewName[1]]) {
                var filter = evalView(ddoc.views[viewName[1]].map);
                opts.filter = filter;
                self.doChanges(opts);
                return;
              }
              var msg = ddoc.views ? 'missing json key: ' + viewName[1] : 'missing json key: views';
              if (!err) {
                err = errors.error(errors.MISSING_DOC, msg);
              }
              callback(err);
              return;
            });
          } else {
            // fetch a filter from a design doc
            var filterName = opts.filter.split('/');
            this.db.get('_design/' + filterName[0], function (err, ddoc) {
              if (self.isCancelled) {
                callback(null, { status: 'cancelled' });
                return;
              }
              if (err) {
                callback(errors.generateErrorFromResponse(err));
                return;
              }
              if (ddoc && ddoc.filters && ddoc.filters[filterName[1]]) {
                var filter = evalFilter(ddoc.filters[filterName[1]]);
                opts.filter = filter;
                self.doChanges(opts);
                return;
              } else {
                var msg = ddoc && ddoc.filters ? 'missing json key: ' + filterName[1] : 'missing json key: filters';
                if (!err) {
                  err = errors.error(errors.MISSING_DOC, msg);
                }
                callback(err);
                return;
              }
            });
          }
        };
      },
      {
        '20': 20,
        '28': 28,
        '29': 29,
        '30': 30,
        '35': 35,
        '39': 39
      }
    ],
    15: [
      function (_dereq_, module, exports) {
        'use strict';
        var utils = _dereq_(35);
        var pouchCollate = _dereq_(63);
        var collate = pouchCollate.collate;
        function updateCheckpoint(db, id, checkpoint, returnValue) {
          return db.get(id)['catch'](function (err) {
            if (err.status === 404) {
              if (db.type() === 'http') {
                utils.explain404('PouchDB is just checking if a remote checkpoint exists.');
              }
              return { _id: id };
            }
            throw err;
          }).then(function (doc) {
            if (returnValue.cancelled) {
              return;
            }
            doc.last_seq = checkpoint;
            return db.put(doc)['catch'](function (err) {
              if (err.status === 409) {
                // retry; someone is trying to write a checkpoint simultaneously
                return updateCheckpoint(db, id, checkpoint, returnValue);
              }
              throw err;
            });
          });
        }
        function Checkpointer(src, target, id, returnValue) {
          this.src = src;
          this.target = target;
          this.id = id;
          this.returnValue = returnValue;
        }
        Checkpointer.prototype.writeCheckpoint = function (checkpoint) {
          var self = this;
          return this.updateTarget(checkpoint).then(function () {
            return self.updateSource(checkpoint);
          });
        };
        Checkpointer.prototype.updateTarget = function (checkpoint) {
          return updateCheckpoint(this.target, this.id, checkpoint, this.returnValue);
        };
        Checkpointer.prototype.updateSource = function (checkpoint) {
          var self = this;
          if (this.readOnlySource) {
            return utils.Promise.resolve(true);
          }
          return updateCheckpoint(this.src, this.id, checkpoint, this.returnValue)['catch'](function (err) {
            var isForbidden = typeof err.status === 'number' && Math.floor(err.status / 100) === 4;
            if (isForbidden) {
              self.readOnlySource = true;
              return true;
            }
            throw err;
          });
        };
        Checkpointer.prototype.getCheckpoint = function () {
          var self = this;
          return self.target.get(self.id).then(function (targetDoc) {
            return self.src.get(self.id).then(function (sourceDoc) {
              if (collate(targetDoc.last_seq, sourceDoc.last_seq) === 0) {
                return sourceDoc.last_seq;
              }
              return 0;
            }, function (err) {
              if (err.status === 404 && targetDoc.last_seq) {
                return self.src.put({
                  _id: self.id,
                  last_seq: 0
                }).then(function () {
                  return 0;
                }, function (err) {
                  if (err.status === 401) {
                    self.readOnlySource = true;
                    return targetDoc.last_seq;
                  }
                  return 0;
                });
              }
              throw err;
            });
          })['catch'](function (err) {
            if (err.status !== 404) {
              throw err;
            }
            return 0;
          });
        };
        module.exports = Checkpointer;
      },
      {
        '35': 35,
        '63': 63
      }
    ],
    16: [
      function (_dereq_, module, exports) {
        (function (process, global) {
          /*globals cordova */
          'use strict';
          var Adapter = _dereq_(1);
          var utils = _dereq_(35);
          var TaskQueue = _dereq_(34);
          var Promise = utils.Promise;
          function defaultCallback(err) {
            if (err && global.debug) {
              console.error(err);
            }
          }
          utils.inherits(PouchDB, Adapter);
          function PouchDB(name, opts, callback) {
            if (!(this instanceof PouchDB)) {
              return new PouchDB(name, opts, callback);
            }
            var self = this;
            if (typeof opts === 'function' || typeof opts === 'undefined') {
              callback = opts;
              opts = {};
            }
            if (name && typeof name === 'object') {
              opts = name;
              name = undefined;
            }
            if (typeof callback === 'undefined') {
              callback = defaultCallback;
            }
            opts = opts || {};
            this.__opts = opts;
            var oldCB = callback;
            self.auto_compaction = opts.auto_compaction;
            self.prefix = PouchDB.prefix;
            Adapter.call(self);
            self.taskqueue = new TaskQueue();
            var promise = new Promise(function (fulfill, reject) {
                callback = function (err, resp) {
                  if (err) {
                    return reject(err);
                  }
                  delete resp.then;
                  fulfill(resp);
                };
                opts = utils.clone(opts);
                var originalName = opts.name || name;
                var backend, error;
                (function () {
                  try {
                    if (typeof originalName !== 'string') {
                      error = new Error('Missing/invalid DB name');
                      error.code = 400;
                      throw error;
                    }
                    backend = PouchDB.parseAdapter(originalName, opts);
                    opts.originalName = originalName;
                    opts.name = backend.name;
                    if (opts.prefix && backend.adapter !== 'http' && backend.adapter !== 'https') {
                      opts.name = opts.prefix + opts.name;
                    }
                    opts.adapter = opts.adapter || backend.adapter;
                    self._adapter = opts.adapter;
                    self._db_name = originalName;
                    if (!PouchDB.adapters[opts.adapter]) {
                      error = new Error('Adapter is missing');
                      error.code = 404;
                      throw error;
                    }
                    if (!PouchDB.adapters[opts.adapter].valid()) {
                      error = new Error('Invalid Adapter');
                      error.code = 404;
                      throw error;
                    }
                  } catch (err) {
                    self.taskqueue.fail(err);
                    self.changes = utils.toPromise(function (opts) {
                      if (opts.complete) {
                        opts.complete(err);
                      }
                    });
                  }
                }());
                if (error) {
                  return reject(error);  // constructor error, see above
                }
                self.adapter = opts.adapter;
                // needs access to PouchDB;
                self.replicate = {};
                self.replicate.from = function (url, opts, callback) {
                  return self.constructor.replicate(url, self, opts, callback);
                };
                self.replicate.to = function (url, opts, callback) {
                  return self.constructor.replicate(self, url, opts, callback);
                };
                self.sync = function (dbName, opts, callback) {
                  return self.constructor.sync(self, dbName, opts, callback);
                };
                self.replicate.sync = self.sync;
                self.destroy = utils.adapterFun('destroy', function (callback) {
                  var self = this;
                  var opts = this.__opts || {};
                  self.info(function (err, info) {
                    if (err) {
                      return callback(err);
                    }
                    opts.internal = true;
                    self.constructor.destroy(info.db_name, opts, callback);
                  });
                });
                PouchDB.adapters[opts.adapter].call(self, opts, function (err) {
                  if (err) {
                    if (callback) {
                      self.taskqueue.fail(err);
                      callback(err);
                    }
                    return;
                  }
                  function destructionListener(event) {
                    if (event === 'destroyed') {
                      self.emit('destroyed');
                      PouchDB.removeListener(originalName, destructionListener);
                    }
                  }
                  PouchDB.on(originalName, destructionListener);
                  self.emit('created', self);
                  PouchDB.emit('created', opts.originalName);
                  self.taskqueue.ready(self);
                  callback(null, self);
                });
                if (opts.skipSetup) {
                  self.taskqueue.ready(self);
                  process.nextTick(function () {
                    callback(null, self);
                  });
                }
                if (utils.isCordova()) {
                  //to inform websql adapter that we can use api
                  cordova.fireWindowEvent(opts.name + '_pouch', {});
                }
              });
            promise.then(function (resp) {
              oldCB(null, resp);
            }, oldCB);
            self.then = promise.then.bind(promise);
            self['catch'] = promise['catch'].bind(promise);
          }
          PouchDB.debug = _dereq_(41);
          module.exports = PouchDB;
        }.call(this, _dereq_(40), typeof global !== 'undefined' ? global : typeof self !== 'undefined' ? self : typeof window !== 'undefined' ? window : {}));
      },
      {
        '1': 1,
        '34': 34,
        '35': 35,
        '40': 40,
        '41': 41
      }
    ],
    17: [
      function (_dereq_, module, exports) {
        (function (process) {
          'use strict';
          var request = _dereq_(25);
          var buffer = _dereq_(19);
          var errors = _dereq_(20);
          var utils = _dereq_(35);
          function ajax(options, adapterCallback) {
            var requestCompleted = false;
            var callback = utils.getArguments(function (args) {
                if (requestCompleted) {
                  return;
                }
                adapterCallback.apply(this, args);
                requestCompleted = true;
              });
            if (typeof options === 'function') {
              callback = options;
              options = {};
            }
            options = utils.clone(options);
            var defaultOptions = {
                method: 'GET',
                headers: {},
                json: true,
                processData: true,
                timeout: 10000,
                cache: false
              };
            options = utils.extend(true, defaultOptions, options);
            function onSuccess(obj, resp, cb) {
              if (!options.binary && !options.json && options.processData && typeof obj !== 'string') {
                obj = JSON.stringify(obj);
              } else if (!options.binary && options.json && typeof obj === 'string') {
                try {
                  obj = JSON.parse(obj);
                } catch (e) {
                  // Probably a malformed JSON from server
                  return cb(e);
                }
              }
              if (Array.isArray(obj)) {
                obj = obj.map(function (v) {
                  if (v.error || v.missing) {
                    return errors.generateErrorFromResponse(v);
                  } else {
                    return v;
                  }
                });
              }
              cb(null, obj, resp);
            }
            function onError(err, cb) {
              var errParsed, errObj;
              if (err.code && err.status) {
                var err2 = new Error(err.message || err.code);
                err2.status = err.status;
                return cb(err2);
              }
              try {
                errParsed = JSON.parse(err.responseText);
                //would prefer not to have a try/catch clause
                errObj = errors.generateErrorFromResponse(errParsed);
              } catch (e) {
                errObj = errors.generateErrorFromResponse(err);
              }
              cb(errObj);
            }
            if (options.json) {
              if (!options.binary) {
                options.headers.Accept = 'application/json';
              }
              options.headers['Content-Type'] = options.headers['Content-Type'] || 'application/json';
            }
            if (options.binary) {
              options.encoding = null;
              options.json = false;
            }
            if (!options.processData) {
              options.json = false;
            }
            function defaultBody(data) {
              if (process.browser) {
                return '';
              }
              return new buffer('', 'binary');
            }
            return request(options, function (err, response, body) {
              if (err) {
                err.status = response ? response.statusCode : 400;
                return onError(err, callback);
              }
              var error;
              var content_type = response.headers && response.headers['content-type'];
              var data = body || defaultBody();
              // CouchDB doesn't always return the right content-type for JSON data, so
              // we check for ^{ and }$ (ignoring leading/trailing whitespace)
              if (!options.binary && (options.json || !options.processData) && typeof data !== 'object' && (/json/.test(content_type) || /^[\s]*\{/.test(data) && /\}[\s]*$/.test(data))) {
                data = JSON.parse(data);
              }
              if (response.statusCode >= 200 && response.statusCode < 300) {
                onSuccess(data, response, callback);
              } else {
                if (options.binary) {
                  data = JSON.parse(data.toString());
                }
                error = errors.generateErrorFromResponse(data);
                error.status = response.statusCode;
                callback(error);
              }
            });
          }
          module.exports = ajax;
        }.call(this, _dereq_(40)));
      },
      {
        '19': 19,
        '20': 20,
        '25': 25,
        '35': 35,
        '40': 40
      }
    ],
    18: [
      function (_dereq_, module, exports) {
        (function (global) {
          'use strict';
          //Abstracts constructing a Blob object, so it also works in older
          //browsers that don't support the native Blob constructor. (i.e.
          //old QtWebKit versions, at least).
          function createBlob(parts, properties) {
            parts = parts || [];
            properties = properties || {};
            try {
              return new Blob(parts, properties);
            } catch (e) {
              if (e.name !== 'TypeError') {
                throw e;
              }
              var BlobBuilder = global.BlobBuilder || global.MSBlobBuilder || global.MozBlobBuilder || global.WebKitBlobBuilder;
              var builder = new BlobBuilder();
              for (var i = 0; i < parts.length; i += 1) {
                builder.append(parts[i]);
              }
              return builder.getBlob(properties.type);
            }
          }
          module.exports = createBlob;
        }.call(this, typeof global !== 'undefined' ? global : typeof self !== 'undefined' ? self : typeof window !== 'undefined' ? window : {}));
      },
      {}
    ],
    19: [
      function (_dereq_, module, exports) {
        // hey guess what, we don't need this in the browser
        module.exports = {};
      },
      {}
    ],
    20: [
      function (_dereq_, module, exports) {
        'use strict';
        var inherits = _dereq_(44);
        inherits(PouchError, Error);
        function PouchError(opts) {
          Error.call(opts.reason);
          this.status = opts.status;
          this.name = opts.error;
          this.message = opts.reason;
          this.error = true;
        }
        PouchError.prototype.toString = function () {
          return JSON.stringify({
            status: this.status,
            name: this.name,
            message: this.message
          });
        };
        exports.UNAUTHORIZED = new PouchError({
          status: 401,
          error: 'unauthorized',
          reason: 'Name or password is incorrect.'
        });
        exports.MISSING_BULK_DOCS = new PouchError({
          status: 400,
          error: 'bad_request',
          reason: 'Missing JSON list of \'docs\''
        });
        exports.MISSING_DOC = new PouchError({
          status: 404,
          error: 'not_found',
          reason: 'missing'
        });
        exports.REV_CONFLICT = new PouchError({
          status: 409,
          error: 'conflict',
          reason: 'Document update conflict'
        });
        exports.INVALID_ID = new PouchError({
          status: 400,
          error: 'invalid_id',
          reason: '_id field must contain a string'
        });
        exports.MISSING_ID = new PouchError({
          status: 412,
          error: 'missing_id',
          reason: '_id is required for puts'
        });
        exports.RESERVED_ID = new PouchError({
          status: 400,
          error: 'bad_request',
          reason: 'Only reserved document ids may start with underscore.'
        });
        exports.NOT_OPEN = new PouchError({
          status: 412,
          error: 'precondition_failed',
          reason: 'Database not open'
        });
        exports.UNKNOWN_ERROR = new PouchError({
          status: 500,
          error: 'unknown_error',
          reason: 'Database encountered an unknown error'
        });
        exports.BAD_ARG = new PouchError({
          status: 500,
          error: 'badarg',
          reason: 'Some query argument is invalid'
        });
        exports.INVALID_REQUEST = new PouchError({
          status: 400,
          error: 'invalid_request',
          reason: 'Request was invalid'
        });
        exports.QUERY_PARSE_ERROR = new PouchError({
          status: 400,
          error: 'query_parse_error',
          reason: 'Some query parameter is invalid'
        });
        exports.DOC_VALIDATION = new PouchError({
          status: 500,
          error: 'doc_validation',
          reason: 'Bad special document member'
        });
        exports.BAD_REQUEST = new PouchError({
          status: 400,
          error: 'bad_request',
          reason: 'Something wrong with the request'
        });
        exports.NOT_AN_OBJECT = new PouchError({
          status: 400,
          error: 'bad_request',
          reason: 'Document must be a JSON object'
        });
        exports.DB_MISSING = new PouchError({
          status: 404,
          error: 'not_found',
          reason: 'Database not found'
        });
        exports.IDB_ERROR = new PouchError({
          status: 500,
          error: 'indexed_db_went_bad',
          reason: 'unknown'
        });
        exports.WSQ_ERROR = new PouchError({
          status: 500,
          error: 'web_sql_went_bad',
          reason: 'unknown'
        });
        exports.LDB_ERROR = new PouchError({
          status: 500,
          error: 'levelDB_went_went_bad',
          reason: 'unknown'
        });
        exports.FORBIDDEN = new PouchError({
          status: 403,
          error: 'forbidden',
          reason: 'Forbidden by design doc validate_doc_update function'
        });
        exports.INVALID_REV = new PouchError({
          status: 400,
          error: 'bad_request',
          reason: 'Invalid rev format'
        });
        exports.FILE_EXISTS = new PouchError({
          status: 412,
          error: 'file_exists',
          reason: 'The database could not be created, the file already exists.'
        });
        exports.MISSING_STUB = new PouchError({
          status: 412,
          error: 'missing_stub'
        });
        exports.error = function (error, reason, name) {
          function CustomPouchError(reason) {
            // inherit error properties from our parent error manually
            // so as to allow proper JSON parsing.
            /* jshint ignore:start */
            for (var p in error) {
              if (typeof error[p] !== 'function') {
                this[p] = error[p];
              }
            }
            /* jshint ignore:end */
            if (name !== undefined) {
              this.name = name;
            }
            if (reason !== undefined) {
              this.reason = reason;
            }
          }
          CustomPouchError.prototype = PouchError.prototype;
          return new CustomPouchError(reason);
        };
        // Find one of the errors defined above based on the value
        // of the specified property.
        // If reason is provided prefer the error matching that reason.
        // This is for differentiating between errors with the same name and status,
        // eg, bad_request.
        exports.getErrorTypeByProp = function (prop, value, reason) {
          var errors = exports;
          var keys = Object.keys(errors).filter(function (key) {
              var error = errors[key];
              return typeof error !== 'function' && error[prop] === value;
            });
          var key = reason && keys.filter(function (key) {
              var error = errors[key];
              return error.message === reason;
            })[0] || keys[0];
          return key ? errors[key] : null;
        };
        exports.generateErrorFromResponse = function (res) {
          var error, errName, errType, errMsg, errReason;
          var errors = exports;
          errName = res.error === true && typeof res.name === 'string' ? res.name : res.error;
          errReason = res.reason;
          errType = errors.getErrorTypeByProp('name', errName, errReason);
          if (res.missing || errReason === 'missing' || errReason === 'deleted' || errName === 'not_found') {
            errType = errors.MISSING_DOC;
          } else if (errName === 'doc_validation') {
            // doc validation needs special treatment since
            // res.reason depends on the validation error.
            // see utils.js
            errType = errors.DOC_VALIDATION;
            errMsg = errReason;
          } else if (errName === 'bad_request' && errType.message !== errReason) {
            // if bad_request error already found based on reason don't override.
            // attachment errors.
            if (errReason.indexOf('unknown stub attachment') === 0) {
              errType = errors.MISSING_STUB;
              errMsg = errReason;
            } else {
              errType = errors.BAD_REQUEST;
            }
          }
          // fallback to error by statys or unknown error.
          if (!errType) {
            errType = errors.getErrorTypeByProp('status', res.status, errReason) || errors.UNKNOWN_ERROR;
          }
          error = errors.error(errType, errReason, errName);
          // Keep custom message.
          if (errMsg) {
            error.message = errMsg;
          }
          // Keep helpful response data in our error messages.
          if (res.id) {
            error.id = res.id;
          }
          if (res.status) {
            error.status = res.status;
          }
          if (res.statusText) {
            error.name = res.statusText;
          }
          if (res.missing) {
            error.missing = res.missing;
          }
          return error;
        };
      },
      { '44': 44 }
    ],
    21: [
      function (_dereq_, module, exports) {
        (function (process, global) {
          'use strict';
          var crypto = _dereq_(38);
          var Md5 = _dereq_(74);
          var setImmediateShim = global.setImmediate || global.setTimeout;
          var MD5_CHUNK_SIZE = 32768;
          // convert a 64-bit int to a binary string
          function intToString(int) {
            var bytes = [
                int & 255,
                int >>> 8 & 255,
                int >>> 16 & 255,
                int >>> 24 & 255
              ];
            return bytes.map(function (byte) {
              return String.fromCharCode(byte);
            }).join('');
          }
          // convert an array of 64-bit ints into
          // a base64-encoded string
          function rawToBase64(raw) {
            var res = '';
            for (var i = 0; i < raw.length; i++) {
              res += intToString(raw[i]);
            }
            return btoa(res);
          }
          function appendBuffer(buffer, data, start, end) {
            if (start > 0 || end < data.byteLength) {
              // only create a subarray if we really need to
              data = new Uint8Array(data, start, Math.min(end, data.byteLength) - start);
            }
            buffer.append(data);
          }
          function appendString(buffer, data, start, end) {
            if (start > 0 || end < data.length) {
              // only create a substring if we really need to
              data = data.substring(start, end);
            }
            buffer.appendBinary(data);
          }
          module.exports = function (data, callback) {
            if (!process.browser) {
              var base64 = crypto.createHash('md5').update(data).digest('base64');
              callback(null, base64);
              return;
            }
            var inputIsString = typeof data === 'string';
            var len = inputIsString ? data.length : data.byteLength;
            var chunkSize = Math.min(MD5_CHUNK_SIZE, len);
            var chunks = Math.ceil(len / chunkSize);
            var currentChunk = 0;
            var buffer = inputIsString ? new Md5() : new Md5.ArrayBuffer();
            var append = inputIsString ? appendString : appendBuffer;
            function loadNextChunk() {
              var start = currentChunk * chunkSize;
              var end = start + chunkSize;
              currentChunk++;
              if (currentChunk < chunks) {
                append(buffer, data, start, end);
                setImmediateShim(loadNextChunk);
              } else {
                append(buffer, data, start, end);
                var raw = buffer.end(true);
                var base64 = rawToBase64(raw);
                callback(null, base64);
                buffer.destroy();
              }
            }
            loadNextChunk();
          };
        }.call(this, _dereq_(40), typeof global !== 'undefined' ? global : typeof self !== 'undefined' ? self : typeof window !== 'undefined' ? window : {}));
      },
      {
        '38': 38,
        '40': 40,
        '74': 74
      }
    ],
    22: [
      function (_dereq_, module, exports) {
        'use strict';
        var errors = _dereq_(20);
        var uuid = _dereq_(27);
        function toObject(array) {
          return array.reduce(function (obj, item) {
            obj[item] = true;
            return obj;
          }, {});
        }
        // List of top level reserved words for doc
        var reservedWords = toObject([
            '_id',
            '_rev',
            '_attachments',
            '_deleted',
            '_revisions',
            '_revs_info',
            '_conflicts',
            '_deleted_conflicts',
            '_local_seq',
            '_rev_tree',
            '_replication_id',
            '_replication_state',
            '_replication_state_time',
            '_replication_state_reason',
            '_replication_stats',
            '_removed'
          ]);
        // List of reserved words that should end up the document
        var dataWords = toObject([
            '_attachments',
            '_replication_id',
            '_replication_state',
            '_replication_state_time',
            '_replication_state_reason',
            '_replication_stats'
          ]);
        // Determine id an ID is valid
        //   - invalid IDs begin with an underescore that does not begin '_design' or
        //     '_local'
        //   - any other string value is a valid id
        // Returns the specific error object for each case
        exports.invalidIdError = function (id) {
          var err;
          if (!id) {
            err = errors.error(errors.MISSING_ID);
          } else if (typeof id !== 'string') {
            err = errors.error(errors.INVALID_ID);
          } else if (/^_/.test(id) && !/^_(design|local)/.test(id)) {
            err = errors.error(errors.RESERVED_ID);
          }
          if (err) {
            throw err;
          }
        };
        function parseRevisionInfo(rev) {
          if (!/^\d+\-./.test(rev)) {
            return errors.error(errors.INVALID_REV);
          }
          var idx = rev.indexOf('-');
          var left = rev.substring(0, idx);
          var right = rev.substring(idx + 1);
          return {
            prefix: parseInt(left, 10),
            id: right
          };
        }
        function makeRevTreeFromRevisions(revisions, opts) {
          var pos = revisions.start - revisions.ids.length + 1;
          var revisionIds = revisions.ids;
          var ids = [
              revisionIds[0],
              opts,
              []
            ];
          for (var i = 1, len = revisionIds.length; i < len; i++) {
            ids = [
              revisionIds[i],
              { status: 'missing' },
              [ids]
            ];
          }
          return [{
              pos: pos,
              ids: ids
            }];
        }
        // Preprocess documents, parse their revisions, assign an id and a
        // revision for new writes that are missing them, etc
        exports.parseDoc = function (doc, newEdits) {
          var nRevNum;
          var newRevId;
          var revInfo;
          var opts = { status: 'available' };
          if (doc._deleted) {
            opts.deleted = true;
          }
          if (newEdits) {
            if (!doc._id) {
              doc._id = uuid();
            }
            newRevId = uuid(32, 16).toLowerCase();
            if (doc._rev) {
              revInfo = parseRevisionInfo(doc._rev);
              if (revInfo.error) {
                return revInfo;
              }
              doc._rev_tree = [{
                  pos: revInfo.prefix,
                  ids: [
                    revInfo.id,
                    { status: 'missing' },
                    [[
                        newRevId,
                        opts,
                        []
                      ]]
                  ]
                }];
              nRevNum = revInfo.prefix + 1;
            } else {
              doc._rev_tree = [{
                  pos: 1,
                  ids: [
                    newRevId,
                    opts,
                    []
                  ]
                }];
              nRevNum = 1;
            }
          } else {
            if (doc._revisions) {
              doc._rev_tree = makeRevTreeFromRevisions(doc._revisions, opts);
              nRevNum = doc._revisions.start;
              newRevId = doc._revisions.ids[0];
            }
            if (!doc._rev_tree) {
              revInfo = parseRevisionInfo(doc._rev);
              if (revInfo.error) {
                return revInfo;
              }
              nRevNum = revInfo.prefix;
              newRevId = revInfo.id;
              doc._rev_tree = [{
                  pos: nRevNum,
                  ids: [
                    newRevId,
                    opts,
                    []
                  ]
                }];
            }
          }
          exports.invalidIdError(doc._id);
          doc._rev = nRevNum + '-' + newRevId;
          var result = {
              metadata: {},
              data: {}
            };
          for (var key in doc) {
            if (doc.hasOwnProperty(key)) {
              var specialKey = key[0] === '_';
              if (specialKey && !reservedWords[key]) {
                var error = errors.error(errors.DOC_VALIDATION, key);
                error.message = errors.DOC_VALIDATION.message + ': ' + key;
                throw error;
              } else if (specialKey && !dataWords[key]) {
                result.metadata[key.slice(1)] = doc[key];
              } else {
                result.data[key] = doc[key];
              }
            }
          }
          return result;
        };
      },
      {
        '20': 20,
        '27': 27
      }
    ],
    23: [
      function (_dereq_, module, exports) {
        'use strict';
        //
        // Parsing hex strings. Yeah.
        //
        // So basically we need this because of a bug in WebSQL:
        // https://code.google.com/p/chromium/issues/detail?id=422690
        // https://bugs.webkit.org/show_bug.cgi?id=137637
        //
        // UTF-8 and UTF-16 are provided as separate functions
        // for meager performance improvements
        //
        function decodeUtf8(str) {
          return decodeURIComponent(window.escape(str));
        }
        function hexToInt(charCode) {
          // '0'-'9' is 48-57
          // 'A'-'F' is 65-70
          // SQLite will only give us uppercase hex
          return charCode < 65 ? charCode - 48 : charCode - 55;
        }
        // Example:
        // pragma encoding=utf8;
        // select hex('A');
        // returns '41'
        function parseHexUtf8(str, start, end) {
          var result = '';
          while (start < end) {
            result += String.fromCharCode(hexToInt(str.charCodeAt(start++)) << 4 | hexToInt(str.charCodeAt(start++)));
          }
          return result;
        }
        // Example:
        // pragma encoding=utf16;
        // select hex('A');
        // returns '4100'
        // notice that the 00 comes after the 41 (i.e. it's swizzled)
        function parseHexUtf16(str, start, end) {
          var result = '';
          while (start < end) {
            // UTF-16, so swizzle the bytes
            result += String.fromCharCode(hexToInt(str.charCodeAt(start + 2)) << 12 | hexToInt(str.charCodeAt(start + 3)) << 8 | hexToInt(str.charCodeAt(start)) << 4 | hexToInt(str.charCodeAt(start + 1)));
            start += 4;
          }
          return result;
        }
        function parseHexString(str, encoding) {
          if (encoding === 'UTF-8') {
            return decodeUtf8(parseHexUtf8(str, 0, str.length));
          } else {
            return parseHexUtf16(str, 0, str.length);
          }
        }
        module.exports = parseHexString;
      },
      {}
    ],
    24: [
      function (_dereq_, module, exports) {
        'use strict';
        // originally parseUri 1.2.2, now patched by us
        // (c) Steven Levithan <stevenlevithan.com>
        // MIT License
        var options = {
            strictMode: false,
            key: [
              'source',
              'protocol',
              'authority',
              'userInfo',
              'user',
              'password',
              'host',
              'port',
              'relative',
              'path',
              'directory',
              'file',
              'query',
              'anchor'
            ],
            q: {
              name: 'queryKey',
              parser: /(?:^|&)([^&=]*)=?([^&]*)/g
            },
            parser: {
              strict: /^(?:([^:\/?#]+):)?(?:\/\/((?:(([^:@]*)(?::([^:@]*))?)?@)?([^:\/?#]*)(?::(\d*))?))?((((?:[^?#\/]*\/)*)([^?#]*))(?:\?([^#]*))?(?:#(.*))?)/,
              loose: /^(?:(?![^:@]+:[^:@\/]*@)([^:\/?#.]+):)?(?:\/\/)?((?:(([^:@]*)(?::([^:@]*))?)?@)?([^:\/?#]*)(?::(\d*))?)(((\/(?:[^?#](?![^?#\/]*\.[^?#\/.]+(?:[?#]|$)))*\/?)?([^?#\/]*))(?:\?([^#]*))?(?:#(.*))?)/
            }
          };
        function parseUri(str) {
          var o = options;
          var m = o.parser[o.strictMode ? 'strict' : 'loose'].exec(str);
          var uri = {};
          var i = 14;
          while (i--) {
            var key = o.key[i];
            var value = m[i] || '';
            var encoded = [
                'user',
                'password'
              ].indexOf(key) !== -1;
            uri[key] = encoded ? decodeURIComponent(value) : value;
          }
          uri[o.q.name] = {};
          uri[o.key[12]].replace(o.q.parser, function ($0, $1, $2) {
            if ($1) {
              uri[o.q.name][$1] = $2;
            }
          });
          return uri;
        }
        module.exports = parseUri;
      },
      {}
    ],
    25: [
      function (_dereq_, module, exports) {
        'use strict';
        var createBlob = _dereq_(18);
        var utils = _dereq_(35);
        module.exports = function (options, callback) {
          var xhr, timer, hasUpload;
          var abortReq = function () {
            xhr.abort();
          };
          if (options.xhr) {
            xhr = new options.xhr();
          } else {
            xhr = new XMLHttpRequest();
          }
          // cache-buster, specifically designed to work around IE's aggressive caching
          // see http://www.dashbay.com/2011/05/internet-explorer-caches-ajax/
          if (options.method === 'GET' && !options.cache) {
            var hasArgs = options.url.indexOf('?') !== -1;
            options.url += (hasArgs ? '&' : '?') + '_nonce=' + Date.now();
          }
          xhr.open(options.method, options.url);
          xhr.withCredentials = true;
          if (options.json) {
            options.headers.Accept = 'application/json';
            options.headers['Content-Type'] = options.headers['Content-Type'] || 'application/json';
            if (options.body && options.processData && typeof options.body !== 'string') {
              options.body = JSON.stringify(options.body);
            }
          }
          if (options.binary) {
            xhr.responseType = 'arraybuffer';
          }
          if (!('body' in options)) {
            options.body = null;
          }
          for (var key in options.headers) {
            if (options.headers.hasOwnProperty(key)) {
              xhr.setRequestHeader(key, options.headers[key]);
            }
          }
          if (options.timeout > 0) {
            timer = setTimeout(abortReq, options.timeout);
            xhr.onprogress = function () {
              clearTimeout(timer);
              timer = setTimeout(abortReq, options.timeout);
            };
            if (typeof hasUpload === 'undefined') {
              // IE throws an error if you try to access it directly
              hasUpload = Object.keys(xhr).indexOf('upload') !== -1;
            }
            if (hasUpload) {
              // does not exist in ie9
              xhr.upload.onprogress = xhr.onprogress;
            }
          }
          xhr.onreadystatechange = function () {
            if (xhr.readyState !== 4) {
              return;
            }
            var response = { statusCode: xhr.status };
            if (xhr.status >= 200 && xhr.status < 300) {
              var data;
              if (options.binary) {
                data = createBlob([xhr.response || ''], { type: xhr.getResponseHeader('Content-Type') });
              } else {
                data = xhr.responseText;
              }
              callback(null, response, data);
            } else {
              var err = {};
              try {
                err = JSON.parse(xhr.response);
              } catch (e) {
              }
              callback(err, response);
            }
          };
          if (options.body && options.body instanceof Blob) {
            utils.readAsBinaryString(options.body, function (binary) {
              xhr.send(utils.fixBinary(binary));
            });
          } else {
            xhr.send(options.body);
          }
          return { abort: abortReq };
        };
      },
      {
        '18': 18,
        '35': 35
      }
    ],
    26: [
      function (_dereq_, module, exports) {
        'use strict';
        var upsert = _dereq_(73).upsert;
        module.exports = function (db, doc, diffFun, cb) {
          return upsert.call(db, doc, diffFun, cb);
        };
      },
      { '73': 73 }
    ],
    27: [
      function (_dereq_, module, exports) {
        'use strict';
        // BEGIN Math.uuid.js
        /*!
Math.uuid.js (v1.4)
http://www.broofa.com
mailto:robert@broofa.com

Copyright (c) 2010 Robert Kieffer
Dual licensed under the MIT and GPL licenses.
*/
        /*
 * Generate a random uuid.
 *
 * USAGE: Math.uuid(length, radix)
 *   length - the desired number of characters
 *   radix  - the number of allowable values for each character.
 *
 * EXAMPLES:
 *   // No arguments  - returns RFC4122, version 4 ID
 *   >>> Math.uuid()
 *   "92329D39-6F5C-4520-ABFC-AAB64544E172"
 *
 *   // One argument - returns ID of the specified length
 *   >>> Math.uuid(15)     // 15 character ID (default base=62)
 *   "VcydxgltxrVZSTV"
 *
 *   // Two arguments - returns ID of the specified length, and radix. 
 *   // (Radix must be <= 62)
 *   >>> Math.uuid(8, 2)  // 8 character ID (base=2)
 *   "01001010"
 *   >>> Math.uuid(8, 10) // 8 character ID (base=10)
 *   "47473046"
 *   >>> Math.uuid(8, 16) // 8 character ID (base=16)
 *   "098F4D35"
 */
        var chars = ('0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ' + 'abcdefghijklmnopqrstuvwxyz').split('');
        function getValue(radix) {
          return 0 | Math.random() * radix;
        }
        function uuid(len, radix) {
          radix = radix || chars.length;
          var out = '';
          var i = -1;
          if (len) {
            // Compact form
            while (++i < len) {
              out += chars[getValue(radix)];
            }
            return out;
          }
          // rfc4122, version 4 form
          // Fill in random data.  At i==19 set the high bits of clock sequence as
          // per rfc4122, sec. 4.1.5
          while (++i < 36) {
            switch (i) {
            case 8:
            case 13:
            case 18:
            case 23:
              out += '-';
              break;
            case 19:
              out += chars[getValue(16) & 3 | 8];
              break;
            default:
              out += chars[getValue(16)];
            }
          }
          return out;
        }
        module.exports = uuid;
      },
      {}
    ],
    28: [
      function (_dereq_, module, exports) {
        'use strict';
        module.exports = evalFilter;
        function evalFilter(input) {
          /*jshint evil: true */
          return eval([
            '(function () { return ',
            input,
            ' })()'
          ].join(''));
        }
      },
      {}
    ],
    29: [
      function (_dereq_, module, exports) {
        'use strict';
        module.exports = evalView;
        function evalView(input) {
          /*jshint evil: true */
          return eval([
            '(function () {',
            '  return function (doc) {',
            '    var emitted = false;',
            '    var emit = function (a, b) {',
            '      emitted = true;',
            '    };',
            '    var view = ' + input + ';',
            '    view(doc);',
            '    if (emitted) {',
            '      return true;',
            '    }',
            '  }',
            '})()'
          ].join('\n'));
        }
      },
      {}
    ],
    30: [
      function (_dereq_, module, exports) {
        'use strict';
        var extend = _dereq_(66);
        // for a better overview of what this is doing, read:
        // https://github.com/apache/couchdb/blob/master/src/couchdb/couch_key_tree.erl
        //
        // But for a quick intro, CouchDB uses a revision tree to store a documents
        // history, A -> B -> C, when a document has conflicts, that is a branch in the
        // tree, A -> (B1 | B2 -> C), We store these as a nested array in the format
        //
        // KeyTree = [Path ... ]
        // Path = {pos: position_from_root, ids: Tree}
        // Tree = [Key, Opts, [Tree, ...]], in particular single node: [Key, []]
        // classic binary search
        function binarySearch(arr, item, comparator) {
          var low = 0;
          var high = arr.length;
          var mid;
          while (low < high) {
            mid = low + high >>> 1;
            if (comparator(arr[mid], item) < 0) {
              low = mid + 1;
            } else {
              high = mid;
            }
          }
          return low;
        }
        // assuming the arr is sorted, insert the item in the proper place
        function insertSorted(arr, item, comparator) {
          var idx = binarySearch(arr, item, comparator);
          arr.splice(idx, 0, item);
        }
        // Turn a path as a flat array into a tree with a single branch
        function pathToTree(path) {
          var doc = path.shift();
          var root = [
              doc.id,
              doc.opts,
              []
            ];
          var leaf = root;
          var nleaf;
          while (path.length) {
            doc = path.shift();
            nleaf = [
              doc.id,
              doc.opts,
              []
            ];
            leaf[2].push(nleaf);
            leaf = nleaf;
          }
          return root;
        }
        // compare the IDs of two trees
        function compareTree(a, b) {
          return a[0] < b[0] ? -1 : 1;
        }
        // Merge two trees together
        // The roots of tree1 and tree2 must be the same revision
        function mergeTree(in_tree1, in_tree2) {
          var queue = [{
                tree1: in_tree1,
                tree2: in_tree2
              }];
          var conflicts = false;
          while (queue.length > 0) {
            var item = queue.pop();
            var tree1 = item.tree1;
            var tree2 = item.tree2;
            if (tree1[1].status || tree2[1].status) {
              tree1[1].status = tree1[1].status === 'available' || tree2[1].status === 'available' ? 'available' : 'missing';
            }
            for (var i = 0; i < tree2[2].length; i++) {
              if (!tree1[2][0]) {
                conflicts = 'new_leaf';
                tree1[2][0] = tree2[2][i];
                continue;
              }
              var merged = false;
              for (var j = 0; j < tree1[2].length; j++) {
                if (tree1[2][j][0] === tree2[2][i][0]) {
                  queue.push({
                    tree1: tree1[2][j],
                    tree2: tree2[2][i]
                  });
                  merged = true;
                }
              }
              if (!merged) {
                conflicts = 'new_branch';
                insertSorted(tree1[2], tree2[2][i], compareTree);
              }
            }
          }
          return {
            conflicts: conflicts,
            tree: in_tree1
          };
        }
        function doMerge(tree, path, dontExpand) {
          var restree = [];
          var conflicts = false;
          var merged = false;
          var res;
          if (!tree.length) {
            return {
              tree: [path],
              conflicts: 'new_leaf'
            };
          }
          tree.forEach(function (branch) {
            if (branch.pos === path.pos && branch.ids[0] === path.ids[0]) {
              // Paths start at the same position and have the same root, so they need
              // merged
              res = mergeTree(branch.ids, path.ids);
              restree.push({
                pos: branch.pos,
                ids: res.tree
              });
              conflicts = conflicts || res.conflicts;
              merged = true;
            } else if (dontExpand !== true) {
              // The paths start at a different position, take the earliest path and
              // traverse up until it as at the same point from root as the path we
              // want to merge.  If the keys match we return the longer path with the
              // other merged After stemming we dont want to expand the trees
              var t1 = branch.pos < path.pos ? branch : path;
              var t2 = branch.pos < path.pos ? path : branch;
              var diff = t2.pos - t1.pos;
              var candidateParents = [];
              var trees = [];
              trees.push({
                ids: t1.ids,
                diff: diff,
                parent: null,
                parentIdx: null
              });
              while (trees.length > 0) {
                var item = trees.pop();
                if (item.diff === 0) {
                  if (item.ids[0] === t2.ids[0]) {
                    candidateParents.push(item);
                  }
                  continue;
                }
                if (!item.ids) {
                  continue;
                }
                /*jshint loopfunc:true */
                item.ids[2].forEach(function (el, idx) {
                  trees.push({
                    ids: el,
                    diff: item.diff - 1,
                    parent: item.ids,
                    parentIdx: idx
                  });
                });
              }
              var el = candidateParents[0];
              if (!el) {
                restree.push(branch);
              } else {
                res = mergeTree(el.ids, t2.ids);
                el.parent[2][el.parentIdx] = res.tree;
                restree.push({
                  pos: t1.pos,
                  ids: t1.ids
                });
                conflicts = conflicts || res.conflicts;
                merged = true;
              }
            } else {
              restree.push(branch);
            }
          });
          // We didnt find
          if (!merged) {
            restree.push(path);
          }
          restree.sort(function (a, b) {
            return a.pos - b.pos;
          });
          return {
            tree: restree,
            conflicts: conflicts || 'internal_node'
          };
        }
        // To ensure we dont grow the revision tree infinitely, we stem old revisions
        function stem(tree, depth) {
          // First we break out the tree into a complete list of root to leaf paths,
          // we cut off the start of the path and generate a new set of flat trees
          var stemmedPaths = PouchMerge.rootToLeaf(tree).map(function (path) {
              var stemmed = path.ids.slice(-depth);
              return {
                pos: path.pos + (path.ids.length - stemmed.length),
                ids: pathToTree(stemmed)
              };
            });
          // Then we remerge all those flat trees together, ensuring that we dont
          // connect trees that would go beyond the depth limit
          return stemmedPaths.reduce(function (prev, current) {
            return doMerge(prev, current, true).tree;
          }, [stemmedPaths.shift()]);
        }
        var PouchMerge = {};
        PouchMerge.merge = function (tree, path, depth) {
          // Ugh, nicer way to not modify arguments in place?
          tree = extend(true, [], tree);
          path = extend(true, {}, path);
          var newTree = doMerge(tree, path);
          return {
            tree: stem(newTree.tree, depth),
            conflicts: newTree.conflicts
          };
        };
        // We fetch all leafs of the revision tree, and sort them based on tree length
        // and whether they were deleted, undeleted documents with the longest revision
        // tree (most edits) win
        // The final sort algorithm is slightly documented in a sidebar here:
        // http://guide.couchdb.org/draft/conflicts.html
        PouchMerge.winningRev = function (metadata) {
          var leafs = [];
          PouchMerge.traverseRevTree(metadata.rev_tree, function (isLeaf, pos, id, something, opts) {
            if (isLeaf) {
              leafs.push({
                pos: pos,
                id: id,
                deleted: !!opts.deleted
              });
            }
          });
          leafs.sort(function (a, b) {
            if (a.deleted !== b.deleted) {
              return a.deleted > b.deleted ? 1 : -1;
            }
            if (a.pos !== b.pos) {
              return b.pos - a.pos;
            }
            return a.id < b.id ? 1 : -1;
          });
          return leafs[0].pos + '-' + leafs[0].id;
        };
        // Pretty much all below can be combined into a higher order function to
        // traverse revisions
        // The return value from the callback will be passed as context to all
        // children of that node
        PouchMerge.traverseRevTree = function (revs, callback) {
          var toVisit = revs.slice();
          var node;
          while (node = toVisit.pop()) {
            var pos = node.pos;
            var tree = node.ids;
            var branches = tree[2];
            var newCtx = callback(branches.length === 0, pos, tree[0], node.ctx, tree[1]);
            for (var i = 0, len = branches.length; i < len; i++) {
              toVisit.push({
                pos: pos + 1,
                ids: branches[i],
                ctx: newCtx
              });
            }
          }
        };
        PouchMerge.collectLeaves = function (revs) {
          var leaves = [];
          PouchMerge.traverseRevTree(revs, function (isLeaf, pos, id, acc, opts) {
            if (isLeaf) {
              leaves.push({
                rev: pos + '-' + id,
                pos: pos,
                opts: opts
              });
            }
          });
          leaves.sort(function (a, b) {
            return b.pos - a.pos;
          });
          leaves.forEach(function (leaf) {
            delete leaf.pos;
          });
          return leaves;
        };
        // returns revs of all conflicts that is leaves such that
        // 1. are not deleted and
        // 2. are different than winning revision
        PouchMerge.collectConflicts = function (metadata) {
          var win = PouchMerge.winningRev(metadata);
          var leaves = PouchMerge.collectLeaves(metadata.rev_tree);
          var conflicts = [];
          leaves.forEach(function (leaf) {
            if (leaf.rev !== win && !leaf.opts.deleted) {
              conflicts.push(leaf.rev);
            }
          });
          return conflicts;
        };
        PouchMerge.rootToLeaf = function (tree) {
          var paths = [];
          PouchMerge.traverseRevTree(tree, function (isLeaf, pos, id, history, opts) {
            history = history ? history.slice(0) : [];
            history.push({
              id: id,
              opts: opts
            });
            if (isLeaf) {
              var rootPos = pos + 1 - history.length;
              paths.unshift({
                pos: rootPos,
                ids: history
              });
            }
            return history;
          });
          return paths;
        };
        module.exports = PouchMerge;
      },
      { '66': 66 }
    ],
    31: [
      function (_dereq_, module, exports) {
        'use strict';
        var utils = _dereq_(35);
        var EE = _dereq_(39).EventEmitter;
        var Checkpointer = _dereq_(15);
        var MAX_SIMULTANEOUS_REVS = 50;
        var RETRY_DEFAULT = false;
        function randomNumber(min, max) {
          min = parseInt(min, 10);
          max = parseInt(max, 10);
          if (min !== min) {
            min = 0;
          }
          if (max !== max || max <= min) {
            max = (min || 1) << 1;  //doubling
          } else {
            max = max + 1;
          }
          var ratio = Math.random();
          var range = max - min;
          return ~~(range * ratio + min);  // ~~ coerces to an int, but fast.
        }
        function defaultBackOff(min) {
          var max = 0;
          if (!min) {
            max = 2000;
          }
          return randomNumber(min, max);
        }
        function backOff(repId, src, target, opts, returnValue, result, error) {
          if (opts.retry === false) {
            returnValue.emit('error', error);
            returnValue.removeAllListeners();
            return;
          }
          opts.default_back_off = opts.default_back_off || 0;
          opts.retries = opts.retries || 0;
          if (typeof opts.back_off_function !== 'function') {
            opts.back_off_function = defaultBackOff;
          }
          opts.retries++;
          if (opts.max_retries && opts.retries > opts.max_retries) {
            returnValue.emit('error', new Error('tried ' + opts.retries + ' times but replication failed'));
            returnValue.removeAllListeners();
            return;
          }
          returnValue.emit('requestError', error);
          if (returnValue.state === 'active') {
            returnValue.emit('paused', error);
            returnValue.state = 'stopped';
            returnValue.once('active', function () {
              opts.current_back_off = opts.default_back_off;
            });
          }
          opts.current_back_off = opts.current_back_off || opts.default_back_off;
          opts.current_back_off = opts.back_off_function(opts.current_back_off);
          setTimeout(function () {
            replicate(repId, src, target, opts, returnValue);
          }, opts.current_back_off);
        }
        // We create a basic promise so the caller can cancel the replication possibly
        // before we have actually started listening to changes etc
        utils.inherits(Replication, EE);
        function Replication() {
          EE.call(this);
          this.cancelled = false;
          this.state = 'pending';
          var self = this;
          var promise = new utils.Promise(function (fulfill, reject) {
              self.once('complete', fulfill);
              self.once('error', reject);
            });
          self.then = function (resolve, reject) {
            return promise.then(resolve, reject);
          };
          self['catch'] = function (reject) {
            return promise['catch'](reject);
          };
          // As we allow error handling via "error" event as well,
          // put a stub in here so that rejecting never throws UnhandledError.
          self['catch'](function () {
          });
        }
        Replication.prototype.cancel = function () {
          this.cancelled = true;
          this.state = 'cancelled';
          this.emit('cancel');
        };
        Replication.prototype.ready = function (src, target) {
          var self = this;
          function onDestroy() {
            self.cancel();
          }
          src.once('destroyed', onDestroy);
          target.once('destroyed', onDestroy);
          function cleanup() {
            src.removeListener('destroyed', onDestroy);
            target.removeListener('destroyed', onDestroy);
          }
          this.then(cleanup, cleanup);
        };
        // TODO: check CouchDB's replication id generation
        // Generate a unique id particular to this replication
        function genReplicationId(src, target, opts) {
          var filterFun = opts.filter ? opts.filter.toString() : '';
          return src.id().then(function (src_id) {
            return target.id().then(function (target_id) {
              var queryData = src_id + target_id + filterFun + JSON.stringify(opts.query_params) + opts.doc_ids;
              return utils.MD5(queryData).then(function (md5) {
                // can't use straight-up md5 alphabet, because
                // the char '/' is interpreted as being for attachments,
                // and + is also not url-safe
                md5 = md5.replace(/\//g, '.').replace(/\+/g, '_');
                return '_local/' + md5;
              });
            });
          });
        }
        function replicate(repId, src, target, opts, returnValue, result) {
          var batches = [];
          // list of batches to be processed
          var currentBatch;
          // the batch currently being processed
          var pendingBatch = {
              seq: 0,
              changes: [],
              docs: []
            };
          // next batch, not yet ready to be processed
          var writingCheckpoint = false;
          // true while checkpoint is being written
          var changesCompleted = false;
          // true when all changes received
          var replicationCompleted = false;
          // true when replication has completed
          var last_seq = 0;
          var continuous = opts.continuous || opts.live || false;
          var batch_size = opts.batch_size || 100;
          var batches_limit = opts.batches_limit || 10;
          var changesPending = false;
          // true while src.changes is running
          var doc_ids = opts.doc_ids;
          var state = { cancelled: false };
          var checkpointer = new Checkpointer(src, target, repId, state);
          var allErrors = [];
          var changedDocs = [];
          result = result || {
            ok: true,
            start_time: new Date(),
            docs_read: 0,
            docs_written: 0,
            doc_write_failures: 0,
            errors: []
          };
          var changesOpts = {};
          returnValue.ready(src, target);
          function writeDocs() {
            if (currentBatch.docs.length === 0) {
              return;
            }
            var docs = currentBatch.docs;
            return target.bulkDocs({
              docs: docs,
              new_edits: false
            }).then(function (res) {
              if (state.cancelled) {
                completeReplication();
                throw new Error('cancelled');
              }
              var errors = [];
              var errorsById = {};
              res.forEach(function (res) {
                if (res.error) {
                  result.doc_write_failures++;
                  errors.push(res);
                  errorsById[res.id] = res;
                }
              });
              result.errors = errors;
              allErrors = allErrors.concat(errors);
              result.docs_written += currentBatch.docs.length - errors.length;
              var non403s = errors.filter(function (error) {
                  return error.name !== 'unauthorized' && error.name !== 'forbidden';
                });
              changedDocs = [];
              docs.forEach(function (doc) {
                var error = errorsById[doc._id];
                if (error) {
                  returnValue.emit('denied', utils.clone(error));
                } else {
                  changedDocs.push(doc);
                }
              });
              if (non403s.length > 0) {
                var error = new Error('bulkDocs error');
                error.other_errors = errors;
                abortReplication('target.bulkDocs failed to write docs', error);
                throw new Error('bulkWrite partial failure');
              }
            }, function (err) {
              result.doc_write_failures += docs.length;
              throw err;
            });
          }
          function processDiffDoc(id) {
            var diffs = currentBatch.diffs;
            var allMissing = diffs[id].missing;
            // avoid url too long error by batching
            var missingBatches = [];
            for (var i = 0; i < allMissing.length; i += MAX_SIMULTANEOUS_REVS) {
              missingBatches.push(allMissing.slice(i, Math.min(allMissing.length, i + MAX_SIMULTANEOUS_REVS)));
            }
            return utils.Promise.all(missingBatches.map(function (missing) {
              var opts = {
                  revs: true,
                  open_revs: missing,
                  attachments: true
                };
              return src.get(id, opts).then(function (docs) {
                docs.forEach(function (doc) {
                  if (state.cancelled) {
                    return completeReplication();
                  }
                  if (doc.ok) {
                    result.docs_read++;
                    currentBatch.pendingRevs++;
                    currentBatch.docs.push(doc.ok);
                  }
                });
                delete diffs[id];
              });
            }));
          }
          function getAllDocs() {
            var diffKeys = Object.keys(currentBatch.diffs);
            return utils.Promise.all(diffKeys.map(processDiffDoc));
          }
          function getRevisionOneDocs() {
            // filter out the generation 1 docs and get them
            // leaving the non-generation one docs to be got otherwise
            var ids = Object.keys(currentBatch.diffs).filter(function (id) {
                var missing = currentBatch.diffs[id].missing;
                return missing.length === 1 && missing[0].slice(0, 2) === '1-';
              });
            if (!ids.length) {
              // nothing to fetch
              return utils.Promise.resolve();
            }
            return src.allDocs({
              keys: ids,
              include_docs: true
            }).then(function (res) {
              if (state.cancelled) {
                completeReplication();
                throw new Error('cancelled');
              }
              res.rows.forEach(function (row) {
                if (row.doc && !row.deleted && row.value.rev.slice(0, 2) === '1-' && (!row.doc._attachments || Object.keys(row.doc._attachments).length === 0)) {
                  result.docs_read++;
                  currentBatch.pendingRevs++;
                  currentBatch.docs.push(row.doc);
                  delete currentBatch.diffs[row.id];
                }
              });
            });
          }
          function getDocs() {
            return getRevisionOneDocs().then(getAllDocs);
          }
          function finishBatch() {
            writingCheckpoint = true;
            return checkpointer.writeCheckpoint(currentBatch.seq).then(function () {
              writingCheckpoint = false;
              if (state.cancelled) {
                completeReplication();
                throw new Error('cancelled');
              }
              result.last_seq = last_seq = currentBatch.seq;
              var outResult = utils.clone(result);
              outResult.docs = changedDocs;
              returnValue.emit('change', outResult);
              currentBatch = undefined;
              getChanges();
            })['catch'](function (err) {
              writingCheckpoint = false;
              abortReplication('writeCheckpoint completed with error', err);
              throw err;
            });
          }
          function getDiffs() {
            var diff = {};
            currentBatch.changes.forEach(function (change) {
              // Couchbase Sync Gateway emits these, but we can ignore them
              if (change.id === '_user/') {
                return;
              }
              diff[change.id] = change.changes.map(function (x) {
                return x.rev;
              });
            });
            return target.revsDiff(diff).then(function (diffs) {
              if (state.cancelled) {
                completeReplication();
                throw new Error('cancelled');
              }
              // currentBatch.diffs elements are deleted as the documents are written
              currentBatch.diffs = diffs;
              currentBatch.pendingRevs = 0;
            });
          }
          function startNextBatch() {
            if (state.cancelled || currentBatch) {
              return;
            }
            if (batches.length === 0) {
              processPendingBatch(true);
              return;
            }
            currentBatch = batches.shift();
            getDiffs().then(getDocs).then(writeDocs).then(finishBatch).then(startNextBatch)['catch'](function (err) {
              abortReplication('batch processing terminated with error', err);
            });
          }
          function processPendingBatch(immediate) {
            if (pendingBatch.changes.length === 0) {
              if (batches.length === 0 && !currentBatch) {
                if (continuous && changesOpts.live || changesCompleted) {
                  returnValue.state = 'pending';
                  returnValue.emit('paused');
                  returnValue.emit('uptodate', result);
                }
                if (changesCompleted) {
                  completeReplication();
                }
              }
              return;
            }
            if (immediate || changesCompleted || pendingBatch.changes.length >= batch_size) {
              batches.push(pendingBatch);
              pendingBatch = {
                seq: 0,
                changes: [],
                docs: []
              };
              if (returnValue.state === 'pending' || returnValue.state === 'stopped') {
                returnValue.state = 'active';
                returnValue.emit('active');
              }
              startNextBatch();
            }
          }
          function abortReplication(reason, err) {
            if (replicationCompleted) {
              return;
            }
            if (!err.message) {
              err.message = reason;
            }
            result.ok = false;
            result.status = 'aborting';
            result.errors.push(err);
            allErrors = allErrors.concat(err);
            batches = [];
            pendingBatch = {
              seq: 0,
              changes: [],
              docs: []
            };
            completeReplication();
          }
          function completeReplication() {
            if (replicationCompleted) {
              return;
            }
            if (state.cancelled) {
              result.status = 'cancelled';
              if (writingCheckpoint) {
                return;
              }
            }
            result.status = result.status || 'complete';
            result.end_time = new Date();
            result.last_seq = last_seq;
            replicationCompleted = state.cancelled = true;
            var non403s = allErrors.filter(function (error) {
                return error.name !== 'unauthorized' && error.name !== 'forbidden';
              });
            if (non403s.length > 0) {
              var error = allErrors.pop();
              if (allErrors.length > 0) {
                error.other_errors = allErrors;
              }
              error.result = result;
              backOff(repId, src, target, opts, returnValue, result, error);
            } else {
              result.errors = allErrors;
              returnValue.emit('complete', result);
              returnValue.removeAllListeners();
            }
          }
          function onChange(change) {
            if (state.cancelled) {
              return completeReplication();
            }
            var filter = utils.filterChange(opts)(change);
            if (!filter) {
              return;
            }
            if (pendingBatch.changes.length === 0 && batches.length === 0 && !currentBatch) {
              returnValue.emit('outofdate', result);
            }
            pendingBatch.seq = change.seq;
            pendingBatch.changes.push(change);
            processPendingBatch(batches.length === 0);
          }
          function onChangesComplete(changes) {
            changesPending = false;
            if (state.cancelled) {
              return completeReplication();
            }
            // if no results were returned then we're done,
            // else fetch more
            if (changes.results.length > 0) {
              changesOpts.since = changes.last_seq;
              getChanges();
            } else {
              if (continuous) {
                changesOpts.live = true;
                getChanges();
              } else {
                changesCompleted = true;
              }
            }
            processPendingBatch(true);
          }
          function onChangesError(err) {
            changesPending = false;
            if (state.cancelled) {
              return completeReplication();
            }
            abortReplication('changes rejected', err);
          }
          function getChanges() {
            if (!(!changesPending && !changesCompleted && batches.length < batches_limit)) {
              return;
            }
            changesPending = true;
            function abortChanges() {
              changes.cancel();
            }
            function removeListener() {
              returnValue.removeListener('cancel', abortChanges);
            }
            returnValue.once('cancel', abortChanges);
            var changes = src.changes(changesOpts).on('change', onChange);
            changes.then(removeListener, removeListener);
            changes.then(onChangesComplete)['catch'](onChangesError);
          }
          function startChanges() {
            checkpointer.getCheckpoint().then(function (checkpoint) {
              last_seq = checkpoint;
              changesOpts = {
                since: last_seq,
                limit: batch_size,
                batch_size: batch_size,
                style: 'all_docs',
                doc_ids: doc_ids,
                returnDocs: true
              };
              if (opts.filter) {
                if (typeof opts.filter !== 'string') {
                  // required for the client-side filter in onChange
                  changesOpts.include_docs = true;
                } else {
                  // ddoc filter
                  changesOpts.filter = opts.filter;
                }
              }
              if (opts.query_params) {
                changesOpts.query_params = opts.query_params;
              }
              if (opts.view) {
                changesOpts.view = opts.view;
              }
              getChanges();
            })['catch'](function (err) {
              abortReplication('getCheckpoint rejected with ', err);
            });
          }
          if (returnValue.cancelled) {
            // cancelled immediately
            completeReplication();
            return;
          }
          returnValue.once('cancel', completeReplication);
          if (typeof opts.onChange === 'function') {
            returnValue.on('change', opts.onChange);
          }
          if (typeof opts.complete === 'function') {
            returnValue.once('error', opts.complete);
            returnValue.once('complete', function (result) {
              opts.complete(null, result);
            });
          }
          if (typeof opts.since === 'undefined') {
            startChanges();
          } else {
            writingCheckpoint = true;
            checkpointer.writeCheckpoint(opts.since).then(function () {
              writingCheckpoint = false;
              if (state.cancelled) {
                completeReplication();
                return;
              }
              last_seq = opts.since;
              startChanges();
            })['catch'](function (err) {
              writingCheckpoint = false;
              abortReplication('writeCheckpoint completed with error', err);
              throw err;
            });
          }
        }
        exports.toPouch = toPouch;
        function toPouch(db, opts) {
          var PouchConstructor = opts.PouchConstructor;
          if (typeof db === 'string') {
            return new PouchConstructor(db, opts);
          } else if (db.then) {
            return db;
          } else {
            return utils.Promise.resolve(db);
          }
        }
        exports.replicate = replicateWrapper;
        function replicateWrapper(src, target, opts, callback) {
          if (typeof opts === 'function') {
            callback = opts;
            opts = {};
          }
          if (typeof opts === 'undefined') {
            opts = {};
          }
          if (!opts.complete) {
            opts.complete = callback || function () {
            };
          }
          opts = utils.clone(opts);
          opts.continuous = opts.continuous || opts.live;
          opts.retry = 'retry' in opts ? opts.retry : RETRY_DEFAULT;
          /*jshint validthis:true */
          opts.PouchConstructor = opts.PouchConstructor || this;
          var replicateRet = new Replication(opts);
          toPouch(src, opts).then(function (src) {
            return toPouch(target, opts).then(function (target) {
              return genReplicationId(src, target, opts).then(function (repId) {
                replicate(repId, src, target, opts, replicateRet);
              });
            });
          })['catch'](function (err) {
            replicateRet.emit('error', err);
            opts.complete(err);
          });
          return replicateRet;
        }
      },
      {
        '15': 15,
        '35': 35,
        '39': 39
      }
    ],
    32: [
      function (_dereq_, module, exports) {
        'use strict';
        var PouchDB = _dereq_(16);
        var utils = _dereq_(35);
        var Promise = utils.Promise;
        var EventEmitter = _dereq_(39).EventEmitter;
        PouchDB.adapters = {};
        PouchDB.preferredAdapters = _dereq_(9);
        PouchDB.prefix = '_pouch_';
        var eventEmitter = new EventEmitter();
        var eventEmitterMethods = [
            'on',
            'addListener',
            'emit',
            'listeners',
            'once',
            'removeAllListeners',
            'removeListener',
            'setMaxListeners'
          ];
        eventEmitterMethods.forEach(function (method) {
          PouchDB[method] = eventEmitter[method].bind(eventEmitter);
        });
        PouchDB.setMaxListeners(0);
        PouchDB.parseAdapter = function (name, opts) {
          var match = name.match(/([a-z\-]*):\/\/(.*)/);
          var adapter, adapterName;
          if (match) {
            // the http adapter expects the fully qualified name
            name = /http(s?)/.test(match[1]) ? match[1] + '://' + match[2] : match[2];
            adapter = match[1];
            if (!PouchDB.adapters[adapter].valid()) {
              throw 'Invalid adapter';
            }
            return {
              name: name,
              adapter: match[1]
            };
          }
          // check for browsers that have been upgraded from websql-only to websql+idb
          var skipIdb = 'idb' in PouchDB.adapters && 'websql' in PouchDB.adapters && utils.hasLocalStorage() && localStorage['_pouch__websqldb_' + PouchDB.prefix + name];
          if (typeof opts !== 'undefined' && opts.db) {
            adapterName = 'leveldb';
          } else {
            for (var i = 0; i < PouchDB.preferredAdapters.length; ++i) {
              adapterName = PouchDB.preferredAdapters[i];
              if (adapterName in PouchDB.adapters) {
                if (skipIdb && adapterName === 'idb') {
                  continue;  // keep using websql to avoid user data loss
                }
                break;
              }
            }
          }
          adapter = PouchDB.adapters[adapterName];
          if (adapterName && adapter) {
            var use_prefix = 'use_prefix' in adapter ? adapter.use_prefix : true;
            return {
              name: use_prefix ? PouchDB.prefix + name : name,
              adapter: adapterName
            };
          }
          throw 'No valid adapter found';
        };
        PouchDB.destroy = utils.toPromise(function (name, opts, callback) {
          if (typeof opts === 'function' || typeof opts === 'undefined') {
            callback = opts;
            opts = {};
          }
          if (name && typeof name === 'object') {
            opts = name;
            name = undefined;
          }
          if (!opts.internal) {
            console.log('PouchDB.destroy() is deprecated and will be removed. ' + 'Please use db.destroy() instead.');
          }
          var backend = PouchDB.parseAdapter(opts.name || name, opts);
          var dbName = backend.name;
          var adapter = PouchDB.adapters[backend.adapter];
          var usePrefix = 'use_prefix' in adapter ? adapter.use_prefix : true;
          var baseName = usePrefix ? dbName.replace(new RegExp('^' + PouchDB.prefix), '') : dbName;
          var fullName = (backend.adapter === 'http' || backend.adapter === 'https' ? '' : opts.prefix || '') + dbName;
          function destroyDb() {
            // call destroy method of the particular adaptor
            adapter.destroy(fullName, opts, function (err, resp) {
              if (err) {
                callback(err);
              } else {
                PouchDB.emit('destroyed', name);
                //so we don't have to sift through all dbnames
                PouchDB.emit(name, 'destroyed');
                callback(null, resp || { 'ok': true });
              }
            });
          }
          var createOpts = utils.extend(true, {}, opts, { adapter: backend.adapter });
          new PouchDB(baseName, createOpts, function (err, db) {
            if (err) {
              return callback(err);
            }
            db.get('_local/_pouch_dependentDbs', function (err, localDoc) {
              if (err) {
                if (err.status !== 404) {
                  return callback(err);
                } else {
                  // no dependencies
                  return destroyDb();
                }
              }
              var dependentDbs = localDoc.dependentDbs;
              var deletedMap = Object.keys(dependentDbs).map(function (name) {
                  var trueName = usePrefix ? name.replace(new RegExp('^' + PouchDB.prefix), '') : name;
                  var subOpts = utils.extend(true, opts, db.__opts || {});
                  return db.constructor.destroy(trueName, subOpts);
                });
              Promise.all(deletedMap).then(destroyDb, function (error) {
                callback(error);
              });
            });
          });
        });
        PouchDB.adapter = function (id, obj) {
          if (obj.valid()) {
            PouchDB.adapters[id] = obj;
          }
        };
        PouchDB.plugin = function (obj) {
          Object.keys(obj).forEach(function (id) {
            PouchDB.prototype[id] = obj[id];
          });
        };
        PouchDB.defaults = function (defaultOpts) {
          function PouchAlt(name, opts, callback) {
            if (typeof opts === 'function' || typeof opts === 'undefined') {
              callback = opts;
              opts = {};
            }
            if (name && typeof name === 'object') {
              opts = name;
              name = undefined;
            }
            opts = utils.extend(true, {}, defaultOpts, opts);
            PouchDB.call(this, name, opts, callback);
          }
          utils.inherits(PouchAlt, PouchDB);
          PouchAlt.destroy = utils.toPromise(function (name, opts, callback) {
            if (typeof opts === 'function' || typeof opts === 'undefined') {
              callback = opts;
              opts = {};
            }
            if (name && typeof name === 'object') {
              opts = name;
              name = undefined;
            }
            opts = utils.extend(true, {}, defaultOpts, opts);
            return PouchDB.destroy(name, opts, callback);
          });
          eventEmitterMethods.forEach(function (method) {
            PouchAlt[method] = eventEmitter[method].bind(eventEmitter);
          });
          PouchAlt.setMaxListeners(0);
          PouchAlt.preferredAdapters = PouchDB.preferredAdapters.slice();
          Object.keys(PouchDB).forEach(function (key) {
            if (!(key in PouchAlt)) {
              PouchAlt[key] = PouchDB[key];
            }
          });
          return PouchAlt;
        };
        module.exports = PouchDB;
      },
      {
        '16': 16,
        '35': 35,
        '39': 39,
        '9': 9
      }
    ],
    33: [
      function (_dereq_, module, exports) {
        'use strict';
        var utils = _dereq_(35);
        var replication = _dereq_(31);
        var replicate = replication.replicate;
        var EE = _dereq_(39).EventEmitter;
        utils.inherits(Sync, EE);
        module.exports = sync;
        function sync(src, target, opts, callback) {
          if (typeof opts === 'function') {
            callback = opts;
            opts = {};
          }
          if (typeof opts === 'undefined') {
            opts = {};
          }
          opts = utils.clone(opts);
          /*jshint validthis:true */
          opts.PouchConstructor = opts.PouchConstructor || this;
          src = replication.toPouch(src, opts);
          target = replication.toPouch(target, opts);
          return new Sync(src, target, opts, callback);
        }
        function Sync(src, target, opts, callback) {
          var self = this;
          this.canceled = false;
          var onChange, complete;
          if ('onChange' in opts) {
            onChange = opts.onChange;
            delete opts.onChange;
          }
          if (typeof callback === 'function' && !opts.complete) {
            complete = callback;
          } else if ('complete' in opts) {
            complete = opts.complete;
            delete opts.complete;
          }
          this.push = replicate(src, target, opts);
          this.pull = replicate(target, src, opts);
          var emittedCancel = false;
          function onCancel(data) {
            if (!emittedCancel) {
              emittedCancel = true;
              self.emit('cancel', data);
            }
          }
          function pullChange(change) {
            self.emit('change', {
              direction: 'pull',
              change: change
            });
          }
          function pushChange(change) {
            self.emit('change', {
              direction: 'push',
              change: change
            });
          }
          function pushDenied(doc) {
            self.emit('denied', {
              direction: 'push',
              doc: doc
            });
          }
          function pullDenied(doc) {
            self.emit('denied', {
              direction: 'pull',
              doc: doc
            });
          }
          var listeners = {};
          var removed = {};
          function removeAll(type) {
            // type is 'push' or 'pull'
            return function (event, func) {
              var isChange = event === 'change' && (func === pullChange || func === pushChange);
              var isCancel = event === 'cancel' && func === onCancel;
              var isOtherEvent = event in listeners && func === listeners[event];
              if (isChange || isCancel || isOtherEvent) {
                if (!(event in removed)) {
                  removed[event] = {};
                }
                removed[event][type] = true;
                if (Object.keys(removed[event]).length === 2) {
                  // both push and pull have asked to be removed
                  self.removeAllListeners(event);
                }
              }
            };
          }
          if (opts.live) {
            this.push.on('complete', self.pull.cancel.bind(self.pull));
            this.pull.on('complete', self.push.cancel.bind(self.push));
          }
          this.on('newListener', function (event) {
            if (event === 'change') {
              self.pull.on('change', pullChange);
              self.push.on('change', pushChange);
            } else if (event === 'denied') {
              self.pull.on('denied', pullDenied);
              self.push.on('denied', pushDenied);
            } else if (event === 'cancel') {
              self.pull.on('cancel', onCancel);
              self.push.on('cancel', onCancel);
            } else if (event !== 'error' && event !== 'removeListener' && event !== 'complete' && !(event in listeners)) {
              listeners[event] = function (e) {
                self.emit(event, e);
              };
              self.pull.on(event, listeners[event]);
              self.push.on(event, listeners[event]);
            }
          });
          this.on('removeListener', function (event) {
            if (event === 'change') {
              self.pull.removeListener('change', pullChange);
              self.push.removeListener('change', pushChange);
            } else if (event === 'cancel') {
              self.pull.removeListener('cancel', onCancel);
              self.push.removeListener('cancel', onCancel);
            } else if (event in listeners) {
              if (typeof listeners[event] === 'function') {
                self.pull.removeListener(event, listeners[event]);
                self.push.removeListener(event, listeners[event]);
                delete listeners[event];
              }
            }
          });
          this.pull.on('removeListener', removeAll('pull'));
          this.push.on('removeListener', removeAll('push'));
          var promise = utils.Promise.all([
              this.push,
              this.pull
            ]).then(function (resp) {
              var out = {
                  push: resp[0],
                  pull: resp[1]
                };
              self.emit('complete', out);
              if (complete) {
                complete(null, out);
              }
              self.removeAllListeners();
              return out;
            }, function (err) {
              self.cancel();
              self.emit('error', err);
              if (complete) {
                complete(err);
              }
              self.removeAllListeners();
              throw err;
            });
          this.then = function (success, err) {
            return promise.then(success, err);
          };
          this['catch'] = function (err) {
            return promise['catch'](err);
          };
        }
        Sync.prototype.cancel = function () {
          if (!this.canceled) {
            this.canceled = true;
            this.push.cancel();
            this.pull.cancel();
          }
        };
      },
      {
        '31': 31,
        '35': 35,
        '39': 39
      }
    ],
    34: [
      function (_dereq_, module, exports) {
        'use strict';
        module.exports = TaskQueue;
        function TaskQueue() {
          this.isReady = false;
          this.failed = false;
          this.queue = [];
        }
        TaskQueue.prototype.execute = function () {
          var d, func;
          if (this.failed) {
            while (d = this.queue.shift()) {
              if (typeof d === 'function') {
                d(this.failed);
                continue;
              }
              func = d.parameters[d.parameters.length - 1];
              if (typeof func === 'function') {
                func(this.failed);
              } else if (d.name === 'changes' && typeof func.complete === 'function') {
                func.complete(this.failed);
              }
            }
          } else if (this.isReady) {
            while (d = this.queue.shift()) {
              if (typeof d === 'function') {
                d();
              } else {
                d.task = this.db[d.name].apply(this.db, d.parameters);
              }
            }
          }
        };
        TaskQueue.prototype.fail = function (err) {
          this.failed = err;
          this.execute();
        };
        TaskQueue.prototype.ready = function (db) {
          if (this.failed) {
            return false;
          } else if (arguments.length === 0) {
            return this.isReady;
          }
          this.isReady = db ? true : false;
          this.db = db;
          this.execute();
        };
        TaskQueue.prototype.addTask = function (name, parameters) {
          if (typeof name === 'function') {
            this.queue.push(name);
            if (this.failed) {
              this.execute();
            }
          } else {
            var task = {
                name: name,
                parameters: parameters
              };
            this.queue.push(task);
            if (this.failed) {
              this.execute();
            }
            return task;
          }
        };
      },
      {}
    ],
    35: [
      function (_dereq_, module, exports) {
        (function (process, global) {
          /*jshint strict: false */
          /*global chrome */
          var merge = _dereq_(30);
          exports.extend = _dereq_(66);
          exports.ajax = _dereq_(17);
          exports.createBlob = _dereq_(18);
          exports.uuid = _dereq_(27);
          exports.getArguments = _dereq_(37);
          var buffer = _dereq_(19);
          var errors = _dereq_(20);
          var EventEmitter = _dereq_(39).EventEmitter;
          var collections = _dereq_(65);
          exports.Map = collections.Map;
          exports.Set = collections.Set;
          var parseDoc = _dereq_(22);
          if (typeof global.Promise === 'function') {
            exports.Promise = global.Promise;
          } else {
            exports.Promise = _dereq_(48);
          }
          var Promise = exports.Promise;
          exports.lastIndexOf = function (str, char) {
            for (var i = str.length - 1; i >= 0; i--) {
              if (str.charAt(i) === char) {
                return i;
              }
            }
            return -1;
          };
          exports.clone = function (obj) {
            return exports.extend(true, {}, obj);
          };
          // like underscore/lodash _.pick()
          exports.pick = function (obj, arr) {
            var res = {};
            for (var i = 0, len = arr.length; i < len; i++) {
              var prop = arr[i];
              res[prop] = obj[prop];
            }
            return res;
          };
          exports.inherits = _dereq_(44);
          function isChromeApp() {
            return typeof chrome !== 'undefined' && typeof chrome.storage !== 'undefined' && typeof chrome.storage.local !== 'undefined';
          }
          // Pretty dumb name for a function, just wraps callback calls so we dont
          // to if (callback) callback() everywhere
          exports.call = exports.getArguments(function (args) {
            if (!args.length) {
              return;
            }
            var fun = args.shift();
            if (typeof fun === 'function') {
              fun.apply(this, args);
            }
          });
          exports.isLocalId = function (id) {
            return /^_local/.test(id);
          };
          // check if a specific revision of a doc has been deleted
          //  - metadata: the metadata object from the doc store
          //  - rev: (optional) the revision to check. defaults to winning revision
          exports.isDeleted = function (metadata, rev) {
            if (!rev) {
              rev = merge.winningRev(metadata);
            }
            var dashIndex = rev.indexOf('-');
            if (dashIndex !== -1) {
              rev = rev.substring(dashIndex + 1);
            }
            var deleted = false;
            merge.traverseRevTree(metadata.rev_tree, function (isLeaf, pos, id, acc, opts) {
              if (id === rev) {
                deleted = !!opts.deleted;
              }
            });
            return deleted;
          };
          exports.revExists = function (metadata, rev) {
            var found = false;
            merge.traverseRevTree(metadata.rev_tree, function (leaf, pos, id) {
              if (pos + '-' + id === rev) {
                found = true;
              }
            });
            return found;
          };
          exports.filterChange = function filterChange(opts) {
            var req = {};
            var hasFilter = opts.filter && typeof opts.filter === 'function';
            req.query = opts.query_params;
            return function filter(change) {
              if (!change.doc) {
                // CSG sends events on the changes feed that don't have documents,
                // this hack makes a whole lot of existing code robust.
                change.doc = {};
              }
              if (opts.filter && hasFilter && !opts.filter.call(this, change.doc, req)) {
                return false;
              }
              if (!opts.include_docs) {
                delete change.doc;
              } else if (!opts.attachments) {
                for (var att in change.doc._attachments) {
                  if (change.doc._attachments.hasOwnProperty(att)) {
                    change.doc._attachments[att].stub = true;
                  }
                }
              }
              return true;
            };
          };
          exports.parseDoc = parseDoc.parseDoc;
          exports.invalidIdError = parseDoc.invalidIdError;
          exports.isCordova = function () {
            return typeof cordova !== 'undefined' || typeof PhoneGap !== 'undefined' || typeof phonegap !== 'undefined';
          };
          exports.hasLocalStorage = function () {
            if (isChromeApp()) {
              return false;
            }
            try {
              return localStorage;
            } catch (e) {
              return false;
            }
          };
          exports.Changes = Changes;
          exports.inherits(Changes, EventEmitter);
          function Changes() {
            if (!(this instanceof Changes)) {
              return new Changes();
            }
            var self = this;
            EventEmitter.call(this);
            this.isChrome = isChromeApp();
            this.listeners = {};
            this.hasLocal = false;
            if (!this.isChrome) {
              this.hasLocal = exports.hasLocalStorage();
            }
            if (this.isChrome) {
              chrome.storage.onChanged.addListener(function (e) {
                // make sure it's event addressed to us
                if (e.db_name != null) {
                  //object only has oldValue, newValue members
                  self.emit(e.dbName.newValue);
                }
              });
            } else if (this.hasLocal) {
              if (typeof addEventListener !== 'undefined') {
                addEventListener('storage', function (e) {
                  self.emit(e.key);
                });
              } else {
                // old IE
                window.attachEvent('storage', function (e) {
                  self.emit(e.key);
                });
              }
            }
          }
          Changes.prototype.addListener = function (dbName, id, db, opts) {
            if (this.listeners[id]) {
              return;
            }
            var self = this;
            var inprogress = false;
            function eventFunction() {
              if (!self.listeners[id]) {
                return;
              }
              if (inprogress) {
                inprogress = 'waiting';
                return;
              }
              inprogress = true;
              db.changes({
                style: opts.style,
                include_docs: opts.include_docs,
                attachments: opts.attachments,
                conflicts: opts.conflicts,
                continuous: false,
                descending: false,
                filter: opts.filter,
                doc_ids: opts.doc_ids,
                view: opts.view,
                since: opts.since,
                query_params: opts.query_params
              }).on('change', function (c) {
                if (c.seq > opts.since && !opts.cancelled) {
                  opts.since = c.seq;
                  exports.call(opts.onChange, c);
                }
              }).on('complete', function () {
                if (inprogress === 'waiting') {
                  process.nextTick(function () {
                    self.notify(dbName);
                  });
                }
                inprogress = false;
              }).on('error', function () {
                inprogress = false;
              });
            }
            this.listeners[id] = eventFunction;
            this.on(dbName, eventFunction);
          };
          Changes.prototype.removeListener = function (dbName, id) {
            if (!(id in this.listeners)) {
              return;
            }
            EventEmitter.prototype.removeListener.call(this, dbName, this.listeners[id]);
          };
          Changes.prototype.notifyLocalWindows = function (dbName) {
            //do a useless change on a storage thing
            //in order to get other windows's listeners to activate
            if (this.isChrome) {
              chrome.storage.local.set({ dbName: dbName });
            } else if (this.hasLocal) {
              localStorage[dbName] = localStorage[dbName] === 'a' ? 'b' : 'a';
            }
          };
          Changes.prototype.notify = function (dbName) {
            this.emit(dbName);
            this.notifyLocalWindows(dbName);
          };
          if (typeof atob === 'function') {
            exports.atob = function (str) {
              return atob(str);
            };
          } else {
            exports.atob = function (str) {
              var base64 = new buffer(str, 'base64');
              // Node.js will just skip the characters it can't encode instead of
              // throwing and exception
              if (base64.toString('base64') !== str) {
                throw 'Cannot base64 encode full string';
              }
              return base64.toString('binary');
            };
          }
          if (typeof btoa === 'function') {
            exports.btoa = function (str) {
              return btoa(str);
            };
          } else {
            exports.btoa = function (str) {
              return new buffer(str, 'binary').toString('base64');
            };
          }
          // From http://stackoverflow.com/questions/14967647/ (continues on next line)
          // encode-decode-image-with-base64-breaks-image (2013-04-21)
          exports.fixBinary = function (bin) {
            if (!process.browser) {
              // don't need to do this in Node
              return bin;
            }
            var length = bin.length;
            var buf = new ArrayBuffer(length);
            var arr = new Uint8Array(buf);
            for (var i = 0; i < length; i++) {
              arr[i] = bin.charCodeAt(i);
            }
            return buf;
          };
          // shim for browsers that don't support it
          exports.readAsBinaryString = function (blob, callback) {
            var reader = new FileReader();
            var hasBinaryString = typeof reader.readAsBinaryString === 'function';
            reader.onloadend = function (e) {
              var result = e.target.result || '';
              if (hasBinaryString) {
                return callback(result);
              }
              callback(exports.arrayBufferToBinaryString(result));
            };
            if (hasBinaryString) {
              reader.readAsBinaryString(blob);
            } else {
              reader.readAsArrayBuffer(blob);
            }
          };
          // simplified API. universal browser support is assumed
          exports.readAsArrayBuffer = function (blob, callback) {
            var reader = new FileReader();
            reader.onloadend = function (e) {
              var result = e.target.result || new ArrayBuffer(0);
              callback(result);
            };
            reader.readAsArrayBuffer(blob);
          };
          exports.once = function (fun) {
            var called = false;
            return exports.getArguments(function (args) {
              if (called) {
                throw new Error('once called  more than once');
              } else {
                called = true;
                fun.apply(this, args);
              }
            });
          };
          exports.toPromise = function (func) {
            //create the function we will be returning
            return exports.getArguments(function (args) {
              var self = this;
              var tempCB = typeof args[args.length - 1] === 'function' ? args.pop() : false;
              // if the last argument is a function, assume its a callback
              var usedCB;
              if (tempCB) {
                // if it was a callback, create a new callback which calls it,
                // but do so async so we don't trap any errors
                usedCB = function (err, resp) {
                  process.nextTick(function () {
                    tempCB(err, resp);
                  });
                };
              }
              var promise = new Promise(function (fulfill, reject) {
                  var resp;
                  try {
                    var callback = exports.once(function (err, mesg) {
                        if (err) {
                          reject(err);
                        } else {
                          fulfill(mesg);
                        }
                      });
                    // create a callback for this invocation
                    // apply the function in the orig context
                    args.push(callback);
                    resp = func.apply(self, args);
                    if (resp && typeof resp.then === 'function') {
                      fulfill(resp);
                    }
                  } catch (e) {
                    reject(e);
                  }
                });
              // if there is a callback, call it back
              if (usedCB) {
                promise.then(function (result) {
                  usedCB(null, result);
                }, usedCB);
              }
              promise.cancel = function () {
                return this;
              };
              return promise;
            });
          };
          exports.adapterFun = function (name, callback) {
            var log = _dereq_(41)('pouchdb:api');
            function logApiCall(self, name, args) {
              if (!log.enabled) {
                return;
              }
              var logArgs = [
                  self._db_name,
                  name
                ];
              for (var i = 0; i < args.length - 1; i++) {
                logArgs.push(args[i]);
              }
              log.apply(null, logArgs);
              // override the callback itself to log the response
              var origCallback = args[args.length - 1];
              args[args.length - 1] = function (err, res) {
                var responseArgs = [
                    self._db_name,
                    name
                  ];
                responseArgs = responseArgs.concat(err ? [
                  'error',
                  err
                ] : [
                  'success',
                  res
                ]);
                log.apply(null, responseArgs);
                origCallback(err, res);
              };
            }
            return exports.toPromise(exports.getArguments(function (args) {
              if (this._closed) {
                return Promise.reject(new Error('database is closed'));
              }
              var self = this;
              logApiCall(self, name, args);
              if (!this.taskqueue.isReady) {
                return new exports.Promise(function (fulfill, reject) {
                  self.taskqueue.addTask(function (failed) {
                    if (failed) {
                      reject(failed);
                    } else {
                      fulfill(self[name].apply(self, args));
                    }
                  });
                });
              }
              return callback.apply(this, args);
            }));
          };
          //Can't find original post, but this is close
          //http://stackoverflow.com/questions/6965107/ (continues on next line)
          //converting-between-strings-and-arraybuffers
          exports.arrayBufferToBinaryString = function (buffer) {
            var binary = '';
            var bytes = new Uint8Array(buffer);
            var length = bytes.byteLength;
            for (var i = 0; i < length; i++) {
              binary += String.fromCharCode(bytes[i]);
            }
            return binary;
          };
          exports.cancellableFun = function (fun, self, opts) {
            opts = opts ? exports.clone(true, {}, opts) : {};
            var emitter = new EventEmitter();
            var oldComplete = opts.complete || function () {
              };
            var complete = opts.complete = exports.once(function (err, resp) {
                if (err) {
                  oldComplete(err);
                } else {
                  emitter.emit('end', resp);
                  oldComplete(null, resp);
                }
                emitter.removeAllListeners();
              });
            var oldOnChange = opts.onChange || function () {
              };
            var lastChange = 0;
            self.on('destroyed', function () {
              emitter.removeAllListeners();
            });
            opts.onChange = function (change) {
              oldOnChange(change);
              if (change.seq <= lastChange) {
                return;
              }
              lastChange = change.seq;
              emitter.emit('change', change);
              if (change.deleted) {
                emitter.emit('delete', change);
              } else if (change.changes.length === 1 && change.changes[0].rev.slice(0, 1) === '1-') {
                emitter.emit('create', change);
              } else {
                emitter.emit('update', change);
              }
            };
            var promise = new Promise(function (fulfill, reject) {
                opts.complete = function (err, res) {
                  if (err) {
                    reject(err);
                  } else {
                    fulfill(res);
                  }
                };
              });
            promise.then(function (result) {
              complete(null, result);
            }, complete);
            // this needs to be overwridden by caller, dont fire complete until
            // the task is ready
            promise.cancel = function () {
              promise.isCancelled = true;
              if (self.taskqueue.isReady) {
                opts.complete(null, { status: 'cancelled' });
              }
            };
            if (!self.taskqueue.isReady) {
              self.taskqueue.addTask(function () {
                if (promise.isCancelled) {
                  opts.complete(null, { status: 'cancelled' });
                } else {
                  fun(self, opts, promise);
                }
              });
            } else {
              fun(self, opts, promise);
            }
            promise.on = emitter.on.bind(emitter);
            promise.once = emitter.once.bind(emitter);
            promise.addListener = emitter.addListener.bind(emitter);
            promise.removeListener = emitter.removeListener.bind(emitter);
            promise.removeAllListeners = emitter.removeAllListeners.bind(emitter);
            promise.setMaxListeners = emitter.setMaxListeners.bind(emitter);
            promise.listeners = emitter.listeners.bind(emitter);
            promise.emit = emitter.emit.bind(emitter);
            return promise;
          };
          exports.MD5 = exports.toPromise(_dereq_(21));
          // designed to give info to browser users, who are disturbed
          // when they see 404s in the console
          exports.explain404 = function (str) {
            if (process.browser && 'console' in global && 'info' in console) {
              console.info('The above 404 is totally normal. ' + str);
            }
          };
          exports.info = function (str) {
            if (typeof console !== 'undefined' && 'info' in console) {
              console.info(str);
            }
          };
          exports.parseUri = _dereq_(24);
          exports.compare = function (left, right) {
            return left < right ? -1 : left > right ? 1 : 0;
          };
          exports.updateDoc = function updateDoc(prev, docInfo, results, i, cb, writeDoc, newEdits) {
            if (exports.revExists(prev, docInfo.metadata.rev)) {
              results[i] = docInfo;
              return cb();
            }
            // TODO: some of these can be pre-calculated, but it's safer to just
            // call merge.winningRev() and exports.isDeleted() all over again
            var previousWinningRev = merge.winningRev(prev);
            var previouslyDeleted = exports.isDeleted(prev, previousWinningRev);
            var deleted = exports.isDeleted(docInfo.metadata);
            var isRoot = /^1-/.test(docInfo.metadata.rev);
            if (previouslyDeleted && !deleted && newEdits && isRoot) {
              var newDoc = docInfo.data;
              newDoc._rev = previousWinningRev;
              newDoc._id = docInfo.metadata.id;
              docInfo = exports.parseDoc(newDoc, newEdits);
            }
            var merged = merge.merge(prev.rev_tree, docInfo.metadata.rev_tree[0], 1000);
            var inConflict = newEdits && (previouslyDeleted && deleted || !previouslyDeleted && merged.conflicts !== 'new_leaf' || previouslyDeleted && !deleted && merged.conflicts === 'new_branch');
            if (inConflict) {
              var err = errors.error(errors.REV_CONFLICT);
              results[i] = err;
              return cb();
            }
            var newRev = docInfo.metadata.rev;
            docInfo.metadata.rev_tree = merged.tree;
            if (prev.rev_map) {
              docInfo.metadata.rev_map = prev.rev_map;  // used by leveldb
            }
            // recalculate
            var winningRev = merge.winningRev(docInfo.metadata);
            var winningRevIsDeleted = exports.isDeleted(docInfo.metadata, winningRev);
            // calculate the total number of documents that were added/removed,
            // from the perspective of total_rows/doc_count
            var delta = previouslyDeleted === winningRevIsDeleted ? 0 : previouslyDeleted < winningRevIsDeleted ? -1 : 1;
            var newRevIsDeleted = exports.isDeleted(docInfo.metadata, newRev);
            writeDoc(docInfo, winningRev, winningRevIsDeleted, newRevIsDeleted, true, delta, i, cb);
          };
          exports.processDocs = function processDocs(docInfos, api, fetchedDocs, tx, results, writeDoc, opts, overallCallback) {
            if (!docInfos.length) {
              return;
            }
            function insertDoc(docInfo, resultsIdx, callback) {
              // Cant insert new deleted documents
              var winningRev = merge.winningRev(docInfo.metadata);
              var deleted = exports.isDeleted(docInfo.metadata, winningRev);
              if ('was_delete' in opts && deleted) {
                results[resultsIdx] = errors.error(errors.MISSING_DOC, 'deleted');
                return callback();
              }
              var delta = deleted ? 0 : 1;
              writeDoc(docInfo, winningRev, deleted, deleted, false, delta, resultsIdx, callback);
            }
            var newEdits = opts.new_edits;
            var idsToDocs = new exports.Map();
            var docsDone = 0;
            var docsToDo = docInfos.length;
            function checkAllDocsDone() {
              if (++docsDone === docsToDo && overallCallback) {
                overallCallback();
              }
            }
            docInfos.forEach(function (currentDoc, resultsIdx) {
              if (currentDoc._id && exports.isLocalId(currentDoc._id)) {
                api[currentDoc._deleted ? '_removeLocal' : '_putLocal'](currentDoc, { ctx: tx }, function (err) {
                  if (err) {
                    results[resultsIdx] = err;
                  } else {
                    results[resultsIdx] = { ok: true };
                  }
                  checkAllDocsDone();
                });
                return;
              }
              var id = currentDoc.metadata.id;
              if (idsToDocs.has(id)) {
                docsToDo--;
                // duplicate
                idsToDocs.get(id).push([
                  currentDoc,
                  resultsIdx
                ]);
              } else {
                idsToDocs.set(id, [[
                    currentDoc,
                    resultsIdx
                  ]]);
              }
            });
            // in the case of new_edits, the user can provide multiple docs
            // with the same id. these need to be processed sequentially
            idsToDocs.forEach(function (docs, id) {
              var numDone = 0;
              function docWritten() {
                if (++numDone < docs.length) {
                  nextDoc();
                } else {
                  checkAllDocsDone();
                }
              }
              function nextDoc() {
                var value = docs[numDone];
                var currentDoc = value[0];
                var resultsIdx = value[1];
                if (fetchedDocs.has(id)) {
                  exports.updateDoc(fetchedDocs.get(id), currentDoc, results, resultsIdx, docWritten, writeDoc, newEdits);
                } else {
                  insertDoc(currentDoc, resultsIdx, docWritten);
                }
              }
              nextDoc();
            });
          };
          exports.preprocessAttachments = function preprocessAttachments(docInfos, blobType, callback) {
            if (!docInfos.length) {
              return callback();
            }
            var docv = 0;
            function parseBase64(data) {
              try {
                return exports.atob(data);
              } catch (e) {
                var err = errors.error(errors.BAD_ARG, 'Attachments need to be base64 encoded');
                return { error: err };
              }
            }
            function preprocessAttachment(att, callback) {
              if (att.stub) {
                return callback();
              }
              if (typeof att.data === 'string') {
                // input is a base64 string
                var asBinary = parseBase64(att.data);
                if (asBinary.error) {
                  return callback(asBinary.error);
                }
                att.length = asBinary.length;
                if (blobType === 'blob') {
                  att.data = exports.createBlob([exports.fixBinary(asBinary)], { type: att.content_type });
                } else if (blobType === 'base64') {
                  att.data = exports.btoa(asBinary);
                } else {
                  // binary
                  att.data = asBinary;
                }
                exports.MD5(asBinary).then(function (result) {
                  att.digest = 'md5-' + result;
                  callback();
                });
              } else {
                // input is a blob
                exports.readAsArrayBuffer(att.data, function (buff) {
                  if (blobType === 'binary') {
                    att.data = exports.arrayBufferToBinaryString(buff);
                  } else if (blobType === 'base64') {
                    att.data = exports.btoa(exports.arrayBufferToBinaryString(buff));
                  }
                  exports.MD5(buff).then(function (result) {
                    att.digest = 'md5-' + result;
                    att.length = buff.byteLength;
                    callback();
                  });
                });
              }
            }
            var overallErr;
            docInfos.forEach(function (docInfo) {
              var attachments = docInfo.data && docInfo.data._attachments ? Object.keys(docInfo.data._attachments) : [];
              var recv = 0;
              if (!attachments.length) {
                return done();
              }
              function processedAttachment(err) {
                overallErr = err;
                recv++;
                if (recv === attachments.length) {
                  done();
                }
              }
              for (var key in docInfo.data._attachments) {
                if (docInfo.data._attachments.hasOwnProperty(key)) {
                  preprocessAttachment(docInfo.data._attachments[key], processedAttachment);
                }
              }
            });
            function done() {
              docv++;
              if (docInfos.length === docv) {
                if (overallErr) {
                  callback(overallErr);
                } else {
                  callback();
                }
              }
            }
          };
          // compact a tree by marking its non-leafs as missing,
          // and return a list of revs to delete
          exports.compactTree = function compactTree(metadata) {
            var revs = [];
            merge.traverseRevTree(metadata.rev_tree, function (isLeaf, pos, revHash, ctx, opts) {
              if (opts.status === 'available' && !isLeaf) {
                revs.push(pos + '-' + revHash);
                opts.status = 'missing';
              }
            });
            return revs;
          };
          var vuvuzela = _dereq_(75);
          exports.safeJsonParse = function safeJsonParse(str) {
            try {
              return JSON.parse(str);
            } catch (e) {
              return vuvuzela.parse(str);
            }
          };
          exports.safeJsonStringify = function safeJsonStringify(json) {
            try {
              return JSON.stringify(json);
            } catch (e) {
              return vuvuzela.stringify(json);
            }
          };
        }.call(this, _dereq_(40), typeof global !== 'undefined' ? global : typeof self !== 'undefined' ? self : typeof window !== 'undefined' ? window : {}));
      },
      {
        '17': 17,
        '18': 18,
        '19': 19,
        '20': 20,
        '21': 21,
        '22': 22,
        '24': 24,
        '27': 27,
        '30': 30,
        '37': 37,
        '39': 39,
        '40': 40,
        '41': 41,
        '44': 44,
        '48': 48,
        '65': 65,
        '66': 66,
        '75': 75
      }
    ],
    36: [
      function (_dereq_, module, exports) {
        module.exports = '3.4.0';
      },
      {}
    ],
    37: [
      function (_dereq_, module, exports) {
        'use strict';
        module.exports = argsArray;
        function argsArray(fun) {
          return function () {
            var len = arguments.length;
            if (len) {
              var args = [];
              var i = -1;
              while (++i < len) {
                args[i] = arguments[i];
              }
              return fun.call(this, args);
            } else {
              return fun.call(this, []);
            }
          };
        }
      },
      {}
    ],
    38: [
      function (_dereq_, module, exports) {
      },
      {}
    ],
    39: [
      function (_dereq_, module, exports) {
        // Copyright Joyent, Inc. and other Node contributors.
        //
        // Permission is hereby granted, free of charge, to any person obtaining a
        // copy of this software and associated documentation files (the
        // "Software"), to deal in the Software without restriction, including
        // without limitation the rights to use, copy, modify, merge, publish,
        // distribute, sublicense, and/or sell copies of the Software, and to permit
        // persons to whom the Software is furnished to do so, subject to the
        // following conditions:
        //
        // The above copyright notice and this permission notice shall be included
        // in all copies or substantial portions of the Software.
        //
        // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
        // OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
        // MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
        // NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
        // DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
        // OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
        // USE OR OTHER DEALINGS IN THE SOFTWARE.
        function EventEmitter() {
          this._events = this._events || {};
          this._maxListeners = this._maxListeners || undefined;
        }
        module.exports = EventEmitter;
        // Backwards-compat with node 0.10.x
        EventEmitter.EventEmitter = EventEmitter;
        EventEmitter.prototype._events = undefined;
        EventEmitter.prototype._maxListeners = undefined;
        // By default EventEmitters will print a warning if more than 10 listeners are
        // added to it. This is a useful default which helps finding memory leaks.
        EventEmitter.defaultMaxListeners = 10;
        // Obviously not all Emitters should be limited to 10. This function allows
        // that to be increased. Set to zero for unlimited.
        EventEmitter.prototype.setMaxListeners = function (n) {
          if (!isNumber(n) || n < 0 || isNaN(n))
            throw TypeError('n must be a positive number');
          this._maxListeners = n;
          return this;
        };
        EventEmitter.prototype.emit = function (type) {
          var er, handler, len, args, i, listeners;
          if (!this._events)
            this._events = {};
          // If there is no 'error' event listener then throw.
          if (type === 'error') {
            if (!this._events.error || isObject(this._events.error) && !this._events.error.length) {
              er = arguments[1];
              if (er instanceof Error) {
                throw er;  // Unhandled 'error' event
              }
              throw TypeError('Uncaught, unspecified "error" event.');
            }
          }
          handler = this._events[type];
          if (isUndefined(handler))
            return false;
          if (isFunction(handler)) {
            switch (arguments.length) {
            // fast cases
            case 1:
              handler.call(this);
              break;
            case 2:
              handler.call(this, arguments[1]);
              break;
            case 3:
              handler.call(this, arguments[1], arguments[2]);
              break;
            // slower
            default:
              len = arguments.length;
              args = new Array(len - 1);
              for (i = 1; i < len; i++)
                args[i - 1] = arguments[i];
              handler.apply(this, args);
            }
          } else if (isObject(handler)) {
            len = arguments.length;
            args = new Array(len - 1);
            for (i = 1; i < len; i++)
              args[i - 1] = arguments[i];
            listeners = handler.slice();
            len = listeners.length;
            for (i = 0; i < len; i++)
              listeners[i].apply(this, args);
          }
          return true;
        };
        EventEmitter.prototype.addListener = function (type, listener) {
          var m;
          if (!isFunction(listener))
            throw TypeError('listener must be a function');
          if (!this._events)
            this._events = {};
          // To avoid recursion in the case that type === "newListener"! Before
          // adding it to the listeners, first emit "newListener".
          if (this._events.newListener)
            this.emit('newListener', type, isFunction(listener.listener) ? listener.listener : listener);
          if (!this._events[type])
            // Optimize the case of one listener. Don't need the extra array object.
            this._events[type] = listener;
          else if (isObject(this._events[type]))
            // If we've already got an array, just append.
            this._events[type].push(listener);
          else
            // Adding the second element, need to change to array.
            this._events[type] = [
              this._events[type],
              listener
            ];
          // Check for listener leak
          if (isObject(this._events[type]) && !this._events[type].warned) {
            var m;
            if (!isUndefined(this._maxListeners)) {
              m = this._maxListeners;
            } else {
              m = EventEmitter.defaultMaxListeners;
            }
            if (m && m > 0 && this._events[type].length > m) {
              this._events[type].warned = true;
              console.error('(node) warning: possible EventEmitter memory ' + 'leak detected. %d listeners added. ' + 'Use emitter.setMaxListeners() to increase limit.', this._events[type].length);
              if (typeof console.trace === 'function') {
                // not supported in IE 10
                console.trace();
              }
            }
          }
          return this;
        };
        EventEmitter.prototype.on = EventEmitter.prototype.addListener;
        EventEmitter.prototype.once = function (type, listener) {
          if (!isFunction(listener))
            throw TypeError('listener must be a function');
          var fired = false;
          function g() {
            this.removeListener(type, g);
            if (!fired) {
              fired = true;
              listener.apply(this, arguments);
            }
          }
          g.listener = listener;
          this.on(type, g);
          return this;
        };
        // emits a 'removeListener' event iff the listener was removed
        EventEmitter.prototype.removeListener = function (type, listener) {
          var list, position, length, i;
          if (!isFunction(listener))
            throw TypeError('listener must be a function');
          if (!this._events || !this._events[type])
            return this;
          list = this._events[type];
          length = list.length;
          position = -1;
          if (list === listener || isFunction(list.listener) && list.listener === listener) {
            delete this._events[type];
            if (this._events.removeListener)
              this.emit('removeListener', type, listener);
          } else if (isObject(list)) {
            for (i = length; i-- > 0;) {
              if (list[i] === listener || list[i].listener && list[i].listener === listener) {
                position = i;
                break;
              }
            }
            if (position < 0)
              return this;
            if (list.length === 1) {
              list.length = 0;
              delete this._events[type];
            } else {
              list.splice(position, 1);
            }
            if (this._events.removeListener)
              this.emit('removeListener', type, listener);
          }
          return this;
        };
        EventEmitter.prototype.removeAllListeners = function (type) {
          var key, listeners;
          if (!this._events)
            return this;
          // not listening for removeListener, no need to emit
          if (!this._events.removeListener) {
            if (arguments.length === 0)
              this._events = {};
            else if (this._events[type])
              delete this._events[type];
            return this;
          }
          // emit removeListener for all listeners on all events
          if (arguments.length === 0) {
            for (key in this._events) {
              if (key === 'removeListener')
                continue;
              this.removeAllListeners(key);
            }
            this.removeAllListeners('removeListener');
            this._events = {};
            return this;
          }
          listeners = this._events[type];
          if (isFunction(listeners)) {
            this.removeListener(type, listeners);
          } else {
            // LIFO order
            while (listeners.length)
              this.removeListener(type, listeners[listeners.length - 1]);
          }
          delete this._events[type];
          return this;
        };
        EventEmitter.prototype.listeners = function (type) {
          var ret;
          if (!this._events || !this._events[type])
            ret = [];
          else if (isFunction(this._events[type]))
            ret = [this._events[type]];
          else
            ret = this._events[type].slice();
          return ret;
        };
        EventEmitter.listenerCount = function (emitter, type) {
          var ret;
          if (!emitter._events || !emitter._events[type])
            ret = 0;
          else if (isFunction(emitter._events[type]))
            ret = 1;
          else
            ret = emitter._events[type].length;
          return ret;
        };
        function isFunction(arg) {
          return typeof arg === 'function';
        }
        function isNumber(arg) {
          return typeof arg === 'number';
        }
        function isObject(arg) {
          return typeof arg === 'object' && arg !== null;
        }
        function isUndefined(arg) {
          return arg === void 0;
        }
      },
      {}
    ],
    40: [
      function (_dereq_, module, exports) {
        // shim for using process in browser
        var process = module.exports = {};
        process.nextTick = function () {
          var canSetImmediate = typeof window !== 'undefined' && window.setImmediate;
          var canMutationObserver = typeof window !== 'undefined' && window.MutationObserver;
          var canPost = typeof window !== 'undefined' && window.postMessage && window.addEventListener;
          ;
          if (canSetImmediate) {
            return function (f) {
              return window.setImmediate(f);
            };
          }
          var queue = [];
          if (canMutationObserver) {
            var hiddenDiv = document.createElement('div');
            var observer = new MutationObserver(function () {
                var queueList = queue.slice();
                queue.length = 0;
                queueList.forEach(function (fn) {
                  fn();
                });
              });
            observer.observe(hiddenDiv, { attributes: true });
            return function nextTick(fn) {
              if (!queue.length) {
                hiddenDiv.setAttribute('yes', 'no');
              }
              queue.push(fn);
            };
          }
          if (canPost) {
            window.addEventListener('message', function (ev) {
              var source = ev.source;
              if ((source === window || source === null) && ev.data === 'process-tick') {
                ev.stopPropagation();
                if (queue.length > 0) {
                  var fn = queue.shift();
                  fn();
                }
              }
            }, true);
            return function nextTick(fn) {
              queue.push(fn);
              window.postMessage('process-tick', '*');
            };
          }
          return function nextTick(fn) {
            setTimeout(fn, 0);
          };
        }();
        process.title = 'browser';
        process.browser = true;
        process.env = {};
        process.argv = [];
        function noop() {
        }
        process.on = noop;
        process.addListener = noop;
        process.once = noop;
        process.off = noop;
        process.removeListener = noop;
        process.removeAllListeners = noop;
        process.emit = noop;
        process.binding = function (name) {
          throw new Error('process.binding is not supported');
        };
        // TODO(shtylman)
        process.cwd = function () {
          return '/';
        };
        process.chdir = function (dir) {
          throw new Error('process.chdir is not supported');
        };
      },
      {}
    ],
    41: [
      function (_dereq_, module, exports) {
        /**
 * This is the web browser implementation of `debug()`.
 *
 * Expose `debug()` as the module.
 */
        exports = module.exports = _dereq_(42);
        exports.log = log;
        exports.formatArgs = formatArgs;
        exports.save = save;
        exports.load = load;
        exports.useColors = useColors;
        /**
 * Use chrome.storage.local if we are in an app
 */
        var storage;
        if (typeof chrome !== 'undefined' && typeof chrome.storage !== 'undefined')
          storage = chrome.storage.local;
        else
          storage = localstorage();
        /**
 * Colors.
 */
        exports.colors = [
          'lightseagreen',
          'forestgreen',
          'goldenrod',
          'dodgerblue',
          'darkorchid',
          'crimson'
        ];
        /**
 * Currently only WebKit-based Web Inspectors, Firefox >= v31,
 * and the Firebug extension (any Firefox version) are known
 * to support "%c" CSS customizations.
 *
 * TODO: add a `localStorage` variable to explicitly enable/disable colors
 */
        function useColors() {
          // is webkit? http://stackoverflow.com/a/16459606/376773
          return 'WebkitAppearance' in document.documentElement.style || window.console && (console.firebug || console.exception && console.table) || navigator.userAgent.toLowerCase().match(/firefox\/(\d+)/) && parseInt(RegExp.$1, 10) >= 31;
        }
        /**
 * Map %j to `JSON.stringify()`, since no Web Inspectors do that by default.
 */
        exports.formatters.j = function (v) {
          return JSON.stringify(v);
        };
        /**
 * Colorize log arguments if enabled.
 *
 * @api public
 */
        function formatArgs() {
          var args = arguments;
          var useColors = this.useColors;
          args[0] = (useColors ? '%c' : '') + this.namespace + (useColors ? ' %c' : ' ') + args[0] + (useColors ? '%c ' : ' ') + '+' + exports.humanize(this.diff);
          if (!useColors)
            return args;
          var c = 'color: ' + this.color;
          args = [
            args[0],
            c,
            'color: inherit'
          ].concat(Array.prototype.slice.call(args, 1));
          // the final "%c" is somewhat tricky, because there could be other
          // arguments passed either before or after the %c, so we need to
          // figure out the correct index to insert the CSS into
          var index = 0;
          var lastC = 0;
          args[0].replace(/%[a-z%]/g, function (match) {
            if ('%' === match)
              return;
            index++;
            if ('%c' === match) {
              // we only are interested in the *last* %c
              // (the user may have provided their own)
              lastC = index;
            }
          });
          args.splice(lastC, 0, c);
          return args;
        }
        /**
 * Invokes `console.log()` when available.
 * No-op when `console.log` is not a "function".
 *
 * @api public
 */
        function log() {
          // this hackery is required for IE8/9, where
          // the `console.log` function doesn't have 'apply'
          return 'object' === typeof console && console.log && Function.prototype.apply.call(console.log, console, arguments);
        }
        /**
 * Save `namespaces`.
 *
 * @param {String} namespaces
 * @api private
 */
        function save(namespaces) {
          try {
            if (null == namespaces) {
              storage.removeItem('debug');
            } else {
              storage.debug = namespaces;
            }
          } catch (e) {
          }
        }
        /**
 * Load `namespaces`.
 *
 * @return {String} returns the previously persisted debug modes
 * @api private
 */
        function load() {
          var r;
          try {
            r = storage.debug;
          } catch (e) {
          }
          return r;
        }
        /**
 * Enable namespaces listed in `localStorage.debug` initially.
 */
        exports.enable(load());
        /**
 * Localstorage attempts to return the localstorage.
 *
 * This is necessary because safari throws
 * when a user disables cookies/localstorage
 * and you attempt to access it.
 *
 * @return {LocalStorage}
 * @api private
 */
        function localstorage() {
          try {
            return window.localStorage;
          } catch (e) {
          }
        }
      },
      { '42': 42 }
    ],
    42: [
      function (_dereq_, module, exports) {
        /**
 * This is the common logic for both the Node.js and web browser
 * implementations of `debug()`.
 *
 * Expose `debug()` as the module.
 */
        exports = module.exports = debug;
        exports.coerce = coerce;
        exports.disable = disable;
        exports.enable = enable;
        exports.enabled = enabled;
        exports.humanize = _dereq_(43);
        /**
 * The currently active debug mode names, and names to skip.
 */
        exports.names = [];
        exports.skips = [];
        /**
 * Map of special "%n" handling functions, for the debug "format" argument.
 *
 * Valid key names are a single, lowercased letter, i.e. "n".
 */
        exports.formatters = {};
        /**
 * Previously assigned color.
 */
        var prevColor = 0;
        /**
 * Previous log timestamp.
 */
        var prevTime;
        /**
 * Select a color.
 *
 * @return {Number}
 * @api private
 */
        function selectColor() {
          return exports.colors[prevColor++ % exports.colors.length];
        }
        /**
 * Create a debugger with the given `namespace`.
 *
 * @param {String} namespace
 * @return {Function}
 * @api public
 */
        function debug(namespace) {
          // define the `disabled` version
          function disabled() {
          }
          disabled.enabled = false;
          // define the `enabled` version
          function enabled() {
            var self = enabled;
            // set `diff` timestamp
            var curr = +new Date();
            var ms = curr - (prevTime || curr);
            self.diff = ms;
            self.prev = prevTime;
            self.curr = curr;
            prevTime = curr;
            // add the `color` if not set
            if (null == self.useColors)
              self.useColors = exports.useColors();
            if (null == self.color && self.useColors)
              self.color = selectColor();
            var args = Array.prototype.slice.call(arguments);
            args[0] = exports.coerce(args[0]);
            if ('string' !== typeof args[0]) {
              // anything else let's inspect with %o
              args = ['%o'].concat(args);
            }
            // apply any `formatters` transformations
            var index = 0;
            args[0] = args[0].replace(/%([a-z%])/g, function (match, format) {
              // if we encounter an escaped % then don't increase the array index
              if (match === '%')
                return match;
              index++;
              var formatter = exports.formatters[format];
              if ('function' === typeof formatter) {
                var val = args[index];
                match = formatter.call(self, val);
                // now we need to remove `args[index]` since it's inlined in the `format`
                args.splice(index, 1);
                index--;
              }
              return match;
            });
            if ('function' === typeof exports.formatArgs) {
              args = exports.formatArgs.apply(self, args);
            }
            var logFn = enabled.log || exports.log || console.log.bind(console);
            logFn.apply(self, args);
          }
          enabled.enabled = true;
          var fn = exports.enabled(namespace) ? enabled : disabled;
          fn.namespace = namespace;
          return fn;
        }
        /**
 * Enables a debug mode by namespaces. This can include modes
 * separated by a colon and wildcards.
 *
 * @param {String} namespaces
 * @api public
 */
        function enable(namespaces) {
          exports.save(namespaces);
          var split = (namespaces || '').split(/[\s,]+/);
          var len = split.length;
          for (var i = 0; i < len; i++) {
            if (!split[i])
              continue;
            // ignore empty strings
            namespaces = split[i].replace(/\*/g, '.*?');
            if (namespaces[0] === '-') {
              exports.skips.push(new RegExp('^' + namespaces.substr(1) + '$'));
            } else {
              exports.names.push(new RegExp('^' + namespaces + '$'));
            }
          }
        }
        /**
 * Disable debug output.
 *
 * @api public
 */
        function disable() {
          exports.enable('');
        }
        /**
 * Returns true if the given mode name is enabled, false otherwise.
 *
 * @param {String} name
 * @return {Boolean}
 * @api public
 */
        function enabled(name) {
          var i, len;
          for (i = 0, len = exports.skips.length; i < len; i++) {
            if (exports.skips[i].test(name)) {
              return false;
            }
          }
          for (i = 0, len = exports.names.length; i < len; i++) {
            if (exports.names[i].test(name)) {
              return true;
            }
          }
          return false;
        }
        /**
 * Coerce `val`.
 *
 * @param {Mixed} val
 * @return {Mixed}
 * @api private
 */
        function coerce(val) {
          if (val instanceof Error)
            return val.stack || val.message;
          return val;
        }
      },
      { '43': 43 }
    ],
    43: [
      function (_dereq_, module, exports) {
        /**
 * Helpers.
 */
        var s = 1000;
        var m = s * 60;
        var h = m * 60;
        var d = h * 24;
        var y = d * 365.25;
        /**
 * Parse or format the given `val`.
 *
 * Options:
 *
 *  - `long` verbose formatting [false]
 *
 * @param {String|Number} val
 * @param {Object} options
 * @return {String|Number}
 * @api public
 */
        module.exports = function (val, options) {
          options = options || {};
          if ('string' == typeof val)
            return parse(val);
          return options.long ? long(val) : short(val);
        };
        /**
 * Parse the given `str` and return milliseconds.
 *
 * @param {String} str
 * @return {Number}
 * @api private
 */
        function parse(str) {
          var match = /^((?:\d+)?\.?\d+) *(milliseconds?|msecs?|ms|seconds?|secs?|s|minutes?|mins?|m|hours?|hrs?|h|days?|d|years?|yrs?|y)?$/i.exec(str);
          if (!match)
            return;
          var n = parseFloat(match[1]);
          var type = (match[2] || 'ms').toLowerCase();
          switch (type) {
          case 'years':
          case 'year':
          case 'yrs':
          case 'yr':
          case 'y':
            return n * y;
          case 'days':
          case 'day':
          case 'd':
            return n * d;
          case 'hours':
          case 'hour':
          case 'hrs':
          case 'hr':
          case 'h':
            return n * h;
          case 'minutes':
          case 'minute':
          case 'mins':
          case 'min':
          case 'm':
            return n * m;
          case 'seconds':
          case 'second':
          case 'secs':
          case 'sec':
          case 's':
            return n * s;
          case 'milliseconds':
          case 'millisecond':
          case 'msecs':
          case 'msec':
          case 'ms':
            return n;
          }
        }
        /**
 * Short format for `ms`.
 *
 * @param {Number} ms
 * @return {String}
 * @api private
 */
        function short(ms) {
          if (ms >= d)
            return Math.round(ms / d) + 'd';
          if (ms >= h)
            return Math.round(ms / h) + 'h';
          if (ms >= m)
            return Math.round(ms / m) + 'm';
          if (ms >= s)
            return Math.round(ms / s) + 's';
          return ms + 'ms';
        }
        /**
 * Long format for `ms`.
 *
 * @param {Number} ms
 * @return {String}
 * @api private
 */
        function long(ms) {
          return plural(ms, d, 'day') || plural(ms, h, 'hour') || plural(ms, m, 'minute') || plural(ms, s, 'second') || ms + ' ms';
        }
        /**
 * Pluralization helper.
 */
        function plural(ms, n, name) {
          if (ms < n)
            return;
          if (ms < n * 1.5)
            return Math.floor(ms / n) + ' ' + name;
          return Math.ceil(ms / n) + ' ' + name + 's';
        }
      },
      {}
    ],
    44: [
      function (_dereq_, module, exports) {
        if (typeof Object.create === 'function') {
          // implementation from standard node.js 'util' module
          module.exports = function inherits(ctor, superCtor) {
            ctor.super_ = superCtor;
            ctor.prototype = Object.create(superCtor.prototype, {
              constructor: {
                value: ctor,
                enumerable: false,
                writable: true,
                configurable: true
              }
            });
          };
        } else {
          // old school shim for old browsers
          module.exports = function inherits(ctor, superCtor) {
            ctor.super_ = superCtor;
            var TempCtor = function () {
            };
            TempCtor.prototype = superCtor.prototype;
            ctor.prototype = new TempCtor();
            ctor.prototype.constructor = ctor;
          };
        }
      },
      {}
    ],
    45: [
      function (_dereq_, module, exports) {
        'use strict';
        module.exports = INTERNAL;
        function INTERNAL() {
        }
      },
      {}
    ],
    46: [
      function (_dereq_, module, exports) {
        'use strict';
        var Promise = _dereq_(49);
        var reject = _dereq_(52);
        var resolve = _dereq_(53);
        var INTERNAL = _dereq_(45);
        var handlers = _dereq_(47);
        module.exports = all;
        function all(iterable) {
          if (Object.prototype.toString.call(iterable) !== '[object Array]') {
            return reject(new TypeError('must be an array'));
          }
          var len = iterable.length;
          var called = false;
          if (!len) {
            return resolve([]);
          }
          var values = new Array(len);
          var resolved = 0;
          var i = -1;
          var promise = new Promise(INTERNAL);
          while (++i < len) {
            allResolver(iterable[i], i);
          }
          return promise;
          function allResolver(value, i) {
            resolve(value).then(resolveFromAll, function (error) {
              if (!called) {
                called = true;
                handlers.reject(promise, error);
              }
            });
            function resolveFromAll(outValue) {
              values[i] = outValue;
              if (++resolved === len & !called) {
                called = true;
                handlers.resolve(promise, values);
              }
            }
          }
        }
      },
      {
        '45': 45,
        '47': 47,
        '49': 49,
        '52': 52,
        '53': 53
      }
    ],
    47: [
      function (_dereq_, module, exports) {
        'use strict';
        var tryCatch = _dereq_(56);
        var resolveThenable = _dereq_(54);
        var states = _dereq_(55);
        exports.resolve = function (self, value) {
          var result = tryCatch(getThen, value);
          if (result.status === 'error') {
            return exports.reject(self, result.value);
          }
          var thenable = result.value;
          if (thenable) {
            resolveThenable.safely(self, thenable);
          } else {
            self.state = states.FULFILLED;
            self.outcome = value;
            var i = -1;
            var len = self.queue.length;
            while (++i < len) {
              self.queue[i].callFulfilled(value);
            }
          }
          return self;
        };
        exports.reject = function (self, error) {
          self.state = states.REJECTED;
          self.outcome = error;
          var i = -1;
          var len = self.queue.length;
          while (++i < len) {
            self.queue[i].callRejected(error);
          }
          return self;
        };
        function getThen(obj) {
          // Make sure we only access the accessor once as required by the spec
          var then = obj && obj.then;
          if (obj && typeof obj === 'object' && typeof then === 'function') {
            return function appyThen() {
              then.apply(obj, arguments);
            };
          }
        }
      },
      {
        '54': 54,
        '55': 55,
        '56': 56
      }
    ],
    48: [
      function (_dereq_, module, exports) {
        module.exports = exports = _dereq_(49);
        exports.resolve = _dereq_(53);
        exports.reject = _dereq_(52);
        exports.all = _dereq_(46);
        exports.race = _dereq_(51);
      },
      {
        '46': 46,
        '49': 49,
        '51': 51,
        '52': 52,
        '53': 53
      }
    ],
    49: [
      function (_dereq_, module, exports) {
        'use strict';
        var unwrap = _dereq_(57);
        var INTERNAL = _dereq_(45);
        var resolveThenable = _dereq_(54);
        var states = _dereq_(55);
        var QueueItem = _dereq_(50);
        module.exports = Promise;
        function Promise(resolver) {
          if (!(this instanceof Promise)) {
            return new Promise(resolver);
          }
          if (typeof resolver !== 'function') {
            throw new TypeError('resolver must be a function');
          }
          this.state = states.PENDING;
          this.queue = [];
          this.outcome = void 0;
          if (resolver !== INTERNAL) {
            resolveThenable.safely(this, resolver);
          }
        }
        Promise.prototype['catch'] = function (onRejected) {
          return this.then(null, onRejected);
        };
        Promise.prototype.then = function (onFulfilled, onRejected) {
          if (typeof onFulfilled !== 'function' && this.state === states.FULFILLED || typeof onRejected !== 'function' && this.state === states.REJECTED) {
            return this;
          }
          var promise = new Promise(INTERNAL);
          if (this.state !== states.PENDING) {
            var resolver = this.state === states.FULFILLED ? onFulfilled : onRejected;
            unwrap(promise, resolver, this.outcome);
          } else {
            this.queue.push(new QueueItem(promise, onFulfilled, onRejected));
          }
          return promise;
        };
      },
      {
        '45': 45,
        '50': 50,
        '54': 54,
        '55': 55,
        '57': 57
      }
    ],
    50: [
      function (_dereq_, module, exports) {
        'use strict';
        var handlers = _dereq_(47);
        var unwrap = _dereq_(57);
        module.exports = QueueItem;
        function QueueItem(promise, onFulfilled, onRejected) {
          this.promise = promise;
          if (typeof onFulfilled === 'function') {
            this.onFulfilled = onFulfilled;
            this.callFulfilled = this.otherCallFulfilled;
          }
          if (typeof onRejected === 'function') {
            this.onRejected = onRejected;
            this.callRejected = this.otherCallRejected;
          }
        }
        QueueItem.prototype.callFulfilled = function (value) {
          handlers.resolve(this.promise, value);
        };
        QueueItem.prototype.otherCallFulfilled = function (value) {
          unwrap(this.promise, this.onFulfilled, value);
        };
        QueueItem.prototype.callRejected = function (value) {
          handlers.reject(this.promise, value);
        };
        QueueItem.prototype.otherCallRejected = function (value) {
          unwrap(this.promise, this.onRejected, value);
        };
      },
      {
        '47': 47,
        '57': 57
      }
    ],
    51: [
      function (_dereq_, module, exports) {
        'use strict';
        var Promise = _dereq_(49);
        var reject = _dereq_(52);
        var resolve = _dereq_(53);
        var INTERNAL = _dereq_(45);
        var handlers = _dereq_(47);
        module.exports = race;
        function race(iterable) {
          if (Object.prototype.toString.call(iterable) !== '[object Array]') {
            return reject(new TypeError('must be an array'));
          }
          var len = iterable.length;
          var called = false;
          if (!len) {
            return resolve([]);
          }
          var resolved = 0;
          var i = -1;
          var promise = new Promise(INTERNAL);
          while (++i < len) {
            resolver(iterable[i]);
          }
          return promise;
          function resolver(value) {
            resolve(value).then(function (response) {
              if (!called) {
                called = true;
                handlers.resolve(promise, response);
              }
            }, function (error) {
              if (!called) {
                called = true;
                handlers.reject(promise, error);
              }
            });
          }
        }
      },
      {
        '45': 45,
        '47': 47,
        '49': 49,
        '52': 52,
        '53': 53
      }
    ],
    52: [
      function (_dereq_, module, exports) {
        'use strict';
        var Promise = _dereq_(49);
        var INTERNAL = _dereq_(45);
        var handlers = _dereq_(47);
        module.exports = reject;
        function reject(reason) {
          var promise = new Promise(INTERNAL);
          return handlers.reject(promise, reason);
        }
      },
      {
        '45': 45,
        '47': 47,
        '49': 49
      }
    ],
    53: [
      function (_dereq_, module, exports) {
        'use strict';
        var Promise = _dereq_(49);
        var INTERNAL = _dereq_(45);
        var handlers = _dereq_(47);
        module.exports = resolve;
        var FALSE = handlers.resolve(new Promise(INTERNAL), false);
        var NULL = handlers.resolve(new Promise(INTERNAL), null);
        var UNDEFINED = handlers.resolve(new Promise(INTERNAL), void 0);
        var ZERO = handlers.resolve(new Promise(INTERNAL), 0);
        var EMPTYSTRING = handlers.resolve(new Promise(INTERNAL), '');
        function resolve(value) {
          if (value) {
            if (value instanceof Promise) {
              return value;
            }
            return handlers.resolve(new Promise(INTERNAL), value);
          }
          var valueType = typeof value;
          switch (valueType) {
          case 'boolean':
            return FALSE;
          case 'undefined':
            return UNDEFINED;
          case 'object':
            return NULL;
          case 'number':
            return ZERO;
          case 'string':
            return EMPTYSTRING;
          }
        }
      },
      {
        '45': 45,
        '47': 47,
        '49': 49
      }
    ],
    54: [
      function (_dereq_, module, exports) {
        'use strict';
        var handlers = _dereq_(47);
        var tryCatch = _dereq_(56);
        function safelyResolveThenable(self, thenable) {
          // Either fulfill, reject or reject with error
          var called = false;
          function onError(value) {
            if (called) {
              return;
            }
            called = true;
            handlers.reject(self, value);
          }
          function onSuccess(value) {
            if (called) {
              return;
            }
            called = true;
            handlers.resolve(self, value);
          }
          function tryToUnwrap() {
            thenable(onSuccess, onError);
          }
          var result = tryCatch(tryToUnwrap);
          if (result.status === 'error') {
            onError(result.value);
          }
        }
        exports.safely = safelyResolveThenable;
      },
      {
        '47': 47,
        '56': 56
      }
    ],
    55: [
      function (_dereq_, module, exports) {
        // Lazy man's symbols for states
        exports.REJECTED = ['REJECTED'];
        exports.FULFILLED = ['FULFILLED'];
        exports.PENDING = ['PENDING'];
      },
      {}
    ],
    56: [
      function (_dereq_, module, exports) {
        'use strict';
        module.exports = tryCatch;
        function tryCatch(func, value) {
          var out = {};
          try {
            out.value = func(value);
            out.status = 'success';
          } catch (e) {
            out.status = 'error';
            out.value = e;
          }
          return out;
        }
      },
      {}
    ],
    57: [
      function (_dereq_, module, exports) {
        'use strict';
        var immediate = _dereq_(58);
        var handlers = _dereq_(47);
        module.exports = unwrap;
        function unwrap(promise, func, value) {
          immediate(function () {
            var returnValue;
            try {
              returnValue = func(value);
            } catch (e) {
              return handlers.reject(promise, e);
            }
            if (returnValue === promise) {
              handlers.reject(promise, new TypeError('Cannot resolve promise with itself'));
            } else {
              handlers.resolve(promise, returnValue);
            }
          });
        }
      },
      {
        '47': 47,
        '58': 58
      }
    ],
    58: [
      function (_dereq_, module, exports) {
        'use strict';
        var types = [
            _dereq_(38),
            _dereq_(60),
            _dereq_(59),
            _dereq_(61),
            _dereq_(62)
          ];
        var draining;
        var queue = [];
        //named nextTick for less confusing stack traces
        function nextTick() {
          draining = true;
          var i, oldQueue;
          var len = queue.length;
          while (len) {
            oldQueue = queue;
            queue = [];
            i = -1;
            while (++i < len) {
              oldQueue[i]();
            }
            len = queue.length;
          }
          draining = false;
        }
        var scheduleDrain;
        var i = -1;
        var len = types.length;
        while (++i < len) {
          if (types[i] && types[i].test && types[i].test()) {
            scheduleDrain = types[i].install(nextTick);
            break;
          }
        }
        module.exports = immediate;
        function immediate(task) {
          if (queue.push(task) === 1 && !draining) {
            scheduleDrain();
          }
        }
      },
      {
        '38': 38,
        '59': 59,
        '60': 60,
        '61': 61,
        '62': 62
      }
    ],
    59: [
      function (_dereq_, module, exports) {
        (function (global) {
          'use strict';
          exports.test = function () {
            if (global.setImmediate) {
              // we can only get here in IE10
              // which doesn't handel postMessage well
              return false;
            }
            return typeof global.MessageChannel !== 'undefined';
          };
          exports.install = function (func) {
            var channel = new global.MessageChannel();
            channel.port1.onmessage = func;
            return function () {
              channel.port2.postMessage(0);
            };
          };
        }.call(this, typeof global !== 'undefined' ? global : typeof self !== 'undefined' ? self : typeof window !== 'undefined' ? window : {}));
      },
      {}
    ],
    60: [
      function (_dereq_, module, exports) {
        (function (global) {
          'use strict';
          //based off rsvp https://github.com/tildeio/rsvp.js
          //license https://github.com/tildeio/rsvp.js/blob/master/LICENSE
          //https://github.com/tildeio/rsvp.js/blob/master/lib/rsvp/asap.js
          var Mutation = global.MutationObserver || global.WebKitMutationObserver;
          exports.test = function () {
            return Mutation;
          };
          exports.install = function (handle) {
            var called = 0;
            var observer = new Mutation(handle);
            var element = global.document.createTextNode('');
            observer.observe(element, { characterData: true });
            return function () {
              element.data = called = ++called % 2;
            };
          };
        }.call(this, typeof global !== 'undefined' ? global : typeof self !== 'undefined' ? self : typeof window !== 'undefined' ? window : {}));
      },
      {}
    ],
    61: [
      function (_dereq_, module, exports) {
        (function (global) {
          'use strict';
          exports.test = function () {
            return 'document' in global && 'onreadystatechange' in global.document.createElement('script');
          };
          exports.install = function (handle) {
            return function () {
              // Create a <script> element; its readystatechange event will be fired asynchronously once it is inserted
              // into the document. Do so, thus queuing up the task. Remember to clean up once it's been called.
              var scriptEl = global.document.createElement('script');
              scriptEl.onreadystatechange = function () {
                handle();
                scriptEl.onreadystatechange = null;
                scriptEl.parentNode.removeChild(scriptEl);
                scriptEl = null;
              };
              global.document.documentElement.appendChild(scriptEl);
              return handle;
            };
          };
        }.call(this, typeof global !== 'undefined' ? global : typeof self !== 'undefined' ? self : typeof window !== 'undefined' ? window : {}));
      },
      {}
    ],
    62: [
      function (_dereq_, module, exports) {
        'use strict';
        exports.test = function () {
          return true;
        };
        exports.install = function (t) {
          return function () {
            setTimeout(t, 0);
          };
        };
      },
      {}
    ],
    63: [
      function (_dereq_, module, exports) {
        'use strict';
        var MIN_MAGNITUDE = -324;
        // verified by -Number.MIN_VALUE
        var MAGNITUDE_DIGITS = 3;
        // ditto
        var SEP = '';
        // set to '_' for easier debugging 
        var utils = _dereq_(64);
        exports.collate = function (a, b) {
          if (a === b) {
            return 0;
          }
          a = exports.normalizeKey(a);
          b = exports.normalizeKey(b);
          var ai = collationIndex(a);
          var bi = collationIndex(b);
          if (ai - bi !== 0) {
            return ai - bi;
          }
          if (a === null) {
            return 0;
          }
          switch (typeof a) {
          case 'number':
            return a - b;
          case 'boolean':
            return a === b ? 0 : a < b ? -1 : 1;
          case 'string':
            return stringCollate(a, b);
          }
          return Array.isArray(a) ? arrayCollate(a, b) : objectCollate(a, b);
        };
        // couch considers null/NaN/Infinity/-Infinity === undefined,
        // for the purposes of mapreduce indexes. also, dates get stringified.
        exports.normalizeKey = function (key) {
          switch (typeof key) {
          case 'undefined':
            return null;
          case 'number':
            if (key === Infinity || key === -Infinity || isNaN(key)) {
              return null;
            }
            return key;
          case 'object':
            var origKey = key;
            if (Array.isArray(key)) {
              var len = key.length;
              key = new Array(len);
              for (var i = 0; i < len; i++) {
                key[i] = exports.normalizeKey(origKey[i]);
              }
            } else if (key instanceof Date) {
              return key.toJSON();
            } else if (key !== null) {
              // generic object
              key = {};
              for (var k in origKey) {
                if (origKey.hasOwnProperty(k)) {
                  var val = origKey[k];
                  if (typeof val !== 'undefined') {
                    key[k] = exports.normalizeKey(val);
                  }
                }
              }
            }
          }
          return key;
        };
        function indexify(key) {
          if (key !== null) {
            switch (typeof key) {
            case 'boolean':
              return key ? 1 : 0;
            case 'number':
              return numToIndexableString(key);
            case 'string':
              // We've to be sure that key does not contain \u0000
              // Do order-preserving replacements:
              // 0 -> 1, 1
              // 1 -> 1, 2
              // 2 -> 2, 2
              return key.replace(/\u0002/g, '\x02\x02').replace(/\u0001/g, '\x01\x02').replace(/\u0000/g, '\x01\x01');
            case 'object':
              var isArray = Array.isArray(key);
              var arr = isArray ? key : Object.keys(key);
              var i = -1;
              var len = arr.length;
              var result = '';
              if (isArray) {
                while (++i < len) {
                  result += exports.toIndexableString(arr[i]);
                }
              } else {
                while (++i < len) {
                  var objKey = arr[i];
                  result += exports.toIndexableString(objKey) + exports.toIndexableString(key[objKey]);
                }
              }
              return result;
            }
          }
          return '';
        }
        // convert the given key to a string that would be appropriate
        // for lexical sorting, e.g. within a database, where the
        // sorting is the same given by the collate() function.
        exports.toIndexableString = function (key) {
          var zero = '\0';
          key = exports.normalizeKey(key);
          return collationIndex(key) + SEP + indexify(key) + zero;
        };
        function parseNumber(str, i) {
          var originalIdx = i;
          var num;
          var zero = str[i] === '1';
          if (zero) {
            num = 0;
            i++;
          } else {
            var neg = str[i] === '0';
            i++;
            var numAsString = '';
            var magAsString = str.substring(i, i + MAGNITUDE_DIGITS);
            var magnitude = parseInt(magAsString, 10) + MIN_MAGNITUDE;
            if (neg) {
              magnitude = -magnitude;
            }
            i += MAGNITUDE_DIGITS;
            while (true) {
              var ch = str[i];
              if (ch === '\0') {
                break;
              } else {
                numAsString += ch;
              }
              i++;
            }
            numAsString = numAsString.split('.');
            if (numAsString.length === 1) {
              num = parseInt(numAsString, 10);
            } else {
              num = parseFloat(numAsString[0] + '.' + numAsString[1]);
            }
            if (neg) {
              num = num - 10;
            }
            if (magnitude !== 0) {
              // parseFloat is more reliable than pow due to rounding errors
              // e.g. Number.MAX_VALUE would return Infinity if we did
              // num * Math.pow(10, magnitude);
              num = parseFloat(num + 'e' + magnitude);
            }
          }
          return {
            num: num,
            length: i - originalIdx
          };
        }
        // move up the stack while parsing
        // this function moved outside of parseIndexableString for performance
        function pop(stack, metaStack) {
          var obj = stack.pop();
          if (metaStack.length) {
            var lastMetaElement = metaStack[metaStack.length - 1];
            if (obj === lastMetaElement.element) {
              // popping a meta-element, e.g. an object whose value is another object
              metaStack.pop();
              lastMetaElement = metaStack[metaStack.length - 1];
            }
            var element = lastMetaElement.element;
            var lastElementIndex = lastMetaElement.index;
            if (Array.isArray(element)) {
              element.push(obj);
            } else if (lastElementIndex === stack.length - 2) {
              // obj with key+value
              var key = stack.pop();
              element[key] = obj;
            } else {
              stack.push(obj);  // obj with key only
            }
          }
        }
        exports.parseIndexableString = function (str) {
          var stack = [];
          var metaStack = [];
          // stack for arrays and objects
          var i = 0;
          while (true) {
            var collationIndex = str[i++];
            if (collationIndex === '\0') {
              if (stack.length === 1) {
                return stack.pop();
              } else {
                pop(stack, metaStack);
                continue;
              }
            }
            switch (collationIndex) {
            case '1':
              stack.push(null);
              break;
            case '2':
              stack.push(str[i] === '1');
              i++;
              break;
            case '3':
              var parsedNum = parseNumber(str, i);
              stack.push(parsedNum.num);
              i += parsedNum.length;
              break;
            case '4':
              var parsedStr = '';
              while (true) {
                var ch = str[i];
                if (ch === '\0') {
                  break;
                }
                parsedStr += ch;
                i++;
              }
              // perform the reverse of the order-preserving replacement
              // algorithm (see above)
              parsedStr = parsedStr.replace(/\u0001\u0001/g, '\0').replace(/\u0001\u0002/g, '\x01').replace(/\u0002\u0002/g, '\x02');
              stack.push(parsedStr);
              break;
            case '5':
              var arrayElement = {
                  element: [],
                  index: stack.length
                };
              stack.push(arrayElement.element);
              metaStack.push(arrayElement);
              break;
            case '6':
              var objElement = {
                  element: {},
                  index: stack.length
                };
              stack.push(objElement.element);
              metaStack.push(objElement);
              break;
            default:
              throw new Error('bad collationIndex or unexpectedly reached end of input: ' + collationIndex);
            }
          }
        };
        function arrayCollate(a, b) {
          var len = Math.min(a.length, b.length);
          for (var i = 0; i < len; i++) {
            var sort = exports.collate(a[i], b[i]);
            if (sort !== 0) {
              return sort;
            }
          }
          return a.length === b.length ? 0 : a.length > b.length ? 1 : -1;
        }
        function stringCollate(a, b) {
          // See: https://github.com/daleharvey/pouchdb/issues/40
          // This is incompatible with the CouchDB implementation, but its the
          // best we can do for now
          return a === b ? 0 : a > b ? 1 : -1;
        }
        function objectCollate(a, b) {
          var ak = Object.keys(a), bk = Object.keys(b);
          var len = Math.min(ak.length, bk.length);
          for (var i = 0; i < len; i++) {
            // First sort the keys
            var sort = exports.collate(ak[i], bk[i]);
            if (sort !== 0) {
              return sort;
            }
            // if the keys are equal sort the values
            sort = exports.collate(a[ak[i]], b[bk[i]]);
            if (sort !== 0) {
              return sort;
            }
          }
          return ak.length === bk.length ? 0 : ak.length > bk.length ? 1 : -1;
        }
        // The collation is defined by erlangs ordered terms
        // the atoms null, true, false come first, then numbers, strings,
        // arrays, then objects
        // null/undefined/NaN/Infinity/-Infinity are all considered null
        function collationIndex(x) {
          var id = [
              'boolean',
              'number',
              'string',
              'object'
            ];
          var idx = id.indexOf(typeof x);
          //false if -1 otherwise true, but fast!!!!1
          if (~idx) {
            if (x === null) {
              return 1;
            }
            if (Array.isArray(x)) {
              return 5;
            }
            return idx < 3 ? idx + 2 : idx + 3;
          }
          if (Array.isArray(x)) {
            return 5;
          }
        }
        // conversion:
        // x yyy zz...zz
        // x = 0 for negative, 1 for 0, 2 for positive
        // y = exponent (for negative numbers negated) moved so that it's >= 0
        // z = mantisse
        function numToIndexableString(num) {
          if (num === 0) {
            return '1';
          }
          // convert number to exponential format for easier and
          // more succinct string sorting
          var expFormat = num.toExponential().split(/e\+?/);
          var magnitude = parseInt(expFormat[1], 10);
          var neg = num < 0;
          var result = neg ? '0' : '2';
          // first sort by magnitude
          // it's easier if all magnitudes are positive
          var magForComparison = (neg ? -magnitude : magnitude) - MIN_MAGNITUDE;
          var magString = utils.padLeft(magForComparison.toString(), '0', MAGNITUDE_DIGITS);
          result += SEP + magString;
          // then sort by the factor
          var factor = Math.abs(parseFloat(expFormat[0]));
          // [1..10)
          if (neg) {
            // for negative reverse ordering
            factor = 10 - factor;
          }
          var factorStr = factor.toFixed(20);
          // strip zeros from the end
          factorStr = factorStr.replace(/\.?0+$/, '');
          result += SEP + factorStr;
          return result;
        }
      },
      { '64': 64 }
    ],
    64: [
      function (_dereq_, module, exports) {
        'use strict';
        function pad(str, padWith, upToLength) {
          var padding = '';
          var targetLength = upToLength - str.length;
          while (padding.length < targetLength) {
            padding += padWith;
          }
          return padding;
        }
        exports.padLeft = function (str, padWith, upToLength) {
          var padding = pad(str, padWith, upToLength);
          return padding + str;
        };
        exports.padRight = function (str, padWith, upToLength) {
          var padding = pad(str, padWith, upToLength);
          return str + padding;
        };
        exports.stringLexCompare = function (a, b) {
          var aLen = a.length;
          var bLen = b.length;
          var i;
          for (i = 0; i < aLen; i++) {
            if (i === bLen) {
              // b is shorter substring of a
              return 1;
            }
            var aChar = a.charAt(i);
            var bChar = b.charAt(i);
            if (aChar !== bChar) {
              return aChar < bChar ? -1 : 1;
            }
          }
          if (aLen < bLen) {
            // a is shorter substring of b
            return -1;
          }
          return 0;
        };
        /*
 * returns the decimal form for the given integer, i.e. writes
 * out all the digits (in base-10) instead of using scientific notation
 */
        exports.intToDecimalForm = function (int) {
          var isNeg = int < 0;
          var result = '';
          do {
            var remainder = isNeg ? -Math.ceil(int % 10) : Math.floor(int % 10);
            result = remainder + result;
            int = isNeg ? Math.ceil(int / 10) : Math.floor(int / 10);
          } while (int);
          if (isNeg && result !== '0') {
            result = '-' + result;
          }
          return result;
        };
      },
      {}
    ],
    65: [
      function (_dereq_, module, exports) {
        'use strict';
        exports.Map = LazyMap;
        // TODO: use ES6 map
        exports.Set = LazySet;
        // TODO: use ES6 set
        // based on https://github.com/montagejs/collections
        function LazyMap() {
          this.store = {};
        }
        LazyMap.prototype.mangle = function (key) {
          if (typeof key !== 'string') {
            throw new TypeError('key must be a string but Got ' + key);
          }
          return '$' + key;
        };
        LazyMap.prototype.unmangle = function (key) {
          return key.substring(1);
        };
        LazyMap.prototype.get = function (key) {
          var mangled = this.mangle(key);
          if (mangled in this.store) {
            return this.store[mangled];
          } else {
            return void 0;
          }
        };
        LazyMap.prototype.set = function (key, value) {
          var mangled = this.mangle(key);
          this.store[mangled] = value;
          return true;
        };
        LazyMap.prototype.has = function (key) {
          var mangled = this.mangle(key);
          return mangled in this.store;
        };
        LazyMap.prototype.delete = function (key) {
          var mangled = this.mangle(key);
          if (mangled in this.store) {
            delete this.store[mangled];
            return true;
          }
          return false;
        };
        LazyMap.prototype.forEach = function (cb) {
          var self = this;
          var keys = Object.keys(self.store);
          keys.forEach(function (key) {
            var value = self.store[key];
            key = self.unmangle(key);
            cb(value, key);
          });
        };
        function LazySet(array) {
          this.store = new LazyMap();
          // init with an array
          if (array && Array.isArray(array)) {
            for (var i = 0, len = array.length; i < len; i++) {
              this.add(array[i]);
            }
          }
        }
        LazySet.prototype.add = function (key) {
          return this.store.set(key, true);
        };
        LazySet.prototype.has = function (key) {
          return this.store.has(key);
        };
        LazySet.prototype.delete = function (key) {
          return this.store.delete(key);
        };
      },
      {}
    ],
    66: [
      function (_dereq_, module, exports) {
        'use strict';
        // Extends method
        // (taken from http://code.jquery.com/jquery-1.9.0.js)
        // Populate the class2type map
        var class2type = {};
        var types = [
            'Boolean',
            'Number',
            'String',
            'Function',
            'Array',
            'Date',
            'RegExp',
            'Object',
            'Error'
          ];
        for (var i = 0; i < types.length; i++) {
          var typename = types[i];
          class2type['[object ' + typename + ']'] = typename.toLowerCase();
        }
        var core_toString = class2type.toString;
        var core_hasOwn = class2type.hasOwnProperty;
        function type(obj) {
          if (obj === null) {
            return String(obj);
          }
          return typeof obj === 'object' || typeof obj === 'function' ? class2type[core_toString.call(obj)] || 'object' : typeof obj;
        }
        function isWindow(obj) {
          return obj !== null && obj === obj.window;
        }
        function isPlainObject(obj) {
          // Must be an Object.
          // Because of IE, we also have to check the presence of
          // the constructor property.
          // Make sure that DOM nodes and window objects don't pass through, as well
          if (!obj || type(obj) !== 'object' || obj.nodeType || isWindow(obj)) {
            return false;
          }
          try {
            // Not own constructor property must be Object
            if (obj.constructor && !core_hasOwn.call(obj, 'constructor') && !core_hasOwn.call(obj.constructor.prototype, 'isPrototypeOf')) {
              return false;
            }
          } catch (e) {
            // IE8,9 Will throw exceptions on certain host objects #9897
            return false;
          }
          // Own properties are enumerated firstly, so to speed up,
          // if last one is own, then all properties are own.
          var key;
          for (key in obj) {
          }
          return key === undefined || core_hasOwn.call(obj, key);
        }
        function isFunction(obj) {
          return type(obj) === 'function';
        }
        var isArray = Array.isArray || function (obj) {
            return type(obj) === 'array';
          };
        function extend() {
          // originally extend() was recursive, but this ended up giving us
          // "call stack exceeded", so it's been unrolled to use a literal stack
          // (see https://github.com/pouchdb/pouchdb/issues/2543)
          var stack = [];
          var i = -1;
          var len = arguments.length;
          var args = new Array(len);
          while (++i < len) {
            args[i] = arguments[i];
          }
          var container = {};
          stack.push({
            args: args,
            result: {
              container: container,
              key: 'key'
            }
          });
          var next;
          while (next = stack.pop()) {
            extendInner(stack, next.args, next.result);
          }
          return container.key;
        }
        function extendInner(stack, args, result) {
          var options, name, src, copy, copyIsArray, clone, target = args[0] || {}, i = 1, length = args.length, deep = false, numericStringRegex = /\d+/, optionsIsArray;
          // Handle a deep copy situation
          if (typeof target === 'boolean') {
            deep = target;
            target = args[1] || {};
            // skip the boolean and the target
            i = 2;
          }
          // Handle case when target is a string or something (possible in deep copy)
          if (typeof target !== 'object' && !isFunction(target)) {
            target = {};
          }
          // extend jQuery itself if only one argument is passed
          if (length === i) {
            /* jshint validthis: true */
            target = this;
            --i;
          }
          for (; i < length; i++) {
            // Only deal with non-null/undefined values
            if ((options = args[i]) != null) {
              optionsIsArray = isArray(options);
              // Extend the base object
              for (name in options) {
                //if (options.hasOwnProperty(name)) {
                if (!(name in Object.prototype)) {
                  if (optionsIsArray && !numericStringRegex.test(name)) {
                    continue;
                  }
                  src = target[name];
                  copy = options[name];
                  // Prevent never-ending loop
                  if (target === copy) {
                    continue;
                  }
                  // Recurse if we're merging plain objects or arrays
                  if (deep && copy && (isPlainObject(copy) || (copyIsArray = isArray(copy)))) {
                    if (copyIsArray) {
                      copyIsArray = false;
                      clone = src && isArray(src) ? src : [];
                    } else {
                      clone = src && isPlainObject(src) ? src : {};
                    }
                    // Never move original objects, clone them
                    stack.push({
                      args: [
                        deep,
                        clone,
                        copy
                      ],
                      result: {
                        container: target,
                        key: name
                      }
                    });  // Don't bring in undefined values
                  } else if (copy !== undefined) {
                    if (!(isArray(options) && isFunction(copy))) {
                      target[name] = copy;
                    }
                  }
                }
              }
            }
          }
          // "Return" the modified object by setting the key
          // on the given container
          result.container[result.key] = target;
        }
        module.exports = extend;
      },
      {}
    ],
    67: [
      function (_dereq_, module, exports) {
        'use strict';
        var upsert = _dereq_(71);
        var utils = _dereq_(72);
        var Promise = utils.Promise;
        module.exports = function (opts) {
          var sourceDB = opts.db;
          var viewName = opts.viewName;
          var mapFun = opts.map;
          var reduceFun = opts.reduce;
          var temporary = opts.temporary;
          // the "undefined" part is for backwards compatibility
          var viewSignature = mapFun.toString() + (reduceFun && reduceFun.toString()) + 'undefined';
          if (!temporary && sourceDB._cachedViews) {
            var cachedView = sourceDB._cachedViews[viewSignature];
            if (cachedView) {
              return Promise.resolve(cachedView);
            }
          }
          return sourceDB.info().then(function (info) {
            var depDbName = info.db_name + '-mrview-' + (temporary ? 'temp' : utils.MD5(viewSignature));
            // save the view name in the source PouchDB so it can be cleaned up if necessary
            // (e.g. when the _design doc is deleted, remove all associated view data)
            function diffFunction(doc) {
              doc.views = doc.views || {};
              var fullViewName = viewName;
              if (fullViewName.indexOf('/') === -1) {
                fullViewName = viewName + '/' + viewName;
              }
              var depDbs = doc.views[fullViewName] = doc.views[fullViewName] || {};
              /* istanbul ignore if */
              if (depDbs[depDbName]) {
                return;  // no update necessary
              }
              depDbs[depDbName] = true;
              return doc;
            }
            return upsert(sourceDB, '_local/mrviews', diffFunction).then(function () {
              return sourceDB.registerDependentDatabase(depDbName).then(function (res) {
                var db = res.db;
                db.auto_compaction = true;
                var view = {
                    name: depDbName,
                    db: db,
                    sourceDB: sourceDB,
                    adapter: sourceDB.adapter,
                    mapFun: mapFun,
                    reduceFun: reduceFun
                  };
                return view.db.get('_local/lastSeq')['catch'](function (err) {
                  /* istanbul ignore if */
                  if (err.status !== 404) {
                    throw err;
                  }
                }).then(function (lastSeqDoc) {
                  view.seq = lastSeqDoc ? lastSeqDoc.seq : 0;
                  if (!temporary) {
                    sourceDB._cachedViews = sourceDB._cachedViews || {};
                    sourceDB._cachedViews[viewSignature] = view;
                    view.db.on('destroyed', function () {
                      delete sourceDB._cachedViews[viewSignature];
                    });
                  }
                  return view;
                });
              });
            });
          });
        };
      },
      {
        '71': 71,
        '72': 72
      }
    ],
    68: [
      function (_dereq_, module, exports) {
        'use strict';
        module.exports = function (func, emit, sum, log, isArray, toJSON) {
          /*jshint evil:true,unused:false */
          return eval('\'use strict\'; (' + func.replace(/;\s*$/, '') + ');');
        };
      },
      {}
    ],
    69: [
      function (_dereq_, module, exports) {
        (function (process) {
          'use strict';
          var pouchCollate = _dereq_(63);
          var TaskQueue = _dereq_(70);
          var collate = pouchCollate.collate;
          var toIndexableString = pouchCollate.toIndexableString;
          var normalizeKey = pouchCollate.normalizeKey;
          var createView = _dereq_(67);
          var evalFunc = _dereq_(68);
          var log;
          /* istanbul ignore else */
          if (typeof console !== 'undefined' && typeof console.log === 'function') {
            log = Function.prototype.bind.call(console.log, console);
          } else {
            log = function () {
            };
          }
          var utils = _dereq_(72);
          var Promise = utils.Promise;
          var persistentQueues = {};
          var tempViewQueue = new TaskQueue();
          var CHANGES_BATCH_SIZE = 50;
          function parseViewName(name) {
            // can be either 'ddocname/viewname' or just 'viewname'
            // (where the ddoc name is the same)
            return name.indexOf('/') === -1 ? [
              name,
              name
            ] : name.split('/');
          }
          function isGenOne(changes) {
            // only return true if the current change is 1-
            // and there are no other leafs
            return changes.length === 1 && /^1-/.test(changes[0].rev);
          }
          function emitError(db, e) {
            try {
              db.emit('error', e);
            } catch (err) {
              console.error('The user\'s map/reduce function threw an uncaught error.\n' + 'You can debug this error by doing:\n' + 'myDatabase.on(\'error\', function (err) { debugger; });\n' + 'Please double-check your map/reduce function.');
              console.error(e);
            }
          }
          function tryCode(db, fun, args) {
            // emit an event if there was an error thrown by a map/reduce function.
            // putting try/catches in a single function also avoids deoptimizations.
            try {
              return { output: fun.apply(null, args) };
            } catch (e) {
              emitError(db, e);
              return { error: e };
            }
          }
          function sortByKeyThenValue(x, y) {
            var keyCompare = collate(x.key, y.key);
            return keyCompare !== 0 ? keyCompare : collate(x.value, y.value);
          }
          function sliceResults(results, limit, skip) {
            skip = skip || 0;
            if (typeof limit === 'number') {
              return results.slice(skip, limit + skip);
            } else if (skip > 0) {
              return results.slice(skip);
            }
            return results;
          }
          function rowToDocId(row) {
            var val = row.value;
            // Users can explicitly specify a joined doc _id, or it
            // defaults to the doc _id that emitted the key/value.
            var docId = val && typeof val === 'object' && val._id || row.id;
            return docId;
          }
          function createBuiltInError(name) {
            var message = 'builtin ' + name + ' function requires map values to be numbers' + ' or number arrays';
            return new BuiltInError(message);
          }
          function sum(values) {
            var result = 0;
            for (var i = 0, len = values.length; i < len; i++) {
              var num = values[i];
              if (typeof num !== 'number') {
                if (Array.isArray(num)) {
                  // lists of numbers are also allowed, sum them separately
                  result = typeof result === 'number' ? [result] : result;
                  for (var j = 0, jLen = num.length; j < jLen; j++) {
                    var jNum = num[j];
                    if (typeof jNum !== 'number') {
                      throw createBuiltInError('_sum');
                    } else if (typeof result[j] === 'undefined') {
                      result.push(jNum);
                    } else {
                      result[j] += jNum;
                    }
                  }
                } else {
                  // not array/number
                  throw createBuiltInError('_sum');
                }
              } else if (typeof result === 'number') {
                result += num;
              } else {
                // add number to array
                result[0] += num;
              }
            }
            return result;
          }
          var builtInReduce = {
              _sum: function (keys, values) {
                return sum(values);
              },
              _count: function (keys, values) {
                return values.length;
              },
              _stats: function (keys, values) {
                // no need to implement rereduce=true, because Pouch
                // will never call it
                function sumsqr(values) {
                  var _sumsqr = 0;
                  for (var i = 0, len = values.length; i < len; i++) {
                    var num = values[i];
                    _sumsqr += num * num;
                  }
                  return _sumsqr;
                }
                return {
                  sum: sum(values),
                  min: Math.min.apply(null, values),
                  max: Math.max.apply(null, values),
                  count: values.length,
                  sumsqr: sumsqr(values)
                };
              }
            };
          function addHttpParam(paramName, opts, params, asJson) {
            // add an http param from opts to params, optionally json-encoded
            var val = opts[paramName];
            if (typeof val !== 'undefined') {
              if (asJson) {
                val = encodeURIComponent(JSON.stringify(val));
              }
              params.push(paramName + '=' + val);
            }
          }
          function checkQueryParseError(options, fun) {
            var startkeyName = options.descending ? 'endkey' : 'startkey';
            var endkeyName = options.descending ? 'startkey' : 'endkey';
            if (typeof options[startkeyName] !== 'undefined' && typeof options[endkeyName] !== 'undefined' && collate(options[startkeyName], options[endkeyName]) > 0) {
              throw new QueryParseError('No rows can match your key range, reverse your ' + 'start_key and end_key or set {descending : true}');
            } else if (fun.reduce && options.reduce !== false) {
              if (options.include_docs) {
                throw new QueryParseError('{include_docs:true} is invalid for reduce');
              } else if (options.keys && options.keys.length > 1 && !options.group && !options.group_level) {
                throw new QueryParseError('Multi-key fetches for reduce views must use {group: true}');
              }
            }
            if (options.group_level) {
              if (typeof options.group_level !== 'number') {
                throw new QueryParseError('Invalid value for integer: "' + options.group_level + '"');
              }
              if (options.group_level < 0) {
                throw new QueryParseError('Invalid value for positive integer: ' + '"' + options.group_level + '"');
              }
            }
          }
          function httpQuery(db, fun, opts) {
            // List of parameters to add to the PUT request
            var params = [];
            var body;
            var method = 'GET';
            // If opts.reduce exists and is defined, then add it to the list
            // of parameters.
            // If reduce=false then the results are that of only the map function
            // not the final result of map and reduce.
            addHttpParam('reduce', opts, params);
            addHttpParam('include_docs', opts, params);
            addHttpParam('attachments', opts, params);
            addHttpParam('limit', opts, params);
            addHttpParam('descending', opts, params);
            addHttpParam('group', opts, params);
            addHttpParam('group_level', opts, params);
            addHttpParam('skip', opts, params);
            addHttpParam('stale', opts, params);
            addHttpParam('conflicts', opts, params);
            addHttpParam('startkey', opts, params, true);
            addHttpParam('endkey', opts, params, true);
            addHttpParam('inclusive_end', opts, params);
            addHttpParam('key', opts, params, true);
            // Format the list of parameters into a valid URI query string
            params = params.join('&');
            params = params === '' ? '' : '?' + params;
            // If keys are supplied, issue a POST request to circumvent GET query string limits
            // see http://wiki.apache.org/couchdb/HTTP_view_API#Querying_Options
            if (typeof opts.keys !== 'undefined') {
              var MAX_URL_LENGTH = 2000;
              // according to http://stackoverflow.com/a/417184/680742,
              // the de facto URL length limit is 2000 characters
              var keysAsString = 'keys=' + encodeURIComponent(JSON.stringify(opts.keys));
              if (keysAsString.length + params.length + 1 <= MAX_URL_LENGTH) {
                // If the keys are short enough, do a GET. we do this to work around
                // Safari not understanding 304s on POSTs (see pouchdb/pouchdb#1239)
                params += (params[0] === '?' ? '&' : '?') + keysAsString;
              } else {
                method = 'POST';
                if (typeof fun === 'string') {
                  body = JSON.stringify({ keys: opts.keys });
                } else {
                  // fun is {map : mapfun}, so append to this
                  fun.keys = opts.keys;
                }
              }
            }
            // We are referencing a query defined in the design doc
            if (typeof fun === 'string') {
              var parts = parseViewName(fun);
              return db.request({
                method: method,
                url: '_design/' + parts[0] + '/_view/' + parts[1] + params,
                body: body
              });
            }
            // We are using a temporary view, terrible for performance but good for testing
            body = body || {};
            Object.keys(fun).forEach(function (key) {
              if (Array.isArray(fun[key])) {
                body[key] = fun[key];
              } else {
                body[key] = fun[key].toString();
              }
            });
            return db.request({
              method: 'POST',
              url: '_temp_view' + params,
              body: body
            });
          }
          function defaultsTo(value) {
            return function (reason) {
              /* istanbul ignore else */
              if (reason.status === 404) {
                return value;
              } else {
                throw reason;
              }
            };
          }
          // returns a promise for a list of docs to update, based on the input docId.
          // the order doesn't matter, because post-3.2.0, bulkDocs
          // is an atomic operation in all three adapters.
          function getDocsToPersist(docId, view, docIdsToChangesAndEmits) {
            var metaDocId = '_local/doc_' + docId;
            var defaultMetaDoc = {
                _id: metaDocId,
                keys: []
              };
            var docData = docIdsToChangesAndEmits[docId];
            var indexableKeysToKeyValues = docData.indexableKeysToKeyValues;
            var changes = docData.changes;
            function getMetaDoc() {
              if (isGenOne(changes)) {
                // generation 1, so we can safely assume initial state
                // for performance reasons (avoids unnecessary GETs)
                return Promise.resolve(defaultMetaDoc);
              }
              return view.db.get(metaDocId)['catch'](defaultsTo(defaultMetaDoc));
            }
            function getKeyValueDocs(metaDoc) {
              if (!metaDoc.keys.length) {
                // no keys, no need for a lookup
                return Promise.resolve({ rows: [] });
              }
              return view.db.allDocs({
                keys: metaDoc.keys,
                include_docs: true
              });
            }
            function processKvDocs(metaDoc, kvDocsRes) {
              var kvDocs = [];
              var oldKeysMap = {};
              for (var i = 0, len = kvDocsRes.rows.length; i < len; i++) {
                var row = kvDocsRes.rows[i];
                var doc = row.doc;
                if (!doc) {
                  // deleted
                  continue;
                }
                kvDocs.push(doc);
                oldKeysMap[doc._id] = true;
                doc._deleted = !indexableKeysToKeyValues[doc._id];
                if (!doc._deleted) {
                  var keyValue = indexableKeysToKeyValues[doc._id];
                  if ('value' in keyValue) {
                    doc.value = keyValue.value;
                  }
                }
              }
              var newKeys = Object.keys(indexableKeysToKeyValues);
              newKeys.forEach(function (key) {
                if (!oldKeysMap[key]) {
                  // new doc
                  var kvDoc = { _id: key };
                  var keyValue = indexableKeysToKeyValues[key];
                  if ('value' in keyValue) {
                    kvDoc.value = keyValue.value;
                  }
                  kvDocs.push(kvDoc);
                }
              });
              metaDoc.keys = utils.uniq(newKeys.concat(metaDoc.keys));
              kvDocs.push(metaDoc);
              return kvDocs;
            }
            return getMetaDoc().then(function (metaDoc) {
              return getKeyValueDocs(metaDoc).then(function (kvDocsRes) {
                return processKvDocs(metaDoc, kvDocsRes);
              });
            });
          }
          // updates all emitted key/value docs and metaDocs in the mrview database
          // for the given batch of documents from the source database
          function saveKeyValues(view, docIdsToChangesAndEmits, seq) {
            var seqDocId = '_local/lastSeq';
            return view.db.get(seqDocId)['catch'](defaultsTo({
              _id: seqDocId,
              seq: 0
            })).then(function (lastSeqDoc) {
              var docIds = Object.keys(docIdsToChangesAndEmits);
              return Promise.all(docIds.map(function (docId) {
                return getDocsToPersist(docId, view, docIdsToChangesAndEmits);
              })).then(function (listOfDocsToPersist) {
                var docsToPersist = utils.flatten(listOfDocsToPersist);
                lastSeqDoc.seq = seq;
                docsToPersist.push(lastSeqDoc);
                // write all docs in a single operation, update the seq once
                return view.db.bulkDocs({ docs: docsToPersist });
              });
            });
          }
          function getQueue(view) {
            var viewName = typeof view === 'string' ? view : view.name;
            var queue = persistentQueues[viewName];
            if (!queue) {
              queue = persistentQueues[viewName] = new TaskQueue();
            }
            return queue;
          }
          function updateView(view) {
            return utils.sequentialize(getQueue(view), function () {
              return updateViewInQueue(view);
            })();
          }
          function updateViewInQueue(view) {
            // bind the emit function once
            var mapResults;
            var doc;
            function emit(key, value) {
              var output = {
                  id: doc._id,
                  key: normalizeKey(key)
                };
              // Don't explicitly store the value unless it's defined and non-null.
              // This saves on storage space, because often people don't use it.
              if (typeof value !== 'undefined' && value !== null) {
                output.value = normalizeKey(value);
              }
              mapResults.push(output);
            }
            var mapFun;
            // for temp_views one can use emit(doc, emit), see #38
            if (typeof view.mapFun === 'function' && view.mapFun.length === 2) {
              var origMap = view.mapFun;
              mapFun = function (doc) {
                return origMap(doc, emit);
              };
            } else {
              mapFun = evalFunc(view.mapFun.toString(), emit, sum, log, Array.isArray, JSON.parse);
            }
            var currentSeq = view.seq || 0;
            function processChange(docIdsToChangesAndEmits, seq) {
              return function () {
                return saveKeyValues(view, docIdsToChangesAndEmits, seq);
              };
            }
            var queue = new TaskQueue();
            // TODO(neojski): https://github.com/daleharvey/pouchdb/issues/1521
            return new Promise(function (resolve, reject) {
              function complete() {
                queue.finish().then(function () {
                  view.seq = currentSeq;
                  resolve();
                });
              }
              function processNextBatch() {
                view.sourceDB.changes({
                  conflicts: true,
                  include_docs: true,
                  style: 'all_docs',
                  since: currentSeq,
                  limit: CHANGES_BATCH_SIZE
                }).on('complete', function (response) {
                  var results = response.results;
                  if (!results.length) {
                    return complete();
                  }
                  var docIdsToChangesAndEmits = {};
                  for (var i = 0, l = results.length; i < l; i++) {
                    var change = results[i];
                    if (change.doc._id[0] !== '_') {
                      mapResults = [];
                      doc = change.doc;
                      if (!doc._deleted) {
                        tryCode(view.sourceDB, mapFun, [doc]);
                      }
                      mapResults.sort(sortByKeyThenValue);
                      var indexableKeysToKeyValues = {};
                      var lastKey;
                      for (var j = 0, jl = mapResults.length; j < jl; j++) {
                        var obj = mapResults[j];
                        var complexKey = [
                            obj.key,
                            obj.id
                          ];
                        if (collate(obj.key, lastKey) === 0) {
                          complexKey.push(j);  // dup key+id, so make it unique
                        }
                        var indexableKey = toIndexableString(complexKey);
                        indexableKeysToKeyValues[indexableKey] = obj;
                        lastKey = obj.key;
                      }
                      docIdsToChangesAndEmits[change.doc._id] = {
                        indexableKeysToKeyValues: indexableKeysToKeyValues,
                        changes: change.changes
                      };
                    }
                    currentSeq = change.seq;
                  }
                  queue.add(processChange(docIdsToChangesAndEmits, currentSeq));
                  if (results.length < CHANGES_BATCH_SIZE) {
                    return complete();
                  }
                  return processNextBatch();
                }).on('error', onError);
                /* istanbul ignore next */
                function onError(err) {
                  reject(err);
                }
              }
              processNextBatch();
            });
          }
          function reduceView(view, results, options) {
            if (options.group_level === 0) {
              delete options.group_level;
            }
            var shouldGroup = options.group || options.group_level;
            var reduceFun;
            if (builtInReduce[view.reduceFun]) {
              reduceFun = builtInReduce[view.reduceFun];
            } else {
              reduceFun = evalFunc(view.reduceFun.toString(), null, sum, log, Array.isArray, JSON.parse);
            }
            var groups = [];
            var lvl = options.group_level;
            results.forEach(function (e) {
              var last = groups[groups.length - 1];
              var key = shouldGroup ? e.key : null;
              // only set group_level for array keys
              if (shouldGroup && Array.isArray(key) && typeof lvl === 'number') {
                key = key.length > lvl ? key.slice(0, lvl) : key;
              }
              if (last && collate(last.key[0][0], key) === 0) {
                last.key.push([
                  key,
                  e.id
                ]);
                last.value.push(e.value);
                return;
              }
              groups.push({
                key: [[
                    key,
                    e.id
                  ]],
                value: [e.value]
              });
            });
            for (var i = 0, len = groups.length; i < len; i++) {
              var e = groups[i];
              var reduceTry = tryCode(view.sourceDB, reduceFun, [
                  e.key,
                  e.value,
                  false
                ]);
              if (reduceTry.error && reduceTry.error instanceof BuiltInError) {
                // CouchDB returns an error if a built-in errors out
                throw reduceTry.error;
              }
              // CouchDB just sets the value to null if a non-built-in errors out
              e.value = reduceTry.error ? null : reduceTry.output;
              e.key = e.key[0][0];
            }
            // no total_rows/offset when reducing
            return { rows: sliceResults(groups, options.limit, options.skip) };
          }
          function queryView(view, opts) {
            return utils.sequentialize(getQueue(view), function () {
              return queryViewInQueue(view, opts);
            })();
          }
          function queryViewInQueue(view, opts) {
            var totalRows;
            var shouldReduce = view.reduceFun && opts.reduce !== false;
            var skip = opts.skip || 0;
            if (typeof opts.keys !== 'undefined' && !opts.keys.length) {
              // equivalent query
              opts.limit = 0;
              delete opts.keys;
            }
            function fetchFromView(viewOpts) {
              viewOpts.include_docs = true;
              return view.db.allDocs(viewOpts).then(function (res) {
                totalRows = res.total_rows;
                return res.rows.map(function (result) {
                  // implicit migration - in older versions of PouchDB,
                  // we explicitly stored the doc as {id: ..., key: ..., value: ...}
                  // this is tested in a migration test
                  /* istanbul ignore next */
                  if ('value' in result.doc && typeof result.doc.value === 'object' && result.doc.value !== null) {
                    var keys = Object.keys(result.doc.value).sort();
                    // this detection method is not perfect, but it's unlikely the user
                    // emitted a value which was an object with these 3 exact keys
                    var expectedKeys = [
                        'id',
                        'key',
                        'value'
                      ];
                    if (!(keys < expectedKeys || keys > expectedKeys)) {
                      return result.doc.value;
                    }
                  }
                  var parsedKeyAndDocId = pouchCollate.parseIndexableString(result.doc._id);
                  return {
                    key: parsedKeyAndDocId[0],
                    id: parsedKeyAndDocId[1],
                    value: 'value' in result.doc ? result.doc.value : null
                  };
                });
              });
            }
            function onMapResultsReady(rows) {
              var finalResults;
              if (shouldReduce) {
                finalResults = reduceView(view, rows, opts);
              } else {
                finalResults = {
                  total_rows: totalRows,
                  offset: skip,
                  rows: rows
                };
              }
              if (opts.include_docs) {
                var docIds = utils.uniq(rows.map(rowToDocId));
                return view.sourceDB.allDocs({
                  keys: docIds,
                  include_docs: true,
                  conflicts: opts.conflicts,
                  attachments: opts.attachments
                }).then(function (allDocsRes) {
                  var docIdsToDocs = {};
                  allDocsRes.rows.forEach(function (row) {
                    if (row.doc) {
                      docIdsToDocs['$' + row.id] = row.doc;
                    }
                  });
                  rows.forEach(function (row) {
                    var docId = rowToDocId(row);
                    var doc = docIdsToDocs['$' + docId];
                    if (doc) {
                      row.doc = doc;
                    }
                  });
                  return finalResults;
                });
              } else {
                return finalResults;
              }
            }
            var flatten = function (array) {
              return array.reduce(function (prev, cur) {
                return prev.concat(cur);
              });
            };
            if (typeof opts.keys !== 'undefined') {
              var keys = opts.keys;
              var fetchPromises = keys.map(function (key) {
                  var viewOpts = {
                      startkey: toIndexableString([key]),
                      endkey: toIndexableString([
                        key,
                        {}
                      ])
                    };
                  return fetchFromView(viewOpts);
                });
              return Promise.all(fetchPromises).then(flatten).then(onMapResultsReady);
            } else {
              // normal query, no 'keys'
              var viewOpts = { descending: opts.descending };
              if (typeof opts.startkey !== 'undefined') {
                viewOpts.startkey = opts.descending ? toIndexableString([
                  opts.startkey,
                  {}
                ]) : toIndexableString([opts.startkey]);
              }
              if (typeof opts.endkey !== 'undefined') {
                var inclusiveEnd = opts.inclusive_end !== false;
                if (opts.descending) {
                  inclusiveEnd = !inclusiveEnd;
                }
                viewOpts.endkey = toIndexableString(inclusiveEnd ? [
                  opts.endkey,
                  {}
                ] : [opts.endkey]);
              }
              if (typeof opts.key !== 'undefined') {
                var keyStart = toIndexableString([opts.key]);
                var keyEnd = toIndexableString([
                    opts.key,
                    {}
                  ]);
                if (viewOpts.descending) {
                  viewOpts.endkey = keyStart;
                  viewOpts.startkey = keyEnd;
                } else {
                  viewOpts.startkey = keyStart;
                  viewOpts.endkey = keyEnd;
                }
              }
              if (!shouldReduce) {
                if (typeof opts.limit === 'number') {
                  viewOpts.limit = opts.limit;
                }
                viewOpts.skip = skip;
              }
              return fetchFromView(viewOpts).then(onMapResultsReady);
            }
          }
          function httpViewCleanup(db) {
            return db.request({
              method: 'POST',
              url: '_view_cleanup'
            });
          }
          function localViewCleanup(db) {
            return db.get('_local/mrviews').then(function (metaDoc) {
              var docsToViews = {};
              Object.keys(metaDoc.views).forEach(function (fullViewName) {
                var parts = parseViewName(fullViewName);
                var designDocName = '_design/' + parts[0];
                var viewName = parts[1];
                docsToViews[designDocName] = docsToViews[designDocName] || {};
                docsToViews[designDocName][viewName] = true;
              });
              var opts = {
                  keys: Object.keys(docsToViews),
                  include_docs: true
                };
              return db.allDocs(opts).then(function (res) {
                var viewsToStatus = {};
                res.rows.forEach(function (row) {
                  var ddocName = row.key.substring(8);
                  Object.keys(docsToViews[row.key]).forEach(function (viewName) {
                    var fullViewName = ddocName + '/' + viewName;
                    /* istanbul ignore if */
                    if (!metaDoc.views[fullViewName]) {
                      // new format, without slashes, to support PouchDB 2.2.0
                      // migration test in pouchdb's browser.migration.js verifies this
                      fullViewName = viewName;
                    }
                    var viewDBNames = Object.keys(metaDoc.views[fullViewName]);
                    // design doc deleted, or view function nonexistent
                    var statusIsGood = row.doc && row.doc.views && row.doc.views[viewName];
                    viewDBNames.forEach(function (viewDBName) {
                      viewsToStatus[viewDBName] = viewsToStatus[viewDBName] || statusIsGood;
                    });
                  });
                });
                var dbsToDelete = Object.keys(viewsToStatus).filter(function (viewDBName) {
                    return !viewsToStatus[viewDBName];
                  });
                var destroyPromises = dbsToDelete.map(function (viewDBName) {
                    return utils.sequentialize(getQueue(viewDBName), function () {
                      return new db.constructor(viewDBName, db.__opts).destroy();
                    })();
                  });
                return Promise.all(destroyPromises).then(function () {
                  return { ok: true };
                });
              });
            }, defaultsTo({ ok: true }));
          }
          exports.viewCleanup = utils.callbackify(function () {
            var db = this;
            if (db.type() === 'http') {
              return httpViewCleanup(db);
            }
            return localViewCleanup(db);
          });
          function queryPromised(db, fun, opts) {
            if (db.type() === 'http') {
              return httpQuery(db, fun, opts);
            }
            if (typeof fun !== 'string') {
              // temp_view
              checkQueryParseError(opts, fun);
              var createViewOpts = {
                  db: db,
                  viewName: 'temp_view/temp_view',
                  map: fun.map,
                  reduce: fun.reduce,
                  temporary: true
                };
              tempViewQueue.add(function () {
                return createView(createViewOpts).then(function (view) {
                  function cleanup() {
                    return view.db.destroy();
                  }
                  return utils.fin(updateView(view).then(function () {
                    return queryView(view, opts);
                  }), cleanup);
                });
              });
              return tempViewQueue.finish();
            } else {
              // persistent view
              var fullViewName = fun;
              var parts = parseViewName(fullViewName);
              var designDocName = parts[0];
              var viewName = parts[1];
              return db.get('_design/' + designDocName).then(function (doc) {
                var fun = doc.views && doc.views[viewName];
                if (!fun || typeof fun.map !== 'string') {
                  throw new NotFoundError('ddoc ' + designDocName + ' has no view named ' + viewName);
                }
                checkQueryParseError(opts, fun);
                var createViewOpts = {
                    db: db,
                    viewName: fullViewName,
                    map: fun.map,
                    reduce: fun.reduce
                  };
                return createView(createViewOpts).then(function (view) {
                  if (opts.stale === 'ok' || opts.stale === 'update_after') {
                    if (opts.stale === 'update_after') {
                      process.nextTick(function () {
                        updateView(view);
                      });
                    }
                    return queryView(view, opts);
                  } else {
                    // stale not ok
                    return updateView(view).then(function () {
                      return queryView(view, opts);
                    });
                  }
                });
              });
            }
          }
          exports.query = function (fun, opts, callback) {
            if (typeof opts === 'function') {
              callback = opts;
              opts = {};
            }
            opts = utils.extend(true, {}, opts);
            if (typeof fun === 'function') {
              fun = { map: fun };
            }
            var db = this;
            var promise = Promise.resolve().then(function () {
                return queryPromised(db, fun, opts);
              });
            utils.promisedCallback(promise, callback);
            return promise;
          };
          function QueryParseError(message) {
            this.status = 400;
            this.name = 'query_parse_error';
            this.message = message;
            this.error = true;
            try {
              Error.captureStackTrace(this, QueryParseError);
            } catch (e) {
            }
          }
          utils.inherits(QueryParseError, Error);
          function NotFoundError(message) {
            this.status = 404;
            this.name = 'not_found';
            this.message = message;
            this.error = true;
            try {
              Error.captureStackTrace(this, NotFoundError);
            } catch (e) {
            }
          }
          utils.inherits(NotFoundError, Error);
          function BuiltInError(message) {
            this.status = 500;
            this.name = 'invalid_value';
            this.message = message;
            this.error = true;
            try {
              Error.captureStackTrace(this, BuiltInError);
            } catch (e) {
            }
          }
          utils.inherits(BuiltInError, Error);
        }.call(this, _dereq_(40)));
      },
      {
        '40': 40,
        '63': 63,
        '67': 67,
        '68': 68,
        '70': 70,
        '72': 72
      }
    ],
    70: [
      function (_dereq_, module, exports) {
        'use strict';
        /*
 * Simple task queue to sequentialize actions. Assumes callbacks will eventually fire (once).
 */
        var Promise = _dereq_(72).Promise;
        function TaskQueue() {
          this.promise = new Promise(function (fulfill) {
            fulfill();
          });
        }
        TaskQueue.prototype.add = function (promiseFactory) {
          this.promise = this.promise['catch'](function () {
          }).then(function () {
            return promiseFactory();
          });
          return this.promise;
        };
        TaskQueue.prototype.finish = function () {
          return this.promise;
        };
        module.exports = TaskQueue;
      },
      { '72': 72 }
    ],
    71: [
      function (_dereq_, module, exports) {
        'use strict';
        var upsert = _dereq_(73).upsert;
        module.exports = function (db, doc, diffFun) {
          return upsert.apply(db, [
            doc,
            diffFun
          ]);
        };
      },
      { '73': 73 }
    ],
    72: [
      function (_dereq_, module, exports) {
        (function (process, global) {
          'use strict';
          /* istanbul ignore if */
          if (typeof global.Promise === 'function') {
            exports.Promise = global.Promise;
          } else {
            exports.Promise = _dereq_(48);
          }
          exports.inherits = _dereq_(44);
          exports.extend = _dereq_(66);
          var argsarray = _dereq_(37);
          exports.promisedCallback = function (promise, callback) {
            if (callback) {
              promise.then(function (res) {
                process.nextTick(function () {
                  callback(null, res);
                });
              }, function (reason) {
                process.nextTick(function () {
                  callback(reason);
                });
              });
            }
            return promise;
          };
          exports.callbackify = function (fun) {
            return argsarray(function (args) {
              var cb = args.pop();
              var promise = fun.apply(this, args);
              if (typeof cb === 'function') {
                exports.promisedCallback(promise, cb);
              }
              return promise;
            });
          };
          // Promise finally util similar to Q.finally
          exports.fin = function (promise, cb) {
            return promise.then(function (res) {
              var promise2 = cb();
              if (typeof promise2.then === 'function') {
                return promise2.then(function () {
                  return res;
                });
              }
              return res;
            }, function (reason) {
              var promise2 = cb();
              if (typeof promise2.then === 'function') {
                return promise2.then(function () {
                  throw reason;
                });
              }
              throw reason;
            });
          };
          exports.sequentialize = function (queue, promiseFactory) {
            return function () {
              var args = arguments;
              var that = this;
              return queue.add(function () {
                return promiseFactory.apply(that, args);
              });
            };
          };
          exports.flatten = function (arrs) {
            var res = [];
            for (var i = 0, len = arrs.length; i < len; i++) {
              res = res.concat(arrs[i]);
            }
            return res;
          };
          // uniq an array of strings, order not guaranteed
          // similar to underscore/lodash _.uniq
          exports.uniq = function (arr) {
            var map = {};
            for (var i = 0, len = arr.length; i < len; i++) {
              map['$' + arr[i]] = true;
            }
            var keys = Object.keys(map);
            var output = new Array(keys.length);
            for (i = 0, len = keys.length; i < len; i++) {
              output[i] = keys[i].substring(1);
            }
            return output;
          };
          var crypto = _dereq_(38);
          var Md5 = _dereq_(74);
          exports.MD5 = function (string) {
            /* istanbul ignore else */
            if (!process.browser) {
              return crypto.createHash('md5').update(string).digest('hex');
            } else {
              return Md5.hash(string);
            }
          };
        }.call(this, _dereq_(40), typeof global !== 'undefined' ? global : typeof self !== 'undefined' ? self : typeof window !== 'undefined' ? window : {}));
      },
      {
        '37': 37,
        '38': 38,
        '40': 40,
        '44': 44,
        '48': 48,
        '66': 66,
        '74': 74
      }
    ],
    73: [
      function (_dereq_, module, exports) {
        (function (global) {
          'use strict';
          var PouchPromise;
          /* istanbul ignore next */
          if (typeof window !== 'undefined' && window.PouchDB) {
            PouchPromise = window.PouchDB.utils.Promise;
          } else {
            PouchPromise = typeof global.Promise === 'function' ? global.Promise : _dereq_(48);
          }
          // this is essentially the "update sugar" function from daleharvey/pouchdb#1388
          // the diffFun tells us what delta to apply to the doc.  it either returns
          // the doc, or false if it doesn't need to do an update after all
          function upsertInner(db, docId, diffFun) {
            return new PouchPromise(function (fulfill, reject) {
              if (typeof docId !== 'string') {
                return reject(new Error('doc id is required'));
              }
              db.get(docId, function (err, doc) {
                if (err) {
                  /* istanbul ignore next */
                  if (err.status !== 404) {
                    return reject(err);
                  }
                  doc = {};
                }
                // the user might change the _rev, so save it for posterity
                var docRev = doc._rev;
                var newDoc = diffFun(doc);
                if (!newDoc) {
                  // if the diffFun returns falsy, we short-circuit as
                  // an optimization
                  return fulfill({
                    updated: false,
                    rev: docRev
                  });
                }
                // users aren't allowed to modify these values,
                // so reset them here
                newDoc._id = docId;
                newDoc._rev = docRev;
                fulfill(tryAndPut(db, newDoc, diffFun));
              });
            });
          }
          function tryAndPut(db, doc, diffFun) {
            return db.put(doc).then(function (res) {
              return {
                updated: true,
                rev: res.rev
              };
            }, function (err) {
              /* istanbul ignore next */
              if (err.status !== 409) {
                throw err;
              }
              return upsertInner(db, doc._id, diffFun);
            });
          }
          exports.upsert = function upsert(docId, diffFun, cb) {
            var db = this;
            var promise = upsertInner(db, docId, diffFun);
            if (typeof cb !== 'function') {
              return promise;
            }
            promise.then(function (resp) {
              cb(null, resp);
            }, cb);
          };
          exports.putIfNotExists = function putIfNotExists(docId, doc, cb) {
            var db = this;
            if (typeof docId !== 'string') {
              cb = doc;
              doc = docId;
              docId = doc._id;
            }
            var diffFun = function (existingDoc) {
              if (existingDoc._rev) {
                return false;  // do nothing
              }
              return doc;
            };
            var promise = upsertInner(db, docId, diffFun);
            if (typeof cb !== 'function') {
              return promise;
            }
            promise.then(function (resp) {
              cb(null, resp);
            }, cb);
          };
          /* istanbul ignore next */
          if (typeof window !== 'undefined' && window.PouchDB) {
            window.PouchDB.plugin(exports);
          }
        }.call(this, typeof global !== 'undefined' ? global : typeof self !== 'undefined' ? self : typeof window !== 'undefined' ? window : {}));
      },
      { '48': 48 }
    ],
    74: [
      function (_dereq_, module, exports) {
        /*jshint bitwise:false*/
        /*global unescape*/
        (function (factory) {
          if (typeof exports === 'object') {
            // Node/CommonJS
            module.exports = factory();
          } else if (typeof define === 'function' && define.amd) {
            // AMD
            define(factory);
          } else {
            // Browser globals (with support for web workers)
            var glob;
            try {
              glob = window;
            } catch (e) {
              glob = self;
            }
            glob.SparkMD5 = factory();
          }
        }(function (undefined) {
          'use strict';
          ////////////////////////////////////////////////////////////////////////////
          /*
     * Fastest md5 implementation around (JKM md5)
     * Credits: Joseph Myers
     *
     * @see http://www.myersdaily.org/joseph/javascript/md5-text.html
     * @see http://jsperf.com/md5-shootout/7
     */
          /* this function is much faster,
      so if possible we use it. Some IEs
      are the only ones I know of that
      need the idiotic second function,
      generated by an if clause.  */
          var add32 = function (a, b) {
              return a + b & 4294967295;
            }, cmn = function (q, a, b, x, s, t) {
              a = add32(add32(a, q), add32(x, t));
              return add32(a << s | a >>> 32 - s, b);
            }, ff = function (a, b, c, d, x, s, t) {
              return cmn(b & c | ~b & d, a, b, x, s, t);
            }, gg = function (a, b, c, d, x, s, t) {
              return cmn(b & d | c & ~d, a, b, x, s, t);
            }, hh = function (a, b, c, d, x, s, t) {
              return cmn(b ^ c ^ d, a, b, x, s, t);
            }, ii = function (a, b, c, d, x, s, t) {
              return cmn(c ^ (b | ~d), a, b, x, s, t);
            }, md5cycle = function (x, k) {
              var a = x[0], b = x[1], c = x[2], d = x[3];
              a = ff(a, b, c, d, k[0], 7, -680876936);
              d = ff(d, a, b, c, k[1], 12, -389564586);
              c = ff(c, d, a, b, k[2], 17, 606105819);
              b = ff(b, c, d, a, k[3], 22, -1044525330);
              a = ff(a, b, c, d, k[4], 7, -176418897);
              d = ff(d, a, b, c, k[5], 12, 1200080426);
              c = ff(c, d, a, b, k[6], 17, -1473231341);
              b = ff(b, c, d, a, k[7], 22, -45705983);
              a = ff(a, b, c, d, k[8], 7, 1770035416);
              d = ff(d, a, b, c, k[9], 12, -1958414417);
              c = ff(c, d, a, b, k[10], 17, -42063);
              b = ff(b, c, d, a, k[11], 22, -1990404162);
              a = ff(a, b, c, d, k[12], 7, 1804603682);
              d = ff(d, a, b, c, k[13], 12, -40341101);
              c = ff(c, d, a, b, k[14], 17, -1502002290);
              b = ff(b, c, d, a, k[15], 22, 1236535329);
              a = gg(a, b, c, d, k[1], 5, -165796510);
              d = gg(d, a, b, c, k[6], 9, -1069501632);
              c = gg(c, d, a, b, k[11], 14, 643717713);
              b = gg(b, c, d, a, k[0], 20, -373897302);
              a = gg(a, b, c, d, k[5], 5, -701558691);
              d = gg(d, a, b, c, k[10], 9, 38016083);
              c = gg(c, d, a, b, k[15], 14, -660478335);
              b = gg(b, c, d, a, k[4], 20, -405537848);
              a = gg(a, b, c, d, k[9], 5, 568446438);
              d = gg(d, a, b, c, k[14], 9, -1019803690);
              c = gg(c, d, a, b, k[3], 14, -187363961);
              b = gg(b, c, d, a, k[8], 20, 1163531501);
              a = gg(a, b, c, d, k[13], 5, -1444681467);
              d = gg(d, a, b, c, k[2], 9, -51403784);
              c = gg(c, d, a, b, k[7], 14, 1735328473);
              b = gg(b, c, d, a, k[12], 20, -1926607734);
              a = hh(a, b, c, d, k[5], 4, -378558);
              d = hh(d, a, b, c, k[8], 11, -2022574463);
              c = hh(c, d, a, b, k[11], 16, 1839030562);
              b = hh(b, c, d, a, k[14], 23, -35309556);
              a = hh(a, b, c, d, k[1], 4, -1530992060);
              d = hh(d, a, b, c, k[4], 11, 1272893353);
              c = hh(c, d, a, b, k[7], 16, -155497632);
              b = hh(b, c, d, a, k[10], 23, -1094730640);
              a = hh(a, b, c, d, k[13], 4, 681279174);
              d = hh(d, a, b, c, k[0], 11, -358537222);
              c = hh(c, d, a, b, k[3], 16, -722521979);
              b = hh(b, c, d, a, k[6], 23, 76029189);
              a = hh(a, b, c, d, k[9], 4, -640364487);
              d = hh(d, a, b, c, k[12], 11, -421815835);
              c = hh(c, d, a, b, k[15], 16, 530742520);
              b = hh(b, c, d, a, k[2], 23, -995338651);
              a = ii(a, b, c, d, k[0], 6, -198630844);
              d = ii(d, a, b, c, k[7], 10, 1126891415);
              c = ii(c, d, a, b, k[14], 15, -1416354905);
              b = ii(b, c, d, a, k[5], 21, -57434055);
              a = ii(a, b, c, d, k[12], 6, 1700485571);
              d = ii(d, a, b, c, k[3], 10, -1894986606);
              c = ii(c, d, a, b, k[10], 15, -1051523);
              b = ii(b, c, d, a, k[1], 21, -2054922799);
              a = ii(a, b, c, d, k[8], 6, 1873313359);
              d = ii(d, a, b, c, k[15], 10, -30611744);
              c = ii(c, d, a, b, k[6], 15, -1560198380);
              b = ii(b, c, d, a, k[13], 21, 1309151649);
              a = ii(a, b, c, d, k[4], 6, -145523070);
              d = ii(d, a, b, c, k[11], 10, -1120210379);
              c = ii(c, d, a, b, k[2], 15, 718787259);
              b = ii(b, c, d, a, k[9], 21, -343485551);
              x[0] = add32(a, x[0]);
              x[1] = add32(b, x[1]);
              x[2] = add32(c, x[2]);
              x[3] = add32(d, x[3]);
            },
            /* there needs to be support for Unicode here,
       * unless we pretend that we can redefine the MD-5
       * algorithm for multi-byte characters (perhaps
       * by adding every four 16-bit characters and
       * shortening the sum to 32 bits). Otherwise
       * I suggest performing MD-5 as if every character
       * was two bytes--e.g., 0040 0025 = @%--but then
       * how will an ordinary MD-5 sum be matched?
       * There is no way to standardize text to something
       * like UTF-8 before transformation; speed cost is
       * utterly prohibitive. The JavaScript standard
       * itself needs to look at this: it should start
       * providing access to strings as preformed UTF-8
       * 8-bit unsigned value arrays.
       */
            md5blk = function (s) {
              var md5blks = [], i;
              /* Andy King said do it this way. */
              for (i = 0; i < 64; i += 4) {
                md5blks[i >> 2] = s.charCodeAt(i) + (s.charCodeAt(i + 1) << 8) + (s.charCodeAt(i + 2) << 16) + (s.charCodeAt(i + 3) << 24);
              }
              return md5blks;
            }, md5blk_array = function (a) {
              var md5blks = [], i;
              /* Andy King said do it this way. */
              for (i = 0; i < 64; i += 4) {
                md5blks[i >> 2] = a[i] + (a[i + 1] << 8) + (a[i + 2] << 16) + (a[i + 3] << 24);
              }
              return md5blks;
            }, md51 = function (s) {
              var n = s.length, state = [
                  1732584193,
                  -271733879,
                  -1732584194,
                  271733878
                ], i, length, tail, tmp, lo, hi;
              for (i = 64; i <= n; i += 64) {
                md5cycle(state, md5blk(s.substring(i - 64, i)));
              }
              s = s.substring(i - 64);
              length = s.length;
              tail = [
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0
              ];
              for (i = 0; i < length; i += 1) {
                tail[i >> 2] |= s.charCodeAt(i) << (i % 4 << 3);
              }
              tail[i >> 2] |= 128 << (i % 4 << 3);
              if (i > 55) {
                md5cycle(state, tail);
                for (i = 0; i < 16; i += 1) {
                  tail[i] = 0;
                }
              }
              // Beware that the final length might not fit in 32 bits so we take care of that
              tmp = n * 8;
              tmp = tmp.toString(16).match(/(.*?)(.{0,8})$/);
              lo = parseInt(tmp[2], 16);
              hi = parseInt(tmp[1], 16) || 0;
              tail[14] = lo;
              tail[15] = hi;
              md5cycle(state, tail);
              return state;
            }, md51_array = function (a) {
              var n = a.length, state = [
                  1732584193,
                  -271733879,
                  -1732584194,
                  271733878
                ], i, length, tail, tmp, lo, hi;
              for (i = 64; i <= n; i += 64) {
                md5cycle(state, md5blk_array(a.subarray(i - 64, i)));
              }
              // Not sure if it is a bug, however IE10 will always produce a sub array of length 1
              // containing the last element of the parent array if the sub array specified starts
              // beyond the length of the parent array - weird.
              // https://connect.microsoft.com/IE/feedback/details/771452/typed-array-subarray-issue
              a = i - 64 < n ? a.subarray(i - 64) : new Uint8Array(0);
              length = a.length;
              tail = [
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0
              ];
              for (i = 0; i < length; i += 1) {
                tail[i >> 2] |= a[i] << (i % 4 << 3);
              }
              tail[i >> 2] |= 128 << (i % 4 << 3);
              if (i > 55) {
                md5cycle(state, tail);
                for (i = 0; i < 16; i += 1) {
                  tail[i] = 0;
                }
              }
              // Beware that the final length might not fit in 32 bits so we take care of that
              tmp = n * 8;
              tmp = tmp.toString(16).match(/(.*?)(.{0,8})$/);
              lo = parseInt(tmp[2], 16);
              hi = parseInt(tmp[1], 16) || 0;
              tail[14] = lo;
              tail[15] = hi;
              md5cycle(state, tail);
              return state;
            }, hex_chr = [
              '0',
              '1',
              '2',
              '3',
              '4',
              '5',
              '6',
              '7',
              '8',
              '9',
              'a',
              'b',
              'c',
              'd',
              'e',
              'f'
            ], rhex = function (n) {
              var s = '', j;
              for (j = 0; j < 4; j += 1) {
                s += hex_chr[n >> j * 8 + 4 & 15] + hex_chr[n >> j * 8 & 15];
              }
              return s;
            }, hex = function (x) {
              var i;
              for (i = 0; i < x.length; i += 1) {
                x[i] = rhex(x[i]);
              }
              return x.join('');
            }, md5 = function (s) {
              return hex(md51(s));
            },
            ////////////////////////////////////////////////////////////////////////////
            /**
     * SparkMD5 OOP implementation.
     *
     * Use this class to perform an incremental md5, otherwise use the
     * static methods instead.
     */
            SparkMD5 = function () {
              // call reset to init the instance
              this.reset();
            };
          // In some cases the fast add32 function cannot be used..
          if (md5('hello') !== '5d41402abc4b2a76b9719d911017c592') {
            add32 = function (x, y) {
              var lsw = (x & 65535) + (y & 65535), msw = (x >> 16) + (y >> 16) + (lsw >> 16);
              return msw << 16 | lsw & 65535;
            };
          }
          /**
     * Appends a string.
     * A conversion will be applied if an utf8 string is detected.
     *
     * @param {String} str The string to be appended
     *
     * @return {SparkMD5} The instance itself
     */
          SparkMD5.prototype.append = function (str) {
            // converts the string to utf8 bytes if necessary
            if (/[\u0080-\uFFFF]/.test(str)) {
              str = unescape(encodeURIComponent(str));
            }
            // then append as binary
            this.appendBinary(str);
            return this;
          };
          /**
     * Appends a binary string.
     *
     * @param {String} contents The binary string to be appended
     *
     * @return {SparkMD5} The instance itself
     */
          SparkMD5.prototype.appendBinary = function (contents) {
            this._buff += contents;
            this._length += contents.length;
            var length = this._buff.length, i;
            for (i = 64; i <= length; i += 64) {
              md5cycle(this._state, md5blk(this._buff.substring(i - 64, i)));
            }
            this._buff = this._buff.substr(i - 64);
            return this;
          };
          /**
     * Finishes the incremental computation, reseting the internal state and
     * returning the result.
     * Use the raw parameter to obtain the raw result instead of the hex one.
     *
     * @param {Boolean} raw True to get the raw result, false to get the hex result
     *
     * @return {String|Array} The result
     */
          SparkMD5.prototype.end = function (raw) {
            var buff = this._buff, length = buff.length, i, tail = [
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0
              ], ret;
            for (i = 0; i < length; i += 1) {
              tail[i >> 2] |= buff.charCodeAt(i) << (i % 4 << 3);
            }
            this._finish(tail, length);
            ret = !!raw ? this._state : hex(this._state);
            this.reset();
            return ret;
          };
          /**
     * Finish the final calculation based on the tail.
     *
     * @param {Array}  tail   The tail (will be modified)
     * @param {Number} length The length of the remaining buffer
     */
          SparkMD5.prototype._finish = function (tail, length) {
            var i = length, tmp, lo, hi;
            tail[i >> 2] |= 128 << (i % 4 << 3);
            if (i > 55) {
              md5cycle(this._state, tail);
              for (i = 0; i < 16; i += 1) {
                tail[i] = 0;
              }
            }
            // Do the final computation based on the tail and length
            // Beware that the final length may not fit in 32 bits so we take care of that
            tmp = this._length * 8;
            tmp = tmp.toString(16).match(/(.*?)(.{0,8})$/);
            lo = parseInt(tmp[2], 16);
            hi = parseInt(tmp[1], 16) || 0;
            tail[14] = lo;
            tail[15] = hi;
            md5cycle(this._state, tail);
          };
          /**
     * Resets the internal state of the computation.
     *
     * @return {SparkMD5} The instance itself
     */
          SparkMD5.prototype.reset = function () {
            this._buff = '';
            this._length = 0;
            this._state = [
              1732584193,
              -271733879,
              -1732584194,
              271733878
            ];
            return this;
          };
          /**
     * Releases memory used by the incremental buffer and other aditional
     * resources. If you plan to use the instance again, use reset instead.
     */
          SparkMD5.prototype.destroy = function () {
            delete this._state;
            delete this._buff;
            delete this._length;
          };
          /**
     * Performs the md5 hash on a string.
     * A conversion will be applied if utf8 string is detected.
     *
     * @param {String}  str The string
     * @param {Boolean} raw True to get the raw result, false to get the hex result
     *
     * @return {String|Array} The result
     */
          SparkMD5.hash = function (str, raw) {
            // converts the string to utf8 bytes if necessary
            if (/[\u0080-\uFFFF]/.test(str)) {
              str = unescape(encodeURIComponent(str));
            }
            var hash = md51(str);
            return !!raw ? hash : hex(hash);
          };
          /**
     * Performs the md5 hash on a binary string.
     *
     * @param {String}  content The binary string
     * @param {Boolean} raw     True to get the raw result, false to get the hex result
     *
     * @return {String|Array} The result
     */
          SparkMD5.hashBinary = function (content, raw) {
            var hash = md51(content);
            return !!raw ? hash : hex(hash);
          };
          /**
     * SparkMD5 OOP implementation for array buffers.
     *
     * Use this class to perform an incremental md5 ONLY for array buffers.
     */
          SparkMD5.ArrayBuffer = function () {
            // call reset to init the instance
            this.reset();
          };
          ////////////////////////////////////////////////////////////////////////////
          /**
     * Appends an array buffer.
     *
     * @param {ArrayBuffer} arr The array to be appended
     *
     * @return {SparkMD5.ArrayBuffer} The instance itself
     */
          SparkMD5.ArrayBuffer.prototype.append = function (arr) {
            // TODO: we could avoid the concatenation here but the algorithm would be more complex
            //       if you find yourself needing extra performance, please make a PR.
            var buff = this._concatArrayBuffer(this._buff, arr), length = buff.length, i;
            this._length += arr.byteLength;
            for (i = 64; i <= length; i += 64) {
              md5cycle(this._state, md5blk_array(buff.subarray(i - 64, i)));
            }
            // Avoids IE10 weirdness (documented above)
            this._buff = i - 64 < length ? buff.subarray(i - 64) : new Uint8Array(0);
            return this;
          };
          /**
     * Finishes the incremental computation, reseting the internal state and
     * returning the result.
     * Use the raw parameter to obtain the raw result instead of the hex one.
     *
     * @param {Boolean} raw True to get the raw result, false to get the hex result
     *
     * @return {String|Array} The result
     */
          SparkMD5.ArrayBuffer.prototype.end = function (raw) {
            var buff = this._buff, length = buff.length, tail = [
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0
              ], i, ret;
            for (i = 0; i < length; i += 1) {
              tail[i >> 2] |= buff[i] << (i % 4 << 3);
            }
            this._finish(tail, length);
            ret = !!raw ? this._state : hex(this._state);
            this.reset();
            return ret;
          };
          SparkMD5.ArrayBuffer.prototype._finish = SparkMD5.prototype._finish;
          /**
     * Resets the internal state of the computation.
     *
     * @return {SparkMD5.ArrayBuffer} The instance itself
     */
          SparkMD5.ArrayBuffer.prototype.reset = function () {
            this._buff = new Uint8Array(0);
            this._length = 0;
            this._state = [
              1732584193,
              -271733879,
              -1732584194,
              271733878
            ];
            return this;
          };
          /**
     * Releases memory used by the incremental buffer and other aditional
     * resources. If you plan to use the instance again, use reset instead.
     */
          SparkMD5.ArrayBuffer.prototype.destroy = SparkMD5.prototype.destroy;
          /**
     * Concats two array buffers, returning a new one.
     *
     * @param  {ArrayBuffer} first  The first array buffer
     * @param  {ArrayBuffer} second The second array buffer
     *
     * @return {ArrayBuffer} The new array buffer
     */
          SparkMD5.ArrayBuffer.prototype._concatArrayBuffer = function (first, second) {
            var firstLength = first.length, result = new Uint8Array(firstLength + second.byteLength);
            result.set(first);
            result.set(new Uint8Array(second), firstLength);
            return result;
          };
          /**
     * Performs the md5 hash on an array buffer.
     *
     * @param {ArrayBuffer} arr The array buffer
     * @param {Boolean}     raw True to get the raw result, false to get the hex result
     *
     * @return {String|Array} The result
     */
          SparkMD5.ArrayBuffer.hash = function (arr, raw) {
            var hash = md51_array(new Uint8Array(arr));
            return !!raw ? hash : hex(hash);
          };
          return SparkMD5;
        }));
      },
      {}
    ],
    75: [
      function (_dereq_, module, exports) {
        'use strict';
        /**
 * Stringify/parse functions that don't operate
 * recursively, so they avoid call stack exceeded
 * errors.
 */
        exports.stringify = function stringify(input) {
          var queue = [];
          queue.push({ obj: input });
          var res = '';
          var next, obj, prefix, val, i, arrayPrefix, keys, k, key, value, objPrefix;
          while (next = queue.pop()) {
            obj = next.obj;
            prefix = next.prefix || '';
            val = next.val || '';
            res += prefix;
            if (val) {
              res += val;
            } else if (typeof obj !== 'object') {
              res += typeof obj === 'undefined' ? null : JSON.stringify(obj);
            } else if (obj === null) {
              res += 'null';
            } else if (Array.isArray(obj)) {
              queue.push({ val: ']' });
              for (i = obj.length - 1; i >= 0; i--) {
                arrayPrefix = i === 0 ? '' : ',';
                queue.push({
                  obj: obj[i],
                  prefix: arrayPrefix
                });
              }
              queue.push({ val: '[' });
            } else {
              // object
              keys = [];
              for (k in obj) {
                if (obj.hasOwnProperty(k)) {
                  keys.push(k);
                }
              }
              queue.push({ val: '}' });
              for (i = keys.length - 1; i >= 0; i--) {
                key = keys[i];
                value = obj[key];
                objPrefix = i > 0 ? ',' : '';
                objPrefix += JSON.stringify(key) + ':';
                queue.push({
                  obj: value,
                  prefix: objPrefix
                });
              }
              queue.push({ val: '{' });
            }
          }
          return res;
        };
        // Convenience function for the parse function.
        // This pop function is basically copied from
        // pouchCollate.parseIndexableString
        function pop(obj, stack, metaStack) {
          var lastMetaElement = metaStack[metaStack.length - 1];
          if (obj === lastMetaElement.element) {
            // popping a meta-element, e.g. an object whose value is another object
            metaStack.pop();
            lastMetaElement = metaStack[metaStack.length - 1];
          }
          var element = lastMetaElement.element;
          var lastElementIndex = lastMetaElement.index;
          if (Array.isArray(element)) {
            element.push(obj);
          } else if (lastElementIndex === stack.length - 2) {
            // obj with key+value
            var key = stack.pop();
            element[key] = obj;
          } else {
            stack.push(obj);  // obj with key only
          }
        }
        exports.parse = function (str) {
          var stack = [];
          var metaStack = [];
          // stack for arrays and objects
          var i = 0;
          var collationIndex, parsedNum, numChar;
          var parsedString, lastCh, numConsecutiveSlashes, ch;
          var arrayElement, objElement;
          while (true) {
            collationIndex = str[i++];
            if (collationIndex === '}' || collationIndex === ']' || typeof collationIndex === 'undefined') {
              if (stack.length === 1) {
                return stack.pop();
              } else {
                pop(stack.pop(), stack, metaStack);
                continue;
              }
            }
            switch (collationIndex) {
            case ' ':
            case '\t':
            case '\n':
            case ':':
            case ',':
              break;
            case 'n':
              i += 3;
              // 'ull'
              pop(null, stack, metaStack);
              break;
            case 't':
              i += 3;
              // 'rue'
              pop(true, stack, metaStack);
              break;
            case 'f':
              i += 4;
              // 'alse'
              pop(false, stack, metaStack);
              break;
            case '0':
            case '1':
            case '2':
            case '3':
            case '4':
            case '5':
            case '6':
            case '7':
            case '8':
            case '9':
            case '-':
              parsedNum = '';
              i--;
              while (true) {
                numChar = str[i++];
                if (/[\d\.\-e\+]/.test(numChar)) {
                  parsedNum += numChar;
                } else {
                  i--;
                  break;
                }
              }
              pop(parseFloat(parsedNum), stack, metaStack);
              break;
            case '"':
              parsedString = '';
              lastCh = void 0;
              numConsecutiveSlashes = 0;
              while (true) {
                ch = str[i++];
                if (ch !== '"' || lastCh === '\\' && numConsecutiveSlashes % 2 === 1) {
                  parsedString += ch;
                  lastCh = ch;
                  if (lastCh === '\\') {
                    numConsecutiveSlashes++;
                  } else {
                    numConsecutiveSlashes = 0;
                  }
                } else {
                  break;
                }
              }
              pop(JSON.parse('"' + parsedString + '"'), stack, metaStack);
              break;
            case '[':
              arrayElement = {
                element: [],
                index: stack.length
              };
              stack.push(arrayElement.element);
              metaStack.push(arrayElement);
              break;
            case '{':
              objElement = {
                element: {},
                index: stack.length
              };
              stack.push(objElement.element);
              metaStack.push(objElement);
              break;
            default:
              throw new Error('unexpectedly reached end of input: ' + collationIndex);
            }
          }
        };
      },
      {}
    ],
    76: [
      function (_dereq_, module, exports) {
        (function (process) {
          'use strict';
          var PouchDB = _dereq_(32);
          module.exports = PouchDB;
          PouchDB.ajax = _dereq_(17);
          PouchDB.utils = _dereq_(35);
          PouchDB.Errors = _dereq_(20);
          PouchDB.replicate = _dereq_(31).replicate;
          PouchDB.sync = _dereq_(33);
          PouchDB.version = _dereq_(36);
          var httpAdapter = _dereq_(2);
          PouchDB.adapter('http', httpAdapter);
          PouchDB.adapter('https', httpAdapter);
          PouchDB.adapter('idb', _dereq_(8));
          PouchDB.adapter('websql', _dereq_(13));
          PouchDB.plugin(_dereq_(69));
          if (!process.browser) {
            var ldbAdapter = _dereq_(38);
            PouchDB.adapter('ldb', ldbAdapter);
            PouchDB.adapter('leveldb', ldbAdapter);
          }
        }.call(this, _dereq_(40)));
      },
      {
        '13': 13,
        '17': 17,
        '2': 2,
        '20': 20,
        '31': 31,
        '32': 32,
        '33': 33,
        '35': 35,
        '36': 36,
        '38': 38,
        '40': 40,
        '69': 69,
        '8': 8
      }
    ]
  }, {}, [76])(76);
});
//! moment.js
//! version : 2.10.6
//! authors : Tim Wood, Iskren Chernev, Moment.js contributors
//! license : MIT
//! momentjs.com
(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() : typeof define === 'function' && define.amd ? define(factory) : global.moment = factory();
}(this, function () {
  'use strict';
  var hookCallback;
  function utils_hooks__hooks() {
    return hookCallback.apply(null, arguments);
  }
  // This is done to register the method called with moment()
  // without creating circular dependencies.
  function setHookCallback(callback) {
    hookCallback = callback;
  }
  function isArray(input) {
    return Object.prototype.toString.call(input) === '[object Array]';
  }
  function isDate(input) {
    return input instanceof Date || Object.prototype.toString.call(input) === '[object Date]';
  }
  function map(arr, fn) {
    var res = [], i;
    for (i = 0; i < arr.length; ++i) {
      res.push(fn(arr[i], i));
    }
    return res;
  }
  function hasOwnProp(a, b) {
    return Object.prototype.hasOwnProperty.call(a, b);
  }
  function extend(a, b) {
    for (var i in b) {
      if (hasOwnProp(b, i)) {
        a[i] = b[i];
      }
    }
    if (hasOwnProp(b, 'toString')) {
      a.toString = b.toString;
    }
    if (hasOwnProp(b, 'valueOf')) {
      a.valueOf = b.valueOf;
    }
    return a;
  }
  function create_utc__createUTC(input, format, locale, strict) {
    return createLocalOrUTC(input, format, locale, strict, true).utc();
  }
  function defaultParsingFlags() {
    // We need to deep clone this object.
    return {
      empty: false,
      unusedTokens: [],
      unusedInput: [],
      overflow: -2,
      charsLeftOver: 0,
      nullInput: false,
      invalidMonth: null,
      invalidFormat: false,
      userInvalidated: false,
      iso: false
    };
  }
  function getParsingFlags(m) {
    if (m._pf == null) {
      m._pf = defaultParsingFlags();
    }
    return m._pf;
  }
  function valid__isValid(m) {
    if (m._isValid == null) {
      var flags = getParsingFlags(m);
      m._isValid = !isNaN(m._d.getTime()) && flags.overflow < 0 && !flags.empty && !flags.invalidMonth && !flags.invalidWeekday && !flags.nullInput && !flags.invalidFormat && !flags.userInvalidated;
      if (m._strict) {
        m._isValid = m._isValid && flags.charsLeftOver === 0 && flags.unusedTokens.length === 0 && flags.bigHour === undefined;
      }
    }
    return m._isValid;
  }
  function valid__createInvalid(flags) {
    var m = create_utc__createUTC(NaN);
    if (flags != null) {
      extend(getParsingFlags(m), flags);
    } else {
      getParsingFlags(m).userInvalidated = true;
    }
    return m;
  }
  var momentProperties = utils_hooks__hooks.momentProperties = [];
  function copyConfig(to, from) {
    var i, prop, val;
    if (typeof from._isAMomentObject !== 'undefined') {
      to._isAMomentObject = from._isAMomentObject;
    }
    if (typeof from._i !== 'undefined') {
      to._i = from._i;
    }
    if (typeof from._f !== 'undefined') {
      to._f = from._f;
    }
    if (typeof from._l !== 'undefined') {
      to._l = from._l;
    }
    if (typeof from._strict !== 'undefined') {
      to._strict = from._strict;
    }
    if (typeof from._tzm !== 'undefined') {
      to._tzm = from._tzm;
    }
    if (typeof from._isUTC !== 'undefined') {
      to._isUTC = from._isUTC;
    }
    if (typeof from._offset !== 'undefined') {
      to._offset = from._offset;
    }
    if (typeof from._pf !== 'undefined') {
      to._pf = getParsingFlags(from);
    }
    if (typeof from._locale !== 'undefined') {
      to._locale = from._locale;
    }
    if (momentProperties.length > 0) {
      for (i in momentProperties) {
        prop = momentProperties[i];
        val = from[prop];
        if (typeof val !== 'undefined') {
          to[prop] = val;
        }
      }
    }
    return to;
  }
  var updateInProgress = false;
  // Moment prototype object
  function Moment(config) {
    copyConfig(this, config);
    this._d = new Date(config._d != null ? config._d.getTime() : NaN);
    // Prevent infinite loop in case updateOffset creates new moment
    // objects.
    if (updateInProgress === false) {
      updateInProgress = true;
      utils_hooks__hooks.updateOffset(this);
      updateInProgress = false;
    }
  }
  function isMoment(obj) {
    return obj instanceof Moment || obj != null && obj._isAMomentObject != null;
  }
  function absFloor(number) {
    if (number < 0) {
      return Math.ceil(number);
    } else {
      return Math.floor(number);
    }
  }
  function toInt(argumentForCoercion) {
    var coercedNumber = +argumentForCoercion, value = 0;
    if (coercedNumber !== 0 && isFinite(coercedNumber)) {
      value = absFloor(coercedNumber);
    }
    return value;
  }
  function compareArrays(array1, array2, dontConvert) {
    var len = Math.min(array1.length, array2.length), lengthDiff = Math.abs(array1.length - array2.length), diffs = 0, i;
    for (i = 0; i < len; i++) {
      if (dontConvert && array1[i] !== array2[i] || !dontConvert && toInt(array1[i]) !== toInt(array2[i])) {
        diffs++;
      }
    }
    return diffs + lengthDiff;
  }
  function Locale() {
  }
  var locales = {};
  var globalLocale;
  function normalizeLocale(key) {
    return key ? key.toLowerCase().replace('_', '-') : key;
  }
  // pick the locale from the array
  // try ['en-au', 'en-gb'] as 'en-au', 'en-gb', 'en', as in move through the list trying each
  // substring from most specific to least, but move to the next array item if it's a more specific variant than the current root
  function chooseLocale(names) {
    var i = 0, j, next, locale, split;
    while (i < names.length) {
      split = normalizeLocale(names[i]).split('-');
      j = split.length;
      next = normalizeLocale(names[i + 1]);
      next = next ? next.split('-') : null;
      while (j > 0) {
        locale = loadLocale(split.slice(0, j).join('-'));
        if (locale) {
          return locale;
        }
        if (next && next.length >= j && compareArrays(split, next, true) >= j - 1) {
          //the next array item is better than a shallower substring of this one
          break;
        }
        j--;
      }
      i++;
    }
    return null;
  }
  function loadLocale(name) {
    var oldLocale = null;
    // TODO: Find a better way to register and load all the locales in Node
    if (!locales[name] && typeof module !== 'undefined' && module && module.exports) {
      try {
        oldLocale = globalLocale._abbr;
        require('./locale/' + name);
        // because defineLocale currently also sets the global locale, we
        // want to undo that for lazy loaded locales
        locale_locales__getSetGlobalLocale(oldLocale);
      } catch (e) {
      }
    }
    return locales[name];
  }
  // This function will load locale and then set the global locale.  If
  // no arguments are passed in, it will simply return the current global
  // locale key.
  function locale_locales__getSetGlobalLocale(key, values) {
    var data;
    if (key) {
      if (typeof values === 'undefined') {
        data = locale_locales__getLocale(key);
      } else {
        data = defineLocale(key, values);
      }
      if (data) {
        // moment.duration._locale = moment._locale = data;
        globalLocale = data;
      }
    }
    return globalLocale._abbr;
  }
  function defineLocale(name, values) {
    if (values !== null) {
      values.abbr = name;
      locales[name] = locales[name] || new Locale();
      locales[name].set(values);
      // backwards compat for now: also set the locale
      locale_locales__getSetGlobalLocale(name);
      return locales[name];
    } else {
      // useful for testing
      delete locales[name];
      return null;
    }
  }
  // returns locale data
  function locale_locales__getLocale(key) {
    var locale;
    if (key && key._locale && key._locale._abbr) {
      key = key._locale._abbr;
    }
    if (!key) {
      return globalLocale;
    }
    if (!isArray(key)) {
      //short-circuit everything else
      locale = loadLocale(key);
      if (locale) {
        return locale;
      }
      key = [key];
    }
    return chooseLocale(key);
  }
  var aliases = {};
  function addUnitAlias(unit, shorthand) {
    var lowerCase = unit.toLowerCase();
    aliases[lowerCase] = aliases[lowerCase + 's'] = aliases[shorthand] = unit;
  }
  function normalizeUnits(units) {
    return typeof units === 'string' ? aliases[units] || aliases[units.toLowerCase()] : undefined;
  }
  function normalizeObjectUnits(inputObject) {
    var normalizedInput = {}, normalizedProp, prop;
    for (prop in inputObject) {
      if (hasOwnProp(inputObject, prop)) {
        normalizedProp = normalizeUnits(prop);
        if (normalizedProp) {
          normalizedInput[normalizedProp] = inputObject[prop];
        }
      }
    }
    return normalizedInput;
  }
  function makeGetSet(unit, keepTime) {
    return function (value) {
      if (value != null) {
        get_set__set(this, unit, value);
        utils_hooks__hooks.updateOffset(this, keepTime);
        return this;
      } else {
        return get_set__get(this, unit);
      }
    };
  }
  function get_set__get(mom, unit) {
    return mom._d['get' + (mom._isUTC ? 'UTC' : '') + unit]();
  }
  function get_set__set(mom, unit, value) {
    return mom._d['set' + (mom._isUTC ? 'UTC' : '') + unit](value);
  }
  // MOMENTS
  function getSet(units, value) {
    var unit;
    if (typeof units === 'object') {
      for (unit in units) {
        this.set(unit, units[unit]);
      }
    } else {
      units = normalizeUnits(units);
      if (typeof this[units] === 'function') {
        return this[units](value);
      }
    }
    return this;
  }
  function zeroFill(number, targetLength, forceSign) {
    var absNumber = '' + Math.abs(number), zerosToFill = targetLength - absNumber.length, sign = number >= 0;
    return (sign ? forceSign ? '+' : '' : '-') + Math.pow(10, Math.max(0, zerosToFill)).toString().substr(1) + absNumber;
  }
  var formattingTokens = /(\[[^\[]*\])|(\\)?(Mo|MM?M?M?|Do|DDDo|DD?D?D?|ddd?d?|do?|w[o|w]?|W[o|W]?|Q|YYYYYY|YYYYY|YYYY|YY|gg(ggg?)?|GG(GGG?)?|e|E|a|A|hh?|HH?|mm?|ss?|S{1,9}|x|X|zz?|ZZ?|.)/g;
  var localFormattingTokens = /(\[[^\[]*\])|(\\)?(LTS|LT|LL?L?L?|l{1,4})/g;
  var formatFunctions = {};
  var formatTokenFunctions = {};
  // token:    'M'
  // padded:   ['MM', 2]
  // ordinal:  'Mo'
  // callback: function () { this.month() + 1 }
  function addFormatToken(token, padded, ordinal, callback) {
    var func = callback;
    if (typeof callback === 'string') {
      func = function () {
        return this[callback]();
      };
    }
    if (token) {
      formatTokenFunctions[token] = func;
    }
    if (padded) {
      formatTokenFunctions[padded[0]] = function () {
        return zeroFill(func.apply(this, arguments), padded[1], padded[2]);
      };
    }
    if (ordinal) {
      formatTokenFunctions[ordinal] = function () {
        return this.localeData().ordinal(func.apply(this, arguments), token);
      };
    }
  }
  function removeFormattingTokens(input) {
    if (input.match(/\[[\s\S]/)) {
      return input.replace(/^\[|\]$/g, '');
    }
    return input.replace(/\\/g, '');
  }
  function makeFormatFunction(format) {
    var array = format.match(formattingTokens), i, length;
    for (i = 0, length = array.length; i < length; i++) {
      if (formatTokenFunctions[array[i]]) {
        array[i] = formatTokenFunctions[array[i]];
      } else {
        array[i] = removeFormattingTokens(array[i]);
      }
    }
    return function (mom) {
      var output = '';
      for (i = 0; i < length; i++) {
        output += array[i] instanceof Function ? array[i].call(mom, format) : array[i];
      }
      return output;
    };
  }
  // format date using native date object
  function formatMoment(m, format) {
    if (!m.isValid()) {
      return m.localeData().invalidDate();
    }
    format = expandFormat(format, m.localeData());
    formatFunctions[format] = formatFunctions[format] || makeFormatFunction(format);
    return formatFunctions[format](m);
  }
  function expandFormat(format, locale) {
    var i = 5;
    function replaceLongDateFormatTokens(input) {
      return locale.longDateFormat(input) || input;
    }
    localFormattingTokens.lastIndex = 0;
    while (i >= 0 && localFormattingTokens.test(format)) {
      format = format.replace(localFormattingTokens, replaceLongDateFormatTokens);
      localFormattingTokens.lastIndex = 0;
      i -= 1;
    }
    return format;
  }
  var match1 = /\d/;
  //       0 - 9
  var match2 = /\d\d/;
  //      00 - 99
  var match3 = /\d{3}/;
  //     000 - 999
  var match4 = /\d{4}/;
  //    0000 - 9999
  var match6 = /[+-]?\d{6}/;
  // -999999 - 999999
  var match1to2 = /\d\d?/;
  //       0 - 99
  var match1to3 = /\d{1,3}/;
  //       0 - 999
  var match1to4 = /\d{1,4}/;
  //       0 - 9999
  var match1to6 = /[+-]?\d{1,6}/;
  // -999999 - 999999
  var matchUnsigned = /\d+/;
  //       0 - inf
  var matchSigned = /[+-]?\d+/;
  //    -inf - inf
  var matchOffset = /Z|[+-]\d\d:?\d\d/gi;
  // +00:00 -00:00 +0000 -0000 or Z
  var matchTimestamp = /[+-]?\d+(\.\d{1,3})?/;
  // 123456789 123456789.123
  // any word (or two) characters or numbers including two/three word month in arabic.
  var matchWord = /[0-9]*['a-z\u00A0-\u05FF\u0700-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF]+|[\u0600-\u06FF\/]+(\s*?[\u0600-\u06FF]+){1,2}/i;
  var regexes = {};
  function isFunction(sth) {
    // https://github.com/moment/moment/issues/2325
    return typeof sth === 'function' && Object.prototype.toString.call(sth) === '[object Function]';
  }
  function addRegexToken(token, regex, strictRegex) {
    regexes[token] = isFunction(regex) ? regex : function (isStrict) {
      return isStrict && strictRegex ? strictRegex : regex;
    };
  }
  function getParseRegexForToken(token, config) {
    if (!hasOwnProp(regexes, token)) {
      return new RegExp(unescapeFormat(token));
    }
    return regexes[token](config._strict, config._locale);
  }
  // Code from http://stackoverflow.com/questions/3561493/is-there-a-regexp-escape-function-in-javascript
  function unescapeFormat(s) {
    return s.replace('\\', '').replace(/\\(\[)|\\(\])|\[([^\]\[]*)\]|\\(.)/g, function (matched, p1, p2, p3, p4) {
      return p1 || p2 || p3 || p4;
    }).replace(/[-\/\\^$*+?.()|[\]{}]/g, '\\$&');
  }
  var tokens = {};
  function addParseToken(token, callback) {
    var i, func = callback;
    if (typeof token === 'string') {
      token = [token];
    }
    if (typeof callback === 'number') {
      func = function (input, array) {
        array[callback] = toInt(input);
      };
    }
    for (i = 0; i < token.length; i++) {
      tokens[token[i]] = func;
    }
  }
  function addWeekParseToken(token, callback) {
    addParseToken(token, function (input, array, config, token) {
      config._w = config._w || {};
      callback(input, config._w, config, token);
    });
  }
  function addTimeToArrayFromToken(token, input, config) {
    if (input != null && hasOwnProp(tokens, token)) {
      tokens[token](input, config._a, config, token);
    }
  }
  var YEAR = 0;
  var MONTH = 1;
  var DATE = 2;
  var HOUR = 3;
  var MINUTE = 4;
  var SECOND = 5;
  var MILLISECOND = 6;
  function daysInMonth(year, month) {
    return new Date(Date.UTC(year, month + 1, 0)).getUTCDate();
  }
  // FORMATTING
  addFormatToken('M', [
    'MM',
    2
  ], 'Mo', function () {
    return this.month() + 1;
  });
  addFormatToken('MMM', 0, 0, function (format) {
    return this.localeData().monthsShort(this, format);
  });
  addFormatToken('MMMM', 0, 0, function (format) {
    return this.localeData().months(this, format);
  });
  // ALIASES
  addUnitAlias('month', 'M');
  // PARSING
  addRegexToken('M', match1to2);
  addRegexToken('MM', match1to2, match2);
  addRegexToken('MMM', matchWord);
  addRegexToken('MMMM', matchWord);
  addParseToken([
    'M',
    'MM'
  ], function (input, array) {
    array[MONTH] = toInt(input) - 1;
  });
  addParseToken([
    'MMM',
    'MMMM'
  ], function (input, array, config, token) {
    var month = config._locale.monthsParse(input, token, config._strict);
    // if we didn't find a month name, mark the date as invalid.
    if (month != null) {
      array[MONTH] = month;
    } else {
      getParsingFlags(config).invalidMonth = input;
    }
  });
  // LOCALES
  var defaultLocaleMonths = 'January_February_March_April_May_June_July_August_September_October_November_December'.split('_');
  function localeMonths(m) {
    return this._months[m.month()];
  }
  var defaultLocaleMonthsShort = 'Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec'.split('_');
  function localeMonthsShort(m) {
    return this._monthsShort[m.month()];
  }
  function localeMonthsParse(monthName, format, strict) {
    var i, mom, regex;
    if (!this._monthsParse) {
      this._monthsParse = [];
      this._longMonthsParse = [];
      this._shortMonthsParse = [];
    }
    for (i = 0; i < 12; i++) {
      // make the regex if we don't have it already
      mom = create_utc__createUTC([
        2000,
        i
      ]);
      if (strict && !this._longMonthsParse[i]) {
        this._longMonthsParse[i] = new RegExp('^' + this.months(mom, '').replace('.', '') + '$', 'i');
        this._shortMonthsParse[i] = new RegExp('^' + this.monthsShort(mom, '').replace('.', '') + '$', 'i');
      }
      if (!strict && !this._monthsParse[i]) {
        regex = '^' + this.months(mom, '') + '|^' + this.monthsShort(mom, '');
        this._monthsParse[i] = new RegExp(regex.replace('.', ''), 'i');
      }
      // test the regex
      if (strict && format === 'MMMM' && this._longMonthsParse[i].test(monthName)) {
        return i;
      } else if (strict && format === 'MMM' && this._shortMonthsParse[i].test(monthName)) {
        return i;
      } else if (!strict && this._monthsParse[i].test(monthName)) {
        return i;
      }
    }
  }
  // MOMENTS
  function setMonth(mom, value) {
    var dayOfMonth;
    // TODO: Move this out of here!
    if (typeof value === 'string') {
      value = mom.localeData().monthsParse(value);
      // TODO: Another silent failure?
      if (typeof value !== 'number') {
        return mom;
      }
    }
    dayOfMonth = Math.min(mom.date(), daysInMonth(mom.year(), value));
    mom._d['set' + (mom._isUTC ? 'UTC' : '') + 'Month'](value, dayOfMonth);
    return mom;
  }
  function getSetMonth(value) {
    if (value != null) {
      setMonth(this, value);
      utils_hooks__hooks.updateOffset(this, true);
      return this;
    } else {
      return get_set__get(this, 'Month');
    }
  }
  function getDaysInMonth() {
    return daysInMonth(this.year(), this.month());
  }
  function checkOverflow(m) {
    var overflow;
    var a = m._a;
    if (a && getParsingFlags(m).overflow === -2) {
      overflow = a[MONTH] < 0 || a[MONTH] > 11 ? MONTH : a[DATE] < 1 || a[DATE] > daysInMonth(a[YEAR], a[MONTH]) ? DATE : a[HOUR] < 0 || a[HOUR] > 24 || a[HOUR] === 24 && (a[MINUTE] !== 0 || a[SECOND] !== 0 || a[MILLISECOND] !== 0) ? HOUR : a[MINUTE] < 0 || a[MINUTE] > 59 ? MINUTE : a[SECOND] < 0 || a[SECOND] > 59 ? SECOND : a[MILLISECOND] < 0 || a[MILLISECOND] > 999 ? MILLISECOND : -1;
      if (getParsingFlags(m)._overflowDayOfYear && (overflow < YEAR || overflow > DATE)) {
        overflow = DATE;
      }
      getParsingFlags(m).overflow = overflow;
    }
    return m;
  }
  function warn(msg) {
    if (utils_hooks__hooks.suppressDeprecationWarnings === false && typeof console !== 'undefined' && console.warn) {
      console.warn('Deprecation warning: ' + msg);
    }
  }
  function deprecate(msg, fn) {
    var firstTime = true;
    return extend(function () {
      if (firstTime) {
        warn(msg + '\n' + new Error().stack);
        firstTime = false;
      }
      return fn.apply(this, arguments);
    }, fn);
  }
  var deprecations = {};
  function deprecateSimple(name, msg) {
    if (!deprecations[name]) {
      warn(msg);
      deprecations[name] = true;
    }
  }
  utils_hooks__hooks.suppressDeprecationWarnings = false;
  var from_string__isoRegex = /^\s*(?:[+-]\d{6}|\d{4})-(?:(\d\d-\d\d)|(W\d\d$)|(W\d\d-\d)|(\d\d\d))((T| )(\d\d(:\d\d(:\d\d(\.\d+)?)?)?)?([\+\-]\d\d(?::?\d\d)?|\s*Z)?)?$/;
  var isoDates = [
      [
        'YYYYYY-MM-DD',
        /[+-]\d{6}-\d{2}-\d{2}/
      ],
      [
        'YYYY-MM-DD',
        /\d{4}-\d{2}-\d{2}/
      ],
      [
        'GGGG-[W]WW-E',
        /\d{4}-W\d{2}-\d/
      ],
      [
        'GGGG-[W]WW',
        /\d{4}-W\d{2}/
      ],
      [
        'YYYY-DDD',
        /\d{4}-\d{3}/
      ]
    ];
  // iso time formats and regexes
  var isoTimes = [
      [
        'HH:mm:ss.SSSS',
        /(T| )\d\d:\d\d:\d\d\.\d+/
      ],
      [
        'HH:mm:ss',
        /(T| )\d\d:\d\d:\d\d/
      ],
      [
        'HH:mm',
        /(T| )\d\d:\d\d/
      ],
      [
        'HH',
        /(T| )\d\d/
      ]
    ];
  var aspNetJsonRegex = /^\/?Date\((\-?\d+)/i;
  // date from iso format
  function configFromISO(config) {
    var i, l, string = config._i, match = from_string__isoRegex.exec(string);
    if (match) {
      getParsingFlags(config).iso = true;
      for (i = 0, l = isoDates.length; i < l; i++) {
        if (isoDates[i][1].exec(string)) {
          config._f = isoDates[i][0];
          break;
        }
      }
      for (i = 0, l = isoTimes.length; i < l; i++) {
        if (isoTimes[i][1].exec(string)) {
          // match[6] should be 'T' or space
          config._f += (match[6] || ' ') + isoTimes[i][0];
          break;
        }
      }
      if (string.match(matchOffset)) {
        config._f += 'Z';
      }
      configFromStringAndFormat(config);
    } else {
      config._isValid = false;
    }
  }
  // date from iso format or fallback
  function configFromString(config) {
    var matched = aspNetJsonRegex.exec(config._i);
    if (matched !== null) {
      config._d = new Date(+matched[1]);
      return;
    }
    configFromISO(config);
    if (config._isValid === false) {
      delete config._isValid;
      utils_hooks__hooks.createFromInputFallback(config);
    }
  }
  utils_hooks__hooks.createFromInputFallback = deprecate('moment construction falls back to js Date. This is ' + 'discouraged and will be removed in upcoming major ' + 'release. Please refer to ' + 'https://github.com/moment/moment/issues/1407 for more info.', function (config) {
    config._d = new Date(config._i + (config._useUTC ? ' UTC' : ''));
  });
  function createDate(y, m, d, h, M, s, ms) {
    //can't just apply() to create a date:
    //http://stackoverflow.com/questions/181348/instantiating-a-javascript-object-by-calling-prototype-constructor-apply
    var date = new Date(y, m, d, h, M, s, ms);
    //the date constructor doesn't accept years < 1970
    if (y < 1970) {
      date.setFullYear(y);
    }
    return date;
  }
  function createUTCDate(y) {
    var date = new Date(Date.UTC.apply(null, arguments));
    if (y < 1970) {
      date.setUTCFullYear(y);
    }
    return date;
  }
  addFormatToken(0, [
    'YY',
    2
  ], 0, function () {
    return this.year() % 100;
  });
  addFormatToken(0, [
    'YYYY',
    4
  ], 0, 'year');
  addFormatToken(0, [
    'YYYYY',
    5
  ], 0, 'year');
  addFormatToken(0, [
    'YYYYYY',
    6,
    true
  ], 0, 'year');
  // ALIASES
  addUnitAlias('year', 'y');
  // PARSING
  addRegexToken('Y', matchSigned);
  addRegexToken('YY', match1to2, match2);
  addRegexToken('YYYY', match1to4, match4);
  addRegexToken('YYYYY', match1to6, match6);
  addRegexToken('YYYYYY', match1to6, match6);
  addParseToken([
    'YYYYY',
    'YYYYYY'
  ], YEAR);
  addParseToken('YYYY', function (input, array) {
    array[YEAR] = input.length === 2 ? utils_hooks__hooks.parseTwoDigitYear(input) : toInt(input);
  });
  addParseToken('YY', function (input, array) {
    array[YEAR] = utils_hooks__hooks.parseTwoDigitYear(input);
  });
  // HELPERS
  function daysInYear(year) {
    return isLeapYear(year) ? 366 : 365;
  }
  function isLeapYear(year) {
    return year % 4 === 0 && year % 100 !== 0 || year % 400 === 0;
  }
  // HOOKS
  utils_hooks__hooks.parseTwoDigitYear = function (input) {
    return toInt(input) + (toInt(input) > 68 ? 1900 : 2000);
  };
  // MOMENTS
  var getSetYear = makeGetSet('FullYear', false);
  function getIsLeapYear() {
    return isLeapYear(this.year());
  }
  addFormatToken('w', [
    'ww',
    2
  ], 'wo', 'week');
  addFormatToken('W', [
    'WW',
    2
  ], 'Wo', 'isoWeek');
  // ALIASES
  addUnitAlias('week', 'w');
  addUnitAlias('isoWeek', 'W');
  // PARSING
  addRegexToken('w', match1to2);
  addRegexToken('ww', match1to2, match2);
  addRegexToken('W', match1to2);
  addRegexToken('WW', match1to2, match2);
  addWeekParseToken([
    'w',
    'ww',
    'W',
    'WW'
  ], function (input, week, config, token) {
    week[token.substr(0, 1)] = toInt(input);
  });
  // HELPERS
  // firstDayOfWeek       0 = sun, 6 = sat
  //                      the day of the week that starts the week
  //                      (usually sunday or monday)
  // firstDayOfWeekOfYear 0 = sun, 6 = sat
  //                      the first week is the week that contains the first
  //                      of this day of the week
  //                      (eg. ISO weeks use thursday (4))
  function weekOfYear(mom, firstDayOfWeek, firstDayOfWeekOfYear) {
    var end = firstDayOfWeekOfYear - firstDayOfWeek, daysToDayOfWeek = firstDayOfWeekOfYear - mom.day(), adjustedMoment;
    if (daysToDayOfWeek > end) {
      daysToDayOfWeek -= 7;
    }
    if (daysToDayOfWeek < end - 7) {
      daysToDayOfWeek += 7;
    }
    adjustedMoment = local__createLocal(mom).add(daysToDayOfWeek, 'd');
    return {
      week: Math.ceil(adjustedMoment.dayOfYear() / 7),
      year: adjustedMoment.year()
    };
  }
  // LOCALES
  function localeWeek(mom) {
    return weekOfYear(mom, this._week.dow, this._week.doy).week;
  }
  var defaultLocaleWeek = {
      dow: 0,
      doy: 6
    };
  function localeFirstDayOfWeek() {
    return this._week.dow;
  }
  function localeFirstDayOfYear() {
    return this._week.doy;
  }
  // MOMENTS
  function getSetWeek(input) {
    var week = this.localeData().week(this);
    return input == null ? week : this.add((input - week) * 7, 'd');
  }
  function getSetISOWeek(input) {
    var week = weekOfYear(this, 1, 4).week;
    return input == null ? week : this.add((input - week) * 7, 'd');
  }
  addFormatToken('DDD', [
    'DDDD',
    3
  ], 'DDDo', 'dayOfYear');
  // ALIASES
  addUnitAlias('dayOfYear', 'DDD');
  // PARSING
  addRegexToken('DDD', match1to3);
  addRegexToken('DDDD', match3);
  addParseToken([
    'DDD',
    'DDDD'
  ], function (input, array, config) {
    config._dayOfYear = toInt(input);
  });
  // HELPERS
  //http://en.wikipedia.org/wiki/ISO_week_date#Calculating_a_date_given_the_year.2C_week_number_and_weekday
  function dayOfYearFromWeeks(year, week, weekday, firstDayOfWeekOfYear, firstDayOfWeek) {
    var week1Jan = 6 + firstDayOfWeek - firstDayOfWeekOfYear, janX = createUTCDate(year, 0, 1 + week1Jan), d = janX.getUTCDay(), dayOfYear;
    if (d < firstDayOfWeek) {
      d += 7;
    }
    weekday = weekday != null ? 1 * weekday : firstDayOfWeek;
    dayOfYear = 1 + week1Jan + 7 * (week - 1) - d + weekday;
    return {
      year: dayOfYear > 0 ? year : year - 1,
      dayOfYear: dayOfYear > 0 ? dayOfYear : daysInYear(year - 1) + dayOfYear
    };
  }
  // MOMENTS
  function getSetDayOfYear(input) {
    var dayOfYear = Math.round((this.clone().startOf('day') - this.clone().startOf('year')) / 86400000) + 1;
    return input == null ? dayOfYear : this.add(input - dayOfYear, 'd');
  }
  // Pick the first defined of two or three arguments.
  function defaults(a, b, c) {
    if (a != null) {
      return a;
    }
    if (b != null) {
      return b;
    }
    return c;
  }
  function currentDateArray(config) {
    var now = new Date();
    if (config._useUTC) {
      return [
        now.getUTCFullYear(),
        now.getUTCMonth(),
        now.getUTCDate()
      ];
    }
    return [
      now.getFullYear(),
      now.getMonth(),
      now.getDate()
    ];
  }
  // convert an array to a date.
  // the array should mirror the parameters below
  // note: all values past the year are optional and will default to the lowest possible value.
  // [year, month, day , hour, minute, second, millisecond]
  function configFromArray(config) {
    var i, date, input = [], currentDate, yearToUse;
    if (config._d) {
      return;
    }
    currentDate = currentDateArray(config);
    //compute day of the year from weeks and weekdays
    if (config._w && config._a[DATE] == null && config._a[MONTH] == null) {
      dayOfYearFromWeekInfo(config);
    }
    //if the day of the year is set, figure out what it is
    if (config._dayOfYear) {
      yearToUse = defaults(config._a[YEAR], currentDate[YEAR]);
      if (config._dayOfYear > daysInYear(yearToUse)) {
        getParsingFlags(config)._overflowDayOfYear = true;
      }
      date = createUTCDate(yearToUse, 0, config._dayOfYear);
      config._a[MONTH] = date.getUTCMonth();
      config._a[DATE] = date.getUTCDate();
    }
    // Default to current date.
    // * if no year, month, day of month are given, default to today
    // * if day of month is given, default month and year
    // * if month is given, default only year
    // * if year is given, don't default anything
    for (i = 0; i < 3 && config._a[i] == null; ++i) {
      config._a[i] = input[i] = currentDate[i];
    }
    // Zero out whatever was not defaulted, including time
    for (; i < 7; i++) {
      config._a[i] = input[i] = config._a[i] == null ? i === 2 ? 1 : 0 : config._a[i];
    }
    // Check for 24:00:00.000
    if (config._a[HOUR] === 24 && config._a[MINUTE] === 0 && config._a[SECOND] === 0 && config._a[MILLISECOND] === 0) {
      config._nextDay = true;
      config._a[HOUR] = 0;
    }
    config._d = (config._useUTC ? createUTCDate : createDate).apply(null, input);
    // Apply timezone offset from input. The actual utcOffset can be changed
    // with parseZone.
    if (config._tzm != null) {
      config._d.setUTCMinutes(config._d.getUTCMinutes() - config._tzm);
    }
    if (config._nextDay) {
      config._a[HOUR] = 24;
    }
  }
  function dayOfYearFromWeekInfo(config) {
    var w, weekYear, week, weekday, dow, doy, temp;
    w = config._w;
    if (w.GG != null || w.W != null || w.E != null) {
      dow = 1;
      doy = 4;
      // TODO: We need to take the current isoWeekYear, but that depends on
      // how we interpret now (local, utc, fixed offset). So create
      // a now version of current config (take local/utc/offset flags, and
      // create now).
      weekYear = defaults(w.GG, config._a[YEAR], weekOfYear(local__createLocal(), 1, 4).year);
      week = defaults(w.W, 1);
      weekday = defaults(w.E, 1);
    } else {
      dow = config._locale._week.dow;
      doy = config._locale._week.doy;
      weekYear = defaults(w.gg, config._a[YEAR], weekOfYear(local__createLocal(), dow, doy).year);
      week = defaults(w.w, 1);
      if (w.d != null) {
        // weekday -- low day numbers are considered next week
        weekday = w.d;
        if (weekday < dow) {
          ++week;
        }
      } else if (w.e != null) {
        // local weekday -- counting starts from begining of week
        weekday = w.e + dow;
      } else {
        // default to begining of week
        weekday = dow;
      }
    }
    temp = dayOfYearFromWeeks(weekYear, week, weekday, doy, dow);
    config._a[YEAR] = temp.year;
    config._dayOfYear = temp.dayOfYear;
  }
  utils_hooks__hooks.ISO_8601 = function () {
  };
  // date from string and format string
  function configFromStringAndFormat(config) {
    // TODO: Move this to another part of the creation flow to prevent circular deps
    if (config._f === utils_hooks__hooks.ISO_8601) {
      configFromISO(config);
      return;
    }
    config._a = [];
    getParsingFlags(config).empty = true;
    // This array is used to make a Date, either with `new Date` or `Date.UTC`
    var string = '' + config._i, i, parsedInput, tokens, token, skipped, stringLength = string.length, totalParsedInputLength = 0;
    tokens = expandFormat(config._f, config._locale).match(formattingTokens) || [];
    for (i = 0; i < tokens.length; i++) {
      token = tokens[i];
      parsedInput = (string.match(getParseRegexForToken(token, config)) || [])[0];
      if (parsedInput) {
        skipped = string.substr(0, string.indexOf(parsedInput));
        if (skipped.length > 0) {
          getParsingFlags(config).unusedInput.push(skipped);
        }
        string = string.slice(string.indexOf(parsedInput) + parsedInput.length);
        totalParsedInputLength += parsedInput.length;
      }
      // don't parse if it's not a known token
      if (formatTokenFunctions[token]) {
        if (parsedInput) {
          getParsingFlags(config).empty = false;
        } else {
          getParsingFlags(config).unusedTokens.push(token);
        }
        addTimeToArrayFromToken(token, parsedInput, config);
      } else if (config._strict && !parsedInput) {
        getParsingFlags(config).unusedTokens.push(token);
      }
    }
    // add remaining unparsed input length to the string
    getParsingFlags(config).charsLeftOver = stringLength - totalParsedInputLength;
    if (string.length > 0) {
      getParsingFlags(config).unusedInput.push(string);
    }
    // clear _12h flag if hour is <= 12
    if (getParsingFlags(config).bigHour === true && config._a[HOUR] <= 12 && config._a[HOUR] > 0) {
      getParsingFlags(config).bigHour = undefined;
    }
    // handle meridiem
    config._a[HOUR] = meridiemFixWrap(config._locale, config._a[HOUR], config._meridiem);
    configFromArray(config);
    checkOverflow(config);
  }
  function meridiemFixWrap(locale, hour, meridiem) {
    var isPm;
    if (meridiem == null) {
      // nothing to do
      return hour;
    }
    if (locale.meridiemHour != null) {
      return locale.meridiemHour(hour, meridiem);
    } else if (locale.isPM != null) {
      // Fallback
      isPm = locale.isPM(meridiem);
      if (isPm && hour < 12) {
        hour += 12;
      }
      if (!isPm && hour === 12) {
        hour = 0;
      }
      return hour;
    } else {
      // this is not supposed to happen
      return hour;
    }
  }
  function configFromStringAndArray(config) {
    var tempConfig, bestMoment, scoreToBeat, i, currentScore;
    if (config._f.length === 0) {
      getParsingFlags(config).invalidFormat = true;
      config._d = new Date(NaN);
      return;
    }
    for (i = 0; i < config._f.length; i++) {
      currentScore = 0;
      tempConfig = copyConfig({}, config);
      if (config._useUTC != null) {
        tempConfig._useUTC = config._useUTC;
      }
      tempConfig._f = config._f[i];
      configFromStringAndFormat(tempConfig);
      if (!valid__isValid(tempConfig)) {
        continue;
      }
      // if there is any input that was not parsed add a penalty for that format
      currentScore += getParsingFlags(tempConfig).charsLeftOver;
      //or tokens
      currentScore += getParsingFlags(tempConfig).unusedTokens.length * 10;
      getParsingFlags(tempConfig).score = currentScore;
      if (scoreToBeat == null || currentScore < scoreToBeat) {
        scoreToBeat = currentScore;
        bestMoment = tempConfig;
      }
    }
    extend(config, bestMoment || tempConfig);
  }
  function configFromObject(config) {
    if (config._d) {
      return;
    }
    var i = normalizeObjectUnits(config._i);
    config._a = [
      i.year,
      i.month,
      i.day || i.date,
      i.hour,
      i.minute,
      i.second,
      i.millisecond
    ];
    configFromArray(config);
  }
  function createFromConfig(config) {
    var res = new Moment(checkOverflow(prepareConfig(config)));
    if (res._nextDay) {
      // Adding is smart enough around DST
      res.add(1, 'd');
      res._nextDay = undefined;
    }
    return res;
  }
  function prepareConfig(config) {
    var input = config._i, format = config._f;
    config._locale = config._locale || locale_locales__getLocale(config._l);
    if (input === null || format === undefined && input === '') {
      return valid__createInvalid({ nullInput: true });
    }
    if (typeof input === 'string') {
      config._i = input = config._locale.preparse(input);
    }
    if (isMoment(input)) {
      return new Moment(checkOverflow(input));
    } else if (isArray(format)) {
      configFromStringAndArray(config);
    } else if (format) {
      configFromStringAndFormat(config);
    } else if (isDate(input)) {
      config._d = input;
    } else {
      configFromInput(config);
    }
    return config;
  }
  function configFromInput(config) {
    var input = config._i;
    if (input === undefined) {
      config._d = new Date();
    } else if (isDate(input)) {
      config._d = new Date(+input);
    } else if (typeof input === 'string') {
      configFromString(config);
    } else if (isArray(input)) {
      config._a = map(input.slice(0), function (obj) {
        return parseInt(obj, 10);
      });
      configFromArray(config);
    } else if (typeof input === 'object') {
      configFromObject(config);
    } else if (typeof input === 'number') {
      // from milliseconds
      config._d = new Date(input);
    } else {
      utils_hooks__hooks.createFromInputFallback(config);
    }
  }
  function createLocalOrUTC(input, format, locale, strict, isUTC) {
    var c = {};
    if (typeof locale === 'boolean') {
      strict = locale;
      locale = undefined;
    }
    // object construction must be done this way.
    // https://github.com/moment/moment/issues/1423
    c._isAMomentObject = true;
    c._useUTC = c._isUTC = isUTC;
    c._l = locale;
    c._i = input;
    c._f = format;
    c._strict = strict;
    return createFromConfig(c);
  }
  function local__createLocal(input, format, locale, strict) {
    return createLocalOrUTC(input, format, locale, strict, false);
  }
  var prototypeMin = deprecate('moment().min is deprecated, use moment.min instead. https://github.com/moment/moment/issues/1548', function () {
      var other = local__createLocal.apply(null, arguments);
      return other < this ? this : other;
    });
  var prototypeMax = deprecate('moment().max is deprecated, use moment.max instead. https://github.com/moment/moment/issues/1548', function () {
      var other = local__createLocal.apply(null, arguments);
      return other > this ? this : other;
    });
  // Pick a moment m from moments so that m[fn](other) is true for all
  // other. This relies on the function fn to be transitive.
  //
  // moments should either be an array of moment objects or an array, whose
  // first element is an array of moment objects.
  function pickBy(fn, moments) {
    var res, i;
    if (moments.length === 1 && isArray(moments[0])) {
      moments = moments[0];
    }
    if (!moments.length) {
      return local__createLocal();
    }
    res = moments[0];
    for (i = 1; i < moments.length; ++i) {
      if (!moments[i].isValid() || moments[i][fn](res)) {
        res = moments[i];
      }
    }
    return res;
  }
  // TODO: Use [].sort instead?
  function min() {
    var args = [].slice.call(arguments, 0);
    return pickBy('isBefore', args);
  }
  function max() {
    var args = [].slice.call(arguments, 0);
    return pickBy('isAfter', args);
  }
  function Duration(duration) {
    var normalizedInput = normalizeObjectUnits(duration), years = normalizedInput.year || 0, quarters = normalizedInput.quarter || 0, months = normalizedInput.month || 0, weeks = normalizedInput.week || 0, days = normalizedInput.day || 0, hours = normalizedInput.hour || 0, minutes = normalizedInput.minute || 0, seconds = normalizedInput.second || 0, milliseconds = normalizedInput.millisecond || 0;
    // representation for dateAddRemove
    this._milliseconds = +milliseconds + seconds * 1000 + minutes * 60000 + hours * 3600000;
    // 1000 * 60 * 60
    // Because of dateAddRemove treats 24 hours as different from a
    // day when working around DST, we need to store them separately
    this._days = +days + weeks * 7;
    // It is impossible translate months into days without knowing
    // which months you are are talking about, so we have to store
    // it separately.
    this._months = +months + quarters * 3 + years * 12;
    this._data = {};
    this._locale = locale_locales__getLocale();
    this._bubble();
  }
  function isDuration(obj) {
    return obj instanceof Duration;
  }
  function offset(token, separator) {
    addFormatToken(token, 0, 0, function () {
      var offset = this.utcOffset();
      var sign = '+';
      if (offset < 0) {
        offset = -offset;
        sign = '-';
      }
      return sign + zeroFill(~~(offset / 60), 2) + separator + zeroFill(~~offset % 60, 2);
    });
  }
  offset('Z', ':');
  offset('ZZ', '');
  // PARSING
  addRegexToken('Z', matchOffset);
  addRegexToken('ZZ', matchOffset);
  addParseToken([
    'Z',
    'ZZ'
  ], function (input, array, config) {
    config._useUTC = true;
    config._tzm = offsetFromString(input);
  });
  // HELPERS
  // timezone chunker
  // '+10:00' > ['10',  '00']
  // '-1530'  > ['-15', '30']
  var chunkOffset = /([\+\-]|\d\d)/gi;
  function offsetFromString(string) {
    var matches = (string || '').match(matchOffset) || [];
    var chunk = matches[matches.length - 1] || [];
    var parts = (chunk + '').match(chunkOffset) || [
        '-',
        0,
        0
      ];
    var minutes = +(parts[1] * 60) + toInt(parts[2]);
    return parts[0] === '+' ? minutes : -minutes;
  }
  // Return a moment from input, that is local/utc/zone equivalent to model.
  function cloneWithOffset(input, model) {
    var res, diff;
    if (model._isUTC) {
      res = model.clone();
      diff = (isMoment(input) || isDate(input) ? +input : +local__createLocal(input)) - +res;
      // Use low-level api, because this fn is low-level api.
      res._d.setTime(+res._d + diff);
      utils_hooks__hooks.updateOffset(res, false);
      return res;
    } else {
      return local__createLocal(input).local();
    }
  }
  function getDateOffset(m) {
    // On Firefox.24 Date#getTimezoneOffset returns a floating point.
    // https://github.com/moment/moment/pull/1871
    return -Math.round(m._d.getTimezoneOffset() / 15) * 15;
  }
  // HOOKS
  // This function will be called whenever a moment is mutated.
  // It is intended to keep the offset in sync with the timezone.
  utils_hooks__hooks.updateOffset = function () {
  };
  // MOMENTS
  // keepLocalTime = true means only change the timezone, without
  // affecting the local hour. So 5:31:26 +0300 --[utcOffset(2, true)]-->
  // 5:31:26 +0200 It is possible that 5:31:26 doesn't exist with offset
  // +0200, so we adjust the time as needed, to be valid.
  //
  // Keeping the time actually adds/subtracts (one hour)
  // from the actual represented time. That is why we call updateOffset
  // a second time. In case it wants us to change the offset again
  // _changeInProgress == true case, then we have to adjust, because
  // there is no such time in the given timezone.
  function getSetOffset(input, keepLocalTime) {
    var offset = this._offset || 0, localAdjust;
    if (input != null) {
      if (typeof input === 'string') {
        input = offsetFromString(input);
      }
      if (Math.abs(input) < 16) {
        input = input * 60;
      }
      if (!this._isUTC && keepLocalTime) {
        localAdjust = getDateOffset(this);
      }
      this._offset = input;
      this._isUTC = true;
      if (localAdjust != null) {
        this.add(localAdjust, 'm');
      }
      if (offset !== input) {
        if (!keepLocalTime || this._changeInProgress) {
          add_subtract__addSubtract(this, create__createDuration(input - offset, 'm'), 1, false);
        } else if (!this._changeInProgress) {
          this._changeInProgress = true;
          utils_hooks__hooks.updateOffset(this, true);
          this._changeInProgress = null;
        }
      }
      return this;
    } else {
      return this._isUTC ? offset : getDateOffset(this);
    }
  }
  function getSetZone(input, keepLocalTime) {
    if (input != null) {
      if (typeof input !== 'string') {
        input = -input;
      }
      this.utcOffset(input, keepLocalTime);
      return this;
    } else {
      return -this.utcOffset();
    }
  }
  function setOffsetToUTC(keepLocalTime) {
    return this.utcOffset(0, keepLocalTime);
  }
  function setOffsetToLocal(keepLocalTime) {
    if (this._isUTC) {
      this.utcOffset(0, keepLocalTime);
      this._isUTC = false;
      if (keepLocalTime) {
        this.subtract(getDateOffset(this), 'm');
      }
    }
    return this;
  }
  function setOffsetToParsedOffset() {
    if (this._tzm) {
      this.utcOffset(this._tzm);
    } else if (typeof this._i === 'string') {
      this.utcOffset(offsetFromString(this._i));
    }
    return this;
  }
  function hasAlignedHourOffset(input) {
    input = input ? local__createLocal(input).utcOffset() : 0;
    return (this.utcOffset() - input) % 60 === 0;
  }
  function isDaylightSavingTime() {
    return this.utcOffset() > this.clone().month(0).utcOffset() || this.utcOffset() > this.clone().month(5).utcOffset();
  }
  function isDaylightSavingTimeShifted() {
    if (typeof this._isDSTShifted !== 'undefined') {
      return this._isDSTShifted;
    }
    var c = {};
    copyConfig(c, this);
    c = prepareConfig(c);
    if (c._a) {
      var other = c._isUTC ? create_utc__createUTC(c._a) : local__createLocal(c._a);
      this._isDSTShifted = this.isValid() && compareArrays(c._a, other.toArray()) > 0;
    } else {
      this._isDSTShifted = false;
    }
    return this._isDSTShifted;
  }
  function isLocal() {
    return !this._isUTC;
  }
  function isUtcOffset() {
    return this._isUTC;
  }
  function isUtc() {
    return this._isUTC && this._offset === 0;
  }
  var aspNetRegex = /(\-)?(?:(\d*)\.)?(\d+)\:(\d+)(?:\:(\d+)\.?(\d{3})?)?/;
  // from http://docs.closure-library.googlecode.com/git/closure_goog_date_date.js.source.html
  // somewhat more in line with 4.4.3.2 2004 spec, but allows decimal anywhere
  var create__isoRegex = /^(-)?P(?:(?:([0-9,.]*)Y)?(?:([0-9,.]*)M)?(?:([0-9,.]*)D)?(?:T(?:([0-9,.]*)H)?(?:([0-9,.]*)M)?(?:([0-9,.]*)S)?)?|([0-9,.]*)W)$/;
  function create__createDuration(input, key) {
    var duration = input,
      // matching against regexp is expensive, do it on demand
      match = null, sign, ret, diffRes;
    if (isDuration(input)) {
      duration = {
        ms: input._milliseconds,
        d: input._days,
        M: input._months
      };
    } else if (typeof input === 'number') {
      duration = {};
      if (key) {
        duration[key] = input;
      } else {
        duration.milliseconds = input;
      }
    } else if (!!(match = aspNetRegex.exec(input))) {
      sign = match[1] === '-' ? -1 : 1;
      duration = {
        y: 0,
        d: toInt(match[DATE]) * sign,
        h: toInt(match[HOUR]) * sign,
        m: toInt(match[MINUTE]) * sign,
        s: toInt(match[SECOND]) * sign,
        ms: toInt(match[MILLISECOND]) * sign
      };
    } else if (!!(match = create__isoRegex.exec(input))) {
      sign = match[1] === '-' ? -1 : 1;
      duration = {
        y: parseIso(match[2], sign),
        M: parseIso(match[3], sign),
        d: parseIso(match[4], sign),
        h: parseIso(match[5], sign),
        m: parseIso(match[6], sign),
        s: parseIso(match[7], sign),
        w: parseIso(match[8], sign)
      };
    } else if (duration == null) {
      // checks for null or undefined
      duration = {};
    } else if (typeof duration === 'object' && ('from' in duration || 'to' in duration)) {
      diffRes = momentsDifference(local__createLocal(duration.from), local__createLocal(duration.to));
      duration = {};
      duration.ms = diffRes.milliseconds;
      duration.M = diffRes.months;
    }
    ret = new Duration(duration);
    if (isDuration(input) && hasOwnProp(input, '_locale')) {
      ret._locale = input._locale;
    }
    return ret;
  }
  create__createDuration.fn = Duration.prototype;
  function parseIso(inp, sign) {
    // We'd normally use ~~inp for this, but unfortunately it also
    // converts floats to ints.
    // inp may be undefined, so careful calling replace on it.
    var res = inp && parseFloat(inp.replace(',', '.'));
    // apply sign while we're at it
    return (isNaN(res) ? 0 : res) * sign;
  }
  function positiveMomentsDifference(base, other) {
    var res = {
        milliseconds: 0,
        months: 0
      };
    res.months = other.month() - base.month() + (other.year() - base.year()) * 12;
    if (base.clone().add(res.months, 'M').isAfter(other)) {
      --res.months;
    }
    res.milliseconds = +other - +base.clone().add(res.months, 'M');
    return res;
  }
  function momentsDifference(base, other) {
    var res;
    other = cloneWithOffset(other, base);
    if (base.isBefore(other)) {
      res = positiveMomentsDifference(base, other);
    } else {
      res = positiveMomentsDifference(other, base);
      res.milliseconds = -res.milliseconds;
      res.months = -res.months;
    }
    return res;
  }
  function createAdder(direction, name) {
    return function (val, period) {
      var dur, tmp;
      //invert the arguments, but complain about it
      if (period !== null && !isNaN(+period)) {
        deprecateSimple(name, 'moment().' + name + '(period, number) is deprecated. Please use moment().' + name + '(number, period).');
        tmp = val;
        val = period;
        period = tmp;
      }
      val = typeof val === 'string' ? +val : val;
      dur = create__createDuration(val, period);
      add_subtract__addSubtract(this, dur, direction);
      return this;
    };
  }
  function add_subtract__addSubtract(mom, duration, isAdding, updateOffset) {
    var milliseconds = duration._milliseconds, days = duration._days, months = duration._months;
    updateOffset = updateOffset == null ? true : updateOffset;
    if (milliseconds) {
      mom._d.setTime(+mom._d + milliseconds * isAdding);
    }
    if (days) {
      get_set__set(mom, 'Date', get_set__get(mom, 'Date') + days * isAdding);
    }
    if (months) {
      setMonth(mom, get_set__get(mom, 'Month') + months * isAdding);
    }
    if (updateOffset) {
      utils_hooks__hooks.updateOffset(mom, days || months);
    }
  }
  var add_subtract__add = createAdder(1, 'add');
  var add_subtract__subtract = createAdder(-1, 'subtract');
  function moment_calendar__calendar(time, formats) {
    // We want to compare the start of today, vs this.
    // Getting start-of-today depends on whether we're local/utc/offset or not.
    var now = time || local__createLocal(), sod = cloneWithOffset(now, this).startOf('day'), diff = this.diff(sod, 'days', true), format = diff < -6 ? 'sameElse' : diff < -1 ? 'lastWeek' : diff < 0 ? 'lastDay' : diff < 1 ? 'sameDay' : diff < 2 ? 'nextDay' : diff < 7 ? 'nextWeek' : 'sameElse';
    return this.format(formats && formats[format] || this.localeData().calendar(format, this, local__createLocal(now)));
  }
  function clone() {
    return new Moment(this);
  }
  function isAfter(input, units) {
    var inputMs;
    units = normalizeUnits(typeof units !== 'undefined' ? units : 'millisecond');
    if (units === 'millisecond') {
      input = isMoment(input) ? input : local__createLocal(input);
      return +this > +input;
    } else {
      inputMs = isMoment(input) ? +input : +local__createLocal(input);
      return inputMs < +this.clone().startOf(units);
    }
  }
  function isBefore(input, units) {
    var inputMs;
    units = normalizeUnits(typeof units !== 'undefined' ? units : 'millisecond');
    if (units === 'millisecond') {
      input = isMoment(input) ? input : local__createLocal(input);
      return +this < +input;
    } else {
      inputMs = isMoment(input) ? +input : +local__createLocal(input);
      return +this.clone().endOf(units) < inputMs;
    }
  }
  function isBetween(from, to, units) {
    return this.isAfter(from, units) && this.isBefore(to, units);
  }
  function isSame(input, units) {
    var inputMs;
    units = normalizeUnits(units || 'millisecond');
    if (units === 'millisecond') {
      input = isMoment(input) ? input : local__createLocal(input);
      return +this === +input;
    } else {
      inputMs = +local__createLocal(input);
      return +this.clone().startOf(units) <= inputMs && inputMs <= +this.clone().endOf(units);
    }
  }
  function diff(input, units, asFloat) {
    var that = cloneWithOffset(input, this), zoneDelta = (that.utcOffset() - this.utcOffset()) * 60000, delta, output;
    units = normalizeUnits(units);
    if (units === 'year' || units === 'month' || units === 'quarter') {
      output = monthDiff(this, that);
      if (units === 'quarter') {
        output = output / 3;
      } else if (units === 'year') {
        output = output / 12;
      }
    } else {
      delta = this - that;
      output = units === 'second' ? delta / 1000 : units === 'minute' ? delta / 60000 : units === 'hour' ? delta / 3600000 : units === 'day' ? (delta - zoneDelta) / 86400000 : units === 'week' ? (delta - zoneDelta) / 604800000 : delta;
    }
    return asFloat ? output : absFloor(output);
  }
  function monthDiff(a, b) {
    // difference in months
    var wholeMonthDiff = (b.year() - a.year()) * 12 + (b.month() - a.month()),
      // b is in (anchor - 1 month, anchor + 1 month)
      anchor = a.clone().add(wholeMonthDiff, 'months'), anchor2, adjust;
    if (b - anchor < 0) {
      anchor2 = a.clone().add(wholeMonthDiff - 1, 'months');
      // linear across the month
      adjust = (b - anchor) / (anchor - anchor2);
    } else {
      anchor2 = a.clone().add(wholeMonthDiff + 1, 'months');
      // linear across the month
      adjust = (b - anchor) / (anchor2 - anchor);
    }
    return -(wholeMonthDiff + adjust);
  }
  utils_hooks__hooks.defaultFormat = 'YYYY-MM-DDTHH:mm:ssZ';
  function toString() {
    return this.clone().locale('en').format('ddd MMM DD YYYY HH:mm:ss [GMT]ZZ');
  }
  function moment_format__toISOString() {
    var m = this.clone().utc();
    if (0 < m.year() && m.year() <= 9999) {
      if ('function' === typeof Date.prototype.toISOString) {
        // native implementation is ~50x faster, use it when we can
        return this.toDate().toISOString();
      } else {
        return formatMoment(m, 'YYYY-MM-DD[T]HH:mm:ss.SSS[Z]');
      }
    } else {
      return formatMoment(m, 'YYYYYY-MM-DD[T]HH:mm:ss.SSS[Z]');
    }
  }
  function format(inputString) {
    var output = formatMoment(this, inputString || utils_hooks__hooks.defaultFormat);
    return this.localeData().postformat(output);
  }
  function from(time, withoutSuffix) {
    if (!this.isValid()) {
      return this.localeData().invalidDate();
    }
    return create__createDuration({
      to: this,
      from: time
    }).locale(this.locale()).humanize(!withoutSuffix);
  }
  function fromNow(withoutSuffix) {
    return this.from(local__createLocal(), withoutSuffix);
  }
  function to(time, withoutSuffix) {
    if (!this.isValid()) {
      return this.localeData().invalidDate();
    }
    return create__createDuration({
      from: this,
      to: time
    }).locale(this.locale()).humanize(!withoutSuffix);
  }
  function toNow(withoutSuffix) {
    return this.to(local__createLocal(), withoutSuffix);
  }
  function locale(key) {
    var newLocaleData;
    if (key === undefined) {
      return this._locale._abbr;
    } else {
      newLocaleData = locale_locales__getLocale(key);
      if (newLocaleData != null) {
        this._locale = newLocaleData;
      }
      return this;
    }
  }
  var lang = deprecate('moment().lang() is deprecated. Instead, use moment().localeData() to get the language configuration. Use moment().locale() to change languages.', function (key) {
      if (key === undefined) {
        return this.localeData();
      } else {
        return this.locale(key);
      }
    });
  function localeData() {
    return this._locale;
  }
  function startOf(units) {
    units = normalizeUnits(units);
    // the following switch intentionally omits break keywords
    // to utilize falling through the cases.
    switch (units) {
    case 'year':
      this.month(0);
    /* falls through */
    case 'quarter':
    case 'month':
      this.date(1);
    /* falls through */
    case 'week':
    case 'isoWeek':
    case 'day':
      this.hours(0);
    /* falls through */
    case 'hour':
      this.minutes(0);
    /* falls through */
    case 'minute':
      this.seconds(0);
    /* falls through */
    case 'second':
      this.milliseconds(0);
    }
    // weeks are a special case
    if (units === 'week') {
      this.weekday(0);
    }
    if (units === 'isoWeek') {
      this.isoWeekday(1);
    }
    // quarters are also special
    if (units === 'quarter') {
      this.month(Math.floor(this.month() / 3) * 3);
    }
    return this;
  }
  function endOf(units) {
    units = normalizeUnits(units);
    if (units === undefined || units === 'millisecond') {
      return this;
    }
    return this.startOf(units).add(1, units === 'isoWeek' ? 'week' : units).subtract(1, 'ms');
  }
  function to_type__valueOf() {
    return +this._d - (this._offset || 0) * 60000;
  }
  function unix() {
    return Math.floor(+this / 1000);
  }
  function toDate() {
    return this._offset ? new Date(+this) : this._d;
  }
  function toArray() {
    var m = this;
    return [
      m.year(),
      m.month(),
      m.date(),
      m.hour(),
      m.minute(),
      m.second(),
      m.millisecond()
    ];
  }
  function toObject() {
    var m = this;
    return {
      years: m.year(),
      months: m.month(),
      date: m.date(),
      hours: m.hours(),
      minutes: m.minutes(),
      seconds: m.seconds(),
      milliseconds: m.milliseconds()
    };
  }
  function moment_valid__isValid() {
    return valid__isValid(this);
  }
  function parsingFlags() {
    return extend({}, getParsingFlags(this));
  }
  function invalidAt() {
    return getParsingFlags(this).overflow;
  }
  addFormatToken(0, [
    'gg',
    2
  ], 0, function () {
    return this.weekYear() % 100;
  });
  addFormatToken(0, [
    'GG',
    2
  ], 0, function () {
    return this.isoWeekYear() % 100;
  });
  function addWeekYearFormatToken(token, getter) {
    addFormatToken(0, [
      token,
      token.length
    ], 0, getter);
  }
  addWeekYearFormatToken('gggg', 'weekYear');
  addWeekYearFormatToken('ggggg', 'weekYear');
  addWeekYearFormatToken('GGGG', 'isoWeekYear');
  addWeekYearFormatToken('GGGGG', 'isoWeekYear');
  // ALIASES
  addUnitAlias('weekYear', 'gg');
  addUnitAlias('isoWeekYear', 'GG');
  // PARSING
  addRegexToken('G', matchSigned);
  addRegexToken('g', matchSigned);
  addRegexToken('GG', match1to2, match2);
  addRegexToken('gg', match1to2, match2);
  addRegexToken('GGGG', match1to4, match4);
  addRegexToken('gggg', match1to4, match4);
  addRegexToken('GGGGG', match1to6, match6);
  addRegexToken('ggggg', match1to6, match6);
  addWeekParseToken([
    'gggg',
    'ggggg',
    'GGGG',
    'GGGGG'
  ], function (input, week, config, token) {
    week[token.substr(0, 2)] = toInt(input);
  });
  addWeekParseToken([
    'gg',
    'GG'
  ], function (input, week, config, token) {
    week[token] = utils_hooks__hooks.parseTwoDigitYear(input);
  });
  // HELPERS
  function weeksInYear(year, dow, doy) {
    return weekOfYear(local__createLocal([
      year,
      11,
      31 + dow - doy
    ]), dow, doy).week;
  }
  // MOMENTS
  function getSetWeekYear(input) {
    var year = weekOfYear(this, this.localeData()._week.dow, this.localeData()._week.doy).year;
    return input == null ? year : this.add(input - year, 'y');
  }
  function getSetISOWeekYear(input) {
    var year = weekOfYear(this, 1, 4).year;
    return input == null ? year : this.add(input - year, 'y');
  }
  function getISOWeeksInYear() {
    return weeksInYear(this.year(), 1, 4);
  }
  function getWeeksInYear() {
    var weekInfo = this.localeData()._week;
    return weeksInYear(this.year(), weekInfo.dow, weekInfo.doy);
  }
  addFormatToken('Q', 0, 0, 'quarter');
  // ALIASES
  addUnitAlias('quarter', 'Q');
  // PARSING
  addRegexToken('Q', match1);
  addParseToken('Q', function (input, array) {
    array[MONTH] = (toInt(input) - 1) * 3;
  });
  // MOMENTS
  function getSetQuarter(input) {
    return input == null ? Math.ceil((this.month() + 1) / 3) : this.month((input - 1) * 3 + this.month() % 3);
  }
  addFormatToken('D', [
    'DD',
    2
  ], 'Do', 'date');
  // ALIASES
  addUnitAlias('date', 'D');
  // PARSING
  addRegexToken('D', match1to2);
  addRegexToken('DD', match1to2, match2);
  addRegexToken('Do', function (isStrict, locale) {
    return isStrict ? locale._ordinalParse : locale._ordinalParseLenient;
  });
  addParseToken([
    'D',
    'DD'
  ], DATE);
  addParseToken('Do', function (input, array) {
    array[DATE] = toInt(input.match(match1to2)[0], 10);
  });
  // MOMENTS
  var getSetDayOfMonth = makeGetSet('Date', true);
  addFormatToken('d', 0, 'do', 'day');
  addFormatToken('dd', 0, 0, function (format) {
    return this.localeData().weekdaysMin(this, format);
  });
  addFormatToken('ddd', 0, 0, function (format) {
    return this.localeData().weekdaysShort(this, format);
  });
  addFormatToken('dddd', 0, 0, function (format) {
    return this.localeData().weekdays(this, format);
  });
  addFormatToken('e', 0, 0, 'weekday');
  addFormatToken('E', 0, 0, 'isoWeekday');
  // ALIASES
  addUnitAlias('day', 'd');
  addUnitAlias('weekday', 'e');
  addUnitAlias('isoWeekday', 'E');
  // PARSING
  addRegexToken('d', match1to2);
  addRegexToken('e', match1to2);
  addRegexToken('E', match1to2);
  addRegexToken('dd', matchWord);
  addRegexToken('ddd', matchWord);
  addRegexToken('dddd', matchWord);
  addWeekParseToken([
    'dd',
    'ddd',
    'dddd'
  ], function (input, week, config) {
    var weekday = config._locale.weekdaysParse(input);
    // if we didn't get a weekday name, mark the date as invalid
    if (weekday != null) {
      week.d = weekday;
    } else {
      getParsingFlags(config).invalidWeekday = input;
    }
  });
  addWeekParseToken([
    'd',
    'e',
    'E'
  ], function (input, week, config, token) {
    week[token] = toInt(input);
  });
  // HELPERS
  function parseWeekday(input, locale) {
    if (typeof input !== 'string') {
      return input;
    }
    if (!isNaN(input)) {
      return parseInt(input, 10);
    }
    input = locale.weekdaysParse(input);
    if (typeof input === 'number') {
      return input;
    }
    return null;
  }
  // LOCALES
  var defaultLocaleWeekdays = 'Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday'.split('_');
  function localeWeekdays(m) {
    return this._weekdays[m.day()];
  }
  var defaultLocaleWeekdaysShort = 'Sun_Mon_Tue_Wed_Thu_Fri_Sat'.split('_');
  function localeWeekdaysShort(m) {
    return this._weekdaysShort[m.day()];
  }
  var defaultLocaleWeekdaysMin = 'Su_Mo_Tu_We_Th_Fr_Sa'.split('_');
  function localeWeekdaysMin(m) {
    return this._weekdaysMin[m.day()];
  }
  function localeWeekdaysParse(weekdayName) {
    var i, mom, regex;
    this._weekdaysParse = this._weekdaysParse || [];
    for (i = 0; i < 7; i++) {
      // make the regex if we don't have it already
      if (!this._weekdaysParse[i]) {
        mom = local__createLocal([
          2000,
          1
        ]).day(i);
        regex = '^' + this.weekdays(mom, '') + '|^' + this.weekdaysShort(mom, '') + '|^' + this.weekdaysMin(mom, '');
        this._weekdaysParse[i] = new RegExp(regex.replace('.', ''), 'i');
      }
      // test the regex
      if (this._weekdaysParse[i].test(weekdayName)) {
        return i;
      }
    }
  }
  // MOMENTS
  function getSetDayOfWeek(input) {
    var day = this._isUTC ? this._d.getUTCDay() : this._d.getDay();
    if (input != null) {
      input = parseWeekday(input, this.localeData());
      return this.add(input - day, 'd');
    } else {
      return day;
    }
  }
  function getSetLocaleDayOfWeek(input) {
    var weekday = (this.day() + 7 - this.localeData()._week.dow) % 7;
    return input == null ? weekday : this.add(input - weekday, 'd');
  }
  function getSetISODayOfWeek(input) {
    // behaves the same as moment#day except
    // as a getter, returns 7 instead of 0 (1-7 range instead of 0-6)
    // as a setter, sunday should belong to the previous week.
    return input == null ? this.day() || 7 : this.day(this.day() % 7 ? input : input - 7);
  }
  addFormatToken('H', [
    'HH',
    2
  ], 0, 'hour');
  addFormatToken('h', [
    'hh',
    2
  ], 0, function () {
    return this.hours() % 12 || 12;
  });
  function meridiem(token, lowercase) {
    addFormatToken(token, 0, 0, function () {
      return this.localeData().meridiem(this.hours(), this.minutes(), lowercase);
    });
  }
  meridiem('a', true);
  meridiem('A', false);
  // ALIASES
  addUnitAlias('hour', 'h');
  // PARSING
  function matchMeridiem(isStrict, locale) {
    return locale._meridiemParse;
  }
  addRegexToken('a', matchMeridiem);
  addRegexToken('A', matchMeridiem);
  addRegexToken('H', match1to2);
  addRegexToken('h', match1to2);
  addRegexToken('HH', match1to2, match2);
  addRegexToken('hh', match1to2, match2);
  addParseToken([
    'H',
    'HH'
  ], HOUR);
  addParseToken([
    'a',
    'A'
  ], function (input, array, config) {
    config._isPm = config._locale.isPM(input);
    config._meridiem = input;
  });
  addParseToken([
    'h',
    'hh'
  ], function (input, array, config) {
    array[HOUR] = toInt(input);
    getParsingFlags(config).bigHour = true;
  });
  // LOCALES
  function localeIsPM(input) {
    // IE8 Quirks Mode & IE7 Standards Mode do not allow accessing strings like arrays
    // Using charAt should be more compatible.
    return (input + '').toLowerCase().charAt(0) === 'p';
  }
  var defaultLocaleMeridiemParse = /[ap]\.?m?\.?/i;
  function localeMeridiem(hours, minutes, isLower) {
    if (hours > 11) {
      return isLower ? 'pm' : 'PM';
    } else {
      return isLower ? 'am' : 'AM';
    }
  }
  // MOMENTS
  // Setting the hour should keep the time, because the user explicitly
  // specified which hour he wants. So trying to maintain the same hour (in
  // a new timezone) makes sense. Adding/subtracting hours does not follow
  // this rule.
  var getSetHour = makeGetSet('Hours', true);
  addFormatToken('m', [
    'mm',
    2
  ], 0, 'minute');
  // ALIASES
  addUnitAlias('minute', 'm');
  // PARSING
  addRegexToken('m', match1to2);
  addRegexToken('mm', match1to2, match2);
  addParseToken([
    'm',
    'mm'
  ], MINUTE);
  // MOMENTS
  var getSetMinute = makeGetSet('Minutes', false);
  addFormatToken('s', [
    'ss',
    2
  ], 0, 'second');
  // ALIASES
  addUnitAlias('second', 's');
  // PARSING
  addRegexToken('s', match1to2);
  addRegexToken('ss', match1to2, match2);
  addParseToken([
    's',
    'ss'
  ], SECOND);
  // MOMENTS
  var getSetSecond = makeGetSet('Seconds', false);
  addFormatToken('S', 0, 0, function () {
    return ~~(this.millisecond() / 100);
  });
  addFormatToken(0, [
    'SS',
    2
  ], 0, function () {
    return ~~(this.millisecond() / 10);
  });
  addFormatToken(0, [
    'SSS',
    3
  ], 0, 'millisecond');
  addFormatToken(0, [
    'SSSS',
    4
  ], 0, function () {
    return this.millisecond() * 10;
  });
  addFormatToken(0, [
    'SSSSS',
    5
  ], 0, function () {
    return this.millisecond() * 100;
  });
  addFormatToken(0, [
    'SSSSSS',
    6
  ], 0, function () {
    return this.millisecond() * 1000;
  });
  addFormatToken(0, [
    'SSSSSSS',
    7
  ], 0, function () {
    return this.millisecond() * 10000;
  });
  addFormatToken(0, [
    'SSSSSSSS',
    8
  ], 0, function () {
    return this.millisecond() * 100000;
  });
  addFormatToken(0, [
    'SSSSSSSSS',
    9
  ], 0, function () {
    return this.millisecond() * 1000000;
  });
  // ALIASES
  addUnitAlias('millisecond', 'ms');
  // PARSING
  addRegexToken('S', match1to3, match1);
  addRegexToken('SS', match1to3, match2);
  addRegexToken('SSS', match1to3, match3);
  var token;
  for (token = 'SSSS'; token.length <= 9; token += 'S') {
    addRegexToken(token, matchUnsigned);
  }
  function parseMs(input, array) {
    array[MILLISECOND] = toInt(('0.' + input) * 1000);
  }
  for (token = 'S'; token.length <= 9; token += 'S') {
    addParseToken(token, parseMs);
  }
  // MOMENTS
  var getSetMillisecond = makeGetSet('Milliseconds', false);
  addFormatToken('z', 0, 0, 'zoneAbbr');
  addFormatToken('zz', 0, 0, 'zoneName');
  // MOMENTS
  function getZoneAbbr() {
    return this._isUTC ? 'UTC' : '';
  }
  function getZoneName() {
    return this._isUTC ? 'Coordinated Universal Time' : '';
  }
  var momentPrototype__proto = Moment.prototype;
  momentPrototype__proto.add = add_subtract__add;
  momentPrototype__proto.calendar = moment_calendar__calendar;
  momentPrototype__proto.clone = clone;
  momentPrototype__proto.diff = diff;
  momentPrototype__proto.endOf = endOf;
  momentPrototype__proto.format = format;
  momentPrototype__proto.from = from;
  momentPrototype__proto.fromNow = fromNow;
  momentPrototype__proto.to = to;
  momentPrototype__proto.toNow = toNow;
  momentPrototype__proto.get = getSet;
  momentPrototype__proto.invalidAt = invalidAt;
  momentPrototype__proto.isAfter = isAfter;
  momentPrototype__proto.isBefore = isBefore;
  momentPrototype__proto.isBetween = isBetween;
  momentPrototype__proto.isSame = isSame;
  momentPrototype__proto.isValid = moment_valid__isValid;
  momentPrototype__proto.lang = lang;
  momentPrototype__proto.locale = locale;
  momentPrototype__proto.localeData = localeData;
  momentPrototype__proto.max = prototypeMax;
  momentPrototype__proto.min = prototypeMin;
  momentPrototype__proto.parsingFlags = parsingFlags;
  momentPrototype__proto.set = getSet;
  momentPrototype__proto.startOf = startOf;
  momentPrototype__proto.subtract = add_subtract__subtract;
  momentPrototype__proto.toArray = toArray;
  momentPrototype__proto.toObject = toObject;
  momentPrototype__proto.toDate = toDate;
  momentPrototype__proto.toISOString = moment_format__toISOString;
  momentPrototype__proto.toJSON = moment_format__toISOString;
  momentPrototype__proto.toString = toString;
  momentPrototype__proto.unix = unix;
  momentPrototype__proto.valueOf = to_type__valueOf;
  // Year
  momentPrototype__proto.year = getSetYear;
  momentPrototype__proto.isLeapYear = getIsLeapYear;
  // Week Year
  momentPrototype__proto.weekYear = getSetWeekYear;
  momentPrototype__proto.isoWeekYear = getSetISOWeekYear;
  // Quarter
  momentPrototype__proto.quarter = momentPrototype__proto.quarters = getSetQuarter;
  // Month
  momentPrototype__proto.month = getSetMonth;
  momentPrototype__proto.daysInMonth = getDaysInMonth;
  // Week
  momentPrototype__proto.week = momentPrototype__proto.weeks = getSetWeek;
  momentPrototype__proto.isoWeek = momentPrototype__proto.isoWeeks = getSetISOWeek;
  momentPrototype__proto.weeksInYear = getWeeksInYear;
  momentPrototype__proto.isoWeeksInYear = getISOWeeksInYear;
  // Day
  momentPrototype__proto.date = getSetDayOfMonth;
  momentPrototype__proto.day = momentPrototype__proto.days = getSetDayOfWeek;
  momentPrototype__proto.weekday = getSetLocaleDayOfWeek;
  momentPrototype__proto.isoWeekday = getSetISODayOfWeek;
  momentPrototype__proto.dayOfYear = getSetDayOfYear;
  // Hour
  momentPrototype__proto.hour = momentPrototype__proto.hours = getSetHour;
  // Minute
  momentPrototype__proto.minute = momentPrototype__proto.minutes = getSetMinute;
  // Second
  momentPrototype__proto.second = momentPrototype__proto.seconds = getSetSecond;
  // Millisecond
  momentPrototype__proto.millisecond = momentPrototype__proto.milliseconds = getSetMillisecond;
  // Offset
  momentPrototype__proto.utcOffset = getSetOffset;
  momentPrototype__proto.utc = setOffsetToUTC;
  momentPrototype__proto.local = setOffsetToLocal;
  momentPrototype__proto.parseZone = setOffsetToParsedOffset;
  momentPrototype__proto.hasAlignedHourOffset = hasAlignedHourOffset;
  momentPrototype__proto.isDST = isDaylightSavingTime;
  momentPrototype__proto.isDSTShifted = isDaylightSavingTimeShifted;
  momentPrototype__proto.isLocal = isLocal;
  momentPrototype__proto.isUtcOffset = isUtcOffset;
  momentPrototype__proto.isUtc = isUtc;
  momentPrototype__proto.isUTC = isUtc;
  // Timezone
  momentPrototype__proto.zoneAbbr = getZoneAbbr;
  momentPrototype__proto.zoneName = getZoneName;
  // Deprecations
  momentPrototype__proto.dates = deprecate('dates accessor is deprecated. Use date instead.', getSetDayOfMonth);
  momentPrototype__proto.months = deprecate('months accessor is deprecated. Use month instead', getSetMonth);
  momentPrototype__proto.years = deprecate('years accessor is deprecated. Use year instead', getSetYear);
  momentPrototype__proto.zone = deprecate('moment().zone is deprecated, use moment().utcOffset instead. https://github.com/moment/moment/issues/1779', getSetZone);
  var momentPrototype = momentPrototype__proto;
  function moment__createUnix(input) {
    return local__createLocal(input * 1000);
  }
  function moment__createInZone() {
    return local__createLocal.apply(null, arguments).parseZone();
  }
  var defaultCalendar = {
      sameDay: '[Today at] LT',
      nextDay: '[Tomorrow at] LT',
      nextWeek: 'dddd [at] LT',
      lastDay: '[Yesterday at] LT',
      lastWeek: '[Last] dddd [at] LT',
      sameElse: 'L'
    };
  function locale_calendar__calendar(key, mom, now) {
    var output = this._calendar[key];
    return typeof output === 'function' ? output.call(mom, now) : output;
  }
  var defaultLongDateFormat = {
      LTS: 'h:mm:ss A',
      LT: 'h:mm A',
      L: 'MM/DD/YYYY',
      LL: 'MMMM D, YYYY',
      LLL: 'MMMM D, YYYY h:mm A',
      LLLL: 'dddd, MMMM D, YYYY h:mm A'
    };
  function longDateFormat(key) {
    var format = this._longDateFormat[key], formatUpper = this._longDateFormat[key.toUpperCase()];
    if (format || !formatUpper) {
      return format;
    }
    this._longDateFormat[key] = formatUpper.replace(/MMMM|MM|DD|dddd/g, function (val) {
      return val.slice(1);
    });
    return this._longDateFormat[key];
  }
  var defaultInvalidDate = 'Invalid date';
  function invalidDate() {
    return this._invalidDate;
  }
  var defaultOrdinal = '%d';
  var defaultOrdinalParse = /\d{1,2}/;
  function ordinal(number) {
    return this._ordinal.replace('%d', number);
  }
  function preParsePostFormat(string) {
    return string;
  }
  var defaultRelativeTime = {
      future: 'in %s',
      past: '%s ago',
      s: 'a few seconds',
      m: 'a minute',
      mm: '%d minutes',
      h: 'an hour',
      hh: '%d hours',
      d: 'a day',
      dd: '%d days',
      M: 'a month',
      MM: '%d months',
      y: 'a year',
      yy: '%d years'
    };
  function relative__relativeTime(number, withoutSuffix, string, isFuture) {
    var output = this._relativeTime[string];
    return typeof output === 'function' ? output(number, withoutSuffix, string, isFuture) : output.replace(/%d/i, number);
  }
  function pastFuture(diff, output) {
    var format = this._relativeTime[diff > 0 ? 'future' : 'past'];
    return typeof format === 'function' ? format(output) : format.replace(/%s/i, output);
  }
  function locale_set__set(config) {
    var prop, i;
    for (i in config) {
      prop = config[i];
      if (typeof prop === 'function') {
        this[i] = prop;
      } else {
        this['_' + i] = prop;
      }
    }
    // Lenient ordinal parsing accepts just a number in addition to
    // number + (possibly) stuff coming from _ordinalParseLenient.
    this._ordinalParseLenient = new RegExp(this._ordinalParse.source + '|' + /\d{1,2}/.source);
  }
  var prototype__proto = Locale.prototype;
  prototype__proto._calendar = defaultCalendar;
  prototype__proto.calendar = locale_calendar__calendar;
  prototype__proto._longDateFormat = defaultLongDateFormat;
  prototype__proto.longDateFormat = longDateFormat;
  prototype__proto._invalidDate = defaultInvalidDate;
  prototype__proto.invalidDate = invalidDate;
  prototype__proto._ordinal = defaultOrdinal;
  prototype__proto.ordinal = ordinal;
  prototype__proto._ordinalParse = defaultOrdinalParse;
  prototype__proto.preparse = preParsePostFormat;
  prototype__proto.postformat = preParsePostFormat;
  prototype__proto._relativeTime = defaultRelativeTime;
  prototype__proto.relativeTime = relative__relativeTime;
  prototype__proto.pastFuture = pastFuture;
  prototype__proto.set = locale_set__set;
  // Month
  prototype__proto.months = localeMonths;
  prototype__proto._months = defaultLocaleMonths;
  prototype__proto.monthsShort = localeMonthsShort;
  prototype__proto._monthsShort = defaultLocaleMonthsShort;
  prototype__proto.monthsParse = localeMonthsParse;
  // Week
  prototype__proto.week = localeWeek;
  prototype__proto._week = defaultLocaleWeek;
  prototype__proto.firstDayOfYear = localeFirstDayOfYear;
  prototype__proto.firstDayOfWeek = localeFirstDayOfWeek;
  // Day of Week
  prototype__proto.weekdays = localeWeekdays;
  prototype__proto._weekdays = defaultLocaleWeekdays;
  prototype__proto.weekdaysMin = localeWeekdaysMin;
  prototype__proto._weekdaysMin = defaultLocaleWeekdaysMin;
  prototype__proto.weekdaysShort = localeWeekdaysShort;
  prototype__proto._weekdaysShort = defaultLocaleWeekdaysShort;
  prototype__proto.weekdaysParse = localeWeekdaysParse;
  // Hours
  prototype__proto.isPM = localeIsPM;
  prototype__proto._meridiemParse = defaultLocaleMeridiemParse;
  prototype__proto.meridiem = localeMeridiem;
  function lists__get(format, index, field, setter) {
    var locale = locale_locales__getLocale();
    var utc = create_utc__createUTC().set(setter, index);
    return locale[field](utc, format);
  }
  function list(format, index, field, count, setter) {
    if (typeof format === 'number') {
      index = format;
      format = undefined;
    }
    format = format || '';
    if (index != null) {
      return lists__get(format, index, field, setter);
    }
    var i;
    var out = [];
    for (i = 0; i < count; i++) {
      out[i] = lists__get(format, i, field, setter);
    }
    return out;
  }
  function lists__listMonths(format, index) {
    return list(format, index, 'months', 12, 'month');
  }
  function lists__listMonthsShort(format, index) {
    return list(format, index, 'monthsShort', 12, 'month');
  }
  function lists__listWeekdays(format, index) {
    return list(format, index, 'weekdays', 7, 'day');
  }
  function lists__listWeekdaysShort(format, index) {
    return list(format, index, 'weekdaysShort', 7, 'day');
  }
  function lists__listWeekdaysMin(format, index) {
    return list(format, index, 'weekdaysMin', 7, 'day');
  }
  locale_locales__getSetGlobalLocale('en', {
    ordinalParse: /\d{1,2}(th|st|nd|rd)/,
    ordinal: function (number) {
      var b = number % 10, output = toInt(number % 100 / 10) === 1 ? 'th' : b === 1 ? 'st' : b === 2 ? 'nd' : b === 3 ? 'rd' : 'th';
      return number + output;
    }
  });
  // Side effect imports
  utils_hooks__hooks.lang = deprecate('moment.lang is deprecated. Use moment.locale instead.', locale_locales__getSetGlobalLocale);
  utils_hooks__hooks.langData = deprecate('moment.langData is deprecated. Use moment.localeData instead.', locale_locales__getLocale);
  var mathAbs = Math.abs;
  function duration_abs__abs() {
    var data = this._data;
    this._milliseconds = mathAbs(this._milliseconds);
    this._days = mathAbs(this._days);
    this._months = mathAbs(this._months);
    data.milliseconds = mathAbs(data.milliseconds);
    data.seconds = mathAbs(data.seconds);
    data.minutes = mathAbs(data.minutes);
    data.hours = mathAbs(data.hours);
    data.months = mathAbs(data.months);
    data.years = mathAbs(data.years);
    return this;
  }
  function duration_add_subtract__addSubtract(duration, input, value, direction) {
    var other = create__createDuration(input, value);
    duration._milliseconds += direction * other._milliseconds;
    duration._days += direction * other._days;
    duration._months += direction * other._months;
    return duration._bubble();
  }
  // supports only 2.0-style add(1, 's') or add(duration)
  function duration_add_subtract__add(input, value) {
    return duration_add_subtract__addSubtract(this, input, value, 1);
  }
  // supports only 2.0-style subtract(1, 's') or subtract(duration)
  function duration_add_subtract__subtract(input, value) {
    return duration_add_subtract__addSubtract(this, input, value, -1);
  }
  function absCeil(number) {
    if (number < 0) {
      return Math.floor(number);
    } else {
      return Math.ceil(number);
    }
  }
  function bubble() {
    var milliseconds = this._milliseconds;
    var days = this._days;
    var months = this._months;
    var data = this._data;
    var seconds, minutes, hours, years, monthsFromDays;
    // if we have a mix of positive and negative values, bubble down first
    // check: https://github.com/moment/moment/issues/2166
    if (!(milliseconds >= 0 && days >= 0 && months >= 0 || milliseconds <= 0 && days <= 0 && months <= 0)) {
      milliseconds += absCeil(monthsToDays(months) + days) * 86400000;
      days = 0;
      months = 0;
    }
    // The following code bubbles up values, see the tests for
    // examples of what that means.
    data.milliseconds = milliseconds % 1000;
    seconds = absFloor(milliseconds / 1000);
    data.seconds = seconds % 60;
    minutes = absFloor(seconds / 60);
    data.minutes = minutes % 60;
    hours = absFloor(minutes / 60);
    data.hours = hours % 24;
    days += absFloor(hours / 24);
    // convert days to months
    monthsFromDays = absFloor(daysToMonths(days));
    months += monthsFromDays;
    days -= absCeil(monthsToDays(monthsFromDays));
    // 12 months -> 1 year
    years = absFloor(months / 12);
    months %= 12;
    data.days = days;
    data.months = months;
    data.years = years;
    return this;
  }
  function daysToMonths(days) {
    // 400 years have 146097 days (taking into account leap year rules)
    // 400 years have 12 months === 4800
    return days * 4800 / 146097;
  }
  function monthsToDays(months) {
    // the reverse of daysToMonths
    return months * 146097 / 4800;
  }
  function as(units) {
    var days;
    var months;
    var milliseconds = this._milliseconds;
    units = normalizeUnits(units);
    if (units === 'month' || units === 'year') {
      days = this._days + milliseconds / 86400000;
      months = this._months + daysToMonths(days);
      return units === 'month' ? months : months / 12;
    } else {
      // handle milliseconds separately because of floating point math errors (issue #1867)
      days = this._days + Math.round(monthsToDays(this._months));
      switch (units) {
      case 'week':
        return days / 7 + milliseconds / 604800000;
      case 'day':
        return days + milliseconds / 86400000;
      case 'hour':
        return days * 24 + milliseconds / 3600000;
      case 'minute':
        return days * 1440 + milliseconds / 60000;
      case 'second':
        return days * 86400 + milliseconds / 1000;
      // Math.floor prevents floating point math errors here
      case 'millisecond':
        return Math.floor(days * 86400000) + milliseconds;
      default:
        throw new Error('Unknown unit ' + units);
      }
    }
  }
  // TODO: Use this.as('ms')?
  function duration_as__valueOf() {
    return this._milliseconds + this._days * 86400000 + this._months % 12 * 2592000000 + toInt(this._months / 12) * 31536000000;
  }
  function makeAs(alias) {
    return function () {
      return this.as(alias);
    };
  }
  var asMilliseconds = makeAs('ms');
  var asSeconds = makeAs('s');
  var asMinutes = makeAs('m');
  var asHours = makeAs('h');
  var asDays = makeAs('d');
  var asWeeks = makeAs('w');
  var asMonths = makeAs('M');
  var asYears = makeAs('y');
  function duration_get__get(units) {
    units = normalizeUnits(units);
    return this[units + 's']();
  }
  function makeGetter(name) {
    return function () {
      return this._data[name];
    };
  }
  var milliseconds = makeGetter('milliseconds');
  var seconds = makeGetter('seconds');
  var minutes = makeGetter('minutes');
  var hours = makeGetter('hours');
  var days = makeGetter('days');
  var months = makeGetter('months');
  var years = makeGetter('years');
  function weeks() {
    return absFloor(this.days() / 7);
  }
  var round = Math.round;
  var thresholds = {
      s: 45,
      m: 45,
      h: 22,
      d: 26,
      M: 11
    };
  // helper function for moment.fn.from, moment.fn.fromNow, and moment.duration.fn.humanize
  function substituteTimeAgo(string, number, withoutSuffix, isFuture, locale) {
    return locale.relativeTime(number || 1, !!withoutSuffix, string, isFuture);
  }
  function duration_humanize__relativeTime(posNegDuration, withoutSuffix, locale) {
    var duration = create__createDuration(posNegDuration).abs();
    var seconds = round(duration.as('s'));
    var minutes = round(duration.as('m'));
    var hours = round(duration.as('h'));
    var days = round(duration.as('d'));
    var months = round(duration.as('M'));
    var years = round(duration.as('y'));
    var a = seconds < thresholds.s && [
        's',
        seconds
      ] || minutes === 1 && ['m'] || minutes < thresholds.m && [
        'mm',
        minutes
      ] || hours === 1 && ['h'] || hours < thresholds.h && [
        'hh',
        hours
      ] || days === 1 && ['d'] || days < thresholds.d && [
        'dd',
        days
      ] || months === 1 && ['M'] || months < thresholds.M && [
        'MM',
        months
      ] || years === 1 && ['y'] || [
        'yy',
        years
      ];
    a[2] = withoutSuffix;
    a[3] = +posNegDuration > 0;
    a[4] = locale;
    return substituteTimeAgo.apply(null, a);
  }
  // This function allows you to set a threshold for relative time strings
  function duration_humanize__getSetRelativeTimeThreshold(threshold, limit) {
    if (thresholds[threshold] === undefined) {
      return false;
    }
    if (limit === undefined) {
      return thresholds[threshold];
    }
    thresholds[threshold] = limit;
    return true;
  }
  function humanize(withSuffix) {
    var locale = this.localeData();
    var output = duration_humanize__relativeTime(this, !withSuffix, locale);
    if (withSuffix) {
      output = locale.pastFuture(+this, output);
    }
    return locale.postformat(output);
  }
  var iso_string__abs = Math.abs;
  function iso_string__toISOString() {
    // for ISO strings we do not use the normal bubbling rules:
    //  * milliseconds bubble up until they become hours
    //  * days do not bubble at all
    //  * months bubble up until they become years
    // This is because there is no context-free conversion between hours and days
    // (think of clock changes)
    // and also not between days and months (28-31 days per month)
    var seconds = iso_string__abs(this._milliseconds) / 1000;
    var days = iso_string__abs(this._days);
    var months = iso_string__abs(this._months);
    var minutes, hours, years;
    // 3600 seconds -> 60 minutes -> 1 hour
    minutes = absFloor(seconds / 60);
    hours = absFloor(minutes / 60);
    seconds %= 60;
    minutes %= 60;
    // 12 months -> 1 year
    years = absFloor(months / 12);
    months %= 12;
    // inspired by https://github.com/dordille/moment-isoduration/blob/master/moment.isoduration.js
    var Y = years;
    var M = months;
    var D = days;
    var h = hours;
    var m = minutes;
    var s = seconds;
    var total = this.asSeconds();
    if (!total) {
      // this is the same as C#'s (Noda) and python (isodate)...
      // but not other JS (goog.date)
      return 'P0D';
    }
    return (total < 0 ? '-' : '') + 'P' + (Y ? Y + 'Y' : '') + (M ? M + 'M' : '') + (D ? D + 'D' : '') + (h || m || s ? 'T' : '') + (h ? h + 'H' : '') + (m ? m + 'M' : '') + (s ? s + 'S' : '');
  }
  var duration_prototype__proto = Duration.prototype;
  duration_prototype__proto.abs = duration_abs__abs;
  duration_prototype__proto.add = duration_add_subtract__add;
  duration_prototype__proto.subtract = duration_add_subtract__subtract;
  duration_prototype__proto.as = as;
  duration_prototype__proto.asMilliseconds = asMilliseconds;
  duration_prototype__proto.asSeconds = asSeconds;
  duration_prototype__proto.asMinutes = asMinutes;
  duration_prototype__proto.asHours = asHours;
  duration_prototype__proto.asDays = asDays;
  duration_prototype__proto.asWeeks = asWeeks;
  duration_prototype__proto.asMonths = asMonths;
  duration_prototype__proto.asYears = asYears;
  duration_prototype__proto.valueOf = duration_as__valueOf;
  duration_prototype__proto._bubble = bubble;
  duration_prototype__proto.get = duration_get__get;
  duration_prototype__proto.milliseconds = milliseconds;
  duration_prototype__proto.seconds = seconds;
  duration_prototype__proto.minutes = minutes;
  duration_prototype__proto.hours = hours;
  duration_prototype__proto.days = days;
  duration_prototype__proto.weeks = weeks;
  duration_prototype__proto.months = months;
  duration_prototype__proto.years = years;
  duration_prototype__proto.humanize = humanize;
  duration_prototype__proto.toISOString = iso_string__toISOString;
  duration_prototype__proto.toString = iso_string__toISOString;
  duration_prototype__proto.toJSON = iso_string__toISOString;
  duration_prototype__proto.locale = locale;
  duration_prototype__proto.localeData = localeData;
  // Deprecations
  duration_prototype__proto.toIsoString = deprecate('toIsoString() is deprecated. Please use toISOString() instead (notice the capitals)', iso_string__toISOString);
  duration_prototype__proto.lang = lang;
  // Side effect imports
  addFormatToken('X', 0, 0, 'unix');
  addFormatToken('x', 0, 0, 'valueOf');
  // PARSING
  addRegexToken('x', matchSigned);
  addRegexToken('X', matchTimestamp);
  addParseToken('X', function (input, array, config) {
    config._d = new Date(parseFloat(input, 10) * 1000);
  });
  addParseToken('x', function (input, array, config) {
    config._d = new Date(toInt(input));
  });
  // Side effect imports
  utils_hooks__hooks.version = '2.10.6';
  setHookCallback(local__createLocal);
  utils_hooks__hooks.fn = momentPrototype;
  utils_hooks__hooks.min = min;
  utils_hooks__hooks.max = max;
  utils_hooks__hooks.utc = create_utc__createUTC;
  utils_hooks__hooks.unix = moment__createUnix;
  utils_hooks__hooks.months = lists__listMonths;
  utils_hooks__hooks.isDate = isDate;
  utils_hooks__hooks.locale = locale_locales__getSetGlobalLocale;
  utils_hooks__hooks.invalid = valid__createInvalid;
  utils_hooks__hooks.duration = create__createDuration;
  utils_hooks__hooks.isMoment = isMoment;
  utils_hooks__hooks.weekdays = lists__listWeekdays;
  utils_hooks__hooks.parseZone = moment__createInZone;
  utils_hooks__hooks.localeData = locale_locales__getLocale;
  utils_hooks__hooks.isDuration = isDuration;
  utils_hooks__hooks.monthsShort = lists__listMonthsShort;
  utils_hooks__hooks.weekdaysMin = lists__listWeekdaysMin;
  utils_hooks__hooks.defineLocale = defineLocale;
  utils_hooks__hooks.weekdaysShort = lists__listWeekdaysShort;
  utils_hooks__hooks.normalizeUnits = normalizeUnits;
  utils_hooks__hooks.relativeTimeThreshold = duration_humanize__getSetRelativeTimeThreshold;
  var _moment = utils_hooks__hooks;
  return _moment;
}));
/* angular-moment.js / v0.10.3 / (c) 2013, 2014, 2015 Uri Shaked / MIT Licence */
'format amd';
/* global define */
(function () {
  'use strict';
  function angularMoment(angular, moment) {
    /**
		 * @ngdoc overview
		 * @name angularMoment
		 *
		 * @description
		 * angularMoment module provides moment.js functionality for angular.js apps.
		 */
    return angular.module('angularMoment', []).constant('angularMomentConfig', {
      preprocess: null,
      timezone: '',
      format: null,
      statefulFilters: true
    }).constant('moment', moment).constant('amTimeAgoConfig', {
      withoutSuffix: false,
      serverTime: null,
      titleFormat: null,
      fullDateThreshold: null,
      fullDateFormat: null
    }).directive('amTimeAgo', [
      '$window',
      'moment',
      'amMoment',
      'amTimeAgoConfig',
      'angularMomentConfig',
      function ($window, moment, amMoment, amTimeAgoConfig, angularMomentConfig) {
        return function (scope, element, attr) {
          var activeTimeout = null;
          var currentValue;
          var currentFormat = angularMomentConfig.format;
          var withoutSuffix = amTimeAgoConfig.withoutSuffix;
          var titleFormat = amTimeAgoConfig.titleFormat;
          var fullDateThreshold = amTimeAgoConfig.fullDateThreshold;
          var fullDateFormat = amTimeAgoConfig.fullDateFormat;
          var localDate = new Date().getTime();
          var preprocess = angularMomentConfig.preprocess;
          var modelName = attr.amTimeAgo;
          var currentFrom;
          var isTimeElement = 'TIME' === element[0].nodeName.toUpperCase();
          function getNow() {
            var now;
            if (currentFrom) {
              now = currentFrom;
            } else if (amTimeAgoConfig.serverTime) {
              var localNow = new Date().getTime();
              var nowMillis = localNow - localDate + amTimeAgoConfig.serverTime;
              now = moment(nowMillis);
            } else {
              now = moment();
            }
            return now;
          }
          function cancelTimer() {
            if (activeTimeout) {
              $window.clearTimeout(activeTimeout);
              activeTimeout = null;
            }
          }
          function updateTime(momentInstance) {
            var daysAgo = getNow().diff(momentInstance, 'day');
            var showFullDate = fullDateThreshold && daysAgo >= fullDateThreshold;
            if (showFullDate) {
              element.text(momentInstance.format(fullDateFormat));
            } else {
              element.text(momentInstance.from(getNow(), withoutSuffix));
            }
            if (titleFormat && !element.attr('title')) {
              element.attr('title', momentInstance.local().format(titleFormat));
            }
            if (!showFullDate) {
              var howOld = Math.abs(getNow().diff(momentInstance, 'minute'));
              var secondsUntilUpdate = 3600;
              if (howOld < 1) {
                secondsUntilUpdate = 1;
              } else if (howOld < 60) {
                secondsUntilUpdate = 30;
              } else if (howOld < 180) {
                secondsUntilUpdate = 300;
              }
              activeTimeout = $window.setTimeout(function () {
                updateTime(momentInstance);
              }, secondsUntilUpdate * 1000);
            }
          }
          function updateDateTimeAttr(value) {
            if (isTimeElement) {
              element.attr('datetime', value);
            }
          }
          function updateMoment() {
            cancelTimer();
            if (currentValue) {
              var momentValue = amMoment.preprocessDate(currentValue, preprocess, currentFormat);
              updateTime(momentValue);
              updateDateTimeAttr(momentValue.toISOString());
            }
          }
          scope.$watch(modelName, function (value) {
            if (typeof value === 'undefined' || value === null || value === '') {
              cancelTimer();
              if (currentValue) {
                element.text('');
                updateDateTimeAttr('');
                currentValue = null;
              }
              return;
            }
            currentValue = value;
            updateMoment();
          });
          if (angular.isDefined(attr.amFrom)) {
            scope.$watch(attr.amFrom, function (value) {
              if (typeof value === 'undefined' || value === null || value === '') {
                currentFrom = null;
              } else {
                currentFrom = moment(value);
              }
              updateMoment();
            });
          }
          if (angular.isDefined(attr.amWithoutSuffix)) {
            scope.$watch(attr.amWithoutSuffix, function (value) {
              if (typeof value === 'boolean') {
                withoutSuffix = value;
                updateMoment();
              } else {
                withoutSuffix = amTimeAgoConfig.withoutSuffix;
              }
            });
          }
          attr.$observe('amFormat', function (format) {
            if (typeof format !== 'undefined') {
              currentFormat = format;
              updateMoment();
            }
          });
          attr.$observe('amPreprocess', function (newValue) {
            preprocess = newValue;
            updateMoment();
          });
          attr.$observe('amFullDateThreshold', function (newValue) {
            fullDateThreshold = newValue;
            updateMoment();
          });
          attr.$observe('amFullDateFormat', function (newValue) {
            fullDateFormat = newValue;
            updateMoment();
          });
          scope.$on('$destroy', function () {
            cancelTimer();
          });
          scope.$on('amMoment:localeChanged', function () {
            updateMoment();
          });
        };
      }
    ]).service('amMoment', [
      'moment',
      '$rootScope',
      '$log',
      'angularMomentConfig',
      function (moment, $rootScope, $log, angularMomentConfig) {
        /**
				 * @ngdoc property
				 * @name angularMoment:amMoment#preprocessors
				 * @module angularMoment
				 *
				 * @description
				 * Defines the preprocessors for the preprocessDate method. By default, the following preprocessors
				 * are defined: utc, unix.
				 */
        this.preprocessors = {
          utc: moment.utc,
          unix: moment.unix
        };
        /**
				 * @ngdoc function
				 * @name angularMoment.service.amMoment#changeLocale
				 * @methodOf angularMoment.service.amMoment
				 *
				 * @description
				 * Changes the locale for moment.js and updates all the am-time-ago directive instances
				 * with the new locale. Also broadcasts an `amMoment:localeChanged` event on $rootScope.
				 *
				 * @param {string} locale Locale code (e.g. en, es, ru, pt-br, etc.)
				 * @param {object} customization object of locale strings to override
				 */
        this.changeLocale = function (locale, customization) {
          var result = moment.locale(locale, customization);
          if (angular.isDefined(locale)) {
            $rootScope.$broadcast('amMoment:localeChanged');
          }
          return result;
        };
        /**
				 * @ngdoc function
				 * @name angularMoment.service.amMoment#changeTimezone
				 * @methodOf angularMoment.service.amMoment
				 *
				 * @description
				 * Changes the default timezone for amCalendar, amDateFormat and amTimeAgo. Also broadcasts an
				 * `amMoment:timezoneChanged` event on $rootScope.
				 *
				 * @param {string} timezone Timezone name (e.g. UTC)
				 */
        this.changeTimezone = function (timezone) {
          angularMomentConfig.timezone = timezone;
          $rootScope.$broadcast('amMoment:timezoneChanged');
        };
        /**
				 * @ngdoc function
				 * @name angularMoment.service.amMoment#preprocessDate
				 * @methodOf angularMoment.service.amMoment
				 *
				 * @description
				 * Preprocess a given value and convert it into a Moment instance appropriate for use in the
				 * am-time-ago directive and the filters.
				 *
				 * @param {*} value The value to be preprocessed
				 * @param {string} preprocess The name of the preprocessor the apply (e.g. utc, unix)
				 * @param {string=} format Specifies how to parse the value (see {@link http://momentjs.com/docs/#/parsing/string-format/})
				 * @return {Moment} A value that can be parsed by the moment library
				 */
        this.preprocessDate = function (value, preprocess, format) {
          if (angular.isUndefined(preprocess)) {
            preprocess = angularMomentConfig.preprocess;
          }
          if (this.preprocessors[preprocess]) {
            return this.preprocessors[preprocess](value, format);
          }
          if (preprocess) {
            $log.warn('angular-moment: Ignoring unsupported value for preprocess: ' + preprocess);
          }
          if (!isNaN(parseFloat(value)) && isFinite(value)) {
            // Milliseconds since the epoch
            return moment(parseInt(value, 10));
          }
          // else just returns the value as-is.
          return moment(value, format);
        };
        /**
				 * @ngdoc function
				 * @name angularMoment.service.amMoment#applyTimezone
				 * @methodOf angularMoment.service.amMoment
				 *
				 * @description
				 * Apply a timezone onto a given moment object. It can be a named timezone (e.g. 'America/Phoenix') or an offset from UTC (e.g. '+0300')
				 * moment-timezone.js is needed when a named timezone is used, otherwise, it'll not apply any timezone shift.
				 *
				 * @param {Moment} aMoment a moment() instance to apply the timezone shift to
				 * @param {string=} timezone The timezone to apply. If none given, will apply the timezone
				 *        configured in angularMomentConfig.timezone. It can be a named timezone (e.g. 'America/Phoenix') or an offset from UTC (e.g. '+0300')
				 *
				 * @returns {Moment} The given moment with the timezone shift applied
				 */
        this.applyTimezone = function (aMoment, timezone) {
          timezone = timezone || angularMomentConfig.timezone;
          if (!timezone) {
            return aMoment;
          }
          if (timezone.match(/^Z|[+-]\d\d:?\d\d$/i)) {
            aMoment = aMoment.utcOffset(timezone);
          } else if (aMoment.tz) {
            aMoment = aMoment.tz(timezone);
          } else {
            $log.warn('angular-moment: named timezone specified but moment.tz() is undefined. Did you forget to include moment-timezone.js?');
          }
          return aMoment;
        };
      }
    ]).filter('amCalendar', [
      'moment',
      'amMoment',
      'angularMomentConfig',
      function (moment, amMoment, angularMomentConfig) {
        function amCalendarFilter(value, preprocess, timezone) {
          if (typeof value === 'undefined' || value === null) {
            return '';
          }
          value = amMoment.preprocessDate(value, preprocess);
          var date = moment(value);
          if (!date.isValid()) {
            return '';
          }
          return amMoment.applyTimezone(date, timezone).calendar();
        }
        // Since AngularJS 1.3, filters have to explicitly define being stateful
        // (this is no longer the default).
        amCalendarFilter.$stateful = angularMomentConfig.statefulFilters;
        return amCalendarFilter;
      }
    ]).filter('amDifference', [
      'moment',
      'amMoment',
      'angularMomentConfig',
      function (moment, amMoment, angularMomentConfig) {
        function amDifferenceFilter(value, otherValue, unit, usePrecision, preprocessValue, preprocessOtherValue) {
          if (typeof value === 'undefined' || value === null) {
            return '';
          }
          value = amMoment.preprocessDate(value, preprocessValue);
          var date = moment(value);
          if (!date.isValid()) {
            return '';
          }
          var date2;
          if (typeof otherValue === 'undefined' || otherValue === null) {
            date2 = moment();
          } else {
            otherValue = amMoment.preprocessDate(otherValue, preprocessOtherValue);
            date2 = moment(otherValue);
            if (!date2.isValid()) {
              return '';
            }
          }
          return amMoment.applyTimezone(date).diff(amMoment.applyTimezone(date2), unit, usePrecision);
        }
        amDifferenceFilter.$stateful = angularMomentConfig.statefulFilters;
        return amDifferenceFilter;
      }
    ]).filter('amDateFormat', [
      'moment',
      'amMoment',
      'angularMomentConfig',
      function (moment, amMoment, angularMomentConfig) {
        function amDateFormatFilter(value, format, preprocess, timezone, inputFormat) {
          var currentFormat = inputFormat || angularMomentConfig.format;
          if (typeof value === 'undefined' || value === null) {
            return '';
          }
          value = amMoment.preprocessDate(value, preprocess, currentFormat);
          var date = moment(value);
          if (!date.isValid()) {
            return '';
          }
          return amMoment.applyTimezone(date, timezone).format(format);
        }
        amDateFormatFilter.$stateful = angularMomentConfig.statefulFilters;
        return amDateFormatFilter;
      }
    ]).filter('amDurationFormat', [
      'moment',
      'angularMomentConfig',
      function (moment, angularMomentConfig) {
        function amDurationFormatFilter(value, format, suffix) {
          if (typeof value === 'undefined' || value === null) {
            return '';
          }
          return moment.duration(value, format).humanize(suffix);
        }
        amDurationFormatFilter.$stateful = angularMomentConfig.statefulFilters;
        return amDurationFormatFilter;
      }
    ]).filter('amTimeAgo', [
      'moment',
      'amMoment',
      'angularMomentConfig',
      function (moment, amMoment, angularMomentConfig) {
        function amTimeAgoFilter(value, preprocess, suffix, from) {
          var date, dateFrom;
          if (typeof value === 'undefined' || value === null) {
            return '';
          }
          value = amMoment.preprocessDate(value, preprocess);
          date = moment(value);
          if (!date.isValid()) {
            return '';
          }
          dateFrom = moment(from);
          if (typeof from !== 'undefined' && dateFrom.isValid()) {
            return amMoment.applyTimezone(date).from(dateFrom, suffix);
          }
          return amMoment.applyTimezone(date).fromNow(suffix);
        }
        amTimeAgoFilter.$stateful = angularMomentConfig.statefulFilters;
        return amTimeAgoFilter;
      }
    ]).filter('amSubtract', [
      'moment',
      'angularMomentConfig',
      function (moment, angularMomentConfig) {
        function amSubtractFilter(value, amount, type) {
          if (typeof value === 'undefined' || value === null) {
            return '';
          }
          return moment(value).subtract(parseInt(amount, 10), type);
        }
        amSubtractFilter.$stateful = angularMomentConfig.statefulFilters;
        return amSubtractFilter;
      }
    ]).filter('amAdd', [
      'moment',
      'angularMomentConfig',
      function (moment, angularMomentConfig) {
        function amAddFilter(value, amount, type) {
          if (typeof value === 'undefined' || value === null) {
            return '';
          }
          return moment(value).add(parseInt(amount, 10), type);
        }
        amAddFilter.$stateful = angularMomentConfig.statefulFilters;
        return amAddFilter;
      }
    ]);
  }
  if (typeof define === 'function' && define.amd) {
    define([
      'angular',
      'moment'
    ], angularMoment);
  } else if (typeof module !== 'undefined' && module && module.exports) {
    angularMoment(angular, require('moment'));
    module.exports = 'angularMoment';
  } else {
    angularMoment(angular, (typeof global !== 'undefined' ? global : window).moment);
  }
}());
'use strict';
angular.module('ng-multiselecttree', []).directive('tree', [
  '$compile',
  '$log',
  '$timeout',
  function ($compile, $log, $timeout) {
    return {
      restrict: 'E',
      terminal: true,
      scope: {
        maxnumberchildren: '@',
        root: '@',
        fieldid: '@',
        fieldlabel: '@',
        fieldchildren: '@',
        val: '=?',
        fetch: '=?',
        selected: '='
      },
      link: function (scope, element, attrs) {
        scope.onCheckChange = function () {
          //scope.val.checked = !scope.val.checked;
          var checked = scope.val.checked;
          // now set all child nodes to match this value
          function _setChecked(node) {
            node.checked = checked;
            if (node.children)
              angular.forEach(node.children, _setChecked);
          }
          angular.forEach(scope.val.children, _setChecked);
          var _findChecked = function (node, result) {
            if (!angular.isDefined(result)) {
              result = [];
            }
            if (node.checked === true && !node.isGrouping) {
              result.push(node.getId());
            }
            angular.forEach(node.children, function (child) {
              _findChecked(child, result);
            });
            return result;
          };
          scope.val.rootScope.selected = _findChecked(scope.val.rootScope.val);
        };
        scope.expand = function (val) {
          val.show = !val.show;
          if (val.hasChildren && val.rootScope.fetch && val.children == null) {
            val.isLoading = true;
            val.rootScope.fetch(val.getId(), function (fetchedItems) {
              var items = [];
              var index = 0;
              var fetchedItem = null;
              var label = null;
              var hasChildren = false;
              angular.forEach(fetchedItems, function (_fetchedItem) {
                fetchedItem = _fetchedItem[0];
                hasChildren = _fetchedItem[1];
                index = fetchedItem.lastIndexOf('/') > -1 ? fetchedItem.lastIndexOf('/') : fetchedItem.lastIndexOf('\\');
                label = fetchedItem.substr(index + 1);
                items.push(createTreeData(fetchedItem, label, null, val.rootScope, hasChildren, false, false));
              });
              _setChildren(val, items);
              val.isLoading = false;
            });
          }
        };
        function _subDivideChildren(val, items) {
          if (items.length > val.rootScope.maxnumberchildren) {
            var groups = _subDivideChildrenIntoGroups(val.rootScope, items, 1);
            var result = [];
            var label = '';
            angular.forEach(groups, function (items, group) {
              if (items.length == 1) {
                result.push(items[0]);
              } else {
                label = group + '... [' + items.length + ' items]';
                result.push(createTreeData(val.id + '.' + group, label, items, val.rootScope, true, false, true));
              }
            });
            return result.sort(function (a, b) {
              return _getLabel(a, val.rootScope) - _getLabel(b, val.rootScope);
            });
          } else {
            return items;
          }
        }
        function _subDivideChildrenIntoGroups(rootScope, items, numberChars) {
          var groups = {};
          angular.forEach(items, function (item) {
            var prefix = angular.lowercase(_getLabel(item, rootScope).substr(0, numberChars));
            if (!(prefix in groups)) {
              groups[prefix] = [];
            }
            groups[prefix].push(item);
          });
          angular.forEach(groups, function (items, group) {
            if (items.length > rootScope.maxnumberchildren && numberChars < 10) {
              delete groups[group];
              var newGroups = _subDivideChildrenIntoGroups(rootScope, items, numberChars + 1);
              angular.forEach(newGroups, function (items, group) {
                groups[group] = items;
              });
            }
          });
          return groups;
        }
        scope.$watch('root', function (root, oldRoot) {
          if (root) {
            initialiseRootScope(scope);
            scope.val = createTreeData(root, root, null, scope, true, true, false);
          }
        });
        function createTreeData(id, label, children, rootScope, hasChildren, isRoot, isGrouping) {
          var treeData = {
              isRoot: isRoot,
              rootScope: rootScope,
              hasChildren: hasChildren,
              _children: children,
              children: children,
              show: false,
              checked: false,
              isLoading: false,
              isGrouping: isGrouping,
              filterText: null,
              getId: function () {
                return treeData[treeData.rootScope.fieldid];
              }
            };
          treeData[rootScope.fieldid] = id;
          treeData[rootScope.fieldlabel] = label;
          return treeData;
        }
        function _refresh(treeData) {
          var filteredItems = [];
          if (treeData.filterText != null) {
            angular.forEach(treeData._children, function (item) {
              if (_getLabel(item).indexOf(treeData.filterText) == 0) {
                filteredItems.push(item);
              }
            });
          } else {
            filteredItems = treeData._children;
          }
          treeData.children = treeData.rootScope.maxnumberchildren > 0 ? _subDivideChildren(treeData, filteredItems) : filteredItems;
        }
        function _getLabel(treeData, rootScope) {
          return treeData[rootScope ? rootScope.fieldlabel : treeData.rootScope ? treeData.rootScope.fieldlabel : 'name'];
        }
        function _setChildren(treeData, children) {
          treeData._children = children;
          _refresh(treeData);
        }
        function initialiseStaticTree(val, scope) {
          if (!angular.isDefined(val.rootScope)) {
            initialiseRootScope(scope);
            val.rootScope = scope;
            var _recurse = function (node, rootScope) {
              if (!angular.isDefined(node.show)) {
                node.show = false;
              }
              if (!angular.isDefined(node.checked)) {
                node.checked = false;
              }
              if (!angular.isDefined(node.rootScope)) {
                node.rootScope = rootScope;
              }
              node.getId = function () {
                return node[node.rootScope.fieldid];
              };
              var items = node[node.rootScope.fieldchildren];
              if (angular.isDefined(items)) {
                node.hasChildren = true;
                _setChildren(node, items);
                for (var i in items) {
                  _recurse(items[i], rootScope);
                }
              }
            };
            _recurse(val, val.rootScope);
          }
        }
        function initialiseRootScope(rootScope) {
          rootScope.maxnumberchildren = angular.isDefined(rootScope.maxnumberchildren) ? parseInt(rootScope.maxnumberchildren) : 25;
          rootScope.fieldid = angular.isDefined(rootScope.fieldid) ? rootScope.fieldid : 'id';
          rootScope.fieldlabel = angular.isDefined(rootScope.fieldlabel) ? rootScope.fieldlabel : 'label';
          rootScope.fieldchildren = angular.isDefined(rootScope.fieldchildren) ? rootScope.fieldchildren : 'items';
        }
        scope.$watch('val.filterText', function (val, old) {
          if (val != null) {
            _refresh(scope.val);
          }
        });
        scope.$watch('val', function (val, oldVal) {
          if (val && angular.isDefined(val)) {
            initialiseStaticTree(val, scope);
            var nt = val.isGrouping ? 'span' : 'ul';
            var template = '';
            template += val.isRoot ? '<div class="treeView">' : '';
            template += '<div class="treeItem">';
            template += '  <input ng-if="!val.isGrouping" ng-model="val.checked" type="checkbox" ng-click="onCheckChange()" class="treeItemCheck">';
            template += '  <a ng-if="val.hasChildren " href="" ng-click="expand(val)" class="treeItemLink" title="{{ val.getId() }}">{{ val[val.rootScope.fieldlabel] }}</a>';
            template += '  <input ng-if="val._children.length > 5 && val.show" type="text" placeholder="filter" ng-model="val.filterText" class="treeItemFilter">';
            template += '  <span ng-if="!val.hasChildren"  title="{{ val.getId() }}">{{ val[val.rootScope.fieldlabel]}}</span>';
            template += '  <' + nt + ' ng-if="val.show" class="treeItemChildren">';
            template += '    <li ng-repeat="item in val.children track by item.getId()" class="treeItemChild_{{ item.hasChildren }}">';
            template += '      <tree val="item" fetch="fetch"></tree>';
            template += '    </li>';
            template += '  </' + nt + '>';
            template += '  <div class="treeItemLoading" ng-show="val.isLoading">Loading...</div>';
            template += '</div>';
            template += val.isRoot ? '</div>' : '';
            var newElement = angular.element(template);
            $compile(newElement)(scope);
            element.replaceWith(newElement);
            element = newElement;
          }
        });
      }
    };
  }
]);
//todo - Delimiter isnt ideal
//todo - process the data on the background.
/*! jstz - v1.0.4 - 2012-12-12 */
(function (e) {
  var t = function () {
      'use strict';
      var e = 's', n = function (e) {
          var t = -e.getTimezoneOffset();
          return t !== null ? t : 0;
        }, r = function (e, t, n) {
          var r = new Date();
          return e !== undefined && r.setFullYear(e), r.setDate(n), r.setMonth(t), r;
        }, i = function (e) {
          return n(r(e, 0, 2));
        }, s = function (e) {
          return n(r(e, 5, 2));
        }, o = function (e) {
          var t = e.getMonth() > 7 ? s(e.getFullYear()) : i(e.getFullYear()), r = n(e);
          return t - r !== 0;
        }, u = function () {
          var t = i(), n = s(), r = i() - s();
          return r < 0 ? t + ',1' : r > 0 ? n + ',1,' + e : t + ',0';
        }, a = function () {
          var e = u();
          return new t.TimeZone(t.olson.timezones[e]);
        };
      return {
        determine: a,
        date_is_dst: o
      };
    }();
  t.TimeZone = function (e) {
    'use strict';
    var n = null, r = function () {
        return n;
      }, i = function () {
        var e = t.olson.ambiguity_list[n], r = e.length, i = 0, s = e[0];
        for (; i < r; i += 1) {
          s = e[i];
          if (t.date_is_dst(t.olson.dst_start_dates[s])) {
            n = s;
            return;
          }
        }
      }, s = function () {
        return typeof t.olson.ambiguity_list[n] != 'undefined';
      };
    return n = e, s() && i(), { name: r };
  }, t.olson = {}, t.olson.timezones = {
    '-720,0': 'Etc/GMT+12',
    '-660,0': 'Pacific/Pago_Pago',
    '-600,1': 'America/Adak',
    '-600,0': 'Pacific/Honolulu',
    '-570,0': 'Pacific/Marquesas',
    '-540,0': 'Pacific/Gambier',
    '-540,1': 'America/Anchorage',
    '-480,1': 'America/Los_Angeles',
    '-480,0': 'Pacific/Pitcairn',
    '-420,0': 'America/Phoenix',
    '-420,1': 'America/Denver',
    '-360,0': 'America/Guatemala',
    '-360,1': 'America/Chicago',
    '-360,1,s': 'Pacific/Easter',
    '-300,0': 'America/Bogota',
    '-300,1': 'America/New_York',
    '-270,0': 'America/Caracas',
    '-240,1': 'America/Halifax',
    '-240,0': 'America/Santo_Domingo',
    '-240,1,s': 'America/Santiago',
    '-210,1': 'America/St_Johns',
    '-180,1': 'America/Godthab',
    '-180,0': 'America/Argentina/Buenos_Aires',
    '-180,1,s': 'America/Montevideo',
    '-120,0': 'Etc/GMT+2',
    '-120,1': 'Etc/GMT+2',
    '-60,1': 'Atlantic/Azores',
    '-60,0': 'Atlantic/Cape_Verde',
    '0,0': 'Etc/UTC',
    '0,1': 'Europe/London',
    '60,1': 'Europe/Berlin',
    '60,0': 'Africa/Lagos',
    '60,1,s': 'Africa/Windhoek',
    '120,1': 'Asia/Beirut',
    '120,0': 'Africa/Johannesburg',
    '180,0': 'Asia/Baghdad',
    '180,1': 'Europe/Moscow',
    '210,1': 'Asia/Tehran',
    '240,0': 'Asia/Dubai',
    '240,1': 'Asia/Baku',
    '270,0': 'Asia/Kabul',
    '300,1': 'Asia/Yekaterinburg',
    '300,0': 'Asia/Karachi',
    '330,0': 'Asia/Kolkata',
    '345,0': 'Asia/Kathmandu',
    '360,0': 'Asia/Dhaka',
    '360,1': 'Asia/Omsk',
    '390,0': 'Asia/Rangoon',
    '420,1': 'Asia/Krasnoyarsk',
    '420,0': 'Asia/Jakarta',
    '480,0': 'Asia/Shanghai',
    '480,1': 'Asia/Irkutsk',
    '525,0': 'Australia/Eucla',
    '525,1,s': 'Australia/Eucla',
    '540,1': 'Asia/Yakutsk',
    '540,0': 'Asia/Tokyo',
    '570,0': 'Australia/Darwin',
    '570,1,s': 'Australia/Adelaide',
    '600,0': 'Australia/Brisbane',
    '600,1': 'Asia/Vladivostok',
    '600,1,s': 'Australia/Sydney',
    '630,1,s': 'Australia/Lord_Howe',
    '660,1': 'Asia/Kamchatka',
    '660,0': 'Pacific/Noumea',
    '690,0': 'Pacific/Norfolk',
    '720,1,s': 'Pacific/Auckland',
    '720,0': 'Pacific/Tarawa',
    '765,1,s': 'Pacific/Chatham',
    '780,0': 'Pacific/Tongatapu',
    '780,1,s': 'Pacific/Apia',
    '840,0': 'Pacific/Kiritimati'
  }, t.olson.dst_start_dates = function () {
    'use strict';
    var e = new Date(2010, 6, 15, 1, 0, 0, 0);
    return {
      'America/Denver': new Date(2011, 2, 13, 3, 0, 0, 0),
      'America/Mazatlan': new Date(2011, 3, 3, 3, 0, 0, 0),
      'America/Chicago': new Date(2011, 2, 13, 3, 0, 0, 0),
      'America/Mexico_City': new Date(2011, 3, 3, 3, 0, 0, 0),
      'America/Asuncion': new Date(2012, 9, 7, 3, 0, 0, 0),
      'America/Santiago': new Date(2012, 9, 3, 3, 0, 0, 0),
      'America/Campo_Grande': new Date(2012, 9, 21, 5, 0, 0, 0),
      'America/Montevideo': new Date(2011, 9, 2, 3, 0, 0, 0),
      'America/Sao_Paulo': new Date(2011, 9, 16, 5, 0, 0, 0),
      'America/Los_Angeles': new Date(2011, 2, 13, 8, 0, 0, 0),
      'America/Santa_Isabel': new Date(2011, 3, 5, 8, 0, 0, 0),
      'America/Havana': new Date(2012, 2, 10, 2, 0, 0, 0),
      'America/New_York': new Date(2012, 2, 10, 7, 0, 0, 0),
      'Asia/Beirut': new Date(2011, 2, 27, 1, 0, 0, 0),
      'Europe/Helsinki': new Date(2011, 2, 27, 4, 0, 0, 0),
      'Europe/Istanbul': new Date(2011, 2, 28, 5, 0, 0, 0),
      'Asia/Damascus': new Date(2011, 3, 1, 2, 0, 0, 0),
      'Asia/Jerusalem': new Date(2011, 3, 1, 6, 0, 0, 0),
      'Asia/Gaza': new Date(2009, 2, 28, 0, 30, 0, 0),
      'Africa/Cairo': new Date(2009, 3, 25, 0, 30, 0, 0),
      'Pacific/Auckland': new Date(2011, 8, 26, 7, 0, 0, 0),
      'Pacific/Fiji': new Date(2010, 11, 29, 23, 0, 0, 0),
      'America/Halifax': new Date(2011, 2, 13, 6, 0, 0, 0),
      'America/Goose_Bay': new Date(2011, 2, 13, 2, 1, 0, 0),
      'America/Miquelon': new Date(2011, 2, 13, 5, 0, 0, 0),
      'America/Godthab': new Date(2011, 2, 27, 1, 0, 0, 0),
      'Europe/Moscow': e,
      'Asia/Yekaterinburg': e,
      'Asia/Omsk': e,
      'Asia/Krasnoyarsk': e,
      'Asia/Irkutsk': e,
      'Asia/Yakutsk': e,
      'Asia/Vladivostok': e,
      'Asia/Kamchatka': e,
      'Europe/Minsk': e,
      'Australia/Perth': new Date(2008, 10, 1, 1, 0, 0, 0)
    };
  }(), t.olson.ambiguity_list = {
    'America/Denver': [
      'America/Denver',
      'America/Mazatlan'
    ],
    'America/Chicago': [
      'America/Chicago',
      'America/Mexico_City'
    ],
    'America/Santiago': [
      'America/Santiago',
      'America/Asuncion',
      'America/Campo_Grande'
    ],
    'America/Montevideo': [
      'America/Montevideo',
      'America/Sao_Paulo'
    ],
    'Asia/Beirut': [
      'Asia/Beirut',
      'Europe/Helsinki',
      'Europe/Istanbul',
      'Asia/Damascus',
      'Asia/Jerusalem',
      'Asia/Gaza'
    ],
    'Pacific/Auckland': [
      'Pacific/Auckland',
      'Pacific/Fiji'
    ],
    'America/Los_Angeles': [
      'America/Los_Angeles',
      'America/Santa_Isabel'
    ],
    'America/New_York': [
      'America/Havana',
      'America/New_York'
    ],
    'America/Halifax': [
      'America/Goose_Bay',
      'America/Halifax'
    ],
    'America/Godthab': [
      'America/Miquelon',
      'America/Godthab'
    ],
    'Asia/Dubai': ['Europe/Moscow'],
    'Asia/Dhaka': ['Asia/Yekaterinburg'],
    'Asia/Jakarta': ['Asia/Omsk'],
    'Asia/Shanghai': [
      'Asia/Krasnoyarsk',
      'Australia/Perth'
    ],
    'Asia/Tokyo': ['Asia/Irkutsk'],
    'Australia/Brisbane': ['Asia/Yakutsk'],
    'Pacific/Noumea': ['Asia/Vladivostok'],
    'Pacific/Tarawa': ['Asia/Kamchatka'],
    'Africa/Johannesburg': [
      'Asia/Gaza',
      'Africa/Cairo'
    ],
    'Asia/Baghdad': ['Europe/Minsk']
  }, typeof exports != 'undefined' ? exports.jstz = t : e.jstz = t;
}(this));
'use strict';
// MODIFIED FOR KIDREPORTS
// drawer width measures in pixels on large displays but becomes 100% when <768px
(function (root, factory) {
  if (typeof define === 'function' && define.amd) {
    define(['angular'], factory);
  } else if (typeof module === 'object' && module.exports) {
    module.exports = factory(require('angular'));
  } else {
    factory(root.angular);
  }
}(this, function (angular) {
  angular.module('pageslide-directive', []).directive('pageslide', [
    '$document',
    '$timeout',
    '$window',
    function ($document, $timeout, $window) {
      var defaults = {};
      return {
        restrict: 'EA',
        transclude: false,
        scope: {
          psOpen: '=?',
          psAutoClose: '@',
          psSide: '@',
          psSpeed: '@',
          psClass: '@',
          psSize: '@',
          psZindex: '@',
          psPush: '@',
          psContainer: '@',
          psKeyListener: '@',
          psBodyClass: '@',
          psClickOutside: '@',
          onopen: '=?',
          onclose: '=?'
        },
        link: function ($scope, el, attrs) {
          var param = {};
          param.side = $scope.psSide || 'right';
          param.speed = $scope.psSpeed || '0.5';
          param.size = $scope.psSize || '300px';
          param.zindex = $scope.psZindex || 1000;
          param.className = $scope.psClass || 'ng-pageslide';
          param.push = $scope.psPush === 'true';
          param.container = $scope.psContainer || false;
          param.keyListener = $scope.psKeyListener === 'true';
          param.bodyClass = $scope.psBodyClass || false;
          param.clickOutside = $scope.psClickOutside !== 'false';
          param.push = param.push && !param.container;
          el.addClass(param.className);
          function getResponsiveSize() {
            return $window.innerWidth < 768 ? '100%' : param.size;
          }
          ;
          /* DOM manipulation */
          var content, slider, body, isOpen = false;
          if (param.container) {
            body = document.getElementById(param.container);
          } else {
            body = document.body;
          }
          function onBodyClick(e) {
            if ($scope.psOpen && !slider.contains(e.target)) {
              isOpen = false;
              $scope.psOpen = false;
              $scope.$apply();
            }
            if ($scope.psOpen) {
              isOpen = true;
            }
          }
          function setBodyClass(value) {
            if (param.bodyClass) {
              var bodyClass = param.className + '-body';
              var bodyClassRe = new RegExp(' ' + bodyClass + '-closed| ' + bodyClass + '-open');
              body.className = body.className.replace(bodyClassRe, '');
              body.className += ' ' + bodyClass + '-' + value;
            }
          }
          setBodyClass('closed');
          slider = el[0];
          if (slider.tagName.toLowerCase() !== 'div' && slider.tagName.toLowerCase() !== 'pageslide') {
            throw new Error('Pageslide can only be applied to <div> or <pageslide> elements');
          }
          body.appendChild(slider);
          slider.style.zIndex = param.zindex;
          slider.style.position = 'fixed';
          slider.style.transitionDuration = param.speed + 's';
          slider.style.webkitTransitionDuration = param.speed + 's';
          slider.style.height = param.size;
          slider.style.transitionProperty = 'top, bottom, left, right';
          if (param.push) {
            body.style.position = 'absolute';
            body.style.transitionDuration = param.speed + 's';
            body.style.webkitTransitionDuration = param.speed + 's';
            body.style.transitionProperty = 'top, bottom, left, right';
          }
          if (param.container) {
            slider.style.position = 'absolute';
            body.style.position = 'relative';
            body.style.overflow = 'hidden';
          }
          function onTransitionEnd() {
            if ($scope.psOpen) {
              if (typeof $scope.onopen === 'function') {
                $scope.onopen();
              }
            } else {
              if (typeof $scope.onclose === 'function') {
                $scope.onclose();
              }
            }
          }
          slider.addEventListener('transitionend', onTransitionEnd);
          initSlider();
          function initSlider() {
            switch (param.side) {
            case 'right':
              slider.style.width = getResponsiveSize();
              slider.style.height = '100%';
              slider.style.top = '0px';
              slider.style.bottom = '0px';
              slider.style.right = $scope.psOpen ? '0px' : '-' + getResponsiveSize();
              break;
            case 'left':
              slider.style.width = getResponsiveSize();
              slider.style.height = '100%';
              slider.style.top = '0px';
              slider.style.bottom = '0px';
              slider.style.left = $scope.psOpen ? '0px' : '-' + getResponsiveSize();
              break;
            case 'top':
              slider.style.height = param.size;
              slider.style.width = '100%';
              slider.style.left = '0px';
              slider.style.top = '0px';
              slider.style.right = '0px';
              break;
            case 'bottom':
              slider.style.height = param.size;
              slider.style.width = '100%';
              slider.style.bottom = '0px';
              slider.style.left = '0px';
              slider.style.right = '0px';
              break;
            }
          }
          function resizeSlider() {
            switch (param.side) {
            case 'right':
              slider.style.width = getResponsiveSize();
              if (!$scope.psOpen) {
                slider.style.right = '-' + getResponsiveSize();
              }
              break;
            case 'left':
              slider.style.width = getResponsiveSize();
              if (!$scope.psOpen) {
                slider.style.left = '-' + getResponsiveSize();
              }
              break;
            }
          }
          function psClose(slider, param) {
            switch (param.side) {
            case 'right':
              slider.style.right = '-' + getResponsiveSize();
              if (param.push) {
                body.style.right = '0px';
                body.style.left = '0px';
              }
              break;
            case 'left':
              slider.style.left = '-' + getResponsiveSize();
              if (param.push) {
                body.style.left = '0px';
                body.style.right = '0px';
              }
              break;
            case 'top':
              slider.style.top = '-' + param.size;
              if (param.push) {
                body.style.top = '0px';
                body.style.bottom = '0px';
              }
              break;
            case 'bottom':
              slider.style.bottom = '-' + param.size;
              if (param.push) {
                body.style.bottom = '0px';
                body.style.top = '0px';
              }
              break;
            }
            if (param.keyListener) {
              $document.off('keydown', handleKeyDown);
            }
            if (param.clickOutside) {
              $document.off('click', onBodyClick);
            }
            isOpen = false;
            setBodyClass('closed');
            $scope.psOpen = false;
          }
          function psOpen(slider, param) {
            switch (param.side) {
            case 'right':
              slider.style.right = '0px';
              if (param.push) {
                body.style.right = param.size;
                body.style.left = '-' + param.size;
              }
              break;
            case 'left':
              slider.style.left = '0px';
              if (param.push) {
                body.style.left = param.size;
                body.style.right = '-' + param.size;
              }
              break;
            case 'top':
              slider.style.top = '0px';
              if (param.push) {
                body.style.top = param.size;
                body.style.bottom = '-' + param.size;
              }
              break;
            case 'bottom':
              slider.style.bottom = '0px';
              if (param.push) {
                body.style.bottom = param.size;
                body.style.top = '-' + param.size;
              }
              break;
            }
            $scope.psOpen = true;
            if (param.keyListener) {
              $document.on('keydown', handleKeyDown);
            }
            if (param.clickOutside) {
              $document.on('click', onBodyClick);
            }
            setBodyClass('open');
          }
          function handleKeyDown(e) {
            var ESC_KEY = 27;
            var key = e.keyCode || e.which;
            if (key === ESC_KEY) {
              psClose(slider, param);
              // FIXME check with tests
              // http://stackoverflow.com/questions/12729122/angularjs-prevent-error-digest-already-in-progress-when-calling-scope-apply
              $timeout(function () {
                $scope.$apply();
              });
            }
          }
          // Watchers
          $scope.$watch('psOpen', function (value) {
            if (!!value) {
              psOpen(slider, param);
            } else {
              psClose(slider, param);
            }
          });
          $scope.$watch('psSize', function (newValue, oldValue) {
            if (oldValue !== newValue) {
              param.size = newValue;
              initSlider();
            }
          });
          // Events
          angular.element($window).on('resize', resizeSlider);
          $scope.$on('$destroy', function () {
            if (slider.parentNode === body) {
              if (param.clickOutside) {
                $document.off('click', onBodyClick);
              }
              body.removeChild(slider);
            }
            slider.removeEventListener('transitionend', onTransitionEnd);
          });
          if ($scope.psAutoClose) {
            $scope.$on('$locationChangeStart', function () {
              psClose(slider, param);
            });
            $scope.$on('$stateChangeStart', function () {
              psClose(slider, param);
            });
          }
        }
      };
    }
  ]);
}));